ADVANCEMENT_OPTION = MakeEnum{
    --Allies don't scale as much
    "ALLY_SCALE",
    -- Additive modifier to all enemy resolve
    "NPC_RESOLVE",
    -- Probablity that a CharacterDef can be Promoted when spawned.
    "NPC_PROMOTION_CHANCE",
    -- Scaling damage factor applied to enemy intents
    "NPC_RESOLVE_DAMAGE",
    -- Scaling modifier to non-boss Agent health
    "NPC_HEALTH",
    -- Scaling modifier to non-boss enemy attacks
    "NPC_DAMAGE",
    -- Scaling modifier to boss Agent health
    "NPC_BOSS_HEALTH",
    -- Number that bosses use to change behavior / damage
    "NPC_BOSS_DIFFICULTY",
    -- Scaling modifier to enemy combat buffs
    "NPC_ABILITY_STRENGTH",
    -- Improved version of enemy arguments.
    "NPC_ARGUMENT_PLUS",
    -- Modifier for sleep health restoration
    "SLEEP_HEALTH_RESTORE",
    -- Modifier for sleep resolve restoration
    "SLEEP_RESOLVE_RESTORE",
    -- Modifier for post-battle resolve restoration
    "BATTLE_RESOLVE_RESTORE",
    -- More powerful species boons.
    "UPGRADED_SPECIES_BOONS",

   -- Higher costs
    "HIGHER_COSTS",
    "HIGHER_COSTS_STR",
    "NPC_ARGUMENT_RESOLVE",
    "LOWER_MAX_HP",
    "LOWER_MAX_RESOLVE",
    "FEWER_COMBAT_GRAFT_SLOTS",
    "FEWER_NEGOTIATION_GRAFT_SLOTS",

    --turn of mettle upgrades
    "METTLE_OFF",
}

DEFAULT_ADVANCEMENT_OPTION =
{
    [ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH] = 0,
    [ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY] = 2,
    [ADVANCEMENT_OPTION.BATTLE_RESOLVE_RESTORE] = 0.20,
    [ADVANCEMENT_OPTION.UPGRADED_SPECIES_BOONS] = 1.0,
    [ADVANCEMENT_OPTION.NPC_BOSS_HEALTH] = 1.0,
    [ADVANCEMENT_OPTION.NPC_HEALTH] = 1.0,
    [ADVANCEMENT_OPTION.NPC_PROMOTION_CHANCE] = 0,
}

STORY_ADVANCEMENT_OPTION =
{
    [ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH] = 0,
    [ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY] = 1,
    [ADVANCEMENT_OPTION.BATTLE_RESOLVE_RESTORE] = 0.333,
    [ADVANCEMENT_OPTION.UPGRADED_SPECIES_BOONS] = 1.0,
    [ADVANCEMENT_OPTION.NPC_BOSS_HEALTH] = 0.6,
    [ADVANCEMENT_OPTION.NPC_HEALTH] = 0.5,
    [ADVANCEMENT_OPTION.NPC_PROMOTION_CHANCE] = 0,
    [ADVANCEMENT_OPTION.ALLY_SCALE] = 1,
    [ADVANCEMENT_OPTION.NPC_DAMAGE ] = 0.6,
}

-- All advancement levels are cumulative (all previous modifiers also apply)
DEFAULT_ADVANCEMENT =
{
    { -- 1
        name = "ADVANCEMENT.PRESTIGE_1_NAME",
        desc = "ADVANCEMENT.PRESTIGE_1_DESC",
        [ ADVANCEMENT_OPTION.NPC_PROMOTION_CHANCE ] = 0.25,
        [ ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ] = 1,
        [ ADVANCEMENT_OPTION.NPC_ARGUMENT_RESOLVE ] = { 1, 1, 2, 2, 3 },
        [ ADVANCEMENT_OPTION.NPC_ARGUMENT_PLUS ] = true,
    },
    { -- 2
        name = "ADVANCEMENT.PRESTIGE_2_NAME",
        desc = "ADVANCEMENT.PRESTIGE_2_DESC",
        [ ADVANCEMENT_OPTION.NPC_BOSS_HEALTH ] = 1.2,
        [ ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ] = 3,
        [ ADVANCEMENT_OPTION.NPC_HEALTH ] = 1.3,
        [ ADVANCEMENT_OPTION.NPC_DAMAGE ] = 1.2,
        [ ADVANCEMENT_OPTION.NPC_RESOLVE ] = { 0, 5, 5, 10, 10 },
        [ ADVANCEMENT_OPTION.NPC_RESOLVE_DAMAGE ] = 1.2,
    },
    { -- 3
        name = "ADVANCEMENT.PRESTIGE_3_NAME",
        desc = "ADVANCEMENT.PRESTIGE_3_DESC",
        [ ADVANCEMENT_OPTION.BATTLE_RESOLVE_RESTORE ] = 0,
        [ ADVANCEMENT_OPTION.SLEEP_RESOLVE_RESTORE ] = 0.5,
        [ ADVANCEMENT_OPTION.SLEEP_HEALTH_RESTORE ] = 0.5,
    },
    { -- 4
        name = "ADVANCEMENT.PRESTIGE_4_NAME",
        desc = "ADVANCEMENT.PRESTIGE_4_DESC",
        [ ADVANCEMENT_OPTION.ALLY_SCALE ] = -1,
        [ ADVANCEMENT_OPTION.UPGRADED_SPECIES_BOONS ] = 2,
        [ ADVANCEMENT_OPTION.NPC_BOSS_HEALTH ] = 1.35,
        [ ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ] = 4,
    },
    { -- 5
        name = "ADVANCEMENT.PRESTIGE_5_NAME",
        desc = "ADVANCEMENT.PRESTIGE_5_DESC",
        [ ADVANCEMENT_OPTION.LOWER_MAX_RESOLVE ] = .8,
        [ ADVANCEMENT_OPTION.LOWER_MAX_HP ] = .8,
    },
    { -- 6
        name = "ADVANCEMENT.PRESTIGE_6_NAME",
        desc = "ADVANCEMENT.PRESTIGE_6_DESC",
        [ ADVANCEMENT_OPTION.HIGHER_COSTS ] = 0.25,
        [ ADVANCEMENT_OPTION.HIGHER_COSTS_STR ] = "ADVANCEMENT.ADVANCEMENT_LEVEL_HIGHER_PRICES_DESC",
        [ ADVANCEMENT_OPTION.FEWER_NEGOTIATION_GRAFT_SLOTS ] = 1,
        [ ADVANCEMENT_OPTION.FEWER_COMBAT_GRAFT_SLOTS ] = 1,
    },
    { -- 6
        name = "ADVANCEMENT.PRESTIGE_7_NAME",
        desc = "ADVANCEMENT.PRESTIGE_7_DESC",
        [ ADVANCEMENT_OPTION.METTLE_OFF ] = true,
    },
}

-- Auto-assign icons.
for i, t in ipairs( DEFAULT_ADVANCEMENT ) do
    local icon_path = string.format( "UI/advancement_level_%d.tex", i )
    t.icon = engine.asset.Texture( icon_path )
end

ADVANCEMENT_CONVERSION =
{
    [1] = 1,
    [2] = 1,
    [3] = 2,
    [4] = 2,
    [5] = 2,
    [6] = 2,
    [7] = 3,
    [8] = 3,
    [9] = 3,
    [10] = 4,
    [11] = 4,
    [12] = 5,
    [13] = 5,
    [14] = 6,
    [15] = 6,
}