require "ui/screens/upgradecardpopup"

local AgentUtil = class( "AgentUtil" )

function AgentUtil.MakeDrunk( agent )
    if agent:IsPlayer() then
        agent.negotiator:LearnCard( "drunk_player" )
        agent.battler:LearnCard( "drunk" )
    else
        agent:GainAspect( "drunk" )
    end
end

function AgentUtil.TakeJob(agent, location, work_position_id)
    local work = location:GetWorkPosition(work_position_id)
    if work == nil then
        work = location:AddWorkPosition( work_position_id )
    elseif work:GetAgent() ~= agent then
        if work:GetAgent() then
            work:Fire()
        end
    end
    if work:GetAgent() ~= agent then
        work:Hire(agent)
    end
    assert(work:GetAgent(agent))
end


    
function AgentUtil.GetCombatSupport( agent, subject, self_defence, no_bystanders )

    local player_rel = agent:GetRelationship()
    local subject_rel = agent:GetRelationship( subject ) or RELATIONSHIP.NEUTRAL
    local sup, reason, txt

    for k,quest in TheGame:GetGameState():ActiveQuests() do
        sup, reason, txt = quest:GetFightSupport(agent, subject)
    end

    if not sup then
        
        if agent:IsCrewMember() and agent:IsInPlayerParty() then
            sup, reason, txt = SUPPORT_ACTION.HELP, SUPPORT_REASON.CREW, LOC"SUPPORT_DESC.CREW"

        elseif AgentUtil.HasPlotArmour(agent) then
            return SUPPORT_ACTION.STAY_OUT, SUPPORT_REASON.VIP, LOC"SUPPORT_DESC.VIP"

        elseif agent:IsCaptiveMember() and agent:IsInPlayerParty() then
            if subject:IsSentient() then
                sup, reason, txt = SUPPORT_ACTION.HINDER, SUPPORT_REASON.CAPTIVE, LOC"SUPPORT_DESC.TRY_TO_BREAK_FREE"
            end
        elseif not no_bystanders then
            if player_rel == RELATIONSHIP.LOVED then
                if subject_rel == RELATIONSHIP.LOVED then
                    sup, reason, txt = SUPPORT_ACTION.STAY_OUT, SUPPORT_REASON.TARGET_FRIENDSHIP, LOC"SUPPORT_DESC.LIKES_YOU_BOTH"
                else
                    sup, reason, txt = SUPPORT_ACTION.HELP, SUPPORT_REASON.PLAYER_FRIENDSHIP, LOC"SUPPORT_DESC.LIKES_YOU"
                end
            else

                if not sup and subject_rel > RELATIONSHIP.NEUTRAL then
                    sup, reason, txt = SUPPORT_ACTION.HINDER, SUPPORT_REASON.TARGET_FRIENDSHIP, loc.format( LOC"SUPPORT_DESC.LIKES_SUBJECT", subject:LocTable())
                end

                if subject_rel >= RELATIONSHIP.NEUTRAL then
                    
                    if agent:GetParty() and agent:GetParty() == subject:GetParty() then
                        sup, reason, txt = SUPPORT_ACTION.HINDER, SUPPORT_REASON.TARGET_FRIENDSHIP, LOC"SUPPORT_DESC.SAME_PARTY"

                    elseif agent:GetFactionID() == subject:GetFactionID() and agent:GetFaction():Defends(agent, subject) then
                        sup, reason, txt = SUPPORT_ACTION.HINDER, SUPPORT_REASON.FACTION, LOC"SUPPORT_DESC.PROTECTING_FACTION"
                    end
                end
                
                if agent:GetRoleAtLocation() == CHARACTER_ROLES.GUARD and (subject:GetRoleAtLocation() == CHARACTER_ROLES.PROPRIETOR or subject:GetRoleAtLocation() == CHARACTER_ROLES.CLERK or subject:GetRoleAtLocation() == CHARACTER_ROLES.VENDOR) then
                    sup, reason, txt = SUPPORT_ACTION.HINDER, SUPPORT_REASON.DUTY, LOC"SUPPORT_DESC.PROTECT_WORKPLACE"
                end

                if agent:GetRoleAtLocation() == CHARACTER_ROLES.GUARD and subject:GetFactionID() ~= agent:GetFactionID() and self_defence then
                    sup, reason, txt = SUPPORT_ACTION.HELP, SUPPORT_REASON.DUTY, LOC"SUPPORT_DESC.PROTECT_WORKPLACE"
                end

            end


        end

    end

    --give the aspects a chance to modify our reasons
    for k,aspect in pairs(agent:GetAspects()) do
        local aspect_support, aspect_reason, aspect_text = aspect:CheckCombatSupport(subject, sup)
        if aspect_support then
            sup, reason, txt = aspect_support, aspect_reason, aspect_text
        end
    end

    return sup, reason, txt
end

function AgentUtil.GetNegotiationSupport( agent, subject, is_hostile, no_bystanders )

    --no card to contribute
    if not agent.support_negotiation and not agent:IsSentient() then
        return nil
    end

    local sup, reason, txt
    for k,quest in TheGame:GetGameState():ActiveQuests() do
        sup, reason, txt = quest:GetNegotiationSupport(agent, subject, is_hostile)
    end

    if not sup then
        if agent:IsCaptiveMember() then
            sup, reason, txt = SUPPORT_ACTION.STAY_OUT, SUPPORT_REASON.CAPTIVE, LOC"SUPPORT_DESC.IS_CAPTIVE"

        elseif agent:IsInPlayerParty() then
            if agent:IsPet() then
                 -- Pets always help.
                sup, reason, txt = SUPPORT_ACTION.HELP, SUPPORT_REASON.DUTY, loc.format( LOC"SUPPORT_DESC.IS_YOUR_PET", agent )

            elseif agent:IsEscortMember() then
                sup, reason, txt = SUPPORT_ACTION.HELP, SUPPORT_REASON.PARTY, loc.format( LOC"SUPPORT_DESC.ESCORT", agent )

            elseif agent:IsCrewMember() then
                sup, reason, txt = SUPPORT_ACTION.HELP, SUPPORT_REASON.PARTY, loc.format( LOC"SUPPORT_DESC.HIRED", agent )
            end
        elseif not no_bystanders then
            local player = TheGame:GetGameState():GetPlayerAgent()
            local player_rel = agent:GetRelationship()
            local subject_rel = subject and agent:GetRelationship( subject ) or RELATIONSHIP.NEUTRAL
     
            if player_rel < RELATIONSHIP.NEUTRAL then
                --agent dislikes you. Will join in unless manipulated, or they also dislike the subject
                if subject_rel < RELATIONSHIP.NEUTRAL then
                    sup, reason, txt = SUPPORT_ACTION.STAY_OUT, SUPPORT_REASON.PLAYER_HATRED, LOC"SUPPORT_DESC.DISLIKES_BOTH"
                else
                    sup, reason, txt = SUPPORT_ACTION.HINDER, SUPPORT_REASON.PLAYER_HATRED, loc.format( LOC"SUPPORT_DESC.DISLIKES_YOU", agent:LocTable() )
                end

            elseif player_rel > RELATIONSHIP.NEUTRAL then
                if subject_rel < player_rel then
                    sup, reason, txt = SUPPORT_ACTION.HELP, SUPPORT_REASON.PLAYER_FRIENDSHIP, loc.format( LOC"SUPPORT_DESC.LIKES_YOU", agent:LocTable() )
                elseif is_hostile then
                    sup, reason, txt = SUPPORT_ACTION.STAY_OUT, SUPPORT_REASON.PLAYER_FRIENDSHIP, LOC"SUPPORT_DESC.LIKES_YOU_BOTH"
                end

            elseif subject_rel > RELATIONSHIP.NEUTRAL then
                if is_hostile then
                    sup, reason, txt =  SUPPORT_ACTION.HINDER, SUPPORT_REASON.TARGET_FRIENDSHIP, loc.format( LOC"SUPPORT_DESC.LIKES_SUBJECT", subject:LocTable() )
                end
            end
        end
    end

    --give the aspects a chance to modify our reasons
    for k,aspect in pairs(agent:GetAspects()) do
        local aspect_support, aspect_reason, aspect_txt = aspect:CheckNegotiationSupport(subject, sup, is_hostile)
        if aspect_support then
            sup, reason, txt = aspect_support, aspect_reason, aspect_txt
        end
    end
    local params = {sup = sup, reason = reason, txt = txt}
    TheGame:BroadcastEvent( "get_negotiation_support", agent, subject, params )

    return params.sup, params.reason, params.txt
end

function AgentUtil.GetLocationSupport( agent )
    if agent:InLimbo() then
        return false, false, false, false
    end

    local can_bribe_combat, can_bribe_negotiation, can_payoff_combat, can_payoff_negotiation

    -- Loop through people in the room to see what manipulations are available.
    for k, target_agent in agent:GetLocation():Agents() do
        local can_fight = AgentUtil.IsCombatTarget( target_agent )
        local can_negotiate = AgentUtil.IsNegotiationTarget( target_agent )
        local is_target = can_fight or can_negotiate

        local your_captive = target_agent:IsInPlayerParty() and target_agent:IsCaptiveMember()
        if is_target and not your_captive then
            if can_fight then
                local combat_sup, combat_reason = AgentUtil.GetCombatSupport( agent, target_agent, false )
                if combat_sup == nil then
                    can_bribe_combat = true
                elseif combat_sup == SUPPORT_ACTION.HINDER then
                    can_payoff_combat = true
                end
            end
            
            if can_negotiate then
                local negotiation_sup, negotiation_reason = AgentUtil.GetNegotiationSupport( agent, target_agent )
                if negotiation_sup == nil then
                    can_bribe_negotiation = true
                elseif negotiation_sup == SUPPORT_ACTION.HINDER then
                    can_payoff_negotiation = true
                end
            end
        end
    end

    return can_bribe_combat, can_bribe_negotiation, can_payoff_combat, can_payoff_negotiation
end


function AgentUtil.RemoveNegotiationCard( agent, on_select_fn )
    local function OnSelectCard( screen, widget, card )
        if widget and widget.card then
            local defs = require "negotiation/negotiation_defs"
            local body

            if widget.card:IsFlagged( defs.CARD_FLAGS.PARASITE ) then 
                body = loc.format(LOC"UI.CARDS.REMOVE_PARASITE_CONFIRM_TEXT", widget.card:GetName(), PARASITE_HEALTH_LOSS)
            else
                body = loc.format(LOC"UI.CARDS.REMOVE_CARD_CONFIRM_TEXT", widget.card:GetName())
            end
            TheGame:FE():PushScreen( Screen.YesNoPopup( LOC"UI.CARDS.REMOVE_CARD_CONFIRM_TITLE", body ) )
                :SetFn( function(v)
                    if v == Screen.YesNoPopup.YES then 
                        if widget then
                            screen:ShowRemoval( widget )
                            AUDIO:PlayEvent("event:/ui/select_cards/remove_card")
                            agent.negotiator:RemoveCard( widget.card )
                            StatUtil.RemovedCard(widget.card.id)
                			SendMetricsData("REMOVED_NEGOTIATION_CARD", {SELECTED_ID = widget.card.id})
                        end
                        if on_select_fn then
                            on_select_fn( widget and widget.card )
                        end
                        
                    end 

                end )
        else
            if on_select_fn then
                on_select_fn( )
            end
        end
    end

    local cards = agent.negotiator:GetCards()
    local screen = Screen.DeckScreen( cards, OnSelectCard, Widget.NegotiationCard )
    screen:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
    screen:SetTitles( LOC"UI.SHOW_DECK_SCREEN.NEGOTIATION_DECK_UPPERCASE", LOC"UI.CARDS.REMOVE_NEGOTIATION_CARD" )
    TheGame:FE():InsertScreen( screen )
end


function AgentUtil.RemoveBattleCard( agent, on_select_fn, no_skip )
    
    local function OnSelectCard( screen, widget, card )
        if widget and widget.card then
            local defs = require "battle/battle_defs"
            local body
            if widget.card:IsFlagged( defs.CARD_FLAGS.PARASITE ) then 
                body = loc.format(LOC"UI.CARDS.REMOVE_PARASITE_CONFIRM_TEXT", widget.card:GetName(), PARASITE_HEALTH_LOSS)
            else
                body = loc.format(LOC"UI.CARDS.REMOVE_CARD_CONFIRM_TEXT", widget.card:GetName() )
            end

            TheGame:FE():PushScreen( Screen.YesNoPopup( LOC"UI.CARDS.REMOVE_CARD_CONFIRM_TITLE", body ))
                :SetFn( function(v)
                    if v == Screen.YesNoPopup.YES then 
                        if widget then
                            screen:ShowRemoval( widget )
                            AUDIO:PlayEvent("event:/ui/select_cards/remove_card")
                            agent.battler:RemoveCard( widget.card )
                            StatUtil.RemovedCard(widget.card.id)
                            SendMetricsData("REMOVED_BATTLE_CARD", {SELECTED_ID = widget.card.id})
                        end
                        if on_select_fn then
                            on_select_fn( widget and widget.card )
                        end
                    end 
                end )
        else
            if on_select_fn then
                on_select_fn( )
            end
        end

    end

    local cards = agent.battler:GetCards()
    local screen = Screen.DeckScreen( cards, OnSelectCard, Widget.BattleCard, nil, nil, no_skip )
    screen:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
    screen:SetTitles( LOC"UI.SHOW_DECK_SCREEN.BATTLE_DECK_UPPERCASE", LOC"UI.CARDS.REMOVE_BATTLE_CARD" )
    TheGame:FE():InsertScreen( screen )
end

local function ShowUpgradeCard( widget_class, aspect_id, on_select_fn )
    local upgraded_card

    local function OnSelectCard( screen, widget, card )
        if widget and upgraded_card == nil then
            local preview_only = false
            local popup = Screen.UpgradeCardPopup( widget.card, widget.card.owner, preview_only, function( chosen_card )
                if chosen_card then
                    upgraded_card = chosen_card
                    screen:ShowUpgradeAndClose( widget, upgraded_card )
                else

                end
            end )
            TheGame:FE():PushScreen( popup )
        end
    end

    local function OnDone()
        if on_select_fn then
            on_select_fn( upgraded_card )
        end
    end

    local agent = TheGame:GetGameState():GetPlayerAgent()
    local cards = {}
    for i, card in ipairs( agent:GetAspect( aspect_id ):GetCards() ) do
        if card:HasUpgrades() and not card:IsHatched() then
            table.insert( cards, card )
        end
    end
    local screen = Screen.DeckScreen( cards, OnSelectCard, widget_class, OnDone )
    screen:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
    return screen
end

function AgentUtil.UpgradeNegotiationCard( on_select_fn )
    
    local screen = ShowUpgradeCard( Widget.NegotiationCard, "negotiator", function(...) on_select_fn(...) end )
    screen:SetTitles( LOC"UI.SHOW_DECK_SCREEN.NEGOTIATION_DECK_UPPERCASE", LOC"UI.CARDS.UPGRADE_NEGOTIATION_CARD" )
    TheGame:FE():InsertScreen( screen )
end

function AgentUtil.UpgradeBattleCard( on_select_fn )
    
    local screen = ShowUpgradeCard( Widget.BattleCard, "battler",function(...) on_select_fn(...) end )
    screen:SetTitles( LOC"UI.SHOW_DECK_SCREEN.BATTLE_DECK_UPPERCASE", LOC"UI.CARDS.UPGRADE_BATTLE_CARD" )
    TheGame:FE():InsertScreen( screen )
end

function AgentUtil.ChooseBattleCard( agent, on_select_fn )
    local function OnSelectCard( screen, widget, card )
        if widget then
            screen:Close()
        end

        if on_select_fn then
            on_select_fn( widget and widget.card )
        end
    end

    local cards = agent.battler:GetCards()
    local screen = Screen.DeckScreen( cards, OnSelectCard, Widget.BattleCard )
    screen:SetTitles( LOC"UI.SHOW_DECK_SCREEN.BATTLE_DECK_UPPERCASE", LOC"UI.CARDS.CHOOSE_BATTLE_CARD" )
    TheGame:FE():InsertScreen( screen )
end

function AgentUtil.LearnCard( card_id )
    local def = Content.GetBattleCard( card_id )
    if def then
        return TheGame:GetGameState():GetPlayerAgent().battler:LearnCard( card_id )
    end

    def = Content.GetNegotiationCard( card_id )
    if def then
        return TheGame:GetGameState():GetPlayerAgent().negotiator:LearnCard( card_id )
    end
end

function AgentUtil.IsCoworker(agent1, agent2)
    return agent1:GetBrain() and agent2:GetBrain() and (agent1:GetBrain():GetWorkplace() == agent2:GetBrain():GetWorkplace())
end

function AgentUtil.IsFriend(agent1, agent2)
    return agent1.social_connections and agent1.social_connections:IsFriend(agent2)
end

function AgentUtil.IsEnemy(agent1, agent2)
    return agent1.social_connections and agent1.social_connections:IsEnemy(agent2)
end

function AgentUtil.HasEnemies(agent)
    return agent.social_connections and agent.social_connections:HasEnemies(agent)
end

function AgentUtil.IsProprietor(agent)
    return agent:GetRoleAtLocation() == "PROPRIETOR" or agent:GetRoleAtLocation() == "ASSISTANT"
end

function AgentUtil.IsInHiding(agent)
    if not agent:IsPlayer() then
        if agent.in_hiding then
            return true
        end

        for k,quest in TheGame:GetGameState():ActiveQuests() do
            if quest:IsInHiding(agent) then
                return true
            end
        end
    end
    
    return false
end

function AgentUtil.CanAct(agent)
    if agent:IsDead() then
        return false
    end

    if agent:IsInPlayerParty() then
        return false
    end

    if AgentUtil.IsInHiding(agent) then
        return false
    end


    return true
end

function AgentUtil.SpawnTransientParty(defs, leader, try_find_existing_leader, leader_filter_fn, num_skinned_followers)
    print ("AgentUtil.SpawnTransientParty", leader)
    local existing_defs = ObtainWorkTable()
    if leader then
        print ("\tuse existing leader", leader)
        for i, v in leader:Party() do
            if v ~= leader then
                table.insert( existing_defs, v:GetContentID() )
            end
        end
    end

    local agents = {}
    for k, def_id in ipairs(defs) do
        if table.arrayremove( existing_defs, def_id ) then
            print ("\tSKIPPING ", def_id)
            -- Skip: already exists.
        else
            local agent
            if not leader and try_find_existing_leader then
                agent = AgentUtil.GetFreeAgent(def_id, leader_filter_fn)
                print ("\tGOT FREE AGENT FOR LEADER", agent)
            else
                if not leader or (num_skinned_followers and k <= num_skinned_followers+1) then
                    agent = TheGame:GetGameState():AddSkinnedAgent( def_id )
                    print ("\tGOT SKINNED AGENT", agent)
                else
                    agent = TheGame:GetGameState():AddAgent( Agent(def_id ))
                    print ("\tGOT GENERIC AGENT", agent)
                end
                
            end

            if leader then
                agent:JoinParty(leader)
                agent:MoveToAgent( leader )
            else
                leader = agent
            end    
            table.insert(agents, agent)
        end
    end

    ReleaseWorkTable( existing_defs )

    return agents, leader
end

function AgentUtil.CommitMurder( victim_agent, ignore_agents )
    if not victim_agent:IsSentient()  then      --or not victim_agent:IsUnique()
        return 
    end

    local friend
    if victim_agent.social_connections then
        friend = victim_agent.social_connections:GetBestFriend(function(agent)
            return not agent:IsPlayer() and agent:IsUnique() and not AgentUtil.HasPlotArmour(agent)
                and agent:GetRelationship() ~= RELATIONSHIP.HATED
                and not (ignore_agents and ignore_agents[agent] )
                and not agent:HasQuestMembership() end)
        --

        if not friend then
            friend = AgentUtil.PickAgentWeight(function(agent) 
                    local should_ignore = ignore_agents and ignore_agents[agent]
                    if not should_ignore and (agent ~= victim_agent and agent:IsSentient() and agent:IsUnique() and not AgentUtil.HasPlotArmour(agent) and agent:GetFaction() == victim_agent:GetFaction() and agent:GetRelationship(victim_agent) == RELATIONSHIP.NEUTRAL) and agent:GetRelationship() ~= RELATIONSHIP.HATED and not agent:HasQuestMembership() then
                        local score = 1
                        if agent:GetContentID() == victim_agent:GetContentID() then
                            score = score + .5
                        end

                        if agent:GetRelationship() == RELATIONSHIP.DISLIKED then
                            score = score + 1.5
                        end
                        return score

                    end
                end)

            if friend then
                friend.social_connections:SetRelationship(RELATIONSHIP.LIKED, victim_agent)
            end
        end

        if not friend and not AgentUtil.HasPlotArmour(victim_agent) and not victim_agent.unique then
            friend = TheGame:GetGameState():AddSkinnedAgent( victim_agent:GetContentID() )
            if friend then
                friend.social_connections:SetRelationship(RELATIONSHIP.LIKED, victim_agent)
            end
        end

        local murder = { victim_agent = victim_agent, friend = friend }
        TheGame:BroadcastEvent( "commit_murder", murder )
        friend = murder.friend

        if friend then
            friend:OpinionEvent(OPINION.KILLED_FRIEND, {revenge_agent = victim_agent} )
        end
    end

    return friend
end

function AgentUtil.HasPlotArmour(agent)
    if agent:HasTag("plot_armour") then
        return true
    end

    return AgentUtil.HasDynamicPlotArmour(agent)
end

function AgentUtil.HasDynamicPlotArmour(agent)
    if agent.quest_membership then
        for _,quest in pairs(agent.quest_membership) do
            if quest:IsActive() and quest:GetQuestDef().plot_armour_fn then
                if quest:GetQuestDef().plot_armour_fn(quest, agent) then
                    return true
                end
            end
        end
    end
    return false
end


function AgentUtil.HasSideQuest(agent)
    if agent.quest_membership then
        for _,quest in pairs(agent.quest_membership) do
            if quest:GetProvider() == agent and quest:IsInactive() then
                return true, quest
            end
        end
    end
end

function AgentUtil.CancelInactiveSideQuests(agent)
    if agent.quest_membership then
        local t = {}
        for _,quest in pairs(agent.quest_membership) do
            if quest:GetProvider() == agent and quest:IsInactive() then
                table.insert(t, quest)
            end
        end
        
        for k,quest in ipairs(t) do
            quest:Cancel()
        end

    end
end

function AgentUtil.CanBePicked(agent)
    return not agent:HasQuestMembership() and 
        agent:IsUnique() and 
        (agent:GetBrain() and not agent:GetBrain():IsOnDuty()) and 
        AgentUtil.CanAct(agent)
end



function AgentUtil.GetFreeAgent(def, filter)
    local candidates = ObtainWorkTable()

    for k,agent in pairs(TheGame:GetGameState().agents) do
        if agent:GetContentID() == def and 
        (filter == nil or filter(agent)) and 
        AgentUtil.CanBePicked(agent) then

            table.insert(candidates, agent)
        end
    end    

    local ret = table.arraypick(candidates)
    ReleaseWorkTable(candidates)
    
    if ret then
        if ret:GetParty() then
            ret:GetParty():Disband()
        end
    else
        ret = TheGame:GetGameState():AddSkinnedAgent(def)
    end

    return ret
end

function AgentUtil.PickAgentWeight(weightfn)
    local candidates = ObtainWorkTable()

    for k,agent in pairs(TheGame:GetGameState().agents) do
        local w = weightfn(agent)
        if w and w > 0 then
            candidates[agent] = w
        end
    end    

    local ret
    if next(candidates) then
        ret =  weightedpick( candidates )
    end

    return ret
end

function AgentUtil.CanAgentPatronizeLocation(loc, agent)
    
    if not agent:IsSentient() then
        return false
    end
    
    if not loc:GetContent().patron_data then
        return false
    end

    if loc:GetContent().patron_data.filter then
        if not loc:GetContent().patron_data.filter(loc, agent) then
            return false
        end
    end

    if AgentUtil.IsAgentBarredFromLocation(agent, loc) then
        return false
    end

    if agent:GetFaction():GetFactionRelationship( loc:GetFactionID() ) == RELATIONSHIP.HATED then
        return false
    end

    if AgentUtil.HasSideQuest(agent) then
        for _, other_agent in loc:Agents() do
            if AgentUtil.HasSideQuest(other_agent) then
                return false
            end
        end
    end

    return true
end


function AgentUtil.PutAgentInWorld(agent)

    if not agent:InLimbo() then
        return
    end

    local health = agent:GetAspect( "health" )
    if health then
        health:Restore( 0.25 )
    end

    local location, role = TheGame:GetGameState():GetQuestLocationForAgent(agent)
    if location then
        agent:ClearParty()
        agent:MoveToLocation(location)
        agent:SetLocationRole(role)
        return
    end

    
    if agent:GetBrain():GetWorkPosition() and agent:GetBrain():GetWorkPosition():ShouldBeWorking() and not AgentUtil.IsAgentBarredFromLocation(agent, agent:GetBrain():GetWorkplace()) then
        agent:GetBrain():SendToWork()
        return
    end

    local candidates = {}
    if math.random() > 0.33 then
        local player_location = TheGame:GetGameState():GetPlayerAgent():GetLocation()
        for _, location in TheGame:GetGameState():AllLocations() do
            if location ~= player_location then
                local cap = location:GetCurrentPatronCapacity()
                local num = location:GetNumCurrentPatrons()
                if cap > 0 and num < cap and AgentUtil.CanAgentPatronizeLocation(location, agent) then
                    table.insert(candidates, location)
                end
            end
        end
    end
    
    if #candidates > 0 then
        local location = table.arraypick(candidates)
        agent:GetBrain():SendToPatronize(location)
    else
        agent:GetBrain():MoveToHome()
    end
end


function AgentUtil.CanUseCard( agent, card_def)
    if agent then
        if agent.card_series and table.arrayfind(agent.card_series, card_def.series) then
            return true
        end
    end
    return false
end

function AgentUtil.CanUseGraft(agent, graft_def)
    if graft_def.type == GRAFT_TYPE.SOCIAL then
        return true
    end
    
    if agent.graft_series and table.arrayfind(agent.graft_series, graft_def.series) then
        if graft_def.restrictions == nil then
            return true
        else
            return graft_def.restrictions(agent)
        end
    else
        return false
    end
end

function AgentUtil.IsSympathizer(agent, faction_id)
    return agent:GetAspect("sympathizer") and agent:GetAspect("sympathizer").faction == faction_id
end


--this is dumb
local ANTIS = 
{
    SPARK_BARONS = "RISE",
    RISE = "SPARK_BARONS",
    ADMIRALTY = "BANDITS",
    BANDITS = "ADMIRALTY",
}

function AgentUtil.IsAntiSympathizer(agent, faction_id)
    if ANTIS[faction_id] then
        return agent:GetAspect("sympathizer") and agent:GetAspect("sympathizer").faction == ANTIS[faction_id]
    end
end

function AgentUtil.MakeSympathizer(agent, faction_id)
    if agent:GetAspect("sympathizer") then
        agent:GetAspect("sympathizer").faction = faction_id
    else
        agent:AddAspect("sympathizer", faction_id, true)
    end
end

function AgentUtil.CreateSympathyModifiers(agent, positive, negative, modifiers)
    if positive and AgentUtil.IsSympathizer(agent, positive) then
        modifiers = modifiers or {}
        table.insert( modifiers, {value = -6, text = loc.format(LOC"SUPPORT_DESC.SYMPATHIZES_WITH", TheGame:GetGameState():GetFaction(positive):GetName())})
    end

    if negative and AgentUtil.IsSympathizer(agent, negative) then
        modifiers = modifiers or {}
        table.insert( modifiers, {value = 6, text = loc.format(LOC"SUPPORT_DESC.SYMPATHIZES_WITH", TheGame:GetGameState():GetFaction(negative):GetName())})
    end

    return modifiers
end


function AgentUtil.CountAlive(table_of_agents)
    local num = 0
    for k,v in ipairs(table_of_agents) do
        if not v:IsDead() and v:IsSentient() then
            num = num + 1   
        end
    end
    return num
end


function AgentUtil.IsCombatTarget(agent)
    for k,quest in TheGame:GetGameState():ActiveQuests() do
        if quest:IsCombatTarget(agent) then
            return true
        end
    end
    return false
end

function AgentUtil.IsNegotiationTarget(agent)
    for k,quest in TheGame:GetGameState():ActiveQuests() do
        if quest:IsNegotiationTarget(agent) then
            return true
        end
    end
    return false
end


function AgentUtil.IsAgentBarredFromLocation(agent, location)
    for _,quest in TheGame:GetGameState():ActiveQuests() do
        if quest:IsAgentBarredFromLocation(agent, location) then
            return true
        end
    end

    return false
end

function AgentUtil.HasJob(agent)
    return agent:GetBrain():GetWorkPosition() ~= nil
end

function AgentUtil.CollectDeathLoot(agent, loot)
    loot = loot or {money = 0, items = {}, grafts = {}}
    
    local money = agent:GetDeathMoney()
    if money > 0 then
        loot.money = (loot.money or 0) + math.round( math.randomGauss( money*.8, money*1.2 ))
    end
        
    local item = agent:GetDeathItem()
    if item then
        table.insert( loot.items,item)
    end
        
    return loot
end

function AgentUtil.IsFullyUpgraded( agent )
    if agent.battler and not agent.battler:IsFullyUpgraded() then
        return false
    end

    if agent.negotiator and not agent.negotiator:IsFullyUpgraded() then
        return false
    end

    if agent.graft_owner and not agent.graft_owner:IsFullyUpgraded() then
        return false
    end

    return true
end


function AgentUtil.DebugSpeakTo(agent)
    local location = agent:GetLocation()
    if location then
        TheGame:GetGameState():GetCaravan():MoveToLocation( location )

        local screen = TheGame:FE():GetTopScreen()
        if is_instance( screen, Screen.LocationScreen ) then
            screen:EnterConvo( agent )
        end
    end
end

function AgentUtil.GetOrSpawnAgentbyAlias(alias)
    local agent = TheGame:GetGameState():GetAgentOrMemento( alias )
    if agent == nil then
        local def = Content.GetCharacterDef( alias )
        local content_id, skin_table
        if def then
            content_id = def.id
            local skins = Content.GetAllCharacterSkins( content_id )
            if skins and #skins == 1 then
                -- If there's a singular skin, use it automatically.
                skin_table = skins[1]
            end

        else
            content_id, skin_table = Content.GetCharacterSkinByAlias( alias )
            print (alias, content_id, skin_table)
            if skin_table and not TheGame:GetGameState():IsSkinAvailable( skin_table.uuid ) then
                content_id, skin_table = nil, nil
            end
        end
        if content_id then
            agent = Agent( content_id, skin_table )
        end
    end
    return agent
end