local Aspect = class ("Aspect")

Aspect.SortCompare = function( x, y )
    return x.id < y.id -- *shrug*
end
Aspect.can_reload = true

function Aspect:init( agent )
    self.agent = agent
end

function Aspect:GetDef()
    return self.agent and self.agent.def and self.agent.def.aspects and self.agent.def.aspects[self.id]
end

function Aspect:GetContentID()
    return self.id -- Assigned in Content.AddAspect.
end

function Aspect:ShouldShow( agent )
    return self.texture ~= nil
end

function Aspect:GetStacks()
    return self.current_stacks or 1, self.max_stacks or 1
end

function Aspect:DeltaStacks( delta )
    local stacks, max_stacks = self:GetStacks()
    self.current_stacks = math.min( stacks + delta, max_stacks )
    
    self.agent:BroadcastEvent( "aspects_changed", self.agent, true, self )
    
    if self.current_stacks <= 0 then
        self.agent:RemoveAspect( self.id )
    end
end

function Aspect:CheckNegotiationSupport(target, base_reason, is_hostile)
end

function Aspect:CheckCombatSupport(target, base_reason)
end

function Aspect:GetTexture()
    return self.texture
end

function Aspect:IsUnknownTrait()
    return false
end

function Aspect:IsSkill()
    return self.skill == true
end

function Aspect:IsTrait()
    return self.trait == true
end

function Aspect:IsStatusEffect()
    return self.status_effect == true
end

function Aspect:IsQuipTag()
    return self.trait or self.status_effect or self.quippable
end

function Aspect:GetName()
	local key = self:GetLocNameKey()
	local name = loc.format( LOC(key), self.agent )
	return name
end


function Aspect:UpdateAspect()
end

function Aspect:FormatDesc()
    local desc = ObtainWorkTable()

    table.insert(desc, loc.format( "<#ASPECT><b>{1}</></>", self:GetName() ))

    local aspect_desc
    if self.GetDesc then
        aspect_desc = self:GetDesc( TheGame:GetGameState(), self.agent )
    elseif self.desc then
		local desc_key = self:GetLocDescKey()
		local desc_str = LOC(desc_key)

		if self.desc_fn then
			aspect_desc = self.desc_fn(self, desc_str)
		else
			aspect_desc = desc_str

		end
    end
    if aspect_desc then
        table.insert( desc, aspect_desc )
    end

    if self.negotiation_cards then
        for i = 1, #self.negotiation_cards, 2 do
            local card_id, count = self.negotiation_cards[i], ( self.negotiation_cards[i+1] * self:GetStacks() )
          
            table.insert(desc, loc.format(LOC"ASPECT.NEGOTIATION_CARD_INFO", count, card_id))

        end
    end

    if self.battle_cards then
        for i = 1, #self.battle_cards, 2 do
            local card_id, count = self.battle_cards[i], ( self.battle_cards[i+1] * self:GetStacks() )
            table.insert(desc, loc.format(LOC"ASPECT.BATTLE_CARD_INFO", count, card_id ))
        end
    end


    local desc_concat = table.concat(desc, "\n")
    ReleaseWorkTable(desc)

    return desc_concat
end

function Aspect:Save()
    return self
end

function Aspect:GetLocPrefix()
	return "ASPECT." .. string.upper(self.id)
end

function Aspect:GetLocNameKey()
	return self:GetLocPrefix() .. ".NAME"
end

function Aspect:GetLocDescKey()
	return self:GetLocPrefix() .. ".DESC"
end

function Aspect:HarvestStrings(t)
	if self.name then
		t[self:GetLocNameKey()] = self.name
	end

	if self.desc then
		t[self:GetLocDescKey()] = self.desc
	end
end
