local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

local MAX_DAMAGE = 9999

---------------------------------------------------------------------
-- Used for calculating attack damage using BATTLE_EVENT.CALC_DAMAGE

local DamageAccumulator = class( "Battle.DamageAccumulator" )
DamageAccumulator.can_reload = true

function DamageAccumulator:init( battle )
    self.battle = battle
    self.details = {}
    self.sources = {}
end

function DamageAccumulator:IsSource( source )
    return table.contains( self.sources, source )
end

function DamageAccumulator:IsMinDamagePenalized()
    return self.min_damage_penalized == true
end

function DamageAccumulator:IsMaxDamagePenalized()
    return self.max_damage_penalized == true
end

function DamageAccumulator:PreviewDamage( card, target, hit )
    table.clear( self.details )
    table.clear( self.sources )
    self.preview = true
    self.min_damage_modified = nil
    self.max_damage_modified = nil

    -- Accumulate modifiers.
    if card.CalculateHitDamage then
        self.min_damage, self.max_damage = card:CalculateHitDamage( target, nil, hit )
    else
        self.min_damage, self.max_damage = card.min_damage, card.max_damage
    end
    
    self.battle:BroadcastEvent( BATTLE_EVENT.CALC_DAMAGE, card, target, self, hit )
    
    if card:IsFlagged(battle_defs.CARD_FLAGS.RESTRAINED) then
        self.min_damage = math.min(self.min_damage, card.min_damage)
        self.max_damage = math.min(self.max_damage, card.max_damage)
        self.min_damage_modified, self.max_damage_modified = nil, nil
    end

    local str
    if #self.details > 0 then
        str = table.concat( self.details, "\n" )
    else
        str = nil
    end

    self.max_damage = clamp( self.max_damage, 0, MAX_DAMAGE )
    self.min_damage = clamp( self.min_damage, 0, self.max_damage )

    return self.min_damage, self.max_damage, str, self
end

function DamageAccumulator:AddDamage( min_damage, max_damage, source )
    self:ModifyDamage( self.min_damage + min_damage, self.max_damage + max_damage, source )
end

function DamageAccumulator:ModifyDamage( min_damage, max_damage, source )
    if min_damage ~= self.min_damage then
        self.min_damage_penalized = self.min_damage_penalized or min_damage < self.min_damage
        self.min_damage = min_damage --clamp( min_damage, 0, MAX_DAMAGE )
    end
    if max_damage ~= self.max_damage then
        self.max_damage_penalized = self.max_damage_penalized or max_damage < self.max_damage
        self.max_damage_modified = true
    end
    self.max_damage = max_damage --clamp( max_damage, self.min_damage, MAX_DAMAGE )

    local detail
    if is_instance( source, Battle.Card ) then
        detail = source:GetName()
    elseif is_instance( source, CombatCondition ) then
        detail = source:GetName()
    else
        detail = tostring(source)
    end

    if detail then
        table.insert( self.details, detail )
    end
    if source then
        table.insert( self.sources, source )
    end
end
