local FighterManager = class("FighterManager")

function FighterManager:init( sim )
    self.sim = sim
    self.ent_by_fighter = {}
end

function FighterManager:AssignBattle( battle )
    self.battle = battle
end

function FighterManager:CalculateHomePositions( team, move_home )
    local fighters = team:GetFighters()
    local primary_fighter = team:Primary()
    local formation = {}
    if team:GetCustomFormation() then
        formation = shallowcopy( team:GetCustomFormation() )
    else
        formation = shallowcopy( FIGHT_FORMATIONS[ #fighters ] )
    end
        
    local winning_assignment = nil
    local winning_score = nil

    local assignments = permutations(fighters)
    for k,assignment in ipairs(assignments) do
        
        local total_score = 0
        
        for k, fighter in ipairs(assignment) do
            local slot = formation[k]
            
            local fighter_formation = fighter.formation or 0
            local slot_formation = slot[3] or 0
            local fighter_score = 0

            if (fighter_formation & FIGHTER_FORMATION.FRONT_X) ~= 0 then
                if (slot_formation & FIGHTER_FORMATION.FRONT_X) ~= 0 then
                    fighter_score = fighter_score + 10
                elseif (slot_formation & FIGHTER_FORMATION.BACK_X) ~= 0 then
                    fighter_score = fighter_score - 10
                end
            elseif (fighter_formation & FIGHTER_FORMATION.BACK_X) ~= 0 then 
                if (slot_formation & FIGHTER_FORMATION.BACK_X) ~= 0 then
                    fighter_score = fighter_score + 10
                elseif (slot_formation & FIGHTER_FORMATION.FRONT_X) ~= 0 then
                    fighter_score = fighter_score - 10
                end
            end

            if (fighter_formation & FIGHTER_FORMATION.FRONT_Z) ~= 0 then
                if (slot_formation & FIGHTER_FORMATION.FRONT_Z) ~= 0 then
                    fighter_score = fighter_score + 10
                elseif (slot_formation & FIGHTER_FORMATION.BACK_Z) ~= 0 then
                    fighter_score = fighter_score - 10
                end
            elseif (fighter_formation & FIGHTER_FORMATION.BACK_Z) ~= 0 then 
                if (slot_formation & FIGHTER_FORMATION.BACK_Z) ~= 0 then
                    fighter_score = fighter_score + 10
                elseif (slot_formation & FIGHTER_FORMATION.FRONT_Z) ~= 0 then 
                    fighter_score = fighter_score - 10
                end
            end

            if fighter.agent.boss then
                fighter_score = fighter_score * 2
            end

            local ent = self.ent_by_fighter[ fighter ]
            if ent then
                local cx, cy, cz = ent.cmp.Transform:GetLocalPosition()
                if team:GetID() == TEAM.BLUE then
                    cx = -cx
                end
                local d = distsq(cx, cz, slot[1], slot[2])
                if d <= 10 then
                    if fighter_score >= 0 then
                        fighter_score = fighter_score + 5 
                    end
                end
                
            end


            total_score = total_score + fighter_score
        end

        if winning_score == nil or total_score > winning_score then
            winning_score = total_score
            winning_assignment = assignment
        end
    end


    if winning_assignment then

        for k,fighter in ipairs(winning_assignment) do
            local slot = formation[k]
            local x0, z0 = slot[ 1 ], slot[ 2 ]
            if team:GetID() == TEAM.BLUE then
                x0 = -x0
            end
            
            local ent = self.ent_by_fighter[ fighter ]
            if ent then
                local prevx, prevz = ent.cmp.AnimFighter:GetHomePosition()
                
                if prevx ~= x0 or prevz ~= z0 then
                    ent.cmp.AnimFighter:SetHomePosition( x0, z0 )
                    if move_home and not ent.cmp.AnimFighter:IsBusy() then
                        ent.cmp.AnimFighter:DoSequence( AnimFighter.ResetFighter, ent.cmp.AnimFighter )
                    end
                end            
            end
        end
    end



end

function FighterManager:AddFighter(fighter)
    local ent = self.sim:SpawnPrefab("fight_agent")
    ent.cmp.AnimFighter:SetFighter(fighter, self.battle)
    ent.cmp.AnimController:SetAnimSpeed( "idle", 1 / self.sim:GetTimeScale() )

    self.ent_by_fighter[fighter] = ent

    if fighter:GetTeam() then
        self:CalculateHomePositions( fighter:GetTeam() )
    end

    return ent
end

function FighterManager:RemoveFighter(fighter)
    local ent = self.ent_by_fighter[fighter]
    self.ent_by_fighter[ fighter ] = nil
end

function FighterManager:GetAnimFighterByFighter(fighter)
    return self.ent_by_fighter[fighter]
end
