local Brain = class("Brain")

function Brain:init( agent )
    self.agent = agent
end

function Brain:IsOnDuty()
    return self.workposition and self.workposition:ShouldBeWorking() and self.workposition:GetLocation() == self.agent:GetLocation()
end

function Brain:IsPatronizing()
    return self.agent:GetRoleAtLocation() == CHARACTER_ROLES.PATRON
end

function Brain:SendToWork()
    if self.workposition and self.workposition:ShouldBeWorking() then
        self.agent:MoveToLocation(self.workposition:GetLocation())
        self.agent:SetLocationRole(self.workposition:GetRole())
    end
end

function Brain:PostLoad()
    if self.workposition and not self.workposition.content then
        self.workposition = nil
    end
end
    


function Brain:SendToPatronize(loc)
    self.agent:ClearParty()
    self.agent:MoveToLocation(loc)
    self.agent:SetLocationRole(CHARACTER_ROLES.PATRON)

    if loc:GetContent().patron_data and loc:GetContent().patron_data.start_fn then
        loc:GetContent().patron_data.start_fn(loc, self.agent)
    end
end

function Brain:Clear()
    self:SetHome( nil )
    if self.workposition then
        self.workposition:Fire()
    end
    assert( self.home_location == nil )
end



function Brain:GetWorkPosition()
    return self.workposition
end


function Brain:Save()
    return
    {
        agent = self.agent,
        home_location = self.home_location,
        home_generator = self.home_generator,
        workposition = self.workposition,
    }
end


function Brain:GetWorkplace()
    return self.workposition and self.workposition:GetLocation()
end


function Brain:SetWork( workposition )
    if workposition and self.workposition then
        assert(nil, "We already have a job!")
    end
    self.workposition = workposition
    self.agent:BroadcastEvent("work_changed", self.agent)
end


function Brain:GenerateHome( )
    
    if not self.agent:IsSentient() or self.agent:IsPlayer() then
        return
    end
    if not self.home_location then
        if TheGame:GetGameState():GetWorldRegion():GetContent().home_generator then
            local location_id = TheGame:GetGameState():GetWorldRegion():GetContent().home_generator(self.agent)
            if location_id then
                local location =  Location( location_id )
                TheGame:GetGameState():AddLocation(location)
                self:SetHome(location)
            end
        end
    end
end

function Brain:IsAtHome()
    return self.agent:GetLocation() and self.agent:GetLocation() == self.home_location
end

function Brain:SetHome( home )
    if home ~= self.home_location then
        if self.home_location then
            self.home_location:RemoveRoomOccupant( self.agent )
        end

        self.home_location = home
        if home then
            home:AddRoomOccupant( self.agent )
        end
    end
end

function Brain:MoveToHome()
    self:GenerateHome() -- generate a home if we need one
    if self.home_location then
        self.agent:MoveToLocation(self.home_location)
        self.agent:SetLocationRole(CHARACTER_ROLES.RESTING)
    else
        self.agent:MoveToLimbo()
    end
end

function Brain:GetHome()
    self:GenerateHome() -- generate a home if we need one
    return self.home_location
end
