----------------------------------------------
-- Save/loadable collection of cards

local CardSet = class( "CardSet" )

function CardSet:init( card_class, owner )
    self.card_class = card_class
    self.owner = owner
    self.cards = {}
end

function CardSet:AddCard( card, idx )
    assert( card.id, card )
    idx = idx or (#self.cards + 1)
    table.insert( self.cards, idx, card )
end

function CardSet:RemoveCard( card )
    local idx = table.find( self.cards, card )
    if idx then
        table.remove( self.cards, idx )
    end
    return idx
end

function CardSet:ClearCards()
    table.clear(self.cards)
end

function CardSet:ClearInvalidCards()
    for i = #self.cards, 1, -1 do
        if self.cards[i]:GetContent() == nil then
            print( "### REMOVED INVALID CARD:", self.cards[i] )
            table.remove( self.cards, i )
        end
    end
end

function CardSet:RemoveCardByID( card_id, userdata )
    for i, card in ipairs( self.cards ) do
        if card.id == card_id and (card.userdata == userdata or userdata == nil) then
            table.remove( self.cards, i )
            return i, card
        end
    end
end

function CardSet:GetCards()
    return self.cards
end

function CardSet:Save()
    return { owner = self.owner, class_name = self.card_class._classname, cards = self.cards }
end

function CardSet:Load()
    self.card_class = CLASSES[ self.class_name ]
    self.class_name = nil
end
