---------------------------------------------------------------------
-- Used for calculating a scalar property

local ScalarAccumulator = class( "CardEngine.ScalarAccumulator" )
ScalarAccumulator.can_reload = true

function ScalarAccumulator:init( engine, event_name )
    self.engine = engine
    self.event_name = event_name
    self.details = {}
end

function ScalarAccumulator:IsPreview()
    return self.preview == true
end

function ScalarAccumulator:CalculateValue( initial_value, ... )
    -- Accumulate modifiers.
    self.value = initial_value
    self.engine:BroadcastEvent( self.event_name, self, ... )

    return self.value
end

function ScalarAccumulator:PreviewValue( initial_value, ... )
    table.clear( self.details )
    self.preview = true

    self:CalculateValue( initial_value, ... )

    self.preview = false

    local str
    if #self.details > 0 then
        str = table.concat( self.details, "\n" )
    else
        str = nil
    end

    return self.value, str
end

function ScalarAccumulator:ModifyValue( value, source )
    self.value = value

    local detail
    if is_instance( source, Battle.Card ) then
        detail = source:GetName()
    elseif is_instance( source, Negotiation.Card ) then
        detail = source:GetName()
    elseif is_instance( source, CombatCondition ) then
        detail = source:GetName()
    elseif is_instance( source, Negotiation.Modifier ) then
        detail = source:GetName()
    else
        detail = tostring(source)
    end

    if detail then
        table.insert( self.details, detail )
    end
end

function ScalarAccumulator:AddValue( delta, source )
    self:ModifyValue( self.value + delta, source )
end

function ScalarAccumulator:ClearValue( source )
    self:ModifyValue( -self.value, source )
end
