local function QueryVoiceActors( skin )
    local props = SPECIES_PROPS[ skin.species ]
    local voice_actors = props.voice_actors and props.voice_actors[ skin.gender ]
    return voice_actors
end

local function ApplyVoiceActor( agent )
    if agent.voice_actor == nil then

        --does the def give us anything?
        if agent.voice_actors then
            agent.voice_actor = agent.voice_actors and agent.voice_actors[ agent.gender ] and table.arraypick( agent.voice_actors[ agent.gender ] )
        end

        if agent.voice_actor == nil then
            local voice_actors = QueryVoiceActors( agent )
            if voice_actors then
                agent.voice_actor = table.arraypick( voice_actors )
            end
        end

    end
end

local function ApplyColours( agent )
    local head_content = Content.GetCharacterHead( agent.head )
    local props = SPECIES_PROPS[ agent.species ]

    local hair_colours = agent.HAIR_PALETTE or props.HAIR_PALETTE
    local eye_colours = agent.EYES_PALETTE or props.EYES_PALETTE

    local skin_colours
    local palette_map = props.SKIN_TONES
    if palette_map then
        local skin_tone = head_content and head_content.skin_tone
        if skin_tone == nil then
            skin_tone = table.arraypick( copykeys( palette_map ))
        end
        skin_colours = palette_map[ skin_tone ]

    else
        skin_colours = agent.SKIN_PALETTE or props.SKIN_PALETTE
    end

    if agent.species == SPECIES.SNAIL then
        local shell_colours = props.SHELL_PALETTE
        agent.shell_colour = shell_colours[math.random(#shell_colours)]
        agent.snail_colour = skin_colours[math.random(#skin_colours)]

    else
        if hair_colours and agent.hair_colour == nil then
            agent.hair_colour = table.arraypick( hair_colours )
        end
        if skin_colours and agent.skin_colour == nil then
            agent.skin_colour = table.arraypick( skin_colours )
        end
        if eye_colours and agent.eye_colour == nil then
            agent.eye_colour = table.arraypick( eye_colours )
        end
    end
end

local function QueryHeads( def, skin, candidates, fn )
    table.clear( candidates )

    local all_heads = Content.GetCharacterHeads()
    local priority = 0
    for head_id,head in pairs( all_heads ) do
        if (head.gender == nil or head.gender == skin.gender) and
           (head.species == nil or head.species == skin.species) and
           (head.faction_id == nil or head.faction_id == def.faction_id) and
           ((head.priority or 0) >= priority) and
           (head.match == nil or head.match( def, skin )) then

           if fn == nil then
               table.insert( candidates, head_id )
            else
                fn( head_id, head, candidates )
            end
        end
    end
end

local function QueryGenericHeads( def, skin, candidates )

    local priority = 0
    local function IsGenericHead( head_id, head, candidates )
        if head.generic and (head.priority or 0) >= priority then
            if (head.priority or 0) > priority then
                -- Only take the HIGHEST priority of the generic heads available.
                table.clear( candidates )
                priority = head.priority
            end
            table.insert( candidates, head_id )
        end
    end

    QueryHeads( def, skin, candidates, IsGenericHead )
end


local function IsUnusedHead( head_id, head, candidates )
    if not head.generic then
        for id, def in pairs( Content.GetAllCharacterDefs() ) do
            if def.head == head_id then
                return false
            end

            local skins = Content.GetAllCharacterSkins( id )
            if skins then
                for i, skin in ipairs( skins ) do
                    if skin.head == head_id then
                        return false
                    end
                end
            end
        end
    end

    if candidates then
        table.insert( candidates, head_id )
    end
    return true
end

local function QueryUnusedHeads( def, skin, candidates )
    -- Prune any heads already referred to by any existing CharacterDef,
    -- UNLESS they are generic.
    QueryHeads( def, skin, candidates, IsUnusedHead )
end

local function ApplyHead(agent)
    local head

    if agent.def.nohead then
        return

    elseif agent.head ~= nil then
        return

    else
        local candidates = ObtainWorkTable()
        local def = Content.GetCharacterDef( agent:GetContentID() )
        QueryGenericHeads( def, agent, candidates )

        local wt = WeightedList()
        for i, head_id in ipairs( candidates ) do
            local weight = Content.GetCharacterHead( head_id ).weight or 1
            wt:AddChoice( head_id, weight )
        end
        head = wt:GetChoice()

        ReleaseWorkTable( candidates )
    end

    if head == nil then
        print( "WARN: no head candidates for ", agent, agent.gender, agent.species, agent.def.id )
    end
    agent.head = head
end

local function ApplyBuild(agent)
    ApplyHead( agent )
    ApplyColours( agent )

    if agent.voice_actor == nil then
        ApplyVoiceActor( agent )
    end
end

return
{
    QueryVoiceActors = QueryVoiceActors,
    QueryHeads = QueryHeads,
    QueryUnusedHeads = QueryUnusedHeads,
    IsUnusedHead = IsUnusedHead,

    ApplyBuild = ApplyBuild,
    ApplyColours = ApplyColours,
    ApplyVoiceActor = ApplyVoiceActor,
}

