local Collection = class( "Collection" )
function Collection:init()
    self.items = {}
end

function Collection:Filter(fn, ...)
    if fn then
        local t = {}
        for k,v in ipairs(self.items) do
            if fn(v, ...) then
                table.insert(t, v)
            end
        end
        self.items = t
    end
    return self
end


function Collection:AddItems(items, fn, ...)
    for k,v in pairs(items) do
        if fn == nil or fn(v, ...) then
            table.insert(self.items, v)
        end
    end
    return self
end

function Collection:Pick(max_count)
    local defs = shallowcopy(self.items)
    table.shuffle(defs)
    local ret = {}
    for k = 1, max_count do
        if defs[k] then
            table.insert(ret, defs[k])
        else
            break
        end
    end
    return ret
end

function Collection:SeededPick(max_count, rng)
    local defs = shallowcopy(self.items)
    table.seeded_shuffle(defs, rng)
    local ret = {}
    for k = 1, max_count do
        if defs[k] then
            table.insert(ret, defs[k])
        else
            break
        end
    end
    return ret
end
