class ("Component")

function Component:init()
end
Component.can_reload = true

function Component:StartWallUpdating()
    if not self.wallupdating then
        self.wallupdating = true
        self.entity.sim:StartUpdatingComponent(self, "wall")
        assert( self.OnUpdate )
    end
end

function Component:StopUpdating()
    self:StopWallUpdating()
    self:StopTickUpdating()
    self:StopLateUpdating()
end

function Component:StopWallUpdating()
    if self.wallupdating and self.entity then 
        self.wallupdating = false
        self.entity.sim:StopUpdatingComponent(self, "wall")
        assert( self.OnUpdate )
    end
end

function Component:StartTickUpdating()
    if not self.tickupdating then
        self.tickupdating = true
        self.entity.sim:StartUpdatingComponent(self, "tick")
        assert( self.OnTickUpdate )
    end
end

function Component:StopTickUpdating()
    if self.tickupdating and self.entity then 
        self.tickupdating = false
        self.entity.sim:StopUpdatingComponent(self, "tick")
    end
end


function Component:StartLateUpdating()
    if not self.lateupdating then
        self.lateupdating = true
        self.entity.sim:StartUpdatingComponent(self, "late")
        assert( self.OnLateUpdate )
    end
end

function Component:StopLateUpdating()
    if self.lateupdating and self.entity then 
        self.lateupdating = false
        self.entity.sim:StopUpdatingComponent(self, "late")
    end
end


function Component:DoAddedToEntity(ent)

    self.entity = ent
    self.sim = ent.sim

    if self.OnAddedToEntity then
        self:OnAddedToEntity(ent.spawn_data and ent.spawn_data.cmp and ent.spawn_data.cmp[self._classname])
    end

    self.sim:AddEntCmp(ent, self)
end

-- the entity isn't being removed, but this component is being removed from it
function Component:DoRemovedFromEntity()
    self:StopUpdating()
    if self.OnRemovedFromEntity then
        self:OnRemovedFromEntity()
    end
    self.sim:RemoveEntCmp(self.entity, self)
    self.entity, self.sim = nil, nil
    self:StopCoroutines()
end

function Component:GetSpawnDataVar(var)
    return self.entity.spawn_data.cmp and self.entity.spawn_data.cmp[self._classname] and self.entity.spawn_data.cmp[self._classname][var]
end

function Component:SetSpawnDataVar(var, val)
    self.entity.spawn_data.cmp = self.entity.spawn_data.cmp or {}
    self.entity.spawn_data.cmp[self._classname] = self.entity.spawn_data.cmp[self._classname] or {} 
    self.entity.spawn_data.cmp[self._classname][var] = val
    return self.entity.spawn_data.cmp[self._classname]
end


function Component:StartCoroutine(fn, ...)
    local coro = Coro(fn, ...)
    self.coros = self.coros or {}
    self.coros[coro] = coro
    return coro
end

function Component:StopCoroutine( coro )
    coro:Stop()
    self.coros[ coro ] = nil
end

function Component:StopCoroutines()
    if self.coros then
        for k,v in pairs(self.coros) do
            self:StopCoroutine(v)
        end
    end 
end

function Component:UpdateCoroutines( dt )
    if self.coros then
        for k,v in pairs(self.coros) do
            if not v:Update( dt ) then
                self.coros[k] = nil
            end
        end
    end
end

function Component:CoroDelay(t)
    t = t or 0
    while (t > 0) do 
        local dt = coroutine.yield()
        t = t - dt
    end
end


