require "component"
require "colours"

local ColourTweener = class("ColourTweener", Component)

function ColourTweener:init(owner)
	ColourTweener._base.init(self)
end

function ColourTweener:ValidateModel()

	self.i_colour = nil
	self.tweening_colour = nil
	self.t_colour = nil
	self.ease = nil
	self.callback = nil
	self.endTime = nil
	self.startTime = nil
	
end

function ColourTweener:StartTween(startColour, endColour, ease, time, callback)
	startColour = startColour or self.entity.cmp.AnimController:GetTintMult()
	
	self.i_colour = startColour
	self.tweening_colour = {}
	self.t_colour = endColour or Colours.GhostWhite
	self.ease = ease or easing.linear
	self.endTime = self.entity.sim:GetTime() + (time or 1)
	self.startTime = self.entity.sim:GetTime()
	self.callback = callback

	if #self.i_colour ~= #self.t_colour then
		print("INVALID COLOUR TWEEN")
		self:EndTween()
		return
	end

	self:StartTickUpdating()
end

function ColourTweener:EndTween()
	self.entity.cmp.AnimController:SetTintMult(table.unpack(self.t_colour))

	self:StopTickUpdating()

	if self.callback then
		self.callback(self.entity)
	end
end

function ColourTweener:OnTickUpdate(dt)
	local total_t = self.endTime - self.startTime
	local elapsed_t = total_t - (self.endTime - self.entity.sim:GetTime())
	local t = elapsed_t/total_t

	if t >= 1 then
		self:EndTween()
		return
	end

	for i = 1, #self.t_colour do
		self.tweening_colour[i] = self.ease(elapsed_t, self.i_colour[i], self.t_colour[i] - self.i_colour[i], total_t)
	end

	self.entity.cmp.AnimController:SetTintMult(table.unpack(self.tweening_colour))
end
