require "component"

local assets =
{
    fightarrow = engine.asset.Texture("rgba/fightarrow.tex")
}

local FightArrowRenderer = class("FightArrowRenderer", Component)

function FightArrowRenderer:init()
    FightArrowRenderer._base.init(self)  

    self.model = engine.model.FightArrowModel()
    self.model:SetTextures(assets.fightarrow, engine.asset.Texture("rgba/character_hover.tex"))
    self.model:SetStretchPoints(34.0/200.0, 166.0/200.0);
    self.model:SetWidthAndLength(130,400)
    self.model:SetLayerSpeeds(0.06, 0.14, 0.6)
    self.model:SetLayerScale(0.5)
    self.model:SetLayerAlphas(0.5, 0.3, 0.7)

    self.node = engine.rendering.SceneGraphNode(self.model) 
    --self.model:SetTextureSampler(0, "linear","linear","linear","wrap", "wrap", true)

    self:SetColour(0xFF0000FF)
    self:SetBloom(0.3)    
end

function FightArrowRenderer:OnAddedToEntity(data)
    self.entity.cmp.Transform:AddChildNode(self.node)
end

function FightArrowRenderer:SetBloom(b)
    self.model:SetBloom(b)
    return self
end

function FightArrowRenderer:IsOnScreen( screenx, screeny )
    return self.model:IsOnScreen(self.sim:GetCameraInfo())
end

function FightArrowRenderer:RayTest( screenx, screeny )
    if self.model:IsOnScreen(self.sim:GetCameraInfo()) then
        return self.model:RayTest( self.sim:GetCameraInfo(), screenx, screeny )
    end
end


function FightArrowRenderer:SetStretchPoints(head, tail)
    self.model:SetStretchPoints(head, tail)
    return self
end

function FightArrowRenderer:SetColour(colour)
    local r,g,b,a = HexColour(colour)
    self.model:SetColour(r, g, b, a)
    return self
end

function FightArrowRenderer:SetWidthAndLength(startwidth, l, endwidth)
    self.model:SetWidthAndLength(startwidth, l, endwidth)
    return self
end


function FightArrowRenderer:SetTextures(texture_asset, fill_texture_asset)
    
    self.model:SetTextures(texture_asset, fill_texture_asset)
    return self
end


function FightArrowRenderer:SetLayerSpeeds(layer1, layer2, layer3)
    self.model:SetLayerSpeeds(layer1, layer2, layer3);
    return self
end

function FightArrowRenderer:SetLayerScale(scale)
    self.model:SetLayerScale(scale);
    return self
end

function FightArrowRenderer:SetLayerAlphas(layer1, layer2, layer3)
    self.model:SetLayerAlphas(layer1, layer2, layer3);
    return self
end

function FightArrowRenderer:SetMiddleUVSkew(skew)
    self.model:SetMiddleUVSkew(skew);
    return self
end
