require "component"

-- A container for FX attached to an entity.
---------------------------------------------

local FXContainer = class("FXContainer", Component)

function FXContainer:init()
    self.playing_fx = {}
end

function FXContainer:OnDespawn()
    for i, fx in ipairs( self.playing_fx ) do
        self.sim.fx_mgr:KillFX( fx )
    end
end

function FXContainer:KillFX( fx )
    self.sim.fx_mgr:KillFX( fx )
end

function FXContainer:InsertFX( fx )
    self.entity.cmp.Transform:AddChildNode( fx.node )
    fx.on_dispose = function( fx )
        table.arrayremove( self.playing_fx, fx )
    end

    table.insert(self.playing_fx, fx)
end

function FXContainer:PlayFX( anim, state, dx, dy, dz, sx, sy, sz, rx, ry, rz, flip, is_looping )
    local fx = self.sim.fx_mgr:PlayFXInternal( anim, state, dx, dy, dz, sx, sy, sz, rx, ry, rz, flip, is_looping )
    if fx == nil then
        LOGWARN( "Missing FX: %s.%s", anim, state )
        return
    end

    self:InsertFX( fx )
    return fx
end

function FXContainer:PlayFXOnGround(anim, state, dx, dz, sx, sy, r, flip, loop, zz)
    local fx = self.sim.fx_mgr:PlayFXOnGround( anim, state, dx, dz, sx, sy, r, flip, loop)
    if fx == nil then
        LOGWARN( "Missing FX: %s.%s", anim, state )
        return
    end

    self:InsertFX( fx )

    -- FIXME: We don't have height-varying entities, but if we do these ground FX are not going to
    -- to stay on the ground.
    local _, height = self.entity:GetLocalPosition()
    fx.node:SetTranslation( nil, -height + 0.01, nil )

    return fx
end

function FXContainer:PlayBillboardFX( anim, state, dx, dy, dz, sx, sy, sz, r, flip, loop )
    local fx = self.sim.fx_mgr:PlayBillboardFX( anim, state, dx, dy, dz, sx, sy, sz, r, flip, loop )
    if fx == nil then
        LOGWARN( "Missing FX: %s.%s", anim, state )
        return
    end

    self:InsertFX( fx )
    return fx
end
