require "component"

local PlaneRenderer = class("PlaneRenderer", Component)


function PlaneRenderer:init()
    PlaneRenderer._base.init(self)  

    self.model = engine.model.PlaneModel()
    self.node = engine.rendering.SceneGraphNode(self.model) 
    --self.model:SetTextureSampler(0, "linear","linear","linear","wrap", "wrap", true)
end

function PlaneRenderer:OnAddedToEntity(data)
    self.entity.cmp.Transform:AddChildNode(self.node)
    if data and data.yoffset then
        self:SetYOffset(data.yoffset)
    end
end

function PlaneRenderer:SetSize(x, z)
    self.model:SetSize(x, z)
end

function PlaneRenderer:SetBloom(b)
    self.model:SetBloom(b)
end


function PlaneRenderer:SetBillboard(bb)
    -- PlaneModels have its vertices such that its normal points to +y: so rotate the billboard 90' on the x-axis.
    self.node:BillboardToCamera( true, -math.pi/2, 0, 0 )
end

function PlaneRenderer:IsOnScreen( screenx, screeny )
    return self.model:IsOnScreen(self.sim:GetCameraInfo())
end

function PlaneRenderer:RayTest( screenx, screeny )
    if self.model:IsOnScreen(self.sim:GetCameraInfo()) then
        return self.model:RayTest( self.sim:GetCameraInfo(), screenx, screeny )
    end
end

function PlaneRenderer:SetTexture(texture, sampler)
    self.tex = self.tex or {}
    local idx = sampler or 0
    self.tex[idx] = texture
    self.model:SetTexture(idx, texture)
end

function PlaneRenderer:SetWrapMode(modeu, modev)
    modev = modev or modeu
    self.model:SetTextureSampler(0, "linear", "linear", "linear", modeu, modev, false)
end

function PlaneRenderer:SetUVScale(u, v)
    self.model:SetUVScale(u, v)
end

function PlaneRenderer:SetUVOffset(u, v)
    self.model:SetUVOffset(u, v)
end

function PlaneRenderer:TintColour( r, g, b, a )
    if not self.tint_mult_constant then
        self.tint_mult_constant = engine.rendering.ShaderConstantValue("TINT_MULT", r, g, b, a)
        self.model:SetShaderConstant(self.tint_mult_constant)
    else
        self.tint_mult_constant:SetValue(r or 1, g or 1, b or 1, a or 1)
    end
end

function PlaneRenderer:ClearTint()
    self.model:ClearShaderConstant( "TINT_MULT" )
    self.tint_mult_constant = nil
end

function PlaneRenderer:HiliteModel( hilite )
    if hilite then
        self:TintColour( 1.5, 1.5, 1.5, 1 )
    else
        self:ClearTint()
    end
end

function PlaneRenderer:SetYOffset( y )    
    self.yoffset = y
    self.entity.cmp.PlaneRenderer.model:SetOffset(0, y, 0)
end