require "component"


local Projectile = class("Projectile", Component)


function Projectile:init( rotation )
    Projectile._base.init(self)
end

function Projectile:OnAddedToEntity()  
end

--function Projectile:Fire(startx, starty, startz, destx, desty, destz, speed, arc_height, anim_file, anim_state, hit_fx, hit_fx_state)  
function Projectile:Fire(parent, target, data)  
    local px,py,pz = parent.cmp.Transform:GetLocalPosition()
    local tx,ty,tz = target.cmp.Transform:GetLocalPosition()


    --lcoal startx, starty, startz, destx, desty, destz, speed, arc_height, anim_file, anim_state, hit_fx, hit_fx_state

    self.hit_fx_file = data.hit_fx_file
    self.hit_fx = data.hit_fx_state
    self.destx, self.desty, self.destz = tx + (data.target_x or 0)*(target.cmp.AnimController.flip and -1 or 1), ty + (data.target_y or 0), tz + (data.target_z or 0)
    self.startx, self.starty, self.startz = px + (data.spawn_x or 0)*(parent.cmp.AnimController.flip and -1 or 1), py + (data.spawn_y or 0), pz + (data.spawn_z or 0)

    self.entity.cmp.AnimController:SetBuild(GetAnim(data.anim))
    self.entity.cmp.AnimController:AddAnims(GetAnim(data.anim))
    self.entity.cmp.AnimController:Play(data.state, true)

    self.entity.cmp.Transform:SetPosRot(self.startx, self.starty, self.startz)
    self.entity.cmp.AnimController:SetXFlip(parent.cmp.AnimController.flip)
    
    local dx, dy, dz = self.destx-self.startx, self.desty-self.starty, self.destz-self.startz
    local distsq =  dx*dx+dy*dy*dz*dz

    if distsq > 0 then
        
        self.linear_velocity = (data.speed or 30)

        local angle = math.atan(dz, dx)
        local adj_amt = .67
        local pct_throttle = (1-adj_amt) + adj_amt*math.abs(math.sin(angle))
        self.linear_velocity = self.linear_velocity *pct_throttle        

        self.total_dist = math.sqrt(distsq)
        self.total_time = self.total_dist/self.linear_velocity
        self.velx, self.vely, self.velz = dx/self.total_time, dy/self.total_time, dz/self.total_time
        
        self.t = 0
        self.interp_t = 0
        self.arc_height = data.arc_height or 0
        self:OnTickUpdate()
        self:StartTickUpdating()
        self:StartWallUpdating()

        return self.total_time
    else
        self:Done()
        return 0
    end
end

--this is cheesy, but makes for nice 60fps motion
function Projectile:OnUpdate(dt)
    self.interp_t = self.interp_t + dt
    if self.next_x and self.last_x then
        local t = clamp( self.interp_t / self.total_interp_t,0, 1)
        local x = self.last_x + (self.next_x - self.last_x)*t
        local y = self.last_y + (self.next_y - self.last_y)*t
        local z = self.last_z + (self.next_z - self.last_z)*t
        self.entity.cmp.Transform:SetPosRot(x,y,z)
    end
end

function Projectile:OnTickUpdate()
    local dt = SIM_TICK_TIME

    local last_arc_height = self.arc_height * math.sin( PI*self.t/self.total_time ) 
    self.last_x = self.startx + self.t*self.velx
    self.last_y = self.starty+ self.t*self.vely + last_arc_height
    self.last_z = self.startz + self.t*self.velz

    self.t = self.t + dt
    
    local arc_height = self.arc_height * math.sin( PI*self.t/self.total_time ) 
    local x = self.startx + self.t*self.velx
    local y = self.starty+ self.t*self.vely + arc_height
    local z = self.startz + self.t*self.velz
    self.next_x = x
    self.next_y = y
    self.next_z = z


    self.interp_t = 0
    self.total_interp_t = SIM_TICK_TIME

    
    if self.t >= self.total_time and not self.entity.removed then
        self:Done()
        self:StopTickUpdating()
    end
    
end

function Projectile:Done()
    self.entity:Despawn()
    if self.hit_fx then
        self.entity.sim.fx_mgr:PlayBillboardFX(self.hit_fx, self.hit_fx_state or "idle", self.destx, self.desty, self.destz)
    end
end

