require "component"

local SoundEmitter = class("SoundEmitter", Component)

function SoundEmitter:init()
    SoundEmitter._base.init(self)  
    self.sound_offset_x, self.sound_offset_y, self.sound_offset_z = 0, 0, 0
end


function SoundEmitter:PlayEvent(name, ...)
    
    local inst, desc = AUDIO:CreateEventInstance(name)
    local params
    if inst then
        local x, y, z = self.entity:GetWorldPosition()
        inst:SetPos(x,y,z)
        for n=1,select('#',...), 2 do --do this without creating garbage
            params = params or {}
            local name = select(n,...)
            local val = select(n+1,...)
            if name and val then
                inst:SetParameterValue(name, val)
                params[name] = val
            end

        end
        inst:Start()
        inst:SetUserData(self.entity.guid)
        
        if desc:IsOneShot() then
            inst:Release()
        else
            self.active_events = self.active_events or {}
            table.insert( self.active_events, inst)
            self:StartTickUpdating()
            return inst
        end
        if params then
            params.x = x
            params.y = y
            params.z = z
        end
        self.x, self.y, self.z = x, y, z
    end

    AUDIO:LogSound(name, self.entity.guid, params)
end

function SoundEmitter:OnTickUpdate(dt)
    local x, y, z = self:GetSoundEmitPoint()



    for k,v in ipairs(self.active_events) do
        if v:GetPlaybackState() == engine.sound.STOPPED then
            self.active_events[k]:Release()
            self.active_events[k], self.active_events[#self.active_events] = self.active_events[#self.active_events], nil
            print ("retired stopped sound event")
        else
            
            if self.x ~= x or self.y ~= y or self.z ~= z then
                v:SetPos(x,y,z)
            end
        end
    end

    if #self.active_events == 0 then
        self:StopTickUpdating()
    end
end

function SoundEmitter:OnDespawn()
    if self.active_events then
        for k,v in pairs(self.active_events) do
            v:Stop()
            v:Release()
        end
    end
end

function SoundEmitter:PopulateEditorPanel( ui, cxt )
    cxt:ShowSliderFloat( "sound_offset_x", -8, 8 )
    cxt:ShowSliderFloat( "sound_offset_y", -8, 8 )
    cxt:ShowSliderFloat( "sound_offset_z", -8, 8 )
end



function SoundEmitter:Save(  )
    return
    {
        sound_offset_z = self.sound_offset_z,
        sound_offset_x = self.sound_offset_x,
        sound_offset_y = self.sound_offset_y,
    }
end

function SoundEmitter:OnEntityReady( data )
    if data then
        self.sound_offset_z = data.sound_offset_z or self.sound_offset_z
        self.sound_offset_x = data.sound_offset_x or self.sound_offset_x
        self.sound_offset_y = data.sound_offset_y or self.sound_offset_y
    end
end


function SoundEmitter:SetSpawnDataVar( key, value )
    Component.SetSpawnDataVar( self, key, value )
    if key == "sound_offset_x" then
        self:SetOffset( value, self:GetSpawnDataVar( "sound_offset_y" ) or 0, self:GetSpawnDataVar( "sound_offset_z" ) or 0 )
    elseif key == "sound_offset_y" then
        self:SetOffset( self:GetSpawnDataVar( "sound_offset_x" ) or 0, value, self:GetSpawnDataVar( "sound_offset_z" ) or 0 )
    elseif key == "sound_offset_z" then
        self:SetOffset( self:GetSpawnDataVar( "sound_offset_x" ) or 0,  self:GetSpawnDataVar( "sound_offset_y" ) or 0, value )
    end
end

function SoundEmitter:GetSoundEmitPoint()
    local x0, y0, z0 = self.entity:GetWorldTransform()
    x0 = x0 + self.sound_offset_x
    z0 = z0 + self.sound_offset_z
    y0 = y0 + self.sound_offset_y
    return x0, y0, z0
end

function SoundEmitter:SetOffset(x, y, z)
    self.sound_offset_x, self.sound_offset_y, self.sound_offset_z = x, y, z

    x, y, z = self:GetSoundEmitPoint()
    for k,v in ipairs(self.active_events) do
        v:SetPos(x,y,z)
    end
end
