require "component"

local Transform = class("Transform", Component)

Transform.Priority = 1

function Transform:init()
    Transform._base.init(self)   

    self.scenegraphnode = engine.rendering.SceneGraphNode()    
    self.facing = true
end

function Transform:AddPosWatcher(fn, caller)
    --print(self.entity, "Transform:AddPosWatcher", fn)
    self.poswatchers = self.poswatchers or {}
    local w = { fn = fn, caller = caller }
    table.insert( self.poswatchers, w )
    return w
end

function Transform:RemovePosWatcher(caller)
    if self.poswatchers then
        for i,v in ipairs( self.poswatchers ) do
            if v.caller == caller then
                table.remove( self.poswatchers, i )
                break
            end
        end
        if not next(self.poswatchers) then
            self.poswatchers = nil
        end
    end
end

function Transform:UpdateWatchers(x,y,z)
    if self.poswatchers then
        --print(self.entity, "Transform:UpdateWatchers()")
        for i,v in ipairs(self.poswatchers) do
            v.fn(v.caller, self.entity, x, y, z)
        end
    end
end

function Transform:Hide()
    if self.scenegraphnode then
        self.scenegraphnode:Hide()
    end
end

function Transform:Show()
    if self.scenegraphnode then
        self.scenegraphnode:Show()
    end
end

function Transform:OnAddedToEntity()
    --self.scenegraphnode:SetName( tostring(self.entity) )
    self.scenegraphnode:SetID(self.entity:GetGUID())

    --print(self.entity, "Transform:OnAddedToEntity", self.def and self.def.scale)

    if self.def and self.def.scale then
        self:SetLocalScale(self.def.scale)
    end
end

function Transform:OnDespawn()
    if self.scenegraphnode then
        self.scenegraphnode:SetModel(nil)
        self.scenegraphnode:RemoveAllChildren()
        self.scenegraphnode:Remove()
    end
    self.scenegraphnode = nil
    -- self.rendermodel = nil -- this "fixes" the models hanging out, but hides the component/entity hanging around
end

function Transform:GetLocalPosition()
    return self.scenegraphnode:GetTranslation()
end

function Transform:TransformPoint(x,y,z)
    return self.scenegraphnode:TransformPoint(x,y,z)
end

function Transform:TransformVector(x,y,z)
    return self.scenegraphnode:TransformVector(x,y,z)
end

function Transform:GetLocalScale()
    return self.scenegraphnode:GetScale()
end

function Transform:GetLocalRotation()
    return self.scenegraphnode:GetRotation()    
end

function Transform:SetLocalPosition(x, y, z)
    self.scenegraphnode:SetTranslation(x,y,z)    
    self:UpdateWatchers(x,y,z)
end

-- only use SetLocalTransform if you actually want to change the scale
-- see SetPosRot
function Transform:SetLocalTransform(x, y, z, sx, sy, sz, rx, ry, rz)
    --print (x,y,z,sx,sy,sz, r)
    self.scenegraphnode:SetTransform(x,y, z, sx, sy, sz, rx, ry, rz)
    self:UpdateWatchers(x,y,z)
end

function Transform:SetPosRot(x, y, z, r)
    --print (x,y,z, r)
    self.scenegraphnode:SetPosRot(x, y, z, r)    
    self:UpdateWatchers(x,y,z)
end

function Transform:SetLocalScale(x, y, z)
    --print(self.entity, "Transform:SetLocalScale", x, y, z)
    self.scenegraphnode:SetScale(x, y or x, z or x)
end

function Transform:GetFacing()
    return self.facing
end

function Transform:SetFacing(f)
    if self.facing ~= f then
        self.facing = f
        local x,y,z = self.scenegraphnode:GetScale()
        x = math.abs(x)
        self.scenegraphnode:SetScale(x*(f and 1 or -1), y, z)
    end
end

function Transform:SetHeading(angle)
    self.scenegraphnode:SetRotation(0, angle, 0) 
end

function Transform:GetForward()
    local x, y, z = self.scenegraphnode:TransformVector(1, 0, 0)
    return x, y, z
end

function Transform:SetLocalRotation(x, y, z)
    self.scenegraphnode:SetRotation(x, y, z) 
end

function Transform:Orphan()
    self.scenegraphnode:Orphan()
end

function Transform:GetNumChildren()
    return self.scenegraphnode:GetNumChildren()
end

function Transform:GetChildByIndex(idx)
    return self.scenegraphnode:GetChildByIndex(idx)
end

function Transform:AddChild(child)
    if child.cmp.Transform then
        self.scenegraphnode:AddChild(child.cmp.Transform.scenegraphnode)
    end
    return child
end

function Transform:AddChildNode(childnode)
    self.scenegraphnode:AddChild(childnode)
end

function Transform:RemoveAllChildren()
    self.scenegraphnode:RemoveAllChildren()
end

function Transform:RemoveChild(child)
    if child.cmp.Transform and child.cmp.Transform.scenegraphnode and self.scenegraphnode then
        self.scenegraphnode:RemoveChild(child.cmp.Transform.scenegraphnode)
    end
    return child
end

function Transform:RemoveChildNode(childnode)
    return self.scenegraphnode:RemoveChild(childnode)
end

function Transform:ScreenToLocal( camera_info, screen_x, screen_y)
    return self.scenegraphnode:ScreenToLocal( camera_info, screen_x, screen_y)
end

function Transform:ScreenToGround( camera_info, screen_x, screen_y)
    return self.scenegraphnode:ScreenToGround( camera_info, screen_x, screen_y)
end

function Transform:LocalToScreen( camera_info, x,y,z)
    return self.scenegraphnode:LocalToScreen( camera_info, x,y,z)
end

function Transform:LocalToWorld(x,y,z)
    return self.scenegraphnode:LocalToWorld(x,y,z)
end

function Transform:WorldToLocal(x,y,z)
    return self.scenegraphnode:WorldToLocal(x, y, z or 0)
end

function Transform:GetWorldPosition()
    return self.scenegraphnode:GetWorldPosition()
end

function Transform:SetModel(model)
    -- self.rendermodel = model
    if self.scenegraphnode then
        return self.scenegraphnode:SetModel(model)
    end
end

function Transform:GetModel()
    -- assert(self.scenegraphnode or not self.rendermodel)
    return self.scenegraphnode:GetModel()
end


function Transform:GetWorldTransform()
    return self.scenegraphnode:GetWorldTransform()
end


function Transform:InheritTransform(inherit)
    return self.scenegraphnode:InheritTransform(inherit)
end

function Transform:Copy(other_transform)
    local lx, ly, lz = other_transform:GetLocalPosition()
    local rx, ry, rz = other_transform:GetLocalRotation()
    local sx, sy, sz = other_transform:GetLocalScale()
    self:SetLocalTransform(lx, ly, lz, sx, sy, sz, rx, ry, rz)
end


function Transform:RenderDebugUI( ui, panel )
    if self.scenegraphnode then
        local x, y, z = self:GetWorldPosition()
        local lx, ly, lz = self:GetLocalPosition()
        local rx, ry, rz = self:GetLocalRotation()

        ui:Text( "Pos: ")
        ui:SameLine()
        ui:TextColored( 0xFF7700FF, string.format( "%2.2f, %2.2f, %2.2f", x,y,z ))
        ui:Text( "LocalPos: ")
        ui:SameLine()
        ui:TextColored( 0xcc5500FF, string.format( "%2.2f, %2.2f, %2.2f", lx,ly,lz ))
        ui:Text( string.format( "ScreenPos: %.0f, %.0f\n", self.entity:LocalToScreen( 0, 0, 0 )))
        ui:Text( string.format( "Rot: %2.2f, %2.2f, %2.2f\n", rx/DEGREES,ry/DEGREES,rz/DEGREES ))
    end
end