require "tuning_autogen"
require "uicolours"
require "advancement"
DEFAULT_SCREEN_MARGIN = 15
PI = 3.14159265359
DEGREES = PI/180
RES_X = 1920
RES_Y = 1080
DEFAULT_COMPONENT_PRIORITY = 100

TEST_RUN_TIME = 60*30

SIM_TICK_TIME = 1/60
MAX_SIM_TICKS = 2 -- Max allowed in a single frame.

-- anim stuff
FRAMES = 1/30


NAVLINK = MakeEnum{"QUEST", "NEGOTIATION_DECK", "BATTLE_DECK", "GRAFTS", "RELATIONSHIPS", "PAUSE"}

DBG_FLAGS = MakeBitField{ "RENDER", "UI", "NAVMESH", "PATHS", "PATH_CACHE", "VISION", "DETECT_ALL", "STATS", "COLLISION", "SIM" }

CHARACTER_ROLES = MakeEnum{"PROPRIETOR", "CLERK", "PATRON", "PRISONER", "GUARD", "QUARTERMASTER", "JOB_BOARD", "HEALER", "GARRISON", "COMMANDER", "VISITOR", "LABOUR", "ASSISTANT", "RESTING", "VENDOR", "CONTACT", "WAITING_PET", "MERCENARY" }

QUEST_FOCUS = MakeEnum{"COMBAT", "NEGOTIATION", "BALANCED"}

GAME_TYPE = MakeEnum{ "CAMPAIGN", "BRAWL", "DAILY" }

EXTRA_QUEST_REWARD = MakeEnum{"GRAFT_CHOICE", "CARD_REMOVE", "FREE_ITEM", "EXTRA_CASH", "RESTORATION", "CARD_UPGRADE", "METTLE_POINTS", "NONE"}

FEAT_STATUS = MakeEnum{"COMPLETE", "INCOMPLETE", "FAILED"}

PERK_TIER_UNLOCK_THRESH = {0, 6, 6, 5, 3}
PERK_SLOT_UNLOCK_THRESHOLDS = {1, 4, 10}
PERK_TIER_EQUIP_LIMITS = {3, 3, 2, 2, 1}

QUEST_METTLE_REWARD = {1, 2, 3, 4, 5}
BATTLE_METTLE_REWARD = {1, 1, 1, 2, 2}
NEGOTIATION_METTLE_REWARD = {1, 1, 1, 2, 2}


CONVERSATION_FLAGS = MakeBitField{ 
        "HOSTILE",         --We are looking at the hostile aside, or are otherwise allowed to do nasty things to someone 
        "MANIPULATE",       --used for the manipulate hub.
        "TIMELY",           --used to filter out actions that take time, and don't make sense to do when the person we are talking to is driving the conversation
        "FRIENDLY",         --used for friend request hub (gross)
        "BUSINESS",         --we are allowed to talk about business things (ie, quests)
        "ASIDE",            --we are looking at a voluntary, non-default hub. Mostly used to filter out quest actions from these hubs
        "ROOT",             --the standard "strike up a convo" hub
}            

MIN_RUN_TURN = 0

SLEEP_HEALTH_PERCENT = 0.1
SLEEP_RESOLVE_PERCENT = 0.4

-- Controls card-fanning curvature.  Set to nil to disable.
-- Bigger == less curvature.
CARD_FAN_HEIGHT = 2000

-- TEXT OUTLINE WIDTH
SMALL_TEXT_OUTLINE = 0.3

DEFAULT_CARD_XP = 10

CARD_RARITY = MakeEnum
{
    "BASIC",
    "COMMON",
    "UNCOMMON",
    "RARE",
    "UNIQUE", -- Not found in any drops. Uniquely acquired.
    "BOSS", -- Used for the boss grafts
    "COSMIC", -- Used for special cosmic grafts (previously brawl grafts)
}

CARD_SERIES = MakeEnum{ "NPC", "GENERAL", "DAILY" }

CARD_RARITY_ORDER =
{
    CARD_RARITY.BASIC,
    CARD_RARITY.COMMON,
    CARD_RARITY.UNCOMMON,
    CARD_RARITY.RARE,
    CARD_RARITY.UNIQUE,
    CARD_RARITY.BOSS,
}

CARD_RARITY_ORDINALS =
{
    [CARD_RARITY.BASIC] = 0,
    [CARD_RARITY.COMMON] = 1,
    [CARD_RARITY.UNCOMMON] = 2,
    [CARD_RARITY.RARE] = 3,
    [CARD_RARITY.UNIQUE] = 4,
    [CARD_RARITY.BOSS] = 5,
}

CARD_RARITY_ICON_STRINGS =
{
    [CARD_RARITY.BASIC] = "<p img='UI/rarity_basic.tex' scale=0.9>",
    [CARD_RARITY.COMMON] = "<p img='UI/rarity_common.tex' scale=0.9>",
    [CARD_RARITY.UNCOMMON] = "<p img='UI/rarity_uncommon.tex' scale=0.9>",
    [CARD_RARITY.RARE] = "<p img='UI/rarity_rare.tex' scale=0.9>",
    [CARD_RARITY.UNIQUE] = "<p img='UI/rarity_unique.tex' scale=0.9>",
    [CARD_RARITY.BOSS] = "<p img='UI/rarity_boss.tex' scale=0.9>",
    [CARD_RARITY.COSMIC] = "<p img='UI/rarity_cosmic.tex' scale=0.9>",
}

CARD_RARITY_SOUNDS =
{
    [CARD_RARITY.BASIC] = "event:/ui/select_cards/aquired_basic",
    [CARD_RARITY.COMMON] = "event:/ui/select_cards/aquired_common",
    [CARD_RARITY.UNCOMMON] = "event:/ui/select_cards/aquired_uncommon",
    [CARD_RARITY.RARE] = "event:/ui/select_cards/aquired_rare",
    [CARD_RARITY.UNIQUE] = "event:/ui/select_cards/aquired_unique",
}

function GetCardRarityString(rarity)
	local result = nil
	if rarity then
		local key = "CONSTANTS.CARD_RARITY_STRINGS." .. rarity
		result = LOC(key)
	end
	return result
end

function GetCardRarityIconString(rarity)
    if rarity and CARD_RARITY_ICON_STRINGS[rarity] then
        return CARD_RARITY_ICON_STRINGS[rarity]
    end
    return ""
end

CARD_RARITY_COLOURS =
{
    [CARD_RARITY.COSMIC] = UICOLOURS.CARD_COSMIC,
    [CARD_RARITY.BOSS] = UICOLOURS.CARD_UNIQUE,  
    [CARD_RARITY.UNIQUE] = UICOLOURS.CARD_UNIQUE,   
    [CARD_RARITY.BASIC] = UICOLOURS.CARD_BASIC,   
    [CARD_RARITY.COMMON] = UICOLOURS.CARD_COMMON,   
    [CARD_RARITY.UNCOMMON] = UICOLOURS.CARD_UNCOMMON, 
    [CARD_RARITY.RARE] = UICOLOURS.CARD_RARE,
}


CARD_TYPE = MakeEnum
{
    "NEGOTIATION_HOSTILE",
    "NEGOTIATION_DIPLOMACY",
    "NEGOTIATION_MANIPULATE",
    "NEGOTIATION_BYSTANDER",
    "NEGOTIATION_STATUS",
    "NEGOTIATION_OPPONENT",
    "FIGHT_MELEE",
    "FIGHT_SKILL",
    "FIGHT_STATUS",
    "FIGHT_SPECIAL",
    "ITEM",
    "PARASITE",
    "FLOURISH",
    "SCORE",
}

PARASITE_HEALTH_LOSS = 10

function GetCardType( card )

    if is_instance( card, Negotiation.Card ) then

        local negotiation_defs = require "negotiation/negotiation_defs"
        if CheckBits( card.flags, negotiation_defs.CARD_FLAGS.ITEM ) then
            return CARD_TYPE.ITEM
        elseif CheckBits( card.flags, negotiation_defs.CARD_FLAGS.PARASITE ) then
            return CARD_TYPE.PARASITE
        elseif CheckBits( card.flags, negotiation_defs.CARD_FLAGS.FLOURISH ) then
            return CARD_TYPE.FLOURISH
        elseif CheckBits( card.flags, negotiation_defs.CARD_FLAGS.HOSTILE ) then
            return CARD_TYPE.NEGOTIATION_HOSTILE
        elseif CheckBits( card.flags, negotiation_defs.CARD_FLAGS.DIPLOMACY ) then
            return CARD_TYPE.NEGOTIATION_DIPLOMACY
        elseif CheckBits( card.flags, negotiation_defs.CARD_FLAGS.MANIPULATE ) then
            return CARD_TYPE.NEGOTIATION_MANIPULATE
        elseif CheckBits( card.flags, negotiation_defs.CARD_FLAGS.STATUS ) then
            return CARD_TYPE.NEGOTIATION_STATUS
        end

        -- This shouldn't happen..
        return CARD_TYPE.NEGOTIATION_OPPONENT

    elseif is_instance( card, Battle.Card ) then
        local battle_defs = require "battle/battle_defs"
        if CheckBits( card.flags, battle_defs.CARD_FLAGS.SCORE ) then
            return CARD_TYPE.SCORE
        elseif CheckBits( card.flags, battle_defs.CARD_FLAGS.ITEM ) then
            return CARD_TYPE.ITEM
        elseif CheckBits( card.flags, battle_defs.CARD_FLAGS.FLOURISH ) then
            return CARD_TYPE.FLOURISH
        elseif CheckBits( card.flags, battle_defs.CARD_FLAGS.PARASITE ) then
            return CARD_TYPE.PARASITE
        elseif CheckAnyBits( card.flags, battle_defs.CARD_FLAGS.MELEE | battle_defs.CARD_FLAGS.RANGED ) then
            return CARD_TYPE.FIGHT_MELEE
        elseif CheckBits( card.flags, battle_defs.CARD_FLAGS.SKILL ) then
            return CARD_TYPE.FIGHT_SKILL
        elseif CheckBits( card.flags, battle_defs.CARD_FLAGS.STATUS ) then
            return CARD_TYPE.FIGHT_STATUS
        elseif CheckBits( card.flags, battle_defs.CARD_FLAGS.SPECIAL ) then
            return CARD_TYPE.FIGHT_SPECIAL
        end

        -- This shouldn't happen..
        return CARD_TYPE.FIGHT_SPECIAL

    end

    print( "Invalid card type", debugstack() )

    return nil
end

CARD_TYPE_SOUNDS =
{
    [CARD_TYPE.NEGOTIATION_HOSTILE] = "event:/ui/select_cards/aquired_hostility",
    [CARD_TYPE.NEGOTIATION_DIPLOMACY] = "event:/ui/select_cards/aquired_diplomacy",
    [CARD_TYPE.NEGOTIATION_MANIPULATE] = "event:/ui/select_cards/aquired_manipulate",
    [CARD_TYPE.NEGOTIATION_STATUS] = "event:/ui/select_cards/aquired_status",
    [CARD_TYPE.NEGOTIATION_OPPONENT] = "event:/ui/select_cards/aquired_opponent",
    [CARD_TYPE.FIGHT_MELEE] = "event:/ui/select_cards/aquired_attack",
    [CARD_TYPE.FIGHT_SKILL] = "event:/ui/select_cards/aquired_maneuver",
    [CARD_TYPE.FIGHT_STATUS] = "event:/ui/select_cards/aquired_status",
    [CARD_TYPE.FIGHT_SPECIAL] = "event:/ui/select_cards/aquired_special",
    [CARD_TYPE.ITEM] = "event:/ui/select_cards/aquired_item",
    [CARD_TYPE.PARASITE] = "event:/ui/select_cards/aquired_parasite",
    [CARD_TYPE.FLOURISH] = "event:/ui/select_cards/aquired_flourish",
}

function GetCardTypeIcon( card_type )
    if card_type == CARD_TYPE.FLOURISH then return engine.asset.Texture("UI/cardview_typeicon_flourish.tex") end
    if card_type == CARD_TYPE.PARASITE then return engine.asset.Texture("UI/cardview_typeicon_parasite.tex") end
    if card_type == CARD_TYPE.SCORE then return engine.asset.Texture("UI/cardview_typeicon_score.tex") end
    if card_type == CARD_TYPE.NEGOTIATION_HOSTILE then return engine.asset.Texture("UI/cardview_typeicon_negotiation_hostility.tex") end
    if card_type == CARD_TYPE.NEGOTIATION_DIPLOMACY then return engine.asset.Texture("UI/cardview_typeicon_negotiation_diplomacy.tex") end
    if card_type == CARD_TYPE.NEGOTIATION_MANIPULATE then return engine.asset.Texture("UI/cardview_typeicon_negotiation_manipulation.tex") end
    if card_type == CARD_TYPE.NEGOTIATION_STATUS then return engine.asset.Texture("UI/cardview_typeicon_negotiation_status.tex") end
    if card_type == CARD_TYPE.NEGOTIATION_OPPONENT then return engine.asset.Texture("UI/cardview_typeicon_opponent.tex") end
    if card_type == CARD_TYPE.FIGHT_MELEE then return engine.asset.Texture("UI/cardview_typeicon_combat_attack.tex") end
    if card_type == CARD_TYPE.FIGHT_SKILL then return engine.asset.Texture("UI/cardview_typeicon_combat_maneuver.tex") end
    if card_type == CARD_TYPE.FIGHT_STATUS then return engine.asset.Texture("UI/cardview_typeicon_combat_status.tex") end
    if card_type == CARD_TYPE.FIGHT_SPECIAL then return engine.asset.Texture("UI/cardview_typeicon_combat_status.tex") end
    if card_type == CARD_TYPE.ITEM then return engine.asset.Texture("UI/cardview_typeicon_item.tex") end
    return engine.asset.Texture("UI/cardview_typeicon_item.tex")
end

function GetCardTypeColour( card_type )
    if card_type == CARD_TYPE.FLOURISH then return UICOLOURS.FLOURISH end
    if card_type == CARD_TYPE.PARASITE then return UICOLOURS.PARASITE end
    if card_type == CARD_TYPE.SCORE then return UICOLOURS.FIGHT_SCORE end
    if card_type == CARD_TYPE.NEGOTIATION_HOSTILE then return UICOLOURS.NEGOTIATION_HOSTILE end   
    if card_type == CARD_TYPE.NEGOTIATION_DIPLOMACY then return UICOLOURS.NEGOTIATION_DIPLOMACY end   
    if card_type == CARD_TYPE.NEGOTIATION_MANIPULATE then return UICOLOURS.NEGOTIATION_MANIPULATE end   
    if card_type == CARD_TYPE.NEGOTIATION_STATUS then return UICOLOURS.NEGOTIATION_STATUS end
    if card_type == CARD_TYPE.NEGOTIATION_OPPONENT then return UICOLOURS.NEGOTIATION_OPPONENT end
    if card_type == CARD_TYPE.FIGHT_MELEE then return UICOLOURS.FIGHT_ATTACK end
    if card_type == CARD_TYPE.FIGHT_SKILL then return UICOLOURS.FIGHT_SKILL end
    if card_type == CARD_TYPE.FIGHT_STATUS then return UICOLOURS.FIGHT_STATUS end
    if card_type == CARD_TYPE.FIGHT_SPECIAL then return UICOLOURS.FIGHT_SPECIAL end
    if card_type == CARD_TYPE.ITEM then return UICOLOURS.FIGHT_ITEM end
end

function GetColourblindModeColours()
    local colours = {
        [CARD_TYPE.NEGOTIATION_HOSTILE] = { 0xE26E77FF, 0x66e4ffFF, 0x3380ffFF, 0xE26E77FF },
        [CARD_TYPE.NEGOTIATION_DIPLOMACY] = { 0x7ddc8dFF, 0x7ddc8dFF, 0x7ddc8dFF, 0x7ddc8dFF },
        [CARD_TYPE.NEGOTIATION_MANIPULATE] = { 0xAF68A8FF, 0xAF68A8FF, 0xAF68A8FF, 0x6a7cd8FF },
        [CARD_TYPE.NEGOTIATION_STATUS] = { 0xECCD53FF, 0xECCD53FF, 0xECCD53FF, 0xffdc51FF },
        -- [CARD_TYPE.NEGOTIATION_OPPONENT] = { 0xFFCA78FF, 0xFFCA78FF, 0xFFCA78FF, 0xFFCA78FF },
        [CARD_TYPE.FIGHT_MELEE] = { 0xFF9C5AFF, 0xc2570fFF, 0xa0720fFF, 0xFF9C5AFF },
        [CARD_TYPE.FIGHT_SKILL] = { 0x6AC8C9FF, 0x6AC8C9FF, 0x6AC8C9FF, 0xf65d4dFF },
        [CARD_TYPE.FIGHT_STATUS] = { 0xECCD53FF, 0xECCD53FF, 0xECCD53FF, 0xffdc51FF },
        -- [CARD_TYPE.FIGHT_SPECIAL] = { 0xFFCA78FF, 0xFFCA78FF, 0xFFCA78FF, 0xFFCA78FF },
        [CARD_TYPE.ITEM] = { 0x6CA280FF, 0x178affFF, 0x6CA280FF, 0x4fa58aFF },
        [CARD_TYPE.PARASITE] = { 0xA3B725ff, 0xdcf918FF, 0x18f9bcFF, 0xb1ec24FF },

        [CARD_TYPE.FLOURISH] = { 0xA3B725ff, 0xdcf918FF, 0x18f9bcFF, 0xb1ec24FF },
    }
    return colours
end

function GetCardTypeString( card_type )
    if card_type == CARD_TYPE.FLOURISH then
        return LOC"UI.CARD_WIDGET.CARD_TYPE.FLOURISH"
    elseif card_type == CARD_TYPE.PARASITE then
        return LOC"UI.CARD_WIDGET.CARD_TYPE.PARASITE"
    elseif card_type == CARD_TYPE.SCORE then
        return LOC"UI.CARD_WIDGET.CARD_TYPE.SCORE"   
    elseif card_type == CARD_TYPE.NEGOTIATION_HOSTILE then
        return LOC"UI.CARD_WIDGET.CARD_TYPE.HOSTILE"
    elseif card_type == CARD_TYPE.NEGOTIATION_DIPLOMACY then
        return LOC"UI.CARD_WIDGET.CARD_TYPE.DIPLOMACY"
    elseif card_type == CARD_TYPE.NEGOTIATION_MANIPULATE then
        return LOC"UI.CARD_WIDGET.CARD_TYPE.MANIPULATE"
    elseif card_type == CARD_TYPE.NEGOTIATION_BYSTANDER then
        return LOC"UI.CARD_WIDGET.CARD_TYPE.BYSTANDER"
    elseif card_type == CARD_TYPE.NEGOTIATION_STATUS then
        return LOC"UI.CARD_WIDGET.CARD_TYPE.STATUS"
    elseif card_type == CARD_TYPE.NEGOTIATION_OPPONENT then
        return LOC"UI.CARD_WIDGET.CARD_TYPE.OPPONENT"
    elseif card_type == CARD_TYPE.FIGHT_MELEE then
        return LOC"UI.FIGHT_SCREEN.CARD_TYPE.ATTACK_TOOLTIP"
    elseif card_type == CARD_TYPE.FIGHT_SKILL then
        return LOC"UI.FIGHT_SCREEN.CARD_TYPE.MANEUVER_TOOLTIP"
    elseif card_type == CARD_TYPE.FIGHT_STATUS then
        return LOC"UI.FIGHT_SCREEN.CARD_TYPE.STATUS_TOOLTIP"
    elseif card_type == CARD_TYPE.FIGHT_SPECIAL then
        return LOC"UI.FIGHT_SCREEN.CARD_TYPE.SPECIAL_TOOLTIP"
    elseif card_type == CARD_TYPE.ITEM then
        return LOC"UI.FIGHT_SCREEN.CARD_TYPE.ITEM_TOOLTIP"
    end
    return ""
end

CARD_PLAY_REASONS = MakeEnum{
    "NO_ACTIONS",
    "UNPLAYABLE",
    "STUNNED",
    "INVALID_TARGET",
    "NO_VALID_TARGETS",
    "NOT_IN_HAND",
    "NO_COMBO",
    "NO_CHARGE",
    "NO_OVERCHARGE",
    "NO_COMPOSURE",
    "NO_TARGET_RESOLVE",
    "REQ_INFLUENCE",
    "REQ_HOSTILITY",
    "REQ_DOMINANCE",
    "REQ_RENOWN",
    "REQ_MONEY",
    "REQ_NOT_MET",
    "NO_SLOTS",
    "NO_BOTTLES",
    "TEAM_FULL",
    "NOT_ENOUGH_HEALTH",
    "MAX_PLAYED",

    "TARGET_DEAD",
    "TARGET_FLED",
    "TARGET_SURRENDERED",
    "TARGET_SELF",
    "TARGET_ALLY_OR_SELF",
    "TARGET_ALLY",
    "TARGET_ENEMY",
    "TARGET_ANY_NOT_SELF",
}

EVENT_PRIORITY_PRESETTOR = 50
EVENT_PRIORITY_ADDITIVE = 100
EVENT_PRIORITY_MULTIPLIER = 200
EVENT_PRIORITY_SETTOR = 300
EVENT_PRIORITY_CLAMP = 400


---------------------------------------------------------------------------

ASPECT_RATIO = MakeEnum
{
    "R4_3",
    "R16_10",
    "R16_9",
    "R21_9",
    "UNRESTRICTED",
}

ASPECT_RATIO_VALUE =
{
    [ASPECT_RATIO.R4_3] = 4/3,
    [ASPECT_RATIO.R16_10] = 16/10,
    [ASPECT_RATIO.R16_9] = 16/9,
    [ASPECT_RATIO.R21_9] = 21/9,
}

ASPECT_RATIO_STRING = 
{
    [ASPECT_RATIO.R4_3] = "4:3",
    [ASPECT_RATIO.R16_10] = "16:10",
    [ASPECT_RATIO.R16_9] = "9:10",
    [ASPECT_RATIO.R21_9] = "21:9",
}

HUD_FLAGS, HUD_FLAGS_STRINGS = MakeBitField{
    "HIDDEN",           -- The entire HUD is hidden
    "SHOW_CHAT_BUBBLES", -- Always show Chat Bubbles
    "SHOW_CHAT_OPTIONS", -- Always show Chat options (Multichoice)
    "SHOW_CARDS",        -- Always show cards in Combat and Negotiation.
}

DEFAULT_TT_WIDTH = 365

-- Camera shake params: Magnitude, Speed, Duration
FIGHT_SHAKE_NORMAL = { 0.25, 55, 0.3 }
FIGHT_SHAKE_CRIT = { 0.4, 60, 0.5 }
FIGHT_SHAKE_ABILITY = { 0.3, 60, 0.2 }

NEGOTIATION_SHAKE = { 0.4, 70, 0.20 }
NEGOTIATION_SHAKE_CRIT = { 0.5, 70, 0.30 }
NEGOTIATION_SHAKE_ABILITY = { 0.3, 60, 0.2 }

CONVO_SHAKE = { 0.25, 50, 0.80 }

NEGOTIATION_CHARACTER_SHAKE = 1.5 -- Multiplier on character shake

FIGHT_PLAX_VOFFSET = -400 -- Plax offset when framing combat.

--fight field geometry

FIGHT_MAX_DIST = 12
FIGHT_FIELD_DEPTH = 7

-- Fakes perspective; additional scaling applied proportional to z-depth
FIGHT_PERSPECTIVE_SCALE = .25

FIGHT_CAM_X_RANGE = 5
FIGHT_CAM_Z_RANGE = 3

FIGHT_CAM_FOV = 5*DEGREES
FIGHT_CAM_ANGLE = 24*DEGREES

FIGHT_CAM_DIST = 180
FIGHT_CAM_HEIGHT = 2

FIGHT_CAM_DEFAULTS =
{
    direction = 90 * DEGREES,
    distance = 180,
    fov = FIGHT_CAM_FOV,
    angle = FIGHT_CAM_ANGLE,
    basey = FIGHT_CAM_HEIGHT,
    basez = 0,
    farview = 500,
    gain = 0.1,
}

FIGHT_CAM_ORTHO_DEFAULTS =
{
    orthoscale = 64,
    basey = 2,
    direction = 90 * DEGREES,
    angle = FIGHT_CAM_ANGLE,   
}

DAMAGE_TYPE = MakeEnumValues{ "SHARP", "DULL", "BULLET", "BOTTLESMSH", "PUNCH" } 

-- Priorities for assigning FIGHT_FORMATION placements
FIGHTER_FORMATION = MakeBitField{
        "FRONT_X", "BACK_X",  --front is close to enemies
        "FRONT_Z", "BACK_Z"}

-- Formation positions for teams of various sizes.
--x, y, mask. x is distance from the centerline. y is depth-spacing. (negative is farther back). Mask is for scoring positions for particular fighters
FIGHT_FORMATIONS =
{
    [1] = { { 7.0, 3.0, nil } },
    [2] =
    {
        {4.5, -2.5, FIGHTER_FORMATION.FRONT_X | FIGHTER_FORMATION.BACK_Z },
        {9.0, 4.5, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.FRONT_Z},
    },
    [3] =
    {
        {3.0, 3.25, FIGHTER_FORMATION.FRONT_X | FIGHTER_FORMATION.FRONT_Z},
        {7.5, -1.0, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.BACK_Z},
        {12.0, 5.0, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.FRONT_Z},
    },
    [4] =
    {
        {3.0, -3.5, FIGHTER_FORMATION.FRONT_X | FIGHTER_FORMATION.BACK_Z},
        {6.0, 3.75, FIGHTER_FORMATION.FRONT_X | FIGHTER_FORMATION.FRONT_Z},
        {8.0, -3.5, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.BACK_Z},
        {12.0, 4.5, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.FRONT_Z},
    },
}


-- Visibility status for FX timeline frames.
VIS_STATUS = MakeEnum{ "DEFAULT", "ALWAYS_SHOW", "HIDE_IN_EDITOR", "HIDE_IN_GAME"}

-------------------------------------------------------------------

-- match DirectoryUtils::FileType
LIST_FILES = 
{
    FILES = 1,
    DIRS = 2,
    BOTH = 3,
}

-- WARNING: update GameRenderer.h enum RenderPass:Type
-- ALSO WARNING: update renderpass.inc
RENDER_PASS_ZPRE = 0
RENDER_PASS_OPAQUE = 1
RENDER_PASS_ALPHA = 2
RENDER_PASS_SELECTION = 3


PIXEL_FORMAT =
{
    DXT1 = 0,
    DXT3 = 1,
    DXT5 = 2,

    A8R8G8B8 = 4,
    R8G8B8 = 5,
    L16 = 6,
    L32 = 7,
    A8 = 8,
    R16G16B16 = 20,
    A16R16G16B16 = 21,
    A32R32G32B32 = 22,
    DEPTH24STENCIL8 = 23,
    A8B8G8R8 = 24,
    ATLASREF = 25,
} 
PIXEL_FORMAT_REV = {}
for k,v in pairs(PIXEL_FORMAT) do
    PIXEL_FORMAT_REV[v] = k
end

-- WARNING: must match SceneGraphModel.cpp
BLEND_MODE =
{
    DISABLED = 0,
    BLENDED = 1,
    ADDITIVE = 2, -- Only use if your source image is premultiplied, which it probably isn't.
    PREMULTIPLIED = 3, -- If you use PREMULTIPLIED, Brook will be sad.
    INVERSEALPHA = 4,
    PREMULTIPLIED_MIN = 5,
    MULTIPLY = 6,
    MAX = 7,
    EMISSIVE = 8,
}



STENCIL_MODES =
{
    OFF = 0,
    SET = 1,
    CLEAR = 2,
}



DEPTH_TEST =
{
    NEVER = 0,
    LESS = 1,
    EQUAL = 2,
    LESSEQUAL = 3,
    GREATER = 4,
    NOTEQUAL = 5,
    GREATEREQUAL = 6,
    ALWAYS = 7,
    DISABLED = 8,
}

COMPRESSION_FORMAT =
{
    DXT1 = 0,
    DXT3 = 1,
}

--must match enum in renderlayer.h
RENDER_LAYER_WORLDOVERLAY = 0
RENDER_LAYER_WORLD = 1
RENDER_LAYER_DECOR = 2
RENDER_LAYER_SHADOW = 3
RENDER_LAYER_GROUNDCOVER = 4
RENDER_LAYER_FLOOR = 5
RENDER_LAYER_SKY = 6

UI_BLOOM = 0.2
UI_VIGNETTE = 0.5
--UI_FILMGRAIN = 0.1

-- Widget focus group policies.
FOCUS_GROUP = MakeEnum{
    "LAST_FOCUS", -- Give focus to last child with focus
    "NEAREST", -- Give focus to the nearest appropriate child 
    "ALWAYS_DEFAULT" -- Give focus to a specified default child widget.
 }

-- How much desaturation is applied to darkened areas.
DARKNESS_DESAT = 0.0

MOOD = MakeEnum{"ANGRY", "NEUTRAL", "HAPPY", "SCARED", "DRUNK", "INJURED"}

TARGET_STATE = MakeEnum{ "NONE", "VALID", "INVALID" }

------------------------------------------------------
-- Character def constants.

GENDER = MakeEnum{"FEMALE", "MALE", "UNDISCLOSED"}
GENDER_META =
{
    [ GENDER.FEMALE ] = { __index = function( t, k )
        local key = TheGame:CreateLocKey( "GENDER_NOUNS.FEMALE", k )
        return TheGame:LookupStr( key )
    end },
    [ GENDER.MALE ] = { __index = function( t, k )
        local key = TheGame:CreateLocKey( "GENDER_NOUNS.MALE", k )
        return TheGame:LookupStr( key )
    end },
    [ GENDER.UNDISCLOSED ] = { __index = function( t, k )
        local key = TheGame:CreateLocKey( "GENDER_NOUNS.UNDISCLOSED", k )
        return TheGame:LookupStr( key )
    end },
}

SPECIES = MakeEnum{"HUMAN", "KRADESHI", "SHROKE", "BEAST", "MECH", "JARACKLE", "SNAIL", "PHICKET" }

SKIN_TONE = MakeEnum{"DARK", "MEDIUM", "LIGHT"}

UNNAMED_SKIN = "UNNAMED"

local GENERIC_HAIR_PALETTE =
{
    0x2c222bff, 0x3b3024ff, 0x4e433fff, 0x3F3326FF,
    0x504444ff, 0x6a4e42ff, 0x554838ff, 0xa7856aff,
    0xb89778ff, 0xdcd0baff, 0x977961ff, 0x3F4837FF,
    0xa56b46ff, 0x91553dff, 0x4D5064FF, 0x6B4756FF,
    0x533d32ff, 0x71635aff, 0xb7a69eff, 0xd6c4c2ff,
    0xD5D09FFF, 0xcabfb1ff, 0x8d4a43ff, 0xb55239ff,
    0xCCBC75FF, 0xD2854BFF, 0x343850FF,
}

local GENERIC_VOICE_ACTORS =
{
    [GENDER.MALE] = { "admiraltyLowClassMale01" },
    [GENDER.FEMALE] = { "admiraltyHighClassFemale01" },
    [GENDER.UNDISCLOSED] = { "admiraltyLowClassMale01", "admiraltyHighClassFemale01" },
}

SPECIES_PROPS =
{
    [ SPECIES.HUMAN ] =
    {
        sentient = true,
        has_hood = true,
        scale_chart =
        {
            [GENDER.MALE] = 1, 
            [GENDER.FEMALE] = .97, 
            [GENDER.UNDISCLOSED] = 1,
        },

        voice_actors = GENERIC_VOICE_ACTORS,

        HAIR_PALETTE = GENERIC_HAIR_PALETTE,

        SKIN_TONES =
        {
            [SKIN_TONE.DARK] = { 
            0x2e2219ff, 0x432d24ff, 0x67453bff, 
            0x2f281cff, 0x44362aff, 0x695046ff, 
            0x2d291dff, 0x433d2dff, 0x69614bff, 
            },

            [SKIN_TONE.MEDIUM] = { 
            0x7A544CFF, 0x845951FF, 0x905E54FF, 
            0x87675aff, 0x967264ff, 0xa57e6eff, 
            0x7C644FFF, 0x8B7759FF, 0x96805EFF, 
            },

            [SKIN_TONE.LIGHT] = {
            0xc17c6cff, 0xd19382ff, 0xDC9D8DFF, 
            0xc39582ff, 0xd2a18cff, 0xf0b8a0ff, 
            0xc59669ff, 0xcfa165ff, 0xefb884ff, 
            },
        }
    },
    [ SPECIES.KRADESHI ] =
    {
        sentient = true,
        nohat = true,
        scale_chart =
        {
            [GENDER.MALE] = 1, 
            [GENDER.FEMALE] = .97,
            [GENDER.UNDISCLOSED] = 1,
        },

        voice_actors = GENERIC_VOICE_ACTORS,

        HAIR_PALETTE = GENERIC_HAIR_PALETTE,

        SKIN_PALETTE =
        {
            0x7b8c5bff,
            0x627754FF,
            0xbec867ff,
            0x8D9947FF,
            0x4e6959ff,
            0xc6b755ff,
        },
        EYES_PALETTE =
        {
            0xfffe8bff,
            0xEF0000FF,
            0xF2F1AAFF,
            0xe2fe8cff,
        },
    },
    [ SPECIES.SHROKE ] =
    {
        sentient = true,
        nohat = true,
        scale_chart =
        {
            [GENDER.MALE] = 1, 
            [GENDER.FEMALE] = .97,
            [GENDER.UNDISCLOSED] = 1,
        },

        voice_actors = GENERIC_VOICE_ACTORS,

        HAIR_PALETTE = GENERIC_HAIR_PALETTE,

        SKIN_PALETTE =
        {
            0x627873FF,
            0x6D8A91FF,
            0x5F5F76FF,
        },
        EYES_PALETTE =
        {
            0xF8EDAFFF,
        },
    },
    [ SPECIES.JARACKLE ] =
    {
        sentient = true,
        scale_chart =
        {
            [GENDER.MALE] = 1,
            [GENDER.FEMALE] = .97,
            [GENDER.UNDISCLOSED] = 1,
        },

        voice_actors = GENERIC_VOICE_ACTORS,

        SKIN_PALETTE = 
        {
            0xeae0cbFF,
            0xB8A792FF,
            0x856E58FF,
        },
        EYES_PALETTE =
        {
            0xFBE3B7FF,
        },
    },
    [ SPECIES.PHICKET ] =
    {
        sentient = true,
        scale_chart =
        {
            [GENDER.MALE] = 1, 
            [GENDER.FEMALE] = 0.97, 
            [GENDER.UNDISCLOSED] = 1,
        },

        voice_actors = GENERIC_VOICE_ACTORS,

        HAIR_PALETTE = GENERIC_HAIR_PALETTE,

        SKIN_PALETTE = 
        {
            0x7A544CFF, 
            0x845951FF, 
            0x905E54FF, 
            0x87675aff, 
            0x967264ff, 
            0xa57e6eff, 
            0x7C644FFF,
            0x8B7759FF,
            0x96805EFF,
        },
    },
    [ SPECIES.SNAIL ] =
    {
        name_pool = "SNAILS",
        sentient = false,

        SKIN_PALETTE =
        {
            0x7b8c5bff,
            0xbec867ff,
            0xa6b779ff,
        },

        SHELL_PALETTE =
        {
            0x736357ff,--mid brown
            0xcfc7acff,--tan
            0x534741ff,--dark brown    
        },
    },
    [ SPECIES.BEAST ] =
    {
        sentient = false,
        name_pool = "PET",
        scale_chart =
        {
            [GENDER.MALE] = 0.85, 
            [GENDER.FEMALE] = 0.85,
            [GENDER.UNDISCLOSED] = 0.85,
        },
        
        SKIN_PALETTE = table.empty,
        HAIR_PALETTE = table.empty,
    },
    
    [ SPECIES.MECH ] =
    {
        sentient = false,
        base_build = "gradle",
        name_pool = "MECH",
        SKIN_PALETTE = table.empty,
        HAIR_PALETTE = table.empty,
    },

}
for k, v in pairs( SPECIES ) do
    assert( SPECIES_PROPS[ k ], "Missing species properties for: "..k )
end


----------------------------------------------------------------------
-- Sim constants

-- Special identifier for the "player's" faction
PLAYER_FACTION = "PLAYER"
-- Special identifier for monsters' factions: hostile to everybody
MONSTER_FACTION = "MONSTER"
-- Special identifier for 'factionless'.
NEUTRAL_FACTION = "NEUTRAL"



GLOVES = MakeEnum{"NONE", "SIMPLE", "ARMOURED"}

-------------------------------

ANIM_IMPORT_SCALE = 0.0035

PLAX_FLAGS, PLAX_FLAG_STRINGS = MakeBitField
{
    "ALLOW_SNAILS", -- Allows snails to appear in this plax slot.
    "ALLOW_MONSTERS", -- Allows monsters to appear in this plax slot.
    "DAY", -- Shows in the day
    "EVENING", -- Shows in the evening (No longer used, but DON'T remove this, as the bits are saved into plax_defs.lua)
    "NIGHT", -- Shows at night
    "BATTLE", -- Shows in battle.
    "CONVERSATION", -- Shows in conversation.
    "EXPENSIVE", -- Expensive to render, skip on low LOD.
}

PLAX_DEFAULT_FLAGS = PLAX_FLAGS.ALLOW_SNAILS | PLAX_FLAGS.ALLOW_MONSTERS | PLAX_FLAGS.DAY | PLAX_FLAGS.NIGHT | PLAX_FLAGS.CONVERSATION | PLAX_FLAGS.BATTLE

LEVEL_OF_DETAIL_LOW = 1
LEVEL_OF_DETAIL_HIGH = 4


INLINE_NOTIFICATION_DISMISS_TIME = 1.5

TEAM = MakeEnum{ "BLUE", "RED"}
BATTLE_RESULT = MakeEnum{ "WON", "LOST", "NEUTRAL", "RUNAWAY" }

PT_RANGES = {
    [1] = {1, 2},
    [2] = {3, 4},
    [3] = {5, 6},
    [4] = {7, 8},
    [5] = {9, 12},
}

PT_COUNT_RANGES = {
    [1] = {1,2},
    [2] = {2,3},
    [3] = {2,3},
    [4] = {3,4},
    [5] = {4,4},
}


MAX_BATTLE_SIZE = 4 -- maximum # of people that can be on side at once

BATTLE_FLAGS, BATTLE_FLAGS_STRINGS = MakeBitField
{
    "NIGHT",        -- Fighting at night.  It's dark, etc.
    "SEND_TO_LIMBO",        -- Anyone alive on losing team is sent to Limbo.
    "NO_SURRENDER",     -- No surrendering.
    "NO_FLEE",          -- No running.
    "NO_DEATH",         -- Nobody can die.
    "NO_REWARDS",       -- No post-battle loot or card reward
    "NO_DRAFT",         -- Awards death loot but no drafts
    "REWARD_DEATH_LOOT", -- Reward death loot even if enemies are not killed
    "SELF_DEFENCE",     -- Player is being attacked.
    "NO_BACKUP",        -- Nobody will join, unless specified
    "NO_BYSTANDERS",    -- bystanders will not get involved
    "BOSS_FIGHT",       -- it's a boss fight
    "ISOLATED",         -- no broad social repercussions - you can kill and friends won't hate you
    "SURVIVAL",         -- Win condition is to survive 'survival_turns' turns.  Implies NO_FLEE, SELF_DEFENCE.
    "NO_DEATH_LOOT",    -- Recieve normal loot, but no death items
    "NO_BURRS"
}

-------------------------------------------------------------------
-- decides what is valid for the primary_target (default to ENEMY)
TARGET_TYPE = MakeEnum{
    "FRIENDLY", -- any allied fighter, but not the acting fighter
    "FRIENDLY_OR_SELF", -- any allied fighter, including self
    "ENEMY", -- any fighter on the opposing team can be the primary target
    "SELF", -- only the acting fighter can be the target
    "ANY", -- anyone can be the primary target
    "ANY_NOT_SELF", -- anyone except yourself
}

-- decides which targets are derived from the 'primary target' (default to SINGLE)
TARGET_MOD = MakeEnum{
    "SINGLE", -- the primary target, only
    "TEAM", -- all fighters in the same team as the primary target
    "ALL", -- all fighters on the field
    "RANDOM1", -- 1 random fighter on the same team as the primary target
    "RANDOM2", -- 2 random fighters on the same team as the primary target
    "RANDOM3", -- 3 random fighters on the same team as the primary target
    "CUSTOM", -- Targets dictated by a custom target_fn
}

FIGHT_STATUS = MakeEnum{"DEAD", "ACTIVE", "SURRENDER", "FLED", "INACTIVE", "DEPARTED"}
IDLE_STATE = MakeEnum{"NEUTRAL", "STUNNED", "AIMING", "SURRENDER", "DEAD"}
SURRENDER_DURATION = 3

MOUTH_SHAPE =
{
    A = {[MOOD.NEUTRAL]="neutral_a", [MOOD.HAPPY]="happy_a" },
    B = {[MOOD.NEUTRAL]="neutral_b", [MOOD.HAPPY]="happy_b" },
    C = {[MOOD.NEUTRAL]="neutral_c", [MOOD.HAPPY]="happy_c" },
    D = {[MOOD.NEUTRAL]="neutral_d", [MOOD.HAPPY]="happy_d" },
    E = {[MOOD.NEUTRAL]="neutral_e", [MOOD.HAPPY]="happy_e" },
    F = {[MOOD.NEUTRAL]="neutral_f", [MOOD.HAPPY]="happy_f" },
    H = {[MOOD.NEUTRAL]="neutral_h", [MOOD.HAPPY]="happy_h" },


    IDLE = {[MOOD.NEUTRAL]="neutral_idle", [MOOD.HAPPY]="happy_idle" },
}

CTYPE = MakeEnum{"BUFF", "DEBUFF", "INNATE", "NONE"}

CONVO_PRIORITY_LOWEST = -10000
CONVO_PRIORITY_LOW = -1000
CONVO_PRIORITY_HIGH = 1000
CONVO_PRIORITY_HIGHEST = 10000

POPUP_W, POPUP_H = RES_X * .75, RES_Y * .8

-------------------------------------------------------------------
-- NEW OPINION STUFF
RELATIONSHIP, RELATIONSHIP_ARRAY = MakeEnumValues{"HATED", "DISLIKED", "NEUTRAL", "LIKED", "LOVED"}

RELATIONSHIP_COST_MODIFIERS = 
{
    [RELATIONSHIP.HATED] = 1.0,
    [RELATIONSHIP.DISLIKED] = 0.5,
    [RELATIONSHIP.LIKED] = -0.25,
    [RELATIONSHIP.LOVED] = -0.5,
}

RELATIONSHIP_COLOURS =
{
    [RELATIONSHIP.HATED ] = UICOLOURS.RELATIONSHIP_HATED,
    [RELATIONSHIP.DISLIKED ] = UICOLOURS.RELATIONSHIP_DISLIKED,
    [RELATIONSHIP.NEUTRAL ] = UICOLOURS.RELATIONSHIP_NEUTRAL,
    [RELATIONSHIP.LIKED ] = UICOLOURS.RELATIONSHIP_LIKED,
    [RELATIONSHIP.LOVED ] = UICOLOURS.RELATIONSHIP_LOVED,
}

function GetRelationshipString(rel)
	local result = nil
	if rel then
		local key = "CONSTANTS.RELATIONSHIP_STRINGS." .. rel
		result = LOC(key)
	end
	return result
end

function GetRelationshipPreviewString(rel, agent)
    local result = nil
    if rel then
        local key = "CONSTANTS.RELATIONSHIP_PREVIEW_STRINGS." .. rel
        result = loc.format( LOC(key), agent )
    end
    return result
end

function GetRelationshipPlayerToolTipString(rel)
	local result = nil
	if rel then
		local key = "CONSTANTS.RELATIONSHIP_PLAYER_TOOLTIPS." .. rel
		return LOC(key)
	end
	return result
end

function GetRelationshipModifierString(rel, agent )
    if rel == RELATIONSHIP.HATED then
        return loc.format( LOC( "UI.NEGOTIATION_PANEL.MODIFIER_HATE" ), agent:GetName() )
    elseif rel == RELATIONSHIP.DISLIKED then
        return loc.format( LOC( "UI.NEGOTIATION_PANEL.MODIFIER_DISLIKE" ), agent:GetName() )
    elseif rel == RELATIONSHIP.LIKED then
        return loc.format( LOC( "UI.NEGOTIATION_PANEL.MODIFIER_LIKE" ), agent:GetName() )
    elseif rel == RELATIONSHIP.LOVED then
        return loc.format( LOC( "UI.NEGOTIATION_PANEL.MODIFIER_LOVE" ), agent:GetName() )
    else
        return loc.format( LOC( "UI.NEGOTIATION_PANEL.MODIFIER_NONE" ), agent:GetName() )
    end
end

RELATIONSHIP_COLOURS =
{
    [RELATIONSHIP.HATED] = UICOLOURS.RELATIONSHIP_HATED,
    [RELATIONSHIP.DISLIKED] = UICOLOURS.RELATIONSHIP_DISLIKED,
    [RELATIONSHIP.NEUTRAL] = UICOLOURS.RELATIONSHIP_NEUTRAL,
    [RELATIONSHIP.LIKED] = UICOLOURS.RELATIONSHIP_LIKED,
    [RELATIONSHIP.LOVED] = UICOLOURS.RELATIONSHIP_LOVED,
}

SUPPORT_ACTION = MakeEnum{
    "HELP",         -- Will help the player in the effort
    "HINDER",       -- Will hinder the effort
    "STAY_OUT",
    "SUPPRESSED",
}

SUPPORT_REASON = MakeEnum{
    "PARTY",
    "VIP",
    "QUEST",
    "MODIFIED",
    "CREW",
    "CAPTIVE",
    "DUTY",
    "FACTION",
    "TARGET_FRIENDSHIP",
    "PLAYER_HATRED",
    "PLAYER_FRIENDSHIP",
    "ALLY"
}

local support_rank = {
    SUPPORT_REASON.MODIFIED,
    SUPPORT_REASON.CREW,
    SUPPORT_REASON.QUEST,
    SUPPORT_REASON.ALLY,
    SUPPORT_REASON.PARTY,
    SUPPORT_REASON.VIP,
    SUPPORT_REASON.CAPTIVE,
    SUPPORT_REASON.DUTY,
    SUPPORT_REASON.TARGET_FRIENDSHIP,
    SUPPORT_REASON.PLAYER_HATRED,
    SUPPORT_REASON.PLAYER_FRIENDSHIP,
    SUPPORT_REASON.FACTION,
}

function CompareSupportReason(sup1, sup2)
    local rank1 = table.arrayfind(support_rank, sup1) or 0
    local rank2 = table.arrayfind(support_rank, sup2) or 0
    
    return rank1 < rank2
end


QUEST_CONVO_HOOK = MakeEnum{"INTRO", "ACCEPTED", "DECLINED"}

BASE_PLAYER_RESOLVE = 36

OPINION_REWARD_VALUE_MOD = 
{
    [RELATIONSHIP.HATED] = .25,
    [RELATIONSHIP.DISLIKED] = .5,
    [RELATIONSHIP.NEUTRAL] = 1,
    [RELATIONSHIP.LIKED] = 1.25,
    [RELATIONSHIP.LOVED] = 1.5,
}

function GetLocalizedRelationshipChangedMessage(rel)
	local result = nil
	if rel then
		local key = "CONSTANTS.RELATIONSHIP_CHANGED_MESSAGE." .. rel
		result = LOC(key)
	end
	return result
end


EVENT_COOLDOWN = 
{
    NONE = 0,
    SHORT = 1,  --not in the same "time"
    MED = 2,    --not in two back to back times
    LONG = 10,  --probably not in the same game
}

DAY_PHASE, PHASE_MASK_STRINGS = MakeBitField{ "DAY", "NIGHT"}
PHASE_MASK_ALL = DAY_PHASE.DAY | DAY_PHASE.NIGHT

LOCAL_ITEMS = MakeBitField{
    "BAY",
    "BOG",
    "PEARL",
}

ITEM_TAGS = MakeBitField{   
                    "COMBAT",
                    "NEGOTIATION",
                    "CHEMICAL", 
                    "ILLICIT",
                    "RISE", 
                    "ADMIRALTY", 
                    "OFFENCE", 
                    "DEFENCE", 
                    "SUPPORT", 
                    "MONSTER", 
                    "SPARK", 
                    "LUMIN", 
                    "GRENADE",
                    "UTILITY",
                    "HOSTILE",
                }


GRAFT_TYPE = MakeEnum{"COMBAT", "NEGOTIATION", "MUTATOR", "SOCIAL", "COIN", "BLASTER", "STORY", "FEAT", "PERK", "METTLE_UPGRADE"}

GRAFT_TYPE_NAMES = {
    [GRAFT_TYPE.COMBAT] = "GRAFT_TYPES.COMBAT",
    [GRAFT_TYPE.NEGOTIATION] = "GRAFT_TYPES.NEGOTIATION",
    [GRAFT_TYPE.MUTATOR] = "GRAFT_TYPES.MUTATOR",
    [GRAFT_TYPE.SOCIAL] = "GRAFT_TYPES.SOCIAL",
    [GRAFT_TYPE.COIN] = "GRAFT_TYPES.COIN",
    [GRAFT_TYPE.BLASTER] = "GRAFT_TYPES.BLASTER",
    [GRAFT_TYPE.STORY] = "GRAFT_TYPES.STORY",
    [GRAFT_TYPE.FEAT] = "GRAFT_TYPES.FEAT",
    [GRAFT_TYPE.PERK] = "GRAFT_TYPE.PERK",
    [GRAFT_TYPE.METTLE_UPGRADE] = "GRAFT_TYPE.METTLE_UPGRADE"
}

GRAFT_MAX_XP = {
    [ CARD_RARITY.COMMON ] = 12,
    [ CARD_RARITY.UNCOMMON ] = 8,
    [ CARD_RARITY.RARE ] = 6,
}

------------------------MONEY STUFF

-- The number of times skipped is indexed into this array. (Anything out of bounds is 0)
SKIP_CARD_REWARD = 10
GRAFT_SKIP_REWARD = 30
BOSS_GRAFT_SKIP_REWARD = 100


BAR_DRINK_COST = 30
DRINK_RESTORE_RESOLVE_AMOUNT = 15
NOODLE_COST = 40

FSSHCAKE_COST = 50
FSSHCAKE_HEAL = 30

ITEM_CARD_RARITY_VALUE = 
{
    [ CARD_RARITY.BASIC ] = 50,
    [ CARD_RARITY.COMMON ] = 50,
    [ CARD_RARITY.UNCOMMON ] = 75,
    [ CARD_RARITY.RARE ] = 150,
}

CARD_RARITY_VALUE = 
{
    [ CARD_RARITY.BASIC ] = 50,
    [ CARD_RARITY.COMMON ] = 50,
    [ CARD_RARITY.UNCOMMON ] = 100,
    [ CARD_RARITY.RARE ] = 200,
}

GRAFT_COSTS = 
{
    [ CARD_RARITY.BASIC ] = 50,
    [ CARD_RARITY.COMMON ] = 100,
    [ CARD_RARITY.UNCOMMON ] = 200,
    [ CARD_RARITY.RARE ] = 300,
}

SIDEQUEST_REWARD = 
{
    60,
    80,
    100,
    120,
    140,
}

CONTRACT_REWARD = 
{
    200,
    200,
    200,
    200,
    200,
}

-- Game over results, see GameState:Win()
GAMEOVER = MakeEnum{
    -- Unmitigated victory!
    "VICTORY",
    -- Victory, for now.  To be continued.
    "ALPHA_VICTORY",
    -- Main player died.
    "DEATH",
    -- Player abandoned this game.
    "ABANDONED"
}

BRIBE_MEDIUM = 40

REMOVE_CARD_PRICE, REMOVE_CARD_RAMP = 50, 50

DEATH_MONEY_LOW = 5
DEATH_MONEY_MEDIUM = 10
DEATH_MONEY_HIGH = 20
DEATH_MONEY_VERY_HIGH = 50

---------------------------

CARD_DROP_START_XP = 
{
    [1] = {0, 0},
    [2] = {0, .25},
    [3] = {.25, .5},
    [4] = {.5, .75},
    [5] = {.5, .75},
}

GRAFT_DROP_RARITY = 
{
    [1] = { CARD_RARITY.COMMON, 1},
    [2] = { CARD_RARITY.COMMON, 1, CARD_RARITY.UNCOMMON, .5},
    [3] = { CARD_RARITY.COMMON, 1, CARD_RARITY.UNCOMMON, 1},
    [4] = { CARD_RARITY.COMMON, .5, CARD_RARITY.UNCOMMON, 2, CARD_RARITY.RARE, .5},
    [5] = { CARD_RARITY.UNCOMMON, 1, CARD_RARITY.RARE, .5},
}

CARD_DROP_RARITY = 
{
    [1] = { CARD_RARITY.COMMON, 1, CARD_RARITY.UNCOMMON, .2},
    [2] = { CARD_RARITY.COMMON, 1, CARD_RARITY.UNCOMMON, .5},
    [3] = { CARD_RARITY.COMMON, 1, CARD_RARITY.UNCOMMON, 0.75, CARD_RARITY.RARE, .1},
    [4] = { CARD_RARITY.COMMON, 1, CARD_RARITY.UNCOMMON, 1, CARD_RARITY.RARE, .5},
    [5] = { CARD_RARITY.COMMON, 1, CARD_RARITY.UNCOMMON, 1, CARD_RARITY.RARE, .5},
}

BOSS_CARD_DROP_RARITY =
{
    [1] = {
        [CARD_RARITY.UNCOMMON] = 2,
        [CARD_RARITY.RARE] = 1
    },
    [2] = {
        [CARD_RARITY.UNCOMMON] = 1,
        [CARD_RARITY.RARE] = 2
    },
    [3] = {
        [CARD_RARITY.RARE] = 3
    },
    [4] = {
        [CARD_RARITY.RARE] = 3
    },
}


VOLUME_SETTINGS =
{
    "master_volume",
    "music_volume",
    "fx_volume",
    "vo_volume",

    "ambience_volume",
    "actions_volume",
    "cards_volume",
    "ui_volume",
}

---------------------------------------------------------------------------

MOD_FOLDER = "mods"

MOD_SETTING = MakeEnum{
    "ENABLED", -- boolean: whether a user mod is enabled or not
}

MOD_STATUS = MakeEnum{
    -- A builtin engine mod.  Cannot be disabled, or uploaded to workshop.
    "BUILTIN",
    -- Local mod content, not registered with Steam Workshop.
    "LOCAL",
    -- Local "dev" mod content, same as LOCAL but not enabled by default
    "DEV",
    -- A workshop mod.
    "WORKSHOP",
}
