local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.EVENT

local attacks =
{
    --- GENERAL TROPHIES

    -- Humanoids

    -- Admiralty Patrol Leader
    spear_head = 
    {
        name = "Spear Head",
        anim = "taunt",
        flavour = "'The wide blade of the Admiralty spears are both impressive and helpful when dealing with the masses.'",
        desc = "For the rest of your turn, your attacks hit all enemies.",

        cost = 1,
        max_charges = 2,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.REPLENISH,

        target_type = TARGET_TYPE.SELF,
        
        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("spear_head")
        end,

        condition =
        {
            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_TARGET_MOD ] = function( self, acc, card, target )
                    if card.owner == self.owner and card:IsAttackCard() and self.battle:GetCurrentTeam() == self.owner:GetTeam() then
                        acc:ModifyValue( TARGET_MOD.TEAM, self )
                    end
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition(self.id, 1, self)
                end
            },
        },
    },

    -- Admiralty Clerk
    letter_opener = 
    {
        name = "Letter Opener",
        anim = "melee_item",
        flavour = "'It's just a nickname, I never opened any letters with it.'",
        anims = { "anim/weapon_melee_letter_opener.zip"},
        desc = "Apply {1} {IMPAIR}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.impair_amt)
        end,

        cost = 0,
        max_charges = 3,
        impair_amt = 2,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE | CARD_FLAGS.REPLENISH,
        
        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                hit.target:AddCondition("IMPAIR", self.impair_amt, self)
            end
        end,
    },

    -- Bandit goon
    makeshift_dagger =
    {
    	name = "Makeshift Dagger",
    	anim = "melee_item",
        flavour = "'This one is my lucky shiv!'",
        anims = { "anim/weapon_melee_makeshift_dagger.zip"},
    	desc = "Apply {1} {WOUND}.\n50% chance to {CONSUME} this card.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.wound_amt )
        end,
        max_xp = 0,

    	cost = 0,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE,

    	min_damage = 2,
    	max_damage = 3,

        wound_amt = 1,

    	OnPostResolve = function( self, battle, attack )
    		attack:AddCondition("WOUND", self.wound_amt, self)
            if math.random() < .5 then
                self:SetFlags( CARD_FLAGS.CONSUME )
    		end
    	end
    },

    makeshift_dagger_plus =
    {
        name = "Sharpened Makeshift Dagger",
        desc = "Apply <#UPGRADE>{1}</> {WOUND}.\n50% chance to {CONSUME} this card.",
        min_damage = 3,
        wound_amt = 2,
    },

    -- Bandit captain
    heavy_cleaver = 
    {
    	name = "Heavy Cleaver",
    	anim = "melee_item",
        flavour = "'I'm not carrying that.'",
        anims = { "anim/weapon_melee_cleaver.zip"},

    	cost = 1,
    	max_charges = 2,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE,

    	min_damage = 4,
    	max_damage = 6,

    	features =
    	{
    		BLEED = 3
    	},
    },

    -- Heavy Laborer
    anchor =
    {
    	name = "Anchor",
    	anim = "melee_item",
        flavour = "'If you keep that momentum going, your next strike is going to be spectacular.'",
    	desc = "At the start of your next turn, gain 3 {TEMP_POWER}.",
        anims = { "anim/weapon_melee_anchor.zip"},

    	cost = 2,
    	max_charges = 3,
        max_xp = 0,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE,

    	min_damage = 8,
    	max_damage = 10,

    	OnPostResolve = function( self, battle, attack )
    		self.owner:AddCondition("anchor", 3)
    	end,

    	condition =
    	{
    		hidden = true,

    		event_handlers = 
    		{
    			[ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
    				self.owner:AddCondition("POWER", self.stacks)
                    self.owner:AddCondition("POWER_LOSS", self.stacks)
    				self.power_added = (self.power_added or 0) + self.stacks
    			end,

    			[ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
    				if self.power_added then
    					self.owner:RemoveCondition("anchor", self.power_added)
                        self.power_added = 0
    				end
    			end
    		},
    	},
    },

    anchor_plus =
    {
        name = "Polished Anchor",
        desc = "At the start of your next turn, gain <#UPGRADE>5</> {TEMP_POWER}.",
        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("anchor", 5)
        end,
    },

    -- Laborer 
    workers_gloves =
    {
        name = "Worker's Gloves",
        anim = "taunt",
        flavour = "'Anything that can manipulate that much weight can also just wreck soft targets.'",
        desc = "Your next attack this turn deals double damage.",

        cost = 1,
        max_charges = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("workers_gloves")
        end,

        condition = 
        {
            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition("workers_gloves")
                end,

                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner then
                        dmgt:AddDamage(dmgt.min_damage, dmgt.max_damage, self)
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if attack.owner == self.owner and attack.card:IsAttackCard() then
                        self.owner:RemoveCondition("workers_gloves", 1, self)
                    end
                end
            }
        },
    },

    -- Chemist
    vapor_vial = 
    {
        name = "Vapor Vial",
        anim = "drink",
        flavour = "'I never told you to drink the Heshing thing, now I'm going to have to make a whole new batch.'",
        desc = "For the rest of your turn, all of your cards have a random cost between 0 and 2 until they are played.",

        cost = 0,
        max_charges = 3,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("vapor_vial", 1, self)
        end,

        condition = 
        {
            OnApply = function( self )
                self.randomized_costs = {}
            end,

            event_handlers = 
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition("vapor_vial")
                end,

                [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                    if self.owner == card.owner and self.randomized_costs then
                        local cost = self.randomized_costs[ card ]
                        if cost == nil and not (self.used_cards and table.contains(self.used_cards, card)) then
                            cost = math.random( 0, 2 )
                            self.randomized_costs[ card ] = cost
                        end
                        if cost then
                            cost_acc:ModifyValue( cost, self )
                        end
                    end
                end,

                [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, card )
                    if card.owner == self.owner then
                        self.used_cards = self.used_cards or {}
                        table.insert(self.used_cards, card)
                        self.randomized_costs[card] = nil
                    end
                end
            },
        },
    },

    -- Heshian Priest
    heshian_amulet =
    {
    	name = "Heshian Amulet",
    	anim = "taunt",
        flavour = "'Once the mind is clear, the void can enter and all will be revealed.'",
    	desc = "Discard any number of cards. {HEAL} 3 per card.",

    	cost = 1,

        max_charges = 1,
        max_xp = 0,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL,

    	target_type = TARGET_TYPE.SELF,

    	OnPostResolve = function( self, battle, attack )
    		local cards = battle:DiscardCards()
            local health = #cards * 3
    		if health > 0 then
    			self.owner:HealHealth( health, self )
    		end
    	end
    },

    heshian_amulet_plus =
    {
        name = "Blessed Heshian Amulet",
        max_charges = 2,
    },

    -- Luminari
    heshian_mask =
    {
        name = "Heshian Mask",
        anim = "taunt",
        flavour = "'That much lumin that close to your eyes can't be good. Those zealots can't seem to get enough of it though, says it helps them see the truth.'",
        desc = "Gain {1} {TEMP_POWER}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.power_amt)
        end,

        cost = 1,

        max_charges = 2,
        power_amt = 3,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("POWER", self.power_amt, self)
            self.owner:AddCondition("POWER_LOSS", self.power_amt, self)
        end,
    },

    -- Jakes Smuggler
    triage =
    {
    	name = "Triage",
    	anim = "taunt",
        flavour = "'The Jakes always seem to have some of the best medics, probably because they are always scrapping with everybody.'",
    	desc = "Remove all debuffs and {EXPEND} all Status cards in your hand and draw pile.",

    	cost = 2,
    	max_charges = 2,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.REPLENISH,

    	target_type = TARGET_TYPE.SELF,

    	OnPostResolve = function( self, battle, attack )
    		self.owner:RemoveAllDebuffs( self )

    		local cards_to_destroy = {}

    		for i,card in battle:GetDrawDeck():Cards() do
    			if card:IsFlagged( CARD_FLAGS.STATUS ) then
    				table.insert(cards_to_destroy, card)
    			end
    		end

    		for i,card in battle:GetHandDeck():Cards() do
                if card:IsFlagged( CARD_FLAGS.STATUS ) then
    				table.insert(cards_to_destroy, card)
    			end
    		end

    		if #cards_to_destroy > 0 then
    			for i,card in ipairs(cards_to_destroy) do
                    battle:ExpendCard( card )
    			end
    		end
    	end
    },

    -- Jakes Dockhand
    gaff =
    {
    	name = "Gaff",
    	anim = "melee_item",
        flavour = "'Yikes, that looks nasty.'",
    	desc = "Gain {DEFEND} equal to the damage dealt by this card.",
        anims = { "anim/weapon_melee_gaff.zip"},

    	cost = 1,
    	max_charges = 3,
        max_xp = 0,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE,

    	min_damage = 2,
    	max_damage = 6,

    	event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if hit.card == self then
                    self.owner:AddCondition("DEFEND", hit.damage or 0, self)
                end
            end
        },
    },

    gaff_plus =
    {
        name = "Boosted Gaff",
        min_damage = 4,
        max_damage = 7,
    },

    -- Monsters

    -- Crayote
    carapace = 
    {
    	name = "Carapace",
        anim = "taunt",
        flavour = "'I keep a plate right over my hearts, for luck you know.'",
    	desc = "Apply {1} {DEFEND}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amount ))
        end,

        shop_price = 100,
    	cost = 1,
    	max_charges = 2,
        defend_amount = 8,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.REPLENISH,

    	target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        

        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                hit.target:AddCondition("DEFEND", self.defend_amount, self)
            end
        end
    },

    -- Crayote Spitter
    razorspine =
    {
    	name = "Razorspine",
        anim = "throw",
        flavour = "'Tearing them out of the glands is some really horrible dreck, but they are a joy to throw.'",
    	desc = "Draw a card.",
        anims = { "anim/weapon_grenade_razorspine.zip"},
        hit_tags = { "ranged" },

    	cost = 1,
    	max_charges = 2,

    	min_damage = 6,
    	max_damage = 6,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED,

    	OnPostResolve = function( self, battle, attack )
    		battle:DrawCards(1)
    	end
    },

    -- Flead
    mandible =
    {
        name = "Mandible",
        anim = "melee_item",
        flavour = "'Tough enough for a few swings, and the splinters they leave in the target are just perfection.'",
        desc = "Apply {IMPAIR} equal to damage dealt by this card.",
        anims = { "anim/weapon_melee_mandible.zip"},

        shop_price = 100,
        cost = 1,
        max_charges = 3,

        min_damage = 2,
        max_damage = 4,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE,

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if hit.card == self then
                    hit.target:AddCondition("IMPAIR", hit.damage or 0, self)
                end
            end
        },
    },

    -- Vroc
    vroc_plume =
    {
        name = "Vroc Plume",
        anim = "taunt",
        flavour = "'Those Vrocs are a tricky bunch, something in the feathers and the musk that confuses the attackers.'",
        desc = "Gain 1 {EVASION}.",

        shop_price = 125,
        cost = 1,
        max_charges = 2,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("EVASION", 1, self)
        end
    },

    -- Lumicyte
    lumicyte_dorsal_spines =
    {
        name = "Lumicyte Dorsal Spines",
        anim = "melee_item",
        anims = { "anim/weapon_melee_lumicyte_spine.zip"},
        flavour = "'The smaller ones are used in jewelry, when they get this big and dangerous they’re only good for combat.'",
        -- anims = { "anim/.zip"},

        shop_price = 100,
        cost = 1,
        max_charges = 2,

        min_damage = 3,
        max_damage = 4,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,

        features =
        {
            EXPOSED = 3,
        },
    },

    targeting_core =
    {
        name = "Targeting Core",
        anim = "taunt",
        flavour = "'These make those robots seem a little bit more than half witted, I guess three-quarters witted?'",
        desc = "Apply {TARGETED}.",
        cost = 0,
        max_charges = 2,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.REPLENISH,
        local_item = LOCAL_ITEMS.BOG,
        
        target_type = TARGET_TYPE.ANY,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("TARGETED", 1, self)
        end
    },

    --- BOSS TROPHIES

    -- Drone Master Jeol
    prototype_drone =
    {
        name = "Prototype Drone",
        flavour = "'THANK YOU FOR YOUR QUICK ACTIVATION. I SEE THAT YOU HAVE NOT PURCHASED THE EXTENDED WARRANTY.'",
        desc = "Summons a prototype drone to fight for you.",
        anim = "taunt",
        cost = 1,
        rarity = CARD_RARITY.UNIQUE,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        CanPlayCard = function( self, battle, target )
            return self.owner:GetTeam():NumActiveFighters() < self.owner:GetTeam():GetMaxFighters(), CARD_PLAY_REASONS.TEAM_FULL
        end,

        OnPostResolve = function( self, battle, attack )
            local agent = Agent( "COMBAT_DRONE" )
            local new_fighter = Fighter.CreateFromAgent( agent, self.owner:GetScale() )
            new_fighter.drone_owner = self.owner

            self.owner:GetTeam():AddFighter( new_fighter )
            self.owner:GetTeam():ActivateNewFighters( battle )
            new_fighter:AddCondition("unstable_drone", 3)
        end,
    },

    -- Sparky
    sparkys_oppressor_cell = 
    {
    	name = "Sparky's Oppressor Cell",
    	desc = "Gain 3 {TEMP_POWER}.",
        anim = "taunt",
        flavour = "'To actually incorporate spark into your cybernetics is crazy as far as I'm concerned, who wants to walk around with a bomb in their chest?'",
    	cost = 0,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("POWER", 3)
            self.owner:AddCondition("POWER_LOSS", 3)
        end,        
    },

    -- Zyn
    zyns_razor =
    {
    	name = "Zyn's Razor",
    	desc = "All of your attacks apply 1 {BLEED} while this is in your hand.",
        flavour = "'There's something engraved on the blade but you can't seem to make it out.'",
    	cost = 0,

    	rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.SKILL,

        event_handlers =
        {
        	[ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if attack.attacker == self.owner and attack.card:IsAttackCard() then
                    if not hit.evaded then
                        hit.target:AddCondition("BLEED", 1, self)
                    end
                end
            end,
        },
    },

    -- Hanbi
    hanbis_power_supply =
    {
    	name = "Hanbi's Power Supply",
    	desc = "Gain 2 {POWER} while this is in your hand.",
        flavour = "'You haven't seen this make before, but it's easy enough to figure out.'",
    	cost = 0,

    	rarity = CARD_RARITY.UNIQUE,
    	flags = CARD_FLAGS.ITEM | CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.SKILL | CARD_FLAGS.REPLENISH,

        deck_handlers = { DECK_TYPE.DISCARDS, DECK_TYPE.IN_HAND, DECK_TYPE.DRAW},

    	event_handlers =
        {
            [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if card == self then
                    if target_deck and target_deck:GetDeckType() == DECK_TYPE.IN_HAND then
                        self.owner:AddCondition( "POWER", 2 )

                    elseif source_deck and source_deck:GetDeckType() == DECK_TYPE.IN_HAND then
                        self.owner:RemoveCondition( "POWER", 2 )
                    end
                end
            end,
        },
    },

    -- Shroog
    shroog_fang =
    {
        name = "Shroog Fang",
        anim = "melee_item",
        anims = { "anim/weapon_melee_shroog_fang.zip"},
        flavour = "'A very impressive trophy, best I can do is fifteen.'",
        desc = "Apply {1} {WOUND}. Gain {2} {POWER}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.wound_amt, self.power_amt)
        end,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,

        wound_amt = 2,
        power_amt = 2,

        min_damage = 4,
        max_damage = 5,

        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                hit.target:AddCondition("WOUND", self.wound_amt, self)
            end
            self.owner:AddCondition("POWER", self.power_amt, self)
        end
    },

    -- Drusk
    drusk_core = 
    {
        name = "Drusk Core",
        anim = "shoot",
        flavour = "'After I cleaned it off, added some new wiring, changed the projection angle, and replaced the resonator core, It was good as new!'",
        desc = "Hits all enemies.",
        anims = { "anim/weapon_blaster_druskcore.zip"},
        hit_tags = { "acid" },

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,

        min_damage = 7,
        max_damage = 12,
        target_mod = TARGET_MOD.TEAM,
    },

    -- Rise Rebel
    guerrilla_tactic =
    {
        name = "Guerrilla Tactic",
        anim = "melee_item",
        flavour = "'Those filthy Rise! Nobody should be allowed to fight like that!'",
        anims = { "anim/weapon_melee_guerrilla.zip"},
        
        cost = 1,
        max_charges = 1,
        max_xp = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE,

        min_damage = 3,
        max_damage = 4, 

        features =
        {
            STUN = 1,
        },
    },

    guerrilla_tactic_plus =
    {
        name = "Advanced Guerrilla Tactic",
        max_charges = 2,
        min_damage = 4,
        max_damage = 4,
    },

    -- Rise Automech
    overloaded_core =
    {
        name = "Overloaded Core",
        anim = "throw",
        flavour = "'It really shakes up those buckets! Makes me laugh to see them fall apart haha!'",
        anims = { "anim/weapon_grenade_overloaded.zip"},
        hit_tags = { "spark" },
        desc = "Hits all enemies. {STUN} enemies with {METALLIC}.",

        cost = 1,
        max_charges = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED | CARD_FLAGS.PIERCING,

        target_mod = TARGET_MOD.TEAM,

        min_damage = 6,
        max_damage = 10,

        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                if hit.target:HasCondition("METALLIC") then
                    hit.target:AddCondition("STUN", 1, self)
                end
            end
        end
    },

    -- Rise Radical
    rpg =
    {
        name = "RPG",
        anim = "shoot",
        flavour = "'Not too practical but it really does some impressive amounts of damage.'",
        desc = "Hits all enemies.",
        anims = { "anim/weapon_rpg.zip"},
        hit_tags = { "burn" },

        cost = 1,
        max_charges = 2,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,

        target_mod = TARGET_MOD.TEAM,

        min_damage = 8,
        max_damage = 10,
    },

    -- Spark Baron Professional
    utility_belt =
    {
        name = "Utility Belt",
        anim = "taunt",
        flavour = "'Always have your favorite toys on deck.'",
        desc = "{IMPROVISE_PLUS} 2 item cards from your deck.",

        cost = 0,
        max_charges = 4,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            local options = {}
            for i,card in battle:GetDrawDeck():Cards() do
                if card:IsItemCard() then
                    table.insert(options, card)
                end
            end
            for i,card in battle:GetDiscardDeck():Cards() do
                if card:IsItemCard() then
                    table.insert(options, card)
                end
            end
            if #options > 0 then
                local cards = table.multipick( options, 5 )
                battle:ImproviseCards(cards, 2, nil, false, nil, self)
            end
        end,
    },

    -- Spark Baron Goon
    shock_core =
    {
        name = "Shock Core",
        anim = "throw",
        flavour = "'You are going to want to throw it before the sizzle, but you get more out of it if you wait.'",
        desc = "Deal 5 damage to yourself.",
        anims = { "anim/weapon_grenade_coil.zip"},
        hit_tags = {"spark"},

        cost = 1,
        max_charges = 1,
        max_xp = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED,

        min_damage = 5,
        max_damage = 5,

        OnPostResolve = function( self, battle, attack )
            self.owner:ApplyDamage(5, self.owner, self)
        end,

        features = 
        {
            STUN = 1
        },
    },

    shock_core_plus =
    {
        name = "Stable Shock Core",
        desc = "",
        OnPostResolve = function( self, battle, attack )
        end
    },

    -- Spark Baron Automech
    automech_blaster =
    {
        name = "Automech Blaster",
        flavour = "'REMOVAL OF BLASTER TECHNOLOGY VOIDS ALL WARRANTY. YOU HAVE BEEN WARNED.'",
        anim = "blast",
        pre_anim = "blast_pre",
        post_anim = "blast_pst",
        desc = "Targets a random enemy. Fires four times.",
        anims = { "anim/weapon_blaster_automech.zip"},
        hit_tags = { "spark" },

        cost = 1,
        max_charges = 3,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED,

        target_mod = TARGET_MOD.RANDOM1,

        min_damage = 2,
        max_damage = 3,
        target_count = 4,
   },

    -- Feral Autodog
    rusty_power_core =
    {
        name = "Rusty Power Core",
        anim = "throw",
        flavour = "'I don't think you can save this one, just throw it back.'",
        anims = { "anim/weapon_grenade_power_core.zip"},
        hit_tags = { "spark" },

        cost = 1,
        max_charges = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED | CARD_FLAGS.PIERCING | CARD_FLAGS.REPLENISH,

        min_damage = 5,
        max_damage = 5,
    },

    
    red_scarf =
    {
        name = "Red Scarf",
        flavour = "'Not sure how this ended up here...'",
        desc = "When this is drawn, gain 1 {POWER}.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.REPLENISH | CARD_FLAGS.UNPLAYABLE,

        event_handlers =
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card == self then
                      card:NotifyTriggered()
                      self.owner:AddCondition("POWER", 1, self)            
                end
            end
        },
    },

    swamp_smasher =
    {
        name = "Swamp Smasher",
        anim = "melee_item",
        flavour = "'All teeth and tumors, a heavy swing that's straight from the heart of the bog.'",
        desc = "If the target is stunned, deal {1} bonus damage.",
        anims = { "anim/weapon_melee_swamp_smasher.zip"},
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,

        max_charges = 2,

        bonus_damage = 15,
        min_damage = 5,
        max_damage = 5,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card == self and target and target:HasCondition("STUN") then
                        dmgt:AddDamage(self.bonus_damage, self.bonus_damage, self)
                    end
                end,
        },
    },

    bog_scimitar =
    {
        name = "Bog Scimitar",
        anim = "melee_item",
        flavour = "'The blade crackles with a strange energy, you feel linked to it while it writhes in your hand.'",
        desc = "{HEAL} an amount equal to the health lost by the target.",
        anims = { "anim/weapon_melee_scimitar.zip"},

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE,

        max_charges = 2,
        min_damage = 3,
        max_damage = 6,

        event_handlers =
        {
            [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
                if source and source.card == self and delta > 0 then
                    self.owner:HealHealth(delta, self)
                end
            end
        },
    },

    seedling =
    {
        name = "Seedling",
        anim = "taunt",
        desc = "Summon 1 Spark Mine Burr on the enemy's side.",
        flavour = "'Just add water!'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.SELF,

        max_charges = 3,

        CanPlayCard = function( self, battle, target )
            return not battle:GetTeam(TEAM.RED):IsFull()
        end,

        OnPostResolve = function( self, battle, attack )
            local team = battle:GetTeam(TEAM.RED)
            team:AddFighter( Fighter.CreateFromAgent( Agent("GROUT_SPARK_MINE" ) ) )
            team:ActivateNewFighters()
        end
    },

    -- Nadan
    nadans_blade =
    {
        name = "Nadan's Blade",
        anim = "melee_item",
        anims = { "anim/weapon_melee_nadans_blade.zip"},
        desc = "Apply 4 stacks of a random debuff chosen from {WOUND}, {BLEED} or {IMPAIR}.",
        flavour = "'Spark tech and bandit savagery together at last.'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,

        min_damage = 6,
        max_damage = 8,

        options = {"WOUND", "BLEED", "IMPAIR"},

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition(self.options[math.random(#self.options)], 4, self)
        end,
    },

    -- Oolo
    charged_disc = 
    {
        name = "Charged Disc",
        anim = "throw",
        desc = "Hits all enemies. Apply 2 {WOUND}.",
        flavour = "'Lumin disc throwing requires a lot of practice if you want the Heshing thing to come back to you.'",
        anims = { "anim/weapon_grenade_disc.zip"},

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,

        min_damage = 5,
        max_damage = 6,

        target_mod = TARGET_MOD.TEAM,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("WOUND", 2, self)
        end
    },

    -- Rook
    rooks_spare_coin =
    {
        name = "Rook's Spare Coin",
        anim = "throw",
        desc = "Draw a card.",
        flavour = "'Call it.'",
        hit_tags = { "critical" },

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,

        min_damage = 4,
        max_damage = 6,

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(1)
        end
    },

    -- Frizz
    lumin_jolt =
    {
        name = "Lumin Jolt",
        anim = "taunt",
        desc = "Gain {1} {SURGE}.",
        flavour = "'The Blue goes so seamlessly with your own bio-metrics because it’s just as alive as you are.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.surge_amt)
        end,

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.SELF,

        surge_amt = 5,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("SURGE", self.surge_amt, self)
        end
    },

    -- Automech boss
    screamer = 
    {
        name = "Screamer",
        anim = "taunt",
        desc = "Summon a Feral Autodog.",
        flavour = "'DEFENSIVE PROTOCOLS ENABLED.'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.SELF,

        CanPlayCard = function( self, battle, target )
            return self.owner:GetTeam():NumActiveFighters() < self.owner:GetTeam():GetMaxFighters(), CARD_PLAY_REASONS.TEAM_FULL
        end,

        OnPostResolve = function( self, battle, attack )
            local agent = Agent( "FERAL_AUTODOG" )
            self.owner:GetTeam():AddFighter( Fighter.CreateFromAgent( agent, 3 ) )
            self.owner:GetTeam():ActivateNewFighters( battle )
        end
    },

    -- Horned Crayote
    yote_horn = 
    {
        name = "Yote Horn",
        anim = "melee_item",
        anims = { "anim/weapon_melee_shroog_fang.zip"},
        desc = "Apply {1} {WOUND} and {2} {IMPAIR}.",
        flavour = "'If it survives the fight be sure to put it on your mantle, they are quite decorative.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.wound_amt, self.impair_amt)
        end,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,

        max_charges = 2,

        min_damage = 6,
        max_damage = 8,

        wound_amt = 2,
        impair_amt = 2,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("WOUND", self.wound_amt, self)
            attack:AddCondition("IMPAIR", self.impair_amt, self)
        end
    },

    battle_plan =
    {
        name = "Battle Plan",
        anim = "taunt",
        desc = "The first attack played on your next turn is played twice.",
        flavour = "'Pay attention.'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("battle_plan", 1, self)
        end,

        condition = 
        {
            desc = "The first attack you play {1:this|next} turn is played twice.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.active and 1 or 2 )
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.EXECUTES_ADDED ] = function( self )
                    self.owner:RemoveCondition( self.id )
                end,

                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.active = true
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    if self.active then
                        self.owner:RemoveCondition( self.id, 1 )
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if self.active and attack.attacker == self.owner and attack.card:IsAttackCard() then
                        self.active = false
                        self.owner:RemoveCondition( self.id, 1 )
                        battle:PlayCard( attack.card )
                    end
                end
            },
        },
    },

    -- Hesh zealots
    lumin_darts =
    {
        name = "Lumin Darts",
        desc = "While this is in your hand, deal 2 damage to a random enemy whenever you draw a card.",
        flavour = "'There’s actually some technology in there that stabilizes it during flight, just watch.'",

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.BURNOUT,

        Trigger = function( self, battle )
            local target_fighters = {}
            battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )

            self.owner:BroadcastEvent( BATTLE_EVENT.CUSTOM_BEGIN, self )
            self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "throw", true, nil, { "anim/weapon_knife_thrown.zip"} )

            for i=1, #target_fighters do
                target_fighters[i]:ApplyDamage( 2, self.owner, self, false, {"lumin"})
            end
            self.owner:BroadcastEvent( BATTLE_EVENT.CUSTOM_END, self )

            for i, target in ipairs(target_fighters) do
                target:CheckForSurrender()
            end
        end,

        event_handlers =
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card ~= self then
                    self:Trigger(battle)
                end
            end,
        },
    },

    -- Jake Lifter
    lifter_claw =
    {
        name = "Lifter Claw",
        anim = "melee_item",
        anims = { "anim/weapon_melee_lifter_claw.zip"},
        desc = "{IMPROVISE} an item card from your draw pile and play it with a random target.",
        flavour = "'Why are there lobsters on the Dock? We weren’t expecting a shipment today.'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,

        min_damage = 3,
        max_damage = 4,

        OnPostResolve = function( self, battle, attack )
            local options = {}
            for i,card in battle:GetDrawDeck():Cards() do
                if card:IsItemCard() then
                    table.insert(options, card)
                end
            end
            if #options > 0 then
                local cards = table.multipick( options, 3 )
                local card = battle:ImproviseCards( cards, 1, nil, nil, nil, self )[1] --table.multipick(options, 3), 1, 1)[1]
                if card then
                    card:TransferCard(battle:GetHandDeck())
                    battle:PlayCard(card)
                end
            end
        end
    },

    -- Rise Turncoat boss
    bolstered_plating =
    {
        name = "Bolstered Plating",
        desc = "Gain {1} {DEFEND} per card in your hand.{2}",
        flavour = "'Forget trying to punch through that, we need to try something else.'",
        loc_strings =
        {
            GAIN_DEFEND = "\n(Gain {1} {DEFEND})",
        },
        desc_fn = function( self, fmt_str )
            if self.engine then
                local defend_amount = self:CalcDefend()
                local str = loc.format( self.def:GetLocalizedString( "GAIN_DEFEND" ), self:CalculateDefendText( defend_amount, self.target ))
                return loc.format( fmt_str, self.defend_amount, str )
            else
                return loc.format( fmt_str, self.defend_amount, "" )
            end
        end,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        defend_amount = 1,

        CalcDefend = function(self)
            local num_cards = self.engine:GetHandDeck():CountCards()
            return math.floor( num_cards ) * self.defend_amount
        end,

        OnPostResolve = function( self, battle, attack )
            --NOTE: Add an additional defend_amount, to accomodate for this card not being in the hand during resolve.
             local defend_amount = self:CalcDefend() + self.defend_amount
            self.owner:AddCondition("DEFEND", defend_amount, self)
        end,
    },

    bladder_gland = 
    {
        name = "Bladder Gland",
        desc = "Take 3 damage, gain {ADRENALINE 4}.",
        flavour = "'Wow. That's really potent.'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags  = CARD_FLAGS.ITEM | CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        max_charges = 1,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("ADRENALINE", 4, self)
            self.owner:SelfDamage( 3, self )
        end
    },

    defective_grenade =
    {
        name = "Defective Grenade",
        desc = "Targets all enemies.",
        flavour = "'Throw it really hard so if it doesn’t explode it at least hurts the guy.'",
        anim = "throw",
        anims = { "anim/weapon_grenade_coil.zip"},
        hit_tags = {"spark"},

        cost = 1,
        max_charges = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.ITEM,
        target_mod = TARGET_MOD.TEAM,
        hit_tags = { "spark" },

        min_damage = 0,
        max_damage = 6,
    },

    overdriver =
    {
        name = "Overdriver",
        anim = "knee",
        desc = "This card gains {1} bonus damage for every card played this turn.",
        flavour = "'Charges up through kinetic motion, just make sure that you vent it properly or you’ll be in a world of hurt.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.bonus_dmg )
        end,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.ITEM | CARD_FLAGS.EXPEND,

        bonus_dmg = 3,

        min_damage = 2,
        max_damage = 4,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local bonus = self.bonus_dmg * self.engine:CountCardsPlayed()
                    dmgt:AddDamage(bonus, bonus, self)
                end
            end,
        },
    },

    underdriver =
    {
        name = "Underdriver",
        anim = "taunt",
        desc = "Apply {1} {DEFEND}. Applies {2} bonus {DEFEND} for every card played this turn.",
        flavour = "'You’ll be a lot tougher than you look.'",
        desc_fn = function( self, fmt_str )
            local current_bonus = self.base_def + (self.engine and self.engine:CountCardsPlayed() or 0) * self.bonus_def
            return loc.format( fmt_str, self:CalculateDefendText(current_bonus), self.bonus_def )
        end,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.ITEM | CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        bonus_def = 2,
        base_def = 3,

        OnPostResolve =function( self, battle, attack )
            attack:AddCondition("DEFEND", self.base_def + battle:CountCardsPlayed() * self.bonus_def, self)
        end,
    },

    bruts_bat =
    {
        name = "Brut's Bat",
        anim = "melee_item",
        desc = "Apply 1 {TRAUMATIZED}.",
        flavour = "'A brutal relic from an outlawed sport.'",
        anims = { "anim/weapon_melee_club.zip"},
        hit_anim = true,

        cost = 2,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND | CARD_FLAGS.ITEM,

        min_damage = 6,
        max_damage = 8,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("TRAUMATIZED")
        end
    },

    gluey_antennae = 
    {
        name = "Gluey Antennae",
        anim = "melee_item",
        anims = { "anim/weapon_melee_gluey_antennae.zip"},
        desc = "Apply {gluey_antennae}.",
        flavour = "'A kind of conduit is formed between the user and the target, it's also super gross.'",

        cost = 1,

        flags = CARD_FLAGS.MELEE | CARD_FLAGS.ITEM,
        target_type = TARGET_TYPE.ENEMY,
        rarity = CARD_RARITY.UNIQUE,
        
        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                hit.target:AddCondition("gluey_antennae", self.impair_amt, self)
            end
        end,

        condition =
        {
            name = "Glued",
            desc = "When <b>Glued</b> reaches {1} stacks, remove <b>Glued</b> and apply {STUN}.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stun_stacks or 3 )
            end,

            CanApply = function( self, source )
                return not self.owner:IsPlayer()
            end,

            ctype = CTYPE.DEBUFF,
        
            apply_sound = "event:/sfx/battle/status/system/Status_Debuff_Physical",

            target_type = TARGET_TYPE.ENEMY,
            stun_stacks = 3,

            OnSetStacks = function( self, delta, old_stacks, new_stacks )
                if new_stacks >= self.stun_stacks then
                    self.owner:AddCondition( "STUN", 1, self )
                    self:SetStacks( new_stacks - self.stun_stacks )
                end
            end,
        }
    },
    
    weezils_scope =
    {
        name = "Weezil's Scope",
        anim = "taunt",
        desc = "Apply {fixed 3}.",
        flavour = "'Pretty standard model, but there’s been a lot of work done on it and it’s still warm.'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.ITEM,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("fixed", 3, self)
        end
    },

    predictive_blast =
    {
        name = "Predictive Blast",
        anim = "shoot",
        desc = "This card deals bonus damage equal to the targets current intent.",
        flavour = "'SCANNING POTENTIAL THREATS. IMPLEMENTING COUNTERMEASURES.'",
        anims = { "anim/weapon_blaster_bogblaster.zip"},
        cost = 1,
        hit_tags = { "spark" },

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND | CARD_FLAGS.ITEM,

        min_damage = 3,
        max_damage = 4,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self and target then
                    local prepared_attacks = target.prepared_attacks
                    local damage_bonus = 0
                    if prepared_attacks then
                        for i,attack in ipairs(prepared_attacks) do
                            for i,hit in ipairs(attack.hits) do
                                if hit.damage then
                                    damage_bonus = damage_bonus + hit.damage
                                end
                            end
                        end
                    end
                    dmgt:AddDamage(damage_bonus, damage_bonus, self)
                end
            end
        },
    },

    denikus_blaster =
    {
        name = "Denikus' Blaster",
        desc = "This applies a random debuff if no other cards have been played this turn.",
        flavour = "'Such impressive craftsmanship! Where did you say you found it again?'",
        anim = "shoot",
        anims = { "anim/weapon_blaster_denikus.zip"},
        hit_tags = { "red_energy" },

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.ITEM,

        min_damage = 2,
        max_damage = 6,

        -- probability
        debuff_options = {
            [ "WOUND" ] = 10,
            [ "BLEED" ] = 8,
            [ "IMPAIR" ] = 8,
            [ "STUN" ] = 2,
            [ "DEFECT" ] = 3,
            [ "TARGETED" ] = 3,
            [ "EXPOSED" ] = 5,
            [ "SPARK_VISOR_MARK" ] = 6,
            [ "TRAUMA" ] = 8,
        },

        debuff_stacks =
        {
            [ "WOUND" ] = 2,
            [ "BLEED" ] = 4,
            [ "IMPAIR" ] = 3,
            [ "STUN" ] = 1,
            [ "DEFECT" ] = 1,
            [ "TARGETED" ] = 1,
            [ "EXPOSED" ] = 2,
            [ "SPARK_VISOR_MARK" ] = 3,
            [ "TRAUMA" ] = 3,
        },

        OnPostResolve = function( self, battle, attack )
            if battle:CountCardsPlayed() == 0 then
                local chosen = weightedpick(self.debuff_options)
                attack:AddCondition(chosen, self.debuff_stacks[chosen], self)
            end
        end,
    },

    high_yield_lumin_bomb =
    {
        name = "High Yield Lumin Bomb",
        desc = "Apply 2 {IMPAIR}.\nHits all enemies.",
        flavour = "'Try not to jostle around too much.'",
        anim = "throw",
        anims = { "anim/weapon_grenade_lumin.zip"},
        hit_tags = { "lumin" },

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.ITEM | CARD_FLAGS.EXPEND,
        target_mod = TARGET_MOD.TEAM,

        min_damage = 4,
        max_damage = 6,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("IMPAIR", 2, self)
        end
    },

    smiths_canteen =
    {
        name = "Smith's Canteen",
        desc = "Gain {POWER 2}, {HEAL} 5 Health and add 1 {drunk} card to your discard pile.",
        flavour = "'Hesh! What is this stuff? Auto-mech cleaner?'",
        anim = "drink",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.ITEM,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("POWER", 2, self)
            self.owner:HealHealth(5)
            local card = Battle.Card( "drunk", self.owner )
            battle:DealCard( card, battle:GetDeck( DECK_TYPE.DISCARDS ) )
        end
    },

    shredmaw_shell =
    {
        name = "Shredmaw Shell",
        desc = "Gain {RIPOSTE 5}.",
        anim = "taunt",
        flavour = "'Fits like a disgustingly warm glove.'",
        icon = "battle/snug_shell.tex",

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.ITEM,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("RIPOSTE", 5, self)
        end
    },

    droad_tail_spike =
    {
        name = "Droad Tail Spike",
        desc = "Apply {WOUND 2}.",
        anim = "melee_item",
        anims = { "anim/weapon_melee_droad_tail.zip"},
        flavour = "'Wait... you killed that thing?'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND | CARD_FLAGS.ITEM,

        min_damage = 4,
        max_damage = 6,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("WOUND", 2, self)
        end
    },

    arachni_blade =
    {
        name = "Arachni-blade",
        desc = "Apply {1} {lumin_burnt}.",
        flavour = "'Whoa there, I didn’t think the installation would be this invasive.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.lumin_burnt_amt )
        end,
        anim = "melee_item",
        anims = { "anim/weapon_melee_arachniblade.zip"},

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND | CARD_FLAGS.ITEM,

        lumin_burnt_amt = 5,

        min_damage = 4,
        max_damage = 5,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("lumin_burnt", self.lumin_burnt_amt, self)
       end
    },

    seemlis_dagger =
    {
        name = "Seemli's Dagger",
        anim = "melee_item",
        anims = { "anim/weapon_melee_knife_seemli.zip"},
        desc = "Apply {WOUND {1}}.\nGain {EVASION {2}}.",
        flavour = "'I could only get the one before the other self-destructed.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.wound_amt, self.evasion_amt )
        end,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,

        wound_amt = 2,
        evasion_amt = 1,

        min_damage = 3,
        max_damage = 6,

        OnPreResolve = function( self, battle, attack )
            self.owner:AddCondition("EVASION", self.evasion_amt, self)
        end,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("WOUND", self.wound_amt, self)
        end
    },

    lumin_canister =
    {
        name = "Lumin Canister",
        anim = "throw",
        desc = "Hits all enemies.",
        flavour = "'Hit the deck!'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,

        target_mod = TARGET_MOD.TEAM,

        min_damage = 10,
        max_damage = 12,
    },

    tei_drone_strike =
    {
        name = "Tei Drone Strike",
        anim = "shoot",
        anims = { "anim/weapon_blaster_tei.zip"},
        desc = "Apply {WOUND 2}.",
        flavour = "'TARGET ACQUIRED.'",
        hit_tags = {"skybarrage"},

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,

        min_damage = 3,
        max_damage = 6,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("WOUND", 2, self)
        end
    },
}

for i, id, data in sorted_pairs(attacks) do
    data.series = "GENERAL"
    data.item_tags = (data.item_tags or 0) | ITEM_TAGS.COMBAT
    assert( data.flags == nil or CheckBits( data.flags, CARD_FLAGS.ITEM))
    Content.AddBattleCard( id, data )
end


-- Negotiation loot
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT
local CARD_FLAGS = negotiation_defs.CARD_FLAGS

local negotiation_loot = 
{
    luminitiate_coin =
    {
        name = "Luminitiate Coin",
        flavour = "'Two more stacks and I’ll get to sleep inside the halls!'",
        desc = "{STACKING}: Double this card's damage.",

        cost = 1,

        max_charges = 3,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM,

        min_persuasion = 2,
        max_persuasion = 2,

        PostLoad = function( self, game_state )
            self._base.PostLoad( self, game_state )
            if self.userdata and self.userdata.stacks ~= nil then
                self:SetStacks( self.userdata.stacks )
            end
        end,

        SetStacks = function( self, stacks )
            self.userdata.stacks = math.min( 9, stacks or 1 )
            self.userdata.min_persuasion_bonus = math.floor(self.def.min_persuasion^self.userdata.stacks) - self.def.min_persuasion
            self.userdata.max_persuasion_bonus = math.floor(self.def.max_persuasion^self.userdata.stacks) - self.def.max_persuasion
        end,

        global_event_handlers =
        {
            [ "card_added" ] = function( self, card )
                if card.id == self.id and card ~= self then
                    self:SetStacks( (self.userdata.stacks or 1) + 1 )
                    self.owner.negotiator:RemoveCard(card)
                end
            end,
        }
    },

    a_hot_tip = 
    {
        name = "A Hot Tip",
        desc = "Gain {1#money}.",
        flavour = "'Snellick’s Fastest is sick today so those odds are way off, swear to Hesh!.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.money_amt)
        end,

        cost = 1,

        max_charges = 2,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM,

        money_amt = 30,

        OnPostResolve = function( self, minigame, persuasion )
            minigame:ModifyMoney( self.money_amt )
        end
    },

    brain_pearl =
    {
        name = "Brain Pearl",
        desc = "While this is in your hand, all cards gain 2 minimum damage.",
        flavour = "'This is some top tier implantation, I'm kind of jealous.",

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.REPLENISH | CARD_FLAGS.UNPLAYABLE,

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion, minigame, target )
                if self.deck == minigame:GetHandDeck() and source.negotiator == self.negotiator and is_instance( source, Negotiation.Card ) then
                    persuasion:AddPersuasion(2, 0, self)
                end
            end
        },
    },

    veil =
    {
        name = "Veil",
        desc = "Dismiss a random enemy intent.",
        flavour = "'Protective and keeps your germs to yourself, it’s just common courtesy.'",

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.REPLENISH | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            local intents = minigame:GetOtherNegotiator(self.negotiator):GetIntents()
            if intents and #intents > 0 then
                minigame:GetOtherNegotiator(self.negotiator):DismissIntent(intents[math.random(#intents)])
            end
        end 
    },

    pearlie_wallet =
    {
        name = "Pearlie Wallet",
        desc = "Gain {1#money}.",
        flavour = "'They won’t be needing this anymore.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.money )
        end,

        cost = 1,
        money = 15,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.CONSUME,

        min_persuasion = 3,
        max_persuasion = 4,

        OnPostResolve = function( self, minigame, targets )
            minigame:ModifyMoney( self.money )
        end,
    },

    banquod_pendant =
    {
        name = "Banquod Pendant",
        desc = "While this is in your hand, all your cards deal 2 bonus damage.",
        flavour = "'Kra’deshi family pendants range drastically in influence as families make trades and lose or gain followers.'",

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.REPLENISH,

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion, minigame, target )
                if self.deck == minigame:GetHandDeck() and source.negotiator == self.negotiator and is_instance( source, Negotiation.Card ) then
                    persuasion:AddPersuasion(2, 2, self)
                end
            end
        },
    },

    classified_file =
    {
        name = "Classified File",
        desc = "Destroy an opponent argument.",
        flavour = "'If it’s shady, the Admiralty probably already knows about it and has it filed somewhere.'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.REPLENISH | CARD_FLAGS.EXPEND,
        max_charges = 2,

        target_enemy = TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY,

        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target.negotiator:DestroyModifier(target, self)
            end
        end
    },

    hat_tip =
    {
        name = "Hat Tip",
        desc = "Gain 3 {RENOWN} and 1 {INFLUENCE}.",

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RENOWN", 3, self)
            self.negotiator:AddModifier("INFLUENCE", 1, self)
        end
    },

    stolen_jewelry =
    {
        name = "Stolen Jewelry",
        desc = "Gain {1#money}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.money_amt)
        end,

        cost = 1,

        max_charges = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.REPLENISH,

        money_amt = 50,

        OnPostResolve = function( self, minigame, persuasion )
            minigame:ModifyMoney( self.money_amt )
        end
    },

    diplomats_badge =
    {
        name = "Diplomat's Badge",
        desc = "Dismiss a random enemy intent.",

        cost = 0,

        max_charges = 3,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.ITEM | CARD_FLAGS.REPLENISH | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            local intents = minigame:GetOtherNegotiator(self.negotiator):GetIntents()
            if intents and #intents > 0 then
                minigame:GetOtherNegotiator(self.negotiator):DismissIntent(intents[math.random(#intents)])
            end
        end 
    },
}

for i, id, carddef in sorted_pairs( negotiation_loot ) do
    carddef.series = "GENERAL"
    assert( carddef.flags == nil or CheckBits( carddef.flags, CARD_FLAGS.ITEM))
    Content.AddNegotiationCard( id, carddef )
end