local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.BATTLE_EVENT


local ITEMS =
{
    combat_gauze =
    {
        name = "Combat Gauze",
        desc = "Remove a random debuff.",
        anim = "taunt",
        flavour = "'I don't understand, does that make it unroll faster or something?'",
        cost = 1,
        max_charges = 3,
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.REPLENISH,

        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                hit.target:RemoveDebuff( self )
            end
        end,
    },

    salve =
    {
        name = "Salve",
        anim = "taunt",
        flavour = "'It numbs really fast, all those microbes get right to work.'",
        desc = "{HEAL {1}}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.heal_amount )
        end,

        max_charges = 1,
        heal_amount = 10,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 1,
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,
        flags = CARD_FLAGS.REPLENISH,

        OnPostResolve = function( self, battle, attack)
            self.target:HealHealth( self.heal_amount, self )
        end,
    },

    healing_vapors = 
    {
        name = "Healing Vapors",
        anim = "taunt",
        flavour = "'You aren't supposed to like the smell, it keeps you standing.'",
        desc = "Apply {MENDING {1}}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.mending_amt )
        end,

        max_charges = 1,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,
        flags = CARD_FLAGS.REPLENISH,

        cost = 1,

        mending_amt = 6,

        OnPostResolve = function( self, battle, attack )
            self.target:AddCondition("MENDING", self.mending_amt, self)
        end,
    },


    tincture =
    {
        name = "Tincture",
        desc = "Gain {POWER 2}.",
        flavour = "'Effects are temporary, and it isn't advisable to use it more than once daily.'",
        anim = "drink",

        item_tags = ITEM_TAGS.CHEMICAL | ITEM_TAGS.ILLICIT | ITEM_TAGS.OFFENCE,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.REPLENISH,

        cost = 0,
        max_charges = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition( "POWER", 2, self )
        end,
    },

    vial_of_slurry =
    {
        name = "Vial of Slurry",
        desc = "Gain an action.",
        flavour = "'I love this stuff! I can just keep on drinking!'",
        anim = "drink",
        item_tags = ITEM_TAGS.CHEMICAL | ITEM_TAGS.ILLICIT | ITEM_TAGS.OFFENCE,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.REPLENISH,

        cost = 0,
        max_charges = 3,

        OnPostResolve = function( self, battle, attack )
            battle:ModifyActionCount( 1 )
        end,
    },

    vroc_whistle  =
    {
        name = "Vroc Whistle",
        desc = "Summons a vroc to fight for you until the end of this fight.",
        anim = "taunt",
        flavour = "'I can't hear anything, can you?'",
        item_tags = ITEM_TAGS.SUPPORT,
        rarity = CARD_RARITY.RARE,
        cost = 0,
        flags = CARD_FLAGS.REPLENISH,
        max_charges = 3,
        target_type = TARGET_TYPE.SELF,
        sound = "event:/sfx/battle/cards/gen/play_card/vroc_whistle",
        synchronous = true,

        CanPlayCard = function( self, battle, target )
            return self.owner:GetTeam():NumActiveFighters() < self.owner:GetTeam():GetMaxFighters(), CARD_PLAY_REASONS.TEAM_FULL
        end,

        OnPostResolve = function( self, battle, attack )
            local vroc = Agent( "VROC" )
            local fighter = Fighter.CreateFromAgent( vroc, battle:GetScenario():GetAllyScale() )
            self.owner:GetTeam():AddFighter( fighter )
            self.owner:GetTeam():ActivateNewFighters()
        end,
    },

    combat_drone =
    {
        name = "Combat Drone",
        desc = "Summons a combat drone to fight for you.",
        flavour = "'THANK YOU FOR PURCHASING THE EXTRA WARRANTY CONSUMER NUMBER 0048.'",
        anim = "taunt",
        cost = 0,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,
        item_tags = ITEM_TAGS.SUPPORT,
        flags = CARD_FLAGS.REPLENISH,

        max_charges = 1,

        CanPlayCard = function( self, battle, target )
            return self.owner:GetTeam():NumActiveFighters() < self.owner:GetTeam():GetMaxFighters(), CARD_PLAY_REASONS.TEAM_FULL
        end,

        OnPostResolve = function( self, battle, attack )
            local summon = Agent( "COMBAT_DRONE" )
            local fighter = Fighter.CreateFromAgent( summon, battle:GetScenario():GetAllyScale() )
            self.owner:GetTeam():AddFighter( fighter )
            self.owner:GetTeam():ActivateNewFighters()
        end,
    },

    crayote_claw =
    {
        name = "Crayote Claw",
        desc = "Draw a card.",
        anim = "throw",
        flavour = "'I've seen them cut through Shrokian combat armor, mostly just the big ones though.'",
        anims = { "anim/weapon_grenade_crayote_claw.zip"},
        cost = 0,
        max_charges = 1,
        hit_tags = { "bleed" },

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.MONSTER,
        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.COMMON,
        features = {
            BLEED = 2
        },

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards( 1 )
        end,
    },

    krill_ichor =
    {
        name = "Krill Ichor",
        desc = "Gain {1} {POWER} and {KRILLED_UP}.",
        flavour = "'Just hold your nose and down the whole thing, also I hope you didn't have big plans tomorrow.'",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.power_amt, self.stacks)
        end,
        
        max_charges = 1,

        anim = "drink",
        cost = 0,
        power_amt = 6,

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.MONSTER,
        
        flags = CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition( "POWER", self.power_amt, self )
            self.owner:AddCondition( "KRILLED_UP", self.stacks, self )
        end,
    },

    spark_grenade =
    {
        name = "Spark Grenade",
        desc = "Targets 2 random enemies.",
        anim = "throw",
        flavour = "'You know you could power a house for a week on this charge right?'",
        anims = { "anim/weapon_grenade_coil.zip"},
        hit_tags = {"spark"},

        cost = 1,
        max_charges = 1,

        rarity = CARD_RARITY.COMMON,
        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.GRENADE | ITEM_TAGS.SPARK,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.PIERCING,
        target_mod = TARGET_MOD.RANDOM2,
        hit_tags = { "spark", "critical" },

        min_damage = 6,
        max_damage = 9,
    },

    stun_gun =
    {
        name = "Stun Gun",
        desc = "Has a 50% chance to {STUN} the target.",
        anim = "shoot",
        flavour = "'Yeah it hits you right in the equilibrium, seems to have varying degrees of effectiveness though.'",
        anims = { "anim/weapon_blaster_stungun.zip"},

        cost = 1,
        max_charges = 2,

        min_damage = 2,
        max_damage = 4,

        item_tags = ITEM_TAGS.OFFENCE,
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.RANGED,
        hit_tags = { "spark", "critical" },

        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                if not hit.evaded and math.random() < 0.5 then
                    hit.target:AddCondition( "STUN" )
                end
            end
        end,
    },

    lumin_grenade =
    {
        name = "Lumin Grenade",
        desc = "Hits all enemies.",
        anim = "throw",
        flavour = "'The blue stuff is supposed to be helpful. It's always a shame to see it used like this here.'",
        hit_tags = {"lumin"},
        anims = { "anim/weapon_grenade_lumin.zip"},
        hit_tags = { "lumin" },

        cost = 1,
        max_charges = 1,

        hit_tags = { "lumin", "critical" },
        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.GRENADE | ITEM_TAGS.LUMIN,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.RANGED,
        target_mod = TARGET_MOD.TEAM,
        
        min_damage = 9,
        max_damage = 14,
    },

    crackler = 
    {
        name = "Crackler",
        anim = "shoot",
        flavour = "'You might want to close your eyes when you pull the trigger.'",
        desc = "Increases the target's {SURRENDER} meter by damage done.",
        cost = 1,
        anims = { "anim/weapon_blaster_bogblaster.zip"},
        
        min_damage = 6,
        max_damage = 12,

        max_charges = 1,

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.GRENADE,
        flags = CARD_FLAGS.RANGED,
        hit_tags = { "spark" },
        --target_mod = TARGET_MOD.TEAM,
        rarity = CARD_RARITY.UNCOMMON,

        OnPostResolve = function( self, battle, attack )
            local dmg = attack:GetTotalDamage()
            for i, hit in attack:Hits() do
                if hit.target:HasMorale() then
                    local health = hit.target:GetMaxHealth()
                    hit.target:DeltaMorale( (dmg)/health )
                end
            end
        end
    },

    shrapnel_grenade =
    {
        name = "Shrapnel Grenade",
        desc = "Hits all enemies.",
        anim = "throw",
        flavour = "'Pull the pin and throw, easy enough. You barely have to aim with a blast radius like that.'",
        anims = { "anim/weapon_grenade_coil.zip"},
        
        hit_tags = {"spark"},

        cost = 1,
        max_charges = 1,

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.GRENADE,
        flags = CARD_FLAGS.RANGED,
        target_mod = TARGET_MOD.TEAM,
        rarity = CARD_RARITY.UNCOMMON,
        min_damage = 5,
        max_damage = 7,

        features =
        {
            WOUND = 2,
            EXPOSED = 2,
        },
    },

    portable_bunker =
    {
        name = "Portable Bunker",
        desc = "You and all allies gain {1} {DEFEND}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:CalculateDefendText( self.defend_amt ))
        end,
        flavour = "'Springs up in an instant! I can't believe the waste on these things to be honest.'",
        cost = 2,
        max_charges = 2,

        rarity = CARD_RARITY.RARE,
        item_tags = ITEM_TAGS.DEFENCE,
        target_type = TARGET_TYPE.SELF,
        target_mod = TARGET_MOD.TEAM,
        flags = CARD_FLAGS.REPLENISH,

        defend_amt = 30,
        
        OnPostResolve= function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
        end
    },

    speed_tonic =
    {
        name = "Speed Tonic",
        desc = "2 random cards in hand cost 0 until played.",
        anim = "drink",
        flavour = "'Who needs sleep?'",

        cost = 1,
        max_charges = 1,
        flags = CARD_FLAGS.REPLENISH,

        item_tags = ITEM_TAGS.CHEMICAL,
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.COMMON,

        OnPostResolve = function( self, battle, attack )
            local hand = battle:GetHandDeck()

            local freebies = {}
            for i = 1, 2 do
                local freebie_card = hand:PeekRandom( function( x ) 
                    local cost = battle:CalculateActionCost( x )
                    return x ~= self and table.find( freebies, x ) == nil and cost > 0
                end )
                if freebie_card then
                    table.insert(freebies, freebie_card)
                    freebie_card.flags = SetBits( freebie_card.flags, CARD_FLAGS.FREEBIE )
                end
            end
        end,
    },

    enfeebling_ray =
    {
        name = "Enfeebling Ray",
        anim = "shoot",
        flavour = "'That injury is so old I forgot I had it. Why is it acting up now?'",
        anims = { "anim/weapon_blaster_enfeebling.zip"},
        hit_tags = { "surge" },

        item_tags = ITEM_TAGS.OFFENCE,
        rarity = CARD_RARITY.COMMON,

        cost = 0,
        max_charges = 2,

        flags = CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        target_type = TARGET_TYPE.ENEMY,

        features =
        {
            IMPAIR = 2,
        },
    },

    spark_shot =
    {
        name = "Spark Shot",
        desc = "Maximum damage is increased by 2 per available use.",
        flavour = "'I'm not here to tell you the safety tips on using this stuff, just know that there probably are a bunch and they are probably very important.'",
        anim = "shoot",
        anims = { "anim/weapon_blaster_spark_shot.zip"},
        hit_tags = { "spark" },

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.SPARK,

        cost = 1,
        max_charges = 4,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 4,
        max_damage = 5,

        deck_handlers = ALL_DECKS,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if self == card then
                    local charges = self:GetCharges() or 0
                    dmgt:AddDamage( 0, charges * 2, self )
                end
            end,
        }
    },

    adrenaline_shot =
    {
        name = "Adrenaline Shot",
        anim = "taunt",
        flavour = "'That's the stuff.'",
        desc = "Add {1} copies of {haste} into your draw pile.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.num_copies)
        end,

        item_tags = ITEM_TAGS.CHEMICAL,

        rarity = CARD_RARITY.COMMON,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.REPLENISH,
        cost = 1,
        max_charges = 3,
        num_copies = 2,

        OnPostResolve = function( self, battle, attack )
            local cards = {}
            for i = 1, self.num_copies do
                local incepted_card = Battle.Card( "haste", self.owner )
                table.insert(cards, incepted_card )
            end
            battle:DealCards( cards )
        end,
    },

    tripwire =
    {
        name = "Tripwire",
        anim = "taunt",
        flavour = "'All of that and it fits right in your pocket!'",
        rarity = CARD_RARITY.RARE,
        cost = 0,
        max_charges = 2,

        features =
        {
            STUN = 1,
        },
    },

    nano_lattice =
    {
        name = "Nano Lattice",
        anim = "taunt",
        flavour = "'The netting manages to disrupt projectiles as well as closer attacks. Enough battery for a couple uses anyways.'",
        desc = "Draw a card.",
        rarity = CARD_RARITY.COMMON,

        cost = 1,
        max_charges = 2,
        target_type = TARGET_TYPE.SELF,
        item_tags = ITEM_TAGS.UTILITY,
        features =
        {
            DEFEND = 9,
        },

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards( 1 )
        end,
    },

    admiralty_requisition =
    {
        name = "Admiralty Requisition",
        anim = "taunt",
        flavour = "'Hey I'm just the messenger.'",
        desc = "Draw 3 cards.",

        cost = 0,
        max_charges = 1,

        rarity = CARD_RARITY.UNCOMMON,
        item_tags = ITEM_TAGS.UTILITY | ITEM_TAGS.ADMIRALTY,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards( 3 )
        end,
    },

    auxiliary =
    {
        name = "Auxiliary",
        anim = "taunt",
        flavour = "'What's the point of carrying them if they aren't always at the ready?'",
        desc = "While in hand, item cards cost 0.",

        item_tags = ITEM_TAGS.UTILITY,
        rarity = CARD_RARITY.RARE,
        local_item = LOCAL_ITEMS.BOG,

        flags = CARD_FLAGS.UNPLAYABLE,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card ~= self and card:IsFlagged( CARD_FLAGS.ITEM ) then
                    cost_acc:ModifyValue( 0, self )
                end
            end
        }
    },

    sequencer =
    {
        name = "Sequencer",
        anim = "taunt",
        flavour = "'Universal energy my friend, that's what the blue is all about.'",
        desc = "Restore all uses to an item card in your deck.",

        item_tags = ITEM_TAGS.UTILITY, 
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,
        max_charges = 3,

        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.REPLENISH | CARD_FLAGS.EXPEND,

        CanPlayCard = function( self, battle, target )
            for i, card in ipairs(battle:GetCardsOwnedBy( self.owner )) do
                if card:IsPartiallySpent() then
                    return true
                end
            end
            return false, CARD_PLAY_REASONS.NO_VALID_TARGETS
        end,

        OnPostResolve = function( self, battle, attack )
            local card = battle:ChooseCardsFromTable( battle:GetCardsOwnedBy( self.owner ), 1, 1, Battle.Card.IsPartiallySpent, LOC "CARD_ENGINE.CHOOSE_SPENT_CARD", self )[1]
            if card then
                local original_deck = card.deck
                original_deck:RemoveCard(card)
                card:RestoreCharges()
                card:TransferCard(original_deck)
            end
        end,
    },

    aerostat_coilgun =
    {
        name = "Aerostat Coilgun",
        anim = "shoot",
        flavour = "'An older model but extremely reliable. The later recruits replaced this with a faster body that self-loaded.'",
        anims = { "anim/weapon_blaster_aerostat.zip"},
        hit_tags = { "lumin" },

        desc = "Gains +{1} bonus damage for every card played this turn.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.strength_gain )
        end,

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        rarity = CARD_RARITY.UNIQUE,
        cost = 1,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH | CARD_FLAGS.EXPEND,

        min_damage = 3,
        max_damage = 3,
        strength_gain = 2,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                local stacks = self.engine:CountCardsPlayed()
                if card == self and stacks > 0 then
                    dmgt:AddDamage( stacks * self.strength_gain, stacks * self.strength_gain, self )
                end
            end,
        },
    },

    bile_grenade = 
    {
        name = "Bile Grenade",
        anim = "throw",
        flavour = "'Ahh Hesh! It's heating up in my hand again!'",
        anims = { "anim/weapon_grenade_bile.zip"},
        hit_tags = {"acid"},
        desc = "At the end of your turn, take 2 damage and increase the damage of this card by 3.\nHits all enemies.",

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        rarity = CARD_RARITY.UNIQUE,
        cost = 1,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,

        target_mod = TARGET_MOD.TEAM,

        min_damage = 6,
        max_damage = 6,

        self_damage = 2,
        strength_gain = 3,

        event_handlers = 
        {
            [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                if self.deck == battle:GetHandDeck() and battle:GetBattleResult() == nil then
                    self:NotifyTriggered()
                    self.owner:ApplyDamage( 2, self.owner, nil, nil, { "acid" } )
                    self.min_damage = self.min_damage + self.strength_gain
                    self.max_damage = self.max_damage + self.strength_gain 
                end
            end
        },
    },

    serrated_dirk = 
    {
        name = "Serrated Dirk",
        anim = "stab",
        flavour = "'That's going to leave a mark.'",
        desc = "Apply 3 {WOUND} and insert {roloach_karambit} into your hand.",

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        rarity = CARD_RARITY.UNIQUE,
        cost = 1,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,

        min_damage = 3,
        max_damage = 3,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition( "WOUND", 3, self )

            local incepted_card = Battle.Card( "roloach_karambit", self:GetOwner() )
            battle:DealCards( {incepted_card}, battle:GetHandDeck() )
        end,
    },

    roloach_karambit =
    {
        name = "Roaloch Karambit",
        anim = "slice",
        flavour = "'A quick wrist movement and the handle becomes just as dangerous.'",
        desc = "Apply 3 {BLEED}.",

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        rarity = CARD_RARITY.UNIQUE,
        cost = 1,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,

        min_damage = 3,
        max_damage = 3,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition( "BLEED", 3, self )
        end,        
    },

    warp_vial =
    {
        name = "Warp Vial",
        anim = "drink",
        flavour = "'There's a lot of things going on in there...'",
        desc = "{WARPING}.{1}",
        desc_fn = function( self, fmt_str )
            if self.bonus_desc then
                return loc.format( fmt_str, self.bonus_desc)
            else
                return loc.format( fmt_str, "" )
            end
        end,
        loc_strings =
        {
            POWER_GAIN = "\nGain 2 {POWER}.",
            HEAL = "\nHeal 8 health.",
            COMBO = "\nGain 4 {COMBO}.",
            MULTI_ATTACK = "\nHits all enemy targets.",
            WOUND = "\nApply 3 {WOUND}.",
        },

        item_tags = ITEM_TAGS.UTILITY | ITEM_TAGS.COMBAT,
        rarity = CARD_RARITY.UNIQUE,
        cost = 1,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,

        effects =
        {
            power_gain =
            {
                desc = "POWER_GAIN",
                anim = "drink",
                target_type = TARGET_TYPE.SELF,
                target_mod = TARGET_MOD.SINGLE,
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,
                effect = function( self, battle, attack )
                    self.owner:AddCondition("POWER", 2, self)
                end,
                min_damage = nil,
                max_damage = nil,
            },

            heal = 
            {
                desc = "HEAL",
                anim = "drink",
                target_type = TARGET_TYPE.SELF,
                target_mod = TARGET_MOD.SINGLE,
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,
                effect = function( self, battle, attack )
                    self.owner:HealHealth(8, self)
                end,
                min_damage = nil,
                max_damage = nil,
            },

            combo = 
            {
                desc = "COMBO",
                anim = "drink",
                target_type = TARGET_TYPE.SELF,
                target_mod = TARGET_MOD.SINGLE,
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,
                effect = function( self, battle, attack )
                    self.owner:AddCondition("COMBO", 4, self)
                end,
                min_damage = nil,
                max_damage = nil,
            },

            multi_attack = 
            {
                desc = "MULTI_ATTACK",
                anim = "throw",
                hit_tags = {"acid"},
                target_type = TARGET_TYPE.ENEMY,
                target_mod = TARGET_MOD.TEAM,
                flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,
                effect = nil,
                min_damage = 3,
                max_damage = 3,
            },

            wound = 
            {
                desc = "WOUND",
                anim = "throw",
                hit_tags = {"acid"},
                target_type = TARGET_TYPE.ENEMY,
                target_mod = TARGET_MOD.SINGLE,
                flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,
                effect = function( self, battle, attack )
                    attack:AddCondition( "WOUND", 3, self )
                end,
                min_damage = 2,
                max_damage = 2,
            },
        },

        ChangeEffect = function( self, effect_id )
            self:NotifyTriggered()
            local effect_table = self.effects[effect_id]
            self.bonus_desc = self.def:GetLocalizedString( effect_table.desc )
            self.anim = effect_table.anim
            self.target_type = effect_table.target_type
            self.target_mod = effect_table.target_mod
            if CheckBits(self.flags, CARD_FLAGS.FREEBIE) then
                self.flags = effect_table.flags | CARD_FLAGS.FREEBIE
            else
                self.flags = effect_table.flags
            end
            self.OnPostResolve = effect_table.effect
            self.min_damage = effect_table.min_damage
            self.max_damage = effect_table.max_damage
        end,

        event_priorities =
        {
            [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = 9,
        },

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS, DECK_TYPE.IN_HAND },

        event_handlers =
        {
            [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                local available_effects = {}
                for id,effect in pairs(self.effects) do
                    if self.current_effect == nil or self.current_effect ~= id then
                        table.insert(available_effects, id)
                    end
                end
                self:ChangeEffect(available_effects[math.random(#available_effects)])
            end,
        }
    },

    robone =
    {
        name = "Robone",
        anim = "taunt",
        flavour = "'CONGRATULATIONS ON YOUR AUTO-DOG RENTAL. THE COUNTDOWN HAS BEEN INITIATED.'",
        desc = "Summon an Autodog to fight for you for the rest of this battle.",
        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        local_item = LOCAL_ITEMS.BOG,
        rarity = CARD_RARITY.RARE,
        cost = 1,
        flags = CARD_FLAGS.REPLENISH,
        target_type = TARGET_TYPE.SELF,

        max_charges = 3,

        CanPlayCard = function( self, battle, target )
            return self.owner:GetTeam():NumActiveFighters() < self.owner:GetTeam():GetMaxFighters(), CARD_PLAY_REASONS.TEAM_FULL
        end,

        OnPostResolve = function( self, battle, attack )
            local summon = Agent( "AUTODOG" )
            local fighter = Fighter.CreateFromAgent( summon, battle:GetScenario():GetAllyScale() )
            self.owner:GetTeam():AddFighter( fighter )
            self.owner:GetTeam():ActivateNewFighters()
        end,
    },

    coil_grenade =
    {
        name = "Coil Grenade",
        anim = "throw",
        flavour = "'Just a few adjustments and these make quite the little explosion.'",
        anims = { "anim/weapon_grenade_coil.zip"},
        hit_tags = {"spark"},
        desc = "Hits all enemies. Apply {1} {IMPAIR}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.impair_amt)
        end,

        max_charges = 2,
        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        local_item = LOCAL_ITEMS.BOG,
        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,

        target_mod = TARGET_MOD.TEAM,

        cost = 1,

        min_damage = 4,
        max_damage = 6,
        impair_amt = 2,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("IMPAIR", self.impair_amt, self)
       end
    },

    oshnu_bile =
    {
        name = "Oshnu Bile",
        anim = "throw",
        flavour = "'The extraction process doesn't hurt them at all. They are pretty much all bile.'",
        anims = { "anim/weapon_grenade_oshnu_bile.zip"},
        hit_tags = {"acid"},
        desc = "Apply {1} {WOUND}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.wound_amt)
        end,

        max_charges = 3,
        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        local_item = LOCAL_ITEMS.BOG,
        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,

        cost = 1,
        min_damage = 2,
        max_damage = 4,
        wound_amt = 3,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("WOUND", self.wound_amt, self)
        end
    },

    energy_siphon =
    {
        name = "Energy Siphon",
        anim = "taunt",
        flavour = "'You do all this work, then some fancy whip just takes it all away?'",
        desc = "Steal all of the target's {POWER}.",
        max_charges = 3,
        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        local_item = LOCAL_ITEMS.BOG,
        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,
        target_type = TARGET_TYPE.ANY_NOT_SELF,

        cost = 1,

        loc_strings =
        {
            NO_POWER = "Target has no Power",
        },

        CanPlayCard = function( self, battle, target )
            if target and not target:HasCondition("POWER") then
                return false, self.def:GetLocalizedString("NO_POWER")
            end
            return true
        end,

        OnPostResolve = function( self, battle, attack )
            local power_stolen = 0
            for i, hit in attack:Hits() do
                if hit.target:HasCondition("POWER") then
                    power_stolen = power_stolen + hit.target:GetConditionStacks("POWER")
                    hit.target:RemoveCondition("POWER")
                end
            end
            if power_stolen > 0 then self.owner:AddCondition("POWER", power_stolen, self) end
        end
    },

    robodex =
    {
        name = "Robodex",
        flavour = "'ALL INTERACTIONS HAVE BEEN LOGGED. INSURANCE IS NOW VOID'",
        desc = "While this is in your hand, gain 1 {TEMP_POWER} whenever you draw a card.",

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        local_item = LOCAL_ITEMS.BOG,
        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.UNPLAYABLE,

        cost = 0,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS, DECK_TYPE.IN_HAND },

        event_handlers =
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if self.deck and self.deck:GetDeckType() == DECK_TYPE.IN_HAND and card.owner == self.owner and self ~= card then
                    self.owner:AddCondition("POWER", 1, self)
                    self.owner:AddCondition("POWER_LOSS", 1, self)
                end
            end,
        },
    },

    replicator =
    {
        name = "Replicator",
        anim = "taunt",
        flavour = "'People use these on the strangest things.'",
        desc = "Duplicate a card in your hand.",

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        local_item = LOCAL_ITEMS.BOG,
        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.SELF,

        cost = 1,

        OnPostResolve = function( self, battle, attack )
            local txt = loc.format( LOC"CARD_ENGINE.CHOOSE_MAX_CARDS_DUPLICATE", 1 )
            local card = battle:ChooseCardsFromTable( battle:GetHandDeck(), 1, 1, nil, txt)[1]
            if card then
                local copy = card:Duplicate()
                copy.show_dealt = false
                battle:DealCard( copy, battle:GetHandDeck())
            end
        end,
    },

    oshnu_glue =
    {
        name = "Oshnu Glue",
        anim = "taunt",
        flavour = "'It's harder to find something that isn't sticky on these beasts.'",
        desc = "Choose a card in your hand and give it {STICKY} for the rest of this fight.",

        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.REPLENISH,
        target_type = TARGET_TYPE.SELF,

        max_charges = 2,

        cost = 0,
        loc_strings = 
        {
            CHOOSE_STICKY = "Choose a card and give it <b>Sticky</>"
        },

        OnPostResolve = function( self, battle, attack )
            local card = battle:ChooseCardsFromTable( battle:GetHandDeck(), 1, 1, nil, self.def:GetLocalizedString("CHOOSE_STICKY"))[1]
            if card then
                card:SetFlags(CARD_FLAGS.STICKY)
            end
        end

    },

    shock_box =
    {
        name = "Shock-Box",
        anim = "shoot",
        flavour = "'Slapped together by the Rise, Shock-Boxes are effective against automechs because of the reboot algorithms added to the signal dump.'",
        desc = "If the target has {METALLIC}, apply {STUN}.",
        anims = { "anim/weapon_blaster_shockbox.zip"},
        hit_tags = { "spark", "electric" },

        rarity = CARD_RARITY.RARE,
        series = CARD_SERIES.GENERAL,
        item_tags = ITEM_TAGS.OFFENCE | ITEM_TAGS.COMBAT,
        local_item = LOCAL_ITEMS.BOG,
        cost = 1,

        min_damage = 2,
        max_damage = 4,

        flags = battle_defs.CARD_FLAGS.RANGED | battle_defs.CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            for i, hit in attack:Hits() do
                if hit.target:HasCondition("METALLIC") then
                    hit.target:AddCondition("STUN", 1, self)
                end
            end
        end,
    },

    nadan_signal =
    {
        name = "Call Nadan",
        flavour = "'Nadan! There's an opening!'",
        anim = "call_in",
        desc = "Call in Nadan to attack your target.",

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,

        hide_in_cardex = true,
        max_charges = 3,
        cost = 0,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("MURDER_BAY_BANDIT_CONTACT"))
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST + 12)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("nadan_leap_and_leave", fighter)
            battle:PlayCard(card, attack.target)
        end
    },

    oolo_signal =
    {
        name = "Call Oolo",
        anim = "call_in",
        flavour = "'Step aside Sal, I got this.'",
        desc = "Call in Oolo to attack all enemies.",

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,

        target_mod = TARGET_MOD.TEAM,

        hide_in_cardex = true,
        max_charges = 3,    
        cost = 0,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("MURDER_BAY_ADMIRALTY_CONTACT"))
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("oolo_throw_disc_charged_and_leave", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    bandage =
    {
        name = "Bandage",
        anim = "taunt",
        flavour = "'Basic triage is a necessity for someone who gets in as much trouble as I do.'",
        desc = "Heal 10 health.",

        cost = 2,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,
        rarity = CARD_RARITY.UNCOMMON,

        max_charges = 3,

        OnPostResolve = function( self, battle, attack)
            attack.target:DeltaHealth( 10 )
        end,
    },
}


for i, id, data in sorted_pairs( ITEMS ) do
    data.item_tags = (data.item_tags or 0) | ITEM_TAGS.COMBAT
    data.flags = (data.flags or 0) | CARD_FLAGS.ITEM 
    assert(data.rarity, loc.format("Rarity missing on {1}", data.name))
    data.series = CARD_SERIES.GENERAL

    Content.AddBattleCard( id, data )
end

