local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local DEBUFF_DAMAGE_MULT = 0.5


local attacks =
{
    drone_discharge = table.extend(NPC_ATTACK)
    {
        name = "Discharge",
        anim = "shoot",
        flags = CARD_FLAGS.RANGED,
        damage_mult = 0.5,

        CanPlayCard = function(self, battle, target)
            return self.owner:GetConditionStacks("drone_charge") > 0
        end,

        OnPostResolve = function(self, battle, attack)
            self.owner:RemoveCondition( "drone_charge" )
        end,
    },

    drone_charge =
    {
        name = "Electro-Charge",
        anim = "taunt",
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.BUFF,

        CanPlayCard = function(self, battle, target)
            return self.owner:GetConditionStacks("drone_charge") < 5
        end,

        OnPostResolve = function(self, battle, attack)
            self.owner:AddCondition( self.id, 3 )
        end,

        condition = 
        {
            desc = "Increases firepower.",
            target_type = TARGET_TYPE.SELF,

            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and card.id == "drone_discharge" then
                        dmgt:ModifyDamage( dmgt.min_damage + self.stacks, dmgt.max_damage + self.stacks, self )
                    end
                end,
            }
        },
    },

    
}

for id, data in pairs(attacks) do
    data.cost = data.cost or 1
    data.rarity = CARD_RARITY.UNIQUE
    data.flags = SetBits( data.flags or 0, CARD_FLAGS.NPC )
    data.series = CARD_SERIES.NPC

    Content.AddBattleCard(id, data)
end
