local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS

local BATTLE_EVENT = ExtendEnum( battle_defs.EVENT,
{
    "PRIMED_GAINED",
    "PRIMED_LOST",
    "LUMIN_CHARGED",
    "LUMIN_SPENT",
})

-- Shared PreReq function
local function NoCharges( self, battle )
    local tracker = self.owner:GetCondition("lumin_tracker") 
    return tracker and tracker:GetCharges() == 0
end

---------------------------------------------------------------------------------

local attacks =
{
    crank = 
    {
        name = "Crank",
        anim = "reload_01",
        desc = "Gain {1} {CHARGE} and gain {2} {DEFEND}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.charge_amt, self:CalculateDefendText(self.defend_amt))
        end,
        flavour = "'You are doing it wrong, really give it a good pull.'",
        cost = 1,

        rarity = CARD_RARITY.BASIC,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,
        defend_amt = 2,
        charge_amt = 2,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then
                tracker:AddLuminCharges(self.charge_amt, self)
            end
            self.owner:AddCondition("DEFEND", self.defend_amt, self)
        end
    },

    crank_plus =
    {
        name = "Induction Crank",
        desc = "Gain <#UPGRADE>{1} {CHARGE}</> and gain {2} {DEFEND}.",
        on_play_screen_shake = true,
        charge_amt = 3,
    },

    crank_plus2 =
    {
        name = "Stone Crank",
        desc = "Gain {1} {CHARGE} and gain <#UPGRADE>{2} {DEFEND}</>.",
        defend_amt = 4,
    },

    hair_trigger = 
    {
        name = "Hair Trigger",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'You have to be careful, I've seen these go off holstered.'",
        post_anim = "blast_pst",
        desc = "Spend 1 {CHARGE}: deal {1} bonus damage.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,

        cost = 1,

        rarity = CARD_RARITY.BASIC,
        flags = CARD_FLAGS.RANGED,

        min_damage = 2,
        max_damage = 3,
        bonus_damage = 2,

        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            return tracker and tracker:GetCharges() > 0
        end,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(1, self)
            end
        end,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetCharges() > 0 then
                        dmgt:AddDamage( self.bonus_damage, self.bonus_damage, self)
                    end
                end
            end
        },
    },

    hair_trigger_plus =
    {
        name = "Enhanced Trigger",
        desc = "Spend <#UPGRADE>up to 2 {CHARGE}: deal <#UPGRADE>{1}</> bonus damage per charge spent.</>",
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(math.min(2, tracker:GetCharges()), self)
            end
        end,
        bonus_damage = 3,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetCharges() > 0 then
                        dmgt:AddDamage( math.min(2, tracker:GetCharges()) * self.bonus_damage, math.min(2, tracker:GetCharges()) * self.bonus_damage, self)
                    end
                end
            end
        },
    },

    hair_trigger_plus2 =
    {
        name = "Boosted Trigger",
        desc = "Spend 1 {CHARGE}: deal <#UPGRADE>{1}</> bonus damage.",
        bonus_damage = 6,
    },

    blast = 
    {
        name = "Blast",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'Center mass!'",
        post_anim = "blast_pst",

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.BASIC,
        cost = 1,
        max_xp = 6,
        wild = true,

        min_damage = 2,
        max_damage = 4,
    },
    
    blast_plus2a =
    {
        name = "Charged Blast",
        desc = "<#UPGRADE>Spend 1 {CHARGE}: Deal 2 bonus damage.</>",

        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            return tracker and tracker:GetCharges() > 0
        end,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 and self.bonus then
                tracker:RemoveCharges(1, self)
            end
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetCharges() > 0 then
                        dmgt:AddDamage( 2, 2, self)
                        self.bonus = true
                    else
                        self.bonus = false
                    end
                end
            end
        },
    },

    blast_plus2b =
    {
        name = "Induction Blast",
        desc = "<#UPGRADE>Gain 1 {CHARGE}.</>",
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then tracker:AddLuminCharges(1, self) end
        end,
    },

    blast_plus2c =
    {
        name = "Lucid Blast",
        min_damage = 7,
        max_damage = 7,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.CONSUME,
    },

    blast_plus2d =
    {
        name = "Visionary Blast",
        desc = "<#UPGRADE>Draw a card.</>",
        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(1)
        end,
    },

    blast_plus2e =
    {
        name = "Fiery Blast",
        pre_anim = nil,
        anim = "burner",
        post_anim = nil,
        desc = "<#UPGRADE>Apply 2 {BURN}.</>",
        flags = CARD_FLAGS.MELEE,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("BURN", 2, self)
        end,
    },

    blast_plus2f = 
    {
        name = "Loaded Blast",
        desc = "<#UPGRADE>{PRIMED}: this costs 0.</>",
        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            return tracker and tracker.primed
        end,

        event_priorities =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )

                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker.primed then
                        cost_acc:ModifyValue(0, self)
                    end
                end
            end,
        },
    },

    blast_plus2g =
    {
        name = "Mirrored Blast",
        desc = "<#UPGRADE>Attack twice.</>",
        hit_count = 2,
    },
    kick =
    {
        name = "Kick",
        anim = "kick",
        flavour = "'Keep it low and use your hips, they'll never see it coming.'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.BASIC,
        cost = 2,
        max_xp = 5,
        wild = true,

        min_damage = 8,
        max_damage = 8,
    },

    kick_plus2a =
    {
        name = "Kickochet",
        desc = "<#UPGRADE>Apply 1 {RICOCHET}.</>",

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("RICOCHET", 1, self)
        end,
    },

    kick_plus2b =
    {
        name = "Pale Kick",
        desc = "<#UPGRADE>{EMPTY}: This costs 1 less.</>",
        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetCharges() == 0 then
                        cost_acc:AddValue(-1, self)
                    end
                end
            end
        },
    },

    kick_plus2c =
    {
        name = "Lucid Kick",
        min_damage = 14,
        max_damage = 14,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.CONSUME,
    },

    kick_plus2d =
    {
        name = "Visionary Kick",
        desc = "<#UPGRADE>Draw a card.</>",
        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(1)
        end,
    },

    kick_plus2g = 
    {
        name = "Double Kick",
        desc = "<#UPGRADE>Attack twice.</>",
        hit_anim = true,

        hit_count = 2,
        min_damage = 5,
        max_damage = 5,
    },

    robo_kick =
    {
        name = "Robo-kick",
        anim = "kick2",
        flavour = "'A simple snap kick with a cybernetic suprise.'",
        desc = "Spend up to {1} {SURGE}: Apply {2} {IMPAIR} per {SURGE} spent.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.surge_spend, self.impair_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 5,
        max_damage = 5,

        surge_spend = 2,
        impair_amt = 2,

        OnPostResolve = function( self, battle, attack )
            if self.owner:HasCondition("SURGE") then
                local surge_spent = math.min(self.surge_spend, self.owner:GetConditionStacks("SURGE"))
                attack:AddCondition("IMPAIR", self.impair_amt * surge_spent)
                self.owner:RemoveCondition("SURGE", surge_spent, self)
            end
        end,
    },

    robo_kick_plus = 
    {
        name = "Boosted Robo-Kick",
        min_damage = 8,
        max_damage = 8,
    },

    robo_kick_plus2 =
    {
        name = "Enhanced Robo-Kick",
        desc = "Spend up to <#UPGRADE>{1}</> {SURGE}: apply {2} {IMPAIR} per {SURGE} spent.",
        surge_spend = 3,
    },

    hunker_down = 
    {
        name = "Hunker Down",
        desc = "Apply {1} {DEFEND}.",
        flavour = "'Hit the dirt!'",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amt ))
        end,
        wild = true,
        anim = "taunt",
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,
        rarity = CARD_RARITY.BASIC,
        cost = 1,
        flags = CARD_FLAGS.SKILL,

        defend_amt = 3,
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
        end
    },

    hunker_down_plus2a =
    {
        name = "Boosted Hunker Down",
        desc = "Apply <#UPGRADE>{1}</> {DEFEND}.",
        defend_amt = 5,
    },

    hunker_down_plus2b =
    {
        name = "Visionary Hunker Down",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>Draw a card.</>",

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
            battle:DrawCards(1)
        end,
    },

    hunker_down_plus2c =
    {
        name = "Induction Hunker Down",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>Gain 1 {CHARGE}.</>",
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then tracker:AddLuminCharges(1, self) end
        end,
    },

    hunker_down_plus2d =
    {
        name = "Surging Hunker Down",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>Gain 2 {SURGE}.</>",
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
            self.owner:AddCondition("SURGE", 2, self)
        end
    },

    hunker_down_plus2e = 
    {
        name = "Lucid Hunker Down",
        desc = "Apply <#UPGRADE>{1}</> {DEFEND}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.CONSUME,
        defend_amt = 8,
    },

    fan_the_hammer = 
    {
        name = "Fan The Hammer",
        desc = "Spend all {CHARGE}: Attack once for each {CHARGE}.",
        pre_anim = "hammer_pre",
        anim = "hammer",
        flavour = "'Fill them full of holes! We've got plenty of ammo!'",
        post_anim = "hammer_pst",
        cost = 1,
        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,
        max_xp = 6,

        min_damage = 2,
        max_damage = 4,

        CanPlayCard = function( self, battle, target )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                return true
            end
            return false, CARD_PLAY_REASONS.NO_CHARGE
        end,

        RefreshHitCount = function( self )
            local tracker = self.owner:GetCondition("lumin_tracker")
            local count = tracker and tracker:GetCharges() or 0
            if count > 0 then
                self.hit_count = count
                self.charge_consumed = count
            else
                self.hit_count = 1
            end
        end,

        event_handlers =
        {
            [ BATTLE_EVENT.LUMIN_CHARGED ] = function( self, battle, charged, max_cells )
                self:RefreshHitCount()
            end,

            [ BATTLE_EVENT.LUMIN_SPENT ] = function( self, battle, charged, max_cells )
                self:RefreshHitCount()
            end,

            [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if card == self then
                    self:RefreshHitCount()
                end
            end,
        },


        OnPreResolve = function( self, battle, attack )
            if self.charge_consumed then
                self.owner:GetCondition("lumin_tracker"):RemoveCharges(self.charge_consumed, self)
                self.charge_consumed = nil
            end
            self.hit_count = 1
            self.calculated = false
        end,
    },

    fan_the_hammer_plus =
    {
        name = "Visionary Hammer",
        desc = "Spend all {CHARGE}: Attack once <#UPGRADE>and draw a card</> for each {CHARGE}.",

        OnPreResolve = function( self, battle, attack )
            if self.charge_consumed then
                battle:DrawCards(self.charge_consumed)
                if self.owner and self.owner:GetCondition( "lumin_tracker" ) then
                    self.owner:GetCondition("lumin_tracker"):RemoveCharges(self.charge_consumed, self)
                end
                self.charge_consumed = nil
            end
            self.hit_count = 1
            self.calculated = false
        end,
    },

    fan_the_hammer_plus2 =
    {
        name = "Overcharging Hammer",
        desc = "Spend all {CHARGE}: Attack once <#UPGRADE>and gain 1 {SURGE}</> for each {CHARGE}.",
        OnPreResolve = function( self, battle, attack )
            if self.charge_consumed then
                self.owner:AddCondition("SURGE", self.charge_consumed, self)
                self.owner:GetCondition("lumin_tracker"):RemoveCharges(self.charge_consumed, self)
                self.charge_consumed = nil
            end
            self.hit_count = 1
            self.calculated = false
        end,
    },

    vent = 
    {
        name = "Vent",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'These things have a tendency to overheat. Oh, and so do the guns.'",
        post_anim = "blast_pst",
        desc = "Spend all {CHARGE}. This card costs 1 less per charge. Hits all enemies.",

        cost = 4,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,
        target_mod = TARGET_MOD.TEAM,

        min_damage = 5,
        max_damage = 8,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    cost_acc:AddValue(-tracker:GetCharges())
                end
            end
        },

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            tracker:RemoveCharges(tracker:GetCharges(), self)
        end
    },

    vent_plus =
    {
        name = "Boosted Vent",
        min_damage = 6,
        max_damage = 10,
    },

    vent_plus2 = 
    {
        name = "Pale Vent",
        cost = 3,
    },

    gemini =
    {
        name = "Gemini",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'Guns akimbo and fresh magazines.'",
        post_anim = "blast_pst",
        desc = "{PRIMED}: Attack twice.",

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 3,
        max_damage = 5,

        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then
                if tracker.primed then
                    self.hit_count = 2
                    return true
                else
                    self.hit_count = 1
                end
            end
            return false
        end,

        event_handlers =
        {
            [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker.primed then
                        self.hit_count = 2
                    else
                        self.hit_count = 1
                    end
                end
            end,
        },
    },

    gemini_plus =
    {
        name = "Tall Gemini",
        min_damage = 3,
        max_damage = 8,
    },

    gemini_plus2 =
    {
        name = "Rooted Gemini",
        min_damage = 5,
        max_damage = 5,
    },

    zero = 
    {
        name = "Zero",
        anim = "psionic",
        flavour = "'I don't think I've ever felt anything quite like that. Are you sure that you didn't see anything?'",
        desc = "{CONCENTRATED}: Apply {1} {WOUND}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.wound_amt )
        end,

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 2,
        max_damage = 5,

        wound_amt = 2,

        PreReq = function( self, battle )
            return self.owner:HasCondition("CONCENTRATION")
        end,

        OnPostResolve = function( self, battle, attack )
            if self.owner:HasCondition("CONCENTRATION") then
                attack:AddCondition("WOUND", self.wound_amt, self)
            end
        end
    },

    zero_plus =
    {
        name = "Boosted Zero",
        min_damage = 4,
        max_damage = 7,
    },

    zero_plus2 =
    {
        name = "Enhanced Zero",
        desc = "{CONCENTRATED}: Apply <#UPGRADE>{1}</> {WOUND}.",
        wound_amt = 3,
    },

    arc_deflection =
    {
        name = "Arc Deflection",
        anim = "reload_02",
        flavour = "'When the pistols are spun up enough, you can actually get a pretty decent shield going.'",
        desc = "Spend up to {1} {SURGE}: Gain {2} {DEFEND} and {3} {RIPOSTE} for each {SURGE} spent.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.surge_cap, self.defend_amt, self.counter_amt )
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.SELF,

        surge_cap = 3,
        defend_amt = 2,
        counter_amt = 2,

        CanPlayCard = function( self, battle, target )
            if not self.owner:GetCondition("SURGE") then
                return false, CARD_PLAY_REASONS.NO_OVERCHARGE
            end
            return true
        end,

        OnPostResolve = function( self, battle, attack )
            local count = math.min(self.owner:GetConditionStacks("SURGE"), self.surge_cap)
            self.owner:RemoveCondition("SURGE", count, self)
            self.owner:AddCondition("DEFEND", count * self.defend_amt, self)
            self.owner:AddCondition("RIPOSTE", count * self.counter_amt, self)
        end
    },

    arc_deflection_plus =
    {
        name = "Boosted Arc Deflection",
        desc = "Spend up to {1} {SURGE}: Gain <#UPGRADE>{2} {DEFEND}</> and {3} {RIPOSTE} for each {SURGE} spent.",
        defend_amt = 3,
    },

    arc_deflection_plus2 =
    {
        name = "Thorny Arc Deflection",
        desc = "Spend up to {1} {SURGE}: Gain {2} {DEFEND} and <#UPGRADE>{3} {RIPOSTE}</> for each {SURGE} spent.",
        counter_amt = 3,
    },

    squeeze =
    {
        name = "Squeeze",
        flavour = "'It's a hidden switch, not regulation by any standard.'",
        anim = "reload_01",
        desc = "Draw {1} cards, gain {SURGE} equal to their combined cost.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.card_draw)
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.SELF,

        card_draw = 2,
        multiplier = 1,

        OnPostResolve = function( self, battle, attack )
            local cards = battle:DrawCards(self.card_draw)
            local cost = 0
            for i,card in ipairs(cards) do
                cost = cost + battle:CalculateActionCost(card)
            end
            if cost > 0 then self.owner:AddCondition("SURGE", cost * self.multiplier, self) end
        end,
    },

    squeeze_plus =
    {
        name = "Visionary Squeeze",
        desc = "Draw <#UPGRADE>{1}</> cards, gain {SURGE} equal to their combined cost.",
        card_draw = 3,
    },

    squeeze_plus2 =
    {
        name = "Boosted Squeeze",
        desc = "Draw {1} cards, gain {SURGE} equal to <#UPGRADE>double</> their combined cost.",
        multiplier = 2,
    },

    spurs =
    {
        name = "Spurs",
        flavour = "'Keeps people from following and makes a Hesh of a noise.'",
        desc = "Spend all {CHARGE}, gain {1} {RIPOSTE} for each {CHARGE} spent.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.counter_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.SELF,

        counter_amt = 2,

        CanPlayCard = function( self, battle, target )
            if self.owner:GetCondition("lumin_tracker") == nil or self.owner:GetCondition("lumin_tracker"):GetCharges() == 0 then
                return false, CARD_PLAY_REASONS.NO_CHARGE
            end
            return true
        end,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                self.owner:AddCondition("RIPOSTE", tracker:GetCharges() * self.counter_amt, self)
                if self.defend_amt then self.owner:AddCondition("DEFEND", tracker:GetCharges() * self.defend_amt, self) end
                tracker:RemoveCharges(tracker:GetCharges(), attack)
            end
        end,
    },

    spurs_plus =
    {
        name = "Boosted Spurs",
        desc = "Spend all {CHARGE}, gain <#UPGRADE>{1}</> {RIPOSTE} for each {CHARGE} spent.",
        counter_amt = 3,
    },

    spurs_plus2 =
    {
        name = "Stony Spurs",
        desc = "Spend all {CHARGE}, gain {1} {RIPOSTE} <#UPGRADE>and {2} {DEFEND}</> for each {CHARGE} spent.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.counter_amt, self:CalculateDefendText(self.defend_amt))
        end,
        defend_amt = 1,
    },

    direct_hit =
    {
        name = "Direct Hit",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'You've been practicing!'",
        post_anim = "blast_pst",
        desc = "Next turn, draw 2 additional cards.",

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 3,
        max_damage = 4,
        draw_bonus = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition( "NEXT_TURN_DRAW", self.draw_bonus )
        end,

    },

    direct_hit_plus =
    {
        name = "Boosted Direct Hit",
        min_damage = 6,
        max_damage = 8,
    },

    direct_hit_plus2 =
    {
        name = "Visionary Hit",
        desc = "Next turn, draw <#UPGRADE>3</> additional cards.",
        draw_bonus = 3,
    },


    quick_fire =
    {
        name = "Quick Fire",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'They didn't stand a chance moving that slow.'",
        post_anim = "blast_pst",
        desc = "Spend 1 {CHARGE}: Deal {1} bonus damage.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.bonus_damage)
        end,
        cost = 1,
        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.COMMON,
        max_xp = 8,

        min_damage = 3,
        max_damage = 5,

        bonus_damage = 3,

        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            return tracker and tracker:GetCharges() > 0
        end,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 and self.bonus then
                tracker:RemoveCharges(1, self)
            end
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetCharges() > 0 then
                        dmgt:AddDamage( self.bonus_damage, self.bonus_damage, self)
                        self.bonus = true
                    else
                        self.bonus = false
                    end
                end
            end
        },
    },

    quick_fire_plus =
    {
        name = "Rooted Quick Fire",
        min_damage = 6,
        max_damage = 6,
    },

    quick_fire_plus2 =
    {
        name = "Boosted Quick Fire",
        desc = "Spend 1 {CHARGE}: Deal <#UPGRADE>{1}</> bonus damage.",
        bonus_damage = 5,
    },

    take_aim =
    {
        name = "Take Aim",
        anim = "mark",
        flavour = "'Hold your breath and keep it steady.'",
        desc = "Apply 2 {MARK}.\nDraw a card.",
        anim = "taunt",
        cost = 1,
        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.COMMON,

        max_xp = 6,

        mark_amt = 2,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("MARK", self.mark_amt, self)
            battle:DrawCards(self.draw_count)
        end
    },

    take_aim_plus = 
    {
        name = "Maintained Aim",
        desc = "Apply <#UPGRADE>4</> {MARK}.\nDraw a card.",
        mark_amt = 4,
    },

    take_aim_plus2 =
    {
        name = "Visionary Aim",
        desc = "Apply 2 {MARK}.\nDraw <#UPGRADE>2 cards</>.",
        draw_count = 2,
    },

    ammo_pouch = 
    {
        name = "Ammo Pouch",
        anim = "reload_01",
        flavour = "'I can never find anything in these damned things.'",
        desc = "{IMPROVISE} a card from a pool of special cards.",

        target_type = TARGET_TYPE.SELF,

        rarity = CARD_RARITY.BASIC,
        flags = CARD_FLAGS.SKILL,
        cost = 1,
        max_xp = 8,

        pool_size = 3,

        pool_cards = {"improvise_spare_ammo", "improvise_laser_sight", "improvise_scour", "improvise_rat_shot", "improvise_yote_shot", "improvise_tracer", "improvise_flow", "blowout", "deep_breath",},

        OnPostResolve = function( self, battle, attack)
            local cards = ObtainWorkTable()

            cards = table.multipick( self.pool_cards, self.pool_size )
            for k,id in pairs(cards) do
                cards[k] = Battle.Card( id, self.owner  )
            end
            battle:ImproviseCards( cards, 1, nil, nil, nil, self )
            ReleaseWorkTable(cards)
        end,
    },

    ammo_pouch_plus =
    {
        name = "Promoted Ammo Pouch",
        desc = "{IMPROVISE} a card from a pool of <#UPGRADE>upgraded</> special cards.",
        pool_cards = {"improvise_spare_ammo_upgraded", "improvise_laser_sight_upgraded", "improvise_scour_upgraded", "improvise_rat_shot_upgraded", "improvise_yote_shot_upgraded", "improvise_tracer_upgraded", "improvise_flow_upgraded", "blowout_upgraded", "deep_breath_upgraded",},
    },

    ammo_pouch_plus2 =
    {
        name = "Boosted Ammo Pouch",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> a card from a pool of special cards.",
        pool_size = 5,
    },

    blowout = 
    {
        name = "Blowout",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'Sure you can customize it to do that, just don't let the Admiralty find out.'",
        post_anim = "blast_pst",
        desc = "Spend all {CHARGE}: deal {1} bonus damage per {CHARGE} spent.",
        desc_fn= function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_damage = 3,
        max_damage = 4,

        bonus_damage = 1,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local lumin_tracker = self.owner:GetCondition("lumin_tracker")
                    dmgt:AddDamage(lumin_tracker:GetCharges() * self.bonus_damage, lumin_tracker:GetCharges() * self.bonus_damage, self)
                end
            end
        },

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            tracker:RemoveCharges(tracker:GetCharges(), self)
        end
    },

    blowout_upgraded =
    {
        name = "Boosted Blowout",
        icon = "battle/blowout.tex",
        pre_anim = "blast_pre",
        anim = "blast",
        post_anim = "blast_pst",
        desc = "Spend all {CHARGE}: deal {1} bonus damage per {CHARGE} spent.",
        desc_fn= function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,

        cost = 0,

        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_damage = 4,
        max_damage = 8,

        bonus_damage = 2,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local lumin_tracker = self.owner:GetCondition("lumin_tracker")
                    dmgt:AddDamage(lumin_tracker:GetCharges() * self.bonus_damage, lumin_tracker:GetCharges() * self.bonus_damage, self)
                end
            end
        },

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            tracker:RemoveCharges(tracker:GetCharges(), self)
        end
    },

    deep_breath = 
    {
        name = "Deep Breath",
        desc = "Gain 2 {CONCENTRATION}. Draw a card.",
        anim = "taunt",
        flavour = "'I love the smell of lumin in the morning.'",
        max_xp = 0,
        cost = 0,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("CONCENTRATION", 2, self)
            battle:DrawCards(1)
        end
    },

    deep_breath_upgraded =
    {
        name = "Deeper Breath",
        desc = "Gain 4 {CONCENTRATION}. Draw a card.",
        icon = "battle/deep_breath.tex",
        anim = "taunt",

        cost = 0,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("CONCENTRATION", 4, self)
            battle:DrawCards(1)
        end
    },

    improvise_spare_ammo =
    {
        name = "Spare Battery",
        anim = "reload_01",
        flavour = "'Never leave home without it.'",
        desc = "Gain 2 {CHARGE}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then tracker:AddLuminCharges(2) end
        end
    },

    improvise_spare_ammo_upgraded =
    {
        name = "Boosted Spare Battery",
        icon = "battle/improvise_spare_ammo.tex",
        anim = "reload_01",
        desc = "Gain 2 {CHARGE} and gain 2 {DEFEND}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then tracker:AddLuminCharges(2) end
            self.owner:AddCondition("DEFEND", 2, self)
        end
    },

    improvise_laser_sight =
    {
        name = "Laser Sight",
        anim = "mark",
        flavour = "'Be sure to get the adjustments right, the dealers are always calibrating these things poorly.'",
        desc = "Apply 2 {MARK}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("MARK", 2, self)
        end
    },

    improvise_laser_sight_upgraded =
    {
        name = "Focused Laser Sight",
        anim = "mark",

        icon = "battle/improvise_laser_sight.tex",
        desc = "Apply 3 {MARK}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("MARK", 3, self)
        end
    },

    improvise_scour =
    {
        name = "Scour",
        anim = "reload_01",
        flavour = "'Cleaned them just this morning, this swamp ruins the guts of these things.'",
        desc = "Draw 2 cards.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,
        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(2)
        end
    },

    improvise_scour_upgraded =
    {
        name = "Visionary Scour",
        anim = "reload_01",
        desc = "Draw 3 cards.",
        icon = "battle/improvise_scour.tex",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,
        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(3)
        end
    },

    improvise_rat_shot =
    {

        name = "Rat Shot",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'A wider blast setting, useful for small targets.'",
        post_anim = "blast_pst",
        desc = "Spend 1 {CHARGE}: Apply 1 {IMPAIR}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,
        min_damage = 2,
        max_damage = 2,
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(1, self)
                attack:AddCondition("IMPAIR", 1, self)
            end
        end,
    },

    improvise_rat_shot_upgraded =
    {
        name = "Boosted Rat Shot",
        pre_anim = "blast_pre",
        anim = "blast",
        post_anim = "blast_pst",
        icon = "battle/improvise_rat_shot.tex",
        desc = "Spend 1 {CHARGE}: Apply 2 {IMPAIR}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,
        min_damage = 2,
        max_damage = 2,
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(1, self)
                attack:AddCondition("IMPAIR", 2, self)
            end
        end,
    },

    improvise_yote_shot =
    {
        name = "Yote Shot",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'A focused setting on the lumin round, it can punch through a carapace easier.'",
        post_anim = "blast_pst",
        desc = "Spend 1 {CHARGE}: Apply 1 {WOUND}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,
        min_damage = 2,
        max_damage = 2,
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(1, self)
                attack:AddCondition("WOUND", 1, self)
            end
        end,
    },

    improvise_yote_shot_upgraded =
    {
        name = "Boosted Yote Shot",
        pre_anim = "blast_pre",
        anim = "blast",
        post_anim = "blast_pst",
        icon = "battle/improvise_yote_shot.tex",
        desc = "Spend 1 {CHARGE}: Apply 2 {WOUND}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,
        min_damage = 2,
        max_damage = 2,
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(1, self)
                attack:AddCondition("WOUND", 2, self)
            end
        end,
    },

    improvise_tracer =
    {
        name = "Tracer Round",
        anim = "burner",
        flavour = "'It's just a four degree twist on the casing there, yeah you can't get these shells anywhere else.'",
        desc = "Spend 1 {CHARGE}: Apply 2 {BURN}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MELEE,
        min_damage = 1,
        max_damage = 2,
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(1, self)
                attack:AddCondition("BURN", 2, self)
            end
        end
    },

    improvise_tracer_upgraded =
    {
        name = "Boosted Tracer Round",
        anim = "burner",
        post_anim = "blast_pst",
        icon = "battle/improvise_tracer.tex",
        desc = "Spend 1 {CHARGE}: Apply 3 {BURN}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MELEE,
        min_damage = 1,
        max_damage = 2,
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(1, self)
                attack:AddCondition("BURN", 3, self)
            end
        end
    },

    improvise_flow =
    {
        name = "Flow",
        pre_anim = "surge_pre",
        anim = "surge",
        flavour = "'Make sure no one catches the color of the muzzle flare. It's a dead giveaway.'",
        post_anim = "surge_pst",
        desc = "Gain 1 {SURGE}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,
        min_damage = 1,
        max_damage = 2,
        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("SURGE", 1, self)
        end
    },

    improvise_flow_upgraded =
    {
        name = "Boosted Flow",
        pre_anim = "surge_pre",
        anim = "surge",
        post_anim = "surge_pst",
        icon = "battle/improvise_flow.tex",
        desc = "Gain 2 {SURGE}.",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,
        min_damage = 2,
        max_damage = 3,
        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("SURGE", 2, self)
        end
    },

    gallery = 
    {
        name = "Gallery",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'It's like shooting tallowfish in a barrel!'",
        post_anim = "blast_pst",
        desc = "{PRIMED}: deals {1} bonus damage and hits all enemies.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,
        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.RANGED,
        cost = 1,
        max_xp = 7,

        bonus_damage = 2,

        target_mod = TARGET_MOD.SINGLE,

        min_damage = 3,
        max_damage = 3,

        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            return tracker and tracker.primed
        end,


        event_handlers = 
        {
            [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if card == self and target_deck:GetDeckType() == DECK_TYPE.IN_HAND then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker.primed then
                        self.target_mod = TARGET_MOD.TEAM
                    else
                        self.target_mod = TARGET_MOD.SINGLE
                    end
                end
            end,

            [ BATTLE_EVENT.PRIMED_GAINED ] = function( self, battle )
                self.target_mod = TARGET_MOD.TEAM
            end,

            [ BATTLE_EVENT.PRIMED_LOST ] = function( self, battle )
                self.target_mod = TARGET_MOD.SINGLE
            end,

            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker.primed then
                        dmgt:AddDamage(self.bonus_damage, self.bonus_damage, self)
                    end
                end
            end
        },
    },

    gallery_plus =
    {
        name = "Twisted Gallery",
        desc = "<#UPGRADE>Hits all enemies.</>",
        desc_fn = function ( self, fmt_str ) return loc.format(fmt_str) end,

        target_mod = TARGET_MOD.TEAM,

        min_damage = 4,
        max_damage = 4,

        event_handlers = {},
        PreReq = function( self, battle ) end,
    },

    gallery_plus2 =
    {
        name = "Boosted Gallery",

        bonus_damage = 4,
        desc = "{PRIMED}: deals <#UPGRADE>{1}</> bonus damage and hits all enemies.",
    },

    gunsmoke = 
    {
        name = "Gun Smoke",
        anim = "reload_02",
        flavour = "'You just get more options with these cartridges, it's almost not fair.'",
        desc = "Spend 1 {CHARGE}: Gain 2 actions.",
        cost = 1,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity= CARD_RARITY.UNCOMMON,

        max_xp = 8,

        target_type = TARGET_TYPE.SELF,


        CanPlayCard = function( self, battle, target )
            if self.owner:GetCondition("lumin_tracker") == nil or self.owner:GetCondition("lumin_tracker"):GetCharges() == 0 then
                return false, CARD_PLAY_REASONS.NO_CHARGE
            end
            return true
        end,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(1, attack)
                battle:ModifyActionCount(2)
            end
        end
    },

    gunsmoke_plus =
    {
        name = "Overcharged Gun Smoke",
        desc = "Spend 1 {CHARGE}: Gain 2 actions <#UPGRADE>and gain 2 {SURGE}.</>",
        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                tracker:RemoveCharges(1, attack)
                battle:ModifyActionCount(2)
                self.owner:AddCondition("SURGE", 2, self)
            end
        end
    },

    gunsmoke_plus2 = 
    {
        name = "Pale Smoke",
        cost = 0,
    },

    shill_shot = 
    {

        name = "Shill Shot",
        anim = "reload_02",
        flavour = "'You can never be too prepared, I bought all the best gear before I arrived.'",
        desc = "Spend all {CHARGE}: Draw a card for each {CHARGE} spent.",
        cost = 1,

        flags = CARD_FLAGS.SKILL,

        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.SELF,

        max_xp = 8,
        bonus_draw = 0,
        bonus_defend = 0,

        CanPlayCard = function( self, battle, target )
            if self.owner:GetCondition("lumin_tracker") == nil or self.owner:GetCondition("lumin_tracker"):GetCharges() == 0 then
                return false, CARD_PLAY_REASONS.NO_CHARGE
            end
            return true
        end,

        OnPostResolve = function( self, battle, attack )

            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                local draw = tracker:RemoveCharges( tracker:GetCharges(), attack )
                battle:DrawCards( draw + self.bonus_draw )
                if self.bonus_defend > 0 then
                    self.owner:AddCondition("DEFEND", draw * self.bonus_defend, self)
                end
            end
        end,
    },

    shill_shot_plus =
    {
        name = "Boosted Shill Shot",
        desc = "Spend all {CHARGE}: Draw a card for each {CHARGE} spent <#UPGRADE>+ 1</>.",

        bonus_draw = 1,
    },

    shill_shot_plus2 = 
    {
        name = "Stone Shill Shot",
        desc = "Spend all {CHARGE}: Draw a card <#UPGRADE>and gain {1} {DEFEND}</> for each {CHARGE} spent.",
        desc_fn = function (self, fmt )
            return loc.format( fmt, self:CalculateDefendText( self.bonus_defend ))
        end,
        bonus_defend = 2,
    },

    pistol_whip =
    {
        name = "Pistol Whip",
        anim = "whip",
        flavour = "'I thought he was reloading...'",
        desc = "{EMPTY}: this costs 0.",
        cost = 1,
        max_xp = 7,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,

        min_damage = 4,
        max_damage = 4,

        PreReq = NoCharges,

        event_priorities =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetCharges() == 0 then
                        cost_acc:ModifyValue(0, self)
                    end
                end
            end
        },
    },

    pistol_whip_plus =
    {
        name = "Tall Pistol Whip",
        max_damage = 8,
    },

    pistol_whip_plus2 = 
    {
        name = "Rooted Pistol Whip",
        min_damage = 6,
        max_damage = 6,
    },

    take_cover =
    {
        name = "Take Cover",
        anim = "reload_02",
        flavour = "'This is some quality furniture! My compliments to the Barkeep.'",
        desc = "Apply {1} {DEFEND} per empty cell.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt))
        end,
        cost = 1,
        max_xp = 8,

        defend_amt = 2,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            local empty = tracker and tracker:GetEmptyCharges()
            if empty > 0 then
                for i=1,empty do
                    attack:AddCondition("DEFEND", self.defend_amt, self)
                end

                if self.riposte_bonus then
                    for i=1,empty do
                        self.owner:AddCondition("RIPOSTE", self.riposte_bonus, self )
                    end
                end
            end

            if self.bonus_charge and tracker then
                tracker:AddLuminCharges(self.bonus_charge, self)
            end
        end
    },

    take_cover_plus =
    {
        name = "Induction Cover",
        desc = "Apply {1} {DEFEND} per empty cell<#UPGRADE>, then gain 1 {CHARGE}.</>",
        bonus_charge = 1,
    },

    take_cover_plus2 =
    {
        name = "Spined Cover",
        desc = "Apply {1} {DEFEND} <#UPGRADE>and gain 1 {RIPOSTE}</> per empty cell.",
        riposte_bonus = 1,
    },

    double_time = 
    {
        name = "Double Time",
        pre_anim = "double_pre",
        anim = "double",
        flavour = "'When they are properly spun up, the power these babies deal out is tremendous.'",
        post_anim = "double_pst",
        desc = "{PRIMED}: Deals max damage.",

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 1,
        max_xp = 6,

        min_damage = 2,
        max_damage = 7,


        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            return tracker and tracker.primed
        end,

        event_priorities =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
            [ BATTLE_EVENT.CALC_DAMAGE ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker.primed then
                        dmgt:ModifyDamage(dmgt.max_damage, dmgt.max_damage, self)
                    end
                end
            end,
        },
    },

    double_time_plus =
    {
        name = "Pale Double Time",
        desc = "{PRIMED}: Deals max damage <#UPGRADE>and costs 0.</>",

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )

                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker.primed then
                        dmgt:ModifyDamage(dmgt.max_damage, dmgt.max_damage, self)
                    end
                end
            end,


            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker.primed then
                        cost_acc:ModifyValue(0)
                    end
                end
            end,
        },
    },

    double_time_plus2 =
    {
        name = "Boosted Double Time",
        max_damage = 10,
    },

    lucky_shot = 
    {
        name = "Lucky Shot",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'Right between the eyes! And they were on the move! I tell you, one in a million!'",
        post_anim = "blast_pst",
        desc = "Spend 1 {CHARGE}: deal 2 bonus damage per empty cell.",

        cost = 1,
        max_xp = 6,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 1,
        max_damage = 3,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then
                tracker:RemoveCharges(1, self)
            end
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )

                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    local empty = tracker and tracker:GetEmptyCharges() or 0
                    if empty > 0 and tracker:GetCharges() > 0 then
                        dmgt:AddDamage( empty * 2, empty * 2, self)
                    end
                end
            end,
        },
    },

    lucky_shot_plus = 
    {
        name = "Twisted Lucky Shot",
        desc = "Spend 1 {CHARGE}: deal 2 bonus damage per <#UPGRADE>{CHARGE}.</>",
        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )

                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetCharges() > 0 then
                        dmgt:AddDamage( tracker:GetCharges() * 2, tracker:GetCharges() * 2, self )
                    end
                end
            end,
        },
    },

    lucky_shot_plus2 =
    {
        name = "Boosted Lucky Shot",
        min_damage = 2,
        max_damage = 5,
    },

    gunslinger = 
    {
        name = "Gunslinger",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'It's that cool head of his that's the real danger.'",
        post_anim = "blast_pst",
        desc = "{CONCENTRATED}: Attack twice.",

        cost = 1,
        max_xp = 8,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        PreReq = function( self, battle )
            if self.owner:HasCondition("CONCENTRATION") then
                self.hit_count = 2
                return true
            else
                self.hit_count = 1
                return false
            end
        end,

        min_damage = 1,
        max_damage = 5,

        event_handlers =
        {
            [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                if card == self then
                    if self.owner:HasCondition("CONCENTRATION") then
                        self.hit_count = 2
                    else
                        self.hit_count = 1
                    end
                end
            end,

        },
    },

    gunslinger_plus =
    {
        name = "Focused Gunslinger",
        desc = "{CONCENTRATED}: Attack <#UPGRADE>three times.</>",
        PreReq = function( self, battle )
            if self.owner:HasCondition("CONCENTRATION") then
                self.hit_count = 3
                return true
            else
                self.hit_count = 1
                return false
            end
        end,
        event_handlers =
        {
            [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                if card == self then
                    if self.owner:HasCondition("CONCENTRATION") then
                        self.hit_count = 3
                    else
                        self.hit_count = 1
                    end
                end
            end,

        },

        max_damage = 4,
    },

    gunslinger_plus2 =
    {
        name = "Boosted Gunslinger",
        min_damage = 3,
        max_damage = 7,
    },

    telegraph = 
    {
        name = "Telegraph",
        anim = "psionic",
        flavour = "'How did he see that coming? Is he psychic or something?!.'",
        desc = "Gain {CONCENTRATION {1}}.\nApply {2} {DEFEND} per {CONCENTRATION} you have.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.concentration_amt, self.defend_amt)
        end,

        cost = 2,
        max_xp = 8,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        defend_amt = 2,
        concentration_amt = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("CONCENTRATION", self.concentration_amt, self)
            local count = self.owner:GetConditionStacks("CONCENTRATION")
            if count > 0 then
                attack:AddCondition("DEFEND", count * self.defend_amt, self)
            end
        end
    },

    telegraph_plus =
    {
        name = "Spined Telegraph",
        desc = "Gain {CONCENTRATION {1}}.\nApply 2 {DEFEND} <#UPGRADE>and gain 1 {RIPOSTE}</> per {CONCENTRATION}.",
        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("CONCENTRATION", 2, self)
            local count = self.owner:GetConditionStacks("CONCENTRATION")
            if count > 0 then
                attack:AddCondition("DEFEND", count * self.defend_amt, self)
                self.owner:AddCondition("RIPOSTE", count, self)
            end
        end
    },

    telegraph_plus2 =
    {
        name = "Stone Telegraph",
        desc = "Gain <#UPGRADE>{CONCENTRATION {1}}.</>\nApply 2 {DEFEND} per {CONCENTRATION}.",
        concentration_amt = 3,
    },

    blinders = 
    {
        name = "Blinders",
        anim = "taunt",
        flavour = "'It's an ironic name, I can see a lot better with these things.'",
        desc = "{ABILITY}: For each {CHARGE} you spend, apply 1 {MARK} to a random enemy.",
        on_play_screen_shake = true,

        cost = 2,
        max_xp = 6,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("blinders", 1, self)
        end,

        condition = 
        {
            name = "Blinders",
            desc = "Whenever you spend {CHARGE}, apply <#HILITE>{1} {MARK}</> to a random enemy.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks )
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.LUMIN_SPENT ] = function( self, battle, charges, max_cells )
                    local target_fighters = {}
                    self.battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )
                    if target_fighters[1] then
                        target_fighters[1]:AddCondition("MARK", self.stacks, self)
                    end
                end
            },
        },
    },

    blinders_plus =
    {
        name = "Pale Blinders",
        cost = 1,
    },

    blinders_plus2 =
    {
        name = "Initial Blinders",
        desc = "{ABILITY}: For each {CHARGE} you spend, apply 1 {MARK} to a random enemy.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    ripper =
    {
        name = "Ripper",
        anim = "psionic",
        flavour = "'I don't feel so good, boss...'",
        desc = "Remove a random debuff from target enemy: Gain 1 action.",

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.RARE,

        min_damage = 3,
        max_damage = 6,

        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                if hit.target:RemoveDebuff( self ) then
                    battle:ModifyActionCount(1)
                end
            end
        end
    },

    ripper_plus =
    {
        name = "Tall Ripper",
        max_damage = 9,
    },

    ripper_plus2 = 
    {
        name = "Rooted Ripper",
        min_damage = 6,
    },

    induction =
    {
        name = "Induction",
        anim = "psionic",
        flavour = "'It's hard to explain, but I can use the energy in a different way.'",
        desc = "{ABILITY}: Whenever you spend {SURGE}, draw a card.",

        cost = 2,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("induction", 1, self)
        end,

        condition =
        {
            desc = "Whenever you spend {SURGE}, draw a card.",
            event_handlers = 
            {
                [ BATTLE_EVENT.CONDITION_REMOVED ] = function( self, fighter, condition, stacks, source )
                    if self.owner == fighter and condition.id == "SURGE" and source and source.owner == self.owner and source.id ~= "SURGE" then
                        self.battle:DrawCards(self.stacks)
                    end
                end,
            },
        },
    },

    induction_plus =
    {
        name = "Pale Induction",
        cost = 1,
    },

    induction_plus2 =
    {
        name = "Initial Induction",
        desc = "{ABILITY}: Whenever you spend {SURGE}, draw a card.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    loader = 
    {
        name = "Loader",
        desc = "Spend all {CHARGE}, gain {1} {TEMP_POWER} for each {CHARGE} spent.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.power_gain)
        end,

        cost = 1,
        flavour = "'The newest model is twenty percent more efficient, and it also comes in blue!'",

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        power_gain = 2,
        bonus = 0,

        CanPlayCard = function( self, battle, target )
            if self.owner:GetCondition("lumin_tracker") == nil or self.owner:GetCondition("lumin_tracker"):GetCharges() == 0 then
                return false, CARD_PLAY_REASONS.NO_CHARGE
            end
            return true
        end,

        OnPostResolve = function( self, battle, attack )
            if self.draw then
                battle:DrawCards(self.draw)
            end
            local tracker = self.owner:GetCondition("lumin_tracker")
            local power = (tracker and tracker:GetCharges() or 0) * self.power_gain + self.bonus
            if power > 0 then
                self.owner:AddCondition("POWER", power, self)
                self.owner:AddCondition("POWER_LOSS", power, self)
                tracker:RemoveCharges( tracker:GetCharges() )
            end
        end,
    },

    loader_plus =
    {
        name = "Visionary Loader",
        desc = "<#UPGRADE>Draw a card.</> Spend all {CHARGE}, gain {1} {TEMP_POWER} for each {CHARGE} spent.",
        draw = 1,
    },

    loader_plus2 =
    {
        name = "Boosted Loader",
        desc = "Spend all {CHARGE}, gain {1} {TEMP_POWER} for each {CHARGE} spent <#UPGRADE>+ 2</>.",
        bonus = 2,
    },

    muddle = 
    {
        name = "Muddle",
        desc = "{ABILITY}: Whenever you apply a debuff to an enemy, gain 2 {DEFEND}.",
        on_play_screen_shake = true,

        cost = 2,
        flavour = "'I just can't seem to do anything right today...'",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("muddle", 2, self)
        end,

        condition = 
        {
            desc = "Whenever you apply a debuff to an enemy, gain <#HILITE>{1}</> {DEFEND}.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,
            event_handlers =
            {
                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if fighter:GetTeam() ~= self.owner:GetTeam() and source and source.owner == self.owner and condition.ctype == CTYPE.DEBUFF then
                        self.owner:AddCondition("DEFEND", self.stacks, self)
                    end
                end
            },
        },
    },

    muddle_plus =
    {
        name = "Initial Muddle",
        desc = "{ABILITY}: Whenever you apply a debuff to an enemy, gain 2 {DEFEND}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    muddle_plus2 =
    {
        name = "Pale Muddle",
        cost = 1,
    },

    offset = 
    {
        name = "Offset",
        desc = "{ABILITY}: All your attacks deal 2 bonus damage if the target has a debuff.",
        flavour = "'Be sure to hit even harder when they are off balance. You aren't here to fight fair.'",
        on_play_screen_shake = true,
        cost = 2,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("offset", 2, self)
        end,

        condition = 
        {
            desc = "All your attacks deal <#HILITE>{1}</> bonus damage if the target has a debuff.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and target then
                        for i, condition in pairs( target.conditions ) do
                            if condition.ctype == CTYPE.DEBUFF and not condition.hidden then
                                dmgt:AddDamage(self.stacks, self.stacks, self)
                                break
                            end
                        end
                    end
                end
            },
        },
    },

    offset_plus =
    {
        name = "Pale Offset",
        cost = 1,
    },

    offset_plus2 = 
    {
        name = "Initial Offset",
        desc = "{ABILITY}: All your attacks deal 2 bonus damage if the target has a debuff.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    weak_points =
    {
        name = "Weak Points",
        desc = "All attacks are {PIERCING} until the end of your turn.",
        flavour = "'Yeah, I'm familiar with that vest. It's why I'm not wearing one myself actually.'",

        cost = 1,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("weak_points", 1, self)
        end,

        condition = 
        {
            event_priorities =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = 0, -- First! (before METALLIC)
            },

            event_handlers = 
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition(self.id)
                end,

                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt, hit )
                    if hit and card.owner == self.owner and card:IsAttackCard() then
                        hit.piercing = true
                    end
                end,

                [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, card )
                    if card.owner == self.owner and card:IsAttackCard() then
                        card.flags = card.def.flags
                    end
                end,
            },
        },
    },

    weak_points_plus =
    {
        name = "Pale Weak Points",
        cost = 0,
    },

    weak_points_plus2 =
    {
        name = "Enduring Weak Points",
        flags = CARD_FLAGS.SKILL,
    },

    readied = 
    {
        name = "Readied",
        desc = "{ABILITY}: Whenever you fully mitigate an attack, gain 2 {CONCENTRATION}.",
        on_play_screen_shake = true,
        flavour = "'Glancing blows just don't shake them up enough, you need to do some real damage!'",
        cost = 2,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("readied", 2, self)
        end,

        condition = 
        {
            desc = "Whenever you fully mitigate an attack, gain <#HILITE>{1} {CONCENTRATION}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,
            event_handlers = 
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if hit.target == self.owner and (hit.defended or hit.evaded) then
                        self.owner:AddCondition( "CONCENTRATION", self.stacks, self )
                    end
                end,
            },
        },
    },

    readied_plus =
    {
        name = "Pale Readied",
        cost = 1,
    },

    readied_plus2 =
    {
        name = "Initial Readied",
        desc = "{ABILITY}: Whenever you fully mitigate an attack, gain 2 {CONCENTRATION}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    scatterphase =
    {
        name = "Scatterphase",
        anim = "reload_01",
        on_play_screen_shake = true,
        flavour = "'Look, I'm not technically supposed to have these, but I could tell when you walked in that you needed an edge.'",
        desc = "{ABILITY}: Whenever you gain {CHARGE}, deal {1} damage to a random enemy.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.scatter_stacks )
        end,

        cost = 2,
        max_xp = 6,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        target_type = TARGET_TYPE.SELF,

        scatter_stacks = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("scatterphase", self.scatter_stacks, self)
        end,

        condition = 
        {
            desc = "Whenever you gain {CHARGE}, deal <#HILITE>{1}</> damage to a random enemy.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks )
            end,

            shots_to_fire = 0,

            Trigger = function( self, battle, count )
                for i=1,count do
                    local target_fighters = {}
                    battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )
                    self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "blast")
                    battle:BroadcastEvent( BATTLE_EVENT.DELAY, .45)
                    if #target_fighters > 0 then
                        target_fighters[1]:ApplyDamage( self.stacks, self.owner, nil, nil, {"lumin"} )
                        target_fighters[1]:CheckForSurrender()
                    end
                end            
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.LUMIN_CHARGED ] = function( self, battle, charged, max_cells )

                    if self.battle.resolving then
                        self.shots_to_fire = self.shots_to_fire + 1
                    else
                        self:Trigger(self.battle, 1)
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if self.shots_to_fire > 0 then
                        self:Trigger(self.battle, self.shots_to_fire)
                        self.shots_to_fire = 0
                    end
                end
            },
        },
    },

    scatterphase_plus =
    {
        name = "Pale Scatterphase",
        cost = 1,
    },

    scatterphase_plus2 =
    {
        name = "Initial Scatterphase",
        desc = "{ABILITY}: Whenever you gain {CHARGE}, deal {1} damage to a random enemy.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    tank =
    {
        name = "Tank",
        anim = "taunt",
        flavour = "'They are going to wish they never took a swing at me.'",
        desc = "{ABILITY}: Whenever you gain {CHARGE}, gain 2 {RIPOSTE}.",
        on_play_screen_shake = true,

        cost = 2,
        max_xp = 6,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("tank", 2, self)
        end,

        condition = 
        {
            desc = "Whenever you gain {CHARGE}, gain <#HILITE>{1} {RIPOSTE}</>.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks )
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.LUMIN_CHARGED ] = function( self, battle, charged, max_cells )
                    self.owner:AddCondition("RIPOSTE", self.stacks, self)
                end,
            },
        },
    },

    tank_plus =
    {
        name = "Pale Tank",
        cost = 1,
    },

    tank_plus2 =
    {
        name = "Initial Tank",
        desc = "{ABILITY}: Whenever you gain {CHARGE}, gain 2 {RIPOSTE}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    backfire = 
    {
        name = "Backfire",
        anim = "taunt",
        flavour = "'Now you are just showing off.'",
        desc = "{ABILITY}: At the end of your turn, gain 1 {RIPOSTE} per {CHARGE}.",
        on_play_screen_shake = true,

        cost = 2,
        max_xp = 6,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("backfire", 1, self)
        end,

        condition =
        {
            desc = "At the end of your turn, gain <#HILITE>{1}</> {RIPOSTE} per {CHARGE}.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks )
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    local count = tracker and tracker:GetCharges() or 0
                    if count > 0 then
                        self.owner:AddCondition("RIPOSTE", count * self.stacks, self)
                    end
                end
            },
        },
    },

    backfire_plus =
    {
        name = "Pale Backfire",
        cost = 1,
    },

    backfire_plus2 =
    {
        name = "Initial Backfire",
        desc = "{ABILITY}: At the end of your turn, gain 1 {RIPOSTE} per {CHARGE}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    fistful = 
    {
        name = "Fistful",
        anim = "reload_01",
        flavour = "'I don't care what you grab! Just get me some ammunition!'",
        desc = "Draw 2 cards, gain 2 {CHARGE} for each attack card drawn.",

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.SELF,

        card_draw = 2,

        OnPostResolve = function( self, battle, attack )
            local cards = battle:DrawCards(self.card_draw)
            local count = 0
            for i,card in ipairs(cards) do
                if card:IsAttackCard() then
                    count = count + 1
                end
            end
            if count > 0 then
                local tracker = self.owner:GetCondition("lumin_tracker")
                if tracker then tracker:AddLuminCharges(count * 2, self) end
            end
        end
    },

    fistful_plus =
    {
        name = "Visionary Fistful",
        desc = "Draw <#UPGRADE>3</> cards, gain 2 {CHARGE} for each attack card drawn.",
        card_draw = 3,
    },

    fistful_plus2 = 
    {
        name = "Boosted Fistful",
        desc = "Draw 2 cards, gain 2 {CHARGE} for each attack card drawn <#UPGRADE>+ 1</>.",
        OnPostResolve = function( self, battle, attack )
            local cards = battle:DrawCards(self.card_draw)
            local count = 0
            for i,card in ipairs(cards) do
                if card:IsAttackCard() then
                    count = count + 1
                end
            end
            if count > 0 then
                local tracker = self.owner:GetCondition("lumin_tracker")
                if tracker then tracker:AddLuminCharges((count * 2) + 1, self) end
            end
        end
    },

    fully_loaded = 
    {
        name = "Fully Loaded",
        anim = "reload_01",
        flavour = "'We can be shooting until next week with this many rounds.'",
        desc = "Apply {1} {DEFEND}.\n{PRIMED}: Gain {2} {RIPOSTE}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.defend_amt, self.counter_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.COMMON,
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            return tracker and tracker.primed
        end,

        defend_amt = 4,
        counter_amt = 4,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker.primed then
                self.owner:AddCondition("RIPOSTE", self.counter_amt, self)
            end
            attack:AddCondition("DEFEND", self.defend_amt, self)
        end,
    },

    fully_loaded_plus =
    {
        name = "Stone Fully Loaded",
        desc = "Apply <#UPGRADE>{1}</> {DEFEND}.\n{PRIMED}: Gain {2} {RIPOSTE}.",
        defend_amt = 6,
    },

    fully_loaded_plus2 =
    {
        name = "Spined Fully Loaded",
        desc = "Apply {1} {DEFEND}.\n{PRIMED}: Gain <#UPGRADE>{2}</> {RIPOSTE}.",
        counter_amt = 6,
    },

    dynamo_round = 
    {
        name = "Dynamo Round",
        pre_anim = "dynamo_pre",
        anim = "dynamo",
        flavour = "'Hard on the barrel, but much harder on the target.'",
        post_anim = "dynamo_pst",

        desc = "Deal 2 bonus damage for every {SURGE}. Consumes all {SURGE}.",
        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.COMMON,

        cost = 1,

        min_damage = 3,
        max_damage = 3,

        OnPostResolve = function( self, battle, attack )
            if self.owner:HasCondition("SURGE") then
                self.owner:RemoveCondition("SURGE", self.owner:GetConditionStacks("SURGE"), self)
            end
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self and self.owner:HasCondition("SURGE") then
                    local stacks = self.owner:GetConditionStacks("SURGE")
                    dmgt:AddDamage(stacks * 2, stacks * 2, self)
                end
            end
        },
    },

    dynamo_round_plus =
    {
        name = "Efficient Dynamo Round",
        desc = "Deal 2 bonus damage for every {SURGE}. Consumes <#UPGRADE>half your {SURGE}</>.",
        OnPostResolve = function( self, battle, attack )
            if self.owner:HasCondition("SURGE") then
                self.owner:RemoveCondition("SURGE", math.ceil(self.owner:GetConditionStacks("SURGE") * 0.5), self)
            end
        end,
    },

    dynamo_round_plus2 =
    {
        name = "Boosted Dynamo Round",
        min_damage = 5, 
        max_damage = 5,
    },

    overheater =
    {
        name = "Overheater",
        pre_anim = "surge_pre",
        anim = "surge",
        flavour = "'It's a lot like drag racing your weaponry, right off the start you are dealing with power.'",
        post_anim = "surge_pst",
        desc = "Double your {SURGE}.",
        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 2,

        min_damage = 4,
        max_damage = 8,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("SURGE", self.owner:GetConditionStacks( "SURGE" ), self)
        end,
    },

    overheater_plus =
    {
        name = "Boosted Overheater",
        min_damage = 6,
        max_damage = 10,
    },

    overheater_plus2 =
    {
        name = "Loaded Overheater",
        desc = "Double your {SURGE}.",
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
    },

    searing_bullet =
    {
        name = "Searing Bullet",
        pre_anim = "dynamo_pre",
        anim = "dynamo",
        flavour = "'Hey I don't make the rules, you aren't supposed to have incendiary rounds in those things.'",
        post_anim = "dynamo_pst",
        desc = "Spend up to 3 {SURGE}: Apply 1 {WOUND} per {SURGE} spent.",
        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.COMMON,

        cost = 1,

        min_damage = 2,
        max_damage = 4,

        PreReq = function( self, battle )
            return self.owner:HasCondition("SURGE")
        end,

        OnPostResolve = function( self, battle, attack )
            local stacks = math.min(self.owner:GetConditionStacks("SURGE"), 3)
            attack:AddCondition("WOUND", stacks, self)
            self.owner:RemoveCondition("SURGE", stacks, self)
        end
    },

    searing_bullet_plus =
    {
        name = "Promoted Searing Bullet",
        desc = "Spend up to <#UPGRADE>5</> {SURGE}: Apply 1 {WOUND} per {SURGE} spent.",
        OnPostResolve = function( self, battle, attack )
            local stacks = math.min(self.owner:GetConditionStacks("SURGE"), 5)
            attack:AddCondition("WOUND", stacks, self)
            self.owner:RemoveCondition("SURGE", stacks, self)
        end
    },

    searing_bullet_plus2 =
    {
        name = "Boosted Searing Bullet",
        min_damage = 4,
        max_damage = 6,
    },

    charged_barrage =
    {
        name = "Charged Barrage",
        pre_anim = "surge_pre",
        flavour = "'Cools down the guns while consuming the excess gas build up. I don't know, it's science or something.'",
        anim = "surge",
        post_anim = "surge_pst",
        desc = "Spend up to {1} {SURGE}: Attack once extra per {SURGE}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.max_surge)
        end,
        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,

        max_xp = 6,

        min_damage = 2,
        max_damage = 3,

        max_surge = 3,

        PreReq = function( self, battle )
            return self.owner:HasCondition("SURGE")
        end,

        RefreshHitCount = function( self )
            local stacks = math.min(self.owner:GetConditionStacks("SURGE"), self.max_surge) 
            self.hit_count = stacks + 1
            self.surge_used = stacks
        end,

        OnPostResolve = function( self, battle, attack )
            if self.surge_used then
                self.owner:RemoveCondition("SURGE", self.surge_used, self)
                self.surge_used = nil
            end
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                if card == self then
                    self:RefreshHitCount()
                end
            end,

            [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                if fighter == self.owner and condition.id == "SURGE" then
                    self:RefreshHitCount()
                end
            end,

            [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if card == self then
                    self:RefreshHitCount()
                end
            end,
        },
    },

    charged_barrage_plus =
    {
        name = "Rooted Barrage",
        min_damage = 4,
        max_damage = 4,
    },

    charged_barrage_plus2 =
    {
        name = "Boosted Barrage",
        desc = "Spend up to <#UPGRADE>{1}</> {SURGE}: Attack once extra per {SURGE}.",
        max_surge = 4,
    },

    clear_shot = 
    {
        name = "Clear Shot",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'They are out in the open just begging to be hit.'",
        post_anim = "blast_pst",
        desc = "Gain 1 minimum damage per empty cell.",
        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.COMMON,

        cost = 1,

        min_damage = 1,
        max_damage = 6,


        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetEmptyCharges() > 0 then
                        dmgt:AddDamage( tracker:GetEmptyCharges(), 0, self )
                    end
                end
            end
        },
    },

    clear_shot_plus = 
    {
        name = "Tall Clear Shot",
        max_damage = 8,
    },

    clear_shot_plus2 =
    {
        name = "Boosted Clear Shot",
        min_damage = 3,
    },

    wounding_shot =
    {
        name = "Wounding Shot",
        pre_anim = "blast_pre",
        flavour = "'Take them apart one piece at a time.'",
        anim = "blast",
        post_anim = "blast_pst",
        desc = "Apply 1 {WOUND} per empty cell.",
        desc_fn = function ( self, fmt_str )
            return loc.format(fmt_str, self.lumin_charge or 1)
        end,

        cost = 2,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 4,
        max_damage = 5,

        wound_boost = 0,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetEmptyCharges() > 0 then
                attack:AddCondition("WOUND", tracker:GetEmptyCharges() + self.wound_boost, self)
            end
        end
    },

    wounding_shot_plus =
    {
        name = "Boosted Wounding Shot",
        min_damage = 6,
        max_damage = 8,
    },

    wounding_shot_plus2 = 
    {
        name = "Enhanced Wounding Shot",
        desc = "Apply 1 {WOUND} per empty cell<#UPGRADE> + 1</>.",
        wound_boost = 1,
    },

    cheap_shot =
    {
        name = "Cheap Shot",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'Ouch...'",
        post_anim = "blast_pst",
        desc = "Costs 1 less per empty cell.",
        desc_fn = function ( self, fmt_str )
            return loc.format(fmt_str, self.lumin_charge or 1)
        end,

        cost = 5,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.COMMON,

        min_damage = 4,
        max_damage = 7,
        lumin_charge = 1,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetEmptyCharges() > 0 then
                        cost_acc:AddValue(-tracker:GetEmptyCharges(), self)
                    end
                end
            end,
        },
    },

    cheap_shot_plus = 
    {
        name = "Very Cheap Shot",
        cost = 4,
    },

    cheap_shot_plus2 =
    {
        name = "Boosted Cheap Shot",
        min_damage = 5,
        max_damage = 10,
    },

    suppressing_fire =
    {
        name = "Suppressing Fire",
        anim = "suppressing",
        flavour = "'Keep them pinned down! I'll loop around and flank them!'",
        desc = "Target all enemies. Gain {1} {DEFEND} per target.",
        desc_fn = function ( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amt ))
        end,
        
        cost = 2,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,
        target_mod = TARGET_MOD.TEAM,

        min_damage = 2,
        max_damage = 6,
        defend_amt = 2,

        OnPostResolve = function( self, battle, attack )
            -- NOTE: This is done this way so that DEFEND bonuses apply PER target, like the wording would suggest.
            for i = 1, attack:CountUniqueTargets() do
                self.owner:AddCondition( "DEFEND", self.defend_amt, self )
            end
        end
    },

    suppressing_fire_plus =
    {
        name = "Promoted Suppressing Fire",
        desc = "Target all enemies. Gain <#UPGRADE>{1} {DEFEND}</> per target.",
        defend_amt = 3,
    },

    suppressing_fire_plus2 =
    {
        name = "Boosted Suppressing Fire",
        min_damage = 4,
        max_damage = 8,
    },

    lifeline = 
    {
        name = "Lifeline",
        anim = "reload_01",
        flavour = "'The implants work really well with this tech, lumin really is some kind of miracle.'",
        desc = "Draw {1} cards.\n{EMPTY}: Gain {2} {CHARGE} and this costs 0.",
        desc_fn = function ( self, fmt_str )
            return loc.format(fmt_str, self.card_draw or 2, self.lumin_charge or 1)
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.SELF,

        card_draw = 2,
        lumin_charge = 1,

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(self.card_draw)
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() == 0 then
                tracker:AddLuminCharges(self.lumin_charge, self)
            end
        end,

        PreReq = NoCharges,

        event_priorities =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetCharges() == 0 then
                        cost_acc:ModifyValue(0, self)
                    end
                end
            end,
        }
    },

    lifeline_plus =
    {
        name = "Visionary Lifeline",
        desc = "Draw <#UPGRADE>{1}</> cards.\n{EMPTY}: Gain {2} {CHARGE} and this costs 0.",
        card_draw = 3,
    },

    lifeline_plus2 =
    {
        name = "Inductive Lifeline",
        desc = "Draw {1} cards.\n{EMPTY}: Gain <#UPGRADE>{2}</> {CHARGE} and this costs 0.",
        lumin_charge = 2,
    },

    sucker_punch =
    {
        name = "Sucker Punch",
        anim = "punch",
        flavour = "'Name's Rook, nice to meet you.'",
        desc = "{EMPTY}: Deal 2 bonus damage and apply {1} {IMPAIR}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.impair_amt or 1)
        end,
        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,

        cost = 1,

        min_damage = 5,
        max_damage = 5,

        impair_amt = 1,

        PreReq = NoCharges,

        OnPreResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker") 
            if tracker and tracker:GetCharges() == 0 then
                attack:AddCondition("IMPAIR", self.impair_amt, self)
            end
        end,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                local tracker = self.owner:GetCondition("lumin_tracker")
                if tracker and tracker:GetCharges() == 0 then
                    dmgt:AddDamage( 2, 2, self )
                end
            end
        }
    },

    sucker_punch_plus =
    {
        name = "Boosted Sucker Punch",
        min_damage = 7,
        max_damage = 7,
    },

    sucker_punch_plus2 =
    {
        name = "Stifling Sucker Punch",
        desc = "{EMPTY}: Deal 2 bonus damage and apply <#UPGRADE>{1}</> {IMPAIR}.",
        impair_amt = 2,
    },

    crackle = 
    {
        name = "Crackle",
        anim = "crackle",
        flavour = "'There's a reason you can't use these, they are unstable as all Hesh.'",
        desc = "If the target has {BURN}, gain {1} {CHARGE}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.lumin_charge or 1)
        end,

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,

        min_damage = 4,
        max_damage = 4,

        lumin_charge = 2,

        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                if hit.target:HasCondition("BURN") then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker then tracker:AddLuminCharges(self.lumin_charge, self) end
                end
            end
        end
    },

    crackle_plus =
    {
        name = "Inductive Crackle",
        desc = "If the target has {BURN}, gain <#UPGRADE>{1}</> {CHARGE}.",
        lumin_charge = 3,
    },

    crackle_plus2 =
    {
        name = "Boosted Crackle",
        min_damage = 6,
        max_damage = 6,
    },

    accelerant = 
    {
        name = "Accelerant",
        anim = "throw",
        flavour = "'What's that smell?'",
        anims = { "anim/weapon_grenade_accelerant.zip"},
        hit_tags = { "spark" },
        desc = "Apply {1} {SCORCHED} and {2} {BURN} to an enemy.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.scorched_amt or 1, self.burn_amt)
        end,

        cost = 1,
        scorched_amt = 2,
        burn_amt = 2,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.COMMON,
       

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("SCORCHED", self.scorched_amt, self)
            attack:AddCondition("BURN", self.burn_amt, self)
        end
    },

    accelerant_plus =
    {
        name = "Boosted Accelerant",
        desc = "Apply <#UPGRADE>{SCORCHED {1}}</> and <#UPGRADE>{BURN {2}}</>  to an enemy.",
        scorched_amt = 3,
        burn_amt = 3,
    },

    accelerant_plus2 = 
    {
        name = "Wide Accelerant",
        desc = "Apply {1} {SCORCHED} and {2} {BURN} to <#UPGRADE>all enemies.</>",
        target_mod = TARGET_MOD.TEAM,
    },

    firestorm =
    {
        name = "Firestorm",
        anim = "crackle",
        flavour = "'There's no way we are insured for this.'",
        desc = "Spend all {CHARGE}, apply 2 {BURN} per {CHARGE} spent.",

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 3,
        max_damage = 4,
        burn_bonus = 2,

        OnPostResolve = function( self, battke, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                attack:AddCondition("BURN", tracker:GetCharges() * self.burn_bonus, self)
                tracker:RemoveCharges(tracker:GetCharges(), self)
            end
        end
    },

    firestorm_plus =
    {
        name = "Boosted Firestorm",
        desc = "Spend all {CHARGE}, apply <#UPGRADE>3</> {BURN} per {CHARGE} spent.",
        burn_bonus = 3,
    },

    firestorm_plus2 =
    {
        name = "Wide Firestorm",
        desc = "Spend all {CHARGE}, apply 2 {BURN} per {CHARGE} spent.\n<#UPGRADE>Hits all enemies.</>",
        target_mod = TARGET_MOD.TEAM,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,
    },

    cauterize = 
    {
        name = "Cauterize",
        anim = "crackle",
        flavour = "'So it's just regular spark rounds with the safety sheared off? I didn't know that it was that easy.'",
        desc = "Spend 1 {CHARGE}: Apply {1} {SCORCHED}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.scorched_amt or 1)
        end,

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 2,
        max_damage = 5,

        scorched_amt = 2,

        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            return tracker and tracker:GetCharges() > 0
        end,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() > 0 then
                attack:AddCondition("SCORCHED", self.scorched_amt, self)
                tracker:RemoveCharges(1, self)
            end
        end,
    },

    cauterize_plus =
    {
        name = "Rooted Cauterize",
        min_damage = 5,
        max_damage = 5,
    },

    cauterize_plus2 =
    {
        name = "Boosted Cauterize",
        desc = "Spend 1 {CHARGE}: Apply <#UPGRADE>{1}</> {SCORCHED}.",
        scorched_amt = 3,
    },

    char =
    {
        name = "Char",
        anim = "crackle",
        flavour = "'Everyone has a plan until they are set on fire.'",
        desc = "If the target has {BURN}, gain {1} action.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.action_gain)
        end,

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,

        min_damage = 2,
        max_damage = 3,
        action_gain = 1,

        event_handlers =
        {
            [ BATTLE_EVENT.PRE_RESOLVE ] = function( self, battle, attack )
                if attack.card and attack.card == self then
                    for i, hit in attack:Hits() do
                        if hit.target:HasCondition("BURN") then
                            battle:ModifyActionCount(self.action_gain)
                        end
                    end
                end
            end,
        },
    },

    char_plus =
    {
        name = "Visionary Char",
        desc = "If the target has {BURN}, gain <#UPGRADE>{1}</> actions.",
        action_gain = 2,
    },

    char_plus2 =
    {
        name = "Boosted Char",
        min_damage = 4,
        max_damage = 5,
    },

    sear = 
    {
        name = "Sear",
        anim = "crackle",
        flavour = "'Makes them a lot easier to hit in the dark, I'll tell you that much.'",
        desc = "Apply {BURN} equal to damage dealt by this card.",

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,

        min_damage = 1,
        max_damage = 4,
        burn_bonus = 0,

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if hit.card == self and not hit.evaded then
                    hit.target:AddCondition("BURN", self.burn_bonus + (hit.damage or 0), self)
                end
            end
        },
    },

    sear_plus = 
    {
        name = "Tall Sear",
        max_damage = 7,
    },

    sear_plus2 = 
    {
        name = "Rooted Sear",
        min_damage = 4,
        max_damage = 4,
    },

    clean_open =
    {
        name = "Clean Open",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'Shoot first.'",
        post_anim = "blast_pst",
        desc = "{EMPTY}: Attack twice.\nGain {1} {CHARGE}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.lumin_charge)
        end,

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity= CARD_RARITY.UNCOMMON,

        min_damage = 2,
        max_damage = 5,

        lumin_charge = 1,

        PreReq = function( self, battle )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() == 0 then
                self.hit_count = 2
                return true
            else
                self.hit_count = 1
                return false
            end
        end,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then tracker:AddLuminCharges(self.lumin_charge, self) end
        end,
    },

    clean_open_plus =
    {
        name = "Rooted Clean Open",
        min_damage = 5,
        max_damage = 5,
    },

    clean_open_plus2 =
    {
        name = "Tall Clean Open",
        min_damage = 2,
        max_damage = 8,
    },

    ego_rip =
    {
        name = "Ego Rip",
        anim = "whip",
        flavour = "'I've never heard that kind of taste before.'",
        desc = "Deals +{1} damage for every debuff on the target.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,

        min_damage = 2,
        max_damage = 2,

        bonus_damage = 2,

        max_xp = 8,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self and target then
                    local count = 0
                    for i,condition in pairs(target:GetConditions()) do
                        if condition.ctype == CTYPE.DEBUFF then
                            count = count + 1
                        end
                    end
                    dmgt:AddDamage(count * self.bonus_damage, count * self.bonus_damage, self)
                end
            end
        },
    },

    ego_rip_plus =
    {
        name = "Boosted Ego Rip",
        min_damage = 4,
        max_damage = 4,
    },

    ego_rip_plus2 =
    {
        name = "Enhanced Ego Rip",
        desc = "Deals <#UPGRADE>+{1}</> damage for every debuff on the target.",
        bonus_damage = 3,
    },

    scanner =
    {
        name = "Scanner",
        anim = "taunt",
        flavour = "'Old targeting tech from the Aerostat, I've made a few adjustments.'",
        desc = "Apply {SCANNED {1}} to a random enemy.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.scanned_amt )
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        target_mod = TARGET_MOD.RANDOM1,
        scanned_amt = 2,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("SCANNED", self.scanned_amt, self)
        end
    },

    scanner_plus =
    {
        name = "Boosted Scanner",
        desc = "Apply <#UPGRADE>{SCANNED {1}}</> to a random enemy.",
        scanned_amt = 3,
    },

    scanner_plus2 =
    {
        name = "Precise Scanner",
        desc = "Apply {SCANNED {1}} to <#UPGRADE>an enemy.</>",
        target_mod = TARGET_MOD.SINGLE,
    },

    cyclone =
    {
        name = "Cyclone",
        anim = "whip",
        flavour = "'What's happening? I can't even think straight.'",
        desc = "Remove all debuffs on a target and deal {1} bonus damage per debuff.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,

        OnPostResolve = function( self, battle, attack )
            for i,hit in attack:Hits() do
                hit.target:RemoveAllDebuffs( self )
            end
        end,

        min_damage = 1,
        max_damage = 1,

        bonus_damage = 4,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self and target then
                    local count = 0
                    for i,condition in pairs(target:GetConditions()) do
                        if condition.ctype == CTYPE.DEBUFF then
                            count = count + 1
                        end
                    end
                    dmgt:AddDamage(count * self.bonus_damage, count * self.bonus_damage, self)
                end
            end
        },
    },

    cyclone_plus = 
    {
        name = "Boosted Cyclone",
        min_damage = 4,
        max_damage = 4,
    },

    cyclone_plus2 =
    {
        name = "Enhanced Cyclone",
        desc = "Remove all debuffs on a target and deal <#UPGRADE>{1}</> bonus damage per debuff.",
        bonus_damage = 5,
    },

    brain_tick =
    {
        name = "Brain Tick",
        anim = "psionic",
        flavour = "'As ugly as they are effective.'",
        desc = "Remove all debuffs on a target and {HEAL} {1} per debuff.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.heal_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.ANY,

       heal_amt = 2,

        OnPostResolve = function( self, battle, attack )
            local count = 0
            for i, hit in attack:Hits() do
                for i,condition in pairs(hit.target:GetConditions()) do
                    if condition.ctype == CTYPE.DEBUFF then
                        count = count + 1
                    end
                end
                hit.target:RemoveAllDebuffs( self )
            end
            if count > 0 then
                self.owner:HealHealth(count * self.heal_amt, self)
            end

        end,
    },

    brain_tick_plus =
    {
        name = "Enhanced Brain Tick",
        desc = "Remove all debuffs on a target and {HEAL} <#UPGRADE>{1}</> per debuff.",
        heal_amt = 3,
    },

    brain_tick_plus2 =
    {
        name = "Boosted Brain Tick",
        min_damage = 3,
        max_damage = 8,
    },

    pulse = 
    {
        name = "Pulse",
        flavour = "'How many settings do these Heshing things have?!'",
        pre_anim = "blast_pre",
        anim = "blast",
        post_anim = "blast_pst",
        desc = "Apply {1} {RICOCHET}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.ricochet_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.COMMON,

        min_damage = 2,
        max_damage = 2,

        ricochet_amt = 2,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("RICOCHET", self.ricochet_amt, self)
        end,
    },

    pulse_plus =
    {
        name = "Boosted Pulse",
        min_damage = 4,
        max_damage = 4,
    },

    pulse_plus2 = 
    {
        name = "Enhanced Pulse",
        desc = "Apply <#UPGRADE>{1}</> {RICOCHET}.",
        ricochet_amt = 3,
    },

    resonator =
    {
        name = "Oscillator",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'The initial blast causes a reflective energy coating that makes it so consecutive... are you even listening?'",
        post_anim = "blast_pst",
        desc = "Apply {RICOCHET} equal to damage dealt by this card.",

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 2,
        max_damage = 5,

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if hit.card == self then
                    hit.target:AddCondition("RICOCHET", hit.damage or 0, self)
                end
            end
        },
    },

    resonator_plus =
    {
        name = "Tall Oscillator",
        max_damage = 7,
    },

    resonator_plus2 =
    {
        name = "Rooted Oscillator",
        min_damage = 4,
        max_damage = 4,
    },

    tremor =
    {
        name = "Tremor",
        anim = "mark",
        flavour = "'After this, be sure to toss in that grenade, it's going to be beautiful.'",
        desc = "Apply {1} {RICOCHET} to all enemies.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.ricochet_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        target_mod = TARGET_MOD.TEAM,

        ricochet_amt = 1,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("RICOCHET", self.ricochet_amt, self)
        end,
    },

    tremor_plus =
    {
        name = "Boosted Tremor",
        desc = "Apply <#UPGRADE>{1}</> {RICOCHET} to all enemies.",
        ricochet_amt = 2,
    },

    tremor_plus2 =
    {
        name = "Inductive Tremor",
        desc = "Apply {1} {RICOCHET} to all enemies.\n<#UPGRADE>Gain 1 {CHARGE}.</>",
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("RICOCHET", self.ricochet_amt, self)

            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then tracker:AddLuminCharges(1, self) end
        end,
    },

    jolt =
    {
        name = "Jolt",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'I tell you, surging your pistols lights up every little machine in your whole Heshing body!'",
        post_anim = "blast_pst",
        desc = "This card costs 1 less per {SURGE}.",

        cost = 4,

        flags = CARD_FLAGS.RANGED,
        rarity= CARD_RARITY.COMMON,

        min_damage = 4,
        max_damage = 6,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self then
                    local count = self.owner:GetConditionStacks("SURGE")
                    if count > 0 then
                        cost_acc:AddValue( -count, self)
                    end
                end
            end,
        },
    },

    jolt_plus =
    {
        name = "Boosted Jolt",
        min_damage = 5,
        max_damage = 8,
    },

    jolt_plus2 = 
    {
        name = "Pale Jolt",
        cost = 3,
    },

    shock_therapy =
    {
        name = "Shock Therapy",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'It's about momentum, you can feel the weapons hum when they are cycling properly.'",
        post_anim = "blast_pst",
        desc = "Deal {1} bonus damage per {SURGE}. Gain {2} {SURGE}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage, self.surge_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.RARE,

        min_damage = 2,
        max_damage = 3,

        surge_amt = 3,
        bonus_damage = 1,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("SURGE", self.surge_amt, self)
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local count = self.owner:GetConditionStacks("SURGE")
                    if count then
                        dmgt:AddDamage(count * self.bonus_damage, count * self.bonus_damage, self)
                    end
                end
            end,
        },
    },

    shock_therapy_plus =
    {
        name = "Boosted Shock Therapy",
        min_damage = 4,
        max_damage = 5,
    },

    shock_therapy_plus2 =
    {
        name = "Enhanced Shock Therapy",
        desc = "Deal {1} bonus damage per {SURGE}. Gain <#UPGRADE>{2}</> {SURGE}.",
        surge_amt = 5,
    },

    striker =
    {
        name = "Striker",
        anim = "whip",
        flavour = "'So the timing is a little tricky, but yeah, just smash the bottom priming cap and you should be able to get some extra juice.'",
        desc = "{THRESHOLD} {1}: Gain {2} {SURGE}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateThresholdText(self), self.surge_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,

        min_damage = 2,
        max_damage = 5,
        threshold = 4,
        surge_amt = 2,

        ThresholdEffect = function( self, battle, attack )
            self.owner:AddCondition("SURGE", self.surge_amt, self)
        end,
    },

    striker_plus =
    {
        name = "Tall Striker",
        max_damage = 7,
    },

    striker_plus2 =
    {
        name = "Boosted Striker",
        desc = "{THRESHOLD} {1}: Gain <#UPGRADE>{2}</> {SURGE}.",
        surge_amt = 4,
    },

    trick_shot =
    {
        name = "Trick Shot",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'I told you to hold still!'",
        post_anim = "blast_pst",
        desc = "{THRESHOLD} {1}: Gain {2} {RIPOSTE}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateThresholdText(self), self.counter_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 1,
        max_damage = 8,

        threshold = 8,
        counter_amt = 8,

        ThresholdEffect = function( self, battle, attack )
            self.owner:AddCondition("RIPOSTE", self.counter_amt, self)
        end
    },

    trick_shot_plus =
    {
        name = "Boosted Trick Shot",
        desc = "{THRESHOLD} {1}: Gain <#UPGRADE>{2}</> {RIPOSTE}.",
        counter_amt = 12,
    },

    trick_shot_plus2 =
    {
        name = "Enhanced Trick Shot",
        desc = "{THRESHOLD} <#UPGRADE>{1}</>: Gain {2} {RIPOSTE}.",
        threshold = 6,
    },

    bolster_shot =
    {
        name = "Bolster Shot",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'These are going to leave some wicked holes my friend.'",
        post_anim = "blast_pst",
        desc = "Deal bonus damage equal to the cost of the two most expensive cards in your hand.",

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 3,
        max_damage = 5,

        card_number = 2,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local options = {}
                    for i,hand_card in self.engine:GetHandDeck():Cards() do
                        if hand_card ~= self then
                            table.insert(options, hand_card)
                        end
                    end
                    table.sort(options, function(a,b) return self.engine:CalculateActionCost(a) < self.engine:CalculateActionCost(b) end)
                    local cost_added = 0
                    if #options > 0 then
                        for i=1, math.min(self.card_number, #options) do
                            cost_added = cost_added + self.engine:CalculateActionCost(options[#options - i + 1])
                        end
                    end
                    dmgt:AddDamage(cost_added, cost_added, self)
                end
            end
        },
    },

    bolster_shot_plus = 
    {
        name = "Rooted Bolster Shot",
        min_damage = 5,
        max_damage = 5,
    },

    bolster_shot_plus2 =
    {
        name = "Tall Bolster Shot",
        max_damage = 8,
    },

    boat_anchor =
    {
        name = "Boat Anchor",
        anim = "taunt",
        flavour = "'Probably a little too heavy to do what you want.'",
        desc = "Apply {1} {DEFEND}.\nThis card costs 1 more per other card in your hand.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.defend_amt )
        end,

        cost = 1,
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,
        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        defend_amt = 8,

        OnPostResolve = function( self, battle, attack )
            attack.target:AddCondition("DEFEND", self.defend_amt, self)
        end,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.IN_HAND, DECK_TYPE.DISCARDS },

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self then
                    cost_acc:AddValue(self.engine:GetHandDeck():CountCards() - 1)
                end
            end
        },
    },

    boat_anchor_plus =
    {
        name = "Stone Anchor",
        desc = "Apply <#UPGRADE>{1}</> {DEFEND}.\nThis card costs 1 more per other card in your hand.",
        defend_amt = 12,
    },

    boat_anchor_plus2 =
    {
        name = "Heavy Anchor",
        cost = 5,
    },

    wildfire =
    {
        name = "Wildfire",
        anim = "taunt",
        flavour = "'You idiot! We needed that!'",
        desc = "Apply {1} {BURN} to target enemy, then apply {BURN} to all other enemies equal to {BURN} on the target.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.burn_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.RARE,

        burn_amt = 1,

        OnPostResolve = function( self, battle, attack )
            local count = 0
            for i, hit in attack:Hits() do
                hit.target:AddCondition("BURN", self.burn_amt, self)
                count = count + hit.target:GetConditionStacks("BURN")
            end

            if count > 0 then
                for i,fighter in ipairs(self.owner:GetEnemyTeam().fighters) do
                    if not attack:IsTarget( fighter ) then
                        fighter:AddCondition("BURN", count, self)
                    end
                end
            end
        end
    },

    wildfire_plus =
    {
        name = "Boosted Wildfire",
        desc = "Apply <#UPGRADE>{1}</> {BURN} to target enemy, then apply {BURN} to all other enemies equal to {BURN} on the target.",
        burn_amt = 2,
    },

    wildfire_plus2 = 
    {
        name = "Twisted Wildfire",
        desc = "Apply {1} <#UPGRADE>{SCORCHED}</> to target enemy, then apply <#UPGRADE>{SCORCHED}</> to all other enemies equal to <#UPGRADE>{SCORCHED}</> on the target.",

        OnPostResolve = function( self, battle, attack )
            local count = 0
            for i, hit in attack:Hits() do
                hit.target:AddCondition("SCORCHED", self.burn_amt, self)
                count = count + hit.target:GetConditionStacks("SCORCHED")
            end

            if count > 0 then
                for i,fighter in ipairs(self.owner:GetEnemyTeam().fighters) do
                    fighter:AddCondition("SCORCHED", count, self)
                end
            end
        end
    },

    burner =
    {
        name = "Burner",
        anim = "burner",
        flavour = "'It just ends up smelling so bad.'",
        desc = "Apply {1} {BURN}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.burn_amt, self.scorched_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,

        min_damage = 2,
        max_damage = 3,
        burn_amt = 2,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("BURN", self.burn_amt, self)
        end,
    },

    burner_plus =
    {
        name = "Boosted Burner",
        min_damage = 3,
        max_damage = 6,
    },

    burner_plus2 =
    {
        name = "Enhanced Burner",
        desc = "Apply <#UPGRADE>{1}</> {BURN}.",
        burn_amt = 4,
    },

    brain_burn =
    {
        name = "Brain Burn",
        
        anim = "psionic",
        flavour = "'I picked this one up after Operation Reed Walker. Haven't used it since.'",
        
        desc = "Apply 1 {BURN} per {CONCENTRATION}.",

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 1,
        max_damage = 4,

        burn_bonus = 0,

        OnPostResolve = function( self, battle, attack )
            local count = self.owner:GetConditionStacks("CONCENTRATION") + self.burn_bonus
            if count > 0 then
                attack:AddCondition("BURN", count, self)
            end
        end
    },

    brain_burn_plus =
    {
        name = "Tall Brain Burn",
        max_damage = 6,
    },

    brain_burn_plus2 =
    {
        name = "Boosted Brain Burn",
        desc = "Apply 1 {BURN} per {CONCENTRATION}<#UPGRADE> +1</>.",
        burn_bonus = 1,
    },

    chimney = 
    {
        name = "Chimney",
        anim = "crackle",
        flavour = "'Okay, be sure you hit them in the right spot, otherwise you aren't going to be too impressed.'",
        desc = "Apply {1} {BURN}.\n{THRESHOLD} {2}: Apply {3} {SCORCHED}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.burn_amt, self:CalculateThresholdText(self), self.scorched_amt)
        end,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON, 

        cost = 1,
        min_damage = 1,
        max_damage = 6,

        threshold = 4,
        burn_amt = 2,
        scorched_amt = 2,

        ThresholdEffect = function( self, battle, attack )
            attack:AddCondition("SCORCHED", self.scorched_amt, self)
        end,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("BURN", self.burn_amt, self)
        end
    },

    chimney_plus =
    {
        name = "Boosted Chimney",
        desc = "Apply <#UPGRADE>{1}</> {BURN}.\n{THRESHOLD} {2}: Apply {3} {SCORCHED}.",
        burn_amt = 4,
    },

    chimney_plus2 =
    {
        name = "Tall Chimney",
        max_damage = 8,
    },

    mantle =
    {
        name = "Mantle",
        anim = "burner",
        flavour = "'They pull their juice from the spooled up reserves, just make sure you are running hot.'",
        desc = "Apply {1} {SCORCHED} to all enemies.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.scorched_amt, self.burn_amt )
        end,

        cost = 2,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.UNCOMMON,
        target_mod = TARGET_MOD.TEAM,

        burn_amt = 0,
        scorched_amt = 2,

        OnPostResolve = function( self, battle, attack )
            if self.burn_amt > 0 then
                attack:AddCondition( "BURN", self.burn_amt, self )
            end
            attack:AddCondition( "SCORCHED", self.scorched_amt, self )
        end,
    },

    mantle_plus =
    {
        name = "Boosted Mantle",
        desc = "Apply <#UPGRADE>{1}</> {SCORCHED} to all enemies.",
        scorched_amt = 4,
    },

    mantle_plus2 =
    {
        name = "Enhanced Mantle",
        desc = "Apply {1} {SCORCHED} <#UPGRADE>and {2} {BURN}</> to all enemies.",
        burn_amt = 3,
    },

    vantage =
    {
        name = "Vantage",
        anim = "reload_01",
        flavour = "'You have the high ground? Oh wow, that guy is toast.'",
        desc = "Draw {1} cards, gain {CONCENTRATION} equal to their combined cost.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.card_draw)
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.COMMON,

        target_type = TARGET_TYPE.SELF,

        card_draw = 2,

        base_gain = 0,

        OnPostResolve = function( self, battle, attack )
            local cards = battle:DrawCards(self.card_draw)
            local cost = self.base_gain
            for i,card in ipairs(cards) do
                cost = cost + battle:CalculateActionCost( card )
            end
            if cost > 0 then
                self.owner:AddCondition("CONCENTRATION", cost, self)
            end
        end
    },

    vantage_plus =
    {
        name = "Visionary Vantage",
        desc = "Draw <#UPGRADE>{1}</> cards, gain {CONCENTRATION} equal to their combined cost.",
        card_draw = 3,
    },

    vantage_plus2 =
    {
        name = "Boosted Vantage",
        desc = "Draw {1} cards, gain {CONCENTRATION} equal to their combined cost<#UPGRADE> +2</>.",
        base_gain = 2,
    },

    posture =
    {
        name = "Posture",
        anim = "taunt",
        flavour = "'Good to keep that back straight, you've been doing this for long enough.'",
        desc = "Apply {1} {DEFEND}. Gain {2} {CONCENTRATION}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt), self.concentration_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.COMMON,

        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        defend_amt = 3,
        concentration_amt = 3,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
            self.owner:AddCondition("CONCENTRATION", self.concentration_amt, self)
        end
    },

    posture_plus =
    {
        name = "Stone Posture",
        desc = "Apply <#UPGRADE>{1}</> {DEFEND}. Gain {2} {CONCENTRATION}.",
        defend_amt = 6,
    },

    posture_plus2 =
    {
        name = "Boosted Posture",
        desc = "Apply {1} {DEFEND}. Gain <#UPGRADE>{2}</> {CONCENTRATION}.",
        concentration_amt = 5,
    },

    focused_strike =
    {
        name = "Focused Strike",
        anim = "punch",
        flavour = "'Best to take advantage of the cooling pistols.'",
        desc = "{EMPTY}: Gain {1} {CONCENTRATION}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.concentration_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,
        
        PreReq = NoCharges,

        min_damage = 2,
        max_damage = 5,

        concentration_amt = 3,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker and tracker:GetCharges() == 0 then
                self.owner:AddCondition("CONCENTRATION", self.concentration_amt, self)
            end
        end
    },

    focused_strike_plus =
    {
        name = "Tall Focused Strike",
        max_damage = 8,
    },

    focused_strike_plus2 =
    {
        name = "Boosted Focused Strike",
        desc = "{EMPTY}: Gain <#UPGRADE>{1}</> {CONCENTRATION}.",
        concentration_amt = 5,
    },

    cataclysm =
    {
        name = "Cataclysm",
        anim = "psionic",
        flavour = "'How did that storm happen inside? My head is still ringing.'",
        desc = "Remove all debuffs on all enemies and deal {1} bonus damage per debuff removed. Hits all enemies.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,

        cost = 2,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.RARE,
        target_mod = TARGET_MOD.TEAM,

        min_damage = 3,
        max_damage = 3,

        bonus_damage = 3,

        OnPostResolve = function( self, battle, attack )
            for k,target in pairs(self.owner:GetEnemyTeam().fighters) do
                target:RemoveAllDebuffs( self )
            end
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local count = 0
                    for k,target in pairs(self.owner:GetEnemyTeam().fighters) do
                        for i,condition in pairs(target:GetConditions()) do
                            if condition.ctype == CTYPE.DEBUFF then
                                count = count + 1
                            end
                        end
                    end
                   dmgt:AddDamage(count * self.bonus_damage, count * self.bonus_damage, self)
                end
            end
        },
    },

    cataclysm_plus =
    {
        name = "Boosted Cataclysm",
        min_damage = 5,
        max_damage = 5,
    },

    cataclysm_plus2 =
    {
        name = "Pale Cataclysm",
        cost = 1,
    },

    psionic_storm =
    {
        name = "Psionic Storm",
        anim = "psionic",
        flavour = "'What's wrong with me? I can't see myself think.'",
        desc = "Insert {1} {affliction} cards into your draw pile.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.num_cards)
        end,

        cost = 2,

        target_type = TARGET_TYPE.SELF,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        num_cards = 3,

        options = { "current", "wave", "riptide", "shear", "whirl", "twist", "melt"},

        OnPostResolve = function( self, battle, attack )
            local cards = {}
            for i=1,self.num_cards do
                local card = Battle.Card(self.options[math.random(#self.options)], self.owner)
                table.insert(cards, card)
            end
            battle:DealCards( cards )
        end
    },

    psionic_storm_plus =
    {
        name = "Pale Psionic Storm",
        cost = 1,
    },

    psionic_storm_plus2 =
    {
        name = "Boosted Psionic Storm",
        desc = "Insert <#UPGRADE>{1}</> {affliction} cards into your draw pile.",
        num_cards = 5,
    },

    current = 
    {
        name = "Current",
        desc = "When drawn, apply 2 {IMPAIR} to a random enemy.",
        flavour = "'Never thought that you could weaponize a headache.'",
        cost = 0,

        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNIQUE,

        event_handlers = 
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card == self then
                    local target_fighters = {}
                    self.engine:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )
                    if #target_fighters > 0 then
                        target_fighters[math.random(#target_fighters)]:AddCondition("IMPAIR", 2, self)
                        self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "psionic")
                        self:NotifyTriggered()
                    end
                end
            end,
        },
    },

    wave = 
    {
        name = "Wave",
        desc = "When drawn, apply 2 {WOUND} to a random enemy.",

        cost = 0,
        flavour = "'There's some mental resistance added, and it makes them a very easy target.'",

        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNIQUE,

        event_handlers = 
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card == self then
                    local target_fighters = {}
                    self.engine:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )
                    if #target_fighters > 0 then
                        target_fighters[math.random(#target_fighters)]:AddCondition("WOUND", 2, self)
                        self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "psionic")
                        self:NotifyTriggered()
                    end
                end
            end,
        },
    },

    riptide = 
    {
        name = "Riptide",
        desc = "When drawn, apply 3 {RICOCHET} to a random enemy.",
        flavour = "'You are going to get a lot more value out of your shots after they are primed with this.'",
        cost = 0,

        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNIQUE,

        event_handlers = 
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card == self then
                    local target_fighters = {}
                    self.engine:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )
                    if #target_fighters > 0 then
                        target_fighters[math.random(#target_fighters)]:AddCondition("RICOCHET", 3, self)
                        self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "psionic")
                        self:NotifyTriggered()
                    end
                end
            end,
        },
    },

    shear = 
    {
        name = "Shear",
        desc = "When drawn, apply 3 {MARK} to a random enemy.",
        flavour = "'Staggered and confused, plenty of time to reload.'",
        cost = 0,

        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNIQUE,

        event_handlers = 
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card == self then
                    local target_fighters = {}
                    self.engine:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )
                    if #target_fighters > 0 then
                        target_fighters[math.random(#target_fighters)]:AddCondition("MARK", 3, self)
                        self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "psionic")
                        self:NotifyTriggered()
                    end
                end
            end,
        },
    },

    whirl = 
    {
        name = "Whirl",
        desc = "When drawn, apply 2 {SCANNED} to a random enemy.",
        flavour = "'The rhythm is a little tricky to get down, but if the offset is right you'll automatically reload.'",
        cost = 0,

        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNIQUE,

        event_handlers = 
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card == self then
                    local target_fighters = {}
                    self.engine:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )
                    if #target_fighters > 0 then
                        target_fighters[math.random(#target_fighters)]:AddCondition("SCANNED", 2, self)
                        self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "psionic")
                        self:NotifyTriggered()
                    end
                end
            end,
        },
    },

    twist = 
    {
        name = "Twist",
        desc = "When drawn, apply 3 {BURN} to a random enemy.",
        flavour = "'Spontaneous combustion anyone? The fire was inside them this whole time.'",
        cost = 0,

        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNIQUE,

        event_handlers = 
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card == self then
                    local target_fighters = {}
                    self.engine:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )
                    if #target_fighters > 0 then
                        target_fighters[math.random(#target_fighters)]:AddCondition("BURN", 3, self)
                        self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "psionic")
                        self:NotifyTriggered()
                    end
                end
            end,
        },
    },

    melt = 
    {
        name = "Melt",
        desc = "When drawn, apply 2 {SCORCHED} to a random enemy.",

        cost = 0,
        flavour = "'Could be something to do with their glands, I never cared enough to look at the corpses.'",
        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNIQUE,

        event_handlers = 
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card == self then
                    local target_fighters = {}
                    self.engine:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )
                    if #target_fighters > 0 then
                        target_fighters[math.random(#target_fighters)]:AddCondition("SCORCHED", 2, self)
                        self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "psionic")
                        self:NotifyTriggered()
                    end
                end
            end,
        },
    },

    affliction = 
    {
        name = "Affliction",
        desc = "When drawn, apply a random debuff to a random enemy.",
        flavour = "'Hey it's a strange world, there could be any number of things wrong with them.'",
        cost = 0,

        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNIQUE,
    },

    reserve_round =
    {
        name = "Reserve Round",
        pre_anim = "surge_pre",
        anim = "surge",
        flavour = "'Just don't forget to take them out before you wash your clothes.'",
        post_anim = "surge_pst",
        desc = "Spend {1} {SURGE}: this card costs 0.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.surge_cost)
        end,

        cost = 2,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.COMMON,

        min_damage = 5,
        max_damage = 8,
        surge_cost = 3,

        PreReq = function( self, battle )
            return self.owner:GetConditionStacks("SURGE") >= self.surge_cost
        end,

        OnPostResolve = function( self, battle, attack )
            if self.owner:GetConditionStacks("SURGE") >= self.surge_cost then
                self.owner:RemoveCondition("SURGE", self.surge_cost, self)
            end
        end,

        event_priorities =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self and self.owner:GetConditionStacks("SURGE") >= self.surge_cost then
                    cost_acc:ModifyValue(0)
                end
            end
        },
    },

    reserve_round_plus =
    {
        name = "Pale Reserve Round",
        desc = "Spend <#UPGRADE>{1}</> {SURGE}: this card costs 0.",
        surge_cost = 2,
    },

    reserve_round_plus2 =
    {
        name = "Tall Reserve Round",
        max_damage = 10,
    },

    surge_coil =
    {
        name = "Surge Coil",
        pre_anim = "surge_pre",
        flavour = "'Yeah I could get in a lot of trouble for installing this, but you are just so damned persuasive.'",
        anim = "surge",
        post_anim = "surge_pst",
        desc = "If you have any {SURGE}: Attack twice.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.surge_cost)
        end,

        cost = 1,

        flags = CARD_FLAGS.RANGED,
        rarity = CARD_RARITY.COMMON,

        min_damage = 2,
        max_damage = 5,


        PreReq = function( self, battle )
            if self.owner:HasCondition("SURGE") then
                self.hit_count = 2
                return true
            else
                self.hit_count = 1
                return false
            end
        end,
    },

    surge_coil_plus =
    {
        name = "Rooted Surge Coil",
        min_damage = 5,
        max_damage = 5,
    },

    surge_coil_plus2 =
    {
        name = "Boosted Surge Coil",
        min_damage = 3,
        max_damage = 7,
    },

    amplifier =
    {
        name = "Amplifier",
        anim = "taunt",
        flavour = "'The box is a little clunky but when you see the ignition bonuses hit your targets it makes the awkwardness well worth it.'",
        desc = "{ABILITY}: Apply 2 {BURN} whenever you apply {SCORCHED}.",
        on_play_screen_shake = true,

        cost = 2,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("amplifier", 2, self)
        end,

        condition =
        {
            desc = "Apply <#HILITE>{1} {BURN}</> whenever you apply {SCORCHED}.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,

            event_handlers = 
            {
                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if source and source.owner == self.owner and condition.id == "SCORCHED" then
                        fighter:AddCondition("BURN", self.stacks, self)
                    end
                end
            },
        },
    },

    amplifier_plus = 
    {
        name = "Initial Amplifier",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    amplifier_plus2 =
    {
        name = "Pale Amplifier",
        cost = 1,
    },

    shovel = 
    {
        name = "Shovel",
        anim = "punch",
        desc = "Deal 2 bonus damage per empty cell. Gain 2 {DEFEND} per {CHARGE}.",
        flavour = "'Yeah, I won't be doing too much digging...'",
        cost = 0,

        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_damage = 2,
        max_damage = 2,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            if tracker then
                self.owner:AddCondition("DEFEND", tracker:GetCharges() * 2, self)
            end
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self then
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    local empty = tracker and tracker:GetEmptyCharges() or 0
                    dmgt:AddDamage( empty * 2, empty * 2, self )
                end
            end,
        },
    },

    sentinel =
    {
        name = "Sentinel",
        desc = "Gain {1} {DEFEND}. Until the start of your next turn, gain {2} {CHARGE} whenever you are hit by an attack.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amt or 10 ), self.charge_amt or 1)
        end,
        flavour = "’You don’t last as long as Rook by not being able to take a beating.’",

        cost = 2,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.SELF,
        defend_amt = 10,
        charge_amt = 1,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("DEFEND", self.defend_amt, self)
            self.owner:AddCondition("sentinel", self.charge_amt, self)
        end,

        condition =
        {
            desc = "Until the start of your next turn, gain <#HILITE>{1} {CHARGE}</> whenever you are hit by an attack.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks or 1)
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if hit.target == self.owner and attack.card:IsAttackCard() then
                        local tracker = self.owner:GetCondition("lumin_tracker")
                        if tracker then
                            tracker:AddLuminCharges( self.stacks, self )
                        end
                    end
                end,

                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition(self.id, self.stacks, self)
                end
            },
        },
    },

    sentinel_plus =
    {
        name = "Stone Sentinel",
        desc = "Gain <#UPGRADE>{1} {DEFEND}</>. Until the start of your next turn, gain {2} {CHARGE} whenever you are hit by an attack.",
        defend_amt = 15,
    },

    sentinel_plus2 = 
    {
        name = "Induction Sentinel",
        desc = "Gain {1} {DEFEND}. Until the start of your next turn, gain <#UPGRADE>{2} {CHARGE}</> whenever you are hit by an attack.",
        charge_amt = 2,
    },

    challenger =
    {
        name = "Challenger",
        desc = "Gain {1} {DEFEND} per enemy targeting you.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt))
        end,
        flavour = "’You think you and your crew scare me? I’ve been taking on worse before you could even crawl!’",

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.COMMON,

        defend_amt = 3,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            local count = 0
            for i,enemy in battle:GetEnemyTeam():Fighters() do
                count = count + (enemy:IsTargetting( self.owner ) and 1 or 0)
            end
            for i=1,count do
                self.owner:AddCondition("DEFEND", self.defend_amt, self)
                if self.riposte_amt then
                    self.owner:AddCondition("RIPOSTE", self.riposte_amt, self)
                end
            end
        end
    },

    challenger_plus =
    {
        name = "Stone Challenger",
        desc = "Gain <#UPGRADE>{1} {DEFEND}</> per enemy targeting you.",
        defend_amt = 5,
    },

    challenger_plus2 =
    {
        name = "Thorny Challenger",
        desc = "Gain {1} {DEFEND} <#UPGRADE>and {2} {RIPOSTE}</> per enemy targeting you.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt), self.riposte_amt)
        end,
        riposte_amt = 2,
    },

    fixed =
    {
        name = "Fixed",
        desc = "Apply {fixed {1}}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.fixed_amt)
        end,
        flavour = "’I’ve got your number, punk.’",

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        fixed_amt = 2,

        OnPostResolve = function( self, battle, attack )
            local con = attack:AddCondition("fixed", self.fixed_amt, self)
            if con then
                con.applier = self.owner
            end
        end,

        condition =
        {
            name = "Fixed",
            desc = "Whenever this target is hit, you gain <#HILITE>{1} {DEFEND}</>. At the start of {2}'s turn, remove all <b>Fixed</>.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks or 1, self:GetOwnerName())
            end,

            ctype = CTYPE.DEBUFF,

            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if hit.target == self.owner then
                        battle:GetPlayerFighter():AddCondition("DEFEND", self.stacks, self)
                    end
                end,

                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        self.owner:RemoveCondition(self.id, self.stacks, self)
                    end
                end
            },
        },
    },

    fixed_plus =
    {
        name = "Boosted Fixed",
        desc = "Apply <#UPGRADE>{fixed {1}}</>.",
        fixed_amt = 3,
    },

    fixed_plus2 =
    {
        name = "Wide Fixed",
        desc = "Apply {fixed {1}} <#UPGRADE>to all enemies</>.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_mod = TARGET_MOD.TEAM,
    },

    release_valve =
    {
        name = "Release Valve",
        desc = "Spend all {CHARGE}: Gain {1} {DEFEND} per {CHARGE} spent.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt))
        end,
        flavour = "’Look, if they get a little too charged up, just hit this switch and remember to hold your breath, the backblast is heavy and thick as stew.’",

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.COMMON,
        target_type = TARGET_TYPE.SELF,

        defend_amt = 2,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            local count = tracker:GetCharges()
            tracker:RemoveCharges(count, self)
            for i=1,count do
                self.owner:AddCondition("DEFEND", self.defend_amt, self)
            end
            if self.temp_power_amt then
                self.owner:AddCondition("POWER", self.temp_power_amt * count, self)
                self.owner:AddCondition("POWER_LOSS", self.temp_power_amt * count, self)
            end
        end,
    },

    release_valve_plus =
    {
        name = "Stone Release Valve",
        desc = "Spend all {CHARGE}: Gain <#UPGRADE>{1} {DEFEND}</> per {CHARGE} spent.",
        defend_amt = 3,
    },

    release_valve_plus2 =
    {
        name = "Boosted Release Valve",
        desc = "Spend all {CHARGE}: Gain {1} {DEFEND}<#UPGRADE> and {2} {TEMP_POWER}</> per {CHARGE} spent.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt), self.temp_power_amt)
        end,
        temp_power_amt = 1,
    },

    casings =
    {
        name = "Casings",
        desc = "{ABILITY}: Whenever you spend {CHARGE}, gain {1} {DEFEND} per {CHARGE} spent.",
        desc_fn = function( self, fmt_str )
            self.dummy_condition = self.dummy_condition or CombatCondition( "casings", self.owner )
            return loc.format(fmt_str, self.dummy_condition:CalculateDefendText(self.defend_amt))
        end,
        on_play_screen_shake = true,
        flavour = "’Hey these polished shells aren’t just here for looks you know!’",

        cost = 3,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        defend_amt = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("casings", self.defend_amt, self)
        end,

        condition =
        {
            name = "Casings",
            desc = "Whenever you spend {CHARGE}, gain <#HILITE>{1} {DEFEND}</> per {CHARGE} spent.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self:CalculateDefendText(self.stacks or 1))
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.LUMIN_SPENT ] = function( self, battle, charged, max_cells )
                    self.owner:AddCondition("DEFEND", self.stacks, self)
                end
            },
        },
    },

    casings_plus =
    {
        name = "Pale Casings",
        cost = 2,
    },

    casings_plus2 =
    {
        name = "Initial Casings",
        desc = "{ABILITY}: Whenever you spend {CHARGE}, gain {1} {DEFEND} per {CHARGE} spent.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    tempered =
    {
        name = "Tempered",
        desc = "{ABILITY}: Whenever you attack an enemy that has {BURN}, gain {1} {DEFEND}.",
        on_play_screen_shake = true,
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.defend_amt)
        end,
        flavour = "’It sounds obvious, but when enemies are on fire they are a lot more distracted.’",

        cost = 2,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        defend_amt = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("tempered", self.defend_amt, self)
        end,

        condition =
        {
            name = "Tempered",
            desc = "Whenever you attack an enemy that has {BURN}, gain <#HILITE>{1} {DEFEND}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self:CalculateDefendText(self.stacks or 1))
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if hit.target:HasCondition("BURN") and attack.attacker == self.owner then
                        self.owner:AddCondition("DEFEND", self.stacks, self)
                    end
                end,
            },
        },
    },

    tempered_plus =
    {
        name = "Pale Tempered",
        cost = 1,
    },

    tempered_plus2 =
    {
        name = "Initial Tempered",
        desc = "{ABILITY}: Whenever you attack an enemy that has {BURN}, gain {1} {DEFEND}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    garbage_day =
    {
        name = "Garbage Day",
        anim = "taunt",
        desc = "Gain {DEFEND} equal to the number of cards in your discard.{1}",
        loc_strings =
        {
            GAIN_DEFEND = "\n(Gain {1} {DEFEND})",
        },
        desc_fn = function( self, fmt_str )
            if self.engine then
                local str = loc.format( self.def:GetLocalizedString( "GAIN_DEFEND" ), self:CalculateDefendText(self.engine:GetDiscardDeck():CountCards() + self.bonus) )
                return loc.format( fmt_str, str )
            else
                return loc.format( fmt_str, "" )
            end
        end,
        flavour = "’Where did all this crap come from!?’",

        cost = 2,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.RARE,

        target_type = TARGET_TYPE.SELF,

        bonus = 0,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("DEFEND", self.engine:GetDiscardDeck():CountCards() + self.bonus, self)
        end
    },

    garbage_day_plus =
    {
        name = "Visionary Garbage Day",
        desc = "Gain {DEFEND} equal to the number of cards in your discard. <#UPGRADE>Draw a card.</>{1}",
        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("DEFEND", self.engine:GetDiscardDeck():CountCards(), self)
            battle:DrawCards(1)
        end
    },

    garbage_day_plus2 =
    {
        name = "Boosted Garbage Day",
        desc = "Gain {DEFEND} equal to the number of cards in your discard <#UPGRADE>+3</>.{1}",
        bonus = 3,
    },

    energy_loop =
    {
        name = "Energy Loop",
        anim = "reload_02",
        desc = "Spend all {CHARGE} and {HEAL} {1} for every {CHARGE} spent.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.heal_amt)
        end,
        flavour = "’The juice can be redirected into your own cybernetics. It’s not really comfortable but you’re probably already used to that.’",

        cost = 1,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.BURNOUT,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.SELF,

        heal_amt = 2,

        OnPostResolve = function( self, battle, attack )
            local tracker = self.owner:GetCondition("lumin_tracker")
            self.owner:HealHealth(tracker:GetCharges() * self.heal_amt, self)
            tracker:RemoveCharges(tracker:GetCharges(), self)
        end,
    },

    energy_loop_plus =
    {
        name = "Stable Energy Loop",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
    },

    energy_loop_plus2 =
    {
        name = "Boosted Energy Loop",
        desc = "Spend all {CHARGE} and <#UPGRADE>{HEAL} {1}</> for every {CHARGE} spent.",
        heal_amt = 3,
    },

    dugout =
    {
        name = "Dugout",
        anim = "taunt",
        desc = "Apply {1} {DEFEND}.\nGain 1 {WIDE_OPEN} at the beginning of your next turn.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt))
        end,
        flavour = "’He’s over here!’", 

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.COMMON,

        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        defend_amt = 10,

        OnPostResolve = function( self, battle, attack)
            attack:AddCondition("DEFEND", self.defend_amt, self)
            self.owner:AddCondition("dugout", 1, self)
        end,

        condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    self.owner:AddCondition("WIDE_OPEN", self.stacks, self)
                    self.owner:RemoveCondition(self.id, self.stacks, self)
                end
            },
        },
    },

    dugout_plus =
    {
        name = "Stone Dugout",
        desc = "Apply <#UPGRADE>{1} {DEFEND}</>.\nGain 1 {WIDE_OPEN} at the beginning of your next turn.",
        defend_amt = 14,
    },

    dugout_plus2 =
    {
        name = "Purified Dugout",
        desc = "Apply {1} {DEFEND}.\nGain 1 <#UPGRADE>{EXPOSED}</> at the beginning of your next turn.",
        OnPostResolve = function( self, battle, attack)
            attack:AddCondition("DEFEND", self.defend_amt, self)
            self.owner:AddCondition("dugout_plus2", 1, self)
        end,

        condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    self.owner:AddCondition("EXPOSED", self.stacks, self)
                    self.owner:RemoveCondition(self.id, self.stacks, self)
                end
            },
        },
    },

    tight_spot =
    {
        name = "Tight Spot",
        anim = "taunt",
        desc = "Apply {1} {DEFEND}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt))
        end,
        flavour = "’Get behind that pillar before it disintegrates!’",

        cost = 1,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.BURNOUT,
        rarity = CARD_RARITY.COMMON,
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        defend_amt = 8,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
        end
    },

    tight_spot_plus =
    {
        name = "Stone Tight Spot",
        desc = "Apply <#UPGRADE>{1} {DEFEND}</>.",
        defend_amt = 12,
    },

    tight_spot_plus2 =
    {
        name = "Stable Tight Spot",
        flags = CARD_FLAGS.SKILL,
    },

    lever =
    {
        name = "Lever",
        anim = "taunt",
        desc = "Apply {1} {DEFEND} and gain {2} {SURGE}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt), self.surge_amt)
        end,
        flavour = "’Let them charge up for a second after you hit the switch.’",

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        defend_amt = 5,
        surge_amt = 2,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
            self.owner:AddCondition("SURGE", self.surge_amt, self)
        end
    },

    lever_plus =
    {
        name = "Stone Lever",
        desc = "Apply <#UPGRADE>{1} {DEFEND}</> and gain {2} {SURGE}.",
        defend_amt = 7,
    },

    lever_plus2 =
    {
        name = "Surging Lever",
        desc = "Apply {1} {DEFEND} and gain <#UPGRADE>{2} {SURGE}</>.",
        surge_amt = 3,
    },
}

for i, id, data in sorted_pairs(attacks) do
    data.series = "ROOK"

    Content.AddBattleCard( id, data )
end


local CONDITIONS =
{
    lumin_tracker =
    {
        hidden = true,
        defend_per_charge = 1,

        OnApply = function( self )
            self.charges = 0
            self.max_charges = self.owner:GetAgent():GetAspect("graft_owner"):GetGraft("rook_blasters") and self.owner:GetAgent():GetAspect("graft_owner"):GetGraft("rook_blasters").userdata.max_charges or 4
            self.primed = false
        end,

        ModifyMaxCharges = function( self, delta )
            self.max_charges = math.max( 0, self.max_charges + delta )
        end,

        AddLuminCharges = function( self, count, source )
            local counter = 0
            for i=1,count do
                local added = self:AddLuminCharge( source )
                if added then
                    counter = counter + 1
                end
            end
            return counter
        end,
        AddLuminCharge = function( self, source )
            if self.charges < self.max_charges then
                self.charges = self.charges + 1
                self:CheckPrimed()
                self.battle:BroadcastEvent( BATTLE_EVENT.LUMIN_CHARGED, self.battle, self:GetCharges() )
                return true
            else
                self.owner:AddCondition("SURGE", 1, self)
                return false
            end
        end,
        RemoveCharges = function( self, count, source )
            local removed = 0
            for i=1,count do
                if self.charges > 0 then
                    self.charges = self.charges - 1
                    self.battle:BroadcastEvent( BATTLE_EVENT.LUMIN_SPENT, self.battle, self:GetCharges() )
                    removed = removed + 1
                end
            end
            self:CheckPrimed()
            return removed
        end,

        GetEmptyCharges = function( self )
            return self.max_charges - self.charges
        end,

        GetCharges = function( self )
            return self.charges, self.max_charges
        end,

        CheckPrimed = function( self )
            local primed = (self.charges >= self.max_charges)
            if primed ~= self.primed then
                self.primed = primed
                if primed then
                    self.battle:BroadcastEvent( BATTLE_EVENT.PRIMED_GAINED, self.battle )
                else
                    self.battle:BroadcastEvent( BATTLE_EVENT.PRIMED_LOST, self.battle )
                end
            end
        end,

        event_priorities =
        {
            [ BATTLE_EVENT.END_PLAYER_TURN  ] = 10,
        },

        event_handlers = 
        {
            [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                self:AddLuminCharges(1, self, true)
            end,

            [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                self.owner:AddCondition("DEFEND", self:GetEmptyCharges() * self.defend_per_charge, self)
            end
        },

        fighter_info_widget = function( fighter, condition, width, silent )
            return Widget.ChargeCellInfoWidget( fighter, condition, width, silent )
        end
    },

    CONCENTRATION = 
    {
        name = "Concentration",
        desc = "Increases minimum damage of all attacks by <#HILITE>{1}</>. Halve your <b>Concentration</b> when you take enemy damage.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.stacks or 1)
        end,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

        event_priorities =
        {
            -- after: surge
            [ BATTLE_EVENT.CALC_DAMAGE  ] = 11,
        },

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card.owner == self.owner then
                    dmgt:ModifyDamage( dmgt.min_damage + self.stacks, dmgt.max_damage, self )
                end
            end,

            [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
                if fighter == self.owner and self.battle:IsEnemySource( fighter, source ) then
                    if self.stacks > 0 then
                        self.owner:RemoveCondition( "CONCENTRATION", math.ceil(self.stacks * 0.5) )
                    end
                end
            end

        },
    },

    SURGE = 
    {
        hidden = true,
        name = "Overcharge",
        desc = "Increases max damage of all cards by 1 for each stack of <b>Overcharge</b>.\nGaining {CHARGE} while Fully Charged generates Overcharge instead. Reduce Overcharge by half at the end of your turn.",
        icon = "battle/conditions/heat.tex",

        event_priorities =
        {
            -- Precede: concentration
            [ BATTLE_EVENT.CALC_DAMAGE  ] = 10,
        },

        event_handlers =
        {
            [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                self.owner:RemoveCondition("SURGE", math.ceil(self.stacks * 0.5), self)
            end,

            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card.owner == self.owner then
                    dmgt:AddDamage( 0, self.stacks, self)
                end
            end
        },
    },

    MARK =
    {
        name = "Mark",
        desc = "When you attack this target, gain 1 {CHARGE} and reduce <b>Mark</b> by 1.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetOwnerName() )
        end,

        ctype = CTYPE.DEBUFF,
        persist_on_death = true,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

        TriggerMark = function( self )
            local tracker = self.battle:GetPlayerFighter():GetCondition("lumin_tracker")
            if tracker then
                tracker:AddLuminCharges( 1, self)
            end
            self.owner:RemoveCondition("MARK", 1)
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if hit.target == self.owner and attack.card:IsAttackCard() and attack.attacker:IsPlayer() and not hit.evaded then
                    self:TriggerMark()
                end
            end,
        },
    },

    BURN = 
    {
        name = "Burn",
        desc = "At the start of your turn, take damage equal to the stack of <b>Burn</b> and deal half as much to all allies. Remove half of the stacks of <b>Burn</b>.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.dmg_per_round or 1 )
        end,
        hud_fx = {"burn"},
        
        dmg_per_round = 2,
        persist_on_death = true,

        ctype = CTYPE.DEBUFF,
        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack_Burn",

        CalculatePreviewDamage = function( self )
            return self.stacks
        end,

        CalculateDamage = function( self, fighter_table )
            fighter_table[self.owner] = (fighter_table[self.owner] or 0) + self.stacks
            local aoe = math.floor(self.stacks*.5)
            for i,target in ipairs(self.owner:GetTeam():GetFighters()) do
                if target ~= self.owner then
                    fighter_table[target] = (fighter_table[target] or 0) + aoe
                end
            end
            self.calculated = true
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                if fighter == self.owner then
                    if not self.calculated then
                        local total_damage = {}
                        for i,ally in ipairs(self.owner:GetTeam():GetFighters()) do
                            if ally:HasCondition("BURN") then
                                ally:GetCondition("BURN"):CalculateDamage(total_damage)
                            end
                        end

                        for fighter, damage in pairs(total_damage) do
                            if fighter:IsAlive() then
                                fighter:ApplyDamage(damage, nil, nil, true, {"burn"})
                                fighter:CheckForSurrender()
                            end
                        end
                    end

                    if self.owner:HasCondition("SCORCHED") then
                        self.owner:RemoveCondition("SCORCHED", 1)
                        self.owner:RemoveCondition("BURN", math.min(2, math.ceil(0.5*self.stacks)))
                    else
                        self.owner:RemoveCondition("BURN", math.ceil(0.5*self.stacks) )
                    end
                end
            end,

            [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                if fighter == self.owner then
                    self.calculated = false
                end
            end
        },
    },

    SCORCHED = 
    {
        name = "Scorched",
        desc = "{BURN} can not decay by more than 2 stacks per turn. Whenever <b>Burn</> is triggered, remove 1 <b>Scorched</>.",
        --"Whenever {BURN} is triggered, remove 1 <b>Scorched</b> and 3 {BURN} instead of removing half of {BURN}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetOwnerName() )
        end,

        ctype = CTYPE.DEBUFF,
        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack_Scorched",
    },

    SCANNED = 
    {
        name = "Scanned",
        desc = "Whenever you attack this target, draw a card. At the start of {1}'s turn, remove 1 <b>Scanned</>.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:GetOwnerName())
        end,

        ctype = CTYPE.DEBUFF,
        persist_on_death = true,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Other",

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT] = function( self, battle, attack, hit )
                local resolve_card = attack.card
                if resolve_card.owner and resolve_card.owner == battle:GetPlayerFighter() and resolve_card:IsAttackCard() and (attack.card.max_damage and attack.card.max_damage > 0) then
                    if hit.target == self.owner then
                        battle:DrawCards(1)
                    end
                end
            end,

            [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                if fighter == self.owner then
                    self.owner:RemoveCondition(self.id, 1, self)
                end
            end
        },
    },

    RICOCHET =
    {
        name = "Ricochet",
        desc = "Whenever this target takes damage from an attack, deal that much damage to one of their allies and reduce <b>Ricochet</b> by 1.",

        ctype = CTYPE.DEBUFF,
        persist_on_death = true,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT] = function( self, battle, attack, hit )
                if hit.target == self.owner and not attack.is_counter and hit.damage and hit.damage > 0 then
                    local valid_targets = {}
                    for i,fighter in self.owner:GetTeam():Fighters() do
                        if fighter ~= self.owner then
                            table.insert(valid_targets, fighter)
                        end
                    end
                    if #valid_targets > 0 then
                        valid_targets[math.random(#valid_targets)]:ApplyDamage(hit.damage, self.battle:GetFighterFromSource( attack ), nil, nil, { "ricochet" } )
                        self.owner:RemoveCondition("RICOCHET", 1)
                    end
                end
            end,
        },
    },

    WIDE_OPEN =
    {
        name = "Wide Open",
        desc = "Cannot gain {DEFEND}. At the end of {1}'s turn, reduce <b>Wide Open</> by 1.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:GetOwnerName())
        end,

        ctype = CTYPE.DEBUFF,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_MODIFY_STACKS ] = function( self, acc, condition_id, fighter, source )
                if condition_id == "DEFEND" and fighter == self.owner then
                    if acc.value > 0 then
                        acc:ModifyValue( 0, self )
                    end
                end
            end,

            [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                if self.owner == fighter then
                    self.owner:RemoveCondition( self.id, 1 )
                end
            end,
        }
    },
}


for id, def in pairs( CONDITIONS ) do
    Content.AddBattleCondition( id, def )

end


local FEATURES =
{
    PRIMED =
    {
        name = "Fully Charged",
        desc = "This card has an additional effect if all cells are charged.",
    },

    EMPTY = 
    {
        name = "Empty",
        desc = "This card has an additional effect if you have no Charge.",
    },

    CONCENTRATED =
    {
        name = "Concentrated",
        desc = "This card has an additional effect if you have {CONCENTRATION}."
    },

    CHARGE = 
    {
        name = "Charge",
        desc = "Rook can store <b>Charge</b> in his empty cells. <b>Charge</b> can be used by certain cards for additional effects.",
    },
}

for id, data in pairs( FEATURES ) do
    local def = BattleFeatureDef(id, data)
    Content.AddBattleCardFeature(id, def)
end