local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.EVENT
local DECK_TYPE = battle_defs.DECK_TYPE

local attacks =
{
    jammed_blaster =
    {
        name = "Jammed Blaster",
        anim = "punch",
        flavour = "'Caked in muck and slime, a few solid whacks should get this thing working again.'",

        max_xp = 4,
        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.HATCH,

        min_damage = 3,
        max_damage = 5,

        hatch = true,
        hatch_fn = function( self, battle )
            self:TransferCard( battle.trash_deck )
            self:Consume()
            local card = TheGame:GetGameState():GetPlayerAgent().battler:LearnCard("bog_blaster")
            local battle_card = card:Clone()
            battle_card.owner = self.owner
            battle:DealCard( battle_card, battle:GetHandDeck( ) )
        end,
    },

    bog_blaster =
    {
        name = "Bog Blaster",
        anim = "shoot",
        flavour = "'Good as new! I haven't seen one of these in a while.'",
        anims = { "anim/weapon_blaster_bogblaster.zip"},
        hit_tags = { "acid" },

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.REPLENISH,

        min_damage = 4,
        max_damage = 10,
    },

    flead_egg =
    {
        name = "Flead Egg",
        anim = "taunt",
        flavour = "'You can tell it's a girl by holding it up to the light.'",
        desc = "Draw a card.",

        max_xp = 4,
        hatch = true,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.HATCH,
        target_type = TARGET_TYPE.SELF,

        hatch_fn = function( self, battle )
            self:TransferCard( battle.trash_deck )
            self:Consume()
            local card = Battle.Card( "flead_egg_hatch", self.owner )
            card:TransferCard( battle:GetHandDeck() )
            card:TransferCard(battle.trash_deck)
            card:Consume()
            local flead = Agent( "FLEAD" )
            flead:GenerateRandomName()
            local fighter = Fighter.CreateFromAgent( flead, battle:GetScenario():GetAllyScale() )
            self.owner:GetTeam():AddFighter( fighter )
            self.owner:GetTeam():ActivateNewFighters()
            TheGame:GetGameState():AddAgent(flead)
            flead:Recruit(PARTY_MEMBER_TYPE.CREW)
        end,

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(1)
        end,
    },

    flead_egg_hatch =
    {
        name = "Hatched Egg",
        flavour = "'Don't worry, it thinks you're its mother.'",
        desc = "The egg cracks and a Flead emerges from within.",

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.UNPLAYABLE,
    },

    overloaded_spark_hammer =
    {
        name = "Overloaded Spark Hammer",
        anim = "melee_item",
        flavour = "'This is dangerous technology! We still don't really understand what it's capable of!.'",
        desc = "Draw a card.",
        anims = { "anim/weapon_melee_spark_hammer.zip"},
        hit_tags = { "spark" },

        cost = 1,

        max_xp = 4,
        hatch = true,
        
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.HATCH,

        min_damage = 4,
        max_damage = 8,

        hatch_fn = function( self, battle )
            self:TransferCard( battle.trash_deck )
            self:Consume()
            local card = Battle.Card( "overloaded_spark_hammer_hatch", self.owner )
            card:TransferCard( battle:GetHandDeck() )
            card:TransferCard(battle.trash_deck)
            card:Consume()
            for i,target in ipairs(battle.player_team:GetFighters()) do
                target:ApplyDamage( 6, nil, nil, true, {"spark"}) 
            end
            for i,target in ipairs(battle.enemy_team:GetFighters()) do
                target:ApplyDamage( 6, nil, nil, true, {"spark"}) 
            end
        end,

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(1)
        end
    },

    overloaded_spark_hammer_hatch =
    {
        name = "Violent Explosion",
        flavour = "'Why doesn't anyone listen to me?!'",
        desc = "Deal 6 damage to all characters.",

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.UNPLAYABLE
    },

    learning_drone =
    {
        name = "Learning Drone",
        flavour = "'ENGAGING OPTICS.'",
        anim = "mark",
        desc = "Apply 2 {SCANNED} to target enemy.",

        max_xp = 4,
        hatch = true,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.HATCH,

        hatch_fn = function( self, battle )
            self:TransferCard( battle.trash_deck )
            self:Consume()
            local card = TheGame:GetGameState():GetPlayerAgent().battler:LearnCard("learning_drone_hatch")
            local battle_card = card:Clone()
            battle_card.owner = self.owner
            battle:DealCard( battle_card, battle:GetHandDeck( ) )
        end,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("SCANNED", 2, self)
        end
    },

    learning_drone_hatch =
    {
        name = "Learned Drone",
        flavour = "'SUFFICIENT INFORMATION GATHERED. RELAYING ALL DATA TO THE CLOUD.'",
        desc = "Summon a Combat Drone to fight for you.",
        anim = "taunt",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
        target_type = TARGET_TYPE.SELF,

        CanPlayCard = function( self, battle, target )
            return self.owner:GetTeam():NumActiveFighters() < self.owner:GetTeam():GetMaxFighters(), CARD_PLAY_REASONS.TEAM_FULL
        end,

        OnPostResolve = function( self, battle, attack )
            local summon = Agent( "COMBAT_DRONE" )
            local fighter = Fighter.CreateFromAgent( summon, battle:GetScenario():GetAllyScale() )
            self.owner:GetTeam():AddFighter( fighter )
            self.owner:GetTeam():ActivateNewFighters()
        end,
    },

    lock_box =
    {
        name = "Lockbox",
        anim = "melee_item",
        flavour = "'Ahh yes, I'm familiar with this locking mechanism, just stand over there for a second.'",
        anims = { "anim/weapon_melee_lockbox.zip"},

        max_xp = 4,
        hatch = true,

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.HATCH,

        min_damage = 2,
        max_damage = 4,

        item_ids = {"lumin_grenade", "shock_box", "shock_grenade", "bog_brew"},

        hatch_fn = function( self, battle )
            self:TransferCard( battle.trash_deck )
                self:Consume()
                local card = Battle.Card( "lock_box_hatch", self.owner )
                card:TransferCard( battle:GetHandDeck() )
                card:TransferCard(battle.trash_deck)
                card:Consume()
                local cards = {}
                for i,item_id in ipairs(self.item_ids) do
                    
                    local new_card = self.owner.agent.battler:LearnCard(item_id)
                    local battle_card = new_card:Clone()
                    battle_card.owner = self.owner
                    table.insert(cards, battle_card)
                end
                battle:DealCards( cards, battle:GetHandDeck( ) )
        end,
    },

    lock_box_hatch =
    {
        name = "Open Lockbox",
        flavour = "'Just what I needed.'",
        desc = "The lock finally breaks and equipment spills out of the lockbox.",

        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.UNPLAYABLE,
    },
}


for i, id, data in sorted_pairs(attacks) do
    data.series = "GENERAL"
    data.item_tags = (data.item_tags or 0) | ITEM_TAGS.COMBAT
    data.flags = (data.flags or 0) | CARD_FLAGS.ITEM 
    data.rarity = data.rarity or CARD_RARITY.UNIQUE
    Content.AddBattleCard( id, data )
end