local negotiation_defs = require "negotiation/negotiation_defs"
local CARD_FLAGS = negotiation_defs.CARD_FLAGS
local EVENT = negotiation_defs.EVENT

local CARDS =
{

    spark_baron_permit = 
    {
        name = "Spark Baron Permit",
        rarity = CARD_RARITY.COMMON,
        flavour = "'It’s an older permit Sir, but it checks out.'",

        target_self = TARGET_ANY_RESOLVE,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.REPLENISH,

        cost = 0,
        max_charges = 2,
        features =
        {
            COMPOSURE = 5,
        },
    },

    level_orange_clearance =
    {
        name = "Level Orange Clearance",
        flavour = "'Usually required a tissue sample, but it just slowed down the whole process too much.'",
        desc = "Remove all of opponent's intents.",

        rarity = CARD_RARITY.UNIQUE,

        cost = 2,

        flags = CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            minigame:GetOtherNegotiator(self.negotiator):ClearIntents()
        end,
    },

    two_faced = 
    {
        name = "Two Faced",
        desc = "Gain {INFLUENCE 2} and {DOMINANCE 2}.",
        flavour = "'Fellemo had dirt on everyone, always ready to be displayed in his visor.'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("INFLUENCE", 2, self)
            self.negotiator:AddModifier("DOMINANCE", 2, self)
        end
    },
}




for i, id, carddef in sorted_pairs( CARDS ) do
    if not id:find( "_plus$") then
        carddef.item_tags = (carddef.item_tags or 0) | ITEM_TAGS.NEGOTIATION
        carddef.flags = (carddef.flags or 0) | CARD_FLAGS.ITEM
        carddef.rarity = carddef.rarity or CARD_RARITY.COMMON
        carddef.series = CARD_SERIES.GENERAL

        Content.AddNegotiationCard( id, carddef )
    end
end

