local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.EVENT

local attacks =
{
    --- GENERAL TROPHIES

    -- Humanoids

    carapace_fragment =
    {
        name = "Carapace Fragment",
        anim = "taunt",
        flavour = "'These things are usually so scarred up from fighting they shatter after one solid blow.'",
        desc = "Gain 8 {SLATED_CARAPACE}.",

        cost = 1,
        max_charges = 2,

        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition( "SLATED_CARAPACE", 8 )
        end,
    },

    giant_stinger =
    {
        name = "Giant Stinger",
        anim = "stab",
        flavour = "'Large and deadly.'",

        flags = CARD_FLAGS.PIERCING | CARD_FLAGS.MELEE | CARD_FLAGS.ITEM | CARD_FLAGS.EXPEND | CARD_FLAGS.STICKY,

        min_damage = 9,
        max_damage = 14,
        cost = 2,

        features =
        {
            WOUND = 2,
        }
    },

    spark_cannon =
    {
        name = "Spark Cannon",
        anim = "shoot",
        desc = "Adds bonus damage equal to 25% of target's health.",
        flavour = "'Something's gonna go boom.'",
        anims = { "anim/weapon_blaster_sparkcannon.zip"},
        hit_tags = { "spark" },

        cost = 2,
        max_charges = 1,
        flags = CARD_FLAGS.RANGED,

        target_type = TARGET_TYPE.ENEMY,

        min_damage = 4,
        max_damage = 10,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self and target then
                    local bonus = math.floor( target:GetHealth() * 0.25 )
                    dmgt:AddDamage( bonus, bonus )
                end
            end,
        },


        features =
        {
            EXERT = 1,
        }
    },

    -- Hesh boss
    lumin_generator =
    {
        name = "Lumin Bio-Accelerator",
        anim = "taunt",
        desc = "When this card is drawn, gain 1 {EVASION}.",
        flavour = "'Feels great! And I just have to keep it charged?'",
        rarity = CARD_RARITY.UNIQUE,
        series = "ROOK",

        cost = 0,

        flags = battle_defs.CARD_FLAGS.UNPLAYABLE | battle_defs.CARD_FLAGS.BURNOUT,

        event_handlers = 
        {
            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card == self then
                    self.owner:AddCondition("EVASION", 1, self)
                    self:NotifyTriggered()
                end
            end,
        },
    },

    force_spanner =
    {
        name = "Force Spanner",
        anim = "punch",
        flavour = "'Flekfis' wrench is mostly used for opening stuck doors and intimidating people.'",
        desc = "On your next turn, gain {TEMP_POWER} equal to damage dealt by this card.",

        rarity = CARD_RARITY.UNIQUE,

        cost = 2,

        flags = battle_defs.CARD_FLAGS.MELEE | battle_defs.CARD_FLAGS.EXPEND,

        min_damage = 4,
        max_damage = 8,

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if hit.card == self then
                    self.owner:AddCondition("force_spanner", hit.damage or 0, self)
                end
            end
        },

        condition =
        {
            hidden = true,

            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.owner:AddCondition("POWER", self.stacks, self)
                    self.owner:AddCondition("POWER_LOSS", self.stacks, self)
                    self.owner:RemoveCondition(self.id)
                end,
            },
        },
    },
}


for i, id, data in sorted_pairs(attacks) do
    data.series = "GENERAL"
    data.item_tags = (data.item_tags or 0) | ITEM_TAGS.COMBAT
    data.flags = (data.flags or 0) | CARD_FLAGS.ITEM 
    data.rarity = data.rarity or CARD_RARITY.UNIQUE
    Content.AddBattleCard( id, data )
end