local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

local attacks =
{
    improvise_mud = 
    {
        name = "Mud Tosser",
        icon = "battle/throw_rock.tex",
        anim = "throw",
        flavour = "'Is that mud in your eye? Or my fists?'",
        min_damage = 1,
        max_damage = 2,
        max_xp = 0,

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,

        features = 
        {
            COMBO = 2,
        },
    },

    improvise_mud_upgraded =
    {
        name = "Boosted Mud Tosser",
        icon = "battle/throw_rock.tex",
        anim = "throw",

        min_damage = 2,
        max_damage = 2,

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,

        features =
        {
            COMBO = 3,
        },
    },

    improvise_dust = 
    {
        name = "Throw Dust",
        icon = "battle/throw_dust.tex",
        anim = "throwdust",
        flavour = "'I always keep a little sand in my pocket for emergencies.'",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,

        features =
        {
            IMPAIR = 1,
        },
    },

    improvise_dust_upgraded =
    {
        name = "Boosted Throw Dust",
        icon = "battle/throw_dust.tex",
        anim = "throwdust",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.RANGED,

        features =
        {
            IMPAIR = 2,
        },
    },

    improvise_hide = 
    {
        name = "Hide",
        icon = "battle/hide.tex",
        anim = "taunt",
        flavour = "'Where the Hesh did they go?'",
        target_type = TARGET_TYPE.SELF,
        flavour = "'You can't fight what you can't find.'",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        features =
        {
            DEFEND = 3,
        },
    },

    improvise_hide_upgraded = 
    {
        name = "Boosted Hide",
        icon = "battle/hide.tex",
        anim = "taunt",
        target_type = TARGET_TYPE.SELF,

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        features =
        {
            DEFEND = 5,
        },
    },

    improvise_trip = 
    {
        name = "Trip",
        icon = "battle/trip.tex",
        anim = "trip",
        flavour = "'And stay down!'",
        
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MELEE,

        features =
        {
            WOUND = 2,
        },
    },

    improvise_trip_upgraded = 
    {
        name = "Boosted Trip",
        icon = "battle/trip.tex",
        anim = "trip",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MELEE,

        features =
        {
            WOUND = 3,
        },
    },

    improvise_slash = 
    {
        name = "Slash",
        icon = "battle/slash.tex",    
        anim = "slash",
        flavour = "'If it bleeds, I can kill it.'",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MELEE,

        min_damage = 1,
        max_damage = 2,

        features =
        {
            BLEED = 2,
        },
    },

    improvise_slash_upgraded = 
    {
        name = "Boosted Slash",
        icon = "battle/slash.tex",    
        anim = "slash",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,

        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MELEE,

        min_damage = 1,
        max_damage = 2,

        features =
        {
            BLEED = 3,
        },    
    },

    improvise_draw = 
    {
        name = "Scrounge",
        icon = "battle/scrounge.tex",
		desc = "Draw {1} {1*card|cards}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.num_draw)
        end,
        flavour = "'Which pocket did I put that in, again?'",

        anim = "taunt2",
        target_type = TARGET_TYPE.SELF,

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        max_xp = 0,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        num_draw = 2,

        OnPostResolve = function( self, battle, attack)
            battle:DrawCards( self.num_draw )
        end,
    },

    improvise_draw_upgraded = 
    {
        name = "Boosted Scrounge",
        icon = "battle/scrounge.tex",
        desc = "Draw {1} {1*card|cards}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.num_draw)
        end,

        anim = "taunt2",
        target_type = TARGET_TYPE.SELF,

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        num_draw = 3,

        OnPostResolve = function( self, battle, attack)
            battle:DrawCards( self.num_draw )
        end,
    },

    improvise_power = 
    {
        name = "Enrage",
        icon = "battle/improvised_power.tex",
        flavour = "'Now you've gone and made this personal!'",
        cost = 1,
        max_xp = 0,
        
        anim = "taunt3",
        target_type = TARGET_TYPE.SELF,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        features = 
        {
            POWER = 1
        },
    },

    improvise_power_upgraded = 
    {
        name = "Pale Enrage",
        icon = "battle/improvised_power.tex",
        cost = 0,

        anim = "taunt3",
        target_type = TARGET_TYPE.SELF,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        features = 
        {
            POWER = 1
        },
    },

    improvise_discard =
    {
        name = "Scrapper",
        desc = "Draw a card, discard a card.",
        flavour = "'Getting rid of the wrong thing is almost as good as getting the right thing.'",
        cost = 0,
        max_xp = 0,
        
        anim = "taunt2",
        target_type = TARGET_TYPE.SELF,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        discard_amount = 1,
        draw_count = 1,

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(self.draw_count)
            battle:DiscardCards(self.discard_amount, nil, self)
        end
    },

    improvise_discard_upgraded =
    {
        name = "Boosted Scrapper",
        icon = "battle/improvise_discard.tex",
        desc = "Draw 2 cards, discard 2 cards.",
        cost = 0,
        
        anim = "taunt2",
        target_type = TARGET_TYPE.SELF,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        discard_amount = 2,
        draw_count = 2,

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(self.draw_count)
            battle:DiscardCards(self.discard_amount, nil, self)
        end
    },

    muscle_memory =
    {
        name = "Muscle Memory",
        desc = "{IMPROVISE} a card from your draw pile. Duplicate it.",
        flavour = "'Practice makes perfect. Or at least good enough.'",
        anim = "taunt2",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 2,
        count = 1,

        active = false,

        OnPostResolve = function( self, battle, attack)
            self.active = true
            local cards = {}
            for i, card in battle:GetDrawDeck():Cards() do
                table.insert(cards, card)
            end
            if #cards == 0 then
                battle:ShuffleDiscardToDraw()
                for i, card in battle:GetDrawDeck():Cards() do
                    table.insert(cards, card)
                end
            end

            local chosen_cards = battle:ImproviseCards( table.multipick(cards, 3), self.count, "off_hand", nil, nil, self )
            if chosen_cards[1] then
                local copy = chosen_cards[1]:Duplicate()
                copy.show_dealt = false

                battle:DealCard( copy, battle:GetHandDeck())
            end
        end,
    },

    muscle_memory_plus =
    {
        name = "Pale Muscle Memory",
        cost = 1,
    },

    muscle_memory_plus2 =
    {
        name = "Graven Muscle Memory",
        desc = "{IMPROVISE} a card from your <#UPGRADE>discard</> pile. Duplicate it.",

        OnPostResolve = function( self, battle, attack)
            self.active = true
            local cards = {}
            for i, card in battle:GetDiscardDeck():Cards() do
                table.insert(cards, card)
            end

            local chosen_cards = battle:ImproviseCards( table.multipick(cards, 3), self.count, nil, nil, nil, self )
            if chosen_cards[1] then
                local copy = chosen_cards[1]:Duplicate()
                copy.show_dealt = false

                battle:DealCard( copy, battle:GetHandDeck())
            end
        end,
    },

    into_the_night =
    {
        name = "Into the Night",
		desc = "{ABILITY}: Whenever you discard a card, gain 1 {DEFEND}.",
		on_play_screen_shake = true,
        cost = 1,
        anim = "taunt",
        flavour = "'Thugs and bandits quickly took advantage of how dark the roads get in Havaria.'",
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        target_type = TARGET_TYPE.SELF,
        defend_amount = 1,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition("into_the_night", self.defend_amount, self)
        end,

        condition = 
        {
            desc = "Whenever you discard a card, gain <#HILITE>{1} {DEFEND}</>.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks)
            end,
            event_handlers = 
            {
                [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                    if card.owner == self.owner then
                        self.owner:AddCondition("DEFEND", self.stacks, self)
                    end
                end,
            },
        },
    },
    into_the_night_plus = 
    {
        name = "Stone Into the Night",
        desc = "{ABILITY}: Whenever you discard a card, gain <#UPGRADE>2 {DEFEND}</>.",
        defend_amount = 2,
    },
     
    into_the_night_plus2 = 
    {
        name = "Initial Into the Night",
        desc = "{ABILITY}: Whenever you discard a card, gain 1 {DEFEND}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    switch_blade =
    {
        name = "Switch Blade",
        desc = "Add a copy of this card to your discard.\nSpend {1} {COMBO}: Draw a card.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.combo_cost)
        end,

        anim = "double_stab",
        flavour = "'It just keeps coming.'",

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.COMBO_FINISHER,
        min_damage = 2,
        max_damage = 2,
        cost = 0,
        wound_chance = 0,
        combo_cost = 2,

        PreReq = function( self, minigame )
            return self.owner:GetConditionStacks("COMBO") >= self.combo_cost
        end,

        OnPostResolve = function( self, battle, attack )
            local copy = self:Duplicate()
            copy.show_dealt = false
            battle:DealCard( copy, battle:GetDiscardDeck() )

            if self.owner:GetConditionStacks("COMBO") >= self.combo_cost then
                self.owner:RemoveCondition("COMBO", self.combo_cost)

                battle:DrawCards( 1 )
            end

            if math.random() < self.wound_chance then
                for i, hit in attack:Hits() do
                    if not hit.evaded then
                        hit.target:AddCondition("WOUND", 1, self)
                    end
                end
            end

        end,

    },
    switch_blade_plus =
    {
        name = "Barbed Switch Blade",
        desc = "Add a copy of this card to your discard.\n<#UPGRADE>50% chance of applying {WOUND 1}.</>\nSpend {1} {COMBO}: Draw a card.",

        wound_chance = 0.5,
    },
    switch_blade_plus2 =
    {
        name = "Boosted Switch Blade",

        min_damage = 4,
        max_damage = 4,
    },


    backstab =
    {
        name = "Backstab",
        desc = "If this card was {IMPROVISE|Improvised}, double its damage until the end of the turn.",

        anim = "backstab",
        flavour = "'Nobody ever said it was fair, they just said that it works.'",

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MELEE,
        min_damage = 3,
        max_damage = 5,
        cost = 1,
        multiplier = 2,

        deck_handlers = ALL_DECKS,

        event_priorities =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = EVENT_PRIORITY_MULTIPLIER,
        },

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                if card == self then
                    self.damage_mult = self.multiplier
                end
            end,

            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self and self.damage_mult then
                    dmgt:AddDamage( dmgt.min_damage * (self.damage_mult - 1), dmgt.max_damage * (self.damage_mult - 1), self )
                end
            end,

            [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                self.damage_mult = nil
            end,
        }
    },
    backstab_plus =
    {
        name = "Boosted Backstab",
        min_damage = 5,
        max_damage = 7,
    },
    backstab_plus2 =
    {
        name = "Enhanced Backstab",
        desc = "If this card was {IMPROVISE|Improvised}, <#UPGRADE>triple</> its damage until the end of the turn.",
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,
        multiplier = 3,
    },

    hemorrhage =
    {
        name = "Hemorrhage",
		desc = "Apply {BLEED {1}}, then deal damage equal to target's {BLEED}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.bleed_amt)
        end,

        anim = "hemorrhage",
        flavour = "'I can smell it, it's almost over.'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,
        bleed_amt = 3,

        OnPostResolve = function( self, battle, attack)
            for i, hit in attack:Hits() do
                if not attack:CheckHitResult( hit.target, "evaded" ) then
                    hit.target:AddCondition("BLEED", self.bleed_amt, self)
                    hit.target:ApplyDamage( hit.target:GetConditionStacks("BLEED"), attack.attacker, hit, self.piercing )
                end
            end
        end,
    },
    hemorrhage_plus =
    {
        name = "Savage Hemorrhage",
        desc = "Apply <#UPGRADE>{BLEED {1}}</>, then deal damage equal to target's {BLEED}.",
        bleed_amt = 5,
    },
    hemorrhage_plus2 =
    {
        name = "Piercing Hemorrhage",
        desc = "Apply {BLEED {1}}, then deal <#UPGRADE>{PIERCING}</> damage equal to target's {BLEED}.",
        piercing = true,
    },

    burst =
    {
        name = "Burst",
        desc = "Apply {BLEED} equal to damage dealt by this card.",
        anim = "slash",
        flavour = "'The claws ripped right through his armor like it wasn't even there.'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,
        cost = 2,

        min_damage = 3,
        max_damage = 6,

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if hit.card == self and not hit.evaded then
                    hit.target:AddCondition("BLEED", hit.damage or 0, self)
                end
            end
        },

    },

    burst_plus = 
    {
        name = "Rooted Burst",
        min_damage = 5,
        max_damage = 5,
    },

    burst_plus2 =
    {
        name = "Tall Burst",
        max_damage = 9,
    },

    barnacle =
    {
        name = "Barnacle",
        desc = "Apply {1} {BLEED}, then gain {DEFEND} equal to target's {BLEED}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.bleed_amt)
        end,

        anim = "lacerate",
        flavour = "'One of the more unconventional blood strikes, this attack strengthens ones durability.'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 2,
        bleed_amt = 3,

        OnPostResolve = function( self, battle, attack)
            for i, hit in attack:Hits() do
                if not attack:CheckHitResult( hit.target, "evaded" ) then
                    hit.target:AddCondition("BLEED", self.bleed_amt, self)
                end
                self.owner:AddCondition("DEFEND", hit.target:GetConditionStacks("BLEED"), self)
            end
        end,
    },

    barnacle_plus = 
    {
        name = "Savage Barnacle",
        desc = "Apply <#UPGRADE>{1} {BLEED}</>, then gain {DEFEND} equal to target's {BLEED}.",
        bleed_amt = 5,
    },

    barnacle_plus2 = 
    {
        name = "Pale Barnacle",
        cost = 1,
    },

    slippery =
    {
        name = "Slippery",
        desc = "{IMPROVISE} a card from your discard pile.\nGain {1} {DEFEND}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amt ))
        end,

        anim = "taunt",
        flavour = "'Hold still so I can kill you!'",
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.COMMON,

        cost = 1,

        defend_amt = 4,

        OnPreResolve = function( self, battle, attack)
            local cards = battle:GetDiscardDeck():GetCards()
            cards = table.multipick(cards, 3)
            battle:ImproviseCards( cards, 1, "off_hand", nil, nil, self )
            attack:AddCondition("DEFEND", self.defend_amt, self)
        end,
    },
    slippery_plus =
    {
        name = "Stone Slippery",
        desc = "{IMPROVISE} a card from your discard pile.\nGain <#UPGRADE>{1} {DEFEND}</>.",

        defend_amt = 6,      
    },

    slippery_plus2 = 
    {
        name = "Boosted Slippery",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> a card from your discard pile.\nGain {1} {DEFEND}.",

        OnPreResolve = function( self, battle, attack)
            local cards = battle:GetDiscardDeck():GetCards()
            cards = table.multipick(cards, 5)
            battle:ImproviseCards( cards, 1, "off_hand", nil, nil, self )
            attack:AddCondition("DEFEND", self.defend_amt, self)
        end,
    },

    deception =
    {
        name = "Deception",
        desc = "{IMPROVISE} an attack card from your draw pile. That card deals +{1} damage until played.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.bonus_damage )
        end,

        anim = "taunt2",
        flavour = "'That's why I don't take prisoners, you just can't trust anyone.'",
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.COMMON,
        cost = 1,
        show = 3,
        bonus_damage = 4,

        condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card and type(card.deception_bonus) == "number" then
                        dmgt:AddDamage( card.deception_bonus, card.deception_bonus, self )
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function (self, battle, attack)
                    if attack.card and attack.card.deception_bonus ~= nil then
                        attack.card.deception_bonus = nil
                    end
                end,
            },
        },

        OnPostResolve = function( self, battle, attack)
            local cards = {}
            for i, card in battle:GetDrawDeck():Cards() do
                if card:IsAttackCard() then
                    table.insert(cards, card)
                end
            end
            if #cards == 0 then
                battle:ShuffleDiscardToDraw()
                for i, card in battle:GetDrawDeck():Cards() do
                    if card:IsAttackCard() then
                        table.insert(cards, card)
                    end
                end
            end

            cards = table.multipick( cards, self.show )
            local chosen_card = battle:ImproviseCards( cards, 1, "off_hand", nil, nil, self )[1]
            if chosen_card then
                local con = self.owner:AddCondition("deception", 1, self)
                if con then
                    chosen_card.deception_bonus = (chosen_card.deception_bonus or 0) + self.bonus_damage
                end
            end
        end,
    },

    deception_plus =
    {
        name = "Tall Deception",
        desc = "{IMPROVISE} an attack card from your draw pile. That card deals <#UPGRADE>+{1}</> damage until played.",
        bonus_damage = 6,
    },

    deception_plus2 =
    {
        name = "Boosted Deception",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> an attack card from your draw pile. That card deals +{1} damage until played.",
        show = 5,
    },

    fightdirty =
    {
        name = "Fight Dirty",
        anim = "taunt",
        desc = "{IMPROVISE} a card from a pool of special cards.",
        flavour = "'I fight to win, not to make friends.'",
        target_type = TARGET_TYPE.SELF,

        rarity = CARD_RARITY.BASIC,
        flags = CARD_FLAGS.SKILL,
        cost = 1,
        has_checked = false,

        pool_size = 3,

        pool_cards = {"improvise_dust", "improvise_hide", "improvise_trip", "improvise_slash", "improvise_draw", "improvise_power", "improvise_discard", "improvise_mud"},

        OnPostResolve = function( self, battle, attack)
            local cards = ObtainWorkTable()

            cards = table.multipick( self.pool_cards, self.pool_size )
            for k,id in pairs(cards) do
                cards[k] = Battle.Card( id, self.owner  )
            end
            battle:ImproviseCards( cards, 1, nil, nil, nil, self )
            ReleaseWorkTable(cards)
        end,
    },

    fightdirty_plus =
    {
        name = "Promoted Fight Dirty",
        desc = "{IMPROVISE} a card from a pool of <#UPGRADE>upgraded</> special cards.",
        pool_cards = {"improvise_dust_upgraded", "improvise_hide_upgraded", "improvise_trip_upgraded", "improvise_slash_upgraded", "improvise_draw_upgraded", "improvise_power_upgraded", "improvise_discard_upgraded", "improvise_mud_upgraded"},
    },

    fightdirty_plus2 = 
    {
        name = "Boosted Fight Dirty",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> a card from a pool of special cards.",
        pool_size = 5,
    },


    leech =
    {
        name = "Leech",
        anim = "lacerate",
        flavour = "'It's a little gruesome, but the worms can put all that mess to work for you.'",

        desc = "{HEAL} {1} health per {BLEED} on the target.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.heal_amt)
        end,

        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.COMMON,
        cost = 1,
        target_type = TARGET_TYPE.ENEMY,

        heal_amt = 1,

        OnPostResolve = function( self, battle, attack )
            for i,hit in ipairs(attack.hits) do
                if not hit.evaded then
                    local bleed = hit.target:GetConditionStacks( "BLEED" )
                    if bleed > 0 then
                        self.owner:HealHealth( bleed * self.heal_amt, self )
                    end
                end
            end
        end
    },

    leech_plus =
    {
        name = "Boosted Leech",
        desc = "{HEAL} <#UPGRADE>{1} health</> per {BLEED} on the target.",
        heal_amt = 2,
    },

    leech_plus2 =
    {
        name = "Pale Leech",
        cost = 0,
    },

    sals_daggers =
    {
        name = "Sal's Daggers",
        anim = "taunt",
        desc = "Insert {hammer_grip} or {saber_grip} into your hand.",
        flavour = "'This one's called Dag. The other one's Gurr.\nYeah? Well I don't like <b>your</> name, either, pal.'",

        rarity = CARD_RARITY.BASIC,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        cost = 1,

        sound = "event:/sfx/battle/status/special/sals_daggers",

        OnPostResolve = function( self, battle, attack)
            local cards = {
                Battle.Card( "hammer_grip", self.owner ),
                Battle.Card( "saber_grip", self.owner ),
            }
            battle:ChooseCardsForHand( cards, nil, nil, nil, nil, nil, self )
        end,
    },
 
    hammer_grip = 
    {
        name = "Hammer Grip",
        desc = "Deal 2 bonus damage if you have any {COMBO}.\nGain 2 {COMBO}.",
        anim = "hiltbash",
        flavour = "'It's a simple plan. I'm going to hit you, and then hit you again, but harder.'",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.MELEE,

        min_damage = 1,
        max_damage = 2,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if self == card and self.owner:HasCondition("COMBO") then
                    dmgt:AddDamage( 2, 2, self )
                end
            end,
        },        

        PreReq = function( self, minigame )
            return self.owner:HasCondition("COMBO")
        end,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition("COMBO", 2, self)
        end,
    },

    saber_grip = 
    {
        name = "Saber Grip",
        anim = "slash",
        flavour = "'You seem to be having trouble keeping your insides in, friend.'",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MELEE,

        min_damage = 2,
        max_damage = 2,

        features =
        {
            BLEED = 2,
        },
    },

    sals_daggers_plus = 
    {
        name = "Sal's Bloody Daggers",
        anim = "taunt",
        flavour = "'You don't need to know where to hit them if you go deep enough.'",

        desc = "Insert <#UPGRADE>{gouge}</> or <#UPGRADE>{vertical_slash}</> into your hand.",
        icon = "battle/sals_daggers_bleed.tex",

        rarity = CARD_RARITY.BASIC,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        cost = 1,

        OnPostResolve = function( self, battle, attack)
            local cards = {
                Battle.Card( "gouge", self.owner ),
                Battle.Card( "vertical_slash", self.owner ),
            }
            battle:ChooseCardsForHand( cards, nil, nil, nil, nil, nil, self )
        end,
    },

    gouge =
    {
        name = "Gouge",
        anim = "wildlunge",
        flavour = "'Once you're in, just give it a good twist. The blade does the rest.'",
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MELEE,

        min_damage = 3,
        max_damage = 3,

        features =
        {
            BLEED = 3,
        },
    },

    vertical_slash =
    {
        name = "Vertical Slash",
        anim = "uppercut",
        flavour = "'That's going to leave one Hesh of a cool scar if you make it through this.'",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MELEE,

        features =
        {
            BLEED = 5,
        },
    },
    
    sals_daggers_plus2 =
    {
        name = "Sal's Combo Daggers",
        anim = "taunt",
        desc = "Insert <#UPGRADE>{rain_of_blades}</> or <#UPGRADE>{gut_shot}</> into your hand.",
        icon = "battle/sals_daggers_combo.tex",

        rarity = CARD_RARITY.BASIC,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        cost = 1,

        OnPostResolve = function( self, battle, attack)
            local cards = {
                Battle.Card( "rain_of_blades", self.owner ),
                Battle.Card( "gut_shot", self.owner ),
            }
            battle:ChooseCardsForHand( cards, nil, nil, nil, nil, nil, self )
        end,
    },

    rain_of_blades = 
    {
        name = "Rain of Blades",
        desc = "Spend 2 {COMBO}: deals 2 bonus damage and hits all enemies instead.",
        anim = "targetpractice1",
        flavour = "'Where did they get all those knives?!'",
        cost = 0,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND | CARD_FLAGS.COMBO_FINISHER,

        min_damage = 3,
        max_damage = 3,

        PreReq = function( self, battle )
            if self.owner:GetConditionStacks("COMBO") >= 2 then
                self.target_mod = TARGET_MOD.TEAM
                return true
            else
                self.target_mod = TARGET_MOD.SINGLE
                return false
            end
        end,

        event_handlers =
        {
            [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                if card == self then
                    if self.owner:GetConditionStacks("COMBO") >= 2  then
                        self.target_mod = TARGET_MOD.TEAM
                        self.owner:RemoveCondition("COMBO", 2, self)
                        self.using_combo = true
                    else
                        self.target_mod = TARGET_MOD.SINGLE
                        self.using_combo = false
                    end
                end
            end,

            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self and (self.owner:GetConditionStacks("COMBO") >= 2 or self.using_combo) then
                    dmgt:AddDamage(2, 2, self)
                end
            end,

            [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if card == self and target_deck == self.engine:GetHandDeck() then
                    self.using_combo = false
                end
            end
        },
    },

    gut_shot = 
    {
        name = "Gut Shot",
        desc = "Attack twice.",
        anim = "elbow_strike",
        hit_anim = true,
        flavour = "'It seems that we aren't too different internally, the wind can always be knocked out of us.'",

        rarity = CARD_RARITY.UNIQUE,
        cost = 0,
        flags = CARD_FLAGS.MELEE,

        min_damage = 2,
        max_damage = 2,

        hit_count = 2,

        features =
        {
            COMBO = 2,
        },
    },

    stab =
    {
        name = "Stab",
        anim = "stab",
        flavour = "'Why do I have two daggers? Well, I have two hands, don't I?'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.BASIC,
        cost = 1,
        max_xp = 6,
        wild = true,

        min_damage = 2,
        max_damage = 5,
    },
    
    stab_plus2a =
    {
        name = "Stab of Clarity",
        min_damage = 10,
        max_damage = 10,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.CONSUME,    
    },

    stab_plus2b =
    {
        name = "Stab of the Stone",
        desc = "<#UPGRADE>Gain {1} {DEFEND}.</>",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amt ))
        end,

        defend_amt = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("DEFEND", self.defend_amt, self)
        end
    },

    -- stab_plus2c =
    -- {
    --     name = "Rival's Stab",
    --     desc = "<#UPGRADE>{COMBO 1}</>",
    --     features =
    --     {
    --         COMBO = 1,
    --     },

    -- },

    stab_plus2d =
    {
        name = "Stab of Savagery",
        desc = "<#UPGRADE>Apply 2 {BLEED}.</>",
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("BLEED", 2, self)
        end
    },

    stab_plus2e =
    {
        name = "Stab of the Deep",
        desc = "<#UPGRADE>Apply 1 {WOUND}.</>",
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("WOUND", 1, self)
        end

    },

    stab_plus2f =
    {
        name = "Stab of Vision",
        desc = "<#UPGRADE>Draw a card.</>",
        OnPostResolve = function( self, battle, attack)
            battle:DrawCards( 1 )
        end,
    },

    stab_plus2g =
    {
        name = "Stab of the Mirror",
        desc = "<#UPGRADE>Attack twice.</>",
        hit_anim = true,

        hit_count = 2,
        max_damage = 4,
    },

    -- stab_plus2h =
    -- {
    --     name = "Stab of the Spined",
    --     desc = "<#UPGRADE>{RIPOSTE 3}</>",
    --     features =
    --     {
    --         RIPOSTE = 3,
    --     },
    -- },

    stab_plus2j =
    {
        name = "Stab of the Nail",
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,
    },

    stab_plus2k =
    {
        name = "Rooted Stab",
        min_damage = 4,
    },

    stab_plus_2l =
    {
        name = "Strained Stab",
        desc = "<#UPGRADE>Discard a card.</>",

        min_damage = 4,
        max_damage = 6,

        OnPreResolve = function ( self, battle, attack )
            battle:DiscardCards(1, nil, self)
        end,
    },

    elbow_strike =
    {
        name = "Elbow Strike",
        anim = "elbow_strike",
        flavour = "'See, people always expect the daggers. That's why I hit them with the elbow sometimes.'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.BASIC,
        cost = 1,
        max_xp = 4,
        wild = true,
        min_damage = 3,
        max_damage = 3,
    },
    
    elbow_strike_plus2a =
    {
        name = "Strike of Clarity",
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.CONSUME,
        min_damage = 7,
        max_damage = 7,
                
    },
    
    elbow_strike_plus2b =
    {
        name = "Rival's Strike",
        desc = "<#UPGRADE>Gain 1 {COMBO}.</>",
        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("COMBO", 1, self)
        end    
    },

    elbow_strike_plus2c =
    {
        name = "Strike of Vision",
        desc = "<#UPGRADE>Draw a card.</>",
        OnPostResolve = function( self, battle, attack)
            battle:DrawCards( 1 )
        end,
                
    },

    elbow_strike_plus2d =
    {
        name = "Counter Strike",
        desc = "<#UPGRADE>Gain {RIPOSTE 3}.</>",
        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("RIPOSTE", 3, self)
        end  
                
    },


    elbow_strike_plus2e =
    {
        name = "Stone Strike",
        desc = "<#UPGRADE>Gain {1} {DEFEND}.</>",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amt ))
        end,

        defend_amt = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("DEFEND", self.defend_amt, self)
        end  
                
    },

    elbow_strike_plus2f =
    {
        name = "Lucid Strike",
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,
        
        min_damage = 5,
        max_damage = 5,      
    },

    elbow_strike_plus2g =
    {
        name = "Strained Strike",
        desc = "<#UPGRADE>Discard a card.</>",

        min_damage = 4,
        max_damage = 4,

        OnPreResolve = function ( self, battle, attack )
            battle:DiscardCards(1, nil, self)
        end,
    },

    elbow_strike_plus2h = 
    {
        name = "Finishing Strike",
        desc = "<#UPGRADE>Spend up to {1} {COMBO}: Deal {2} bonus damage per {COMBO} spent.</>",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.combo_limit, self.combo_bonus )
        end,

        flags = CARD_FLAGS.MELEE | CARD_FLAGS.COMBO_FINISHER,

        combo_bonus = 2,
        combo_limit = 3,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if self == card then
                    local combo = self.combo_consumed or  math.min(self.owner:GetConditionStacks( "COMBO" ), self.combo_limit)
                    dmgt:AddDamage( self.combo_bonus * combo, self.combo_bonus * combo, self )
                end
            end,
        },        

        PreReq = function( self, minigame )
            return self.owner:GetConditionStacks("COMBO") > 0
        end,

        OnPreResolve = function( self, battle, attack)
            if self.owner:HasCondition("COMBO") then
                local combo = math.min(self.owner:GetConditionStacks("COMBO"), self.combo_limit)
                self.owner:RemoveCondition("COMBO", combo, self)
                self.combo_consumed = combo
            end
        end,

        OnPostResolve = function( self, battle, attack )
            self.combo_consumed= nil
        end,
    },
                

    jab =
    {
        name = "Hilt Bash",
        anim = "hiltbash",
        flavour = "'Be sure to use every part of the weapon, that's just good value.'",
        loc_strings =
        {
            GAIN_COMBO = "(Gain {COMBO {1}})",
        },
        desc = "Gain {COMBO 1} for every card in your hand.\n{1}",
        desc_fn = function( self, fmt_str )
            if self.engine then
                local str = loc.format( self.def:GetLocalizedString( "GAIN_COMBO" ), self:CalcCombo(false) )
                return loc.format( fmt_str, str )
            else
                return loc.format( fmt_str, "" )
            end
        end,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.RARE,
        cost = 1,

        min_damage = 2,
        max_damage = 3,
        bonus = 0,

        CalcCombo = function(self, count_self)
            local num_cards = count_self and self.engine:GetHandDeck():CountCards() + 1 or self.engine:GetHandDeck():CountCards()
            return math.floor( num_cards ) + self.bonus
        end,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("COMBO", self:CalcCombo(true), self)
        end,
    },

    jab_plus = 
    {
        name = "Rival's Hilt Bash",
        desc = "Gain {COMBO 1} for every card in your hand <#UPGRADE>+ 2.</>\n{1}",
        bonus = 2,
        
    },

    jab_plus2 =
    {
        name = "Boosted Hilt Bash",
        min_damage = 4,
        max_damage = 6,
    },

    slice =
    {
        name = "Slice",
        anim = "attack1",
        desc = "Apply {BLEED} for each card played this turn.\n{1}",
        flavour = "'The cuts were so clean they didn't even realize anything was wrong until it was too late.'",

        desc_fn = function( self, fmt_str )
            if self.engine then
                local str = loc.format( LOC"CARD_ENGINE.CARDS_PLAYED", self.engine:CountCardsPlayed() )
                return loc.format( fmt_str, str )
            else
                return loc.format( fmt_str, "" )
            end
        end,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 1,

        min_damage = 2,
        max_damage = 3,
        bonus = 0,

        OnPostResolve = function( self, battle, attack )
            local stacks = battle:CountCardsPlayed() + self.bonus
            if stacks > 0 then
                attack:AddCondition( "BLEED", stacks, self )
            end
        end,
    },

    slice_plus =
    {
        name = "Boosted Slice",
        desc = "Apply {BLEED} for each card played this turn <#UPGRADE> + 1</>.\n{1}",
        
        bonus = 1
    },

    slice_plus2 =
    {
        name = "Promoted Slice",
        desc = "Apply {BLEED} for each card played this turn.\n<#UPGRADE>If the target has {BLEED 5} or more: Apply {IMPAIR 2}.</>\n{1}",

        PreReq = function( self, minigame )
            return self:GetTarget() and self:GetTarget():GetConditionStacks("BLEED") >= 5
        end,

        OnPostResolve = function( self, battle, attack )
            local stacks = battle:CountCardsPlayed()
            if stacks > 0 then
                attack:AddCondition( "BLEED", stacks, self )
            end
            for i, hit in attack:Hits() do
                if hit.target:GetConditionStacks("BLEED") >= 5 then
                    hit.target:AddCondition("IMPAIR", 2, self)
                end
            end
            
        end,
    },

    cross =
    {
        name = "Cross",
        desc = "Gain {COMBO} equal to damage dealt by this card.",
        anim = "uppercut",
        flavour = "'Sure it's slow, but the follow up is amazing.'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,
        cost = 2,

        min_damage = 3,
        max_damage = 6,

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if hit.card == self then
                    self.owner:AddCondition("COMBO", hit.damage or 0, self)
                end
            end
        },
    },

    cross_plus =
    {
        name = "Rooted Cross",
        min_damage = 6,
    },

    cross_plus2 = 
    {
        name = "Tall Cross",
        max_damage = 8,
    },

    combination =
    {
        name = "Combination",
        anim = "double_stab",
        flavour = "'I couldn't even follow the movements of the knife, it's like she was holding five of them or something.'",
        desc = "Deal 3 bonus damage if you have any {COMBO}.\nGain {COMBO {1}}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.combo_amt)
        end,

        cost = 1,
        min_damage = 2,
        max_damage = 4,
        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MELEE,

        combo_amt = 2,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if self == card and self.owner:HasCondition("COMBO") then
                    dmgt:AddDamage( 3, 3, self )
                end
            end,
        },

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("COMBO", self.combo_amt, self)
        end
    },
    combination_plus =
    {
        name = "Rival's Combination",
        desc = "Deal 3 bonus damage if you have any {COMBO}.\nGain <#UPGRADE>{COMBO {1}}</>.",

        combo_amt = 3,
    },
    combination_plus2 =
    {
        name = "Boosted Combination",
        min_damage = 4,
        max_damage = 6,
    },

    bolo_strike =
    {
        name = "Shoulder Roll",
        anim = "taunt",
        flavour = "'Hey you can't bring a Shroke in here to fight, I'll break my hand!'",
        desc = "Spend all your {COMBO}: Gain {1} {DEFEND} per {COMBO}. Regain your spent {COMBO} next turn.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.defend_amt )
        end,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.COMBO_FINISHER,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.SELF,

        cost = 1,

        bonus_combo = 0,
        defend_amt = 2,

        OnPreResolve = function( self, battle, attack)
            local combo = self.owner:GetConditionStacks( "COMBO" )
            self.owner:AddCondition( "DEFEND", combo * self.defend_amt, self )
            self.owner:RemoveCondition( "COMBO" )
            self.owner:AddCondition("COMBO_CARRY", combo + self.bonus_combo, self)
        end,
    },

    bolo_strike_plus = 
    {
        name = "Stone Roll",
        desc = "Spend all your {COMBO}: Gain <#UPGRADE>{1} {DEFEND}</> per {COMBO}. Regain your spent {COMBO} next turn.",
        defend_amt = 3,
    },

    bolo_strike_plus2 =
    {
        name = "Rival's Roll",
        desc = "Spend all your {COMBO}: Gain {1} {DEFEND} per {COMBO}. Regain your spent <#UPGRADE>{COMBO} + 2</> next turn.",
        bonus_combo = 2,
    },

    chamber = 
    {
        name = "Chamber",
        anim = "taunt3",
        flavour = "'Murder Bay Boxing has come a long way. You could almost say the fighters actually know what they are doing.'",
        desc = "Gain {1} {DEFEND}. At the start of your next turn gain {2} {COMBO}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amount ), self.combo_amt )
        end,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.COMMON,
        target_type = TARGET_TYPE.SELF,

        cost = 1,

        defend_amount = 4,
        combo_amt = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("DEFEND", self.defend_amount, self)
            self.owner:AddCondition("COMBO_CARRY", self.combo_amt, self)
        end,
    },

    chamber_plus = 
    {
        name = "Stone Chamber",
        desc = "Gain <#UPGRADE>{1} {DEFEND}</>, at the start of your next turn gain {2} {COMBO}.",
        defend_amount = 6,
    },

    chamber_plus2 = 
    {
        name = "Rival's Chamber",
        desc = "Gain {1} {DEFEND}, at the start of your next turn gain <#UPGRADE>{2} {COMBO}</>.",
        combo_amt = 3,
    },

    freighter =
    {
        name = "Freighter",
        anim = "uppercut",
        flavour = "'Sometimes it just takes determination.'",
        desc = "Deal bonus damage equal to twice the cost of your hand.",

        rarity = CARD_RARITY.COMMON,

        cost = 2,
        min_damage = 2,
        max_damage = 2,
        flags = CARD_FLAGS.MELEE,

        GetCardCost = function( self )
            local cost = 0
            if self.engine then
                for i, hand_card in self.engine:GetHandDeck():Cards() do
                    cost = cost + self.engine:CalculateActionCost( hand_card )
                end
                if self.deck ~= self.engine:GetHandDeck() then
                    cost = cost + self.engine:CalculateActionCost(self)
                end
            end
            return cost
        end,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if self == card then
                    local cost = self:GetCardCost() * 2
                    dmgt:AddDamage( cost, cost, self )
                end
            end,
        },        

        PreReq = function( self, battle )
            return self:GetCardCost() > self.cost
        end,
    },
    freighter_plus =
    {
        name = "Restored Freighter",
        desc = "Deal bonus damage equal to twice the cost of your hand.",

        flags = CARD_FLAGS.MELEE | CARD_FLAGS.REPLENISH,
    },
    freighter_plus2 =
    {
        name = "Rooted Freighter",
        min_damage = 5,
        max_damage = 5,
    },

    haymaker =
    {
        name = "Haymaker",
        anim = "uppercut",
        flavour = "'Sound the damn bell! It's over!'",
        desc = "Spend all your {COMBO}: Deal {2} bonus damage per {COMBO}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.combo_limit, self.bonus_damage )
        end,

        rarity = CARD_RARITY.UNCOMMON,

        cost = 2,
        min_damage = 3,
        max_damage = 5,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.COMBO_FINISHER,
        bonus_damage = 3,

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if self == card then
                    local combo = self.combo_consumed or self.owner:GetConditionStacks( "COMBO" )
                    dmgt:AddDamage( self.bonus_damage * combo, self.bonus_damage * combo, self )
                end
            end,
        },        

        PreReq = function( self, minigame )
            return self.owner:GetConditionStacks("COMBO") > 0
        end,

        OnPreResolve = function( self, battle, attack )
            local combo = self.owner:GetConditionStacks("COMBO")
            if combo > 0 then 
                self.combo_consumed = combo
                self.owner:RemoveCondition("COMBO", combo, self)
            end
        end,

        OnPostResolve = function( self, battle, attack)
            self.combo_consumed = nil
        end,
    },
    haymaker_plus =
    {
        name = "Boosted Haymaker",
        desc = "Spend all your {COMBO}: Deal <#UPGRADE>{2} bonus damage</> per {COMBO}.",
        bonus_damage = 4,
    },
    haymaker_plus2 =
    {
        name = "Rival's Haymaker",
        desc = "Spend all your {COMBO}: Deal {2} bonus damage per {COMBO}.\n<#UPGRADE>Gain {COMBO 2}.</>",
        OnPostResolve = function ( self, battle, attack )
            self.combo_consumed = nil
            self.owner:AddCondition("COMBO", 2, self)
        end
    },

    lacerate =
    {
        name = "Lacerate",
        desc = "Apply {BLEED 4}.",

        anim = "lacerate",
        flavour = "'You shouldn't be playing with knives.'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,
        cost = 1,

        min_damage = 2,
        max_damage = 4,
        bleed_amt = 4,
        
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("BLEED", self.bleed_amt, self)
        end,
    },

    lacerate_plus =
    {
        name = "Boosted Lacerate",
        desc = "Apply <#UPGRADE>{BLEED 5}</>.",
        bleed_amt = 5,
    },

    lacerate_plus2 =
    {
        name = "Savage Lacerate",
        desc = "<#UPGRADE>If the target has {BLEED}, apply {1} {BLEED}.</>",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.bleed_amt)
        end,

        bleed_amt = 6,

        PreReq = function( self, minigame )
            return self:GetTarget() and self:GetTarget():HasCondition("BLEED")
        end,

        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                if hit.target:HasCondition("BLEED") and not hit.evaded then
                    hit.target:AddCondition("BLEED", self.bleed_amt, self)
                end
            end
        end,

        features = {},
    },

    inside_fighting =
    {
        name = "Inside Fighting",
        anim = "taunt3",
        flavour = "'Make some space! She has you right where she wants you!'",
        desc = "{ABILITY}: Whenever you attack, gain {COMBO 1}.",
        rarity = CARD_RARITY.RARE,
        on_play_screen_shake = true,

        cost = 2,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "inside_fighting", 1, self )
        end,

        condition =
        {
            desc = "Whenever you attack, gain <#HILITE>{COMBO {1}}</>.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks or 1)
            end,
            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.attacker == self.owner and attack.card:IsAttackCard() then
                        self.owner:AddCondition( "COMBO", self.stacks, self)
                    end
                end,
            }
        }
    },

    inside_fighting_plus =
    {
        name = "Initial Inside Fighting",
        desc = "{ABILITY}: Whenever you attack, gain {COMBO 1}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
        on_play_screen_shake = FIGHT_SHAKE_ABILITY,
    },

    inside_fighting_plus2 = 
    {
        
        name = "Broad Inside Fighting",
        desc = "<#UPGRADE>Gain {COMBO 2}.</>\n{ABILITY}: Whenever you attack, gain {COMBO 1}.",
        on_play_screen_shake = { 0.3, 60, 0.2 },

        OnPostResolve = function ( self, battle, attack )
            self.owner:AddCondition("COMBO", 2, self)
            self.owner:AddCondition( "inside_fighting", 1, self )
        end
    },

    efficient_disposal =
    {
        name = "Efficient Disposal",
        desc = "{ABILITY}: Whenever you discard a card, deal 2 damage to a random enemy.",
        on_play_screen_shake = true,
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.RARE,
        cost = 2,
        flavour = "'Lumin cells and other batteries can be recycled in the gray bins, or you can just hurl them at someone.'",

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        condition =
        {
            desc = "Whenever you discard a card, deal <#HILITE>{1}</> damage to a random enemy",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,

            apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

            Trigger = function( self, battle )
                local target_fighters = {}
                battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )

                self.owner:BroadcastEvent( BATTLE_EVENT.CUSTOM_BEGIN, self )
                self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "targetpractice1", true )

                for i=1, #target_fighters do
                    target_fighters[i]:ApplyDamage( self.stacks, self.owner, self, nil, { "discard" } )
                end
                self.owner:BroadcastEvent( BATTLE_EVENT.CUSTOM_END, self )
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                    self:Trigger( battle )
                end,
            },
        },

        OnPostResolve = function( self, battle, attack )
            local con = self.owner:AddCondition( "efficient_disposal", 2, self )
            if con then
                con.ignore_card = self
            end
        end,
    },
    
    efficient_disposal_plus2 =
    {
        name = "Pale Disposal",
        cost = 1,
    },

    efficient_disposal_plus =
    {
        name = "Serial Disposal",
        desc = "{ABILITY}: Whenever you <#UPGRADE>discard or {EXPEND}</> a card, deal {1} damage to a random enemy.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_amt)
        end,

        damage_amt = 2,

        condition =
        {
            desc = "Whenever you discard or {EXPEND} a card, deal <#HILITE>{1}</> damage to a random enemy.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,

            Trigger = function( self, battle )
                local target_fighters = {}
                battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1)

                self.owner:BroadcastEvent( BATTLE_EVENT.CUSTOM_BEGIN, self )
                self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, "targetpractice1", true )

                for i=1, #target_fighters do
                    target_fighters[i]:ApplyDamage( self.stacks, self.owner, nil, nil, { "discard" } )
                end
                self.owner:BroadcastEvent( BATTLE_EVENT.CUSTOM_END, self )
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                    self:Trigger( battle )
                end,

                [ BATTLE_EVENT.CARD_EXPENDED ] = function( self, battle, card )
                    if card ~= self.ignore_card then
                        self:Trigger( battle )
                    end
                end,
            },
        },

        OnPostResolve = function( self, battle, attack )
            local con = self.owner:AddCondition( "efficient_disposal_plus", self.damage_amt, self )
            if con then
                con.ignore_card = self
            end
        end,
    },

    reckless_swing =
    {
        name = "Reckless Swing",
        desc = "Discard a card.",
        anim = "uppercut",
        flavour = "'Jarackle Free-Fighting is a martial art with such a loose structure that it appears that no one knows what they are doing.'",

        cost = 1,
        
        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MELEE,

        min_damage = 5,
        max_damage = 5,

        OnPreResolve = function ( self, battle, attack )
            battle:DiscardCards(1, nil, self)
        end,
    },

    reckless_swing_plus =
    {
        name = "Boosted Swing",
        min_damage = 8,
        max_damage = 8,
    },

    reckless_swing_plus2 =
    {
        name = "Focused Swing",
        desc = "<#UPGRADE>Draw a card.</>\nDiscard a card.",

        OnPreResolve = function ( self, battle, attack )
            battle:DrawCards( 1 )
            battle:DiscardCards(1, nil, self)
        end,
    },

    bedlam =
    {
        name = "Bedlam",
        desc = "Discard a card.\nAttack a random enemy.",
        anim = "double_stab",
        flavour = "'It's great to be outnumbered when there's a good chance you don't know who you'll be hitting.'",

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MELEE,

        min_damage = 8,
        max_damage = 8,

        target_mod = TARGET_MOD.RANDOM1,

        OnPreResolve = function( self, battle, attack )
            battle:DiscardCards(1, nil, self)
        end,
    },

    bedlam_plus = 
    {
        name = "Boosted Bedlam",
        min_damage = 12,
        max_damage = 12,
    },

    bedlam_plus2 = 
    {
        name = "Focused Bedlam",
        desc = "<#UPGRADE>Draw a card</>. Discard a card.\nAttack a random enemy.",

        OnPreResolve = function( self, battle, attack )
            battle:DrawCards(1)
            battle:DiscardCards(1, nil, self)
        end,
    },

    psionic_feedback =
    {
        name = "Psionic Feedback",
        desc = "If you have discarded a card this turn, this card costs 0.",
        anim = "attack1",
        flavour = "'Some individuals have been shown to exhibit strange abilities, I wouldn't exactly call them lucky or anything.'",

        cost = 2,
        
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE,

        min_damage = 3,
        max_damage = 5,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS },

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self and self.engine:CountCardsDiscarded() > 0 then
                    cost_acc:ModifyValue(0)
                end
            end
        },
 
    },
   psionic_feedback_plus =
    {
        name = "Boosted Feedback",
        min_damage = 5,
        max_damage = 7,
    },

    psionic_feedback_plus2 =
    {
        name = "Pale Feedback",
        cost = 1,
    },

    cynotrainer =
    {
        name = "Cynotrainer",
        desc = "Discard a card.\nDouble the damage on a random card in your hand this turn.",
        anim = "taunt2",
        flavour = "'Cybernetics and neuron enhancements give people an extra edge in battle. The rich ones anyways.'",

        mod_xp = -2,

        cost = 1,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.SKILL,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            battle:DiscardCards(1, nil, self)
            local hand = battle:GetHandDeck()
            
            local card = hand:PeekRandom( function( x ) return x ~= self and x:IsDamageCard() end )
            if card then
                local con = self.owner:GetCondition("cynotrainer")
                if con then
                    if con.buffed_cards then
                        table.insert(con.buffed_cards, card)
                    else
                        con.buffed_cards = {card}
                    end
                else
                    con = self.owner:AddCondition("cynotrainer", 1, self)
                    con.buffed_cards = {card}
                end
            end
        end,

        condition = 
        {
            hidden = true,

            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if self.buffed_cards and table.contains(self.buffed_cards, card) then
                        local count = 0
                        for i,buff_card in ipairs(self.buffed_cards) do
                            if buff_card == card then
                                count = count + 1
                            end
                        end
                        dmgt:ModifyDamage(math.floor(dmgt.min_damage * 2 ^ count) , math.floor(dmgt.max_damage * 2 ^ count), self)
                    end
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition("cynotrainer")
                end,
            },

        },
    },

    cynotrainer_plus =
    {
        name = "Visionary Cynotrainer",
        desc = "<#UPGRADE>Draw a card.</>\nDiscard a card.\nDouble the damage on a random card in your hand this turn.",
        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(1)
            battle:DiscardCards(1, nil, self)

            local hand = battle:GetHandDeck()
            local card = hand:PeekRandom( function( x ) return x ~= self and x:IsDamageCard() end )
            if card then
                local con = self.owner:GetCondition("cynotrainer")
                if con then
                    if con.buffed_cards then
                        table.insert(con.buffed_cards, card)
                    else
                        con.buffed_cards = {card}
                    end
                else
                    con = self.owner:AddCondition("cynotrainer", 1, self)
                    con.buffed_cards = {card}
                end
            end
        end,
    },

    cynotrainer_plus2 = 
    {
        name = "Focused Cynotrainer",
        desc = "Discard a card.\nDouble the damage on a <#UPGRADE>chosen</> card in your hand this turn.",

        OnPostResolve = function( self, battle, attack )
            battle:DiscardCards(1, nil, self)
            local hand = battle:GetHandDeck()

            local card = battle:ChooseCard( function(x) return x:IsDamageCard() end)
            if card and card:IsDamageCard() then
                local con = self.owner:GetCondition("cynotrainer") or self.owner:AddCondition("cynotrainer", 1, self)
                if con then
                    if con.buffed_cards then
                        table.insert(con.buffed_cards, card)
                    else
                        con.buffed_cards = {card}
                    end
                end
            end
        end,
    },
    
    stringer =
    {
        name = "Stringer",
        desc = "Gain {1} {DEFEND}. \nDiscard this card: Gain {1} {DEFEND} and {2} {RIPOSTE}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amount ), self.riposte_amount)
        end,

        anim = "taunt",
        flavour = "'Not bad, but try staying a little lower to the ground.'",

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.SKILL,

        target_type = TARGET_TYPE.SELF,

        cost = 1,

        defend_amount = 2,
        riposte_amount = 2,

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                if card == self then
            
                   self.owner:AddCondition("DEFEND", self.defend_amount, self)
                   self.owner:AddCondition("RIPOSTE", self.riposte_amount, self)
                   self:AddXP(1)
                end
            end,
        },

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition("DEFEND", self.defend_amount, self)
        end, 
    },

    stringer_plus =
    {
        name = "Boosted Stringer",
        desc = "Gain <#UPGRADE>{1} {DEFEND}.</> \nDiscard this card: Gain <#UPGRADE>{1} {DEFEND} </>and <#UPGRADE>{2} {RIPOSTE}.</>",
        defend_amount = 4,
        riposte_amount = 4,
    },
    stringer_plus2 =
    {
        name = "Pale Stringer",
        cost = 0,
    },

    spines = 
    {
        name = "Spines",
        anim = "taunt3",
        flavour = "'Now where the Hesh did those come from?'",
        desc = "{ABILITY}: Gain {1} {RIPOSTE} whenever you gain {DEFEND}.",
        on_play_screen_shake = true,
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.riposte_amount)
        end,

        cost = 2,

        riposte_amount = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("spines", self.riposte_amount, self)
        end,

        condition = 
        {
            name = "Spines",
            desc = "Gain <#HILITE>{1} {RIPOSTE}</> whenever you gain {DEFEND}.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks)
            end,

            event_handlers = 
            {
                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if fighter == self.owner and condition.id == "DEFEND" then
                        self.owner:AddCondition("RIPOSTE", self.stacks, self)
                    end
                end
            },
        },
    },

    spines_plus =
    {
        name = "Thorny Spines",
        desc = "{ABILITY}: Gain <#UPGRADE>{1}</> {RIPOSTE} whenever you gain {DEFEND}.",

        riposte_amount = 2,
    },

    spines_plus2 = 
    {
        name = "Pale Spines",
        cost = 1,
    },

    viciousness =
    {
        name = "Viciousness",
        desc = "For the rest of the turn, gain {RIPOSTE} equal to half of any attack damage you deal.",

        cost = 1,
        flavour = "'Lumin-laced banderclaws seem to bring out the worst in fighters.'",
        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("viciousness", 1, self)
        end,

        condition = 
        {
            name = "Viciousness",

            event_handlers = 
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.attacker == self.owner and (hit.damage or 0) > 0 then
                        self.owner:AddCondition("RIPOSTE", math.round(hit.damage*0.5), self)
                    end
                end,

                [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        self.owner:RemoveCondition(self.id, 1, self)
                    end
                end
            },
        },
    },

    viciousness_plus = 
    {
        name = "Pale Viciousness",
        cost = 0,
    },

    viciousness_plus2 =
    {
        name = "Enduring Viciousness",
        flags = CARD_FLAGS.SKILL,
    },

    seeker = 
    {
        name = "Seeker",
        desc = "Gain {1} {DEFEND}. \nDiscard this card: {HEAL} {2}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amount ), self.heal_amount )
        end,
        anim = "taunt",
        flavour = "'You can barely see the implant through the gloves, and it works like a charm.'",

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.SKILL,

        target_type = TARGET_TYPE.SELF,

        cost = 1,

        defend_amount = 6,
        heal_amount = 3,

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                if card == self then
                   self.owner:HealHealth(self.heal_amount, self)
                   self:AddXP(1)
                end
            end,
        },

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition("DEFEND", self.defend_amount, self)
        end, 
    },

    seeker_plus =
    {
        name = "Boosted Seeker",
        desc = "Gain <#UPGRADE>{1} {DEFEND}.</> \nDiscard this card: {HEAL} {2}.",
        defend_amount = 8,
    },
    seeker_plus2 =
    {
        name = "Pale Seeker",
        cost = 0,
    },

    sharpen = 
    {
        name = "Sharpen",
        desc = "Apply {BLEED {1}} whenever you hit an enemy with an attack for the rest of the turn.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, (self.stacks or 2) )
        end,

        anim = "taunt3",
        flavour = "'Well worth the time invested.'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 1,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        condition =
        {
            desc = "Apply <#HILITE>{BLEED {1}}</> whenever you hit an enemy with an attack for the rest of the turn.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, (self.stacks or 1) )
            end,
            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.card:IsAttackCard() and attack.attacker == self.owner then
                        if not hit.evaded then
                            hit.target:AddCondition( "BLEED", self.stacks, attack.card )
                        end
                    end
                end,

                [ battle_defs.BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition( "sharpen" )
                end,
            }
        },

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "sharpen", 2, self )
        end,
    },

    sharpen_plus = 
    {
        name = "Enduring Sharpen",
        flags = CARD_FLAGS.SKILL,
    },

    sharpen_plus2 = 
    {
        name = "Sharpen of Vision",
        desc = "Apply {BLEED {1}} whenever you hit an enemy with an attack for the rest of the turn.\n<#UPGRADE>Draw a card.</>",

         OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "sharpen", 2, self )
            battle:DrawCards( self.num_cards )
        end,
    },

    bloodbath = 
    {
        name = "Bloodbath",
        desc = "{ABILITY}: Whenever you apply {BLEED}, apply {BLEED {1}} to all other enemies.",
        on_play_screen_shake = true,
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.stacks or 1)
        end,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Other",

        anim = "taunt3",
        flavour = "'Just extend your range a little more and those blades will be biting into everyone!'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 2,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        condition =
        {
            desc = "Whenever you apply {BLEED}, apply <#HILITE>{BLEED {1}}</> to all other enemies.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks or 1)
            end,
            event_handlers =
            {
                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if condition:GetID() == "BLEED" then
                        if source and source.owner == self.owner and source ~= self then
                            local target_fighters = self.owner:GetEnemyTeam().fighters
                            for i,target in ipairs(target_fighters) do
                                if target ~= fighter then
                                    target:AddCondition("BLEED", self.stacks, self)
                                    self.battle:BroadcastEvent( BATTLE_EVENT.PLAY_ANIM, target, "hit_mid", false, true)
                                end
                            end
                        end
                    end
                end,
            }
        },

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "bloodbath", 1, self)
        end,
    },

    bloodbath_plus =
    {
        name = "Pale Bloodbath",
        cost = 1,
    },

    bloodbath_plus2 = 
    {
        name = "Initial Bloodbath",
        desc = "{ABILITY}: Whenever you apply {BLEED}, apply {BLEED {1}} to all other enemies.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    gash =
    {
        name = "Gash",
        anim = "gash",
        flavour = "'A bleeding target can get a whole lot worse in a matter of seconds.'",

		desc = "If the target has {BLEED}, apply {IMPAIR {1}} and {WOUND {2}}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.impair_per_bleed, self.wound_amount)
        end,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON,

        mod_xp = -2,

        min_damage = 2,
        max_damage = 3,
        cost = 1,
        impair_per_bleed = 1,
        wound_amount = 1,

        PreReq = function( self, battle )
            return self.target and self.target:HasCondition("BLEED")
        end,

        OnPostResolve = function( self, battle, attack)
            for i, hit in attack:Hits() do
                local target = hit.target
                if not hit.evaded and target:GetConditionStacks("BLEED") > 0 then 
                    target:AddCondition("IMPAIR", self.impair_per_bleed, self)
                    target:AddCondition("WOUND", self.wound_amount, self)
                end
            end
        end,
    },
    gash_plus =
    {
        name = "Enhanced Gash",
        desc = "If the target has {BLEED}, apply <#UPGRADE>{IMPAIR {1}}</> and <#UPGRADE>{WOUND {2}}</>.",
        impair_per_bleed = 2,
        wound_amount = 2,
    },
    gash_plus2 =
    {
        name = "Boosted Gash",
        min_damage = 4,
        max_damage = 6,
    },

    carve = 
    {
        name = "Carve",
        desc = "Spend up to {1} {COMBO}: Apply {WOUND 1} per {COMBO} spent.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.combo_limit)
        end,
        anim = "gash",
        flavour = "'Look, I'm not a doctor, but there's no way they're going to make it.'",

        flags = CARD_FLAGS.MELEE | CARD_FLAGS.COMBO_FINISHER,
        rarity = CARD_RARITY.UNCOMMON,

        combo_limit = 2,

        cost = 1,

        min_damage = 3,
        max_damage = 4,

        PreReq = function( self, minigame )
            return self.owner:GetConditionStacks("COMBO") > 0
        end,

        OnPreResolve = function( self, battle, attack )
            if self.owner:HasCondition("COMBO") then
                local count = math.min(self.owner:GetConditionStacks("COMBO"), self.combo_limit)
                self.owner:RemoveCondition("COMBO", count)
                self.combo_consumed = count    
            end
        end,
        
        OnPostResolve = function( self, battle, attack )
            if self.combo_consumed then
                for i, hit in attack:Hits() do
                    if not hit.evaded then
                        hit.target:AddCondition("WOUND", self.combo_consumed, self)
                    end
                end
                self.combo_consumed = nil
            end
        end
    },

    carve_plus =
    {
        name = "Tall Carve",
        desc = "Spend up to <#UPGRADE>{1} {COMBO}</>: Apply {WOUND 1} per {COMBO} spent.",
        combo_limit = 3,
    },

    carve_plus2 =
    {
        name = "Boosted Carve",
        min_damage = 4,
        max_damage = 6,
    },

    ravenous =
    {
        name = "Ravenous",
        desc = "{ABILITY}: Whenever an enemy dies, {HEAL} 8.",
        on_play_screen_shake = true,
        anim = "hero_taunt2",
        flavour = "'Hey, buddy, you gonna eat that?'",

        rarity = CARD_RARITY.RARE,
        cost = 2,

        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition("ravenous", 8, self)            
        end,

        condition =
        {
            desc = "Whenever an enemy dies, <#HILITE>{HEAL} {1}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,
            ctype = CTYPE.BUFF,
            event_handlers = 
            {
                [ BATTLE_EVENT.FIGHTER_KILLED ] = function( self, fighter )
                    if self.owner:GetTeam() ~= fighter:GetTeam() then
                        self.owner:HealHealth( self.stacks, self )
                    end
                end
            },
        },
    },

    ravenous_plus = 
    {
        name = "Initial Ravenous", 
        desc = "{ABILITY}: Whenever an enemy dies, {HEAL} 8.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,             
    },

    ravenous_plus2 =
    {
        name = "Pale Ravenous",
        cost = 1,
    },

    barbed_defense =
    {
        name = "Barbed Defense",
        desc = "{ABILITY}: Your {RIPOSTE} attacks apply {BLEED {1}}.",
        on_play_screen_shake = true,
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.bleed_amt)
        end,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Defend",

        anim = "taunt2",
        flavour = "'Nobody knows who designed the original razor-dust emitters but there have been many copycats.'",

        rarity = CARD_RARITY.UNCOMMON,
        cost = 1,

        bleed_amt = 2,

        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            local con = self.owner:AddCondition( "barbed_defense", self.bleed_amt, self )
        end,

        condition =
        {
            desc = "{RIPOSTE} attacks apply <#HILITE>{BLEED {1}}</>.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks)
            end,
            event_handlers = 
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.owner == self.owner and attack.is_counter and not hit.evaded then
                        hit.target:AddCondition( "BLEED", self.stacks, self )
                    end
                end,
            },
        },
    },

    barbed_defense_plus =
    {
        name = "Barbed Dawn",
        desc = "{ABILITY}: Your {RIPOSTE} attacks apply {BLEED {1}}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    barbed_defense_plus2 =
    {
        name = "Savage Defense",
        desc = "{ABILITY}: Your {RIPOSTE} attacks apply <#UPGRADE>{BLEED {1}}</>.",
        bleed_amt = 3,
    },

    readiness = 
    {
        name = "Readiness",
		desc = "Draw {1} cards.\nGain {RIPOSTE {2}}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.num_cards, self.riposte_amt )
        end,

        anim = "taunt",
        flavour = "'Plan? I never have a plan, things always just fall into place.'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.COMMON,
        cost = 1,
        num_cards = 3,
        flags = CARD_FLAGS.SKILL,

        riposte_amt = 2,

        OnPostResolve = function( self, battle, attack)
            battle:DrawCards( self.num_cards )
            self.owner:AddCondition("RIPOSTE", self.riposte_amt, self)
        end,
    },
    readiness_plus = 
    {
        name = "Boosted Readiness",
        desc = "Draw <#UPGRADE>{1} cards</>.\nGain {RIPOSTE {2}}.",
        num_cards = 4
    },
    readiness_plus2 = 
    {
        name = "Stone Readiness",
        desc = "Draw {1} cards.\nGain {RIPOSTE {2}}<#UPGRADE> and {DEFEND {3}}</>.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.num_cards, self.riposte_amt, self:CalculateDefendText(self.defend_amt) )
        end,

        defend_amt = 2,

        OnPostResolve = function( self, battle, attack)
            battle:DrawCards( self.num_cards )
            self.owner:AddCondition("RIPOSTE", self.riposte_amt, self)
            self.owner:AddCondition("DEFEND", self.defend_amt, self)
        end,
    },

    feint = 
    {
        name = "Feint",
        desc = "Apply {1} {DEFEND}.",

        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amount ))
        end,

        flavour = "'The best way to take a hit is to be in another place when it comes.'",
        
        anim = "taunt",
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,
        rarity = CARD_RARITY.BASIC,
        cost = 1,
        max_xp = 6,
        flags = CARD_FLAGS.SKILL,
        wild = true,
        defend_amount = 4,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition( "DEFEND", self.defend_amount, self )
        end
    },
        
    feint_plus = 
    {
        name = "Boosted Feint",
        desc = "<#UPGRADE>Apply {1} {DEFEND}.</>",        
        flags = CARD_FLAGS.SKILL,
        defend_amount = 6,
    },

    feint_plus2a = 
    {
        name = "Spined Feint",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>Gain 2 {RIPOSTE}.</>",
        flags = CARD_FLAGS.SKILL,
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition( "DEFEND", self.defend_amount, self )
            self.owner:AddCondition("RIPOSTE", 2, self)
        end
    },
    feint_plus2b = 
    {
        name = "Feint of Vision",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>Draw a card.</>",
        flags = CARD_FLAGS.SKILL,
        
        OnPostResolve = function( self, battle, attack)
            battle:DrawCards( 1 )
            attack:AddCondition( "DEFEND", self.defend_amount, self )
        end,
    },
    feint_plus2c = 
    {
        name = "Feint of Clarity",
        desc = "<#UPGRADE>Apply {1} {DEFEND}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.CONSUME,    
    
        defend_amount = 10,
    },
    feint_plus2d = 
    {
        name = "Lucid Feint",
        desc = "<#UPGRADE>Apply {1} {DEFEND}.",
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,    
  
        defend_amount = 7,
    },
    feint_plus2e = 
    {
        name = "Rival's Feint",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>Gain 1 {COMBO}.</>",
        flags = CARD_FLAGS.SKILL,
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition( "DEFEND", self.defend_amount, self )
            self.owner:AddCondition("COMBO", 1, self)
        end
    },
    feint_plus2f = 
    {
        name = "Following Feint",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>Your next attack this turn deals maximum damage.</>",
        flags = CARD_FLAGS.SKILL,
        
        condition = 
        {
            desc = "Your next attack this turn deals maximum damage.",
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if self.owner == card.owner then
                        dmgt:ModifyDamage( dmgt.max_damage, dmgt.max_damage, self )
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function (self, battle, attack)
                    if attack.card:IsAttackCard() then
                        self.owner:RemoveCondition("feint_plus2f", 1)
                    end
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition("feint_plus2f")
                end,
            },
        },

        OnPostResolve = function( self, battle, attack)
            local con = self.owner:AddCondition("feint_plus2f", 1, self)
            if con then
                attack.target:AddCondition( "DEFEND", self.defend_amount, self )
            end
        end,  
    },
    feint_plus2g = 
    {
        name = "Feint of Hesh",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>Remove a random debuff.</>",
        flags = CARD_FLAGS.SKILL,

        OnPostResolve = function( self, battle, attack )
            attack.target:AddCondition("DEFEND", self.defend_amount, self)
            attack.target:RemoveDebuff( self )
        end,
    },
    feint_plus2h = 
    {
        name = "Kra'deshi Feint",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>{HEAL} 4.</>",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack )
            attack.target:HealHealth(4, self)
            attack.target:AddCondition("DEFEND", self.defend_amount, self )
        end,
    },
    feint_plus2i = 
    {
        name = "Feint of the Healer",
        desc = "Apply {1} {DEFEND}.\n<#UPGRADE>{HEAL} 1.</>",
        flags = CARD_FLAGS.SKILL,

        OnPostResolve = function( self, battle, attack )
            attack.target:HealHealth(1, self)
            attack.target:AddCondition("DEFEND", self.defend_amount, self)
        end,
    },

    concentrate = 
    {
        name = "Concentrate",
        desc = "Draw a card. It costs 0 until played.",
        anim = "taunt2",
        flavour = "'Usually a little difficult to do, the more focused the combatant the better the results.'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 1,
        mod_xp = -1,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            local cards = battle:DrawCards( 1 )
            if cards[1] then
                cards[1]:SetFlags( CARD_FLAGS.FREEBIE )
            end
        end,
    },
    concentrate_plus =
    {
        name = "Enduring Concentrate",
        flags = CARD_FLAGS.SKILL,
    },

    concentrate_plus2 =
    {
        name = "Vision Concentrate",
        desc = "Draw <#UPGRADE>2</> cards. They cost 0 until played.",

        OnPostResolve = function( self, battle, attack)
            local cards = battle:DrawCards( 2 )
            for i = 1, math.min( #cards, 2 ) do
                cards[i]:SetFlags( CARD_FLAGS.FREEBIE )
            end
        end,
    },

    thieves_instinct = 
    {
        name = "Thieves' Instinct",
        desc = "{IMPROVISE} a random common or uncommon maneuver card. It costs 0 until played.",
        anim = "taunt",
        flavour = "'It was right there a second ago...'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 1,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        upgraded = false,

        OnPostResolve = function( self, battle, attack)
            
            local card_defs = BattleCardCollection():IsUnlocked():Filter(function(card_def) 
                                                                return self.id ~= card_def.id and 
                                                                        (card_def.rarity == CARD_RARITY.COMMON or card_def.rarity == CARD_RARITY.UNCOMMON) and
                                                                        AgentUtil.CanUseCard(self:GetOwner().agent, card_def) and 
                                                                        CheckBits( card_def.flags, CARD_FLAGS.SKILL ) and
                                                                        not CheckBits( card_def.flags, CARD_FLAGS.ITEM ) and 
                                                                        CheckBits( card_def.flags, CARD_FLAGS.UPGRADED ) == self.upgraded 
                                                            end):Pick(3)
            local cards = table.map(card_defs, 
                    function(card_def) 
                        local card = Battle.Card(card_def.id, self:GetOwner()) 
                        card.flags = SetBits( card.flags, CARD_FLAGS.FREEBIE )
                        card:ClearXP()
                        return card
                    end)
            battle:ImproviseCards( cards, nil, nil, nil, nil, self )
        end,
    },
    thieves_instinct_plus = 
    {
        name = "Boosted Instinct",
        desc = "{IMPROVISE} a random <#UPGRADE>uncommon</> or <#UPGRADE>rare</> maneuver card. It costs 0 until played.",
        OnPostResolve = function( self, battle, attack)
            local card_defs = BattleCardCollection():IsUnlocked():Filter(function(card_def) 
                                                                return self.id ~= card_def.id and 
                                                                        (card_def.rarity == CARD_RARITY.UNCOMMON or card_def.rarity == CARD_RARITY.RARE) and
                                                                        AgentUtil.CanUseCard(self:GetOwner().agent, card_def) and 
                                                                        CheckBits( card_def.flags, CARD_FLAGS.SKILL ) and
                                                                        not CheckBits( card_def.flags, CARD_FLAGS.ITEM ) and  
                                                                        CheckBits( card_def.flags, CARD_FLAGS.UPGRADED ) == self.upgraded 
                                                            end):Pick(3)
            local cards = table.map(card_defs, 
                    function(card_def) 
                        local card = Battle.Card(card_def.id, self:GetOwner()) 
                        card.flags = SetBits( card.flags, CARD_FLAGS.FREEBIE )
                        card:ClearXP()
                        return card
                    end)
            battle:ImproviseCards( cards, nil, nil, nil, nil, self )
        end,
    },

    thieves_instinct_plus2 = 
    {
        name = "Promoted Instinct",
        desc = "{IMPROVISE} a random <#UPGRADE>upgraded uncommon</> maneuver card. It costs 0 until played.",
        upgraded = true,
        OnPostResolve = function( self, battle, attack)
            local card_defs = BattleCardCollection():IsUnlocked():Filter(function(card_def) 
                                                                return self.id ~= card_def.id and 
                                                                        card_def.rarity == CARD_RARITY.UNCOMMON and
                                                                        AgentUtil.CanUseCard(self:GetOwner().agent, card_def) and 
                                                                        CheckBits( card_def.flags, CARD_FLAGS.SKILL ) and 
                                                                        not CheckBits( card_def.flags, CARD_FLAGS.ITEM ) and 
                                                                        CheckBits( card_def.flags, CARD_FLAGS.UPGRADED ) == self.upgraded 
                                                            end):Pick(3)
            local cards = table.map(card_defs, 
                    function(card_def) 
                        local card = Battle.Card(card_def.id, self:GetOwner()) 
                        card.flags = SetBits( card.flags, CARD_FLAGS.FREEBIE )
                        card:ClearXP()
                        return card
                    end)
            battle:ImproviseCards( cards, nil, nil, nil, nil, self )
        end,
    },

    footwork = 
    {
        name = "Footwork",
        desc = "Discard a card. Gain actions equal to its cost + 1.",
        anim = "taunt",
        flavour = "'Jarackle dance instructors tended to be the only ones standing after a street fight.'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 1,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,

        OnPostResolve = function( self, battle, attack)
            local cards = battle:DiscardCards(1, nil, self)
            if #cards > 0 then
                local discard_cost = battle:CalculateActionCost(cards[1])
                battle:ModifyActionCount( discard_cost + 1 )
            end
        end,
    },
    footwork_plus = 
    {
        name = "Boosted Footwork",
        desc = "Discard <#UPGRADE>any number</> of cards. Gain actions equal to their <#UPGRADE>combined cost</> + 1.",

        OnPostResolve = function( self, battle, attack)
            local cards = battle:DiscardCards(nil, nil, self)
            local discard_cost = 0
            for k,card in pairs(cards) do
                discard_cost = discard_cost + battle:CalculateActionCost(card)
            end
            battle:ModifyActionCount( discard_cost + 1)
        end,
    },
    footwork_plus2 =
    {
        name = "Visionary Footwork",
        desc = "Discard a card. Gain actions and <#UPGRADE>draw cards</> equal to its cost + 1.",

        OnPostResolve = function( self, battle, attack)
            local cards = battle:DiscardCards(1, nil, self)
            if #cards > 0 then
                local discard_cost = battle:CalculateActionCost(cards[1])
                battle:ModifyActionCount( discard_cost + 1)
                battle:DrawCards( discard_cost + 1)
            end
        end,
    },

    exertion =
    {
        name = "Exertion",
		desc = "Gain {1} actions.\nLose {2} {2*action|actions} next turn.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.action_gain, self.action_loss)
        end,

        anim = "taunt",
        flavour = "'People idle at about sixty percent, you can push it further.'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNCOMMON,
        cost = 0,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        action_gain = 2,
        action_loss = 1,
        draw_count = 0,

        OnPostResolve = function( self, battle, attack)
            battle:ModifyActionCount( self.action_gain )
            self.owner:AddCondition("exertion", self.action_loss, self)
            if self.draw_count > 0 then
                battle:DrawCards(self.draw_count)
            end
        end,

        condition =
        {
            hidden = true,

            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    battle:ModifyActionCount( -self.stacks )
                    self.owner:RemoveCondition("exertion")
                end
            },
        },
    },
    exertion_plus =
    {
        name = "Boosted Exertion",
        desc = "Gain <#UPGRADE>{1}</> actions.\nLose {2} {2*action|actions} next turn.",
        action_gain = 3,
    },
    exertion_plus2 =
    {
        name = "Visionary Exertion",
        desc = "Gain {1} actions <#UPGRADE>and draw a card</>.\nLose {2} {2*action|actions} next turn.",
        draw_count = 1,
    },

    uppercut =
    {
        name = "Uppercut",
		desc = "Gain {POWER {1}}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.power )
        end,
        anim = "uppercut",
        flavour = "'Glass jaw. Knew it.'",

        rarity = CARD_RARITY.UNCOMMON,
        flags =  CARD_FLAGS.MELEE,
        mod_xp = -1,

        min_damage = 2,
        max_damage = 5,
        cost = 2,
        power = 1,

        OnPostResolve = function(self, battle, attack)
            self.owner:AddCondition("POWER", self.power, self)
        end,
    },
    uppercut_plus = 
    {
        name = "Pale Uppercut",
        cost = 1,
    },
    uppercut_plus2 =
    {
        name = "Boosted Uppercut",
        desc = "Gain <#UPGRADE>{POWER {1}}.</>",
        power = 2,
    },

    silent_shiv =
    {
        name = "Silent Steel",

        anim  ="silentshiv",
        flavour = "'The noisy ones are scarier, but a lot easier to avoid.'",
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.AMBUSH | CARD_FLAGS.EXPEND,
        min_damage = 5,
        max_damage = 5,
        cost = 0,
    },
    silent_shiv_plus =
    {
        name = "Rooted Steel",
        min_damage = 8,
        max_damage = 8,
    },

    silent_shiv_plus2 =
    {
        name = "Deep Silent Steel",
        desc = "<#UPGRADE>Apply {IMPAIR 1}.</>",
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("IMPAIR", 1, self)
        end  
    },

    target_practice =
    {
        name = "Target Practice",
		desc = "Attack 2 random enemies.",
        flavour = "'Okay, the NEXT one will just barely miss you...'",

        anim = "targetpractice1",
        hit_anim = true,

        rarity = CARD_RARITY.COMMON,
        min_damage = 2,
        max_damage = 5,
        cost = 1,

        target_mod = TARGET_MOD.RANDOM1,
        target_count = 2,

        flags = CARD_FLAGS.RANGED,
    },
    
    target_practice_plus = 
    {
        name = "Boosted Target Practice",
        desc = "Attack <#UPGRADE>3</> random enemies.",
        target_count = 3,
    },
    
    target_practice_plus2 = 
    {
        name = "Savage Target Practice",
        desc = "Attack 2 random enemies.\n<#UPGRADE>Apply {BLEED 1}.</>",

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("BLEED", 1, self)
        end
    },

    hemophage =
    {
        name = "Hemophage",
		desc = "{ABILITY}: Whenever you apply {BLEED}, {HEAL} 1.",
		on_play_screen_shake = true,
        anim = "taunt3",
        flavour = "'Some of it got in my mouth and I'm not gonna lie; I could get used to it.'",
        rarity = CARD_RARITY.RARE,
        cost = 2,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "HEMOPHAGE", 1, self )
        end,
    },

    hemophage_plus =
    {
        name = "Initial Hemophage",
        desc = "{ABILITY}: Whenever you apply {BLEED}, {HEAL} 1.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    hemophage_plus2 =
    {
        name = "Boosted Hemophage",
        desc = "{ABILITY}: Whenever you apply {BLEED}, {HEAL} <#UPGRADE>2</>.",
        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "HEMOPHAGE", 2, self )
        end,
    },

    daggerstorm =
    {
        name = "Dagger Storm",
        anim = "taunt2",
        flavour = "'Where do they keep coming from!?'",
		desc = "Insert {1} {blade_flash} cards into your draw pile.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.num_cards )
        end,

        rarity = CARD_RARITY.UNCOMMON,
        cost = 2,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,
        num_cards = 2,

        OnPostResolve = function( self, battle, attack)
            local cards = {}
            for i = 1, self.num_cards do
                local incepted_card = Battle.Card( "blade_flash", self:GetOwner() )
                incepted_card.auto_deal = true
                table.insert( cards, incepted_card )
            end
            battle:DealCards( cards )
        end,
    },

    daggerstorm_plus = 
    {
        name = "Boosted Dagger Storm",
        desc = "Insert <#UPGRADE>{1}</> {blade_flash} cards into your draw pile.",
        num_cards = 3,
    },

    daggerstorm_plus2 = 
    {
        name = "Pale Dagger Storm",
        cost = 1,
    },

    blade_flash =
    {
        name = "Blade Flash",
        anim = "bladeflash",
        flavour = "'Composite materials make weapons almost too quick to see coming.'",

        rarity = CARD_RARITY.UNIQUE,
        min_damage = 3,
        max_damage = 4,
        cost = 0,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,
    },

    shadow_mastery =
    {
        name = "Shadow Mastery",
        anim = "taunt",
        flavour = "'One strike is usually all you need when emerging from darkness.'",
        desc = "{ABILITY}: Whenever you discard a card, gain 1 {ADRENALINE}.",
        on_play_screen_shake = true,

        rarity = CARD_RARITY.RARE,
        cost = 2,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "shadow_mastery", 1, self )
        end,

        condition =
        {
            desc = "Whenever you discard a card, gain <#HILITE>{1} {ADRENALINE}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks or 1 )
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                    self.owner:AddCondition("ADRENALINE", self.stacks, self)
                end,
            },
        }
    },

    shadow_mastery_plus =
    {
        name = "Initial Shadow Mastery",
        desc = "{ABILITY}: Whenever you discard a card, gain 1 {ADRENALINE}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    shadow_mastery_plus2 =
    {
        name = "Pale Shadow Mastery",
        cost = 1,
    },

    wild_lunge =
    {
        name = "Wild Lunge",
        anim = "stab",
        flavour = "'They came at me like some kind of wild animal.'",
        desc = "Attack 1 random enemy, apply 1 {WOUND}.",
        
        flags = CARD_FLAGS.MELEE,
        
        rarity = CARD_RARITY.COMMON,
        cost = 1,
        min_damage = 2,
        max_damage = 8,
        target_mod = TARGET_MOD.RANDOM1,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("WOUND", 1, self)
        end
    },
    wild_lunge_plus =
    {
        name = "Tall Wild Lunge",
        max_damage = 14,
    },
    wild_lunge_plus2 =
    {
        name = "Wide Wild Lunge",
        desc = "Attack <#UPGRADE>2</> random enemies, apply 1 {WOUND}.",
        anim = "stab",
        hit_anim = true,
        target_count = 2,
    },

    dagger_throw =
    {
        name = "Dagger Throw",
        anim = "daggerthrow",
        flavour = "'Don't worry, I have a spare.'",

        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,
        min_damage = 4,
        max_damage = 6,

        flags = CARD_FLAGS.PIERCING | CARD_FLAGS.RANGED,
    },
    dagger_throw_plus = 
    {
        name = "Tall Dagger Throw",
        min_damage = 4,
        max_damage = 8,
    },
    dagger_throw_plus2 = 
    {
        name = "Rooted Throw",
        min_damage = 6,
        max_damage = 6,
    },

    rummage =
    {
        name = "Rummage",
        anim = "taunt",
        flavour = "'I know it's in here somewhere, who drew all these pockets?'",
        desc = "Draw a card.\nDiscard a card.",

        cost = 0,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack)
            battle:DrawCards(1)
            battle:DiscardCards(1, nil, self)
        end,
    },
    rummage_plus =
    {
        name = "Boosted Rummage",
        desc = "Draw <#UPGRADE>2</> cards.\nDiscard <#UPGRADE>2</> cards.",

        OnPostResolve = function( self, battle, attack)
            battle:DrawCards(2)
            battle:DiscardCards(2, nil, self)
        end,
    },

    rummage_plus2 =
    {
        name = "Lucid Rummage",
        desc = "Draw <#UPGRADE>2</> cards.\n<#UPGRADE>Expend</> a card.",
        OnPostResolve = function( self, battle, attack)
            battle:DrawCards(2)
            battle:ExpendCards(1, nil, self)
        end,
    },

    survival_reflexes =
    {
        name = "Survival Reflexes",
        anim = "taunt",
        flavour = "'You either got them or you're done.'",
        desc = "Discard a card.\nGain {1} {DEFEND}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:CalculateDefendText(self.defend_amt))
        end,

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        defend_amt = 5,
        OnPostResolve = function( self, battle, attack )
            battle:DiscardCards(1, nil, self)
            self.owner:AddCondition("DEFEND", self.defend_amt, self)
        end,    
    },
    survival_reflexes_plus =
    {
        name = "Visionary Reflexes",
        desc = "<#UPGRADE>Draw a card.</> Discard a card.\nGain {1} {DEFEND}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( 5 ))
        end,

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(1)
            battle:DiscardCards(1, nil, self)
            self.owner:AddCondition("DEFEND", self.defend_amt, self)
        end,
    },
    survival_reflexes_plus2 =
    {
        name = "Pale Reflexes",
        cost = 0,
    },

    boulder_rush = 
    {
        name = "Tackle",
        anim = "hiltbash",
        flavour = "'Nobody got by me, I was a wall back in the day!'",
        desc = "Gain {1} {DEFEND}.\nApply {2} {IMPAIR}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:CalculateDefendText( self.defend_amt, self.owner ), self.impair_amt)
        end,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MELEE,

        cost = 1,

        min_damage = 3,
        max_damage = 5,

        defend_amt = 2,
        impair_amt = 1,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("DEFEND", self.defend_amt, self)
            attack:AddCondition("IMPAIR", self.impair_amt, self)
        end
    },

    boulder_rush_plus = 
    {
        name = "Boosted Tackle",
        desc = "<#UPGRADE>Gain {1} {DEFEND}.</>\nApply {2} {IMPAIR}",

        defend_amt = 4,
    },

    boulder_rush_plus2 =
    {
        name = "Thorny Tackle",
        min_damage = 5,
        max_damage = 7,
    },

    bouldering_charge = 
    {
        name = "Defensive Surge",
        desc = "Gain {1} {DEFEND}. Increase by {2} for each card played this turn.\n{3}",
        desc_fn = function(self, fmt_str)
            if self.engine then
                local str = loc.format( LOC"CARD_ENGINE.CARDS_PLAYED", self.engine:CountCardsPlayed() )
                return loc.format(fmt_str, self:CalculateDefendText( self.defend_amount + (self.defend_bonus * self.engine:CountCardsPlayed()) ), self.defend_bonus, str )
            else
                return loc.format(fmt_str, self:CalculateDefendText( self.defend_amount ), self.defend_bonus, "" )
            end
        end,

        anim = "taunt",
        flavour = "'I don't know what to tell you, I hit them like, six times...'",

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        cost = 1,

        defend_amount = 2,
        defend_bonus = 2,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "DEFEND", self.defend_amount + (self.defend_bonus * self.engine:CountCardsPlayed()), self )
        end,
    },

    bouldering_charge_plus = 
    {
        name = "Boosted Surge",
        desc = "Gain <#UPGRADE>{1} {DEFEND}</>. Increase by {2} for each card played this turn.\n{3}",
        defend_amount = 5,
    },

    bouldering_charge_plus2 = 
    {
        name = "Stone Surge",
        desc = "Gain {1} {DEFEND}. Increase by <#UPGRADE>{2}</> for each card played this turn.\n{3}",
        defend_bonus = 3,
    },

    boulder_stance = 
    {
        name = "Boulder Stance",
        desc = "{ABILITY}: Whenever you play a card, gain {DEFEND {1}}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.defend_bonus)
        end,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Defend",

        anim = "taunt",
        flavour = "'Well, I sure as Hesh can't move the damn thing...'",

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        target_type = TARGET_TYPE.SELF,

        cost = 2,

        defend_bonus = 1,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("boulder_stance", self.defend_bonus, self)
        end,

        condition = 
        {
            desc = "Whenever you play a card, gain <#HILITE>{DEFEND {1}}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,
            event_handlers = 
            {
                [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                    if card.owner == self.owner and not card:IsFlagged( CARD_FLAGS.TOOLBOX ) then
                        card.owner:AddCondition("DEFEND", self.stacks, self)
                    end
                end,
            },
        },
    },

    boulder_stance_plus =
    {
        name = "Pale Stance",
        cost = 1,
    },

    boulder_stance_plus2 =
    {
        name = "Initial Stance",
        desc = "{ABILITY}: Whenever you play a card, gain {DEFEND {1}}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    adapt = 
    {
        name = "Adapt",
        anim = "taunt2",
        flavour = "'Sometimes a change of approach is necessary.'",
		desc = "Discard your hand.\nDraw {1} cards.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.num_cards)
        end,

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.SELF,
        num_cards = 5,

        OnPostResolve = function( self, battle, attack)

            local tbl = ObtainWorkTable()
            for i, card in battle:GetHandDeck():Cards() do
                table.insert(tbl, card)
            end

            for i, card in ipairs(tbl) do
                battle:DiscardCard(card)
            end

            ReleaseWorkTable(tbl)

            battle:DrawCards(self.num_cards)
        end,
    },
    adapt_plus = 
    {
        name = "Boosted Adapt",
        desc = "Discard your hand.\nDraw <#UPGRADE>{1}</> cards.",
        num_cards = 7,
    },

    adapt_plus2 = 
    {
        name = "Enduring Adapt",
        flags = CARD_FLAGS.SKILL,
    },

    scatter =
    {
        name = "Scatter",
        anim = "taunt2",
        flavour = "'Loop back around, I'll keep them distracted.'",
        desc = "Discard up to {1} cards.\nGain 1 {TEMP_POWER} for each card discarded.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.discard_count )
        end,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        cost = 1,
        discard_count = 2,

        OnPostResolve = function( self, battle, attack )
            if self.draw_count then
                battle:DrawCards(self.draw_count)
            end
            local cards = battle:DiscardCards(0, self.discard_count, self)
            if #cards > 0 then
                self.owner:AddCondition("POWER", #cards, self)
                self.owner:AddCondition("POWER_LOSS", #cards, self)
            end
        end,
    },

    scatter_plus = 
    {
        name = "Boosted Scatter",
        desc = "Discard up to<#UPGRADE> {1} </>cards.\nGain 1 {TEMP_POWER} for each card discarded.",
        discard_count = 3,
    },

    scatter_plus2 = 
    {
        name = "Visionary Scatter",
        desc = "<#UPGRADE>Draw a card</>\nDiscard up to {1} cards.\nGain 1 {TEMP_POWER} for each card discarded.",
        draw_count = 1,
    },

    duster = 
    {
        name = "Duster",
        anim = "quickdraw",
        flavour = "'Spiked knuckles are a common addition to the local street toughs, flashy and effective.'",

        desc = "Discard {1} cards. Attack a random enemy.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.discard_count)
        end,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MELEE,
        target_mod = TARGET_MOD.RANDOM1,

        cost = 0,
        discard_count = 2,

        min_damage = 2,
        max_damage = 2,

        OnPreResolve = function( self, battle, attack )
            battle:DiscardCards(self.discard_count, nil, self)
        end,
    },

    duster_plus = 
    {
        name = "Enhanced Duster",
        desc = "Discard <#UPGRADE>{1}</> cards. Attack a random enemy.",
        discard_count = 3,
    },

    duster_plus2 =
    {
        name = "Boosted Duster",
        min_damage = 3,
        max_damage = 3,
    },

    spare_blades = 
    {
        name = "Spare Blades",
        desc = "If this is discarded, insert a {blade_flash} into your hand.",

        flavour = "'Feels like I'm carrying nothing at all!'",

        mod_xp = -2,

        cost = 0,
        
        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.SKILL,

        num_cards = 1,

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                if card == self then
                    local cards = {}
                    for i = 1, self.num_cards do
                        local incepted_card = Battle.Card( "blade_flash", self:GetOwner() )
                        table.insert( cards, incepted_card )
                    end
                    battle:DealCards( cards, battle:GetHandDeck() )
                    self:AddXP(1)
                    if self.defend_amount then
                        self.owner:AddCondition("DEFEND", self.defend_amount, self)
                    end
                end
            end,
        },
    },

    spare_blades_plus =
    {
        name = "Boosted Blades",
        desc = "If this is discarded, insert <#UPGRADE>2</> {blade_flash} into your hand.",
        num_cards = 2,
    },

    spare_blades_plus2 =
    {
        name = "Stone Blades",
        desc = "If this is discarded, insert a {blade_flash} into your hand <#UPGRADE>and gain {1} {DEFEND}</>.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateDefendText( self.defend_amount ))
        end,
        defend_amount = 3,
    },

    blade_fury =
    {
        name = "Blade Fury",
        anim = "bladefury",
        flavour = "'One of the more ferocious dagger combinations seen on the mainland.'",
		desc = "Spend up to {1} {COMBO}: Attack once extra per {COMBO}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.combo_limit )
        end,

        rarity = CARD_RARITY.UNCOMMON,
        cost = 1,
        min_damage = 3,
        max_damage = 4,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.COMBO_FINISHER,

        combo_limit = 3,

        PreReq = function( self, minigame )
            local count =  math.min(self.owner:GetConditionStacks( "COMBO" ), self.combo_limit)
            if count > 0 then
                self.hit_count = 1 + count
            else
                self.hit_count = 1
            end
            return self.owner:GetConditionStacks("COMBO") > 0
        end,

        OnPreResolve = function( self, battle, attack )
            if self.combo_used > 0 then
                attack.hit_frames = "hit_frames_combo"
            end
        end,

        OnPostResolve = function( self, battle, attack )
            self.hit_count = 1
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                if card == self then
                    local count =  math.min(self.owner:GetConditionStacks( "COMBO" ), self.combo_limit)
                    if count > 0 then
                        self.hit_count = 1 + count
                    else
                        self.hit_count = 1
                    end
                    self.owner:RemoveCondition("COMBO", count, self)
                    self.combo_used = count
                end
            end
        },
    },
    blade_fury_plus =
    {
        name = "Boosted Blade Fury",
        min_damage = 4,
        max_damage = 6,
    },
    
    blade_fury_plus2 =
    {
        name = "Rival's Blade Fury",
        desc = "Spend up to {1} {COMBO}: Attack once extra per {COMBO}.\n<#UPGRADE>Gain {COMBO 2}.</>",
        OnPostResolve = function ( self, battle, attack )
            self.owner:AddCondition("COMBO", 2, self)
        end
    },

    echo_strike =
    {
        name = "Echo Strike",
        desc = "Return to your hand when played.\nGain 1-2 {COMBO}.",
        anim = "punch",
        flavour = "'Why change the vector of attack if they keep falling for it?'",

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE,

        cost = 1,

        min_damage = 2,
        max_damage = 2,
        max_combo = 2,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("COMBO", math.random( 1, self.max_combo), self)
        end,

        deck_handlers = { DECK_TYPE.DISCARDS },

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if card == self and target_deck and target_deck:GetDeckType() == DECK_TYPE.DISCARDS and source_deck and source_deck:GetDeckType() == DECK_TYPE.RESOLVE then
                    -- Return!
                    self:TransferCard( self.engine:GetHandDeck() )
                end
            end
        }
    },

    echo_strike_plus = 
    {
        name = "Rival's Echo Strike",
        desc = "Return to your hand when played.\nGain <#UPGRADE>1-3</> {COMBO}.",
        max_combo = 3,
    },

    echo_strike_plus2 = 
    {
        name = "Boosted Echo Strike",
        min_damage = 4,
        max_damage = 4,
    },

    specialty = 
    {
        name = "Specialty",
        anim = "taunt",
        flavour = "'I call this one the Murder Bay Mauler!'",
        desc = "{IMPROVISE} a {COMBO_FINISHER} from your deck. Gain {1} {COMBO}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.combo_gain)
        end,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        cost = 1,

        combo_gain = 2,
        pool_size = 3,

        OnPostResolve = function( self, battle, attack )
            local cards = {}
            for i, card in ipairs(battle:GetCardsOwnedBy(self.owner)) do
                if card:IsFlagged(CARD_FLAGS.COMBO_FINISHER) and card.deck:GetDeckType() ~= DECK_TYPE.IN_HAND then
                    table.insert(cards, card)
                end
            end

            battle:ImproviseCards(table.multipick(cards, self.pool_size), 1, "phantom_finisher", nil, nil, self)
            self.owner:AddCondition("COMBO", self.combo_gain, self)
        end,
    },

    specialty_plus =
    {
        name = "Rival's Specialty",
        desc = "{IMPROVISE} a {COMBO_FINISHER} from your deck. Gain <#UPGRADE>{1}</> {COMBO}.",
        combo_gain = 4,
    },

    specialty_plus2 = 
    {
        name = "Wide Specialty",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> a {COMBO_FINISHER} from your deck. Gain {1} {COMBO}.",
        pool_size = 5,
    },

    size_up =
    {
        name = "Size Up",
        anim = "taunt3",
        flavour = "'I probably know your jacket size too, let's do this.'",
        desc = "Draw {1} cards, gain {COMBO} equal to their combined cost.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.draw_count)
        end,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        cost = 1,
        draw_count = 2,

        OnPostResolve = function( self, battle, attack )
            local cards = battle:DrawCards(self.draw_count)
            local cost = self.bonus_combo or 0
            for i,card in ipairs(cards) do
                cost = cost + card.cost
            end
            if cost > 0 then
                self.owner:AddCondition("COMBO", cost, self)
            end
        end
    },

    size_up_plus =
    {
        name = "Visionary Size Up",
        desc = "Draw <#UPGRADE>{1}</> cards, gain {COMBO} equal to their combined cost.",
        draw_count = 3,
    },

    size_up_plus2 = 
    {
        name = "Rival's Size Up",
        desc = "Draw {1} cards, gain {COMBO} equal to their combined cost <#UPGRADE>+2</>.",
        bonus_combo = 2,
    },

    reversal = 
    {
        name = "Reversal",
        anim = "uppercut",
        flavour = "'She was just waiting for the right moment.'",
        desc = "Spend all your {COMBO}: Gain {1} {RIPOSTE} per {COMBO}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.riposte_amount)
        end,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.COMBO_FINISHER,

        cost = 1,
        riposte_amount = 1,

        min_damage = 4,
        max_damage = 6,

        OnPreResolve = function( self, battle, attack )
            if self.owner:HasCondition("COMBO") then
                self.owner:AddCondition("RIPOSTE", self.riposte_amount * self.owner:GetConditionStacks("COMBO"), self)
                self.owner:RemoveCondition("COMBO")
            end
        end,
    },

    reversal_plus = 
    {
        name = "Boosted Reversal",
        min_damage = 6,
        max_damage = 9,
    },

    reversal_plus2 = 
    {
        name = "Focused Reversal",
        desc = "<#UPGRADE>Gain 3 {RIPOSTE}.</>\nSpend all your {COMBO}: Gain {1} {RIPOSTE} per {COMBO}.",
        OnPreResolve = function( self, battle, attack )
            self.owner:AddCondition("RIPOSTE", 3, self)
            if self.owner:HasCondition("COMBO") then
                self.owner:AddCondition("RIPOSTE", self.riposte_amount * self.owner:GetConditionStacks("COMBO"), self)
                self.owner:RemoveCondition("COMBO")
            end
        end
    },

    active_defense =
    {
        name = "Active Defense",
        anim = "taunt",
        flavour = "'Being aware of all options can keep you safe and in fighting shape.'",
        desc = "Apply {1} {DEFEND}.\nIncrease by {2} for each card discarded this turn.\n{3}",
        desc_fn = function(self, fmt_str)
            if self.engine then
                local count = self.engine:CountCardsDiscarded()
                local str = loc.format( LOC"CARD_ENGINE.CARDS_DISCARDED", count )
                return loc.format(fmt_str, self:CalculateDefendText( self.defend_base + count * self.defend_increment ), self.defend_increment, str )
            else
                return loc.format(fmt_str, self:CalculateDefendText( self.defend_base ), self.defend_increment, "" )
            end
        end,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        cost = 1,
        defend_base = 4,
        defend_increment = 2,

        OnPostResolve = function( self, battle, attack)
            local count = self.engine:CountCardsDiscarded()
            local defend_amt = self.defend_base + count * self.defend_increment
            attack:AddCondition( "DEFEND", defend_amt, self )
        end,
    },

    active_defense_plus =
    {
        name = "Boosted Defense",
        desc = "Apply <#UPGRADE>{1} {DEFEND}.</>\nIncrease by {2} for each card discarded this turn.",
        defend_base = 6,
        
    },
    active_defense_plus2 =
    {
        name = "Enhanced Defense",
        desc = "Apply {1} {DEFEND}.\nIncrease by <#UPGRADE>{2}</> for each card discarded this turn.",
        defend_increment = 3,
    },

    kidney_shot =
    {
        name = "Kidney Shot",
        anim = "stab",
        flavour = "'You sure that guy even has kidneys?'",

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,
        cost = 1,

        min_damage = 1,
        max_damage = 3,

        features =
        {
            STUN = 1,
        }
    },
    kidney_shot_plus =
    {
        name = "Boosted Kidney Shot",
        min_damage = 2,
        max_damage = 6,
    },
    kidney_shot_plus2 =
    {
        name = "Pale Kidney Shot",
        cost = 0,
    },

    improve_accuracy =
    {
        name = "Improve Accuracy",
        anim = "taunt3",
        flavour = "'I don't think that target is regulation size.'",
        desc = "Your attacks deal max damage for the next {1} turns.",
        on_play_screen_shake = true,
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.duration)
        end,

        rarity = CARD_RARITY.COMMON,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        cost = 1,
        duration = 2,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "improve_accuracy", self.duration, self )
        end,

        condition =
        {
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks)
            end,

            event_priorities =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = EVENT_PRIORITY_SETTOR,
            },

            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if self.owner == card.owner then
                        dmgt:ModifyDamage( dmgt.max_damage, dmgt.max_damage, self )
                    end
                end,

                [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                    if self.owner == fighter then
                        self.owner:RemoveCondition( "improve_accuracy", 1 )
                    end
                end,
            },
        }
    },
    improve_accuracy_plus =
    {
        name = "Extended Accuracy",
        desc = "Your attacks deal max damage for the next <#UPGRADE>{1}</> turns.",
        duration = 3,
    },
    improve_accuracy_plus2 =
    {
        name = "Pale Accuracy",
        cost = 0,
    },

    brazen_attack =
    {
        name = "Brazen Attack",
        desc = "Apply {IMPAIR {1}}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.impair_amt)
        end,
        anim = "uppercut",
        flavour = "'Big and loud attacks are riskier but leave your opponents reeling.'",

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,
        cost = 2,

        min_damage = 4,
        max_damage = 8,

        impair_amt = 2,
        
        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("IMPAIR", self.impair_amt, self)
        end
    },
    brazen_attack_plus =
    {
        name = "Heavy Brazen Attack",
        desc = "Apply <#UPGRADE>{IMPAIR {1}}</>.",
        impair_amt = 3,
    },

    brazen_attack_plus2 =
    {
        name = "Boosted Brazen Attack",
        min_damage = 6,
        max_damage = 12,
    },

    crushing_blow =
    {
        name = "Crushing Blow",
        anim = "gash",
        flavour = "'They'll be seeing stars after I get through with them!'",

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,

        cost = 3,

        features =
        {
            STUN = 1,
        },
        min_damage = 8,
        max_damage = 12,
    },

    crushing_blow_plus =
    {
        name = "Boosted Crushing Blow",
        min_damage = 12,
        max_damage = 16
    },

    crushing_blow_plus2 =
    {
        name = "Nailed Crushing Blow", 
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING | CARD_FLAGS.EXPEND,
    },
    
    slaughter = 
    {
        name = "Slaughter",
        desc = "Draw cards equal to the damage dealt by this card.",

        anim = "wildlunge",
        flavour = "'It was a massacre, people get named after things like that.'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON, 

        cost = 2,
        min_damage = 2,
        max_damage = 5,

        event_handlers = 
        {
            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                if hit.card == self and not hit.evaded then
                    battle:DrawCards(hit.damage)
                end
            end
        },
    },

    slaughter_plus =
    {
        name = "Pale Slaughter",
        cost = 1,
    },

    slaughter_plus2 = 
    {
        name = "Boosted Slaughter",
        min_damage = 3,
        max_damage = 6,
    },

    ghost_strike =
    {
        name = "Ghost Strike",
        desc = "Draw a card.\nDiscard this card: Your attacks deal 2 bonus damage this turn.",

        anim = "elbow_strike",
        flavour = "'Kra'deshi pugilists tend to feint a lot of their strikes, waiting for the perfect opening.'",

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.COMMON, 

        cost = 1,
        min_damage = 1,
        max_damage = 2,
        bonus_damage = 2,

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(1)
        end,

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                if card == self then
                    self:NotifyTriggered()

                    self.owner:AddCondition("ghost_strike", self.bonus_damage, self)
                    self:AddXP(1)
                end
            end,
        },

        condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if self.owner == card.owner then
                        dmgt:AddDamage( self.stacks, self.stacks, self )
                    end
                end,

                [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                    if self.owner == fighter then
                        self.owner:RemoveCondition( "ghost_strike", self.stacks )
                    end
                end,
            },
        }
    },

    ghost_strike_plus =
    {
        name = "Boosted Ghost Strike",
        desc = "Draw a card.\nDiscard this card: Your attacks deal <#UPGRADE>3 bonus damage</> this turn.",
        bonus_damage = 3,
    },

    ghost_strike_plus2 =
    {
        name = "Visionary Ghost Strike",
        desc = "<#UPGRADE>Draw two cards</>.\nDiscard this card: Your attacks deal 2 bonus damage this turn.",

        OnPostResolve = function( self, battle, attack )
            battle:DrawCards(2)
        end,
    },

    follow_through = 
    {
        name = "Follow Through",
        icon = "battle/follow_up.tex",
        desc = "{COMBO} points spent for the rest of this turn are regained at the start of your next turn.",
        anim = "taunt",
        flavour = "'Waiting for the right moment can be the difference between victory and defeat.'",

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.RARE,
        target_type = TARGET_TYPE.SELF,

        mod_xp = -2,

        cost = 1,

        OnPostResolve = function( self, battle, attack)
            if self.combo_gain then
                self.owner:AddCondition("COMBO", self.combo_gain, self)
            end
            self.owner:AddCondition("follow_through", 1, self)
        end,

        condition = 
        {
            desc = "{COMBO} points spent this turn are regained at the start of your next turn. Current {COMBO} points spent: {1}.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.combo_spent or 0)
            end,

            combo_spent = 0,
            active = true,

            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.active = false
                end,

                [ BATTLE_EVENT.CONDITION_REMOVED ] = function( self, fighter, condition, stacks, source )
                    if self.active and self.owner == fighter and condition.id == "COMBO" and stacks > 0 then
                        self.combo_spent = self.combo_spent + (stacks * self.stacks)
                    end
                end,

                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    if self.combo_spent > 0 then
                        self.owner:AddCondition("COMBO", self.combo_spent, self)
                    end
                    self.owner:RemoveCondition(self.id)
                end,
            },
        },
    },

    follow_through_plus =
    {
        name = "Sticky Follow Through",
        desc = "{COMBO} points spent for the rest of this turn are regained at the start of your next turn.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.STICKY,
    },

    follow_through_plus2 = 
    {
        name = "Pale Follow Through",
        cost = 0,
    },

    sneak_attack =
    {
        name = "Sneak Attack",
        anim = "backstab",
        flavour = "'It was probably the cheapest shot I've ever seen in my life.'",
        icon = "battle/rabbit_punch.tex",
        desc = "Spend {1} {COMBO}: This card deals max damage and costs 1 less.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.combo_cost)
        end,

        flags = CARD_FLAGS.MELEE | CARD_FLAGS.COMBO_FINISHER,
        rarity = CARD_RARITY.COMMON, 

        cost = 2,
        min_damage = 3,
        max_damage = 8,
        combo_cost = 2,
        
        PreReq = function( self, battle )
            return self.owner:GetConditionStacks("COMBO") >= self.combo_cost
        end,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if self == card and (self.combo_used or self.combo_used == nil and self.owner:GetConditionStacks("COMBO") >= self.combo_cost) then
                    dmgt:ModifyDamage( dmgt.max_damage, dmgt.max_damage, self )                       
                end
            end,

            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self and (self.combo_used or self.combo_used == nil and self.owner:GetConditionStacks("COMBO") >= self.combo_cost) then
                    cost_acc:AddValue(-1)
                end
            end
        },

        OnPreResolve = function( self, battle, attack )
            if self.owner:GetConditionStacks("COMBO") >= self.combo_cost then
                self.owner:RemoveCondition("COMBO", self.combo_cost)
                self.combo_used = true
            else
                self.combo_used = false
            end
        end,

        OnPostResolve = function( self, battle, attack)
            self.combo_used = nil
        end,
    },

    sneak_attack_plus = 
    {
        name = "Pale Sneak Attack",
        desc = "Spend <#UPGRADE>{1} {COMBO}</>: This card deals max damage and costs 1 less.",
        combo_cost = 1,
    },

    sneak_attack_plus2 =
    {
        name = "Tall Sneak Attack",
        max_damage = 12,
    },

    breather = 
    {
        name = "Breather",
        anim = "taunt2",
        flavour = "'Take a second, organize your thoughts, and plan your next move.'",
        desc = "Spend {1} {COMBO}: Gain 1 Action.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.combo_cost )
        end,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.COMBO_FINISHER,
        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.SELF,

        mod_xp = -2,
        cost = 0,
        combo_cost = 3,
        action_bonus = 1,

        PreReq = function( self, battle )
            return self.owner:GetConditionStacks("COMBO") >= self.combo_cost
        end,

        CanPlayCard = function( self, battle )
            if self.owner:GetConditionStacks("COMBO") < self.combo_cost then
                return false, CARD_PLAY_REASONS.NO_COMBO
            end
            return true
        end,

        OnPreResolve = function( self, battle )
            if self.owner:GetConditionStacks("COMBO") >= self.combo_cost then
                self.owner:RemoveCondition("COMBO", self.combo_cost)
                self.engine:ModifyActionCount( self.action_bonus )
            end
        end,
    },

    breather_plus = 
    {
        name = "Pale Breather",
        desc = "Spend <#UPGRADE>{1}</> {COMBO}: Gain 1 action.",

        combo_cost = 2,
    },

    breather_plus2 = 
    {
        name = "Delayed Breather",
        desc = "Spend {1} {COMBO}: Gain <#UPGRADE>2</> actions <#DOWNGRADE>next turn.</>",

        OnPreResolve = function( self, battle )
            if self.owner:GetConditionStacks("COMBO") >= self.combo_cost then
                self.owner:RemoveCondition("COMBO", self.combo_cost)
                self.owner:AddCondition("NEXT_TURN_ACTION", 2 )
            end
        end,
    },

    wretched_strike =
    {
        name = "Wretched Strike",
        anim = "uppercut",
        flavour = "'That was the worst thing that I ever saw...'",
        desc = "Deals 1 bonus damage for every card drawn this turn.",
        alt_desc = "Deals 1 bonus damage for every card drawn this turn. ({1})",
        desc_fn = function( self, fmt_str )
            if self.engine then
                return loc.format( self.def:GetLocalizedString( "ALT_DESC" ), self.engine:CountCardsDrawn() )
            else
                return fmt_str
            end
        end,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.RARE,

        cost = 1,
        bonus = 0,
        min_damage = 1,
        max_damage = 3,

        deck_handlers = ALL_DECKS,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self and self.bonus > 0 then
                    dmgt:AddDamage( self.bonus, self.bonus, self )
                end
            end,

            [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                if card.owner == self.owner then
                    self.bonus = self.bonus + 1
                end
            end,

            [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                if fighter == self.owner then
                    self.bonus = 0
                end
            end
        },
    },

    wretched_strike_plus = 
    {
        name = "Rooted Wretched Strike",
        min_damage = 3,
        max_damage = 3,
    },

    wretched_strike_plus2 = 
    {
        name = "Tall Wretched Strike",
        max_damage = 6,
    },

    stomp = 
    {
        name = "Stomp",
        desc = "Spend {1} {COMBO}: this card costs 0.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.combo_cost )
        end,

        anim = "stomp",
        flavour = "'That's gotta hurt.'",

        mod_xp = -2,

        cost = 1,
        combo_cost = 2,

        min_damage = 3,
        max_damage = 6,

        rarity = CARD_RARITY.COMMON,
        flags =  CARD_FLAGS.MELEE | CARD_FLAGS.COMBO_FINISHER,

        PreReq = function( self, battle )
            return self.owner:GetConditionStacks("COMBO") >= self.combo_cost
        end,

        event_priorities =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers =
        {
            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self and self.owner:GetConditionStacks("COMBO") >= self.combo_cost then
                    cost_acc:ModifyValue(0)
                end
            end
        },

        OnPreResolve = function( self, battle, attack )
            if self.owner:GetConditionStacks("COMBO") >= self.combo_cost then
                self.owner:RemoveCondition("COMBO", self.combo_cost)
            end
        end
    },

    stomp_plus =
    {
        name = "Rooted Stomp",
        min_damage = 6,
    },

    stomp_plus2 = 
    {
        name = "Boosted Stomp",
        min_damage = 4,
        max_damage = 9
    },

    doomed = 
    {
        name = "Doom",
        desc = "Double the {SURRENDER} meter of all fighters.",

        anim = "hero_taunt2",
        flavour = "'Game over man! GAME OVER!'",
        target_anim = "surrender",

        cost = 3,
        multiplier = 1,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.SKILL, 

        target_type = TARGET_TYPE.ANY,
        target_mod = TARGET_MOD.CUSTOM,
        auto_target = true,

        target_fn = function( self, battle, primary_target, target_fighters )
            for i, fighter in battle:ActiveFighters() do
                if fighter:HasMorale() and fighter:GetStatus() ~= FIGHT_STATUS.SURRENDER then
                    table.insert( target_fighters, fighter )
                end
            end
        end,

        OnPostResolve = function( self, battle, attack )
            for i, hit in attack:Hits() do
                if hit.target:HasMorale() then
                    hit.target:DeltaMorale( hit.target:GetMorale() * self.multiplier )
                end
            end
        end        
    },

    doomed_plus =
    {
        name = "Pale Doom",
        cost = 2,
    },

    doomed_plus2 =
    {
        name = "Focused Doom",
        desc = "Double the {SURRENDER} meter of <#UPGRADE>all enemies.</>",

        target_type = TARGET_TYPE.ENEMY,

        target_fn = function( self, battle, primary_target, target_fighters )
            for i, fighter in primary_target:GetTeam():Fighters() do
                if fighter:HasMorale() and fighter:GetStatus() ~= FIGHT_STATUS.SURRENDER then
                    table.insert( target_fighters, fighter )
                end
            end
        end,

        flags = CARD_FLAGS.SKILL,
    },

    --THE SPICY DOZEN

    discharge = 
    {
        name = "Discharge",
        desc = "Target a random enemy for each card in your hand.\n{1}",
        desc_fn = function( self, fmt_str )
            if self.engine then
                local str = loc.format( LOC"CARD_ENGINE.CARDS_IN_HAND", self.engine:GetHandDeck():CountCards() )
                return loc.format( fmt_str, str )
            else
                return loc.format( fmt_str, "" )
            end
        end,

        anim = "targetpractice1",
        hit_anim = true,
        flavour = "'Try to catch them off guard. They always come prepared.'",
        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.RANGED,

        target_type = TARGET_TYPE.ENEMY,
        target_mod = TARGET_MOD.RANDOM1,

        mod_xp = -1,

        cost = 2,

        min_damage = 2,
        max_damage = 2,

        PreReq = function( self, battle, target )
            self.target_count = battle:GetHandDeck():CountCards()
            return self.target_count > 1
        end,
    },

    discharge_plus = 
    {
        name = "Mirrored Discharge",
        desc = "Target a random enemy for each card in your hand <#UPGRADE>+ 1</>.\n{1}",
        PreReq = function( self, battle, target )
            self.target_count = battle:GetHandDeck():CountCards() + 1
            return self.target_count > 1
        end,
    },

    discharge_plus2 = 
    {
        name = "Boosted Discharge",
        min_damage = 3,
        max_damage = 3,
    },

    churn = 
    {
        name = "Churn",
        desc = "Draw cards until you draw a Maneuver card and {HEAL} 1 for each card drawn. The card costs 1 less until played.",

        anim = "taunt2",
        flavour = "'I picture it in my mind like a wave, and when it crashes it all just comes together.'",
        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        target_type = TARGET_TYPE.SELF,

        cost = 2,

        OnPostResolve = function( self, battle, attack )
            while true do
                local cards = battle:DrawCards( 1 )
                local card = cards[1]
                if not card then
                    break
                end

                self.owner:HealHealth(1, self)

                if GetCardType(card) == CARD_TYPE.FIGHT_SKILL then
                    local con = self.owner:GetCondition("churn") or self.owner:AddCondition("churn", 1, self)
                    if con then 
                        con.cards = con.cards or {}
                        for i,card in ipairs(cards) do
                            table.insert(con.cards, card)
                        end
                    end
                    break
                end
            end
        end,

        condition =
        {
            hidden = true,

            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                    if self.cards and table.contains(self.cards, card) then
                        local count = table.count_if(self.cards, function( v ) return v == card end)
                        acc:AddValue(-1 * count, self)
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    while self.cards and table.contains(self.cards, attack.card) do
                        table.arrayremove(self.cards, attack.card)
                    end
                    if self.cards and #self.cards == 0 then
                        self.owner:RemoveCondition( self.id, self.stacks, self )
                    end
                end
            },
        },
    },

    churn_plus = 
    {
        name = "Pale Churn",
        cost = 1,
    },

    churn_plus2 =
    {
        name = "Enduring Churn",
        flags = CARD_FLAGS.SKILL,
    },

    drunken_master = 
    {
        name = "Drunken Master",
        desc = "Add a {drunk} card to your discards.\nDeals max damage if you have a {drunk} card in your hand.",

        anim = "wildlunge",
        flavour = "'Bar brawling has become something of a profession for Smith.'",
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE,

        cost = 1,

        min_damage = 2,
        max_damage = 10,

        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if self == card then
                    for i, hand_card in self.engine:GetHandDeck():Cards() do
                        if hand_card.id == "drunk" then
                            dmgt:ModifyDamage( dmgt.max_damage, dmgt.max_damage, self )
                        end
                    end                        
                end
            end,
        },

        OnPostResolve = function( self, battle, attack )
            local card = Battle.Card( "drunk", self.owner )
            battle:DealCard( card, battle:GetDeck( DECK_TYPE.DISCARDS ) )
        end,
    },

    drunken_master_plus = 
    {
        name = "Tall Drunken Master",
        max_damage = 14,
    },

    drunken_master_plus2 =
    {
        name = "Drunken-er Master",
        desc = "Add a {drunk} card to your discards.\nDeals max damage if you have a {drunk} card in your hand.",

        flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,
    },

    over_extension = 
    {
        name = "Over Extension",
        desc = "Add an {status_injury} card to your discard pile. \nCannot be played with an {status_injury} in your hand.",

        anim = "stab",
        flavour = "'I think we are BOTH going to feel that in the morning.'",

        loc_strings =
        {
            NO_INJURIES = "Cannot play with an Injury card in hand",
        },

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE,

        cost = 1,

        min_damage = 8,
        max_damage = 10,

        CanPlayCard = function(self, battle)
            local can_play = true
            for i, hand_card in battle:GetHandDeck():Cards() do
                if hand_card.id == "status_injury" then
                    can_play = false
                end
            end
            return can_play, self.def:GetLocalizedString( "NO_INJURIES" )
        end,

        OnPostResolve = function( self, battle, attack )
            local card = Battle.Card( "status_injury", self.owner )
            card:TransferCard( battle:GetDiscardDeck() )
        end,
    },

    over_extension_plus = 
    {
        name = "Rooted Over Extension",
        min_damage = 11,
        max_damage = 11,
    },

    over_extension_plus2 = 
    {
        name = "Tall Over Extension",
        max_damage = 13,
    },

    focused_assault = 
    {
        name = "Focused Assault",
        desc = "If this is the only card in your hand, attack 2 times with {1} bonus damage.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.bonus_damage )
        end,

        anim = "uppercut",
        hit_anim = true,
        flavour = "'Single out one person from the whole group, and bring them all of the pain.'",
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE,

        mod_xp = -2,

        cost = 1,

        min_damage = 4,
        max_damage = 6,
        bonus_damage = 4,
        bonus_hits = 1,
        draw_deck_count = math.huge,

        PreReq = function( self, battle )
            return battle:GetHandDeck():CountCards() <= 1 and battle:GetDrawDeck():CountCards() <= self.draw_deck_count
        end,

        deck_handlers = ALL_DECKS,

        event_handlers = 
        {
            [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                local count = self.engine:GetHandDeck():CountCards()
                if self.deck and ((count == 1 and self.deck:GetDeckType() == DECK_TYPE.IN_HAND) or (count == 0 and self.deck:GetDeckType() == DECK_TYPE.RESOLVE)) and
                    self.engine:GetDrawDeck():CountCards() <= self.draw_deck_count then
                    -- All conditions satisfied!
                    self.hit_count = 1 + self.bonus_hits
                else
                    self.hit_count = 1
                end
            end,

            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if self == card and self.engine:GetHandDeck():CountCards() <= (card.deck == self.engine:GetHandDeck() and 1 or 0) then
                    dmgt:AddDamage(self.bonus_damage, self.bonus_damage, self)
                end
            end,
        },
    },

    focused_assault_plus =
    {
        name = "Boosted Assault",
        desc = "If this is the only card in your hand, attack 2 times with <#UPGRADE>{1}</> bonus damage.",
        bonus_damage = 6,
    },

    focused_assault_plus2 = 
    {
        name = "Final Assault",
        desc = "If this is the only card in your hand <#DOWNGRADE>and your draw pile is empty,</> attack <#UPGRADE>4</> times with {1} bonus damage.",

        draw_deck_count = 0,
        bonus_hits = 3,
    },

    battle_scars = 
    {
        name = "Battle Scars",
        desc = "Discard your hand. Gain {1} {DEFEND} and {2} {RIPOSTE} for each card discarded.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.defend_bonus, self.counter_bonus )
        end,

        anim = "taunt2",
        flavour = "'Shroke combat scarring has become its own fashion statement in Havaria.'",
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        target_type = TARGET_TYPE.SELF,

        cost = 1,

        defend_bonus = 2,
        counter_bonus = 2,

        OnPostResolve = function( self, battle, attack )
            local tbl = ObtainWorkTable()
            for i, card in battle:GetHandDeck():Cards() do
                table.insert(tbl, card)
            end
            local count = 0
            for i, card in ipairs(tbl) do
                battle:DiscardCard(card)
                count = count + 1
            end

            self.owner:AddCondition("DEFEND", self.defend_bonus * count, self)
            self.owner:AddCondition("RIPOSTE", self.counter_bonus * count, self)

            ReleaseWorkTable(tbl)
        end,
    },

    battle_scars_plus = 
    {
        name = "Stone Scars",
        desc = "Discard your hand. Gain {1} {DEFEND} and <#UPGRADE>{2} {RIPOSTE}</> for each card discarded.",
        counter_bonus = 3,
    },

    battle_scars_plus2 =
    {
        name = "Enduring Scars",
        flags = CARD_FLAGS.SKILL,
    },

    kradeshi_barbstorm = 
    {
        name = "Kra'deshi Barbstorm",
        desc = "Apply {1} {BLEED}.\nDiscard this card: Apply {2} {BLEED} to all enemies and put this card back in your hand.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.bleed_amt, self.discard_bleed )
        end,

        anim = "attack1",
        flavour = "'Kra'deshi tend to fight with chaotically bladed weapons, relying on their healing factor to take care of superficial lacerations.'",
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE,

        cost = 1,

        bleed_amt = 2,
        discard_bleed = 2,

        deck_handlers = { DECK_TYPE.DISCARDS },

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                if card == self then
                    local target_fighters = {}
                    battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 4 )

                    if #target_fighters > 0 then
                        self:NotifyTriggered()
                        for i,target in ipairs(target_fighters) do
                            target:AddCondition("BLEED", self.discard_bleed, self)
                            target:PlayAnim( "hit_mid" )
                        end
 
                        self:AddXP(1)
                        self.move_to_hand = true
                    end
                end
            end,

            [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if self == card and self.move_to_hand and target_deck ~= self.engine:GetHandDeck() then
                    self.engine:PushPostHandler( function()
                        self.move_to_hand = false
                        self:TransferCard( self.engine:GetHandDeck() )
                    end )
                end
            end,
        },

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("BLEED", self.bleed_amt, self)
        end, 
    },

    kradeshi_barbstorm_plus = 
    {
        name = "Tall Kra'deshi Barbstorm",
        desc = "Apply <#UPGRADE>{1} {BLEED}</>.\nDiscard this card: Apply {2} {BLEED} to all enemies and put this card back in your hand.",
        bleed_amt = 4,
    },

    kradeshi_barbstorm_plus2 =
    {
        name = "Boosted Kra'deshi Barbstorm",
        desc = "Apply {1} {BLEED}.\nDiscard this card: Apply <#UPGRADE>{2} {BLEED}</> to all enemies and put this card back in your hand.",
        discard_bleed = 3,
    },

    wind_up = 
    {
        name = "Wind up",
        desc = "Discard this card: Increase the damage of this card by {1} until played and return it to your hand.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.damage_boost )
        end,

        anim = "gash",
        flavour = "'If it was so obvious then why did you get hit by it?'",
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MELEE,

        cost = 1,

        min_damage = 4,
        max_damage = 4,

        damage_boost = 4,

        deck_handlers = { DECK_TYPE.DISCARDS },

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                if card == self then
                    self.min_damage = self.min_damage + self.damage_boost
                    self.max_damage = self.max_damage + self.damage_boost
                    self.move_to_hand = true
                end
            end,

            [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if self == card and self.move_to_hand and target_deck ~= self.engine:GetHandDeck() then
                    self.engine:PushPostHandler( function()
                        self.move_to_hand = false
                        self:TransferCard( self.engine:GetHandDeck() )
                    end )
                end
            end,
        },

        OnPostResolve = function( self, battle, attack)
            self.min_damage = self.def.min_damage
            self.max_damage = self.def.max_damage
        end, 
    },

    wind_up_plus = 
    {
        name = "Twisted Wind up",
        desc = "Discard this card: Increase the damage of this card by <#DOWNGRADE>{1}</> <#UPGRADE>for the rest of this battle </>and return it to your hand.",
        damage_boost = 2,

        OnPostResolve = function( self, battle, attack)
            -- Do not reset damage!
        end,
    },

    wind_up_plus2 = 
    {
        name = "Boosted Wind up",
        min_damage = 6,
        max_damage = 6,
    },

    deepstance = 
    {
        name = "Deepstance",
        desc = "{ABILITY}: Your {RIPOSTE} attacks apply {WOUND {1}}.",
        on_play_screen_shake = true,
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.wound_amt )
        end,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

        anim = "taunt",
        flavour = "'Footwork that's popular in Jarackle wrestling, you can get a lot more heft in your attacks apparently.'",
        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        target_type = TARGET_TYPE.SELF,

        cost = 1,

        wound_amt = 1,
        
        condition = 
        {
            desc = "Your {RIPOSTE} attacks apply <#HILITE>{WOUND {1}}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks )
            end,
            event_handlers = 
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.attacker == self.owner and attack.is_counter and not hit.evaded then
                        hit.target:AddCondition( "WOUND", self.stacks, self )
                    end
                end,
            },
        },

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("deepstance", self.wound_amt, self)
        end
    },

    deepstance_plus = 
    {
        name = "Initial Deepstance",
        desc = "{ABILITY}: Your {RIPOSTE} attacks apply {WOUND {1}}.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    deepstance_plus2 =
    {
        name = "Deep Deepstance",
        desc = "{ABILITY}: Your {RIPOSTE} attacks apply <#UPGRADE>{WOUND {1}}</>.",
        wound_amt = 2,
    },

    rebound = 
    {
        name = "Rebound",
        desc = "Gain {RIPOSTE {1}} for each card played this turn.\n{3}",
        desc_fn = function( self, fmt_str )
            if self.engine then
                local str = loc.format( LOC"CARD_ENGINE.CARDS_PLAYED", self.engine:CountCardsPlayed() )
                return loc.format( fmt_str, self.riposte_amount, self.defend_amount, str )
            else
                return loc.format( fmt_str, self.riposte_amount, self.defend_amount, "" )
            end
        end,
        anim = "taunt3",
        flavour = "'Right back at you!'",

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.SKILL,

        target_type = TARGET_TYPE.SELF,

        cost = 1,

        riposte_amount = 2,
        defend_amount = 0,

        OnPostResolve = function( self, battle, attack )
            local stacks = battle:CountCardsPlayed()
            if stacks > 0 then
                if self.defend_amount > 0 then
                    self.owner:AddCondition("DEFEND", stacks * self.defend_amount, self)
                end
                self.owner:AddCondition("RIPOSTE", stacks * self.riposte_amount, self)          
            end
        end,
    },

    rebound_plus =
    {
        name = "Stone Rebound",
        desc = "Gain {RIPOSTE {1}} <#UPGRADE>and {2} {DEFEND}</> for each card played this turn.\n{3}",
        defend_amount = 1,
    },

    rebound_plus2 = 
    {
        name = "Spined Rebound",
        desc = "Gain <#UPGRADE>{RIPOSTE {1}}</> for each card played this turn.\n{3}",
        riposte_amount = 3,
    },
    
    -------------------------------------------------
    --UNIQUE FACTION CARDS

    --SPREE
    relentless = 
    {
        name = "Relentless",
        desc = "Replay every card played this turn with random targets.",
        anim = "taunt3",
        flavour = "'Hesh's Bells! Don't you ever get tired!'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNIQUE,
        cost = 2,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        --faction = "BANDITS",

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS, DECK_TYPE.TRASH },

        event_handlers =
        {
            [ BATTLE_EVENT.POST_RESOLVE] = function( self, battle, attack )
                if self.executing or attack.card == self then
                    self:ReplayNext()

                elseif attack.card.owner == self.owner then
                    self.cards_played = self.cards_played or {}
                    table.insert(self.cards_played, attack.card)
                end
            end,    

            [ battle_defs.BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                self.cards_played = nil
            end,
        },

        ReplayNext = function( self )
            local card = self.cards_played and table.remove( self.cards_played, 1 ) 
            self.engine:PrintLog( BATTLE_LOG.GENERAL, "Relentless! Replaying {1} ({2} cards left)", card, self.cards_played and #self.cards_played or 0 )
            if card == nil then
                self.executing = nil
            else
                self.executing = true
                 -- This is only here beacuse otherwise Assigning a target will fail. (see PlayCard -> CollectAllTargets)
                card:TransferCard( self.engine:GetHandDeck() )
                card:SetFlags( CARD_FLAGS.FREEBIE )
                card:NotifyTriggered()
                self.engine:PlayCard( card )
                self.engine:BroadcastEvent( BATTLE_EVENT.DELAY, 0.25 )
            end
        end,

        -- OnPostResolve = function( self, battle, attack)
        --     self.executing = true
        -- end,
    },

    thirst = 
    {
        name = "Thirst",
        desc = "{IMPROVISE} a random upgraded {BLEED} card, it costs 1 less until played.",
        anim = "taunt",
        flavour = "'Blood Striking is ruthless, cruel, and very effective.'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNIQUE,
        cost = 1,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        faction = "BANDITS",

        OnPostResolve = function( self, battle, attack)
            local card_defs = BattleCardCollection():IsUnlocked():Filter(
                function(card_def) 
                    local desc = card_def.desc or ""
                    local is_bleed = desc:match("BLEED") and true or false
                    return self.id ~= card_def.id and 
                            AgentUtil.CanUseCard(self:GetOwner().agent, card_def) and 
                            is_bleed and
                            (card_def.rarity == CARD_RARITY.COMMON or card_def.rarity == CARD_RARITY.UNCOMMON or card_def.rarity == CARD_RARITY.RARE) and
                            CheckBits( card_def.flags, CARD_FLAGS.UPGRADED ) == true
                end):Pick(3)

            local cards = table.map(card_defs, 
                    function(card_def) 
                        local card = Battle.Card(card_def.id, self:GetOwner()) 
                        card:ClearXP()
                        return card
                    end)
            local chosen_card = battle:ImproviseCards( cards, nil, nil, nil, nil, self )[1]
            local con = self.owner:GetCondition("thirst") or self.owner:AddCondition("thirst", 1, self)
            if con then 
                con.cards = con.cards or {}
                table.insert(con.cards, chosen_card)
            end

        end,

        condition =
        {
            hidden = true,

            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                    if self.cards and table.contains(self.cards, card) then
                        local count = table.count_if(self.cards, function( v ) return v == card end)
                        acc:AddValue(-1 * count, self)
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    while self.cards and table.contains(self.cards, attack.card) do
                        table.arrayremove(self.cards, attack.card)
                    end
                    if self.cards and #self.cards == 0 then
                        self.owner:RemoveCondition( self.id, self.stacks, self )
                    end
                end
            },
        },
    },

    vital_strikes = 
    {
        name = "Vital Strikes",
        desc = "Apply {WOUND 1} whenever you attack for the rest of the turn.",
        anim = "taunt3",
        flavour = "'Just below the shoulder, above the abdomen, and then finish it with anywhere on the neck.'",
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.UNIQUE,
        cost = 0,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        faction = "BANDITS",

        condition =
        {
            desc = "Apply <#HILITE>{WOUND 1}</> whenever you attack for the rest of the turn.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,
            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.attacker == self.owner and attack.card:IsAttackCard() and not hit.evaded then
                        hit.target:AddCondition( "WOUND", self.stacks )
                    end
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition( "vital_strikes" )
                end,
            }
        },

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "vital_strikes", 1, self)
        end,
    },

    assault =
    {
        name = "Assault",
        anim = "taunt2",
        flavour = "'They'll never see it coming... again.'",
        desc = "The next attack card is played twice.",
        cost = 0,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,
        target_type = TARGET_TYPE.SELF,
        faction = "BANDITS",

        OnPostResolve = function( self, battle, attack)
            local modifier = self.owner:AddCondition( "assault", 1, self )
            if modifier then
                modifier.skip_card = attack.card
            end
        end,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

        condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.EXECUTES_ADDED ] = function( self )
                    self.owner:RemoveCondition( self.id )
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if attack.card ~= self.skip_card and attack.attacker == self.owner and attack.card:IsAttackCard() then
                        self.owner:RemoveCondition( self.id, 1 )
                        battle:PlayCard( attack.card )
                    end
                end
            },
        },
    },

    terrorize = 
    {
        name = "Terrorize",
        desc = "If this attack kills an enemy or makes them {SURRENDER}, gain {POWER {1}} and {HEAL} {2}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.power_amount, self.heal_amount)
        end,
        anim = "uppercut",
        flavour = "'The Spree have a history of viciousness that permeates all of Murder Bay, it's time to get in line.'",
        faction = "BANDITS",
        
        power_amount = 3,
        heal_amount = 5,

        min_damage = 8,
        max_damage = 8,

        cost = 1,

        flags = CARD_FLAGS.MELEE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, battle, attack )
            for i, hit in ipairs( attack.hits ) do
                if hit.target and not hit.target:IsActive() then
                    self.owner:AddCondition("POWER", self.power_amount, self)
                    self.owner:HealHealth(self.heal_amount, self)

                elseif hit.target then
                    local con = self.owner:AddCondition(self.id, 1, self)
                    if con then
                        if con.target_fighters == nil then
                            con.target_fighters = {}
                        end
                        table.insert( con.target_fighters, hit.target )
                        con.heal_amount = self.heal_amount
                        con.power_amount = self.power_amount
                    end
                end
            end
        end,

        condition =
        {
            name = "",
            hidden = true,

            event_handlers = 
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition(self.id)
                end,

                [ BATTLE_EVENT.PRE_RESOLVE ] = function( self, battle, attack )
                    if attack.owner == self.owner then
                        self.owner:RemoveCondition(self.id)
                    end
                end,

                [ BATTLE_EVENT.STATUS_CHANGED ] = function( self, fighter, status )
                    if table.contains( self.target_fighters or table.empty, fighter ) and status ~= FIGHT_STATUS.ACTIVE then
                        table.arrayremove( self.target_fighters, fighter )
                        self.owner:AddCondition("POWER", self.power_amount, self)
                        self.owner:HealHealth(self.heal_amount, self)
                        self.owner:RemoveCondition(self.id, 1)
                    end
                end,
            },
        },
    },

    trained_strike =
    {
        name = "Trained Strike",
        anim = "punch",
        desc = "Whenever this card is {IMPROVISE|Improvised} it gains {1} damage until the end of combat.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_gain)
        end,
        flavour = "'One thousand times a day, every day of the week'",

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 2,
        max_damage = 4,
        damage_gain = 2,

        deck_handlers = ALL_DECKS,

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                if card == self then
                    self.bonus_damage = (self.bonus_damage or 0) + self.damage_gain
                end
            end,

            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                if card == self and self.bonus_damage then
                    dmgt:AddDamage(self.bonus_damage, self.bonus_damage, self)
                end
            end
        },
    },

    trained_strike_plus =
    {
        name = "Boosted Trained Strike",
        desc = "Whenever this card is {IMPROVISE|Improvised} it gains <#UPGRADE>{1}</> damage until the end of combat.",
        damage_gain = 3,
    },

    trained_strike_plus2 =
    {
        name = "Nailing Trained Strike",
        desc = "Whenever this card is {IMPROVISE|Improvised} it gains {1} damage until the end of combat.",
        flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,
    },

    prediction =
    {
        name = "Prediction",
        anim = "taunt",
        desc = "Apply {1} {DEFEND}. Increase by 2 for the rest of this battle whenever this card is {IMPROVISE|Improvised}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.base_defend + self.bonus_defend), self.defense_gain)
        end,
        flavour = "'Well, I wasn’t expecting that.'",

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.COMMON,

        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        base_defend = 3,
        bonus_defend = 0,
        defense_gain = 2,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.base_defend + self.bonus_defend, self)
        end,

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                if card == self then
                    self.bonus_defend = self.bonus_defend + self.defense_gain
                end
            end,
        },
    },

    prediction_plus =
    {
        name = "Patient Prediction",
        desc = "Apply {1} {DEFEND}. Increase by <#UPGRADE>3</> for the rest of this battle whenever this card is {IMPROVISE|Improvised}.",
        defense_gain = 3,
    },

    prediction_plus2 =
    {
        name = "Stone Prediction",
        desc = "Apply <#UPGRADE>{1} {DEFEND}</>. Increase by 2 for the rest of this battle whenever this card is {IMPROVISE|Improvised}.",
        base_defend = 5,
    },

    zig_zag =
    {
        name = "Zig Zag",
        desc = "{IMPROVISE} a card from your draw pile.\nGain {1} {DEFEND} if it’s a maneuver, apply {2} {WOUND} to a random enemy if it’s an attack.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt), self.wound_amt)
        end,
        flavour = "'Hold still!'",

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.SELF,

        defend_amt = 3,
        wound_amt = 1,

        OnPostResolve = function( self, battle, attack )
            if battle:GetDrawDeck():CountCards() == 0 then
                battle:ShuffleDiscardToDraw()
            end
            local cards = battle:ImproviseCards(table.multipick(battle:GetDrawDeck().cards, 3), 1, "off_hand", nil, nil, self)
            for i,card in ipairs(cards) do
                if card:IsFlagged(CARD_FLAGS.SKILL) then
                    self.owner:AddCondition("DEFEND", self.defend_amt, self)
                end
                if card:IsAttackCard() then 
                    local targets = {}
                    battle:CollectRandomTargets( targets, self.owner:GetEnemyTeam().fighters, 1 )
                    if targets[1] then
                        targets[1]:AddCondition("WOUND", self.wound_amt, self)
                    end
                end
            end
        end
    },

    zig_zag_plus =
    {
        name = "Boosted Zig Zag",
        desc = "{IMPROVISE} a card from your draw pile.\nGain <#UPGRADE>{1}</> {DEFEND} if it’s a maneuver, apply <#UPGRADE>{2}</> {WOUND} to a random enemy if it’s an attack.",
        defend_amt = 5,
        wound_amt = 2,
    },

    zig_zag_plus2 =
    {
        name = "Pale Zig Zag",
        cost = 0,
    },

    projector =
    {
        name = "Projector",
        anim = "taunt2",
        desc = "{ABILITY}: Any attack cards you {IMPROVISE} deal {1} bonus damage until played.",
        on_play_screen_shake = true,
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,
        flavour = "'It took a bit of tinkering, but your servo will have some extra punch.'",

        cost = 2,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,
        target_type = TARGET_TYPE.SELF,

        bonus_damage = 3,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("projector", self.bonus_damage, self)
        end,

        condition =
        {
            desc = "Any attack cards you {IMPROVISE} deal <#HILITE>{1}</> bonus damage until played.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                    if card.owner == self.owner then
                        card.projector_bonus = (card.projector_bonus or 0) + self.stacks
                    end
                end,

                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card and type(card.projector_bonus) == "number" then
                        dmgt:AddDamage( card.projector_bonus, card.projector_bonus, self )
                    end
                end,

                [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, card )
                    if card and card.projector_bonus ~= nil then
                        card.projector_bonus = nil
                    end
                end,
            },
        },
    },

    projector_plus =
    {
        name = "Initial Projector",
        desc = "{ABILITY}: Any attack cards you {IMPROVISE} deal {1} bonus damage until played.",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    projector_plus2 =
    {
        name = "Pale Projector",
        cost = 1,
    },

    crusher =
    {
        name = "Crusher",
        anim = "uppercut",
        desc = "Whenever you {IMPROVISE} while this card is in your hand, reduce its cost by 1 for the rest of this battle.",
        flavour = "'Yeah it was Kleeta’s move back on the derrick, I just make it look better.'", 

        cost = 4,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 6,
        max_damage = 10,

        deck_handlers = ALL_DECKS,

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                if self.deck and self.deck:GetDeckType() == DECK_TYPE.IN_HAND and card ~= self then
                    self.count = (self.count or 0) + 1
                end
            end,

            [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self and self.count then
                    cost_acc:AddValue(-self.count, self)
                end
            end
        },
    },

    crusher_plus =
    {
        name = "Rooted Crusher",
        min_damage = 10,
    },

    crusher_plus2 = 
    {
        name = "Pale Crusher",
        cost = 3,
    },

    resonance =
    {
        name = "Resonance",
        desc = "{ABILITY}: Whenever you {IMPROVISE} a card, a random enemy gains {1} {RESONATED}.",
        on_play_screen_shake = true,
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.stacks)
        end,
        target_type = TARGET_TYPE.SELF,
        rarity = CARD_RARITY.RARE,
        cost = 2,
        flavour = "'There was a high pitched noise, and then my chest plate started malfunctioning.'", 

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        stacks = 1,

        condition =
        {
            name = "Resonator",
            desc = "Whenever you {IMPROVISE} a card, a random enemy gains <#HILITE>{1} {RESONATED}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,

            apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

            Trigger = function( self, battle )
                local target_fighters = {}
                battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )

                for i=1, #target_fighters do
                    target_fighters[i]:AddCondition( "RESONATED", self.stacks )
                end
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                    self:Trigger( battle )
                end,
            },
        },

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition( "resonance", self.stacks, self )
        end,
    },

    resonance_plus =
    {
        name = "Boosted Resonance",
        desc = "{ABILITY}: Whenever you {IMPROVISE} a card, a random enemy gains <#UPGRADE>{1}</> {RESONATED}.",
        stacks = 2,
    },

    resonance_plus2 =
    {
        name = "Wide Resonance",
        desc = "{ABILITY}: Whenever you {IMPROVISE} a card, <#UPGRADE>ALL</> enemies gain {1} {RESONATED}.",

        condition =
        {
            name = "Wide Resonator",
            desc = "Whenever you {IMPROVISE} a card, ALL enemies gain <#HILITE>{1} {RESONATED}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,
            icon = "battle/conditions/resonance.tex",

            apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

            Trigger = function( self, battle )
                for i, fighter in self.owner:GetEnemyTeam():Fighters() do
                    fighter:AddCondition( "RESONATED", self.stacks )
                end
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                    self:Trigger( battle )
                end,
            },
        },

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition( "resonance_plus2", self.stacks, self )
        end,
    },

    the_slip = 
    {
        name = "The Slip",
        anim = "taunt",
        desc = "Apply {1} {DEFEND}. If this card was {IMPROVISE|Improvised} this turn, also gain {2} {EVASION}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:CalculateDefendText(self.defend_amt), self.evasion_amt)
        end,
        flavour = "'She’s too quick! I can’t get a bead on her!'",

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.UNCOMMON,

        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        defend_amt = 4,
        evasion_amt = 1,

        PreReq = function( self, battle )
            return self.improvised
        end,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
            if self.improvised then
                self.owner:AddCondition("EVASION", self.evasion_amt, self)
            end
        end,

        deck_handlers = ALL_DECKS,

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                if card == self then
                    self.improvised = true
                end
            end,

            [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                self.improvised = false
            end
        },
    },

    the_slip_plus = 
    {
        name = "The Stone Slip",
        desc = "Apply <#UPGRADE>{1}</> {DEFEND}. If this card was {IMPROVISE|Improvised} this turn, also gain {2} {EVASION}.",
        defend_amt = 8,
    },

    the_slip_plus2 =
    {
        name = "The Evasive Slip",
        desc = "Apply {1} {DEFEND}. If this card was {IMPROVISE|Improvised} this turn, also gain <#UPGRADE>{2}</> {EVASION}.",
        evasion_amt = 2,
    },

    blademouth_beating =
    {
        name = "Blademouth Beating",
        anim = "uppercut",
        desc = "If this card was {IMPROVISE|Improvised} this turn, attack three times. Apply {1} {BLEED} per attack.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bleed_amt)
        end,
        flavour = "'Blades forged on the Derricks had considerably more heft to them on the mainland.'",

        cost = 1,

        flags = CARD_FLAGS.MELEE,
        rarity = CARD_RARITY.UNCOMMON,

        min_damage = 1,
        max_damage = 2,
        bleed_amt = 2,

        PreReq = function( self, battle )
            return self.improvised
        end,

        OnPostResolve = function( self, battle, attack )
            for i,hit in ipairs(attack.hits) do
                if not hit:CheckHitResult( "evaded" ) then
                    hit.target:AddCondition("BLEED", self.bleed_amt, self)
                end
            end
        end,
        deck_handlers = ALL_DECKS,

        event_handlers =
        {
            [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                if card == self then
                    self.improvised = true
                    self.hit_count = 3
                    self.hit_anim = true
                end
            end,

            [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                self.improvised = false
                self.hit_count = 1
                self.hit_anim = false
            end
        },
    },

    blademouth_beating_plus =
    {
        name = "Boosted Blademouth Beating",
        min_damage = 3,
        max_damage = 3,
    },

    blademouth_beating_plus2 =
    {
        name = "Savage Blademouth Beating",
        desc = "If this card was {IMPROVISE|Improvised} this turn, attack three times. Apply <#UPGRADE>{1}</> {BLEED} per attack.",
        bleed_amt = 3,
    },

    blood_guard =
    {
        name = "Blood Guard",
        desc = "For the rest of your turn, gain {DEFEND} equal to the amount of {BLEED} you apply.",
        flavour = "'The violence she brought on made it difficult for anyone to approach her.'",

        cost = 1,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        target_type = TARGET_TYPE.SELF,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("blood_guard", 1, self)
        end,

        condition =
        {
            desc = "For the rest of your turn, gain {DEFEND} equal to {1* |{1} times }the amount of {BLEED} you apply.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks or 1)
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if self.battle:GetFighterFromSource(source) == self.owner and condition.id == "BLEED" then
                        self.owner:AddCondition("DEFEND", stacks * self.stacks, self)
                    end
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:RemoveCondition(self.id, self.stacks, self)
                end
            },
        },
    },

    blood_guard_plus =
    {
        name = "Pale Blood Guard",
        cost = 0,
    },

    blood_guard_plus2 =
    {
        name = "Enduring Blood Guard",
        flags = CARD_FLAGS.SKILL,
    },

    waylay =
    {
        name = "Waylay",
        desc = "Gain {1} {DEFEND} and insert {2} {blade_flash} {2*card|cards} into your hand.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt), self.num_cards)
        end,
        flavour = "’I don’t know how many she has!’",

        cost = 1,

        flags = CARD_FLAGS.SKILL,
        rarity = CARD_RARITY.RARE,

        target_type = TARGET_TYPE.SELF,

        defend_amt = 8,
        num_cards = 1,

        OnPostResolve = function( self, battle, attack )
            self.owner:AddCondition("DEFEND", self.defend_amt, self)
            local cards = {}
            for i = 1, self.num_cards do
                local incepted_card = Battle.Card( "blade_flash", self:GetOwner() )
                incepted_card.auto_deal = true
                table.insert( cards, incepted_card )
            end
            battle:DealCards( cards, battle:GetHandDeck() )
        end,
    },

    waylay_plus =
    {
        name = "Stone Waylay",
        desc = "Gain <#UPGRADE>{1} {DEFEND}</> and insert {2} {blade_flash} {2*card|cards} into your hand.",
        defend_amt = 12,
    },

    waylay_plus2 =
    {
        name = "Wide Waylay",
        desc = "Gain {1} {DEFEND} and insert <#UPGRADE>{2} {blade_flash} {2*card|cards}</> into your hand.",
        num_cards = 2,
    },

    phantom_finisher =
    {
        name = "Phantom Finisher",
        desc = "Spend 1 {COMBO}: Gain {1} {DEFEND}.",
        flavour = "'The first thing you learn.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt))
        end,

        cost = 0,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.COMBO_FINISHER | CARD_FLAGS.DUD,
        rarity = CARD_RARITY.UNIQUE,
        target_type = TARGET_TYPE.SELF,

        defend_amt = 3,

        OnPostResolve = function( self, battle, attack )
            if self.owner:GetConditionStacks("COMBO") > 0 then
                self.owner:RemoveCondition("COMBO", 1, self)
                attack:AddCondition("DEFEND", self.defend_amt, self)
            end
        end
    },

    murder_bay_boxing =
    {
        name = "Murder Bay Boxing",
        anim = "taunt3",
        flavour = "'The toughest ones are always right off the Derrick.'",
        desc = "{ABILITY}: At the start of your turn, draw an additional card if you have at least {1} {COMBO}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, 2)
        end,

        rarity = CARD_RARITY.UNCOMMON,
        on_play_screen_shake = true,

        cost = 2,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "murder_bay_boxing", 1, self )
        end,

        condition =
        {
            desc = "At the start of your turn, draw <#HILITE>{1}</> additional {1*card|cards} if you have at least {2} {COMBO}.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks or 1, 2)
            end,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner and self.owner:IsPlayer() and self.owner:GetConditionStacks("COMBO") >= 2 then
                        self.battle:DrawCards( self.stacks )
                    end
                end,
            }
        }
    },
    murder_bay_boxing_plus =
    {
        name = "Initial Bay Boxing",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },
    murder_bay_boxing_plus2 =
    {
        name = "Rival's Bay Boxing",
        desc = "{ABILITY}: At the start of your turn, draw an additional card if you have at least {1} {COMBO}.\n<#UPGRADE>Gain 2 {COMBO}.</>",

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "murder_bay_boxing", 1, self )
            self.owner:AddCondition( "COMBO", 2, self )
        end,
    },

    jarackle_free_fighting =
    {
        name = "Jarackle Free-Fighting",
        anim = "taunt3",
        flavour = "'This jewelry isn’t just for looks you know.'",
        desc = "{ABILITY}: While you have at least {2} {COMBO}, gain {POWER {1}}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.stacks, 4 )
        end,
        rarity = CARD_RARITY.COMMON,
        on_play_screen_shake = true,

        cost = 2,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        stacks = 2,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "jarackle_free_fighting", self.stacks, self )
        end,

        condition =
        {
            desc = "While you have at least {2} {COMBO}, gain <#HILITE>{1}</> {POWER}.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks or 1, 4)
            end,

            OnSetStacks = function( self, delta, old_stacks, stacks )
                if self.owner:GetConditionStacks("COMBO") >= 4 then
                    self.owner:DeltaCondition("POWER", stacks - old_stacks, self)
                end
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if fighter == self.owner and condition.id == "COMBO" then
                        if condition:GetStacks() >= 4 and condition:GetStacks() - stacks < 4 then
                            fighter:AddCondition("POWER", self.stacks, self)
                        end
                    end
                end,
                [ BATTLE_EVENT.CONDITION_REMOVED ] = function( self, fighter, condition, stacks, source )
                    if fighter == self.owner and condition.id == "COMBO" then
                        if condition:GetStacks() < 4 and condition:GetStacks() + stacks >= 4 then
                            fighter:RemoveCondition("POWER", self.stacks, self)
                        end
                    end
                end,
            }
        }
    },
    jarackle_free_fighting_plus =
    {
        name = "Evasive Free-Fighting",
        desc = "<#UPGRADE>Gain 1 {EVASION}.</>\n{ABILITY}: While you have at least {2} {COMBO}, gain {POWER {1}}.",

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "jarackle_free_fighting", 2, self )
            self.owner:AddCondition( "EVASION" )
        end,
    },
    jarackle_free_fighting_plus2 =
    {
        name = "Boosted Free-Fighting",
        desc = "{ABILITY}: While you have at least {2} {COMBO}, gain <#UPGRADE>{POWER {1}}</>.",
        stacks = 3,
    },

    absolute_domination =
    {
        name = "Absolute Domination",
        anim = "taunt3",
        flavour = "'This is brutal to watch.'",
        desc = "{ABILITY}: While you have at least {1} {COMBO}, play your first attack each turn twice.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, 8 )
        end,
        rarity = CARD_RARITY.RARE,
        on_play_screen_shake = true,

        cost = 3,
        target_type = TARGET_TYPE.SELF,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            self.owner:AddCondition( "absolute_domination", 1, self )
        end,

        condition =
        {
            desc = "{ABILITY}: While you have at least {1} {COMBO}, play your first attack <#HILITE>{2}</> extra {1*time|times}.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, 8, self.stacks )
            end,
            event_handlers =
            {
                [ BATTLE_EVENT.EXECUTES_ADDED ] = function( self )
                    self.owner:RemoveCondition( self.id )
                end,

                -- NOTE: This is START_RESOLVE so that the combo evaluation happens BEFORE the played cards consume their combo.
                [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                    if self.owner:GetConditionStacks("COMBO") >= 8 and card.owner == self.owner and card:IsAttackCard() then
                        local cards_played = battle:GetCardsPlayed()
                        if table.count_if( cards_played, function( card ) return card:IsAttackCard() end ) < self.stacks then
                            battle:PlayCard( card )
                        end
                    end
                end
            },
        },
    },

    absolute_domination_plus =
    {
        name = "Initial Domination",
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    absolute_domination_plus2 =
    {
        name = "Pale Domination",
        cost = 2,
    },

}

for i, id, data in sorted_pairs(attacks) do
    data.series = "SAL"
    
    Content.AddBattleCard( id, data )
end

----------------------------------------------------------------

local CONDITIONS =
{
    RESONATED =
    {
        name  = "Resonance",
        desc = "Whenever a card is {IMPROVISE|Improvised}, lose <#HILITE>{1} health</>.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.stacks )
        end,

        icon = "battle/conditions/resonance.tex",
        ctype = CTYPE.DEBUFF,

        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

        event_handlers = 
        {
            [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                local attacker = self:GetRecentApplicant()
                if attacker and not is_instance( attacker, Fighter ) then
                    attacker = attacker.owner
                end

                self.owner:ApplyDamage( self.stacks, attacker, nil, true, {"spark"} )
            end,
        },
    },
}


for id, def in pairs( CONDITIONS ) do
    Content.AddBattleCondition( id, def )

end