------------------------------------------------------------------------
-- Unlocks can be of the following types:
--
--      player_background: specifies the id of a starting character
--
--      battle_card: specifies the id of a Battle.Card
--
--      negotiation_card: specifies the id of a Negotiation.Card
--
--      graft_id: specifies the id of a graft
--
--      unlock_id: an arbitrary (not-clashing) string identifier for a "generic" unlock.
--          icon: a texture path to represent this generic unlock

--      deck_id: a starting deck


local ROOK_TRACK = UnlockTrackDef{
    UnlockableDef{ 
        --ONE
        id = "fixated",
        name = "Gambler and Fixated",
        negotiation_name = "Gambler",
        battle_name = "Fixated",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 500,
        items = 
        {
            { negotiation_card = "dilemma"},
            { negotiation_card = "gab"},
            { negotiation_card = "spotty"},
            { battle_card = "focused_strike"},
            { battle_card = "gunslinger"},
            { battle_card = "zero"},
        }
    },
    
    UnlockableDef{
        --TWO
        id = "transaction",
        name = "Discard and Transaction",
        negotiation_name = "Discard",
        battle_name = "Transaction",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 1000,
        items = 
        {
            { negotiation_card = "snail_in_the_pocket"},
            { negotiation_card = "kicker"},
            { negotiation_card = "disregard"},
            { battle_card = "scatterphase"},
            { battle_card = "tank"},
            { battle_card = "blinders"},
            { outfit_id = "rook_outfit_01" },
        }
    },
    
    UnlockableDef{
        --THREE
        id = "stocked",
        name = "Spender and Stocked",
        negotiation_name = "Spender",
        battle_name = "Stocked",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 1600,
        items = 
        {
            { negotiation_card = "trade"},
            { negotiation_card = "venture"},
            { negotiation_card = "prominence"},
            { battle_card = "fully_loaded"},
            { battle_card = "gallery"},
            { battle_card = "gemini"},
        }
    },

    UnlockableDef{
        --FOUR
        id = "threshold",
        name = "Defensive and Threshold",
        negotiation_name = "Defensive",
        battle_name = "Threshold",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 2250,
        items = 
        {
            { negotiation_card = "matter_of_fact"},
            { negotiation_card = "blockade"},
            { negotiation_card = "prattle"},
            { battle_card = "chimney"},
            { battle_card = "striker"},
            { battle_card = "trick_shot"},
            { outfit_id = "rook_outfit_02" },
        }
    },

    UnlockableDef{
        --FIVE
        id = "migrainer",
        name = "Livid and Migrainer",
        negotiation_name = "Livid",
        battle_name = "Migrainer",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 2900,
        items = 
        {
            { negotiation_card = "seethe"},
            { negotiation_card = "ante"},
            { negotiation_card = "tantrum"},
            { battle_card = "muddle"},
            { battle_card = "offset"},
            { battle_card = "cataclysm"},
            { outfit_id = "rook_outfit_03" },
        }
    }, 

    UnlockableDef{
        --SIX
        id = "surged",
        name = "Inclined and Overcharged",
        negotiation_name = "Inclined",
        battle_name = "Overcharged",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 3600,
        items = 
        {
            { negotiation_card = "decency"},
            { negotiation_card = "weight"},
            { negotiation_card = "obligation"},
            { battle_card = "shock_therapy"},
            { battle_card = "jolt"},
            { battle_card = "arc_deflection"},
        }
    },
    
    UnlockableDef{
        --SEVEN
        id = "shoot_out",
        name = "Pickup and Shoot Out",
        negotiation_name = "Pickup",
        battle_name = "Shoot Out",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 4400,
        items = 
        {
            { negotiation_card = "drawback"},
            { negotiation_card = "radula"},
            { negotiation_card = "square_up"},
            { battle_card = "shill_shot"},
            { battle_card = "spurs"},
            { battle_card = "take_cover"},
        }
    },
    
    UnlockableDef{
        --EIGHT
        id = "brain_surgery",
        name = "Prepared and Brain Surgery",
        negotiation_name = "Prepared",
        battle_name = "Brain Surgery",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 5200,
        items = 
        {
            { negotiation_card = "compromise"},
            { negotiation_card = "upright"},
            { negotiation_card = "strategy"},
            { battle_card = "ego_rip"},
            { battle_card = "brain_tick"},
            { battle_card = "psionic_storm"},
        }
    },

    UnlockableDef{
        --NINE
        id = "options",
        name = "Activator and Options",
        negotiation_name = "Activator",
        battle_name = "Options",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 6000,
        items = 
        {
            { negotiation_card = "head_shot"},
            { negotiation_card = "snail_bite"},
            { negotiation_card = "red_herring"},
            { battle_card = "squeeze"},
            { battle_card = "fistful"},
            { battle_card = "lifeline"},
        }
    },

    UnlockableDef{
        --TEN
        id = "swing",
        name = "Support and Swing",
        negotiation_name = "Support",
        battle_name = "Swing",
        icon = "battle/decks/card_unlock_sal.tex",
        icon_locked = "battle/decks/card_unlock_sal_locked.tex",
        pts = 7000,
        items = 
        {
            { negotiation_card = "fallout"},
            { negotiation_card = "do_over"},
            { negotiation_card = "pinch"},
            { battle_card = "firestorm"},
            { battle_card = "vent"},
            { battle_card = "clear_shot"},
        }
    },
}


Content.AddUnlockTrack("ROOK", ROOK_TRACK)
