local HEAL_AMOUNT = 20
local HEAL_RESOLVE_AMOUNT = 10

local function CalcScore(scores, def)
    local score = 1
    for k,v in pairs(scores) do
        if CheckBits( (def.item_tags or 0), k ) then
            score = score + v
        end
    end
    return score
end

local RENOWN_RANGES = 
{
    {CARD_RARITY.COMMON, CARD_RARITY.COMMON},
    {CARD_RARITY.COMMON, CARD_RARITY.UNCOMMON},
    {CARD_RARITY.COMMON, CARD_RARITY.UNCOMMON},
    {CARD_RARITY.UNCOMMON, CARD_RARITY.RARE},
    {CARD_RARITY.UNCOMMON, CARD_RARITY.RARE},
}

local BATTLE_RANGES = 
{
    {CARD_RARITY.COMMON, CARD_RARITY.COMMON},
    {CARD_RARITY.COMMON, CARD_RARITY.UNCOMMON},
    {CARD_RARITY.COMMON, CARD_RARITY.UNCOMMON},
    {CARD_RARITY.UNCOMMON, CARD_RARITY.RARE},
    {CARD_RARITY.UNCOMMON, CARD_RARITY.RARE},
}

local function GetSellingItem(agent)

    local rarity

    local IGNORE_MASK = ITEM_TAGS.MONSTER
    local REQUIRE_MASK = 0
    
    local scores = {}


    if agent:GetFactionID() ~= "ADMIRALTY" then
        IGNORE_MASK = IGNORE_MASK | ITEM_TAGS.ADMIRALTY
    end

    if agent:GetFaction():IsLawful() then
        IGNORE_MASK = IGNORE_MASK | ITEM_TAGS.ILLICIT
    else
        scores[ITEM_TAGS.ILLICIT] = 5
    end

    if agent:GetCombatStrength() > agent:GetRenown() then
        scores[ITEM_TAGS.OFFENCE] = 5
    elseif agent:GetCombatStrength() < agent:GetRenown() then
        scores[ITEM_TAGS.DEFENCE] = 5
    end

    local MIN_RARITY_BATTLE, MAX_RARITY_BATTLE = table.unpack( RENOWN_RANGES[math.round( agent:GetRenown())])
    local MIN_RARITY_NEGOTIATION, MAX_RARITY_NEGOTIATION = table.unpack( BATTLE_RANGES[math.round( agent:GetCombatStrength())])
    
    local battle_cards = BattleCardCollection.AllLocalItems(function(def) 
            return CARD_RARITY_ORDINALS[def.rarity] <= CARD_RARITY_ORDINALS[MAX_RARITY_BATTLE] and CARD_RARITY_ORDINALS[def.rarity] >= CARD_RARITY_ORDINALS[MIN_RARITY_BATTLE] and
                    
                    (not def.item_tags or not CheckAnyBits( def.item_tags, IGNORE_MASK )) and
                    CheckBits(def.item_tags or 0, REQUIRE_MASK)
        end)

    local negotiation_cards = NegotiationCardCollection.AllLocalItems(function(def) 
            return CARD_RARITY_ORDINALS[def.rarity] <= CARD_RARITY_ORDINALS[MAX_RARITY_NEGOTIATION] and CARD_RARITY_ORDINALS[def.rarity] >= CARD_RARITY_ORDINALS[MIN_RARITY_NEGOTIATION] and
                    (not def.item_tags or not CheckAnyBits( def.item_tags, IGNORE_MASK )) and
                    CheckBits(def.item_tags or 0, REQUIRE_MASK)
        end)
    
    local candidates = {}
    for k,v in pairs(battle_cards.items) do
        local score = CalcScore(scores, v)
        if score > 0 then 
            candidates[v.id] = score
        end
    end
    
    for k,v in pairs(negotiation_cards.items) do
        local score = CalcScore(scores, v)
        if score > 0 then 
            candidates[v.id] = score
        end
    end
    
    if next(candidates) then
        return weightedpick(candidates)
    end

end

local function GetAssassinationTargets( player, agent )
    local targets = player.social_connections:GetConnectionsOfLevel( RELATIONSHIP.HATED )
    for i = #targets, 1, -1 do
        local target = targets[i]
        if target:IsInPlayerParty() or
            target:IsRetired() or
            AgentUtil.HasPlotArmour( target ) or
            target:IsWithAgent( agent ) then
            --
            table.remove( targets, i )
        end
    end
    return targets
end


BOON_SERVICES = 
{

    GIVE_ITEM = 
    {
        on_init = function(quest) 
            quest.param.item = GetSellingItem(quest:GetCastMember("friend"))
        end,

        name = function(quest)
            return loc.format( LOC"CONVO.OPPO_FRIEND_BOON.GIVE_ITEM.OPT_GIVE_ITEM", quest.param.item )
        end,
        
        desc = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.GIVE_ITEM.DESC"
        end,

        collect_tooltip_items = function(quest, t) table.insert(t, quest.param.item ) end,


        txt = {
            DESC = "Your friend {friend} has a gift for you.",

            OPT_GIVE_ITEM = "Get {1#card}",
            OPT_DONT_TAKE = "Refuse",
            DIALOG_INTRO = [[
                agent:
                    !greeting
                    $happyDefault
                    Hey, {player}! I found something the other day. Made me think of you.
                    You don't have to take it if it'll bog you down, but it's yours.
            ]],
            DIALOG_GAVE = [[
                player:
                    !take
                    $happyThanks
                    Sure, {agent}. Thanks.
                agent:
                    !give 
                    $happyDefault
                    Put that to good use, yeah?
            ]],
            DIALOG_DID_NOT_TAKE = [[
                player:
                    !shrug
                    $neutralThanks
                    I travel light. Sorry.
                agent:
                    !placate
                    Hey, no harm. I'll use it myself, then.
            ]],
        },
        icon =  engine.asset.Texture( "icons/quests/oppo_friend_give_item.tex"),
        fn = function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_GIVE_ITEM", cxt.quest.param.item)
                :PreIcon( global_images.receiving )
                --:Quip( cxt:GetAgent(), "sell_item", cxt.quest.param.item )
                :GainCards{cxt.quest.param.item}
                :Dialog("DIALOG_GAVE")
            cxt:Opt("OPT_DONT_TAKE")
                :PreIcon( global_images.close )
                :Dialog("DIALOG_DID_NOT_TAKE")
                :DoneConvo()
        end
    },

    SOCIALIZE = 
    {
        on_init = function(quest)
            quest.param.restoration = HEAL_RESOLVE_AMOUNT
        end,

        name = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.SOCIALIZE.NAME"
        end,

        can_offer = function(agent) 
            return agent:GetRelationship() == RELATIONSHIP.LIKED
        end,
        
        desc = function(quest)
            return loc.format( LOC"CONVO.OPPO_FRIEND_BOON.SOCIALIZE.DESC", nil, quest.param.restoration )
        end,

        txt = 
        {
            NAME = "Socialize with {friend}",
            DESC = "{friend} is feeling friendly. Spending some time with {friend.himher} will restore <#RESOLVE>{2} Resolve</>, and make {friend.himher} like you more.",

            DIALOG_INTRO = [[
                agent:
                    !greeting
                    $happyGreeting
                    {player}! I was just thinking about you.
                    You busy?
            ]],
            OPT_SOCIALIZE = "Chat with {agent}",
            TT_SOCIALIZE = "Restore <#RESOLVE>{1} Resolve</>",
            DIALOG_SOCIALIZE = [[
                * {agent} chats with you a while.
            ]],
            REQ_NOT_TIRED = "You already have full Resolve",
        },
        icon =  engine.asset.Texture( "icons/quests/oppo_friend_socialize.tex"),
        fn = function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            local resolve, max_resolve = cxt.caravan:GetResolve()
            cxt:Opt("OPT_SOCIALIZE")
                :PreIcon(global_images.like)
                :DeltaResolve( cxt.quest.param.restoration )
                :Dialog( "DIALOG_SOCIALIZE" )
                :Fn( function( cxt )
                    if cxt:GetAgent().smalltalk then
                        cxt:Dialog( cxt:GetAgent().smalltalk )
                    else
                        cxt.encounter:SayQuip( cxt:GetAgent(), "smalltalk" )
                    end
                end )
                :ReceiveOpinion(OPINION.SOCIALIZED)
                :Fn(function()
                    TheGame:GetEvents():BroadcastEvent( "socialize", cxt.quest.param.restoration, cxt:GetAgent() ) 
                end)
                :DoneConvo()
        end
    },


    HEALING = 
    {

        on_init = function(quest)
            quest.param.restoration = HEAL_AMOUNT
        end,
        

        name = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.HEALING.NAME"
        end,

        desc = function(quest)
            return loc.format( LOC"CONVO.OPPO_FRIEND_BOON.HEALING.DESC", nil, quest.param.restoration)
        end,

        can_offer = function(agent) 
            return TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):GetPercent() < 1
        end,

        txt = 
        {
            NAME = "Get healed by {friend}",
            DESC = "You're injured. {friend} can heal you for <#HEALTH>{2} HP</>",

            DIALOG_INTRO = [[
                agent:
                    !dubious
                    $neutralDubious
                    Hesh, {player}, you aren't looking too good.
                    $miscNostalgic
                    You want some help with that? I got an old remedy my pappy always used.
            ]],
            OPT_HEAL = "Heal for {1} HP",
            DIALOG_REQUEST_HEALING = [[
                player:
                    !cringe
                    $neutralThoughtful
                    I <i>am</i> a little stiff. 
                agent:
                    Just sit tight. This'll be over in a jiff.
                * {agent} pulls out a gray paste that smells like pickled sea slug. 
            ]],
            DIALOG_DONE = [[
                * By the time {agent.heshe}'s done, the ache in your joints has been rubbed away.
                agent:
                    !happy
                    $happyDefault
                    Better than new! It might take a few days for the smell to wash off, mind.
                player:
                    !thought
                    $neutralThoughtful
                    Ah... ha.
            ]],
        },
        icon =  engine.asset.Texture( "icons/quests/oppo_friend_heal.tex"),
        fn = function(cxt)
            
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_HEAL", cxt.quest.param.restoration)
                :Dialog("DIALOG_REQUEST_HEALING")
                :DeltaHealth( cxt.quest.param.restoration )
                :Dialog("DIALOG_DONE")
                :DoneConvo()
        end
    },


    ASSASSINATE =
    {

        name = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.ASSASSINATE.NAME"
        end,

        desc = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.ASSASSINATE.DESC"
        end,

        can_offer = function( agent )
            local player = TheGame:GetGameState():GetPlayerAgent()
            local targets = GetAssassinationTargets( player, agent )
            return #targets > 0
        end,

        txt = 
        {
            NAME = "Assassination Ploy",
            DESC = "{friend} owes you, and is willing to make one of your enemies disappear.",

            DIALOG_INTRO = [[
                agent:
                    !cagey
                    $miscHushedUrgent
                    Pssst, {player}. I've heard you're having some trouble with {1#agent}.
                    !wink
                    $miscMischievous
                    I still owe you. So if you want...
                player:
                    Yeah, I get what you're implying.
            ]],
            OPT_HIRE = "Ask {agent} to assassinate {1#agent}",
            DIALOG_ACCEPT_HIRE = [[
                player:
                    !shrug
                    If you can do it, then I say... go for it.
                    But don't take any unnecessary risks, yeah?
                    I don't want any of this getting back to me.
                agent:
                    !happy
                    $miscMischievous
                    Me? Never. I'll make good. I promise.
                * {agent} slinks off, looking a bit too eager for your tastes.
                    !exit
            ]],
            OPT_DONT_TAKE = "Refuse",
            DIALOG_DID_NOT_TAKE = [[
                player:
                    !handwave
                    $neutralThanks
                    Naw, too hot. Besides, no need for you to clean up my mess.
                agent:
                    !shrug
                    $neutralWhatever
                    Your call, of course. 
            ]]
        },
        icon =  engine.asset.Texture( "icons/quests/oppo_assassination.tex"),
        fn = function(cxt)
            local targets = GetAssassinationTargets( cxt.player, cxt:GetAgent() )
            local target = table.arraypick( targets )

            cxt:Dialog("DIALOG_INTRO", target )
            cxt:Opt("OPT_HIRE", target )
                :Dialog("DIALOG_ACCEPT_HIRE" )
                :Fn( function()
                    TheGame:GetGameState():LogNotification( NOTIFY.ENEMY_KILLED, target ) 
                    target:Kill()
                    cxt:GetAgent():MoveToLimbo()
                end )
                :DoneConvo()

            
            cxt:Opt("OPT_DONT_TAKE")
                :Dialog("DIALOG_DID_NOT_TAKE")
                :DoneConvo()
        end
    },

    HIRE = 
    {
        name = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.HIRE.NAME"
        end,
        desc = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.HIRE.DESC"
        end,

        can_offer = function( agent )
            if not TheGame:GetGameState():IsHiringAvailable() then
                return false
            end

            if agent:GetBrain():GetWorkPosition() or agent:GetCombatStrength() > TheGame:GetGameState():GetCurrentBaseDifficulty() then
                return false
            end

            if TheGame:GetGameState():GetCaravan():GetHireling() then
                return false
            end
            return true
        end,
        txt = 
        {
            NAME = "{friend} wants to join you",
            DESC = "{friend} is looking for something to do today, if you need helpful company.",

            DIALOG_INTRO = [[
                agent:
                    !greeting
                    $happyGreeting
                    {player}! Just the hunter I wanted to see. I'm bored, and you always keep things interesting.
                    You need any help with something?
            ]],
            OPT_HIRE = "Let {agent} tag along",
            DIALOG_ACCEPT_HIRE = [[
                player:
                    !happy
                    $happyDefault
                    Always happy to have you along, {agent}.
                agent:
                    !chuckle
                    $miscMischievous
                    Great! Just, uh, don't get me killed.
                player:
                    !placate
                    Whoa, no promises.
                agent:
                    Hah! Lead the way.
                    !exit
                player:
                    !disappoint
                    $neutralResigned
                    I wasn't kidding.
                * {agent} joins you.
            ]],
            OPT_DONT_TAKE = "Refuse",
            DIALOG_DID_NOT_TAKE = [[
                player:
                    !placate
                    $neutralResigned
                    Dunno, {agent}. The stuff I have planned is pretty dangerous.
                    Might be more than you bargained for.
                agent:
                    !thought
                    $neutralThoughtful
                    Ah, good point. Sometimes your brand of fun is a mite suicidal for my tastes.
                    Maybe some other time, eh, {player}?
            ]]
        },
        icon =  engine.asset.Texture( "icons/quests/oppo_friend_hire.tex"),
        fn = function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_HIRE")
                :RecruitMember( PARTY_MEMBER_TYPE.HIRED )
                :Dialog("DIALOG_ACCEPT_HIRE")
                :DoneConvo()
            
            cxt:Opt("OPT_DONT_TAKE")
                :Dialog("DIALOG_DID_NOT_TAKE")
                :DoneConvo()
        end
    },

    UPGRADE_NEGOTIATION_CARD = 
    {

        on_init = function(quest)
            local cards = TheGame:GetGameState():GetPlayerAgent().negotiator:GetCards()
            for i=1, #cards do
                if cards[i]:HasUpgrades() and not cards[i]:UpgradeReady() and not cards[i]:IsHatched() then
                    return true
                end                        
            end


        end,
        
        name = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.UPGRADE_NEGOTIATION_CARD.NAME"
        end,
        
        desc = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.UPGRADE_NEGOTIATION_CARD.DESC"
        end,

        txt = {
            NAME = "Debate with {friend}",
            DESC = "Practice debate with {friend} to upgrade one of your negotiation cards. This will cost some resolve.",

            DIALOG_INTRO = [[
                agent:
                    !greeting
                    $happyGreeting
                    Oh, hey {player}. Thanks again for your help.
                    I picked up a new trick recently I think you'll really like. Wanna hear it?
            ]],
            OPT_UPGRADE_CARD = "Upgrade <!card_1>{1}</>",

            DIALOG_UPGRADED = [[
                * {agent} launches into the nuances of debate. You lose a bit in the frenzy but, hey, you pick up a bit, too.
                player:
                    !thought
                    $happyAmazed
                    Huh. I think I might try that out. 
                agent:
                    !wink
                    $happyDefault
                    Knock 'em dead out there, {player}. Err, metaphorically, of course. 
                    !dubious
                    $neutralDubious
                    If you do it literally then I don't think you've been puttin' this lesson to use.
            ]],
            OPT_DONT_TAKE = "Refuse",
            
            DIALOG_DID_NOT_TAKE = [[
                player:
                    !thumb
                    No thanks. I've got enough stuff in my head as it is.
                agent:
                    $neutralWhatever
                    Suit yourself, {player}.
            ]],

        },
        icon =  engine.asset.Texture( "icons/quests/oppo_friend_debate.tex"),
        fn = function(cxt)

            local cards = TheGame:GetGameState():GetPlayerAgent().negotiator:GetCards()
            local options = {}
            for i=1, #cards do
                if cards[i]:HasUpgrades() and not cards[i]:UpgradeReady() and not cards[i]:IsHatched() then
                    table.insert(options, cards
                        [i])
                end                        
            end
            table.shuffle(options)
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoopingFn( function()
                for k = 1, math.min( 3, #options) do
                    
                    local card = options[k]
                    local resolve_cost = 1
                    local xp, max_xp = card:GetXP()
                    if xp and max_xp then
                        local xp_gain = math.min(max_xp - xp, TheGame:GetGameState():GetCaravan():GetResolve())
                        resolve_cost = math.max( 1, math.floor(xp_gain/2))
                    end

                    local opt = cxt:Opt("OPT_UPGRADE_CARD", card:GetName() )
                    opt:SetHoverLinkParameters( card )

                    opt:PreIcon(global_images.upgradenegotiation)
                        :Fn(function(cxt)
                            local popup = Screen.UpgradeCardPopup( card, card.owner )
                            TheGame:FE():PushScreen( popup )

                            cxt.encounter:YieldEncounter( function()
                                return not TheGame:FE():FindScreen( popup )
                            end )

                            if not popup.upgraded_card then
                                opt:ClearHandlers() -- Abort
                            end
                        end)
                        :Dialog("DIALOG_UPGRADED", card.id)
                        :DeltaResolve( -resolve_cost )
                        :Pop()
                end

                cxt:Opt("OPT_DONT_TAKE")
                    :PreIcon(global_images.close)
                    :Dialog("DIALOG_DID_NOT_TAKE")
                    :DoneConvo()
            end )

        end
    },
    
    UPGRADE_BATTLE_CARD = 
    {

        on_init = function(quest)
            local cards = TheGame:GetGameState():GetPlayerAgent().battler:GetCards()
            for i=1, #cards do
                if cards[i]:HasUpgrades() and not cards[i]:UpgradeReady() and not cards[i]:IsHatched() then
                    return true
                end                        
            end
        end,
        
        name = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.UPGRADE_BATTLE_CARD.NAME"
        end,
        
        desc = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.UPGRADE_BATTLE_CARD.DESC"
        end,

        txt = {
            NAME = "Spar with {friend}",
            DESC = "Spar with {friend} to upgrade one of your battle cards. This will cost some health.",

            DIALOG_INTRO = [[
                agent:
                    !crossed
                    $happyCocky
                    Hey {player}, you feel like throwing down? I have a new move I wanna try out.
                    !wink
                    Watch closely, and you might even pick up a thing or two. And I promise not to hurt ya. 
            ]],
            OPT_UPGRADE_CARD = "Upgrade <!card_1>{1}</>",

            DIALOG_UPGRADED = [[
                player:
                    !cruel 
                    $miscMischievous
                    Sure. Try to keep up, though, eh?
                    !exit
                agent:
                    !exit
                * You and {agent} spar for a while, both pulling your punches 'cause you've got the luxury. You trade tips as you trade blows.
            ]],
            DIALOG_UPGRADED_2 = [[
                player:
                    !left
                    !tired 
                agent:
                    !right
                    !cringe
                    Now <b>that</> was a good workout! Gonna feel it for days.
                player:
                    !happy
                    $happyDefault
                    That's how you know it was a good one. Thanks for the match, {agent}!
            ]],
            OPT_DONT_TAKE = "Refuse",
            
            DIALOG_DID_NOT_TAKE = [[
                player:
                    !shrug
                    Thanks, but naw. I gotta keep my strength for the fights I can't plan.
                agent:
                    !dubious
                    $neutralDubious
                    Never knew someone to pass up free tickets to the gun show. 
                    But your call. 
            ]],

        },
        
        icon = engine.asset.Texture( "icons/quests/oppo_friend_spar.tex"),

        fn = function(cxt)
            local cards = TheGame:GetGameState():GetPlayerAgent().battler:GetCards()
            local options = {}
            for i=1, #cards do
                if cards[i]:HasUpgrades() and not cards[i]:UpgradeReady() and not cards[i]:IsHatched() then
                    table.insert(options, cards[i])
                end                        
            end
            table.shuffle(options)
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoopingFn( function()
                local health = cxt.player:GetHealth()
                for k = 1, math.min( 3, #options) do
                    local card = options[k]
                    local xp, max_xp = card:GetXP()
                    local health_cost = 1
                    if xp and max_xp then
                        local xp_gain = math.floor( math.min( max_xp - xp, (health-1)* 2 ) )
                        health_cost = math.max( 1, math.floor(xp_gain/2))
                    end

                    local opt = cxt:Opt("OPT_UPGRADE_CARD", card:GetName())
                    opt:SetHoverLinkParameters( card )

                    opt:PreIcon( global_images.upgradecombat )
                        :Fn(function(cxt)
                            local popup = Screen.UpgradeCardPopup( card, card.owner )
                            TheGame:FE():PushScreen( popup )

                            cxt.encounter:YieldEncounter( function()
                                return not TheGame:FE():FindScreen( popup )
                            end )

                            if not popup.upgraded_card then
                                opt:ClearHandlers() -- Abort
                            end
                        end)
                        :Dialog("DIALOG_UPGRADED", card.id)
                        :DeltaHealth(-health_cost)
                        :Dialog("DIALOG_UPGRADED_2", card.id)
                        :Pop()
                end

                cxt:Opt("OPT_DONT_TAKE")
                    :Dialog("DIALOG_DID_NOT_TAKE")
                    :DoneConvo()
            end )

        end
    },

    SPECIAL_NOODLES =
    {
        
        name = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.SPECIAL_NOODLES.NAME"
        end,
        
        desc = function(quest)
            return LOC"CONVO.OPPO_FRIEND_BOON.SPECIAL_NOODLES.DESC"
        end,


        txt = {
            NAME = "{friend}'s Special Noodles",
            DESC = "{friend} is cooking up {friend.hisher} special fortifying noodles. They increase your max health.",

            DIALOG_INTRO = [[
                agent:
                    Hey {player}. You have a refined palate, right?
                    !eureka
                    $happyCocky
                    Want to try some of my VIP noodles? They're extra spicy. Toughens up your insides!
            ]],
            OPT_SPECIAL_NOODLES = "Eat the noodles",
            DIALOG_SERVE = [[
                player:
                    !left
                    !overthere
                    $miscNostalgic
                    {player_sal?
                        With the slop we ate on the derrick, most of my tastebuds have been irradiated. 
                    }
                    {player_smith?
                        You learn not to taste things when you eat like I do.
                    }
                    {player_rook?
                        My tastebuds built up healthy calluses 'eating local' when on assignment.
                    }
                    Bring on the heat! It's the only way I feel alive.
                * {agent} smiles proudly.
                agent:
                    !wink
                    $happyThanks
                    You won't be disappointed. 
                !give
            ]],
            DIALOG_SERVE_2 = [[
                player:
                    !take
                    !burp
                    * {agent} wasn't kidding. The heat blasts your tonsils and leaves your pores gasping for air.
                    !burp
                    *** Gained <#HEALTH>{1} Max HP</>
            ]],
            TT_NOODLES = "Special noodles, available only to VIPs.\nPermanently increase max Health by {1}.",
            OPT_DONT_TAKE = "Refuse",
            DIALOG_DID_NOT_TAKE = [[
                player:
                    Naw. I'm feeling a little gassy as it is.
                agent:
                    !shrug
                    Suit yourself. But you know what they say in the Bay: no pass without passin' gas.
                player:
                    !dubious
                    $neutralDubious
                    Is that... what they say?
                agent:
                    !thumb
                    Well, s'what <i>I</i> say. 
                    Maybe next time, eh, {player}?
            ]],
        },
        icon =  engine.asset.Texture( "icons/quests/oppo_friend_special_noodles.tex"),

        fn = function( cxt )
            
            cxt:Dialog("DIALOG_INTRO")
            
            local current_health = cxt.player.health:Get()
            local health_gain = 5


            cxt:Opt("OPT_SPECIAL_NOODLES")
                
                :Dialog( "DIALOG_SERVE", health_gain)
                :PostText("TT_NOODLES", health_gain)
                :Fn(function(cxt)
                    cxt.player.health:AddStatModifier( "ATE_NOODLES", health_gain )
                    TheGame:GetGameState():LogNotification( NOTIFY.HEALTH_UPGRADED, health_gain )
                end)
                :Dialog( "DIALOG_SERVE_2", health_gain)

                :DoneConvo()

            cxt:Opt("OPT_DONT_TAKE")
                :Dialog("DIALOG_DID_NOT_TAKE")
                :DoneConvo()


        end
    }
  
}

for id, t in pairs( BOON_SERVICES ) do
    t.id = id
end
