function AddCardToShop(t, id)
    local def = Content.GetBattleCard(id) or Content.GetNegotiationCard(id)
    if not def then
        assert(false, "UNKNOWN CARD: " .. (id or "") )
    end
    
    local price = Util.GetCardPrice(def.id)
    
    local card = def.is_negotiation and Negotiation.Card( id, TheGame:GetGameState():GetPlayerAgent() ) or Battle.Card( id,  TheGame:GetGameState():GetPlayerAgent() )
    if card:GetMaxXP() ~= nil then
        local diff = TheGame:GetGameState():GetCurrentBaseDifficulty() 
        local xp_range = CARD_DROP_START_XP[math.max(1, math.min( diff, #CARD_DROP_START_XP))]
        card:AddRandomXP( xp_range[1], xp_range[2] )
    end


    table.insert(t, 
    {
        card_id = id,
        card = card,
        is_negotiation = def.is_negotiation,
        price = price,
        sold = false,
    })
end

function AddShopItems(stock, max_items, possible_items)
    table.shuffle (possible_items)
    for k = 1, math.min(#possible_items, max_items) do
        AddCardToShop(stock, possible_items[k])
    end
end


CARD_SHOP_DEFS = 
{

    --mixture of negotiation and combat items
    ADMIRALTY_CLERK_SHOP = function(stock)
        AddShopItems(stock, 1, {"admiralty_intel","admiralty_orders"})
        AddShopItems(stock, 2, {"admiralty_requisition", "lumin_grenade", "shrapnel_grenade"})
        AddShopItems(stock, 2, {"admiralty_requisition", "lumin_grenade", "shrapnel_grenade"})
    end,
    
    --combat items
    CHEMIST_ITEMS = function(stock)
        AddShopItems(stock, 3, {"tincture","vial_of_slurry", "speed_tonic", "adrenaline_shot", "salve", "healing_vapors", "brain_gills"})
    end,

    SAL_RISE_OUTPOST_CARD_SHOP = function(stock)

        local NUM_STOCK = 6
        for i, card in ipairs(BattleCardCollection.AllItems():Filter(function(cd) return cd.local_item == LOCAL_ITEMS.BOG end):Pick(NUM_STOCK)) do
            AddCardToShop(stock, card.id)
        end


        --AddShopItems(stock, 4, {"targeting_core", "robone", "shock_box", "coil_grenade", "oshnu_bile", "energy_siphon", "robodex", "replicator", "oshnu_glue"})
    end,

    GROG_N_DOG_ITEMS = function(stock)
        AddShopItems(stock, 3, {"hip_flask", "liquid_courage", "tincture", "vial_of_slurry", "salve", "adrenaline_shot", "brain_gills"})
    end,
    --primarily negotiation items
    BARTENDER_ITEMS = function(stock)
        AddShopItems(stock, 3, {"hip_flask", "liquid_courage", "tincture", "vial_of_slurry"})
    end,

    ALL_COMBAT_ITEMS = function(stock)
        local NUM_STOCK = 3
        for i, card in ipairs(BattleCardCollection.AllLocalItems():Pick(NUM_STOCK)) do
            AddCardToShop(stock, card.id)
        end
    end,
    
    MURDER_BAY_BEASTMASTER_SHOP = function(stock)
        AddShopItems(stock, 4, {"vroc_whistle","crayote_claw","carapace", "vroc_plume","mandible", "krill_ichor"})
    end,

    MURDER_BAY_NIGHT_MARKET_BATTLE = function(stock) 
        local NUM_STOCK = 6
        local cards = {}
        table.arrayadd( cards, BattleCardCollection.AllRewardableCards(function(cd) return cd.rarity == CARD_RARITY.COMMON end):Pick(3) )
        table.arrayadd( cards, BattleCardCollection.AllRewardableCards(function(cd) return cd.rarity == CARD_RARITY.UNCOMMON end):Pick(1) )
        table.arrayadd( cards, BattleCardCollection.AllRewardableCards(function(cd) return cd.rarity == CARD_RARITY.RARE end):Pick(1) )
        local remaining_num = math.max(0, NUM_STOCK - #cards)
        if remaining_num > 0 then
            table.arrayadd( cards, BattleCardCollection.AllRewardableCards(function(cd) for k,v in pairs(cards) do if v.id == cd.id then return false end end return true end):Pick(remaining_num) )
        end

        for i, card in ipairs(cards) do
            AddCardToShop(stock, card.id)
        end
    end,
    
    MURDER_BAY_NIGHT_MARKET_NEGOTIATION = function(stock) 
        local NUM_STOCK = 6
        local cards = {}
        table.arrayadd( cards, NegotiationCardCollection.AllRewardableCards(function(cd) return cd.rarity == CARD_RARITY.COMMON end):Pick(3) )
        table.arrayadd( cards, NegotiationCardCollection.AllRewardableCards(function(cd) return cd.rarity == CARD_RARITY.UNCOMMON end):Pick(1) )
        table.arrayadd( cards, NegotiationCardCollection.AllRewardableCards(function(cd) return cd.rarity == CARD_RARITY.RARE end):Pick(1) )
        local remaining_num = math.max(0, NUM_STOCK - #cards)
        if remaining_num > 0 then
            table.arrayadd( cards, NegotiationCardCollection.AllRewardableCards(function(cd) for k,v in pairs(cards) do if v.id == cd.id then return false end end return true end):Pick(remaining_num) )
        end

        for i, card in ipairs(cards) do
            AddCardToShop(stock, card.id)            
        end
    end,

}
