require "aspect"
local negotiation_defs = require "negotiation/negotiation_defs"

------------------------------------------------------------------------------------
-- Generic container for buff modifiers that are added/removed to the agent as a set.

local Buff  = class( "Buff", Aspect)
Buff.status_effect = true
Buff.texture = engine.asset.Texture("images/white.tex")

--------------------------------------------------------------------------

local Drunk = class( "Aspect.Drunk", Buff )
Content.AddAspect( "drunk", Drunk )

Drunk.name = "Drunk"
Drunk.texture = engine.asset.Texture("icons/aspects/status_drunk.tex" )

Drunk.desc = "Gains a Drunk argument when negotiating"
-- Drunk.negotiation_cards = 
-- {
--     "drunk", 1,
-- }


--------------------------------------------------------------------------

local Intimidated = class( "Aspect.Intimidated", Buff )

Intimidated.name = "Intimidated"
Intimidated.texture = engine.asset.Texture("icons/aspects/status_intimidated.tex")

Intimidated.desc = "Intimidated people will not join battle against you"

function Intimidated:CheckCombatSupport(target, base_reason)
    if base_reason == SUPPORT_ACTION.HINDER then
        return SUPPORT_ACTION.SUPPRESSED, SUPPORT_REASON.MODIFIED, loc.format( LOC"SUPPORT_DESC.INTIMIDATED", self.agent:LocTable())
    end
end

function Intimidated:CheckNegotiationSupport(target, base_reason)
    if base_reason == SUPPORT_ACTION.HINDER then
        return SUPPORT_ACTION.SUPPRESSED, SUPPORT_REASON.MODIFIED, LOC"SUPPORT_DESC.INTIMIDATED"
    end
end

Content.AddAspect( "intimidated", Intimidated )



--------------------------------------------------------------------------

local PaidOff = class( "Aspect.PaidOff", Buff )

PaidOff.name = "Paid Off"
PaidOff.texture = engine.asset.Texture("icons/aspects/status_paid_off_minor.tex")
PaidOff.desc = [[Paid-off people will not hinder you in negotiations and fights.]]

function PaidOff:CheckCombatSupport(target, base_reason)
    if base_reason == SUPPORT_ACTION.HINDER then
        return SUPPORT_ACTION.SUPPRESSED, SUPPORT_REASON.MODIFIED, LOC"SUPPORT_DESC.PAID_OFF"
    end
end

function PaidOff:CheckNegotiationSupport(target, base_reason)
    if base_reason == SUPPORT_ACTION.HINDER then
        return SUPPORT_ACTION.SUPPRESSED, SUPPORT_REASON.MODIFIED, LOC"SUPPORT_DESC.PAID_OFF"
    end
end

Content.AddAspect( "paidoff", PaidOff )

--------------------------------------------------------------------------

local Bribed = class( "Aspect.Bribed", Buff)
Bribed.name = "Bribed"
Bribed.silent = true
Bribed.texture = engine.asset.Texture("icons/aspects/status_bribed_minor.tex" )
Bribed.desc = [[Bribed people will help you in negotiations and fights.]]

function Bribed:CheckNegotiationSupport(target, base_reason, is_hostile)
    if base_reason ~= SUPPORT_ACTION.HELP then
        return SUPPORT_ACTION.HELP, SUPPORT_REASON.MODIFIED, LOC"SUPPORT_DESC.BRIBED"
    end
end

function Bribed:CheckCombatSupport(target, base_reason)
    if base_reason ~= SUPPORT_ACTION.HELP then
        return SUPPORT_ACTION.HELP, SUPPORT_REASON.MODIFIED, LOC"SUPPORT_DESC.BRIBED"
    end
end

Content.AddAspect( "bribed", Bribed )

