-- manages what cards this guy has for sale, when they refresh, the prices, etc.

local CardShop = class("Aspect.CardShop", Aspect)

function CardShop:init( agent, restock_fn_id )
    CardShop._base.init( self, agent )
    self.shop_stock = {}
    if restock_fn_id then
        self:SetStockID(restock_fn_id)
    end
    
end

function CardShop:CanBuyCard(idx)
    local caravan = TheGame:GetGameState():GetCaravan()
    local data = self.shop_stock[idx]
    local price = self:CalculatePrice( idx )
    return not data.sold and caravan:GetMoney() >= price
end

function CardShop:SetStockID(stock_id)
    self.restock_fn_id = stock_id
    self.restock_pending = true
    --self:DoRestock()
    return self
end

function CardShop:GetStock()
    self:DoRestock()

    if #self.shop_stock == 0 then
        self:RefreshStock()
    end

    return self.shop_stock
end

function CardShop:Save()
    return self
end

function CardShop:HasItemsForSale()
    for k,v in pairs(self:GetStock()) do
        if not v.sold then
            return true
        end
    end
    return false
end

function CardShop:UpdateAspect()
    self.restock_pending = true
    --self:DoRestock()
end

local IS_SHOP = { is_shop = true }
function CardShop:CalculatePrice( idx )
    local data = self.shop_stock[ idx ]
    if data then
        local price, details = CalculatePayment( self.agent, data.price, 1, IS_SHOP )
        return price, details, data.price
    end
end

function CardShop:DoBuyCard( idx, on_success, free )
    local owner = self.shop_stock[idx].is_negotiation and TheGame:GetGameState():GetPlayerAgent().negotiator or TheGame:GetGameState():GetPlayerAgent().battler
    
    local price = nil
    if not free then
        price = self:CalculatePrice( idx )
        TheGame:GetGameState():GetCaravan():PayMoney( price, self.agent )
        self.shop_stock[idx].sold = true
    end

    local stock = self.shop_stock[ idx ]
    StatUtil.BoughtAtShop( stock.card_id )

    owner:AddCard( stock.card )

    TheGame:AddGameplayStat( "cards_bought", 1 )

	local metrics_data = {
		ITEM_INFO = { ID = stock.card_id, PRICE=price },
		PLAYER_DATA = TheGame:GetGameState():GetPlayerState(),
	}
	SendMetricsData("ITEM_PURCHASED", metrics_data )
	
    if on_success then
        on_success()
    end
end

function CardShop:ForceRestock()
    self.restock_pending = true
    self:DoRestock()
end

function CardShop:DoRestock(restock_fn)
    if self.restock_pending then
        self.restock_pending = false
        table.clear(self.shop_stock)
        
        if self.restock_fn_id and CARD_SHOP_DEFS[self.restock_fn_id] then 
            CARD_SHOP_DEFS[self.restock_fn_id](self.shop_stock)
            for k,v in pairs(self.shop_stock) do
                v.sold = false
            end
        end

        table.sort(self.shop_stock, function (a, b) 
                if a.is_negotiation == b.is_negotiation then
                    return a.price < b.price
                else
                    return a.is_negotiation
                end
            end)
    end
end

function CardShop:PostLoad( game_state )
    local player = game_state:GetPlayerAgent()
    for i, stock in ipairs( self.shop_stock ) do
        if stock.card == nil then
            local card = stock.is_negotiation and Negotiation.Card( stock.card_id, player ) or Battle.Card( stock.card_id, player )
            if stock.start_xp then
                card:AddXP( stock.start_xp )
            end
            stock.card = card
        end
    end
end

Content.AddAspect( "cardshop", CardShop )
