-- These are global helpers pulled from the storyline quests, which really should be part of this Aspect but aren't for legacy reasons.

function GenerateGrafts(type)   
    local graft_ids = {}
    local function AlreadyAdded( graft_def )
        return table.arrayfind(graft_ids, graft_def.id) == nil
    end
    local player = TheGame:GetGameState():GetPlayerAgent()
    table.insert(graft_ids, GraftCollection.Rewardable(player, AlreadyAdded):Type( type ):Rarity(CARD_RARITY.COMMON):Pick(1)[1])
    table.insert(graft_ids, GraftCollection.Rewardable(player, AlreadyAdded):Type( type ):Rarity(CARD_RARITY.UNCOMMON):Pick(1)[1])
    table.insert(graft_ids, GraftCollection.Rewardable(player, AlreadyAdded):Type( type ):Rarity(CARD_RARITY.RARE):Pick(1)[1])
    
    local grafts = {}
    for _, def in ipairs(graft_ids) do
         TheGame:GetGameProfile():SetSeenGraft( def.id )
         table.insert(grafts, GraftInstance(def.id) )
    end
    return grafts
end

function PresetGrafts(grafts)            
    return function(cxt)
        for k,graft in ipairs (grafts) do
            cxt:Opt("OPT_SELL_GRAFT", graft)
                :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex") )
                :ReqCondition(not cxt.player.graft_owner:HasGraft(graft.id), "REQ_ALREADY_HAVE_GRAFT")
                :RequireMoney(Util.GetGraftPrice(graft.id), { is_shop = true })
                :Fn( function( cxt, who )
                    local installed = ConvoUtil.GiveGraft(cxt, graft)
                    if installed then
                        local amount = CalculatePayment(cxt:GetAgent(), Util.GetGraftPrice(graft.id), 1, { is_shop = true })
                        cxt.caravan:PayMoney( amount, cxt:GetAgent() )
                        table.arrayremove(grafts, graft)
                        cxt:Dialog( "DIALOG_SELL_GRAFT" )
                    end
                end )
        end
        StateGraphUtil.AddBackButton(cxt)  
    end
end

-- manages what grafts this guy has for sale, when they refresh, the prices, etc.
------------------------------------------------------------------------------------

local GraftShop = class("Aspect.GraftShop", Aspect)

function GraftShop:GetGrafts( graft_type )
    if self.stock == nil then
        self:DoRestock()
    end
    return self.stock[ graft_type ]
end

function GraftShop:Save()
    return self
end

function GraftShop:UpdateAspect()
    self:DoRestock()
end

function GraftShop:DoRestock()
    self.stock = {}
    self.stock[ GRAFT_TYPE.NEGOTIATION ] = GenerateGrafts(GRAFT_TYPE.NEGOTIATION)
    self.stock[ GRAFT_TYPE.COMBAT ] = GenerateGrafts(GRAFT_TYPE.COMBAT)
end

Content.AddAspect( "graftshop", GraftShop )
