require "aspect"
require "eventsystem"
local memtokens = require "memtokens"

local MettleHandler = class( "Aspect.MettleHandler", Aspect )

function MettleHandler:init( agent )
    MettleHandler._base.init( self, agent )
    self.agent = agent
    self.character_id = agent:GetContentID()
end

function MettleHandler:ApplyUpgrades()

    if not self:Enabled() then
        return
    end

    local upgrades = TheGame:GetGameProfile():GetMettleUpgrades(self.character_id)
    if upgrades then
    	for k, v in pairs(upgrades) do
    		local graft = self.agent.graft_owner:AddGraft( GraftInstance(k) )
    		graft:GetDef().ApplyUpgrade( graft, v )
    	end
    end
end

function MettleHandler:PostLoad( game_state )
    self:VerifyUpgrades(game_state)
end

function MettleHandler:Enabled()
    if GetAdvancementModifier( ADVANCEMENT_OPTION.METTLE_OFF ) then
        return false
    end

    return not (self.agent and self.agent.graft_owner:HasGraft("bare_mettle"))
end

function MettleHandler:VerifyUpgrades( game_state )
    
    if not self:Enabled() then
        return
    end

	local upgrades = TheGame:GetGameProfile():GetMettleUpgrades(self.character_id)
    if upgrades then
    	for k, v in pairs(upgrades) do
    		local installed = self.agent.graft_owner:GetGraft(k)
    		if not installed then
    			local graft = self.agent.graft_owner:AddGraft( GraftInstance(k) )
    			graft:GetDef().ApplyUpgrade( graft, v )
    		else
    			if installed.userdata.level ~= v then
    				installed:GetDef().DeltaUpgrade( installed, v )
    			end
    		end
    	end
    end
end

Content.AddAspect( "mettle_handler", MettleHandler )