require "aspect"
--------------------------------------------------------------------------
-- Exists for any agent within the Caravan.

PARTY_MEMBER_TYPE = MakeEnum{
    "LEADER",  -- Party leader: there is always exactly 1.
    "CREW", -- Full-time crew member. Helps in all things.
    "ESCORT", -- Escort only: won't engage in battle
    "CAPTIVE", -- Captive: won't help at all.
    "HIRED", -- Temporary crew member.  Will leave after job completion or phase change.
}

local PartyMember = class("Aspects.PartyMember", Aspect)
PartyMember.name = "Party Member"

function PartyMember:init( agent, party, member_type)
    Aspect.init( self, agent )
    assert( IsEnum( member_type, PARTY_MEMBER_TYPE ))
    self.member_type = member_type
    self.party = party
end

function PartyMember:SetMemberType( member_type )
    assert( IsEnum( member_type, PARTY_MEMBER_TYPE ))
    if self.member_type == PARTY_MEMBER_TYPE.HIRED then
        TheGame:GetEvents():RemoveListener( self )
    end

    self.member_type = member_type

    if self.member_type == PARTY_MEMBER_TYPE.HIRED then
        TheGame:GetEvents():ListenForEvents( self, "quests_changed", "resolve_battle" )
    end
end

function PartyMember:GetMemberType()
    return self.member_type
end

function PartyMember:OnAgentReady()
    -- Save-game compatability: REMOVE ME.
    if self.member_type == "HELPER" then
        self.member_type = PARTY_MEMBER_TYPE.HIRED
    end

    if self.member_type == PARTY_MEMBER_TYPE.HIRED then
        TheGame:GetEvents():ListenForEvents( self,  "quests_changed", "resolve_battle" )
    end
end

function PartyMember:OnLoseAspect()
    if self.member_type == PARTY_MEMBER_TYPE.HIRED then
        TheGame:GetEvents():RemoveListener( self )
    end
end

function PartyMember:IsDeparting()
    return self.is_departing == true
end

function PartyMember:IsLeader()
    return self.member_type == PARTY_MEMBER_TYPE.LEADER
end

function PartyMember:IsCrew()
    return self.member_type == PARTY_MEMBER_TYPE.CREW or
            self.member_type == PARTY_MEMBER_TYPE.LEADER or
            self.member_type == PARTY_MEMBER_TYPE.HIRED
end

function PartyMember:IsHired()
    return self.member_type == PARTY_MEMBER_TYPE.HIRED
end

function PartyMember:IsEscort()
    return self.member_type == PARTY_MEMBER_TYPE.ESCORT
end

function PartyMember:IsCaptive()
    return self.member_type == PARTY_MEMBER_TYPE.CAPTIVE
end

function PartyMember:GetParty()
    return self.party
end

function PartyMember:CheckDeparture()
    if (self.quests_completed or 0) > 1 then
        print( self.agent, " is departing:", self.quests_completed, self.battles_fought )
        self.is_departing = true

    elseif (self.quests_completed or 0) > 0 and (self.battles_fought or 0) > 1 then
        print( self.agent, " is departing:", self.quests_completed, self.battles_fought )
        self.is_departing = true
    end
end


function PartyMember:OnGlobalEvent( event_name, ... )
    if event_name == "quests_changed" then
        local quest = ...
        if (quest:IsComplete() or quest:IsFailed()) and (quest:GetType() == QTYPE.SIDE or quest:GetType() == QTYPE.CONTRACT) then
            self.quests_completed = (self.quests_completed or 0) + 1
            self:CheckDeparture()
        end

    elseif event_name == "resolve_battle" then
        local battle, primary_enemy = ...
        for i, fighter in battle:AllFighters() do
            if fighter.agent == self.agent then
                self.battles_fought = (self.battles_fought or 0) + 1
                self:CheckDeparture()
            end
        end
    end
end

return PartyMember

