local Stat = class("Aspect.Stat", Aspect)

function Stat:init( agent )
    Stat._base.init( self, agent )
    self.current = 1
    self.base_max = 1
    self.max = 1
    self.name = "stat"
end


function Stat:Get()
    return self.current, self.max, 0
end

function Stat:_RecalcMax()
    
    --for old save compat
    if self.base_max == nil then
        self.base_max = self.max or 1
    end

    local new_max = self.base_max
    if self.modifiers then
        for k,v in pairs(self.modifiers) do
            new_max = new_max + v
        end
    end
    if new_max ~= self.max then
        local old_max = self.max
        local old_val = self.current
        local delta = new_max - self.max
        
        if delta > 0 then
            self.current = self.current + delta
        end

        self.max = new_max
        self.current = clamp(self.current, 0, self.max)
        local event_name = string.format("%s_change", self.name)
        self.agent:BroadcastEvent( event_name, self.agent, self.current, old_val, new_max, old_max )
    end
end

function Stat:SetBaseMax( max )
    self.base_max = max
    self:_RecalcMax()
end

function Stat:AddStatModifier(id, amt)
    local old_max = self.max
    self.modifiers = self.modifiers or {}
    self.modifiers[id] = (self.modifiers[id] or 0) + amt
    if self.modifiers[id] == 0 then
        self.modifiers[id] = nil
    end
    self:_RecalcMax()
end

function Stat:Set( new_val )
    local old_val = self.current
    new_val = clamp( math.ceil( new_val ), 0, self.max)
    if old_val ~= new_val then
        self.current = new_val
    end

    local event_name = string.format("%s_change", self.name)
    self.agent:BroadcastEvent( event_name, self.agent, new_val, old_val, self.max, self.max )

    return new_val, old_val
end

function Stat:Restore( percent )
    if self.current < self.max then
        local delta = (percent or 1)*self.max
        self:Delta( delta )
    end
end

function Stat:SetPercent( val )
    local new = self.max * val
    self:Set(new)
end

function Stat:GetPercent()
    return self.current / self.max
end

function Stat:Delta( delta )
    local old = self.current
    local new, old = self:Set( self.current + delta )
    return new - old, new, old
end

function Stat:Save()
    return self
end

function Stat:__tostring()
    return loc.format("{1} {2}: {3}/{4}", tostring(self.agent), tostring(self.name), self.current, self.max)
end