local combat_anims = 
{
    "anim/med_combat.zip",
    --"anim/med_combat_common.zip",
}

local convo_anims =
{
    "anim/med_portrait.zip",
    "anim/med_dial_neutral.zip",
    "anim/med_dial_neutral2.zip",
    "anim/med_dial_happy.zip",
    "anim/med_dial_angry.zip",
    "anim/med_dial_scared.zip",
    "anim/med_dial_drunk.zip",
    "anim/med_dial_injured.zip",
    "anim/med_dial_weapon_idle.zip",
    "anim/med_dial_plax.zip",
}

for k,v in pairs(convo_anims) do CacheAnim(v) end
for k,v in pairs(combat_anims) do CacheAnim(v) end

function CreatePersonBuild(args)
    return {
        combatanims = args.combat_anims or combat_anims,
        anims = convo_anims,
        gender = args.gender,
        build = args.file,
        colour= args.colours,
        glow = args.glow,
        gloves = args.gloves or GLOVES.NONE,
        hand_build_override = args.hand_build_override,
        no_hand_swap = args.no_hand_swap,
        hide_symbols = args.hide_symbols,
        force_hat = args.force_hat,

        lighting_height =
        {
            top = 750,
            bottom = 300,
        },
        shadow_height =
        {
            top = 200,
            bottom = -200,
        },
    }
end

function CreateSpecialBuild(args)
    return {
        combatanims = args.combat_anims or table.empty,
        anims = args.anims,
        gender = args.gender,
        build = args.file,
        colour= args.colours,
        glow = args.glow,
        --gloves = args.gloves or GLOVES.NONE,
        --hand_build_override = args.hand_build_override,
        no_hand_swap = args.no_hand_swap,
    }
end

function CreateSnailBuild( args )
    return 
    { 
        
        anims = 
        {
            "anim/snail_dial.zip",
            "anim/snail_portrait.zip",
            "anim/snail_dial_plax.zip",
        },

        combatanims =
        {
            "anim/snail_combat.zip",
        },

        species = SPECIES.SNAIL,
        gender = GENDER.UNDISCLOSED,
        colour = args.colours,
        glow = args.glow,
        build = args.build,
    }
end


function CreateCreatureBuild( args )
    local t=  {
        anims =
        {
            args.dialog_anim,
            args.portrait_anim
        },

        combatanims = args.combat_anims,
        dialoganims = args.dialog_anims,
        species = SPECIES.BEAST,
        gender = GENDER.UNDISCLOSED,
        build = args.dialog_build,
        colour= {},
        glow = args.glow,

        lighting_height =
        {
            top = 750,
            bottom = 300,
        },
        shadow_height =
        {
            top = 200,
            bottom = -200,
        },
    }

    return t
end




local builds = 
{
    
    --Fallback builds
    medium_male = CreatePersonBuild{
        gender=GENDER.MALE, 
        file = "anim/med_male_undergarment_build.zip", 
        gloves = GLOVES.NONE,
    },
    medium_female = CreatePersonBuild{
        gender=GENDER.FEMALE,
        file = "anim/med_female_undergarment_build.zip", 
        gloves=GLOVES.NONE,
    },

    --uniques
     sal = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_sal_build.zip",
        gloves = GLOVES.ARMOURED,
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        colours = {base_glove = 0x4D292Aff,accent = 0x91A0A5ff},
    },

    --uniques
     sal_outfit_02 = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_sal_outfit_02_build.zip",
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        gloves = GLOVES.ARMOURED,
        colours = {base_glove = 0x4D292Aff,accent = 0x91A0A5ff},
    },

    smith = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_smith_build.zip", 
        glow = { colour = 0x66FFFFFF, bloom = 0.5, threshold = 0.02 },
        gloves = GLOVES.NONE,
        colours={base_glove = 0x312926ff},
    },

    rook = CreatePersonBuild{
        gender=GENDER.MALE,
        file = "anim/med_rook_build.zip",
        glow = { colour = 0x57FFFBFF, bloom = 0.15, threshold = 0.02 }, 
        gloves=GLOVES.SIMPLE,
        colours={base_glove = 0x312926ff},
    },

    telvin = CreatePersonBuild{
        gender=GENDER.MALE,
        file = "anim/med_telvin_build.zip", 
        gloves=GLOVES.SIMPLE,
        colours = {base_glove = 0x261E1Eff},
    },
    hebbel = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_hebbel_build.zip", 
        gloves = GLOVES.SIMPLE,
        colours={base_glove = 0x302825ff, skin=0x969982ff},
    },
     female_shroke = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_shroke_build.zip", 
        gloves = GLOVES.NONE,
        colours={skin=0x969982ff},
    },

    female_shroke_rich_merchant = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_shroke_rich_merchant_build.zip", 
        gloves = GLOVES.NONE,
        colours={skin=0x969982ff},
        glow = { colour = 0xE18183FF, bloom = 0.15, threshold = 0.02 },
    },

    
    bogger = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_kalandra_build.zip",
        no_hand_swap = true,
        --gloves = GLOVES.NONE,
        --colours = {base_glove = 0x533D32ff,accent = 0x533D32ff, skin=0xCA9B57ff},
    },

    martin = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_martin.zip",
        gloves = GLOVES.NONE,
        colours = {skin=0xA6834EFF},
    },

    gaft = CreatePersonBuild{
        gender=GENDER.FEMALE,
        file = "anim/med_gaft_build.zip", 
        gloves=GLOVES.SIMPLE,
        colours={base_glove = 0x261E1Eff,accent = 0x261E1Eff},
    },

     skatchet = CreatePersonBuild{
        gender=GENDER.MALE,
        file = "anim/med_skatchet_build.zip", 
        gloves=GLOVES.SIMPLE,
        colours={base_glove = 0x261E1Eff},
    },

     gradle = CreatePersonBuild{
        gender=GENDER.MALE,
        file = "anim/med_gradle_build.zip", 
        gloves=GLOVES.SIMPLE,
        hand_build_override = "anim/hand_kra_male_glove.zip",
        colours={base_glove = 0x805A45ff},
        glow = { colour = 0xCB5D3CFF, bloom = 0.15, threshold = 0.02 },
    },


    job_vendor = CreateSpecialBuild{
        gender=GENDER.UNDISCLOSED,
        anims = {"anim/med_dial_job_vendor.zip", "anim/med_portrait_job_vendor.zip"},
        file = "anim/med_job_vendor_build.zip", 
        no_hand_swap = true,
        --gloves=GLOVES.SIMPLE,
        --hand_build_override = "anim/hand_kra_male_glove.zip",
        --colours={base_glove = 0x805A45ff},
    },

    fssh = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_fssh_build.zip", 
        gloves = GLOVES.SIMPLE,
        glow = { colour = 0x7777FFFF, bloom = 0.15, threshold = 0.02 },
        colours={base_glove = 0x392D2Dff, accent = 0x392D2Dff, skin=0xD6D06Bff},
    },

    luminari_male = CreatePersonBuild{
        gender=GENDER.MALE,
        file = "anim/med_male_luminari_build.zip", 
        no_hand_swap = true,
        --gloves=GLOVES.SIMPLE,
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        hide_symbols = {"hair", "hair_side", "hair_back", "hari_swap"},
    },

     luminari_male_02 = CreatePersonBuild{
        gender=GENDER.MALE,
        file = "anim/med_male_luminari_02_build.zip", 
        no_hand_swap = true,
        --gloves=GLOVES.SIMPLE,
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        hide_symbols = {"hair", "hair_side", "hair_back", "hari_swap"},
    },


    luminari_female = CreatePersonBuild{
        gender=GENDER.FEMALE,
        file = "anim/med_female_luminari_build.zip", 
        no_hand_swap = true,
        --gloves=GLOVES.SIMPLE,
        --colours={base_glove = 0x261E1Eff},
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        hide_symbols = {"hair", "hair_side", "hair_back", "hari_swap"},
    },    

    priest_male = CreatePersonBuild{
        gender=GENDER.MALE,
        gloves = GLOVES.NONE,
        --no_hand_swap = true,
        file = "anim/med_male_pilgrim_build.zip", 
        --gloves=GLOVES.SIMPLE,
        --colours={base_glove = 0x261E1Eff},
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        hide_symbols = {"hair", "hair_side", "hair_back", "hari_swap"},
    },

    priest_female = CreatePersonBuild{
        gender=GENDER.FEMALE,
        gloves = GLOVES.NONE,
        --no_hand_swap = true,
        file = "anim/med_female_pilgrim_build.zip", 
        --gloves=GLOVES.SIMPLE,
        --colours={base_glove = 0x261E1Eff},
        --hide_symbols = {"hair", "hair_side", "hair_back", "hari_swap"},
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
    },

    male_spark_baron_goon = CreatePersonBuild{
        gender=GENDER.MALE,
        gloves = GLOVES.SIMPLE,
        glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        colours = {base_glove = 0x1D282Cff, accent=0x1D282Cff},
        file = "anim/med_male_spark_baron_goon_build.zip", 
    },

    female_spark_baron_goon = CreatePersonBuild{
        gender=GENDER.FEMALE,
        gloves=GLOVES.SIMPLE, 
        glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        colours = {base_glove = 0x1D282Cff, accent=0x1D282Cff},
        file = "anim/med_female_spark_baron_goon_build.zip", 
    },


    dredger_male = CreatePersonBuild{
        gender=GENDER.MALE,
        gloves = GLOVES.DREDGER,
        colours = {base_glove = 0x533025ff, accent=0x331718ff},
        file = "anim/med_male_dredger_build.zip", 
        glow = { colour = 0xCD5465FF, bloom = 0.15, threshold = 0.02 },
        hide_symbols = {"hair", "hair_side", "hair_back", "hair_swap"},
    },

    dredger_female = CreatePersonBuild{
        gender=GENDER.FEMALE,
        gloves = GLOVES.NONE,
        glow = { colour = 0xE4DF55FF, bloom = 0.03, threshold = 0.02 },
        colours = {base_glove = 0x331718ff, accent=0x533025ff},
        file = "anim/med_female_dredger_build.zip", 
        --hide_symbols = {"hat", "hood"},
    },    

    guard_male = CreatePersonBuild{
        gender=GENDER.MALE,
        file = "anim/med_male_guard_build.zip", 
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x271F1Fff},
    },

    guard_male_offduty = CreatePersonBuild{
        gender=GENDER.MALE,
        file = "anim/med_male_guard_offduty_build.zip",
        gloves=GLOVES.SIMPLE,
        colours = {base_glove = 0x7F4E44ff,accent = 0x554339ff},
    },

     guard_corporal_male = CreatePersonBuild{
        gender=GENDER.MALE,
        file = "anim/med_male_guard_corporal_build.zip", 
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x271F1Fff},
    },

    guard_corporal_female = CreatePersonBuild{
        gender=GENDER.FEMALE,
        file = "anim/med_female_guard_corporal_build.zip",
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x261E1Eff,accent = 0x261E1Eff},
    },

    guard_female = CreatePersonBuild{
        gender=GENDER.FEMALE,
        file = "anim/med_female_guard_build.zip",
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x271F1Fff,accent = 0x271F1Fff},
    },
    guard_female_civ = CreatePersonBuild{
        gender=GENDER.FEMALE,
        
        file = "anim/med_female_guard_civ_build.zip",
        gloves=GLOVES.NONE, 
        --colours = {base_glove = 0x271F1Fff,accent = 0x271F1Fff},
    },

    guard_female_offduty = CreatePersonBuild{
        gender=GENDER.FEMALE,
        file = "anim/med_female_guard_offduty_build.zip",
        glow = { colour = 0x11FFFEFF, bloom = 0.13, threshold = 0.2 },
        gloves=GLOVES.SIMPLE,
        colours = {base_glove = 0x7F4E44ff,accent = 0x7F4E44ff},
    },

    guard_female_weaponsmith = CreatePersonBuild{
        gender=GENDER.FEMALE,
        file = "anim/med_female_weaponsmith_build.zip",
        glow = { colour = 0x66FFFFFF, bloom = 0.13, threshold = 0.2 },
        gloves=GLOVES.SIMPLE,
        colours = {base_glove = 0x7F4E44ff,accent = 0x7F4E44ff},
    },

    bandit_male = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_bandit_build.zip", 
        gloves = GLOVES.ARMOURED, 
        colours = {base_glove = 0x583435ff, accent = 0x5A666Bff},
        hide_symbols = {"hair", "hair_side", "hair_back", "hair_swap"},
    },
    bandit_female = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_bandit_build.zip",
        gloves = GLOVES.SIMPLE, 
        colours = {base_glove = 0x533D32ff,accent = 0x533D32ff}
    },

    hunter_male = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_shroke_hunter_build.zip", 
        gloves = GLOVES.SIMPLE, 
        colours = {base_glove = 0x403734ff},
    },

    hunter_female = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_kradeshi_hunter_build.zip", 
        gloves = GLOVES.SIMPLE, 
        colours = {base_glove = 0x38302Eff,accent = 0x7D848Aff},
    },

    merchant_male_rich = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_kradeshi_rich_merchant_build.zip", 
        gloves = GLOVES.NONE, 
        colours = {base_glove = 0x38302Eff,accent = 0x7D848Aff},
        glow = { colour = 0xF7E269FF, bloom = 0.15, threshold = 0.02 },
    },

    merchant_female = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_merchant_build.zip", 
        gloves = GLOVES.NONE,
        glow = { colour = 0x11FFFEFF, bloom = 0.2, threshold = 0.02 },

    },
    merchant_male = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_merchant_build.zip", 
        gloves = GLOVES.NONE,
        glow = { colour = 0x11FFFEFF, bloom = 0.2, threshold = 0.02 }, 
    },

    farmer_male = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_farmer_build.zip",
        gloves = GLOVES.NONE,
        glow = { colour = 0x11FFFEFF, bloom = 0.5, threshold = 0.02 },
    }, 

    farmer_female = CreatePersonBuild{
        gender=GENDER.FEMALE,
        file = "anim/med_female_farmer_build.zip",
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x271F1Fff,accent = 0x271F1Fff},
        glow = { colour = 0x11FFFEFF, bloom = 0.5, threshold = 0.02 },
    },
    
    shopkeep_female = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_shopkeep_build.zip",
        gloves = GLOVES.NONE, 
    },
    shopkeep_male = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_shopkeep_build.zip",
        gloves = GLOVES.NONE, 
    },
    
    shopkeep_male_jarackle = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_jarackle_shopkeep_build.zip",
        gloves = GLOVES.NONE, 
    },

    shopkeep_female_jarackle = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_jarackle_shopkeep_build.zip",
        gloves = GLOVES.NONE, 
    },
    
    merc_male = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_merc_build.zip",
        gloves = GLOVES.SIMPLE,
        colours =  {base_glove = 0x271F1Fff}
    },
    merc_female = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_merc_build.zip",
        gloves = GLOVES.SIMPLE, 
        colours = {base_glove = 0x271F1Fff,accent = 0x271F1Fff}
    }, 

    rise_male = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_rise_build.zip", 
        gloves = GLOVES.SIMPLE, 
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
    },

    rise_male_02 = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_rise_02_build.zip", 
        gloves = GLOVES.NONE,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
    },

    rise_female = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_rise_build.zip", 
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x814E45ff,accent =0x814E45ff},
    },

    foundry_worker_male = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_jarackle_foundry_worker_build.zip", 
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x725142ff,accent =0x725142ff},
        glow = { colour = 0x24B78DFF, bloom = 0.3, threshold = 0.02 },
    },

    foundry_worker_female = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_jarackle_foundry_worker_build.zip", 
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x725142ff,accent =0x725142ff},
        glow = { colour = 0x24B78DFF, bloom = 0.3, threshold = 0.02 },
    },

    female_undergarment = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_undergarment_build.zip",
        gloves = GLOVES.NONE, 
    },

    lumin_worker_male = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_lumin_worker_build.zip", 
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x302825ff,accent =0x302825ff},
        glow = { colour = 0xDEFAFFFF, bloom = 0.2, threshold = 0.02 },
    },

    lumin_worker_female = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_lumin_worker_build.zip", 
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x302825ff,accent =0x302825ff},
        glow = { colour = 0xDEFAFFFF, bloom = 0.15, threshold = 0.02 },
    },

     male_undergarment = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_undergarment_build.zip",
        gloves = GLOVES.NONE, 
    },

    male_guard_02 = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_guard_02_build.zip",
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x261E1EFF,accent = 0x271F1Fff}, 
    },

    male_phicket = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_phicket_build.zip",
        glow = { colour = 0xF2FE79FF, bloom = 0.12, threshold = 0.02 },
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x27211FFF,accent = 0x27211Fff}, 
    },

    male_guard_civ = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_guard_civ_build.zip",
        gloves=GLOVES.NONE, 
        --colours = {base_glove = 0x242323FF,accent = 0x271F1Fff}, 
    },

    male_guard_offduty = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_guard_offduty_build.zip",
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x554339FF,accent = 0x554339ff}, 
    },


    female_guard_02 = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_guard_02_build.zip",
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x261E1EFF,accent = 0x271F1Fff}, 
    },

    teamster_male  = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_teamster_build.zip", 
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x302825ff,accent =0x302825ff},
    },

    teamster_female = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_shroke_teamster_build.zip", 
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x302825ff,accent =0x302825ff},
    },

    male_spark_baron_professional = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_spark_baron_professional_build.zip",
        glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x1D282Cff,accent =0x1D282Cff},
    },

    female_spark_baron_professional = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_spark_baron_professional_build.zip",
        glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x1D282Cff,accent =0x1D282Cff},
    },

    male_spark_baron_taskmaster = CreatePersonBuild{ 
        gender = GENDER.MALE,
        file = "anim/med_male_spark_baron_taskmaster_build.zip",
        glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        gloves = GLOVES.NONE,
        --colours = {base_glove = 0x302825ff,accent =0x302825ff},
    },

    female_spark_baron_taskmaster = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_spark_baron_taskmaster_build.zip",
        glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        gloves = GLOVES.NONE,
        colours = {base_glove = 0x302825ff,accent =0x302825ff},
    },

    male_clust_trademaster = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_clust_trademaster_build.zip",
        gloves = GLOVES.NONE,
        colours = {base_glove = 0x302825ff,accent =0x302825ff},
    },

    female_clust_trademaster = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_clust_trademaster_build.zip",
        gloves = GLOVES.NONE,
        colours = {base_glove = 0x302825ff,accent =0x302825ff},
    },

--    male_hesh_zealot = CreatePersonBuild{
--        gender = GENDER.MALE,
--        file = "anim/med_male_hesh_zealot_build.zip",
--        gloves = GLOVES.NONE,
--       colours = {base_glove = 0x302825ff,accent =0x302825ff},
--       glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
--   },

--    female_hesh_zealot = CreatePersonBuild{
--        gender = GENDER.FEMALE,
--       file = "anim/med_female_hesh_zealot_build.zip",
--       gloves = GLOVES.NONE,
--       colours = {base_glove = 0x302825ff,accent =0x302825ff},
--   },
-- why is there duplicates of hesh zealot characters?...

    male_hesh_zealot = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_hesh_zealot_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x271F1Fff,accent = 0x271F1Fff},
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },

    },

    female_hesh_zealot = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_hesh_zealot_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x271F1Fff,accent = 0x271F1Fff},
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },

    },

    male_bandit_04 = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_bandit_04_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x583435ff,accent =0x583435ff},
    },

    female_bandit_04 = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_female_bandit_04_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x533D32ff,accent =0x533D32ff},
    },

    male_phicket_02 = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_phicket_02_build.zip",
        glow = { colour = 0x59FFFAFF, bloom = 0.13, threshold = 0.02 },
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x27211FFF,accent = 0x27211Fff}, 
    },

    male_thaft = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_thaft_build.zip",
        gloves=GLOVES.SIMPLE, 
        colours = {base_glove = 0x27211FFF,accent = 0x27211Fff}, 
    },

    female_dax = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_dax_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x533D32ff,accent =0x533D32ff},

    },

    female_jarackle_bandit_02 = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_jarackle_bandit_02_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x533D32ff,accent =0x533D32ff},

    },

    male_jarackle_bandit_02 = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_jarackle_bandit_02_build.zip",
        gloves = GLOVES.ARMOURED,
        colours = {base_glove = 0x533D32ff,accent =0x5A666Bff},

    },
    
    male_jake_smuggler = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_jake_smuggler_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff,accent =0x3C2829ff},

    },

     female_jake_smuggler = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_jake_smuggler_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
        glow = { colour = 0x617B7AFF, bloom = 0.7, threshold = 0.04 },

    },

     male_jake_runner = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_jake_runner_build.zip",
        gloves = GLOVES.NONE,
        colours = {base_glove = 0x3C2829ff,accent =0x3C2829ff},

    },
     
    female_jake_runner = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_jake_runner_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x3C2829ff,accent =0x3C2829ff},

    },

    male_grifter = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_grifter_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x3C2829ff,accent =0x3C2829ff},

    },

    female_grifter = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_grifter_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x523524ff,accent =0x523524ff},

    },

    male_bandit_03 = CreatePersonBuild{
        gender = GENDER.MALE, 
        file = "anim/med_male_bandit_03_build.zip",
        gloves = GLOVES.SIMPLE,
--        hide_symbols = {"hair_back", "hair_swap"},
        colours = {base_glove = 0x583435ff,accent =0x5A666Bff},

    },

    male_nadan = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_nadan_build.zip",
        gloves = GLOVES.SIMPLE,
        hide_symbols = {"hair_back", "hair_swap"},
        colours = {base_glove = 0x583435ff,accent =0x5A666Bff},

    },

    female_bandit_03 = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_bandit_03_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x583435ff,accent =0x5A666Bff},

    },

    female_kashio = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_kashio_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x26201Fff,accent =0x26201Fff},
        glow = { colour = 0xFF9501ff, bloom = 0.15, threshold = 0.02 },

    },

    male_spark_baron_boss = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_spark_baron_boss_build.zip",
        gloves = GLOVES.SIMPLE,
        --hide_symbols = {"hair_back", "hair_swap"},
        glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        colours = {base_glove = 0x1D282Cff,accent =0x1D282Cff},

    },

    rise_male_rebel = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_rise_rebel_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
        glow = { colour = 0xDEEF57FF, bloom = 0.3, threshold = 0.02 },

    },

    rise_female_rebel = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_rise_rebel_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
        glow = { colour = 0xDEEF57FF, bloom = 0.3, threshold = 0.02 },

    },

    rise_female_ = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_rise_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},

    },

    sal_outfit_01 = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_sal_outfit_01_build.zip",
        --gloves = GLOVES.NONE,
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        no_hand_swap = true,
        colours = {base_glove = 0x4D292Aff,accent = 0x91A0A5ff},
    
    },

    male_spark_baron_dronemaster = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_spark_baron_dronemaster_build.zip",
        glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},

    },

    automech_spark_baron_01 = CreateSpecialBuild{
        gender = GENDER.UNDISCLOSED,
        anims = {"anim/med_dial_automech.zip", "anim/med_dial_plax_automech.zip", "anim/med_portrait.zip"},
        file = "anim/med_automech_spark_baron_01_build.zip",
        no_hand_swap = true,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
        glow = { colour = 0xFF9950FF, bloom = 0.1, threshold = 0.02 },

    },

    automech_rise_01 = CreateSpecialBuild{
        gender = GENDER.UNDISCLOSED,
        anims = {"anim/med_dial_automech.zip", "anim/med_dial_plax_automech.zip", "anim/med_portrait.zip"},
        file = "anim/med_automech_rise_01_build.zip",
        no_hand_swap = true,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
        glow = { colour = 0xF08C49FF, bloom = 0.2, threshold = 0.02 },

    },

    male_automech_boss = CreateSpecialBuild{
        gender = GENDER.MALE,
        file = "anim/med_automech_boss_build.zip",
        anims = {"anim/med_dial_automech.zip", "anim/med_dial_plax_automech.zip", "anim/med_portrait.zip"},
        no_hand_swap = true,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
        glow = { colour = 0xCB5D3CFF, bloom = 0.7, threshold = 0.02 },

    },

    male_rise_pamphleteer = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_rise_pamphleteer_build.zip",
        --no_hand_swap = true,
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
        glow = { colour = 0xEBF695FF, bloom = 0.07, threshold = 0.02 },

    },

    sal_outfit_03 = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_sal_outfit_03_build.zip",
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        no_hand_swap = true,
        --colours = {base_glove = 0x33282Aff,accent = 0x33282Aff},

    },

    female_rise_pamphleteer = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_rise_pamphleteer_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},
        glow = { colour = 0xEBF695FF, bloom = 0.07, threshold = 0.02 },

    },

    male_rentorian = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_rentorian_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x2E3046ff,accent =0x2E3046ff},
        glow = { colour = 0xFDCD90FF, bloom = 0.12, threshold = 0.02 },

    },

    female_foreman_kalandra_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_foreman_kalandra_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x27211Fff,accent =0x27211Fff},
    
    },

    female_rise_kalandra_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_rise_kalandra_build.zip",
        no_hand_swap = true,
        colours = {base_glove = 0x27211Fff,accent =0x27211Fff},
        glow = { colour = 0xFF9900FF, bloom = 0.3, threshold = 0.02 },
    
    },

    male_rise_flekfis_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_rise_flekfis_build.zip",
        --no_hand_swap = true,
        no_hand_swap = true,
        colours = {base_glove = 0x312A2Aff,accent =0x312A2Aff},

    },

    female_spark_arint_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_spark_arint_build.zip",
        hand_build_override = "anim/hand_automech.zip",
        --colours = {base_glove = 0x27211Fff,accent =0x27211Fff},
        glow = { colour = 0xEDA36DFF, bloom = 0.15, threshold = 0.02 },
    
    },

    male_spark_baron_fellemo_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_spark_baron_fellemo_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x1D282Cff,accent =0x1D282Cff},
        glow = { colour = 0xEDA36DFF,  bloom = 0.15, threshold = 0.02 },
    
    },

    male_shroke_beastmaster_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_shroke_beastmaster_build.zip",
        gloves = GLOVES.NONE,
        colours = {base_glove = 0x1D282Cff,accent =0x1D282Cff},
    
    },

    female_hesh_boss_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_hesh_boss_build.zip",
        no_hand_swap = true,
        colours = {base_glove = 0x1D282Cff,accent =0x1D282Cff},
        glow = { colour = 0x11FFFEFF,  bloom = 0.15, threshold = 0.02 }, 
    
    },

    rook_outfit_01 = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_rook_outfit_01_build.zip",
        --gloves = GLOVES.NONE,
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        no_hand_swap = true,
        colours = {base_glove = 0x4D292Aff,accent = 0x91A0A5ff},
    
    },

    rook_outfit_02 = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_rook_outfit_02_build.zip",
        --gloves = GLOVES.NONE,
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        no_hand_swap = true,
        colours = {base_glove = 0x353537ff,accent = 0x353537ff},
    
    },

    male_bogger = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_bogger_build.zip",
        --no_hand_swap = true,
        colours = {base_glove = 0x353537ff,accent = 0x353537ff},
        glow = { colour = 0xF2E669FF, bloom = 0.3, threshold = 0.02 },
    
    },

    female_bogger = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_bogger_build.zip",
        --no_hand_swap = true,
        colours = {base_glove = 0x353537ff,accent = 0x353537ff},
        glow = { colour = 0xF2E669FF, bloom = 0.3, threshold = 0.02 },
    
    },

    male_hesh_auctioneer = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_hesh_auctioneer_build.zip",
        --no_hand_swap = true,
        colours = {base_glove = 0x271F1Fff},
        glow = { colour = 0x11FFFEFF,  bloom = 0.15, threshold = 0.02 },
    
    },

    female_bogger_clobber = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_bogger_clobber_build.zip",
        glow = { colour = 0xF0F0B1FF, bloom = 0.15, threshold = 0.02 },
        --no_hand_swap = true,
        colours = {base_glove = 0x353537ff,accent = 0x353537ff},

    },

    male_bogger_clobber = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_bogger_clobber_build.zip",
        --no_hand_swap = true,
        colours = {base_glove = 0x353537ff,accent = 0x353537ff},
        glow = { colour = 0xF2E669FF, bloom = 0.15, threshold = 0.02 },

    },

    male_bogger_boss = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_bogger_boss_build.zip",
        --no_hand_swap = true,
        colours = {base_glove = 0x353537ff,accent = 0x353537ff},
        glow = { colour = 0xF2E669FF, bloom = 0.15, threshold = 0.02 },

    },

    male_weezil = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_weezil_build.zip",
        no_hand_swap = true,
        --colours = {base_glove = 0x353537ff,accent = 0x353537ff},
        glow = { colour = 0x63FFFFFF, bloom = 0.15, threshold = 0.02 },

    },

    male_brut_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_brut_build.zip",
        no_hand_swap = true,
        colours = {base_glove = 0x271F1Fff,accent = 0x271F1Fff},
        glow = { colour = 0x63FFFFFF, bloom = 0.3, threshold = 0.02 },

    },
    
    male_denikus_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_denikus_build.zip",
        no_hand_swap = true,
        --colours = {base_glove = 0x3D3225ff,accent = 0x271F1Fff},
        glow = { colour = 0xF46565FF, bloom = 0.3, threshold = 0.02 },

    },

    smith_outfit_01_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_smith_outfit_01_build.zip",
        --no_hand_swap = true,
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x3D3326ff,accent = 0x3D3326ff},
        glow = { colour = 0x63FFFFFF, bloom = 0.15, threshold = 0.02 },

    },

    smith_outfit_02_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_smith_outfit_02_build.zip",
        --no_hand_swap = true,
        colours = {base_glove = 0x3D3326ff,accent = 0x3D3326ff},
        glow = { colour = 0x11FFFFFF, bloom = 0.15, threshold = 0.02 },

    },

    smith_outfit_03_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_smith_outfit_03_build.zip",
        gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        colours = {base_glove = 0x775043ff,accent = 0x775043ff},
        glow = { colour = 0x11FFFFFF, bloom = 0.15, threshold = 0.02 },

    },

    smith_outfit_04_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_smith_outfit_04_build.zip",
        force_hat = true,
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        --colours = {base_glove = 0x775043ff,accent = 0x775043ff},
        glow = { colour = 0x11FFFFFF, bloom = 0.15, threshold = 0.02 },

    },

    male_sweet_moreef_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_sweet_moreef_build.zip",
        --gloves = GLOVES.SIMPLE,
        --colours = {base_glove = 0x775043ff,accent = 0x775043ff},
        
    },

    male_jake_elite_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_jake_elite_build.zip",
        no_hand_swap = true,
        glow = { colour = 0x63FFFFFF, bloom = 0.15, threshold = 0.02 },

    },

    female_jake_elite_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_jake_elite_build.zip",
        no_hand_swap = true,
        glow = { colour = 0x63FFFFFF, bloom = 0.15, threshold = 0.02 },

    },

    male_vixmali_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_vixmali_build.zip",
        --no_hand_swap = true,
        --colours = {base_glove = 0x775043ff,accent = 0x775043ff},
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 },
        
    },

    female_mullifee_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_mullifee_build.zip",
        --no_hand_swap = true,
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x271F1Fff},
        glow = { colour = 0x11FFFFFF, bloom = 0.2, threshold = 0.02 },
        
    },

    male_theroux_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_theroux_build.zip",
        --no_hand_swap = true,
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x533D34ff,accent = 0x533D34ff},
        glow = { colour = 0xFD787DFF, bloom = 0.15, threshold = 0.02 },
        
    },

    male_guard_promoted_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_guard_promoted_build.zip",
        --no_hand_swap = true,
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x271F1Fff,accent = 0x271F1Fff},
        glow = { colour = 0x66FFFFFF, bloom = 0.15, threshold = 0.02 },
        
    },

    female_guard_promoted_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_guard_promoted_build.zip",
        --no_hand_swap = true,
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x271F1Fff,accent = 0x271F1Fff},
        glow = { colour = 0x66FFFFFF, bloom = 0.15, threshold = 0.02 },
        
    },

    male_bandit_promoted_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_bandit_promoted_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x403833ff},
        --no_hand_swap = true,
        
    },

    female_bandit_promoted_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_bandit_promoted_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x403734ff},
        --no_hand_swap = true,
    },

    male_spark_baron_promoted_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_spark_baron_promoted_build.zip",
        no_hand_swap = true,
        --gloves = GLOVES.SIMPLE,
        glow = { colour = 0xF08C49FF, bloom = 0.2, threshold = 0.02 },
        
    },

    male_turncoat_boss_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_turncoat_boss_build.zip",
        no_hand_swap = true,
        glow = { colour = 0xFF9950FF, bloom = 0.45, threshold = 0.04 },
    },

    female_spark_baron_promoted_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_spark_baron_promoted_build.zip",
        no_hand_swap = true,
        --gloves = GLOVES.SIMPLE,
        glow = { colour = 0xF08C49FF, bloom = 0.2, threshold = 0.02 },
        
    },

    female_rise_promoted_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_rise_promoted_build.zip",
        --no_hand_swap = true,
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff},
        glow = { colour = 0xDEEF57FF, bloom = 0.15, threshold = 0.02 },
        
    },

    male_rise_promoted_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_rise_promoted_build.zip",
        --no_hand_swap = true,
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff},
        glow = { colour = 0xDEEF57FF, bloom = 0.15, threshold = 0.02 },
        
    },

    male_jake_promoted_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_jake_promoted_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff},
        glow = { colour = 0x66FFFFFF, bloom = 0.15, threshold = 0.02 },
        
    },
        
    female_jake_promoted_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_jake_promoted_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312A2Aff},
        glow = { colour = 0x66FFFFFF, bloom = 0.15, threshold = 0.02 },
        
    },

    female_labourer_promoted_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_labourer_promoted_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x725142ff},
        
    },

    male_hesh_luminiciate_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_hesh_luminiciate_build.zip",
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        --colours = {base_glove = 0x312A2Aff},
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 }
   
    },

    female_hesh_luminiciate_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_hesh_luminiciate_build.zip",
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        --colours = {base_glove = 0x312A2Aff},
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 }
   
    },

    male_labourer_promoted_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_labourer_promoted_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x725142ff},
        
    },

    female_priest_promoted_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_priest_promoted_build.zip",
        --gloves = GLOVES.SIMPLE,
        no_hand_swap = true,
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 }
        
    },

    male_priest_promoted_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_priest_promoted_build.zip",
        --gloves = GLOVES.SIMPLE,
        no_hand_swap = true,
        glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 }
        
    },

    male_mercenary_boss_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_mercenary_boss_build.zip",
        --gloves = GLOVES.SIMPLE,
        no_hand_swap = true,
        glow = { colour = 0x11FFFEFF, bloom = 0.4, threshold = 0.02 }
        
    },

    male_grifter_02_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_grifter_02_build.zip",
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        glow = { colour = 0xE96440FF, bloom = 0.2, threshold = 0.02 }
        
    },

    male_grifter_03_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_grifter_03_build.zip",
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        glow = { colour = 0xE96440FF, bloom = 0.2, threshold = 0.02 }
        
    },

    rook_outfit_03_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_rook_outfit_03_build.zip",
        gloves = GLOVES.SIMPLE,
        colours = {base_glove = 0x312926ff},
        --no_hand_swap = true,
        glow = { colour = 0xF5A85AFF, bloom = 0.15, threshold = 0.02 }
        
    },

    female_tei_utaro_build = CreatePersonBuild{
                gender = GENDER.FEMALE,
                file = "anim/med_female_tei_utaro_build.zip",
                --gloves = GLOVES.SIMPLE,
                no_hand_swap = true,
                glow = { colour = 0x11FFFEFF, bloom = 0.15, threshold = 0.02 }    
                
    },

    female_andwanette_build = CreatePersonBuild{
                gender = GENDER.FEMALE,
                file = "anim/med_female_andwanette_build.zip",
                --gloves = GLOVES.SIMPLE,
                --no_hand_swap = true,
                glow = { colour = 0xFFB860FF, bloom = 0.15, threshold = 0.02 }
        
    },

    male_automech_bodyguard = CreateSpecialBuild{
        gender = GENDER.UNDISCLOSED,
        anims = {"anim/med_dial_automech.zip", "anim/med_dial_plax_automech.zip", "anim/med_portrait.zip"},
        file = "anim/med_automech_bodyguard_build.zip",
        gloves = GLOVES.SIMPLE,
        no_hand_swap = true,
        colours = {base_glove = 0x6D4A36ff,accent =0x6D4A36ff},
        glow = { colour = 0xCB5D3CFF, bloom = 0.5, threshold = 0.02 },

    },

    female_rentorian_diplomat_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_rentorian_diplomat_build.zip",
        colours = {base_glove = 0x223249ff},
        no_hand_swap = true,
        glow = { colour = 0xFDCF94FF, bloom = 0.15, threshold = 0.02 }

    },

    female_hesh_luminiciate_promoted_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_hesh_luminiciate_promoted_build.zip",
        --gloves = GLOVES.SIMPLE,
        no_hand_swap = true,
        glow = { colour = 0x11FFFEFF, bloom = 0.3, threshold = 0.1 }
              
    },

    male_hesh_luminiciate_promoted_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_hesh_luminiciate_promoted_build.zip",
        --gloves = GLOVES.SIMPLE,
        no_hand_swap = true,
        glow = { colour = 0x11FFFEFF, bloom = 0.3, threshold = 0.1 }
              
    },

    male_merchant_promoted_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_merchant_promoted_build.zip",
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        glow = { colour = 0x11FFFEFF, bloom = 0.3, threshold = 0.1 }
              
    },

    female_merchant_promoted_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_merchant_promoted_build.zip",
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        glow = { colour = 0x11FFFEFF, bloom = 0.3, threshold = 0.1 }
              
    },

    male_pearl_resident_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_pearl_resident_build.zip",
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        glow = { colour = 0x157FFFBFF, bloom = 0.2, threshold = 0.1 }
              
    },

    female_pearl_resident_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_pearl_resident_build.zip",
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        glow = { colour = 0x57FFFBFF, bloom = 0.2, threshold = 0.1 }
              
    },

    female_seemli_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_seemli_build.zip",
        gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        colours = {base_glove = 0x3F2620ff,accent =0x3F2620ff},
        glow = { colour = 0xDEEF57FF, bloom = 0.2, threshold = 0.1 }
        
    },

    female_eden_yllonwa_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_eden_yllonwa_build.zip",
        gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
        colours = {base_glove = 0x261E1Eff,accent =0x261E1Eff},
        glow = { colour = 0x33FFFFFF, bloom = 0.2, threshold = 0.1 }
        
    },

    female_rise_valet_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_rise_valet_build.zip",
        glow = { colour = 0xF08C49FF, bloom = 0.6, threshold = 0.3 }
        --no_hand_swap = true,
        
    },male_rise_valet_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_rise_valet_build.zip",
        glow = { colour = 0xF08C49FF, bloom = 0.6, threshold = 0.3 }
        --no_hand_swap = true,
        
    },

    female_guard_investigator_build = CreatePersonBuild{
        gender = GENDER.FEMALE,
        file = "anim/med_female_guard_investigator_build.zip",
        glow = { colour = 0x00FFFFFF, bloom = 0.3, threshold = 0.02 },
        colours = {base_glove = 0x261E1Eff},
        gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
              
    },

    male_guard_investigator_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_guard_investigator_build.zip",
        glow = { colour = 0x00FFFFFF, bloom = 0.3, threshold = 0.02 },
        colours = {base_glove = 0x261E1Eff},
        gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
              
    },

    male_hesh_bishop_build = CreatePersonBuild{
        gender = GENDER.MALE,
        file = "anim/med_male_hesh_bishop_build.zip",
        glow = { colour = 0x11FFFEFF, bloom = 0.2, threshold = 0.05 },
        --gloves = GLOVES.SIMPLE,
        --no_hand_swap = true,
              
    },

    male_deltrean_dignitary_build = CreatePersonBuild{
        gender = GENDER.MALE,
        no_hand_swap = true,
        file = "anim/med_male_deltrean_dignitary_build.zip",
        glow = { colour = 0x8D55FFFF, bloom = 0.5, threshold = 0.02 },
             
    },

    male_bilebroker_build = CreatePersonBuild{
        gender = GENDER.MALE,
        no_hand_swap = true,
        file = "anim/med_male_bilebroker_build.zip",
        glow = { colour = 0xFFC85DFF, bloom = 0.3, threshold = 0.02 },
             
    },





    --creatures
    dire_dog = CreateCreatureBuild({
       
                dialog_build = "anim/med_crayote_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_crayote.zip" },
                portrait_anim = "anim/med_portrait_crayote.zip",
            }),  
    alpha_dire_dog = CreateCreatureBuild({
       
                dialog_build = "anim/med_crayote_alpha_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_crayote.zip", },
                portrait_anim = "anim/med_portrait_crayote.zip",
            }),
    upgraded_alpha_dire_dog = CreateCreatureBuild({
       
                dialog_build = "anim/med_crayote_alpha_upgrade_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_crayote.zip", },
                portrait_anim = "anim/med_portrait_crayote.zip",
            }),
    spitter_dire_dog = CreateCreatureBuild({
       
                dialog_build = "anim/med_crayote_spitter_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_crayote.zip", },
                portrait_anim = "anim/med_portrait_crayote.zip",
                glow = { colour = 0xBC6464FF, bloom = 0.3, threshold = 0.02 } 
            }),
    upgraded_dire_dog = CreateCreatureBuild({
       
                dialog_build = "anim/med_crayote_upgrade1_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_crayote.zip" },
                portrait_anim = "anim/med_portrait_crayote.zip",
            }),
    combat_drone = CreateCreatureBuild({
       
                dialog_build = "anim/med_combat_drone_build.zip",
                dialog_anim = "anim/med_dial_combat_drone.zip",
                combat_anims = { "anim/med_combat_drone.zip", },
                portrait_anim = "anim/med_portrait_drone.zip",
                glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 }
            }),
    lumin_turret = CreateCreatureBuild({
       
                dialog_build = "anim/med_combat_drone_tei_build.zip",
                dialog_anim = "anim/med_dial_lumin_turret.zip",
                combat_anims = { "anim/med_combat_drone_tei.zip", },
                portrait_anim = "anim/med_portrait_lumin_turret.zip",
                glow = { colour = 0x57FFFBFF, bloom = 0.15, threshold = 0.02 }
            }),
    critter = CreateCreatureBuild({
       
                dialog_build = "anim/beast_dial.zip",
                dialog_anim = "anim/beast_dial.zip",
                combat_anims = { "anim/beast_dial.zip", },
                portrait_anim = "anim/beast_dial.zip",
            }),
    vroc = CreateCreatureBuild({
       
                dialog_build = "anim/med_vroc_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_crayote.zip", },
                portrait_anim = "anim/med_portrait_crayote.zip", 
            }),
    upgraded_vroc = CreateCreatureBuild({
       
                dialog_build = "anim/med_vroc_upgrade1_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_crayote.zip", },
                portrait_anim = "anim/med_portrait_crayote.zip", 
            }),
    flead = CreateCreatureBuild({
       
                dialog_build = "anim/med_flead_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_crayote.zip", "anim/med_combat_flead.zip", },
                portrait_anim = "anim/med_portrait_crayote.zip",
                glow = { colour = 0x8BCAD7FF, bloom = 0.09, threshold = 0.02 },
            }),
    flead_larvae = CreateCreatureBuild({
       
                dialog_build = "anim/med_flead_larvae.zip",
                combat_anims = { "anim/med_combat_flead_larvae.zip"},
            }),
    flead_queen = CreateCreatureBuild({
                glow = { colour = 0x8BCAD7FF, bloom = 0.05, threshold = 0.02 },
                dialog_build = "anim/med_flead_queen.zip",
                dialog_anim = "anim/med_dial_flead_queen.zip",
                combat_anims = { "anim/lrg_combat_flead_queen.zip", },
                portrait_anim = "anim/med_portrait_flead_queen.zip",
            }),
    shroog = CreateCreatureBuild({
                glow = { colour = 0x81C67EFF, bloom = 0.1, threshold = 0.02 },
                dialog_build = "anim/lrg_shroog_build.zip",
                dialog_anim = "anim/med_dial_shroog.zip",
                combat_anims = { "anim/lrg_combat_shroog.zip", },
                portrait_anim = "anim/med_portrait_shroog.zip",
            }),
    shrooglet = CreateCreatureBuild({
   
                dialog_build = "anim/med_young_shroog_build.zip",
                dialog_anim = "anim/med_dial_young_shroog.zip",
                combat_anims = { "anim/med_combat_young_shroog.zip", },
                portrait_anim = "anim/med_portrait_young_shroog.zip",
            }),
    upgraded_shrooglet = CreateCreatureBuild({
       
                dialog_build = "anim/med_young_shroog_upgrade1_build.zip",
                dialog_anim = "anim/med_dial_young_shroog.zip",
                combat_anims = { "anim/med_combat_young_shroog.zip", },
                portrait_anim = "anim/med_portrait_young_shroog.zip",
            }),

    drusk_01 = CreateCreatureBuild({

                dialog_build = "anim/med_drusk_01_build.zip",
                combat_anims = {"anim/med_combat_drusk_01.zip"},
                dialog_anim = "anim/med_dial_drusk.zip",
                glow = { colour = 0xC3DE2EFF, bloom = 0.15, threshold = 0.2 },
                portrait_anim = "anim/med_portrait_drusk.zip",
        }),

    drusk_02 = CreateCreatureBuild({

                dialog_build = "anim/med_drusk_02_build.zip",
                combat_anims = {"anim/med_combat_drusk_02.zip"},
                dialog_anim = "anim/med_dial_drusk.zip",
                glow = { colour = 0xFFA720FF, bloom = 0.2, threshold = 0.1 },
                portrait_anim = "anim/med_portrait_drusk.zip",
        }),

    luminthian = CreateCreatureBuild({
                combat_anims = { "anim/med_combat_luminthian.zip", },
                dialog_build = "anim/med_luminthian_build.zip",

       
                dialog_anim = "anim/med_dial_luminthian.zip",
                portrait_anim = "anim/med_portrait_luminthian.zip",
                glow = { colour = 0x63FFFDFF, bloom = 0.05, threshold = 0.02 },
            }),

    basic_snail = CreateSnailBuild({
                build = "anim/med_snail_build.zip",
                
            }),

    combat_snail = CreateSnailBuild({
                glow = { colour = 0x59FFFAFF, bloom = 0.15, threshold = 0.02 },
                build = "anim/med_snail_build.zip",
            }),

    merchant_snail = CreateSnailBuild({
                build = "anim/med_snail_merchant_build.zip",
                
            }),

    racing_snail = CreateSnailBuild({
                build = "anim/med_snail_racer_build.zip",
                glow = { colour = 0x11FFFEFF, bloom = 0.18, threshold = 0.02 },
            }),    

    autodog = CreateCreatureBuild({
                dialog_build = "anim/med_autodog_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_autodog.zip" },
                portrait_anim = "anim/med_portrait_crayote.zip",
                glow = { colour = 0xFFD57FFF, bloom = 0.15, threshold = 0.02 },
            }),

    feral_autodog = CreateCreatureBuild({
                dialog_build = "anim/med_autodog_feral_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_autodog_feral.zip" },
                portrait_anim = "anim/med_portrait_crayote.zip",
                glow = { colour = 0xFFD57FFF, bloom = 0.15, threshold = 0.02 },
            }),

    rise_autodog = CreateCreatureBuild({
                dialog_build = "anim/med_autodog_rise_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_autodog.zip" },
                portrait_anim = "anim/med_portrait_crayote.zip",
                glow = { colour = 0xFFD57FFF, bloom = 0.15, threshold = 0.02 },
            }),

    erchin = CreateCreatureBuild({
                dialog_build = "anim/med_erchin_build.zip",
                dialog_anim = "anim/med_dial_erchin.zip",
                combat_anims = { "anim/med_combat_erchin.zip" },
                portrait_anim = "anim/med_portrait_young_shroog.zip",
            }),

    upgraded_erchin = CreateCreatureBuild({
                dialog_build = "anim/med_erchin_upgrade1_build.zip",
                dialog_anim = "anim/med_dial_erchin.zip",
                combat_anims = { "anim/med_combat_erchin.zip" },
                portrait_anim = "anim/med_portrait_young_shroog.zip",
            }),


    flead_armoured = CreateCreatureBuild({
                glow = { colour = 0xFE8500FF, bloom = 0.15, threshold = 0.02 },
                dialog_build = "anim/med_flead_armoured_build.zip",
                dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_flead_armoured.zip" },
                portrait_anim = "anim/med_portrait_crayote.zip",
            }),

    grout_knuckles = CreateCreatureBuild({
                glow = { colour = 0xE1D771FF, bloom = 0.06, threshold = 0.02 },
                dialog_build = "anim/med_burs1_build.zip",
                -- dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_burs1.zip" },
                -- portrait_anim = "anim/med_portrait_crayote.zip",
            }),

    grout_spark_mine = CreateCreatureBuild({
                glow = { colour = 0xF5B25EFF, bloom = 0.1, threshold = 0.02 },
                dialog_build = "anim/med_burs2_spark_build.zip",
                -- dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_burs2_spark.zip" },
                -- portrait_anim = "anim/med_portrait_crayote.zip",
            }),

    grout_egg_batch = CreateCreatureBuild({
                glow = { colour = 0xC9D269FF, bloom = 0.015, threshold = 0.02 },
                dialog_build = "anim/med_burs3_egg_build.zip",
                -- dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_burs3_egg.zip" },
                -- portrait_anim = "anim/med_portrait_crayote.zip",
            }),    
    grout_loot = CreateCreatureBuild({
                glow = { colour = 0xC9D269FF, bloom = 0.08, threshold = 0.02 },
                dialog_build = "anim/med_burs4_loot_box_build.zip",
                -- dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_burs4_loot_box.zip" },
                -- portrait_anim = "anim/med_portrait_crayote.zip",
            }),

    grout_eyes = CreateCreatureBuild({
                glow = { colour = 0xE0D770FF, bloom = 0.09, threshold = 0.02 },
                dialog_build = "anim/med_burs5_eye_build.zip",
                -- dialog_anim = "anim/med_dial_crayote.zip",
                combat_anims = { "anim/med_combat_burs5_eyeball.zip" },
                -- portrait_anim = "anim/med_portrait_crayote.zip",
            }),

    bog_burr_boss = CreateCreatureBuild({
                glow = { colour = 0xF5B25EFF, bloom = 0.1, threshold = 0.02 },
                dialog_build = "anim/med_burs_boss_build.zip",
                dialog_anim = "anim/med_dial_burs_boss.zip",
                combat_anims = { "anim/med_combat_burs_boss.zip", },
                portrait_anim = "anim/med_portrait_burs_boss.zip",
            }),


    lumicyte = CreateCreatureBuild({
       
                dialog_build = "anim/med_lumicyte_build.zip",
                dialog_anim = "anim/med_dial_lumicyte.zip",
                combat_anims = { "anim/med_combat_lumicyte.zip"},
                portrait_anim = "anim/med_portrait_lumicyte.zip",
                glow = { colour = 0x11FFFEFF, bloom = 0.1, threshold = 0.3 },
            }),

    lumicyte_upgrade = CreateCreatureBuild({
       
                dialog_build = "anim/med_lumicyte_upgrade1_build.zip",
                dialog_anim = "anim/med_dial_lumicyte.zip",
                combat_anims = { "anim/med_combat_lumicyte.zip"},
                portrait_anim = "anim/med_portrait_lumicyte.zip",
                glow = { colour = 0x11FFFEFF, bloom = 0.1, threshold = 0.3 },
            }), 

    lumicyte_vix = CreateCreatureBuild({
       
                dialog_build = "anim/med_lumicyte_vix_build.zip",
                dialog_anim = "anim/med_dial_lumicyte.zip",
                combat_anims = { "anim/med_combat_lumicyte.zip"},
                portrait_anim = "anim/med_portrait_lumicyte.zip",
                glow = { colour = 0xDFFF5EFF, bloom = 0.15, threshold = 0.3 },
            }),       

    heshbladder = CreateCreatureBuild({
       
                dialog_build = "anim/med_combat_jelly_build.zip",
                dialog_anim = "anim/med_dial_jelly.zip",
                combat_anims = { "anim/med_combat_jelly.zip"},
                glow = { colour = 0x52D7DCFF, bloom = 0.2, threshold = 0.2 },
                portrait_anim = "anim/med_portrait_jelly.zip",
            }),

    heshbladder_vix = CreateCreatureBuild({
       
                dialog_build = "anim/med_combat_jelly_vix_build.zip",
                dialog_anim = "anim/med_dial_jelly.zip",
                combat_anims = { "anim/med_combat_jelly.zip"},
                glow = { colour = 0xDFFF5EFF, bloom = 0.07, threshold = 0.2 },
                portrait_anim = "anim/med_portrait_jelly.zip",
            }),

    droad = CreateCreatureBuild({
                dialog_build = "anim/lrg_combat_droad_build.zip",
                dialog_anim = "anim/med_dial_droad.zip",
                combat_anims = { "anim/lrg_combat_droad.zip"},
                portrait_anim = "anim/med_portrait_droad.zip",
            }),

    snug = CreateCreatureBuild({
                dialog_build = "anim/lrg_combat_snug_build.zip",
                dialog_anim = "anim/med_dial_snug.zip",
                combat_anims = { "anim/lrg_combat_snug.zip"},
                portrait_anim = "anim/med_portrait_snug.zip",
                glow = { colour = 0xF4BE5FFF, bloom = 0.2, threshold = 0.2 },
            }),          

    mutant_vix = CreateCreatureBuild({
                dialog_build = "anim/lrg_combat_vixmalli_build.zip",
                dialog_anim = "anim/med_dial_mutant_vix.zip",
                combat_anims = { "anim/lrg_combat_vixmalli.zip"},
                dialog_anims = { "anim/med_male_vixmali_build.zip", "anim/head_male_vixmali_build.zip" },
                portrait_anim = "anim/med_portrait_mutant_vix.zip",
                glow = { colour = 0x27FFFFFF, bloom = 0.2, threshold = 0.2 },
            }),  
            
}





Content.AddCharacterBuilds(builds)