local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local DroneManager = class("DroneManager")
DroneManager.can_reload = true

local drone_positions = 
{
    {-2.8, 3.5},
    {-0.4, 6},
    {2.8, 3.5},
}

function DroneManager:init(battle, fighter, ent)
    self.battle = battle
    self.fighter = fighter
    self.ent = ent
    self.drones = {}
end

function DroneManager:Reorganize()
    for i,drone in ipairs(self.drones) do
        drone.cmp.DroneController:SetOffset(drone_positions[i][1], drone_positions[i][2])
    end
end

function DroneManager:ReplenishDrones( ignore_reorganize )
    local fighter_scale = self.ent.cmp.AnimFighter:GetFakeScale()
    local fighter_x, fighter_y, fighter_z = self.ent.cmp.Transform:GetLocalPosition()

    local destx = fighter_x 
    local desty = fighter_y
    local destz = fighter_z - 1
    for i=1,3 do
        local drone = self.ent.sim:SpawnPrefab("andwanette_drone", {entity_to_follow=self.ent, team = self.fighter.team, fighter = self.fighter}, destx, desty, destz)
        table.insert(self.drones, drone)
    end
    if not ignore_reorganize then
        self:Reorganize()
    end
end

function DroneManager:DoAction( action_id, single, target )
    if single then
        local next_drone = self.drones[1]
        next_drone.cmp.DroneController:PlayAction( action_id, target, self, self.battle )
    else
        for i,drone in ipairs(self.drones) do
            drone.cmp.DroneController:PlayAction( action_id, target, self, self.battle )
        end
    end
end

function DroneManager:KillAllDrones( )
    for i,drone in ipairs(self.drones) do
        drone.cmp.DroneController:PlayAction("kill", nil, self, self.battle)
    end
end

function DroneManager:RemoveDrone( drone )
    table.arrayremove(self.drones, drone)
end

function DroneManager:GetDroneCount( )
    return #self.drones
end

local def = CharacterDef("ANDWANETTE",
{
    unique = true,

    base_def = "WEALTHY_MERCHANT",
    alias = "ANDWANETTE",
    species = SPECIES.SHROKE,
    gender = GENDER.FEMALE,
    title = "Socialite",
    name = "Andwanette",
    faction_id = "FEUD_CITIZEN",
    renown = 3,
    combat_strength = 2,
    theme_music = "event:/music/story/smith/story_smith_andwanette",
    --boss = true,
    tags = {"wealthy" },
    tags = {"plot_armour", "no_patron"},
    
    loved_graft = "celebrity",
    hated_graft = "ostracized",
    death_item = "brain_pearl",

    -- battle_preview_anim = "anim/boss_hesh_slide.zip",
    -- battle_preview_offset = { x = 185, y = 10 },
    -- battle_preview_glow = { colour = 0x10ffffFF, bloom = 0.3, threshold = 0.02 },
    -- battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_heshboss",
    -- icon = engine.asset.Texture( "icons/b_hesh_boss.tex"),
    -- title = "",
    -- voice_actor = "",
    
    anims = { "anim/med_female_andwanette_build.zip", },
    combat_anims = { "anim/med_combat_unarmed_andwanette.zip" },
    head = "head_female_andwanette",
    build = "female_andwanette_build",

    -- negotiation_data = 
    -- {
    --     modifiers =
    --     {

    --     },
    --     behaviour = 
    --     {
           
    --     }
    -- },
    
    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 85,

        conditions = 
        { 
            expanded_orb_drone =
            {
                hidden = true,
                -- This is just for tracking
            },

        },

        attacks = 
        {
            andwanette_attack1 = table.extend(NPC_ATTACK)
            {
                name = "Self-Destruct",
                anim = "attack1",
                flags = CARD_FLAGS.RANGED,
                CanPlayCard = function( self, battle, target )
                    return self.owner.manager and self.owner.manager:GetDroneCount() > 0 and not self.owner:HasCondition("expanded_orb_drone")
                end,
                synchronous = true,
                combo_starter = true,

                OnPostResolve = function( self, battle, attack )
                    if self.owner.manager then
                        self.owner.manager:DoAction("self_destruct", true, attack.target)
                    end
                end
            },

            andwanette_attack2 = table.extend(NPC_ATTACK)
            {
                name = "Blast",
                anim = "attack2",
                flags = CARD_FLAGS.RANGED,
                CanPlayCard = function( self, battle, target )
                    return self.owner:HasCondition("expanded_orb_drone")
                end,
                synchronous = true,
                combo_starter = true,

                OnPostResolve = function( self, battle, attack )
                    if self.owner.manager then
                        self.owner.manager:DoAction("blast", true, attack.target)
                        self.owner:RemoveCondition("expanded_orb_drone", 1, self)
                    end
                end
            },

            andwanette_expand = table.extend(NPC_BUFF)
            {
                name = "Expand",
                anim = "taunt",
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,
                synchronous = true,

                CanPlayCard = function( self, battle, target )
                    return self.owner.manager and self.owner.manager:GetDroneCount() > 0 and not self.owner:HasCondition("expanded_orb_drone")
                end,

                OnPreResolve = function( self, battle, attack )
                    if self.owner.manager then
                        self.owner.manager:DoAction("expand", true, attack.target)
                        self.owner:AddCondition("expanded_orb_drone", 1, self)
                    end
                end
            },

            andwanette_replenish = table.extend(NPC_BUFF)
            {
                name = "Replenish",
                anim = "taunt",
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                CanPlayCard = function( self, battle, target )
                    return self.owner.manager and self.owner.manager:GetDroneCount() == 0
                end,

                PrePresAnim = function( self, anim_fighter )
                    if self.owner.manager then
                        self.owner.manager:ReplenishDrones()
                    end
                end,
            },
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                self.attacks = self:MakePicker()
                    :AddID("andwanette_attack1", 2)
                    :AddID("andwanette_attack2", 1)
                    :AddID("andwanette_expand", 1)

                self.replenish = self:AddCard("andwanette_replenish")

                self:Present( function( screen, ent )
                        fighter.manager = DroneManager( self.engine, fighter, ent)
                        self.manager = fighter.manager
                        fighter.battle:PlayCard(self.replenish)
                    end )
                self:SetPattern(self.Cycle)
            end,
            OnDeactivateFighter = function( self, fighter )
                if self.manager then
                    self.manager:KillAllDrones()
                end
            end,

            Cycle = function( self )
                if self.manager and self.manager:GetDroneCount() == 0 then
                    self:ChooseCard(self.replenish)
                else
                    self.attacks:ChooseCard()
                end
            end
        },
    },
})

Content.AddCharacterDef( def )

-- DELETE THIS DEF WHEN ANDWANETTE IS DONE
Content.AddCharacterDef
(
    CharacterDef("ANDWANETTE_DRONE_TEST",
    {
        base_def = "DRONE",
        anims = { "anim/med_combat_drone_orb_build.zip", },
        combat_anims = { "anim/med_combat_drone_orb.zip" },
    })
)