local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local FLAGS = battle_defs.CARD_FLAGS

---------------------------------------------------

Content.AddCharacterDef
(
    CharacterDef("AUTODOG",
    {
        title = "AutoDog",
        build = "autodog",
        death_money = 0,
        species = SPECIES.MECH,
        gender = GENDER.UNDISCLOSED,
        voice_event = "event:/vo/compendium/speak_autodog",

        faction_id = "SPARK_BARONS",
        death_item = "targeting_core",
        negotiation_ally_image = engine.asset.Texture( "negotiation/modifiers/recruit_spark_baron_autodog.tex"),

        combat_strength = 1.5,
        renown = 1,

        plax_flags = PLAX_FLAGS.ALLOW_MONSTERS, -- Only fits in slots flagged "ALLOW_MONSTERS"

        nohead = true,

        fight_data = 
        {
            MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW,
            MAX_MORALE = MAX_MORALE_LOOKUP.MECH,

            status_widget_head_dy = 2.0,
            status_widget_head_dx = 0.25,

            formation = FIGHTER_FORMATION.FRONT_Z,

            conditions =
            {
                dread =
                {
                    name = "Dread",
                    desc = "Deal 20% less damage and receive 20% more damage. Remove 1 stack at the end of {1}'s turn.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName() )
                    end,

                    ctype = CTYPE.DEBUFF,

                    -- Don't apply to robots
                    CanApply = function( self, source )
                        return not (not self.owner.agent:IsSentient() and self.owner:HasCondition("METALLIC"))
                    end,

                    event_handlers = 
                    {
                        [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                            if target == self.owner then
                                dmgt:AddDamage( math.round(dmgt.min_damage * .2) , math.round(dmgt.max_damage * .2), self )
                            end
                            if card.owner == self.owner then
                                dmgt:AddDamage( -math.round(dmgt.min_damage * .2) , -math.round(dmgt.max_damage * .2), self )
                            end
                        end,

                        [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                            if fighter == self.owner then
                                self.owner:RemoveCondition( self.id, 1 )
                            end
                        end,
                    },
                }
            },

            cards = { "autodog_shoot", },
            attacks = {
                autodog_shoot = table.extend(NPC_RANGED)
                {
                    name = "Spark Blast",
                    anim = "attack1",

                    
                },

                autodog_bite = table.extend(NPC_MELEE)
                {
                    name = "Bite",
                    anim = "attack2",
                    flags = FLAGS.MELEE | FLAGS.DEBUFF,
                    features = 
                    {
                        WOUND = 2,
                    },
                },

                autodog_howl = 
                {
                    name = "Threatening Howl",
                    anim = "taunt",

                    flags = FLAGS.SKILL | FLAGS.DEBUFF,
                    target_mod = TARGET_MOD.TEAM,

                    OnPostResolve = function( self, battle, attack )
                        attack:AddCondition("dread", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ), self)
                    end
                },
            },

            hit_tags = { "small" },

            behaviour =
            {
                OnActivate = function( self )
                    self.fighter:AddCondition( "METALLIC" )
                    self.attacks = self:MakePicker()
                        :AddID( "autodog_shoot", 4 )
                        :AddID( "autodog_bite", 2)
                        :AddID( "autodog_howl", 1 )
                    self.howl = self:AddCard("autodog_howl")
                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    self.attacks:ChooseCard()
                end,
            },
        },
    })
)

Content.AddCharacterDef
(

    CharacterDef("FERAL_AUTODOG",
    {
        base_def = "AUTODOG",

        title = "Feral AutoDog",
        build = "feral_autodog",
        death_money = 0,
        death_item = "rusty_power_core",
        species = SPECIES.MECH,

        faction_id = "MONSTER",
        combat_strength = .75,
        
        fight_data = 
        {
            MAX_HEALTH = 8,
            MAX_MORALE = MAX_MORALE_LOOKUP.MECH,

            formation = FIGHTER_FORMATION.FRONT_Z,

            status_widget_head_dy = 2.0,
            status_widget_head_dx = 0.25,

            attacks = {
                feral_autodog_bite = table.extend(NPC_MELEE)
                {
                    name = "Bite",
                    anim = "bite",
                    flags = FLAGS.MELEE | FLAGS.DEBUFF,
                    features = 
                    {
                        WOUND = 1,
                    },
                    damage_mult = .5,
                },
            },

            hit_tags = { "small" },

            behaviour =
            {
                OnActivate = function( self )
                    self.fighter:AddCondition( "METALLIC" )
                    self.attacks = self:MakePicker()
                        :AddID( "feral_autodog_bite", 4 )
                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    self.attacks:ChooseCard()
                end,
            },
        },
    })    
)


Content.AddCharacterDef
(
    CharacterDef("RISE_AUTODOG",
    {
        --designer_note = "Nothing special, they just power up over time and bleed you once in a while.",
        base_def = "AUTODOG",

        title = "Cobbledog",
        build = "rise_autodog",
        death_money = 0,
        species = SPECIES.MECH,

        faction_id = "MONSTER",
        death_item = nil,
        combat_strength = 1,
        negotiation_ally_image = engine.asset.Texture( "negotiation/modifiers/recruit_rise_cobbledog.tex"),
        
        fight_data = 
        {
            MAX_HEALTH = MAX_HEALTH_LOOKUP.CREATURE,
            MAX_MORALE = MAX_MORALE_LOOKUP.MECH,

            status_widget_head_dy = 2.0,
            status_widget_head_dx = 0.25,

            formation = FIGHTER_FORMATION.FRONT_Z,

            attacks = {
                rise_autodog_shoot = table.extend(NPC_RANGED)
                {
                    name = "Spark Blast",
                    anim = "attack1",
                    damage_mult = .5,
                },
                rise_autodog_bite = table.extend(NPC_MELEE)
                {
                    name = "Bite",
                    anim = "attack2",
                    flags = FLAGS.MELEE | FLAGS.DEBUFF,
                    features = 
                    {
                        WOUND = 1,
                    },
                    damage_mult = .5,
                },
            },

            hit_tags = { "small" },

            behaviour =
            {
                OnActivate = function( self )
                    self.fighter:AddCondition( "METALLIC" )
                    self.attacks = self:MakePicker()
                        :AddID( "rise_autodog_shoot", 4 )
                        :AddID( "rise_autodog_bite", 2)
                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    self.attacks:ChooseCard()
                end,
            },
        },
    })    
)