require "content/characters/base_types"
local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

Content.AddCharacterDef
(
	CharacterDef("AUTOMECH_BOSS",
	{

		base_def = "NPC_BASE",
		title = "Automech",
		species = SPECIES.MECH,
		alias = "AUTOMECH_BOSS",
		voice_actor = "robotBoss",
        faction_id = "NEUTRAL",


		combat_strength = 4,
		renown = 1,
		boss = true,
        battle_preview_anim = "anim/boss_automech_slide.zip",
        battle_preview_offset = { x = 120, y = 70 },
        battle_preview_glow = { colour = 0xCB5D3CFF, bloom = 0.7, threshold = 0.02 },
        battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_automech_boss",
        icon = engine.asset.Texture( "icons/b_automech_boss.tex" ),

		unique = true,
		can_talk = true,
        aspects = {
            social_connections = false,
        },

		bio = "",

		gender = "MALE",
	    hair_colour = 3434903039,
	    head = "head_automech_boss",
	    name = "Bossbit",
	    skin_colour = 1652061183,

		combat_anims = { "anim/med_combat_automech_boss.zip" },
		death_item = "screamer",

		fight_data = 
		{
			MAX_HEALTH = 140,
        	MAX_MORALE = MAX_MORALE_LOOKUP.IMMUNE,
			cards = { "automech_boss_kick" },
			battle_scale= 1.25,

            fx =
            {
            	automech_leg_thrust = { auto = true },
            },

            conditions =
			{
				disruptor_field = 
				{
					name = "Disruptor Field",
					desc = "{1} is immune to damage and debuffs while any allies with {disruptor_pylon} are alive.",
					desc_fn = function( self, fmt_str )
						return loc.format(fmt_str, loc.cap( self:GetOwnerName() ))
					end,

					ctype = CTYPE.BUFF,
					hud_fx = {"disruptor_field"},
					apply_fx = {"disruptor_field_in"},
					unapply_fx = {"disruptor_field_break"},

					event_priorities =
			        {
			            [ BATTLE_EVENT.CALC_DAMAGE ] = 999, -- Always last.
			        },

			        CanBeTargetted = function( self, card, fighter )
			            if fighter == self.owner and card.owner:GetTeam() ~= self.owner:GetTeam() and not card.owner:IsPlayer() then
			                return false
			            end
			            return true
			        end,

			        OnPreDamage = function( self, damage, attacker, battle, source )
			            return 0
		            end,

					event_handlers =
					{
						[ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
							if target == self.owner then
								dmgt:ModifyDamage(0, 0, self)
							end
						end,

						[ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
			                if hit and hit.target == self.owner and not hit.evaded then
			                    hit.defended = true
			                    self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_FX, "disruptor_field_hit" )
			                end
			            end,

						[ BATTLE_EVENT.DEACTIVATED_FIGHTER ] = function( self, fighter )
							if fighter:GetTeam() == self.owner:GetTeam() then
								local pylon_left = false
								for i,ally in self.owner:GetTeam():Fighters() do
									if ally ~= fighter and ally:HasCondition("disruptor_pylon") then
										pylon_left = true
										break
									end
								end
								if not pylon_left then
									self.owner:RemoveCondition(self.id)
								end
							end
						end,

						[ BATTLE_EVENT.CALC_MODIFY_STACKS ] = function( self, acc, condition_id, fighter, source )
			                if fighter == self.owner and self.owner:IsActive() then
			                    local con_def = Content.GetBattleCondition(condition_id)
			                    if con_def and con_def.ctype == CTYPE.DEBUFF then
			                        acc:ModifyValue( 0, self )
			                    end
			                end
			            end,
					},
				},

				automech_flame_cannon =
				{
					name = "Automech Flame Cannon",
					desc = "{1}'s multi-attack applies {BURN} unless mitigated.",
					desc_fn = function( self, fmt_str )
						return loc.format(fmt_str, self:GetOwnerName())
					end,
				},

				disruptor_pylon = 
				{
					name = "Disruptor Pylon",
					desc = "{1} is syphoning power from this unit, protecting {1} from all damage.",
					desc_fn = function( self, fmt_str )
						return loc.format(fmt_str, self.field_owner and self.field_owner:GetName() or "Disruptor Field")
					end,

					OnApply = function( self, battle )
						for i,fighter in self.owner:GetTeam():Fighters() do
							if fighter:HasCondition("disruptor_field") then
								self.field_owner = fighter
								break
							end
						end
					end
				},
			},

			attacks =
			{
				automech_boss_kick = table.extend(NPC_ATTACK)
                {
                    name = "Kick",
                    anim = "attack1",
                    flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

                    base_damage = {6, 7, 8, 9},
                    scorched_amt = {1, 1, 2, 2},

                    OnPostResolve = function( self, battle, attack )
                    	attack:AddCondition("SCORCHED", self.scorched_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                    end,
                },

                automech_boss_distributed_fire = table.extend(NPC_RANGED)
                {
                	name = "Distributed Fire",
                	pre_anim = "shoot_pre",
			        anim = "shoot",
			        post_anim = "shoot_pst",

                	flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
                	target_mod = TARGET_MOD.RANDOM1,
                	target_count = 6,
                	count_per_fighters = { 4, 5, 6, 7 },
                	base_damage = { 2, 3, 3, 4 },

                	burn_amt = { 1, 1, 2, 3 },

                	generate_buff_fn = function( self )
                		self.target_count = self.count_per_fighters[ self.engine:GetPlayerTeam():NumActiveFighters() or 1]
                	end,

                    event_handlers =
                    {
                    	[ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
			                if source and source.card == self then
			                    fighter:AddCondition("BURN", self.burn_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
			                end
			            end,
                    },
                },

                automech_boss_summon = table.extend(NPC_BUFF)
                {
                	name = "Mustering Call",
                	anim = "taunt",
                	flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                	target_type = TARGET_TYPE.SELF,

                	autodog_scaling = { 3, 3, 4, 4 },

                	OnPostResolve = function( self, battle, attack )
                		self.owner.behaviour.last_summon = battle:GetTurns()
                		local open_slots = self.owner:GetTeam():GetMaxFighters() - self.owner:GetTeam():NumActiveFighters()
                		if open_slots > 0 then
	                		self.owner:AddCondition("disruptor_field", 1, self)
	                		for i=1, open_slots do
	                			local agent = Agent( "FERAL_AUTODOG" )
					            local new_fighter = Fighter.CreateFromAgent( agent, self.autodog_scaling[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] )
					            new_fighter.no_death_loot = true
					            self.owner:GetTeam():AddFighter( new_fighter )
					            new_fighter:AddCondition("disruptor_pylon", 1, self)
	                		end
	                		self.owner:GetTeam():ActivateNewFighters( battle )
	                	end
                	end
                },
			},

			behaviour =
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID( "automech_boss_kick", 3 )
						:AddID( "automech_boss_distributed_fire", 1 )
					self.summon = self:AddCard("automech_boss_summon")

					self.fighter:AddCondition("ARMOURED", 6)
					self.fighter:AddCondition("METALLIC")
					self.fighter:AddCondition("automech_flame_cannon")

					self:SetPattern( self.Cycle )
				end,

				Cycle = function( 	self )    
					if self.battle:GetTurns() == 0 or (self.battle:GetTurns() - (self.last_summon or -5) >= 4 and self.fighter:GetTeam():GetMaxFighters() - self.fighter:GetTeam():NumActiveFighters() > 0) then
                        self:ChooseCard(self.summon)
                    else
                    	self.attacks:ChooseCard()
                  	end
				end
			},
		},

		negotiation_data = 
		{
			modifiers =
			{
				MECH_CONDUCTOR =
	            {
	                name = "Mech Conductor",
	                desc = "While {1} has any {BARK_BOX} arguments, his core argument is shielded.",
	                desc_fn = function( self, fmt_str )
	                    return loc.format(fmt_str, self:GetOwnerName())
	                end,

	                modifier_type = MODIFIER_TYPE.CORE,

	                loc_strings =
	                {
	                    BARK_BOX_SHIELD_DESC = "{SHIELDED}. Destroy or remove all <b>Bark Box</b> arguments to release the shield!",
	                },

	                CheckForBoxes = function( self )
	                	if self.negotiator:HasModifier("BARK_BOX") then
	                		local core = self.negotiator:FindCoreArgument()
	                		if not core:GetShieldStatus() then
                    			core:SetShieldStatus( true, self.def:GetLocalizedString("BARK_BOX_SHIELD_DESC") )
                    		end
	                	else
	                		local core = self.negotiator:FindCoreArgument()
	                		if core:GetShieldStatus() then
	                			core:SetShieldStatus( false)
	                		end
	                	end
	                end,

	                event_handlers =
	                {
	                    [ EVENT.MODIFIER_ADDED ] = function ( self, modifier, source )
		                    if modifier.negotiator == self.negotiator and modifier.id == "BARK_BOX" then
		                        self:CheckForBoxes()
		                    end
		                end,

		                [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
		                	if modifier.negotiator == self.negotiator and modifier.id == "BARK_BOX" then
		                		self:CheckForBoxes()
		                	end
		                end
	                },
	            },

	            BARK_BOX =
	            {
	            	name = "Bark Box",

                    modifier_type = MODIFIER_TYPE.ARGUMENT,
                    target_enemy = TARGET_ANY_RESOLVE,

                    OnApply = function( self )
                        self.negotiator.behaviour.latest_box = self.engine:GetTurns()
                    end,

                    OnBeginTurn = function( self, minigame )
                        self:ApplyPersuasion()
                    end,

                    max_resolve = 3,
                    min_persuasion = 2,
                    max_persuasion = 2,
                    max_stacks = 1,
	            },

	            GAS = 
	            {
	            	name = "Gas",
	            	desc = "When destroyed, deal {1} damage to all friendly arguments.",
	            	desc_fn = function( self, fmt_str )
	            		return loc.format(fmt_str, self.bounty_damage)
	            	end,

	            	modifier_type = MODIFIER_TYPE.BOUNTY,
	            	max_resolve = 5,

	            	bounty_damage = 3,

	            	OnBounty = function( self, card )
                        local targets = self.engine:CollectAlliedTargets(self)
                        for i,mod in ipairs(targets) do
                        	if mod ~= self then
                        		mod:AttackResolve(self.bounty_damage, self)
                        	end
                        end
                    end,
	            },

	            LIGHT =
	            {
	            	name = "Light",
	            	desc = "This argument can only target <b>Gas</>.",

	            	max_resolve = 8,
	            	modifier_type = MODIFIER_TYPE.ARGUMENT,
                    target_enemy = TARGET_ANY_RESOLVE,

                    OnBeginTurn = function( self, minigame )
                        self:ApplyPersuasion()
                    end,

	            	min_persuasion = 2,
	            	max_persuasion = 2,

	            	CanPlayCard = function( self, source, engine, target )
			            if source == self then
			            	return target.id == "GAS"
			            end
			            return true
			        end,
	            },
			},

			behaviour = 
	        {
	            OnInit = function( self )
	                self.bark_boxes = self:MakePicker()
	                    :AddArgument("BARK_BOX", 1)
	                    :AddArgument("BARK_BOX", 1)
	                    :AddArgument("BARK_BOX", 1)

	                self.gas = self:AddCard("bossbit_gas")
	                self.light = self:AddArgument("LIGHT")

	                self:SetPattern( self.BossCycle )
	                self.negotiator:AddModifier("MECH_CONDUCTOR")
	            end,

	            BossCycle = function( self, turns )
	                self:ChooseFlatNumbers( 1, 1 )
	                if turns % 2 == 0 then
	                	self:ChooseComposure( 1, 2, 3 )
	                end
	                if (self.latest_box or -3) <= turns - 3 then
	                	self.bark_boxes:ChooseCards(3)
	                end
	                if (turns + 1) % 4 == 0 and not self.player_negotiator:HasModifier("GAS") then
	                	self:ChooseCard(self.gas)
	                end
	                if self.player_negotiator:HasModifier("GAS") and not self.negotiator:HasModifier("LIGHT") then
	                	self:ChooseCard(self.light)
	                end
	            end,

	        }
		},

		base_builds = {
			[ GENDER.MALE ] = "male_automech_boss",
		},
	})
)

