local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local def = CharacterDef("BISHOP_OF_FOAM",
{
    unique = true,
    base_def = "ZEALOT",
    alias = "BISHOP_OF_FOAM",
    tags = {"plot_armour", "no_patron"},
    species = SPECIES.SHROKE,
    gender = GENDER.MALE,
    renown = 4,
    combat_strength = 4,
    name = "Pallitriax",
    
    loved_graft = "anointed",
    hated_graft = "",
    death_item = "lumin_canister",
    faction_id = "CULT_OF_HESH",
    title = "Bishop of the Foam",
    -- voice_actor = "",
    
    anims = { "anim/weapon_gun_hesh_bishop.zip", },
    combat_anims = { "anim/med_combat_lumin_sludge_hesh_bishop.zip" },
    head = "head_male_shroke_17",
    build = "male_hesh_bishop_build",

    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.MEDIUM,
        MAX_HEALTH = 85,

        conditions =
        {
            lumin_charged =
            {
                name = "Lumin Charged",
                desc = "{1}'s next attack applies {WOUND {2}} if it causes health loss.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self:GetOwnerName(), self.stacks or 1 )
                end,

                event_handlers = 
                {
                    [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                        if not hit:CheckHitResult( "defended" ) and not hit:CheckHitResult( "evaded" ) and hit.card:IsDamageCard() and hit.attacker == self.owner then
                            hit.target:AddCondition("WOUND", self.stacks, self)
                        end
                    end,

                    [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        if fighter == self.owner then
                            -- Skip the turn where this was applied
                            if self.skipped then
                                self.owner:RemoveCondition(self.id, self.stacks, self)
                            else
                                self.skipped = true
                            end
                        end
                    end
                },
            },
        },

        attacks = 
        {
            bishop_single = table.extend(NPC_ATTACK)
            {
                name = "Blast",
                anim = "shoot",
                flags = CARD_FLAGS.RANGED,
                CanPlayCard = function( self, battle, target )
                    return not self.owner:HasCondition("lumin_charged")
                end,
            },

            bishop_double = table.extend(NPC_ATTACK)
            {
                name = "Lumin Burst",
                anim = "shoot2",
                flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
                target_mod = TARGET_MOD.TEAM,

                CanPlayCard = function( self, battle, target )
                    return self.owner:HasCondition("lumin_charged")
                end,
            },

            bishop_taunt = table.extend(NPC_BUFF)
            {
                name = "Brace",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                OnPostResolve = function( self, battle, attack )
                    attack:AddCondition("DEFEND", 8, self)
                    self.owner:AddCondition("lumin_charged", 2 + (1 *GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )), self)
                end
            },        
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                self.attacks = self:MakePicker()
                    :AddID("bishop_single", 3)
                    :AddID("bishop_double", 2)
                    :AddID("bishop_taunt", 1)

                self:SetPattern(self.Cycle)
            end,


            Cycle = function( self )
                self.attacks:ChooseCard()
            end
        },
    },
})

Content.AddCharacterDef( def )
    