local fun = require "util/fun"
local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
--local CARD_FLAGS = negotiation_defs.CARD_FLAGS
local NEGOTIATION_EVENT = negotiation_defs.EVENT


local CAMERA_SHAKE_PARAMS = { 0.25, 55, 1.25 } -- Magnitude, speed, time

local BURR_BOSS_FORMATION =
{
    { 13.5, 1.5, FIGHTER_FORMATION.FRONT_X },  -- Burr Boss

    { 4, -5, nil },
    { 6, 5, nil },
    { 2, 1, nil }, 
}

local BOGGER_BOSS_FORMATION =
{
    { 7, 1.4, FIGHTER_FORMATION.FRONT_X },  -- Bogger Priest (same as burr boss but X is 7.5 less)

    { 4, -5, nil },
    { 6, 5, nil },
    { 2, 1, nil }, 
}


local function MakeCardBounty(name, card_id)
    return {
        name = name,
        card_to_win = card_id,

        desc = "Destroy this argument to gain {{1}} when fighting {2.name}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.card_to_win, self.negotiator.agent:LocTable())
        end,
        modifier_type = MODIFIER_TYPE.BOUNTY,
        min_resolve = 2,
        max_resolve = 6,
        OnBounty = function( self )
            local core = self.negotiator:FindCoreArgument()
            if core and core.cards_to_win then
                table.insert(core.cards_to_win, self.card_to_win)
            end
        end,
    }
end

local function GroutJoin( fighter, anim_fighter )
    anim_fighter.entity.cmp.AnimController:SetXFlip(fighter:GetTeamID() == TEAM.RED)
    anim_fighter.entity:SetLocalPosition( FIGHT_MAX_DIST * 2, 0, 0 ) -- Placed off screen until they are manually place and play their emerge
    fighter:AddCondition("grout_burr", 3)
end

local function PlaceAndEmerge( fighter, battle )
    for i,ally in fighter:GetTeam():Fighters() do
        if ally ~= fighter and ally.fight_data.is_boss_burr then
            battle:BroadcastEvent( BATTLE_EVENT.CUSTOM, ally, function( fight_screen, ent )
                local anim_fighter = ent.cmp.AnimFighter
                local x, z = anim_fighter:GetHomePosition()
                anim_fighter.entity:SetLocalPosition( x, 0, z )
                anim_fighter:DoSequence( function()
                    anim_fighter:PlayAnim( "emerge" )
                    anim_fighter:WaitAnim()
                end )
                
            end )
        end
    end
end

local function ClearExistingGrout( fighter )
    if fighter:GetTeam():NumActiveFighters() > 1 then
        local to_flee = {}
        for i,ally in fighter:GetTeam():Fighters() do
            if ally ~= fighter then
                table.insert(to_flee, ally)
            end
        end
        for i,ally in ipairs(to_flee) do
            ally:Flee()
        end
    end
end

local StageConditions = {"burr_boss_eye_tracker", "burr_boss_eye", "burr_boss_enrage"}

local function ClearStageConditions( fighter )
    for i,con_id in ipairs(StageConditions) do
        fighter:RemoveCondition(con_id)
    end
end

Content.AddCharacterDef
(
    CharacterDef("BOG_BURR_BOSS",
    {
        base_def = "MONSTER",
        name = "Grout Monster",
        build = "bog_burr_boss",
        combat_strength = 4,
        boss = true,
        -- hide_in_compendium = true,
        alt_boss_id = "BOGGER_BOSS",
        tags = {"no_patron"},

        compendium_offset = { -200, 0 },

        -- battle_preview_anim = "anim/.zip",
        -- battle_preview_offset = { x = 30, y = 70 },
        -- battle_preview_glow = { colour = 0x81C67EFF, bloom = 0.1, threshold = 0.02 },
        battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_bog_burr_boss",
        -- death_item = "",

        negotiation_data = 
        {
            ring_offset = 
            {
                x = 0,
                y = 600,
                z = 0,
            },
            modifiers =
            {
                DARK_INTELLECT =
                {
                    name = "Dark Intellect",
                    desc = "{1}",
                    desc_fn = function( self, fmt_str )
                        local desc = self.current_desc or "DEFAULT"
                        return loc.format(fmt_str, self.def:GetLocalizedString( desc ) or "")
                    end,

                    OnInit = function(self) 
                        self.tentacle_icon = engine.asset.Texture( "negotiation/modifiers/tentacle_phase.tex" )
                        self.blinding_icon = engine.asset.Texture( "negotiation/modifiers/blinding_phase.tex" )
                        self.rash_icon = engine.asset.Texture( "negotiation/modifiers/rash_phase.tex" )
                    end,

                    loc_strings =
                    {
                        DEFAULT = "The Grout Monster is biding its time.",
                        TENTACLE = "The Grout Monster looks angry and is attacking with several appendages.",
                        BLINDING = "The Grout Monster is calm, almost alluring.",
                        RASH = "The Grout Monster is dormant, slowly regenerating.",
                    },

                    modifier_type = MODIFIER_TYPE.CORE,
                    icon = "negotiation/modifiers/blinding_phase.tex",

                    event_handlers =
                    {
                        [ NEGOTIATION_EVENT.MODIFIER_ADDED ] = function ( self, modifier, source )
                            if modifier.negotiator == self.negotiator then
                                if modifier.id == "WRITHING_APPENDAGES" and (self.current_desc == nil or self.current_desc ~= "TENTACLE") then
                                    self.current_desc = "TENTACLE"
                                    self.engine:BroadcastEvent( NEGOTIATION_EVENT.UPDATE_MODIFIER_ICON, self, self.tentacle_icon)
                                end
                                if modifier.id == "SPORE_CLOUD" and (self.current_desc == nil or self.current_desc ~= "BLINDING") then
                                    self.current_desc = "BLINDING"
                                    self.engine:BroadcastEvent( NEGOTIATION_EVENT.UPDATE_MODIFIER_ICON, self, self.blinding_icon)
                                end
                                if modifier.id == "BLISTER" and (self.current_desc == nil or self.current_desc ~= "RASH") then
                                    self.current_desc = "RASH"
                                    self.engine:BroadcastEvent( NEGOTIATION_EVENT.UPDATE_MODIFIER_ICON, self, self.rash_icon)
                                end
                            end
                        end
                    },
                },

                BOG_RASH = 
                {
                    name = "Bog Rash",
                    desc = "At the end of your turn remove 1 <b>Bog Rash</>.",

                    OnApply = function( self )
                        self:SetStacks(3)
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,

                    max_resolve = 2,
                    target_self = TARGET_ANY_RESOLVE,

                    min_persuasion = 2,
                    max_persuasion = 2,

                    IsValidTarget = function( target )
                        return target ~= self
                    end,

                    OnApply = function( self )
                        self:PrepareTurn()
                    end,

                    event_handlers =
                    {
                        [ NEGOTIATION_EVENT.END_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator then
                                self:ApplyPersuasion()
                                negotiator:RemoveModifier( self, 1 )
                            end
                        end,
                    },
                },

                SPORE_CLOUD =
                {
                    name = "Spore Cloud",
                    desc = "The first attack card {1} plays each turn has a random target. At the end of {1}’s turn, remove 1 <b>Spore Cloud</>.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOpponentName(), self:GetOwnerName() )
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,
                    max_resolve = 8,

                    OnApply = function( self )
                        self.active = true
                        self:SetStacks(3)
                    end,

                    event_handlers =
                    {
                        [ NEGOTIATION_EVENT.END_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator then
                                self.active = true
                                negotiator:RemoveModifier( self, 1 )
                            end
                        end,

                        [ NEGOTIATION_EVENT.CALC_TARGET_MOD ] = function( self, acc, source, target )
                            if self.active and source.negotiator ~= self.negotiator and is_instance( source, Negotiation.Card ) and source.min_persuasion ~= nil then
                                acc:ModifyValue( TARGET_MOD.RANDOM1, self )
                            end
                        end,

                        [ NEGOTIATION_EVENT.POST_RESOLVE ] = function( self, minigame, card )
                            if card.negotiator ~= self.negotiator and card.min_persuasion ~= nil then
                                self.active = false
                            end
                        end
                    },
                },

                WRITHING_APPENDAGES =
                {
                    name = "Writhing Appendages",
                    desc = "At the end of {1}'s turn, remove 1 <b>Writhing Appendages</>.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOwnerName() )
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,

                    max_resolve = 5,
                    target_enemy = TARGET_ANY_RESOLVE,

                    min_persuasion = 2,
                    max_persuasion = 3,

                    OnApply = function( self )
                        self:PrepareTurn()
                        self:SetStacks(3)
                    end,

                    event_handlers =
                    {
                        [ NEGOTIATION_EVENT.END_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator then
                                self:ApplyPersuasion()
                                negotiator:RemoveModifier( self, 1 )
                            end
                        end,
                    },
                },

                BOG_BOSS_BOIL =
                {
                    name = "Bog Boil",
                    desc = "If destroyed, deal {1} damage to {2}'s core argument. At the end of {2}'s turn, remove 1 <b>Bog Boil</>.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self.damage_amt, self:GetOwnerName() )
                    end,
                    icon = "negotiation/modifiers/bog_boil.tex",

                    OnApply = function( self )
                        self:SetStacks(3)
                    end,

                    modifier_type = MODIFIER_TYPE.BOUNTY,

                    max_resolve = 2,

                    damage_amt = 6,

                    OnBounty = function( self )
                        self.engine:ApplyPersuasion( self, self.negotiator, self.damage_amt, self.damage_amt )
                    end,

                    event_handlers =
                    {
                        [ NEGOTIATION_EVENT.END_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator then
                                negotiator:RemoveModifier( self, 1 )
                            end
                        end,
                    },
                },

                FUME_BUBBLE = 
                {
                    name = "Fume Bubble",
                    desc = "If destroyed, deals 4 damage to {1}'s core argument. At the end of {1}'s turn, remove 1 <b>Fume Bubble</>.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOpponentName() )
                    end,

                    OnApply = function( self )
                        self:SetStacks(3)
                    end,

                    modifier_type = MODIFIER_TYPE.BOUNTY,

                    max_resolve = 1,

                    OnBounty = function( self )
                        self.engine:ApplyPersuasion( self, self.anti_negotiator, self.damage_amt, self.damage_amt )
                    end,
                    event_handlers =
                    {
                        [ NEGOTIATION_EVENT.END_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator then
                                negotiator:RemoveModifier( self, 1 )
                            end
                        end,
                    },
                },

                BLISTER = 
                {
                    name = "Blister",
                    desc = "At the start of {1}’s turn, restore {2} resolve to its core argument and remove 1 <b>Blister</>.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOwnerName(), self.restore_amt )
                    end,

                    OnApply = function( self )
                        self:SetStacks(3)
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,
                    max_resolve = 3,

                    restore_amt = 2,

                    event_handlers =
                    {
                        [ NEGOTIATION_EVENT.END_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator and self.negotiator:FindCoreArgument() then
                                self.negotiator:FindCoreArgument():RestoreResolve(self.restore_amt, self)
                                negotiator:RemoveModifier( self, 1 )
                            end
                        end,
                    },
                },
            },
            behaviour = 
            {
                cycle_options = 
                {
                    -- Tentacle
                    function( self, turns )
                        if self.cycle_turns == 0 then
                            self.tentacle:ChooseCards(2)
                            self.boil:ChooseCards(1)
                        end
                        self:ChooseGrowingNumbers( 2, 1 )
                    end,

                    -- Blinding
                    function( self, turns )
                        if self.cycle_turns == 0 then
                            self:ChooseCard(self.spore)
                            self.fume:ChooseCards(2)
                        end
                        self:ChooseGrowingNumbers( 1, 1 )
                        self:ChooseComposure( 1, 2, 3 )
                    end,

                    -- Rash
                    function( self, turns )
                        if self.cycle_turns == 0 then
                            self.rash:ChooseCards(3)
                            self.blister:ChooseCards(2)
                        end
                        self:ChooseComposure( 2, 2, 3 )
                    end,
                },

                OnInit = function( self, difficulty )
                    if CheckBits(self.engine:GetFlags(), NEGOTIATION_FLAGS.WORDSMITH) then
                        self.tentacle = self:MakePicker()
                            :AddID("grout_bog_tentacle", 1)
                            :AddID("grout_bog_tentacle", 1)
                        self.boil = self:MakePicker()
                            :AddID("grout_bog_boil", 1)

                        self.spore = self:AddArgument("SPORE_CLOUD")
                        self.fume = self:MakePicker()
                            :AddID("grout_bog_fume", 1)
                            :AddID("grout_bog_fume", 1)

                        self.rash = self:MakePicker()
                            :AddID("grout_bog_rash", 1)
                            :AddID("grout_bog_rash", 1)
                            :AddID("grout_bog_rash", 1)
                        self.blister = self:MakePicker()
                            :AddID("grout_bog_blister", 1)
                            :AddID("grout_bog_blister", 1)


                        self.negotiator:AddModifier("DARK_INTELLECT")
                        self.cycle_order = shallowcopy(self.cycle_options)
                        table.shuffle(self.cycle_order)
                        self.current_cycle = table.remove(self.cycle_order)
                        self.cycle_turns = 0
                        self:SetPattern(self.BossCycle)
                    end
                end,

                BossCycle = function( self, turns )
                    self.current_cycle( self, turns)
                    self.cycle_turns = self.cycle_turns + 1
                    if self.cycle_turns >= 3 then
                        self.cycle_turns = 0
                        if #self.cycle_order <= 0 then
                            self.cycle_order = shallowcopy(self.cycle_options)
                            table.shuffle(self.cycle_order)
                        end
                        self.current_cycle = table.remove(self.cycle_order)
                    end

                end,
            }
        },

        fight_data =
        {
            MAX_HEALTH = 300,
            battle_scale = 1.2,

            status_widget_dx = 2.5,
            status_widget_head_dx = 4,
            status_widget_head_dy = 5,
            -- death_fade_delay = 2.5,
            shadow_scale = 0.0,
            death_fade_time = -1,
            formation = FIGHTER_FORMATION.FRONT_X,

            stationary = true,
            approach_distance = 6,
            no_despawn = true,

            ranged_riposte = true,

            OnJoinBattle = function( fighter, anim_fighter, fight_screen )
                local x, z = anim_fighter:GetHomePosition()
                anim_fighter.entity.cmp.AnimController:SetXFlip(fighter:GetTeamID() == TEAM.RED)
                anim_fighter.entity:SetLocalPosition( x, 0, z )
                anim_fighter:CoroDelay(.6)
                fight_screen:ScreenShake( CAMERA_SHAKE_PARAMS )
                anim_fighter:CoroDelay(1.2)
                anim_fighter:PlayAnim( "emerge" )
                anim_fighter:WaitAnim()
            end,

            conditions =
            {
                burr_boss_enrage =
                {
                    name = "Bog Rage",
                    desc = "{1} deals {2} bonus damage. Reduce the bonus damage by {3} every time {1} is hit.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName(), self.current_bonus or 0, self.decay or 1)
                    end,

                    ctype = CTYPE.BUFF,

                    decay = 4,
                    current_bonus = 0,

                    start_bonus = { 10, 12, 13, 14 },

                    event_handlers =
                    {
                        [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                            if card.owner == self.owner and self.current_bonus > 0 then
                                dmgt:AddDamage(self.current_bonus, self.current_bonus, self)
                            end
                        end,

                        [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                            if hit.target == self.owner and attack and attack.card and attack.card:IsAttackCard() then
                                self.current_bonus = math.max( self.current_bonus - self.decay, 0 )
                            end
                        end,

                        [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                            if fighter == self.owner then
                                self.current_bonus = self.start_bonus[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
                            end
                        end,

                        [ BATTLE_EVENT.DEACTIVATED_FIGHTER ] = function ( self, fighter )
                            if fighter:GetTeam() == self.owner:GetTeam() then
                                if self.owner:GetTeam():NumActiveFighters() == 1 then
                                    self.owner:RemoveCondition(self.id)
                                end
                            end
                        end,
                    },
                },

                burr_boss_eye_tracker =
                {
                    hidden = true,

                    OnApply = function( self, battle )
                        self.owner:BroadcastEvent( BATTLE_EVENT.FIGHTER_REMAP_ANIMS, nil )
                        for i,ally in self.owner:GetTeam():Fighters() do
                            if ally.agent:GetContentID() == "GROUT_EYE" then
                                ally:AddCondition("burr_eye_stalk_vision")
                            end
                        end
                    end,

                    OnUnapply = function( self, battle )
                        self.owner:BroadcastEvent( BATTLE_EVENT.FIGHTER_REMAP_ANIMS, nil )
                    end,

                    EyeClosed = function( self, eye )
                        if self.open_eyes then
                            table.remove(self.open_eyes, self.open_eyes[eye])
                            if #self.open_eyes <= 0 and self.owner:HasCondition("burr_boss_eye") then
                                self.owner:PlayAnim("taunt")
                                self.battle:Delay(1)
                                self.owner:BroadcastEvent( BATTLE_EVENT.FIGHTER_REMAP_ANIMS, nil )
                                self.owner:RemoveCondition("burr_boss_eye")
                            end
                        end
                    end,

                    EyeOpened = function( self, eye )
                        self.open_eyes = self.open_eyes or {}
                        if #self.open_eyes == 0 then
                            self.owner:BroadcastEvent( BATTLE_EVENT.FIGHTER_REMAP_ANIMS, self.owner.fight_data.eye_closed_anim_mapping )
                            self.owner:AddCondition("burr_boss_eye")
                        end
                        table.insert(self.open_eyes, eye)
                    end,

                    event_priorities =
                    {
                        [ BATTLE_EVENT.CALC_DAMAGE ] = 999,
                    },

                    event_handlers =
                    {
                        [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                            if target == self.owner and self.open_eyes and #self.open_eyes > 0 then
                                dmgt:ModifyDamage(math.floor(dmgt.min_damage * .6), math.floor(dmgt.max_damage * .6), self)
                            end
                        end,
                    },
                },

                burr_boss_eye = 
                {
                    name = "Bog Eyes",
                    desc = "{1}'s eye is closed, reducing damage taken by 40%.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName() )
                    end,

                    ctype = CTYPE.BUFF,
                },

                burr_eye_stalk_vision = 
                {
                    name = "Bog Vision",
                    desc = "{1} is using this unit to see. Whenever {2} is hit, it closes its eye and is stunned until the start of its turn.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self.bog_monster and self.bog_monster:GetName(), self:GetOwnerName() )
                    end,

                    OnApply = function( self, battle )
                        for i,fighter in self.owner:GetTeam():Fighters() do
                            if fighter:HasCondition("burr_boss_eye_tracker") then
                                self.bog_monster = fighter
                                self.boss_eye_con = fighter:GetCondition("burr_boss_eye_tracker")
                                self.boss_eye_con:EyeOpened(self)
                                self.active = true
                                break
                            end
                        end
                    end,

                    OnUnapply = function( self, battle )
                        if self.active and self.boss_eye_con then
                            self.boss_eye_con:EyeClosed(self)
                        end
                    end,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                            if hit.target == self.owner and attack and attack.card and attack.card:IsAttackCard() and (not self.owner:HasCondition("STUN") or self.active) then
                                self.active = false
                                if self.boss_eye_con then
                                    self.boss_eye_con:EyeClosed(self)
                                end
                                self.owner:AddCondition("STUN")
                            end
                        end,

                        [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
                            if fighter == self.owner and (not self.owner:HasCondition("STUN") or self.active) then
                                self.active = false
                                if self.boss_eye_con then
                                    self.boss_eye_con:EyeClosed(self)
                                end
                                self.owner:AddCondition("STUN")
                            end
                        end,

                        [ BATTLE_EVENT.CONDITION_REMOVED ] = function( self, fighter, condition, stacks, source )
                            if fighter == self.owner and condition.id == "STUN" then
                                self.active = true
                                -- open eye
                                if self.boss_eye_con then
                                    self.boss_eye_con:EyeOpened(self)
                                end
                            end
                        end,
                    },
                },

                eyes_of_the_bog =
                {
                    name = "Empowered Eyes of the Bog",
                    desc = "The Eyestalks have 3 bonus {POWER} and only target you.",
                    hide_gained = true,

                    OnApply = function( self, battle )
                        for i,ally in self.owner:GetTeam():Fighters() do
                            if ally.agent:GetContentID() == "GROUT_EYE" then
                                ally:AddCondition("POWER", 3, self)
                                ally:AddCondition("grout_eye_focus", 1, self)
                            end
                        end
                    end,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                            if fighter:GetTeam() == self.owner:GetTeam() and fighter.agent:GetContentID() == "GROUT_EYE" then
                                fighter:AddCondition("POWER", 3, self)
                                fighter:AddCondition("grout_eye_focus", 1, self)
                            end
                        end
                    },
                },

                eyes_of_the_bog_debuff =
                {
                    name = "Weakened Eyes of the Bog",
                    desc = "Whenever an Eyestalk dies, gain 1 {IMPAIR}.",
                    hide_gained = true,

                    ctype = CTYPE.DEBUFF,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.STATUS_CHANGED ] = function( self, fighter, status )
                            if fighter.agent:GetContentID() == "GROUT_EYE" and status == FIGHT_STATUS.DEAD then
                                self.owner:AddCondition("IMPAIR", 1, self)
                            end
                        end
                    },
                },

                grout_eye_focus =
                {
                    hidden = true,
                },

                brain_of_the_bog = 
                {
                    name = "Empowered Brain of the Bog",
                    desc = "Whenever a Grout Loot Cluster returns to the Bog, this unit gains 2 {POWER} and heals 20 health.",

                    event_handlers =
                    {
                        [ BATTLE_EVENT.STATUS_CHANGED ] = function( self, fighter, status )
                            if fighter.agent:GetContentID() == "GROUT_LOOT_CLUSTER_BOSS" and status == FIGHT_STATUS.FLED then
                                self.owner:AddCondition("POWER", 2, self)
                                self.owner:HealHealth(30, self)
                            end
                        end
                    },
                },

                brain_of_the_bog_debuff = 
                {
                    name = "Weakened Brain of the Bog",
                    desc = "Whenever a Grout Loot Cluster is killed, this unit gains 2 {WOUND}.",
                    ctype = CTYPE.DEBUFF,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.STATUS_CHANGED ] = function( self, fighter, status )
                            if fighter.agent:GetContentID() == "GROUT_LOOT_CLUSTER_BOSS" and status == FIGHT_STATUS.DEAD then
                                self.owner:AddCondition("WOUND", 2, self)
                            end
                        end
                    },
                },

                heart_of_the_bog =
                {
                    name = "Empowered Heart of the Bog",
                    desc = "Spark mines and Grout Knuckles have {1} additional health.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self.health_bonus)
                    end,
                    health_bonus = 10,
                    event_handlers =
                    {
                        [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                            if fighter:GetTeam() == self.owner:GetTeam() and (fighter.agent:GetContentID() == "GROUT_SPARK_MINE_BOSS" or fighter.agent:GetContentID() == "GROUT_KNUCKLE_BOSS") then
                                fighter:DeltaMaxHealth(self.health_bonus)
                                fighter:HealHealth(self.health_bonus, self)
                            end
                        end
                    },
                },

                heart_of_the_bog_debuff =
                {
                    name = "Weakened Heart of the Bog",
                    desc = "Spark mines and Grout Knuckles lose {1} health when they emerge.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self.health_loss)
                    end,
                    ctype = CTYPE.DEBUFF,
                    health_loss = 6,
                    event_handlers =
                    {
                        [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                            if fighter:GetTeam() == self.owner:GetTeam() and (fighter.agent:GetContentID() == "GROUT_SPARK_MINE_BOSS" or fighter.agent:GetContentID() == "GROUT_KNUCKLE_BOSS") then
                                fighter:DeltaHealth(-self.health_loss)
                            end
                        end
                    },
                },

                snackrifice =
                {
                    name = "Snackrifice",
                    desc = "{1} has devoured the Bogger Priest, gaining {2} health.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName(), self.health_bonus)
                    end,

                    health_bonus = 0,
                    timer = 3,
                    OnApply = function( self, battle )
                        self.owner:DeltaMaxHealth(self.stacks)
                        self.owner:HealHealth(self.stacks)
                        self.health_bonus = self.stacks
                        self.stacks = 1
                    end,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                            self.turns = (self.turns or 0) + 1
                            if self.turns >= self.timer then
                                self.owner:RemoveCondition(self.id)
                            end
                        end
                    },
                },
            },

            attacks =
            {
                burr_boss_projectile = table.extend(NPC_RANGED)
                {
                    name = "Spur Spit",
                    anim = "projectile",

                    base_damage = { 10, 12, 13, 14 },
                },

                burr_boss_projectile_knuckle = table.extend(NPC_RANGED)
                {
                    name = "Spur Spit",
                    anim = "projectile",

                    base_damage = { 12, 14, 15, 16 },
                },

                burr_boss_taunt = table.extend(NPC_BUFF)
                {
                    name = "Roar",
                    anim = "taunt",

                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,
                    target_mod = TARGET_MOD.TEAM,

                    defend_amt = { 12, 14, 16, 18},
                    power_amt = { 2, 2, 3, 3},

                    OnPostResolve = function( self, battle, attack )
                        self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                        for i,ally in self.owner:GetTeam():Fighters() do
                            if ally ~= self.owner then
                                ally:AddCondition("POWER", self.power_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                            end
                        end
                    end
                },

                burr_boss_taunt_heal = table.extend(NPC_BUFF)
                {
                    name = "Roar",
                    anim = "taunt",

                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF | CARD_FLAGS.HEAL,
                    target_type = TARGET_TYPE.SELF,
                    target_mod = TARGET_MOD.TEAM,

                    defend_amt = {12, 14, 16, 18},
                    heal_amt = { 6, 8, 9, 10},

                    OnPostResolve = function( self, battle, attack )
                        self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                        for i,ally in self.owner:GetTeam():Fighters() do
                            if ally ~= self.owner then
                                ally:HealHealth(self.heal_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                            end
                        end
                    end
                },

                burr_boss_card_steal = table.extend(NPC_RANGED)
                {
                    name = "Pilfering Lash",
                    anim = "tentacle",

                    flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF | CARD_FLAGS.PIERCING,
                    
                    CanPlayCard = function( self, battle, target )
                        return (target == nil or target:IsPlayer())
                    end,

                    burrs_to_spawn = { 1, 2, 3, 3 },

                    base_damage = { 6, 8, 9, 10 },

                    OnPostResolve = function( self, battle, attack )
                        -- ClearExistingGrout(self.owner)
                        ClearStageConditions(self.owner)
                        local spawned_burrs = {}
                        -- Only spawn enough burrs that the player has at least 6 cards left
                        local burr_count = math.min(math.floor((#battle:GetCardsOwnedBy(battle:GetPlayerFighter()) - 6) / 3), self.burrs_to_spawn[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
                        for i=1, burr_count do
                            local fighter = Fighter.CreateFromAgent( Agent("GROUT_LOOT_CLUSTER_BOSS"), 1 )
                            table.insert(spawned_burrs, fighter)
                            self.owner:GetTeam():AddFighter( fighter )
                        end
                        self.owner:GetTeam():ActivateNewFighters()
                        PlaceAndEmerge(self.owner, battle)
                        for i,ally in ipairs(spawned_burrs) do
                            local con = ally:AddCondition("STOLEN_CARD_CLUSTER", 1, self)
                            if con then 
                                con:StoreCards( attack.target, self:FindCardsToSteal(attack.target, 3, battle), 3) 
                            end
                        end
                        self.owner:AddCondition("burr_boss_enrage", 1, self)
                    end,

                    FindCardsToSteal = function( self, target, num_cards, battle )
                        local chosen_cards = {}
                        local cards = battle:GetCardsOwnedBy( target )
                        if #cards >= 5 + num_cards then
                            table.shuffle(cards)

                            local RARITY_SORT = 
                            {
                                [CARD_RARITY.BASIC] = 0,
                                [CARD_RARITY.COMMON] = 1,
                                [CARD_RARITY.UNCOMMON] = 2,
                                [CARD_RARITY.RARE] = 2,
                                [CARD_RARITY.UNIQUE] = 2,
                                [CARD_RARITY.BOSS] = 2,
                            }

                            table.stable_sort(cards, function(a, b) return RARITY_SORT[a.rarity] > RARITY_SORT[b.rarity] end)

                            for i, card in ipairs(cards) do
                                if not CheckBits( card.flags, CARD_FLAGS.STATUS ) then
                                    table.insert( chosen_cards, card)
                                    if #chosen_cards >= num_cards then
                                        break
                                    end
                                end
                            end
                        end
                        
                        return chosen_cards
                    end,
                },

                burr_boss_rage_attack = table.extend(NPC_RANGED)
                {
                    name = "Lashing Strike",
                    anim = "projectile",

                    target_mod = TARGET_MOD.TEAM,

                    base_damage = { 6, 8, 9, 10 },
                },

                summon_eye_stalks = table.extend(NPC_BUFF)
                {
                    name = "Extended Sight",
                    anim = "taunt",

                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    eyes_to_spawn = { 1, 2, 3, 3},
                    defend_amt = { 10, 12, 14, 16 },

                    OnPostResolve = function( self, battle, attack )
                        self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                        -- ClearExistingGrout(self.owner)
                        ClearStageConditions(self.owner)
                        local to_spawn = self.eyes_to_spawn[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] - (self.owner:GetTeam():NumActiveFighters() - 1)
                        for i=1, to_spawn do
                            local fighter = Fighter.CreateFromAgent( Agent("GROUT_EYE"), 1 )
                            self.owner:GetTeam():AddFighter( fighter )
                        end
                        self.owner:GetTeam():ActivateNewFighters()
                        PlaceAndEmerge(self.owner, battle)
                        self.owner:AddCondition("burr_boss_eye_tracker", 1, self)
                    end,
                },

                summon_knuckles = table.extend(NPC_BUFF)
                {
                    name = "Bog Defenses",
                    anim = "taunt",

                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    knuckles_to_spawn = { 2, 2, 2, 2 },
                    defend_amt = { 10, 12, 14, 16 },

                    OnPostResolve = function( self, battle, attack )
                        self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                        -- ClearExistingGrout(self.owner)
                        ClearStageConditions(self.owner)
                        self.owner:GetTeam():AddFighter( Fighter.CreateFromAgent( Agent("GROUT_SPARK_MINE_BOSS"), 1 ) )
                        for i=1, self.knuckles_to_spawn[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] do
                            self.owner:GetTeam():AddFighter( Fighter.CreateFromAgent( Agent("GROUT_KNUCKLE_BOSS"), 1 ) )
                        end
                        self.owner:GetTeam():ActivateNewFighters()
                        PlaceAndEmerge(self.owner, battle)

                        local player_team = battle:GetPlayerFighter():GetTeam()
                        if not player_team:IsFull() then
                            player_team:AddFighter( Fighter.CreateFromAgent( Agent("GROUT_SPARK_MINE_BOSS"), 1 ) )
                            player_team:ActivateNewFighters()
                            PlaceAndEmerge(battle:GetPlayerFighter(), battle)
                        end
                    end,
                },
            },

            behaviour =
            {
                CUSTOM_FIGHT_FORMATIONS =
                {
                    [1] = BURR_BOSS_FORMATION,
                    [2] = BURR_BOSS_FORMATION,
                    [3] = BURR_BOSS_FORMATION,
                    [4] = BURR_BOSS_FORMATION,
                    [5] = BURR_BOSS_FORMATION,
                    [6] = BURR_BOSS_FORMATION,
                },

                OnActivate = function( self )
                    self.fighter:GetTeam():SetCustomFormation( self.CUSTOM_FIGHT_FORMATIONS )
                    self.chosen_order = { self.EyeStalkCycle, self.TreasureCycle, self.KnuckleCycle}
                    local has_eyes = false
                    for i,ally in self.fighter:GetTeam():Fighters() do
                        if ally.agent:GetContentID() == "GROUT_EYE" then
                            has_eyes = true
                            self.fighter:AddCondition("burr_boss_eye_tracker")
                            break
                        end
                    end
                    if has_eyes then
                        table.shuffle(self.chosen_order, 2) -- always start with the eyestalk
                    else
                        table.shuffle(self.chosen_order) -- Random!
                    end

                    self.spawn_treasure = self:AddCard("burr_boss_card_steal")
                    self.spawn_eyes = self:AddCard("summon_eye_stalks")
                    self.spawn_knuckles = self:AddCard("summon_knuckles")

                    self.eye_stage_moves = self:MakePicker()
                        :AddID("burr_boss_projectile", 2)
                        :AddID("burr_boss_taunt", 1)

                    self.treasure_stage_moves = self:MakePicker()
                        :AddID("burr_boss_rage_attack", 3)
                        :AddID("burr_boss_taunt_heal", 1)

                    self.knuckle_stage_moves = self:MakePicker()
                        :AddID("burr_boss_projectile_knuckle", 2)
                        :AddID("burr_boss_taunt", 1)

                    self.stage_position = 1
                    self:SetPattern(self.chosen_order[self.stage_position])
                end,

                CheckForStageSwitch = function( self )
                    if self.battle:GetTurns() - (self.last_turn or -1) > 0 then
                        self.stage_turns = (self.stage_turns or 0) + 1
                        self.last_turn = self.battle:GetTurns()
                    end
                    if self.fighter:GetTeam():NumActiveFighters() == 1 and self.stage_turns > 1 then
                        -- Always leave the player with at least 6 cards after stealing (minimum steal is 6)
                        if #self.battle:GetCardsOwnedBy(self.battle:GetPlayerFighter()) < 12 then
                            table.arrayremove(self.chosen_order, self.TreasureCycle)
                        end
                        self.stage_turns = 1
                        self.stage_position = self.stage_position + 1
                        if self.stage_position > #self.chosen_order then
                            table.shuffle(self.chosen_order)
                            self.stage_position = 1
                        end
                        self:SetPattern(self.chosen_order[self.stage_position ])
                        return true
                    else
                        return false
                    end
                end,

                EyeStalkCycle = function( self )
                    if not self:CheckForStageSwitch() then
                        if self.stage_turns <= 1 and not self.fighter:HasCondition("burr_boss_eye_tracker") then
                            self:ChooseCard(self.spawn_eyes)
                        else
                            self.eye_stage_moves:ChooseCard()
                        end
                    else
                        self:RunBehaviour( self.battle )
                    end
                end,

                TreasureCycle = function( self )
                    if not self:CheckForStageSwitch() then
                        if self.stage_turns <= 1 then
                            self:ChooseCard(self.spawn_treasure)
                        else
                            self.treasure_stage_moves:ChooseCard()
                        end
                    else
                        self:RunBehaviour( self.battle )
                    end
                end,

                KnuckleCycle = function( self )
                    if not self:CheckForStageSwitch() then
                        if self.stage_turns <= 1 then
                            self:ChooseCard(self.spawn_knuckles)
                        else
                            self.knuckle_stage_moves:ChooseCard()
                        end
                    else
                        self:RunBehaviour( self.battle )
                    end
                end
            },

            anim_mapping =
            {
                idle = "idle2",
                hit_mid_pst_idle = "hit_mid_pst_idle2",
                defend_pst = "defend_pst2",
                stunned_pst = "stunned_pst2",
            },

            eye_closed_anim_mapping =
            {
            },
        },
    })
)

Content.AddCharacterDef
(
    CharacterDef("GROUT_LOOT_CLUSTER_BOSS",
    {
        base_def = "MONSTER",

        title = "Grout Loot Cluster",
        build = "grout_loot",
        combat_strength = 1,

        fight_data =
        {
            MAX_HEALTH = 20,

            death_fade_delay = 0.8,
            shadow_scale = 0.0, -- Doesn't add a shadow prefab if 0

            status_widget_head_dx = 1.0,
            status_widget_head_dy = 2.5,
            stationary = true,
            
            OnJoinBattle = GroutJoin,
            is_boss_burr = true,

            anim_mapping =
            {
                stunned = "idle",
            },

            conditions = 
            {
                STOLEN_CARD_CLUSTER =
                {
                    name = "Stolen Treasure",

                    desc = "Grants when killed:",
                    desc_fn = function(self, fmt_str)
                        local ret = { fmt_str }
                        local ids = {}

                        for k,v in ipairs(self.items) do
                            table.insert(ret, "{card.{" .. k .. "}}")
                            table.insert(ids, v.id)
                        end

                        local str = loc.format( table.concat(ret, " "), table.unpack(ids) )
                        return str
                    end,

                    random_items = {"bog_blaster", "lumin_grenade", "spark_grenade", "fuel_canister", "bog_brew"},

                    StoreCards = function( self, fighter, cards, num_cards )
                        self.items = self.items or {}
                        if cards and #cards > 0 then 
                            for i,card in ipairs(cards) do
                                card:RemoveCard()
                                table.insert(self.items, card)
                            end
                        else
                            for i=1,num_cards do
                                local item = Battle.Card(self.random_items[math.random(#self.random_items)], fighter)
                                table.insert(self.items, item)
                            end
                        end
                        self.battle:BroadcastEvent( BATTLE_EVENT.CARDS_STOLEN, self.items, self.owner ) -- Still needs presentation
                    end,
                    event_handlers =
                    {
                        [ BATTLE_EVENT.STATUS_CHANGED ] = function( self, fighter, status )
                            if fighter == self.owner and status == FIGHT_STATUS.DEAD and self.battle then
                                self.battle:BroadcastEvent( BATTLE_EVENT.DELAY, 0.5 )
                                if self.items then
                                    self.battle:DealCards(self.items, self.battle:GetHandDeck())
                                end
                            end
                        end,
                    }


                },
            },

            attacks =
            {

            },

            behaviour =
            {
                OnActivate = function( self )
                    
                end,
            },
        },
    })
)

Content.AddCharacterDef
(
    CharacterDef("GROUT_EYE",
    {
        base_def = "MONSTER",
        
        title = "Grout Eyestalk",
        build = "grout_eyes",
        combat_strength = 1,

        fight_data =
        {
            MAX_HEALTH = 22,

            status_widget_head_dx = 1.0,
            status_widget_head_dy = 2.5,
            shadow_scale = 0.0, -- Doesn't add a shadow prefab if 0
            stationary = true,

            OnJoinBattle = GroutJoin,
            is_boss_burr = true,
            anim_mapping =
            {
                surrender = "death",
            },

            attacks =
            {
                grout_eye_attack = table.extend(NPC_ATTACK)
                {
                    name = "Shoot",
                    anim = "attack1",
                    flags = CARD_FLAGS.RANGED,
                    base_damage = { 7, 9, 10, 11 },
                    CanPlayCard = function ( self, battle, target )
                        if self.owner:HasCondition("grout_eye_focus") then
                            return target == nil or target:IsPlayer()
                        else
                            return true
                        end
                    end,
                },

                grout_eye_heal = table.extend(NPC_BUFF)
                {
                    name = "Intensify",
                    anim = "taunt",

                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF | CARD_FLAGS.HEAL,

                    target_mod = TARGET_MOD.TEAM,
                    target_type = TARGET_TYPE.SELF,

                    healing = { 4, 6, 8, 10 },

                    OnPostResolve = function( self, battle, attack )
                        for i,ally in self.owner:GetTeam():Fighters() do
                            if ally ~= self.owner then
                                ally:HealHealth(self.healing[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
                            end
                        end
                    end
                },

            },

            behaviour =
            {
                OnActivate = function( self )
                    self.moves = self:MakePicker()
                        :AddID( "grout_eye_heal", 1 )
                        :AddID( "grout_eye_attack", 1)

                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    self.moves:ChooseCard()
                end,
            },
        },
    })
)

Content.AddCharacterDef
(
    CharacterDef("GROUT_KNUCKLE_BOSS",
    {
        base_def = "MONSTER",
        
        title = "Grout Knuckle",
        build = "grout_knuckles",
        combat_strength = 1,

        fight_data =
        {
            MAX_HEALTH = 20,

            status_widget_head_dx = 1.0,
            status_widget_head_dy = 2.5,
            stationary = true,

            OnJoinBattle = GroutJoin,
            is_boss_burr = true,

            ranged_riposte = true,
            anim_mapping =
            {
                riposte = "attack1",
            },

            attacks =
            {
                boss_knuckle_puncture = table.extend(NPC_ATTACK)
                {
                    name = "Puncture",
                    anim = "attack1",
                    flags = CARD_FLAGS.RANGED,
                    base_damage = { 7, 9, 11, 13 }
                },

                boss_knuckle_fossilize =
                {
                    name = "Fossilize",
                    anim = "pose",
                    flags = CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,
                    OnPostResolve = function( self, battle, attack )
                        for i, hit in attack:Hits() do
                            hit.target:AddCondition( "DEFEND", 6 )
                        end                       
                    end,
                }
            },

            conditions =
            {
                grout_boss_knuckle_counter = 
                {
                    hidden = true,
                    event_handlers =
                    {
                        [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                            if fighter == self.owner then
                                if not self.owner:HasCondition("RIPOSTE") then
                                    self.owner:AddCondition("RIPOSTE", 8, self)
                                end
                            end
                        end
                    },
                },
            },

            behaviour =
            {
                OnActivate = function( self )
                    self.buffs = self:MakePicker()
                        :AddID( "boss_knuckle_puncture", 1 )
                        :AddID( "boss_knuckle_fossilize", 1 )

                    self:SetPattern( self.Cycle )
                    self.fighter:AddCondition("RIPOSTE", 8)
                    self.fighter:AddCondition("grout_boss_knuckle_counter", 1)
                end,

                Cycle = function( self )
                    self.buffs:ChooseCard()
                end,
            },
        },
    })
)


local def = CharacterDef("BOGGER_BOSS",
{
    unique = true,

    base_def = "NPC_BASE",
    alias = "BOGGER_BOSS",
    species = SPECIES.SHROKE,
    gender = GENDER.MALE,
    renown = 5,
    combat_strength = 3,
    boss = true,
    tags = {"plot_armour", "no_patron"},
    
    name = "Glofriam",
    icon = engine.asset.Texture( "icons/b_bogger_boss.tex"),

    --death_item = "lumin_generator",
    --loved_graft = "heshian_armature",
    --hated_graft = "gremlins",
    theme_music = "event:/music/story/rook/story_rook_glofriam",
    battle_preview_anim = "anim/boss_glofrian_slide.zip",
    battle_preview_offset = { x = 80, y = 20 },
    battle_preview_glow = { colour = 0xF2E669FF, bloom = 0.3, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_glofriam",

    faction_id = "BOGGERS",
    title = "High Priest",
    voice_actor = "civilianHighClassMale01",
    
    combat_anims = {"anim/med_combat_blunderbuss_bogger_boss.zip"},
    anims = {"anim/weapon_blaster_blunderbuss.zip"},
    build = "male_bogger_boss",
    head = "head_male_shroke_09",

    negotiation_data = 
    {


        modifiers = {
            HIGH_PRIEST_BOGGER = 
            {
                name = "Faith in the Bog",
                desc = "Undermine {1.name}'s faith to drop the shield around {1.hisher} core argument, and give yourself specific advantages in the coming fight.\n\nArticles destroyed: {2#listing}",
                
                desc_fn = function(self, fmt_str )
                    local defeated_articles = {}
                    for k,v in ipairs(self.defeated_articles) do
                        table.insert(defeated_articles, v.name)
                    end

                    return loc.format(fmt_str, self.negotiator.agent:LocTable(), defeated_articles)
                end,

                modifier_type = MODIFIER_TYPE.CORE,

                OnInit = function( self )
                    self:SetShieldStatus(true)
                    self.articles_of_faith = {}
                    self.defeated_articles = {}
                    self.cards_to_win = {}
                    self.modifiers_to_remove = {}
                end,

                event_handlers =
                {
                    [ NEGOTIATION_EVENT.MODIFIER_ADDED ] = function ( self, modifier, source )
                        if modifier.negotiator == self.negotiator and modifier == self then
                            
                            local eyes = self.negotiator:AddModifier("article_of_faith_eyes", 1, self)
                            local heart = self.negotiator:AddModifier("article_of_faith_heart", 1, self)
                            local brain = self.negotiator:AddModifier("article_of_faith_brain", 1, self)
                            
                            table.insert(self.articles_of_faith, heart)
                            table.insert(self.articles_of_faith, eyes)
                            table.insert(self.articles_of_faith, brain)
                        end
                    end,
                    
                    [ NEGOTIATION_EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                        if self.engine:GetResult() == nil and table.arrayfind( self.articles_of_faith, modifier) then
                            table.arrayremove( self.articles_of_faith, modifier)
                            table.insert(self.defeated_articles, modifier)
                            
                            if modifier.bog_modifier then
                                table.insert(self.modifiers_to_remove, modifier.bog_modifier)
                            end
                            self:SetShieldStatus(false)
                            self.negotiator:AddModifier("IMPATIENCE", 1, self)
                        end
                    end,
                },
            },   

            article_of_faith_eyes = 
            {
                name = "The Bog Sees All",
                desc = "Spawn a {article_of_faith_eyes_spawned} at the start of {1.name}'s turn if this argument did not lose resolve.\n\nDestroying this argument will reduce {1.name}'s attack strength in battle.",
                
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.negotiator.agent:LocTable(), Content.GetNegotiationModifier("article_of_faith_eyes_spawned").name)
                end,

                OnInit = function(self) 
                    self.eye_image_closed = engine.asset.Texture( "negotiation/modifiers/article_of_faith_eyes_closed.tex" )
                    self.eye_image_open = engine.asset.Texture( "negotiation/modifiers/article_of_faith_eyes.tex" )
                end,

                min_resolve = 20,
                max_resolve = 30,

                bog_modifier = "eyes_of_the_bog_faith",

                modifier_type = MODIFIER_TYPE.BOUNTY,
    
                OnBeginTurn = function(self, minigame)
                    if not self.was_attacked then
                        self:NotifyTriggered()
                        if self.negotiator:GetModifierInstances("article_of_faith_eyes_spawned") < 2 then
                            self.negotiator:CreateModifier( "article_of_faith_eyes_spawned", 1, self )
                        else
                            local mods = {}
                            for _,mod in self.negotiator:Modifiers() do
                                if mod.id == "article_of_faith_eyes_spawned" then
                                    table.insert(mods, mod)
                                end
                            end
                            table.sort(mods, function(a,b) return a.stacks or 0 < b.stacks or 0 end)
                            self.negotiator:DeltaModifier( mods[1], 1, self )
                        end
                    end
                end,

                event_handlers =
                {
                    [ NEGOTIATION_EVENT.BEGIN_PLAYER_TURN ] = function( self )
                        if self.was_attacked then
                            self.icon = self.eye_image_open
                            self.engine:BroadcastEvent( NEGOTIATION_EVENT.UPDATE_MODIFIER_ICON, self, self.eye_image_open)
                            self:NotifyTriggered()
                        end
                        self.was_attacked = false
                    end,

                    [ NEGOTIATION_EVENT.DELTA_RESOLVE ] = function( self, modifier, resolve, max_resolve, delta, source, params )
                        if modifier == self and delta < 0 and resolve > 0 then
                            if not self.was_attacked then
                                self.was_attacked = true
                                self.icon = self.eye_image_closed
                                self.engine:BroadcastEvent( NEGOTIATION_EVENT.UPDATE_MODIFIER_ICON, self, self.eye_image_closed)
                                self:NotifyTriggered()
                            end

                        end
                    end,
                },
                OnBounty = function( self )
                    self.negotiator:CreateModifier( "bogboss_eye_shot_bounty", 1, self )
                end,
            }, 
            article_of_faith_eyes_spawned = 
            {
                name = "Peeper",
                desc = "Each turn, attack an opponent argument for {1}-{2} damage.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.min_persuasion, self.max_persuasion)
                end,

                max_resolve = 3,

                target_enemy = TARGET_ANY_RESOLVE,

                OnSetStacks = function( self )
                    self.min_persuasion =  2 + (self.stacks-1)*2 
                    self.max_persuasion = math.round(self.min_persuasion*1.5)
                end,

                OnBeginTurn = function( self, minigame )
                    self:ApplyPersuasion()
                end,

                min_persuasion = 2,
                max_persuasion = 6,
                --target_self = TARGET_FLAG.BOUNTY,
                --target_mod = TARGET_MOD.RANDOM2,
            }, 

            article_of_faith_brain = 
            {
                name = "The Bog Knows All",
                desc = "When one of {1.name}'s arguments is destroyed, {INCEPT} {2} {article_of_faith_brain_spawned}.\n\nDestroying this argument will increase your card draw when fighting {1.name}.",
                
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.negotiator.agent:LocTable(), self.num_stacks_to_spawn)
                end,
                
                event_handlers = {
                    [ NEGOTIATION_EVENT.MODIFIER_REMOVED ] = function( self, modifier, card )
                        if modifier.negotiator == self.negotiator and card and card.negotiator == self.engine:GetPlayerNegotiator() then

                            if self.anti_negotiator:GetModifierInstances("article_of_faith_brain_spawned") < 2 then
                                self.anti_negotiator:CreateModifier( "article_of_faith_brain_spawned", self.num_stacks_to_spawn, self )
                            else
                                local mods = {}
                                for _,mod in self.anti_negotiator:Modifiers() do
                                    if mod.id == "article_of_faith_brain_spawned" then
                                        table.insert(mods, mod)
                                    end
                                end
                                table.sort(mods, function(a,b) return a.stacks or 0 < b.stacks or 0 end)
                                self.anti_negotiator:DeltaModifier( mods[1], self.num_stacks_to_spawn, self )
                            end
                            self:NotifyTriggered()
                        end
                    end,
                },

                OnBounty = function( self )
                    self.negotiator:CreateModifier( "bogboss_concussion_bounty", 1, self )
                end,

                num_stacks_to_spawn = 6,
                min_resolve = 20,
                max_resolve = 30,
                bog_modifier = "brain_of_the_bog_faith",

                modifier_type = MODIFIER_TYPE.BOUNTY,
            }, 
            
            article_of_faith_brain_spawned = 
            {
                name = "Bog Thought",
                desc = "When destroyed, apply {1} damage to own core argument.\n\nRemove {2} {2*stack|stacks} at the start of every turn.",
                
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.stacks*self.damage_mult, self.reduction)
                end,
                
                min_resolve = 1,
                max_resolve = 3,
                reduction = 2,

                OnBeginTurn = function( self, minigame )
                    self.negotiator:DeltaModifier( self, -self.reduction, self )
                    self:NotifyTriggered()
                end,


                damage_mult = 1,
                modifier_type = MODIFIER_TYPE.BOUNTY,
                
                OnBounty = function( self )
                    self.negotiator:AttackResolve( self.stacks*self.damage_mult, self )
                end,

                --target_self = TARGET_FLAG.BOUNTY,
                --target_mod = TARGET_MOD.RANDOM2,
            },             


            
            article_of_faith_heart = 
            {
                name = "The Bog Protects All",
                desc = "At the start of each turn, apply {2} {COMPOSURE} to a friendly argument.\n\nDestroying this argument will cause {1.name} to have lower health in battle.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.negotiator.agent:LocTable(), self.composure_amt)
                end,
                
                min_resolve = 20,
                max_resolve = 30,
                bog_modifier = "heart_of_the_bog_faith",
                modifier_type = MODIFIER_TYPE.BOUNTY,
                composure_amt = 6,

                --desc = "Add {1} resolve and {COMPOSURE {2}} to a friendly argument.",

                OnBounty = function( self )
                    self.negotiator:CreateModifier( "bogboss_heartburn_bounty", 1, self )
                end,
            

                event_handlers =
                {
                    [ NEGOTIATION_EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                        
                        
                        if negotiator == self.negotiator then
                            local targets = {}
                            for i, modifier in self.negotiator:ModifierSlots() do
                                if modifier:GetResolve() ~= nil and not modifier:GetShieldStatus() and modifier ~= self then
                                    table.insert( targets, modifier )
                                end
                            end
                            local target = #targets > 0 and targets[math.random(#targets)]

                            if target then
                                target:DeltaComposure(self.composure_amt, self )
                                self:NotifyTriggered() 
                            end

                        end
                    end
                }
            }, 

            bogboss_concussion_bounty = MakeCardBounty("Concussion", "concussion"),
            bogboss_heartburn_bounty = MakeCardBounty("Heartburn", "heartburn"),
            bogboss_eye_shot_bounty = MakeCardBounty("Eyeshot", "eye_shot"),
        },



        behaviour = {
            OnInit = function( self, difficulty )
                self:SetPattern( self.Cycle )
                self.negotiator:AddModifier( "HIGH_PRIEST_BOGGER" )
            end,

            Cycle = function( self, turns )
                self:ChooseGrowingNumbers( 1, 1 )
            end,

        }
    },
    --]]
    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.IMMUNE,
        MAX_HEALTH = 150,
        battle_scale = 1.0,
        death_fade_delay = 1,
        shadow_symbol = "hips",
        shadow_scale = 8.5,

        death_fade_delay = 1.85,
        death_fade_time = 0,

        attacks = 
        {
            bogger_boss_shoot = table.extend(NPC_ATTACK)
            {
                name = "Shoot",
                anim = "shoot",

                CanPlayCard = function( self, battle, target )
                    return self.owner:GetHealthPercent() > 0.2
                end,

                flags = CARD_FLAGS.RANGED,

                base_damage = { 6, 8, 10, 12},
            },

            bogger_boss_bail = table.extend(NPC_BUFF)
            {
                name = "Retreat",

                OnPostResolve = function( self, battle, attack )
                    self.owner:Depart()
                end
            },

            bogger_boss_burr = table.extend(NPC_BUFF)
            {   
                name = "Invigorate Burrs",
                anim = "summon",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                CanPlayCard = function( self, battle, target )
                    local player_team = battle:GetPlayerFighter():GetTeam()
                    return player_team:GetMaxFighters() - player_team:NumActiveFighters() >= 1 and self.owner:GetHealthPercent() > 0.2
                end,

                OnPostResolve = function( self, battle, attack )
                    local player_team = battle:GetPlayerFighter():GetTeam()
                    for i=1, math.min(2, player_team:GetMaxFighters() - player_team:NumActiveFighters()) do
                        player_team:AddFighter( Fighter.CreateFromAgent( Agent("GROUT_BOG_CYST"), 3 ) )
                    end
                    player_team:ActivateNewFighters()
                end,
            },

            bogger_boss_rally_burrs = table.extend(NPC_BUFF)
            {
                name = "Rally Burrs",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                CanPlayCard = function( self, battle, target )
                    return self.owner:GetHealthPercent() > 0.2
                end,

                OnPostResolve = function( self, battle, attack )
                    for i=1, math.min(2, self.owner:GetTeam():GetMaxFighters() - self.owner:GetTeam():NumActiveFighters()) do
                        self.owner:GetTeam():AddFighter( Fighter.CreateFromAgent( Agent("GROUT_EYE"), 1 ) )
                    end
                    self.owner:GetTeam():ActivateNewFighters()
                    PlaceAndEmerge(self.owner, battle)
                end,
            },

            bogger_boss_summon_boss = table.extend(NPC_BUFF)
            {
                name = "Bog",
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,
                
                OnPostResolve = function( self, battle, attack )
                    local max_fighters = self.owner:GetTeam():GetMaxFighters()
                    self.owner:GetTeam():SetMaxFighters( max_fighters + 1 )

                    local boss = self.owner:GetTeam():AddFighter( Fighter.CreateFromAgent( Agent("BOG_BURR_BOSS"), 1 ) )
                    self.owner:GetTeam():ActivateNewFighters()
                    local snack_count = math.round(self.owner:GetHealth() * 0.5)
                    local cons_to_add = {}
                    for k,con in pairs(self.owner:GetConditions()) do
                        if con.boss_condition then
                            table.insert(cons_to_add, con.boss_condition)
                        end
                    end
                    self.owner:Kill()
                    -- battle:BroadcastEvent( BATTLE_EVENT.DELAY, .65 )
                    boss:AddCondition("snackrifice", snack_count, self)
                    for i,con_id in ipairs(cons_to_add) do
                        boss:AddCondition(con_id, 1, self)
                    end

                    self.owner:GetTeam():SetMaxFighters( max_fighters )
                end
            },
        },

        conditions = 
        {
            bogger_boss_health_tracker =
            {
                hidden = true,

                event_handlers =
                {
                    [ BATTLE_EVENT.DELTA_STAT ] = function( self, fighter, stat, delta, value, mitigated  )
                        if fighter == self.owner and stat == COMBAT_STAT.HEALTH then
                            if self.owner:GetHealthPercent() < 0.2 then
                                self.owner:AddCondition("bogger_boss_invincible", 1, self)
                                self.owner:RemoveCondition("bogger_boss_health_tracker")
                            end
                        end
                    end,
                },
            },

            bogger_boss_invincible =
            {
                name = "Power of the Bog",
                desc = "The power of the Bog is keeping this fighter alive. Cannot be killed.",
            },

            eyes_of_the_bog_faith =
            {
                name = "Faith in the Bog Eyes",
                desc = "{1} has {2} bonus {POWER} while there are Eyestalks alive.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName(), self.bonus_power)
                end,
                
                icon = "battle/conditions/eyes_of_the_bog.tex",
                boss_condition = "eyes_of_the_bog",

                bonus_power = 3,
                event_handlers =
                {
                    [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                        if fighter:GetTeam() == self.owner:GetTeam() and fighter.agent:GetContentID() == "GROUT_EYE" and not self.active then
                            self.owner:AddCondition("POWER", self.bonus_power, self)
                            self.active = true
                        end
                    end,

                    [ BATTLE_EVENT.DEACTIVATED_FIGHTER ] = function ( self, fighter )
                        if self.active and fighter:GetTeam() == self.owner:GetTeam() and fighter.agent:GetContentID() == "GROUT_EYE" then
                            local no_eyes = true
                            for i,ally in self.owner:GetTeam():Fighters() do
                                if ally.agent:GetContentID() == "GROUT_EYE" then
                                    no_eyes = false
                                    break
                                end
                            end
                            if no_eyes then
                                self.active = false
                                self.owner:RemoveCondition("POWER", self.bonus_power, self)
                            end
                        end
                    end
                },
            },

            eyes_of_the_bog_doubt =
            {
                name = "Doubt in the Bog Eyes",
                desc = "Eyestalks gain {1} {IMPAIR} when they emerge.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self.impair_amt)
                end,

                icon = "battle/conditions/eyes_of_the_bog_debuff.tex",
                boss_condition = "eyes_of_the_bog_debuff",
                ctype = CTYPE.DEBUFF,

                impair_amt = 2,
                event_handlers =
                {
                    [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                        if fighter:GetTeam() == self.owner:GetTeam() and fighter.agent:GetContentID() == "GROUT_EYE" then
                            fighter:AddCondition("IMPAIR", self.impair_amt, self)
                        end
                    end,
                },
            },

            brain_of_the_bog_faith =
            {
                name = "Faith in the Bog Brain",
                desc = "The player draws 1 less card per turn.",
                
                icon = "battle/conditions/brain_of_the_bog.tex",
                boss_condition = "brain_of_the_bog",

                event_handlers =
                {
                    [ BATTLE_EVENT.CALC_TURN_DRAW ] = function( self, acc )
                        acc:AddValue( -1, self )
                    end,
                },
            },

            brain_of_the_bog_doubt =
            {
                name = "Doubt in the Bog Brain",
                desc = "The player draws 1 additional card per turn.",
                
                icon = "battle/conditions/brain_of_the_bog_debuff.tex",
                boss_condition = "brain_of_the_bog_debuff",
                ctype = CTYPE.DEBUFF,

                event_handlers =
                {
                    [ BATTLE_EVENT.CALC_TURN_DRAW ] = function( self, acc )
                        acc:AddValue( 1, self )
                    end,
                },
            },

            heart_of_the_bog_faith =
            {
                name = "Faith in the Bog Heart",
                desc = "{1} has {2} bonus Health.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName(), self.health_bonus)
                end,

                icon = "battle/conditions/heart_of_the_bog.tex",
                boss_condition = "heart_of_the_bog",

                OnApply = function( self, battle )
                    self.owner:DeltaMaxHealth(self.health_bonus)
                    self.owner:HealHealth(self.health_bonus)
                end,

                health_bonus = 20,
            },

            heart_of_the_bog_doubt =
            {
                name = "Doubt in the Bog Heart",
                desc = "{1} has {2} less Health.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName(), self.health_loss)
                end,

                icon = "battle/conditions/heart_of_the_bog_debuff.tex",
                boss_condition = "heart_of_the_bog_debuff",
                ctype = CTYPE.DEBUFF,

                OnApply = function( self, battle )
                    self.owner:DeltaHealth(-self.health_loss, self.owner)
                end,

                health_loss = 20,
            },
        },

        behaviour =
        {
            bog_buffs = {"eyes_of_the_bog_faith", "brain_of_the_bog_faith", "heart_of_the_bog_faith"},
            bog_debuffs = {"eyes_of_the_bog_doubt", "brain_of_the_bog_doubt", "heart_of_the_bog_doubt"},
            CUSTOM_FIGHT_FORMATIONS =
            {
                [1] = BOGGER_BOSS_FORMATION,
                [2] = BOGGER_BOSS_FORMATION,
                [3] = BOGGER_BOSS_FORMATION,
                [4] = BOGGER_BOSS_FORMATION,
                [5] = BOGGER_BOSS_FORMATION,
                [6] = BOGGER_BOSS_FORMATION,
            }, 

            AddBossConditions = function( self, fighter )
                for i,con_id in ipairs(self.bog_buffs) do
                    if not (self.suppress_conditions and self.suppress_conditions[con_id]) then
                        self.fighter:AddCondition(con_id, 1, self)
                    else
                        self.fighter:AddCondition(self.bog_debuffs[i], 1, self)
                    end
                end
            end,                  

            OnActivate = function( self, fighter )
                self.fighter.stat_bounds[ COMBAT_STAT.HEALTH ].min = 1 -- cannot be killed
                self.shoot = self:AddCard("bogger_boss_shoot")
                self.bail = self:AddCard("bogger_boss_bail")
                self.summon = self:AddCard("bogger_boss_summon_boss")
                self.rally = self:AddCard("bogger_boss_rally_burrs")
                self.incept_burs = self:AddCard("bogger_boss_burr")

                self.attacks = self:MakePicker()
                    :AddID("bogger_boss_shoot", 1)
                
                self.fighter:AddCondition("bogger_boss_health_tracker")

                if CheckBits(self.engine:GetFlags(), BATTLE_FLAGS.BOSS_FIGHT) then
                    self.fighter:GetTeam():SetCustomFormation( self.CUSTOM_FIGHT_FORMATIONS )
                    self:SetPattern(self.Cycle)
                else
                    self:SetPattern(self.SingleShot)
                end
            end,

            Cycle = function( self )
                if self.battle:GetTurns() - (self.last_turn or -1) > 0 then 
                    self.turns = (self.turns or 0) + 1
                    self.last_turn = self.battle:GetTurns()
                end
                self.attacks:ChooseCard()
                if self.turns == 1 and self.fighter:GetHealthPercent() > 0.2 then
                    self:ChooseCard(self.incept_burs)
                elseif self.turns == 2 and self.fighter:GetHealthPercent() > 0.2 then
                    self:ChooseCard(self.rally)
                elseif self.turns >= 3 or self.fighter:GetHealthPercent() < 0.2then
                    self:ChooseCard(self.summon)
                end
            end,

            SingleShot = function( self )
                self:ChooseCard(self.shoot)
                self:ChooseCard(self.bail)
            end
        },
    },
})

Content.AddCharacterDef( def )


local CARDS = 
{
    eye_shot =
    {
        name = "Eye Shot",
        anim = "suppressing",
        desc = "Hits all enemies.",
        flavour = "'You can see everything through the stalks.'",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,
        series = CARD_SERIES.NPC,

        target_mod = TARGET_MOD.TEAM,

        min_damage = 5,
        max_damage = 5,
    },

    concussion =
    {
        name = "Concussion",
        pre_anim = "blast_pre",
        anim = "blast",
        flavour = "'A squeeze on that cluster there, I can feel it erupt through the whole swamp.'",
        post_anim = "blast_pst",
        hit_anim = true,
        hit_tags = { "lumin","psionic" },
        desc = "Attack with this card three times.",
        anims = { "anim/weapon_blaster_sal.zip"},

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND,
        series = CARD_SERIES.NPC,

        target_count = 3,
        min_damage = 3,
        max_damage = 3,
    },

    heartburn =
    {
        name = "Heartburn",
        anim = "taunt",
        flavour = "'The heat erupts from within and never stops burning.'",
        desc = "Apply 4 {BURN} to all enemies.",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        series = CARD_SERIES.NPC,

        target_mod = TARGET_MOD.TEAM,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("BURN", 4, self)
        end
    },

    -- bog_attunement =
    -- {
    --     name = "Bog Attunement",
    --     anim = "taunt",
    --     desc = "{IMPROVISE} one of your acquired Bog Secrets.",

    --     cost = 0,

    --     rarity = CARD_RARITY.UNIQUE,
    --     flags = CARD_FLAGS.TOOLBOX | CARD_FLAGS.SKILL | CARD_FLAGS.ITEM,
    --     target_type = TARGET_TYPE.SELF,
    -- },
}

for i, id, data in sorted_pairs( CARDS ) do
    data.series = CARD_SERIES.GENERAL
    Content.AddBattleCard( id, data )
end