local battle_defs = require "battle/battle_defs"
local negotiation_defs = require "negotiation/negotiation_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local BOGGER_NEGOTIATION =
{
	OnInit = function( self, difficulty )
		self.bog_boil = self:AddCard("bog_boil")
		if difficulty <= 2 then
			self:SetPattern( self.BasicCycle )
		else
			self:SetPattern( self.Cycle )
		end
		self.negotiator:AddModifier("UNSHAKABLE")
	end,

	-- Duplicated from Bandits. Needs revision
	BasicCycle = function( self, turns )
		-- Double attack every 2 rounds; Single attack otherwise.
		if self.difficulty >= 4 and turns % 2 == 0 then
			self:ChooseGrowingNumbers( 3, -1 )
		elseif turns % 2 == 0 then
			self:ChooseGrowingNumbers( 2, 1 )
		else
			self:ChooseGrowingNumbers( 1, 3 )
		end
		if turns % 3 == 0 then
			self:ChooseCard(self.bog_boil)
		end
	end,

	Cycle = function( self, turns )
		-- Double attack every 2 rounds; Single attack otherwise.
		if self.difficulty >= 4 and turns % 2 == 0 then
			self:ChooseGrowingNumbers( 3, -1 )
		elseif turns % 2 == 0 then
			self:ChooseGrowingNumbers( 2, 1 )
		else
			self:ChooseGrowingNumbers( 1, 3 )
		end
		if turns % 3 == 0 then
			self:ChooseCard(self.bog_boil)
		end
	end,
}

Content.AddCharacterDef
(
	CharacterDef("BOGGER_BASE",
	{
		base_def = "NPC_BASE",
		is_template = true,

		title = "Bogger",

		tags = {  },
		faction_id = "BOGGERS",

		support_negotiation = "bog_support",

		base_builds = {
			[ GENDER.MALE ] = "male_bogger",
			[ GENDER.FEMALE ] = "female_bogger"
		},
	})
)

Content.AddCharacterDef(
	CharacterDef("BOGGER_CULTIVATOR",
	{
	    base_def = "BOGGER_BASE",
	    renown = 1,
	    combat_strength = 2,
	    title = "Cultivator",

	    death_item = "bog_scimitar",
	    loved_graft = "inoculated",
	    hated_graft = "bog_sensitive",

	    anims = { "anim/weapon_sword_bogger.zip" },
	    combat_anims = { "anim/med_combat_bogger.zip" },

	    negotiation_data = 
	    {
	        behaviour = BOGGER_NEGOTIATION,
	    },
	    fight_data = 
	    {
	        MAX_MORALE = MAX_MORALE_LOOKUP.MEDIUM,
	        MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW,

	        conditions = 
	        { 
	            leeching_blade =
	            {
	            	name = "Leeching Blade",
	            	desc = "{1}'s attacks with their blade siphon lifeforce equal to {2}x the damage they deal, storing it as {siphoned_lifeforce}.",
	            	desc_fn = function( self, fmt_str )
	            		return loc.format(fmt_str, self.owner and self.owner:GetName() or loc.cap( LOC "UI.CARDS.OWNER" ), self.stacks)
	            	end,

	            	event_handlers =
	            	{
	            		[ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
	            			if self.battle:GetFighterFromSource( source ) == self.owner then
	            				self.owner:AddCondition("siphoned_lifeforce", delta * self.stacks, self)
	            			end
	            		end
	            	},
	            },

	            siphoned_lifeforce =
	            {
	            	name = "Siphoned Lifeforce",
	            	desc = "Can be spent to heal allies. Whenever {1} takes damage, this is reduced by the amount of damage taken.",
	            	desc_fn = function( self, fmt_str )
	            		return loc.format(fmt_str, self.owner and self.owner:GetName() or LOC "UI.CARDS.OWNER")
	            	end,

	            	event_handlers = 
	            	{
	            		[ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
	            			if fighter == self.owner then
	            				self.owner:RemoveCondition(self.id, math.min( damage, self.stacks), self)
	            			end
	            		end
	            	},
	            },

	            pruned = 
	            {
	            	name = "Pruned",
	            	desc = "Whenever you play a card, take 1 damage. Reduce this by 1 at the end of your turn.",
	            	ctype = CTYPE.DEBUFF,

	            	event_handlers =
	            	{
	            		[ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
	            			if attack.attacker == self.owner and attack.card.id ~= "execute" then
	            				self.owner:ApplyDamage(1, self.owner, self)
	            			end
	            		end,

	            		[ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
	            			self.owner:RemoveCondition(self.id, 1, self)
	            		end,
	            	},
	            },
	        },

	        attacks = 
	        {
	            cultivator_slice = table.extend(NPC_MELEE)
	            {
	            	name = "Leeching Slice",
	            	anim = "attack1",
	            },

	            cultivator_throw = table.extend(NPC_RANGED)
	            {
	            	name = "Throw",
	            	anim = "throw",

	            	flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
	            	target_mod = TARGET_MOD.TEAM,

	            	damage_mult = 0.5,

	            	OnPostResolve = function( self, battle, attack )
	            		for i,hit in attack:Hits() do
	            			if hit.target:IsPlayer() and not hit.evaded then
	            				hit.target:AddCondition("pruned", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ), self)
	            			else
	            				hit.target:AddCondition("EXPOSED", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ), self)
	            			end
	            		end
	            	end
	            },

	            cultivator_heal = table.extend(NPC_BUFF)
	            {
	            	name = "Rejuvenation",
	            	anim = "taunt",

	            	flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF | CARD_FLAGS.HEAL,
	            	target_type = TARGET_TYPE.SELF,
	            	target_mod = TARGET_MOD.TEAM,

	            	OnPostResolve = function( self, battle, attack )
	            		local count = self.owner:GetConditionStacks("siphoned_lifeforce")
	            		for i,hit in attack:Hits() do
	            			hit.target:HealHealth(count, self)
	            		end
	            		self.owner:RemoveCondition("siphoned_lifeforce", count, self)
	            	end
	            },
	        },

	        behaviour =
	        {
	            OnActivate = function( self, fighter )
	                self.attacks = self:MakePicker()
	                	:AddID("cultivator_slice", 3)
	                	:AddID("cultivator_throw", 1)

	                self.heal = self:AddCard("cultivator_heal")
	                self:SetPattern(self.Cycle)
	                self.fighter:AddCondition("leeching_blade", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ), self)
	            end,

	            HasHurtAlly = function( self )
	            	for i,ally in self.fighter:GetTeam():Fighters() do
	            		if ally:GetHealthPercent() < 1 then
	            			return true
	            		end
	            	end
	            	return false
	            end,

	            Cycle = function( self )
	            	if self.fighter:GetConditionStacks("siphoned_lifeforce") >= 8 and self:HasHurtAlly() then
	            		self:ChooseCard(self.heal)
	            	else
	            		self.attacks:ChooseCard()
	            	end
	            end
	        },
	    },
	})
)

Content.AddCharacterDef(
	CharacterDef("BOGGER_CLOBBER",
	{
	    base_def = "BOGGER_BASE",
	    renown = 1,
	    combat_strength = 3,
		title = "Clobber",
	    base_builds = {
			[ GENDER.MALE ] = "male_bogger_clobber",
			[ GENDER.FEMALE ] = "female_bogger_clobber"
		},

	    death_item = "swamp_smasher",
	    loved_graft = "bogger_pauldrons",
	    hated_graft = "concussed",

	    anims = {"anim/weapon_shield_bogger_clobber.zip"},
	    combat_anims = {"anim/med_combat_shield_bogger_clobber.zip"},

	    negotiation_data = 
	    {
	        behaviour = BOGGER_NEGOTIATION,
	    },
	    fight_data = 
	    {
	        MAX_MORALE = MAX_MORALE_LOOKUP.MEDIUM,
	        MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW,

	        conditions = 
	        { 
	            shield_bash = 
	            {
	            	name = "Shield Bash",
	            	desc = "Remove 1 stack whenever you play a card. At 0 stacks, {1} attacks a random enemy. Set stacks to {2} whenever {1} gets hit by an attack or takes damage.",
	            	desc_fn = function( self, fmt_str )
	            		return loc.format(fmt_str, self.owner and self.owner:GetName() or LOC "UI.CARDS.OWNER", self.base_stacks)
	            	end,

	            	base_stacks = 6,

	            	event_handlers = 
	            	{
	            		[ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
		                    if hit.target == self.owner and hit.card:IsAttackCard() then
	            				self.owner:AddCondition(self.id, self.base_stacks - self.stacks, self)
	            			end
	            		end,

	            		[ BATTLE_EVENT.PRE_RESOLVE ] = function( self, battle, attack )
	            			if self.owner:IsActive() and attack.card and attack.card.owner == battle:GetPlayerFighter() and not self.owner:HasCondition("STUN") then
	            				local targetted = false
	            				for i,hit in attack:Hits() do
	            					if hit.target == self.owner then
	            						targetted = true
	            						break
	            					end
	            				end
	            				if not targetted then
	            					if self.stacks == 1 then
	            						local card = Battle.Card("shield_bash_attack", self.owner)
	            						battle:PlayCard(card)
	            						self.owner:AddCondition(self.id, self.base_stacks - self.stacks, self)
	            					else
	            						self.owner:RemoveCondition(self.id, 1, self)
	            					end
	            				end
	            			end
	            		end,

	            		[ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
			                if fighter == self.owner then
								self.owner:AddCondition(self.id, self.base_stacks - self.stacks, self)
							end
			            end,
	            	},
	            },

	            trample =
	            {
	            	name = "Trample",
	            	desc = "{1}'s attacks deal bonus damage equal to their {DEFEND}. Remove 1 <b>Trample</b> at the end of their turn.",
	            	desc_fn = function( self, fmt_str )
	            		return loc.format( fmt_str, self.owner and self.owner:GetName() or loc.cap( LOC "UI.CARDS.OWNER"))
	            	end,

	            	event_handlers =
	            	{
	            		[ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
	            			if card.owner == self.owner and self.owner:HasCondition("DEFEND") then
	            				local count = self.owner:GetConditionStacks("DEFEND")
	            				dmgt:AddDamage(count, count, self)
	            			end
	            		end,

	            		[ BATTLE_EVENT.END_TURN ] = function( self, fighter )
	            			if fighter == self.owner then
	            				if self.skipped_first then
	            					self.owner:RemoveCondition(self.id, 1, self)
	            				else
	            					self.skipped_first = true
	            				end
	            			end
	            		end
	            	},
	            },
	        },

	        attacks = 
	        {
	            clobber_smack = table.extend(NPC_MELEE)
	            {
	            	name = "Shield Smack",
	            	anim = "attack1",
	            	damage_mult = .8,
	            },

	            clobber_charge = table.extend(NPC_MELEE)
	            {
	            	name = "Charge",
	            	anim = "attack2",

	            	flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,
	            	damage_mult = .7,

	            	features =
	            	{
	            		IMPAIR = 1,
	            	},
	            },

	            clobber_rally = table.extend(NPC_BUFF)
	            {
	            	name = "Rally",
	            	anim = "taunt",

	            	flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
	            	target_type = TARGET_TYPE.SELF,
	            	target_mod = TARGET_MOD.TEAM,

	            	OnPostResolve = function( self, battle, attack )
	            		self.owner:AddCondition("trample", 2, self)
	            		attack:AddCondition("DEFEND", 3, self)
	            	end,
	            },

	            shield_bash_attack = table.extend(NPC_MELEE)
	            {
	            	name = "Shield Bash",
	            	anim = "attack1",

	            	target_mod = TARGET_MOD.RANDOM1,
	            	damage_mult = 0.5
	            },
	        },

	        behaviour =
	        {
	            OnActivate = function( self, fighter )
	                self.attacks = self:MakePicker()
	                	:AddID("clobber_smack", 3)
	                	:AddID("clobber_charge", 2)
	                	:AddID("clobber_rally", 1)

	               	local base_stacks = 6 - 2 * GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )
	                if self.fighter:GetTeamID() == TEAM.RED then
		                local shield_bash = self.fighter:AddCondition("shield_bash", base_stacks , self)
		                shield_bash.base_stacks = base_stacks
		            end

	                self.fighter:AddCondition("ARMOURED", 3 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ), self)
	                self:SetPattern(self.Cycle)
	            end,

	            Cycle = function( self )
	            	self.attacks:ChooseCard()
	            end
	        },
	    },
	})
)
