local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT


local def = CharacterDef("BORDENKRA",
{
    unique = true,
    alias = "BORDENKRA",
    base_def = "FOREMAN",
    species = SPECIES.SHROKE,
    gender = GENDER.FEMALE,
    name = "Bordenkra",
    title = "Diplomat",
    faction_id = "RENTORIAN",
    build = "female_rentorian_diplomat_build",
    head = "head_female_shroke_13",
    anims = { "anim/weapon_double_bladed_staff_rentorian.zip" },
    combat_anims = { "anim/med_combat_knife_rentorian_diplomat.zip" },

    loved_graft = "envoy_pendant",
    hated_graft = "scorned",
    death_item = "veil",

    fight_data =
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
        MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,
        conditions =
        {

        },
        attacks =
        {
            bordenkra_attack1 = table.extend(NPC_ATTACK)
            {
                anim = "attack1",
                flags = CARD_FLAGS.MELEE,

                damage_mult = 1.2,
            },

            bordenkra_attack2 = table.extend(NPC_ATTACK)
            {
                anim = "attack2",
                flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

                damage_mult = 0.6,
                bleed_amt = 4,

                OnPostResolve = function( self, battle, attack )
                    for i,hit in ipairs(attack.hits) do
                        if hit.target:IsPlayer() then
                            local cards = {}
                            for i=1,2 do
                                table.insert(cards, Battle.Card( "status_bleed", hit.target) )                            
                            end
                            battle:DealCards( cards, battle:GetDrawDeck() )
                            self.has_spoken = false
                        else
                            hit.target:AddCondition("BLEED", self.bleed_amt, attack)
                        end
                    end
                end,
            },

            bordenkra_taunt = table.extend(NPC_BUFF)
            {
                anim = "taunt",
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                defend_amt = 5,

                OnPostResolve = function( self, battle, attack )
                    attack:AddCondition ("DEFEND", self.defend_amt, self)
                    attack:AddCondition( "EVASION", 2, self)
                end,
            },
        },

        behaviour =
        {
            OnActivate = function( self )
                self.attacks = self:MakePicker()
                    :AddID("bordenkra_attack1", 1)
                    :AddID("bordenkra_attack2", 2)
                    :AddID("bordenkra_taunt", 1)

                self:SetPattern( self.Cycle )
            end,

            Cycle = function(   self )    
                self.attacks:ChooseCard()
            end
        },
    },
})

Content.AddCharacterDef( def )
    