local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT


local def = CharacterDef("DELTREAN_DIGNITARY",
{
    unique = true,
    alias = "DELTREAN_DIGNITARY",
    base_def = "PEARLIE",
    species = SPECIES.HUMAN,
    gender = GENDER.MALE,
    name = "Palamborat",
    title = "Diplomat",
    faction_id = "DELTREAN",
    build = "male_deltrean_dignitary_build",
    head = "head_male_deltrean_dignitary",
    anims = { "anim/weapon_rapier_deltrean_dignitary.zip" },
    combat_anims = { "anim/med_combat_rapier_deltrean_dignitary.zip" },
    tags = {"plot_armour", "no_patron"},
    loved_graft = "",
    hated_graft = "bad_credit",
    death_item = "diplomats_badge",

    fight_data =
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,

        anim_mapping = 
        {
            riposte = "stab",
        },

        attacks =
        {
            dignitary_stab = table.extend(NPC_ATTACK)
            {
                name = "Lunge",
                anim = "stab",

                flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,
                damage_mult = 0.8,
            },

            dignitary_stab2 = table.extend(NPC_ATTACK)
            {
                name = "Double Strike",
                anim = "stab2",

                flags = CARD_FLAGS.MELEE,

                damage_mult = 0.7,
                hit_count = 2,
            },

            dignitary_taunt = 
            {
                name = "Brandish",
                anim = "taunt",
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,

                target_type = TARGET_TYPE.SELF,

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("DEFEND", 6 + (4 * GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )), self)
                    self.owner:AddCondition("POWER", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ), self)
                    self.owner:AddCondition("RIPOSTE", 2 + (3 * GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )), self)
                end
            },
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                self.attacks = self:MakePicker()
                    :AddID( "dignitary_stab", 2 )
                    :AddID( "dignitary_stab2", 1 )
                    :AddID( "dignitary_taunt", 1)

                self:SetPattern( self.Cycle )
            end,

            Cycle = function( self )
                self.attacks:ChooseCard()
            end
        }
    },
})

Content.AddCharacterDef( def )
    