local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local def = CharacterDef("DENIKUS",
{
    unique = true,

    base_def = "BANDIT_BASE",
    alias = "DENIKUS",
    species = SPECIES.HUMAN,
    gender = GENDER.MALE,
    renown = 3,
    combat_strength = 2,
    --boss = true,
    name = "Denikus",
    
    tags = {"plot_armour", "no_patron"},
    loved_graft = "con_man",
    hated_graft = "bad_mouthed",
    death_item = "denikus_blaster",
    theme_music = "event:/music/story/smith/story_smith_denikus",
    -- battle_preview_anim = "anim/boss_hesh_slide.zip",
    -- battle_preview_offset = { x = 185, y = 10 },
    -- battle_preview_glow = { colour = 0x10ffffFF, bloom = 0.3, threshold = 0.02 },
    -- battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_heshboss",
    -- icon = engine.asset.Texture( "icons/b_hesh_boss.tex"),
    faction_id = "BANDITS",
    title = "Heartbreaker",
    -- voice_actor = "",
    
    anims = { "anim/weapon_rapier_denikus.zip", },
    combat_anims = { "anim/med_combat_rapier_denikus.zip" },
    head = "head_denikus_build",
    build = "male_denikus_build",

    negotiation_data = 
    {
        modifiers =
        {

        },
        behaviour = 
        {
           -- TODO: Replace this. Currently just a duplicate of Bandit Captain
           KINGPIN = 1,

            OnInit = function( self )
                self.escalation = self:AddArgument( "ESCALATION" )
                self.kingpin = self:AddArgument( "KINGPIN" )
                self.greedy = self:AddArgument( "GREEDY" )

                if self.difficulty <= 2 then
                    self:SetPattern( self.BasicCycle )
                else
                    self:SetPattern( self.Cycle )
                end
                
                self.negotiator:AddModifier("SHORT_FUSE")
            end,
            
            BasicCycle = function( self, turns )
                if turns == 2 then
                    self:ChooseCard( self.escalation )
                    self:ChooseGrowingNumbers( 1, 1 )
                elseif (turns-1) % 3 == 0 then
                    self:ChooseGrowingNumbers( 2, 1 )

                else
                    self:ChooseGrowingNumbers( 1, 1 )
                end
            end,

            Cycle = function( self, turns )
                if turns % 3 == 0 then
                    self:ChooseCard( self.greedy )
                end
                if self.difficulty >= 4 and turns % 2 == 0 then
                    self:ChooseGrowingNumbers( 3, -1 )
                elseif turns % 2 == 0 then
                    self:ChooseGrowingNumbers( 2, 1 )
                else
                    self:ChooseGrowingNumbers( 1, 3 )
                end
                if (turns - 1) % 5 == 0 and not self.negotiator:FindModifier( "ESCALATION" ) then
                    self:ChooseCard( self.escalation )
                end
                if self.KINGPIN > 0 then
                    if (turns + 3) % 5 == 0 and not self.negotiator:FindModifier( "KINGPIN" ) then
                        self:ChooseCard( self.kingpin )
                    end
                end
            end,
        }
    },
    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 85,

        conditions = 
        { 
            SENTENCED = 
            {
                name = "Sentenced",
                desc = "Any attacks that cause health loss against {1} also apply <#HILITE>{2} {BLEED}</>. Reduce by 1 at the end of {1}’s turn.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self:GetOwnerName(), self.stacks and self.stacks or 1 )
                end,

                ctype = CTYPE.DEBUFF,

                event_handlers =
                {
                    [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                        if hit.target == self.owner and not hit.evaded and not hit.defended and (hit.damage or 0) > 0 then
                            if hit.target.player_controlled then
                                local cards = {}
                                for i=1, self.stacks do
                                    local incepted_card = Battle.Card( "status_bleed", hit.target)
                                    table.insert(cards, incepted_card )                                  
                                end
                                battle:DealCards( cards, battle:GetDiscardDeck() )
                            else
                                hit.target:AddCondition("BLEED", self.stacks, self)
                            end
                        end
                    end,

                    [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        if fighter == self.owner then
                            self.owner:RemoveCondition("SENTENCED", 1, self)
                        end
                   end,
                },
            },
        },

        attacks = 
        {
            denikus_jab = table.extend(NPC_ATTACK)
            {
                name = "Slice",
                anim = "jabs",

                flags = CARD_FLAGS.MELEE,
                target_count = 3,
                damage_mult = 0.4,
            },

            denikus_shoot = table.extend(NPC_ATTACK)
            {
                name = "Shoot",
                anim = "shoot",

                flags = CARD_FLAGS.RANGED,

                features =
                {
                    EXPOSED = 2,
                }
            },

            denikus_taunt = table.extend(NPC_BUFF)
            {
                name = "Taunt",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.DEBUFF,

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("DEFEND", 6, self)
                    attack:AddCondition("SENTENCED", 2, self)
                end
            },
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                self.attacks = self:MakePicker()
                    :AddID("denikus_jab", 1)
                    :AddID("denikus_shoot", 1)
                    :AddID("denikus_taunt", 1)

                self:SetPattern(self.Cycle)
            end,

            Cycle = function( self )
                self.attacks:ChooseCard()
            end
        },
    },
})

Content.AddCharacterDef( def )
    