local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
require "content/characters/base_types"

Content.AddCharacterDef
(
	CharacterDef("DREDGER_BASE",
	{
		base_def = "NPC_BASE",
		is_template = true,

		title = "Dredger",

		tags = { "dredger" },
		faction_id = "BILEBROKERS",


		base_builds = {
			[ GENDER.MALE ] = "dredger_male",
			[ GENDER.FEMALE ] = "dredger_female"
		},

		
	})
)

--------------------------------------------------------------------------------


Content.AddCharacterDef
(
	CharacterDef("CHEMIST",
	{
		base_def = "DREDGER_BASE",

		title = "Chemist",
		tags = { "chemist" },
		renown = 2,
		combat_strength = 2,

		loved_graft = "doze_bug",
		hated_graft = "spiked_drink",

		social_boons =
		{
			[ RELATIONSHIP.LIKED ] = { HEALING = 1, SOCIALIZE = 1 },
			[ RELATIONSHIP.LOVED ] = { HEALING = 3, GIVE_ITEM = 1 },
		},

		voice_actors = {
			[GENDER.FEMALE] = { "chemist" },
    	},

    	death_item = "vapor_vial",

		fight_data =
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.MEDIUM,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW,

			cards = { "oshnu_acid", "volatile_substance", "vapours" },

			behaviour =
			{
				OnActivate = function( self )
					self.acid = self:AddCard( "oshnu_acid" )

					self.attacks = self:MakePicker()
						:AddID( "volatile_substance", 2 )
						:AddID( "ai_defend_lo", 1 )
						:AddID( "vapours", 1 )

					self:SetPattern( self.Acid )
					if self.fighter:GetTeamID() == TEAM.RED then
						self.fighter:AddCondition("bombard")
					end
				end,

				Acid = function( self )
					self:ChooseCard( self.acid )
					self:SetPattern( self.Cycle )
				end,

				Cycle = function( self )
					self.attacks:ChooseCard()
				end,
			},
		},
		combat_anims = {"anim/med_combat_unarmed_chemist.zip"},
		anims={"anim/weapon_knife_common.zip"},

		support_negotiation = "chemist_support",

		negotiation_data = 
		{
			modifiers =
			{
				CURIOSITY =
				{
					name = "Curiosity",
					desc = "When destroyed, draw up to your maximum hand size. ({1})",
					desc_fn = function( self, fmt_str )
						return loc.format( fmt_str, self.engine.MAX_HAND_SIZE )
					end,

					modifier_type = MODIFIER_TYPE.BOUNTY,
					max_stacks = 1,
	                min_resolve = 3,
	                max_resolve = 3,

					OnBounty = function( self )
						local draw = self.engine.MAX_HAND_SIZE - self.engine:GetHandDeck():CountCards()
						self.engine:DrawCards( draw )
					end,
				}
			},
			behaviour =
			{
				OnInit = function( self, difficulty )
					self.distracted = self:AddArgument( "DISTRACTED" )
					self.distracted.stacks = math.floor(difficulty * 1.5)

					self.eccentricity = self:AddArgument( "ECCENTRICITY" )
					self.curiosity = self:AddArgument( "CURIOSITY" )

					self:SetPattern( self.Cycle )

					self.card_total = 3
					self.negotiator:AddModifier("PURPLE_HAZE")
				end,

				Cycle = function( self, turn )
					if turn == 1 then
						self:ChooseCard( self.eccentricity )
					end

					-- Turn 3/9: Distracted.
					if self.difficulty >= 2 and (turn == 3 or turn == 9) then
						self:ChooseCard( self.distracted )
					end

					-- Every 4 turns: Bounty
					if turn % 4 == 0 and not self.negotiator:HasModifier( "CURIOSITY" ) then
						self:ChooseCard( self.curiosity )
					end

					self:ChooseGrowingNumbers( 1 )
				end,
			}
		},
	})
)

Content.AddCharacterDef
(
	CharacterDef("BILEBROKER_SAMPLER",
	{
		base_def = "DREDGER_BASE",

		title = "Sampler",
		tags = { "chemist" },
		renown = 2,
		combat_strength = 2,
		unique = true,
		gender = GENDER.MALE,

		loved_graft = "doze_bug",
		hated_graft = "spiked_drink",
    	death_item = "vapor_vial",

		combat_anims = {"anim/med_combat_unarmed_bilebroker.zip"},
		-- anims={"anim/.zip"},
		
		base_builds = {
            [ GENDER.MALE ] = "male_bilebroker_build",
        },

		fight_data =
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM_LOW,

			attacks =
			{
				sampler_punch = table.extend(NPC_ATTACK)
				{
					name = "Punch",
					anim = "punch",
					flags = CARD_FLAGS.MELEE,
				},

				sampler_throw = table.extend(NPC_ATTACK)
				{
					name = "Grenade",
					anim = "throw",

					flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
					target_mod = TARGET_MOD.TEAM,
					damage_mult = 0.4,

					debuff_options = {
			            [ "WOUND" ] = 10,
			            [ "BLEED" ] = 8,
			            [ "IMPAIR" ] = 8,
			            [ "EXPOSED" ] = 6,
			            [ "DISEASED" ] = 4,
			        },

			        debuff_stacks =
			        {
			            [ "WOUND" ] = 1,
			            [ "BLEED" ] = 2,
			            [ "IMPAIR" ] = 1,
			            [ "EXPOSED" ] = 2,
			            [ "DISEASED" ] = 1,
			        },

			        OnPostResolve = function( self, battle, attack )
			            local chosen = weightedpick(self.debuff_options)
			            attack:AddCondition(chosen, self.debuff_stacks[chosen], self)
			        end,
				},

				sampler_taunt = table.extend(NPC_ATTACK)
				{
					name = "Rouse",
					anim = "taunt",

					flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
					target_mod = TARGET_MOD.TEAM,
					target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

					OnPostResolve = function( self, battle, attack )
						attack:AddCondition("DEFEND", 6, self)
						attack:AddCondition("POWER", 1, self)
						attack:AddCondition("POWER_LOSS", 1, self)
					end
				},
			},

			behaviour =
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID( "sampler_punch", 2 )
						:AddID( "sampler_throw", 3 )
						:AddID( "sampler_taunt", 1 )

					self:SetPattern( self.Cycle )
				end,

				Cycle = function( self )
					self.attacks:ChooseCard()
				end,
			},
		},

		support_negotiation = "chemist_support",

		negotiation_data = 
		{
			modifiers =
			{
			},
			behaviour =
			{
				OnInit = function( self, difficulty )
					self.distracted = self:AddArgument( "DISTRACTED" )
					self.distracted.stacks = math.floor(difficulty * 1.5)

					self.eccentricity = self:AddArgument( "ECCENTRICITY" )
					self.curiosity = self:AddArgument( "CURIOSITY" )

					self:SetPattern( self.Cycle )

					self.card_total = 3
					self.negotiator:AddModifier("PURPLE_HAZE")
				end,

				Cycle = function( self, turn )
					if turn == 1 then
						self:ChooseCard( self.eccentricity )
					end

					-- Turn 3/9: Distracted.
					if self.difficulty >= 2 and (turn == 3 or turn == 9) then
						self:ChooseCard( self.distracted )
					end

					-- Every 4 turns: Bounty
					if turn % 4 == 0 and not self.negotiator:HasModifier( "CURIOSITY" ) then
						self:ChooseCard( self.curiosity )
					end

					self:ChooseGrowingNumbers( 1 )
				end,
			}
		},
	})
)