local battle_defs = require "battle/battle_defs"
require "content/characters/monsters"
local MONSTER_DEF = Content.GetCharacterDef("MONSTER")
assert( MONSTER_DEF )

local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local FLAGS = battle_defs.CARD_FLAGS

Content.AddCharacterDef
(
    CharacterDef("DROAD",
    {
        base_def = "MONSTER",

        boss = true,
        title = "Lumbering Brute",
        name = "Droad",
        build = "droad",
        alias = "DROAD",
        
        -- death_money = 0,
        death_item= "droad_tail_spike",

        battle_preview_anim = "anim/boss_droad_slide.zip",
        battle_preview_offset = { x = 10, y = 50 },
        battle_preview_glow = { colour = 0x33ffffFF, bloom = 0.8, fx_bloom = 0.2, threshold = 0.02 },
        battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_droad",
        icon = engine.asset.Texture( "icons/b_droad.tex" ),

        fight_data = table.extend( MONSTER_DEF.fight_data )
        {
            MAX_HEALTH = 375,
            MAX_MORALE = MAX_MORALE_LOOKUP.IMMUNE,
            battle_scale = 1.25,
            status_widget_dx = 0,
            status_widget_dy = -0.8,

            attacks = 
            {   

                droad_bite = table.extend(NPC_ATTACK)
                {
                    name = "Bite",
                    anim = "bite",
                    flags = FLAGS.MELEE | FLAGS.DEBUFF,
                    anim_mapping =
                    {
                        run_forward_pre = "jump_pre",
                        run_forward = "jump",
                        run_forward_pst = "jump_pst",
                    },
                    run_forward_pst = true,
                    base_damage = { 6, 10, 12, 14 },

                    impair_amt = { 1, 2, 2, 3 },
                    muddled_amt = { 2, 3, 3, 4 },

                    OnPostResolve = function( self, battle, attack )
                        for i,hit in ipairs(attack.hits) do
                            if hit.target:IsPlayer() then
                                hit.target:AddCondition("MUDDLED", self.muddled_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                            else
                                hit.target:AddCondition("IMPAIR", self.impair_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                            end
                        end
                    end
                },

                droad_tongue = table.extend(NPC_ATTACK)
                {
                    name = "Tongue Lash",
                    anim = "tongue",
                    flags = FLAGS.RANGED,
                    target_mod = TARGET_MOD.TEAM,
                    base_damage = { 8, 12, 14, 16 },
                },

                droad_taunt = table.extend(NPC_BUFF)
                {
                    name = "Roar",
                    anim = "taunt",
                    flags = FLAGS.SKILL | FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    defend_amt = { 5, 12, 14, 18 },
                    power_amt = { 1, 1, 2, 3 },

                    OnPostResolve = function( self, battle, attack )
                        self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                        self.owner:AddCondition("POWER", self.power_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                    end
                }
            },

            conditions =
            {
                COLOSSAL = 
                {
                    name = "Colossal",
                    desc = "{1} takes <#HILITE>{2}</> bonus damage from all attacks. Increase by 1 whenever {1} is hit. Reset to 1 at the start of {1}’s turn.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOwnerName(), self.stacks )
                    end,

                    ctype = CTYPE.DEBUFF,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                            if target == self.owner then
                                dmgt:AddDamage( self.stacks, self.stacks, self )
                            end
                        end,

                        [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                            if self.owner == fighter then
                                self.owner:RemoveCondition(self.id, self.stacks - 1, self)
                            end
                        end,

                        [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                            if hit.target == self.owner and attack.card:IsAttackCard() and not hit.evaded then
                                self.owner:AddCondition(self.id, 1, self)
                            end
                        end
                    }
                },

                MUDDLED =
                {
                    name = "Muddled",
                    desc = "Whenever you play an attack, there's a 50% chance that it will miss. Reduce <b>Muddled</> by 1 whenever you play a card.",

                    ctype = CTYPE.DEBUFF,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.PRE_RESOLVE ] = function( self, battle, attack )
                            if attack.attacker == self.owner then
                                if attack.card:IsAttackCard() and math.random() < 0.5 then
                                    for i,hit in ipairs(attack.hits) do
                                        hit.evaded = true
                                        hit.damage = 0
                                        battle:BroadcastEvent( BATTLE_EVENT.ON_DODGE, hit.target, nil, attack.attacker )
                                    end
                                end
                                self.owner:RemoveCondition(self.id, 1, self)
                            end
                        end,
                    },
                },
            },

            behaviour =
            {
                OnActivate = function( self )
                    self.attacks = self:MakePicker()
                        :AddID("droad_bite", 3)
                        :AddID("droad_tongue", 2)
                        :AddID("droad_taunt", 1)

                    self.fighter:AddCondition("COLOSSAL")

                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    self.attacks:ChooseCard()
                end,
            }
        },
    })
)