local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local FLAGS = battle_defs.CARD_FLAGS


--local drusk_health_changes = {1, .67, .5, .25}
local DruskBrain = class("DruskBrain")
DruskBrain.can_reload = true

function DruskBrain:init(battle)
    self.battle = battle
    self.current_attacker = math.random(2)
    self.drusk = {}
    battle:ListenForAny( self, self.OnBattleEvent )

    self.single_attacker_id = "DRUSK_1"
end

function DruskBrain:OnBattleEvent(event, ...)
    if event == BATTLE_EVENT.BEGIN_PLAYER_TURN then
        self.single_attacker_id = self.single_attacker_id == "DRUSK_1" and "DRUSK_2" or "DRUSK_1"
    end
end


function DruskBrain:AddDrusk(fighter)
    local id = fighter.agent:GetContentID()
    local first = next(self.drusk) == nil
    self.drusk[id] = self.drusk[id] or {}
    self.drusk[id].fighter = fighter
    self.drusk[id].num_spawned = (self.drusk[id].num_spawned or 0) + 1
    
    local other_drusk = self:GetOtherDrusk(fighter)

    if self.drusk[id].num_spawned > 1 then
        fighter:SetStat(COMBAT_STAT.HEALTH, other_drusk:GetHealth())
    end

    if first then
        self.battle:PlayCard(fighter.behaviour.moves.summon)
    end
end

function DruskBrain:GetNumActive()
    local num = 0
    for k,v in pairs(self.drusk) do
        if v.fighter:IsActive() then
            num = num + 1
        end
    end
    return num
end

function DruskBrain:GetOtherDrusk(fighter)
    for k,v in pairs(self.drusk) do
        if v.fighter ~= fighter then
            return v.fighter
        end
    end
end

function DruskBrain:GetLeastHealthy()
    local lowest, ret

    for k,v in pairs(self.drusk) do
        if v.fighter:IsActive() then
            local p = v.fighter:GetHealthPercent()
            if lowest == nil or p < lowest then
                lowest = p
                ret = v
            end
        end
    end
    
end

function DruskBrain:CountCards(id)
    return self.battle:GetDrawDeck():CountCards(id) + self.battle:GetDiscardDeck():CountCards(id) + self.battle:GetHandDeck():CountCards(id)
end

--local MAX_DRUSK_1_CARDS = {6, 8, 9} 
--local MAX_DRUSK_2_CARDS = {2, 3, 4} 

local MAX_DRUSK_1_CARDS = {2, 2, 3, 4} 
local MAX_DRUSK_2_CARDS = {1, 1, 2, 3} 


function DruskBrain:MakeDecision(fighter)
    
    if self:GetNumActive() == 2 then
        local least_healthy = self:GetLeastHealthy()

        local max_1 = MAX_DRUSK_1_CARDS[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
        local max_2 = MAX_DRUSK_2_CARDS[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
        local insert_card_id = fighter.agent:GetContentID() == "DRUSK_1" and "drusk_01_card" or "drusk_02_card"
        
        local card_max = fighter.agent:GetContentID() == "DRUSK_1" and max_1 or max_2
        local card_min = 1
        
        --if we are low on health and in trouble...
        if least_healthy == fighter and fighter:GetHealthPercent() < .5 then
            if math.random() <= .67 then
                fighter.behaviour:ChooseCard(fighter.behaviour.moves.defend)    
                return
            end
        end
       
        --if we need to get some cards in there...
        local card_count = self:CountCards(insert_card_id)
        if card_count < card_min then
            fighter.behaviour:ChooseCard(fighter.behaviour.moves.aoe)    
            return
        end

        --just do something-or-other
        local valid = {"attack"}
        if fighter:GetHealthPercent() < .67 then
            table.insert(valid, "defend")
        end
        
        if card_count < card_max then
            table.insert(valid, "aoe")
        end

        local card = table.arraypick(valid)
        fighter.behaviour:ChooseCard(fighter.behaviour.moves[card])

    else
        fighter.behaviour:ChooseCard(fighter.behaviour.moves.summon)
    end
end


local function druskbehaviour(moves)
    return 
    {
        OnActivate = function( self )
            self.moves = {}
            for k,v in pairs(moves) do
                self.moves[k] = self:AddCard(v)
            end
            self.fighter.battle.druskbrain = self.fighter.battle.druskbrain or DruskBrain(self.fighter.battle)
            self.druskbrain = self.fighter.battle.druskbrain
            self.druskbrain:AddDrusk(self.fighter)
            self:SetPattern( self.Cycle )

            self.fighter:AddCondition("DRUSK_LIFE_PACT")

        end,

        Cycle = function( self )
            self.druskbrain:MakeDecision(self.fighter)
        end,
        
    }
end


Content.AddCharacterDef
(
    CharacterDef("DRUSK_1",
    {
        base_def = "MONSTER",

        name = "Crimson Drusk",
        shorttitle = "Forgotten Relic",
        build = "drusk_01",
        title = "Shambling Conglomeration",
        gender = GENDER.MALE,

		boss = true,
        battle_preview_name_fn = function(agent) return agent.agent_def:GetLocalizedString("TWIN_BOSS_PREVIEW_NAMES") end,
        battle_preview_title_fn = function(agent) return agent.agent_def:GetLocalizedString("TWIN_BOSS_PREVIEW_TITLE") end,
        battle_preview_anim = "anim/boss_drusk_slide.zip",
        battle_preview_offset = { x = 135, y = 0 },
        battle_preview_glow = { colour = 0xC3DE2EFF, bloom = 0.15, threshold = 0.02 },
        battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_drusk1",
        icon = engine.asset.Texture( "icons/b_drusk1.tex"),

        loc_strings =
        {
            TWIN_BOSS_PREVIEW_NAMES = "Ochre and Crimson Drusks",
            TWIN_BOSS_PREVIEW_TITLE = "Forgotten Relics",
        },

        combat_anims = { "anim/med_combat_drusk_01.zip" },

        death_item = "drusk_core",

        fight_data =
        {       
            MAX_HEALTH = 150,
            MAX_MORALE = MAX_MORALE_LOOKUP.IMMUNE,
            status_widget_dx = 0.5,
            status_widget_dy = -0.95,

            cards = { "drusk_01_splort", "drusk_01_stab", "drusk_01_taunt" },
            attacks = {

                drusk_01_splort = table.extend(NPC_RANGED)
                {
                    name = "Fume",
                    anim = "splort",
                    num_to_insert = {2, 2, 2, 3},
                    target_mod = TARGET_MOD.TEAM,
                    base_damage = {4,5,6,8},
                    OnPostResolve = function(self, battle, attack)
                        local cards = {}
                        local num = self.num_to_insert[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
                        for i=1, num do
                            local incepted_card = Battle.Card( "drusk_01_card", battle:GetPlayerTeam():Primary())
                            incepted_card.source = self.owner
                            --incepted_card.bonus_damage = self.bonus_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
                            table.insert(cards, incepted_card )                                  
                        end

                        battle:Delay( 0.5 )
                        battle:DealCards( cards, battle:GetDiscardDeck() )
                    end,

                },

                drusk_01_stab = table.extend(NPC_MELEE)
                {
                    name = "Stab",
                    anim = "stab",
                    base_damage = {10,11,12,14},
                    flags = FLAGS.DEBUFF | FLAGS.MELEE,
                    features =
                    {
                        IMPAIR = 1,
                    },

                },

                drusk_01_taunt = table.extend(NPC_BUFF)
                {
                    name = "Defend",
                    anim = "taunt_defend",
                    target_type = TARGET_TYPE.SELF,
                    flags = FLAGS.SKILL | FLAGS.BUFF,

                    features = 
                    {
                        DEFEND = 10,
                    },
                },
                drusk_01_summon = table.extend(NPC_BUFF)
                {
                    name = "Call",
                    anim = "taunt",

                    flags = FLAGS.SKILL | FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    CanPlayCard = function(self, battle)
                        return self.owner:GetTeam():NumActiveFighters() == 1
                    end,

                    features = 
                    {
                        DEFEND = 10,
                    },

                    OnPostResolve = function( self, battle, attack )
                        local agent = Agent( "DRUSK_2" )
                        local new_fighter = Fighter.CreateFromAgent( agent, self.owner:GetScale() )
                        new_fighter.no_death_loot = true
                        self.owner:GetTeam():AddFighter( new_fighter )
                        self.owner:GetTeam():ActivateNewFighters( battle )
                        battle:BroadcastEvent( BATTLE_EVENT.UPDATE_HOME_POSITIONS, self.owner:GetTeam())

                    end,
                },

            },

            conditions = {
                --this is just for display. the drusk behavior takes care of the details.
                DRUSK_LIFE_PACT = 
                {
                    name = "Life Pact",
                    desc = "Brings their partner back to life if they have died.",
                },
            },

            behaviour = druskbehaviour{
                attack = "drusk_01_stab",
                aoe = "drusk_01_splort",
                summon ="drusk_01_summon",
                defend = "drusk_01_taunt",
            }
        },
    })
)

Content.AddCharacterDef
(
    CharacterDef("DRUSK_2",
    {
        base_def = "MONSTER",

        name = "Ochre Drusk",
        build = "drusk_02",
        boss = true,
        title = "Shambling Conglomeration",
        gender = GENDER.FEMALE,

        combat_anims = { "anim/med_combat_drusk_02.zip" },

        fight_data =
        {
            MAX_HEALTH = 170,
            MAX_MORALE = MAX_MORALE_LOOKUP.IMMUNE,
            status_widget_dx = 0.5,
            status_widget_dy = -0.95,

            cards = { "drusk_02_attack1", "drusk_02_attack2", "drusk_02_taunt" },
            attacks = {
                drusk_02_attack1 = table.extend(NPC_RANGED)
                {
                    name = "Ignite",
                    anim = "splat",

                    flags = FLAGS.RANGED,
                    target_mod = TARGET_MOD.TEAM,
                    base_damage = {4,5,7,9},
                    num_to_insert = {1,1,1,2},

                    OnPostResolve = function(self, battle, attack)
                        local cards = {}
                        
                        local num = self.num_to_insert[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
                        for i=1, num do
                            local incepted_card = Battle.Card( "drusk_02_card", battle:GetPlayerTeam():Primary())
                            incepted_card.source = self.owner
                            --incepted_card.bonus_damage = self.bonus_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
                            table.insert(cards, incepted_card )                                  
                        end
                        
                        battle:Delay( 0.5 )
                        battle:DealCards( cards, battle:GetDiscardDeck() )
                    end,
                },

                drusk_02_attack2 = table.extend(NPC_MELEE)
                {
                    name = "Stab",
                    anim = "stab",
                    base_damage = {8,9,10,12},
                    flags = FLAGS.MELEE | FLAGS.DEBUFF,
                    features =
                    {
                        WOUND = 2,
                    },

                },

                drusk_02_taunt = table.extend(NPC_BUFF)
                {
                    name = "Defend",
                    anim = "taunt_defend",
                    target_type = TARGET_TYPE.SELF,
                    flags = FLAGS.SKILL | FLAGS.BUFF,

                    features = 
                    {
                        DEFEND = 10,
                    },
                },       
                drusk_02_summon = table.extend(NPC_BUFF)
                {
                    name = "Call",
                    anim = "taunt",

                    flags = FLAGS.SKILL | FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    CanPlayCard = function(self, battle)
                        return self.owner:GetTeam():NumActiveFighters() == 1
                    end,

                    features = 
                    {
                        DEFEND = 10,
                    },

                    OnPostResolve = function( self, battle, attack )
                        local agent = Agent( "DRUSK_1" )
                        local new_fighter = Fighter.CreateFromAgent( agent, self.owner:GetScale() )
                        new_fighter.no_death_loot = true
                        self.owner:GetTeam():AddFighter( new_fighter )
                        self.owner:GetTeam():ActivateNewFighters( battle )
                        battle:BroadcastEvent( BATTLE_EVENT.UPDATE_HOME_POSITIONS, self.owner:GetTeam())
                    end,
                },

            },
            behaviour = druskbehaviour{
                attack = "drusk_02_attack2",
                aoe = "drusk_02_attack1",
                summon = "drusk_02_summon",
                defend = "drusk_02_taunt",
            }
        },
    })
)



Content.AddBattleCard("drusk_01_card", {
    name = "Toxic Fumes",
    flavour = "'These things are made from corrupted technology and mutated wildlife, I'm not sure how to deal with this.'",
    desc = "Take {1} damage.\nIf ignited by {drusk_02_card}, take {2} damage.",
    desc_fn = function(self, fmt_str)
        local ignite_damage = self.ignite_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
        local dump_damage = self.dump_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
        return loc.format(fmt_str, dump_damage, ignite_damage)
    end,
    ignite_damage = {10,11,12,14},
    dump_damage = {2,3,3,4},
    --anim = "taunt",
    cost = 0,
    
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.NPC,
    flags = battle_defs.CARD_FLAGS.STATUS | battle_defs.CARD_FLAGS.EXPEND | battle_defs.CARD_FLAGS.REPLENISH,
    target_type = TARGET_TYPE.SELF,
    OnPostResolve = function ( self, battle, attack )
        local ignite_damage = self.ignite_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
        local dump_damage = self.dump_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
        self.owner:ApplyDamage( self.ignited and ignite_damage or dump_damage, nil, nil, nil, {"spark"} )
    end,
})


Content.AddBattleCard("drusk_02_card", {
    name = "Volatile Spark",
    flavour = "'Get out of the way! It's going to blow!'",
    desc = "At the end of your turn, ignite all {drusk_01_card} in your hand and {EXPEND} this card.",
    desc_fn = function(self, fmt_str)
        local ignite_damage = self.ignite_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
        return loc.format(fmt_str, ignite_damage)
    end,
    
    ignite_damage = {2,3,3,4},

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.NPC,

    flags = battle_defs.CARD_FLAGS.STATUS | battle_defs.CARD_FLAGS.UNPLAYABLE,
    target_type = TARGET_TYPE.SELF,

    event_priorities =
    {
        [ BATTLE_EVENT.END_PLAYER_TURN ] = 11, -- after skeletal suspension
    },

    event_handlers =
    {
        [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
            self:NotifyTriggered()
            local ignited = false
            for k,v in pairs(battle:GetHandDeck().cards) do
                if v.id == "drusk_01_card" then
                    v.ignited = true
                    ignited = true
                end
            end
            if ignited then
                battle:PlayCards( battle:GetHandDeck(), function(card) return card.id == "drusk_01_card" end )
            end
            battle:ExpendCard(self)
        end
    },
})


