local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local FLAGS = battle_defs.CARD_FLAGS

Content.AddCharacterDef
(
    CharacterDef("ERCHIN",
    {
        base_def = "MONSTER",

        title = "Erchin",
        build = "erchin",

        HAIR_PALETTE =
        {
            0xEC8455FF,
            0xB04C4CFF,
            
            
            0x963867FF,
        },

         SKIN_PALETTE =
        {
            0x50314AFF,
            0x434E83FF,
            0x4E7F7BFF,            
        },

        death_item = "erchin_musk",
        support_negotiation = "untrained_pet",
        trained_support_negotiation = "trained_pet",
        voice_event = "event:/vo/compendium/speak_erchin",

        fight_data = 
        {
            MAX_HEALTH = 12,
            MAX_MORALE = .5,

            -- Morale for generations AFTER the 0th (which uses MAX_MORALE)
            GENERATION_MAX_MORALE =
            {
                0.3,
                0.15,
            },

            status_widget_head_dy = 2.0,
            status_widget_head_dx = 0.25,

            cards = { "erchin_attack1", "erchin_attack2" },
            attacks = {
                erchin_attack1 = table.extend(NPC_MELEE)
                {
                    name = "Kick",
                    anim = "attack1",

                    flags = battle_defs.CARD_FLAGS.MELEE | battle_defs.CARD_FLAGS.DEBUFF,
                    
                    damage_mult = 0.75,

                    features =
                    {
                        WOUND = 1,
                    },
                },

                erchin_attack2 = table.extend(NPC_MELEE)
                {
                    name = "Headbutt",
                    anim = "attack2",
                },

                erchin_defend = table.extend(NPC_BUFF)
                {
                    name = "Defend",
                    anim = "taunt",

                    target_type = TARGET_TYPE.SELF,

                    flags = battle_defs.CARD_FLAGS.SKILL | battle_defs.CARD_FLAGS.BUFF,

                    features = 
                    {
                        DEFEND = 4,
                    },  
                },
            },

            hit_tags = { "small" },

            behaviour =
            {
                OnActivate = function( self )
                    if not self.fighter:GetAgent():IsInPlayerParty() then
                        self.fighter:AddCondition( "MULTITUDES" )
                    else
                        self.fighter:DeltaMorale(-self.fighter:GetMorale() * .5)
                    end

                    self.fighter:AddCondition( "DISEASE_RIDDEN", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ) )
                    self.moves = self:MakePicker()
                        :AddID( "erchin_attack1", 1 )
                        :AddID( "erchin_attack2", 3 )
                        :AddID( "erchin_defend", 1 )
                    if self.fighter.generation then
                        local morale = self.fighter.agent.fight_data.GENERATION_MAX_MORALE[ self.fighter.generation ]
                        if morale then
                            self.fighter:DeltaMorale( -(self.fighter:GetMorale() - morale ))
                        else
                            self.fighter:DeltaMorale( -self.fighter:GetMorale() )
                        end
                    end
                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    self.moves:ChooseCard()
                end,
            },

            conditions = {
                MULTITUDES = 
                {
                    name = "Multitudes",
                    desc = "If this creature flees, a new Erchin will take its place.",
                    apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",
                    event_handlers =
                    {
                        [ BATTLE_EVENT.STATUS_CHANGED ] = function( self, fighter, status )
                            if fighter == self.owner and status == FIGHT_STATUS.FLED then
                                local agent = Agent( "ERCHIN" )
                                local new_fighter = Fighter.CreateFromAgent( agent, self.owner:GetScale() )
                                new_fighter.generation = (fighter.generation or 0) + 1
                                self.owner:GetTeam():AddFighter( new_fighter, - 1 )
                            end
                        end
                    },  
                },

                DISEASE_RIDDEN = 
                {
                    name = "Disease Ridden",
                    desc = "When {2} causes you to lose health, gain <#HILITE>{1} {DISEASED}</>.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self.stacks, self:GetOwnerName() )
                    end,

                    apply_sound = "event:/sfx/battle/status/system/Status_Buff_Defend",

                    event_handlers = 
                    {
                        [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
                            if self.battle:GetFighterFromSource( source ) == self.owner then
                                fighter:AddCondition("DISEASED", self.stacks, self)
                            end
                        end,
                    },
                },

                DISEASED = 
                {
                    name = "Diseased",
                    desc = "Reduces maximum health by <#HILITE>{1}</> at the end of {2}'s turn until the end of the fight.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self.stacks, self:GetOwnerName() )
                    end,

                    ctype = CTYPE.DEBUFF,
                    
                    CanApply = function( self, source )
                        return not self.owner:HasCondition("METALLIC")
                    end,

                    event_handlers = 
                    {
                        [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                            if self.owner == fighter then
                                self.owner:DeltaMaxHealth(-self.stacks)
                            end
                        end,
                    },
                },
            },
        },
    })
)

Content.AddCharacterDef
(
    CharacterDef("ERCHIN_UPGRADED",
    {
        base_def = "ERCHIN",

        build = "upgraded_erchin",

        fight_data = 
        {
            MAX_HEALTH = 24,
            MAX_MORALE = MAX_MORALE_LOOKUP.MEDIUM,

            status_widget_head_dy = 2.0,
            status_widget_head_dx = 0.25,

            cards = { "erchin_upgraded_attack1", "erchin_upgraded_attack2" },
            attacks = {
                erchin_upgraded_attack1 = table.extend(NPC_MELEE)
                {
                    name = "Kick",
                    anim = "attack1",
                    
                    damage_mult = 0.8,

                    features =
                    {
                        WOUND = 1,
                    },
                },

                erchin_upgraded_attack2 = table.extend(NPC_MELEE)
                {
                    name = "Headbutt",
                    anim = "attack2",
                    damage_mult = 1.2,
                },

                erchin_upgraded_defend = table.extend(NPC_BUFF)
                {
                    name = "Defend",
                    anim = "taunt",

                    target_type = TARGET_TYPE.SELF,

                    flags = battle_defs.CARD_FLAGS.SKILL | battle_defs.CARD_FLAGS.BUFF,

                    features = 
                    {
                        DEFEND = 4,
                    },  
                },
            },

            hit_tags = { "small" },

            behaviour =
            {
                OnActivate = function( self )
                    self.fighter:AddCondition("ARMOURED", 4, self)
                    self.fighter:DeltaMorale(-self.fighter:GetMorale() * .5)

                    self.fighter:AddCondition( "DISEASE_RIDDEN", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ))
                    self.moves = self:MakePicker()
                        :AddID( "erchin_upgraded_attack1", 1 )
                        :AddID( "erchin_upgraded_attack2", 3 )
                        :AddID( "erchin_upgraded_defend", 1 )
                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    self.moves:ChooseCard()
                end,
            },
        },
    })
)
