local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

require "content/characters/base_types"
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

----------------------------------------------------
-- Anyone a citizen of the Feud.

Content.AddCharacterDef
(
	CharacterDef("FEUDALITE",
	{
		base_def = "NPC_BASE",
	    is_template = true,

		faction_id = "FEUD_CITIZEN",

		social_boons =
		{
			[ RELATIONSHIP.LIKED ] = { GIVE_ITEM = 1 },
			[ RELATIONSHIP.LOVED ] = { GIVE_ITEM = 1, SOCIALIZE = 3, HIRE = 2 },
		},
	})
)


----------------------------------------------------


Content.AddCharacterDef
(
	CharacterDef("FOREMAN",
	{
		base_def = "FEUDALITE",
		
		title = "Foreman",
		renown = 2,
		combat_strength = 2,

		loved_graft = "authority",
		hated_graft = "blacklisted",

		voice_actors = {
		    [GENDER.MALE] = { "civilianHighClassMale01" },
    		[GENDER.FEMALE] = { "civilianHighClassFemale01" },
		},

    	death_money = DEATH_MONEY_MEDIUM,
    	death_item = "adrenaline_shot",

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,

	        status_widget_head_dx = 1.0,
	        status_widget_head_dy = 5.5,

			cards = { "blaster_barrage", "whip" },

			attacks = 
			{
				foreman_reload = table.extend(NPC_BUFF)
				{
					name = "Reload",
					anim = "reload",
					CanPlayCard = function(self, battle)
                        return self.owner:HasCondition("foreman_empty")
                    end,
					target_type = TARGET_TYPE.SELF,
					flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,

					OnPostResolve = function( self, battle, attack )
						if self.owner:HasCondition("foreman_empty") then
							self.owner:RemoveCondition("foreman_empty")
						end
					end,
				},

				foreman_blaster_barrage = table.extend(NPC_ATTACK)
			    {
			        name = "Blaster Barrage",
			        anim = "spray",
			        flags = CARD_FLAGS.RANGED,

			        CanPlayCard = function(self, battle)
                        return not self.owner:HasCondition("foreman_empty")
                    end,

			        target_type = TARGET_TYPE.ENEMY,

			        hit_count = 3,

			        damage_mult = 0.4,
			        OnPostResolve = function( self, battle, attack )
						self.owner:AddCondition("foreman_empty")
					end,
			    },
			},

			conditions = 
			{
				foreman_empty = 
				{
					name = "Out of Ammo",
					desc = "{1} is out of ammo and can't fire their gun until they reload.",
					desc_fn = function( self, fmt_str )
						return loc.format(fmt_str, self:GetOwnerName())
					end,
					ctype = CTYPE.DEBUFF,
				},
			},

			behaviour =
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID( "whip", 1 )
						:AddID( "foreman_blaster_barrage", 1 )
						:AddID("foreman_reload", 1)
					self:SetPattern( self.Cycle )
				end,

				Cycle = function( self )
					self.attacks:ChooseCard()
				end,
			},
		},

		support_negotiation = "foreman_support",

		negotiation_data =
		{
			modifiers =
			{
				GRINDSTONE = 
				{
					name = "Grindstone",

					min_persuasion = 2,
					max_persuasion = 2,
					max_stacks = 1,
					target_enemy = TARGET_ANY_RESOLVE,
					modifier_type = MODIFIER_TYPE.ARGUMENT,

					OnInit = function( self )
						self:SetResolve( 1, MODIFIER_SCALING.LOW )
					end,

			        OnBeginTurn = function( self, minigame )
			            self:ApplyPersuasion()
			        end,
				},
				GRINDSTONE_plus =
				{
					desc = "This argument only targets your core argument.",
					target_enemy = TARGET_FLAG.CORE,

					min_persuasion = 3,
					max_persuasion = 3,
				},

				DEADLINE =
				{
					name = "Deadline",
					desc = "When destroyed, {1}'s core argument gains {2} {COMPOSURE}.",
					desc_fn = function( self, fmt_str )
						return loc.format( fmt_str, self:GetOpponentName(),self.stacks * self.bounty )
					end,
			        sound = "event:/sfx/battle/cards/neg/create_argument/tithe",
			        modifier_type = MODIFIER_TYPE.BOUNTY,
			        bounty = 10,

			        OnInit = function( self )
			            self:SetResolve( 1, MODIFIER_SCALING.LOW )
			        end,

			        OnBounty = function( self )
			        	local core = self.anti_negotiator:FindCoreArgument()
			        	if core then
			        		core:DeltaComposure( self.stacks * self.bounty, self )
			        	end
			        end,
				}
			},
			behaviour =
			{
				OnInit = function( self, difficulty )
					self.grindstone = self:AddArgument( "GRINDSTONE" )
					self.deadline = self:AddArgument( "DEADLINE" )
					self:SetPattern( self.Cycle )
					self.negotiator:AddModifier("DELEGATOR")
				end,

				Cycle = function( self, turn )
					if turn % 3 == 0 then
						self:ChooseComposure( 1, self.difficulty * 2, self.difficulty * 2 )
					end
					self:ChooseGrowingNumbers( 1, 0 )
					if (turn+1) % 5 == 0 then
						self:ChooseCard( self.deadline )
					else
						self:ChooseCard( self.grindstone )
					end
				end,
			}
		},

		base_builds = {
			[ GENDER.MALE ] = "teamster_male",
			[ GENDER.FEMALE ] = "teamster_female"
		},
		anims = { "anim/weapon_blaster_foreman.zip"},
		combat_anims = { "anim/med_combat_blaster_foreman.zip" },

		tags = { "civilian", "foreman", "wealthy" },
	})
)


Content.AddCharacterDef
(
	CharacterDef("BARTENDER",
	{
		designer_note = "Loads up their old trusty rifle and unloads it on enemies, sometimes doing a ton of burst damage.",
		base_def = "FEUDALITE",

		title = "Bartender",
		renown = 3,
		combat_strength = 3,

		voice_actors = {
		    [GENDER.MALE] = { "civilianHighClassMale01" },
    		[GENDER.FEMALE] = { "civilianHighClassFemale01" },
		},

		loved_graft = "mixology",
		hated_graft = "tainted_supply",

    	death_money = DEATH_MONEY_HIGH,
    	death_item = "liquid_courage",

		tags = { "civilian", "bartender", "wealthy" },
		base_builds = {
			[ GENDER.MALE ] = "shopkeep_male_jarackle",
			[ GENDER.FEMALE ] = "shopkeep_female_jarackle"
		},

		social_boons =
		{
			[ RELATIONSHIP.LIKED ] = { SOCIALIZE = 1, SPECIAL_NOODLES = 1 },
			[ RELATIONSHIP.LOVED ] = { SPECIAL_NOODLES = 1 },
		},

		support_negotiation = "bartender_support",

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,

			cards = { "ai_defend_med" },

			conditions = 
			{
			    ammo =
			    {
			        name = "Ammo",
			        desc = "{1}'s attacks require ammo.",
			        desc_fn = function( self, fmt_str )
			        	return loc.format( fmt_str, self.owner.agent:GetName())
			    	end,

			        target_type = TARGET_TYPE.ENEMY,

			        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",
			    },
			},

			attacks = 
			{
			    blaster_blast = table.extend(NPC_ATTACK)
			    {
			        name = "Blast",
			        pre_anim = "blast_pre",
			        post_anim = "blast_pst",
			        anim = "blast",
			        base_damage = 2,

			        flags = CARD_FLAGS.RANGED,
			        target_mod = TARGET_MOD.RANDOM1,
			        quip = "bartender_blaster_blast",

			        CanPlayCard = function(self, battle, target)
			        	return self.owner:GetConditionStacks("ammo") >= 2
			       	end,

			        OnPostResolve = function( self, battle, attack)
			        	self.owner:RemoveCondition( "ammo" )
			        end,

			        deck_handlers = { DECK_TYPE.AI, DECK_TYPE.RESOLVE },

			        event_handlers = 
			        {
						[ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
							if condition.owner == self.owner and condition.id == "ammo" then
								self.target_count = condition.stacks
							end
						end,

					    [ BATTLE_EVENT.CONDITION_REMOVED ] = function( self, fighter, condition, stacks, source )
							if condition.owner == self.owner and condition.id == "ammo" then
								self.target_count = condition.stacks
							end
						end,
			        },
			    },

			    blaster_aim = table.extend(NPC_BUFF)
			    {
			        name = "Reload",
			        anim = "taunt",
			        target_type = TARGET_TYPE.SELF,
			        flags = CARD_FLAGS.BUFF,

			        GetAmmoAmt = function(self, fighter)
			        	local idx = fighter:GetScale() + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )
			        	return self.ammo_amt[ math.min( idx, #self.ammo_amt ) ]
				    end,

			        ammo_amt = {3, 4, 5, 5, 5},

			        OnPostResolve = function( self, battle, attack)
			            self.owner:AddCondition( "ammo", self:GetAmmoAmt(self.owner) )
			        end,
			    },

			    blaster_headshot = table.extend(NPC_ATTACK)
			    {
			        name = "Headshot",
			        anim = "head_shot",

			        damage_mult = 1.25,

			        flags = CARD_FLAGS.RANGED | CARD_FLAGS.PIERCING,

			        OnPostResolve = function( self, battle, attack)
		                self.owner:RemoveCondition( "ammo", 1 )
			        end,
			    },
			},

			behaviour =
			{
				OnActivate = function( self, fighter )
					self.reload = self:AddCard( "blaster_aim" )
					self.attacks = self:MakePicker()
						:AddID( "ai_power_boost", 1 )
						:AddID( "blaster_blast", 2 )
						:AddID( "blaster_headshot", 2, 1 )
						:AddID( "ai_shoot", 2 )

					self:SetPattern( self.AttackPattern )
					fighter:AddCondition("ammo", self.reload:GetAmmoAmt(fighter) )
				end,

				AttackPattern = function( self )
					if self.fighter:GetConditionStacks("ammo") > 0 then
						self.attacks:ChooseCard()
					else
						self:ChooseCard( self.reload )
					end
				end,
			}   
		},
	    anims = { "anim/weapon_rifle_bartender.zip"},
    	combat_anims = { "anim/med_combat_rifle_bartender.zip" },

		negotiation_data = 
		{
			modifiers =
			{
			    ON_THE_HOUSE =
			    {
			        name = "On The House",
			        desc = "Gain 2 {COMPOSURE} whenever you play a Manipulate card.",

			        modifier_type = MODIFIER_TYPE.CORE,

			        event_handlers =
			        {
			            [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
			                if card:GetNegotiator() == self.engine:GetPlayerNegotiator() and CheckBits( card.flags, negotiation_defs.CARD_FLAGS.MANIPULATE) then
			                	self.negotiator:DeltaComposure( 2, self )
			                end
			            end,
			        },
			    },
			},

			behaviour =
			{
				OnInit = function( self, difficulty )
					self.rumour = self:AddArgument( "RUMOR_MONGER" )

					self.bounty = self:AddArgument( "CHASER" )

					self:SetPattern( self.Cycle )

					self.intuition = self:AddArgument( "intuition" )
					self.negotiator:AddModifier("ON_THE_HOUSE")
				end,

				Cycle = function( self, turn )
					if (turn - 1) % 6 == 0 and not self.negotiator:HasModifier( "intuition" ) then
						self:ChooseCard( self.intuition )
					else
						self:ChooseComposure( 1, self.difficulty, self.difficulty )
					end

					if turn == 2 or turn == 5 or turn == 11 then
						self:ChooseCard( self.bounty )
					end

					if self.negotiator:HasModifier( "intuition" ) then
						self:ChooseGrowingNumbers( 1, 0, 0.75 )
					else
						self:ChooseGrowingNumbers( 1, 2 )
					end
				end,
			}
		},
	})
)

local MERCHANT_SUMMON_DRONE_CHANCE = { 1, .5, .25, 0}


local MERCHANT_FIGHT = 
{
    MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW,
    MAX_MORALE = MAX_MORALE_LOOKUP.VERY_LOW,

    status_widget_head_dx = 0.5,
    status_widget_head_dy = 5.2,

	cards = { "spark_canister", "gunsha_powder" },

	attacks = 
	{
	    spark_canister = table.extend(NPC_ATTACK)
	    {
	        name = "Spark Canister",
	        anim = "spark_canister",

	        flags = CARD_FLAGS.RANGED,
	        target_mod = TARGET_MOD.TEAM,

	        damage_mult = 1,
	    },
	    spark_canister_2 = table.extend(NPC_ATTACK)
	    {
	        name = "Overcharged Spark Canister",
	        anim = "spark_canister",

	        flags = CARD_FLAGS.RANGED,
	        target_mod = TARGET_MOD.TEAM,
	        damage_mult = 1.5,
	    },

	    gunsha_powder = table.extend(NPC_ATTACK)
	    {
	        condition = 
	        {
	            name = "Stinging Eyes",
	            desc = "All your attacks have random targets. Reduce <b>Stinging Eyes</b> by 1 at the end of your turn.",
	            hud_fx = {"stunned"},

	            ctype = CTYPE.DEBUFF,

	            CanApply = function( self, source )
		            return self.owner:IsPlayer()
		        end,

	            event_handlers = 
	            {
	                [ BATTLE_EVENT.CALC_TARGET_MOD ] = function( self, acc, card, target )
	                    if card.owner == self.owner and card:IsAttackCard() and self.battle:GetCurrentTeam() == self.owner:GetTeam() and acc.value ~= TARGET_MOD.TEAM and acc.value ~= TARGET_MOD.ALL then
	                        acc:ModifyValue( TARGET_MOD.RANDOM1, self )
	                    end
	                end,

		            [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
		                if self.owner == fighter then
		                    self.owner:RemoveCondition( self.id, 1)
		                end
		            end
    			}
	        },

	        name = "Gunsha Powder",
	        anim = "gunsha_powder",

	        CanPlayCard = function(self, battle, target)
	        	if target then
		            return target:IsPlayer()
		        else
		        	return true
		        end
	        end,

	        flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,

	        OnPostResolve = function(self, battle, attack)
	            attack:AddCondition(self.id, 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ))
	        end,
	    },

	    oshnu_vapors = table.extend(NPC_ATTACK)
	    {
	        name = "Oshnu Vapors",
	        anim = "gunsha_powder",

	        flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,

	        OnPostResolve = function(self, battle, attack)
                if attack.target.player_controlled then
                	local num_cards = 2 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )
                	local cards = {}
                	for i = 1, num_cards do
                		table.insert( cards, Battle.Card( "drunk", attack.target ))
                	end
                    battle:DealCards( cards, battle:GetDiscardDeck() )
                else
                    attack.target:AddCondition( "IMPAIR", 1 )
                end
	        end,
	    },

	},

	behaviour =
	{
		OnActivate = function( self )

			self.big_attacks = self:MakePicker()
				:AddID( "spark_canister_2", 2 )
				:AddID( "oshnu_vapors", 2 )
				:AddID( "ai_defend_med", 1 )
			
			self.attacks = self:MakePicker()
				:AddID( "spark_canister", 2 )
				:AddID( "oshnu_vapors", 1 )
				:AddID( "ai_defend_med", 1 )

			self.gunsha_powder = self:AddCard("gunsha_powder")
			self:SetPattern( self.Cycle )
		end,

		Cycle = function( self )
			local turns = self.fighter:GetBattle():GetTurns()

			local player = self.battle:GetPlayerFighter()
			if player and player:GetConditionStacks("gunsha_powder") == 0 
				and self.fighter:GetTeam():NumFighters() > 1 
				and self.fighter:GetTeam() ~= self.battle:GetPlayerFighter():GetTeam() and
				math.random() < 0.65 then
				--
				self:ChooseCard( self.gunsha_powder )
			else
				if turns > 3 then
					self.big_attacks:ChooseCard()
				else
					self.attacks:ChooseCard()
				end
			end
		end
	}
}

local MERCHANT_NEGOTIATION = 
{
	behaviour =
	{
		OnInit = function( self )
			self.bounty = self:AddArgument( "TRIBUTE" )

			self.money_talks = self:AddArgument( "MONEY_TALKS" )

			self.swindle = self:AddCard( "swindle" )
			if self.difficulty >= 4 then
				self.swindle.count = 2
			end
			self.negotiator:AddModifier("PROPOSITION")
			self:SetPattern( self.Cycle )
		end,

		Cycle = function ( self, turns )
			if turns == 1 then
				if self.difficulty >= 2 then
					self:ChooseCard( self.money_talks )
				end
				self:ChooseGrowingNumbers( 1, 0 )

			elseif turns == 2 or turns == 6 then
				self:ChooseCard( self.bounty )
				self:ChooseGrowingNumbers( 1, -1 )

			elseif turns % 3 == 0 or turns % 5 == 0 then
				if math.random() < 0.5 then
					self:ChooseCard( self.swindle )
					self:ChooseGrowingNumbers( 1, -1 )
				else
					self:ChooseGrowingNumbers( 2, 0, 0.8 )
				end
			else
				self:ChooseGrowingNumbers( 2, 0, 0.6 )
				self:ChooseComposure( 1, self.difficulty * 2, self.difficulty * 2 )
			end
		end,
	},
}

Content.AddCharacterDef
(
	CharacterDef("WEALTHY_MERCHANT",
	{
		designer_note = "Makes you attack random targets all fight, and then slowly makes you drunk",
		base_def = "FEUDALITE",

		title = "Wealthy Merchant",

		tags = { "merchant", "wealthy" },

		loved_graft = "haggle_badge",
		hated_graft = "bad_credit",

		promotion_def = "WEALTHY_MERCHANT_PROMOTED",

		renown = 3,
		combat_strength = 2,
	    negotiation_ally_image = engine.asset.Texture( "negotiation/modifiers/recruit_civilian_wealthy_merchant.tex"),

		voice_actors = {
		    [GENDER.MALE] = { "civilianHighClassMale01" },
    		[GENDER.FEMALE] = { "civilianHighClassFemale01" },
		},

	    death_money = DEATH_MONEY_VERY_HIGH,
	    death_item = "spark_grenade",

		support_negotiation = "deep_pockets",

		social_boons =
		{
			[ RELATIONSHIP.LIKED ] = { UPGRADE_NEGOTIATION_CARD = 1, GIVE_ITEM = 1, SOCIALIZE = 1 },
			[ RELATIONSHIP.LOVED ] = { UPGRADE_NEGOTIATION_CARD = 1, GIVE_ITEM = 1 },
		},

		base_builds = {
			[ GENDER.MALE ] = "merchant_male_rich",
			[ GENDER.FEMALE ] = "female_shroke_rich_merchant"
		},

		anims = { "anim/weapon_gunsha_powder_wealthy_merchant.zip","anim/weapon_spark_canister_wealthy_merchant.zip"},

		combat_anims = { "anim/med_combat_wealthy_merchant.zip" },

		negotiation_data = MERCHANT_NEGOTIATION,
		fight_data = MERCHANT_FIGHT,

	})
)

Content.AddCharacterDef
(
	CharacterDef("WEALTHY_MERCHANT_PROMOTED",
	{
		base_def = "WEALTHY_MERCHANT",

		title = "Promoted Wealthy Merchant",

		renown = 4,
		combat_strength = 3,

	    death_money = DEATH_MONEY_VERY_HIGH,
	    death_item = "lumin_grenade",

		support_negotiation = "deep_pockets",

		base_builds = {
			[ GENDER.MALE ] = "male_merchant_promoted_build",
			[ GENDER.FEMALE ] = "female_merchant_promoted_build"
		},

		anims = { "anim/weapon_crossbow_promoted_merchant.zip"},

		combat_anims = { "anim/med_combat_crossbow_promoted_merchant.zip" },

		negotiation_data = MERCHANT_NEGOTIATION,
		fight_data = 
		{
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,
		    MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
		    anim_mapping =
			{
				riposte = "shoot",
			},
			ranged_riposte = true,
			conditions =
			{
				cha_ching =
				{
					name = "Cha-ching!",
					desc = "Whenever you attack {1}, gain {2#money}. At the start of {1}'s turn, remove this.",
					desc_fn = function( self, fmt_str )
						return loc.format( fmt_str, self:GetOwnerName(), self.stacks )
					end,

					event_handlers =
					{
						[ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
				            if hit.target == self.owner and attack.attacker and attack.attacker == battle:GetPlayerFighter() and attack.attacker:IsPlayer() then
				                attack.attacker.agent:DeltaMoney( self.stacks )
				                TheGame:GetGameState():GetCaravan():AddMoney( self.stacks )
			                    self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_FX, "steal_money" )
			                    -- self.owner:BroadcastEvent( BATTLE_EVENT.PLAY_FLOATER, self:GetName(), 0.8, UICOLOURS.MONEY )

                                attack.target:BroadcastEvent( BATTLE_EVENT.DELTA_MONEY, -self.stacks )
				            end
				        end,

				        [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
				        	if fighter == self.owner then
				        		self.owner:RemoveCondition(self.id, self.stacks, self)
				        	end
				        end
					},
				}
			},
			attacks = 
			{
			    wealthy_merchant_single = table.extend(NPC_ATTACK)
			    {
			        name = "Shoot",
			        anim = "shoot",

			        flags = CARD_FLAGS.RANGED,

			        damage_mult = 1,
			    },
			    wealthy_merchant_multi = table.extend(NPC_ATTACK)
			    {
			        name = "Multi-shot",
			        anim = "shoot_multiple",

			        flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
			        target_mod = TARGET_MOD.TEAM,
			        damage_mult = 0.6,

			        features = 
			        {
			        	IMPAIR = 1,
			        },
			    },
			    wealthy_merchant_taunt = table.extend(NPC_BUFF)
			    {
			        name = "Taunt",
			        anim = "taunt",

			        flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
			        target_type = TARGET_TYPE.SELF,
			        cha_ching_amt = { 8, 10, 15, 20, 25 },
			        defend_amt = { 4, 5, 6, 7, 8 },
			        riposte_amt = { 3, 3, 4, 4, 5 },
			        OnPostResolve = function( self, battle, attack )
			        	local scale = self.owner:GetScale()
			        	if self.owner:GetTeam() == battle:GetEnemyTeam() then
			        		self.owner:AddCondition("cha_ching", self.cha_ching_amt[math.min(#self.cha_ching_amt, scale)], self)
			        	end
			        	self.owner:AddCondition("DEFEND", self.defend_amt[math.min(#self.defend_amt, scale)], self)
			        	self.owner:AddCondition("RIPOSTE", self.riposte_amt[math.min(#self.riposte_amt, scale)], self)
			        end
			    },
			},

			behaviour =
			{
				OnActivate = function( self )					
					self.attacks = self:MakePicker()
						:AddID( "wealthy_merchant_single", 2 )
						:AddID( "wealthy_merchant_multi", 1 )
						:AddID( "wealthy_merchant_taunt", 1 )

					self:SetPattern( self.Cycle )
				end,

				Cycle = function( self )
					self.attacks:ChooseCard()
				end
			}
		},

	})
)

Content.AddCharacterDef
(
	CharacterDef("POOR_MERCHANT",
	{
		
		title = "Poor Merchant",
		tags = { "merchant"},
		
		base_def = "FEUDALITE",
		base_builds = {
			[ GENDER.MALE ] = "merchant_male",
			[ GENDER.FEMALE ] = "merchant_female"
		},

		loved_graft = "haggle_badge",
		hated_graft = "bad_credit",

		renown = 1,
		combat_strength = 2,

		voice_actors = {
		    [GENDER.MALE] = { "civilianLowClassMale01" },
			[GENDER.FEMALE] = { "civilianLowClassFemale01" },
		},

	    death_money = DEATH_MONEY_MEDIUM,
	    death_item = "defective_grenade",

		support_negotiation = "deep_pockets",

		social_boons =
		{
			[ RELATIONSHIP.LIKED ] = { UPGRADE_NEGOTIATION_CARD = 1, GIVE_ITEM = 1, SOCIALIZE = 1 },
			[ RELATIONSHIP.LOVED ] = { UPGRADE_NEGOTIATION_CARD = 1, GIVE_ITEM = 1 },
		},	


		anims = { "anim/weapon_sling_poor_merchant.zip"},

		combat_anims = { "anim/med_combat_sling_poor_merchant.zip" },

		negotiation_data =
		{
			behaviour =
			{
				OnInit = function( self )
					self.negotiator:AddModifier( "PENNY_PINCHER" )
					self.bounty = self:AddArgument( "bonus_payment" )

					self:SetPattern( self.Cycle )
				end,

				Cycle = function ( self, turns )
					if turns == 2 or turns == 6 then
						self.bounty.stacks = math.random( 5, 20 )
						self:ChooseCard( self.bounty )
					end

					if turns % 3 == 1 then
						self:ChooseGrowingNumbers( 2, -1 )
					else
						self:ChooseComposure( 1, self.difficulty * 2, self.difficulty * 2 )
						self:ChooseGrowingNumbers( 1, 1 )
					end
				end,
			},
		},

		fight_data = 
		{
			MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW,
		    MAX_MORALE = MAX_MORALE_LOOKUP.VERY_LOW,

		    ranged_riposte = true,
		    anim_mapping =
			{
				riposte = "throw",
			},

			attacks = 
			{	
				sling_throw = table.extend(NPC_ATTACK)
				{
					name = "Throw",
					anim = "throw",
					flags = CARD_FLAGS.RANGED,
					damage_mult = 0.4,
				},

				sling_swing = table.extend(NPC_ATTACK)
				{
					name = "Upswing",
					anim = "upswing",
					flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,
					damage_mult = 0.8,

					features =
					{
						WOUND = 2,
					}
				},

				sling_taunt = table.extend(NPC_BUFF)
				{
					name = "Taunt",
					anim = "taunt",
					flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
					target_type = TARGET_TYPE.SELF,

					features = 
					{
						DEFEND = 5,
						RIPOSTE = 3,
					},
				}
			},

			conditions =
			{
				PELTED = 
				{
					name = "Pelted",
					desc = "{1}'s attacks apply <#HILITE>{2}</> {IMPAIR} if they cause health loss.",
					desc_fn = function( self, fmt_str )
						return loc.format( fmt_str, self:GetOwnerName(), self.stacks or 1 )
					end,

					event_handlers =
					{
						[ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
			                if hit.attacker == self.owner and not hit:CheckHitResult( "defended" ) and not hit:CheckHitResult( "evaded" ) and hit.card:IsDamageCard() then
			                    hit.target:AddCondition( "IMPAIR", self.stacks, self )
			                end
			            end,
					},
				},
			},

			behaviour =
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID("sling_throw", 2)
						:AddID("sling_swing", 1)
						:AddID("sling_taunt", 1)

					self:SetPattern( self.Cycle )
					self.fighter:AddCondition("PELTED")
				end,

				Cycle = function( self )
					self.attacks:ChooseCard()
				end
			}
		}

	})
)

----------------------------------------------------

local LABORER_FIGHT_BEHAVIOUR =
{
	elite = false,
	OnActivate = function( self )
		self.attacks = self:MakePicker()
			:AddID( "thwack", 2 )
			:AddID( "laborer_concussion", 1 )
			:AddID( "gather", 1 )

		self.elite_attacks = self:MakePicker()
			:AddID( "laborer_smash", 1)
			:AddID( "laborer_concussion", 1 )
			:AddID( "gather", 1 )

		self:SetPattern( self.elite and self.EliteCycle or self.Cycle )
	end,

	Cycle = function( self )
		self.attacks:ChooseCard()
	end,

	EliteCycle = function( self )
		self.elite_attacks:ChooseCard()
	end
}
----------------------------------------------------
Content.AddCharacterDef
(
	CharacterDef("LABORER",
	{
		designer_note = "Just a basic attack and power buff cycle",
		base_def = "FEUDALITE",

		title = "Laborer",
		tags = { "civilian", "laborer", "mercenary" },

		renown = 1,
		combat_strength = 1,

		loved_graft = "voice_of_the_people",
		hated_graft = "elitist",
		death_item = "anchor",
		support_negotiation = "stoic",
		promotion_def = "LABORER_PROMOTED",

		voice_actors = {
		    [GENDER.MALE] = { "civilianLowClassMale01" },
    		[GENDER.FEMALE] = { "civilianLowClassFemale01" },
		},

    	death_money = DEATH_MONEY_LOW,

		base_builds = {
			[ GENDER.MALE ] = "foundry_worker_male",
			[ GENDER.FEMALE ] = "foundry_worker_female"
		},

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.VERY_LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW,

	        status_widget_head_dx = 1.0,
	        status_widget_head_dy = 4.6,

			cards = { "thwack", "laborer_concussion", "gather" },

			attacks = 
			{
			    thwack = table.extend(NPC_ATTACK)
			    {
			        name = "Thwack",
			        anim = "gut_smash",
			        flags = CARD_FLAGS.MELEE,
			        damage_mult = .8,
			    },

			    gather = table.extend(NPC_BUFF)
			    {
			        name = "Gather",
			        anim = "taunt",
			        quip = "laborer_gather",
			        target_type = TARGET_TYPE.SELF,
			        flags = CARD_FLAGS.BUFF,

			        OnPostResolve = function( self, battle, attack )
			        	attack:AddCondition( "POWER", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ) )
			        end,

			        features =
			        {
			            DEFEND = 5,
			        }
			    },

			    laborer_concussion = table.extend(NPC_ATTACK)
			    {
			        name = "Concussion",
			        anim = "smack",
			        flags = CARD_FLAGS.MELEE,

			        damage_mult = 1.2,
			    },

			    laborer_smash = table.extend(NPC_ATTACK)
			    {
			    	name = "Smash",
			    	anim = "smash",
			    	flags = CARD_FLAGS.MELEE,

			    	damage_mult = 1.3,
			    },
			},

			behaviour = table.extend(LABORER_FIGHT_BEHAVIOUR)
			{
				
			},
		},

		anims = {"anim/weapon_club_common.zip"},
		combat_anims = { "anim/med_combat_club_laborer.zip" },

		negotiation_data = 
		{
			behaviour =
			{
				OnInit = function( self, difficulty )
					self.bounty = self:AddArgument( "bone_tired" )

					self.suspicion = self:AddArgument( "SUSPICION" )
					self.suspicion.stacks = 1 + math.ceil( difficulty / 2 )

					self.buffs = self:MakePicker()
					self.buffs:AddCard( self.suspicion, 1 )

					self:SetPattern( self.Cycle )
					self.negotiator:AddModifier("DEFENSIVE")
				end,

				Cycle = function( self, turns )
					if turns % 3 == 0 then
						self:ChooseCard( self.bounty )
					end

					if (turns-1) % 5 == 0 then
						self.buffs:ChooseCards( 1 )

					elseif turns % math.max( 2, 5 - self.difficulty ) == 0 then
						self:ChooseComposure( 1, 2 + self.difficulty, 2 + self.difficulty )
					end

					self:ChooseGrowingNumbers( 1, 1 )
				end
			}
		},

	})
)


Content.AddCharacterDef
(
	CharacterDef("LABORER_PROMOTED",
	{
		base_def = "LABORER",
		title = "Promoted Laborer",

		death_item = "anchor_plus",

		 base_builds = {
		 	[ GENDER.MALE ] = "male_labourer_promoted_build",
		 	[ GENDER.FEMALE ] = "female_labourer_promoted_build"
		 },

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.VERY_LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW*1.25,

	        status_widget_head_dx = 1.0,
	        status_widget_head_dy = 4.6,

			cards = { "laborer_concussion", "gather", "laborer_smash" },

			behaviour = table.extend(LABORER_FIGHT_BEHAVIOUR)
			{
				elite = true
			},
		},

		anims = {"anim/weapon_club_promoted_laborer.zip"},
		combat_anims = { "anim/med_combat_club_promoted_laborer.zip" },
	})
)
----------------------------------------------------


--------------------------------------------------

Content.AddCharacterDef
(
	CharacterDef("HEAVY_LABORER",
	{
		designer_note = "Gut punches you, making you winded. Stuns other enemies.",
		base_def = "FEUDALITE",

		title = "Heavy Laborer",
		renown = 1,
		combat_strength = 2,

		support_negotiation = "stoic",

		loved_graft = "heavy_lifting",
		hated_graft = "whispered_rumors",

		voice_actors = {
		    [GENDER.MALE] = { "civilianLowClassMale01" },
    		[GENDER.FEMALE] = { "civilianLowClassFemale01" },
		},

    	death_money = DEATH_MONEY_LOW,
    	death_item = "workers_gloves",

		
		tags = { "civilian", "laborer", "mercenary" },
		
		base_builds = {
			[ GENDER.MALE ] = "lumin_worker_male",
			[ GENDER.FEMALE ] = "lumin_worker_female"
		},
		anims = { "anim/weapon_knuckles_lumin_worker.zip"},
		combat_anims = { "anim/med_combat_knuckles_lumin_worker.zip" },
	    negotiation_ally_image = engine.asset.Texture( "negotiation/modifiers/recruit_civilian_heavy_laborer.tex"),

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.VERY_LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW*1.25,

	        status_widget_head_dx = 1.0,
	        status_widget_head_dy = 4.6,

			conditions = 
			{
			    worn_out =
			    {
			        name = "Worn Out",
			        desc = "Remove 1 stack when attacked. If <b>Worn Out</> reaches 0, apply 1 {STUN}.",

        			apply_sound = "event:/sfx/battle/status/system/Status_Debuff_Other",

			        event_handlers =
			        {
			            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
			                if hit.target == self.owner and attack.card:IsAttackCard() then
			                    self.owner:RemoveCondition( self.id, 1 )

				                if self.owner:GetConditionStacks( self.id ) == 0 then
				                	self.owner:AddCondition("STUN", 1)
				                end
				            end
			            end 
			        },
			    },


			},

			attacks = 
			{
			    ai_hard_labourer_basic_attack = table.extend(NPC_ATTACK)
			    {
			        name = "Smack",
			        anim = "smack",
			        flags = CARD_FLAGS.MELEE,

			        damage_mult = 1.2,
			    },

			    heavy_labourer_gut_smash = table.extend(NPC_ATTACK)
			    {
			        name = "Gut Smash",
			        anim = "smash",
			        flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

			        damage_mult = 0.4,

			        OnPostResolve = function( self, battle, attack )
						if not attack:CheckHitResult( attack.target, "evaded" ) and not attack:CheckHitResult( attack.target, "defended" ) then
			                if attack.target:IsPlayer() then
			                    local cards = {
			                        Battle.Card( "status_winded", attack.target ),
			                        Battle.Card( "status_winded", attack.target )
			                    }
			                    battle:DealCards( cards, battle:GetDiscardDeck() )
			                else
			                    attack.target:AddCondition( "STUN", 1 )
			                end
			            end
			        end,
			    },

			},

			behaviour =
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID( "heavy_labourer_gut_smash", 1 )
						:AddID( "ai_hard_labourer_basic_attack", 2 )
					self:SetPattern( self.Cycle )
				end,

				Cycle = function( self )
					if self.fighter:GetConditionStacks("STUN") == 0 and self.fighter:GetConditionStacks("worn_out") == 0 then
						local stacks = 5 + 2 * GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )
						self.fighter:AddCondition( "worn_out", stacks )
					end

					self.attacks:ChooseCard()
				end
			}
		},

		negotiation_data = 
		{
			behaviour =
			{
				OnInit = function( self, difficulty )
					self.bounty = self:AddArgument( "bone_tired" )

					self.irritable = self:AddArgument( "IRRITABLE" )
					self.irritable.stacks = 1

					self.suspicion = self:AddArgument( "SUSPICION" )
					self.suspicion.stacks = math.ceil( difficulty / 2 )

					self.buffs = self:MakePicker()
					self.buffs:AddCard( self.suspicion, 1 )
					self.buffs:AddCard( self.irritable, 1 )

					self:SetPattern( self.Cycle )
					self.negotiator:AddModifier("DEFENSIVE")
				end,

				Cycle = function( self, turns )
					if turns % 3 == 0 then
						self:ChooseCard( self.bounty )
					end

					if (turns-1) % 5 == 0 then
						self.buffs:ChooseCards( 1 )
					end

					if (turns-2) % 3 == 0 then
						self:ChooseComposure( 1, self.difficulty * 2, self.difficulty * 2 )
					end

					self:ChooseGrowingNumbers( 1, 1 )
				end
			}
		},
	})
)

Content.AddCharacterDef(
	CharacterDef("OSHNU_WRANGLER",
	{
		base_def = "HEAVY_LABORER",

		title = "Oshnu Wrangler",
		tags = { "civilian", "laborer", "mercenary" },

		base_builds = {
			[ GENDER.MALE ] = "farmer_male",
			[ GENDER.FEMALE ] = "farmer_female"
		},

		loved_graft = "adhesive",
		hated_graft = "bad_bookie",
		death_item = "a_hot_tip",

		anims = { "anim/weapon_pitchfork_oshnu_wrangler.zip"},
		combat_anims = { "anim/med_combat_pitchfork_oshnu_wrangler.zip" },

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.VERY_LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,

			conditions = 
			{

			},

			attacks = 
			{
			    wrangler_shock = table.extend(NPC_ATTACK)
			    {
			        name = "Shock",
			        anim = "shock",
			        flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

			        OnPostResolve = function( self, battle, attack )
			        	for i, hit in attack:Hits() do
			        		if not hit.evaded then
				                if hit.target:IsPlayer() then
				                	hit.target:AddCondition("static_shock", 2 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ))
				                else
				                    hit.target:AddCondition( "WOUND", 2 )
				                end
				            end
				        end
			        end
			    },

			    wrangler_double_stab = table.extend(NPC_ATTACK)
			    {
			        name = "Double Stab",
			        anim = "double_stab",
			        flags = CARD_FLAGS.MELEE,
			        target_count = 2,
			        damage_mult = 0.4,
			    },

			    wrangler_taunt = 
			    {
			    	name = "Taunt",
			    	anim = "taunt",
			    	flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
			    	target_type = TARGET_TYPE.SELF,

			    	features =
			    	{
			    		DEFEND = 4,
			    		POWER_LOSS = 2,
			    		POWER = 2,
			    	},
			    },
			},

			behaviour =
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID( "wrangler_shock", 1 )
						:AddID( "wrangler_double_stab", 2 )
						:AddID( "wrangler_taunt", 1)
					self:SetPattern( self.Cycle )
				end,

				Cycle = function( self )
					self.attacks:ChooseCard()
				end
			}
		},
	})
)
Content.AddCharacterDef(
	CharacterDef("PEARLIE",
	{
		base_def = "LABORER",

		title = "Pearlie",
		tags = { "civilian"},

		base_builds = {
			[ GENDER.MALE ] = "male_pearl_resident_build",
			[ GENDER.FEMALE ] = "female_pearl_resident_build"
		},

		loved_graft = "local_chatter",
		hated_graft = "disapproval",
		death_item = "pearlie_wallet",
		combat_strength = 1,

		anims = { "anim/weapon_shield_pearlies.zip"},
		combat_anims = { "anim/med_combat_shield_pearlies.zip" },

		negotiation_data = 
		{
			modifiers =
			{
				PREOCCUPIED =
				{
					name = "Preoccupied",
					desc = "Whenever you destroy one of {1}'s arguments, they incept {2} {TOLERANCE}.",
					desc_fn = function( self, fmt_str )
						return loc.format( fmt_str, self:GetOwnerName(), self.tolerance_amt or 1)
					end,

					OnInit = function( self )
						if GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ARGUMENT_PLUS ) then
							self.tolerance_amt = 3
						else
							self.tolerance_amt = 2
						end
					end,

					modifier_type = MODIFIER_TYPE.CORE,

					event_handlers =
					{
						[ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
		                    if source and modifier.negotiator == self.negotiator then
		                        self.anti_negotiator:InceptModifier("TOLERANCE", self.tolerance_amt, self)
		                    end
		                end
					},
				},

				TOLERANCE =
				{
					name = "Tolerance",
					desc = "Your maximum hand size is reduced by {1}. At the start of your turn, remove one <b>Tolerance</>. <b>Tolerance</> can have up to 8 stacks.",
					desc_fn = function( self, fmt_str )
						return loc.format( fmt_str, self.stacks)
					end,

					modifier_type = MODIFIER_TYPE.INCEPTION,
					max_stacks = 8,

					OnSetStacks = function( self, old_stacks )
						local deck = self.engine:GetHandDeck()
                        deck:SetMaxSize( (self.engine.MAX_HAND_SIZE or 10) - self.stacks )
					end,

					event_handlers =
					{
						[ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
							if negotiator == self.negotiator then
								self.negotiator:RemoveModifier(self, 1, self)
							end
						end
					},
				},

				IMPARTIAL =
				{
					name = "Impartial",
					desc = "At the start of {1}'s turn, remove all {RENOWN} from both negotiators.",
					desc_fn = function( self,fmt_str )
						return loc.format( fmt_str, self:GetOwnerName() )
					end,

					modifier_type = MODIFIER_TYPE.ARGUMENT,

					max_resolve = 6,

					event_handlers =
					{
						[ EVENT.END_TURN ] = function( self, minigame, negotiator )
							if negotiator == self.negotiator then
								self:NotifyTriggered()
								local stacks = self.negotiator:GetModifierStacks("RENOWN")
								if stacks and stacks > 0 then
									self.negotiator:RemoveModifier("RENOWN", stacks, self)
								end

								local opponent_stacks = self.anti_negotiator:GetModifierStacks("RENOWN")
								if opponent_stacks and opponent_stacks > 0 then
									self.anti_negotiator:RemoveModifier("RENOWN", opponent_stacks, self)
								end
							end
						end
					},
				},

				CONFIDENCE =
				{
					name = "Confidence",

					modifier_type = MODIFIER_TYPE.ARGUMENT,
					target_enemy = TARGET_ANY_RESOLVE,
					max_stacks = 1,

					max_resolve = 3,
					min_persuasion = 1,
					max_persuasion = 1,
					OnBeginTurn = function( self, minigame )
				        self:ApplyPersuasion()
				    end,
				},
			},

			behaviour = 
			{
				OnInit = function( self, difficulty )
					local difficulty = self.engine and self.engine:GetDifficulty() or 1
					self.confidence_freq = difficulty > 2 and 1 or 2
					self.confidence = self:AddArgument("CONFIDENCE")
					self.impartial = self:AddArgument("IMPARTIAL")

					self.negotiator:AddModifier( "PREOCCUPIED" )

					if self.engine:GetPlayer():GetContentID() == "SMITH" then
						self:SetPattern( self.SmithCycle )
					else
						self:SetPattern( self.Cycle )
					end
				end,

				SmithCycle = function( self, turns )
					self:ChooseGrowingNumbers( 1, 1, 1, 2 )
					self:ChooseComposure( 2, 2, 2 )
					if (turns + 1) % self.confidence_freq == 0 and self.negotiator:GetModifierInstances("CONFIDENCE") < 5 then
						self:ChooseCard(self.confidence)
					end
					if turns % 3 == 0 and not self.negotiator:HasModifier("IMPARTIAL") then
						self:ChooseCard(self.impartial)
					end
				end,

				Cycle = function( self, turns )
					self:ChooseGrowingNumbers( 1, 1, 1, 2 )
					self:ChooseComposure( 2, 1, 2 )
					if (turns + 1) % self.confidence_freq == 0 and self.negotiator:GetModifierInstances("CONFIDENCE") < 5 then
						self:ChooseCard(self.confidence)
					end
				end,
			}
		},

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW,

			anim_mapping =
			{
				run_forward_pre = "run_forward_entry",
				run_forward = "run_forward_entry",
				run_forward_pst = "run_forward_entry_pst",
			},

			expanded_anim_mapping = {},

			anim_transitions =
			{
				run_forward_entry_pst = 
				{
					idle = "idle_pre",
				},
			},

			conditions = 
			{
				shock_shield =
				{
					name = "Shock Shield",
					desc = "While {1} has any {DEFEND}, attacking them deals <#HILITE>{2} damage</> to the attacker.",
					desc_fn = function( self, fmt_str )
						return loc.format( fmt_str, self:GetOwnerName(), self.stacks )
					end,

					ctype = CTYPE.BUFF,

					event_handlers =
					{
						[ BATTLE_EVENT.ON_HIT] = function( self, battle, attack, hit )
			                if hit.target == self.owner and ((hit.damage or 0) > 0 or (attack.card.max_damage and attack.card.max_damage > 0)) and not attack.is_counter and self.owner:IsActive() and not self.owner:HasCondition( "STUN" ) and not self.owner:HasCondition("SURRENDER") and self.owner:HasCondition("DEFEND") then
			                    attack.attacker:ApplyDamage( self.stacks, self.owner, self, false, {"electric"} )
			                end
			            end,
					},
				},
			},

			attacks = 
			{
			    pearlie_shoot = table.extend(NPC_ATTACK)
			    {
			        name = "Shoot",
			        anim = "shoot",
			        flags = CARD_FLAGS.RANGED,
			    },

			    pearlie_uppercut = table.extend(NPC_ATTACK)
			    {
			        name = "Uppercut",
			        anim = "uppercut",
			        flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

			        features = 
			        {
			        	IMPAIR = 1,
			        },
			    },

			    pearlie_taunt = 
			    {
			    	name = "Taunt",
			    	anim = "taunt",
			    	flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
			    	target_type = TARGET_TYPE.SELF,

			    	features =
			    	{
			    		DEFEND = 8,
			    	},
			    },

			    pearlie_anim_switch = 
			    {
		            OnPostResolve = function( self, battle, attack )
		            	self.owner:BroadcastEvent( BATTLE_EVENT.FIGHTER_REMAP_ANIMS, self.owner.agent.fight_data.expanded_anim_mapping )
		            end
			    },
			},

			behaviour =
			{
				shock_amt = { 2, 2, 3, 4 },
				OnActivate = function( self, fighter )
					self.attacks = self:MakePicker()
						:AddID( "pearlie_shoot", 1 )
						:AddID( "pearlie_uppercut", 1 )
						:AddID( "pearlie_taunt", 1)

					self.anim_swap = self:AddCard("pearlie_anim_switch")

					self:SetPattern( self.Cycle )
					self:Present( function( screen, ent )
                        fighter.battle:PlayCard(self.anim_swap)
                    end )
                    self.fighter:AddCondition("shock_shield", self.shock_amt[math.min(#self.shock_amt, fighter:GetScale() )] + (2*GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )), self)
				end,

				Cycle = function( self )
					self.attacks:ChooseCard()
				end
			}
		},
	})
)