local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local JAKES_NEGOTIATION =
{
	OnInit = function( self, difficulty )
		self.crafty = self:AddArgument( "CRAFTY" )
		self.snare = self:AddArgument( "SNARE" )
		self.ploy = self:AddArgument( "ploy" )

		if difficulty <= 2 then
			self:SetPattern( self.BasicCycle )
		else
			self:SetPattern( self.Cycle )
		end

		self.negotiator:AddModifier("DOUBLE_EDGE")
	end,

	BasicCycle = function( self, turns )
		-- CRAFTY every once in awhile if it does't exist.
		if (turns - 1) % 4 == 0 and not self.negotiator:FindModifier( "CRAFTY" ) then
			self:ChooseCard( self.crafty )
		end

		if turns % 3 == 0 then
			if math.random() < 0.5 then
				self:ChooseGrowingNumbers( 1, 1 )
			else
				self:ChooseCard( self.ploy )
			end
		end

		self:ChooseGrowingNumbers( 1, 0, 0.75 )
	end,

	Cycle = function( self, turns )
		-- CRAFTY every once in awhile if it does't exist.
		if (turns - 1) % 4 == 0 and not self.negotiator:FindModifier( "CRAFTY" ) then
			self:ChooseCard(self.crafty )

		elseif math.random() < 0.5 then
			if math.random() < 0.5 then
				self:ChooseCard( self.snare )
			else
				self:ChooseCard( self.ploy )
			end
		end

		self:ChooseGrowingNumbers( 1, 0 )
	end,
}

Content.AddCharacterDef
(
	CharacterDef("JAKES_BASE",
	{
		base_def = "NPC_BASE",
		is_template = true,

		tags = { "mercenary" },
    	death_money = DEATH_MONEY_MEDIUM,
    	death_item = "vial_of_slurry",

		social_boons =
		{
			[ RELATIONSHIP.LIKED ] = { GIVE_ITEM = 1 },
			[ RELATIONSHIP.LOVED ] = { GIVE_ITEM = 2, HIRE = 1 },
		},

		faction_id = "JAKES",
		support_negotiation = "jakes_support",
		combat_backup = "JAKES_BACKUP",

		base_builds = {
			[ GENDER.MALE ] = "merc_male",
			[ GENDER.FEMALE ] = "merc_female"
		},

		negotiation_data =
		{
			behaviour = table.extend( JAKES_NEGOTIATION )
			{
			}
		}
	})
)

local JAKES_RUNNER_BEHAVIOUR =
{
	elite = false,
	OnActivate = function( self )
		self.attacks = self:MakePicker()
			:AddID( "kneecap", 1 )
			:AddID( "slip_away", 1 )
			:AddID( "shiv", 1 )

		self.elite_attacks = self:MakePicker()
			:AddID( "blunderbuster", 1 )
			:AddID( "slip_away", 1 )
			:AddID( "covering_fire", 1 )

		self:SetPattern( self.elite and self.EliteCycle or self.Cycle )
		self.fighter:AddCondition("cruelty", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ))

	end,

	Cycle = function( self )
		self.attacks:ChooseCard()
	end,

	EliteCycle = function( self )
		self.elite_attacks:ChooseCard()
	end,
}

Content.AddCharacterDef
(
	CharacterDef("JAKES_RUNNER",
	{
		designer_note = "Whenever they successfully hit you, they wound you. The also expose you, making it more likely to get wounded.",
		base_def = "JAKES_BASE",
		title = "Dockhand",
		tags = { "enforcer" },
		combat_strength = 1.5,
		renown = 1,

		voice_actors = {
		    [GENDER.MALE] = { "jakeLowClassMale01" },
    		[GENDER.FEMALE] = { "jakeLowClassFemale01" },
		},

		promotion_def = "JAKES_RUNNER_PROMOTED",

		death_item = "gaff",

		anims = {"anim/weapon_hook_jakes_runner.zip"},
		combat_anims = { "anim/med_combat_hook_jakes_runner.zip" },
	    negotiation_ally_image = engine.asset.Texture( "negotiation/modifiers/recruit_jake_runner.tex"),

		loved_graft = "bargaining",
		hated_graft = "thin_skin",

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM_LOW,

			conditions = 
			{
			    cruelty =
			    {
			        name = "Cruelty",
			        desc = "{1}'s attacks apply <#HILITE>{2} {WOUND}</> if not defended.",
			        desc_fn = function( self, fmt_str )
			            return loc.format( fmt_str, loc.cap( self:GetOwnerName() ), self.stacks )
			        end,

			        ctype = CTYPE.INNATE,

			        event_handlers = 
			        {
			            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
			                if not hit.defended and not hit.evaded and attack.card:IsDamageCard() and attack.attacker == self.owner then
			                    hit.target:AddCondition("WOUND", self.stacks, attack)
			                end
			            end,
			        },
			    },
			},

			attacks = 
			{
			    shiv = table.extend(NPC_ATTACK)
			    {
			        name = "Shiv",
			        anim = "raid",
			        flags = CARD_FLAGS.MELEE,
			        hit_count = 2,

			        base_damage = 0.4,
			    },

			    kneecap = table.extend(NPC_ATTACK)
			    {
			        name = "Knee Cap",
			        anim = "kneecap",
			        flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

			        OnPostResolve = function( self, battle, attack )
			        	attack:AddCondition( "EXPOSED", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ) )
			       	end,
			    },

			    slip_away = table.extend(NPC_BUFF)
			    {
			        name = "Slip Away",
			        anim = "taunt",
			        flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
			        target_type = TARGET_TYPE.SELF,

			        OnPostResolve = function( self, battle, attack )
			        	attack:AddCondition( "EVASION", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ) )
			       	end,
			    },

			    blunderbuster = table.extend(NPC_ATTACK)
			    {
			    	name = "Blunderbuster",
			    	anim = "shoot",
			    	flags = CARD_FLAGS.RANGED,
			    },

			    covering_fire = table.extend(NPC_ATTACK)
			    {
			    	name = "Covering Fire",
			    	anim = "aoe_shot",
			    	flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,

			    	target_mod = TARGET_MOD.TEAM,
			    	OnPostResolve = function( self, battle, attack )
			        	attack:AddCondition( "EXPOSED", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ) )
			       	end,
			    },  
			},

			cards = { "kneecap", "slip_away", "shiv" },

			behaviour = JAKES_RUNNER_BEHAVIOUR,
		},
		
		idle_anims = 
		{
			guarding = "idle_neutral_guarding_ranged"
		},
			
		base_builds = {
			[ GENDER.MALE ] = "male_jake_runner",
			[ GENDER.FEMALE ] = "female_jake_runner"
		},

	})
)

Content.AddCharacterDef
(
	CharacterDef("JAKES_RUNNER_PROMOTED",
	{
		base_def = "JAKES_RUNNER",
		title = "Promoted Dockhand",
		death_item = "gaff_plus",
		combat_strength = 2,

		anims = {"anim/weapon_rifle_promoted_jakes.zip"},
		combat_anims = { "anim/med_combat_rifle_promoted_jakes.zip" },
		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,

			cards = {"slip_away", "blunderbuster", "covering_fire"},

			behaviour = table.extend(JAKES_RUNNER_BEHAVIOUR)
			{
				elite = true,
			},
		},
		
		idle_anims = 
		{
			guarding = "idle_neutral_guarding_ranged"
		},
			
		base_builds = {
			[ GENDER.MALE ] = "male_jake_promoted_build",
			[ GENDER.FEMALE ] = "female_jake_promoted_build"
		},
	})
)

Content.AddCharacterDef
(
	CharacterDef("JAKES_SMUGGLER",
	{
		designer_note = "Heals others, removing their negative effects. Also hits multiple targets.",
		base_def = "JAKES_BASE",
		title = "Smuggler",
		tags = { "enforcer" },
		renown = 3,
		combat_strength = 3,
		death_item = "triage",

		voice_actors = {
		    [GENDER.MALE] = { "jakeLowClassMale01" },
    		[GENDER.FEMALE] = { "jakeLowClassFemale01" },
		},

		loved_graft = "first_aid",
		hated_graft = "fragile_health",

		anims = {"anim/weapon_blaster_derrick_common.zip"},
		combat_anims = { "anim/med_combat_blaster_jakes_smuggler.zip" },

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM_HIGH,

			conditions = 
			{
			    triage_condition =
			    {
			        name = "Medic",
			        desc = "Whenever {1} heals a target, also remove all negative conditions on that target.",
			        desc_fn = function( self, fmt_str )
			            return loc.format( fmt_str, self:GetOwnerName(), self.extra_cards)
			        end,
			    },
			},

			attacks = 
			{
			    aimed_shot = table.extend(NPC_ATTACK)
			    {
			        name = "Aimed Shot",
			        anim = "blast",
			        flags = CARD_FLAGS.RANGED,
			        damage_mult = 1.2,
			    },

			    blaster_barrage = table.extend(NPC_ATTACK)
			    {
			        -- Does multi-target damage
			        name = "Blaster Barrage",
			        anim = "spray",
			        flags = CARD_FLAGS.RANGED,

			        target_type = TARGET_TYPE.ENEMY,

			        hit_count = 3,

			        damage_mult = 0.4,
			    },

			    jakes_triage = table.extend(NPC_BUFF)
			    {
			        name = "Triage",
			        anim = "taunt",
			        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,
			        flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF | CARD_FLAGS.HEAL,

			        min_triage = 7,
			        max_triage = 11,

			        OnInit = function( self )
			        	local bonus = math.floor( self.owner:GetScale() * (1.0 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )))
			        	self.min_triage = self.min_triage + bonus
			        	self.max_triage = self.max_triage + bonus
			        end,

			        CanPlayCard = function(self, battle, target )
			            if target then
			                local health = target:GetHealth()
			                local min_health, max_health = target:GetStatBounds( COMBAT_STAT.HEALTH )
			                if max_health - health < self.min_triage then
			                    return false
			                end
			            end

			            return true
			        end,

			        OnPostResolve = function(self, battle, attack)
			            local delta = math.random( self.min_triage, self.max_triage )
			            attack.target:HealHealth( delta, self )
			            attack.target:RemoveAllDebuffs( self )
			        end,
			    },
			},

			cards = { "jakes_triage", "blaster_barrage", "aimed_shot" },

			behaviour =
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID( "blaster_barrage", 1 )
						:AddID( "aimed_shot", 1 )
						:AddID( "ai_defend_other", 1 )
						:AddID( "jakes_triage", 1, 1 )

					self:SetPattern( self.Cycle )
					self.fighter:AddCondition("triage_condition")
				end,

				Cycle = function( self )
					self.attacks:ChooseCard()
				end,
			}
		},
		
		base_builds = {
			[ GENDER.MALE ] = "male_jake_smuggler",
			[ GENDER.FEMALE ] = "female_jake_smuggler"
		},

		idle_anims = 
		{
			guarding = "idle_neutral_guarding_ranged"
		},
	})
)

Content.AddCharacterDef
(
	CharacterDef("JAKES_LIFTER",
	{
		base_def = "JAKES_BASE",
		title = "Lifter",
		renown = 2,
		combat_strength = 4,
		death_item = "lifter_claw",

		voice_actors = {
		    [GENDER.MALE] = { "jakeLowClassMale01" },
    		[GENDER.FEMALE] = { "jakeLowClassFemale01" },
		},

		loved_graft = "quality_materials",
		hated_graft = "back_of_the_line",

		anims = {"anim/weapon_claws_jake_elite.zip"},
		combat_anims = { "anim/med_combat_claws_jake_elite.zip" },

		fight_data = 
		{
			battle_scale = 1.11,
			MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.HIGH,

			conditions = 
			{
			    
			},

			attacks = 
			{
			    lifter_attack1 = table.extend(NPC_MELEE)
			    {
			    	name = "Ground Pound",
			    	anim = "attack1",
			    	damage_mult = 0.7,
			    	hit_count = 2,

			    	flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

			    	features = 
			    	{
			    		EXPOSED = 2,
			    	},
			    },

			    lifter_attack2 = table.extend(NPC_MELEE)
			    {
			    	name = "Swing",
			    	anim = "attack2",
			    	damage_mult = 1.8,
			    },

			    lifter_taunt = table.extend(NPC_BUFF)
			    {
			    	name = "Brace",
			    	anim = "taunt",

			    	flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,

			    	target_type = TARGET_TYPE.SELF,
			    	OnPostResolve = function( self, battle, attack )
			    		attack:AddCondition("DEFEND", 8, self)
			    	end
			    }
			},

			behaviour =
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID("lifter_attack1", 3)
						:AddID("lifter_attack2", 1)
						:AddID("lifter_taunt", 1)

					self:SetPattern( self.Cycle )
				end,

				Cycle = function( self )
					self.attacks:ChooseCard()
				end,
			}
		},
		
		base_builds = {
			[ GENDER.MALE ] = "male_jake_elite_build",
			[ GENDER.FEMALE ] = "female_jake_elite_build"
		},
	})
)

Content.AddCharacterDef
(
	CharacterDef("JAKES_ASSASSIN",
	{
		designer_note = "Every time he hits you, you bleed. Defends himself with counter, making it more likely you continue bleeding.",

		base_def = "JAKES_BASE",
		title = "Assassin",
        shorttitle = "Jakes Assassin",
		combat_strength = 4,
		renown = 1,
		boss = true,
        battle_preview_anim = "anim/boss_assasin_zyn_slide.zip",
        battle_preview_offset = { x = 80, y = -10 },
        battle_preview_glow = { colour = 0xF2FE79FF, bloom = 0.18, fx_bloom = 0.2, threshold = 0.02 },
        battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_assassin_zyn",
        icon = engine.asset.Texture( "icons/b_jakes_assassin.tex"),

		tags = {"zyn"},

		unique = true,

		voice_actors = {
		    [GENDER.MALE] = { "assassin1Male01" },
    		[GENDER.FEMALE] = { "assassin1Male01" },
		},

    	death_money = DEATH_MONEY_HIGH,
    	death_item = "zyns_razor",

    	loved_graft = "sharpened_blades",
    	hated_graft = "hemophiliac",

		social_boons =
		{
			[ RELATIONSHIP.LIKED ] = { ASSASSINATE = 1, NOTHING = 3 },
			[ RELATIONSHIP.LOVED ] = { ASSASSINATE = 1 },
		},


		anims = {"anim/weapon_sword_phicket_assassin.zip"},
		combat_anims = { "anim/med_combat_sword_phicket_assassin.zip" },

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
			MAX_HEALTH = 130,

	        status_widget_head_dx = 0.0,
	        status_widget_head_dy = 4.5,

			anim_mapping =
 			{
 				riposte = "debilitate",
			},

			cards = { "slicing_strikes", "piercing_strikes", "singing_sword", "jakes_assassin_defend" },
			
			conditions = 
			{
			    SHARPENED_BLADE =
			    {
			        name = "Sharpened Blade",
			        desc = "Insert a <b>Bleed</> card into your draw pile whenever {1} deals damage to you.",
			        desc_fn = function( self, fmt_str )
			            return loc.format( fmt_str, self.owner.agent:GetName() or 1 )
			        end,

			        bleed_damage = { 3, 3, 4, 4 },

			        apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",

			        event_handlers = 
			        {
			            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
			                if attack.card:IsDamageCard() and attack.attacker == self.owner and not hit.defended and not hit.evaded then
	                            if hit.target:IsPlayer() then
	                                local cards = {}
	                                for i=1,1 do
	                                    local incepted_card = Battle.Card( "status_bleed", hit.target)
	                                    incepted_card.bleed_damage = self.bleed_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
	                                    table.insert(cards, incepted_card )                                  
	                                end
	                                battle:DealCards( cards, battle:GetDrawDeck() )
	                                self.has_spoken = false
	                            else
	                                hit.target:AddCondition("BLEED", 2, attack)
	                            end
			                end

			                if attack.card.id == "status_bleed" then
			                	if math.random() < 0.2 and not self.has_spoken then
			                		battle:BroadcastEvent( BATTLE_EVENT.QUIP, self.owner, "status_bleed" )
			                		self.has_spoken = true
			                	end
			                end
			            end,
			        },
			    },


			},

			attacks = 
			{
			    slicing_strikes = table.extend(NPC_ATTACK)
			    {
			        name = "Slicing Strikes",
			        anim = "bloodbind",
			        flags = CARD_FLAGS.MELEE,

			        base_damage = { 4, 4, 5, 6 },

			        hit_count = 2,
			    },

			    piercing_strikes = table.extend(NPC_ATTACK)
			    {
			        name = "Piercing Strikes",
			        anim = "attack3",
			        flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,

			        base_damage = { 2, 2, 2, 3 },

			        hit_count = 3,
			    },

			    singing_sword = table.extend(NPC_ATTACK)
			    {
			        name = "Singing Sword",
			        anim = "slash",
			        flags = CARD_FLAGS.MELEE,
			        
			        base_damage = {7, 8, 9, 12},
			    },

			    jakes_assassin_defend = table.extend(NPC_BUFF)
			    {
			        name = "Defend",
			        anim = "taunt",
			        flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
			        target_type = TARGET_TYPE.SELF,

			        riposte = {3,3,4,5},
			        defend = {6,7,8,10},

			        OnPostResolve = function( self, battle, attack )
			            self.owner:AddCondition("DEFEND", self.defend[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
			            self.owner:AddCondition("RIPOSTE", self.riposte[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
			        end
			    },
			},

			behaviour =
			{
				OnActivate = function( self, fighter )
					self.attacks = self:MakePicker()
						:AddID( "slicing_strikes", 4 )
						:AddID( "piercing_strikes", 2)
						:AddID( "singing_sword", 3)
					self.buff = self:AddCard("jakes_assassin_defend")
					self:SetPattern( self.Cycle )
					fighter:AddCondition( "SHARPENED_BLADE" )

					self:Present( function( screen, ent )
						screen:DelayForFighter( fighter )
						ent.cmp.AnimFighter:PlayAnim( "taunt" )
					end )
				end,

				Cycle = function( self )
					if self.battle:GetTurns() % 3 == 0 and self.battle:GetTurns() > 0 then
						return self:ChooseCard(self.buff)
					end
					return self.attacks:ChooseCard()
				end,
			},
		},

		base_builds = {
			[ GENDER.MALE ] = "male_phicket",
			[ GENDER.FEMALE ] = "male_phicket"
		},
	})
)

Content.AddCharacterDef
(
	CharacterDef("JAKES_ASSASSIN2",
	{
		designer_note = "Gives you cards that increases damage to his attacks. Also makes you draw into a lot of them.",

		base_def = "JAKES_BASE",
		title = "Assassin",
        shorttitle = "Jakes Assassin",
		combat_strength = 3,
		renown = 3,
		boss = true,
		unique = true,
		tags = {"hanbi"},
        battle_preview_anim = "anim/boss_assasin_hanbi_slide.zip",
        battle_preview_offset = { x = 190, y = -40 },
        battle_preview_glow = { colour = 0x59FFFAFF, bloom = 0.13, threshold = 0.02 },
        battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_assassin_hanbi",
        icon = engine.asset.Texture( "icons/b_jakes_assassin2.tex"),

		voice_actors = {
		    [GENDER.MALE] = { "assassin2Male01" },
    		[GENDER.FEMALE] = { "assassin2Male01" },
		},

    	death_money = DEATH_MONEY_HIGH,
    	death_item = "hanbis_power_supply",

    	loved_graft = "steady_hands",
    	hated_graft = "fumbling_fingers",

		social_boons =
		{
			[ RELATIONSHIP.LIKED ] = { ASSASSINATE = 1, NOTHING = 3 },
			[ RELATIONSHIP.LOVED ] = { ASSASSINATE = 1 },
		},


		anims = {"anim/weapon_blaster_phicket_assassin_02.zip", "anim/weapon_rifle_phicket_assassin_02.zip"},
		combat_anims = { "anim/med_combat_blaster_phicket_assassin_02.zip" },

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.MEDIUM,
			MAX_HEALTH = 130,

			anim_mapping =
 			{
 				riposte = "plasma_shot",
			},

			ranged_riposte = true,

			conditions = 
			{
			    hilt_slam_condition =
			    {
			        name = "Hilt Slam",
			        desc = "If {1}'s next attack deals damage, it will insert {2} additional Assassin's Marks into your deck.",
			        desc_fn = function( self, fmt_str )
			            return loc.format( fmt_str, self:GetOwnerName(), self.extra_cards)
			        end,

			        max_stacks = 1,

			        extra_cards = 2
			    },

			    steady_hands_debuff =
			    {
			        name = "Steady Hands",
			        desc = "Draw {1} extra cards at the start of your turn. Remove at the end of your turn.",
			        desc_fn = function( self, fmt_str )
			            return loc.format( fmt_str, self.extra_draw)
			        end,

			        extra_draw = 3,

			        event_handlers = 
			        {
			            [ BATTLE_EVENT.CALC_TURN_DRAW ] = function( self, acc )
			                acc:AddValue( self.extra_draw, self )
			            end,

			            [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
			                self.owner:RemoveCondition(self.id)
			            end
			        },
			    },
			},

			attacks = 
			{
			    assassins_mark_opener = 
			    {
			        name = "Mark",
			        anim = "taunt",
			        target_type = TARGET_TYPE.SELF,
			        quip = "assassin_steady_hands",

			        OnPostResolve = function( self, battle, attack )
			            local cards = {}
			        	local card_id
			        	if GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) >= 4 then
			        		card_id = "assassins_mark_plus"
			        	else
			        		card_id = "assassins_mark"
			        	end

			            for i = 1, 4 do
			                local incepted_card = Battle.Card( card_id, battle:GetPlayerTeam():Primary())
			                incepted_card.show_dealt = true
			                incepted_card.source = self.owner
			                table.insert(cards, incepted_card )                                  
			            end
			            battle:DealCards( cards, battle:GetDrawDeck() )
			            self.owner:AddCondition("assassin_mark_tracker")
			        end
			    },

			    steady_hands = table.extend(NPC_BUFF)
			    {
			        name = "Steady Hands",
			        anim = "taunt",
			        quip = "assassin_steady_hands",
			        flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
			        target_type = TARGET_TYPE.SELF,

			        defend_amount = { 10, 12, 15, 18 },
			        OnPostResolve = function( self, battle, attack )
			            battle:GetPlayerTeam():Primary():AddCondition("steady_hands_debuff")
			            self.owner.buffed_last_turn = true
			            self.owner:AddCondition("DEFEND", self.defend_amount[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
			        end
			    },

			    hilt_slam = table.extend(NPC_ATTACK)
			    {
			        name = "Hilt Slam",
			        anim = "attack3",
			        flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

			        base_damage = { 6, 7, 8, 10 },

			        bonus_damage = { 3, 3, 4, 4 },

			        OnPostResolve = function( self, battle, attack)
			            if not attack:CheckHitResult( attack.target, "evaded" ) and
			               not attack:CheckHitResult( attack.target, "defended" ) then
			                if attack.target:IsPlayer() then
			                    local cards = {}
			                    local count = self.owner:HasCondition("hilt_slam_condition") and self.owner:GetCondition("hilt_slam_condition").extra_cards or 2
			                    for i=1, count do
			                        local incepted_card = Battle.Card( "assassins_mark", battle:GetPlayerTeam():Primary())
			                        incepted_card.source = self.owner
			                        incepted_card.bonus_damage = self.bonus_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
			                        table.insert(cards, incepted_card )                                  
			                    end
			                    battle:DealCards( cards, battle:GetDrawDeck() )
			                end
			            end
			            if self.owner:HasCondition("hilt_slam_condition") then
			            	self.owner:RemoveCondition("hilt_slam_condition")
			            end
			        end,
			    },

			    focused_barrage = table.extend(NPC_ATTACK)
			    {
			        name = "Focused Barrage",
			        anim = "attack2",
			        quip = "focused_barrage",
			        flags = CARD_FLAGS.RANGED,
			        base_damage = { 3, 3, 4, 4 },
			        hit_count = 3,
			    },			    

			    plasma_shot = table.extend(NPC_ATTACK)
			    {
			        name = "Plasma Shot",
			        anim = "attack1",
			        flags = CARD_FLAGS.RANGED,

			        base_damage = { 12, 14, 16, 18 },
			    },
			},

			cards = { "plasma_shot", "focused_barrage", "hilt_slam", "steady_hands", "assassins_mark_opener" },
			
			behaviour =
			{
				OnActivate = function( self, fighter )						
					self.attack = self:AddCard("plasma_shot")
					self.hilt_slam = self:AddCard("hilt_slam")
					self.defense = self:AddCard("steady_hands")
					self.multi_hit = self:AddCard("focused_barrage")
					self:SetPattern( self.Cycle )
				end,

				Cycle = function( self )
					
					if self.fighter.battle:GetPlayerTeam():Primary():HasCondition("steady_hands_debuff") then
						return self:ChooseCard(self.multi_hit)
					end

					--every 4th turn, defend
					if self.fighter.battle:GetTurns() % 4 == 3 then
						return self:ChooseCard(self.defense)
					else
						if math.random() < 0.3 then
							if self.fighter:GetTeamID() == TEAM.RED then
								self.fighter:AddCondition("hilt_slam_condition")
							end
							return self:ChooseCard(self.hilt_slam)
						else
							return self:ChooseCard(self.attack)
						end
					end
				end,

				event_handlers = 
				{
					[ BATTLE_EVENT.SETTLE_FIGHTERS ] = function( self )
						if not self.given_cards and self.fighter:GetTeamID() == TEAM.RED then
							local card = Battle.Card( "assassins_mark_opener", self.fighter )
		                    card.show_dealt = false
		                    self.battle:PlayCard( card, self.fighter )
		                    self.given_cards = true
		                end
					end
				},
			},
		},

		negotiation_data =
		{
			modifiers =
			{
				MANIPULATOR =
				{
					name = "Manipulator",
					desc = "At the start of {1}'s turn, insert {2} {3#card} {2*card|cards} into your draw pile.",
					desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName(), self.insert_count[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.insert_count)], self.card_id)
                    end,

                    modifier_type = MODIFIER_TYPE.CORE,

                    insert_count = { 1, 1, 2, 2 },
                    card_id = "sitting_duck",

                    OnBeginTurn = function( self, minigame )
                    	local cards = {}
                    	for i=1, self.insert_count[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.insert_count)] do
                    		table.insert(cards, Negotiation.Card( self.card_id, self.anti_negotiator.agent ))
                    	end
                    	minigame:InceptCards( cards, self )
                    end,

                    CalculateBonus = function( self )
                    	if self.calculated_bonus then
                    		return self.calculated_bonus
                    	else
                    		local bonus = 0
                    		for i,card in self.engine:GetHandDeck():Cards() do
                    			if card.id == "sitting_duck" then
                    				bonus = bonus + 1
                    			end
                    		end
                    		if (self.latest or 0) ~= bonus then
                    			self.engine:BroadcastEvent( EVENT.CUSTOM, function( panel, ... )
					                panel.intents:RefreshIntents()
					            end )
                    			self.latest = bonus
                    		end
                    		return bonus
                    	end
                    end,

                    event_priorities =
                    {
                        [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
                    },

                    event_handlers =
                    {
                        [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion, minigame, target )
                            if source.negotiator == self.negotiator and is_instance( source, Negotiation.Card ) then
                                persuasion:AddPersuasion( self:CalculateBonus(), self:CalculateBonus(), self )
                            end
                        end,

                        [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                        	local bonus = 0
                    		for i,card in self.engine:GetHandDeck():Cards() do
                    			if card.id == "sitting_duck" then
                    				bonus = bonus + 1
                    			end
                    		end
                    		self.calculated_bonus = bonus
                        end,

                        [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                        	self.calculated_bonus = nil
                        end
                    }
				},

				ALLUREMENT = 
				{
					name = "Allurement",
					desc = "At the start of {1}'s turn, insert {2} {3#card} {2*card|cards} into your draw pile.\nWhen this is destroyed, {CONSUME} all {3#card} cards in your deck.",
					desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName(), self.insert_count[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.insert_count)], self.card_id, self:GetOpponentName(), self.draw_bonus)
                    end,

                    insert_count = { 2, 2, 2, 3 },
                    card_id = "sitting_duck",
                    draw_bonus = 2,

                    max_stacks = 1,
                    max_instances = 2,
                    max_resolve = 6,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,

                    OnBeginTurn = function( self, minigame )
                    	local cards = {}
                    	for i=1, self.insert_count[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.insert_count)] do
                    		table.insert(cards, Negotiation.Card( self.card_id, self.anti_negotiator.agent ))
                    	end
                    	minigame:InceptCards( cards, self )
                    end,

                    OnBounty = function( self )
	                    local cards = table.merge( self.engine.hand_deck.cards, self.engine.discard_deck.cards, self.engine.draw_deck.cards )
	                    for i,card in ipairs(cards) do
	                    	if card.id == "sitting_duck" then
	                    		self.engine:ExpendCard(card)
	                    	end
	                    end
	                end,
				},
			},

			behaviour = table.extend( JAKES_NEGOTIATION )
			{
				OnInit = function( self, difficulty )
					if CheckBits( self.engine.start_params.flags, NEGOTIATION_FLAGS.WORDSMITH ) then
						self.allure = self:AddArgument("ALLUREMENT")

						self.negotiator:AddModifier("MANIPULATOR")
						self:SetPattern(self.BossCycle)
					else
						self.crafty = self:AddArgument( "CRAFTY" )
						self.snare = self:AddArgument( "SNARE" )
						self.ploy = self:AddArgument( "ploy" )

						if difficulty <= 2 then
							self:SetPattern( self.BasicCycle )
						else
							self:SetPattern( self.Cycle )
						end

						self.negotiator:AddModifier("DOUBLE_EDGE")
					end
				end,

				BossCycle = function( self, turns )
					self:ChooseFlatNumbers( 2, 1 )
					if self.negotiator:GetModifierInstances("ALLUREMENT") < 2 and (turns - 1) % 3 == 0 then
						self:ChooseCard(self.allure)
					end
					if turns % 3 == 0 then
	                    self:ChooseComposure( 1, 1, 3 )
	                end
				end
			}
		},

		base_builds = {
			[ GENDER.MALE ] = "male_phicket_02",
			[ GENDER.FEMALE ] = "male_phicket_02"
		},
	})
)

Content.AddNegotiationCard("sitting_duck", 
{
    name = "Sitting Duck",
    desc = "While this is in your hand, {1}'s intents deal 1 bonus damage.",
    flavour = "'...Quack?'",
    desc_fn = function( self, fmt_str )
    	return loc.format(fmt_str, self.anti_negotiator and self.anti_negotiator:GetName() or LOC"UI.CARDS.OPPONENT")
    end,
    
    cost = 1,
    flags = negotiation_defs.CARD_FLAGS.STATUS | negotiation_defs.CARD_FLAGS.EXPEND | negotiation_defs.CARD_FLAGS.BURNOUT | negotiation_defs.CARD_FLAGS.REPLENISH,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
})