local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

Content.AddCharacterDef
(
	CharacterDef("KALANDRA",
	{
		base_def = "RISE_BASE",

		name = "Kalandra",
		title = "Organizer",
		renown = 2,
		combat_strength = 2,
		
		alias = "KALANDRA",
		desc = "Mysterious Woman",

		initial_connection_filter = function(agent)
			return not agent:GetFactionID() == "BANDITS"
		end,
		voice_actor = "prindokalandra",
		
		tags = {"no_patron"},
		faction_id = "RISE",
		gender = GENDER.FEMALE,
		species = SPECIES.HUMAN,
		head = "head_kalandra",
		build = "female_rise_kalandra_build",
        theme_music = "event:/music/story/rook/story_rook_kalandra",

		combat_anims = { "anim/med_combat_knife_kalandra.zip" },
		anims = { "anim/weapon_knife_kalandra.zip" },

		loved_graft = "excavator",
		hated_graft = "buried",
		death_item = "battle_plan",

		fight_data = 
		{
			ignore_anim_symbols = {"knife_layer 8", "knife_glow"},
			MAX_MORALE = MAX_MORALE_LOOKUP.MEDIUM,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,

			attacks = 
            {
                kalandra_slice = table.extend(NPC_ATTACK)
                {
                    name = "Slicing Strike",
                    anim = "attack1",
                    flags = CARD_FLAGS.MELEE,
                    target_count = 2,
                    damage_mult = 0.4,
                },

                kalandra_grenade = table.extend(NPC_ATTACK)
                {
                	name = "Grenade Throw",
                	anim = "grenade",
                	flags = CARD_FLAGS.RANGED,
                	target_mod = TARGET_MOD.TEAM,
                },

                kalandra_taunt = table.extend(NPC_BUFF)
                {
                	name = "Knife Flourish",
                	anim = "taunt",
                	flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                	target_type = TARGET_TYPE.SELF,
                	OnPostResolve = function( self, battle, attack )
                		self.owner:AddCondition("DEFEND", 6, self)
                		self.owner:AddCondition("POWER", 2, self)
                	end
                },

                kalandra_tag_in_attack = table.extend(NPC_ATTACK)
                {
                    name = "Grenade Lob",
                    anim = "special_attack",
                    flags = CARD_FLAGS.RANGED | CARD_FLAGS.SPECIAL,
                    damage_mult = 1.5,
                    target_mod = TARGET_MOD.TEAM,
                    OnPostResolve = function( self, battle, attack )
                        self.owner:Depart()
                    end
                },
            },


			behaviour = 
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID( "kalandra_slice", 4 )
						:AddID( "kalandra_grenade", 2 )
						:AddID( "kalandra_taunt", 1)
					self:SetPattern( self.Attack )
				end,

				Attack = function( self )
					self.attacks:ChooseCards( 1 )
				end,
			},

		},

		hair_colour = 0x382919ff,
		skin_colour = 0x714F47ff,

		text_colour = 0x318c9bFF,
	})
)

