local battle_defs = require "battle/battle_defs"
require "content/characters/monsters"
local MONSTER_DEF = Content.GetCharacterDef("MONSTER")
assert( MONSTER_DEF )

local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local FLAGS = battle_defs.CARD_FLAGS

Content.AddCharacterDef
(
    CharacterDef("LUMICYTE",
    {
        base_def = "MONSTER",

        title = "Lumicyte",
        build = "lumicyte",
        
        death_money = 0,
        death_item= "lumicyte_dorsal_spines",

        support_negotiation = "untrained_pet",
        trained_support_negotiation = "trained_pet",
        voice_event = "event:/vo/compendium/speak_lumicyte",

        fight_data = table.extend( MONSTER_DEF.fight_data )
        {
            hit_tags = { "small" },

            status_widget_dx = -0.3,
            status_widget_dy = -0.8,
            status_widget_head_dx = 0,
            status_widget_head_dy = 3.0,

            attacks = 
            {
                lumicyte_attack1 = table.extend(NPC_ATTACK)
                {
                    name = "Leap Strike",
                    anim = "attack1",
                    flags = FLAGS.MELEE,
                    damage_mult = 1,
                    hit_tags = {"attack_regular", "lumin"},
                },

                lumicyte_attack2 = table.extend(NPC_ATTACK)
                {
                    name = "Rending Claws",
                    anim = "attack2",
                    flags = FLAGS.MELEE | FLAGS.DEBUFF,
                    hit_count = 2,
                    damage_mult = 0.3,
                    features =
                    {
                        EXPOSED = 1,
                    },
                    hit_tags = {"attack_regular"},
                },

                lumicyte_taunt = table.extend(NPC_BUFF)
                {
                    name = "Screech",
                    anim = "taunt",
                    flags = FLAGS.SKILL | FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,
                    hit_tags = {"attack_regular"},
                    features = 
                    {
                        DEFEND = 3,
                    },

                    OnPostResolve = function( self, battle, attack )
                        local con = self.owner:AddCondition( "KINDLING", 3 )
                        if con then
                            con.just_applied = true
                        end
                    end,
                },
            },

            conditions =
            {
                KINDLING =
                {
                    name = "Kindling",
                    desc = "When {1} receives damage, reduce <b>Kindling</b> by 1 and apply <b>Kindling</b> to the attacker.\nAt the end of {1}'s turn, convert all <b>Kindling</b> to {TEMP_POWER}.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOwnerName(), 1 )
                    end,
                    ctype = CTYPE.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
                            local source = self.battle:GetFighterFromSource( source )
                            if fighter == self.owner and source ~= self.owner then
                                self.owner:RemoveCondition( "KINDLING", 1, self )
                                if source then
                                    source:AddCondition( "KINDLING", 1, self )
                                end
                            end
                        end,

                        [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                            if fighter == self.owner then
                                if self.just_applied then
                                    self.just_applied = nil
                                else
                                    fighter:AddCondition( "POWER", self.stacks, self )
                                    fighter:AddCondition( "POWER_LOSS", self.stacks, self )
                                    fighter:RemoveCondition( "KINDLING", self.stacks, self )
                                end
                            end
                        end,            
                    }
                },
            },

            behaviour =
            {
                OnActivate = function( self )
                    self.attacks = self:MakePicker()
                        :AddID( "lumicyte_attack1", 2 )
                        :AddID( "lumicyte_attack2", 1 )
                        :AddID( "lumicyte_taunt", 1 )
                    self:SetPattern( self.Cycle )

                    self.fighter:AddCondition( "KINDLING", 1 + self.fighter:GetScale() )
                    self.fighter:AddCondition( "DEFEND", 3 + 2 * GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ))
                end,

                Cycle = function( self )
                    if self.fighter:HasCondition( "POWER_LOSS" ) then
                        self.attacks:ChooseCard( "lumicyte_attack1" )
                    else
                        self.attacks:ChooseCard()
                    end
                end,
            }
        },
    })
)

Content.AddCharacterDef
(
    CharacterDef("LUMICYTE_UPGRADED",
    {
        base_def = "LUMICYTE",

        title = "Lumicyte",
        build = "lumicyte_upgrade",
        
        death_money = 0,

        fight_data = table.extend( MONSTER_DEF.fight_data )
        {
            hit_tags = { "small" },
            MAX_HEALTH = 28,

            status_widget_dx = -0.3,
            status_widget_dy = -0.8,

            attacks = 
            {
                lumicyte_attack2_upgraded = table.extend(NPC_ATTACK)
                {
                    name = "Rending Claws",
                    anim = "attack2",
                    flags = FLAGS.MELEE | FLAGS.DEBUFF,
                    target_count = 2,
                    damage_mult = 0.4,
                    hit_tags = {"attack_regular"},
                    features =
                    {
                        EXPOSED = 1,
                    },
                },

                lumicyte_taunt_upgraded = table.extend(NPC_BUFF)
                {
                    name = "Screech",
                    anim = "taunt",
                    flags = FLAGS.SKILL | FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,
                    hit_tags = {"attack_regular"},
                    features = 
                    {
                        DEFEND = 4,
                    }
                },
            },

            behaviour =
            {
                OnActivate = function( self )
                    self.attacks = self:MakePicker()
                        :AddID( "lumicyte_attack1", 2 )
                        :AddID( "lumicyte_attack2_upgraded", 1 )
                        :AddID( "lumicyte_taunt_upgraded", 1 )
                    self:SetPattern( self.Cycle )

                    self.fighter:AddCondition( "KINDLING", 2 + self.fighter:GetScale() )
                    self.fighter:AddCondition( "ARMOURED", 2 )
                    self.fighter:AddCondition( "DEFEND", 3 + 2 * GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ))
                end,

                Cycle = function( self )
                    if self.fighter:HasCondition( "POWER_LOSS" ) then
                        self.attacks:ChooseCard( "lumicyte_attack1" )
                    else
                        self.attacks:ChooseCard()
                    end
                end,
            }
        },
    })
)

Content.AddCharacterDef
(
    CharacterDef("LUMICYTE_VIX",
    {
        base_def = "MONSTER",

        title = "Mutated Lumicyte",
        build = "lumicyte_vix",
        
        death_money = 0,

        fight_data = table.extend( MONSTER_DEF.fight_data )
        {
            hit_tags = { "small" },
            MAX_HEALTH = 16,
            MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,

            status_widget_dx = -0.3,
            status_widget_dy = -0.8,
            status_widget_head_dx = 0,
            status_widget_head_dy = 3.0,
            battle_scale = 1.1,

            attacks = 
            {
                lumicyte_attack1_vix = table.extend(NPC_ATTACK)
                {
                    name = "Leap Strike",
                    anim = "attack1",
                    flags = FLAGS.MELEE,
                    damage_mult = 1,
                    hit_tags = {"attack_alt","acid"},
                },

                lumicyte_attack2_vix = table.extend(NPC_ATTACK)
                {
                    name = "Rending Claws",
                    anim = "attack2",
                    flags = FLAGS.MELEE,
                    hit_count = 2,
                    damage_mult = 0.3,
                    hit_tags = {"attack_alt"},
                },

                lumicyte_taunt_vix = table.extend(NPC_BUFF)
                {
                    name = "Screech",
                    anim = "taunt",
                    flags = FLAGS.SKILL | FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,
                    hit_tags = {"attack_alt"},
                    features = 
                    {
                        DEFEND = 3,
                    },

                    OnPostResolve = function( self, battle, attack )
                        local con = self.owner:AddCondition( "KINDLING", 3 )
                        if con then
                            con.just_applied = true
                        end
                    end,
                },
            },

            behaviour =
            {
                OnActivate = function( self )
                    self.attacks = self:MakePicker()
                        :AddID( "lumicyte_attack1_vix", 2 )
                        :AddID( "lumicyte_attack2_vix", 1 )
                        :AddID( "lumicyte_taunt_vix", 1 )
                    self:SetPattern( self.Cycle )

                    self.fighter:AddCondition( "KINDLING", self.fighter:GetScale() )
                    -- self.fighter:AddCondition( "DEFEND", 2 * self.fighter:GetScale())
                end,

                Cycle = function( self )
                    if self.fighter:HasCondition( "POWER_LOSS" ) then
                        self.attacks:ChooseCard( "lumicyte_attack1_vix" )
                    else
                        self.attacks:ChooseCard()
                    end
                end,
            }
        },
    })
)