require "content/characters/base_types"
local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

Content.AddCharacterDef
(
	CharacterDef("MARK_NINE_NINE",
	{

		base_def = "NPC_BASE",
		title = "Defensive Program",
		species = SPECIES.MECH,
		alias = "MARK_NINE_NINE",
		voice_actor = "robotMark99",
        faction_id = "NEUTRAL",


		combat_strength = 4,
		renown = 1,
		boss = true,
		can_talk = true,
		battle_preview_anim = "anim/boss_mark_nine_nine.zip",
		battle_preview_offset = { x = 115, y = -80 },
		battle_preview_glow = { colour = 0xE96440FF, bloom = 0.4, threshold = 0.02 },
		battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_markninenine",
		icon = engine.asset.Texture( "icons/b_mark_nine_nine.tex" ),

		unique = true,
        aspects = {
            social_connections = false,
        },

		bio = "",

		gender = GENDER.UNDISCLOSED,
	    hair_colour = 3434903039,
	    head = "head_automech_bodyguard",
		build = "male_automech_bodyguard",
	    name = "Mark Nine Nine",
	    skin_colour = 1652061183,

		anims = { "anim/weapon_rifle_automech_bodyguard.zip" },
		combat_anims = { "anim/med_combat_rifle_automech_bodyguard.zip" },

		death_item = "predictive_blast",

		fight_data = 
		{
			MAX_HEALTH = 110,
        	MAX_MORALE = MAX_MORALE_LOOKUP.IMMUNE,
        	death_fade_delay = 1,
        	
            conditions =
			{
				WOUND_UP =
				{
					name = "Wound Up",
					desc = "{1}'s attacks deal <#HILITE>{2}</> additional damage. Remove 2 <b>Wound Up</> at the end of {1}'s turn.",
					desc_fn = function( self, fmt_str )
						return loc.format(fmt_str, self:GetOwnerName(), self.stacks or 1)
					end,

					event_handlers =
			        {
			            [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
			                if card.owner == self.owner and card:IsAttackCard() then
			                    dmgt:AddDamage( self.stacks, self.stacks, self )
			                end
			            end,

			            [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
			            	if fighter == self.owner then
			            		if self.skipped then
			            			self.owner:RemoveCondition(self.id, math.min(2, self.stacks), self)
			            		else
			            			self.skipped = true
			            		end
			            	end
			            end
			        }
				},

				COMBAT_ANALYSIS =
				{
					name = "Combat Analysis",
					desc = "Whenever you attack {1}, the base damage of their next attack becomes the damage dealt by your attack.{2}",
					desc_fn = function( self, fmt_str )
						if self.current_dmg then
							local str = loc.format(self:GetLocalizedString("CURRENT_BONUS"), self.current_dmg)
							return loc.format(fmt_str, self:GetOwnerName(), str)
						else
							return loc.format(fmt_str, self:GetOwnerName(), "")
						end
					end,
					loc_strings = 
					{
						CURRENT_BONUS = "\n(Current Base Damage: {1})"
					},
					-- This sets the base damage, so we still want things like wound and cripple to affect it
					event_priorities =
			        {
			            [ BATTLE_EVENT.CALC_DAMAGE ] = 0,
			        },
					event_handlers =
					{
						[ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
							if hit.target == self.owner and attack.attacker == battle:GetPlayerFighter() and hit.damage then
								self.current_dmg = hit.damage
							end
						end,

						[ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
			                if self.current_dmg and card.owner == self.owner and card:IsAttackCard() then
			                    dmgt:ModifyDamage( self.current_dmg, self.current_dmg, self )
			                end
			            end,
					},
				},
			},

			attacks =
			{
				autoguard_shoot = table.extend(NPC_ATTACK)
				{
					name = "Shoot",
					anim = "shoot",
					flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
					damage_mult = 1,

					OnPostResolve = function( self, battle, attack )
						for i,hit in ipairs(attack.hits) do
							if not hit.evaded and not hit.defended then
								hit.target:AddCondition("WOUND", 2, self)
							end
						end
					end,
				},
				autoguard_shoot2 = table.extend(NPC_ATTACK)
				{
					name = "Grenade Launch",
					anim = "shoot2",
					flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
					target_mod = TARGET_MOD.TEAM,
					damage_mult = 0.6,
					OnPostResolve = function( self, battle, attack )
						for i,hit in ipairs(attack.hits) do
							if not hit.evaded and not hit.defended then
								hit.target:AddCondition("WOUND", 2, self)
							end
						end
					end,
				},
				autoguard_shoot3 = table.extend(NPC_ATTACK)
				{
					name = "Rapid Fire",
					anim = "shoot3",
					flags = CARD_FLAGS.RANGED,
					target_mod = TARGET_MOD.RANDOM1,
					target_count = 3,
					damage_mult = 0.3,
				},
				autoguard_taunt =
				{
					anim = "taunt",
					flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
					target_type = TARGET_TYPE.SELF,

					CanPlayCard = function( self, battle, target )
			            return self.owner:GetConditionStacks("WOUND_UP") <= 4
			        end,

					wound_up_amt = { 4, 4, 6, 8 },
					defend_amt = { 3, 3, 5, 7 },
					heal_amt = { 8, 8, 10, 12 },

					OnPostResolve = function( self, battle, attack )
						attack:AddCondition("WOUND_UP", self.wound_up_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.wound_up_amt)], self)
						attack:AddCondition("DEFEND", self.defend_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.defend_amt)], self)
						self.owner:HealHealth(self.heal_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.heal_amt)], self)
					end
				},
			},

			behaviour =
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID("autoguard_shoot", 2)
						:AddID("autoguard_shoot2", 2)
						:AddID("autoguard_taunt", 2)
						:AddID("autoguard_shoot3", 1)

					self:SetPattern( self.Cycle )
					self.fighter:AddCondition("METALLIC")
					self.fighter:AddCondition("COMBAT_ANALYSIS")
				end,

				Cycle = function( 	self )    
					self.attacks:ChooseCard()
				end
			},
		},

		negotiation_data = 
		{

		},



		base_builds = {
			[ GENDER.UNDISCLOSED ] = "male_automech_bodyguard",
		},
	})
)

