local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local def = CharacterDef("MERCENARY_BOSS",
{
    unique = true,

    base_def = "NPC_BASE",
    alias = "MERCENARY_BOSS",
    species = SPECIES.SHROKE,
    gender = GENDER.MALE,
    renown = 3,
    combat_strength = 2,
    boss = true,

    death_item = "high_yield_lumin_bomb",
    loved_graft = "capitalization",
    hated_graft = "rattled",

    battle_preview_anim = "anim/boss_mercenary_slide.zip",
    battle_preview_offset = { x = 150, y = 20 },
    battle_preview_glow = { colour = 0x25FAFBFF, bloom = 0.3, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_mercenaryboss",
    icon = engine.asset.Texture( "icons/b_mercenary_boss.tex" ),
    faction_id = "JAKES",
    name = "Fralx",
    title = "Protégé",
    -- voice_actor = "jakeLowClassFemale01",
    
    combat_anims = {"anim/med_combat_pistols_mercenary.zip"},
    anims = {"anim/weapon_blaster_mercenary.zip"},
    build = "male_mercenary_boss_build",
    head = "head_male_mercenary_boss",

    negotiation_data = 
    {
        modifiers =
        {

        },
        behaviour = 
        {
            OnInit = function( self, difficulty )
                self.moment_of_weakness = self:AddCard( "moment_of_weakness" )
                self.negotiator:AddModifier( "UNCERTAINTY" )
                self:SetPattern( self.BasicCycle )
            end,

            BasicCycle = function( self, turns )
                if not self.player_negotiator:HasModifier("moment_of_weakness") then
                    self:ChooseCard( self.moment_of_weakness )
                    self:ChooseGrowingNumbers( 2 )
                else
                    self:ChooseGrowingNumbers( 2, 2 )
                    if math.random() < .5 then
                        self:ChooseComposure( 1, 2, 2 )
                    end
                end
            end,
        }
    },
    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 95,
        battle_scale = 1,
        death_fade_delay = 1,
        shadow_symbol = "hips",
        shadow_scale = 8.5,

        conditions = 
        { 
            LUMIN_DAZE =
            {
                name = "Lumin Daze",
                desc = "Reduce the minimum damage of all your cards by <#HILITE>{1}</>. Reduce <b>Lumin Daze</> by 1 whenever you play a Maneuver card and at the end of your turn.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.stacks or 1 )
                end,

                ctype = CTYPE.DEBUFF,

                event_priorities =
                {
                    -- after: concentration
                    [ BATTLE_EVENT.CALC_DAMAGE  ] = 12,
                },

                event_handlers = 
                {
                    [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                        if card.owner == self.owner then
                            dmgt:ModifyDamage( dmgt.min_damage - self.stacks, dmgt.max_damage, self )
                        end
                    end,

                    [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, card )
                        if card.owner == self.owner and card:IsFlagged(CARD_FLAGS.SKILL) then
                            self.owner:RemoveCondition("LUMIN_DAZE", 1, self)
                        end
                    end,

                    [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        if fighter == self.owner then
                            self.owner:RemoveCondition("LUMIN_DAZE", 1, self)
                        end
                    end
                },
            },
        },

        attacks = 
        {
            merc_double_hit =table.extend(NPC_ATTACK)
            {
                name = "Double Hit",
                anim = "attack1",

                flags = CARD_FLAGS.RANGED,
                target_count = 2,
                damage_mult = 0.7,
            },

            merc_salvo = table.extend(NPC_ATTACK)
            {
                name = "Salvo",
                anim = "attack2",

                flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,

                CanPlayCard = function( self, battle, target )
                    return target == nil or target:IsPlayer()
                end,

                daze_amt = { 2, 4, 6, 7 },
                wound_amt = { 1, 3, 3, 4 },

                OnPostResolve = function( self, battle, attack )
                    local adv = GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1
                    attack:AddCondition( "LUMIN_DAZE", self.daze_amt[math.min(#self.daze_amt, adv)], self )
                    attack:AddCondition( "WOUND", self.wound_amt[math.min(#self.wound_amt, adv)], self )
                end
            },

            merc_gun_flourish = table.extend(NPC_BUFF)
            {
                name = "Gun Flourish",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                defend_amt = { 3, 6, 8, 10},

                OnPostResolve = function( self, battle, attack )
                    local adv = GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1
                    attack:AddCondition( "DEFEND", self.defend_amt[math.min(#self.defend_amt, adv)], self )
                end
            },
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                self.double = self:AddCard("merc_double_hit")
                self.salvo = self:AddCard("merc_salvo")
                self.taunt = self:AddCard("merc_gun_flourish")

                self:SetPattern(self.Cycle)
            end,

            Cycle = function( self, turns )
                if turns % 3 == 0 then
                    self:ChooseCard(self.taunt)
                elseif turns % 3 == 1 then
                    self:ChooseCard(self.salvo)
                else
                    self:ChooseCard(self.double)
                end
            end
        },
    },
})

Content.AddCharacterDef( def )
    