local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local def = CharacterDef("MULLIFEE",
{
    unique = true,
    base_def = "ADMIRALTY_PATROL_LEADER",
    alias = "MULLIFEE",
    species = SPECIES.KRADESHI,
    gender = GENDER.FEMALE,
    renown = 2,
    combat_strength = 3,
    name = "Mullifee",
    
    loved_graft = "family_favor",
    hated_graft = "shunned",
    death_item = "banquod_pendant",
    faction_id = "ADMIRALTY",
    title = "Guard Captain",
     
    theme_music = "event:/music/story/smith/story_smith_mullifee",
    -- voice_actor = "",
    
    anims = { "anim/weapon_spear_mulifee.zip", },
    combat_anims = { "anim/med_combat_spear_mulifee.zip" },
    head = "head_female_mullifee",
    build = "female_mullifee_build",

    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 85,
        death_fade_delay = 1,

        conditions =
        {

        },

        attacks = 
        {
            mullifee_thrust = table.extend(NPC_ATTACK)
            {
                name = "Thrust",
                anim = "thrust",
                flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,
                OnPostResolve = function( self, battle, attack )
                    attack:AddCondition("WOUND", 2, self)
                end,
            },

            mullifee_multithrust = table.extend(NPC_ATTACK)
            {
                name = "Attack",
                anim = "multi_thrust",
                flags = CARD_FLAGS.MELEE,
                hit_count = 3,
                damage_mult = 0.4,
            },

            mullifee_taunt = table.extend(NPC_BUFF)
            {
                name = "Taunt",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("DEFEND", 6, self)
                    self.owner:AddCondition("POWER", 1, self)
                end
            },        
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                self.attacks = self:MakePicker()
                    :AddID("mullifee_thrust", 3)
                    :AddID("mullifee_multithrust", 2)
                    :AddID("mullifee_taunt", 1)

                self:SetPattern(self.Cycle)
            end,


            Cycle = function( self )
                self.attacks:ChooseCard()
            end
        },
    },
})

Content.AddCharacterDef( def )
    