local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local MUTANT_FORMATION =
{
    { 10, 1.5, FIGHTER_FORMATION.FRONT_X },  -- Vix

    { 3, -5, nil },
    { 5, 5, nil }, 
}

Content.AddCharacterDef(CharacterDef("MUTANT_VIX", {
    base_def = "MONSTER",
    name = "Vix of Hesh",
    gender = GENDER.MALE,
    build = "mutant_vix",
    combat_strength = 4,
    boss = true,
    unique = true,
    species = SPECIES.KRADESHI,
    skin_colour = 3333903871,
    battle_preview_anim = "anim/boss_mutant_vix_slide.zip",
    battle_preview_offset = { x = 0, y = 30 },
    battle_preview_glow = { colour = 0x27FFFFFF, bloom = 0.12, threshold = 0.02 },
    -- battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_shroog",
    icon = engine.asset.Texture( "icons/b_mutant_vix.tex" ),
    compendium_offset = { -280, 0 },
    --death_item = "shroog_fang",
    voice_actor = "mutant_vix",

    combat_anims = {"anim/lrg_combat_vixmalli.zip"},
    anims = {"anim/lrg_combat_vixmalli.zip"},

    fight_data =
    {
        formation = FIGHTER_FORMATION.FRONT_X,
        battle_scale = 1,
        MAX_HEALTH = 220,
        MAX_MORALE = MAX_MORALE_LOOKUP.VERY_HIGH,
        death_fade_delay = 1.5,
        status_widget_head_dx = 2,
        status_widget_head_dy = 6.5,

        hud_fx_remapping =
        {
            surrender = "surrender_vix",
        },

        conditions =
        {
            TOXIC_BUILDUP =
            {
                name = "Toxic Buildup",
                desc = "{1}’s next attack applies a Bile debuff. Whenever {1} is attacked, the Bile debuff is randomized.{2}",
                desc_fn = function( self, fmt_str )
                    if self.selected_bile then
                        local txt = self:GetLocalizedString("CURRENT_SELECTION") .. self:GetLocalizedString(self.selected_bile.id)
                        return loc.format( fmt_str, self:GetOwnerName(), txt)
                    else
                        return loc.format( fmt_str, self:GetOwnerName())
                    end
                end,
                icon = "battle/conditions/toxic_buildup1.tex",

                loc_strings =
                {
                    CURRENT_SELECTION = "\nCurrent debuff: ",
                    CORROSIVE_BILE = "{CORROSIVE_BILE}",
                    SAPPING_BILE = "{SAPPING_BILE}",
                    NOXIOUS_BILE = "{NOXIOUS_BILE}",
                },

                debuff_options = {
                    {id = "CORROSIVE_BILE", icon = "toxic_buildup1", apply_fx = {"toxic_green"}, hud_fx = {"toxic_idle_green"}}, 
                    {id = "SAPPING_BILE", icon = "toxic_buildup2", apply_fx = {"toxic_red"}, hud_fx = {"toxic_idle_red"}}, 
                    {id = "NOXIOUS_BILE", icon = "toxic_buildup3", apply_fx = {"toxic_purple"}, hud_fx = {"toxic_idle_purple"}},
                },

                OnApply = function( self, battle )
                    self.sequence = shallowcopy(self.debuff_options)
                    table.shuffle(self.sequence)
                    local choice = table.remove(self.sequence)
                    self.selected_bile = choice
                    self.icon_path = string.format( "battle/conditions/%s.tex", choice.icon )
                    self.icon = engine.asset.Texture( self.icon_path, true )
                    self.apply_fx = choice.apply_fx
                    self.hud_fx = choice.hud_fx
                end,

                OnUnapply = function( self, battle )
                    battle:BroadcastEvent( BATTLE_EVENT.CUSTOM, self.owner, function( fight_screen, ent, fighter )
                                local anim_fighter = ent.cmp.AnimFighter
                                anim_fighter:KillUpdatingFX(self.selected_bile.hud_fx[1])
                            end )
                end,

                event_handlers =
                {
                    [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit  )
                        if attack.card:IsAttackCard() and hit.target == self.owner  then
                            if self.sequence == nil or #self.sequence < 1 then
                                self.sequence = shallowcopy(self.debuff_options)
                                table.shuffle(self.sequence)
                                -- prevent the shuffled sequence from starting with our current state
                                if self.sequence[1].id == self.selected_bile.id then
                                    local streak = table.remove(self.sequence)
                                    table.insert(self.sequence, streak)
                                end
                            end
                            local previous = self.selected_bile
                            local choice = table.remove(self.sequence)
                            self.selected_bile = choice
                            self.icon_path = string.format( "battle/conditions/%s.tex", choice.icon )
                            self.icon = engine.asset.Texture( self.icon_path, true )
                            battle:BroadcastEvent( BATTLE_EVENT.REFRESH_CONDITIONS, self.owner, self )
                            battle:BroadcastEvent( BATTLE_EVENT.CUSTOM, self.owner, function( fight_screen, ent, fighter )
                                local anim_fighter = ent.cmp.AnimFighter
                                anim_fighter:KillUpdatingFX(previous.hud_fx[1])
                                anim_fighter:PlayUpdatingFX(choice.hud_fx[1])
                            end )
                        end
                    end,
                }
            },

            CORROSIVE_BILE =
            {
                name = "Corrosive Bile",
                desc = "Your {smith_regeneration} no longer heals, but is still halved at the end of your turn. Remove 1 <b>Corrosive Bile</> at the end of your turn.",

                ctype = CTYPE.DEBUFF,

                event_handlers =
                {
                    [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        if fighter == self.owner then
                            self.owner:RemoveCondition(self.id, 1, self)
                        end
                    end
                }
            },

            SAPPING_BILE =
            {
                name = "Sapping Bile",
                desc = "Temporarily remove all {POWER} and {ADRENALINE}. You cannot gain {POWER} or {ADRENALINE}. Remove 1 <b>Sapping Bile</> at the end of your turn.",

                ctype = CTYPE.DEBUFF,

                OnApply = function( self, battle )
                    self.power_stacks = self.owner:GetConditionStacks("POWER")
                    self.power_loss_stacks = self.owner:GetConditionStacks("POWER_LOSS")
                    self.adrenaline_stacks = self.owner:GetConditionStacks("ADRENALINE")
                    self.owner:RemoveCondition("POWER", nil, self)
                    self.owner:RemoveCondition("POWER_LOSS", nil, self)
                    self.owner:RemoveCondition("ADRENALINE", nil, self)
                end,

                OnUnapply = function( self, battle )
                    self.owner:AddCondition("POWER", self.power_stacks, self)
                    self.owner:AddCondition("POWER_LOSS", self.power_loss_stacks, self)
                    self.owner:AddCondition("ADRENALINE", self.adrenaline_stacks, self)
                end,

                event_priorities =
                {
                    [ BATTLE_EVENT.CALC_MODIFY_STACKS ] = 999,
                },

                event_handlers =
                {
                    [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        if fighter == self.owner then
                            self.owner:RemoveCondition(self.id, 1, self)
                        end
                    end,

                    [ BATTLE_EVENT.CALC_MODIFY_STACKS ] = function( self, acc, condition_id, fighter, source )
                        if fighter == self.owner and (condition_id == "POWER" or condition_id == "ADRENALINE" or condition_id == "POWER_LOSS") and source ~= self then
                            acc:ModifyValue( 0, self )
                        end
                    end,
                }
            },

            NOXIOUS_BILE =
            {
                name = "Noxious Bile",
                desc = "Whenever you drink, discard a random card in your hand. Remove 1 <b>Noxious Bile</> at the end of your turn.",

                ctype = CTYPE.DEBUFF,

                event_handlers =
                {
                    [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        if fighter == self.owner then
                            self.owner:RemoveCondition(self.id, 1, self)
                        end
                    end,

                    [ BATTLE_EVENT.DRINK ] = function( self, battle, count, cards )
                        local card = battle:GetHandDeck():PeekRandom()
                        if card then
                            battle:DiscardCard( card )
                        end
                    end
                }
            },

            UNSTOPPABLE =
            {
                name = "Unstoppable",
                desc = "{1} cannot be stunned. At the start of {1}'s turn, remove 1 <b>Unstoppable</>.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName())
                end,

                event_priorities =
                {
                    [ BATTLE_EVENT.CALC_MODIFY_STACKS ] = 999,
                },

                event_handlers =
                {
                    [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                        if fighter == self.owner then
                            self.owner:RemoveCondition(self.id, 1, self)
                        end
                    end,

                    [ BATTLE_EVENT.CALC_MODIFY_STACKS ] = function( self, acc, condition_id, fighter, source )
                        if condition_id == "STUN" and fighter == self.owner then
                            if acc.value > 0 then
                                acc:ModifyValue( 0, self )
                            end
                        end
                    end,
                }
            },

            VIX_SPAWN = 
            {
                hidden = true,

                event_handlers = 
                {

                    [ BATTLE_EVENT.STATUS_CHANGED ] = function( self, fighter, status )
                        if fighter:GetTeam() and fighter:GetTeam() == self.owner:GetTeam() and fighter:GetAgent():GetContentID() == "MUTANT_VIX" then
                            if status == FIGHT_STATUS.FLED or status == FIGHT_STATUS.SURRENDER or status == FIGHT_STATUS.DEAD then
                                self.owner:Flee()
                            end
                        end
                    end,
                }, 
            },
        },
        
        attacks =
        {
            vix_acid = table.extend(NPC_ATTACK)
            {
                name = "Acid Spray",
                anim = "acid",
                flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
                target_mod = TARGET_MOD.TEAM,

                base_damage = { 6, 8, 10, 10 },
                debuff_stacks = { 2, 2, 3, 4 },

                CanPlayCard = function( self, battle, target )
                    return self.owner:HasCondition("TOXIC_BUILDUP")
                end,

                OnPostResolve = function( self, battle, attack )
                    local buildup = self.owner:GetCondition("TOXIC_BUILDUP")
                    for i,hit in ipairs(attack.hits) do
                        if not hit.evaded then
                            if hit.target:IsPlayer() and buildup then
                                hit.target:AddCondition(buildup.selected_bile.id, self.debuff_stacks[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )], self)
                            else
                                hit.target:AddCondition("IMPAIR", 2, self)
                            end
                        end
                    end
                    self.owner:RemoveCondition("TOXIC_BUILDUP")
                    self.owner.behaviour.last_turn = battle:GetTurns()
                end
            },

            vix_waters = table.extend(NPC_ATTACK)
            {
                name = "Waters of Hesh",
                anim = "waters_of_hesh",
                flags = CARD_FLAGS.RANGED,
                target_mod = TARGET_MOD.CUSTOM,
                target_type = TARGET_TYPE.ENEMY,
                target_fn = function( self, battle, primary_target, target_fighters )
                    for i, fighter in battle:ActiveFighters() do
                        if fighter ~= self.owner then
                            table.insert( target_fighters, fighter )
                        end
                    end
                end,
                base_damage = { 8, 10, 12, 14 },

                event_handlers =
                {
                    [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                        if card == self and target:GetTeam() == self.owner:GetTeam() then
                            local damage = target:GetHealth() + target:GetConditionStacks("DEFEND")
                            dmgt:ModifyDamage(damage, damage, self)
                        end
                    end,
                },

                OnPostResolve = function( self, battle, attack )
                    self.owner.behaviour.last_turn = battle:GetTurns()
                end,
            },

            vix_grab = table.extend(NPC_ATTACK)
            {
                name = "Grab",
                anim = "grab",
                flags = CARD_FLAGS.MELEE,
                base_damage = { 8, 10, 12, 12 },
                OnPostResolve = function( self, battle, attack )
                    self.owner.behaviour.last_turn = battle:GetTurns()
                end,
            },

            vix_taunt =
            {
                name = "Roar",
                anim = "taunt",
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                defend_amt = { 12, 14, 16, 18},
                monster_level = { 1, 2, 2, 3 },

                summon_options = {"LUMICYTE_VIX", "HESHBLADDER_VIX"},

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("TOXIC_BUILDUP", 1, self)
                    self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                    self.owner:AddCondition("UNSTOPPABLE", 1, self)
                    local chosen = self.summon_options[math.random(#self.summon_options)]
                    local to_summon = 3 - self.owner:GetTeam():NumActiveFighters()
                    for i=1,to_summon do
                        local agent = Agent( chosen )
                        local fighter = self.owner:GetTeam():AddFighter( Fighter.CreateFromAgent( agent, self.monster_level[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )] ) )
                        fighter:AddCondition("VIX_SPAWN")
                    end
                    self.owner:GetTeam():ActivateNewFighters( battle )
                    self.owner.behaviour.last_turn = battle:GetTurns()
                end
            },
        },

        behaviour =
        {
            species_amt = {0, 1, 2, 3},

            CUSTOM_FIGHT_FORMATIONS =
            {
                [1] = MUTANT_FORMATION,
                [2] = MUTANT_FORMATION,
                [3] = MUTANT_FORMATION,
                [4] = MUTANT_FORMATION,
                [5] = MUTANT_FORMATION,
                [6] = MUTANT_FORMATION,
            },

            OnActivate = function( self )
                self.fighter:GetTeam():SetCustomFormation( self.CUSTOM_FIGHT_FORMATIONS )
                self.waters = self:AddCard("vix_waters")
                self.taunt = self:AddCard("vix_taunt")
                self.acid = self:AddCard("vix_acid")
                self.grab = self:AddCard("vix_grab")

                self.sequence = {self.taunt, self.acid, self.grab, self.grab, self.waters},

                self:SetPattern(self.Cycle)
                self.fighter:AddCondition("KRADESHI_BOON", self.species_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
                self.fighter.mute = true
            end,

            Cycle = function( self, turns )
                self.idx = self.idx or 1
                if self.last_turn and self.last_turn < turns then
                    self.idx = self.idx + 1
                    if self.idx > #self.sequence then
                        self.idx = 1
                    end
                end
                self:ChooseCard(self.sequence[self.idx])
            end,
        },
    }
}))




