local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

-- Weakness cards
Content.AddBattleCard( "weakness_blind_spot",
{
    name = "Weakness: Blind Spot",
    anim = "whip",
    flavour = "'Rightmost top horn, chipped, always throws off vision, I can use that.'",

    argument_icon = "negotiation/modifiers/arint_weakness_blind_spot.tex",

    cost = 1,

    rarity = CARD_RARITY.UNIQUE,
    flags = CARD_FLAGS.MELEE,
    series = CARD_SERIES.NPC,

    min_damage = 10,
    max_damage = 15,
})

Content.AddBattleCard( "weakness_telegraphed",
{
    name = "Weakness: Telegraphed",
    anim = "taunt",
    flavour = "'Just keep moving, that chest cannon has to fire to keep her from overheating.'",
    desc = "Gain 1 {EVASION}.",

    argument_icon = "negotiation/modifiers/arint_weakness_telegraphed.tex",

    cost = 2,

    rarity = CARD_RARITY.UNIQUE,
    flags = CARD_FLAGS.SKILL,
    series = CARD_SERIES.NPC,
    target_type = TARGET_TYPE.SELF,

    OnPostResolve = function( self, battle, attack )
        self.owner:AddCondition("EVASION", 1, self)
    end
})

Content.AddBattleCard( "weakness_dependant",
{
    name = "Weakness: Dependent",
    anim = "taunt",
    flavour = "'She really needs that staff to go off, I think that I can keep clear with enough foresight.'",
    desc = "{EXPEND} all {strange_chemical} cards in your deck.",

    argument_icon = "negotiation/modifiers/arint_weakness_dependent.tex",

    cost = 2,

    rarity = CARD_RARITY.UNIQUE,
    flags = CARD_FLAGS.SKILL,
    target_type = TARGET_TYPE.SELF,
    series = CARD_SERIES.NPC,

    OnPostResolve = function( self, battle, attack )
        local to_expend = {}
        for i,card in battle:GetHandDeck():Cards() do
            if card.id == "strange_chemical" then
                table.insert(to_expend, card)
            end
        end
        for i,card in battle:GetDrawDeck():Cards() do
            if card.id == "strange_chemical" then
                table.insert(to_expend, card)
            end
        end
        for i,card in battle:GetDiscardDeck():Cards() do
            if card.id == "strange_chemical" then
                table.insert(to_expend, card)
            end
        end

        if #to_expend > 0 then
            for i,card in ipairs(to_expend) do
                battle:ExpendCard(card)
            end
        end
    end
})

-- Strange chemical
Content.AddBattleCard( "strange_chemical", 
{
    name = "Strange Chemical",
    anim = "taunt",
    flavour = "'A strange gel that has explosive properties when reacting to high energy projections.'",
    desc = "Arint's chest cannon deals {1} bonus damage while this is in your hand.",
    desc_fn = function( self, fmt_str )
        return loc.format(fmt_str, self.chemical_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
    end,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.NPC,

    PreReq = function( self, battle )
        local arint = battle:GetEnemyTeam():Primary()
        return arint and arint:HasCondition("arint_beam_charge")
    end,

    cost = 1,
    target_type = TARGET_TYPE.SELF,

    chemical_damage = { 3, 4, 4, 5 },

    flags = CARD_FLAGS.STATUS | CARD_FLAGS.CONSUME | CARD_FLAGS.STICKY,
})

local def = CharacterDef("SPARK_SECOND",
{
    unique = true,
    base_def = "SPARK_BARON_BASE",
    title = "Lieutenant",
    alias = "SPARK_SECOND",
    theme_music = "event:/music/story/rook/story_rook_arint",
    renown = 3,
    combat_strength = 3,
    name = "Arint",
    head = "head_female_spark_arint",
    build = "female_spark_arint_build",
    gender = "FEMALE",
    species = "SHROKE",
    voice_actor = "arint",
    loved_graft = "on_the_list",
    hated_graft = "faulty_connections",
    death_item = "level_orange_clearance",

    combat_anims = {"anim/med_combat_arint.zip"},
    anims = {"anim/weapon_staff_arint.zip"},

    boss = true,
    battle_preview_anim = "anim/boss_arint_slide.zip",
    battle_preview_offset = { x = 210, y = -30 },
    battle_preview_glow = { colour = 0xE4DF55FF, bloom = 0.10, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_arint",
    icon = engine.asset.Texture( "icons/b_spark_second.tex"),

    weakness_cards = {"weakness_dependant", "weakness_telegraphed", "weakness_blind_spot"},

    negotiation_data =
    {
        behaviour =
        {
            OnInit = function( self )
                if CheckBits(self.engine:GetFlags(), NEGOTIATION_FLAGS.NO_CORE_RESOLVE) then
                    self.negotiator:AddModifier("ARINT_WEAKNESS")
                    self:SetPattern( self.BossCycle )
                else
                    self.all_business = self:AddArgument( "ALL_BUSINESS" )

                    self.attacks = self:MakePicker()
                    self.attacks:AddID( "straw_man", 1 )

                    self.bounty = self:AddArgument( "SCRUPLE" )
                    self.negotiator:AddModifier("APPROPRIATOR")
                    self:SetPattern( self.Cycle )
                end
            end,

            -- Generic Spark Baron behaviour
            Cycle = function( self, turn )
                if (turn - 1) % 6 == 0 and not self.negotiator:HasModifier( "ALL_BUSINESS" ) then
                    self:ChooseCard( self.all_business )
                    self:ChooseGrowingNumbers( 1, 0 )
                else
                    if turn % 2 == 0 then
                        self.attacks:ChooseCard( 1 )
                        self:ChooseGrowingNumbers( 1, 0 )

                        if math.random() < 0.3 and not self.negotiator:HasModifier( "SCRUPLE" ) and self.negotiator:GetResolve() ~= nil then
                            self:ChooseCard( self.bounty )
                        end
                    else
                        self:ChooseGrowingNumbers( 3, 0, 0.75 )
                    end
                end
            end,

            BossCycle = function( self, turn )
                self:ChooseComposure( 2, 4, 6 )
                self:ChooseGrowingNumbers( 3, 2)
            end,
        },
    },

    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 160,
        death_fade_delay = 1,

        conditions = 
        {
            arint_beam_charge = 
            {
                name = "Beam Charge",
                desc = "{1}'s chest piece is powered up and ready to fire.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self.owner:GetName())
                end,
            },

            arint_power_surge =
            {
                name = "Power Surge",
                desc = "{1} is on the brink of death and will overcharge her chest piece every turn, dealing damage to all enemies.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self.owner:GetName())
                end
            },
        },
        attacks = 
        {

            arint_attack1 = table.extend(NPC_ATTACK)
            {
                name = "Staff whip",
                anim = "attack1",

                flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

                CanPlayCard = function( self, battle, target )
                    return target == nil or target:IsPlayer()
                end,

                accelerant_amount = { 3, 3, 3, 3 },

                base_damage = { 7, 8, 9, 10 },

                OnPostResolve = function( self, battle, attack )
                    for i,hit in attack:Hits() do
                        if hit.target:IsPlayer() and not hit.evaded then
                            local cards = {}
                            for i=1,self.accelerant_amount[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] do
                                table.insert(cards, Battle.Card("strange_chemical", hit.target))
                            end
                            battle:DealCards(cards, battle:GetDrawDeck())
                        end
                    end
                end
            },

            arint_charge_up = table.extend(NPC_BUFF)
            {
                name = "Charge up",
                anim = "taunt",

                CanPlayCard = function( self, battle, target )
                    return not self.owner:HasCondition("arint_beam_charge")
                end,

                flags = CARD_FLAGS.BUFF | CARD_FLAGS.SKILL,

                target_type = TARGET_TYPE.SELF,

                charge_amt = { 1, 1, 2, 3 },
                defend_amt = { 10, 11, 12, 14},

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("arint_beam_charge", self.charge_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                    self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                end
            },

            arint_chest_beam = table.extend(NPC_ATTACK)
            {
                name = "Chest Beam",
                anim = "chest_beam",

                flags = CARD_FLAGS.RANGED,

                CanPlayCard = function( self, battle, target )
                    return self.owner:HasCondition("arint_beam_charge")
                end,

                base_damage = { 12, 13, 14, 15 },

                target_mod = TARGET_MOD.TEAM,

                OnPostResolve = function( self, battle, attack )
                    self.owner:RemoveCondition("arint_beam_charge", 1, self)
                    local to_expend = {}
                    for i,hand_card in battle:GetHandDeck():Cards() do
                        if hand_card.id == "strange_chemical" then
                            table.insert(to_expend, hand_card)
                        end
                    end
                    for i,card in ipairs(to_expend) do
                        battle:ExpendCard(card)
                    end
                end,

                event_handlers =
                {
                    [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                        if card == self then
                            local bonus_damage = 0
                            for i, hand_card in self.engine:GetHandDeck():Cards() do
                                if hand_card.id == "strange_chemical" then
                                    bonus_damage = bonus_damage + (hand_card.chemical_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] or 3)
                                end
                            end
                            dmgt:AddDamage(bonus_damage, bonus_damage, self)
                        end
                    end
                },
            },

            arint_surge = table.extend(NPC_ATTACK)
            {
                name = "Surge",
                anim = "surge",

                flags = CARD_FLAGS.RANGED,

                target_mod = TARGET_MOD.TEAM,

                base_damage = { 12, 13, 14, 15},
            },

            arint_ambush = table.extend(NPC_RANGED)
            {
                name = "First Strike",
                anim = "chest_beam",

                flags = CARD_FLAGS.RANGED,

                CanPlayCard = function( self, battle, target )
                    return target ==  nil or target:IsPlayer()
                end,

                base_damage = {0, 0, 0, 0},

                event_handlers = 
                {
                    [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                        if card == self then
                            local damage = math.ceil(math.max(target:GetMaxHealth() * (target:GetHealthPercent() - .65), 8))
                            dmgt:AddDamage(damage, damage, self)
                        end
                    end
                },
            },
        },


        behaviour =
        {
            OnActivate = function( self, fighter )
                self.attacks = self:MakePicker()
                    :AddID( "arint_attack1", 3)
                    :AddID( "arint_charge_up", 1)

                self.tag = self:AddCard("arint_attack1")

                self.chest_beam = self:AddCard( "arint_chest_beam")
                    
                self.surge = self:AddCard("arint_surge")

                self.ambush_attack = self:AddCard("arint_ambush")

                self:SetPattern( self.Cycle )
            end,

            Cycle = function( self )
                if self.engine:IsAmbushing(self.fighter:GetTeam()) then
                    self:ChooseCard(self.ambush_attack)
                else
                    if self.fighter:GetHealthPercent() < 0.35 then
                        self.fighter:AddCondition("arint_power_surge", 1, self)
                        self:SetPattern(self.SurgeCycle)
                        self.fighter:PlayAnim("taunt")
                        self.battle:Delay(.7)
                        self.fighter:BroadcastEvent( BATTLE_EVENT.FIGHTER_REMAP_ANIMS, self.surge_anim_mapping )
                        self:RunBehaviour(self.battle)
                    else
                        if self.fighter:HasCondition("arint_beam_charge") then
                            self:ChooseCard(self.chest_beam)
                        else
                            if self.battle:GetTurns() == 0 then
                                self:ChooseCard(self.tag)
                            else
                                self.attacks:ChooseCard()
                            end
                        end

                    end
                end
            end,

            SurgeCycle = function( self )
                self:ChooseCard(self.surge)
            end,

            surge_anim_mapping = 
            {
                idle = "idle2",
                hit_mid_pst_idle = "hit_mid_pst_idle2"
            },

        },
    },
})

Content.AddCharacterDef( def )