local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local def = CharacterDef("BRUT",
{
    unique = true,
    title = "Bruiser",
    base_def = "SPARK_BARON_BASE",
    species = SPECIES.SHROKE,
    gender = GENDER.MALE,
    renown = 1,
    combat_strength = 1,
    boss = true,


    -- faction_id = "",
    name = "Brut",

    combat_anims = {"anim/med_combat_club_brut.zip"},
    anims = {"anim/weapon_club_brut.zip"},
    
    build = "male_brut_build",
    head = "head_male_brut",

    loved_graft = "hard_knocks",
    hated_graft = "shellshock",
    death_item = "bruts_bat",

    battle_preview_anim = "anim/boss_brut_slide.zip",
    battle_preview_offset = { x = 260, y = -15, scale = 1.1 },
    battle_preview_glow = { colour = 0x33ffffFF, bloom = 0.8, fx_bloom = 0.2, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_brut",
    icon = engine.asset.Texture( "icons/b_brut.tex" ),
    -- voice_actor = "",

    fight_data = 
    {
        battle_scale = 1.15,
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 85,

        conditions = 
        { 
            DRIVE = 
            {
                name = "Drive",
                desc = "Whenever {1}'s attacks cause health loss, {1} gains <#HILITE>{2}</> {FUROR}.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName(), self.stacks)
                end,

                event_handlers =
                {
                    [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                        if attack.attacker == self.owner and not hit.defended and not hit.evaded and hit.damage and hit.damage > 0 then
                            self.owner:AddCondition("FUROR", self.stacks, self)
                        end
                        
                    end,
                },
            },

            FUROR = 
            {
                name = "Furor",
                desc = "If {1} reaches {2} <b>Furor</>, he unleashes a special attack that removes all <b>Furor</>.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName(), self.furor_cap)
                end,

                furor_cap = 10,
            },

            SINGLED_OUT = 
            {
                name = "Singled Out",
                desc = "{1}'s next attack against you will deal <#HILITE>{2}</> bonus damage. Remove {3} <b>Singled Out</> whenever you play a Maneuver card.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetRecentApplicant() and self:GetRecentApplicant().owner:GetName() or LOC "UI.CARDS.OPPONENT" , self.stacks or 1, self.decay)
                end,

                decay = 3,

                ctype = CTYPE.DEBUFF,

                -- ccc: some feedback suggests this should also force Brut to attack the owner of this condition, but the description doesn't
                -- technically mention that.  I'll just leave this code here in case.
                CanBeTargetted = function( self, card, fighter )
                    if card.owner == (self:GetRecentApplicant() and self:GetRecentApplicant().owner) and fighter ~= self.owner and card:IsAttackCard() then
                        return false
                    end
                    return true
                end,

                event_handlers =
                {
                    [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                        if attack.attacker == self.owner and attack.card:IsFlagged(CARD_FLAGS.SKILL) then
                            self.owner:RemoveCondition(self.id, math.min(self.stacks, self.decay), self)
                        end
                        if attack.attacker == (self:GetRecentApplicant() and self:GetRecentApplicant().owner) and attack.card:IsAttackCard() then
                            self.owner:RemoveCondition(self.id, self.stacks, self)
                        end
                    end,

                    [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                        local applicant = self.battle:GetFighterFromSource( self:GetRecentApplicant() )
                        if card.owner == applicant and target == self.owner then
                            dmgt:AddDamage(self.stacks, self.stacks, self)
                        end
                    end
                },
            },

            HARDKNOCKS = 
            {
                name = "Hard Knocks",
                desc = "Your next attack deals 8 bonus damage.",


                event_handlers =
                {
                    [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                        if attack.attacker == self.owner and attack.card:IsAttackCard() then
                            self.owner:RemoveCondition(self.id, self.stacks, self)
                        end
                    end,

                    [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                        if card.owner == self.owner then
                            dmgt:AddDamage(8, 8, self)
                        end
                    end
                },
            },
        },

        attacks = 
        {
            brut_swing = table.extend(NPC_MELEE)
            {
                name = "Swing",
                anim = "swing",

                damage_mult = 2.35,

                OnPostResolve = function( self, battle, attack )
                    self.owner:RemoveCondition("FUROR")
                end
            },

            brut_attack2 = table.extend(NPC_MELEE)
            {
                name = "Brutalize",
                anim = "attack2",
                hit_count = 2,

                flags = CARD_FLAGS.MELEE,

                damage_mult = 0.4,
            },

            brut_taunt = 
            {
                name = "Single Out",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.DEBUFF,

                defend_amt = { 4, 4, 5, 6 },
                singled_amt = { 8, 8, 10, 12 },

                CanPlayCard = function( self, battle, target )
                    return target == nil or target:IsPlayer()
                end,

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("DEFEND", self.defend_amt[math.min(#self.defend_amt, GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1)], self)
                    attack:AddCondition("SINGLED_OUT", self.singled_amt[math.min(#self.singled_amt, GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1)], self)
                end
            }
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                self.attacks = self:MakePicker()
                    :AddID("brut_attack2", 2)
                    :AddID("brut_taunt", 1)

                self.big_swing = self:AddCard("brut_swing")

                self:SetPattern( self.Cycle )
                self.fighter:AddCondition("DRIVE", 2)
            end,

            Cycle = function( self, turns )
                if self.fighter:HasCondition("FUROR") and self.fighter:GetConditionStacks("FUROR") >= self.fighter:GetCondition("FUROR").furor_cap then
                    self:ChooseCard(self.big_swing)
                else
                    self.attacks:ChooseCard()
                end
            end
        },
    },

    negotiation_data = 
    {
        behaviour = 
        {
            OnInit = function( self, difficulty )
                self.stubborn = self:AddArgument("STUBBORN")
                self.drunk = self:AddArgument("DRUNK")
                self.negotiator:AddModifier( "ROWDY_PATRON" )
                self:SetPattern( self.BasicCycle )
            end,

            BasicCycle = function( self, turns )
                if turns % 3 == 0 then
                    self:ChooseGrowingNumbers( 1, 4 )
                else
                    self:ChooseGrowingNumbers( 3, 2, 1 )
                end

                self:ChooseComposure( 1, 2, 3 )
                
                if (turns + 1) % 3 == 0 and not self.negotiator:HasModifier("STUBBORN") then
                    self:ChooseCard(self.stubborn)
                end
                if (turns + 1) % 2 == 0 and not self.negotiator:HasModifier("DRUNK") then
                    self:ChooseCard(self.drunk)
                end
            end,
        }
    },
})

Content.AddCharacterDef( def )
    