local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local KASHIO_FLASH_VALUES = 
{ 
    colour = 0xff00ffff,
    time_in = 0.1, 
    time_out = 0.3, 
    max_intensity = 1
}
local KASHIO_HOLOGRAM_VALUES =
{
     scale = -1.0, 
   -- colour = 0x9900ffff,
    colour = 0x9900FFff,
    alpha = {0.7, 0.3, 2.0}, -- Alpha for each layer of the FX
    speed = {0.05, 0.07, 0.5},
    layersTexture = engine.asset.Texture("rgba/character_hover.tex"),

    glitchSpeed = 0.07, 
    glitchScale = 6.0,
    glitchTexture = engine.asset.Texture("glitch/holo_glitch.tex"),

    colormatrixHue = {1.0,0.5,3.0},
    colormatrixSaturation = 0.0,
    colormatrixBrightness = 0.0,
    colormatrixContrast = 1.5,

    -- hsvtable = {
    --     highlights = {
    --         colour = {
    --             0.22352941176471,
    --             0.36470588235294,
    --             1.0
    --         },
    --         hsv = {
    --             0.34900000691414,
    --             0.0,
    --             0.83399999141693
    --         }
    --     },
    --     midtones = {
    --         colour = {
    --             0.83137254901961,
    --             0.57647058823529,
    --             1.0
    --         },
    --         hsv = {
    --             0.49300000071526,
    --             0.0,
    --             0.63899999856949
    --         }
    --     },
    --     shadows = {
    --         colour = {
    --             0.91764705882353,
    --             0.84313725490196,
    --             1.0
    --         },
    --         hsv = {
    --             0.59799998998642,
    --             0.0,
    --             0.052999999374151
    --         }
    --     }
    -- },    
}

local function AddHoloEffect( anim_fighter )
    local x, y = anim_fighter.sim:GetCamera():WorldToScreen( anim_fighter:GetStatusWidgetPosition() )
    local width = TheGame:FE():GetScreenDims()
    local screen_pos = math.abs(x)/width
    local pan_pos = easing.linear( screen_pos, -1, 2, 1 )
    AUDIO:PlayParamEvent("event:/sfx/battle/atk_anim/kashio/hologram_on", "position", pan_pos)
    anim_fighter:Flash(KASHIO_FLASH_VALUES.colour, KASHIO_FLASH_VALUES.time_in, KASHIO_FLASH_VALUES.time_out, KASHIO_FLASH_VALUES.max_intensity)
    anim_fighter:SetHologramEffect(true, KASHIO_HOLOGRAM_VALUES)
end

local function RemoveHoloEffect( fighter )
    fighter.battle:BroadcastEvent( BATTLE_EVENT.CUSTOM, fighter, function( fight_screen, ent )
        local anim_fighter = ent.cmp.AnimFighter
        local x, y = anim_fighter.sim:GetCamera():WorldToScreen( anim_fighter:GetStatusWidgetPosition() )
        local width = TheGame:FE():GetScreenDims()
        local screen_pos = math.abs(x)/width
        local pan_pos = easing.linear( screen_pos, -1, 2, 1 )
        AUDIO:PlayParamEvent("event:/sfx/battle/atk_anim/kashio/hologram_off", "position", pan_pos)
        anim_fighter:Flash(KASHIO_FLASH_VALUES.colour, KASHIO_FLASH_VALUES.time_in, KASHIO_FLASH_VALUES.time_out, KASHIO_FLASH_VALUES.max_intensity) 
        anim_fighter:SetHologramEffect(false)
    end )
end

local function HologramDeath( fighter )
    
    fighter.battle:BroadcastEvent( BATTLE_EVENT.CUSTOM, fighter, function( fight_screen, ent )
        local anim_fighter = ent.cmp.AnimFighter
        anim_fighter:StopMoving()
        anim_fighter:PlayAnim( "death_holo" )

        local x, y = anim_fighter.sim:GetCamera():WorldToScreen( anim_fighter:GetStatusWidgetPosition() )
        local width = TheGame:FE():GetScreenDims()
        local screen_pos = math.abs(x)/width
        local pan_pos = easing.linear( screen_pos, -1, 2, 1 )
        AUDIO:PlayParamEvent("event:/sfx/battle/atk_anim/kashio/hologram_off", "position", pan_pos)
        
        anim_fighter.entity.cmp.AnimController:HideOutline()
        anim_fighter:SetAnimOutline(3, 0x000000ff)

        if anim_fighter.status_widget then
            anim_fighter.status_widget:AlphaTo( 0, 0.5 )
        end

        local t = 0 
        local total_t = 0.25
        while t < total_t do
            local dt = coroutine.yield()
            t = t + dt
            local alpha = 1 - (t/total_t)
            anim_fighter.entity.cmp.AnimController:SetTintMult(1,1,1, alpha)
            if anim_fighter.dropshadow then
                anim_fighter.dropshadow.cmp.PlaneRenderer:TintColour(0,0,0, .5*alpha)
            end
        end

        -- anim_fighter:WaitAnim()
        anim_fighter.entity:Despawn()
    end )
    -- if math.random() < .5 then
    --     fighter.parent.behaviour:SaySpeech(0,  )
    -- end
    local team = fighter.team
    fighter.status = FIGHT_STATUS.DEAD
    team:RemoveFromField( fighter )
    local holos_left = 0
    for i,enemy in team:Fighters() do
        if enemy.agent:GetContentID() == "HOLO_KASHIO" then
            holos_left = holos_left + 1
        end
    end
    if holos_left == 0 then
        RemoveHoloEffect(fighter.parent)
    end
end


KASHIO_FIGHT_DATA = 
{
    MAX_MORALE = MAX_MORALE_LOOKUP.VERY_HIGH,
    MAX_HEALTH = 220,
    anim_mapping =
    {
        riposte = "gun1",
    },
    ranged_riposte = true,

    cards = { "kashio_flail_crack", "kashio_flail_smash" },
    
    conditions = 
    {
        kashio_afterburner_gloves =
        {
            name = "Afterburner Gloves",
            desc = "All {1}'s attacks apply {2} {BURN} if not mitigated.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self:GetOwnerName(), self.stacks * self.burn_amount or 1)
            end,

            burn_amount = 3,

            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.card:IsAttackCard() and attack.card.id ~= "counter_attack" and attack.attacker == self.owner then
                        if not hit.evaded and not hit.defended then
                            hit.target:AddCondition( "BURN", self.burn_amount * self.stacks, attack.card )
                        end
                    end
                end,
            },
        },

        kashio_blast_chamber = 
        {
            name = "Blast Chamber",
            desc = "{1} attacks one additional time per turn.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, loc.cap( self:GetOwnerName() ))
            end,

            -- To avoid doubling Kashio's DPS, apply a .9 damage mod to her attacks
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner then
                        dmgt:ModifyDamage( math.round( dmgt.min_damage * 0.9 ),
                                       math.round( dmgt.max_damage * 0.9 ),
                                       self )
                    end
                end
            },
        },

        kashio_macro_rasp =
        {
            name = "Macro Rasp",
            desc = "All {1}'s attacks apply {2} {BLEED} if not mitigated.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self:GetOwnerName(), self.bleed_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] or 1)
            end,

            bleed_amt = { 1, 1, 2, 2 },

            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.card:IsAttackCard() and attack.card.id ~= "counter_attack" and attack.attacker == self.owner then
                        if not hit.evaded and not hit.defended then
                            if hit.target:IsPlayer() then
                                local cards = {}
                                for i=1, self.bleed_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] do
                                    local incepted_card = Battle.Card( "status_bleed", hit.target)
                                    table.insert(cards, incepted_card )                                  
                                end
                                battle:DealCards( cards, battle:GetDiscardDeck() )
                                self.has_spoken = false
                            else
                                hit.target:AddCondition("BLEED", self.bleed_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], attack)
                            end
                        end
                    end
                end,
            },
        },

        kashio_hammerthyst_ring = 
        {
            name = "Hammerthyst Ring",
            desc = "At the start of your turn, {1} gains {2} {DEFEND}.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self:GetOwnerName(), self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
            end,

            defend_amt = { 8, 10, 15, 20},

            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, fighter )
                    self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] )
                end
            },
        },

        kashio_sage_conversion =
        {
            name = "Sage Conversion",
            desc = "At the start of battle, gain 80 health.",

            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    if not self.applied and not self.owner.parent then
                        self.owner:DeltaMaxHealth( 80 )
                        self.owner:DeltaHealth( 80 )
                        self.applied = true
                    end
                end
            },
        },

        kashio_dermal_cast =
        {
            name = "Dermal Cast",
            desc = "Whenever {1} gains a debuff, {1} heals {2} health.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self:GetOwnerName(), self.heal_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
            end,

            heal_amt = { 2, 3, 4, 5 },

            event_handlers = 
            {
                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if fighter == self.owner and condition.ctype == CTYPE.DEBUFF and not condition.hidden and not self.owner.parent then
                        self.owner:HealHealth(self.heal_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
                    end
                end
            },
        },

        kashio_holo_image = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if (hit.target == self.owner or hit.target.agent:GetContentID() == "KASHIO") and attack.card:IsAttackCard() then
                        HologramDeath(self.owner)
                    end
                end,

                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if self.owner == fighter then
                        HologramDeath(self.owner)
                    end
                end,

                [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
                    if (fighter == self.owner or fighter.agent:GetContentID() == "KASHIO") and damage > 0 then
                        HologramDeath(self.owner)
                    end
                end,

                [ BATTLE_EVENT.DELTA_STAT ] = function( self, fighter, stat, delta, value, mitigated  )
                    if fighter.agent:GetContentID() == "KASHIO" and stat == COMBAT_STAT.HEALTH then
                        self.owner:SetMaxHealth(fighter:GetMaxHealth())
                        self.owner:SetHealthPercent(fighter:GetHealthPercent())
                    end
                end,
            },
        },

        kashio_hologram_belt = 
        {
            name = "Hologram Projection Belt",
            desc = "At the end of {1}'s turn, summons {2} holograms of {1}.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.owner:GetName(), self.clone_count[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
            end,

            clone_count = { 1, 2, 3, 3 },

            hide_gained = true,
        },

        kashio_pistol =
        {
            name = "Sonic Pistol",
            desc = "At the start of every second of {1}'s turns, {1} gains {2} {RIPOSTE}.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self:GetOwnerName(), self.counter_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
            end,

            hide_gained = true,

            counter_amt = { 2, 4, 6, 8 },

            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.skip then
                            self.skip = false
                        else
                            self.owner:AddCondition("RIPOSTE", self.counter_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                            self.skip = true
                        end
                    end
                end,
            },
        },

        kashio_glaive =
        {
            name = "Power Glaive",
            desc = "{1}'s Power Glaive deals additional damage.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self:GetOwnerName())
            end,
            hide_gained = true,
        },

        kashio_suitcase_grenades =
        {
            name = "Suitcase Grenades",
            desc = "At the end of {1}'s turn, {1} throws a grenade.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self:GetOwnerName())
            end,
            hide_gained = true,
        },

        kashio_robot_backup =
        {
            name = "Robot Backup",
            hidden = true,
        },
    },

    attacks = 
    {
        kashio_flail_smash = table.extend(NPC_MELEE)
        {
            name = "Smash",
            anim = "smash",
            flags = CARD_FLAGS.MELEE,

            base_damage = { 8, 10, 12, 14 },
            CanPlayCard = function(self, battle, target)
                return battle:GetPlayerTeam():NumActiveFighters() <= 1
            end,
        },

        kashio_flail_crack = table.extend(NPC_MELEE)
        {
            name = "Crack",
            anim = "crack",
            flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,
            base_damage = { 6, 8, 10, 12 },

            features =
            {
                WOUND = 2,
            },
        },

        kashio_flail_slam = table.extend(NPC_MELEE)
        {
            name = "Slam",
            anim = "slam",
            flags = CARD_FLAGS.MELEE,

            CanPlayCard = function(self, battle, target)
                return battle:GetPlayerTeam():NumActiveFighters() > 1
            end,

            target_mod = TARGET_MOD.TEAM,

            base_damage = { 6, 8, 10, 12 },
        },

        kashio_glaive_1 = table.extend(NPC_MELEE)
        {
            name = "Slicing Glaive",
            anim = "spin_attack",
            flags = CARD_FLAGS.MELEE,

            base_damage = { 12, 15, 17, 20 },
        },

        kashio_glaive_2 = table.extend(NPC_MELEE)
        {
            name = "Piercing Glaive",
            anim = "slash_up",
            flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,
            base_damage = { 8, 10, 12, 14 },

            features =
            {
                EXPOSED = 2,
            },
        },

        kashio_grenade = table.extend(NPC_ATTACK)
        {
            name = "Suitcase Grenade",
            anim = "throw1",
            flags = CARD_FLAGS.RANGED,
            base_damage = { 4, 6, 8, 10 },
            target_mod = TARGET_MOD.TEAM,
        },

        kashio_holo_belt_active = table.extend(NPC_BUFF)
        {
            name = "Holo Belt",
            anim = "taunt",
            clone_count = {1, 2, 3, 3},
            synchronous = true,

            -- target_type = TARGET_TYPE.SELF,

            OnPostResolve = function( self, battle, attack )
                if self.owner:IsActive() and battle.result == nil then
                    local first_clone
                    local all_clones = {}
                    local team = self.owner:GetTeam()
                    local max_clones = team:GetMaxFighters() - team:NumActiveFighters()
                    local num_clones = math.min( max_clones, self.clone_count[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
                    
                    for i=1, num_clones do
                        local agent = Agent("HOLO_KASHIO")
                        local new_fighter = Fighter.CreateFromAgent( agent )
                        new_fighter.parent = self.owner
                        new_fighter.mute = true
                        if i == num_clones then 
                            new_fighter.last_clone = true
                            new_fighter.first_clone = first_clone 
                        elseif i == 1 then
                            first_clone = new_fighter
                        end
                        new_fighter:AddCondition("kashio_holo_image")
                        new_fighter:SetMaxHealth(self.owner:GetMaxHealth())
                        new_fighter:SetHealthPercent(self.owner:GetHealthPercent())
                        self.owner:GetTeam():AddFighter( new_fighter )
                        self.owner:GetTeam():ActivateNewFighters()
                        new_fighter:ClearPreparedCards()
                        table.insert(all_clones, new_fighter)
                    end
                    table.insert(all_clones, self.owner)
                    for i,clone in ipairs(all_clones) do
                        clone:PlayAnim("holo_spawn_2")
                    end
                end
            end,
        },

        kashio_robot = table.extend(NPC_BUFF)
        {
            name = "Summon Mech",

            target_type = TARGET_TYPE.SELF,
            flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,

            mech_level = { 1, 1, 2, 3 },

            OnPostResolve = function( self, battle, attack )
                local agent = Agent("SPARK_BARON_AUTOMECH")
                local fighter = Fighter.CreateFromAgent( agent, self.mech_level[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] )
                fighter:AddCondition("no_alone_drone")
                self.owner:GetTeam():AddFighter( fighter )
                self.owner:GetTeam():ActivateNewFighters()
                self.owner.spawned_mech = true
            end
        },

        kashio_glaive_swap = table.extend(NPC_BUFF)
        {
            name = "Power Glaive",
            anim = "transition1",
            flags = CARD_FLAGS.BUFF | CARD_FLAGS.SKILL,
            target_type = TARGET_TYPE.SELF,

            PostPresAnim = function( self, anim_fighter )
                anim_fighter:SetAnimMapping(self.owner.agent.fight_data.anim_mapping_glaive)
            end,

            OnPostResolve = function( self, battle, attack )
                self.owner.swapped_to_glaive = true
            end
        },
    },

    behaviour =
    {
        -- possible_items =
        -- {
        --     "kashio_pistol",
        --     "kashio_hologram_belt",
        --     -- "kashio_suitcase_grenades",
        --     "kashio_robot_backup",
        -- },
        -- possible_grafts =
        -- {
        --     "kashio_afterburner_gloves",
        --     "kashio_blast_chamber",
        --     "kashio_dermal_cast",
        --     "kashio_hammerthyst_ring",
        --     "kashio_macro_rasp",
        --     "kashio_sage_conversion",
        -- },
        OnActivate = function( self, fighter )
            self.flail_attacks = self:MakePicker()
                :AddID( "kashio_flail_crack", 1)
                :AddID( "kashio_flail_smash", 2)
                :AddID( "kashio_flail_slam", 2)
            self.glaive_attacks = self:MakePicker()
                :AddID( "kashio_glaive_1", 3)
                :AddID( "kashio_glaive_2", 1)
            self.glaive_swap = self:AddCard( "kashio_glaive_swap" )
            self.grenade = self:AddCard( "kashio_grenade" )
            self.spawn_robot = self:AddCard( "kashio_robot" )
            self.holograms = self:AddCard( "kashio_holo_belt_active" )
            self:SetPattern( self.Cycle )

            -- Purely for testing. The quest will eventually handle this.
            -- if not self.fighter.parent then
            --     local chosen_items = table.multipick(self.possible_items, 3)
            --     local chosen_grafts = table.multipick(self.possible_grafts, 2)
            --     for k,id in pairs(chosen_items) do
            --         self.fighter:AddCondition(id)
            --     end
            --     for k,id in pairs(chosen_grafts) do
            --         self.fighter:AddCondition(id)
            --     end
            -- end
        end,

        Cycle = function( self )
            if self.fighter.anim_fighter then
                self.fighter.anim_fighter.hide_condition_gained = nil -- Ensures that hologram clones show new conditions gained
                self.fighter.anim_fighter.no_condition_sound = nil
            end
            if (self.fighter:HasCondition("kashio_robot_backup") and not self.fighter.spawned_mech) or (self.fighter:HasCondition("kashio_glaive") and not self.fighter.swapped_to_glaive and not self.fighter.parent) then
                if self.fighter:HasCondition("kashio_robot_backup") then
                    self:ChooseCard( self.spawn_robot )
                end
                if self.fighter:HasCondition("kashio_glaive") then 
                    self:ChooseCard( self.glaive_swap )
                    self.glaive_attacks:ChooseCard()
                end
            else
                if self.fighter:HasCondition("kashio_glaive") then
                    self.glaive_attacks:ChooseCards( self.fighter:HasCondition("kashio_blast_chamber") and 2 or 1)
                else
                    self.flail_attacks:ChooseCards( self.fighter:HasCondition("kashio_blast_chamber") and 2 or 1)
                end
            end

            if self.fighter:HasCondition("kashio_suitcase_grenades") then
                self:ChooseCard(self.grenade)
            end
            if self.fighter:HasCondition("kashio_hologram_belt") then
                self:ChooseCard(self.holograms)
            end
        end,
    },

    anim_mapping_glaive =
    {
        idle = "idle2",
        defend = "defend2",
        death = "death2",
        hit_mid = "hit_mid2",
        hit_mid_pst_idle = "hit_mid2_pst_idle",
        hit_mid_pst_stunned = "hit_mid2_pst_stunned",
        hit_mid_pst_surrender = "hit_mid2_pst_surrender",
        run_forward = "run_forward2",
        run_forward_pre = "run_forward2_pre",
        run_forward_pre_turn = "run_forward2_pre_turn",
        run_forward_pst = "run_forward2_pst",
        run_forward_pst_turn = "run_forward_pst2_turn",
        step_back = "step_back2",
        step_forward = "step_forward2",
        stunned = "stunned2",
        stunned_pre = "stunned2_pre",
        stunned_pst = "stunned2_pst",
        surrender_pre = "surrender2_pre",
        surrender_pst = "surrender2_pst",
        surrender = "surrender2",
        riposte = "gun2",
        throw1 = "throw2",
        death_holo = "death_holo2",
        taunt = "taunt4",
        holo_spawn_2 = "holo_spawn",
    },
}

local def = CharacterDef("KASHIO",
{
    base_def = "SPARK_BARON_BASE",
    alias = "KASHIO",
    unique = true,
    title = "Freelancer",
    shorttitle = "Freelancer",
    species = SPECIES.SHROKE,
    gender = GENDER.FEMALE,
    build = "female_kashio",
    head = "head_female_kashio",
    skin_colour = 1768522495,
    theme_music = "event:/music/story/sal/story_sal_kashio",

    renown = 5,
    combat_strength = 5,

    voice_actor = "kashio",

    hated_graft = "troubled_past",
    boss = true,

    battle_preview_anim = "anim/boss_kashio_slide.zip",
    battle_preview_offset = { x = 365, y = 50 },
    battle_preview_glow = { colour = 0xFF9501ff, bloom = 0.3, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_kashio",
    icon = engine.asset.Texture( "icons/b_kashio.tex"),

    name = "Kashio",
    tags = {"plot_armour", "no_friends", "no_patron"},
    faction_id = "SPARK_BARONS",
   
    combat_anims = {"anim/med_combat_flail_kashio.zip"},
    anims = { "anim/weapon_flail_kashio.zip"},

    fight_data = KASHIO_FIGHT_DATA,

    negotiation_data =
    {
        -- Big Up-Front damage, then cycles Composure/Damage.
        -- Incepts Doubt.

        modifiers =
        {
            HOLOGRAM_BELT_ARGUMENT =
            {
                name = "Hologram Belt",
                desc = "At the end of every other turn, {1} creates duplicates of her core argument that disappear when attacked or at the end of {1}'s next turn.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName())
                end,

                modifier_type = MODIFIER_TYPE.CORE,

                target_enemy = TARGET_ANY_RESOLVE,
                min_persuasion = 1,
                max_persuasion = 2,

                OnBeginTurn = function( self, minigame )
                    self:ApplyPersuasion()
                end,

                holos = { 1, 2, 3, 4 },

                event_priorities =
                {
                    [ EVENT.END_TURN ] = 999,
                },

                event_handlers =
                {
                    [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                        if negotiator == self.negotiator then
                            if not self.original then
                                self.negotiator:RemoveModifier(self, 1, self)
                            end
                            local cur, max = self:GetResolve()
                            if (minigame:GetTurns() - 1) % 2 == 0 then
                                local to_add = self.holos[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.holos)]
                                local chosen_argument = math.random(1,to_add + 1)
                                for i=1,to_add do
                                    local mod = self.negotiator:CreateModifier("HOLOGRAM_BELT_ARGUMENT", 1, self)
                                    if i == chosen_argument then
                                        mod.original = true
                                    else
                                        mod.original = false
                                    end
                                    mod:SetResolve(cur, nil, max)
                                end
                                if chosen_argument == to_add + 1 then
                                    self.original = true
                                else
                                    self.original = false
                                end
                            end
                        end
                    end,

                    [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                        if target == self then
                            if not self.original then
                                self.negotiator:RemoveModifier(self, 1, self)
                            else
                                local to_remove = {}
                                for i, modifier in ipairs( self.negotiator.all_modifiers ) do
                                    if modifier.id == "HOLOGRAM_BELT_ARGUMENT" and not modifier.original then
                                        table.insert(to_remove, modifier)
                                    end
                                end
                                for i,modifier in ipairs(to_remove) do
                                    self.negotiator:RemoveModifier(modifier, modifier.stacks, self)
                                end
                            end
                        end
                    end,
                },
            },

            GUILT_TRIP =
            {
                name = "Guilt Trip",
                desc = "Whenever you play a card that deals damage, this deals {1} damage to your core argument.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self.damage_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.damage_amt)])
                end,

                max_resolve = 8,
                modifier_type = MODIFIER_TYPE.ARGUMENT,

                damage_amt = { 1, 1, 2, 3 },

                event_handlers =
                {
                    [ EVENT.POST_RESOLVE ] = function(self, minigame, card)
                        if self.anti_negotiator and card.min_persuasion and card.max_persuasion and card.negotiator == self.anti_negotiator then
                            self.anti_negotiator:AttackResolve( self.damage_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.damage_amt)], self )
                        end
                    end,
                },
            },

            MISANTHROPIC =
            {
                name = "Misanthropic",
                desc = "Whenever this argument is attacked, its damage is increased by {1}.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.damage_amt )
                end,

                max_resolve = 12,
                modifier_type = MODIFIER_TYPE.ARGUMENT,
                target_enemy = TARGET_ANY_RESOLVE,

                min_persuasion = 1,
                max_persuasion = 3,

                damage_amt = 2,

                OnApply = function( self )
                    self:PrepareTurn()
                end,
                OnBeginTurn = function( self, minigame )
                    self:ApplyPersuasion()
                end,

                event_handlers =
                {
                    [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                        if target == self then
                            self.counter = (self.counter or 0) + 1
                            self.engine:BroadcastEvent( EVENT.CUSTOM, function( panel, ... )
                                    panel.opponent_modifiers:UpdatePersuasionLabels()
                                end )
                        end
                    end,

                    [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion, minigame, target )
                        if source == self and self.counter and self.counter > 0 then
                            persuasion:AddPersuasion( self.counter * self.damage_amt, self.counter * self.damage_amt, self)
                        end
                    end
                },
            },
        },

        behaviour =
        {
            OnInit = function( self )
                if CheckBits( self.engine.start_params.flags, NEGOTIATION_FLAGS.WORDSMITH ) then
                    self.guilt = self:AddArgument( "GUILT_TRIP" )
                    self.misanthropic = self:AddArgument("MISANTHROPIC")
                    self:SetPattern( self.BossCycle )
                    local mod = self.negotiator:AddModifier("HOLOGRAM_BELT_ARGUMENT")
                    mod.original = true
                else
                    self.negotiator:DeltaComposure( 5 )

                    self.doubt = self:AddArgument( "DOUBT" )
                    self.doubt.stacks = 3

                    self:SetPattern( self.Intro )
                end

            end,

            Intro = function( self, turns )
                self:SetPattern( self.Cycle )
                self:ChooseComposure( 1, 2, 4 )
                self:ChooseFlatNumbers( 2, 5 )
            end,

            Cycle = function( self, turns )
                self:ChooseComposure( 1, 2, 4 )

                if turns % 2 == 0 then
                    self:ChooseCard( self.doubt )
                end
                self:ChooseGrowingNumbers( 1, -1 )
            end,

            BossCycle = function( self, turns )
                if turns % 2 == 0 then
                    self:ChooseComposure( 1, 2, 3 )
                end
                self:ChooseFlatNumbers( 1, 1 )
                if turns % 3 == 0 and not self.negotiator:HasModifier("GUILT_TRIP") then
                    self:ChooseCard(self.guilt)
                end
                if (turns - 1) % 3 == 0 and not self.negotiator:HasModifier("MISANTHROPIC") then
                    self:ChooseCard(self.misanthropic)
                end
            end,
        }
    },
})

Content.AddCharacterDef( def )


local holo_def = CharacterDef("HOLO_KASHIO",
{
    base_def = "KASHIO",
    name = "Kashio",
    build = "female_kashio",
    head = "head_female_kashio",
    skin_colour = 1768522495,
    hide_in_compendium = true,
    is_hologram = true,

    fight_data = table.extend(KASHIO_FIGHT_DATA)
    {
        OnJoinBattle = function( fighter, anim_fighter )
            
            local fight_mgr = anim_fighter.sim.mgr
            local delays = table.shuffle{.1, .2, 0}

            local parent_ent = fight_mgr:GetAnimFighterByFighter( fighter.parent )
            if parent_ent then
                anim_fighter.dropshadow:Hide()
                if anim_fighter.status_widget then 
                    anim_fighter.status_widget:AlphaTo(1,0)
                end
                anim_fighter.hide_condition_gained = true -- don't show the fighter gaining these
                anim_fighter.no_condition_sound = true
                fighter.anim_fighter = anim_fighter
                for i, condition_w in ipairs( parent_ent.cmp.AnimFighter.status_widget.condition_widgets ) do
                    if condition_w:GetCondition() ~= "SHROKE_BOON" then 
                        fighter:AddCondition( condition_w:GetCondition(), fighter.parent:GetConditionStacks(condition_w:GetCondition()) ) 
                    end
                end
                if fighter:HasCondition("kashio_glaive") then
                    anim_fighter:SetAnimMapping(fighter.agent.fight_data.anim_mapping_glaive)
                end
                fighter:SetMaxHealth(fighter.parent:GetMaxHealth())
                fighter:SetHealthPercent(fighter.parent:GetHealthPercent())
                fighter:SetStat(COMBAT_STAT.MORALE, (fighter.parent:GetMorale() or 0) )
                local x, y, z = parent_ent:GetLocalPosition()
                anim_fighter:SetPos(parent_ent:GetLocalPosition())
                if fighter.last_clone then
                    for i,ally in ipairs(fighter:GetTeam():GetFighters()) do
                        if ally.agent:GetContentID() == "KASHIO" or ally.agent:GetContentID() == "HOLO_KASHIO" then
                            local ent = fight_mgr:GetAnimFighterByFighter(ally)
                            if ent then 
                                ent.cmp.AnimFighter:SetPos( x, z )
                            end
                        end
                    end
                    if fighter.battle then
                        table.shuffle(fighter.battle.active_fighters)
                    end
                    table.shuffle(fighter:GetTeam().fighters)
                    anim_fighter.sim.mgr:CalculateHomePositions(fighter:GetTeam(), false)
                    if fighter.first_clone then
                        local ent = fight_mgr:GetAnimFighterByFighter(fighter.first_clone)
                        if ent then
                            ent.cmp.AnimFighter:DoSequence( AnimFighter.ResetFighter, ent.cmp.AnimFighter, nil, table.remove(delays) )
                        end
                    end
                    parent_ent.cmp.AnimFighter:DoSequence( AnimFighter.ResetFighter, parent_ent.cmp.AnimFighter, nil, table.remove(delays) )
                    AddHoloEffect(parent_ent.cmp.AnimFighter)
                end
                anim_fighter.dropshadow:Show()
                AddHoloEffect(anim_fighter)
            end
        end,
    },


})

Content.AddCharacterDef( holo_def )