local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT


local NADAN_RETRACT_TURNS = 3
local NADAN_CLAW_TURNS = 3
local BACKUP_LEVEL = 3
local NADAN_DEFENSE_AMOUNT = { 6, 8, 10, 12 }

local REINFORCMENT_TABLE = 
{
    {"BANDIT_GOON", "FLEAD"},
    {"BANDIT_GOON2", "FLEAD"},
    {"BANDIT_GOON2", "BANDIT_GOON"},
    {"BANDIT_GOON", "BANDIT_RAIDER"},
    {"BANDIT_GOON2", "BANDIT_RAIDER"},
}



local def = CharacterDef("MURDER_BAY_BANDIT_CONTACT",
{
    alias = "MURDER_BAY_BANDIT_CONTACT",
    unique = true,
    theme_music = "event:/music/story/sal/story_sal_nadan",
    base_def = "BANDIT_RAIDER",
    species = SPECIES.HUMAN,
    gender = GENDER.MALE,
    renown = 4,
    combat_strength = 4,
    boss = true,
    battle_preview_anim = "anim/boss_nadan_slide.zip",
    battle_preview_offset = { x = 150, y = -40 },
    battle_preview_glow = { colour = 0xFE8500FF, bloom = 0.3, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_nadan",
    icon = engine.asset.Texture( "icons/b_murder_bay_bandit_contact.tex"),

    voice_actor = "nadan",
    title = "Leader",
    name = "Nadan",
    bio = "It's not clear where, exactly, Nadan falls on the Spree pecking order, but he's important enough to get you what you want in Murder Bay as long as you stay in his good books.\nDespite being a rotten crook, his general demeanor is much more charming than his blaster-burned grin.",

    hair_colour = 2370454527,
    head = "head_telvin",
    skin_colour = 2911199743,
    build = "male_nadan",

    loved_graft = "chieftain",
    hated_graft = "kill_list",
    death_item = "nadans_blade",

    combat_anims = {"anim/med_combat_claws_nadan.zip"},
    anims = {"anim/weapon_claws_nadan.zip"},

    fight_data = 
    {
        battle_scale = 1.1,
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 250,
        formation = FIGHTER_FORMATION.BACK_X,

        status_widget_dx = 0.8,

        conditions = 
        {

            nadan_enraged = 
            {
                hidden = true,
                hide_gained = true,

                OnApply = function( self, battle )
                    battle:BroadcastEvent( BATTLE_EVENT.UPDATE_HOME_POSITIONS, self.owner:GetTeam())
                    --sim.mgr:CalculateHomePositions( self.owner:GetTeam(), true )
                end,

                event_priorities =
                {
                    [ BATTLE_EVENT.END_TURN ] = 0, -- before stun is removed
                },

                event_handlers = 
                {
                    [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                        if condition == self and fighter == self.owner then
                            fighter:BroadcastEvent( battle_defs.BATTLE_EVENT.FIGHTER_REMAP_ANIMS, fighter.agent.fight_data.anim_mapping_enraged )
                        end
                        if condition.id == "SURRENDER" then
                            fighter:BroadcastEvent( battle_defs.BATTLE_EVENT.FIGHTER_REMAP_ANIMS )
                        end
                    end,

                    [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        if fighter == self.owner and not self.owner:HasCondition("SURRENDER") and not self.owner:HasCondition("STUN") then
                            if self.stacks == 1 then
                                local card = Battle.Card("nadan_end_rage", self.owner)
                                self.battle:PlayCard(card)
                            else
                                self.owner:RemoveCondition("nadan_enraged", 1)
                            end
                        end
                    end,
                },
            },
            nadan_retracted = 
            {
                --name = "Retract",
                --desc = "Nadan's sparklers have retracted. He'll have them back again in {1} {1*turn|turns}, or when his backup is killed.",
                hidden = true,
                hide_gained = true,
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.stacks or 1 )
                end,

                OnApply = function( self, battle )
                    battle:BroadcastEvent( BATTLE_EVENT.UPDATE_HOME_POSITIONS, self.owner:GetTeam())
                    --sim.mgr:CalculateHomePositions( self.owner:GetTeam(), true )
                end,

                event_handlers = 
                {
                    [ BATTLE_EVENT.END_ENEMY_TURN ] = function( self )
                        if self.owner:HasCondition("SURRENDER") then
                            return
                        end
                        
                        local no_more_friends = #self.owner:GetTeam().fighters == 1
                        
                        if self.stacks == 1 or no_more_friends and self.stacks <= NADAN_RETRACT_TURNS then
                            
                            local card = Battle.Card("nadan_enrage", self.owner)
                            self.battle:PlayCard(card)

                            self.battle:BroadcastEvent( BATTLE_EVENT.DELAY, .75)
                            
                            --Make all of the other fighters run away
                            local fighters = self.owner:GetTeam().fighters
                            local to_flee = {}
                            for _, fighter in ipairs(fighters) do
                                if fighter.nadan_backup and fighter ~= self.owner and not fighter:IsPlayer() then
                                    table.insert(to_flee, fighter)
                                end
                            end

                            if #to_flee > 0 then 
                                if self.owner:GetTeamID() == TEAM.RED then
                                    self.owner.behaviour:SayQuip( "nadan_dismiss" )
                                end
                                self.battle:BroadcastEvent( BATTLE_EVENT.DELAY, .5)
                                for k,fighter in pairs(to_flee) do
                                    fighter:Flee()
                                end
                            else
                                self.owner.behaviour:SayQuip( "nadan_enrage" )
                            end

                            self.battle:BroadcastEvent( BATTLE_EVENT.DELAY, .5)

                        else
                            self.owner:RemoveCondition("nadan_retracted", 1)
                        end
                    end,
                },
            },

            sentry =
            {
                name = "Sentry",
                desc = "Whenever {1} is attacked, {2} deals <#HILITE>{3}</> damage to the attacker.",
                desc_fn = function( self, fmt_str )
                    local applier = self:GetRecentApplicant()
                    return loc.format(fmt_str, applier:GetName() or "", self:GetOwnerName(), self.stacks)
                end,

                -- Basically modified RIPOSTE
                event_handlers =
                {
                    [ BATTLE_EVENT.ON_HIT] = function( self, battle, attack, hit )
                        local resolve_card = attack.card
                        if self.owner:IsActive() and not self.owner:HasCondition( "STUN" ) and not self.owner:HasCondition("SURRENDER") then
                            if resolve_card.owner and resolve_card.owner ~= self.owner and not resolve_card.owner.call_in and resolve_card:IsAttackCard() and hit.damage and (hit.damage > 0 or (attack.card.max_damage and attack.card.max_damage > 0)) and not attack.is_counter then
                                --
                                if hit.target == self:GetRecentApplicant() then
                                    self.counter_attack_counter = (self.counter_attack_counter or 0) + 1
                                    attack.riposted = true
                                end
                            end
                        end
                    end,

                    [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, resolve_card )
                        if self.counter_attack_counter and self.counter_attack_counter > 0 then
                            self.owner:CheckForSurrender()
                            if self.owner:IsActive() and not self.owner:HasCondition( "STUN" ) and not self.owner:HasCondition("SURRENDER") and resolve_card.owner:IsActive() then
                                --
                                local card = Battle.Card( "counter_attack", self.owner )
                                card.engine = battle -- HAX
                                card.min_damage = self.stacks
                                card.max_damage = card.min_damage 

                                local fight_data = self.owner.agent.fight_data
                                if fight_data.riposte_tags then
                                    card.hit_tags = self.owner.riposte_tags
                                end
                                if fight_data.ranged_riposte then
                                    card.flags = CARD_FLAGS.SPECIAL | CARD_FLAGS.RANGED
                                end

                                card:AssignTarget( resolve_card.owner, { { resolve_card.owner } } )

                                for i=1, self.counter_attack_counter do
                                    if resolve_card.owner:IsAlive() then
                                        local counter_attack = Battle.Attack( card )
                                        counter_attack:GenerateHits()
                                        counter_attack.is_counter = true
                                        battle:BroadcastEvent( BATTLE_EVENT.ON_RIPOSTE, counter_attack, i == 1, resolve_card, i == self.counter_attack_counter, true )
                                        counter_attack:RunAttack()
                                    end
                                end

                                resolve_card.owner:CheckForSurrender()
                            end
                        end
                        self.counter_attack_counter = 0
                    end,
                }
            },                               
        },
        attacks = 
        {

            nadan_enrage = table.extend(NPC_BUFF)
            {
                --name = "Enrage",
                anim = "taunt",
                flags = CARD_FLAGS.BUFF | CARD_FLAGS.SKILL,
                target_type = TARGET_TYPE.SELF,

                OnPostResolve = function( self, battle, attack )
                    
                    self.owner:AddCondition("POWER")
                    
                    self.owner:RemoveCondition("nadan_retracted")
                    
                    if battle:GetTurns() == 0 then
                        self.owner:AddCondition("nadan_enraged", 3)
                    else
                        self.owner:AddCondition("nadan_enraged", NADAN_CLAW_TURNS)
                    end
                end
            },
            nadan_end_rage = table.extend(NPC_BUFF)
            {
                --name = "Retract",
                anim = "enraged_normal",

                flags = CARD_FLAGS.SKILL,
                target_type = TARGET_TYPE.SELF,

                PostPresAnim = function( self, anim_fighter )
                    anim_fighter:SetAnimMapping(nil)
                end,

                OnPostResolve = function( self, battle, attack )
                    self.owner:RemoveCondition("nadan_enraged")
                    self.owner:AddCondition("nadan_retracted", NADAN_RETRACT_TURNS)

                    if self.owner:GetTeam() ~= battle:GetPlayerTeam() then
                        local card = Battle.Card("nadan_activate_guards", self.owner)
                        battle:PlayCard(card)
                    end
                end
            },


            nadan_flurry = table.extend(NPC_ATTACK)
            {

                name = "Flurry",
                anim = "flurry",
                flags = CARD_FLAGS.MELEE,
                base_damage = { 5, 6, 7, 8},
                hit_count = 3,
                CanPlayCard = function(self, battle, target)
                    return self.owner:HasCondition("nadan_enraged")
                end,
            },

            nadan_leap = table.extend(NPC_ATTACK)
            {
                name = "Leap",
                anim = "attack2",
                --base_damage = { 18, 22, 26 },
                base_damage = { 10, 11, 13, 15 },
                flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,
                
                CanPlayCard = function(self, battle, target)
                    return self.owner:HasCondition("nadan_enraged")
                end,


                OnPostResolve = function( self, battle, attack )
                    local possible_conditions =
                    {
                        WOUND = {3, 3, 4, 5},
                        BLEED = {2, 2, 3, 3},
                        IMPAIR = {1, 1, 2, 2},
                    }
                    
                    for i,hit in ipairs(attack.hits) do
                        if not hit.evaded then
                            local candidates = {}
                            for condition,v in pairs(possible_conditions) do
                                if not hit.target:HasCondition(condition) then
                                    table.insert(candidates, condition)
                                end
                            end
                            if #candidates > 0 then
                                local condition = table.arraypick(candidates)
                                if condition == "BLEED" and hit.target:IsPlayer() then
                                    local cards = {}
                                    for i=1,possible_conditions["BLEED"][GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] do
                                        table.insert(cards, Battle.Card( "status_bleed", hit.target) )
                                    end
                                    battle:DealCards( cards, battle:GetDiscardDeck() )
                                else
                                    hit.target:AddCondition(condition, possible_conditions[condition][GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
                                end
                            end
                        end
                    end
                end
            },

            nadan_leap_and_leave = table.extend(NPC_ATTACK)
            {
                name = "Leap",
                anim = "attack2",
                --base_damage = { 18, 22, 26 },
                base_damage = { 10, 12, 13, 15 },
                flags = CARD_FLAGS.MELEE,

                OnPostResolve = function( self, battle, attack )                  
                    local possible_conditions =
                    {
                        WOUND = {3, 3, 4, 5},
                        BLEED = {2, 2, 3, 3},
                        IMPAIR = {1, 1, 2, 2},
                    }
                    
                    for i,hit in ipairs(attack.hits) do
                        local candidates = {}
                        for condition,v in pairs(possible_conditions) do
                            if not hit.target:HasCondition(condition) then
                                table.insert(candidates, condition)
                            end
                        end
                        if #candidates > 0 then
                            local condition = table.arraypick(candidates)
                            hit.target:AddCondition(condition, possible_conditions[condition][GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
                        end
                    end

                    self.owner:Depart()
                end
            },
            nadan_punch = table.extend(NPC_ATTACK)
            {
                CanPlayCard = function(self, battle, target)
                    return self.owner:HasCondition("nadan_retracted")
                end,

                name = "Backhand",
                anim = "regular_attack",
                --base_damage = { 18, 22, 26 },
                base_damage = { 6, 7, 8, 10 },
                flags = CARD_FLAGS.MELEE,
            },


            nadan_activate_guards =
            {
                name = "Call Protection",
                anim = "call_goon",
                flags = CARD_FLAGS.SKILL,
                
                loc_strings =
                {
                    CALL1 = "You're on <b>my</> turf, grifter.",
                    CALL2 = "Spree! To me!",
                    CALL3 = "You'll never wipe out the Spree!",
                    CALL4 = "Kill this grifter!",
                    CALL5 = "Just DIE already!",
                },

                defend_amt = { 5, 8, 10, 12 },
                sentry_amt = { 2, 4, 6, 8 },

                target_type = TARGET_TYPE.SELF,

                OnPostResolve = function( self, battle, attack )
                    local times_used = self.owner.reinforcements_called or 1
                    local idx = math.min(times_used, #REINFORCMENT_TABLE)
                    local quip = self.def:GetLocalizedString( string.format( "CALL%d", idx ))
                    self.owner.behaviour:SaySpeech(0, quip)
                    local entry = REINFORCMENT_TABLE[ idx ]

                    for k,content_id in ipairs(entry) do
                        local agent = Agent( content_id )
                        local new_fighter = Fighter.CreateFromAgent( agent, BACKUP_LEVEL )
                        new_fighter.nadan_backup = true
                        self.owner:GetTeam():AddFighter( new_fighter )
                        self.owner:GetTeam():ActivateNewFighters( battle )
                        local sentry_stacks = self.sentry_amt[math.min(#self.sentry_amt, GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1)]
                        new_fighter:AddCondition("sentry", sentry_stacks, self.owner)

                        if k ~= #entry then
                            battle:BroadcastEvent( BATTLE_EVENT.DELAY, .5)
                        end
                    end

                    self.owner.reinforcements_called = times_used + 1                        

                    self.owner:AddCondition("DEFEND", self.defend_amt[math.min(#self.defend_amt, GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1)], self)
                end
            },
        },


        behaviour =
        {
            OnActivate = function( self, fighter )
                
                self.enrage = self:AddCard("nadan_enrage")
                self.flurry = self:AddCard("nadan_flurry")
                self.attack = self:AddCard("nadan_leap")
                
                self.punch = self:AddCard("nadan_punch")

                self.activate_guards = self:AddCard("nadan_activate_guards")
                
                self:SetPattern( self.Cycle )
                self.fighter.battle:PlayCard(self.activate_guards)
                self.fighter:AddCondition("nadan_retracted", NADAN_RETRACT_TURNS)

                self.fighter:AddCondition("ARMOURED", NADAN_DEFENSE_AMOUNT[math.min(#NADAN_DEFENSE_AMOUNT, GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1)])
            end,

            Cycle = function( self )
                if self.fighter:HasCondition("nadan_enraged") then
                    if self.fighter:GetConditionStacks("nadan_enraged") == 1 then
                        self:ChooseCard(self.flurry)
                    else
                        self:ChooseCard(self.attack)
                    end
                else
                    self:ChooseCard(self.punch)
                    --self:ChooseCard(self.defend)
                end
            end,

            event_handlers = 
            {
                [ BATTLE_EVENT.STATUS_CHANGED ] = function( self, fighter, status )
                    
                    if self.fighter == fighter and (status == FIGHT_STATUS.SURRENDER or status == FIGHT_STATUS.DEAD) then
                        local fighters = self.fighter:GetTeam().fighters
                        local to_flee = {}
                        for _, fighter in ipairs(fighters) do
                            if fighter ~= self.fighter and fighter:IsActive() then
                                table.insert(to_flee, fighter)
                            end
                        end

                        if #to_flee > 0 then 
                            for k,fighter in pairs(to_flee) do
                                fighter:Flee()
                            end
                        end
                    elseif self.fighter ~= fighter and fighter.team == self.fighter.team and (status == FIGHT_STATUS.SURRENDER or status == FIGHT_STATUS.DEAD) then
                        self.fighter:ApplyMoraleDamage(fighter.agent:GetCombatStrength()*3)
                        -- Required specifically because this is likely occuring within a CheckSurrenders call, and self.fighter may have already been processsed.
                        self.fighter:CheckForSurrender()
                    end

                 end
            },
        },

        anim_mapping_enraged =
        {
            idle = "idle_enraged",
            defend = "defend_enraged",
            death = "death_enraged",
            hit_mid = "hit_mid_enraged",
            hit_mid_pst_idle = "hit_mid_pst_idle_enraged",
            hit_mid_pst_stunned = "hit_mid_pst_stunned_enraged",
            hit_mid_pst_surrender = "hit_mid_pst_surrender_enraged",
            run_forward = "run_forward_enraged",
            run_forward_pre = "run_forward_pre_enraged",
            run_forward_pre_turn = "run_forward_pre_turn_enraged",
            run_forward_pst = "run_forward_pst_enraged",
            run_forward_pst_turn = "run_forward_pst_turn_enraged",
            step_back = "step_back_enraged",
            step_forward = "step_forward_enraged",
            stunned = "stunned_enraged",
            stunned_pre = "stunned_pre_enraged",
            stunned_pst = "stunned_pst_enraged",
            surrender_pre = "surrender_pre_enraged",
            surrender_pst = "surrender_pst_enraged",
        },
    },

    negotiation_data =
    {
        modifiers =
        {
            RING_LEADER =
            {
                name = "Ring Leader",
                desc = "Increase the damage of {1}'s intents and argument by {2} for every {3} cards you play per turn.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName(), self.damage_bonus[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.damage_bonus)], self.card_count)
                end,

                modifier_type = MODIFIER_TYPE.CORE,

                damage_bonus = { 1, 1, 1, 2 },
                card_count = 4,

                event_handlers =
                {
                    [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion, minigame, target )
                        if source.negotiator == self.negotiator and self.bonus_count and self.bonus_count > 0 then
                            local bonus = self.damage_bonus[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.damage_bonus)]
                            persuasion:AddPersuasion( bonus * self.bonus_count, bonus * self.bonus_count, self )
                        end
                    end,

                    [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                        if negotiator == self.negotiator then
                            self.bonus_count = 0
                            self.counter = 0
                        end
                    end,

                    [ EVENT.POST_RESOLVE ] = function(self, minigame, card)
                        if self.anti_negotiator and card.negotiator == self.anti_negotiator then
                            self.counter = (self.counter or 0) + 1
                            if self.counter >= self.card_count then
                                self.bonus_count = (self.bonus_count or 0) + 1
                                self.counter = 0
                                self:NotifyTriggered()
                            end
                        end
                    end
                },
            },

            NO_FILTER =
            {
                name = "No Filter",
                desc = "{1}’s intents deal {2} bonus damage. At the end of {1}’s turn, remove 1 <b>No Filter</>.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName(), self.bonus_damage )
                end,

                OnBounty = function( self )
                    self.negotiator.behaviour:SetPattern(self.negotiator.behaviour.GoonCycle)
                end,

                OnInit = function( self, source )
                    self.apply_turn = self.engine:GetTurns()
                    self:SetStacks(3)
                end,

                modifier_type = MODIFIER_TYPE.ARGUMENT,
                max_resolve = 10,

                bonus_damage = 3,

                event_priorities =
                {
                    [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
                },

                event_handlers =
                {
                    [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion, minigame, target )
                        if minigame:GetTurns() > self.apply_turn and source.negotiator == self.negotiator and is_instance( source, Negotiation.Card ) then
                            persuasion:AddPersuasion( self.bonus_damage, self.bonus_damage, self )
                        end
                    end,

                    [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                        if negotiator == self.negotiator and minigame:GetTurns() > self.apply_turn then
                            self.negotiator:RemoveModifier(self.id, 1, self)
                        end
                    end,
                }
            },

            BANDIT_SCHMUCK = 
            {
                name = "Bandit Schmuck",
                desc = "While this is alive, {1}'s core argument is shielded and {1} will not attack. At the end of {1}'s turn, <b>Bandit Schmuck</> gains 1 damage.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self:GetOwnerName())
                end,

                loc_strings =
                {
                    SHIELD_DESC = "{SHIELDED}. Destroy or remove all <b>Bandit Schmuck</b> arguments to release the shield!",
                },

                OnInit = function( self, source )
                    local core = self.negotiator:FindCoreArgument()
                    core:SetShieldStatus( true, self.def:GetLocalizedString("SHIELD_DESC") )
                    self.apply_turn = self.engine:GetTurns()
                end,

                target_enemy = TARGET_ANY_RESOLVE,
                max_resolve = 5,
                min_persuasion = 1,
                max_persuasion = 2,
                max_stacks = 1,

                OnBeginTurn = function( self, minigame )
                    self:ApplyPersuasion()
                end,

                OnUnapply = function( self, minigame )
                    if self.negotiator:GetModifierInstances("BANDIT_SCHMUCK") <= 1 then
                        local core = self.negotiator:FindCoreArgument()
                        if core then
                            core:SetShieldStatus( false)
                        end
                        self.negotiator.behaviour:SetPattern(self.negotiator.behaviour.BossCycle)
                    end
                end,

                event_priorities =
                {
                    [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
                },

                event_handlers =
                {
                    [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion, minigame, target )
                        if source == self and self.bonus then
                            persuasion:AddPersuasion( self.bonus, self.bonus, self )
                        end
                    end,

                    [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                        if negotiator == self.negotiator and minigame:GetTurns() > self.apply_turn then
                            self.bonus = (self.bonus or 0) + 1
                        end
                    end,
                }
            },
        },

        behaviour = 
        {
            OnInit = function( self )
                if CheckBits( self.engine.start_params.flags, NEGOTIATION_FLAGS.WORDSMITH ) then
                    self.no_filter = self:AddArgument("NO_FILTER")
                    self.bandit = self:MakePicker()
                        :AddID("nadan_schmuck", 1)
                        :AddID("nadan_schmuck", 1)

                    self:SetPattern( self.BossCycle )
                    self.negotiator:AddModifier("RING_LEADER")
                else
                    self.KINGPIN = 0
                    self.escalation = self:AddArgument( "ESCALATION" )
                    self.kingpin = self:AddArgument( "KINGPIN" )
                    self.greedy = self:AddArgument( "GREEDY" )

                    if self.difficulty <= 2 then
                        self:SetPattern( self.BasicCycle )
                    else
                        self:SetPattern( self.Cycle )
                    end
                    
                    self.negotiator:AddModifier("SHORT_FUSE")
                end
            end,

            BossCycle = function( self, turns )
                self:ChooseGrowingNumbers( 2, 1 )
                if not self.negotiator:HasModifier("NO_FILTER") then
                    self:ChooseCard(self.no_filter)
                end
                if turns % 3 == 0 then
                    self:ChooseComposure( 1, 3, 7 )
                end
            end,

            GoonCycle = function( self, turns )
                if not self.negotiator:HasModifier("NO_FILTER") and not self.negotiator:HasModifier("BANDIT_SCHMUCK") then
                    self.bandit:ChooseCards(2)
                end
                if turns % 3 == 0 or self.negotiator:HasModifier("BANDIT_SCHMUCK") then
                    self:ChooseComposure( 1, 3, 7 )
                end
                if not self.negotiator:HasModifier("BANDIT_SCHMUCK") then
                    self:ChooseGrowingNumbers( 2, 1 )
                end
            end,

            BasicCycle = function( self, turns )
                if turns == 2 then
                    self:ChooseCard( self.escalation )
                    self:ChooseGrowingNumbers( 1, 1 )

                elseif (turns-1) % 3 == 0 then
                    self:ChooseGrowingNumbers( 2, 1 )
                else
                    self:ChooseGrowingNumbers( 1, 1 )
                end
            end,
 
            Cycle = function( self,turns )
                if turns % 3 == 0 then
                    self:ChooseCard( self.greedy )
                end
                if self.difficulty >= 4 and turns % 2 == 0 then
                    self:ChooseGrowingNumbers( 3, -1 )
                elseif turns % 2 == 0 then
                    self:ChooseGrowingNumbers( 2, 1 )
                else
                    self:ChooseGrowingNumbers( 1, 3 )
                end
                if (turns - 1) % 5 == 0 and not self.negotiator:FindModifier( "ESCALATION" ) then
                    self:ChooseCard( self.escalation )
                end
                if self.KINGPIN > 0 then
                    if (turns + 3) % 5 == 0 and not self.negotiator:FindModifier( "KINGPIN" ) then
                        self:ChooseCard( self.kingpin )
                    end
                end
            end,
        }
    }
})

Content.AddCharacterDef( def )