local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS


local def = CharacterDef("MURDER_BAY_ADMIRALTY_CONTACT",
{
    alias = "MURDER_BAY_ADMIRALTY_CONTACT",
    unique = true,
    designer_note = "Gains power ever X card played, has a Snail that is also powering up, but is easy to reset.",

    base_def = "ADMIRALTY_CLERK",
    title = "Intelligence Officer",
    species = SPECIES.KRADESHI,
    gender = GENDER.FEMALE,
    theme_music = "event:/music/story/sal/story_sal_oolo",
    
    renown = 4,
    combat_strength = 4,
    boss = true,
    battle_preview_anim = "anim/boss_oolo_slide.zip",
    battle_preview_offset = { x = 460, y = 0 },
    battle_preview_glow = { colour = 0x11FFFEFF, bloom = 0.2, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_oolo",
    icon = engine.asset.Texture( "icons/b_murder_bay_admiralty_contact.tex"),

    voice_actor = "oolo",
    name = "Oolo",
    bio = "Oolo doesn't often ask for help, so it says volumes of how much crime has come to a head in Murder Bay. Nevertheless, she's tenacious enough to see the Admiralty to victory, and cunning enough to navigate the quagmire of bureaucracy and corruption in the ranks, while still coming out on top.",
    build = "guard_female_offduty",
    head = "head_female_kradeshi_02",

    loved_graft = "admiralty_medals",
    hated_graft = "wrong_papers",
    death_item = "charged_disc",

    combat_anims = {"anim/med_combat_oolo.zip"},
    anims = {"anim/weapon_disc_oolo.zip"},

    -- lore_unlocks = {
    --     picked_admiralty = "Custom string here because you decided to work for Oolo at some point.",
    --     killed = "Custom string here because you killed Oolo at some point.",
    --     killed_by = "Custom string here because you were killed by Oolo at some point.",
    --     drank_with = "Custom string here because you drank with Oolo at some point.",
    --     loved_by = "Custom string here because Oolo loved you at some point.",
    --     hated_by = "Custom string here because Oolo hated you at some point.",
    -- },

    fight_data =
    {
        battle_scale = 1.05,
        MAX_HEALTH = 200,
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        formation = FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.FRONT_Z,

        status_widget_dx = 0.8,

        cards = { "oolo_throw_disc", "oolo_throw_disc_charged", "oolo_defend"},

        conditions =
        {
            oolo_charged_disc =
            {
                name = "Charged Disc",
                desc = "{1}'s disc is charged and her next attack deals bonus damage and inflicts {WOUND {2}}.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.owner.agent:GetName(), self.stacks )
                end,
            },

            oolo_instincts =
            {
                name = "Battle Instincts",
                desc = "{1} gains {POWER 1} every {2} cards you play. ({3} remaining)",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.owner.agent:GetName(), self.num_cards, self.stacks )
                end,

                num_cards = 99,

                event_handlers = 
                {
                    [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                        if attack.owner == self.battle:GetPlayerFighter() then
                            self.owner:RemoveCondition( self.id, 1)
                            if self.owner:GetConditionStacks(self.id) == 0 then
                                local card = Battle.Card("oolo_powerup", self.owner)
                                card:AssignTarget( self.owner )
                                battle:PlayCard( card )
                                local condition = self.owner:AddCondition(self.id, self.num_cards)
                                condition.num_cards = self.num_cards
                            end
                        end
                    end,

                },
            },
        },

        anim_mapping =
        {
            riposte = "attack1",
        },

        ranged_riposte = true,

        attacks = 
        {
            oolo_throw_disc = table.extend(NPC_ATTACK)
            {
                name = "Throw Disc",
                anim = "attack1",

                flags = CARD_FLAGS.RANGED,

                base_damage = {12,13,14,16},
            },

            oolo_throw_disc_charged = table.extend(NPC_ATTACK)
            {
                name = "Charged Disc",
                anim = "attack1",

                target_mod = TARGET_MOD.TEAM,
                flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,

                base_damage = {6,7,8,10},

                OnPostResolve = function( self, battle, attack )
                    local stacks = self.owner:GetConditionStacks("oolo_charged_disc")
                    attack:AddCondition("WOUND", stacks, self)
                    self.owner:RemoveCondition("oolo_charged_disc")
                end,

            },

            oolo_throw_disc_charged_and_leave = table.extend(NPC_ATTACK)
            {
                name = "Charged Disc",
                anim = "attack1",

                target_mod = TARGET_MOD.TEAM,
                flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,

                base_damage = {6,7,8,10},

                OnPostResolve = function( self, battle, attack )
                    local stacks = self.owner:GetConditionStacks("oolo_charged_disc")
                    attack:AddCondition("WOUND", stacks, self)
                    self.owner:RemoveCondition("oolo_charged_disc")
                    self.owner:Depart()
                end,
            },

            oolo_powerup = table.extend(NPC_BUFF)
            {
                name = "Battle Instincts",
                anim = "taunt",

                target_type = TARGET_TYPE.SELF,

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("POWER", 1)
                end
            },

            oolo_defend = table.extend(NPC_BUFF)
            {
                name = "Charge Disc",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                defend_amt = {10,12,15,20},

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("oolo_charged_disc", 3, attack)
                    self.owner:AddCondition("DEFEND", self.defend_amt[ GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1 ] )
                end
            },

            oolo_summon_snail = table.extend(NPC_BUFF)
            {
                name = "Summon Oshnu",
                anim = "taunt",

                quip = "summon_oshnu_oolo",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                OnPostResolve = function( self, battle, attack )
                    local agent = Agent( "OOLO_SNAIL" )
                    local new_fighter = Fighter.CreateFromAgent( agent, self.owner:GetScale() )
                    self.owner:GetTeam():AddFighter( new_fighter )
                    self.owner:GetTeam():ActivateNewFighters( battle )
                end
            },


        },

        behaviour = 
        {
            OnActivate = function( self )
                self.fighter:GetTeam():SetCustomFormation( self.CUSTOM_FIGHT_FORMATIONS )

                self.attack = self:MakePicker()
                    :AddID( "oolo_throw_disc", 3 )
                    :AddID( "oolo_defend", 1 )
                self.charged_attack = self:AddCard("oolo_throw_disc_charged")
                self:SetPattern( self.Attack )

                local NUM_CARDS_BY_DIFF = {5, 5, 4, 3}
                local num_cards = NUM_CARDS_BY_DIFF[ GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1 ]
                local condition = self.fighter:AddCondition( "oolo_instincts", num_cards )
                condition.num_cards = num_cards

                --starting move
                self.opener = self:AddCard("oolo_summon_snail")
                self.fighter.battle:PlayCard(self.opener)
            end,

            Attack = function( self )
                if self.fighter:HasCondition("oolo_charged_disc") then
                    return self:ChooseCard(self.charged_attack)
                end
                return self.attack:ChooseCard()
            end,

            CUSTOM_FIGHT_FORMATIONS =
            {
                [1] = { { 7.0, 3.0, nil } },
                [2] =
                {
                    {4, 0, FIGHTER_FORMATION.FRONT_X | FIGHTER_FORMATION.BACK_Z },
                    {11.0, 3.5, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.FRONT_Z},
                },
                [3] =
                {
                    {5.0, 3.25, FIGHTER_FORMATION.FRONT_X | FIGHTER_FORMATION.FRONT_Z},
                    {8.5, -1.0, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.BACK_Z},
                    {12.0, 5.0, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.FRONT_Z},
                },
                [4] =
                {
                    {3.0, -3.5, FIGHTER_FORMATION.FRONT_X | FIGHTER_FORMATION.BACK_Z},
                    {6.0, 4.5, FIGHTER_FORMATION.FRONT_X | FIGHTER_FORMATION.FRONT_Z},
                    {8.0, -3.5, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.BACK_Z},
                    {12.0, 4.5, FIGHTER_FORMATION.BACK_X | FIGHTER_FORMATION.FRONT_Z},
                },
            },
        },
    },
})

Content.AddCharacterDef( def )