local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local def = CharacterDef("WEEZIL",
{
    unique = true,

    --base_def = "NPC_BASE",
    base_def = "SPARK_BARON_BASE", --sure.
    species = SPECIES.JARACKLE,
    gender = GENDER.MALE,
    renown = 1,
    combat_strength = 1,
    boss = true,

    -- faction_id = "",

    title = "Sharpshooter",
    name = "Weezil",
    -- voice_actor = "",

    combat_anims = {"anim/med_combat_sniper_weezil.zip"},
    anims = {"anim/weapon_rifle_weezil.zip"},
    
    build = "male_weezil",
    head = "head_male_weezil",

    loved_graft = "conditioning",
    hated_graft = "hounded",
    death_item = "weezils_scope",

    battle_preview_anim = "anim/boss_weezil_slide.zip",
    battle_preview_offset = { x = 180, y = 100 },
    battle_preview_glow = { colour = 0x33ffffFF, bloom = 0.3, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_weezil",
    icon = engine.asset.Texture( "icons/b_weezil.tex" ),
    -- voice_actor = "",

    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 85,

        conditions = 
        { 
            snipers_poise =
            {
                name = "Sniper's Poise",
                desc = "{1}'s attacks with his rifle deal {2} bonus damage. Reduce by {3} whenever {1} is hit by an attack.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName(), self.stacks, self.hit_reduction)
                end,

                hit_reduction = 5,

                event_handlers =
                {
                    [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                        if not hit.evaded and hit.target == self.owner then
                            self.owner:RemoveCondition(self.id, self.hit_reduction, self)
                        end
                    end,
                },
            },

            snipers_poise_cooldown =
            {
                hidden = true,

                event_handlers =
                {
                    [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        self.owner:RemoveCondition(self.id, 1, self)
                    end
                },
            },
        },

        attacks = 
        {
            weezil_single_shot = table.extend(NPC_RANGED)
            {
                name = "Shoot",
                anim = "shoot",

                damage_mult = 1,

                event_handlers =
                {
                    [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                        if card == self then
                            local con = self.owner:GetCondition("snipers_poise")
                            if con then
                                dmgt:AddDamage( con.stacks, con.stacks, self )
                            end
                        end
                    end,
                },

                OnPostResolve = function( self, battle, attack )
                    if self.owner:HasCondition("snipers_poise") then
                        self.owner:RemoveCondition("snipers_poise")
                    end
                    self.owner:AddCondition("snipers_poise_cooldown", 1, self)
                end
            },

            weezil_multi_shot = table.extend(NPC_RANGED)
            {
                name = "Blaster Flurry",
                anim = "multi_shot",
                hit_count = 2,
                damage_mult = 0.5,
            },

            weezil_taunt = table.extend(NPC_BUFF)
            {
                name = "Adjust",
                anim = "taunt",
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                defend_amt = { 8, 8, 10, 12 },
                focus_amt = { 20, 20, 25, 30 },

                CanPlayCard = function( self, battle )
                    return not self.owner:HasCondition("snipers_poise_cooldown")
                end,

                OnPostResolve = function ( self, battle, attack )
                    self.owner:AddCondition("DEFEND", self.defend_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.defend_amt)], self)
                    self.owner:AddCondition("snipers_poise", self.focus_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.focus_amt)], self)
                end
            },
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                self.base_moves = self:MakePicker()
                    :AddID("weezil_multi_shot", 2)
                    :AddID("weezil_taunt", 1)

                self.single = self:AddCard("weezil_single_shot")

                self:SetPattern(self.BasicCycle)
                self.fighter:AddCondition("snipers_poise_cooldown", 1, self)
            end,

            BasicCycle = function( self )
                if self.fighter:HasCondition("snipers_poise") then
                    self:ChooseCard(self.single)
                else
                    self.base_moves:ChooseCard()
                end
            end
        },
    },

    negotiation_data = 
    {
        behaviour = 
        {
            OnInit = function( self, difficulty )
                self.stubborn = self:AddArgument("STUBBORN")
                self.drunk = self:AddArgument("DRUNK")
                self.negotiator:AddModifier( "ROWDY_PATRON" )
                self:SetPattern( self.BasicCycle )
            end,

            BasicCycle = function( self, turns )
                if turns % 3 == 0 then
                    self:ChooseGrowingNumbers( 1, 4 )
                else
                    self:ChooseGrowingNumbers( 3, 2, 1 )
                end

                self:ChooseComposure( 1, 2, 3 )
                
                if (turns + 1) % 3 == 0 and not self.negotiator:HasModifier("STUBBORN") then
                    self:ChooseCard(self.stubborn)
                end
                if (turns + 1) % 2 == 0 and not self.negotiator:HasModifier("DRUNK") then
                    self:ChooseCard(self.drunk)
                end
            end,
        }
    },
})

Content.AddCharacterDef( def )
    