local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local chars = 
{
    CharacterDef("ENDO",
	{
        base_def = "PRIEST",
        bio = "Though he doesn't look frail, his hands have a slight tremor when he prays. ",
        name = "Endo",
        tags = {"plot_armour", "no_patron"},
        loved_graft = "",
        hated_graft = "",
        gender = "MALE",
        species = "HUMAN",

        social_boons = table.empty,
    }),
    CharacterDef("RAKE",
    {
        base_def = "JAKES_SMUGGLER",
        bio = "Something you've heard about Rake comes to mind. She plays a mean deck of cards? Or maybe she's just mean. Or that she... means well?",
        name = "Rake",
        tags = {"plot_armour", "no_patron"},
        gender = "FEMALE",
        species = "HUMAN",
        loved_graft = "",
        hated_graft = "",

        social_boons = table.empty,
    }),

    CharacterDef("PLOCKA",
    {
        base_def = "SPARK_BARON_PROFESSIONAL",
        --bio = "insert bio here",
        name = "Plocka",
        tags = {"plot_armour", "no_patron"},
        loved_graft = "",
        hated_graft = "",
        gender = "FEMALE",
        species = "SHROKE",
    }),

    CharacterDef("FSSH",
    {
        base_def = "BARTENDER",
        tags = {"plot_armour", "no_patron"},
        species = SPECIES.KRADESHI,
        gender = GENDER.FEMALE,
        name = "Fssh",
        desc = "One-Eyed Bartender",
        faction_id = "JAKES",
        voice_actor = "fssh",
        renown = 3,

        social_boons = table.empty,
        hated_graft = "",
        loved_graft = "cozy",

        no_outfit_swap = true,
        bio = "The owner and proprietor of the Grog 'n Dog. She must be running a side angle, because nobody goes to the Grog for the food.",
    }),
	
    CharacterDef("HEBBEL",
    {
        base_def = "BARTENDER",
        species = SPECIES.SHROKE,
        gender = GENDER.MALE,
        name = "Hebbel",
        faction_id = "BANDITS",
        voice_actor = "hebbel",
        loved_graft = "",
        hated_graft = "",
    }),

    CharacterDef("SWEET_MOREEF",
    {
        base_def = "BARTENDER",
        species = SPECIES.JARACKLE,
        build = "male_sweet_moreef_build",
        gender = GENDER.MALE,
        name = "Sweet Moreef",
        faction_id = "JAKES",
        voice_actor = "sweetmoreef",
        loved_graft = "top_shelf",
    }),
    
    CharacterDef("SPARK_CONTACT",
    {
        designer_note = "** UNUSED **",
        base_def = "SPARK_BARON_BASE",
        title = "Regional Manager",
        renown = 5,
        combat_strength = 3,
        tags = {"no_patron"},
        build = "male_spark_baron_fellemo_build",
        head = "head_male_spark_baron_fellemo",
        theme_music = "event:/music/story/rook/story_rook_fellemo",
        name = "Fellemo",
        gender = "MALE",
        species = "KRADESHI",
		voice_actor = "lellynfellemo",
        loved_graft = "scheme",
        hated_graft = "scandal",
        death_item = "two_faced",

        anims = { "anim/weapon_blaster2_fellemo.zip", },
        combat_anims = { "anim/med_combat_blaster_fellemo.zip" },


        fight_data = 
        {
            attacks = 
            {
                fellemo_tag_in_attack = table.extend(NPC_ATTACK)
                {
                    name = "Blaster Barrage",
                    anim = "guns_blazing",
                    anims = { "anim/weapon_blaster_fellemo.zip"},
                    flags = battle_defs.CARD_FLAGS.RANGED | battle_defs.CARD_FLAGS.SPECIAL,
                    target_count = 4,
                    damage_mult = 0.5,
                    OnPostResolve = function( self, battle, attack )
                        self.owner:Depart()
                    end
                },


                fellemo_shoot = table.extend(NPC_ATTACK)
                {
                    name = "Pew! Pew!",
                    anim = "attack1",
                    flags = CARD_FLAGS.RANGED,
                    hit_count = 2,
                    target_count = 1,
                    damage_mult = .5,
                },

                fellemo_shoot2 = table.extend(NPC_ATTACK)
                {
                    name = "Pew!",
                    anim = "attack2",
                    flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
                    damage_mult = 1,
                    features = 
                    {
                        WOUND = 2,
                    },
                },

                fellemo_war_cry = table.extend(NPC_BUFF)
                {
                    name = "War Cry",
                    anim = "taunt",
                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,
                    target_mod = TARGET_MOD.TEAM,
                    OnPostResolve = function( self, battle, attack )
                        self.owner:AddCondition("DEFEND", 6, self)
                        attack:AddCondition("POWER", 2, self)
                    end,
                },
            },

            behaviour = 
            {
                OnActivate = function( self )
                    self.attacks = self:MakePicker()
                        :AddID( "fellemo_shoot", 4 )
                        :AddID( "fellemo_shoot2", 3 )
                        :AddID( "fellemo_war_cry", 1 )
                    self:SetPattern( self.Attack )
                end,

                Attack = function( self )
                    self.attacks:ChooseCards( 1 )
                end,
            },                

            call_in_anim_mapping = 
            {
                run_foward = "run_foward_special",
                run_foward_pre_turn = "run_foward_special",
            },

            MAX_MORALE = MAX_MORALE_LOOKUP.MEDIUM,
            MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,
        },
    }),

    CharacterDef("MURDER_BAY_ADMIRALTY_CONTACT_SIBLING",
    {
        base_def = "WEALTHY_MERCHANT",
        title = "Dilettante",
        species = SPECIES.KRADESHI,
        gender = GENDER.MALE,
        renown = 4,
        combat_strength = 3,
        --voice_actor = "oolo",
        name = "Foolo",
        --bio = "",
        --build = "guard_female_offduty",
        head = "head_male_generic_kradeshi_01",
    }),



    

    CharacterDef("SPARK_BARON_BOSS",
    {
        designer_note = "Charges up to do large attacks. Counters while charging.",

        base_def = "SPARK_BARON_BASE",

        title = "Boss",
        shorttitle = "Spark Baron",
        species = SPECIES.JARACKLE,
        gender = GENDER.MALE,

        build = "male_spark_baron_boss",
        head = "head_male_spark_baron_boss",
        
        combat_anims = {"anim/med_combat_blaster_spark_baron_boss.zip"},
        anims = { "anim/weapon_blaster_spark_baron_boss.zip"},

        renown = 4,
        -- combat_strength = 1,
        boss = true,
        battle_preview_anim = "anim/boss_sparky_slide.zip",
        battle_preview_offset = { x = 45, y = 65 },
        battle_preview_glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_sparkbaronboss",
        icon = engine.asset.Texture( "icons/b_spark_baron_boss.tex"),

        name = "Sparky",
        tags = {"no_friends"},
        faction_id = "SPARK_BARONS",

        voice_actors = {
            [GENDER.MALE] = { "jakeLowClassMale01" },
            [GENDER.FEMALE] = { "jakeLowClassFemale01" },
        },

        death_money = DEATH_MONEY_HIGH,
        death_item = "sparkys_oppressor_cell",

        loved_graft = "system_shock",
        hated_graft = "short_circuit",

        fight_data = 
        {
            battle_scale = 1.22,

            MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
            MAX_HEALTH = 100,

            cards = { "ai_sparky_shoot", "ai_sparky_throw_gadget", "ai_sparky_counter", "ai_sparky_defend", "ai_sparky_charge" },

            attacks = 
            {
                ai_sparky_shoot = table.extend(NPC_ATTACK)
                {
                    name = "Cannon Blast",
                    anim = "shoot",
                    flags = CARD_FLAGS.RANGED,
                    target_mod = TARGET_MOD.TEAM,

                    CanPlayCard = function(self, battle, target)
                        return self.owner:GetConditionStacks("ai_sparky_charge") >= 2
                    end,

                    PostPresAnim = function( self, anim_fighter )
                        anim_fighter:SetAnimMapping()
                    end,

                    apply_sound = "event:/sfx/battle/status/system/Status_Buff_Attack",
                    
                    base_damage = { 15, 17, 20, 25 },

                    OnPostResolve = function( self, battle, attack )
                        if self.owner:HasCondition("unstable_spark_core") then
                            self.owner:RemoveCondition("unstable_spark_core")
                        end         
                    end,
                },

                ai_sparky_throw_gadget = table.extend(NPC_ATTACK)
                {
                    name = "Spark Grenade",
                    anim = "throw_gadget",
                    flags = CARD_FLAGS.RANGED,

                    target_mod = TARGET_MOD.TEAM,

                    base_damage = { 7, 8, 10, 12 }
                },

                ai_sparky_counter = table.extend(NPC_BUFF)
                {
                    name = "Counter",
                    anim = "taunt",

                    target_type = TARGET_TYPE.SELF,
                    flags = CARD_FLAGS.BUFF,

                    features = 
                    {
                        RIPOSTE = 3,
                        DEFEND = 4,
                    },
                },

                ai_sparky_pre_charge = table.extend(NPC_BUFF)
                {
                    name = "",
                    anim = "charge_pre",

                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    PostPresAnim = function( self, anim_fighter )
                        anim_fighter:SetAnimMapping(self.owner.agent.fight_data.anim_mapping_charging)
                    end,

                    OnPostResolve = function( self, battle, attack )
                        local card = Battle.Card("ai_sparky_charge", self.owner)
                        card:AssignTarget( self.owner )
                        battle:PlayCard( card )
                    end
                },

                ai_sparky_charge = table.extend(NPC_BUFF)
                {
                    name = "Spark Charge",
                    anim = "taunt_charge",
                    quip = "sparky_charge",
                    target_type = TARGET_TYPE.SELF,
                    flags = CARD_FLAGS.BUFF,

                    riposte = {3,4,5,7},
                    defend = {6,7,8,10},

                    CanPlayCard = function(self, battle, target)
                        return self.owner:GetConditionStacks("ai_sparky_charge") < 2
                    end,

                    OnPostResolve = function(self, battle, attack)
                        local con = self.owner:AddCondition( self.id, 1 )

                        local riposte = self.riposte[ GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1 ]
                        local defend = self.defend[ GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1 ]

                        self.owner:AddCondition("RIPOSTE", math.floor(riposte))
                        self.owner:AddCondition("DEFEND", math.floor(defend))
                    end,

                    condition = 
                    {
                        desc = "Charging weapon!",
                        target_type = TARGET_TYPE.SELF,

                        event_handlers =
                        {
                            [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                                if attack.owner == self.owner and attack.card.id == "ai_sparky_shoot" then
                                    self.owner:RemoveCondition(self.id)
                                end
                            end,
                        },
                    },
                },

                ai_sparky_prepare_shot = table.extend(NPC_BUFF)
                {
                    name = "Take Aim",
                    anim = "shoot_pre",
                    quip = "sparky_pre_shoot",
                    target_type = TARGET_TYPE.SELF,
                    flags = CARD_FLAGS.BUFF,

                    buff_strength = 1,

                    CanPlayCard = function(self, battle, target)
                        return self.owner:GetConditionStacks("ai_sparky_charge") == 1
                    end,

                    PostPresAnim = function( self, anim_fighter )
                        anim_fighter:SetAnimMapping(self.owner.agent.fight_data.anim_mapping_shoot)
                    end,

                    OnPostResolve = function(self, battle, attack)
                        local con = self.owner:AddCondition( "ai_sparky_charge", 1 )
                        con.buff_strength = self.buff_strength
                    end,
                },

                ai_sparky_defend = table.extend(NPC_BUFF)
                {
                    name = "Defend",
                    anim = "defend",
                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    buff_strength = 1,

                    features =
                    {
                        DEFEND = 10,
                    },

                    OnPostResolve = function( self, battle, attack )
                        self.owner:AddCondition("regeneration_barrier")
                    end,
                },


            },
            behaviour =
            {
                OnActivate = function( self, fighter )
                    self.basic = self:MakePicker()
                        :AddID( "ai_sparky_throw_gadget", 1)
                    self.pre_charge = self:AddCard("ai_sparky_pre_charge")
                    self.charge = self:MakePicker()
                        --:AddID( "ai_sparky_charge", 1)
                        :AddID( "ai_sparky_prepare_shot", 1)
                        :AddID( "ai_sparky_shoot", 1 )
                    self.buffs = self:MakePicker()
                        :AddID( "ai_sparky_defend", 1 )
                    self:SetPattern( self.Cycle )

                    self:GenerateCountdown()
                end,

                GenerateCountdown = function(self)
                    --if the player is hurt, give them at least one easy turn
                    local min_turns = self.battle:GetPlayerFighter():GetHealthPercent() < 0.5 and 2 or 1
                    self.turns_to_charge = math.random(min_turns,3)
                end,

                Cycle = function( self )
                    self.turn_count = (self.turn_count or 0) + 1
                    if self.turn_count == 1 then
                        return self.basic:ChooseCards(1)
                    end
                    

                    if self.fighter:GetConditionStacks("ai_sparky_charge") == 0 then
                        self.turns_to_charge = self.turns_to_charge - 1
                        if self.turns_to_charge > 0 then
                            return self.basic:ChooseCards(1)
                        else
                            return self:ChooseCard(self.pre_charge)
                        end
                    else
                        self:GenerateCountdown()
                        return self.charge:ChooseCard()
                    end
                end,
            },

            anim_mapping = 
            {
                riposte = "counter"
            },

            anim_mapping_shoot = 
            {
                idle = "idle_shoot",
                hit_mid_pst_idle = "hit_mid_pst_shoot",
                riposte = "counter"
            },

            anim_mapping_charging = 
            {
                idle = "idle_charge",
                hit_mid_pst_idle = "hit_mid_pst_charge",
                riposte = "counter"
        },
        },
    }),

    CharacterDef("DRONE_GOON",
    {
        designer_note = "Drones do significant damage if you don't deal with them. Drones are killed with number of hits rather than hitpoints, and deal damage to boss when exploding.",

        base_def = "SPARK_BARON_GOON",
        title = "Drone Master",
        shorttitle = "Spark Baron",
        renown = 4,
        combat_strength = 2,

        boss = true,
        battle_preview_anim = "anim/boss_drone_master_slide.zip",
        battle_preview_offset = { x = 200, y = -20 },
        battle_preview_glow = { colour = 0xF08C49FF, bloom = 0.15, threshold = 0.02 },
        battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_dronegoon",
        icon = engine.asset.Texture( "icons/b_drone_goon.tex"),
        
        build = "male_spark_baron_dronemaster",
        head = "head_male_spark_baron_dronemaster",

        tags = {"no_friends"},
        faction_id = "SPARK_BARONS",

        voice_actors = {
            [GENDER.MALE] = { "jakeLowClassMale01" },
        },
        
        combat_anims = {"anim/med_combat_rifle_dronemaster.zip"},
        anims = { "anim/weapon_rifle_dronemaster.zip"},

        death_money = DEATH_MONEY_HIGH,
        death_item = "prototype_drone",

        loved_graft = "drone_mastery",
        hated_graft = "drone_supplier",

        fight_data = 
        {
            MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
            MAX_HEALTH = 100,

            cards = { "drone_goon_blast", "drone_goon_melee", "drone_goon_summon_drones" },
            
            attacks = 
            {
                --DRONE GOON
                drone_goon_blast = table.extend(NPC_ATTACK)
                {
                    name = "Spark Blast",
                    anim = "shoot",

                    target_mod = TARGET_MOD.TEAM,

                    flags = CARD_FLAGS.RANGED,
                    base_damage = { 8, 9, 10, 12 },
                },

                drone_goon_melee = table.extend(NPC_ATTACK)
                {
                    name = "Rifle butt",
                    anim = "butt",

                    flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

                    base_damage = { 5, 6, 8, 10 },

                    OnPostResolve = function( self, battle, attack )
                        if not attack:CheckHitResult( attack.target, "evaded" ) and
                           not attack:CheckHitResult( attack.target, "defended" ) then
                            if attack.target:IsPlayer() then
                                local cards = {
                                    Battle.Card( "status_injury", attack.target ),
                                }
                                battle:DealCards( cards, battle:GetDiscardDeck() )

                            elseif math.random() < 0.5 then
                                attack.target:AddCondition( "IMPAIR", 1 )
                            end
                        end
                    end
                },

                drone_goon_summon_drones = table.extend(NPC_BUFF)
                {
                    name = "Summon Drones",
                    anim = "taunt",

                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    unstable_drone_max_stacks = {3, 3, 4, 5},

                    SetCount = function( self, count )
                        self.count = count
                    end,

                    OnPostResolve = function( self, battle, attack )
                        local team = self.owner:GetTeam()
                        for i = 1, self.count or 1 do
                            if team:NumActiveFighters() < team:GetMaxFighters() then
                                local agent = Agent( "COMBAT_DRONE" )
                                local new_fighter = Fighter.CreateFromAgent( agent, self.owner:GetScale() )
                                new_fighter.drone_owner = self.owner

                                self.owner:GetTeam():AddFighter( new_fighter )
                                self.owner:GetTeam():ActivateNewFighters( battle )
                                local max_stacks = self.unstable_drone_max_stacks[ GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1 ]
                                new_fighter:AddCondition("unstable_drone", max_stacks)

                                if self.owner:GetEnemyTeam():NumActiveFighters() > 1 then
                                    new_fighter:AddCondition("drone_charge", math.max(1, math.min(self.owner:GetEnemyTeam():NumActiveFighters(), 2)) * 3 )
                                end
                            end
                        end
                    end
                },

            },

            behaviour =
            {
                OnActivate = function( self, fighter )
                    self.basic_attacks = self:MakePicker()
                        :AddID( "drone_goon_blast", 3)
                        :AddID( "drone_goon_melee", 2)
                    self.summon = self:MakePicker()
                        :AddID( "drone_goon_summon_drones", 3)
                        --:AddID( "drone_goon_blast", 1)
                    self:SetPattern( self.Cycle )

                    local count = math.min( 2, fighter:GetTeam():GetMaxFighters() - fighter:GetTeam():NumActiveFighters() )
                    local opener = Battle.Card( "drone_goon_summon_drones", self.fighter )
                    opener:SetCount( count )
                    self.fighter.battle:PlayCard(opener)
                end,

                Cycle = function( self )
                    if #self.fighter:GetTeam().fighters < 3 and math.random() > 0.5 then
                        return self.summon:ChooseCard()
                    end

                    return self.basic_attacks:ChooseCard()
                end,

                event_handlers = 
                {
                    [ BATTLE_EVENT.SETTLE_FIGHTERS ] = function( self )
                        if not self.fighter:HasCondition("spark_aura") then
                            self.fighter:AddCondition("spark_aura")
                        end
                    end
                },

                OnAllyKill = function( self )
                    if math.random() < 0.4 then
                        self.fighter:SayQuip("drone_goon_death_quips")
                    end
                end,
            },
        },

        negotiation_data = 
        {
            modifiers = 
            {
                GADGETEER = 
                {
                    name = "Gadgeteer",
                    desc = "At the end of {1}'s turn, {1} creates 2 random <b>Gadget</> arguments. {1} can at most have 8 <b>Gadget</> arguments at a time.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName(), self.spawn_rate, self.max_gadgets)
                    end,

                    modifier_type = MODIFIER_TYPE.CORE,
                },

                psychic_siren = 
                {
                    name = "Psychic Siren",
                    desc = "When this is destroyed, {1}'s core argument takes {2} damage.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOwnerName(), self.bounty_damage)
                    end,
                    gadget = true,

                    OnApply = function( self )
                        local core = self.negotiator:FindCoreArgument()
                        self.bounty_damage = core and core.bounty_damage or 5
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,
                    target_enemy = TARGET_ANY_RESOLVE,

                    OnBeginTurn = function( self, minigame )
                        self:ApplyPersuasion()
                    end,

                    OnBounty = function( self, card )
                        local core = self.negotiator:FindCoreArgument()
                        if core and self.bounty_damage then
                            core:AttackResolve(self.bounty_damage, self)
                        end
                    end,

                    max_resolve = 3,
                    min_persuasion = 3,
                    max_persuasion = 3,
                    max_stacks = 1,
                },

                distortion_generator = 
                {
                    name = "Distortion Generator",
                    desc = "At the start of {1}'s turn, apply {3} {COMPOSURE} to a random argument. When this is destroyed, {1}'s core argument takes {2} damage.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOwnerName(), self.bounty_damage, self.composure_amt)
                    end,
                    gadget = true,

                    OnApply = function( self )
                        local core = self.negotiator:FindCoreArgument()
                        self.bounty_damage = core and core.bounty_damage or 5
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,

                    OnBounty = function( self, card )
                        local core = self.negotiator:FindCoreArgument()
                        if core and self.bounty_damage then
                            core:AttackResolve(self.bounty_damage, self)
                        end
                    end,

                    max_resolve = 3,
                    max_stacks = 1,
                    max_instances = 2,
                    composure_amt = 3,

                    event_handlers =
                    {
                        [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator then
                                local targets = {}
                                for i, modifier in self.negotiator:ModifierSlots() do
                                    if modifier:GetResolve() ~= nil then
                                        table.insert( targets, modifier )
                                    end
                                end
                                local target = table.arraypick( targets )
                                if target then
                                    target:DeltaComposure( self.composure_amt, self )
                                    self:NotifyTriggered() 
                                end
                            end
                        end,
                    },     
                },

                muffler = 
                {
                    name = "Muffler",
                    desc = "At the start of {1}'s turn, restore {3} resolve to {1}'s core argument. When this is destroyed, {1}'s core argument takes {2} damage.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOwnerName(), self.bounty_damage, self.resolve_amt)
                    end,
                    gadget = true,

                    OnApply = function( self )
                        local core = self.negotiator:FindCoreArgument()
                        self.bounty_damage = core and core.bounty_damage or 5
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,

                    OnBounty = function( self, card )
                        local core = self.negotiator:FindCoreArgument()
                        if core and self.bounty_damage then
                            core:AttackResolve(self.bounty_damage, self)
                        end
                    end,

                    max_resolve = 6,
                    resolve_amt = 2,
                    max_stacks = 1,
                    max_instances = 2,

                    event_handlers =
                    {
                        [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator then
                                local core = self.negotiator:FindCoreArgument()
                                if core then
                                    core:RestoreResolve(self.resolve_amt, self)
                                    self:NotifyTriggered() 
                                end
                            end
                        end
                    }
                },

                sonic_decoy =
                {
                    name = "Sonic Decoy",
                    desc = "Must be targeted before anything else. When this is destroyed, {1}'s core argument takes {2} damage.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self:GetOwnerName(), self.bounty_damage)
                    end,
                    gadget = true,

                    OnApply = function( self )
                        local core = self.negotiator:FindCoreArgument()
                        self.bounty_damage = core and core.bounty_damage or 5
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,

                    OnBounty = function( self, card )
                        local core = self.negotiator:FindCoreArgument()
                        if core and self.bounty_damage then
                            core:AttackResolve(self.bounty_damage, self)
                        end
                    end,

                    max_resolve = 5,
                    force_target = true,
                    max_instances = 1,
                    max_stacks = 1,

                    CanPlayCard = function( self, source, engine, target )
                        -- Only verify forced targets if you are not targetting yourself
                        if source:IsAttack() and target:GetNegotiator() == self.negotiator then
                            if source.modifier_type == MODIFIER_TYPE.INCEPTION or source:GetNegotiator() ~= self.negotiator then
                                if not target.force_target then
                                    return false, loc.format( LOC "CONVO_COMMON.MUST_TARGET", self:GetName() )
                                end
                            end
                        end

                        return true
                    end,
                },
            },

            behaviour =
            {
                OnInit = function( self, difficulty )
                    if CheckBits( self.engine.start_params.flags, NEGOTIATION_FLAGS.WORDSMITH ) then
                        self.attacks = self:MakePicker()
                            :AddArgument( "psychic_siren", 3 )
                            :AddArgument( "distortion_generator", 2 )
                            :AddArgument( "muffler", 1 )
                            :AddArgument( "sonic_decoy", 1 )

                        self:SetPattern( self.Cycle )

                        self.negotiator:AddModifier("GADGETEER")
                    else
                        self.all_business = self:AddArgument( "ALL_BUSINESS" )

                        self.attacks = self:MakePicker()
                        self.attacks:AddID( "straw_man", 1 )

                        self.bounty = self:AddArgument( "SCRUPLE" )

                        self.negotiator:AddModifier("APPROPRIATOR")
                        self:SetPattern( self.SparkCycle )
                    end
                end,

                Cycle = function( self, turns )
                    self:ChooseGrowingNumbers( 2, 1 )

                    local count = 0
                    for i, modifier in ipairs( self.negotiator.all_modifiers ) do
                        if modifier.gadget then
                            count = count + 1
                        end
                    end
                    if count < 8 then
                        self.attacks:ChooseCards( math.min( 2, 8 - count ) )
                    end
                    self:ChooseComposure( 1, 1, 3 )
                end,

                SparkCycle = function( self, turns )
                    if (turns - 1) % 6 == 0 and not self.negotiator:HasModifier( "ALL_BUSINESS" ) then
                        self:ChooseCard( self.all_business )
                        self:ChooseGrowingNumbers( 1, 0 )
                    else
                        if turns % 2 == 0 then
                            self.attacks:ChooseCard( 1 )
                            self:ChooseGrowingNumbers( 1, 0 )

                            if math.random() < 0.3 and not self.negotiator:HasModifier( "SCRUPLE" ) and self.negotiator:GetResolve() ~= nil then
                                self:ChooseCard( self.bounty )
                            end
                        else
                            self:ChooseGrowingNumbers( 3, 0, 0.75 )
                        end
                    end
                end,
            }
        },
    }),

    CharacterDef("WANDERING_CHEF",
    {
        base_def = "BARTENDER",
        species = SPECIES.KRADESHI,
        gender = GENDER.MALE,
        name = "Rug",
    }),

    CharacterDef("GAMBLER",
    {
        base_def = "JAKES_RUNNER",
        species = SPECIES.JARACKLE,
        gender = GENDER.MALE,
        name = "Krog",

    }),    

    CharacterDef("COIN_TRADER",
    {
        base_def = "ADMIRALTY_PATROL_LEADER",
        build = "guard_male_offduty",
        head = "head_male_kradeshi_05",
        title = "Coin Trader",
        species = SPECIES.KRADESHI,
        gender = GENDER.MALE,
        name = "Pengemunt",
    }),    


    CharacterDef("WEAPONSMITH",
    {
        base_def = "ADMIRALTY_CLERK",
        build = "guard_female_offduty",
        --head = "head_male_kradeshi_05",
        title = "Weaponsmith",
        species = SPECIES.JARACKLE,
        gender = GENDER.FEMALE,
        name = "Frizz",

        loved_graft = "weapons_expert",
        hated_graft = "treachery",
        death_item = "lumin_jolt",
    }),    


    CharacterDef("DANGEROUS_STRANGER",
    {
        base_def = "CHEMIST",
        species = SPECIES.HUMAN,
        gender = GENDER.MALE,
        name = "Ximmon",

        voice_actors = {
            [GENDER.MALE] = { "mysteriousstranger" },
            },
    }),    

    CharacterDef("HESH_OUTPOST_LUMINARI",
    {
        base_def = "LUMINARI",
        species = SPECIES.SHROKE,
        gender = GENDER.MALE,
        name = "Murkot",
        build = "luminari_male_02",
        head = "head_male_shroke_05",
    }),    


    CharacterDef("BEASTMASTER",
    {
        base_def = "BANDIT_RAIDER",
        species = SPECIES.SHROKE,
        gender = GENDER.MALE,
        name = "Phroluk",
        build = "male_shroke_beastmaster_build",
        head = "head_male_shroke_beastmaster",
        has_hood = true,
        tags = {"plot_armour", "no_patron"},
        loved_graft = "",
        hated_graft = "",
    }),    

    
    -- DONT DO THIS, this should be a skin not a new Def.
    CharacterDef("HESH_OUTPOST_PRIEST",
    {
        base_def = "PRIEST",
        species = SPECIES.KRADESHI,
        gender = GENDER.FEMALE,
        head = "head_female_kradeshi_09",
        name = "Phrylotas",
    }),    

    -- DONT DO THIS, this should be a skin not a new Def.
    CharacterDef("RISE_OUTPOST_REBEL1",
    {
        base_def = "RISE_REBEL",
        species = SPECIES.KRADESHI,
        gender = GENDER.FEMALE,
        --head = "head_female_kradeshi_09",
        name = "Brenna",
        bio = "Brenna is a true believer in the goals of the Rise, even if the Rise is not always a true believer in the goals of Brenna."
    }),    

    CharacterDef("BRAVE_MERCHANT",
    {
        base_def = "WEALTHY_MERCHANT",
        title = "Downtrodden Merchant",
        build = "shopkeep_female",
        species = SPECIES.HUMAN,
        gender = GENDER.FEMALE,
        name = "Shel",
        fight_data =
        {
            MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,
            MAX_MORALE = MAX_MORALE_LOOKUP.LOW,

            attacks = 
            {
                slash = table.extend(NPC_ATTACK)
                {
                    name = "Slash",
                    anim = "slice",
                    flags = CARD_FLAGS.MELEE,

                    --damage_mult = 0.75,
                },

                rally = table.extend(NPC_BUFF)
                {
                    name = "Rally",
                    anim = "taunt",
                    quip = "shel_rally",
                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.FRIENDLY_OR_SELF,
                    
                    CanPlayCard = function(self, battle, target )
                        if target then
                            return target:GetConditionStacks("POWER") <= 3
                        else
                            return true
                        end
                    end,

                    OnPostResolve = function(self, battle, attack)
                        attack.target:AddCondition("POWER")
                    end,
                },
            },

            cards = { "slash", "rally" },

            behaviour = 
            {
                OnActivate = function( self )
                    self.attacks = self:MakePicker()
                        :AddID( "slash", 1 )
                        :AddID( "rally", 1 )
                    self:SetPattern( self.Attack )
                end,
                Attack = function( self )
                    self.attacks:ChooseCard()
                end,
            },
        },
        anims = {"anim/weapon_knife_guard_lumin.zip"},
        combat_anims = { "anim/med_combat_knife_admiralty_clerk.zip" },

    }),    
    
    CharacterDef("DODGY_SCAVENGER",
    {
        base_def = "JAKES_RUNNER",
        title = "Dodgy Scavenger",
        build = "female_grifter",
        head = "head_female_grifter",
        species = SPECIES.HUMAN,
        gender = GENDER.FEMALE,
        name = "Plundak",
        loved_graft = "",
        hated_graft = "",
        tags = {"plot_armour", "no_patron"},
    }),    

    CharacterDef("DEEP_BOGGER",
    {
        base_def = "CHEMIST",
        title = "Bogger",
        build = "bogger",
        faction_id = "BOGGERS",
        gender = GENDER.FEMALE,
        name = "Grimdellia",
        voice_actor = "spreeLowClassFemale01",
        loved_graft = "talisman",
        hated_graft = "bogling",
        death_item = "seedling",
    }),  

    CharacterDef("HESH_AUCTIONEER",
    {
        base_def = "PRIEST",
        title = "Auctioneer",
        build = "male_hesh_auctioneer",
        faction_id = "CULT_OF_HESH",
        species = SPECIES.KRADESHI,
        gender = GENDER.MALE,
        name = "Hundruthor",
        voice_actor = "civilianHighClassMale01",
        loved_graft = "",
        hated_graft = "",
        negotiation_data =
        {
            behaviour = {

                OnInit = function( self, difficulty )
                    self.negotiator:AddModifier( "AUCTIONEER" )

                    self.bids = self:AddArgument( "BIDDER" )
                    self.bids2 = self:AddArgument( "BIDDER" )
                    self.defend = self:AddCard("auctioneer_defend_items" )
                    self.add_item = self:AddCard( "auctioneer_add_item" )

                    self:SetPattern( self.Cycle )
                end,   
                
                Cycle = function( self, turns )
                     
                    local bids, items = 0, 0
                    for i, mod in self.negotiator:Modifiers() do
                        if mod.id == "BIDDER" then
                            bids = bids + 1   
                        end
                        if mod.id == "auction_item" then
                            items = items + 1   
                        end
                    end

                    if self.engine:CanPlayCard( self.add_item ) and items < 2 then
                        self:ChooseCard( self.add_item )
                    end

                    if items > 0 then
                        if bids == 0 then
                            -- No bids? Always add 2 more.
                            self:ChooseCard( self.bids )
                            self:ChooseCard( self.bids2 )
                        elseif bids >= 3 then
                            self:ChooseCard( self.defend )
                        else
                            -- Either another bidder, or defend.
                            local options = { self.bids, self.defend }
                            local option = table.arraypick(options)
                            if option then
                                self:ChooseCard( option )
                            end
                        end
                    else
                        -- SHould be over by now!
                        self:ChooseGrowingNumbers( 3, 3 )
                    end
                end,
            }
        }
    }),
    
    CharacterDef("THEROUX",
    {
        base_def = "WEALTHY_MERCHANT",
        species = SPECIES.KRADESHI,
        head = "head_male_theroux",
        build = "male_theroux_build",
        gender = GENDER.MALE,
        name = "Theroux",
        theme_music = "event:/music/story/smith/story_smith_banquods",
        title = "Entrepreneur",
        tags = {"plot_armour", "no_patron"},
        faction_id = "FEUD_CITIZEN",
        death_item = "hat_tip",
        loved_graft = "fan_mail",
        hated_graft = "celebrity_gossip"
    }),

    CharacterDef("STEVEN",
    {
        base_def = "NPC_BASE",
        can_talk = true,
        species = SPECIES.MECH,
        gender = GENDER.UNDISCLOSED,
        name = "Steven",
        title = "Party Machine",
        voice_actor = "robotSteven",
        build = "job_vendor",
        head = "head_job_vendor",
        faction_id = "FEUD_CITIZEN",
        tags = {"plot_armour", "no_patron"},

        aspects = {
            social_connections = false,
        },
    }),    


}

for _, def in pairs(chars) do
    def.alias = def.id
    def.unique = true
    Content.AddCharacterDef( def )
end
