local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local def = CharacterDef("RISE_TURNCOAT_BOSS",
{
    unique = true,

    base_def = "RISE_BASE",
    alias = "RISE_TURNCOAT_BOSS",
    species = SPECIES.SHROKE,
    gender = GENDER.MALE,
    renown = 4,
    combat_strength = 2,
    boss = true,
    death_item = "bolstered_plating",
    -- name = "",

    loved_graft = "boosted_cybernetics",
    hated_graft = "misinformation",

    faction_id = "SPARK_BARONS",
    title = "Turncoat",
    voice_actor = "riseTurncoatBoss",
    
    combat_anims = {"anim/med_combat_turncoat_boss.zip"},
    anims = {"anim/weapon_staff_turncoat_boss.zip"},

    build = "male_turncoat_boss_build",
    -- head = "",
    has_hood = true,

    battle_preview_anim = "anim/boss_turncoat_slide.zip",
    battle_preview_offset = { x = 120, y = 5 },
    battle_preview_glow = { colour = 0xFF9950FF, bloom = 0.4, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_turncoat",
    icon = engine.asset.Texture( "icons/b_rise_turncoat.tex" ),

    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 95,
        battle_scale = 1.12,

        conditions =
        {
            voltaic = 
            {
                name = "Voltaic",
                desc = "At the end of your turn, take <#HILITE>{1} damage</> for every {GALVANIZED} card you have played this turn.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self.stacks or 1)
                end,

                ctype = CTYPE.DEBUFF,

                event_handlers =
                {
                    [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                        if attack.attacker == self.owner and attack.card:IsFlagged(CARD_FLAGS.GALVANIZED) then
                            self.owner:AddCondition("static_buildup", self.stacks, self)
                        end
                    end,
                }
            },

            static_buildup =
            {
                name = "Static Buildup",
                desc = "Generated by Voltaic. At the end of your turn, take <#HILITE>{1}</> damage and remove this condition.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self.stacks or 1)
                end,

                ctype = CTYPE.DEBUFF,

                PreviewFn = function( self, damage_preview, targets )
                    if targets[self.owner] == nil then
                       targets[self.owner] = {} 
                    end
                    table.insert(targets[self.owner], { condition = self, min_damage = self.stacks, max_damage = self.stacks, owner = self.owner })
                end,

                event_handlers =
                {
                    [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        if self.owner == fighter then
                            self.owner:ApplyDamage(self.stacks, self.owner, self)
                            self.owner:RemoveCondition(self.id, self.stacks, self)
                        end
                    end,
                }
            },

            galvanized_tracker =
            {
                hidden = true,
                event_handlers =
                {
                    [ BATTLE_EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                        if target_deck == self.battle:GetDiscardDeck() and card:IsFlagged(CARD_FLAGS.GALVANIZED) then
                            card:ClearFlags(CARD_FLAGS.GALVANIZED)
                        end
                    end
                },
            },

            powered_weapons =
            {
                name = "Powered Weapons",
                desc = "{1}'s attacks apply {GALVANIZED} to random cards in your deck.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName())
                end,
            },
        },

        attacks = 
        {
            turncoat_smash = table.extend(NPC_ATTACK)
            {
                name = "Smash",
                anim = "smash",
                flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,
                target_mod = TARGET_MOD.TEAM,

                voltaic_amt = { 2, 2, 2, 2},
                base_damage = { 6, 8, 10, 12 },
                galvanized_amt = { 1, 2, 3, 3 }, -- per 10 cards in draw
                voltaic_cap = { 2, 4, 4, 6 },

                OnPostResolve = function( self, battle, attack )
                    for i,hit in ipairs(attack.hits) do
                        if hit.target:IsPlayer() and not hit.evaded then
                            hit.target:AddCondition("voltaic", self.voltaic_amt[ math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.voltaic_amt) ], self)
                            local deck = table.merge( battle:GetDiscardDeck().cards, battle:GetDrawDeck().cards )
                            local deck_size_mult = #deck / 10
                            local chosen_cards = table.multipick(deck, math.floor( self.galvanized_amt[ math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.galvanized_amt)] * deck_size_mult) )
                            for i,card in ipairs(chosen_cards) do
                                card:SetFlags(CARD_FLAGS.GALVANIZED)
                            end
                            if hit.target:GetConditionStacks("voltaic") >= self.voltaic_cap[ math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.voltaic_amt) ] then
                                self.owner.behaviour:SetPattern( self.owner.behaviour.SlowCycle )
                            end
                        end
                    end
                end
            },

            turncoat_laser_eye = table.extend(NPC_ATTACK)
            {
                name = "Jolting Beam",
                anim = "laser_eye",
                flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,

                CanPlayCard = function( self, battle, target )
                    return target == nil or target:IsPlayer()
                end,

                galvanized_amt = { 1, 2, 3, 3}, -- per 10 cards in draw
                base_damage = { 4, 6, 8, 10 },

                OnPostResolve = function( self, battle, attack )
                    for i,hit in ipairs(attack.hits) do
                        if hit.target:IsPlayer() and not hit.evaded then
                            local deck = table.merge( battle:GetDiscardDeck().cards, battle:GetDrawDeck().cards )
                            local deck_size_mult = #deck / 10
                            local chosen_cards = table.multipick(deck, math.ceil( self.galvanized_amt[ math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.galvanized_amt)] * deck_size_mult) )
                            for i,card in ipairs(chosen_cards) do
                                card:SetFlags(CARD_FLAGS.GALVANIZED)
                            end
                        end
                    end
                    
                end
            },

            turncoat_taunt = table.extend(NPC_BUFF)
            {
                name = "Reinforce",
                anim = "taunt",
                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                defend_amt = { 6, 10, 12, 15},

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                end
            },
        },

        behaviour =
        {
            OnActivate = function( self )
                    self.smash = self:AddCard("turncoat_smash")
                    self.attacks = self:MakePicker()
                        :AddID( "turncoat_smash", 1 )
                        :AddID( "turncoat_laser_eye", 2 )
                    self.slow_cycle = self:MakePicker()
                        :AddID( "turncoat_smash", 1 )
                        :AddID( "turncoat_laser_eye", 3 )
                        :AddID( "turncoat_taunt", 2 )
                    self.taunt = self:AddCard("turncoat_taunt")

                    self:SetPattern( self.Cycle )
                    self.fighter:AddCondition("galvanized_tracker")
                    self.fighter:AddCondition("ARMOURED", 3, self)
                    self.fighter:AddCondition("powered_weapons", 1, self)
                end,

                Cycle = function( self )
                    local player = self.fighter.battle:GetPlayerFighter()
                    if player and not player:HasCondition("voltaic") then
                        self:ChooseCard(self.smash)
                    elseif self.battle:GetTurns() % 3 == 0 then
                        self:ChooseCard(self.taunt)
                    else
                        self.attacks:ChooseCard()
                    end
                end,

                SlowCycle = function( self )
                    self.slow_cycle:ChooseCard()
                end
        },
    },
})

Content.AddCharacterDef( def )
    