local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

Content.AddCharacterDef
(
    CharacterDef("SHREDMAW",
    {
        base_def = "MONSTER",

        name = "Shredmaw",
        title = "Devouring Horror",
        alias = "SHREDMAW",

		boss = true,
        battle_preview_anim = "anim/boss_snug_slide.zip",
        battle_preview_offset = { x = 190, y = 20, scale = 0.90 },
        battle_preview_glow = { colour = 0xF4BE5FFF, bloom = 0.2, threshold = 0.02 },
        -- battle_preview_audio = "event:/ui/prebattle_overlay/",
        icon = engine.asset.Texture( "icons/b_shredmaw.tex" ),
        compendium_offset = { -200, -50 },
        
        build = "snug",

        death_item = "shredmaw_shell",

        fight_data =
        {   
            battle_scale = 1.2,
            MAX_HEALTH = 180,
            MAX_MORALE = MAX_MORALE_LOOKUP.IMMUNNE,
            death_fade_delay = 1,
            status_widget_head_dx = -2.0,
            status_widget_head_dy = 5.4,

            conditions = 
            {
                caustic_spit =
                {
                    name = "Caustic Spit",
                    desc = "{1}'s attacks apply {SLIMY} to {2} cards in your deck.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName(), self.stacks or 1)
                    end,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                            if not hit:CheckHitResult( "evaded" ) and hit.card:IsDamageCard() and hit.attacker == self.owner and hit.target:IsPlayer() then
                                -- hit.target:AddCondition("slimed", self.stacks, self)
                                local deck = table.merge( battle:GetDiscardDeck().cards, battle:GetDrawDeck().cards, battle:GetHandDeck().cards )
                                local options = ObtainWorkTable()
                                for i,v in ipairs(deck) do
                                    if not v:IsFlagged(CARD_FLAGS.SLIMY) then
                                        table.insert(options, v)
                                    end
                                end
                                local chosen_cards = table.multipick(options, self.stacks )
                                for i,card in ipairs(chosen_cards) do
                                    card:SetFlags(CARD_FLAGS.SLIMY)
                                end
                                ReleaseWorkTable(options)
                            end
                        end,
                    },
                },

                LEECHING_FANGS  =
                {
                    name = "Leeching Fangs",
                    desc = "{1} heals health equal to half of any health loss it causes.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName() )
                    end,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                            if not hit:CheckHitResult( "evaded" ) and hit.damage_dealt and hit.damage_dealt > 0 and hit.card:IsDamageCard() and hit.attacker == self.owner then
                                self.owner:HealHealth(math.floor(hit.damage_dealt*0.5), self)
                            end
                        end,
                    },
                },

                -- slimed =
                -- {
                --     name = "Slimed",
                --     desc = "At the start of your turn, apply {SLIMY} to <#HILITE>{1}</> {1*cards|cards} in your hand and remove <b>Slimed</>.",
                --     desc_fn = function( self, fmt_str )
                --         return loc.format(fmt_str, self.stacks or 1)
                --     end,

                --     ctype = CTYPE.DEBUFF,

                --     event_handlers =
                --     {
                --         [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                --             local options = ObtainWorkTable()
                --             for i,v in ipairs(battle:GetHandDeck().cards) do
                --                 if not v:IsFlagged(CARD_FLAGS.SLIMY) then
                --                     table.insert(options, v)
                --                 end
                --             end
                --             local chosen_cards = table.multipick(options, self.stacks )
                --             for i,card in ipairs(chosen_cards) do
                --                 card:SetFlags(CARD_FLAGS.SLIMY)
                --             end
                --             self.owner:RemoveCondition("slimed", self.stacks, self)
                --             ReleaseWorkTable(options)
                --         end
                --     },
                -- },

                slimy_tracker =
                {
                    hidden = true,
                    event_handlers =
                    {
                        -- [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                        --     if fighter == self.battle:GetPlayerFighter() then
                        --         for i,card in ipairs(self.battle:GetHandDeck().cards) do
                        --             if card:IsFlagged(CARD_FLAGS.SLIMY) then
                        --                 card:NotifyTriggered()
                        --                 card.owner:ApplyDamage( 3 )
                        --                 -- Don't remove sticky if the card usually has it
                        --                 local remove_flags = ClearBits(CARD_FLAGS.SLIMY | CARD_FLAGS.STICKY, card.def.flags)
                        --                 card:ClearFlags(remove_flags)
                        --             end
                        --         end
                        --     end
                        -- end

                        [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, resolve_card )
                            if resolve_card:IsFlagged(CARD_FLAGS.SLIMY) then
                                resolve_card:ClearFlags(CARD_FLAGS.SLIMY)
                                resolve_card.owner:ApplyDamage( 3, resolve_card.owner, nil, false, {"acid"} )
                            end
                        end
                    },
                }
            },
            attacks = 
            {
                shredmaw_bite = table.extend(NPC_ATTACK)
                {
                    name = "Monch",
                    anim = "bite",
                    CanPlayCard = function( self, battle, target )
                        return target == nil or not target:GetAgent():IsPet()
                    end,

                    flags = CARD_FLAGS.MELEE,
                    base_damage = { 6, 10, 12, 16 },
                },

                shredmaw_pet_consume = table.extend(NPC_ATTACK)
                {
                    name = "Consume",
                    anim = "bite",

                    flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,
                    base_damage = 1,

                    CanPlayCard = function( self, battle, target )
                        return target == nil or target:GetAgent():IsPet()
                    end,

                    OnPreResolve = function( self, battle, attack )
                        for i,hit in ipairs(attack.hits) do
                            -- Leave the pet at 5 health
                            hit.damage = math.max( 0, hit.target:GetHealth() - 5 )
                        end
                    end,
                    OnPostResolve = function( self, battle, attack )
                        local function InstaKill( fighter, battle )
                            battle:BroadcastEvent( BATTLE_EVENT.CUSTOM, fighter, function( fight_screen, ent )
                                local anim_fighter = ent.cmp.AnimFighter
                                anim_fighter:StopMoving()

                                if anim_fighter.status_widget then
                                    anim_fighter.status_widget:AlphaTo( 0, 0.5 )
                                end

                                local t = 0 
                                local total_t = 0.25
                                while t < total_t do
                                    local dt = coroutine.yield()
                                    t = t + dt
                                    local alpha = 1 - (t/total_t)
                                    anim_fighter.entity.cmp.AnimController:SetTintMult(1,1,1, alpha)
                                    if anim_fighter.dropshadow then
                                        anim_fighter.dropshadow.cmp.PlaneRenderer:TintColour(0,0,0, .5*alpha)
                                    end
                                end
                                anim_fighter.entity:Despawn()
                            end )
                            local team = fighter.team
                            fighter.status = FIGHT_STATUS.DEPARTED
                            team:RemoveFromField( fighter )

                        end
                        for i,hit in ipairs(attack.hits) do
                            if not hit.evaded then
                                local agent = hit.target:GetAgent()
                                local quest = battle:GetScenario().quest
                                if quest then
                                    quest.param.pets_eaten = quest.param.pets_eaten or {}
                                    table.insert(quest.param.pets_eaten, agent) 
                                end
                                InstaKill(hit.target, battle)
                            end
                        end
                    end,

                    event_priorities =
                    {
                        [ BATTLE_EVENT.CALC_DAMAGE ] = EVENT_PRIORITY_MULTIPLIER,
                    },

                    event_handlers =
                    {
                        [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                            if card == self and target then
                                local max = target:GetMaxHealth()
                                dmgt:ModifyDamage( max, max, self )
                            end
                        end,
                    }
                },

                shredmaw_tongue = table.extend(NPC_ATTACK)
                {
                    name = "Tongue Lash",
                    anim = "tongue",

                    flags = CARD_FLAGS.MELEE | CARD_FLAGS.PIERCING,
                    force_run = false,
                    base_damage = { 3, 6, 8, 10 },
                },

                shredmaw_taunt = 
                {
                    name = "Shriek",
                    anim = "taunt",

                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.DEBUFF,

                    target_mod = TARGET_MOD.TEAM,

                    defend_amt = { 4, 8, 10, 12 },
                    impair_amt = { 1, 2, 2, 3 },
                    morale_damage = { 0.05, 0.1, 0.12, 0.14 },
                    power_amt = { 1, 3, 5, 8 },

                    OnPostResolve = function( self, battle, attack )
                        self.owner:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                        self.owner:AddCondition("POWER", self.power_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                        self.owner:AddCondition("POWER_LOSS", self.power_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                        local morale_damage = self.morale_damage[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )]
                        for i, hit in attack:Hits() do
                            if hit.target:HasMorale() then
                                hit.target:DeltaMorale( morale_damage )
                            end
                            hit.target:AddCondition("IMPAIR", self.impair_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self )
                        end
                    end
                },
            },

            behaviour = 
            {
                shell_stacks = { 3, 6, 8, 10 },
                caustic_amt = { 2, 3, 4, 6 },
                OnActivate = function( self )
                    self.fighter:AddCondition( "thick_shell", self.shell_stacks[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] )
                    self.fighter:AddCondition("slimy_tracker")
                    self.fighter:AddCondition("caustic_spit", self.caustic_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1])
                    self.fighter:AddCondition("LEECHING_FANGS")
                    self.fighter:AddCondition( "DEFEND", self.shell_stacks[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1] )

                    self.attacks = self:MakePicker()
                        :AddID("shredmaw_bite", 3)
                        :AddID("shredmaw_tongue", 2)
                        :AddID("shredmaw_taunt", 1)
                        :AddID("shredmaw_pet_consume", 1)

                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    return self.attacks:ChooseCard()
                end,
            },
        },
    })
)