local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local SHROOGLET_SUMMON_CHANCE = { 1, .5, .25, 0}
Content.AddCharacterDef
(
    CharacterDef("SHROOG",
    {
        designer_note = "Calls shrooglets that do lots of damage and are easily killed. Everytime one is killed Momma Shroog powers up.",

        base_def = "MONSTER",
        name = "Shroog",
        title = "Slobbering Aberration",
        shorttitle = "Slobbering Aberration",
        gender = GENDER.FEMALE,
        build = "shroog",
        combat_strength = 4,
        boss = true,
        battle_preview_anim = "anim/boss_shroog_slide.zip",
        battle_preview_offset = { x = 165, y = 70 },
        battle_preview_glow = { colour = 0x81C67EFF, bloom = 0.12, threshold = 0.02 },
        battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_shroog",
        icon = engine.asset.Texture( "icons/b_shroog.tex"),
        compendium_offset = { -200, 0 },
        death_item = "shroog_fang",

        preload_defs = {"SHROOGLET"},
        
        fight_data =
        {
            battle_scale = 1.25,
            MAX_HEALTH = 240,
            MAX_MORALE = MAX_MORALE_LOOKUP.IMMUNNE,
            
            formation = FIGHTER_FORMATION.BACK_Z | FIGHTER_FORMATION.BACK_X,
            target_weight = 5,
            shadow_scale = 14.5,

            status_widget_head_dx = 1.0,
            status_widget_head_dy = 3.0,
            head_symbol_dy = -3.0,

            cards = { "shroog_claw", "shroog_swipe", "shroog_summon_shrooglet" , "shroog_enrage"},

            conditions = 
            {
                vengeful =
                {
                    name = "Maternal Instinct",
                    desc = "Gains {POWER {1}} whenever an ally dies.",
                    desc_fn = function(self, fmt_str)
                        return loc.format( fmt_str, self.stacks )
                    end,
                    ctype = CTYPE.BUFF,
                },
            },

            attacks = 
            {
                shroog_claw = table.extend(NPC_ATTACK)
                {
                    name = "Claw",
                    anim = "claw",
                    flags = FLAGS.MELEE,

                    base_damage = {6,8,9,9},
                },

                shroog_swipe = table.extend(NPC_ATTACK)
                {
                    name = "Swipe",
                    anim = "swipe",
                    flags = FLAGS.RANGED,
                    target_mod = TARGET_MOD.TEAM,

                    base_damage = {4,5,6,8},
                },

                shroog_taunt = table.extend(NPC_BUFF)
                {
                    name = "Taunt",
                    anim = "taunt",
                    
                    flags = FLAGS.SKILL,
                    
                    target_type = TARGET_TYPE.SELF,
                    
                    features =
                    {
                        DEFEND = 16,
                    },

                },

                shroog_enrage = table.extend(NPC_BUFF)
                {
                    name = "Enrage",
                    anim = "taunt",
                    target_type = TARGET_TYPE.SELF,

                    OnPostResolve = function(self, battle, attack)
                        self.owner:AddCondition("POWER", self.owner:GetConditionStacks("vengeful")*self.count)
                        self.count = 0
                    end,
                },

                shroog_summon_shrooglet = table.extend(NPC_BUFF)
                {
                    name = "Summon Shrooglets",
                    anim = "taunt",

                    flags = FLAGS.SKILL | FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    CanPlayCard = function(self, battle)
                        return self.owner:GetTeam():NumActiveFighters() < 4
                    end,

                    defend_amt = {15, 17, 20, 30},

                    OnPostResolve = function( self, battle, attack )
                        local to_summon = 4 - self.owner:GetTeam():NumActiveFighters()
                        for k =1 , to_summon do
                            local agent = Agent( "SHROOGLET" )
                            local new_fighter = Fighter.CreateFromAgent( agent, self.owner:GetScale() )
                            new_fighter:AddCondition("run_away_shrooglet")
                            self.owner:GetTeam():AddFighter( new_fighter )
                        end
                        self.owner:GetTeam():ActivateNewFighters( battle )
                        self.owner:AddCondition("DEFEND", self.defend_amt[ GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1 ] )
                        --sim.mgr:CalculateHomePositions( self.owner:GetTeam(), true )
                    end,
                },
            },

            behaviour =
            {
                OnActivate = function( self )

                    local VENGEFUL_STACKS = {1,1,1,2}

                    self.fighter:AddCondition( "vengeful", VENGEFUL_STACKS[ GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1 ] )
                    self.enrage_card = self:AddCard("shroog_enrage")
                    
                    self.single_attacks = self:MakePicker()
                        :AddID( "shroog_claw", 1 )
                        --:AddID( "shroog_taunt", 1 )   

                    self.multi_attacks = self:MakePicker()
                        :AddID( "shroog_claw", 1 )
                        :AddID( "shroog_swipe", 2 )
                        --:AddID( "shroog_taunt", 3 )   -- defends

                    self.summon_attacks = self:MakePicker()
                        :AddID( "shroog_summon_shrooglet", 1 )
                        --:AddID( "shroog_taunt", 2 )   -- defends

                    self:SetPattern( self.Cycle )
                    
                    self.opener = self:AddCard("shroog_summon_shrooglet")
                    self.fighter.battle:PlayCard(self.opener)
                end,


                Cycle = function( self )
                    
                    local chance_summon = SHROOGLET_SUMMON_CHANCE[self.fighter:GetTeam():NumActiveFighters()] or 0
                    
                    if math.random() < chance_summon then
                        self.summon_attacks:ChooseCard()
                    else

                        if self.fighter:GetTeam():GetEnemyTeam():NumActiveFighters() > 1 then
                            self.multi_attacks:ChooseCard()
                        else
                            self.single_attacks:ChooseCard()
                        end
                        
                    end
                end,

                OnAllyKill = function( self, turns, ally )
                    if ally and ally.agent:GetContentID() == "SHROOGLET" and not self.fighter:HasCondition("STUN") then
                        self.enrage_card.count = self.enrage_card.count and self.enrage_card.count + 1 or 1
                        self.fighter.battle:PlayCard(self.enrage_card)
                    end
                end,

            },
        },

        negotiation_data =
        {
            ring_offset = 
            {
                x = 0,
                y = 200,
                z = 0,
            },
            modifiers =
            {
                MATERNAL_INSTINCTS_ARGUMENT =
                {
                    name = "Maternal Instincts",
                    desc = "At the end of {1}'s turn, restore {2} resolve to all <b>Shrooglet</> arguments.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName(), self.resolve_amt)
                    end,

                    modifier_type = MODIFIER_TYPE.CORE,

                    resolve_amt = 2,

                    event_handlers =
                    {
                        [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator then
                                for i, modifier in ipairs( self.negotiator.all_modifiers ) do
                                    if modifier.id == "SHROOGLET_ARGUMENT" then
                                        modifier:RestoreResolve(self.resolve_amt, self)
                                    end
                                end
                            end
                        end,
                    }
                },

                SHROOG_FLOCK =
                {
                    name = "Shroog Flock",
                    desc = "At the end of {1}'s turn, if they have less than 6 {SHROOGLET_ARGUMENT} arguments, create 2 new ones.",
                    desc_fn = function( self, fmt_str )
                        return loc.format(fmt_str, self:GetOwnerName())
                    end,

                    OnApply = function( self )
                        self.apply_turn = self.engine:GetTurns()
                        self.negotiator.behaviour.latest_flock = self.engine:GetTurns()
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,

                    max_resolve = 8,

                    event_handlers =
                    {
                        [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                            if negotiator == self.negotiator and minigame:GetTurns() > self.apply_turn and self.negotiator:GetModifierInstances("SHROOGLET_ARGUMENT") < 6 then
                                for i=1,2 do
                                    self.negotiator:CreateModifier("SHROOGLET_ARGUMENT", 1, self)
                                end
                            end
                        end,
                    }
                },

                SHROOGLET_ARGUMENT = 
                {
                    name = "Shrooglet",
                    desc = "When destroyed, restore {1} resolve to {SHROOG_FLOCK}.",
                    desc_fn = function( self, fmt_str )
                        return loc.format( fmt_str, self.resolve_amt)
                    end,

                    modifier_type = MODIFIER_TYPE.ARGUMENT,
                    target_enemy = TARGET_ANY_RESOLVE,

                    max_resolve = 2,
                    min_persuasion = 1,
                    max_persuasion = 2,

                    bonus_dmg = 1,
                    resolve_amt = 4,

                    OnBeginTurn = function( self, minigame )
                        self:ApplyPersuasion()
                    end,

                    OnBounty = function( self, card )
                        local mod = self.negotiator:FindModifier("SHROOG_FLOCK")
                        if mod then
                            mod:RestoreResolve(self.resolve_amt, self)
                        end
                    end,

                    -- event_priorities =
                    -- {
                    --     [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
                    -- },

                    -- event_handlers =
                    -- {
                    --     [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion, minigame, target )
                    --         if source.negotiator == self.negotiator and is_instance( source, Negotiation.Card ) then
                    --             persuasion:AddPersuasion( self.bonus_dmg, self.bonus_dmg, self )
                    --         end
                    --     end,
                    -- }
                },
            },
            behaviour =
            {
                OnInit = function( self, difficulty )
                    self.attacks = self:MakePicker()
                    self.attacks:AddArgument( "SHROOG_FLOCK", 1 )
                    self.lone_shroog = self:AddArgument("SHROOGLET_ARGUMENT")

                    self:SetPattern( self.Cycle )

                    self.negotiator:AddModifier("MATERNAL_INSTINCTS_ARGUMENT")
                    for i=1,2 do
                        self.negotiator:CreateModifier("SHROOGLET_ARGUMENT", 1, self)
                    end
                end,

                Cycle = function( self, turns )
                    self:ChooseGrowingNumbers( 1, 0, 0.5 )

                    if (self.latest_flock or -3) <= turns - 3 and not self.negotiator:HasModifier("SHROOG_FLOCK")  then
                        self.attacks:ChooseCards( 1 )
                        if not self.negotiator:HasModifier("SHROOGLET_ARGUMENT") then
                            self:ChooseCard(self.lone_shroog)
                        end
                    end
                    if turns % 2 == 0 then
                        self:ChooseComposure( 1, 3, 5 )
                    end
                end, 
            }
        }
    })
)



