require "content/characters/base_types"

local negotiation_defs = require "negotiation/negotiation_defs"
local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS


local SPARK_NEGOTIATION =
{
	OnInit = function( self, difficulty )
		self.all_business = self:AddArgument( "ALL_BUSINESS" )

		self.attacks = self:MakePicker()
		self.attacks:AddID( "straw_man", 1 )

		self.bounty = self:AddArgument( "SCRUPLE" )

		self.negotiator:AddModifier("APPROPRIATOR")
		self:SetPattern( self.Cycle )
	end,

	Cycle = function( self, turns )
		if (turns - 1) % 6 == 0 and not self.negotiator:HasModifier( "ALL_BUSINESS" ) then
			self:ChooseCard( self.all_business )
			self:ChooseGrowingNumbers( 1, 0 )
		else
			if turns % 2 == 0 then
				self.attacks:ChooseCard( 1 )
				self:ChooseGrowingNumbers( 1, 0 )

				if math.random() < 0.3 and not self.negotiator:HasModifier( "SCRUPLE" ) and self.negotiator:GetResolve() ~= nil then
					self:ChooseCard( self.bounty )
				end
			else
				self:ChooseGrowingNumbers( 3, 0, 0.75 )
			end
		end
	end,
}


Content.AddCharacterDef
(
	CharacterDef("SPARK_BARON_BASE",
	{
		base_def = "NPC_BASE",
		is_template = true,
		combat_backup = "SPARK_BARON_BACKUP",
		tags = {"clust"},
		faction_id = "SPARK_BARONS",
		renown = 1,

		voice_actors = {
			[GENDER.MALE] = { "sparkbaronHighClassMale01" },
    		[GENDER.FEMALE] = { "sparkbaronHighClassFemale01" },
    	},

		combat_backup = "SPARK_BARON_BACKUP",
		support_negotiation = "double_duty",

		negotiation_data = 
		{
			behaviour = SPARK_NEGOTIATION
		},

	})
)

-----------------------------------------------------


--PROFESSIONAL:
    -- Tech-using heavy combat unit
    -- Uses grenades, shields, gadgets
    -- Will bail if the going gets too tough.

Content.AddCharacterDef
(
	CharacterDef("SPARK_BARON_PROFESSIONAL",
	{
		designer_note = "** UNUSED **",
		base_def = "SPARK_BARON_BASE",
		title = "Professional",
		renown = 3,
		combat_strength = 5,

		loved_graft = "spark_visor",
		hated_graft = "faulty_armor",
		death_item = "utility_belt",
	    negotiation_ally_image = engine.asset.Texture( "negotiation/modifiers/recruit_spark_baron_professional.tex"),

		base_builds = {
	        [ GENDER.MALE ] = "male_spark_baron_professional",
			[ GENDER.FEMALE ] = "female_spark_baron_professional"
		},

		voice_actors = {
			[GENDER.MALE] = { "sparkbaronHighClassMale01" },
    		[GENDER.FEMALE] = { "sparkbaronHighClassFemale01" },
    	},

		idle_anims = 
		{
			guarding = "idle_neutral_guarding_ranged"
		},


		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.HIGH,
			conditions =
			{
				ENCUMBERED = 
				{
					name = "Encumbered",
					desc = "{1} takes bonus damage from consecutive attacks and after firing their cannons. Resets at the start of {1}'s turn.\nCurrent Bonus: {2}.",
					desc_fn = function( self, fmt_str )
						return loc.format(fmt_str, self:GetOwnerName(), self.stacks - 1)
					end,

					event_handlers =
					{
						[ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
			                if target == self.owner and self.stacks > 1 then
			                    dmgt:AddDamage( self.stacks - 1, self.stacks - 1, self )
			                end
			            end,

			            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
		                	if hit.target == self.owner and attack.card:IsAttackCard() then
		                		self.owner:AddCondition("ENCUMBERED", 1, self)
		                	end
			            end,

			            [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
			            	if self.stacks > 1 then
			            		self.owner:RemoveCondition("ENCUMBERED", self.stacks - 1, self)
			            	end
			            end
					},
				},

				CHARGED_STRIKES =
				{
					name = "Charged Strikes",
					desc = "{1}'s melee attacks deal 3 bonus damage and apply 3 {WOUND} if they cause health loss.",
					desc_fn = function( self, fmt_str )
						return loc.format(fmt_str, self:GetOwnerName() )
					end,

					event_handlers = 
					{
						[ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
			                if card.owner and card.owner == self.owner and card.id == "professional_whack" then
			                    dmgt:AddDamage( 3, 3, self )
			                end
			            end,

			            [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
			            	if attack.attacker == self.owner and attack.card and attack.card.id == "professional_whack" then
			            		self.owner:RemoveCondition(self.id, 1, self)
			            	end
			            end,

			            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
			                if attack.card and attack.card.id == "professional_whack" and attack.attacker == self.owner and not hit.defended and not hit.evaded then
		                    	hit.target:AddCondition("WOUND", 3)
			                end
			                
			            end,
					},
				},
			},

			attacks = 
			{
				professional_whack = table.extend(NPC_ATTACK)
				{
					name = "Whack",
					anim = "whack",
					flags = battle_defs.CARD_FLAGS.MELEE,
					damage_mult = 0.7,
				},

				professional_blast = table.extend(NPC_ATTACK)
				{
					name = "Blast",
					anim = "blast",
					flags = battle_defs.CARD_FLAGS.RANGED,
					damage_mult = 1,
					OnPostResolve = function( self, battle, attack )
						self.owner:AddCondition("ENCUMBERED", 3, self)
					end
				},

				professional_blast_call_in = table.extend(NPC_ATTACK)
				{
					name = "Blast",
					anim = "blast",
					flags = battle_defs.CARD_FLAGS.RANGED | battle_defs.CARD_FLAGS.SPECIAL,
					target_mod = TARGET_MOD.TEAM,
					damage_mult = 1.1,
					OnPostResolve = function( self, battle, attack )
						self.owner:Depart()
					end
				},

				professional_taunt = table.extend(NPC_BUFF)
				{
					name = "Bracing Stance",
					anim = "taunt",

					flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
					target_type = TARGET_TYPE.SELF,

					features =
					{
						DEFEND = 8,
					},
					OnPostResolve = function( self, battle, attack )
						self.owner:AddCondition("CHARGED_STRIKES", 2, self)
					end
				},
			},
			behaviour = 
			{
				OnActivate = function( self )
					self.attacks = self:MakePicker()
						:AddID( "professional_whack", 3 )
						:AddID( "professional_blast", 1 )
					self.taunt = self:AddCard("professional_taunt")

					self:SetPattern( self.Cycle )
					self.fighter:AddCondition( "ENCUMBERED", 1 )
				end,

				Cycle = function( self )
					if not self.fighter:HasCondition("CHARGED_STRIKES") then
						self:ChooseCard(self.taunt)
					else
						self.attacks:ChooseCard()
					end
				end
			},
		},

		negotiation_data = 
		{
			behaviour = table.extend( SPARK_NEGOTIATION )
			{
			}
		},

		anims = {"anim/weapon_tonfa_spark_baron_professional.zip"},
		combat_anims = { "anim/med_combat_tonfa_spark_baron_professional.zip"},

	})
)


--TASKMASTER:
    -- Spark baron leaders
    -- low health, high morale
    -- huge buffing effects for low renown teammates
    -- uses a gun
    -- probably watches a lot of ted talks


Content.AddCharacterDef
(
	CharacterDef("SPARK_BARON_TASKMASTER",
	{
		base_def = "SPARK_BARON_BASE",
		
		title = "Taskmaster",
		renown = 4,
		combat_strength = 3,
		death_item = "spark_cannon",
		tags = {"wealthy"},

		loved_graft = "torpor_fiend",
		hated_graft = "crisis",

		base_builds = {
	        [ GENDER.MALE ] = "male_spark_baron_taskmaster",
			[ GENDER.FEMALE ] = "female_spark_baron_taskmaster"
		},

		voice_actors = {
			[GENDER.MALE] = { "sparkbaronHighClassMale01" },
    		[GENDER.FEMALE] = { "sparkbaronHighClassFemale01" },
    	},
		
	    negotiation_ally_image = engine.asset.Texture( "negotiation/modifiers/recruit_spark_baron_taskmaster.tex"),

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,

			attacks = 
			{
				executive_order = table.extend(NPC_RANGED)
				{
					name = "Executive Order",
					anim = "exectuive_order",

					flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
					damage_mult = .67,

					features = 
					{
						EXPOSED = 1,
					},
				},

				master_blaster = table.extend(NPC_ATTACK)
				{
					name = "Master Blaster",
					anim = "master_blaster",
					flags = battle_defs.CARD_FLAGS.RANGED,
					hit_count = 3,
					damage_mult = .3,
				},

				master_blaster_call_in = table.extend(NPC_ATTACK)
				{
					name = "Master Blaster",
					anim = "master_blaster",
					flags = battle_defs.CARD_FLAGS.RANGED | battle_defs.CARD_FLAGS.SPECIAL,
					hit_count = 3,
					damage_mult = .4,
					OnPostResolve = function( self, battle, attack )
						self.owner:Depart()
					end
				},

				invigorate = table.extend(NPC_BUFF)
				{
					name = "Invigorate",
					anim = "taunt",

					flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
					target_type = TARGET_TYPE.SELF,
					target_mod = TARGET_MOD.TEAM,

					CanPlayCard = function(self, battle)
			            local has_sentients = false
			            for i,fighter in self.owner:GetTeam():Fighters() do
			            	if fighter:HasMorale() then has_sentients = true end
			            end
			            return has_sentients
			        end,

					OnPostResolve = function( self, battle, attack )
						for i, hit in attack:Hits() do
							local current_morale = hit.target:GetMorale()
							if current_morale then							
								local max_health = hit.target:GetMaxHealth()
								local min_percent = 2 / max_health

								local delta = -.1
								if current_morale + delta < min_percent then
									delta = current_morale - min_percent
								end
								if delta < 0 then 
									hit.target:DeltaMorale(delta)
								end
							end
							hit.target:AddCondition("POWER", 1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ))
						end
					end
				},
			},

			behaviour =
            {
                OnActivate = function( self )
                    self.attacks = self:MakePicker()
                        :AddID( "executive_order", 2 )
                        :AddID( "master_blaster", 4 )
                        :AddID( "invigorate", 1 )
                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    self.attacks:ChooseCard()
                end,
            },
		},

		negotiation_data = 
		{
			behaviour = table.extend( SPARK_NEGOTIATION )
			{
			}
		},

		anims = { "anim/weapon_blaster_spark_baron_taskmaster.zip"},
		combat_anims = { "anim/med_combat_blaster_spark_baron_taskmaster.zip" },
	})
)


--SECURITY GUARDS:
    -- Spark baron rent-a-cops
    -- use crowd-control attacks, clubs
    -- low morale, easily bribed.
    -- just doing it for the money

Content.AddCharacterDef
(
	CharacterDef("SPARK_BARON_GOON",
	{
		designer_note = "Whenever you play a card, discard a random card.",
		base_def = "SPARK_BARON_BASE",
		
		title = "Security Guard",
		renown = 2,
		combat_strength = 2,
		
		loved_graft = "weapons_supplier",
		hated_graft = "sketchy_equipment",
		death_item = "shock_core",
 	  	negotiation_ally_image = engine.asset.Texture( "negotiation/modifiers/recruit_spark_baron_goon.tex"),

		promotion_def = "SPARK_BARON_GOON_PROMOTED",

		base_builds = {
	        [ GENDER.MALE ] = "male_spark_baron_goon",
			[ GENDER.FEMALE ] = "female_spark_baron_goon"
		},

		voice_actors = {
			[GENDER.MALE] = { "sparkbaronHighClassMale01" },
    		[GENDER.FEMALE] = { "sparkbaronHighClassFemale01" },
    	},


		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM_LOW,

			cards = { "ai_spark_baron_billy_club", "ai_spark_baron_smack", "ai_spark_baron_goon_buff" },
            
			conditions = 
			{
			    static_shock =
			    {
			        name = "Static Shock",
					desc = "Whenever you play a card, discard a random card and reduce <b>Static Shock</b> by 1.",
					ctype = CTYPE.DEBUFF,

			        event_handlers =
			        {
		                [ BATTLE_EVENT.EXECUTES_ADDED ] = function( self )
		                    self.owner:RemoveCondition( self.id )
		                end,

                		[ BATTLE_EVENT.POST_RESOLVE ] = function(self, battle, card)
			                if self.owner == card.owner then
			                    local card = battle:GetHandDeck():PeekRandom()
			                    if card then
			                        battle:DiscardCard( card )
					                self.owner:RemoveCondition(self.id,1)
			                    end
			                end
			            end,
			        }
			    },

			    sparkled =
			    {
			    	name = "Sparkled",
			    	desc = "At the start of your turn, lose 1 action. Reduce <b>Sparkled</> by 1 at the end of your turn.",

			    	ctype = CTYPE.DEBUFF,

			    	event_handlers = 
			    	{
			    		[ BATTLE_EVENT.END_TURN ] = function( self, fighter )
			    			if fighter == self.owner then
			    				self.owner:RemoveCondition(self.id, 1, self)
			    			end
			    		end,

			    		[ BATTLE_EVENT.BEGIN_TURN ] = function ( self, fighter )
			    			if fighter == self.owner then
			    				self.battle:ModifyActionCount(-1)
			    			end
			    		end,
			    	},
			    },
			},

			attacks = 
			{
			    ai_spark_baron_billy_club = table.extend(NPC_ATTACK)
			    {
			        name = "Billy Club",
			        anim = "smack",

			        flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF,

			        damage_mult = .65,

			        OnPostResolve = function( self, battle, attack )
			        	for i, hit in attack:Hits() do
			        		if not hit.evaded and not hit.defended then
				                if hit.target:IsPlayer() then
				                	hit.target:AddCondition("static_shock", 2 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ))
				                elseif math.random() < 0.5 then
				                    hit.target:AddCondition( "STUN", 1 )
				                end
				            end
				        end
			        end
			    },

			    ai_spark_baron_billy_club_call_in = table.extend(NPC_ATTACK)
			    {
			        name = "Billy Club",
			        anim = "smack",

			        flags = CARD_FLAGS.MELEE | CARD_FLAGS.DEBUFF | battle_defs.CARD_FLAGS.SPECIAL,

			        OnPostResolve = function( self, battle, attack )
			        	for i, hit in attack:Hits() do
			        		if not hit.evaded then
				                hit.target:AddCondition( "IMPAIR", 1 )
				            end
				        end
				        self.owner:Depart()
			        end
			    },

			    ai_spark_baron_smack = table.extend(NPC_ATTACK)
			    {
			        name = "Smack",
			        anim = "attack1",

			        flags = CARD_FLAGS.MELEE,
			    },

			    ai_spark_baron_goon_buff = table.extend(NPC_BUFF)
			    {
			        name = "Power Up",
			        anim = "taunt",

			        flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
			        target_type = TARGET_TYPE.SELF,

			        features = 
			        {
			            POWER = 2,
			        },
			    },

			    spark_baron_goon_discharge = table.extend(NPC_ATTACK)
			    {
			        name = "Discharge",
			        anim = "laser",

			        flags = CARD_FLAGS.RANGED | CARD_FLAGS.DEBUFF,
			        target_mod = TARGET_MOD.TEAM,

			        sparkled_count = { 1, 2, 3 },

			        OnPostResolve = function( self, battle, attack )
			        	for i,hit in ipairs(attack.hits) do
			        		if hit.target:IsPlayer() and not hit.evaded then
			        			hit.target:AddCondition("sparkled", self.sparkled_count[math.min(#self.sparkled_count, self.owner:GetScale())], self)
			        		end
			        	end
			        end
			    },
			},

            behaviour =
            {
                OnActivate = function( self, fighter )
                    self.basic_attacks = self:MakePicker()
                        :AddID( "ai_spark_baron_billy_club", 1)
                        :AddID( "ai_spark_baron_smack", 2)
                        :AddID( "ai_defend_med", 1)
                        :AddID( "ai_spark_baron_goon_buff", 1 )
                        --:AddID( "ai_spark_baron_goon_buff", 1 )
                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self )
                    return self.basic_attacks:ChooseCard()
                end,
            },
		},

		negotiation_data = 
		{
			behaviour = table.extend( SPARK_NEGOTIATION )
			{
			}
		},

		anims = {"anim/weapon_club_spark_baron_goon.zip"},
		combat_anims = { "anim/med_combat_club_spark_baron_goon.zip" },
	})
)

Content.AddCharacterDef
(
	CharacterDef("SPARK_BARON_GOON_PROMOTED",
	{
		base_def = "SPARK_BARON_GOON",
		
		title = "Promoted Security Guard",
		renown = 2,
		combat_strength = 2,
		
		loved_graft = "weapons_supplier",
		hated_graft = "sketchy_equipment",
		death_item = "shock_core_plus",

		base_builds = {
	        [ GENDER.MALE ] = "male_spark_baron_promoted_build",
			[ GENDER.FEMALE ] = "female_spark_baron_promoted_build"
		},

		fight_data = 
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.LOW,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.MEDIUM,
            

			attacks = 
			{

			    spark_baron_goon_promoted_attack = table.extend(NPC_ATTACK)
			    {
			        name = "Smack",
			        anim = "attack1",

			        flags = CARD_FLAGS.MELEE,
			    },
			},

			cards = {"spark_baron_goon_promoted_attack", "spark_baron_goon_discharge"},

            behaviour =
            {
                OnActivate = function( self, fighter )
                    self.basic_attacks = self:MakePicker()
                        :AddID( "spark_baron_goon_promoted_attack", 2)
                        :AddID( "spark_baron_goon_discharge", 1)

                    self.taunts = self:MakePicker()
                        :AddID( "ai_defend_med", 2)
                        :AddID( "ai_spark_baron_goon_buff", 1 )

                    self:SetPattern( self.AttackCycle )
                end,

                AttackCycle = function( self )
                    self.basic_attacks:ChooseCard()
                    self:SetPattern( self.DefendCycle )
                end,

                DefendCycle = function( self )
                	self.taunts:ChooseCard()
                	self:SetPattern( self.AttackCycle )
                end
            },
		},

		anims = {"anim/weapon_club_promoted_spark_baron.zip"},
		combat_anims = { "anim/med_combat_club_promoted_spark_baron.zip" },
	})
)



Content.AddCharacterDef
(
	CharacterDef("SPARK_BARON_AUTOMECH",
	{
		base_def = "SPARK_BARON_BASE",

		title = "Enforcer",
		can_talk = true,
		renown = 1,
		combat_strength = 4,
		species = SPECIES.MECH,

		death_item = "automech_blaster",
	    negotiation_ally_image = engine.asset.Texture( "negotiation/modifiers/recruit_spark_baron_automech.tex"),

        aspects = {
            social_connections = false,
        },

		base_builds = {
			[ GENDER.UNDISCLOSED ] = "automech_spark_baron_01",
		},
		voice_actor = "robot1",

		anims = {"anim/weapon_blaster_automech_spark_baron.zip"},
		combat_anims = { "anim/med_combat_automech_sparkbaron.zip" },
		head = "head_automech_spark_baron_01",

		fight_data =
		{
			MAX_MORALE = MAX_MORALE_LOOKUP.IMMUNE,
			MAX_HEALTH = MAX_HEALTH_LOOKUP.LOW,
			shadow_symbol = "hips",

			fx =
			{
				flead_dust = { auto = true },
            },

			attacks =
			{
				enforcer_shot = table.extend(NPC_ATTACK)
				{
					name = "Spark Shot",
					anim = "shoot",
					pre_anim = "shoot_pre",
					post_anim = "shoot_pst",
					hit_tags = { "spark" },

					flags = battle_defs.CARD_FLAGS.RANGED,
					damage_mult = 1.5,
				},

				enforcer_shot_call_in = table.extend(NPC_ATTACK)
				{
					name = "Spark Shot",
					anim = "shoot",
					pre_anim = "shoot_pre",
					post_anim = "shoot_pst",
					hit_tags = { "spark" },

					flags = battle_defs.CARD_FLAGS.RANGED | battle_defs.CARD_FLAGS.SPECIAL,
					damage_mult = 1.2,
					features =
					{
						WOUND = 2,
					},
					OnPostResolve = function( self, battle, attack )
						self.owner:Depart()
					end
				},
				auto_cannon = table.extend(NPC_ATTACK)
				{
					name = "Auto-cannon",
					desc = "Usable during hover only.\nWhen {1} is hit, reduce the number of shots by 1.",
					desc_fn = function( self, fmt_str )
						return loc.format( fmt_str, self.owner:GetName() )
					end,

					anim = "blast",
			        pre_anim = "blast_pre",
			        post_anim = "blast_pst",

					flags = battle_defs.CARD_FLAGS.RANGED,

					CanPlayCard = function( self, battle, target )
						return self.hit_count == nil or self.hit_count > 0
					end,

					damage_mult = .8,

					generate_damage_fn = function( self )
						NPC_ATTACK.generate_damage_fn( self )
						self.hit_count = self.owner:GetConditionStacks("HOVER_PACK")
						self.min_damage = 1
					end,

					event_handlers =
					{
			            [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
							if attack.card:IsAttackCard() and hit.target == self.owner and not hit.evaded then
								--
								self.hit_count = self.hit_count - 1
								self.owner:RemoveCondition( "HOVER_PACK", 1, self )
								if self.hit_count == 0 then
									self.owner:AddCondition( "STUN" )
								end
							end
						end
					}
				},

				engage_hover =
				{
					name = "Engage Hover",
					flags = battle_defs.CARD_FLAGS.BUFF,
					target_type = TARGET_TYPE.SELF,

					OnPostResolve = function( self, battle, attack )
						self.owner:AddCondition( "HOVER_PACK", math.random( 3, 4 ) )
					end
				},

				self_repair =
				{
					name = "Self Repair",
					flags = battle_defs.CARD_FLAGS.BUFF | battle_defs.CARD_FLAGS.HEAL,
					target_type = TARGET_TYPE.SELF,

					OnPostResolve = function( self, battle, attack )
						local health = math.ceil( (self.owner:GetMaxHealth() - self.owner:GetHealth()) / 3 )
						health = health * (1 + GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH ))
						self.owner:HealHealth( health )
					end,

					features =
					{
						POWER = 2,
					}
				}
			},

			conditions =
			{
				HOVER_PACK =
				{
					name = "Hover Pack",
					desc = "When {1} is hit, reduce its number of attacks this turn by 1. If no attacks remain, {1} becomes {STUN|Stunned}.",
					desc_fn = function( self, fmt_str )
						return loc.format( fmt_str, self.owner:GetName() )
					end,

					hud_fx = { "automech_thrust" },

					-- max_stacks = 1,
                    anim_mapping =
                    {
                        idle = "idle",
                    },

                    OnApply = function( self, battle )
                        self.owner:BroadcastEvent( battle_defs.BATTLE_EVENT.FIGHTER_REMAP_ANIMS )
      --                   if battle:GetCurrentTeam() ~= self.owner:GetTeam() then
						-- 	self.owner:AddCondition( "EVASION", self.stacks )
						-- end
                    end,

                    OnUnapply = function( self, battle )
                    	self.owner:BroadcastEvent( battle_defs.BATTLE_EVENT.FIGHTER_REMAP_ANIMS, self.ground_anims )
                    end,

		            ground_anims =
		            {
		                hit_mid = "hit_mid_ground",
		                hit_mid_pst_idle = "hit_mid_ground_pst_idle",
		                hit_mid_pst_stunned = "hit_mid_ground_pst_stunned",

		                idle = "idle_ground",
		                step_back = "step_back_ground",
		                step_forward = "step_forward_ground",

		                stunned = "stunned_ground",
		                stunned_pre = "stunned_ground_pre",
		                stunned_pst = "stunned_ground_pst",
		            },

					-- event_handlers =
					-- {
					-- 	[ BATTLE_EVENT.END_TURN ] = function( self, fighter )
					-- 		if fighter == self.owner then
					-- 			self.owner:AddCondition( "EVASION", self.stacks )
					-- 		end
					-- 	end,
					-- }
				},
			},

			behaviour =
			{
				OnActivate = function( self )
					self.ground_moves = self:MakePicker()
						:AddID( "enforcer_shot", 1 )
						:AddID( "self_repair", 2 )
						:AddID( "engage_hover", 1 )

					self.auto_cannon = self:AddCard( "auto_cannon" )
					self.hover_moves = self:MakePicker():AddCard( self.auto_cannon, 1 )

					self:SetPattern( self.Cycle )
					self.fighter:AddCondition( "METALLIC", 1 )
					self.fighter:AddCondition( "HOVER_PACK", math.random(3,4))
				end,

				Cycle = function( self )
					if self.fighter:HasCondition( "HOVER_PACK" ) then
						self.hover_moves:ChooseCard( 1 )
					else
						self.ground_moves:ChooseCard( 1 )
					end
				end
			}

		},
		
		idle_anims = 
		{
			guarding = "idle_neutral_guarding_ranged"
		},

	})
)
