local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

Content.AddCharacterDef
(
    CharacterDef("NPC_SAL",
    {
        unique = true,
        base_def = "NPC_BASE",
        species = SPECIES.HUMAN,
        gender = GENDER.FEMALE,
        renown = 3,
        combat_strength = 4,
        name = "Sal",
        title = "Bounty Hunter",
        head = "head_sal_build",
        bio = "A bounty hunter with a chip on her shoulder and a glint in her eye, Sal escapes a life of servitude to claim the ultimate bounty—and get her revenge.",
            
        faction_id = "GRIFTER",

        loved_graft = "people_person",
        hated_graft = "wrong_side",

        death_item = "red_scarf",

        fight_data = 
        {
            MAX_HEALTH = 65,
            MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
            anim_mapping =
            {
                riposte = "targetpractice1",
                execute = "gash",
            },
            ranged_riposte = true,
            
            conditions = 
            { 
                npc_sal_nailed_glove = 
                {
                    name = "Nailed Glove",
                    desc = "Lose only 1 {COMBO} when receiving damage.",
                    event_handlers = 
                    {
                        [ BATTLE_EVENT.CALC_COMBO_LOSS ] = function( self, acc, source, target )
                            if target == self.owner then
                                if acc.value > 0 then
                                    acc:ModifyValue( 1, self )
                                end
                            end
                        end
                    },
                },
                npc_sal_combo_pattern = 
                {
                    name = "Brawler",
                    desc = "{1}'s attacks generate {COMBO}. When at or above 5 {COMBO}, {1} will use a strong finisher attack.",
                    desc_fn = function ( self, fmt_str )
                        return loc.format(fmt_str, loc.cap( self:GetOwnerName()))
                    end,
                },

                trained_fighter = 
                {
                    name = "Trained Fighter",
                    desc = "Lose only 1 {COMBO} when receiving damage.",
                    event_handlers = 
                    {
                        [ BATTLE_EVENT.CALC_COMBO_LOSS ] = function( self, acc, source, target )
                            if target == self.owner then
                                if acc.value > 0 then
                                    acc:ModifyValue( 1, self )
                                end
                            end
                        end
                    },
                },
            },

            attacks = 
            {
                -- COMBO ATTACKS
                npc_sal_gut_shot = table.extend(NPC_ATTACK)
                {
                    name = "Gut Shot",
                    anim = "elbow_strike",
                    hit_anim = true,

                    flags = CARD_FLAGS.MELEE,
                    base_damage = 3,

                    hit_count = 2,

                    features =
                    {
                        COMBO = 4,
                    },
                },

                npc_sal_shoulder_roll = table.extend(NPC_BUFF)
                {
                    name = "Shoulder Roll",
                    anim = "taunt",

                    CanPlayCard = function( self, battle )
                        return self.owner:GetConditionStacks("COMBO") > 0 
                    end,

                    flags = CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,

                    OnPostResolve = function( self, battle, attack )
                        local combo = self.owner:GetConditionStacks("COMBO")
                        self.owner:AddCondition("DEFEND", math.max(3, combo*3), self)
                        self.owner:AddCondition("COMBO", 3, self)
                    end
                },

                npc_sal_haymaker = table.extend(NPC_ATTACK)
                {
                    name = "Haymaker",
                    anim = "uppercut",

                    base_damage = 4,
                    flags = CARD_FLAGS.MELEE,
                    bonus_damage = 2,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                            if self == card then
                                local combo = self.combo_consumed or self.owner:GetConditionStacks( "COMBO" )
                                dmgt:AddDamage( self.bonus_damage * combo, self.bonus_damage * combo, self )
                            end
                        end,
                    },        

                    OnPreResolve = function( self, battle, attack )
                        local combo = self.owner:GetConditionStacks("COMBO")
                        if combo > 0 then 
                            self.combo_consumed = combo
                            self.owner:RemoveCondition("COMBO", combo, self)
                        end
                    end,

                    OnPostResolve = function( self, battle, attack)
                        self.combo_consumed = nil
                    end,
                },

                npc_sal_blade_fury =
                {
                    name = "Blade Fury",
                    anim = "bladefury",

                    base_damage = 3,

                    flags = CARD_FLAGS.MELEE,

                    OnPreResolve = function( self, battle, attack )
                        if self.hit_count > 1 then
                            attack.hit_frames = "hit_frames_combo"
                        end
                    end,

                    OnPostResolve = function( self, battle, attack )
                        self.owner:RemoveCondition("COMBO")
                    end,

                    generate_damage_fn = function( self )
                        NPC_ATTACK.generate_damage_fn( self )
                        self.hit_count = self.owner:GetConditionStacks("COMBO")
                    end,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.CONDITION_REMOVED ] = function( self, fighter, condition, stacks, source )
                            if self.owner == fighter and condition.id == "COMBO" then
                                self.hit_count = self.owner:GetConditionStacks("COMBO")
                            end
                        end,
                    }
                },

                npc_sal_feint_combo = 
                {
                    name = "Feint",
                    anim = "taunt",

                    target_type = TARGET_TYPE.SELF,
                    flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,

                    features =
                    {
                        DEFEND = 5,
                        COMBO = 2,
                    },
                },     
            },


            behaviour =
            {
                OnActivate = function( self, fighter )
                    self.attacks = self:MakePicker()
                        :AddID( "npc_sal_gut_shot", 2 )
                        :AddID( "npc_sal_shoulder_roll", 1 )
                        :AddID( "npc_sal_feint_combo", 1 )
                    self.finishers = self:MakePicker()
                        :AddID( "npc_sal_haymaker", 1)
                        :AddID( "npc_sal_blade_fury", 1)

                    self.fighter:AddCondition("npc_sal_nailed_glove")
                    self.fighter:AddCondition("npc_sal_combo_pattern")
                    self:SetPattern(self.ComboCycle)
                end,

                ComboCycle = function( self )
                    if self.fighter:GetConditionStacks("COMBO") >= 5 then
                        self.finishers:ChooseCard()
                    else
                        self.attacks:ChooseCard()
                    end
                end
            },
        },

        negotiation_data =
        {
            behaviour =
            {
                OnInit = function( self, difficulty )
                    self.negotiator:AddModifier( "GRIFTER" )
                    self:SetPattern( self.Cycle )
                end,

                Cycle = function( self, turns )
                    self:ChooseGrowingNumbers( 1, 1 )

                    if not self.negotiator:HasModifier( "NPC_INFLUENTIAL" ) then
                        self.negotiator:CreateModifier( "NPC_INFLUENTIAL", 1 )
                    elseif not self.negotiator:HasModifier( "NPC_DOMINANT" ) then
                        local min = math.ceil(turns/2)
                        local max = math.ceil(min * 1.5)
                        self.negotiator:CreateModifier( "NPC_DOMINANT", math.min(8, min + math.random(max-min)) )
                    end
                end,
            }
        },

        anims = { "anim/weapon_knife_guard_lumin.zip","anim/weapon_blaster_sal.zip"},
        build = "sal",
        combat_anims = { "anim/med_combat_sal.zip" },
        voice_actor = "sal",

        hair_colour = 0xCB5D3Cff,
        skin_colour = 0xd2a18cff,
    })
)

Content.AddCharacterDef
(
    CharacterDef("NPC_ROOK",
    {
        unique = true,
        base_def = "NPC_BASE",
        gender = GENDER.MALE,
        species = SPECIES.HUMAN,
        hair_colour = 0x382919ff,
        skin_colour = 0xA37E55ff,
        head = "head_rook",
        title = "Operative",
        build = "rook",
        renown = 4,
        combat_strength = 3,
        name = "Rook",
        faction_id = "GRIFTER",
        loved_graft = "grumpy_scowl",
        hated_graft = "contagious",
        death_item = "rooks_spare_coin",

        fight_data = 
        {
            --MAX_HEALTH = 65,
            MAX_HEALTH = 48,

            anim_mapping =
            {
                riposte = "whip",
            },

            ranged_riposte = false,

            MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
            
            conditions = 
            { 
                npc_rook_charges = 
                {
                    name = "Charged Pistols",
                    desc = "Increase attack damage by <#HILITE>{1}</>.",
                    
                    desc_fn = function ( self, fmt_str )
                        return loc.format(fmt_str, self.stacks)
                    end,

                    event_handlers =
                    {
                        [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                            if card.owner == self.owner then
                                dmgt:ModifyDamage( dmgt.min_damage + self.stacks, dmgt.max_damage + self.stacks )
                            end
                        end,
                    }
                },
        },

            attacks = 
            {
                -- COMBO ATTACKS
                npc_rook_charge_up = table.extend(NPC_BUFF)
                {
                    name = "Crank",
                    anim = "reload_01",
                    flags = CARD_FLAGS.BUFF,
                    target_type = TARGET_TYPE.SELF,
                    defend_amt = 8,
                    charge_amt = 1,
                    riposte_amt = 3,

                    OnPostResolve = function( self, battle, attack )
                        self.owner:AddCondition("npc_rook_charges", self.charge_amt, self)
                        self.owner:AddCondition("DEFEND", self.defend_amt, self)
                        self.owner:AddCondition("RIPOSTE", self.riposte_amt, self)
                    end
                },

                npc_rook_blast = table.extend(NPC_ATTACK)
                {
                    name = "Blast",
                    pre_anim = "blast_pre",
                    anim = "blast",
                    post_anim = "blast_pst",
                    flags = CARD_FLAGS.RANGED,
                    target_type = TARGET_TYPE.ENEMY,

                    min_damage = 2,
                    max_damage = 3,
                },


                npc_rook_fan_the_hammer = table.extend(NPC_ATTACK)
                {
                    name = "Fan The Hammer",
                    pre_anim = "hammer_pre",
                    anim = "hammer",
                    post_anim = "hammer_pst",
                    flags = CARD_FLAGS.RANGED,
                    target_type = TARGET_TYPE.ENEMY,
                    target_mod = TARGET_MOD.TEAM,

                    min_damage = 1,
                    max_damage = 3,
                    hit_count = 3,
                    riposte_amt = 5,

                    OnPostResolve = function( self, battle, attack )
                        self.owner:RemoveCondition("npc_rook_charges")
                        self.owner:AddCondition("RIPOSTE", self.riposte_amt, self)
                    end,
                },
            },


            behaviour =
            {
                OnActivate = function( self, fighter )
                    self.empty = self:MakePicker()
                        :AddID( "npc_rook_charge_up", 1)                        
                    
                    self.attacks = self:MakePicker()
                        :AddID( "npc_rook_charge_up", 1)                        
                        :AddID( "npc_rook_blast", 1 )
                    
                    self.full = self:MakePicker()
                        :AddID( "npc_rook_fan_the_hammer", 1 )

                    self.fighter:AddCondition("npc_rook_charges", 2)
                    self:SetPattern(self.ComboCycle)
                end,

                ComboCycle = function( self )
                    local charges = self.fighter:GetConditionStacks("npc_rook_charges")
                    
                    if charges == 0 then
                        self.empty:ChooseCard()
                    elseif charges >= 4 then
                        self.full:ChooseCard()
                    else
                        self.attacks:ChooseCard()
                    end
                end
            },
        },
        negotiation_data =
        {
            behaviour =
            {
                OnInit = function( self, difficulty )
                    --self.negotiator:AddModifier( "GRIFTER" )
                end,
            }
        },

        anims = { "anim/weapon_blaster_rook.zip"},
        combat_anims = { "anim/med_combat_rook.zip" },
        voice_actor = "rook",
    })
)

Content.AddCharacterDef
(
    CharacterDef("NPC_SMITH",
    {
        unique = true,
        base_def = "NPC_BASE",
        species = SPECIES.KRADESHI,
        gender = GENDER.MALE,
        renown = 2,
        combat_strength = 4,
        voice_actor = "smith",
        title = "Walking Disaster",
        name = "Smith",
        head = "head_smith",
        build = "smith",
        anims = { "anim/weapon_hammer_guard.zip"},
        combat_anims = { "anim/med_combat_hammer_admiralty_guard.zip"},
        bio = "Fearsome Kra'deshi",
        death_item = "smiths_canteen",

        faction_id = "GRIFTER",

        fight_data =
        {
            MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
            MAX_HEALTH = 65,

            status_widget_head_dx = 0.0,
            status_widget_head_dy = 4.6,

            attacks = 
            {
                smith_smash = table.extend(NPC_ATTACK)
                {
                    name = "Hammer Smash",
                    anim = "smash",
                    flags = CARD_FLAGS.MELEE,
                },
            },  

            cards = { "smith_smash", "ai_defend_lo" },

            behaviour =
            {
                OnActivate = function( self, fighter )
                    self.attack = self:AddCard( "hammer_smash" )
                    self.buffs = self:MakePicker()
                        :AddID( "ai_defend_lo", 1 )

                    fighter:AddCondition( "OPPORTUNISTIC", 3 )

                    if math.random() < 0.5 then
                        self:SetPattern( self.Attack )
                    else
                        self:SetPattern( self.Buff )
                    end
                end,

                Attack = function( self, fighter, battle )
                    self:ChooseCard( self.attack )
                    self:SetPattern( self.Buff )
                end,

                Buff = function( self, fighter, battle )
                    self.buffs:ChooseCard()
                    self:SetPattern( self.Attack )
                end
            },
        },
        negotiation_data =
        {
            behaviour =
            {
                OnInit = function( self, difficulty )
                    self.negotiator:AddModifier( "GRIFTER" )
                end,
            }
        },
    })
)