local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local def = CharacterDef("TEI",
{
    unique = true,

    base_def = "LUMINARI",
    name = "Tei",
    build = "female_tei_utaro_build",
    head = "head_female_tei_utaro",
    tags = {"plot_armour", "no_patron"},
    gender = GENDER.FEMALE,
    species = SPECIES.HUMAN,
    theme_music = "event:/music/story/smith/story_smith_tei",
    
    anims = { "anim/weapon_drone_tei_utaro.zip", },
    combat_anims = { "anim/med_combat_unarmed_tei_utaro.zip" },

    loved_graft = "puppeteer",
    hated_graft = "sway",
    death_item = "tei_drone_strike",
    -- negotiation_data = 
    -- {
    --     modifiers =
    --     {

    --     },
    --     behaviour = 
    --     {
           
    --     }
    -- },
    
    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 60,

        attacks = 
        {
            tei_shoot = table.extend(NPC_ATTACK)
            {
                name = "Single Fire",
                anim = "shoot",
                flags = CARD_FLAGS.RANGED,
            },

            tei_barrage = table.extend(NPC_ATTACK)
            {
                name = "Barrage",
                anim = "attack1",
                flags = CARD_FLAGS.RANGED,
                
                target_count = 4,
                damage_mult = 0.3,
            },

            tei_taunt = table.extend(NPC_BUFF)
            {
                name = "Zero In",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.DEBUFF,

                OnPostResolve = function( self, battle, attack )
                    self.owner:AddCondition("DEFEND", 6, self)
                    attack:AddCondition("WOUND", 2, self)
                end,
            },

            tei_launch = table.extend(NPC_BUFF)
            {
                name = "Launch Drone",
                anim = "launch",
                flags = CARD_FLAGS.SKILL,
                target_type = TARGET_TYPE.SELF,
            },
        
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                self.attacks = self:MakePicker()
                    :AddID("tei_shoot", 2)
                    :AddID("tei_barrage", 1)
                    :AddID("tei_taunt", 1)

                self.launch = self:AddCard("tei_launch")

                fighter.battle:PlayCard(self.launch)
                self:SetPattern(self.Cycle)
            end,


            Cycle = function( self )
                self.attacks:ChooseCard()
            end
        },
    },
})

Content.AddCharacterDef( def )