local battle_defs = require "battle/battle_defs" 

--------------------------------------------------------------------------------
-- The Grout.  Spawns Burrs & Spectators.

local GroutModule = class( "Battle.GroutModule" )
GroutModule.can_reload = true

GroutModule.event_handlers =
{
	[ battle_defs.EVENT.BEGIN_ENEMY_TURN ] = function( self )
		self:Germinate()
	end
}

local function GetFormation( team )
	return FIGHT_FORMATIONS[ 4 ]
end

function GroutModule:OnAddedToBattle( battle )
	self.battle = battle
	self.burrs_spawned = 0

	self.battle:GetTeam( TEAM.RED ):SetCustomFormation( GetFormation )

    if self.event_handlers then
        for event_name, fn in pairs( self.event_handlers ) do
            local priority = self.event_priorities and self.event_priorities[ event_name ]
            battle:ListenForEvent( event_name, self, self.OnEvent, priority )
        end
    end
end

function GroutModule:OnRemovedFromBattle( battle )
	self.battle = nil
    if self.event_handlers then
        battle:RemoveListener( self )
    end
end

function GroutModule:Germinate()
	local team = self.battle:GetTeam( TEAM.RED )
	if team:NumActiveFighters() < team:GetMaxFighters() then
		local agent = Agent( "BURR" )
		local fighter = Fighter.CreateFromAgent( agent )

		self.battle:GetTeam( TEAM.RED ):AddFighter( fighter )
		self.burrs_spawned = self.burrs_spawned + 1
	end
end

function GroutModule:OnEvent( eventname, ... )
    if self.event_handlers and self.event_handlers[ eventname ] then
        self.event_handlers[ eventname ]( self, ... )
    end
end
