local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local def = CharacterDef("TWIN_BOSS_1",
{
    unique = true,

    base_def = "NPC_BASE",
    alias = "TWIN_BOSS_1",
    title = "Freelance Bodyguard",
    species = SPECIES.HUMAN,
    gender = GENDER.MALE,
    renown = 3,
    combat_strength = 3,
    boss = true,
    -- title = "",
    -- name = "",
    -- faction_id = "",

    loved_graft = "wrist_rocket",
    hated_graft = "punchy",
    death_item = "overdriver",

    battle_preview_name_fn = function(agent) return agent.agent_def:GetLocalizedString("TWIN_BOSS_PREVIEW_NAMES") end,
    battle_preview_title_fn = function(agent) return agent.agent_def:GetLocalizedString("TWIN_BOSS_PREVIEW_TITLE") end,
    battle_preview_anim = "anim/boss_twins_slide.zip",
    battle_preview_offset = { x = 400, y = 15 },
    battle_preview_glow = { colour = 0xFF6600FF, bloom = 0.3, threshold = 0.02 },
    battle_preview_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_boss_twins",
    icon = engine.asset.Texture( "icons/b_twin_boss.tex" ),
    -- voice_actor = "",

    loc_strings =
    {
        TWIN_BOSS_PREVIEW_NAMES = "Felek and Tazz",
        TWIN_BOSS_PREVIEW_TITLE = "Freelance Bodyguards",
    },

    anims = {"anim/weapon_knuckles_grifter01.zip"},
    combat_anims = {"anim/med_combat_unarmed_grifter01.zip"},
    build = "male_grifter_02_build",
    head = "head_male_grifter_02",

    negotiation_data = Content.GetCharacterDef( "JAKES_BASE" ).negotiation_data,

    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 70,
        death_fade_delay = 1,

        status_widget_head_dx = -0.2,
        status_widget_head_dy = 5.3,

        conditions = 
        { 
            HUBRIS = 
            {
                name = "Hubris",
                desc = "{1}'s next attack deals <#HILITE>{2}</> bonus damage. Whenever you hit {1}, <b>Hubris</> is reduced by 2.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self:GetOwnerName(), self.stacks )
                end,

                event_priorities =
                {
                    [ BATTLE_EVENT.CALC_DAMAGE ] = EVENT_PRIORITY_ADDITIVE,
                },

                event_handlers = 
                {
                    [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                        if card.owner == self.owner then
                            dmgt:AddDamage(self.stacks, self.stacks, self)
                        end
                    end,

                    [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                        if hit.target == self.owner and not hit:CheckHitResult( "evaded" ) and hit.card:IsDamageCard() then
                            self.owner:RemoveCondition(self.id, 2, self)
                        end
                    end,

                    [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                        if attack.attacker == self.owner and attack.card:IsAttackCard() then
                            self.owner:RemoveCondition(self.id)
                        end
                    end
                },
            },

            RETRIBUTION = 
            {
                name = "Retribution",
                desc = "If {1}'s brother is killed or panics, {1} gains {2} {POWER}.",
                desc_fn = function( self, fmt_str )
                    return loc.format(fmt_str, self:GetOwnerName(), self.power_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.power_amt)])
                end,

                power_amt = { 4, 4, 5, 6 },

                event_handlers = 
                {
                    [ BATTLE_EVENT.STATUS_CHANGED ] = function( self, fighter, status )
                        if not self.used and fighter ~= self.owner and (status == FIGHT_STATUS.DEAD or status == FIGHT_STATUS.SURRENDER ) and self.owner:IsActive() then
                            local id = fighter:GetAgent():GetContentID()
                            if id == "TWIN_BOSS_2" or id == "TWIN_BOSS_1" then
                                self.owner:AddCondition("POWER", self.power_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.power_amt)], self)
                                self.owner:PlayAnim( "taunt", 0, true )
                                self.used = true
                            end
                        end
                    end,
                },
            },
        },

        attacks = 
        {
            twin_1_shockwave = table.extend(NPC_ATTACK)
            {
                name = "Shockwave",
                anim = "shockwave",

                flags = CARD_FLAGS.RANGED,
                combo_starter = true,
                synchronous = true,
                damage_mult = 0.5,

                CanPlayCard = function( self, battle, target )
                    return target == nil or target:IsPlayer()
                end,

                OnPostResolve = function( self, battle, attack )
                end,
            },
            twin_1_punch = table.extend(NPC_ATTACK)
            {
                name = "Punch",
                anim = "punch",

                flags = CARD_FLAGS.MELEE,

                OnPostResolve = function( self, battle, attack )
                    
                end,
                target_count = 2,
                damage_mult = 0.5,
            },


            twin_1_taunt =
            {
                name = "Taunt",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                defend_amt = { 6, 6, 10, 12},
                hubris_amt = { 3, 5, 8, 10 },

                OnPostResolve = function( self, battle, attack )
                    attack:AddCondition("DEFEND", self.defend_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.defend_amt)], self)
                    attack:AddCondition("HUBRIS", self.hubris_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.hubris_amt)], self)
                end,
            },
        },

        behaviour =
        {
            OnActivate = function( self )
                self.punch = self:AddCard("twin_1_punch")
                self.taunt = self:AddCard("twin_1_taunt")
                self.special_attack = self:AddCard("twin_1_shockwave")

                self:SetPattern( self.Cycle )
                self.fighter:AddCondition("RETRIBUTION")

                --add the other twin
                local agent = TheGame:GetGameState():AddSkinnedAgent( "TWIN_BOSS_2" )
                agent:GetSocialConnections():SetRelationship( RELATIONSHIP.LOVED, self.fighter:GetAgent() )

                local new_fighter = Fighter.CreateFromAgent( agent, self.fighter:GetScale() )
                self.fighter:GetTeam():AddFighter( new_fighter )
                self.fighter:GetTeam():ActivateNewFighters( self.fighter.battle )
                self.fighter.battle:BroadcastEvent( BATTLE_EVENT.UPDATE_HOME_POSITIONS, self.fighter:GetTeam())

            end,

            Cycle = function( self, turns )
                if turns % 3 == 0 then
                    if not self:ChooseCard(self.special_attack) then
                        self:ChooseCard(self.punch)
                    end
                elseif (turns + 1) % 3 == 0 then
                    self:ChooseCard(self.taunt)
                else
                    self:ChooseCard(self.punch)
                end
            end,
        },
    },
})

Content.AddCharacterDef( def )

local def2 = CharacterDef("TWIN_BOSS_2",
{
    unique = true,

    base_def = "NPC_BASE",
    alias = "TWIN_BOSS_2",
    title = "Freelance Bodyguard",
    species = SPECIES.HUMAN,
    gender = GENDER.MALE,
    renown = 3,
    combat_strength = 3,
    boss = true,
    -- title = "",
    -- name = "",
    -- faction_id = "",

    loved_graft = "hard_headed",
    hated_graft = "butt_kicked",
    death_item = "underdriver",

    -- battle_preview_anim = "anim/.zip",
    -- battle_preview_offset = { x = 185, y = 10 },
    -- battle_preview_glow = { colour = 0x10ffffFF, bloom = 0.3, threshold = 0.02 },
    -- battle_preview_audio = "event:/ui/prebattle_overlay/",
    -- icon = engine.asset.Texture( "icons/.tex"),
    -- voice_actor = "",

    anims = {"anim/weapon_armoured_legs_grifter02.zip"},
    combat_anims = {"anim/med_combat_unarmed_grifter02.zip"},
    build = "male_grifter_03_build",
    head = "head_male_grifter_02",

    negotiation_data = Content.GetCharacterDef( "JAKES_BASE" ).negotiation_data,

    fight_data = 
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.HIGH,
        MAX_HEALTH = 70,
        death_fade_delay = 1,

        status_widget_head_dx = -0.2,
        status_widget_head_dy = 5,

        conditions = 
        { 

        },

        attacks = 
        {
            twin_2_axe_kick = table.extend(NPC_ATTACK)
            {
                name = "Axe Kick",
                anim = "axe_kick",

                flags = CARD_FLAGS.MELEE,

                CanPlayCard = function( self, battle, target )
                    return target == nil or target:IsPlayer()
                end,

                combo_followup = true,
                anim_delay = 0.85,
                damage_mult = 0.5,

                OnPostResolve = function( self, battle, attack )
                end,
            },
            twin_2_double_kick = table.extend(NPC_ATTACK)
            {
                name = "Double Kick",
                anim = "double_kick",

                flags = CARD_FLAGS.MELEE,

                OnPostResolve = function( self, battle, attack )
                    
                end,
                hit_count = 2,
                damage_mult = 0.5,
            },


            twin_2_taunt =
            {
                name = "Taunt",
                anim = "taunt",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                defend_amt = { 6, 6, 10, 12},
                hubris_amt = { 3, 5, 8, 10 },

                OnPostResolve = function( self, battle, attack )
                    attack:AddCondition("DEFEND", self.defend_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.defend_amt)], self)
                    attack:AddCondition("HUBRIS", self.hubris_amt[math.min(GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #self.hubris_amt)], self)
                end,
            },
        },

        behaviour =
        {
            OnActivate = function( self )
                self.double = self:AddCard("twin_2_double_kick")
                self.taunt = self:AddCard("twin_2_taunt")
                self.special_attack = self:AddCard("twin_2_axe_kick")

                self:SetPattern( self.Cycle )
                self.fighter:AddCondition("RETRIBUTION")
            end,

            Cycle = function( self, turns )
                if turns % 3 == 0 then
                    if not self:ChooseCard(self.special_attack) then
                        self:ChooseCard(self.double)
                    end
                elseif (turns + 1) % 3 == 0 then
                    self:ChooseCard(self.taunt)
                else
                    self:ChooseCard(self.double)
                end
            end,
        },
    },
})

Content.AddCharacterDef( def2 )