local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS
local negotiation_defs = require "negotiation/negotiation_defs"
local NEGOTIATION_EVENT = negotiation_defs.EVENT
local EVENT = negotiation_defs.EVENT

local ALL_VIX_RESENTMENTS = {"vix_resentment_1", "vix_resentment_2", "vix_resentment_3", "vix_resentment_4"}

Content.AddCharacterDef(CharacterDef("VIXMALLI",
{
    base_def = "PRIEST",
    species = SPECIES.KRADESHI,
    build = "male_vixmali_build",
    head = "head_male_vixmali",
    gender = GENDER.MALE,
    boss = true,
    name = "Vixmalli",
    title = "Cardinal",
    tags = {"plot_armour", "no_patron"},
    faction_id = "CULT_OF_HESH",
    voice_actor = "vixmalli",
    battle_preview_anim = "anim/boss_vix_slide.zip",
    battle_preview_offset = { x = 200, y = -10 },
    battle_preview_glow = { colour = 0x33FFFFFF, bloom = 0.12, threshold = 0.02 },
    icon = engine.asset.Texture( "icons/b_vixmalli.tex" ),
    
    theme_music = "event:/music/story/smith/story_smith_vixmalli",
    combat_anims = {"anim/med_combat_blaster_vix.zip"},
    anims = {"anim/weapon_blaster_vix.zip"},

    renown = 5,
    combat_strength = 3,
    loved_graft = "",

    loc_strings =
    {
        SHIELD_DESC = "Shielded by Vix's Core argument this turn.",
    },

    negotiation_data = 
    {
        modifiers = {
            VIXMALLI_CORE = 
            {
                name = "Vixmalli's Bag of Resentments",
                desc = "At the beginning of every turn, Vix pulls out another old axe to grind.\n\nOne resentment gains {SHIELDED} at the start of each turn.\n\nIf below 33% resolve, gains {IMPATIENCE} at the start of each turn.",
                
                desc_fn = function(self, fmt_str )
                    return loc.format(fmt_str, self.negotiator.agent:LocTable(), self.impatient_thresh)
                end,

                modifier_type = MODIFIER_TYPE.CORE,
                impatient_thresh = .33333,

                OnInit = function( self )
                    self.resentments = {}
                end,


                event_handlers =
                {
                    [ NEGOTIATION_EVENT.MODIFIER_ADDED ] = function ( self, modifier, source )
                        if modifier.negotiator == self.negotiator and modifier == self then
                            local t = shallowcopy(ALL_VIX_RESENTMENTS)
                            table.shuffle(t)
                            local START_SPAWN = 2
                            for k = 1, START_SPAWN do
                                self.negotiator:AddModifier(t[k], 1, self)
                            end
                        end
                    end,

                    [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                        
                        if target == self and damage > 0 then
                            local res, max_res = self:GetResolve()
                            if res / max_res <= self.impatient_thresh then
                                if not self.negotiator:HasModifier("IMPATIENCE") then
                                    self.negotiator:AddModifier("IMPATIENCE")
                                end
                            end
                        end
                    end,


                },
            },

            vix_resentment_1 = {
                name = "Under Your Skin",
                modifier_type = MODIFIER_TYPE.ARGUMENT,
                
                stack_growth = 1,
                stack_thresh = 3,
                num_to_spawn = 2,

                min_resolve = 8,
                max_resolve = 10,


                OnInit = function( self )
                    local THRESH = {6, 4, 3, 3}
                    self.stack_thresh = THRESH[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )]

                    local NUM = {1, 2, 3, 3}
                    self.num_to_spawn = NUM[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )]
                end,

                
                desc = "Gain {1} {1*stack|stacks} whenever you play a Manipulate card.\n\nIf this argument reaches {2} stacks: Incept {3} {vix_self_doubt} and destroy this argument.",
                desc_fn = function( self, fmt_str, engine, widget )
                    return loc.format( fmt_str, self.stack_growth, self.stack_thresh, self.num_to_spawn )
                end,
                
                event_handlers =
                {
                    [ EVENT.POST_RESOLVE ] = function(self, minigame, card)
                        if self.active and card.owner ~= self.owner and card:IsFlagged( negotiation_defs.CARD_FLAGS.MANIPULATE ) then
                            self.negotiator:DeltaModifier( self, 1, self )
                        end
                    end,

                    [ EVENT.MODIFIER_CHANGED ] = function( self, modifier, delta, clone, source )
                        if modifier == self and delta > 0 and self.stacks >= self.stack_thresh then
                            for k = 1, self.num_to_spawn do
                                self.anti_negotiator:InceptModifier( "vix_self_doubt", math.random(2, 4), self, true )
                            end 
                            self.negotiator:DestroyModifier( self, self )
                        end
                    end,

                    [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                        if negotiator == self.anti_negotiator and not self.active then
                            self.active = true
                        end
                    end
                },
            },

            vix_self_doubt = 
            {
                name = "Self-Doubt",
                desc = "Must be targeted before anything else.\n\nApply {1} damage to core argument when destroyed.\n\nRemove {2} {2*stack|stacks} at the start of every turn.",
                
                min_resolve = 1,
                max_resolve = 3,
                reduction = 1,
                self_damage = 5,

                OnInit = function( self )
                    local DMG = {3, 5, 7, 8}
                    self.self_damage = DMG[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )]
                    self.active = self.engine:GetCurrentNegotiator() == self.negotiator
                    self.force_target = self.engine:GetCurrentNegotiator() == self.negotiator
                end,

                OnBeginTurn = function( self, minigame )
                    self.negotiator:DeltaModifier( self, -self.reduction, self )
                    self:NotifyTriggered()
                end,

                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self.self_damage, self.reduction)
                end,

                OnBounty = function( self )
                    self.negotiator:AttackResolve( self.self_damage, self )
                end,

                CanPlayCard = function( self, source, engine, target )
                    -- Only verify forced targets if you are not targetting yourself
                     if self.active and source:IsAttack() and target:GetNegotiator() == self.negotiator then
                        if source.modifier_type == MODIFIER_TYPE.INCEPTION or source:GetNegotiator() ~= self.negotiator then
                            if not target.force_target then
                                return false, loc.format( LOC "CONVO_COMMON.MUST_TARGET", self:GetName() )
                            end
                        end
                    end
                    return true
                end,

                event_handlers =
                {
                    [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                        if negotiator == self.negotiator and not self.active then
                            self.active = true
                            self.force_target = true
                        end
                    end
                },

            }, 

            vix_resentment_2 = {
                name = "Self-Reinforcing Grievance",
                modifier_type = MODIFIER_TYPE.ARGUMENT,
                stack_growth = 1,
                stack_thresh = 3,
        
                min_resolve = 8,
                max_resolve = 10,

                OnInit = function( self )
                    local THRESH = {6, 4, 3, 3}
                    self.stack_thresh = THRESH[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )]
                end,

                desc = "Gains {1} {1*stack|stacks} whenever {2}'s core argument takes damage.\n\nIf this argument reaches {3} stacks: Double all of {4}'s other Resentments and destroy this argument.",

                desc_fn = function( self, fmt_str, engine, widget )
                    return loc.format( fmt_str, self.stack_growth, self:GetOpponentName(),  self.stack_thresh, self:GetOwnerName())
                end,
                
                event_handlers =
                {
                    [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                        
                        if self.active and target == self.anti_negotiator:FindCoreArgument() and damage > defended then
                            self.negotiator:DeltaModifier( self, 1, self )
                        end
                    end,

                    [ EVENT.MODIFIER_CHANGED ] = function( self, modifier, delta, clone, source )
                        if modifier == self and delta > 0 and self.stacks >= self.stack_thresh then

                        self:NotifyTriggered()
                        for k,v in ipairs(ALL_VIX_RESENTMENTS) do
                            local mod = self.negotiator:FindModifier(v)
                            if mod and mod ~= self then
                                self.negotiator:AddModifier(v, mod.stacks, self)
                            end
                        end
                         self.negotiator:DestroyModifier( self, self )
                        end
                    end,

                    [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                        if negotiator == self.anti_negotiator and not self.active then
                            self.active = true
                        end
                    end
                },
            },

            vix_resentment_3 = {
                name = "Bottled Rage",
                modifier_type = MODIFIER_TYPE.ARGUMENT,
                damage_growth = 1,

                min_resolve = 8,
                max_resolve = 10,
                
                target_enemy = TARGET_ANY_RESOLVE,
                min_persuasion = 1,
                max_persuasion = 2,

                OnInit = function( self )
                    local DMG = {.5, 1, 2, 2.5}
                    self.damage_growth = DMG[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )]
                end,

                desc = "Gains a stack whenever {2}'s core argument takes damage from a card.\n\nHalves stacks at the start of each turn.",

                desc_fn = function( self, fmt_str, engine, widget )
                    return loc.format( fmt_str, self.stack_growth, self:GetOwnerName())
                end,

                OnBeginTurn = function( self, minigame )
                    self:ApplyPersuasion()
                    if self.stacks > 1 then
                        self.negotiator:DeltaModifier( self, -math.round(self.stacks/2), self )
                    end
                end,

                event_handlers =
                {
                    [ EVENT.MODIFIER_CHANGED ] = function( self, modifier, delta, clone, source )
                        if modifier == self then
                            self.min_persuasion = 1
                            self.max_persuasion = math.round( (self.stacks)*self.damage_growth)
                            self.engine:BroadcastEvent( EVENT.CUSTOM, function( panel )
                                panel.opponent_modifiers:UpdatePersuasionLabels()
                            end )
                            self:NotifyTriggered()
                        end
                    end,

                    [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                        if target == self.negotiator:FindCoreArgument() and damage > defended and is_instance( source, Negotiation.Card ) then
                            self.min_persuasion = 1
                            self.max_persuasion = (self.stacks+1)*2
                            self.negotiator:DeltaModifier( self, 1, self )
                        end
                    end,
                },
            },                   

            vix_resentment_4 = {
                name = "Prickly",
                modifier_type = MODIFIER_TYPE.ARGUMENT,
                
                stack_growth = 1,
                composure_amt = 2,

                min_resolve = 8,
                max_resolve = 10,

                OnInit = function( self )
                    local CMP = {1, 2, 2.5, 3}
                    local idx = GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )
                    self.composure_amt = CMP[ clamp( idx, 1, #CMP ) ]
                end,


                desc = "Gains {1} {1*stack|stacks} whenever one of {2}'s arguments is destroyed.\n\nGives a random friendly argument {3} {COMPOSURE} at the start of every turn.",

                desc_fn = function( self, fmt_str, engine, widget )
                    return loc.format( fmt_str, self.stack_growth, self:GetOwnerName(), math.round(self.composure_amt*self.stacks))
                end,

                event_handlers =
                {

                    [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                        local targets = self.engine:CollectAlliedTargets(self.negotiator)
                        if targets and #targets > 0 then
                            targets[math.random(#targets)]:DeltaComposure(math.round(self.composure_amt*self.stacks), self)
                        end
                    end,

                    [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                        if source and modifier.negotiator == self.negotiator then
                            self.negotiator:DeltaModifier( self, 1, self )
                        end
                    end,
                },
            },                                 
        },


        behaviour = {
            OnInit = function( self, difficulty )
                self.negotiator:AddModifier( "VIXMALLI_CORE" )
                self:SetPattern( self.Cycle )
                   
                self.resentments = {} 
                for k,v in ipairs(ALL_VIX_RESENTMENTS) do
                    self.resentments[v] = self:AddArgument( v )
                end

            end,

            Cycle = function( self, turn )

                --replenish destroyed resentments
                
                local MAX_NUM_TO_SPAWN = 1
                if turn >= 6 then
                    MAX_NUM_TO_SPAWN = 3
                elseif turn >= 3 then
                    MAX_NUM_TO_SPAWN = 2
                end

                
                local missing_resentments = {}
                local possible_shield_targets = {}
                local previous_shield_target
                for k,v in pairs(self.resentments) do
                    local mod = self.negotiator:FindModifier(k)
                    if mod then
                        if mod:GetShieldStatus() then
                            previous_shield_target = mod
                        end
                        table.insert(possible_shield_targets, mod)
                    else
                        table.insert(missing_resentments, v)
                    end
                end
                if #missing_resentments > 0 then
                    table.shuffle(missing_resentments)
                    for k = 1, math.min(#missing_resentments, MAX_NUM_TO_SPAWN) do
                        self:ChooseCard(missing_resentments[k])
                    end
                end

                --once impatience starts, it really starts...
                if self.negotiator:HasModifier("IMPATIENCE") then
                    local impatience_card = self:AddCard("impatience")
                    impatience_card.max_delay = 1
                    self:ChooseCard(impatience_card)
                end
                
                for k,v in ipairs(possible_shield_targets) do
                    v:SetShieldStatus(false)
                end
                
                --move the shield around, if possibles
                if previous_shield_target and #possible_shield_targets > 1 then
                    table.arrayremove(possible_shield_targets, previous_shield_target)
                end

                if #possible_shield_targets > 0 then
                    possible_shield_targets[math.random(#possible_shield_targets)]:SetShieldStatus(true, LOC"CHARACTER.VIXMALLI.SHIELD_DESC")
                end

                --if there are self-dobuts in play, go broad
                if self.engine:GetPlayerNegotiator():HasModifier("vix_self_doubt") then
                    self:ChooseGrowingNumbers( 2, 0, 1, 1)
                else
                    self:ChooseGrowingNumbers( 1, 0, 1)
                end
            end,

        }
    },

    fight_data =
    {
        MAX_MORALE = MAX_MORALE_LOOKUP.MEDIUM,
        MAX_HEALTH = 110,

        conditions =
        {
            VIX_IMMORTAL = 
            {
                hidden = true,

                max_turns = 5,

                event_handlers = 
                {
                    -- [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                    --     if fighter == self.owner and self.battle:GetTurns() >= self.max_turns and self.owner:GetHealthPercent() < 1 then
                    --         self.active = true
                    --     end
                    -- end,

                    -- [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    --     if attack:IsTarget( self.owner ) and attack.card:IsAttackCard() and self.active then
                    --         self.owner:Surrender()
                    --     end
                    -- end,
                    
                    [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
                        -- This only exists as a fallback, because of eg. Booster Clamp
                        if self.owner:GetHealth() == 1 and self.battle:GetBattleResult() == nil then
                            self.battle:Win()
                        end
                    end,

                    [BATTLE_EVENT.STATUS_CHANGED] = function( self, fighter, status )
                        if status == FIGHT_STATUS.SURRENDER and fighter == self.owner then
                            self.battle:Win()
                        end
                    end,
                },
            },

            COAGULANT_ROUNDS =
            {
                name = "Coagulant Rounds",
                desc = "Whenever {1}'s attacks cause health loss, the target permanently loses {2} maximum health.",
                desc_fn = function( self, fmt_str )
                    return loc.format( fmt_str, self:GetOwnerName(), self.health_loss or 2)
                end,

                health_amts = { 1, 2, 2, 3 },

                OnApply = function( self, battle )
                    self.health_loss = self.health_amts[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1]
                end,

                event_handlers =
                {
                    [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
                        if self.battle:GetFighterFromSource( source ) == self.owner then
                            fighter:DeltaMaxHealth(-self.health_loss)
                            fighter:GetAgent().health:AddStatModifier( "COAGULANT_ROUNDS", -self.health_loss )
                        end
                    end,
                },
            },
        },

        attacks =
        {
            vix_shoot = table.extend(NPC_ATTACK)
            {
                name = "Shoot",
                anim = "shoot",

                flags = CARD_FLAGS.RANGED,
                damage_mult = 0.8,
            },

            vix_power_shot = table.extend(NPC_ATTACK)
            {
                name = "Power Shot",
                anim = "power_shot",

                flags = CARD_FLAGS.RANGED | CARD_FLAGS.PIERCING,
                damage_mult = 1.6,
            },

            reg_vix_taunt = 
            {
                name = "Taunt",
                anim = "taun",

                flags = CARD_FLAGS.SKILL | CARD_FLAGS.BUFF,
                target_type = TARGET_TYPE.SELF,

                defend_amt = { 7, 8, 9, 11},

                OnPostResolve = function( self, battle, attack )
                    attack:AddCondition("DEFEND", self.defend_amt[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1], self)
                end
            },
        },

        behaviour =
        {
            OnActivate = function( self, fighter )
                if CheckBits(self.battle:GetFlags(), BATTLE_FLAGS.BOSS_FIGHT) then
                    self.fighter.stat_bounds[ COMBAT_STAT.HEALTH ].min = 1 -- cannot be killed
                    self.fighter:AddCondition("VIX_IMMORTAL")
                    self.fighter:AddCondition("COAGULANT_ROUNDS")
                end
                self.attacks = self:MakePicker()
                    :AddID( "vix_shoot", 2 )
                    :AddID( "vix_power_shot", 1 )
                    :AddID( "reg_vix_taunt", 1 )
                self:SetPattern(self.Cycle)
            end,

            Cycle = function( self )
                self.attacks:ChooseCard()
            end,
        },
    }
}))

