local paperdoll = require "paperdoll_util"


---------------------------Machinery for the generated combat parties


local function TeamExists(found_teams, team)
    for k,v in ipairs(found_teams) do
        if shallowcompare(team, v) then
            return true
        end
    end

    return false
end

local function CheckTeam(team, count_min, count_max, pt_min, pt_max)
    
    if #team > count_max or #team < count_min then
        return
    end
    local pts = 0
    for k,v in ipairs(team) do
        local def = Content.GetCharacterDef(v)
        pts = pts + def.combat_strength

    end
    
    if pts < pt_min or pts > pt_max then
        return false
    end

    return true
end


function FindValidTeams(defs, count_min, count_max, pt_min, pt_max, found_teams, filter, diff, team, is_backup)
    
    team = team or {}
    table.sort(team)
    if #team > 0 and CheckTeam(team, count_min, count_max, pt_min, pt_max) and not TeamExists(found_teams, team) and (filter == nil or filter(team, diff, is_backup)) then
        table.insert(found_teams, team)
    end

    if #team + 1 <= count_max then
        for k,def in ipairs(defs) do
            local new_team = shallowcopy(team)
            table.insert(new_team, def)
            FindValidTeams(defs, count_min, count_max, pt_min, pt_max, found_teams, filter, diff, new_team, is_backup)
        end
    end
end

local function IsDefSentient( def )
    return def.species == nil
end



function PickTeams(defs, filter, difficulty, leader)
    
    difficulty = math.max(1, difficulty)
    
    local pt_min, pt_max = table.unpack( PT_RANGES[math.min(#PT_RANGES, difficulty)])
    local count_min, count_max  = table.unpack ( PT_COUNT_RANGES[math.min(#PT_COUNT_RANGES, difficulty)] )

    if leader then
        pt_min = math.max(0, pt_min - leader:GetCombatStrength())
        pt_max = math.max(0, pt_max - leader:GetCombatStrength())
        count_min = math.max(0, count_min -1)
        count_max = math.max(0, count_max -1)
    end

    local found_teams = {}
    

    FindValidTeams(defs, count_min, count_max, pt_min, pt_max, found_teams, filter, difficulty, nil, leader ~= nil)
    for k,v in pairs(found_teams) do
        table.sort(v, function(a,b) 
            if IsDefSentient(Content.GetCharacterDef(a)) == IsDefSentient(Content.GetCharacterDef(b)) then
                return Content.GetCharacterDef(a).renown > Content.GetCharacterDef(b).renown 
            else
                return IsDefSentient(Content.GetCharacterDef(a))
            end
        end )
    end    
    return found_teams
end


-------------------------------------------

--rules for various factional parties

local admiralty_defs = {"ADMIRALTY_GOON", "ADMIRALTY_GUARD", "VROC", "ADMIRALTY_PATROL_LEADER" , "ADMIRALTY_CLERK", "ADMIRALTY_INVESTIGATOR"}

local admiralty_filter = function(team, diff, is_backup)
    local counts = {}
    for k,v in pairs(team) do
        counts[v] = counts[v] and counts[v] + 1 or 1
    end

    if (counts["ADMIRALTY_PATROL_LEADER"] or 0) > 1 then
        return false
    end

    if (counts["ADMIRALTY_CLERK"] or 0) > 1 then
        return false
    end

    if (counts["ADMIRALTY_CLERK"] or 0) > 0 and diff > 3 then
        return false
    end

    if (counts["ADMIRALTY_CLERK"] or 0) + (counts["VROC"] or 0) == #team then
        return false
    end

    if (counts["ADMIRALTY_INVESTIGATOR"] or 0) > 1 then
        return false
    end

    if diff == 2 then
        if #team > 2 and (counts["VROC"] or 0) < 2 then
            return false
        end
    end

    if diff == 1 then
        if #team > 1 and (counts["VROC"] or 0) < 1 then
            return false
        end
    end


    for k,v in pairs(counts) do
        if v > 2 then
            return false
        end
    end

    return true
end


local bandit_defs = {"BANDIT_GOON", "BANDIT_GOON2", "BANDIT_RAIDER", "BANDIT_CAPTAIN", "FLEAD"}

local bandit_filter = function(team, diff)
    local counts = {}
    for k,v in pairs(team) do
        counts[v] = counts[v] and counts[v] + 1 or 1
    end

    if (counts["BANDIT_CAPTAIN"] or 0) > 1 then
        return false
    end

    if (counts["BANDIT_RAIDER"] or 0) > 0 and diff < 3 then
        return false
    end

    if (counts["BANDIT_CAPTAIN"] or 0) > 0 and diff < 4 then
        return false
    end

    if (counts["FLEAD"] or 0) == #team then
        return false
    end

    if diff == 2 then
        if #team > 2 and (counts["FLEAD"] or 0) < 2 then
            return false
        end
    end

    if diff == 1 then
        if #team > 1 and (counts["FLEAD"] or 0) < 1 then
            return false
        end
    end

    for k,v in pairs(counts) do
        if v > 2 then
            return false
        end
    end

    return true
end



local jakes_defs = {"JAKES_SMUGGLER", "JAKES_RUNNER", "JAKES_LIFTER", "COMBAT_DRONE"}

local jakes_filter = function(team, diff)
    local counts = {}
    for k,v in pairs(team) do
        counts[v] = counts[v] and counts[v] + 1 or 1
    end

    if (counts["COMBAT_DRONE"] or 0) == #team then
        return false
    end

    for k,v in pairs(counts) do
        if v > 3 then
            return false
        end
    end

    return true
end

local hesh_defs = {"LUMINITIATE", "PRIEST", "LUMINARI", "ZEALOT", "LUMICYTE", "LUMIN_TURRET"}

local hesh_filter = function(team, diff, is_backup)

    local counts = {}
    for k,v in pairs(team) do
        counts[v] = counts[v] and counts[v] + 1 or 1
    end

    if not is_backup then
        if (counts["LUMICYTE"] or 0) + (counts["LUMIN_TURRET"] or 0) == #team then
            return false
        end
    end

    return true
end

local rise_defs = {"RISE_REBEL", "RISE_RADICAL", "RISE_PAMPHLETEER", "RISE_AUTOMECH", "RISE_AUTODOG", "RISE_VALET"}

local rise_filter = function(team, diff, is_backup)
    local counts = {}
    for k,v in pairs(team) do
        counts[v] = counts[v] and counts[v] + 1 or 1
    end

    if (counts["RISE_PAMPHLETEER"] or 0) > (is_backup and 0 or 1) then
        return false
    end

    if (counts["RISE_RADICAL"] or 0) > 1 then
        return false
    end

    if (counts["RISE_VALET"] or 0) > 1 then
        return false
    end

    if (counts["RISE_VALET"] or 0) > 0 and TheGame:GetGameState():GetPlayerAgent():GetContentID() ~= "SMITH" then
        return false
    end


    if not is_backup then
        if (counts["RISE_AUTOMECH"] or 0) + (counts["RISE_AUTODOG"] or 0) == #team then
            return false
        end
    end

    for k,v in pairs(counts) do
        if v > 2 then
            return false
        end
    end

    return true
end


local civilian_defs = {"LABORER", "HEAVY_LABORER", "FOREMAN", "WEALTHY_MERCHANT", "CHEMIST", "BARTENDER"}
local civilian_filter = function(team, diff)
    return true
end


local barons_defs = {"SPARK_BARON_GOON", "SPARK_BARON_PROFESSIONAL", "SPARK_BARON_TASKMASTER", "SPARK_BARON_AUTOMECH", "COMBAT_DRONE", "AUTODOG"}

local barons_filter = function(team, diff)
    local counts = {}
    for k,v in pairs(team) do
        counts[v] = counts[v] and counts[v] + 1 or 1
    end

    if (counts["SPARK_BARON_TASKMASTER"] or 0) > 1 then
        return false
    end

    if (counts["SPARK_BARON_PROFESSIONAL"] or 0) > 1 then
        return false
    end

    if (counts["SPARK_BARON_AUTOMECH"] or 0) + (counts["COMBAT_DRONE"] or 0) + (counts["AUTODOG"] or 0) == #team then
        return false
    end

    for k,v in pairs(counts) do
        if v > 2 then
            return false
        end
    end

    return true
end


local bogger_defs = {"BOGGER_CULTIVATOR", "BOGGER_CLOBBER", "ERCHIN"}

local bogger_filter = function(team, diff, is_backup)
    local counts = {}
    for k,v in pairs(team) do
        counts[v] = counts[v] and counts[v] + 1 or 1
    end

    if (counts["ERCHIN"] or 0) == #team then
        return false
    end

    for k,v in pairs(counts) do
        if v > 2 then
            return false
        end
    end

    return true
end

--the actual party generators. Some of these use set lists, others use combinations and filtering


local GENERATORS = 
{
    --just a pair of guards; doesn't scale with difficulty
    INVARIANT_ADMIRALTY_GUARDS = function(difficulty_level, leader)
        return {"ADMIRALTY_GUARD", "ADMIRALTY_GUARD"}
    end,

    --just a pair of guards; doesn't scale with difficulty
    INVARIANT_BARON_GUARDS = function(difficulty_level, leader)
        return {"SPARK_BARON_AUTOMECH", "AUTODOG"}
    end,

    BOGGER_PATROL = function(difficulty_level)
        local candidates = PickTeams(bogger_defs, bogger_filter, difficulty_level)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"BOGGER_CULTIVATOR"}
    end,
    ADMIRALTY_PATROL = function(difficulty_level)
        local candidates = PickTeams(admiralty_defs, admiralty_filter, difficulty_level)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"ADMIRALTY_GOON"}
    end,

    ADMIRALTY_PATROL_BACKUP = function(difficulty_level, leader)
        local candidates = PickTeams(admiralty_defs, admiralty_filter, difficulty_level, leader)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"VROC"}
    end,    


    JAKES_PATROL = function(difficulty_level)
        local candidates = PickTeams(jakes_defs, jakes_filter, difficulty_level)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"JAKES_RUNNER"}
    end,

    JAKES_BACKUP = function(difficulty_level, leader)
        local candidates = PickTeams(jakes_defs, jakes_filter, difficulty_level, leader)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"COMBAT_DRONE"}
    end,    



    BANDIT_PATROL = function(difficulty_level)
        local candidates = PickTeams(bandit_defs, bandit_filter, difficulty_level)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"BANDIT_GOON"}
    end,

    BANDIT_PATROL_BACKUP = function(difficulty_level, leader)
        local candidates = PickTeams(bandit_defs, bandit_filter, difficulty_level, leader)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"FLEAD"}
    end,



    YOTES = function(difficulty_level)
        if difficulty_level <= 1 then
            return {"CRAYOTE"}
        elseif difficulty_level == 2 then
            return table.arraypick{
                {"CRAYOTE", "CRAYOTE"},
                {"CRAYOTE_SPITTER", "CRAYOTE"}
            }
        elseif difficulty_level == 3 then
            return table.arraypick{
                {"CRAYOTE_SPITTER", "CRAYOTE", "CRAYOTE"},
                {"HORNED_CRAYOTE", "CRAYOTE_SPITTER"}
            }
        elseif difficulty_level >= 4 then
            return table.arraypick{
                {"CRAYOTE_SPITTER", "CRAYOTE", "CRAYOTE", "CRAYOTE_SPITTER"},
                {"CRAYOTE_SPITTER", "HORNED_CRAYOTE", "CRAYOTE"}
            }
        end
    end,

    FLEADS = function(difficulty_level)
        if difficulty_level <= 1 then
            return {"FLEAD"}
        elseif difficulty_level == 2 then
            return {"FLEAD", "FLEAD"}
        elseif difficulty_level >= 3 then
            return {"FLEAD", "FLEAD"}
        end
    end,

    ERCHINS = function(difficulty_level)
        if difficulty_level <= 1 then
            return {"ERCHIN"}
        elseif difficulty_level == 2 then
            return {"ERCHIN", "ERCHIN"}
        elseif difficulty_level >= 3 then
            return {"ERCHIN", "ERCHIN", "ERCHIN" }
        end
    end,

    MECHS = function(difficulty_level)
        if difficulty_level <= 1 then
            return table.arraypick{
                { "AUTODOG", "AUTODOG", },
                { "COMBAT_DRONE", "COMBAT_DRONE", },
            }
        elseif difficulty_level == 2 then
            return table.arraypick{
                { "RISE_AUTOMECH", "AUTODOG", },
                { "COMBAT_DRONE", "COMBAT_DRONE", "COMBAT_DRONE", },
            }
        elseif difficulty_level >= 3 then
            return table.arraypick{
                { "RISE_AUTOMECH", "RISE_AUTOMECH", "RISE_AUTOMECH", },
                { "COMBAT_DRONE", "SPARK_BARON_AUTOMECH", "COMBAT_DRONE", },
                { "SPARK_BARON_AUTOMECH", "SPARK_BARON_AUTOMECH", },
                { "SPARK_BARON_AUTOMECH", "AUTODOG", "AUTODOG", "AUTODOG" },
            }
        end
    end,

    MONSTER_MASH = function(difficulty_level)
        local easy_monsters = {"CRAYOTE", "CRAYOTE_SPITTER", "VROC", "FLEAD", "FERAL_AUTODOG", "LUMICYTE", "HORNED_CRAYOTE"} 
        local hard_monsters = {"LUMINTHIAN", "GRAWKIT"}
        --local special monsters = ERCHIN
        local num_easy, num_hard = 0, 0
        
        if difficulty_level == 1 then
            num_easy, num_hard = 1, 0
        elseif difficulty_level == 2 then
            num_easy, num_hard = 2, 0
        elseif difficulty_level == 3 then
            num_easy, num_hard = 3, 0
        elseif difficulty_level == 4 then
            if math.random() < .5 then
                num_easy, num_hard = 0, 1
            else
                num_easy, num_hard = 4, 0
            end
        elseif difficulty_level >= 5 then
            num_easy, num_hard = 2, 1
        end

        local ret = {}
        table.shuffle(easy_monsters)
        table.shuffle(hard_monsters)
        
        for k = 1, num_hard do
            table.insert(ret, table.remove(hard_monsters))
        end
        for k = 1, num_easy do
            table.insert(ret, table.remove(easy_monsters))
        end

        return ret

    end,
    
    MONSTERS = function(difficulty_level)
        if difficulty_level <= 1 then
            return table.arraypick{
                {"CRAYOTE"},
                {"VROC"},
                {"FLEAD"},
            }
        elseif difficulty_level == 2 then
            return table.arraypick
            {
                {"CRAYOTE_SPITTER", "CRAYOTE"},
                {"HORNED_CRAYOTE", "CRAYOTE"},
                {"FLEAD", "FLEAD"},
                {"VROC", "VROC"},
            }
        elseif difficulty_level == 3 then
            return table.arraypick
            {
                {"LUMINTHIAN"},
                {"CRAYOTE_SPITTER", "CRAYOTE", "CRAYOTE"},
                {"CRAYOTE_SPITTER", "HORNED_CRAYOTE"},
                {"FLEAD", "FLEAD", "FLEAD"},
            }
        elseif difficulty_level >= 4 then
            return table.arraypick
            {
                {"LUMINTHIAN", "CRAYOTE_SPITTER", "CRAYOTE"},
                {"CRAYOTE_SPITTER", "CRAYOTE", "CRAYOTE", "CRAYOTE"},
                {"CRAYOTE_SPITTER", "HORNED_CRAYOTE", "CRAYOTE"},
            }
        end
    end,
    
    BOG_MONSTERS = function(difficulty_level)
        if difficulty_level <= 1 then
            return table.arraypick{
                {"ERCHIN"},
                {"FLEAD"},
                {"FERAL_AUTODOG", "FERAL_AUTODOG"},
            }
        elseif difficulty_level == 2 then
            return table.arraypick
            {
                {"ERCHIN", "ERCHIN"},
                {"FLEAD", "FLEAD"},
                {"FERAL_AUTODOG", "FERAL_AUTODOG", "FERAL_AUTODOG"},
            }
        elseif difficulty_level >= 3 then
            return table.arraypick
            {
                {"ERCHIN", "ERCHIN", "ERCHIN"},
                {"GRAWKIT"},
                {"FERAL_AUTODOG", "FERAL_AUTODOG", "FERAL_AUTODOG","FERAL_AUTODOG"},
            }
        end        
    end,

    MERCENARY_BACKUP = function(difficulty_level, leader)
        if difficulty_level <= 1 then
            return {"COMBAT_DRONE"}
        elseif difficulty_level == 2 then
            return {"COMBAT_DRONE", "JAKES_RUNNER"}
        elseif difficulty_level == 3 then
            return {"JAKES_RUNNER", "JAKES_RUNNER"}
        elseif difficulty_level >= 4 then
            return {"JAKES_SMUGGLER", "JAKES_RUNNER", "COMBAT_DRONE"}
        end
    end,
    
    CIVILIAN_PATROL = function(difficulty_level, leader)
        local candidates = PickTeams(civilian_defs, civilian_filter, difficulty_level)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"LABORER"}
    end,

    HESH_PATROL = function(difficulty_level, leader)
        local candidates = PickTeams(hesh_defs, hesh_filter, difficulty_level)

        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"PRIEST"}
    end,

    ----------
    SPARK_BARON_PATROL_MB = function(difficulty_level, leader)
        if difficulty_level <= 1 then
            return {"SPARK_BARON_GOON", "COMBAT_DRONE"}
        elseif difficulty_level == 2 then
            return {"SPARK_BARON_GOON", "COMBAT_DRONE", "SPARK_BARON_GOON"}
        elseif difficulty_level == 3 then
            return {"SPARK_BARON_PROFESSIONAL", "SPARK_BARON_GOON", "COMBAT_DRONE"}
        elseif difficulty_level == 4 then
            return {"SPARK_BARON_PROFESSIONAL", "SPARK_BARON_GOON",  "SPARK_BARON_GOON"}
        elseif difficulty_level >= 5 then
            return {"SPARK_BARON_PROFESSIONAL", "SPARK_BARON_PROFESSIONAL",  "SPARK_BARON_GOON"}
        end
    end,
    
    SPARK_BARON_PATROL = function(difficulty_level, leader)
        local candidates = PickTeams(barons_defs, barons_filter, difficulty_level)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"SPARK_BARON_GOON"}
    end,

    SPARK_BARON_BACKUP = function(difficulty_level, leader)
        local candidates = PickTeams(barons_defs, barons_filter, difficulty_level, leader)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"SPARK_BARON_GOON"}
    end,    


    RISE_PATROL = function(difficulty_level, leader)
        local candidates = PickTeams(rise_defs, rise_filter, difficulty_level)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"RISE_REBEL"}
    end,

    RISE_BACKUP = function(difficulty_level, leader)
        local candidates = PickTeams(rise_defs, rise_filter, difficulty_level, leader)
        if #candidates > 0 then
            return table.arraypick(candidates)
        end
        return {"RISE_REBEL"}

    end,
}

--creates full transient party for use in a quest convo when you just want to fight someone
function CreateCombatParty(generator, difficulty_level, location, try_find_existing_leader, leader_filter_fn, num_skinned_followers)

    local def_list
    if type(generator) == "function" then
        def_list = generator(difficulty_level)
    elseif type(generator) == "table" then
        def_list = generator
    else 
        assert (GENERATORS[generator], generator)
        def_list = GENERATORS[generator](difficulty_level)
    end
    
    def_list = def_list or {}
    local agents, leader =  AgentUtil.SpawnTransientParty(def_list, nil, try_find_existing_leader, leader_filter_fn, num_skinned_followers)
    if location then
        for k,v in pairs(agents) do
            v:MoveToLocation(location)
        end
    end

    for k,v in pairs(agents) do
        paperdoll.LoadAgentAnims(v)
    end

    return agents, leader
end

--surrounds a real agent with a transient party that will follow them around until dismissed (usually managed by a job)
function CreateCombatBackup(leader_agent, generator, difficulty_level)
    assert( leader_agent )
    
    local defs 
    if type(generator) == "function" then
        defs = generator( difficulty_level, leader_agent )
    elseif type(generator) == "table" then
        defs = generator
    else 
        generator = generator or leader_agent.combat_backup
        if generator == nil then
            return {}
        end
        assert( GENERATORS[generator], generator )
        defs = GENERATORS[generator]( difficulty_level, leader_agent )
    end

    defs = defs or {}
    local agents = AgentUtil.SpawnTransientParty( defs, leader_agent )
    for k,v in pairs(agents) do
        paperdoll.LoadAgentAnims(v)
    end

    return agents
end

function AddCombatPartyDef(id, fn)
    if GENERATORS[id] then
        print (loc.format( "Combat party generator {1} is already defined. If you aren't reloading combat, look into this", id ))
    end
    GENERATORS[id] = fn
end



return GENERATORS


