local negotiation_defs = require "negotiation/negotiation_defs"

Content.AddNegotiationModifier( "bonus_payment",
{
    name = "Bonus",
    desc = "When destroyed, add {1#money} to your bonus\nReduce by {2#percent} at end of your turn.\nWhen destroyed, {4} gains {IMPATIENCE 1}.\n\n<#PENALTY>If you lose the negotiation, you lose all bonuses!</>",
    modifier_type = MODIFIER_TYPE.BOUNTY,

    desc_fn = function(self, fmt_str)
        return loc.format(fmt_str, self.stacks or 1, self.decay_rate, self.modify_resolve, self.negotiator:GetName() )
    end,

    decay_rate = .25,
    modify_resolve = 5,

    loc_strings =
    {
        DIALOG = [[
agent:
    How about I give you another {1#money}?
]],
    },

    removed_sound = "event:/sfx/battle/cards/neg/gain_shills",

    OnInit = function( self, source )
        self:SetResolve( clamp( math.round((self.stacks)*.125), 2, 8 ))
        AUDIO:PlayEvent("event:/sfx/battle/cards/neg/create_argument/bonus")
    end,

    OnBounty = function( self, card )

        if card then
            if self.result_table then
                self.won_money = self.stacks
                table.insert(self.result_table, self.won_money)
            end
        end

        self.negotiator:AddModifier("IMPATIENCE")
        --self.negotiator:ModifyResolve( -self.modify_resolve, self )
    end,

    event_handlers =
    {

        [ negotiation_defs.EVENT.END_PLAYER_TURN ] = function( self, minigame )
            self.negotiator:DeltaModifier(self, -math.ceil( self.stacks*self.decay_rate))
            AUDIO:PlayEvent("event:/sfx/battle/cards/neg/bonus_tick_down")
        end

    },

    OnRemoveWidget = function( self, widget, panel )
        if self.won_money then
            panel.encounter:Dialog( loc.format( self.def:GetLocalizedString( "DIALOG" ), self.won_money ) )
            panel:Delay( 1.0 )
        end
    end,
})



Convo("ASK_FOR_A_BETTER_DEAL_CHAT")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_BETTER_DEAL = "<b>[{1#graft}]</b> Negotiate for a better deal...",
        OPT_BETTER_DEAL_TT = "If you win this negotiation, increase your job payout by {1#money} to {2#money}.",
        DIALOG_PRE_NEGOTIATE = [[
            player:
                I'm going to need a little more money.
        ]],
        DIALOG_SUCCESS_NO_BONUS = [[
            agent:
                Let's just agree to disagree.
                You'll get {1#money} upon completion, and that's my final offer.
        ]],
        DIALOG_SUCCESS = [[
            agent:
                Fine. I'll give you an extra {1#money} if you finish the job.
                That comes to {2#money}... you better not fail!
        ]],
        DIALOG_FAILURE = [[
            agent:
                !angry
                I can't afford that!
                Stop asking for handouts, grifter.
                !neutral
        ]],
        NEGOTIATION_REASON = "Convince {agent} to pay you more (will gain {1#money} on win)",
    }
    :Hub(function(cxt, who)
        local haggle_count = cxt.player.graft_owner:CountGraftsByID( "haggle_badge" )
        if who and haggle_count > 0 then
            
            local _, quest = who:IsQuestProvider(who)
            if quest and quest:IsActive() and (quest:GetType() == QTYPE.SIDE or TheGame:GetGameState():GetPlayerAgent():GetContentID() == "ROOK") and not quest.param.has_tried_to_negotiate_better_reward and quest.param.rewards and quest.param.rewards ~= 0 then
                local won_bonuses = {10} --give a default 10 shills.

                cxt:Opt("OPT_BETTER_DEAL", "haggle_badge" )
                    :IsHubOption(true)
                    --:PostText( "OPT_BETTER_DEAL_TT", math.round(BONUS_PERCENT*quest.param.rewards), math.round((1+BONUS_PERCENT)*quest.param.rewards) )
                    :SetQuestMark(quest)
                    :Dialog("DIALOG_PRE_NEGOTIATE")
                    :Fn(function() 
                        quest.param.has_tried_to_negotiate_better_reward = true
                    end)
                    :Negotiation{
                        on_start_negotiation = function(minigame)
                            
                            local amounts = {}
                            local val = quest.param.rewards
                            table.insert(amounts, math.ceil( val*math.randomGauss(.25, .5 )))
                            for i = 1, haggle_count do
                                table.insert(amounts, math.ceil( val*math.randomGauss(.5, 1.5) ))
                            end
                            
                            for k,amt in ipairs(amounts) do
                                local mod = minigame.opponent_negotiator:CreateModifier( "bonus_payment", amt )
                                mod.result_table = won_bonuses
                            end
                        end,
                        flags = NEGOTIATION_FLAGS.NO_AUTOFAIL,
                        --reason = "NEGOTIATION_REASON",
    
                        reason_fn = function(minigame)
                            local total_amt = 0
                            for k,v in pairs(won_bonuses) do
                                total_amt = total_amt + v
                            end
                            return loc.format(cxt:GetLocString("NEGOTIATION_REASON"), total_amt )
                        end,

                        enemy_resolve_required = 10,

                        difficulty = quest:GetRank(),
                        on_success = function() 
                            local total_bonus = 0
                            for k,v in ipairs(won_bonuses) do 
                                total_bonus = total_bonus + v
                            end
                            
                            quest.param.reward_negotiated_bonus = total_bonus
                            quest:VerifyRewards()

                            if total_bonus == 0 then
                                cxt:Dialog("DIALOG_SUCCESS_NO_BONUS", quest.param.rewards)
                            else
                                cxt:Dialog("DIALOG_SUCCESS", total_bonus, quest.param.rewards )
                            end
                        end,
                        on_fail = function() 
                            cxt:Dialog("DIALOG_FAILURE")
                        end,
                    }
            end

        end

    end)    


