local battle_defs = require "battle/battle_defs"
local negotiation_defs = require "negotiation/negotiation_defs"

Content.AddBattleCard( "spicy_noodles", 
{
    name = "Spicy Noodles",
    anim = "taunt",
    desc = "Gain 2 {POWER}.",
    flavour = "'That’s a good heat, plenty of flavour in there.'",

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    cost = 0,
    target_type = TARGET_TYPE.SELF,

    max_charges = 2,

    flags = battle_defs.CARD_FLAGS.AMBUSH | battle_defs.CARD_FLAGS.ITEM,

    OnPostResolve = function( self, battle, attack)
        self.owner:AddCondition("POWER", 2)
    end,
})
Content.AddBattleCard( "hearty_noodles", 
{
    name = "Hearty Noodles",
    anim = "taunt",
    desc = "Gain 2 actions. Draw a card.",
    flavour = "'Stupendous flavours and toothsome noodles at a great price!'",

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    cost = 1,
    target_type = TARGET_TYPE.SELF,

    max_charges = 2,

    flags = battle_defs.CARD_FLAGS.AMBUSH | battle_defs.CARD_FLAGS.ITEM,

    OnPostResolve = function( self, battle, attack)
        battle:ModifyActionCount(2)
        battle:DrawCards(1)
    end,
})
Content.AddNegotiationCard("zesty_noodles", 
{
    name = "Zesty Noodles",

    desc = "{INCEPT} 2 {FLUSTERED}. Draw a card.",
    flavour = "'Oh these are my favorite! They sell out so fast!'",
    max_charges = 2,
    
    cost = 0,
    flags = negotiation_defs.CARD_FLAGS.ITEM |negotiation_defs.CARD_FLAGS.AMBUSH | negotiation_defs.CARD_FLAGS.EXPEND,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    OnPostResolve = function( self, minigame )
        self.anti_negotiator:InceptModifier("FLUSTERED", 2 , self )
        minigame:DrawCards(1)
    end,
})

Convo("NOODLE_SHOP_CHAT")
    :Loc{
        OPT_BUY_SPICY = "Buy the Shrokian Twistpepper Noodles. (Gain {1#card})",
        DIALOG_BUY_SPICY = [[
            player:
                !take
                !neutral_burp
                Phew! Those are definitely not for everyone!
        ]],
        OPT_BUY_ZESTY = "Buy the Kra'Deshi Zest Bowl. (Gain {1#card})",
        DIALOG_BUY_ZESTY = [[
            player:
                !take
                !neutral_burp
                Delicious!
        ]],
        OPT_BUY_HOUSE_SPECIAL = "Buy the House Special. (Gain {1#card})",
        DIALOG_BUY_HOUSE_SPECIAL = [[
            player:
                !take
                !neutral_burp
                House Special never fails to impress!
        ]],
    }

    :Hub( function(cxt, who)
        if who:GetRoleAtLocation() == CHARACTER_ROLES.PROPRIETOR and cxt.location:GetContentID() == "MURDERBAY_NOODLE_SHOP" then
            cxt:Opt("OPT_BUY_SPICY", "spicy_noodles")
                :Dialog( "DIALOG_BUY_SPICY" )
                :PreIcon( global_images.drink )
                :ReqTimer("BOUGHT_NOODLES", 1, cxt:GetAgent(), true)
                :DeliverMoney( NOODLE_COST, { is_shop = true } )
                :Fn( function(cxt)
                    cxt:ForceTakeCards{"spicy_noodles"}
                end)

            cxt:Opt("OPT_BUY_ZESTY", "zesty_noodles")
                :Dialog( "DIALOG_BUY_ZESTY" )
                :PreIcon( global_images.drink )
                :ReqTimer("BOUGHT_NOODLES", 1, cxt:GetAgent(), true)
                :DeliverMoney( NOODLE_COST, { is_shop = true } )
                :Fn( function(cxt)
                    cxt:ForceTakeCards{"zesty_noodles"}
                end)

            cxt:Opt("OPT_BUY_HOUSE_SPECIAL", "hearty_noodles")
                :Dialog( "DIALOG_BUY_HOUSE_SPECIAL" )
                :PreIcon( global_images.drink )
                :ReqTimer("BOUGHT_NOODLES", 1, cxt:GetAgent(), true)
                :DeliverMoney( NOODLE_COST, { is_shop = true } )
                :Fn( function(cxt)
                    cxt:ForceTakeCards{"hearty_noodles"}
                end)

        end
    end)
