local no_keyboard = SPECIAL_BUILD_TYPE == (1024+64)
--------------------------------------------------------------------

Convo("REST_AND_RELAXATION")
    
    :Priority(CONVO_PRIORITY_LOWEST)
    :ConfrontState("STATE_MAIN_LOOP", function(cxt) return cxt.location:HasTag("player_room") end)
        :Loc{
            OPT_CHANGE_OUTFIT = "Change outfit",
            OPT_CALL_PET = "Call {1#agent}",
        }
        :SetLooping(true)        
        :Fn(function(cxt) 
                cxt:Opt("OPT_CHANGE_OUTFIT")
                    :PreIcon( global_images.outfit )
                    :GoTo("STATE_OUTFIT")
                
                for i, pet in ipairs( cxt.caravan:GetPets() ) do
                    cxt:Opt( "OPT_CALL_PET", pet )
                        :PreIcon( global_images.petcall )
                        :Fn( function(cxt)
                            cxt:TalkTo(pet)
                            cxt:GoTo("STATE_PET")    
                       end )
                end

                for _, agent in pairs(cxt.location.agents) do
                    if agent:GetRoleAtLocation() == CHARACTER_ROLES.WAITING_PET then
                        cxt:Opt( "OPT_CALL_PET", agent )
                            :PreIcon( global_images.petcall )
                            :Fn( function(cxt)
                                cxt:TalkTo(agent)
                                cxt:GoTo("STATE_PET")    
                            end )
                    end
                end
                StateGraphUtil.AddLeaveLocation(cxt)
            end)


    :State( "STATE_OUTFIT" ):SetLooping()
        :Loc
        {
            DIAL_INTRO = [[
                player:
                    !left
                    !thought
                    What should I wear today?
            ]],
            OPT_OUTFIT = "Put on \"{1}\"",
            ALREADY_WEARING = "You are already wearing this outfit.",
        }
        :Fn( function( cxt )

            if cxt:FirstLoop() then
                cxt:Dialog( "DIAL_INTRO" )
            end
            local selected_outfit = cxt.player.build
            -- Show unlocked outfits
            for k, v in ipairs( Content.GetOutfitsForCharacter(cxt.player.id) ) do
                if v.unlocked or TheGame:GetGameProfile():HasUnlock( v.id ) then
                    cxt:Opt( "OPT_OUTFIT", v:GetName() )
                        :PreIcon( v.build == selected_outfit and global_images.outfit_selected or global_images.outfit, 
                            v.build == selected_outfit and UICOLOURS.MONEY_LIGHT or UICOLOURS.SUBTITLE )
                        :PostText( v:GetDesc() )
                        :ReqCondition( v.build ~= selected_outfit, "ALREADY_WEARING" )
                        :Fn(function()

                            -- This outfit was selected, let's make the old character leave
                            cxt.encounter:ExitAgent( cxt.player )

                            -- Set the agent's outfit to the new one
                            local outfit = v.id and Content.GetOutfit(v.id)
                            cxt.player.build = outfit and outfit.build or cxt.player.build

                            -- Set the agent's outfit on the save slot as well
                            if TheGame:GetGameState().options and TheGame:GetGameState().options.custom and TheGame:GetGameState().options.custom.outfit then
                                TheGame:GetGameState().options.custom.outfit = v.id
                            end

                            -- And make the character come back in
                            cxt.encounter:PresentAgent( cxt.player, SIDE.LEFT )
                            cxt:Quip( cxt.player, "changed_outfit", v.build )

                            -- Save the selected outfit for when the convo options are regenerated
                            selected_outfit = v.id
                        end
                    )
                end
            end
            
            StateGraphUtil.AddBackButton(cxt)

        end)

    :State("STATE_PET")
        :Loc{
            DIAL_INTRO = [[
                player:
                    !left
                * You gesture towards {1}, who saunters over.
                agent:
                    !right
                player:
                    Heh. Cute.
            ]],
            OPT_PET_PET = "Pet {1}",
            OPT_LEAVE_PET = "Leave {1} here",
            OPT_SET_NAME = "Set new name for {1}",
            DIAL_PET_START = [[
                player:
                    !left
                    !happy
                    {1}!
                agent:
                    !right

            ]],
            DIAL_PET_MID = [[
                player:
                    !happy_call_pet
                agent:
                    !happy_pet
                player:
                    %happy_call_pet
                    !happy
            ]],
            DIAL_PET_END = [[
                player:
                    !happy
                agent:
                    !neutral
                    %happy_pet
            ]],
            DIALOG_LEAVE_PET = [[
                player:
                    Stay here, {agent}.
                agent:
                    !exit
                * {agent} goes to rest.
            ]],
            REQ_CANT_HAVE_A_PET = "You can't have another pet right now",

            OPT_TAKE_PET = "Take {1} with you",
            DIALOG_TAKE_PET = [[
                player:
                    Come, {agent}!
                agent:
                    !neutral

            ]],
        }
        :SetLooping( true )
        :Fn( function( cxt )
                if cxt:FirstLoop() then
                    cxt:Dialog( "DIAL_INTRO", cxt:GetAgent():GetName() )
                end

                if cxt:GetAgent():IsInPlayerParty() then
                    cxt:Opt("OPT_LEAVE_PET", cxt:GetAgent():GetName())
                        :PreIcon( global_images.order )
                        :Dialog("DIALOG_LEAVE_PET")
                        :Fn(function() 
                            cxt:GetAgent():Dismiss()
                            cxt:GetAgent():AddTag("NO_AUTO_CULL")
                            cxt:GetAgent():SetLocationRole(CHARACTER_ROLES.WAITING_PET)

                        end)
                        :Pop()
                else
                    cxt:Opt("OPT_TAKE_PET", cxt:GetAgent():GetName())
                        :ReqCondition(TheGame:GetGameState():GetCaravan():CanOwnPet(), "REQ_CANT_HAVE_A_PET")
                        :Dialog("DIALOG_TAKE_PET")
                        :Fn(function() 
                            cxt:GetAgent():SetLocationRole(nil)
                            cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.CREW)
                        end)
                end

                if not cxt:GetAgent().agent_def.can_talk then
                    cxt:Opt( "OPT_PET_PET", cxt:GetAgent():GetName() )
                        :PreIcon( global_images.petcall )
                        :Fn(function( cxt )
                            cxt:Dialog( "DIAL_PET_START", cxt:GetAgent():GetName() )
                            -- Get position of the character slots, and their anims too
                            local stage = TheGame:FE():FindScreen( Screen.ConversationScreen ):GetCharacterStage()
                            local left_widget = stage:GetCharacterSlot( cxt.encounter.left_agent )
                            local left_slot_x, left_slot_y = left_widget:GetPos()
                            local right_widget = stage:GetCharacterSlot( cxt.encounter.right_agent )
                            local right_slot_x, right_slot_y = right_widget:GetPos()
                            local left_character_x, left_character_y = stage:GetCharacterPos( cxt.encounter.left_agent )
                            local right_character_x, right_character_y = stage:GetCharacterPos( cxt.encounter.right_agent )
                            -- Calculate the distance the characters will have to move towards each other
                            local anim_fuzziness = 470
                            -- Move player agent towards the pet
                            stage:MoveCharacterToXPos( cxt.encounter.left_agent, -left_slot_x - anim_fuzziness )
                            stage:MoveCharacterToXPos( cxt.encounter.right_agent, right_slot_x - anim_fuzziness )
                            cxt:Dialog( "DIAL_PET_MID", cxt:GetAgent():GetName() )
                            -- Move both characters back to their spots
                            stage:MoveCharacterToXPos( cxt.encounter.left_agent, left_character_x )
                            stage:MoveCharacterToXPos( cxt.encounter.right_agent, right_character_x )
                            cxt:Dialog( "DIAL_PET_END", cxt:GetAgent():GetName() )
                            SendMetricsData( "PET_PETTED", { PET_DATA = { TYPE = cxt:GetAgent().def.id, NAME = cxt:GetAgent():GetName() } } )
                        end )
                end
                
                if not no_keyboard then
                    cxt:Opt( "OPT_SET_NAME", cxt:GetAgent():GetName() )
                        :PreIcon( global_images.petname )
                        :GoTo("STATE_NAME")
                end

                StateGraphUtil.AddBackButton(cxt)
        end )

    :State( "STATE_NAME" )
        :Loc
        {
            DIAL_NAME_CHANGED = [[
                player:
                    !left
                    !point
                    {1}!
                    !happy
                    Yes! That's a good name for you.
                agent:
                    !right
                    !bark
                    %happy_pet
            ]],
            DIAL_NAME_CANCELED = [[
                player:
                    !left
                    !thought
                    Well, maybe that name fits you after all.
                    !give
                agent:
                    !right
                    !bark
                    %happy_pet
            ]],
            POPUP_TITLE = "New name for your pet!",
            POPUP_SUBTITLE = "Input {1}'s new name",
        }
        :Fn( function( cxt )
            UIHelpers.EditString( 
                cxt:GetLocString( "POPUP_TITLE" ), 
                loc.format( cxt:GetLocString( "POPUP_SUBTITLE" ), cxt:GetAgent():GetName() ), 
                cxt:GetAgent():GetName(), 
                function( val )
                    cxt.enc:ResumeEncounter( val )
                end )

            local val = cxt.encounter:YieldEncounter()
            if val and val ~= cxt:GetAgent():GetName() then
                -- Name was changed to <val>
                cxt:Dialog( "DIAL_NAME_CHANGED", val )
                SendMetricsData( "PET_RENAMED", { PET_DATA = { TYPE = cxt:GetAgent().def.id, OLD_NAME = cxt:GetAgent():GetName(), NAME = val } } )
                cxt:GetAgent():SetCustomName( val )
                cxt:GetAgent():GenerateLocTable()
                TheGame:GetEvents():BroadcastEvent( "caravan_member_rename", cxt:GetAgent() )
                cxt:Pop()
            else
                -- Canceled
                cxt:Dialog( "DIAL_NAME_CANCELED" )
                cxt:Pop()
            end
        end )