local battle_defs = require "battle/battle_defs"

function GetDefaultAllyScale()
    local diff = TheGame and TheGame:GetGameState() and TheGame:GetGameState():GetCurrentBaseDifficulty() or 1
    local adv_scale_adjust = GetAdvancementModifier( ADVANCEMENT_OPTION.ALLY_SCALE ) or 0
    return math.max(1, diff - 1 + adv_scale_adjust)
end

function GetDefaultEnemyScale(quest_diff)
    local diff = quest_diff or ( TheGame and TheGame:GetGameState() and TheGame:GetGameState():GetCurrentBaseDifficulty() or 1 ) or 1
    return diff
end

MAX_HEALTH_LOOKUP = {}
MAX_HEALTH_LOOKUP.CREATURE = 28
MAX_HEALTH_LOOKUP.VERY_LOW = 35
MAX_HEALTH_LOOKUP.LOW = 40
MAX_HEALTH_LOOKUP.MEDIUM_LOW = 55
MAX_HEALTH_LOOKUP.MEDIUM = 70
MAX_HEALTH_LOOKUP.MEDIUM_HIGH = 85
MAX_HEALTH_LOOKUP.HIGH = 100

MAX_MORALE_LOOKUP = 
{
    VERY_LOW = .3,
    LOW = .25,
    MEDIUM = .2,
    HIGH = .1,
    VERY_HIGH = .05,
    IMMUNE = 0,
}

local DAMAGE_SCALE = .2
local DAMAGE_TEAM_SCALE = .2

local BASE_DAMAGE = 7
local BASE_VARIANCE = 0.25

local MAX_QUEST_DAMAGE_SCALE = .8
local MAX_TEAM_DAMAGE_SCALE = .6

local BUFF_SCALE = 0.2
local buff_scale_fn = function(diff)
    diff = diff or 1 
    return clamp(1 + (diff-1)*BUFF_SCALE, .25, 10)
end


local function GetCurveVal(val, maxval)
    local t = clamp( (val-1)/(maxval-1), 0, 1)
    local out = (t + t*t)*.5
    return out
end

NPC_ATTACK  =
{
    generate_damage_fn = function(self)
        if self.owner and self.owner.agent and self.owner.agent.def.combat_strength then
            
            local base_damage = self.base_damage or BASE_DAMAGE
            local scale = 1

            if self.owner.agent.boss then
                base_damage = type(base_damage) == "table" and base_damage[ math.min( GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY ) or 1, #base_damage) ] or base_damage
            else
                if type(base_damage) == "table" then
                    base_damage = base_damage[ math.min( self.owner:GetScale(), #base_damage ) ]
                else
                    local quest_scale = 1 + GetCurveVal(self.owner:GetScale(), 5)*MAX_QUEST_DAMAGE_SCALE
                    scale = scale * quest_scale
                end

                scale = scale * (GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_DAMAGE ) or 1)
            end

            local damage = base_damage*(self.damage_mult or 1)*(self.owner.agent.fight_data.damage_mult or 1)
            
            local num_other_side_fighters = self.owner.team:GetEnemyTeam():NumActiveFighters()
            
            damage = damage * scale
            if self.features then
                --jcheng: only scale defend feature, otherwise the other stuff gets ridiculous
                if self.features["DEFEND"] then
                    self.features["DEFEND"] = math.round(self.features["DEFEND"] * scale)
                end
            end

            local variance = damage*(self.damage_variance or BASE_VARIANCE)
            self.min_damage = math.max(1, math.round(damage - variance))
            self.max_damage = math.max(1, math.round(damage + variance*.5))
        end
    end,
}

NPC_MELEE = table.extend( NPC_ATTACK )
{
    flags =  battle_defs.CARD_FLAGS.MELEE
}

NPC_RANGED = table.extend( NPC_ATTACK )
{
    flags = battle_defs.CARD_FLAGS.RANGED
}


NPC_BUFF =
{
    ScaleBuff = function( self, t )
        local idx = math.min( self.owner:GetScale(), #t )
        return t[ idx ]
    end,

    generate_buff_fn = function(self)
        self.buff_strength = self.def.buff_strength or 1

        local scale = 1.0 + 0.5 * GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_ABILITY_STRENGTH )

        local buff_scale_fn = self.owner.agent and self.owner.agent.buff_scale_fn or buff_scale_fn
        scale = scale * buff_scale_fn(self.owner:GetScale())

        if scale then
            self.buff_strength = math.floor(self.buff_strength * scale)

            if self.features then
                --jcheng: only scale defend feature, otherwise the other stuff gets ridiculous
                if self.def and self.def.features and self.def.features["DEFEND"] then
                    self.features["DEFEND"] = math.round(self.def.features["DEFEND"] * scale)
                end
            end
        end
    end
}

FIGHT_DATA =
{
    NONE =
    {
        --jcheng: don't ever use this
        MAX_HEALTH = 1,
        MAX_MORALE = .1,
        behaviour = FighterBehaviour_Basic( table.empty )
    },

    MERCANTILE =
    {
        MAX_HEALTH = MAX_HEALTH_LOOKUP.VERY_LOW,
        MAX_MORALE = MAX_MORALE_LOOKUP.VERY_LOW,
    },
}

for k, v in pairs( FIGHT_DATA ) do
    v.template = k -- Do this mainly just for debug tracking (can inspect base table id)
end
