local battle_defs = require "battle/battle_defs"

local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

--------------------------------------------------------------------

local BATTLE_GRAFTS =
{

    -----------------------------------------------------------------
    -- COMMONS

    red_bandana =
    {
        name = "Red Bandana",
        desc = "Your allies receive {POWER 2} when they join battle.",
        flavour = "'Bright and aggressive, unless worn as an ascot.'",
        rarity = CARD_RARITY.COMMON,
        power_amt = 2,
        battle_condition = 
        {
            event_handlers =
            {
                [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                    if fighter ~= self.owner and fighter:GetTeam() == self.owner:GetTeam() then
                        fighter:AddCondition( "POWER", self.graft:GetDef().power_amt, self )
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            },
            
        },
    },

    red_bandana_plus =
    {
        name = "Boosted Red Bandana",
        desc = "Your allies receive <#UPGRADE>{POWER 3}</> when they join battle.",
        power_amt = 3,
    },

    company_band =
    {
        name ="Company Band",
        desc = "Pets have +{1} health in battle.",
        flavour = "'First class trainers get these bracelets and the animals seem to take note.'",

        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().health_bonus )
        end,
        rarity = CARD_RARITY.COMMON,

        health_bonus = 10,

        battle_condition = 
        {
            event_handlers =
            {
                [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                    if fighter ~= self.owner and fighter:GetTeam() == self.owner:GetTeam() and fighter:GetAgent():IsPet() then
                        local health = fighter:GetHealth()
                        local max = fighter:GetMaxHealth()
                        fighter:DeltaMaxStat(COMBAT_STAT.HEALTH, self.graft:GetDef().health_bonus)
                        fighter:DeltaStat(COMBAT_STAT.HEALTH, self.graft:GetDef().health_bonus)
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            },
        },
    },

    company_band_plus =
    {
        name = "Boosted Company Band",
        desc = "Pets have <#UPGRADE>+{1}</> health in battle.",
        health_bonus = 15,
    },

    rejig =
    {
        name = "Rejig",
        desc = "At the start of each battle, discard up to 2 cards and draw that many new cards.",
        flavour = "'Hey! No take backs!'",

        rarity = CARD_RARITY.COMMON,
        max_cards = 2,

        battle_condition = 
        {
            event_priorities =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = 10, -- happens AFTER warp_vial changes
            },

            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local discarded_cards = battle:DiscardCards(0, self.graft:GetDef().max_cards, self)
                    if #discarded_cards > 0 then
                        battle:DrawCards(#discarded_cards)
                    end
                    self.owner:RemoveCondition("rejig")
                end
            },
        },
    },

    rejig_plus =
    {
        name = "Wide Rejig",
        desc = "At the start of each battle, discard up to <#UPGRADE>4</> cards and draw that many new cards.",
        max_cards = 4,
    },

    robustify =
    {
        name = "Robustify",
        desc = "Gain 10 max health.",
        flavour = "'Always good to have a spare cybernetic organ, right?'",

        rarity = CARD_RARITY.COMMON,
        health_bonus = 10,

        OnAdded = function(self, agent)
            agent.health:AddStatModifier( "ROBUSTIFY", self:GetDef().health_bonus )
        end,
        
        OnUpgraded = function(self, agent)
            agent.health:AddStatModifier( "ROBUSTIFY", self:GetDef().health_bonus - self:GetDef().base_def.health_bonus )
        end,

        OnRemoved = function( self, agent )
            agent.health:AddStatModifier( "ROBUSTIFY", -self:GetDef().health_bonus )
            local health, max_health = agent.health:Get()
            if health > max_health then
                agent.health:SetPercent( 1 )
            end
        end,
    },

    robustify_plus =
    {
        name = "Boosted Robustify",
        desc = "Gain <#UPGRADE>15</> max health.",
        health_bonus = 15,
    },

    rugged_wiring =
    {
        name = "Rugged Wiring",
        desc = "Gain {POWER {1}} at the start of each battle. Lose {1} {POWER} the first time you take enemy damage.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:GetDef().power_amt)
        end,
        flavour = "'Mine haven’t given me any problems in days.'",

        rarity = CARD_RARITY.COMMON,

        power_amt = 2,

        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    if not self.used then
                        self.owner:AddCondition("POWER", self.graft:GetDef().power_amt, self)
                        self.used = true
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft)
                    end
                end,

                [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
                    if self.used and fighter == self.owner and self.battle:IsEnemySource( fighter, source ) then
                        self.owner:RemoveCondition( "POWER", self.graft:GetDef().power_amt, self)
                        self.owner:RemoveCondition( self.id )
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft)
                    end
                end
            },
        }        
    },

    rugged_wiring_plus =
    {
        name = "Boosted Rugged Wiring",
        desc = "Gain <#UPGRADE>{POWER {1}}</> at the start of each battle. Lose {1} {POWER} the first time you take enemy damage.",
        power_amt = 4,
    },

    -----------------------------------------------------------------
    -- UNCOMMONS
    the_spider =
    {
        name = "The Spider",
        desc = "After every 8th card played, gain {1} {TEMP_POWER}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:GetDef().power_amt)
        end,
        flavour = "'Just come back in a week so I can make sure it’s still healthy.'",

        rarity = CARD_RARITY.UNCOMMON,

        display_number = function( self )
            return self.userdata.counter or 0
        end,

        power_amt = 2,
        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, card )
                    if card.owner == self.owner then
                        self.graft:IncrementCounter()
                        if self.graft.userdata.counter >= 8 then
                            self.owner:AddCondition("POWER", self.graft:GetDef().power_amt, self)
                            self.owner:AddCondition("POWER_LOSS", self.graft:GetDef().power_amt, self)
                            self.graft:ResetCounter()
                            battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft)
                        end
                        battle:BroadcastEvent( BATTLE_EVENT.REFRESH_GRAFTS )
                    end
                end,
            },
        },
    },

    the_spider_plus =
    {
        name = "The Boosted Spider",
        desc = "After every 8th card played, gain <#UPGRADE>{1} {TEMP_POWER}</>.",
        power_amt = 3,
    },

    bottom_edge =
    {
        name = "Bottom Edge",
        rarity = CARD_RARITY.UNCOMMON,
        desc = "All attacks gain +{1} to minimum damage, up to their maximum damage.",
        flavour = "'Ground to perfection, watch those fingers.'",

        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:GetDef().strength)
        end,

        strength = 1,
        
        battle_condition = 
        { 
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.graft:GetDef().strength )
            end,

            event_priorities =
            {
                --Happens after Surge
                [BATTLE_EVENT.CALC_DAMAGE] = 11,
            },

            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if self.owner == card.owner then
                        local bonus = dmgt.max_damage - dmgt.min_damage
                        if bonus > 0 then
                            dmgt:AddDamage( math.min( bonus, self.graft:GetDef().strength * self.stacks ), 0, self )
                        end
                    end
                end,
            },
        },
    },

    bottom_edge_plus =
    {
        name = "Boosted Bottom Edge",
        desc = "All attacks gain <#UPGRADE>+{1}</> to minimum damage, up to their maximum damage.",
        strength = 2,
    },

    illicit_shifter =
    {
        name = "Illicit Shifter",
        desc = "When installed, upgrade 3 random battle cards.",
        flavour = "'One use but you’re going to find yourself moving a lot easier in some areas.'",

        rarity = CARD_RARITY.UNCOMMON,

        OnAdded = function( self, agent )
            local cards = agent.battler:GetCards()
            local candidates = {}
            for i=1,#cards do
                if cards[i]:HasUpgrades() and not cards[i]:UpgradeReady() and not cards[i]:IsHatched() then
                    table.insert(candidates, cards[i])
                end
            end
            for i = 1, math.min( #candidates, 3 ) do
                local pick = math.random(#candidates)
                candidates[pick]:AddXP(candidates[pick].max_xp)
                local popup = Screen.UpgradeCardPopup( candidates[pick], agent )
                TheGame:FE():PushScreen( popup )
                table.remove(candidates, pick)         
            end
        end
    },

    illicit_shifter_plus =
    {
        name = "Boosted Shifter",
        desc = "When upgraded, upgrade <#UPGRADE>3 additional</> battle cards.",

        OnUpgraded = function( self, agent )
            self:GetDef().OnAdded( self, agent )
        end
    },

    fatigue_spring =
    {
        name = "Fatigue Spring",
        desc = "At the end of your turn, gain {1} {DEFEND} if you have {WOUND} or {IMPAIR}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:GetDef().defend_amt)
        end,
        flavour = "'You lose about 3 hours worth of good sleep, but the payoff is well worth it.'",

        rarity = CARD_RARITY.UNCOMMON,

        defend_amt = 4,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    if self.owner:HasCondition("IMPAIR") or self.owner:HasCondition("WOUND") then
                        self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft)
                    end
                end
            },
        },
    },

    fatigue_spring_plus =
    {
        name = "Stone Fatigue Spring",
        desc = "At the end of your turn, gain <#UPGRADE>{1}</> {DEFEND} if you have {WOUND} or {IMPAIR}.",
        defend_amt = 6,
    },

    -----------------------------------------------------------------
    -- RARE

    muscle_bank =
    {
        name = "Muscle Bank",
        desc = "Draw 3 extra cards at the start of every turn. At the end of your turn, take 1 damage for every card in your hand above 3.",
        flavour = "'You’re going to be sore in the morning.'",
        
        rarity = CARD_RARITY.RARE,

        draw_count = 3,

        battle_condition = 
        {
            PreviewFn = function( self, damage_preview, targets )
                local damage = self.battle:GetHandDeck():CountCards() - 3
                if damage > 0 then
                    if targets[self.owner] == nil then
                       targets[self.owner] = {} 
                    end
                    table.insert(targets[self.owner], { condition = self, min_damage = damage, max_damage = damage, owner = self.owner })
                end
            end,

            event_priorities =
            {
                -- After Rook's passive defense
                [ BATTLE_EVENT.END_PLAYER_TURN  ] = 11,
            },
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function ( self, battle )
                    battle:DrawCards(self.graft:GetDef().draw_count)
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft)
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function ( self, battle )
                    local damage = battle:GetHandDeck():CountCards()
                    if damage > 3 and not battle:CheckGameOver() then
                        self.owner:ApplyDamage(damage - 3)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft)
                    end
                end
            },
        },
    },

    muscle_bank_plus = 
    {
        name = "Visionary Muscle Bank",
        desc = "Draw <#UPGRADE>4</> extra cards at the start of every turn. At the end of your turn, take 1 damage for every card in your hand above 3.",
        draw_count = 4,
    },

    wretched_jar =
    {
        name = "Wretched Jar",
        desc = "Whenever you draw {1} or more cards after your initial hand in a single turn, play a random card in your hand with a random target.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:GetDef().draw_amt)
        end,
        flavour = "'Just don’t ask where I found it.'",

        rarity = CARD_RARITY.RARE,

        draw_amt = 6,

        display_number = function( self )
            return self:GetCounter() or 0
        end,

        battle_condition =
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                    if not start_of_turn and not self.used then
                        self.graft:IncrementCounter()
                        if self.graft:GetCounter() >= self.graft:GetDef().draw_amt then
                            local card = table.arraypick(battle:GetHandDeck().cards)
                            if card then
                                card:NotifyTriggered()
                                battle:PlayCard(card)
                                battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft)
                                self.used = true
                            end
                        end
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,

                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.used = false
                    self.graft:ResetCounter()
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end
            },
        },
    },

    wretched_jar_plus =
    {
        name = "Boosted Wretched Jar",
        desc = "Whenever you draw <#UPGRADE>{1}</> or more cards after your initial hand in a single turn, play a random card in your hand with a random target.",
        draw_amt = 5,
    },

    dynamo =
    {
        name = "Dynamo",
        desc = "Whenever you shuffle your deck, gain {POWER {1}}.",
        flavour = "'It winds off raw adrenaline and takes a bit to get going.'",

        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().strength)
        end,
        xp_desc = "This graft gains XP whenever you shuffle your deck.",

        mod_xp = 5,
        manual_xp = true,
        strength = 1,
        adrenaline = 0,
        rarity = CARD_RARITY.RARE,
        battle_condition = 
        {
            name = "Dynamo",
            desc = "Whenever you shuffle your deck, gain {POWER {1}}.",
            desc_fn = function( self, fmt_str ) return loc.format( fmt_str, self.graft:GetDef().strength ) end,
            event_handlers =
            {
                [ BATTLE_EVENT.SHUFFLE_DISCARDS ] = function( self, num_cards )
                    self.owner:AddCondition( "POWER", self.graft:GetDef().strength * self.stacks, self )
                    self.owner:AddCondition( "ADRENALINE", self.graft:GetDef().adrenaline * self.stacks, self )
                    self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    self.graft:AddXP(1)
                end
            }
        },
    },

    dynamo_plus =
    {
        name = "Boosted Dynamo",
        desc = "Whenever you shuffle your deck, gain {POWER {1}} <#UPGRADE>and {ADRENALINE {2}}</>.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().strength, self:GetDef().adrenaline)
        end,
        adrenaline = 1,
    },

    feedback_loop =
    {
        name = "Feedback Loop",
        desc = "Your first attack each battle does x{1} damage.",
        flavour = "'Burns itself out right away but it just wrecks house.'",

        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().multiplier )
        end,
        rarity = CARD_RARITY.RARE,
        multiplier = 2,

        battle_condition =
        {
            hidden = false,
            desc = "Your first attack each battle does x{1} damage.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.graft:GetDef().multiplier * self.stacks )
            end,

            event_priorities =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = EVENT_PRIORITY_MULTIPLIER,
            },

            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt, hit )
                    if self.owner == card.owner and (hit == nil or hit.hit_index == 1) then
                        local multi = self.graft:GetDef().multiplier * self.stacks
                        dmgt:ModifyDamage( dmgt.min_damage * multi, dmgt.max_damage * multi, self )
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if attack.attacker == self.owner and attack.card:IsAttackCard() and attack:IsDamageModifier( self ) then
                        self.owner:RemoveCondition( self.id )
                    end
                end,
            },
        },
    },

    feedback_loop_plus =
    {
        name = "Boosted Feedback Loop",
        desc = "Your first attack each battle does <#UPGRADE>x{1}</> damage.",
        multiplier = 3,
    },

    salvager =
    {
        name = "Salvager",
        desc = "If you discarded at least 4 cards on your previous turn, gain {1} {TEMP_POWER}.",
        flavour = "'Always important to shrink that carbon footprint.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().power_amt )
        end,

        rarity = CARD_RARITY.RARE,
        power_amt = 3,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    if self.discarded and self.discarded >= 4 then
                        self.discarded = 0
                        self.owner:AddCondition("POWER", self.graft:GetDef().power_amt, self)
                        self.owner:AddCondition("POWER_LOSS", self.graft:GetDef().power_amt, self)
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,

                [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                    self.discarded = (self.discarded or 0) + 1
                end,
            },
        },
    },

    salvager_plus =
    {
        name = "Boosted Salvager",
        desc = "If you discarded at least 4 cards on your previous turn, gain <#UPGRADE>{1}</> {TEMP_POWER}.",
        power_amt = 4,
    },

    master_chip =
    {
        name = "Master Chip",
        desc = "At the start of each battle, {IMPROVISE} a random item card.",
        flavour = "'There’s a lot of tech in this. Just hope you are packing enough raw material or it could get ugly.'",

        rarity = CARD_RARITY.RARE,

        pool_size = 3,

        battle_condition =
        {
            hidden = true,
            loc_strings =
            {
                MASTER_CHIP_IMPROVISE = "Choose an item card to insert into your hand",
            },
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local defs = BattleCardCollection.AllLocalItems():Pick(self.graft:GetDef().pool_size)
                    local cards = {}
                    for i,def in ipairs(defs) do
                        table.insert(cards, Battle.Card(def.id, self.owner))
                    end
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    battle:BroadcastEvent( BATTLE_EVENT.DELAY, 0.3 ) -- lets the draw completely settle into your hand.
                    battle:ImproviseCards(cards, 1, nil, nil, self:GetLocalizedString("MASTER_CHIP_IMPROVISE"), self)
                    self.owner:RemoveCondition(self.id)
                end
            },
        },
    },

    master_chip_plus =
    {
        name = "Boosted Master Chip",
        desc = "At the start of each battle, <#UPGRADE>{IMPROVISE_PLUS}</> a random item card.",
        pool_size = 5,
    },

    -- BOSS GRAFTS
    dangerous_bulb = 
    {
        name = "Dangerous Bulb",
        desc = "Gain 1 action at the start of your turn. Take 3 damage at the end of every turn.",
        flavour = "'No returns, it’s right in the name for Hesh’s sake.'",

        rarity = CARD_RARITY.BOSS,
        card_draw = 0,

        battle_condition = 
        {
            PreviewFn = function( self, damage_preview, targets )
                if self.active then
                    if targets[self.owner] == nil then
                       targets[self.owner] = {} 
                    end
                    table.insert(targets[self.owner], { condition = self, min_damage = 3, max_damage = 3, owner = self.owner })
                end
            end,
            event_priorities =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = 11, -- After Skeletal suspension
            },

            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.owner.player_controlled then
                            self.battle:ModifyActionCount( self.stacks )
                            local draw = self.graft:GetDef().card_draw
                            if draw > 0 then self.battle:DrawCards(draw) end
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                            self.active = true
                        end
                    end
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    if not battle:CheckGameOver() then
                        self.owner:ApplyDamage(3)
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        self.active = false
                    end
                end
            },
        },
    },

    dangerous_bulb_plus =
    {
        name = "Visionary Dangerous Bulb",
        desc = "Gain 1 action <#UPGRADE>and draw a card</> at the start of your turn. Take 3 damage at the end of every turn.",
        card_draw = 1,
    },

    calamity_jar = 
    {
        name = "Calamity Jar",
        desc = "Gain 1 action at the start of your turn. You can no longer own a pet.",
        flavour = "'Animals just can’t handle the noise. I actually kind of like it.'",

        rarity = CARD_RARITY.BOSS,
        card_draw = 0,

        OnAdded = function(self, agent)
            local pets = {}
            for i,agent in agent:Party() do
                if agent:IsPet() then
                    table.insert(pets, agent)
                end
            end
            for i=1,#pets do
                pets[i]:Dismiss()
            end
        end,

        event_handlers =
        {
            [ "calc_max_pets" ] = function ( self, pet_details )
                pet_details.max_pets = 0
            end,
        },

        battle_condition = 
        {
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.owner.player_controlled then
                            self.battle:ModifyActionCount( self.stacks )
                            local draw = self.graft:GetDef().card_draw
                            if draw > 0 then self.battle:DrawCards(draw) end
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,
            },
        },
    },

    calamity_jar_plus =
    {
        name = "Visionary Calamity Jar",
        desc = "Gain 1 action <#UPGRADE>and draw a card</> at the start of your turn. You can no longer own a pet.",
        card_draw = 1,
    },

    quick_strap =
    {
        name = "Quick Strap",
        desc = "Gain 1 action at the start of your turn. Battle card upgrades with multiple upgrades now have 1 option fewer.",
        flavour = "'A belt with a series of enhancement drug injectors. Seems to really affect the ol' attention span as well.'",

        rarity = CARD_RARITY.BOSS,
        card_draw = 0,

        event_handlers =
        {
            [ "modify_upgrades" ] = function ( self, card, upgrade_ids, graft_modifiers )
                if is_instance( card, Battle.Card ) and #upgrade_ids > 1 then
                    table.remove( upgrade_ids )
                    table.insert( graft_modifiers, self )
                end   
            end,
        },

        battle_condition = 
        {
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.owner.player_controlled then
                            self.battle:ModifyActionCount( self.stacks )
                            local draw = self.graft:GetDef().card_draw
                            if draw > 0 then self.battle:DrawCards(draw) end
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,
            },
        },
    },

    quick_strap_plus =
    {
        name = "Visionary Quick Strap",
        desc = "Gain 1 action <#UPGRADE>and draw a card</> at the start of your turn. Battle card upgrades now have 1 option fewer.",
        card_draw = 1,
    },

    booster_clamp =
    {
        name = "Booster Clamp",
        desc = "Gain 1 action at the start of your turn. Enemies no longer {SURRENDER}.",
        flavour = "'Kind of leaves your empathy in shambles. Makes room for fighting I guess.'",

        rarity = CARD_RARITY.BOSS,
        card_draw = 0,

        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.owner.player_controlled then
                            self.battle:ModifyActionCount( self.stacks )
                            local draw = self.graft:GetDef().card_draw
                            if draw > 0 then self.battle:DrawCards(draw) end
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,

                [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                    if fighter ~= self.owner and fighter:GetTeam() ~= self.owner:GetTeam() then
                        fighter:EnableMorale( false )
                    end
                end, 
            },
        },

    },

    booster_clamp_plus =
    {
        name = "Visionary Booster Clamp",
        desc = "Gain 1 action<#UPGRADE> and draw 1 card</> at the start of your turn. Enemies no longer {SURRENDER}.",
        card_draw = 1,
    },

    quick_charger =
    {
        name = "Quick Charger",
        desc = "At the start of your turn, reduce the cost of a random card in your hand by 1 until played.",
        flavour = "'Obvious upgrade from last year’s slow charger.'",

        cost_reduction = 1,
        rarity = CARD_RARITY.BOSS,

        battle_condition = 
        {
            CanReduceCost = function( card )
                return card.engine:CalculateActionCost( card ) > 0 and not card:IsFlagged( CARD_FLAGS.UNPLAYABLE )
            end,

            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local hand = battle:GetHandDeck()
                    local card = hand:PickCardIf( self.CanReduceCost )
                    if card then
                        if self.chosen_cards == nil then
                            self.chosen_cards = {}
                        end
                        table.insert(self.chosen_cards, card)
    
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,

                [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                    if self.chosen_cards and table.contains(self.chosen_cards, card) then
                        cost_acc:AddValue(-self.graft:GetDef().cost_reduction, self)
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if self.chosen_cards then
                        table.arrayremove( self.chosen_cards, attack.card )
                    end
                end,
            },
        },
    },

    quick_charger_plus =
    {
        name = "Boosted Quick Charger",
        desc = "At the start of your turn, reduce the cost of a random card in your hand by <#UPGRADE>2</> until played.",
        cost_reduction = 2,
    },

    reckless_idol =
    {
        name = "Reckless Idol",
        desc = "Gain 1 action at the start of your turn, reduce {DEFEND} gained from cards by 2.",
        flavour = "'Under the pillow every night and you dream real wicked.'",

        rarity = CARD_RARITY.BOSS,

        defense_reduction = 2,
        card_draw = 0,
        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.owner.player_controlled then
                            self.battle:ModifyActionCount( self.stacks )
                            local draw = self.graft:GetDef().card_draw
                            if draw > 0 then self.battle:DrawCards(draw) end
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,

                [ BATTLE_EVENT.CALC_MODIFY_STACKS ] = function( self, acc, condition_id, fighter, source )
                    if condition_id == "DEFEND" and is_instance( source, Battle.Card ) and source.owner == self.owner then
                        if acc.value > 0 then
                            acc:AddValue( math.max(-acc.value, -self.graft:GetDef().defense_reduction), self )
                        end
                    end
                end, 
            },
        },
    },

    reckless_idol_plus =
    {
        name = "Visionary Reckless Idol",
        desc = "Gain 1 action<#UPGRADE> and draw 1 card</> at the start of your turn, reduce {DEFEND} gained from cards by 2.",
        card_draw = 1,
    },

    heirloom_knucks =
    {
        name = "Heirloom Knucks",
        desc = "Gain 1 action at the start of your turn, your maximum hand size is reduced by 3.",
        flavour = "'Grandma didn’t mess around.'",

        rarity = CARD_RARITY.BOSS,
        card_draw = 0,
        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.owner.player_controlled then
                            if not self.reduced_hand then
                                self.battle:GetHandDeck():SetMaxSize(self.battle:GetHandDeck():GetMaxSize() - 3)
                                self.reduced_hand = true
                            end
                            self.battle:ModifyActionCount( self.stacks )
                            local draw = self.graft:GetDef().card_draw
                            if draw > 0 then self.battle:DrawCards(draw) end
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,
            },
        },
    },

    heirloom_knucks_plus =
    {
        name = "Visionary Heirloom Knucks",
        desc = "Gain 1 action<#UPGRADE> and draw 1 card</> at the start of your turn, your maximum hand size is reduced by 3.",
        card_draw = 1,
    },
}

---------------------------------------------------------------------------------------------

for i, id, graft in sorted_pairs( BATTLE_GRAFTS ) do
    graft.card_defs = battle_defs
    graft.series = "GENERAL"
    graft.type = GRAFT_TYPE.COMBAT
    if graft.battle_condition and graft.battle_condition.hidden == nil then
        graft.battle_condition.hidden = true
    end

    Content.AddGraft( id, graft )
end
