local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

------------------------------------------------------------------------------------------------

local FEATS =
{
    -- LEVEL 1
    vicious_feat = 
    {
        name = "Vicious",
        img = engine.asset.Texture( "UI/daily_feat_1.tex", true ),
        desc = "Win the challenge without letting any enemies surrender or flee in battle.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,

        points = 100,
        feat_icon = "UI/daily_feat_1.tex",

        battle_condition =
        {
            hidden = true,
            desc = "Win the challenge without letting any enemies surrender or flee in battle.",
            event_handlers =
            {
                [BATTLE_EVENT.STATUS_CHANGED] = function( self, fighter, status )
                    if status == FIGHT_STATUS.FLED and fighter:GetTeamID() ~= self.owner:GetTeamID() then
                        self.graft.userdata.status = FEAT_STATUS.FAILED
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.owner:RemoveCondition(self.id, 1, self)
                    end
                end,
            },
        },
        event_handlers =
        {
            [ "resolve_battle" ] = function( self, battle, primary_enemy, repercussions )
                for i, fighter in battle:Fighters( TEAM.RED ) do
                    if not fighter.agent:IsDead() then
                        self.userdata.status = FEAT_STATUS.FAILED
                    end
                end
            end,

            [ "GAME_OVER" ] = function( self, gamestate, result )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE and result == GAMEOVER.VICTORY then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end,
        }
    },

    smashing = 
    {
        name = "Smashing",
        img = engine.asset.Texture( "UI/daily_feat_1.tex", true ),
        desc = "Deal a total of 30 damage or more in a single turn.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,

        points = 100,
        feat_icon = "UI/daily_feat_1.tex",

        battle_condition =
        {
            hidden = true,
            desc = "Deal a total of 30 damage or more in a single turn.",
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    if self.graft.userdata.status == FEAT_STATUS.COMPLETE then
                        self.owner:RemoveCondition(self.id, 1, self)
                    end
                    self.dmg_counter = 0
                end,

                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.attacker == self.owner and (hit.damage or 0) > 0 then
                        self.dmg_counter = (self.dmg_counter or 0) + hit.damage
                        if self.dmg_counter >= 30 then
                            self.graft.userdata.status = FEAT_STATUS.COMPLETE
                            TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                            self.owner:RemoveCondition(self.id, 1, self)
                        end
                    end
                end,
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    the_backup =
    {
        name = "The Backup",
        desc = "Exceed 5 friendly arguments at one time.",

        points = 100,
        feat_icon = "UI/daily_feat_1.tex",
        img = engine.asset.Texture( "UI/daily_feat_1.tex", true ),

        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.MODIFIER_ADDED ] = function ( self, modifier, source )
                    if modifier.negotiator == self.negotiator and (self.negotiator:GetModifierCount(MODIFIER_TYPE.ARGUMENT) + self.negotiator:GetModifierCount(MODIFIER_TYPE.BOUNTY) > 5) then
                        self.graft.userdata.status = FEAT_STATUS.COMPLETE
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end,
                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.graft.userdata.status == FEAT_STATUS.COMPLETE then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    brain_rack =
    {
        name = "Brain Rack",
        desc = "Cycle your negotiation deck 20 times.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
            self.userdata.counter = 0
        end,
        
        points = 100,
        feat_icon = "UI/daily_feat_1.tex",
        img = engine.asset.Texture( "UI/daily_feat_1.tex", true ),

        GetProgressText = function( self )
            return loc.format( "{1}/{2}", self:GetCounter() or 0, 20 )
        end,

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.SHUFFLE_DISCARDS ] = function( self, num_cards )
                    if self.graft:GetCounter() < 20 then
                        self.graft:IncrementCounter()
                        if self.graft:GetCounter() >= 20 then
                            self.graft.userdata.status = FEAT_STATUS.COMPLETE
                            TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                            self.negotiator:RemoveModifier(self.id, 1, self)
                        end
                    else
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end,
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    blow_out =
    {
        name = "Blow Out",
        desc = "Deal a total of 25 resolve damage in a single turn.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        
        points = 100,
        feat_icon = "UI/daily_feat_1.tex",
        img = engine.asset.Texture( "UI/daily_feat_1.tex", true ),

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                    if source.negotiator == self.negotiator then
                        self.total_damage = (self.total_damage or 0) + damage
                        if self.total_damage >= 25 then
                            self.graft.userdata.status = FEAT_STATUS.COMPLETE
                            TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                            self.negotiator:RemoveModifier(self.id, 1, self)
                        end
                    end
                end,

                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        self.total_damage = 0
                        if self.graft.userdata.status == FEAT_STATUS.COMPLETE then
                            self.negotiator:RemoveModifier(self.id, 1, self)
                        end
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    endless_ideas =
    {
        name = "Endless Ideas",
        desc = "Create 30 or more arguments during the challenge.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
            self.userdata.counter = 0
        end,
        img = engine.asset.Texture( "UI/daily_feat_1.tex", true ),

        points = 100,
        feat_icon = "UI/daily_feat_1.tex",

        GetProgressText = function( self )
            return loc.format( "{1}/{2}", self:GetCounter() or 0, 30 )
        end,

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.MODIFIER_ADDED ] = function ( self, modifier, source )
                    if modifier.negotiator == self.negotiator and not modifier.hidden and self.graft then
                        self.graft:IncrementCounter()
                        if self.graft:GetCounter() >= 30 then
                            self.graft.userdata.status = FEAT_STATUS.COMPLETE
                            TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                            self.negotiator:RemoveModifier(self.id, 1, self)
                        end
                    end
                end,
                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.graft.userdata.status == FEAT_STATUS.COMPLETE then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate, result )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end,
        }
    },

    -- LEVEL 2
    no_sweat = 
    {
        name = "No Sweat",
        desc = "Win the challenge without dropping below 50% health.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),

        points = 250,
        feat_icon = "UI/daily_feat_2.tex",

        battle_condition =
        {
            hidden = true,
            desc = "Win the challenge without dropping below 50% health.",
            event_handlers =
            {
                [ BATTLE_EVENT.DELTA_STAT ] = function( self, fighter, stat, delta, value, mitigated  )
                    if fighter == self.owner and stat == COMBAT_STAT.HEALTH then
                        if self.owner:GetHealthPercent() < 0.50 then
                            self.graft.userdata.status = FEAT_STATUS.FAILED
                            TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                            self.owner:RemoveCondition(self.id, 1, self)
                        end
                    end
                end,
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate, result )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE and result == GAMEOVER.VICTORY then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end,
        }
    },

    cheapskate =
    {
        name = "Cheapskate",
        desc = "Win the challenge without upgrading any cards.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,

        points = 250,
        feat_icon = "UI/daily_feat_2.tex",
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),

        event_handlers =
        {
            [ "card_upgraded" ] = function( self, card )
                self.userdata.status = FEAT_STATUS.FAILED
                TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
            end,

            [ "GAME_OVER" ] = function( self, gamestate, result )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE and result == GAMEOVER.VICTORY then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end,
        },
    },

    play_dead = 
    {
        name = "Play Dead",
        desc = "Complete the challenge with 10 health or less remaining.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),

        points = 250,
        feat_icon = "UI/daily_feat_2.tex",

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate, result )
                if result == GAMEOVER.VICTORY and gamestate:GetPlayerAgent():GetAspect("health"):Get() <= 10 then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                else
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end,
        },
    },

    thorough_search =
    {
        name = "Thorough Search",
        desc = "Cycle your battle deck 20 times.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
            self.userdata.counter = 0
        end,
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),

        points = 250,
        feat_icon = "UI/daily_feat_2.tex",

        GetProgressText = function( self )
            return loc.format( "{1}/{2}", self:GetCounter() or 0, 20 )
        end,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.SHUFFLE_DISCARDS ] = function( self, num_cards )
                    self.graft:IncrementCounter()
                    if self.graft:GetCounter() >= 20 then
                        self.graft.userdata.status = FEAT_STATUS.COMPLETE
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.owner:RemoveCondition(self.id, 1, self)
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    tank_up =
    {
        name = "Tank Up",
        desc = "Exceed 40 <b>Defend</>.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),

        points = 250,
        feat_icon = "UI/daily_feat_2.tex",
        series = "SAL",

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if fighter == self.owner and condition.id == "DEFEND" and condition.stacks >= 40 then
                        self.graft.userdata.status = FEAT_STATUS.COMPLETE
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.owner:RemoveCondition(self.id, self.stacks, self)
                    end
                end,
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    ice_cold =
    {
        name = "Ice Cold",
        desc = "Exceed 30 <b>Composure</> on a single argument.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        
        points = 250,
        feat_icon = "UI/daily_feat_2.tex",
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.DELTA_COMPOSURE ] =  function( self, modifier, new_value, old_value, source, start_of_turn )
                    if modifier.negotiator == self.negotiator and new_value >= 30 then
                        self.graft.userdata.status = FEAT_STATUS.COMPLETE
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end,

                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.graft.userdata.status == FEAT_STATUS.COMPLETE then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    record_breaker =
    {
        name = "Record Breaker",
        desc = "Destroy 40 or more arguments during the challenge.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
            self.userdata.counter = 0
        end,
        
        points = 250,
        feat_icon = "UI/daily_feat_2.tex",
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),

        GetProgressText = function( self )
            return loc.format( "{1}/{2}", self:GetCounter() or 0, 40 )
        end,

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                    if source and source.negotiator == self.negotiator then
                        self.graft:IncrementCounter()
                        if self.graft:GetCounter() >= 40 then
                            self.graft.userdata.status = FEAT_STATUS.COMPLETE
                            TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                            self.negotiator:RemoveModifier(self.id, 1, self)
                        end
                    end
                end,

                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.graft.userdata.status == FEAT_STATUS.COMPLETE then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    to_the_point =
    {
        name = "To the Point",
        desc = "Win the challenge without creating any friendly arguments.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        
        points = 250,
        feat_icon = "UI/daily_feat_2.tex",
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),


        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.MODIFIER_ADDED ] = function ( self, modifier, source )
                    if source and source.negotiator == self.negotiator and modifier.negotiator == self.negotiator and modifier.modifier_type ~= MODIFIER_TYPE.HIDDEN then
                        self.graft.userdata.status = FEAT_STATUS.FAILED
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end,

                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.graft.userdata.status == FEAT_STATUS.FAILED then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end
        }
    },

    stalwart =
    {
        name = "Stalwart",
        desc = "Win the challenge without falling below 50% resolve.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        
        points = 250,
        feat_icon = "UI/daily_feat_2.tex",
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),


        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.DELTA_RESOLVE ] = function( self, modifier, resolve, max_resolve, delta, source, params )
                    if modifier == self.negotiator:FindCoreArgument() then
                        if modifier.resolve < modifier.max_resolve * 0.5 then
                            self.graft.userdata.status = FEAT_STATUS.FAILED
                        end
                    end
                end,

                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.graft.userdata.status == FEAT_STATUS.FAILED then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end
        }
    },

    tall_tale = 
    {
        name = "Tall Tale",
        desc = "Play 10 negotiation cards in a single turn.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        
        points = 250,
        feat_icon = "UI/daily_feat_2.tex",
        img = engine.asset.Texture( "UI/daily_feat_2.tex", true ),

        GetProgressText = function( self )
            return loc.format( "{1}/{2}", self:GetCounter() or 0, 10 )
        end,

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.POST_RESOLVE ] = function(self, minigame, card)
                    if card.negotiator == self.negotiator then
                        self.graft:IncrementCounter()
                        if self.graft:GetCounter() >= 10 then
                            self.graft.userdata.status = FEAT_STATUS.COMPLETE
                            TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                            self.negotiator:RemoveModifier(self.id, self.stacks, self)
                        end
                    end
                end,

                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.graft.userdata.status == FEAT_STATUS.FAILED then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                    self.graft:ResetCounter()
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    -- LEVEL 3
    not_a_scratch =
    {
        name = "Not a Scratch",
        desc = "Finish the challenge with full health.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,

        points = 500,
        feat_icon = "UI/daily_feat_3.tex",
        img = engine.asset.Texture( "UI/daily_feat_3.tex", true ),

        -- battle_condition =
        -- {
        --     hidden = true,
        --     desc = "Finish the challenge with full health.",
        --     event_handlers =
        --     {
        --         [ BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
        --             if fighter == self.owner and delta > 0 then
        --                 self.graft.userdata.status = FEAT_STATUS.FAILED
        --                 TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
        --                 self.owner:RemoveCondition(self.id, 1, self)
        --             end
        --         end,
        --     },
        -- },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate, result )
                if result == GAMEOVER.VICTORY then
                    local cur, max = gamestate:GetPlayerAgent():GetHealth()
                    if cur < max then
                        self.userdata.status = FEAT_STATUS.FAILED
                    else
                        self.userdata.status = FEAT_STATUS.COMPLETE
                    end
                end
            end,
        },
    },

    back_to_basics =
    {
        name = "Back to Basics",
        desc = "Win the challenge without drafting a single card.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,

        points = 500,
        feat_icon = "UI/daily_feat_3.tex",
        img = engine.asset.Texture( "UI/daily_feat_3.tex", true ),

        event_handlers =
        {
            [ "card_added" ] = function( self, card )
                if card.rarity ~= CARD_RARITY.BASIC then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end,

            [ "GAME_OVER" ] = function( self, gamestate, result )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE and result == GAMEOVER.VICTORY then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end,
        }
    },

    reckless = 
    {
        name = "Reckless",
        desc = "Win the challenge without playing any non-attack cards in battle.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,

        points = 500,
        feat_icon = "UI/daily_feat_3.tex",
        img = engine.asset.Texture( "UI/daily_feat_3.tex", true ),

        battle_condition =
        {
            hidden = true,
            desc = "Win the challenge without playing any non-attack cards in battle.",
            event_handlers =
            {
                [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                    if card.owner == self.owner and not card:IsAttackCard() then
                        self.graft.userdata.status = FEAT_STATUS.FAILED
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.owner:RemoveCondition(self.id, 1, self)
                    end
                end,
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate, result )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE and result == GAMEOVER.VICTORY then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end,
        },

    },

    take_your_shot = 
    {
        name = "Take Your Shot",
        desc = "Play no cards on the first round of every combat and negotiation.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,

        points = 500,
        feat_icon = "UI/daily_feat_3.tex",
        img = engine.asset.Texture( "UI/daily_feat_3.tex", true ),

        battle_condition =
        {
            hidden = true,
            desc = "Play no cards on the first round of every combat and negotiation.",
            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle, card )
                    self.owner:RemoveCondition(self.id, 1, self)
                end,

                [ BATTLE_EVENT.START_RESOLVE ] = function( self, battle, card )
                    if card.owner == self.owner then
                        self.graft.userdata.status = FEAT_STATUS.FAILED
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.owner:RemoveCondition(self.id, 1, self)
                    end
                end,
            },
        },

        negotiation_modifier =
        {
            hidden = true,
            desc = "Play no cards on the first round of every combat and negotiation.",
            event_handlers =
            {
                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,

                [ EVENT.START_RESOLVE ] = function( self, minigame, card )
                    if card.owner == self.owner then
                        self.graft.userdata.status = FEAT_STATUS.FAILED
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end,
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate, result )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE and result == GAMEOVER.VICTORY then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end,
        },
    },

    clean_fight =
    {
        name = "Clean Fight",
        desc = "Win the challenge without applying any debuffs to your enemies in battle.",
        OnAdded = function(self)
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,

        points = 500,
        feat_icon = "UI/daily_feat_3.tex",
        img = engine.asset.Texture( "UI/daily_feat_3.tex", true ),

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if self.battle:GetFighterFromSource(source) == self.owner and fighter:GetTeam() ~= self.owner:GetTeam() and condition.ctype == CTYPE.DEBUFF then
                        self.graft.userdata.status = FEAT_STATUS.FAILED
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.owner:RemoveCondition(self.id, self.stacks, self)
                    end
                end,
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate, result )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE and result == GAMEOVER.VICTORY then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end,
        },
    },

    quick_work =
    {
        name = "Quick Work",
        desc = "Win a battle in a single turn.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        img = engine.asset.Texture( "UI/daily_feat_3.tex", true ),

        points = 500,
        feat_icon = "UI/daily_feat_3.tex",

        event_handlers =
        {
            [ "resolve_battle" ] = function( self, battle, primary_enemy, repercussions )
                if battle:GetTurns() <= 1 and battle:GetBattleResult() == BATTLE_RESULT.WON then
                    self.userdata.status = FEAT_STATUS.COMPLETE
                end
            end,

            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end,
        },
    },

    absolute_focus =
    {
        name = "Absolute Focus",
        desc = "Win the challenge without your core argument taking any damage.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        
        points = 500,
        feat_icon = "UI/daily_feat_3.tex",
        img = engine.asset.Texture( "UI/daily_feat_3.tex", true ),

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                    if target == self.negotiator:FindCoreArgument() and damage > defended then
                        self.graft.userdata.status = FEAT_STATUS.FAILED
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end,

                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.graft.userdata.status == FEAT_STATUS.FAILED then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate, result )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    if result == GAMEOVER.VICTORY then
                        self.userdata.status = FEAT_STATUS.COMPLETE
                    else
                        self.userdata.status = FEAT_STATUS.FAILED
                    end
                end
            end
        }
    },

    quick_wit = 
    {
        name = "Quick Wit",
        desc = "Win a negotiation in a single turn.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        
        points = 500,
        feat_icon = "UI/daily_feat_3.tex",
        img = engine.asset.Texture( "UI/daily_feat_3.tex", true ),

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.END_NEGOTIATION ] = function( self, minigame )
                    if minigame:GetResult() == minigame.RESULT.WIN and minigame:GetTurns() <= 1 then
                        self.graft.userdata.status = FEAT_STATUS.COMPLETE
                        TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },

    filibuster = 
    {
        name = "Filibuster",
        desc = "Get to turn 15 in a negotiation.",
        OnAdded = function( self )
            self.userdata.status = FEAT_STATUS.INCOMPLETE
        end,
        
        points = 500,
        feat_icon = "UI/daily_feat_3.tex",
        img = engine.asset.Texture( "UI/daily_feat_3.tex", true ),

        GetProgressText = function( self )
            return loc.format( "{1}/{2}", self:GetCounter() or 0, 15 )
        end,

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.graft.userdata.status == FEAT_STATUS.COMPLETE then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                    if negotiator == self.negotiator then
                        self.graft:IncrementCounter()
                        if self.graft:GetCounter() >= 15 then
                            self.graft.userdata.status = FEAT_STATUS.COMPLETE
                            TheGame:BroadcastEvent( "FEAT_TRIGGERED" )
                            self.negotiator:RemoveModifier(self.id, self.stacks, self)
                        end
                    end
                end
            },
        },

        event_handlers =
        {
            [ "GAME_OVER" ] = function( self, gamestate )
                if self.userdata.status == FEAT_STATUS.INCOMPLETE then
                    self.userdata.status = FEAT_STATUS.FAILED
                end
            end
        }
    },
}

---------------------------------------------------------------------------------------------

for id, graft in pairs( FEATS ) do
    graft.hidden = true
    Content.AddFeatGraft( id, graft )
end
