local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

------------------------------------------------------------------------------------------------
local METTLE_UPGRADE =
{
    {
        id = "mettle_health_upgrade",
        desc = "Increase your maximum health.",

        values = { 2, 4, 6, 8, 10 },
        upgrade_costs = { 5, 10, 20, 40, 80 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
            local health_vals = self:GetDef().values
            self.owner.health:AddStatModifier( "GRAFT_METTLE_HEALTH_UPGRADE", health_vals[math.min( #health_vals, level )] )
        end,

        DeltaUpgrade = function( self, new_level )
            local health_vals = self:GetDef().values
            local difference = health_vals[math.min( #health_vals, new_level )]  - health_vals[math.min( #health_vals, self.userdata.level )]
            self.owner.health:AddStatModifier( "GRAFT_METTLE_HEALTH_UPGRADE", difference )
            self.userdata.level = new_level
        end,
    },

     
    {
        id = "mettle_resolve_upgrade",
        desc = "Increase your maximum Resolve.",

        values = { 1, 2, 3, 4, 5, 6 },
        upgrade_costs = { 5, 10, 20, 40, 80, 120 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
            local resolve_vals = self:GetDef().values
            TheGame:GetGameState():GetCaravan():DeltaMaxResolve( resolve_vals[math.min( #resolve_vals, level )] )
        end,

        DeltaUpgrade = function( self, new_level )
            local resolve_vals = self:GetDef().values
            local difference = resolve_vals[math.min( #resolve_vals, new_level )]  - resolve_vals[math.min( #resolve_vals, self.userdata.level )]
            TheGame:GetGameState():GetCaravan():DeltaMaxResolve( difference )
            self.userdata.level = new_level
        end,
    },

     
    {
        id = "mettle_starting_shills",
        desc = "Start the game with additional Shills.",

        values = { 10, 20, 30, 40, 50 },
        upgrade_costs = { 5, 10, 20, 40, 80 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
            local vals = self:GetDef().values
            TheGame:GetGameState():GetCaravan():AddMoney(vals[math.min( #vals, level )])
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            if difference > 0 then
                TheGame:GetGameState():GetCaravan():AddMoney(difference)
            end
            self.userdata.level = new_level
        end,
    },

    
    {
        id = "mettle_battle_card_xp",
        desc = "Basic battle cards start with bonus XP.",

        values = { 1, 2, 3 },
        upgrade_costs = { 10, 25, 75 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            if difference > 0 then
                if self.owner.battler then
                    for i,card in ipairs(self.owner.battler.cards) do
                        local cur, max = card:GetXP()
                        if card.rarity == CARD_RARITY.BASIC and max > 0 and cur < max then
                            card:AddXP(vals[math.min(#vals, level)])
                        end
                    end
                end
            end
            self.userdata.level = new_level
        end,

        event_handlers = 
        {
            [ "card_added" ] = function( self, card )
                if is_instance( card, Battle.Card ) and card.owner == self.owner and card:GetMaxXP() ~= nil and card.rarity == CARD_RARITY.BASIC then
                    local vals = self:GetDef().values
                    card:AddXP(vals[math.min(#vals, self.userdata.level)], true)
                end
            end,
        },
    },

    
    {
        id = "mettle_negotiation_card_xp",
        desc = "Basic negotiation cards start with bonus XP.",

        values = { 1, 2, 3 },
        upgrade_costs = { 10, 25, 75 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            if difference > 0 then
                if self.owner.negotiator then
                    for i,card in ipairs(self.owner.negotiator.cards) do
                        local cur, max = card:GetXP()
                        if card.rarity == CARD_RARITY.BASIC and max > 0 and cur < max then
                            card:AddXP(vals[math.min(#vals, level)])
                        end
                    end
                end
            end
            self.userdata.level = new_level
        end,

        event_handlers = 
        {
            [ "card_added" ] = function( self, card )
                if is_instance( card, Negotiation.Card ) and card.owner == self.owner and card:GetMaxXP() ~= nil and card.rarity == CARD_RARITY.BASIC then
                    local vals = self:GetDef().values
                    card:AddXP(vals[math.min(#vals, self.userdata.level)], true)
                end
            end,
        },
    },

    
    {
        id = "mettle_battle_resolve",
        desc = "Recover resolve after every battle.",

        values = { 2, 3, 4, 5 },
        upgrade_costs = { 15, 30, 60, 120 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
        end,

        DeltaUpgrade = function( self, new_level )
            self.userdata.level = new_level
        end,

        event_handlers =
        {
            [ "award_loot" ] = function( self, loot, data)
                if data.battle then
                    local vals = self:GetDef().values
                    loot.resolve = (loot.resolve or 0) + vals[math.min(#vals, self.userdata.level)]
                end
            end,
        },
    },

    
    {
        id = "mettle_negotiation_health",
        desc = "Recover health after every negotiation.",

        values = { 3, 5, 6, 8 },
        upgrade_costs = { 15, 30, 60, 120 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
        end,

        DeltaUpgrade = function( self, new_level )
            self.userdata.level = new_level
        end,

        event_handlers =
        {
            [ "resolve_negotiation" ] = function( self, minigame, repercussions )
                local vals = self:GetDef().values
                repercussions.loot.health = (repercussions.loot.health or 0) + vals[math.min(#vals, self.userdata.level)]
                -- TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):Delta(vals[math.min(#vals, self.userdata.level)])
            end,
        },
    },

    
    {
        id = "mettle_battle_graft_xp",
        desc = "Battle grafts start with bonus XP.",

        values = { 1, 2, 3 },
        upgrade_costs = { 10, 30, 60 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            if difference > 0 then
                if self.owner.graft_owner then
                    for i,graft in ipairs(self.owner.graft_owner:GetGrafts(GRAFT_TYPE.COMBAT)) do
                        graft:AddXP(vals[math.min(#vals, self.userdata.level)], true)
                    end
                end
            end
            self.userdata.level = new_level
        end,

        event_handlers = 
        {
            [ "graft_added" ] = function( self, graft )
                if graft:GetType() == GRAFT_TYPE.COMBAT then
                    local vals = self:GetDef().values
                    graft:AddXP(vals[math.min(#vals, self.userdata.level)], true)
                end
            end,
        },
    },

    
    {
        id = "mettle_negotiation_graft_xp",
        desc = "Negotiation grafts start with bonus XP.",

        values = { 1, 2, 3 },
        upgrade_costs = { 10, 30, 60 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            if difference > 0 then
                if self.owner.graft_owner then
                    for i,graft in ipairs(self.owner.graft_owner:GetGrafts(GRAFT_TYPE.NEGOTIATION)) do
                        graft:AddXP(vals[math.min(#vals, self.userdata.level)], true)
                    end
                end
            end
            self.userdata.level = new_level
        end,

        event_handlers = 
        {
            [ "graft_added" ] = function( self, graft )
                if graft:GetType() == GRAFT_TYPE.NEGOTIATION then
                    local vals = self:GetDef().values
                    graft:AddXP(vals[math.min(#vals, self.userdata.level)], true)
                end
            end,
        },
    },

    
    {
        id = "mettle_battle_damage_flourish",
        desc = "Flourish gained from dealing damage in combat is increased.",

        values = {0.6, 0.7, 0.8, 1},
        upgrade_costs = { 15, 30, 60, 120 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
            local graft = self.owner.graft_owner:GetGraft("flourish_tracker")
            if graft then
                local vals = self:GetDef().values
                graft.userdata.battle_gains = graft.userdata.battle_gains or {}
                graft.userdata.battle_gains["DAMAGE_DEALT"] = vals[math.min( #vals, self.userdata.level )]
            end
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            self.userdata.level = new_level
            if difference > 0 then
                local graft = self.owner.graft_owner:GetGraft("flourish_tracker")
                if graft then
                    local vals = self:GetDef().values
                    graft.userdata.battle_gains = graft.userdata.battle_gains or {}
                    graft.userdata.battle_gains["DAMAGE_DEALT"] = vals[math.min( #vals, self.userdata.level )]
                end
            end
        end,

        event_handlers = 
        {
            [ "graft_added" ] = function( self, graft )
                if graft.id == "flourish_tracker" then
                    local vals = self:GetDef().values
                    graft.userdata.battle_gains = graft.userdata.battle_gains or {}
                    graft.userdata.battle_gains["DAMAGE_DEALT"] = vals[math.min( #vals, self.userdata.level )]
                end
            end,
        },
    },

    
    {
    	id = "mettle_battle_defend_flourish",
        desc = "Flourish gained from receiving damage in combat is increased.",

        values = {0.6, 0.7, 0.8, 1},
        upgrade_costs = { 15, 30, 60, 120 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
            local graft = self.owner.graft_owner:GetGraft("flourish_tracker")
            if graft then
                local vals = self:GetDef().values
                graft.userdata.battle_gains = graft.userdata.battle_gains or {}
                graft.userdata.battle_gains["DEFENDED"] = vals[math.min( #vals, self.userdata.level )]
            end
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            self.userdata.level = new_level
            if difference > 0 then
                local graft = self.owner.graft_owner:GetGraft("flourish_tracker")
                if graft then
                    local vals = self:GetDef().values
                    graft.userdata.battle_gains = graft.userdata.battle_gains or {}
                    graft.userdata.battle_gains["DEFENDED"] = vals[math.min( #vals, self.userdata.level )]
                end
            end
        end,

        event_handlers = 
        {
            [ "graft_added" ] = function( self, graft )
                if graft.id == "flourish_tracker" then
                    local vals = self:GetDef().values
                    graft.userdata.battle_gains = graft.userdata.battle_gains or {}
                    graft.userdata.battle_gains["DEFENDED"] = vals[math.min( #vals, self.userdata.level )]
                end
            end,
        },
    },

    
    {
        id = "mettle_negotiation_damage_flourish",
        desc = "Flourish gained from dealing damage in negotiation is increased.",

        values = {0.6, 0.7, 0.8, 1},
        upgrade_costs = { 15, 30, 60, 120 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
            local graft = self.owner.graft_owner:GetGraft("flourish_tracker")
            if graft then
                local vals = self:GetDef().values
                graft.userdata.negotiation_gains = graft.userdata.negotiation_gains or {}
                graft.userdata.negotiation_gains["DAMAGE_DEALT"] = vals[math.min( #vals, self.userdata.level )]
            end
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            self.userdata.level = new_level
            if difference > 0 then
                local graft = self.owner.graft_owner:GetGraft("flourish_tracker")
                if graft then
                    local vals = self:GetDef().values
                    graft.userdata.negotiation_gains = graft.userdata.negotiation_gains or {}
                    graft.userdata.negotiation_gains["DAMAGE_DEALT"] = vals[math.min( #vals, self.userdata.level )]
                end
            end
        end,

        event_handlers = 
        {
            [ "graft_added" ] = function( self, graft )
                if graft.id == "flourish_tracker" then
                    local vals = self:GetDef().values
                    graft.userdata.negotiation_gains = graft.userdata.negotiation_gains or {}
                    graft.userdata.negotiation_gains["DAMAGE_DEALT"] = vals[math.min( #vals, self.userdata.level )]
                end
            end,
        },
    },

    
    {
        id = "mettle_negotiation_mitigation_flourish",
        desc = "Flourish gained from mitigating damage in negotiation is increased.",

        values = {0.6, 0.7, 0.8, 1},
        upgrade_costs = { 15, 30, 60, 120 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
            local graft = self.owner.graft_owner:GetGraft("flourish_tracker")
            if graft then
                local vals = self:GetDef().values
                graft.userdata.negotiation_gains = graft.userdata.negotiation_gains or {}
                graft.userdata.negotiation_gains["MITIGATED"] = vals[math.min( #vals, self.userdata.level )]
            end
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            self.userdata.level = new_level
            if difference > 0 then
                local graft = self.owner.graft_owner:GetGraft("flourish_tracker")
                if graft then
                    local vals = self:GetDef().values
                    graft.userdata.negotiation_gains = graft.userdata.negotiation_gains or {}
                    graft.userdata.negotiation_gains["MITIGATED"] = vals[math.min( #vals, self.userdata.level )]
                end
            end
        end,

        event_handlers = 
        {
            [ "graft_added" ] = function( self, graft )
                if graft.id == "flourish_tracker" then
                    local vals = self:GetDef().values
                    graft.userdata.negotiation_gains = graft.userdata.negotiation_gains or {}
                    graft.userdata.negotiation_gains["MITIGATED"] = vals[math.min( #vals, self.userdata.level )]
                end
            end,
        },
    },

    
    {
        id = "mettle_card_skip",
        desc = "Skipping card drafts grants additional shills.",
        values = { 5, 10, 20 },
        upgrade_costs = { 25, 50, 100 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
        end,

        DeltaUpgrade = function( self, new_level )
            self.userdata.level = new_level
        end,

        event_handlers = 
        {
            [ "calc_card_skip_reward" ] = function( self, card_skip_details )
                local vals = self:GetDef().values
                card_skip_details.money = (card_skip_details.money or 0) + vals[math.min( #vals, self.userdata.level )]
            end,
        },
    },

    
    {
        id = "mettle_pet_health",
        desc = "Increase the max health of pets.",
        values = { 5, 10, 15 },
        upgrade_costs = { 30, 75, 150 },

        OnApply = function( self )
            self.userdata.affected = {}
        end,

        ApplyUpgrade = function( self, level )
            local vals = self:GetDef().values
            self.userdata.level = level
            self.userdata.affected = self.userdata.affected or {}
            for k,agent in pairs(TheGame:GetGameState():GetCaravan():GetPets()) do
                if self.userdata.affected == nil or self.userdata.affected[agent] == nil then
                    local health = agent:GetAspect("health")
                    agent:GetAspect("health"):AddStatModifier( "mettle_pet_health", vals[math.min( #vals, self.userdata.level )])
                    self.userdata.affected[agent] = level
                elseif self.userdata.affected[agent] and self.userdata.affected[agent] < level then
                    local dif = vals[math.min( #vals, level )] - vals[math.min(#vals, self.userdata.affected[agent])]
                    agent:GetAspect("health"):AddStatModifier( "mettle_pet_health", dif)
                    self.userdata.affected[agent] = level
                end
            end
        end,

        DeltaUpgrade = function( self, new_level )
            local vals = self:GetDef().values
            local difference = vals[math.min( #vals, new_level )]  - vals[math.min( #vals, self.userdata.level )]
            self.userdata.level = new_level
            if difference > 0 then
                self.userdata.affected = self.userdata.affected or {}
                for k,agent in pairs(TheGame:GetGameState():GetCaravan():GetPets()) do
                    if self.userdata.affected == nil or self.userdata.affected[agent] == nil then
                        agent:GetAspect("health"):AddStatModifier( "mettle_pet_health", vals[math.min( #vals, self.userdata.level )])
                        self.userdata.affected[agent] = new_level
                    elseif self.userdata.affected[agent] and self.userdata.affected[agent] < new_level then
                        local dif = vals[math.min( #vals, new_level )] - vals[math.min(#vals, self.userdata.affected[agent])]
                        agent:GetAspect("health"):AddStatModifier( "mettle_pet_health", dif)
                        self.userdata.affected[agent] = new_level
                    end
                end
            end
        end,

        event_handlers = 
        {
            [ "caravan_member_add" ] = function( self, agent )
                if agent:IsPet() then
                    local vals = self:GetDef().values
                    if self.userdata.affected == nil or self.userdata.affected[agent] == nil then
                        agent:GetAspect("health"):AddStatModifier( "mettle_pet_health", vals[math.min( #vals, self.userdata.level )])
                        self.userdata.affected[agent] = self.userdata.level
                    elseif self.userdata.affected[agent] and self.userdata.affected[agent] < self.userdata.level then
                        local dif = vals[math.min( #vals, self.userdata.level )] - vals[math.min(#vals, self.userdata.affected[agent])]
                        agent:GetAspect("health"):AddStatModifier( "mettle_pet_health", dif)
                        self.userdata.affected[agent] = self.userdata.level
                    end
                end
            end
        },
    },

    
    {
        id = "mettle_eat_bonus",
        desc = "Restore additional health when eating.",
        values = { 5, 8, 11, 15 },
        upgrade_costs = { 20, 40, 80, 150 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
        end,

        DeltaUpgrade = function( self, new_level )
            self.userdata.level = new_level
        end,

        event_handlers =
        {
            ["do_eat"] = function( self, eat_effects )
                local vals = self:GetDef().values
                eat_effects.health_gain = (eat_effects.health_gain or 0) + vals[math.min(#vals, self.userdata.level)]
            end,
        },
    },

    
    {
        id = "mettle_drink_bonus",
        desc = "Restore additional resolve when drinking.",
        values = { 4, 6, 8, 10 },
        upgrade_costs = { 20, 40, 80, 150 },

        ApplyUpgrade = function( self, level )
            self.userdata.level = level
        end,

        DeltaUpgrade = function( self, new_level )
            self.userdata.level = new_level
        end,

        event_handlers =
        {
            ["do_drink"] = function( self, drink_effects )
                local vals = self:GetDef().values
                drink_effects.resolve_gain = (drink_effects.resolve_gain or 0) + vals[math.min(#vals, self.userdata.level)]
            end,
        },
    },

}

---------------------------------------------------------------------------------------------

for i, graft in ipairs( METTLE_UPGRADE ) do
    local id = graft.id
    graft.hidden = true
    graft.suppressed = true
    graft.sort_key = i
    assert_warning( graft.ApplyUpgrade ~= nil, "Missing ApplyUpgrade function on: %s", id )
    assert_warning( graft.DeltaUpgrade ~= nil, "Missing DeltaUpgrade function on: %s", id )
    assert(#graft.values == #graft.upgrade_costs, loc.format("Incorrect number of values or upgrade costs on {1}", id))
    Content.AddMettleGraft( id, graft )
    
end

