require "ui/screens/DraftMulliganPopup"
require "ui/screens/griddraftpopup"

local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

------------------------------------------------------------------------------------------------

local MUTATORS =
{
    bare_mettle =
    {
        name = "Bare Mettle",
        desc = "Mettle Upgrades do not apply, and you gain no mettle!",
    },

    roll_with_it =
    {
        name = "Roll With It",
        desc = "You can no longer skip card drafts.",

        event_handlers = 
        {
            [ "card_draft" ] = function( self, draft_options )
                draft_options.no_skip = true
            end
        },
    },

    charisma =
    {
        name = "Charisma",
        desc = "Draft a starting negotiation deck of 10 cards.",

        exclusion_ids = { "picky_draft_negotiation", "havarian_mulligan_negotiation", "negotiation_bundle" },

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetPlayerAgent().negotiator:ClearCards()

                local coro = coroutine.running()
                local function OnDone()
                    coroutine.resume( coro )
                end
                local draft_count = 10
                for i = 1, draft_count do
                    local cards = RewardUtil.GetNegotiationCards( 1, 3, gamestate:GetPlayerAgent() )
                    local draft_popup = Screen.DraftChoicePopup()
                    local title = loc.format( LOC "UI.DRAFT_SCREEN.DRAFT_CARDS", i, draft_count )
                    draft_popup:DraftCards( gamestate:GetPlayerAgent(), Negotiation.Card, cards, OnDone, title, nil, true )
                    TheGame:FE():InsertScreen( draft_popup )

                    coroutine.yield()
                end
            end
        }
    },

    parasitology = 
    {
        name = "Parasitology",
        desc = "Start the run infected with bog parasites.",
        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetPlayerAgent().battler:LearnCard("twig")
                gamestate:GetPlayerAgent().negotiator:LearnCard("stem")
            end
        }
    },

    veteran =
    {
        name = "Veteran",
        desc = "Draft a starting battle deck of 10 cards.",

        exclusion_ids = { "havarian_mulligan_battle", "picky_draft_battle", "battle_bundle" },

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetPlayerAgent().battler:ClearCards()

                local coro = coroutine.running()
                local function OnDone()
                    coroutine.resume( coro )
                end
                local draft_count = 10
                for i = 1, draft_count do
                    local cards = RewardUtil.GetBattleCards( 1, 3, gamestate:GetPlayerAgent() )
                    local draft_popup = Screen.DraftChoicePopup()
                    local title = loc.format( LOC "UI.DRAFT_SCREEN.DRAFT_CARDS", i, draft_count )
                    draft_popup:DraftCards( gamestate:GetPlayerAgent(), Battle.Card, cards, OnDone, title, nil, true )
                    TheGame:FE():InsertScreen( draft_popup )

                    coroutine.yield()
                end
            end
        }
    },

    havarian_mulligan_battle =
    {
        name = "Havarian Mulligan (Battle)",
        desc = "Draft two sets of 5 cards each for your starting battle deck.",

        exclusion_ids = { "veteran", "picky_draft_battle", "battle_bundle" },

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetPlayerAgent().battler:ClearCards()

                local coro = coroutine.running()
                local function OnDone()
                    coroutine.resume( coro )
                end
                local difficulty = gamestate:GetCurrentBaseDifficulty()
                local rng = gamestate:GetRNG( "CARDS" )
                local draft_count = 2
                for i = 1, draft_count do
                    local mulligans = 2

                    local cards = RewardUtil.GetBattleCards( difficulty, 5 * (mulligans + 1), gamestate:GetPlayerAgent() )

                    local title = loc.format( LOC "UI.DRAFT_SCREEN.DRAFT_MULLIGAN_TITLE", i, draft_count )
                    local draft_popup = Screen.DraftMulliganPopup( gamestate:GetPlayerAgent(), cards, mulligans, OnDone, title, true )
                    draft_popup:DraftCards( 5 )
                    TheGame:FE():InsertScreen( draft_popup )

                    coroutine.yield()
                end
            end
        }
    },

    havarian_mulligan_negotiation =
    {
        name = "Havarian Mulligan (Negotiation)",
        desc = "Draft two sets of 5 cards each for your starting negotiation deck.",

        exclusion_ids = { "picky_draft_negotiation", "charisma", "negotiation_bundle" },

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetPlayerAgent().negotiator:ClearCards()

                local coro = coroutine.running()
                local function OnDone()
                    coroutine.resume( coro )
                end
                local difficulty = gamestate:GetCurrentBaseDifficulty()
                local rng = gamestate:GetRNG( "CARDS" )
                local draft_count = 2
                for i = 1, draft_count do
                    local mulligans = 2

                    if rng then
                        push_random( function( n, m )
                            print( n, m )
                            return rng:Random( n, m )
                        end )
                    end

                    local cards = RewardUtil.GetNegotiationCards( difficulty, 5 * (mulligans + 1), gamestate:GetPlayerAgent() )

                    if rng then
                        pop_random()
                    end

                    local title = loc.format( LOC "UI.DRAFT_SCREEN.DRAFT_MULLIGAN_TITLE", i, draft_count )
                    local draft_popup = Screen.DraftMulliganPopup( gamestate:GetPlayerAgent(), cards, mulligans, OnDone, title, true )
                    draft_popup:DraftCards( 5 )
                    TheGame:FE():InsertScreen( draft_popup )

                    coroutine.yield()
                end
            end
        }
    },

    picky_draft_battle =
    {
        name = "Picky Battler",
        desc = "Draft a selection from a large grid of cards to augment your starting battle deck.",

        exclusion_ids = { "veteran", "havarian_mulligan_battle", "battle_bundle" },

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                -- gamestate:GetPlayerAgent().battler:ClearCards()

                local coro = coroutine.running()
                local function OnDone()
                    coroutine.resume( coro )
                end

                local difficulty = gamestate:GetCurrentBaseDifficulty()
                local rng = gamestate:GetRNG( "CARDS" )

                local screen = Screen.GridDraftPopup()
                local num_drafts, min_pick, max_pick, choices = 1, 5, 8, 14

                TheGame:FE():InsertScreen(screen)

                screen:DoDraft( gamestate:GetPlayerAgent(), Battle.Card, difficulty, num_drafts, min_pick, max_pick, choices, OnDone, rng)
                coroutine.yield()
            end
        }
    },    

    picky_draft_negotiation =
    {
        name = "Picky Negotiator",
        desc = "Draft a selection from a large grid of cards to augment your starting negotiation deck.",

        exclusion_ids = { "charisma", "havarian_mulligan_negotiation", "negotiation_bundle" },

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                -- gamestate:GetPlayerAgent().negotiator:ClearCards()

                local coro = coroutine.running()
                local function OnDone()
                    coroutine.resume( coro )
                end

                local difficulty = gamestate:GetCurrentBaseDifficulty()
                local rng = gamestate:GetRNG( "CARDS" )

                local screen = Screen.GridDraftPopup()
                local num_drafts, min_pick, max_pick, choices = 1, 5, 8, 14

                TheGame:FE():InsertScreen(screen)

                screen:DoDraft( gamestate:GetPlayerAgent(), Negotiation.Card, difficulty, num_drafts, min_pick, max_pick, choices, OnDone, rng)
                coroutine.yield()
            end
        }
    },    

    battle_bundle =
    {
        name = "Battle Bundle",
        desc = "Draft 3 packs of battle cards to augment your starting deck, then remove 3 cards from your deck.",
        series = "SAL",

        exclusion_ids = { "veteran", "havarian_mulligan_battle", "picky_draft_battle" },

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )

                local coro = coroutine.running()
                local function OnDone()
                    coroutine.resume( coro )
                end

                for i = 1, 3 do
                    local subtitle_text = loc.format( LOC"UI.DRAFT_SCREEN.DRAFT_PACKS", i, 3 )
                    local draft_popup = Screen.DraftPackPopup()
                    local difficulty = gamestate:GetCurrentBaseDifficulty()
                    draft_popup:DraftPacks( gamestate:GetPlayerAgent(), Battle.Card, difficulty, OnDone, subtitle_text, gamestate:GetRNG( "CARDS" ))
                    TheGame:FE():InsertScreen( draft_popup )

                    coroutine.yield()
                end
                for i=1,3 do
                    AgentUtil.RemoveBattleCard( gamestate:GetPlayerAgent(), OnDone)
                    coroutine.yield()
                end
            end
        }
    },

    negotiation_bundle =
    {
        name = "Negotiation Bundle",
        desc = "Draft 3 packs of negotiation cards to augment your starting deck, then remove 3 cards from your deck.",
        series = "SAL",

        exclusion_ids = { "charisma", "havarian_mulligan_negotiation", "picky_draft_negotiation" },

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )

                local coro = coroutine.running()
                local function OnDone()
                    coroutine.resume( coro )
                end

                for i = 1, 3 do
                    local subtitle_text = loc.format( LOC"UI.DRAFT_SCREEN.DRAFT_PACKS", i, 3 )
                    local draft_popup = Screen.DraftPackPopup()
                    local difficulty = gamestate:GetCurrentBaseDifficulty()
                    draft_popup:DraftPacks( gamestate:GetPlayerAgent(), Negotiation.Card, difficulty, OnDone, subtitle_text, gamestate:GetRNG( "CARDS" ))
                    TheGame:FE():InsertScreen( draft_popup )

                    coroutine.yield()
                end

                for i=1,3 do
                    AgentUtil.RemoveNegotiationCard( gamestate:GetPlayerAgent(), OnDone)
                    coroutine.yield()
                end
            end
        }
    },

    insomnia =
    {
        name = "Insomnia",
        desc = "Sleeping no longer restores health, resolve, or removes cards.",

        event_priorities =
        {
            [ "do_sleep" ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers =
        {
            [ "do_sleep" ] = function( self, player, sleep_data )
                sleep_data.health_gain = 0
                sleep_data.resolve_gain = 0
                table.clear( sleep_data.cards_removed )
            end,
        }
    },

    -- Grafts can't tooltip other grafts yet. *facepalm*
    -- dynamic =
    -- {
    --     name = "Dynamic",
    --     desc = "Start with {dynamo}.",

    --     OnAdded = function( self, agent )
    --         agent.graft_owner:AddGraft( GraftInstance( "dynamo", agent ))
    --     end,
    -- },

    elimination =
    {
        name = "Elimination",
        desc = "Removing cards at shops can be done any number of times.",

        event_handlers =
        {
            [ "pre_remove_card" ] = function( self, remove_data )
                remove_data.agent:Forget( "BATTLE_CARD_REMOVED" )
                remove_data.agent:Forget( "NEGOTIATION_CARD_REMOVED" )
            end,
        }        
    },

    first_impressions =
    {
        name = "First Impressions",
        desc = "Receive a random relationship each time you meet someone for the first time.",

        exclusion_ids = { "relatable" },

        event_priorities =
        {
            [ "start_convo" ] = 10, -- Prioritized before first_impressions
        },

        event_handlers =
        {
            [ "start_convo" ] = function( self, agent )
                if agent:IsSentient() and not agent:KnowsPlayer() and not AgentUtil.HasPlotArmour( agent ) then
                    --
                    agent:OpinionEvent( OPINION.FIRST_IMPRESSIONS )
                    if agent:GetRelationship() == RELATIONSHIP.NEUTRAL then
                        local txt = loc.format( LOC "NOTIFICATION.FIRST_IMPRESSIONS", agent )
                        TheGame:GetGameState():LogNotification( NOTIFY.AGENT_MSG, agent, txt )
                    end
                end
            end,
        }
    },

    juggernaut =
    {
        name = "Juggernaut",
        desc = "All fighters gain 1 {battle.POWER} at the end of their turn.",

        OnActivateFighter = function(self, fighter)
            self:AddBattleCondition(fighter)
        end,

        battle_condition = 
        {
            hidden = false,

            event_handlers =
            {
                [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                    fighter:AddCondition( "POWER", 1, self )
                end,
            },
        },
    },

    -- hollow =
    -- {
    --     name = "Hollow",
    --     desc = "You no longer restore resolve after battle.",

    --     event_handlers =
    --     {
    --         [ "award_loot" ] = function( self, loot, data )
    --             if data.battle then
    --                 loot.resolve = nil
    --             end
    --         end,
    --     }
    -- },

    starved =
    {
        name = "Starved",
        desc = "Battle and negotiation no longer offer card rewards.",

        event_handlers =
        {
            [ "award_loot" ] = function( self, loot, data )
                loot.battle_card_draft_difficulty = nil
                loot.negotiation_card_draft_difficulty = nil
            end,
        }
    },

    barbarous =
    {
        name = "Barbarous",
        desc = "There are no repercussions for murder.",

        event_handlers =
        {
            [ "resolve_battle" ] = function( self, battle, primary_enemy, repercussions )
                table.clear( repercussions.murders )
            end,

            [ "commit_murder" ] = function( self, murder )
                murder.friend = nil
            end,
        }
    },

    why_me =
    {
        name = "Why Me",
        desc = "Every enemy has {battle.VENDETTA}.",

        OnActivateFighter = function(self, fighter)
            self:AddBattleCondition(fighter)
        end,

        battle_condition = 
        {
            hidden = true,

            event_handlers =
            {
                [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                    if fighter:GetTeamID() == TEAM.RED then
                        fighter:AddCondition( "VENDETTA" )
                    end
                end,
            },
        },
    },

    mutator =
    {
        name = "Mutator",
        desc = "When you upgrade a card, choose from two random cards instead.",

        event_handlers =
        {
            [ "roll_upgrades" ] = function( self, card )
                if card.owner == self.owner and card:HasUpgrades() then
                    local card_defs
                    local function ExcludeThis( def )
                        return def ~= card.def
                    end
                    local rng = TheGame:GetGameState():GetRNG("CARD_UPGRADES")
                    if is_instance( card, Negotiation.Card ) then
                        if rng then
                            card_defs = NegotiationCardCollection.AllRewardableCards( ExcludeThis, card.owner ):SeededPick(2, rng)
                        else
                            card_defs = NegotiationCardCollection.AllRewardableCards( ExcludeThis, card.owner ):Pick( 2 )
                        end
                    elseif is_instance( card, Battle.Card ) then
                        if rng then
                            card_defs = BattleCardCollection.AllRewardableCards( ExcludeThis, card.owner ):SeededPick( 2, rng )
                        else
                            card_defs = BattleCardCollection.AllRewardableCards( ExcludeThis, card.owner ):Pick( 2 )
                        end
                    end

                    if card_defs and #card_defs == 2 then
                        card.userdata.upgrade_ids = {}
                        for i, def in ipairs( card_defs ) do
                            -- Sometimes, assign an upgrade.
                            local card_id
                            if math.random() < 0.25 and def.upgrade_ids then
                                card_id = table.arraypick( def.upgrade_ids )
                            else
                                card_id = def.id
                            end
                            table.insert( card.userdata.upgrade_ids, card_id )
                        end
                        card.wild = true
                    end
                end
            end
        }
    },

    relatable =
    {
        name = "Relatable",
        desc = "Everyone you meet for the first time will dislike you.",

        exclusion_ids = { "first_impressions" },

        event_handlers =
        {
            [ "start_convo" ] = function( self, agent )
                if not agent:KnowsPlayer() then
                    agent:OpinionEvent( OPINION.RELATABLE )
                end
            end,
        }
    },

    loyalty =
    {
        name = "Loyalty",
        desc = "Hired allies no longer leave your party.",

        -- See: hired_leave_chat.lua.
    },

    giant_mechs =
    {
        name = "Giant Mechs",
        desc = "Mechs are bigger, have more health, and do more damage.",

        OnActivateFighter = function(self, fighter)
            self:AddBattleCondition(fighter)
        end,

        battle_condition = 
        {
            hidden = true,

            event_handlers =
            {
                [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                    if fighter.agent:GetSpecies() == SPECIES.MECH then
                        local delta = math.ceil( fighter:GetMaxHealth() * 0.5 )
                        fighter:DeltaMaxHealth( delta )
                        fighter:DeltaHealth( delta )
                        fighter:SetAnimScale( fighter:GetAnimScale() * 1.25 )
                    end
                end,

                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner.agent:GetSpecies() == SPECIES.MECH then
                        dmgt:AddDamage(
                            math.floor( dmgt.min_damage * 0.5 ),
                            math.floor( dmgt.max_damage * 0.5 ),
                            self )
                    end
                end
            },
        },
    },

    animal_lover =
    {
        name = "Animal Lover",
        desc = "Own up to 3 pets.",
        
        event_handlers =
        {
            [ "calc_max_pets" ] = function( self, pet_details )
                pet_details.max_pets = 3
            end,
        }
    },

    pet_owner =
    {
        name = "Pet Owner",
        desc = "You start the run with a random pet.",
        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )


                local possible_pets = {"FLEAD_UPGRADED", "ERCHIN_UPGRADED", "CRAYOTE_UPGRADED", "VROC_UPGRADED", "SHROOGLET_UPGRADED", "LUMICYTE_UPGRADED"}
                local pet_def = possible_pets[gamestate:SeededRandom(#possible_pets)]
                local pet = gamestate:AddAgent(Agent(pet_def))
                pet:GenerateRandomName()
                pet:Recruit(PARTY_MEMBER_TYPE.CREW)

            end
        }        
    },


    toolbox =
    {
        name = "Tool Box",
        desc = "Draft two item cards at the start of the game. They have 1 additional charge.",

        event_priorities =
        {
            [ "new_game" ] = 99, -- AFTER Veteran.
        },

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                local coro = coroutine.running()
                local function OnDone()
                    coroutine.resume( coro )
                end

                local cards = BattleCardCollection.AllItems()
                local ncards = NegotiationCardCollection.AllItems()
                cards:AddItems( ncards.items )

                local draft_count = 2
                for i = 1, draft_count do
                    local picks = cards:Pick( 3 )

                    for i, def in ipairs( picks ) do
                        local card
                        if Content.GetNegotiationCard( def.id ) then
                            card = Negotiation.Card( def.id, gamestate:GetPlayerAgent() )
                        else
                            card = Battle.Card( def.id, gamestate:GetPlayerAgent() )
                        end
                        if card.userdata.max_charges or card.max_charges ~= nil then
                            card.userdata.max_charges = (card.userdata.max_charges or card.max_charges) + 1
                            card:RestoreCharges( 1 )
                        end
                        picks[ i ] = card
                    end

                    local draft_popup = Screen.DraftChoicePopup()
                    local title = loc.format( LOC "UI.DRAFT_SCREEN.DRAFT_CARDS", i, draft_count )
                    draft_popup:DraftCards( gamestate:GetPlayerAgent(), nil, picks, OnDone, title, nil, true )
                    TheGame:FE():InsertScreen( draft_popup )

                    coroutine.yield()
                end
            end
        }
    },

    unnatural_growth =
    {
        name = "Unnatural Growth",
        desc = "All cards require a random amount of experience to upgrade.",

        exclusion_ids = {"brilliance"},

        event_handlers =
        {
            [ "roll_upgrades" ] = function( self, card )
                if card.owner == self.owner and card:GetMaxXP() ~= nil then
                    card.userdata.max_xp = math.random( 1, 5 ) + math.random( 0, 15 )
                end
            end
        }
    },

    land_of_plenty =
    {
        name = "Land of Plenty",
        desc = "After every battle and negotiation, draft both a negotiation card and a battle card.",

        event_handlers =
        {
            [ "award_loot" ] = function( self, loot, data )
                local diff = loot.battle_card_draft_difficulty or loot.negotiation_card_draft_difficulty
                loot.battle_card_draft_difficulty = loot.battle_card_draft_difficulty or diff
                loot.negotiation_card_draft_difficulty = loot.negotiation_card_draft_difficulty or diff
            end,
        },
    },

    contaminated_equipment = 
    {
        name = "Contaminated Equipment",
        desc = "Whenever you install a graft, gain {exhaustion} and {mental_block}.",

        event_handlers =
        {
            [ "graft_added" ] = function( self, graft )
                if graft:GetType() == GRAFT_TYPE.NEGOTIATION or graft:GetType() == GRAFT_TYPE.COMBAT then
                    local player = TheGame:GetGameState():GetPlayerAgent()
                    player.negotiator:LearnCard("mental_block")
                    player.battler:LearnCard("exhaustion")
                end
            end,
        }
    },

    mental_overload = 
    {
        name = "Mental Overload",
        desc = "You only discard <b>Unplayable</> cards at the end of your turn. Affects both combat and negotiations.",

        battle_condition = 
        {
            hidden = true,

            event_priorities =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = 999,
            },
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    battle.no_discard = true
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    local to_discard = {}
                    for i,card in battle:GetHandDeck():Cards() do
                        if card:IsFlagged(battle_defs.CARD_FLAGS.UNPLAYABLE) then
                            table.insert(to_discard, card)
                        end
                    end
                    for i,card in ipairs(to_discard) do
                        card:TransferCard(battle:GetDiscardDeck())
                    end
                end,
            }
        },

        negotiation_modifier = 
        {
            hidden = true,
            event_priorities =
            {
                [ EVENT.END_PLAYER_TURN ] = 999,
            },
            event_handlers = 
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    minigame.no_discard = true
                end,

                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    local to_discard = {}
                    for i,card in minigame:GetHandDeck():Cards() do
                        if card:IsFlagged(negotiation_defs.CARD_FLAGS.UNPLAYABLE) then
                            table.insert(to_discard, card)
                        end
                    end
                    for i,card in ipairs(to_discard) do
                        card:TransferCard(minigame:GetDiscardDeck())
                    end
                end,
            }
        },
    },

    brilliance =
    {
        name = "Brilliance",
        desc = "The max XP of all cards and grafts is reduced by 50%.",

        exclusion_ids = {"unnatural_growth"},

        event_handlers =
        {
            [ "roll_upgrades" ] = function( self, card )
                if card.owner == self.owner and card:GetMaxXP() ~= nil then
                    card.userdata.max_xp = math.round(card:GetMaxXP() * 0.5)
                end
            end,

            [ "roll_graft_upgrades" ] = function( self, graft )
                if graft.userdata.max_xp then
                    graft.userdata.max_xp = math.round(graft.userdata.max_xp * 0.5)
                end
            end,
        }
    },

    daily_boss_health =
    {
        name = "Reduced Boss Health",
        desc = "All bosses have less maximum health.",
        hidden = true,
        restricted = true,

        battle_condition =
        {
            hidden = true,
            percent_per_difficulty = {0.3, 0.2, 0.1},
            event_handlers =
            {
                [ BATTLE_EVENT.ACTIVATED_FIGHTER ] = function( self, fighter )
                    -- really gross exception but it works...
                    if fighter.agent.boss and fighter.agent:GetContentID() ~= "HOLO_KASHIO" then
                        local delta = math.ceil( fighter:GetMaxHealth() * self.percent_per_difficulty[QuestUtil.GetDifficultyFromPrestige()] )
                        fighter:DeltaMaxHealth( -delta )
                    end
                end,
            },
        },
    },

    score_card_mutator =
    {
        name = "Score Cards",
        desc = "At the start of the game, gain a random Score card.",
        restricted = true,
        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                local options = BattleCardCollection():Filter( function( card_def ) return CheckBits( card_def.flags, battle_defs.CARD_FLAGS.SCORE ) and not CheckBits( card_def.flags, battle_defs.CARD_FLAGS.ITEM )  end ).items
                local rng = gamestate:GetRNG("CARDS")
                local card = TheGame:GetGameState():GetPlayerAgent().battler:LearnCard(options[rng:Random(#options)].id)
                UIHelpers.ShowGainCards({card})
            end
        }
    },

    keepsake_mutator =
    {
        name = "Playing for Keeps",
        desc = "At the start of the game, gain a random {battle.KEEPSAKE} card.",
        restricted = true,
        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                local options = BattleCardCollection():Filter( function( card_def ) return CheckBits( card_def.flags, battle_defs.CARD_FLAGS.SCORE ) and card_def.base_points ~= nil end ).items
                local rng = gamestate:GetRNG("CARDS")
                local card = TheGame:GetGameState():GetPlayerAgent().battler:LearnCard(options[rng:Random(#options)].id)
                UIHelpers.ShowGainCards({card})
            end
        }
    },

    on_your_toes =
    {
        name = "On Your Toes",
        desc = "At the end of each battle, remove 3 cards from your deck and draft 3 new ones.",
        event_handlers =
        {
            [ "award_loot" ] = function( self, loot, data )
                loot.battle_card_draft_difficulty = nil
                loot.battle_card_swap = 3
                loot.battle_card_swap_difficulty = TheGame:GetGameState():GetCurrentBaseDifficulty()
            end,
        }
    },

    multicard =
    {
        name = "Multicard",
        desc = "Whenever you draft a card, add two additional copies of it to your deck.",
        event_handlers =
        {
            [ "card_added" ] = function( self, card, is_draft )
                if is_draft then
                    local copies = {}
                    if is_instance(card, Battle.Card) then
                        for i=1,2 do
                            local copy = card:Duplicate()
                            copy.temporary = false
                            self.owner.battler:AddCard( copy, false )
                            table.insert( copies, copy )
                        end
                    else
                        for i=1,2 do
                            local copy = card:Duplicate()
                            copy.temporary = false
                            self.owner.negotiator:AddCard( copy, false )
                            table.insert( copies, copy )
                        end
                    end
                    UIHelpers.ShowGainCards( copies )
                end
            end,
        }
    },

    resourceful = 
    {
        name = "Resourceful",
        desc = "At the start of each battle, choose any number of cards in your hand and permanently replace them with random cards of the same or higher quality.",

        battle_condition =
        {
            hidden = true,

            loc_strings =
            {
                CHOOSE_RESOURCEFUL_CARDS = "Choose any number of cards to replace",
            },

            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local cards = battle:ChooseCardsFromTable( battle:GetHandDeck(), nil, nil, nil, self:GetLocalizedString("CHOOSE_RESOURCEFUL_CARDS"))
                    if cards then
                        for i,card in ipairs(cards) do
                            card:TransferCard( battle.trash_deck )
                            card:Consume()
                        end
                        local new_cards = {}
                        for i=1,#cards do
                            local new_card = BattleCardCollection.AllRewardableCards(function(cd) 
                                return cd.rarity == cd.rarity == cards[i].rarity or cd.rarity == CARD_RARITY_ORDER[math.min(4, CARD_RARITY_ORDINALS[cards[i].rarity] + 2)] 
                            end, self.owner.agent):Pick(1)[1]
                            if new_card then
                                new_card = Battle.Card(new_card.id, self.owner.agent)
                                self.owner.agent.battler:AddCard(new_card)
                                local clone = new_card:Clone()
                                clone.show_dealt = false
                                clone.owner = self.owner
                                table.insert(new_cards, clone)
                            end
                        end
                        battle:DealCards(new_cards, battle:GetHandDeck())
                    end

                    self.owner:RemoveCondition(self.id, self.stacks, self)
                end
            },
        },
    },

    precision =
    {
        name = "Precision",
        desc = "Whenever you play a card add a copy of it to your discard pile. At the end of your turn, Expend any cards in your hand.",
        battle_condition =
        {
            hidden = true,
            event_priorities =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = 999
            },
            event_handlers =
            {
                [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, card )
                    if card.owner == self.owner then
                        local clone = card:Duplicate()
                        clone.show_dealt = false
                        battle:DealCard(clone, battle:GetDiscardDeck())
                    end
                end,

                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    local cards_to_expend = shallowcopy(battle:GetHandDeck().cards)
                    for i,card in ipairs(cards_to_expend) do
                        battle:ExpendCard(card)
                    end
                end
            },
        },

        negotiation_modifier =
        {
            hidden = true,
            event_priorities =
            {
                [ EVENT.END_TURN ] = 999,
            },
            event_handlers =
            {
                [ EVENT.END_RESOLVE ] = function( self, minigame, card )
                    if card.negotiator == self.negotiator and not card:IsFlagged(negotiation_defs.CARD_FLAGS.BYSTANDER) then
                        local clone = card:Duplicate()
                        clone.show_dealt = false
                        minigame:DealCard(clone, minigame:GetDiscardDeck())
                    end
                end,

                [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        local cards_to_expend = shallowcopy(minigame:GetHandDeck().cards)
                        for i,card in ipairs(cards_to_expend) do
                            minigame:ExpendCard(card)
                        end
                    end
                end
            },
        },
    },

    nesting_cards =
    {
        name = "Nesting Cards",
        desc = "All cards have <b>Hatch 4</> and transform into a random card when hatched.",
        event_handlers =
        {
            [ "card_added" ] = function( self, card )
                if is_instance(card, Battle.Card) then
                    local cards = BattleCardCollection.AllRewardableCards(function(cd) 
                                return cd.rarity == card.rarity or cd.rarity == CARD_RARITY_ORDER[math.min(#CARD_RARITY_ORDER, CARD_RARITY_ORDINALS[card.rarity] + 2)] 
                            end, self.owner):Pick(1)
                    if #cards > 0 then
                        card.userdata.upgrade_ids = nil
                        card.userdata.max_xp = 4
                        card.userdata.xp = 0
                        card.userdata.hatch_id = cards[1].id

                        card:SetFlags(battle_defs.CARD_FLAGS.HATCH)
                    end
                else
                    local cards = NegotiationCardCollection.AllRewardableCards(function(cd) 
                                return cd.rarity == card.rarity or cd.rarity == CARD_RARITY_ORDER[math.min(#CARD_RARITY_ORDER, CARD_RARITY_ORDINALS[card.rarity] + 2)] 
                            end, self.owner):Pick(1)
                    if #cards > 0 then
                        card.userdata.upgrade_ids = nil
                        card.userdata.max_xp = 4
                        card.userdata.xp = 0
                        card.userdata.hatch_id = cards[1].id

                        card:SetFlags(negotiation_defs.CARD_FLAGS.HATCH)
                    end
                end
            end
        },
    },

    cosmic_grafts =
    {
        name = "Cosmic Interference",
        desc = "At the start of the game, draft two Cosmic Grafts, one of each type.",

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                local player = gamestate:GetPlayerAgent()
                local battle_grafts = GraftCollection(function(graft_def) return graft_def.type == GRAFT_TYPE.COMBAT end):NotInstalled(player):NotRestricted(player):Rarity(CARD_RARITY.COSMIC):Generate(3)
                local popup = Screen.PickGraftScreen(battle_grafts)
                TheGame:FE():InsertScreen( popup )

                local negotiation_grafts = GraftCollection(function(graft_def) return graft_def.type == GRAFT_TYPE.NEGOTIATION end):NotInstalled(player):NotRestricted(player):Rarity(CARD_RARITY.COSMIC):Generate(3)
                local popup = Screen.PickGraftScreen(negotiation_grafts)
                TheGame:FE():InsertScreen( popup )
            end
        }
    },
}


local FIRST_IMPRESSION_DELTAS =
{
    [OPINION_DELTAS.NEUTRALIZING] = 8,
    [OPINION_DELTAS.OPINION_UP] = 3,
    [OPINION_DELTAS.OPINION_DOWN] = 3,
    [OPINION_DELTAS.MAJOR_GOOD] = 1,
    [OPINION_DELTAS.MAJOR_BAD] = 1,
}
local function CalculateFirstImpression( agent )
    local opt, wt = weightedpick( FIRST_IMPRESSION_DELTAS )
    return opt
end

AddOpinionEvent("FIRST_IMPRESSIONS", 
{
    delta = function( agent, data, from_agent )
        return agent:CalculateProperty( "FIRST_IMPRESSION", CalculateFirstImpression )
    end,

    txt = "First Impressions",
})

AddOpinionEvent("RELATABLE", 
{
    delta = OPINION_DELTAS.DISLIKE,
    txt = "Relatable",
})

Content.AddBattleCard( "exhaustion", 
{
    name = "Exhaustion",

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    flavour = "'So tired.'",

    cost = 0,

    flags = battle_defs.CARD_FLAGS.STATUS | battle_defs.CARD_FLAGS.UNPLAYABLE,
})
Content.AddNegotiationCard("mental_block", 
{
    name = "Mental Block",
    
    cost = 0,
    flavour = "'You’re not getting in here!'",
    flags = negotiation_defs.CARD_FLAGS.STATUS | negotiation_defs.CARD_FLAGS.UNPLAYABLE,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
})
---------------------------------------------------------------------------------------------

for id, graft in pairs( MUTATORS ) do
    Content.AddMutatorGraft( id, graft )
end
