local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT

local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

------------------------------------------------------------------------------------------------
AddOpinionEvent("BENEFITS", 
{
    delta = OPINION_DELTAS.LIKE,
    txt = "Affected by Benefits",
})
AddOpinionEvent("CHARMING", 
{
    delta = OPINION_DELTAS.MAJOR_GOOD,
    txt = "Affected by Charming",
})

AddOpinionEvent("FORGIVENESS", 
{
    delta = OPINION_DELTAS.NORMALIZING,
    txt = "Affected by Forgiveness",
})

-- Temporary values, please change
local unlock_per_tier = { 1, 2, 3, 4, 5}

local PERKS =
{
    -- TIER 1
    benefits_01 = 
    {
        name = "Benefits",
        desc = "At the start of the game, 1 random person will like you.",
        tier = 1,

        exclude = {"benefits_02", "benefits_03"},

        event_handlers = 
        {
            [ "new_game" ] = function( self, gamestate )
                local candidates = ObtainWorkTable()

                for k,agent in pairs(gamestate.agents) do
                    if not AgentUtil.HasPlotArmour(agent) and
                        not agent:IsInPlayerParty() and
                        agent:GetRelationship() < RELATIONSHIP.LIKED and
                        agent:IsSentient() then
                        table.insert(candidates, agent)
                    end
                end

                for i = #candidates + 1, 1 do
                    if TheGame:GetGameState() then
                        local new_agent = gamestate:AddSkinnedAgent()
                        table.insert( candidates, new_agent )
                    end
                end

                table.shuffle( candidates )

                for i=1, math.min( #candidates, 1 ) do
                    local chosen = table.remove( candidates )
                    chosen:OpinionEvent( OPINION.BENEFITS )
                end
                
                ReleaseWorkTable(candidates)
            end
        }
    },

    researcher =
    {
        name = "Researcher",
        desc = "Gain 10 shills every time you use an item card with a charge.",
        tier = 1,

        battle_condition =
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if attack.attacker == self.owner and attack.card:IsFlagged(battle_defs.CARD_FLAGS.ITEM) and attack.card:GetCharges() ~= nil then
                        TheGame:GetGameState():GetCaravan():AddMoney( 10 )
                    end
                end
            }
        },

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.START_RESOLVE ] = function( self, minigame, card )
                    if card.owner == self.owner and card:IsFlagged(negotiation_defs.CARD_FLAGS.ITEM) and card:GetCharges() ~= nil then
                        TheGame:GetGameState():GetCaravan():AddMoney( 10 )
                    end
                end,
            },
        },

    },

    premium_quality =
    {
        name = "Premium Quality",
        desc = "The first graft you install is immediately upgraded.",

        tier = 1,

        OnAdded = function(self)
            self.userdata.available_upgrades = 1
        end,

        display_number = function( self )
            if type(self.userdata.available_upgrades) == "number" then
                return self.userdata.available_upgrades, 1
            end
        end,

        event_handlers =
        {
            [ "graft_added" ] = function( self, graft )
                local graft_type = graft:GetType()
                if graft ~= self and graft:HasUpgrades() and (graft_type == GRAFT_TYPE.COMBAT or graft_type == GRAFT_TYPE.NEGOTIATION) and type(self.userdata.available_upgrades) == "number" and self.userdata.available_upgrades > 0 then
                    self.userdata.available_upgrades = self.userdata.available_upgrades - 1
                    graft:Upgrade()
                end
            end,
        },
    },

    innate_connection =
    {
        name = "Innate Connection",
        desc = "New pets are immediately trained.",

        tier = 1,

        event_handlers =
        {
            [ "caravan_member_add" ] = function( self, agent )
                if agent:IsPet() and agent.support_negotiation == "untrained_pet" then
                    agent:TrainPet()
                end
            end
        }
    },

    good_health_01 = 
    {
        name = "Good Health",
        desc = "Gain 6 maximum Health.",

        tier = 1,
        exclude = {"good_health_02", "good_health_03"},

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetCaravan():UpgradeHealth( 6, "PERK_GOOD_HEALTH" )
            end
        },
    },

    resolute_01 = 
    {
        name = "Resolute",
        desc = "Gain 4 maximum Resolve.",

        tier = 1,
        exclude = {"resolute_02", "resolute_03"},

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetCaravan():UpgradeResolve( 4 )
            end
        },
    },

    perfect_shape =
    {
        name = "Perfect Shape",
        desc = "Gain 1 <b>Power</> at the start of each battle.",

        tier = 1,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.owner:AddCondition("POWER", 1, self)
                    self.owner:RemoveCondition(self.id, self.stacks, self)
                end
            }
        },
    },

    tough_constitution =
    {
        name = "Tough Constitution",
        desc = "Once per day, drink and eat without any repercussions.",
        OnAdded = function( self )
            self.userdata.eat_active = true
            self.userdata.drink_active = true
        end,

        tier = 1,

        event_handlers =
        {
            ["do_drink"] = function( self, drink_effects )
                if self.userdata.drink_active and drink_effects.cards then
                    table.clear(drink_effects.cards)
                end
            end,

            ["do_eat"] = function( self, eat_effects )
                if self.userdata.eat_active and eat_effects.cards then
                    table.clear(eat_effects.cards)
                end
            end,

            ["do_sleep"] = function( self )
                self.userdata.eat_active = true
                self.userdata.drink_active = true
            end,

            ["had_drink"] = function( self, drink_effects )
                if self.userdata.drink_active then
                    self.userdata.drink_active = false
                end
            end,

            ["did_eat"] = function( self, drink_effects )
                if self.userdata.eat_active then
                    self.userdata.eat_active = false
                end
            end,
        },
    },

    stamina_01 =
    {
        name = "Stamina",
        desc = "Fatigued starts 1 turn later than normal.",

        tier = 1,

        exclude = {"stamina_02"},

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    minigame:SetFatiguedTurn(minigame:GetFatiguedTurn() + 1)
                    self.negotiator:RemoveModifier(self.id, self.stacks, self)
                end
            }
        },
        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    battle:SetFatiguedTurn(battle:GetFatiguedTurn() + 1)
                    self.owner:RemoveCondition(self.id, self.stacks, self)
                end
            },
        },
    },

    swear_jar =
    {
        name = "Swear Jar",
        desc = "Start your run with an additional 75 shills.",

        tier = 1,

        event_handlers =
        {
            ["new_game"] = function( self, gamestate )
                gamestate:GetCaravan():AddMoney(75)
            end
        },
    },

    -- TIER 2

    stamina_02 =
    {
        name = "Stamina II",
        desc = "Fatigued starts 2 turns later than normal.",

        tier = 2,

        exclude = {"stamina_01"},

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    minigame:SetFatiguedTurn(minigame:GetFatiguedTurn() + 2)
                    self.negotiator:RemoveModifier(self.id, self.stacks, self)
                end
            }
        },
        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    battle:SetFatiguedTurn(battle:GetFatiguedTurn() + 2)
                    self.owner:RemoveCondition(self.id, self.stacks, self)
                end
            },
        },
    },

    animal_bond = 
    {
        name = "Animal Bond",
        desc = "Eating food also restores the health of your pet.",

        tier = 2,

        event_handlers =
        {
            ["did_eat"] = function( self, eat_effects )
                if eat_effects.health_gain then
                    local pets = TheGame:GetGameState():GetCaravan():GetPets()
                    if pets and #pets > 0 then
                        for i,pet in ipairs(pets) do
                            pet:GetAspect("health"):Delta(eat_effects.health_gain)
                        end
                    end
                end
            end,
        },  
    },

    benefits_02 = 
    {
        name = "Benefits II",
        desc = "At the start of the game, 2 random people will like you.",
        tier = 2,

        exclude = {"benefits_01", "benefits_03"},

        event_handlers = 
        {
            [ "new_game" ] = function( self, gamestate )
                local candidates = ObtainWorkTable()

                for k,agent in pairs(gamestate.agents) do
                    if not AgentUtil.HasPlotArmour(agent) and
                        not agent:IsInPlayerParty() and
                        agent:GetRelationship() < RELATIONSHIP.LIKED and
                        agent:IsSentient() then
                        table.insert(candidates, agent)
                    end
                end

                for i = #candidates + 1, 2 do
                    if TheGame:GetGameState() then
                        local new_agent = gamestate:AddSkinnedAgent()
                        table.insert( candidates, new_agent )
                    end
                end

                table.shuffle( candidates )

                for i=1, math.min( #candidates, 2 ) do
                    local chosen = table.remove( candidates )
                    chosen:OpinionEvent( OPINION.BENEFITS )
                end
                
                ReleaseWorkTable(candidates)
            end
        }
    },

    good_health_02 = 
    {
        name = "Good Health II",
        desc = "Gain 9 maximum Health.",

        tier = 2,
        exclude = {"good_health_01", "good_health_03"},

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetCaravan():UpgradeHealth( 9, "PERK_GOOD_HEALTH_2" )
            end
        },
    },

    resolute_02 = 
    {
        name = "Resolute II",
        desc = "Gain 6 maximum Resolve.",

        tier = 2,
        exclude = {"resolute_01", "resolute_03"},

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetCaravan():UpgradeResolve( 6 )
            end
        },
    },

    charming =
    {
        name = "Charming",
        desc = "At the start of each day, a random person that likes you will love you instead.",

        tier = 2,

        event_handlers = 
        {
            ["morning_mail"] = function( self, cxt )
                if (self.userdata.last_mail or -1) < cxt.now then
                    local candidates = {}
                    local player = TheGame:GetGameState():GetPlayerAgent()
                    for i,agent in ipairs(player.social_connections:GetConnectionsOfLevel(RELATIONSHIP.LIKED)) do
                        if agent:IsSentient() and not agent:IsDead() then
                            table.insert(candidates, agent)
                        end
                    end
                    local chosen = table.arraypick(candidates)
                    if chosen then
                        chosen:OpinionEvent( OPINION.CHARMING )
                    end
                    self.userdata.last_mail = cxt.now
                end
            end,
        },
    },

    alert = 
    {
        name = "Alert",
        desc = "Draw {1} additional {1*card|cards} at the start of every battle and negotiation.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().draw )
        end,

        tier = 2,
        draw = 2,

        negotiation_modifier =
        {
            hidden = true,

            event_handlers =
            {
                [ EVENT.CALC_TURN_DRAW ] = function( self, acc )
                    acc:AddValue( self.graft:GetDef().draw, self )
                end,

                [ EVENT.HAND_DRAWN ] = function( self )
                    self.negotiator:RemoveModifier( self )
                end,
            }
        },

        battle_condition =
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_TURN_DRAW ] = function( self, acc )
                    acc:AddValue( self.graft:GetDef().draw, self )
                end,

                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self )
                    self.owner:RemoveCondition( self.id )
                end,
            },
        },
    },

    gourmet =
    {
        name = "Gourmet",
        desc = "Eating heals 8 bonus health and drinking restores 6 extra resolve.",

        tier = 2,

        event_handlers =
        {
            ["do_eat"] = function( self, eat_effects )
                eat_effects.health_gain = (eat_effects.health_gain or 0) + 8
            end,

            ["do_drink"] = function( self, drink_effects )
                drink_effects.resolve_gain = (drink_effects.resolve_gain or 0) + 6
            end,
        },
    },

    -- TIER 3
    handy = 
    {
        name = "Handy",
        desc = "All items have 1 additional use.",

        tier = 3,

        event_handlers =
        {
            [ "card_added" ] = function( self, card )
                if card:IsFlagged(battle_defs.CARD_FLAGS.ITEM) or card:IsFlagged(negotiation_defs.CARD_FLAGS.ITEM) then
                    if (card:GetCharges() or 0) > 0 and (self.modified_cards == nil or not table.contains(self.modified_cards, card)) then
                        card.userdata.charges = card.userdata.charges + 1
                        card.userdata.max_charges = (card.max_charges or card.userdata.max_charges) + 1
                        self.modified_cards = self.modified_cards or {}
                        table.insert(self.modified_cards, card)
                    end
                end
            end,
        },

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CARD_DEALT ] = function( self, card, deck )
                    if card.owner == self.owner and card:IsItemCard() and (card:GetCharges() or 0) > 0 and (self.modified_cards == nil or not table.contains(self.modified_cards, card)) then
                        card.userdata.max_charges = (card.max_charges or card.userdata.max_charges) + 1
                        card.userdata.charges = math.min( card.userdata.max_charges, card.userdata.charges + 1 )
                        self.modified_cards = self.modified_cards or {}
                        table.insert(self.modified_cards, card)
                    end
                end
            },
        },

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.BEGIN_NEGOTIATION ] = function( self, minigame )
                    self.activated = true
                end,

                [ EVENT.CARD_DEALT ] = function( self, card, deck )
                    if card.owner == self.owner and card:IsItemCard() and (card:GetCharges() or 0) > 0 and self.activated and (self.modified_cards == nil or not table.contains(self.modified_cards, card)) then
                        card.userdata.max_charges = (card.max_charges or card.userdata.max_charges) + 1
                        card.userdata.charges = math.min( card.userdata.max_charges, card.userdata.charges + 1 )
                        self.modified_cards = self.modified_cards or {}
                        table.insert(self.modified_cards, card)
                    end
                end,
            },
        },
    },
    
    basic_training =
    {
        name = "Basic Training",
        desc = "The max XP on both your basic decks is halved.",

        tier = 3,

        event_handlers = 
        {
            [ "roll_upgrades" ] = function( self, card )
                if card.owner == self.owner and card:GetMaxXP() ~= nil and card.rarity == CARD_RARITY.BASIC then
                    card.userdata.max_xp = math.round(card:GetMaxXP() * 0.5)
                end
            end,
        }
    },

    ancient_warrior =
    {
        name = "Ancient Warrior",
        desc = "The first 2 cards you draft in each deck come with full xp.",

        tier = 3,

        upgrades = 
        {
            ["BATTLE"] = 2,
            ["NEGOTIATION"] = 2,
        },
        OnAdded = function(self)
            self.userdata.available_upgrades = shallowcopy(self:GetDef().upgrades)
        end,

        event_handlers =
        {
            [ "card_added" ] = function( self, card )
                if card.rarity ~= CARD_RARITY.BASIC and card:GetMaxXP() ~= nil and card:GetMaxXP() > 0 and not card:IsHatched() then
                    local card_type = is_instance( card, Battle.Card ) and "BATTLE" or "NEGOTIATION"
                    if self.userdata.available_upgrades[card_type] and self.userdata.available_upgrades[card_type] > 0 then
                        self.userdata.available_upgrades[card_type] = self.userdata.available_upgrades[card_type] - 1
                        card:AddXP(card:GetMaxXP())
                    end
                end
            end,
        },
    },

    unencumbered =
    {
        name = "Unencumbered",
        desc = "Gain 1 additional graft slot of each type.",

        tier = 3,
        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                local player = TheGame:GetGameState():GetPlayerAgent()
                player.graft_owner:IncreaseMaxGrafts( GRAFT_TYPE.COMBAT, 1 )
                player.graft_owner:IncreaseMaxGrafts( GRAFT_TYPE.NEGOTIATION, 1 )
            end
        }
    },

    benefits_03 = 
    {
        name = "Benefits III",
        desc = "At the start of the game, 3 random people will like you.",
        tier = 3,

        exclude = {"benefits_01", "benefits_02"},

        event_handlers = 
        {
            [ "new_game" ] = function( self, gamestate )
                local candidates = ObtainWorkTable()

                for k,agent in pairs(gamestate.agents) do
                    if not AgentUtil.HasPlotArmour(agent) and
                        not agent:IsInPlayerParty() and
                        agent:GetRelationship() < RELATIONSHIP.LIKED and
                        agent:IsSentient() then
                        table.insert(candidates, agent)
                    end
                end

                for i = #candidates + 1, 3 do
                    if TheGame:GetGameState() then
                        local new_agent = gamestate:AddSkinnedAgent()
                        table.insert( candidates, new_agent )
                    end
                end

                table.shuffle( candidates )

                for i=1, math.min( #candidates, 3 ) do
                    local chosen = table.remove( candidates )
                    chosen:OpinionEvent( OPINION.BENEFITS )
                end
                
                ReleaseWorkTable(candidates)
            end
        }
    },

    good_health_03 = 
    {
        name = "Good Health III",
        desc = "Gain 12 maximum Health.",

        tier = 3,
        exclude = {"good_health_01", "good_health_02"},

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetCaravan():UpgradeHealth( 12, "PERK_GOOD_HEALTH_3" )
            end
        },
    },

    resolute_03= 
    {
        name = "Resolute III",
        desc = "Gain 8 maximum Resolve.",

        tier = 3,
        exclude = {"resolute_01", "resolute_02"},

        event_handlers =
        {
            [ "new_game" ] = function( self, gamestate )
                gamestate:GetCaravan():UpgradeResolve( 8 )
            end
        },
    },

    -- TIER 4
    fast_learner =
    {
        name = "Fast Learner",
        desc = "Lower the max XP of all non-basic cards by 2.",

        tier = 4,

        event_handlers = 
        {
            [ "roll_upgrades" ] = function( self, card )
                if card.owner == self.owner and card:GetMaxXP() ~= nil and card.rarity ~= CARD_RARITY.BASIC then
                    card.userdata.max_xp = card:GetMaxXP() - 2
                end
            end,
        }
    },

    animal_handler =
    {
        name = "Animal Handling",
        desc = "You can own 1 additional pet.",

        tier = 4,

        event_handlers =
        {
            [ "calc_max_pets" ] = function( self, pet_details )
                pet_details.max_pets = (pet_details.max_pets or 1) + 1
            end,
        }
    },

    survivalist =
    {
        name = "Survivalist",
        desc = "After each battle, heal 4 health.",

        tier = 4,

        event_handlers =
        {
            [ "resolve_battle" ] = function( self, battle, primary_enemy, repercussions )
                repercussions.loot.health = (repercussions.loot.health or 0) + 4
                -- TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):Delta(4)
            end,
        },
    },

    socializer =
    {
        name = "Socializer",
        desc = "Gain 1 maximum resolve whenever a new person loves you.",

        tier = 4,

        event_handlers =
        {
            [ "agent_relationship_changed" ] = function( self, agent, old_rel, new_rel )
                if new_rel == RELATIONSHIP.LOVED and old_rel ~= RELATIONSHIP.LOVED then
                    TheGame:GetGameState():GetCaravan():DeltaMaxResolve(1)
                end
            end
        },
    },

    forgiveness =
    {
        name = "Forgiveness",
        desc = "At the start of each day, a random person that hates you will dislike you instead.",

        tier = 4,

        event_handlers =
        {
            [ "do_sleep" ] = function( self )
                local candidates = {}
                local player = TheGame:GetGameState():GetPlayerAgent()
                for i,agent in ipairs(player.social_connections:GetConnectionsOfLevel(RELATIONSHIP.HATED)) do
                    if agent:IsSentient() and not agent:IsDead() then
                        table.insert(candidates, agent)
                    end
                end
                local chosen = table.arraypick(candidates)
                if chosen then
                    chosen:OpinionEvent( OPINION.FORGIVENESS )
                end
            end
        },
    },

    -- TIER 5
    perseverance = 
    {
        name = "Perseverance",
        desc = "When you would die, instead heal to 20 health. Only works once.",

        tier = 5,

        OnAdded = function( self )
            self.userdata.uses = 1
        end,

        display_number = function( self )
            return self.userdata.uses, 1
        end,

        OnActivateFighter = function( self, fighter )
            if (self.userdata.uses or 1) > 0 then
                self:AddBattleCondition( fighter )
            end
        end,

        battle_condition =
        {
            hidden = true,
            OnPreDamage = function( self, damage, attacker, battle, source, piercing )
                if damage >= self.owner:GetHealth() then
                    self.activated = true
                    if self.graft then
                        self.graft.userdata.uses = (self.graft.userdata.uses or 1) - 1
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                    if is_instance( source, Battle.Hit ) then 
                        source:SetHitResult( "defended" )
                    end
                    return self.owner:GetHealth() - 1
                end
                return damage
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if self.activated then
                        battle:PrintLog( BATTLE_LOG.GENERAL, "Triggered Perseverance!" )
                        self.owner:HealHealth(19)
                        self.owner:RemoveCondition(self.id, 1, self)
                    end
                end
            },
        },
    },

    -- wildlife_expert =
    -- {
    --     name = "Wildlife Expert",
    --     desc = "At the start of the game, choose a pet.",

    --     tier = 5,

    --     event_handlers =
    --     {
    --         [ "new_game" ] = function( self, gamestate )
    --             -- TO DO: Convo or screen that allows for pet selection
    --         end
    --     },
    -- },

    terror =
    {
        name = "Terror",
        desc = "Whenever you kill an enemy, increase the <b>Panic</> of all their allies.",

        tier = 5,

        battle_condition =
        {
            hidden = true,

            event_handlers =
            {
                [ BATTLE_EVENT.FIGHTER_KILLED ] = function ( self, fighter )
                    if fighter:GetTeam() ~= self.owner:GetTeam() then
                        for i,enemy in fighter:GetTeam():Fighters() do
                            enemy:ApplyMoraleDamage(5, self)
                        end
                    end
                end,
            },
        },
    },

    partner_in_crime =
    {
        name = "Partner In Crime",
        desc = "Once per day, you can hire someone who loves you to help you on a quest.",

        tier = 5,

        OnAdded = function( self )
            self.userdata.active = true
        end,

        display_number = function( self )
            if self.userdata.active ~= nil then
                return self.userdata.active and 1 or 0, 1
            end
        end,

        event_handlers =
        {
            ["do_sleep"] = function( self )
                self.userdata.active = true
            end
        }
    },

    expanded_assortment =
    {
        name = "Expanded Assortment",
        desc = "Graft drafts have 1 additional option.",

        tier = 5,

        event_handlers =
        {
            [ "calc_graft_draft" ] = function( self, graft_draft_details )
                graft_draft_details.count = (graft_draft_details.count or 3) + 1
            end,
        }
    },
}

---------------------------------------------------------------------------------------------

for id, graft in pairs( PERKS ) do
    graft.hidden = true
    assert_warning( graft.tier ~= nil, "Missing perk tier: %s", id )
    graft.tier = graft.tier or 1
    graft.unlock_cost = graft.unlock_cost or unlock_per_tier[math.min(graft.tier, #unlock_per_tier)]
    Content.AddPerkGraft( id, graft )
end
