local battle_defs = require "battle/battle_defs"

local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

--------------------------------------------------------------------

local BATTLE_GRAFTS =
{
    --COMMONS
    twinshot =
    {
        name = "Twinshot",
        desc = "At the start of each battle, gain 2 {CHARGE}.",
        flavour = "'First thing in the morning, shower, shave, and load your twinshot.'",

        rarity = CARD_RARITY.COMMON,
        charge_gain = 2,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.tracker = self.tracker or self.owner:GetCondition("lumin_tracker")
                    if self.tracker then
                        self.tracker:AddLuminCharges(self.graft:GetDef().charge_gain)
                    end
                    self.owner:RemoveCondition("twinshot")
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end
            },
        },
    },

    twinshot_plus = 
    {
        name = "Boosted Twinshot",
        desc = "At the start of each battle, gain <#UPGRADE>3</> {CHARGE}.",
        charge_gain = 3,
    },

    clicker = 
    {
        name = "Clicker",
        desc = "Whenever you spend your last {CHARGE}, draw a card.",
        flavour = "'When I can’t find mine it’s usually between the cushions.'",
        rarity = CARD_RARITY.COMMON,

        draw_count = 1,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.LUMIN_SPENT ] = function( self, battle, charges, max_cells )
                    if charges == 0 then
                        battle:DrawCards(self.graft:GetDef().draw_count)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    clicker_plus =
    {
        name = "Visionary Clicker",
        desc = "Whenever you spend your last {CHARGE}, draw <#UPGRADE>2</> cards.",
        draw_count = 2,
    },

    spare_magazine =
    {
        name = "Spare Magazine",
        desc = "Whenever you gain {PRIMED}, gain 2 {TEMP_POWER}.",
        flavour = "'I always make sure to bring it to the bathroom.'",

        rarity = CARD_RARITY.COMMON,

        power_amt = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.PRIMED_GAINED ] = function( self, battle )
                    self.owner:AddCondition("POWER", self.graft:GetDef().power_amt, self)
                    self.owner:AddCondition("POWER_LOSS", self.graft:GetDef().power_amt, self)
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end,
            },
        },
    },

    spare_magazine_plus =
    {
        name = "Boosted Spare Magazine",
        desc = "Whenever you gain {PRIMED}, gain <#UPGRADE>3</> {TEMP_POWER}.",
        power_amt = 3,
    },

    armor_plating =
    {
        name = "Armor Plating",
        desc = "Whenever you gain {CHARGE}, gain 2 {DEFEND}.",
        flavour = "'Small lumin fields are generated by gun cycling and this here makes sure to localize them onto your person.'",

        rarity= CARD_RARITY.UNCOMMON,

        defend_amt = 2,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.LUMIN_CHARGED ] = function( self, battle, charged, max_cells )
                    self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end
            },
        },
    },

    armor_plating_plus =
    {
        name = "Stone Armor Plating",
        desc = "Whenever you gain {CHARGE}, gain <#UPGRADE>3</> {DEFEND}.",
        defend_amt = 3,
    },

    auto_tracker =
    {
        name = "Auto Tracker",
        desc = "At the start of each turn, apply 2 {MARK} to a random enemy.",
        flavour = "'Every sharpshooter’s secret.'",

        rarity = CARD_RARITY.UNCOMMON,

        mark_amt = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local target_fighters = {}
                    battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )

                    for i=1, #target_fighters do
                        local target = target_fighters[i]
                        target:AddCondition("MARK", self.graft:GetDef().mark_amt, self)
                    end
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end
            },
        },
    },

    auto_tracker_plus =
    {
        name = "Boosted Auto Tracker",
        desc = "At the start of each turn, apply <#UPGRADE>3</> {MARK} to a random enemy.",
        mark_amt = 3,
    },

    easy_listening =
    {
        name = "Easy Listening",
        desc = "At the start of battle, gain {CONCENTRATION 3}.",
        flavour = "'The audio manages to bring the listener to a more focused state with a combination of wilderness sounds and inoffensive music.'",

        rarity = CARD_RARITY.COMMON,

        concentration_amt = 3,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.owner:AddCondition("CONCENTRATION", self.graft:GetDef().concentration_amt, self)
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    self.owner:RemoveCondition("easy_listening")
                end
            },
        },
    },

    easy_listening_plus = 
    {
        name = "Boosted Easy Listening",
        desc = "At the start of battle, gain <#UPGRADE>{CONCENTRATION 5}</>.",
        concentration_amt = 5,
    },

    perpetual_charger =
    {
        name = "Perpetual Charger",
        desc = "At the start of each turn, gain 1 {SURGE}.",
        flavour = "'Ok it’s not perpetual, but it lasts longer than what you are currently using.'",

        rarity = CARD_RARITY.COMMON,

        surge_amt = 1,

        battle_condition =
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.owner:AddCondition("SURGE", self.graft:GetDef().surge_amt, self)
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end
            },
        },
    },

    perpetual_charger_plus =
    {
        name = "Boosted Perpetual Charger",
        desc = "At the start of each turn, gain <#UPGRADE>2</> {SURGE}.",
        surge_amt = 2,
    },

    scattershot =
    {
        name = "Scattershot",
        desc = "Deal 2 bonus damage to targets with {RICOCHET}.",
        flavour = "'It might be a couple micrometers under legal size for something this dangerous but at least it’s easy to hide.'",

        rarity= CARD_RARITY.COMMON,

        bonus_damage = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner then
                        if target and target:HasCondition("RICOCHET") then
                            dmgt:AddDamage(self.graft:GetDef().bonus_damage, self.graft:GetDef().bonus_damage, self)
                        end
                    end
                end
            },
        },
    },

    scattershot_plus =
    {
        name = "Boosted Scattershot",
        desc = "Deal <#UPGRADE>3</> bonus damage to targets with {RICOCHET}.",
        bonus_damage = 3,
    },

    -- UNCOMMONS

    speed_wheel =
    {
        name = "Speed Wheel",
        desc = "At the start of each turn, gain 1 {CHARGE}.",
        flavour = "'This here keeps those cells charged at a rate faster than even you could cycle.'",

        rarity = CARD_RARITY.RARE,

        charge_gain = 1,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.tracker = self.tracker or self.owner:GetCondition("lumin_tracker")
                    if self.tracker then
                        self.tracker:AddLuminCharges(self.graft:GetDef().charge_gain)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    speed_wheel_plus =
    {
        name = "Inductive Speed Wheel",
        desc = "At the start of each turn, gain <#UPGRADE>2</> {CHARGE}.",
        charge_gain = 2,
    },

    flash_powder =
    {
        name = "Flash Powder",
        desc = "Your first attack each battle applies 2 {SCORCHED}.",
        flavour = "'I couldn’t figure out anything to use this stuff for except starting violent fires.'",

        rarity = CARD_RARITY.UNCOMMON,

        num_stacks = 2,

        battle_condition =
        {
            hidden = false,
            desc = "Your first attack each battle applies <#HILITE>{1} {SCORCHED}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks )
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if attack.attacker == self.owner and attack.card:IsAttackCard() then
                        for i, hit in attack:Hits() do
                            hit.target:AddCondition("SCORCHED", self.stacks, attack.card)
                        end
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        self.owner:RemoveCondition( self.id )
                    end
                end,
            },
        },
    },

    flash_powder_plus =
    {
        name = "Boosted Flash Powder",
        desc = "Your first attack each battle applies <#UPGRADE>4</> {SCORCHED}.",
        num_stacks = 4,
    },

    arc_loader =
    {
        name = "Arc Loader",
        desc = "Whenever you gain {PRIMED}, gain 3 {SURGE}.",
        flavour = "'Cycling the guns up to maximum still creates energy that we can use for other things.'",

        rarity = CARD_RARITY.UNCOMMON,

        surge_amt = 3,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.PRIMED_GAINED ] = function( self, battle )
                    self.owner:AddCondition("SURGE", self.graft:GetDef().surge_amt, self)
                    self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end,
            },
        },
    },

    arc_loader_plus =
    {
        name = "Boosted Arc Loader",
        desc = "Whenever you gain {PRIMED}, gain <#UPGRADE>5</> {SURGE}.",
        surge_amt = 5,
    },

    pinball =
    {
        name = "Pinball",
        desc = "At the start of each battle, apply 4 {RICOCHET} to a random target.",
        flavour = "'Keeps your enemies tilted.'",

        rarity = CARD_RARITY.UNCOMMON,
        ricochet_amt = 4,
        
        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local target_fighters = {}
                    battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, self.stacks )

                    for i=1, #target_fighters do
                        local target = target_fighters[i]
                        target:AddCondition("RICOCHET", self.graft:GetDef().ricochet_amt, self)
                    end
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    self.owner:RemoveCondition(self.id)
                end
            },
        },
    },

    pinball_plus =
    {
        name = "Boosted Pinball",
        desc = "At the start of each battle, apply <#UPGRADE>6</> {RICOCHET} to a random target.",
        ricochet_amt = 6,
    },

    fumes = 
    {
        name = "Fumes",
        desc = "While you have no charge, all your attacks deal 2 bonus damage.",
        flavour = "'There’s a large energy vacuum in some of the internal tubes that you can use to bolster the stopping power of regular shots.'",

        rarity = CARD_RARITY.UNCOMMON,
        bonus_damage = 2,

        battle_condition =
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if not self.tracker then self.tracker = self.owner:GetCondition("lumin_tracker") end
                    if self.tracker and self.tracker:GetCharges() == 0 and card.owner == self.owner then
                        dmgt:AddDamage(self.graft:GetDef().bonus_damage, self.graft:GetDef().bonus_damage, self)
                    end
                end
            },
        },
    },

    fumes_plus =
    {
        name = "Boosted Fumes",
        desc = "While you have no charge, all your attacks deal <#UPGRADE>3</> bonus damage.",
        bonus_damage = 3,
    },

    -- RARES
    scan_code =
    {
        name = "Scan Code",
        desc = "At the start of each battle, apply 3 {SCANNED} to a random enemy.",
        flavour = "'The cutting edge of ranged combat.'",

        rarity = CARD_RARITY.RARE,
        num_targets = 1,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local target_fighters = {}
                    battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, self.graft:GetDef().num_targets )

                    for i=1, #target_fighters do
                        local target = target_fighters[i]
                        target:AddCondition("SCANNED", 3, self)
                    end
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    self.owner:RemoveCondition(self.id)
                end
            },
        },
    },

    scan_code_plus =
    {
        name = "Wide Scan Code",
        desc = "At the start of each battle, apply 3 {SCANNED} to <#UPGRADE>2</> random enemies.",
        num_targets = 2,
    },

    power_sink =
    {
        name = "Power Sink",
        desc = "Whenever you gain {PRIMED}, reduce the cost of a random card in your hand by 1.",
        flavour = "'Gun cycling takes years of practice, this here shaves off a couple months to get the most from those pistols.'",
        rarity = CARD_RARITY.RARE,

        cost_reduction = 1,

        battle_condition = 
        {
            hidden = true,
            chosen_cards = {},
            event_handlers = 
            {
                [ BATTLE_EVENT.PRIMED_GAINED ] = function( self, battle )
                    local hand = battle:GetHandDeck()

                    local card = hand:PeekRandom()
                    if card then
                        table.insert(self.chosen_cards, card)
                    end
                    self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end,

                [ BATTLE_EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                    if self.chosen_cards and table.contains(self.chosen_cards, card) then
                        cost_acc:AddValue(-self.graft:GetDef().cost_reduction, self)
                    end
                end,

                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if table.contains(self.chosen_cards, attack.card) then
                        table.remove(self.chosen_cards, self.chosen_cards[attack.card])
                    end
                end,
            },
        },
    },

    power_sink_plus =
    {
        name = "Boosted Power Sink",
        desc = "Whenever you gain {PRIMED}, reduce the cost of a random card in your hand by <#UPGRADE>2</>.",
        cost_reduction = 2,
    },

    trench_knife = 
    {
        name = "Trench Knife",
        desc = "At the end of each turn, gain {DEFEND 4} and {RIPOSTE 3} if all your cells are empty.",
        flavour = "'A brutal relic from the old wars, still sharp and mean.'",

        rarity = CARD_RARITY.RARE,

        defend_amt = 4,
        riposte_amt = 3,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    local tracker = self.owner:GetCondition("lumin_tracker")
                    if tracker and tracker:GetCharges() == 0 then
                        self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                        self.owner:AddCondition("RIPOSTE", self.graft:GetDef().riposte_amt, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    trench_knife_plus =
    {
        name = "Stone Trench Knife",
        desc = "At the end of each turn, gain <#UPGRADE>{DEFEND 6}</> and {RIPOSTE 3} if all your cells are empty.",
        defend_amt = 6,
    },
    
    glancer = 
    {
        name = "Glancer",
        desc = "At the start of each turn, apply 1 {RICOCHET} to a random target.",
        flavour = "'An old Aerostat favorite. Always makes the firefights a little more chaotic.'",

        rarity = CARD_RARITY.RARE,
        ricochet_amt = 1,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local target_fighters = {}
                    battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, self.stacks )

                    for i=1, #target_fighters do
                        local target = target_fighters[i]
                        target:AddCondition("RICOCHET", self.graft:GetDef().ricochet_amt, self)
                    end
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end
            },
        },
    },

    glancer_plus =
    {
        name = "Boosted Glancer",
        desc = "At the start of each turn, apply <#UPGRADE>2</> {RICOCHET} to a random target.",
        ricochet_amt = 2,
    },

    study_guide =
    {
        name = "Study Guide",
        desc = "At the start of each turn, gain {CONCENTRATION 2}.",
        flavour = "'This will keep you focused on the task at hand.'",

        rarity = CARD_RARITY.RARE,
        concentration_amt = 2,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.owner:AddCondition("CONCENTRATION", self.graft:GetDef().concentration_amt, self)
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end
            },
        },
    },

    study_guide_plus =
    {
        name = "Boosted Study Guide",
        desc = "At the start of each turn, gain <#UPGRADE>{CONCENTRATION 3}</>.",
        concentration_amt = 3,
    },

    overflow = 
    {
        name = "Overflow",
        desc = "Whenever you deal the maximum damage of a card that has a damage range, draw a card.",
        flavour = "'You can’t even feel the kickback, use that space to find another target.'",

        rarity = CARD_RARITY.RARE,
        draw_count = 1,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.attacker == self.owner then
                        if hit.damage_dealt and hit.damage_dealt == hit.max_damage and attack.card and attack.card.min_damage ~= attack.card.max_damage then
                            battle:DrawCards(self.graft:GetDef().draw_count)
                            battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,
            },
        },
    },

    overflow_plus =
    {
        name = "Visionary Overflow",
        desc = "Whenever you deal the maximum damage of a card that has a damage range, draw <#UPGRADE>2</> cards.",
        draw_count = 2,
    },

    totem =
    {
        name = "Totem",
        desc = "While you have {CONCENTRATION}, all cards gain 2 maximum damage.",
        flavour = "'Rook! I didn’t know you were superstitious!.'",

        rarity = CARD_RARITY.RARE,
        bonus_damage = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and self.owner:HasCondition("CONCENTRATION") then
                        dmgt:AddDamage(0, self.graft:GetDef().bonus_damage, self)
                    end
                end
            },
        },
    },

    totem_plus =
    {
        name = "Boosted Totem",
        desc = "While you have {CONCENTRATION}, all cards gain <#UPGRADE>3</> maximum damage.",
        bonus_damage = 3,
    },

    forecaster =
    {
        name = "Forecaster",
        desc = "At the end of your turn, gain 4 {DEFEND} if you have any {CONCENTRATION}.",
        flavour = "'It’s actually a pretty simple targeting computer that has a predictive algorithm.'",

        rarity = CARD_RARITY.RARE,
        defend_amt = 4,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    if self.owner:HasCondition("CONCENTRATION") then
                        self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    forecaster_plus =
    {
        name = "Stone Forecaster",
        desc = "At the end of your turn, gain <#UPGRADE>6</> {DEFEND} if you have any {CONCENTRATION}.",
        defend_amt = 6,
    },

    flint_tips =
    {
        name = "Flint Tips",
        desc = "All attacks deal 1 bonus damage per stack of {SCORCHED} on the target.",
        flavour = "'I’ll do the whole box, no extra charge.'",

        rarity = CARD_RARITY.RARE,
        bonus_damage = 1,
        battle_condition =
        {   
            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and card:IsAttackCard() and target and target:HasCondition("SCORCHED") then
                        local count = target:GetConditionStacks("SCORCHED") * self.graft:GetDef().bonus_damage
                        dmgt:AddDamage(count, count, self)
                    end
                end
            },
        },
    },

    flint_tips_plus =
    {
        name = "Boosted Flint Tips",
        desc = "All attacks deal <#UPGRADE>2</> bonus damage per stack of {SCORCHED} on the target.",
        bonus_damage = 2,
    },

    splintershot =
    {
        name = "Splintershot",
        desc = "Whenever you spend your last {CHARGE}, apply {WOUND 2} to a random enemy.",
        flavour = "'Alright now these rounds need to be fired at the right cycle or else they’ll split your barrel right apart.'",

        rarity = CARD_RARITY.RARE,

        wound_amt = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.LUMIN_SPENT ] = function( self, battle, charges, max_cells )
                    if charges == 0 then
                        local target_fighters = {}
                        battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )

                        for i=1, #target_fighters do
                            local target = target_fighters[i]
                            target:AddCondition("WOUND", self.graft:GetDef().wound_amt, self)
                        end
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            },
            
        },
    },

    splintershot_plus =
    {
        name = "Deep Splintershot",
        desc = "Whenever you spend your last {CHARGE}, apply <#UPGRADE>{WOUND 3}</> to a random enemy.",
        wound_amt = 3,
    },

    un_marked_weapons =
    {
        name = "Un-Marked Weapons",
        desc = "Gain 1 action at the start of your turn. You no longer gain {DEFEND} from empty cells.",
        flavour = "'Look, I don’t want to tell you what to do or anything, but do NOT let the Admiralty catch you with these.'",

        rarity = CARD_RARITY.BOSS,

        card_draw = 0,


        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.owner.player_controlled then
                            self.battle:ModifyActionCount( self.stacks )
                            local draw = self.graft:GetDef().card_draw
                            if draw > 0 then self.battle:DrawCards(draw) end
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                            if not self.applied then
                                local tracker = self.owner:GetCondition("lumin_tracker")
                                if tracker then
                                    tracker.defend_per_charge = 0
                                end
                            end
                        end
                    end
                end,
            },
        },
    },

    un_marked_weapons_plus =
    {
        name = "Visionary Un-Marked Weapons",
        desc = "Gain 1 action<#UPGRADE> and draw a card</> at the start of your turn. You no longer gain {DEFEND} from empty cells.",
        card_draw = 1,
    },
}

---------------------------------------------------------------------------------------------

for i, id, graft in sorted_pairs( BATTLE_GRAFTS ) do
    graft.card_defs = battle_defs
    graft.series = "ROOK"
    graft.type = GRAFT_TYPE.COMBAT
    Content.AddGraft( id, graft )
end
