local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT
local CARD_FLAGS = negotiation_defs.CARD_FLAGS

local COIN_GRAFTS =
{
    -- BASIC
    spare_coin = 
    {
        name = "Spare Coin",
        desc = "{HEADS}: Apply 1 {COMPOSURE} to a random friendly argument.\n{SNAILS}: Deal 1 damage to a random opponent argument.",
        
        rarity = CARD_RARITY.BASIC,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_spare_heads.tex",
            snails_icon = "negotiation/modifiers/coin_spare_snails.tex",
            coin_build = "anim/coin_spare_build_ui_fx.zip",

            min_persuasion = 1,
            max_persuasion = 1,

            heads_effect = function( self, minigame )
                local targets = minigame:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    local target = targets[math.random(#targets)]
                    target:DeltaComposure(1, self)
                end
            end,
            snails_effect = function( self, minigame )
                self.target_enemy = TARGET_ANY_RESOLVE
                minigame:ApplyPersuasion( self )
                self.target_enemy = nil
                self:ClearTarget()
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    -- COMMON

    -- visionary_coin =
    -- {
    --     name = "Visionary Coin",
    --     desc = "{HEADS}: {PREPARE} a card.\n{SNAILS}:  Deal 2 damage to a random target.",
    --     rarity = CARD_RARITY.COMMON,

    --     negotiation_modifier =
    --     {
    --         heads_icon = "negotiation/modifiers/coin_visionary_heads.tex",
    --         snails_icon = "negotiation/modifiers/coin_visionary_snails.tex",
    --         heads_effect = function( self, minigame )
    --             minigame:PrepareCard( nil, self )
    --         end,
    --         snails_effect = function( self, minigame )
    --             local targets = minigame:CollectAlliedTargets(minigame:GetOtherNegotiator(self.negotiator))
    --             if #targets > 0 then
    --                 local target = targets[math.random(#targets)]
    --                 target:AttackResolve(2, self)
    --             end
    --         end,
    --         event_handlers =
    --         {
    --             [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
    --                 local coin = self.negotiator:FindModifier("LUCKY_COIN")
    --                 if coin then 
    --                     coin:SwapCoins( self )
    --                 end
    --                 self.negotiator:RemoveModifier(self.id, 1, self)
    --             end,
    --         }
    --     },
    -- },

    sifting_coin =
    {
        name = "Sifting Coin",
        desc = "{HEADS}: Draw a card.\n{SNAILS}: Discard a card.",
        
        rarity = CARD_RARITY.COMMON,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_sifting_heads.tex",
            snails_icon = "negotiation/modifiers/coin_sifting_snails.tex",
            coin_build = "anim/coin_sifting_build_ui_fx.zip",
            heads_effect = function( self, minigame )
                minigame:DrawCards(1)
            end,
            snails_effect = function( self, minigame )
                minigame:DiscardCards(1, nil, self)
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    marsh_coin =
    {
        name = "Marsh Coin",
        desc = "{HEADS}: Apply 2 {COMPOSURE} to a random friendly argument.\n{SNAILS}: Deal 1 damage to a random opponent argument.",
        
        rarity = CARD_RARITY.COMMON,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_marsh_heads.tex",
            snails_icon = "negotiation/modifiers/coin_marsh_snails.tex",
            coin_build = "anim/coin_marsh_build_ui_fx.zip",

            min_persuasion = 1,
            max_persuasion = 1,

            heads_effect = function( self, minigame )
                local targets = minigame:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    local target = targets[math.random(#targets)]
                    target:DeltaComposure(2, self)
                end
            end,
            snails_effect = function( self, minigame )
                self.target_enemy = TARGET_ANY_RESOLVE
                minigame:ApplyPersuasion( self )
                self.target_enemy = nil
                self:ClearTarget()
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    mud_coin =
    {
        name = "Mud Coin",
        desc = "{HEADS}: Apply 1 {COMPOSURE} to a random friendly argument.\n{SNAILS}: Deal 2 damage to a random opponent argument.",
        
        rarity = CARD_RARITY.COMMON,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_mud_heads.tex",
            snails_icon = "negotiation/modifiers/coin_mud_snails.tex",
            coin_build = "anim/coin_mud_build_ui_fx.zip",

            min_persuasion = 2,
            max_persuasion = 2,

            heads_effect = function( self, minigame )
                local targets = minigame:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    local target = targets[math.random(#targets)]
                    target:DeltaComposure(1, self)
                end
            end,
            snails_effect = function( self, minigame )
                self.target_enemy = TARGET_ANY_RESOLVE
                minigame:ApplyPersuasion( self )
                self.target_enemy = nil
                self:ClearTarget()
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    -- UNCOMMON

    bog_coin =
    {
        name = "Bog Coin",
        desc = "{HEADS}: Apply 2 {COMPOSURE} to a random friendly argument.\n{SNAILS}: Deal 2 damage to a random opponent argument.",
        
        rarity = CARD_RARITY.UNCOMMON,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_bog_heads.tex",
            snails_icon = "negotiation/modifiers/coin_bog_snails.tex",
            coin_build = "anim/coin_bog_build_ui_fx.zip",

            min_persuasion = 2,
            max_persuasion = 2,

            heads_effect = function( self, minigame )
                local targets = minigame:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    local target = targets[math.random(#targets)]
                    target:DeltaComposure(2, self)
                end
            end,
            snails_effect = function( self, minigame )
                self.target_enemy = TARGET_ANY_RESOLVE
                minigame:ApplyPersuasion( self )
                self.target_enemy = nil
                self:ClearTarget()
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },
    
    seekers_coin =
    {
        name = "Seeker's Coin",
        desc = "{HEADS}: Draw a card.\n{SNAILS}: Deal 2 damage to a random opponent argument.",
        
        rarity = CARD_RARITY.UNCOMMON,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_seekers_heads.tex",
            snails_icon = "negotiation/modifiers/coin_seekers_snails.tex",
            coin_build = "anim/coin_seekers_build_ui_fx.zip",

            min_persuasion = 2,
            max_persuasion = 2,

            heads_effect = function( self, minigame )
                minigame:DrawCards(1)
            end,
            snails_effect = function( self, minigame )
                self.target_enemy = TARGET_ANY_RESOLVE
                minigame:ApplyPersuasion( self )
                self.target_enemy = nil
                self:ClearTarget()
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    emissary_coin =
    {
        name = "Emissary Coin",
        desc = "{HEADS}: Gain 1 {INFLUENCE}.\n{SNAILS}: Deal 2 damage to a random opponent argument.",
        
        rarity = CARD_RARITY.UNCOMMON,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_emmisary_heads.tex",
            snails_icon = "negotiation/modifiers/coin_emmisary_snails.tex",
            coin_build = "anim/coin_emissary_build_ui_fx.zip",

            min_persuasion = 2,
            max_persuasion = 2,

            heads_effect = function( self, minigame )
                self.negotiator:AddModifier("INFLUENCE", 1, self)
            end,
            snails_effect = function( self, minigame )
                self.target_enemy = TARGET_ANY_RESOLVE
                minigame:ApplyPersuasion( self )
                self.target_enemy = nil
                self:ClearTarget()
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    psychic_coin =
    {
        name = "Psychic Coin",
        desc = "{HEADS}: Incept 1 {DOUBT}.\n{SNAILS}: Deal 2 damage to a random opponent argument.",
        
        rarity = CARD_RARITY.UNCOMMON,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_psychic_heads.tex",
            snails_icon = "negotiation/modifiers/coin_psychic_snails.tex",
            coin_build = "anim/coin_psychic_build_ui_fx.zip",

            min_persuasion = 2,
            max_persuasion = 2,

            heads_effect = function( self, minigame )
                self.anti_negotiator:InceptModifier( "DOUBT", 1, self )
            end,
            snails_effect = function( self, minigame )
                self.target_enemy = TARGET_ANY_RESOLVE
                minigame:ApplyPersuasion( self )
                self.target_enemy = nil
                self:ClearTarget()
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    -- RARE
    turtle_coin =
    {
        name = "Turtle Coin",
        desc = "{HEADS}: Apply 3 {COMPOSURE} to your core argument.\n{SNAILS}: Apply 1 {COMPOSURE} to all friendly arguments.",
        
        rarity = CARD_RARITY.RARE,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_turtle_heads.tex",
            snails_icon = "negotiation/modifiers/coin_turtle_snails.tex",
            coin_build = "anim/coin_turtle_build_ui_fx.zip",
            heads_effect = function( self, minigame )
                self.negotiator:DeltaComposure(3, self)
            end,
            snails_effect = function( self, minigame )
                local targets = minigame:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    for i,target in ipairs(targets) do
                        target:DeltaComposure(1, self)
                    end
                end
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    tyrants_coin =
    {
        name = "Tyrant's Coin",
        desc = "{HEADS}: Apply 2 {COMPOSURE} to a random friendly argument.\n{SNAILS}: Gain {DOMINANCE 1}.",
        
        rarity = CARD_RARITY.RARE,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_tyrant_heads.tex",
            snails_icon = "negotiation/modifiers/coin_tyrant_snails.tex",
            coin_build = "anim/coin_tyrant_build_ui_fx.zip",
            heads_effect = function( self, minigame )
                local targets = minigame:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    local target = targets[math.random(#targets)]
                    target:DeltaComposure(2, self)
                end
            end,
            snails_effect = function( self, minigame )
                self.negotiator:AddModifier("DOMINANCE", 1, self)
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    chaos_coin =
    {
        name = "Chaos Coin",
        desc = "{HEADS}: Apply 2 {COMPOSURE} to a random friendly argument.\n{SNAILS}: Draw a card. Discard a card.",
        
        rarity = CARD_RARITY.RARE,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_chaos_heads.tex",
            snails_icon = "negotiation/modifiers/coin_chaos_snails.tex",
            coin_build = "anim/coin_chaos_build_ui_fx.zip",
            heads_effect = function( self, minigame )
                local targets = minigame:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    local target = targets[math.random(#targets)]
                    target:DeltaComposure(2, self)
                end
            end,
            snails_effect = function( self, minigame )
                minigame:DrawCards(1)
                minigame:DiscardCards(1, nil, self)
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then 
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },

    krogs_weighted_coin =
    {
        name = "Krog's Weighted Coin",
        desc = "Comes up {HEADS} 90% of the time.",
        
        rarity = CARD_RARITY.UNIQUE,

        negotiation_modifier =
        {
            heads_icon = "negotiation/modifiers/coin_krogs_heads.tex",
            snails_icon = "negotiation/modifiers/coin_krogs_snails.tex",
            coin_build = "anim/coin_krogs_build_ui_fx.zip",
            heads_chance = 90,
            
            heads_effect = function( self, minigame )
            end,
            snails_effect = function( self, minigame )
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin then
                        coin:SwapCoins( self )
                    end
                    self.negotiator:RemoveModifier(self.id, 1, self)
                end,
            }
        },
    },    
}


for id, graft in pairs( COIN_GRAFTS ) do
    graft.card_defs = negotiation_defs
    graft.series = "ROOK"
    
    if graft.negotiation_modifier and graft.negotiation_modifier.modifier_type == nil then
        graft.negotiation_modifier.modifier_type = MODIFIER_TYPE.HIDDEN
    end
    graft.hide_on_day_summary = true
    graft.type = GRAFT_TYPE.COIN
    Content.AddGraft( id, graft )
end
