local battle_defs = require "battle/battle_defs"

local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

--------------------------------------------------------------------

local BATTLE_GRAFTS =
{

    -----------------------------------------------------------------
    -- COMMONS

    dark_cowl =
    {
        name = "Dark Cowl",
        flavour = "'Did you hear something?'",
        desc = "Start battle with {COMBO 3}.",
        rarity = CARD_RARITY.COMMON,
        combo_amt = 3,
        OnActivateFighter = function(self, fighter)
            fighter:AddCondition( "COMBO", self:GetDef().combo_amt )
        end,
    },

    dark_cowl_plus =
    {
        name = "Rival's Dark Cowl",
        desc = "Start battle with <#UPGRADE>{COMBO 5}</>.",
        combo_amt = 5,
    },

    first_blood = 
    {
        name = "First Blood",
        flavour = "'Spring loaded blades are commonly used by the more dishonorable street fighters.'",
        desc = "Your first attack each battle deals an extra 4 {BLEED}.",
        rarity = CARD_RARITY.COMMON,
        card_defs = battle_defs,

        bleed_amt = 4,

        battle_condition =
        {
            hidden = false,
            desc = "Your first attack each battle deals an extra {1} {BLEED}.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.graft:GetDef().bleed_amt or 4)
            end,
            event_handlers =
            {
                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if attack.attacker == self.owner and attack.card:IsAttackCard() then
                        attack:AddCondition("BLEED", self.graft:GetDef().bleed_amt * self.stacks, attack.card)
                        self.owner:RemoveCondition( self.id )
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    first_blood_plus =
    {
        name = "Boosted First Blood",
        desc = "Your first attack each battle deals an extra <#UPGRADE>6</> {BLEED}.",
        bleed_amt = 6,
    },

    snap_talon =
    {
        name = "Snap Talon",
        flavour = "'A hybrid technology that incorporates the claws of a Vroc with laser sharpening.'",
        desc = "Your attacks deal 2 bonus damage to targets with {BLEED}.",
        rarity= CARD_RARITY.COMMON,

        bonus_damage = 2,

        battle_condition =
        {   
            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and card:IsAttackCard() and target and target:HasCondition("BLEED") then
                        self.bonus_damage = self.bonus_damage or self.graft:GetDef().bonus_damage or 1
                        dmgt:AddDamage(self.bonus_damage, self.bonus_damage, self)
                    end
                end
            },
        },
    },

    snap_talon_plus =
    {
        name = "Boosted Snap Talon",
        desc = "Your attacks deal <#UPGRADE>3</> bonus damage to targets with {BLEED}.",
        bonus_damage = 3,
    },

    nailed_glove =
    {
        name = "Nailed Glove",
        flavour = "'Usually used in the Derrick fighting pits, these gloves are inexpensive and have a little extra bite.'",
        desc = "When receiving damage, you only lose up to 4 {COMBO}.",
        rarity = CARD_RARITY.UNCOMMON,
        mod_xp = 4,
        combo_loss = 4,

        battle_condition = 
        {
            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_COMBO_LOSS ] = function( self, acc, source, target )
                    if target == self.owner then
                        if acc.value > 0 then
                            acc:ModifyValue( math.min(self.graft:GetDef().combo_loss, acc.value), self )
                        end
                    end
                end
            },
        },
    },

    nailed_glove_plus =
    {
        name = "Boosted Nailed Glove",
        desc = "When receiving damage, you only lose up to <#UPGRADE>2</> {COMBO}.",
        combo_loss = 2,
    },

    buster = 
    {
        name = "Buster",
        flavour = "'The designer was just sick to death of not being able to get through those armored bandits.'",
        desc = "Whenever you break {DEFEND} with an attack, apply {BLEED 2} and gain {DEFEND 4}.",
        card_defs = battle_defs,
        rarity = CARD_RARITY.COMMON,
        bleed_amt = 2,
        defend_amt = 4,
        battle_condition = 
        {

            event_handlers =
            {
                [ BATTLE_EVENT.CONDITION_REMOVED ] = function( self, fighter, condition, stacks, source )
                    if condition:GetID() == "DEFEND" and condition:GetStacks() == 0 then
                        if is_instance( source, Battle.Hit ) and source.card:IsAttackCard() and source.attacker == self.owner then
                            --
                            fighter:AddCondition("BLEED", self.graft:GetDef().bleed_amt * self.stacks, self)
                            self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt * self.stacks, self)
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,
            },
       },
    },

    buster_plus =
    {
        name = "Savage Buster",
        desc = "Whenever you break {DEFEND} with an attack, apply <#UPGRADE>{BLEED 4}</> and gain {DEFEND 4}.",
        bleed_amt = 4,
    },
      
    predictive_brawling = 
    {
        name = "Predictive Brawling",
        flavour = "'There's just only so many ways somebody can throw a punch.",
        desc = "Gain {COMBO 1} at the beginning of your turn.",
        card_defs = battle_defs,
        rarity = CARD_RARITY.COMMON,
        combo_gain = 1,

        battle_condition =
        {
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.owner:AddCondition("COMBO", self.graft:GetDef().combo_gain, self)
                end
            },
        },
    },

    predictive_brawling_plus =
    {
        name = "Rival's Predictive Brawling",
        desc = "Gain <#UPGRADE>{COMBO 2}</> at the beginning of your turn.",
        combo_gain = 2,
    },
  
    Buckshot =
    {
        name = "Buckshot",
        flavour = "'Reloads quickly and fits right in your pocket. Just don't bother aiming.'",
        desc = "Apply {WOUND 1} to a random enemy at the beginning of your turn.",
        rarity = CARD_RARITY.COMMON,

        wound_amt = 1,

        battle_condition = 
        {
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        local target_fighters = {}
                        fighter.battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )

                        for i=1, #target_fighters do
                            local target = target_fighters[i]
                            target:AddCondition("WOUND", self.graft:GetDef().wound_amt, self)
                        end
                    end
                end,
            },
            
        },
    },

    Buckshot_plus =
    {
        name = "Boosted Buckshot",
        desc = "Apply <#UPGRADE>{WOUND 2}</> to a random enemy at the beginning of your turn.",
        wound_amt = 2,
    },

    recycler = 
    {
        name = "Recycler",
        flavour = "'Previously, all that blood and sweat was just wasted.'",
        desc = "The first time you {EXPEND} a card each turn, draw a card.",
        rarity = CARD_RARITY.COMMON,
        card_defs = battle_defs,
        draw_count = 1,
        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle, card )
                    self.has_triggered = false
                end,

                [ BATTLE_EVENT.CARD_EXPENDED ] = function( self, battle, card )
                    if card.owner == self.owner and not self.has_triggered then
                        battle:DrawCards(self.graft:GetDef().draw_count)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        self.has_triggered = true
                    end
                end,
            }
        },
    },

    recycler_plus =
    {
        name = "Visionary Recycler",
        desc = "The first time you {EXPEND} a card each turn, draw <#UPGRADE>2 cards</>.",
        draw_count = 2,
    },

    shuffle_step =
    {
        name = "Shuffle Step",
        desc = "Whenever you {IMPROVISE} a maneuver card, gain {1} {DEFEND}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:GetDef().defend_amt)
        end,
        flavour = "'Keeps you a little quicker on your feet.'",

        rarity = CARD_RARITY.COMMON,

        defend_amt = 2,

        battle_condition =
        {
            hidden = true,

            event_handlers =
            {
                [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                    if card:IsFlagged(CARD_FLAGS.SKILL) then
                        self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            }
        },
    },

    shuffle_step_plus =
    {
        name = "Stone Shuffle Step",
        desc = "Whenever you {IMPROVISE} a maneuver card, gain <#UPGRADE>{1}</> {DEFEND}.",
        defend_amt = 3,
    },

    -----------------------------------------------------------------
    -- UNCOMMONS

    wounding_barbs = 
    {
        name = "Wounding Barbs",
        flavour = "'Nasty things these.'",
        desc = "Your first attack each battle applies {WOUND 3}.",
        card_defs = battle_defs,
        rarity = CARD_RARITY.UNCOMMON,
        wound_amt = 3,
        battle_condition = 
        {
            hidden = false,
            event_handlers =
            {
                [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
                    if attack.attacker == self.owner and attack.card:IsAttackCard() and attack.target:GetTeam() ~= self.owner:GetTeam() then
                        attack:AddCondition("WOUND", self.graft:GetDef().wound_amt * self.stacks, attack.card)
                        self.owner:RemoveCondition( self.id, 1 )
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    wounding_barbs_plus =
    {
        name = "Deep Wounding Barbs",
        desc = "Your first attack each battle applies <#UPGRADE>{WOUND 4}</>.",
        wound_amt = 4,
    },

    body_piercing =
    {
        name = "Body Piercing",
        flavour = "'Good luck getting a job there pal.",
        desc = "At the end of every turn, gain {RIPOSTE 1} for every 3 {DEFEND} you have.",
        rarity = CARD_RARITY.UNCOMMON,
        riposte_amt = 1,

        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        local count = self.owner:GetConditionStacks("DEFEND")
                        if count > 3 then
                            self.owner:AddCondition( "RIPOSTE", math.floor(count / 3) * self.graft:GetDef().riposte_amt , self)
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,
            },
        },
    },

    body_piercing_plus =
    {
        name = "Thorny Body Piercing",
        desc = "At the end of every turn, gain <#UPGRADE>{RIPOSTE 2}</> for every 3 {DEFEND} you have.",
        riposte_amt = 2,
    },
    
    shell =
    {
        name = "Shell",
        rarity = CARD_RARITY.UNCOMMON,
        flavour = "'Great for people with no friends!'",
        card_defs = battle_defs,
        desc = "If you are outnumbered, gain {DEFEND 4} and {RIPOSTE 3} at the end of your turn.",
        riposte_amt = 3,
        defend_amt = 4,
        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    local count = #self.owner:GetTeam():GetFighters()
                    local enemy_count = #self.owner:GetEnemyTeam():GetFighters()
                    if count < enemy_count then
                        self.owner:AddCondition( "DEFEND", self.graft:GetDef().defend_amt, self )
                        self.owner:AddCondition( "RIPOSTE", self.graft:GetDef().riposte_amt, self )
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            }
        },
    },

    shell_plus =
    {
        name = "Thorny Shell",
        desc = "If you are outnumbered, gain <#UPGRADE>{DEFEND 6}</> and <#UPGRADE>{RIPOSTE 4}</> at the end of your turn.",
        riposte_amt = 4,
        defend_amt = 6,
    },

    skeletal_suspension = 
    {
        name = "Skeletal Suspension",
        flavour = "'It runs along the femur, your opponent is going to feel it more than you are.",
        desc = "At the end of your turn, gain {DEFEND} equal to half of your {COMBO}.",
        card_defs = battle_defs,
        rarity = CARD_RARITY.UNCOMMON,
        combo_conversion = .5,

        battle_condition = 
        {
            event_priorities =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = 10, -- Before Dangerous Bulb
            },

            event_handlers = 
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    if self.owner:GetConditionStacks("COMBO") > 0 then
                        self.owner:AddCondition( "DEFEND", math.floor(self.owner:GetConditionStacks("COMBO") * self.graft:GetDef().combo_conversion), self )
                    end
                end
            },
        },
    },

    skeletal_suspension_plus =
    {
        name = "Stone Skeletal Suspension",
        desc = "At the end of your turn, gain {DEFEND} equal to <#UPGRADE>75%</> of your {COMBO}.",
        combo_conversion = .75,
    },

    serrated_edge = 
    {
        name = "Serrated Edge",
        flavour = "'I didn't think this thing could get through a shroke's hide, sometimes I surprise myself.'",
        desc = "{WOUND} is twice as effective.",
        card_defs = battle_defs,
        rarity = CARD_RARITY.UNCOMMON,
        wound_bonus = 1,
        mod_xp = 2,

        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and target then
                        local wound_stacks = target:GetConditionStacks("WOUND")
                        dmgt:ModifyDamage( dmgt.min_damage + wound_stacks, dmgt.max_damage + wound_stacks, self )
                    end
                end,
            }
        },
    },

    serrated_edge_plus = 
    {
        name = "Boosted Serrated Edge",
        desc = "{WOUND} is twice as effective <#UPGRADE>and {IMPAIR} reduces damage by 50%.</>",
        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if self.owner:GetConditionStacks("SERRATED_EDGE") and target then
                        local wound_stacks = target:GetConditionStacks("WOUND")
                        dmgt:ModifyDamage( dmgt.min_damage + wound_stacks, dmgt.max_damage + wound_stacks )
                    end
                end,

                [ BATTLE_EVENT.CONDITION_ADDED ] = function( self, fighter, condition, stacks, source )
                    if condition.id == "IMPAIR" and source and source.owner == self.owner then
                        condition.damage_mult = .5
                    end
                end
            }
        },
    },
    
    arterial_scanner = 
    {
        name = "Arterial Scanner",
        desc = "{BLEED} decays 33% at the end of the turn instead of 50%.",
        flavour = "'What do you mean there's been a software update? Did people's veins move around or something?'",
        card_defs = battle_defs,
        rarity = CARD_RARITY.UNCOMMON,
        decay_rate = .33,

        battle_condition = 
        {
            GetDecayRate = function( self )
                return self.graft:GetDef().decay_rate
            end,
        },
    },

    arterial_scanner_plus =
    {
        name = "Boosted Arterial Scanner",
        desc = "{BLEED} decays <#UPGRADE>20%</> at the end of the turn instead of 50%.",
        decay_rate = .2,
    },

    unstable_augmentation =
    {
        name = "Unstable Augmentation",
        desc = "Gain 3 {TEMP_POWER} on the 3rd, 6th and 9th turns of battle.",
        flavour = "'Take two quarter-doses three times daily, how many times do I have to tell you?'' ",
        rarity = CARD_RARITY.UNCOMMON,

        power_amt = 3,

        battle_condition = 
        {
            hidden = true,
            turn_options = {3,6,9},

            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    if table.contains(self.turn_options, battle.turns) then
                        self.owner:AddCondition("POWER", self.graft:GetDef().power_amt * self.stacks, self)
                        self.owner:AddCondition("POWER_LOSS", self.graft:GetDef().power_amt * self.stacks, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft)
                    end
                end,
            },
        },
    },

    unstable_augmentation_plus = 
    {
        name = "Boosted Unstable Augmentation",
        desc =  "Gain <#UPGRADE>5</> {TEMP_POWER} on the 3rd, 6th and 9th turn of battle.",
        power_amt = 5,
    },

    slider =
    {
        name = "Slider",
        desc = "At the start of every 3rd turn, {IMPROVISE} a card from your draw pile.",
        rarity = CARD_RARITY.UNCOMMON,
        flavour = "'These take a bit of time to acclimatize to the user but the results are always impressive.'",

        frequency = 3,

        battle_condition =
        {
            hidden = true,
            loc_strings =
            {
                SLIDER_IMPROVISE = "Improvise a card from your draw pile"
            },

            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    if battle:GetTurns() % self.graft:GetDef().frequency == 0 then
                        local cards = {}
                        for i, card in battle:GetDrawDeck():Cards() do
                            table.insert(cards, card)
                        end
                        if #cards == 0 then
                            battle:ShuffleDiscardToDraw()
                            for i, card in battle:GetDrawDeck():Cards() do
                                table.insert(cards, card)
                            end
                        end
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        battle:BroadcastEvent( BATTLE_EVENT.DELAY, 0.3 ) -- lets the draw completely settle into your hand.
                        battle:ImproviseCards( table.multipick(cards, 3), 1, nil, false, self:GetLocalizedString("SLIDER_IMPROVISE"), self )
                    end
                end
            },
        },
    },

    slider_plus =
    {
        name = "Boosted Slider",
        desc = "At the start of every <#UPGRADE>2nd</> turn, {IMPROVISE} a card from your draw pile.",
        frequency = 2,
    },

    -----------------------------------------------------------------
    -- RARE
    perpetual_recycler = 
    {
        name = "Perpetual Recycler",
        desc = "Whenever you {EXPEND} a card, draw a card.",
        flavour = "'No, you can't turn it off. You'll get used to the noise.'",
        rarity = CARD_RARITY.RARE,
        card_defs = battle_defs,
        mod_xp = 3,
        draw_count = 1,

        battle_condition =
        {
            event_handlers =
            {
                [ BATTLE_EVENT.CARD_EXPENDED ] = function( self, battle, card )
                    if card.owner == self.owner then
                        battle:DrawCards(self.graft:GetDef().draw_count)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            }
        },
    },

    perpetual_recycler_plus =
    {
        name = "Visionary Perpetual Recycler",
        desc = "Whenever you {EXPEND} a card, draw <#UPGRADE>2 cards</>.",
        draw_count = 2,
    },

    bloody_mess = 
    {
        name = "Bloody Mess",
        desc = "At the start of your turn, apply {BLEED 1} to all enemies.",
        flavour = "'There isn't a lot of elegance that goes into making one of these. It's also frowned on in pretty much every circle.'",
        rarity = CARD_RARITY.RARE,
        card_defs = battle_defs,
        bleed_amt = 1,

        battle_condition = 
        {
            event_handlers =
            {
                
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local target_fighters = battle:GetEnemyTeam().fighters
                    for i,target in ipairs(target_fighters) do
                        target:AddCondition("BLEED", self.graft:GetDef().bleed_amt, self)
                        target:PlayAnim( "hit_mid" )
                    end
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end
            },
        },
    },

    bloody_mess_plus =
    {
        name = "Savage Bloody Mess",
        desc = "At the start of your turn, apply <#UPGRADE>{BLEED 2}</> to all enemies.",
        bleed_amt = 2,
    },

    hemophage = 
    {
        name = "Hemophage",
        desc = "Whenever you apply {BLEED}, restore 1 health.",
        flavour = "'It's best not to think about it. The mechanisms involved are downright gruesome.'",
        rarity = CARD_RARITY.RARE,

        OnActivateFighter = function( self, fighter )
            fighter:AddCondition( "HEMOPHAGE" )
        end,
    },

    hemophage_plus =
    {
        name = "Leeching Hemophage",
        desc = "Whenever you apply {BLEED}, restore <#UPGRADE>2</> health.",
        OnActivateFighter = function( self, fighter )
            fighter:AddCondition( "HEMOPHAGE", 2, self)
        end,
    },

    fragmenter =
    {
        name = "Fragmenter",
        desc = "Whenever you discard a card, apply 1 {WOUND} to a random enemy.",
        flavour = "'Basic explosives usually found in the more colorful areas of Havaria. Very illegal.'",
        rarity = CARD_RARITY.RARE,

        wound_amt = 1,
        impair_amt = 0,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.CARD_DISCARDED ] = function( self, card, battle )
                    if card.owner == self.owner then
                        local target_fighters = {}
                        battle:CollectRandomTargets( target_fighters, self.owner:GetEnemyTeam().fighters, 1 )

                        for i=1, #target_fighters do
                            local target = target_fighters[i]
                            target:AddCondition("WOUND", self.graft:GetDef().wound_amt, self)
                            if self.graft:GetDef().impair_amt > 0 then
                                target:AddCondition("IMPAIR", self.graft:GetDef().impair_amt, self)
                            end
                        end
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            },
        },
    },

    fragmenter_plus =
    {
        name = "Deep Fragmenter",
        desc = "Whenever you discard a card, apply 1 {WOUND} <#UPGRADE>and 1 {IMPAIR}</> to a random enemy.",
        impair_amt = 1
    },

    best_offense =
    {
        name = "Best Offense",
        desc = "Whenever you spend all your combo, gain 5 {DEFEND}.",
        flavour = "'It's all about the follow through.'",
        rarity = CARD_RARITY.RARE,

        defend_amt = 5,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CONDITION_REMOVED ] = function( self, fighter, condition, stacks, source )
                    if condition.owner == self.owner and condition.id == "COMBO" and condition.stacks == 0 then
                        self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            },
        },
    },

    best_offense_plus =
    {
        name = "Stone Best Offense",
        desc = "Whenever you spend all your combo, gain <#UPGRADE>8</> {DEFEND}.",
        defend_amt = 8,
    },

    counter_band =
    {
        name = "Counter Band",
        desc = "Whenever you prevent all damage from an attack, gain 2 {COMBO}.",
        flavour = "'That's about as reinforced as I can make it while keeping the weight down.'",
        rarity = CARD_RARITY.RARE,
        combo_amt = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if hit.target == self.owner and (hit.defended or hit.evaded) then 
                        self.owner:AddCondition("COMBO", self.graft:GetDef().combo_amt, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            },
        },
    },

    counter_band_plus = 
    {
        name = "Rival's Counter Band",
        desc = "Whenever you prevent all damage from an attack, gain <#UPGRADE>3</> {COMBO}.",
        combo_amt = 3,
    },

    clot_hammer = 
    {
        name = "Clot Hammer",
        flavour = "'Remember to keep the chamber clean. These things are finicky.'",
        desc = "Attacking targets affected by {BLEED} grants 2 {DEFEND}.",
        rarity = CARD_RARITY.RARE,
        defend_amt = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if attack.attacker == self.owner then
                        if hit.target:HasCondition("BLEED") then
                            self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                            battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,
            },
        },
    },

    clot_hammer_plus =
    {
        name = "Stone Clot Hammer",
        desc = "Attacking targets affected by {BLEED} grants <#UPGRADE>3</> {DEFEND}.",
        defend_amt = 3,
    },

    cloister =
    {
        name = "Cloister",
        flavour = "'There's an even better reason to keep your guard up with this. Those spark fuses should last a while.'",
        desc = "Deal 2 bonus damage while you have any {DEFEND}.",
        rarity = CARD_RARITY.RARE,

        bonus_damage = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and self.owner:HasCondition("DEFEND") then
                        dmgt:AddDamage(self.graft:GetDef().bonus_damage, self.graft:GetDef().bonus_damage, self)
                    end
                end
            },
        },
    },

    cloister_plus =
    {
        name = "Boosted Cloister",
        desc = "Deal <#UPGRADE>3</> bonus damage while you have any {DEFEND}.",
        bonus_damage = 3,
    },

    heartless = 
    {
        name = "Heartless",
        flavour = "'It's named after how you leave your opponent.'",
        desc = "If you are under 30% health, all attacks inflict 2 {WOUND}.",
        rarity = CARD_RARITY.RARE,

        wound_amt = 2,

        battle_condition =
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.ON_HIT ] = function( self, battle, attack, hit )
                    if self.owner == attack.owner and not hit.evaded and attack.card:IsAttackCard() and self.owner:GetHealthPercent() < .30 then
                        hit.target:AddCondition("WOUND", self.graft:GetDef().wound_amt, self)
                    end
                end
            },
        },
    },

    heartless_plus =
    {
        name = "Deep Heartless",
        desc = "If you are under 30% health, all attacks inflict <#UPGRADE>3</> {WOUND}.",
        wound_amt = 3,
    },

    synapse_board =
    {
        name = "Synapse Board",
        desc = "Whenever you {IMPROVISE} an attack card, gain {1} {TEMP_POWER}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:GetDef().power_amt)
        end,
        flavour = "'I don’t know where those attacks were coming from, it was like fighting three people at once.'", 

        rarity = CARD_RARITY.RARE,
        power_amt = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CARD_IMPROVISED ] = function( self, card, battle )
                    if card:IsAttackCard() then
                        self.owner:AddCondition("POWER", self.graft:GetDef().power_amt, self)
                        self.owner:AddCondition("POWER_LOSS", self.graft:GetDef().power_amt, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            },
        },
    },


    synapse_board_plus =
    {
        name = "Boosted Synapse Board",
        desc = "Whenever you {IMPROVISE} an attack card, gain <#UPGRADE>{1}</> {TEMP_POWER}.",
        power_amt = 3,
    },

    truncheon =
    {
        name = "Truncheon",
        desc = "Gain 1 action at the start of your turn. {BLEED} you apply is removed completely instead of being halved.",
        flavour = "'A relic from a simpler time. There’s only a couple ways to use the thing.'",

        rarity = CARD_RARITY.BOSS,

        card_draw = 0,


        battle_condition =
        {
            event_priorities =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = 11, -- After Bleed
            },

            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.owner.player_controlled then
                            self.battle:ModifyActionCount( self.stacks )
                            local draw = self.graft:GetDef().card_draw
                            if draw > 0 then self.battle:DrawCards(draw) end
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                    local bleed = fighter:GetCondition("BLEED")
                    local applicator = bleed and self.battle:GetFighterFromSource( bleed:GetRecentApplicant() )
                    if applicator == self.owner then
                        fighter:RemoveCondition("BLEED")
                    end
                end,
            },
        },
    },

    truncheon_plus =
    {
        name = "Visionary Truncheon",
        desc = "Gain 1 action<#UPGRADE> and draw a card</> at the start of your turn, {BLEED} you apply is removed completely instead of being halved.",
        card_draw = 1,
    },
}

---------------------------------------------------------------------------------------------

for i, id, graft in sorted_pairs( BATTLE_GRAFTS ) do
    graft.card_defs = battle_defs
    graft.series = "SAL"
    graft.type = GRAFT_TYPE.COMBAT
    if graft.battle_condition and graft.battle_condition.hidden == nil then
        graft.battle_condition.hidden = true
    end

    Content.AddGraft( id, graft )
end
