local battle_defs = require "battle/battle_defs"

local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

--------------------------------------------------------------------

local BATTLE_GRAFTS =
{
    --COMMONS
    two_fisted =
    {
        name = "Two Fisted",
        desc = "At the start of battle, add 2 {empty_bottle} cards to your discard pile.",
        flavour = "'What can I say, there was a sale on.'",
        rarity = CARD_RARITY.COMMON,
        bottle_count = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    local cards = {}
                    for i=1, self.graft:GetDef().bottle_count do
                        local card = Battle.Card( "empty_bottle", self.owner )
                        card.show_dealt = false
                        table.insert(cards, card)
                    end
                    battle:DealCards( cards, battle:GetDiscardDeck() )
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    self.owner:RemoveCondition(self.id)
                end
            },
        },
    },

    two_fisted_plus =
    {
        name = "Double Two Fisted",
        desc = "At the start of battle, add <#UPGRADE>4</> {empty_bottle} cards to your discard pile.",
        bottle_count = 4,
    },

    chafe = 
    {
        name = "Chafe",
        desc = "At the start of your turn, take 1 piercing damage.",
        flavour = "'Oooh yeah, that’s the spot.'",
        rarity = CARD_RARITY.COMMON,

        damage_amt = 1,
        moxie_gain = 0,

        battle_condition =
        {
            event_priorities =
            {
                -- after: masochist
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN  ] = 11,
            },
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    if self.owner:GetHealth() > self.graft:GetDef().damage_amt then
                        self.owner:SelfDamage(self.graft:GetDef().damage_amt, self, true)
                    end
                    if self.graft:GetDef().moxie_gain then
                        if not self.moxie then
                            self.moxie = self.owner:GetCondition("smith_regeneration")
                        end
                        if self.moxie then
                            self.moxie:IncreaseMax(self.graft:GetDef().moxie_gain, self)
                        end
                    end
                end
            },
        },
    },

    chafe_plus =
    {
        name = "Boosted Chafe",
        desc = "At the start of your turn, take 1 piercing damage <#UPGRADE>and increase your {smith_regeneration} by 1.</>",
        moxie_gain = 1,
    },

    piston =
    {
        name = "Piston",
        desc = "Whenever you play 3 attacks in a row, gain 2 {DEFEND}.",
        flavour = "'You can probably feel it in your bones because that’s where we put it.'",
        rarity = CARD_RARITY.COMMON,

        defend_amt = 2,

        display_number = function( self )
            return self.userdata.counter or 0
        end,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, card )
                    if card.owner == self.owner then
                        if card:IsAttackCard() then
                            self.graft:IncrementCounter()
                            if self.graft:GetCounter() >= 3 then
                                self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                                battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                                self.graft:ResetCounter()
                            end
                        else
                            self.graft:ResetCounter()
                        end
                        battle:BroadcastEvent( BATTLE_EVENT.REFRESH_GRAFTS )
                    end
                end
            },
        },
    },

    piston_plus =
    {
        name = "Stone Piston",
        desc = "Whenever you play 3 attacks in a row, gain <#UPGRADE>4</> {DEFEND}.",
        defend_amt = 4,
    },

    razorglass =
    {
        name = "Razorglass",
        desc = "Whenever you play an {empty_bottle} card, take 2 damage.",
        flavour = "'Extremely fragile and extremely sharp, are you sure you want to be carrying all those around?.'",

        rarity = CARD_RARITY.COMMON,
        damage_amt = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, resolve_card )
                    if resolve_card.id == "empty_bottle" then
                        self.owner:SelfDamage(self.graft:GetDef().damage_amt, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end 
            },
        },
    },
    razorglass_plus =
    {
        name = "Softened Razorglass",
        desc = "Whenever you play an {empty_bottle} card, take <#UPGRADE>1 damage</>.",
        damage_amt = 1,
    },

    -- UNCOMMONS
    fleximator =
    {
        name = "Fleximator",
        desc = "At the end of your turn, gain {DEFEND} equal to half of your {RECOVERABLE_HEALTH}.",
        flavour = "'I’m still trying to figure out your healing variable but this should at least cauterize your injuries a little faster.'",

        rarity = CARD_RARITY.UNCOMMON,

        multiplier = .5,

        battle_condition = 
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    if self.owner:HasCondition("RECOVERABLE_HEALTH") then
                        self.owner:AddCondition("DEFEND", math.round(self.graft:GetDef().multiplier * self.owner:GetConditionStacks("RECOVERABLE_HEALTH")), self)
                    end
                end
            },
        },
    },

    fleximator_plus = 
    {
        name = "Boosted Fleximator",
        desc = "At the end of your turn, gain {DEFEND} equal to <#UPGRADE>75%</> of your {RECOVERABLE_HEALTH}.",
        multiplier = .75,
    },

    collateralator = 
    {
        name = "Collateralator",
        desc = "{empty_bottle} cards deal 1 damage to a random enemy target when played, but also gain {RESTRAINED}.",
        flavour = "'This simple mechanical catch and sling can hurl those spare bottles for you.'",

        rarity = CARD_RARITY.UNCOMMON,
        min_damage = 1,
        max_damage = 1,

        battle_condition =
        {
            hidden = true,
            event_handlers = 
            {
                [ BATTLE_EVENT.CARD_DEALT ] = function( self, card, deck )
                    if card.id == "empty_bottle" then
                        card.min_damage = self.graft:GetDef().min_damage
                        card.max_damage = self.graft:GetDef().max_damage
                        card.target_mod = TARGET_MOD.RANDOM1
                        card.target_type = TARGET_TYPE.ENEMY
                        card.anim = "bottle_throw"
                        card.flags = CARD_FLAGS.RANGED | CARD_FLAGS.EXPEND | CARD_FLAGS.RESTRAINED
                    end
                end
            },
        },
    },

    collateralator_plus =
    {
        name = "Boosted Collateralator",
        desc = "{empty_bottle} cards deal <#UPGRADE>2</> damage to a random enemy target when played, but also gain {RESTRAINED}.",
        min_damage = 2,
        max_damage = 2,
    },

    reflector = 
    {
        name = "Reflector",
        desc = "Whenever you play an {empty_bottle}, gain 2 {DEFEND}.",
        flavour = "'A good addition to any bar brawler’s repertoire.'",

        rarity = CARD_RARITY.UNCOMMON,

        defend_amt = 2,
        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, card )
                    if card.id == "empty_bottle" then
                        self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                    end
                end
            },
        },
    },

    reflector_plus =
    {
        name = "Stone Reflector",
        desc = "Whenever you play an {empty_bottle}, gain <#UPGRADE>3</> {DEFEND}.",
        defend_amt = 3,
    },

    aleternator =
    {
        name = "Aleternator",
        desc = "Whenever you draw an {empty_bottle}, gain 2 {DEFEND}.",
        flavour = "'A severe increase to your metabolism, keeps you tough and lasts longer.'",

        rarity = CARD_RARITY.UNCOMMON,

        defend_amt = 2,
        alleviate_amt = 0,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
                    if card.owner == self.owner and card.id == "empty_bottle" then
                        self.owner:AddCondition("DEFEND", self.graft:GetDef().defend_amt, self)
                        if self.graft:GetDef().alleviate_amt > 0 then
                            self.owner:AlleviateHealth(self.graft:GetDef().alleviate_amt, self)
                        end
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    aleternator_plus =
    {
        name = "Soothing Aleternator",
        desc = "Whenever you draw an {empty_bottle}, gain 2 {DEFEND} and <#UPGRADE>{ALLEVIATE} 1.</>",

        alleviate_amt = 1,
    },

    calcifier =
    {
        name = "Calcifier",
        desc = "At the end of your turn, gain {DEFEND} equal to any {ADRENALINE} you have.",
        flavour = "'I don’t know if your physiology can take much more experimentation, but it’s your money.'",

        rarity = CARD_RARITY.UNCOMMON,

        bonus = 0,

        battle_condition = 
        {
            hidden = true,

            event_priorities =
            {
                -- before adrenaline
                [ BATTLE_EVENT.END_TURN  ] = 9,
            },
            event_handlers = 
            {
                [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                    if fighter == self.owner and self.owner:HasCondition("ADRENALINE") then
                        self.owner:AddCondition("DEFEND", self.owner:GetConditionStacks("ADRENALINE") + self.graft:GetDef().bonus, self)
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    calcifier_plus =
    {
        name = "Boosted Calcifier",
        desc = "At the end of your turn, gain {DEFEND} equal to any {ADRENALINE} you have <#UPGRADE>+ 2</>.",
        bonus = 2,
    },

    critical_pump =
    {
        name = "Critical Pump",
        desc = "While your {smith_regeneration} is at or below 3, all your attacks deal 2 bonus damage.",
        flavour = "'Your guts want to fix themselves all the time, when they aren’t worried about that you can get some more blood into those arms of yours.'",
        
        rarity = CARD_RARITY.UNCOMMON,

        limit = 3,
        bonus_damage = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    if not self.moxie then
                        self.moxie = self.owner:GetCondition("smith_regeneration")
                    end
                end,

                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and self.moxie and self.moxie.max_heal <= self.graft:GetDef().limit then
                        dmgt:AddDamage(self.graft:GetDef().bonus_damage, self.graft:GetDef().bonus_damage, self)
                    end
                end
            },
        },
    },

    critical_pump_plus =
    {
        name = "Boosted Critical Pump",
        desc = "While your {smith_regeneration} is at or below 3, all your attacks deal <#UPGRADE>3</> bonus damage.",
        bonus_damage = 3,
    },

    looper =
    {
        name = "Looper",
        desc = "At the end of your turn, {ALLEVIATE} 2.",
        flavour = "'Keeping kra’deshis in fights for longer!'",

        rarity = CARD_RARITY.UNCOMMON,

        alleviate_amt = 2,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    self.owner:AlleviateHealth(self.graft:GetDef().alleviate_amt, self)
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                end
            },
        },
    },

    looper_plus =
    {
        name = "Soothing Looper",
        desc = "At the end of your turn, {ALLEVIATE} <#UPGRADE>4</>.",
        alleviate_amt = 4,
    },

    -- enhanced_regeneration =
    -- {
    --     name = "Enhanced Regeneration",
    --     desc = "Whenever you take damage on your turn, increase the maximum healing of {smith_regeneration} by 1.",

    --     rarity = CARD_RARITY.UNCOMMON,

    --     max_heal = 1,

    --     battle_condition = 
    --     {
    --         hidden = true,

    --         event_handlers =
    --         {
    --             [ BATTLE_EVENT.DELTA_STAT ] = function( self, fighter, stat, delta, value, mitigated  )
    --                 if self.battle:GetCurrentTeam() == self.owner:GetTeam() and fighter == self.owner and stat == COMBAT_STAT.HEALTH and delta < 0 then
    --                     local healing = self.owner:GetCondition("smith_regeneration")
    --                     if healing then healing:IncreaseMax(self.graft:GetDef().max_heal, self) end
    --                 end
    --             end,
    --         },
    --     },
    -- },

    -- enhanced_regeneration_plus =
    -- {
    --     name = "Even More Enhanced Regeneration",
    --     desc = "Whenever you take damage on your turn, increase the maximum healing of {smith_regeneration} by <#UPGRADE>2</>.",
    --     max_heal = 2,
    -- },

    -- metabolizer =
    -- {
    --     name = "Metabolizer",
    --     desc = "At the start of battle, set the minimum healing of your {smith_regeneration} to 2",

    --     rarity = CARD_RARITY.UNCOMMON,

    --     min_heal = 2,
    --     max_heal = 3,
    --     base_max = 3,

    --     battle_condition = 
    --     {
    --         hidden = true,

    --         event_handlers = 
    --         {
    --             [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
    --                 local healing = self.owner:GetCondition("smith_regeneration")
    --                 if healing then
    --                     healing.min_heal = self.graft:GetDef().min_heal
    --                     healing.max_heal = self.graft:GetDef().max_heal
    --                     healing.base_max = self.graft:GetDef().base_max
    --                     self.owner:RemoveCondition(self.id)
    --                 end
    --             end
    --         },
    --     },
    -- },

    -- metabolizer_plus =
    -- {
    --     name = "Boosted Metabolizer",
    --     desc = "At the start of battle, set the <#UPGRADE>healing range of your {smith_regeneration} to 3-6.</>"
    -- },

    shatterguard =
    {
        name = "Shatterguard",
        desc = "At the end of your turn, gain {1} {DEFEND} for every {empty_bottle} in your hand.",
        flavour = "'Transforming your evenings out into solid defenses.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:GetDef().defend_amt)
        end,

        rarity = CARD_RARITY.UNCOMMON,

        defend_amt = 1,

        battle_condition = 
        {
            hidden = true,

            event_handlers =
            {
                [ BATTLE_EVENT.END_PLAYER_TURN ] = function( self, battle )
                    local count = 0
                    for i,card in battle:GetHandDeck():Cards() do
                        if card.id == "empty_bottle" then
                            count = count + 1
                        end
                    end
                    if count > 0 then
                        self.owner:AddCondition("DEFEND", count * self.graft:GetDef().defend_amt, self)
                    end
                end
            },
        }
    },

    shatterguard_plus = 
    {
        name = "Stone Shatterguard",
        desc = "At the end of your turn, gain <#UPGRADE>{1}</> {DEFEND} for every {empty_bottle} in your hand.",
        defend_amt = 2,
    },

    lucky_strike =
    {
        name = "Lucky Strike",
        desc = "Lower the {THRESHOLD} on all cards by 1.",

        rarity = CARD_RARITY.UNCOMMON,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_THRESHOLD ] = function( self, threshold, card )
                    if card.owner == self.owner then
                        threshold:AddValue(-self.stacks, self)
                    end
                end
            },
        },
    },

    bandit =
    {
        name = "Bandit",
        desc = "Whenever you trigger a card's {THRESHOLD}, gain {ADRENALINE {1}}.",
        desc_fn = function ( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().adrenaline_amt )
        end,
        flavour = "'Feel that rush? Might as well use it.'",

        rarity = CARD_RARITY.UNCOMMON,
        adrenaline_amt = 2,

        battle_condition =
        {
            hidden = true,
            event_handlers=
            {
                [ BATTLE_EVENT.THRESHOLD_TRIGGERED ] = function( self, battle, card, target )
                    if card.owner == self.owner then
                        self.owner:AddCondition("ADRENALINE", self.graft:GetDef().adrenaline_amt, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft)
                    end
                end
            },
        },
    },

    bandit_plus =
    {
        name = "Boosted Bandit",
        desc = "Whenever you trigger a card's {THRESHOLD}, gain <#UPGRADE>{ADRENALINE {1}}</>.",
        adrenaline_amt = 3,
    },

    lucky_strike =
    {
        name = "Lucky Strike",
        desc = "Lower the {THRESHOLD} of the first card you play each turn by 1.",
        flavour = "'Keeps you nice and alert for when the time comes.'",

        rarity = CARD_RARITY.UNCOMMON,

        battle_condition =
        {
            hidden = true,
            event_handlers=
            {
                [ BATTLE_EVENT.CALC_THRESHOLD ] = function( self, threshold, card )
                    if card.owner == self.owner and self.active then
                        threshold:AddValue(-1, self)
                    end
                end,

                [ BATTLE_EVENT.END_RESOLVE ] = function( self, battle, card )
                    if card.owner == self.owner and card.threshold then
                        self.active = false
                    end
                end,

                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        self.active = true
                    end
                end
            },
        },
    },

    lucky_strike_plus =
    {
        name = "Luckier Strike",
        desc = "Lower the {THRESHOLD} of <#UPGRADE>all cards</> by 1.",
        battle_condition =
        {
            hidden = true,
            event_handlers=
            {
                [ BATTLE_EVENT.CALC_THRESHOLD ] = function( self, threshold, card )
                    if card.owner == self.owner then
                        threshold:AddValue(-1, self)
                    end
                end,
            },
        },
    },

    -- RARES
    speed_shot = 
    {
        name = "Speed Shot",
        desc = "At the start of every turn, insert {lean_green} into your hand.",
        flavour = "'Always have your favorite spirits at the ready with this handy device!'",

        rarity = CARD_RARITY.RARE,

        card_id = "lean_green",

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if self.owner == fighter then
                        local card = Battle.Card(self.graft:GetDef().card_id, self.owner)
                        card.auto_deal = true
                        self.battle:DealCard( card, self.battle:GetHandDeck())
                        self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end,
            },
        },
    },

    speed_shot_plus =
    {
        name = "Boosted Speed Shot",
        desc = "At the start of every turn, insert <#UPGRADE>{lean_green_plus}</> into your hand.",
        card_id = "lean_green_plus",
    },

    auto_sorter =
    {
        name = "Auto Sorter",
        desc = "At the start of your turn, automatically discard up to 3 {empty_bottle} cards and draw that many cards.",
        flavour = "'This is perfect! My life is such a mess right now.'",

        rarity = CARD_RARITY.RARE,

        limit = 3,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    battle:Delay(0.4)
                    battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                    local to_discard = {}
                    for i,v in ipairs(battle:GetHandDeck().cards)do
                        if v.id == "empty_bottle" then
                            table.insert(to_discard, v)
                            if self.graft:GetDef().limit and #to_discard >= self.graft:GetDef().limit then
                                break
                            end
                        end
                    end
                    for i,v in ipairs(to_discard) do
                        battle:DiscardCard(v)
                    end
                    battle:DrawCards(#to_discard)
                end,
            },
        },
    },

    auto_sorter_plus = 
    {
        name = "Visionary Auto Sorter",
        desc = "At the start of your turn, automatically discard <#UPGRADE>all {empty_bottle} cards</> and draw that many cards.",
        limit = 999,
    },

    second_opinion =
    {
        name = "Second Opinion",
        desc = "Increase the minimum damage by {1} on your attacks against targets that have {TRAUMA} or {TRAUMATIZED}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().bonus_damage )
        end,

        flavour = "'Exactly what I needed!'",

        rarity = CARD_RARITY.RARE,

        bonus_damage = 2,

        battle_condition = 
        {
            hidden = true,
            event_priorities =
            {
                [ BATTLE_EVENT.CALC_DAMAGE  ] = 11,
            },
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and target and (target:HasCondition("TRAUMA") or target:HasCondition("TRAUMATIZED")) then
                        dmgt:ModifyDamage( dmgt.min_damage + self.graft:GetDef().bonus_damage, dmgt.max_damage, self )
                    end
                end
            },
        },
    },

    second_opinion_plus =
    {
        name = "Boosted Second Opinion",
        desc = "Increase the minimum damage by <#UPGRADE>{1}</> on your attacks against targets that have {TRAUMA} or {TRAUMATIZED}.",
        bonus_damage = 3,
    },

    battlestein = 
    {
        name = "Battle Stein",
        desc = "Whenever you {DRINK} more than 5 times in a turn, gain {ADRENALINE {1}}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().adrenaline_amt )
        end,

        flavour = "'Fill ‘er up.'",

        rarity = CARD_RARITY.RARE,

        display_number = function( self )
            return self.userdata.counter or 0
        end,

        adrenaline_amt = 3,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.DRINK ] = function( self, battle, count, cards )
                    self.graft:IncrementCounter()
                    if self.graft:GetCounter() >= 5 and not self.used then
                        self.owner:AddCondition("ADRENALINE", self.graft:GetDef().adrenaline_amt, self)
                        battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        self.used = true
                    end
                    battle:BroadcastEvent( BATTLE_EVENT.REFRESH_GRAFTS )
                end,

                [ BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self, battle )
                    self.graft:ResetCounter()
                    self.used = false
                    battle:BroadcastEvent( BATTLE_EVENT.REFRESH_GRAFTS )
                end
            },
        },
    },

    battlestein_plus =
    {
        name = "Boosted Battle Stein",
        desc = "Whenever you {DRINK} more than 5 times in a turn, gain <#UPGRADE>{ADRENALINE {1}}</>.",
        adrenaline_amt = 5,
    },

    bottle_shot =
    {
        name = "Bottle Shot",
        desc = "All attacks deal {1} bonus damage if you have spent a bottle with another card this turn.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().damage_bonus)
        end,
        flavour = "'The crosshairs are just for looks.'",
        rarity = CARD_RARITY.RARE,

        damage_bonus = 2,

        battle_condition = 
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
                    if card.owner == self.owner and self.active then
                        dmgt:AddDamage( self.graft:GetDef().damage_bonus, self.graft:GetDef().damage_bonus, self )
                    end
                end,

                [ BATTLE_EVENT.BOTTLES_SPENT ] = function( self, count, source )
                    if is_instance(source, Battle.Card) and source.owner == self.owner then
                        self.active = true
                    end
                end,

                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        self.active = false
                    end
                end
            },
        },
    },

    bottle_shot_plus =
    {
        name = "Boosted Bottle Shot",
        desc = "All attacks deal <#UPGRADE>{1} bonus damage</> if you have spent a bottle with another card this turn.",
        damage_bonus = 3,
    },

    klipper =
    {
        name = "Klipper",
        desc = "Gain 1 action at the start of your turn, lose 15 maximum health when installed.",
        flavour = "'Kra'deshi klippers are nasty weapons used by seasoned fighters to speed up the longer combats.'",

        rarity = CARD_RARITY.BOSS,

        card_draw = 0,

        OnAdded = function( self, agent )
            TheGame:GetGameState():GetPlayerAgent().health:AddStatModifier( "klipper", -15 )
        end,

        battle_condition =
        {
            hidden = true,
            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if fighter == self.owner then
                        if self.owner.player_controlled then
                            self.battle:ModifyActionCount( self.stacks )
                            local draw = self.graft:GetDef().card_draw
                            if draw > 0 then self.battle:DrawCards(draw) end
                            self.battle:BroadcastEvent( BATTLE_EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,
            },
        },
    },

    klipper_plus =
    {
        name = "Visionary Klipper",
        desc = "Gain 1 action<#UPGRADE> and draw a card</> at the start of your turn, lose 15 maximum health when installed.",
        card_draw = 1,
        OnAdded = function( self, agent )
        end,
    },
}

---------------------------------------------------------------------------------------------

for i, id, graft in sorted_pairs( BATTLE_GRAFTS ) do
    graft.card_defs = battle_defs
    graft.series = "SMITH"
    graft.type = GRAFT_TYPE.COMBAT
    Content.AddGraft( id, graft )
end

Content.AddBattleCard("lean_green", 
{
    name = "Lean and Green",
    anim = "drink",
    desc = "{DRINK} 1 and gain 1 {ADRENALINE}.",
    flavour = "'Some pep in your step.'",

    cost = 0,
    flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,

    rarity = CARD_RARITY.UNIQUE,
    max_xp = 0,
    series = "SMITH",
    target_type = TARGET_TYPE.SELF,

    OnPostResolve = function( self, battle, attack )
        local card = Battle.Card( "empty_bottle", self.owner )
        card.show_dealt = false
        battle:DealCards( {card}, battle:GetDiscardDeck() )
        battle:BroadcastEvent( BATTLE_EVENT.DRINK, battle, 1, {card} )
        self.owner:AddCondition("ADRENALINE", 1, self)
    end,
})

Content.AddBattleCard("lean_green_plus", 
{
    name = "Boosted Lean and Green",
    anim = "drink",
    desc = "{DRINK} 1 and gain <#UPGRADE>2</> {ADRENALINE}.",

    cost = 0,
    flags = CARD_FLAGS.EXPEND | CARD_FLAGS.SKILL,

    rarity = CARD_RARITY.UNIQUE,
    series = "SMITH",
    target_type = TARGET_TYPE.SELF,

    OnPostResolve = function( self, battle, attack )
        local card = Battle.Card( "empty_bottle", self.owner )
        card.show_dealt = false
        battle:DealCards( {card}, battle:GetDiscardDeck() )
        battle:BroadcastEvent( BATTLE_EVENT.DRINK, battle, 1, {card} )
        self.owner:AddCondition("ADRENALINE", 2, self)
    end,
})