local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT
local CARD_FLAGS = negotiation_defs.CARD_FLAGS

local NEGOTIATION_GRAFTS =
{
    -----------------------------------------------------------------
    -- COMMONS
    clan_ring =
    {
        name = "Clan Ring",
        desc = "Whenever you gain {RENOWN}, apply 1 {COMPOSURE} to a random friendly argument.",
        flavour = "'Wait a minute, where did you get that?'",

        rarity = CARD_RARITY.COMMON,

        composure_amt = 1,

        negotiation_modifier =
        {
            hidden = true,
            event_handlers = 
            {
                [ EVENT.MODIFIER_ADDED ] = function ( self, modifier, source )
                    if modifier.negotiator == self.negotiator and modifier.id == "RENOWN" then
                        local targets = self.engine:CollectAlliedTargets(self.negotiator)
                        if #targets > 0 then
                            local target = targets[math.random(#targets)]
                            target:DeltaComposure(self.graft:GetDef().composure_amt, self)
                            self.engine:BroadcastEvent( EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,

                [ EVENT.MODIFIER_CHANGED ] = function( self, modifier, delta, clone, source )
                    if modifier.negotiator == self.negotiator and modifier.id == "RENOWN" and delta and delta > 0 then
                        local targets = self.engine:CollectAlliedTargets(self.negotiator)
                        if #targets > 0 then
                            local target = targets[math.random(#targets)]
                            target:DeltaComposure(self.graft:GetDef().composure_amt, self)
                            self.engine:BroadcastEvent( EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end,
            },
        },
    },

    clan_ring_plus =
    {
        name = "Stone Clan Ring",
        desc = "Whenever you gain {RENOWN}, apply <#UPGRADE>2</> {COMPOSURE} to a random friendly argument.",
        composure_amt = 2,
    },

    gang_pendant =
    {
        name = "Gang Pendant",
        desc = "At the start of each negotiation, gain 3 {RENOWN}",
        flavour = "'Of course I know what that means, I just wanted to see if you did.'",

        rarity = CARD_RARITY.COMMON,
        renown_amt = 3,

        negotiation_modifier = 
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        self.negotiator:AddModifier("RENOWN", self.graft:GetDef().renown_amt, self)
                        minigame:BroadcastEvent( EVENT.GRAFT_TRIGGERED, self.graft )
                        self.negotiator:RemoveModifier(self, self.stacks, self)
                    end
                end
            },
        },
    },

    gang_pendant_plus =
    {
        name = "Boosted Gang Pendant",
        desc = "At the start of each negotiation, gain <#UPGRADE>5</> {RENOWN}",
        renown_amt = 5,
    },

    lumin_relic = 
    {
        name = "Lumin Relic",
        desc = "If you have 5 {RENOWN} or more at the start of your turn, draw a card.",
        flavour = "'Wait, did that just move?'",

        rarity = CARD_RARITY.COMMON,
        card_draw = 1,

        negotiation_modifier = 
        {
            hidden = true,
            event_handlers = 
            {
                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    if self.negotiator:GetModifierStacks("RENOWN") >= 5 then
                        minigame:DrawCards(self.graft:GetDef().card_draw)
                        minigame:BroadcastEvent( EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    lumin_relic_plus =
    {
        name = "Visionary Lumin Relic",
        desc = "If you have 5 {RENOWN} or more at the start of your turn, draw <#UPGRADE>2 cards</>.",
        card_draw = 2,
    },

    wreckrider_badge = 
    {
        name = "Wreckrider Badge",
        desc = "Whenever a friendly argument is destroyed, deal 2 damage to a random opponent argument.",
        flavour = "I earned mine from almost jumping the Riptide Ravine. It’s more about guts I guess.'",

        rarity = CARD_RARITY.COMMON,
        damage_amt = 2,

        negotiation_modifier =
        {
            hidden = true,
            target_enemy = TARGET_ANY_RESOLVE,
            event_handlers =
            {
                [ EVENT.MODIFIER_REMOVED ] = function ( self, modifier, source )
                    if modifier.negotiator == self.negotiator and source and modifier.stacks > 0 then
                        self.min_persuasion, self.max_persuasion = self.graft:GetDef().damage_amt, self.graft:GetDef().damage_amt
                        self.engine:ApplyPersuasion(self)
                    end
                end
            },
        },
    },

    wreckrider_badge_plus =
    {
        name = "Boosted Wreckrider Badge",
        desc = "Whenever a friendly argument is destroyed, deal <#UPGRADE>3</> damage to a random opponent argument.",
        damage_amt = 3,
    },
    
    -----------------------------------------------------------------
    -- UNCOMMONS
    cerebral_tackle = 
    {
        name = "Cerebral Tackle",
        desc = "Whenever any argument is removed or destroyed, apply 2 {COMPOSURE} to a random friendly argument.",
        flavour = "'Let’s keep this conversation interesting.'",

        rarity = CARD_RARITY.UNCOMMON,
        composure_amt = 2,

        negotiation_modifier = 
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.MODIFIER_REMOVED ] = function ( self, modifier, source )
                    if modifier.modifier_type ~= MODIFIER_TYPE.CORE and not modifier.hidden then
                        local targets = self.engine:CollectAlliedTargets(self.negotiator)
                        if #targets > 0 then
                            local target = targets[math.random(#targets)]
                            target:DeltaComposure(self.graft:GetDef().composure_amt, self)
                            self.engine:BroadcastEvent( EVENT.GRAFT_TRIGGERED, self.graft )
                        end
                    end
                end
            },
        },
    },

    cerebral_tackle_plus =
    {
        name = "Stone Cerebral Tackle",
        desc = "Whenever any argument is removed or destroyed, apply <#UPGRADE>3</> {COMPOSURE} to a random friendly argument.",
        composure_amt = 3,
    },

    rebounder = 
    {
        name = "Rebounder",
        desc = "The first time your core argument takes damage, ignore the damage and deal that much damage to a random opponent argument.",
        flavour = "'Well, I’m rubber and you’re oshnu glue.'",
        
        rarity = CARD_RARITY.UNCOMMON,
        damage_bonus = 0,

        negotiation_modifier = 
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                    if target == self.negotiator:FindCoreArgument() and damage > defended then
                        local options = self.engine:CollectAlliedTargets(self.engine:GetOtherNegotiator(self.negotiator))
                        if options and #options > 0 then
                            options[math.random(#options)]:AttackResolve(damage + self.graft:GetDef().damage_bonus, self)
                        end
                        target.composure = target.composure + damage
                        self.negotiator:RemoveModifier(self, self.stacks, self)
                    end
                end,
            },
        },
    },

    rebounder_plus =
    {
        name = "Boosted Rebounder",
        desc = "The first time your core argument takes damage, ignore the damage and deal that much damage <#UPGRADE>+ 3</> to a random opponent argument.",
        damage_bonus = 3,
    },

    cranial_coolant =
    {
        name = "Cranial Coolant",
        desc = "Diplomacy cards deal 2 bonus damage while you have 5 {RENOWN} or more.",
        flavour = "'Always good to keep a cool head during these discussions.'",

        rarity = CARD_RARITY.UNCOMMON,
        bonus_damage = 2,

        negotiation_modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.CALC_PERSUASION ] = function ( self, source, persuasion, minigame )
                    if source.negotiator == self.negotiator and self.negotiator:GetModifierStacks("RENOWN") >= 5 and is_instance( source, Negotiation.Card) and source:IsFlagged(CARD_FLAGS.DIPLOMACY) then
                        persuasion:AddPersuasion(self.graft:GetDef().bonus_damage, self.graft:GetDef().bonus_damage, self)
                    end
                end
            },
        },
    },

    cranial_coolant_plus =
    {
        name = "Boosted Cranial Coolant",
        desc = "Diplomacy cards deal <#UPGRADE>3</> bonus damage while you have 5 {RENOWN} or more.",
        bonus_damage = 3,
    },

    resolute_signet = 
    {
        name = "Resolute Signet",
        desc = "Whenever you deal damage to your own arguments, apply 2 {COMPOSURE} to your core argument.",
        flavour = "A simple signet ring that was used to seal important documents. Now it is a symbol of lineage and status.'",

        rarity = CARD_RARITY.UNCOMMON,
        composure_amt = 2,

        negotiation_modifier = 
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                    if target.negotiator == self.negotiator and source.negotiator == self.negotiator then
                        self.negotiator:DeltaComposure(self.graft:GetDef().composure_amt, self)
                    end
                end,
            },
        },
    },

    resolute_signet_plus =
    {
        name = "Stone Resolute Signet",
        desc = "Whenever you deal damage to your own arguments, apply <#UPGRADE>3</> {COMPOSURE} to your core argument.",
        composure_amt = 3,
    },

    ego_pump =
    {
        name = "Ego Pump",
        desc = "Whenever your {RENOWN} is removed or destroyed, deal {1} damage to a random opponent argument.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetDef().damage_amt )
        end,
        flavour = "'The ringing in your ears? Yeah that’s permanent.'",

        rarity = CARD_RARITY.UNCOMMON,
        damage_amt = 3,

        negotiation_modifier = 
        {
            hidden = true,
            target_enemy = TARGET_ANY_RESOLVE,
            event_handlers =
            {
                [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                    if modifier.negotiator == self.negotiator and modifier.id == "RENOWN" then
                        self.engine:BroadcastEvent( EVENT.GRAFT_TRIGGERED, self.graft )
                        self.min_persuasion, self.max_persuasion = self.graft:GetDef().damage_amt, self.graft:GetDef().damage_amt
                        self.engine:ApplyPersuasion(self)
                    end
                end,
            },
        },
    },

    ego_pump_plus =
    {
        name = "Boosted Ego Pump",
        desc = "Whenever your {RENOWN} is removed or destroyed, deal <#UPGRADE>{1} damage</> to a random opponent argument.",
        damage_amt = 4,
    },

    scatterbrain =
    {
        name = "Scatterbrain",
        desc = "At the end of your turn, deal damage to a random opponent argument equal to the total number of friendly arguments.",
        flavour = "'It actually helps you keep track of all those topics you started.'",

        rarity = CARD_RARITY.UNCOMMON,
        bonus = 0,

        negotiation_modifier =
        {
            hidden = true,
            target_enemy = TARGET_ANY_RESOLVE,
            event_handlers =
            {
                [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        local damage = self.graft:GetDef().bonus or 0
                        for i, modifier in self.negotiator:Modifiers() do
                            if modifier:GetResolve() ~= nil then
                                damage = damage + 1
                            end
                        end
                        minigame:BroadcastEvent( EVENT.GRAFT_TRIGGERED, self.graft )
                        self.min_persuasion, self.max_persuasion = damage, damage
                        self.engine:ApplyPersuasion(self)
                    end
                end
            },
        },
    },

    scatterbrain_plus =
    {
        name = "Boosted Scatterbrain",
        desc = "At the end of your turn, deal damage to a random opponent argument equal to the total number of friendly arguments<#UPGRADE> +2</>.",
        bonus = 2,
    },
    
    -----------------------------------------------------------------
    -- RARES
    clan_chain =
    {
        name = "Clan Chain",
        desc = "At the start of each turn, gain 1 {RENOWN}",
        flavour = "'I pity the fool who doesn’t own a necklace as fine as this.'",

        rarity = CARD_RARITY.RARE,

        renown_amt = 1,

        negotiation_modifier = 
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        self.negotiator:AddModifier("RENOWN", self.graft:GetDef().renown_amt, self)
                        minigame:BroadcastEvent( EVENT.GRAFT_TRIGGERED, self.graft )
                    end
                end
            },
        },
    },

    clan_chain_plus =
    {
        name = "Boosted Clan Chain",
        desc = "At the start of each turn, gain <#UPGRADE>2</> {RENOWN}",
        renown_amt = 2,
    },

    neural_braid =
    {
        name = "Defective Neural Braid",
        desc = "At the start of each negotiation, lose resolve equal to the cost of your starting hand.",
        flavour = "'I think it just needs to get to know you.'",

        rarity = CARD_RARITY.UNIQUE,
        multiplier = -1,
        max_xp = 5,

        negotiation_modifier =
        {
            modifier_type = MODIFIER_TYPE.HIDDEN,

            event_handlers = 
            {
                [ EVENT.HAND_DRAWN ] = function( self, minigame )
                    local cost = 0
                    for i, card in minigame:GetHandDeck():Cards() do
                        cost = cost + minigame:CalculateActionCost( card )
                    end
                    local amt = cost * self.stacks * self.graft:GetDef().multiplier
                    if amt > 0 then
                        self.negotiator:RestoreResolve( amt, self )
                    else
                        self.negotiator:LoseResolve( -amt, self )
                    end
                    self.negotiator:RemoveModifier( self )
                end
            }
        }
    },
    neural_braid_plus =
    {
        name = "Neural Braid",
        desc = "At the start of each negotiation, <#UPGRADE>restore</> resolve equal to the cost of your starting hand.",
        multiplier = 1,
    },

    finely_crafted_amulet =
    {
        name = "Finely Crafted Amulet",
        desc = "Gain 1 action at the start of your turn. At the start of your turn remove 1 {RENOWN}.",
        flavour = "'All custom carved gemstones. Nope, this one is not for sale'",

        card_draw = 0,

        rarity = CARD_RARITY.BOSS,

        negotiation_modifier =
        {
            hidden = true,

            event_handlers =
            {

                [ EVENT.CALC_ACTIONS_PER_TURN ] = function( self, acc )
                    self.engine:BroadcastEvent( EVENT.GRAFT_TRIGGERED, self.graft )
                    acc:AddValue( 1 )
                end,

                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    local draw = self.graft:GetDef().card_draw
                    if draw > 0 then minigame:DrawCards(draw) end
                    self.negotiator:RemoveModifier("RENOWN", 1, self)
                end,
            },
            
        },
    },

    finely_crafted_amulet_plus =
    {
        name = "Visionary Finely Crafted Amulet",
        desc = "Gain 1 action<#UPGRADE> and draw a card</> at the start of your turn. At the start of your turn remove 1 {RENOWN}.",
        card_draw = 1,
    },
}

---------------------------------------------------------------------------------------------

for i, id, graft in sorted_pairs( NEGOTIATION_GRAFTS ) do
    graft.card_defs = negotiation_defs
    graft.series = "SMITH"
    
    if graft.negotiation_modifier and graft.negotiation_modifier.modifier_type == nil then
        graft.negotiation_modifier.modifier_type = MODIFIER_TYPE.HIDDEN
    end
    graft.type = GRAFT_TYPE.NEGOTIATION
    Content.AddGraft( id, graft )
end
