local PATRON_DEFS = 
{
    SPARK_BARON_PROFESSIONAL = 1,
    SPARK_BARON_GOON = 1,
    SPARK_BARON_TASKMASTER = 1,

    RISE_REBEL = 1,
    RISE_PAMPHLETEER = 1,
    RISE_RADICAL = 1,

    JAKES_RUNNER = .25,
    JAKES_SMUGGLER = .25,
    JAKES_LIFTER = .1,

    PRIEST = .25,
    LUMINARI = .25,
    LUMINITIATE = .1,

    LABORER = 1,
    HEAVY_LABORER = 1,
    POOR_MERCHANT = .5,
    WEALTHY_MERCHANT = .5,
    FOREMAN = 1,

    BOGGER_CLOBBER = .125,
    BOGGER_CULTIVATOR = .125,

}

local faction_weights = 
{
    [RELATIONSHIP.HATED] = 0,
    [RELATIONSHIP.DISLIKED] = .5,
    [RELATIONSHIP.NEUTRAL] = 1,
    [RELATIONSHIP.LIKED] = 1.25,
    [RELATIONSHIP.LOVED] = 1.5,
}


local function generate_patrons(location)

    local candidates = {}


    for def_id, base_w in pairs(PATRON_DEFS) do
        local w = base_w

        local def = Content.GetCharacterDef(def_id)
        w = w * (faction_weights[TheGame:GetGameState():GetFaction(location:GetFactionID()):GetFactionRelationship( def.faction_id )] or 1)
        if w > 0 then
            candidates[def_id] = w
        end
        
    end
    if next(candidates) then
        
        local def = weightedpick(candidates)
        TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
    end
end


local ws_work = CreateProductionWorkplace( 2, DAY_PHASE.DAY, "SPARK_BARON_TASKMASTER", "HEAVY_LABORER", "Digmaster", "Bogger")
ws_work.guard = CreateLabourJob( DAY_PHASE.DAY, "Muscle", CHARACTER_ROLES.GUARD, "SPARK_BARON_GOON" )
ws_work.light_worker = CreateLabourJob( DAY_PHASE.DAY, "Light Bogger", CHARACTER_ROLES.LABOUR, "LABORER" )

Content.AddLocationContent{
    id = "GB_WORKSITE_1",
    name = "Worksite A",
    plax = "EXT_SB_Bog_Worksite_1",
    indoors = false,
    tags = {"industry"},
    work = ws_work,
    show_agents = true,
}

Content.AddLocationContent{
    id = "WORKERS_QUARTERS",
    name = "{1#agent}'s Bunk",
    plax = "INT_WorkCamp_Res",
    
    name_fn = function(location, fmt_str) return loc.format(fmt_str, location:GetPrimaryRoomOccupant() )end,
    show_agents = true,
    entry_scenario = "WORKERS_QUARTERS_ENCOUNTER",
    --[[on_violence = function(location, victim) 
        QuestUtil.SpawnQuest("VIOLENCE_FOLLOWUP", { parameters = { response_team = "SPARK_BARON_PATROL", location = location, victim = victim } })
    end,-]]
    map_tags = {"quarters"},
    flags = LOCFLAGS.HOME,
}

Content.AddLocationContent{
    id = "GB_WORKSITE_2",
    name = "Worksite B",
    plax = "EXT_SB_Bog_Worksite_1",
    indoors = false,
    tags = {"industry"},
    work = ws_work,
    show_agents = true,
}

Content.AddLocationContent{
    id = "GB_METTLE_SHOP",
    name = "Mettle Shop",
    desc = "The place the mysterious stranger told you to go to.",
    plax = "ACT2_META_SHOP",
    indoors = true,
}

Content.AddLocationContent{
    id = "GB_AUTOMECH_FACTORY",
    name = "Automech Factory",
    faction = "SPARK_BARONS",
    plax = "INT_SparkBaron_Robot_Factory",
    show_agents = true,
    indoors = true,
    work = 
    {
        admin = CreateMerchantJob( PHASE_MASK_ALL, "Automech Administrator", CHARACTER_ROLES.PROPRIETOR, "SPARK_BARON_TASKMASTER"),
        guard1 = CreateLabourJob( PHASE_MASK_ALL, "Guard", CHARACTER_ROLES.GUARD, "SPARK_BARON_AUTOMECH" ),
        wrangler = CreateLabourJob( PHASE_MASK_ALL, "Automech Wrangler", CHARACTER_ROLES.LABOUR, "LABORER" ),
    },

}


Content.AddLocationContent{
    id = "GB_BLACK_MARKET",
    name = "Black Market",
    plax = "EXT_WorkCamp_BlackMarket",
    indoors = false,
    faction = "JAKES",
    --tags = {"industry"},
    work = {
        guard = CreateLabourJob( PHASE_MASK_ALL, "Guard", CHARACTER_ROLES.GUARD, "JAKES_SMUGGLER"),  
        vendor = CreateMerchantJob( PHASE_MASK_ALL, "Vendor", CHARACTER_ROLES.PROPRIETOR, "JAKES_RUNNER", "GROUT_BOG_BLACK_MARKET"),  
    }, 
    show_agents = true,
}


Content.AddLocationContent{
    id = "GB_DEAD_DROP_SPOT",
    name = "Dead Drop Spot",
    plax = "EXT_DROPSITE",
    indoors = false,
    show_agents = true,
}

Content.AddLocationContent{
    id = "GB_CAFFY",
    name = "The Caffy",
    plax = "INT_WorkCamp_Caffy",
    indoors = true,
    faction = "SPARK_BARONS",
    show_agents = true,
    patron_data = {
        
        num_patrons = 5,
        
        filter = function(location, agent)
            return agent:GetContentID() == "HEAVY_LABORER" or agent:GetContentID() == "LABORER" or agent:GetContentID() == "FOREMAN"
        end,
        
        patron_generator = function(location)
            local def = math.random() > .5 and "HEAVY_LABORER" or "LABORER"
            TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
        end,
    },

    work = 
    {
        cook = CreateMerchantJob( PHASE_MASK_ALL, "Cook", CHARACTER_ROLES.PROPRIETOR, "SPARK_BARON_TASKMASTER"),
        guard = CreateLabourJob( PHASE_MASK_ALL, "Muscle", CHARACTER_ROLES.GUARD, "SPARK_BARON_AUTOMECH" ),
    },
}

Content.AddLocationContent{
    id = "GB_COMPANY_STORE",
    name = "The Company Store",
    desc = "Sells essential items at outrageous prices to a captive audience of bogworkers.",
    plax = "INT_WorkCamp_CompanyStore",
    indoors = true,
    faction = "SPARK_BARONS",
    show_agents = true,
    patron_data = {
        num_patrons = 3,
        filter = function(location, agent)
            return agent:GetContentID() == "HEAVY_LABORER" or agent:GetContentID() == "LABORER" or agent:GetContentID() == "FOREMAN"
        end,
        patron_generator = function(location)
            local def = math.random() > .5 and "HEAVY_LABORER" or "LABORER"
            TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
        end,
    },

    work = 
    {
        storemaster = CreateMerchantJob( PHASE_MASK_ALL, "Storemaster", CHARACTER_ROLES.PROPRIETOR, "WEALTHY_MERCHANT"),
        guard = CreateLabourJob( PHASE_MASK_ALL, "Muscle", CHARACTER_ROLES.GUARD, "SPARK_BARON_GOON" ),
    },
}


Content.AddLocationContent{
    id = "RISE_TINKER_SHOP",
    name = "Rise Tinker Shop",
    plax = "INT_Rise_Tinkershop_1",
    indoors = true,
    faction = "RISE",
    show_agents = true,
    patron_data = {
        num_patrons = 1,
        filter = function(location, agent)
            return agent:GetFactionID() == "RISE"
        end,
        patron_generator = function(location)
            local def = table.arraypick{"RISE_RADICAL", "RISE_PAMPHLETEER", "RISE_REBEL"}
            TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
        end,
    },

    work = 
    {
        admin = CreateMerchantJob( PHASE_MASK_ALL, "Tinkerer", CHARACTER_ROLES.PROPRIETOR, "RISE_PAMPHLETEER"),
        guard1 = CreateLabourJob( PHASE_MASK_ALL, "Guard", CHARACTER_ROLES.GUARD, "RISE_AUTOMECH" ),
    },
}


Content.AddLocationContent{
    id = "GB_REC_CENTER",
    name = "Spark Baron Rec Center",
    plax = "INT_SB_RecClub",
    indoors = true,
    faction = "SPARK_BARONS",
    show_agents = true,
    patron_data = {
        num_patrons = 4,
        filter = function(location, agent)
            return agent:GetFactionID() == "SPARK_BARONS"
        end,
        patron_generator = function(location)
            local def = table.arraypick{"SPARK_BARON_PROFESSIONAL", "SPARK_BARON_TASKMASTER", "SPARK_BARON_GOON"}
            TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
        end,
    },

    work = 
    {
        guard = CreateLabourJob( PHASE_MASK_ALL, "Guard", CHARACTER_ROLES.GUARD, "SPARK_BARON_AUTOMECH" ),
    },
}


Content.AddLocationContent{
    id = "GB_GATE",
    name = "Grout Bog Access Gate",
    plax = "EXT_TerritoryGate_Entrance",
    show_agents = true,
    indoors = false,
    work = 
    {
        gatekeeper = CreateMerchantJob( PHASE_MASK_ALL, "Officer of the Gate", CHARACTER_ROLES.PROPRIETOR, "SPARK_BARON_TASKMASTER"),
        guard = CreateLabourJob( PHASE_MASK_ALL, "Muscle", CHARACTER_ROLES.GUARD, "SPARK_BARON_AUTOMECH" ),
    },
}

Content.AddLocationContent{
    id = "GB_RISE_HQ",
    plax = "INT_Rise_HQ",
    name = "Rise HQ",
    faction_id = "RISE",
    show_agents = true,
    indoors = true,
    tags = {"HQ"},
    work = 
    {
        garrison1 = CreateLabourJob( PHASE_MASK_ALL, "Garrison", CHARACTER_ROLES.GUARD, "RISE_REBEL" ),


    },
}

Content.AddLocationContent{
    id = "GB_LABOUR_OFFICE",
    name = "Labor Office",
    plax = "INT_WORK_CENTER", --replace with something custom
    faction_id = "FEUD_CITIZEN",
    show_agents = true,
    indoors = true,
    tags = {"HQ"},
    patron_data = {
        num_patrons = 1,
        filter = function(location, agent)
            if agent:HasActiveQuestMembership() then
                return false
            end

            return agent:GetContentID() == "HEAVY_LABORER" or agent:GetContentID() == "LABORER" or agent:GetContentID() == "FOREMAN"
        end,
        patron_generator = function(location)
            local def = math.random() > .5 and "HEAVY_LABORER" or "LABORER"
            TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
        end,
    },

    work = 
    {
        garrison1 = CreateLabourJob( PHASE_MASK_ALL, "guard", CHARACTER_ROLES.GUARD, "JAKES_SMUGGLER" ),
    },
}


local baron_patron_list = {"ADMIRALTY_PATROL_LEADER", "WEALTHY_MERCHANT", "FOREMAN", "SPARK_BARON_TASKMASTER", "SPARK_BARON_GOON"}

Content.AddLocationContent{
    id = "GB_BARON_HQ",
    plax = "INT_SB_HQ_Lobby_1",
    name = "Baron Regional Headquarters",
    faction_id = "SPARK_BARONS",
    show_agents = true,
    indoors = true,
    tags = {"HQ"},
    patron_data = {
        filter = function(location, agent)
            
            if agent:HasActiveQuestMembership() then
                return false
            end

            return table.arrayfind( baron_patron_list, agent:GetContentID()) ~= nil
        end,
        
        num_patrons = 1,
        patron_generator = function(location)
            TheGame:GetGameState():AddSkinnedAgent(table.arraypick(baron_patron_list)):GetBrain():SendToPatronize(location)
        end,
    },

    work = 
    {
        --garrison1 = CreateLabourJob( PHASE_MASK_ALL, "Garrison", CHARACTER_ROLES.GUARD, "SPARK_BARON_GOON" ),
        garrison = CreateLabourJob( PHASE_MASK_ALL, "Garrison", CHARACTER_ROLES.GUARD, "SPARK_BARON_PROFESSIONAL" ),

        contact = {
            hiring = { open = false }, 
            no_boss = true,
            title = "Spark Baron Fixer", 
            role = CHARACTER_ROLES.CONTACT, 
            hours = PHASE_MASK_ALL, 
        },
    },
}


Content.AddLocationContent{
    id = "GB_NEUTRAL_BAR",
    plax = "INT_Neutral_Bar",
    name = "The Last Stand",
    faction_id = "FEUD_CITIZEN",
    desc = "Neutral ground in the Grout Bog.",
    show_agents = true,
    indoors = true,
    tags = {"tavern"},
    patron_data = {
        patron_generator = generate_patrons,
        start_fn = function(location, agent)
            if math.random() > .5 then
                agent:GainAspect( "drunk" )
            end
        end,
        num_patrons = 
        {
            [DAY_PHASE.DAY] = 2,
            [DAY_PHASE.NIGHT] = 4,
        },
    },
    
    work = 
    {

        bartender = CreateMerchantJob( PHASE_MASK_ALL, "Bartender", CHARACTER_ROLES.PROPRIETOR, "BARTENDER", "GROG_N_DOG_ITEMS"),    
        bouncer = CreateLabourJob( PHASE_MASK_ALL, "Bouncer", CHARACTER_ROLES.GUARD, "JAKES_SMUGGLER"),
    },
    sublocations = 
    {
        inn_room = 
        {
            id = "LAST_STAND_ROOM",
            show_player = true,
            show_agents = true,
            indoors = true,
            tags = {"player_room"},
            name = "Inn room",
            plax = "INT_Neutral_Bar_Backroom",
            --desc = "At least it's dry.",
        }
    },
}
