local PATRON_DEFS = 
{
    ADMIRALTY_CLERK = 1,
    ADMIRALTY_GOON = 1,
    ADMIRALTY_GUARD = 1,
    ADMIRALTY_PATROL_LEADER = 1,
    ADMIRALTY_INVESTIGATOR = .5,

    BANDIT_GOON = 1,
    BANDIT_GOON2 = 1,
    BANDIT_RAIDER = 1,
    BANDIT_CAPTAIN = .5,

    JAKES_RUNNER = 1,
    JAKES_SMUGGLER = 1,
    JAKES_LIFTER = 1,

    LABORER = 1,
    HEAVY_LABORER = 1,
    WEALTHY_MERCHANT = 1,
    POOR_MERCHANT = 1,
    FOREMAN = 1,

    ZEALOT = .5,
    PRIEST = .5,
    LUMINARI = .5,
    LUMINITIATE = .2,
}

local faction_weights = 
{
    [RELATIONSHIP.HATED] = 0,
    [RELATIONSHIP.DISLIKED] = .5,
    [RELATIONSHIP.NEUTRAL] = 1,
    [RELATIONSHIP.LIKED] = 1.25,
    [RELATIONSHIP.LOVED] = 1.5,
}


local function generate_patrons(location)

    local candidates = {}


    for def_id, base_w in pairs(PATRON_DEFS) do
        local w = base_w

        local def = Content.GetCharacterDef(def_id)
        w = w * (faction_weights[TheGame:GetGameState():GetFaction(location:GetFactionID()):GetFactionRelationship( def.faction_id )] or 1)
        if w > 0 then
            candidates[def_id] = w
        end
        
    end
    if next(candidates) then
        
        local def = weightedpick(candidates)
        TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
    end
end


Content.AddLocationContent{
    desc = "The Admiralty's military headquarters.",
    exterior_loc = "BUILDING_EXTERIOR",
    show_agents = true,
    faction_id = "ADMIRALTY",
    id = "ADMIRALTY_BARRACKS",
    indoors = true,
    map_art = 'map_props/test_barracks1.tex',
    name = "Admiralty Headquarters",
    plax = "INT_Admiralty_HQ",
    work = 
    {
        captain = CreateMerchantJob( PHASE_MASK_ALL, "Head Bureaucrat", CHARACTER_ROLES.VENDOR, "ADMIRALTY_CLERK", "ADMIRALTY_CLERK_SHOP"),
        garrison1 = CreateLabourJob( DAY_PHASE.DAY, "Garrison", CHARACTER_ROLES.GUARD, "ADMIRALTY_GUARD"),
        faction_contact = CreateClosedJob( PHASE_MASK_ALL, "Spree Operative", CHARACTER_ROLES.CONTACT ),
    },
}

Content.AddLocationContent{
    id = "MURDER_BAY_HESH_OUTPOST",
    name = "Hesh Outpost",
    plax = "EXT_HESH_OUTPOST",
    indoors = false,
}

Content.AddLocationContent{
    id = "MURDER_BAY_RISE_OUTPOST",
    name = "Rise Camp",
    plax = "EXT_RiseOutpost_01",
    indoors = false,
}


Content.AddLocationContent{
    id = "MURDERBAY_LUMIN_DOCKS",
    name = [[Imperial Lumin Work Office]],
    show_agents = true,
    plax = "INT_Lumindocks_01",
    icon = engine.asset.Texture("UI/location_lumindocks.tex"),

    desc = "The administration offices for the rig gathering lumin offshore of New Deltree.",
    exterior_loc = "BUILDING_EXTERIOR",
    map_art = 'map_props/guard_tower_01.tex',
    tags = {"industry"},
    work = CreateProductionWorkplace( 2, DAY_PHASE.DAY, "FOREMAN", "HEAVY_LABORER", "Chief Lumineer", "Lumin Hauler"),
}

Content.AddLocationContent{
    id = "MURDERBAY_NOODLE_SHOP",
    name = "The Slurping Snail",
    show_agents = true,
    plax = "INT_Noodleshop_1",
    icon = engine.asset.Texture("UI/location_noodleshop.tex"),

    desc =  [[People come here to eat and relax.]],
    exterior_loc = "BUILDING_EXTERIOR",
    indoors = true,
    patron_data = {
        num_patrons = 5,
        patron_generator = generate_patrons,
    },
    tags = { "shop", "restaurant" },
    work = 
    {
        cook = CreateMerchantJob( PHASE_MASK_ALL, "Cook", CHARACTER_ROLES.PROPRIETOR, "BARTENDER"),
        bouncer = CreateLabourJob( PHASE_MASK_ALL, "Bouncer", CHARACTER_ROLES.GUARD, "ADMIRALTY_GUARD" ),
    },
}


local harbour_work = CreateProductionWorkplace(2, DAY_PHASE.DAY, "ADMIRALTY_CLERK", "LABORER", "Shipping Clerk", "Stevedore")
harbour_work.guard = CreateLabourJob( DAY_PHASE.DAY, "Muscle", CHARACTER_ROLES.GUARD, "ADMIRALTY_GUARD" )

local harbour = Content.AddLocationContent{
    
    id = "MURDER_BAY_HARBOUR",
    name = "Harbour",
    desc = "The first point of entry for anyone entering Havaria.",
    show_agents = true,
    indoors = true,
    work = {},
    plax = "INT_Harbour_1",
    work = harbour_work,    
}



Content.AddLocationContent{
    id = "LIGHTHOUSE",
    name = "The Murder Torch",
    show_agents = true,
    plax = "EXT_Lighthouse_MurderTorch",
    desc = "A lonely lighthouse on the banks for murder bay.",
    icon = engine.asset.Texture("UI/location_lighthouse.tex"),
    indoors = false,
    tags = {},
}

Content.AddLocationContent{
    id = "MARKET_STALL",
    name = "Market Stall",
    desc = "An outdoor market selling hard to find goods and services.",
    show_agents = true,
    icon = engine.asset.Texture("UI/location_battleshop.tex"),
    plax = "EXT_RoadsideStall_MurderBay_1",
    indoors = false,
    map_tags = {"road"},
    work = {
        negotiation_shop = CreateClosedJob( PHASE_MASK_ALL, "Card Seller", CHARACTER_ROLES.VENDOR, "MURDER_BAY_NIGHT_MARKET_NEGOTIATION"),
        battle_shop = CreateClosedJob( PHASE_MASK_ALL, "Card Seller", CHARACTER_ROLES.VENDOR, "MURDER_BAY_NIGHT_MARKET_BATTLE"),    
        graft_shop = CreateClosedJob( PHASE_MASK_ALL, "Graft Seller", CHARACTER_ROLES.VENDOR),    
        beastmaster_shop = CreateClosedJob( PHASE_MASK_ALL, "Beastmaster", CHARACTER_ROLES.VENDOR, "MURDER_BAY_BEASTMASTER_SHOP"),    
    }
}

Content.AddLocationContent{
    id = "MB_METTLE_SHOP",
    name = "Mettle Shop",
    desc = "The place the mysterious stranger told you to go to.",
    plax = "ACT1_META_SHOP",
    indoors = true,
}

Content.AddLocationContent{
    id = "GROG_N_DOG",
    name = "Grog n' Dog",
    show_agents = true,
    plax = "INT_GrogNDog_1",
    desc = "A dingy bar owned by a notorious retired bounty hunter.",
    icon = engine.asset.Texture("UI/location_grogndog.tex"),

    faction_id = "JAKES",
    exterior_loc = "BUILDING_EXTERIOR",
    tags = {"tavern"},
    indoors = true,

    patron_data = 
    {
        role = CHARACTER_ROLES.PATRON,
        start_fn = function(location, agent)
            if math.random() > 0.5 then
                AgentUtil.MakeDrunk( agent )
            end
        end,
        num_patrons = 
        {
            [DAY_PHASE.DAY] = 2,
            [DAY_PHASE.NIGHT] = 4,
        },
        patron_generator = generate_patrons,

    },


    sublocations = 
    {
        inn_room = 
        {
            id = "GROG_N_DOG_ROOM",
            show_player = true,
            show_agents = true,
            indoors = true,
            tags = {"player_room"},
            -- scene_scale = 1.2,
            name = "Inn room",
            plax = "INT_Grog_Backroom_01",
            desc = "At least it's dry.",
        }
    },
    work = 
    {
        bartender = CreateClosedJob( PHASE_MASK_ALL, "Bartender", CHARACTER_ROLES.PROPRIETOR, "GROG_N_DOG_ITEMS"),    
        bouncer = CreateLabourJob( PHASE_MASK_ALL, "Bouncer", CHARACTER_ROLES.GUARD, "JAKES_SMUGGLER"),
    }
}


Content.AddLocationContent{
    id = "MURDER_BAY_CHEMIST",
    name = "Better Living Chemists",
    show_agents = true,
    plax = "INT_Chemist_MurderBay_1",
    icon = engine.asset.Texture("UI/location_murderbaychemist.tex"),

    desc =  "Sells healing items",

    tags = {"chemist", "shop"},

    indoors = true,
    exterior_loc = "BUILDING_EXTERIOR",
    
    patron_data = {
        num_patrons = 1,
        patron_generator = generate_patrons,
    },
    work =
    {
        chemist = CreateMerchantJob( DAY_PHASE.DAY, "Chemist", CHARACTER_ROLES.PROPRIETOR, "CHEMIST", "CHEMIST_ITEMS"),
        guard = CreateLabourJob( DAY_PHASE.DAY, "Guard", CHARACTER_ROLES.GUARD, "JAKES_SMUGGLER" ),
    }
}

Content.AddLocationContent{
    id = "NEWDELTREE_OUTFITTERS",
    name = "Trusty's Outfitting",
    show_agents = true,
    plax = "INT_Outfitter",
    icon = engine.asset.Texture("UI/location_newdeltreeoutfitters.tex"),

    desc = "Sells goods and sundries to the denizens of Murder Bay.",

    tags = {"market", "shop", "outfitter"},

    faction_id = "FEUD_CITIZEN",
    exterior_loc = "BUILDING_EXTERIOR",
    patron_data = {
        num_patrons = 1,
        patron_generator = generate_patrons,
    },
    
    work = 
    {
        shopkeeper = CreateMerchantJob( DAY_PHASE.DAY, "Shopkeeper", CHARACTER_ROLES.PROPRIETOR, "WEALTHY_MERCHANT", "ALL_COMBAT_ITEMS"),
        guard = CreateLabourJob( DAY_PHASE.DAY, "Muscle", CHARACTER_ROLES.GUARD, "ADMIRALTY_GUARD" ),
    },
}

Content.AddLocationContent{
    id = "SPREE_INN",
    plax = "INT_SpreeBar_1",
    show_agents = true,
    icon = engine.asset.Texture("UI/location_guttedyote.tex"),
    name = "The Gutted Yote",
    desc = "A rough and tumble tavern frequented by members of the Spree.",
    tags = {"tavern"},
    work = 
    {
        bartender = CreateMerchantJob( PHASE_MASK_ALL, "Bartender", CHARACTER_ROLES.PROPRIETOR, "BARTENDER", "BARTENDER_ITEMS"),    
        bouncer = CreateLabourJob( PHASE_MASK_ALL, "Bouncer", CHARACTER_ROLES.GUARD, "BANDIT_RAIDER"),
        faction_contact = CreateClosedJob( PHASE_MASK_ALL, "Admiralty Operative", CHARACTER_ROLES.CONTACT ),
    },

    faction_id = "BANDITS",

    exterior_loc = "BUILDING_EXTERIOR",
    patron_data = {
        start_fn = function(location, agent)
            if math.random() > .5 then
                agent:GainAspect( "drunk" )
            end
        end,
        patron_generator = generate_patrons,
        num_patrons = 
        {
            [DAY_PHASE.DAY] = 4,
            [DAY_PHASE.NIGHT] = 4,
        },

    },
}

