local PATRON_DEFS = 
{
    JAKES_RUNNER = .25,
    JAKES_SMUGGLER = .25,
    JAKES_LIFTER = .1,
    
    LUMINITIATE = 1,
    PRIEST = .75,
    LUMINARI = .5,
    ZEALOT = .25,

    LABORER = 1,
    HEAVY_LABORER = 1,
    WEALTHY_MERCHANT = .5,
    POOR_MERCHANT = .5,
    FOREMAN = 1,
    PEARLIE = 1.25,
    
    SPARK_BARON_TASKMASTER = .2,
    SPARK_BARON_PROFESSIONAL = .2,
    SPARK_BARON_GOON = .1,

    ADMIRALTY_CLERK = .3,
    ADMIRALTY_GOON = 1,
    ADMIRALTY_GUARD = 1,
    ADMIRALTY_PATROL_LEADER = 1,
    ADMIRALTY_INVESTIGATOR = .5,

    BANDIT_GOON = .3,
    BANDIT_GOON2 = .3,
    BANDIT_RAIDER = .2,
    BANDIT_CAPTAIN = .1,

    RISE_VALET = .5,
}




local faction_weights = 
{
    [RELATIONSHIP.HATED] = 0,
    [RELATIONSHIP.DISLIKED] = .5,
    [RELATIONSHIP.NEUTRAL] = 1,
    [RELATIONSHIP.LIKED] = 1.25,
    [RELATIONSHIP.LOVED] = 1.5,
}


local function generate_patrons(location)

    local candidates = {}


    for def_id, base_w in pairs(PATRON_DEFS) do
        local w = base_w

        local def = Content.GetCharacterDef(def_id)
        w = w * (faction_weights[TheGame:GetGameState():GetFaction(location:GetFactionID()):GetFactionRelationship( def.faction_id )] or 1)
        if w > 0 then
            candidates[def_id] = w
        end
        
    end
    if next(candidates) then
        
        local def = weightedpick(candidates)
        TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
    end
end


Content.AddLocationContent{
    id = "MOREEF_BAR",
    plax = "INT_SMITHBAR",
    name = "Moreef's Place",
    desc = "Sweet Moreef's bar.",
    show_agents = true,
    indoors = true,
    tags = {"tavern"},
    
    patron_data = {
        patron_generator = generate_patrons,
        start_fn = function(location, agent)
            if math.random() > .5 then
                agent:GainAspect( "drunk" )
            end
        end,
        num_patrons = 
        {
            [DAY_PHASE.DAY] = 2,
            [DAY_PHASE.NIGHT] = 4,
        },
    },
    
    work = 
    {
        bartender = CreateMerchantJob( PHASE_MASK_ALL, "Bartender", CHARACTER_ROLES.PROPRIETOR, "BARTENDER", "GROG_N_DOG_ITEMS"),    
        bouncer = CreateClosedJob( PHASE_MASK_ALL, "Bouncer", CHARACTER_ROLES.GUARD ),

    },
    sublocations = 
    {
        inn_room = 
        {
            id = "SWEET_MOREEF_ROOM",
            show_player = true,
            show_agents = true,
            indoors = true,
            tags = {"player_room"},
            name = "Inn room",
            plax = "INT_SMITHBAR_BACKROOM",
            --desc = "At least it's dry.",
        }
    },
}

Content.AddLocationContent{
    id = "PEARL_FANCY_EATS",
    name = "Bistro Rentori",
    desc = "Fill your face with fancy food",
    plax = "INT_SMITH_FANCY_DINNER_PLACE",
    show_agents = true,
    indoors = true,
    map_tags = { "building" },
    work = 
    {
        proprietor = CreateMerchantJob( PHASE_MASK_ALL, "Chef", CHARACTER_ROLES.PROPRIETOR, "BARTENDER"),    
    },

    patron_data = {
        patron_generator = generate_patrons,
        num_patrons = 
        {
            [DAY_PHASE.DAY] = 2,
            [DAY_PHASE.NIGHT] = 4,
        },
    },
}

Content.AddLocationContent{
    id = "PEARL_LUMIN_TANK",
    name = "Lumin Tank",
    desc = "The Cult stores lumin here.",
    plax = "EXT_SMITH_LUMINTANK",
    show_agents = true,
    indoors = false,
}

Content.AddLocationContent{
    id = "PEARL_OSHNUDROME",
    name = "The Oshnudrome",
    desc = "Havaria's most prestigious Oshnu races are held here.",
    plax = "INT_SMITH_OSHNUDROME",
    indoors = true,
    show_agents = true,
    work = 
    {
        proprietor = CreateMerchantJob( PHASE_MASK_ALL, "Pit Boss", CHARACTER_ROLES.PROPRIETOR, "WEALTHY_MERCHANT"),    
        bouncer = CreateLabourJob( PHASE_MASK_ALL, "Security", CHARACTER_ROLES.GUARD, "LUMINARI"),
    },

    patron_data = {
        patron_generator = generate_patrons,
        num_patrons = 
        {
            [DAY_PHASE.DAY] = 2,
            [DAY_PHASE.NIGHT] = 4,
        },
    },
}

Content.AddLocationContent{
    id = "PEARL_CULT_COMPOUND",
    name = "Cult of Hesh Compound",
    --desc = "Havaria's most prestigious Oshnu races are held here.",
    plax = "INT_SMITH_HESHCOMPOUND",
    indoors = true,
    work = 
    {
        guard_1 = CreateLabourJob( PHASE_MASK_ALL, "Security", CHARACTER_ROLES.GUARD, "LUMINARI"),
        guard_2 = CreateLabourJob( PHASE_MASK_ALL, "Security", CHARACTER_ROLES.GUARD, "LUMINARI"),
        guard_3 = CreateLabourJob( PHASE_MASK_ALL, "Security", CHARACTER_ROLES.GUARD, "LUMINARI"),
    },
}

Content.AddLocationContent{
    id = "PEARL_CULT_TRAINING",
    name = "Training Facility",
    --desc = "Havaria's most prestigious Oshnu races are held here.",
    plax = "INT_SMITH_HESHDOJO",
    indoors = true,
    work = 
    {
        guard_1 = CreateLabourJob( PHASE_MASK_ALL, "Security", CHARACTER_ROLES.GUARD, "LUMINITIATE"),
        guard_2 = CreateLabourJob( PHASE_MASK_ALL, "Security", CHARACTER_ROLES.GUARD, "LUMINITIATE"),
        guard_3 = CreateLabourJob( PHASE_MASK_ALL, "Security", CHARACTER_ROLES.GUARD, "LUMINITIATE"),
    },
}

Content.AddLocationContent{
    id = "SMITH_OUTER_GATE",
    name = "Banquod Estate Gate",
    indoors = false,
    plax = "EXT_BANQUOD_GATE",
}

Content.AddLocationContent{
    id = "SMITH_HOME",
    plax = "PLACEHOLDER",
    name = "The Banquod Estate",
    desc = "Where Smith is from",
    show_agents = true,
    indoors = true,
    exterior_loc = "SMITH_OUTER_GATE",
}

Content.AddLocationContent{
    id = "CONTRACT_CHAPEL",
    plax = "INT_SMITH_MUNICIPAL_CHAPEL",
    name = "Pearl-on-the-Foam Municipal Contract Chapel",
    desc = "Your one-stop shop for marriage contracts, tax payments, and building permits.",
    patron_data = {
        patron_generator = generate_patrons,
        num_patrons = 
        {
            [DAY_PHASE.DAY] = 3,
        },
    },
}

Content.AddLocationContent{
        id = "PEARL_HAT_SHOP",
        name = "Pearl Exquisite Millinery",
        plax = "INT_SMITH_MERCHANTSHOP1",
        show_agents = true,
        indoors = true,
        work = 
        {
            proprietor = CreateMerchantJob( PHASE_MASK_ALL, "Proprietor", CHARACTER_ROLES.PROPRIETOR, "WEALTHY_MERCHANT"),    
        },

        patron_data = {
            patron_generator = generate_patrons,
            num_patrons = 
            {
                [DAY_PHASE.DAY] = 3,
            },
        },
    }


Content.AddLocationContent{
    id = "PEARL_METTLE_SHOP",
    name = "Mettle Emporium",
    desc = "The place the mysterious stranger told you to go to.",
    plax = "ACT3_META_SHOP",
    indoors = true,
}


Content.AddLocationContent{
    id = "TEI_SHRINE",
    plax = "EXT_SMITH_TEI",
    name = "Tei's Shrine",
    desc = "A secluded shrine dedicated to Hesh",
    show_agents = true,
}

Content.AddLocationContent{
    id = "PEARL_PARTY_STORE",
    plax = "INT_SMITH_PARTYSTORE",
    name = "Party Supply Store",
    desc = "Sells everything you need to create a good time.",
    show_agents = true,
    work = 
    {
        proprietor = CreateMerchantJob( PHASE_MASK_ALL, "Proprietor", CHARACTER_ROLES.PROPRIETOR, "POOR_MERCHANT"),    
    },

    sublocations = 
    {
        back_market = 
        {
            id = "PARTY_IN_THE_BACK",
            show_agents = true,
            indoors = true,
            name = "The Party In The Back",
            plax = "INT_SMITH_PARTYSTORE_BACKROOM",
            work = 
            {
                proprietor = CreateMerchantJob( PHASE_MASK_ALL, "Proprietor", CHARACTER_ROLES.PROPRIETOR, "STEVEN", "SMITH_PARTY_STORE_STOCK"),    
            },

        }
    },    
}

Content.AddLocationContent{
    id = "PEARL_POOR_HOUSE",
    name = "{1#agent}'s Hovel",
    plax = "EXT_SMITH_POORRES1",
    name_fn = function(location, fmt_str) return loc.format(fmt_str, location:GetPrimaryRoomOccupant() )end,
    show_agents = true,
    map_tags = {"quarters"},
    flags = LOCFLAGS.HOME,
}

Content.AddLocationContent{
    id = "PEARL_RICH_HOUSE",
    name = "{1#agent}'s Home",
    plax = "EXT_SMITH_RICHRESIDENCE",
    name_fn = function(location, fmt_str) return loc.format(fmt_str, location:GetPrimaryRoomOccupant() )end,
    show_agents = true,
    map_tags = {"quarters"},
    flags = LOCFLAGS.HOME,
}

Content.AddLocationContent{
    id = "PEARL_PARK",
    name = "Public Park",
    desc = "A public park.",
    plax = "EXT_Smith_Park",
    show_agents = true,
    patron_data = {
        num_patrons = 4,
        patron_generator = generate_patrons,
    },
}

Content.AddLocationContent{
    id = "PEARL_JAIL",

    name = "Pearl Jail",
    desc = "A holding spot for those who run afoul of the authorities in the Pearl.",
    faction = "ADMIARALTY",
    plax = "INT_SMITH_DRUNKTANK",
    show_agents = true,

}

Content.AddLocationContent{
    id = "PEARL_ADMIRALTY_STATION",
    name = "Admiralty Station",
    desc = "Pearl-on-the-Foam Regional Admiralty Substation #358",
    plax = "INT_SMITH_ADMIRALTYSTATION",
    faction = "ADMIARALTY",
    show_agents = true,
    work = 
    {
        clerk = CreateMerchantJob( DAY_PHASE.DAY, "Head Bureaucrat", CHARACTER_ROLES.VENDOR, "ADMIRALTY_CLERK", "ADMIRALTY_CLERK_SHOP"),
        garrison1 = CreateLabourJob( DAY_PHASE.DAY, "Garrison", CHARACTER_ROLES.GUARD, "ADMIRALTY_GUARD"),
    },
}



