local negotiation_defs = require "negotiation/negotiation_defs"
local CARD_FLAGS = negotiation_defs.CARD_FLAGS
local EVENT = negotiation_defs.EVENT

----------------------------------------
--UNIQUE FACTION CARDS

local CARDS =
{
    admiralty_manipulate =
    {
        name = "Frisk",
        flavour = "'Well well well, what do we have here?'",
        desc = "Gain 3 {frisk}.",

        faction = "ADMIRALTY",
        icon = "negotiation/frisk.tex",
        quip = "admiralty_orders_card",

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        cost = 1,

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier("frisk", 3, self)
        end,
    },

    admiralty_manipulate2 = 
    {
        name = "Flash Badge",
        flavour = "'Move along.'",
        desc = "Destroy target opponent argument. If your opponent is Admiralty, this card costs 0.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_amt )
        end,
        faction = "ADMIRALTY",
        icon = "negotiation/flash_badge.tex",
        quip = "admiralty_orders_card",

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        target_enemy = TARGET_ANY_RESOLVE,

        CanTarget = function( self, target )
            if target and target.modifier_type == MODIFIER_TYPE.CORE then
                return false, CARD_PLAY_REASONS.INVALID_TARGET
            end
            return true
        end,

        cost = 1,

        PreReq = function( self, minigame )
            return self.anti_negotiator.agent:GetFactionID() == "ADMIRALTY"
        end,

        deck_handlers = ALL_DECKS,

        event_handlers = 
        {
            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card == self and self.anti_negotiator.agent:GetFactionID() == "ADMIRALTY" then
                    acc:ModifyValue( 0, self )
                end
            end,
        },

        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                if target.modifier_type ~= MODIFIER_TYPE.CORE then
                    target.negotiator:DestroyModifier(target, self)
                end
            end
        end,
    },

    admiralty_manipulate3 = 
    {
        name = "Resilience",
        flavour = "'I’m like a bad penny.'",
        desc = "Deal damage to an argument equal to the {COMPOSURE} of all your own arguments.",
        faction = "ADMIRALTY",
        quip = "admiralty_orders_card",
        icon = "negotiation/resilience.tex",

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        cost = 1,

        target_enemy = TARGET_ANY_RESOLVE,

        CanPlayCard = function( self, card, engine, target )
            if card == self then
                for i,arg in self.negotiator:Modifiers() do
                    if arg:GetComposure() > 0 then
                        return true
                    end
                end
                return false
            end
        end,

        OnPostResolve = function( self, minigame, targets )
            local damage = 0
            for i,arg in self.negotiator:Modifiers() do
                damage = damage + arg:GetComposure()
            end
            if damage > 0 then
                for i,target in ipairs(targets) do
                    minigame:ApplyPersuasion( self, target, damage, damage )
                end
            end
        end,
    },

    admiralty_diplomacy2 = 
    {
        name = "Oolo's Requisition",
        flavour = "'Just hand them this. They’ll know what it means and they won’t be a problem anymore.'",
        desc = "Win this negotiation.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_amt )
        end,
        faction = "ADMIRALTY",
        quip = "admiralty_orders_card",
        icon = "negotiation/oolos_requisition.tex",

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.CONSUME,
        rarity = CARD_RARITY.UNIQUE,

        cost = 3,

        OnPostResolve = function( self, minigame )
            minigame:Win()
        end,
    },

    very_convincing = 
    {
        name = "Very Convincing",
        flavour = "'Oh I’m sure that they’ll be seeing things my way.'",
        desc = "Draw 2 cards and gain 1 action.",
        faction = "ADMIRALTY",
        rarity = CARD_RARITY.UNIQUE,

        cost = 0,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        OnPostResolve = function(self, minigame)
            minigame:DrawCards(2)
            minigame:ModifyActionCount(1)
        end,
    },

    -- very_convincing2 = 
    -- {
    --     name = "Very Much Convinced",
    --     desc = "Play both {card.{1}} and {card.{2}} with random targets.",
    --     desc_fn = function(self, fmt_str)
    --         if self.cards then
    --             return loc.format(fmt_str, self.cards[1].id or "", self.cards[2].id or "")
    --         else
    --             return fmt_str -- this should only be possible in debug.
    --         end
    --     end,
    --     icon = "negotiation/very_much_convinced.tex",

    --     cost = 10,
    --     flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
    --     rarity = CARD_RARITY.UNIQUE,

    --     hide_in_cardex = true,

    --     event_handlers =
    --     {
    --         [ EVENT.CARD_IMPROVISED ] = function ( self, card, minigame )
    --             if card == self and self.cards then
    --                 self.cost = self.cards[1].cost + self.cards[2].cost
    --             end
    --         end,
    --     },

    --     OnPostResolve = function(self, minigame)
    --         if self.cards then
    --             for i, card in ipairs(self.cards) do
    --                 card:SetFlags( CARD_FLAGS.FREEBIE | CARD_FLAGS.EXPEND )
    --                 minigame:PlayCard( card )
    --             end
    --         end
    --     end,
    -- },

    admiralty_hostile = 
    {
        name  ="Bolstered",
        flavour = "'I don’t want any trouble, I can see the badge.'",
        desc = "Each of your non-core arguments gains 1 stack and 3 resolve.",
        quip = "admiralty_orders_card",
        faction = "ADMIRALTY",
        icon = "negotiation/bolstered.tex",   
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        cost = 1,
        multiplier = 1,

        OnPostResolve = function( self, minigame )
            for i, modifier in self:GetNegotiator():ModifierSlots() do
                if modifier.modifier_type == MODIFIER_TYPE.ARGUMENT then
                    modifier:ModifyResolve( 3, self )
                    self.negotiator:AddModifier( modifier, 1 )
                end
            end
        end        
    },

    admiralty_diplomacy = 
    {
        name = "Enforcement",
        flavour = "'Code sixteen, paragraph 4, line 3....'",
        desc = "{admiralty_diplomacy|}Gain: At the end of your turn, deal 1 damage to a random opponent argument for every stack of {INFLUENCE}.",
        quip = "admiralty_orders_card",
        icon = "negotiation/enforcement.tex",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,
        faction = "ADMIRALTY",
        cost = 2,

        modifier =
        {
            desc = "At the end of your turn, deal <#HILITE>{1}</> damage to a random opponent argument for every stack of {INFLUENCE}.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks or 1 )
            end,

            max_resolve = 4,

            sound = "event:/sfx/battle/cards/neg/create_argument/enforcement",

            event_handlers =
            {
                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    self.target_enemy = TARGET_ANY_RESOLVE
                    for i = 1, self.negotiator:GetModifierStacks("INFLUENCE") do
                        self.min_persuasion, self.max_persuasion = self.stacks, self.stacks
                        minigame:ApplyPersuasion( self )
                        self.min_persuasion, self.max_persuasion = nil, nil
                    end
                    self.target_enemy = nil
                end
            },
        },

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "admiralty_diplomacy", 1, self )
        end,
    },

    insider_information = 
    {
        name = "Insider Information",
        flavour = "'Well, I might know something about that.'",
        desc = "{IMPROVISE} a random uncommon or rare manipulate card. It costs 0 until played.",
        faction = "ADMIRALTY",
        icon = "negotiation/insider_information.tex",

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        upgraded = false,

        OnPostResolve = function( self, minigame)
            local card_defs = NegotiationCardCollection():Filter(function(card_def) 
                                                                return self.id ~= card_def.id and 
                                                                        (card_def.rarity == CARD_RARITY.RARE or card_def.rarity == CARD_RARITY.UNCOMMON)and
                                                                        AgentUtil.CanUseCard(self:GetOwner(), card_def) and 
                                                                        CheckBits( card_def.flags, CARD_FLAGS.MANIPULATE ) and 
                                                                        CheckBits( card_def.flags, CARD_FLAGS.UPGRADED ) == self.upgraded 
                                                            end):Pick(3)
            local cards = table.map(card_defs, 
                    function(card_def) 
                        local card = Negotiation.Card(card_def.id, self:GetOwner()) 
                        card.flags = SetBits( card.flags, CARD_FLAGS.FREEBIE )
                        card:ClearXP()
                        return card
                    end)
            minigame:ImproviseCards( cards, nil, nil, nil, nil, self )
        end,        
    },


}

for i, id, carddef in sorted_pairs( CARDS ) do
    carddef.series = "SAL"
    Content.AddNegotiationCard( id, carddef )
end

