--[[local negotiation_defs = require "negotiation/negotiation_defs"
local CARD_FLAGS = negotiation_defs.CARD_FLAGS
local EVENT = negotiation_defs.EVENT

local CARDS =
{
    deduction =
    {
        name = "Deduction",
        desc = "Deal {1} damage for each {SPENT} card in your hand. Expend them all.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.multiplier )
        end,
        icon = "battle/deduction.tex",
        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 3,
        multiplier = 2,

        OnPostResolve = function( self, minigame )
            local deck = minigame:GetDeck( DECK_TYPE.IN_HAND )
            local spent_cards = 0
            for i,v in ipairs(deck.cards) do
                if v:GetCharges() == 0 then
                    minigame:ExpendCard(v)
                end
            end
        end,

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, acc, minigame )
                if source == self then
                    local deck = minigame:GetDeck( DECK_TYPE.IN_HAND )
                    local spent_cards = 0
                    for i,v in ipairs(deck.cards) do
                        if v:GetCharges() == 0 then
                            spent_cards = spent_cards + 1
                        end
                    end

                    acc:AddPersuasion( spent_cards * self.multiplier, spent_cards * self.multiplier, self )
                end
            end,
        },
    },
    deduction_plus =
    {
        multiplier = 3,
    },
    deduction_plus2 =
    {
        cost = 2,
    },

    induction = 
    {
        name = "Induction",
        desc = "Restore 1 resolve for each {SPENT} card in your hand. Expend them all.",
        icon = "battle/induction.tex",
        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        amount = 0,

        cost = 3,

        CanPlayCard = function ( self, card, engine, target )
            local deck = engine:GetDeck( DECK_TYPE.IN_HAND )
            local spent_cards = 0
            for i,v in ipairs(deck.cards) do
                if v:GetCharges() == 0 then
                    return true
                end
            end
            return false
        end,

        OnPostResolve = function( self, minigame )
            local deck = minigame:GetDeck( DECK_TYPE.IN_HAND )
            local spent_cards = 0
            for i,v in ipairs(deck.cards) do
                if v:GetCharges() == 0 then
                    minigame:ExpendCard(v)
                    spent_cards = spent_cards + 1
                end
            end
            self.negotiator:ModifyResolve(spent_cards, self)
        end,


    },

    induction_plus =
    {
        cost = 2,
    },

    induction_plus2 =
    {
     desc = "Restore 2 resolve for each {SPENT} card in your hand. Expend them all.",
     OnPostResolve = function( self, minigame )
            local deck = minigame:GetDeck( DECK_TYPE.IN_HAND )
            local spent_cards = 0
            for i,v in ipairs(deck.cards) do
                if v:GetCharges() == 0 then
                    minigame:ExpendCard(v)
                    spent_cards = spent_cards + 2
                end
            end
            self.negotiator:ModifyResolve(spent_cards, self)
        end,
    },

    -- breather =
    -- {
    --     name = "Breather",
    --     desc = "Restore 1 charge to a card in your hand",
    --     cost = 1,

    --     flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
    --     rarity = CARD_RARITY.COMMON,

    --     OnPostResolve = function( self, minigame )
    --         local card = minigame:ChooseCard( function( card ) return card:GetCharges() ~= nil end )
    --         if card then
    --             card:RestoreCharges( 1 )
    --         end
    --     end,
    -- },
    -- breather_plus =
    -- {
    --     desc = "Restore 1 charge to all cards in your hand",

    --     OnPostResolve = function( self, minigame )
    --         for i, card in minigame:GetHandDeck():Cards() do
    --             card:RestoreCharges( 1 )
    --         end
    --     end,
    -- },

    -- breather_plus2 =
    -- {
    --     desc = "Restore all charges to a card in your hand",

    --     OnPostResolve = function( self, minigame )
    --         local card = minigame:ChooseCard( function( card ) return card:GetCharges() ~= nil end )
    --         if card then
    --             local cur, max = card:GetCharges()
    --             card:RestoreCharges( max - cur )
    --         end
    --     end,
    -- },

    eureka =
    {
        name = "Eureka!",
        desc = "Discard a card in your hand and recover all of its charges.",

        cost = 2,
        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,

        OnPostResolve = function( self, minigame )
            local card = minigame:ChooseCard()
            if card then
                card:RestoreCharges()
                minigame:DiscardCard( card )
            end
        end,
    },
    eureka_plus =
    {
        cost = 1,
    },

}

for i, id, carddef in sorted_pairs( CARDS ) do
    carddef.series = "GRADLE"

    Content.AddNegotiationCard( id, carddef )
end]]--

