local negotiation_defs = require "negotiation/negotiation_defs"
local CARD_FLAGS = negotiation_defs.CARD_FLAGS
local EVENT = negotiation_defs.EVENT

local CARDS =
{
    hip_flask = 
    {
        name = "Hip Flask",
        cost = 0,
        max_charges = 3,
        flavour = "'Just a quick sip and I’ll be good to go.'",
        item_tags = ITEM_TAGS.CHEMICAL,
        rarity = CARD_RARITY.COMMON,
        features =
        {
            INFLUENCE = 1,
        },
        flags = CARD_FLAGS.EXPEND,
    },

    liquid_courage = 
    {
        name = "Liquid Courage",
        flavour = "'I’ll have what he’s having.'",
        item_tags = ITEM_TAGS.CHEMICAL | ITEM_TAGS.DEFENCE,
        rarity = CARD_RARITY.COMMON,
        quip = "hip_flask_card",

        target_self = TARGET_ANY_RESOLVE,
        flags = CARD_FLAGS.EXPEND,

        cost = 0,
        max_charges = 3,
        features =
        {
            COMPOSURE = 6,
        },
    },

    admiralty_orders =
    {
        name = "Admiralty Orders",
        flavour = "'New orders, listen up.'",
        rarity = CARD_RARITY.UNCOMMON,

        item_tags = ITEM_TAGS.ADMIRALTY | ITEM_TAGS.HOSTILE,
        cost = 0,
        max_charges = 2,

        features =
        {
            DOMINANCE = 2,
        },
    },    
    
    admiralty_intel =
    {
        name = "Admiralty Intel",
        flavour = "'So that’s where that was.'",
        rarity = CARD_RARITY.UNCOMMON,

        quip = "rumour",
        
        item_tags = ITEM_TAGS.ADMIRALTY,
        cost = 0,
        max_charges = 2,
        max_xp = 0,

        features =
        {
            INFLUENCE = 2,
        },
    },

    admiralty_intel_plus =
    {
        name = "Important Admiralty Intel",
        max_charges = 3,
    },

    brain_gills =
    {
        name = "Brain Gills",
        desc = "The next card played this turn gains 3 additional XP.",
        flavour = "'Learn the most popular Kra’deshi poems in two days!'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.xp_gain )
        end,

        item_tags = ITEM_TAGS.UTILITY,
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 0,
        xp_gain = 3,

        OnPostResolve = function( self, minigame )
            local modifier = self.negotiator:CreateModifier( self.id, 1, self )
            modifier.owner_card = self
            modifier.xp_gain = self.xp_gain
        end,

        modifier =
        {
            hidden = true,

            event_handlers =
            {
                [ EVENT.POST_RESOLVE ] = function(self, minigame, card)
                    if card ~= self.owner_card then
                        if not card:IsHatched() then
                            card:AddXP( self.xp_gain, true )
                        end
                        self.negotiator:RemoveModifier( self )
                    end
                end,

                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    self.negotiator:RemoveModifier( self )
                end
            }
        }
    },

    amnesiator =
    {
        name = "Amnesiator",
        flavour = "'Out of your head with a bang!'",
        desc = "The next card played is played twice.\nThen, {CONSUME} it.",

        item_tags = ITEM_TAGS.UTILITY,
        rarity = CARD_RARITY.RARE,

        cost = 1,
        max_charges = 2,

        OnPostResolve = function( self, minigame )
            local modifier = self.negotiator:AddModifier( self.id )
            modifier.skip_card = self
        end,

        modifier =
        {
            hidden = true,

            event_priorities =
            {
                [ EVENT.POST_RESOLVE ] = EVENT_PRIORITY_CLAMP,
            },
            
            event_handlers =
            {
                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    if card ~= self.skip_card and card.owner:IsPlayer() and not card:IsFlagged( CARD_FLAGS.SPECIAL ) then
                        -- Play it again.
                        self.negotiator:RemoveModifier( self, 1 )
                        card:SetFlags( CARD_FLAGS.CONSUME )
                        minigame:PlayCard( card )
                    end
                end,
            },
        },
    },


    rise_manifesto =
    {
        name = "Rise Manifesto",
        flavour = "'I’ve read it a few times and things are starting to make a lot more sense.'",
        desc = "{INCEPT} {1} {INDOCTRINATION}.",
        
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.num_indoctrination )
        end,
        
        local_item = LOCAL_ITEMS.BOG,

        rarity = CARD_RARITY.UNCOMMON,

        --quip = "rumour",
        flags = CARD_FLAGS.REPLENISH,        
        item_tags = ITEM_TAGS.RISE,
        cost = 0,
        max_charges = 2,
        num_indoctrination = 5,

        OnPostResolve = function( self, minigame )
            minigame:GetOpponentNegotiator():AddModifier( "INDOCTRINATION", self.num_indoctrination )
        end,
    },


    erchin_musk =
    {
        name = "Erchin Musk",
        flavour = "'You come in here smelling like that, it’s really distracting to say the least.'",
        desc = "Remove {COMPOSURE} from opponent argument. Gain {DOMINANCE} equal to {COMPOSURE} removed.",

        cost = 1,

        rarity = CARD_RARITY.UNIQUE,
        cost = 1,
        max_charges = 1,

        target_enemy = TARGET_ANY_RESOLVE,

        CanPlayCard = function( self, card, engine, target )
            if card == self then
                return target == nil or (target:GetComposure() or 0) > 0
            end
        end,

        OnPostResolve = function( self, minigame, targets )
            local composure_removed = 0
            for i,target in ipairs(targets) do
                composure_removed = composure_removed + target:GetComposure()
                target:DeltaComposure(-target:GetComposure(), self)
            end
            self.negotiator:AddModifier("DOMINANCE", composure_removed, self)
        end
    },

    sealed_admiralty_envelope =
    {
        name = "Sealed Admiralty Envelope",
        flavour = "'A series of seals are in place to keep things confidential, but who cares?'",
        desc = "Gain 2 {INFLUENCE}.",
        quip = "admiralty_orders_card",
        
        cost = 1,
        max_xp = 4,
        hatch = true,
        flags = negotiation_defs.CARD_FLAGS.ITEM | negotiation_defs.CARD_FLAGS.HATCH,

        rarity = CARD_RARITY.UNIQUE,
        series = "SAL",

        hatch_fn = function( self, minigame )
            self:TransferCard( minigame.trash_deck )
            self:Consume()
            local card = Negotiation.Card( "sealed_admiralty_envelope_hatch", self.owner )
            minigame:DealCard( card, minigame:GetHandDeck( ) )
            card:TransferCard( minigame.trash_deck )
            card:Consume()
            QuestUtil.SpawnQuest("HATCH_ENVELOPE_SPREE")
            QuestUtil.SpawnQuest("HATCH_ENVELOPE_ADMIRALTY")
        end,

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier("INFLUENCE", 2, self)
        end,
    },

    sealed_admiralty_envelope_hatch =
    {
        name = "Opened Admiralty Envelope",
        flavour = "'Seal’s broken.'",
        desc = "The envelope opens to reveal secret admiralty documents. The Admiralty would probably like these back, but the Spree might also be interested.",
        
        cost = 0,
        flags = negotiation_defs.CARD_FLAGS.ITEM | negotiation_defs.CARD_FLAGS.UNPLAYABLE,

        rarity = CARD_RARITY.UNIQUE,
        series = "SAL",
    },

    grisly_trophy = 
    {
        name = "Grisly Trophy",
        desc = "{grisly_trophy|}Gain: At the end of {1} {1*turn|turns}, deal {2} damage to all opponent arguments.",
        flavour = "'The head of some highwayman.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.def.modifier.turns, self.def.modifier.damage )
        end,

        rarity = CARD_RARITY.RARE,
        cost = 1,
        flags = CARD_FLAGS.EXPEND,

        OnPostResolve = function( self )
            self.negotiator:CreateModifier(self.id, 3, self)
        end,

        modifier = 
        {
            desc = "At the end of {1} {1*turn|turns}, deal {2} damage to all opponent arguments.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.negotiator and self.stacks or self.turns, self.damage )
            end,
            max_resolve = 1,
            turns = 3,
            damage = 8,

            target_enemy = TARGET_ANY_RESOLVE,
            target_mod = TARGET_MOD.TEAM,

            OnEndTurn = function( self, minigame )
                if self.stacks == 2 then
                    self.min_persuasion, self.max_persuasion = self.damage, self.damage
                    self:NotifyChanged()

                elseif self.stacks == 1 then
                    self:ApplyPersuasion()

                elseif self.min_persuasion then
                    self.min_persuasion, self.max_persuasion = nil, nil
                    self:NotifyChanged()

                end

                self.negotiator:RemoveModifier( self, 1 )
            end,
        },
    },

    token_of_hesh =
    {
        name = "Token of Hesh",
        desc = "{token_of_hesh|}Gain: At the start of your turn, add {SHIELDED} to a friendly argument.",
        flavour = "'You see, I’ve had this the whole time. A gift from the temple for the most intune with the void.'",

        rarity = CARD_RARITY.RARE,
        cost = 1,
        flags = CARD_FLAGS.EXPEND,
        max_charges = 2,

        OnPostResolve = function( self )
            self.negotiator:CreateModifier(self.id, 1, self)
        end,

        modifier = 
        {
            max_stacks = 1,
            max_resolve = 2,
            desc = "At the start of {1}'s turn, add {SHIELDED} to a friendly argument.",
            desc_fn = function( self, fmt_str )
                local bonus = self.bonus or 0
                return loc.format( fmt_str, self:GetOwnerName()) 
            end,

            ShieldArgument = function( self, target )
                if self.last_shield and self.last_shield:IsApplied() then
                    self.last_shield:SetShieldStatus( nil )
                end

                if target and target:IsApplied() then
                    target:SetShieldStatus( true )
                    self.last_shield = target
                end
            end,

            OnUnapply = function( self )
                self:ShieldArgument( nil )
            end,

            event_handlers =
            {
                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        local targets = {}
                        for i, modifier in self.negotiator:ModifierSlots() do
                            if modifier:GetResolve() ~= nil and not modifier:GetShieldStatus() then
                                table.insert( targets, modifier )
                            end
                        end

                        local target = table.arraypick( targets )
                        if target then
                            self:NotifyTriggered()
                            self:ShieldArgument( target )
                        end

                    end
                end,
            }
        },
    },
}




for i, id, carddef in sorted_pairs( CARDS ) do
    if not id:find( "_plus$") then
        carddef.item_tags = (carddef.item_tags or 0) | ITEM_TAGS.NEGOTIATION
        carddef.flags = (carddef.flags or 0) | CARD_FLAGS.ITEM
        carddef.rarity = carddef.rarity or CARD_RARITY.COMMON
        carddef.series = CARD_SERIES.GENERAL
    end

    Content.AddNegotiationCard( id, carddef )
end

