local negotiation_defs = require "negotiation/negotiation_defs"
local CARD_FLAGS = negotiation_defs.CARD_FLAGS

local EVENT = ExtendEnum( negotiation_defs.EVENT,
{
    "PRE_GAMBLE",
    "GAMBLE",
})

------------------------------------------------------------------

local CARDS =
{
    ------------------------------------------------------------------

    pleasantries =
    {
        name = "Pleasantries",
        desc = "{HEADS}: Deal max damage.",
        flavour = "'Oh, I’ve been well thank you.'",
        cost = 1,
        min_persuasion = 1,
        max_persuasion = 3,
        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.BASIC,
        max_xp = 8,
        wild = true,
        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.negotiator:FindModifier("LUCKY_COIN") and self.negotiator:FindModifier("LUCKY_COIN"):GetCurrentResult() == "HEADS" then
                    persuasion:ModifyPersuasion( persuasion.max_persuasion, persuasion.max_persuasion, self )
                end
            end,
        },
    },

    pleasantries_plus2a =
    {
        name = "Sifting Pleasantries",
        desc = "<#UPGRADE>{PREPARE} a card.</>\n{HEADS}: Deal max damage.",
        OnPostResolve = function( self, minigame, targets )
            minigame:PrepareCard( nil, self )
        end
    },

    pleasantries_plus2b =
    {
        name = "Cordial Pleasantries",
        desc = "<#UPGRADE>Gain 1 {INFLUENCE}.</>\n{HEADS}: Deal max damage.",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("INFLUENCE", 1, self)
        end
    },

    pleasantries_plus2c =
    {
        name = "Tall Pleasantries",
        max_persuasion = 5,
    },

    pleasantries_plus2d =
    {
        name = "Visionary Pleasantries",
        desc = "<#UPGRADE>Draw a card.</>\n{HEADS}: Deal max damage.",
        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(1)
        end
    },

    pleasantries_plus2e = 
    {
        name = "Speculative Pleasantries",
        desc = "<#UPGRADE>{GAMBLE}.</>\n{HEADS}: Deal max damage.",
        OnPreResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            
            if coin and coin:Gamble(self) == "HEADS" then
                self.bonus_damage = true
            else
                self.bonus_damage = false
            end
        end,

        OnPostResolve = function( self, minigame, targets)
            self.bonus_damage = false
        end,


        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.bonus_damage then
                    persuasion:ModifyPersuasion( persuasion.max_persuasion, persuasion.max_persuasion, self )
                end
            end,
        },
    },

    pleasantries_plus2f =
    {
        name = "Decisive Pleasantries",
        desc = "<#UPGRADE>{SET}.</>\n{HEADS}: Deal max damage.",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,

        OnPreResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin and coin:SetCoin(nil, self) == "HEADS" then
                self.bonus_damage = true
            else
                self.bonus_damage = false
            end
        end,

        OnPostResolve = function( self, minigame, targets)
            self.bonus_damage = false
        end,


        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.bonus_damage then
                    persuasion:ModifyPersuasion( persuasion.max_persuasion, persuasion.max_persuasion, self )
                end
            end,
        },
    },

    pleasantries_plus2g = 
    {
        name = "Lucid Pleasantries",
        min_persuasion = 5,
        max_persuasion = 7,
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.CONSUME,
    },

    pleasantries_plus2h =
    {
        name = "Rigged Pleasantries",
        desc = "<#UPGRADE>{RIG} 1 {HEADS}.</>\n{HEADS}: Deal max damage.",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RIG_HEADS", 1, self)
        end,
    },

    grumble =
    {
        name = "Grumble",
        desc = "{SNAILS}: Gain 1 bonus damage.",
        flavour = "'I don’t want to talk to him when he’s like this.'",
        cost = 1,
        min_persuasion = 1,
        max_persuasion = 3,

        max_xp = 6,
        wild = true,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.BASIC,

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.negotiator:FindModifier("LUCKY_COIN") and self.negotiator:FindModifier("LUCKY_COIN"):GetCurrentResult() == "SNAILS" then
                    persuasion:AddPersuasion( 1, 1, self )
                end
            end,
        },
    },

    grumble_plus2a =
    {
        name = "Strained Grumble",
        desc = "<#UPGRADE>Discard a card.</>\n{SNAILS}: Gain 1 bonus damage.",
        OnPostResolve = function( self, minigame, targets )
            minigame:DiscardCards(1, nil, self)
        end,
        min_persuasion = 2,
        max_persuasion = 4,
    },

    grumble_plus2b = 
    {
        name = "Assertive Grumble",
        desc = "<#UPGRADE>Gain {DOMINANCE 1}.</>\n{SNAILS}: Gain 1 bonus damage.",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("DOMINANCE", 1, self)
        end
    },

    grumble_plus2c =
    {
        name = "Rooted Grumble",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    grumble_plus2d = 
    {
        name = "Visionary Grumble",
        desc = "<#UPGRADE>Draw a card.</>\n{SNAILS}: Gain 1 bonus damage.",

        OnPostResolve = function ( self, minigame, targets )
            minigame:DrawCards(1)
        end,
    },

    grumble_plus2e =
    {
        name = "Speculative Grumble",
        desc = "<#UPGRADE>{GAMBLE}.</>\n{SNAILS}: Gain 1 bonus damage.",

        OnPreResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                coin:Gamble(self)
                self:NotifyChanged()
            end
        end,
    },

    grumble_plus2f = 
    {
        name = "Decisive Grumble",
        desc = "<#UPGRADE>{SET}.</>\n{SNAILS}: Gain 1 bonus damage.",
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,

        OnPreResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                coin:SetCoin(nil, self)
                self:NotifyChanged()
            end
        end,
    },

    grumble_plus2g = 
    {
        name = "Lucid Grumble",
        desc = "{SNAILS}: Gain 1 bonus damage.",
        min_persuasion = 6,
        max_persuasion = 6,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.CONSUME,
    },

    grumble_plus2h =
    {
        name = "Mirrored Grumble",
        desc = "<#UPGRADE>Attack with this card twice.</>\n{SNAILS}: Gain 1 bonus damage.",
        min_persuasion = 2,
        max_persuasion = 2,

        OnPostResolve = function( self, minigame, targets )
            minigame:ApplyPersuasion( self )
        end,
    },

    grumble_plus2i =
    {
        name = "Rigged Grumble",
        quip = "improvise_set_card",
        desc = "<#UPGRADE>{RIG} 1 {SNAILS}.</>\n{SNAILS}: Gain 1 bonus damage.",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RIG_SNAILS", 1, self)
        end
    },

    rationale = 
    {
        name = "Rationale",
        desc = "Apply {1} {COMPOSURE}.",
        flavour = "'Let’s think about this here for a second.'",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateComposureText( self.composure_amt ))
        end,

        max_xp = 8,
        wild = true,

        cost = 1,
        composure_amt = 3,
        
        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.BASIC,

        target_self = TARGET_ANY_RESOLVE,

        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
        end
    },

    rationale_plus2a =
    {
        name = "Boosted Rationale",
        desc = "Apply <#UPGRADE>{1} {COMPOSURE}</>.",
        composure_amt = 4,
    },

    rationale_plus2b =
    {
        name = "Visionary Rationale",
        desc = "Apply {1} {COMPOSURE}.\n<#UPGRADE>Draw a card.</>",

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(1)
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
        end,
    },

    rationale_plus2c =
    {
        name = "Crown Rationale",
        desc = "<#UPGRADE>{GAMBLE}.\n{HEADS}: Apply {1} {COMPOSURE}.</>",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateComposureText( 5 ))
        end,
        features = {},
        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                if coin:Gamble(self) == "HEADS" then
                    for i,target in ipairs(targets) do
                        target:DeltaComposure(5, self)
                    end
                end
            end
        end
    },

    rationale_plus2d =
    {
        name = "Sluggish Rationale",
        desc = "Apply {1} {COMPOSURE}.\n<#UPGRADE>{RIG} 1 {SNAILS}.</>",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RIG_SNAILS", 1, self)
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
        end
    },

    rationale_plus2e =
    {
        name = "Headstrong Rationale",
        desc = "Apply {1} {COMPOSURE}.\n<#UPGRADE>{RIG} 1 {HEADS}.</>",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RIG_HEADS", 1, self)
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
        end
    },

    rationale_plus2f =
    {
        name = "Foot Rationale",
        desc = "<#UPGRADE>{GAMBLE}.\n{SNAILS}: Apply {1} {COMPOSURE} to all friendly arguments.</>",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateComposureText( 2 ))
        end,
        target_mod = TARGET_MOD.TEAM,
        target_self = TARGET_ANY_RESOLVE,
        auto_target = true,
        features = {},
        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")            
            if coin and coin:Gamble(self) == "SNAILS" then
                for i, modifier in ipairs(targets) do
                    if modifier:GetResolve() ~= nil then
                        modifier:DeltaComposure(2, self)
                    end
                end
            end
        end
    },

    call_it =
    {
        name = "Call it",
        desc = "{GAMBLE}.",
        flavour = "'Heads or snails?'",

        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.STICKY,
        rarity = CARD_RARITY.BASIC,

        gamble_count = 1,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                for i=1,self.gamble_count do
                    coin:Gamble(self)
                end
            end
        end,
    },

    call_it_plus =
    {
        name = "Decisive Call it",
        desc = "<#UPGRADE>{SET}.</>",
        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                for i=1,self.gamble_count do
                    coin:SetCoin(nil, self)
                end
            end
        end,
    },

    call_it_plus2 =
    {
        name = "Restored Call it",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.STICKY | CARD_FLAGS.REPLENISH,
    },

    trickery =
    {
        name = "Trickery",
        desc = "{IMPROVISE} a card from a pool of special cards.",
        flavour = "'I can’t help but think something fishy is going on here.'",
        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.BASIC,

        pool_size = 3,

        pool_cards = {"improvise_shifty", "improvise_sleight", "improvise_aloof", "improvise_bank", "improvise_reversal", "improvise_gruff", "improvise_carry_over", "improvise_options", "improvise_set"},

        OnPostResolve = function( self, minigame, targets)
            local cards = ObtainWorkTable()

            cards = table.multipick( self.pool_cards, self.pool_size )
            for k,id in pairs(cards) do
                cards[k] = Negotiation.Card( id, self.owner  )
            end
            minigame:ImproviseCards( cards, 1, nil, nil, nil, self )
            ReleaseWorkTable(cards)
        end,
    },

    trickery_plus =
    {
        name = "Promoted Trickery",
        desc = "{IMPROVISE} a card from a pool of <#UPGRADE>upgraded</> special cards.",
        pool_cards = {"improvise_shifty_upgraded", "improvise_sleight_upgraded", "improvise_aloof_upgraded", "improvise_bank_upgraded", "improvise_reversal_upgraded", "improvise_gruff_upgraded", "improvise_carry_over_upgraded", "improvise_options_upgraded", "improvise_set_upgraded"},
    },

    trickery_plus2 = 
    {
        name = "Wide Trickery",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> a card from a pool of special cards.",
        pool_size = 5,
    },

    improvise_set =
    {
        name = "Set",
        desc = "Choose:\n{RIG} 1 {HEADS} or {RIG} 1 {SNAILS}.",
        flavour = "'All on this one cast here.'",
        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            local cards = {
                Negotiation.Card( "coin_heads", self.owner ),
                Negotiation.Card( "coin_snails", self.owner ),
            }
            local pick = self.engine:ImproviseCards( cards, 1, nil, nil, nil, self )[1]
            local side = pick and pick.side or "HEADS"
            if pick then self.engine:ExpendCard(pick) end
            if side == "HEADS" then
                self.negotiator:AddModifier("RIG_HEADS", 1, self)
            else
                self.negotiator:AddModifier("RIG_SNAILS", 1, self)
            end
        end
    },

    improvise_set_upgraded =
    {
        name = "Boosted Set",
        icon = "negotiation/improvise_set.tex",
        desc = "Choose:\n{RIG} 2 {HEADS} or {RIG} 2 {SNAILS}.",
        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            local cards = {
                Negotiation.Card( "coin_heads", self.owner ),
                Negotiation.Card( "coin_snails", self.owner ),
            }
            local pick = self.engine:ImproviseCards( cards, 1, nil, nil, nil, self )[1]
            local side = pick and pick.side or "HEADS"
            if pick then self.engine:ExpendCard(pick) end
            if side == "HEADS" then
                self.negotiator:AddModifier("RIG_HEADS", 2, self)
            else
                self.negotiator:AddModifier("RIG_SNAILS", 2, self)
            end
        end
    },

    improvise_reversal =
    {
        name = "Flipside",
        desc = "Reverse your lucky coin.",
        flavour = "'Wait. I thought it was the other side.'",
        icon = "negotiation/reversal.tex",
        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then coin:Reverse(self) end
        end
    },

    improvise_reversal_upgraded =
    {
        name = "Twisted Flipside",
        icon = "negotiation/reversal.tex",
        desc = "{SET}.",
        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then coin:SetCoin(nil, self) end
        end
    },

    improvise_shifty =
    {
        name = "Shifty",
        desc = "{PREPARE} a card.",
        flavour = "'I don’t trust him but he’s the only game in town.'",

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            minigame:PrepareCard( nil, self )
        end,
    },

    improvise_shifty_upgraded =
    {
        name = "Visionary Shifty",
        desc = "Draw a card, then {PREPARE} a card.",
        icon = "negotiation/improvise_shifty.tex",
        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(1)
            minigame:PrepareCard( nil, self )
        end,
    },

    improvise_sleight =
    {
        name = "Sleight",
        desc = "Draw a card, discard a card.",
        flavour = "'We triple the winning bid but only if we match the opening ante twice.'",

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(1)
            minigame:DiscardCards(1, nil, self)
        end,
    },

    improvise_sleight_upgraded =
    {
        name = "Boosted Sleight",
        desc = "Draw two cards, discard a card.",
        icon = "negotiation/improvise_sleight.tex",
        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(2)
            minigame:DiscardCards(1, nil, self)
        end,
    },

    improvise_aloof = 
    {
        name = "Aloof",
        desc = "Apply 3 {COMPOSURE}.",
        flavour = "'I’m not too worried about it.'",

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        target_self = TARGET_ANY_RESOLVE,

        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(3, self)
            end
        end,
    },

    improvise_aloof_upgraded = 
    {
        name = "Boosted Aloof",
        icon = "negotiation/improvise_aloof.tex",
        desc = "Apply 4 {COMPOSURE}.",

        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        target_self = TARGET_ANY_RESOLVE,

        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(4, self)
            end
        end,
    },

    improvise_bank =
    {
        name = "Bank",
        desc = "Give your currently {PREPARED} card +2 bonus damage until the end of this turn.",
        flavour = "'Put your money where your mouth is. Bets are closing.'",

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            local mod = self.negotiator:AddModifier("improvise_bank", 1, self)
            local hand = self.engine:GetHandDeck().cards
            if mod and hand then table.insert(mod.chosen_cards, hand[1]) end
        end,

        modifier =
        {
            hidden = true,
            chosen_cards = {},
            event_handlers =
            {
                [ EVENT.CALC_PERSUASION] = function( self, source, persuasion )
                    if self.chosen_cards and table.contains(self.chosen_cards, source) then
                        for i,card in ipairs(self.chosen_cards) do
                            if card == source then
                                persuasion:AddPersuasion( 2, 2, self )
                            end
                        end
                    end
                end,

                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    self.negotiator:RemoveModifier("improvise_bank", 1)
                end,
            },
        },
    },

    improvise_bank_upgraded =
    {
        name = "Boosted Bank",
        desc = "Give your currently {PREPARED} card +3 bonus damage until the end of this turn.",
         icon = "negotiation/improvise_bank.tex",
        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            local mod = self.negotiator:AddModifier("improvise_bank_upgraded", 1, self)
            local hand = self.engine:GetHandDeck().cards
            if mod and hand then table.insert(mod.chosen_cards, hand[1]) end
        end,

        modifier =
        {
            hidden = true,
            chosen_cards = {},
            event_handlers =
            {
                [ EVENT.CALC_PERSUASION] = function( self, source, persuasion )
                    if self.chosen_cards and table.contains(self.chosen_cards, source) then
                        for i,card in ipairs(self.chosen_cards) do
                            if card == source then
                                persuasion:AddPersuasion( 3, 3, self )
                            end
                        end
                    end
                end,

                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    self.negotiator:RemoveModifier("improvise_bank_upgraded", 1)
                end,
            },
        },
    },

    ante = 
    {
        name = "Ante",
        desc = "Discard 2 cards, {SET}.",
        flavour = "'I’m in.'",
        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,

        max_xp = 6,
        OnPostResolve = function( self, minigame, targets )
            minigame:DiscardCards(2, nil, self)
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then coin:SetCoin(nil, self) end
        end,
    },

    ante_plus =
    {
        name = "Pale Ante",
        cost = 0,
    },

    ante_plus2 =
    {
        name = "Sticky Ante",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.STICKY,
    },

    blank = 
    {
    	name = "Blank",
    	desc = "{GAMBLE}.\n{HEADS}: Gain {1} bonus damage.",
        flavour = "'Hey pay attention, this here is important.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_boost)
        end,

    	cost = 1,
        min_persuasion = 1,
        max_persuasion = 3,
        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        bonus_damage = false,
        damage_boost = 3,

        OnPostResolve = function( self, minigame, targets)
            self.bonus_damage = false
        end,


        event_handlers = 
        {
        	[ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.bonus_damage then
                    persuasion:AddPersuasion( self.damage_boost, self.damage_boost, self )
                end
            end,

            [ EVENT.START_RESOLVE ] = function( self, minigame, card )
                if card == self then
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
            
                    if coin and coin:Gamble(self) == "SNAILS" then
                        self.bonus_damage = false
                    else
                        self.bonus_damage = true
                    end
                end
            end
        },
    },

    blank_plus =
    {
        name = "Enhanced Blank",
        desc = "{GAMBLE}.\n{HEADS}: Gain <#UPGRADE>{1}</> bonus damage.",
        damage_boost = 5,
    },

    blank_plus2 =
    {
        name = "Boosted Blank",
        min_persuasion = 2,
        max_persuasion = 4,
    },

    burn =
    {
    	name = "Burn",
    	desc = "{SNAILS}: Hits all opponent arguments.",
        flavour = "'Alright already! I said no guns!'",
    	cost = 1,
        min_persuasion = 2,
        max_persuasion = 2,
        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,

        event_handlers = 
        {

            [ EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if card == self and target_deck == self.engine:GetHandDeck() then
                    if self.negotiator:FindModifier("LUCKY_COIN") and self.negotiator:FindModifier("LUCKY_COIN"):GetCurrentResult() == "HEADS" then
                        self.target_mod = TARGET_MOD.SINGLE
                        self.auto_target = nil
                    else
                        self.target_mod = TARGET_MOD.TEAM
                        self.auto_target = true
                    end
                end
            end,

            [ EVENT.GAMBLE ] = function( self, result, source )
                if result == "HEADS" then
                    self.target_mod = TARGET_MOD.SINGLE
                    self.auto_target = nil
                else
                    self.target_mod = TARGET_MOD.TEAM
                    self.auto_target = true
                end
            end,

            [ EVENT.UPDATE_COIN ] = function( self, modifier, got_heads )
                if got_heads then
                    self.target_mod = TARGET_MOD.SINGLE
                    self.auto_target = nil
                else
                    self.target_mod = TARGET_MOD.TEAM
                    self.auto_target = true
                end
            end,

        },
    },

    burn_plus =
    {
        name = "Tall Burn",
        max_persuasion = 4,
    },

    burn_plus2 =
    {
        name = "Boosted Burn",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    bluff = 
    {
    	name = "Bluff",
    	desc = "{HEADS}: This card costs 0.\nDraw 2 cards.",
        flavour = "'I am dead serious.'",
    	cost = 1,
    	flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,
        mod_xp = -1,

        draw_count = 2,

        OnPostResolve = function( self, minigame, targets )
        	minigame:DrawCards(self.draw_count)
        end,

        desired_result = "HEADS",

        event_priorities =
        {
            [ EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers = 
        {
        	[ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card == self and self.negotiator:FindModifier("LUCKY_COIN") and self.negotiator:FindModifier("LUCKY_COIN"):GetCurrentResult() == self.desired_result then
                    acc:ModifyValue( 0, self )
                end
            end,
        },
    },

    bluff_plus =
    {
        name = "Twisted Bluff",
        desc = "{SNAILS}: This card costs 0.\nDraw 2 cards.",
        desired_result = "SNAILS",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.REPLENISH,
    },

    bluff_plus2 =
    {
        name = "Visionary Bluff",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.REPLENISH,
    },

    muck = 
    {
    	name = "Muck",
    	desc = "{GAMBLE}.\n{SNAILS}: Attack with this card twice.",
        flavour = "'Double or nothing, come on you know I’m good for it.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_boost)
        end,
    	cost = 1,

    	flags = CARD_FLAGS.HOSTILE,
    	rarity = CARD_RARITY.COMMON,

    	min_persuasion = 2,
    	max_persuasion = 2,
        bonus = 1,

    	deck_handlers = { DECK_TYPE.DISCARDS },

        OnPostResolve = function( self, minigame )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                if coin:Gamble(self) == "SNAILS" then
                    for i = 1, self.bonus do
                        minigame:ApplyPersuasion( self )
                    end
                end
            end
        end,
    },

    muck_plus =
    {
        name = "Focused Muck",
        desc = "{GAMBLE}.\n{SNAILS}: Attack with this card <#UPGRADE>three times</>.",
        bonus = 2,
    },

    muck_plus2 =
    {
        name = "Boosted Muck",
        min_persuasion = 4,
        max_persuasion = 4,
    },

    dogged = 
    {
        name = "Dogged",
        desc = "{PREPARE} a card.\n{PREPARED}: This card costs 1 less.",
        flavour = "'It’s been a rough week for me, give me a second here.'",
        cost = 2,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 2,
        max_persuasion = 6,

        PreReq = function( self, minigame )
            return self:IsPrepared()
        end,

        OnPostResolve = function( self, minigame, targets )
            minigame:PrepareCard( nil, self )
        end,

        event_priorities =
        {
            [ EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_ADDITIVE,
        },

        event_handlers = 
        {
            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card == self and self:IsPrepared() then
                    acc:AddValue(-1, self)
                end
            end,
        },
    },

    dogged_plus =
    {
        name = "Tall Dogged",
        min_persuasion = 2,
        max_persuasion = 8,
    },

    dogged_plus2 =
    {
        name = "Rooted Dogged",
        min_persuasion = 6,
    },

    pressure =
    {
        name = "Pressure",
        desc = "Spend 1 {INFLUENCE}: This card gains {1} damage until the end of this negotiation.",
        flavour = "'I can’t handle this, It’s just too much.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.gain )
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        min_persuasion = 1,
        max_persuasion = 3,

        gain = 3,

        PreReq = function( self, minigame )
            return self.negotiator:HasModifier("INFLUENCE")
        end,

        OnPreResolve = function( self, minigame, targets )
            if self.negotiator:HasModifier("INFLUENCE") then
                self.bonus = (self.bonus or 0) + self.gain
                self.negotiator:RemoveModifier( "INFLUENCE", 1 )
                self:NotifyChanged()
            end
        end,

        deck_handlers = ALL_DECKS,

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and (self.bonus or 0) > 0 then
                    persuasion:AddPersuasion( self.bonus, self.bonus, self )
                end
            end
        }
    },

    pressure_plus =
    {
        name = "Tall Pressure",
        min_persuasion = 1,
        max_persuasion = 6,
    },

    pressure_plus2 =
    {
        name = "Boosted Pressure",
        desc = "Spend 1 {INFLUENCE}: This card gains <#UPGRADE>{1}</> damage until the end of this negotiation.",
        gain = 5,
    },

    weight = 
    {
        name = "Weight",
        desc = "Costs 1 less per {INFLUENCE}. {GAMBLE}.",
        flavour = "'This one is for all the marbles.'",

        cost = 4,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        min_persuasion = 3,
        max_persuasion = 5,

        PreReq = function( self, minigame )
            return self.negotiator:HasModifier("INFLUENCE")
        end,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then coin:Gamble(self) end
        end,

        event_priorities =
        {
            [EVENT.CALC_ACTION_COST ] = 999, -- AFTER Force Compression
        },

        event_handlers = 
        {
            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card == self and self.negotiator:HasModifier("INFLUENCE") then
                    acc:AddValue(-self.negotiator:GetModifierStacks("INFLUENCE"))
                end
            end
        },
    },

    weight_plus =
    {
        name = "Tall Weight",
        min_persuasion = 3,
        max_persuasion = 7,
    },

    weight_plus2 =
    {
        name = "Pale Weight",
        cost = 3,
    },

    prominence = 
    {
        name = "Prominence",
        desc = "Spend all {INFLUENCE}. {GAMBLE} that many times.",
        flavour = "'Pay day!'",
        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        base_count = 0,

        CanPlayCard = function( self, card, engine, target )
            if card == self then
                if self.negotiator:GetModifierStacks("INFLUENCE") + self.base_count > 0 then
                    return true
                else
                    return false, loc.format( LOC"CONVO_COMMON.REQ_INFLUENCE", 1 )
                end
            end
        end,

        OnPostResolve = function( self, minigame, targets )
            local count = self.negotiator:GetModifierStacks("INFLUENCE") + self.base_count
            self.negotiator:RemoveModifier("INFLUENCE", count)
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            for i=1,count do
                if coin then coin:Gamble(self) end
            end
        end,
    },

    prominence_plus = 
    {
        name = "Boosted Prominence",
        desc = "Spend all {INFLUENCE}. {GAMBLE} that many times<#UPGRADE> +1</>.",
        base_count = 1,
    },

    prominence_plus2 =
    {
        name = "Twisted Prominence",
        desc = "Spend all <#UPGRADE>{DOMINANCE}</>. {GAMBLE} that many times<#UPGRADE> +1</>.",
        base_count = 1,

        CanPlayCard = function( self, card, engine, target )
            if card == self then
                return self.negotiator:GetModifierStacks("DOMINANCE") + self.base_count > 0
            end
        end,

        OnPostResolve = function( self, minigame, targets )
            local count = self.negotiator:GetModifierStacks("DOMINANCE") + self.base_count
            self.negotiator:RemoveModifier("DOMINANCE", count)
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            for i=1,count do
                if coin then coin:Gamble(self) end
            end
        end,
    },

    agency = 
    {
        name = "Agency",
        desc = "Attack 2 random arguments.",
        flavour = "'This is what we need to discuss now.'",
        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 1,
        max_persuasion = 3,

        target_mod = TARGET_MOD.RANDOM1,
        auto_target = true,

        OnPostResolve = function( self, minigame, targets )
            minigame:ApplyPersuasion( self )
        end,
    },

    agency_plus =
    {
        name = "Tall Agency",
        min_persuasion = 1,
        max_persuasion = 6,
    },

    agency_plus2 =
    {
        name = "Rooted Agency",
        min_persuasion = 4,
        max_persuasion = 4,
    },

    blockade = 
    {
        name = "Blockade",
        desc = "{GAMBLE}.\n{HEADS}: Apply {1} {COMPOSURE} to all friendly arguments.",
        flavour = "'He’s a tough nut to crack, I’ll give you that.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText( self.composure_amt ))
        end,
        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,
        desired_result = "HEADS",

        composure_amt = 3,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                if coin:Gamble(self) == self.desired_result then
                    for i, modifier in self.negotiator:Modifiers() do
                        modifier:DeltaComposure(self.composure_amt, self)
                    end
                end
            end
        end
    },

    blockade_plus =
    {
        name = "Boosted Blockade",
        desc = "{GAMBLE}.\n{HEADS}: Apply <#UPGRADE>{1}</> {COMPOSURE} to all friendly arguments.",
        composure_amt = 4,
    },

    blockade_plus2 =
    {
        name = "Twisted Blockade",
        desc = "{GAMBLE}.\n{SNAILS}: Apply <#UPGRADE>{1}</> {COMPOSURE} to all friendly arguments.",
        desired_result = "SNAILS",
        composure_amt = 4,
    },

    overwhelm = 
    {
        name = "Overwhelm",
        desc = "Deal +{1} damage for every argument you have.",
        flavour = "'Am I losing or is it already over?'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.dmg_per_arg)
        end,
        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 1,
        max_persuasion = 1,
        dmg_per_arg = 1,

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self then
                    local count = 0
                    for i, modifier in self.negotiator:Modifiers() do
                        if modifier:GetResolve() ~= nil then
                            count = count + 1
                        end
                    end
                    persuasion:AddPersuasion(count * self.dmg_per_arg, count * self.dmg_per_arg, self)
                end
            end
        },
    },

    overwhelm_plus =
    {
        name = "Boosted Overwhelm",
        min_persuasion = 4,
        max_persuasion = 4,
    },

    overwhelm_plus2 =
    {
        name = "Enhanced Overwhelm",
        desc = "Deal <#UPGRADE>+{1}</> damage for every argument you have.",
        dmg_per_arg = 2,
    },

    dig =
    {
        name = "Dig",
        desc = "{SNAILS}: This card costs 0.",
        flavour = "'The sweetest prey are always buried deep.'",

        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 3,
        max_persuasion = 3,

        desired_result = "SNAILS",

        PreReq = function( self, minigame )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            return coin and coin:GetCurrentResult() == self.desired_result
        end,

        event_priorities =
        {
            [ EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers = 
        {
            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card == self then
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    if coin and coin:GetCurrentResult() == self.desired_result then
                        acc:ModifyValue(0)
                    end
                end
            end
        },
    },

    dig_plus =
    {
        name = "Boosted Dig",
        min_persuasion = 5,
        max_persuasion = 5,
    },

    dig_plus2 = 
    {
        name = "Twisted Dig",
        desc = "<#UPGRADE>{HEADS}</>: This card costs 0.",
        desired_result = "HEADS",
        min_persuasion = 5,
        max_persuasion = 5,
    },

    slugstorm =
    {
        name = "Slugstorm",
        desc = "{GAMBLE}.\n{SNAILS}: Deal {1} damage to all opponent arguments.",
        flavour = "'See? You lose the matriarch in the swarm. It’s what they want.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_amt)
        end,
        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,
        target_enemy = TARGET_ANY_RESOLVE,
        auto_target = true,

        damage_amt = 2,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                local arguments = {}
                for i, modifier in self.anti_negotiator:Modifiers() do
                    if modifier:GetResolve() ~= nil then
                        table.insert(arguments, modifier)
                    end
                end
                if coin:Gamble(self) == "SNAILS" then
                    for i,target in ipairs(arguments) do
                        minigame:ApplyPersuasion(self, target, self.damage_amt, self.damage_amt)   
                    end
                end
            end
        end,
    },

    slugstorm_plus =
    {
        name = "Boosted Slugstorm",
        desc = "{GAMBLE}.\n{SNAILS}: Deal <#UPGRADE>{1}</> damage to all opponent arguments.",
        damage_amt = 4,
    },

    slugstorm_plus2 =
    {
        name = "Wide Slugstorm",
        desc = "{GAMBLE}.\n{SNAILS}: Deal {1} damage to all opponent arguments.\n<#UPGRADE>{HEADS}: Deal 1 damage to all opponent arguments.</>",
        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                local arguments = {}
                for i, modifier in self.anti_negotiator:Modifiers() do
                    if modifier:GetResolve() ~= nil then
                        table.insert(arguments, modifier)
                    end
                end

                if coin:Gamble(self) == "SNAILS" then
                    for i,target in ipairs(arguments) do
                        minigame:ApplyPersuasion(self, target, self.damage_amt, self.damage_amt)   
                    end
                else
                    for i,target in ipairs(arguments) do
                        minigame:ApplyPersuasion(self, target, 1, 1)   
                    end
                end
            end
        end,
    },

    trade = 
    {
        name = "Trade",
        desc = "Spend {1} {INFLUENCE}: Gain {DOMINANCE {2}}.",
        flavour = "'I hope I don’t regret this.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.influence_cost, self.dominance_gain)
        end,
        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,

        influence_cost = 2,
        dominance_gain = 3,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost
        end,

        CanPlayCard = function( self, card, engine, target )
            if card == self then
                return self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost, loc.format( GetLocalizedConvoCommonString( CARD_PLAY_REASONS.REQ_INFLUENCE ), self.influence_cost )
            end
        end,

        OnPostResolve = function( self, minigame, targets )
            local stacks = self.negotiator:GetModifierStacks("INFLUENCE")
            if stacks >= self.influence_cost then
                self.negotiator:RemoveModifier("INFLUENCE", self.influence_cost, self)
                self.negotiator:AddModifier("DOMINANCE", self.dominance_gain, self)
            end
        end,
    },

    trade_plus =
    {
        name = "Pale Trade",
        desc = "Spend <#UPGRADE>{1}</> {INFLUENCE}: Gain {DOMINANCE {2}}.",
        influence_cost = 1,
    },

    trade_plus2 = 
    {
        name = "Boosted Trade",
        desc = "Spend {1} {INFLUENCE}: Gain <#UPGRADE>{DOMINANCE {2}}</>.",
        dominance_gain = 4,
    },

    kicker = 
    {
    	name = "Kicker",
    	desc = "Discard this card: Deal {1} damage to a random target.\n{SNAILS}: Draw a card.",
        flavour = "'This boosts the bet but only if it lands in my favor, understand?'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_amt)
        end,

    	flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.UNPLAYABLE,
    	rarity = CARD_RARITY.UNCOMMON,
        target_enemy = TARGET_ANY_RESOLVE,

        damage_amt = 3,

    	event_handlers =
        {
            [ EVENT.CARD_DISCARDED ] = function( self, card, minigame )
                local coin = self.negotiator:FindModifier("LUCKY_COIN")
                if card == self and coin then
                    self.min_persuasion, self.max_persuasion = self.damage_amt, self.damage_amt
                    minigame:ApplyPersuasion( self )
                    self.min_persuasion, self.max_persuasion = nil, nil
		        	local result = coin:GetCurrentResult()
                    if result == "SNAILS" then
                        minigame:DrawCards(1)
		        	end
                    
                   self:AddXP(1)
                end
            end,
        },
    },

    kicker_plus =
    {
        name = "Boosted Kicker",
        desc = "Discard this card: Deal <#UPGRADE>{1}</> damage to a random target\n{SNAILS}: Draw a card.",
        damage_amt = 5,
    },

    kicker_plus2 =
    {
        name = "Twisted Kicker",
        desc = "Discard <#UPGRADE>or play</> this card: Deal {1} damage to a random target\n{SNAILS}: Draw a card.",
        flags = CARD_FLAGS.HOSTILE,
        cost = 1,
        target_mod = TARGET_MOD.RANDOM1,
        auto_target = true,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            local result = coin:GetCurrentResult()
            self.min_persuasion, self.max_persuasion = self.damage_amt, self.damage_amt
            minigame:ApplyPersuasion( self )
            self.min_persuasion, self.max_persuasion = nil, nil
            if result == "SNAILS" then
                minigame:DrawCards(1)
            end
        end
    },

    dead_draw = 
    {
    	name = "Dead Draw",
    	desc = "Draw 2 cards then {GAMBLE}.\n{HEADS}: Those cards cost 1 less until played.",
        flavour = "'Looks like you are all out of options friend.'",
    	cost = 2,
        mod_xp = 1,

    	flags = CARD_FLAGS.MANIPULATE,
    	rarity = CARD_RARITY.UNCOMMON,

        desired_result = "HEADS",

    	OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                local cards = minigame:DrawCards(2)
                local result = coin:Gamble(self)
        		if result == self.desired_result then
        			
        			local mod = self.negotiator:FindModifier("dead_draw") or self.negotiator:AddModifier("dead_draw", 1, self)
        			if mod then 
                        mod.cards = mod.cards or {}
                        for i,card in ipairs(cards) do
                            table.insert(mod.cards, card)
                        end
                    end
        		end
            end
    	end,

    	modifier =
    	{
    		hidden = true,

    		event_handlers = 
    		{
    			[ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
    				if self.cards and table.contains(self.cards, card) then
                        local count = table.count_if(self.cards, function( v ) return v == card end)
    					acc:AddValue(-1 * count, self)
    				end
    			end,

    			[ EVENT.POST_RESOLVE ] = function( self, minigame, card )
    				while self.cards and table.contains(self.cards, card) do
    					table.arrayremove(self.cards, card)
    				end
    				if self.cards and #self.cards == 0 then
    					self.negotiator:RemoveModifier( self )
    				end
    			end
    		},
    	},
    },

    dead_draw_plus =
    {
        name = "Pale Dead Draw",
        cost = 1,
    },

    dead_draw_plus2 =
    {
        name = "Twisted Dead Draw",
        cost = 1,
        desc = "Draw 2 cards then {GAMBLE}.\n<#UPGRADE>{SNAILS}</>: Those cards cost 1 less until played.",
        
        desired_result = "SNAILS",
    },

    matter_of_fact =
    {
        name = "Matter of Fact",
        loc_strings =
        {
            ALT_DESC = "Lose {COMPOSURE} from all friendly arguments, deal damage equal to the {COMPOSURE} lost ({1} damage).",
        },
        desc = "Lose {COMPOSURE} from all friendly arguments, deal damage equal to the {COMPOSURE} lost.",
        flavour = "'You’re wrong, and here’s why.'",
        desc_fn = function( self, fmt_str )
            if self.engine then
                return loc.format( self.def:GetLocalizedString( "ALT_DESC" ), self:CalculateTotalComposure() )
            else
                return fmt_str
            end
        end,

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.COMMON, 

        target_enemy = TARGET_ANY_RESOLVE,

        damage_boost = 0,

        CanPlayCard = function( self )
            if self:CalculateTotalComposure() <= 0 then
                return false, CARD_PLAY_REASONS.NO_COMPOSURE
            end

            return true
        end,

        CalculateTotalComposure = function( self )
            local composure_used = 0
            for i, modifier in self.negotiator:Modifiers() do
                local comp = modifier:GetComposure()
                if comp and comp > 0 then
                    composure_used = composure_used + comp 
                end
            end
            return composure_used
        end,

        OnPostResolve = function( self, minigame, targets )
            local composure_used = 0
            for i, modifier in self.negotiator:Modifiers() do
                local comp = modifier:GetComposure()
                if comp and comp > 0 then
                    modifier:DeltaComposure(-comp, self)
                    composure_used = composure_used + comp 
                end
            end

            if composure_used > 0 then
                local dmg = composure_used + self.damage_boost
                for i,target in ipairs(targets) do
                    minigame:ApplyPersuasion( self, target, dmg, dmg )
                end
            end
        end
    },

    matter_of_fact_plus =
    {
        name = "Pale Matter of Fact",
        cost = 0,
    },

    matter_of_fact_plus2 =
    {
        name = "Sticky Matter of Fact",
        loc_strings =
        {
            ALT_DESC = "Lose {COMPOSURE} from all friendly arguments, deal damage equal to the {COMPOSURE} lost ({1} damage).",
        },
        desc = "Lose {COMPOSURE} from all friendly arguments, deal damage equal to the {COMPOSURE} lost.",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.STICKY,
    },

    strategy =
    {
        name = "Strategy",
        desc = "{strategy 2|}Gain: Whenever you {PREPARE} a card, deal 2 damage to a random opponent argument.",
        flavour = "'Stick to the plan and we’ll all get what we want.'",

        cost = 2,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("strategy", 2, self)
        end,

        modifier =
        {
            desc = "Whenever you {PREPARE} a card, deal <#HILITE>{1}</> damage to a random opponent argument.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.min_persuasion or 2)
            end,
            max_resolve = 4,
            
            no_damage_tt = true,

            min_persuasion = 2,
            max_persuasion = 2,

            OnSetStacks = function( self )
                self.min_persuasion = self.stacks
                self.max_persuasion = self.stacks
            end,

            event_handlers =
            {
                [ EVENT.CARDS_SWAPPED ] = function( self, minigame, card )
                    self.target_enemy = TARGET_ANY_RESOLVE
                    self:ApplyPersuasion()
                    self.target_enemy = nil
                end
            },
        },
    },

    strategy_plus =
    {
        name = "Pale Strategy",
        cost = 1,
    },

    strategy_plus2 =
    {
        name = "Initial Strategy",
        desc = "{strategy 2|}Gain: Whenever you {PREPARE} a card, deal 2 damage to a random opponent argument.",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    compromise =
    {
        name = "Compromise",
        desc = "{compromise|}Gain: At the start of your turn, gain 1 action and draw 1 less card.",
        flavour = "'This is going to work out really well for us both.'",

        cost = 2,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        modifier = 
        {
            desc = "Gain <#HILITE>{1}</> extra {1*action|actions} and draw <#HILITE>{2}</> less {2*card|cards} at the start of your turn.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks, self.stacks )
            end,

            max_resolve = 4,

            event_handlers =
            {
                [ EVENT.CALC_ACTIONS_PER_TURN ] = function( self, acc )
                    acc:AddValue( self.stacks )
                end,

                [ EVENT.CALC_TURN_DRAW ] = function( self, acc )
                    acc:AddValue( -self.stacks, self )
                end
            },
        },

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier( "compromise", 1, self )
        end
    },

    compromise_plus =
    {
        name = "Initial Compromise",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
   },

    compromise_plus2 =
    {
        name = "Enduring Compromise",
        flags = CARD_FLAGS.DIPLOMACY,
    },

    final_favor =
    {
        name = "Final Favor",
        desc = "Spend all {INFLUENCE}: Apply 2 {COMPOSURE} per {INFLUENCE} spent to all friendly arguments.",
        flavour = "'This is the last one, I promise.'",

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.RARE,

        bonus = 0,

        OnPostResolve = function( self, minigame, targets )
            local count = self.negotiator:GetModifierStacks("INFLUENCE")
            self.negotiator:RemoveModifier("INFLUENCE", count, self)
            for i,modifier in self.negotiator:Modifiers() do
                modifier:DeltaComposure((count *  2) + self.bonus, self)
            end
        end,
    },

    final_favor_plus =
    {
        name = "Pale Final Favor",
        cost = 0,
    },

    final_favor_plus2 =
    {
        name = "Boosted Final Favor",
        desc = "Spend all {INFLUENCE}: Apply 2 {COMPOSURE} per {INFLUENCE} spent <#UPGRADE>+2</> to all friendly arguments.",
        bonus = 2,
    },

    tantrum =
    {
        name = "Tantrum",
        desc = "Discard up to 2 cards, gain {DOMINANCE} equal to their combined cost.",
        flavour = "'It was a stupid game anyway!'",

        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.RARE,

        bonus = 0,
        max_discard = 2,

        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:DiscardCards(0, self.max_discard, self)
            local count = self.bonus
            for i,card in ipairs(cards) do
                count = count + minigame:CalculateActionCost(card)
            end
            if count > 0 then self.negotiator:AddModifier("DOMINANCE", count, self) end
        end
    },

    tantrum_plus =
    {
        name = "Boosted Tantrum",
        desc = "Discard up to 2 cards, gain {DOMINANCE} equal to their combined cost <#UPGRADE> + 2</>.",
        bonus = 2,
    },

    tantrum_plus2 =
    {
        name = "Enhanced Tantrum",
        desc = "Discard up to <#UPGRADE>3</> cards, gain {DOMINANCE} equal to their combined cost.",
        max_discard = 3,
    },

    cash_out =
    {
        name = "Cash Out",
        desc = "This card gains 1 damage every time you {GAMBLE}.",
        flavour = "'No thanks, it’s getting late and I best be going.'",

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        min_persuasion = 2,
        max_persuasion = 2,

        OnPostResolve = function( self, minigame, targets )
            self.bonus = 0
        end,

        deck_handlers = { DECK_TYPE.DISCARDS, DECK_TYPE.IN_HAND, DECK_TYPE.DRAW},

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and (self.bonus or 0) > 0 then
                    persuasion:AddPersuasion( self.bonus, self.bonus, self )
                end
            end,

            [ EVENT.GAMBLE ] = function( self, result, source )
                self.bonus = (self.bonus or 0) + 1
            end
        },
    },

    cash_out_plus =
    {
        name = "Boosted Cash Out",
        min_persuasion = 4,
        max_persuasion = 4,
    },

    cash_out_plus2 =
    {
        name = "Enduring Cash Out",
        desc = "This card gains 1 damage every time you {GAMBLE}. Damage is reset when played.",
        flags = CARD_FLAGS.DIPLOMACY,
    },

    do_over =
    {
        name = "Do Over",
        
        desc = "{CONSUME} a card in hand.\nChoose one of {1} cards to add to your deck permanently.",
        flavour = "'That was just a warmup.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.pool_size )
        end,

        loc_strings =
        {
            CHOOSE_CARD = "Destroy a card from your deck.",
            DRAFT_CARD = "Choose a card to add to your deck.",
        },

        cost = 2,
        pool_size = 3,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        CanPlayCard = function( self, source, engine, target )
            if engine:GetHandDeck():CountCards() == 1 then
                return false, CARD_PLAY_REASONS.NO_VALID_TARGETS
            end

            return true
        end,

        Filter = function( card_def, self, original_card )
            if card_def.rarity == CARD_RARITY.BOSS or card_def.rarity == CARD_RARITY.UNIQUE then
                return false
            end
            
            if CheckBits( card_def.flags, CARD_FLAGS.ITEM ) then
                return false
            end

            if original_card then
                return CheckBits( card_def.flags, CARD_FLAGS.UPGRADED ) == original_card:IsFlagged( CARD_FLAGS.UPGRADED )
            else
                return true
            end
        end,


        OnPostResolve = function( self, minigame, targets )
            local function CanChoose( card )
                -- Any card is fine.
                return true
                -- return minigame:GetPlayer().negotiator:FindCardByID( card.id, card.userdata )
            end
            local card = minigame:ChooseCard( CanChoose, self.def:GetLocalizedString( LOC "CHOOSE_CARD" ))
            if card then
                card:TransferCard( minigame:GetTrashDeck() )
                card:Consume()
            end

            local collection = NegotiationCardCollection.AllUsableCards()
            collection:Filter( self.Filter, self, card )

            local cards = collection:GenerateRandom( self.pool_size )
            local chosen_cards = minigame:ImproviseCards( cards, nil, self.def:GetLocalizedString( "DRAFT_CARD" ), self)
            for i,v in ipairs(chosen_cards) do
                v:MakeTemporary( false )
                local clone = v:Clone()
                clone.deck = nil
                clone.engine = nil
                minigame:GetPlayer().negotiator:AddCard( clone )
            end
        end
    },

    do_over_plus =
    {
        name = "Final Do Over",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.CONSUME,
    },

    do_over_plus2 =
    {
        name = "Wide Do Over",
        desc = "{CONSUME} a card in hand.\nChoose one of <#UPGRADE>{1} cards</> to add to your deck permanently.",
        pool_size = 5,
    },

    foresight =
    {
        name = "Foresight",
        desc = "{foresight|}Gain: At the end of your turn, {RIG} 2 {HEADS} or 2 {SNAILS}.",
        flavour = "'Just lucky I guess.'",

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("foresight", 1, self)
        end,

        modifier = 
        {
            desc = "At the end of your turn, {RIG} 2 {HEADS} or 2 {SNAILS}.",
            max_resolve = 1,
            resolve_per_stack = 2,

            OnSetStacks = function( self, old_stacks )
                local delta = self.stacks - old_stacks
                self.max_resolve = self.max_resolve + (delta * self.resolve_per_stack)
                self.resolve = self.resolve + (delta * self.resolve_per_stack)
            end,

            event_handlers = 
            {
                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    local cards = {
                        Negotiation.Card( "coin_heads", self.owner ),
                        Negotiation.Card( "coin_snails", self.owner ),
                    }
                    local pick = self.engine:ImproviseCards( cards, 1, nil, nil, nil, self )[1]
                    local side = pick and pick.side or "HEADS"
                    if pick then self.engine:ExpendCard(pick) end
                    if side == "HEADS" then
                        self.negotiator:AddModifier("RIG_HEADS", 2, self)
                    else
                        self.negotiator:AddModifier("RIG_SNAILS", 2, self)
                    end
                end
            },
        },
    },

    foresight_plus =
    {
        name = "Initial Foresight",
        desc = "{foresight|}Gain: At the end of your turn, {RIG} 2 {HEADS} or 2 {SNAILS}.",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    foresight_plus2 =
    {
        name = "Pale Foresight",
        cost = 0,
    },

    stacked_deck = 
    {
    	name = "Stacked Deck",
    	desc = "{stacked_deck|}Gain: At the start of your turn, choose a card in your hand and {PREPARE} it.",
        flavour = "'I brought my own from home.'",
    	cost = 1,

    	flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
    	rarity = CARD_RARITY.RARE,

    	OnPostResolve = function( self, minigame, targets )
    		self.negotiator:AddModifier("stacked_deck", 1, self)
    	end,

    	modifier = 
    	{
    		name = "Stacked Deck",
    		desc = "At the start of your turn, choose a card in your hand and {PREPARE} it.",
    		max_resolve = 2,

            resolve_per_stack = 2,

            OnSetStacks = function( self, old_stacks )
                local delta = self.stacks - old_stacks
                self.max_resolve = self.max_resolve + (delta * self.resolve_per_stack)
                self.resolve = self.resolve + (delta * self.resolve_per_stack)
            end,


    		event_handlers =
    		{
    			[ EVENT.HAND_DRAWN ] = function( self, minigame )
    				minigame:PrepareCard( nil, self )
    			end
    		},
    	},
    },

    stacked_deck_plus =
    {
        name = "Initial Stacked Deck",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    stacked_deck_plus2 =
    {
        name = "Pale Stacked Deck",
        cost = 0,
    },

    headbang = 
    {
        name = "Headbang",
        desc = "{HEADS}: Double this card's damage and {GAMBLE}.",
        flavour = "'One second, I love this track.'",
        cost = 2,

        min_persuasion = 3,
        max_persuasion = 8,
        gamble_count = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.RARE,

        desired_result = "HEADS",

        PreReq = function( self, minigame )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            return coin and coin:GetCurrentResult() == self.desired_result
        end,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin and coin:GetCurrentResult() == self.desired_result then
                for i = 1, self.gamble_count do
                    coin:Gamble(self)
                end
            end
        end,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_MULTIPLIER,
        },

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.negotiator:FindModifier("LUCKY_COIN") and self.negotiator:FindModifier("LUCKY_COIN"):GetCurrentResult() == self.desired_result then
                    persuasion:AddPersuasion( persuasion.min_persuasion, persuasion.max_persuasion, self )
                end
            end,
        },
    },

    headbang_plus =
    {
        name = "Rooted Headbang",
        min_persuasion = 6,
        max_persuasion = 6,
    },

    headbang_plus2 =
    {
        name = "Boosted Headbang",
        desc = "{HEADS}: Double this card's damage and {GAMBLE} <#UPGRADE>twice</>.",
        gamble_count = 2,
    },

    helmet =
    {
        name = "Helmet",
        desc = "While this is in your hand, apply {1} {COMPOSURE} to a random argument whenever you {PREPARE} a card.",
        flavour = "'Wow, great hat! Mind if I try it on?'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:CalculateComposureText( self.composure_amt ))
        end,

        cost = 0,

        composure_amt = 3,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.UNPLAYABLE,
        rarity = CARD_RARITY.UNCOMMON,

        target_self = TARGET_ANY_RESOLVE,

        event_handlers =
        {
            [ EVENT.CARDS_SWAPPED ] = function( self, source, card, idx )
                self:AddXP(1)
                local targets = self.engine:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    local target = targets[math.random(#targets)]
                    target:DeltaComposure(self.composure_amt, self)
                end
            end,

        },
    },

    helmet_plus =
    {
        name = "Boosted Helmet",
        desc = "While this is in your hand, apply <#UPGRADE>{1}</> {COMPOSURE} to a random argument whenever you {PREPARE} a card.",

        composure_amt = 4,
    },

    helmet_plus2 =
    {
        name = "Enhanced Helmet",
        desc = "While this is in your hand, apply <#UPGRADE>{1} {COMPOSURE} to all friendly arguments</> whenever you {PREPARE} a card.",
        composure_amt = 2,
        event_handlers =
        {
            [ EVENT.CARDS_SWAPPED ] = function( self, source, card, idx )
                local targets = self.engine:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    for i,target in ipairs(targets) do
                        target:DeltaComposure(self.composure_amt, self)
                    end
                end
            end,

        },
    },

    head_split = 
    {
        name = "Head Split",
        desc = "{GAMBLE}.\n{HEADS}: Deal {1} damage to all opponent arguments.",
        flavour = "'You can split the bet after that landing, haven’t you played before?'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        damage_amt = 3,
        coin_result = "HEADS",

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if not coin then
                return
            end
            if coin:Gamble(self) == self.coin_result then
                local targets = {}
                for i, modifier in minigame:GetOpponentNegotiator():Modifiers() do
                    table.insert(targets, modifier)
                end
                for i,target in ipairs(targets) do
                    minigame:ApplyPersuasion( self, target, self.damage_amt, self.damage_amt )
                end
            end
        end
    },

    head_split_plus =
    {
        name = "Twisted Head Split",
        desc = "{GAMBLE}.\n<#UPGRADE>{SNAILS}</>: Deal <#UPGRADE>{1}</> damage to all opponent arguments.",

        damage_amt = 5,
        coin_result = "SNAILS",
    },

    head_split_plus2 =
    {
        name = "Boosted Head Split",
        desc = "{GAMBLE}.\n{HEADS}: Deal <#UPGRADE>{1}</> damage to all opponent arguments.",
        damage_amt = 5,
    },

    head_turner =
    {
        name = "Head Turner",
        desc = "{HEADS}: Deal max damage.\n{PREPARED}: Gain 1 action.",
        flavour = "'Wait what was that?'",

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        min_persuasion = 1,
        max_persuasion = 4,

        PreReq = function( self, minigame )
            return self:IsPrepared()
        end,

        OnPostResolve = function( self, minigame, targets )
            if self:WasPrepared() then
                minigame:ModifyActionCount(1)
            end
        end,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.negotiator:FindModifier("LUCKY_COIN") and self.negotiator:FindModifier("LUCKY_COIN"):GetCurrentResult() == "HEADS" then
                    persuasion:ModifyPersuasion( persuasion.max_persuasion, persuasion.max_persuasion, self )
                end
            end,
        },
    },

    head_turner_plus =
    {
        name = "Tall Head Turner",
        max_persuasion = 6,
    },

    head_turner_plus2 =
    {
        name = "Visionary Head Turner",
        desc = "{HEADS}: Deal max damage.\n{PREPARED}: Gain 1 action<#UPGRADE> and draw a card.</>",

        OnPostResolve = function( self, minigame, targets )
            if self:WasPrepared() then
                minigame:ModifyActionCount(1)
                minigame:DrawCards(1)
            end
        end,
    },

    head_shot = 
    {
        name = "Head Shot",
        desc = "Targets a random opponent argument.\n{EVOKE}: {GAMBLE} {HEADS} twice.",
        flavour = "'Slipped, sorry.'",

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        target_mod = TARGET_MOD.RANDOM1,

        min_persuasion = 3,
        max_persuasion = 3,
        auto_target = true,
        evoke_max = 2,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS },

        desired_result = "HEADS",

        event_handlers =
        {
            [ EVENT.GAMBLE ] = function( self, result, source )
                if result == self.desired_result then
                    self:Evoke( self.evoke_max )
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ResetEvoke()
                end
            end,
        },
    },

    head_shot_plus =
    {
        name = "Boosted Head Shot",
        min_persuasion = 5,
        max_persuasion = 5,
    },

    head_shot_plus2 =
    {
        name = "Pale Head Shot",
        desc = "Targets a random opponent argument.\n{EVOKE}: {GAMBLE} {HEADS}.",
        evoke_max = 1,
    },

    snail_bite =
    {
        name = "Snail Bite",
        desc = "This card gains 1 damage every time you {GAMBLE} and get {SNAILS}.",
        flavour = "'You should definitely get that looked at.'",

        cost = 2,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.RARE,

        min_persuasion = 1,
        max_persuasion = 4,

        damage_boost = 0,
        damage_scaling = 1,

        deck_handlers = { DECK_TYPE.DISCARDS, DECK_TYPE.IN_HAND, DECK_TYPE.DRAW},

        event_handlers =
        {
            [ EVENT.GAMBLE ] = function( self, result, source )
                if result == "SNAILS" then
                    self.damage_boost = self.damage_boost + self.damage_scaling
                end
            end,

            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self then
                    persuasion:AddPersuasion( self.damage_boost, self.damage_boost, self )
                end
            end,
        },
    },

    snail_bite_plus = 
    {
        name = "Rooted Snail Bite",
        min_persuasion = 4,
        max_persuasion = 4,
    },

    snail_bite_plus2 =
    {
        name = "Tall Snail Bite",
        min_persuasion = 3,
        max_persuasion = 6,
    },

    snail_in_the_pocket =
    {
        name = "Snail in the Pocket",
        desc = "Set the coin to {SNAILS} and discard a card.",
        flavour = "'For emergencies of course.'",

        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,

        discard_amt = 1,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                coin:SetCoin("SNAILS")
            end
            if self.card_draw then
                minigame:DrawCards(self.card_draw)
            end
            minigame:DiscardCards(self.discard_amt, nil, self)
        end
    },

    snail_in_the_pocket_plus =
    {
        name = "Pale Snail in the Pocket",
        cost = 0,
    },

    snail_in_the_pocket_plus2 =
    {
        name = "Visionary Snail in the Pocket",
        desc = "Set the coin to {SNAILS} and <#UPGRADE>draw a card, then</> discard a card.",
        card_draw = 1,
    },

    bottom_snail = 
    {
        name = "Bottom Snail",
        desc = "Discard this card: Draw {1} {1*card|cards} and gain {DOMINANCE {2}}.",
        flavour = "'The one on the bottom tends to hunt more ferociously.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.card_draw, self.dom_gain )
        end,

        cost = 0,

        card_draw = 1,
        dom_gain = 1,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.UNPLAYABLE,
        rarity = CARD_RARITY.COMMON,

        event_handlers =
        {
            [ EVENT.CARD_DISCARDED ] = function( self, card, minigame )
                if card == self then
                    minigame:DrawCards(self.card_draw)
                    self.negotiator:AddModifier("DOMINANCE", self.dom_gain, self)
                    self:AddXP(1)
                end
            end,
        },    
    },

    bottom_snail_plus =
    {
        name = "Visionary Bottom Snail",
        desc = "Discard this card: Draw <#UPGRADE>{1} {1*card|cards}</> and gain {DOMINANCE 1}.",
        card_draw = 2,
    },

    bottom_snail_plus2 =
    {
        name = "Boosted Bottom Snail",
        desc = "Discard this card: Draw {1} {1*card|cards} and gain <#UPGRADE>{DOMINANCE {2}}</>.",
        dom_gain = 2,
    },

    red_herring =
    {
        name = "Red Herring",
        desc = "Gain {DIVERSION {1}}.",
        flavour = "'Is that him? I thought he was dead!'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.diversion_amt or 1)
        end,
        icon =  "negotiation/diversion.tex",

        cost = 2,

        diversion_amt = 2,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("DIVERSION", self.diversion_amt, self)
        end
    },

    red_herring_plus =
    {
        name = "Boosted Red Herring",
        desc = "Gain <#UPGRADE>{DIVERSION {1}}</>.",
        diversion_amt = 3,
    },

    red_herring_plus2 =
    {
        name = "Pale Red Herring",
        cost = 1,
    },

    spin =
    {
        name = "Spin",
        desc = "Deal 1 to 4 damage. {GAMBLE} once per resolve lost.",
        flavour = "'Where it stops on the table is where the scoring bracket is placed, try and keep up please.'",

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.RARE,
        target_enemy = TARGET_ANY_RESOLVE,

        min = 1,
        max = 4,

        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                minigame:ApplyPersuasion( self, target, self.min, self.max )
            end

            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin and self.damage_dealt then
                for i = 1, self.damage_dealt do
                    coin:Gamble(self)
                end
            end
            self.damage_dealt = nil
        end,

        event_handlers =
        {
            [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                if source == self then
                    self.damage_dealt = damage - defended
                end
            end,
        }, 
    },

    spin_plus =
    {
        name = "Rooted Spin",
        desc = "Deal <#UPGRADE>3</> damage. {GAMBLE} once per resolve lost.",
        min = 3,
        max = 3,
    },

    spin_plus2 =
    {
        name = "Tall Spin",
        desc = "Deal 1 to <#UPGRADE>6</> damage. {GAMBLE} once per resolve lost.",
        max = 6,
    },

    pinch =
    {
        name = "Pinch",
        desc = "Gain {1} {VANISH}.",
        flavour = "'Nothing up my sleeves.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.vanish_count)
        end,

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.RARE,
        vanish_count = 4,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("VANISH", self.vanish_count, self)
        end,

    },

    pinch_plus =
    {
        name = "Boosted Pinch",
        desc = "Gain <#UPGRADE>{1}</> {VANISH}.",
        vanish_count = 5,
    },

    pinch_plus2 =
    {
        name = "Initial Pinch",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.AMBUSH,
    },

    grunt = 
    {
        name = "Grunt",
        desc = "{PREPARED}: +{1} damage and draw a card.",
        flavour = "'Those aren’t even words.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.bonus_damage)
        end,

        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 2,
        max_persuasion = 4,
        bonus_damage = 3,

        PreReq = function( self, minigame )
            return self:IsPrepared()
        end,

        OnPostResolve = function( self, minigame, targets )
            if self:WasPrepared() then
                minigame:DrawCards(1)
            end
        end,

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and (self:IsPrepared() or self:WasPrepared()) then
                    persuasion:AddPersuasion(self.bonus_damage, self.bonus_damage, self)
                end
            end
        },
    },

    grunt_plus =
    {
        name = "Boosted Grunt",
        min_persuasion = 3,
        max_persuasion = 5,
    },

    grunt_plus2 =
    {
        name = "Enhanced Grunt",
        desc = "{PREPARED}: +<#UPGRADE>{1}</> damage and draw a card.",
        bonus_damage = 5,
    },

    jargon =
    {
        name = "Jargon",
        desc = "{PREPARED}: apply {1} {COMPOSURE} to a random friendly argument every time you play a card.",
        flavour = "'I’ve never heard that term before but I think I catch your drift.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText(self.composure_amt))
        end,

        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.UNPLAYABLE,
        rarity = CARD_RARITY.UNCOMMON,

        composure_amt = 1,

        event_handlers = 
        {
            [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                if card.negotiator == self.negotiator then
                    if self:IsPrepared() then
                        local targets = self.engine:CollectAlliedTargets(self.negotiator)
                        if #targets > 0 then
                            local target = targets[math.random(#targets)]
                            target:DeltaComposure(self.composure_amt, self)
                            self:AddXP(1)
                        end
                    end
                end
            end
        },
    },

    jargon_plus =
    {
        name = "Boosted Jargon",
        desc = "{PREPARED}: apply <#UPGRADE>{1}</> {COMPOSURE} to a random friendly argument every time you play a card.",
        composure_amt = 2,
    },

    jargon_plus2 =
    {
        name = "Wide Jargon",
        desc = "{PREPARED}: apply {1} {COMPOSURE} to <#UPGRADE>all friendly arguments</> every time you play a card.",
        composure_amt = 1,
        event_handlers = 
        {
            [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                if card.negotiator == self.negotiator then
                    local hand = self.engine:GetHandDeck().cards
                    if hand[1] == self then
                        local targets = self.engine:CollectAlliedTargets(self.negotiator)
                        if #targets > 0 then
                            for i,target in ipairs(targets) do
                                target:DeltaComposure(self.composure_amt, self)
                            end                            
                        end
                    end
                end
            end
        },
    },

    jabber =
    {
        name = "Jabber",
        desc = "{HEADS}: Gain {1} {INFLUENCE}.",
        flavour = "'I think he just talks fast to make it look like he knows what he’s doing.'",

        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.inf_gain )
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 3,
        max_persuasion = 4,
        inf_gain = 1,

        gamble = false,

        PreReq = function( self, minigame )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            return coin and coin:GetCurrentResult() == "HEADS"
        end,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin and coin:GetCurrentResult() == "HEADS" then
                self.negotiator:AddModifier("INFLUENCE", self.inf_gain, self)
            end
        end,
    },

    jabber_plus = 
    {
        name = "Tall Jabber",
        max_persuasion = 6,
    },

    jabber_plus2 =
    {
        name = "Diplomatic Jabber",
        desc = "{HEADS}: Gain <#UPGRADE>{1}</> {INFLUENCE}.",
        inf_gain = 2,
    },

    prattle =
    {
        name = "Prattle",
        desc = "Apply {1} {COMPOSURE}.\n{PREPARED}: Apply {2} {COMPOSURE} to all friendly arguments.",
        flavour = "'Don’t get me started.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText( self.base_composure ), self:CalculateComposureText( self.wide_composure ))
        end,

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,

        base_composure = 3,
        wide_composure = 1,

        target_self = TARGET_ANY_RESOLVE,

        PreReq = function( self, minigame )
            return self:IsPrepared()
        end,

        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.base_composure, self)
            end
            if self:WasPrepared() then
                local targets = self.engine:CollectAlliedTargets(self.negotiator)
                if #targets > 0 then
                    for i,target in ipairs(targets) do
                        target:DeltaComposure(self.wide_composure, self)
                    end                            
                end
            end
        end,
    },

    prattle_plus =
    {
        name = "Focused Prattle",
        desc = "Apply <#UPGRADE>{1}</> {COMPOSURE}.\n{PREPARED}: Apply {2} {COMPOSURE} to all friendly arguments.",
        base_composure = 5,
    },

    prattle_plus2 =
    {
        name = "Wide Prattle",
        desc = "Apply <#DOWNGRADE>{1}</> {COMPOSURE}.\n{PREPARED}: Apply <#UPGRADE>{2}</> {COMPOSURE} to all friendly arguments.",
        base_composure = 2,
        wide_composure = 2,
    },

    praise =
    {
        name = "Praise",
        desc = "{PREPARED}: Gain {1} {INFLUENCE}.",
        flavour = "'A for effort.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.influence_gain)
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY, 
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 1,
        max_persuasion = 6,

        influence_gain = 2,

        PreReq = function( self, minigame )
            return self:IsPrepared()
        end,

        OnPostResolve = function( self, minigame, targets )
            if self:WasPrepared() then
                self.negotiator:AddModifier("INFLUENCE", self.influence_gain, self)
            end
        end
    },

    praise_plus =
    {
        name = "Cordial Praise",
        desc = "{PREPARED}: Gain <#UPGRADE>{1}</> {INFLUENCE}.",
        influence_gain = 3,
    },

    praise_plus2 = 
    {
        name = "Tall Praise",
        max_persuasion = 8,
    },

    obligation =
    {
        name = "Obligation",
        desc = "{PREPARE} a card, gain {INFLUENCE} equal to its cost.",
        flavour = "'Look they helped me out, my hands are tied.'",

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        desired_mod = "INFLUENCE",
        bonus = 0,

        OnPostResolve = function( self, minigame, targets )
            local card = minigame:PrepareCard( nil, self )
            if card then
                self.negotiator:AddModifier(self.desired_mod, minigame:CalculateActionCost(card) + self.bonus, self)
            end
        end,
    },

    obligation_plus =
    {
        name = "Boosted Obligation",
        desc = "{PREPARE} a card, gain {INFLUENCE} equal to its cost<#UPGRADE> +1</>.",
        bonus = 1,
    },

    obligation_plus2 =
    {
        name = "Twisted Obligation",
        desc = "{PREPARE} a card, gain <#UPGRADE>{DOMINANCE}</> equal to its cost<#UPGRADE> +1</>.",
        desired_mod = "DOMINANCE",
        bonus = 1,
    },

    rant = 
    {
        name = "Rant",
        desc = "Discard a card. Gain {DOMINANCE {1}}.",
        flavour = "'And another thing!'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.dom_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,

        dom_amt = 1,

        OnPostResolve = function( self, minigame, targets )
            if self.draw_count then
                minigame:DrawCards(self.draw_count)
            end
            local card = minigame:DiscardCards(1, nil, self)[1]
            if card then
                self.negotiator:AddModifier("DOMINANCE", self.dom_amt, self)
            end
        end
    },

    rant_plus =
    {
        name = "Boosted Rant",
        desc = "Discard a card. Gain <#UPGRADE>{DOMINANCE {1}}</>.",
        dom_amt = 2,
    },

    rant_plus2 =
    {
        name = "Visionary Rant",
        desc = "<#UPGRADE>Draw a card, then</> discard a card. Gain {DOMINANCE {1}}.",
        draw_count = 1,
    },

    gab =
    {
        name = "Gab",
        desc = "{GAMBLE} twice.\n{EVOKE}: {PREPARE} a card.",
        flavour = "'I’m not one to gossip, but I did see him yesterday with a suspiciously noisy bag.'",


        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        gamble_count = 2,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            if coin then
                for i=1,self.gamble_count do
                    coin:Gamble(self)
                end
            end
        end,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS },

        event_handlers =
        {
            [ EVENT.CARDS_SWAPPED ] = function( self, source, card, idx )
                self:Evoke(1)
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ResetEvoke()
                end
            end,
        },
    },

    gab_plus = 
    {
        name = "Boosted Gab",
        desc = "{GAMBLE} <#UPGRADE>three</> times.\n{EVOKE}: {PREPARE} a card.",
        gamble_count = 3,
    },

    gab_plus2 =
    {
        name = "Flexible Gab",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
    },

    upright =
    {
        name = "Upright",
        desc = "If you have {INFLUENCE}, {PREPARE} a card.",
        flavour = "'It’s all in how you carry yourself.'",

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 2,
        max_persuasion = 4,

        PreReq = function( self, minigame )
            return self.negotiator:HasModifier("INFLUENCE")
        end,

        OnPostResolve = function( self, minigame, targets )
            if self.negotiator:HasModifier("INFLUENCE") then
                minigame:PrepareCard( nil, self )
            end
        end
    },

    upright_plus =
    {
        name = "Tall Upright",
        max_persuasion = 7,
    },

    upright_plus2 =
    {
        name = "Rooted Upright",
        min_persuasion = 4,
    },

    spotty =
    {
        name = "Spotty",
        desc = "When this card is drawn, gain {1} {1*action|actions} and {EXPEND}.",
        flavour = "'You start again when the flips are that pattern, everyone knows that.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.actions )
        end,

        cost = 0,
        actions = 1,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.UNPLAYABLE,
        rarity = CARD_RARITY.UNCOMMON,

        event_handlers =
        {
            [ EVENT.DRAW_CARD ] = function( self, minigame, card, start_of_turn )
                if card == self then
                    self:NotifyTriggeredPre()
                    minigame:ModifyActionCount( self.actions )
                    self:NotifyTriggeredPost()
                    if self.draw then
                        minigame:DrawCards( 1 )
                    end
                    self:AddXP(1)
                    minigame:ExpendCard( self )
                end
            end
        },
    },

    spotty_plus =
    {
        name = "Boosted Spotty",
        desc = "When this card is drawn, gain <#UPGRADE>{1} {1*action|actions}</> and {EXPEND}.",
        actions = 2,
    },

    spotty_plus2 =
    {
        name = "Visionary Spotty",
        desc = "When this card is drawn, gain {1} {1*action|actions}, <#UPGRADE>draw 1 card</>, and {EXPEND}.",
        draw = 1,
    },

    dilemma =
    {
        name = "Dilemma",
        desc = "Every time you {GAMBLE}, reduce the cost of this card by 1 until played.",
        flavour = "'Give me time to think it over.'",

        cost = 4,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 2,
        max_persuasion = 6,
        cost_reduction = 0,

        deck_handlers = { DECK_TYPE.DISCARDS, DECK_TYPE.IN_HAND, DECK_TYPE.DRAW},

        OnPostResolve = function( self, minigame, targets )
            self.cost_reduction = 0
        end,

        event_priorities =
        {
            [EVENT.CALC_ACTION_COST ] = 999, -- AFTER Force Compression
        },

        event_handlers =
        {
            [ EVENT.GAMBLE ] = function( self, result, source )
                self.cost_reduction = self.cost_reduction + 1
            end,

            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card == self then
                    acc:AddValue( -self.cost_reduction, self )
                end
            end,
        },
    },

    dilemma_plus = 
    {
        name = "Boosted Dilemma",
        max_persuasion = 8,
    },

    dilemma_plus2 =
    {
        name = "Pale Dilemma",
        cost = 3,
    },

    decency =
    {
        name = "Decency",
        desc = "Gain {INFLUENCE {1}}. Draw {2} card.",
        flavour = "'I'm sure that we can work this out like adults.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_amt, self.draw_count)
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,
        draw_count = 1,
        influence_amt = 1,

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards( self.draw_count )
            self.negotiator:AddModifier("INFLUENCE", self.influence_amt, self)
        end,

    },

    decency_plus =
    {
        name = "Boosted Decency",
        desc = "Gain {INFLUENCE {1}}. Draw <#UPGRADE>{2} cards</>.",

        draw_count = 2,
    },

    decency_plus2 =
    {
        name = "Enhanced Decency",
        desc = "Gain <#UPGRADE>{INFLUENCE {1}}</>. Draw {2} card.",

        influence_amt = 2,
    },

    foul_mouth =
    {
        name = "Foul Mouth",
        desc = "Discard a card.",
        flavour = "'You kiss your mother with that mouth?'",
        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 2,
        max_persuasion = 4,

        OnPostResolve = function( self, minigame, targets )
            minigame:DiscardCards(1, nil, self)
        end,
    },

    foul_mouth_plus =
    {
        name = "Boosted Foul Mouth",
        min_persuasion = 3,
        max_persuasion = 6,
    },

    foul_mouth_plus2 =
    {
        name = "Visionary Foul Mouth",
        desc = "<#UPGRADE>Draw a card,</> discard a card.",
        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(1)
            minigame:DiscardCards(1, nil, self)
        end,
    },

    seethe =
    {
        name = "Seethe",
        desc = "Create {1} {HEATED}.\nDiscard this card: Add a copy of this card to your draw pile.",
        flavour = "'When I finally get my hands on him he’s in a whole heap of trouble.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.heated_count)
        end,

        cost = 1,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,
        mod_xp = 2,

        heated_count = 1,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:CreateModifier("HEATED", self.heated_count, self)
        end,

        event_handlers =
        {
            [ EVENT.CARD_DISCARDED ] = function( self, card, minigame )
                if card == self then
                    local copy = self:Duplicate()
                    minigame:DealCard( copy, minigame:GetDeck( DECK_TYPE.DRAW ))
                end
            end,
        },
    },

    seethe_plus =
    {
        name = "Boosted Seethe",
        desc = "Create <#UPGRADE>{1}</> {HEATED}.\nDiscard this card: Add a copy of this card to your draw pile.",
        heated_count = 2,
    },

    seethe_plus2 =
    {
        name = "Visionary Seethe",
        desc = "Create {1} {HEATED}.\nDiscard this card: Add a copy of this card to your draw pile<#UPGRADE> and draw a card</>.",

        event_handlers =
        {
            [ EVENT.CARD_DISCARDED ] = function( self, card, minigame )
                if card == self then
                    local copy = self:Duplicate()
                    minigame:DealCard( copy, minigame:GetDeck( DECK_TYPE.DRAW ))
                    minigame:DrawCards(1)
                end
            end,
        },
    },

    vulgar =
    {
        name = "Vulgar",
        desc = "Draw {1} cards. {GAMBLE} for each Hostility card drawn.",
        flavour = "'Don’t sit at the table if you don’t know the rules you idiot.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.card_draw)
        end,

        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        card_draw = 2,

        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:DrawCards(self.card_draw)
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            for i,card in ipairs(cards) do
                if CheckBits( card.flags, CARD_FLAGS.HOSTILE) then
                    if coin then coin:Gamble(self) end
                end
            end
        end
    },

    vulgar_plus =
    {
        name = "Visionary Vulgar",
        desc = "Draw <#UPGRADE>{1}</> cards. {GAMBLE} for each Hostility card drawn.",
        card_draw = 3,
    },

    vulgar_plus2 =
    {
        name = "Boosted Vulgar",
        desc = "Draw {1} cards. {GAMBLE} for each Hostility card drawn<#UPGRADE> +1</>.",
        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:DrawCards(self.card_draw)
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            for i,card in ipairs(cards) do
                if CheckBits( card.flags, CARD_FLAGS.HOSTILE) then
                    if coin then coin:Gamble(self) end
                end
            end
            if coin then coin:Gamble(self) end
        end
    },

    raw =
    {
        name = "Raw",
        desc = "When this card is drawn, {GAMBLE}.",
        flavour = "'Watch the birdie.'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.strength )
        end,

        cost = 0,
        rarity = CARD_RARITY.COMMON,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.UNPLAYABLE,
        discard = false,
        draw = false,
        mod_xp = 1,

        event_handlers =
        {
           [ EVENT.DRAW_CARD ] = function( self, minigame, card, start_of_turn )
                 if card == self then
                    self.engine:PushPostHandler( function()
                        self:NotifyTriggeredPre()
                        local coin = self.negotiator:FindModifier("LUCKY_COIN")
                        if coin then coin:Gamble(self) end
                        self:AddXP(1)

                        self:NotifyTriggeredPost()
                        if self.discard then self.engine:DiscardCards(1, nil, self) end
                        if self.draw then self.engine:DrawCards(1) end
                    end )
                end
            end,
        }
    },

    raw_plus =
    {
        name = "Boosted Raw",
        desc = "When this card is drawn, {GAMBLE}<#UPGRADE> and draw a card</>.",
        draw = true,
    },

    raw_plus2 =
    {
        name = "Strained Raw",
        desc = "When this card is drawn, {GAMBLE}. <#UPGRADE>Then discard a card.</>",
        discard = true,
    },

    snail_hunt =
    {
        name = "Snail Hunt",
        desc = "{RIG} {1} {SNAILS}, then {IMPROVISE} a Hostility card from your draw pile.",
        flavour = "'I’ve never seen them this aggressive, must be starving.'",
        desc_fn = function ( self, fmt_str )
            return loc.format(fmt_str, self.rig_amt)
        end,
        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        rig_amt = 2,
        improvise_size = 3,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RIG_SNAILS", self.rig_amt, self)
            local cards = {}
            for i, card in minigame:GetDrawDeck():Cards() do
                if CheckBits(card.flags, CARD_FLAGS.HOSTILE) then
                    table.insert(cards, card)
                end
            end
            if #cards == 0 then
                minigame:ShuffleDiscardToDraw()
                for i, card in minigame:GetDrawDeck():Cards() do
                    if CheckBits(card.flags, CARD_FLAGS.HOSTILE) then
                        table.insert(cards, card)
                    end
                end
            end

            minigame:ImproviseCards( table.multipick(cards, self.improvise_size), self.count, nil, "fake_it", nil, self )
        end
    },

    snail_hunt_plus =
    {
        name = "Rigged Snail Hunt",
        desc = "{RIG} <#UPGRADE>{1}</> {SNAILS}, then {IMPROVISE} a Hostility card from your draw pile.",
        rig_amt = 3,
    },

    snail_hunt_plus2 =
    {
        name = "Enhanced Snail Hunt",
        desc = "{RIG} {1} {SNAILS}, then <#UPGRADE>{IMPROVISE_PLUS}</> a Hostility card from your draw pile.",
        improvise_size = 5,
    },

    radula = 
    {
        name = "Radula",
        desc = "{RIG} {1} {SNAILS}. Draw a card.",
        flavour = "'See the markings on the shell? This one's a fighter, that’s for sure.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.rig_amt)
        end,

        quip = "improvise_set_card",
        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        rig_amt = 3,
        card_draw = 1,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RIG_SNAILS", self.rig_amt, self)
            minigame:DrawCards(self.card_draw)
        end
    },

    radula_plus = 
    {
        name = "Rigged Radula",
        desc = "{RIG} <#UPGRADE>{1}</> {SNAILS}. Draw a card.",
        rig_amt = 4,
    },

    radula_plus2 =
    {
        name = "Visionary Radula",
        desc = "{RIG} {1} {SNAILS}. Draw <#UPGRADE>2 cards</>.",
        card_draw = 2,
    },

    disregard = 
    {
        name = "Disregard",
        desc = "Discard your hand. Draw {1} cards.",
        flavour = "'I’ve stopped listening.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.card_draw)
        end,

        cost = 1,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        card_draw = 5,

        OnPostResolve = function( self, minigame, targets )
            local tbl = ObtainWorkTable()
            for i, card in minigame:GetHandDeck():Cards() do
                table.insert(tbl, card)
            end

            for i, card in ipairs(tbl) do
                minigame:DiscardCard(card)
            end

            ReleaseWorkTable(tbl)
            minigame:DrawCards(self.card_draw)
        end,
    },

    disregard_plus =
    {
        name = "Enduring Disregard",
        flags = CARD_FLAGS.HOSTILE,
    },

    disregard_plus2 =
    {
        name = "Pale Disregard",
        desc = "Discard your hand. Draw <#UPGRADE>{1}</> cards.",
        cost = 0,
        card_draw = 7,
    },

    clear_head =
    {
        name = "Clear Head",
        desc = "{RIG} {1} {HEADS}, then {PREPARE} a card.",
        flavour = "'I just needed to get some air.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.rig_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY, 
        rarity = CARD_RARITY.UNCOMMON,

        rig_amt = 2,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RIG_HEADS", self.rig_amt, self)
            if self.draw_card then minigame:DrawCards(1) end
            minigame:PrepareCard( nil, self )
        end
    },

    clear_head_plus =
    {
        name = "Rigged Clear Head",
        desc = "{RIG} <#UPGRADE>{1}</> {HEADS}, then {PREPARE} a card.",
        rig_amt = 4,
    },

    clear_head_plus2 =
    {
        name = "Visionary Clear Head",
        desc = "{RIG} {1} {HEADS}, <#UPGRADE>draw a card</> then {PREPARE} a card.",
        draw_card = true,
    },

    venture =
    {
        name = "Venture",
        desc = "Spend up to {1} {INFLUENCE}: {RIG} {2} {HEADS} for each {INFLUENCE} spent.",
        flavour = "'You can lay in with this bet, but it costs you a little more than a good word.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.influence_max, self.rig_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        rig_amt = 3,
        influence_max = 2,
        bonus = 0,

        OnPostResolve = function( self, minigame, targets )
            local count = math.min(self.negotiator:GetModifierStacks("INFLUENCE"), self.influence_max)
            self.negotiator:AddModifier("RIG_HEADS", (self.rig_amt * count) + self.bonus, self)
            self.negotiator:RemoveModifier("INFLUENCE", count)
        end
    },

    venture_plus =
    {
        name = "Boosted Venture",
        desc = "Spend up to {1} {INFLUENCE}: {RIG} {2} {HEADS} for each {INFLUENCE} spent<#UPGRADE> + 1</>.",
        bonus = 1,
    },

    venture_plus2 =
    {
        name = "Enhanced Venture",
        desc = "Spend up to <#UPGRADE>{1}</> {INFLUENCE}: {RIG} {2} {HEADS} for each {INFLUENCE} spent.",
        influence_max = 3,
    },

    square_up =
    {
        name = "Square Up",
        desc = "Gain {1} {MOMENTUM}.",
        flavour = "'Alright, you asked for it.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.momentum_stacks)
        end,

        cost = 2,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        momentum_stacks = 1,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("MOMENTUM", self.momentum_stacks, self)
        end,
    },

    square_up_plus =
    {
        name = "Boosted Square Up",
        desc = "Gain <#UPGRADE>{1}</> {MOMENTUM}.",
        momentum_stacks = 2,
    },

    square_up_plus2 =
    {
        name = "Pale Square Up",
        cost = 1,
    },

    brute =
    {
        name = "Brute",
        desc = "Hits all opponent arguments. Lose all {DOMINANCE}.",
        flavour = "'The job over on the docks? That was this guy?'",

        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        target_mod = TARGET_MOD.TEAM,
        auto_target = true,

        min_persuasion = 2,
        max_persuasion = 3,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:RemoveModifier("DOMINANCE", self.negotiator:GetModifierStacks("DOMINANCE"), self)
        end,
    },

    brute_plus =
    {
        name = "Boosted Brute",
        min_persuasion = 3,
        max_persuasion = 4,
    },

    brute_plus2 =
    {
        name = "Maintained Brute",
        desc = "Hits all opponent arguments. Lose all {DOMINANCE}.\n<#UPGRADE>Gain {DOMINANCE 1}.</>",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:RemoveModifier("DOMINANCE", self.negotiator:GetModifierStacks("DOMINANCE"), self)
            self.negotiator:AddModifier("DOMINANCE", 1, self)
        end,
    },

    reckless_insults =
    {
        name = "Reckless Insults",
        desc = "Spend all {DOMINANCE}. {GAMBLE} and deal {1} damage to a random opponent argument for every {DOMINANCE} spent.",
        flavour = "'Wow, those were some harsh words. Are you going to be alright?'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_amt)
        end,
        cost = 2,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        damage_amt = 2,

        OnPostResolve = function( self, minigame, targets )
            local coin = self.negotiator:FindModifier("LUCKY_COIN")
            local count = self.negotiator:GetModifierStacks("DOMINANCE")
            self.negotiator:RemoveModifier("DOMINANCE", count, self)
            if coin then
                for i=1,count do
                    coin:Gamble(self)
                    local targets = minigame:CollectAlliedTargets(minigame:GetOtherNegotiator(self.negotiator))
                    if #targets > 0 then
                        minigame:ApplyPersuasion(self, targets[math.random(#targets)], self.damage_amt, self.damage_amt )
                    end
                end
            end
        end,
    },

    reckless_insults_plus =
    {
        name = "Pale Reckless Insults",
        cost = 1,
    },

    reckless_insults_plus2 =
    {
        name = "Boosted Reckless Insults",
        desc = "Spend all {DOMINANCE}. {GAMBLE} and deal <#UPGRADE>{1}</> damage to a random opponent argument for every {DOMINANCE} spent.",
        damage_amt = 3,
    },

    snag =
    {
        name = "Snag",
        desc = "Discard a card.",
        flavour = "'Not your lucky day.'",

        cost = 0,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.STICKY,
        rarity = CARD_RARITY.COMMON,
        discards = 1,

        OnPostResolve = function( self, minigame, targets )
            if self.draws then
                minigame:DrawCards( self.draws )
            end
            minigame:DiscardCards( self.min_discards or self.discards, self.discards, self )
        end,
    },

    snag_plus =
    {
        name = "Boosted Snag",
        desc = "Discard up to <#UPGRADE>2</> cards.",

        discards = 2,
        min_discards = 0,
    },

    snag_plus2 = 
    {
        name = "Assertive Snag",
        desc = "<#UPGRADE>Draw a card,</> discard a card.",
        draws = 1,
    },

    roughneck = 
    {
        name = "Roughneck",
        desc = "{GAMBLE}.\n{HEADS}: Remove {COMPOSURE}.\n{SNAILS}: Steal {COMPOSURE}.",
        flavour = "'He just walked in here like he owned the place. Put everyone on edge.'",

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,

        desired_result = "SNAILS",

        target_enemy = TARGET_ANY_RESOLVE,

        event_handlers =
        {
            [ EVENT.PRE_RESOLVE ] = function( self, minigame, card, targets )
                if card == self then
                    local coin = self.negotiator:FindModifier("LUCKY_COIN")
                    local result = coin and coin:Gamble(self)
                    for i, target in ipairs( targets ) do
                        local composure = target:GetComposure()
                        target:DeltaComposure( -composure, self )
                        if result == self.desired_result then
                            self.negotiator:DeltaComposure( composure, self )
                        end
                    end
                end
            end
        }
    },

    roughneck_plus =
    {
        name = "Twisted Roughneck",
        desc = "{GAMBLE}.\n{HEADS}: Steal {COMPOSURE}.\n{SNAILS}: Remove {COMPOSURE}.",
        desired_result = "HEADS",
        min_persuasion = 1,
        max_persuasion = 3,
    },

    roughneck_plus2 =
    {
        name = "Boosted Roughneck",
        min_persuasion = 1,
        max_persuasion = 3,
    },

    fallout =
    {
        name = "Fallout",
        desc = "{fallout 2|}Create: Whenever you {GAMBLE}, deal 2 damage to a random opponent argument.",
        flavour = "'This is getting real messy real fast.'",

        cost = 2,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:CreateModifier("fallout", 2, self)
        end,

        modifier = 
        {
            name = "Fallout",
            desc = "Whenever you {GAMBLE}, deal <#HILITE>{1}</> damage to a random opponent argument.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.min_persuasion or 2)
            end,

            max_resolve = 4,

            min_persuasion = 2,
            max_persuasion = 2,

            no_damage_tt = true,
            
            OnSetStacks = function( self, old_stacks )
                self.min_persuasion = self.stacks
                self.max_persuasion = self.stacks
            end,

            event_handlers = 
            {
                [ EVENT.GAMBLE ] = function( self, result, source )
                    self.target_enemy = TARGET_ANY_RESOLVE
                    self:ApplyPersuasion()
                    self.target_enemy = nil
                end
            },
        },
    },

    fallout_plus =
    {
        name = "Pale Fallout",
        cost = 1,
    },

    fallout_plus2 =
    {
        name = "Initial Fallout",
        desc = "{fallout 2|}Create: Whenever you {GAMBLE}, deal 2 damage to a random opponent argument.",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    drawback =
    {
        name = "Drawback",
        desc = "Draw {1} cards. All Hostility cards drawn cost 1 less until played.",
        flavour = "'Two cards, please.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.card_draw)
        end,

        cost = 1,
        mod_xp = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        card_draw = 2,
        discount = 1,

        OnPostResolve = function( self, minigame, targets )
            local drawn_cards = minigame:DrawCards(self.card_draw)
            local mod = self.negotiator:FindModifier("drawback") or self.negotiator:AddModifier("drawback", 1, self)
            mod.discount = self.discount
            mod.cards = mod.cards or {}
            for i,card in ipairs(drawn_cards) do
                if CheckBits(card.flags, CARD_FLAGS.HOSTILE) then
                    table.insert(mod.cards, card)
                end
            end
        end,

        modifier = 
        {
            hidden = true,

            discount = 1,

            event_handlers = 
            {
                [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                    if self.cards and table.contains(self.cards, card) then
                        local count = table.count_if(self.cards, function( v ) return v == card end)
                        acc:AddValue(-1 * count, self)
                    end
                end,

                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    while self.cards and table.contains(self.cards, card) do
                        table.arrayremove(self.cards, card)
                    end
                    if self.cards and #self.cards == 0 then
                        self.negotiator:RemoveModifier( self )
                    end
                end
            },
        },
    },

    drawback_plus =
    {
        name = "Visionary Drawback",
        desc = "Draw <#UPGRADE>{1}</> cards. All Hostility cards drawn cost 1 less until played.",
        card_draw = 3,
    },

    drawback_plus2 =
    {
        name = "Pale Drawback",
        desc = "Draw {1} cards. All Hostility cards drawn cost <#UPGRADE>2</> less until played.",
        discount = 2,
    },

    blacklist = 
    {
        name = "Blacklist",
        desc = "{TAKE_NAMES}.\nGain 1 damage for each name taken ({1} {1*name|names}).",
        flavour = "'How do you spell that?'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.userdata.names_taken and #self.userdata.names_taken or 0)
        end,

        blacklist_name_string = "\n{1}. {2}",
        flavour = "Names Taken: {1}",
        flavour_fn = function( self, fmt_str )
            if self == nil then
                return loc.format(fmt_str, "")
            else
                if self.userdata.names_taken then
                    local names = {}
                    local count = 1
                    for i,agent in ipairs(self.userdata.names_taken) do
                        table.insert(names, loc.format(self.blacklist_name_string, count, agent:GetName()))
                        count = count + 1
                    end
                    return loc.format(fmt_str, table.concat(names)) 
                end
            end
        end,

        cost = 2,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_persuasion = 2,
        max_persuasion = 2,

        PreReq = function( self, minigame )
            local agent = minigame:GetOpponentNegotiator().agent
            return self.userdata.names_taken == nil or not table.contains(self.userdata.names_taken, agent )
        end,

        OnPostResolve = function( self, minigame, targets )
            if self.userdata.names_taken == nil then
                self.userdata.names_taken = {}
            end
            local agent = minigame:GetOpponentNegotiator().agent
            if not table.contains(self.userdata.names_taken, agent ) then
                table.insert(self.userdata.names_taken, agent)
                self.userdata.min_persuasion_bonus = (self.userdata.min_persuasion_bonus or 0) + 1
                self.userdata.max_persuasion_bonus = (self.userdata.max_persuasion_bonus or 0) + 1
            end
        end,

        -- ccc: this won't preview outside of Negotiation.  I think it's ok if the card changes its base damage rather than applying as a modifier.
        -- event_handlers = 
        -- {
        --     [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
        --         if source == self and self.userdata.names_taken then
        --             persuasion:AddPersuasion( #self.userdata.names_taken, #self.userdata.names_taken, self )
        --         end
        --     end,
        -- },
    },
    

    -- Special cards for showing choices to the player
    coin_heads = 
    {
        name = "Heads",
        desc = "{HEADS}.",
        cost = 0,
        side = "HEADS",
        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNIQUE,
        hide_in_cardex = true,
    },

    coin_snails =
    {
        name = "Snails",
        desc = "{SNAILS}.",
        cost = 0,
        side = "SNAILS",
        flags = CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNIQUE,
        hide_in_cardex = true,
    },
}

for i, id, carddef in sorted_pairs( CARDS ) do
    carddef.series = "ROOK"

    Content.AddNegotiationCard( id, carddef )
end

local MODIFIERS = 
{
	NERVOUS =
    {
        name = "Nervous",
        desc = "Whenever this argument loses resolve, deal {1} damage to all targets.",
        desc_fn = function(self, fmt_str )
            return loc.format(fmt_str, self.damage * self.stacks)
        end,

        max_resolve = 10,

        damage = 2,
        target_mod = TARGET_MOD.TEAM,
        target_enemy = TARGET_ANY_RESOLVE,

        event_handlers =
        {
            [ EVENT.DELTA_RESOLVE ] = function( self, modifier, resolve, max_resolve, delta, source, params )
                if modifier == self and delta < 0 and resolve > 0 then
                    local damage = self.damage * self.stacks
                    self:ApplyPersuasion( damage, damage )
                end
            end,
        },
    },

    MOMENTUM =
    {
        name = "Momentum",
        desc = "Whenever you discard a card, draw a card.\nGains 2 resolve per stack.",
        max_resolve = 2,

        OnSetStacks = function( self, old_stacks )
            self.max_resolve = self.stacks * 2
            self:RestoreResolve( (self.stacks - old_stacks) * 2, self)
        end,

        event_handlers =
        {
            [ EVENT.CARD_DISCARDED ] = function( self, card, minigame )
                if card.negotiator == self.negotiator then
                    minigame:DrawCards(1)
                end
            end,
        },
    },

    RIG_HEADS = 
    {
        name = "Rigged: Heads",
        desc = "Guarantees that your next <#HILITE>{1}</> {1*gamble|gambles} {1*is|are} Heads.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.stacks or 1)
        end,

        OnApply = function( self )
            local mod = self.negotiator:FindModifier("RIG_SNAILS")
            while mod do
                self.negotiator:RemoveModifier( mod )
                mod = self.negotiator:FindModifier("RIG_SNAILS")
            end
        end,

        modifier_type = MODIFIER_TYPE.PERMANENT,

        event_handlers = 
        {
            [ EVENT.PRE_GAMBLE ] = function( self, source, prepared_flips )
                table.insert(prepared_flips, "HEADS")
                self.negotiator:RemoveModifier("RIG_HEADS", 1)
            end,
        },
    },

    RIG_SNAILS = 
    {
        name = "Rigged: Snails",
        desc = "Guarantees that your next <#HILITE>{1}</> {1*gamble|gambles} {1*is|are} Snails.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.stacks or 1)
        end,

        OnApply = function( self )
            local mod = self.negotiator:FindModifier("RIG_HEADS")
            while mod do
                self.negotiator:RemoveModifier( mod )
                mod = self.negotiator:FindModifier("RIG_HEADS")
            end
        end,

        modifier_type = MODIFIER_TYPE.PERMANENT,

        event_handlers = 
        {
            [ EVENT.PRE_GAMBLE ] = function( self, source, prepared_flips )
                table.insert(prepared_flips, "SNAILS")
                self.negotiator:RemoveModifier("RIG_SNAILS", 1)
            end,
        },
    },

    LUCKY_COIN = 
	{
		name = "Lucky Coin",
		desc = "Whenever this coin is flipped:\n{1}",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.current_desc )
        end,

        modifier_type = MODIFIER_TYPE.CORE,
        widget_class = Widget.NegotiationCoinArgument,

        current_desc = "",
        heads_effect = function( self, minigame )
        end,
        snails_effect = function( self, minigame )
        end,
        streak = 1,
        ping_delay = 0.7,

        heads_icon = "negotiation/modifiers/HEADS.tex",
        snails_icon = "negotiation/modifiers/SNAILS.tex",

		OnApply = function( self )
			self.heads_chance = 50
            self.prepared_flips = {}
		end,

		Gamble = function( self, source )
			local result
            local roll
            self.engine:BroadcastEvent( EVENT.PRE_GAMBLE, source, self.prepared_flips)
            if self.prepared_flips then
                roll = table.remove(self.prepared_flips, 1)
            end
            roll = roll or math.random(100) <= self.heads_chance and "HEADS" or "SNAILS"

            if roll == self.current_status then
                self.streak = self.streak + 1
            else
                self.streak = 1
            end

            if roll == "HEADS" then
				self.current_status = "HEADS"
                result = "HEADS"
				self:UpdateConditions(true)
			else
				self.current_status = "SNAILS"
                result = "SNAILS"
				self:UpdateConditions(false)
			end
			self.engine:BroadcastEvent( EVENT.GAMBLE, result, source)
			return result
		end,
        Reverse = function( self, source )
            if self.current_status == "HEADS" then
                self.current_status = "SNAILS"
                self:UpdateConditions(false)
            else
                self.current_status = "HEADS"
                self:UpdateConditions(true)
            end
        end,
        SetCoin = function( self, side, source )
            if side == nil then
                local cards = {
                    Negotiation.Card( "coin_heads", self.owner ),
                    Negotiation.Card( "coin_snails", self.owner ),
                }
                local pick = self.engine:ImproviseCards( cards, 1, nil, nil, nil, self )[1]
                side = pick and pick.side or "HEADS"
                if pick then self.engine:ExpendCard(pick) end
            end
            if side == "HEADS" then
                self.current_status = "HEADS"
                self:UpdateConditions(true)
            else
                self.current_status = "SNAILS"
                self:UpdateConditions(false)
            end
            return side
        end,

		GetCurrentResult = function( self )
			return self.current_status
		end,
		UpdateConditions = function( self, got_heads )
            self.engine:BroadcastEvent( EVENT.UPDATE_COIN, self, got_heads)
		end,

        SwapCoins = function( self, coin_modifier )
            assert( is_instance( coin_modifier, Negotiation.Modifier ))

            self.heads_effect = coin_modifier.heads_effect
            self.snails_effect = coin_modifier.snails_effect
            self.current_desc = coin_modifier:GetDesc()
            self.custom_name = coin_modifier:GetName()
            self.heads_icon = coin_modifier.heads_icon
            self.snails_icon = coin_modifier.snails_icon
            self.coin_build = coin_modifier.coin_build
            self.heads_chance = coin_modifier.heads_chance or 50
            self.min_persuasion = coin_modifier.min_persuasion
            self.max_persuasion = coin_modifier.max_persuasion 
            self.coin_swapped = true
            self.graft = coin_modifier.graft
            -- if math.random(100) <= self.heads_chance then
            --     self.current_status = "HEADS"
            --     self:UpdateConditions(true)
            -- else
            --     self.current_status = "SNAILS"
            --     self:UpdateConditions(false)
            -- end
        end,

        event_handlers = 
        {
            [ EVENT.UPDATE_COIN ] = function( self, modifier, got_heads )
                if got_heads then
                    self:heads_effect(self.engine)
                else
                    self:snails_effect(self.engine)
                end
            end,

            [ EVENT.HAND_DRAWN ] = function( self, minigame )
                if self.coin_swapped then
                    if math.random(100) <= self.heads_chance then
                        self.current_status = "HEADS"
                        self:UpdateConditions(true)
                    else
                        self.current_status = "SNAILS"
                        self:UpdateConditions(false)
                    end
                    self.coin_swapped = false
                end
            end
        },
	},

    DIVERSION = 
    {
        name = "Diversion",
        desc = "This argument gains <#HILITE>{1}</> resolve every time you {GAMBLE}, up to 12. This argument must be targeted before any other arguments.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.stacks and self.stacks or 1)
        end,
        force_target = true,

        max_resolve = 2,

        CanPlayCard = function( self, source, engine, target )
            if source:IsAttack() and target:GetNegotiator() == self.negotiator then
                if source.modifier_type == MODIFIER_TYPE.INCEPTION or source:GetNegotiator() ~= self.negotiator then
                    if not target.force_target then
                        return false, loc.format( LOC "CONVO_COMMON.MUST_TARGET", self:GetName() )
                    end
                end
            end

            return true
        end,

        event_handlers =
        {
            [ EVENT.GAMBLE ] = function( self, result, source )
                if self.max_resolve < 12 then
                    self:ModifyResolve(math.min(self.stacks, 12- self.max_resolve), self)
                else
                    self:RestoreResolve(self.stacks, self)
                end
            end
        },
    },

    VANISH =
    {
        name = "Vanish",
        desc = "Every time you play a card, {GAMBLE} and reduce this by 1.",

        max_resolve = 4,

        OnApply = function( self )
            self.coin = self.negotiator:FindModifier("LUCKY_COIN")
        end,

        event_handlers =
        {
            [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                if card.negotiator == self.negotiator and self.coin then
                    self.coin:Gamble(self)
                    self.negotiator:RemoveModifier(self, 1)
                end
            end
        },
    },
}

for id, def in pairs( MODIFIERS ) do
    Content.AddNegotiationModifier( id, def )
end

local FEATURES =
{
    RIG = {
        name = "Rig",
        desc = "Guarantees the result of your next gamble. Only one outcome can be rigged at a time.",
    },

    SET = {
        name = "Set the coin",
        desc = "Choose a side of the coin and flip it to that side."
    },

    GAMBLE = {
        name = "Gamble",
        desc = "Flip your {1}.",
        desc_fn = function( self, fmt_str, stacks, engine )
            local coin = engine and engine:GetPlayerNegotiator():FindModifier( "LUCKY_COIN" )
            if coin then
                local txt = loc.format( fmt_str, coin:GetName() )
                if coin.current_desc then
                    txt = txt .. "\n" .. coin.current_desc
                end
                return txt
            else
                return loc.format( fmt_str, LOC "UI.CARDS.DEFAULT_COIN" )
            end
        end,
    },

    TAKE_NAMES =
    {
        name = "Take Names",
        desc = "Playing this card against a new unique character adds their name to the list.",
    },
}

for id, data in pairs(FEATURES) do
    local def = NegotiationFeatureDef(id, data)
    Content.AddNegotiationCardFeature(id, def)
end

