local negotiation_defs = require "negotiation/negotiation_defs"
local CARD_FLAGS = negotiation_defs.CARD_FLAGS
local EVENT = negotiation_defs.EVENT

local CARDS =
{
    fast_talk = 
    {
        name = "Fast Talk",
        flavour = "'You're just looking at it wrong. Here, let me step you through it...'",
        cost = 1,
        min_persuasion = 1,
        max_persuasion = 3,
        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.BASIC,
        max_xp = 6,
        wild = true,
    },
    

    fast_talk_plus2a =
    {
        name = "Clarity Fast Talk",
        min_persuasion = 6,
        max_persuasion = 6,
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.CONSUME,
    },

    fast_talk_plus2b =
    {
        name = "Pale Fast Talk",
        desc = "<#UPGRADE>Spend {INFLUENCE {1}}: This card costs 0 to play.</>",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_cost )
        end,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= 1
        end,

        influence_cost = 1,

        min_persuasion = 2,
        max_persuasion = 3,

        deck_handlers = ALL_DECKS,
        event_priorities =
        {
            [ EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers = 
        {
            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card == self and self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost then
                    acc:ModifyValue( 0, self )
                end
            end,
        },

        OnPostResolve = function( self, minigame, targets )
            if self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost then
                self.negotiator:RemoveModifier("INFLUENCE", self.influence_cost, self)
            end
        end
    },

    fast_talk_plus2c = 
    {
        name = "Visionary Fast Talk",
        desc = "<#UPGRADE>Draw a card.</>",
        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(1)
        end,
    },

    -- fast_talk_plus2d =
    -- {
    --     name = "Thorny Fast Talk",
    --     desc = "<#UPGRADE>Spend 2 {INFLUENCE}, deal 2 bonus damage.</>",
    --     event_handlers =
    --     {
    --         [ EVENT.CALC_PERSUASION ] = function( self, card, persuasion )
    --             if card == self and self.negotiator:GetModifierStacks("INFLUENCE") >= 2 then
    --                 persuasion:AddPersuasion( 2, 2, self )
    --             end
    --         end,
    --     },

    --     OnPostResolve = function( self, minigame, targets )
    --         if self.negotiator:GetModifierStacks("INFLUENCE") >= 2 then
    --             self.negotiator:RemoveModifier("INFLUENCE", 2)
    --         end
    --     end
    -- },

    fast_talk_plus2f =
    {
        name = "Inspirational Talk",
        desc = "<#UPGRADE>Gain {INFLUENCE 1}.</>",
        flags = CARD_FLAGS.DIPLOMACY,
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("INFLUENCE", 1, self)
        end
    },

    -- fast_talk_plus2g = 
    -- {
    --     name = "Rooted Fast Talk",
    --     min_persuasion = 4,
    --     max_persuasion = 4
    -- },

    fast_talk_plus2h = 
    {
        name = "Stone Fast Talk",
        desc = "<#UPGRADE>Gain {1} {COMPOSURE}.</>",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText( self.composure ))
        end,

        composure = 3,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:DeltaComposure( self.composure, self )
        end,
    },

    deflection = 
    {
        name = "Deflection",
        desc = "Apply {1} {COMPOSURE}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateComposureText( self.composure_amt ))
        end,

        flavour = "'I'm not even going to address that.'",

        max_xp = 6,
        wild = true,

        cost = 1,
        composure_amt = 3,
        
        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.BASIC,

        target_self = TARGET_ANY_RESOLVE,
        OnPostResolve = function ( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
        end
    },

    deflection_plus2b =
    {
        name = "Boosted Deflection",
        desc = "Apply <#UPGRADE>{1} {COMPOSURE}</>.",
        composure_amt = 4,
    },

    deflection_plus2c = 
    {
        name = "Visionary Deflection",
        desc = "Apply {1} {COMPOSURE}\n<#UPGRADE>Draw a card.</>",
        OnPostResolve = function( self, minigame, targets )
        	for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
            minigame:DrawCards(1)
        end
    },

    deflection_plus2d =
    {
    	name = "Wide Deflection",
    	desc = "<#UPGRADE>Apply {1} {COMPOSURE} to all friendly arguments.</>",

    	flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        
        target_self = TARGET_ANY_RESOLVE,
        target_mod = TARGET_MOD.TEAM,       
        auto_target = true,

        composure_amt = 4,       
    },

    deflection_plus2e = 
    {
    	name = "Deflection of Clarity",
    	desc = "<#UPGRADE>Restore 5 resolve and apply {1} {COMPOSURE}.</>",

    	flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.CONSUME,

    	composure_amt = 5,

    	OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
            for i, target in ipairs(targets) do
        		target:RestoreResolve(5, self)
            end
    	end
    },

    deflection_plus2f = 
    {
    	name = "Lucid Deflection",
    	desc = "Apply <#UPGRADE>{1} {COMPOSURE}</>.",

    	flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

    	composure_amt = 6,
    },

    threaten =
    {
        name = "Threaten",
        flavour = "'I don't want to have to hurt you, but I will if I have to.'",
        cost = 1,
        min_persuasion = 1,
        max_persuasion = 4,

        max_xp = 6,
        wild = true,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.BASIC,
    },

    threaten_plus2a =
    {
        name = "Threaten of Clarity",
        min_persuasion = 8,
        max_persuasion = 8,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.CONSUME,
    },

    -- threaten_plus2b =
    -- {
    --     name = "Pale Threaten",
    --     cost = 0,
    -- },

    threaten_plus2c = 
    {
        name = "Mirrored Threaten",
        desc = "<#UPGRADE>Attack with this card twice.</>",

        max_persuasion = 2,

        OnPostResolve = function( self, minigame )
            minigame:ApplyPersuasion( self )
        end,
    },

    threaten_plus2d = 
    {
        name = "Visionary Threaten",
        desc = "<#UPGRADE>Draw a card.</>",
        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(1)
        end,
    },

    threaten_plus2e = 
    {
        name = "Boosted Threaten",
        min_persuasion = 3,
        max_persuasion = 4,
    },

    threaten_plus2f = 
    {
        name = "Lucid Threaten",
        desc = "<#UPGRADE>Gain 1 {DOMINANCE}.</>",
        flags = CARD_FLAGS.HOSTILE,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("DOMINANCE", 1, self)
        end
    },


    elucidate =
    {
        name = "Elucidate",
        flavour = "'It’s all going to be cleared up, don’t you worry.'",

        loc_strings =
        {
            NO_DUPLICATE_CORE = "Cannot duplicate core argument",
        },

        desc = "Duplicate a random friendly argument.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.bonus )
        end,

        cost = 1,
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        target_self = TARGET_FLAG.ARGUMENT,
        auto_target = true,

        CanPlayCard = function ( self, card, engine, target )
            if is_instance( target, Negotiation.Modifier ) and target.modifier_type == MODIFIER_TYPE.CORE then
                return false, self.def:GetLocalizedString( "NO_DUPLICATE_CORE" )
            end
            return true
        end,

        OnPostResolve = function( self, minigame, targets )
            for i, target in ipairs( targets ) do
                target:Duplicate()
            end
        end,
    },

    elucidate_plus =
    {
        name = "Focused Elucidate",
        desc = "Duplicate a <#UPGRADE>chosen</> friendly argument.",

        auto_target = false,
    },

    elucidate_plus2 =
    {
        name = "Enduring Elucidate",
        flags = CARD_FLAGS.DIPLOMACY,
    },

    keep_cool =
    {
        name = "Keep Cool",
        flavour = "'Nothing fazes them, it’s unreal.'",
        desc = "Double {COMPOSURE} on all arguments.",

        mod_xp = -2,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        OnPostResolve = function( self, minigame )
            for i, modifier in self.negotiator:ModifierSlots() do
                if modifier:GetComposure() > 0 then
                    modifier:DeltaComposure(modifier:GetComposure())
                end
            end
        end,
    },

    keep_cool_plus =
    {
        name = "Keep Composed",
        desc = "<#UPGRADE>Apply {1} {COMPOSURE}.</>\nDouble {COMPOSURE} on all arguments.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText( self.composure ))
        end,

        composure = 3,
        target_self = TARGET_ANY_RESOLVE,

        OnPostResolve = function( self, minigame, targets )
            for i, target in ipairs( targets ) do
                target:DeltaComposure( self.composure, self )
            end
            for i, modifier in self.negotiator:ModifierSlots() do
                if modifier:GetComposure() > 0 then
                    modifier:DeltaComposure(modifier:GetComposure())
                end
            end
        end,
    },

    keep_cool_plus2 =
    {
        name = "Sticky Keep Cool",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.STICKY,
    },

    beguile =
    {
        name = "Beguile",
        flavour = "'Well if it isn’t everybody’s favorite bounty hunter.'",
        desc = "Gains {1} damage each time this is drawn while you have {INFLUENCE}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.bonus )
        end,

        cost = 2,
        min_persuasion = 4,
        max_persuasion = 4,
        bonus = 3,
        total_bonus = 0,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        deck_handlers = ALL_DECKS,

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self then
                    persuasion:ModifyPersuasion( persuasion.min_persuasion + self.total_bonus, persuasion.max_persuasion + self.total_bonus, self )
                end
            end,

           [ EVENT.DRAW_CARD ] = function( self, minigame, card, start_of_turn )
                 if card == self then
                    if self.negotiator:HasModifier("INFLUENCE") then
                        self.total_bonus = self.total_bonus + self.bonus
                        --self.min_persuasion = self.min_persuasion + self.bonus
                        --self.max_persuasion = self.max_persuasion + self.bonus
                    end
                end
            end,
        }
    },

    beguile_plus =
    {
        name = "Boosted Beguile",
        desc = "Gains <#UPGRADE>{1}</> damage each time this is drawn while you have {INFLUENCE}.",
        bonus = 4,
    },

    beguile_plus2 =
    {
        name = "Tall Beguile",
        min_persuasion = 6,
        max_persuasion = 6,
    },

    evil_eye = 
    {
        name = "Evil Eye",
        desc = "The damage of the next card played this turn is doubled.",
        flavour = "'I wouldn’t exactly call that F’ssh’s good eye.'",
        
        cost = 1,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,
        
        sound = "event:/sfx/battle/cards/neg/damage_increased",

        modifier =
        {
            hidden = true,

            event_priorities =
            {
                [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_MULTIPLIER,
            },

            event_handlers =
            {
                [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                    if source.owner == self.owner and is_instance( source, Negotiation.Card ) then
                        if persuasion.preview or self.engine:IsResolving( source ) then
                            local min = persuasion.min_persuasion
                            local max = persuasion.max_persuasion
                            for i=1,self.stacks do
                                min = min * 2
                                max = max * 2
                            end
                            persuasion:ModifyPersuasion( min, max, self )
                        end
                    end
                end,

                [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                    if source.owner == self.owner and self.engine:IsResolving( source ) then
                        self.used = true
                    end
                end,

                [ EVENT.POST_RESOLVE ] = function(self, minigame, card)
                    if card.owner == self.owner and self.used then
                        self.used = false
                        self.negotiator:RemoveModifier( self )
                    end
                end,

                [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        self.negotiator:RemoveModifier( self )
                    end
                end,
            }
        },

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "evil_eye", 1, self )
        end,
    },
    evil_eye_plus =
    {
        name = "Pale Evil Eye",
        cost = 0,
    },

    evil_eye_plus2 = 
    {
        name = "Visionary Evil Eye",
        desc = "<#UPGRADE>Draw a card.</>\nThe damage of the next card played this turn is doubled.",
        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "evil_eye", 1, self )
            minigame:DrawCards(1)
        end,
    },

    invective =
    {
        name = "Invective",
        flavour = "'This argument is heating up!'",
        desc = "Gain {HEATED 1}\n{EVOKE}: Play {1} Hostility cards in a single turn. {2}",

        desc_fn = function( self, fmt_str )
            local def = self.def or self -- This is either a card instance or a def itself. :(
            if self.evoke_count ~= nil then
                local str = loc.format( LOC"CARD_ENGINE.CARDS_PLAYED", self.evoke_count )
                return loc.format( fmt_str, self.evoke_max, str )
            else
                local fmt_str = def:GetLocalizedDesc()
                return loc.format( fmt_str, self.evoke_max, "" )
            end
        end,

        cost = 0,

        evoke_max = 3,
        no_invective = true,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("HEATED", 1, self)
        end,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS },

        event_handlers = 
        {
            [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                if card.owner == self.owner and not card.no_invective then
                    if CheckBits( card.flags, CARD_FLAGS.HOSTILE ) then
                        self:Evoke( self.evoke_max )
                    end
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ResetEvoke()
                end
            end,
        },
    },

    invective_plus = 
    {
        name = "Boosted Invective",
        desc = "Gain <#UPGRADE>{HEATED 2}</>\n{EVOKE}: Play {1} Hostility cards in a single turn. {2}",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("HEATED", 2, self)
        end,
    },

    invective_plus2 =
    {
        name = "Enduring Invective",
        desc = "Gain {HEATED 1}\n{EVOKE}: Play {1} Hostility cards in a single turn. {2}",
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.UNPLAYABLE,
    },

    caprice =
    {
        name = "Caprice",
        flavour = "'One second everything was cool, and the next, he was screaming!'",
        desc = "{IMPROVISE} {1} random Hostility {1*card|cards}. {1*It costs|They cost} 1 less until played.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.draw_count )
        end,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        cost = 1,
        draw_count = 1,
        pool_size = 3,
        upgraded = false,

        CapriceFilter = function( card_def, self )
            if "caprice" == card_def.id or "caprice" == card_def.base_id then
                return false -- Can't re-spawn Caprice
            end
            if card_def.rarity == CARD_RARITY.BOSS or card_def.rarity == CARD_RARITY.UNIQUE then
                return false
            end
            
            return CheckBits( card_def.flags, CARD_FLAGS.HOSTILE ) and 
                    CheckBits( card_def.flags, CARD_FLAGS.UPGRADED ) == self.upgraded
        end,

        OnPostResolve = function( self, minigame )
            local collection = NegotiationCardCollection.AllUsableCards()
            collection:Filter( self.CapriceFilter, self )

            local card_defs = collection:Pick(self.pool_size)
            local cards = table.map(card_defs, 
                    function(card_def) 
                        local card = Negotiation.Card(card_def.id, self:GetOwner()) 
                        card:ClearXP()
                        return card
                    end)
            local chosen_cards = minigame:ImproviseCards( cards, self.draw_count, nil, nil, nil, self )
            local mod = self.negotiator:AddModifier("caprice", 1, self)
            mod.affected_cards = mod.affected_cards or {}
            for i,v in ipairs(chosen_cards) do
                table.insert(mod.affected_cards, v)
            end
        end,

        modifier = 
        {
            hidden = true,

            event_handlers =
            {
                [ EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                    if self.affected_cards and table.contains(self.affected_cards, card) then
                        cost_acc:AddValue( -1, self )
                    end
                end,

                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    if self.affected_cards and table.contains(self.affected_cards, card) then
                        table.arrayremove(self.affected_cards, card)
                        if #self.affected_cards == 0 then
                            self.negotiator:RemoveModifier(self.id, self.stacks, self)
                        end
                    end
                end,
            },
        },
    },
    caprice_plus =
    {
        name = "Twisted Caprice",
        desc = "<#UPGRADE>{IMPROVISE_PLUS} {1} random Hostility {1*card|cards}</>. {1*It costs|They cost} 1 less until played.",
        cost = 2,
        pool_size = 5,
        draw_count = 2,
    },

    caprice_plus2 =
    {
        name = "Boosted Caprice",
        desc = "{IMPROVISE} {1} random <#UPGRADE>upgraded</> Hostility {1*card|cards}. {1*It costs|They cost} 1 less until played.",
        upgraded = true,
    },

    clean_slate = 
    {
        name = "Clean Slate",
        flavour = "'Okay, we got off on the wrong foot here.'",
        desc = "Remove all arguments in play.",
        
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        cost = 2,

        RemoveArguments = function( self, negotiator )
            local slots = shallowcopy( negotiator.modifier_slots )
            for i, modifier in ipairs( slots ) do
                -- Must ensure this slot is still present AS we remove them
                if modifier.modifier_type == MODIFIER_TYPE.ARGUMENT or modifier.modifier_type == MODIFIER_TYPE.BOUNTY and
                    table.arrayfind( negotiator.modifier_slots, modifier ) ~= nil then
                    --
                    negotiator:RemoveModifier( modifier, modifier.stacks, self )
                end
            end
        end,

        OnPostResolve = function( self, minigame)
            local modifiers = {}
            self:RemoveArguments( self.negotiator )
            self:RemoveArguments( self.anti_negotiator )
            if self.clear_intents then
                self.anti_negotiator:ClearIntents()
            end

            minigame:BroadcastEvent( EVENT.CUSTOM, function( screen, ... )
                AUDIO:PlayEvent( "event:/sfx/battle/cards/neg/destroy_all_arguments" )
            end )
        end,
    },
    clean_slate_plus =
    {
        name = "Boosted Slate",
        desc = "Remove all arguments in play.\n<#UPGRADE>Dismiss all opponent intents.</>",
        clear_intents = true,
    },

    clean_slate_plus2 =
    {
        name = "Pale Slate",
        cost = 1,
    },

    reconsider =
    {
        name = "Reconsider",
        flavour = "'Well, when you put it that way...'",
        desc = "Remove one of your arguments, inceptions, or bounties.\nGain {1} actions.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.action_count)
        end,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,
        action_count = 2,

        target_self = TARGET_FLAG.ARGUMENT | TARGET_FLAG.INCEPTION | TARGET_FLAG.BOUNTY,

        Reconsider = function( self, minigame, targets )
            for i, target in ipairs( targets ) do
                target:GetNegotiator():RemoveModifier( target )
            end
            minigame:ModifyActionCount( self.action_count, self )
        end,

        OnPostResolve = function( self, minigame, targets )
            self:Reconsider(minigame, targets)
        end,
    },

    reconsider_plus =
    {
        name = "Pale Reconsider",
        cost = 0,
    }, 

    reconsider_plus2 =
    {
        name = "Visionary Reconsider",
        desc = "Remove one of your arguments, inceptions, or bounties.\nGain {1} actions.\n<#UPGRADE>Draw a Card.</>",

        OnPostResolve = function( self, minigame, targets)
            self:Reconsider(minigame, targets)
            minigame:DrawCards( 1 )
        end,
    },

    back_pedal = 
    {
        name = "Back Pedal",
        flavour = "'Look, you guys got me all wrong!'",
        desc = "{EXPEND} {1} chosen {1*card|cards} in your hand.\nApply {2} {COMPOSURE}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.max_choose, self:CalculateComposureText( self.composure_gain ))
        end,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,
        target_self = TARGET_ANY_RESOLVE,

        cost = 1,
        composure_gain = 5,
        min_choose = 1,
        max_choose = 1,

        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:ExpendCards( self.min_choose, self.max_choose, self )

            for i, target in ipairs( targets ) do
                target:DeltaComposure( self.composure_gain, self )
            end
        end,
    },
    back_pedal_plus = 
    {
        name = "Boosted Pedal",
        desc = "{EXPEND} {1} chosen {1*card|cards} in your hand.\nApply <#UPGRADE>{2}</> {COMPOSURE}.",
        composure_gain = 8,
    },
    back_pedal_plus2 = 
    {
        name = "Lucid Pedal",
        desc = "{EXPEND} <#UPGRADE>up to {1}</> chosen {1*card|cards} in your hand.\nApply {2} {COMPOSURE}.",
        min_choose = 0,
        max_choose = 2,
    },

    setup =
    {
        name = "Setup",
        flavour = "'Okay, stop me if you’ve heard this one....'",
        desc = "Gain {INFLUENCE {1}}. At the end of your turn, lose {1} {INFLUENCE}.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.stacks)
        end,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        cost = 0,
        stacks = 2,

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "INFLUENCE", self.stacks, self )
            self.negotiator:AddModifier( "setup", self.stacks, self )
        end,

        modifier =
        {
            max_resolve = 2,

            sound = "event:/sfx/battle/cards/neg/create_argument/setup",

            desc = "At the end of your turn, lose {1} {INFLUENCE}, then remove this argument.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks or 1)
            end,

            OnEndTurn = function(self, minigame)
                local stacks = self.negotiator:GetModifierStacks("INFLUENCE")
                self.negotiator:RemoveModifier("INFLUENCE", self.stacks, self)
                self.negotiator:RemoveModifier( self )
            end,
        }
    },
    setup_plus =
    {
        name = "Boosted Setup",
        desc = "Gain <#UPGRADE>{INFLUENCE {1}}</>. At the end of your turn, lose <#DOWNGRADE>{1} {INFLUENCE}</>.",
        stacks = 3,
    },
    setup_plus2 =
    {
        name = "Pale Setup",
        desc = "Gain {INFLUENCE {1}}. At the end of your turn, lose <#UPGRADE>{2} {INFLUENCE}</>.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.stacks, self.setup_stacks)
        end,
        setup_stacks = 1,
        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "INFLUENCE", self.stacks, self )
            self.negotiator:AddModifier( "setup", self.setup_stacks, self )
        end,
    },

    plead =
    {
        name = "Plead",
        flavour = "'I’m begging here.'",
        desc = "Lose {INFLUENCE 1}.",

        cost = 0,
        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,
        min_persuasion = 0,
        max_persuasion = 3,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:RemoveModifier( "INFLUENCE", 1, self )
        end
    },

    plead_plus =
    {
        name = "Simple Plead",
        desc = "",

        OnPostResolve = function( self, minigame, targets )
        end,
    },

    plead_plus2 =
    {
        name = "Tall Plead",
        min_persuasion = 0,
        max_persuasion = 5,
    },

    intrigue =
    {
        name = "Intrigue",
        flavour = "'Very interesting.'",
        desc = "Spend {INFLUENCE {1}}: Deal an additional {2} damage.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_cost, self.bonus_damage )
        end,

        cost = 1,

        influence_cost = 1,
        bonus_damage = 3,

        min_persuasion = 2,
        max_persuasion = 3,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        spend = true,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost
        end,

        OnPostResolve = function( self, minigame, targets )
            if self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost then
                minigame:ApplyPersuasion( self, nil, self.bonus_damage, self.bonus_damage )
                if self.spend then
                    self.negotiator:RemoveModifier("INFLUENCE", self.influence_cost, self)
                end
            end
        end
    },

    intrigue_plus =
    {
        name = "Enhanced Intrigue",
        desc = "Spend {INFLUENCE {1}}: Deal an additional<#UPGRADE> {2} damage</>.",
        bonus_damage = 5,
    },

    intrigue_plus2 = 
    {
        name = "Sparing Intrigue",
        desc = "<#UPGRADE>If you have any {INFLUENCE}</>: Deal an additional {2} damage.",
        spend = false,
    },

    calling_in_all_favors = 
    {
        name = "Calling in all Favors",
        flavour = "'You know the Ollowano family? The same, they’ll be interested to hear about this.'",
        desc = "Double your {INFLUENCE}.\nGain {1} {VULNERABILITY}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.vulnerability_stacks )
        end,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        cost = 1,
        vulnerability_stacks = 3,

        OnPostResolve = function( self, minigame, targets )
            local stacks = self.negotiator:GetModifierStacks("INFLUENCE")
            self.negotiator:AddModifier("INFLUENCE", stacks, self)
            self.negotiator:AddModifier("VULNERABILITY", self.vulnerability_stacks, self)
        end,
    },

    calling_in_all_favors_plus =
    {
        name = "Tall Favors",
        desc = "<#UPGRADE>Gain 1 {INFLUENCE}, then</> Double your {INFLUENCE}.\nGain {1} {VULNERABILITY}.",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("INFLUENCE", 1, self)
            local stacks = self.negotiator:GetModifierStacks("INFLUENCE")
            self.negotiator:AddModifier("INFLUENCE", stacks, self)
            self.negotiator:AddModifier("VULNERABILITY", self.vulnerability_stacks, self)
        end,

    },

    calling_in_all_favors_plus2 =
    {
        name = "Short Favors",
        desc = "Double your {INFLUENCE}.\nGain <#UPGRADE>{1}</> {VULNERABILITY}.",
        vulnerability_stacks = 2,
    },

    magnetic_charm =
    {
        name = "Magnetic Charm",
        flavour = "'The room just lights right up.'",
        desc = "Draw {1} cards. Gain {INFLUENCE 1} for each Diplomacy card drawn.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.draw_count )
        end,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,
        draw_count = 2,
        influence_bonus = 0,

        OnPostResolve = function( self, minigame )
            local cards = minigame:DrawCards( self.draw_count )
            local influence = 0
            for i, card in ipairs( cards ) do
                if card:IsFlagged( CARD_FLAGS.DIPLOMACY ) then
                    influence = influence + 1
                end
            end
            self.negotiator:AddModifier( "INFLUENCE", influence + self.influence_bonus, self )
        end,
    },
    magnetic_charm_plus =
    {
        name = "Enhanced Charm",
        desc = "Draw <#UPGRADE>{1}</> cards. Gain {INFLUENCE 1} for each Diplomacy card drawn.",
        draw_count = 3,
    },
    magnetic_charm_plus2 =
    {
        name = "Boosted Charm",
        desc = "Draw {1} cards. Gain {INFLUENCE 1} for each Diplomacy card drawn <#UPGRADE>+ 1</>.",
        influence_bonus = 1,
    },

    aplomb =
    {
        name = "Aplomb",
        flavour = "'You aren’t changing my mind.'",
        desc = "Spend {INFLUENCE {1}}: Apply {2} {COMPOSURE}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_cost, self:CalculateComposureText( self.composure ))
        end,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,
        target_self = TARGET_ANY_RESOLVE,

        mod_xp = -2,

        cost = 1,
        composure = 8,
        influence_cost = 2,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost
        end,

        CanPlayCard = function ( self, card, engine, target )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost, loc.format( LOC"CONVO_COMMON.REQ_INFLUENCE", self.influence_cost)
        end,

        OnPostResolve = function( self, minigame, targets )
            if self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost then
                self.negotiator:RemoveModifier( "INFLUENCE", self.influence_cost )
                for i,target in ipairs(targets) do
                    target:DeltaComposure( self.composure, self )
                end
            end
        end,
    },

    aplomb_plus =
    {
        name = "Boosted Aplomb",
        desc = "Spend {INFLUENCE {1}}: Apply <#UPGRADE>{2} {COMPOSURE}.</>",
        composure = 12,
    },

    aplomb_plus2 =
    {
        name = "Pale Aplomb",
        desc = "Spend <#UPGRADE>{INFLUENCE {1}}</>: Apply {2} {COMPOSURE}.",
        influence_cost = 1,
    },

    tactical_mind =
    {
        name = "Tactical Mind",
        flavour = "'Smith squinted and thought long and hard. His dinner order was finally falling into place.'",

        desc = "{tactical_mind {1}|}Gain: Draw {1} extra {1*card|cards} at the start of your turn.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.draw_count )
        end,

        -- keywords = { "tactical_mind" },
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,
        draw_count = 1,

        cost = 1,
        mod_xp = 2,

        OnPostResolve = function( self )
            self.negotiator:AddModifier( "tactical_mind", self.draw_count, self )
        end,

        modifier =
        {
            desc = "Draw <#HILITE>{1}</> extra {1*card|cards} at the start of your turn.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks )
            end,
            max_resolve = 5,

            sound = "event:/sfx/battle/cards/neg/create_argument/tactical_mind",

            event_handlers =
            {
                [ EVENT.CALC_TURN_DRAW ] = function( self, acc )
                    acc:AddValue( self.stacks, self )
                end
            }
        },
    },

    tactical_mind_plus =
    {
        name = "Pale Mind",
        cost = 0,
    },

    tactical_mind_plus2 = 
    {
        name = "Boosted Mind",
        desc = "{tactical_mind {1}|}Gain: Draw <#UPGRADE>{1}</> extra cards at the start of your turn.",
        draw_count = 2,
    },

    good_impression =
    {
        name = "Good Impression",
        desc = "This card loses 1 damage each time you end your turn.",
        flavour = "'What do you mean? They show up on time and get their work done.'",

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        min_persuasion = 5,
        max_persuasion = 7,
        cost = 1,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS, DECK_TYPE.IN_HAND },

        event_handlers = 
        {
            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self.min_persuasion = math.max( 0, self.min_persuasion - 1 )
                    self.max_persuasion = math.max( 0, self.max_persuasion - 1 )
                end
            end,
        }
    },
    good_impression_plus =
    {
        name = "Boosted Impression",
        min_persuasion = 7,
        max_persuasion = 9,
    },
    good_impression_plus2 =
    {
        name = "Pale Impression",
        cost = 0,
    },

    collected = 
    {
        name = "Collected",
        flavour = "'The key to winning an argument, is never losing your cool.'",
        desc = "Apply {1} {COMPOSURE} to your {INFLUENCE}.\n{EVOKE}: Gain {INFLUENCE}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.composure )
        end,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.UNPLAYABLE,
        rarity = CARD_RARITY.RARE,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS },

        cost = 1,
        composure = 3,

        OnPostResolve = function( self, minigame )
            for i,v in self.negotiator:Modifiers() do
                if v.id == "INFLUENCE" then
                    v:DeltaComposure(self.composure, self)
                end
            end
        end,

        event_handlers = 
        {
            [ EVENT.MODIFIER_ADDED ] = function( self, modifier, source )
                if modifier.id == "INFLUENCE" then
                    self:Evoke( 1 )
                end
            end,

            [ EVENT.MODIFIER_CHANGED ] = function( self, modifier, delta, clone, source )
                if modifier.id == "INFLUENCE" and delta > 0 then
                    self:Evoke( 1 )
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ResetEvoke()
                end
            end,
        },
    },

    collected_plus = 
    {
        name = "Flexible Collected",
        flags = CARD_FLAGS.DIPLOMACY,

        CanPlayCard = function ( self, card, engine, target )
            if self.negotiator:GetModifierStacks( "INFLUENCE" ) == 0 then
                return false, loc.format( LOC"CONVO_COMMON.REQ_INFLUENCE", 1 )
            end
            return true
        end,
    },

    collected_plus2 = 
    {
        name = "Boosted Collected",
        desc = "Apply <#UPGRADE>{1} {COMPOSURE}</> to your {INFLUENCE}.\n{EVOKE}: Gain {INFLUENCE}.",
        composure = 4,
    },

    veiled_anger =
    {
        name = "Veiled Anger",
        flavour = "'He didn’t even have to say anything. The whole place could feel it.'",

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 0,
        min_persuasion = 0,
        max_persuasion = 3,
    },

    veiled_anger_plus =
    {
        name = "Tall Anger",

        min_persuasion = 1,
        max_persuasion = 5,
    },

    veiled_anger_plus2 =
    {
        name = "Mirrored Anger",
        desc = "Spend {DOMINANCE 1}: Attack with this card twice.",

        OnPostResolve = function( self, minigame, targets )
            if self.negotiator:HasModifier("DOMINANCE") then
                for i, target in ipairs( targets ) do
                    minigame:ApplyPersuasion( self, target )
                end
                -- Do this after targetting, so we don't lose our dominance bonus.
                self.negotiator:DeltaModifier("DOMINANCE", -1, self)
            end
        end,
    },

    swift_rebuttal =
    {
        name = "Swift Rebuttal",
        flavour = "'A'ha! Got you!'",
        desc = "{IMPROVISE} a card that costs 0 from your discard pile.",

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,
        min_persuasion = 1,
        max_persuasion = 3,
        num_cards = 1,

        OnPostResolve = function( self, minigame )
            local function IsZeroCostCard( card )
                return minigame:CalculateActionCost( card ) == 0 and not card:IsFlagged( CARD_FLAGS.UNPLAYABLE )
            end

            local cards = {}
            for i, card in minigame:GetDiscardDeck():Cards() do
                if IsZeroCostCard(card) then
                    table.insert(cards, card)
                end
            end

            cards = table.multipick(cards, 3)
            minigame:ImproviseCards( cards, self.num_cards, nil, "ad_lib", nil, self )
        end,
    },
    
    swift_rebuttal_plus =
    {
        name = "Boosted Rebuttal",
        min_persuasion = 3,
        max_persuasion = 5,
    },

    swift_rebuttal_plus2 =
    {
        name = "Visionary Rebuttal",
        desc = "{IMPROVISE} <#UPGRADE>two</> cards that cost 0 from your discard pile.",
        num_cards = 2,
    }, 

    brainstorm =
    {
        name = "Brainstorm",
        flavour = "'Alright, I have a few ideas.'",
        desc = "Draw {1} cards.",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.draw_count)
        end,

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.COMMON,
        draw_count = 3,

        OnPostResolve = function( self, minigame )
            minigame:DrawCards( self.draw_count )
        end,
    },
    brainstorm_plus =
    {
        name = "Boosted Brainstorm",
        desc = "Draw <#UPGRADE>{1}</> cards.",
        draw_count = 4,
    },
    brainstorm_plus2 =
    {
        name = "Stone Brainstorm",
        desc = "Draw {1} cards.\n<#UPGRADE>Gain {2} {COMPOSURE}.</>",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.draw_count, self:CalculateComposureText( self.composure_amt ))
        end,

        draw_count = 3,

        composure_amt = 2,
        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards( self.draw_count )
            self.negotiator:DeltaComposure(self.composure_amt, self)
        end,
    },

    ergo =
    {
        name = "Ergo...",
        flavour = "'Therefore...'",
        desc = "{IMPROVISE} a card from your discard pile.",

        cost = 1,
        num_cards = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,
        pool_size = 3,
        mod_xp = -2,

        OnPostResolve = function( self, minigame )
            local cards = {}
            for i, card in minigame:GetDiscardDeck():Cards() do
                table.insert(cards, card)
            end

            local cards = table.multipick(cards, self.pool_size)
            minigame:ImproviseCards( cards, self.num_cards, nil, "ad_lib", nil, self )
        end,        
    },
    ergo_plus =
    {
        name = "Visionary Ergo...",
        desc = "{IMPROVISE} <#UPGRADE>two</> cards from your discard pile.",
        num_cards = 2,
    },
    ergo_plus2 =
    {
        name = "Boosted Ergo...",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> a card from your discard pile.",
        pool_size = 5,
    },

    browbeat = 
    {
        name = "Browbeat",
        flavour = "'You’ve made your point! Put the damn gun away!'",
        desc = "Attack once for each action available.",
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.VARIABLE_COST,
        rarity = CARD_RARITY.UNCOMMON,

        mod_xp = -2,

        cost = 0,

        min_persuasion = 1,
        max_persuasion = 2,

        bonus = 0,

        OnPreResolve = function( self, minigame, targets )
            table.clear( targets )
        end,

        OnPostResolve = function( self, minigame, targets )
            local actions = minigame:GetActionCount()
            minigame:ModifyActionCount( -actions )
            for i = 1, actions + self.bonus do
                minigame:ApplyPersuasion( self )
            end
        end,
    },
    browbeat_plus =
    {
        name = "Boosted Browbeat",
        min_persuasion = 2,
        max_persuasion = 3,
    },

    browbeat_plus2 =
    {
        name = "Extended Browbeat",
        desc = "Attack once for each action available <#UPGRADE>+ 1</>.",
        bonus = 1,
    },

    ipso_facto = 
    {
        name = "Ipso Facto",
        flavour = "'I think it’s Latin? Sure makes it hard to figure out.'",
        desc = "Gain {INFLUENCE {1}}.\n{EVOKE}: An opponent's argument is removed or destroyed.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence )
        end,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.UNPLAYABLE,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,
        influence = 1,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS },

        event_handlers =
        {
            [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, card )
                if modifier.negotiator ~= self.negotiator and
                    (modifier.modifier_type == MODIFIER_TYPE.ARGUMENT or modifier.modifier_type == MODIFIER_TYPE.BOUNTY) then
                    --
                    self:Evoke( 1 )
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ResetEvoke()
                end
            end,
        },

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "INFLUENCE", self.influence, self )
        end,
    },

    ipso_facto_plus =
    {
        name = "Boosted Facto",
        desc = "Gain <#UPGRADE>{INFLUENCE {1}}.</>\n{EVOKE}: An opponent's argument is removed or destroyed.",
        influence = 2,
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.UNPLAYABLE
    },

    ipso_facto_plus2 =
    {
        name = "Flexible Facto",
        flags = CARD_FLAGS.DIPLOMACY,
        cost = 0,
    },

    oppress = 
    {
        name = "Oppress",
        flavour = "'You haven’t made a decent point in hours! Just give up already.'",
        desc = "If this card destroys an argument, gain {DOMINANCE 1}.",

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.COMMON,

        cost = 1,
        min_persuasion = 1,
        max_persuasion = 4,

        event_handlers =
        {
            [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, card )
                if card == self then
                    self.negotiator:AddModifier( "DOMINANCE", 1, self )
                end
            end,
        },
    },
    
    oppress_plus =
    {
        name = "Twisted Oppress",
        desc = "If this card destroys an argument, gain <#UPGRADE>{DOMINANCE 2}</>.",
        min_persuasion = 0,
        max_persuasion = 3,

        event_handlers =
        {
            [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, card )
                if card == self then
                    self.negotiator:AddModifier( "DOMINANCE", 2, self )
                end
            end,
        },
    },

    oppress_plus2 =
    {
        name = "Tall Oppress",
        min_persuasion = 1,
        max_persuasion = 8,
    },

    domineer =
    {
        name = "Domineer",
        flavour = "'You’re going to be picking up your teeth after I’m through with you!'",
        desc = "Spend {1} {DOMINANCE}: Gain 2 actions.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.dom_cost )
        end,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.RARE,
        cost = 1,

        mod_xp = -2,

        min_persuasion = 3,
        max_persuasion = 3,
        dom_cost = 3,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("DOMINANCE") >= self.dom_cost
        end,

        OnPostResolve = function( self, minigame )
            if self.negotiator:GetModifierStacks("DOMINANCE") >= self.dom_cost then
                self.negotiator:RemoveModifier("DOMINANCE", self.dom_cost)
                minigame:ModifyActionCount(2)
                if self.card_draw then
                    minigame:DrawCards(self.card_draw)
                end
            end
        end,
    },
    domineer_plus =
    {
        name = "Pale Domineer",
        desc = "Spend <#UPGRADE>{1}</> {DOMINANCE}: Gain 2 actions.",
        dom_cost = 2,
    },

    domineer_plus2 =
    {
        name = "Visionary Domineer",
        desc = "Spend {1} {DOMINANCE}: Gain 2 actions and <#UPGRADE>draw 2 cards</>.",
        card_draw = 2,
    },

    menacing_air = 
    {
        name = "Menacing Air",
        flavour = "'I don’t know what it is. But I don’t like it.'",
        desc = "{menacing_air|}Gain: At the beginning of your turn, gain {DOMINANCE 1}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.period )
        end,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 3,

        OnPostResolve = function( self )
            self.negotiator:AddModifier( "menacing_air", 1, self )
        end,

        modifier =
        {
            desc = "At the beginning of your turn, gain <#HILITE>{1}</> {DOMINANCE}.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks or 1 )
            end,

            max_resolve = 5,

            sound = "event:/sfx/battle/cards/neg/create_argument/menacing_air",

            OnBeginTurn = function( self, minigame )
                self.negotiator:AddModifier( "DOMINANCE", self.stacks, self )
            end
        }
    },

    menacing_air_plus =
    {
        name = "Pale Menacing Air",
        cost = 2,
    },

    menacing_air_plus2 = 
    {
        name = "Initial Menacing Air",
        desc = "{menacing_air|}Gain: At the beginning of your turn, gain {DOMINANCE 1}.",
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    abrupt_remark = 
    {
        name = "Abrupt Remark",
        flavour = "'Wrong!'",
        desc = "Draw 1 card.\n{EVOKE}: An opponent argument is removed or destroyed.",
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.UNPLAYABLE,
        rarity = CARD_RARITY.COMMON,

        max_xp = 6,
        cost = 0,
        draw = 1,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS },

        event_handlers =
        {
            [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, card )
                if modifier.negotiator ~= self.negotiator and
                    (modifier.modifier_type == MODIFIER_TYPE.ARGUMENT or modifier.modifier_type == MODIFIER_TYPE.BOUNTY) then
                    self:Evoke( 1 )
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ResetEvoke()
                end
            end,
        },

        OnPostResolve = function( self, minigame )
            minigame:DrawCards( self.draw )
        end,
    },

    abrupt_remark_plus =
    {
        name = "Visionary Remark",
        desc = "Draw <#UPGRADE>2 cards</>.\n{EVOKE}: An opponent argument is removed or destroyed.",
        draw = 2,
    },

    abrupt_remark_plus2 =
    {
        name = "Wide Remark",
        desc = "Draw 1 card.\n{EVOKE}: <#UPGRADE>Any argument</> is removed or destroyed.",

        event_handlers =
        {
            [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, card )
                if (modifier.modifier_type == MODIFIER_TYPE.ARGUMENT or modifier.modifier_type == MODIFIER_TYPE.BOUNTY) then
                    self:Evoke( 1 )
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ResetEvoke()
                end
            end,
        },
    },

    erupt = 
    {
        name = "Erupt",
        desc = "Attack a random opponent argument.\n{EVOKE}: Play 4 Hostility cards in a single turn.\n{1}",
        flavour = "'Do you have any idea who I am!?'",
        desc_fn = function( self, fmt_str )
            if self.engine and self.evoke_count then
                local str = loc.format( LOC"CARD_ENGINE.CARDS_PLAYED", self.evoke_count )
                return loc.format( fmt_str, str )
            else
                return loc.format( fmt_str, "" )
            end
        end,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.UNPLAYABLE,
        rarity = CARD_RARITY.COMMON,
        auto_target = true,
        target_mod = TARGET_MOD.RANDOM1,

        cost = 0,

        min_persuasion = 2,
        max_persuasion = 2,

        evoke_max = 4,

        deck_handlers = { DECK_TYPE.DRAW, DECK_TYPE.DISCARDS },

        event_handlers = 
        {
            [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                if card.owner == self.owner then
                    if CheckBits( card.flags, CARD_FLAGS.HOSTILE ) then
                        self:Evoke( self.evoke_max )
                    end
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ResetEvoke()
                end
            end,
        },
    },

    erupt_plus = 
    {
        name = "Boosted Erupt",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    erupt_plus2 = 
    {
        name = "Pale Erupt",
        desc = "Attack a random opponent argument.\n{EVOKE}: Play <#UPGRADE>3</> Hostility cards in a single turn.\n{1}",
        evoke_max = 3,
    },

    agitation = 
    {
        name = "Agitation",
        flavour = "'I’m a patient person but this is getting ridiculous.'",
        desc = "Create 3 {HEATED} arguments.",
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        cost = 3,

        OnPostResolve = function( self, minigame )
            for i = 1, 3 do
                self.negotiator:CreateModifier("HEATED", 1, self)
            end
        end,
    },
    agitation_plus =
    {
        name = "Pale Agitation", 
        cost = 2,
    },

    agitation_plus2 =
    {
        name = "Boosted Agitation", 
        desc = "<#UPGRADE>Create 4</> {HEATED} arguments.",

        OnPostResolve = function( self, minigame )
            for i = 1, 4 do
                self.negotiator:CreateModifier("HEATED", 1, self)
            end
        end,
    },

    bulldoze = 
    {
        name = "Bulldoze",
        flavour = "'Outta my way!'",
        desc = "{bulldoze|}Gain: Whenever you destroy an argument, gain an action.",

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        cost = 2,

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "bulldoze", 1, self)
        end,

        modifier =
        {
            desc = "Whenever you destroy an argument, gain <#HILITE>{1}</> {1*action|actions}.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks or 1 )
            end,

            max_resolve = 8,

            sound = "event:/sfx/battle/cards/neg/create_argument/bulldoze",

            event_handlers = 
            {
                [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                    if source and source.owner == self.owner then
                        self.engine:ModifyActionCount( self.stacks )
                    end
                end
            },
        }
    },

    bulldoze_plus =
    {
        name = "Dominant Bulldoze",
        desc = "<#UPGRADE>Gain {DOMINANCE 1}.</>\n{bulldoze|}Gain: Whenever you destroy an argument, gain an action.",

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "bulldoze", 1, self)
            self.negotiator:AddModifier( "DOMINANCE", 1, self)
        end,
    },

    bulldoze_plus2 =
    {
        name = "Stone Bulldoze",
        desc = "<#UPGRADE>Gain {1} {COMPOSURE}.</>\n{bulldoze|}Gain: Whenever you destroy an argument, gain an action.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText( self.composure_amt ))
        end,

        composure_amt = 4,
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier( "bulldoze", 1, self)
            self.negotiator:DeltaComposure(self.composure_amt, self)
        end  
    },

    low_blow = 
    {
        name = "Low Blow",
        flavour = "'That’s gotta hurt.'",
        desc = "Remove target argument's {COMPOSURE}.",

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,
        min_persuasion = 3,
        max_persuasion = 4,

        event_handlers =
        {
            [ EVENT.PRE_RESOLVE ] = function( self, minigame, card, targets )
                if card == self then
                    for i, target in ipairs( targets ) do
                        local composure = target:GetComposure()
                        target:DeltaComposure( -composure, self )
                        if self.steal then
                            self.negotiator:DeltaComposure( composure, self )
                        end
                    end
                end
            end
        }
    },
    low_blow_plus =
    {
        name = "Thief's Low Blow",
        desc = "<#UPGRADE>Steal</> target argument's {COMPOSURE}.",
        steal = true,
    },

    low_blow_plus2 =
    {
        name = "Boosted Low Blow",
        min_persuasion = 4,
        max_persuasion = 7,
    },

    bellow = 
    {
        name = "Bellow",
        flavour = "'Sometimes the loudest thing in the room wins.'",
        desc = "Deal 1 bonus damage for every other Hostility card in your hand.",

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,
        min_persuasion = 1,
        max_persuasion = 2,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
        },

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self then
                    local count = 0
                    for i,card in self.engine:GetHandDeck():Cards() do
                        if card ~= self and CheckBits(card.flags, CARD_FLAGS.HOSTILE) then
                            count = count + 1
                        end
                    end
                    persuasion:AddPersuasion( count, count, self )
                end
            end,
        },
    },

    bellow_plus = 
    {
        name = "Tall Bellow",
        max_persuasion = 6,
    },

    bellow_plus2 = 
    {
        name = "Rooted Bellow",
        min_persuasion = 4,
        max_persuasion = 4,

    },

    this_ends_now =
    {
        name = "This Ends Now",
        flavour = "'It’s all over!'",
        desc = "Spend all {DOMINANCE}: Gain 1 action and draw a card for each {DOMINANCE}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.resolve_amount )
        end,

        PreReq = function( self, minigame )
            return self:CanPlayCard( self, minigame )
        end,

        CanPlayCard = function( self, card, engine, target )
            if self.negotiator:GetModifierStacks("DOMINANCE") <= 0 then
                return false, loc.format( LOC "CONVO_COMMON.REQ_DOMINANCE", 1 )
            end
            return true
        end,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        cost = 2,

        OnPostResolve = function( self, minigame )
            local dom_stacks = self.negotiator:GetModifierStacks("DOMINANCE")
            self.negotiator:RemoveModifier("DOMINANCE", dom_stacks)
            minigame:ModifyActionCount(dom_stacks)
            minigame:DrawCards(dom_stacks)
        end
    },

    this_ends_now_plus =
    {
        name = "Pale This Ends Now",
        cost = 1,
    },

    this_ends_now_plus2 =
    {
        name = "Enduring This Ends Now",
        flags = CARD_FLAGS.HOSTILE,
    },

    heavy_handed = 
    {
        name = "Heavy Handed",
        flavour = "'What is this? Some kind of Bandit deal?!'",
        desc = "Spend {DOMINANCE 1}: Destroy an opponent argument.",

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        target_enemy = TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY,

        cost = 1,

        PreReq = function( self, minigame )
            return self:CanPlayCard( self, minigame )
        end,

        CanPlayCard = function( self, card, engine, target )
            if self.negotiator:GetModifierStacks("DOMINANCE") < 1 then
                return false, loc.format( LOC "CONVO_COMMON.REQ_DOMINANCE", 1 )
            end
            return true
        end,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:RemoveModifier("DOMINANCE", 1)
            for i, target in ipairs( targets ) do
                self.anti_negotiator:DestroyModifier( target, self )
            end
        end
    },
    heavy_handed_plus =
    {
        name = "Enhanced Heavy Handed",
        desc = "Spend <#UPGRADE>{DOMINANCE 3}</>: Destroy <#UPGRADE>ALL</> opponent arguments.",

        target_enemy = TARGET_FLAG.NONE,

        CanPlayCard = function( self, card, engine, target )
            if self.negotiator:GetModifierStacks("DOMINANCE") < 3 then
                return false, loc.format( LOC "CONVO_COMMON.REQ_DOMINANCE", 3 )
            end
            return true
        end,


        OnPostResolve = function( self, minigame, targets )
            self.negotiator:RemoveModifier("DOMINANCE", 3)

            local to_destroy = {}
            for k, argument in self.anti_negotiator:ModifierSlots() do
                if argument.modifier_type == MODIFIER_TYPE.ARGUMENT or argument.modifier_type == MODIFIER_TYPE.BOUNTY then
                   table.insert(to_destroy, argument) 
                end
            end

            for i=1, #to_destroy do
                self.anti_negotiator:DestroyModifier( to_destroy[i], self )
            end
        end
    },
    heavy_handed_plus2 =
    {
        name = "Pale Heavy Handed",
        cost = 0,
    },

    overbear =
    {
        name = "Overbear",
        flavour = "'Wait. Why is he standing up?'",
        desc = "Attack a random opponent argument once for each opponent argument.",
        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        target_mod = TARGET_MOD.RANDOM1,
        auto_target = true,

        cost = 1,
        min_persuasion = 1,
        max_persuasion = 3,

        OnPreResolve = function( self, minigame, targets )
            local count = 0
            for i, modifier in self.anti_negotiator:Modifiers() do
                if modifier.modifier_type == MODIFIER_TYPE.ARGUMENT or modifier.modifier_type == MODIFIER_TYPE.BOUNTY or modifier.modifier_type == MODIFIER_TYPE.CORE then
                    count = count + 1
                end
            end
            for i = 1, count -1 do
                minigame:ApplyPersuasion( self )
                self:AssignTarget( nil ) -- Ensures a new target is chosen each iteration.
            end
            if self.draw_count and count > 0 then
                minigame:DrawCards( count * self.draw_count )
            end
        end,
    },

    overbear_plus = 
    {
        name = "Tall Overbear",
        min_persuasion = 1,
        max_persuasion = 5,
    },
    
    overbear_plus2 = 
    {
        name = "Enhanced Overbear",
        desc = "Attack a random opponent argument once and <#UPGRADE>draw a card</> for each opponent argument.",

        draw_count = 1,
    },

    subtlety =
    {
        name = "Subtlety",
        flavour = "'If you do a job well enough, nobody can tell you were there at all.'",
        desc = "Spend {INFLUENCE {1}}: Draw {2} cards.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_cost, self.draw_count )
        end,

        cost = 1,
        draw_count = 2,
        influence_cost = 1,

        min_persuasion = 3,
        max_persuasion = 3,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost
        end,

        OnPostResolve = function( self, minigame )
            if self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost then
                self.negotiator:RemoveModifier("INFLUENCE", self.influence_cost)
                minigame:DrawCards(self.draw_count)
            end
        end,
    },

    subtlety_plus =
    {
        name = "Rooted Subtlety",
        min_persuasion = 5,
        max_persuasion = 5,
    },
    subtlety_plus2 =
    {
        name = "Boosted Subtlety",
        desc = "Spend {INFLUENCE {1}}: Draw <#UPGRADE>{2}</> cards.",
        draw_count = 3,
    },

    buying_time = 
    {
        name = "Buying Time",
        flavour = "'In Havaria, just knowing the right people can be enough.'",
        desc = "Spend {INFLUENCE {1}}: Gain {2} actions.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_cost, self.action_count )
        end,
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 0,
        influence_cost = 2,
        action_count = 2,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost
        end,


        CanPlayCard = function ( self, card, engine, target )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost, loc.format( LOC"CONVO_COMMON.REQ_INFLUENCE", self.influence_cost)
        end,

        OnPostResolve = function( self, minigame )
            if self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost then
                minigame:ModifyActionCount(self.action_count)
                self.negotiator:RemoveModifier("INFLUENCE", self.influence_cost, self)
            end
        end
    },

    buying_time_plus = 
    {
        name = "Lean Time",
        desc = "Spend <#UPGRADE>{INFLUENCE {1}}</>: Gain {2} actions.",
        influence_cost = 1,
    },

    buying_time_plus2 = 
    {
        name = "Boosted Time",
        desc = "Spend {INFLUENCE {1}}: Gain <#UPGRADE>{2} actions.</>",
        action_count = 3,
    },

    nonsequitur =
    {
        name = "Non Sequitur",
        desc = "Your next card this turn costs 0 actions.",
        flavour = "'What does any of that have to do with apples?'",

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        OnPostResolve = function( self, minigame )
            local modifier = self.negotiator:AddModifier( "nonsequitur", 1, self )
            if modifier then
                modifier.ignore_card = modifier.ignore_card or self
            end
        end,

        modifier =
        {
            hidden = true,

            event_priorities =
            {
                [ EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
            },

            event_handlers =
            {
                [ EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                    if card:GetNegotiator() == self.negotiator and
                        card.deck and (card.deck:GetDeckType() == DECK_TYPE.IN_HAND or card.deck:GetDeckType() == DECK_TYPE.RESOLVE) then
                        --
                        cost_acc:ModifyValue( 0, self )
                    end
                end,

                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    if card:GetNegotiator() == self.negotiator and self.ignore_card ~= card then
                        self.negotiator:RemoveModifier( self, 1 )
                    end
                end,

                [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        negotiator:RemoveModifier( self )
                    end
                end,
            }
        }
    },
    nonsequitur_plus =
    {
        name = "Enduring Non Sequitur",
        flags = CARD_FLAGS.MANIPULATE,
    },

    nonsequitur_plus2 =
    {
        name = "Pale Non Sequitur",
        cost = 0,
    },

    build_rapport =
    {
        name = "Build Rapport",
        flavour = "'Let me get that for you.'",
        desc = "Gain {INFLUENCE {1}} and apply {2} {COMPOSURE} to it.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.influence_amt, self:CalculateComposureText( self.composure ))
        end,

        cost = 1,
        composure = 3,

        influence_amt = 2,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        OnPostResolve = function( self, minigame, targets )
            local influence = self.negotiator:AddModifier("INFLUENCE", self.influence_amt, self)
            if influence then
                influence:DeltaComposure( self.composure, self )
            end
        end
    },
    
    build_rapport_plus =
    {
        name = "Initial Rapport",
        desc = "Gain {INFLUENCE {1}} and apply {2} {COMPOSURE} to it.",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.AMBUSH,
    },

    build_rapport_plus2 =
    {
        name = "Boosted Rapport",
        desc = "Gain <#UPGRADE>{INFLUENCE {1}}</> and apply {2} {COMPOSURE} to it.",
        flags = CARD_FLAGS.DIPLOMACY,
        influence_amt = 3,
    },

    tyrannize =
    {
        name = "Tyrannize",
        flavour = "'Listen up you little punk!'",
        desc = "Spend {INFLUENCE {1}}: Gain {DOMINANCE {2}}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_cost, self.conversion )
        end,

        cost = 1,

        influence_cost = 1,
        conversion = 2,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= self.influence_cost
        end,

        CanPlayCard = function ( self, card, engine, target )
            if self.negotiator:GetModifierStacks( "INFLUENCE" ) < self.influence_cost then
                return false, loc.format( LOC"CONVO_COMMON.REQ_INFLUENCE", self.influence_cost )
            end
            return true
        end,

        OnPostResolve = function( self, minigame )
            if self.negotiator:GetModifierStacks( "INFLUENCE" ) >= self.influence_cost then
                self.negotiator:RemoveModifier( "INFLUENCE", self.influence_cost )
                self.negotiator:AddModifier( "DOMINANCE", self.conversion, self )
            end
        end,
    },

    tyrannize_plus =
    {
        name = "Pale Tyrannize",
        cost = 0,
    },

    tyrannize_plus2 =
    {
        name = "Enhanced Tyrannize",
        desc = "Spend {INFLUENCE {1}}: Gain <#UPGRADE>{DOMINANCE {2}}</>.",
        conversion = 3,
    },

    segue =
    {
        name = "Segue",
        flavour = "'Ah! That reminds me.'",
        desc = "Draw {1} {1*card|cards}.\nReturn the next card you play this turn to your draw pile.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.draw )
        end,
        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,
        draw = 1,
        stacks = 1,

        OnPostResolve = function( self, minigame )
            minigame:DrawCards( self.draw )
            self.negotiator:CreateModifier( "segue", self.stacks, self )
        end,

        modifier =
        {
            hidden = true,
            OnInit = function( self, source )
                self.source = source
            end,

            event_handlers =
            {
                [ EVENT.END_TURN ] = function( self, engine, negotiator )
                    if negotiator == self.negotiator then
                        negotiator:RemoveModifier( self )
                    end
                end,

                [ EVENT.POST_RESOLVE ] = function( self, minigame, card, targets )
                    if self.owner == card.owner and card ~= self.source then
                        if card.deck ~= minigame:GetDrawDeck() and card.deck ~= minigame:GetTrashDeck() then
                            card:TransferCard( minigame:GetDrawDeck() )
                            self.negotiator:RemoveModifier( self, 1 )
                        end
                    end
                end
            }
        }
    },
    segue_plus =
    {
        name = "Enhanced Segue",
        desc = "Draw {1} {1*card|cards}.\nReturn the next <#UPGRADE>2</> cards you play this turn to your draw pile.",
        stacks = 2,
    },

    segue_plus2 = 
    {
        name = "Visionary Segue",
        desc = "Draw <#UPGRADE>{1}</> {1*card|cards}.\nReturn the next card you play this turn to your draw pile.",
        draw = 2,
    },

    airtight = 
    {
        name = "Airtight",
        flavour = "'Veep grumbled and slumped back into his chair. Deflated.'",
        desc = "{airtight 2|}Gain: Whenever you play a Diplomacy card, apply 2 {COMPOSURE} to a random argument.",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        cost = 3,

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "airtight", 2, self )
        end,

        modifier =
        {
            desc = "Whenever you play a Diplomacy card, apply <#HILITE>{1}</> {COMPOSURE} to a random friendly argument.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks or 1 )
            end,

            target_self = TARGET_ANY_RESOLVE,

            max_resolve = 8,

            sound = "event:/sfx/battle/cards/neg/create_argument/airtight",

            event_handlers = 
            {
                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    if self.owner == card.owner and CheckBits( card.flags, CARD_FLAGS.DIPLOMACY ) then
                        local target = minigame:CollectPrimaryTarget(self)
                        target:DeltaComposure(self.stacks, self)
                        self:ClearTarget()
                    end
                end
            },
        },
    },

    airtight_plus =
    {
        name = "Initial Airtight",
        desc = "{airtight|}Gain: Whenever you play a Diplomacy card, apply 2 {COMPOSURE} to a random argument.",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    airtight_plus2 =
    {
        name = "Pale Airtight",
        cost = 2,
    },

    quick_thinking =
    {
        name = "Quick Thinking",
        desc = "{IMPROVISE} a card from a pool of special cards.",
        flavour = "'Some of my best ideas come at the last possible moment.'",
        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.BASIC,

        sound = "event:/sfx/battle/cards/gen/play_card/promoted_thinking",

        pool_size = 3,

        pool_cards = {"improvise_gruff", "improvise_carry_over", "improvise_options", "improvise_withdrawn", "improvise_diplomacy", "improvise_hostile", "improvise_wide_composure", "improvise_bait", "improvise_vulnerability"},

        OnPostResolve = function( self, minigame, targets)
            local cards = ObtainWorkTable()

            cards = table.multipick( self.pool_cards, self.pool_size )
            for k,id in pairs(cards) do
                cards[k] = Negotiation.Card( id, self.owner  )
            end
            minigame:ImproviseCards( cards, 1, nil, nil, nil, self )
            ReleaseWorkTable(cards)
        end,
    },

    quick_thinking_plus =
    {
        name = "Promoted Thinking",
        desc = "{IMPROVISE} a card from a pool of <#UPGRADE>upgraded</> special cards.",
        pool_cards = {"improvise_gruff_upgraded", "improvise_carry_over_upgraded", "improvise_options_upgraded", "improvise_withdrawn_upgraded", "improvise_diplomacy_upgraded", "improvise_hostile_upgraded", "improvise_wide_composure_upgraded", "improvise_bait_upgraded", "improvise_vulnerability_upgraded"},
    },

    quick_thinking_plus2 = 
    {
        name = "Boosted Thinking",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> a card from a pool of special cards.",
        pool_size = 5,
    },

    

    improvise_gruff = 
    {
        name = "Gruff",
        flavour = "'Itches...'",
        desc = "{INCEPT} 1 {FLUSTERED}.",

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            self.anti_negotiator:InceptModifier("FLUSTERED", 1 , self )
        end
    },

    improvise_gruff_upgraded = 
    {
        name = "Boosted Gruff",
        desc = "{INCEPT} 2 {FLUSTERED}.",
        icon =  "negotiation/improvise_gruff.tex",

        cost = 0,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            self.anti_negotiator:InceptModifier("FLUSTERED", 2 , self )
        end
    },

    improvise_carry_over =
    {
        name = "Carry Over",
        flavour = "'Let me get this straight.'",

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        features =
        {
            FREE_ACTION = 1,
        },
    },

    improvise_carry_over_upgraded =
    {
        name = "Boosted Carry Over",
        icon =  "negotiation/improvise_carry_over.tex",

        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        features =
        {
            FREE_ACTION = 2,
        },
    },

    improvise_options =
    {
        name = "Options",
        flavour = "'Always keep them open.'",
        desc = "Draw 2 cards.",

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(2)
        end
    },

    improvise_options_upgraded =
    {
        name = "Boosted Options",
        desc = "Draw 3 cards.",
        icon =  "negotiation/improvise_options.tex",

        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(3)
        end
    },

    improvise_withdrawn =
    {
        name = "Withdrawn",
        flavour = "'Give me some space.'",
        cost = 0,
        max_xp = 0,

        target_self = TARGET_ANY_RESOLVE,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        features = 
        {
            COMPOSURE = 3,
        },
    },

    improvise_withdrawn_upgraded =
    {
        name = "Boosted Withdrawn",
        icon =  "negotiation/improvise_withdrawn.tex",

        cost = 0,

        target_self = TARGET_ANY_RESOLVE,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        features = 
        {
            COMPOSURE = 5,
        },
    },

    improvise_vulnerability = 
    {
        name = "Stammer",
        flavour = "'Uhh… er... maybe.'",
        desc = "{INCEPT} 2 {VULNERABILITY}.",
        icon = "negotiation/stammer.tex",

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, card )
            self.anti_negotiator:AddModifier("VULNERABILITY", 2, self)
        end
    },

    improvise_vulnerability_upgraded = 
    {
        name = "Boosted Stammer",
        desc = "{INCEPT} 3 {VULNERABILITY}.",
        cost = 0,
        icon = "negotiation/stammer.tex",

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, card )
            self.anti_negotiator:AddModifier("VULNERABILITY", 3, self)
        end
    },

    improvise_diplomacy = 
    {
        name = "Observation",
        flavour = "'Hmmm...'",
        cost = 0,
        icon = "negotiation/observation.tex",

        min_persuasion = 0,
        max_persuasion = 4,
        max_xp = 0,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,
    },

    improvise_diplomacy_upgraded = 
    {
        name = "Tall Observation",
        cost = 0,
        icon = "negotiation/observation.tex",

        min_persuasion = 0,
        max_persuasion = 6,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,
    },

    improvise_hostile =
    {
        name = "Debate",
        flavour = "'I hope you’re ready, I’m a master at this.'",
        icon = "negotiation/debate.tex",
        cost = 0,
        max_xp = 0,

        min_persuasion = 1,
        max_persuasion = 2,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,
    },

    improvise_hostile_upgraded =
    {
        name = "Boosted Debate",
        icon = "negotiation/debate.tex",
        cost = 0,

        min_persuasion = 2,
        max_persuasion = 3,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,
    },

    improvise_wide_composure =
    {
        name = "Consideration",
        flavour = "'I haven’t thought of it that way before.'",
        icon = "negotiation/consideration.tex",
        desc = "Apply {1} {COMPOSURE} on 2 random friendly arguments.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText( self.composure_amt ))
        end,

        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        target_mod = TARGET_MOD.RANDOM2,
        target_self = TARGET_ANY_RESOLVE,
        auto_target = true,

        composure_amt = 2,
        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
        end
    },

    improvise_wide_composure_upgraded =
    {
        name = "Boosted Consideration",
        icon = "negotiation/consideration.tex",
        desc = "Apply {1} {COMPOSURE} on 2 random friendly arguments.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText( self.composure_amt ))
        end,
        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        target_mod = TARGET_MOD.RANDOM2,
        target_self = TARGET_ANY_RESOLVE,
        auto_target = true,

        composure_amt = 3,
        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
        end
    },

    improvise_bait =
    {
        name = "Gossip",
        flavour = "'And he didn’t know that SHE knew that he already knew all about it!'",
        icon = "negotiation/gossip.tex",
        desc = "Deploy {BAIT}.",
        cost = 0,
        max_xp = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier( "BAIT", 1, self)
        end,
    },

    improvise_bait_upgraded =
    {
        name = "Boosted Gossip",
        icon = "negotiation/gossip.tex",
        desc = "Deploy 2 {BAIT}.",
        cost = 0,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier( "BAIT", 2, self)
        end,
    },

    sals_instincts = 
    {
        name = "Sal's Instincts",
        desc = "Insert {inspiration} or {attitude} into your hand.",
        flavour = "'Consistency is just going to get you dead. If something's not working, it's time to try the opposite.'",
        max_xp = 8,
        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.BASIC,


        OnPostResolve = function( self, minigame, targets )
            local cards = {
                Negotiation.Card( "inspiration", self.owner ),
                Negotiation.Card( "attitude", self.owner ),
            }

            minigame:ImproviseCards( cards, 1, nil, nil, nil, self )
        end,
    },

    inspiration = 
    {
        name = "Inspiration",
        icon = "negotiation/instincts_diplomatic.tex",
        flavour = "'Let's look at what we have in common, then.'",

        cost = 0,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_persuasion = 2,
        max_persuasion = 2,

        features =
        {
            INFLUENCE = 2,
        },
    },

    attitude = 
    {
        name = "Attitude",
        icon = "negotiation/instincts_hostile.tex",
        flavour = "'You're lucky I'm even talking to you'",

        cost = 0,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_persuasion = 2,
        max_persuasion = 2,

        features =
        {
            DOMINANCE = 1,
        },
    },

    sals_instincts_plus2 = 
    {
        name = "Diplomatic Instincts",
        desc = "Insert {flatter}, {compliment} or {attitude} into your hand.",
        icon = "negotiation/instincts_diplomatic.tex",
        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.BASIC,


        OnPostResolve = function( self, minigame, targets )
            local cards = {
                Negotiation.Card( "flatter", self.owner ),
                Negotiation.Card( "compliment", self.owner ),
                Negotiation.Card( "attitude", self.owner ),
            }

            minigame:ImproviseCards( cards, 1, nil, nil, nil, self )
        end,
    },

    flatter =
    {
        name = "Flatter",
        flavour = "'Looking good there F’ssh.'",
        desc = "Spend 1 {INFLUENCE}: deals 2 bonus damage and hits all opponent arguments.",

        cost = 0,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_persuasion = 3,
        max_persuasion = 3,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= 1
        end,

        OnPostResolve = function( self, minigame, targets )
            if self.negotiator:GetModifierStacks("INFLUENCE") >= 1 then
                self.negotiator:RemoveModifier("INFLUENCE", 1)
            end
        end,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
        },

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self then
                    if self.negotiator:HasModifier("INFLUENCE") then
                        persuasion:AddPersuasion( 2, 2, self )
                        self.auto_target = true
                    else
                        self.auto_target = false
                    end
                end
            end,

            [ EVENT.START_RESOLVE ] = function( self, minigame, card )
                if card == self and self.negotiator:HasModifier("INFLUENCE") then
                    card.target_mod = TARGET_MOD.TEAM
                end
            end
        },
    },

    compliment =
    {
        name = "Compliment",
        flavour = "'Have you been working out?'",
        icon = "negotiation/improvise_compliment.tex",
        desc = "Gain 2 {INFLUENCE}, then deal damage equal to half your {INFLUENCE} to a random opponent argument.",

        cost = 0,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        target_enemy = TARGET_ANY_RESOLVE,
        auto_target = true,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("INFLUENCE", 2, self)

            local damage = math.ceil( self.negotiator:GetModifierStacks("INFLUENCE") * 0.5 )
            if damage > 0 then
                self.min_persuasion, self.max_persuasion = damage, damage
                minigame:ApplyPersuasion( self )
                self.min_persuasion, self.max_persuasion = nil, nil
            end
        end
    },

    sals_instincts_plus = 
    {
        name = "Hostile Instincts",
        desc = "Insert {obtuse}, {mean} or {inspiration} into your hand.",
        icon = "negotiation/instincts_hostile.tex",

        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.BASIC,


        OnPostResolve = function( self, minigame, targets )
            local cards = {
                Negotiation.Card( "obtuse", self.owner ),
                Negotiation.Card( "mean", self.owner ),
                Negotiation.Card( "inspiration", self.owner ),
            }

            minigame:ImproviseCards( cards, 1, nil, nil, nil, self )
        end,
    },

    obtuse = 
    {
        name = "Obtuse",
        flavour = "'Remind me again why I should listen to you.'",
        icon =  "negotiation/improvise_obtuse.tex",

        cost = 0,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_persuasion = 2,
        max_persuasion = 2,

        features = 
        {
            DOMINANCE = 2
        },
    },

    mean = 
    {
        name = "Mean",
        flavour = "'Get out of here you mutt!'",
        icon = "negotiation/improvise_mean.tex",

        cost = 0,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_persuasion = 4,
        max_persuasion = 4,
    },

    empathy =
    {
        name = "Empathy",
        flavour = "'I just saw a lot of myself back there.'",
        desc = "You and your opponent restore {1} resolve.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.resolve_gain )
        end,

        cost = 1,
        resolve_gain = 6,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        OnPostResolve = function( self, minigame, card )
            self.negotiator:RestoreResolve( self.resolve_gain, self )
            self.anti_negotiator:RestoreResolve( self.resolve_gain, self )
        end,
    },

    empathy_plus =
    {
        name = "Initial Empathy",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND | CARD_FLAGS.AMBUSH,
    },

    empathy_plus2 =
    {
        name = "Boosted Empathy",
        desc = "You and your opponent restore <#UPGRADE>{1}</> resolve.",
        resolve_gain = 10,
    },

    duplicity =
    {
        name = "Duplicity",
        flavour = "'Alright, that’s getting a little too underhanded.'", 
        desc = "The next card played this turn is played twice.",

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        OnPostResolve = function( self, minigame )
            local modifier = self.negotiator:AddModifier( "duplicity", 1, self )
            modifier.skip_card = self
        end,

        modifier =
        {
            hidden = true,

            event_priorities =
            {
                [ EVENT.POST_RESOLVE ] = EVENT_PRIORITY_CLAMP,
            },

            event_handlers =
            {
                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    if card ~= self.skip_card then
                        -- Play it again.
                        self.negotiator:RemoveModifier( self, 1 )
                        minigame:PlayCard( card )
                    end
                end,
            },

            OnEndTurn = function( self, minigame )
                self.negotiator:RemoveModifier( self )
            end,
        },
    },
    duplicity_plus  =
    {
        name = "Enduring Duplicity",
        flags = CARD_FLAGS.MANIPULATE,
    },

    duplicity_plus2 =
    {
        name = "Pale Duplicity",
        cost = 0,
    },

    recall =
    {
        name = "Recall",
        flavour = "'Medium build, great teeth, smelled terrible.'",
        desc = "{recall|}Gain: Whenever you {EXPEND} a card, draw {1} {1*card|cards}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.stacks )
        end,
        cost = 1,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,
        stacks = 1,

        OnPostResolve = function( self, minigame, card )
            self.negotiator:AddModifier( "recall", self.stacks, self )
        end,

        modifier =
        {
            desc = "Whenever you {EXPEND} a card, draw <#HILITE>{1}</> {1*card|cards}.",
            max_resolve = 5,
            modifier_type = MODIFIER_TYPE.ARGUMENT,
            event_handlers =
            {
                [ EVENT.CARD_EXPENDED ] = function( self, minigame )
                    minigame:DrawCards( self.stacks )
                end,
            },
        }
    },
    recall_plus =
    {
        name = "Boosted Recall",
        desc = "{recall 2|}Gain: Whenever you {EXPEND} a card, draw <#UPGRADE>{1}</> {1*card|cards}.",
        stacks = 2,
    },
    recall_plus2 =
    {
        name = "Pale Recall",
        cost = 0,
    },

    bluster = 
    {
        name = "Bluster",
        flavour = "'And another thing!.'",
        desc = "{INCEPT} {1} {FLUSTERED}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.stacks )
        end,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 2,

        min_persuasion = 8,
        max_persuasion = 8,
        stacks = 2,
        
        OnPostResolve = function( self, minigame, card )
            self.anti_negotiator:InceptModifier("FLUSTERED", self.stacks, self )
        end,
    },

    bluster_plus = 
    {
        name = "Twisted Bluster",
        desc = "{INCEPT} {1} <#UPGRADE>{VULNERABILITY}.</>",

        OnPostResolve = function( self, minigame, card )
            self.anti_negotiator:InceptModifier("VULNERABILITY", self.stacks, self )
        end,
    },

    bluster_plus2 =
    {
        name = "Boosted Bluster",
        desc = "{INCEPT} <#UPGRADE>{1}</> {FLUSTERED}.",
        stacks = 4,
    },

    rapid_fire =
    {
        name = "Rapid Fire",
        flavour = "'I just couldn’t keep up with them!'",
        desc = "Attack a random argument.\nSpend all {INFLUENCE}: Attack again per {INFLUENCE} spent.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_limit )
        end,

        cost = 1,
        min_persuasion = 2,
        max_persuasion = 2,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        auto_target = true,
        target_mod = TARGET_MOD.RANDOM1,
        influence_retained = 0,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("INFLUENCE") >= 1
        end,

        OnPostResolve = function( self, minigame, targets )
            local stacks = self.negotiator:GetModifierStacks( "INFLUENCE" )
            for i = 1, stacks do
                minigame:ApplyPersuasion( self )
                self:AssignTarget( nil ) -- Ensures a new target is chosen each iteration.
                -- Decrement Inf. one at a time for presentational effect.
                self.negotiator:RemoveModifier( "INFLUENCE", 1 )
            end

            if self.influence_retained > 0 then
                self.negotiator:AddModifier( "INFLUENCE", self.influence_retained, self )
            end
        end,
    },

    rapid_fire_plus =
    {
        name = "Maintained Fire",
        desc = "Attack a random argument.\nSpend all {INFLUENCE}: Attack again per {INFLUENCE} spent.\n<#UPGRADE> Gain 1 {INFLUENCE}.</>",
        
        influence_retained = 1
    },

    rapid_fire_plus2 =
    {
        name = "Boosted Fire",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    seeds_of_doubt =
    {
        name = "Seeds of Doubt",
        flavour = "'Well now I don’t know WHAT to think.'",
        desc = "{INCEPT} {DOUBT {1}}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.stacks )
        end,

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.COMMON,
        stacks = 2,

        OnPostResolve = function( self, minigame, targets )
            self.anti_negotiator:InceptModifier("DOUBT", self.stacks, self )
        end,
    },
    seeds_of_doubt_plus = 
    {
        name = "Boosted Doubt",
        desc = "{INCEPT} <#UPGRADE>{DOUBT {1}}.</>",
        stacks = 3,
    },

    seeds_of_doubt_plus2 = 
    {
        name = "Visionary Doubt",
        desc = "{INCEPT} {DOUBT {1}}.\n<#UPGRADE>Draw a Card.</>",
        OnPostResolve = function( self, minigame, targets)
            minigame:DrawCards( 1 )
            self.anti_negotiator:InceptModifier("DOUBT", self.stacks, self )
         end,
        stacks = 2,
    },


    level_playing_field = 
    {
        name = "Level Playing Field",
        flavour = "'Let’s start this over.'",
        desc = "Gain {VULNERABILITY {1}}.\nAll cards deal +{2} damage this turn.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.vulnerability_stacks, self.damage_boost )
        end,

        cost = 1,
       
        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        vulnerability_stacks = 2,
        damage_boost = 3,

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "VULNERABILITY", self.vulnerability_stacks, self )
            self.negotiator:AddModifier("level_playing_field", self.damage_boost, self)
        end,

        modifier = 
        {
            hidden = true,

            event_priorities =
            {
                [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
            },

            event_handlers = 
            {
                [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                    if self.negotiator == source.negotiator and is_instance( source, Negotiation.Card) then
                        persuasion:AddPersuasion( self.stacks, self.stacks, self)
                    end
                end,

                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    self.negotiator:RemoveModifier("level_playing_field", self.stacks)
                end
            },
        },
    },
    level_playing_field_plus =
    {
        name = "Purified Playing Field",
        desc = "Gain <#UPGRADE>{VULNERABILITY {1}}</>.\nAll cards deal +{2} damage this turn.",
        vulnerability_stacks = 1,
    },
    level_playing_field_plus2 =
    {
        name = "Boosted Playing Field",
        desc = "Gain {VULNERABILITY {1}}.\nAll cards deal <#UPGRADE>+{2}</> damage this turn.",
        damage_boost = 4,
    },

    appeal_to_reason = 
    {
        name = "Appeal to Reason",
        flavour = "'Look, you’re a smart guy.'",
        desc = "Costs 1 less for each {INFLUENCE} you have.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.influence_cost )
        end,

        mod_xp = 2,

        cost = 4,
        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.RARE,
        influence_cost = 1,

        deck_handlers = ALL_DECKS,
        
        event_handlers =
        {
            [ EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                if card == self then
                    cost_acc:ModifyValue( card.def.cost - self.negotiator:GetModifierStacks("INFLUENCE"), self )
                end
            end,
        },

        min_persuasion = 4,
        max_persuasion = 6,
    },

    appeal_to_reason_plus = 
    {
        name = "Boosted Appeal",
        min_persuasion = 5,
        max_persuasion = 8,
    },

    appeal_to_reason_plus2 =
    {
        name = "Pale Appeal",
        cost = 3,
    },

    second_wind = 
    {
        name = "Second Wind",
        flavour = "'I could do this all day!'",
        desc = "Gain 1 action for every 3 cards in your discard pile.",
        loc_strings =
        {
            ALT_DESC = "Gain 1 action for every 3 cards in your discard pile ({1} actions).",
        },
        desc_fn = function( self, fmt_str )
            if self.engine then
                return loc.format( self.def:GetLocalizedString( "ALT_DESC" ), self:CalculateActions() )
            else
                return fmt_str
            end
        end,


        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        cost = 1,

        CalculateActions = function( self )
            return math.floor( self.engine:GetDeck( DECK_TYPE.DISCARDS ):CountCards() / 3 )
        end,

        SecondWind = function(self, minigame)
            minigame:ModifyActionCount( self:CalculateActions() )
        end,

        OnPostResolve = function( self, minigame )
            self:SecondWind(minigame)
        end,
    },
    second_wind_plus = 
    {
        name = "Pale Wind",
        cost = 0,
    },

    second_wind_plus2 = 
    {
        name = "Visionary Wind",
        desc = "Gain 1 action for every 3 cards in your discard pile.\n<#UPGRADE>Draw a card.</>",
        loc_strings =
        {
            ALT_DESC = "Gain 1 action for every 3 cards in your discard pile ({1} actions).\nDraw a card.",
        },

        OnPostResolve = function( self, minigame, targets)
            self:SecondWind(minigame)
            minigame:DrawCards( 1 )
        end,
    },

    scorched_earth = 
    {
        name = "Scorched Earth",
        flavour = "'This was never going to end well.'",
        loc_strings =
        {
            CARD_COUNT = " ({1} cards)"
        },
        desc = "{EXPEND} all non-Hostility cards in your discard pile.\nDeal {1} damage to a random opponent argument for every card expended.{2}",
        desc_fn = function( self, fmt_str )
            if self.engine then
                self.engine.persuasion_amount:CalculatePersuasion( self, nil, self.damage, self.damage )
                local damage = self.engine.persuasion_amount.min_persuasion
                local str = loc.format( self.def:GetLocalizedString( "CARD_COUNT" ), self:CountNonHostiles() )
                return loc.format( fmt_str, damage, str )
            else
                return loc.format( fmt_str, self.damage, "" )
            end
        end,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.RARE,

        cost = 2,
        damage = 2,
        
        target_enemy = TARGET_ANY_RESOLVE,
        auto_target = true,

        CountNonHostiles = function( self )
            local count = 0
            local deck = self.engine:GetDeck( DECK_TYPE.DISCARDS )
            for i, card in ipairs(deck:GetCards()) do
                if not CheckBits( card.flags, CARD_FLAGS.HOSTILE ) then
                    count = count + 1
                end
            end
            return count
        end,

        OnPostResolve = function( self, minigame )
            local function ExpendNonHostile( deck )
                self:AssignTarget( nil ) -- Ensures a new target is chosen each iteration.
                for i, card in ipairs(deck:GetCards()) do
                    if not CheckBits( card.flags, CARD_FLAGS.HOSTILE ) then
                        minigame:ExpendCard(card)
                        self.min_persuasion, self.max_persuasion = self.damage, self.damage
                        minigame:ApplyPersuasion( self )
                        self.min_persuasion, self.max_persuasion = nil, nil
                    end
                end
            end

            self.min_persuasion = 0 -- Temporarily makes this card an 'attack' card so that it respects Bait.
            ExpendNonHostile( minigame:GetDeck( DECK_TYPE.DISCARDS ) )
            self.min_persuasion = nil
        end,
    },
    scorched_earth_plus = 
    {
        name = "Pale Scorched Earth",
        cost = 1,
    },
    scorched_earth_plus2 = 
    {
        name = "Boosted Scorched Earth",
        desc = "{EXPEND} all non-Hostility cards in your discard pile.\nDeal <#UPGRADE>{1}</> damage to a random opponent argument for every card expended.{2}",
        damage = 3,
    },

    solid_point =
    {
        name = "Solid Point",
        flavour = "'A few more of those and I’m all yours.'",
        desc = "When this card is drawn, deal {1} damage to a random opponent argument and gain {INFLUENCE {2}}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.strength, self.influence )
        end,

        strength = 1,
        influence = 1,
        cost = 0,
        rarity = CARD_RARITY.COMMON,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.UNPLAYABLE,
        target_enemy = TARGET_ANY_RESOLVE,

        event_handlers =
        {
           [ EVENT.DRAW_CARD ] = function( self, minigame, card, start_of_turn )
                 if card == self then
                    self.engine:PushPostHandler( function()
                        self:NotifyTriggeredPre()
                        self.min_persuasion, self.max_persuasion = self.strength, self.strength
                        self.engine:ApplyPersuasion( self )
                        self.min_persuasion, self.max_persuasion = nil, nil
                        self.negotiator:AddModifier("INFLUENCE", self.influence, self)
                        self:AddXP(1)
                        self:NotifyTriggeredPost()
                    end )
                end
            end,
        }
    },

    solid_point_plus =
    {
        name = "Strong Point",
        desc = "When this card is drawn, deal <#UPGRADE>{1} damage</> to a random opponent argument and gain {INFLUENCE {2}}.",
        strength = 3,
    },

    solid_point_plus2 =
    {
        name = "Visionary Point",
        desc = "When this card is drawn, deal {1} damage to a random opponent argument and gain <#UPGRADE>{INFLUENCE 2}</>.",
        influence = 2,
    },  

    calm =
    {
        name = "Calm",
        flavour = "'Sorry boss, we just couldn’t rattle them.'",
        desc = "Apply {1} {COMPOSURE} to all friendly arguments.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText( self.composure_amt ))
        end,

        cost = 1,
        composure_amt= 2,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.DIPLOMACY,

        target_mod = TARGET_MOD.TEAM,
        target_self = TARGET_ANY_RESOLVE,
        auto_target = true,
        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
        end
    },
    calm_plus =
    {
        name = "Boosted Calm",
        desc = "Apply <#UPGRADE>{1} {COMPOSURE}</> to all friendly arguments.",
        composure_amt = 3,
    },
    calm_plus2 =
    {
        name = "Pale Calm",
        desc = "Apply {1} {COMPOSURE} to all friendly arguments.",
        cost = 0,
    },

    planning =
    {
        name = "Planning",
        flavour = "'Alright, make some room.'",
        desc = "Draw {1} card. Choose a card from your hand to place on top of your draw pile.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.draw_count )
        end,
        loc_strings =
        {
            CHOOSE_CARD = "Choose a card to place on your draw pile" 
        },

        cost = 0,
        draw_count = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.RARE,

        OnPostResolve = function( self, minigame )
            minigame:DrawCards( self.draw_count )
            local card = minigame:ChooseCard( nil, self.def:GetLocalizedString( "CHOOSE_CARD" ))
            if card then
                card:TransferCard( minigame:GetDrawDeck() )
            end
        end,
    },
    planning_plus =
    {
        name = "Boosted Planning",
        desc = "Draw <#UPGRADE>{1}</> cards. Choose a card from your hand to place on top of your draw pile.",
        draw_count = 2,
    },

    planning_plus2 =
    {
        name = "Improvised Planning",
        desc = "<#UPGRADE>{IMPROVISE}</> {1} card from your draw pile. Choose a card from your hand to place on top of your draw pile.",
        OnPostResolve = function( self, minigame )
            local cards = {}
            if minigame:GetDrawDeck():CountCards() == 0 then
                minigame:ShuffleDiscardToDraw()
            end

            for i, card in minigame:GetDrawDeck():Cards() do
                table.insert(cards, card)
            end
            cards = table.multipick( cards, 3 )
            minigame:ImproviseCards( cards, 1, nil, nil, nil, self )
            local card = minigame:ChooseCard( nil, self.def:GetLocalizedString( "CHOOSE_CARD" ))
            if card then
                card:TransferCard( minigame:GetDrawDeck() )
            end
        end,
    },

    know_it_all =
    {
        name = "Know It All",
        flavour = "'Um, Actually...'",
        desc = "Gain {SMARTS {1}} and {VULNERABILITY {2}}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.smarts_amt, self.vulnerability_stacks )
        end,

        cost = 1,
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE,
        vulnerability_stacks = 1,

        smarts_amt = 3,

        OnPostResolve = function( self, minigame, targets )
            if self.vulnerability_stacks > 0 then
                self.negotiator:AddModifier("VULNERABILITY", self.vulnerability_stacks, self)
            end
            self.negotiator:AddModifier("SMARTS", self.smarts_amt, self)
        end,
    },

    know_it_all_plus =
    {
        name = "Boosted Know it All",
        desc = "Gain <#UPGRADE>{SMARTS {1}}</> and {VULNERABILITY {2}}.",
        smarts_amt = 4,
    },

    know_it_all_plus2 =
    {
        name = "Purified Know it All",
        desc = "Gain {SMARTS {1}}.",
        vulnerability_stacks = 0,
    },

    just_the_facts =
    {
        name = "Just the Facts",
        flavour = "'Just get to the point already.'",
        desc = "{EXPEND} any number of cards.\nGain 1 {SMARTS} for each card expended.",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,

        flags = CARD_FLAGS.MANIPULATE,
        bonus = 0,

        OnPostResolve = function( self, minigame )
            local cards = minigame:ExpendCards( nil, nil, self )
            local count = #cards

            self.negotiator:AddModifier( "SMARTS", count + self.bonus, self )
        end,
    },

    just_the_facts_plus =
    {
        name = "The Boosted Facts",
        desc = "{EXPEND} any number of cards.\nGain 1 {SMARTS} for each card expended<#UPGRADE> + 1</>.",
        bonus = 1,
    },

    just_the_facts_plus2 =
    {
        name = "The Visionary Facts",
        desc = "{EXPEND} any number of cards.\nGain 1 {SMARTS} for each card expended.\n<#UPGRADE> Draw 1 card.</>",

        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:ExpendCards( nil, nil, self )
            local count = #cards
            
            self.negotiator:AddModifier( "SMARTS", count + self.bonus, self )

            minigame:DrawCards( 1 )
        end,
    },

    hard_facts =
    {
        name = "Hard Facts",
        flavour = "'You can’t fight these.'",
        desc = "Apply {1} {COMPOSURE} to a random argument. Increase by {2} for each card played this turn.\n{3}",
        desc_fn = function( self, fmt_str )
            if self.engine then
                local count = self.engine:CountCardsPlayed()
                local str = loc.format( LOC"CARD_ENGINE.CARDS_PLAYED", count )
                return loc.format( fmt_str, self:CalculateComposureText( self.composure + self.stacks * count ), self.stacks, str )
            else
                return loc.format( fmt_str, self:CalculateComposureText( self.composure ), self.stacks, "" )
            end
        end,

        mod_xp = -2,

        cost = 1,
        stacks = 1,
        composure = 2,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        target_self = TARGET_ANY_RESOLVE,
        auto_target = true,

        GainComposure = function( self, minigame, targets )
            local composure = self.composure + self.stacks * minigame:CountCardsPlayed()
            for i, target in ipairs( targets ) do
                target:DeltaComposure( composure, self )
            end
        end,

        OnPostResolve = function( self, minigame, targets )
            self:GainComposure( minigame, targets )
        end
    },
    hard_facts_plus =
    {
        name = "Boosted Facts",
        desc = "Apply {1} {COMPOSURE} to a random argument. Increase by <#UPGRADE>{2}</> for each card played this turn.\n{3}",
        stacks = 2,
    },

    hard_facts_plus2 =
    {
        name = "Focused Facts",
        desc = "Apply {1} {COMPOSURE}. Increase by {2} for each card played this turn.\n{3}",
        auto_target = false,
    },

    entrapment =
    {
        name = "Entrapment",
        flavour = "'I can’t believe I trusted you.'",
        desc = "Add 3 {BAIT}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, 3 + (self.resolve_bonus or 0) )
        end,

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,
        stacks = 3,

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier("BAIT", self.stacks, self)
        end,
    },
    entrapment_plus =
    {
        name = "Boosted Entrapment",
        desc = "Add <#UPGRADE>4</> {BAIT}.",
        stacks = 4,
    },
    entrapment_plus2 =
    {
        name = "Pale Entrapment",
        cost = 0,
    },

    white_lie =
    {
        name = "White Lie",
        flavour = "'Green is totally your color.'",
        desc = "Add a copy of this card to your discards.\nWhen drawn, {INCEPT} {DOUBT {1}}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.stacks )
        end,

        cost = 1,
        stacks = 1,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.REPLENISH,
        rarity = CARD_RARITY.UNCOMMON,

        OnPostResolve = function( self, minigame )
            local copy = self:Duplicate()
            minigame:DealCard( copy, minigame:GetDeck( DECK_TYPE.DISCARDS ))
        end,

        event_handlers = 
        {
            [ EVENT.DRAW_CARD ] = function( self, minigame, card, start_of_turn )
                if self == card then
                    self.engine:PushPostHandler( function()
                        self:NotifyTriggeredPre()
                        self.anti_negotiator:InceptModifier( "DOUBT", self.stacks, self )
                        self:NotifyTriggeredPost()
                    end )
                end
            end
        },
    },
    white_lie_plus =
    {
        name = "Boosted Lie",
        desc = "Add a copy of this card to your discards.\nWhen drawn, {INCEPT} <#UPGRADE>{DOUBT {1}}</>.",
        stacks = 2,
    },
    white_lie_plus2 =
    {
        name = "Pale Lie",
        cost = 0,
    },

    pure_style =
    {
        name = "Pure Style",
        flavour = "'Slick duds pal.'",
        desc = "Draw 2 cards and upgrade them for this negotiation.",
        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,
        draw = 2,

        OnPostResolve = function( self, minigame )
            local cards = minigame:DrawCards( self.draw )
            for i, card in ipairs( cards ) do
                card:UpgradeCard()
            end
        end,
    },
    pure_style_plus =
    {
        name = "Boosted Style",
        desc = "Draw <#UPGRADE>3</> cards and upgrade them for this negotiation.",
        draw = 3,
    },

    pure_style_plus2 =
    {
        name = "Sticky Style",
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.STICKY,
    },

    deceive =
    {
        name = "Deceive",
        flavour = "'Trust me.'",
        desc = "Each card in hand is assigned a random cost from 0 to 2 until the end of negotiation.",

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.RARE,

        cost = 1,
        max_cost = 2,

        OnPostResolve = function( self, minigame )
            local mod = self.negotiator:FindModifier( "deceive" )
            if not mod then
                mod = self.negotiator:AddModifier( "deceive", 1, self )
            end

            mod:AssignCosts( minigame, minigame:GetHandDeck(), self.until_played, self.max_cost )
            mod:AssignCosts( minigame, minigame:GetDeck( DECK_TYPE.RESOLVE ), self.until_played, self.max_cost )

            if self.all_cards then
                mod:AssignCosts( minigame, minigame:GetDiscardDeck(), self.until_played, self.max_cost )
                mod:AssignCosts( minigame, minigame:GetDrawDeck(), self.until_played, self.max_cost )
            end
        end,

        modifier =
        {
            hidden = true,
            max_stacks = 1,

            AssignCosts = function( self, minigame, deck, until_played, max_cost )
                if self.randomized_costs == nil then
                    self.randomized_costs = {}
                end

                for i, card in deck:Cards() do
                    if card.owner == self.owner then
                        self.randomized_costs[ card ] = math.random( 0, max_cost )

                        if until_played then
                            if self.until_played == nil then
                                self.until_played = {}
                            end
                            self.until_played[ card ] = true

                        elseif self.until_played then
                            self.until_played[ card ] = nil
                        end
                    end
                end
            end,

            event_priorities =
            {
                [ EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_PRESETTOR
            },

            event_handlers =
            {
                [ EVENT.CALC_ACTION_COST ] = function( self, cost_acc, card, target )
                    local cost = self.randomized_costs and self.randomized_costs[ card ]
                    if cost then
                        cost_acc:ModifyValue( cost, self )
                    end
                end,

                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    if self.until_played and self.until_played[ card ] then
                        self.randomized_costs[ card ] = nil
                        self.until_played[ card ] = nil
                    end
                end,
            }
        },
    },

    deceive_plus =
    {
        name = "Wide Deceive",
        desc = "<#UPGRADE>ALL</> your cards are assigned a random cost from 0 to 2 until the end of negotiation.",

        all_cards = true,
    },

    deceive_plus2 =
    {
        name = "Pale Deceive",
        desc = "Each card in hand is assigned a random cost from <#UPGRADE>0 to 1</><#DOWNGRADE> until played</>.",
        max_cost = 1,
        until_played = true,
    },

    exploit_weakness =
    {
        name = "Exploit Weakness",
        flavour = "'Okay Okay, let’s not bring Kashio into this.'",
        desc = "Duplicate target bounty or inception.",

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        target_enemy = TARGET_FLAG.BOUNTY | TARGET_FLAG.INCEPTION,

        mod_xp = -1,

        cost = 1,

        OnPostResolve = function( self, minigame, targets )
            for i, target in ipairs( targets ) do
                target:Duplicate()
            end
        end,
    },

    exploit_weakness_plus =
    {
        name = "Pale Weakness",
        cost = 0,
    },

    exploit_weakness_plus2 =
    {
        name = "Enduring Weakness",
        flags = CARD_FLAGS.MANIPULATE,
    },

    drain_resolve =
    {
        name = "Drain Resolve",
        flavour = "'The conversation was quite energizing. Thank you.'",
        desc = "Target argument loses {1} resolve. You gain {2} resolve.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.resolve_loss, self.resolve_gain )
        end,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        target_self = TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY,
        target_enemy = TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY | TARGET_FLAG.CORE,

        cost = 1,
        resolve_loss = 2,
        resolve_gain = 2,

        CanPlayCard = function( self, card, engine, target )
            if card == self and target then
                if target:GetResolve() == nil then
                    return false, CARD_PLAY_REASONS.NO_TARGET_RESOLVE
                end
            end
            return true
        end,

        OnPostResolve = function( self, minigame, targets )
            for i, target in ipairs( targets ) do
                local resolve = target:GetResolve() or 0
                resolve = math.min( self.resolve_loss, resolve )
                target:ModifyResolve( -resolve, self )
                self.negotiator:ModifyResolve( self.resolve_gain, self )
            end
        end,
    },
    drain_resolve_plus =
    {
        name = "Boosted Drain",
        desc = "Target argument loses <#UPGRADE>{1}</> resolve. You gain {2} resolve.",
        resolve_loss = 4,
    },
    drain_resolve_plus2 =
    {
        name = "Mending Drain",
        desc = "Target argument loses {1} resolve. You gain <#UPGRADE>{2}</> resolve.",
        resolve_gain = 4,
    },

}

for i, id, carddef in sorted_pairs( CARDS ) do
    carddef.series = "SAL"

    Content.AddNegotiationCard( id, carddef )
end

