local negotiation_defs = require "negotiation/negotiation_defs"
local CARD_FLAGS = negotiation_defs.CARD_FLAGS

local EVENT = ExtendEnum( negotiation_defs.EVENT,
{
})

------------------------------------------------------------------

local function Confuse( self, minigame )
    local card = Negotiation.Card("confused", self.negotiator.agent)
    minigame:DealCard(card, minigame:GetDiscardDeck())
end

local CARDS =
{
    confused = 
    {
        name = "Confused",
        desc = "Draw a card.",

        cost = 1,

        hide_in_cardex = true,

        flags = CARD_FLAGS.STATUS,
        rarity = CARD_RARITY.BASIC,

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(1)
        end
    },

    nepotism =
    {
        name = "Nepotism",
        desc = "Insert {card.self_promotion} or {clout} into your hand.",
        flavour = "'Well that’s just unfair.'",
 
        cost = 1,
 
        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.BASIC,
 
        OnPostResolve = function( self, minigame, targets )
            local cards = {
                Negotiation.Card( "self_promotion", self.owner ),
                Negotiation.Card( "clout", self.owner ),
            }
 
            minigame:ImproviseCards( cards, 1, nil, nil, nil, self )
        end
    },
 
    self_promotion =
    {
        name = "Self Promotion",
        desc = "Gain {INFLUENCE {1}}. At the end of your turn, lose {1} {INFLUENCE}.",
        flavour = "'Smith and the Smithereens! You’ve never heard of us?'",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.stacks)
        end,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        cost = 0,
        stacks = 1,

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "INFLUENCE", self.stacks, self )
            self.negotiator:AddModifier( "self_promotion", self.stacks, self )
        end,

        modifier =
        {
            max_resolve = 2,

            sound = "event:/sfx/battle/cards/neg/create_argument/setup",

            desc = "At the end of your turn, lose {1} {INFLUENCE}, then remove this argument.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks or 1)
            end,

            OnEndTurn = function(self, minigame)
                local stacks = self.negotiator:GetModifierStacks("INFLUENCE")
                self.negotiator:RemoveModifier("INFLUENCE", self.stacks, self)
                self.negotiator:RemoveModifier( self )
            end,
        }
    },
 
    clout = 
    {
        name = "Clout",
        desc = "Play this card twice.",
        flavour = "'Did you see what he’s wearing? It’s terrible!'",
 
        cost = 0,
 

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_persuasion = 1,
        max_persuasion = 1,
        OnPostResolve = function( self, minigame, targets )
            if not self.ignore then
                self.ignore = true
                minigame:PlayCard(self)
            else
                self.ignore = false
            end
        end
    },


    nepotism_plus =
    {
        name = "Courtly Nepotism",
        desc = "Insert {buffer} or {compel} into your hand.",

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.BASIC,

        OnPostResolve = function( self, minigame, targets )
            local cards = {
                Negotiation.Card( "buffer", self.owner ),
                Negotiation.Card( "compel", self.owner ),
            }

            minigame:ImproviseCards( cards, 1, nil, nil, nil, self )
        end
    },

    buffer =
    {
        name = "Buffer",
        desc = "Gain {INFLUENCE {1}}.",
        flavour = "'A few seconds put aside for Hesh, a small but important consideration.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.influence_amt)
        end,

        cost = 0,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        influence_amt = 2,

        OnPostResolve = function( self, minigame, targets )
            local mod = self.negotiator:AddModifier("INFLUENCE", self.influence_amt, self)
        end
    },

    compel = 
    {
        name = "Compel",
        desc = "{RENOWNED} {1}: This card deals double damage and hits all opponent arguments.",
        flavour = "'I knew that you’d see things our way eventually.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.renown_amt or 3)
        end,

        cost = 0,

        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_persuasion = 2,
        max_persuasion = 3,

        renown_amt = 3,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt
        end,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
        },

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self then
                    if self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt then
                        persuasion:AddPersuasion( persuasion.min_persuasion, persuasion.max_persuasion, self )
                        self.auto_target = true
                    else
                        self.auto_target = false
                    end
                end
            end,

            [ EVENT.START_RESOLVE ] = function( self, minigame, card )
                if card == self and self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt then
                    card.target_mod = TARGET_MOD.TEAM
                end
            end
        },
    },

    nepotism_plus2 =
    {
        name = "Degrading Nepotism",
        desc = "Insert {card.notion} or {overturn} into your hand.",

        cost = 1,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.BASIC,

        OnPostResolve = function( self, minigame, targets )
            local cards = {
                Negotiation.Card( "notion", self.owner ),
                Negotiation.Card( "overturn", self.owner ),
            }

            minigame:ImproviseCards( cards, 1, nil, nil, nil, self )
        end
    },

    notion =
    {
        name = "Notion",
        desc = "Gain 2 {DOMINANCE}. At the end of your turn, lose 2 {DOMINANCE}.",
        flavour = "'Listen up.'",

        cost = 0,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,
        stacks  = 2,

        OnPostResolve = function( self, minigame )
            self.negotiator:AddModifier( "DOMINANCE", self.stacks, self )
            self.negotiator:AddModifier( "notion", self.stacks, self )
        end,

        modifier =
        {
            max_resolve = 2,

            sound = "event:/sfx/battle/cards/neg/create_argument/setup",

            desc = "At the end of your turn, lose {1} {DOMINANCE}, then remove this argument.",
            desc_fn = function(self, fmt_str)
                return loc.format(fmt_str, self.stacks or 1)
            end,

            OnEndTurn = function(self, minigame)
                local stacks = self.negotiator:GetModifierStacks("DOMINANCE")
                self.negotiator:RemoveModifier("DOMINANCE", self.stacks, self)
                self.negotiator:RemoveModifier( self )
            end,
        }
    },

    overturn =
    {
        name = "Overturn",
        desc = "Play this card twice.",
        flavour = "'I’m going to reiterate.'",
 
        cost = 0,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNIQUE,

        min_persuasion = 2,
        max_persuasion = 2,
        OnPostResolve = function( self, minigame, targets )
            if not self.ignore then
                self.ignore = true
                minigame:PlayCard(self)
            else
                self.ignore = false
            end
        end
    },

    brag = 
    {
        name = "Brag",
        desc = "{RENOWNED} {1}: increase the maximum damage of this card by {2}.",
        flavour = "'I don’t mean to come off too cocky but…'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.renown_amt, self.damage_bonus)
        end,
        cost = 1,
        min_persuasion = 1,
        max_persuasion = 3,
        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.BASIC,
        max_xp = 6,
        wild = true,

        renown_amt = 1,
        damage_bonus = 2,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_MULTIPLIER,
        },

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt then
                    persuasion:AddPersuasion( 0, self.damage_bonus, self )
                end
            end,
        },
    },

    brag_plus2a =
    {
        name = "Rooted Brag",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    brag_plus2b = 
    {
        name = "Tall Brag",
        max_persuasion = 5,
    },

    brag_plus2c =
    {
        name = "Lucid Brag",
        desc = "{RENOWNED} {1}: increase the maximum damage of this card by {2}.",
        min_persuasion = 5,
        max_persuasion = 5,
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,
    },

    brag_plus2d =
    {
        name = "Visionary Brag",
        desc = "<#UPGRADE>Draw a card.</>\n{RENOWNED} {1}: increase the maximum damage of this card by {2}.",
        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(1)
        end
    },

    brag_plus2e =
    {
        name = "Famous Brag",
        desc = "{RENOWNED} {1}: increase the maximum damage of this card by {2}.\n<#UPGRADE>Gain 1 {RENOWN}.</>",
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RENOWN", 1, self)
        end
    },

    brag_plus2f =
    {
        name = "Brag of Clarity",
        desc = "{RENOWNED} {1}: increase the maximum damage of this card by {2}.",
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.CONSUME,
        min_persuasion = 7,
        max_persuasion = 7
    },

    bewilder = 
    {
        name = "Bewilder",
        desc = "Apply {1} {COMPOSURE}.",
        flavour = "'Wait, give me a second here.'",
        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self:CalculateComposureText( self.composure_amt ))
        end,

        max_xp = 6,
        wild = true,

        cost = 1,
        composure_amt = 3,
        
        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.BASIC,

        target_self = TARGET_ANY_RESOLVE,
        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
        end
    },

    bewilder_plus2a =
    {
        name = "Stone Bewilder",
        desc = "Apply <#UPGRADE>{1}</> {COMPOSURE}.",
        composure_amt = 5,
    },

    bewilder_plus2b = 
    {
        name = "Visionary Bewilder",
        desc = "Apply {1} {COMPOSURE}.\n<#UPGRADE>Draw a card.</>",
        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
            minigame:DrawCards(1)
        end
    },

    bewilder_plus2c =
    {
        name = "Lucid Bewilder",
        desc = "Apply <#UPGRADE>{1} {COMPOSURE}</>.",
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MANIPULATE,
        composure_amt = 7
    },

    bewilder_plus2d =
    {
        name = "Bewilder of Clarity",
        desc = "Apply <#UPGRADE>{1} {COMPOSURE}</>.",
        flags = CARD_FLAGS.CONSUME | CARD_FLAGS.MANIPULATE,
        composure_amt = 10
    },

    bewilder_plus2e =
    {
        name = "Wide Bewilder",
        desc = "Apply <#DOWNGRADE>{1}</> {COMPOSURE} <#UPGRADE>to all friendly arguments</>.",
        target_mod = TARGET_MOD.TEAM,
        auto_target = true,
        composure_amt = 2,
    },

    bewilder_plus2f =
    {
        name = "Famous Bewilder",
        desc = "Apply {1} {COMPOSURE} <#UPGRADE>and gain 1 {RENOWN}</>.",
        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
            self.negotiator:AddModifier("RENOWN", 1, self)
        end
    },

    bully =
    {
        name = "Bully",
        desc = "{BRAVADO}.",
        flavour = "'Come here you.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.bonus_damage)
        end,
        
        cost = 1,
        min_persuasion = 1,
        max_persuasion = 2,

        max_xp = 6,
        wild = true,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.BASIC,

        deck_handlers = ALL_DECKS,

        bonus_damage = 1,

        event_handlers = 
        {
            [ EVENT.END_RESOLVE ] = function( self, minigame, card )
                if card.negotiator == self.negotiator then
                    if CheckBits(card.flags, CARD_FLAGS.HOSTILE) then
                        self.bonus = (self.bonus or 0) + self.bonus_damage
                    else
                        self.bonus = 0
                    end
                end
            end,

            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.bonus and self.bonus > 0 then
                    persuasion:AddPersuasion(self.bonus, self.bonus, self)
                end
            end,

            [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                self.bonus = 0
            end,
        },
    },

    bully_plus2a = 
    {
        name = "Rooted Bully",
        min_persuasion = 2,
    },

    bully_plus2b =
    {
        name = "Tall Bully",
        max_persuasion = 3,
    },

    bully_plus2c =
    {
        name = "Lucid Bully",
        desc = "{BRAVADO}.",
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        min_persuasion = 5,
        max_persuasion = 5,
    },

    bully_plus2d =
    {
        name = "Bully of Clarity",
        desc = "{BRAVADO}.",
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.CONSUME,
        min_persuasion = 8,
        max_persuasion = 8,
    },

    bully_plus2e =
    {
        name = "Mirrored Bully",
        desc = "{BRAVADO}.\n<#UPGRADE>Attack twice with this card.</>",
        OnPostResolve = function( self, minigame )
            minigame:ApplyPersuasion( self )
        end,
    },

    bully_plus2f =
    {
        name = "Visionary Bully",
        desc = "{BRAVADO}.\n<#UPGRADE>Draw a card.</>",
        OnPostResolve = function( self, minigame )
            minigame:DrawCards(1)
        end,
    },

    name_drop =
    {
        name = "Name Drop",
        desc = "Gain {1} {RENOWN}, draw a card.",
        flavour = "'Oh yeah, we go way back.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.renown_amt, self.composure_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.BASIC,

        renown_amt = 2,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RENOWN", self.renown_amt, self)
            minigame:DrawCards(1)
        end
    },

    name_drop_plus =
    {
        name = "Prestigious Name Drop",
        desc = "Gain <#UPGRADE>{1}</> {RENOWN}, draw a card.",
        renown_amt = 3,
    },

    name_drop_plus2 =
    {
        name = "Stone Name Drop",
        desc = "Gain {1} {RENOWN}, apply <#UPGRADE>{2} {COMPOSURE} to it.</>",
        composure_amt = 4,
        OnPostResolve = function( self, minigame, targets )
            local prom = self.negotiator:AddModifier("RENOWN", self.renown_amt, self)
            if prom then
                prom:DeltaComposure(self.composure_amt, self)
            end
        end
    },

    standing =
    {
        name = "Standing",
        desc = "Apply {1} {COMPOSURE}.\n{RENOWNED} {2}: apply {3} bonus {COMPOSURE}.",
        flavour = "'Looking good there Smith.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText(self.composure_amt), self.renown_amt, self.bonus_comp)
        end,

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MANIPULATE,

        composure_amt = 3,
        renown_amt = 3,
        bonus_comp = 3,

        target_self = TARGET_ANY_RESOLVE,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt
        end,

        OnPostResolve = function( self, minigame, targets )
            for i, target in ipairs(targets) do
                
                if self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt then
                    target:DeltaComposure(self.composure_amt + self.bonus_comp, self)
                else
                    target:DeltaComposure(self.composure_amt, self)
                end
            end
        end,
    },

    standing_plus =
    {
        name = "Stone Standing",
        desc = "Apply {1} {COMPOSURE}.\n{RENOWNED} {2}: apply <#UPGRADE>{3}</> bonus {COMPOSURE}.",
        bonus_comp = 5,
    },

    standing_plus2 =
    {
        name = "Pale Standing",
        desc = "Apply {1} {COMPOSURE}.\n<#UPGRADE>{RENOWNED} {2}</>: apply {3} bonus {COMPOSURE}.",
        renown_amt = 2,
    },

    -- connections = 
    -- {
    --     name = "Connections",
    --     desc = "{BLIND}.\nSpend up to {1} {RENOWN}: Target 1 additional target per 2 {RENOWN} spent.",
    --     desc_fn = function( self, fmt_str )
    --         return loc.format(fmt_str, self.prominence_cap)
    --     end,

    --     cost = 2,

    --     flags = CARD_FLAGS.HOSTILE,
    --     rarity = CARD_RARITY.UNCOMMON,

    --     target_mod = TARGET_MOD.CUSTOM,
    --     auto_target = true,

    --     PreReq = function( self, minigame )
    --         return self.negotiator:GetModifierStacks("RENOWN") >= 2
    --     end,

    --     prominence_cap = 4,

    --     min_persuasion = 2,
    --     max_persuasion = 4,

    --     num_targets = 1,

    --     target_fn = function( self, minigame, primary_target, targets, source )
    --         if source == self then
    --             local candidates = minigame:CollectAlliedTargets( primary_target )
    --             for i=1, self.num_targets do
    --                 table.insert(targets, candidates[math.random(#candidates)])
    --             end
    --         end
    --     end,

    --     event_handlers = 
    --     {
    --         [ EVENT.START_RESOLVE ] = function( self, minigame, card )
    --             if card == self then
    --                 local count = math.min(self.negotiator:GetModifierStacks("RENOWN"), self.prominence_cap)
    --                 self.num_targets = 1
    --                 for i=1, math.floor(count * 0.5) do
    --                     self.num_targets = self.num_targets + 1
    --                     self.negotiator:RemoveModifier("RENOWN", 2, self)
    --                 end
    --             end
    --         end
    --     },
    -- },

    -- connections_plus =
    -- {
    --     name = "Rooted Connections",
    --     min_persuasion = 4,
    -- },

    -- connections_plus2 =
    -- {
    --     name = "Boosted Connections",
    --     desc = "{BLIND}.\nSpend up to <#UPGRADE>{1}</> {RENOWN}: Target 1 additional target per 2 {RENOWN} spent.",
    --     prominence_cap = 6,
    -- },

    stool_pigeon =
    {
        name = "Stool Pigeon",
        desc = "Spend up to {1} {RENOWN}: Create a {BAIT} argument with 2 maximum resolve per {RENOWN} spent.",
        flavour = "'We know what you did, you better sing!'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.prominence_cap)
        end,

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        PreReq = function( self, minigame )
            return self.negotiator:HasModifier("RENOWN")
        end,

        CanPlayCard = function( self, card, engine, target )
            if card == self then
                return self.negotiator:HasModifier("RENOWN")
            end
        end,

        prominence_cap = 2,
        resolve_amt = 2,

        auto_target = true,
        target_mod = TARGET_MOD.TEAM,
        target_self = TARGET_ANY_RESOLVE,

        OnPostResolve = function( self, minigame, targets )
            local count = math.min(self.negotiator:GetModifierStacks("RENOWN"), self.prominence_cap)
            if count > 0 then
                self.negotiator:AddModifier("BAIT", count, self)
                self.negotiator:RemoveModifier("RENOWN", count, self)
            end
        end
    },

    stool_pigeon_plus =
    {
        name = "Boosted Stool Pigeon",
        desc = "Spend up to <#UPGRADE>{1}</> {RENOWN}: Create a {BAIT} argument with 2 maximum resolve per {RENOWN} spent.",
        prominence_cap = 3,
    },

    stool_pigeon_plus2 =
    {
        name = "Enduring Stool Pigeon",
        flags = CARD_FLAGS.MANIPULATE,
    },

    cachet =
    {
        name = "Cachet",
        desc = "{IMPROVISE} a card from your discard pile. Gain {RENOWN} equal to twice its cost.",
        flavour = "'Told you that I was good for it.'",

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,

        improvise_count = 1,
        pool_size = 3,

        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:GetDiscardDeck().cards
            local chosen = minigame:ImproviseCards(table.multipick(cards, self.pool_size), self.improvise_count, nil, "ad_lib", nil, self)
            for i,v in ipairs(chosen) do
                self.negotiator:AddModifier("RENOWN", minigame:CalculateActionCost(v) * 2, self)
            end
        end
    },

    cachet_plus =
    {
        name = "Wide Cachet",
        desc = "{IMPROVISE} <#UPGRADE>2 cards</> from your discard pile. Gain {RENOWN} equal to their combined cost.",
        improvise_count = 2,
        pool_size = 5,
        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:GetDiscardDeck().cards
            local chosen = minigame:ImproviseCards(table.multipick(cards, self.pool_size), self.improvise_count, nil, "ad_lib", nil, self)
            for i,v in ipairs(chosen) do
                self.negotiator:AddModifier("RENOWN", minigame:CalculateActionCost(v), self)
            end
        end
    },

    cachet_plus2 =
    {
        name = "Twisted Cachet",
        desc = "{IMPROVISE} a card from your <#UPGRADE>draw</> pile. Gain {RENOWN} equal to twice its cost.",
        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:GetDrawDeck().cards
            local chosen = minigame:ImproviseCards(table.multipick(cards, self.pool_size), self.improvise_count, nil, "ad_lib", nil, self)
            for i,v in ipairs(chosen) do
                self.negotiator:AddModifier("RENOWN", minigame:CalculateActionCost(v) * 2, self)
            end
        end
    },

    immunity = 
    {
        name = "Immunity",
        desc = "Spend up to {1} {RENOWN}: Apply {2} {COMPOSURE} per {RENOWN} spent to all friendly arguments.",
        flavour = "'Good luck getting that flimsy case to stick buddy.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.prominence_cap, self.composure_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.RARE,

        PreReq = function( self, minigame )
            return self.negotiator:HasModifier("RENOWN")
        end,

        prominence_cap = 4,
        composure_amt = 1,

        auto_target = true,
        target_mod = TARGET_MOD.TEAM,
        target_self = TARGET_ANY_RESOLVE,

        OnPostResolve = function( self, minigame, targets )
            local count = math.min(self.negotiator:GetModifierStacks("RENOWN"), self.prominence_cap)
            if count > 0 then
                for i,target in ipairs(targets) do
                    target:DeltaComposure(count * self.composure_amt, self)
                end
                self.negotiator:RemoveModifier("RENOWN", count, self)
            end
        end
    },

    immunity_plus =
    {
        name = "Stone Immunity",
        desc = "Spend up to {1} {RENOWN}: Apply <#UPGRADE>{2}</> {COMPOSURE} per {RENOWN} spent to all friendly arguments.",
        composure_amt = 2,
    },

    immunity_plus2 =
    {
        name = "Enduring Immunity",
        flags = CARD_FLAGS.HOSTILE,
    },

    know_a_guy =
    {
        name = "Know a Guy",
        desc = "Gain {1} {RENOWN}. At the end of your turn remove all your {RENOWN} and apply that much {COMPOSURE} to all your arguments.",
        flavour = "'Yeah they'll fix you right up. No problem, just mention my name.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.renown_amt)
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        renown_amt = 2,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RENOWN", self.renown_amt, self)
            local mod = self.negotiator:AddModifier("know_a_guy", 1, self)
            mod.source = self
        end,

        modifier =
        {
            desc = "At the end of {1}'s turn, remove all {1}'s {RENOWN} and apply that much {COMPOSURE} to all {1}'s arguments.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.negotiator:GetName())
            end,

            max_resolve = 5,

            event_handlers = 
            {
                [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        local stacks = self.negotiator:GetModifierStacks("RENOWN")
                        if stacks and stacks > 0 then
                            self.negotiator:RemoveModifier("RENOWN", stacks, self.source or self)
                            for i,arg in self.negotiator:Modifiers() do
                                if arg ~= self then
                                    arg:DeltaComposure(stacks, self)
                                end
                            end
                        end
                        self.negotiator:RemoveModifier(self)
                    end
                end
            }
        },
    },

    know_a_guy_plus =
    {
        name = "Boosted Know a Guy",
        desc = "Gain <#UPGRADE>{1} {RENOWN}</>. At the end of your turn remove all your {RENOWN} and apply that much {COMPOSURE} to all your arguments.",
        renown_amt = 4,
    },

    know_a_guy_plus2 =
    {
        name = "Twisted Know a Guy",
        desc = "Gain {1} {RENOWN}. At the end of your turn remove all your {RENOWN} and <#UPGRADE>deal that much damage to the opponent's core argument.</>",
        attack = true,
        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RENOWN", self.renown_amt, self)
            self.negotiator:AddModifier("know_a_guy_plus2", 1, self)
        end,
        modifier =
        {
            desc = "At the end of {1}'s turn, remove all {1}'s {RENOWN} and deal that much damage to the opponent's core argument.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.negotiator:GetName())
            end,

            max_resolve = 5,

            event_handlers = 
            {
                [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        local stacks = self.negotiator:GetModifierStacks("RENOWN")
                        self.negotiator:RemoveModifier("RENOWN", stacks, self)
                        minigame:ApplyPersuasion(self, self.anti_negotiator, stacks, stacks)
                        self.negotiator:RemoveModifier(self)
                    end
                end
            }
        },
    },

    contacts = 
    {
        name = "Contacts",
        desc = "Gain {RENOWN} equal to damage dealt by this card.",
        flavour = "'Some of these are even good people.'",

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        min_persuasion = 2,
        max_persuasion = 4,

        event_handlers =
        {
            [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                if source == self and damage > 0 then
                    self.negotiator:AddModifier("RENOWN", damage, self)
                end
            end
        },
    },

    contacts_plus =
    {
        name = "Rooted Contacts",
        min_persuasion = 4,
    },

    contacts_plus2 =
    {
        name = "Tall Contacts",
        max_persuasion = 6,
    },

    influencer =
    {
        name = "Influencer",
        desc = "{RENOWNED} {1}: this deals {2} bonus damage.",
        flavour = "'You’re tuned into Radio Smith! All Smith, all day!'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.renown_amt, self.bonus_damage)
        end,

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.DIPLOMACY,

        min_persuasion = 2,
        max_persuasion = 3,

        renown_amt = 2,
        bonus_damage = 3,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt
        end,

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt then
                    persuasion:AddPersuasion(self.bonus_damage, self.bonus_damage, self)
                end
            end
        },
    },

    influencer_plus =
    {
        name = "Pale Influencer",
        desc = "{RENOWNED} <#UPGRADE>{1}</>: this deals {2} bonus damage.",
        renown_amt = 1,
    },

    influencer_plus2 = 
    {
        name = "Boosted Influencer",
        desc = "{RENOWNED} {1}: this deals <#UPGRADE>{2}</> bonus damage.",
        bonus_damage = 5,
    },

    networker =
    {
        name = "Networker",
        desc = "{RENOWNED} {1}: Draw {2} {2*card|cards}.",
        flavour = "'Yeah it’s going to be big, no plus ones though.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.renown_amt, self.draw_count)
        end,

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.DIPLOMACY,

        min_persuasion = 4,
        max_persuasion = 4,

        renown_amt = 2,
        draw_count = 2,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt
        end,

        OnPostResolve = function( self, minigame, targets )
            if self.negotiator:GetModifierStacks("RENOWN") >= self.renown_amt then
                minigame:DrawCards(self.draw_count)
            end
        end
    },

    networker_plus =
    {
        name = "Visionary Networker",
        desc = "{RENOWNED} {1}: Draw <#UPGRADE>{2}</> {2*card|cards}.",
        draw_count = 3,
    },

    networker_plus2 =
    {
        name = "Pale Networker",
        desc = "{RENOWNED} <#UPGRADE>{1}</>: Draw {2} {2*card|cards}.",
        renown_amt = 1,
    },

    high_places =
    {
        name = "High Places",
        desc = "{RENOWNED} {1}: This card costs 2.\n{RENOWNED} {2}: This card costs 0.",
        flavour = "'Look, I know a lot of very important people, the most important people.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.first_level, self.second_level)
        end,

        cost = 4,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.DIPLOMACY,

        min_persuasion = 8,
        max_persuasion = 10,

        first_level = 10,
        second_level = 15,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("RENOWN") >= self.first_level
        end,

        event_priorities =
        {
            [ EVENT.CALC_ACTION_COST ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers =
        {
            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card == self then
                    if self.negotiator:GetModifierStacks("RENOWN") >= self.second_level then
                        acc:ModifyValue(0)
                    elseif self.negotiator:GetModifierStacks("RENOWN") >= self.first_level then
                        acc:ModifyValue(2)
                    end
                end
            end
        },
    },

    high_places_plus =
    {
        name = "Pale High Places",
        desc = "{RENOWNED} <#UPGRADE>{1}</>: This card costs 2.\n{RENOWNED} <#UPGRADE>{2}</>: This card costs 0.",
        first_level = 8,
        second_level = 12,
    },

    high_places_plus2 =
    {
        name = "Boosted High Places",
        min_persuasion = 10,
        max_persuasion = 12,
    },

    executive =
    {
        name = "Executive",
        desc = "{RENOWNED} {1}: This card deals double damage.\n{RENOWNED} {2}: This card deals triple damage.",
        flavour = "'And this here is your office, right by the window.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.first_level, self.second_level)
        end,

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.DIPLOMACY,

        first_level = 5,
        second_level = 10,

        min_persuasion = 3,
        max_persuasion = 4,

        PreReq = function( self, minigame )
            return self.negotiator:GetModifierStacks("RENOWN") >= self.first_level
        end,

        event_priorities = 
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_MULTIPLIER,
        },

        event_handlers = 
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self then
                    if self.negotiator:GetModifierStacks("RENOWN") >= self.second_level then
                        persuasion:AddPersuasion(2 * persuasion.min_persuasion, 2 * persuasion.max_persuasion, self)
                    elseif self.negotiator:GetModifierStacks("RENOWN") >= self.first_level then
                        persuasion:AddPersuasion(persuasion.min_persuasion, persuasion.max_persuasion, self)
                    end
                end
            end
        },
    },

    executive_plus =
    {
        name = "Pale Executive",
        desc = "{RENOWNED} <#UPGRADE>{1}</>: This card deals double damage.\n{RENOWNED} <#UPGRADE>{2}</>: This card deals triple damage.",
        first_level = 4,
        second_level = 8,
    },

    executive_plus2 =
    {
        name = "Boosted Executive",
        min_persuasion = 4,
        max_persuasion = 5,
    },

    who =
    {
        name = "Who?",
        desc = "Double your {RENOWN}. At the end of your turn, lose all {RENOWN}.",
        flavour = "'...Me?'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE,

        PreReq = function( self, minigame )
            return self.negotiator:HasModifier("RENOWN")
        end,

        base_gain = 0,

        OnPostResolve = function( self, minigame, targets )
            if self.base_gain > 0 then
                self.negotiator:AddModifier("RENOWN", self.base_gain, self)
            end
            local count = self.negotiator:GetModifierStacks("RENOWN")
            if count > 0 then
                for i,mod in self.negotiator:Modifiers() do
                    if mod.id == "RENOWN" then
                        self.negotiator:AddModifier(mod, mod.stacks, self)
                    end
                end
                self.negotiator:AddModifier("unknown", 1, self)
            end
        end,
    },

    who_plus =
    {
        name = "Boosted Who?",
        desc = "<#UPGRADE>Gain 2 {RENOWN}, then</> double your {RENOWN}. At the end of your turn, lose all {RENOWN}.",
        base_gain = 2,
    },

    who_plus2 = 
    {
        name = "Pale Who?",
        cost = 0,
    },

    associates =
    {
        name = "Associates",
        desc = "Gain 2 {RENOWN} for every person that loves you.{1}",
        desc_fn = function( self, fmt_str )
            local rel = self.negotiator and self.negotiator.agent.social_connections
            if rel then
                local how_many = loc.format( self.def:GetLocalizedString("HOW_MANY"), rel:GetNumberOfConnections(RELATIONSHIP.LOVED) * 2)
                return loc.format( fmt_str, how_many )
            else
                return loc.format( fmt_str, "" )
            end
        end,
        loc_strings =
        {
            HOW_MANY = " ({1} {RENOWN})",
        },
        flavour = "'Hold on, I’m pretty sure I have her card here somewhere.'",
        cost = 1,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        bonus = 0,

        OnPostResolve = function( self, minigame, targets )
            local rel = self.negotiator.agent.social_connections
            local loved = 0
            if rel then
                loved = rel:GetNumberOfConnections(RELATIONSHIP.LOVED) * 2  
            end
            if loved + self.bonus > 0 then
                self.negotiator:AddModifier("RENOWN", loved + self.bonus, self)
            end
        end
    },

    associates_plus =
    {
        name = "Boosted Associates",
        desc = "Gain 2 {RENOWN} for every person that loves you <#UPGRADE>+ 3 {RENOWN}</>.",
        bonus = 3,
    },

    associates_plus2 =
    {
        name = "Pale Associates",
        cost = 0,
    },

    networked = 
    {
        name = "Networked",
        desc = "Gain {1} {RENOWN} for every card in your hand.",
        flavour = "'People are looking at your profile!'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.renown_amt)
        end,

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE,

        renown_amt = 1,
        bonus = 0,
        draw_count = 0,

        OnPostResolve = function( self, minigame, targets )
            if self.draw_count > 0 then
                minigame:DrawCards(self.draw_count)
            end
            local count = minigame:GetHandDeck():CountCards()
            if count > 0 then
                self.negotiator:AddModifier("RENOWN", count * self.renown_amt + self.bonus, self)
            end
        end
    },

    networked_plus =
    {
        name = "Boosted Networked",
        desc = "Gain {1} {RENOWN} for every card in your hand<#UPGRADE> + 2</>.",
        bonus = 2,
    },

    networked_plus2 =
    {
        name = "Visionary Networked",
        desc = "<#UPGRADE>Draw a card.</>\nGain {1} {RENOWN} for every card in your hand.",
        draw_count = 1,
    },

    go_between = 
    {
        name = "Go Between",
        desc = "{go_between|}Gain: Damage spills over to this argument before your core argument.",
        flavour = "'Come on, you’re my guy! It’ll be worth it!'",

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("go_between", 1, self)
        end,

        modifier =
        {
            desc = "Damage spills over to this argument before your core argument.\nThis argument gains 2 bonus resolve per additional stack.",

            max_resolve = 4,

            OnSetStacks = function( self, old_stacks )
                if self.stacks > math.max(old_stacks,1) then
                    self:ModifyResolve( (self.stacks - old_stacks) * 2, self)
                end
            end,
            event_handlers =
            {
                [ EVENT.SPLASH_RESOLVE ] = function( self, modifier, overflow, params )
                    if modifier.owner == self.owner and params.splashed_modifier and not self:GetShieldStatus() then
                        params.splashed_modifier = self
                    end
                end,
            }   
        }
    },

    go_between_plus =
    {
        name = "Pale Go Between",
        cost = 0,
    },

    go_between_plus2 =
    {
        name = "Enduring Go Between",
        flags = CARD_FLAGS.MANIPULATE,
    },

    ad_hominem =
    {
        name = "Ad Hominem",
        desc = "{ad_hominem {1}|}Gain: At the end of your turn, spend 1 {RENOWN} and apply {1} {COMPOSURE} to all your arguments.",
        flavour = "'Rook? That Fink!?'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.ad_hominem_stacks )
        end,

        cost = 1,
        ad_hominem_stacks = 2,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("ad_hominem", self.ad_hominem_stacks, self)
        end,

        modifier =
        {
            desc = "At the end of your turn, spend 1 {RENOWN} and apply <#HILITE>{1}</> {COMPOSURE} to all your arguments.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks or 1)
            end,

            max_resolve = 4,

            event_handlers = 
            {
                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    if self.negotiator:GetModifierStacks("RENOWN") >= 1 then
                        self.negotiator:RemoveModifier("RENOWN", 1, self)
                        for i,arg in self.negotiator:Modifiers() do
                            arg:DeltaComposure(self.stacks, self)
                        end
                    end
                end
            },
        }
    },

    ad_hominem_plus =
    {
        name = "Boosted Ad Hominem",
        desc = "{ad_hominem {1}|}Gain: At the end of your turn, spend 1 {RENOWN} and apply <#UPGRADE>{1} {COMPOSURE}</> to all your arguments.",
        ad_hominem_stacks = 3,    
    },

    ad_hominem_plus2 =
    {
        name = "Pale Ad Hominem",
        cost = 0,
    },

    slick =
    {
        name = "Subtle Setup",
        desc = "{subtle_setup|}Create: Whenever this argument takes damage, deal that much damage to a random opponent argument.",
        flavour = "'That's just so slick.'",
        
        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:CreateModifier("subtle_setup", 1, self)
        end
    },

    slick_plus =
    {
        name = "Enduring Subtle Setup",
        flags = CARD_FLAGS.MANIPULATE,
    },

    slick_plus2 =
    {
        name = "Pale Subtle Setup",
        cost = 0,
    },

    even_footing =
    {
        name = "Even Footing",
        desc = "Apply {1} {COMPOSURE} to a random friendly argument. Repeat once per enemy non-core argument.",
        flavour = "'Hey, No fair!'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateComposureText(self.composure_amt))
        end,

        cost = 1,
        
        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE,
        target_self = TARGET_ANY_RESOLVE,
        auto_target = true,

        composure_amt = 2,
        bonus = 0,

        OnPostResolve = function( self, minigame, targets )
            local options = minigame:CollectAlliedTargets( self.negotiator, nil, nil, self )
            if options and #options > 0 then
                for i=1, #self.anti_negotiator.modifier_slots + self.bonus do
                    options[math.random(#options)]:DeltaComposure(self.composure_amt, self)
                end
            end
        end,
    },

    even_footing_plus =
    {
        name = "Stone Even Footing",
        desc = "Apply <#UPGRADE>{1}</> {COMPOSURE} to a random friendly argument. Repeat once per enemy non-core argument.",
        composure_amt = 3,
    },

    even_footing_plus2 =
    {
        name = "Boosted Even Footing",
        desc = "Apply {1} {COMPOSURE} to a random friendly argument. Repeat once per enemy non-core argument <#UPGRADE>+ 2</>.",
        bonus = 3,
    },

    unyielding =
    {
        name = "Unyielding",
        desc = "If this card destroys an argument, gain 1 action and return it to your hand.",
        flavour = "'He just never shuts his trap!'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE,

        min_persuasion = 2,
        max_persuasion = 2,
        deck_handlers = { DECK_TYPE.DISCARDS },

        event_handlers =
        {
            [ EVENT.START_RESOLVE ] = function( self, minigame, card )
                if card == self then
                    self.to_return = false
                end
            end,

            [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                if source == self then
                    self.to_return = true
                end
            end,

            [ EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if card == self and self.to_return and target_deck and target_deck:GetDeckType() == DECK_TYPE.DISCARDS and source_deck and source_deck:GetDeckType() == DECK_TYPE.RESOLVE then
                    -- Return!
                    self.engine:ModifyActionCount(1)
                    self:TransferCard( self.engine:GetHandDeck() )
                end
            end
        },
    },

    unyielding_plus =
    {
        name = "Rooted Unyielding",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    unyielding_plus2 =
    {
        name = "Tall Unyielding",
        max_persuasion = 4,
    },

    refusal =
    {
        name = "Refusal",
        desc = "{refusal {1}|}Create: Whenever you spend {RENOWN} with a card, deal {1} damage to a random opponent argument.",
        flavour = "'Look pal, you have to give me SOMETHING here.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.stonewall_stacks or 1 )
        end,

        cost = 1,

        stonewall_stacks = 3,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:CreateModifier("refusal", self.stonewall_stacks, self)
        end,
        modifier =
        {
            icon = "negotiation/modifiers/stonewall.tex",
            desc = "Whenever you spend {RENOWN} with a card, deal <#HILITE>{1}</> damage to a random opponent argument.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks or 1 )
            end,

            max_resolve = 4,
            target_enemy = TARGET_ANY_RESOLVE,

            event_handlers = 
            {
                [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                    if modifier.negotiator == self.negotiator and modifier.id == "RENOWN" and is_instance( source, Negotiation.Card ) and source.negotiator == self.negotiator then
                        local options = self.engine:CollectAllTargets( self )
                        if options and #options > 0 then
                            options[math.random(#options)]:AttackResolve(self.stacks, self)
                        end
                    end
                end,

                [ EVENT.MODIFIER_CHANGED ] = function( self, modifier, delta, clone, source )
                    if delta and delta < 0 and modifier.negotiator == self.negotiator and modifier.id == "RENOWN" and is_instance( source, Negotiation.Card ) and source.negotiator == self.negotiator then
                        local options = self.engine:CollectAllTargets( self )
                        if options and #options > 0 then
                            options[math.random(#options)]:AttackResolve(self.stacks, self)
                        end
                    end
                end,
            },
        }
    },

    refusal_plus =
    {
        name = "Boosted Refusal",
        desc = "{refusal {1}|}Create: Whenever you spend {RENOWN} with a card, deal <#UPGRADE>{1} damage</> to a random opponent argument.",
        stonewall_stacks = 4,
    },

    refusal_plus2 =
    {
        name = "Pale Refusal",
        cost = 0,
    },

    barrage = 
    {
        name = "Barrage",
        desc = "{BLIND}.\nAttack with this card 2 times.",
        flavour = "'I think that he’s forgotten his point all together.'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE,

        min_persuasion = 2,
        max_persuasion = 2,

        target_mod = TARGET_MOD.RANDOM1,
        auto_target = true,

        extra_targets = 1,

        OnPostResolve = function( self, minigame, targets )
            for i=1, self.extra_targets do
                minigame:ApplyPersuasion(self)
            end
        end
    },

    barrage_plus = 
    {
        name = "Wide Barrage",
        desc = "{BLIND}.\nAttack with this card <#UPGRADE>3</> times.",
        extra_targets = 2,
    },

    barrage_plus2 =
    {
        name = "Boosted Barrage",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    fall_guy =
    {
        name = "Fall Guy",
        desc = "Apply {1} {COMPOSURE} to a random friendly argument and force all enemy intents and arguments to target it.",
        flavour = "'Wrong place at the wrong time.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.composure_amt)
        end,

        cost = 1 ,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MANIPULATE,

        target_mod = TARGET_MOD.RANDOM1,
        target_self = TARGET_ANY_RESOLVE,
        auto_target = true,

        composure_amt = 3,

        OnPostResolve = function( self, minigame, targets )
            for i,target in ipairs(targets) do
                target:DeltaComposure(self.composure_amt, self)
            end
            for i,intent in ipairs(minigame:GetOtherNegotiator(self.negotiator):GetIntents()) do
                if intent.target then
                    intent.target = targets[1]
                end
            end
            for i,argument in minigame:GetOtherNegotiator(self.negotiator):Modifiers() do
                if argument.target then
                    argument.target = targets[1]
                end
            end
        end
    },

    fall_guy_plus =
    {
        name = "Stone Fall Guy",
        desc = "Apply <#UPGRADE>{1}</> {COMPOSURE} to a random friendly argument and force all enemy intents and arguments to target it.",
        composure_amt = 5,
    },

    fall_guy_plus2 =
    {
        name = "Focused Fall Guy",
        desc = "Apply {1} {COMPOSURE} to a friendly argument and force all enemy intents and arguments to target it.",
        target_mod = TARGET_MOD.SINGLE,
        auto_target = false,
    },

    quip =
    {
        name = "Quip",
        desc = "{quip {1}|}Gain: Whenever your core argument is attacked, deal {1} damage to a random target.",
        flavour = "'This guy gets it.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.quip_stacks)
        end,

        cost = 1,
        quip_stacks = 3,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("quip", self.quip_stacks, self)
        end,

        modifier =
        {
            desc = "Whenever your core argument is attacked, deal <#HILITE>{1}</> damage to a random target.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks or 1)
            end,

            max_resolve = 3,

            target_enemy = TARGET_ANY_RESOLVE,

            event_handlers =
            {
                [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                    if target == self.negotiator:FindCoreArgument() and damage > 0 then
                        local options = self.engine:CollectAllTargets( self )
                        if options and #options > 0 then
                            options[math.random(#options)]:AttackResolve(self.stacks, self)
                        end
                    end
                end
            },
        }
    },

    quip_plus =
    {
        name = "Enduring Quip",
        flags = CARD_FLAGS.HOSTILE,
    },

    quip_plus2 =
    {
        name = "Pale Quip",
        cost = 0,
    },

    -- pandemonium =
    -- {
    --     name = "Pandemonium",
    --     desc = "Deal 1 damage to a random enemy or friendly non-core argument. Repeat until this destroys an argument.",

    --     cost = 1,

    --     rarity = CARD_RARITY.UNCOMMON,
    --     flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,

    --     min_persuasion = 1,
    --     max_persuasion = 1,

    --     target_mod = TARGET_MOD.RANDOM1,
    --     target_self = TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY,
    --     target_enemy = TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY,
    --     auto_target = true,

    --     deck_handlers = { DECK_TYPE.DISCARDS },

    --     event_handlers =
    --     {
    --         [ EVENT.START_RESOLVE ] = function( self, minigame, card )
    --             if card == self then
    --                 self.killed_something = false
    --             end
    --         end,

    --         [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
    --             if source == self then
    --                 self.killed_something = true
    --             end
    --         end,

    --         [ EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
    --             if card == self and not self.killed_something and target_deck and target_deck:GetDeckType() == DECK_TYPE.DISCARDS and source_deck and source_deck:GetDeckType() == DECK_TYPE.RESOLVE then
    --                 -- Return!
    --                 self.engine:ModifyActionCount(1)
    --                 self.engine:PlayCard(self)
    --             end
    --         end
    --     },
    -- },

    -- pandemonium_plus =
    -- {
    --     name = "Enduring Pandemonium",
    --     flags = CARD_FLAGS.HOSTILE,
    -- },

    -- pandemonium_plus2 =
    -- {
    --     name = "Boosted Pandemonium",
    --     desc = "Deal <#UPGRADE>2</> damage to a random enemy OR friendly non-core argument. Repeat until this destroys an argument.",
    --     min_persuasion = 2,
    --     max_persuasion= 2,
    -- },

    discord =
    {
        name = "Chaos Theory",
        desc = "{chaos_theory|}Gain: At the start of your turn, play a random card in your hand for free.",
        flavour = "'How familiar are you with the area around the crash site?'",

        cost = 1,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("chaos_theory", 1, self)
        end,
    },

    discord_plus =
    {
        name = "Pale Chaos Theory",
        cost = 0,
    },

    discord_plus2 =
    {
        name = "Enduring Chaos Theory",
        flags = CARD_FLAGS.MANIPULATE,
    },

    degrade = 
    {
        name = "Degrade",
        desc = "Hits twice. Can target opponent or friendly arguments.",
        flavour = "'What an absolute joke.'",

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.HOSTILE,

        min_persuasion = 1,
        max_persuasion = 1,

        target_self = TARGET_ANY_RESOLVE,
        target_enemy = TARGET_ANY_RESOLVE,
        OnPostResolve = function( self, minigame, targets )
            minigame:ApplyPersuasion( self )
        end,

        EvaluateTargetWeight = function( self, v, t )
            if v.negotiator == self.negotiator and self.target ~= v then
                return 0
            else
                return 1
            end
        end
    },

    degrade_plus =
    {
        name = "Boosted Degrade",
        min_persuasion = 2,
        max_persuasion = 2,
    },

    degrade_plus2 = 
    {
        name = "Mirrored Degrade",
        desc = "Hits <#UPGRADE>three times</>. Can target opponent or friendly arguments.",
        OnPostResolve = function( self, minigame, targets )
            for i=1,2 do
                minigame:ApplyPersuasion( self )
            end
        end,
    },

    topple = 
    {
        name = "Topple", 
        desc = "Can only target friendly arguments.\nDismiss a random enemy intent.",
        flavour = "'Pretty sure that was just a distraction.'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE,

        min_persuasion = 2,
        max_persuasion = 2,

        target_self = TARGET_ANY_RESOLVE,

        CanPlayCard = function( self, card, engine, target )
            if card == self then
                return target == nil or target.negotiator == self.negotiator
            end
        end,

        OnPostResolve = function( self, minigame, targets )
            local intents = minigame:GetOtherNegotiator(self.negotiator):GetIntents()
            if intents and #intents > 0 then
                minigame:GetOtherNegotiator(self.negotiator):DismissIntent(intents[math.random(#intents)])
            end
        end
    },

    topple_plus =
    {
        name= "Softened Topple",
        min_persuasion = 1,
        max_persuasion = 1,
    },

    topple_plus2 =
    {
        name = "Pale Topple",
        min_persuasion = 3,
        max_persuasion = 3,
        cost = 0,
    },

    instigate =
    {
        name = "Instigate",
        desc = "{instigate|}Create: Whenever an argument is destroyed, increase <b>Instigate</> by 1 and draw a card. At 8 stacks, remove this argument and deal 14 damage to a random enemy target.",
        flavour = "'What the Hesh?'",
        
        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:CreateModifier("instigate", 1, self)
        end,

        modifier = 
        {
            desc = "Whenever an argument is destroyed, increase <b>Instigate</> by 1 and draw a card. At {1} stacks, remove this argument and deal {2} damage to a random enemy target.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stack_limit, self.damage)
            end,

            max_resolve = 4,

            stack_limit = 8,
            damage = 14,

            target_enemy = TARGET_ANY_RESOLVE,

            event_handlers =
            {
                [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                    if source then
                        self.engine:DrawCards(1)
                        self:ApplyStacks(1, self)
                        self:NotifyChanged(1, self)
                        if self.stacks >= self.stack_limit then
                            local options = self.engine:CollectAllTargets( self )
                            if options and #options > 0 then
                                options[math.random(#options)]:AttackResolve(self.damage, self)
                            end
                            if self:IsApplied() then
                                self.negotiator:DestroyModifier(self, self)
                            end
                        end
                    end
                end
            },
        }
    },

    instigate_plus =
    {
        name = "Enduring Instigate",
        flags = CARD_FLAGS.HOSTILE,
    },

    instigate_plus2 =
    {
        name = "Pale Instigate",
        cost = 0,
    },

    double_down =
    {
        name = "Double Down",
        desc = "Choose a card in your hand. Play it twice with a random target and then {EXPEND} it.",
        flavour = "'I can't lose.'",

        cost = 2,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        expend_card = true,

        loc_strings =
        {
            DOUBLE_DOWN_CHOOSE = "Choose a card to play twice and <b>Expend</>"
        },

        OnPostResolve = function( self, minigame, targets )
            local chosen_card = minigame:ChooseCard( nil, self.def:GetLocalizedString("DOUBLE_DOWN_CHOOSE") )
            if chosen_card then
                for i=1,2 do
                    if i == 2 and self.expend_card then
                        chosen_card:SetFlags(CARD_FLAGS.EXPEND)
                    end
                    minigame:PlayCard(chosen_card)
                end
            end
        end
    },

    double_down_plus =
    {
        name = "Pale Double Down",
        cost = 1,
    },

    double_down_plus2 =
    {
        name = "Soft Double Down",
        desc = "Choose a card in your hand. Play it twice with a random target.",
        expend_card = false
    },

    chase = 
    {
        name = "Chase",
        desc = "If this destroys an argument, play this card a second time.",
        flavour = "'Oh I really got you there didn’t I?'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE,

        min_persuasion = 2,
        max_persuasion = 4,

        deck_handlers = { DECK_TYPE.DISCARDS },

        event_handlers =
        {
            [ EVENT.START_RESOLVE ] = function( self, minigame, card )
                if card == self then
                    self.to_repeat = false
                end
            end,

            [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                if source == self then
                    self.to_repeat = true
                end
            end,

            [ EVENT.CARD_MOVED ] = function( self, card, source_deck, source_idx, target_deck, target_idx )
                if card == self and target_deck and target_deck:GetDeckType() == DECK_TYPE.DISCARDS and source_deck and source_deck:GetDeckType() == DECK_TYPE.RESOLVE then
                    if self.to_repeat and not self.repeating then
                        self.repeating = true
                        self.engine:PlayCard(self)
                    else
                        self.repeating = false
                    end
                end
            end
        },
    },

    chase_plus = 
    {
        name = "Rooted Chase",
        min_persuasion = 4,
    },

    chase_plus2 = 
    {
        name = "Tall Chase",
        max_persuasion = 6,
    },

    escalate = 
    {
        name = "Escalate",
        desc = "Gain {1} {VULNERABILITY}.\nAll your cards deal double damage until the end of your turn.",
        flavour = "'I will NOT calm down!'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.vulnerability_stacks)
        end,

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,

        vulnerability_stacks = 2,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("VULNERABILITY", self.vulnerability_stacks, self)
            self.negotiator:AddModifier("escalate", 1, self)
        end,

        modifier =
        {
            hidden = true,
            event_priorities = 
            {
                [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_MULTIPLIER,
            },
            event_handlers = 
            {
                [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                    if  source.negotiator == self.negotiator and is_instance(source, Negotiation.Card) then
                        persuasion:AddPersuasion( persuasion.min_persuasion * self.stacks, persuasion.max_persuasion * self.stacks, self )
                    end
                end,

                [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                    self.negotiator:RemoveModifier(self.id, self.stacks, self)
                end
            },
        },
    },

    escalate_plus =
    {
        name = "Softened Escalate",
        desc = "Gain <#UPGRADE>{1}</> {VULNERABILITY}.\nAll your cards deal double damage until the end of your turn.",
        vulnerability_stacks = 1,
    },

    escalate_plus2 = 
    {
        name = "Pale Escalate",
        cost = 0,
    },

    all_out = 
    {
        name = "All Out",
        desc = "Gain {1} {DEFENSELESS}.\nReduce the cost of 2 random cards in your hand by 1 until the end of your turn.",
        flavour = "'You asked for it!'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.defenseless_amt)
        end,

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE,

        defenseless_amt = 2,
        num_cards = 2,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("DEFENSELESS", self.defenseless_amt, self)
            self.negotiator:AddModifier("all_out", self.num_cards, self)
        end,

        modifier =
        {
            hidden = true,

            CanReduceCost = function( card )
                return card.engine:CalculateActionCost( card ) > 0 and not card:IsFlagged( CARD_FLAGS.UNPLAYABLE )
            end,

            OnSetStacks = function( self, old_stacks )
                self.chosen_cards = self.chosen_cards or {}
                local delta = self.stacks - old_stacks
                if delta > 0 then
                    local hand = self.engine:GetHandDeck()
                    local cards = hand:PickCardsIf( self.CanReduceCost, delta )
                    if cards then
                        for i,card in ipairs(cards) do
                            table.insert(self.chosen_cards, card)
                        end
                    end
                end
            end,

            event_handlers =
            {
                [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                    if self.chosen_cards and table.contains(self.chosen_cards, card) then
                        acc:AddValue(-1, self)
                    end
                end,

                [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                    self.negotiator:RemoveModifier(self.id, self.stacks, self)
                end,
            },
        },
    },

    all_out_plus =
    {
        name = "Boosted All Out",
        desc = "Gain {1} {DEFENSELESS}.\nReduce the cost of <#UPGRADE>3</> random cards in your hand by 1 until the end of your turn.",
        num_cards = 3,
    },

    all_out_plus2 =
    {
        name = "Softened All Out",
        desc = "Gain <#UPGRADE>{1}</> {DEFENSELESS}.\nReduce the cost of 2 random cards in your hand by 1 until the end of your turn.",
        defenseless_amt = 1,
    },

    crass = 
    {
        name = "Crass",
        desc = "Gain {DOMINANCE {1}} and {2} {VULNERABILITY}.",
        flavour = "'No YOU have something on your face!'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.dominance_amt, self.vulnerability_stacks)
        end,

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.HOSTILE,

        dominance_amt = 2,
        vulnerability_stacks = 1,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("DOMINANCE", self.dominance_amt, self)
            self.negotiator:AddModifier("VULNERABILITY", self.vulnerability_stacks, self)
        end
    },

    crass_plus =
    {
        name = "Boosted Crass",
        desc = "Gain <#UPGRADE>{DOMINANCE {1}}</> and {2} {VULNERABILITY}.",
        dominance_amt = 3,
    },

    crass_plus2 =
    {
        name = "Pure Crass",
        desc = "Gain {DOMINANCE {1}}.",
        vulnerability_stacks = 0,
    },

    rescind = 
    {
        name = "Rescind",
        desc = "Attack a random argument.\nSpend all {RENOWN}: Attack again per {RENOWN} spent.",
        flavour = "'The deal has changed.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_amt)
        end,

        cost = 2,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE,

        target_enemy = TARGET_ANY_RESOLVE,
        target_mod = TARGET_MOD.RANDOM1,
        auto_target = true,

        min_persuasion = 2,
        max_persuasion = 2,

        OnPostResolve = function( self, minigame, targets )
            local stacks = self.negotiator:GetModifierStacks( "RENOWN" )
            for i = 1, stacks do
                minigame:ApplyPersuasion( self )
                self:AssignTarget( nil ) -- Ensures a new target is chosen each iteration.
                -- Decrement Renown one at a time for presentational effect.
                self.negotiator:RemoveModifier( "RENOWN", 1 )
            end
        end,
    },

    rescind_plus =
    {
        name = "Boosted Rescind",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    rescind_plus2 =
    {
        name = "Focused Rescind",
        desc = "Attack a <#UPGRADE>specific</> argument.\nSpend all {RENOWN}: Attack again per {RENOWN} spent.",
        auto_target = false,
        target_mod = TARGET_MOD.SINGLE,
    },

    renounce =
    {
        name = "Renounce",
        desc = "Attack a random argument. {EXPEND} any number of cards in your hand, attack again per card <b>Expended</>.",
        flavour = "'I've changed my mind!'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,

        target_enemy = TARGET_ANY_RESOLVE,
        target_mod = TARGET_MOD.RANDOM1,
        auto_target = true,

        min_persuasion = 2,
        max_persuasion = 2,
        OnPostResolve = function( self, minigame, targets )
            if self.draw_count then
                minigame:DrawCards(self.draw_count)
            end
            local cards = minigame:ExpendCards()
            for i = 1, #cards do
                minigame:ApplyPersuasion( self )
                self:AssignTarget( nil ) -- Ensures a new target is chosen each iteration.
            end
        end,
    },

    renounce_plus =
    {
        name = "Boosted Renounce",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    renounce_plus2 =
    {
        name = "Visionary Renounce",
        desc = "Attack a random argument.<#UPGRADE> Draw 2 cards,</> then {EXPEND} any number of cards in your hand, attack again per card <b>Expended</>.",
        draw_count = 2,
    },

    long_winded = 
    {
        name = "Long Winded",
        desc = "Draw {1} cards and play them for free.",
        flavour = "'Great, you got him started. We are going to be here a while.'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.draw_count)
        end,

        cost = 1,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        draw_count = 2,

        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:DrawCards(self.draw_count)
            for i,card in ipairs(cards) do
                minigame:PlayCard(card)
            end
        end,
    },

    long_winded_plus =
    {
        name = "Visionary Long Winded",
        desc = "Draw <#UPGRADE>{1}</> cards and play them for free.",
        draw_count = 3,
    },

    long_winded_plus2 = 
    {
        name = "Pale Long Winded",
        cost = 0,
    },

    plastered =
    {
        name = "Plastered",
        desc = "Destroy a friendly argument, destroy a random opponent argument.",
        flavour = "'What a walking disaster.'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        target_self = TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY,

        OnPostResolve = function( self, minigame, targets )
            local options = {}
            for i,arg in self.anti_negotiator:Modifiers() do
                if arg.modifier_type ~= MODIFIER_TYPE.CORE and arg:GetResolve() then
                    table.insert(options, arg)
                end
            end
            if options and #options > 0 then
                local chosen = options[math.random(#options)]
                self.anti_negotiator:DestroyModifier(chosen, self)
            end
            for i,target in ipairs(targets) do
                target.negotiator:DestroyModifier(target, self)
            end
        end
    },

    plastered_plus =
    {
        name = "Pale Plastered",
        cost = 0,
    },

    plastered_plus2 =
    {
        name = "Enduring Plastered",
        flags = CARD_FLAGS.MANIPULATE,
    },

    rehearsal =
    {
        name = "Rehearsal",
        desc = "Choose a card in your hand and double its damage.",
        flavour = "'Alas, poor Rook. I knew him well.'",

        cost = 2,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        loc_strings =
        {
            REHEARSAL_CHOOSE = "Choose a card and double its damage"
        },

        OnPostResolve = function( self, minigame, targets )
            local mod = self.negotiator:AddModifier("rehearsal", 1, self)
            local chosen_card = minigame:ChooseCard( nil, self.def:GetLocalizedString("REHEARSAL_CHOOSE"))
            if chosen_card then
                local min_persuasion, max_persuasion = minigame:PreviewPersuasion( chosen_card )
                chosen_card.rehearsal_min_bonus = (chosen_card.rehearsal_min_bonus or 0) + min_persuasion
                chosen_card.rehearsal_max_bonus = (chosen_card.rehearsal_max_bonus or 0) + max_persuasion
            end
        end,

        modifier = 
        {
            hidden = true,

            event_priorities =
            {
                [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_MULTIPLIER,
            },

            event_handlers =
            {
                [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                    if source and type(source.rehearsal_min_bonus) == "number" and type(source.rehearsal_max_bonus) == "number" then
                        persuasion:AddPersuasion( source.rehearsal_min_bonus, source.rehearsal_max_bonus, self )
                    end
                end,
            },
        },
    },

    rehearsal_plus =
    {
        name = "Pale Rehearsal",
        cost = 1,
    },

    rehearsal_plus2 =
    {
        name = "Enduring Rehearsal",
        flags = CARD_FLAGS.MANIPULATE,
    },

    amplify = 
    {
        name = "Amplify",
        desc = "Choose a card in your hand and increase its damage by 2 this negotiation.",
        flavour = "'Smith? Yeah I would say that guy is a little intense.'",

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MANIPULATE,

        bonus_damage = 2,
        num_cards = 1,
        loc_strings =
        {
            AMPLIFY_CHOOSE = "Choose a card and increase its damage"
        },

        OnPostResolve = function( self, minigame, targets )
            local chosen_cards = minigame:ChooseCards(0, self.num_cards, nil, self.def:GetLocalizedString("AMPLIFY_CHOOSE"))
            if chosen_cards then
                local mod = self.negotiator:AddModifier("amplify", 1, self)
                for i,card in ipairs(chosen_cards) do
                    card.amplify_bonus = (card.amplify_bonus or 0) + self.bonus_damage
                end
            end
        end,

        modifier = 
        {
            hidden = true,

            event_priorities =
            {
                [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
            },
            event_handlers =
            {
                [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                    if source and type(source.amplify_bonus) == "number" then
                        persuasion:AddPersuasion( source.amplify_bonus, source.amplify_bonus, self )
                    end
                end,
            },
        },
    },

    amplify_plus =
    {
        name = "Boosted Amplify",
        desc = "Choose a card in your hand and increase its damage by <#UPGRADE>3</> this negotiation.",
        bonus_damage = 3,
    },

    amplify_plus2 =
    {
        name = "Wide Amplify",
        desc = "Choose <#UPGRADE>up to 2 cards</> in your hand and increase their damage by 2 this negotiation.",
        num_cards = 2,
    },

    forgo = 
    {
        name = "Forgo",
        desc = "{EXPEND} a card in your hand, all other cards in your hand gain 2 damage until played.",
        flavour = "'Dispense with the formalities.'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE,

        CanPlayCard = function( self, card, engine, target )
            if card == self then
                if engine:GetHandDeck():CountCards() <= 1 then
                    return false, CARD_PLAY_REASONS.NO_VALID_TARGETS
                end
            end
            return true
        end,

        forgo_stacks = 2,

        OnPostResolve = function( self, minigame, targets )
            local txt = loc.format( LOC"CARD_ENGINE.CHOOSE_EXPEND", 1 )
            local chosen_card = minigame:ChooseCard( nil, txt )
            if chosen_card then
                minigame:ExpendCard(chosen_card)
            end
            local cards = minigame:GetHandDeck().cards
            if #cards > 0 then
                local mod = self.negotiator:AddModifier("forgo", 1, self)
                for i,card in ipairs(cards) do
                    card.forgo_bonus = (card.forgo_bonus or 0) + self.forgo_stacks
                end
            end
        end,

        modifier = 
        {
            hidden = true,

            event_priorities =
            {
                [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
            },
            event_handlers =
            {
                [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                    if source and type(source.forgo_bonus) == "number" then
                        persuasion:AddPersuasion( source.forgo_bonus, source.forgo_bonus, self )
                    end
                end,

                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    if card and card.forgo_bonus ~= nil then
                        card.forgo_bonus = nil
                    end
                end,
            },
        },
    },

    forgo_plus =
    {
        name = "Pale Forgo",
        cost = 0,
    },

    forgo_plus2 =
    {
        name = "Boosted Forgo",
        desc = "{EXPEND} a card in your hand, all other cards in your hand gain <#UPGRADE>3 damage</> until played.",
        forgo_stacks = 3,
    },

    pursue =
    {
        name = "Pursue",
        desc = "{IMPROVISE} a card from your discard pile and increase its damage by 2 this negotiation.",
        flavour = "'I'm gonna get you.'",

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MANIPULATE,

        improvise_count = 1,
        damage_amt = 2,
        pool_size = 3,

        OnPostResolve = function( self, minigame, targets )
            local discard = minigame:GetDiscardDeck().cards
            local cards = minigame:ImproviseCards(table.multipick(discard, self.pool_size), self.improvise_count, nil, "ad_lib", nil, self)
            if cards then
                local mod = self.negotiator:AddModifier("pursue", 1, self)
                for i,card in ipairs(cards) do
                    card.pursue_bonus = (card.pursue_bonus or 0) + self.damage_amt
                end
            end
        end,

        modifier = 
        {
            hidden = true,

            event_priorities =
            {
                [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
            },
            event_handlers =
            {
                [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                    if source and type(source.pursue_bonus) == "number" then
                        persuasion:AddPersuasion( source.pursue_bonus, source.pursue_bonus, self )
                    end
                end,
            },
        },
    },

    pursue_plus =
    {
        name = "Boosted Pursue",
        desc = "{IMPROVISE} a card from your discard pile and increase its damage by <#UPGRADE>3</> this negotiation.",
        damage_amt = 3,
    },

    pursue_plus2 =
    {
        name = "Seeking Pursue",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> a card from your discard pile and increase its damage by 2 this negotiation.",
        pool_size = 5,
    },

    carbon_copy =
    {
        name = "Carbon Copy",
        desc = "Choose a card in your hand and create an exact copy of it.",
        flavour = "'They're the same picture.'",

        cost = 1,

        count = 1,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, minigame, targets )
            local txt = loc.format( LOC"CARD_ENGINE.CHOOSE_MAX_CARDS_DUPLICATE", 1 )
            local chosen_card = minigame:ChooseCard( nil, txt )
            if chosen_card then
                for i=1,self.count do
                    local copy = chosen_card:Duplicate()
                    copy.deck = nil
                    copy.cost = minigame:CalculateActionCost(chosen_card)
                    minigame:DealCard(copy, minigame:GetHandDeck())
                end
            end
        end,
    },

    carbon_copy_plus = 
    {
        name = "Pale Carbon Copy",
        cost = 0,
    },

    carbon_copy_plus2 =
    {
        name = "Mirrored Carbon Copy",
        desc = "Choose a card in your hand and create <#UPGRADE>2</> exact copies of it.",
        count = 2,
    },

    fixate = 
    {
        name = "Fixate",
        desc = "Remove all {COMPOSURE} from all friendly arguments, apply that much {COMPOSURE} to your target.",
        flavour = "'I can't let this go.'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE,

        target_self = TARGET_ANY_RESOLVE,
        bonus = 0,

        CanPlayCard = function( self, card, engine, target )
            if card == self then
                for i,arg in self.negotiator:Modifiers() do
                    if arg:GetComposure() > 0 then
                        return true
                    end
                end
                return false
            end
        end,

        OnPostResolve = function( self, minigame, targets )
            local comp = 0
            for i,arg in self.negotiator:Modifiers() do
                comp = comp + arg:GetComposure()
                arg:DeltaComposure(-arg:GetComposure(), self)
            end
            if comp + self.bonus > 0 then
                for i,target in ipairs(targets) do
                    target:DeltaComposure(comp + self.bonus, self)
                end
            end
        end
    },

    fixate_plus =
    {
        name = "Boosted Fixate",
        desc = "Remove all {COMPOSURE} from all friendly arguments, apply that much {COMPOSURE} to your target <#UPGRADE>+ 3</>.",
        bonus = 3,
    },

    fixate_plus2 =
    {
        name = "Pale Fixate",
        cost = 0,
    },

    insistence = 
    {
        name = "Insistence",
        desc = "Duplicate the next argument you create.",
        flavour = "'I don't think you heard me the first time.'",

        cost = 1,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        copies = 1,
        arguments = 1,

        OnPostResolve = function( self, minigame, targets )
            local mod = self.negotiator:AddModifier("insistence", self.arguments, self)
            mod.copies = self.copies
        end,

        modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.MODIFIER_ADDED ] = function( self, modifier, source )
                    if modifier ~= self and source and source.negotiator and source.negotiator == self.negotiator and source ~= self and not (is_instance(source, Negotiation.Card) and CheckBits(source.flags, CARD_FLAGS.BYSTANDER)) then
                        self.negotiator:RemoveModifier(self.id, 1)
                        for i=1, self.copies or 1 do
                            modifier.negotiator:CreateModifier(modifier.id, modifier.stacks, self)
                        end
                    end
                end
            },
        },
    },

    insistence_plus =
    {
        name = "Boosted Insistence",
        desc = "Duplicate the next argument you create <#UPGRADE>twice</>.",
        copies = 2,
    },

    insistence_plus2 =
    {
        name = "Extended Insistence",
        desc = "Duplicate the next <#UPGRADE>two</> arguments you create.",
        arguments = 2,
    },

    dissect = 
    {
        name = "Dissect",
        desc = "Duplicate a friendly argument and halve the resolve of both arguments.",
        flavour = "'It’s going to be impossible to solve.'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,

        target_self = TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY,

        OnPostResolve = function( self, minigame, targets )
            if targets[1] then
                local resolve_loss = math.floor(targets[1]:GetResolve() * .5)
                targets[1]:LoseResolve(resolve_loss)
                local new_mod = targets[1]:Duplicate(targets[1].negotiator)
            end
        end
    },

    dissect_plus =
    {
        name = "Pale Dissect",
        cost = 0,
    },

    dissect_plus2 =
    {
        name = "Enduring Dissect",
        flags = CARD_FLAGS.MANIPULATE,
    },

    turnabout =
    {
        name = "Turnabout",
        desc = "Destroy a friendly argument, apply {COMPOSURE} equal to its maximum resolve to all friendly arguments.",
        flavour = "'Did someone say my name?'",

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MANIPULATE,

        target_self = TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY,

        bonus = 0,

        OnPostResolve = function( self, minigame, targets )
            if targets and targets[1] then
                local cur, max = targets[1]:GetResolve()
                max = max + self.bonus
                self.negotiator:DestroyModifier(targets[1], self)
                local options = self.engine:CollectAlliedTargets(self.negotiator)
                for i,v in ipairs(options) do
                    v:DeltaComposure(max, self)
                end
            end
        end
    },

    turnabout_plus =
    {
        name = "Boosted Turnabout",
        desc = "Destroy a friendly argument, apply {COMPOSURE} equal to its maximum resolve <#UPGRADE>+ 2</> to all friendly arguments.",
        bonus = 2,
    },

    turnabout_plus2 =
    {
        name = "Pale Turnabout",
        cost = 0,
    },

    double_entendre =
    {
        name = "Double Entendre",
        desc = "Apply {COMPOSURE} to all friendly arguments equal to the damage dealt by this card.",
        flavour = "'You know what I mean, right?'",

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE,

        min_persuasion = 1,
        max_persuasion = 3,
        mult = 1,

        event_handlers = 
        {
            [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                if source == self then
                    local targets = self.engine:CollectAlliedTargets(self.negotiator)
                    if targets and #targets > 0 then
                        for i,target in ipairs(targets) do
                            target:DeltaComposure(math.round(damage * self.mult), self)
                        end
                    end
                end
            end,
        },
    },

    double_entendre_plus =
    {
        name = "Rooted Entendre",
        min_persuasion = 3,
        max_persuasion = 3,
    },

    double_entendre_plus2 =
    {
        name = "Tall Entendre",
        min_persuasion = 1,
        max_persuasion = 5,
    },

    goon = 
    {
        name = "Goon",
        desc = "{goon|}Create: At the start of your turn, apply {1} {COMPOSURE} to this argument.",
        flavour = "'I never seen that guy in all my life! I swear!'",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.goon_stacks * 2)
        end,

        cost = 1,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        count = 1,
        goon_stacks = 1,

        OnPostResolve = function( self, minigame, targets )
            for i=1,self.count do
                self.negotiator:CreateModifier("goon", self.goon_stacks, self)
            end
        end,

        modifier = 
        {
            desc = "At the start of your turn, apply <#HILITE>{1}</> {COMPOSURE} to this argument.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.composure_amt * (self.stacks or 1), self.min_persuasion)
            end,

            max_resolve = 4,

            min_persuasion = 1,
            max_persuasion = 1,

            composure_amt = 2,

            target_enemy = TARGET_ANY_RESOLVE,

            IsValidTarget = function( target )
                return target:GetResolve() ~= nil
            end,

            OnSetStacks = function( self, old_stacks )
                self.min_persuasion, self.max_persuasion = self.stacks, self.stacks
            end,

            OnApply = function( self )
                self:PrepareTurn()
            end,

            event_handlers = 
            {
                [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        self:ApplyPersuasion()
                    end
                end,

                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        self:DeltaComposure(self.composure_amt * self.stacks, self)
                    end
                end
            },
        }
    },

    goon_plus =
    {
        name = "Mirrored Goon",
        desc = "{goon|}Create <#UPGRADE>2</>: At the start of your turn, apply {1} {COMPOSURE} to this argument.",
        count = 2,
    },

    goon_plus2 =
    {
        name = "Pale Goon",
        cost = 0,
    },

    boiler =
    {
        name = "Powder Keg",
        desc = "{boiler {1}|}Create: When destroyed, deal {1} damage to a random opponent argument and create a new <b>Powder Keg</>.",
        flavour = "'I thought I told you guys to never come back here.'",

        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.boiler_stacks )
        end,

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,

        count = 1,
        boiler_stacks = 8,

        OnPostResolve = function( self, minigame, targets )
            for i=1,self.count do
                self.negotiator:CreateModifier("boiler", self.boiler_stacks, self)
            end
        end,
        modifier =
        {
            icon = "negotiation/modifiers/powder_keg.tex",
            desc = "When destroyed, deal <#HILITE>{1}</> damage to a random opponent argument and create a new <b>Powder Keg</>.",
            desc_fn = function( self, fmt_str )
                return loc.format(fmt_str, self.stacks)
            end,

            max_resolve = 6,
            modifier_type = MODIFIER_TYPE.BOUNTY,

            target_enemy = TARGET_ANY_RESOLVE,

            OnBounty = function( self, source )
                local options = self.engine:CollectAllTargets( self )
                if options and #options > 0 then
                    options[math.random(#options)]:AttackResolve( self.stacks, self)
                end
                if self.negotiator:GetSlotCount() + 1 < self.negotiator:GetMaxSlots() then
                    self.negotiator:CreateModifier("boiler", self.stacks, self)
                end
            end,
        }
    },

    boiler_plus =
    {
        name = "Boosted Powder Keg",
        desc = "{boiler {1}|}Create: When destroyed, deal <#UPGRADE>{1} damage</> to a random opponent argument and create a new <b>Powder Keg</>.",
        boiler_stacks = 12,
    },

    boiler_plus2 =
    {
        name = "Pale Powder Keg",
        cost = 0,
    },

    classic_smith =
    {
        name = "Classic Smith",
        desc = "Replace all {brag} cards in your deck with {bully} cards and add them to your hand.",
        flavour = "'It’s Smithering time!'",

        cost = 1,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.COMMON,

        OnPostResolve = function( self, minigame, targets )
            local cards_to_remove = {}
            for i,card in minigame:GetDrawDeck():Cards() do
                if card.id == "brag" or card.base_id == "brag" then
                    table.insert(cards_to_remove, card)
                end 
            end
            for i,card in minigame:GetDiscardDeck():Cards() do
                if card.id == "brag" or card.base_id == "brag" then
                    table.insert(cards_to_remove, card)
                end 
            end
            for i,card in minigame:GetHandDeck():Cards() do
                if card.id == "brag" or card.base_id == "brag" then
                    table.insert(cards_to_remove, card)
                end 
            end

            if #cards_to_remove > 0 then
                -- Duplicate from a prototype bully, to ensure these transient cards don't have xp, etc.
                local bully = Negotiation.Card("bully", self.owner)
                for i=1, #cards_to_remove do
                    minigame:DealCard( bully:Duplicate(), minigame:GetHandDeck())
                end
                for i,card in ipairs(cards_to_remove) do
                    minigame:ExpendCard(card)
                end
            end
        end,
    },

    classic_smith_plus =
    {
        name = "Promoted Smith",
        desc = "Replace all {brag} cards in your deck with <#UPGRADE>randomly upgraded</> {bully} cards and add them to your hand.",
        OnPostResolve = function( self, minigame, targets )
            local cards_to_remove = {}
            for i,card in minigame:GetDrawDeck():Cards() do
                if card.id == "brag" or card.base_id == "brag" then
                    table.insert(cards_to_remove, card)
                end 
            end
            for i,card in minigame:GetDiscardDeck():Cards() do
                if card.id == "brag" or card.base_id == "brag" then
                    table.insert(cards_to_remove, card)
                end 
            end
            for i,card in minigame:GetHandDeck():Cards() do
                if card.id == "brag" or card.base_id == "brag" then
                    table.insert(cards_to_remove, card)
                end 
            end

            if #cards_to_remove > 0 then
                -- Duplicate from a prototype bully, to ensure these transient cards don't have xp, etc.
                local bully = Negotiation.Card("bully", self.owner)
                for i=1, #cards_to_remove do
                    local card = Negotiation.Card(table.arraypick(bully.def.upgrade_ids), self.owner)
                    minigame:DealCard( card:Clone(), minigame:GetHandDeck())
                end
                for i,card in ipairs(cards_to_remove) do
                    minigame:ExpendCard(card)
                end
            end
        end,
    },

    classic_smith_plus2 =
    {
        name = "Pale Smith",
        cost = 0
    },

    hot_air=
    {
        name = "Hot Air", 
        desc = "{hot_air {1}|}Create: If this argument is destroyed by you, recreate it and increase its damage by {1}.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.boost )
        end,
        flavour = "'Nothing you've been saying has meant much of anything.'",

        cost = 1,
        count = 1,
        boost = 3,

        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        OnPostResolve = function( self, minigame, targets )
            for i=1,self.count do
                self.negotiator:CreateModifier("hot_air", self.boost, self)
            end
        end,

        modifier =
        {
            desc = "If this argument is destroyed by you, recreate it and increase its damage by <#HILITE>{1}</>.",
            desc_fn = function( self, fmt_str )
                return loc.format( fmt_str, self.stacks )
            end,

            modifier_type = MODIFIER_TYPE.ARGUMENT,

            max_resolve = 2,

            target_enemy = TARGET_ANY_RESOLVE,
            min_persuasion = 2,
            max_persuasion = 2,

            OnApply = function( self )
                self:PrepareTurn()
            end,

            OnBounty = function( self, source )
                if source and source.negotiator == self.negotiator and self.negotiator:GetSlotCount() + 1 < self.negotiator:GetMaxSlots() then
                    local new_mod = self.negotiator:CreateModifier("hot_air", self.stacks, self)
                    local new_dmg = self.min_persuasion + self.stacks
                    new_mod.min_persuasion, new_mod.max_persuasion = new_dmg, new_dmg
                    new_mod:NotifyChanged(1, self)
                end
            end,

            event_handlers =
            {
                [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        self:ApplyPersuasion()
                    end
                end,
            },
        },
    },

    hot_air_plus =
    {
        name = "Boosted Hot Air",
        desc = "{hot_air {1}|}Create: If this argument is destroyed by you, recreate it and increase its damage by <#UPGRADE>{1}</>.",
        boost = 5,
    },

    hot_air_plus2 = 
    {
        name = "Enduring Hot Air",
        flags = CARD_FLAGS.HOSTILE,
    },

    circle_back = 
    {
        name = "Circle Back",
        desc = "Duplicate a friendly argument, then destroy the original.",
        flavour = "'Now wait a second!'",

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,
        count = 1,

        target_self = TARGET_FLAG.BOUNTY | TARGET_FLAG.ARGUMENT,

        OnPostResolve = function( self, minigame, targets )
            if targets and #targets > 0 then
                for i=1,self.count do
                    for i,v in ipairs(targets) do
                        v:Duplicate(self.negotiator)
                    end
                end
                for i,v in ipairs(targets) do
                    self.negotiator:DestroyModifier(v, self)
                end     
            end
        end
    },

    circle_back_plus =
    {
        name = "Pale Circle Back",
        cost = 0,
    },

    circle_back_plus2 = 
    {
        name = "Boosted Circle Back",
        desc = "Duplicate a friendly argument <#UPGRADE>twice</>, then destroy the original.",
        count = 2,
        cost = 2,
    },

    scapegoat = 
    {
        name = "Scapegoat",
        desc = "Force all friendly arguments to target a specific opponent argument.",
        flavour = "'What have you been telling everyone about me?'",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.composure_amt)
        end,

        cost = 1 ,

        rarity = CARD_RARITY.COMMON,
        flags = CARD_FLAGS.MANIPULATE,

        target_enemy = TARGET_ANY_RESOLVE,
        target_self = false,

        OnPostResolve = function( self, minigame, targets )
            for i,argument in self.negotiator:Modifiers() do
                if argument.target then
                    argument.target = targets[1]
                end
            end
        end
    },

    scapegoat_plus =
    {
        name = "Pale Scapegoat",
        cost = 0,
    },

    scapegoat_plus2 =
    {
        name = "Biting Scapegoat",
        min_persuasion = 2,
        max_persuasion = 2,
    },

    revision =
    {
        name = "Revision",
        desc = "Destroy all friendly arguments and create new copies of them.",
        flavour = "'A few more rewrites and we'll be a lot closer.'",

        cost = 2,

        flags = CARD_FLAGS.MANIPULATE | CARD_FLAGS.EXPEND,
        rarity = CARD_RARITY.UNCOMMON,

        target_self = TARGET_FLAG.BOUNTY | TARGET_FLAG.ARGUMENT,
        target_mod = TARGET_MOD.TEAM,
        auto_target = true,
        draw = false,
 
        OnPostResolve = function( self, minigame, targets )
            local new_mods = {}
            local count = 0
            for i,modifier in ipairs(targets) do
                if modifier.modifier_type ~= MODIFIER_TYPE.CORE then
                    table.insert(new_mods, {id = modifier.id, stacks = modifier.stacks})
                    self.negotiator:DestroyModifier(modifier, self)
                    count = count + 1
                end
            end

            for i,mod in ipairs(new_mods) do
                self.negotiator:CreateModifier(mod.id, mod.stacks, self)
            end
            if count > 0 and self.draw then
                minigame:DrawCards(count)
            end
        end
    },

    revision_plus =
    {
        name = "Pale Revision",
        cost = 1,
    },

    revision_plus2 =
    {
        name = "Visionary Revision",
        desc = "Destroy all friendly arguments and create new copies of them.\n<#UPGRADE>Draw a card for every argument destroyed.</>",
        draw = true,
    },

    ditch =
    {
        name = "Ditch",
        desc = "Discard any number of cards, gain {1} {RENOWN} per card discarded.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.renown_amt )
        end,
        flavour = "'He was here a second ago.'",

        cost = 1,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,

        renown_amt = 2,
        draw_count = 0,

        OnPostResolve = function( self, minigame, targets )
            if self.draw_count and self.draw_count > 0 then
                minigame:DrawCards(self.draw_count)
            end            
            local cards = minigame:DiscardCards(nil, nil, self)
            if #cards > 0 then
                self.negotiator:AddModifier("RENOWN", self.renown_amt * #cards, self)
            end
        end
    },

    ditch_plus =
    {
        name = "Boosted Ditch",
        desc = "Discard any number of cards, gain <#UPGRADE>{1} {RENOWN}</> per card discarded.",
        renown_amt = 3,
    },

    ditch_plus2 =
    {
        name = "Visionary Ditch",
        desc = "<#UPGRADE>Draw {2} {2*card|cards}, then</> discard any number of cards, gain {1} {RENOWN} per card discarded.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.renown_amt, self.draw_count )
        end,
        draw_count = 2,
    },

    dominion =
    {
        name = "Dominion",
        desc = "Spend up to {1} {RENOWN}: Gain {DOMINANCE} equal to the {RENOWN} spent.",
        flavour = "'You thought this was just a game?'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.renown_cap, self.renown_per)
        end,

        cost = 1,

        CanPlayCard = function( self, card, engine, target )
            return self.negotiator:GetModifierStacks("RENOWN") >= self.renown_per, loc.format( LOC"CONVO_COMMON.REQ_RENOWN", self.renown_per)
        end,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.UNCOMMON,

        renown_cap = 3,
        renown_per = 1,

        OnPostResolve = function( self, minigame, targets )
            local renown_count = math.min(self.negotiator:GetModifierStacks("RENOWN"), self.renown_cap)
            if renown_count >= self.renown_per then
                local full = math.floor(renown_count / self.renown_per)
                self.negotiator:RemoveModifier("RENOWN", full * self.renown_per, self)
                self.negotiator:AddModifier("DOMINANCE", full, self)
            end
        end
    },

    dominion_plus =
    {
        name = "Pale Dominion",
        cost = 0,
    },

    dominion_plus2 =
    {
        name = "Boosted Dominion",
        desc = "Spend up to <#UPGRADE>{1} {RENOWN}</>: Gain {DOMINANCE} equal to the {RENOWN} spent.",
        renown_cap = 5,
    },

    save_face = 
    {
        name = "Save Face",
        desc = "Spend {1} {RENOWN}: Apply {2} {COMPOSURE}.",
        flavour = "'Yeah, I stay pretty calm during intense negotiations.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.renown_cost, self:CalculateComposureText(self.composure_amt) )
        end,

        CanPlayCard = function( self, card, engine, target )
            return self.negotiator:GetModifierStacks("RENOWN") >= self.renown_cost + (target and target.id == "RENOWN" and 1 or 0), loc.format( LOC"CONVO_COMMON.REQ_RENOWN", self.renown_cost + (target and target.id == "RENOWN" and 1 or 0))
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.COMMON,

        renown_cost = 2,
        composure_amt = 5,

        target_self = TARGET_ANY_RESOLVE,

        OnPostResolve = function( self, minigame, targets )
            if self.negotiator:GetModifierStacks("RENOWN") >= self.renown_cost then
                self.negotiator:RemoveModifier("RENOWN", self.renown_cost, self)
                for i,v in ipairs(targets) do
                    v:DeltaComposure(self.composure_amt, self)
                end
            end
        end
    },

    save_face_plus =
    {
        name = "Stone Save Face",
        desc = "Spend {1} {RENOWN}: Apply <#UPGRADE>{2} {COMPOSURE}</>.",
        composure_amt = 7,
    },

    save_face_plus2 =
    {
        name = "Pale Save Face",
        desc = "Spend <#UPGRADE>{1} {RENOWN}</>: Apply {2} {COMPOSURE}.",
        renown_cost = 1,
    },

    ransack =
    {
        name = "Ransack",
        desc = "Draw {1} cards, discard {2} random cards from your hand.",
        flavour = "'I’m keeping all the good parts in.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.draw_count, self.discard_count)
        end,

        cost = 0,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.COMMON,

        draw_count = 3,
        discard_count = 2,
        choice = false,

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards(self.draw_count)
            if self.choice then
                minigame:DiscardCards(self.discard_count, self.discard_count, self)
            else
                for i=1, self.discard_count do
                    minigame:DiscardCard()
                end
            end
        end
    },

    ransack_plus =
    {
        name = "Precise Ransack",
        desc = "Draw {1} cards, discard {2} cards from your hand.",
        cost = 1,
        choice = true,
    },

    ransack_plus2 =
    {
        name = "Visionary Ransack",
        desc = "Draw <#UPGRADE>{1}</> cards, discard {2} random cards from your hand.",
        draw_count = 4,
    },

    fame =
    {
        name = "Fame",
        desc = "Whenever you draw this, increase its damage by {1} for the rest of this negotiation if you have any {RENOWN}.",
        flavour = "'I'm always amazed that people take what I say seriously. I don't even take what I am seriously.'",

        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage_increase)
        end,

        cost = 1,

        flags = CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.UNCOMMON,

        min_persuasion = 1,
        max_persuasion = 2,

        damage_increase = 2,

        deck_handlers = ALL_DECKS,

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source == self and self.bonus then
                    persuasion:AddPersuasion(self.bonus, self.bonus, self)
                end
            end,

            [ EVENT.DRAW_CARD ] = function( self, minigame, card, start_of_turn )
                 if card == self then
                    if self.negotiator:HasModifier("RENOWN") then
                        self.bonus = (self.bonus or 0) + self.damage_increase
                    end
                end
            end,
        },
    },

    fame_plus =
    {
        name = "Planned Fame",
        min_persuasion = 4,
        max_persuasion = 5,
    },

    fame_plus2 =
    {
        name = "Boosted Fame",
        desc = "Whenever you draw this, increase its damage by <#UPGRADE>{1}</> for the rest of this negotiation if you have any {RENOWN}.",
        damage_increase = 3,
    },

    tempt =
    {
        name = "Tempt",
        desc = "Whenever a friendly argument is destroyed, reduce the cost of this card by 1 for the rest of this negotiation.",
        flavour = "'It's just so juicy!'",

        cost = 6,

        flags = CARD_FLAGS.HOSTILE,
        rarity = CARD_RARITY.RARE,

        min_persuasion = 6,
        max_persuasion = 10,

        deck_handlers = ALL_DECKS,

        event_handlers =
        {
            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card == self and self.bonus then
                    acc:AddValue(-self.bonus, self)
                end
            end,

            [ EVENT.MODIFIER_REMOVED ] = function( self, modifier, source )
                if source and modifier.negotiator == self.negotiator then
                    self.bonus = (self.bonus or 0) + 1
                end
            end
        }
    },

    tempt_plus =
    {
        name = "Pale Tempt",
        cost = 4,
    },

    tempt_plus2 =
    {
        name = "Boosted Tempt",
        min_persuasion = 8,
        max_persuasion = 12,
    },

    amnesty =
    {
        name = "Amnesty",
        desc = "{SHIELDED|Shield} all friendly arguments until the start of your next turn.",
        flavour = "'Ahh yes. Thank you brother, be well.'",

        cost = 2,

        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.DIPLOMACY,
        rarity = CARD_RARITY.RARE,

        target_self = TARGET_ANY_RESOLVE,
        target_mod = TARGET_MOD.TEAM,
        auto_target = true,

        loc_strings =
        {
            AMNESTY_SHIELD_DESC = "{SHIELDED}. This argument is shielded until your next turn!",
        },

        OnPostResolve = function( self, minigame, targets )
            local mod = self.negotiator:AddModifier("amnesty", 1, self)
            mod.affected_args = mod.affected_args or {}
            for i,arg in ipairs(targets) do
                arg:SetShieldStatus(true, self.def:GetLocalizedString("AMNESTY_SHIELD_DESC"))
                table.insert(mod.affected_args, arg)
            end
        end,

        modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator then
                        if self.affected_args then
                            for i,v in ipairs(self.affected_args) do
                                v:SetShieldStatus(false)
                            end
                        end
                        self.negotiator:RemoveModifier(self.id, self.stacks, self)
                    end
                end
            },
        },
    },

    amnesty_plus =
    {
        name = "Pale Amnesty",
        cost = 1,
    },

    amnesty_plus2 =
    {
        name = "Fixed Amnesty",
        desc = "{SHIELDED|Shield} all friendly arguments until the start of your next turn.",
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.DIPLOMACY | CARD_FLAGS.STICKY,
    },

    overdrive =
    {
        name = "Overdrive",
        desc = "On your next turn, all your cards are played twice but also {EXPEND}.",
        flavour = "'What took you so long!? I'm on my 16th cup!'",

        cost = 3,

        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MANIPULATE | CARD_FLAGS.BURNOUT,
        rarity = CARD_RARITY.RARE,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("overdrive", 1, self)
        end,

        modifier = 
        {
            hidden = true,
            OnApply = function( self )
                self.ignore = {}
            end,
            event_handlers =
            {
                [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                    if self.negotiator == negotiator then
                        self.active = true
                    end
                end,

                [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                    if negotiator == self.negotiator and self.active then
                        self.negotiator:RemoveModifier(self.id, 1, self)
                    end
                end,

                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    if self.active then
                        if not table.contains(self.ignore, card) then
                            card:SetFlags(CARD_FLAGS.EXPEND)
                            minigame:PlayCard( card )
                            table.insert(self.ignore, card)
                        end
                    end
                end,
            },
        },
    },

    overdrive_plus =
    {
        name = "Pale Overdrive",
        cost = 2,
    },

    overdrive_plus2 =
    {
        name = "Stable Overdrive",
        flags = CARD_FLAGS.EXPEND | CARD_FLAGS.MANIPULATE,
    },

    braggart =
    {
        name = "Braggart",
        desc = "Fill your hand with {brag}. They cost 0 until played.",
        flavour = "'Oh yeah, I've heard of you before.'",

        cost = 3,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,

        bonus = 0,

        OnPostResolve = function( self, minigame, targets )
            local count = minigame:GetHandDeck():GetMaxSize() - minigame:GetHandDeck():CountCards()
            local cards = {}
            for i = 1, count do
                local card = Negotiation.Card( "brag", self.owner )
                if self.upgraded then
                    card:UpgradeCard()
                end
                card:SetFlags( CARD_FLAGS.FREEBIE )
                card:ClearXP()
                card:MakeTemporary()
                table.insert( cards, card )
            end
            minigame:DealCards( cards, minigame:GetHandDeck() )
        end,
    },

    braggart_plus =
    {
        name = "Pale Braggart",
        cost = 2,
    },

    braggart_plus2 =
    {
        name = "Boosted Braggart",
        desc = "Fill your hand with an <#UPGRADE>upgraded</> {brag}. They cost 0 until played.",
        upgraded = true
    },

    incredulous = 
    {
        name = "Incredulous",
        desc = "Gain {1} {RENOWN}.\nCan only be played if you have no {RENOWN}.",
        flavour = "'Nothing gets through to that guy.'",

        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.renown_amt )
        end,

        cost = 0,

        renown_amt = 3,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.DIPLOMACY,

        CanPlayCard = function( self, source, engine, target )
            return not self.negotiator:HasModifier("RENOWN")
        end,

        OnPostResolve = function( self, minigame, targets )
            self.negotiator:AddModifier("RENOWN", self.renown_amt, self)
        end,
    },

    incredulous_plus =
    {
        name = "Famous Incredulous",
        desc = "Gain <#UPGRADE>{1}</> {RENOWN}.\nCan only be played if you have no {RENOWN}.",
        renown_amt = 5,
    },

    incredulous_plus2 =
    {
        name = "Pure Incredulous",
        desc = "Gain <#DOWNGRADE>{1}</> {RENOWN}.",
        renown_amt = 2,
        CanPlayCard = function( self, source, engine, target )
            return true
        end,
    },

    consolidate =
    {
        name = "Consolidate",
        desc = "Spend {1} {RENOWN}: Add {2} resolve to a friendly argument.",
        flavour = "'My associates will be here momentarily.'",

        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.renown_cost, self.resolve_gain )
        end,

        cost = 1,

        rarity = CARD_RARITY.UNCOMMON,
        flags = CARD_FLAGS.DIPLOMACY,

        renown_cost = 3,
        resolve_gain = 2,
        
        target_self = TARGET_ANY_RESOLVE,

        CanPlayCard = function( self, source, engine, target )
            return self.negotiator:GetModifierStacks("RENOWN") >= self.renown_cost, loc.format( LOC"CONVO_COMMON.REQ_RENOWN", self.renown_cost)
        end,

        OnPostResolve = function( self, minigame, targets )
            local stacks = self.negotiator:GetModifierStacks("RENOWN")
            if stacks >= self.renown_cost then
                self.negotiator:RemoveModifier( "RENOWN", self.renown_cost, self )
                for i,target in ipairs(targets) do
                    if target:IsApplied() then
                        target:ModifyResolve(self.resolve_gain, self)
                    end
                end
            end
        end,
    },

    consolidate_plus =
    {
        name = "Pale Consolidate",
        desc = "Spend <#UPGRADE>{1} {RENOWN}</>: Add {2} resolve to a friendly argument.",
        renown_cost = 2,
    },

    consolidate_plus2 =
    {
        name = "Boosted Consolidate",
        desc = "Spend {1} {RENOWN}: Add <#UPGRADE>{2} resolve</> to a friendly argument.",
        resolve_gain = 3,
    },

    parvenu =
    {
        name = "Parvenu",
        desc = "Whenever you gain {RENOWN}, reduce the cost of this card by 1.",
        flavour = "'I don't know where that one came from. He's waiting for you in the back.'",

        cost = 9,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.DIPLOMACY,

        min_persuasion = 6,
        max_persuasion = 8,

        deck_handlers = ALL_DECKS,

        event_handlers =
        {
            [ EVENT.MODIFIER_ADDED ] = function( self, modifier, source )
                if modifier.id == "RENOWN" then
                    self.bonus = (self.bonus or 0) + 1
                end
            end,

            [ EVENT.MODIFIER_CHANGED ] = function( self, modifier, delta, clone )
                if delta and delta > 0 and modifier.id == "RENOWN" then
                    self.bonus = (self.bonus or 0) + 1
                end
            end, 

            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if self.bonus and card == self then
                    acc:AddValue(-self.bonus, self)
                end
            end
        },
    },

    parvenu_plus =
    {
        name = "Pale Parvenu",
        cost = 7,
    },

    parvenu_plus2 =
    {
        name = "Boosted Parvenu",
        min_persuasion = 8,
        max_persuasion = 10,
    },

    thoughtless =
    {
        name = "Thoughtless",
        desc = "{EXPEND} any number of cards in your hand, gain 1 {DOMINANCE} per card expended.",
        flavour = "'Just try and think yourself out of this one!'",

        cost = 2,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        bonus = 0,

        OnPostResolve = function( self, minigame, targets )
            local cards = minigame:ExpendCards()
            local count = #cards

            self.negotiator:AddModifier( "DOMINANCE", count + self.bonus, self )
        end,
    },

    thoughtless_plus =
    {
        name = "Boosted Thoughtless",
        desc = "{EXPEND} any number of cards in your hand, gain 1 {DOMINANCE} per card expended <#UPGRADE>+2</>.",
        bonus =  2,
    },

    thoughtless_plus2 =
    {
        name = "Pale Thoughtless",
        cost = 1,
    },

    domain =
    {
        name = "Domain",
        desc = "Apply {1} {COMPOSURE} to all friendly arguments. Increase by {2} for each Hostile card played this turn.",
        flavour = "'This is my house!'",

        desc_fn = function( self, fmt_str )
            local played = self.engine and self:CountCardsPlayed( self.engine ) or 0
            return loc.format( fmt_str, self.base_comp + played * self.comp_increment, self.comp_increment )
        end,

        cost = 1,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.HOSTILE,
        target_mod = TARGET_MOD.TEAM,
        target_self = TARGET_ANY_RESOLVE,
        auto_target = true,

        base_comp = 2,
        comp_increment = 1,

        CountCardsPlayed = function( self, minigame )
            return minigame:CountCardsPlayed( function(card) return card:IsFlagged(CARD_FLAGS.HOSTILE) end )
        end,

        OnPostResolve = function( self, minigame, targets )
            for i,arg in ipairs(targets) do
                arg:DeltaComposure(self.base_comp + self:CountCardsPlayed( minigame ) * self.comp_increment, self)
            end
        end,
    },

    domain_plus =
    {
        name = "Boosted Domain",
        desc = "Apply <#UPGRADE>{1}</> {COMPOSURE} to all friendly arguments. Increase by {2} for each Hostile card played this turn.",
        base_comp = 4,
    },

    domain_plus2 =
    {
        name = "Stone Domain",
        desc = "Apply {1} {COMPOSURE} to all friendly arguments. Increase by <#UPGRADE>{2}</> for each Hostile card played this turn.",
        comp_increment = 2,
    },

    wild_rant =
    {
        name = "Wild Rant",
        desc = "{IMPROVISE} 2 Hostile cards from your draw pile and play them with random targets.",
        flavour = "'He just came in here screaming something about voices!'",

        cost = 1,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.EXPEND,
        pool_size = 3,

        OnPostResolve = function( self, minigame, targets )
            local options = {}
            for i,card in minigame:GetDrawDeck():Cards() do
                if card:IsFlagged(CARD_FLAGS.HOSTILE) then
                    table.insert(options, card)
                end
            end
            local cards = minigame:ImproviseCards(table.multipick(options, self.pool_size), 2, nil, "fake_it", nil, self)
            for i,card in ipairs(cards) do
                minigame:PlayCard(card)
            end
        end
    },

    wild_rant_plus =
    {
        name = "Enduring Wild Rant",
        flags = CARD_FLAGS.HOSTILE,
    },

    wild_rant_plus2 =
    {
        name = "Boosted Wild Rant",
        desc = "<#UPGRADE>{IMPROVISE_PLUS}</> 2 Hostile cards from your draw pile and play them with random targets.",
        pool_size = 5,
    },

    simmer =
    {
        name = "Simmer",
        desc = "When drawn, reduce the cost of a random Hostile card in your hand by 1 until played.",
        flavour = "'You might want to blow on that one.'",

        cost = 0,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.UNPLAYABLE,
        num_cards = 2,

        event_handlers =
        {
            [ EVENT.DRAW_CARD ] = function( self, minigame, card, start_of_turn )
                 if card == self then
                    self.engine:PushPostHandler( function()
                        self:NotifyTriggeredPre()
                        local options = {}
                        for i,card in minigame:GetHandDeck():Cards() do
                            if card:IsFlagged(CARD_FLAGS.HOSTILE) and minigame:CalculateActionCost(card) > 0 then
                                table.insert(options, card)
                            end
                        end
                        local mod = self.negotiator:FindModifier("simmer")
                        if mod == nil then
                            mod = self.negotiator:AddModifier("simmer", 1, self)
                        end
                        mod.chosen_cards = mod.chosen_cards or {}
                        for i=1,self.num_cards do
                            local chosen = table.arraypick(options)
                            table.arrayremove(options, chosen)
                            table.insert(mod.chosen_cards, chosen)
                        end
                        self:AddXP(1)
                        self:NotifyTriggeredPost()
                    end )
                end
            end,
        },

        modifier =
        {
            hidden = true,
            event_handlers =
            {
                [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                    if self.chosen_cards and table.contains(self.chosen_cards, card) then
                        acc:AddValue(-table.count(self.chosen_cards, card), self)
                    end
                end,

                [ EVENT.POST_RESOLVE ] = function( self, minigame, card )
                    if self.chosen_cards and table.contains(self.chosen_cards, card) then
                        table.arrayremoveall(self.chosen_cards, card)
                    end
                end
            },
        },
    },

    simmer_plus =
    {
        name = "Visionary Simmer",
        desc = "When drawn, reduce the cost of a random Hostile card in your hand by 1 until played.",
        flags = CARD_FLAGS.HOSTILE | CARD_FLAGS.UNPLAYABLE | CARD_FLAGS.REPLENISH,
    },

    simmer_plus2 =
    {
        name = "Boosted Simmer",
        desc = "When drawn, reduce the cost of <#UPGRADE>2 random Hostile cards</> in your hand by 1 until played.",
        num_cards = 2,
    },

    indecision =
    {
        name = "Indecision",
        desc = "Draw {1} {1*card|cards}, discard {2} {2*card|cards}.",
        flavour = "'No wait, I changed my mind.'",

        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.draw_count, self.discard_count )
        end,

        cost = 0,

        flags = CARD_FLAGS.MANIPULATE,
        rarity = CARD_RARITY.UNCOMMON,

        draw_count = 1,
        discard_count = 1,

        OnPostResolve = function( self, minigame, targets )
            minigame:DrawCards( self.draw_count )
            minigame:DiscardCards( self.discard_count, self.discard_count, self )
        end,
    },

    indecision_plus =
    {
        name = "Visionary Indecision",
        desc = "Draw <#UPGRADE>{1} {1*card|cards}</>, discard {2} {2*card|cards}.",
        draw_count = 2,
    },

    indecision_plus2 =
    {
        name = "Boosted Indecision",
        desc = "Draw <#UPGRADE>{1} {1*card|cards}</>, discard <#UPGRADE>{2} {2*card|cards}</>.",
        draw_count = 3,
        discard_count = 3,
    },

    center_of_attention =
    {
        name = "Center of Attention",
        desc = "Add {1} resolve to all friendly arguments.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.resolve_gain)
        end,
        flavour = "'Wouldn’t miss it for the world.'",

        cost = 2,

        rarity = CARD_RARITY.RARE,
        flags = CARD_FLAGS.DIPLOMACY | CARD_FLAGS.EXPEND,

        target_self = TARGET_ANY_RESOLVE,
        target_mod = TARGET_MOD.TEAM,
        auto_target = true,

        resolve_gain = 2,

        OnPostResolve = function( self, minigame, targets )
            for i,arg in ipairs(targets) do
                arg:ModifyResolve( self.resolve_gain, self )
            end
        end
    },

    center_of_attention_plus =
    {
        name = "Pale Center of Attention",
        cost = 1,
    },

    center_of_attention_plus2 =
    {
        name = "Boosted Center of Attention",
        desc = "Add <#UPGRADE>{1} resolve</> to all friendly arguments.",
        resolve_gain = 3,
    },
}

for i, id, carddef in sorted_pairs( CARDS ) do
    carddef.series = "SMITH"

    Content.AddNegotiationCard( id, carddef )
end

local MODIFIERS = 
{
	confrontational =
    {
        name = "Confrontational",
        desc = "Whenever any argument is created, deal 1 damage to a random target.",

        modifier_type = MODIFIER_TYPE.CORE,

        target_enemy = TARGET_ANY_RESOLVE,

        CanPlayCard = function( self, source, engine, target )
            if source == self and self.ignore then
                return target ~= self.ignore
            end

            return true
        end,

        event_handlers =
        {
            [ EVENT.MODIFIER_ADDED ] = function( self, modifier, source )
                if self.active and modifier.modifier_type ~= MODIFIER_TYPE.CORE and not modifier.hidden then
                    self.ignore = modifier
                    self.min_persuasion, self.max_persuasion = 1, 1
                    self.engine:ApplyPersuasion( self )
                    self.min_persuasion, self.max_persuasion = nil, nil
                end
            end,

            -- Only relevant on first turn. 
            [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                self.active = true
            end
        },
    },

    DEFENSELESS = 
    {
        name = "Defenseless",
        desc = "You can not apply {COMPOSURE} while this is active. At the start of your turn, reduce <b>Defenseless</> by 1.",

        modifier_type = MODIFIER_TYPE.ARGUMENT,

        max_resolve = 3,

        -- OnApply = function( self, minigame )
        --     for i,mod in self.negotiator:Modifiers() do
        --         if mod:GetComposure() > 0 then
        --             mod:DeltaComposure(-mod:GetComposure(), self)
        --         end
        --     end
        -- end,

        event_handlers =
        {
            [ EVENT.CALC_DELTA_COMPOSURE ] = function( self, composure_acc, target, source )
                if source and source.negotiator == self.negotiator and composure_acc.value > 0 then
                    composure_acc:ModifyValue(0, self)
                end
            end,

            [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                self.negotiator:RemoveModifier(self.id, 1, self)
            end
        },
    },

    RENOWN = 
    {
        name = "Renown",
        desc = "When destroyed, deal 1 damage to your core argument per stack of <b>Renown</>.\nAt the end of your turn, this deals 1 damage for every two stacks to a random opponent argument. At the start of your turn, <b>Renown</> is reduced by 1.",
        icon = "negotiation/modifiers/prominence.tex",
        no_damage_tt = true,

        max_resolve = 3,

        modifier_type = MODIFIER_TYPE.BOUNTY,

        OnBounty = function( self, card )
            if self.stacks > 0 then
                self.negotiator:AttackResolve(self.stacks, self)
            end
        end,
        target_enemy = TARGET_ANY_RESOLVE,

        OnSetStacks = function( self, old_stacks )
            local dmg = math.floor(self.stacks * 0.5)
            self.min_persuasion, self.max_persuasion = dmg, dmg
        end,

        OnApply = function( self )
            self:PrepareTurn()
        end,

        event_priorities =
        {
            [ EVENT.END_TURN ] = 10,
        },

        event_handlers =
        {
            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ApplyPersuasion()
                end
            end,
            [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self.negotiator:RemoveModifier(self, 1, self)
                end
            end,
        },
    },

    obscurity =
    {
        name = "Obscurity",
        desc = "At the end of {1}'s turn, {1} will lose <#HILITE>{2}</> {RENOWN}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.negotiator:GetName(), self.stacks)
        end,

        modifier_type = MODIFIER_TYPE.ARGUMENT,
        max_resolve = 8,
        event_priorities =
        {
            [ EVENT.END_TURN ] = 12, -- After Renown passive
        },
        event_handlers = 
        {
            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self.negotiator:RemoveModifier("RENOWN", self.stacks, self)
                    self.negotiator:RemoveModifier(self)
                end
            end
        }        
    },

    unknown =
    {
        name = "Unknown",
        desc = "At the end of {1}'s turn, {1} loses all {RENOWN}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.negotiator:GetName())
        end,
        modifier_type = MODIFIER_TYPE.ARGUMENT,
        max_resolve = 10,
        event_priorities =
        {
            [ EVENT.END_TURN ] = 12, -- After Renown passive
        },
        event_handlers = 
        {
            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self.negotiator:RemoveModifier("RENOWN", self.negotiator:GetModifierStacks("RENOWN"), self)
                    self.negotiator:RemoveModifier(self)
                end
            end
        }
    },

    subtle_setup =
    {
        name = "Subtle Setup",
        desc = "Whenever this argument takes damage, deal that much damage to a random opponent argument.",

        max_resolve = 4,

        OnSetStacks = function( self, old_stacks )
            if old_stacks > 0 then
                local delta = self.stacks - old_stacks
                self:ModifyResolve( delta * 4 )
            end
        end,

        target_enemy = TARGET_ANY_RESOLVE,

        event_handlers =
        {
            [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                if target == self and damage > 0 then
                    local options = self.engine:CollectAllTargets( self )
                    if options and #options > 0 then
                        options[math.random(#options)]:AttackResolve(damage, self)
                    end
                end
            end
        },
    },

    chaos_theory =
    {
        name = "Chaos Theory",
        desc = "At the start of your turn, play <#HILITE>{1}</> random {1*card|cards} in your hand for free.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.stacks or 1)
        end,

        max_resolve = 2,

        event_handlers = 
        {
            [ EVENT.HAND_DRAWN ] = function( self, minigame )
                for i=1,self.stacks do
                    local hand = minigame:GetHandDeck().cards
                    if hand and #hand > 0 then
                        self:NotifyTriggered()
                        minigame:PlayCard(hand[math.random(#hand)])
                    end
                end 
            end
        },
    },
}

for id, def in pairs( MODIFIERS ) do
    Content.AddNegotiationModifier( id, def )
end

local FEATURES =
{
    BLIND = {
        name = "Blind",
        desc = "This card selects a target randomly.",
    },

    RENOWNED = {
        name = "Renowned",
        desc = "This card has an additional effect if you have enough {RENOWN}.",
    },

    CONFUSION = {
        name = "Confusion",
        desc = "Insert a {confused} card into your discard pile.",
    },

    BRAVADO = {
        name = "Bravado",
        desc = "This card deals bonus damage equal to the number of Hostility cards played in a row this turn.",
     },
}

for id, data in pairs(FEATURES) do
    local def = NegotiationFeatureDef(id, data)
    Content.AddNegotiationCardFeature(id, def)
end