require "constants"

OPINION = {}

function AddOpinionEvent(id, t)
    t.delta = t.delta or OPINION_DELTAS.NONE
    t.id = id

    t = Content.AddOpinionEvent( t )
    OPINION[id] = t
    return t
end

function FormatOpinionEvent( ev )
    local opinion_event = OPINION[ ev.id ]
    local txt = opinion_event:GetLocalizedTxt()

    local function OpinionFormatter( word, operator, rest )
        if word == "player" then
            local player = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent()
            return loc.format( string.format( "{1%s%s}", operator, rest ), player:LocTable(), ev.data )
        else
            return string.format( "{%s%s%s}", word, operator, rest )
        end
    end
    txt = loc.custom_format( txt, OpinionFormatter )

    if opinion_event.txt_fn then
        txt = opinion_event.txt_fn(txt, ev.agent, ev.data)
    else
        txt = loc.format( txt, ev.agent:LocTable(), ev.data )
    end
    return txt
end

function VerifyOpinionEvent( ev )
    local opinion_event = OPINION[ ev.id ]
    return opinion_event ~= nil
end



OPINION_DELTAS =
{
    NONE = 
    {
        relationship_delta = {},
        desc = "meh."
    },

    DISLIKE = 
    {
        relationship_delta = 
        {
            [RELATIONSHIP.LOVED] = RELATIONSHIP.DISLIKED,
            [RELATIONSHIP.LIKED] = RELATIONSHIP.DISLIKED,
            [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.DISLIKED,
        },
        desc = "-> Disliked",
    },
    TO_DISLIKED = 
    {
        relationship_delta = 
        {
            [RELATIONSHIP.LOVED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.LIKED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.DISLIKED,
        },
        desc = "-> Toward Disliked",
    },

    MAJOR_BAD =
    {
        relationship_delta = 
        {
            
            [RELATIONSHIP.LOVED] = RELATIONSHIP.HATED,
            [RELATIONSHIP.LIKED] = RELATIONSHIP.HATED,
            [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.HATED,
            [RELATIONSHIP.DISLIKED] = RELATIONSHIP.HATED,
        },
        desc = "-> Hated",
    },  

    LIKE = 
    {
        relationship_delta = 
        {
            [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.LIKED,
            [RELATIONSHIP.DISLIKED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.HATED] = RELATIONSHIP.DISLIKED,
        },
        desc = "-> Liked",
    },

    MAJOR_GOOD =
    {
        relationship_delta = 
        {

            [RELATIONSHIP.LIKED] = RELATIONSHIP.LOVED,
            [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.LOVED,
            [RELATIONSHIP.DISLIKED] = RELATIONSHIP.LOVED,
            [RELATIONSHIP.HATED] = RELATIONSHIP.NEUTRAL,   --not so sure about this... but I think it makes sense given our time scale?
        },
        desc = "-> Loved",
    },  

    DIMINISH =
    {
        relationship_delta = 
        {
            [RELATIONSHIP.LOVED] = RELATIONSHIP.LIKED,
            [RELATIONSHIP.LIKED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.DISLIKED,
        },
        desc = "-> Diminish",
    },  

    NEUTRALIZING =
    {
        relationship_delta = 
        {
            [RELATIONSHIP.HATED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.DISLIKED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.LIKED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.LOVED] = RELATIONSHIP.NEUTRAL,
        },
        desc = "-> Neutral",
    },

    NORMALIZING =
    {
        relationship_delta = 
        {
            [RELATIONSHIP.HATED] = RELATIONSHIP.DISLIKED,
            [RELATIONSHIP.DISLIKED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.LIKED] = RELATIONSHIP.LIKED,
            [RELATIONSHIP.LOVED] = RELATIONSHIP.LOVED,
        },
        desc = "-> Neutral",
    },   

    BAD =
    {
        relationship_delta = 
        {
            [RELATIONSHIP.DISLIKED] = RELATIONSHIP.HATED,
            [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.DISLIKED,
            [RELATIONSHIP.LIKED] = RELATIONSHIP.DISLIKED,
            [RELATIONSHIP.LOVED] = RELATIONSHIP.DISLIKED,
        },
        desc = "-> Disliked",
    },

    OPINION_DOWN =
    {
        relationship_delta = 
        {
            [RELATIONSHIP.DISLIKED] = RELATIONSHIP.HATED,
            [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.DISLIKED,
            [RELATIONSHIP.LIKED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.LOVED] = RELATIONSHIP.LIKED,
        },
        desc = "-> Disliked",
    },

    OPINION_UP =
    {
        relationship_delta = 
        {
            [RELATIONSHIP.DISLIKED] = RELATIONSHIP.NEUTRAL,
            [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.LIKED,
            [RELATIONSHIP.LIKED] = RELATIONSHIP.LOVED,
            [RELATIONSHIP.HATED] = RELATIONSHIP.DISLIKED,
        },
        desc = "-> Disliked",
    },
}

----------------------------------------------------------------------------------------------------------------------------------------------



local common_events = 
{
    DEEPENED_RELATIONSHIP_FAIL = 
    {
        delta = OPINION_DELTAS.DIMINISH,
        txt = "Insulted them by being too familiar",
    },
    DEEPENED_RELATIONSHIP = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Gave them a big gift",
    },

    SOLD_OUT_TO_ADMIRALTY = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Sold them out to the Admiralty",
    },
    TRIED_MINOR_GRAFT = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Tried (unsuccessfully) to pull off a minor graft",
    },

    GOT_WORKER_TO_LEAVE = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Got one of their workers to leave their shift",
    },

    BESTED_IN_BATTLE = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Bested them in battle",
    },

    FOUGHT_WITH = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Fought with them in battle",
    },
    HATE_WORKING_FOR_ADMIRALTY = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Hates the Admiralty, and especially those who work with Admiralty",
    },
    ABANDONED = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Abandoned them in battle",
    },

    SETTLED_DEBT = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Paid off their debt",
    },
    
    NEGOTIATED_PEACE_WITH_ENEMY = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Helped them reconcile with an enemy",
    },

    REFUSED_DRINK = {
        delta =OPINION_DELTAS.DISLIKE,
        txt = "Refused to drink with them",
    },

    STARTED_FIGHT_IN_BAR = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Started a fight they had to stop",
    },

    GOT_PATRON_TO_LEAVE = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Made them lose customers",
    },
    
    DID_NOT_HELP = 
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Did not help them out",
    },

    FLATTERY =
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "You flattered them",
    },
    
    GRANT_EXTENSION = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Granted them an extension on their protection money",
    },
    
    SAVED_LIFE = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Saved their life",
    },

    DID_LOYALTY_QUEST = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Helped them with a hard problem",
    },

    EXTORTION =
    {
        delta = OPINION_DELTAS.BAD,
        txt = "Extorted them for money",
    },
    MET = 
    {
        txt = "Met",
    },
    TRIED_TO_SET_UP = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Tried to set them up",
    },

    WASTED_TIME = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Wasted their time",
    },

    -- TODO: remove me (only here for backwards compatability for save games with legacies)
    LEGACY_FACTION_LIKE = {
        delta = OPINION_DELTAS.LIKE,
        txt = "You have a good reputation with this faction",

    },

    -- TODO: remove me (only here for backwards compatability for save games with legacies)
    LEGACY_LOVE = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Loved you in a past life",
    },
    ELITIST_FACTION_DISLIKE = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "You have a poor reputation with this faction."
    },

    TRIED_TO_SWINDLE = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Tried to swindle them out of money",
    },

    CAUGHT_STEALING = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Caught Stealing",
    },

    GOT_KICKED_OUT = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Got them kicked out",
    },
    GOT_FIRED = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Got them fired",
    },

    CAUGHT_THIEF = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Caught a thief for them",
    },
    
    SOCIAL_FAVOUR = {
        delta = OPINION_DELTAS.LIKE,
        txt = "'Helped' them out socially",
    },

    SOCIALIZED = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Socialized with them",
    },

    CALL_IN_FAVOUR = {
        delta = OPINION_DELTAS.DIMINISH,
        txt = "Called in a favour",
    },

    -- SHADY_CONNECTION = {
    --     delta = OPINION_DELTAS.LIKE,
    --     txt = "Became a shady connection",
    -- },

    REFUSED_TO_HELP = {
        delta = OPINION_DELTAS.DIMINISH,
        txt = "Refused to help them when asked"
    },

    WORKED_AGAINST_THE_ADMIRALTY =  {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Worked against the Admiralty",
    },
    
    HELPED = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Helped them out of a tough situation",
    },
    INTIMIDATED = {
        delta = OPINION_DELTAS.DIMINISH,
        txt = "Got them intimidated",
    },

    ATTEMPT_INTIMIDATION = {
        delta = OPINION_DELTAS.DIMINISH,
        txt = "Attempted to intimidate",
    },

    TOOK_JOB = {
        delta = OPINION_DELTAS.NONE,
        txt = "Agreed to do a job for them",
    },

    INSTIGATED = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Got into an argument",
    },
    
    MADE_PEACE = {
        delta = OPINION_DELTAS.NEUTRALIZING,
        txt = "Made peace",
    },

    BOUGHT_DRINK = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Bought them a drink",
    },

    MURDERED = { 
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Murdered",
    },

    KILLED_FRIEND = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Killed their friend {2#agent}",
        txt_fn = function(txt, agent, data)
            return loc.format(txt, agent:LocTable(), data.revenge_agent)
        end,
    },

    RECONCILED_GRUDGE = {
        delta = OPINION_DELTAS.NEUTRALIZING,
        txt = "Reconciled a grudge",
    },

    ATTACKED = { 
        delta = OPINION_DELTAS.BAD,
        txt="Was attacked by you",
    },

    BETRAYED = { 
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt="Betrayed",
    },

    PAID_OFF = {
        delta = OPINION_DELTAS.NEUTRALIZING,
        txt = "Paid off"
    },

    LET_LEAVE_TOWN = {
        delta = OPINION_DELTAS.NEUTRALIZING,
        txt = "Let them leave town"
    },


    DEFENDED = { 
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt="Defended them",
    },

    SPARED_LIFE = { 
        delta = OPINION_DELTAS.LIKE,
        txt = "Spared their life",
    },
    
    SPARED_LIFE_LOVED = { 
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Spared their life",
    },
   
    SAVED_FROM_CAPTIVITY = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Rescued them from captivity",
    },

    TOOK_CAPTIVE =
    {
        delta = OPINION_DELTAS.BAD,
        txt = "Took them captive",
    },
    HELPED_THEM_DISAPPEAR = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Helped them skip town to escape their enemies",
    },
    TRIED_TAKE_CAPTIVE =
    {
        delta = OPINION_DELTAS.BAD,
        txt = "Tried to take them captive",
    },

    ROBBED = { 
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Robbed them",
    },
    
    PICKPOCKETED = { 
        delta = OPINION_DELTAS.DISLIKE,
        txt="Attempted theft",
    },

    DID_VIOLENCE_IN_SHOP = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Committed violence in their place of business",
    },
    
    BREAK_AND_ENTER = {
        delta = OPINION_DELTAS.DISLIKE,
        txt="Broke into their place",
    },
    
    DISAPPROVE_MINOR = { 
        delta = OPINION_DELTAS.DISLIKE,
        txt="Did something they didn't like",
    },

    THREATENED_TO_COLLECT_BOUNTY = { 
        delta = OPINION_DELTAS.DISLIKE,
        txt="Threatened to collect their bounty",
    },
    
    DISAPPROVE_MAJOR = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt="Did something they really didn't like",
    },
    
    APPROVE = {
        delta = OPINION_DELTAS.LIKE,
        txt="Did something they liked",
    },

    INSULT = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "You insulted them",
    },
    
    JOB_FAILED_MINOR = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Job failed: <#QUEST>{2#quest}</>",
        txt_fn = function(txt, agent, data)
            return loc.format( txt, agent:LocTable(), data.quest )
        end
    },
    
    JOB_MINOR = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Job completed: <#QUEST>{2#quest}</>",
        txt_fn = function(txt, agent, data)
            return loc.format( txt, agent:LocTable(), data.quest )
        end
    },

    JOB_MINOR_POOR = {
        delta = OPINION_DELTAS.TO_DISLIKE,
        txt = "Job completed poorly: <#QUEST>{2#quest}</>",
        txt_fn = function(txt, agent, data)
            return loc.format( txt, agent:LocTable(), data.quest )
        end
    },

    FOUND_A_JOB = {
        delta = OPINION_DELTAS.OPINION_UP,
        txt = "Found a job for them",
    },

    LOST_A_JOB = {
        delta = OPINION_DELTAS.OPINION_DOWN,
        txt = "Made them lose their job",
    },

    RAN_AWAY = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Ran away from trouble",
    },

    HEARD_KERFUFFLE = {
        delta = OPINION_DELTAS.NONE,
        txt = "Heard a suspicious commotion",
    },

    WITNESS_WANTON_VIOLENCE = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Saw you commit an act of wanton violence",
    },
    
    REFUSE_JOB = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "You refused a job",
    },

    ANIMOSITY = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Remembers your animosity during negotiation",
    },
}

for k,v in pairs(common_events) do
    AddOpinionEvent(k, v)
end
