local patron_weights = {
    RISE_REBEL = 1,
    RISE_PAMPHLETEER = 1,
    RISE_RADICAL = 1,

    SPARK_BARON_GOON = 1,
    SPARK_BARON_TASKMASTER = 1,
    SPARK_BARON_PROFESSIONAL = 1,

    JAKES_RUNNER = 1,
    JAKES_SMUGGLER = 1,
    JAKES_LIFTER = 1,

    BOGGER_CLOBBER = .5,
    BOGGER_CULTIVATOR = .5,
}


local brawl = require "content/quests/brawl/brawl_util"


local data = table.extend(brawl.base_data)
{
    home_loc_name = "The Last Stand",
    home_loc_desc = "Hebbel's bar",
    home_loc_plax = "INT_Neutral_Bar",
    bartender_alias = "HEBBEL",

    merchant_list = {
        "grafts", "negotiation", "battle", "pets"
    },

    bosses = {
        {"SPARK_BARON_BOSS", "DRONE_GOON", "HESH_BOSS", "MERCENARY_BOSS", "RENTORIAN_BOSS", "WEEZIL", "BRUT"},
        {"JAKES_ASSASSIN", "JAKES_ASSASSIN2", "FLEAD_QUEEN", "AUTOMECH_BOSS", "MARK_NINE_NINE", "TWIN_BOSS_1"},
        {"SPARK_SECOND", "RISE_SECOND", "SHROOG", "DRUSK_1", "SHREDMAW", "DROAD"},
        {"MURDER_BAY_BANDIT_CONTACT", "MURDER_BAY_ADMIRALTY_CONTACT"},
        {"BOGGER_BOSS"},
    }
}


data.MakeBrawlSchedule = function(data)

    local all_valid_quests = {}
    for id, def in pairs( Content.GetAllQuests() ) do
        if def.qtype == QTYPE.SIDE and not def:HasTag("manual_spawn") and def:FilterForAct( "SAL_BRAWL" ) and (def.character_specific == nil or table.contains(def.character_specific, "ROOK")) then
            table.insert(all_valid_quests, def)
        end
    end
    local used_bosses = {}
    local selected_quests = {}

    local bs = BrawlSchedule()
    bs:SetCurrentHome("home_bar")
        :Merchants(data.merchant_list)
        :QuestPhase("starting")
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 1, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 1, 2, 1, 1))
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 1, 2, 1, 1))
        :SetCurrentHome("drop_zone")
        :QuestPhase("campsite_recovery")
        :Night()        
        :Merchants({"coins", "mettle"})
        :Boss(brawl.PickBoss(data.bosses[1], used_bosses), true)
        :SetCurrentHome("home_bar")
        :Bonus(data.all_bonuses, 2)
        :Sleep()

    bs:SetDifficulty(2)
        :Merchants(data.merchant_list)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 2, 2, 1, 1))
        :QuestPhase("get_parasites")
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 2, 2, 1, 1))
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 2, 2, 1, 1))
        :SetCurrentHome("drop_zone")
        :QuestPhase("campsite_recovery")
        :Night()
        :Merchants({"coins", "mettle"})
        :Boss(brawl.PickBoss(data.bosses[2], used_bosses), true)
        :SetCurrentHome("home_bar")
        :Sleep()

    bs:SetDifficulty(3)
        :Merchants(data.merchant_list)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 3, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 3, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Boss(brawl.PickBoss(data.bosses[1], used_bosses) )
        :SetCurrentHome("drop_zone")
        :QuestPhase("campsite_recovery")
        :Night()
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 3, 2, 1, 1))
        :Merchants({"coins", "mettle"})
        :QuestPhase("campsite_recovery_2")
        :Boss(brawl.PickBoss(data.bosses[3], used_bosses), true)
        :SetCurrentHome("home_bar")
        :Sleep()

    bs:SetDifficulty(4)
        :Merchants(data.merchant_list)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 4, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 4, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Boss(brawl.PickBoss(data.bosses[2], used_bosses))
        :SetCurrentHome("drop_zone")
        :QuestPhase("campsite_recovery")        
        :Night()
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 4, 2, 1, 1))
        :QuestPhase("campsite_recovery_2")
        :Merchants({"coins", "mettle"})      
        :Boss(brawl.PickBoss(data.bosses[4], used_bosses), true)
        :SetCurrentHome("home_bar")
        :Sleep()

    bs:SetDifficulty(5)
        :Merchants(data.merchant_list)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 5, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 5, 2, 1, 1))
        :SetCurrentHome("drop_zone")
        :QuestPhase("campsite_recovery")
        :Night(3)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 5, 2, 1, 1))
        :SetCurrentHome("home_bar")
        :Merchants(data.merchant_list)
        :Boss(brawl.PickBoss( data.bosses[5], used_bosses))
        :Win()
    
    return bs.events
end

local QDEF = brawl.CreateBrawlQuest("ROOK_BRAWL", data)


QDEF:AddObjective{
    id = "get_parasites",
    desc = "",
    hide_in_overlay = true,
}

QDEF:AddConvo("get_parasites")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !bashful
                * You feel that old, familiar sting...
            ]],
            DIALOG_INTRO_2 = [[
                player:
                    %angryTakenAback
                    What will I become?
            ]],

        }
        :Fn(function(cxt) 
            cxt.quest:Complete("get_parasites")
            cxt:Dialog("DIALOG_INTRO")
            cxt:GainCards{"twig", "stem"}
            cxt:Dialog("DIALOG_INTRO_2")
            StateGraphUtil.AddEndOption(cxt)
        end)

QDEF:AddObjective{
    id = "campsite_recovery",
    desc = "",
    hide_in_overlay = true,
}

QDEF:AddConvo("campsite_recovery")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You assess the campsite and try to figure out what to do before nightfall.
            ]],
            OPT_RESTORE_RESOLVE = "Meditate for a while",
            OPT_HEAL_HEALTH = "Scavenge for fruits and berries",
            OPT_LIGHT_A_FIRE_AND_WAIT = "Light a fire and wait for a visitor",
            DIALOG_LIGHT_A_FIRE_AND_WAIT = [[
                * You light a big, obvious campfire.
            ]],
        }
        :Fn(function(cxt)
            
            local VISITOR_CHANCE = .5
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_HEAL_HEALTH")
                :DeltaHealth(15)
                :Fn(function(cxt)
                    cxt.quest:Complete("campsite_recovery")
                end)

            local resolve, resolve_max = cxt.caravan:GetResolve()
            cxt:Opt("OPT_RESTORE_RESOLVE")
                :ReqCondition(resolve < resolve_max, "REQ_AT_FULL_RESOLVE" )
                :DeltaResolve(10)
                :Fn(function(cxt)
                    cxt.quest:Complete("campsite_recovery")
                end)

            cxt:Opt("OPT_LIGHT_A_FIRE_AND_WAIT")
                :Dialog("DIALOG_LIGHT_A_FIRE_AND_WAIT")
                :Fn( function( cxt ) 
                    cxt:GoTo("STATE_VISITOR")
                end )
        end)

    :State("STATE_VISITOR")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !injured
                * A weary traveller arrives.
            ]],
            OPT_HIRE = "Hire {agent}",
            DIALOG_HIRE = [[
                agent:
                    !agree
                    I could use the money!
                    !exit
            ]],

            OPT_SHARE_FIRE = "Share your camp fire",
            DIALOG_SHARE_FIRE = [[
                agent:
                    !happy
                * You make room at your campfire. The two of you swap battle stories, and make a real connection.
                agent:
                    !exit
                * The hour is late, and your new friend has to leave.
            ]],

            OPT_ROB = "Rob {agent}",
            DIALOG_ROB = [[
                agent:
                    !scared
                * You invite the traveller to stay, and then when their guard is down you liberate all of the money from their pockets.
                player:
                    !shrug
                agent:
                    !exit
            ]],
        }
        :Fn(function(cxt)

            local defs = {
                {
                    "HEAVY_LABORER",
                    "PEARLIE",
                    "ADMIRALTY_GOON",
                    "SPARK_BARON_GOON",
                    "RISE_REBEL",
                },
                {
                    "HEAVY_LABORER",
                    "OSHNU_WRANGLER",
                    "JAKES_RUNNER",
                    "ADMIRALTY_GUARD",
                    "SPARK_BARON_TASKMASTER",
                },
                {
                    "SPARK_BARON_PROFESSIONAL",
                    "JAKES_LIFTER",
                    "RISE_RADICAL",
                    "CHEMIST",
                },
                {
                    "SPARK_BARON_PROFESSIONAL",
                    "BOGGER_CULTIVATOR",
                    "BOGGER_CLOBBER",
                },
                {
                    "LUMINARI",
                    "BOGGER_CULTIVATOR",
                    "BOGGER_CLOBBER",
                },
            }
            local def = table.arraypick( defs[math.min( TheGame:GetGameState():GetCurrentBaseDifficulty(), #defs)] )
            local agent = TheGame:GetGameState():AddSkinnedAgent(def)

            cxt:TalkTo(agent)
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_HIRE")
                :Dialog("DIALOG_HIRE")
                :DeliverMoney(cxt:GetAgent():GetCombatStrength()*75)
                :RecruitMember( PARTY_MEMBER_TYPE.HIRED )
                :CompleteQuest("campsite_recovery")
                :DoneConvo()

            cxt:Opt("OPT_SHARE_FIRE")
                :Dialog("DIALOG_SHARE_FIRE")
                :ReceiveOpinion("shared_fire")
                :CompleteQuest("campsite_recovery")
                :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                :DoneConvo()

            cxt:Opt("OPT_ROB")
                :Dialog("DIALOG_ROB")
                :ReceiveMoney( cxt:GetAgent():GetRenown()*75 )
                :ReceiveOpinion("robbed")
                :CompleteQuest("campsite_recovery")
                :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                :DoneConvo()

        end)

QDEF:AddObjective{
    id = "campsite_recovery_2",
    desc = "",
    hide_in_overlay = true,
}

QDEF:AddConvo("campsite_recovery_2")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You sense a greater danger approaching. You have some time to prepare.
            ]],
            OPT_RESTORE_RESOLVE = "Meditate for a while",
            OPT_HEAL_HEALTH = "Scavenge for fruits and berries",
            OPT_CONTINUE = "Continue",
            DIALOG_CONTINUE = "You decide you've had your fill already."
        }
        :Fn(function(cxt)
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_HEAL_HEALTH")
                :DeltaHealth(20)
                :Fn(function(cxt)
                    cxt.quest:Complete("campsite_recovery_2")
                end)

            local resolve, resolve_max = cxt.caravan:GetResolve()
            cxt:Opt("OPT_RESTORE_RESOLVE")
                :ReqCondition(resolve < resolve_max, "REQ_AT_FULL_RESOLVE" )
                :DeltaResolve(15)
                :Fn(function(cxt)
                    cxt.quest:Complete("campsite_recovery_2")
                end)

            cxt:Opt("OPT_CONTINUE")
                :Dialog("DIALOG_CONTINUE")
                :Fn(function(cxt) 
                    cxt.quest:Complete("campsite_recovery_2")
                end)
        end)


QDEF:AddOpinionEvents{

    shared_fire =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Shared your camp fire",
    },
    robbed =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Robbed them",
    },
}

QDEF:AddQuestLocation{
    cast_id = "home_bar",
    name = "The Last Stand",
    desc = "Hebbel's bar",
    plax = "INT_Neutral_Bar",
    show_agents = true,
    tags = {"tavern"},
    indoors = true,
    work = 
    {
        bartender = CreateClosedJob( PHASE_MASK_ALL, "Bartender", CHARACTER_ROLES.PROPRIETOR, "GROG_N_DOG_ITEMS"),
    },
    patron_data = {
        num_patrons = 4,
        patron_generator = function(location)
            local def = weightedpick(patron_weights)
            TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
        end
    },
    on_assign = function(quest, location)
        AgentUtil.TakeJob(quest:GetCastMember("bartender"), location, "bartender")
    end,
}





QDEF:AddQuestLocation{
    cast_id = "drop_zone",
    name = "The Drop Spot",
    plax = "EXT_DROPSITE",

}
