local patron_weights = {
    ADMIRALTY_CLERK = 1,
    ADMIRALTY_GOON = 1,
    ADMIRALTY_GUARD = 1,
    ADMIRALTY_PATROL_LEADER = 1,

    BANDIT_GOON = 1,
    BANDIT_GOON2 = 1,
    BANDIT_RAIDER = 1,

    JAKES_RUNNER = 1,
    JAKES_SMUGGLER = 1,
    JAKES_LIFTER = 1,

    LABORER = 1,
    HEAVY_LABORER = 1,
    WEALTHY_MERCHANT = 1,
    FOREMAN = 1,

    ZEALOT = .5,
    PRIEST = .5,
    LUMINARI = .5,
}


local brawl = require "content/quests/brawl/brawl_util"
local data = table.extend(brawl.base_data)
{
    home_loc_name = "The Grog n' Dog",
    home_loc_desc = "Fssh's bar, and Sal's home base",
    home_loc_plax = "INT_GrogNDog_1",
    bartender_alias = "FSSH",
    merchant_list = {
        "grafts", "negotiation", "battle", "pets"
    },
    bosses = {
        {"SPARK_BARON_BOSS", "DRONE_GOON", "HESH_BOSS", "MERCENARY_BOSS", "RENTORIAN_BOSS", "WEEZIL", "BRUT", "RISE_TURNCOAT_BOSS"},
        {"JAKES_ASSASSIN", "JAKES_ASSASSIN2", "FLEAD_QUEEN", "AUTOMECH_BOSS", "MARK_NINE_NINE", "TWIN_BOSS_1"},
        {"SPARK_SECOND", "RISE_SECOND", "SHROOG", "DRUSK_1", "SHREDMAW", "DROAD"},
        {"MURDER_BAY_BANDIT_CONTACT", "MURDER_BAY_ADMIRALTY_CONTACT"},
        {"KASHIO"},
    }
}

data.MakeBrawlSchedule = function(data)

    local all_valid_quests = {}
    for id, def in pairs( Content.GetAllQuests() ) do
        if def.qtype == QTYPE.SIDE and not def:HasTag("manual_spawn") and def:FilterForAct( "SAL_BRAWL" ) and (def.character_specific == nil or table.contains(def.character_specific, "SAL")) then
            table.insert(all_valid_quests, def)
        end
    end
    local used_bosses = {}
    local selected_quests = {}

    --local day_1_quests = 
    --local day_2_quests = brawl.PickQuests(all_valid_quests, selected_quests, 2, 3, 1, 1)
    --local day_3_quests = brawl.PickQuests(all_valid_quests, selected_quests, 3, 3, 1, 1)
    --local day_4_quests = brawl.PickQuests(all_valid_quests, selected_quests, 4, 3, 1, 1)
    --local day_5_quests = brawl.PickQuests(all_valid_quests, selected_quests, 5, 3, 1, 1)

    
    local bs = BrawlSchedule()
    
    bs:SetCurrentHome("grog")
        :Merchants(data.merchant_list)
        :QuestPhase("starting")
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 1, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 1, 2, 1, 1))
        :Night()
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 1, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Merchants({"mettle"})
        :Boss(brawl.PickBoss(data.bosses[1], used_bosses), true)
        :Sleep()

    bs:SetDifficulty(2)
        :Merchants(data.merchant_list)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 2, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 2, 2, 1, 1))
        :Night()
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 2, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Merchants({"mettle"})
        :Boss(brawl.PickBoss(data.bosses[2], used_bosses), true)
        :Sleep()

    bs:SetDifficulty(3)
        :Merchants(data.merchant_list)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 3, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 3, 2, 1, 1))
        :Night()
        :Boss(brawl.PickBoss(data.bosses[1], used_bosses) )
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 3, 2, 1, 1))
        :Merchants({"mettle"})
        :Boss(brawl.PickBoss(data.bosses[3], used_bosses), true)
        :Sleep()

    bs:SetDifficulty(4)
        :Merchants(data.merchant_list)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 4, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 4, 2, 1, 1))
        :Night()
        :Boss(brawl.PickBoss(data.bosses[2], used_bosses))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 4, 2, 1, 1))
        :Merchants({"mettle"})
        :Boss(brawl.PickBoss(data.bosses[4], used_bosses), true)
        :Sleep()

    bs:SetDifficulty(5)
        :Merchants(data.merchant_list)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 5, 2, 1, 1))
        :Bonus(data.all_bonuses, 2)
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 5, 2, 1, 1))
        :Night(3)
        :Bonus(data.all_bonuses, 2)
        :Merchants({"mettle"})
        :Quests(brawl.PickQuests(all_valid_quests, selected_quests, 5, 2, 1, 1))
        :Merchants(data.merchant_list)
        :Boss(brawl.PickBoss( data.bosses[5], used_bosses))
        :Win()
    
    print (serpent.block(bs.events))
    return bs.events
end

local QDEF = brawl.CreateBrawlQuest("SAL_BRAWL", data)

QDEF.on_post_load = function(quest)
    if not quest:GetCastMember("home") then
        quest:AssignCastMember("home", quest:GetCastMember("grog"))
    end

    if not quest:GetCastMember("bartender") then
        quest:AssignCastMember("bartender", quest:GetCastMember("grog"):GetWorkPosition("bartender"):GetAgent())
    end
end



QDEF:AddQuestLocation{
    cast_id = "grog",
    name = "The Grog n' Dog",
    desc = "Fssh's bar, and Sal's home base",
    plax = "INT_GrogNDog_1",
    show_agents = true,
    tags = {"tavern"},
    indoors = true,
    work = 
    {
        bartender = CreateClosedJob( PHASE_MASK_ALL, "Bartender", CHARACTER_ROLES.PROPRIETOR, "GROG_N_DOG_ITEMS"),
    },
    patron_data = {
        num_patrons = 4,
        patron_generator = function(location)
            local def = weightedpick(patron_weights)
            TheGame:GetGameState():AddSkinnedAgent(def):GetBrain():SendToPatronize(location)
        end

    },
    on_assign = function(quest, location)
        AgentUtil.TakeJob(quest:GetCastMember("bartender"), location, "bartender")
    end,

}
