local fun = require "util/fun"
local BrawlUtil = require "content/quests/brawl/brawl_util"


local QDEF = QuestDef.Define
{
    title = "Defeat the Boss",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    act_filter = "SAL_BRAWL",
    desc = [[{target} is looking to kill you. Get {target.himher} first!]],
    cooldown = EVENT_COOLDOWN.NONE, 
    qtype = QTYPE.STORY,
    focus = QUEST_FOCUS.COMBAT,
}

:AddCast{
    cast_id = "target",
    no_validation = true,
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( quest.param.boss_id or "ADMIRALTY_GOON" ) )
    end,
}

:AddCast{
    cast_id = "possibly_ally",
    cast_fn = function(quest, t)
        local def_id = fun(Content.GetAllCharacterDefs())
                            :filter(function(v) return not v.is_template and v.renown and v.species == nil and not v.boss and not v.unique and math.abs(v.renown - quest:GetRank()) <= 1 and QuestUtil.FilterPromoted(v) end)
                            :keys()
                            :randomPick()
        if def_id then
            table.insert( t, quest:CreateSkinnedAgent( def_id ) )
        end
    end,
}
    
:AddObjective{
    id = "fight_boss",
    title = "Defeat the boss!",
    --desc = "kill {target} for {giver}.",
}

:AddObjective{
    id = "start",
    title = "Acquire some help!",
    desc = "Get someone to back you up in your fight.",
    on_complete = function(quest)
        quest:Activate("fight_boss")
    end
}



QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * You meet {agent} on the road.
            ]],

            OPT_CONVINCE = "Convince {agent} to help you",
            DIALOG_CONVINCE = [[
                player:
                    You know, I could really use some help here.
            ]],
            DIALOG_WON = [[
                * You've convinced {agent} to help you in your upcoming fight.
            ]],
            DIALOG_LOST = [[
                agent:
                    I don't think so. I prefer being alive, you know?
                * Unconvinced, {agent} leaves.
                agent:
                    !exit
            ]],
            OPT_GO_ALONE = "Continue on alone",
            DIALOG_GO_ALONE = [[
                player:
                    !exit
                agent:
                    !exit
            ]]
        }
        :Fn(function(cxt) 
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
            cxt:TalkTo("possibly_ally")
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    difficulty = cxt.quest:GetRank() + 1,
                }
                    :OnSuccess()
                        :Fn(function( cxt, minigame ) 
                            cxt:Dialog("DIALOG_WON")
                            cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.HIRED )
                            cxt.quest:Complete("start")
                            cxt:GoTo( "FIGHT_BOSS" )
                    end)
                    :OnFailure()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_LOST")
                            cxt.quest:Complete("start")
                            cxt:GoTo( "FIGHT_BOSS" )
                    end)

            cxt:Opt("OPT_GO_ALONE")
                :Dialog("DIALOG_GO_ALONE")
                :Fn(function( cxt )
                    cxt.quest:Complete("start")
                    cxt:GoTo( "FIGHT_BOSS" )
                end)
        end)

    :State("FIGHT_BOSS")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * You find your target.
            ]],

            OPT_ATTACK = "Attack!",
            DIALOG_ATTACK = [[
                player:
                    !fight
                agent:
                    !right
            ]],

            DIALOG_WON = [[
                * You win.
            ]],

        }
        :Fn(function(cxt) 
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
            cxt:TalkTo("target")
            cxt:Dialog("DIALOG_INTRO")

            local final_boss = BrawlUtil.PeekNextStep( "win" )
            local battle_flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.BOSS_FIGHT
            local add_conditions = {}
            local enemies = {cxt:GetAgent()}
            if cxt:GetAgent():GetContentID() == "KASHIO" then
                local AUCTION_DATA = require "content/grafts/kashio_boss_fight_defs"
                local candidates = {}
                for k,v in pairs(AUCTION_DATA.CONDITION_LOOKUPS) do
                    table.insert(candidates, v)
                end
                table.shuffle(candidates)
                
                local NUM_PICKS = 3
                
                for k = 1, math.min(NUM_PICKS, #candidates) do
                    table.insert(add_conditions, candidates[k])
                end
            end
            if final_boss then
                battle_flags = battle_flags | BATTLE_FLAGS.NO_REWARDS
            end

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    flags = battle_flags,
                    enemies = enemies,
                    post_fade = final_boss,
                    on_start_battle = function(battle) 
                        local fighter = battle:GetFighterForAgent(cxt:GetAgent())
                        if fighter then
                            for k,v in ipairs(add_conditions) do
                                fighter:AddCondition(v)
                            end
                            fighter:ClearPreparedCards()
                            fighter:PrepareTurn()
                        end
                    end

                }

                    :OnWin()
                        :Fn(function() 
                            -- cxt:Dialog("DIALOG_WON")
                            if final_boss then
                                -- Don't give rewards after the final boss
                            else
                                ConvoUtil.GiveBossRewards(cxt)
                            end
                            cxt.quest:Complete()
                            cxt:End()
                        end)
        end)

