local fun = require "util/fun"


local factions = {
    {   faction = "ADMIRALTY", 
        backup = "ADMIRALTY_PATROL_BACKUP", 
        patrol = "ADMIRALTY_PATROL"
    },
    
    {   faction = "BANDITS", 
        backup = "BANDIT_PATROL_BACKUP", 
        patrol = "BANDIT_PATROL"
    },
    
    {
        faction = "SPARK_BARONS", 
        backup = "SPARK_BARON_BACKUP", 
        patrol = "SPARK_BARON_PATROL"
    },
    
    {
        faction = "RISE", 
        backup = "RISE_BACKUP", 
        patrol = "RISE_PATROL"
    },
}

local by_faction = {}
for k,v in pairs(factions) do
    by_faction[v.faction] = v
end



local QDEF = QuestDef.Define
{
    title = "Faction Battle",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    act_filter = "SAL_BRAWL",
    desc = [[The {1} want you to lead a group of goons against the {2}.]],
    desc_fn = function( quest, fmt_str )
        return loc.format(fmt_str, Content.GetFaction( quest.param.giver_faction):GetLocalizedName(), Content.GetFaction(quest.param.enemy_faction):GetLocalizedName())
    end,
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.COMBAT,
    qtype = QTYPE.SIDE,

    postcondition = function(quest)
        quest.param.giver_faction = table.arraypick(factions).faction

        local t = {} 
        for k,v in ipairs(factions) do
            if v.faction ~= quest.param.giver_faction then
                table.insert(t, v.faction)
            end
        end
        quest.param.enemy_faction = table.arraypick(t)
        return true
    end,
}
    

:AddObjective{
    id = "start",
    title = "Fight the Battle",
    --desc = "kill {target} for {giver}.",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !angry
                * You arrive at the battlefield.
                    
            ]],

            OPT_ATTACK = "Attack!",
            DIALOG_ATTACK = [[
                player:
                    !fight
                agent:
                    !right
            ]],
            DIALOG_WON = [[
                * Any battle you can walk away from is its own kind of victory.
            ]]

        }
        :Fn(function(cxt) 

            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))

            local opfor =  CreateCombatParty(by_faction[cxt.quest.param.enemy_faction].patrol, cxt.quest:GetRank()+1, cxt.location)
            local faction = cxt.quest.param.giver_faction
            local allies =  CreateCombatParty(by_faction[faction].backup, math.floor(cxt.quest:GetRank()/2))
            cxt:TalkTo(opfor[1])
            cxt:Dialog("DIALOG_INTRO")


            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    allies = allies,
                    no_oppo_limit = true,
                    flags = BATTLE_FLAGS.SELF_DEFENCE
                }
                    :OnWin()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_WON")
                            ConvoUtil.GiveQuestRewards(cxt)
                            cxt.quest:Complete()
                        end)
        end)