local fun = require "util/fun"


local QDEF = QuestDef.Define
{
    title = "Be the Heavy",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    desc = [[Intimidate or beat up {target}. But remember: don't kill {target.himher}!]],
    act_filter = "SAL_BRAWL",
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.BALANCED,
    qtype = QTYPE.SIDE,

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("target") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,    
}


:AddCast{
    cast_id = "target",
    cast_fn = function(quest, t)
        local def_id = fun(Content.GetAllCharacterDefs())
                            :filter(function(v) return not v.is_template and v.renown and v.species == nil and not v.boss and not v.unique and
                                v.base_def ~= "PLAYER_BASE" and
                                ( math.abs(v.renown - quest:GetRank()) <= 1 or math.abs(v.combat_strength - quest:GetRank()) <= 1) and
                                QuestUtil.FilterPromoted(v)
                            end)
                            :keys()
                            :randomPick()
        if def_id then
            table.insert( t, quest:CreateSkinnedAgent( def_id ) )
        end
    end,
}


:AddObjective{
    id = "start",
    title = "Scare {target}",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                target:
                    !right
                player:
                    !left
                target:
                    What are you looking at? You don't scare me!
            ]],

            OPT_INTIMIDATE = "Intimidate",
            DIALOG_INTIMIDATE = [[
                player:
                    Don't make me hurt you!
            ]],

            DIALOG_SUCCESS = [[
                target:
                    !scared
                    Please don't hurt me!
                * {target} is sufficiently scared.
            ]],

            DIALOG_FAILURE = [[
                target:
                    !laugh
                    Ha! They should have sent someone with a backbone!
                    !exit
                * You have failed to intimidate {target}.
            ]],

            OPT_ATTACK = "Attack {target}",
            DIALOG_ATTACK = [[
                player:
                    !fight
                agent:
                    !fight
            ]],

            DIALOG_WON = [[
                {dead?
                    * {agent} is dead. Whoops.
                }
                {not dead?
                    target:
                        !scared
                        Please don't hurt me!
                }
            ]],
            FIGHT_REASON = "Don't Kill {target}!",

        }
        :Fn(function(cxt) 
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
            cxt:TalkTo('target')


            CreateCombatBackup(cxt:GetCastMember("target"), cxt:GetCastMember("target").combat_backup or "MERCENARY_BACKUP", cxt.quest:GetRank())

            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_INTIMIDATE")
                :Dialog("DIALOG_INTIMIDATE")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.INTIMIDATION
                }
                    :OnSuccess()
                        :Fn(function() 
                            ConvoUtil.GiveQuestRewards(cxt)
                            cxt.quest:Complete()
                        end)
                    :OnFailure()
                        :Fn(function() 
                            cxt.quest:Fail()
                        end)

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    reason_fn = function(minigame)
                        return loc.format(cxt:GetLocString("FIGHT_REASON"))
                    end,
                }
                    :OnWin()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_WON")
                            if cxt:GetAgent():IsDead() then
                                
                                cxt.quest:Fail()
                            else
                                ConvoUtil.GiveQuestRewards(cxt)
                                cxt.quest:Complete()
                            end
                        end)
        end)