local fun = require "util/fun"
local negotiation_defs = require "negotiation/negotiation_defs"

Content.AddNegotiationModifier( "secret_intel",
{
    name = "Secret Intel",
    desc = "When destroyed, gain this argument and a bonus quest reward upon winning the negotiation. If <b>Secret Intel</> is destroyed under your ownership, you lose the negotiation!",

    max_resolve = 10,
    max_stacks = 1,
    modifier_type = negotiation_defs.MODIFIER_TYPE.BOUNTY,

    OnInit = function( self )
        self:SetResolve( 10, MODIFIER_SCALING.MED )
    end,

    OnBounty = function( self )
        if self.negotiator:IsPlayer() then
            self.engine:Lose()
        else
            local mod = self.anti_negotiator:CreateModifier( "secret_intel" )
            mod:SetResolve( 5, MODIFIER_SCALING.LOW )
        end
    end,
})


local QDEF = QuestDef.Define
{
    title = "Pry For Intel!",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    desc = [[Ferret out as much intel out of {target} without letting them get suspicious!]],
    act_filter = "SAL_BRAWL",
    cooldown = EVENT_COOLDOWN.NONE,
    focus = QUEST_FOCUS.NEGOTIATION,
    qtype = QTYPE.SIDE,
}


:AddCast{
    cast_id = "target",
    cast_fn = function(quest, t)
        local def_id = fun(Content.GetAllCharacterDefs())
                            :filter(function(v) return not v.is_template and v.renown and v.species == nil and not v.boss and not v.unique and math.abs(v.renown - quest:GetRank()) <= 1 and QuestUtil.FilterPromoted(v) end)
                            :keys()
                            :randomPick()
        if def_id then
            table.insert( t, quest:CreateSkinnedAgent( def_id ) )
        end
    end,
}


:AddObjective{
    id = "start",
    title = "Pry {target} for intel",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                target:
                    !right
                    Looking for something?
            ]],

            OPT_NEGOTIATE = "Pry for intel",
            DIALOG = [[
                player:
                    Oh, just looking to chat...
            ]],

            DIALOG_SUCCESS = [[
                target:
                    Well, thanks for the chat!
            ]],

            DIALOG_FAILURE = [[
                target:
                    !laugh
                    Ha! You're as transparent as they come!
                    I'm not telling you a thing!
                    !exit
            ]],

            DIALOG_INTEL = [[
                * Rewards for <b>Secret Intel</b>: {1}!
            ]]
        }
        :Fn(function(cxt) 
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
            cxt:TalkTo('target')

            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_NEGOTIATE")
                :Dialog("DIALOG")
                :Negotiation{
                    on_start_negotiation = function(minigame)
                        for i = 1, 3 do
                            minigame:GetOpponentNegotiator():CreateModifier( "secret_intel", 1 )
                        end
                    end
                    }
                    :OnSuccess()
                        :Fn(function( cxt, minigame ) 
                            cxt:Dialog("DIALOG_SUCCESS")

                            local count = minigame:GetPlayerNegotiator():GetModifierStacks( "secret_intel" )
                            cxt:Dialog( "DIALOG_INTEL", count )
                            for i = 1, count do
                                cxt:Run( function()
                                    local extra_reward, extra_reward_data = QuestUtil.GenerateExtraReward( cxt.quest:GetRank() )
                                    ConvoUtil.GiveReward(cxt, cxt.quest:GetRank(), extra_reward, extra_reward_data )
                                end )
                            end

                            ConvoUtil.GiveQuestRewards(cxt)
        
                            cxt.quest:Complete()
                    end)
                    :OnFailure()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_FAILURE")
                            cxt.quest:Fail()
                    end)
        end)