local fun = require "util/fun"


local QDEF = QuestDef.Define
{
    title = "Murder for Hire",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    act_filter = "SAL_BRAWL",
    desc = [[You've been hired to make {target} dead.]],
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.COMBAT,
    qtype = QTYPE.SIDE,
    
    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("target") then
            fighter:AddCondition("WANTED_DEAD")
        end
    end,    

}


:AddCast{
    cast_id = "target",
    cast_fn = function(quest, t)
        local def_id = fun(Content.GetAllCharacterDefs())
                            :filter(function(v) return not v.is_template and v.combat_strength and v.species == nil and not v.boss and not v.unique and math.abs(v.combat_strength - quest:GetRank()) <= 1 and QuestUtil.FilterPromoted(v)  end)
                            :keys()
                            :randomPick()
        if def_id then
            table.insert( t, quest:CreateSkinnedAgent( def_id ) )
        end
    end,
}


:AddObjective{
    id = "start",
    title = "Kill {target}",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                target:
                    !right
                {player_sal?
                    * You find {target}. Your daggers are thirsty.
                }
                {player_rook?
                    * Your trigger fingers itch when you find {target}.
                }
                player:
                    !left
                target:
                    What do <i>you</> want?
                *** <b>Your job is to kill {target}.</>
            ]],

            OPT_ATTACK = "Attack!",
            DIALOG_ATTACK = [[
                player:
                    !fight
                target:
                    !right
            ]],

            DIALOG_KILLED = [[
                * You've killed the target.
            ]],

            DIALOG_SPARED = [[
                agent:
                    !injured
                    Ahhhh!
                    !exit
                * You let {target} get away.
            ]],

            FIGHT_REASON = "Kill {target}!"
        }
        :Fn(function(cxt) 
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
            cxt:TalkTo('target')
            if cxt:GetAgent().combat_backup then
                CreateCombatBackup(cxt:GetAgent(), cxt:GetAgent().combat_backup, cxt.quest:GetRank())
            end
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    no_oppo_limit = true,
                    reason_fn = function(minigame)
                        return loc.format(cxt:GetLocString("FIGHT_REASON"))
                    end,

                }
                    :OnWin()
                        :Fn(function() 

                            if (cxt:GetAgent():IsDead()) then
                                cxt:Dialog("DIALOG_KILLED")
                                ConvoUtil.GiveQuestRewards(cxt)
                                cxt.quest:Complete()
                            else
                                cxt:Dialog("DIALOG_SPARED")
                                cxt.quest:Fail()
                            end
                        end)
        end)