local fun = require "util/fun"


local QDEF = QuestDef.Define
{
    title = "Make Peace",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    desc = [[{target1} and {target2} are going to kill each other! Convince them not to.]],
    act_filter = "SAL_BRAWL",
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.NEGOTIATION,
    qtype = QTYPE.SIDE,
}


:AddCast{
    cast_id = "target1",
    cast_fn = function(quest, t)
        local def_id = fun(Content.GetAllCharacterDefs())
                            :filter(function(v) return not v.is_template and v.renown and v.species == nil and not v.boss and not v.unique and math.abs(v.renown - quest:GetRank()) <= 1 and QuestUtil.FilterPromoted(v)  end)
                            :keys()
                            :randomPick()
        if def_id then
            table.insert( t, quest:CreateSkinnedAgent( def_id ) )
        end
    end,
}
:AddCast{
    cast_id = "target2",
    cast_fn = function(quest, t)
        local def_id = fun(Content.GetAllCharacterDefs())
                            :filter(function(v) return not v.is_template and v.renown and v.species == nil and not v.boss and not v.unique and math.abs(v.renown - quest:GetRank()) <= 1 and QuestUtil.FilterPromoted(v) end)
                            :keys()
                            :randomPick()
        if def_id then
            table.insert( t, quest:CreateSkinnedAgent( def_id ) )
        end
    end,
}


:AddObjective{
    id = "start",
    title = "Prevent the battle!",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                target2:
                    !right
                    !angry
                target1:
                    !left
                    !angry
                * You find {target1} and {target2} about to murder each other.
            ]],

            OPT_CONVINCE = "Convince {1} to stop",
            DIALOG_CONVINCE_FIRST = [[
                player:
                    !left
                first:
                    !right
                    Why should I stop?

            ]],
            DIALOG_CONVINCE_FIRST_SUCCESS = [[
                first:
                    !crossed
                    OK, but you need to convince {second}, too.
                second:
                    !right
            ]],
            
            DIALOG_CONVINCE_FAIL_FIRST = [[
                first:
                    You know what? We should both just kill <b>you</>!
            ]],
            DIALOG_CONVINCE_FAIL_SECOND = [[
                second:
                    !fight
                    You're both working against me!
            ]],

            DIALOG_CONVINCE_SECOND = [[
                agent:
                    I'm not a pushover!
            ]],

            DIALOG_CONVINCE_SECOND_SUCCESS = [[
                second:
                    !happy
                    You know what, maybe {first}'s not so bad.
                    !exit
                * You did some good here today.
            ]],

            DIALOG_WON_FIGHT = [[
                * This was <b>not</> what you were hired to do.
            ]],
            
            SIT_MOD = "{agent} is no pushover!",


        }
        :Fn(function(cxt) 
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
            cxt:TalkTo('target1')

            cxt:Dialog("DIALOG_INTRO")

            local casting = {
                {first = cxt.quest:GetCastAgent("target1"), second = cxt.quest:GetCastAgent("target2")},
                {first = cxt.quest:GetCastAgent("target2"), second = cxt.quest:GetCastAgent("target1")}
            }

            for _, data in pairs(casting) do

                cxt:Opt("OPT_CONVINCE", data.first)
                    :Fn(function() 
                        cxt:ReassignCastMember('first', data.first)
                        cxt:ReassignCastMember('second', data.second)
                    end)
                    :Dialog("DIALOG_CONVINCE_FIRST")
                    :Negotiation{ 
                        target_agent = data.first,
                    }
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_FIRST_SUCCESS")
                            :Fn(function()
                                cxt:Opt("OPT_CONVINCE", data.second)
                                    :Negotiation{
                                            target_agent = data.second,
                                            situation_modifiers =
                                            {
                                                { value = 5*math.ceil(cxt.quest:GetRank()/2), text = cxt:GetLocString("SIT_MOD") }
                                            },
                                        }
                                        :OnSuccess()
                                            :Dialog("DIALOG_CONVINCE_SECOND_SUCCESS")
                                            :Fn(function() 
                                                ConvoUtil.GiveQuestRewards(cxt)
                                                cxt.quest:Complete()
                                            end)
                                        :OnFailure()
                                            :Dialog("DIALOG_CONVINCE_FAIL_SECOND")
                                            :Fn(function() 
                                                
                                                cxt:Opt("OPT_DEFEND")
                                                    :Battle{
                                                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                                                            enemies = {data.second},
                                                            allies = {data.first}
                                                        }
                                                        :OnWin()
                                                            :Dialog("DIALOG_WON_FIGHT")
                                                            :FailQuest()
                                            end)

                            end)
                            
                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_FAIL_FIRST")
                            :Fn(function() 

                                cxt:Opt("OPT_DEFEND")
                                    :Battle{
                                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                                            enemies = {cxt.quest:GetCastAgent('target1'), cxt.quest:GetCastAgent('target2')}
                                        }
                                        :OnWin()
                                            :Dialog("DIALOG_WON_FIGHT")
                                            :FailQuest()
                            end)
            end
        end)