local fun = require "util/fun"

local SURVIVAL_TURNS = 6

local QDEF = QuestDef.Define
{
    title = "Mech survival!",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    act_filter = "SAL_BRAWL",
    desc = [[Survive 6 rounds against the robot horde]],
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.COMBAT,
    qtype = QTYPE.SIDE,
    rank = {3,5},
    character_specific = {"ROOK"},
}

    
:AddObjective{
    id = "start",
    title = "Survive 6 rounds!",
}

:AddCastByAlias{
    cast_id = "giver",
    alias = "RISE_REBEL",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    Just in time!
                    You gotta help me hold off these Baron mechs in time for everyone to hide!
            ]],

            OPT_ACCEPT_ITEM = "Ask for equipment (Receive {1#card})",
            DIALOG_GIVE_ITEM = [[
                agent:
                    !giver
                    Use this if you have to, it's going to be a rough ride.
            ]],

            OPT_ATTACK = "Hold off the mechs!",
            DIALOG_WON = [[
                * You high tail it with the survivors through a hidden tunnel to escape the army of mechs.
                {not dead_ally?
                    agent:
                        !thumb
                        That was close. Thanks!
                }
                {dead_ally?
                    * One of the survivors thanks you for the time you bought them and pays your due.
                }
            ]],

        }
        :SetLooping( true )
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
                local enemies = {}
                for i = 1, 3 do
                    local mechs = CreateCombatParty("MECHS", cxt.quest:GetRank(), cxt.location)
                    table.arrayadd( enemies, mechs )
                end
                table.shuffle( enemies )
                cxt.enc.scratch.enemies = enemies

                cxt:TalkTo( cxt.quest:GetCastAgent( "giver" ))
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_ACCEPT_ITEM", "portable_bunker" )
                    :Dialog( "DIALOG_GIVE_ITEM" )
                    :GainCards( {"portable_bunker"} )
            end

            cxt:Opt("OPT_ATTACK")
                :Battle{
                    enemies = cxt.enc.scratch.enemies,
                    allies = { cxt.quest:GetCastAgent( "giver" ) },
                    survival_turns = SURVIVAL_TURNS,
                    flags = BATTLE_FLAGS.SURVIVAL,
                }

                    :OnWin()
                        :Fn(function() 
                            cxt.quest.param.dead_ally = cxt.quest:GetCastAgent("giver"):IsDead()
                            cxt:Dialog("DIALOG_WON")
                            ConvoUtil.GiveQuestRewards(cxt)
                            cxt.quest:Complete()
                            cxt:End()
                        end)
        end)