local fun = require "util/fun"


local factions = {
    ADMIRALTY = "ADMIRALTY_PATROL",
    BANDITS = "BANDIT_PATROL",
    SPARK_BARONS = "SPARK_BARON_PATROL",
    RISE = "RISE_PATROL",
    --MONSTERS = "MONSTERS",
    JAKES = "JAKES_PATROL",
    BOGGERS = "BOGGER_PATROL",
    CULT_OF_HESH = "HESH_PATROL",
    FEUD_CITIZEN = "CIVILIAN_PATROL",
}


local QDEF = QuestDef.Define
{
    title = "Choosing Sides",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    act_filter = "SAL_BRAWL",
    desc = [[You need to pick sides.]],
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.COMBAT,
    qtype = QTYPE.SIDE,
}
    

:AddObjective{
    id = "start",
    title = "Fight the Battle",
    --desc = "kill {target} for {giver}.",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                leader1:
                    !left
                    !angry
                leader2:
                    !right
                    !angry
                * A battle is about to start. You need to choose a side.
                    
            ]],

            OPT_PICK_SIDE = "Side with {1}",
            DIALOG_ATTACK = [[
                left:
                    !fight                
                right:
                    !fight                
                * You enter the fray!
            ]],
            DIALOG_WON_SURVIVORS = [[
                player:
                    !left
                right:
                    !exit
                * As the dust settles, your allies turn on you!
                agent:
                    !angry
                    !right
                * You're going to have to fight for your life!
            ]],

            DIALOG_WON_FINISHED = [[
                right:
                    !exit
                * As the dust settles, those still left standing limp off to nurse their wounds.
            ]],

            DIALOG_WON_NO_SURVIVORS = [[
                player:
                    !left
                * You are the last one standing. Is this what victory looks like?
            ]],

            DIALOG_WON_2 = [[
                * You defeat the turncoats.
            ]],

        }
        :Fn(function(cxt) 

            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))

            local picked = fun(factions):keys():shuffle():slice(1, 3):value()
            local faction_1 = picked[1]
            local faction_2 = picked[2]

            local opfor1 =  CreateCombatParty(factions[faction_1], cxt.quest:GetRank()+1, cxt.location)
            local opfor2 =  CreateCombatParty(factions[faction_2], cxt.quest:GetRank()+1, cxt.location)

            cxt:ReassignCastMember("leader1", opfor1[1])
            cxt:ReassignCastMember("leader2", opfor2[1])
            cxt:Dialog("DIALOG_INTRO")
            local teams = {
                {opfor1, opfor2},
                {opfor2, opfor1},
            }

            for _, data in ipairs(teams) do
                cxt:Opt("OPT_PICK_SIDE",  data[1][1])
                    :Dialog("DIALOG_ATTACK")
                    :Battle{
                        allies = data[1],
                        enemies = data[2],
                        no_oppo_limit = true,
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                    }
                        :OnWin()
                            :Fn(function() 
                                local new_enemies = {}
                                local new_allies = {}
                                

                                for k,v in pairs(data[1]) do
                                    if not v:IsRetired() then
                                        table.insert(new_enemies, v)
                                    end
                                end

                                for k,v in pairs(data[2]) do
                                    if not v:IsRetired() then
                                        table.insert(new_allies, v)
                                    end
                                end
                                table.sort(new_allies, function(a,b) return b:GetCombatStrength() < a:GetCombatStrength()end)
                                table.sort(new_enemies, function(a,b) return b:GetCombatStrength() < a:GetCombatStrength()end)
                                

                                if #new_allies == 0 then
                                    cxt:Dialog("DIALOG_WON_NO_SURVIVORS")
                                    ConvoUtil.GiveQuestRewards(cxt)
                                    cxt.quest:Complete()

                                elseif #new_enemies == 0 then
                                    cxt:Dialog("DIALOG_WON_FINISHED")
                                    ConvoUtil.GiveQuestRewards(cxt)
                                    cxt.quest:Complete()
                                    
                                else
                                    cxt:TalkTo(new_enemies[1])
                                    cxt:Dialog("DIALOG_WON_SURVIVORS")
                                    cxt:Opt("OPT_DEFEND")
                                        
                                        :Battle{
                                            enemies = new_enemies,
                                            allies = new_allies,
                                            no_oppo_limit = true,
                                            flags = BATTLE_FLAGS.SELF_DEFENCE
                                        }
                                        :OnWin()
                                            :Dialog("DIALOG_WON_2")
                                            :Fn(function() 
                                                ConvoUtil.GiveQuestRewards(cxt)
                                                cxt.quest:Complete()
                                            end)
                                end
                            end)
            end
        end)

