local fun = require "util/fun"


local QDEF = QuestDef.Define
{
    title = "Rescue Mission",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    act_filter = "SAL_BRAWL",
    desc = [[Rescue {giver#agent} from {giver.hisher} enemies, using your words or your fists.]],
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.BALANCED,
    qtype = QTYPE.SIDE,

}


:AddCast{
    cast_id = "giver",
    provider = true,
    condition = function(agent, quest) 
        return agent:GetRelationship() >= RELATIONSHIP.NEUTRAL and math.abs(agent:GetRenown() - quest:GetRank()) <= 1
    end,
    events = {
        agent_retired = function(quest) end,
    }
    
}
:AddCastFallback{
    cast_fn = function(quest, t)

        local def_id = fun(Content.GetAllCharacterDefs())
                            :filter(function(v) return not v.is_template and v.renown and v.species == nil and not v.boss and not v.unique and math.abs(v.renown - quest:GetRank()) <= 1 and QuestUtil.FilterPromoted(v) end)
                            :keys()
                            :randomPick()

        if def_id then
            table.insert( t, quest:CreateSkinnedAgent( def_id ) )
        end
    end,
}
    

:AddObjective{
    id = "start",
    title = "Rescue {giver}",
    --desc = "kill {target} for {giver}.",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !right
                * You find {giver}'s captors.
                player:
                    !left
                agent:
                    You're not getting away from us, {giver}!
                    
            ]],

            OPT_ATTACK = "Attack!",
            DIALOG_ATTACK = [[
                player:
                    !fight
                target:
                    !right
            ]],

            DIALOG_KILLED = [[
                * {giver} died in the fight. You get nothing.
            ]],

            DIALOG_RESCUED = [[
                giver:
                    !right
                    Thanks, {player}!
                    !exit
            ]],

            OPT_INTIMIDATE = "Demand {agent} release {giver}",
            DIALOG_INTIMIDATION = [[
                player:
                    !angry
                    You're going to let {giver} go!
                    Or else!
            ]],
            DIALOG_NEGOTIATION_SUCCESS = [[
                agent:
                    Fine! Take {giver.himher}!
                giver:
                    !happy
                    !right
                    Thank you!
            ]],
            DIALOG_NEGOTIATION_FAILURE = [[
                agent:
                    Ha! You aren't scary at all!
                player:
                    !injured
                agent:
                    !exit
                * The captors beat you soundly, and leave with {giver}

            ]],
            FIGHT_REASON = "Rescue {giver}!",

        }
        :Fn(function(cxt) 
            
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
            local party_defs = {}
            
            if cxt.quest:GetCastMember("giver"):GetFactionID() ~= "ADMIRALTY" then
                table.insert(party_defs, "ADMIRALTY_PATROL")
            end

            if cxt.quest:GetCastMember("giver"):GetFactionID() ~= "BANDITS" then
                table.insert(party_defs, "BANDIT_PATROL")
            end
            
            if cxt.quest:GetCastMember("giver"):GetFactionID() ~= "CULT_OF_HESH" then
                table.insert(party_defs, "HESH_PATROL")
            end
            
            if cxt.quest:GetCastMember("giver"):GetFactionID() ~= "JAKES" then
                table.insert(party_defs, "JAKES_PATROL")
            end
            
            if cxt.quest:GetCastMember("giver"):GetFactionID() ~= "RISE" then
                table.insert(party_defs, "RISE_PATROL")
            end

            if cxt.quest:GetCastMember("giver"):GetFactionID() ~= "SPARK_BARONS" then
                table.insert(party_defs, "SPARK_BARON_PATROL")
            end


            local opfor =  CreateCombatParty(table.arraypick(party_defs), cxt.quest:GetRank(), cxt.location, true)

            cxt:TalkTo(opfor[1])
            cxt:Dialog("DIALOG_INTRO")


            cxt:Opt("OPT_INTIMIDATE")
                :Dialog("DIALOG_INTIMIDATION")
                :Negotiation{
                            flags = NEGOTIATION_FLAGS.INTIMIDATION,
                            helpers = {"giver"}
                        }
                    :OnSuccess()
                        :Dialog("DIALOG_NEGOTIATION_SUCCESS")
                        :Fn(function() 
                            ConvoUtil.GiveQuestRewards(cxt)
                            cxt.quest:Complete()
                        end)
                        :End()
                    :OnFailure()
                        :Dialog("DIALOG_NEGOTIATION_FAILURE")
                        :DeltaHealth(-10)
                        :Fn(function() 
                            cxt.quest:Fail()                            
                        end)
                        :End()


            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    allies = {cxt.quest:GetCastMember("giver")},
                    no_oppo_limit = true,
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    reason_fn = function(minigame)
                        return loc.format(cxt:GetLocString("FIGHT_REASON"))
                    end,

                }
                    :OnWin()
                        :Fn(function() 

                            if (cxt.quest:GetCastAgent("giver"):IsDead()) then
                                cxt:Dialog("DIALOG_KILLED")
                                cxt.quest:Fail()
                            else
                                cxt:Dialog("DIALOG_RESCUED")
                                ConvoUtil.GiveQuestRewards(cxt)
                                cxt.quest:Complete()
                            end
                        end)
        end)