local fun = require "util/fun"


local QDEF = QuestDef.Define
{
    title = "Try to Look Scary",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    desc = [[Scare off as many goons as possible before the fight.]],
    act_filter = "SAL_BRAWL",
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.BALANCED,
    qtype = QTYPE.SIDE,
}

:AddCast{
    cast_id = "target",
    cast_fn = function(quest, t)
        local defs = { 
                            {"ADMIRALTY_CLERK", "SPARK_BARON_GOON", "RISE_REBEL"}, 
                            {"ADMIRALTY_CLERK", "SPARK_BARON_GOON", "RISE_REBEL"}, 
                            {"ADMIRALTY_PATROL_LEADER", "SPARK_BARON_TASKMASTER", "RISE_PAMPHLETEER"}, 
                            {"ADMIRALTY_PATROL_LEADER", "SPARK_BARON_TASKMASTER", "RISE_PAMPHLETEER"}, 
                            {"ADMIRALTY_PATROL_LEADER", "SPARK_BARON_TASKMASTER", "RISE_PAMPHLETEER"} } 
        local def= table.arraypick(defs[math.min(quest:GetRank(), #defs)])
        table.insert( t, quest:CreateSkinnedAgent( def ) )
    end,
}


:AddObjective{
    id = "start",
    title = "Scare away {target}'s support",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                target:
                    !right
                * You find {target}.
                player:
                    !left
                target:
                    I'm not afraid of you! I have backup!
            ]],

            OPT_SCARE_OFF = "Scare off {agent}'s protectors",
            DIALOG_SCARE_OFF = [[
                player:
                    !threaten
                    Yeah? We'll see about that!
            ]],

            DIALOG_SUCCESS = [[
                agent:
                    !scared
                    Wait, where are you going?!
                    
            ]],

            DIALOG_FAILURE = [[
                {scared_goons?
                    agent:
                        Get back here, you quitters!
                    * The goons you scared away rejoin the fight.
                }

                {not scared_goons?
                    agent:
                        Ha! We don't scare so easy!
                }

                agent:
                    !cruel
                    This is going to be fun!

            ]],

            NEGOTIATION_REASON = "Scare off the supporters!",

        }
        :Fn(function(cxt) 
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
            cxt:TalkTo('target')

            cxt:Dialog("DIALOG_INTRO")

            local fight_allies = CreateCombatBackup(cxt:GetAgent(), cxt:GetAgent().combat_backup, cxt.quest:GetRank()+1)
            

            cxt:Opt("OPT_SCARE_OFF")
                :Dialog("DIALOG_SCARE_OFF")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.ALLY_SCARE, -- | NEGOTIATION_FLAGS.NO_CORE_RESOLVE,
                    enemy_resolve_required = 10 + cxt.quest:GetRank()*5,
                    fight_allies = fight_allies,
                    reason_txt = cxt:GetLocString("NEGOTIATION_REASON")
                }
                    :OnSuccess()
                        :Fn(function(cxt, minigame) 
                            
                            local keep_allies = {}
                            for i, modifier in minigame:GetOpponentNegotiator():Modifiers() do
                                if modifier.id == "FIGHT_ALLY_SCARE" and modifier.ally_agent then
                                    table.insert( keep_allies, modifier.ally_agent )
                                end
                            end

                            for k,v in pairs(fight_allies) do
                                if not table.arrayfind(keep_allies, v) then
                                    v:MoveToLimbo()
                                end
                            end

                        end)
                        :Dialog("DIALOG_SUCCESS")
                        :GoTo("STATE_FIGHT")

                    :OnFailure()
                        :Fn(function(cxt, minigame) 
                            cxt.enc.scratch.scared_goons = not (minigame:GetOpponentNegotiator():FindCoreArgument() and minigame:GetOpponentNegotiator():FindCoreArgument():GetShieldStatus())
                        end)
                        :Dialog("DIALOG_FAILURE")
                        :GoTo("STATE_FIGHT")
        end)

        :State("STATE_FIGHT")
            :Loc{
                DIALOG_WIN = [[
                    agent:
                        !exit
                    * Words or blades, you always make your point.
                ]],
            }
            :Fn(function(cxt) 

                    cxt:Opt("OPT_DEFEND")
                        :Battle{
                            no_oppo_limit = true,
                        }
                            :OnWin()
                                :Dialog("DIALOG_WIN")
                                :Fn(function() 
                                    ConvoUtil.GiveQuestRewards(cxt)
                                    cxt.quest:Complete()
                                end)
            end)
