local fun = require "util/fun"


local QDEF = QuestDef.Define
{
    title = "Undermine",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    desc = [[You need to kill {target}. But you'll need to deal with {target.hisher} bodyguard {bodyguard} first!]],
    act_filter = "SAL_BRAWL",
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.BALANCED,
    qtype = QTYPE.SIDE,
    
    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("target") then
            fighter:AddCondition("WANTED_DEAD")
        end
    end,    

}


:AddCast{
    cast_id = "target",
    cast_fn = function(quest, t)
        local def_id = fun(Content.GetAllCharacterDefs())
                            :filter(function(v) return not v.is_template and v.renown and v.species == nil and not v.boss and not v.unique and math.abs(v.renown - quest:GetRank()) <= 1 and QuestUtil.FilterPromoted(v)  end)
                            :keys()
                            :randomPick()
        if def_id then
            table.insert( t, quest:CreateSkinnedAgent( def_id ) )
        end
    end,
}

:AddCast{
    cast_id = "bodyguard",
    cast_fn = function(quest, t)
            local def_ids = {
                {"BANDIT_GOON", "BANDIT_GOON2", "ADMIRALTY_GOON", "RISE_REBEL"},
                {"ADMIRALTY_GUARD", "JAKES_RUNNER", "RISE_RADICAL", "SPARK_BARON_GOON"},
                {"LUMINARI", "SPARK_BARON_PROFESSIONAL","JAKES_SMUGGLER"},
                {"BANDIT_RAIDER", "JAKES_LIFTER", "ZEALOT", "ADMIRALTY_PATROL_LEADER"},
            }
            table.insert( t, quest:CreateSkinnedAgent( table.arraypick(def_ids[math.min( quest:GetRank(), #def_ids)]) ))
        end,
}

:AddObjective{
    id = "start",
    title = "Kill {target}",
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                target:
                    !right
                * You find {target} and {target.hisher} guard.
                player:
                    !left
                bodyguard:
                    !right
                    Nobody talks to {target} without going through me first.
            ]],

            OPT_CONVINCE_LEAVE = "Scare {agent} away",
            DIALOG_CONVINCE_LEAVE = [[
                player:
                    !angry
                    You better leave if you want to live.
            ]],
            DIALOG_CONVINCE_LEAVE_SUCCESS = [[
                agent:
                    !scared
                    Oh no!
                    !exit
                target:
                    !right
                    Where are you going?
                *** <b>Now kill {target}!</>
            ]],
            
            DIALOG_FAILURE = [[
                bodyguard:
                    !fight
                    Stop talking!
                *** <b>Now kill {target}!</>
            ]],

            OPT_CONVINCE_HELP = "Convince {agent} to switch sides",
            DIALOG_CONVINCE_HELP = [[
                player:
                    You're on the wrong side of this...
            ]],
            DIALOG_CONVINCE_HELP_SUCCESS = [[
                agent:
                    !happy
                    You're right!
                target:
                    !right
                    What are you two talking about?
                *** <b>Now kill {target}!</>
            ]],
            
            SIT_MOD = "{agent} would be breaking {agent.hisher} contract!",

        }
        :Fn(function(cxt) 
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))
            
            cxt:TalkTo(cxt.quest:GetCastMember("bodyguard"))
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_CONVINCE_LEAVE")
                :Dialog("DIALOG_CONVINCE_LEAVE")
                :Negotiation{
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        hinders =  {"target"},
                    }
                    :OnSuccess()
                        :Dialog("DIALOG_CONVINCE_LEAVE_SUCCESS")
                        :Fn(function() 
                            cxt.quest.param.guard_left = true 
                            cxt:GetAgent():MoveToLimbo()
                        end)
                        :GoTo("STATE_FIGHT")
                    
                    :OnFailure()
                        :Dialog("DIALOG_FAILURE")
                        :Fn(function() cxt.quest.param.guard_blocking = true end)
                        :GoTo("STATE_FIGHT")
            
            cxt:Opt("OPT_CONVINCE_HELP")
                :Dialog("DIALOG_CONVINCE_HELP")
                    :Negotiation{
                        situation_modifiers = {
                                { value = 5*math.ceil(cxt.quest:GetRank()/2), text = cxt:GetLocString("SIT_MOD") }                    
                            },
                            hinders = {"target"},
                        }
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_HELP_SUCCESS")
                            :Fn(function() cxt.quest.param.guard_helping = true end)
                            :GoTo("STATE_FIGHT")
                        
                        :OnFailure()
                            :Dialog("DIALOG_FAILURE")
                            :Fn(function() cxt.quest.param.guard_blocking = true end)
                            :GoTo("STATE_FIGHT")

        end)

    :State("STATE_FIGHT")
        :Loc{
            OPT_FIGHT = "Kill {target}!",
            DIALOG_FAILURE_GOT_AWAY = [[
                * {target} got away as you where fighting.
            ]],

            DIALOG_WON_FIGHT = [[
                {dead?
                    * The target is dead.
                }
                {not dead?
                    agent:
                        !exit
                    * The target slips away.
                }
            ]],
            FIGHT_REASON = "Kill {target}!",
        }
        :Fn(function(cxt) 

            local enemies = {}
            local allies = {}
            local flags = nil

            if cxt.quest.param.guard_blocking then
                table.insert(enemies, cxt.quest:GetCastMember("bodyguard"))
                cxt:TalkTo(cxt.quest:GetCastMember("bodyguard"))
            else
                if cxt.quest.param.guard_helping then
                    table.insert(allies, cxt.quest:GetCastMember("bodyguard")) 
                end
                table.insert(enemies, cxt.quest:GetCastMember("target"))
                cxt:TalkTo(cxt.quest:GetCastMember("target"))
            end

            cxt:Opt("OPT_FIGHT")
                :Battle{
                    flags = flags,
                    enemies = enemies,
                    allies = allies,
                    reason_fn = function(minigame)
                        return loc.format(cxt:GetLocString("FIGHT_REASON"))
                    end,

                }
                :OnWin()
                    :Fn(function() 
                        
                        if cxt.quest.param.guard_blocking then
                            cxt:Dialog("DIALOG_FAILURE_GOT_AWAY")
                            cxt.quest:Fail()
                        else
                            cxt:Dialog("DIALOG_WON_FIGHT")
                            if cxt:GetAgent():IsDead() then
                                ConvoUtil.GiveQuestRewards(cxt)
                                cxt.quest:Complete()
                            else
                                cxt.quest:Fail()
                            end
                        end
                    end)
        end)