local fun = require "util/fun"


local QDEF = QuestDef.Define
{
    title = "Win The Crowd",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    desc = [[Convince the weakest member of the crowd.]],
    act_filter = "SAL_BRAWL",
    cooldown = EVENT_COOLDOWN.NONE, 
    focus = QUEST_FOCUS.NEGOTIATION,
    qtype = QTYPE.SIDE,
}

:AddObjective{
    id = "start",
    title = "Convince one of the crowd!",
}

:AddOpinionEvents{

    won =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Convinced their crowd",
    },
    
    lost =  
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Angered their crowd",
    },
}


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !right
                    !angry
                * A motley crew accosts you. If you convince the right person, you can turn the tide!
                player:
                    !left

            ]],

            OPT_CONVINCE = "Convince {1}!",
            DIALOG_ATTACK = [[
                agent:
                    !right
                player:
                    <i>You</> know what I mean, right?

            ]],

            DIALOG_SUCCESS = [[
                agent:
                    !happy
                    We should all listen to this grifter!
                    !exit
                * You're making friends!
            ]],

            DIALOG_FAILURE = [[
                agent:
                    !threaten
                    Nobody listen to this idiot!
                    !exit
                * The crowd grumbles and disperses.
            ]],

        }
        :Fn(function(cxt) 
            
            cxt.encounter:DoLocationTransition(TheGame:GetGameState():GetMainQuest():DefFn("GetRandomLocation"))

        
            local crowd_size = math.min(2+cxt.quest:GetRank(), 4)
            local agents = {}
            local picked = {}
            for k = 1, crowd_size do

                local def_id = fun(Content.GetAllCharacterDefs())
                                :filter(function(v) return not v.is_template and v.renown and v.species == nil and not v.boss and not v.unique and math.abs(v.renown - cxt.quest:GetRank()) <= 2 and not picked[v.id] and QuestUtil.FilterPromoted(v)  end)
                                :keys()
                                :randomPick()
                if def_id then
                    picked[def_id] = true
                    local agent = TheGame:GetGameState():AddSkinnedAgent( def_id )
                    table.insert(agents, agent)
                end
            end
            cxt:TalkTo(agents[1])

            cxt:Dialog("DIALOG_INTRO")

            for k, agent in ipairs(agents) do
                
                local hinders = {}
                for k,v in ipairs( agents) do
                    if v ~= agent then
                        table.insert(hinders, v)
                    end
                end

                cxt:Opt("OPT_CONVINCE", agent)
                    :Fn(function() cxt:TalkTo(agent) end)
                    :Dialog("DIALOG_ATTACK")
                    :Negotiation{
                        target_agent = agent,
                        hinders = hinders,
                    }
                        :OnSuccess()
                            :Dialog("DIALOG_SUCCESS")
                            :Fn(function() 
                                for k,v in pairs(agents) do
                                    v:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("won"))
                                end

                                ConvoUtil.GiveQuestRewards(cxt)
                                cxt.quest:Complete()
                            end)
                        :OnFailure()
                            :Dialog("DIALOG_FAILURE")
                            :Fn(function() 
                                for k,v in pairs(agents) do
                                    v:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("lost"))
                                end

                                cxt.quest:Fail()
                            end)
            
            end

        end)