
local QDEF = QuestDef.Define{
    title = "All Play And No Work",
    qtype = QTYPE.CONTRACT,
    faction = "SPARK_BARONS",
    rank = {1, 3},
    desc = [[{target} has been shirking {target.hisher} duties. {giver} wants you to beat {target.himher} up to set an example for the others.]],
    icon = engine.asset.Texture("icons/quests/baron_all_play_and_no_work.tex"),
    focus = QUEST_FOCUS.COMBAT,
    reward_size = QREWARDSIZE.SMALLER,
    avoids_fn = function(quest, agent, location)
        
        if agent == quest:GetCastMember("target") and agent:GetBrain():GetWorkplace() == location then
            return true
        end
    end,

    on_start = function(quest)
        quest:Activate("confront_target")
    end,

    avoids = 
    {
        {"giver", "target"},
    },
}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddCast{
    cast_id = "target",
    condition = function(agent, quest) 
        return agent:GetContentID() == "HEAVY_LABORER"
    end,
}
:AddCastFallback
{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "HEAVY_LABORER" ) )
    end,
}

:AddObjective{
    id = "confront_target",
    title = [[Confront {target}]],
    desc = "Confront {target} and beat them up.",
    mark = { "target" },
    negotiation_targets = {"target"},
    combat_targets = {"target"},

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("target") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,

    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end,
    
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[Go tell {giver} that {target} has been dealt with.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}


QDEF:AddConvo("confront_target", "target")
    :Loc{
        OPT_CONFRONT = "Confront {agent} about shirking",
        DIALOG_CONFRONT = [[
            player:
                I hear that you've been shirking your responsibilities, laborer. 
            agent:
                !dubious 
                $neutralDubious
                You a bounty hunter or something? I ain't no truant. I still show up to every shift.
                Besides, if the Barons don't like the work I put in, they can just fire me.
                !angry_shrug
                $angryDefault
                That's the power <i>they</i> got, innit? I gotta serve my contract Hesh and high water, but they can do what they want.
        ]],
        OPT_BEAT_UP_TARGET = "Beat up {agent}",
        DIALOG_BEAT_UP_TARGET = [[
            player:
                !fight
                There is <i>one</i> way out, of course. 
        ]],
        DIALOG_FOUGHT_ALIVE = [[
            agent:
                !injured
                $scaredPowerless
                S-stop! The Barons have made their point. 
                I'll work myself to the grave, if it means putting it off a few years. 
            player:
                !happy
                Excellent. I, too, pride myself on a job well done.
                I'm glad we could come to an agreement. 
        ]],
        DIALOG_FOUGHT_DEAD = [[
            * {agent} is dead. {agent.HeShe} certainly can't work harder now.
        ]],

        OPT_THREATEN = "Let {agent} off with a warning",
        DIALOG_THREATEN = [[
            player:
                !throatcut
                I'm supposed to give you a motivational beating. 
                If you act sufficiently scared, we can skip to just improving your work output, yes?
        ]],
        DIALOG_THREATEN_SUCCESS = [[
            player:
                The fact of the matter is, the Barons have been doing this for a long time.
                They'd rather break you than terminate your contract. 
                !crossed
                $neutralDirect
                Does that sound like an agreeable arrangement to you?
            agent:
                !scared
                $scaredDefault
                N-no. They'd really do that?
            player:
                !agree
                $neutralResigned
                I'm afraid so.
            agent:
                ...
                $neutralResigned
                Okay. I get the picture. I'll work harder from now on, I promise. 
            player:
                !point
                See to it that you do. 
        ]],
        DIALOG_THREATEN_FAIL = [[
            player:
                Besides, don't you want to do work you're proud of?
            agent:
                !angry
                $angryDefault
                Hah! So the Spark Barons can grind me under their heel and pretend I ain't got rights?
                No thanks. Go tell them to keep their goons to themselves. 
                I'm doing my part. If they don't like it, they should cut me loose. 
        ]],
        DIALOG_BACK = [[
            player:
                Hold that thought...
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_CONFRONT")
            :LoopingFn(function(cxt) 
                    if cxt:FirstLoop() then
                        cxt:Dialog("DIALOG_CONFRONT")
                    end
                    
                    cxt:Opt("OPT_THREATEN")
                        :Dialog("DIALOG_THREATEN")
                        :Negotiation{
                            flags = NEGOTIATION_FLAGS.INTIMIDATION,
                            
                            on_success = function(cxt) 
                                cxt.quest.param.did_threaten = true
                                cxt:Dialog("DIALOG_THREATEN_SUCCESS")
                                cxt.quest:Complete("confront_target")
                                StateGraphUtil.AddEndOption(cxt)
                            end,
                            on_fail = function(cxt) 
                                cxt:Dialog("DIALOG_THREATEN_FAIL")
                            end,
                        }
                    cxt:Opt("OPT_BEAT_UP_TARGET")
                        :Dialog("DIALOG_BEAT_UP_TARGET")
                        :Battle{
                            oppo_limit = 2,
                            on_win = function(cxt) 
                                cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_FOUGHT_DEAD" or "DIALOG_FOUGHT_ALIVE")
                                cxt.quest:Complete("confront_target")
                                StateGraphUtil.AddEndOption(cxt)
                            end,
                        }
                    
                    StateGraphUtil.AddBackButton(cxt)
                        :Dialog("DIALOG_BACK")
            end)

    end)



QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_KILLED = [[
            player:
                I confronted {target} but I'm afraid they fought back more passionately than either of us anticipated.
                $neutralResigned
                {target.HeShe}'s dead.
            agent:
                !dubious
                I see. Well, so be it. Some people <i>will</i> fight to the death for what they believe. 
                !crossed
                $neutralDirect
                But {player}? Never imply that your failings are also my own. 
                I didn't botch the job. You did. Remember that. 
        ]],
        DIALOG_THREATENED = [[
            player:
                !interest
                I spoke to the target and—
            agent:
                !angry_shrug
                $angryDefault
                Spoke to...? You were supposed to <i>punish</i>, not coddle. 
            player:
                Come, {agent}, you know how I work. If it gets the results you desire, I'd prefer if you didn't micromanage.
            agent:
                !palm
                $neutralResigned
                The problem here is a miscommunication.
                I wanted {target} terrified. Next time, remember the objective. 
        ]],
        DIALOG_BEAT = [[
            player:
                {target} has been dealt with.
            agent:
                You didn't hurt {target.himher} too badly, I hope?
                !happy
                $happyJoke
                {target.HeShe} still needs to get back to work in the morning.
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Fn(function() 
                    if cxt.quest:GetCastMember("target"):IsDead() then
                        cxt.quest.param.poor_performance = true
                        cxt:Dialog("DIALOG_KILLED")
                    elseif cxt.quest.param.did_threaten then
                        cxt.quest.param.poor_performance = true
                        cxt:Dialog("DIALOG_THREATENED")
                    else
                        cxt:Dialog("DIALOG_BEAT")
                    end

                end)
            :CompleteQuest()
    end)    



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !overthere
                $angryReluctant
                My people tell me that {target}, a laborer, has been shirking responsibility. 
                {target.HeShe}'s been given demerits, but it's clear {target.heshe}'s angling for us to cancel {target.hisher} contract.
                !eureka
                $angryDefault
                I've seen this <i>voluntary termination</i> technique before. It can't be permitted.
            player:
                $neutralDubious
                Seems the simplest solution is to just give {target} what {target.heshe} wants, no?
            agent:
                !dubious 
                $neutralDubious
                That's rather naive of you, {player}. This is clearly a ploy by the Rise, testing our authority. 
                So what I'd like you to do is remind {target} of {target.hisher} place. Contracts are meant to be kept. 
                !cruel 
                $angryCruel
                See to it that {target.heshe} gets a punishment {target.heshe} can't ignore.
            
            *** {giver} wants you to punish {target} for neglecting {target.hisher} duties.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            player:
                !interest
                You know, this {target} might surprise you with {target.hisher} resilience.
            agent:
                Well, certainly. If I thought that this would put {target} out of commission, then I'd simply ask you to kill {target.himher}.
                !placate
                $neutralResigned
                Your hand was always well-weighted, though. I think you'll see it through. 
                My sources tell me you can expect to find {target} drinking. Typical.
                !salute
                $neutralDirect
                Well, no more futzing about—have at it, Captain.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)




QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.DECLINED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                $miscPersuasive
                This may seem unsavory, but when employees bend the rules, they make life more unpleasant for everyone.
                !shrug
                Eliminating {target} is good for the laborers, truly. 
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)

