local fun = require "util/fun"

local function SetDebugRel(quest, cast_id, rel_param)
    if quest.param[rel_param] == "FRIEND" then
        quest:GetCastMember(cast_id).social_connections:SetRelationship(RELATIONSHIP.LIKED, TheGame:GetGameState():GetPlayerAgent())
    elseif quest.param[rel_param] == "ENEMY" then
        quest:GetCastMember(cast_id).social_connections:SetRelationship(RELATIONSHIP.DISLIKED, TheGame:GetGameState():GetPlayerAgent())
    elseif quest.param[rel_param] == "NEUTRAL" then
        quest:GetCastMember(cast_id).social_connections:SetRelationship(RELATIONSHIP.NEUTRAL, TheGame:GetGameState():GetPlayerAgent())
    end
end


local QDEF = QuestDef.Define{
    title = "Crime Scene",
    qtype = QTYPE.CONTRACT,
    faction = "SPARK_BARONS",
    tags = {"late_game"},
    rank = {1, 5},
    desc = [[Investigate a crime scene for the Barons.]],
    
    icon = engine.asset.Texture("icons/quests/baron_crime_scene.tex"),
    
    focus = QUEST_FOCUS.NEGOTIATION,

    on_init = function(quest)
    end,

    on_start = function(quest)
        quest.param.suspect_1_guilty = quest:GetCastMember("guilty") == quest:GetCastMember("suspect_1")
        quest.param.suspect_2_guilty = quest:GetCastMember("guilty") == quest:GetCastMember("suspect_2")
        quest.param.suspect_3_guilty = quest:GetCastMember("guilty") == quest:GetCastMember("suspect_3")

        if quest.param.treasure_exists == nil then
            quest.param.treasure_exists = math.random() < .5
        end


        quest.param.suspect_1_knows = not quest.param.suspect_1_guilty and ( quest.param.suspect_1_knows == nil and (math.random() < .5))
        quest.param.suspect_2_knows = not quest.param.suspect_2_guilty and ( quest.param.suspect_2_knows == nil and (math.random() < .5))
        quest.param.suspect_3_knows = not quest.param.suspect_3_guilty and ( quest.param.suspect_3_knows == nil and (math.random() < .5))

        SetDebugRel(quest, "suspect_1", "suspect_1_relationship")
        SetDebugRel(quest, "suspect_2", "suspect_2_relationship")
        SetDebugRel(quest, "suspect_3", "suspect_3_relationship")

        quest:Activate("go_to_crime_scene")
    end,

    on_destroy = function(quest)
        if quest:GetCastMember("the_accused") and quest:GetCastMember("the_accused"):IsInPlayerParty() then
            quest:GetCastMember("the_accused"):Dismiss()
        end
    end,

    collect_agent_locations = function(quest, t)
        if quest:IsActive("go_to_crime_scene") or quest:IsActive("interrogate_suspects") then
            table.insert (t, { agent = quest:GetCastMember("suspect_1"), location = quest:GetCastMember("crime_scene")} )
            table.insert (t, { agent = quest:GetCastMember("suspect_2"), location = quest:GetCastMember("crime_scene")} )
            table.insert (t, { agent = quest:GetCastMember("suspect_3"), location = quest:GetCastMember("crime_scene")} )
            table.insert (t, { agent = quest:GetCastMember("investigator"), location = quest:GetCastMember("crime_scene")} )
        end
    end,

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("the_accused") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,
    

}

QDEF:AddLocationDefs{
    CRIME_SCENE = {
        name = "The Crime Scene",
        plax = "EXT_CRIMESCENE",
        map_tags = {"slum"},
        indoors = false,
        show_agents = true,
    },
    
    TREASURE_LOC = {
        name = "Buried Treasure",
        plax = "EXT_Bog_Outcrop_01",
        map_tags = {"bog"},
        indoors = false,
    },
}

QDEF:AddLocationCast{
    cast_id = "crime_scene",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("CRIME_SCENE"))
    end,
}

QDEF:AddLocationCast{
    cast_id = "treasure_loc",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("TREASURE_LOC"))
    end,
}

:AddLocationCast{
    cast_id = "gate",
    cast_fn = function(quest, t)
        table.insert( t, TheGame:GetGameState():GetLocation("GB_GATE"))
    end,
}


QDEF:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

QDEF:TryAddKnownCast("investigator", "SPARK_BARON_TASKMASTER")
QDEF:TryAddKnownCast("suspect_1", "LABORER")
QDEF:TryAddKnownCast("suspect_2", "HEAVY_LABORER")
QDEF:TryAddKnownCast("suspect_3", "FOREMAN")


QDEF:AddCast{
    cast_id = "guilty",
    cast_fn = function(quest, t) 
        table.insert(t, table.arraypick{quest:GetCastMember("suspect_1"), quest:GetCastMember("suspect_2"), quest:GetCastMember("suspect_3")})
    end,
    no_validation = true
}

QDEF:AddCast{
    cast_id = "the_accused",
    when = QWHEN.MANUAL,
    events = {
        agent_retired = function(quest, agent) 

            if quest:IsActive("escort_suspect_to_barons") then
                quest:Cancel("escort_suspect_to_barons")
            end
            
            if quest:IsActive("find_treasure") then 
                quest:Cancel("find_treasure")
            end
            
            if not quest:IsActive("convince_giver_accused_was_killed") and not quest.param.delivered then
                quest:Activate("tell_about_death")
            end
        end,
    }

}



QDEF:AddObjective{
    id = "go_to_crime_scene",
    title = [[Go to the scene of the crime]],
    desc = "{giver} wants you to help {investigator} get the bottom of some suspected Rise activity in the workers' living quarters.",
    mark = {"crime_scene"},
    
    on_complete = function(quest) 
        quest:Activate("interrogate_suspect_1")
        quest:Activate("interrogate_suspect_2")
        quest:Activate("interrogate_suspect_3")
        quest:Activate("tell_investigator")
    end
}

QDEF:AddObjective{
    id = "interrogate_suspect_1",
    title = "Interrogate {suspect_1}",
    desc = "See what {suspect_1} knows",
    mark = {"suspect_1"},

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("suspect_1") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,

}

QDEF:AddObjective{
    id = "interrogate_suspect_2",
    title = "Interrogate {suspect_2}",
    desc = "See what {suspect_2} knows",
    mark = {"suspect_2"},
    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("suspect_2") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,
}

QDEF:AddObjective{
    id = "interrogate_suspect_3",
    title = "Interrogate {suspect_3}",
    desc = "See what {suspect_3} knows",
    mark = {"suspect_3"},
    
    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("suspect_3") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,

}

QDEF:AddObjective{
    id = "tell_investigator",
    title = "Tell {investigator} your findings",
    desc = "Once you have decided who to accuse, you should tell {investigator}",
    on_complete = function(quest) 
        quest:Cancel("interrogate_suspect_1")
        quest:Cancel("interrogate_suspect_2")
        quest:Cancel("interrogate_suspect_3")
        quest:Activate("escort_suspect_to_barons")
        quest:GetCastMember("the_accused"):Recruit(PARTY_MEMBER_TYPE.CAPTIVE)

    end,
    mark = {"investigator"},
}


QDEF:AddObjective{
    id = "escort_suspect_to_barons",
    title = "Bring {the_accused} to the gate",
    desc = [[{the_accused} is being sent out of the Bog for further interrogation. ]],
    mark = { "gate" },
}

QDEF:AddObjective{
    id = "find_treasure",
    title = "Follow {the_accused} to the loot",
    desc = [[{the_accused} will show you where their life savings are stashed away if you let them go.]],
    mark = { "treasure_loc" },
}

QDEF:AddObjective{
    id = "convince_giver_accused_was_killed",
    title = "Convince {giver} that {the_accused} was killed",
    desc = [[You let {the_accused} go. Convince {giver} that {the_accused.heshe} died in a firefight.]],
    mark = { "giver" },
    terminal = true,
}

QDEF:AddObjective{
    id = "tell_about_death",
    title = "Tell {giver} that {the_accused} was killed",
    desc = [[{the_accused} is dead. Tell {giver} what happened.]],
    mark = { "giver" },
    terminal = true,
}


QDEF:AddOpinionEvents{
    beat_them_up_for_info = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Beat them up to interrogate them",
    },
    opinion_let_go_for_pay = {
        delta = OPINION_DELTAS.NEUTRALIZING,
        txt = "Let them go (for money)",
    },
    opinion_let_go_for_free = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Let them go (for free)",
    },
    stole_loot = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Stole their life savings",
    },
}

----------QUEST INTRODUCTION

QDEF:AddIntro(
    --attract spiel
    [[
        agent:
            Ah, you should find this one interesting--
            Should remind you of the frontlines, I imagine. Seems the Rise have gotten ahold of some explosives.
        player:
            Do you know where they plan to use them?
        agent:
            Planned. They've already been used. 
            The ingrates blew up one of the worker's dormitories.
        player:
            They blew up their own housing? Why?
        agent:
            Perhaps to make a point. Spark Barons sweep the bunks during shifts to ensure no one is smuggling out any relics.
            Fortunately, the bombs went off too early--all they managed to kill was a couple of their own stragglers.
            Good luck aside, we need to seize this opportunity to vilify the Rise. 
    ]],
    
    --on accept
    [[
        agent:
            There's an investigator looking into it, but you've got more experience with this kind of sabotage.
            Find out who's responsible--or at least who could believably be held responsible.
            We'll find a suitable use for their, ah, disciplinary proceedings.
    ]])

---------------------------------------------------------------------------------------------------


---------------------------------------------------------------------------------------------------
-- Introduction to the crime scene. You meet the investigator, and they tell you about the suspects.

---------------------------------------------------------------------------------------------------

QDEF:AddConvo("go_to_crime_scene")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("crime_scene") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find {investigator} flanked by security and berating a trio of unfortunate civilians.
                investigator:   
                    !right
                    !angry_accuse
                    $angryCruel
                    You lot are hiding something, and if I have to whip it out of you--
                player:
                    !left
                    Excuse me.
                investigator:
                { liked?
                    !neutral
                    {player}! Help me fish the truth out from these pond scum.
                    You're not here to meddle, are you?
                }
                { disliked?
                    !crossed
                    Oh. {giver}'s pet grifter is here.
                    Just what I need, more meddling.
                }
                { not disliked and not liked ?
                    Who the Hesh are you?
                    Oh. You're {giver}'s pet grifter, is that right?
                    !sigh
                    I was told I should expect your meddling. 
                }
                player:
                    I assure you I'll keep my meddling to a minimum. 
            ]],

            OPT_ASK_WHAT_HAPPENED = "Ask {investigator} about the crime",
            DIALOG_ASK_WHAT_HAPPENED = [[
                player:
                    What exactly happened?
                investigator:
                    The Rise blew up the dorm here. 
                player:
                    I'm aware of that much, thank you. I was more interested in the greater context.
                investigator:
                    Greater context? The Rise are anarchic miscreants.
                    They stole a bomb, they used it. Even were dumb enough to blow up some of their own. 
                player:
                    Dumb enough... or clever enough?
                investigator:
                    !angry_shrug
                    Huh?
            ]],

            OPT_ASK_ABOUT_SUSPECTS = "Ask about the suspects",
            DIALOG_ASK_ABOUT_SUSPECTS = [[
                player:
                    Who are the suspects?
                investigator:
                    {suspect_1} is a worker who lives in the pod. {suspect_1.HeShe} was late for {suspect_1.hisher} shift the day before the explosion. 
                    Probably spent the time getting the explosives.
                    {suspect_2} works in the deeper bog, and has been seen loitering in the area. 
                    No doubt scouting out opportunities for vandalism!
                    Finally, {suspect_3} is the foreman of the local dig pit. You'd think that {suspect_3.heshe}'d know who butters {suspect_3.hisher} bread, but {suspect_3.heshe} hasn't met quota all quarter. 
                    I'll bet real shills that {suspect_3.heshe}'s letting {suspect_3.hisher} Rise buddies shirk and organize.
                player:
                    You seem pretty convinced that they're all guilty.
                investigator:
                    !shrug
                    Everybody's guilty of something.
            ]],

            OPT_START_INVESTIGATION = "Start your investigation",
            DIALOG_START_INVESTIGATION = [[
                player:
                    I'm going to talk to the suspects.
                investigator:
                    Figure out which one did it, and then we can talk about the next step.
            ]]

        }
        :Fn(function(cxt) 
            cxt:TalkTo("investigator")
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Complete("go_to_crime_scene")

            cxt:RunLoop(function()
                cxt:Question("OPT_ASK_WHAT_HAPPENED", "DIALOG_ASK_WHAT_HAPPENED")
                cxt:Question("OPT_ASK_ABOUT_SUSPECTS", "DIALOG_ASK_ABOUT_SUSPECTS")
                
                cxt:Opt("OPT_START_INVESTIGATION")
                    :Dialog("DIALOG_START_INVESTIGATION")
                    :End()
            end)
        end)






local interrogation_fn = function(cxt) 
    local suspect_number 

    if cxt:GetAgent() == cxt.quest:GetCastMember("suspect_1") then
        suspect_number = 1
    elseif cxt:GetAgent() == cxt.quest:GetCastMember("suspect_2") then
        suspect_number = 2 
    elseif cxt:GetAgent() == cxt.quest:GetCastMember("suspect_3") then
        suspect_number = 3
    end

    cxt.enc.scratch.suspect_guilty = cxt.quest:GetCastMember("guilty") == cxt:GetAgent()
    cxt.enc.scratch.suspect_knows = cxt.quest.param["suspect_".. suspect_number .."_knows"]
    cxt.enc.scratch.suspect_clammed = cxt.quest.param["suspect_".. suspect_number .."_clammed"]
    cxt.enc.scratch.suspect_talked = cxt.quest.param["suspect_".. suspect_number .."_talked"]

    if cxt.enc.scratch.suspect_talked then
        cxt:Dialog("DIALOG_ATTRACT_TALKED")
        StateGraphUtil.AddEndOption(cxt)                
    else
        if cxt.enc.scratch.suspect_clammed then
            cxt:Dialog("DIALOG_ATTRACT_CLAMMED")
        else
            cxt:Dialog("DIALOG_ATTRACT_INTRO")
        end

        if not cxt.quest:IsActive("tell_investigator") then
            cxt.quest:Activate("tell_investigator")
        end

        cxt:RunLoop(function() 
            
            if not cxt.enc.scratch.suspect_clammed then
                cxt:Question("OPT_ASK_QUESTION_1", "DIALOG_ASK_QUESTION_1")
                cxt:Question("OPT_ASK_QUESTION_2", "DIALOG_ASK_QUESTION_2")
            end

            cxt:Opt("OPT_INTERROGATE")
                :Dialog("DIALOG_INTERROGATE")
                :Negotiation()
                    :OnSuccess()
                        :Fn(function() 
                            cxt.quest:Complete("interrogate_suspect_"..suspect_number)
                            if cxt.quest:GetCastMember("guilty") == cxt:GetAgent() then
                                cxt:Dialog("DIALOG_INTERROGATE_SUCCESS_GUILTY")
                                cxt.quest.param.has_confession = true
                            else
                                cxt:Dialog("DIALOG_INTERROGATE_SUCCESS_INNOCENT")    
                            end
                            StateGraphUtil.AddEndOption(cxt)
                            cxt.quest.param["suspect_"..suspect_number.."_talked"] = true
                        end)
                        
                    :OnFailure()
                        :Dialog("DIALOG_INTERROGATE_FAILURE")
                        :Fn(function()
                            cxt.quest.param["suspect_"..suspect_number.."_clammed"] = true
                            StateGraphUtil.AddEndOption(cxt) 
                        end)
            cxt:Opt("OPT_BEAT")
                :Dialog("DIALOG_BEAT")
                :ReceiveOpinion("beat_them_up_for_info")
                :Battle{
                    flags = BATTLE_FLAGS.NO_BYSTANDERS
                    }
                    :OnWin()
                        :Fn(function() 
                            if cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_BEAT_SUSPECT_DEAD")
                                cxt.quest:Fail()
                                StateGraphUtil.AddLeaveLocation(cxt)
                            else
                                cxt.quest:Complete("interrogate_suspect_"..suspect_number)
                                if cxt.quest:GetCastMember("guilty") == cxt:GetAgent() then
                                    cxt:Dialog("DIALOG_BEAT_SUCCESS_GUILTY")
                                    cxt.quest.param.has_confession = true
                                else
                                    cxt:Dialog("DIALOG_BEAT_SUCCESS_INNOCENT")    
                                end
                                cxt.quest.param["suspect_"..suspect_number.."_talked"] = true
                                StateGraphUtil.AddEndOption(cxt)
                            end
                            
                        end)

            StateGraphUtil.AddEndOption(cxt)
        end)
    end
end

---------------------------------------------------------------------------------------------------
-- For the interrogation you talk to the suspects, and negotiate. If you win, they tell you if they or guilty. If they aren't there's a 50% chance they tell you who is.
-- Once you've talked to at least one of these people, you can go back to the investigator and blame one of them.
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- Suspect 1:
--  A laborer who lives here and has been late a lot lately.
---------------------------------------------------------------------------------------------------

QDEF:AddConvo(nil, "suspect_1")
    :Loc{
        DIALOG_ATTRACT_INTRO = [[
            {liked ?
                agent:
                    {player}, I'm so glad you're here! You have to save me!
            }

            {disliked ?
                agent:
                    Oh, it's you. Big surprise you're doing the Barons' dirty work.
            }

            {not disliked and not liked ?
                agent:
                    I keep tellin' you people, I don't know anything!
            }
        ]],
        DIALOG_ATTRACT_CLAMMED = [[
            agent:
                !crossed
                I'm <i>done</i> talking.
        ]],
        DIALOG_ATTRACT_TALKED = [[
            {suspect_guilty?
                agent: 
                    What are you going to do to me?
            }
            
            {suspect_knows?
                agent: 
                    Go on, I told you who did it!
            }
            
            {not suspect_guilty and not suspect_knows?
                agent: 
                    I already told you I don't know anything!
            }
        ]],
        OPT_ASK_QUESTION_1 = "Ask {agent} what happened here",
        DIALOG_ASK_QUESTION_1 = [[
            player:
                What do you know about the bombing?
            agent:
                {liked?
                    You know me, {player}. I didn't do anything!
                }
                {disliked?
                    What does it matter what I say to you? You're not going to believe me anyway.
                }
                
                {not disliked and not liked ?
                    How should I know? I just work here.
                }
        ]],

        OPT_ASK_QUESTION_2 = "Ask why {agent} was late for shift",
        DIALOG_ASK_QUESTION_2 = [[
            player:
                Why are you always late for your shifts?
            agent:
                Why is anyone ever late for shift?
                I work at the bottom of a stinkin' hole, and I barely make more than I spend on food.
        ]],

        OPT_INTERROGATE = "Interrogate {agent}",
        DIALOG_INTERROGATE = [[
            player:
                Perhaps by our powers combined, we can solve this, hmm?
        ]],
        
        DIALOG_INTERROGATE_SUCCESS_INNOCENT = [[
            { suspect_knows ?
                agent:
                    It wasn't me! But...
                    It was {guilty}! I saw {guilty.himher} do it!
            }
            { not suspect_knows ?
                agent:
                    I swear, I don't know anything about what happened!
                    Please, you have to believe me!
                * You're pretty sure {agent} is telling the truth.
            }

        ]],

        DIALOG_INTERROGATE_SUCCESS_GUILTY = [[
            agent:
                {liked ?
                    Alright, I admit it. I have been helping the Rise.
                    Please, you have to get me out of this. Can't you pin it on someone else?
                }

                {not liked ?
                    Yeah, I did it. And I'd do it again!
                }

        ]],
        DIALOG_INTERROGATE_FAILURE = [[
            agent:
                I'm done talking to you, grifter.
        ]],

        OPT_BEAT = "Beat up {agent}",
        DIALOG_BEAT = [[
            player:
                Let's see if we can put you into a more talkative mood.
            investigator:
                !right
                Oh, you're going to do that, eh?
                Just make sure you don't kill {agent.himher}. The Baron brass wants the guilty party alive.
            agent:
                !right
        ]],
        DIALOG_BEAT_SUSPECT_DEAD = [[
            * {agent} crumples like a sack of oshnu dung.
            investigator:
                !right
                What the hell! You weren't supposed to kill the suspects!
                Get out of here, and let me do my job!

        ]],
        DIALOG_BEAT_SUCCESS_GUILTY = [[
            agent:
                Stop! Stop! I confess!
                I did it! Long live the Rise!
        ]],
        DIALOG_BEAT_SUCCESS_INNOCENT = [[
            agent:
                Please! Stop!
                { suspect_knows ?
                    It was {guilty}! I saw {guilty.himher} do it!
                }
                { not suspect_knows ?
                    I would tell you if I knew! But I have no idea!
                }
        ]]


    }
    :AttractState("STATE_ATTRACT", function(cxt) return cxt.quest:IsActive("interrogate_suspect_1") or cxt.quest:IsActive("tell_investigator") end)
        :Fn(interrogation_fn) 


---------------------------------------------------------------------------------------------------
-- Suspect 2:
--  A heavy laborer who doesn't live here but has been seen loitering around
---------------------------------------------------------------------------------------------------

QDEF:AddConvo(nil, "suspect_2")
    :Loc{
        DIALOG_ATTRACT_INTRO = [[
            {liked ?
                agent:
                    {player}, what are you doin' here? You're not caught up in this mess too, are you?
            }

            {disliked ?
                agent:
                    Today just couldn't get any worse, could it?
            }

            {not disliked and not liked ?
                agent:
                    If you're gonna question me, I'll just tell you the same thing I told the other guy.
            }
        ]],
        DIALOG_ATTRACT_CLAMMED = [[
            agent:
                I'm done talking to you!
        ]],
        DIALOG_ATTRACT_TALKED = [[
            {suspect_guilty?
                agent: 
                    What are you going to do to me?
            }
            
            {suspect_knows?
                agent: 
                    Go on, I told you who did it!
            }
            
            {not suspect_guilty and not suspect_knows?
                agent: 
                    I already told you I don't know anything!
            }
        ]],
        OPT_ASK_QUESTION_1 = "Ask {agent} what happened here",
        DIALOG_ASK_QUESTION_1 = [[
            player:
                What do you know about the bombing?
            agent:
                {liked?
                    You know me, {player}. I didn't do anything!
                }
                {disliked?
                    What does it matter what I say to you? You're not going to believe me anyway.
                }
                
                {not disliked and not liked ?
                    How should I know? I just work here.
                }
        ]],

        OPT_ASK_QUESTION_2 = "Ask why {agent} has been loitering",
        DIALOG_ASK_QUESTION_2 = [[
            player:
                You don't live here, why have you been loitering?
            agent:
                I have friends in this pod. We're allowed to have friends, right?
        ]],

        OPT_INTERROGATE = "Interrogate {agent}",
        DIALOG_INTERROGATE = [[
            player:
                Alright, let's get to the bottom of this...
        ]],
        
        DIALOG_INTERROGATE_SUCCESS_INNOCENT = [[
            { suspect_knows ?
                agent:
                    It wasn't me! But...
                    It was {guilty}! I saw {guilty.himher} do it!
            }
            { not suspect_knows ?
                agent:
                    I swear, I don't know anything about what happened!
                    Please, you have to believe me!
                * You're pretty sure {agent} is telling the truth.
            }

        ]],

        DIALOG_INTERROGATE_SUCCESS_GUILTY = [[
            agent:
                {liked ?
                    Alright, I admit it. I have been helping the Rise.
                    Please, you have to get me out of this. Can't you pin it on someone else?
                }

                {not liked ?
                    Yeah, I did it. And I'd do it again!
                }

        ]],
        DIALOG_INTERROGATE_FAILURE = [[
            agent:
                I'm done talking to you, grifter.
        ]],
        OPT_BEAT = "Beat up {agent}",
        DIALOG_BEAT = [[
            player:
                Let's see if we can put you into a more talkative mood.
            investigator:
                !right
                Oh, you're going to do that, eh?
                Just make sure you don't kill {agent.himher}. The Baron brass wants the guilty party alive.
            agent:
                !right
        ]],
        DIALOG_BEAT_SUSPECT_DEAD = [[
            * {agent} crumples like a sack of oshnu dung.
            investigator:
                !right
                What the hell! You weren't supposed to kill the suspects!
                Get out of here, and let me do my job!

        ]],
        DIALOG_BEAT_SUCCESS_GUILTY = [[
            agent:
                Stop! Stop! I confess!
                I did it! Long live the Rise!
        ]],
        DIALOG_BEAT_SUCCESS_INNOCENT = [[
            agent:
                Please! Stop!
                { suspect_knows ?
                    It was {guilty}! I saw {guilty.himher} do it!
                }
                { not suspect_knows ?
                    I would tell you if I knew! But I have no idea!
                }
        ]]


    }
    :AttractState("STATE_ATTRACT", function(cxt) return cxt.quest:IsActive("interrogate_suspect_2") or cxt.quest:IsActive("tell_investigator") end)
        :Fn(interrogation_fn) 


---------------------------------------------------------------------------------------------------
-- Suspect 3:
--  A foreman who works nearby and has been going easy on their laborers.
---------------------------------------------------------------------------------------------------

QDEF:AddConvo(nil, "suspect_3")
    :Loc{
        DIALOG_ATTRACT_INTRO = [[
            {liked ?
                agent:
                    You investigating this mess, {player}? Am I glad to see you.
            }

            {disliked ?
                agent:
                    Well. If it smells like a flatworm and looks like a flatworm...
            }

            {not disliked and not liked ?
                agent:
                    What, you got more questions for me?
            }
        ]],
        DIALOG_ATTRACT_CLAMMED = [[
            agent:
                I'm done talking to you!
        ]],
        DIALOG_ATTRACT_TALKED = [[
            {suspect_guilty?
                agent: 
                    What are you going to do to me?
            }
            
            {suspect_knows?
                agent: 
                    Go on, I told you who did it!
            }
            
            {not suspect_guilty and not suspect_knows?
                agent: 
                    I already told you I don't know anything!
            }
        ]],
        OPT_ASK_QUESTION_1 = "Ask {agent} what happened here",
        DIALOG_ASK_QUESTION_1 = [[
            player:
                What do you know about the bombing?
            agent:
                {liked?
                    You know me, {player}. I didn't do anything!
                }
                {disliked?
                    What does it matter what I say to you? You're not going to believe me anyway.
                }
                
                {not disliked and not liked ?
                    How should I know? I just work here.
                }
        ]],

        OPT_ASK_QUESTION_2 = "Ask {agent} why they haven't met quota lately",
        DIALOG_ASK_QUESTION_2 = [[
            player:
                You haven't met your quota lately. That sounds like something the Rise would like.
            agent:
                I... I just want people to like me! Is that a crime?
            player:
                Depends on the people, I guess.
        ]],

        OPT_INTERROGATE = "Interrogate {agent}",
        DIALOG_INTERROGATE = [[
            player:
                Alright, let's get to the bottom of this...
        ]],
        
        DIALOG_INTERROGATE_SUCCESS_INNOCENT = [[
            { suspect_knows ?
                agent:
                    It wasn't me! But...
                    It was {guilty}! I saw {guilty.himher} do it!
            }
            { not suspect_knows ?
                agent:
                    I swear, I don't know anything about what happened!
                    Please, you have to believe me!
                * You're pretty sure {agent} is telling the truth.
            }

        ]],

        DIALOG_INTERROGATE_SUCCESS_GUILTY = [[
            agent:
                {liked ?
                    Alright, I admit it. I have been helping the Rise.
                    Please, you have to get me out of this. Can't you pin it on someone else?
                }

                {not liked ?
                    Yeah, I did it. And I'd do it again!
                }

        ]],
        DIALOG_INTERROGATE_FAILURE = [[
            agent:
               I'm done talking to you, grifter.
        ]],
        OPT_BEAT = "Beat up {agent}",
        DIALOG_BEAT = [[
            player:
                Let's see if we can put you into a more talkative mood.
            investigator:
                !right
                Oh, you're going to do that, eh?
                Just make sure you don't kill {agent.himher}. The Baron brass wants the guilty party alive.
            agent:
                !right
        ]],
        DIALOG_BEAT_SUSPECT_DEAD = [[
            * {agent} crumples like a sack of oshnu dung.
            investigator:
                !right
                What the hell! You weren't supposed to kill the suspects!
                Get out of here, and let me do my job!

        ]],
        DIALOG_BEAT_SUCCESS_GUILTY = [[
            agent:
                Stop! Stop! I confess!
                I did it! Long live the Rise!
        ]],
        DIALOG_BEAT_SUCCESS_INNOCENT = [[
            agent:
                Please! Stop!
                { suspect_knows ?
                    It was {guilty}! I saw {guilty.himher} do it!
                }
                { not suspect_knows ?
                    I would tell you if I knew! But I have no idea!
                }
        ]]


    }
    :AttractState("STATE_ATTRACT", function(cxt) return cxt.quest:IsActive("interrogate_suspect_3") or cxt.quest:IsActive("tell_investigator") end)
        :Fn(interrogation_fn) 


---------------------------------------------------------------------------------------------------
-- Here's where you tell the investigator whodunnit. OR you just make something up and make them believe it because rook is like that.
--  If you have a confession, you can just turn them in. If nobody has confessed, you want to pin it on someone else, you need to negotiate to convince.
---------------------------------------------------------------------------------------------------


QDEF:AddAttract("tell_investigator", "investigator",
    [[
        agent:
            !cruel
            Well, grifter? Have you figured it out with your superior intellect?
            Or are you just plain stumped?
    ]])

QDEF:AddConvo("tell_investigator", "investigator")
    :Loc{
        OPT_TELL_CONFESSION = "Tell {agent} about {guilty}'s confession",
        DIALOG_TELL_CONFESSION = [[
            player:
                We have our bomber, Inspector. {guilty} did it.
            agent:
                !right
                {guilty}? You sure?
            player:
                Quite.
            agent:
                Huh. Well, I wouldn't'a guessed that... but if it ever turns out you're wrong, at least it'll be you explaining that to {giver}.
            player:
                So are we done?
            agent:
                Hah! Done! Naw, you've gotta escort them to the interrogation site for questioning. 
                Have fun. 
        ]],

        OPT_PIN_BLAME_ON = "Pin the blame on {1}",
        DIALOG_PIN_BLAME_ON = [[
            player:
                I'm afraid so. {pinned_blame} did it, without a doubt.
            agent:
                {pinned_blame}? That sad sack of worms?
                You sure about that?
            player:
                Certainly. Allow me to walk you through it. 
        ]],
        DIALOG_PIN_BLAME_ON_SUCCESS = [[
            player:
                But imagine my surprise when I discovered the sick grandmother was nothing but a ruse!
                {pinned_blame} had never even <i>had</i> a grandmother.
            agent:
                !surprise
                That sick sunnova sea slug. 
                {pinned_blame}, you're under arrest. 
            the_accused:
                !left
                Wait, what? For what?? 
                !overthere
                I didn't do nothin', I told your guy--I didn't do it!
            agent:
                !right
                Fool me once, shame on you. Fool me twice...
                Take {pinned_blame.himher} away. 
            the_accused:
                !exit
            player:
                !left
                So are we done?
            agent:
                Hah! Done! Naw, you've gotta escort them to the interrogation site for questioning. 
                Have fun. 
        
        ]],
        DIALOG_PIN_BLAME_ON_FAILED = [[
            player:
                Besides, look at {pinned_blame.hisher} eyes. They're small. Beady.
                You can't trust eyes like that!
            agent:
                !angry_shrug
                Hey, my mother always said I had beautiful small, beady eyes. 
                What you got against 'em, huh?
            player:
                !interest
                Oh, well--
            agent:
                Enough of your pseudoscience. I don't got time for your false accusations. 
            player:
                !exit
            * Your short-lived career as a detective is over.
        ]],
    }
    :Hub(function(cxt) 
        if cxt.quest.param.has_confession then
            cxt:Opt("OPT_TELL_CONFESSION")
                :Fn(function() 
                    cxt.quest:AssignCastMember("the_accused", cxt.quest:GetCastMember("guilty"))
                    cxt:Dialog("DIALOG_TELL_CONFESSION")
                    cxt.quest:Complete("tell_investigator")
                    StateGraphUtil.AddLeaveLocation(cxt)                    
                end)
        end

        local targets = fun{cxt.quest:GetCastMember("suspect_1"), cxt.quest:GetCastMember("suspect_2"), cxt.quest:GetCastMember("suspect_3")}
                            :filter(function(agent) return not (cxt.quest.param.has_confession and agent == cxt.quest:GetCastMember("guilty")) end)
                            :value()

        for _, target in ipairs(targets) do
            cxt:Opt("OPT_PIN_BLAME_ON", target)
                :Fn(function() 
                    cxt:ReassignCastMember("pinned_blame", target)
                    cxt:Dialog("DIALOG_PIN_BLAME_ON")
                end)
                :Negotiation{
                    acting_against = {target},
                }
                    :OnSuccess()
                        :Fn(function() 
                            cxt.quest.param.did_pinned_blame = true
                            cxt.quest:AssignCastMember("the_accused", target)
                            cxt:Dialog("DIALOG_PIN_BLAME_ON_SUCCESS")
                            cxt.quest:Complete("tell_investigator")
                            StateGraphUtil.AddLeaveLocation(cxt)                    

                        end)
                    :OnFailure()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_PIN_BLAME_ON_FAILED")
                            cxt.quest:Fail()
                            StateGraphUtil.AddLeaveLocation(cxt)    
                        end)
        end
   end)


---------------------------------------------------------------------------------------------------
-- If you are transporting the guilty party... the rise tries to rescue them! Do a fight, or let them go.
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
-- If you are transporting an innocent... they beg for you to let them go. They might have money!
---------------------------------------------------------------------------------------------------

QDEF:AddConvo("escort_suspect_to_barons")
    :TravelConfront("INTERRUPT", function(cxt) return not cxt.quest.param.escort_interrupt end)
    :Fn(function(cxt) 
        cxt.quest.param.escort_interrupt = true
        cxt:GoTo(cxt.quest.param.did_pinned_blame and "STATE_BEG_FOR_LIFE" or "STATE_RISE_RESCUE_ATTEMPT")
    end)
    
    :State("STATE_BEG_FOR_LIFE")
        :Loc{
            DIALOG_INTRO = [[
                * After a long period of silence, {the_accused} speaks up.
                the_accused:
                    !right
                    I have money.
                player:
                    !left
                    I'm listening.
                the_accused:
                    It's buried out in the bog, for safekeeping. I can show you where it is, if you let me go.
                    There's a lot there. Hundreds of shills. It's worth it, trust me.
            ]],

            OPT_AGREE_TO_THE_DEAL = "Agree to the offer",
            DIALOG_AGREE_TO_THE_DEAL = [[
                player:
                    Well, let it never be said I'm not amenable to change. 
            ]],
            OPT_DO_NOT_AGREE_TO_THE_DEAL = "Reject the offer",
            DIALOG_DO_NOT_AGREE = [[
                player:
                    I'm afraid all stories need an ending. Even yours. 
                    The answer is no.
            ]]


        }
        :Fn(function(cxt) 
            cxt:TalkTo(cxt:GetCastMember("the_accused"))
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_AGREE_TO_THE_DEAL")
                :Dialog("DIALOG_AGREE_TO_THE_DEAL")
                :Fn(function() 
                    cxt.quest:Cancel("escort_suspect_to_barons")
                    cxt.quest:Activate("find_treasure")
                end)
                :Travel()

            cxt:Opt("OPT_DO_NOT_AGREE_TO_THE_DEAL")
                :Dialog("DIALOG_DO_NOT_AGREE")
                :Travel()
        end)

    
    :State("STATE_RISE_RESCUE_ATTEMPT")
        :Loc{
            DIALOG_INTRO = [[
                * You sense that you are being followed.
                player:
                    !left
                agent:
                    !right
                    Alright, grifter. Hand over {the_accused}, and we'll let you walk away.
                    But if you don't... then I'll have to kill you. It's just the way it is.
            ]],

            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Indeed, you'll have to kill me. 
                    Though I suspect you won't get very far.
            ]],
            DIALOG_DEFEND_WON = [[
                * As the smoke clears, you rush off with {the_accused}. No telling how many more Rise are about.
            ]],

            OPT_HAND_OVER = "Hand over {the_accused}",
            DIALOG_HAND_OVER = [[
                * You size up the opposition, and decide not to risk your life for the job.
                player:
                    Take {the_accused.himher}.
                the_accused:
                    !right
                    Hah! So long, sucker. 
                agent:
                    !right
                    Now get out of here, before we kill you to make a point!
            ]],

        }
        :Fn(function(cxt) 
            local rise = CreateCombatParty("RISE_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(rise[1])
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    noncombatants = {cxt.quest:GetCastMember("the_accused")}
                }
                    :OnWin()
                        :Dialog("DIALOG_DEFEND_WON")
                        :Travel()


            cxt:Opt("OPT_HAND_OVER")
                :Dialog("DIALOG_HAND_OVER")
                :Fn(function() 
                    cxt.quest:Fail()
                end)
                :Travel()

        end)



---------------------------------------------------------------------------------------------------
-- You dig up the suspect's buried money, and then decide to let them go or not
-- afterwards.... monsters!
---------------------------------------------------------------------------------------------------

QDEF:AddConvo("find_treasure")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("treasure_loc") end)
        :Loc{
            DIALOG_INTRO = [[
                * {the_accused} leads you to a nondescript stretch of bog.
                the_accused:
                    !right
                        Here it is.

                player:
                    !left
                * {the_accused} is pointing at a mound of recently disturbed peat.
                player:
                    !permit
                    Well. Go ahead.
            ]],

            DIALOG_NO_LOOT = [[
                the_accused:
                    !exit
                * {the_accused} digs for a long time, with a growing sense of panic. They're several feet deep when they come back up.
                the_accused:
                    !right
                    I swear, it was right here!
                player:
                    Uh huh.
                the_accused:
                    I - uh...
                    Please, just let me go, and I'll disappear. You can tell your masters that I fought back and you had to kill me! 
            ]],

            OPT_LET_GO = "Let {the_accused} leave",
            OPT_DO_NOT_LET_GO = "Take {the_accused} to the gate",

            
            DIALOG_LET_GO_NO_LOOT = [[
                player:
                    Fine. But don't let anyone in the bog ever hear from you again.
                the_accused:
                    Oh, thank you!
                    !exit
            ]],

            DIALOG_DO_NOT_LET_GO_NO_LOOT = [[
                player:
                    Perhaps you fail to understand the concept of an agreement.
                    But "no money" and "a pile of dirt" were not the terms of ours. 
                    The deal is off. Come along, now. 
                the_accused:
                    Bastard.
                    !exit
            ]],
            

            DIALOG_FOUND_LOOT = [[
                the_accused:
                    !exit
                * {the_accused} digs for a long time, eventually turning up a small metal box. {the_accused.HeShe} hands it to you triumphantly.
                the_accused:
                    !right
                    It's all here.
                    !give
                * You open the box and count up the contents: {1#money}.
                the_accused:
                    So, I guess I'll be going now?
            ]],

            DIALOG_LET_GO_LOOT = [[
                player:
                    Alright. We have an agreement.
                the_accused:
                    !exit
                * You start rehearsing your excuse for {giver} in your head as you head back to town.
            ]],
            DIALOG_DO_NOT_LET_GO_LOOT = [[
                player:
                    There's something you should learn, my friend. And you still have some time on this world to learn it.
                    When you're in handcuffs, no one needs to keep their promises. 
                the_accused:
                    ...
                    Bastard.    
            ]],
        }
        :Fn(function(cxt) 
            cxt:TalkTo("the_accused")
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Complete("find_treasure")

            if cxt.quest.param.treasure_exists then
                local loot_amt = (cxt.quest:GetRank() + math.random(3)) *50
                cxt:Dialog("DIALOG_FOUND_LOOT", loot_amt)
                cxt.enc:GainMoney(loot_amt)

                cxt:Opt("OPT_LET_GO")
                    :Dialog("DIALOG_LET_GO_LOOT")
                    :ReceiveOpinion("opinion_let_go_for_pay")
                    :ActivateQuest('convince_giver_accused_was_killed')
                    :Fn(function() 
                        cxt:GetAgent():Dismiss():Retire() 
                        cxt.quest.param.let_go = true
                    end)
                    :GoTo("STATE_MONSTER_FIGHT")

                
                cxt:Opt("OPT_DO_NOT_LET_GO")
                    :Dialog("DIALOG_DO_NOT_LET_GO_LOOT")
                    :ReceiveOpinion('stole_loot')
                    :ActivateQuest('escort_suspect_to_barons')
                    :GoTo("STATE_MONSTER_FIGHT")
            else
                cxt:Dialog("DIALOG_NO_LOOT")

                cxt:Opt("OPT_LET_GO")
                    :Dialog("DIALOG_LET_GO_NO_LOOT")
                    :ReceiveOpinion("opinion_let_go_for_free")
                    :ActivateQuest('convince_giver_accused_was_killed')
                    :Fn(function() 
                        cxt:GetAgent():Dismiss():Retire() 
                        cxt.quest.param.let_go = true
                    end)
                    :GoTo("STATE_MONSTER_FIGHT")
                
                cxt:Opt("OPT_DO_NOT_LET_GO")
                    :Dialog("DIALOG_DO_NOT_LET_GO_NO_LOOT")
                    :ActivateQuest('escort_suspect_to_barons')
                    :GoTo("STATE_MONSTER_FIGHT")
            end
        end)

    :State("STATE_MONSTER_FIGHT")
        :Loc{
            DIALOG_INTRO = [[
                * You are just about to march {the_accused} out of the bog when you realize that the noise of digging has attracted the attention of the local fauna.
                agent:
                    !right
            ]],
            DIALOG_INTRO_ALONE = [[
                * As {the_accused} leaves, you realize that the noise of digging has attracted the attention of the local fauna.
                agent:
                    !right
            ]],
            
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND_WITH_OTHER = [[
                player:
                    !fight
                agent:
                    !right
                * You and {the_accused} face down the monsters.
            ]],
            DIALOG_DEFEND_ALONE = [[
                player:
                    !fight
                * You size up the beasts and charge your blasters.
            ]],

            DIALOG_DEFEND_WON = [[
                {not let_go ?
                    {the_accused_dead ?
                        * {the_accused} lies dead in a tangle of bodies and ichor.
                        * You decide to cut your losses and leave.
                    }

                    {not the_accused_dead ?
                        the_accused:
                            !right
                            We did it! We...
                        * {the_accused} looks at your weapons and sighs.
                        player:
                            Time to go.
                    }
                }
                {let_go?
                    * Time to go convince {giver} that none of this happened.
                }
            ]],

            DIALOG_DEFEND_ALONE_WON = [[
                player:
                    !fight
                * The clearing smells strongly of charred meat. Time to leave before the rest of the bog decides to join the feast.
            ]],

        }
        :Fn(function(cxt) 
            local alone = not cxt.quest:GetCastMember("the_accused"):IsInPlayerParty()
            local opfor = CreateCombatParty("BOG_MONSTERS", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(opfor[1])
            
            cxt:Dialog(alone and "DIALOG_INTRO_ALONE" or "DIALOG_INTRO")
            
            
            cxt:Opt("OPT_DEFEND")
                :Battle{
                    allies = not alone and {cxt.quest:GetCastMember("the_accused")}
                }
                    :OnWin()
                        :Fn(function()
                            local died_in_fight = not alone and cxt.quest:GetCastMember("the_accused"):IsDead()
                            cxt.enc.scratch.the_accused_dead = died_in_fight
                            cxt:Dialog("DIALOG_DEFEND_WON")
                            if died_in_fight then
                                cxt.quest:Cancel("escort_suspect_to_barons")
                                cxt.quest:Activate("tell_about_death")
                            end

                            StateGraphUtil.AddLeaveLocation(cxt)                            
                        end)
            end)



---------------------------------------------------------------------------------------------------
-- Your quarry is dead. Go tell.
---------------------------------------------------------------------------------------------------

QDEF:AddAttract("tell_about_death", "giver",
    [[
        agent:
            Where's the prisoner? I haven't heard anything from the gate yet.
    ]])

QDEF:AddConvo("tell_about_death", "giver")
    :Loc{
        OPT_TELL_GIVER = "Tell {giver} that {the_accused} died",
        DIALOG_TELL_GIVER = [[
            player:
                I'm sorry but {the_accused} is dead.
            agent:
                We wanted to make an example of {the_accused.himher}! I'll pay you, but consider yourself on notice!
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_TELL_GIVER")
            :Dialog("DIALOG_TELL_GIVER")
            :Fn(function() cxt.quest.param.poor_performance = true end)
            :CompleteQuest()
            :End()
    end)

---------------------------------------------------------------------------------------------------
-- If you let your quarry go, you can tell {giver} that they died for partial credit
---------------------------------------------------------------------------------------------------

QDEF:AddAttract("convince_giver_accused_was_killed", "giver",
    [[
        agent:
            Where's the prisoner? I haven't heard anything from the gate yet.
    ]])

QDEF:AddConvo("convince_giver_accused_was_killed", "giver")
    :Loc{
        OPT_CONVINCE_GIVER = "Convince {giver} that {the_accused} died in an ambush",
        DIALOG_CONVINCE_GIVER = [[
            player:
                I'm afraid there was an ambush. The Rise must've been watching for our passage.
                I was able to deal with them, but {the_accused} was killed in the fray.
        ]],
        DIALOG_CONVINCE_GIVER_SUCCESS = [[
            player:
                Perhaps I'd have managed, if the investigator you'd assigned had seen fit to provide me with an escort. 
            agent:
                You <i>were</i> the escort. But fine--I suppose the Rise had the upperhand today. 
        ]],
        DIALOG_CONVINCE_GIVER_FAILURE = [[
            player:
                And as {the_accused.heshe} lay dying in my arms, I promised... I promised to never forget.
            agent:
                Yes, well, it certainly <i>reads</i> like a melodrama. 
                I'm not sure what you've actually done with {the_accused}...
                But let us both hope it never comes back to bite you where it truly hurts, hm?
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE_GIVER")
            :Dialog("DIALOG_CONVINCE_GIVER")
            :Negotiation{}
                :OnSuccess()
                    :Dialog("DIALOG_CONVINCE_GIVER_SUCCESS")
                    :Fn(function() cxt.quest.param.poor_performance = true end)
                    :CompleteQuest()
                    :End()
                :OnFailure()
                    :Dialog("DIALOG_CONVINCE_GIVER_FAILURE")
                    :FailQuest()
                    :End()
    end)

---------------------------------------------------------------------------------------------------
-- You deliver your quarry to the gate, and get paid there.
---------------------------------------------------------------------------------------------------



QDEF:AddConvo("escort_suspect_to_barons")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("gate") end)
        :Loc{
            DIALOG_INTRO = [[
                * You approach the gate. {giver} is waiting for you.
                giver:
                    !right
                    I see you have our terrorist.
                player:
                    !left
                    {did_pinned_blame?
                        Be careful with this one--{the_accused.heshe} may not look it, but {the_accused.heshe}'s a ravenous killer.
                    }
                    {not did_pinned_blame?
                       Indeed, believe it or not. 
                    }
                giver:
                    Hand them over. We're gonna escort them to the Spark mining operation over at Lakespit. 
                    Teach 'em how to harvest, then sell them at a mark-up to the derricks.
                the_accused:
                    !right
                    N-no! I got family in Havaria! You can't!
                * It turns out, they very much can. 
                    !exit
            ]],
            OPT_GET_PAID = "Get paid",
            DIALOG_GET_PAID = [[
                giver:
                    !right
                player:
                    And what about payment? Surely one good turn...
                giver:
                    Right. Of course. Here you go.
                    !give
            ]],
            DIALOG_GET_PAID_2 = [[
                giver:
                    !exit
                * {giver} heads back to {giver.hisher} bunker, flanked by guards.
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            if cxt.quest:GetCastMember("the_accused") then
                cxt.quest.param.delivered = true
                cxt.quest:GetCastMember("the_accused"):Dismiss()
                if not cxt.quest:GetCastMember("the_accused"):IsRetired() then
                    
                    cxt.quest:GetCastMember("the_accused"):Retire()
                end
            end

            cxt:Opt("OPT_GET_PAID")
                :Dialog("DIALOG_GET_PAID")
                :Fn(function() 
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)        
                end)
                :Dialog("DIALOG_GET_PAID_2")
                :Fn(function() 
                    cxt.quest:GetCastMember("giver"):MoveToLimbo()
                    StateGraphUtil.AddEndOption(cxt)                
                end)
        end)        



-----------------------------------------------------------------------------------------------------------

QDEF:AddDebugOption("suspect_1_relationship", {"FRIEND", "NEUTRAL", "ENEMY"})
QDEF:AddDebugOption("suspect_2_relationship", {"FRIEND", "NEUTRAL", "ENEMY"})
QDEF:AddDebugOption("suspect_3_relationship", {"FRIEND", "NEUTRAL", "ENEMY"})

QDEF:AddDebugOption("suspect_1_knows", {false, true})
QDEF:AddDebugOption("suspect_2_knows", {false, true})
QDEF:AddDebugOption("suspect_3_knows", {false, true})

QDEF:AddDebugOption("treasure_exists", {false, true})
