local QDEF = QuestDef.Define{
    title = "Food Fight",
    qtype = QTYPE.CONTRACT,
    faction = "SPARK_BARONS",
    rank = {1, 3},
    desc = [[{giver} needs you to defuse a potentially violent situation at {caffy}.]],
    icon = engine.asset.Texture("icons/quests/baron_food_fight.tex"),
    focus = QUEST_FOCUS.NEGOTIATION,
    reward_size = QREWARDSIZE.SMALLER,
    on_start = function(quest)
        quest:Activate("deal_with_caffy")
    end,
}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddLocationCast{
    cast_id = "caffy",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_CAFFY" ))
    end,
}

:AddObjective{
    id = "deal_with_caffy",
    title = [[Go to {caffy} and deal with the upset workers.]],
    desc = "Some workers have gotten unruly at the Baron cafeteria. Go deescalate the situation before too much damage occurs.",
    --state = QSTATUS.ACTIVE,
    mark = {"caffy"},

    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[Go tell {giver} that the trouble at {caffy} is over.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}

:AddOpinionEvents{

    got_them_extra_food =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Got them their fair share of rations",
    },
    
    help_them_keep_skimming =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Helped them continue skimming rations",
    },
}



local WORKER_SPAWNER = function(difficulty_level)
    if difficulty_level <= 1 then
        return {"LABORER", "LABORER"}
    elseif difficulty_level == 2 then
        return {"LABORER", "HEAVY_LABORER"}
    elseif difficulty_level >= 3 then
        return {"HEAVY_LABORER", "HEAVY_LABORER"}
    end
end


QDEF:AddConvo("deal_with_caffy")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("caffy") and cxt.location:GetProprietor() ~= nil end)
        :Loc{
            DIALOG_INTRO = [[
                * You step through the door and find a group of workers facing off against a solitary Baron.
                baron:
                    !right
                    !angry
                    You already got your rations. Now take a walk!
                worker:
                    !left
                    !angry
                    That wasn't enough, and you know it!
                    How are we supposed to get our job done if we're half-starved?
                    !angry_threaten
                    Your robot ain't here to protect you today, Baron!
                baron:
                    !exit
                worker:
                    !exit
                * Seems you've arrived in time to extinguish the spark of a riot. 

                player:
                    !left
            ]],
            OPT_TALK_TO_WORKER = "Talk to the lead worker",
            OPT_TALK_TO_BARON = "Talk to the Spark Baron",
        }
        :Fn(function(cxt) 
            cxt.quest.param.baron =  cxt.location:GetProprietor()
            cxt.quest.param.guard =  cxt.location:GetAgentByRole(CHARACTER_ROLES.GUARD)
            LocationUtil.SendPatronsAway(cxt.location)
            if cxt.quest.param.guard then
                cxt.quest.param.guard:MoveToLimbo()
            end
            cxt.quest.param.workers =  CreateCombatParty(WORKER_SPAWNER, cxt.quest:GetRank(), cxt.location, true)
            cxt.quest.param.baron:MoveToLocation(cxt.location)
            
            
            cxt:ReassignCastMember("worker", cxt.quest.param.workers[1])
            cxt:ReassignCastMember("baron", cxt.quest.param.baron)

            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_TALK_TO_WORKER")
                :GoTo("STATE_TALK_TO_WORKER")

            cxt:Opt("OPT_TALK_TO_BARON")
                :GoTo("STATE_TALK_TO_BARON")
        end)


    :State("STATE_TALK_TO_WORKER")
        :Loc{
            DIALOG_INTRO_FIRST_FRIEND = [[
                player:
                    !greeter
                    Hello, there, {worker}! What seems to be the trouble?
                worker:
                    !right
                    $miscRelieved
                    {player}! This oshnooze has been cheating us out of our rations for weeks now, and making us pay out of pocket when we're hungry!
                    !shrug
                    Maybe you can scare {baron.himher} straight. Surely even the bosses must see we can't work like this.
            ]],
            DIALOG_INTRO_FIRST = [[
                * You turn to the leader of the unhappy laborers.
                player:
                    Hullo. What's going on here, if I might ask?
                worker:
                    !right
                    Our contract says we are supposed to get two square a day, but this slop ain't enough.
                    This Baron is cutting our rations and expecting us to pay {baron.himher} to make up the difference.
                    We won't be extorted any more!
            ]],
            DIALOG_INTRO_NOT_FIRST = [[
                worker:
                    !right
                    So? What are you going to do?
            ]],


            --Threaten the workers. Makes the baron happy with you
            OPT_THREATEN = "Threaten {agent} into submission",
            DIALOG_THREATEN = [[
                player:
                    If you have a problem with your contract, take it up with your foreman.
                    Now get out of here before I lose my temper.
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                worker:
                    You haven't heard the last of this.
                    You might want be careful walking around the bog at night, grifter.
                    !exit
                * The workers storm out.
                baron:
                    !right
                    !happy
                    Thanks, pal. You really have to be firm with these scrubs, eh? Give 'em an inch and they'll come take all of your inches.
            ]],
            DIALOG_THREATEN_FAIL = [[
                worker:
                    You don't scare me.
                    I'm tired of being pushed around!
            ]],

            --Fight the workers. The baron will back you up if you arranged that beforehand. Makes a lot of enemies.
            OPT_FORCE_LEAVE = "Force {worker} to leave",
            DIALOG_FORCE_LEAVE = [[
                player:
                    !fight
                    Fun's over. The door's this way.
                worker:
                    !fight
                    Oppressor! You're as bad as they are!
            ]],
            DIALOG_FORCE_LEAVE_NO_KILLS_BARON = [[
                survivor:
                    !injured
                    I'm going! I'm going!
                    !exit
                * The troublemakers all leave the building.
                baron:
                    !right
                    This didn't solve anything. They're just going to be back again later.
                player:
                    I did my part. Long-term solutions aren't in my job description.
            ]],
            DIALOG_FORCE_LEAVE_NO_KILLS_NO_BARON = [[
                survivor:
                    !injured
                    I'm going! I'm going!
                    !exit
                * The troublemakers all leave the building.
                * Of course, {baron} was killed in the fight, so you're not sure what was really gained.
            ]],
            DIALOG_FORCE_LEAVE_SURVIVORS_BARON = [[
                survivor:
                    !injured
                    You're a killer!
                    !exit
                * The surviving troublemakers file out the door, watching you nervously.
                baron:
                    !right
                    Well <b>that</> escalated.
                player:
                    !shrug
                    Maybe now they'll think twice. Or they'll run straight to the Rise. Either way, it's no longer my problem.
            ]],
            DIALOG_FORCE_LEAVE_SURVIVORS_NO_BARON = [[
                survivor:
                    !injured
                    You're a killer!
                    !exit
                * The surviving troublemakers file out the door, watching you nervously.
                * You step over {baron}'s corpse on your way out.
            ]],
            DIALOG_FORCE_LEAVE_NO_SURVIVORS_BARON = [[
                * You turn over the bodies with your foot to make sure there are no survivors.
                baron:
                    !right
                    What the hell? Look at the mess you left for me!
                player:
                    Take it up with {giver}, if you have a problem. 
                    If I start a fight, I end it.
            ]],
            DIALOG_FORCE_LEAVE_NO_SURVIVORS_NO_BARON = [[
                * You were supposed to prevent violence, and ended up killing literally everyone involved. 
                * Seems you really made a mess of this.
            ]],

            --if you can convince both sides to meet half-way, you can walk away without anyone hating (or liking) you.
            OPT_COMPROMISE = "Convince {worker} to compromise",
            TT_COMPROMISE = "You will have to convince both sides to accept the compromise.",
            DIALOG_COMPROMISE = [[
                player:
                    $miscPersausive
                    I can't promise you everything you're asking for, but what if I could get you a little more?
                    Enough to sate the growling in your bellies, at least.
                    Would you stand down then?
            ]],
            DIALOG_COMPROMISE_SUCCESS = [[
                player:
                    $miscPersuasive
                    After all, if you can reach an agreement, then the Barons won't be so quick to dismiss your concerns.
                    !overthere
                    Not when you've shown them how reasonable you can be.
                worker:
                    !palm
                    Something <b>is</> better than nothing.
                    And I'm too hungry to put up much of a fight. 
                    We'll listen to their offer.
            ]],
            DIALOG_COMPROMISE_FAIL = [[
                player:
                    After all, you wouldn't want to spoil your appetite with too <i>much</i> food, would you?
                worker:
                    !angry_shrug
                    $angryPatienceLost
                    Are you serious?!
                    Clearly the only way we're going to get anything is with a show of force.
                    Step aside, Baron dog, and let us solve our own problems. 
            ]],

            OPT_TALK_TO_BARON = "Talk to {baron} instead",
            DIALOG_TALK_TO_BARON = [[
                player:
                    Hold that thought.
            ]],
            REQ_ALREADY_CONVINCED_TO_FIGHT = "{baron} is ready to fight",
            REQ_READY_TO_COMPROMISE = "{agent} is ready to compromise",
            REQ_COMPROMISE_FAILED = "Compromise is no longer an option",
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.quest.param.workers[1])
                if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and not cxt.quest.param.heard_worker_intro then
                    cxt:Dialog("DIALOG_INTRO_FIRST_FRIEND")
                else                    
                    cxt:Dialog(not cxt.quest.param.heard_worker_intro and "DIALOG_INTRO_FIRST" or "DIALOG_INTRO_NOT_FIRST")
                end
                cxt.quest.param.heard_worker_intro = true

            end

            cxt:Opt("OPT_THREATEN")
                :Dialog("DIALOG_THREATEN")
                :ReqCondition(not cxt.quest.param.convince_worker_compromise or cxt.quest.param.compromise_failed, "REQ_READY_TO_COMPROMISE" )
                :Negotiation{
                    suppressed = {cxt.quest.param.baron },
                    flags = NEGOTIATION_FLAGS.INTIMIDATION,
                    on_success = function()
                        cxt:Dialog("DIALOG_THREATEN_SUCCESS")
                        for k,v in ipairs(cxt.quest.param.workers) do
                            v:MoveToLimbo()
                        end
                        
                        cxt.quest.param.baron:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("help_them_keep_skimming"))
                        
                        cxt.quest:Complete("deal_with_caffy")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function()
                        cxt:Dialog("DIALOG_THREATEN_FAIL")
                    end,
                }

            cxt:Opt("OPT_COMPROMISE")
                :PostText("TT_COMPROMISE")
                :Dialog("DIALOG_COMPROMISE")
                :ReqCondition(not cxt.quest.param.baron_convinced_fight, "REQ_ALREADY_CONVINCED_TO_FIGHT")
                :ReqCondition(not cxt.quest.param.compromise_failed, "REQ_COMPROMISE_FAILED" )
                :Negotiation{
                    suppressed = {cxt.quest.param.baron },
                    on_success = function()
                        cxt:Dialog("DIALOG_COMPROMISE_SUCCESS")
                        cxt.quest.param.convince_worker_compromise = true
                        if cxt.quest.param.convince_baron_compromise then
                            cxt:GoTo("STATE_COMPROMISE")
                        end
                    end,
                    on_fail = function()
                        cxt:Dialog("DIALOG_COMPROMISE_FAIL")
                        cxt.quest.param.compromise_failed = true
                    end,
                }
            
            local fight_allies = {}
            if cxt.quest.param.baron_convinced_fight then
                table.insert(fight_allies, cxt.quest.param.baron)
                --table.insert(fight_allies, cxt.quest.param.guard)
            end

            cxt:Opt("OPT_FORCE_LEAVE")
                :Dialog("DIALOG_FORCE_LEAVE")
                :Battle{
                    flags = BATTLE_FLAGS.NO_BYSTANDERS,
                    enemies = cxt.quest.param.workers,                    
                    allies = fight_allies,
                    on_win = function(cxt) 
                        local survivors, killed = ConvoUtil.CollectSurvivors(cxt.quest.param.workers, true)
                        local baron_dead = cxt.quest.param.baron:IsDead()

                        if #survivors > 0 then
                            cxt:ReassignCastMember("survivor", survivors[1])
                            cxt.enc:PresentRight( survivors[1] )
                        end

                        if #killed == 0 then
                            cxt:Dialog(baron_dead and "DIALOG_FORCE_LEAVE_NO_KILLS_NO_BARON" or "DIALOG_FORCE_LEAVE_NO_KILLS_BARON")
                        elseif #survivors > 0 then
                            cxt:Dialog(baron_dead and "DIALOG_FORCE_LEAVE_SURVIVORS_NO_BARON" or "DIALOG_FORCE_LEAVE_SURVIVORS_BARON")
                            cxt.quest.param.used_extreme_violence = true
                        else
                            cxt:Dialog(baron_dead and "DIALOG_FORCE_LEAVE_NO_SURVIVORS_NO_BARON" or "DIALOG_FORCE_LEAVE_NO_SURVIVORS_BARON")
                            cxt.quest.param.used_extreme_violence = true
                        end
                        cxt.quest.param.baron_killed = baron_dead
                        cxt.quest.param.used_violence = true
                        cxt.quest:Complete("deal_with_caffy")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }

            
            cxt:Opt("OPT_TALK_TO_BARON")
                :Dialog("DIALOG_TALK_TO_BARON")
                :GoTo("STATE_TALK_TO_BARON")
        end)


    :State("STATE_TALK_TO_BARON")
        :Loc{
            DIALOG_INTRO_FIRST_FRIEND = [[
                baron:
                    !right
                    !happy
                    Oh thank the Spark you showed up.
                    !shrug
                    These scrubs disabled my security bot, and now they're making a power play.
                    I'd appreciate your help laying down the law—they've got numbers on their side, and they're mean when they're hungry.
            ]],
            DIALOG_INTRO_FIRST = [[
                baron:
                    !right
                    !sigh
                    $miscRelieved
                    You're {giver}'s man, right?
                    My usual Enforcer's in the repair shop, so of course the local rabble are pushing their luck today.
                    Please take care of this so I can get back to business.
            ]],
            DIALOG_INTRO_NOT_FIRST = [[
                baron:
                    !right
                    Just crack a couple skulls and get them out of here!
                    They're not going to listen to anything else.
            ]],

            OPT_CONVINCE_FIGHT = "Convince {baron} to back you up in a fight",
            DIALOG_CONVINCE_FIGHT = [[
                player:
                    This might get violent. I need to know I can count on you.
            ]],
            DIALOG_CONVINCE_FIGHT_FAIL = [[
                player:
                    I'd rather not fight your battles for you, you see.
                baron:
                    I'm not fighting this lot—I'm above that. And also in the middle of my lunch. 
            ]],
            DIALOG_CONVINCE_FIGHT_SUCCESS = [[
                player:
                    We'll handle them together.
                baron:
                    Alright. Make your move, and I've got your back.
                    !placate
                    Just try not to make too much of a mess. I'll be the one cleaning it up after.
            ]],

            OPT_THREATEN = "Threaten {baron} to stop skimming food",
            DIALOG_THREATEN = [[
                player:
                    !angry
                    You and I both know that you're skimming off the top.
                    If you cut that out, there'll be plenty to go around. 
                    Or I could put a word or two into the right ears, and you'll have some awkward questions to answer.
            ]],
            DIALOG_THREATEN_FAIL = [[
                player:
                    !hips
                    What do you think of that?
                baron:
                    You're threatening to expose me? 
                    !cruel
                    Ha! Go ahead, tell anyone you want. This is Grout Bog, not a knitting party. We're <i>all</> corrupt as Hesh here.
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                player:
                    !thought
                    $neutralThoughtful
                    I know laborers can be promoted to Spark Barons if they do well enough. I wonder if the opposite is also true?
                baron:
                    !placate
                    Fine. I hear you. 
                    The money's not even that good, anyway.
                    !spit
                    Nobody except the workers will eat this slop, and they're all poor as bog spit.
                    !exit
                * {baron} heads to the back room to get more rations to distribute.
                worker:
                    !left
                    $neutralThanks
                    Thank you, grifter. It's not often that we get a fair shake here.
            ]],


            OPT_CONVINCE_COMPROMISE = "Convince {baron} to compromise",
            TT_COMPROMISE = "You will have to convince both sides to accept the compromise.",
            DIALOG_CONVINCE_COMPROMISE = [[
                player:
                    Look, they just want a little more to eat. Is that really such a problem?
                    You could meet them half-way, couldn't you?
            ]],
            DIALOG_CONVINCE_COMPROMISE_FAIL = [[
                player:
                    Surely you see the benefit in a well-fed workforce.
                baron:
                    !angry
                    $angryPatienceLost
                    Give them an inch, and they'll cut my throat. I'm not negotiating with these rats.
            ]],
            DIALOG_CONVINCE_COMPROMISE_SUCCESS = [[
                player:
                    $miscPersuasive
                    They're not striking or shirking their duties, {baron}. They're only hungry.
                    !interest
                    Feed them, and their productivity will increase.
                baron:
                    !shrug
                    $neutralResigned
                    Ach, fine. I guess they have been looking a little gaunter lately.
            ]],

            OPT_TALK_TO_WORKER = "Talk to {worker} instead",
            DIALOG_TALK_TO_WORKER = [[
                player:
                    Excuse me.
            ]],
            REQ_ALREADY_CONVINCED_TO_FIGHT = "{baron} is ready to fight!",
            REQ_READY_TO_COMPROMISE = "{baron} is ready to compromise",
            REQ_COMPROMISE_FAILED = "Compromise is no longer an option",

        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.quest.param.baron)
                if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL then
                    cxt:Dialog("DIALOG_INTRO_FIRST_FRIEND")
                else
                    cxt:Dialog(not cxt.quest.param.heard_baron_intro and "DIALOG_INTRO_FIRST" or "DIALOG_INTRO_NOT_FIRST")
                end
                cxt.quest.param.heard_baron_intro = true
            end

            cxt:Opt("OPT_THREATEN")
                :ReqCondition(not cxt.quest.param.convince_baron_compromise or cxt.quest.param.compromise_failed, "REQ_READY_TO_COMPROMISE" )
                :Dialog("DIALOG_THREATEN")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.INTIMIDATION,
                    subject = cxt.quest.param.workers[1],
                    on_success = function()
                        cxt:Dialog("DIALOG_THREATEN_SUCCESS")
                        cxt.quest:Complete("deal_with_caffy")
                        cxt.quest.param.workers[1]:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("got_them_extra_food"))
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,

                    on_fail = function()
                        cxt:Dialog("DIALOG_THREATEN_FAIL")
                    end,
                }

            cxt:Opt("OPT_CONVINCE_COMPROMISE")
                :Dialog("DIALOG_CONVINCE_COMPROMISE")
                :PostText("TT_COMPROMISE")
                :ReqCondition(not cxt.quest.param.baron_convinced_fight, "REQ_ALREADY_CONVINCED_TO_FIGHT")
                :ReqCondition(not cxt.quest.param.compromise_failed, "REQ_COMPROMISE_FAILED" )
                :Negotiation{
                    subject = cxt.quest.param.workers[1],
                    on_success = function()
                        cxt:Dialog("DIALOG_CONVINCE_COMPROMISE_SUCCESS")
                        cxt.quest.param.convince_baron_compromise = true
                        if cxt.quest.param.convince_worker_compromise then
                            cxt:GoTo("STATE_COMPROMISE")
                        end
                    end,
                    on_fail = function()
                        cxt:Dialog("DIALOG_CONVINCE_COMPROMISE_FAIL")
                        cxt.quest.param.compromise_failed = true
                    end,
                }

            cxt:Opt("OPT_CONVINCE_FIGHT")
                :Dialog("DIALOG_CONVINCE_FIGHT")
                :ReqCondition(not cxt.quest.param.convince_baron_compromise or cxt.quest.param.compromise_failed, "REQ_READY_TO_COMPROMISE" )
                :Negotiation{
                    subject = cxt.quest.param.workers[1],
                    acting_against = cxt.quest.param.workers,
                    on_success = function()
                        cxt:Dialog("DIALOG_CONVINCE_FIGHT_SUCCESS")
                        cxt.quest.param.baron_convinced_fight = true
                    end,
                    on_fail = function()
                        cxt:Dialog("DIALOG_CONVINCE_FIGHT_FAIL")
                    end,
                }
            
            cxt:Opt("OPT_TALK_TO_WORKER")
                :Dialog("DIALOG_TALK_TO_WORKER")
                :GoTo("STATE_TALK_TO_WORKER")        
        end)

    :State("STATE_COMPROMISE")
        :Loc{
            DIALOG_INTRO_WORKER = [[
                player:
                    It's settled, then, isn't it?
                baron:
                    !left
            ]],
            DIALOG_INTRO_BARON = [[
                player:
                    It's settled, then, isn't it?
                worker:
                    !left
            ]],
            DIALOG_INTRO_COMMON = [[
                baron:
                    I talked it over with {player}, and I can give you all a <i>bit</> more.
                worker:
                    We'll take that as a first step. But we're watching you.
                baron:
                    And I'm watching you. If so much as a spoon goes missing from this place, I'll spike your food with flead oil and give you all a stain no amount of washing will solve.
                worker:
                    !exit
                baron:
                    !exit
                * You leave them to work out the finer points.
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog( cxt:GetCastMember("right") == cxt.quest.param.workers[1] and "DIALOG_INTRO_WORKER" or "DIALOG_INTRO_BARON" )
            cxt:Dialog("DIALOG_INTRO_COMMON")
            cxt.quest:Complete("deal_with_caffy")
            StateGraphUtil.AddLeaveLocation(cxt)
        end)


QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_INTRO = [[
            player:
                The trouble at the caffy is over.
        ]],
        DIALOG_SUCCESS_VIOLENCE = [[
            agent:
                You were supposed to talk everyone down, not beat them up.
                We're just lucky nobody got killed.
        ]],
        DIALOG_SUCCESS_KILLED = [[
            agent:
                Yes, I heard how you dealt with that.
                Next time, try doing it with a little less bloodshed, will you?
        ]],
        DIALOG_SUCCESS = [[
            agent:
                A calm situation is good for business. I knew I could count on you.
        ]],
        DIALOG_SUCCESS_BARON_KILLED = [[
            agent:
                Yeah, and we lost a valued employee.
                !point
                Be more careful with company assets!
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_INTRO")
            :Fn(function() 
                if cxt.quest.param.baron_killed then
                    cxt.quest.param.poor_performance = true
                    cxt:Dialog("DIALOG_SUCCESS_BARON_KILLED")
                elseif cxt.quest.param.used_extreme_violence then
                    cxt.quest.param.poor_performance = true
                    cxt:Dialog("DIALOG_SUCCESS_KILLED")
                elseif cxt.quest.param.used_violence then
                    cxt.quest.param.poor_performance = true
                    cxt:Dialog("DIALOG_SUCCESS_VIOLENCE")
                else
                    cxt:Dialog("DIALOG_SUCCESS")
                end
                
                ConvoUtil.GiveQuestRewards(cxt)
            end)
            :CompleteQuest()
    end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                Now, we see to it that the Barons dine like the kra'deshi dynasties of old. 
                !shrug
                $neutralWhatever
                But for the laborers, we process some of the biomass extraction byproduct into a nutritional paste.
            player:
                $miscMocking
                I'm surprised you haven't cracked under the strain of your jealousy.
            agent:
                !handwave
                Please, Rook, I've tasted it myself—no worse than Admiralty field rations. 
                It's calories and nutrients, exactly what's needed to keep the workforce healthy.
                !angry
                $angryDefault
                Unfortunately, the ingrates aren't satisfied. 
                They've been causing chaos at the serving lines and it's holding up meal breaks.
            player:
                !agree
                Ah. A delay in operations jeopardizes the whole mission.
            agent:
                As you well know. 
            *** {giver} needs you to clear up some trouble at the local caffy.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                In this instance I honestly don't care for the details—the whole mess is tiresome. 
                See that the ruckus is quelled, so I can turn my brain power back to the bottom line, hm?
            player:
                !salute
                As you like. 
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            
        end)
