local INFO_PRICE = 100
local SILENCE_BRIBE = 50

local function HasAutoDog(cxt)
    if TheGame:GetGameState():GetCaravan():GetPet() ~= nil then
        local pets = TheGame:GetGameState():GetCaravan():GetPets()
        for k,pet in pairs(pets) do
            if pet:GetContentID() == "AUTODOG" then
                cxt.enc.scratch.autodog = pet
                return true
            end
        end
    end
    return false
end

local QDEF = QuestDef.Define{
        title = "Caught in the Chaos",
        desc = "{target} is being held for ransom by the Rise. {giver} wants you to get {target} out by whatever means necessary.",
        icon = engine.asset.Texture("icons/quests/baron_hostage_situation.tex"),
        rank = {1,2},
        qtype = QTYPE.CONTRACT,
        focus = QUEST_FOCUS.BALANCED,
        reward_size = QREWARDSIZE.SMALLER,
        faction = "SPARK_BARONS",
        
        is_in_hiding = function(quest, agent)
            if quest:IsActive("investigate_location") and agent == quest:GetCastMember("target") then
                return true
            end
        end,

        on_destroy = function(quest)
            if quest:GetCastMember("target") and quest:GetCastMember("target"):IsInPlayerParty() then
                quest:GetCastMember("target"):Dismiss()
            end
        end,

        avoids = 
        {
            {"giver", "target"},
        },
    }

    :AddOpinionEvents{
        rescued_them = {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Rescued them from the Rise",
        },
        dismissed_concerns = {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Dismissed their concerns about the Rise",
        },
    }

    
    :AddObjective{
        id = "investigate_location",
        title = "Investigate {camp_exterior#location}",
        desc = "{target} is being held at {camp_exterior#location}. Go to the encampment and find {target}.",
        mark = { "camp_exterior" },
        state = QSTATUS.ACTIVE,
        
        on_complete = function( quest )
            quest:GetCastMember("interior"):SetMapPos(quest:GetCastMember("camp_exterior"):GetMapPos())
            quest:Activate("bargain_for_release")
        end,

        on_fail = function( quest)
            TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("camp_exterior"))
        end,    
    }

    :AddObjective{
        id = "bargain_for_release",
        title = "Freedom for a price",
        desc = "Find a way to make the Rise release {target}.",
        state = QSTATUS.INACTIVE,

        on_complete = function(quest)
            quest:Activate("return_for_reward")
        end
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Report back to {giver}",
        desc = "The situation with {target} has been taken care of. Go collect your reward.",
        mark = { "giver" },
    }

    :AddLocationDefs{
        RISE_ENCAMPMENT_EXT = {
            name = "Rise Encampment",
            desc = "A hastily-constructed base of operations for the Rise.",
            plax = "EXT_Rise_Encampment_01",
            map_tags = {"bog"},
            indoors = false,
        },
        
        RISE_ENCAMPMENT_INT = {
            name = "Rise Encampment",
            desc = "A hastily-constructed base of operations for the Rise.",
            plax = "INT_Rise_Encampment_1",
            indoors = true,
        },
    }

    :AddLocationCast{
        cast_id = "camp_exterior",
        cast_fn = function(quest, t)
            table.insert(t,  quest:SpawnTempLocation("RISE_ENCAMPMENT_EXT"))
        end,
    }
    :AddLocationCast{
        cast_id = "interior",
        cast_fn = function(quest, t)
            table.insert(t,  quest:SpawnTempLocation("RISE_ENCAMPMENT_INT"))
        end,
    }


    :AddCast{
        cast_id = "giver",
        provider = true,
        no_validation = true,
    }

    :AddCast{
        cast_id = "target",
        cast_fn = function( quest, t )
            local agent = quest:CreateSkinnedAgent( "SPARK_BARON_TASKMASTER" )
            table.insert(t, agent)
        end,
    }

QDEF:AddConvo("investigate_location")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Confront(function(cxt)
            if cxt.location == cxt.quest:GetCastMember("camp_exterior") then
                return "STATE_ARRIVE"
            end
        end)
    :State("STATE_ARRIVE")
        :Loc{
            DIALOG_INTRO = [[
                * Fortunately it's easy enough to find the Rise encampment, which means they're open to negotiation.
                * After all, they know the bog better than you do—if they didn't want to be found, it'd have been much more of a challenge.
                * Sure enough you're not shot on sight: a Rise member approaches, weapon at hand but not at the ready.
                agent:
                    !right
                player:
                    !left
            ]],
            OPT_CONFIRM = "Find someone who has information",
            OPT_ASK_ABOUT_TARGET = "Ask about {target}",
            DIALOG_ASKED_ABOUT_TARGET = [[
                agent:
                    !crossed
                    What do you want, stranger? This part of the bog ain't exactly hospitable to gawkers.
                player:
                    I've come seeking a Baron, actually—{target}. Have you heard of {target.himher}?
                agent:
                    !dubious
                    $neutralDubious
                    What's it to you? That doesn't seem like any of your business.
            ]],
            OPT_PAY_FOR_INFO = "Pay {agent} to tell you",
            DIALOG_PAY_FOR_INFO = [[
                player:
                    $miscPersuasive
                    Perhaps this sum will convince you of my wholely professional interest.
                    Take it as a sign of good will. For a fair negotiation, of course.
                    !give
                agent:
                    !take
                    Ah, a middleman, is it? Not sure you have the power to grant what we're asking—
                    But we'll take your money, and you can see if you get much further.
                    {target}'s over in the shack. We're watching you, though.
                * {agent} gestures towards an inconspicuous spot in the tree line.
                agent:
                    Between the two ferns, there's a hidden structure. 
                    That's where you'll find your Baron.
                player:
                    !happy
                    I'm quite obliged.
                * You part the leaves and head inside.
            ]],
            OPT_ATTACK_FOR_INFO = "Beat the information out of {agent}",
            DIALOG_ATTACK_FOR_INFO = [[
                player:
                    Unfortunately for you, it very much <i>is</i>. 
                    And when I'm hired for a job, I'm very thorough.
                agent:
                    !angry_accuse
                    $angryDefault
                    Oh, I see. You're working for the Barons.
                    Then you'll be treated like one.
                player:
                    !fight
            ]],
            DIALOG_ATTACK_WON_SURVIVED = [[
                player:
                    !left
                agent:
                    !right
                    !injured
                    Stop! Please! You win, alright? {target.HeShe}'s not worth dying over. 
                * {agent} gestures towards an inconspicuous spot in the tree line.
                agent:
                    They're in there. But don't expect a warm welcome.
                player:
                    $neutralDirect
                    I never do.
                * You leave {agent} behind and head inside.
            ]],
            DIALOG_ATTACK_WON_DIED = [[
                player:
                    !left
                agent:
                    !right
                    !injured
                    Stop! Please! You win, alright? {target.HeShe}'s not worth dying over. 
                * {agent} gestures towards an inconspicuous spot in the tree line.
                agent:
                    They're in there. There's a secret entrance. But don't expect a warm welcome.
                    Not after you killed one of ours.
                player:
                    $neutralDirect
                    Perhaps they'll be more amenable to reason. 
                    After all, I make a <i>very</i> compelling argument.
                * You leave {agent} behind and head inside.
            ]],
            DIALOG_ATTACK_WON_ALL_DEAD = [[ 
                player:
                    !left
                * It seems you've squandered any hope of gathering information. 
                * The Rise are clever. Though you're experienced at reading tracks, they've hid theirs well.
                * So it takes an age to find what you're looking for: a well-hidden shack, blending into the tree line.
                * You duck your head between the rough leaves at its entrance and head inside.
            ]],
            OPT_CONVINCE = "Convince {agent} you have an important task",
            DIALOG_CONVINCE = [[
                agent:
                    !right
                player:
                    !left
                    Kalandra sent me. I'm to escort {target} to a safehouse. 
                    !cagey
                    We have reason to suspect the Barons have a rescue in mind and we need to convince them otherwise.
                agent:
                    !dubious
                    $neutralTakenAback
                    A rescue? Barons don't usually open a vein for one of their own. You sure?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !shrug
                    $miscPersuasive
                    Yes, well, {target} is more important than we first assumed.
                    Which is why it's all the more crucial we keep {target.himher} in pocket, as a bargaining chip. 
                agent:
                    !neutralThoughtful
                    Huh. That's big news. All right, then—the hidden entrance is over there.
                    !point
                * {agent} points you towards an inconspicuous spot in the tree line, and you head inside.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                agent:
                    !crossed
                    Hrm. Sorry, but Rise protocol is to keep our secrets close if we have any doubt at all.
                    And I got heaps. 
            ]],
        }
        :SetLooping(true)
        :Fn(function(cxt )
            if cxt:FirstLoop() then
                local rise_guys = CreateCombatParty("RISE_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.quest.param.rise_guys = rise_guys
                cxt.enc:SetPrimaryCast(rise_guys[1])
                cxt:Dialog("DIALOG_INTRO")
                cxt:Dialog("DIALOG_ASKED_ABOUT_TARGET")
            end

            cxt:Opt("OPT_PAY_FOR_INFO")
                :Dialog("DIALOG_PAY_FOR_INFO")
                :DeliverMoney(INFO_PRICE)
                :Fn(function( cxt )
                    cxt.quest:Complete("investigate_location")
                    cxt:Pop()
                end)

            cxt:Opt("OPT_ATTACK_FOR_INFO")
                :Dialog("DIALOG_ATTACK_FOR_INFO")
                :Battle{
                    on_win = function(cxt)
                        if cxt:GetAgent():IsDead() then
                            local all_dead = true
                            for i,target in ipairs(cxt.quest.param.rise_guys) do
                                if not target:IsDead() then
                                    all_dead = false
                                    cxt.enc:SetPrimaryCast(target)
                                    break
                                end
                            end
                            if all_dead then
                                cxt:Dialog("DIALOG_ATTACK_WON_ALL_DEAD")
                            else
                                cxt:Dialog("DIALOG_ATTACK_WON_DIED")
                            end
                        else
                            cxt:Dialog("DIALOG_ATTACK_WON_SURVIVED")
                        end
                        cxt.quest:Complete("investigate_location")
                        cxt:Pop()
                    end
                }

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest:Complete("investigate_location")
                        cxt:Pop()
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                    end
                }
        end)

QDEF:AddConvo("bargain_for_release")
    :Confront(function(cxt)
        return "STATE_BARGAIN"
    end)
    :State("STATE_BARGAIN")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !right
                player:
                    !left
                agent:
                    !dubious
                    Excuse you. What do you think you're doing in here?
                player:
                    I'm here to pick up {target}. Boss' orders.
                agent:
                    !crossed
                    This is the first I'm hearing of it.
            ]],
            OPT_OFFER_TECH = "Offer to trade {target} for your Autodog",
            DIALOG_OFFER_TECH = [[
                player:
                    Listen, I don't have time to answer all your questions. How about you hand over {target} and I'll give you this Autodog, mint condition.
                * {agent} eyes the autodog carefully, inspecting it closely.
                agent:
                    !shrug
                    Yeah okay, that seems fair. Those things are darn useful when they actually work.
                player:
                    !give
                agent:
                    !take
                player:
                    Glad we could come to an agreement.
                * {target} shoots you a disapproving glare.
            ]],
            OPT_ATTACK = "Attack {agent}",
            DIALOG_ATTACK = [[
                player:
                    I don't have time for all this idle chatter, I'm afraid. Just hand {target.himher} over.
                agent: 
                    !angry
                    You don't tell me what to do.
                player:
                    !fight
                    Well, it's true what they say, yes? "Show, don't tell." Much more evocative.
            ]],
            DIALOG_ATTACK_WON_SURVIVED = [[
                player:
                    !left
                agent:
                    !right
                    !injured
                    Alright, alright! Just take {target.himher}!
                player:
                    About time.
                target:
                    !right
                    Thanks for getting me out of this mess.
            ]],
            DIALOG_ATTACK_WON_DIED = [[
                player:
                    !left
                * {agent} is dead. You untie {target} and leave the encampment unnoticed.
            ]],
            OPT_CONVINCE = "Convince {agent.himher} to give you {target}",
            DIALOG_CONVINCE = [[
                player:
                    !permit
                    $miscAlluring
                    You know how the Rise operates, my dear. The less you know, the less you can give away if compromised.
                agent:
                    I ain't your "dear". You think you can just boss me around?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    $miscPersuasive
                    Of course I don't wish to boss you around. This fight affects us all.
                    !interest
                    But as long as the Baron is here, {target.heshe} poses a danger to us all. 
                    $miscPersuasive
                    I'm taking great pains to ensure your safety.
                agent:
                    !sigh 
                    $neutralResigned
                    I suppose that's all we can ask. Fine, take {target.himher}. I just wish they would tell us these things.
                player:
                    !agree
                    I'm glad you understand. Your frustration won't be in vain, I assure you.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    $miscMocking
                    You're a laborer, are you not? I'd think you'd be used to taking orders by now.
                agent:
                    !angry 
                    $angryReluctant
                    I am. And last I heard, my orders were to guard {target}. 
                    And I'm not gonna take your word otherwise.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("interior") )
                local rise_guys = CreateCombatParty("RISE_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt:TalkTo(rise_guys[1])
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    flags = BATTLE_FLAGS.NO_BYSTANDERS,
                    on_win = function(cxt)
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_ATTACK_WON_DIED")
                        else
                            cxt:Dialog("DIALOG_ATTACK_WON_SURVIVED")
                        end
                        cxt.quest:GetCastMember("target"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest:Complete("bargain_for_release")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                }

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest:GetCastMember("target"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest:Complete("bargain_for_release")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                    end
                }

            if HasAutoDog(cxt) then
                cxt:Opt("OPT_OFFER_TECH")
                    :Dialog("DIALOG_OFFER_TECH")
                    :Fn(function(cxt)
                        cxt.enc.scratch.autodog:Dismiss()
                        cxt.quest:GetCastMember("target"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest.param.gave_tech = true
                        cxt.quest:Complete("bargain_for_release")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end 
        end)
QDEF:AddConvo("return_for_reward")
    :Confront(function(cxt)
        if cxt.quest.param.gave_tech and cxt.location:HasTag("in_transit") and not cxt.quest.param.done_interrupt then
            return "STATE_INTERRUPT"
        end
    end)
    :State("STATE_INTERRUPT")
        :Loc{
            DIALOG_INTRO = [[
                * {target} grabs you by the shoulder.
                player:
                    !left
                agent:
                    !right
                    {giver} isn't going to be happy that you just handed the Rise some high quality tech.
                    You know they're going to use that thing against our guys!
            ]],
            OPT_DISMISS = "Dismiss {target}'s concerns",
            DIALOG_DISMISS = [[
                player:
                    Listen, I got you out, that's what matters. 
                agent:
                    Yeah, but-
                player:
                    Let's keep walking.
            ]],
            OPT_CONVINCE = "Convince {target} to keep quiet",
            DIALOG_CONVINCE = [[
                player:
                    Listen, I got you out, that's what matters. {giver} doesn't need to know about the tech.
                agent:
                    Give me one reason I shouldn't tell {giver.himher}.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    Do you see what I mean?
                agent:
                    Fine. I won't tell {giver.himher} anything, but don't think you'll get away with this again.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    You see? It's really just better if {giver} doesn't know.
                agent:
                    I'm not going to lie to my boss just to save your reputation. Forget about it.
            ]],
            OPT_BRIBE = "Bribe {target} to keep quiet",
            DIALOG_BRIBE = [[
                player:
                    Listen, I did what I had to do to get you out. How about we make an agreement? 
                    I'll pay you for your troubles and you keep quiet.
                    !give
                agent:
                    !take
                    Hmm, well when you put it like that.. You've got yourself a deal.
            ]], 
            OPT_CONTINUE = "Keep moving",
            TT_CONTINUE = "{giver} won't be happy.",
            DIALOG_CONTINUE = [[
                player:
                    Listen, you can do whatever you feel like you need to do.
                agent:
                    !angry
                    There's a right way to deal with these Rise guys and giving them more tech to attack us with isn't it.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt:TalkTo("target")
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest.param.done_interrupt = true
            end

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest.param.gave_tech = false
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                    end
                }

            cxt:Opt("OPT_BRIBE")
                :DeliverMoney( SILENCE_BRIBE )
                :Dialog("DIALOG_BRIBE")
                :Fn(function(cxt)
                    cxt.quest.param.gave_tech = false
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_DISMISS")
                :Dialog("DIALOG_DISMISS")
                :ReceiveOpinion("dismissed_concerns")
                :Travel()
        end)



--turn the quest in to the giver.
QDEF:AddConvo("return_for_reward", "giver")

    :Loc{
        OPT_REPORT_RESCUE = "Tell {agent} that {target} has been freed",   
        DIALOG_INTRO = [[
            player:
                !left
            giver:
                !right
                !greeting
                $happyGreeting
                Glad to see you in one piece, old boy. You never really know when dealing with the Rise, you know?
                I trust everything went smoothly?
            player:
                !agree
                Quite. {target} has been removed from Rise clutches, as requested.
        ]],
        DIALOG_INTRO_GAVE_AUTODOG = [[
            target:
                !left
            giver:
                !right
                !dubious
                $neutralDubious
                Good to see you, {target}. I'm glad {player} got you out.
            target:
                !shrug
                It wasn't without a cost. {player} gave them an Autodog in exchange. 
            player:
                !left
                !crossed
                $neutralDirect
                I believe I was invited to accomplish this task at my discretion. 
            giver:
                !angry
                $angryAside
                I'd rather think arming the enemy with state-of-the-art tech would've been an obvious no-no, Rook.
                $angryReluctant
                You did the work, but I hope you remember your covert tactics training next time.
        ]],
        DIALOG_TAKE_DOG = [[
            giver:
                !interest
                And how did the Autodog serve you, eh? A trusted ally, no?
            player:
                !dubious
                Seems to work just as well as the mechs I've encountered.
                $miscNostalgic
                Though the four legs are a charming touch.
        ]],
        DIALOG_AUTODOG_DIED = [[
            giver:
                !interest
                And how did the Autodog serve you, eh? A trusted ally, no?
            player:
                !agree
                $miscAlluring
                Certainly—up to and including its untimely demise. 
                $happyCocky
                But I'm sure you'll agree it wasn't destroyed in vain. I'm quite well.
            giver: 
                !scared
                $neutralTakenAback
                Ah. That's disappointing. 
                !neutral
                But that's what field tests are for, I suppose.
        ]],
        DIALOG_THANKS = [[
            target:
                !right
                $neutralThanks
                Thanks for the rescue. Never thought the boss would send an old war buddy to pull me outta the muck.
            player:
                !wink
                $miscMischievous
                Fellemo is full of surprises. Our targets used to learn that the hard way. 
        ]]
        }
    :Hub(function(cxt) 
        cxt:Opt("OPT_REPORT_RESCUE", cxt.quest.param.buyer)
            :SetQuestMark()
            :Fn(function() 
                if cxt.quest.param.gave_tech then
                    cxt:Dialog("DIALOG_INTRO_GAVE_AUTODOG")
                else
                    cxt:Dialog("DIALOG_INTRO")
                    if cxt.quest.param.borrowed_autodog ~= nil then
                        if cxt.quest.param.borrowed_autodog:IsInPlayerParty() then
                            cxt:Dialog("DIALOG_TAKE_DOG")
                            cxt.quest.param.borrowed_autodog:Dismiss()
                        else
                            cxt:Dialog("DIALOG_AUTODOG_DIED")
                        end
                    end
                    cxt:Dialog("DIALOG_THANKS")
                    cxt.quest:GetCastMember("target"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("rescued_them"))
                end
                cxt.quest:Complete()
                ConvoUtil.GiveQuestRewards(cxt)
            end)
            :End()
    end)

------------------------------------------------------------------------------------------------------------------
    
QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            player:
                !left
            giver:
                !right
                This one requires the utmost discretion.
            player:
                $neutralDirect
                Of course. Frankly it's insulting you'd even specify.
            giver:
                !shrug
                $neutralDirect
                Your track record has been, shall we say, <i>spotty</i> as of late, no?
                And I'm the client, so I'll set the parameters as I see fit.
            player:
                !crossed
                $angryReluctant
                Indeed. Carry on, then.
            giver:
                !interest
                There was a small upset in one of the work camps today. We sent in the usual suppression force and, well...
                The workers have been quelled, but the Rise made off with a captive. A <i>Baron</i> captive.
                !sigh 
                $neutralResigned
                It'd be one thing if it were just a foreman, but you can see how this would be damaging to the brand.
                Make the problem go away, won't you?
        ]],
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_ON_TAKEN = [[
            player:
                !salute
                $neutralDirect
                Well, as we used to say: mission accepted.
            giver:
                Glad to hear it. Whatever it takes, old chap, you understand?
                !point
                Just get {target} back. But discreetly.
        ]],
        DIALOG_GIVE_AUTODOG = [[
            giver:
                Oh, and perhaps you ought to take one of our Autodogs with you. Just in case you get into a spot.
                !wink
                It might pay to have <i>Spot</i> on hand, eh?
            player:
                ...
                !agree
                Quite.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_ON_TAKEN")
            if not HasAutoDog(cxt) then
                cxt:Dialog("DIALOG_GIVE_AUTODOG")
                local autodog = Agent("AUTODOG")
                autodog:GenerateRandomName()
                TheGame:GetGameState():AddAgent( autodog )
                autodog:Recruit(PARTY_MEMBER_TYPE.CREW)
                cxt.quest.param.borrowed_autodog = autodog
            end
        end)

