
local PAY_FOR_PRESSURE_COST = 35
local PAY_FOR_INFO_COST = 50
local PAY_TO_LEAVE_ALONE = 100

local QDEF = QuestDef.Define{
    title = "Message Control",
    qtype = QTYPE.CONTRACT,
    faction = "SPARK_BARONS",
    rank = {1, 3},
    desc = [[{giver} wants you to scare off a Rise recruiter who's been sniffing around one of the work sites.]],
    icon = engine.asset.Texture("icons/quests/baron_intimidate_a_rise_recruiter.tex"),
    reward_size = QREWARDSIZE.SMALLER,
    focus = QUEST_FOCUS.NEGOTIATION,

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("target"), location = quest:GetCastMember("hiding_spot") } )
        
        if quest:IsActive("return_for_reward") then
            table.insert (t, { agent = quest:GetCastMember("giver"), location = TheGame:GetGameState():GetLocation("GB_BARON_HQ") } )
        end

    end,
    on_start = function(quest)
        quest:Activate("find_target")
    end,

    on_post_load = function(quest)
        local target = quest:GetCastMember( "target" )
        if quest:IsActive( "confront_target" ) and (target == nil or target:IsRetired()) then
            quest:Cancel()
        end
    end,


    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("target") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,

    avoids = 
    {
        {"giver", "target"},
    },
}

:AddLocationCast{
    cast_id = "hiding_spot",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("HIDING_SPOT"))
    end,
}

:AddLocationDefs{
    HIDING_SPOT = {
        name = "Hidden Grove",
        map_tags = {"forest"},
        plax = "EXT_RISE_CAMPSITE",
        indoors = false,
    },
}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddCast{
    cast_id = "target",
    condition = function(agent, quest) 
        return agent:GetContentID() == "RISE_PAMPHLETEER"
    end,
}

:AddCastFallback
{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "RISE_PAMPHLETEER" ) )
    end,
}

:AddLocationCast{
    cast_id = "worksite",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_WORKSITE_1" ))
        table.insert(t, TheGame:GetGameState():GetLocation("GB_WORKSITE_2" ))
    end,
}

:AddObjective{
    id = "find_target",
    title = [[Find {target}]],
    desc = "Talk to the workers at {worksite} to find out where {target} is hiding.",
    --combat_targets = {"target"},

    mark = function(quest, t, in_location)
        if not in_location then
            table.insert(t, quest:GetCastMember("worksite"))
        elseif TheGame:GetGameState():GetPlayerAgent():GetLocation() == quest:GetCastMember("worksite") then
            for _, agent in quest:GetCastMember("worksite"):Agents() do
                if agent:GetBrain():GetWorkplace() == quest:GetCastMember("worksite") then
                    table.insert(t, agent)
                end
            end
        end
    end,

    on_complete = function(quest) 
        quest:Activate("confront_target")
    end
}

:AddObjective{
    id = "confront_target",
    title = [[Confront {target}]],
    desc = "Confront {target} and make {target.himher} leave {worksite} alone.",
    --combat_targets = {"target"},
    mark = { "target" },
    on_activate = function(quest)
        quest:GetCastMember("target"):MoveToLimbo()
    end,

    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[Go tell {giver} that {target} is no longer a problem.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}

:AddOpinionEvents{
    killed_a_worker = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Killed one of their workers",
    },
    roughed_up_a_worker = 
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Roughed up one of their workers",
    },
    got_pressure_from_boss =  
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Got their boss to put pressure on them",
    },
}


local function ApplyPressure(cxt)
    for k,v in cxt.location:Agents() do
        if v:GetRoleAtLocation() == CHARACTER_ROLES.LABOUR then
            cxt.quest.param.got_help_from_boss = true
            v:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("got_pressure_from_boss"))
        end
    end
end


QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_INTRO = [[
            player:
                !hips
                The work at {worksite} should continue uninterrupted. 
                Though you may want to see about getting them some entertainment. 
            agent:
                !thought
                Entertainment? Hm. I suppose there might be a laborer who might sing, or... or tell jokes!
                $neutralDirect
                Only when they're not on shift, of course.     
        ]],
        DIALOG_RESPONSE = [[
            player:
                It took some convincing, but I was able to deal with the pamphleteer discreetly. 
            agent:
                Excellent. How wonderful something could go so smoothly for you.
        ]],
        DIALOG_RESPONSE_PAID_RISE = [[
            player:
                Despite their ideals, the Rise are susceptible to a pay-off as much as any bureaucrat.
            agent:
                !dubious
                You're supposed to be <i>stopping</i> the Rise, not funding them.
                But I suppose it was a mere trifle, and you wouldn't spend your shills frivolously. 
        ]],
        DIALOG_RESPONSE_HURT_WORKERS = [[
            player:
                However, there may have been an upset.
            agent:
                !angry
                Ah, yes. I <i>heard</i>. 
                I see what subtlety you once had has been disposed of, along with your leg and your commission. 
                You'll have to do far better than that in the future, <i>Captain</i>, if you want to maintain my once-high esteem.
        ]],
        DIALOG_RESPONSE_KILLED_RISE = [[
            player:
                The pamphleteer was eliminated. I assure you, there were no witnesses.
            agent:
                !interest
                Ah. That's perhaps a firmer touch than I would've recommended... 
                !throatcut
                Remove one head with the Rise and three more will spring forth from the wound, I find. 
                Still, I appreciate that you kept it discreet. 
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_INTRO")
            :Fn(function() 
                    if cxt.quest.param.hurt_a_worker then
                        cxt.quest.param.poor_performance = true
                        cxt:Dialog("DIALOG_RESPONSE_HURT_WORKERS")
                    elseif cxt.quest.param.paid_rise then
                        cxt.quest.param.poor_performance = true
                        cxt:Dialog("DIALOG_RESPONSE_PAID_RISE")
                    elseif cxt.quest:GetCastMember("target"):IsDead() then
                        cxt.quest.param.poor_performance = true
                        cxt:Dialog("DIALOG_RESPONSE_KILLED_RISE")
                    else
                        cxt:Dialog("DIALOG_RESPONSE")
                    end

                end)
            :CompleteQuest()
    end)

QDEF:AddConvo( "confront_target")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("target"):GetLocation() end)
    :Loc{
        DIALOG_INTRO = [[
            * You make your way through the bog forest to the spot marked on your map. You're just about to give up when you spot a ramshackle tent.
            * {target} emerges as you approach.
            target:
                !right
                !angry
                Who's there?
            player:
                !left
        ]],
        DIALOG_REACTION= [[
            agent:
                !dubious
                You lost, sir? This ain't the road.
            player:
                Afraid not. The Barons have sent me.
            agent:
                !crossed
                What for? I'm not on shift yet.
        ]],
        DIALOG_REACTION_FRIEND = [[
            agent:
                !happy
                Well, I'll be! How'd you find me all the way out here?
                You're not in trouble with the Barons, are you?
            player:
                $neutralResigned
                Quite the opposite, I'm afraid. The Barons sent me.
            agent:
                !dubious
                $neutralDubious
                Alright. Lay it out for me. 
        ]],
        OPT_CONVINCE_BACK_OFF = "Convince {agent} to stay away from {worksite}",
        DIALOG_CONVINCE_BACK_OFF = [[
            player:
                $miscPersuasive
                I need you to leave {worksite} alone. 
                !permit
                Cut your losses and find a new stomping ground.
            agent:
                !crossed
                I'm not doing anything wrong. I'm just talking to the workers there.
                The Barons haven't outlawed <i>talking</i> on shift. Not yet, anyway.
            player:
                Surely there must be something else you can do for your cause.
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                !overthere
                I know you don't fear demerits, but the Barons are highly motivated to get {worksite} back to its quota.
                For now, they've asked me to talk to you. But perhaps they'll resort to less savory tactics. 
                $miscPersuasive
                !interest
                You can't do much righteous rabble-rousing if you're, ah, <i>retired</i>.
            agent:
                !thought
                $neutralThoughtful
                Hm. They're <i>that</i> peeved, huh?
                !agree
                Good. Let them crack. Others will work their way under their skin soon enough.
                As for you... think long and hard about your motivations.
                !throatcut 
                You wouldn't wanna end up on the wrong side of history.
                !exit
            * {agent} starts packing up {agent.hisher} tent. You suspect {agent.heshe} intends to disappear.
        ]],
        DIALOG_CONVINCE_FAIL = [[
            player:
                !overthere
                The Barons are very upset with you. Surely you don't want the demerits that are soon to come?
            agent:
                !cruel
                Hah! Demerits! 
                They're upset 'cause the pamphlets are <i>working</i>. This just confirms it!
                I'm making a difference. What can you say for yourself, huh?
        ]],

        OPT_BEAT_UP = "Beat up {agent}",
        DIALOG_BEAT_UP = [[
            player:
                !fight
                I'm afraid your workplace performance has been found lacking. 
                You're to be penalized in a way you'll understand.
            agent:
                You wanna <i>fight</i> me?
                !fight
                Fine. I don't even need to know why. 
                You got a death wish, that's your prerogative.
        ]],
        DIALOG_WON_KILLED = [[
            * {agent} is dead. You're probably not going to capture the hearts and minds of the locals this way.
        ]],
        DIALOG_WON_ALIVE = [[
            agent:
                !injured
                H-hey, I yield, alright? 
                This ain't worth dying for.
            player:
                I'm pleased you agree. Just stay away from {worksite} and with any luck our paths won't cross again.
            agent:
                Yeah, sure. 'Cause I'm feeling real lucky right now.
                !exit
            * {agent} retreats to {agent.hisher} tent to nurse {agent.hisher} wounds.
        ]],
        OPT_PAY_TO_LEAVE = "Pay {agent} to disappear",
        DIALOG_PAY_TO_LEAVE = [[
            player:
                The Barons have asked me to ensure you stay away from {worksite}. No more stirring up dissent for you.
                $miscPersuasive
                As I imagine you have some sense of integrity, I will ask you as nicely as possible.
                With shills. 
            agent:
                !dubious
                So you pay me, and I stop recruiting at {worksite}?
            player:
                !agree
                That is the arrangement I'm proposing, yes.
            agent: 
                Deal. Pamphlets aren't the only trick up my sleeve. 
                This'll fund some other very nice initiatives.
                !take
            player:
                !give
                As you like. 
            agent:
                But if the Barons want to keep sending you after me, I hope you keep paying me.
                !exit
            * {agent} leaves, {agent.hisher} chuckles trailing after {agent.himher}.
        ]],
    }
    :SetLooping()
    :Fn(function(cxt) 
        
        if cxt:FirstLoop() then
            cxt:Dialog("DIALOG_INTRO")
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("target"))
            cxt:Dialog(cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_REACTION_FRIEND" or "DIALOG_REACTION")
        end

        cxt:Opt("OPT_PAY_TO_LEAVE")
            :Dialog("DIALOG_PAY_TO_LEAVE")
            :DeliverMoney(PAY_TO_LEAVE_ALONE)
            :Fn(function(cxt) 
                cxt.quest.param.paid_rise = true
            end)
            :CompleteQuest("confront_target")
            :Travel()

        cxt:Opt("OPT_CONVINCE_BACK_OFF")
            :Dialog("DIALOG_CONVINCE_BACK_OFF")
            :Negotiation{
                on_success = function() 
                    cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                    cxt.quest:Complete("confront_target")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end,
                on_fail = function() 
                    cxt:Dialog("DIALOG_CONVINCE_FAIL")
                end,
            }

        cxt:Opt("OPT_BEAT_UP")
            :Dialog("DIALOG_BEAT_UP")
            :Battle{
                on_win = function(cxt) 
                    cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_WON_KILLED" or "DIALOG_WON_ALIVE")
                    cxt.quest:Complete("confront_target")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end,
            }
    end)


QDEF:AddConvo( "find_target")
    :Loc{
        OPT_ASK_ABOUT_PAMPHLETEER = "Ask about the pamphleteer",
        DIALOG_ASK_PROPRIETOR_ABOUT_PAMPHLETEER = [[
            player:
                What can you tell me about this pamphleteer?
            agent:
                !shrug
                Whoever they are, they're sly. I blink for one second and suddenly they're all holding leaflets.
                You'll have to ask the workers. But don't hurt 'em too bad. I have quotas to keep.
        ]],
        DIALOG_ASK_PROPRIETOR_FOR_HELP = [[
            player:
                !thought
                $neutralThoughtful
                I suspect your workers will be, ah, hostile upon interrogation. 
                Can you exert any pressure to make them more amenable?
            agent:
                !dubious
                Look, bud—this worksite is a pressure pot. 
                If I flex, they'll be even harder to keep in line once you're gone.
            player:
                !crossed
                And how disobedient will they be once the Rise has taken hold?
            agent:
                $neutralThoughtful
                Hrm. I'm listening.
        ]],
        
        OPT_PAY_FOR_PRESSURE = "Pay {agent} to pressure the workers",
        DIALOG_PAY_FOR_PRESSURE = [[
            player:
                Your time and expertise are valuable. I will pay you for your assistance.
            agent:
                Hey now—that's the kind of compromise I like.
        ]],

        OPT_CONVINCE_PRESSURE = "Convince {agent} to pressure the workers",
        DIALOG_CONVINCE_PRESSURE = [[
            player:
                !overthere
                Surely you understand the notion of mutually beneficial.
        ]],
        DIALOG_PROP_NEGOTIATION_SUCCESS = 
        [[
            player:
                !point 
                $miscPersuasive
                Help me help you, my good fellow. 
                You know your laborers best.
            agent:
                !palm 
                $neutralResigned
                Ugh, fine. But if this bites me in the ass, I'll remember that smug mustache of yours.
        ]],
        DIALOG_PROP_NEGOTIATION_FAIL = [[
            player:
                !eyeroll
                $miscMocking
                Would that I could do your job for you, this would already be over.
            agent:
                !angry
                $angryDefault
                Don't tell me how to run my business, grifter!
                Go see to your own responsibilities and stay out of mine.
        ]],

        DIALOG_WORKERS_PRESSURED = [[
            agent:
                !agree
                I'll put the word out that anyone who doesn't help you gets a double shift.
                That oughta loosen some tongues.
            player:
                !happy
                Many thanks.
            * You hear some angry grumbling from the assembled workers. Seems like some of them were eavesdropping.
        ]],

        DIALOG_ASK_GUARD_ABOUT_PAMPHLETEER = [[
            player:
                !interest
                I'm looking for a Rise pamphleteer that's been causing trouble. Have you seen them?
            agent:
                !crossed
                Nope. They're smarter than that.
                $neutralResigned
                If I saw them we wouldn't have this problem anymore, would we?
        ]],
        DIALOG_ASK_LABOUR_ABOUT_PAMPHLETEER_PRESSURED = [[
            player:
                Tell me where the Rise agent is, or your boss is going to make life very difficult for you.
            agent:
                !angry
                Fine. I'll tell you.
            * {agent} marks a location on your map.
            agent:
                Now get out of my face. I have a living to make here.

        ]],
        DIALOG_ASK_LABOUR_ABOUT_PAMPHLETEER = [[
            player:
                !salute
                Hello, fellow worker. I don't suppose you've seen a Rise pamphleteer?
            agent:
                !dubious
                $neutralDubious
                And why would I tell you if I had?
        ]],

        OPT_ASK_FOR_HELP = "Ask for some help",
        OPT_BRIBE_LABOUR = "Pay for the information",
        
        DIALOG_BRIBE_LABOUR_ABOUT_PAMPHLETEER = [[
            player:
                !give
                Let me make it worth your while.
            agent:
                !interest
                $happyTakenAback
                That's enough to pay off a few demerits, make no mistake.
                Here, I'll show you where to find the pamphlets. Just don't let anyone know it was me that talked.
            * {agent} marks your map.
        ]],
        
        
        OPT_CONVINCE_INFO = "Convince {agent} to talk",
        DIALOG_NEGOTIATE_LABOUR_ABOUT_PAMPHLETEER = [[
            player:
                Consider your options.
        ]],        

        DIALOG_NEGOTIATE_LABOUR_ABOUT_PAMPHLETEER_SUCCESS = [[
            player:
                !interest
                $miscPersuasive
                Surely this isn't worth the demerits. After all, what has the Rise ever done for you, really?
            agent:
                !cagey
                Okay, okay. Keep your voice down.
            * {agent} marks your map.
            agent:
                Doing this is gonna get me blacklisted with some of the foremen for sure.
                So don't let this come back to me, all right?
        ]],

        DIALOG_NEGOTIATE_LABOUR_ABOUT_PAMPHLETEER_FAIL = [[
            player:
                !angry
                You wouldn't want to be written up, would you?
            agent:
                !angry_crossed
                $angryDefault
                Oh, so that's how it is, huh? I'm not telling you anything, scab.
        ]],

        DIALOG_ASK_LABOUR_ABOUT_PAMPHLETEER_LIKED = [[
            player:
                Why, hello, {agent}. 
            agent:
                !happy
                Hey, {player}. Didn't expect to see you here.
            player:
                I'm looking for someone. A Rise pamphleteer, who may frequent this worksite.
            agent:
                !neutral
                $neutralResigned
                Oh. Ah. You're here about that, huh?
                !cagey
                I'll tell you where they are, but you can't let anyone know I talked.
            player:
                I value little more than discretion, I assure you.
            agent:
                !give
            * {agent} marks a spot on your map.
            agent:
                That's where you'll find 'em. Keep out of trouble, all right?
            player:
                I will certainly give it my level best.
        ]],
        DIALOG_ASK_GUARD_FOR_HELP = [[
            player:
                Anything you can do to help me out with this Rise problem?
            agent:
                !shrug
                Eh. If crackin' skulls was enough to fix the problem it'd be fixed already.
                Try talking to the workers.
        ]],

        OPT_ROUGH_UP_LABOUR = "Beat the information out of {agent}",
        DIALOG_ROUGH_UP_LABOUR = [[
            * You cast a steely eye around the room, making sure everyone knows this is official Spark Baron business, and that they shouldn't interfere.
            player:
                !fight
                You're going to tell me what I need to know, one way or another.
            agent:
                !scared
                H-hey! This ain't right! I'm on duty!
        ]],
        DIALOG_ROUGHED_UP_LABOUR = [[
            agent:
                !injured
                S-stop! You made your point. 
                The guards aren't gonna do nothing to stop you. So I'll tell you.
            * {agent} scribbles some marks on your map.
            agent:
                Now leave me alone!
        ]],
        DIALOG_KILLED_LABOUR = [[
            * {agent} lies at your feet.
            * You rifle through their pockets and find the info written down on a scrap of paper.
            * You're pretty sure that {giver} is not going to approve of these methods, if the looks around you are anything to go by.

        ]]
    }
    :Hub(function(cxt) 
        if cxt.location == cxt.quest:GetCastMember("worksite") and cxt:GetAgent():GetBrain():GetWorkplace() == cxt.location then
            if cxt:GetAgent():GetRoleAtLocation() == CHARACTER_ROLES.PROPRIETOR then
                
                cxt:Opt("OPT_ASK_ABOUT_PAMPHLETEER")
                    :SetQuestMark()
                    :Dialog("DIALOG_ASK_PROPRIETOR_ABOUT_PAMPHLETEER")

                
                if not cxt.quest.param.got_help_from_boss then                
                    cxt:Opt("OPT_ASK_FOR_HELP")
                        :SetQuestMark()
                        :Dialog("DIALOG_ASK_PROPRIETOR_FOR_HELP")
                        :LoopingFn(function() 
                            
                            cxt:Opt("OPT_PAY_FOR_PRESSURE")
                                :DeliverMoney(PAY_FOR_PRESSURE_COST)
                                :Dialog("DIALOG_PAY_FOR_PRESSURE")
                                :Dialog("DIALOG_WORKERS_PRESSURED")
                                :Fn(function()
                                    ApplyPressure(cxt)
                                    cxt:Pop()
                                end)
                                :End()
                            
                            cxt:Opt("OPT_CONVINCE_PRESSURE")
                                :Dialog("DIALOG_CONVINCE_PRESSURE")
                                :Negotiation{
                                    on_success = function(cxt) 
                                        cxt:Dialog("DIALOG_PROP_NEGOTIATION_SUCCESS")
                                        cxt:Dialog("DIALOG_WORKERS_PRESSURED")
                                        ApplyPressure(cxt)
                                        cxt:Pop()
                                    end,
                                    on_fail = function(cxt) 
                                        cxt:Dialog("DIALOG_PROP_NEGOTIATION_FAIL")
                                    end,
                                }

                            StateGraphUtil.AddBackButton(cxt)

                        end)
                
                end


            elseif cxt:GetAgent():GetRoleAtLocation() == CHARACTER_ROLES.GUARD then
                
                cxt:Opt("OPT_ASK_ABOUT_PAMPHLETEER")
                    :SetQuestMark()
                    :Dialog("DIALOG_ASK_GUARD_ABOUT_PAMPHLETEER")
                
                cxt:Opt("OPT_ASK_FOR_HELP")
                    :SetQuestMark()
                    :Dialog("DIALOG_ASK_GUARD_FOR_HELP")

            elseif cxt:GetAgent():GetRoleAtLocation() == CHARACTER_ROLES.LABOUR then
                
                if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL then
                    
                    cxt:Opt("OPT_ASK_ABOUT_PAMPHLETEER")
                        :SetQuestMark()
                        :Dialog("DIALOG_ASK_LABOUR_ABOUT_PAMPHLETEER_LIKED")
                        :CompleteQuest("find_target")
                elseif cxt.quest.param.got_help_from_boss then
                    
                    cxt:Opt("OPT_ASK_ABOUT_PAMPHLETEER")
                        :SetQuestMark()
                        :Dialog("DIALOG_ASK_LABOUR_ABOUT_PAMPHLETEER_PRESSURED")
                        :CompleteQuest("find_target")
                        :Fn(function() end)
                        :End()
                else    
                    
                    cxt:Opt("OPT_ASK_ABOUT_PAMPHLETEER")
                        :SetQuestMark()
                        :Dialog("DIALOG_ASK_LABOUR_ABOUT_PAMPHLETEER")
                        :LoopingFn(function()

                            cxt:Opt("OPT_BRIBE_LABOUR")
                                :DeliverMoney(PAY_FOR_INFO_COST)
                                :Dialog("DIALOG_BRIBE_LABOUR_ABOUT_PAMPHLETEER")
                                :CompleteQuest("find_target")
                                :End()
                            
                            
                            cxt:Opt("OPT_CONVINCE_INFO")
                                :Dialog("DIALOG_NEGOTIATE_LABOUR_ABOUT_PAMPHLETEER")
                                :Negotiation{
                                    timer_id = "CONVINCE_PAMPHLETEER_QUEST" .. tostring( cxt:GetAgent():GetID() ),
                                    on_success = function(cxt) 
                                        cxt:Dialog("DIALOG_NEGOTIATE_LABOUR_ABOUT_PAMPHLETEER_SUCCESS")
                                        cxt.quest:Complete("find_target")
                                        StateGraphUtil.AddEndOption(cxt)
                                    end,
                                    on_fail = function(cxt) 
                                        cxt:Dialog("DIALOG_NEGOTIATE_LABOUR_ABOUT_PAMPHLETEER_FAIL")
                                    end,
                                }
                            
                            cxt:Opt("OPT_ROUGH_UP_LABOUR")
                                :Dialog("DIALOG_ROUGH_UP_LABOUR")
                                :Fn(function() cxt.quest.param.hurt_a_worker = true end)
                                :Battle{
                                    flags = BATTLE_FLAGS.NO_BYSTANDERS,
                                    on_win = function(cxt) 
                                        if cxt:GetAgent():IsDead() then
                                            if cxt.location:GetProprietor() then
                                                cxt.location:GetProprietor():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("killed_a_worker"))
                                            end
                                            cxt:Dialog("DIALOG_KILLED_LABOUR")
                                            
                                            cxt.quest:Complete("find_target")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        else
                                            if cxt.location:GetProprietor() then
                                                cxt.location:GetProprietor():OpinionEvent( cxt.quest:GetQuestDef():GetOpinionEvent("roughed_up_a_worker"))
                                            end
                                            cxt:Dialog("DIALOG_ROUGHED_UP_LABOUR")
                                            cxt.quest:Complete("find_target")
                                            StateGraphUtil.AddEndOption(cxt)
                                        end
                                    end,

                                }

                            StateGraphUtil.AddBackButton(cxt)

                        end)
                end
            end
        end

    end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                The Rise have been getting aggressive with their recruitment. 
                Only <i>some</i> of the laborers are malcontent, but they work to sow dissent among the others, radicalizing the workforce.
                One pamphleteer has been encouraging the workers to <i>strike</i>. If you can imagine.
            player:
                !dubious
                That must be in breach of contract, no?
            agent:
                !agree
                Oh, indeed. Distributing disruptive material is strictly against the rules, but rules mean nothing to these savages.
                I'd like you to find the pamphleteer—you'll get further than we can, without Spark Baron colors. 
                $neutralDirect
                See to it that they keep their <i>library</i> to themselves, hm?
            *** {giver} wants you to find the Rise pamphleteer that's been buzzing around {worksite} and make them go away.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                Go to the {worksite} and use your select set of skills to find the pamphleteer. 
                But try to avoid antagonizing the laborers too much, hm? That'll just push them straight into the Rise's arms.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)




QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.DECLINED )
    :Loc{
        DIALOG_INTRO = [[
            player:
                I'm not sure this is the best use of my time, if I'm honest. 
            agent:
                !angry
                $angryDefault
                You aren't developing political symphathies on me, are you?
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)

