local QDEF = QuestDef.Define{
    title = "Culling the Herd",
    qtype = QTYPE.CONTRACT,
    faction = "SPARK_BARONS",
    tags = {"nasty"},
    rank = {1, 5},
    desc = [[The Barons want {target} dead, no questions asked.]],
    icon = engine.asset.Texture("icons/quests/baron_kill_a_rise.tex"),
    focus = QUEST_FOCUS.COMBAT,

    on_start = function(quest)
        quest:Activate("kill_target")
    end,

    avoids = 
    {
        {"giver", "target"},
    },
    avoids_fn = function( quest, agent, location )
        if quest:GetCastAgent("target") == agent and location:HasTag("HQ") then
            return true
        end
        return false
    end,

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("target") then
            fighter:AddCondition("WANTED_DEAD")
        end
    end,

}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}


:AddCast{
    cast_id = "target",
    condition = function(agent, quest)
        if (agent:GetFactionID() == "RISE" or (agent:GetContentID() == "LABORER" or agent:GetContentID() == "HEAVY_LABORER" or agent:GetContentID() == "FOREMAN")) and agent:GetSpecies() ~= SPECIES.MECH then
            
            if QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, false) then
                return true
            end

        end
    end,

    events = 
    {
        agent_retired = function( quest, agent ) 
            if quest:IsActive("kill_target") then
                quest:Complete("kill_target")
            end
        end,
    }
}
:AddCastFallback
{
    cast_fn = function(quest, t)
        local defs = {"LABORER", "RISE_PAMPHLETEER", "FOREMAN", "RISE_REBEL", "RISE_RADICAL"}
        local def = defs[math.min( quest:GetRank(), #defs)]
        local agent = quest:CreateSkinnedAgent( def )
        table.insert( t, agent )
    end,
}

:AddObjective{
    id = "kill_target",
    title = [[Kill {target}]],
    desc = "Fight {target}, and make sure they don't survive.",
    mark = { "target" },
    negotiation_targets = {"target"},
    combat_targets = {"target"},

    on_activate = function(quest)
        quest:GetCastMember("target"):GetBrain():MoveToHome()
    end,

    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[Go tell {giver} that {target} has been dealt with.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}


QDEF:AddConvo("kill_target", "target")
    :Loc{
        OPT_CONFRONT = "Confront {agent}",
        DIALOG_CONFRONT = [[
            player:
                The Spark Barons want you dead, and they hired me to do it.
            agent:
                Hesh off!
        ]],
        
        OPT_ATTACK = "Attack {agent}",
        DIALOG_ATTACK = [[
            player:
                Sorry, pal, but the Spark Barons want you dead.
        ]],
        DIALOG_ATTACK_KILLED = [[
            * {target} is dead. 
        ]],
        DIALOG_ATTACK_LIVED = [[
            target:
                !injured
                What do you want with me?!
        ]],
        
        OPT_KILL = "Kill {target}",
        DIALOG_KILL = [[
            target:
                No!
                !exit
            * You finish the job.
        ]],
        
        OPT_MAKE_DISAPPEAR = "Bully {target} in to leaving town",
        DIALOG_DISAPPEAR = [[
            player:
                Here's how it's going to go. You leave town and never come back, and I'll let you live.
        ]],
        DIALOG_DISAPPEAR_AGREE = [[
            agent:
                Whatever you say! I'll do it! You'll never see me again!
                !exit
            * {agent} slinks away.
        ]],
        DIALOG_DISAPPEAR_DISAGREE = [[
            agent:
                No! You can't just push me around like that!
        ]],
        DIALOG_BACK = [[
            player:
                This isn't over!
        ]],

    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_CONFRONT")
            :Dialog("DIALOG_CONFRONT")
            :LoopingFn(function(cxt) 
                cxt:Opt("OPT_MAKE_DISAPPEAR")
                    :Dialog("DIALOG_DISAPPEAR")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        
                        on_success = function(cxt) 
                            cxt:Dialog("DIALOG_DISAPPEAR_AGREE")
                            cxt:GetAgent():Retire()
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                        on_fail = function(cxt) 
                            cxt:Dialog("DIALOG_DISAPPEAR_DISAGREE")
                        end,
                    }

                    cxt:Opt("OPT_ATTACK")
                        :Dialog("DIALOG_ATTACK")
                        :Battle{
                            on_win = function(cxt) 
                                if cxt:GetAgent():IsDead() then
                                    cxt:Dialog("DIALOG_ATTACK_KILLED")                                
                                    StateGraphUtil.AddEndOption(cxt)
                                else
                                    cxt:Opt("OPT_KILL")
                                        :Dialog("DIALOG_KILL")
                                        :Fn(function() 
                                            AgentUtil.CommitMurder(cxt:GetAgent()) 
                                            StateGraphUtil.AddEndOption(cxt)
                                            cxt:GetAgent():Kill() 
                                        end)
                                end
                            end,
                            on_runaway = function(cxt, battle)
                                StateGraphUtil.DoRunAway( cxt, battle, cxt:GetAgent():IsDead() )
                            end,
                        }
                    
                StateGraphUtil.AddBackButton(cxt)
                    :Dialog("DIALOG_BACK")
            end)

    end)



QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_DEAD = [[
            player:
                {target} has been eliminated.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_DEAD")
            :CompleteQuest()
    end)    



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                We need {target} dead.
            player:
                What did this particular sap do to deserve it?
            agent:
                We're paying for results, not questions. 
            player:
                !salute
                Understood.

            *** {agent} wants you to kill {target} for the Barons.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)
