local QDEF = QuestDef.Define{
    title = "Supervised Learning",
    qtype = QTYPE.CONTRACT,
    faction = "SPARK_BARONS",
    rank = {1, 3},
    desc = [[{giver} needs you to teach a new Enforcer how to fight.]],
    icon = engine.asset.Texture("icons/quests/barons_supervised_learning.tex"),
    tags = {"nasty"},
    
    focus = QUEST_FOCUS.COMBAT,

    on_destroy = function(quest) 
        if quest:GetCastMember("robot") and quest:GetCastMember("robot"):GetParty()  then
            quest:GetCastMember("robot"):Dismiss()
        end
    end,

    on_start = function(quest)
        quest:Activate("get_robot")
    end,
}

:AddLocationCast{
    cast_id = "robot_factory",
    cast_fn = function(quest, t)
        table.insert(t,  TheGame:GetGameState():GetLocation("GB_AUTOMECH_FACTORY"))
    end,
}

:AddCast{
    cast_id = "robot",
    no_validation = true,
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "SPARK_BARON_AUTOMECH" ) )
    end,
}

:AddLocationCast{
    cast_id = "proving_ground",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("PROVING_GROUND"))
    end,
}

:AddLocationDefs{
    PROVING_GROUND = {
        name = "Proving Ground",
        map_tags = {"forest"},
        plax = "EXT_Bog_Forest_01",
        indoors = false,
    },
}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}


:AddObjective{
    id = "get_robot",
    title = [[Pick up the Automech]],
    desc = "Take possession of the automech at the automech factory.",
    --combat_targets = {"target"},

    mark = {"robot_factory"},
    on_complete = function(quest) 
        quest:Activate("go_to_proving_ground")
        quest:GetCastMember('robot'):Recruit(PARTY_MEMBER_TYPE.ESCORT)
    end
}

:AddObjective{
    id = "go_to_proving_ground",
    title = [[Take the automech to the {proving_ground}]],
    desc = "Take the automech to its training session.",
    mark = { "proving_ground" },
    on_complete = function(quest) 
       quest:Activate("train_robot")
    end,
}

:AddObjective{
    id = "train_robot",
    title = [[Train the automech how to fight]],
    desc = "Automechs learn by observation. Show yours how to fight.",
    --combat_targets = {"target"},
    mark = { "target" },
    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[The automech is trained. Tell {giver} the job is done.]],
    ready_for_completion = true,
    mark = { "giver" },
}

:AddObjective{
    id = "convince_attacked",
    title = "Cover your tracks with {giver}",
    desc = [[Convince {giver} that the automech was defective, and it went on a rampage.]],
    mark = { "giver" },
}

:AddOpinionEvents{
    took_crappy_robot = 
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Took a difficult automech off their hands"
    },
    fought_to_free_them = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Fought to free them from the Barons"
    },
    defended_them_against_malfunction = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Thinks you saved them from a malfunctioning automech"
    },
    suspects_you_of_sabotage = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Suspects you of sabotage"
    },
    refused_to_attack_rise = 
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Didn't follow their orders to attack Rise prisoners"
    }
  
}




QDEF:AddConvo( "get_robot")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("robot_factory") end)
        :Loc{
            DIALOG_INTRO = [[
                * As you enter the Automech factory, dozens of half-completed automechs turn their sensors toward you.
                * A harried-looking administrator walks over to you.
                agent:
                    !right
                    !point
                    This area's off-limits! What are you doing here?
                player:
                    !left
                    !dubious
                    {giver} sent me. I'm supposed to help train some automechs?
                agent:
                    Oh. Oh! You're the guinea pi- I mean grifter.
                    It's about time you got here.
                player:
                    So what do you need me to do?
                agent:
                    I've got two models ready for the proving ground out in the bog.
                    Just pick one and take them there, the technicians will show you what to do.
            ]],
            

            OPT_LOOK_AT_ROBOT_1 = "Ask about Automech ENF-301a...",
            
            OPT_LOOK_AT_ROBOT_2 = "Ask about Automech ENF-302b...",
            OPT_TAKE_ROBOT = "Take this automech",
            
            DIALOG_LOOK_ROBOT_1 = [[
                player:
                    Tell me about ENF-301a.
                agent:
                    That's a pretty standard model. We just assembled it.
                    I don't foresee any issues with that one. Should be straightforward.
            ]],

            DIALOG_LOOK_ROBOT_2 = [[
                player:
                    What's the story with ENF-302b?
                agent:
                    That one has been to the proving ground like three times now, and each time it actually got <i>worse</> at fighting.
                    I dialed its violence inhibitors way back. It might work? Or it might go amok. Or just shut down again.
                    Honestly, I'd be grateful to get it out of my hands. If it comes back again we're going to have to bust it down for scrap.
            ]],

            DIALOG_TAKE_ROBOT_1 = [[
                player:
                    !point
                    I'll take that one.
                agent:
                    Big surprise.
            ]],
            DIALOG_TAKE_ROBOT_2 = [[
                player:
                    !point
                    I'll take this one.
                agent:
                    !happy
                    Really? You're looking for a challenge, eh?
                    I have to thank you. This thing has been a pain in my side for months.
            ]],
            DIALOG_TAKE_ROBOT_COMMON = [[
                agent:
                    I'll set it to follow you to the proving grounds.
                    Try to be discreet - it won't be any use to you in a fight. 
                robot:
                    !right
                    FOLLOW MODE ON
                * The automech locks sensors on you and starts to follow.
            ]],


        }
        
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.location:GetProprietor())
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:AssignCastMember("robot")
            end

            cxt:AskAbout("STATE_QUESTIONS")


            cxt:Opt("OPT_LOOK_AT_ROBOT_1")
                :Dialog("DIALOG_LOOK_ROBOT_1")
                :IsHubOption(true)
                :LoopingFn(function()  
                    cxt:Opt("OPT_TAKE_ROBOT")
                        :Fn(function() 
                            cxt.quest.param.crappy_robot = false 
                        end)
                        :Dialog("DIALOG_TAKE_ROBOT_1")
                        :Dialog("DIALOG_TAKE_ROBOT_COMMON")
                        :CompleteQuest("get_robot")
                        :Travel()

                    StateGraphUtil.AddBackButton(cxt)
                end)

            cxt:Opt("OPT_LOOK_AT_ROBOT_2")
                :Dialog("DIALOG_LOOK_ROBOT_2")
                :IsHubOption(true)
                :LoopingFn(function()  
                    cxt:Opt("OPT_TAKE_ROBOT")
                        :Fn(function() 
                            cxt.quest.param.crappy_robot = true 
                        end)
                        :ReceiveOpinion("took_crappy_robot")
                        :Dialog("DIALOG_TAKE_ROBOT_2")
                        
                        :Dialog("DIALOG_TAKE_ROBOT_COMMON")
                        :CompleteQuest("get_robot")
                        :Travel()

                    StateGraphUtil.AddBackButton(cxt)
                end)
        end)
    
    :AskAboutHub("STATE_QUESTIONS", 
    {
        "Ask about the training process",
        [[
            player:
                I don't know much about automechs. How am I supposed to train one?
            agent:
                It's easier than you'd expect. 
                When you get to the proving ground, our technician will turn on learning mode.
                Then you just have to show it how to fight. 
                They learn by example.
            player:
                So I need to fight it?
            agent:
                No, of course not.
                That would teach it how to take a beating, which is the opposite of what we want.
                You'll be fighting against some, uh, "training dummies". 
                It'll watch, and hopefully learn.
        ]],
        "Ask about results",
        [[
            player:
                What if the automech doesn't catch on?
            agent:
                It takes a couple of sessions, sometimes. 
                So long as the field technician's happy with your effort, you'll get paid.
        ]],
    })




QDEF:AddConvo( "go_to_proving_ground")
    :TravelConfront("CONFRONT", function(cxt) return not cxt.quest.param.interrupted_by_rise end)
        :Loc{
            DIALOG_INTRO = [[
                * Rise step out of the bushes, brandishing weapons.
                agent:
                    !right
                    Tell that thing to stop walking.
                player:
                    !left
                agent:
                    I have a proposition for you, grifter.
                    You're taking the automech to the Baron proving ground, are you not? 
                    Let me... make a couple of adjustments to it first.
                    Or you could try to stop us. But I wouldn't recommend it.
            ]],

            DIALOG_WON_FIGHT = [[
                right:
                    !exit
                * The fight over, you hustle out of there. {robot} follows, ever obedient.
            ]],

            OPT_ASK_ABOUT_MODIFICATIONS = "Ask about the modifications",
            DIALOG_ASK_ABOUT_MODIFICATIONS = [[
                player:
                    What kind of modifications are you proposing?
                agent:
                    Nothing visible from the outside. Just a little something to clarify its allegiances.
                    Shouldn't affect you, unless you stand between it and a Spark Baron.
            ]],

            OPT_ALLOW_MODIFICATION = "Allow {agent} to modify {robot}",
            DIALOG_ALLOW_MODIFICATION = [[
                player:
                    Alright. Do it.
                agent:
                    Good choice.
                    !exit
                * {agent} opens an access panel on the back of {robot}, and tinkers with the internals for what feels like an eternity.
                agent:
                    !right
                    There. It's done. Now just keep your mouth shut about it, and we'll have no problems.
                * {agent} signals to {agent.hisher} companions, and they melt back into the bog.
                robot:
                    !right
                    INTERNAL MODIFICATION DETECTED
                player:
                    !dubious
                    Let's, uh, keep that between the two of us.
            ]],

            OPT_THREATEN = "Scare {agent} away",
            DIALOG_THREATEN = [[
                player:
                    Oh, I'm not headed to training.
                    !cruel
                    You don't want me to turn this thing on, do you?
            ]],
            DIALOG_FAILURE = [[
                agent:
                    Nice try.
                    My automech-hacking skills are renowned throughout the bog! You think I can't tell that that thing's not active?
                    If it <i>were</> active it would have attacked us already.
                    So step aside, or we flatten you.
            ]],
            DIALOG_SUCCESS= [[
                * {agent} stares at {robot}, weighing your bluff.
                agent:
                    Something doesn't smell right.
                    Let's get moving, Rise.
                    !exit
                * The Rise depart. Your words were weapon enough, this time.
            ]],
            DIALOG_BATTLE = [[
                player:
                    !fight
                    I don't need an automech to deal with the likes of you.
            ]],

            
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.quest.param.interrupted_by_rise = true
                cxt.enc.scratch.rise = CreateCombatParty("RISE_PATROL", math.max(2, cxt.quest:GetRank()), cxt.location)
                cxt.enc:SetPrimaryCast(cxt.enc.scratch.rise[1])
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_ASK_ABOUT_MODIFICATIONS")
                :Dialog("DIALOG_ASK_ABOUT_MODIFICATIONS")

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_BATTLE")
                :Battle{ flags = BATTLE_FLAGS.SELF_DEFENCE }
                    :OnWin()
                        :Dialog("DIALOG_WON_FIGHT")
                        :Travel()

            cxt:Opt("OPT_THREATEN")
                :Dialog("DIALOG_THREATEN")
                :Negotiation{flags = NEGOTIATION_FLAGS.INTIMIDATION}
                    
                    :OnSuccess()
                        :Dialog("DIALOG_SUCCESS")
                        :Travel()
                    
                    :OnFailure()
                        :Dialog("DIALOG_FAILURE")


            cxt:Opt("OPT_ALLOW_MODIFICATION")
                :Dialog("DIALOG_ALLOW_MODIFICATION")
                :Fn(function() cxt.quest.param.robot_modified = true end)
                :Travel()


        end)
    



QDEF:AddConvo("go_to_proving_ground")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("proving_ground") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find the proving ground. Automech parts litter the ground around a technician, who is adjusting some equipment.
                * Some demoralized Rise members mill about in a makeshift jail, guarded by a couple Barons.
                * The technician looks up as you approach.
                player:
                    !left
                tech:
                    !right
                    Have you brought me a new mech?

            ]],
            DIALOG_INTRO_MECH_BAD = [[
                robot:
                    !left
                    LOOP DETECTED
                tech:
                    !facepalm
                    Oh no, not <i>this one</> again.
                player:
                    !left
                tech:
                    I told those pencil pushers back at the factory that this one's untrainable. Why do they keep wasting my time?
                    Let's get it over with, I guess.
            ]],
            DIALOG_INTRO_MECH_GOOD = [[
                robot:
                    !left
                    INITIALIZE
                tech:
                    !happy
                    Nice, a new model. This should work well.
                player:
                    !left
                tech:
                    !neutral
                    Let's get started, then.
            ]],

            DIALOG_FIGHT_INTRO = [[
                tech:
                    See those Rise over there? I'm going to release them, and you're going to fight them.
                    The automech will observe, and if you do your job right it will learn a thing or two.
                    You ready?
                player:
                    !dubious
                    Wait, you want me to attack unarmed captives?
                tech:
                    No! Of course not! 
                    We're going to give them their weapons back first.
                    It has to be a fight, not a massacre, or the automech won't learn anything.
                *** {tech} wants you to attack the Rise prisoners to demonstrate proper fighting technique to {robot}.
            ]],


            OPT_FIGHT_THE_RISE = "Fight the Rise",
            
            OPT_REFUSE = "Refuse to participate",
            DIALOG_REFUSE = [[
                player:
                    This isn't right. I'm not going to attack prisoners for you.
                tech:
                    !sigh
                    Fine.
                    Then the automech will learn by watching <i>them</> attack <i>you</>.
                    !exit
                * {tech} retreats behind a blast shield and presses a button. The bars of the prisoners' box recede, releasing the Rise prisoners.
                * They grab their weapons and charge at you.
                player:
                    !left
                    !fight
                rise:
                    !right
                    !fight
                    Kill the grifter!
            ]],

            DIALOG_FIGHT_THE_RISE = [[
                player:
                    !sigh
                    A job's a job I suppose.
                tech:
                    That's the spirit!

                player:
                    !exit
                tech:
                    Alright, you Rise scum. If you can get through this grifter, you can go free! 
                    !exit
                * {tech} retreats behind a blast shield and presses a button. The bars of the prisoners' box recede, releasing the Rise prisoners.
                * They grab their weapons and charge at you.
                player:
                    !left
                    !fight
                rise:
                    !right
                    !fight
                    Kill the grifter!
            ]],
            OPT_CONVINCE_THE_RISE_TO_ATTACK_THE_BARONS = "Convince the Rise you want to help them",
            DIALOG_CONVINCE_THE_RISE_TO_ATTACK_THE_BARONS = [[
                player:
                    We don't have to do this...
            ]],
            DIALOG_CONVINCE_THE_RISE_TO_ATTACK_THE_BARONS_SUCCESS = [[
                agent:
                    OK, grifter. Get me out of this thing and you've won yourself a friend.

            ]],
            DIALOG_CONVINCE_THE_RISE_TO_ATTACK_THE_BARONS_FAILURE = [[
                agent:
                    I'm not listening to your lies, grifter!
            ]],
            DIALOG_WON_WITH_RISE_SURVIVOR = [[
                right:
                    !exit
                * The Barons are dead. The remaining Rise run away, terrified of being captured again.
                * You trash the still-passive automech, and make the scene look like a training accident.
            ]],
            DIALOG_WON_WITH_RISE_NO_SURVIVOR = [[
                right:
                    !exit
                * The Barons and the Rise are all dead.
                * You trash the still-passive automech, and make the scene look like a training accident.
            ]],
            OPT_ATTACK = "Wipe out the Barons",
        }

        :Fn(function(cxt) 
            cxt.quest:Complete("go_to_proving_ground")
            local goon = AgentUtil.GetFreeAgent("SPARK_BARON_GOON")
            local tech = AgentUtil.GetFreeAgent("SPARK_BARON_TASKMASTER")
            goon:MoveToLocation(cxt.location)
            tech:MoveToLocation(cxt.location)
            cxt:ReassignCastMember("guard", goon)
            cxt:ReassignCastMember("tech", tech)
            cxt:TalkTo(tech)
            

            local rise_defs = 
            {
                {"RISE_REBEL"},
                {"RISE_REBEL", "RISE_REBEL"},
                {"RISE_RADICAL", "RISE_REBEL"},
                {"RISE_RADICAL", "RISE_RADICAL"},
            }

            cxt.enc.scratch.rise = CreateCombatParty(rise_defs[math.min( cxt.quest:GetRank(), #rise_defs)], cxt.quest:GetRank(), cxt.location)
            cxt:ReassignCastMember('rise', cxt.enc.scratch.rise[1])
            cxt:Dialog("DIALOG_INTRO")
            cxt:Dialog(cxt.quest.param.crappy_robot and "DIALOG_INTRO_MECH_BAD" or "DIALOG_INTRO_MECH_GOOD")
            cxt:Dialog("DIALOG_FIGHT_INTRO")


            cxt:Opt("OPT_FIGHT_THE_RISE")
                :Dialog("DIALOG_FIGHT_THE_RISE")
                :Battle{
                    enemies = cxt.enc.scratch.rise,
                    flags = BATTLE_FLAGS.NO_BYSTANDERS,
                    on_win = function()
                        cxt:GoTo("STATE_POST_DEMONSTRATION")
                    end,
                }

            cxt:Opt("OPT_REFUSE")
                :ReceiveOpinion("refused_to_attack_rise")
                :Dialog("DIALOG_REFUSE")
                :Fn(function(cxt) 
                    cxt:TalkTo(cxt.enc.scratch.rise[1])
                    cxt:Opt("OPT_CONVINCE_THE_RISE_TO_ATTACK_THE_BARONS")
                        :Dialog("DIALOG_CONVINCE_THE_RISE_TO_ATTACK_THE_BARONS")
                        :Negotiation{}
                            :OnSuccess()
                                :Dialog("DIALOG_CONVINCE_THE_RISE_TO_ATTACK_THE_BARONS_SUCCESS")
                                :Fn(function() 
                                    cxt:Opt("OPT_ATTACK")
                                        :Battle{
                                            enemies = {tech, goon},
                                            allies = cxt.enc.scratch.rise,
                                            flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.NO_SURRENDER,
                                            on_win = function()
                                                cxt:Dialog(AgentUtil.CountAlive(cxt.enc.scratch.rise) == 0 and "DIALOG_WON_WITH_RISE_NO_SURVIVOR" or "DIALOG_WON_WITH_RISE_SURVIVOR")
                                                for k,v in ipairs(cxt.enc.scratch.rise) do
                                                    if not v:IsDead() then
                                                        v:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("fought_to_free_them"))
                                                    end

                                                end
                                                cxt.quest.param.accident = true
                                                cxt.quest:Complete("train_robot")
                                                StateGraphUtil.AddLeaveLocation(cxt)
                                            end,
                                        }
                                end)
                            :OnFailure()
                                :Dialog("DIALOG_CONVINCE_THE_RISE_TO_ATTACK_THE_BARONS_FAILURE")
                                :Fn(function() 
                                    cxt:Opt("OPT_DEFEND")
                                        :Battle{
                                            enemies = cxt.enc.scratch.rise,
                                            flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.SELF_DEFENCE,
                                            on_win = function()
                                                cxt:GoTo("STATE_POST_DEMONSTRATION")
                                            end,
                                        }
                                end)



                    cxt:Opt("OPT_DEFEND")
                        :Battle{
                            enemies = cxt.enc.scratch.rise,
                            flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.SELF_DEFENCE,
                            on_win = function()
                                cxt:GoTo("STATE_POST_DEMONSTRATION")
                            end,
                        }
                end)
        end)

    :State("STATE_POST_DEMONSTRATION")
        :Loc{

            DIALOG_CRAPPY_MODIFIED_ROBOT_LEARNED = [[
                robot:
                    !right
                player:
                    !left
                * The fight is over. You turn to {robot} as it processes the result.
                robot:
                    TRAINING COMPLETE
                tech:
                    !left
                    Wow, you really taught this rust bucket how to fight!
                robot:
                    TARGET ACQUIRED
                tech:
                    !scared
                    Huh? What?!
                * {robot} powers up its weapons and opens fire on the barons!
            ]], --tech surprised, robot turns on barons
            
            DIALOG_MODIFIED_ROBOT_LEARNED = [[
                robot:
                    !right
                player:
                    !left
                * The fight is over. You turn to {robot} as it processes the result.
                robot:
                    TRAINING COMPLETE
                tech:
                    !left
                    Great. These latest models are pretty reliable.
                robot:
                    TARGET ACQUIRED
                tech:
                    !scared
                    Huh? What?!
                * {robot} powers up its weapons and opens fire on the barons!

            ]], --robot turns on barons

            OPT_DEFEND_BARONS = "Defend the Barons",
            
            DIALOG_DEFENDED_TECH_SURVIVED = [[
                player:
                    !left
                tech:
                    !right
                    !angry
                    What the hell just happened? 
                    I've never seen one of our automechs malfunction that way.
                    Was it acting strangely when you picked it up at the factory?
                    Did anything happen to it on the way here?
                
            ]],

            DIALOG_DEFENDED_TECH_DIED = [[
                right:
                    !exit
                * {tech} is dead, but so is any evidence of your dealings with the rise. You hope you can spin this as an unfortunate accident with {giver}.
            ]],
            

            OPT_HELP_ROBOT = "Help the automech attack the Barons",
            OPT_WATCH = "Watch the fight",
            DIALOG_WATCH = [[
                right:
                    !exit
                left:
                    !exit
                * You watch as the automech tears into the Barons.
            ]],
            DIALOG_WATCH_AUTOMECH_WON = [[
                * It doesn't take long for the Barons to all die. The Automech powers down.
            ]],
            DIALOG_WATCH_BARONS_WON = [[
                * The automech gets in a few good hits, but it is no match for its biological opponents.
                player:
                    !left
                tech:
                    !right
                    What the hell was that?
                    Automechs don't just make decisions on their own. It must have been tampered with!
                    What did you do?!
            ]],
            DIALOG_POST_HELP_ROBOT = [[
                player:
                    !left
                right:
                    !exit
                * The Barons are dead. At least you clean up your messes.
                * You arrange the bodies and debris to make it look like an accident.
            ]],


            DIALOG_CRAPPY_MODIFIED_ROBOT_NOT_LEARNED = [[
                robot:
                    !right
                player:
                    !left
                * The fight is over. You turn to {robot} as it processes the result.
                robot:
                    ...
                    ...
                    ERROR
                tech:
                    !left
                    !facepalm
                    Yeah, this is what happened last time with this thing.
                    No surprises there. Just leave it with me, I'll harvest it for spare parts.
                * You wonder if the modifications that the Rise made to the automech are somehow to blame for its performance.
                * Best leave before {tech} discovers the truth and starts looking to lay blame.
            ]], --robot doesn't do anything, noone is surprised


            DIALOG_MODIFIED_ROBOT_NOT_LEARNED = [[
                robot:
                    !right
                player:
                    !left
                * The fight is over. You turn to {robot} as it processes the result.
                robot:
                    ...
                    ...
                    ERROR
                tech:
                    !left
                    Aw, another bad model? I really have to talk to those lunks over at the factory.
                    Just leave it here. I'll take care of the details.
                * You wonder if the modifications that the Rise made to the automech are somehow to blame for its performance.                    
                * Best leave before {tech} discovers the truth and starts looking to lay blame.
            ]], --robot doesn't do anything. 


            DIALOG_CRAPPY_ROBOT_LEARNED = [[
                robot:
                    !right
                player:
                    !left
                * The fight is over. You turn to {robot} as it processes the result.
                robot:
                    TARGET ACQUIRED
                tech:
                    !left
                    !happy
                    By Hesh's sweaty pustules, that thing finally works!
                    Maybe it just needed the right teacher?
                    Thanks, grifter. I owe you one.
            ]], --tech is surprised, grateful

            DIALOG_ROBOT_LEARNED = [[
                robot:
                    !right
                player:
                    !left
                * The fight is over. You turn to {robot} as it processes the result.
                robot:
                    TARGET ACQUIRED
                tech:
                    !left
                    !happy
                    Another successful training session. 
                    Thanks, grifter.
            ]], -- expected result

            DIALOG_CRAPPY_ROBOT_NOT_LEARNED = [[
                robot:
                    !right
                player:
                    !left
                * The fight is over. You turn to {robot} as it processes the result.
                robot:
                    ...
                tech:
                    !left
                    !facepalm
                    Yeah, this is what happened last time with this thing.
                    No surprises there. Just leave it with me, I'll harvest it for spare parts.
            ]], -- tech says this was expected


            DIALOG_ROBOT_NOT_LEARNED = [[
                robot:
                    !right
                player:
                    !left
                * The fight is over. You turn to {robot} as it processes the result.
                robot:
                    ...
                tech:
                    !left
                    Aw, another bad model? I really have to talk to those lunks over at the factory.
                    Just leave it here. I'll take care of the details.
            ]], -- tech is surprised, but shrugs it off.

            DIALOG_HELP_ROBOT =
            [[
                * You decide it best to try to cover your tracks.
                player:
                    !left
                    !fight

            ]]
        

        }
        :Fn(function(cxt) 
            for k, v in pairs( cxt.enc.scratch.rise ) do
                if not v:IsDead() then
                    v:MoveToLimbo()
                end
            end

            cxt.quest:Complete("train_robot")
            local chance_of_learning = (cxt.quest.param.crappy_robot and .33 or .8)
            
            cxt.quest.param.robot_learned = math.random() <= chance_of_learning
            
            if cxt.quest.param.robot_modified then
                if cxt.quest.param.robot_learned then
                    cxt:Dialog(cxt.quest.param.crappy_robot and "DIALOG_CRAPPY_MODIFIED_ROBOT_LEARNED" or "DIALOG_MODIFIED_ROBOT_LEARNED")
                    
                    cxt:Opt("OPT_DEFEND_BARONS")
                        :Battle{
                            allies = {cxt:GetCastMember("tech"),cxt:GetCastMember("guard")},
                            enemies = {cxt:GetCastMember("robot")},
                            flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.SELF_DEFENCE,
                        }
                            :OnWin()
                                :Fn(function() 
                                    if cxt:GetCastMember("tech"):IsDead() then
                                        cxt:Dialog("DIALOG_DEFENDED_TECH_DIED")
                                        cxt.quest.param.accident = true
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    else
                                        cxt:Dialog("DIALOG_DEFENDED_TECH_SURVIVED")
                                        cxt:GoTo('STATE_ANGRY_POST_FIGHT_TECH')
                                    end
                                end)

                    cxt:Opt("OPT_HELP_ROBOT")
                        :Dialog("DIALOG_HELP_ROBOT")
                        :ReceiveOpinion(OPINION.BETRAYED, nil, "clerk")
                        :Battle{
                            allies = {cxt:GetCastMember("robot")},
                            enemies = {cxt:GetCastMember("tech"),cxt:GetCastMember("guard")},
                            flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.NO_SURRENDER,
                        }
                            :OnWin()
                                :Dialog("DIALOG_POST_HELP_ROBOT")
                                :Fn(function() cxt.quest.param.accident = true end)
                                :Travel()

                    cxt:Opt("OPT_WATCH")
                        :Dialog("DIALOG_WATCH")
                        :Fn(function() 
                            if math.random() > .5 then
                                cxt:Dialog("DIALOG_WATCH_AUTOMECH_WON")
                                cxt:GetCastMember("tech"):Kill()
                                cxt:GetCastMember("guard"):Kill()
                                cxt.quest.param.accident = true
                                StateGraphUtil.AddLeaveLocation(cxt)
                            else
                                cxt:Dialog("DIALOG_WATCH_BARONS_WON")
                                cxt:GoTo('STATE_ANGRY_POST_FIGHT_TECH')
                            end
                        end)
                else
                    cxt:Dialog(cxt.quest.param.crappy_robot and "DIALOG_CRAPPY_MODIFIED_ROBOT_NOT_LEARNED" or "DIALOG_MODIFIED_ROBOT_NOT_LEARNED")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            else
                if cxt.quest.param.robot_learned then
                    cxt:Dialog(cxt.quest.param.crappy_robot and "DIALOG_CRAPPY_ROBOT_LEARNED" or "DIALOG_ROBOT_LEARNED")
                else
                    cxt:Dialog(cxt.quest.param.crappy_robot and "DIALOG_CRAPPY_ROBOT_NOT_LEARNED" or "DIALOG_ROBOT_NOT_LEARNED")
                end
                StateGraphUtil.AddLeaveLocation(cxt)
            end  
        end)

    :State("STATE_ANGRY_POST_FIGHT_TECH")
        :Loc{
            OPT_LIE_ABOUT_RISE = "Lie to {agent}",
            DIALOG_LIE_ABOUT_RISE = [[
                player:
                    I didn't notice anything untoward...
            ]],
            DIALOG_LIE_ABOUT_RISE_SUCCESS = [[
                tech:
                    !neutral
                    !shrug
                    I guess these things just happen sometimes.
                    !happy
                    Thanks for the protection. You're not bad, for a grifter.
                    !exit
                * {tech} leaves you to pick through the debris field.
            ]],
            DIALOG_LIE_ABOUT_RISE_FAILURE = [[
                tech:
                    !angry
                    Something doesn't add up here. 
                    I can't prove it, but I swear you're up to something.
                * {tech} storms off to pick through the debris field for evidence.
            ]],
            OPT_WALK_AWAY = "Walk away",
            DIALOG_WALK_AWAY = [[
                player:
                    I don't have to stand for your insinuations.
                    !exit
                tech:
                    If I find proof, you're done with, grifter!
                * You leave, a hailstorm of invective pelting you in the back.
            ]],
            OPT_MAKE_UP_EXCUSES = "Make up an excuse",

        }
        :Fn(function(cxt) 
            cxt:TalkTo(cxt:GetCastMember("tech"))
            cxt.quest.param.accident_alive = true

            cxt:Opt("OPT_MAKE_UP_EXCUSES")
                :Dialog("DIALOG_LIE_ABOUT_RISE")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_LIE_ABOUT_RISE_SUCCESS")
                        :ReceiveOpinion("defended_them_against_malfunction")
                        :Travel()
                    :OnFailure()
                        :Dialog("DIALOG_LIE_ABOUT_RISE_FAILURE")
                        :ReceiveOpinion("suspects_you_of_sabotage")
                        :Travel()

            cxt:Opt("OPT_WALK_AWAY")
                :Dialog("DIALOG_WALK_AWAY")
                :ReceiveOpinion("suspects_you_of_sabotage")
                :Travel()

        end)


QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Report on the training exercise",
    }
    :Loc{
        DIALOG_ACCIDENT_ALIVE = [[
            player:
                The automech malfunctioned and attacked. It was put down before it could cause too much damage.
            agent:
                Not <i>again</>.
                I'm starting to wonder about the workers at the factory. I'm going to have to order some interrogations to root out Rise sympathizers.
                Not <i>your</> fault, though. Here's your pay.
                !give
        ]],
        DIALOG_ACCIDENT = [[
            player:
                There was a terrible accident at the proving grounds. Everyone is dead.
            agent:
                Eh. It happens.
                I'll dock the factory supervisor's pay for the cleanup.
                But what else are we going to do? Stop using automechs? Ha!
                Here's your pay.
                !give
        ]],
        DIALOG_TRAINED = [[
            player:
                The automech is battle-ready.
            agent:
                Good job, {player}. That's one less person I need to keep on payroll.
        ]],
        DIALOG_UNTRAINED = [[
            player:
                The training didn't stick. 
            agent:
                Hesh damn it. I'm going to have to come down hard on the factory workers.
                !give
                Here's your pay. This is really starting to affect my bottom line.
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Fn(function() 
                if cxt.quest.param.accident_alive then
                    cxt:Dialog("DIALOG_ACCIDENT_ALIVE")
                elseif cxt.quest.param.accident then
                    cxt:Dialog("DIALOG_ACCIDENT")
                elseif cxt.quest.param.robot_learned then
                    cxt:Dialog("DIALOG_TRAINED")
                else
                    cxt:Dialog("DIALOG_UNTRAINED")
                end
            end)
            :CompleteQuest()
    end)    



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                Automechs are cheap to use once they're built, but they are damned expensive to build.
                Each one has to be meticulously pieced together from salvaged tech from the bog, and then programmed to follow commands.
                The techs in the field tell me that they've got a plan to make that second part quicker and easier.
                But, they need someone with some fighting expertise.

            *** {giver} wants you to help train some automechs. There has to be a catch.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                Go pick up an automech at the factory, and take it to the proving grounds out in the Bog.
                The technician there will tell you what to do next.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)

