local fun = require "util/fun"

local QDEF = QuestDef.Define{
    title = "The Best Defense",
    qtype = QTYPE.CONTRACT,
    faction = "SPARK_BARONS",
    tags = {"late_game"},
    rank = {1, 5},
    desc = [[Rescue a Baron from a Rise 'trial'.]],
    
    icon = engine.asset.Texture("icons/quests/baron_the_best_defense.tex"),
    focus = QUEST_FOCUS.NEGOTIATION,

    on_init = function(quest)
    end,

    on_start = function(quest)
        quest:Activate("confront_rise_target")
        quest.param.first_interrupt_type = quest.param.first_interrupt_type or table.arraypick{"BARONS", "MONSTERS"}
        if quest.param.set_rise_target_relationship == "FRIEND" then
            quest:GetCastMember("rise_target").social_connections:SetRelationship(RELATIONSHIP.LIKED, TheGame:GetGameState():GetPlayerAgent())
        elseif quest.param.set_rise_target_relationship == "ENEMY" then
            quest:GetCastMember("rise_target").social_connections:SetRelationship(RELATIONSHIP.DISLIKED, TheGame:GetGameState():GetPlayerAgent())
        elseif quest.param.set_rise_target_relationship == "NEUTRAL" then
            quest:GetCastMember("rise_target").social_connections:SetRelationship(RELATIONSHIP.NEUTRAL, TheGame:GetGameState():GetPlayerAgent())
        end
    end,

    on_destroy = function(quest)

        if quest:GetCastMember("rise_target") and quest:GetCastMember("rise_target"):IsInPlayerParty() then
            quest:GetCastMember("rise_target"):Dismiss()
        end

        if quest:GetCastMember("defendant") and quest:GetCastMember("defendant"):IsInPlayerParty() then
            quest:GetCastMember("defendant"):Dismiss()
        end

    end,

    collect_agent_locations = function(quest, t)
        if quest:IsActive("confront_rise_target")then
            table.insert (t, { agent = quest:GetCastMember("rise_target"), location =  TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"), role = CHARACTER_ROLES.PATRON} )
        end
    end,

    is_in_hiding = function(quest, agent)
        return agent == quest:GetCastMember("defendant")
    end,
}

:AddLocationDefs{
    RISE_ENCAMPMENT_EXT = {
        name = "Rise Encampment",
        desc = "A hastily-constructed base of operations for the Rise.",
        plax = "EXT_Rise_Encampment_01",
        map_tags = {"bog"},
        indoors = false,
    },
    
    RISE_ENCAMPMENT_INT = {
        name = "Rise Encampment",
        desc = "A hastily-constructed base of operations for the Rise.",
        plax = "INT_Rise_Encampment_1",
        indoors = true,
    },
}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddCast{
    cast_id = "defendant",
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "FOREMAN" ) )
    end,
}

:AddCast{
    cast_id = "rise_target",
    condition = function(agent, quest) 
        return agent:GetFactionID() == "RISE"
    end,

    score_fn = function(agent, quest)
        if agent:GetRelationship() ~= RELATIONSHIP.NEUTRAL then
            return 10
        end
        return 1
    end,
}:AddCastFallback{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "RISE_REBEL" ) )
    end,
}

:AddLocationCast{
    cast_id = "encampment_ext",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("RISE_ENCAMPMENT_EXT"))
    end,
}
:AddLocationCast{
    cast_id = "encampment_int",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("RISE_ENCAMPMENT_INT"))
    end,
}


:AddObjective{
    id = "confront_rise_target",
    title = [[Confront {rise_target}]],
    desc = "{rise_target} was involved in the kidnapping of {defendant}. {rise_target.HeShe} probably knows where {defendant} was taken.",
    mark = {"rise_target"},
}

:AddObjective{
    id = "infiltrate_the_encampment",
    title = "Infiltrate the Encampment",
    desc = "Get access to the Rise Encampment, and find {defendant}.",
    mark = {"encampment_ext"},
    on_complete = function(quest) 
    end
}

:AddObjective{
    id = "escort_defendant",
    title = "Bring {defendant} to {giver}",
    desc = [[You've rescued {defendant}, now take {defendant.himher} to safety and collect your pay.]],
    mark = { "giver" },
    terminal = true,
}

:AddObjective{
    id = "explain_self_to_giver",
    title = "Tell {giver} what happened",
    desc = [[You've messed up the rescue. Go tell {giver}.]],
    mark = { "giver" },
    terminal = true,
}


:AddOpinionEvents{
    gave_them_up = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Turned them over to the Barons",
    },
    
    defended_them_from_barons = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Defended them from the Barons",
    },

    let_them_get_beaten = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Let them get beaten",
    },
    
    saved_from_beating = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Saved them from a beating",
    },
    
    defended_them_from_foreman = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Defended them from their boss",
    },

    rescued_them = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Rescued them from the Rise",
    },
    lost_their_job = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Made them lose their job",
    },
}

----------QUEST INTRODUCTION

QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                The Rise kidnapped one of our foremen, and is keeping {defendant.himher} locked up.
            player:
                $neutralThoughtful
                I see. Is this a ransom situation?
            agent:
                Oh! Possibly. But they've taken to holding mock trials, and tally up the alleged crimes.
            player:
                !dubious
                What sort of crimes?
            agent:
                !handwave
                Oh, rights violations. Slavery. Some trumped up charge or another. 
                All poppycock, of course. They signed waivers. 
                Anyway, they'll undoubtedly find {defendant} guilty and execute {defendant.himher} while they're at it.
                !shrug
                And the problem with foremen is that, well, I'll owe a pay out to {defendant.hisher} next of kin.
                I'd prefer that wasn't on the fiscal report for this quarter. You understand.
        ]] 
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                We posted a reward of extra rations and an informant came forward. 
                {rise_target} apparently did the foreman-napping, so that should be your first stop.
        ]] 
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)

----------FIND THE RISE_TARGET
-- You need to convince this person to tell you where defendant is being held.
-- We're going to try to cast a friend - if so, you can ask them to come with to vouch for you, if you want to act as defendant's lawyer.
-- if there are no reasonable friends to cast, it's just some rando. In which case, you need to threaten/beat them to find out where to go next. You can still pull the lawyer trick at the destination, but it will be harder.


QDEF:AddConvo("confront_rise_target", "rise_target")
    :Loc{
        OPT_ASK_ABOUT_DEFENDANT = "Ask {agent} about {defendant}",
        DIALOG_ASK_ABOUT_DEFENDANT = [[
            {not has_talked_to_rise_target?
                player:
                    !interest
                    You're {agent}, yes? I understand you had a part in {defendant}'s, ah, <i>arrest</i>.
                agent:
                    Huh. You've got a grievance, have you? 
            } 
            
            {has_talked_to_rise_target?
                player:
                    So, about {defendant}.
                agent:
                    You got more to say, I guess.
            }

        ]],

        OPT_ASK_ABOUT_WHAT_DEFENDANT_DID = "Ask what {defendant} did",
        DIALOG_ASK_ABOUT_WHAT_DEFENDANT_DID = [[
            player:
                What are {defendant}'s crimes?
            agent:
                !eureka
                $angryDefault
                Working folks to death, for a start. 
                Betraying the union. Spying. Accepting bribes from the Barons. 
                A whole worksite died in the Bog because {defendant} didn't wanna slow down production. 
        ]],


        OPT_CONVINCE_NEED_FAIR_TRIAL = "Convince {agent} that {defendant} deserves a fair trial",
        DIALOG_CONVINCE_NEED_FAIR_TRIAL = [[
            player:
                I see. And who is serving as {defendant.hisher} defense?
            agent:
                !angry
                Nobody! That scum doesn't deserve it.
            player:
                !dubious
                I thought the Rise was on the quest for justice. 
                $neutralDubious
                Seems hardly fair to hold a trial with a predetermined outcome.
        ]],
        DIALOG_CONVINCE_NEED_FAIR_TRIAL_SUCCESS = [[
            player:
                If you truly wish to stand above the Barons, you at least owe {defendant} a fair trial.
            agent:
                !disappoint
                I... yeah. I guess so.
                I just get so mad, you know? We all are. 
                Ain't nobody who'd defend {defendant} whole-heartedly.
            player:
                Perhaps I can be of assistance in that regard, if you'll just tell me where the trial is being held.
            agent:
            agent:
                {liked?
                    Sure, {player}. If you're willing. 
                    I'll mark the route out on your map, but...
                    !thought
                    You know what? I think I wanna see this. I'll come along.
                }
                {not liked?
                    As long as that's all you want, I guess I can help.
                    !overthere
                    Gimme your map. I'll mark it out for you.
                }
        ]],
        DIALOG_CONVINCE_NEED_FAIR_TRIAL_FAILURE = [[
            player:
                !crossed
                Don't you believe in justice?
            agent:
                Sure, when the scales are workin' right.
                !shrug
                But we're at war, here, whether the Barons like it or not.
                Ain't nothing else to do with the people in power 'cept cut them off at the head. 
        ]],

        OPT_INTIMIDATE_TO_GAIN_ACCESS = "Intimidate {agent} to get the location of the trial",
        DIALOG_INTIMIDATE_TO_GAIN_ACCESS = [[
            player:
                Ah, well, I happen to be very concerned with justice.  
                You know the location of the trial, and I don't. It's terribly unfair.
                So I'm forced to take matters into my own hands.
                !throatcut
                I suspect if I were to break your femur, the injustice would right itself.
        ]],
        DIALOG_INTIMIDATE_TO_GAIN_ACCESS_SUCCESS = [[
            player:
                !cruel
                Well? I'll be sporting and let you choose the leg.
            agent:
                H-hey, none of that! I'll tell you where the damn trial is.
            * {agent} gives you detailed directions to the mock trial location.
            agent:
                Good luck convincing anyone to hand {defendant} over, though.
                Now leave me alone, before anyone gets the idea I keep unsavory company.
                Alright?
        ]],
        DIALOG_INTIMIDATE_TO_GAIN_ACCESS_FAILURE = [[
            player:
                !interest
                Now be a good sport and just show me your leg, there...
            agent:
                !angry
                Sure, I'll show you with a knee to the face, how 'bout that?
            player:
                $neutralTakenAback
                That's not exactly what I had in mind.
        ]],

        OPT_BEAT_UP = "Beat up {agent}",
        DIALOG_BEAT_UP = [[
            player:
                Very well. Let's just get the unpleasantries out of the way, shall we?
                !angry_accuse
                The gauntlet is thrown. Fight me, if that's your wish.
        ]],
        DIALOG_AFTER_BEAT_UP = [[
            {dead?
                * {agent} has gone cold, and so has your lead. Nothing for it but to report your failure to {giver}.
            }
            {not dead?
                agent:
                    !injured
                    Stop, stop, already! You wanna risk your neck for {defendant}, I ain't gonna stop you!
                * {agent} spits a tooth into the dirt and scratches a map through the blood.
                agent:
                    $scaredStammering
                    J-just leave me out of it.                   
            }
        ]],

        OPT_TRY_LATER = "Leave for now",
        DIALOG_TRY_LATER = [[
            player:
                I trust you'll still be here enjoying the atmosphere when I come back later, yes?
            agent:
                Sure. Whatever.
        ]],
    }
    :Hub(function(cxt) 
        

        

        cxt:Opt("OPT_ASK_ABOUT_DEFENDANT")
            :SetQuestMark()
            :Dialog("DIALOG_ASK_ABOUT_DEFENDANT")
            :Fn(function() 
                cxt.quest.param.has_talked_to_rise_target = true
            end)
            :LoopingFn(function() 
                    cxt:Question("OPT_ASK_ABOUT_WHAT_DEFENDANT_DID", "DIALOG_ASK_ABOUT_WHAT_DEFENDANT_DID")
                    cxt:Opt("OPT_CONVINCE_NEED_FAIR_TRIAL")
                        :Dialog("DIALOG_CONVINCE_NEED_FAIR_TRIAL")
                        :Negotiation{}
                            :OnSuccess()
                                :Dialog("DIALOG_CONVINCE_NEED_FAIR_TRIAL_SUCCESS")
                                :Fn(function() 
                                    cxt.quest:Activate("infiltrate_the_encampment")
                                    cxt.quest:Complete("confront_rise_target") 
                                    
                                    if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL then
                                        cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.ESCORT )
                                    end
                                end)
                                :ActivateQuest("infiltrate_the_encampment")
                                :End()

                            :OnFailure()
                                :Dialog("DIALOG_CONVINCE_NEED_FAIR_TRIAL_FAILURE")

                    cxt:Opt("OPT_INTIMIDATE_TO_GAIN_ACCESS")
                        :Dialog("DIALOG_INTIMIDATE_TO_GAIN_ACCESS")
                        :Negotiation{
                            flags = NEGOTIATION_FLAGS.INTIMIDATION,
                            }
                            :OnSuccess()
                                :Dialog("DIALOG_INTIMIDATE_TO_GAIN_ACCESS_SUCCESS")
                                :ActivateQuest("infiltrate_the_encampment")
                                :CompleteQuest("confront_rise_target") 
                                :End()

                            :OnFailure()
                                :Dialog("DIALOG_INTIMIDATE_TO_GAIN_ACCESS_FAILURE")

                    cxt:Opt("OPT_BEAT_UP")
                        :Dialog("DIALOG_BEAT_UP")
                        :Battle{}
                            :OnWin()
                                :Fn(function() 
                                    cxt.quest:Complete("confront_rise_target") 

                                    cxt:Dialog("DIALOG_AFTER_BEAT_UP")
                                    if cxt:GetAgent():IsDead() then
                                        cxt.quest:Activate("explain_self_to_giver")
                                        StateGraphUtil.AddEndOption(cxt)
                                    else
                                        cxt.quest:Activate("infiltrate_the_encampment")
                                        StateGraphUtil.AddEndOption(cxt)
                                    end
                                end)

                    cxt:Opt("OPT_TRY_LATER")
                        :Dialog("DIALOG_TRY_LATER")
                        :End()

                end)
    end)

----------INTERRUPTED ON THE WAY
-- If your friend is coming with you to vouch, you might get accosted by a baron patrol or monsters.

QDEF:AddConvo("infiltrate_the_encampment")
    :TravelConfront("INTERRUPT", function(cxt) return not cxt.quest.param.going_there_interrupt and cxt.quest:GetCastMember("rise_target"):IsInPlayerParty() end)
    :Loc{
        DIALOG_INTRO_BARONS = [[
            * You don't make it far before a Baron patrol roots you out, no doubt drawn by the earthy aroma of your Rise escort.
            player:
                !left
            agent:
                !right
                Hold up here, grifter. This worker is wanted for truancy.
        ]],
        OPT_GIVE_UP_FRIEND = "Hand over {rise_target}",
        DIALOG_GIVE_UP_FRIEND = [[
            player:
                !surprise
                <i>What?!</i> I can't believe I find myself in such unsavory company. 
            rise_target:    
                !right
                W-what—hey, wait a second!
            agent:
                !right
                We'll take it from here. On your way now, grifter.
            player:
                !wave 
                Safe travels!
        ]],
        OPT_CONVINCE_LOOK_OTHER_WAY = "Convince {agent} that {rise_target} is your prisoner",
        DIALOG_CONVINCE_LOOK_OTHER_WAY = [[
            player:
                $miscPersuasive
                Ah, a misunderstanding, I'm sure. 
                {rise_target} only missed shift, you see, because I'd requisitioned {rise_target.himher}.
            agent:
                Requisitioned?
            player:
                Indeed. Arint herself sent me on a secret mission, and {rise_target} is crucial to its success.
        ]],
        DIALOG_CONVINCE_LOOK_OTHER_WAY_SUCCESS = [[
            player:
                $miscPersuasive
                Of course, if you'd prefer to face Arint's wrath...
            agent:
                Let's not be hasty! 
                {rise_target} will still be a truant when you're done with {rise_target.himher}. 
                And then, we'll be back.
                !exit
            * {rise_target} waits for {agent} to leave before turning to you.
            rise_target:
                !right
                That was some smooth talking.
                !bashful
                And, uh, that's all it was, right? You ain't actually workin' for Arint, are you?
            player:
                Of course not. You have my word.
            rise_target:
                !eureka
                Good enough for me!

        ]],
        DIALOG_CONVINCE_LOOK_OTHER_WAY_FAILURE = [[
            player:
                I'm afraid you'll have to walk away empty-handed. 
            agent:
                !crossed
                $neutralDirect
                Is that so? Forgive me if I don't take the word of a grifter. 
                My orders are clear, and I plan on following them.
        ]],

        OPT_FIGHT_TO_DEFEND = "Defend {rise_target}",
        DIALOG_FIGHT_TO_DEFEND = [[
            player:
                !shrug
                Ah, well, I suppose this couldn't be <i>too</i> easy.
                It's nothing personal, chaps. 
                !fight 
                Remember that when I knock you down. 
        ]],
        DIALOG_WON_FIGHT = [[
            {rise_target_dead?
                {dead?
                    * Seems you're the only one left standing. Well, at least you have your map.
                }

                {not dead?
                    agent:
                        !injured
                        Enough! If you're really on the side of the Barons, you'll let me live. 
                        Sod off and do whatever it is you have planned. I don't want no more of this. 
                        !exit
                }
            }

            {not rise_target_dead?
                rise_target:
                    !right
                    Watching you fight makes me glad you're on my side. 
                    I owe you one. Don't let me forget it.
                player:
                    Ah, my good friend, I have a mind like a thorn. 
                    I certainly won't. 
            }
        ]],

        DIALOG_INTRO_MONSTERS = [[
        * You hear the sound of mucus hitting the turf before you see them.
            player:
                !left
                !angry
            agent:
                !right
            player:
                Ah.

        ]],
        
        OPT_DEFEND_MONSTERS = "Defend yourself",
        DIALOG_DEFEND_MONSTERS = [[
            player:
                !fight
                Wooooo!
        ]],
        DIALOG_WON_FIGHT_MONSTERS = [[
            {rise_target_dead?
                * When the dust settles, you notice you stand alone. 
                * You kick {rise_target}'s corpse, but it doesn't get up.
            }
            {not rise_target_dead?
                rise_target:
                    !right
                    You see what we put up with in the field? Monsters like that.
                    !spit
                    And not a Baron patrol in sight. Typical!
                    Let's keep going.
                    !exit
            }
        ]],
    }
    
    :Fn(function(cxt) 

        cxt.quest.param.going_there_interrupt = true

        if cxt.quest.param.first_interrupt_type == "BARONS" then
            
            local team =  CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location, true)
            cxt:TalkTo(team[1])

            cxt:Dialog("DIALOG_INTRO_BARONS")
            cxt:RunLoop(function() 
                cxt:Opt("OPT_GIVE_UP_FRIEND")
                    :Dialog("DIALOG_GIVE_UP_FRIEND")
                    :ReceiveOpinion("gave_them_up", nil, "rise_target")
                    :Fn(function() cxt.quest:GetCastMember("rise_target"):Dismiss() end)
                    :Travel()

                cxt:Opt("OPT_CONVINCE_LOOK_OTHER_WAY")
                    :Dialog("DIALOG_CONVINCE_LOOK_OTHER_WAY")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_LOOK_OTHER_WAY_SUCCESS")
                            :Travel()
                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_LOOK_OTHER_WAY_FAILURE")

                cxt:Opt("OPT_FIGHT_TO_DEFEND")
                    :Dialog("DIALOG_FIGHT_TO_DEFEND")
                    :ReceiveOpinion("defended_them_from_barons", nil, "rise_target")
                    :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            allies = {cxt.quest:GetCastMember("rise_target")}
                        }
                        :OnWin()
                            :Fn(function() 
                                    cxt.enc.scratch.rise_target_dead = cxt.quest:GetCastMember("rise_target"):IsDead() 
                                end)
                            :Dialog("DIALOG_WON_FIGHT")
                            :Travel()

            end)

        elseif cxt.quest.param.first_interrupt_type == "MONSTERS" then
            
            local team =  CreateCombatParty("BOG_MONSTERS", cxt.quest:GetRank(), cxt.location, true)
            cxt:TalkTo(team[1])

            cxt:Dialog("DIALOG_INTRO_MONSTERS")
            
            cxt:Opt("OPT_DEFEND_MONSTERS")
                :Dialog("DIALOG_DEFEND_MONSTERS")
                :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        allies = {cxt.quest:GetCastMember("rise_target")}
                    }
                    :OnWin()
                        :Fn(function() 
                                cxt.enc.scratch.rise_target_dead = cxt.quest:GetCastMember("rise_target"):IsDead() 
                            end)
                        :Dialog("DIALOG_WON_FIGHT_MONSTERS")
                        :Travel()

        end
    end)


----------ENCAMPMENT EXTERIOR
-- You find the camp, and need to get access. You can fight a bunch of rise, or convince them to let you in to participate in the trial.
QDEF:AddConvo("infiltrate_the_encampment")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("encampment_ext") end)
        :Loc{
                DIALOG_INTRO_WITH_FRIEND = [[
                    * {rise_target} takes the lead as you approach the encampment. 
                    agent:
                        !right
                    rise_target:
                        !left
                        We didn't miss the trial, did we?
                    agent:
                        !dubious
                        Who's "we"? 
                    rise_target:
                        !interest
                        This is {player}. He's here to represent, uh... justice, I guess?
                    agent:
                        Really. This should be good. 
                    {liked?
                    rise_target:
                        I think it's the right idea. We wanna stand for something, you know?
                }
                {not liked?
                    $neutralJoke
                    Yeah, well, it's not my funeral. 
                    !chuckle
                    I say let him embarrass himself.
                }
                    player:
                        !left
                        I'd like to meet the accused before the trial starts, if I may?
                    * With a shrug, the guards part and let you through. 

                {liked?
                    * {rise_target} wishes you luck, and sounds like {rise_target.heshe} actually means it.
                }
                {not liked?
                    * As you leave, you can hear {rise_target} and the guards laughing cruelly, no doubt at your expense.
                }

                ]],
                DIALOG_INTRO_ALONE = [[
                    * {rise_target} wasn't the best artist, but you eventually find the camp. 
                    player:
                        !left
                    agent:
                        !right
                        That's far enough, grifter.
                        I don't know how you found this place, but it's not for you.
                ]],
                OPT_CONVINCE_NEED_TRIAL = "Convince {agent} that you're here for the trial",
                DIALOG_CONVINCE_NEED_TRIAL = [[
                    player:
                        !wave
                        $neutralGreeting
                        On the contrary. I'm a public defender.
                    agent:
                        !dubious
                        $neutralDubious
                        A what-a-whatter?
                    player:
                        If your movement is ever going to get any traction, it needs outside support to put pressure on the Barons.
                        !hips
                        An unjust trial is bad for your image, and won't win over any sympathizers. 
                        With my help you can at least give the semblance of a fair order.
                ]],
                DIALOG_CONVINCE_NEED_TRIAL_SUCCESS = [[
                    player:
                        $miscPersuasive
                        Just be grateful I'm willing to take this case pro-bono. My time is very valuable, you see.
                    agent:
                        !crossed
                        Does that mean you're <i>good</i>?
                    player:
                        !placate
                        No! On the contrary. I'm a terrible lawyer. 
                        But it will look impressive all the same.
                    agent:
                        !cruel
                        Heh, sure. As long as {defendant} still loses, I don't see the harm in letting you vent some swamp gas. 
                    * {agent} lets you inside with a mock bow. 
                ]],
                DIALOG_CONVINCE_NEED_TRIAL_FAILURE = [[
                    player:
                        !interest
                        Besides, perhaps {defendant} <i>is</i> truly innocent. Wouldn't you want to know?
                    agent:
                        !angry
                        I already <i>do</i> know. And I've got the scars to show it.
                        Sounds to me like you got too much investment in this. 
                        !angry_shrug
                        So no, I don't think I'll let you in. 
                ]],

                OPT_ATTACK = "Fight your way inside",
                DIALOG_ATTACK = [[
                    player:
                        !fight
                        I'm afraid I'll have to do what I must. 
                        But it's all for the pursuit of justice, you see.
                ]],
                DIALOG_WON_FIGHT = [[
                    {dead?
                        * You gingerly kick {agent}'s body out of your way before progressing into the encampment. 
                                            }
                    {not dead?
                        player:
                            Well, villain? Would you truly proceed without a fair trial?
                        agent:
                            !injured
                            Hah! As if that's what this is about.
                            But go on in, stranger. Fight the inevitable, if you can.
                            Just don't be surprised if we line you up right alongside {defendant} when the time comes. 
                            !exit
                        * You step inside.
                    }
                ]],
            }
        :Fn(function(cxt) 

            local team =  CreateCombatParty("RISE_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(team[1])

            
            if cxt.quest:GetCastMember("rise_target"):IsInPlayerParty() then
                cxt:Dialog("DIALOG_INTRO_WITH_FRIEND")
                cxt.quest:GetCastMember("rise_target"):Dismiss()
                cxt:GoTo("STATE_DO_TRIAL")
            else
                cxt:Dialog("DIALOG_INTRO_ALONE")

                cxt:RunLoop(function()
                        cxt:Opt("OPT_CONVINCE_NEED_TRIAL")
                            :Dialog("DIALOG_CONVINCE_NEED_TRIAL")
                            :Negotiation{}
                                :OnSuccess()
                                    :Dialog("DIALOG_CONVINCE_NEED_TRIAL_SUCCESS")
                                    :GoTo("STATE_DO_TRIAL")
                                :OnFailure()
                                    :Dialog("DIALOG_CONVINCE_NEED_TRIAL_FAILURE")
                        
                        cxt:Opt("OPT_ATTACK")
                            :Dialog("DIALOG_ATTACK")
                            :Battle{}
                                :OnWin()
                                    :Dialog("DIALOG_WON_FIGHT")
                                    :GoTo("STATE_FIGHT_WAY_IN")                                    

                    end)
            end

        end)


----------ENCAMPMENT INTERIOR A
-- if you are here as a lawyer, you do a special negotiation. Win and you are allowed to leave with the defendant. Lose, and you need to fight.

    :State("STATE_DO_TRIAL")
        :Loc{
            DIALOG_INTRO = [[
                * Just in the nick of time, too—you enter just as the "judge" is issuing {agent.hisher} final verdict.
                defendant:
                    !left
                agent:
                    !right
                    !angry
                    Well, {defendant}? I think we all know what comes next. But for the sake of fairness...
                    Is there anything you could possibly say in your own defense?
                defendant:
                    !scared_shrug
                    I don't—
                player:
                    !left
                    !point
                    Objection!
                agent:
                    !dubious
                    What in the deep blooming sea...?
            ]],

            OPT_CONVINCE_MERCY = "Convince {agent} to let {defendant} live",
            DIALOG_CONVINCE_MERCY = [[
                player:
                    Your honor, may it please you to consider that a truly just movement leads with mercy?
                agent:
                    !angry
                    That sounds like a lot of words meant to prevent satisfaction.
                player:
                    Sit with me, sirrah, if you'll allow me to at least give some closing remarks.
                    
            ]],
            DIALOG_CONVINCE_MERCY_SUCCESS = [[
                player:
                    !hips
                    Now I may just be a simple swampland lawyer... 
                    But it seems to me that {defendant} is a victim of the same oppressive system as the rest of you!
                    Certainly, {defendant.heshe} enjoys privileges as a foreman. But has {defendant} not been indentured since childhood?
                    Has {defendant.heshe} not been shaped like bogmud into a mere homunculus of the Barons' will?
                agent:
                    ...
                    And what do <i>you</i> have to say to that assertion, {defendant}?
                defendant:
                    !left
                    !scared_shrug
                    I, uh... ditto?
                agent:
                    !palm 
                    Forget it. You're not even worth making an example of. 
                player:
                    !left
                agent:
                    Fine, grifter, you can take the foreman. No doubt {defendant.heshe}'ll slide right back under the Barons' heel. 
                    But I won't strike down a fellow laborer.
                * You bow deeply to the judge and gathered jury before escorting {defendant} out of the encampment. 
            ]],
            DIALOG_CONVINCE_MERCY_FAILURE = [[
                player:
                    Now, ah, you'll find that the, uh...
                    !thought
                    The quality of mercy is not strained... it droppeth as—
                    !eureka
                    Blast. I do always forget how that is meant to go.
                    That old Deltrean doesn't quite sit comfortably on the tongue, eh?
                agent:
                    !angry_shrug
                    Well, thank you for that... utter waste of everyone's time.
                defendant:
                    !left
                    !scared
                agent:
                    Interruptions aside—{defendant}, do you have any final words before we execute our verdict?
                    And by "verdict" I mean "you".
            ]],
            OPT_DEFEND = "Defend {defendant}",
            DIALOG_DEFEND = [[
                player:
                    !left
                    OBJECTION, I say!
                agent:
                    Oh, for the love of—
                    What <i>now</i>?
                player:
                    I'm afraid since your ears are deafened to reason...
                    I'll simply have to box them, instead.
                    !fight
            ]],
            DIALOG_WON_FIGHT = [[
                {defendant_is_dead?
                    * Well, the hammer of justice certainly fell—right down on {defendant}'s head. Perhaps this matter would've benefited from a careful approach.
                }
                {not defendant_is_dead?
                    defendant:
                        !right
                        !scared
                        Enough! Let's get outta here already!
                        I can't see the backside of this camp soon enough. 
                        
                }
            ]],

            OPT_LEAVE_DEFENDANT = "Leave {defendant} to {defendant.hisher} fate",
            DIALOG_LEAVE_DEFENDANT = [[
                player:
                    Ah—apologies, {defendant}. I did my level best.
                defendant:
                    !right
                    <i>That</i> was your best?!
                player:
                    Well, it's certainly what I'll tell {giver} was my best.
                    !exit
                * The Rise line {defendant} up against a tarp they've hung, and the executioners take aim. {defendant} doesn't even get the benefit of a blindfold.
                * You're out of sight, but not out of earshot, when the deed is done. 
                    
            ]],

            OPT_ATTACK = "Attack!",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    I'm afraid I'll have to be a hostile witness.
            ]],

        }
        :Fn(function(cxt) 
        
                cxt.quest:GetCastMember("encampment_int"):SetMapPos(cxt.location:GetMapPos())
                cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("encampment_int") )

                cxt.encounter:PresentAgent(nil, SIDE.LEFT)
                cxt.encounter:PresentAgent(nil, SIDE.RIGHT)
                cxt.quest:GetCastMember("defendant"):MoveToLocation(cxt.location)

                local judge = AgentUtil.GetFreeAgent("RISE_PAMPHLETEER")
                CreateCombatBackup(judge, "RISE_BACKUP", cxt.quest:GetRank())
                cxt:TalkTo(judge)

                cxt.quest:Complete("infiltrate_the_encampment")
                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                        :Battle{
                            allies = {cxt.quest:GetCastMember("defendant")}
                        }
                            :OnWin()
                                :GoTo("STATE_POST_TRIAL_FIGHT")


                cxt:Opt("OPT_CONVINCE_MERCY")
                    :Dialog("DIALOG_CONVINCE_MERCY")
                    :Negotiation{
                        }
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_MERCY_SUCCESS")
                            :Fn(function() 
                                cxt.quest.param.was_good_lawyer = true
                                cxt.quest:GetCastMember("defendant"):Recruit( PARTY_MEMBER_TYPE.ESCORT )
                                cxt.quest:Activate("escort_defendant")
                            end)
                            :Travel()
                        
                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_MERCY_FAILURE")
                            :Fn(function() 
                                cxt:Opt("OPT_DEFEND")
                                    :Dialog("DIALOG_DEFEND")
                                    :Battle{
                                        allies = {cxt.quest:GetCastMember("defendant")}
                                    }
                                        :OnWin()
                                            :GoTo("STATE_POST_TRIAL_FIGHT")

                                cxt:Opt("OPT_LEAVE_DEFENDANT")
                                    :Dialog("DIALOG_LEAVE_DEFENDANT")
                                    :Fn(function() 
                                        cxt.quest:GetCastMember("defendant"):Kill() 
                                        cxt.quest:Activate("explain_self_to_giver")
                                    end)
                                    :Travel()
                            end)

            end)

    :State("STATE_POST_TRIAL_FIGHT")
        :Loc{
            DIALOG_WON_FIGHT = [[
                {defendant_is_dead?
                    * Well, the hammer of justice certainly fell—right down on {defendant}'s head. Perhaps this matter would've benefited from a careful approach.
                }
                {not defendant_is_dead?
                    defendant:
                        !right
                        !scared
                        Enough! Let's get outta here already!
                        I can't see the backside of this camp soon enough. 
                }

            ]],

        }
        :Fn(function(cxt) 
            cxt.enc.scratch.defendant_is_dead = cxt.quest:GetCastMember("defendant"):IsDead()
            cxt:Dialog("DIALOG_WON_FIGHT")
            if cxt.quest:GetCastMember("defendant"):IsDead() then
                cxt.quest:Activate("explain_self_to_giver")
            else
                if not cxt.quest:GetCastMember("defendant"):IsInPlayerParty() then
                    cxt.quest:GetCastMember("defendant"):Recruit( PARTY_MEMBER_TYPE.ESCORT )
                end
                cxt.quest:Activate("escort_defendant")
            end
            StateGraphUtil.AddLeaveLocation(cxt) 
        end)

----------ENCAMPMENT INTERIOR B
-- if you fought outside, you're going to have to fight again inside to rescue the defendant (they will help you in the fight)

    :State("STATE_FIGHT_WAY_IN")
        :Loc{
            DIALOG_INTRO = [[
                * Just in the nick of time, too—you enter just as the "judge" is issuing {agent.hisher} final verdict.
                defendant:
                    !left
                agent:
                    !right
                    !angry
                    Well, {defendant}? I think we all know what comes next. But for the sake of fairness...
                    Is there anything you could possibly say in your own defense?
                defendant:
                    !scared_shrug
                    I don't—
                player:
                    !left
                    !point
                    Objection!
                agent:
                    !dubious
                    What in the deep blooming sea...?
            ]],
            OPT_DO_BATTLE = "Attack the Rise",
            DIALOG_DO_BATTLE = [[
                player:
                    !left
                    OBJECTION, I say!
                agent:
                    Oh, for the love of—
                    What <i>now</i>?
                player:
                    I'm afraid since your ears are deafened to reason...
                    I'll simply have to box them, instead.
                    !fight
            ]],
            DIALOG_WON_FIGHT = [[
                {defendant_is_dead?
                    * Well, the hammer of justice certainly fell—right down on {defendant}'s head. Perhaps this matter would've benefited from a careful approach.
                }
                {not defendant_is_dead?
                    defendant:
                        !right
                        !scared
                        Enough! Let's get outta here already!
                        I can't see the backside of this camp soon enough. 
                }
            ]]
        }
        :Fn(function(cxt) 
            cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("encampment_int") )
            cxt.encounter:PresentAgent(nil, SIDE.LEFT)
            cxt.encounter:PresentAgent(nil, SIDE.RIGHT)
            cxt.quest:GetCastMember("defendant"):MoveToLocation(cxt.location)
            cxt.quest:Complete("infiltrate_the_encampment")

            local judge = AgentUtil.GetFreeAgent("RISE_PAMPHLETEER")
            CreateCombatBackup(judge, "RISE_BACKUP", cxt.quest:GetRank())
            cxt:TalkTo(judge)

            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_DO_BATTLE")
                :Dialog("DIALOG_DO_BATTLE")
                :Battle{
                    allies = {cxt.quest:GetCastMember("defendant")}
                }
                    :OnWin()
                        :Fn(function() 
                            cxt.enc.scratch.defendant_is_dead = cxt.quest:GetCastMember("defendant"):IsDead()
                        end)
                        :Dialog("DIALOG_WON_FIGHT")
                        :Fn(function() 
                            if cxt.quest:GetCastMember("defendant"):IsDead() then
                                cxt.quest:Activate("explain_self_to_giver")
                            else
                                if not cxt.quest:GetCastMember("defendant"):IsInPlayerParty() then
                                    cxt.quest:GetCastMember("defendant"):Recruit( PARTY_MEMBER_TYPE.ESCORT )
                                end
                                cxt.quest:Activate("escort_defendant")
                            end
                        end)
                        :Travel()
        end)


----------ESCORT HOME INTERRUPTION
-- If you lawyered your way through the encampment, some rogue rise show up here to kill defendant anyway, and you have to defend them (or turn them over).
-- If you fought your way through the encampment, the defendant stops you when you come across some workers. 
--    You need to negotiate or fight with the foreman, or they beat up the workers and make them hate you.

QDEF:AddConvo("escort_defendant")
    :TravelConfront("INTERRUPT", function(cxt) return not cxt.quest.param.escort_interrupt end)
    :Fn(function(cxt) 
        cxt.quest.param.escort_interrupt = true
        cxt:GoTo(cxt.quest.param.was_good_lawyer and "STATE_RISE_ATTACK" or "STATE_BAD_BARON")
    end)
    
    :State("STATE_RISE_ATTACK")
        :Loc{
            DIALOG_INTRO = [[
                * It doesn't take long for you to realize you're being trailed. You lead {defendant} into a clearing where you hope you'll have the higher ground.
                player:
                    !left
                agent:
                    !right
                    I dunno who you think you are, grifter, but I think it's time we cleared up your delusions. 
                * The Rise waste no time—battle is upon you, whether you're prepared or not.
            ]],

            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    Some are never content to accept mercy.
                    Very well.
                    !fight
            ]],

            --do we want a heart-to-heart with defendant if they survive?
            DIALOG_WON_FIGHT = [[
                {defendant_is_dead?
                    {dead?
                        * Well, the hammer of justice certainly fell—right down on {defendant}'s head. Perhaps this matter would've benefited from a careful approach.
                    }

                    {not dead?
                        agent:
                            !injured
                            Hah! At least that's done with. 
                            Run back to your Baron master, grifter, and tell him that the Rise got the best of him once again.
                            !exit
                        * Looks like you'll have to do exactly that, unfortunately.
                    }

                }

                {not defendant_is_dead?

                    defendant:
                        !right
                        Aw, Hesh. I knew we got outta there too easy.
                        Let's hurry, yeah? Don't wanna risk another group catching up to us.
                    * Indeed—besides, {giver} always appreciates a prompt delivery.
                }
            ]]
        }
        :Fn(function(cxt) 

            local team =  CreateCombatParty("RISE_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(team[1])

            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = {cxt.quest:GetCastMember("defendant")}
                }
                    :OnWin()
                        :Fn(function() 
                            cxt.enc.scratch.defendant_is_dead = cxt.quest:GetCastMember("defendant"):IsDead()
                        end)
                        :Dialog("DIALOG_WON_FIGHT")
                        :Fn(function() 
                            if cxt.quest:GetCastMember("defendant"):IsDead() then
                                cxt.quest:Cancel("escort_defendant")
                                cxt.quest:Activate("explain_self_to_giver")
                            end
                        end)
                        :Travel()

        end)

    :State("STATE_BAD_BARON")
        :Loc{
            DIALOG_INTRO = [[
                * {defendant} grows more confident as you pass back through occupied worksites—as evidenced by the way {defendant.hisher} eyes narrow suddenly.
                defendant:
                    !left
                    !angry
                    Hey, <i>you</i>! {worker}!
                worker:
                    !right
                    !scared 
                    W-what are you doing here? The Rise shoulda—
                defendant:
                    !fight
                    Arrested me? Yeah, thanks to your stupid trap!
                    Don't think I don't know what you did. You were in on this.
                    I knew I should've flogged you senseless instead of issuing another demerit. 
                    !spit
                    You don't <i>deserve</i> demerits.
                worker:
                    !placate
                    H-hey, that's against contract! You can't—
                defendant:
                    !angry_threaten
                    To Hesh with what I can't do! <i>You</i> report to <i>me</i>, laborer. I'll make sure you remember it.

                * There's murder in {defendant}'s eyes. 

            ]],
            OPT_LET_DEF_ATTACK = "Let {defendant} attack {worker}",
            DIALOG_LET_DEF_ATTACK = [[
                * The beating is thorough, but {defendant} pulls up short just before {worker} succumbs to {worker.hisher} injuries.
                worker:
                    !right
                    !injured
                    W-why...?
                player:
                    !left
                    !shrug
                    I suppose {defendant} isn't one for contrition.
                worker:
                    And you? What do <i>you</i> get outta this?
                player:
                    $neutralResigned
                    I'm wondering that for myself, really.
                defendant:
                    !right
                    !chuckle
                    Let's ditch this worm. We have places to be. 
            ]],
            OPT_CONVINCE_DEF = "Convince {defendant} to back down",
            DIALOG_CONVINCE_DEF = [[
                defendant:
                    !right
                player:
                    !left
                    Excuse you. Is this really how you want to spend your newfound freedom?
                defendant:
                    !angry_shrug
                    And why not? It's mine to spend, ain't it?
                    Despite this ingrate's best intentions.
                player:
                    Perhaps you'll recall who you <i>owe</i> before you speak so flippantly.
            ]],
            DIALOG_CONVINCE_DEF_SUCCESS = [[
                player:
                    If it weren't for me you'd be nothing but a red smear sinking into the bog. 
                    Consider that perhaps I give good counsel. And we should walk away before we draw attention to ourselves.
                    Lest you end up back at the mercy of the workers you're so quick to threaten.
                defendant:
                    Hrm. Suppose you got a point there. Not that I quite like the way you made it.
                    I was doing this scrub a favor, really. When I report what {worker.heshe} did to the Barons, there's gonna be Hesh to pay.
                * You direct {defendant.himher} back to the road, leaving {worker} to ruminate on {defendant}'s words.
            ]],
            DIALOG_CONVINCE_DEF_FAILURE = [[
                player:
                    I saved your life—does my opinion count for nothing?
                defendant:
                    Hah! You saved my life 'cause you were ordered to. 
                    That was your job. Keeping workers in line is mine. 
            ]],
            OPT_DEFEND_WORKER = "Defend {worker}",
            DIALOG_DEFEND_WORKER = [[
                worker:
                    !left
                    !scared
                defendant:
                    !right
                    !throatcut
                    Hope your dues are all paid up, {worker}, because you're gonna be out of commission for a while.
                player:
                    !left
                    !angry
                    I know damn well that Fellemo would prefer the workforce remain <i>optimal</i>.
                    But if you wish to challenge my judgment, so be it.
                    !fight
            ]],
            DIALOG_WON_FIGHT = [[
                {dead? 
                    {worker_is_dead?
                        *  Well, the hammer of justice certainly fell—right down on {defendant}'s head. And to what benefit?
                    }

                    {not worker_is_dead?
                        worker:
                            !right
                            !tired 
                            T-thanks, grifter. I dunno what I did to deserve that but... 
                            You did the right thing. 
                    }
                }
                {not dead?
                    defendant:
                        !right
                        !injured
                        I suppose you're happy now that it's not just your <i>heart</i> that's bleeding. 
                    {worker_is_dead?
                        Funny thing is, it's <i>you</i> that got {worker} killed. I was just gonna break {worker.hisher} legs.
                    }
                    {not worker_is_dead?
                        worker:
                            !right
                            T-thank you. I dunno what kind of trouble this makes for you, but I'll repay it if I can. 
                        * A worksite horn blows in the distance, and {worker} snaps to attention, hurrying off to {worker.hisher} next shift.
                    }

                    defendant:
                        Now what?
                    player:
                        I'm taking you to {giver}, as I was asked. I'll be happy to inform him exactly why you're being delivered in this condition.
                    * You bind {defendant}'s wrists, and march {defendant.himher} back to headquarters.
                }
            ]],
        }
        :Fn(function(cxt) 

            local worker = AgentUtil.GetFreeAgent("HEAVY_LABORER")
            worker:MoveToLocation(cxt.location)
            cxt:TalkTo(cxt.quest:GetCastMember("defendant"))

            cxt:ReassignCastMember("worker", worker)

            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_LET_DEF_ATTACK")
                :Dialog("DIALOG_LET_DEF_ATTACK")
                :ReceiveOpinion("let_them_get_beaten",nil, worker)
                :Travel()

            cxt:Opt("OPT_CONVINCE_DEF")
                :Dialog("DIALOG_CONVINCE_DEF")
                :Negotiation{
                    helpers = {worker}
                }
                    :OnSuccess()
                        :Dialog("DIALOG_CONVINCE_DEF_SUCCESS")
                        :ReceiveOpinion("saved_from_beating", nil, worker)
                        :Travel()

                    :OnFailure()
                        :Dialog("DIALOG_CONVINCE_DEF_FAILURE")

            cxt:Opt("OPT_DEFEND_WORKER")
                :Dialog("DIALOG_DEFEND_WORKER")
                :ReceiveOpinion("defended_them_from_foreman", nil, worker)
                :Battle{
                    allies = {worker},
                }
                    :OnWin()
                        :Fn(function() 
                            cxt.enc.scratch.worker_is_dead = worker:IsDead()
                            cxt:Dialog("DIALOG_WON_FIGHT")
                            if cxt:GetAgent():IsDead() then
                                cxt.quest:Cancel("escort_defendant")
                                cxt.quest:Activate("explain_self_to_giver")
                            else
                                cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.CAPTIVE)
                            end

                        end)
                        :Travel()
        end)

----------RETURN DEFENDANT TO GIVER


QDEF:AddConvo("escort_defendant", "giver")
    
    :Loc{
            
            OPT_HAND_OVER_GIVER_PRISONER = "Turn over {defendant}",
            DIALOG_HAND_OVER_GIVER_PRISONER = [[
                player:
                    The job is done, with only a slight modification of the arrangements.
                agent:
                    "Slight", is it? And why is {defendant} looking so glum?
                player:
                    Because {defendant} doesn't respect the bottom line. 
                    {defendant.HeShe} abused workers to the point of rendering them unfit for their work. 
                    Even after rescuing, {defendant.heshe} seemed content to return to those bad habits.
                agent:
                    !angry
                    Is that so?
                defendant:
                    !left
                    !scared
                    Hey, only as much as my foreman did to me!
                    Y-you don't know what it's like out there, with the Rise lurking in every corner.
                    It gives the workers ideas that they don't gotta listen to me. How else am I supposed to keep them in line?
                agent:
                    Indeed. Well, fortunately the problem should fix itself. 
                    You're demoted. You'll return to workcamp Delta and report to the foreman there for your assignment.
                defendant:
                    W-what?! B-but—
                agent:
                    I have no doubt you'll be very quickly taught all the ways that a worker can be kept in line <i>without</i> sabotaging their productivity. Dismissed.
                defendant:
                    !exit
                * Arint leads {defendant} away with a sharp prod. 
                player:
                    !left
                agent:
                    Excellent work. I appreciate that you looked out for company interests. 
            ]],
            
            OPT_HAND_OVER_GIVER_ESCORT = "Return {defendant} safely",
            DIALOG_HAND_OVER_GIVER_ESCORT = [[
                player:
                    The target has been acquired and, now, safely delivered.
                    !crossed
                    I trust you'll find everything satisfactory. 
                agent:
                    You know what they say about trust, old boy. But in this instance, yes.
                    I am satisfied.
                defendant:
                    !right
                    Thanks for the rescue, {player}. I'm sure I could've talked my own way out of it but...
                    I appreciate it all the same.
                player:
                    !dubious
                agent:
                    !right
                    !give
                    Here you are. That should be all, yes?
            ]],
        }
    :Hub(function(cxt) 
        if cxt:GetCastMember("defendant"):IsEscortMember() then

            cxt:Opt("OPT_HAND_OVER_GIVER_ESCORT")
                :SetQuestMark()
                :Dialog("DIALOG_HAND_OVER_GIVER_ESCORT")
                :ReceiveOpinion("rescued_them", nil, cxt.quest:GetCastMember("defendant"))
                :Fn(function() 
                    cxt.quest:GetCastMember("defendant"):Dismiss()
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)        
                end)

        elseif cxt:GetCastMember("defendant"):IsCaptiveMember() then

            cxt:Opt("OPT_HAND_OVER_GIVER_PRISONER")
                :SetQuestMark()                
                :Dialog("DIALOG_HAND_OVER_GIVER_PRISONER")
                :ReceiveOpinion("lost_their_job", nil, cxt.quest:GetCastMember("defendant"))
                :Fn(function() 
                    cxt.quest:GetCastMember("defendant"):Dismiss()
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)        
                end)
        end
    end)        


----------RETURN EMPTYHANDED TO GIVER


QDEF:AddConvo("explain_self_to_giver", "giver")
    
    :Loc{
            OPT_TELL_GIVER_MESSED_UP = "Tell {agent} the bad news",            
            DIALOG_TELL_GIVER_MESSED_UP = [[
                player:
                    The mission failed. I was unable to acquire the target.
                agent:
                    !disappoint
                    Yes, I'd already heard as much through one of my scouts.
                    I'm at a loss for words, {player}. There once was a time when you could get the job done.
                    
                    Shape up before I'm convinced you're past your prime. 
            ]]
        }
    :Hub(function(cxt) 
        cxt:Opt("OPT_TELL_GIVER_MESSED_UP")
            :Dialog("DIALOG_TELL_GIVER_MESSED_UP")
            :SetQuestMark()
            :Fn(function() 
                if not cxt.quest:GetCastMember("defendant"):IsDead() then
                    cxt.quest:GetCastMember("defendant"):Kill()
                end
                cxt.quest:Fail()
            end)
    end)        



---------------------------------------------------------------------------------------------------

QDEF:AddDebugOption("researcher_motivation", {"MORAL", "GREED", "BOG_MADNESS"})
QDEF:AddDebugOption("set_rise_target_relationship", {"FRIEND", "NEUTRAL", "ENEMY"})
QDEF:AddDebugOption("first_interrupt_type", {"BARONS", "MONSTERS"})
