local QDEF = QuestDef.Define{
    title = "Wild Pack of Baron Dogs",
    qtype = QTYPE.CONTRACT,
    faction = "SPARK_BARONS",
    rank = {1, 3},
    desc = [[A pack of AutoDogs has malfunctioned and is terrorizing the bog.]],
    icon = engine.asset.Texture("icons/quests/baron_wild_pack_of_dogs.tex"),
    focus = QUEST_FOCUS.COMBAT,
    reward_size = QREWARDSIZE.SMALLER,
    on_start = function(quest)
        quest:Activate("investigate_the_attack_site")

    end,

    on_destroy = function(quest)
        if quest:GetCastMember("rise_hunter") and quest:GetCastMember("rise_hunter"):IsInPlayerParty() then
            quest:GetCastMember("rise_hunter"):Dismiss()
        end
    end
}


:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddLocationDefs{
    DOG_DEN = {
        name = "AutoDog Den",
        desc = "A makeshift den constructed by AutoDogs.",
        plax = "EXT_Bog_Forest_01",
        map_tags = {"forest"},
        indoors = false,
    },
    DOG_ATTACK_SITE = {
        name = "AutoDog attack site",
        desc = "The last known location of the pack.",
        plax = "EXT_Bog_Outcrop_01",
        map_tags = {"forest"},
        indoors = false,
    },
}


local rise_defs = {"RISE_REBEL", "RISE_PAMPHLETEER"}
QDEF:AddCast{
    cast_id = "rise_hunter",
    when = QWHEN.MANUAL,
    condition = function( agent, quest )
        return table.arrayfind( rise_defs, agent:GetContentID())
    end,
}
:AddCastFallback
{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( table.arraypick(rise_defs) ) )
    end,
}



:AddLocationCast{
    cast_id = "attack_site",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("DOG_ATTACK_SITE"))
    end,
}

:AddLocationCast{
    cast_id = "den",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("DOG_DEN"))
    end,
}

:AddObjective{
    id = "investigate_the_attack_site",
    title = [[Investigate the attack site]],
    desc = "The AutoDog pack was last seen attacking a group of workers at this location. You might be able to pick up their trail there.",
    mark = {"attack_site"},
    on_complete = function(quest) 
        quest:Activate("attack_the_pack")
    end
}

:AddObjective{
    id = "attack_the_pack",
    title = [[Wipe out the AutoDog pack]],
    desc = "The AutoDogs have made a den here. Wipe them out.",
    mark = {"den"},
    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[Tell {giver} that the pack has been dealt with.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}


:AddOpinionEvents{
    shared_cores_with_friend = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Shared AutoDog cores with them",
    },
    refused_cores_to_friend = 
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Refused to share AutoDog cores with them",
    },
    shared_cores_with_stranger =
    {
        delta = OPINION_DELTAS.OPINION_UP,
        txt = "Shared AutoDog cores with them",
    },
    --[[got_them_extra_food =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Got them their fair share of rations",
    },--]]
    
}


QDEF:AddConvo("investigate_the_attack_site")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("attack_site") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find signs of a recent struggle in the clearing. Bloody scraps of metal and clothing are strewn about, and the distinct odor of ozone hangs heavy in the air.
                player:
                    !left
                * You're kicking over a particular grisly piece of scrap when you realize that you are not alone.
            ]],
            
            DIALOG_RISE_INTRO = [[
                rise_hunter:
                    !right
                * {rise_hunter} eyes you warily.
                rise_hunter:
                    !suspicious
                    What are <i>you</> doing here, grifter?
                player:
                    !placate
                    $miscPersuasive
                    Easy, laborer. I'm simply a contractor, here to recover the scrap. 
                rise_hunter:
                    !spit 
                    $angryReluctant
                    Sure. Likely.
                    I should teach you a lesson.
                * {rise_hunter} reaches for {rise_hunter.hisher} weapon.
            ]],

            OPT_NEGOTIATE_PEACE = "Convince {rise_hunter} to work with you",
            DIALOG_NEGOTIATE_PEACE = [[
                player:
                    !interest
                    Wait a moment, please. I have a proposition.
                    The pack is on the loose, and they might do untold damage before they're reined in.
                    Instead of impeding me, why not work in tandem?
            ]],
            DIALOG_NEGOTIATE_PEACE_SUCCESS = [[
                player:
                    $miscPersuasive
                    You can ensure I behave agreeably, and I can ensure no one else gets hurt.
                    That is, of course, what I've been paid to do.
                agent:
                    !dubious
                    ...
                    !crossed
                    $neutralResigned
                    Fine, grifter. I'll work with you.
                    But only until the rusty curs are in line. 
                player:
                    !happy 
                    $happyDefault
                    Excellent!
                    !eureka
                    Now we simply need to track the beasts.
                agent:
                    !permit
                    $miscMocking
                    Easy enough, innit? They don't exactly pass without trace. 
                * Sure enough, the Autodogs have torn a trench of a trail through the bog. 
                    !exit
            ]],
            DIALOG_NEGOTIATE_PEACE_FAIL = [[
                player:
                    !wink
                    $miscMocking
                    It'll at least get you out of a shift at the dig site, eh?
                agent:
                    !angry 
                    $angryPatienceLost
                    Right. Because I'm sure the Barons will be pleased as punch I went truant to help a contractor.
                    Before I was just gonna let out some steam.
                    But now? 
                    !fight
                    Now I think you're dead.
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                agent:
                    !fight
                player:
                    $neutralResigned
                    As you wish, I suppose.
            ]],
            DIALOG_DEFEND_POST_FAIL = [[
                player:
                    !fight
                    Perhaps you'll find my aim more persuasive than my words.
            ]],


            DIALOG_FOUGHT_DEAD = [[
                * {rise_hunter} slumps over, another corpse for the battlefield.
                * You search {rise_hunter.hisher} pockets and find some hand-scrawled notes indicating the direction the pack went.
                * You can't help but think there was an easier way to find that.
            ]],
            DIALOG_FOUGHT_ALIVE = [[
                rise_hunter:
                    !injured
                    Stop! We don't have to do this.
                player:
                    !cruel
                    $happyCocky
                    Finally, a word of reason.
                rise_hunter:
                    I know which way the pack went. 
                    You seem... capable. I'll tell you where they are.
                    Then you can go put an end to them.
                    !give
                player:
                    !take
                    $neutralDubious
                    We could have started with this, you know.
                rise_hunter:
                    Just make sure they can't hurt anyone else. 
                    !exit
                * {rise_hunter} limps off, leaving a trail of blood.
            ]],
 
            DIALOG_RISE_INTRO_FRIEND = [[
                rise_hunter:
                    !right
                    !happy
                    {player}? What brings you out this way?
                player:
                    !overthere
                    I believe this grim scene holds the clue.
                rise_hunter:
                    You're hunting the pack?
                    !eureka
                    $happyDefault
                    Well, isn't that my luck! I'm doing the same.
                    You'll help me, won't you?
            ]],

            OPT_AGREE_TO_WORK_TOGETHER = "Agree to hunt the pack together",
            DIALOG_AGREE_TO_WORK_TOGETHER_FRIEND = [[
                player:
                    !agree
                    Seeing this carnage, I think we stand a better chance of survival if we work together.
                rise_hunter:   
                    My thoughts exactly.
                    This way—I know where they've gone.
                    !exit
            ]],

            OPT_WORK_ALONE = "Work alone",
            DIALOG_WORK_ALONE_FRIEND = [[
                player: 
                    !placate
                    I don't think that's a good idea. 
                    $neutralThoughtful
                    The Barons are paying me to do this, and there will be questions.
                rise_hunter:
                    !thought 
                    I get it. You wouldn't want your Baron masters seeing what kind of company you keep.
                    Fine. But I'll do you a favor and show you where the pack went.
                    !give
                    Try not to get yourself killed, alright?
                    !exit
            ]],

        }
        :Fn(function(cxt) 
            
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:AssignCastMember("rise_hunter")
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("rise_hunter"))
            
            if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL then
                cxt:Dialog("DIALOG_RISE_INTRO_FRIEND")
                
                cxt:Opt("OPT_AGREE_TO_WORK_TOGETHER")
                    :Dialog("DIALOG_AGREE_TO_WORK_TOGETHER_FRIEND")
                    :RecruitMember(PARTY_MEMBER_TYPE.ESCORT)
                    :CompleteQuest("investigate_the_attack_site")
                    :Travel()

                cxt:Opt("OPT_WORK_ALONE")
                    :Dialog("DIALOG_WORK_ALONE_FRIEND")
                    :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                    :CompleteQuest("investigate_the_attack_site")
                    :Travel()

            else
                cxt:Dialog("DIALOG_RISE_INTRO")
                

                local OnWonBattle = function(cxt)
                    if cxt:GetAgent():IsDead() then
                        cxt:Dialog("DIALOG_FOUGHT_DEAD")    
                    else
                        cxt:Dialog("DIALOG_FOUGHT_ALIVE")    
                        cxt:GetAgent():MoveToLimbo()
                    end

                    cxt.quest:Complete("investigate_the_attack_site")
                    StateGraphUtil.AddLeaveLocation(cxt)

                end


                cxt:Opt("OPT_NEGOTIATE_PEACE")
                    :Dialog("DIALOG_NEGOTIATE_PEACE")
                    :Negotiation{
                        on_fail = function(cxt) 
                                cxt:Dialog("DIALOG_NEGOTIATE_PEACE_FAIL")
                                cxt:Opt("OPT_DEFEND")
                                    :Dialog("DIALOG_DEFEND_POST_FAIL")
                                    :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        on_win = OnWonBattle
                                    }
                            end,
                        on_success = function(cxt)
                                cxt:Dialog("DIALOG_NEGOTIATE_PEACE_SUCCESS")
                                cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                cxt.quest:Complete("investigate_the_attack_site")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end,
                    }

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = OnWonBattle,
                    }
            end
        end)



QDEF:AddConvo("attack_the_pack")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("den") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find the den, though your boots don't thank you for it. 
                * The dogs lay motionless amid an array of discarded scrap metal. High-frequency whine, shrill and suffering, issues from the vents of the largest.
                * In this state it's hard to see them as a threat, which is why they catch you off guard when they spring into action at your approach.
                agent:
                    !right
                    BZZT
                player:
                    !left
                    !surprised
                * The still-functional members of the pack attack!
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND_ALONE = [[
                player:
                    !fight
                agent:
                    BRRRRR
            ]],
            DIALOG_DEFEND_WITH_RISE = [[
                rise_hunter:
                    !left
                    !fight
                    Come and get it, you bolt buckets!
            ]],

            DIALOG_COLLECT_CORES_ALONE = [[
                * You collect up as many of the cores as you can find, though some of the pieces are hard to identify. Hopefully {giver} will be able to make sense of the debris.
            ]],

            DIALOG_COLLECT_CORES_WITH_RISE_FRIEND = [[
                left:
                    !exit
                right:
                    !exit
                * You collect up all of the Autodog cores that you can find. {rise_hunter} watches with interest.
                player:
                    !left
                rise_hunter:
                    !right
                    !bashful
                    $miscEmbarrased
                    You know, the Rise could really use those cores for our own cobblebots.
                    We get enough of those on our side, and suddenly the Spark Barons have more to worry about than just their bottom line. 
                    !shrug
                    I know you have your job to do, but... maybe you could spare half?
            ]],
            
            DIALOG_COLLECT_CORES_WITH_RISE = [[
                left:
                    !exit
                right:
                    !exit
                * You collect the Autodog cores as {rise_hunter} watches.
                player:
                    !left
                * When you're done, {rise_hunter.heshe} speaks up, with a voice that suggests this was {rise_hunter.hisher} plan all along.
                rise_hunter:
                    !right
                    $neutralThanks
                    Thank you for your help, grifter. But the Rise needs those cores.
                    !hips
                    $neutralDirect
                    Hand them over, and your faith won't go unnoticed. 
            ]],
            DIALOG_COLLECT_CORES_RISE_SHOWS_UP = [[
                left:
                    !exit
                right:
                    !exit
                * You collect up all of the Autodog cores you can find in the wreckage.
                player:
                    !left
                * As you're working, {rise_hunter} shows up, getting the drop on you.
                rise_hunter:
                    !right
                    Hand over the cores, grifter.
                    !fight
                    The Rise needs them.
            ]],

            DIALOG_COLLECT_CORES_RISE_SHOWS_UP_FRIEND = [[
                left:
                    !exit
                right:
                    !exit
                * You collect up all of the Autodog cores you can find in the wreckage.
                player:
                    !left
                * When you're done, you're surprised to see {rise_hunter} watching you.
                rise_hunter:
                    !right
                    !placate
                    Sorry, didn't mean to startle you. 
                    But the Rise <i>really</i> needs those cores.
                    You would never have found the den if I hadn't told you where it is.
                    $miscPersuasive
                    How about you give me half as a finder's fee, and we call things even?
            ]],
            DIALOG_COLLECT_CORES_RISE_DIED = [[
                * You gather up the Autodog cores, gingerly avoiding {rise_hunter}'s still-warm corpse.
            ]],

            OPT_GIVE_HALF = "Give {agent} half of the cores",
            OPT_GIVE_CORES = "Give {agent} all of the cores",
            
            DIALOG_GIVE_HALF_FRIEND = [[
                player:
                    I suppose that's only fair.
                    !give
                agent:
                    !take
                    Thanks, {player}.
                    !exit
                * {agent} takes the cores back to the Rise.
            ]],
            DIALOG_REFUSE_FRIEND = [[
                player:
                    Sorry, {agent}, but I have to return these to the Barons.
                agent:
                    !angry
                    Really? Just like that?
                    Guess it's clear where you really hang your hat, huh?
                    I won't forget this.
                * {agent} leaves in a huff.
            ]],

            DIALOG_GIVE_ALL_CORES = [[
                player:
                    !agree
                    Here. Take them.
                    It's the laborers' blood they spilt, after all. 
                    !give
                agent:
                    !take
                    $neutralDubious
                    So easy? I did expect you to argue. 
                    $miscRelieved
                    Thank you. That fight was harder than I expected, and I might not have done it without your help. 
                    !exit
                * {agent} skulks off. You're going to have to explain yourself to {giver}.
            ]],

            DIALOG_NEGOTIATE_HALF = [[
                player:
                    !shrug
                    I need to have something to show the Barons. 
                    $miscPersuasive
                    How about I give you half the cores, eh? Then we both have something to show for ourselves.
            ]],
            DIALOG_NEGOTIATE_HALF_SUCCESS = [[
                player:
                    It's only fair, isn't it?
                agent:
                    !sigh
                    $neutralResigned
                    I suppose so. They did put up a fight, after all, and you were there to lend a hand.
                    !take
                player:
                    !give
                agent:
                    Until next time.
                    !exit
                * {agent} hauls off {agent.hisher} unearned loot.
            ]],
            DIALOG_NEGOTIATE_HALF_FAIL = [[
                player:
                    It's only fair, isn't it?
                agent:
                    $miscMocking
                    Oh, did <i>you</i> have friends get chewed apart by these rampaging fiends?
                    No, I didn't think so.
                    !crossed
                    It's called the <i>Rise</i>, not the compromise. 
            ]],
            DIALOG_REFUSE_CORES = [[
                player:
                    I fought like hell to get these.
                    !fight
                    I'm not giving them up without another fight.
            ]],
            DIALOG_DEFEATED_RISE = [[
                * You gather up the spilled cores and leave before reinforcements arrive.
            ]],

            OPT_NEGOTIATE_HALF = "Offer to give up half of the cores",
            OPT_REFUSE = "Keep all of the cores",
        }
        :Fn(function(cxt) 
            
            local fought_with_rise = false
            local allies = {}
            if cxt:GetCastMember("rise_hunter"):IsInPlayerParty() then
                table.insert(allies, cxt:GetCastMember("rise_hunter"))
                fought_with_rise = true
            end

            local autodogs = {}

            local num_dogs = 4 + cxt.quest:GetRank()
            if fought_with_rise then
                num_dogs = num_dogs + 3  
            end

            for k = 1, num_dogs do
                local agent = TheGame:GetGameState():AddAgent(Agent("FERAL_AUTODOG"))
                table.insert(autodogs, agent)
                agent:MoveToLocation(cxt.location)
            end

            cxt.enc:SetPrimaryCast(autodogs[1])
            cxt:Dialog("DIALOG_INTRO")


            local function DealWithFriend(cxt)
                
                cxt:Opt("OPT_GIVE_HALF")
                    :Dialog("DIALOG_GIVE_HALF_FRIEND")
                    :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("shared_cores_with_friend"))                                        
                    :Fn(function() 
                            cxt.quest.param.half_cores = true
                            cxt:GetCastMember("rise_hunter"):Dismiss() 
                        end)
                    :Travel()
                
                cxt:Opt("OPT_REFUSE")
                    :Dialog("DIALOG_REFUSE_FRIEND")
                    :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("refused_cores_to_friend"))                                        
                    :Fn(function() cxt:GetCastMember("rise_hunter"):Dismiss() end)
                    :Travel()
            end

            local function DealWithNonFriend(cxt)
                cxt:RunLoopingFn( function()

                    cxt:Opt("OPT_GIVE_CORES")
                        :Dialog("DIALOG_GIVE_ALL_CORES")
                        :ReceiveOpinion( "shared_cores_with_stranger" )
                        :Fn(function() 
                                cxt.quest:GetCastMember("rise_hunter"):Dismiss()
                                cxt.quest.param.no_cores = true 
                            end)
                        :Travel()

                    cxt:Opt("OPT_NEGOTIATE_HALF")
                        :Dialog("DIALOG_NEGOTIATE_HALF")
                        :Negotiation{
                            on_success = function()
                                cxt:Dialog("DIALOG_NEGOTIATE_HALF_SUCCESS")
                                cxt.quest:GetCastMember("rise_hunter"):Dismiss()
                                cxt.quest.param.half_cores = true
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end,
                            on_fail = function()
                                cxt:Dialog("DIALOG_NEGOTIATE_HALF_FAIL")
                            end,
                        }

                    cxt:Opt("OPT_REFUSE")
                        :Dialog("DIALOG_REFUSE_CORES")
                        :Fn(function() cxt.quest:GetCastMember("rise_hunter"):Dismiss() end)
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            enemies = {cxt.quest:GetCastMember("rise_hunter")},
                            on_win = function()
                                cxt:Dialog("DIALOG_DEFEATED_RISE")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end,
                        }
                end)
            end

            local autodog_team_size = cxt:GetCastMember("rise_hunter"):IsInPlayerParty() and 4 or 3

            cxt:Opt("OPT_DEFEND")
                :Dialog(cxt:GetCastMember("rise_hunter"):IsInPlayerParty() and "DIALOG_DEFEND_WITH_RISE" or "DIALOG_DEFEND_ALONE")
                :Battle{
                    allies = allies,
                    enemies = autodogs,
                    red_team_size = autodog_team_size,
                    
                    flags = BATTLE_FLAGS.NO_DEATH_LOOT | BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function()

                        cxt.quest:Complete("attack_the_pack")
                        
                        if fought_with_rise then
                            
                            if cxt.quest:GetCastMember("rise_hunter"):IsDead() then
                                cxt:Dialog("DIALOG_COLLECT_CORES_RISE_DIED")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            else 
                                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("rise_hunter"))
                                if cxt:GetCastMember("rise_hunter"):GetRelationship() > RELATIONSHIP.NEUTRAL then
                                    cxt:Dialog("DIALOG_COLLECT_CORES_WITH_RISE_FRIEND")
                                    DealWithFriend(cxt)
                                else
                                    cxt:Dialog("DIALOG_COLLECT_CORES_WITH_RISE")
                                    DealWithNonFriend(cxt)
                                end

                            end
                        else

                            if cxt.quest:GetCastMember("rise_hunter"):IsDead() then
                                cxt:Dialog("DIALOG_COLLECT_CORES_ALONE")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            else
                                
                                cxt:GetCastMember("rise_hunter").health:SetPercent( (1 + cxt:GetCastMember("rise_hunter").health:GetPercent())/2 )
                                cxt.quest:GetCastMember("rise_hunter"):MoveToLocation(cxt.location)
                                --[[local BACKUP = {
                                    {"RISE_AUTOMECH"},
                                    {"RISE_REBEL", "RISE_AUTOMECH"},
                                    {"RISE_REBEL", "RISE_REBEL"},
                                    {"RISE_REBEL", "RISE_RADICAL"},
                                }--]]
                                --CreateCombatBackup(cxt:GetCastMember("rise_hunter"), function(diff) return BACKUP[diff] or BACKUP[#BACKUP] end, cxt.quest:GetRank())
                                CreateCombatBackup(cxt:GetCastMember("rise_hunter"), "RISE_BACKUP", cxt.quest:GetRank())
                                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("rise_hunter"))
                                
                                if cxt:GetCastMember("rise_hunter"):GetRelationship() > RELATIONSHIP.NEUTRAL then
                                    cxt:Dialog("DIALOG_COLLECT_CORES_RISE_SHOWS_UP_FRIEND")
                                    DealWithFriend(cxt)
                                else
                                    cxt:Dialog("DIALOG_COLLECT_CORES_RISE_SHOWS_UP")
                                    DealWithNonFriend(cxt)
                                end
                            end
                        end

                    end,
                }
        end)


QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GIVE_ALL_CORES = "Hand over the cores",
        DIALOG_ALL_CORES = [[
            player:
                Normally I'm the one dodging dogs. It was a nice change of pace to be the one doing the tracking.
                Here are the pack's cores, as promised.
                !give
            agent: 
                !take
                $neutralTakenAback
                So many! If I'd sent a Spark Baron to do it, they would've kept half to fuel their own side projects.
                You really will change the way I think of my underlings, old chap.
        ]],

        OPT_GIVE_NO_CORES = "Explain what happened to the cores",
        DIALOG_GIVE_NO_CORES = [[
            player:
                I tracked down the dogs, you see, but I'm afraid they were all worse for wear.
            agent:
                !agree
                To be expected in their state. And the cores?
            player:
                !placate
                Ah—about that...
        ]],
        DIALOG_GIVE_NO_CORES_SUCCESS = [[
            player:
                The poor things were so badly damaged that a puff of air could've disintegrated them.
                Scraps, being what they are, weren't worth retrieving.
            agent:
                !angry
                $angryAside
                Blast it! It's one thing for them to rampage—another entirely for them to fall apart on the job.
                I'll be having a talk with the lead engineer, and it won't be pleasant.

        ]],
        DIALOG_GIVE_NO_CORES_FAIL = [[
            player:
                I'm afraid the Rise beat me to the punch. They're wily scavengers.
            agent:
                !angry
                So you mean to tell me the cores are in the hands of our enemy? 
                !angry_accuse
                I suppose you thought I'd take that news agreeably, did you?
            player:
                $neutralResigned
                Not exactly, no.
            agent:
                $neutralDirect
                Quite right.
        ]],

        OPT_GIVE_HALF_CORES = "Convince {agent} that half the cores were lost",
        DIALOG_GIVE_HALF_CORES = [[
            player:
                Normally I'm the one dodging dogs. It was a nice change of pace to be the one doing the tracking.
                Here are the pack's cores, as promised.
                !give
            agent:
                !take
                $neutralDubious
                So few? 
            player:
                !placate
                Ah—about that...
        ]],
        DIALOG_GIVE_HALF_CORES_SUCCESS = [[
            player:
                $miscPersuasive
                The poor things were so badly damaged that a puff of air could've disintegrated them.
                $neutralResigned
                Once I realized, I adjusted my tactics, but this was all I was able to recover.
            agent:
                A pity. They're expensive to replace.
                Ah, well. At least you reacted in the moment, hmm? 
                !nudge_nudge
                That's what I like about you, Rook—you always think on your toes. 
                
        ]],
        DIALOG_GIVE_HALF_CORES_FAIL = [[
            player:
                I'm fairly certain only half the dogs had cores in the first place.
                Perhaps that's why they went rogue, hm?
            agent:
                !angry_shrug
                $angryPatienceLost
                Only half the—they can't function without them, you imbecile.
                I imagine you simply destroyed the cores with your recklessness.
                Not your finest moment. 
        ]],

    }
    :Hub(function(cxt) 
        
        if cxt.quest.param.no_cores then
            
            cxt:Opt("OPT_GIVE_NO_CORES")
                :Dialog("DIALOG_GIVE_NO_CORES")
                :SetQuestMark()
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_GIVE_NO_CORES_SUCCESS")
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt.quest.param.poor_performance = true
                        cxt:Dialog("DIALOG_GIVE_NO_CORES_FAIL")
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end,
                }


        elseif cxt.quest.param.half_cores then
            cxt:Opt("OPT_GIVE_HALF_CORES")
                :Dialog("DIALOG_GIVE_HALF_CORES")
                :SetQuestMark()
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_GIVE_HALF_CORES_SUCCESS")
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt.quest.param.poor_performance = true
                        cxt:Dialog("DIALOG_GIVE_HALF_CORES_FAIL")
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end,
                }
        else
            cxt:Opt("OPT_GIVE_ALL_CORES")
                :SetQuestMark()
                :Dialog("DIALOG_ALL_CORES")
                :CompleteQuest()
        end


    end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                The executives keep wondering when we'll come up with another Automech, you know. 
                !dubious
                $neutralDubious
                Our most successful enterprise, and it was brought to us by a <i>laborer</i>.
            player:
                No small wonder. Labor without the labor disputes, as it were.
            agent: 
                !angry_shrug
                Please. They're virtually useless without a handler. Brainless automatons.
                To keep the directors happy we came up with the dogs, but they have the unfortunate habit of going feral. 
                And wouldn't you guess, the blasted things know the bog better than anyone living.
                !overthere
                We have reports there's a whole colony loose, preying on wayward laborers. 
            player:
                !thought
                $neutralThoughtful
                Hm. Bad for morale. Not to mention attendance, I should say.
            agent:
                Indeed. Well, I'm sure you know what I'm asking. 

            *** {giver} needs you to track down the rogue AutoDogs and terminate them.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !overthere
                $neutralDubious
                We know the last place they, ah, made their presence felt among the laborers.
                You should be able to track the pack from there.
            player:
                !interest
                Must've been violent if even <i>you're</i> using euphemisms, old chap.
            agent:
                The less said about that the better. Oh, but when you destroy the chassis, be sure to salvage the core. 
                !eureka
                We can run a diagnostic and likely as not repair the model.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)
