local QDEF = QuestDef.Define{
    title = "Bandit Babysitter",
    qtype = QTYPE.CONTRACT,
    --faction = "SPARK_BARONS",
    rank = {1, 4},
    desc = [[Help an acolyte of Hesh retrieve stolen goods from a Spree encampment.]],
    icon = engine.asset.Texture("icons/quests/drop_smuggle_in_admiralty_weapons.tex"),
    tags = {"dead_drop"},
    reward_size = QREWARDSIZE.LARGER,
    focus = QUEST_FOCUS.BALANCED,

    collect_agent_locations = function(quest, t)

        if quest:IsActive("talk_to_hesh") then
            table.insert (t, { agent = quest:GetCastMember("priest"), location = quest:GetCastMember('bar'), role = CHARACTER_ROLES.PATRON } )
        end

        if quest:IsActive("go_to_bandits") then
            table.insert (t, { agent = quest:GetCastMember("spree"), location = quest:GetCastMember('bandit_hideaway'), role = CHARACTER_ROLES.PROPRIETOR } )
        end

    end,

    --collected location has precedence.
    is_in_hiding = function(quest, agent)
        
        if agent == quest:GetCastMember("priest") then
            return true
        end
    end,


    on_start = function(quest)
        quest:Activate("talk_to_hesh")
    end,

    on_destroy = function(quest)
        if quest:GetCastMember("spree") and quest:GetCastMember("spree"):IsInPlayerParty() then
            quest:GetCastMember("spree"):Dismiss()
        end
    end,

    avoids = 
    {
        --{"handler", "target"},
    },
}



:AddOpinionEvents{
    wiped_out_bandits = 
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Wiped out an entire Spree encampment for them",
    },
    demanded_more_money =  
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Demanded more money for doing a job",
    },
    
    figured_out_betrayal = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Figured out that you betrayed them",
    },
    defended_them = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Defended them from bandits",
    },
}

:AddCast{
    cast_id = "priest",
    cast_fn = function( quest, t )
        table.insert(t, quest:CreateSkinnedAgent( "PRIEST" ))
    end,
}

:AddCast{
    cast_id = "spree",
    cast_fn = function( quest, t )
        table.insert(t, quest:CreateSkinnedAgent( "BANDIT_RAIDER" ))
    end,
}


:AddLocationCast{
    cast_id = "bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"))
    end,
}

:AddLocationDefs{
    BANDIT_HIDEAWAY = {
        name = "Spree Hideaway",
        map_tags = {"bog"},
        plax = "EXT_Bog_Forest_01",
        indoors = false,
    },
    
    MEETING_PLACE = {
        name = "Meeting Point",
        map_tags = {"forest"},
        plax = "EXT_Bog_Forest_01",
        indoors = false,
    },

    ROBBERY_LOCATION = {
        name = "Robbery Location",
        map_tags = {"road"},
        plax = "EXT_Bog_Road_01",
        indoors = false,
    },
}

:AddLocationCast{
    cast_id = "bandit_hideaway",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("BANDIT_HIDEAWAY"))
    end,
}

:AddLocationCast{
    cast_id = "meeting_place",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("MEETING_PLACE"))
    end,
}

:AddLocationCast{
    cast_id = "robbery_location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("ROBBERY_LOCATION"))
    end,
}


:AddObjective{
    id = "talk_to_hesh",
    title = "Talk to {priest} about {priest.hisher} plan.",
    --desc = [[{handler} has a reward for you.]],
    on_complete = function(quest)
        quest:Activate("go_to_bandits")
    end,
    mark = {"priest"},
}

:AddObjective{
    id = "go_to_bandits",
    title = "Convince {spree} to leave camp.",
    on_complete = function(quest)
        quest:GetCastMember("spree"):MoveToLimbo()
        quest:Activate("do_robbery")
    end,
    mark = {"spree"},
}

:AddObjective{
    id = "do_robbery",
    title = "Rob the sabotaged caravan.",
    on_complete = function(quest)
        quest:Activate("get_paid_by_priest")
    end,
    mark = {"robbery_location"},
}

:AddObjective{
    id = "get_paid_by_priest",
    title = "Meet up with {priest}.",
    mark = {"meeting_place"},
}


QDEF:AddConvo( "talk_to_hesh", "priest" )
    :Loc{
        OPT_ASK_ABOUT_JOB = "Ask about the job",
        
        DIALOG_ASK_ABOUT_JOB = [[
            player:
                You're {priest}? I'm {player}. You've paid for help?
            priest:
                You? I was hoping for someone a little more... 
                !hesh_greeting
                ...pious.
            player:
                !overthere
                I could let you work things out with Hesh personally, if you prefer.
            priest:
                No, no - Hesh's tentacles writhe in mysterious ways. Perhaps you are a ripple left by his passing.
            player:
                !dubious
                ...
            priest:
                That's a good thing.
            ]],
        OPT_ASK_ABOUT_JOB_2 = "Ask for details",
        DIALOG_ASK_ABOUT_JOB_2 = [[
            player:
                So these documents...
            priest:
                The Spree have the documents. They stole them, and now they're hiding out in the Bog waiting for things to blow over.
            player:
                And now you want <i>me</> to walk in there guns blazing to get them back? Don't you people have luminari for that kind of thing?
            priest:
                No, I need to keep this quiet. Using the luminari would invite questions that I don't want to answer.
                
                What I want you to do is distract the Spree leader. Offer {spree.himher} a score that {spree.heshe} can't refuse, and take them out of the encampment.
                I'll sneak in and get the documents while you're out. Then we can meet back up for payment.
            player:
                What score should I offer? I'm an honest grifter, not some bandit.
            priest:
                I've arranged something. A merchant caravan has broken down on one of the Bog's back roads. Their cargo will get the Spree's attention.
            *** {priest} wants you to distract {spree} by taking them to a robbery.
        ]],

        OPT_ASK_ABOUT_DOCUMENTS = "Ask about the documents",
        DIALOG_ASK_ABOUT_DOCUMENTS = [[
            player:
                So what's in these documents?
            priest:
                
                The contents are embarrassing. Both to the Cult generally, and to me specifically.
                !crossed
                But I'm paying you for help, not questions.

        ]],
        
        OPT_ASK_ABOUT_CARAVAN = "Ask about the caravan",
        DIALOG_ASK_ABOUT_CARAVAN = [[
            player:
                You just happen to know of a caravan that has broken down nearby? 
                That's suspiciously convenient timing.
            priest:
                !hesh_greeting
                Hesh provides, when properly appeased. 
                !shrug
                And I may have used His tithe money to arrange for some light sabotage.

        ]],

        OPT_HEAD_OUT = "Head out for the Bog",
        DIALOG_HEAD_OUT = [[
            player:
                To the Bog it is, then.
            priest:
                !point
                Yes. Go convince them of the score. 
                I'll follow at a distance - if the Spree sees us together the plan won't work.
        ]]


    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_ABOUT_JOB")
            :SetQuestMark()
            :Dialog("DIALOG_ASK_ABOUT_JOB")
            :Fn(function()
                cxt:Opt("OPT_ASK_ABOUT_JOB_2")
                    :Dialog("DIALOG_ASK_ABOUT_JOB_2")
                    :LoopingFn(function() 

                        cxt:Opt("OPT_ASK_ABOUT_DOCUMENTS")
                            :Dialog("DIALOG_ASK_ABOUT_DOCUMENTS")
                        
                        cxt:Opt("OPT_ASK_ABOUT_CARAVAN")
                            :Dialog("DIALOG_ASK_ABOUT_CARAVAN")

                        cxt:Opt("OPT_HEAD_OUT")
                            :PreIcon(global_images.close)
                            :Dialog("DIALOG_HEAD_OUT")
                            :DoneConvo()
                            :CompleteQuest("talk_to_hesh")
                    end)
            end)
    end)

QDEF:AddConvo("go_to_bandits")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("bandit_hideaway") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find the bandits where {priest} said they would be. The leader, {spree}, hears you approach.
                spree:
                    !right
                    !fight
                    Who's there?
                player:
                    !left
                    !placate
                    Just a simple grifter. I mean you no harm.
                spree:
                    How did you even find this place?
                player:
                    !happy
                    I heard some big players from Murder Bay were in the Bog.
                    My specialty lies in finding things that other people don't know about, and I'm good at my job.
                    Which is why I sought you out. I have a proposition.
                spree:
                    !angry_permit
                    We are big players. You have that part right.
                    Well, spit it out, then.
                    !throatcut
                    But if I don't like what I hear I'll have you killed where you stand. 
            ]],

            OPT_CONVINCE_SCORE = "Convince {agent} you have a lead on a big score",
            DIALOG_CONVINCE_SCORE = [[
                player:
                    There's a merchant caravan nearby, transporting Bog goods to the gate.
                spree:
                    !hips
                player:
                    It's lightly defended, and it's stuck. We could take it, if we worked together...
            ]],
            DIALOG_SUCCESS = [[
                spree:
                    Your offer sounds good.
                    !thought
                    Almost <i>too</> good.
                player:
                    ...
                spree:
                    !happy
                    I <i>like</> good offers.
                    We hit the caravan, and split the loot 80-20. Deal?
                player:
                    !happy
                    Of course.
                spree:
                    Lead the way. Me and my people will follow behind. You do the talking, and we'll step in if we need to.
                    !exit
                player:
                    !salute
                * {spree} rounds up the troops and heads out. You play along.
            ]],

            DIALOG_FAILURE = [[
                spree:
                    Your offer sounds good.
                    !fight
                    <i>Too</> good. Nice try, grifter, but I didn't survive this long trusting offers like that.
                * {spree} signals to {spree.hisher} underlings, and the whole camp jumps you.
                
            ]],
            DIALOG_DEFEND = [[
                player:
                    !fight
                    You're throwing away good shills!
            ]],
            DIALOG_ATTACK = [[
                player:
                    !fight
                    Let's just cut straight to the part where <i>I</> kill <i>you</>.
            ]],
            DIALOG_WON_BATTLE_KILLED = [[
                player:
                    !neutral
                * {spree} is dead, and the camp empties out in a hurry.
            ]],
            DIALOG_WON_BATTLE_LIVED = [[
                player:
                    !neutral
                spree:
                    !right
                    !injured
                    What are you?!
                player:
                    I suggest you run before you find out.
                spree:
                    !exit
                * The camp empties of bandits. Living ones, at least.
            ]],
            OPT_ATTACK = "Attack the <b>entire</> bandit camp",
            OPT_DEFEND = "Defend yourself!",


        }
        :Fn(function(cxt) 
            
            cxt:TalkTo(cxt:GetCastMember("spree"))
            
            --this is not meant to be a scalable, or fair, fight. If you can beat it, great! But you're really meant to negotiate.
            local extras = CreateCombatBackup(cxt.quest:GetCastMember("spree"), function() return {"BANDIT_GOON", "BANDIT_GOON2", "FLEAD", "FLEAD"} end)

            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_CONVINCE_SCORE")
                :Dialog("DIALOG_CONVINCE_SCORE")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_SUCCESS")
                        :CompleteQuest("go_to_bandits")
                        :Travel()
                    
                    :OnFailure()
                        :Dialog("DIALOG_FAILURE")
                        :Fn(function(cxt) 
                                --this is meant to be pretty much a death-sentence.   
                                cxt:Opt("OPT_DEFEND")
                                    :Dialog("DIALOG_DEFEND")
                                    :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE, no_oppo_limit = true }
                                        :OnWin()
                                            :Fn(function(cxt) 
                                                cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_WON_BATTLE_KILLED" or "DIALOG_WON_BATTLE_LIVED")
                                            end)
                                            :GoTo("STATE_FOUGHT_BANDITS_SOLO")

                        end)
                                
            
            --this is meant to be pretty much a death-sentence.   
            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{ no_oppo_limit = true}
                    :OnWin()
                        --you did it?! Well, OK.
                        :Fn(function(cxt) cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_WON_BATTLE_KILLED" or "DIALOG_WON_BATTLE_LIVED") end)
                        :GoTo("STATE_FOUGHT_BANDITS_SOLO")

        end)
    
    :State("STATE_FOUGHT_BANDITS_SOLO")
        :Loc{
            DIALOG_INTRO = [[
                * {priest} emerges from hiding. 
                priest:
                    !right
                    !clap
                    I'm impressed! I constructed a whole scheme to divide and conquer these goons, and then you just go and run them through.
                    I underestimated you, grifter!
                    !exit
                player:
                    !crossed
                * {priest.HeShe} searches through the abandoned tents, and finds a handful of papers.
                priest:
                    !right
                    I found what I came for. Shall we settle up?
            ]],

            OPT_DEMAND_A_BONUS = "Demand more money ({1#money})",
            DIALOG_DEMAND_A_BONUS = [[
                player: 
                    Given that I just wiped out this entire encampment, I believe I'm in a position to renegotiate our deal.
                priest:
                    !angry
                    Oh really?
            ]],
            DIALOG_GOT_BONUS = [[
                priest:
                    !angry
                    You're a hard-nosed oshnu turd, grifter.
                    I'll pay you more, but this is rank unprofessionalism.
                    !exit
                * {priest} practically throws the extra shills in your face before turning on {priest.hisher} heel and walking away.
                * You pick your blood money out of the mud.
            ]],
            DIALOG_NO_BONUS = [[
                priest:
                    !angry
                    No. We agreed to a deal. You deviated from the plan on your own accord - you can't expect me to pay more for it.
                    I'll pay you what we agreed to. And not a shill more.
                    !exit
                * {priest} drops your payment in the mud and storms off.
            ]],
            OPT_GET_PAID = "Get paid",
            DIALOG_GET_PAID = [[
                priest:
                    !give
                player:
                    !take
                    Thanks for the business. Tell your friends.
                priest:
                    If only all grifters were as honest and useful as you, friend.
                    !exit
            ]],
        }
        :Fn(function(cxt) 
            cxt:TalkTo(cxt.quest:GetCastMember("priest"))
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:GetCastMember("priest"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("wiped_out_bandits"))


            local bonus_amt = 300
            cxt:Opt("OPT_DEMAND_A_BONUS", bonus_amt)
                :Dialog("DIALOG_DEMAND_A_BONUS", bonus_amt)
                :Negotiation{}
                    :OnSuccess()
                        :CompleteQuest()
                        :ReceiveOpinion("demanded_more_money")
                        :Dialog("DIALOG_GOT_BONUS")
                        :ReceiveMoney(bonus_amt, {no_scale = true})
                        :Travel()
                    :OnFailure()
                        :CompleteQuest()
                        :ReceiveOpinion("demanded_more_money")
                        :Dialog("DIALOG_NO_BONUS")
                        :Travel()

            cxt:Opt("OPT_GET_PAID")
                :Dialog("DIALOG_GET_PAID")
                :CompleteQuest()
                :Travel()
        end)





QDEF:AddConvo("do_robbery")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("robbery_location") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find the caravan. Someone is arguing loudly with a very disgruntled oshnu.
                oshnu:
                    !left
                merchant:
                    !right
                    Get moving, you useless pile of ooze!
                oshnu:
                    !exit
                merchant:
                    !scared
                    Wait. Did you hear that? Who's there?
                player:
                    !left
                merchant:
                    !neutral
                    Oh. A grifter. What do <i>you</> want?
            ]],

            OPT_THREATEN = "Threaten the caravan",
            DIALOG_THREATEN = [[
                player:
                    It's not so much what <i>I</> want, as what <i>you</> want.
                    As in... do you want to live?
                    You're surrounded by bandits, friend. As a personal favor, I'm giving you one chance to walk away unscathed before my associates pop out of the Bog and cut you down.
            ]],
            DIALOG_INTIMIDATED = [[
                merchant:
                    !scared
                    Take it! Take all of it!
                    This job ain't worth dying for!
                    !exit
                * {merchant} and {merchant.hisher} entourage scuttle off, and {spree} comes out of hiding.
                spree:
                    !right
                    You appear to have a way with words, grifter.
                    Now, let's see what we've got here.
            ]],
            DIALOG_NOT_INTIMIDATED = [[
                merchant:
                    !angry
                    Yeah, right, grifter.
                    I'll believe that when I see it.
            ]],
            
            OPT_ATTACK = "Start the attack",
            
            DIALOG_ATTACK =  [[
                
                player:
                    !fight
                * You signal to the bandits.
                spree:
                    !right
                    !fight
                    Let's do this!
            ]],

        }
        :Fn(function(cxt) 
            
            local merchant = AgentUtil.GetFreeAgent("WEALTHY_MERCHANT")
            cxt:ReassignCastMember("merchant", merchant)
            
            local oshnu = AgentUtil.GetFreeAgent("MERCHANT_SNAIL")
            cxt:ReassignCastMember("oshnu", oshnu)

            local guard1_def = "JAKES_RUNNER"
            local guard2_def = "COMBAT_DRONE"

            local guard1 = AgentUtil.GetFreeAgent(guard1_def)
            cxt:ReassignCastMember("guard1", guard1)

            local guard2 = AgentUtil.GetFreeAgent(guard2_def)
            cxt:ReassignCastMember("guard2", guard2)
            
            cxt.enc.scratch.opfor = {merchant, oshnu, guard1, guard2}


            local extra_bandit = TheGame:GetGameState():AddAgent(Agent("BANDIT_GOON"))
            cxt.enc.scratch.bandits = {cxt.quest:GetCastMember("spree"), extra_bandit}

            cxt:Dialog("DIALOG_INTRO")

            cxt:TalkTo(merchant)

            cxt:RunLoop(function() 
                cxt:Opt("OPT_THREATEN")
                    :Dialog("DIALOG_THREATEN")
                    :Negotiation{flags = NEGOTIATION_FLAGS.INTIMIDATION}
                        :OnSuccess()
                            :Dialog("DIALOG_INTIMIDATED")
                            :Fn(function() 
                                for k,v in pairs(cxt.enc.scratch.opfor) do v:MoveToLimbo() end
                            end)
                            :GoTo("STATE_SPLIT_LOOT")
                        :OnFailure()
                            :Dialog("DIALOG_NOT_INTIMIDATED")
                
                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                        :Battle{ enemies = cxt.enc.scratch.opfor, allies = {cxt.quest:GetCastMember("spree"), extra_bandit} }
                            :OnWin()
                                :GoTo("STATE_FOUGHT")
            end)                        
        end)

        :State("STATE_FOUGHT")
            :Loc{
                
                DIALOG_WON_BATTLE_SPREE_DEAD = [[
                    right:
                        !exit
                    * {spree} is dead. You're not sure if this complicates or simplifies the plan.
                    * The caravan loot is all too big to transport. You gather up what shills you can find, and decide to avoid any Spark Baron patrols.
                ]],

                DIALOG_WON_BATTLE_SPREE_ALIVE_SURVIVORS = [[
                    spree:  
                        !right
                        !neutral
                    * {spree} mops sweat from {spree.hisher} brow.
                    spree:
                        That was a workout! This score better be worth it.
                    * The survivors of the caravan are cowering nearby. They start to back away, drawing {spree}'s attention.
                    spree:
                        !hips
                        Now now. They've seen us. We can't leave any witnesses, can we?
                        !point
                        You kill them, grifter. 
                        This was <i>your</> lead, after all.
                ]],
                OPT_REFUSE = "Refuse to kill the survivors",
                DIALOG_REFUSE = [[
                    player:
                        They're already defeated. I'm not going to be your butcher.
                    spree:
                        !thought
                        Well, in that case...
                        !fight
                    * The survivors, seeing how the wind is blowing, rally to your side.
                    player:
                        !fight
                ]],

                OPT_KILL = "Kill the survivors",
                DIALOG_KILL = [[
                    player:
                        Alright.
                        !exit
                    * You make short work of the survivors, under the watchful eye of {spree}.
                    player:
                        !left
                        It's done.
                ]],
                DIALOG_KILL_2 = [[
                    player:
                        !left
                    spree:
                        !hips
                        There. Now we're all on the same page.
                ]],
                OPT_CONVINCE = "Convince {spree} to let the survivors live",
                DIALOG_CONVINCE = [[
                    player:
                        Wait a minute, {spree}. That's not necessary.
                        They won't talk, if they know what's good for them. Nobody will believe them when they say they encountered Spree in the Bog.
                ]],
                DIALOG_CONVINCE_SUCCESS = [[
                    spree:
                        !thought
                        Maybe you're right.
                        Hesh, this Bog is making me paranoid.
                    * The survivors take the opportunity to run away.
                    spree:
                        Let's see what we've won.
                ]],
                DIALOG_CONVINCE_FAILURE = [[
                    spree:
                        !angry
                        Don't tell me how to run my crew!
                        Kill these weaklings, before I start to doubt your intentions, <i>grifter</>.
                ]],


                DIALOG_WON_BATTLE_SPREE_ALIVE_NO_SURVIVORS = [[
                    spree:  
                        !right
                    * {spree} mops sweat from {spree.hisher} brow. 
                    spree:
                        That was a workout! This score better be worth it.
                        At least we didn't leave witnesses.
                ]],
                DIALOG_WON_BATTLE_VS_SPREE_SURVIVORS = [[
                    * {spree} is dead, and the situation just got really complicated. 
                    * The remaining survivors high-tail it before you can address them.
                    * You quickly rummage through the remains of the caravan, and find a couple of shills.
                ]],
                DIALOG_WON_BATTLE_VS_SPREE_NO_SURVIVORS = [[
                    * You are surrounded by death and mistakes.
                    * You quickly rummage through the remains of the caravan, and find a couple of shills.
                ]],

            }
            :Fn(function(cxt) 
                if cxt.quest:GetCastMember("spree"):IsDead() then
                    cxt:Dialog("DIALOG_WON_BATTLE_SPREE_DEAD")
                    cxt.quest:Complete("do_robbery")
                    cxt.enc:GainMoney( math.floor( 150 + 100*math.random() ))
                    StateGraphUtil.AddLeaveLocation(cxt)
                else
                    cxt:TalkTo(cxt:GetCastMember("spree"))

                    local survivors = {}
                    cxt.enc.scratch.bandit_survivors = {}
                    
                    for k,v in pairs( cxt.enc.scratch.bandits ) do
                        if not v:IsDead() then
                            table.insert(cxt.enc.scratch.bandit_survivors, v)
                        end
                    end

                    for k,v in pairs( cxt.enc.scratch.opfor) do
                        if not v:IsDead() and v:IsSentient() then
                            table.insert(survivors, v)
                        end
                    end
                    
                    if #survivors == 0 then
                        cxt:Dialog("DIALOG_WON_BATTLE_SPREE_ALIVE_NO_SURVIVORS")
                        cxt:GoTo("STATE_SPLIT_LOOT")
                    else
                        cxt:Dialog("DIALOG_WON_BATTLE_SPREE_ALIVE_SURVIVORS")

                        cxt:RunLoop(function() 
                            
                            cxt:Opt("OPT_CONVINCE")
                                :Dialog("DIALOG_CONVINCE")
                                :Negotiation{}
                                    :OnSuccess()
                                        :Dialog("DIALOG_CONVINCE_SUCCESS")
                                        :Fn(function() 
                                            for k,v in pairs(survivors) do
                                                v:MoveToLimbo()
                                            end
                                        end)
                                        :GoTo("STATE_SPLIT_LOOT")
                                    :OnFailure()
                                        :Dialog("DIALOG_CONVINCE_FAILURE")

                            cxt:Opt("OPT_REFUSE")
                                :Dialog("DIALOG_REFUSE")
                                :Battle{
                                        no_oppo_limit = true,
                                        flags = BATTLE_FLAGS.NO_SURRENDER | BATTLE_FLAGS.SELF_DEFENCE,
                                        enemies = cxt.enc.scratch.bandit_survivors,
                                        allies = survivors,
                                    }
                                    :OnWin()
                                        :Fn(function(cxt) cxt:Dialog(AgentUtil.CountAlive(survivors) == 0 and "DIALOG_WON_BATTLE_VS_SPREE_NO_SURVIVORS" or "DIALOG_WON_BATTLE_VS_SPREE_SURVIVORS") end) 
                                        :ReceiveMoney(150, {no_scale = true})
                                        :CompleteQuest("do_robbery")
                                        :Travel()

                            cxt:Opt("OPT_KILL")
                                :Dialog("DIALOG_KILL")
                                :Fn(function() 
                                    for k,v in pairs(survivors) do
                                        v:Kill()
                                        AgentUtil.CommitMurder(v)
                                    end
                                end)
                                :GoTo("STATE_SPLIT_LOOT")
                        end)
                    end

                end
            end)


        :State("STATE_SPLIT_LOOT")
            :Loc{
                DIALOG_INTRO = [[
                    right:
                        !exit
                    * {spree} spends a long time rustling through the various packs and crates attached to the oshnu.
                    player:
                        !crossed
                ]],
                DIALOG_GOOD_HAUL = [[
                    * {spree.HeShe} is happy with what {spree.heshe} finds. 
                    spree:
                        !right
                        !happy
                        You weren't kidding about this haul!
                        We'll be eating the good part of the oshnu once we fence this stuff.
                    * {spree} fumbles around with a coin purse.
                    spree:
                        Here's your finder's fee.
                ]],

                OPT_TAKE_FEE = "Take the {1#money} 'fee'",
                DIALOG_TAKE_FEE = [[
                    spree:
                        !give                    
                    player:
                        !take
                        Thank you, {spree}. I'm glad this worked out to our mutual benefit.
                    spree:
                        I'll clean up here. Be seeing you.
                        !exit
                    * You wonder how happy {spree} will be with you when {spree.heshe} realizes what actually happened here.
                ]],

                OPT_ASK_FOR_CUT = "Ask for your agreed-upon share ({1#money})",
                DIALOG_ASK_FOR_CUT = [[
                    player:
                        !hips
                        Now wait a moment, friend. We had a deal, did we not?
                        I believe I'm owed a 20 percent cut, not a finder's fee.
                    spree:
                        Oh, you want to get legalistic with me about our larceny? 
                ]],
                DIALOG_ASK_FOR_CUT_FAIL = [[
                    spree:
                        You know what? I've changed my mind. I'm going to let you keep something much more valuable than money.
                        !cruel
                        Leave now, and you can keep breathing.
                        Take it or leave it, grifter.
                ]],

                OPT_LEAVE_WITH_NO_MONEY = "Let {spree} keep the money and leave",
                
                DIALOG_LEAVE_WITH_NO_MONEY = [[
                    player:
                        You win, 'friend'.
                        I'll see myself out.
                        !exit
                    spree:
                        That's what I thought!
                    * You leave {spree} to {spree.hisher} spoils, relishing the thought of {spree.himher} discovering your actual goals.
                ]],
                
                OPT_ATTACK = "Attack {spree} for the money",
                DIALOG_ATTACK = [[
                    player:
                        !fight
                        Or, perhaps... I'll just take it all!
                ]],
                DIALOG_FOUGHT_SPREE_DEAD = [[
                    * {spree} is dead. You gather up the what loot you can carry, and hurry off to meet up with {priest}.
                ]],
                DIALOG_FOUGHT_SPREE_ALIVE = [[
                    spree:
                        !injured
                        !right
                        Heshing bastard!
                    player:
                        !point                        
                        Next time, maybe don't try to pull a grift on a grifter.
                        !exit
                    * You gather up what you can carry, and hurry off to meet up with {priest}.
                ]],


                DIALOG_ASK_FOR_CUT_SUCCESS = [[
                    spree:
                        Alright, stow it. Here's your money.
                        !shrug
                        Can't blame me for trying.
                        !exit
                    * {spree} turns to the business of harvesting loot from the caravan. Your thoughts turn back to {priest}.
                ]],

                DIALOG_BAD_HAUL = [[
                    * {spree.HeShe} does not look pleased with {spree.hisher} findings.
                    spree:
                        !right
                        !angry
                        I thought you said this was going to be a rich haul!
                    player:
                        ...
                    spree:
                        Most of this stuff is just Bogshite, and it's all stamped with the Baron's imprint already.
                        I couldn't sell it if I tried.
                        !angry_accuse
                        What kind of scam are you pulling here, grifter?
                ]],
                OPT_MAKE_EXCUSES = "Make up an excuse",
                DIALOG_MAKE_EXCUSES = [[
                    player:
                        This is as big a surprise to me as it is to you!
                        My information said this caravan was loaded...
                ]],
                DIALOG_MAKE_EXCUSES_SUCCESS = [[
                    spree:
                        !neutral
                        Alright, I believe you.
                        Doesn't put any more shills in my pocket, but neither would killing you.
                        Ha!
                        Be seeing you around, grifter.
                        !exit
                    * {spree} heads back toward {spree.hisher} encampment. You hope that {priest} has finished {priest.hisher} task.
                ]],
                DIALOG_MAKE_EXCUSES_FAILURE = [[
                    spree:
                        Enough excuses.
                        !fight
                        I don't know what you're trying to do, but I'm going to take my frustrations out on your corpse.
                ]],
                OPT_DEFEND = "Defend yourself",
                DIALOG_DEFEND = [[
                    player:
                        !fight

                ]],
                DIALOG_DEFENDED_SPREE_DEAD = [[
                    * {spree} dies face-down in the Bog.
                    * You leave before the crayotes come to pick {spree.hisher} bones clean.
                ]],

                DIALOG_DEFENDED_SPREE_ALIVE = [[
                    spree:
                        !injured
                        !right
                        I'll never trust a grifter again!
                    player:
                        You should thank me for teaching you such a valuable lesson.
                        !exit
                    * You leave before any of {spree}'s friends arrive.
                ]],

            }
            :Fn(function(cxt) 
                cxt:TalkTo(cxt:GetCastMember("spree"))
                
                cxt:Dialog("DIALOG_INTRO")

                local fee = 50
                local cut = 150
                local full_amount = 250

                if math.random() < .5 then
                    cxt:Dialog("DIALOG_GOOD_HAUL")

                    cxt:Opt("OPT_TAKE_FEE", fee)
                        :Dialog("DIALOG_TAKE_FEE")
                        :ReceiveMoney(fee, {no_scale = true})
                        :CompleteQuest("do_robbery")
                        :Travel()

                    cxt:Opt("OPT_ASK_FOR_CUT", cut)
                        :Dialog("DIALOG_ASK_FOR_CUT")
                        :Negotiation{}
                            
                            :OnSuccess()
                                :Dialog("DIALOG_ASK_FOR_CUT_SUCCESS")
                                :ReceiveMoney(cut, {no_scale = true})
                                :CompleteQuest("do_robbery")
                                :Travel()
                            
                            :OnFailure()
                                :Dialog("DIALOG_ASK_FOR_CUT_FAIL")
                                :Fn(function() 
                                    cxt:Opt("OPT_ATTACK")
                                        :Dialog("DIALOG_ATTACK")
                                        :Battle{ enemies = cxt.enc.scratch.bandit_survivors}
                                            :OnWin()
                                                :Fn(function(cxt) cxt:Dialog(cxt.quest:GetCastMember("spree"):IsDead() and "DIALOG_FOUGHT_SPREE_DEAD" or "DIALOG_FOUGHT_SPREE_ALIVE") end)
                                                :ReceiveMoney(full_amount, {no_scale = true})
                                                :CompleteQuest("do_robbery")
                                                :Travel()

                                    cxt:Opt("OPT_LEAVE_WITH_NO_MONEY")
                                        :Dialog("DIALOG_LEAVE_WITH_NO_MONEY")
                                        :CompleteQuest("do_robbery")
                                        :Travel()

                                end)

                else
                    cxt:Dialog("DIALOG_BAD_HAUL")

                    cxt:Opt("OPT_MAKE_EXCUSES")
                        :Dialog("DIALOG_MAKE_EXCUSES")
                        :Negotiation{}
                            :OnSuccess()
                                :Dialog("DIALOG_MAKE_EXCUSES_SUCCESS")
                                :CompleteQuest("do_robbery")
                                :Travel()

                            
                            :OnFailure()
                                :Dialog("DIALOG_MAKE_EXCUSES_FAILURE")
                                :Fn(function() 
                                    cxt:Opt("OPT_DEFEND")
                                        :Dialog("DIALOG_DEFEND")
                                        :Battle{ enemies = cxt.enc.scratch.bandit_survivors, flags = BATTLE_FLAGS.SELF_DEFENCE}
                                            :OnWin()
                                                :Fn(function(cxt) cxt:Dialog(cxt.quest:GetCastMember("spree"):IsDead() and "DIALOG_DEFENDED_SPREE_DEAD" or "DIALOG_DEFENDED_SPREE_ALIVE") end) 
                                                :CompleteQuest("do_robbery")
                                                :Travel()
                                end)

                    
                    cxt:Opt("OPT_DEFEND")
                        :Dialog("DIALOG_DEFEND")
                        :Battle{ enemies = cxt.enc.scratch.bandit_survivors, flags = BATTLE_FLAGS.SELF_DEFENCE }
                            :OnWin()
                                :Fn(function(cxt) cxt:Dialog(cxt.quest:GetCastMember("spree"):IsDead() and "DIALOG_DEFENDED_SPREE_DEAD" or "DIALOG_DEFENDED_SPREE_ALIVE") end)
                                :CompleteQuest("do_robbery")
                                :Travel()
                end

            end)


QDEF:AddConvo("get_paid_by_priest")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("meeting_place") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find {priest} waiting for you.
                priest:
                    !right
                player:
                    !left
                    Did you get the documents?
                priest:
                    I did, thank you.
                    I hope {spree} wasn't too much trouble?
            ]],
            DIALOG_SPREE_DEAD = [[
                player:
                    You don't have to worry about {spree} any more.
                    {spree} is an ex-bandit.
                priest:
                    The less I know about that, the better.
                    Here's your payment.
                    !give
                player:
                    !take
                priest:
                    Until next time.
                    !exit
            ]],
            DIALOG_SPREE_ATTACKED = [[
                player:
                    {spree} and I, uh, 'came to terms'.
                priest:
                    Terms? Do you mean - 
                    No, I don't want to know the details.
                    Here's your payment.
                    !give
                player:
                    !take
                priest:
                    Until next time.
                    !exit
            ]],
            DIALOG_SPREE_ATTACK = [[
                player:
                    I successfully occupied {spree}. {spree} has no idea what just happened to h-
                * {spree} arrives, and charges forward with a fresh entourage of followers!
                spree:
                    !right
                    !angry
                    YOU LYING, THIEVING, NO-ACCOUNT CHEAT!
                * Something tells you that {spree} is in no mood to talk things through this time.
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                spree:
                    !fight
                priest:
                    !fight
                    !left
                    Keep me alive, grifter! I can't pay you if I'm dead!
            ]],

            DIALOG_DEFENDED_PRIEST_ALIVE_SPREE_DEAD = [[
                player:
                    !left
                    !neutral                
                priest:
                    !right
                    Good work, grifter. At least now I don't have to worry about this coming back on me.
                    Here's your payment.
            ]],

            DIALOG_DEFENDED_PRIEST_DEAD_SPREE_DEAD = [[
                player:
                    !left
                * You are surrounded by corpses. It is true that {priest} can't pay you, but there's nothing stopping you from helping yourself to the contents of {priest.hisher} pockets.
            ]],

            DIALOG_DEFENDED_PRIEST_ALIVE_SPREE_ALIVE = [[
                player:
                    !left
                spree:
                    !right
                    !injured
                    Hesh take you!
                player:
                    Get out of here before I reconsider my leniency.
                spree:
                    !exit
                * You watch {spree} limp away.
                priest:
                    !right
                    I'm not sure that was the right play. Now we're both going to have to keep one eye over our shoulder.
                    But here's your payment, as promised.
            ]],
            DIALOG_DEFENDED_PRIEST_DEAD_SPREE_ALIVE = [[
                player:
                    !left
                spree:
                    !right
                    !injured
                    Hesh take you!
                player:
                    You're welcome, for sparing your life.
                spree:
                    !exit
                * You help yourself to the contents of {priest}'s pockets.
            ]],

            PRIEST_LEAVE = [[
                priest:
                    !give
                player:
                    !take
                priest:
                    Farewell.
                    !exit
            ]],

            NO_PRIEST_LEAVE = [[
                * You decide to leave before you are out-competed by other, larger scavengers.
            ]],


            OPT_REPORT = "Tell {agent} what happened",

            OPT_GET_PAID = "Get paid",
        }
        :Fn(function(cxt) 
            cxt:TalkTo(cxt.quest:GetCastMember("priest"))
            cxt:Dialog("DIALOG_INTRO")

            if cxt.quest:GetCastMember("spree"):IsDead() then
                cxt:Opt("OPT_REPORT")
                    :Dialog("DIALOG_SPREE_DEAD")
                    :CompleteQuest()
                    :Travel()

            elseif cxt.quest:GetCastMember("spree"):HasAspect("intimidated") then
                
                cxt:Opt("OPT_REPORT")
                    :Dialog("DIALOG_SPREE_ATTACKED")
                    :CompleteQuest()
                    :Travel()

            else
                local extra_bandit = TheGame:GetGameState():AddAgent(Agent("BANDIT_GOON2"))
                local flead = TheGame:GetGameState():AddAgent(Agent("FLEAD"))
                
                local sprees = {cxt.quest:GetCastMember("spree"), extra_bandit, flead}
                cxt:Opt("OPT_REPORT")
                    :Dialog("DIALOG_SPREE_ATTACK")
                    
                    :Fn(function() 
                        cxt.quest:GetCastMember("spree"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("figured_out_betrayal"))
                        cxt:Opt("OPT_DEFEND")
                            :Dialog("DIALOG_DEFEND")
                            :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    allies = {cxt.quest:GetCastMember("priest")},
                                    enemies = sprees,
                                }
                                :OnWin()
                                    :Fn(function() 
                                        if not cxt.quest:GetCastMember("priest"):IsDead() then
                                            cxt.quest:GetCastMember("priest"):OpinionEvent(OPINION.DROP_BANDIT_BABYSITTER_defended_them)
                                        end
                                        if cxt.quest:GetCastMember("spree"):IsDead() then
                                            if cxt.quest:GetCastMember("priest"):IsDead() then
                                                cxt:Dialog("DIALOG_DEFENDED_PRIEST_DEAD_SPREE_DEAD")
                                            else
                                                cxt:Dialog("DIALOG_DEFENDED_PRIEST_ALIVE_SPREE_DEAD")
                                            end
                                        else
                                            if cxt.quest:GetCastMember("priest"):IsDead() then
                                                cxt:Dialog("DIALOG_DEFENDED_PRIEST_DEAD_SPREE_ALIVE")
                                            else
                                                cxt:Dialog("DIALOG_DEFENDED_PRIEST_ALIVE_SPREE_ALIVE")
                                            end
                                        end
                                        
                                        cxt:Opt("OPT_GET_PAID")
                                            :CompleteQuest()
                                            :Dialog(cxt.quest:GetCastMember("priest"):IsDead() and "NO_PRIEST_LEAVE" or "PRIEST_LEAVE")
                                            :Travel()
                                    end)
                    end)
            end                
        end)


QDEF:AddIntro[[
    * The printed instructions read:
    * Documents that are embarrassing to the Cult of Hesh have been stolen and are currently in the possession of an exiled Spree raiding party operating in the Bog.
    * Make contact with {priest}, a Cult Priest in the region, and coordinate to recover the documents.
]]



