local QDEF = QuestDef.Define{
    title = "Special Delivery",
    qtype = QTYPE.CONTRACT,
    --faction = "SPARK_BARONS",
    rank = {1, 4},
    desc = [[Help an Admiralty Clerk smuggle weapons into the bog.]],
    icon = engine.asset.Texture("icons/quests/drop_smuggle_in_admiralty_weapons.tex"),
    tags = {"dead_drop"},
    reward_size = QREWARDSIZE.LARGER,
    focus = QUEST_FOCUS.BALANCED,

    collect_agent_locations = function(quest, t)
    end,

    on_start = function(quest)
        quest:Activate("get_admiralty_in")
    end,

    on_destroy = function(quest)
        if quest:GetCastMember("cargo_snail") and quest:GetCastMember("cargo_snail"):IsInPlayerParty() then
            quest:GetCastMember("cargo_snail"):Dismiss()
        end

        if quest:GetCastMember("admiralty") and quest:GetCastMember("admiralty"):IsInPlayerParty() then
            quest:GetCastMember("admiralty"):Dismiss()
        end
    end,

    avoids = 
    {
        --{"handler", "target"},
    },
}


:AddCastByAlias{
  cast_id = "hebbel",
  alias = "HEBBEL",
}

:AddLocationCast{
    cast_id = "gate",
    cast_fn = function(quest, t)
        table.insert( t, TheGame:GetGameState():GetLocation("GB_GATE"))
    end,
}

:AddLocationCast{
    cast_id = "market",
    cast_fn = function(quest, t)
        table.insert( t, TheGame:GetGameState():GetLocation("GB_BLACK_MARKET"))
    end,
}

:AddCast{
    cast_id = "admiralty",
    cast_fn = function( quest, t )
        table.insert(t, quest:CreateSkinnedAgent( "ADMIRALTY_CLERK" ))
    end,
}

:AddCast{
    cast_id = "cargo_snail",
    no_validation = true,
    cast_fn = function( quest, t )
        table.insert(t, quest:CreateSkinnedAgent( "MERCHANT_SNAIL" ))
    end,
}

:AddLocationDefs{
    SECLUDED_SPOT = {
        name = "Secluded Location",
        map_tags = {"forest"},
        plax = "EXT_Bog_Forest_01",
        indoors = false,
    },
}

:AddLocationCast{
    cast_id = "secluded_spot",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("SECLUDED_SPOT"))
    end,
}


:AddObjective{
    id = "get_admiralty_in",
    title = "Get the cargo through customs",
    --desc = [[{handler} has a reward for you.]],
    on_complete = function(quest)
        if not quest:GetCastMember("admiralty"):IsDead() then
            quest:GetCastMember("admiralty"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
        end

        quest:GetCastMember("cargo_snail"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
        quest:Activate("go_to_market")
    end,
    mark = {"gate"},
}

:AddObjective{
    id = "go_to_market",
    title = "Escort the cargo to {market}",
    --desc = [[{handler} has a reward for you.]],
    mark = {"market"},

    on_complete = function(quest)
        if quest:GetCastMember("cargo_snail") and quest:GetCastMember("cargo_snail"):IsInPlayerParty() then
            quest:GetCastMember("cargo_snail"):Dismiss()
        end
    end,
}

:AddObjective{
    id = "eliminate_admiralty",
    title = "Eliminate {admiralty} at a secluded location",
    --desc = [[{handler} has a reward for you.]],
    mark = {"secluded_spot"},
}

:AddObjective{
    id = "return_to_hebbel",
    title = "Return to {hebbel}",
    --desc = [[{handler} has a reward for you.]],
    mark = {"hebbel"},
}

:AddOpinionEvents{
    let_them_go = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Let them disappear",
    },
}
    

QDEF:AddConvo( "get_admiralty_in" )
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("gate") end)
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !right
            admiralty:
                !left
            * You find {admiralty} in animated conversation with the gate guard.
            admiralty:
                !angry
                I have the proper paperwork! You need to let me in.
            agent:
                I told you already. That authorization is out of date.
            admiralty:
                As of when?
            agent:
                As of now. Because I said so.
                !happy
                Look, I don't make the rules, I just get to enforce them. Get the right stamps, and you can go through.
            admiralty:
                !exit
            * {admiralty} returns to {admiralty.hisher} oshnu to sort through reams of documents. {agent} notices you.
            player:
                !left
            agent:
                !crossed
                What the Hesh are you looking at?
            
            *** You need to help {admiralty} get past {agent}.

        ]],
        OPT_BRIBE = "Pay for a new 'authorization'",
        DIALOG_BRIBE = [[
            player:
                I couldn't help but hear that this unfortunate soul's paperwork is out of sorts.
            agent:
                Yeah, so?
            player:
                !thought
                I would like to offer a small donation to... 
                ...to help you reconsider the, uh, validity of... 
            agent:
                Oh, a bribe.
                Yeah, I'll take that.
                !take
            player:
                I'm glad we came to this understanding.
            agent:
                !exit
            * {admiralty} eyes you skeptically.
            admiralty:
                !right
                You're covering that cost yourself, grifter, as per the contract I signed.
            player:
                Of course. I wouldn't dream of charging you for the incidentals.
            admiralty:
                Good. Let's be on, then.
            * {admiralty} grabs the reins of the oshnu, and you set out.
        ]],

        OPT_NEGOTIATE = "Convince {agent} to let {admiralty} through",
        DIALOG_NEGOTIATE = [[
            player:
                I'd be surprised if the paperwork were wrong. The Admiralty is very diligent about such things.
                Here, let me take a look.
                !take
        ]],
        DIALOG_NEGOTIATE_SUCCESS = [[
            agent:
                Yeah, you are right. I <i>did</> mis-read the second clause of subsection 37 of the third page.
                You can go through, {admiralty}.
                !exit
            * {admiralty} is all smiles.
            admiralty:  
                !right
                !happy
                Impressive work, grifter. I assume you're my contact here?
            player:
                Indeed. I'm your silver tongue or silver bullet, whichever the situation requires.
            admiralty:
                Let's be on, then. The sooner I'm back in Murder Bay, the better.
            * {admiralty} grabs the reins of the oshnu, and you set out.
        ]],
        DIALOG_NEGOTIATE_FAIL = [[
            agent:
                !angry
                Don't tell me how to do my job!
                Entry to the Bog is up to the discretion of the gate agent. 
                !point                
                And that's me, so even if I'm wrong... I'm right.
        ]],
        OPT_ATTACK = "Force your way through",
        DIALOG_ATTACK = [[
            player:
                I hate to press the issue, but...
                !fight
                That oshnu's getting through one way or another.
        ]],
        DIALOG_ADMIRALTY_KILLED = [[
            agent:
                !exit
            * You win the battle, but {admiralty} has bled out. 
            cargo_snail:
                !right
            player:
                Well, looks like it's just you and me, {cargo_snail}.
            * You take the reins of the oshnu.
        ]],
        DIALOG_BARON_KILLED = [[
            admiralty:
                !right
                You killed {agent.himher}!
            player:
                !neutral
                I think legally, <i>we</> killed {agent.himher}.
                The name's {player}, by the way. I am your escort through the Bog.
            admiralty:
                Wonderful. Can we try to keep the explosions to a minimum for the rest of the trip?
            player:
                You're the boss.
            * {admiralty} grabs the reins of the oshnu, and you set out.
        ]],
        DIALOG_BARON_SPARED = [[
            agent:
                !injured
                Just go through! I won't stand in your way.
            admiralty:
                !right
                Thanks for the help, friend. But if you're expecting a reward-
            player:
                !neutral
                We are not friends, at least not yet. I'm your hired guard for this little excursion.
            admiralty:
                Ah. Of course. Let's get this over with, then.
            player:
                Lead the way.
            * {admiralty} grabs the reins of the oshnu, and you set out.
        ]]


    }
    :SetLooping()
    :Fn(function(cxt)
        local BRIBE_AMT = 100

        if cxt:FirstLoop() then
            cxt.quest:GetCastMember("admiralty"):MoveToLocation(cxt.location)
            cxt.quest:GetCastMember("cargo_snail"):MoveToLocation(cxt.location)

            cxt.location:DoHiring() --just to make sure the proprietor's not dead or something.
            -- ccc: ^^^btw, I notice this line doesn't actually help anything because it doesn't send the hired agents to work or give them the appropriate role.
            -- nonetheless, I don't want to touch this house of cards.

            local prop = cxt.location:GetAgentByRole(CHARACTER_ROLES.PROPRIETOR)
            cxt.enc:SetPrimaryCast(prop)
            cxt:Dialog("DIALOG_INTRO")
        end

        cxt:Opt("OPT_BRIBE")
            :Dialog("DIALOG_BRIBE")
            :DeliverMoney(BRIBE_AMT)
            :CompleteQuest("get_admiralty_in")
            :Travel()

        cxt:Opt("OPT_NEGOTIATE")
            :Dialog("DIALOG_NEGOTIATE")
            :Negotiation{
                    helpers = {"admiralty"},
                }
                :OnSuccess()
                    :Dialog("DIALOG_NEGOTIATE_SUCCESS")
                    :CompleteQuest("get_admiralty_in")
                    :Travel()
                :OnFailure()
                    :Dialog("DIALOG_NEGOTIATE_FAIL")

        cxt:Opt("OPT_ATTACK")
            :Dialog("DIALOG_ATTACK")
            :Battle {
                        allies = {cxt.quest:GetCastMember("admiralty")},
                        noncombatants = {cxt.quest:GetCastMember("cargo_snail")}
                    }
                    :OnWin()
                        :Fn(function() 
                            if cxt.quest:GetCastMember("admiralty"):IsDead() then
                                cxt:Dialog("DIALOG_ADMIRALTY_KILLED")
                            else
                                cxt:Dialog( cxt:GetAgent():IsDead() and "DIALOG_BARON_KILLED" or "DIALOG_BARON_SPARED" ) 
                            end
                        end)
                        :CompleteQuest("get_admiralty_in")
                        :Travel()

    end)



QDEF:AddConvo( "go_to_market" )
    :ConfrontState("CONFRONT", function(cxt) return not cxt.quest.param.has_done_rise_interrupt and cxt.location:HasTag("in_transit")  end)
    :Loc{
        DIALOG_INTRO = [[
            * The usual cacophony of the Bog-life falls silent, and you realize you're not alone.
            player:
                !left
                I know you're there. Why not be civilized about this and show yourself so we can talk this through like reasonable adults?
                !hips
            * You hear whispers, and the underbrush rustles. Eventually, a rag-tag band of Rise emerge.
            agent:
                !right
                Halt! In the name of the Rise.
                We want the oshnu. Hand it over, grifter, and we'll let you walk.
            *** The Rise is demanding you turn over the oshnu.                
        ]],
        
        DIALOG_INTRO_AD = [[
            * The usual cacophony of the Bog-life falls silent, and you realize you're being followed.
            player:
                !left
                I know you're there. Why not be civilized about this and show yourself so we can talk this through like reasonable adults?
                !hips
            * You hear whispers, and the underbrush rustles. Eventually, a rag-tag band of Rise emerge.
            agent:
                !right
                Halt! In the name of the Rise.
                We're here to commandeer this shipment, for the good of the cause.
                And we want {admiralty}, too. {admiralty.HeShe} will fetch a good ransom from {admiralty.hisher} Admiralty masters.
            *** The Rise is demanding you turn over {admiralty} and the oshnu.
        ]],

        OPT_NEGOTIATE = "Scare {agent} off",
        DIALOG_NEGOTIATE = [[
            player:
                You don't know who you're dealing with, do you?
                !threaten
                I almost feel sorry for you.
        ]],
        DIALOG_NEGOTIATE_SUCCESS = [[
            agent:
                !scared
                This ain't worth the trouble.
                Rise! Away!
                !exit
            * The Rise agents melt back into the Bog. 
            cargo_snail:
                !right
            player:
                !happy                
                Who's a scary snail?
                !point
                You are! That's who.
        ]],
        DIALOG_NEGOTIATE_SUCCESS_AD = [[
            agent:
                !scared
                This ain't worth the trouble.
                Rise! Away!
                !exit
            * The Rise agents melt back into the Bog. 
            admiralty:
                !right
                Do you think we could have taken them?
            player:
                !shrug
                Don't know. 
                !happy
                Don't need to know, either, so long as <i>they</> think we could have.
        ]],
        DIALOG_NEGOTIATE_FAIL = [[
            agent:
                Ha! You aren't scary, old man.
                We're the Rise! We aren't afraid of anything!
        ]],

        OPT_HAND_OVER_CARGO = "Hand over the cargo",
        OPT_HAND_OVER_CARGO_AND_ADMIRALTY = "Hand over the cargo and {admiralty}",
        DIALOG_HAND_OVER_CARGO_AD = [[
            * You size up the opposition, and carefully weigh your options.
            player:
                !permit
                They're all yours.
            agent:
                !surprised
                Really?
                I mean, of course! No-one dares stand against the might of the Rise!
            admiralty:
                !right
                !angry
                You... grifting bastard!
            player:
                !shrug
                Apologies, {admiralty}, but a dead grifter can grift no more.
            admiralty:
                !exit
            * The Rise lead the oshnu and the admiralty officer away, leaving you alone with your failure.
        ]],
        DIALOG_HAND_OVER_CARGO_ALONE = [[
            * You size up the opposition, and carefully weigh your options.
            player:
                !permit
                Take it.
            agent:
                !surprised
                Really?
                I mean, of course! No-one dares stand against the might of the Rise!
            * The Rise lead the oshnu away, leaving you alone with your failure.
        ]],

        OPT_DEFEND = "Defend the cargo",
        DIALOG_DEFEND = [[
            player:
                !sigh
                Sigh.
                !fight
                Let's get this over with, then.
        ]],
        DIALOG_ADMIRALTY_KILLED = [[
            right:
                !exit
            * {admiralty} didn't make it. You grab the reins of the oshnu, and carry on without {admiralty.himher}.
        ]],
        DIALOG_WON_BATTLE = [[
            right:
                !exit
            * You move on, before more Rise can arrive.
        ]],


    }
    :SetLooping()
    :Fn(function(cxt)
        
        if cxt:FirstLoop() then
            cxt.quest.param.has_done_rise_interrupt = true
            local rise_guys = CreateCombatParty("RISE_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt.enc:SetPrimaryCast(rise_guys[1])
            cxt:Dialog(cxt.quest:GetCastMember("admiralty"):IsInPlayerParty() and "DIALOG_INTRO_AD" or "DIALOG_INTRO")
        end

        if cxt.quest:GetCastMember("admiralty"):IsInPlayerParty() then
            cxt:Opt("OPT_HAND_OVER_CARGO_AND_ADMIRALTY")
                :Dialog("DIALOG_HAND_OVER_CARGO_AD")
                :ReceiveOpinion(OPINION.BETRAYED, nil, cxt.quest:GetCastMember("admiralty"))
                :Fn(function() cxt.quest:GetCastMember("admiralty"):Retire() end)
                :FailQuest()
                :Travel()
        else
            cxt:Opt("OPT_HAND_OVER_CARGO")
                :Dialog("DIALOG_HAND_OVER_CARGO_ALONE")
                :FailQuest()
                :Travel()
        end

        cxt:Opt("OPT_NEGOTIATE")
            :Dialog("DIALOG_NEGOTIATE")
            :Negotiation{
                    helpers = {"admiralty"},
                    flags = NEGOTIATION_FLAGS.INTIMIDATION,
                }
                :OnSuccess()
                    :Dialog(cxt.quest:GetCastMember("admiralty"):IsInPlayerParty() and "DIALOG_NEGOTIATE_SUCCESS_AD" or "DIALOG_NEGOTIATE_SUCCESS")
                    :Travel()

                :OnFailure()
                    :Dialog("DIALOG_NEGOTIATE_FAIL")

        local admiralty_was_dead = cxt.quest:GetCastMember("admiralty"):IsDead()
        
        cxt:Opt("OPT_DEFEND")
            :Dialog("DIALOG_DEFEND")
            :Battle {
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        allies = {cxt.quest:GetCastMember("admiralty")},
                        noncombatants = {cxt.quest:GetCastMember("cargo_snail")}
                    }
                    :OnWin()
                        :Fn(function() 
                            if not admiralty_was_dead and cxt.quest:GetCastMember("admiralty"):IsDead() then
                                cxt:Dialog("DIALOG_ADMIRALTY_KILLED")
                            else
                                cxt:Dialog( "DIALOG_WON_BATTLE" ) 
                            end
                        end)
                        :Travel()
    end)


QDEF:AddConvo( "go_to_market" )
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("market")  end)
    :Loc{
        DIALOG_INTRO_ALONE = [[
            * You approach the {market}.
            agent:
                !right
                Stop right there - what's with the oshnu?
            player:
                !left
                This is a shipment from the Admiralty. I've been sent to deliver it to you.
            agent:
                Where's {admiralty}? What are you trying to pull?

        ]],
        
        DIALOG_INTRO_AD = [[
            * You approach the {market}. {admiralty} pulls you aside.
            player:
                !left
            admiralty:
                !right
                Let me do the talking, alright? You're just the muscle.
                !exit
            player:
                !exit
            * The Jakes spot you.
            agent:
                !right
                Who's there?
            admiralty:
                !left
                Who do you think it is? 
                It's the shipment you ordered, you damned Jake. The admiralty weapons?
            agent:
                Keep it down! I think there might be a couple of Barons in the Bog who didn't hear that.
            admiralty:
                Your little conflict is your business. Here are the weapons. You have the payment.
            agent:
                Right. Here's for you, and here's for the grifter. Just like the contract stated.
                !give
                We're done, then?
            admiralty:
                Yeah, we're done.
            admiralty:
                !exit
            agent:
                !exit
            * The Jakes start unloading the oshnu. {admiralty} turns to you.
            admiralty:
                !right
            player:
                !left
            admiralty:
                Damned Jakes. Can't trust 'em.
        ]],

        OPT_NEGOTIATE_SOLO_DEAL = "Convince {agent} to go through with the deal",
        
        DIALOG_NEGOTIATE_SOLO_DEAL = [[
            player:
                {admiralty} couldn't make it - {admiralty.heshe} got caught up at the border crossing.
                {admiralty.HeShe} sent me along instead.
        ]],

        DIALOG_NEGOTIATE_SOLO_DEAL_SUCCESS = [[
            agent:
                You know what, I don't really care. So long as I get the goods and can sell them, I'm happy.
        ]],

        DIALOG_NEGOTIATE_SOLO_DEAL_FAILURE = [[
            agent:
                Something here doesn't quite add up. I'll take the shipment, but I'm going to have to follow up with the Admiralty.

        ]],

        OPT_FORGOE_BONUS = "Let it go",
        DIALOG_FORGOE_BONUS = [[
            player:
                Are you ready to go?
            admiralty:
                Yes. Let's get out of here before my clothes start smelling like Bog.

        ]],
        OPT_NEGOTIATE_A_BONUS = "Demand a bonus payment",
        
        DIALOG_NEGOTIATE_A_BONUS = [[
            player:
                What was that about <i>my</> cut?
        ]],

        DIALOG_NEGOTIATE_A_BONUS_SUCCESS = [[
            admiralty:
                I was going to give it to you after.
                But here. Take it now, I guess. You did get me here in one piece.
                !give
            player:
                !take
            admiralty:
                Let's get out of here. I don't want to spend any more time in the Bog than I have to.
        ]],

        DIALOG_NEGOTIATE_A_BONUS_FAILURE = [[
            admiralty:
                I signed a contract. You'll get your promised payment when the job is done, and not a shill more.
                Now take me back to the gate.
        ]],
    }
    :SetLooping()
    :Fn(function(cxt)
        
        if cxt:FirstLoop() then
            cxt.quest:Complete("go_to_market")
            cxt.location:DoHiring()
            local prop = cxt.location:GetAgentByRole(CHARACTER_ROLES.PROPRIETOR)
            cxt.enc:SetPrimaryCast(prop)
            cxt:Dialog(cxt.quest:GetCastMember("admiralty"):IsInPlayerParty() and "DIALOG_INTRO_AD" or "DIALOG_INTRO_ALONE")
        end

        if cxt.quest:GetCastMember("admiralty"):IsInPlayerParty() then
            local BONUS_PAYMENT = 100

            cxt:Opt("OPT_NEGOTIATE_A_BONUS")
                :Dialog("DIALOG_NEGOTIATE_A_BONUS")
                :Negotiation{target_agent = cxt.quest:GetCastMember("admiralty")}
                    :OnSuccess()
                        :Dialog("DIALOG_NEGOTIATE_A_BONUS_SUCCESS")
                        :ReceiveMoney(BONUS_PAYMENT)
                        :ActivateQuest("eliminate_admiralty")
                        :Travel()
                    :OnFailure()
                        :Dialog("DIALOG_NEGOTIATE_A_BONUS_FAILURE")
                        :ActivateQuest("eliminate_admiralty")
                        :Travel()
            
            cxt:Opt("OPT_FORGOE_BONUS")
                :Dialog("DIALOG_FORGOE_BONUS")
                :ActivateQuest("eliminate_admiralty")
                :Travel()

        else
            cxt:Opt("OPT_NEGOTIATE_SOLO_DEAL")
                :Dialog("DIALOG_NEGOTIATE_SOLO_DEAL")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_NEGOTIATE_SOLO_DEAL_SUCCESS")
                        :Fn(function() cxt.quest.param.good_solo_deal = true end)
                        :ActivateQuest("return_to_hebbel")
                        :Travel()
                    :OnFailure()
                        :Dialog("DIALOG_NEGOTIATE_SOLO_DEAL_FAILURE")
                        :Fn(function() cxt.quest.param.bad_solo_deal = true end)
                        :ActivateQuest("return_to_hebbel")
                        :Travel()
        end

    end)


QDEF:AddConvo( "eliminate_admiralty" )
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("secluded_spot")  end)
    :Loc{
        DIALOG_INTRO = [[
            * You are in the middle of nowhere.
            admiralty:
                !right
                Is this the way to the gate? Did you get us lost?
            player:
                !left
                Not lost. Secluded.
                My instructions are to eliminate you. It seems your employers don't want to leave any loose ends.
            admiralty:
                !scared
                They <i>what?</> Oh Hesh.
                That's why they gave me this post, isn't it?
                You don't have to do this. I can disappear! 
                I'll go to Docktown! No one will ever hear from me again!
            *** {admiralty} is begging for {admiralty.hisher} life. You'll have to convince {hebbel} that {admiralty.heshe} is really dead if you accept this offer.
        ]],
        
        OPT_ACCEPT_DEAL = "Let {admiralty} go",
        DIALOG_ACCEPT_DEAL = [[
            player:
                Fine. But you better not show your face here or in Murder Bay ever again.
            agent:
                I won't! I promise!
                Thank you!
                !exit
        ]],
        OPT_ATTACK_ADMIRALTY = "Attack {admiralty}",
        DIALOG_ATTACK_ADMIRALTY = [[
            player:
                !fight
                My apologies, but I am nothing if not a professional.
        ]],
        DIALOG_KILLED_ADMIRALTY = [[
            * {admiralty} is dead. You take {admiralty.hisher} badge and give the rest to the Bog, which happily accepts.
        ]],
    }
    :Fn(function(cxt)
        cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("admiralty"))
        cxt:Dialog("DIALOG_INTRO")
        
        cxt:Opt("OPT_ACCEPT_DEAL")
            :Dialog("DIALOG_ACCEPT_DEAL")
            :Fn(function() cxt.quest.param.admiralty_in_hiding = true end)
            :Fn(function() cxt.quest:GetCastMember("admiralty"):Retire() end)
            :ReceiveOpinion("let_them_go")
            :CompleteQuest("eliminate_admiralty")
            :ActivateQuest("return_to_hebbel")
            :Travel()
        
        cxt:Opt("OPT_ATTACK_ADMIRALTY")
            :Dialog("DIALOG_ATTACK_ADMIRALTY")
            :Battle{ flags = BATTLE_FLAGS.NO_SURRENDER | BATTLE_FLAGS.ISOLATED }
                :OnWin()
                    :Dialog("DIALOG_KILLED_ADMIRALTY")
                    :CompleteQuest("eliminate_admiralty")
                    :ActivateQuest("return_to_hebbel")
                    :Travel()
    end)



QDEF:AddConvo("return_to_hebbel", "hebbel")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        
        OPT_GIVE_PROOF = "Give {agent} {admiralty}'s badge",
        DIALOG_GIVE_PROOF = [[
            player:
                Here. The job is done.
                !give
            hebbel:
                !take
                I'll make sure this gets to the right person.
                !shrug
                I'm sure someone up the chain will care about it.
        ]],
        
        DIALOG_GIVE_PROOF_LYING = [[
            player:
                Here's {admiralty}'s badge. They are dead.
                !give
            hebbel:
                !dubious
                Uh huh. And you're telling me this is the truth?

        ]],
        DIALOG_PROOF_FAILURE = [[
            hebbel:
                Nice try. But I'm not going to accept that.
                I don't personally care, of course, but I try to pass that garbage up the chain, it'll reflect poorly on me.
        ]],
        DIALOG_PROOF_SUCCESS = [[
            hebbel:
                Yeah, yeah. Stop talking already - that's good enough for me.
        ]],
        OPT_BAD_DEAL = "Tell {hebbel} about the deal",
        DIALOG_BAD_DEAL = [[
            player:
                So the Jakes have the weapons, but they're not completely happy with the situation...
        ]],
        DIALOG_BAD_DEAL_SUCCESS = [[
            hebbel:
                !shrug
                That's good enough. {admiralty} is dead and the weapons are where they need to be. I'm sure that's all the bosses really cared about.
        ]],
        DIALOG_BAD_DEAL_FAILURE = [[
            hebbel:
                Not good enough. You're paid to fulfill the contract as written, not as you see fit.
                I can't pay you for this.
                !shrug
                Nothing personal.
        ]]

    }
    :Hub(function(cxt) 
            
        if cxt.quest.param.bad_solo_deal then
            
            cxt:Opt("OPT_BAD_DEAL")
                :SetQuestMark()
                :Dialog("DIALOG_BAD_DEAL")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_BAD_DEAL_SUCCESS")
                        :CompleteQuest()
                    :OnFailure()
                        :Dialog("DIALOG_BAD_DEAL_FAILURE")
                        :FailQuest()

        
        elseif cxt.quest.param.admiralty_in_hiding then

            cxt:Opt("OPT_GIVE_PROOF")
                :SetQuestMark()
                :Dialog("DIALOG_GIVE_PROOF_LYING")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_PROOF_SUCCESS")
                        :CompleteQuest()
                    :OnFailure()
                        :Dialog("DIALOG_PROOF_FAILURE")
                        :FailQuest()
        else
            cxt:Opt("OPT_GIVE_PROOF")
                :SetQuestMark()
                :Dialog("DIALOG_GIVE_PROOF")
                :CompleteQuest()
        end

    end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            * The printed instructions read:
            * Proceed to {gate} and facilitate the entry of {admiralty} to the bog, accompanied by {admiralty.hisher} cargo oshnu.
            * Escort {admiralty} north to {market}, ensuring safe arrival of the cargo, and completion of business transaction.
            * After transaction, <#PENALTY>eliminate {admiralty}, <b>with discretion</>.</>
            * Deliver proof of completion to {hebbel} for payment.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


