local QDEF = QuestDef.Define{
    title = "Ambush Predator",
    qtype = QTYPE.CONTRACT,
    faction = "RISE",
    tags = {"nasty"},
    rank = {1, 5},
    desc = [[The Barons are moving some spark through the Bog at night. Go help a Rise cell capture it.]],
    icon = engine.asset.Texture("icons/quests/rise_ambush_a_baron.tex"),
    focus = QUEST_FOCUS.COMBAT,
    on_start = function(quest)
        quest:Activate("go_to_ambush")
    end,
}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddLocationDefs{
    AMBUSH_SITE = {
        name = "Rise Ambush Site",
        map_tags = {"bog"},
        plax = "EXT_Bog_Outcrop_01",
        indoors = false,
    },
}

:AddLocationCast{
    cast_id = "ambush_site",
    cast_fn = function(quest, t)
        table.insert(t, quest:SpawnTempLocation("AMBUSH_SITE"))
    end,
}



:AddObjective{
    id = "go_to_ambush",
    title = [[Go to {ambush_site}]],
    desc = "The Baron convoy is scheduled to pass through this stretch of the bog soon. Be there first.",
    mark = { "ambush_site" },
    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[Go tell {giver} that the ambush was successful.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}

:AddOpinionEvents{
    helped_with_ambush =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Helped them pull off an ambush",
    },
}


QDEF:AddConvo("go_to_ambush")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("ambush_site") end)
        :Loc{
            OPT_DO_THE_AMBUSH = "Start the ambush",
            DIALOG_DO_THE_AMBUSH = [[
                player:
                    Let's finish this.
                robot:
                    !right
                    !fight
                    FINISH IT
                rebel:
                    !right
                    Whoa, be careful what you say around this thing!
                    !exit
                player:
                    !exit
                * You hide and wait for your target.
            ]],
            DIALOG_INTRO = [[
                * You find the ambush spot, and the Rise cell that's waiting there.
                robot:
                    !right
                    INTRUDER DETECTED
                rebel:
                    !right
                    Wait! Who is that?
                player:
                    !left
                    {giver} sent me. Said you need some help.
                rebel:
                    Oh man, do I ever. We're a bit, uh, understaffed here.
                * {rebel} gestures at a fellow Rise member's corpse stuffed unceremoniously behind a bush.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.quest.param.rebel = TheGame:GetGameState():AddSkinnedAgent( "RISE_REBEL" )
                cxt.quest.param.robot = TheGame:GetGameState():AddSkinnedAgent( "RISE_AUTOMECH" )
                
                cxt:ReassignCastMember("rebel", cxt.quest.param.rebel)
                cxt:ReassignCastMember("robot", cxt.quest.param.robot)
                cxt.enc:SetPrimaryCast(cxt.quest.param.rebel)
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_ASK_ABOUT")
               :IsHubOption(true)
               :GoTo("STATE_QUESTIONS")     

            cxt:Opt("OPT_DO_THE_AMBUSH")
                :Dialog("DIALOG_DO_THE_AMBUSH")
                :GoTo("STATE_AMBUSH")
        end)


    :AskAboutHub("STATE_QUESTIONS", 
             {

               "Ask about the corpse",
               [[
                    player:
                        What happened to your friend?
                    rebel:
                        We weren't friends - I barely knew them. 
                        He was trying to adjust the cobblebot here, and it went off.
                    player:
                        The robot killed him?
                    rebel:
                        I'm not sure these things are worth the trouble.
                        
               ]],
               "Ask about the robot",
               [[
                    player:
                        Where did you get the robot?
                    rebel:
                        We liberated it from the last patrol that we hit.
                        It... mostly works.
                    player:
                        Until it goes off and kills someone.
                    rebel:
                        Like I said. It mostly works.
               ]],
               "Ask about the ambush",
               [[
                    player:
                        What are we ambushing here?
                    rebel:
                        Probably just a couple Barons. They keep the transport caravans small to avoid attention.
                    player:
                        Any chance we can resolve it without a fight?
                    rebel:
                        Probably not. There's a bounty on us Rise, so they're going to be gunning for that once they see us.
               ]],

             })  

        :State("STATE_AMBUSH")
            :Loc{
                DIALOG_INTRO = [[
                    * It doesn't take long.
                    baron:
                        !right
                        Something's not right here...
                    robot:
                        !left
                        SELECT ALL
                        DESTROY
                    baron:
                        !surprised
                    rebel:
                        !left
                        Drop the loot, and reach for the sky, Barons!
                    player:
                        !left
                    baron:
                        And why should we do that? I think we can handle the likes of you.
                ]],
                OPT_INTIMIDATE = "Intimidate {agent}",
                DIALOG_INTIMIDATE = [[
                    player:
                        I'd do what {rebel.heshe} says, if I were you.
                    * You point out the Rise corpse.
                    player:
                        The last person to cross {rebel.himher} didn't get off lightly.
                ]],
                DIALOG_INTIMIDATE_SUCCESS = [[
                    baron:
                        !scared
                        Do what they say.
                    * The Barons unload the spark they were carrying.
                    baron:
                        Now let us go, right? That's part of the deal.
                ]],
                DIALOG_INTIMIDATE_FAIL = [[
                    baron:
                        Yeah yeah, real scary.
                        I'm going to kill you once for the bounty, and then a second time just for fun.
                ]],
                OPT_ATTACK = "Attack the Barons",
                DIALOG_ATTACK = [[
                    player:
                        !fight
                        Attack!
                    robot:
                        !left
                        !fight
                        ATTACK
                ]],

            }
            :SetLooping()
            :Fn(function(cxt) 
                if cxt:FirstLoop() then
                    cxt.quest.param.barons = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location)
                    cxt.enc:SetPrimaryCast(cxt.quest.param.barons[1])
                    cxt:ReassignCastMember("baron", cxt.quest.param.barons[1])
                    cxt:Dialog("DIALOG_INTRO")
                end
                
                cxt:Opt("OPT_INTIMIDATE")
                    :Dialog("DIALOG_INTIMIDATE")
                    :Negotiation{
                         
                        helpers =  {cxt.quest.param.rebel},
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        on_success = function() 
                            cxt:Dialog("DIALOG_INTIMIDATE_SUCCESS")
                            cxt:ReassignCastMember("survivor", cxt:GetAgent())
                            cxt:GoTo("STATE_POST_CONFLICT")
                        end,
                        on_fail = function() 
                            cxt:Dialog("DIALOG_INTIMIDATE_FAIL")
                        end
                    }
                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Fn(function() cxt.quest.param.fought_barons = true end)
                    :Battle{
                        allies = {cxt.quest.param.rebel, cxt.quest.param.robot},
                        on_win = function() 
                            cxt:GoTo("STATE_POST_CONFLICT")
                        end,
                    }
            end)
        :State("STATE_POST_CONFLICT")
            :Loc{
                DIALOG_FOUGHT_BARONS_ALL_DEAD_RISE_ALIVE = [[
                    player:
                        !left
                    * The Baron party has been wiped out.
                    rebel:
                        !right
                        Thanks for your help. I'll take care of the cleanup.
                ]],
                DIALOG_DID_NOT_FIGHT = [[
                    rebel:
                        !right
                        They've seen our faces. We have to finish them off.
                    player:
                        That's harsh.
                    rebel:
                        It is. But - but it has to be done.
                
                ]],

                DIALOG_FOUGHT_BARONS_SURVIVORS_RISE_ALIVE = [[
                    player:
                        !left
                    survivor:
                        !injured
                        !right
                    * You have the Baron survivors dead to rights.
                    survivor:
                        I surrender! Please, don't kill me.
                    rebel:
                        !right
                        They've seen our faces. We have to finish them off.
                    player:
                        That's harsh.
                    rebel:
                        It is. But - but it has to be done.
                        
                ]],
                OPT_KILL_THE_BARONS = "Kill the survivors yourself",
                DIALOG_KILL_THE_BARONS_1 = [[
                    player:
                        Stand aside, I've got this.
                        !exit
                    agent:
                        !exit
                    * You make short work of the survivors.
                ]],
                DIALOG_KILL_THE_BARONS_2 = [[
                    player:
                        !left
                    rebel:
                        !right
                        Thanks. I hate that part.
                        I'll handle the cleanup from here. 
                ]],


                OPT_LET_REBEL_KILL_THE_BARONS = "Let {rebel} kill the survivors",
                DIALOG_LET_REBEL_KILL_THE_BARONS = [[
                    player:
                        I'm just here for the paycheque. You do it.
                    rebel:
                        Ugh. Ok.
                        !exit
                    * {rebel} kills the survivors.
                    rebel:
                        !right
                        It's done. I'll make sure the spark gets to where it needs to go.
                ]],
                OPT_CONVINCE_REBEL_TO_SPARE_SURVIVORS = "Convince {rebel} to let the survivors go",
                DIALOG_CONVINCE_REBEL_TO_SPARE = [[
                    player:
                        We don't have to do that, do we?
                        Maybe having a couple of friends on the other side is worth the risk?
                ]],
                DIALOG_CONVINCE_REBEL_TO_SPARE_FAIL = [[
                    rebel:
                        You're not sympathizing with these Barons, are you?
                        They'd kill us if the tables were turned.
                        Just... leave. I'll take care of the rest.
                ]],
                DIALOG_CONVINCE_REBEL_TO_SPARE_SUCCESS = [[
                    rebel:
                        Maybe you're right.
                    survivor:
                        !right
                        I won't forget this. Thank you.
                    rebel:
                        !right
                        I hope we don't end up regretting that.
                        I'll clean up, you can go.
                ]],

                DIALOG_FOUGHT_BARONS_EVERYONE_DEAD = [[
                    * The spark glistens in the pile of corpses from both sides of the fight.
                    * You take a moment to bury it beside the path, so that {giver} can send a team to recover it.
                ]],

                DIALOG_FOUGHT_BARONS_SURVIVORS_RISE_DEAD = [[
                    player:
                        !left
                    survivors:
                        !injured
                        I can keep my mouth shut. Please, let me go.
                ]],
                OPT_FINISH_BARONS_ALONE = "Kill the survivors",
                DIALOG_FINISH_BARONS_ALONE = [[
                    player:
                        Sorry, can't take the chance.
                    agent:
                        !exit
                    * The butchery doesn't take long.
                    * You take a moment to bury it beside the path, so that {giver} can send a team to recover it.
                ]],
                OPT_SPARE_BARONS_ALONE = "Spare the survivors",
                DIALOG_SPARE_BARONS_ALONE = [[
                    player:
                        Get out of here. Now.
                    agent:
                        Thank you!
                        !exit
                    * You take a moment to bury it beside the path, so that {giver} can send a team to recover it.
                ]],
                DIALOG_FOUGHT_BARONS_NO_SURVIVORS_RISE_ALIVE = [[
                    rebel:
                        !right
                        We did it!
                        Thanks for your help, grifter. I'll take it from here.
                ]]
            }
            :Fn(function(cxt) 
                local survivors = {}
                for k, v in ipairs(cxt.quest.param.barons) do
                    if not v:IsDead() and v:IsSentient() then
                        table.insert(survivors, v)
                    end
                end
                if #survivors > 0 then
                    cxt:ReassignCastMember("survivor", survivors[1])
                    if cxt.quest.param.rebel:IsDead() then
                        cxt:Dialog("DIALOG_FOUGHT_BARONS_SURVIVORS_RISE_DEAD")
                        
                        cxt:Opt("OPT_FINISH_BARONS_ALONE")
                            :Dialog("DIALOG_FINISH_BARONS_ALONE")
                            :Fn(function() 
                                for k,v in ipairs(survivors) do
                                    AgentUtil.CommitMurder(v)
                                    v:Kill()
                                end
                            end)
                            :CompleteQuest("go_to_ambush")
                            :Travel()

                        local opt = cxt:Opt("OPT_SPARE_BARONS_ALONE")
                                        :Dialog("DIALOG_SPARE_BARONS_ALONE")
                        for k,v in ipairs(survivors) do
                            opt:ReceiveOpinion(OPINION.SPARED_LIFE, nil, v)
                        end
                        opt:CompleteQuest("go_to_ambush"):Travel()

                    else
                        
                        cxt:Dialog(cxt.quest.param.fought_barons and "DIALOG_FOUGHT_BARONS_SURVIVORS_RISE_ALIVE" or "DIALOG_DID_NOT_FIGHT")

                        cxt:Opt("OPT_LET_REBEL_KILL_THE_BARONS")
                            :Dialog("DIALOG_LET_REBEL_KILL_THE_BARONS")
                            :Fn(function() 
                                for k,v in ipairs(survivors) do
                                    v:Kill()
                                end
                            end)

                            :CompleteQuest("go_to_ambush")
                            :Travel()

                        cxt:Opt("OPT_KILL_THE_BARONS")
                            :Dialog("DIALOG_KILL_THE_BARONS_1")
                            :Fn(function() 
                                for k,v in ipairs(survivors) do
                                    v:Kill()
                                    AgentUtil.CommitMurder(v, survivors)
                                end
                            end)
                            :Dialog("DIALOG_KILL_THE_BARONS_2")
                            :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("helped_with_ambush"),nil, cxt.quest.param.rebel)
                            :CompleteQuest("go_to_ambush")
                            :Travel()

                        cxt:Opt("OPT_CONVINCE_REBEL_TO_SPARE_SURVIVORS")
                            :Dialog("DIALOG_CONVINCE_REBEL_TO_SPARE")
                            :Negotiation{
                                target_agent = cxt.quest.param.rebel,
                                helpers = {survivors[1]},
                                on_success = function() 
                                    cxt:Dialog("DIALOG_CONVINCE_REBEL_TO_SPARE_SUCCESS")   
                                    for k,v in ipairs(survivors) do
                                        v:OpinionEvent(OPINION.SPARED_LIFE)
                                    end
                                    cxt.quest:Complete("go_to_ambush")
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end,
                                
                                on_fail = function() 
                                    cxt:Dialog("DIALOG_CONVINCE_REBEL_TO_SPARE_FAIL")
                                end,
                            }
                    end
                else
                    if cxt.quest.param.rebel:IsDead() then
                        cxt:Dialog("DIALOG_FOUGHT_BARONS_EVERYONE_DEAD")
                    else
                        cxt:Dialog("DIALOG_FOUGHT_BARONS_NO_SURVIVORS_RISE_ALIVE")
                        cxt.quest.param.rebel:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("helped_with_ambush"))
                    end

                    cxt.quest:Complete("go_to_ambush")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            end)



QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Tell {agent} the ambush was a success",
    }
    :Loc{
        DIALOG_DONE = [[
            player:
                The ambush was a success.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_DONE")
            :CompleteQuest()
    end)    



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                The Barons move spark through the bog at night, thinking that the darkness protects them.
                It does not. We are the bog.
                One of our cells is ready to strike, but they're down a fighter. Go make sure they succeed.
            *** A Rise cell has suffered casualties, and needs help springing an ambush on the Barons.
        ]]
    }
    :State("START")
        :Fn(function(cxt)

            cxt:Dialog("DIALOG_INTRO")
        end)
