local FOLLOW_UP_GET_PAID

local QDEF = QuestDef.Define{
        title = "Asset Liberation",
        desc = "Revolutions need weapons if they are to be successful. Check in on {informant}, and use {informant}'s info to liberate a cache of weapons for {giver}.",
        icon = engine.asset.Texture("icons/quests/rise_asset_liberation.tex"),
        faction = "RISE",
        tags = {"nasty"},
        rank = {1, 10},
        qtype = QTYPE.CONTRACT,
        
        on_init = function(quest) 
            quest.param.tab_amount = 200
            quest.param.share_amount = quest:GetRank()*50
            return true
        end,

        collect_agent_locations = function(quest, t)
            if quest:IsActive("contact_informant") or quest:IsActive("extract_informant") then
                table.insert (t, { agent = quest:GetCastMember("informant"), location = quest:GetCastMember('informant_bar'), role = CHARACTER_ROLES.PATRON } )
            end
        end,

        on_start = function(quest)
            quest:Activate("contact_informant")
        end,
        
    }


    :AddObjective{
        id = "contact_informant",
        title = "Contact {informant}",
        desc = "{informant} knows the location of a shipment of weapons and medicine meant for the Spark Barons. Find {informant.himher} and find the shipment.",
        mark = { "informant" },
    }
    
    :AddObjective{
        id = "extract_informant",
        title = "Extract {informant} from the bar",
        desc = "{bartender} won't let {informant} leave the bar until {informant.hisher} tab is paid off.",
        mark = { "informant", "bartender" },
        negotiation_targets = {"informant", "bartender"},
    }


    :AddObjective{
        id = "investigate_cache",
        title = "Investigate the cache site",
        desc = "You got the location of the supposed Spark Baron cache. Investigate the site so that the Rise can recover the goods.",
        mark = { "the_cache" },
        on_activate = function(quest)
            quest:SpawnTempLocation("CACHE_LOCATION", "the_cache" )
        end,
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Tell {giver} the cache is secure",
        desc = "The cache has been secured. Go tell {giver} and collect your reward.",
        ready_for_completion = true,
        mark = { "giver" },
    }

    :AddLocationCast{
        cast_id = "the_cache",
        when = QWHEN.MANUAL,
        no_validation = true,
    }
    

    :AddLocationCast{
        cast_id = "informant_bar",
        cast_fn = function(quest, t)
            table.insert(t, TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"))
        end,
    }
    
    :AddCast{
        cast_id = "bartender",
        cast_fn = function(quest, t) 
            table.insert(t, quest:GetCastMember("informant_bar"):GetProprietor())
        end,
        no_validation = true,
    }
    
    :AddCastByAlias{
        cast_id = "giver",
        provider = true,
        no_validation = true,
        alias = "KALANDRA",
    }

    :AddCast{
        cast_id = "informant",
        cast_fn = function(quest, t) 
            table.insert(t, quest:CreateSkinnedAgent("RISE_RADICAL"))
        end,
    }

    :AddLocationDefs{
        CACHE_LOCATION = {
            name = "Spark Baron Cache",
            desc = "This is where {informant} said the cache would be. Proceed with caution.",
            plax = "EXT_Bog_Forest_01",
            map_tags = {"bog"},
            indoors = false,
        },
    }

    :AddOpinionEvents{
    
        entered_into_a_plot =  
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "Entered into a plot",
        },
        ratted_them_out =  
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Ratted them out to the Rise",
        },
        dealt_with_their_tab =  
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "Dealt with their bar tab",
        },
    }

--meet the informant!
QDEF:AddConvo("contact_informant", "informant")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_MAKE_CONTACT = "Ask about the cache",
        DIALOG_INTRO = [[
            * You flash {agent} the secret Rise handsign that {giver} taught you.
            agent:
                !salute
                You forgot to hook your thumb, but that's close enough. 
            player:
                !interest
                {giver} sent me. {giver.HeShe} says you haven't checked in.
                Seeing as you're warm and cozy here, I expect {giver.heshe}'ll want a good explanation.
                !shrug
                Not that I'm complaining. This is the easiest rescue mission I've ever partaken.
            agent:
                !bashful
                Guess it depends on how deep your pockets are. Thing is, uh... I can't leave.
                Not until I pay off my tab.
            player:
                !dubious
                You haven't reported in because you're in debt <i>to the barkeep</i>?
            agent:
                Hey, freedom fighting is thirsty work.
            player:
                !overthere
                {giver} has a bar at the headquarters! Why not drink there?
            agent:
                I like the swill here better.
            player:
                !facepalm
                How much debt are we talking here?
            agent:  
                Dunno. More than I have, that's for sure.
                !tired
                Honestly, I lost track. I've been bussing tables between double shifts at the dig for four days.
            player:
                Well, what about the cache? Can you tell me where to find it?
                !permit
                That way I can at least take it to {giver}.
            agent:
                !thought
                I <i>could</i> do that... but maybe I can't remember unless I'm actually going there myself, ya know?
                Besides, if I don't pay off the rest of my debt soon, the barkeep will just sell it to the Spark Barons. 
                Then they'll put me on probation, watchin' my every move...
            player:
                !angry
                Some "freedom" fighter you are. Holding yourself hostage and demanding a ransom.
            agent:
                !thumb
                I fight for my own freedom first. Everyone else's comes second.
                Get me outta debt. <i>Then</i> I'll take you to the cache. 
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_MAKE_CONTACT")
            :SetQuestMark()
            :Dialog("DIALOG_INTRO")
            :CompleteQuest("contact_informant")
            :ActivateQuest("extract_informant")
    end)


--deal with the informant's situation
QDEF:AddConvo("extract_informant")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
    
        ------------------------- bartender stuff

        OPT_PAY_TAB = "Pay off {informant}'s tab",
        DIALOG_PAID_TAB = [[
            player:
                Here. Keep the change.                
            agent:
                Heh. That's some real Rise solidarity, right there.
        ]],

        OPT_NEGOTIATE_A_DISCOUNT = "Offer to pay back half of the tab",

        DIALOG_NEGOTIATE_DISCOUNT = [[
            player:
                !overthere
                Look, {informant} is clearly not good for the shills <i>or</i> the labor. 
                Between the red tape and the bad look, you'll be lucky if you see <i>any</> of what you're owed.
                !interest
                What if I threw some shills your way? Could you let the rest slide?
        ]],

        DIALOG_GOT_DISCOUNT = [[
            agent:
                Ugh, fine. I <i>guess</i> this settles {informant.hisher} tab.
                !point
                But from now on, {informant} gets {informant.hisher} fill somewhere else. You tell {informant.himher} that.
            player:
                I'll be sure to pass that along.
        ]],
        DIALOG_NO_DISCOUNT = [[
            agent:
                !angry_shrug
                I'm running a business here. You think the Spark Barons make it easy for me?
                I'm taxed out of a good night's sleep.
                {informant} pays in full, or I get what I'm owed by other means. 
        ]],


        OPT_NEGOTIATE_DEBT_FORGIVENESS = "Ask {bartender} to forgive the tab",
        DIALOG_NEGOTIATE_DEBT_FORGIVENESS = [[
            player:
                !happy
                Look, bud. Buddy. Friendo.
            agent:
                !crossed
                That ain't my name, grifter.
            player:
                !permit
                Not with that attitude. I need you to let {informant} off the hook.
            agent:
                And why would I do that, <i>friend</i>? 
        ]],
        DIALOG_NO_FORGIVE = [[
            agent:
                Nice try, but no. I'm running a business here. 
                !angry_shrug
                You think the Spark Barons make it easy for me? I'm taxed out of a good night's sleep.
                {informant} pays in full, or I get what I'm owed by other means. 

        ]],
        DIALOG_FORGIVE = [[
            player:
                I heard there's a strike going on. The laborers are out for blood.
                !cruel
                If you work one of them to the bone or <i>worse</i>, pass the debt along—that makes you look a lot like a Spark Baron yourself, don't it?
            agent:
                !thought
                Hm.
                That's... got some truth to it. Things are <i>tense</i> lately.
                !agree
                Fine. The debt is canceled. But tell that mooch {informant.hisher} drinks cost double from now on. Shills up front, too.
        ]],
        ------------------------- informant stuff

        OPT_ASK_ABOUT_TAB = "Ask about tab",
        DIALOG_ABOUT_TAB = [[
            player:
                !angry_shrug
                What kind of rebel racks up that kind of bar tab?
            agent:
                !dubious
                Uh, the double agent kind. Spark Barons come here. 
                They talk more if they're drunk. So I buy them drinks. 
                !ovethere
                So you want that cache? You're gonna have to help me pay for it.
        ]],

        OPT_FORCE_SPILL_BEANS = "Force {informant} to tell you where the cache is",
        DIALOG_FORCE_SPILL_BEANS = [[
            player:
                !angry
                Your tab is your problem, not mine. 
                Tell me where the cache is, or you'll have more to worry about then bussing tables. 
            agent:
                !dubious
                Wait, are you <i>threatening</i> me?
        ]],
        DIALOG_NO_BEANS = [[
            player:
                !angry
                You <i>don't</i> wanna see me angry.
            agent:
                !angry
                Pretty sure you said you were sent to rescue me. So things wouldn't go well for you if I wasn't rescued.
                !spit
                Get over yourself and do some work like the rest of us. 
        ]],
        DIALOG_GOT_BEANS = [[
            player:
                !cruel
                {giver} may have asked me to rescue you but {giver.heshe} doesn't have to know you weren't dead when I found you. 
                !throatcut
                So what's it going to be? The cache? Or I pay your debt... and we go for a walk?
            agent:
                !scared
                Fine! Hesh, you're serious, aren't you?
            * {agent} reaches into {informant.hisher} pocket for a wad of paper someone might've once used as a napkin. 
            * The location of the cache is scrawled on it, plain to read, not even written in code.
            agent:
                !give
                Now leave me alone. I have <i>debts</i> to pay.
        ]],

        OPT_ATTACK = "Beat the information out of {informant}",
        

        DIALOG_BEAT_INFORMANT = [[
            agent:
                !injured
                Stop! Stop! Hesh, I can't even be sure you're really on our side, now.
            * {agent} produces a scrap of soiled paper with a location sloppily scrawled upon it.
            agent:
                I'll remember this, grifter. Especially if you make me a traitor. 
            bartender:
                !right
                !angry
                Hey! That one's got a tab to work off. You leave {informant.himher} alone.
                !angry_accuse
                No fighting in my bar, you hear me?
            player:
                !handwave
                Yeah, yeah.
            * You leave on your own terms before {bartender} has the chance to kick you out.
        ]],

        DIALOG_KILLED_INFORMANT = [[
            * {agent} is dead. You quickly search through their pockets and find a the location of the cache scrawled on a piece of paper.
            bartender:
                !right
                Look at this mess. {informant.HeShe} had debts to pay, ya know!
                Who's gonna cover those now, huh? Your pockets as deep as your sheath, pal?
            player:
                That's what you get for employing losers.
            * You leave on your own terms before {bartender} has the chance to kick you out.
        ]],
        DIALOG_HELPED_INFORMANT = [[
            informant:
                !right
                !happy
                Well, by the frothing surf. You actually got me off, huh?
            player:
                That's what you strong-armed me into doing, wasn't it?
            informant:
                !eureka
                That's great work. Real professional. 
            * {informant} pulls a scrap of soiled paper from {informant.hisher} pocket.
            informant:
                Shouldn't be too hard for someone as smart as you to figure out my shorthand.
                Have at it.
        ]],


    }
    :Attract(function(cxt) 
            if cxt:GetAgent() == cxt.quest:GetCastMember("bartender") then
                return "STATE_ATTRACT_BARTENDER"
            elseif cxt:GetAgent() == cxt.quest:GetCastMember("informant") then
                return "STATE_ATTRACT_INFORMANT"
            end
        end)

    :State("STATE_ATTRACT_INFORMANT")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    Time's slipping by, my friend. You want that cache? Get me outta this mess I'm in with {bartender}.
            ]]
        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
            end)


    :State("STATE_ATTRACT_BARTENDER")
        :Loc{
            DIALOG_INTRO_BARTENDER = [[
                agent:
                    Quit botherin' me. Gotta keep an eye on {informant}, make sure {informant.heshe} isn't sneakin' a sip when I'm not looking.
            ]]
        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO_BARTENDER")
            end)


    :State("STATE_TRY_COME_WITH")
        :Loc{

            DIALOG_TRY_COME_WITH = [[
                player:
                    !point
                    Hang on. This tells me where to find it, but what if it's guarded?
                informant:
                    Then you gotta use your wiles or whatever.
                    !thumb
                    <i>I</i> need to get to my next shift before the truancy officer comes looking for me. 
            ]],

            OPT_NEGOTIATION = "Convince {agent} to come with you",
            OPT_NEVER_MIND = "Leave without {agent}",
            DIALOG_NEVER_MIND = [[
                player:
                    !palm
                    I'm always the one who has to play the hero.
                informant:
                    Yeah, well, it suits you. So quit complaining.
                    !exit
                * And with one last wink to the bartender, {informant} is out the door.
                * You didn't even get a chance to say goodbye. Or to kick {informant.himher} in the seat as a parting gesture.
            ]],

            DIALOG_NEGOTIATION = [[
                {helped_with_tab?                    
                    player:
                        !angry_shrug
                            You only got outta your stupid debt thanks to me.
                    agent:
                        !thought
                        Well that's the truth, I suppose.
                }   
                {not helped_with_tab?                    
                    player:
                        !angry_shrug
                            This is Rise business. You're Rise. You need to come with me.
                    agent:
                        !thought
                        That's not the kind of work I signed up for!
                }   
            ]],

            DIALOG_SUCCESS = [[
                player:
                    !salute
                    Remember that freedom fighting we talked about? Now's the time.
                agent:
                    Ugh, <i>fine</i>. I swear you sound just like my ma.
                    !shrug
                    Before she, you know, got killed in a cave-in.
                player:
                    !dubious
                    Huh.
                agent:
                    !overthere
                    That probably won't happen to you. Anyway, we going or what?
            ]],

            DIALOG_FAIL = [[
                player:
                    Don't you think you owe me? Owe <i>Kalandra</i>?
                agent:
                    !placate
                    Look, bud. I'm a spy. 
                    And I can't do that if I'm on probation for being late to my shift.
                    !wave
                    Keep it sleazy. 
                    !exit
                * And with one last wink to the bartender, {informant} is out the door.
                * You didn't even get a chance to say goodbye. Or to kick {informant.himher} in the seat as a parting gesture.
            ]]
        }

        :Fn(function(cxt)
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("informant"))
                cxt.quest:Complete("extract_informant")
                cxt.quest:Activate("investigate_cache")

                cxt:Dialog("DIALOG_TRY_COME_WITH")

                cxt:Opt("OPT_NEGOTIATION")
                    :Dialog("DIALOG_NEGOTIATION")
                    :Negotiation{
                        on_success = function(cxt) 
                            cxt:Dialog("DIALOG_SUCCESS")
                            cxt.quest.param.informat_joined = true
                            cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                        on_fail = function(cxt) 
                            cxt:Dialog("DIALOG_FAIL")
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                }

                cxt:Opt("OPT_NEVER_MIND")
                    :Dialog("DIALOG_NEVER_MIND")
                    :DoneConvo()

            end)

    :Hub(function(cxt) 

        if cxt:GetAgent() == cxt.quest:GetCastMember("bartender") then
            
            cxt:Opt("OPT_PAY_TAB")
                :DeliverMoney(cxt.quest.param.tab_amount)
                :Dialog("DIALOG_PAID_TAB")
                :Dialog("DIALOG_HELPED_INFORMANT")
                :Fn(function(cxt)
                    cxt.quest.param.helped_with_tab = true
                    cxt.quest:GetCastMember("informant"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("dealt_with_their_tab"))
                end)
                :GoTo("STATE_TRY_COME_WITH")            

            local DISCOUNT = .5

            cxt:Opt("OPT_NEGOTIATE_A_DISCOUNT")
                :Dialog("DIALOG_NEGOTIATE_DISCOUNT")
                :RequireMoney(math.floor(cxt.quest.param.tab_amount*DISCOUNT))
                :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent( "dealt_with_their_tab" ), nil, cxt.quest:GetCastMember( "informant" ))
                :Negotiation{
                    timer_id = "FORGIVE_TAB",
                    on_success = function(cxt)
                        cxt.quest.param.helped_with_tab = true
                        cxt:Dialog("DIALOG_GOT_DISCOUNT")
                        cxt.caravan:PayMoney( math.floor(cxt.quest.param.tab_amount*DISCOUNT), cxt:GetAgent() )
                        cxt:Dialog("DIALOG_HELPED_INFORMANT")
                        cxt:GoTo("STATE_TRY_COME_WITH")            
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NO_DISCOUNT")
                    end,
                }
            
            cxt:Opt("OPT_NEGOTIATE_DEBT_FORGIVENESS")
                :Dialog("DIALOG_NEGOTIATE_DEBT_FORGIVENESS")
                :Negotiation{
                    timer_id = "FORGIVE_TAB",
                    on_success = function(cxt)
                        cxt.quest.param.helped_with_tab = true
                        cxt:Dialog("DIALOG_FORGIVE")
                        cxt:Dialog("DIALOG_HELPED_INFORMANT")
                        cxt.quest:GetCastMember("informant"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("dealt_with_their_tab"))
                        cxt:GoTo("STATE_TRY_COME_WITH")
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NO_FORGIVE")
                    end,
                }
        elseif cxt:GetAgent() == cxt:GetCastMember("informant") then

            cxt:Opt("OPT_ASK_ABOUT_TAB")
                :Dialog("DIALOG_ABOUT_TAB")

            cxt:Opt("OPT_FORCE_SPILL_BEANS")
                :Dialog("DIALOG_FORCE_SPILL_BEANS")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.HOSTILE | NEGOTIATION_FLAGS.INTIMIDATION,
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_GOT_BEANS")
                        cxt:GoTo("STATE_TRY_COME_WITH")
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NO_BEANS")
                    end,
                }
            cxt:Opt("OPT_ATTACK")
                :Battle{
                    on_win = function(cxt) 
                        if cxt:GetAgent():IsDead() then
                            cxt.quest.param.killed_informant = true
                            cxt:Dialog("DIALOG_KILLED_INFORMANT")
                        else
                            cxt.quest.param.beat_informant = true
                            cxt:Dialog("DIALOG_BEAT_INFORMANT")
                        end
                        cxt.quest:Complete("extract_informant")
                        cxt.quest:Activate("investigate_cache")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                }     
        end   
    end)




local PostCacheFightFn = function(cxt) 
    local alive_num = AgentUtil.CountAlive(cxt.enc.scratch.barons)
    if alive_num == 0 then
        if cxt:GetCastMember("informant"):IsInPlayerParty() then
            cxt:Dialog("DIALOG_RUTHLESS_INFORMANT")
        else
            cxt:Dialog("DIALOG_RUTHLESS")
        end
    elseif alive_num > 0 then
        cxt:Dialog("DIALOG_RUNAWAY", alive_num)
    end
    cxt:GoTo("STATE_SECURE_CACHE")
end



--deal with the informant's situation
QDEF:AddConvo("investigate_cache")
    :Loc{

    }
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("the_cache") then
                return "STATE_CACHE_BARON_FIGHT"
            end
        end)    

    :State("STATE_CACHE_BARON_FIGHT")
        :Loc{
            DIALOG_INTRO = [[
                * You find the spot marked on {informant}'s map. Surprising nobody, the Barons have posted a guard. 
                * You keep to the bushes, squat in the mud, and wait.
            ]],

            DIALOG_INTRO_INFORMANT = [[
                * {informant} proves a worthy guide, keeping the branches out of your face and the leeches out of your boots. 
                * Eventually {informant.heshe} stops short, gesturing for silence. You do your best to comply as a bloodsucker buzzes angrily in your ear.
                informant:
                    !right
                    We're getting close. Just on the other side of this patch.
                player:
                    !left
                * You slowly lean forward, peering through the sharp branches. On the far side, you can make out Baron uniforms, sparkling clean despite the muck.
                    Careful!
            ]],

            DIALOG_KNOW_BARON_POSITIVE = [[
                * With a second peek, you recognize one of the Barons. It's {baron}. This might be manageable.
            ]],

            DIALOG_KNOW_BARON_NEGATIVE = [[
                * With a second peek, you recognize one of the Barons. It's {baron} again. 
                player:
                    Ah, sea sludge.
            ]],

            OPT_SURPRISE_ATTACK = "Charge in with a surprise attack",
            DIALOG_SURPRISE_ATTACK = [[
                player:
                    !fight
                    !left
                agent:
                    !right
                    !surprised
                * You charge in, blade glinting like a firefly.

            ]],
            OPT_TALK_IT_OUT = "Enter the clearing peacefully",
            DIALOG_RUNAWAY = [[
                agent:
                    !exit
                * {1*The Baron breaks and runs|The last Barons break and run} for the trees.
            ]],
            DIALOG_RUTHLESS_INFORMANT = [[
                informant:
                    !right
                    Yikes. Remind me not to get on your bad side.
            ]],
            DIALOG_RUTHLESS = [[
                * You keep notching your hilt like that and there won't be anything left but the blade. 
            ]],

            DIALOG_NEGOTIATION_INTRO = [[
                * You walk nonchalantly into the clearing, your empty hands clearly visible, just in case someone gets a funny idea.
                player:
                    !left
                    !placate
                agent:
                    !right
                    !dubious
            ]],
            DIALOG_NEGOTIATION_INTRO_POSITIVE = [[
                player:
                    Well, hey there, {agent}. Imagine running into you out here.
                agent:
                    {player}? Yeah, imagine that. 
                    This is a restricted area, you know? You shouldn't be out here.
            ]],
            DIALOG_NEGOTIATION_INTRO_NEGATIVE = [[
                player:
                    Hello, {agent}. Keeping the bugs off alright? They're real bitey today.
                agent:
                    I was pest-free until you showed up. What are you doing here, grifter?
                    This is a restricted area. I'm authorized to deal with trespassers however I like.
            ]],
            DIALOG_NEGOTIATION_INTRO_NEUTRAL = [[
                player:
                    Excuse me, Baron. Don't suppose you could give a lost soul some directions?
                agent:
                    You're lost, all right. This is a restricted area, laborer. 
                    What's your employee ID?
                player:
                    !thought
                    Uh, three two four, uh...
                agent:
                    !crossed
                player:
                    !scared_shrug
                    Bee... two one?
                agent:
                    !angry
                    Let's try again, from the top, huh?

            ]],
            DIALOG_NEGOTIATION_END = [[
                player:
                    !agree
                    I'll be honest. I'm here for the cache. 
                    What would it take to convince you to take a walk? It's just a job, right?
            ]],      

            DIALOG_NEGOTIATION_WON = [[
                player:
                    Thing is, if you don't walk, we fight. And if we fight, you die. 
                    !fight
                    Walking's good for your health.
                agent:
                    !scared
                    H-Hesh, look, we don't want any trouble. Easy enough to go for a walk.
                player:
                    !cruel
                    I reckoned you'd agree.
                * With a look like {baron.heshe} just dislodged a sharp bone from {baron.hisher} gullet, {baron} nods once, and the Barons disappear into the brush.
                    !exit
            ]],

            DIALOG_NEGOTIATION_WON_FRIEND = [[
                player:
                    !interest
                    All I need is ten minutes with the cache. You stretch your legs, get out of this midge swarm.
                    By the time you come back, I'll be gone. You won't even notice anything's missing. 
                agent:
                    !sigh
                    Look, I owe you one. So... fine. 
                    Not like you can do too much damage with this stuff anyway.
                    !dubious
                    Ten minutes?
                player:
                    !placate
                    Ten minutes.
                agent:
                    !wink
                    You were never here.
                player: 
                    !wink
                    Enjoy your walk.
                agent:
                    !exit
            ]],

            DIALOG_NEGOTIATION_WON_ENEMY= [[
                player:
                    !fight
                    Think how badly you want me dead. Know that's how badly I want <i>you</i> dead.
                    And all that's standing between me and that cache is you. 
                    Crunch those numbers, {baron}. Calculate your odds.
                agent:
                    !angry
                    You ever heard of a tactical retreat, {player}?
                player:
                    I perfected it.
                agent:
                    That's what this is.
                    !throatcut
                    Next time? You're <i>dead</i>.
                player:
                    !chuckle
                    Whatever you say.
                agent:
                    !exit

            ]],

            DIALOG_NEGOTIATION_LOST = [[
                player:
                    !overthere
                    So you see, if you just, uh, wanna look the other way for a second...
                agent:
                    !angry_accuse
                    Seems pretty clear you're one of those Rise hooligans.
                    We've got orders to deal with you.
                    !attack
                player:
                    Ah, nuts.
            ]],

            DIALOG_NEGOTIATION_LOST_FRIEND = [[
                player:
                    !overthere
                    So you see, if you just, uh, wanna look the other way for a second...
                agent:
                    !palm
                    Stop. Just stop. What am I supposed to do with this, {player}?
                    Walk away. Else I gotta use force. Don't make me, okay?
            ]],

            DIALOG_NEGOTIATION_LOST_ENEMY = [[
                player:
                    !overthere
                    So you see, if you just, uh, wanna look the other way for a second...
                agent:
                    !cruel
                    Oh, you have no idea what you've just walked into, {player}.
                    You've had this coming for a while. 
                    !attack
            ]]
        }
        :SetLooping()
        :Fn(function(cxt) 
                if cxt:FirstLoop() then
                    local adjust = cxt:GetCastMember("informant"):IsInPlayerParty() and 1 or 0
                    cxt.enc.scratch.barons = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank() + adjust, cxt.location, true)
                    local baron = cxt.enc.scratch.barons[1]
                    cxt:ReassignCastMember("baron", baron)
                    cxt.enc:SetPrimaryCast(baron)

                    if cxt:GetCastMember("informant"):IsInPlayerParty() then
                        cxt:Dialog("DIALOG_INTRO_INFORMANT")
                    else
                        cxt:Dialog("DIALOG_INTRO")
                    end

                    if baron:GetRelationship() > RELATIONSHIP.NEUTRAL then
                        cxt:Dialog("DIALOG_KNOW_BARON_POSITIVE")
                    elseif baron:GetRelationship() < RELATIONSHIP.NEUTRAL then
                        cxt:Dialog("DIALOG_KNOW_BARON_NEGATIVE")
                    end
                end

                local allies = {}
                if cxt:GetCastMember("informant"):IsInPlayerParty() then
                    table.insert(allies, cxt:GetCastMember("informant"))
                end

                 cxt:Opt("OPT_SURPRISE_ATTACK")
                    :Dialog("DIALOG_SURPRISE_ATTACK")
                    :Battle{
                        allies = allies,
                        advantage = TEAM.BLUE,
                        on_win = PostCacheFightFn,
                    }
                cxt:Opt("OPT_TALK_IT_OUT")
                    :Fn(function(cxt) 
                            cxt:Dialog("DIALOG_NEGOTIATION_INTRO")
                            if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL then
                                cxt:Dialog("DIALOG_NEGOTIATION_INTRO_POSITIVE")
                            elseif cxt:GetAgent():GetRelationship() < RELATIONSHIP.NEUTRAL then
                                cxt:Dialog("DIALOG_NEGOTIATION_INTRO_NEGATIVE")
                            else
                                cxt:Dialog("DIALOG_NEGOTIATION_INTRO_NEUTRAL")
                            end
                            cxt:Dialog("DIALOG_NEGOTIATION_END")
                        end)
                    :Negotiation{
                        flags = cxt:GetAgent():GetRelationship() <= RELATIONSHIP.NEUTRAL and NEGOTIATION_FLAGS.INTIMIDATION,
                        on_success = function(cxt) 
                            ConvoUtil.RelationshipDialog( cxt, cxt:GetAgent(), "DIALOG_NEGOTIATION_WON", "DIALOG_NEGOTIATION_WON_FRIEND", "DIALOG_NEGOTIATION_WON_ENEMY")
                            cxt:GoTo("STATE_SECURE_CACHE")
                        end,
                        on_fail = function(cxt) 
                            ConvoUtil.RelationshipDialog( cxt, cxt:GetAgent(), "DIALOG_NEGOTIATION_LOST", "DIALOG_NEGOTIATION_LOST_FRIEND", "DIALOG_NEGOTIATION_LOST_ENEMY")
                            cxt:DoBattle
                            {
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                on_win = PostCacheFightFn,
                            }
                        end,

                    }
            end)
    :State("STATE_SECURE_CACHE")

        :Loc{
            DIALOG_INTRO = [[
                * You check over the cache crates. Surprisingly, they're full of imported Admiralty weapons.
                * You don't want to be caught by Spark Barons with these in your pockets, but you can move them to a spot more agreeable to the next Rise rebel who comes looking. 
            ]],
            
            DIALOG_INFORMANT_OFFER = [[
                * Once you and {informant} have covered your tracks, {informant.heshe} lifts {informant.hisher} hands, cupped with the first mound of dirt to camouflage the boxes.
                * But some plot binds {informant.himher}, and you can tell at a glance it's a juicy one.
                player:
                    !left
                agent:
                    !right
                    !thought
                    This looks like more weapons than the Rise has hands to wield 'em, wouldn't you say?
                player:
                    !dubious
                    Would I?
                agent:
                    I mean, {giver} doesn't know exactly what's in the cache. 
                    !wink
                    {giver.HeShe} would never know if we took... some initiative.
                player:
                    !interest
                    You'd risk getting caught with a weapon? That's contraband. 
                agent:
                    We wouldn't have 'em for long. 
                    Not all the workers are part of the resistance. And some of 'em got bad judgment. 
                    !permit
                    We sell them the weapons to use for their own reasons, and pocket the change.
                    Hesh, more likely than not they'll distract the Barons by making trouble of their own.
                player:
                    !crossed
                    If this is such a good idea, why not run it past Kalandra?
                agent:
                    !shrug
                    Kalandra? Too messy for her tastes.
                    She's out for blood as long as it's a Baron—but too soft on the strikebreakers and scabs, if you ask me.
            ]],
            OPT_TAKE_THE_OFFER = "Take the offer",
            DIALOG_TAKE_OFFER = [[
                player:
                    !agree
                    Let's do it. There's always time for profit, I find.
                agent:
                    !handwring
                    Great. I'll just take a few arms and head off to find a buyer. I already have a couple folks in mind.
                    You keep {giver} off the scent, and we'll meet up after to split the profits.
                * {agent} helps {informant.himher}self to the cache without another word, and you head off to tell {giver} about the weapons. At least, the ones that are left, anyway.

            ]],
            OPT_REFUSE_THE_OFFER = "Refuse the offer",

            DIALOG_REFUSE_OFFER = [[
                player:
                    !sigh
                    Handing out guns is a sure-fire way to get shot, in my experience.
                agent:
                    !dubious
                    Yeah? Yeah... I suppose you're right. 
                    Hang out with the Rise long enough and you start to get a little ruthless. But I'm a big fan of self-preservation.
                * {agent} clears {agent.hisher} throat, and shifts topics.
                agent:
                    Anyway, the cache is secure, so my work here is done.
                    !greeting
                    I'll be seeing you around, grifter.
                    !exit
                * {agent} makes a big show of leaving, making clear that {agent.heshe} knows that you know that {agent.heshe} has made an exit.
            ]],
        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("investigate_cache")
                cxt.quest:Activate("return_for_reward")

                if cxt:GetCastMember("informant"):IsInPlayerParty() then
                    cxt.enc:SetPrimaryCast(cxt:GetCastMember("informant"))
                    cxt:Dialog("DIALOG_INFORMANT_OFFER")
                    
                    cxt:Opt("OPT_TAKE_THE_OFFER")
                        :Dialog("DIALOG_TAKE_OFFER")
                        :Fn(function() 
                            cxt.quest:GetCastMember("informant"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("entered_into_a_plot"))
                            cxt:GetAgent():Dismiss()
                            
                            cxt.quest.param.took_offer = true
                        end)

                        :Travel()

                    cxt:Opt("OPT_REFUSE_THE_OFFER")
                        :Dialog("DIALOG_REFUSE_OFFER")
                        :Fn(function() 
                            cxt:GetAgent():Dismiss()
                            cxt.quest.param.refused_offer = true
                        end)
                        :Travel()                        
                else

                    --cxt:Dialog("DIALOG_ALONE")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            end)

-------------------------------------------------------------------------------------------------------------------


--meet the informant!

QDEF:AddConvo("return_for_reward", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)

    :Loc{
        OPT_TURN_IN = "Tell {giver} the cache is secure",
        DIALOG_INTRO = [[
            player:
                !wink
                Cache secured, rebel leader. 
                You got yourself a new supply of military-grade weapons. 
        ]],
        DIALOG_IF_TOOK_OFFER = [[
            agent:
                !happy
                A lot, I hope? Every weapon <i>we</i> have is one less we have to worry about.
                And what about {informant}? Did you find {informant.himher}?
            player:
                I did. {informant} took some drying out, but {informant.heshe} came around.
            agent:
                Good to know I still have an asset in the field.
        ]],
        DIALOG_IF_REFUSED = [[
            agent:
                !happy
                Never doubted you for a second. 
                !interest
                You found {informant}, too, I hope?
        ]],
        DIALOG_PROTECT_INFORMANT = [[
            player:
                !agree
                {informant} and I even made a decent team, I'd say.
                After some convincing, of course.
            agent:
                Aye. {informant.HeShe}'s a rotten coward half the time.
                But a good spy <i>all</i> the time.
            player:
                Ah. I know the type. 
            
        ]],
        DIALOG_TURN_IN_INFORMANT = [[
            player:
                !thought
                Yeah, {informant.heshe} was all right, with some... convincing. 
                But {informant.heshe} wanted to skim off the top of the cache.
            agent:
                !dubious
                What? Why?
            player:
                !shrug
                Something about selling the weapons? Thought the shills would be more helpful than the excess firepower, I think.
            agent:
                Fat load of good shills do us here. Everything we buy has to come from the company store. 
                !disappoint
                {informant} and I never agree on much. Just add this to the pile. 
            player:
                !thumb
                Well, at least <i>I'm</i> honest.
                !happy
                As far as you know.
        ]],
        DIALOG_DONE = [[
            agent:
                It's good work, is what it is. 
                I'll send a team to collect the cache. Should be much safer thanks to you.
        ]],

        OPT_TELL_ABOUT_OFFER = "Tell {giver} about {informant}'s offer",
        OPT_COVER_UP_OFFER = "Cover for {informant}",

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_TURN_IN")
            :SetQuestMark()
            :Fn(function(cxt) 
                    cxt:Dialog("DIALOG_INTRO")
                    
                    if cxt.quest.param.refused_offer then
                    
                        cxt:Dialog("DIALOG_IF_REFUSED")
                    
                        cxt:Opt("OPT_TELL_ABOUT_OFFER")
                            :Dialog("DIALOG_TURN_IN_INFORMANT")
                            :ReceiveOpinion("ratted_them_out", nil, "informant")
                            :ReceiveMoney(50)
                            :CompleteQuest()
                    
                        cxt:Opt("OPT_COVER_UP_OFFER")
                            :Dialog("DIALOG_PROTECT_INFORMANT")
                            :CompleteQuest()
                    

                    elseif cxt.quest.param.took_offer then
                        cxt:Dialog("DIALOG_IF_TOOK_OFFER")
                        cxt.quest:SpawnFollowQuest(FOLLOW_UP_GET_PAID.id)
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    else
                        cxt:Dialog("DIALOG_DONE")    
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end

                    StateGraphUtil.AddEndOption(cxt)
                end)
    end)
-------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            giver:
                !crossed
                The workers here in Grout Bog outnumber the technocrats three to one. If manpower was all it took, this whole conflict would've been over before it started.
                $miscNostalgic
                Worker's rights, safety regulations—we'd have it all.
            player:
                !dubious
                $neutralDubious
                Sounds like a fantasy.
            giver:
                !eureka
                It is, so long as the Spark Barons keep a tight grip on all the latest and greatest tech. The threat of it is enough to give them power.
                If this revolution's gonna have legs, we <i>need</i> supplies. One of our scouts had a lead but {informant.heshe} hasn't reported back and I'm getting worried.
                !shrug
                I need you to check it out for me: rescue the scout if you can, and secure the supplies no matter what.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                {informant} was last seen at {informant_bar}. Try looking for {informant.himher} there.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)




FOLLOW_UP_GET_PAID = QDEF:AddFollowup()

    :AddOpinionEvents{
    
        cheated_the_system_together =  
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Skimmed some profit off the top together",
        },
        demanded_bigger_cut = 
        {
            delta = OPINION_DELTAS.DIMINISH,
            txt = "Demanded a bigger cut of a shared grift",
        },
    }

    :AddConvo()
        :Priority(CONVO_PRIORITY_HIGHEST)
        :Confront(function(cxt)     
                if cxt.location:HasTag("in_transit") then
                    return "STATE_CONFRONT"
                end
            end)
        :State("STATE_CONFRONT")
            :Loc{
                DIALOG_INTRO = [[
                    * {agent} approaches.
                    player:
                        !left
                    agent:
                        !right
                        Hey, {player}! Good news: the extra weapons pretty much sold themselves.
                        Your share comes out to {share_amount#money}.
                ]],
                OPT_DEMAND_MORE = "Demand a bigger share",
                OPT_TAKE_MONEY = "Take the money",
                DIALOG_TAKE_MONEY = [[
                    player:
                        !dubious
                        Thought we were going to give this money to the Rise?
                    agent:
                        !thumb
                        My money <i>does</i> go to the Rise. And you gotta eat, don't you?
                        Remember in the war on injustice, <i>you're</i> a resource, too. 
                        !handwave
                        See ya around, {player}.
                    * {agent} turns on {informant.hisher} heel, probably to find some new watering hole to roost at.
                        !exit
                ]],
                DIALOG_DEMAND_MORE = [[
                    player:
                        !angry
                        Hope you don't mind me saying that's <i>suspiciously</i> low. 
                    agent:
                        !shrug
                        Times are tough.
                    player:
                        !threaten
                        Yeah? For you, especially, if you don't give me my fair share. 
                ]],
                DIALOG_BIGGER_CUT = [[
                    agent:
                        Fine. I was saving <i>this</i> for the next rebel in need, but I see you don't go for cheap.
                        !give
                        Now if you'll excuse me, the Barons ain't gonna rat on themselves.
                ]],
                DIALOG_BIGGER_CUT_2 = [[
                    * {agent} turns on {informant.hisher} heel, probably to find some new watering hole to roost at.
                    agent:
                        !exit
                ]],
                DIALOG_NO_BIGGER_CUT = [[
                    agent:
                        I risk my life every day for the cause, pal. You think I'm afraid of you?
                        You just <i>got</i> here, and that's what a newcomer gets.
                        Until then, you'll take what I give you, and be glad for it. 
                        And this time, I'm not giving you anything.
                        Now if you don't mind, I'll be going.
                ]],

            }

            :Fn(function(cxt)
                    cxt.quest:GetCastMember("informant"):MoveToLocation(cxt.location)
                    cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("informant"))
                    cxt.quest:Complete()


                    cxt:Dialog("DIALOG_INTRO")
                    cxt:Opt("OPT_TAKE_MONEY")
                        :Dialog("DIALOG_TAKE_MONEY")
                        :ReceiveMoney(cxt.quest.param.share_amount)
                        :ReceiveOpinion( "cheated_the_system_together" )
                        :Travel()


                    cxt:Opt("OPT_DEMAND_MORE")
                        :ReceiveOpinion( "demanded_bigger_cut" )
                        :Dialog("DIALOG_DEMAND_MORE")
                        :Negotiation{
                            flags = NEGOTIATION_FLAGS.HOSTILE,
                            on_success = function() 
                                cxt:Dialog("DIALOG_BIGGER_CUT")
                                
                                cxt.enc:GainMoney( math.floor( cxt.quest.param.share_amount * 1.5) )
                                cxt:Dialog("DIALOG_BIGGER_CUT_2")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end,
                            on_fail = function()
                                cxt:Dialog("DIALOG_NO_BIGGER_CUT")
                                cxt:GoTo("STATE_FALLING_OUT")
                            end 
                        }
                end)
            :State("STATE_FALLING_OUT")
                :Loc{
                    OPT_ATTACK = "Attack {agent}",
                    OPT_LET_GO = "Let {agent} go",
                    DIALOG_LET_GO = [[
                        agent:
                            Long live the Rise!
                            !exit
                        * Your hard-earned shills walk away. 
                    ]],
                    DIALOG_DEAD = [[
                        agent:
                            !exit
                        * At least {agent} died doing what {agent.heshe} loved. Or you assume, anyway—you never really asked {informant.himher} anything about {informant.himher}self.
                        * You push the body off the side of the road and continue along your way.
                    ]],
                    DIALOG_SURVIVED = [[
                        agent:
                            !injured
                            I yield, dammit, I yield! You've made your point.
                            Fool on me to think we became chummy after all that. 
                            Take your shills. And look your buddy {giver} in the eyes next time {giver.heshe} asks how the job went, huh?
                            !exit
                        * You part ways, and the shills jingle merrily in your pocket with each step. 
                    ]]
                }
                :SetLooping()
                :Fn(function(cxt) 
                        cxt:Opt("OPT_ATTACK")
                            :Fn(function(cxt) cxt:GetAgent():DeltaMoney(math.floor( cxt.quest.param.share_amount * 1.5)) end)
                            :Battle{
                                on_win = function(cxt) 
                                    if cxt:GetAgent():IsDead() then
                                        cxt:Dialog("DIALOG_DEAD")            
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    else
                                        cxt:Dialog("DIALOG_SURVIVED")        
                                        cxt.enc:GainMoney( math.floor( cxt.quest.param.share_amount * 1.5) )    
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                end,
                            }
                        cxt:Opt("OPT_LET_GO")
                            :Dialog("DIALOG_LET_GO")
                            :Travel()

                    end)


--by convention, this quest is passed in its giver. This is kind of gross?
QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 },
    }
}
