
local QDEF = QuestDef.Define{
    title = "Business is Booming",
    faction = "RISE",
    desc = [[Escort an explosives expert to a dig site and perform some light industrial sabotage to delay its opening.]],
    icon = engine.asset.Texture("icons/quests/rise_business_is_booming.tex"),
    tags = {"nasty"},
    focus = QUEST_FOCUS.BALANCED,
    --disabled = true,
    rank = {2, 5},
    qtype = QTYPE.CONTRACT,

    get_fight_support = function(quest, agent, target_agent)
        if agent == quest:GetCastMember("explosives_expert") and agent:IsInPlayerParty() then
            return SUPPORT_ACTION.HELP, SUPPORT_REASON.QUEST
        end
    end,

    on_destroy = function(quest)
        if quest:GetCastMember("explosives_expert") and quest:GetCastMember("explosives_expert"):IsInPlayerParty() then
            quest:GetCastMember("explosives_expert"):Dismiss()
        end
    end,

    on_start = function(quest)
        quest:Activate("get_the_explosives")
    end,

}

:AddObjective{
    id = "get_the_explosives",
    title = "Talk to {explosives_expert}",
    --desc = "{informant} knows the location of a shipment of weapons and medicine meant for the Spark Barons. Find them and find the shipment.",
    mark = { "explosives_expert" },
    negotiation_targets = {"explosives_expert"},
    --state = QSTATUS.ACTIVE,
    on_complete = function(quest)
        quest:Activate('go_to_dig_site')
    end,
}

:AddObjective{
    id = "go_to_dig_site",
    title = "Go to the dig site",
    --desc = "{bartender} won't let {informant} leave the bar until {informant.hisher} tab is paid off.",
    mark = { "dig_site" },
    on_activate = function(quest)
        quest:AssignCastMember("dig_site")
    end,
}

:AddObjective{
    id = "return_for_reward",
    title = "Tell {giver} the job is done",
    desc = "The site has been destroyed. Go tell {giver} and collect your reward.",
    ready_for_completion = true,
    mark = { "giver" },
}

:AddLocationCast{
    cast_id = "dig_site",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("NEW_DIG_SITE"))
    end,
}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddCast{
    cast_id = "explosives_expert",
    cast_fn = function(quest, t) 
        local agent = quest:CreateSkinnedAgent("HEAVY_LABORER") --always a new person
        AgentUtil.MakeSympathizer(agent, "RISE")
        table.insert(t, agent) 
    end,
}

:AddLocationDefs{
    NEW_DIG_SITE = {
        name = "New Dig Site",
        --desc = "This is where {informant} said the cache would be. Proceed with caution.",
        plax = "EXT_SB_Bog_Worksite_1",
        indoors = false,
        map_tags = {"bog"},
    },
}

:AddOpinionEvents{
    ignored_workers = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Let workers die",

    },
    failed_to_save_workers = 
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Failed to save workers from an explosion",

    },
    protected_from_barons = 
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Protected them from the Spark Barons",
    },
    handed_over_to_barons = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Handed them over to the Spark Barons",
    },
    informed_barons = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Informed on them to the Spark Barons",
    },
    took_explosives = 
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Took explosives off their hands",
    },
    saved_workers = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Saved workers from an explosion",

    },
    --[[dealt_with_their_tab =  
    {
    },--]]
}


-------------------------------------------------------------------------------------------------------------------

--COLLECT YOUR EXPLOSIVES BUDDY

QDEF:AddConvo("get_the_explosives", "explosives_expert")
    :Loc{
        OPT_GREET = "Ask {agent} about explosives",
        DIALOG_INTRO = [[
            player:
                {giver} sent me. I, uh, imagine you know what for.
            agent:
                !palm
                Yeah. Look, my partner's pregnant.
            player:
                !scared_shrug
                Con...gratulations?
            agent:
                It changes things. I can't go with you.
                !overthere
                But I can give you instructions so you can set it up yourself.
            player:
                !dubious
                That sounds dangerous.
            agent:
                !agree
                It is. That's why I'm not going. But it's the only thing I can do for you.
        ]],
        DIALOG_REINTRO = [[
            player:
                So, about the... uh, thing.
            agent:
                Look, I told you already, I can't do it. But I can teach you how.
        ]],
        OPT_TAKE_INSTRUCTIONS = "Take the instructions",
        OPT_CONVINCE = "Convince {agent} to come with you",
        DIALOG_CONVINCE = [[
            player:
                !interest
                Look, I get you, but... what if something goes wrong?
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                !permit
                You're a demolitions pro, right?
                !overthere
                When your kid is born, do you want them working those conditions?
            agent:
                !scared
                N-no, of course not.
            player:
                !eureka
                Then come with me and make sure this gets done right.
        ]],
        DIALOG_CONVINCE_FAIL = [[
            player:
                !overthere
                Don't do it for me. Do it for the Rise.
            agent:
                !disappoint
                The Rise ain't my family. I'm sorry, but you're on your own here.
        ]],
        DIALOG_TAKE_INSTRUCTIONS = [[
            player:
                Yeah, fine. I'll take care of it, like I always do.
                Hunter, freedom fighter, demolitions expert. What's one more title, huh?
            agent:
                Take this. 
                !give
                Set the charge, back the Hesh up, and light the fuse. 
                It couldn't be any simpler.
            player:
                !dubious
                Whatever you say. 
        ]]
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_GREET")
            :Fn(function(cxt) 
                cxt:Dialog(not cxt.quest.param.heard_expert_intro and "DIALOG_INTRO" or "DIALOG_REINTRO")
                cxt.quest.param.heard_expert_intro = true
                cxt:RunLoop(function(cxt) 
                    cxt:Opt("OPT_CONVINCE")
                        :Dialog("DIALOG_CONVINCE")
                        :Negotiation()
                            :OnSuccess()
                                :Dialog("DIALOG_CONVINCE_SUCCESS")
                                :RecruitMember(PARTY_MEMBER_TYPE.ESCORT)
                                :CompleteQuest("get_the_explosives")
                                :DoneConvo()
                            :OnFailure()
                                :Dialog("DIALOG_CONVINCE_FAIL")
                    
                    cxt:Opt("OPT_TAKE_INSTRUCTIONS")
                        :ReceiveOpinion("took_explosives")
                        :Dialog("DIALOG_TAKE_INSTRUCTIONS")
                        :CompleteQuest("get_the_explosives")
                        :DoneConvo()

                    StateGraphUtil.AddEndOption(cxt)

                end)

            end)
    end)


-------------------------------------------------------------------------------------------------------------------


-- YOU MEET SOME MEAN BARONS

local function OnWinBattle(cxt) 
    if cxt.quest:GetCastMember("explosives_expert"):IsDead() then
        cxt:Dialog("DIALOG_FRIEND_DEAD")
    else
        cxt:Dialog("DIALOG_WON_BATTLE")
        cxt.quest:OpinionEvent("explosives_expert", "protected_from_barons")
    end

    StateGraphUtil.AddLeaveLocation(cxt)
end



local function OnWinBattleSolo(cxt) 
    cxt:Dialog(AgentUtil.CountAlive(cxt.enc.scratch.enemies) > 0 and "DIALOG_SURVIVORS" or "DIALOG_NO_SURVIVORS")
    StateGraphUtil.AddLeaveLocation(cxt)
end



QDEF:AddConvo("go_to_dig_site", nil)
    :TravelConfront("MEET_PATROL", function(cxt) return not cxt.quest.param.met_patrol end)
        :Loc{
            DIALOG_INTRO = [[
                * A Spark Baron patrol is blocking your way forward.
            ]],
        }
        :Fn(function(cxt) 
                cxt.quest.param.met_patrol = true
                cxt.enc.scratch.enemies  = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt:TalkTo(cxt.enc.scratch.enemies[1])
                cxt:Dialog("DIALOG_INTRO")
                local is_present = cxt.quest:GetCastMember("explosives_expert"):IsInPlayerParty()
                cxt:GoTo(is_present and "STATE_PRESENT" or "STATE_NOT_PRESENT")
            end)

    :State("STATE_PRESENT")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !left
                    !dubious
                    Hey, is that... {explosives_expert}?
                explosives_expert:
                    !right
                    !scared
                    Oh no.
                agent:
                    !dubious
                    Your foreman's been looking for you. Says they have some concerns about your requisitions.
                    Why don't you come with us, huh? We could get it all taken care of.
                explosives_expert:
                    !flinch
                    I—I—
            ]],
            OPT_GIVE_THEM_UP = "Hand over {explosives_expert}",
            DIALOG_GIVE_THEM_UP = [[
                player:
                    !left
                    !happy
                    Don't worry about it, {explosives_expert}. I'm sure it's all just a misunderstanding.
                    !overthere
                    You go along with these nice Spark Barons and I'll catch up with you later.
                explosives_expert:
                    !scared_shrug
                    But you said—!
                player: 
                    !wink
                    Be quick, now! Wouldn't wanna leave the foreman waiting.
                agent:
                    !right
                    !handwave
                    Be on your way, now, laborer. We have no business with you.
                player:
                    !salute
                    Aye-aye. 
                * {explosives_expert} limps along between the two Spark Barons, leaving nothing but a trail of whimpers in {explosives_expert.hisher} wake.
            ]],
            OPT_CONVINCE = "Convince {agent} to let {explosives_expert} go",
            DIALOG_CONVINCE = [[
                player:
                    !left
                    Sir, can we report in later, maybe?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !cagey
                    Only {explosives_expert} is feeling a little, uh, <i>explosive</i>, if you get what I mean.
                agent:
                    !angry
                    Is that a <i>threat</i>, laborer?
                player:
                    !placate
                    It's a warning! I mean we tried to make some hooch out of bogberries and {explosives_expert.heshe} tried it and—
                * The Spark Baron takes in the sweat beading on {explosives_expert}'s upper lip and the pallor shining on {explosives_expert.hisher} face.
                agent:
                    !surprised
                    Oh, <i>Hesh!</i>
                    Get out of here, both of you. And get {explosives_expert} off the road!
                    !angry
                    I don't want to be anywhere nearby when that goes off.
                * The Spark Barons make dust trails, putting as much distance as they can between themselves and the coming onslaught.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    !shrug
                    Only we have a shift to get to and—
                agent:
                    !dubious
                    Is that so? And who are <i>you</i>, anyway?
                player:
                    !placate
                    Err—
                agent: 
                    !angry
                    'Cause naturally we checked the roster first and {explosives_expert} isn't on shift for four more hours.
                    So why don't you both get down on the ground, before we put you there?
                    !fight
            ]],
            
            OPT_FIGHT = "Defend {explosives_expert}",
            DIALOG_WON_BATTLE = [[
                player:
                    !left
                explosives_expert:
                    !right
                    !palm
                    P-phew. I knew this was a mistake.
                player:
                    !handwave
                    Yeah, but it's one you've already made. May as well see it through to the end.
                    Come on.
            ]],
            DIALOG_FRIEND_DEAD = [[
                agent:
                    !exit
                * You live to scrape by for another day, but {explosives_expert} is dead. You're gonna have to figure out those explosives on your own.
            ]],
            DIALOG_FIGHT = [[
                player:
                    !left
                    !fight
                    I have an alternate proposal...
            ]]

        }
        :Fn(function(cxt) 

                cxt:Dialog("DIALOG_INTRO")
                
                cxt:Opt("OPT_GIVE_THEM_UP")
                        :Dialog("DIALOG_GIVE_THEM_UP")
                        :Fn(function() cxt.quest:GetCastMember("explosives_expert"):Dismiss() end)
                        :ReceiveOpinion("handed_over_to_barons", nil, "explosives_expert")
                        :Travel()

                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation()
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_SUCCESS")
                            :ReceiveOpinion("protected_from_barons", nil, "explosives_expert")
                            :Travel()
                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_FAIL")
                            :Fn(function() 
                                    cxt:Opt("OPT_DEFEND")
                                        :Battle{
                                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                                            on_win = OnWinBattle,                                        
                                        }
                                end)

                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = OnWinBattle,
                    }
            end)

    :State("STATE_NOT_PRESENT")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !right
                    !angry
                    Hold up, grifter. We've had an anonymous tip says you're smuggling contraband.
                player:
                    !left
                    !angry_shrug
                    Anonymous tip, huh? Sounds like an excuse just to stop me.
                agent:
                    !crossed
                    You think what you want. I hate wasting my <i>own</i> time without you doing it for me.
                    We gotta do a search. 
            ]],
            OPT_CONVINCE_INNOCENT = "Squeal on {explosives_expert}",
            DIALOG_CONVINCE = [[
                player:
                    !cagey
                    You looking for contraband? You <i>should</i> be looking at {explosives_expert}.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !shrug
                    I'm telling you, {explosives_expert} is who you want. I think {explosives_expert.HeShe}'s been skimming some explosives off the top—seems like a terrorist act if you ask me.
                agent:
                    !thought
                    Hm. {explosives_expert} is the one who told us you were up to no good.
                player:
                    !dubious
                    And you <i>believed</i> {explosives_expert.himher}?
                    You really got your nose to that trail, huh?
                agent:
                    !angry
                    ...
                    All right. We'll go have a word with {explosives_expert}. 
                    !angry_accuse
                    But don't go too far, {player}. We'll be looking for you if things don't turn out the way we'd expect.
                    !exit
                player:
                    !sigh 
                * Perhaps your finest moment, perhaps your worst—all the same, you're out of trouble for now.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    !surprised
                    Why—just look right there! Behind you! I see {explosives_expert} heading this way right now!
                agent:
                    !angry
                    Nice try.
                    !fight
            ]],
            DIALOG_NO_SURVIVORS = [[
                agent:
                    !exit
                * Another day, another pile of corpses.
            ]],
            DIALOG_SURVIVORS = [[
                agent:
                    !exit
                * The survivors make a break for the trees, and you hear the tell-tale squelch of one of them falling face-first into a bog hole. 
                * By the time they work themselves out, this job'll be over. Hopefully. 
            ]],
            DIALOG_FIGHT = [[
                player:
                    !fight
                    The only thing I'm packing... is heat.
            ]]

        }
        :Fn(function(cxt) 

                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_CONVINCE_INNOCENT")
                    :Dialog("DIALOG_CONVINCE")
                    :ReceiveOpinion("informed_barons", nil, "explosives_expert")
                    :Negotiation()
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_SUCCESS")
                            :Travel()
                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_FAIL")
                            :Fn(function() 
                                cxt:Opt("OPT_DEFEND")
                                    :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE, 
                                        on_win = OnWinBattleSolo}
                                end)
                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = OnWinBattleSolo,
                    }
            end)



-------------------------------------------------------------------------------------------------------------------


-- YOU ARE AT THE DIG SITE

QDEF:AddConvo("go_to_dig_site", nil)
    :ConfrontState("AT_THE_SITE", function(cxt) return cxt.location == cxt.quest:GetCastMember("dig_site") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * There's a perimeter of barbed wire that you make quick work of, and then you're into the mining camp. 
                * You might not have even noticed the mine, but for the ring of sharp rocks around it: a narrow crack into the earth.
                * You gaze into its snaggletoothed maw. 
            ]],
            DIALOG_INTRO_PRESENT = [[
                player:
                    !dubious
                    We don't have to go <i>inside</i>, right?
                explosives_expert:
                    !right
                    !disappoint
                    Naw. We can drop a few charges down and seal off the entrance from here. 
                    But you see why Kalandra wants it closed for good, right?
            ]],
            DIALOG_INTRO_NOT_PRESENT = [[
                * Looks like the mine is one bad wind away from a total collapse anyway. This should be simple enough.
            ]],
            OPT_SABOTAGE = "Sabotage the dig site",
            DIALOG_PRESENT = [[
                player:
                    Let's just get this over with.
                explosives_expert:
                    You said it.
                    !exit
                * {explosives_expert} unwinds a cable from {explosives_expert.hisher} pocket and makes quick work of the perimeter.
                * Along the way {explosives_expert.heshe} studs the cable with small charges, and throws the end of the cable into the hole.
                explosives_expert:
                    !right
                It's set.
                player:
                    It—it's <i>set</i>?
                    Like it's armed or it's—
                explosives_expert:
                    It's gonna explode. Like, soon. 
                    So we might wanna, uh, leave immediately.
                player:
                    You said it. 

            ]],
            DIALOG_NOT_PRESENT = [[
                * You loop a length of cable around the rocks, winding it back and forth for good measure, leaving charges studded along it like beads of dew on a spider web.
                * When you're done, they flash real prettily. And beep, too. 
                * You get the inkling it's time to get out of there. Fast.
            ]],
            OPT_VAMOOSE = "Leave the Danger Zone",
        }
        :Fn(function(cxt) 
                local is_present = cxt.quest:GetCastMember("explosives_expert"):IsInPlayerParty()
                cxt:Dialog("DIALOG_INTRO")
                cxt:Dialog(is_present and "DIALOG_INTRO_PRESENT" or "DIALOG_INTRO_NOT_PRESENT")
                cxt:Opt("OPT_SABOTAGE")
                    :Fn(function(cxt) 
                            cxt:Dialog(is_present and "DIALOG_PRESENT" or "DIALOG_NOT_PRESENT")                        
                            cxt:Opt("OPT_VAMOOSE")
                                :GoTo("STATE_MEET_WORKERS")
                        end)
            end)

    :State("STATE_MEET_WORKERS")
        :Loc{
            DIALOG_INTRO_SOLO = [[
                * You don't make it far, though, before you run smack-first into a pack idling on the trail outside the camp. Their make-up is motley—only the bags under their eyes mark them out as a group. 
                player:
                    !left
                    !surprised
                    Oh, uh...
                agent:
                    !right
                    !angry_accuse
                    H-hey, where'd you come from? We had a queue going! You can't cut in.
                player:
                    What queue?
                agent:
                    !thumb
                    The one I'm at the head of! Word is the mine's unsafe, and the Barons are expecting the workers to cause a stink.
                    'Cause of those... rabble-rousers. The lazy ones, who don't wanna do no work.
                    !overthere
                    They're gonna need remedial workers, and I got here first. So get in line.
                player:
                    !palm
                    Great.

            ]],
            DIALOG_INTRO_EXPERT =
            [[
                * {explosives_expert} pulls up on the gap in the barbed wire for you to duck your head under, but stops short as {explosives_expert.heshe} looks towards the gate at the perimeter.
                player:
                    !left
                    What is it?
                explosives_expert:
                    !right
                    !overthere
                    At the gate. There's a crowd of folks. 
                player:
                    Barons?
                explosives_expert:
                    Scabs, by the look of it. Laborers hoping to take the work if the Rise doesn't show up to their shift.
                    They weaken the movement, but they're just trying to get by, same as the rest of us.
                player:
                    Well, they're not gonna get by for long if they stay there. 
                explosives_expert:
                    Warn them if you want, but keep it vague. They're already selling us out in their own way.

            ]],
            DIALOG_INTRO_EXPERT_2 = [[
                * You make your way through the fence and wrap around to the queue forming at the gate. 
                * One of the scabs notices you, and the others follow {agent.hisher} lead like a flock of fish.
                * They look haggard and sleep-deprived, like they've been camping out for a day or two and somehow managed to avoid the beasts in the bog.
                agent:
                    !right
                    !angry_shrug
                    Another one, huh? Look, get in the back of the line. We've all been here for days waiting for that damn mine to open.
                    !spit
                    Not gonna give up my spot to some opportunistic parasite like you.
                player:
                    !crossed
                    I see.
                agent:
                    You got a problem with that, newcomer?
            ]],

            OPT_NEGOTIATE = "Convince {agent} to leave",
            DIALOG_NEGOTIATE = [[
                player:
                    !point
                    Listen, you guys gotta get outta here. Pronto.
                agent:
                    !handwave
                    Why, 'cause the mine is dangerous? Save it. 
                    We've heard the Rise propaganda. But starving is their business.
                    !hips
                    I'm not afraid of hard work.
            ]],
            DIALOG_NEGOTIATE_WIN = [[
                player:
                    !shrug
                    Look, don't you think it's odd they haven't gone ahead and opened the gates yet? You've been here for days you said, yeah?
                agent:
                    !spit
                    Just 'cause the Rise are stalling them with negotiations.
                player:
                    Yeah, but the Barons have been busy too. 
                    Think about it. It's a <b>trap</b>.
                    !interest
                    The Barons have <i>rigged</i> the place to cave-in. They wanna deal with the Rise once and for all.
                agent:
                    !dubious
                    Wha—you're joking.
                player:
                    !cagey
                    I wish I were, friend. But trust me: you don't wanna be here when the shift starts.
                    !throatcut
                    The Barons won't care if a couple of transients get caught in the blast.
                agent:
                    !scared_shrug
                    Well... maybe we've wasted enough time on this. I haven't eaten in three days.
                    This is a dead end. I-I'm gonna go look for work elsewhere. 
                * One loose thread was all it took, and the knot unravels. The rest of the scabs waste no time following their leader. 
                    !exit
                player:
                    !exit
            ]],
            DIALOG_NEGOTIATE_LOSE = [[
                player:
                    !interest
                    You should listen to the Rise, bud. They only want what's best for you and <i>all</i> the workers.
                agent:
                    !angry
                    Yeah? Well I want what's best for my belly. And that means pay. 
                    The Rise just wants us to fall in line so we don't see them for what they really are: lazy slobs.
                player:
                    !angry_shrug
                    Look you ingrate—
                * You catch sight of {explosives_expert}, gesturing wildly from the tree line. It's time to go.
                player:
                    !palm
                    Aw, hesh. You've made your grave, scabs. You can lie in it. 
                    !exit
            ]],

            OPT_LEAVE = "Leave the workers to their fate",
            DIALOG_LEAVE = [[
                player:
                    No, just leaving.
                    !exit
            ]],
        }
        :Fn(function(cxt) 

                local worker = AgentUtil.GetFreeAgent("LABORER")
                

                local is_present = cxt.quest:GetCastMember("explosives_expert"):IsInPlayerParty()
                if is_present then
                    cxt:Dialog("DIALOG_INTRO_EXPERT")
                    worker:MoveToLocation(cxt.location)
                    cxt.enc:SetPrimaryCast(worker)
                    cxt:Dialog("DIALOG_INTRO_EXPERT_2")
                else
                    worker:MoveToLocation(cxt.location)
                    cxt.enc:SetPrimaryCast(worker)
                    cxt:Dialog("DIALOG_INTRO_SOLO")
                end
                
                cxt:Opt("OPT_NEGOTIATE")
                    :Negotiation()
                        :OnSuccess()
                            :Dialog("DIALOG_NEGOTIATE_WIN")
                            :Fn(function() cxt.quest.param.workers_saved = true end)
                            :GoTo("STATE_EXPLOSION")
                        :OnFailure()
                            :Dialog("DIALOG_NEGOTIATE_LOSE")
                            :Fn(function() cxt.quest.param.workers_failed_to_convince = true worker:Kill() end)
                            :GoTo("STATE_EXPLOSION")

                cxt:Opt("OPT_LEAVE")
                    :Dialog("DIALOG_LEAVE")
                    :Fn(function(cxt) 
                            cxt.quest.param.workers_did_not_try = true
                            worker:Kill()
                        end)
                    :GoTo("STATE_EXPLOSION")


            end)
    :State("STATE_EXPLOSION")
        :Loc{
            DIALOG_EXPLOSION = [[
                agent:
                    !exit
                player:
                    !exit
                
                * You run to a safe distance, and wait for the explosion.
                * BOOM!
                * The Barons aren't going to be using that dig site any time soon.
            ]],
            DIALOG_LEAVE = [[
                * You leave before the Spark Barons arrive.
            ]],
            DIALOG_EXPERT_MAD = [[
                player:
                    !left
                explosives_expert:
                    !right
                    !angry
                    You didn't even try to help those people!
                    I knew this was a bad idea.
                    I'm done with the Rise. You people are all the same!
                    !exit
            ]],
            DIALOG_EXPERT_SAD = [[
                player:
                    !left
                explosives_expert:
                    !right
                    They're dead. We killed them. 
                    We're murderers.
                    I need to be alone now.
                    !exit
            ]],
            DIALOG_EXPERT_OK = [[
                player:
                    !left
                explosives_expert:
                    !right
                    We did it!
                    And nobody got hurt!
                    Thanks, {player}.
                    I'll be seeing you around.
                    !exit
            ]],
        }
        :Fn(function(cxt) 

                cxt.enc:GetScreen():ClearHistory()
                cxt.enc:GetScreen():Fade( 1, false )
                cxt.enc:Wait(.25)
                cxt:Dialog("DIALOG_EXPLOSION")
                
                local is_present = cxt.quest:GetCastMember("explosives_expert"):IsInPlayerParty()
                if is_present then
                    cxt.enc:GetScreen():Fade( 1, true )
                    if cxt.quest.param.workers_saved then
                        cxt:Dialog("DIALOG_EXPERT_OK")
                        cxt.quest:OpinionEvent("explosives_expert", "saved_workers")
                    elseif cxt.quest.param.workers_did_not_try then
                        cxt:Dialog("DIALOG_EXPERT_MAD")
                        cxt.quest:OpinionEvent("explosives_expert", "ignored_workers")
                    elseif cxt.quest.param.workers_failed_to_convince then
                        cxt:Dialog("DIALOG_EXPERT_SAD")
                        cxt.quest:OpinionEvent("explosives_expert", "failed_to_save_workers")
                    end

                    cxt.quest:GetCastMember("explosives_expert"):Dismiss()
                end
                cxt.quest:Complete("go_to_dig_site")
                cxt.quest:Activate("return_for_reward")
                cxt:Dialog("DIALOG_LEAVE")
                StateGraphUtil.AddLeaveLocation(cxt)

                
            end)

-------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Tell {giver} the job is done",
        
        DIALOG_WORKERS_OK = [[
            player:
                It's done. The thing got blown up.
            giver:
                Good work. Now they'll know we mean business.
        ]],

        DIALOG_WORKERS_DIED = [[
            player:
                It's done. The thing got blown up.
            giver:
                Yeah, so did a bunch of the hearts and minds we need to capture.
                Try harder next time.
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :SetQuestMark()            
            :Dialog(cxt.quest.param.workers_saved and "DIALOG_WORKERS_OK" or "DIALOG_WORKERS_DIED")
            :CompleteQuest()
    end)

-------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                The Barons are bringing a new dig site on-line in the deep bog.
                We were negotiating the labor contracts with them, but they rushed the project ahead before we were done and are opening it without our approval.
                We need to send them a message about the importance of keeping the lines of dialog open.
                An <i>explosive</> message, if you catch my drift.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                Alright, so go and find {explosives_expert} and get their kit.
                Try not to hurt any bystanders - we just want to delay the opening of the site, not put our own people under ground.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)
