local battle_defs = require "battle/battle_defs"

local TOTAL_ROBOTS = 3

--need to loc this
local passwords = 
{
    "Vermillion",
    "Incandescent",
    "Hydrant",
    "Panoramic",
    "Qualification",
    "Superintendent",
    "Contemporary",
    "Memorandum",
    "Remunerate",
}


Content.AddBattleCard("automech_access_code", {
    name = "Automech Access Code",
    desc = "Activate one of the damaged Spark Baron Enforcers to fight by your side.",
    cost = 3,
    flavour = "'It's all encrypted so you need to make sure you have something like this if you want them to listen to you.'",
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    flags = battle_defs.CARD_FLAGS.EXPEND | battle_defs.CARD_FLAGS.REPLENISH | battle_defs.CARD_FLAGS.ITEM,
    target_type = TARGET_TYPE.SELF,

    CanPlayCard = function( self, battle, target )
        return self.owner:GetTeam():NumActiveFighters() < self.owner:GetTeam():GetMaxFighters(), CARD_PLAY_REASONS.TEAM_FULL
    end,
    OnPostResolve = function( self, battle, attack )
        local agent = Agent( "SPARK_BARON_AUTOMECH" )
        agent:GetAspect("health"):SetPercent(math.randomGauss(.4, .6))
        local fighter = Fighter.CreateFromAgent( agent, battle:GetScenario():GetAllyScale() )
        self.owner:GetTeam():AddFighter( fighter )
        self.owner:GetTeam():ActivateNewFighters()
    end,
})


Content.AddNegotiationModifier( "get_automech_code",
{
    name = "Automech Code",
    desc = "When destroyed, grants you one access code for an automech.",

    max_resolve = 15,
    modifier_type = MODIFIER_TYPE.BOUNTY,

    --sound = "event:/sfx/battle/cards/neg/create_argument/discovers_cave",

    loc_strings =
    {
        DIALOG =
        [[
            right:
                Ok, I can give you one...
        ]],
    },

    OnInit = function( self )
        if self.engine then
            self.max_resolve = 4 + 2 * self.engine:GetDifficulty()
            self.resolve = self.max_resolve
        end
    end,

    OnBounty = function( self )
        if self.engine.start_params.quest then
            self.engine.start_params.quest.param.have_codes = (self.engine.start_params.quest.param.have_codes or 0) + 1
        end
    end,

    OnRemoveWidget = function( self, widget, panel )
        panel.encounter:Dialog( self.def:GetLocalizedString( "DIALOG" ))
        panel:Delay( 1.0 )
    end,
})




local QDEF = QuestDef.Define{
    title = "Core Collector",
    qtype = QTYPE.CONTRACT,
    faction = "RISE",
    tags = {"nasty"},
    rank = {1, 5},
    desc = [[Some Baron automechs have broken down during a field test.]],
    icon = engine.asset.Texture("icons/quests/rook_story_rise_core_collector.tex"),
    focus = QUEST_FOCUS.COMBAT,
    on_init = function(quest) 
        quest.param.correct_codeword = table.arraypick(passwords)
    end,
    on_start = function(quest)
        quest:Activate("liberate_cores")
    end,
}


:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddLocationDefs{
    CALIBRATION_POINT = {
        name = "Calibration Point",
        desc = "The Barons are calibrating new enforcers here before deploying them in the field.",
        --icon = engine.asset.Texture("UI/location_cave.tex"),
        plax = "EXT_SB_ResearchTent",
        indoors = false,
        map_tags = {"bog"},
    },
}


:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("CALIBRATION_POINT"))
    end,
}


:AddObjective{
    id = "liberate_cores",
    title = [[Steal as many robot processing cores as you can.]],
    --desc = "Fight {target}, and make sure they don't survive.",
    mark = { "location" },
    negotiation_targets = {"target"},
    combat_targets = {"target"},

    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[Take the cores back to {giver}.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}


QDEF:AddConvo("liberate_cores")
    :ConfrontState("asdf", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
        :Loc{
            DIALOG_INTRO = [[
                * As you approach the testing grounds, you hear a myriad of clanging, banging, screeching, and swearing.
                * You find a good hiding place, and peer out.
                tech:
                    !right
                    !scared
                guard:
                    !left
                    !angry
                tech:
                    I told you already! This is delicate equipment! It takes time to get it right!
                guard:
                    The longer we're out here, the longer the Rise has to find us.
                    !angry_permit
                    So get back to work! And hurry up!
                    !exit
                * The technician returns to {tech.hisher} work. The guard stomps off to patrol the area.
                tech:
                    !angry
                    Heshing Barons. If I didn't need the money...
                * You notice that {tech} is holding a list of activation codes for the mechs.
                *** A technician is trying to finish the mech calibration, while a contingent of Baron guards patrol the area.
            ]],
            DIALOG_GET_ATTENTION_FRIEND = [[
                player:
                    !left
                    Hey, {tech}.
                tech:
                    !surprised
                    {player}! What are you doing here?
                player: 
                    !placate
                    Shhhh. I need your help.
                tech:
                    I'm listening...
            ]],

            DIALOG_GET_ATTENTION = [[
                player:
                    !left
                    Hey.
                tech:
                    !surprised
                    Who? What?!
                * {tech} clutches the mech access codes more tightly.
                player: 
                    !placate
                    Shhhh. I have a proposal for you.
                tech:
                    !cagey
                    What is it?
                * {tech} glances warily in the direction of {guard}. You had better act fast. 
            ]],
            OPT_PAY_FOR_ACCESS_CODES = "Pay {agent} to give you access codes",
            DIALOG_PAY_FOR_ACCESS_CODES = [[
                player:
                    You give me those codes and walk away. I'll make it worth your while.
                tech:
                    Well...
                    I can't say no to the money. But if anyone asks, I'll say you threatened me!
                player:
                    Suits me.
                * {tech} hands you the codes and runs away.
            ]],


            OPT_THREATEN = "Threaten to kill {agent} if you don't get the access codes",
            DIALOG_THREATEN = [[
                player:
                    Hand over the codes or I'm going to hurt you.
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                agent:
                    !scared
                    Why are you doing this to me?
                    Here! Take them!
                    !exit
                * {agent} runs away as the guards return.
            ]],
            DIALOG_THREATEN_FAIL = [[
                agent:
                    !angry
                    Who do you think you are?
                    !exit
                * {agent} runs to get the guards.
            ]],

            OPT_CONVINCE = "Convince {agent} to hand over the codes",
            DIALOG_CONVINCE = [[
                player:
                    You don't look like a Spark Baron. Why are you working with them, anyway?
                    You should help me help some people help your people and hand over those codes.
            ]],
            DIALOG_CONVINCE_FRIEND = [[
                player:
                    I know you're not Spark Baron.
                    Give me the codes, so I can help the Rise.
            ]],
            DIALOG_CONVINCE_SUCCESS_FRIEND = [[
                agent:
                    You're right. Here, take them.
                player:
                    Now get to safety. This might get loud.
                agent:
                    !exit
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                agent:
                    Enough with the recruitment speech. I agree with you already.
                player:
                    Now get out of here, before you blow my cover.    
                agent:
                    !exit
            ]],
            DIALOG_CONVINCE_FAIL = [[
                agent:
                    You're going to get me in trouble!
                * You are interrupted by {guard} returning.
            ]],

            OPT_FIGHT_ALONE = "Beat up {agent} and take the codes",
            DIALOG_FIGHT_ALONE = [[
                player:
                    !fight
                    Nothing personal, but you have something I need.
            ]],
            DIALOG_FIGHT_ALONE_KILLED = [[
                * You take the codes from {tech}'s corpse.
            ]],
            DIALOG_FIGHT_ALONE_ALIVE = [[
                tech:
                    Oh Hesh, please don't hurt me!
                * You snatch the codes away.
                tech:
                    Help! Help! This crazy grifter tried to kill me!
                    !exit
                * {tech} runs away.
            ]],
            DIALOG_POST_FIGHT_GUARDS_ARRIVE = [[
                guard:
                    !right
                    What was all that noise?
                    !surprised
                    What are you doing?!
            ]],

            OPT_ACTIVATE_A_MECH = "Ignore {tech} and use the Rise codeword",
            --OPT_

            DIALOG_SAY_CODEWORD = [[
                * You step out in front of the robots, ready to activate the hacked one.
                player:
                    !left
                tech:
                    What the hell?!
                    !exit
                * {tech} runs to get the patrol.
                mech:
                    !right
                player:
                    !thought
                * Now, what was the right codeword, again?
            ]],

            DIALOG_WRONG_WORD = [[
                player:
                    Uh.... '{1}'.
                * Nothing happens.
                * The guard patrol returns.
                guard:
                    !right
                    !fight
                    Stop!
            ]],
            DIALOG_RIGHT_WORD = [[
                player:
                    Hey! '{1}'!
                mech:
                    OVERRIDE PROTOCOL ACTIVATED
                    LONG LIVE THE RISE
                    !exit
                * The guard patrol returns, attracted by the noise.
                guard:
                    !right
                    !fight
                    Stop!
            ]],
            OPT_GET_ATTENTION = "Get {tech}'s attention",
            OPT_SAY_THE_CODEWORD = "Use the stolen Rise codeword",
            OPT_SAY_CODEWORD = "... '{1}'",
            CODES_TAKEN = "Automech Codes Taken: {1}",
            TT_PASSWORD = "You remember the password, right? Rook certainly doesn't.",

        }
        :Fn(function(cxt) 
            
            local tech = AgentUtil.GetFreeAgent("FOREMAN")
            tech:MoveToLocation(cxt.location)
            cxt:ReassignCastMember("tech", tech)

            local mech = AgentUtil.GetFreeAgent("SPARK_BARON_AUTOMECH")
            mech:MoveToLocation(cxt.location)
            cxt:ReassignCastMember("mech", mech)
            
            cxt.enc.scratch.guards = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location)            
            cxt:ReassignCastMember("guard", cxt.enc.scratch.guards[1])

            cxt.enc:SetPrimaryCast(tech)
            cxt.enc:SetPrimaryCast(cxt.enc:GetCastAgent("tech"))
            cxt:Dialog("DIALOG_INTRO")
                
            cxt:Opt( "OPT_GET_ATTENTION")
                :Dialog(cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_GET_ATTENTION_FRIEND" or "DIALOG_GET_ATTENTION")
                :Fn(function() 
                    cxt:Opt("OPT_PAY_FOR_ACCESS_CODES")
                        :Dialog("DIALOG_PAY_FOR_ACCESS_CODES")
                        :DeliverMoney(50 + cxt.quest:GetRank()*50)
                        :Fn(function()  
                            cxt.enc:GetCastAgent("tech"):MoveToLimbo()
                            cxt.quest.param.have_codes = TOTAL_ROBOTS
                        end)
                        :GoTo("STATE_FIGHT_GUARDS")

                    cxt:Opt("OPT_THREATEN")
                        :Dialog("DIALOG_THREATEN")
                        :Negotiation{
                            flags = NEGOTIATION_FLAGS.INTIMIDATION,

                            on_start_negotiation = function(minigame)
                                for k = 1, TOTAL_ROBOTS do
                                    minigame.opponent_negotiator:CreateModifier( "get_automech_code" )
                                end
                            end,
                            reason_fn = function(minigame)
                                return loc.format(cxt:GetLocString("CODES_TAKEN"), cxt.quest.param.have_codes or 0 )
                            end,

                            on_success = function() 
                                cxt:Dialog("DIALOG_THREATEN_SUCCESS")
                                cxt:GoTo("STATE_FIGHT_GUARDS")
                            end,
                            
                            on_fail = function() 
                                cxt:Dialog("DIALOG_THREATEN_FAIL")
                                cxt.quest.param.tech_hostile = true
                                cxt:GoTo("STATE_FIGHT_GUARDS")
                            end,
                        }

                    cxt:Opt("OPT_CONVINCE")
                        :Dialog(cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_CONVINCE_FRIEND" or "DIALOG_CONVINCE")
                        :Negotiation{
                            
                            on_start_negotiation = function(minigame)
                                for k = 1, TOTAL_ROBOTS do
                                    minigame.opponent_negotiator:CreateModifier( "get_automech_code" )
                                end
                            end,
                            reason_fn = function(minigame)
                                return loc.format(cxt:GetLocString("CODES_TAKEN"), cxt.quest.param.have_codes or 0 )
                            end,

                            on_success = function() 
                                cxt:Dialog(cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_CONVINCE_SUCCESS_FRIEND" or "DIALOG_CONVINCE_SUCCESS")
                                cxt:GoTo("STATE_FIGHT_GUARDS")
                            end,

                            on_fail = function() 
                                cxt:Dialog("DIALOG_CONVINCE_FAIL")
                                cxt:GoTo("STATE_FIGHT_GUARDS")
                            end,
                        }

                    cxt:Opt("OPT_FIGHT")
                        :Dialog("DIALOG_FIGHT_ALONE")
                        :Battle{

                            flags = BATTLE_FLAGS.NO_BYSTANDERS,
                            enemies = {cxt:GetAgent()},
                            on_win = function() 
                                cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_FIGHT_ALONE_KILLED" or "DIALOG_FIGHT_ALONE_ALIVE")
                                cxt.quest.param.have_codes = TOTAL_ROBOTS
                                cxt:GoTo("STATE_FIGHT_GUARDS")
                            end,
                        }
                end)

            cxt:Opt("OPT_SAY_THE_CODEWORD")
                :Dialog("DIALOG_SAY_CODEWORD")
                :PostText("TT_PASSWORD")
                :Fn(function() 
                    local NUM_OPTIONS = 4
                    local words = shallowcopy(passwords)
                    table.arrayremove(words, cxt.quest.param.correct_codeword)
                    table.shuffle(words)
                    local to_remove = #words - (NUM_OPTIONS-1)
                    if to_remove > 0 then
                        for k = 1, to_remove do
                            table.remove(words)
                        end
                    end
                    table.insert(words, cxt.quest.param.correct_codeword)
                    table.shuffle(words)

                    for _, word in ipairs(words) do
                        local opt = cxt:Opt("OPT_SAY_CODEWORD", cxt.quest:GetLocalizedStr( word ))   

                        if word == cxt.quest.param.correct_codeword then
                            opt:Fn(function()
                                cxt:Dialog("DIALOG_RIGHT_WORD", cxt.quest:GetLocalizedStr( word ))
                                cxt.quest.param.hacked_robot = true
                                cxt.quest.param.tech_hostile = true
                                cxt:GoTo("STATE_FIGHT_GUARDS")
                            end)
                        else
                            opt:Fn(function()
                                cxt:Dialog("DIALOG_WRONG_WORD", cxt.quest:GetLocalizedStr( word ))
                                cxt.quest.param.tech_hostile = true
                                cxt:GoTo("STATE_FIGHT_GUARDS")
                            end)
                        end
                    end
                end)

        end)

    :State("STATE_FIGHT_GUARDS")
    
        :Loc{
            DIALOG_GUARD_ARRIVE = [[
                guard:
                    !right
                    !angry
                    We're going to grind you into paste, grifter!
            ]],

            DIALOG_TECH_HOSTILE = [[
                guard:
                    !right
                    !angry
                    Get those robots up and running! We're going to teach this grifter a lesson!
                tech:
                    !right
                    !salute
                    Yes sir!
                    * The technician starts entering codes into the inactive automechs. You had better put down the guards quickly.
            ]],

            DIALOG_HACKED_ROBOT = [[
                mech:
                    !left
                    THREAT DETECTED
                    ENGAGING
                * The automech that you hacked whirrs to life.
            ]],
            DIALOG_HAVE_CODES = [[
                * You fumble with the automech access codes. You're going to have to enter them as you fight if you want the automechs to fight with you.
            ]],
            DIALOG_DEFEND = [[
                player:
                    !left
                    !fight
                    Alright, let's do this.
            ]],
            
            DIALOG_WON_FIGHT = [[
                right:
                    !exit
                player:
                    !left
            ]],
            OPT_DEFEND = "Defend yourself!"

        }
        :Fn(function(cxt) 
            
            
            
        
            local NUM_TECH_CODES = math.max(0, TOTAL_ROBOTS - (cxt.quest.param.have_codes or 0) - ( cxt.quest.param.hacked_robot and 1 or 0))
            
            local allies = {}

            cxt:Dialog("DIALOG_GUARD_ARRIVE")
            
            if cxt.quest.param.tech_hostile and NUM_TECH_CODES > 0 then
                cxt:Dialog("DIALOG_TECH_HOSTILE")
            elseif cxt.quest.param.have_codes and cxt.quest.param.have_codes > 0 then
                cxt:Dialog("DIALOG_HAVE_CODES")
            end

            if cxt.quest.param.hacked_robot then
                cxt:Dialog("DIALOG_HACKED_ROBOT")
                table.insert(allies, cxt.enc:GetCastAgent("mech"))
            end


            local skip_turns = 5
            local num_robots_spawned = 0
            local msg = ""

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.SELF_DEFENCE,
                    allies = allies,
                    enemies = cxt.enc.scratch.guards,
                    on_win = function() 
                        cxt:Dialog("DIALOG_WON_FIGHT")
                        if cxt.quest.param.tech_hostile then
                            cxt:GoTo("STATE_DEAL_WITH_TECH")
                        else
                            cxt:GoTo("STATE_LEAVE")
                        end
                    end,

                    reason_fn = function(battle)
                        if cxt.quest.param.tech_hostile then
                            return msg
                        end
                    end,
                    
                    on_player_turn_start = function(battle)
                        msg = nil
                        if cxt.quest.param.tech_hostile then
                            if num_robots_spawned < NUM_TECH_CODES and battle:GetTeam(TEAM.RED):NumActiveFighters() < battle:GetTeam(TEAM.RED):GetMaxFighters() then
                                skip_turns = skip_turns - 1
                                if skip_turns <= 0 and not battle:GetTeam(TEAM.RED):IsSurrendering() then
                                    skip_turns = math.random(4, 6)
                                    local agent = Agent( "SPARK_BARON_AUTOMECH" )
                                    agent:GetAspect("health"):SetPercent(math.randomGauss(.4, .6))
                                    local fighter = Fighter.CreateFromAgent( agent, battle:GetScenario():GetEnemyScale() )
                                    battle:GetTeam(TEAM.RED):AddFighter( fighter )
                                    battle:GetTeam(TEAM.RED):ActivateNewFighters()
                                    num_robots_spawned = num_robots_spawned + 1
                                    msg = LOC "UI.FIGHT_SCREEN.REINFORCEMENTS_ACTIVE"
                                else
                                    msg = loc.format( LOC "UI.FIGHT_SCREEN.REINFORCEMENTS_IN", skip_turns)
                                end 
                            end
                        end
                    end,

                    on_start_battle = function(battle) 
                        if cxt.quest.param.have_codes and cxt.quest.param.have_codes > 0 then
                            for k = 1, cxt.quest.param.have_codes do
                                local card = Battle.Card( "automech_access_code", battle:GetTeam(TEAM.BLUE):Primary() )
                                card.show_dealt = true                    
                                battle:DealCard( card, battle:GetDrawDeck() )
                            end
                        end
                    end,
                }
        end)

    :State("STATE_DEAL_WITH_TECH")
        :Loc{
            DIALOG_TECH_HOSTILE_INTRO = [[
                tech:
                    !right
                    !scared
                * You corner the technician.
                player:
                    What should I do with you?
                tech:
                    !flinch
                    I can keep my mouth shut! I swear!
            ]],
            OPT_KILL_TECH = "Kill {tech}",
            DIALOG_KILL_TECH = [[
                player:
                    Sorry, I can't leave any loose ends.
                tech:
                    No!
                    !exit
            ]],

            OPT_LET_TECH_GO = "Let {tech} go",
            DIALOG_LET_TECH_GO = [[
                player:
                    Go on - get out of here. 
                tech:
                    Really? Oh thank you! I never liked the Barons anyway - I was just doing what I was told. You're not so bad after all, you know...
                player:
                    Now! Before I change my mind!
                tech:
                    Going!
                    !exit
            ]],

            OPT_ROB_TECH = "Demand money from {tech}",
            DIALOG_ROB_TECH = [[
                player:
                    Empty your pockets, and you can live.
                tech:
                    I - 
                player:
                    I don't want to kill you, but I don't mind doing it if I have to!
                tech:
                    Yes, sir.
                    !exit
                * {tech} practically throws the money at you as {tech.heshe} leaves.
            ]],

        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_TECH_HOSTILE_INTRO")

                cxt:Opt("OPT_KILL_TECH")
                    :Dialog("DIALOG_KILL_TECH")
                    :ReceiveMoney(60)                    
                    :Fn(function() 
                        AgentUtil.CommitMurder(cxt:GetAgent()) 
                        cxt:GetAgent():Kill() 
                    end)
                    :GoTo("STATE_LEAVE")

                cxt:Opt("OPT_LET_TECH_GO")
                    :Dialog("DIALOG_LET_TECH_GO")
                    :ReceiveOpinion(OPINION.SPARED_LIFE_LOVED)
                    :GoTo("STATE_LEAVE")
            end)

    :State("STATE_LEAVE")
        :Loc{
            DIALOG_INTRO = [[
                * You collect up the automech processing cores that lay strewn about. The remainder come out of the silent automechs with a satisfying pop.
            ]],
        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("liberate_cores")
                StateGraphUtil.AddLeaveLocation(cxt)
            end)


QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_INTRO = [[
            player:
                Here are the cores.
            agent:
                That's a nice haul.
                Surprisingly good work, grifter.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_INTRO")
            :CompleteQuest()
    end)    



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                Have you seen our Cobblebots yet?
                We build them ourselves out of scrap, and they're almost a match for the Barons' Enforcers.
                But we're low on processing cores, and need more.
                The Barons are calibrating a new batch of Enforcers out in the Bog. 
                I want you to go there and steal as many processing cores as you can.
            *** {agent} wants you to steal parts from a batch of Baron automechs.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)

QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                One of the mechs has a back-door installed. Just say the word <b>"{1}"</> to it, and it'll fight for you.
                Collect as many cores as you can - each one you bring back means we can build another Cobblebot for the Rise.
                * The automech backdoor password is "<b>{1}</>". You try to remember not to forget it.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO", cxt.quest:GetLocalizedStr( cxt.quest.param.correct_codeword ))
            cxt.quest:Activate("liberate_cores")
        end)

for i, password in ipairs( passwords ) do
    QDEF:AddLocString( password:upper(), password )
end



    --[=[:State("STATE_JAKES")
        :Loc{
            DIALOG_INTRO = [[
                * You are only alone for a few moments when a Jake emerges from the gloom.
                jake:
                    !right
                    Nice work, grifter.
                    Of course, it's only really nice work if you get to taste the fruits of your labor, right?
                    Listen. I know you're working with the Rise. It's obvious.
                    Let me dig through this mess and take a couple of the best bits, and I'll reward you.

            ]],
        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
                StateGraphUtil.AddLeaveLocation(cxt)
            end)
--]=]
