Content.AddNegotiationModifier( "convince_worker_to_fight_erchins",
{
    name = "Rally Worker",
    desc = "When destroyed, another worker is rallied to the fight.",

    max_resolve = 15,
    modifier_type = MODIFIER_TYPE.BOUNTY,

    --sound = "event:/sfx/battle/cards/neg/create_argument/discovers_cave",

    loc_strings =
    {
        DIALOG =
        [[
            Right:
                Alright. One more.
        ]],
    },

    OnInit = function( self )
        if self.engine then
            self.max_resolve = 4 + 2 * self.engine:GetDifficulty()
            self.resolve = self.max_resolve
        end
    end,

    OnBounty = function( self )
        self.engine.start_params.quest.param.workers_rallied = (self.engine.start_params.quest.param.workers_rallied or 0) + 1
    end,

    OnRemoveWidget = function( self, widget, panel )
        panel.encounter:Dialog( self.def:GetLocalizedString( "DIALOG" ))
        panel:Delay( 1.0 )
    end,
})


local QDEF = QuestDef.Define{
    title = "Let Sleeping Bogs Die",
    qtype = QTYPE.CONTRACT,
    faction = "RISE",
    rank = {1, 3},
    tags = {"short"},
    reward_size = QREWARDSIZE.SMALLER,
    desc = [[Workers working a wildcat dig disturbed a nest of vicious erchin. Help them contain the mess before they're exposed.]],
    icon = engine.asset.Texture("icons/quests/rise_cover_up_wildcat_dig.tex"),
    focus = QUEST_FOCUS.COMBAT,
}


:AddOpinionEvents{
    got_workers_killed =  
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Got some of their friends killed fighting erchins.",
    },
    killed_a_erchin_nest_solo =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Destroyed an entire erchin nest for them, single-handed.",
    },
}



:AddCastByAlias{
    cast_id = "handler",
    alias = "KALANDRA",
    no_validation = true,
}

:AddObjective{
    id = "go_to_the_dig_site",
    title = [[Rescue the workers at {dig_site}]],
    desc = "The workers will need your help dealing with the creatures.",
    mark = { "dig_site" },
    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {handler}",
    desc = [[Report back to {handler}.]],
    ready_for_completion = true,
    mark = { "handler" },
    terminal = true,
}

:AddLocationCast{
    cast_id = "dig_site",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("WILDCAT_DIG"))
    end,
}

:AddLocationDefs{
    WILDCAT_DIG = {
        name = "Infected Dig Site",
        map_tags = {"bog"},
        plax = "Ext_Bog_Illegal_Worksite_1",
        indoors = false,
    },
}

local CHANCE = .25

QDEF:AddConvo("go_to_the_dig_site")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("dig_site") end)
    :Loc{
        DIALOG_INTRO = [[   
            * You arrive at the dig site. You find a clutch of workers hunkered down a fair distance from the pit.
            * It doesn't take long to notice why.
            player:
                !left
                !wince2
            * If you tried to describe the smell, you'd only make yourself sick.
            player:
                !left
                $neutralTakenAback
                By the Abyss...
            agent:
                !right
                $miscRelieved
                Oh, thank Hesh. Did {handler} send you?
                !overthere
                The hole's full of erchins, with nowhere to go but up. 
            player:
                !neutralResigned
                !wince_critical
                Yes, so the odor suggests. I don't suppose you've made any progress on your own?
            agent:
                !agree
                Sure we did. We called {handler}, and she sent you. 
                !overthere
                So... go on, then. Get to work. 

            *** The workers are waiting for you to do something about the erchin nest.
        ]],

        OPT_RALLY_WORKERS_TO_CHARGE_THE_NEST = "Convince the workers to charge the nest without you",
        TT_OPT_RALLY_WORKERS_TO_CHARGE_THE_NEST = "The more workers you convince to charge, the better the chance they'll have at surviving.",
        DIALOG_RALLY_WORKERS_TO_CHARGE_THE_NEST = [[
            player:
                !wink
                $miscAlluring
                Fortunately for you, I happen to be a master tactician. 
                $neutralDirect
                You'll all charge in, and I'll stay back to catch any that might escape the bloodbath, and ready the incineration.
        ]],
        DIALOG_RALLY_SUCCESS_LOW = [[
            player:
                !eureka
                $miscPersuasive
                Surely with your mettle and my brains, we can't fail.
            agent:
                !scared_shrug
                $scaredDefault
                S-sure. Yeah. I mean, how hard could it be. We've dealt with worse. 
                Right, guys?
            * Not all the workers seem convinced, but enough give a nod that {agent} summons {agent.hisher} courage.
            player:
                !overthere
                $miscPersuasive
                In you go, then.
                !exit
            agent:
                $scaredDefault
                Here goes.
                !exit
            * What the charge lacks in spirit it makes up for in noise. 
        ]],

        DIALOG_RALLY_SUCCESS = [[
            player:
                !eureka
                $miscPersuasive
                You lot are the backbone of Grout Bog. Surely a few pests can't slow you down?
            agent:
                !thought
                $scaredStammering
                H-hey, that's right. We work for {handler}. Nothing in the bog can stop us.
                !happy
                I guess we <i>can</> do this!
            player:
                !exit
            agent:
                C'mon, you louts! Let's show those erchins who really owns that hole!
                !exit
            * The throng charges ahead, not seeming the least bothered by the ravenous hissing coming out of the hole.
        ]],
        DIALOG_RALLY_NOBODY_DIED = [[
            * Amazingly, they manage to drive the beasts back without anyone getting themselves killed.
            * As soon as the last erchin is out of the way, you step in and begin torching the nest. 
            player:
                !left
            agent:
                !right
                !happy
                $happyAmazed
                We did it!
            player:
                !happy
                $happyDefault
                What did I tell you, old chums? Teamwork, eh?
                !overthere
                $miscPersuasive
                Now, you lot clean up this dig site, and I'll report to {handler} that there's nothing more to worry about.
            agent:
                Phew. Thanks for all your help, {player}.
            player:
                All in a day's work, my friend. 
            * You head back down the road as the workers celebrate with a rousing cheer. 
        ]],

        DIALOG_RALLY_WORKER_DIED = [[
            * Despite your stirring words, you don't manage to make soldiers out of laborers. 
            * They take losses. You shrug it off and, with the last erchin down, set to work torching the nest.
            * You torch the nest, and then address the survivors.
            player:
                !left
            agent:
                !injured
                !right
                $angrySeething
                That's it? {handler} sent you to help!
                $angryDefault
                We lost good people to those blasted beasts!
            player:
                !angry_accuse
                $angryDefault
                If you're seeking someone to blame, look no further than yourself.
                If you were more careful, your friends wouldn't be dead. 
            agent:
                ...
            player:
                You knew the risks of a wildcat dig, I understand.
            agent:
                W-we did.
            player:
                Then live by that understanding. And die by it, as you will.
                !exit
            * You leave the workers to count their dead. 
        ]],
        RALLY_NEGOTIATION_REASON = "Convince workers to fight:\n{1} convinced",

        OPT_ATTACK_THE_NEST_SOLO = "Attack the nest by yourself",
        DIALOG_ATTACK_THE_NEST_SOLO = [[
            player:
                !hips
                $neutralResigned
                Very well. Stand back and observe. 
                !exit
            agent:
                !exit
            * You sneak up on the erchins, lining up your first shot.
            * But clearly their own stink doesn't stop them from catching you on the wind. The nest is waiting for you.
        ]],
        DIALOG_WON_AGAINST_THE_ERCHINS_SOLO = [[
           * The erchins defeated, you turn to burning down the nests. It's easy work with the tools you carry.
            player:
                !left
            agent:
                !right
            agent:
                !interest
                $neutralDubious
                Wow, you're a pro, huh? You dealt with erchins before?
            player:
                !thought
                $neutralThoughtful
                Yes, actually. Well... the homophone, anyway.
                Suffice it to say the tactics are fairly similar.
            agent:
                !dubious
                $neutralDubious
                ... say what?
            player: 
                !chuckle
                Don't worry about it, my friend. Tonight, you must live like a victor.
                Gather your friends and celebrate in the way that befits your custom. 
                !exit
            * You trudge off in the direction of {handler}.
        ]],

        OPT_ASK_WORKER_TO_HELP = "Ask {agent} to help you attack the erchins",
        DIALOG_ASK_WORKER_TO_HELP = [[
            player:
                !dubious
                $miscPersuasive
                I've been sent to help, not handle. If you wish these erchins defeated, you'll join me in battle.
            agent:
                !surprised
                I'll what now?
        ]],
        OPT_ATTACK_WITH_WORKER = "Attack the nest with {agent}",
        
        DIALOG_ASK_WORKER_TO_HELP_SUCCESS = [[
            player:
                !overthere
                $miscPersuasive
                These are merely pests, my friend. No harm will come to you as long as we work together.
            agent:
                !thought
                $neutralThoughtful
                Yeah? You do seem pretty confident.
                And I guess {handler} wouldnt'a sent you if you weren't reliable.
                !agree
                $neutralDirect
                So you're on, old man. Lead the way.
            * It's been a while since you've led a charge, but you summon up the old moxy with {agent} following.
        ]],

        DIALOG_ASK_WORKER_TO_HELP_FAIL = [[
            player:
                !wince_critical
                I'll keel over from the reek alone if you're not there to help. 
            agent:
                !angry_shrug
                $angryDefault
                So what am I supposed to do, pinch your nose?
                Forget it! {handler} sent you, so do your job!
        ]],
        DIALOG_WON_BATTLE_WITH_WORKER = [[
            player:
                !left
            agent:
                !right
            agent:
                !flinch
                Suffering suckers! I'm glad I had you there to help, {player}, sir.
            * The erchins defeated, you turn to burning down the nests. It's easy work with the tools you carry.
            agent:
                !interest
                $neutralDubious
                Wow, you're a pro, huh? You dealt with erchins before?
            player:
                !thought
                $neutralThoughtful
                Yes, actually. Well... the homophone, anyway.
                Suffice it to say the tactics are fairly similar.
            agent:
                !dubious
                $neutralDubious
                ... say what?
            player: 
                !chuckle
                Don't worry about it, my friend. Tonight, you must live like a victor.
                Gather your friends and celebrate in the way that befits your custom. 
                !exit
            * You trudge off in the direction of {handler}.
        ]],
        DIALOG_WON_BATTLE_WORKER_DIED = [[
            agent:
                !exit
            * You torch the erchin nest. {agent} is dead, and the rest of the workers scattered at the first whiff of blood.
            * Nothing left but to head back to {handler}.
        ]],
        DIALOG_RALLY_FAIL = [[
            agent:
                !scared
                No. I've seen what those erchins can do. 
                $scaredStammering                
                It- it's not worth it!
                !neutral
        ]],



    }
    :SetLooping()
    :Fn(function(cxt) 

        if cxt:FirstLoop() then
            
            local worker = AgentUtil.GetFreeAgent("LABORER")
            cxt.enc:SetPrimaryCast(worker)

            --agent = TheGame:GetGameState():AddAgent( Agent("HEAVY_LABOURER" ))
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest.param.erchins =  CreateCombatParty("ERCHINS", math.max(2, cxt.quest:GetRank()), cxt.location)

        end

        cxt:Opt("OPT_ASK_ABOUT")
            :IsHubOption(true)
            :GoTo("STATE_QUESTIONS")

        cxt:Opt("OPT_ATTACK_THE_NEST_SOLO")
            :Dialog("DIALOG_ATTACK_THE_NEST_SOLO")
            :Battle{
                flags = BATTLE_FLAGS.NO_BYSTANDERS,
                enemies = cxt.quest.param.erchins,
                on_win = function(cxt) 
                    cxt.quest:Complete("go_to_the_dig_site")
                    cxt:Dialog("DIALOG_WON_AGAINST_THE_ERCHINS_SOLO")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end,
            }
            :ReceiveOpinion( "killed_a_erchin_nest_solo" )

        cxt:Opt("OPT_ASK_WORKER_TO_HELP")
            :Dialog("DIALOG_ASK_WORKER_TO_HELP")
            :Negotiation{
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_ASK_WORKER_TO_HELP_SUCCESS")
                    cxt:Opt("OPT_ATTACK_WITH_WORKER")
                        :Battle{
                            allies = {cxt:GetAgent()},
                            enemies = cxt.quest.param.erchins,
                            on_win = function(cxt) 
                                cxt.quest:Complete("go_to_the_dig_site")
                                if cxt:GetAgent():IsDead() then
                                    cxt.quest.param.worker_died = true
                                    cxt:Dialog("DIALOG_WON_BATTLE_WORKER_DIED")
                                else
                                    cxt:Dialog("DIALOG_WON_BATTLE_WITH_WORKER")
                                end

                                StateGraphUtil.AddLeaveLocation(cxt)
                            end,
                        }
                end,
                
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_ASK_WORKER_TO_HELP_FAIL")
                end,
            }


        cxt:Opt("OPT_RALLY_WORKERS_TO_CHARGE_THE_NEST")
            :PostText("TT_OPT_RALLY_WORKERS_TO_CHARGE_THE_NEST")
            :Dialog("DIALOG_RALLY_WORKERS_TO_CHARGE_THE_NEST")
            :Negotiation{
                on_start_negotiation = function(minigame)
                    for k = 1, 4 do
                        minigame.opponent_negotiator:CreateModifier( "convince_worker_to_fight_erchins", 1 )
                    end

                end,
                reason_fn = function(minigame)
                    return loc.format(cxt:GetLocString("RALLY_NEGOTIATION_REASON"), (minigame.start_params.quest.param.workers_rallied or 0) )
                end,

                on_success = function(cxt)
                    
                    local num = (cxt.quest.param.workers_rallied or 0)+1
                    local chance_of_no_casualties = num*CHANCE
                    
                    if chance_of_no_casualties < .5 then
                        cxt:Dialog("DIALOG_RALLY_SUCCESS_LOW")
                    else
                        cxt:Dialog("DIALOG_RALLY_SUCCESS")
                    end

                    if math.random() < chance_of_no_casualties then
                        cxt:Dialog("DIALOG_RALLY_NOBODY_DIED")
                    else
                        
                        cxt.quest.param.worker_died = true
                        cxt:Dialog("DIALOG_RALLY_WORKER_DIED")
                        cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("got_workers_killed"))
                    end
                    
                    cxt.quest:Complete("go_to_the_dig_site")
                    StateGraphUtil.AddLeaveLocation(cxt)

                    
                end,
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_RALLY_FAIL")
                end,
            }



    end)
     :AskAboutHub("STATE_QUESTIONS", 
        {
            "Ask about erchins",
            [[
                player:
                    !interest
                    $miscAlluring
                    You're the local, my friend. Anything you can advise me on these beasts?
                agent:
                    $neutralDirect
                    They stink.
                player:
                    $miscMischievous
                    Surprisingly I had inferred as much.
                agent:
                    !thumb
                    Face meat is their favorite. 
                player:
                    !dubious 
                    Face... meat?
                agent:
                    Yeah. So watch your face. 
                player:
                    $miscMischievous
                    I'll keep that in mind. But that's actually a tall order.
                agent:
                    !dubious
                    $neutralDubious
                    Is it?
            ]],
            "Ask about the dig",
            [[
                player:
                    How was the dig otherwise? I hope it was profitable.
                    !permit
                    Before the erchin problem, of course.
                agent:
                    !shrug
                    $neutralResigned
                    It wasn't bad. But it wasn't good, neither. 
                    {handler} can work wonders outta most junk we dig up, so who knows.
                    What looks like trash to me could be a fortune for all of us, once she gets her hands on it.
                player:
                    $neutralThoughtful
                    Interesting. And what does that cost you?
                agent: 
                    !thought
                    $neutralDubious
                    Huh? I don't follow.
                player:
                    $neutralResigned
                    Perhaps I'll ask {handler}, then. 
            ]],
        })




QDEF:AddConvo("return_for_reward", "handler")
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_WORKER_DIED = [[
            player:
                $neutralDirect
                The erchins are eliminated, as requested.
                !crossed
                $neutralResigned
                I'm afraid there were casualties, however. I can craft a full report, if you'll risk the documentation.
            agent:
                !disappoint
                $neutralResigned
                Aw, Hesh. Who was it? Who'd we lose?
            player:
                !dubious
                $neutralDubious
                I, ah, didn't catch the name, I'm afraid.
            agent:
                !angry 
                $neutralDubious
                A full report, you say. But you didn't "catch the name"?
            player:
                I merely did what was asked, {agent}. I'm sensing some of the moralizing you promised to keep to yourself.
            agent:
                ... 
                $neutralDirect
                You're right. My mistake.
                !overthere
                Let's move on then. There's still plenty of work to do. 

        ]],
        DIALOG_NOBODY_DIED = [[
            player:
                The erchins are eliminated, as requested. Furthermore it seemed no one was harmed.
                !interest
                So your operation shouldn't flicker on anyone's radar.
            agent:
                !agree 
                $happyDefault
                Just what I like to hear—clean and thorough. Excellent work, Rook.
                !thought
                $neutralDubious
                The nest worries me. Too many of those around and we might have another erchinlord on our hands.
            player:
                !dubious
                Erchinlord?
            agent:
                !placate
                Big, clotted things—trust me and hope you don't ever see one. 
                For now, just be content with a job well done, hm? 
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Fn(function() 
                    if cxt.quest.param.worker_died then
                        cxt.quest.param.poor_performance = true
                        cxt:Dialog("DIALOG_WORKER_DIED")
                    else
                        cxt:Dialog("DIALOG_NOBODY_DIED")
                    end
                end)
            :CompleteQuest()
    end)    



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                Ever heard of a wildcat dig? 
                !interest
                It's unlicensed excavation, run by off-duty workers hoping luck's on their side.
            player:
                !dubious
                $neutralDubious
                I suppose the Spark Barons don't approve.
            agent:
                !disappoint
                !neutralResigned
                Neither do the erchins, apparently.
                Some of the guys just busted open a nest—and that's sure to draw the Barons' eye.
                !point
                $neutralDirect
                Unless we want the whole bog to feel the crackdown, I need you to help the workers cover it up.
            
            *** {handler} wants you to wipe out a erchin nest to cover up an illegal wildcat dig in the bog.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            player:
                !dubious 
                $neutralDubious
                I'm sure an extermination job isn't beyond the limits of my ability. 
            agent:
                It's simple: first wipe out the erchins, and then burn the egg sacs. 
                !eureka
                $neutralDirect
                Just make sure the Barons don't find out, and we're golden.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Activate("go_to_the_dig_site")
        end)




QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.DECLINED )
    :Loc{
        DIALOG_INTRO = [[
            player:
                !placate
                $neutralDubious
                I'm afraid you have me mistaken for pest control. 
                My skills are finesse and subterfuge—something the vermin won't really appreciate. 
            agent:
                !crossed
                $angryDefault
                The job's <i>pressing</i>, you know. I thought you were multitalented.
                $neutralResigned
                But fine. I'll have to find someone less discriminating to help.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)

