
local QDEF = QuestDef.Define{
    title = "Speak Power to Power",
    qtype = QTYPE.CONTRACT,
    faction = "RISE",
    rank = {1, 3},
    reward_size = QREWARDSIZE.SMALLER,
    desc = [[{target} has been mistreating {target.hisher} workers. {giver} wants you to scare a little empathy into {target.himher}.]],
    icon = engine.asset.Texture("icons/quests/rise_intimidate_a_foreman.tex"),
    focus = QUEST_FOCUS.BALANCED,
    avoids = 
    {
        {"giver", "target"},
    },

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("target") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,

}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddCast{
    cast_id = "target",
    condition = function(agent, quest) 
        return agent:GetContentID() == "FOREMAN"
    end,
}
:AddCastFallback
{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "FOREMAN" ) )
    end,
}

:AddObjective{
    id = "confront_target",
    title = [[Confront {target}]],
    desc = "Intimidate {target}, or beat {target.himher} up.",
    mark = { "target" },
    combat_targets = {"target"},
    negotiation_targets = {"target"},
    
    on_activate = function(quest)
        quest:GetCastMember("target"):MoveToLimbo()
    end,
    
    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[Report back to {giver}.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}


QDEF:AddConvo("confront_target", "target")
    :Loc{
        OPT_CONFRONT = "Confront {agent} about their treatment of the workers.",
        DIALOG_CONFRONT = [[
            player:
                {giver} says you've been pushing your workers too hard.
                I'm here to ask you to consider maybe lightening your touch a bit.
            agent:
                Is that what {giver.heshe} says? 
                You tell {giver.himher} that {giver.heshe}'s too soft.
                You can't let them walk all over you, or you'll never make quota!
        ]],
        OPT_BEAT_UP_TARGET = "Beat up {agent}",
        DIALOG_BEAT_UP_TARGET = [[
            player:
                !fight
                I asked nice, and you didn't listen. Now I'm going to break your bones.
        ]],
        DIALOG_FOUGHT_ALIVE = [[
            agent:
                !injured
                Stop! I'll ease up on 'em.
            player:
                See, it's easy. You play nice, and I'll play nice.
        ]],
        DIALOG_FOUGHT_DEAD = [[
            * {agent} is dead. {giver} didn't want you to take things this far.
        ]],

        OPT_THREATEN = "Threaten {agent}",
        DIALOG_THREATEN = [[
            player:
                Let me rephrase. I'm here to demand that you stop being a little tyrant, or I'll hurt you.
                How does that sound?
        ]],
        DIALOG_THREATEN_SUCCESS = [[
            agent:
                !scared
                {giver}'s not with the Rise, is she?
                I'll back off. The job's not worth my life.
            player:
                !point
                I'm glad we could come to this agreement.
        ]],
        DIALOG_THREATEN_FAILURE = [[
            agent:
                !angry
                I'm not afraid of {giver}, and I'm certainly not afraid of <i>you</>.
        ]],
        DIALOG_BACK = [[
            player:
                I'll be back in a second. Don't go anywhere.
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_CONFRONT")
            :LoopingFn(function(cxt) 
                    if cxt:FirstLoop() then
                        cxt:Dialog("DIALOG_CONFRONT")
                        if not cxt.enc.scratch.confront_enemies then
                            cxt.enc.scratch.confront_enemies = {cxt:GetAgent()}
                            local num_drones = math.floor( (1 + cxt.quest:GetRank())/2 )
                            
                            if not cxt.quest.param.drones then
                                cxt.quest.param.drones = {}
                                for k = 1, num_drones do
                                    local drone = Agent("COMBAT_DRONE")
                                    table.insert(cxt.quest.param.drones, drone)
                                end
                            end

                            for k, v in ipairs( cxt.quest.param.drones ) do
                                table.insert(cxt.enc.scratch.confront_enemies, v)
                            end

                        end
                    end
                    
                    cxt:Opt("OPT_THREATEN")
                        :Dialog("DIALOG_THREATEN")
                        :Negotiation{
                            flags = NEGOTIATION_FLAGS.INTIMIDATION,
                            
                            on_success = function(cxt) 
                                cxt.quest.param.did_threaten = true
                                cxt:Dialog("DIALOG_THREATEN_SUCCESS")
                                cxt.quest:Complete("confront_target")
                                StateGraphUtil.AddEndOption(cxt)
                            end,
                            on_fail = function(cxt) 
                                cxt:Dialog("DIALOG_THREATEN_FAILURE")
                            end,
                        }
                    cxt:Opt("OPT_BEAT_UP_TARGET")
                        :Dialog("DIALOG_BEAT_UP_TARGET")
                        :Battle{
                            enemies = cxt.enc.scratch.confront_enemies,
                            on_win = function(cxt) 
                                cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_FOUGHT_DEAD" or "DIALOG_FOUGHT_ALIVE")
                                cxt.quest:Complete("confront_target")
                                StateGraphUtil.AddEndOption(cxt)
                            end,
                        }
                    
                    StateGraphUtil.AddBackButton(cxt)
                        :Dialog("DIALOG_BACK")
            end)

    end)



QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_KILLED = [[
            player:
                I have good news and bad news.
            agent:
                ...
            player:
                The good news is that {target} isn't going to be a problem any more.
                The bad news is that {target.heshe} also isn't going to be breathing much any more.
            agent:
                I told you be <i>persuasive</>, not <i>lethal</>!
                If the Barons trace this back to me we're in for a world of hurt.
                Don't mess that up again!
        ]],
        DIALOG_THREATENED = [[
            player:
                I convinced {target} to back off. 
            agent:
                Great! What did you have to do?
            player:
                We just talked. I can be very persuasive!
            agent:
                I'm sure. 
        ]],
        DIALOG_BEAT = [[
            player:
                {target}'s going to be limping for a week, but {target.hisher} workers should catch a break.
            agent:
                !happy
                Good job. We have to show these yote-strokers we mean business.
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Fn(function() 
                    if cxt.quest:GetCastMember("target"):IsDead() then
                        cxt.quest.param.poor_performance = true
                        cxt:Dialog("DIALOG_KILLED")
                    elseif cxt.quest.param.did_threaten then
                        cxt:Dialog("DIALOG_THREATENED")
                    else
                        cxt:Dialog("DIALOG_BEAT")
                    end

                end)
            :CompleteQuest()
    end)    



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                {target} is a foreman here in the bog. One of the worst kind.
                {target.HeShe} works {target.hisher} shifts too hard. Accidents happen, and people get hurt.
                I need you to go teach {target.himher} a thing or two about the importance of workplace safety.
            
            *** {giver} wants you to intimidate {target} so that {target.heshe} will stop pushing {target.hisher} workers too hard.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                Great. Go find {target.himher} and give {target.himher} a good scare. Use force if you need to, but don't kill anybody. We don't want to provoke the Barons!
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Activate("confront_target")
        end)




QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.DECLINED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                You've never worked under a petty tyrant before? If you had, you've have a little more empathy for the poor folks under {target}.
                You do you, though. 
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)

