local fun = require "util/fun"

local QDEF = QuestDef.Define{
    title = "Mole Fixer",
    qtype = QTYPE.CONTRACT,
    faction = "RISE",
    tags = {"late_game"},
    rank = {1, 5},
    desc = [[{mole} is a mole working for the Spark Barons and feeding critical information to the Rise. {giver} relies on that information to direct operations, but lately {mole} has stopped talking. Talk to {mole} and figure out how to turn the tap back on.
    ]],
    icon = engine.asset.Texture("icons/quests/rise_mole_fixer.tex"),
    focus = QUEST_FOCUS.NEGOTIATION,

    avoids = {
        {"snitch", "mole"},
        {"giver", "mole"},
        {"giver", "snitch"},
    },
    on_init = function(quest)
    end,

    on_destroy = function(quest)
        if quest:GetCastMember("snitch") and quest:GetCastMember("snitch"):IsInPlayerParty() then
            quest:GetCastMember("snitch"):Dismiss()
        end
    end,

    on_start = function(quest)
        
        if quest.param.set_snitch_relationship == "FRIEND" then
            quest:GetCastMember("snitch").social_connections:SetRelationship(RELATIONSHIP.LIKED, TheGame:GetGameState():GetPlayerAgent())
        elseif quest.param.set_snitch_relationship == "ENEMY" then
            quest:GetCastMember("snitch").social_connections:SetRelationship(RELATIONSHIP.DISLIKED, TheGame:GetGameState():GetPlayerAgent())
        elseif quest.param.set_snitch_relationship == "NEUTRAL" then
            quest:GetCastMember("snitch").social_connections:SetRelationship(RELATIONSHIP.NEUTRAL, TheGame:GetGameState():GetPlayerAgent())
        end
        quest:Activate("contact_mole")
    end,

}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

--new person
:AddCast{
    cast_id = "mole",
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "SPARK_BARON_TASKMASTER" ) )
    end,
}

:AddLocationCast{
    cast_id = "rec_center",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_REC_CENTER"))
    end,
}

:AddLocationDefs{
    SECLUDED_SPOT = {
        name = "Secluded Bog",
        plax = "EXT_Bog_Outcrop_01",
        map_tags = {"bog"},
    },
}

:AddLocationCast{
    cast_id = "secluded_location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("SECLUDED_SPOT"))
    end,
}

--try to find someone we know
:TryAddKnownCast("snitch", "SPARK_BARON_TASKMASTER")

:AddObjective{
    id = "contact_mole",
    title = [[Go ask the mole what's up]],
    mark = {"mole"},

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("mole"), location = quest:GetCastMember("mole"):GetHomeLocation(), role = CHARACTER_ROLES.PROPRIETOR} )
    end,

    on_complete = function(quest)
        quest:Activate("confront_snitch")
    end,
}

:AddObjective{
    id = "confront_snitch",
    title = [[Confront {snitch}]],
    mark = {"snitch"},

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("snitch"), location = quest:GetCastMember("rec_center"), role = CHARACTER_ROLES.VISITOR} )
    end,
}


:AddObjective{
    id = "betray_mole",
    title = [[Kill {mole}]],
    desc = "You've agreed to kill {mole} so that {snitch} can take their place on the Rise's payroll.",
    mark = {"mole"},

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("mole"), location = quest:GetCastMember("mole"):GetHomeLocation(), role = CHARACTER_ROLES.PROPRIETOR} )
    end,

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("mole") then
            fighter:AddCondition("WANTED_DEAD")
        end
    end,

    on_complete = function(quest)
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "kill_snitch",
    title = [[Kill {snitch}]],
    desc = "Kill {snitch}",
    mark = {"secluded_location"},

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("snitch") then
            fighter:AddCondition("WANTED_DEAD")
        end
    end,

    on_complete = function(quest)
        quest:Activate("return_for_reward")
        --quest:Activate("talk_to_snitch")
    end,
}

:AddObjective{
    id = "convince_mole_snitch_dead",
    title = "Convince {mole} that {snitch} is dead",
    desc = [[You let {snitch} skip town. Convince {mole} that {snitch.heshe} is dead.]],
    mark = { "mole" },
}


:AddObjective{
    id = "return_for_reward",
    title = "Report back to {giver}",
    desc = [[Tell {giver} how you dealt with the snitch.]],
    mark = { "giver" },
    terminal = true,
}


:AddOpinionEvents{
    tried_to_extort =  
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Demanded an up-front payment",
    },
    let_them_disappear =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Let them disappear to escape from an enemy",
    },    
    opinion_did_not_kill_snitch = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Let their enemy live",
    }
}


----------QUEST INTRODUCTION

QDEF:AddIntro(
    --attract spiel
    [[
        agent:
            You aren't our only secret weapon, you know.
        player:
            I'm not? This is staggering.
        agent:
            Yeah, well, sometimes you gotta ferret out the secret from more than one direction. 
            We have a mole in the Barons, and {mole.heshe} hasn't been responding to our messages lately. 
            Go figure out what's wrong.
    ]],
    
    --on accept
    [[
        agent:
            I'll show you where the mole lives. I suspect they're being watched, but you know the Barons and can get closer than I can.
            Be careful, all right? I don't want them jeopardized anymore than they already are.
        player:
            Perish the thought. Or perish the Baron who gets in my way. 
            Either will do handily. 

    ]])



------ TALK TO THE MOLE
-- The mole thinks they have been discovered, and needs you to help flush out the person who's suspicious
--  You can shake 'em down for some cash if you want to take the rep hit.
--  They send you to the rec center.
QDEF:AddConvo("contact_mole")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("mole"):GetLocation() end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    H-hey! This ain't public property!
                    I dunno who you are but you better get the heck outta here before I bust in your face.
                player:
                    I'm with {giver}. {giver.HeShe} wants to know why you haven't reported in lately.
                agent:  
                    {giver}? Don't know {giver.himher}. And if I did, I wouldn't admit to it.
                    Sounds like the kind of thing that could blow one's cover.
                    If I had a cover to blow, that is.
                player:
                    I see. I suppose there are many threats that could blow one's cover on this side of the Bog.
                agent:
                    Oh, definitely. Like {snitch} for instance. 
                    {snitch.HeShe}'s a real snitch. And once {snitch.heshe} gets {snitch.hisher} paws on a juicy secret... 
                    Well, they could be a real threat to any operation. If you catch my meaning.
                player:
                    I believe I do. But you won't catch me admitting it.
                agent:
                    !thought
                    {snitch} spends most of {snitch.hisher} time in the Rec Center. If you could lure {snitch.himher} out of there, and into the open, we could, uh...
                    Neutralize the problem. 
                player: 
                    I do enjoy making things... neutral.
            ]],
            
            OPT_ASK_ABOUT_SNITCH = "Ask about {snitch}",
            DIALOG_ASK_ABOUT_SNITCH = [[
                player:
                    Tell me about this {snitch}. 
                agent:
                    {snitch} would sell {snitch.hisher} own mother for a rusty shill. But she'd do the same.
                    If greed has a gene, it's in {snitch}'s bloodline. 
            ]],
            OPT_ASK_ABOUT_MOLE = "Ask why {mole} is a mole",
            DIALOG_ASK_ABOUT_MOLE = [[
                player:
                    It seems a dangerous time to serve two masters, wouldn't you say?
                agent:
                    I would. Good thing I don't know anybody who does.
                player:
                    Yes. Neither would I. 
                agent:
                    Well. Good talk.
            ]],

            OPT_DEMAND_EXTRA_PAYMENT = "Demand payment for your services",
            
            DIALOG_DEMAND_EXTRA_PAYMENT = [[
                player:
                    I'll help you out, but I'm not doing it for free. 
                    Give me some cash up front, or maybe the Barons are going to hear from me.
            ]],
            
            DIALOG_DEMAND_EXTRA_PAYMENT_SUCCESS = [[
                agent:
                    You've got leverage. I respect that. I don't like it, but I respect it.
                    !give
                    Here, take this.
            ]],
            
            DIALOG_DEMAND_EXTRA_PAYMENT_FAILURE = [[
                agent:
                    You're already getting paid by {giver}. Don't try to shake me down, grifter.
            ]],

            OPT_AGREE_TO_TALK_TO_SNITCH = "Agree to deal with {snitch}",
            DIALOG_AGREE_TO_TALK_TO_SNITCH = [[
                player:
                    Well, certainly. I've already come all this way to talk to you--
                agent:
                    Except we didn't talk.
                    In fact, I don't even know what you look like.
                player:
                    Ah--I've momentarily gone blind. Of course, at my age, it's to be expected.
                agent:
                    I'm sorry, did someone say something?
                    I can't hear a damn thing.
                    ...
                    The door's over that way. Best to see yourself out.
                
                agent:
                    {got_extra_payment_from_mole? 
                        You had better do it. I paid you good money.
                    }
                    {not got_extra_payment_from_mole?
                        {disliked?
                            You'd better do it, or {giver} is going to hear about it.
                        }
                        {not disliked?
                            Thank you! I've been so scared.
                        }
                    }
                    Remember, don't let anyone else hear you talking about me to {snitch} on Baron property. Get them out of the rec center before you make your move!
            ]]
        }
        :Fn(function(cxt) 
            cxt:TalkTo(cxt.quest:GetCastMember("mole"))
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 
                cxt:Question("OPT_ASK_ABOUT_SNITCH", "DIALOG_ASK_ABOUT_SNITCH")
                cxt:Question("OPT_ASK_ABOUT_MOLE", "DIALOG_ASK_ABOUT_MOLE")

                cxt:Opt("OPT_DEMAND_EXTRA_PAYMENT")
                    :Dialog("DIALOG_DEMAND_EXTRA_PAYMENT")
                    :ReceiveOpinion("tried_to_extort")
                    :Negotiation{flags = NEGOTIATION_FLAGS.HOSTILE}
                        :OnSuccess()
                            :Dialog("DIALOG_DEMAND_EXTRA_PAYMENT_SUCCESS")
                            :ReceiveMoney(cxt.quest:GetRank()*100)
                            :Fn(function() 
                                cxt.quest.param.got_extra_payment_from_mole = true
                            end)

                        :OnFailure()
                            :Dialog("DIALOG_DEMAND_EXTRA_PAYMENT_FAILURE")

                cxt:Opt("OPT_AGREE_TO_TALK_TO_SNITCH")
                    :Dialog("DIALOG_AGREE_TO_TALK_TO_SNITCH")
                    :CompleteQuest("contact_mole")
                    :Travel()
            end)

        end)




------ TALK TO THE SNITCH
-- You have to convince them to come with.
-- OR if you are friends, you can switch sides
-- OR you can just fight, although you'll have to take on security as well.

QDEF:AddConvo("confront_snitch")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("rec_center") end)
        :Loc{
            DIALOG_INTRO = [[
                * You enter the Rec Center. It smells of Baron sweat.
                guard:
                    !right
                    THIS AREA IS FOR SPARK BARONS ONLY.
                    PROVIDE IDENTIFICATION OR LEAVE.
                player:
                    !left
                    I'm working for Fellemo. You can check for my contract on file.
                guard:
                    ...
                    PROCEED.
                    !exit
                * The guard ambles off. After some searching, you find {snitch}, working on {snitch.hisher} reps.
                snitch:
                    !right
                {liked?
                    agent:
                        Hey {player}, how's it going?
                }
                {disliked?
                    agent:
                        What do YOU want?
                }
                {not disliked and not liked?
                    agent:
                        Can I help you?
                }
            ]],

            OPT_PRESENT_OFFER_FROM_MOLE = "Trick {agent} into following you",
            DIALOG_PRESENT_OFFER_FROM_MOLE = [[
                player:
                    {mole} sent me. {mole.HeShe}'s ready to discuss the terms of your, ah, silence.
                agent: 
                    !cagey
                    Shh! Don't let anyone hear you!
                player:
                    !dubious
                    As I was about to say, we should go somewhere private to discuss this further.

            ]],
            
            DIALOG_PRESENT_OFFER_FROM_MOLE_SUCCESS = [[
                player:
                    I do admire the way you outwitted poor {mole}.
                    Truly a masterful act. Perhaps when this is done we could discuss a mutually beneficial partnership?
                agent:
                    Oh yeah...?
                    Heh, yeah! I bet you could learn a thing or two from me. 
                    Sure. Lead the way, grifter. We can discuss details as we walk.
            ]],
            DIALOG_PRESENT_OFFER_FROM_MOLE_FAILURE = [[
                player:
                    It's a lovely day for a stroll, isn't it? 
                    I hear an isolated place in the bog is particularly lovely this time of year.
                agent:
                    Sure, if you like combusting damselfly larva. 
                    Tell {mole} {mole.heshe} to <i>send</i> the money... or else.
            ]],
            
            OPT_ATTACK_SNITCH = "Attack {agent}",
            DIALOG_ATTACK_SNITCH = [[
                player:
                    I find the simplest solution is always the most direct.
                    !fight
            ]],
            DIALOG_WON_FIGHT = [[
                {dead?
                    * {agent} is dead. That was certainly one way to solve {mole}'s problem, but you best disappear before security catches on to what you've done.
                }

                {not dead?
                    agent:
                        !injured
                        Alright, alright! Stop!
                        You draw too much attention and folks will wanna know exactly how I caught your ire.
                        That'd be bad for both of us. 
                        So... fine, you win. Tell {mole} I'll back off for now.
                    player:
                        I suppose that will do. Your corpse <i>would</i> be rather inconvenient.                
                }
            ]],

            OPT_WARN_ABOUT_MOLE = "Warn {agent} about {mole}'s plan",
            DIALOG_WARN_ABOUT_MOLE = [[
                player:
                    You must know that {mole} is a dangerous person to cross.
                    After all, {mole.heshe} loses everything if you talk.
                agent:
                    Uh, yeah. That's why I'm blackmailing {mole.himher}.
                player: 
                    Yes, and I'm saying that {mole} asked me to kill you. 
                agent:
                    {tried_to_trick?
                        Uh, yeah. That's why I didn't go off to your little Bog picnic slash ambush.                   
                    }
                    {not tried_to_trick?
                        Oh! 
                        O-oh.
                        Does that mean you don't actually want to work together?
                    }
                player:
                    You'll notice I am at least warning you.
                agent: 
                    Okay, well. I tell you what.
                    I'll work for the Rise directly... but you gotta get {mole} out of the way. Permanently. 
            ]],

            OPT_ACCEPT_DEAL = "Accept the deal",
            DIALOG_ACCEPT_DEAL = [[
                player:
                    Seems agreeable to me. I'm sure one informant is as good as another.
                agent:
                    Yeah!
            ]],

            OPT_REJECT_DEAL = "Reject the deal",
            DIALOG_REJECT_DEAL = [[
                player:
                    I'm afraid that's not an ideal solution.
                    The only way out of this for you is to simply sate your greed and leave {mole} alone.
                agent:
                    Yeah... I hear what you're saying...
                    Yeah, I get it.
                    But yeah... no. 
                    !fight
                    
            ]]
        }
        :Fn(function(cxt) 
            cxt.location:ClearAgents() --kinda dirty, but whatever.
            
            local guard = cxt.quest:CreateSkinnedAgent( "SPARK_BARON_AUTOMECH" )            
            guard:MoveToLocation(cxt.location)
            cxt:ReassignCastMember("guard", guard)
            cxt:TalkTo(cxt.quest:GetCastMember("snitch"))
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 

                cxt:Opt("OPT_PRESENT_OFFER_FROM_MOLE")
                    :Dialog("DIALOG_PRESENT_OFFER_FROM_MOLE")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_PRESENT_OFFER_FROM_MOLE_SUCCESS")
                            :CompleteQuest("confront_snitch")
                            :ActivateQuest("kill_snitch")
                            :RecruitMember(PARTY_MEMBER_TYPE.ESCORT)
                            :Travel()

                        :OnFailure()
                            :Dialog("DIALOG_PRESENT_OFFER_FROM_MOLE_FAILURE")
                            :Fn(function() cxt.enc.scratch.tried_to_trick = false end)

                cxt:Opt("OPT_ATTACK_SNITCH")
                    :Dialog("DIALOG_ATTACK_SNITCH")
                    :Battle{ no_oppo_limit = true}
                        :OnWin()
                            :Fn(function() 
                                cxt.quest.param.fought_in_rec_center = true 
                                cxt.quest.param.killed_snitch = cxt:GetAgent():IsDead()
                            end)
                            :Dialog("DIALOG_WON_FIGHT")
                            :CompleteQuest("confront_snitch")
                            :ActivateQuest("return_for_reward")
                            :Travel()
                
                if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL then
                    cxt:Opt("OPT_WARN_ABOUT_MOLE")
                        :Dialog("DIALOG_WARN_ABOUT_MOLE")
                            :Fn(function() 
                                
                                cxt:Opt("OPT_ACCEPT_DEAL")
                                    :Dialog("DIALOG_ACCEPT_DEAL")
                                    :Fn(function() cxt.quest:Cancel("confront_snitch") end)
                                    :ActivateQuest("betray_mole")
                                    :Travel()

                                cxt:Opt("OPT_REJECT_DEAL")
                                    :Battle{ no_oppo_limit = true}
                                        :OnWin()
                                            :Fn(function() 
                                                cxt.quest.param.fought_in_rec_center = true 
                                                cxt.quest.param.killed_snitch = cxt:GetAgent():IsDead()
                                            end)
                                            :Dialog("DIALOG_WON_FIGHT")
                                            :CompleteQuest("confront_snitch")
                                            :ActivateQuest("return_for_reward")
                                            :Travel()

                            end)
                end

            end)

        end)



------ KILL THE SNITCH
-- Fight the snitch OR take their offer to run away (you will have to sell it to mole)

QDEF:AddConvo("kill_snitch")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("secluded_location") end)
        :Loc{
            DIALOG_INTRO = [[
                * You come to a nicely secluded spot.
                player:
                    !left
                agent:
                    !right
                    So where's {mole}?
                player:
                    ...
                agent:
                    You, uh, gonna say something?
                player:
                    ...
                agent:
                    This is creepin' me out. 
                    You're not--you're not gonna kill me, are you?
                player:
                    Well...
                agent:
                    Okay, hold that thought!
                    Because I just figured, hey, you know what? I should take a vacation.
                    I can just disappear, and {mole} won't know the difference.
                    Not if you cover for me. How's that sound?
            ]],

            OPT_ASK_FOR_MONEY = "Demand payment to spare {agent}",
            DIALOG_ASK_FOR_MONEY = [[
                player:
                    What's your life worth? What can you give me?
                agent:
                    If I had money, I wouldn't be blackmailing {mole}.
                    Maybe you could just spare me out of the kindness of your heart?
            ]],


            OPT_TAKE_OFFER_TO_DISAPPEAR = "Let {agent} go",
            DIALOG_TAKE_OFFER_TO_DISAPPEAR = [[
                player:
                    Fine. Get out of here. And don't let anyone ever see you again.
                agent:
                    Thank you!
                    !exit
                * You start formulating a plan to convince {mole} that you actually did your job.                    
            ]],

            OPT_ATTACK = "Attack {agent}",
            DIALOG_ATTACK = [[
                player:
                    You are a terrible negotiator.
                    !fight
                    I hope you can appreciate that before you die. 
            ]],
            DIALOG_ON_WIN_FIGHT = [[
                agent:
                    !exit
                * You hide evidence of your fight, and skulk off.

            ]],

        }
        :Fn(function(cxt) 
            cxt:TalkTo(cxt.quest:GetCastMember("snitch"))
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                cxt:Question("OPT_ASK_FOR_MONEY", "DIALOG_ASK_FOR_MONEY")
                
                cxt:Opt("OPT_TAKE_OFFER_TO_DISAPPEAR")
                    :Dialog("DIALOG_TAKE_OFFER_TO_DISAPPEAR")
                    :ReceiveOpinion("let_them_disappear")
                    :Fn(function() 
                        cxt:GetAgent():Dismiss()
                        cxt:GetAgent():Retire()
                        cxt.quest:Cancel("kill_snitch")
                        cxt.quest:Activate("convince_mole_snitch_dead")
                    end)
                    :Travel()


                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{
                        flags = BATTLE_FLAGS.NO_SURRENDER | BATTLE_FLAGS.ISOLATED,
                        }
                        :OnWin()
                            :Dialog("DIALOG_ON_WIN_FIGHT")
                            :CompleteQuest("kill_snitch")
                            :ActivateQuest("return_for_reward")
                            :Travel()
            end)
        end)



------ BETRAY MOLE
-- You have agreed to kill the mole. They are in no mood to negotiate, so it's a simple fight


QDEF:AddAttract("betray_mole", "mole",
    [[
        agent:
            Hi, stranger. I've never met you before and this is our first conversation.
            But apropos of nothing, have you taken care of our little "problem"?
    ]])


QDEF:AddConvo("betray_mole", "mole")
    :Loc{
        OPT_ATTACK = "Attack {mole}",
        DIALOG_ATTACK = [[
            player:
                !fight
                I'm quite about to, actually.
        ]],
        DIALOG_ATTACK_2 = [[
            agent:
                !fight
                Hah! You're a fool if you don't think I had a plan for this from the start.
        ]],

        DIALOG_WON_FIGHT = [[
            * {agent} falls to the ground with a thud heavy enough to kick up dust. But, like {agent.heshe} said, you were never here.
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_ATTACK")
            :Dialog("DIALOG_ATTACK")
            :ReceiveOpinion(OPINION.BETRAYED)            
            :Dialog("DIALOG_ATTACK_2")
            :Battle{
                flags = BATTLE_FLAGS.NO_SURRENDER
                }
                :OnWin()
                    :Dialog("DIALOG_WON_FIGHT")
                    :CompleteQuest("betray_mole")
                    :Fn(function() cxt.quest.param.betrayed_mole = true end)
                    :ActivateQuest("return_for_reward")
                    :Travel()
    end)



------ CONVICE MOLE THAT SNITCH IS DEAD
-- Negotiation with snitch. If you fail, 

QDEF:AddAttract("convince_mole_snitch_dead", "mole",
    [[
        agent:
            Hi, stranger. I've never met you before and this is our first conversation.
            But apropos of nothing, have you taken care of our little "problem"?
    ]])



QDEF:AddConvo("convince_mole_snitch_dead", "mole")
    :Loc{
        OPT_CONVINCE = "Convince {mole} that {snitch} is dead",
        DIALOG_CONVINCE = [[
            player:
                What problem? 
                It seems to me we never had one, yes?
        ]],

        DIALOG_CONVINCE_SUCCESS = [[
            player:
                !throatcut
                And if we did have one... well, it's certainly long gone by now.
            agent:
                !wink
                I get your meaning loud and clear, {player}.
                Appreciate all you've done.
                Even if you officially didn't do anything at all.
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            player:
                In fact, if I had to say so, I might--
            agent:
                This is getting us nowhere. Speak plainly for a second.
                Is {snitch} dead or what?
            player:
                Ah, well--
            agent:
                !angry
                I see. Well, how about this: if {giver} wants me to keep up the, ah, good work...
                I better see double pay next time.
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE")
            :SetQuestMark()
            :Dialog("DIALOG_CONVINCE")
            :Negotiation{}
                :OnSuccess()
                    :Dialog("DIALOG_CONVINCE_SUCCESS")
                    :CompleteQuest("convince_mole_snitch_dead")
                    :ActivateQuest("return_for_reward")
                    :End()
                
                :OnFailure()
                    :Dialog("DIALOG_CONVINCE_FAILURE")
                    :ReceiveOpinion("opinion_did_not_kill_snitch")
                    :CompleteQuest("convince_mole_snitch_dead")
                    :Fn(function() cxt.quest.param.mole_wants_raise = true end)
                    :ActivateQuest("return_for_reward")
                    :End()
    end)


------ REUTURN FOR REWARD
-- You get reduced rewards if mole wants a raise. You have to negotiate for rewards at all if you replaced mole with snitch 

QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_CONVINCE = "Convince {giver} to accept {snitch} as the new mole",
        DIALOG_CONVINCE = [[
            player:
                The good news is that you still have a mole on that side of Barons.
                The bad news is that it's no longer {mole}.
            agent:
                Uh, what? Who is this <i>new</i> mole?
            player:
                {snitch}. Heard of {snitch.himher}?
            agent:
                !palm 
                I have. You sure about this, {player}? 
                It was risky enough before, but now...
        ]],

        DIALOG_CONVINCE_SUCCESS = [[
            player:
                {mole} made mistakes and had {mole.hisher} duplicity discovered.
                Meanwhile, {snitch} was shrewd enough to catch it.
            agent:
                I guess one Baron is as good as another. I'm just glad this is resolved.
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            player:
                Surely one Baron is as good as another, no?
            agent:
                !angry
                I trusted {mole}. I don't know {snitch}. 
                And now, after this operation, I'm not sure I know you either. 
        ]],

        OPT_TELL_ABOUT_RAISE = "Tell {giver} that {mole} is demanding a raise",
        DIALOG_TELL_ABOUT_RAISE = [[
            player:
                I found {mole}. {mole.HisHer} position was compromised, though I was able to resolve it.
                Unfortunately, my solution wasn't quite to {mole.hisher} liking.
            agent:
                What does that mean? Is {mole} quitting?
            player:
                On the contrary! {mole.HeShe}'s as committed to the cause as ever.
                Provided the financial compensation is adequately inflated.
            agent:
                !palm 
                You say that like it's good news, but my pockets know otherwise.
                Fine. I guess it's better than no informant at all.
        ]],

        OPT_REPORT_SUCCESS = "Report success",
        DIALOG_REPORT_SUCCESS = [[
            player:
                Mission accomplished. You should get your next report in a timely fashion.
            agent:
                That's better news than I've had all day. And thank the tides for that.
                !give
                Good work, {player}. But you should clear out before anyone sees us talking.
            player:
                !wink
                I was never even here.
        ]],

        OPT_TELL_ABOUT_FIGHT = "Tell {giver} about the brawl in the Rec Center",
        DIALOG_TELL_ABOUT_FIGHT = [[
            player:
                Hello, {giver}. I was able to--
            agent:
                !angry
                I just heard that there was a huge disturbance in the Baron Rec Center, and some Baron got {killed_snitch?killed|hurt}.
            player:
                Ah, yes. I'm sure that's--
            agent:
                You've screwed us over. A {killed_snitch?dead|hurt} Baron means increased security, increased scrutiny.
                !angry_point
                Now get out of here before someone traces it back to you, <i>and</i> me. 
        ]]

    }
    :Hub(function(cxt) 

        if cxt.quest.param.betrayed_mole then 
            cxt:Opt("OPT_CONVINCE")
                :SetQuestMark()
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_CONVINCE_SUCCESS")
                        :CompleteQuest()
                        :Fn(function() ConvoUtil.GiveQuestRewards(cxt) end)
                        :End()
                    
                    :OnFailure()
                        :Dialog("DIALOG_CONVINCE_FAILURE")
                        :FailQuest()
                        :End()
        elseif cxt.quest.param.fought_in_rec_center then
            cxt:Opt("OPT_TELL_ABOUT_FIGHT")
                :SetQuestMark()
                :Dialog("DIALOG_TELL_ABOUT_FIGHT")
                :Fn(function() 
                    cxt.quest.param.poor_performance = true
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt) end)
                :End()

        elseif cxt.quest.param.mole_wants_raise then
            cxt:Opt("OPT_TELL_ABOUT_RAISE")
                :SetQuestMark()
                :Dialog("DIALOG_TELL_ABOUT_RAISE")
                :Fn(function() 
                    cxt.quest.param.poor_performance = true
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt) end)
                :End()
        else
            cxt:Opt("OPT_REPORT_SUCCESS")
                :SetQuestMark()
                :Dialog("DIALOG_REPORT_SUCCESS")
                :CompleteQuest()
                :Fn(function() 
                    ConvoUtil.GiveQuestRewards(cxt) end)
                :End()
        end


    end)



QDEF:AddDebugOption("set_snitch_relationship", {"FRIEND", "NEUTRAL", "ENEMY"})
