local fun = require "util/fun"

local QDEF = QuestDef.Define{
    title = "Trade Secrets",
    qtype = QTYPE.CONTRACT,
    faction = "RISE",
    tags = {"late_game"},
    rank = {1, 5},
    desc = [[Steal secret research data from the Barons and sell it to the highest bidder to raise funds for the Rise.]],
    icon = engine.asset.Texture("icons/quests/rise_trade_secrets.tex"),
    focus = QUEST_FOCUS.NEGOTIATION,
    on_init = function(quest)
        quest.param.researcher_motivation = quest.param.researcher_motivation or table.arraypick{"MORAL", "GREED", "BOG_MADNESS"}
        --for dynamic dialog
        quest.param["researcher_motivation_" .. loc.tolower(quest.param.researcher_motivation)] = true
    end,

    on_start = function(quest)
        quest:Activate("contact_researcher")
    end,
    avoids = 
    {
        {"hesh_buyer", "admiralty_buyer"},
        {"hesh_buyer", "jakes_buyer"},
        {"jakes_buyer", "admiralty_buyer"},
    },

}


:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddLocationDefs{
    RESEARCH_CAMP = {
        name = "Research camp",
        desc = "A makeshift research camp.",
        plax = "EXT_SB_ResearchTent",
        map_tags = {"bog"},
        indoors = false,
    },
}

:AddCast{
    cast_id = "researcher",
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "CHEMIST" ) )
    end,
}

:AddLocationCast{
    cast_id = "research_camp",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("RESEARCH_CAMP"))
    end,
}


:AddCast{
    cast_id = "hesh_buyer",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "PRIEST" ) )
    end,
}

:AddCast{
    cast_id = "jakes_buyer",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "JAKES_SMUGGLER" ) )
    end,
}

:AddCast{
    cast_id = "admiralty_buyer",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "ADMIRALTY_CLERK" ) )
    end,
}



:AddObjective{
    id = "contact_researcher",
    title = [[Contact {researcher}]],
    desc = "{researcher} has access to the Barons' latest research. Convince them to give it to you.",
    mark = {"researcher"},
    on_complete = function(quest) 
        quest:Activate("steal_the_research")
    end
}

:AddObjective{
    id = "steal_the_research",
    title = "Steal the Research",
    desc = "Gain access to the research station, and steal the information.",
    mark = {"research_camp"},
    on_complete = function(quest) 
        quest:Activate("sell_to_admiralty")
        quest:Activate("sell_to_jakes")
        quest:Activate("sell_to_hesh")

        local locations = {
            TheGame:GetGameState():GetLocation("GB_COMPANY_STORE"),
            TheGame:GetGameState():GetLocation("GB_BLACK_MARKET"),
            TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"),
        }
        table.shuffle(locations)

        quest:GetCastMember("admiralty_buyer"):GetBrain():SendToPatronize(locations[1])
        quest:GetCastMember("hesh_buyer"):GetBrain():SendToPatronize(locations[2])
        quest:GetCastMember("jakes_buyer"):GetBrain():SendToPatronize(locations[3])

    end
}

:AddObjective{
    id = "sell_to_admiralty",
    title = "Sell the Research to the Admiralty",
    desc = [[{admiralty_buyer} is in the region looking for technology to help the Admiralty fight the Spree in Murder Bay.]],
    mark = { "admiralty_buyer" },
    icon = engine.asset.Texture( "UI/faction_newdeltree.tex" ),

    on_activate = function(quest)
        quest:AssignCastMember("admiralty_buyer")
    end,

    on_complete = function(quest)
        quest:Cancel("sell_to_hesh")
        quest:Cancel("sell_to_jakes")
        quest:Activate("return_for_reward")
    end,
}
:AddObjective{
    id = "sell_to_hesh",
    title = "Sell the Research to The Cult of Hesh",
    desc = [[{hesh_buyer} is buying up artifacts and research to destroy them in the name of Hesh.]],
    mark = { "hesh_buyer"},
    icon = engine.asset.Texture( "UI/faction_cult_of_hesh.tex" ),

    on_activate = function(quest)
        quest:AssignCastMember("hesh_buyer")
    end,

    on_complete = function(quest)
        quest:Cancel("sell_to_admiralty")
        quest:Cancel("sell_to_jakes")
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "sell_to_jakes",
    title = "Sell the Research to the Jakes",
    desc = [[{jakes_buyer} is looking for high-value goods to smuggle out of the Grout Bog.]],
    mark = {"jakes_buyer"},
    icon = engine.asset.Texture( "UI/faction_smugglers_jakes.tex" ),
    on_activate = function(quest)
        quest:AssignCastMember("jakes_buyer")
    end,

    on_complete = function(quest)
        quest:Cancel("sell_to_admiralty")
        quest:Cancel("sell_to_hesh")
        quest:Activate("return_for_reward")
    end,
}

:AddObjective{
    id = "return_for_reward",
    title = "Report back to {giver}",
    desc = [[Tell {giver} about the deal.]],
    mark = { "giver" },
    terminal = true,
}


:AddOpinionEvents{
    chose_to_sell =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Chose to sell them stolen Baron research",
    },--]]
    chose_to_sell_kickback =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Chose to sell them stolen Baron research, but demanded a kickback first",
    },--]]
    
}




----------QUEST INTRODUCTION

QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                I have a lead. A researcher who's none too happy with the terms of their contract.
                If we could convince them to leak some secrets, we could sell them on for a profit.
                That money could go a long way to supporting the cause this quarter.
            player:
                Convince them how, exactly?
            agent:
                That's up to you, ain't it? With honey or vinegar. 
                Both taste sweet compared to bog water, anyway.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                All you need is access to the research camp. It's off-limits to basic laborers.
                Something inside is bound to be valuable.
            player:
                I'll see what can be done. 
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


----------CONFRONT THE RESEARCHER. 
--There are a couple of different reasons that they can be mad, which give you different ways to optionally skip the card encounter. It's mostly just for RP flavour.
-- If you beat this person up, they don't give you the password as well, so you'll end up in trouble in the next step when you get interrupted.

QDEF:AddConvo("contact_researcher", "researcher")
    :Loc{
        OPT_ASK_ABOUT_RESEARCH = "Ask {agent} about {agent.hisher} research",
        DIALOG_ASK_ABOUT_RESEARCH = [[
            player:
                I've heard that you're an extremely gifted researcher of some of the Bog's... deeper secrets.
                The Barons must be very lucky to have you. 
            agent:
                Hah! Supposedly.
            player:
                Oh dear. I'm sorry to hear it. 
                I don't mean to pry, but ... 
                I'm an admirer of your work, and it would be a tragedy if you weren't able to achieve your true potential.
                Surely the Barons must be willing to give you whatever you need for your work.
            {researcher_motivation_moral?
                agent:
                    Oh, certainly, if what I need is silk to line my pockets while the laborers go unrewarded.
                    We dig up Vagrant Age tech and never question what ruined that once-great civilization to begin with.
                player:
                    That does sound frustrating.
                agent:
                    More than that! The Barons are only interested in profiting off the tech we find, not in understanding it.
                    But those who don't study history are doomed to repeat it.
                    Who's to say it won't be the bones of <i>our</i> society that lay rotting, long-forgotten?
            }
            
            {researcher_motivation_greed?
                agent:
                    Certainly they give me what I need. Tools, supplies, a bunk, food.
                    But that only keeps me going while I remain in Baron employ. 
                    I was promised exorbitant pay depending on the rewards I find, but they haven't delivered.
                    When my contract is done here, I'll be as destitute as I was when I took the position. 
            }
            
            {researcher_motivation_bog_madness?
                agent:
                    Pah! How can they know what any of us truly need?
                    They follow their noses to riches all the while ignoring the seductive stench of the Bog itself.
                player:
                    Seductive, you say?
                agent:
                    The Barons think we're digging for Vagrant Age tech to adapt to our needs.
                    But that tech has been down there for millennia. It's changed. Developed.
                    What we have now isn't ancient ruins but the skeleton of the very Bog itself. 
                    The Barons aren't excavators. They're parasites. 
            }
        ]],

        
        OPT_GIVE_BOG_SECRET = "Give {1#card} to {agent}",
        REQ_DONT_HAVE_BOG_SECRETS = "You don't have {1#card}",
        DIALOG_GIVE_BOG_SECRET = [[
            player:
                You know... I did find something recently that made me realize the Bog is much more than it seems.
            agent:
                You found something? What? Show me!
            player:
                You're not the only one with a curiosity too ravenous to satisfy in one sup. 
                I would like access to the research camp. 
            agent:
                Yes, yes, who cares about that. The camp is nothing but Baron-minded junk. 
                What I need is deeper than that. Eternal. 
            * You lean forward to whisper the secrets you've unearthed into {agent}'s ear. {agent.HeShe} swallows the knowledge ravenously.
            agent:
                !give
                Here, take my keycard. And make note of this password. You'll need both to enter the camp.
            * {agent} turns away, seemingly entranced with the wisdom you've provided.
        ]],

        OPT_PAY_MONEY = "Bribe {agent}",
        DIALOG_PAY_MONEY = [[
            player:
                That truly is a shame. Especially because I happen to have money that certainly deserves a new home.
            agent:
                You want to hire me? What for?
            player:
                Access to the research camp. I, too, am intrigued by its secrets.
                But of course the Barons don't see fit to share as generously as they should.
            agent:  
                ...
                You know what? Deal. My contract is up soon anyway, and I'll have nothing to show for it.
                !give
                Here, take my keycard. And make note of this password. You'll need both to enter the camp.
            * {agent} takes the money you offer and hides it quickly in {agent.hisher} coat. 
        ]],

        OPT_BRAG_ABOUT_BARON_HATES = "Tell {agent} about your reputation with the Barons",
        REQ_ENOUGH_BARON_DISLIKES = "<#PENALTY>Not enough Barons dislike you</>",
        TT_BARON_DISLIKES = "Spark Barons who don't like you:\n<#PENALTY>{1#agent_list}</>",
        DIALOG_BRAG_ABOUT_BARON_HATES = [[
            player:
                It just so happens, well...
            agent:
                What?
            player:
                !cagey
                I happen to be aligned with the Rise.
            agent:
                Ah! The workers' rebellion? 
                Challenging the status quo, demanding a better future...
                Yes, that's the way we'll learn from the past. 
                Here, take this keycard. It will give you access to a research camp not too far from here. 
                You'll need a password to get past the guards, as well. 
                That camp is full of Baron secrets. You'll be able to find something to help your cause.
                I'm only sorry this is all I can do.
            player:
                No, my dear friend--this is marvelous. Thank you.
        ]],


        OPT_CONVINCE_TO_GIVE_ACCESS = "Convince {agent} to give you access to the research camp",
        DIALOG_CONVINCE_TO_GIVE_ACCESS = [[
            player:
                If you're displeased with the Barons, perhaps you ought to show them exactly how much.
            agent:
                What do you mean?
            player:
                A little chaos will make them appreciate what they have. 
                How about you give me access to the research camp?
                I promise not to damage anything. Just... make them think.
        ]],
        DIALOG_CONVINCE_TO_GIVE_ACCESS_SUCCESS = [[
            {researcher_motivation_moral?
                player:
                    Besides, it sounds like the Barons have ill intentions for the technology anyway.
                agent:
                    !sigh
                    They do. 
                    I don't know who you are or what your motivation is, but Hesh help me...
                    !give
                    Here, take my keycard. And make note of this password. You'll need both to enter the camp.
            }
            
            {researcher_motivation_greed?
                player:
                    Perhaps then the Barons will realize the value of what they have.
                    And of you. 
                agent:
                    Hm. That's not a bad idea. 
                    I mean, I'm a genius. If I can't parlay that into a raise then what good am I?
                    !give
                    Here, take my keycard. And make note of this password. You'll need both to enter the camp.
            }
            
            {researcher_motivation_bog_madness?
                player:
                    Surely a little bit of added entropy can only help reveal the Bog's secrets?
                agent:
                    Yes... yes, I think perhaps so.
                    Everyone should have a chance to learn as I have. To experience the Bog.
                    Not just the one on the surface but a fathom deep, closer to the beating heart.
                    !give 
                    Here, take my keycard. And make note of this password. You'll need both to enter the camp.
            }

        ]],
        DIALOG_CONVINCE_TO_GIVE_ACCESS_FAILURE = [[
            {researcher_motivation_moral?
                player:
                    Surely the Barons won't use it safely or wisely.
                agent:
                    !dubious
                    But you will? 
                    I have no idea who you are or what your motivations are.
            }
            
            {researcher_motivation_greed?
                player:
                    After all, if you can't profit, why should they?
                agent:
                    Are you kidding? If anything happens to that research camp they'll pin it on me. 
                    Then I'll be <i>indebted</i> to them. And that's even worse than being broke. 
            }
            
            {researcher_motivation_bog_madness?
                player:
                    Perhaps the Bog would be pleased, too.
                agent:
                    The Bog would be <i>pleased</i>? 
                    You know nothing. The Bog is not an oshnu that can be stroked between its eyestalks. 
                    Only I can learn its secrets. And my studies are not yet complete.
            }
        ]],

        OPT_ROUGH_UP = "Beat up {agent}",
        DIALOG_ROUGH_UP = [[
            player:
                Well, I'm sorry, professor. 
                But I'm afraid progress waits for no one. 
                !fight
        ]],
        
        DIALOG_WON_BATTLE = [[
            {dead?
                * {agent} is dead. You search through {agent.hisher} pockets and find a keycard. That will have to do.
            }

            {not dead?
                
                agent:
                    !injured

                {researcher_motivation_moral?
                    agent:
                        So that's what you are, huh? Just another thug?
                        You're no better than the Barons, but maybe you deserve each other. 
                        Take the keycard. And may you fall into a chute on your way.
                }
                
                {researcher_motivation_greed?
                    agent:
                        Stop! I promised myself I wouldn't die for this job. 
                        Take the keycard. Whatever it takes to just let me live.
                }
                
                {researcher_motivation_bog_madness?
                    agent:
                        E-enough! You've beaten me, and the Bog didn't rise up to my defense.
                        Perhaps it--it <i>wants</i> you to go. 
                        Take the keycard. Go inside and see for yourself.
                        And when you've heard the gurgling of its guts, maybe we'll understand each other. 
                }
            }
        ]],

        OPT_LEAVE_FOR_NOW = "Leave {agent} for now",
        DIALOG_LEAVE_FOR_NOW = [[
            player:
                Don't go anywhere. We're not done.
        ]]

    }

    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_ABOUT_RESEARCH")
            :LoopingFn(function() 
                if cxt:FirstLoop() then
                    cxt:Dialog("DIALOG_ASK_ABOUT_RESEARCH")
                end


                if cxt.quest.param.researcher_motivation_greed then
                    cxt:Opt("OPT_PAY_MONEY")
                        :Dialog("DIALOG_PAY_MONEY")
                        :DeliverMoney(cxt.quest:GetRank()*50)
                        :CompleteQuest("contact_researcher")
                        :End()
                end
                
                if cxt.quest.param.researcher_motivation_bog_madness then
                    local bog_secret_card = cxt.player.battler:FindCardByID("deep_bog_battle")
                    cxt:Opt("OPT_GIVE_BOG_SECRET", "deep_bog_battle")
                        :ReqCondition(bog_secret_card ~= nil, "REQ_DONT_HAVE_BOG_SECRETS", "deep_bog_battle")
                        :Dialog("DIALOG_GIVE_BOG_SECRET")
                        :Fn(function() 
                            cxt.player.battler:RemoveCard(bog_secret_card)
                        end)
                        :CompleteQuest("contact_researcher")
                        :End()
                end

                if cxt.quest.param.researcher_motivation_moral then
                    
                    local baron_dislikes = fun(cxt.player.social_connections.relationships)
                                                :filterKeysByValues(function(rel, agent) return rel < RELATIONSHIP.NEUTRAL and agent:GetFactionID() == "SPARK_BARONS" and not agent:IsDead() end)
                                                :value()

                    cxt:Opt("OPT_BRAG_ABOUT_BARON_HATES")
                        :Dialog("DIALOG_BRAG_ABOUT_BARON_HATES")
                        :ReqCondition(#baron_dislikes > 4, "REQ_ENOUGH_BARON_DISLIKES")
                        :PostText("TT_BARON_DISLIKES", baron_dislikes)
                        :Fn(function() 

                        end)
                        :CompleteQuest("contact_researcher")
                        :End()
                end

                cxt:Opt("OPT_CONVINCE_TO_GIVE_ACCESS")
                    :Dialog("DIALOG_CONVINCE_TO_GIVE_ACCESS")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_TO_GIVE_ACCESS_SUCCESS")
                            :CompleteQuest("contact_researcher")
                            :End()
                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_TO_GIVE_ACCESS_FAILURE")

                cxt:Opt("OPT_ROUGH_UP")
                    :Dialog("DIALOG_ROUGH_UP")
                    :Battle{}
                        :OnWin()
                            :Dialog("DIALOG_WON_BATTLE")
                            :CompleteQuest("contact_researcher")
                            :End()

                cxt:Opt("OPT_LEAVE_FOR_NOW")
                    :Dialog("DIALOG_LEAVE_FOR_NOW")
                    :End()

            end)
    end)


----STEAL THE RESEARCH
-- You go to the research site, and take the research. You are discovered by a patrol, and have to talk/fight your way out of it.
QDEF:AddConvo("steal_the_research")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("research_camp") end)
        :Loc{
            DIALOG_INTRO = [[
                * The research camp is easy enough to find, but at first you second guess yourself. Strangely, it's deserted.
                player:
                    !left
                * Most of the supplies and tools have been removed from the site. But a locked shed looks promising.
            ]],

            OPT_USE_KEY = "(Use key) Search the shed",
            DIALOG_USE_KEY = [[
                player:
                    !exit
                * Using the key that you got from {researcher}, you unlock the shed and peer inside.
                * You find some minor Bog trinkets, and a ream of technical notes.
            ]],

            OPT_TAKE_TRINKET = "Take {1#card}",
            DIALOG_TAKE_TRINKET = "* You take it.",
            
            OPT_LEAVE_REST = "Leave the rest",
            DIALOG_LEAVE_REST = [[
                * You leave the rest. It didn't look terribly important or interesting.
            ]],

            OPT_TAKE_NOTES = "Take the notes",
            DIALOG_TAKE_NOTES = [[
                * You take the notes.
            ]],
            DIALOG_EMPTY_SHED = [[
                * There's nothing else of interest in the shed.
            ]],

        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            
            local trinkets = {"jammed_blaster", "flead_egg", "lock_box", "deep_bog_battle"}
            local trinket_1 = fun(trinkets)
                                :randomPick()
            local trinket_2 = fun(trinkets)
                                :filter(function(id) return id ~= trinket_1 end)
                                :randomPick()
            local took_notes = false


            cxt:Opt("OPT_USE_KEY")
                :Dialog("DIALOG_USE_KEY")

                :LoopingFn(function() 
                    if trinket_1 then
                        cxt:Opt("OPT_TAKE_TRINKET", trinket_1)
                            :Dialog("DIALOG_TAKE_TRINKET", trinket_1)
                            :GainCard(trinket_1)
                            :Fn(function() 
                                cxt.quest.param.took_a_trinket = true
                                trinket_1 = nil 
                            end)
                    end


                    if trinket_2 then
                        cxt:Opt("OPT_TAKE_TRINKET", trinket_2)
                            :Dialog("DIALOG_TAKE_TRINKET", trinket_2)
                            :GainCard(trinket_2)
                            :Fn(function() 
                                cxt.quest.param.took_a_trinket = true
                                trinket_2 = nil 
                            end)
                    end

                    if not took_notes then
                        cxt:Opt("OPT_TAKE_NOTES")
                            :Dialog("DIALOG_TAKE_NOTES")
                            :Fn(function() 
                                took_notes = true 
                                if trinket_1 == nil and trinket_2 == nil then
                                    cxt:Dialog("DIALOG_EMPTY_SHED")
                                    cxt:GoTo("STATE_INTERRUPTION")
                                end
                            end)
                    else

                        cxt:Opt("OPT_LEAVE_REST")
                            :Dialog("DIALOG_LEAVE_REST")
                            :GoTo("STATE_INTERRUPTION")
                    end
                end)

         end)
    
    :State("STATE_INTERRUPTION")
        :Loc{
            DIALOG_INTRO = [[
                * On your way out, however, you hear the sounds of footsteps squelching in the mud.
                agent:
                    !right
                    What the...
                    Why's this open? 
                    !surprise
                    H-hey! Who's in there?
                player:
                    !left
                agent:
                    !angry
                    This area's off-limits, grifter! 
                    How'd you even get in here?

                {took_a_trinket?
                    * {agent} notices the trinket you took from inside the shed.
                    agent:
                        !angry_accuse
                        So it's a thief, is it? 
                }
            ]],
            MOD_USED_PASSWORD = "You used the security password",
            MOD_HAS_TRINKET = "You stole items from the shed",


            OPT_GIVE_PASSWORD = "Give {agent} the security password",
            DIALOG_GIVE_PASSWORD = [[
                player:
                    Calm yourself! I'm here on Baron business. Fellemo wants this trinket for himself.
                * You tell {agent} the password. {agent.HeShe} hesitates, but keeps one hand on {agent.hisher} weapon.
                agent:
                    Why send you? Why not just get one of us to deliver it?
            ]],

            OPT_CONVINCE_WORKING_FOR_BARONS = "Convince {agent} you're working for the Barons",
            DIALOG_CONVINCE_WORKING_FOR_BARONS = [[
                player:
                    Because I have, ah, special skills.
            ]],
            DIALOG_CONVINCE_WORKING_FOR_BARONS_SUCCESS = [[
                player:
                    Fellemo suspects corporate sabotage. 
                    Of course, any Barons found to be working against him will be discovered and duly punished.
                agent:
                    !placate
                    O-oh. Sabotage?
                    That ain't got nothing to do with me. If Fellemo's investigating, I'll do whatever I can to help.
                    !exit
                * The patrol backs away, eager to put space between you. 
            ]],
            DIALOG_CONVINCE_WORKING_FOR_BARONS_FAILURE = [[
                player:
                    It's far too complicated for someone like you to understand--
                agent:
                    Sounds like all your skills did was walk you into trouble.
                    If you're really here for Fellemo, he'd understand: security comes first.
                    !fight
            ]],

            OPT_FIGHT_YOUR_WAY_OUT = "Fight your way out",
            DIALOG_FIGHT_YOUR_WAY_OUT = [[
                player:
                    Ah, well. Nothing like a good brawl to finish off a job well done.
                    !fight
            ]],

            DIALOG_WON_BATTLE = [[
                {dead ?
                    player:
                        !exit
                    * You move the bodies to the back of the shed and lock the door behind you once again.
                }
                {not dead?
                    agent:
                        !injured
                        Scum! 
                    player:
                        I've left you with your life, haven't I?
                        Perhaps you should mind yourself. 
                        !exit
                    * You leave quickly, obscuring your trail as you do. 
                }
            ]],

            DIALOG_DEFEND = [[
                player:
                    Ah, well. Nothing like a good brawl to finish off a job well done.
                    !fight

            ]]

        }
        :SetLooping(true)
        :Fn(function(cxt) 
            
            if cxt:FirstLoop() then
                local team =  CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location, true)
                cxt:TalkTo(team[1])
                cxt:Dialog("DIALOG_INTRO")
            end
            
            if not cxt.quest.param.has_password and not cxt.quest.param.gave_password then
                cxt:Opt("OPT_GIVE_PASSWORD")
                    :Dialog("DIALOG_GIVE_PASSWORD")
                    :Fn(function() cxt.quest.param.gave_password = true end)
            end

            local modifiers = {}
            if cxt.quest.param.gave_password then
                table.insert(modifiers, { value = -10, text = cxt:GetLocString("MOD_USED_PASSWORD") })
            end
            if cxt.quest.param.took_a_trinket then
                table.insert(modifiers, { value = 20, text = cxt:GetLocString("MOD_HAS_TRINKET") })
            end

            cxt:Opt("OPT_CONVINCE_WORKING_FOR_BARONS")
                :Dialog("DIALOG_CONVINCE_WORKING_FOR_BARONS")
                :Negotiation{
                            situation_modifiers = modifiers
                        }
                    :OnSuccess()
                        :Dialog("DIALOG_CONVINCE_WORKING_FOR_BARONS_SUCCESS")
                        :CompleteQuest("steal_the_research")
                        :Travel()
                    :OnFailure()
                        :Dialog("DIALOG_CONVINCE_WORKING_FOR_BARONS_FAILURE")
                        :Fn(function() 
                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        on_win = function()
                                            cxt:Dialog("DIALOG_WON_BATTLE")
                                            cxt.quest:Complete("steal_the_research")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end,
                                    }
                        end)

            
            cxt:Opt("OPT_FIGHT_YOUR_WAY_OUT")
                :Dialog("DIALOG_FIGHT_YOUR_WAY_OUT")
                :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function()
                            cxt:Dialog("DIALOG_WON_BATTLE")
                            cxt.quest:Complete("steal_the_research")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,

                    }

        end)




----SELL THE RESEARCH
-- You can sell it to a faction of your choice. They'll all give you something different for it.

--hesh agent gives you a bunch of money
QDEF:AddConvo("sell_to_hesh", "hesh_buyer")
    :Loc{
        OPT_OFFER_RESEARCH = "Offer to sell the research",
        OPT_DIALOG_OFFER_RESEARCH = [[
            player:
                Hail, Heshian. What brings you to this part of the world?
                Barons don't take too kindly to your sort.
            agent:
                Because they pilfer our relics and twist them into abominations. 
            player:
                Perhaps you'd be interested in purchasing some of their research notes?
            agent:
                Indeed I would. If only to expunge it from this world. 
        ]],

        OPT_DEMAND_KICKBACK = "Demand a kickback",
        DIALOG_DEMAND_KICKBACK = [[
            player:
                Surely this is worth more than market value?
                There's a moral need here, as well. 
        ]],
        DIALOG_DEMAND_KICKBACK_SUCCESS = [[
            player:
                The price of purity can never be too high, after all.
            agent:
                Indeed. The coffers of Hesh run deep when it comes to these matters.
        ]],
        DIALOG_DEMAND_KICKBACK_FAILURE = [[
            player:
                Surely Hesh can afford a little extra, can't it?
            agent:
                You're as bad as the Barons. A parasite upon the grace of Hesh.
                You will take what I offer and not a shill more.
        ]],

        OPT_SELL_THE_RESEARCH = "Sell the research to {agent}",
        DIALOG_SELL_THE_RESEARCH = [[
            player:
                Very well. You have a deal.
                !give
            agent:
                !take
                Excellent.
        ]],

        OPT_MAYBE_LATER = "Back out of the deal",
        DIALOG_MAYBE_LATER = [[
            player:
                Well, there are <i>other</i> buyers. It's only fair I hear what they have to say first, hm?
            agent:
                !disappoint
                $miscDepressed
                As is your... right. I suppose.
        ]]

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_OFFER_RESEARCH")
            :Dialog("OPT_DIALOG_OFFER_RESEARCH")
            :LoopingFn(function() 
                
                local HESH_KICKBACK = 200

                cxt:Opt("OPT_DEMAND_KICKBACK")
                    :Dialog("DIALOG_DEMAND_KICKBACK")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_DEMAND_KICKBACK_SUCCESS", HESH_KICKBACK)
                            :Fn(function() 
                                cxt.quest.param.hesh_kickback = true 
                                
                            end)
                        
                        :OnFailure()
                            :Dialog("DIALOG_DEMAND_KICKBACK_FAILURE")

                local opt = cxt:Opt("OPT_SELL_THE_RESEARCH")
                    :Dialog("DIALOG_SELL_THE_RESEARCH")
                    :CompleteQuest("sell_to_hesh")
                    :Fn(function() 
                        cxt.quest.param.buyer = cxt:GetAgent() 
                        cxt.quest.param.sold_to_hesh = true 
                    end)
                    :End()

                if cxt.quest.param.hesh_kickback then
                    opt:ReceiveMoney(HESH_KICKBACK)
                    opt:ReceiveOpinion("chose_to_sell_kickback")
                else
                    opt:ReceiveOpinion("chose_to_sell")
                end

                cxt:Opt("OPT_MAYBE_LATER")
                    :Dialog("DIALOG_MAYBE_LATER")
                    :End()

            end)
    end)


--admiralty agent gives you a pet
QDEF:AddConvo("sell_to_admiralty", "admiralty_buyer")
    :Loc{
        OPT_OFFER_RESEARCH = "Offer to sell the research",
        OPT_DIALOG_OFFER_RESEARCH = [[
            player:
                Officer, I suspect you'll find this <i>very</i> tempting.
                I have some Spark Baron research notes here that could certainly be useful for someone in your profession.
                Would you be interested in purchasing them?
            agent:
                Hah! It would be nice to cut out the Barons for once.
                And we always need more weapons in Murder Bay. 
        ]],

        OPT_DEMAND_KICKBACK = "Demand a kickback",
        DIALOG_DEMAND_KICKBACK = [[
            player:
                And what can you offer me beyond the asking price?
                There's other interested parties, after all.
        ]],
        DIALOG_DEMAND_KICKBACK_SUCCESS = [[
            player:
                Normally I'd do it out of patriotism, but my operations here have quite the overhead.
            agent:
                Hm. I can issue you a trained vroc, if you like. 
                I'm allergic to them anyway.
        ]],
        DIALOG_DEMAND_KICKBACK_FAILURE = [[
            player:
                The Admiralty in particular owes me more than most.
            agent:
                Oh, is that what this is about? 
                <i>You</i> were dishonourably discharged, {player}. That's got nothing to do with me.
                !crossed
                The price is fixed.
        ]],

        OPT_SELL_THE_RESEARCH = "Sell the research to {agent}",
        DIALOG_SELL_THE_RESEARCH = [[
            player:
                player:
                Very well. You have a deal.
                !give
            agent:
                !take
                Excellent.
        ]],
        DIALOG_GET_VROC = [[
            agent:
                Here's the vroc. 
                !palm
                J-just get it out of here quick. My eyes are already itchy.
            vroc:
                !right
            player:
                Come along, my good boy. There's work to be done for a man and his faithful companion.
            vroc:
                Grrrap!
        ]],
        DIALOG_GET_VROC_ROOM = [[
            agent:
                I'll have the vroc sent along to you later.
            player:
                Very good. I have a room at the Last Stand; I'm sure Hebbel wouldn't mind, considering his clientel. 
        ]],

        OPT_MAYBE_LATER = "Back out of the deal",
        DIALOG_MAYBE_LATER = [[
            player:
                Well, there are <i>other</i> buyers. It's only fair I hear what they have to say first, hm?
            agent:
                !disappoint
                $neutralResigned
                As is your... right. I suppose.
        ]],
        TT_KICKBACK = "You will receive a Vroc as a kickback"

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_OFFER_RESEARCH")
            :Dialog("OPT_DIALOG_OFFER_RESEARCH")
            :LoopingFn(function() 

                cxt:Opt("OPT_DEMAND_KICKBACK")
                    :Dialog("DIALOG_DEMAND_KICKBACK")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_DEMAND_KICKBACK_SUCCESS")
                            :Fn(function() cxt.quest.param.admiralty_kickback = true end)
                        
                        :OnFailure()
                            :Dialog("DIALOG_DEMAND_KICKBACK_FAILURE")

                local opt = cxt:Opt("OPT_SELL_THE_RESEARCH")
                    :Dialog("DIALOG_SELL_THE_RESEARCH")
                    :CompleteQuest("sell_to_admiralty")
                    
                    :Fn(function() 
                        cxt.quest.param.buyer = cxt:GetAgent()
                        cxt.quest.param.sold_to_admiralty = true 
                        if cxt.quest.param.admiralty_kickback then
                            local agent = Agent("VROC")
                            agent:GenerateRandomName()
                            TheGame:GetGameState():AddAgent( agent )
                            cxt:ReassignCastMember("vroc", agent)

                            if TheGame:GetGameState():GetCaravan():CanOwnPet() then
                                cxt:Dialog("DIALOG_GET_VROC")
                                agent:Recruit(PARTY_MEMBER_TYPE.CREW)    
                            else
                                cxt:Dialog("DIALOG_GET_VROC_ROOM")
                                agent:AddTag("NO_AUTO_CULL")
                                agent:SetLocationRole(CHARACTER_ROLES.WAITING_PET)
                                agent:MoveToLocation( TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR.inn_room") )
                            end

                            agent:TrainPet()
                        end
                    end)
                    :End()

                if cxt.quest.param.admiralty_kickback then
                    opt:PostText("TT_KICKBACK")
                    opt:ReceiveOpinion("chose_to_sell_kickback")
                else
                    opt:ReceiveOpinion("chose_to_sell")
                end

                cxt:Opt("OPT_MAYBE_LATER")
                    :Dialog("DIALOG_MAYBE_LATER")
                    :End()

            end)
    end)    


--jakes gives you a bunch of items
QDEF:AddConvo("sell_to_jakes", "jakes_buyer")
    :Loc{
        OPT_OFFER_RESEARCH = "Offer to sell the research",
        OPT_DIALOG_OFFER_RESEARCH = [[
            player:
                You look like someone who knows an opportunity when it comes along.
                I have some Spark Baron research notes that are looking to make their way to the right market.
            agent:
                Yep, that's my area of expertise. I'll take the notes if you're selling.
        ]],

        OPT_DEMAND_KICKBACK = "Demand a kickback",
        DIALOG_DEMAND_KICKBACK = [[
            player:
                And what can you offer me that the other buyers can't?
        ]],
        DIALOG_DEMAND_KICKBACK_SUCCESS = [[
            player:
                I know you'll be able to turn this around for a hefty profit. 
                Surely my takeaway should reflect that.
            agent:
                Well, since you're selling stolen materials...
                How about like begets like? I can give you contraband.
        ]],
        DIALOG_DEMAND_KICKBACK_FAILURE = [[
            player:
                I know you'll be able to turn this around for a hefty profit.
                And that's hardly fair, is it?
            agent:
                Fair? It's my job. 
                You don't like it, then sod off.
        ]],

        OPT_SELL_THE_RESEARCH = "Sell the research to {agent}",
        DIALOG_SELL_THE_RESEARCH = [[
            player:
                player:
                Very well. You have a deal.
                !give
            agent:
                !take
                Excellent.
        ]],
        DIALOG_GET_ITEMS = [[
            agent:
                And here's what's yours. 
                Careful who catches you with it. After all, it's contraband.
        ]],

        TT_KICKBACK = "You will get {1#card_list}",

        OPT_MAYBE_LATER = "Back out of the deal",
        DIALOG_MAYBE_LATER = [[
            player:
                Well, there are <i>other</i> buyers. It's only fair I hear what they have to say first, hm?
            agent:
                !disappoint
                $neutralResigned
                As is your... right. I suppose.
        ]]

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_OFFER_RESEARCH")
            :Dialog("OPT_DIALOG_OFFER_RESEARCH")
            :LoopingFn(function() 

                cxt:Opt("OPT_DEMAND_KICKBACK")
                    :Dialog("DIALOG_DEMAND_KICKBACK")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_DEMAND_KICKBACK_SUCCESS")
                            :Fn(function() cxt.quest.param.jakes_kickback = true end)
                        :OnFailure()
                            :Dialog("DIALOG_DEMAND_KICKBACK_FAILURE")

                
                local opt = cxt:Opt("OPT_SELL_THE_RESEARCH")
                    :Dialog("DIALOG_SELL_THE_RESEARCH")
                    :CompleteQuest("sell_to_jakes")
                    :Fn(function() 
                        cxt.quest.param.buyer = cxt:GetAgent() 
                        cxt.quest.param.sold_to_jakes = true 
                    end)
                    :End()

                if cxt.quest.param.jakes_kickback then
                    local card_list = {"tincture", "vial_of_slurry", "spark_grenade", "stun_gun", "sequencer"}
                    opt:PostText("TT_KICKBACK", card_list)
                    opt:GainCards(card_list)
                    opt:ReceiveOpinion("chose_to_sell_kickback")
                else
                    opt:ReceiveOpinion("chose_to_sell")
                end

                cxt:Opt("OPT_MAYBE_LATER")
                    :Dialog("DIALOG_MAYBE_LATER")
                    :End()

            end)
    end)    


--turn the quest in to the giver.
QDEF:AddConvo("return_for_reward", "giver")

    :Loc{
        OPT_REPORT_SALE = "Tell {agent} that {1#agent} is the buyer",   
        DIALOG_REPORT_SALE = [[
            {sold_to_jakes? 
                player:
                    It's done.
                    {jakes_buyer} has agreed to the sale.
                agent:
                    You sold it to the jakes? Hm, I guess they'll be discreet.
                    But they might just go ahead and sell it right back to the Barons.
                player:
                    Yes, but you'll be richer all the same.
            }
            {sold_to_hesh? 
                player:
                    It's done.
                    {hesh_buyer} has agreed to the sale.
                agent:
                    Ah, that's a shame. I mean, the sale is good!
                    But those zealots will just destroy what you give them. 
                    It's a shame to see that technology go to waste is all.
            }
            {sold_to_admiralty? 
                player:
                    It's done.
                    {admiralty_buyer} has agreed to the sale.
                agent:
                    Hrm. The Admiralty has had their own share of run-ins with other Rise cells outside the Bog.
                    But at least they won't use that against us here.
            }
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_REPORT_SALE", cxt.quest.param.buyer)
            :Dialog("DIALOG_REPORT_SALE")
            :CompleteQuest()
            :Fn(function() 
                ConvoUtil.GiveQuestRewards(cxt)
            end)
            :End()
    end)


---------------------------------------------------------------------------------------------------

QDEF:AddDebugOption("researcher_motivation", {"MORAL", "GREED", "BOG_MADNESS"})

