local RALLY_EVENT_OVERRIDE

--to force a particular event at the rally:
--RALLY_EVENT_OVERRIDE = "STATE_BUST_UP"
-- "STATE_PROVACATEUR" "STATE_HECKLER" "STATE_BUST_UP"


local MIN_NUM_ATTENDEES = 2
local MAX_NUM_ATTEMPTS = 3

local QDEF = QuestDef.Define{
        title = "Grist for the Mill",
        desc = "The Rise is holding a recruitment rally in the workers' quarter. Convince a crowd to attend, and provide security at the meeting.",
        icon = engine.asset.Texture("icons/quests/rise_worker_rally.tex"),
        faction = "RISE",

        focus = QUEST_FOCUS.NEGOTIATION,
        rank = {1, 5},
        reward_size = QREWARDSIZE.SMALLER,    
        qtype = QTYPE.CONTRACT,
        on_init = function(quest) 
            quest.param.attendees = {}
            quest.param.detractors = {}
            return true
        end,
        on_start = function(quest)
            quest:Activate("convince_target")
        end,

        avoids = {
            {"target", "giver"},
        }
    }

    :AddObjective{
        id = "convince_target",
        title = "Convince {target} to attend the rally",
        desc = "{target} has expressed interest in the Rise before. Go give {target.himher} a sales pitch {target.heshe} can't ignore.",
        mark = { "target" },
        negotiation_targets = {"target"},
        
        on_activate = function(quest)
            quest:AssignCastMember( "target" )
        end,
    }

    :AddObjective{
        id = "spark_intervention",
    }


    :AddObjective{
        id = "attend_rally",
        title = "Attend the Rally",
        desc = "{giver} was expecting trouble at the rally. Go keep an eye on things.",
        mark = { "rally_location" },
        on_activate = function(quest)
            quest:SpawnTempLocation("RALLY_LOCATION", "rally_location" )
        end,
        on_complete = function(quest)
            quest:Activate("return_for_reward")
        end,
    }


    :AddObjective{
        id = "return_for_reward",
        title = "Report to {giver}",
        desc = [[Tell {giver} how the rally went.]],
        ready_for_completion = true,
        mark = {"giver"},
        terminal = true
    }


    :AddCast{
        when = QWHEN.MANUAL,
        cast_id = "rally_location",
        no_validation = true,
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        no_validation = true,
    }

    :AddCast{
        cast_id = "target",
        when = QWHEN.MANUAL,
        condition = function(agent, quest)
            return (agent.def.id == "HEAVY_LABORER" or agent.def.id == "LABORER") and 
                    not agent:HasAspect("sympathizer") and
                    (quest.param.attendees == nil or table.arrayfind(quest.param.attendees, agent)) and
                    (quest.param.detractors == nil or table.arrayfind(quest.param.detractors, agent))
        end,
        on_assign = function(quest, agent)
            local num = (quest.param.attendees and #quest.param.attendees or 0) + (quest.param.detractors and  #quest.param.detractors or 0)
            if num == 0 then
                AgentUtil.MakeSympathizer(agent, "RISE")
            elseif num >= 2 then
                AgentUtil.MakeSympathizer(agent, "SPARK_BARONS")
            end

        end,
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local def = "HEAVY_LABORER"
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }

    :AddCast{
        cast_id = "speaker",
        when = QWHEN.MANUAL,
        condition = function(agent, quest)
            return agent.def.id == "RISE_PAMPHLETEER"
        end,
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent( "RISE_PAMPHLETEER" )
            table.insert( t, agent )
        end,
    }    


    :AddLocationDefs{
        RALLY_LOCATION = {
            name = "Rise Rally Location",
            plax = "INT_WorkCamp_Res",
            indoors = false,
            map_tags = {"quarters"},
        },
    }



---------------------------------------------------------------------------------------------------------------

function DoPostNegotiation(cxt, success) 

    if AgentUtil.IsSympathizer(cxt:GetAgent(), "RISE") then
        cxt:Dialog(success and "DIALOG_WILL_ATTEND_RISE" or "DIALOG_NO_WAY_RISE")
    elseif AgentUtil.IsSympathizer(cxt:GetAgent(), "SPARK_BARONS") then
        cxt:Dialog(success and "DIALOG_WILL_ATTEND_SPARK_BARONS" or "DIALOG_NO_WAY_SPARK_BARONS")
    else
        cxt:Dialog(success and "DIALOG_WILL_ATTEND" or "DIALOG_NO_WAY")
    end

    if success then
        table.insert( cxt.quest.param.attendees, cxt:GetAgent())
    else
        table.insert( cxt.quest.param.detractors, cxt:GetAgent())
    end

    cxt.quest:Complete("convince_target")
    
    if (#cxt.quest.param.attendees + #cxt.quest.param.detractors) >= MAX_NUM_ATTEMPTS then
        if not cxt.quest:IsActive("attend_rally") then
            cxt.quest:Activate("attend_rally")
        end
        cxt:Dialog("DIALOG_RALLY_TIME")
    else
        cxt.quest:UnassignCastMember("target")
        cxt.quest:Activate("convince_target")

        if #cxt.quest.param.attendees >= MIN_NUM_ATTENDEES and not cxt.quest:IsActive("attend_rally") then
            cxt:Dialog("DIALOG_NEXT_TARGET_OPTIONAL")
            if not cxt.quest:IsActive("attend_rally") then
                cxt.quest:Activate("attend_rally")
            end

        else
            cxt:Dialog("DIALOG_NEXT_TARGET")
        end
        
    end

    if cxt.quest:IsInactive("spark_intervention") then
        cxt.quest:Activate("spark_intervention")
    end
    StateGraphUtil.AddEndOption(cxt)

end


QDEF:AddConvo("convince_target", "target")
    :Loc{
        OPT_CONVINCE_TARGET_TO_ATTEND = "Convince {target} to attend the rally",
        DIALOG_TRY_CONVINCE_RISE = [[
            player:
                !cagey
                I understand you may be interested in learning more about the labor movement.
            target:
                !suspicious
                What are you, a narc? 
                Look, I'm staying out of trouble, see? 
                $angryDefault
                I don't know nothing about no rally and I don't intend to.
            player:
                At ease, there. I'm no narc, I assure you.
                Quite the contrary: I'm a narc-repellant. 
        ]],
        DIALOG_TRY_CONVINCE_NEUTRAL = [[
            player:
                We'll see you at the rally tonight, eh?
                !thought
                I hear it's going to be quite the, ah, "banger", to borrow the local parlance.
            target:
                Rally? Hesh, no. 
                I told Kalandra and that jarackle of hers that I wanted nothing to do with it.
            player:
                Ah, I see. You don't need the Rise, eh? You can look after yourself.
            target:
                More or less, yeah.
            player:
                Well which is it? More, or less?
        ]],
        DIALOG_TRY_CONVINCE_SPARK_BARONS = [[
            player:
                !happy
                I come bringing news: despite what you might have heard, the labor rally is still on.
                Rain or shine, bog fog be damned.
            target:
                $neutralJoke
                Oh, really? That <i>is</i> good news.
                !crossed
                I ain't going, of course.
            player:
                !dubious
                $miscPersuasive
                Are you sure? You wouldn't want to miss the event of the season, would you?
        ]],

        DIALOG_NO_WAY = [[
            player:
                I'm sure you think you can look after yourself.
                After all, you seem to have done well enough already.
                But it's only a matter of time before the Barons come cracking down.
            target:
                !scared_shrug
                Which is exactly why I don't wanna go. I can't afford the demerits, let alone the time.
                I know you lot mean well. But just leave me out of it. 
            * You told {giver} this wasn't exactly your strong suit. But now you have the satisfaction of being right, at least.
        ]],

        DIALOG_NO_WAY_RISE = [[
            player:
                !overthere
                Surely you want to build a better future for yourself, yes?
            target:
                !disappoint
                What I want's got nothing to do with it.
                I can't go—I've got enough heat from the Barons already. 
                One more strike and I get another two years added to my contract. 
            * You can tell from {target.hisher} face that it's no good. At least you tried.
        ]],
        DIALOG_NO_WAY_SPARK_BARONS = [[
            player:
                You must go—not just for your own good but for your fellow laborer's.
                And for your children!
            target:
                !cruel
                Hah! The Barons haven't let us have kids since the riots ten years back.
                So you see what good organizing does. 
                Thanks for the invitation. I'm still not going.
            * You've seen this attitude before, and there's no cracking through it. 
        ]],

        DIALOG_WILL_ATTEND = [[
            player:
                You're not wrong that the rally will be trouble.
                The Spark Barons are sure to send enforcements, and they'll be looking for any excuse to penalize those in attendance.
                But {giver} has sent me to guarantee that you'll be safe in the crowd.
                Wear a mask if you like; if trouble comes to call, I'll be the one to answer the door.
                Giving you a chance to slip out the back, as it were.
            target:
                !dubious
                Huh. Really? You'd do that?
                Well, alright, then. Maybe I can mosey on by. Just to give it a glance.
                Not committing to the Rise or anything, mind. Just... willing to hear what Kalandra's got to say.
            * You hand {target} a pamphlet and cross {target.hisher} name off your list.
        ]],
        DIALOG_WILL_ATTEND_RISE = [[
            player:
                You've stood on your own two feet, have you?
                Never had another worker step up and cover your shift when you were lagging?
                Never had a colleague watch for Barons while you were trying to catch a nap between shifts?
            target:
                Well, I—I mean of course. Who hasn't?
                We gotta look out for each other, here, or the Barons would bleed us dry worse than the grawkits.
            player:
                Exactly.
            target:
                !sigh 
                Okay, smartypants. I get your point.
                I'll go. But Hesh help us if there's trouble.
            * That's one down. Maybe {giver} was right after all.
        ]],
        DIALOG_WILL_ATTEND_SPARK_BARONS = [[
            player:
                You think this is some rally, but it's actually a test of your mettle.
                Barons will be watching, yes, but they'll be looking for those who stand above the rest.
                Workers of merit. Workers of integrity. 
            target:
                Wha—
                Are you saying this rally is some sorta test?
            player:
                I'm not saying anything. 
                !wink
                But I will say that the only way to fail... is to fail to make an appearance.
            target:
                Well, Hesh. Nobody told me it wasn't a real rally.
                Sure I'll be there. With bells on!
                If I can requisition bells, that is. We're only alloted so many per workhouse. 
            * It's not honest, but it's still a win. 
        ]],
        DIALOG_RALLY_TIME = [[
            * You've worked through the whole list from {giver}, though it's hard to imagine it was worth it.
            * You should get to the rally. You'd hate to miss the opening act.
        ]],
        DIALOG_NEXT_TARGET = [[
            * The next name on the list is {target}.
        ]],
        DIALOG_NEXT_TARGET_OPTIONAL = [[
            * The next name on the list is {target}.
            * You probably have enough people convinced already, though, and the rally is starting soon. You <i>could</> just fudge the numbers, if you wanted.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE_TARGET_TO_ATTEND")
            :Fn(function(cxt) 
                    if AgentUtil.IsSympathizer(cxt:GetAgent(), "RISE") then
                        cxt:Dialog("DIALOG_TRY_CONVINCE_RISE")
                    elseif AgentUtil.IsSympathizer(cxt:GetAgent(), "SPARK_BARONS") then
                        cxt:Dialog("DIALOG_TRY_CONVINCE_SPARK_BARONS")
                    else
                        cxt:Dialog("DIALOG_TRY_CONVINCE_NEUTRAL")
                    end
                end)

            :Negotiation{
                situation_modifiers = AgentUtil.CreateSympathyModifiers(cxt:GetAgent(), "RISE", "SPARK_BARONS"),
                on_fail = function(cxt) DoPostNegotiation(cxt, false) end,
                on_success = function(cxt) DoPostNegotiation(cxt, true) end,
            }   
    end)


-------------------------------------------------------------------------------------------------------------------


QDEF:AddConvo("spark_intervention")
    
    :Confront(function(cxt) 
            if cxt.location:HasTag("in_transit") then
                return "STATE_TRY_THREAT"
            end
        end)   
    :State("STATE_TRY_THREAT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * You're stopped by a member of the Spark Barons, but you're used to pat-downs and interrogation. You double-check that the hidden pocket where you've stashed the pamphlets is sealed. 
                agent:
                    !angry
                    I hear someone's been rousin' the local rabble, spreading dissentionist materials. 
                    That wouldn't be you, would it?
                player:
                    !scared
                    Why, <i>no</i>. Wouldn't dream of it.
                agent:
                    !crossed
                    Somehow I find myself in need of convincing. 

            ]],
            OPT_PAY_OFF = "Pay off {agent}",
            DIALOG_BRIBED = [[
                * You open your palm revealing a fistful of shills. 
                player:
                    !permit
                    I respect your time. Why waste it on me?
                agent:
                    !cruel
                    Why indeed. 
                    !burp
                    Well, on your way, laborer. Let me know if you see any trouble-makers, will you?
                player:
                    !wink
                    You'll be the first to know.
                    !exit
                * You hear your money jangling in {agent}'s pockets as {agent.heshe} walks away.
            ]],
            OPT_NEGOTIATION = "Convince {agent} it wasn't you",
            OPT_FIGHT = "Attack {agent}",
            DIALOG_FIGHT = [[
                player:
                    !angry
                    Agh, forget it.
                    Faster just to kill you anyway.
                    !fight
            ]],
            DIALOG_NEGOTIATE = [[
                player:
                    !shrug
                    Look, bud, I get that you have a job to do, but so do I.
                    If I'm late, then the whole dig falls behind. So unless you got proof...
                agent:
                    !dubious
                    Hrm. 
            ]],
            DIALOG_LOST_NEGOTIATION = [[
                player: 
                    My work is very important. I set the explosives and cater the lunches.
                agent:
                    !angry_threaten
                    I don't think so. In fact, I'm not even sure you're a laborer at all.
                player:
                    Ah, well. Your funeral. 
                    !fight
            ]],
            DIALOG_WON_NEGOTIATION = [[
                player:
                    !thumb
                    If you want to hold me up, take it up with the boss. But I'm due to set off the charges in twenty minutes and—
                agent:
                    !palm
                    All right, all right. Get to your workstation then.
                    !angry_accuse
                    But I know your face, laborer! If I smell even a whiff of that union-baking stink about you...
                player:
                    !greeting
                    Can't talk, got a punchcard to fill. Industry don't wait for nobody!
                    !exit
                * The Baron splutters like a faulty spigot but otherwise doesn't stop you. That was closer than you'd like. 
            ]],
            DIALOG_WON_BATTLE = [[
                {dead ? 
                    * You beat feet in case {agent} has friends lurking about.    
                }

                {not dead?
                    agent:
                        !injured
                    * You leave {agent} to tend to {agent.hisher} wounds.
                }
            
                
            ]]
        }
        :SetLooping()
        :Fn(function(cxt) 
                if cxt:FirstLoop() then
                    cxt.quest:Complete("spark_intervention")
                    cxt.enc.scratch.response_team = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location, true)
                    cxt.encounter:SetPrimaryCast(cxt.enc.scratch.response_team[1])
                    cxt:Dialog("DIALOG_INTRO")
                end
                
                local bribe_cost = cxt.quest:GetRank() * 50
                
                cxt:Opt("OPT_PAY_OFF")
                    :DeliverMoney( bribe_cost )
                    :Dialog("DIALOG_BRIBED")
                    :Travel()

                cxt:Opt("OPT_NEGOTIATION")
                    :Negotiation{
                        
                        on_fail = function(cxt) 
                            cxt:Dialog("DIALOG_LOST_NEGOTIATION")
                            cxt:Opt("OPT_DEFEND")
                                :Battle{
                                    noncombatants = {cxt.quest:GetCastMember("giver")},
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function(cxt)
                                        cxt:Dialog("DIALOG_WON_BATTLE")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end,
                                }
                        end,

                        on_success = function(cxt) 
                            cxt:Dialog("DIALOG_WON_NEGOTIATION")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }   

                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{
                        noncombatants = {cxt.quest:GetCastMember("giver")},
                        on_win = function(cxt)
                            cxt:Dialog("DIALOG_WON_BATTLE")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }

            end)

-------------------------------------------------------------------------------------------------------------------

function DoCleanup(cxt)
    local to_move = {}
    for k,agent in cxt.location:Agents() do
        if not agent:IsInPlayerParty() then
            table.insert(to_move, agent)
        end
    end
    for k, agent in pairs(to_move) do
        agent:MoveToLimbo()
    end
end

QDEF:AddConvo("attend_rally")
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("rally_location") then
                return "STATE_RALLY"
            end
        end)

    :State("STATE_RALLY")
        :Loc{
            DIALOG_INTRO = [[
                * You find the corner {giver} told you about. An anchored trolley stands in place of a podium, and Rise volunteers hand out pamphlets with one hand while clenching makeshift signs with the other. 
                * A couple of Barons are standing guard, and one of them snatches a pamphlet and sneers at its pictograph contents.
                * When {speaker} appears {speaker.heshe} silences the onlookers with a single upraised palm before stepping up onto the trolley.
                speaker:
                    !right
                    !salute
                    Hail, laborers! Workers! Agents of <i>making</i>. 'Cause that's what we are, if you can believe it.
                    The Barons might beat the drum—might issue the scrips when they feel like it, and the demerits more often than that.
                    !eureka 
                    But make no mistake. <i>We</i> are the ones making things happen. And that's why we hold all the power.
                    !exit

                * And that's only how it starts. Some of the audience starts eyeing the crowd as if planning their escape, and you can hardly blame 'em.
                * Talk like this on the derrick would only earn the crowd a face full of mustard. 
                * But {speaker}'s a good speaker. And more stick around than not, while the Barons watch on.  
            ]]
        }
        :Fn(function(cxt) 
                cxt.quest:AssignCastMember("speaker")
                cxt.quest:GetCastMember("speaker"):MoveToLocation(cxt.location)
                for k,agent in ipairs(cxt.quest.param.attendees) do
                    if not agent:IsRetired() then
                        agent:MoveToLocation(cxt.location)
                    end
                end

                if cxt.quest:IsActive("convince_target") then
                    cxt.quest:Cancel("convince_target")
                end
                cxt:Dialog("DIALOG_INTRO")
                
                local possible_events = { "STATE_PROVACATEUR", "STATE_HECKLER", "STATE_BUST_UP" }
                local event = RALLY_EVENT_OVERRIDE or table.arraypick(possible_events)
                cxt:GoTo(event)

            end)



    :State("STATE_PROVACATEUR")
        :Loc{
            DIALOG_INTRO = [[
                * You scan the crowd looking for mischief. Sure enough, a group of laborers, at first scattered around the outside of the crowd, begin to converge in the middle.
                * One of them waits for a nod from one of {agent.hisher} buddies, then cups {agent.hisher} hands around {agent.hisher} mouth and bleats.
                agent:
                    !right
                    !eureka
                    Yeah! Yeah! Death to the Barons! Down with the bosses!
                * {agent.HeShe} looks around after {agent.hisher} outburst and sees {agent.heshe}'s failed to have much of an impact.
                agent:
                    !angry_threaten
                    We should take things into our own hands! Rah! Rah!
                * The Barons don't step in. In fact, they're not even watching the loudmouth—almost certainly a plant.
                * Seems like the goon squad has arrived, and they're here to stir up trouble. 
            ]],

            OPT_DEESCALATE = "Try to de-escalate",
            OPT_LET_IT_HAPPEN = "Watch how things play out",


            DIALOG_LET_IT_HAPPEN = [[
                * Left uninterrupted, the goon continues to rant. Eventually {agent.heshe} picks up a crowd, and the outrage builds momentum.
                * Folks get more and more agitated, and those who call for calm are quickly fingered as Baron spies. It's not long before the first punch is thrown.
                * The Barons wait only one more second, exchanging dry smirks with each other, before charging the crowd. The dull flash of their dark batons spark in the air above protestors' heads. 
            ]],
            DIALOG_DEESCALATE_START = [[
                player:
                    !left
                    !dubious
                    A riot would be the first thing the Barons want, ain't it?
                agent:
                    !surprised
                    W-whaddya mean? 
                    !angry
                    We're gonna <i>stomp</i> the Barons!
                player:
                    !crossed
                    Not if we give them an excuse to stomp us first.
                agent:
                    !angry_accuse
                    You better watch what you're insinuating, mister! Or I'll insinuate my foot in your face. 
            ]],
            DIALOG_DEESCALATION_FAIL = [[
                player:
                    !overthere
                    Besides, the Barons are too strong to take down here.
                * {agent} gets a mischievous look in {agent.hisher} eye.
                agent:
                    !cruel
                    Hah! Maybe <i>you're</i> the goon, trying to make us believe we're weak! 
                player:
                    !angry_shrug
                    No, that's—
                * But the damage is done. The crowd takes your rhetoric as a challenge, ready to thrash and flail like a wounded yote caught in a trap. The riot starts, and the Barons close in. 
                * You catch the goon disappearing through the crowd with a smirk, and {speaker} shouting for calm from {speaker.hisher} place on the trolly.
            ]],

            DIALOG_DEESCALATION_SUCCESS = [[
                player:
                    !overthere
                    What about {speaker}, huh? What does {speaker} think we should do?
                * A few voices in the crowd shout in agreement, and all heads turn back towards the trolly. 
                speaker:
                    !right
                    !crossed
                    Thanks for asking, {player}. I think we should deal with the goon squad in our midst.
                    But I trust your discretion on that.
                * The crowd turns treacherously on the goon, who suddenly realizes the fire {agent.heshe} stoked now needs new fuel to burn.
                agent:
                    !right
                    !scared
                    Alright, alright! I'm just a stooge! The Spark Barons are just looking for an excuse to—
                    
                * Speak of the devil: you catch the glint of a crossbow bolt as a Spark Baron takes aim.
                agent:
                    !exit
                * You shove {agent} out of the way before {agent.heshe} can catch a bead.
                player:
                    !injured
                    Son of a—!
                agent:
                    !flinch
                    W-what?!
                player:
                    !angry    
                    Get out of here! Now!
                    Before I regret saving your grimy hide.
                agent:
                    !exit
                * The Baron sniper glares at you, the weapon already concealed back into the folds of their uniform. {agent} slinks out of the crowd.
                * A few of the Rise gather around {speaker} and whisk {speaker.himher} out of harm's way before things can get out of hand, but the Spark Barons' open aggression played its part: the crowd disperses, quickly, with Rise pamphlets clenched tightly in laborer fists. 
            ]]
        }
        :Fn(function(cxt) 
                
                local prov = AgentUtil.GetFreeAgent("HEAVY_LABORER", function(agent) return agent:GetLocation() ~= cxt.location and table.arrayfind(cxt.quest.param.detractors) == nil and not AgentUtil.IsSympathizer(agent, "RISE") end)
                prov:MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(prov)
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_DEESCALATE")
                    :Dialog("DIALOG_DEESCALATE_START")
                    :Negotiation{   
                            helpers = {"speaker"},
                            on_fail = function(cxt) 
                                cxt:Dialog("DIALOG_DEESCALATION_FAIL")
                                cxt:GoTo("STATE_RIOT")
                            end,
                            on_success = function(cxt) 
                                cxt:Dialog("DIALOG_DEESCALATION_SUCCESS")
                                cxt:GoTo("STATE_CONCLUSION")
                            end,
                        }
                cxt:Opt("OPT_LET_IT_HAPPEN")
                    :Dialog("DIALOG_LET_IT_HAPPEN")
                    :Fn(function(cxt) 
                            cxt:GoTo("STATE_RIOT")
                        end)


            end)

    :State("STATE_HECKLER")
        :Loc{
            DIALOG_INTRO = [[
                * But not everyone is persuaded by the speech.
                agent:
                    !right
                    !handwave
                    Pfft. Yeah right. Can you believe this load?
                    Only reason {speaker.heshe} gets to yap at all is because the Barons let {speaker.himher}.
                * {agent.HeShe} glances around at the gathered few, a smirk like old cheese curled across {agent.hisher} face.
                agent:
                    I heard {speaker.hisher} folks were some of the laborers shipped to the derricks the last time they tried to organize.
                    !cruel
                    Guess {speaker.heshe} didn't learn their lesson, huh? 
                * The heckler's voice picks up at the end, but {speaker} doesn't miss a beat. Even a few of the Barons tilt their head as if ready to nod in agreement. 
                * You step up behind the heckler and clear your throat, loud enough to rattle {agent.hisher} bones.
                player:
                    !left
                    !angry
                agent:
                    !angry
                    You got a problem, pal?
                player: 
                    Only that I need you to shut up before I concave your face. 
                    !angry_shrug
                    Think you can do that for me, <i>pal</i>?
            ]],
            DIALOG_HECKLER_FAIL = [[
                player:
                    You probably never believed in anything in your life, huh?
                * The heckler's face clouds over.
                agent:
                    !spit
                    You lot're just as bad as the Barons. All high-and-mighty without valuing our lives anymore than they do.
                    I don't need to take orders from you. Heck, I might even get extra scrip for taking you down!
                * {agent.HeShe} attacks, and the crowd scatters around you.
            ]],
            DIALOG_HECKLER_SURVIVED = [[
                * You release {agent} from your grip on {agent.hisher} torn collar, and the crowd hisses in disappointment, hoping for more of a brawl.
                player:
                    Go soak your mug. You're interrupting the lady on deck.
                agent:
                    !injured
                    F-fine. Stupid speech, anyway. 
                * The heckler wipes the blood from {agent.hisher} lip and slinks through the crowd to a few whistles and claps.
                    !exit
                speaker:
                    !right
                    !placate
                    All right, everyone, settle down—this is meant to be a peaceful demonstration. Don't forget the Barons are watching.
                    !exit 
                * The Barons at the edge of the crowd narrow their eyes, whether they know it or not, reminding the crowd of its common enemy.
                * The crowd settles down and {speaker} is able to finish {speaker.hisher} speech.
            ]],
            DIALOG_KILLED_HECKLER = [[
                agent:
                    !exit
                * There's a moment of silence as {agent} breathes {agent.hisher} last shuddering breath. Then the moment is over, and the space from your brawl is filled in again with shouting protestors.
                * The Barons waste no time in containing the crowd, their batons flailing.
                * A few of the Rise gather around {speaker} and whisk {speaker.himher} out of harm's way before things can get out of hand.
            ]],
            DIALOG_HECKLER_SUCCESS = [[
                player:
                    !angry
                    You don't like what you hear? Leave. 
                agent:
                    !angry
                    Oh, I'm leaving, all right. 
                    !throatcut
                    I don't wanna be here when the Barons decide to deal with you the way they did with the <i>last</i> uprising. 
                    !exit
                player:
                    !sigh
                    About a decade too late for <i>that</i>.
                * The heckler's dealt with, and nobody even knew there was trouble. {speaker} finishes {speaker.hisher} speech, and even earns {speaker.himher}self some scattered applause.
                * Tough crowd. 
            ]],
            OPT_SHUT_DOWN_HECKLER = "Shut down the heckler",
        }
        :Fn(function(cxt) 
                local heckler = AgentUtil.GetFreeAgent("HEAVY_LABORER", function(agent) return agent:GetLocation() ~= cxt.location and table.arrayfind(cxt.quest.param.detractors) == nil and not AgentUtil.IsSympathizer(agent, "RISE") end)
                heckler:MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(heckler)

                cxt:Dialog("DIALOG_INTRO")
                
                cxt:Opt("OPT_SHUT_DOWN_HECKLER")
                    :Negotiation{
                        helpers = {"speaker"},
                        on_fail = function(cxt) 
                            cxt:Dialog("DIALOG_HECKLER_FAIL")
                            cxt:Opt("OPT_FIGHT")
                                :Battle{
                                    noncombatants = {cxt.quest:GetCastMember("speaker")},
                                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_BYSTANDERS,
                                    on_win = function(cxt) 
                                        if heckler:IsDead() then
                                            cxt:Dialog("DIALOG_KILLED_HECKLER")
                                            cxt:GoTo("STATE_RIOT")
                                        else
                                            cxt:Dialog("DIALOG_HECKLER_SURVIVED")
                                            cxt:GoTo("STATE_CONCLUSION")
                                        end
                                    end,
                        }
                        end,
                        on_success = function(cxt) 
                            cxt:Dialog("DIALOG_HECKLER_SUCCESS")
                            cxt:GoTo("STATE_CONCLUSION")
                        end,
                    }

            end)

    :State("STATE_BUST_UP")
        :Loc{
            DIALOG_INTRO = [[
                * But just before {speaker} can build up steam, a goon in Spark Baron colors approaches the podium.
                agent:
                    !right
                    !point
                    Alright, laborer. That's enough. Your time is up.
                speaker:
                    !left
                    !surprise
                    Wha—we've got a right to be here! 
                agent:
                    !shrug
                    New policy. Public gatherings all gotta disperse within twenty minutes "unless otherwise approved in triplicate." 
                speaker:
                    !angry    
                    Since when? Step off, Baron, or we'll consider this an infringement of our rights.
                agent:
                    !angry
                    What rights? You're laborers! Ain't that what you were just complaining about?
                    Seems you made the mistake of foolin' yourself into thinking anyone cares.
                    !angry_accuse
                    Now get down from that cart before I drag you off it myself.
                speaker:
                    !fight
                    Get off me!
                * You're pretty sure this is exactly why {speaker} wanted you here in the first place.
                player:
                    !left
                    !crossed
                    I think you wanna step the Hesh back and think this over a second. 
            ]],
            OPT_NEGOTIATION = "Convince {agent} to let {speaker} finish",
            OPT_FIGHT = "Resort to violence",
            DIALOG_NEGOTIATE = [[
                player:
                    !angry_shrug
                    You gotta admit, this is a bad look. 
            ]],
            DIALOG_BARONS_LEAVE = [[
                player:
                    !overthere
                    The folks want to hear {speaker.hisher} talk. They've gotten... mighty excited to hear {speaker.hisher} talk, too. 
                    !dubious
                    Look around you. Is this a crowd you want to disappoint?
                * For the first time the Spark Baron seems to consider the crowd—worked up and irritable, ready to turn on their rage like a hot faucet.
                agent:
                    !cagey
                    Ah. Uh.
                    Fine. Finish up your stupid speech. It's all a bunch of hot air, anyway.
                    !angry_accuse
                    But I'm ready to issue demerits to the first sad soul who steps outta line, capiche?
                    !exit
                * Now that you've proven yourself useful, {speaker} returns to {speaker.hisher} audience, and finishes with flair.
            ]],

            DIALOG_BARONS_ATTACK_YOU = [[
                player:
                    !hips
                    You wanna mess with {speaker}, you gotta get through me.
                agent:
                    !angry
                    Yeah? Well, if you insist.
                    !fight
            ]],

            DIALOG_WON_FIGHT = [[
                agent:
                    !exit
                * The dust settles and it's only a beat before the crowd disperses, a mad frenzy to get away from the scene of the crime.
                * Pamphlets lay discarded, trampled underfoot. But you think folks got the message all the same: the Spark Barons aren't so tough after all.
                speaker:
                    !right
                    !burp
                    That could've gone better.
                player:
                    !thumb
                    Whaddya mean? I won!
                speaker:
                    At the cost of any future rallies. Next one will have to be done in secret.
                    !disappoint 
                    But I guess that was always gonna be true eventually.
                    !exit
            ]]
        }
        :SetLooping()
        :Fn(function(cxt) 
                
                if cxt:FirstLoop() then
                    local team = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location, true)
                    cxt.encounter:SetPrimaryCast(team[1])
                    cxt:Dialog("DIALOG_INTRO")
                end

                cxt:Opt("OPT_NEGOTIATION")
                    :Dialog("DIALOG_NEGOTIATE")
                    :Negotiation{
                            helpers = {"speaker"},
                            on_success = function() 
                                cxt:Dialog("DIALOG_BARONS_LEAVE")
                                cxt:GoTo("STATE_CONCLUSION")
                            end,
                            on_fail = function() 
                                cxt:Dialog("DIALOG_BARONS_ATTACK_YOU")
                                cxt:Opt("OPT_DEFEND")
                                    :Battle{
                                        noncombatants = {cxt.quest:GetCastMember("speaker")},
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        on_win = function(cxt) 
                                            cxt.quest.param.poor_performance = true
                                            cxt.quest.param.went_well = false
                                            cxt:Dialog("DIALOG_WON_FIGHT")
                                            cxt.quest:Complete("attend_rally")
                                            DoCleanup(cxt)
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end,
                                    }
                            end,

                        }
                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        noncombatants = {cxt.quest:GetCastMember("speaker")},
                        on_win = function(cxt) 
                            cxt.quest.param.poor_performance = true
                            cxt.quest.param.went_well = false
                            cxt:Dialog("DIALOG_WON_FIGHT")
                            cxt.quest:Complete("attend_rally")
                            DoCleanup(cxt)
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,

                    }

            end)

    :State("STATE_RIOT")
        :Loc{
            DIALOG_START_RIOT = [[
                * A Baron makes a bee-line toward you. You're pretty sure you're being specially targeted.
                agent:
                    !right
                    Take that one out. {player.HeShe}'s trouble.
            ]],
            DIALOG_WON_FIGHT = [[
                speaker:
                    !right
                    Damn it, I thought we could get through one rally without that happening.
                    We have to come up with a better way to get through to people.
                    !exit
            ]]
        }
        :Fn(function(cxt) 
                local team = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location, true)
                cxt.encounter:SetPrimaryCast(team[1])
                cxt:Dialog("DIALOG_START_RIOT")
                cxt:DoBattle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    noncombatants = {cxt.quest:GetCastMember("speaker")},
                    on_win = function(cxt) 
                        cxt.quest.param.poor_performance = true
                        cxt.quest.param.went_well = false
                        cxt:Dialog("DIALOG_WON_FIGHT")
                        cxt.quest:Complete("attend_rally")
                        DoCleanup(cxt)
                        StateGraphUtil.AddLeaveLocation(cxt)

                    end,
                }
            end)
    :State("STATE_CONCLUSION")
        :Loc{
            DIALOG_INTRO = [[
                speaker:
                    !right
                    That went well!
                    I think we're really reaching the people.
                    Thanks, {player}.
                    !exit
                * You vacate the area, in case the Barons decide to come back to pick off the stragglers.
            ]]
        }
        :Fn(function(cxt)
                cxt.quest.param.went_well = true
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("attend_rally")
                DoCleanup(cxt)
                StateGraphUtil.AddLeaveLocation(cxt)
            end)





QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Tell {giver} how the rally went",
        DIALOG_GOOD = [[
            player:
                The rally is over.
            agent:
                Good work, {player}. This one really reached people, I can tell.
        ]],
        DIALOG_BAD = [[
            player:
                The rally is over.
            agent:
                That's one word for it.
                You were supposed to keep things calm.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog(cxt.quest.param.went_well and "DIALOG_GOOD" or "DIALOG_BAD")
            :CompleteQuest()
    end)


-------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                We're planning a recruitment rally in a worker's quarters later today.
                !angry_shrug
                $angryReluctant
                But the Barons caught wind, and now people are too scared to attend. 
                I need you to go drum up attendance.
            player:
                And you aren't worried the Barons might try to interfere?
            agent:
                !dubious
                $neutralDubious
                Of course I am. That's why I'm asking you.
                You're going to provide protection. 
            player:
                !crossed
                You realize my skills are of the covert variety, yes?
                This isn't exactly the kind of work I'm best suited for.
            agent:
                Fine. But it's the work that needs doing. 
                And the more even-handed you can be, the better. 

        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            player:
                !salute
                $miscMischievous
                As we used to say in the Aerostat—may my success fly higher than your expectations.
            agent:
                $neutralWhatever
                My expectations are currently set pretty low. Prove me otherwise.
                !give 
                Here's a list of folks I want to see in attendance. Should be straightforward. 
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


