local negotiation_defs = require "negotiation/negotiation_defs"


local CORPSE_LOOT = {"lumin_grenade", "combat_drone"}
local EXPLORE_STATE_OVERRIDE 
--to force a particular order, uncomment this line and rearrange the states as you please.
--EXPLORE_STATE_OVERRIDE = {"STATE_MEET_HESH", "STATE_MEET_BANDITS", "STATE_FIND_TREASURE", "STATE_MEET_MONSTERS"}

Content.AddNegotiationModifier( "discover_caves",
{
    name = "Covert Knowledge",
    desc = "When destroyed, discover the whereabouts of the bandit hideout.",

    max_resolve = 15,
    modifier_type = MODIFIER_TYPE.BOUNTY,

    sound = "event:/sfx/battle/cards/neg/create_argument/discovers_cave",

    loc_strings =
    {
        DIALOG =[[Right:
Hey, you tricked me!]],
    },

    OnInit = function( self )
        if self.engine then
            self.max_resolve = 4 + 2 * self.engine:GetDifficulty()
            self.resolve = self.max_resolve
        end
    end,

    OnBounty = function( self )
        self.engine.start_params.quest.param.clues_found = true
    end,

    OnRemoveWidget = function( self, widget, panel )
        panel.encounter:Dialog( self.def:GetLocalizedString( "DIALOG" ))
        panel:Delay( 1.0 )
    end,
})

---------------------------------------------------------------------------
--

local QDEF = QuestDef.Define{
        title = "Tunnel Rats",
        desc = "Wipe out a bandit hideout deep in the Partok caverns. The Admiralty wants to send a message, so don't leave any survivors.",
        icon = engine.asset.Texture("icons/quests/handler_admiralty_clear_bandit_cave.tex"),
        rank = {1, 5},
        qtype = QTYPE.CONTRACT,
        focus = QUEST_FOCUS.COMBAT,
        faction = "ADMIRALTY",
        precondition = function(quest) 
            return Util.IsPlayerWorkingWithFaction("ADMIRALTY")
        end,
    }

    :AddOpinionEvents{
        did_not_wipe_out_bandits =  
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Did not wipe out Spree when given the chance",
        },
    }

    :AddLocationDefs{
        SPREE_RAT_HOLE = {
            name = "Partok Caves",
            desc = "This is a hideout that the spree uses as a staging-ground for attacks on the Admiralty.",
            icon = engine.asset.Texture("UI/location_cave.tex"),
            plax = "EXT_Cave_Murderbay_1",
            indoors = false,
            map_tags = {"cave"},
        },

        SPREE_RAT_HOLE_INTERIOR = 
        {
            name = "Cave interior",
            plax = "INT_Cave_MurderBay_1",
            indoors = true,
            map_tags = {"cave"},
        },
        SPREE_RAT_HOLE_INTERIOR_DEEP = 
        {
            name = "Deep Cave interior",
            plax = "INT_CaveDeep_MurderBay_1",
            indoors = true,
            map_tags = {"cave"},
        }
    }


    :AddObjective{
        id = "wipe_out_hideout",
        title = "Wipe out {the_cave#location}",
        desc = "A small group of Spree are holed up in a cave in the Partok cavern system. {handler} wants these bandits wiped out so the Cult of Hesh can move in.",
        mark = { "the_cave" },
        state = QSTATUS.ACTIVE,

        on_activate = function( quest)
            quest:SpawnTempLocation("SPREE_RAT_HOLE", "the_cave" )
            quest.param.cave_loc = quest:SpawnTempLocation("SPREE_RAT_HOLE_INTERIOR")
            quest.param.cave_loc:SetFlags( LOCFLAGS.HIDDEN )
            quest.param.cave_loc_deep = quest:SpawnTempLocation("SPREE_RAT_HOLE_INTERIOR_DEEP")
            quest.param.cave_loc_deep:SetFlags( LOCFLAGS.HIDDEN )
        end,
        
        on_complete = function(quest)
            quest:Activate("return_for_reward")
        end,
    }

    :AddCast{
        cast_id = "the_cave",
        when = QWHEN.MANUAL,
        no_validation = true,
    }

    :AddCast{
        cast_id = "bathroom_bandit",
        when = QWHEN.MANUAL,
        no_validation = true,
        no_cleanup = true,
    }

    :AddCast{
        cast_id = "hesh_dude",
        when = QWHEN.MANUAL,
        no_validation = true,
        no_cleanup = true,
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Get reward from {handler}",
        desc = [[{handler} has a reward for you.]],
        ready_for_completion = true,
        mark = {"handler"},
        terminal = true
    }
    
    :AddObjective{
        id = "cover_for_bandits",
        title = "Convince {handler} the bandits are dead",
        desc = [[You let some Spree members live back in the cave. You're going to have to convince {handler} that didn't happen if you want credit for this job.]],
        mark = {"handler"},
        terminal = true
    }

    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_ADMIRALTY_CONTACT",
        provider = true,
    }


BATHROOM_FIGHT_PARAMS = {
    flags = BATTLE_FLAGS.SELF_DEFENCE,
    on_start_battle = function(battle)
        for _, fighter in pairs( battle:GetTeam(TEAM.RED):GetAllFighters()) do
            fighter:AddCondition("WANTED_DEAD")
        end
    end,

    on_win = function(cxt) 
        if cxt:GetAgent():IsDead() then
            cxt:Dialog("DIALOG_KILLED_BANDIT")
        else
            cxt:Dialog("DIALOG_BANDIT_LIVED")
            cxt:Dialog("DIALOG_GET_INFO")
            cxt.quest:GetCastMember("bathroom_bandit"):MoveToLimbo()
            cxt.quest.param.knows_way = true
        end
        cxt:GoTo("STATE_CAVE_CONTINUE")
    end,
}

local function CheckClues(cxt)
    if cxt.quest.param.clues_found then
        cxt:Dialog("DIALOG_FOUND_CAVE_LOCATION")
        cxt.quest.param.knows_way = true
    else
        cxt:Dialog("DIALOG_NO_FOUND_CAVE_LOCATION")    
    end
end

local function DoExplore(cxt)

    local pick_from = {}
    cxt.quest.param.searches_left = cxt.quest.param.searches_left - 1
    cxt.quest.param.times_searched = cxt.quest.param.times_searched + 1
    cxt.quest.param.done_states = cxt.quest.param.done_states or {}

    if EXPLORE_STATE_OVERRIDE then
        for k, id in ipairs(EXPLORE_STATE_OVERRIDE) do
            if not cxt.quest.param.done_states[id] then
                table.insert(pick_from, id)
                break
            end
        end
    else
        local states = {"STATE_MEET_BANDITS", "STATE_FIND_TREASURE", "STATE_MEET_MONSTERS"}
        for k,v in ipairs(states) do
            if not cxt.quest.param.done_states[v] then
                table.insert(pick_from, v)
            end
        end            
    end

    if cxt.quest.param.met_hesh == nil then
        cxt.quest.param.met_hesh = true
        cxt:GoTo("STATE_MEET_HESH")

    elseif #pick_from > 0 then
        local state = table.arraypick(pick_from)
        cxt.quest.param.done_states[state] = true

        cxt:GoTo(state)
    else
        cxt:GoTo("STATE_HIDEOUT")
    end
    
end

local function EnterCave(cxt) 
    if cxt.location ~= cxt.quest.param.cave_loc then
        cxt.encounter:DoLocationTransition( cxt.quest.param.cave_loc )
        cxt.encounter:PresentAgent(nil, SIDE.LEFT)
        cxt.encounter:PresentAgent(nil, SIDE.RIGHT)
    end
end

local function EnterCaveDeep(cxt) 
    if cxt.location ~= cxt.quest.param.cave_loc_deep then
        cxt.encounter:DoLocationTransition( cxt.quest.param.cave_loc_deep )
        cxt.encounter:PresentAgent(nil, SIDE.LEFT)
        cxt.encounter:PresentAgent(nil, SIDE.RIGHT)
    end
end


QDEF:AddConvo("wipe_out_hideout")
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("the_cave") then
                return "STATE_CAVE_ENTRANCE"
            end
        end)

    :State("STATE_CAVE_ENTRANCE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * {handler} marked the cave entrance on your map, but the wild countryside has you double-checking the coordinates.
                player:
                    !dubious
                * If the caves are as deep as you've heard, then the bandits might be better hidden than maggots in a cheese rind.
                * Fortunately for you, luck strikes—a bandit emerges from the bush, clearly returned from a brief break to nature's privvy. 
            ]],
            DIALOG_INTRO_2 = [[
                bathroom_bandit:
                    !right
                    !placate
                    $miscEmbarrassed
                    Oh, I was just—
                    $neutralTakenAback
                    Wait. Who are you?
                    !dubious
                * {bathroom_bandit}'s quick eye snags on your Admiralty graft.
                bathroom_bandit:
                    !angry
                    $neutralTakenAback
                    You lost, sarge? Only switches out <i>here</i> are the ones swinging from trees. 
                    
            ]],
            DIALOG_INTRO_2_FRIEND = [[
                bathroom_bandit:
                    !right
                    !placate
                    $miscEmbarrassed
                    Oh, I was just—
                    $neutralTakenAback
                    Wait. {player}? What are you doing here?
                    !cagey
                    $miscHushedUrgent
                    You should move on, friend. This place is full of folks who'd sooner kill a hunter than speak to them.
                    Let alone one working for the switches like you are. I don't want you to get hurt, even if you are workin' for the boss's worst enemy.
            ]],
            DIALOG_INTRO_2_UNFRIEND = [[
                bathroom_bandit:
                    !right
                    !placate
                    $miscEmbarrassed
                    Oh, I was just—
                    $neutralTakenAback
                    Wait. I know you! You're that damn hunter working with the switches.
                    !fight
                    $angryHostile
                    You gotta lotta nerve coming here!
            ]],

            OPT_CONVINCE_RUN = "Let {bathroom_bandit} go in exchange for information",
            OPT_ATTACK = "Attack {bathroom_bandit}",
            DIALOG_START_CONVINCE = [[
                * {handler} said to kill all the Spree, but there's no harm done if you make the job easier by letting one runner go, surely.
                player:
                    !angry
                    $angryHostile
                    You know why you call 'em switches, right? 'Cause they have the authority to beat you bloodless. 
                    !cruel
                    $angryCruel
                    But that doesn't have to be you, friend. Not if you answer a few questions. 
                ** Remove <b>Covert Knowledge</b> to determine the location of the bandit hideout.
            ]],
            TT_CAVE_CLUES = "{agent} won't give up the location of the Spree hideout willingly, but if you can keep {agent.himher} talking long enough, you might be able to piece it together.",
            DIALOG_KILLED_BANDIT = [[
                bathroom_bandit:
                    !exit
                * {bathroom_bandit} is dead, which brings you <i>some</i> satisfaction, but it's hard to convey your smugness to a corpse. 
                * Still, no sense complaining now: you have a cave to explore.
                player:
                    !exit
            ]],
            DIALOG_BANDIT_LIVED = [[
                * You stay your blade, and {bathroom_bandit} almost quivers with relief.
                bathroom_bandit:
                    !injured
                    $scaredFearful
                    J-just lemme go, all right? And I'll tell you where the camp is. 
                    Otherwise you'll never find it, not even if you hunt for days. 
                    Whaddya say? Let me live?
                player:
                    !shrug
                    $angryCruel
                    Sounds like a deal.
                *** {bathroom_bandit} agrees to tell you how to get into the cave.
            ]],
            DIALOG_GET_INFO = [[
                bathroom_bandit:
                    !exit
                * You enter the cave system.
                player:
                    !exit
            ]],
            DIALOG_LOST_NEGOTIATION = [[
                bathroom_bandit:
                    !fight
                    $angryHostile
                    Forget it, switch-lover. I ain't no traitor. And you don't know what you're up against.
                    Even if you take me down, Nadan would kill me twice to your once if I talked.
                player:
                    $miscMocking
                    That's just overkill.
                *** You fail to convince {bathroom_bandit} to talk.
            ]],
            DIALOG_WON_NEGOTIATION = [[
                player: 
                    !threaten
                    $angryHostile
                    Make my life easier—or I'll make <i>yours</i> shorter. 
            ]],
            DIALOG_FOUND_CAVE_LOCATION = [[
                bathroom_bandit:
                    !flinch
                    $scaredTakenAback
                    Hey, look—I don't know much. Nadan keeps us all in the dark. 
                    But if all you want is directions I can give you that, at least.
                    !suspicious
                * The bandit scratches a map in the dirt, watching you for sudden movements.
                * After {bathroom_bandit.heshe}'s done, you dismiss {bathroom_bandit.himher} with a brief nod, and {bathroom_bandit.heshe} makes for the bushes. When you gotta go, you gotta go. 
                bathroom_bandit:
                    !exit
                player:
                    !neutral_notepad
                * You make careful note of the map.

                *** The bandit tells you how to navigate the caverns, and you spare {bathroom_bandit.himher}.
            ]],
            DIALOG_NO_FOUND_CAVE_LOCATION = [[
                bathroom_bandit:
                    !flinch
                    $scaredTakenAback
                    I—I'm just going to pretend I never saw nothing! Alright?
                    !exit
                * {bathroom_bandit} takes flight. You aren't learning anything more from {bathroom_bandit.himher}.
                * You peek ahead to the tunnels. They're worse than expected: sometimes the branches don't <i>fork</i> so much as <i>octopus</i>. 
                * Without a map to guide you, it's going to take hours to work your way through.

                *** You scared off the bandit, and have to find your own way through the caverns.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :SetLooping()
        :Fn(function(cxt) 
                if cxt:FirstLoop() then  
                    cxt.location:SetFlags( LOCFLAGS.HIDDEN )              
                    cxt:Dialog("DIALOG_INTRO")
                    ConvoUtil.SpawnNewAgent( cxt, math.random() > .5 and "BANDIT_GOON" or "BANDIT_GOON2", "bathroom_bandit" )  
                    
                    ConvoUtil.RelationshipDialog(cxt, cxt:GetAgent(), "DIALOG_INTRO_2", "DIALOG_INTRO_2_FRIEND", "DIALOG_INTRO_2_UNFRIEND")
                    cxt.quest.param.clues_found = false
                    cxt.quest.param.searches_left = math.random(2, 3)

                end
                cxt:Opt("OPT_CONVINCE_RUN")
                    :PostText("TT_CAVE_CLUES")
                    :Dialog("DIALOG_START_CONVINCE")
                    :Negotiation{
                        opponent_modifiers = { discover_caves = 1 },
                        quest = cxt.quest,
                        on_success = function()
                            cxt:Dialog("DIALOG_WON_NEGOTIATION")
                            CheckClues(cxt)
                            cxt:Dialog("DIALOG_GET_INFO")
                            cxt.quest:GetCastMember("bathroom_bandit"):MoveToLimbo()
                            cxt:GoTo("STATE_CAVE_CONTINUE")
                        end,
                        on_fail = function()
                            cxt:Dialog("DIALOG_LOST_NEGOTIATION")
                            CheckClues(cxt)
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle(BATHROOM_FIGHT_PARAMS)
                        end }

                cxt:Opt("OPT_ATTACK")
                    :Battle(BATHROOM_FIGHT_PARAMS)
                
                -- cxt:Opt("IGNORE DEBUG")
                --     :Fn(function() EnterCave(cxt) end)
                --     :GoTo("STATE_CAVE_CONTINUE")
            end)

    :State("STATE_MEET_HESH")
        :Loc{
            DIALOG_INTRO = [[
                * You turn a corner and only notice the Luminari crouched in the shadows when it's too late to do nothing but gawk. 
                player:
                    !left
                    !surprised
                    $scaredTakenAback
                    Uh
                agent:
                    !right
                    !angry_accuse
                    $angryAside
                    Quiet!
                * {agent.HeShe} holds up a blessed finger and waits in the darkness, and you can't help but follow {agent.hisher} lead.
                player:
                    !dubious
                * Finally {agent.heshe} looks you up and down.
                agent:
                    !sigh
                    $neutralResigned
                    Thought I'd heard the bandits. <i>You're</i> not one of them—I can tell that at a glance.
                    My name's {agent}. 
                
                    player:
                        !forgetful
                        $neutralDubious
                        {player}. Weird place to make acquaintance, but here we are.

                {not clues_found ? 
                        player:
                            !dubious
                            $neutralDirect
                            You don't know the way to the bandit cave, do you?
                        agent:
                            !agree
                            $neutralDirect
                            Actually, I do. I've been sent to catalog some of the ruins in these caves—and to do what I can to safeguard them until the bandits were dealt with.
                        player:
                            !thumb
                            $neutralDirect
                            That's where I come in. And immediately get lost in these damn caves.
                            I'll be <i>dealing</i> with those bandits the second I can <i>find</i> them. 
                        agent:
                            $neutralThoughtful
                            I can point you in the right direction, at least.  
                            !tired
                            $neutralResigned
                            There's only so long I can spend squatting among the mushrooms, counting old finger bones. 
                        player:
                            !happy
                            $happyJoke
                            Terrible way to meet new people.
                        agent:
                            $miscMocking
                            But here we are. 
                }
                {clues_found ?
                        player:
                            I'm here to sort out the cave's bandit infestation. 
                            Thought I'd do a little exploring first, see what I can dig up.
                        agent:
                            I've been sent to catalog some of the ruins in these caves—and do what I could to safeguard them until the bandits were dealt with.
                            !happy
                            Seems like you're fixing to get me out of this damnable hole. 
                }
                *** You meet {agent}, a member of the Cult of Hesh. 
            ]],
            DIALOG_LEAVE = [[
                player:
                    !greeting
                    $happyGreeting
                    Well, good luck with the safeguarding.
                agent:
                    $neutralThoughtful
                    Try not to make too much of a mess—likely as not I'll be the one cleaning it up.
                    !salute
                    $neutralThanks
                    Keep to the shallows, and Hesh will protect you.
                player:
                    !dubious
                    $neutralDubious
                    Err, yeah. Sure.
                agent:
                    !exit
                *** You leave {agent} behind.
            ]],
            DIALOG_FAIL_NEGOTIATION = [[
                player:
                    !shrug
                    $happyCocky
                    Come on, let's go beat up bandits. It'll be... fun?
                agent:
                    $neutralWhatever
                    I have my orders, and that's to protect the ruins. 
                    What you do with the guards is your business.
                    But I hope you at least get it over with quickly.
                *** You fail to convince {agent} to join you.
            ]],
            DIALOG_WIN_NEGOTIAITON = [[
                player:
                    !eureka
                    $happyCocky
                    You must be bored out of your mind. But now's the time to strike. 
                agent:
                    !thought
                    $neutralThoughtful
                    Hm, that's true. The ruins won't need guarding the second your work is done.
                player:
                    $neutralDirect
                    And two blades are better than one, I always say.
                agent:
                    $neutralDirect
                    You have a point. Very well.
                    !thumb
                    $neutralDirect
                    Follow me, and stick to the shadows—we don't want them fleeing further into the caves.
                *** You convince {agent} to join you. 
            ]],
            DIALOG_NEGOTITATION_START = [[
                player:
                    !left
                    $neutralJoke
                    Luminari are trained combatants, aren't they? 
                    You content to sit and let me have all the fun?
                agent:
                    !right
                    
            ]],
            OPT_NEGOTIATION = "Convince {agent} to join you against the bandits",
            OPT_LEAVE_HESH = "Leave {agent} to {agent.hisher} own devices",
        }
        :Fn(function(cxt) 
                EnterCaveDeep(cxt)
                local agent = ConvoUtil.SpawnNewAgent( cxt, "LUMINARI", "hesh_dude" )
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_NEGOTIATION")
                    :Dialog("DIALOG_NEGOTITATION_START")
                    :Negotiation{ 
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_WIN_NEGOTIAITON")
                            cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            cxt.quest.param.help = cxt:GetAgent()
                            
                            cxt:GoTo("STATE_CAVE_CONTINUE")
                        end,
                        
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_FAIL_NEGOTIATION")
                            cxt:GoTo("STATE_CAVE_CONTINUE")
                        end 
                }

                cxt:Opt("OPT_LEAVE_HESH")
                    :Dialog("DIALOG_LEAVE")
                    :Fn(function() 
                            agent:MoveToLimbo()
                        end)
                    :GoTo("STATE_CAVE_CONTINUE")
            end)
    
    :State("STATE_MEET_BANDITS")
        :Loc{
            DIALOG_INTRO = [[
                * You move quickly; the only sound you make is the soft splutter of your flickering torch.
                * But it's hard to move in the shadows when you banish them with your left hand. So the bandit finds you alone, and armed with fire.
                agent:
                    !right
                    !fight
                    $angryHostile
                    Hey, we got a live one!
                *** You're discovered by a Spree.
            ]],

            OPT_FIGHT = "Defend yourself",
            DIALOG_ON_WIN = [[
                * The danger has passed; you retrieve your torch before it flickers out.
            ]],
            DIALOG_SURVIVORS = [[
                agent:
                    !exit
                * The survivors run further into the cave system. You don't think you've seen the last of them.
            ]],
            DIALOG_NO_SURVIVORS = [[
                agent:
                    !exit
                * The nice thing is, you'll be able to find your way out again by following the trail of corpses in your wake.
                * You continue deeper into the caves.
            ]]
        }
        :Fn(function(cxt) 
                EnterCaveDeep(cxt)
                local guards = CreateCombatParty("BANDIT_PATROL", math.max( 1, cxt.quest:GetRank()-2), cxt.location)
                cxt.enc:SetPrimaryCast(guards[1])
                cxt:Dialog("DIALOG_INTRO")
                
                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        allies = { cxt.quest.param.help },
                        on_start_battle = function(battle)
                            for _, fighter in pairs( battle:GetTeam(TEAM.RED):GetAllFighters()) do
                                fighter:AddCondition("WANTED_DEAD")
                            end
                        end,
                        on_win = function(cxt) 
                            --if survivors, add them to the final fight.
                            local survivors = {}
                            for k,v in ipairs(guards) do
                                if not v:IsDead() then
                                    if v:IsSentient() then
                                        table.insert(survivors, v)
                                    end
                                    v:MoveToLimbo()
                                end
                            end
                            local hesh = cxt.quest.param.help
                            if hesh and hesh:IsDead() then
                                cxt.quest.param.help = nil
                            end
                            if #survivors > 0 then
                                cxt.quest.param.survivors = survivors
                                cxt:Dialog("DIALOG_SURVIVORS")
                            else
                                cxt:Dialog("DIALOG_NO_SURVIVORS")
                            end
                            cxt:GoTo("STATE_CAVE_CONTINUE")
                        end,
                }

            end)
    :State("STATE_MEET_MONSTERS")
        :Loc{
            DIALOG_INTRO = [[
                * You hear a strange noise in the darkness and ready your blade, hoping for bandits.
                agent:
                    !right
                    $angryHostile
                    <i>Gurrrraauggh</i>.
                player:
                    !left
                    !sigh
                    $scaredFearful
                    Aw, crap.
            ]],
            OPT_FIGHT = "Defend yourself",
            DIALOG_ON_WIN = [[
                * With that dealt with, you only hope the bandits don't put up as much of a fight. 
            ]]
        }
        :Fn(function(cxt) 
                EnterCaveDeep(cxt)
                local guards = CreateCombatParty("YOTES", cxt.quest:GetRank(), cxt.location)
                cxt.enc:SetPrimaryCast(guards[1])
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        allies = { cxt.quest.param.help },
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt) 
                            for k,v in ipairs(guards) do
                                if not v:IsDead() then
                                    v:MoveToLimbo()
                                end
                            end
                            local hesh = cxt.quest.param.help
                            if hesh and hesh:IsDead() then
                                cxt.quest.param.help = nil
                            end
                            cxt:Dialog("DIALOG_ON_WIN")
                            cxt:GoTo("STATE_CAVE_CONTINUE")
                        end,
                    }

            end)
    :State("STATE_FIND_TREASURE")
        :Loc{
            DIALOG_INTRO = [[
                * You reach a dead end—literally, because the only occupant is a corpse.
            ]],
            DIALOG_DONE_LOOT = [[
                * You pick over the remains and reposition the corpse to terrify the next person to pass through the caves.
                player:
                    !left
                    !salute
                    $neutralThanks
                    Thank you for your service.
                * With that, you turn around and head back the way you came. Time to try another route.
            ]]
        }
        :Fn(function(cxt) 
                EnterCaveDeep(cxt)
                cxt:Dialog("DIALOG_INTRO")
                local val = math.round( cxt.quest:GetRank()* 50 * (math.randomGauss(.5, 1.5)) )
                local loot = 
                {
                    money = val,
                    items = {CORPSE_LOOT[math.random(#CORPSE_LOOT)]}
                }
                cxt.enc:AwardLoot(loot, { found=true } )
                cxt:Dialog("DIALOG_DONE_LOOT")
                cxt:GoTo("STATE_CAVE_CONTINUE")
            end)

    :State("STATE_CAVE_CONTINUE")
        :Loc{
            DIALOG_INTRO = [[
                * You're so deep in the Partok Caves that you start to get a little reflective. 
                * Why are you here? What purpose do you serve? You kill some bandits, but then what? 
                * Is the meaning of life really at the end of a contract? Or is there something greater? Could you ever—
                player:
                    !left
                    !palm
                    $angryDefault
                    Gah!
                * Lost in thought, you knock your head on a stalactite. 
                * Whatever you were thinking about evaporates into the darkness as you rub your noggin. 
                player:
                    !wince
                    $miscEmbarrassed
                    Hesh damnit.
                * You shake the stars out of your eyes and keep exploring. 
            ]],
            DIALOG_INTRO_2 = [[
                player:
                    !exit
                agent:
                    !exit
                * You're back in the main cave.
            ]],
            DIALOG_NOTHING_LEFT = [[
                * Nothing in these caves left to root and discover, and you've got a job to do.
            ]],
            DIALOG_KNOWS_WAY = [[
                * You head directly to the hideout.
            ]],
            DIALOG_EXPLORE_FIRST = [[
                * You search some side passages, hunting for anything useful.
            ]],
            DIALOG_EXPLORE_FIRST_2 = [[
                * You explore some more.
            ]],
            DIALOG_SEARCH_CAVE = [[
                * You stumble around in the darkness, looking for a lead.
            ]],
            DIALOG_BLIND_EXPLORATION = [[
            ]],
            DIALOG_FOUND_IT = [[
                * You see a light up ahead, and investigate.
            ]],

            OPT_KNOWS_WAY = "Go to the hideout",
            OPT_EXPLORE_FIRST = "Explore the caves",
            OPT_SEARCH_CAVE = "Search the area",
        }
        :Fn(function(cxt) 
                EnterCave(cxt)
                cxt.quest.param.times_searched = cxt.quest.param.times_searched or 0
                cxt:Dialog(cxt.quest.param.times_searched == 0 and "DIALOG_INTRO" or "DIALOG_INTRO_2")
                if cxt.quest.param.knows_way then
                    
                    if cxt.quest.param.searches_left <= 0 then
                        cxt:Dialog("DIALOG_NOTHING_LEFT")
                        cxt:GoTo("STATE_HIDEOUT")
                    else

                        cxt:Opt("OPT_KNOWS_WAY")
                            :Dialog("DIALOG_KNOWS_WAY")
                            :GoTo("STATE_HIDEOUT")

                        cxt:Opt("OPT_EXPLORE_FIRST")
                            :Dialog(cxt.quest.param.times_searched == 0 and "DIALOG_EXPLORE_FIRST" or "DIALOG_EXPLORE_FIRST_2")
                            :Fn(function(cxt) 
                                    DoExplore(cxt)
                                end)
                    end
                else

                    cxt:Opt("OPT_SEARCH_CAVE")
                        
                        :Fn(function(cxt) 
                                
                                if cxt.quest.param.times_searched == 0 then
                                    cxt:Dialog("DIALOG_SEARCH_CAVE")
                                end

                                if cxt.quest.param.searches_left <= 0 then
                                    cxt:Dialog("DIALOG_FOUND_IT")
                                    cxt:GoTo("STATE_HIDEOUT")
                                else
                                    cxt:Dialog("DIALOG_BLIND_EXPLORATION")
                                    DoExplore(cxt)
                                end
                                
                            end)
                end
 
            end)

    :State("STATE_HIDEOUT")
        :Loc{
            DIALOG_DONE_EXPLORE = [[
                * But it's not long before the smell of sizzling meat hits you. And since stalactites and cave vermin aren't much known for their cook-outs, you follow your nose to the bandits. 
                * Unfortunately something in the air must've told 'em you were coming, too—because you round the corner and find yourself flat-footed.

            ]],
            DIALOG_FIND_SPREE = [[
                player: 
                    !left
                agent:
                    !right
                    !angry_accuse
                    $angrySeething
                    Where d'ya think <i>you're</i> headed? 
                player:
                    !scared_shrug
                    $neutralResigned
                    Ah. I <i>was</i> hoping to slink my way into a good position. Don't suppose you'd allow a do-over?
                agent:
                    $angryDefault
                    Nope.
                player:
                    !fight
                    $neutralResigned
                    Didn't think so. 
                *** You've been discovered by the Spree.
            ]],

            DIALOG_SURVIVORS_FROM_BEFORE = [[
                * You notice the survivors from your previous fight with the Spree in the back rank. You really should have dealt with them up front.
            ]],
            OPT_FINISH_THE_JOB = "Finish the job",
            OPT_LET_THEM_GO = "Let them go",
            DIALOG_SURVIVORS = [[
                agent:
                    !right
                    !scared
                * The fight is over, but in the fray you find yourself with a couple of loose ends.
                agent:
                    !scared
                    $scaredDefault
                    H-hold up! Wait a second! This don't gotta end in blood spilt—not mine, anyway.
                    !scared_shrug
                    $scaredDefault
                    You tell your boss you did the job and I'll make it worth your while. Then we'll be gone, and nobody'll ever know the difference.
                * {handler} explicitly told you to kill the bandits. But that was just to get the caves clear, right?
                *** The bandits beg for mercy.
            ]],
            DIALOG_KILL_SURVIVORS = [[
                player:
                    $neutralJoke
                    Sorry. Turns out <i>nobody</i> gives do-overs.
                agent:
                    !exit
                * You make it quick and painless, though, so it's over before anyone can show remorse.
            ]],
            
            DIALOG_LET_THEM_GO = [[
                * You lower your blade, and the bandit's face slackens with relief.
                agent:
                    $miscHushedUrgent
                    !placate
                    That's good. Yeah. Easy does it.
                    I made a promise, too, that I'd make this worth your while. So here we go.
                ]],
            DIALOG_LET_THEM_GO_2 = [[
                player:
                    !angry_accuse
                    $angryCruel
                    Don't make the mistake of thinking your debt ends here, pal. If I ever see you again...
                agent:
                    $scaredFearful
                    I know, I know. I'm as good as dead.
                player:
                    $miscMischievous
                    Nope. Even worse. You'll owe me a favor.
                agent:
                    !agree
                    $scaredTakenAback
                    Yeah, sure, pal. You need a friend, consider me there. 
                    Turns out I'm really fond of life. 
                    !exit
                * The bandit scurries deeper into the caves, taking with {agent.himher} the last of the Spree presence. 
                * You're going to have to convince {handler} this never happened.
                *** You let the bandits go. 
            ]],
            DIALOG_NO_SURVIVORS = [[
                agent:
                    !exit
                * You made quick work of the bandits. Which is nice, because nothing else about this job was handled that neatly. 
            ]],
            DIALOG_DONE = [[
                * Fortunately, you're able to find your way out by retracing your steps. You always had a good memory for mazes.
            ]],
            DIALOG_HESH_DUDE_HAPPY = [[
                hesh_dude:
                    !right
                    $happyAmazed
                    That went smoothly enough. Your services were no doubt worth the expense.
                    !interest
                    $happyThanks
                    Of course, it's best if you keep this partnership to yourself.
                player:
                    !dubious
                    $neutralDubious
                    What, the Cult doesn't like to associate with riffraff like me?
                hesh_dude:
                    !overthere
                    $neutralThanks
                    Not at all. Come into the fold and they'd be delighted to herald your salvation.
                    All creatures are welcome under the bell of Hesh. 
                player:
                    !shrug
                    $neutralWhatever
                    Enh, yeah. Not for me, I think.
                hesh_dude:
                    $neutralResigned
                    So I assumed. It was nice meeting you, {player}.
                    !exit
                * {hesh_dude} disappears into the darkness of the cave before you can even think to call {hesh_dude.himher} back. Not that you would consider converting—but it might've been nice to hear the benefits package.
                *** You say goodbye to {hesh_dude} on good terms.  
            ]],
            DIALOG_HESH_DUDE_UNHAPPY = [[
                hesh_dude:
                    !right
                    $neutralDirect
                    That wasn't what you had been asked to do. 
                    You dredge up scum in the shallows when leviathans lurk just offshore.
                player:
                    !dubious
                    $neutralDubious
                    Are you threatening me, {hesh_dude}? That was all very poetical so I can't be certain.
                    But I handle things my way. If the cult doesn't like that, then they should do their own dirty work.
                hesh_dude:
                    $neutralDirect
                    I'll be sure to pass along your assessment. 
                    Watch yourself, heretic. Hesh has a greater reach than you could imagine.
                    !exit
                * {hesh_dude} disappears into the darkness of the cave, while you shout after {hesh_dude.himher}, too late to get the final word.
                player:
                    $angryThanks
                    Yeah? Well your slimy god can keep its tentacles to itself!
                    !handwave
                    $neutralThanks
                    Aw, who needs friends, anyway.
                    !exit
                *** You say goodbye to {hesh_dude} on bad terms. 
            ]],
            OPT_EXPLORE = "Explore the cave",
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :Fn(function(cxt)
            EnterCaveDeep(cxt)
            cxt:Dialog("DIALOG_DONE_EXPLORE")
            cxt.enc:GetScreen():WaitOnLine()

            local leader = ConvoUtil.SpawnNewAgent( cxt, cxt.quest:GetRank() <= 3 and "BANDIT_GOON" or "BANDIT_RAIDER" )  
            local extras = CreateCombatBackup(leader, "BANDIT_PATROL_BACKUP", cxt.quest:GetRank())
            table.insert(extras, 1, leader)
            cxt.quest:SetParameter("spree_fighters", extras)

            cxt:Dialog("DIALOG_FIND_SPREE")
            if cxt.quest.param.survivors then
                cxt:ReassignCastMember("survivor", cxt.quest.param.survivors[1])
                cxt:Dialog("DIALOG_SURVIVORS_FROM_BEFORE")
                for k,v in ipairs(cxt.quest.param.survivors) do
                    table.insert( extras, v )
                end
            end
            local hesh_dude_here = cxt.quest:GetCastMember("hesh_dude") and cxt.quest:GetCastMember("hesh_dude"):IsInPlayerParty()
            cxt:Opt("OPT_DEFEND_SELF")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = { cxt.quest.param.help },  
                    on_start_battle = function(battle)
                        for _, fighter in pairs( battle:GetTeam(TEAM.RED):GetAllFighters()) do
                            fighter:AddCondition("WANTED_DEAD")
                        end
                    end,

                    on_win = function(cxt)      
                        local survivors = {}
                        for k,v in pairs(cxt.quest.param.spree_fighters) do
                            if not v:IsDead() and v:IsSentient() then
                                table.insert(survivors, v)
                            elseif not v:IsDead() then
                                v:MoveToLimbo()
                            end
                        end

                        if #survivors > 0  then
                            cxt:TalkTo(survivors[1])
                            cxt:Dialog("DIALOG_SURVIVORS")
                            
                            cxt:Opt("OPT_FINISH_THE_JOB")
                                :Fn(function(cxt) 
                                        cxt:Dialog("DIALOG_KILL_SURVIVORS")
                                        for k,v in ipairs(survivors) do
                                            v:Kill()
                                            AgentUtil.CommitMurder(v, survivors)
                                        end
                                        cxt.quest:Complete("wipe_out_hideout")
                                        
                                        if hesh_dude_here then
                                            cxt:Dialog("DIALOG_HESH_DUDE_HAPPY")
                                            cxt.quest:GetCastMember("hesh_dude"):Dismiss()
                                        end
                                        cxt:Dialog("DIALOG_DONE")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end)

                            cxt:Opt("OPT_LET_THEM_GO")
                                :Fn(function(cxt)
                                        cxt:Dialog("DIALOG_LET_THEM_GO")
                                        local grafts = RewardUtil.GetGrafts(cxt.quest:GetRank(), TheGame:GetGameState():GetGraftDraftDetails().count)
                                        ConvoUtil.GiveGraftChoice(cxt, grafts)
                                    end)
                                :Fn(function(cxt)
                                    cxt:Dialog("DIALOG_LET_THEM_GO_2")

                                    for k,v in ipairs(survivors) do
                                        v:OpinionEvent(OPINION.SPARED_LIFE)
                                        v:MoveToLimbo()
                                    end

                                    cxt.quest:Cancel("wipe_out_hideout")
                                    cxt.quest:Activate("cover_for_bandits")
                                    if hesh_dude_here then
                                        cxt:Dialog("DIALOG_HESH_DUDE_UNHAPPY")
                                        cxt.quest:GetCastMember("hesh_dude"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("did_not_wipe_out_bandits"))
                                        cxt.quest.param.hesh_dude_mad = true
                                        cxt.quest:GetCastMember("hesh_dude"):Dismiss()
                                    end
                                    cxt:Dialog("DIALOG_DONE")
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)
                        else
                            cxt:Dialog("DIALOG_NO_SURVIVORS")
                            cxt.quest:Complete("wipe_out_hideout")
                            
                            if hesh_dude_here then
                                cxt:Dialog("DIALOG_HESH_DUDE_HAPPY")
                                cxt.quest:GetCastMember("hesh_dude"):Dismiss()
                            end
                            cxt:Dialog("DIALOG_DONE")
                            StateGraphUtil.AddLeaveLocation(cxt)

                        end

                    end,

                }
        end)
            
    
    


QDEF:AddConvo("return_for_reward", "handler")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_INTRO = [[
            player:
                !overthere
                $neutralDefault
                Your cavern is all cleared out. Hope the researchers don't mind a few bodies in their way.
            handler:
                !handwave
                $neutralResigned
                Hey, they just asked me to make it safe. Besides, it's gotta be dirty work, rooting up the past.
            player:
                !agree
                $neutralThoughtful
                If you're happy, I'm happy.
            handler:
                !placate
                $miscRelieved
                Well, let's not go too crazy. But I'm satisfied, at least. 
            *** {handler} is satisfied that you've cleared out the caves.

        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_INTRO")
            :CompleteQuest()
    end)

QDEF:AddConvo("cover_for_bandits", "handler")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_CONVINCE = "Convince {handler} the Spree are all dead",
    }
    :Loc{
        DIALOG_INTRO = [[
            player:
                $neutralDirect
                Consider those caverns cleared.
            handler:
                $neutralDubious
                I'll consider them cleared if they're actually cleared. That's the case, right?
        ]],
        DIALOG_WON = [[
            player: 
                !crossed
                $happyCocky
                Yep. I dealt with every last bandit.
            agent:
                $happyAmazed
                That's what I like to hear. 
                !sigh
                $happyResigned
                Clearly I should've hired a hunter from the start. Way better than any grunt, military training or no.
                That's all I got for you today, {player}. Enjoy the rest of your afternoon. 
            *** You convince {handler} that the caves are cleared.
        ]],
        DIALOG_LOST = [[
            player:
                !throatcut
                $happyCocky
                They're all dead. Gutted them myself, with my bare hands. 
            agent:
                !angry
                $angryPatienceLost
                Really? 'Cause I have it on good authority that at least one bandit was seen escaping.
                !disappoint
                $angryPatienceLost
                Now, I'm gonna chalk this up to an honest mistake—those caverns are big, and you might've gotten turned around in there.
                Could happen to anyone.
                !point
                $angryPatienceLost
                But I don't pay for shoddy work. And you're gonna need to do better than that if you want into the auction. 
                Now go find someone else to disappoint. I've got work to do. 
            *** You fail to convince {handler} that the caves are cleared.
        ]],
        DIALOG_HESH_DUDE_SHOWS_UP = [[
            * Just as you're about to tell {handler} the details, {hesh_dude} saunters through the door. 
            hesh_dude:
                !right 
                !crossed
                Yes, {player}. Why don't you give the officer a <i>thorough</i> report?
        ]]
    }
    :Hub(function(cxt) 

        cxt:Opt("OPT_CONVINCE")
            :Dialog("DIALOG_INTRO")
            :Fn(function()
                local hesh = cxt.quest:GetCastMember("hesh_dude")
                if hesh and not hesh:IsDead() and cxt.quest.param.hesh_dude_mad then
                        if hesh:GetLocation() ~= cxt.location then
                            hesh:MoveToLocation(cxt.location)
                            cxt:Dialog("DIALOG_HESH_DUDE_SHOWS_UP")
                        end
                    end

                end)
            :Negotiation{
                on_success = function() 
                    cxt:Dialog("DIALOG_WON")
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                end,
                on_fail = function()
                    cxt:Dialog("DIALOG_LOST")
                    cxt.quest:Fail()
                end 
            }
    end)




QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.DECLINED )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                !dubious
                $neutralDubious
                You're going to have to get your hands dirty with something.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                !dubious
                $neutralDirect
                Remember: no survivors.
            player:
                !thumb
                Except me, right?
            handler:
                !wink
                Enh. I'll leave that up to your discretion.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                $neutralDirect
                You heard of the Partok Caverns?
            player:
                !thought
                $neutralThoughtful
                Sounds familiar. 
            handler:    
                $neutralJoke
                I think it would've left an impression, if not a gaping hole in your side. The place is <i>crawling</i> with Spree.
                !overthere
                $neutralResigned
                Normally I'd just leave them to their rat's nest but apparently there might be ruins under the caves. Old ones. From before the plague.
                I've run it up the chain of command but the Admiralty don't care—they're just as happy to let the Spree rot with the relics.
                The Heshians, on the other hand...
            player:
                !shrug
                $neutralWhatever
                What do you want <i>me</i> to do? I don't know the first thing about holy relics.
            handler:
                $miscPersuasive
                But you <i>do</i> know about killing bandits. Wipe them out so the Cult can send in their archaeologists. 
                !thought
                $neutralThoughtful
                You do the killing, they do the, you know... dusting?
            player:
                !happy
                $happyTakenAback
                Dusting?
            handler:
                !dubious
                $neutralDubious
                B-brushing? Clearing.
                !eureka
                $happyAmazed
                Digging! They're going to dig.
            player:
                !chuckle
                $happyDefault
                You got there eventually.
            *** {handler} wants you to clear out a cave of Spree bandits. 
        ]],
       
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)
