local negotiation_defs = require "negotiation/negotiation_defs"

AddOpinionEvent("COLLECTED_ADMIRALTY_TAXES", 
{
    delta = OPINION_DELTAS.DIMINISH,
    txt = "Helped the Admiralty collect their taxes",
})

AddOpinionEvent("COLLECTED_TAXES_CONSPIRACY", 
{
    delta = OPINION_DELTAS.MAJOR_GOOD,
    txt = "Entered into a conspiracy to steal from the Admiralty",
})

AddOpinionEvent("COLLECT_TAXES_GAVE_FRIEND_CLERK", 
{
    delta = OPINION_DELTAS.MAJOR_GOOD,
    txt = "Delivered Admiralty taxes to them",
})

AddOpinionEvent("COLLECTED_TAXES_TOGETHER", 
{
    delta = OPINION_DELTAS.LIKE,
    txt = "Escorted them on their tax collection run",
})

local function CleanUpFn(quest)
    if quest:GetCastMember("clerk") and quest:GetCastMember("clerk"):IsInPlayerParty() then
        quest:GetCastMember("clerk"):Dismiss()
    end
end

local QDEF = QuestDef.Define{
        title = "Collection Day",
        desc = "Escort an admiralty clerk as they collect taxes from local businesses.",
        icon = engine.asset.Texture("icons/quests/handler_admirality_collection_day.tex"),
        rank = {1, 3},
        qtype = QTYPE.CONTRACT,
        focus = QUEST_FOCUS.NEGOTIATION,
        faction = "ADMIRALTY",
        precondition = function(quest) 
            return Util.IsPlayerWorkingWithFaction("ADMIRALTY")
        end,
        
        on_fail = CleanUpFn,
        on_complete = CleanUpFn,
        on_cancel = CleanUpFn,

       
        on_init = function(quest)
            quest.param.collected_from = {}
            quest.param.num_collected = 0
            quest.param.num_attempted = 0
            quest.param.amount_collected = 0
        end,

        events = 
        {
            caravan_member_remove = function(quest, agent) 
                if agent == quest:GetCastMember("clerk") then
                    if quest:IsActive("collect") then
                        quest:Fail()
                    end
                end
            end,
        }        

    }


    :AddCast{
        cast_id = "clerk",
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent( "ADMIRALTY_CLERK" )
            table.insert(t, agent)
        end,
        events = 
        {
            agent_retired = function( quest, agent ) 
                if quest:IsActive("collect") or quest:IsActive("find_clerk") then
                    quest:Fail()
                end
            end,
        }

    }

    :AddCast{
        cast_id = "target",
        when = QWHEN.MANUAL,
        no_cleanup= true,
        
        on_assign = function(quest, agent)
            quest.param.collected_from[agent] = true
            quest.param.num_attempted = quest.param.num_attempted + 1
        end,

        condition = function( agent, quest )
            if QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) and not quest.param.collected_from[agent] then
                local valid = agent:GetFactionID() ~= "BANDITS" and agent:GetFactionID() ~= "ADMIRALTY" and not AgentUtil.IsFriend(agent, quest:GetCastMember("clerk")) 
                return valid
            end
        end,
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"LABORER", "HEAVY_LABORER", "PRIEST", "WEALTHY_MERCHANT"}
            local def = defs[clamp(quest:GetRank(), 1, #defs)]
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }

    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_ADMIRALTY_CONTACT",
        provider = true,
    }

    :AddCastByAlias{
        cast_id = "fssh",
        alias = "FSSH",
        limit = QLIMIT.NOLIMIT,
    }

    :AddObjective{
        id = "find_clerk",
        title = "Meet up with {clerk}",
        desc = "Find {clerk} so that you can take them on their rounds.",
        mark = { "clerk" },
        state = QSTATUS.ACTIVE,
        on_activate = function(quest) 
            AgentUtil.PutAgentInWorld(quest:GetCastMember("clerk"))
        end,
    }

    :AddObjective{
        id = "collect",
        title = "Collect from {target}",
        negotiation_targets = {"target"},
        desc = "{target} owes taxes to the Admiralty. Take {clerk} to go collect.",
        mark = { "target" },
    }

    :AddObjective{
        id = "return_the_money",
        title = "Return the money to {handler}",
        desc = [[This will complete the job. ]],
        ready_for_completion = true,
        mark = {"handler"},
        --terminal = true
    }
    :AddObjective{
        id = "cheat_handler",
        title = "Convince {handler} that you were robbed",
        negotiation_targets = {"handler"},
        desc = [[{clerk} and you have decided to cheat {handler}. You'll need to convince {handler.himher} that the Spree took the money.]],
        ready_for_completion = true,
        mark = {"handler"},
        --terminal = true
    }

QDEF:AddAttract("find_clerk", "clerk",
    [[
        agent:
            $neutralResigned
            What do you want? I've got shills to count.
    ]])

QDEF:AddConvo("find_clerk", "clerk")
    
    :Loc{
        OPT_COLLECT_CLERK = "Set out to collect taxes",
        DIALOG_START_FRIEND = [[
            * It's nice when folks greet you with pleasant surprise. It doesn't happen often.
            agent:
                $happyGreeting
                Hey, {player}. What brings you here?
            player:
                $neutralDirect
                {handler} sent me. We're supposed to go squeeze some missing taxes out of the citizenry.
            agent:
                !dubious
                $neutralDubious
                Do bounty hunters handle this sort of thing back on the derrick? Seems kinda insulting for your skills, ain't it?
            player:
                !shrug
                $neutralWhatever
                I have an arrangement with {handler}. She gets to be insulting.
            agent:
                !happy
                $happyDefault
                Lucky for me, I guess!
            *** You'll be collecting taxes with {agent}.

        ]],
        DIALOG_START_UNFRIEND = [[
            * After the way you last parted, you can't really expect much better than the greeting you get from {agent}.
            agent:
                !angry
                $angryGreeting
                What the Hesh are <i>you</i> doing here?
            player:
                !sigh
                $miscEbarrassed
                Oh boy. Uh, {handler} sent me. You wanted protection, apparently.
            agent:
                $angryHostile
                Yeah, from jerks like <i>you</i>.
                !facepalm
                $angryReluctant
                Ugh. I'm gonna have a word with {handler} about this, believe me. Let's get this over with before my skin crawls away. 
            player:
                ...
            agent:
                !angry_accuse
                $angrySeething
                Because you make my skin crawl.
            player:
                !overthere
                $neutralResigned
                Yeah I... got it. Lead the way. 
            *** You'll be collecting taxes with {agent}.
        ]],
        DIALOG_START = [[
            player:
                $neutralDirect
                {handler} sent me. We're supposed to go squeeze some missing taxes out of the citizenry.
            agent:
                !dubious
                $neutralDubious
                You ain't Admiralty. What's the deal?
            player:
                !shrug
                $neutralDubious
                Uh... community service?
            agent:
                !take
                $neutralWhatever
                Huh. Well, this wouldn't be the weirdest thing {handler}'s had me do.
                Let's get to it, I guess. 
            *** You'll be collecting taxes with {agent}.
        ]],

        DIALOG_FIRST_COLLECT = [[
            agent:
                $neutralDirect
                The first person on my list is {target}. Think I know {target.himher}, actually.
                Let's head out.
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_COLLECT_CLERK")
            :Fn(function(cxt) 
                    cxt.quest:AssignCastMember("target")
                    ConvoUtil.RelationshipDialog( cxt, cxt:GetAgent(), "DIALOG_START", "DIALOG_START_FRIEND", "DIALOG_START_UNFRIEND")
                    cxt.quest:Complete("find_clerk")
                    cxt.quest:Activate("collect")
                    cxt:Dialog("DIALOG_FIRST_COLLECT")

                    cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                    StateGraphUtil.AddEndOption(cxt)
                end)
        end)   

QDEF:AddConvo("collect", "target")
    :Loc{
        OPT_COLLECT_TAXES = "Collect taxes for the Admiralty",
        OPT_ROUGH_THEM_UP = "Force them to pay up",
        DIALOG_START = [[
            clerk:
                !left
                $angryHostile
                Alright, {target}, it's time to pay up.
            target:
                !dubious
                $neutralDubious
                I just paid you last week!
            clerk:
                $neutralDirect
                Not according to this register. You got a receipt?
            target:
                !scared_shrug
                $scaredFearful
                Well, no, not <i>on</i> me, but—
            clerk:
                !spit
                $angryCruel
                How convenient for you. {player}, talk some sense into this slug.
            player:
                !left
            *** {clerk} wants you to collect tax money from {target}.
        ]],
        DIALOG_START_FRIEND = [[
            target:
                !right
            * While {target} might greet you with a wave, {target.heshe} clearly has {target.hisher} guard up once {target.heshe} sees the company you keep.
            target:
                !dubious
                $neutralDubious
                Hey, {player}. You doing good?
            clerk:
                !left
                $neutralDefault
                Oh, you know each other? Well, good. That makes this easier.
                {player} and I are here to collect your taxes, civilian. 
                Pay up and this all goes easy.
            target:
                $angrySeething
                What? But I already paid!
            clerk:
                $miscPersuasive
                Tsk, lying to your good friend {player}. That's cold.
                What do you make of that, {player}?
            player:
                !left
            *** {clerk} wants you to collect tax money from {target}.
        ]],

        DIALOG_START_2 = [[
            clerk:
                !left
                $neutralDirect
                Hey, {target}. I got a report here saying you owe taxes.
            target:
                !scared
                $scaredTakenAback
                Wh-what? But I already—
            clerk:
                $neutralDirect
                {player}, do your thing.
            player:
                !left
                !crossed
            *** {clerk} wants you to collect tax money from {target}.
        ]],
        DIALOG_START_FRIEND_2 = [[
            target:
                !right
            * {target} looks up when you arrive, and {target.hisher} worn face relaxes for what looks like the first time in months.
            target:
                $neutralDirect
                Howdy, {player}. Who's your friend?
            clerk:
                !left
                $neutralWhatever
                What, no greeting for me, {target}?
            target:
                !dubious
                $neutralDubious
                Huh?
            clerk:
                $neutralDirect
                I got a report here saying you owe taxes.
            target:
                !scared
                $scaredStammering
                Wh-what? But I already—
            clerk:
                $neutralWhatever
                {player}, do your thing.
            player:
                !left
                !crossed
            *** {clerk} wants you to collect tax money from {target}.
        ]],

        DIALOG_START_3 = [[
            clerk:
                !left
                $neutralDirect
                Hey, {target}. I got a report here saying you owe taxes.
            target:
                !scared
                $scaredStammering
                Wh-what? But I already—
            clerk:
                $neutralWhatever
                {player}, do your thing.
            player:
                !left
                !crossed
            *** {clerk} wants you to collect tax money from {target}.
        ]],
        DIALOG_START_FRIEND_3 = [[
            target:
                !right
            * You wish it weren't so, but the third target is none other than {target}. Somehow you don't imagine {target.heshe} will like what you're here to say.
            target:
                $neutralDirect
                {player}? All right, there?
                What can I help you with?
            clerk:
                !clerk
                $happyGreeting
                Well, ain't that nice! I love a helpful face. 
            * {target}'s face might've been helpful, but it sours the moment {clerk} speaks up.
            target:
                !dubious
                $neutralDubious
                I have nothing to say to <i>you</i>, switch.
            clerk:
                !left
                $neutralResigned
                Okay, we do this hard way then. {player}?
            player:
                !left
                !crossed
            target:
                $neutralDirect
                Sal? What's this about?
            *** {clerk} wants you to collect tax money from {target}.
        ]],       

        DIALOG_ON_LOSE_NEGOTIATION = [[
            player:
                So you should pay your bills, 'cause that way the Admiralty can serve <i>you</i> better, you see?
            target:
                You could've served me better by leaving me alone. Now?
                !angry_threaten
                I'm ready to fight about it. 
            *** {target} refuses to pay their taxes and has become hostile.
        ]],

        DIALOG_ROUGH_UP = [[
            clerk:
                !left
                My friend here is going to beat some money out of you now.
                !cruel
                You know, <i>for the government</>.
                !exit
            player:
                !left
                !fight
        ]],
        DIALOG_ON_WIN_NEGOTIATION = [[
            target:
                I—fine. Don't know what else I should expect from you goons.
            clerk:
                !left
                Alright. Now make with the shills.

        ]],
        DIALOG_ON_LOSE_NEGOTIATION_2 = [[
            * {clerk} consults {clerk.hisher} notes.
            clerk:
                !left
                $neutralDirect
                {target}?
            target:
                !dubious
                $neutralDubious
                Err, yeah?
            clerk:
                !left
                $neutralDirect
                Your taxes are due. <i>Overdue</i>, in fact. 
                We've come to collect.
            target:
                !angry
                $angryHostile
                Liar.
                What's your name? I'm gonna report you to your supervisor.
            clerk:
                !spit
                $angryHostile
                My name is pay up or you'll regret it.
            target:
                !dubious
                $neutralDubious
                Uh—
            player:
                !left
                !angry_shrug
                $angryDefault
                Guess that's my cue.
            target:
                !fight
                $angryHostile
                Thugs! 
        ]],
        DIALOG_ON_WIN_NEGOTIATION_2 = [[
            target:
                $scaredPowerless
                You win! Just take it.
                I gotta wonder what's the point of my arrangement if I've gotta endure this treatment anyway.
            clerk:
                !left
                !take
                $neutralWhatever
                Not my problem. If ya like, you can take it up with my supervisor.
                I'm sure {target.heshe}'ll care even less than I do, though.

        ]],
        DIALOG_COLLECT_MONEY_NEXT = [[
            * {target} begrudgingly hands over the shills.
            target:
                !spit
                $angrySeething
                Highway robbery, this is.
            clerk:
                !left
                $miscMocking
                I've <i>been</i> highway robbed, bucko. This ain't it.

        ]],
        DIALOG_COLLECT_MONEY = 
        [[
            * {target} produces a pouch of shills from within some hidden folds of {target.hisher} shirt. {target.HeShe} tosses it to {clerk}, who counts it out with a dampened thumb.
            clerk:
                !left
                $happyCocky
                {1#money}. Not bad!
                See, I knew you had it in you.
                !exit
            * {target} glares at you.
            player:
                !left
            target:
                !angry
                $angrySeething
                Didn't realize hunters did shake-downs, now. 
            player:
                !dubious
                $neutralDubious
                Who, me?
            target:
                Don't see anyone else strong-arming for a couple of shills, do I?
                !angry_accuse
                $angryHostile
                I'll remember this, hunter. 
        ]],
        DIALOG_KILLED_TARGET = [[
            * {target} falls dead to the ground.
            clerk:
                !right
                $angrySeething
                You weren't supposed to <i>kill</> them! 
                Let's get outta here before anyone notices. We'll have to make up for it on the next one, I guess.
                !point
                $angryPatienceLost
                Assuming you don't kill <i>them</i>, too.
        ]],
        DIALOG_SURRENDERED_TARGET = [[
            target:
                !injured
                $scaredPowerless
                Stop! Stop! I'll pay!
        ]],
        DIALOG_NEXT_TARGET = [[
            player:
                !left
            clerk:
                !right
                $happyCocky
                On to the next mark!
                !thought
                $miscMischievous
                Er, I mean, citizen.
                We're looking for {target}. 
                Let's go before they catch wise we're coming for them.
        ]],
        DIALOG_NEXT_TARGET_NEXT = [[
            player:
                !left
            clerk:
                !right
                $neutralDefault
                Let's hit {target} next. 
        ]],        
        DIALOG_COLLECTED_ENOUGH = [[
            player:
                !left
            clerk:
                !right
                $neutralDirect
                I think we have enough for now.
                Let's head back to {handler} so we can turn in the loot.
        ]],
        DIALOG_FAILED_TOO_MUCH = [[
            clerk:
                !angry
                $angrySeething
                Did {handler} really send you, or are you just a stooge? Maybe Nadan sent you to make {handler}'s life harder, huh?
                I'm leaving. And I'm gonna make sure {handler} knows whose fault this is!
                !exit
        ]],
        OPT_DEFEND_SELF = "Defend yourself",
        OPT_PICK_NEW_TARGET = "Convince {clerk} to find another target",
        DIALOG_PICK_NEW_TARGET = [[
            clerk:
                !right
            player:
                !left
                $miscEmbarrassed
                Uh, {target} and I know each other. Do you think we could find someone else from your list?
            clerk:
                $neutralDirect
                Just because you know each other? Didn't take you for a softie.
        ]],
        DIALOG_PICK_NEW_SUCCESS = [[
            clerk:
                $neutralWhatever
                Yeah alright, if it means that much to you, there's always someone else on the list.
            player:
                $happyThanks
                Thanks for being lenient.
        ]],
        DIALOG_PICK_NEW_FAIL = [[
            clerk:
                $neutralDirect
                Listen, this is admiralty business. Your feelings don't change the rules.
            player:
                !sigh
                $neutralResigned
                Alright, alright, I'll do it.
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_COLLECT_TAXES")
            :Fn(function(cxt) 
                    if cxt.quest.param.num_attempted == 1 then
                        ConvoUtil.RelationshipDialog( cxt, cxt:GetAgent(), "DIALOG_START", "DIALOG_START_FRIEND")
                    elseif cxt.quest.param.num_attempted == 2 then
                        ConvoUtil.RelationshipDialog( cxt, cxt:GetAgent(), "DIALOG_START_2", "DIALOG_START_FRIEND_2")
                    else
                        ConvoUtil.RelationshipDialog( cxt, cxt:GetAgent(), "DIALOG_START_3", "DIALOG_START_FRIEND_3")
                    end
            end )
            :Negotiation{
                on_fail = function() 
                    cxt:GetAgent():OpinionEvent(OPINION.COLLECTED_ADMIRALTY_TAXES)
                    cxt:Dialog(cxt.quest.param.num_attempted == 1 and  "DIALOG_ON_LOSE_NEGOTIATION" or "DIALOG_ON_LOSE_NEGOTIATION_2")
                    cxt:Opt("OPT_DEFEND_SELF")
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            
                            on_start_battle = function(battle)
                                local fighter = battle:GetFighterForAgent(cxt:GetAgent())
                                if fighter then
                                    fighter:AddCondition("WANTED_ALIVE")
                                end
                            end,

                            noncombatants = { cxt.quest:GetCastMember("clerk") },
                            on_win = function(cxt) 
                                if cxt:GetAgent():IsDead() then
                                    cxt:Dialog("DIALOG_KILLED_TARGET")
                                    if cxt.quest.param.num_attempted <= 3 then
                                        cxt:GoTo("STATE_NEXT_TARGET")
                                    else
                                        cxt:Dialog("DIALOG_FAILED_TOO_MUCH")
                                        cxt.quest:Fail()
                                        StateGraphUtil.AddEndOption(cxt)
                                    end
                                else
                                    cxt:Dialog("DIALOG_SURRENDERED_TARGET")
                                    cxt:GoTo("STATE_COLLECTED")
                                    
                                end
                            end,
                        }

                end,
                on_success = function() 
                    cxt:Dialog(cxt.quest.param.num_attempted == 1 and  "DIALOG_ON_WIN_NEGOTIATION" or "DIALOG_ON_WIN_NEGOTIATION_2")
                    cxt:GoTo("STATE_COLLECTED")
                end,
            }
            :ReceiveOpinion( OPINION.COLLECTED_ADMIRALTY_TAXES, { only_show = true } )

        if cxt.quest:GetCastAgent("target"):GetRelationship() >= RELATIONSHIP.LIKED and not cxt.enc.scratch.tried_change_target then
            cxt:Opt("OPT_PICK_NEW_TARGET")
                :Dialog("DIALOG_PICK_NEW_TARGET")
                :Negotiation{
                    target_agent = cxt.quest:GetCastAgent("clerk"),
                    on_success = function()
                        cxt:Dialog("DIALOG_PICK_NEW_SUCCESS")
                        cxt:GoTo("STATE_NEXT_TARGET")
                    end,
                    on_fail = function()
                        cxt:Dialog("DIALOG_PICK_NEW_FAIL")
                        cxt.enc.scratch.tried_change_target = true
                    end
                }
        end

        cxt:Opt("OPT_ROUGH_THEM_UP")
            :Dialog("DIALOG_ROUGH_UP")
            :Battle{
                noncombatants = { cxt.quest:GetCastMember("clerk") },
                on_start_battle = function(battle)
                    local fighter = battle:GetFighterForAgent(cxt:GetAgent())
                    if fighter then
                        fighter:AddCondition("WANTED_ALIVE")
                    end
                end,
                
                on_win = function(cxt) 
                    if cxt:GetAgent():IsDead() then
                        cxt:Dialog("DIALOG_KILLED_TARGET")
                        if cxt.quest.param.num_attempted <= 3 then
                            cxt:GoTo("STATE_NEXT_TARGET")
                        else
                            cxt:Dialog("DIALOG_FAILED_TOO_MUCH")
                            cxt.quest:Fail()
                            StateGraphUtil.AddEndOption(cxt)
                        end
                    else
                        cxt:Dialog("DIALOG_SURRENDERED_TARGET")
                        cxt:GoTo("STATE_COLLECTED")
                    end
                end,
            }
        end)   
    
    :State("STATE_COLLECTED")
        :Fn(function(cxt)
                cxt.quest.param.num_collected = cxt.quest.param.num_collected + 1
                cxt:GetAgent():OpinionEvent(OPINION.COLLECTED_ADMIRALTY_TAXES)
                local amt = math.floor(  math.randomGauss( cxt.quest:GetCastMember("target"):GetRenown()*125*.8, cxt.quest:GetCastMember("target"):GetRenown()*125*1.2))
                cxt.quest.param.amount_collected = cxt.quest.param.amount_collected + amt
                cxt.quest:GetCastMember("clerk"):DeltaMoney( amt )
                cxt:Dialog(cxt.quest.param.num_attempted == 1 and "DIALOG_COLLECT_MONEY" or "DIALOG_COLLECT_MONEY_NEXT", amt)
                cxt:GoTo("STATE_NEXT_TARGET")
            end)

    :State("STATE_NEXT_TARGET")
        :Loc{
        }
        
        :Fn(function(cxt) 
            cxt.quest:UnassignCastMember("target")
            
            local thresh = cxt.quest:GetRank() <= 1 and 1 or 2
            if cxt.quest.param.num_collected >= thresh then
                cxt:Dialog("DIALOG_COLLECTED_ENOUGH")
                cxt.quest:Complete("collect")
                cxt.quest:Activate("return_the_money")
                StateGraphUtil.AddEndOption(cxt)
            else
                cxt.quest:AssignCastMember("target")
                    
                cxt:Dialog(cxt.quest.param.num_attempted == 2 and "DIALOG_NEXT_TARGET" or "DIALOG_NEXT_TARGET_NEXT")
                cxt.quest:Cancel("collect")
                cxt.quest:Activate("collect")
                StateGraphUtil.AddEndOption(cxt)
            end
        end)

QDEF:AddConvo("return_the_money")
    :Confront(function(cxt) 
            
            if cxt.quest:GetCastMember("clerk"):IsInPlayerParty() and cxt.location:HasTag("in_transit") and not cxt.quest.param.has_had_crisis then
                if math.random() > .5 then
                    return "STATE_ROBBERY"
                else
                    return "STATE_PLOT"
                end
            end
        end)

    :State("STATE_ROBBERY")
        :Loc{
            DIALOG_INTRO = [[ 
                * The chance comes to earn your keep on the road back to Admiralty HQ. 
                * It's not {clerk}'s fault when you inevitably get ambushed—but it happens all the same.
                bandit:
                    !right
                    $angryGreeting
                    There they are!
                clerk:
                    !left
                    $neutralResigned
                    Oh Hesh.
                bandit:
                    !cruel
                    $angryCruel
                    You must be collecting, officer, because we can hear your shills jingling from a mile off.
                clerk:
                    !angry_accuse
                    $angryHostile
                    Get out of it, scumbag. Tell Nadan to stop dogging {handler} like some mangy runt.
                bandit:
                    !crossed
                    $angryPatienceLost
                    You say that, but all we've seen is profits. Nadan's vendetta is good for business.
                clerk:
                    Yeah? Not for long. 
                    !throatcut
                    $angryCruel
                    Earn your keep, {player}. Teach these thugs a lesson.
                player:
                    !left
                    !sigh
                *** You're mugged by bandits. 
            ]],
            DIALOG_FRIEND = [[
                bandit:
                    !happy
                    $happyGreeting
                    Oh, hey {player}. Didn't know you were running with this loser.
                player:
                    $neutralResigned
                    For the moment, anyway.
                bandit:
                    !shrug
                    $neutralWhatever
                    I ain't got any interest in ruining a good thing for you. And the boss just wants trouble for {handler}. Let me kill {clerk}, and I'll give you your share. 
                    Fifty-fifty. Whaddya say?
                    {clerk} ain't even fighting on your side. You gonna spill blood for that coward?
            ]],
            DIALOG_ENEMY = [[
                bandit:
                    !angry
                    $angryDefault
                    Well, look at that. I'd have been here sooner if I'd known I'd get to profit off <i>your</i> hide. 
                    Pay up or bleed out. I'm happy either way.
                    But I sure wouldn't go to much effort to protect a coward like {clerk}.
            ]],
            DIALOG_BANDIT = [[
                bandit:
                    !dubious
                    $neutralDubious
                    That Admiralty coward is making you fight for them all by yourself?
                    Sounds like a bad deal, friend. 
            ]],
            OPT_DEFEND = "Defend {clerk}",
            OPT_AGREE_TO_SPLIT = "Agree to the offer",
            OPT_ASK_TO_LEAVE = "Convince {bandit} to leave",
            OPT_STEP_ASIDE = "Step aside",

            DIALOG_WON_FIGHT_CLERK_DEAD = [[
                agent:
                    !exit
                * {clerk} is dead. You'll have to return the money to {handler} and confess you botched the bodyguard job.
            ]],

            DIALOG_WON_FIGHT_CLERK_ALIVE = [[
                agent:
                    !exit
                clerk:
                    !right
                    $happyAmazed
                    Looks like you're worth your salt!
                    Let's get back before that happens again.
            ]],

            DIALOG_AGREE_TO_SPLIT = [[
                player:
                    $neutralDirect
                    Deal. This switch is a waste of air anyway.
                clerk:
                    !left
                    $angryTakenAback
                    What?!
                agent:
                    $neutralDefault
                    Nasty shock, ain't it? But it'll be over quick.
                clerk:
                    !exit

                * {clerk} doesn't even manage a fight before the bandit makes quick work of {clerk.himher}.
            ]],

            DIALOG_SPLIT_CLERK_DEAD = [[
                player:
                    !left
                agent:
                    !give
                    $neutralDirect
                    Here's your share, {player}.
                player:
                    !take
                    $neutralDirect
                    Much obliged.
            ]],
    
            
            DIALOG_SPLIT_GET_SHARE = [[
                agent:
                    !interest
                    $neutralDubious
                    You're not going to catch hell for this, are you?
                player:
                    !shrug
                    $neutralWhatever
                    Maybe. But I'll think of something.
                    !wink
                    $happyCocky
                    I always do.
                agent:
                    $neutralDubious  
                    Oh, hey, fair warning: you're okay in my books, but my boss, Nadan?
                    He <i>hates</i> {handler}. Has a million and one ways he'd like to see {handler.himher} dead.
                    And he won't care who he cuts down to make it happen. You, me—we're all fair game.
                    Do what you gotta do, pal. But know that you're on borrowed time as long as you keep {handler}'s company.
            ]],
            
            DIALOG_STEP_ASIDE = [[
                player:
                    $miscEmbarrassed
                    Sorry, {clerk}. 
                    I don't get paid enough for this.
                clerk:
                    !right
                    !angry
                    $angrySeething
                    You lumin-tainted coward!
                    !exit
                * {agent} kills {clerk} and collects the shills with equally ruthless efficiency.
            ]],
            DIALOG_STEP_ASIDE_PST = [[
                agent:
                    !right
                    !angry_threaten
                    $angryHostile
                    You still here? Get lost if you know what's good for you.
                player:
                    !shrug
                    $miscEmbarrassed
                    I'm going, I'm going.
                    !exit
            ]],
            DIALOG_ASK_TO_LEAVE = [[
                player:
                    $neutralDubious
                    Is this really the fight you wanna pick?
                    There's gotta be better uses of your time and talents.
            ]],
            DIALOG_ON_WIN_NEGOTIATION = [[
                player:
                    $neutralDirect
                    There's blood in the water, friend, but it ain't here.
                    Pick your fight somewhere else.
                agent:
                    $neutralThoughtful
                    Hrm. Maybe so. 
                    Until next time, <i>hunter</i>.
                    !exit
                clerk:
                    !right
                    $neutralResigned
                    Not sure bein' peaceful is part of your job description, but I guess it worked. 
                    Let's get out of here before {agent.heshe} changes {agent.hisher} mind.
            ]],
            DIALOG_ON_LOSE_NEGOTIATION = [[
                agent:
                    $happyCocky
                    Ha! I'm not stupid, you know. 
            ]],
        }
        :SetLooping()
        :Fn(function(cxt) 
                if cxt:FirstLoop() then
                    cxt.quest.param.has_had_crisis = true
                    local bandit = AgentUtil.GetFreeAgent(cxt.quest:GetRank() <= 2 and "BANDIT_GOON" or "BANDIT_RAIDER")
                    local extras = CreateCombatBackup(bandit, "BANDIT_PATROL_BACKUP", cxt.quest:GetRank() - (bandit:GetContentID() == "BANDIT_RAIDER" and 1 or 0))
                    cxt:ReassignCastMember("bandit", bandit)
                    cxt.enc:SetPrimaryCast(bandit)
                    cxt:Dialog("DIALOG_INTRO")
                    ConvoUtil.RelationshipDialog(cxt, bandit, "DIALOG_BANDIT", "DIALOG_FRIEND", "DIALOG_ENEMY")
                end

                cxt:Opt("OPT_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        noncombatants = { cxt.quest:GetCastMember("clerk") },
                        on_win = function(cxt) 
                            if cxt.quest:GetCastMember("clerk"):IsDead() then
                                cxt:Dialog("DIALOG_WON_FIGHT_CLERK_DEAD")
                            else
                                cxt:Dialog("DIALOG_WON_FIGHT_CLERK_ALIVE")
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_runaway = function(cxt) 
                            cxt.quest:Fail()
                        end,
                    }

                if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL then
                    cxt:Opt("OPT_AGREE_TO_SPLIT")
                        :Fn(function(cxt)
                            
                            cxt:GetCastAgent("clerk"):OpinionEvent(OPINION.BETRAYED)
                            cxt:GetAgent():OpinionEvent(OPINION.COLLECT_TAXES_GAVE_FRIEND_CLERK)
                            cxt:Dialog("DIALOG_AGREE_TO_SPLIT")

                            cxt.quest:GetCastMember("clerk"):Kill()
                            cxt:Dialog("DIALOG_SPLIT_CLERK_DEAD")
                            
                            local your_share = math.floor( cxt.quest.param.amount_collected /2 )
                            cxt.caravan:AddMoney(your_share)
                            cxt:Dialog("DIALOG_SPLIT_GET_SHARE")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)
                end

                cxt:Opt("OPT_ASK_TO_LEAVE")
                    :Dialog("DIALOG_ASK_TO_LEAVE")
                    :Negotiation
                    {
                        on_success = function() 
                            cxt:Dialog("DIALOG_ON_WIN_NEGOTIATION")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function() 
                            cxt:Dialog("DIALOG_ON_LOSE_NEGOTIATION")
                        end
                    }

                if cxt:GetAgent():GetRelationship() >= RELATIONSHIP.NEUTRAL then
                    cxt:Opt("OPT_STEP_ASIDE")
                        :Fn(function()
                                cxt:Dialog("DIALOG_STEP_ASIDE")
                                cxt.quest:GetCastMember("clerk"):OpinionEvent(OPINION.BETRAYED)
                                cxt.quest:GetCastMember("clerk"):Kill()
                                cxt:Dialog("DIALOG_STEP_ASIDE_PST")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)
                end
            end)

    :State("STATE_PLOT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                clerk:
                    !right
                * {clerk} stops for a moment, weighing the bag of shills in {clerk.hisher} hand while fixing you with a shrewd look.
                clerk:
                    !interest
                    $miscEmbarrassed
                    You know, the last time I did the rounds like this, I got <i>robbed by the Spree</i>.
                player:
                    Yeah? Rough. 
                clerk:
                    $neutralAside
                    There's this one Spree hothead. Nadan? 
                    He <i>hates</i> {handler}. Like he invented the feeling, ya know? Targets {handler.himher} and everyone under {handler.himher}. 
                    Working for {handler} puts a target on your skull. But he looks the other way if you lend him a hand. 
                    Like, say... give him some of this money.
                player:
                    !suspicious
                    $neutralDubious
                    <i>Some</i> of it? Is this where you propose we split the leftovers and feed {handler} a sob story about where it went?
                clerk:
                    You saw what went down today—you think {handler} is gonna start a charity with it?
                    You'll catch some heat, sure, but better our lives and our pockets than {handler.hisher}s, I say.
            ]],
            OPT_AGREE_TO_THE_PLAN = "Agree to cheat {handler} for {1#money}",
            OPT_REFUSE = "Refuse",
            DIALOG_AGREE = [[
                player:
                    Yeah, you're right. I'm in.
                clerk:
                    !happy
                    $happyAmazed
                    Now that's what I'm talking about. 
                    !give
                    Here's your cut. 
                player:
                    !take
                    $happyThanks
                    Nice.
                    
            ]],
            DIALOG_AGREE_PST = [[
                clerk:
                    !cagey
                    $miscHushedUrgent
                    Just, uh... whatever you tell {handler}, make sure it's convincing.
                    I'm not entirely sure {handler.heshe} believed me even when I <i>did</i> get robbed.
                    !point
                    $neutralDirect
                    And if you mess it up? From a pal to another: I'll sell you out in a heartbeat.
                player:
                    !dubious
                    $neutralDubious
                    Right. Good to know.
            ]],
            DIALOG_REFUSE = [[
                player:
                    !crossed
                    $angryAside
                    I've got more on the line than a payday. I'm doing the job {handler} hired me for.
                    And that includes keeping an eye on your sticky fingers.
                clerk:
                    !cagey
                    $miscEmbarrassed
                    Yeah? Well I was just testing you, anyway.
                    So... you passed. Good work. 
                    !point
                    $neutralDirect
                    Now let's get back to {handler} and finish the job.
            ]]
        }
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.quest.param.has_had_crisis = true
                cxt.quest.param.xfer = math.floor( cxt.quest.param.amount_collected * 0.5)
            end
            cxt.quest.param.clerk_is_dirty = true
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("clerk"))
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_AGREE_TO_THE_PLAN", cxt.quest.param.xfer)
                :Fn(function()
                        cxt:Dialog("DIALOG_AGREE")
                        
                        cxt.quest:GetCastMember("clerk"):DeltaMoney(-cxt.quest.param.xfer)
                        cxt.caravan:AddMoney(cxt.quest.param.xfer)
                        cxt:Dialog("DIALOG_AGREE_PST")
                        cxt.quest:Cancel("return_the_money")
                        cxt.quest:Activate("cheat_handler")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            cxt:Opt("OPT_REFUSE")
                :Fn(function()
                        cxt.quest.param.refused_cheat_offer = true
                        cxt.quest:GetCastMember("clerk"):OpinionEvent(OPINION.INSULT)
                        cxt:Dialog("DIALOG_REFUSE")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
                
            end)


QDEF:AddConvo("cheat_handler", "handler")
    :Loc{
        OPT_CONVINCE_WAS_ROBBED = "Convince {handler} you were robbed",
        OPT_BETRAY_CLERK = "Tell {handler} about {clerk}'s treachery",
        DIALOG_CONVINCE_WAS_ROBBED = [[
            handler:
                $happyGreeting
                Aha! The hero of the hour. I've been absolutely beside myself with anticipation.
                Show me what you got, {player}. 
            player:
                !bashful
                $miscEmbarrassed
                Yeah, thing is, uh... we were ambushed. 
                !scared_shrug
                $scaredAnguished
                The Spree ganged up on us. At least six arms to each of ours. And a guy with a face like a burnt out candle.
            handler:
                !dubious
                $neutralDubious
                Is that so?
            *** You tell {handler} you were robbed.
        ]],
        DIALOG_ON_WIN_NEGOTIATION = [[
            player:
                $neutralDirect
                I swear, {handler}, when I track down those Spree I'm gonna clean their insides with salt water. 
            handler:
                !surprised
                $neutralTakenAback
                Must've been bad if they got the drop on even a hunter.
                !thought
                $neutralThoughtful
                This is gonna make the priests mad as the deep, though. They might even put out a bounty for Spree necks. 
                !agree
                $neutralResigned
                Which won't please my superiors none, let me tell you. 
            *** You convince {handler} that you were robbed.
        ]],
        DIALOG_CONSPIRACY = [[
            clerk:
                !right
                $happyDefault
                Thanks for your <i>help</i>, {player}.
                !wink
                I sure am glad you were there for me.
            handler:
                !right
                !dubious
                $neutralDubious
                All right, all right. I'm not paying you to plan your next slumber party. Get back to work, {clerk}.
            * {clerk} throws you another unsubtle wink before making {clerk.hisher} exit. Fortunately, {handler} doesn't seem to notice.
        ]],
        DIALOG_ON_LOSE_NEGOTIATION_CLERK_HERE = [[
            * {handler} looks you up and down with a gaze like a jellyfish sting before turning to {clerk}, who wilts under the scrutiny.
            handler:
                !angry
                $angrySeething
                Is that so, {clerk}?
                What is that, the third time this month?
            clerk:
                !left
                !scared
                $scaredFearful
                N-no, sir! {player} is trying to cheat you!
                She took all the money for herself, said she'd turn me inside out through my ears if I spoke up about it.
            player:
                !left
                !angry
                $angryHostile
                Now, hold on just a dang second—
            handler:
                !chuckle
                $miscMocking
                Hah! I only got where I am because I can smell a rat, waterlogged or not. 
                I know you're new to the griftlands but consider this a word of advice: don't <i>cheat</i> if you're just gonna get <i>caught</i>.
                !handwave
                $neutralDirect
                And I will catch you. Make no mistake about that. So next time, you do what you're told and keep it at that.     
                Fortunately for you, I don't have to trust you to have use of you. But you better be sure the next job goes as planned, you understand me?
            *** You fail to convince {handler} that you were robbed.
        ]],

        DIALOG_ON_LOSE_NEGOTIATION_CLERK_NOT_HERE = [[
            player:
                !overthere
                $miscRelieved
                Lucky to escape with my life. Each of them as big as whales! With teeth like—
            handler:
                !angry_shrug
                $angryDefault
                <i>Enough</i>, hunter. Good Hesh. If you stretched a mile for every lie in that mix you'd be long enough to reach the other shore.
                {fssh} had me believe this auction was important to you. Wouldn't've thought you'd risk it all for some pocket change.
                !disappoint
                $angryReluctant
                But I guess once a derrick, always a derrick, huh?
                !angry
                $angryDefault
                Fortunately for you, I don't have to trust you to have use of you. But you better be sure the next job goes as planned, you understand me?
            *** You fail to convince {handler} that you were robbed.
        ]],
        DIALOG_BETRAY_CLERK_HERE = [[
            * You hand over the money that {clerk} gave to you with an aloof shrug.
            player:
                !shrug
                $neutralWhatever
                {clerk} is cheating you. And is kind of a cheapskate, considering.
                Seems {clerk.heshe} regularly punts money towards Nadan.
            handler:
                !right
                !dubious
                $neutralDubious
                Yeowch. What do you make of that damning indictment, {clerk}?
            clerk:
                !left
                !surprised
                $neutralTakenAback
                I—hey wait, she—
            handler:
                !interest
                $neutralDirect
                It's one thing to cheat your boss but to expect others to do the same? For <i>this</i> paltry sum?
                !disappointed
                $neutralResigned
                I'm not mad, {clerk}, I'm just <i>disappointed</i>. 
                And with Nadan no less. He fancies himself my worst enemy, if you can believe it.
            * {clerk} clearly decides against mounting a defence and makes a break for it. {handler} gives {clerk.himher} a moment's headstart before nodding at a nearby guard to begin their chase.
            player:
                !left
                !chuckle
                $neutralJoke
                Good help is hard to find.
            handler:
                !crossed
                $angryDefault
                And yet, here you are. Spat upon our Havarian shores like a blessing from Hesh itself.
            player:
                !dubious
                $neutralDubious
                Uh—
            handler:
                !sigh
                $neutralThoughtful
                A prophet, belched from the bowels of the sea, with the express purpose of roughing up goons and giving me money.
            player:
                !happy
                $happyThanks
                Gross.
            *** You reveal {clerk}'s betrayl to {handler}.
        ]],
        DIALOG_BETRAY_CLERK_NOT_HERE = [[
            * You hand over the money that {clerk} gave to you with an apologetic shrug.
            player:
                $neutralWhatever
                {clerk} was cheating you, siphoning funds for Nadan. Here's the bribe {clerk.heshe} offered me to keep quiet about it.
            handler:
                !take
                $miscMocking
                What! I'll never recover from the shock.
            player:
                !dubious
                $neutralDubious
                ...
                You totally knew, didn't you?
            handler: 
                !shrug
                $neutralWhatever
                Can't <i>imagine</i> what you could mean. 
                I guess I should be grateful you didn't bring me a body. But it's nice to know this problem is sorted, at least.
            *** You reveal {clerk}'s betrayl to {handler}.
        ]]
    }

    :Hub(function(cxt) 
        
        local clerk = cxt.quest:GetCastMember("clerk")
        if not clerk:IsInPlayerParty() then
            clerk = nil
        end

        cxt:Opt("OPT_CONVINCE_WAS_ROBBED")
            :Dialog("DIALOG_CONVINCE_WAS_ROBBED")
            :Negotiation{
                on_success = function() 
                    if clerk then
                        cxt:Dialog("DIALOG_CONSPIRACY")
                        clerk:OpinionEvent(OPINION.COLLECTED_TAXES_CONSPIRACY)
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end
                    StateGraphUtil.AddEndOption(cxt)
                end,
                on_fail = function()
                    cxt:Dialog( clerk and "DIALOG_ON_LOSE_NEGOTIATION_CLERK_HERE" or "DIALOG_ON_LOSE_NEGOTIATION_CLERK_NOT_HERE")
                    cxt.quest:Fail()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            }
        cxt:Opt("OPT_BETRAY_CLERK")
            :DeliverMoney(math.floor( cxt.quest.param.amount_collected / 2), {no_scale=true})
            :Fn(function(cxt) 
                    cxt:Dialog(clerk and "DIALOG_BETRAY_CLERK_HERE" or "DIALOG_BETRAY_CLERK_NOT_HERE")
                    if not cxt.quest:GetCastMember("clerk"):IsDead() then
                        cxt.quest:GetCastMember("clerk"):OpinionEvent(OPINION.BETRAYED)
                    end
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                end)
    end)       

QDEF:AddConvo("return_the_money", "handler")
    :Loc{
        OPT_TURN_IN_MONEY = "Turn in the tax money",
        OPT_TURN_IN_NO_CLERK = "Explain what happened to {clerk}",
        OPT_RAT_OUT_CLERK = "Tell {handler} that {clerk} wanted to steal the collection",
        OPT_MAKE_AN_EXCUSE = "Make up an excuse",
        
        OPT_WORKING_ON_IT = "Stall for time",
        DIALOG_START = [[
            player:
                !give
                $miscMocking
                I've got a delivery for {handler}. Says on the slip I should hand it over with a smile.
            handler:
                !take
                $neutralWhatever
                Cute. You bring my soldier back safe, too?
            clerk:
                !left
                Yeah, yeah. I'm alive. And still years away from a pension.
            handler:
                Chin up. It'll be over before you know it. 
            player:
                !left
                $neutralDirect
                So? Mission accomplished, officer?
            handler:
                $neutralThanks
                Looks like. So that warrants an award.
            *** You deliver the tax money safely to {handler}.
        ]],
        DIALOG_START_CLERK_DEAD = [[
            player:
                !give
                $neutralResigned
                What would you like first? The good news, or the bad news?
            handler:
                !take
                $happyDefault
                If you got the shills, how could there be any bad news?
            player:
                !dubious
                $neutralDubious
                Uh...
            handler:
                !facepalm
                $neutralResigned
                Aw, Hesh. {clerk} is dead, isn't {clerk.heshe}?
            player:
                $neutralDubious
                Well...
            handler: 
                !sigh
                $neutralDirect
                The good news is that since this activity wasn't officially sanctioned by the chain of command, I don't have to submit any paperwork. Technically, {clerk.heshe} was killed off-duty.
                !happy
                $happyDefault
                So, hey, you're the gift that keeps on giving, I guess!
                !dubious
                $neutralDubious
                Assuming you have the money, at least.
            *** You tell {handler} that {clerk} is dead.
        ]],
        DIALOG_EXCUSE = [[
            player:
                !bashful
                $neutralDubious
                It's a funny story, actually...
        ]],
        DIALOG_ON_LOSE_NEGOTIATION = [[
            * {handler}'s wry smile takes on a shark-like edge, and you realize you might have overstayed your welcome in Admiralty territory.
            handler:
                !angry_accuse
                $angrySeething
                I gave you a job. And it sounds like you failed.
                Fortunately for you, I don't have to trust you to have use of you. But you better be sure the next job goes as planned, you understand me?
                *** You tell {handler} you don't have the tax money. She's not impressed.
        ]],
        DIALOG_ON_WIN_NEGOTIATION = [[
            * {handler} takes a moment to accept the news you've delivered.
            handler:
                !facepalm
                $miscHopeless
                Hesh's briny barnacles.
                !sigh
                Still, at least you told me. I can appreciate how hard <i>that</i> is, since I'm gonna have to deliver this news to the Heshians.
                !point
                $neutralThoughtful
                Only means they're <i>really</i> gonna want the roads clear for the auction. So we got our work cut out for us. 
            *** You tell {handler} you don't have the tax money. She takes it well.
        ]],
        DIALOG_PAID_OFF = [[
            player:
                !give
                $neutralWhatever
                As ordered: one million shills. Or, you know, however much this is. I didn't count it.
            handler:
                !handwring
                $happyThanks
                Well hey, look at this.
                I wasn't sure you'd pull through. Glad to see I don't have to write {fssh} a sternly-worded letter.
            *** You deliver the tax money to {handler}.
        ]],
        DIALOG_WORKING_ON_IT = [[
            player:
                !placate
                $miscEmbarrassed
                It's coming! There's just been... a delay.
            handler:
                !dubious
                $neutralDubious
                What the Hesh does that mean?
            player:
                $neutralJoke
                Be patient and you'll find out!
            handler:
                $neutralDirect
                I can be patient. But only for about five minutes.
                !cruel
                $angryCruel
                After that all bets are off. No telling what might happen.
            player:
                !scared
                $scaredFearful
                Good to know.
            *** You tell {handler} you need more time to get her the tax money.
        ]],
        DIALOG_RAT_OUT_CLERK_INTRO = [[
            player:
                !crossed
                $neutralDirect
                May interest you to know that {clerk} here tried to buy me out.
                Said we could split the money fifty-fifty if we just kept it for ourselves.
            handler:
                !sigh
                $miscDepressed
                No offense, but my soldiers try to set each other up all the time. 
                If I acted on every pointed finger I'd be understaffed pretty much constantly.
                !dubious
                $neutralDubious
                You got any proof?
            *** You tell {handler} that {clerk} is a traitor, but she's unconvinced.
        ]],
        DIALOG_ON_WIN_RAT_NEGOTIATION = [[
            handler:
                !thought
                $neutralThoughtful
                Huh. I guess I should've expected it from {clerk}. 
                Well, good to know. But you wouldn't have told me this without a reason.
                !give
                $neutralDirect
                If you have an angle, you've successfully worked it. What do you want?
            *** You convince {handler} that {clerk} is a traitor.
        ]],
        DIALOG_ON_LOSE_RAT_NEGOTIATION = [[
            handler:
                !handwave
                $neutralWhatever
                Forget it, {player}. I don't blame you for playing the game, but {clerk}'s too valuable to me to take you at your word.
                Next time, maybe pick your target a bit better. 
            *** You fail to convince {handler} that {clerk} is a traitor. 
        ]]
    }
    :Hub(function(cxt) 
        local clerk_here = cxt.quest:GetCastMember("clerk"):IsInPlayerParty()

        if clerk_here then
            
            if cxt.quest.param.refused_cheat_offer then
                
                cxt:Opt("OPT_RAT_OUT_CLERK")
                    :Dialog("DIALOG_RAT_OUT_CLERK_INTRO")
                    :Fn(function(cxt) 
                            cxt.quest:GetCastMember("clerk"):OpinionEvent(OPINION.BETRAYED)
                        end)
                    :Negotiation{
                        hinders = {"clerk"},
                        subject = cxt.quest:GetCastMember("clerk"),
                        on_success = function() 
                            cxt:Dialog("DIALOG_ON_WIN_RAT_NEGOTIATION")
                            cxt.quest:GetCastMember("clerk"):MoveToLimbo()
                            cxt.quest:Complete()
                            ConvoUtil.GiveQuestRewards(cxt)
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                        on_fail = function() 
                            cxt:Dialog("DIALOG_ON_LOSE_RAT_NEGOTIATION")
                            cxt.quest:Complete()
                            ConvoUtil.GiveQuestRewards(cxt)
                            StateGraphUtil.AddEndOption(cxt)
                        end
                    }
            end

                
            cxt:Opt("OPT_TURN_IN_MONEY")
                :Fn(function(cxt) 
                        cxt:Dialog("DIALOG_START")
                        cxt.quest:GetCastMember("clerk"):OpinionEvent(OPINION.COLLECTED_TAXES_TOGETHER)
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                        StateGraphUtil.AddEndOption(cxt)
                    end)
        else
            cxt:Opt("OPT_TURN_IN_NO_CLERK")
                :Fn(function(cxt) 
                        cxt:Dialog("DIALOG_START_CLERK_DEAD")
                        
                        
                        cxt:Opt("OPT_MAKE_AN_EXCUSE")
                            :Dialog("DIALOG_EXCUSE")
                            :Negotiation
                            {
                                on_success = function() 
                                    cxt:Dialog("DIALOG_ON_WIN_NEGOTIATION")
                                    cxt.quest:Complete()
                                    ConvoUtil.GiveQuestRewards(cxt)
                                    StateGraphUtil.AddEndOption(cxt)
                                end,
                                on_fail = function() 
                                    cxt:Dialog("DIALOG_ON_LOSE_NEGOTIATION")
                                    cxt.quest:Fail()
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end
                            }

                        cxt:Opt("OPT_TURN_IN_MONEY")
                            :DeliverMoney(cxt.quest.param.amount_collected, {no_scale=true})
                            :Fn(function(cxt) 
                                    cxt:Dialog("DIALOG_PAID_OFF")
                                    cxt.quest:Complete()
                                    ConvoUtil.GiveQuestRewards(cxt)
                                    StateGraphUtil.AddEndOption(cxt)
                                end)
                        cxt:Opt("OPT_WORKING_ON_IT")                            
                            :Dialog("DIALOG_WORKING_ON_IT")
                            :DoneConvo()

                    end)

        end
    end)   



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                I've been running patrols in preparation for the auction, but the bosses are side-eyeing my expenses like <i>I'm</i> the reason the Spree are bleeding us dry.
                !eureka
                $happyJoke
                So I request the tax records, like the keen officer I am, and lo and behold I find a few folks who are behind on their payments!
            player:
                !humouring
                $neutralJoke
                Odd of the Admiralty to let wayward taxpayers slip through the cracks, ain't it?
            handler:
                !agree
                $neutralThoughtful
                Very odd indeed. <i>Must</i> just be an oversight. 
                I'm going to send one of mine out to collect, but just in case that oversight has an over<i>bite</i>, I'd like {clerk.himher} to have some protection. That's you!
                !point
                $neutralDirect
                Go find {clerk} and take {clerk.himher} out on the rounds. If dirty cops get in your way, play nice—I can't protect you if you start making an enemy of soldiers. 
            *** {handler} wants you to escort one of her agents on a tax collection run. 
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                !dubious
                $neutralDirect
                I don't care if you do a bit of skimming off the top, but the Admiralty gets its cut or <b>you</> get cut, understand?
            player:
                Loud and clear. 
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)




QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.DECLINED )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                !dubious
                $neutralDubious
                I'm surprised. It seems like you'd jump at the opportunity to handle that much money.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)

