AddOpinionEvent("TRIED_TO_RECRUIT_MOLE_AND_FAILED", 
{
    delta = OPINION_DELTAS.DISLIKE,
    txt = "Tried to convince them to be a mole",
})
AddOpinionEvent("TRIED_TO_RECRUIT_MOLE_AND_SUCCEDED", 
{
    delta = OPINION_DELTAS.DISLIKE,
    txt = "Convinced them to be a mole for the Admiralty",
})


local QDEF = QuestDef.Define{
        title = "Making Molehills",
        desc = "Find a new mole inside the Spree for {handler}. \n{handler} is looking for a new mole to gather intel on the Spree's activity. Rumor has it {mole} is willing to sell out their friends and family.",
        
        icon = engine.asset.Texture("icons/quests/handler_admiralty_find_bandit_informant.tex"),
        rank = {1, 3},
        qtype = QTYPE.CONTRACT,
        focus = QUEST_FOCUS.NEGOTIATION,
        faction = "ADMIRALTY",
        
        precondition = function(quest)             
            return Util.IsPlayerWorkingWithFaction("ADMIRALTY")
        end,
    }

    :AddObjective{
        id = "find_a_new_mole",
        mark = {"mole"},
        title = "Compromise {mole}",
        desc = "{handler} wants a new mole in the Spree, and thinks that {mole} might turn. Go convince them.",
        state = QSTATUS.ACTIVE,
        negotiation_targets = {"mole"},
        combat_targets = {"mole"},

        on_complete = function(quest)
            quest:Activate("return_for_reward")
        end,
    }


    :AddObjective{
        id = "return_for_reward",
        title = "Tell {handler} about {mole}",
        desc = [[You've found someone to act as a mole. Go tell {handler}.]],
        mark = {"handler"},
        ready_for_completion = true,
        terminal = true
    }


    :AddCast{
        cast_id = "mole",
        condition = function( agent, quest )
            if agent:GetFactionID() == "BANDITS" and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) then
                return true
            end
        end,        
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"BANDIT_GOON","BANDIT_GOON","BANDIT_GOON2","BANDIT_RAIDER","BANDIT_CAPTAIN"}
            local def = defs[math.min( quest:GetRank(), #defs)]
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }

    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_ADMIRALTY_CONTACT",
        provider = true,
    }

QDEF:AddConvo("find_a_new_mole", "mole")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_WORD_IS_OUT = [[
            agent:
                !dubious
                $neutralDubious
                You're not going to ask me about the Admiralty, are you?
        ]],
        OPT_CONVINCE = "Convince {agent} to inform for the Admiralty",
        DIALOG_CONVINCED = [[
            player:
                Look, what have you got to lose? 
                The Spree'll cut you loose sooner or later and then nobody'll protect you.
                Turn informant, and you can get a new start.
                In the Bread Fields, or Lakespit—heck, maybe even Supplicant City.
            agent:
                You think they'd really let someone like me into Supplicant City?
            player:
                Sure. If you were cooperative enough.
            agent:
                !sigh
                $neutralResigned
                Okay, I'll do it. I've already got enough enemies as it is. 
                But the pay better be good!
        ]],
        DIALOG_NOT_CONVINCED = [[
            player:
                Look, the Spree haven't been very nice to you lately, yeah?
                Why not stick it to 'em with a little information?
            agent:
                !angry_accuse
                $angryHostile
                No. I ain't no rat, and anyone who told you otherwise better watch themselves.
            * You're probably going to have to resort to more violent means to convince {agent} to play along.
        ]],
        DIALOG_TRY_CONVINCE = [[
            player:
                !interest
                Word on the street is you're in over your head.
            agent:
                !dubious
                That sounds like a sales pitch. You selling weapons or something?
            player:
                $miscPersuasive
                Even better. Have you considered the benefits of... Admiralty protection?
        ]],
        DIALOG_FIGHT = [[
            player:
                $neutralResigned
                I guess we'll do this the hard way.
            * You draw your weapons.
        ]],
        DIALOG_KILLED = [[
            * {agent} is dead. {agent.HeShe} is of no use to anyone anymore.
        ]],
        OPT_FIGHT = "Beat {agent} into compliance",
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE")
            :Dialog("DIALOG_TRY_CONVINCE")
            :Negotiation{
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_NOT_CONVINCED")
                    cxt:GetAgent():OpinionEvent(OPINION.TRIED_TO_RECRUIT_MOLE_AND_FAILED)
                end,
                
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_CONVINCED")
                    cxt:GetAgent():AddAspect("mole", "ADMIRALTY", true)
                    cxt.quest:Complete("find_a_new_mole")
                    -- cxt:GetAgent():OpinionEvent(OPINION.TRIED_TO_RECRUIT_MOLE_AND_SUCCEDED)
                end,
            }

        cxt:Opt("OPT_FIGHT")
            :Dialog("DIALOG_FIGHT")
            :Battle{
                on_win = function(cxt)
                    if cxt:GetAgent():IsDead() then
                        cxt:Dialog("DIALOG_KILLED")
                        cxt.quest:Fail()
                    else
                        cxt.enc:Emote( cxt:GetAgent(), "scared" )
                        cxt:Dialog("DIALOG_CONVINCED")
                        cxt:GetAgent():AddAspect("mole", "ADMIRALTY", true)
                        cxt.quest:Complete("find_a_new_mole")
                        -- cxt:GetAgent():OpinionEvent(OPINION.TRIED_TO_RECRUIT_MOLE_AND_SUCCEDED)
                    end
                    StateGraphUtil.AddEndOption(cxt)
                end,
            }


    end)


QDEF:AddConvo("return_for_reward", "handler")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_GET_PAID = "Tell {handler} about {mole}",
        DIALOG_INTRO = [[
            player:
                Well, you were right. {mole} has agreed to work with you.
                !hips
                I did my best to be the persuasive arm of your will.
            handler:
                !happy
                $happyDefault
                Oh yeah? I'd like to see you flex. 
                Nice work. I'll be sure to pick something a little more challenging next time.
                !wink
                $neutralJoke
                Wouldn't want you getting bored with me before the auction. 
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_INTRO")
            :CompleteQuest()
    end)

QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                If we're gonna keep the roads clear for the auction, I need some intel on Spree activity.
                !shrug
                Problem is, they're as paranoid right now as I am. So the usual leaks have been stopped up.
                $neutralDirect
                I need you to find me an informant—someone who will give us the goods without clapping back. 
                Think you can handle that?
        ]],
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")                    
        end)

