local LOCATION_DEF =
{
    id = "SPREE_TEMPORARY_CAMP_EXTERIOR",
    name = "Temporary Spree Camp",
    desc = "A quickly-assembled camp used by the Spree.",
    icon = engine.asset.Texture("UI/location_camp.tex"),
    map_tags = {"forest"},
    plax = "EXT_SpreeCampsite_2",
    indoors = false,
}
Content.AddLocationContent(LOCATION_DEF)

local CAMP_INTERIOR =
{
    id = "SPREE_TEMPORARY_CAMP_INTERIOR",
    indoors = true,
    name = "Spree shack",
    plax = "INT_SpreeShack_1",
    desc = "Inside a Spree's shack.",
}
Content.AddLocationContent(CAMP_INTERIOR)


local QDEF = QuestDef.Define{
        title = "Leave No-one Behind",
        desc = "{target} is being held prisoner by the Spree. {handler} wants you to either rescue or silence {target.himher}, at your discretion.",
        icon = engine.asset.Texture("icons/quests/handler_admiralty_rescue_mission.tex"),
        rank = {1, 5},
        qtype = QTYPE.CONTRACT,
        focus = QUEST_FOCUS.BALANCED,
        faction = "ADMIRALTY",

        on_init = function(quest)
            quest.param.has_flipped = math.random() > .5
        end,

        get_fight_support = function(quest, agent, target_agent)
            
            if quest:IsActive("investigate_location") and agent == quest:GetCastMember("target") then
                if quest.param.has_flipped then
                    return SUPPORT_ACTION.HINDER, SUPPORT_REASON.QUEST
                else
                    return SUPPORT_ACTION.HELP, SUPPORT_REASON.QUEST
                end
            end
        end,

        precondition = function(quest) 
            return Util.IsPlayerWorkingWithFaction("ADMIRALTY")
        end,
        
        is_in_hiding = function(quest, agent)
            if quest:IsActive("investigate_location") and agent == quest:GetCastMember("target") then
                return true
            end
        end,

    }



    :AddObjective{
        id = "investigate_location",
        title = "Investigate {the_location#location}",
        desc = "{handler} has heard rumor that {target} is being held at {the_location#location}. Go there and investigate.",
        mark = { "the_location" },
        state = QSTATUS.ACTIVE,
        
        on_complete = function( quest)
            TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("the_location"))
            quest:GetCastMember("interior"):SetMapPos(quest:GetCastMember("the_location"):GetMapPos())
        end,
        
        on_fail = function( quest)
            TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("the_location"))
        end,
        
        on_activate = function( quest)
            local location = TheGame:GetGameState():AddLocation(Location( LOCATION_DEF.id)) 
            quest:AssignCastMember("the_location", location)

            local interior = TheGame:GetGameState():AddLocation(Location( CAMP_INTERIOR.id ))
            quest:AssignCastMember("interior", interior)

        end,
        
    }

    :AddObjective{
        id = "convince_handler",
        title = "Convince {handler} that {target} didn't flip",
        negotiation_targets = {"handler"},
        desc = "{target} was actually working with the Spree, but had a change of heart when you threatened their life. Convince {handler} that everything is on the up-and-up, and you've earned yourself a nice favor-in-pocket for later.",
        mark = { "handler" },
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Report back to {handler}",
        desc = "The situation with {target} has been taken care of. Go collect your reward.",
        mark = { "handler" },
    }

    :AddCast{
        cast_id = "the_location",
        when = QWHEN.MANUAL,
        on_unassign = function( quest, location )
            TheGame:GetGameState():MarkLocationForDeletion( location )
        end,        
    }

    :AddCast{
        cast_id = "interior",
        when = QWHEN.MANUAL,
        on_unassign = function( quest, location )

            TheGame:GetGameState():MarkLocationForDeletion( location )
        end,        
    }

    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_ADMIRALTY_CONTACT",
        provider = true,
    }

    :AddCast{
        cast_id = "target",
        cast_fn = function( quest, t )
            local agent = quest:CreateSkinnedAgent( "ADMIRALTY_PATROL_LEADER" )
            table.insert(t, agent)
        end,
        
        events = {
            agent_retired = function(quest) end,
        }
    }


local PARTY_TABLE = 
{
    {"BANDIT_GOON", "BANDIT_GOON2"},
    {"BANDIT_GOON", "BANDIT_GOON2", "FLEAD"},
    {"BANDIT_RAIDER", "BANDIT_GOON2", "FLEAD"},
    {"BANDIT_RAIDER", "BANDIT_RAIDER", "FLEAD"},
    {"BANDIT_RAIDER", "BANDIT_RAIDER", "BANDIT_GOON", "FLEAD"},
}

local function DoRescue(cxt)
    local target = cxt.quest:GetCastMember("target")
    target:OpinionEvent(OPINION.SAVED_FROM_CAPTIVITY)
    target:MoveToLimbo()
    cxt.quest:Complete("investigate_location")
    cxt.quest:Activate("return_for_reward")
    StateGraphUtil.AddLeaveLocation(cxt)
end


QDEF:AddConvo("investigate_location")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("the_location") then
                return "STATE_ENCOUNTER"
            end
        end)
    :State("STATE_ENCOUNTER")
        :Loc{
            DIALOG_INTRO = [[
                * You find the bandit's hideout easily enough, and you settle yourself between two rocks: a smudge of lichen, watching and waiting. 
            ]],
            DIALOG_FLIPPED_DISCUSSION = [[
                
                * Two thugs, clearly Spree, emerge from the hideout, rubbing what seems to be ink from their hands.
                goon2:
                    !left
                goon1:
                    !right
                    !angry
                    $angryDefault
                    Hate this clerical crap. Why am I taking notes from some switch still wearing {target.hisher} blues?
                goon2:
                    !dismiss
                    $neutralWhatever
                    Forget it. The intel {target.heshe}'s giving us on the auction's worth more to Nadan than we are.
                    Anyway, you only gotta tolerate it until the escort gets here.  
                * A muffled bark from within draws one of the goon's attention back inside.
                goon2:
                    !sigh
                    $neutralResigned
                    I'll go deal with that. You keep an eye out for the rendezvous. 
                    !exit

            ]],
            DIALOG_DISCUSSION = [[
                
                * Two thugs, clearly Spree, emerge from the hideout, rubbing what seems to be blood from their hands.
                goon2:
                    !left
                goon1:
                    !right
                    !throatcut
                    $angryReluctant
                    If {target} refuses to talk, why do we gotta pull {target.hisher} nails out? Let's just bleed {target.himher} out and move on.
                goon2:
                    !point
                    $neutralDirect
                    You wanna tell Nadan we should give up on the auction, too? Just keep your head down or he'll give you a grin to match his own.
                    Word is we're passing the switch on to an "expert interrogator", anyway.
                * A piercing scream erupts from the shack. Both Spree wince.
                goon1:
                    !wince
                goon2:
                    !palm
                    $angryPatienceLost
                    I've gotta make sure they don't kill {target.himher}. You keep an eye out: the screams might attract a hungry yote.
                    !exit
                * The remaining bandit is left to ponder {goon1.hisher} work.
                goon1:
                    !sigh

            ]],
            DIALOG_PASS_STEALTH = [[
                * You manage to sneak past {goon1} while {goon1.hisher} back is turned.
            ]],
            DIALOG_FAIL_STEALTH = [[
                * Whether it's the crunch of your feet or the flare of your scarf, something gives you away.
                player:
                    !left
                goon1:
                    !angry
                    $angryHostile
                    Hey! Who's that?!
                    !fight
                * {goon1} doesn't wait for an answer before lunging towards you, weapon drawn.
            ]],

            DIALOG_GOON_SURRENDERED = [[
                goon1:
                    $scaredFearful
                    I'll be quiet! Just let me live!
                player:
                    !left
                    $miscHushedUrgent
                    Deal. "Quiet" is the operative condition here, understand?
                * You relax your grip on {goon1}'s throat and {goon1.heshe} scampers away.
                * <i>Loudly.</>
                player:
                    !facepalm
            ]],
            
            OPT_ATTACK_GOON = "Attack {goon1}",
            OPT_CONFRONT = "Confront {goon1}",
            OPT_SNEAK_PAST= "Try to sneak past {goon1}",
            DIALOG_NEGOTIATION_INTRO = [[
                player:
                    !left
                * You step onto the trail and make no effort to hide your approach. All the same, the Spree looks up with alarm.
                goon1:
                    !right
                    !surprised
                    $scaredTakenAback
                    Hesh's salty tentacles. You scared the crap outta me.
                * {goon1} looks you up and down as {goon1.heshe} brushes off the start you gave {goon1.himher}.
                goon1: 
                    !permit
                    $neutralDirect
                    You the interrogation expert? To deal with {target}?
                player:
                    !agree
                    $neutralDirect
                    That's right. 
            ]],
            DIALOG_FAIL_NEGOTIATION = [[
                player:
                    !eureka
                    Whatever it takes: broken fingers, pulled teeth, nostril hairs dipped in wax—I'm an expert in all of 'em.
                    And if I don't get the info you need it's money back guaranteed.
                goon1:
                    !suspicious
                    Never knew a bandit to have a jingle before.
                player:
                    It, uh, helps with branding. 
                    Which I also do. If you provide the iron, of course.
                goon1:
                    You're a real jester, huh?
                    $angrySeething
                    No way you're one of ours.
                * {goon1} attacks! 
            ]],
            OPT_DEFEND_SELF = "Defend yourself",

        }
        :SetLooping()
        :Fn(function(cxt) 
                if cxt:FirstLoop() then
                    cxt:Dialog("DIALOG_INTRO")
                    
                    local location = cxt.quest:GetCastMember("interior")
                    
                    cxt.quest.param.spree_fighters = CreateCombatParty( function(level)
                        return PARTY_TABLE[ clamp(cxt.quest:GetRank() - 1,1,#PARTY_TABLE) ] end,
                        cxt.quest:GetRank(), location,nil,nil, 1 )

                    cxt:ReassignCastMember( "goon1", cxt.quest.param.spree_fighters[1] )
                    cxt:ReassignCastMember( "goon2", cxt.quest.param.spree_fighters[2] )

                    if cxt.quest.param.has_flipped then
                        cxt.quest:GetCastMember("target"):GetAspect("health"):SetPercent( .25 )
                    end

                    cxt:GetCastAgent("goon1"):MoveToLocation(cxt.location)
                    cxt:GetCastAgent("goon2"):MoveToLocation(cxt.location)
                    cxt.quest:GetCastMember("target"):MoveToLocation( location )

                    cxt:Dialog( cxt.quest.param.has_flipped and "DIALOG_FLIPPED_DISCUSSION" or "DIALOG_DISCUSSION")
                    cxt:GetCastAgent("goon2"):MoveToLocation( location )
                    cxt.enc:SetPrimaryCast(cxt.quest.param.spree_fighters[1])
                end

                cxt:Opt("OPT_CONFRONT")
                    :Dialog("DIALOG_NEGOTIATION_INTRO")
                    :Negotiation{
                       
                        on_fail = function() 
                            cxt:Dialog("DIALOG_FAIL_NEGOTIATION")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function(cxt)
                                        cxt:GoTo("STATE_FIGHT_EVERYONE_OUTSIDE")
                                    end,
                                }

                        end,
                        on_success = function() 
                            cxt:GoTo("DIALOG_CHAT_WITH_OUTSIDE_GOON")
                        end,
                    }

                cxt:Opt("OPT_ATTACK_GOON")
                    :Battle{
                        advantage = TEAM.BLUE,
                        on_win = function(cxt) 
                            if not cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_GOON_SURRENDERED")
                                cxt:GetAgent():MoveToLimbo()
                            end

                            cxt:GoTo("STATE_FIGHT_EVERYONE_OUTSIDE")
                        end,
                    }
            end)

    :State("DIALOG_CHAT_WITH_OUTSIDE_GOON")
        :Loc{
            
            OPT_ASK_ABOUT_TARGET = "Ask about {target}",
            DIALOG_INTRO = [[
                goon1:
                    !laugh
                    $happyJoke
                    Took you long enough. What, you get stuck in the mud?
            ]],
            DIALOG_ASKED_ABOUT_TARGET_FLIPPED = [[
                player:
                    !interest
                    $neutralDirect
                    How is {target} faring?
                goon1:
                    !angry_shrug
                    $angryDefault
                    <i>Too good</i> if you ask me. {target.HeShe} acts like royalty just 'cus {target.heshe} gave us intel.
                    As if {target.heshe}'s the first Admiralty officer to spill their guts for pay and another day in the sun.
                    Wouldn't be a shame if someone were to rough {target.himher} up a little on {target.hisher} way out, if you ask me.
                    !spit
                    I'll be glad to see the back of {target.himher}.
            ]],
            DIALOG_ASKED_ABOUT_TARGET = [[
                player:
                    $neutralDirect
                    {target} is still alive, then?
                goon1:
                    $neutralResigned
                    Yeah, barely. {goon2}'s work is messy. But {target} still ain't talking. 
                    
            ]],
            
            OPT_DISMISS_GOON = "Tell {goon1} they can leave",
            DIALOG_GOON_LEAVES = [[
                player:
                    $miscPersuasive
                    Leave it to me. I'll take care of this.
                goon1:
                    Finally. Can't wait to get out of this place.
                    Smells like bloodflies and frogspawn.
                    !exit
                * You wait for {goon1} to make tracks down the trail before heading into the shack.
            ]]
        }

        :SetLooping(true)
        :Fn(function(cxt) 
                if cxt:FirstLoop() then
                    cxt:Dialog("DIALOG_INTRO")
                end

                cxt:Question("OPT_ASK_ABOUT_TARGET", cxt.quest.param.has_flipped and "DIALOG_ASKED_ABOUT_TARGET_FLIPPED" or "DIALOG_ASKED_ABOUT_TARGET")
                cxt:Opt("OPT_DISMISS_GOON")
                    :Dialog("DIALOG_GOON_LEAVES")
                    :Fn(function(cxt) 
                            cxt:GetAgent():MoveToLimbo()
                            cxt:GoTo("STATE_INTERIOR_GOONS")
                        end)
            end)
    
    :State("STATE_FIGHT_EVERYONE_OUTSIDE")
        :Loc{
            
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !exit
                * Hearing the commotion, the other Spree thug lurches out of the shack, ready for a fight.
                goon2:
                    !right
                    !angry
                    $angryHostile
                    What did you do to {goon1}?
                player:
                    !shrug
                    {goon1.HeShe} tripped into my knife.
                    You wanna be careful—the ground here is treacherous.
                * Unfortunately, {goon2} doesn't heed your warning.
            ]],
            DIALOG_WON_FIGHT = [[
                {dead?
                    * You notice as {goon2} falls that the soil is already slick with blood.
                    * Gross. Maybe the shack will be more hospitable.
                }
                {not dead?
                    goon2:
                        S-stop! I ain't dyin' for this!
                        You want the switch? Take {target.himher}! 
                        Nadan can get his own intel on the auction.
                    player:
                        Go on, then. You've caught me in a good mood.
                    goon2:
                        !exit
                    * {goon2} breaks for the tree line, leaving the encampment decidedly Spreeless.
                }
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :Fn(function(cxt) 
                local fighters = cxt.quest.param.spree_fighters
                for i = #fighters, 1, -1 do
                    if fighters[i]:IsDead() then
                        table.remove( fighters, i )
                    else
                        fighters[i]:MoveToLocation(cxt.location)
                    end
                end
                cxt.enc:SetPrimaryCast(fighters[1])
                
                cxt:Dialog("DIALOG_INTRO")
                
                cxt:Opt("OPT_DEFEND_SELF")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt) 
                            cxt:Dialog("DIALOG_WON_FIGHT")
                            cxt:GoTo("STATE_INTERIOR_NO_GOONS")
                        end,
                    }

            end)
    

    :State("STATE_INTERIOR_NO_GOONS")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You enter the shack, and find {target} strapped, {target.hisher} limbs rubbed raw from struggling. You free {target.himher} and {target.heshe} climbs to {target.hisher} feet.
                agent:
                    !right
                    !injured
                    $miscRelieved
                    I don't know who you are, but if it looks like a rescue and acts like a rescue...
                player:
                    !dubious
                    $neutralDubious
                    Then I guess that's what it is. {handler} sent me. You look pretty rough.
                agent:
                    !injured_shrug
                    $neutralDirect
                    I'm tougher than I look. I can make it back to headquarters from here.
                    Cover my exit, if you can manage it. And make sure to kill a few from me.
                    !exit
                * {target} leaves, a look of grim determination in {target.hisher} eyes.


            ]],
            DIALOG_INTRO_FLIPPED = [[
                player:
                    !left
                * You enter the shack, and come face to face with {target}, looking fit as a flipper. 
                target:
                    !right
                    !scared
                    $scaredFearful
                    You're not Spree. 
                player:
                    !dubious
                    $neutralDubious
                    {handler} sent me. I was supposed to <i>rescue</i> you, but it doesn't much look like you're in need.
                    !interest
                    $neutralAside
                    Not sure what {handler.heshe}'d make of this. I suspect it wouldn't be wholesome.
                target:
                    $neutralThanks
                    What {handler} doesn't know won't hurt <i>me</i>, I always say.
                    !fight
                    $angryHostile
                    This ain't personal, hunter. I just can't afford the trouble you bring.
                * {target} attacks.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :Fn(function(cxt) 
                cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("interior") )
                cxt.encounter:SetPrimaryCast(cxt.quest:GetCastMember("target"))
                
                if cxt.quest.param.has_flipped then
                    cxt:Dialog( "DIALOG_INTRO_FLIPPED" )
                    
                    cxt:Opt("OPT_DEFEND_SELF")
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            on_win = function(cxt)
                                cxt:GoTo("STATE_POST_FLIPPED_FIGHT")
                            end,
                        }   
                else
                    cxt:Dialog("DIALOG_INTRO")
                    DoRescue(cxt)
                end
                
            end)
    :State("STATE_POST_FLIPPED_FIGHT")
        :Loc{
            DIALOG_SURRENDERED = [[
                * {target} is at your mercy.
                target:
                    !scared
                    $scaredFearful
                    N-now hold on, hunter. Let's talk about this.
                    {handler} doesn't need to know anything about what I said, or what I might've been prepared to do.
                    !scared_shrug
                    $scaredPowerless
                    Why don't you just let me go back? 
                    We'll both keep nice and quiet, and then you'll have a friend in the Admiralty—that's gotta come in handy, right?
            ]],
            OPT_ACCEPT_DEAL = "Let {target} live",
            OPT_REFUSE_DEAL = "Cut {target} down",
            DIALOG_REFUSED_DEAL = [[
                player:
                    $neutralDirect
                    An Admiralty you just undermined? I'll take my friends where they are.
                target:
                    !exit
                * You kill {target} quickly, and leave {target.hisher} body in the shack for the Spree to find.
            ]],
            DIALOG_ACCEPTED_DEAL = [[
                player:
                    !sigh
                    $neutralResigned
                    Go on then. Get out of here.
                    !angry_accuse
                    $angrySeething
                    But this isn't mercy—it's an arrangement. Remember that.
                target:
                    !scared
                    $scaredTakenAback
                    Really? Thank you!
                    !agree
                    $scaredThanks
                    You won't regret this!
                    !exit
                * You wait a bit to watch for wayward Spree before following {target} out of the camp. You're going to have to explain this to {handler}.
            ]],
            DIALOG_DEAD = [[
                * {target} is dead. The Spree won't be able to use {target.himher} any more.
                * Best to report back to {handler}.
            ]],

        }
        :Fn(function(cxt) 
            

            if cxt:GetAgent():IsDead() then
                cxt.quest:Complete("investigate_location")
                cxt.quest:Activate("return_for_reward")    
                cxt:Dialog("DIALOG_DEAD")
                StateGraphUtil.AddLeaveLocation(cxt)
            else
                cxt:Dialog("DIALOG_SURRENDERED")
                cxt:Opt("OPT_ACCEPT_DEAL")
                    :Fn(function(cxt) 
                            cxt:Dialog("DIALOG_ACCEPTED_DEAL")
                            cxt:GetAgent():MoveToLimbo()
                            cxt.quest:Complete("investigate_location")
                            cxt.quest:Activate("convince_handler")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)
                cxt:Opt("OPT_REFUSE_DEAL")
                    :Fn(function(cxt) 
                            cxt:Dialog("DIALOG_REFUSED_DEAL")
                            cxt:GetAgent():Kill() --I wonder if we need to eject target from the admiralty, so that they won't mess up your reputation?
                            cxt.quest:Complete("investigate_location")
                            cxt.quest:Activate("return_for_reward")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)
            end
        end)

    :State("STATE_INTERIOR_GOONS")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You find {target} folded over in a battered heap. The Spree have been working {target.himher} over, trying to extract information.
                * One of the Spree turns from {goon2.hisher} work and confronts you.
                goon2:
                    !right
                    !interest
                    $angryTakenAback
                    You're not {goon1}. Who the Hesh are you?
                player:
                    $neutralDirect
                    I'm here to take {target} off your hands.
                goon2:
                    !dubious
                    $neutralDubious
                    Wha—? <i>You're</i> the one who's gonna crack {target.himher} open, huh? You don't look the type.
                player:
                    !crossed
                    $angryHostile
                    I'm full of surprises.
            ]],
            DIALOG_INTRO_FLIPPED = [[
                player:
                    !left
                * You enter, and see {target} relaxing in a chair, a Spree to one side of {target.himher} taking notes.
                target:
                    !right
                    !point
                    $angryHostile
                    Who are <i>you</> supposed to be?
                player:
                    !shrug
                    $neutralDubious
                    I'm the escort. For the, you know—escorting.
                target:
                    !dubious
                    $neutralDubious
                    <i>You</i> are? 
            ]],

            DIALOG_CONVINCED = [[
                goon2:
                    !sigh
                    $neutralResigned
                    Alright, {target.heshe}'s all yours.
                    !handwave
                    I'd ask to watch but honestly I've had my fill of this one. Good luck with it all.
                * {goon2} signals to the rest of the Spree, and they leave.
                goon2:
                    !exit

            ]],
            DIALOG_CONVINCED_FLIPPED = [[
                target:
                    !dubious
                    $neutralDubious
                    Huh, well, I guess you look clever enough, even if you're not too impressive otherwise.
                    !chuckle
                    $happyJoke
                    No offense or anything. I just take my safety pretty seriously.
                player:
                    !happy
                    $happyDefault
                    Oh, sure. As you should.
                goon2:
                    !right
                    !sigh
                    $neutralThanks
                    Thank Hesh. We can leave the babysitting to you. I'd give you {target.hisher} nap and feeding schedule but somehow I doubt you'll much care.
                    Later, folks. Try not to knock your head on your way out.
                * {goon2} signals to the rest of the Spree, who round up their loose possessions before heading out.
                goon2:
                    !exit
            ]],
            DIALOG_ALONE_FLIPPED = [[
                target:
                    !right
                    !happy
                    $happyDefault
                    So you've been told about my arrangement, right? I need special protection—from all sides.
                    The Admiralty are gonna be pretty cheesed with me if they found out what I'm up to, get it?
                player:
                    !interest
                    $neutralThoughtful
                    {handler} in particular would be <i>very</i> unimpressed.
                target:
                    !chuckle
                    $happyCocky
                    Hah! You can say that again. {handler.HeShe}—
                    !wince
                    $neutralDubious
                    Wait. How do you know {handler}?
                player:
                    !throatcut
                    $angryHostile
                    Like I said, {target}—I'm here to escort you.
                target:
                    !surprised
                    $angryTakenAback
                    Aw, Hesh. I knew you looked no good.
                    !fight
                * With a great bellow, {target} attacks.
                *** {target} is a traitor and needs to be dealt with. 
            ]],
            DIALOG_ALONE = [[
                * When the room has cleared, you untie {target} and carefully help {target.himher} to {target.hisher} feet.
                player:
                    Easy does it. 
                target:
                    !right
                    !injured
                    $happyDefault
                    I dunno who you are, but if it looks like a rescue and acts like a rescue...
                player:
                    !dubious
                    $neutralDubious
                    Then I guess that's what it is. {handler} sent me. You look pretty rough.
                target:
                    !injured_shrug
                    $neutralWhatever
                    I'm tougher than I look. I can make it back to headquarters from here.
                    Glad I got to see you deal with those miscreants up close and personal, at least. Thanks for that.
                    !exit
                * {target} leaves, a look of grim determination in {target.hisher} eyes.
                *** You've rescued {target}.
            ]],
            OPT_ATTACK = "Attack!",
            OPT_NEGOTIATE = "Convince {agent} that you're with the Spree",
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :Fn(function(cxt) 
                cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("interior") )
                cxt:Dialog(cxt.quest.param.has_flipped and "DIALOG_INTRO_FLIPPED" or "DIALOG_INTRO")
                cxt.enc:SetPrimaryCast(cxt.quest.param.has_flipped and cxt.quest:GetCastMember("target") or cxt.quest.param.spree_fighters[2])
                
                cxt:Opt("OPT_NEGOTIATE")
                    :Negotiation{
                        on_fail = function() 
                            cxt:GoTo(cxt.quest.param.has_flipped and "STATE_FIGHT_EVERYONE_INSIDE_FLIPPED" or "STATE_FIGHT_EVERYONE_INSIDE")                    
                        end,

                        on_success = function() 
                            cxt:Dialog(cxt.quest.param.has_flipped and "DIALOG_CONVINCED_FLIPPED" or "DIALOG_CONVINCED")
                            for k,v in pairs(cxt.quest.param.spree_fighters) do
                                v:MoveToLimbo()
                            end
                            cxt.encounter:SetPrimaryCast(cxt.quest:GetCastMember("target"))
                            cxt:Dialog(cxt.quest.param.has_flipped and "DIALOG_ALONE_FLIPPED" or "DIALOG_ALONE")
                            if cxt.quest.param.has_flipped then
                                cxt:Opt("OPT_DEFEND_SELF")
                                    :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        on_win = function(cxt) cxt:GoTo("STATE_POST_FLIPPED_FIGHT") end,
                                    }
                            else
                                DoRescue(cxt)
                            end
                        end,
                    }
            end)

    :State("STATE_FIGHT_EVERYONE_INSIDE_FLIPPED")
        :Loc{
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :Fn(function(cxt) 
                cxt:Opt("OPT_DEFEND_SELF")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt) 
                            cxt:GoTo("STATE_POST_FLIPPED_FIGHT")
                        end,
                    }
            end)
    

    :State("STATE_FIGHT_EVERYONE_INSIDE")
        :Loc{
            DIALOG_DEAD = [[
                * Looking at the carnage it's clear that {target} is dead. But at least {target.heshe} can't be used by the Spree.
                * You leave.
            ]],
            DIALOG_RESCUED = [[
                * {target} turns to you after the fight.
                target:
                    !right
                    $miscRelieved
                    You fight as bloodthirsty as they do. At least you're on my side.
                    Look me up when you get back to the city—I don't let favors go unrepaid. 
                * You both leave the shack.
            *** You've rescued {target}.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :Fn(function(cxt) 
                cxt:Opt("OPT_DEFEND_SELF")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt) 
                            cxt:Dialog(cxt.quest:GetCastMember("target"):IsDead() and "DIALOG_DEAD" or "DIALOG_RESCUED")
                            DoRescue(cxt)
                        end,
                    }

            end)


QDEF:AddConvo("return_for_reward", "handler")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_DEAD = [[
            player:
                !interest
                $neutralDirect
                So, the bad news: the Spree were trying to squeeze {target} for information.
                The good news: {target.heshe} didn't cave. They were loyal until the end. 
                !bashful
                $miscEmbarrassed
                Their end, specifically. I guess that's, uh, more bad news.
            handler:
                !thought
                $neutralThoughtful
                Wow, really? I'm not sure I would've expected that.
            player:
                $neutralDefault
                Some folks have gotta have scruples somewhere, right?
            handler:
                !dubious
                $neutralDubious
                I <i>guess</i>? I'll leave that sort of thing for lesser mortals, though.
                Still, you did what I asked. Good work.
            *** You tell {handler} that {target} is taken care of.
        ]],
        DIALOG_SURVIVED = [[
            player:
                You were right: {target} was still alive when I got there, and the Spree were trying to get intel out of {target.himher}.
                I got {target.himher} out, though, and it sounds like {target.heshe} never talked. 
            handler:
                Yep, {target.heshe} reported back just before you got here.
                !thought
                $neutralThoughtful
                And {target.heshe} <i>looks</i> sincere enough when {target.heshe} pledges {target.hisher} undying loyalty.
                My mentor would've told me to run {target.himher} through a series of increasingly alienating mind games to really test {target.hisher} commitment.
                !shrug
                But that just sounds like a lot of work to me. And ultimately, I don't really have much use for a soldier who's that inflexible anyway.
            player:
                !sigh
                You run a very strange operation, {handler}.
            handler:
                Well, obviously. Otherwise I wouldn't've been willing to work with <i>you</i>.
            player:
                Fair point.
            *** You tell {handler} that {target} remained loyal, and she seems pleased.
        ]],
        DIALOG_FLIPPED = [[
            player:
                !thought
                $neutralThoughtful
                So, you mentioned that if {target} was defecting, you wanted {target.himher} dead?
            handler:
                !dubious
                $neutralDubious
                Yes ...?
            player:
                !agree
                Yeah, {target.heshe}'s dead now.         
            handler:
                Aha. That's that taken care of, at least. I'm gonna have to disseminate some morale-boosting materials, though. {target} was popular with the troops.
                !thought
                $neutralThoughtful
                Don't suppose you could try your hand at metered verse?
            player:
                !wince
                $miscEmbarrassed
                Uh, well...
            handler: 
                !handwave
                $neutralDirect
                Nevermind, nevermind. The troops are easily distracted. They'll perk up soon. 
                Best keep quiet about the whole treachery thing, though. Some people take that sort of thing personally. 
            *** You tell {handler} that {target} is dead, and she seems satisfied.        
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog(cxt.quest.param.has_flipped and "DIALOG_FLIPPED" or (cxt.quest:GetCastMember("target"):IsDead() and "DIALOG_DEAD" or "DIALOG_SURVIVED") )
            :CompleteQuest()
    end)

QDEF:AddConvo("convince_handler", "handler")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_CONVINCE = "Convince {agent} that {target} is still loyal",
        OPT_ADMIT_DEFEAT = "Admit that you let {target} go",
    }
    :Loc{
        DIALOG_INTRO = [[
            player:
                !greeting
                $happyGreeting
                The prodigal hero has returned. I rescued {target} and return us both in mostly good condition.
                The Spree were torturing {target.himher} for information but {target.heshe} kept {target.hisher} mouth shut.
            handler:
                !dubious
                $neutralDubious
                Did {target.heshe} now? Forgive me, {player}, if I express some incredulity. 
                When you've been in this business as long as I have, loyalty is always the least likely scenario.
            *** You tell {handler} that {target} didn't talk, but she needs convincing.
        ]],
        DIALOG_CONVINCED = [[
            player:
                $neutralDirect
                As far as I can tell, {target} was legit. You can trust {target.himher}.
            handler:
                $neutralTakenAback
                Huh, well. It's unexpected but I suppose not unpleasant news. Thanks for getting them outta there.
            *** {handler} believes your report.
        ]],
        DIALOG_NOT_CONVINCED = [[
            player:
                !placate
                $miscPersuasive
                Hunter's honor, {target.heshe} was being tortured ruthlessly.
            handler:
                !facepalm
                $miscEmbarrassed
                {player}, please. I dunno if you're lying or just incompetent.
            player:
                !scared
            handler:
                I obviously had {target} interrogated as soon as {target.heshe} got in, and {target.heshe} was as treacherous as the midday sun.
                !dubious
                $neutralDubious
                Get your head in the game before I doubt its use, you hear me?
            player:
                !scared_shrug
                $scaredStammering
                Err... yeah. Sure. If you say so. You're the boss. 
            *** {handler} doesn't believe your report.
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE")
            :Dialog("DIALOG_INTRO")
            :Negotiation{
                on_success = function(cxt)  
                    cxt:Dialog("DIALOG_CONVINCED")
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                end,
                
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_NOT_CONVINCED")
                    cxt.quest:GetCastMember("target"):Retire()
                    cxt.quest:Fail()
                end,
            }
       
       cxt:Opt("OPT_ADMIT_DEFEAT")
            :Dialog("DIALOG_NOT_CONVINCED")
            :Fn(function(cxt)
                cxt.quest:GetCastMember("target"):Retire()
                cxt.quest:Fail()
            end )     
    end)

    
QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                !sigh
                $miscEmbarrassed
                One of our patrols was ambushed and as per usual the Spree took a hostage: an officer by the name of {target}.
                I know the Spree scum who ordered it. They're going to press {target.himher} for information, and, unfortunately, {target.heshe}'s got more than enough to spill. 
                !point
                $angrySeething
                I want the mess cleaned up.
            player:
                !dubious
                $neutralDubious
                Is this a rescue, or a hit?
            handler:
                !shrug
                $neutralWhatever
                Enh, use your discretion. A rescue if {target} deserves it. Otherwise? 
                Like I said: clean up.
            *** {handler} asks you to rescue or eliminate {target} before {target.heshe} can reveal information to the Spree.
        ]],
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_ON_TAKEN = [[
            handler:
                !point
                $neutralResigned
                I'm giving you license to solve this how you see fit, but that's a lot of responsibility.
                I don't get mad: just disappointed.
            player:
                !wince
                $miscEmbarrassed
                Ugh, that's worse.
            handler:
                !wink
                $neutralDirect
                Snap to it, hunter. Do me proud.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_ON_TAKEN")
        end)

