local battle_defs = require "battle/battle_defs"

local LOCATION_DEF =
{
    id = "OLD_AMBUSH_SITE",
    name = "Old Ambush Site",
    desc = "The site of a recent skirmish between the Admiralty and the Spree",
    icon = engine.asset.Texture("UI/location_forestambush.tex"),
    map_tags = {"forest"},
    plax = "EXT_Forest_1",
    indoors = false,
}

if not Content.GetLocationContent(LOCATION_DEF.id) then
    Content.AddLocationContent(LOCATION_DEF)
end

AddOpinionEvent("ATTACKED_PATROL_SURVIVOR", 
{
    delta = OPINION_DELTAS.MAJOR_BAD,
    txt = "Attacked them after they survived a Spree ambush",
})



local QDEF = QuestDef.Define{
        title = "Wounded Animals",
        desc = "The Spree used hit-and-run tactics to devastate an Admiralty patrol. The survivors are stranded in the field, and waiting for backup. Get there first and finish them off.",
        icon = engine.asset.Texture("icons/quests/handler_spree_kill_weakened_patrol.tex"),
        rank = {1, 5},
        focus = QUEST_FOCUS.COMBAT,
        faction = "BANDITS",

        qtype = QTYPE.CONTRACT,
        on_destroy = function( quest )
            if quest:GetCastMember("the_location") then
                TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("the_location"))
            end
        end,
        precondition = function(quest) 
            return Util.IsPlayerWorkingWithFaction("BANDITS")
        end,
        
    }

    :AddObjective{
        id = "wipe_out_patrol",
        title = "Kill the survivors at {the_location#location}",
        desc = "The surviving members of the patrol are at {the_location#location}. They're wounded and demoralized, but still dangerous. Proceed with caution.",
        mark = { "the_location" },
        state = QSTATUS.ACTIVE,

        on_activate = function( quest)
            quest:AssignCastMember("the_location", TheGame:GetGameState():AddLocation(Location( LOCATION_DEF.id ) ))
        end,
        
    }

    :AddCast{
        cast_id = "the_location",
        when = QWHEN.MANUAL,
        no_validation = true,
    }


    :AddObjective{
        id = "return_for_reward",
        title = "Return to {handler}",
        desc = [[{handler} has a reward for you, and possibly more work.]],
        ready_for_completion = true,
        mark = {"handler"},
    }

    :AddObjective{
        id = "explain_survivors",
        title = "Explain your actions to {handler}",
        negotiation_targets = {"handler"},
        desc = [[You left survivors at the ambush point, despite {handler}'s express orders. Go fess up to it, and try to salvage your rep.]],
        mark = {"handler"},
    }
    
    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_BANDIT_CONTACT",
        provider = true,
    }


local function DoExecute(cxt) 
    cxt:Dialog("DIALOG_EXECUTE")
    for k,v in pairs(cxt.enc.scratch.admiralty_fighters) do
         if not v:IsRetired() then 
            AgentUtil.CommitMurder(v, cxt.enc.scratch.admiralty_fighters) 
            v:Kill()
        end
    end
    cxt.quest:Complete("wipe_out_patrol")
    cxt.quest:Activate("return_for_reward")
    StateGraphUtil.AddLeaveLocation(cxt)  
end

QDEF:AddConvo("wipe_out_patrol")
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("the_location") then
                return "STATE_ENCOUNTER"
            end
        end)

    :State("STATE_ENCOUNTER")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You find the patrol right where expected. As you approach, the bedraggled survivors reach for their weapons.
                agent:
                    !right
                    !injured
                    $scaredFearful
                    Who goes there?
                
                * {agent}'s eyes dart about, looking for threats.
                * You're about to pull out your best fast talk, but it's clear {agent} isn't in the mood for diplomacy.
                
                agent:
                    !angry
                    $angryReluctant
                    Stay where you are, bandit!
                    !fight
                    $angryHostile
            ]],
            DIALOG_SURVIVOR = [[
                agent:
                    !right
                    !angry
                    $angryHostile
                    Spree scum! What are you waiting for?

                * Now you need to finish the job.
            ]],
            DIALOG_NO_SURVIVORS = [[
                agent:
                    !exit
                * Your work here is done.
            ]],
            OPT_EXECUTE = "Execute the survivors",
            DIALOG_EXECUTE = [[
                * You raise your blade. It doesn't take long.
                agent:
                    !exit
                * The clearing is quiet.

            ]],
            OPT_LET_GO = "Leave without killing the survivors",
            DIALOG_LEAVE_THE_SURVIVORS = [[
                * You start to leave, and {agent} stares at you, confused.
                agent:
                    $angrySeething
                    You expect thanks for this? 
                    My friends are dead. I'm not going to forget what you did here!
                * Are you sure you want to leave this loose end?
            ]],
            OPT_REALLY_LEAVE = "Keep walking",
            DIALOG_REALLY_LEAVE = [[
                player:
                    !exit
                * You leave, a hail of invective pelting your back. You're going to have to convince {handler} this was the right course of action.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",

        }   
        :Fn(function(cxt) 
                TheGame:GetGameState():MarkLocationForDeletion(cxt.quest:GetCastMember("the_location"))
                
                
                cxt.enc.scratch.admiralty_fighters = CreateCombatParty(
                    function(level) 
                        local PARTY_TABLE = 
                        {
                            {"ADMIRALTY_GOON", "VROC"},
                            {"ADMIRALTY_GOON", "ADMIRALTY_GOON"},
                            {"ADMIRALTY_GOON", "ADMIRALTY_GOON", "ADMIRALTY_GUARD"},
                            {"ADMIRALTY_PATROL_LEADER", "ADMIRALTY_GOON", "ADMIRALTY_GUARD"},
                            {"ADMIRALTY_PATROL_LEADER", "ADMIRALTY_GOON", "ADMIRALTY_GUARD", "VROC"},
                        }
                        return PARTY_TABLE[ math.min(#PARTY_TABLE, level) ]
                    end, cxt.quest:GetRank(), cxt.location)
                cxt.enc:SetPrimaryCast(cxt.enc.scratch.admiralty_fighters[1])    

                --make them injured
                for k,agent in pairs(cxt.enc.scratch.admiralty_fighters) do
                    agent:GetAspect("health"):SetPercent(  math.randomGauss(.4, .67) )
                end

                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_DEFEND_SELF")
                    :Battle{
                        flags = battle_defs.BATTLE_FLAGS.SELF_DEFENCE,
                        no_oppo_limit = true,
                        
                        on_start_battle = function(battle)
                            for _, fighter in pairs( battle:GetTeam(TEAM.RED):GetAllFighters()) do
                                fighter:AddCondition("WANTED_DEAD")
                            end
                        end,


                        on_win = function(cxt) 
                            
                            local survivor 
                            for k,v in ipairs(cxt.enc.scratch.admiralty_fighters) do
                                if not v:IsDead() and v:IsSentient() then
                                    survivor = v
                                    break
                                end
                            end

                            if survivor then
                                cxt.enc:SetPrimaryCast(survivor)
                                cxt:Dialog("DIALOG_SURVIVOR")
                                
                                cxt:Opt("OPT_EXECUTE")
                                    :Fn(DoExecute)

                                cxt:Opt("OPT_LEAVE")
                                    :PreIcon(global_images.close)
                                    :Fn(function(cxt) 
                                            cxt:Dialog("DIALOG_LEAVE_THE_SURVIVORS")
                                            cxt:Opt("OPT_EXECUTE")
                                                :Fn(DoExecute)
                                            
                                            cxt:Opt("OPT_REALLY_LEAVE")
                                                :Fn(function() 
                                                        for k,v in ipairs(cxt.enc.scratch.admiralty_fighters) do
                                                        if not v:IsDead() and v:IsSentient() then
                                                                v:OpinionEvent( OPINION.ATTACKED_PATROL_SURVIVOR )
                                                            end
                                                        end
                                                        cxt.quest:Complete("wipe_out_patrol")
                                                        cxt.quest:Activate("explain_survivors")
                                                        StateGraphUtil.AddLeaveLocation(cxt)
                                                    end)
                                        end)
                            else
                                cxt:Dialog("DIALOG_NO_SURVIVORS")
                                cxt.quest:Complete("wipe_out_patrol")
                                cxt.quest:Activate("return_for_reward")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end,
                        on_runaway = function() cxt.quest:Fail() end
                    }
            end)
    
    


QDEF:AddConvo("return_for_reward", "handler")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_GET_PAID = "Get paid",
    }
    :Loc{
        DIALOG_INTRO = [[
            player:
                !agree
                $neutralDirect
                The survivors met the end of their luck. 
                They should be getting a good ol' Havarian den burial right about now.
            handler:
                !cruel
                $happyThanks
                Good. That will send a message.
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_INTRO")
            :CompleteQuest()
    end)


QDEF:AddConvo("explain_survivors", "handler")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_CONVINCE = "Tell {handler} you let some survivors live",
        DIALOG_INTRO = [[
            player:
                $neutralDubious
                I - uh, let some of the survivors live.
            handler:
                $angryPatienceLost
                Why the hell would you do that? Give me one good reason!
        ]],
        DIALOG_WON = [[
            handler:
                $neutralResigned
                Well, at least they're scared. Maybe we can use that, too.
                !point
                $angryThanks
                You need to follow orders next time, if you want my help!
        ]],
        DIALOG_LOST = [[
            handler:
                $angryPatienceLost
                No, that's a terrible reason.
                When I give orders, I expect them to be followed.
                Shape up!
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE")
            :Dialog("DIALOG_INTRO")
            :Negotiation{
                on_success = function() 
                    cxt:Dialog("DIALOG_WON")
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                end,
                on_fail = function()
                    cxt:Dialog("DIALOG_LOST")
                    cxt.quest:Fail()
                end 
            }
    end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                The Admiralty has begun pushing in on Spree territory, rassling up trouble when they know damn well to keep their wet noses out of it.
                We ambushed an Admiralty patrol out in the woods—our people took some losses and ran, but not before striking a heavy blow.
                The remainder of the Admiralty patrol is still out there, waiting for rescue.
                I want you to go finish the job. It'll make the Admiralty think twice about coming into our territory.
                But make sure you kill <b>all</> of them. That's the whole point.
        ]],
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddTest{
    game_setup = function() 
        Util.SetPlayerWorkingWithFaction("BANDITS")
    end,
    tests = {
        {name = "Random", reps = 10 },
    }
}
