local QDEF = QuestDef.Define{
        title = "An Inconvenience",
        desc = "{target} is loyal to the Admiralty, and good at {target.hisher} job. That's a problem for the Spree, so they've asked you to remove {target} from {target.hisher} position one way or another.",
        icon = engine.asset.Texture("icons/quests/handler_spree_neutralize_guard.tex"),
        rank = {1, 5},
        qtype = QTYPE.CONTRACT,
        focus = QUEST_FOCUS.BALANCED,
        faction = "BANDITS",
        precondition = function(quest) 
            return Util.IsPlayerWorkingWithFaction("BANDITS")
        end,

    }

    :AddObjective{
        id = "get_target_fired",
        title = "Make {target} lose {target.hisher} job",
        desc = "The Spree wants {target} to lose {target.hisher} current position. You could get them fired, make them quit, or just wipe them off the face of the planet. Whatever's convenient.",
        mark = { "target" },
        negotiation_targets = {"target"},
        combat_targets = {"target"},
        state = QSTATUS.ACTIVE,

    }
    
    :AddObjective{
        id = "convince_worker_to_talk",
        title = "(Optional) Convince {worker} to talk",
        desc = "{worker} seems to have dirt on {target}. Perhaps you can get some useful information from {worker.himher}.",
        mark = { "worker"},
    }

    :AddObjective{
        id = "confront_target",
        title = "Make {target} lose {target.hisher} job",
        desc = "The Spree wants {target} to lose {target.hisher} current position. You could get them fired, make them quit, or just wipe them off the face of the planet. Whatever's convenient.",
        mark = { "target" },
        negotiation_targets = {"target"},
        combat_targets = {"target"},
        on_complete = function(quest) 
            if quest:IsActive("convince_worker_to_talk") then
                quest:Cancel("convince_worker_to_talk")
            end
            if quest:IsActive("confront_with_friend") then
                quest:Cancel("confront_with_friend")
            end

            quest:Activate("return_for_reward")
        end,

    }

    :AddObjective{
        id = "confront_with_friend",
        title = "Make {target} lose {target.hisher} job",
        desc = "Get rid of {target} with the help of {admiralty_friend}.",
        mark = { "target" },
        negotiation_targets = {"target"},
        combat_targets = {"target"},
        on_complete = function(quest) 
            local friend = quest:GetCastAgent( "admiralty_friend" )
            if friend and not friend:IsRetired() then
                quest:Activate("return_for_reward_friend_version")
            else
                quest:Activate("return_for_reward")
            end
        end,
    }

    :AddCast{
        cast_id = "target",
        bucket_id = "ADMIRALTY",
        condition = function(agent, quest) 
            if agent:GetBrain():GetWorkPosition() then
                local valid, reason = QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true)
                if valid then
                    return true
                end
            end
        end,
        
        events = 
        {
            agent_retired = function( quest, agent ) 
                if quest:IsActive("confront_target") then
                    quest:Complete("confront_target")
                end
                if quest:IsActive("confront_with_friend") then
                    quest:Complete("confront_with_friend")
                    quest:Activate("return_for_reward")
                end
            end,
            
            work_changed = function( quest, agent )
                if agent:GetBrain():GetWorkPosition() == nil then
                    if quest:IsActive("confront_target") then
                        quest:Complete("confront_target")
                    end
                    if quest:IsActive("confront_with_friend") then
                        quest:Complete("confront_with_friend")
                    end
                end
            end,
        }
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs= {"ADMIRALTY_GOON", "ADMIRALTY_GOON", "ADMIRALTY_GUARD", "ADMIRALTY_GUARD", "ADMIRALTY_PATROL_LEADER"}
            local def = defs[math.min(quest:GetRank(), #defs)]
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }


    :AddCast{
        cast_id = "worker",

        condition = function(agent, quest)
            return (agent.def.id == "HEAVY_LABORER" or agent.def.id == "LABORER")
        end,
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local def = "HEAVY_LABORER"
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }

    :AddCast{
        cast_id = "admiralty_friend",
        optional = true,
        condition = function(agent, quest)
            return agent:GetFactionID() == "ADMIRALTY" and agent:GetRelationship() == RELATIONSHIP.LIKED and agent:GetBrain():GetWorkPosition() == nil
        end,
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Return to {handler}",
        desc = [[{target} is out of the picture. {handler} will have a reward for you.]],
        ready_for_completion = true,
        mark = {"handler"},
    }

    :AddObjective{
        id = "return_for_reward_friend_version",
        title = "Return to {handler}",
        desc = [[{target} is out of the picture and {admiralty_friend} is willing to work with {handler}. {handler} will have a reward for you.]],
        ready_for_completion = true,
        mark = {"handler"},
    }
    
    
    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_BANDIT_CONTACT",
        provider = true,
    }

QDEF:AddConvo("return_for_reward", "handler")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_REPORT_SUCCESS = "Tell {handler} {target} is off the job",
        DIALOG_KILLED = [[
            player:
                $neutralWhatever
                {target} is not working any more.
                Or breathing.
            agent:
                $neutralDirect
                That, uh, gets the job done, I guess.
                !shrug
                $neutralWhatever
                I'm not one to question your methods.
        ]],
        DIALOG_DONE = [[
            player:
                $neutralDirect
                {target} won't be a problem any more.
            agent:
                $happyDefault
                Great. Some people just don't appreciate the way things work around here.
        ]],
        

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_REPORT_SUCCESS")
            :Dialog(cxt.quest:GetCastMember("target"):IsDead() and "DIALOG_KILLED" or "DIALOG_DONE")
            :CompleteQuest()
    end)

QDEF:AddConvo("return_for_reward_friend_version", "handler")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_REPORT_SUCCESS = "Tell {handler} {target} is off the job",
        DIALOG_KILLED = [[
            player:
                $neutralWhatever
                {target} is not working any more.
                Or breathing.
            agent:
                $neutralDirect
                That, uh, gets the job done, I guess.
                !shrug
                $neutralWhatever
                I'm not one to question your methods.
            player:
                $neutralAside
                Also, {admiralty_friend} has taken their spot and has expressed interest in working with you.
            agent:
                $happyDefault
                Huh! Two birds with one stone, great!
        ]],
        DIALOG_DONE = [[
            player:
                $neutralDirect
                {target} won't be a problem any more.
            agent:
                $happyDefault
                Great. Some people just don't appreciate the way things work around here.
            player:
                $neutralAside
                Also, {admiralty_friend} has taken their spot and has expressed interest in working with you.
            agent:
                $happyDefault
                Huh! Two birds with one stone, great!
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_REPORT_SUCCESS")
            :Dialog(cxt.quest:GetCastMember("target"):IsDead() and "DIALOG_KILLED" or "DIALOG_DONE")
            :CompleteQuest()
    end)

QDEF:AddConvo("get_target_fired")
    :Confront(function(cxt) 
        if cxt.quest:GetCastAgent("admiralty_friend") ~= nil then
            if not cxt.quest.param.met_friend then
                if cxt.location:HasTag("in_transit") then
                    return "STATE_MEET_FRIEND"
                end
            end
        end

        if cxt.location == cxt.quest:GetCastAgent("target"):GetLocation() then
            return "STATE_TARGET_INTRO"
        end
    end)   

    :State("STATE_TARGET_INTRO")
        :Loc{
            DIALOG_INTRO = [[
                worker:
                    !left
                agent:
                    !right
                    !angry
                    !point
                    $angryDefault
                    Listen, just do what you're told or else!
                    I don't have time to deal with your nonsense.
                worker:
                    !scared
                    $scaredPowerless
                    I'm sorry! I'll figure something out.. I promise!
            ]],
        }
        :Fn(function(cxt) 
            cxt.quest:GetCastAgent("worker"):MoveToLocation(cxt.location)
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("target"))
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Complete("get_target_fired")
            cxt.quest:Activate("confront_target")
            cxt.quest:Activate("convince_worker_to_talk")
            StateGraphUtil.AddEndOption(cxt)
        end)

    :State("STATE_MEET_FRIEND")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !greeting
                    $neutralGreeting
                    Hey! I've heard some rumors that {handler} isn't all too pleased with {target} and is planning to... make some changes.
                player:
                    !dubious
                    $neutralDubious
                    Well, that's an interesting rumor.
                agent:
                    $neutralDirect
                    I was just thinking, that maybe {handler} would want someone reliable to take their place and that maybe you could help out a friend.
                    !wink
                    $happyCocky
                    We could go take care of {target} together and I'll take over for them.

                    *** {agent} proposes you help {agent.himher} take {target}'s job.
            ]],
            OPT_ACCEPT_OFFER = "Accept {admiralty_friend}'s proposition",
            DIALOG_ACCEPT_OFFER = [[
                player:
                    $happyDefault
                    Sure, you help me and I'll help you. Everybody wins!
                agent:
                    $happyJoke
                    Well... Almost everyone.
            ]],
            OPT_DECLINE_OFFER = "Decline the offer and leave",
            DIALOG_DECLINE_OFFER = [[
                player:
                    $neutralThanks
                    I'd prefer to keep this simple and just get the job done.
                agent:
                    $miscDepressed
                    Oh.. I have to admit, that's a bit dissappointing but I suppose I'll get other opportunities.
            ]],
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest.param.met_friend = true
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("admiralty_friend"))
                cxt:Dialog("DIALOG_INTRO")
            end
            cxt:Opt("OPT_ACCEPT_OFFER")
                :Dialog("DIALOG_ACCEPT_OFFER")
                :Fn(function(cxt)
                    local friend = cxt:GetAgent()
                    friend:Recruit()
                    cxt.quest.param.helper = {}
                    table.insert(cxt.quest.param.helper, friend)
                    cxt.quest:Complete("get_target_fired")
                    cxt.quest:Activate("confront_with_friend")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_DECLINE_OFFER")
                :Dialog("DIALOG_DECLINE_OFFER")
                :Travel()
        end)

QDEF:AddConvo("confront_target", "target")
    :Loc{
        OPT_CONVINCE_TO_LEAVE_JOB = "Convince {target} to leave their job",
        DIALOG_TRY_CONVINCE = [[
            player:
                $miscPersuasive
                It's time you consider a new line of employment.
            
            agent:
                $scaredTakenAback
                But my work is important! The Spree must be stopped!
        ]],
        
        DIALOG_KEEP_JOB = [[
            agent:
                $neutralDubious
                You're not on the side of the Spree, are you? 
        ]],
        DIALOG_LEAVE_JOB = [[
            agent:
                !sigh
            * {agent} deflates.

            agent:
                !point
                $neutralResigned
                I'll do as you say.
                I just hope someone else picks up the torch.
        ]],
        OPT_ATTACK = "Attack {target}",
        DIALOG_ATTACK = [[
            player:
                $angryHostile
                I think it's time you retire.
            agent:
                $miscMocking
                Hah! Why don't you make me?
        ]],
        DIALOG_WON_KILLED = [[
            agent:
                !exit
                * {agent} is dead. The Spree should be pleased with that.
        ]],
        DIALOG_WON_SURRENDERED = [[
            agent:
                !injured
                $scaredFearful
                Stop! Stop! I get the message.
                I'll walk away. I swear.
                !exit
        ]],
        SIT_MOD = "Obtained blackmail information",
    }
    :Hub(function(cxt) 
        
        --cxt.quest.param.negotiation_mods = {}
        --table.insert( cxt.quest.param.negotiation_mods,  )
        local sit_mods
        
        if cxt.quest.param.obtained_blackmail then
            sit_mods =  { { value = -20, text = cxt:GetLocString("SIT_MOD") } } 
        end

        cxt:Opt("OPT_CONVINCE_TO_LEAVE_JOB")
            :Negotiation{
                situation_modifiers = sit_mods,
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_KEEP_JOB")
                end,
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_LEAVE_JOB")
                    if cxt:GetAgent():GetBrain():GetWorkPosition() then
                        cxt:GetAgent():GetBrain():GetWorkPosition():Fire()
                    end
                    cxt.quest:Complete("confront_target")
                end,
            }   
        cxt:Opt("OPT_ATTACK")
            :Dialog("DIALOG_ATTACK")
            :Battle{
                on_win = function(cxt) 
                    if cxt:GetAgent():IsDead() then
                        cxt:Dialog("DIALOG_WON_KILLED")
                    else
                        cxt:Dialog("DIALOG_WON_SURRENDERED")
                        if cxt:GetAgent():GetBrain():GetWorkPosition() then
                            cxt:GetAgent():GetBrain():GetWorkPosition():Fire()
                        end
                    end
                    cxt.quest:Complete("confront_target")
                    StateGraphUtil.AddEndOption(cxt)
                end,

            }    

    end)

QDEF:AddConvo("convince_worker_to_talk", "worker")
    :Loc{
        OPT_CONVINCE_TO_TALK = "Convince {worker} to give you information",
        DIALOG_TRY_CONVINCE = [[
            player:
                $neutralAside
                I noticed you talking to {target}. What's going on?
            agent:
                $neutralTakenAback
                That doesn't seem like any of your business.
            player:
                $miscPersuasive
                Maybe I can help you out.
        ]],
        DIALOG_REFUSE_TO_HELP = [[
            agent:
                $scaredThanks
                I'm sorry, but I can't afford to get on {target}'s bad side..
            player:
                $neutralResigned
                Urgh, useless... 
        ]],
        DIALOG_GIVE_INFO = [[
            agent:
                !sigh
            * {agent} deflates.
            agent:
                $neutralDirect
                Alright, here's the deal...
            * {agent} explains that {target} isn't who they claim to be. 
            
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE_TO_TALK")
            :Dialog("DIALOG_TRY_CONVINCE")
            :Negotiation{
                suppressed = {cxt:GetCastAgent("target")},
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_REFUSE_TO_HELP")
                end,
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_GIVE_INFO")
                    cxt.quest.param.obtained_blackmail = true
                    StateGraphUtil.AddEndOption(cxt)
                end,
            }
    end) 

QDEF:AddConvo("confront_with_friend", "target")
    :Loc{
        OPT_CONVINCE_TO_LEAVE_JOB = "Convince {target} to leave their job",
        DIALOG_TRY_CONVINCE = [[
            player:
                $angryHostile
                It's time you consider a new line of employment.
            
            agent:
                $scaredFearful
                But my work is important! The Spree must be stopped!
        ]],
        
        DIALOG_KEEP_JOB = [[
            agent:
                $neutralDubious
                You're not on the side of the Spree, are you? 
        ]],
        DIALOG_LEAVE_JOB = [[
            agent:
                !sigh
            * {agent} deflates.
            agent:
                !point
                $scaredPowerless
                I'll do as you say.
                I just hope someone else picks up the torch.
                !exit
            admiralty_friend:
                !right
                !happy
                $happyCocky
                Huh, that was easier than I had expected. Well done, {player}!
        ]],
        OPT_ATTACK = "Attack {target}",
        DIALOG_ATTACK = [[
            player:
                $angryHostile
                I think it's time you retire.
            agent:
                $miscMocking
                Hah! Why don't you make me?
        ]],
        DIALOG_WON_KILLED = [[
            agent:
                !exit
                * {agent} is dead. The Spree should be pleased with that.
            admiralty_friend:
                !right
                $neutralResigned
                This is kind of a mess, but I'll make it work. I'm glad we could help each other out!
        ]],
        DIALOG_WON_SURRENDERED = [[
            agent:
                !injured
                $scaredFearful
                Stop! Stop! I get the message.
                I'll walk away. I swear.
                !exit
            admiralty_friend:
                !right
                $happyDefault
                Looks like we wont be seeing any more of {agent.himher}! 
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE_TO_LEAVE_JOB")
            :Negotiation{
                helpers = cxt.quest.param.helper,
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_KEEP_JOB")
                end,
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_LEAVE_JOB")
                    local work = cxt:GetAgent():GetBrain():GetWorkPosition()
                    if work then
                        work:Fire()
                    end
                    local friend = cxt.quest:GetCastAgent("admiralty_friend")
                    friend:Dismiss()
                    if work then
                        work:Hire(friend)
                    end
                    friend:OpinionEvent( OPINION.FOUND_A_JOB )
                    cxt.quest:GetCastAgent("target"):MoveToLimbo()
                    cxt.quest:Complete("confront_with_friend")
                end,
            }   
        cxt:Opt("OPT_ATTACK")
            :Dialog("DIALOG_ATTACK")
            :Battle{
                allies = {cxt.quest:GetCastAgent("admiralty_friend")},
                on_win = function(cxt) 
                    local work = cxt:GetAgent() and cxt:GetAgent():GetBrain() and cxt:GetAgent():GetBrain():GetWorkPosition()
                    if cxt:GetAgent():IsDead() then
                        cxt:Dialog("DIALOG_WON_KILLED")
                    else
                        cxt:Dialog("DIALOG_WON_SURRENDERED")
                        if work then
                            work:Fire()
                        end
                        cxt.quest:GetCastAgent("target"):MoveToLimbo()
                    end
                    
                    local friend = cxt.quest:GetCastAgent("admiralty_friend")
                    if not friend:IsRetired() then
                        friend:Dismiss()
                        if work then
                            work:Hire(friend)
                        end
                        friend:OpinionEvent( OPINION.FOUND_A_JOB )
                        cxt.quest:Complete("confront_with_friend")
                    end
                    StateGraphUtil.AddEndOption(cxt)
                end,
            }    
    end)   

QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                !shrug
                $neutralThoughtful
                You know I'm happy to work with the Admiralty that can be brought to heel. But {target} is proving stubborn.
                {target}'s got influence that's useful to me. So I want {target.himher} under my thumb where {target.heshe} belongs.
                !point
                $neutralDirect
                You're not my usual witless goon. Figure it out, hunter. That's an order from your commander.
        ]],

    }

    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)

QDEF:AddConvo()
    :Loc{
        DIALOG_WHY_FIRE = [[
            player:
                I want you to fire {target}.
        ]],
        DIALOG_NO_FIRE = [[
            agent:
                No, you can't tell me how to run my business.
        ]],
        DIALOG_DO_FIRING_HERE = [[
            agent:
                Hey {target}! Get out of here!
            target:
                !right
                What did I do?
                !exit
            agent:
                !right
                I should have done that a long time ago.
        ]],
        DIALOG_DO_FIRING_REMOTE = [[
            agent:
                They're as good as fired.
        ]],
    }

    :Hub(function(cxt) 
        local target = cxt.quest:GetCastMember( "target" )
        if target and not target:IsDead() and cxt:GetAgent() == target:GetWorkBoss() then
            ConvoUtil.DoFireEmployee( cxt, target )
        end
    end)

        