local CRIMINAL_BACKUP_CHANCE = .25

AddOpinionEvent("NEGOTIATED_PRISONER_EXCHANGE", 
{
    delta = OPINION_DELTAS.MAJOR_GOOD,
    txt = "Negotiated their release from the Admiralty",
})

AddOpinionEvent("GAVE_PRISONER_EXCHANGE_REVENGE", 
{
    delta = OPINION_DELTAS.MAJOR_GOOD,
    txt = "Let them have their revenge against the Admiralty",
})

AddOpinionEvent("REFUSED_PRISONER_EXCHANGE_REVENGE", 
{
    delta = OPINION_DELTAS.MAJOR_BAD,
    txt = "Refused their vengeance against the Admiralty",
})


local locations = 
{
    {
        id = "PRISONER_EXCHANGE_SAFEHOUSE",
        name = "Spree Safehouse",
        desc = "A quickly-assembled camp used by the Spree",
        map_tags = {"forest"},
        plax = "EXT_SpreeCampsite_1",
    },

    {
        id = "PRISONER_EXCHANGE_EXCHANGE_POINT",
        name = "Exchange Point",
        desc = "A secluded spot along the shore",
        map_tags = {"beach"},
        plax = "EXT_Beach_MurderBay_1",
    }
}
for k,v in pairs(locations) do
    if not Content.GetLocationContent(v.id) then
        Content.AddLocationContent(v)
    end
end


--this should probably be done with spin-off-quests, so that they can say why they're leaving. Until then, just make it functional.
local function CleanUpFn(quest)
    if quest:GetCastMember("spree_helper") and quest:GetCastMember("spree_helper"):IsInPlayerParty() then
        quest:GetCastMember("spree_helper"):Dismiss()
    end

    if quest:GetCastMember("spree_prisoner") and quest:GetCastMember("spree_prisoner"):GetParty()  then
        quest:GetCastMember("spree_prisoner"):Dismiss()
    end
    if quest:GetCastMember("admiralty_mole") and quest:GetCastMember("admiralty_mole"):GetParty()  then
        quest:GetCastMember("admiralty_mole"):Dismiss()
    end

    if quest:GetCastMember("wanted_criminal") and quest:GetCastMember("wanted_criminal"):GetParty() then
        quest:GetCastMember("wanted_criminal"):Dismiss()
    end
end


local QDEF = QuestDef.Define{
        title = "Prisoner Exchange",
        desc = "The Spree has captured a prisoner. The Admiralty is willing to exchange prisoners if you can get them to the exchange location in one piece.",
        icon = engine.asset.Texture("icons/quests/handler_spree_prisoner_exchange.tex"),
        rank = {1, 5},
        qtype = QTYPE.CONTRACT,
        focus = QUEST_FOCUS.COMBAT,
        faction = "BANDITS",

        precondition = function(quest) 
            return Util.IsPlayerWorkingWithFaction("BANDITS")
        end,

        on_fail = CleanUpFn,
        on_complete = CleanUpFn,
        on_cancel = CleanUpFn,
        --not sure about these...
        events = 
        {
            caravan_member_remove = function(quest, agent) 
                if quest:IsActive("deliver_prisoner") and agent == quest:GetCastMember("admiralty_mole") then
                    quest:Cancel("deliver_prisoner")
                    quest:Activate("solo_rescue")
                end

                if quest:IsActive("take_spree_prisoner_home") and agent == quest:GetCastMember("spree_prisoner") then
                    quest:Cancel("take_spree_prisoner_home")
                    quest:Activate("explain_loss")
                end
            end,
        }        
    }

    :AddObjective{
        id = "pick_up_admiralty_mole",
        title = "Meet the informant",
        desc = "{admiralty_mole}, the Admiralty informant, is being held in a safehouse in the woods. Go pick {admiralty_mole.himher} up.",
        mark = { "pickup_point" },
        state = QSTATUS.ACTIVE,


        on_complete = function(quest) 
            TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("pickup_point"))
        end,

        on_activate = function(quest) 
            quest:AssignCastMember("pickup_point")
            quest:AssignCastMember("admiralty_mole")
            quest:GetCastMember("admiralty_mole"):MoveToLocation(quest:GetCastMember("pickup_point"))
        end
    }

    :AddObjective{
        id = "deliver_prisoner",
        title = "Go to {swap_point#location}",
        desc = "The Admiralty have agreed to meet at {swap_point#location}. Take {admiralty_mole} there.",
        mark = { "swap_point" },

        on_activate = function(quest) 
            quest:AssignCastMember("swap_point")
            quest:AssignCastMember("spree_prisoner")
            quest:GetCastMember("spree_prisoner"):MoveToLocation(quest:GetCastMember("swap_point"))
        end,
    }

    :AddObjective{
        id = "solo_rescue",
        title = "Meet with the Admiralty by yourself",
        desc = "You've lost your easy way in, so you might have to get creative now.",
        mark = { "swap_point" },
    }

    :AddObjective{
        id = "rescue_betrayed",
        title = "Meet with the Admiralty",
        desc = "You've lost your easy way in, but at least you have some backup.",
        mark = { "swap_point" },
    } 

    :AddObjective{
        id = "return_to_handler",
        title = "Return to {handler} for reward",
        desc = "You successfuly tricked the Admiralty, now it's time for your reward.",
        mark = { "handler" },

    }         

    :AddObjective{
        id = "explain_loss",
        title = "Talk to {handler}",
        desc = "Your rescue mission went south, and {spree_prisoner} is no longer. Go try to make good with {handler}.",
        mark = { "handler" },
        negotiation_targets = {"handler"},
    }    

    :AddObjective{
        id = "take_spree_prisoner_home",
        title = "Take {spree_prisoner} to {handler}",
        desc = "{handler} wants to personally debrief {spree_prisoner}. Bring {spree_prisoner.himher} to {handler.himher}.",
        mark = { "handler" },
        on_activate = function (quest)
            local prisoner = quest:GetCastMember("spree_prisoner")
            if not prisoner:IsInPlayerParty() then
                prisoner:Recruit( PARTY_MEMBER_TYPE.ESCORT )
                prisoner:GetParty():MoveToLeader()
            end
        end
    }
    
    :AddLocationCast{
        cast_id = "swap_point",
        when = QWHEN.MANUAL,
        cast_fn = function(quest, t)
            local location = TheGame:GetGameState():AddLocation(Location( "PRISONER_EXCHANGE_EXCHANGE_POINT" )) 
            table.insert(t, location)
        end,
        on_unassign = function( quest, agent )
            TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("swap_point"))
        end,        
    }

    :AddLocationCast{
        cast_id = "pickup_point",
        when = QWHEN.MANUAL,
        cast_fn = function(quest, t)
            local location = TheGame:GetGameState():AddLocation(Location( "PRISONER_EXCHANGE_SAFEHOUSE")) 
            table.insert(t, location)
        end,

        on_unassign = function( quest, agent )
            TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("pickup_point"))
        end,        
    }    

    :AddCast{
        cast_id = "spree_prisoner",
        when = QWHEN.MANUAL,
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent("BANDIT_RAIDER")
            agent:GetAspect("health"):SetPercent(.5)
            table.insert(t, agent)
        end,
        
        events = 
        {
            agent_retired = function( quest, agent ) 
                
            end,
        }
    }

    :AddCast{
        cast_id = "admiralty_mole",
        when = QWHEN.MANUAL,
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent("ADMIRALTY_CLERK")
            agent:GetAspect("health"):SetPercent(.5)           
            table.insert(t, agent)
        end,
        
        events = 
        {
            agent_retired = function( quest, agent ) 
            end,
        }
    }

    :AddCast{
        cast_id = "wanted_criminal",
        when = QWHEN.MANUAL,
    }
    
    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_BANDIT_CONTACT",
        provider = true,
    }

    :AddCast{
        cast_id = "spree_helper",
        when = QWHEN.MANUAL,
        optional = true,
        events = 
        {
            agent_retired = function( quest, agent ) 
            end,
        }
    }

    :AddCast{
        cast_id = "spree_guard",
        when = QWHEN.MANUAL,
        events = 
        {
            agent_retired = function( quest, agent ) 
            end,
        }
    }


QDEF:AddConvo("pick_up_admiralty_mole")
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("pickup_point") then
                return "STATE_ENCOUNTER"
            end
        end)   

    :State("STATE_ENCOUNTER")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                spree_guard:
                    !right
                    $neutralDirect
                    Hold up, there.
                    !dubious
                    $neutralDubious
                    Huh. You the hunter Nadan mentioned?
                player:
                    !shrug
                    $neutralGreeting
                    You got it. I'm here to meet with the, uh, informant?
                spree_guard:
                    !chuckle
                    $neutralJoke
                    Hah! Is that what we're calling {admiralty_mole.himher}? More like a worm slithering to avoid the spade.
                    Take {admiralty_mole.himher}, hunter. But don't hesitate to lop off {admiralty_mole.hisher} head if {admiralty_mole.heshe} slithers too far.
                admiralty_mole:
                    !right
                * The Spree bandit shoves {admiralty_mole} towards you with a force that leaves {admiralty_mole.himher} scowling.
                admiralty_mole:
                    !tired 
                    $neutralResigned
                    All right, all right! I said I'd help, didn't I?
                player:
                    !dubious
                    $neutralDubious
                    You don't sound very sure. 
                admiralty_mole:
                    !agree
                    $neutralResigned
                    Oh, I'm sure. Nadan and his goons made it plenty clear what you'd do to me if I didn't play nice.
                    !crossed
                    $angryDefault
                    We're on the same page, hunter. Just wish it bought me some fair treatment is all.
                * {spree_guard} spits at {admiralty_mole}'s feet before marching off to some new enterprise.
                admiralty_mole:
                    !handwave
                    $angryPatienceLost
                    Oystershucker. 
                    Let's go, hunter. The sooner I'm out from under this heel, the better.
                *** You've collected the Admiralty mole, {admiralty_mole}. 
            ]]
        }
        :Fn(function(cxt) 
                
                local guard = AgentUtil.GetFreeAgent("BANDIT_RAIDER")
                guard:MoveToLocation(cxt.location)
                cxt.quest:AssignCastMember("spree_guard", guard)
                cxt.enc:SetPrimaryCast(guard)

                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:GetCastMember("admiralty_mole"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                cxt.quest:Complete("pick_up_admiralty_mole")
                cxt.quest:Activate("deliver_prisoner")
                StateGraphUtil.AddLeaveLocation(cxt)
            end)



local OnWinExchangeFight = function(cxt) 
    if cxt.quest:GetCastMember("spree_prisoner"):IsDead() then
        cxt:Dialog("DIALOG_WON_FIGHT_SPREE_DIED")
        cxt.quest:Activate("explain_loss")
    else
        cxt:Dialog("DIALOG_WON_FIGHT_SPREE_LIVED")
        cxt.quest:Activate("take_spree_prisoner_home")
    end
    StateGraphUtil.AddLeaveLocation(cxt)
end


QDEF:AddConvo("deliver_prisoner")
    
    :Confront(function(cxt) 
            if not cxt.quest.param.has_spree_tried_to_kill then
                if cxt.location:HasTag("in_transit") then
                    if math.random() < 0.5 then
                        return "STATE_TRY_PRE_KILL"
                    else
                        return "STATE_WANTED_CRIMINAL"
                    end
                end
            end

            if cxt.location == cxt.quest:GetCastMember("swap_point") then
                
                return "STATE_PRISON_WITH_MOLE"
            end
        end)   

    :State("STATE_TRY_PRE_KILL")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * You're not on the road long when {agent} approaches, {agent.hisher} mouth set in a thin grim line.
                agent:
                    !crossed
                    $angryDefault
                    Hold up, hunter. I can't let you pass. 
                    !point
                    $angrySeething
                    I ain't got a problem with you, but {admiralty_mole} has too much to answer for.
                admiralty_mole:
                    !left
                    !angry_shrug
                    $angryTakenAback
                    Hey, Nadan promised I'd be sent back unharmed if I helped out. This is me helping!
                agent:
                    !angry
                    $angrySeething
                    I don't give an urchin what Nadan promised. It's blood for blood, as he damn well knows.
                    You've killed too many of mine. I'd rather watch my own skin crawl away than let <i>you</i> do it. 
                player:
                    !left
                    !placate
                    $miscPersuasive
                    Calm down. I get it, but I've been hired to do a job. 
                agent:
                    !interest
                    $angryReluctant
                    Your job is just to get {spree_prisoner} back, right? Well, that's what I want, too. 
                    !angry_threaten
                    $angryHostile
                    Let me kill {admiralty_mole}, and <i>I'll</i> help you rescue {spree_prisoner}.
                *** You're stopped by {agent}, who wants {admiralty_mole} dead.
            ]],
            DIALOG_ACCEPTED_DEAL = [[
                player:
                    !disappoint
                    $neutralResigned
                    I can't exactly fault you for wanting revenge. 
                    $neutralResigned
                    Sorry {admiralty_mole}, but {agent.heshe} and I have something in common.
                admiralty_mole:
                    !right
                    !scared
                    $scaredStammering
                    D-don't you work for Nadan? You gotta do what he would want, don't you?
                player:
                    !thought
                    $neutralDubious
                    Don't recall ever agreeing to that. This is where we part, friend. I'll go my way, and you'll—
                    !exit
                * {agent} dispatches {admiralty_mole} with a single stroke, and whatever words you might've had die along with {admiralty_mole.himher}.
                player:
                    !left
                    !wince
                    $scaredTakenAback
                    Yikes.
                agent:
                    !right
                    !tired 
                    $happyThanks
                    Thanks, hunter. I've waited too long to let Nadan decide when I should live and let live. 
                    Now, I believe we made a deal. Let's go rescue {spree_prisoner}.
                * {agent} may be the type to commit cold murder, but then, you let it happen. The two of you make a likely pair. 
                *** You allow {agent} to kill {admiralty_mole} in exchange for {agent.hisher} help.
            ]],
            DIALOG_TRY_TALK_DOWN = [[
                player:
                    !facepalm
                    $miscEmbarrassed
                    I know what it's like to want revenge, pal, but this ain't it. 
            ]],
            DIALOG_FAILED_TALK_DOWN = [[
                player:
                    !permit
                    $neutralDirect
                    You're bigger than this. You can be better.
                agent:
                    !angry_accuse
                    $angrySeething
                    Don't patronize me, hunter. You've left the moral high ground for the mire.
                    You're so happy to do Nadan's bidding? Then you can die for it, along with {admiralty_mole}.
                *** You fail to convince {agent} to back down and now {agent.heshe} is hostile.
            ]],
            DIALOG_TALKED_DOWN = [[
                player:
                    !interest
                    $neutralThoughtful
                    You kill {admiralty_mole}, and then what? Nadan comes for you. 
                    !cruel
                    $miscAlluring
                    Revenge tastes better when it's had time to stew, I say. 
                agent:
                    !agree
                    Sure, hunter. I read you loud and clear. 
                admiralty_mole:
                    !left
                    !scared
                agent:
                    !angry_accuse
                    $angryHostile
                    You hear that, {admiralty_mole}? You're getting a reprieve.
                    Once Nadan's done with you he'll leave you like so much trash. And then maybe someone tells your bosses who you <i>really</i> work for.
                    !throatcut
                    $angryCruel
                    You'll be alone. <i>Then</i> I'll find you.
                    !exit
                * {agent} leaves, but it looks like the ghost of {agent.hisher} threats might linger with {admiralty_mole} for a long time. 
                *** You convince {agent} to back off for now.
            ]],
            DIALOG_REFUSED = [[
                player:
                    $angryDefault
                    I said <i>no</i>. I don't give a seaslime <i>what</i> your sob story might be. 
                agent:
                    !angry
                    $angrySeething
                    That's a shame, hunter, 'cause it turns out I can have more than one enemy.
                *** You refuse, provoking {agent} into combat.
            ]],
            DIALOG_ATTACKED_DEAD = [[
                * The dust settles on {agent}'s fallen body. 
                admiralty_mole:
                    !right
                    !angry_shrug
                    $scaredDefault
                    This snitchin' business has been nothing but grief.
                player:
                    !disappoint
                    $neutralResigned
                    You're in it now. Get used to it. 
            ]],
            DIALOG_ATTACKED_SURVIVED = [[
                * The dust settles, leaving {agent} looking crestfallen.
                agent:
                    !injured
                    $angrySeething
                    You left me alive? Why? It doesn't change that I want {admiralty_mole} dead.
                player:
                    !point
                    $miscMocking
                    But you're not killing {admiralty_mole.himher} today. Hobble along, now.
                * {agent} spits out a wad of blood and a loose tooth, then slinks off to nurse {agent.hisher} wounds.
                admiralty_mole:
                    !right
                    !scared
                    $angryReluctant
                    Couldn't you have killed 'em? I would've preferred it.
                player:
                    !handwave
                    $neutralWhatever
                    Do your own dirty work. 
            ]],
            OPT_TALK_DOWN = "Talk {agent} out of revenge",
            OPT_AGREE_TO_DEAL = "Agree to the offer",
            OPT_REFUSE = "Refuse the offer",
        }

        :SetLooping(true)
        :Fn(function(cxt) 

                if cxt:FirstLoop() then
                    cxt.quest.param.has_spree_tried_to_kill = true
                    local agent = AgentUtil.GetFreeAgent("BANDIT_GOON")
                    cxt.quest:AssignCastMember("spree_helper", agent)
                    cxt.encounter:SetPrimaryCast(agent)
                    agent:MoveToLocation(cxt.location)

                    cxt:Dialog("DIALOG_INTRO")
                end

                cxt:Opt("OPT_TALK_DOWN")
                    :Dialog("DIALOG_TRY_TALK_DOWN")
                    :ReceiveOpinion(OPINION.NEGOTIATED_PEACE_WITH_ENEMY, nil, cxt.quest:GetCastAgent("admiralty_mole"))
                    :Negotiation{
                        on_success = function() 
                            cxt:Dialog("DIALOG_TALKED_DOWN")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,

                        on_fail = function()
                            cxt:Dialog("DIALOG_FAILED_TALK_DOWN")
                        end}
                cxt:Opt("OPT_AGREE_TO_DEAL")
                    :ReceiveOpinion(OPINION.GAVE_PRISONER_EXCHANGE_REVENGE)
                    :Fn(function(cxt) 
                            cxt:Dialog("DIALOG_ACCEPTED_DEAL")
                            cxt.quest:Cancel("deliver_prisoner")
                            cxt.quest:Activate("rescue_betrayed")
                            cxt.quest:GetCastMember("admiralty_mole"):Dismiss()
                            cxt.quest:GetCastMember("admiralty_mole"):Kill()
                            cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)
                cxt:Opt("OPT_REFUSE")
                    :Dialog("DIALOG_REFUSED")
                    :ReceiveOpinion(OPINION.REFUSED_PRISONER_EXCHANGE_REVENGE)
                    :Battle{
                        noncombatants = {cxt:GetCastAgent("admiralty_mole")},
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt)
                            local agent = cxt:GetAgent()
                            if agent:IsDead() then
                                cxt:Dialog("DIALOG_ATTACKED_DEAD")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            else
                                cxt:Dialog("DIALOG_ATTACKED_SURVIVED")
                                cxt:GetCastAgent("admiralty_mole"):OpinionEvent(OPINION.HELPED)
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end,
                    }
            end)

    :State("STATE_WANTED_CRIMINAL")
        :Loc{
            DIALOG_INTRO = [[
                * A traveller appears on the road ahead. Before you can be spotted, {admiralty_mole} grabs your elbow and pulls you into the shade.
                admiralty_mole:
                    !right
                    !overthere
                player:
                    !left
                    !surprised
                admiralty_mole:
                    $miscHushedUrgent
                    I know {agent.himher}! That's {agent}—I've seen {agent.hisher} wanted poster! We could arrest {agent.himher} to help our cover.
                *** {admiralty_mole} suggests arresting {agent} to take to the Admiralty.
            ]],
            OPT_TAKE_PRISONER = "Capture {agent}",
            DIALOG_TAKE_PRISONER = [[
                player:
                    $neutralResigned
                    Alright, but you better be sure about this!
                admiralty_mole:
                    $happyCocky
                    I'm a professional! I know a criminal when I see one. 
                * You ready your blades and move in on {agent}.
            ]],
            DIALOG_BOTH_ALIVE = [[
                agent: 
                    !right
                    !injured
                    $scaredPowerless
                    W-what the hell do you want with me, anyway?! I was just minding my own business!
                player:
                    You're under arrest, I guess. Just... keep your clapper shut. I have a headache already.
                * You gag {agent} before {agent.heshe} can make too much of a fuss.
                admiralty_mole:
                    !right
                    $happyThanks
                    !happy
                    You make arrests look easy, hunter. Good work. 
                player:
                    !dubious
                    $neutralResigned
                    Well, we're committed now, so you better be right about this.
                * With your prisoner in tow, you and {admiralty_mole} continue on your way.
                *** You and {admiralty_mole} successfully take {agent} prisoner.
            ]],
            DIALOG_CRIMINAL_DEAD = [[
                admiralty_mole:
                    !right
                    !scared
                player:
                    !left
                    !tired
                    $miscMocking
                    Don't suppose the wanted poster specified whether {agent} should be turned in <i>alive</i>, did it?
                admiralty_mole:
                    !shrug
                    $neutralWhatever
                    {agent.HeShe} resisted arrest! This sorta thing happens all the time. 
                * You and {admiralty_mole} continue on your way, looking a little more disheveled. 
            *** You and {admiralty_mole} accidentally killed {agent} instead of taking {agent.himher} prisoner.
            ]],
            DIALOG_MOLE_DEAD = [[
                * {admiralty_mole} is dead, so you'll have to do this on your own. At least you have a bargaining chip in {agent}.
            ]],
            DIALOG_BOTH_DEAD = [[
                * Both {admiralty_mole} and {agent} lie dead at your feet. You're on your own now.
            ]],
            OPT_LET_GO = "Let {agent} pass by",
            DIALOG_LET_GO = [[
                player:
                    $miscPersuasive
                    There's a lot of movin' parts in play here. Let's not make it even more complicated.
                admiralty_mole:
                    !handwave
                    $miscDepressed
                    Fine. I was just trying to be <i>helpful</i>.
                * You continue along your way, ignoring {agent} as they pass.
                *** You decide not to capture {agent}.
            ]], 
        }

        :SetLooping(true)
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.quest.param.has_spree_tried_to_kill = true
                local agent = AgentUtil.GetFreeAgent("JAKES_RUNNER")
                cxt.quest:AssignCastMember("wanted_criminal", agent)
                cxt.encounter:SetPrimaryCast(agent)
                agent:MoveToLocation(cxt.location)
                if math.random() < CRIMINAL_BACKUP_CHANCE then
                    CreateCombatBackup(cxt:GetAgent(), "JAKES_BACKUP", cxt.quest:GetRank())
                end

                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_TAKE_PRISONER")
                :Dialog("DIALOG_TAKE_PRISONER")
                :Battle{
                    allies = {cxt:GetCastAgent("admiralty_mole")},
                    advantage = TEAM.BLUE,
                    on_win = function( cxt )
                        local criminal = cxt.quest:GetCastMember("wanted_criminal")
                        local mole = cxt.quest:GetCastMember("admiralty_mole")
                        if not mole:IsDead() then
                            if criminal:IsDead() then
                                cxt:Dialog("DIALOG_CRIMINAL_DEAD")
                            else
                                cxt:Dialog("DIALOG_BOTH_ALIVE")
                                if criminal:GetParty() then
                                    criminal:Dismiss()
                                end
                                criminal:Recruit( PARTY_MEMBER_TYPE.CAPTIVE )
                            end
                        else
                            if criminal:IsDead() then
                                cxt:Dialog("DIALOG_BOTH_DEAD")
                            else
                                cxt:Dialog("DIALOG_MOLE_DEAD")
                            end
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }

            cxt:Opt("OPT_LET_GO")
                :Dialog("DIALOG_LET_GO")
                --:ReceiveOpinion( OPINION.DID_NOT_HELP )
                :Travel()
        end)

    :State("STATE_PRISON_WITH_MOLE")
        :Loc{
            DIALOG_INTRO = [[
                * You find {spree_prisoner} flanked by an Admiralty guard. They don't look keen to let {spree_prisoner.himher} go unwatched. 
                agent:
                    !right
                    $angryGreeting
                    !salute
                    Halt there, officer. That's close enough with a stranger in tow.
                player:
                    !left
                    !placate
                    $miscPersuasive
                    I'm a hunter. I've been hired to help escort a dangerous prisoner.
                admiralty_mole:
                    !left
                    !salute
                    $neutralDirect
                    My name's {admiralty_mole}. We've been asked to escort {spree_prisoner} to questioning.
                agent:
                    !interest
                    $neutralTakenAback
                    Is that so?
            ]],

            DIALOG_NOT_CRIMINAL_KILLED = [[
                admiralty_mole:
                    !left
                    By the way, we encountered {wanted_criminal} on the road. I believe I saw a wanted notice for that name.
                    !shrug
                    Unfortunately {wanted_criminal.heshe} resisted arrest, so we had to kill {wanted_criminal.himher}.
                agent:
                    !thought
                    $neutralTakenAback
                    {wanted_criminal}? I don't recall any warrant for that name. Are you sure?
                player:
                    !left
                    !crossed
                    !angry
                    Apparently not. 
                agent:
                    !angry_accuse
                    Seems you can't keep your orders straight. If you've killed a civilian on my patrol, I'm not happy. 
                *** It seems the person you killed was not a wanted criminal after all. {agent} isn't happy.
            ]],

            DIALOG_CRIMINAL_KILLED = [[
                admiralty_mole:
                    !left
                    By the way, we encountered {wanted_criminal} on the road. I believe I saw a wanted notice for that name.
                    !shrug
                    Unfortunately {wanted_criminal.heshe} resisted arrest, so we had to kill {wanted_criminal.himher}.
                agent:
                    !eureka
                    $happyDefault
                    Now <i>that's</i> handy. I'd have preferred {wanted_criminal.himher} alive, but {wanted_criminal} has been on my docket for too long.
                *** {agent} is disappointed that you killed {wanted_criminal}, but happy it's been dealt with.
            ]],
            OPT_TURN_IN_CRIMINAL = "Offer up {wanted_criminal}",
            TT_TURN_IN = "The admiralty may be swayed more easily if you hand over the prisoner",
            DIALOG_TURN_IN_CRIMINAL = [[
                agent:
                    !right
                admiralty_mole:
                    !left
                    $happyGreeting
                    !overthere
                    By the way, we found this riffraff on the road. I believe {wanted_criminal} is a wanted criminal.

            ]],
            DIALOG_TURNED_IN_INNOCENT = [[
                * {agent} takes one glance at {wanted_criminal} and immediately turns red with anger.
                agent:
                    $angryPatienceLost
                    Criminal?! That's {wanted_criminal}--my commander's <i>ward</i>.
                admiralty_mole:
                    !left
                    !scared
                    $miscEmbarrassed
                    Erp. I knew I saw that name written down somewhere. Must've been for a chellophone recital.
                player:
                    !left
                    !bashful
                    Well that's... awful unfortunate. It was an innocent mistake, I promise.
                * {agent} releases and dusts off your prisoner, sticking you with pointed glares.
                *** It seems {wanted_criminal} isn't a wanted criminal after all. {agent} isn't happy.
            ]],
            DIALOG_TURN_IN_ACCEPT = [[
                agent:
                    !eureka
                    $happyThanks
                    That's good news! {wanted_criminal} has been a real thorn in my boot. 
                * You hand over the prisoner and {admiralty_mole} winks in your direction when {agent} isn't looking.
                *** You hand over {wanted_criminal}, pleasing {agent}.
            ]],
            OPT_REQUEST_PRISONER = "Request that they send over {spree_prisoner}",
            DIALOG_REQUEST_PRISONER = [[
                player:
                    !left
                    $miscPersuasive
                    So if you just hand over {spree_prisoner}, we'll get out of your hair and be on our way.
                agent:
                    $neutralDubious
                    That's it, huh?
            ]],
            DIALOG_REQUEST_WON = [[ 
                player: 
                    $miscPersuasive
                    !interest
                    You must be sick of dealing with that scumbag. Bet you can't wait to get {spree_prisoner.himher} out of your jurisdiction.
                agent:
                    !shrug
                    $angryReluctant
                    You have no idea. These Spree reprobates make every job hell. 
                    !eureka
                    $neutralDirect
                    You know what? Good riddance. Here's hoping you can torture something useful out of 'em.
                * {agent} gestures to a nearby guard, who sends {spree_prisoner} over with a painful kick. 
                spree_prisoner:
                    !right
                    !spit 
                    $angrySeething
                    You're dead, switch! And this hunter dog of yours, too!
                player:
                    !cruel
                    $happyJoke
                    An hour in my company, {spree_prisoner}, and you'll surely change your tune.
            ]],           
            DIALOG_REQUEST_DENIED = [[
                agent:
                    $angryHostile
                    Not good enough.
                * {agent} reaches for a dagger.
            ]],
            DIALOG_WON_FIGHT_MOLE_HELPED_DIED = [[
                * {admiralty_mole} didn't make it through that scrap, but at least {admiralty_mole.heshe} had your back.
                spree_prisoner:
                    !right
                player:
                    !left
                    $miscRelieved
                    Alright, let's get moving and get you back home!
                spree_prisoner:
                    $happyThanks
                    No objections here! Let's get the hesh out of here.
            ]],
            DIALOG_WON_FIGHT_MOLE_HELPED_LIVED = [[
                admiralty_mole:
                    !right
                player:
                    !left
                    $happyTakenAback
                    Gotta admit, I had my doubts about you, but you really proved yourself back there.
                admiralty_mole:
                    $miscDepressed
                    Why does no one ever trust me?
                player:
                    !dubious
                    $neutralDubious
                    You're kidding, right? 
                * Your group of three quickly vacate the premises before any more backup can arrive.
            ]],
            DIALOG_WON_FIGHT_MOLE_NO_HELP_DIED = [[
                * {admiralty_mole} turned on you when you needed {admiralty_mole.himher} most, but at least they got what they had coming.
                spree_prisoner:
                    !right
                player:
                    !left
                    $neutralWhatever
                    Are you good to go?
                spree_prisoner:
                    $neutralResigned
                    I've been ready since I got here, let's go!
            ]],
            DIALOG_WON_FIGHT_MOLE_NO_HELP_LIVED = [[
                * {admiralty_mole} turned on you when you needed {admiralty_mole.himher} most, {admiralty_mole.heshe}'s lucky you're merciful.
                spree_prisoner:
                    !right
                player:
                    !left
                    $neutralWhatever
                    Are you good to go?
                spree_prisoner:
                    $neutralResigned
                    I've been ready since I got here, let's go!
            ]],
            OPT_ATTACK = "Attack the guards",
            DIALOG_ATTACK = [[
                player:
                    $angryPatienceLost
                    Urgh, I don't have time for this.
                * You draw your daggers and make your move.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
            SIT_MOD_KILLED_INNOCENT = "Killed an innocent person",
            SIT_MOD_KILLED_CRIMINAL = "Killed a wanted criminal",
            SIT_MOD_TURNED_IN_INNOCENT = "Turned in an innocent person",
            SIT_MOD_TURNED_IN_CRIMINAL = "Turned in a wanted criminal",
            SIT_MOD_COLLEAGUE_APPROVES = "Their colleague approves of you",
            SIT_MOD_COLLEAGUE_DISLIKES = "Their colleague dislikes you",

        }
        :SetLooping()
        :Fn(function(cxt) 
            local criminal = cxt:GetCastAgent("wanted_criminal")
            if cxt:FirstLoop() then
                local opfor = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.quest.param.opfor = opfor
                cxt.enc:SetPrimaryCast(opfor[1])
                
                local mole = cxt:GetCastAgent("admiralty_mole")

                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("deliver_prisoner")
                if cxt.quest.param.killed_criminal then
                    if math.random() < 0.2 then
                        cxt:Dialog("DIALOG_NOT_CRIMINAL_KILLED")
                        cxt.enc.scratch.negotiation_mods = (cxt.enc.scratch.negotiation_mods or {})
                        table.insert(cxt.enc.scratch.negotiation_mods, {value = 10, text = cxt:GetLocString("SIT_MOD_KILLED_INNOCENT")})
                    else
                        cxt:Dialog("DIALOG_CRIMINAL_KILLED")
                        cxt.enc.scratch.negotiation_mods = (cxt.enc.scratch.negotiation_mods or {})
                        table.insert(cxt.enc.scratch.negotiation_mods, {value = -5, text = cxt:GetLocString("SIT_MOD_KILLED_CRIMINAL")})
                    end
                end
                if criminal and criminal:IsInPlayerParty() then
                    if math.random() < 0.2 then
                        cxt.quest.param.captured_innocent = true
                    else
                        cxt.quest.param.captured_innocent = false
                    end     
                end

                local backup = {}
                if mole:GetRelationship() > RELATIONSHIP.NEUTRAL then
                    table.insert(backup, mole)
                end
                cxt.quest.param.combat_backup = backup
                if cxt:GetCastAgent("admiralty_mole"):GetRelationship() > RELATIONSHIP.NEUTRAL then
                    cxt.enc.scratch.negotiation_mods = (cxt.enc.scratch.negotiation_mods or {})
                    table.insert(cxt.enc.scratch.negotiation_mods, {value = -5, text = cxt:GetLocString("SIT_MOD_COLLEAGUE_APPROVES")})
                elseif cxt:GetCastAgent("admiralty_mole"):GetRelationship() < RELATIONSHIP.NEUTRAL then
                    cxt.enc.scratch.negotiation_mods = (cxt.enc.scratch.negotiation_mods or {})
                    table.insert(cxt.enc.scratch.negotiation_mods, {value = 5, text = cxt:GetLocString("SIT_MOD_COLLEAGUE_DISLIKES")})
                end
            end

            local turned_in = false
            if criminal and criminal:IsInPlayerParty() and not turned_in then
                cxt:Opt("OPT_TURN_IN_CRIMINAL")
                    :PostText("TT_TURN_IN")
                    :Dialog("DIALOG_TURN_IN_CRIMINAL")
                    :Fn(function(cxt)
                        if cxt.quest.param.captured_innocent then
                            cxt:Dialog("DIALOG_TURNED_IN_INNOCENT")
                            cxt.enc.scratch.negotiation_mods = (cxt.enc.scratch.negotiation_mods or {})
                            table.insert(cxt.enc.scratch.negotiation_mods, {value = 10, text = cxt:GetLocString("SIT_MOD_TURNED_IN_INNOCENT")})
                        else
                            cxt:Dialog("DIALOG_TURN_IN_ACCEPT")
                            cxt.enc.scratch.negotiation_mods = (cxt.enc.scratch.negotiation_mods or {})
                            table.insert(cxt.enc.scratch.negotiation_mods, {value = -10, text = cxt:GetLocString("SIT_MOD_TURNED_IN_CRIMINAL")})
                        end
                        turned_in = true
                        criminal:Dismiss()
                    end)
            end

            cxt:Opt("OPT_REQUEST_PRISONER")
                :Dialog("DIALOG_REQUEST_PRISONER")
                :Negotiation{
                    situation_modifiers = cxt.enc.scratch.negotiation_mods,
                    
                    on_success = function(cxt)
                        cxt.quest:Complete("deliver_prisoner")
                        cxt.quest:Activate("take_spree_prisoner_home")
                        cxt:Dialog("DIALOG_REQUEST_WON")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,

                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_REQUEST_DENIED")
                        cxt:Opt("OPT_DEFEND_SELF")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                allies = cxt.quest.param.combat_backup,
                                on_win = function(cxt)
                                    cxt.quest:Complete("deliver_prisoner")
                                    cxt.quest:Activate("take_spree_prisoner_home") 
                                    local mole = cxt:GetCastAgent("admiralty_mole")
                                    if mole:IsDead() then
                                        if table.contains(cxt.quest.param.combat_backup, mole) then
                                           cxt:Dialog("DIALOG_WON_FIGHT_MOLE_HELPED_DIED")
                                        else
                                           cxt:Dialog("DIALOG_WON_FIGHT_MOLE_NO_HELP_DIED")
                                        end
                                    else
                                        if table.contains(cxt.quest.param.combat_backup, mole) then
                                           cxt:Dialog("DIALOG_WON_FIGHT_MOLE_HELPED_LIVED")
                                        else
                                           cxt:Dialog("DIALOG_WON_FIGHT_MOLE_NO_HELP_LIVED")
                                        end
                                    end
                                    if cxt:GetCastAgent("wanted_criminal") and cxt:GetCastAgent("wanted_criminal"):IsInPlayerParty() then
                                        cxt:GetCastAgent("wanted_criminal"):Dismiss()
                                    end
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end,
                            }
                    end,
                    }

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    advantage = TEAM.BLUE,
                    allies = cxt.quest.param.combat_backup,
                    on_win = function(cxt)
                        cxt.quest:Complete("deliver_prisoner")
                        cxt.quest:Activate("take_spree_prisoner_home") 
                        local mole = cxt:GetCastAgent("admiralty_mole")
                        if mole:IsDead() then
                            if table.contains(cxt.quest.param.combat_backup, mole) then
                                cxt:Dialog("DIALOG_WON_FIGHT_MOLE_HELPED_DIED")
                            else
                                cxt:Dialog("DIALOG_WON_FIGHT_MOLE_NO_HELP_DIED")
                            end
                        else
                            if table.contains(cxt.quest.param.combat_backup, mole) then
                                cxt:Dialog("DIALOG_WON_FIGHT_MOLE_HELPED_LIVED")
                            else
                                cxt:Dialog("DIALOG_WON_FIGHT_MOLE_NO_HELP_LIVED")
                            end
                        end
                        if cxt:GetCastAgent("wanted_criminal") and cxt:GetCastAgent("wanted_criminal"):IsInPlayerParty() then
                            cxt:GetCastAgent("wanted_criminal"):Dismiss()
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }
                
        end)

QDEF:AddConvo("solo_rescue")
    :Confront(function(cxt) 
        if cxt.location == cxt.quest:GetCastMember("swap_point") then
            return "STATE_SOLO_PRISON"
        end
    end)   
    :State("STATE_SOLO_PRISON")
        :Loc{
            DIALOG_INTRO = [[
                * You find {spree_prisoner} flanked by an Admiralty guard. They don't look keen to let {spree_prisoner.himher} go unwatched. 
                agent:
                    !right
                    $angryGreeting
                    That's close enough.
                player:
                    !left
                    $miscPersuasive
                    I'm here to pick up {spree_prisoner} and escort them to the other facility.
                agent:
                    $angryReluctant
                    You're not admiralty. Who the hesh are you?
            ]],
            OPT_PRISONER_TRADE = "Offer {wanted_criminal} as a trade",
            DIALOG_TRADE_PRISONERS = [[
                player:
                    $miscPersuasive
                    How about this, you send over {spree_prisoner} and in return, I'll give you {wanted_criminal}?
                agent:
                    $neutralThoughtful
                    That doesn't seem like a good trade..
            ]],
            OPT_CONVINCE = "Convince {agent} to send over {spree_prisoner}",
            DIALOG_CONVINCE = [[
                player:
                    $miscPersuasive
                    Listen, I have orders to get {spree_prisoner}
                agent:
                    $neutralDubious
                    This all seems a bit fishy..
            ]],
            DIALOG_CONVINCE_FAILED = [[
                agent:
                    $angryPatienceLost
                    I don't believe a word of it!
                * {agent} pulls out {agent.hisher} weapon and attacks.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                agent:
                    !shrug
                    $neutralWhatever
                    Everything seems to check out, I guess.
                player:
                    $happyCocky
                    Exactly! Just business as usual.
                agent:
                    $neutralDirect
                    Here you go.
                * {agent} gestures to a guard behind him who sends over {spree_prisoner}.
            ]],
            OPT_ATTACK = "Attack {agent}",
            DIALOG_ATTACK = [[
                player:
                    $angryPatienceLost
                    Urgh, I don't have time for this.
                * You draw your daggers and make your move.
            ]],
            DIALOG_WON_FIGHT = [[
                * You grab {spree_prisoner} and leave before anyone else can arrive.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                local opfor = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.quest.param.opfor = opfor
                cxt.enc:SetPrimaryCast(opfor[1])
                cxt:Dialog("DIALOG_INTRO")
            end

            if cxt.quest:GetCastMember("wanted_criminal") and cxt.quest:GetCastMember("wanted_criminal"):IsInPlayerParty() then
                cxt:Opt("OPT_PRISONER_TRADE")
                    :Dialog("DIALOG_TRADE_PRISONERS")
                    :Fn(function(cxt)
                        cxt.quest:GetCastMember("spree_prisoner"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest:Complete("solo_rescue")
                        cxt.quest:Activate("take_spree_prisoner_home")
                    end)

            else
                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation{
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                            cxt.quest:GetCastMember("spree_prisoner"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            cxt.quest:Complete("solo_rescue")
                            cxt.quest:Activate("take_spree_prisoner_home")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,

                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_FAILED")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function(cxt)
                                        cxt:Dialog("DIALOG_WON_FIGHT")
                                        cxt.quest:Complete("solo_rescue")
                                        cxt.quest:Activate("explain_loss")
                                    end,
                                }
                        end,
                    }
            end

            cxt:Opt("OPT_ATTACK")
                :Battle{
                    advantage = TEAM.BLUE,
                    on_win = function(cxt)
                        cxt:Dialog("DIALOG_WON_FIGHT")
                        cxt.quest:Complete("solo_rescue")
                        cxt.quest:Activate("take_spree_prisoner_home")
                    end,
                }         
        end)

QDEF:AddConvo("rescue_betrayed")
    :Confront(function(cxt)
        if cxt.location == cxt.quest:GetCastMember("swap_point") then
            return "STATE_PRISON_WITH_HELP"
        end
    end)
    :State("STATE_PRISON_WITH_HELP")
        :Loc{
            DIALOG_INTRO = [[
                * You find {spree_prisoner} flanked by an Admiralty guard. They don't look keen to let {spree_prisoner.himher} go unwatched. 
                agent:
                    !right
                    $angryGreeting
                    That's close enough.
                player:
                    !left
                    $miscPersuasive
                    I'm here to pick up {spree_prisoner} and escort them to the other facility.
                agent:
                    $angryReluctant
                    You're not admiralty. Who the hesh are you?
            ]],
            OPT_BETRAY_SPREE = "Offer {spree_helper} in exchange for {spree_prisoner}",
            DIALOG_BETRAY_SPREE = [[
                player:
                    $miscPersuasive
                    How about we make a deal?
                agent:
                    !dubious
                    $neutralDubious
                    I'm listening...
                player:
                    $neutralDirect
                    You can take {spree_helper} here and in exchange you give me {spree_prisoner}.
                spree_helper:
                    !right
                    $angryTakenAback
                    What?! Hang on a minute.
            ]],
            TT_BETRAY_SPREE = "Turning over {spree_helper} could help sway the guards.",
            OPT_CONVICE_GO_QUIETLY = "Convince {spree_helper} to go quietly",
            DIALOG_CONVINCE_GO_QUIETLY = [[
                * You pull {spree_helper} aside and talk quietly.
                player:
                    $miscHushedUrgent
                    Listen, {handler} needs {spree_prisoner}. Sometimes you have to take one for the team, right?
                spree_helper:
                    $angryTakenAback
                    Take one for the team?!
            ]],
            DIALOG_GOING_QUIETLY = [[
                spree_helper:
                    $neutralResigned
                    Fine.. I'm doing this for {handler}, not you!
                player:
                    $neutralDirect
                    I'll tell {handler} of your sacrifice.
                spree_helper:
                    !exit
                * {spree_helper} drops {spree_helper.hisher} weapons and walks over to the guards
                player:
                    !exit
            ]],
            DIALOG_NOT_GOING = [[
                spree_helper:
                    $angryHostile
                    No way, I'm turning myself in!
                player:
                    $neutralResigned
                    The hard way it is then.
                * You pull out your weapons and prepare for battle. The guards seem excited but have no intention of interfering.
            ]],
            OPT_GET_HELP = "Supress {spree_helper} with the help of the guards",
            DIALOG_GET_HELP = [[
                * You pull out your weapons and address the guards.
                agent:
                    !right
                player:
                    $miscPersuasive
                    A little help here?
                agent:
                    $neutralResigned
                    Oh hesh, I guess we're doing it the rough way.
            ]],
            DIALOG_BEAT_SPREE_KILLED = [[
                player:
                    !left
                agent:
                    !right
                    $angryHostile
                    Listen, this doesn't look too good for me.
                    If you keep quiet about this, I'll give you {spree_prisoner}, deal?
                player:
                    $happyThanks
                    Deal. My lips are sealed!
                * You grab {spree_prisoner} and head out before {agent} can change {agent.hisher} mind.
            ]],
            DIALOG_BEAT_SPREE_LIVED = [[
                agent:
                    !right
                player:
                    !left  
                    $neutralThanks           
                    Sorry for the trouble, but we have a deal, right?
                agent:
                    $neutralDirect
                    Of course, a deal is a deal.
                * {agent} hands over {spree_prisoner} and the two of you leave.
            ]],
            DIALOG_EVERYONE_DEAD = [[
                player:
                    !left
                * You're the sole survivor of that fight but {spree_prisoner} was wise enough to stay out of it.
                spree_prisoner:
                    !right
                player:
                    $neutralDirect
                    You ready to go?
                spree_prisoner:
                    $happyThanks
                    I've been ready since the second I got here!
                * The two of you take off and leave the mess behind
            ]],
            DIALOG_GUARDS_DEAD_SPREE_LIVED = [[
                player:
                    !left
                agent:
                    !right
                    $neutralResigned
                    Urgh, hard work is the worst.
                player:
                    $neutralDirect
                    Well, you got your man. Time to hand over {spree_prisoner}.
                agent:
                    $happyThanks
                    Of course! Glad to be rid of {spree_prisoner.himher}!
                * You make your exit before {agent.heshe} can change {agent.hisher} mind.
            ]],
            OPT_FAKE_HOSTAGE = "Pretend that {spree_helper} is holding you hostage",
            DIALOG_FAKE_HOSTAGE = [[
                player:
                    $miscPersuasive
                    Listen, just hand over {spree_prisoner} or {spree_helper} is gonna kill me!
                spree_helper:
                    !left
                    $angryHostile
                    That's right! Hand {spree_prisoner.himher} over right now!
                agent:
                    $angryReluctant
                    How do I know this isn't a ruse?
                player:
                    !left
                    $miscPersuasive
                    Please, you have to believe me.
            ]],
            DIALOG_FAKE_HOSTAGE_CONVINCED = [[
                agent:
                    $neutralWhatever
                    Alright, alright! Just take it easy, I don't need blood being spilt on my watch.
                player:
                    $miscRelieved
                    Thank you! You've saved my life!
                * {agent} sends over {spree_prisoner} and watches the three of you until you're out of sight.
            ]],
            DIALOG_FAKE_HOSTAGE_FAILED = [[
                agent:
                    $angrySeething
                    You're lying! You're not gonna con me!
                player:
                    !sigh
                    $neutralWhatever
                    Well, it was worth a shot..
                * You pull out your weapons and move to attack.
            ]],
            DIALOG_ASSISTANCE_DIED_PRISONER_LIVED = [[
                spree_prisoner:
                    $happyTakenAback
                    Thanks for saving me! It's a shame about {spree_helper}.
                * The two of you take off before any more guards can arrive.
            ]],
            DIALOG_FIGHT_BOTH_LIVED = [[
                * The three of you leave the scene of the crime in a hurry.
            ]],
            OPT_ATTACK = "Attack {agent}",
            DIALOG_ATTACK = [[
                player:
                    $angryAside
                    Urgh, I don't have time for this.
                * You draw your daggers and make your move.
            ]],
            DIALOG_WON_FIGHT = [[
                * You grab {spree_prisoner} and leave before anyone else can arrive.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                local opfor = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.quest.param.opfor = opfor
                cxt.enc:SetPrimaryCast(opfor[1])
                cxt:Dialog("DIALOG_INTRO")
            end
            cxt:Opt("OPT_BETRAY_SPREE")
                :PostText("TT_BETRAY_SPREE")            
                :Fn(function(cxt)
                    cxt.enc:SetPrimaryCast(cxt:GetCastAgent("spree_helper"))
                    cxt:Dialog("DIALOG_BETRAY_SPREE")
                    cxt:Opt("OPT_CONVICE_GO_QUIETLY")
                        :Dialog("DIALOG_CONVINCE_GO_QUIETLY")
                        :Negotiation{
                            on_success = function(cxt)
                                cxt:Dialog("DIALOG_GOING_QUIETLY")
                                cxt.quest:GetCastMember("spree_prisoner"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                cxt.quest:Complete("rescue_betrayed")
                                cxt.quest:Activate("take_spree_prisoner_home")
                            end,

                            on_fail = function(cxt)
                                cxt:Dialog("DIALOG_NOT_GOING")
                                cxt:Opt("OPT_ATTACK")
                                    :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        on_win = function(cxt)
                                            local survivors = {}
                                            for k,v in ipairs(cxt.quest.param.opfor) do
                                                if not v:IsDead() then
                                                    if v:IsSentient() then
                                                        table.insert(survivors, v)
                                                    end
                                                end
                                            end

                                            cxt.enc:SetPrimaryCast(cxt.quest.param.opfor[1])

                                            if cxt.quest:GetCastMember("spree_helper"):IsDead() then
                                                if #survivors == 0 then
                                                    cxt:Dialog("DIALOG_EVERYONE_DEAD")
                                                else
                                                    if cxt:GetAgent():IsDead() then
                                                        cxt.enc:SetPrimaryCast(survivors[1])
                                                    end
                                                    cxt:Dialog("DIALOG_BEAT_SPREE_KILLED")
                                                end
                                            else
                                                if #survivors == 0 then
                                                    cxt:Dialog("DIALOG_GUARDS_DEAD_SPREE_LIVED")
                                                else
                                                    if cxt:GetAgent():IsDead() then
                                                        cxt.enc:SetPrimaryCast(survivors[1])
                                                    end
                                                    cxt:Dialog("DIALOG_BEAT_SPREE_LIVED") 
                                                end
                                            end
                                            cxt.quest:Complete("rescue_betrayed")
                                            cxt.quest:Activate("take_spree_prisoner_home")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end,
                                    }
                            end, 
                        }

                    cxt:Opt("OPT_GET_HELP")
                        :Dialog("DIALOG_GET_HELP")
                        :Fn(function(cxt)
                            local admiralty_allies = cxt.quest.param.opfor
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                    noncombatants = {cxt:GetCastAgent("spree_prisoner")},
                                    allies = admiralty_allies,
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function(cxt)
                                        cxt.enc:SetPrimaryCast(cxt.quest.param.opfor[1])
                                        if cxt.quest:GetCastMember("spree_helper"):IsDead() then
                                            cxt:Dialog("DIALOG_BEAT_SPREE_KILLED")
                                            cxt.quest:Complete("rescue_betrayed")
                                            cxt.quest:Activate("explain_loss") 
                                        else
                                            cxt:Dialog("DIALOG_BEAT_SPREE_LIVED")
                                            cxt.quest:Complete("rescue_betrayed")
                                            cxt.quest:Activate("take_spree_prisoner_home") 
                                        end
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end,
                                }
                        end)
                end)

            cxt:Opt("OPT_FAKE_HOSTAGE")
                :Dialog("DIALOG_FAKE_HOSTAGE")
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_FAKE_HOSTAGE_CONVINCED")
                        cxt:GetCastAgent("spree_helper"):Dismiss()
                        cxt.quest:Complete("rescue_betrayed")
                        cxt.quest:Activate("return_to_handler")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,

                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_FAKE_HOSTAGE_FAILED")
                        cxt:Opt("OPT_DEFEND_SELF")
                            :Battle{
                                enemies = cxt.quest.param.opfor,
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                on_win = function(cxt)
                                    local assistance = cxt.quest:GetCastMember("spree_helper")
                                    if assistance:IsDead() then
                                        cxt:Dialog("DIALOG_ASSISTANCE_DIED_PRISONER_LIVED")
                                        cxt.quest:Complete("rescue_betrayed")
                                        cxt.quest:Activate("take_spree_prisoner_home")
                                    else
                                        cxt:Dialog("DIALOG_FIGHT_BOTH_LIVED")
                                        cxt.quest:Complete("rescue_betrayed")
                                        cxt.quest:Activate("take_spree_prisoner_home")
                                    end
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end,
                            }
                    end,
                }

            cxt:Opt("OPT_ATTACK")
                :Battle{
                    advantage = TEAM.BLUE,
                    on_win = function(cxt)
                        local assistance = cxt.quest:GetCastMember("spree_helper")
                        if assistance:IsDead() then
                            cxt:Dialog("DIALOG_ASSISTANCE_DIED_PRISONER_LIVED")
                            cxt.quest:Complete("rescue_betrayed")
                            cxt.quest:Activate("take_spree_prisoner_home")
                        else
                            cxt:Dialog("DIALOG_FIGHT_BOTH_LIVED")
                            cxt.quest:Complete("rescue_betrayed")
                            cxt.quest:Activate("take_spree_prisoner_home")
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }         

        end)


--spree helper will leave you if the rescue has failed.
QDEF:AddConvo("explain_loss")
    :Confront(function(cxt) 
            if cxt.quest:GetCastMember("spree_helper") and cxt.quest:GetCastMember("spree_helper"):IsInPlayerParty() then
                return "STATE_HELPER_LEAVE"
            end
        end)

    :State("STATE_HELPER_LEAVE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                spree_helper:
                    !right
                    So I guess we're done, then.
                    Don't mention me to {handler}, OK?
                    !exit
                * {spree_helper} slinks off.
            ]]
        }

        :Fn(function(cxt)
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:GetCastMember("spree_helper"):Dismiss()
                StateGraphUtil.AddEndOption(cxt)
            end)



QDEF:AddConvo("explain_loss", "handler")
    :Loc{
        DIALOG_TRY_EXPLAIN = [[
            handler:
                $angryPatienceLost
                Where is {spree_prisoner}?
            player:
                $miscPersuasive
                I can explain...    
        ]],
        OPT_EXPLAIN_LOSS = "Explain what happened to {spree_prisoner}",
        DIALOG_GOOD_EXPLAIN = [[
            handler:
                $neutralResigned
                You did as best as could be expected, I guess.
                At least {spree_prisoner}'s secrets are safe.
        ]],
        DIALOG_FAILED_EXPLANATION = [[
            handler:
                $angryPatienceLost
                Not good enough. You were given a job, and you failed.
                Good bye.
        ]],
    }
    :Hub(function(cxt) 
            cxt:Opt("OPT_EXPLAIN_LOSS")            
                :Dialog("DIALOG_TRY_EXPLAIN")
                :Negotiation{
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_GOOD_EXPLAIN")
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_FAILED_EXPLANATION")
                        cxt.quest:Fail()
                        StateGraphUtil.AddEndOption(cxt)
                    end,
                }
        end)


QDEF:AddConvo("return_to_handler")
    :Confront(function(cxt) 
            if cxt.quest:GetCastMember("handler"):GetLocation() == cxt.location then
                return "STATE_ENCOUNTER"
            end            
        end)   
    :State("STATE_ENCOUNTER")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                handler:
                    !right
                * {handler} looks up as you enter.
                handler:
                    $happyGreeting
                    Great, you're here!
                    {spree_prisoner} turned up a while ago. I'm glad you got {spree_prisoner.himher} back.
            ]],
            DIALOG_PST = [[
                spree_prisoner:
                    !right
                    $happyThanks
                    Still can't believe that was a rescue mission.
                    Normally the boss's plans end with a lot more bloodshed on both sides.
                    Thanks for your help back there.
                    I'll see you around, grifter.
                    !exit
            ]],

            DIALOG_PST_HELPER = [[
                spree_helper:
                    !right
                    $happyThanks
                    Yeah, be seeing you.
                    !exit
            ]]
        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete()                
                ConvoUtil.GiveQuestRewards(cxt)

                cxt:Dialog("DIALOG_PST")
                cxt.quest:GetCastMember("spree_prisoner"):OpinionEvent(OPINION.NEGOTIATED_PRISONER_EXCHANGE)
                cxt.quest:GetCastMember("spree_prisoner"):Dismiss()

                if cxt.quest:GetCastMember("spree_helper") and cxt.quest:GetCastMember("spree_helper"):IsInPlayerParty() then
                    cxt:Dialog("DIALOG_PST_HELPER")
                    cxt.quest:GetCastMember("spree_helper"):Dismiss()
                end
                
                StateGraphUtil.AddEndOption(cxt)
            end)



QDEF:AddConvo("take_spree_prisoner_home")
    :Confront(function(cxt) 
            if cxt.quest:GetCastMember("spree_prisoner"):IsInPlayerParty() and cxt.quest:GetCastMember("handler"):GetLocation() == cxt.location then
                return "STATE_ENCOUNTER"
            end            

        end)   


    :State("STATE_ENCOUNTER")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                handler:
                    !right
                * {handler} looks up as you enter.
                handler:
                    $happyThanks
                    I see you've brought back {spree_prisoner} in one piece.
                    Good work.
            ]],
            DIALOG_PST = [[
                spree_prisoner:
                    !right
                    $happyThanks
                    Thanks for your help back there.
                    I'll see you around, grifter.
                    !exit
            ]],

            DIALOG_PST_HELPER = [[
                spree_helper:
                    !right
                    $happyThanks
                    Yeah, be seeing you.
                    !exit
            ]]


        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete()                
                ConvoUtil.GiveQuestRewards(cxt)

                cxt:Dialog("DIALOG_PST")
                cxt.quest:GetCastMember("spree_prisoner"):OpinionEvent(OPINION.NEGOTIATED_PRISONER_EXCHANGE)
                cxt.quest:GetCastMember("spree_prisoner"):Dismiss()

                if cxt.quest:GetCastMember("spree_helper") and cxt.quest:GetCastMember("spree_helper"):IsInPlayerParty() then
                    cxt:Dialog("DIALOG_PST_HELPER")
                    cxt.quest:GetCastMember("spree_helper"):Dismiss()
                end
                
                StateGraphUtil.AddEndOption(cxt)
            end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                !crossed
                $angryDefault
                The Admiralty took down one of my operations. Most of my people got out, save one. 
                I'd cut my losses, but that's the one that had important intel in their brain. Intel I still need. 
                !point
                $angrySeething
                So I need you to bust them outta jail. 
            player:
                !angry_shrug
                $angryHostile
                That's a <i>big ask</i>, don't you think? A jailbreak?
            handler:
                $miscPersuasive
                Relax, salty. I've got someone on the inside, an officer so deep in my pocket they'll never worm their way out.
            player:
                !neutral
            handler:
                $neutralDirect
                You just roll up stinkin' like a hunter and pretend you've been hired to collect.
                You do, then bring 'em on back. And I'm that much closer to likin' you. 
            player:
                !dubious
                $neutralDubious
                Lucky me.
            handler: 
                !cruel
                $angrySeething
                You said it.
            *** Nadan wants you to work with an Admiralty mole to release a Spree prisoner. 
        ]],
    }

    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)
