local CHANCE_FOR_VENDOR = .4
local FAKE_RELIC_PRICE = 50 -- Per Level
local HAGGLE_DISCOUNT = .25
local CHANCE_FOR_LUMINARI = .35


local QDEF = QuestDef.Define{
        title = "Shake Down",
        qtype = QTYPE.CONTRACT,
        rank = {1, 5},

        desc = "{handler} wants you to intercede and destroy a relic being transported to the Cult of Hesh.",
        icon = engine.asset.Texture ("icons/quests/handler_spree_shake_down.tex" ),
        focus = QUEST_FOCUS.NEGOTIATION,
        faction = "BANDITS",
        precondition = function(quest) 
            return Util.IsPlayerWorkingWithFaction("BANDITS")
        end,

        on_init = function(quest)
            quest.param.vendor_fate = math.random() < CHANCE_FOR_VENDOR
            quest.param.luminari_fate = math.random() < CHANCE_FOR_LUMINARI
        end

    }
    
    :AddObjective{
        id = "prepare_ambush",
        title = "Prepare to ambush the convoy",
        desc = "The convoy transporting the relic will be following a set route, intercept and ambush it.",
        mark = { "admiralty_route"},
        state = QSTATUS.ACTIVE,
        negotiation_targets = {"target"},
        combat_targets = {"target"},
        on_complete = function(quest)
            quest:Activate("return_for_reward")
            if quest:IsActive("swap_fake") then
                quest:Cancel("swap_fake")
            end
        end,
        on_activate = function( quest)
            local location = quest:SpawnTempLocation("ADMIRALTY_ROUTE")
            quest:AssignCastMember("admiralty_route", location )
        end,
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Return to {handler} for your reward",
        desc = "The job is done and it's time for your reward.",
        mark = {"handler"},
        ready_for_completion = true,
    }

    :AddObjective{
        id = "swap_fake",
        title = "(Optional) Swap the relics",
        desc = "(Optional) You could use the fake relic you purchased to swap out with the real one.",
    }

    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_BANDIT_CONTACT",
        provider = true,
    }

    :AddCast{
        cast_id = "vendor",
        condition = function(agent, quest)
            return agent.def.id == "WEALTHY_MERCHANT"
        end
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local def = "WEALTHY_MERCHANT"
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }

    :AddCast{
        cast_id = "luminari",
        condition = function(agent, quest)
            return agent.def.id == "LUMINARI"
        end
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local def = "LUMINARI"
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }

    :AddCast{
        cast_id = "guard",
        condition = function(agent, quest)
            return QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) and agent:GetFactionID() == "ADMIRALTY"
        end
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local def = "ADMIRALTY_GUARD"
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }

    :AddLocationCast{
        cast_id = "admiralty_route",
        when = QWHEN.MANUAL,
        no_validation = true,
    }

    :AddLocationDefs{
        ADMIRALTY_ROUTE = {
            name = "Admiralty Road",
            desc = "The planned route for the convoy carrying the relic",
            plax = "EXT_Road01",
            desc = "The Admiralty Convoy will be travelling along this road at some point today",
            indoors = false,
            map_tags = {"road"},
        }

    }
    


QDEF:AddConvo("prepare_ambush")
    :Confront(function ( cxt )
        if cxt.quest:GetCastAgent("vendor") ~= nil then
            if not cxt.quest.param.vendor_check then
                if cxt.location:HasTag("in_transit") then
                    cxt.quest.param.chose_vendor = cxt.quest.param.chose_vendor or cxt.quest.param.vendor_fate
                    if cxt.quest.param.chose_vendor then 
                        return "STATE_MEET_VENDOR"
                    end
                    
                end
            end
        end

        if cxt.location == cxt.quest:GetCastMember("admiralty_route") then
            return "STATE_AMBUSH_INTRO"
        end
    end)

    :State("STATE_MEET_VENDOR")
        :Loc{
            DIALOG_INTRO = [[
                * As you're walking down the road, you spot a merchant standing by a small stall.
                player:
                    !left
                agent:
                    !right
                    !greeting
                    $happyGreeting
                    Greetings! Are you interested in buying a genuine ancient antiquity? A truly invaluable piece of history.
                player:
                    !dubious
                    $neutralDubious
                    These all look a bit... homemade.
                agent:
                    !placate
                    $happyTakenAback
                    No, no. All my items are absolutely authentic! Believe me!
                * You don't.
            ]],
            OPT_BUY_RELIC = "Buy the relic",
            DIALOG_BUY_RELIC = [[
                * You hand the money to {agent} and {agent.heshe} passes you the alleged relic, clearly glad to be rid of it.
                agent:
                    !happy
                    $happyThanks
                    You're gonna love it! It's a real accent piece!
                * You handled antiquities all the time growing up. You can tell at a glance that this is nothing more than pretty garbage. But it might convince someone.
                player:
                    !happy
                    $neutralJoke
                    Truly a bargain at any price.
            ]],
            OPT_HAGGLE = "Haggle for a better price ({1#money})",
            DIALOG_ATTEMPT_HAGGLE = [[
                player:
                    !interest
                    $neutralDubious
                    It's a bit pricey, isn't it?
                agent:
                    !dubious
                    $neutralTakenAback
                    It's a one-of-a-kind relic! I'm not sure what you'd expect.
            ]],
            DIALOG_HAGGLE_SUCCESS = [[
                player:
                    !overthere
                    $miscPersuasive
                    You and I both know this isn't legit. I'll give you a fair price for it. 
                agent:
                    !surprise
                    $angryTakenAback
                    I've never been so insulted in my life! How dare you besmirch my product!
                player:
                    !crossed
                    ...
                agent:
                    ... 
                    !disappoint
                    $neutralResigned
                    All right, fine. But you tell anyone who asks it's the real deal!
                player:
                    $happyThanks
                    Oh don't worry—I intend to do exactly that.
            ]],
            DIALOG_HAGGLE_FAILURE = [[
                player:
                    !point
                    $neutralDubious
                    It's down-right immoral to make a profit off relics of the faith!
                agent:
                    !chuckle
                    $happyDefault
                    Hah! Tell that to the Cult of Hesh. If you have a problem with my practices, you can take it up in prayer. 
            ]],
            OPT_LEAVE = "Decline",
            DIALOG_LEAVE = [[
                player:
                    !handwave
                    $neutralThanks
                    On second thought, I think I'm good. 
                agent:
                    !sigh
                    $neutralResigned
                    Clearly the faith doesn't call to you. Walk in the shallows, friend!
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)



            if cxt:FirstLoop() then
                local vendor = cxt.quest:GetCastAgent("vendor")
                vendor:MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(vendor)
                cxt.quest.param.met_vendor = true
                cxt.quest.param.fake_relic_price = math.floor(FAKE_RELIC_PRICE * cxt.quest:GetRank())
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest.param.vendor_check = true
            end



            cxt:Opt("OPT_BUY_RELIC")
                :DeliverMoney(cxt.quest.param.fake_relic_price)
                :Dialog("DIALOG_BUY_RELIC")
                :Fn(function(cxt)
                    cxt.quest.param.has_fake = true
                    cxt.quest:Activate("swap_fake")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            if not cxt.quest.param.has_haggled then
                local reduced_amount_actual = CalculatePayment(cxt:GetAgent(), cxt.quest.param.fake_relic_price * (1 - HAGGLE_DISCOUNT))
                cxt:Opt("OPT_HAGGLE", reduced_amount_actual)
                    :Dialog("DIALOG_ATTEMPT_HAGGLE")
                    :Negotiation{
                        on_success = function()
                            cxt.quest.param.fake_relic_price = math.floor( cxt.quest.param.fake_relic_price * (1 - HAGGLE_DISCOUNT))
                            cxt.quest.param.has_haggled = true
                            cxt:Dialog("DIALOG_HAGGLE_SUCCESS")
                        end,
                        on_fail = function()
                            cxt.quest.param.has_haggled = true
                            cxt:Dialog("DIALOG_HAGGLE_FAILURE")
                        end,
                    }
            end

            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :Travel()
        end)

    :State("STATE_AMBUSH_INTRO")
        :Loc{
            DIALOG_INTRO = [[
                * You find the route {handler} mentioned and lurk out of sight. It's not long before an ostentatious caravan approaches.
                * The caravan is escorted by a few guards, and a Luminari—the most dangerous of the group. 
                guard:
                    !right
                luminari:
                    !left
                    $neutralAside
                    Let's rest a moment. We've traveled all day and I wish to reflect in prayer.
                guard:
                    $happyDefault
                    As you like, Luminari.
            ]],
            OPT_WAIT = "Wait a bit and observe the group",
            DIALOG_WAIT = [[
                * You hold your breath and wait. The guard begins to sharpen {guard.hisher} weapon on a leather strap.
                luminari:
                    !sigh
                    $neutralThoughtful
                    I'm sorry, but I can't focus while you work. I'll seek solitude in the forest.
                guard:
                    !scared_shrug
                    $neutralTakenAback
                    I-I'm sorry, Luminari. I can—
                luminari:
                    !placate
                    $neutralThanks
                    No. You have your duties, and I have mine. Guard the relic, please. 
                guard:
                    !salute
                luminari:
                    !exit
                * With the Luminari out of the way, you emerge from your hiding spot.  
                player:
                    !left
                    !wave
                    $happyGreeting
                    Why, greetings! 
                guard:
                    !point
                    $neutralDirect
                    Hold, citizen. This is Admiralty business.
                player:
                    !happy
                    $happyDefault
                    Just a question, my good fellow! Do you keep the faith? Do you walk in the shallows?
                guard:
                    !dubious
                    $neutralDubious
                    Of course. We're none of us faithless here. What's it to you, citizen?
            ]],
            OPT_ATTACK_GUARDS = "Attack the guards",
            DIALOG_ATTACK_GUARDS = [[
                player:
                    $angryHostile
                    Just thought you might like to say your prayers before you die.
                    !fight
                * You unsheath your weapons and pounce.
            ]],
            DIALOG_GUARDS_DEAD = [[
                * You dust yourself off and quickly head for the caravan.
            ]],
            DIALOG_GUARDS_SURVIVED = [[
                * You look around menacingly, and the remaining guards take off. Clearly their faith doesn't protect them from the likes of you.
            ]],
            OPT_ASK_TO_INSPECT = "Ask to see the relic",
            DIALOG_ASK_TO_INSPECT = [[
                player:
                    !overthere
                    $miscPersuasive
                    I'm a pilgrim and a scholar. An expert in antiquities, in fact.
                    I was just in Grout Bog, where I made a frightful discovery—several recently-unearthed relics were fakes!
                    $miscHopeless
                    The Spark Barons told me they'd just sold one to the Admiralty, and I knew you had to be found.
                guard:
                    !interest
                    $neutralTakenAback
                    Fakes?! The Spark Barons assured us it was perfectly authentic.
                player:
                    !interest
                    $miscRelieved
                    So you <i>do</i> carry the relic? Well, I could certainly examine it for you, to be sure.
            ]],
            DIALOG_ASK_SUCCESS = [[
                player:
                    !thumb
                    $miscPersuasive
                    I've dedicated my whole life to the study of relics. My eye won't be fooled, I can assure you.
                guard:
                    !thought
                    $neutralThoughtful
                    Well... I suppose there's no harm in having you look at it.
                player:
                    !happy
                    $happyThanks
                    My fingers will be as discreet as a dew drop.
                    !exit
                guard:
                    !exit
                * {guard} steps aside. 
            ]],
            DIALOG_ASK_FAILURE = [[
                player:
                    $miscPersuasive
                    If you just give me a moment alone with the relic, I could—
                guard:
                    !angry
                    $angryDefault
                    Alone with the relic? Do you think we're stupid?
                player:
                    !sigh
                    !neutralResigned
                    Actually... yeah.
                guard:
                    !angry
                    $angryHostile
                    What?!
                player:
                    !angry_shrug
                    $angryPatienceLost
                    'Cause if you'd been smart, you'd've taken the chance when you had it.
                    !cruel 
                    $angryCruel
                    Now I'm gonna have to kill you.
                * You pull out your weapons and attack.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                local luminari = cxt.quest:GetCastAgent("luminari")
                luminari:MoveToLocation(cxt.location)
                local guard = cxt.quest:GetCastAgent("guard")
                guard:MoveToLocation(cxt.location)
                CreateCombatBackup(guard, "ADMIRALTY_PATROL_BACKUP", cxt.quest:GetRank())
                cxt.enc:SetPrimaryCast(guard)
                cxt:Dialog("DIALOG_INTRO")
                luminari:MoveToLimbo()
            end

            if not cxt.quest.param.has_waited then
                cxt:Opt("OPT_WAIT")
                    :Dialog("DIALOG_WAIT")
                    :Fn(function(cxt)
                        cxt.quest:GetCastAgent("luminari"):MoveToLimbo()
                        cxt.quest.param.has_waited = true
                    end)
            else
                cxt:Opt("OPT_ATTACK_GUARDS")
                    :Dialog("DIALOG_ATTACK_GUARDS")
                    :Battle{
                        advantage = TEAM.BLUE,
                        on_win = function()
                            if cxt.quest:GetCastAgent("guard"):IsDead() then
                                cxt:Dialog("DIALOG_GUARDS_DEAD")
                            else
                                cxt:Dialog("DIALOG_GUARDS_SURVIVED")
                            end
                            cxt:GoTo("STATE_GUARDS_GONE")
                        end
                    }

                cxt:Opt("OPT_ASK_TO_INSPECT")
                    :Dialog("DIALOG_ASK_TO_INSPECT")
                    :Negotiation{
                        on_success = function()
                            cxt:Dialog("DIALOG_ASK_SUCCESS")
                            cxt:GoTo("STATE_INSPECT")
                        end,
                        on_fail = function()
                            cxt:Dialog("DIALOG_ASK_FAILURE")
                            cxt:Opt("OPT_ATTACK_GUARDS")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function()
                                        if cxt.quest:GetCastAgent("guard"):IsDead() then
                                            cxt:Dialog("DIALOG_GUARDS_DEAD")
                                        else
                                            cxt:Dialog("DIALOG_GUARDS_SURVIVED")
                                        end
                                        cxt:GoTo("STATE_GUARDS_GONE")
                                    end
                                }
                        end
                    }
            end
        end)

    :State("STATE_GUARDS_GONE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !cagey
                * You crawl inside the caravan and spot an ornate, eelskin box. The relic sits on a bed of silk that feels like tallow.
                * Looking at the relic, it's clear it's genuine—but it's a genuine trinket. No wonder the Spark Barons were willing to pass it on.
                *  Still, the Heshians might like the gift all the same.
            ]],
            OPT_SWAP = "Swap the relics",
            DIALOG_SWAP = [[
                * It's a simple switch to swap out the relic for the fake one you bought earlier. There's not much difference between them, except for one is thousands of years old, and the other...
                * You quickly buff out a merchant's mark with a bit of spit.
                player:
                    !happy
                    !hips
                    !neutralAside
                    There. No one will know the difference... probably. 
                * You slip out of the caravan and take off down the road before anyone can spot you.
                player:
                    !exit
            ]],
            OPT_STEAL = "Steal the relic",
            DIALOG_STEAL = [[
                * The relic fits easily into your pocket, and you slip out of the caravan and back towards the woods before anyone notices.
                player:
                    !exit
            ]],
            OPT_DESTROY = "Destroy the relic",
            DIALOG_DESTROY = [[
                * The relic is old and snaps easily in two; for good measure, you even grind the pieces beneath your heel. 
                player:
                    !happy
                    $neutralAside
                    Oops.
                * You always hated relics anyway. With a sneer, you slip out of the caravan and back into the woods before anyone notices.
                player:
                    !exit

            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")

            if cxt.quest.param.has_fake then
                cxt:Opt("OPT_SWAP")
                    :Dialog("DIALOG_SWAP")
                    :Fn(function(cxt)
                        cxt.quest.param.has_relic = true
                        cxt.quest:Complete("swap_fake")
                        cxt.quest:Complete("prepare_ambush")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end

            cxt:Opt("OPT_STEAL")
                :PreIcon(global_images.item)
                :Dialog("DIALOG_STEAL")
                :Fn(function(cxt)
                    cxt.quest:Complete("prepare_ambush")
                    cxt.quest.param.has_relic = true
                    cxt.quest.param.stole_relic = true
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_DESTROY")
                :PreIcon(global_images.rob)
                :Dialog("DIALOG_DESTROY")
                :Fn(function(cxt)
                    cxt.quest.param.destroyed_relic = true
                    cxt.quest:Complete("prepare_ambush")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

        end)

    :State("STATE_INSPECT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !cagey
                * You crawl inside the caravan and spot an ornate, eelskin box. The relic sits on a bed of silk that feels like tallow.
                * Looking at the relic, it's clear it's genuine—but it's a genuine trinket. No wonder the Spark Barons were willing to pass it on.
                * Still, the Heshians might like the gift all the same.
            ]],
            OPT_SWAP = "Swap the relics",
            DIALOG_SWAP = [[
                * It's a simple switch to swap out the relic for the fake one you bought earlier. There's not much difference between them, except for one is thousands of years old, and the other...
                * You quickly buff out a merchant's mark with a bit of spit.
                guard:
                    !right
                    $neutralDirect
                    Everything in order?
                player:
                    !happy
                    $happyThanks
                    Yes, actually! I feared the worst, but it seems this relic <i>is</i> genuine.
                    !miscRelieved
                    The Spark Barons were mistaken.
                guard:
                    !cruel
                    !neutralJoke
                    Wouldn't be the first time. Still, that's a relief. 
                player:
                    !overthere
                    $neutralThanks
                    I exist to serve, officer. Now I must return to my calling. 
                    Good day.
                player:
                    !exit
                guard:
                    !exit
                * You duck your head and exit the caravan before anyone can look too closely at the relic you left behind. 
                
            ]],
            OPT_STEAL = "Steal the relic",
            DIALOG_STEAL = [[
                * The relic slips easily into your pocket, and you exit the caravan looking prim as you can manage.
                player:
                    !left
                guard:
                    !right
                    !angry_shrug
                    $neutralDirect
                    Well? We carrying a fake or what?
                player:
                    !happy
                    $happyDefault
                    I have good news, soldier: it's genuine. I was blessed to even be in its presence. 
                guard:
                    $angryDefault
                    That's a relief... now scram, civilian. Because now you <i>really</i> have no reason to be here.
                player:
                    !exit
                guard:
                    !exit
                * You give a simpering bow before quickly making your exit.
            ]],
            OPT_DESTROY = "Destroy the relic",
            DIALOG_DESTROY = [[
                * The relic is old and snaps easily in two; for good measure, you even grind the pieces beneath your heel. 
                player:
                    !happy
                    $neutralAside
                    Oops.
                * You always hated relics anyway. With a sneer, you slip out of the caravan.
                player:
                    !left
                guard:
                    !right
                    !dubious
                    $neutralDubious
                    Well? You weren't in there very long.
                player:
                    !crossed
                    $neutralDirect
                    A moment was all I needed—the relic was indeed a fake.
                guard:
                    !surprised
                    $scaredTakenAback
                    What?! Aw, Hesh.
                player:
                    !angry_accuse
                    $angryDefault
                    I wouldn't take the many-tentacled watcher's name in vain if I were you. 
                    !neutral
                    $neutralDirect
                    The fake was heresy, so naturally, I destroyed it.
                guard:
                    $angryTakenAback
                    Y-you—
                player:
                    !salute
                    $neutralDubious
                    And with that I'll be off—frankly I don't want to keep the company of someone so unholy.
                    !exit
                guard:
                    !exit
                * You make for the trees before the guards can recover. 
            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")

            if cxt.quest.param.has_fake then
                cxt:Opt("OPT_SWAP")
                    :Dialog("DIALOG_SWAP")
                    :Fn(function(cxt)
                        cxt.quest.param.has_relic = true
                        cxt.quest:Complete("swap_fake")
                        cxt.quest:Complete("prepare_ambush")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end

            cxt:Opt("OPT_STEAL")
                :Dialog("DIALOG_STEAL")
                :Fn(function(cxt)
                    cxt.quest:Complete("prepare_ambush")
                    cxt.quest.param.has_relic = true
                    cxt.quest.param.stole_relic = true
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_DESTROY")
                :Dialog("DIALOG_DESTROY")
                :Fn(function(cxt)
                    cxt.quest.param.destroyed_relic = true
                    cxt.quest:Complete("prepare_ambush")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)

QDEF:AddConvo("return_for_reward")
    :Confront(function(cxt)
        if not cxt.quest.param.interrupted then
            if cxt.location:HasTag("in_transit") then
                cxt.quest.param.chose_luminari = cxt.quest.param.chose_luminari or cxt.quest.param.luminari_fate
                if cxt.quest.param.chose_luminari and (cxt.quest.param.stole_relic or cxt.quest.param.destroyed_relic) then 
                    return "STATE_LUMINARI"
                elseif cxt.quest.param.has_relic and cxt.quest.param.met_vendor and cxt.quest.param.has_fake then
                    return "STATE_VENDOR_RETURN"
                end
            end
        end
    end)
    :State("STATE_LUMINARI")
        :Loc{
            DIALOG_INTRO_STOLE = [[
                * Unfortunately, the road is long and your luck is short. You're soon joined by a familiar face from the caravan.
                player:
                    !left
                    !scared
                agent:
                    !right
                    !angry_accuse
                    $angrySeething
                    Return the relic now, heathen. It's not too late to wipe the slate clean. 
                    Though we couldn't say the same of your conscience.
                player:
                    !cruel
                    $miscMocking
                    Aw, are you sad you lost the Admiralty's little bribe?
                agent:
                    $angryDefault
                    I don't care at all about the Admiralty. I know their motivations are as insidious as yours. 
                    !throatcut
                    But that relic belongs with the Cult. And I would see it returned... by any means. 
                    
            ]],
            DIALOG_INTRO_DESTROYED = [[
                * Unfortunately, the road is long and your luck is short. You're soon joined by a familiar face from the caravan.
                player:
                    !left
                    !greeting
                    Well, hi there, Luminari! You walking in those shallows?
                agent:
                    !right
                    !angry_accuse
                    $angrySeething
                    You destroyed a priceless relic. Now you mock me, heathen?
                player:
                    !shrug
                    $neutralDirect
                    You and your relic just got caught up in a feud between the Admiralty and the Spree.
                    $miscMocking
                    I am but the messenger—just like you and your god, Luminari.
            ]],
            OPT_DONATE = "Give the relic directly to {agent}",
            DIALOG_DONATE = [[
                player:
                    !crossed
                    $miscPersuasive
                    Look, all I care about is cutting out the Admiralty. And I've done that.
                    $miscAlluring
                    So how about I give you the relic, but you remember where it <i>really</i> came from?
                agent:
                    !thought
                    $neutralThoughtful
                    Hm. Manipulative. But what is Hesh if not a slippery god?
                    Very well, hunter. I accept your deal. You give me the relic, and I let you walk free.
                    !agree
                    What's more, the Admiralty will be remembered for what they are: useless.

            ]],
            OPT_SELL = "Offer to sell the relic to {agent}",
            DIALOG_SELL = [[
                player:
                    !overthere
                    $miscPersuasive
                    Look, you lost the relic. But I have good news: it's for sale.
                agent:
                    !cruel
                    $angryCruel
                    And why should I pay when I can just take it from your corpse, little hunter?
                    Hesh itself will be on my side. 
            ]],
            DIALOG_SELL_SUCCESS = [[
                player:
                    $miscPersuasive
                    If the Admiralty gave this relic to the Heshians, then that would be a favor owed.
                    !cruel
                    The Cult would be <i>tainted</i> by bribery. Me? I'm just a free operator. You pay me, and our business is done. 
                agent:
                    !thought
                    $neutralThoughtful
                    That's not a bad point, heathen. 
                    Very well. I have a budget for acquisitions. You have a deal. 
            ]],
            DIALOG_SELL_FAIL = [[
                player:
                    !happy
                    $happyCocky
                    Hesh works in mysterious ways, don't it?
                agent:
                    !angry_accuse
                    $angryPatienceLost
                    How <i>dare</i> you suggest the watery god works through you. 
            ]],
            OPT_CONVINCE = "Convince {agent} to let you go",
            DIALOG_CONVINCE = [[
                player:
                    $neutralResigned
                    Hey, I dug up relics for a living. Trust me when I say this is nothing more than a bauble. 
                    !overthere
                    The Cult has relics aplenty, and will get plenty more before the moons turn. 
                agent:
                    !angry_shrug
                    $angryReluctant
                    So you would have me just walk away? <i>Ignore</i> your theft?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !interest
                    $miscPersuasive
                    You wanna die for a trinket? You wanna <i>kill</i> for one?
                agent:
                    !disappoint
                    $neutralResigned
                    ... No. I refuse to believe Hesh cares for such petty things. 
                player:
                    !happy
                    $happyThanks
                    That's the spirit. Consider this a failing of the Admiralty. You don't like it? Be sure to trash-talk them to your masters.
                    !placate
                    Now I'll just...
                * You carefully edge away from {agent}, watching {agent.himher} until {agent.heshe} is out of sight.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    If I hadn't stolen, you'd never have learned, uh, the true meaning of... forgiveness?
                agent:
                    $angryDefault
                    Enough prattle. I don't claim to know what Hesh wants, but I do this in it's name. 
                player:
                    !angry
                    $angryHostile
                    Drown in the shallows, Luminari. See if I care.
                    !fight
                agent:
                    !fight
            ]],
            OPT_FIGHT = "Attack {agent}",
            DIALOG_FIGHT = [[
                player:
                    $angryReluctant
                    You just made this a good day, Luminari.
                * You draw your weapons, and feel a bit giddy as you do. Killing the Cult is a favorite of yours.
                agent:
                    !exit
            ]],
            DIALOG_POST_FIGHT_SURVIVED = [[
                player:
                    !angry_threaten
                    $angryCruel
                    Well? You feelin' Hesh's protection?
                    Maybe a little doubt will help you live longer. Think about it. 
                * You leave {agent} behind and hurry along your path.
            ]],
            DIALOG_POST_FIGHT_DIED = [[
                * You're pretty sure the body count's higher than anticipated, but the cleanest jobs leave no witnesses. 
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
            SIT_MOD = "{1} doesn't want to pay"
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest.param.interrupted = true
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("luminari"))
                if cxt.quest.param.stole_relic then
                    cxt:Dialog("DIALOG_INTRO_STOLE")
                elseif cxt.quest.param.destroyed_relic then
                    cxt:Dialog("DIALOG_INTRO_DESTROYED")
                end
            end

            if cxt.quest.param.stole_relic then
                cxt:Opt("OPT_DONATE")
                    :PreIcon(global_images.giving)
                    :Dialog("DIALOG_DONATE")
                    :Fn(function(cxt)
                        cxt.quest.param.has_relic = false
                        cxt.quest:GetCastAgent("luminari"):OpinionEvent(OPINION.APPROVE)
                    end)

                cxt:Opt("OPT_SELL")
                    :Dialog("DIALOG_SELL")
                    :PostText( "RECEIVE_MONEY", math.floor(FAKE_RELIC_PRICE * cxt.quest:GetRank() * 1.25))
                    :Negotiation{
                        situation_modifiers =
                        {
                            { value = 10, text = cxt:GetLocString("SIT_MOD", cxt.quest:GetCastAgent("luminari")) }
                        },
                        on_success = function()
                            cxt.quest.param.has_relic = false
                            cxt:Dialog("DIALOG_SELL_SUCCESS")
                            cxt.enc:GainMoney(math.floor(FAKE_RELIC_PRICE * cxt.quest:GetRank() * 1.25))
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function()
                            cxt:Dialog("DIALOG_SELL_FAIL")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function()
                                        if cxt.quest:GetCastAgent("luminari"):IsDead() then
                                            cxt:Dialog("DIALOG_POST_FIGHT_DIED")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        else
                                            cxt:Dialog("DIALOG_POST_FIGHT_SURVIVED")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end
                                    end
                                }
                        end
                    }
            end

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                        on_success = function()
                            cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function()
                            cxt:Dialog("DIALOG_CONVINCE_FAIL")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function()
                                        if cxt.quest:GetCastAgent("luminari"):IsDead() then
                                            cxt:Dialog("DIALOG_POST_FIGHT_DIED")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        else
                                            cxt:Dialog("DIALOG_POST_FIGHT_SURVIVED")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end
                                    end
                                }
                        end
                    }

            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                    on_win = function()
                        if cxt.quest:GetCastAgent("luminari"):IsDead() then
                            cxt:Dialog("DIALOG_POST_FIGHT_DIED")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        else
                            cxt:Dialog("DIALOG_POST_FIGHT_SURVIVED")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    end
                }
        end)

    :State("STATE_VENDOR_RETURN")
        :Loc{
            DIALOG_INTRO = [[
                * As you're walking down the road, you notice the same merchant you met earlier. 
                * You holler and wave as you jog towards {agent.himher}, but {agent.heshe} doesn't look all that pleased to see you.
                player:
                    !left
                    !tired
                    H-hey! Hi! It's me!
                agent:
                    !right
                    $neutralTakenAback
                    Oh! You're back! Uh, I take it you're, uh, pleased with your purchase?
            ]],
            OPT_CONVINCE_TO_BUY = "Convince {agent} to buy the relic",
            DIALOG_CONVINCE = [[
                player:
                    !happy
                    $happyGreeting
                    Actually, I might have something <i>you</i> would like.
                agent:
                    !dubious
                    $neutralDubious
                    Is that so? I'm not a very big... buyer.
                player:
                    !agree
                    $miscPersuasive
                    I suspect you'll wanna reconsider.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !overthere
                    $miscAlluring
                    Unlike the treasure you sold me, <i>this</i> is authentic. And with the auction coming up...
                agent:
                    !interest
                    $happyTakenAback
                    I could really make waves with a relic like this. 
                    !handwring
                    $happyThanks
                    And you'd sell it to me? Really? 
                player:
                    !give
                    $happyCocky
                    Just be glad I've decided I like you.
                agent:
                    !take
                    $happyAmazed
                    I am, grifter! I am indeed! 
                * {agent} grips the relic gladly, {agent.hisher} eyes shining. 
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    !happy
                    $miscAlluring
                    Well? Whaddya think?
                agent:
                    !palm
                    $neutralResigned
                    Look, grifter, I get what you're trying to pull. But I have trouble enough moving my <i>own</i> fakes.
                    You off-load yours somewhere else, okay?
                player:
                    !neutralTakenAback
                    !dubious
                    What?! This isn't⁠—it's the real⁠—
                agent:
                    !handwave
                    $neutralResigned
                    Nope. Not interested. 
            ]],
            OPT_LEAVE = "Leave",
            DIALOG_LEAVE = [[
                player:
                    !angry_shrug
                    $angryReluctant
                    I guess I'll be going, then.
                agent:
                    !greeting
                    $happyGreeting
                    But hey, you want to buy any MORE counterfeits, you know where to find me!
                player:
                    !angry
                    $angryReluctant
                    Hrmph.
                * You keep travelling. 
            ]],
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest.param.interrupted = true
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("vendor"))
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_CONVINCE_TO_BUY")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    on_success = function()
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest.param.has_relic = false
                        cxt.enc:GetCaravan():AddMoney(math.floor(FAKE_RELIC_PRICE * cxt.quest:GetRank()) * 2)
                    end
                }

            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :Travel()
        end)


QDEF:AddConvo("return_for_reward", "handler")
    :Loc{
        OPT_GIVE_RELIC = "Give {handler} the antiquity",
        DIALOG_GIVE_RELIC = [[
            player:
                !permit
                $happyGreeting
                Hey, here's the antiquity. You were right: it's just decor. But it's your decor now, boss.
            handler:
                !angry_shrug
                $neutralDubious
                What am I supposed to do with this? Hang it in my war room and pretend it's a weapon?
                !thought
                $neutralThoughtful
                Huh. Actually, yeah. That'll do.

        ]],
        OPT_TURN_IN = "Tell {handler} about the antiquity",
        DIALOG_TURN_IN = [[
            player:
                !permit
                $happyGreeting
                You were right—the antiquity was just for decoration.
            handler:
                !shrug
                $neutralWhatever
                Just as I thought. Do you have it?
            player:
                Not... exactly. But the Admiralty doesn't.
            handler:
                Good enough. If the Admiralty don't have it, I'm happy.
                !thought
                Well, not <i>happy</>, but at least not angry.
        ]],
    }
    :Hub(function(cxt) 
        if cxt.quest.param.has_relic then
            cxt:Opt("OPT_GIVE_RELIC")
                :PreIcon(global_images.giving)
                :Dialog("DIALOG_GIVE_RELIC")
                :Fn(function(cxt)
                    cxt.quest:GetCastAgent("handler"):OpinionEvent(OPINION.APPROVE)
                end)
                :CompleteQuest()
        else
            cxt:Opt("OPT_TURN_IN")
                :Dialog("DIALOG_TURN_IN")
                :CompleteQuest()
        end
    end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_ON_TAKEN = [[
            handler:
                !point
                $neutralDirect
                We stole some intel on the route they're taking. All you gotta do is set up camp, and they'll come to you.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_ON_TAKEN")

               cxt:Opt("OPT_ASK_ABOUT")
                   :IsHubOption(true)
                   :GoTo("STATE_QUESTIONS")     
        end)

    :AskAboutHub("STATE_QUESTIONS", 
             {

               "Ask about the antiquity",
               [[
                   player:
                        !dubious
                        Antiquities are dangerous ancient tech. My family used to dig them up for a living.
                   agent:
                        !handwave
                        Enh. The Spark Barons keep everything dangerous for themselves. 
                        !spit
                        Likely this is just some old knick-knack to decorate a priestly pedestal. 
               ]],


             })  


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            handler:
                !permit
                $neutralDirect
                The Admiralty have gotten hold of an antiquity. They want to gift it to the Cult—to butter them up for negotiations.
                Let's sour that butter, shall we?
            player:
                !dubious
                $neutralDubious
                You want me to steal the antiquity? 
            handler:
                !handwring
                $neutralWhatever
                Steal it. Destroy it. Whatever you like, so long as the Admiralty can't profit off of it.
        ]],
        DIALOG_ON_TAKEN = [[
            handler:
                !point
                $neutralDirect
                We got a lead on the route they're taking. All you gotta do is set up camp, and they'll come to you.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)
