local negotiation_defs = require "negotiation/negotiation_defs"
local battle_defs = require "battle/battle_defs"
local EVENT = negotiation_defs.EVENT
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

local ngraft = {
    id = "burnt_out_negotiation",
    type = GRAFT_TYPE.NEGOTIATION,
    name = "Burnt-out Graft",
    desc = "You draw one less card every round.",
    flavour = "'Ugh, who put this in you? Have a little respect for yourself.'",
    
    rarity = CARD_RARITY.UNIQUE,
    series = "GENERAL",

    negotiation_modifier =
    {
        hidden = true,
        event_handlers =
        {
            [ EVENT.CALC_TURN_DRAW ] = function( self, acc )
                acc:AddValue( -1, self )
            end,

        },
    },
}

Content.AddGraft( ngraft.id, ngraft )

local bgraft = {
    id = "burnt_out_battle",
    type = GRAFT_TYPE.COMBAT,
    name = "Burnt-out Graft",
    desc = "You draw one less card every round.",
    flavour = "'Ugh, who put this in you? Have a little respect for yourself.'",

    rarity = CARD_RARITY.UNIQUE,
    series = "GENERAL",
    battle_condition =
    {
        hidden = true,
        event_handlers = 
        {
            [ BATTLE_EVENT.CALC_TURN_DRAW ] = function( self, acc )
                acc:AddValue( -1, self )
            end,

        },
    },
}


Content.AddGraft( bgraft.id, bgraft )


-----------------------------------------


local QDEF = QuestDef.Define{
    title = "Buyer Beware",
    qtype = QTYPE.CONTRACT,
    rank = {1, 4},
    icon = engine.asset.Texture("icons/quests/smith_tei_buyer_beware.tex"),
    act_filter = "SMITH",
    desc = [[{tei} wants you to run some Spark Baron merchants out of town.]],
    tags = {"TEI_INTRO_JOB"},
    focus = QUEST_FOCUS.BALANCED,

    on_destroy = function(quest)
        
        if quest.param.special_graft then
            quest.param.special_graft:SetUnremovable(false)
        end

        if quest.param.ruined_graft then
            quest.param.ruined_graft:SetUnremovable(false)
        end

    end,

    on_debug_start = function(quest)
        TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("TEI_SHRINE"))
        UIHelpers.DoSpecificConvo( quest:GetCastMember("tei"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
    end,

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("seller"), location = quest:GetCastMember("pop_up_shop")} )
    end,

    avoids = {
    }
}

:AddOpinionEvents{
    ruined_business = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "You ruined their business",
    },
    left_behind = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "You left them behind",
    },
}


:AddCastByAlias{
  cast_id = "tei",
  alias = "TEI",
  provider = true,
  no_validation = true,
}

:AddDefCastSpawn("seller", "SPARK_BARON_TASKMASTER")
:AddDefCastSpawn("priest", "PRIEST")
:AddDefCastSpawn("snitch", "SPARK_BARON_GOON")


:AddQuestLocation{
    cast_id = "pop_up_shop",
    name = "The Traveling Miracle Graft Show",
    plax = "EXT_SMITH_GRAFTSHOP",
    map_tags = {"building"},
}

:AddObjective{
    id = "purchase_a_graft",
    title = "Purchase a Graft",
    desc = "Use the chit that {tei} provided to buy a graft at the Traveling Graft Show.",
    mark = {"pop_up_shop"},
    on_complete = function(quest) 
        quest:Activate("visit_priest")
    end,
}

--you get confronted by snitch
:AddObjective{
    id = "visit_priest",
    title = "Take the graft to {priest}",
    desc = "Take your new graft to {priest} so that {priest.heshe} can tamper with it.",
    mark = {"priest"},
    
    on_activate = function(quest)
        quest.param.interrupt_pending = true
    end,
    
    on_complete = function(quest) 
        quest:Activate("make_a_scene")        
    end,
}

:AddObjective{
    id = "make_a_scene",
    title = "Make a scene at the popup",
    desc = "Return to the Traveling Graft Show and make a scene about your defective graft.",
    mark = {"pop_up_shop"},
    on_complete = function(quest) 
        quest:Activate("return_to_priest")
    end,
}

--you get jumped by the seller
:AddObjective{
    id = "return_to_priest",
    title = "Get {priest} to fix your head",
    desc = "Return to {priest} to deal with your tampered-with graft.",
    mark = {"priest"},
    
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("priest"), location = quest:GetCastMember("priest"):GetHomeLocation(), role = CHARACTER_ROLES.PROPRIETOR} )
    end,

    on_complete = function(quest) 
        quest:Activate("return_to_tei")
    end,
}


:AddObjective{
    id = "return_to_tei",
    title = "Tell {tei} what happened",
    mark = {"tei"},
}


QDEF:AddIntro(
    --attract spiel
    [[
        
        agent:
            These are strange times, and everyone's an opportunist. 
            Spark Barons have set up shop down near the Great Beaching.
        player:
            That's bold, ain't it?
        agent:
            Pearl-on-the-Foam isn't <i>technically</i> a Heshian settlement--that honor belongs to Supplicant City alone. 
            Like it or not, Spark Barons command a sizeable part of the trade network here in Havaria, and that affords them the real estate, wherever they choose to put it.
        player:
            Hah, so Heshians aren't the only ones with money. How do your bosses feel about this?
        agent:
            Guess. 
            While the Beaching is certainly a message, nobody's certain what, exactly, it portends.
            Martyrs will be made very easily. Credence can be plucked from the sky.
            Please discredit the Barons--make it clear to the pilgrims that they are not welcome in Hesh's pools.    
    ]],
    
    --on accept
    [[
        player:
            You know, my brother Vix once said I make everything I touch unholy.
            So I guess I'm saying you picked the right guy for the job.
        agent:
            I'm sure I did. Here, take this chit.
            !give
        player:
            !take
        agent:
            It's good for one free graft. Pick something out, and get it installed. 
        player:
            So we're going to discredit them by buying up all of their stock?
        agent:
            You're going to visit {priest}, who's going to tamper with the graft, and make it look defective.
            Then you go back to the shop and make a scene.
        player:
            And, uh, how do I get rid of the broken graft? That has been installed? In my skull?
        agent:
            !shrug
            You're resourceful. You'll figure something out.

    ]],
    "purchase_a_graft"
    )


local function addGraft(cxt, graft)
    cxt:Opt("OPT_PICK_GRAFT", graft)
        :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex") )
        :ReqCondition(not cxt.player.graft_owner:HasGraft(graft.id), "REQ_ALREADY_HAVE_GRAFT")
        :Fn( function( cxt, who )
            
            local installed = ConvoUtil.GiveGraft(cxt, graft)
            if installed then
                cxt.quest.param.special_graft = graft
                graft:SetUnremovable(true)
                cxt:Dialog( "DIALOG_SELL_GRAFT" )
                cxt.quest:Complete("purchase_a_graft")
                StateGraphUtil.AddLeaveLocation(cxt)
            end
        end )
end


QDEF:AddConvo( "purchase_a_graft" )
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("pop_up_shop") end)
        :Loc{

            DIALOG_INTRO = [[
                {first_time?
                    * You find a small group of Barons with an improvised shop.
                    * Their wagon is heavy with merchandise, which the assembled crowd is picking over with all the elegance of a pack of vultures after battle.
                    agent:
                        !right
                        Grafts! Grafts for sale!
                        Welcome the Traveling Graft Show!
                        The finest grafts, direct from the fruitful loam of Grout Bog!
                        We clean them up, test them out, and pass the savings along to you!
                    player:
                        !left
                    agent:
                        You, there, grifter! You have the look of someone who appreciates a fine graft!
                }
                {not first_time?
                    * {agent} is still here, hawking wares like they're on fire.
                    agent:
                        !right
                        Back again, grifter?
                        I knew I had you hooked!
                }
            ]],
            OPT_WARES = "Ask about the grafts for sale",
            DIALOG_WARES = [[
                player:
                    Where are these grafts from?
                agent:
                    You ask too many questions, friend!
            ]],

            OPT_BARONS = "Ask why the Barons are in the Pearl",
            DIALOG_BARONS = [[
                player:
                    Isn't this Cult territory?
                    I didn't think you saw eye-to-eye with them?
                agent:
                    Tell me about it!
                    I dislike every moment spent in this pious hellscape.
                    But that big dead jellyfish on the beach brought all sorts to the Pearl, and all sorts brought their pocketbooks.
                    We're going where the money flows.
            ]],

            OPT_SHOW_CHIT = "Show {agent} your chit",
            DIALOG_SHOW_CHIT= [[
                player:
                    !give
                    I've got this. What'll that give me?
                agent:
                    !take
                    Oh!
                    Oh.
                    This entitles the bearer to one of our finest, slightly-used grafts.
                player:
                    !dubious
                    Slightly used?
                agent:
                    Yeah. Sometimes folks'll get a new graft and think they can take on the world.
                    And sometimes the world just ups and takes them out.
                    But!
                    Their loss is your gain!
                player:
                    This all sounds questionably hygienic.
                agent:
                    !happy
                    Yep!
            ]],
            OPT_BETTER_STOCK = "Ask to see the reserve stock",
            DIALOG_BETTER_STOCK = [[
                player:
                    You may not know this, but I'm kind of a big deal around town.
                    If people see <i>me</> using <i>your</> best grafts, you'll have to hire security to hold back all the customers trying to be just like me.
                    You could say I'm an <i>influencer</>.
            ]],
            DIALOG_BETTER_STOCK_SUCCESS = [[
                agent:
                    !thought
                    That does sound like a solid business decision.
                    I'll tell you what, I'll give you one of our better pieces.
                    But you better tell people where you got it from!
                player:
                    Oh trust me, people will find out.
            ]],
            DIALOG_BETTER_STOCK_FAILURE = [[
                agent:
                    Did you really expect me to fall for that?
                    !crossed
                    You get what's written on the chit. No more.
            ]],
            OPT_SEE_GOOD_STOCK = "Look at the premium grafts",
            OPT_SEE_STOCK = "Look at the 'slightly-used' grafts",
            DIALOG_SEE_STOCK = [[
                {is_influencer?
                    player:
                        Let's see how good this stuff actually is.
                }
                {not is_influencer?
                    player:
                        Alright, show me what you've got.
                }
            ]],

            OPT_PICK_GRAFT = "Take {1#graft}",
            DIALOG_SELL_GRAFT = [[
                {is_influencer?
                    agent:  
                        How's that feel? Nice, right?
                    player:
                        My fingers are a bit tingly...
                    agent:
                        That's how you can tell it's working!
                        Now, get out there and show that bad boy off!
                        !exit
                }
                {not is_influencer?
                    agent:
                        There you go. Feels good, doesn't it?
                    player:
                        Well, actually...
                    agent:
                        Now remember, we have a strict returns policy...
                        We strictly don't accept returns!
                        !exit
                }
            ]]

        }
        :Fn(function(cxt) 

            local ids = GraftCollection.Rewardable(cxt.player):Rarity(CARD_RARITY.COMMON):Pick(3)
            cxt.quest.param.seller_grafts = {}
            for _, def in ipairs(ids) do
                 table.insert(cxt.quest.param.seller_grafts, GraftInstance(def.id) )
            end
            
            local better_ids = GraftCollection.Rewardable(cxt.player):Rarity(CARD_RARITY.RARE):Pick(3)
            cxt.quest.param.better_grafts = {}
            for _, def in ipairs(better_ids) do
                 table.insert(cxt.quest.param.better_grafts, GraftInstance(def.id) )
            end


            cxt:TalkTo("seller")
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                
                cxt:QST("WARES")
                cxt:QST("BARONS")

                if not cxt.quest.param.shown_chit then
                    cxt:Opt("OPT_SHOW_CHIT")
                        :SetQuestMark()
                        :Dialog("DIALOG_SHOW_CHIT")
                        :Fn(function() cxt.quest.param.shown_chit = true end)
                else
                    cxt:BasicNegotiation("BETTER_STOCK")
                        :OnSuccess()
                            :Fn(function() 
                                cxt.quest.param.is_influencer = true
                            end)

                    if cxt.quest.param.is_influencer then
                        cxt:Opt("OPT_SEE_GOOD_STOCK")
                            :SetQuestMark()
                            :Dialog("DIALOG_SEE_STOCK")
                            :Fn(function() 
                                cxt:RunLoop(function()
                                    for k,graft in ipairs (cxt.quest.param.better_grafts) do
                                        addGraft(cxt, graft)
                                    end
                                    StateGraphUtil.AddBackButton(cxt)  
                                end)

                            end)
                    end

                    cxt:Opt("OPT_SEE_STOCK")
                        :Dialog("DIALOG_SEE_STOCK")
                        :SetQuestMark()
                        :Fn(function() 
                            cxt:RunLoop(function()
                                for k,graft in ipairs (cxt.quest.param.seller_grafts) do
                                    addGraft(cxt, graft)
                                end
                                StateGraphUtil.AddBackButton(cxt)  
                            end)

                        end)
                end
            end)
        end)


QDEF:AddConvo("visit_priest")
    :TravelConfront("INTERRUPT", function(cxt) return cxt.quest.param.interrupt_pending end)
        :Loc{
            DIALOG_INTRO = [[
                * Someone has been following you since you left the Traveling Graft Show.
                player:
                    !left
                    You're not half as quiet as you think you are.
                agent:
                    !right
                    Neither are you, grifter!
                player:
                    Whatever do you mean?
                agent:
                {not is_influencer?
                    I don't know what your angle is, but I don't trust it.
                }
                {is_influencer?
                    {seller} might think you're an 'influencer', but I'm not buying it.
                }
                I know you're working with the Cult!
            ]],

            OPT_THREATEN = "Scare off the baron",
            DIALOG_THREATEN = [[
                player:
                    You have a big mouth.
                    I recommend shutting it.
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                agent:
                    Alright. I'll let you go. But I've got my eye on you!
                    !exit
            ]],
            DIALOG_THREATEN_FAILURE = [[
                agent:
                    I don't scare so easily!
            ]],

            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Fisticuffs, then, is it?
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead. You just hope no-one else suspects your intentions.
                }

                {not dead?
                    agent:
                        !injured
                    player:
                        Alright, run along now.
                    agent:
                        !spit
                        You'll regret this!
                        !exit
                }
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest.param.interrupt_pending = false
            cxt:TalkTo('snitch')
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                cxt:BasicNegotiation("THREATEN")
                    :OnSuccess()
                        :Fn(function() 
                            cxt:GetAgent():MoveToLimbo()
                        end)
                        :Travel()

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :Fn(function() 
                                if cxt:GetAgent():IsAlive() then
                                    cxt:GetAgent():MoveToLimbo()
                                end
                            end)
                            :Travel()
                end)

        end)




QDEF:AddConvo("visit_priest", "priest")
    :Loc{
        OPT_ASK_ABOUT_GRAFT = "Ask {priest} to deal with your graft",
        
        DIALOG_ASK_ABOUT_GRAFT = [[
            player:
                {tei} sent me. I have a graft that you need to deal with.
            agent:
                Ah! You're the sacrificial lamb {tei.heshe} was talking about!
            player:
                Now, I don't like the sound of that one bit.
            agent:
                Oh, just a bit of gallows humor. Nothing to worry about.
                You're just going to be stuck with a bit of dead tech in the ol' noggin for a while.
            player:
                You're going to be able to take it out after, right?
            agent:
                Probably!
                You'll probably even survive the process.
                !thought
                Shame to waste good tech, though. 
            player:
                Anything you can do about that?
            agent:
                Well, there is one thing. But I don't recommend it.
        ]],

        OPT_ASK_ABOUT_ONE_THING = "Ask about that 'one thing'...",
        DIALOG_ASK_ONE_THING = [[
            player:
                What's the one thing?
            agent:
                I could temporarily disable the graft, and then re-enable it again after you've done the job.
            player:
                That sounds way better!
                What's the catch?
            agent:
                The catch is that it's going to hurt like Hesh's indifferent embrace!
                I can break the graft permanently from the outside.
                But to do more delicate work, I'm really going to have to get in there.
                It'll probably do permanent damage!
            player:
                Hrmmmm.
        ]],

        OPT_DESTROY_GRAFT = "Get {agent} to destroy the graft",
        DIALOG_DESTROY_GRAFT = [[
            player:
                Alright, just burn the damn thing out. Keeping it intact ain't worth the risk.
            agent:
                Sounds good. Let me see your head, please.
                !exit
            player:
                !exit
            * {agent} utters some strange incantations and waves a lumin-infused device around your head.
        ]],

        DIALOG_DESTROY_GRAFT_2 = [[
            * It tingles a little, and then you feel the graft going offline.
            player:
                !left
            agent:
                !right
                Done!
                Now run along and do attend to {tei}'s business, grifter!
                When you're done that, come back and I can at least remove the broken bits.
                !exit
        ]],

        OPT_DISABLE_GRAFT = "Get {agent} to temporarily disable the graft",
        DIALOG_DISABLE_GRAFT = [[
            player:
                I kind of like this thing. See if you can just switch it off.
            agent:
                Alright, but don't say I didn't warn you.
                !exit
            player:
                !exit
            * {agent} probes your head with a sharp instrument for a while. It does, indeed, hurt like Hesh's indifferent embrace.
        ]],
        DIALOG_DISABLE_GRAFT_2 = [[
            player:
                !left
                !injured
            agent:
                !right
                That should do it!
            player:
                You weren't kidding about that being painful!
            agent:
                You have a particularly thick skull, which complicated matters.
                Go attend to {tei}'s business. When you're done, come back to me and we can see about turning that graft back on.
        ]],
        TT_MAX_HEALTH_REDUCTION = "<#PENALTY>This will reduce your maximum health by {1}!</PENALTY>",
    }
    :Hub(function(cxt)

            cxt:Opt("OPT_ASK_ABOUT_GRAFT")
                :SetQuestMark()
                :Dialog("DIALOG_ASK_ABOUT_GRAFT")
                :Fn(function() 
                    local asked_one_thing = false
                    local delta = 5
                    cxt:RunLoop(function() 

                        if asked_one_thing then
                            cxt:Opt("OPT_DISABLE_GRAFT")
                                :Dialog("DIALOG_DISABLE_GRAFT")
                                :SetQuestMark()
                                :PostText("TT_MAX_HEALTH_REDUCTION", delta)
                                :Fn(function() 
                                    cxt.player.health:AddStatModifier( "EVENT_DISABLED_GRAFT", -delta )
                                    cxt.quest.param.can_fix_graft = true
                                    cxt.quest.param.ruined_graft = GraftInstance(cxt.quest.param.special_graft:IsNegotiationGraft() and "burnt_out_negotiation" or "burnt_out_battle")
                                    TheGame:GetGameState():GetPlayerAgent().graft_owner:ReplaceGraft( cxt.quest.param.special_graft, cxt.quest.param.ruined_graft )
                                    cxt.quest.param.ruined_graft:SetUnremovable(true)
                                end)
                                :Dialog("DIALOG_DISABLE_GRAFT_2")
                                :CompleteQuest("visit_priest")
                                :DoneConvo()
                        else
                            cxt:Opt("OPT_ASK_ABOUT_ONE_THING")
                                :Dialog("DIALOG_ASK_ONE_THING")
                                :Fn(function() asked_one_thing = true end)
                        end

                        cxt:Opt("OPT_DESTROY_GRAFT")
                            :SetQuestMark()
                            :Dialog("DIALOG_DESTROY_GRAFT")
                            :Fn(function() 
                                cxt.quest.param.ruined_graft = GraftInstance(cxt.quest.param.special_graft:IsNegotiationGraft() and "burnt_out_negotiation" or "burnt_out_battle")
                                TheGame:GetGameState():GetPlayerAgent().graft_owner:ReplaceGraft( cxt.quest.param.special_graft, cxt.quest.param.ruined_graft )
                                cxt.quest.param.ruined_graft:SetUnremovable(true)
                            end)
                            :Dialog("DIALOG_DESTROY_GRAFT_2")
                            :CompleteQuest("visit_priest")
                            :DoneConvo()
                    end)
                end)
        end)


QDEF:AddConvo( "make_a_scene" )
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("pop_up_shop") end)
        :Loc{

            DIALOG_INTRO = [[
                * You return to the Traveling Graft Show. The crowd is still there, buying up questionable grafts.
                


                {is_influencer?
                    seller:
                        !right
                        Look who it is, folks, the Pearl's own grifting son, {player}!
                        {player} exclusively uses our grafts!
                        Isn't that right, {player}?
                    player:
                        !left
                        !injured
                        Actually, not too good.
                }
                {not is_influencer?
                    seller:
                        !right
                        Looks like we have a repeat customer, folks!
                        Our grafts here at the Traveling Graft Show are so good, our customers can't stay away!
                        Isn't that right, {player}?
                    player:
                        !left
                        !injured
                        Actually, no.
                }

                player:
                    The graft that you sold me is busted!
                    It really really hurts!
                    !angry
                    What kind of shop are you running here?!

                {snitch_active?
                    * As you are weighing your options, {snitch} emerges from the crowd.
                    
                    snitch:
                        !right
                        !threaten
                        Don't believe this grifter!
                        {player.HeShe} is working with the Cult!
                }


                seller:
                    !right
                    !placate
                    Pay no attention to the disgruntled grifter, folks!
                    !happy
                    There's just no pleasing some people. 
                    !angry
                    Tell you what, folks, whoever gets that grifter out of here gets a graft on the house!

                * A few of the more desperate customers shuffle forward, weighing their options. 
                * If you don't act quickly, you'll be torn apart by the crowd!
            ]],


            OPT_ACT_BERSERK = "Attack one of the customers...",
            DIALOG_ACT_BERSERK = [[
                player:
                    The graft! It's making me do things!
            ]],

            OPT_ATTACK_BYSTANDER = "Attack {1}",
            DIALOG_ATTACK_BYSTANDER = [[
                * You single out {bystander}.
                bystander:
                    !right
                player:
                    !threaten
                    You! You put this in me! Arrrrrrrrrrgh!
                bystander:
                    !scared
                    W-what? I'm just here to buy a graft!
            ]],
            DIALOG_ON_WIN = [[
                right:
                    !exit
                * The crowd, terrified by your 'graft madness', breaks and runs.
            ]],

            DIALOG_DONE = [[
                seller:
                    !right
                    Wait! Where's everyone going?
                    !angry
                    You've ruined me, grifter!
                
                {is_influencer?
                
                    seller:
                        And I thought we were partners!
                    player:
                        Yeah, well.
                        !dismiss
                        That was a dumb idea.
                }

                seller:
                    !threaten
                    You've made a powerful enemy today!
                    !exit

                * {seller} starts packing up the shop, so you leave.
            ]],

            OPT_THREATEN_SELLER = "Threaten {seller}",
            DIALOG_THREATEN_SELLER = [[
                * You address the crowd.
                player:
                    Stay back!
                    This is between me and {seller}.
                    !threaten
                    You did this to me, and I'm going to make you pay!
            ]],
            DIALOG_THREATEN_SELLER_FAILURE = [[
                seller:
                    Nice try, grifter, but nobody's buying it.
                    !exit
                * {seller} takes a step back, and the crowd falls on you.
            ]],
            DIALOG_THREATEN_SELLER_SUCCESS = [[
                seller:
                    Sometimes grafts just fail, alright!
                    It's not my fault!
                * The crowd murmurs disapprovingly, and then starts to dissolve.
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Graaaaaaaaaah!
            ]],
            DIALOG_ON_WIN_CROWD = [[
                agent:
                    !exit
                * You take down enough of the crowd to demoralize the rest.
                * They all turn and run.
            ]],


        }
        :Fn(function(cxt) 

            local bystanders = {}
            table.insert(bystanders, TheGame:GetGameState():AddSkinnedAgent("FOREMAN"))
            table.insert(bystanders, TheGame:GetGameState():AddSkinnedAgent("POOR_MERCHANT"))
            table.insert(bystanders, TheGame:GetGameState():AddSkinnedAgent("HEAVY_LABORER"))

            for k,v in ipairs(bystanders) do
                v:MoveToLocation(cxt.location)
            end


            cxt:TalkTo("seller")


            cxt.enc.scratch.snitch_active = cxt.quest:GetCastMember("snitch"):IsAlive()

            cxt:Dialog("DIALOG_INTRO")

            local hinders = {}
            if cxt.enc.scratch.snitch_active then
                table.insert(hinders, cxt:GetCastMember("snitch"))
            end
            
            cxt:RunLoop(function() 
                
                cxt:Opt("OPT_ACT_BERSERK")
                    :IsHubOption(true)
                    :LoopingFn(function() 
                        

                        for k, target in ipairs( bystanders ) do
                            local enemies = { target }
                            if cxt.enc.scratch.snitch_active then
                                table.insert(enemies, cxt:GetCastMember("snitch"))
                            end

                            cxt:Opt( "OPT_ATTACK_BYSTANDER", target )
                                :Fn(function() cxt:ReassignCastMember("bystander", target) end)
                                :Dialog( "DIALOG_ATTACK_BYSTANDER", target )
                                :Battle{
                                    flags = BATTLE_FLAGS.NO_BYSTANDERS,
                                    enemies = enemies,
                                }
                                    :OnWin()
                                        :Dialog("DIALOG_ON_WIN")
                                        :ReceiveOpinion("ruined_business", nil, "seller")
                                        :Dialog("DIALOG_DONE")
                                        :CompleteQuest("make_a_scene")
                                        :Travel()
                        end                        

                        StateGraphUtil.AddBackButton(cxt)
                    end)


                cxt:BasicNegotiation("THREATEN_SELLER", { hinders = hinders, flags = NEGOTIATION_FLAGS.INTIMIDATION })
                    :OnSuccess()
                        :ReceiveOpinion("ruined_business", nil, "seller")
                        :Dialog("DIALOG_DONE")
                        :CompleteQuest("make_a_scene")
                        :Travel()

                    :OnFailure()
                        :Fn(function() 
                            if cxt.enc.scratch.snitch_active then
                                table.insert(bystanders, cxt.quest:GetCastMember("snitch"))
                            end
                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle{
                                    enemies = bystanders,
                                    flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.SELF_DEFENCE,
                                }
                                    :OnWin()
                                        :Dialog("DIALOG_ON_WIN_CROWD")
                                        :ReceiveOpinion("ruined_business", nil, "seller")
                                        :Dialog("DIALOG_DONE")
                                        :CompleteQuest("make_a_scene")
                                        :Travel()


                        end)
            end)

        end)


QDEF:AddConvo( "return_to_priest", "priest" )
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                * You track down {priest} at {priest.hisher} home.
                agent:
                    !right
                player:
                    !left
                {can_fix_graft?
                    player:
                        You gotta fix this thing. It's driving me nuts!
                    agent:
                        Let's see what I can do.
                }   
                {not can_fix_graft?
                    player:
                        Get this thing out of my head! It's killing me!
                    agent:
                        Alright, alright, hold your Oshnus.
                }

                * Just as {agent} reaches for {agent.hisher} scalpel, {seller} burst through the door, with backup!
                seller:
                    !right
                    !angry
                    I knew this was a con!
                    !threaten
                    I'm going to teach you a lesson now, grifter!
            ]],

            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    I should have just killed you before!
            ]],
            DIALOG_WON = [[
                {dead?
                    * Now {seller} is really out of business.
                }
                {not dead?
                    seller:
                        !injured
                        I'll get you next time!
                        !exit
                }
            ]],
            DIALOG_PRIEST_ALIVE = [[
                priest:
                    !right
                    That was unpleasant!
                    Now, where were we?
                    !exit
                player:
                    !exit
            ]],
            
            DIALOG_GRAFT_FIXED = [[
                * The process of repairing the broken graft is uncomfortable, but does no further damage to your addled cranium.
                player:
                    !left
                priest:
                    !right
                    There. Good as new.
                player:
                    That feels about right.
                    Thanks for that!
                priest:
                    Now kindly get out of my house so I can scrub out the bloodstains.
                    !exit
            ]],
            DIALOG_GRAFT_REMOVED = [[
                * The extraction process is quick and painless. It leaves your head with a comfortable, airy feeling.
                player:
                    !left
                priest:
                    !right
                    There. It's out.
                    You have a remarkably capacious skull.
                player:
                    !dubious
                    Thank you?
                priest:
                    Now kindly get out of my house so I can scrub out the bloodstains.
                    !exit
            ]],

            DIALOG_PRIEST_DEAD = [[
                * {priest} lies dead on the ground, gripping {priest.hisher} surgical implement.
                * Looks like you're stuck with a bunch of junk in your head, now.
            ]],

            OPT_PUSH_AND_RUN = "Push {priest} into {seller} and run like the wind",
            DIALOG_PUSH_AND_RUN = [[
                player:
                    I think you'll find that I am a very bad student.
                    !exit
                * You push {priest} into the Barons, and make a run for it.
                priest:
                    !left
                    !injured
                    Hey!
                seller:
                    !fight
                * You are not followed. It seems that {seller}'s wrath extends to {priest} as well.
            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:TalkTo("seller")
            local backup = cxt.quest:GetRank() < 2 and {"AUTODOG"} or {"SPARK_BARON_GOON"}
            CreateCombatBackup(cxt.quest:GetCastMember("seller"), backup, cxt.quest:GetRank())

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    allies = {"priest"},
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                }
                    :OnWin()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_WON")
                            if cxt.quest:GetCastMember("priest"):IsAlive() then
                                cxt:Dialog("DIALOG_PRIEST_ALIVE")
                                if cxt.quest.param.can_fix_graft then
                                    TheGame:GetGameState():GetPlayerAgent().graft_owner:ReplaceGraft( cxt.quest.param.ruined_graft, cxt.quest.param.special_graft )
                                    cxt.quest.param.special_graft:SetUnremovable(false)
                                else
                                    TheGame:GetGameState():GetPlayerAgent().graft_owner:RemoveGraft(cxt.quest.param.ruined_graft)
                                end
                                cxt:Dialog(cxt.quest.param.can_fix_graft and "DIALOG_GRAFT_FIXED" or "DIALOG_GRAFT_REMOVED")
                            else
                                cxt:Dialog("DIALOG_PRIEST_DEAD")
                            end
                            
                            cxt.quest:Complete("return_to_priest")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)

            cxt:Opt("OPT_PUSH_AND_RUN")
                :Dialog("DIALOG_PUSH_AND_RUN")
                :Fn(function() cxt.quest.param.abandoned_priest = true end)
                :ReceiveOpinion("left_behind", nil, "priest")
                :CompleteQuest("return_to_priest")
                :Travel()
        end)




QDEF:AddConvo( "return_to_tei", "tei" )
    :Loc{
        OPT_HAND_IN = "Give {tei} a report",
        
        DIALOG_HAND_IN = [[
            {seller_dead?
                player:
                    The graft shop has been shut down.
                    {seller} came after me after, and I had to shut {seller.himher} down, too.
                
                {priest_dead?

                    agent:
                        Yeah, and you got {priest} killed too.
                        Why are you so damned reckless?
                    player:
                        !shrug
                        It's an expression of my privileged upbringing, I guess.
                    agent:
                        !sigh
                        I'll clean up the mess, but try a little harder next time, OK?
                }

                {not priest_dead?
                    agent:
                        I wanted a little more subtlety, but that does send a message.
                    player:
                        If you want subtle, why are you using me?
                    agent:
                        That is a very good question.
                }
            }
            
            {not seller_dead?
                player:
                    The shop has been shut down. I last saw {seller} running in the direction of Grout Bog.
                
                {priest_dead?
                    agent:
                        !hips
                        ...And?
                    player:
                        ...
                        !shrug
                        ...and I guess {priest} is dead?
                    agent:
                        Yes. {priest.HeShe} is. I expect you to be more careful with the Cult's assets in the future.
                    player:
                        !salute
                        Will do!
                }

                {abandoned_priest?
                    agent:
                        Have you heard from {priest}?
                    player:
                        Um...
                        I heard some screams, I think?
                    agent:
                        Right. 
                        Sigh. I expect you to be more careful with the Cult's assets in the future.
                    player:
                        !salute
                        Sure thing!
                }
                
                {not priest_dead and not abandoned_priest?
                    agent:
                        Good job, grifter.
                        I wasn't expecting that to go off so cleanly.
                        You are surprisingly competent.
                    player:
                        !happy
                        Aw, you like me!
                    agent:
                        I didn't say that.
                }
            }
        ]]
    }
    :Hub(function(cxt)
            
            cxt.enc.scratch.priest_dead = cxt.quest:GetCastMember("priest"):IsDead()
            cxt.enc.scratch.seller_dead = cxt.quest:GetCastMember("seller"):IsDead()
            
            cxt:Opt("OPT_HAND_IN")
                :Dialog("DIALOG_HAND_IN")
                :Fn(function() 
                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:Complete()
                end)
                :DoneConvo()
    end)


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}
