local negotiation_defs = require "negotiation/negotiation_defs"
local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.EVENT
local EVENT = negotiation_defs.EVENT




--These are Ctenophorian cards. They're supposed to be a little unhinged.
local CARD_BLASPHEMER, CARD_OPEN_MIND, CARD_SACRIFICE, CARD_DAMNATION = "blasphemer", "open_mind", "sacrifice", "damnation"

Content.AddBattleCard( CARD_SACRIFICE, 
{
    name = "Sacrifice",
    desc = "Take {1} damage and gain {2} {POWER} per action available.",
    desc_fn = function( self, fmt_str )
        return loc.format(fmt_str, self.damage_amt, self.power_amt )
    end,

    flavour = "'Hesh is thirsty!'",
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    flags = battle_defs.CARD_FLAGS.VARIABLE_COST | battle_defs.CARD_FLAGS.SKILL,
    target_type = TARGET_TYPE.SELF,
    cost = 0,

    damage_amt = 5,
    power_amt = 1,
    base_power = 0,

    OnPostResolve = function( self, battle, attack )
        if self.base_power and self.base_power > 0 then
            self.owner:AddCondition("POWER", self.base_power, self)
        end
        local actions = battle:GetActionCount()
        battle:ModifyActionCount( -actions )
        for i = 1, actions do
            self.owner:ApplyDamage(self.damage_amt, self.owner, self)
            self.owner:AddCondition("POWER", self.power_amt, self)
        end
    end
})
Content.AddBattleCard( "sacrifice_plus", 
{
    name = "Softened Sacrifice",
    desc = "Take <#UPGRADE>{1} damage</> and gain {2} {POWER} per action available.",

    damage_amt = 4,
    power_amt = 1,
})
Content.AddBattleCard( "sacrifice_plus2", 
{
    name = "Boosted Sacrifice",
    desc = "<#UPGRADE>Gain 1 {POWER}.</>\nTake {1} damage and gain {2} {POWER} per action available.",

    base_power = 1,
})

Content.AddBattleCard( CARD_DAMNATION, 
{
    name = "Damnation",
    desc = "Apply {1} {damnation} and {1} {POWER} to a friendly target.",
    desc_fn = function( self, fmt_str )
        return loc.format(fmt_str, self.damned_amt )
    end,
    flavour = "'I'm not worried about <i>eternity</>! I've got problems <i>right now</>!'",
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    flags = battle_defs.CARD_FLAGS.SKILL | battle_defs.CARD_FLAGS.EXPEND,
    target_type = TARGET_TYPE.FRIENDLY_OR_SELF,
    cost = 2,

    damned_amt = 4,

    OnPostResolve = function( self, battle, attack )
        attack:AddCondition("damnation", self.damned_amt, self)
        attack:AddCondition("POWER", self.damned_amt, self)
    end,

    condition =
    {
        name = "Damned",
        desc = "At the end of {1}'s turn, reduce <b>Damned</> by 1. When <b>Damned</> reaches 0, {1} takes {2} damage.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetOwnerName(), self.damage_amt )
        end,

        damage_amt = 50,

        ctype = CTYPE.DEBUFF,

        event_handlers = 
        {
            [ BATTLE_EVENT.END_TURN ] = function( self, fighter )
                if fighter == self.owner then
                    self.owner:RemoveCondition(self.id, 1, self)
                    if self.stacks == 0 then
                        self.owner:ApplyDamage(self.damage_amt, self.owner, self)
                    end
                end
            end
        },
    },
})
Content.AddBattleCard( "damnation_plus", 
{
    name = "Long Damnation",
    desc = "Apply <#UPGRADE>{1} {damnation} and {1} {POWER}</> to a friendly target.",
    damned_amt = 6,
})
Content.AddBattleCard( "damnation_plus2", 
{
    name = "Enduring Damnation",
    flags = battle_defs.CARD_FLAGS.SKILL,
})

Content.AddNegotiationCard(CARD_BLASPHEMER, 
{
    name = "Blasphemer",
    desc = "{INCEPT} 1 {blasphemer}.",
    flavour = "'Say your piece quietly— Hesh might hear you!'",
    cost = 1,
    flags = negotiation_defs.CARD_FLAGS.HOSTILE,
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    flags = negotiation_defs.CARD_FLAGS.MANIPULATE,

    OnPostResolve = function( self, minigame, targets )
        self.anti_negotiator:InceptModifier("blasphemer", 1 , self )
    end,

    modifier =
    {
        desc = "At the end of {1}'s turn, apply 1 {COMPOSURE} per stack to a random target.\nWhenever this argument is attacked, increase its stacks by 1.\nWhen destroyed, deal 1 damage per stack to all friendly arguments.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self:GetOwnerName() )
        end,
        max_resolve = 5,
        modifier_type = MODIFIER_TYPE.ARGUMENT,
        target_self = TARGET_ANY_RESOLVE,

        OnBounty = function( self, card )
            local targets = self.engine:CollectAlliedTargets( self )
            for i,arg in ipairs(targets) do
                arg:AttackResolve( self.stacks, self )
            end
        end,

        event_handlers =
        {
            [ EVENT.ATTACK_RESOLVE ] = function( self, source, target, damage, params, defended )
                if target == self and damage - defended < self.resolve then
                    self.negotiator:DeltaModifier(self, 1, self)
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    local target = table.arraypick(self.engine:CollectAlliedTargets( self ))
                    target:DeltaComposure(self.stacks, self)
                end
            end,
        },
    },
})
       
Content.AddNegotiationCard(CARD_OPEN_MIND, 
{
    name = "Open Mind",
    desc = "Discard your hand, draw {1} cards.",
    desc_fn = function( self, fmt_str )
        return loc.format( fmt_str, self.draw_cards )
    end,
    flavour = "'Consider all options, no matter how distasteful.'",
    cost = 1,
    flags = negotiation_defs.CARD_FLAGS.MANIPULATE,
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    draw_cards = 3,

    OnPostResolve = function( self, minigame, targets )
        local tbl = ObtainWorkTable()
        for i, card in minigame:GetHandDeck():Cards() do
            table.insert(tbl, card)
        end

        for i, card in ipairs(tbl) do
            minigame:DiscardCard(card)
        end

        ReleaseWorkTable(tbl)

        minigame:DrawCards(self.draw_cards)
    end
})
Content.AddNegotiationCard("open_mind_plus", 
{
    name = "Pale Open Mind",
    cost = 0,
})
Content.AddNegotiationCard("open_mind_plus2", 
{
    name = "Visionary Open Mind",
    desc = "Discard your hand, draw <#UPGRADE>{1} cards</>.",
    draw_cards = 4,
})

-----

local QDEF = QuestDef.Define{
    title = "The Ctenophorian Heresy",
    qtype = QTYPE.CONTRACT,
    rank = {1, 4},
    act_filter = "SMITH",
    desc = [[{tei} wants you to help root out and shut down a heresy within the Cult.]],
    tags = {"TEI_JOB"},
    focus = QUEST_FOCUS.BALANCED,

    on_start = function(quest)
    end,

    on_debug_start = function(quest)
        TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("TEI_SHRINE"))
        UIHelpers.DoSpecificConvo( quest:GetCastMember("tei"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
    end,

    avoids = {
        {"luminitiate", "zealot"},
        {"luminitiate", "priest"},
        {"luminitiate", "luminari"},

        {"zealot", "priest"},
        {"zealot", "luminari"},

        {"priest", "zealot"},
        {"priest", "luminari"},
        {"zealot", "luminari"},

    }
}

:AddOpinionEvents{
    joined_their_cult = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "You joined their cult",
    },
    redeemed = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Got them out of a cult",
    },
}


:AddCastByAlias{
  cast_id = "tei",
  alias = "TEI",
  provider = true,
  no_validation = true,
}

:AddDefCastSpawn("luminitiate", "LUMINITIATE")
:AddDefCastSpawn("luminari", "LUMINARI")
:AddDefCastSpawn("priest", "PRIEST")
:AddDefCastSpawn("zealot", "ZEALOT")

:AddQuestLocation{
    cast_id = "ritual_location",
    name = "Ritual Location",
    plax = "INT_SMITH_RITUALPIT",
    map_tags = {"quarters"},
}

:AddObjective{
    id = "interrogate_luminitiate",
    desc = "{luminitiate} knows where the ritual is going to be held. Extract that information.",
    title = "Interrogate {luminitiate}",
    negotiation_targets = {"luminitiate"},
    combat_targets = {"luminitiate"},
    mark = {"luminitiate"},
    on_complete = function(quest) 
        quest:Activate("disrupt_ritual")
        quest:Activate("interrogate_priest")
    end,
    on_fail = function(quest) 
        quest:Activate("return_to_tei_failure")
    end,

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("luminitiate") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,

}

:AddObjective{
    id = "interrogate_priest",
    title = "(Optional) Interrogate {priest}",
    desc = "{priest} can probably give you more information about the ritual. Might be useful.",
    negotiation_targets = {"priest"},
    mark = {"priest"},

    on_complete = function(quest) 
        quest:Activate("interrogate_luminari")
    end,
}

:AddObjective{
    id = "interrogate_luminari",
    title = "(Optional) Interrogate {luminari}",
    desc = "{luminari} is the security for the sect. You might be able to exploit that.",
    negotiation_targets = {"luminari"},
    combat_targets = {"luminari"},
    mark = {"luminari"},
}


:AddObjective{
    id = "disrupt_ritual",
    title = "Disrupt the Ritual",
    desc = "You've uncovered the site of the ritual. When you've gathered enough information about it, go there and stop it from happening.",
    mark = {"ritual_location"},
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("zealot"), location = quest:GetCastMember("ritual_location")} )
    end,
    on_complete = function(quest) 
        quest:Activate("return_to_tei")
    end
}

:AddObjective{
    id = "return_to_tei",
    title = "Tell {tei} what happened",
    mark = {"tei"},
}
:AddObjective{
    id = "return_to_tei_failure",
    title = "Tell {tei} what happened",
    mark = {"tei"},
}


QDEF:AddIntro(
    --attract spiel
    [[
        agent:
            Much like Hesh's waters, the Cult is often teeming with trouble beneath the surface.
            In this case, a rival sect has been spreading insubordinate mythos. And I would like it silenced.
        player:
            Oh? Tall tales comin' from Heshians?
            What's the big conspiracy? Is Hesh actually a big algae bloom?
        agent:
            Don't be absurd.
            The, ah, theological disagreement is...
            !sigh
            Ctenophore versus cnidarian. 
        player:
            You gotta sneeze there, Luminari?
        agent:
            I'm sure the details elude you. It's a question of taxonomy.
        player:
            I, uh, thought Hesh was kind of an unknowable eldritch being that defied our puny comprehension.
            Dome of a jelly, beak of a squid, spits ink to cloud the waters.
            Seems kinda silly to try and classify it, huh?
        agent:
            I agree. Strongly. 
            The taxonomists must be stopped. At worst their efforts are heretical, and at best...
            They may actually gift themselves madness.
        player:
            Well, ya gotta treat yourself to something nice every now and then.
        agent:
            Infiltrate their meeting and show them the error of their ways.
            Perhaps they'll see how close they are to Spark Barons with this talk.
    ]],
    
    --on accept
    [[
        agent:
            I don't know where the ritual is being held, but I have a lead on someone who does. Go shake the info out of them.
            You might be able to find more people who are attending, and figure out details before you go blundering in.
            But it's up to you.
    ]],
    "interrogate_luminitiate"
    )



QDEF:AddConvo( "interrogate_luminitiate", "luminitiate" )
    :Loc{
        OPT_ASK_ABOUT_HERESY = "Ask {agent} about the ritual",
        DIALOG_ASK_ABOUT_HERESY = [[
            {first_time?
                player:
                    Hey, I have a religious question for you.
                agent:
                    Yeah?
                player:
                    So Hesh is a jellyfish, right?
                    What <i>kind</> of jellyfish is Hesh, <i>exactly?</>
                agent:
                    !crossed
                    The <i>official</i> view is that Hesh is uncategorical. 
            }
            {not first_time?
                player:
                    I'm still struggling with the jellyfish question.
                agent:
                    !sigh
                    As I said before, the official position of the Cult of Hesh is that the Abyss exists outside the tree of life as we know it.
            }
        ]],

        OPT_CONVINCE = "Ask some leading questions",
        DIALOG_CONVINCE = [[
            player:
                Yeah, yeah, we <i>all</> know the <i>official</> answer.
                I'm asking for the <i>real</> answer. 
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                After all, how can we claim to swim in Hesh's waters if we can't even picture Hesh and exactly where it fits in marine biology?
            agent:
                !happy
                Oh, you're a seeker!
                There's a meeting tonight where you might find the answers you're looking for.
            * {agent} gives you an address.
            agent:
                I've got to go prepare now, but if you have any more questions you can ask {priest}.
                Don't tell anyone else about this, got it?
            player:
                See you there!
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            player:
                Like, do Hesh's tentacles <i>actually</i> sting? Or are they more, like, sticky?
            agent:
                !cagey
                Listen, questions like that are dangerous.
                You'd best close your mouth before the wrong person hears what you're saying.
        ]],

        OPT_BEAT_UP = "Beat the information out of {agent}",
        DIALOG_BEAT_UP = [[
            player:
                Alright, I know you're into some weird jelly stuff.
                And you don't want to tell me the details, because I'm just some random grifter, not one of your fellow jelly-fanciers.
                But the thing is, I need to know where your meeting is tonight.
                I'm going to beat you up now, just a little, and then you tell me what I want to know.
                !fight
                Deal?
            agent:
                !fight
                ...
        ]],
        DIALOG_KILLED = [[
            agent:
                !exit
            * You killed {agent}, and now you have no way to find out where the jellyfish club is meeting.
            * You're going to have to explain yourself to {tei}.
        ]],
        DIALOG_WON = [[
            agent:
                !injured
                Alright! Alright!
                Here's the address of the meeting!
                !give
            player:
                !threaten
                Who else is going to be there? Name me some names.
            agent:
                {priest}! But don't tell {priest.himher} I squealed!
            player:
                Yeah, whatever. Get out of here.
            agent:
                !exit
        ]]

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_ABOUT_HERESY")
            :Dialog("DIALOG_ASK_ABOUT_HERESY")
            :SetQuestMark()
            :IsHubOption(true)
            :LoopingFn(function(cxt) 
                
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("interrogate_luminitiate")
                        :Fn(function() 
                            cxt:GetAgent():MoveToLimbo()
                        end)
                        :DoneConvo()
                
                cxt:Opt("OPT_BEAT_UP")
                    :Dialog("DIALOG_BEAT_UP")
                    :Battle{}
                        :Fn(function() 
                            if cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_KILLED")
                                cxt.quest:Fail()
                            else
                                cxt:Dialog("DIALOG_WON")
                                cxt.quest.param.luminitiate_knows = true
                                cxt.quest:Complete("interrogate_luminitiate")
                                cxt:GetAgent():MoveToLimbo()
                            end

                            StateGraphUtil.AddEndOption(cxt)
                        end)

                StateGraphUtil.AddBackButton(cxt)

            end)
    end)


QDEF:AddConvo( "interrogate_priest", "priest" )
    :Loc{
        OPT_ASK_ABOUT_HERESY = "Ask {agent} about the ritual",
        DIALOG_ASK_ABOUT_HERESY = [[
            {first_time?
                player:
                    {luminitiate} told me you knew a lot about Hesh.
                agent:
                    {luminitiate.HeShe} did, did {luminitiate.heshe}?
                player:
                    Yeah, and {luminitiate.heshe} invited me to this thing later today. Said that you'd help me prepare for it.
                agent:
                    Ehhhh. I don't know if I trust you, friend.
            }
            {not first_time?
                player:
                    I really want to make a good impression at this meeting.
                    You don't have anything that will help me out, here?
            }
        ]],

        OPT_CONVINCE = "Convince {agent} that you're trustworthy",
        DIALOG_CONVINCE = [[
            player:
                I'm just an honest seeker looking for enlightenment.
                !happy
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            agent:
                Your curiosity seems legitimate.
                The meeting tonight is for those who are looking for a deeper understanding of the true nature of Hesh.
                There are those who would prevent others from reaching that understanding, so we have to take precautions.
            player:
                So it's guarded?
            agent:
                We have a pack of lumicytes to keep away the undeserving. 
            player:
                Sounds dangerous.
            agent:
                They're trained to stand down when they hear the word 'colloblast'.
            player:
                !thought
                Is that C-O-L-L-O--
            agent:
                B-L-A-S-T, yes. Why?
            player:
                Is that all, though? Just some trained animals?
            agent:
                Oh no, we have {luminari}, too.
            player:
                Good to know. I feel safer already.
                See you tonight!
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            agent:
                !crossed
                No, I don't trust you.
                The group will evaluate your sincerity tonight.
                Do not disappoint them.
        ]],


        OPT_ATTACK = "Attack {agent}",
        DIALOG_ATTACK = [[
            player:
                You don't have to trust me.
                !fight
                I'll settle for fear.
        ]],
        DIALOG_WON_ALIVE = [[
            agent:
                !injured
                Please! Stop!
            player:
                !threaten
                Tell me about the security at this meeting!
            agent:
                There are... holy beasts.
                They'll tear you to shreds on command, but...
                They'll stop if you use the command word 'colloblast'.
            player:
                Too easy. What else?
            agent:
                !flinch
                I don't know! Ask {luminari}... {luminari.heshe} is in charge of security.
            player:
                Great. You can live.
                If you keep quiet about this.
            agent:
                !exit
        ]],
        DIALOG_WON_DEAD = [[
            * {agent} is dead. You won't learn anything more from {agent.himher}.
            * At least there will be one less Cultist to deal with at the ritual.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_ABOUT_HERESY")
            :Dialog("DIALOG_ASK_ABOUT_HERESY")
            :SetQuestMark()
            :IsHubOption(true)
            :LoopingFn(function(cxt) 

                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("interrogate_priest")
                        :Fn(function() 
                            cxt.quest.param.know_codeword = true
                            cxt:GetAgent():MoveToLimbo()
                        end)
                        :DoneConvo()
                    
                    :OnFailure()
                        :Fn(function() 
                            cxt.quest.param.priest_suspicious = true
                        end)

                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{}
                        :OnWin()
                            :Fn(function() 
                                if cxt:GetAgent():IsAlive() then
                                    cxt:Dialog("DIALOG_WON_ALIVE")
                                    cxt.quest.param.priest_knows = true
                                    cxt.quest.param.know_codeword = true
                                    cxt.quest:Complete("interrogate_priest")
                                    cxt:GetAgent():MoveToLimbo()
                                else
                                    cxt:Dialog("DIALOG_WON_DEAD")
                                    cxt.quest:Fail("interrogate_priest")
                                end

                                StateGraphUtil.AddEndOption(cxt)
                            end)
                StateGraphUtil.AddBackButton(cxt)

            end)
    end)


QDEF:AddConvo( "interrogate_luminari", "luminari" )
    :Loc{
        OPT_ASK_ABOUT_HERESY = "Ask {agent} about the ritual",
        DIALOG_ASK_ABOUT_HERESY = [[
            {first_time?
                player:
                    I'm part of your club!
                agent:
                    My... club?
                player:
                    Yeah! The Ten Four Jelly Club! 
                agent:
                    That's... not the name of our organization.
                    We are the <i>taxonomists</>, and we believe that Hesh is-
                player:
                    My first meeting is tonight, I'm really excited!
                agent:
                    !dubious
                    What do you want from me?
            }
            {not first_time?
                player:
                    I have more questions about the Ten Four Jelly Club!
                agent:
                    Keep your voice down!
                    And stop calling it that!
            }
        ]],

        OPT_CONVINCE = "Try to turn {agent} against the Ten Four Jelly Club",
        DIALOG_CONVINCE = [[
            player:
                Okay, I'll level with you.
                !happy
                I'm not really a member of the Ten Four Jelly Club.
                The Cult is making its move against you weirdos, and it's going to be bad.
                I'm going to give you one chance to be on the right side of this thing. Are you going to take it?
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            agent:
                !sigh
                Oh, thank Hesh's indifference.
            player:
                So you're not a true believer, then?
            agent:
                I was curious, and one thing led to another, and then I found myself-
            player:
                !dismiss
                I don't need your life story. Just your help.
            agent:
                I want out.
            player:
                Great. Good enough for me.
                When the tentacles hit the brine at the meeting tonight, follow my lead.
                Back me up, and I'll tell the Cult good things about you.
            agent:
                Agreed.
                !exit

        ]],
        DIALOG_CONVINCE_FAILURE = [[
            agent:
                No. We won't be silenced!
                What is the Great Beaching if not a gift, a chance to understand Hesh's true nature?
                !fight
        ]],

        OPT_DEFEND = "Defend yourself",
        DIALOG_DEFEND = [[
            player:
                !fight
                You can't say I didn't give you a chance!
        ]],
        OPT_ATTACK = "Attack {agent}",
        DIALOG_ATTACK = [[
            player:
                !fight                
                I need you out the way.
        ]],
        DIALOG_WON_ALIVE = [[
            agent:
                !injured
                You- you've beaten me!
            player:
                Ain't you the perceptive one.
                You're going to skip town now. Got it?
            agent:
                I will. I- I have much to think about.
            player:
                !threaten
                And don't go warning your fellow weirdos about me before you leave!
            agent:
                !exit
        ]],
        DIALOG_WON_DEAD = [[
            * The head of security for the taxonomists lies dead at your feet.
            player:
                !happy
            * You always were a blunt instrumnent, Smith Banquod.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_ABOUT_HERESY")
            :Dialog("DIALOG_ASK_ABOUT_HERESY")
            :SetQuestMark()
            :IsHubOption(true)
            :LoopingFn(function(cxt) 
                
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("interrogate_luminari")
                        :Fn(function() 
                            cxt.quest.param.luminari_turned = true
                            cxt:GetAgent():MoveToLimbo()
                        end)
                        :DoneConvo()

                    :OnFailure()
                        :Fn(function() 
                            
                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
                                    :OnWin()
                                        :Fn(function()
                                            cxt.quest:Complete("interrogate_luminari")
                                            if cxt:GetAgent():IsAlive() then
                                                cxt:Dialog("DIALOG_WON_ALIVE")
                                                cxt:GetAgent():Retire()
                                                cxt.quest.param.luminari_gone = true
                                            else
                                                cxt:Dialog("DIALOG_WON_DEAD")
                                            end
                                            StateGraphUtil.AddEndOption(cxt)
                                        end)
                        end)
                        

                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{}
                        :OnWin()
                            :Fn(function() 
                                cxt.quest:Complete("interrogate_luminari")
                                if cxt:GetAgent():IsAlive() then
                                    cxt:Dialog("DIALOG_WON_ALIVE")
                                    cxt:GetAgent():Retire()
                                    cxt.quest.param.luminari_gone = true
                                else
                                    cxt:Dialog("DIALOG_WON_DEAD")
                                end
                                StateGraphUtil.AddEndOption(cxt)
                            end)
                StateGraphUtil.AddBackButton(cxt)

            end)
    end)    


QDEF:AddConvo( "disrupt_ritual" )
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("ritual_location") end)
        :Fn(function(cxt) 

            if cxt.quest:IsActive("interrogate_priest") then
                cxt.quest:Cancel("interrogate_priest")
            end
            if cxt.quest:IsActive("interrogate_luminari") then
                cxt.quest:Cancel("interrogate_luminari")
            end


            cxt.enc.scratch.opfor = {}
            cxt.enc.scratch.allies = {}
            table.insert(cxt.enc.scratch.opfor, cxt.quest:GetCastMember("zealot"))
            cxt:TalkTo("zealot")

            if cxt.quest:GetCastMember("luminitiate"):IsAlive() then
                cxt.quest:GetCastMember("luminitiate"):MoveToLocation(cxt.location)
                table.insert(cxt.enc.scratch.opfor, cxt.quest:GetCastMember("luminitiate"))

            end

            if cxt.quest:GetCastMember("priest"):IsAlive() then
                cxt.quest:GetCastMember("priest"):MoveToLocation(cxt.location)
                table.insert(cxt.enc.scratch.opfor, cxt.quest:GetCastMember("priest"))
            end

            if cxt.quest:GetCastMember("luminari"):IsAlive() and not cxt.quest.param.luminari_gone then
                cxt.quest:GetCastMember("luminari"):MoveToLocation(cxt.location)
                if cxt.quest.param.luminari_turned then
                    table.insert(cxt.enc.scratch.allies, cxt.quest:GetCastMember("luminari"))
                else
                    table.insert(cxt.enc.scratch.opfor, cxt.quest:GetCastMember("luminari"))
                end
            end

            local can_infiltrate = not cxt.quest.param.luminitiate_knows and not cxt.quest.param.priest_knows
            cxt:GoTo(can_infiltrate and "STATE_INFILTRATE" or "STATE_ATTACK")
        end)
    :State("STATE_INFILTRATE")
        :Loc{
            DIALOG_INTRO = [[
                * You find the ritual location.
                player:
                    !left
                    So this is the place, huh?
                luminitiate:
                    !right
                    !greeting
                    Hey, there's the new guy I was talking about!
                luminitiate:
                    Let me introduce you to {zealot}.
                
                * A dour-looking Zealot emerges from the shadows.
                zealot:
                    !right
                    !hips
                    You've brought us another stray, {luminitiate}?
                luminitiate:
                    !left
                    Not a stray. A seeker!
                    This grifter wants to know the true nature of Hesh.
                zealot:
                    !thought
                    Does {player.heshe}, now?
                    Grifter. Convince me of your sincerity.
                player:
                    !left
            ]],

            OPT_CONVINCE = "Convince {agent} you're down for some weird Hesh stuff",
            DIALOG_CONVINCE = [[
                player:
                    Yes.
                    I want to know the truth about Hesh!
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    Like, is Hesh a cnidaroo? Or a cnetaphoo? 
                    Does it go "glorp glorp" through the water or more of a "bwawm, bwawm".
                zealot:
                    !angry
                    What is this nonsense?
                    The grifter is a spy from the Cult!
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    That creature on the beach, it's real! Not multidimensional, not unknowable.
                    It smells bad and leaks, just like we do!
                    So that makes me wonder. Is Hesh the same? And if so, did it evolve that way?
                zealot:
                    Yes, yes, exactly right!
                    And evolution, as we know, is driven by struggle.
                    If Hesh is all-powerful, what environment encouraged it to become so?
                    If Hesh has natural weapons, what are they used for?
                    And if it has natural defenses... from what does it defend itself?
                player:
                    Yeah, what you said.
                    Geez. That's, uh, a lot to think about.
                zealot:
                    !happy
                    Excellent!
                    Since this is your first meeting, you will be the one to face the pit.
                player:
                    The what now?
                zealot:
                    The pit!
                    It is a sacred portal through which the nature of Hesh reveals itself.
                player:
                    What do I have to do with it?
                zealot:
                    It's easy. You plunge your arm into the pit.
                player:
                    That's it?
                zealot:
                    Yes.
                    We will listen to your screams, and decipher Hesh's message.
                player:
                    !dubious
                    My... screams?
            ]],

            OPT_STICK_ARM_IN_PIT = "Stick your arm into the pit",
            DIALOG_STICK_ARM_IN_PIT = [[
                player:
                    Alright, I'm game. Let's do this!
                zealot:
                    Let us begin!
                    !exit
                player:
                    !exit
                * Strong arms grasp you from behind, and lead you over to the gaping pit. Raw lumin roils at the bottom - the heat and the vapours are intoxicating.
                * A chant starts. It increases in intensity as you are lowered.
                * Your arm hits the lumin. The pain is <i>exquisite.</>
            ]],
            OPT_REFUSE_ARM = "Refuse",
            DIALOG_REFUSE_ARM = [[
                player:
                    On second thought, I'd rather just watch you guys do your thing.
                zealot:
                    !angry
                    Interloper!
            ]],

            DIALOG_ROUND_TWO = [[
                * You hear screaming, and it takes some time for you to realize that you are the source. The cultists haul you back up and shove you in front of {zealot} again.
                player:
                    !left
                    !injured
                zealot:
                    !right
                    !happy
                    You did splendidly!
                player:
                    Did you learn anything from that?
                zealot:
                    Oh yes, here, let me show you.
            ]],

            DIALOG_ROUND_TWO_2 = [[
                player:
                    Alright, so we're done now, right?
                zealot:
                    Oh, no that was just a partial immersion, to test your spirit.
                    You passed!
                    Now, for the real thing...
            ]],
            OPT_DO_PART_TWO = "Allow the ritual to continue",
            DIALOG_DO_PART_TWO = [[
                player:
                    Uh... ok?
                    !exit
                zealot:
                    !exit
                * You are dragged back to the pit. The last thing you remember is the taste of your own bile...
            ]],

            OPT_REFUSE = "Refuse",
            DIALOG_REFUSE = [[
                player:
                    I think I've learned enough about Hesh for one day, thank you.
                zealot:
                    !angry
                    It was not a request!
            ]],
            DIALOG_TRANSITION = [[
                * Time passes in the void.
                * You wake up with a head full of ache and strange new ideas.
            ]],
            DIALOG_DONE_RITUAL = [[
                * You stagger to your feet, mentally rehearsing your excuses for {tei}.
            ]],

            OPT_ATTACK = "Drop the ruse and attack the sect",
            DIALOG_ATTACK = [[
                player:
                    !shrug
                    Eh. I'm actually here to kill you all.
                zealot:
                    What?!
            ]]


        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:BasicNegotiation("CONVINCE")
                :OnFailure()
                    :GoTo("STATE_FIGHT")
                :OnSuccess()
                    :Fn(function() 
                        cxt:Opt("OPT_STICK_ARM_IN_PIT")
                            :Dialog("DIALOG_STICK_ARM_IN_PIT")
                            :DeltaHealth(-8)
                            :Dialog("DIALOG_ROUND_TWO")
                            :GainCard(CARD_OPEN_MIND)
                            :Dialog("DIALOG_ROUND_TWO_2")
                            :Fn(function() 
                                
                                cxt:Opt("OPT_DO_PART_TWO")
                                    :Dialog("DIALOG_DO_PART_TWO")
                                    :DeltaHealth(-16)
                                    :FadeOut()
                                    :Dialog("DIALOG_TRANSITION")
                                    :GainCard(CARD_DAMNATION)
                                    :Fn(function()
                                        cxt.quest:Complete("disrupt_ritual")
                                        cxt.quest.param.did_ritual = true
                                        local runaway_loc = TheGame:GetGameState():AddLocation( Location( TheGame:GetGameState():GetWorldRegion():GetOutdoorLocationID() ) )
                                        runaway_loc:SetFlags( LOCFLAGS.TRANSIENT )
                                        cxt.encounter:DoLocationTransition( runaway_loc )
                                    end)
                                    :Dialog("DIALOG_DONE_RITUAL")
                                    :Travel()

                                cxt:Opt("OPT_REFUSE")
                                    :Dialog("DIALOG_REFUSE")
                                    :GoTo("STATE_FIGHT")
                            end)

                        cxt:Opt("OPT_REFUSE_ARM")
                            :Dialog("DIALOG_REFUSE_ARM")
                            :GoTo("STATE_FIGHT")
                    end)

            
            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :GoTo("STATE_FIGHT")

        end)
    :State("STATE_ATTACK")
        :Loc{
            DIALOG_INTRO = [[
                * You find the ritual location.
                * The Cultists have been expecting you.
                player:
                    !left
                zealot:
                    !right
                    Grifter! You are a pawn in a game you do not understand.
                    We taxonomists have no quarrel with you. We seek only truth and understanding.
                    Let us make you an offer: We will teach you one of our sacred teachings, and you will let us disperse.
                    Then you tell your handlers that we escaped.
                player:
                    And if I refuse?
                zealot:
                    !shrug
                    Then you will kill some of us, and we will kill all of you.
                    It's your choice.

            ]],

            OPT_TAKE_TEACHING = "Learn {1#card}",
            DIALOG_TAKE_TEACHING = [[
                player:
                    These sacred learnings sound pretty useful. 
                    Lay it on me.
                zealot:
                    Very well.
                    !exit
                player:
                    !exit
                * What follows is a very long and detailed discourse on the true nature of Hesh and Hesh's many appendages. 
                * You manage to remember the most important bits.
                player:
                    !left
                zealot:
                    !right
                    Now, we will depart. 
                    !hesh_greeting
                    May this knowledge lead you to enlightenment, grifter.
                    !exit
                * You're not sure if this is exactly what {tei} meant by 'showing them the error of their ways'.
            ]],
            OPT_REFUSE = "Refuse the offer",
            DIALOG_REFUSE = [[
                player:
                    Nah, I don't want to know about your made up religion. I've barely got time for the real thing.
                zealot:
                    Very well.
                    !threaten
                    Kill the grifter!
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")


            for _, card  in ipairs{ CARD_BLASPHEMER, CARD_SACRIFICE } do
                cxt:Opt("OPT_TAKE_TEACHING", card)
                    :Dialog("DIALOG_TAKE_TEACHING")
                    :GainCard(card)
                    :CompleteQuest("disrupt_ritual")
                    :Fn(function() 
                        cxt.quest.param.let_cultists_go = true
                        for k,v in pairs(cxt.enc.scratch.opfor) do
                            if v:IsAlive() then
                                v:Retire()
                            end
                        end
                    end)
                    :Travel()
            end
            
            cxt:Opt("OPT_REFUSE")
                :Dialog("DIALOG_REFUSE")
                :GoTo("STATE_FIGHT")
        end)
    :State("STATE_FIGHT")
        :Loc{
            
            DIALOG_FRIEND = [[
                * You signal to {luminari}.
                luminari:
                    !left
                    I've got your back, {player}.
                zealot:
                    !angry
                    Traitor!
                player:
                    !left
                    !fight
                    Let's do this!
            ]],
            
            DIALOG_BEASTS = [[
                zealot:
                    !right
                    Unleash the lumicytes!
                beast:
                    !right
            ]],

            OPT_USE_WORD = "Use the command word",
            DIALOG_USE_WORD = [[
                player:
                    !cough
                    Ahem.
                    "Colloblast".
                beast:
                    !exit
                * The lumicytes retreat with a whimper.
                zealot:
                    !right
                    How did you know that?
                    !fight
                    Kill the grifter!
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND_BEASTS = [[
                * The beasts lunge toward you!
            ]],
            DIALOG_WON_BEASTS = [[
                zealot:
                    !right
                player:
                    !cruel
                    Is that all you got?!
                zealot:
                    Kill {player.himher}!
            ]]

        }
        :Fn(function(cxt) 

            if cxt.quest.param.luminari_turned then
                cxt:Dialog("DIALOG_FRIEND")
            end

            local beasts = CreateCombatParty({"LUMICYTE", "LUMICYTE", "LUMICYTE"}, cxt.quest:GetRank(), cxt.location)
            cxt:ReassignCastMember("beast", beasts[1])
            cxt:Dialog("DIALOG_BEASTS")

            if cxt.quest.param.know_codeword then
                cxt:Opt("OPT_USE_WORD")
                    :Dialog("DIALOG_USE_WORD")
                    :GoTo("STATE_FINAL_FIGHT")
            end

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND_BEASTS")
                :Battle{
                    no_oppo_limit = true,
                    enemies = beasts,
                    allies = cxt.enc.scratch.allies,
                    flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.ISOLATED}
                    :OnWin()
                        :Dialog("DIALOG_WON_BEASTS")
                        :GoTo("STATE_FINAL_FIGHT")
        end)

    :State("STATE_FINAL_FIGHT")
        :Loc{
            

            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                * The gathered cultists attack!
            ]],
            DIALOG_WON = [[
                {dead?
                    * {zealot} is dead, and the sect is shattered.
                }

                {not dead?
                    zealot:
                        !injured
                    player:
                        I'm going to let the Cult finish you off.
                    zealot:
                        !spit
                        Fool. I had such sights to show you.
                        !exit
                    * The sect is broken.
                }
            ]],
            DIALOG_FRIEND_POST = [[
                luminari:
                    !right
                    Thank you, grifter. I am finally free.
                player:
                    !neutral
                    Don't get ahead of yourself. You're still stuck in that tin can.
                luminari:
                    The Luminari armor is a gift, not a prison. 
                    Maybe someday you will understand that.
                    !exit
            ]]
        }
        :Fn(function(cxt) 

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                        no_oppo_limit = true,
                        enemies = cxt.enc.scratch.opfor,
                        allies = cxt.enc.scratch.allies,
                        flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.ISOLATED
                    }
                    :OnWin()
                        :Dialog("DIALOG_WON")
                        :Fn(function() 
                            cxt.quest:Complete("disrupt_ritual")
                            cxt.quest.param.attacked_ritual = true
                            cxt.quest.param.leader_escaped = cxt.quest:GetCastMember("zealot"):IsAlive()
                            if cxt.quest.param.luminari_turned and cxt.quest:GetCastMember("luminari"):IsAlive() then
                                cxt:Dialog("DIALOG_FRIEND_POST")
                                cxt:Opinion("redeemed", "luminari")
                            end   
                        end)
                        :Travel()
                        
        end)




QDEF:AddConvo( "return_to_tei", "tei" )
    :Loc{
        OPT_HAND_IN = "Give {tei} a report",
        
        DIALOG_HAND_IN = [[
            {attacked_ritual and leader_escaped?
                player:
                    I attacked the ritual.
                    The leader of the sect was {zealot}. 
                    {zealot.HeShe} got away.
                agent:
                    The Cult will track {zealot.himher} down.
                    Good work, {player}.
            }
            {attacked_ritual and not leader_escaped?
                player:
                    I attacked the ritual.
                    The leader of the sect was {zealot}, and I killed {zealot.himher}. 
                agent:
                    Very good work, {player}.
                    We can not abide by heresy.
            }
            {let_cultists_go?
                player:
                    I found the ritual.
                    It was too much to take on by myself, though.
                agent:
                    So you chickened out?
                player:
                    It was a tactical retreat.
                agent:
                    Very well. 
                    I would have preferred them dead, but at least they'll have to go underground now.
            }

            {did_ritual?
                player:
                    I found the ritual.
                agent:
                    And?
                player:
                    Those guys are into some weird stuff, let me tell you!
                agent:
                    And you stopped them?
                player:
                    Well...
                    They stopped eventually.
                    I honestly don't remember the details.
                agent:
                    !facepalm
                    ...
                    !angry
                    You were supposed to stop the ritual, not join it!
                player:
                    Hey, I got you names, and a location. Send in your goon squad if you want to crack some heads.
                agent:
                    Whatever. 
                    Go... have a shower.
            }

        ]]
    }
    :Hub(function(cxt)
            cxt:Opt("OPT_HAND_IN")
                :Dialog("DIALOG_HAND_IN")
                :Fn(function() 
                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:Complete()
                end)
                :DoneConvo()
    end)


QDEF:AddConvo( "return_to_tei_failure", "tei" )
    :Loc{
        OPT_HAND_IN = "Tell {tei} what happened",
        DIALOG_HAND_IN = [[
            player:
                I accidentally killed our only lead.
            agent:
                !facepalm
                Nice going, grifter.
                Now they'll just go underground.
        ]]
    }
    :Hub(function(cxt)
            cxt:Opt("OPT_HAND_IN")
                :Dialog("DIALOG_HAND_IN")
                :FailQuest()
                :DoneConvo()
    end)


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}
