local QDEF = QuestDef.Define{
    title = "Dog and Pony Show",
    qtype = QTYPE.CONTRACT,
    rank = {1, 4},

    act_filter = "SMITH",
    desc = [[{tei} needs you to babysit a visiting Rentorian dignitary.]],
    --icon = engine.asset.Texture("icons/quests/smith_tei_dog_and_pony.tex"),
    tags = {"TEI_JOB"},
    focus = QUEST_FOCUS.BALANCED,

    --this is a little bit of boilerplate that lets us test the quest directly from the quest menu
    on_debug_start = function(quest)
        TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("TEI_SHRINE"))
        UIHelpers.DoSpecificConvo( quest:GetCastMember("tei"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
    end,


    on_start = function(quest)
    end,

    on_destroy = function(quest)
        quest:GetCastMember("dignitary"):Dismiss()
    end,

    collect_agent_locations = function(quest, t)
        --table.insert (t, { agent = quest:GetCastMember("tei"), location = TheGame:GetGameState():GetLocation("TEI_SHRINE"), role = CHARACTER_ROLES.VISITOR} )
    end,

    process_fighter = function(quest, fighter)
        --[[if fighter.agent == quest:GetCastMember("recruit1") then
            fighter:AddCondition("WANTED_ALIVE")
        end
        if fighter.agent == quest:GetCastMember("recruit2") then
            fighter:AddCondition("WANTED_ALIVE")
        end--]]
    end

}

--------------Places to go

:AddQuestLocation{
    cast_id = "arrival",
    name = "Rentorian Boat",
    plax = "INT_SMITH_BOAT1",
    map_tags = {"small_boat"},
}

:AddQuestLocation{
    cast_id = "beaching",
    name = "The Beaching",
    plax = "EXT_SMITH_BEACHCREATURE",
    map_tags = {"beach"},
}

:AddQuestLocation{
    cast_id = "secret_meeting_location",
    name = "Secret Meeting Location",
    plax = "EXT_SMITH_POORRES1",
    map_tags = {"quarters"},
}

:AddLocationCast{
    cast_id = "liason_meeting_location",
    cast_fn = function(quest, t)
        table.insert(t,  TheGame:GetGameState():GetLocation("PEARL_FANCY_EATS"))
    end,
}

:AddQuestLocation{
    cast_id = "departure",
    name = "Departure Point",
    plax = "EXT_SMITH_WARF",
    map_tags = {"port"},
}


------------People to see
:AddCastByAlias{
  cast_id = "tei",
  alias = "TEI",
  provider = true,
  no_validation = true,
}

:AddCastByAlias{
  cast_id = "dignitary",
  alias = "BORDENKRA",
  no_validation = true,
}
:AddDefCastSpawn("smuggler", "JAKES_SMUGGLER")
:AddDefCastSpawn("liason", "PRIEST")
:AddDefCastSpawn("zealot", "ZEALOT")

-------------Things to do

:AddObjective{
    id = "meet_dignitary",
    title = "Go meet with {dignitary}",

    on_complete = function(quest)
        quest:Activate("go_to_beaching")
    end,
    
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("dignitary"), location = quest:GetCastMember("arrival")} )
    end,
    mark = {"dignitary"},
}

:AddObjective{
    id = "go_to_beaching",
    title = "Take {dignitary} to the Beaching",

    on_complete = function(quest)
        quest:Activate("go_to_meeting")
    end,
    
    mark = {"beaching"},
}

:AddObjective{
    id = "go_to_meeting",
    title = "Escort {dignitary} to the meeting with {liason}",

    on_complete = function(quest)
        quest:Activate("go_to_departure")
    end,
    
    mark = {"liason_meeting_location"},
}

:AddObjective{
    id = "go_to_secret_meeting",
    title = "(Optional) Attend secret meeting",
    mark = {"secret_meeting_location"},
}

:AddObjective{
    id = "go_to_departure",
    title = "Take {dignitary} to {dignitary.hisher} boat",

    on_complete = function(quest)
        
        if quest:IsActive("go_to_secret_meeting") then
            quest:Cancel("go_to_secret_meeting")
        end

        quest:Activate("return_to_tei")
    end,

    mark = {"departure"},
}

:AddObjective{
    id = "return_to_tei",
    title = "Return to {tei}",
    --desc = [[{handler} has a reward for you.]],
    mark = {"tei"},
}

-----------

:AddOpinionEvents{
    good_impression = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Made a good first impression",
    },
    bad_impression = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Made a bad first impression",
    },
    succeeded_mission_extra = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Helped them succeed at their secret mission",
    },
    betrayed_to_cult = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Betrayed them to the Cult",
    },
    handed_over_rentorian = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Handed over a Rentorian blasphemer",
    }
}




QDEF:AddIntro(
    --attract spiel
    [[
        tei:
            A war is coming with the Spark Barons, but many of the Cult still think the enemy lies to the north. 
        player:
            Uh, I'm not much good with politics.
        tei:
            I'm speaking of Rentoria. A dignitary is coming to tour Pearl-on-the-Foam and see the Great Beaching.
            But some in the Cult believe that even allowing the Rentorian in is an affront to Hesh.
            I want you to protect the Rentorian from any attempts on {dignitary.hisher} life.
    ]],
    
    --on accept
    [[
        agent:
            Go pick up {dignitary}, and take {dignitary.himher} to the Beaching and a meeting with {liason}. Make sure none of the citizens mess with {dignitary.himher} and cause an international incident.
            Oh, and watch out for {zealot}. {zealot.HeShe} is fervent in {zealot.hisher} beliefs, and those beliefs are dangerous.
            
    ]],
    "meet_dignitary"
    )


QDEF:AddConvo( "meet_dignitary", "dignitary" )
    :ConfrontState()
        :Loc{

            DIALOG_INTRO = [[
                * You find {dignitary}'s boat.
                * One of the boat's sails is caught in the wind, and a spinning beam is in danger of hitting them! You could help, or just watch.
            ]],

            OPT_HELP = "Help {dignitary}",
            DIALOG_HELP = [[
                player:
                    !left
                    Hey! Watch out!
                * You catch the beam just before it hits {dignitary} square in the back. It hurts a little, but you'll survive.
            ]],
            DIALOG_HELP_2 = [[                
                agent:
                    !right
                    Thank you. It is refreshing to see good manners in a Havarian.
            ]],
            
            OPT_WATCH = "Watch",
            DIALOG_WATCH = [[
                * The beam whips around towards {dignitary}. {dignitary.HeShe} locks eyes with you a moment before it lands.
                * {dignitary} makes a movement faster than your eyes can follow, and the beam swings harmlessly by.
                player:
                    !left
                    Wow. Pretty impressive, there, Rentorian.
                dignitary:
                    !right
                    I must make a better first impression than you do, I suppose.
                    You were just going to watch it happen?
            ]]
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_HELP")
                :Dialog("DIALOG_HELP")
                :DeltaHealth(-5)
                :Dialog("DIALOG_HELP_2")
                :ReceiveOpinion("good_impression")
                :GoTo("STATE_LEAVE")
            
            cxt:Opt("OPT_WATCH")
                :Dialog("DIALOG_WATCH")
                :ReceiveOpinion("bad_impression")
                :GoTo("STATE_LEAVE")
        end)


    :State("STATE_LEAVE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    Anything I should know before we go? 
                    Like, do Rentorians got extra limbs or stalks that need watering?
                agent:
                    You don't actually believe that we're so different, do you? 
                    We all came from the same Vagrants.
                player:
                    Yeah, well. These days? Ain't much that would surprise me.
                agent:
                    If there's anything specific you'd like to discuss, now seems a good time.
                
            ]],
            OPT_QUESTION = "Ask why they want to see the Beaching",
            DIALOG_ANSWER = [[
                player:
                    Why do you care about a beach blob?
                    You don't strike me as a Heshian.
                agent:
                    On the contrary, I've been a member of the Heshian faith since I was a child.
                    It didn't quite catch on in Rentoria, but there are believers among us—and Rentorians enjoy freedom of belief.
                player:
                    Freedom of what now?
                agent:
                    Precisely, Havarian.
            ]],
            OPT_HEAD_OUT = "Head out",
            DIALOG_HEAD_OUT = [[
                player:
                    !overthere
                    After you.
            ]]
        }

        :SetLooping()
        :Fn(function(cxt) 

            cxt:Question("OPT_QUESTION", "DIALOG_ANSWER")

            cxt:Opt("OPT_HEAD_OUT")
                :Dialog("DIALOG_HEAD_OUT")
                :CompleteQuest("meet_dignitary")
                :RecruitMember( PARTY_MEMBER_TYPE.ESCORT )
                :Travel()
            
        end)

QDEF:AddConvo( "go_to_beaching")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("beaching") end)
        :Loc{
            DIALOG_INTRO = [[
                * You reach the Beaching. There's a crowd, and {dignitary} wants privacy.
                player:
                    !left
                dignitary:
                    !right
                    Deal with the riff-raff, will you?
                    !exit
                * You approach the crowd.
                agent:
                    !right
                    What do you want, grifter?
            ]],

            OPT_CONVINCE_LEAVE = "Convince the crowd to disperse",
            DIALOG_CONVINCE_LEAVE = [[
                player:
                    You probably want to leave. I think the blob over there is going to have an incident.
                agent:
                    What do you mean 'incident'?
                player:
                    It's big, it's rotting.
                    Every so often...
                    Pbbbbbbbbt!
                agent:
                    !dubious
                    It makes wind?
                player:
                    Yeah!
                    Smells so bad that, uh... it kills people. 
                agent:
                    This all sounds very suspect...
            ]],

            DIALOG_CONVINCE_LEAVE_SUCCESS = [[
                player:
                    I'm tellin' ya, you don't want to be here for the death-fart.
                
                agent:
                    I guess the smell right now is bad enough.
                    If it can get worse? I don't wanna know about it.
                    We'll be back later. Thanks for the tip, grifter.

                * The crowd leaves.
            ]],
            DIALOG_CONVINCE_LEAVE_FAILURE = [[
                player:
                    I'm tellin' ya, this thing's gonna blow!
                agent:
                    Hesh is the god of the people!
                    Each of us made our pilgrimage from Hesh's compulsions. 
                    We have every right to be here, Pearly!
            ]],
            OPT_FORCE_LEAVE = "Force the crowd to move",
            DIALOG_FORCE_LEAVE = [[
                player:
                    I asked nice.
                    !fight
                    Now I'm done asking.
                * {dignitary} holds back, just observing the fight.
            ]],
            DIALOG_WON = [[
                {not dead and any_enemies_died?
                    agent:
                        !injured
                        Run! {player.HeShe} is a killer!
                        !exit
                        * The remains of the crowd leave.
                    
                    dignitary:
                        !right
                        I see Havarians don't place much value on life. 
                    player:
                        On the contrary, every life is worth exactly what you can work out of 'em.
                }
                {not dead and not any_enemies_died?
                    agent:
                        !injured
                        Fine! We're leaving!
                        !exit
                    * The crowd leaves, grumbling the whole time.

                    dignitary:
                        !right
                        Is this how you solve your disputes here?

                }
                {dead and not all_enemies_died?
                    * {agent} is dead. The rest of the crowd scatters.
                    
                    dignitary:
                        !right
                        You went straight for the leader. I've seen those tactics before.

                }
                {dead and all_enemies_died?
                    * You murdered the entire crowd.
                    
                    dignitary:
                        !right
                        ...
                        You're not a subtle people.
                }
            ]]

        }
        :Fn(function(cxt) 

            local crowd = CreateCombatParty({"FOREMAN", "LABORER", "HEAVY_LABORER"}, cxt.location)
            cxt:TalkTo(crowd[1])
            local other_crowd = {}
            for k,v in ipairs(crowd) do
                if k ~= 1 then
                    table.insert(other_crowd, v)
                end
            end

            
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 
                cxt:BasicNegotiation("CONVINCE_LEAVE", {helpers = {"dignitary"}, hinders = other_crowd})
                    :OnSuccess()
                        :GoTo("STATE_CROWD_LEFT")
                                    
                
                cxt:Opt("OPT_FORCE_LEAVE")
                    :Dialog("DIALOG_FORCE_LEAVE")
                    :Battle{no_oppo_limit = true, noncombatants = {"dignitary"}}
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :Fn(function() 
                                
                                cxt.quest.param.used_violence_at_beaching = true 
                                
                                local any_dead = false
                                for k,v in pairs(crowd) do
                                    if v:IsDead() then
                                        any_dead = true

                                    end
                                end

                                if any_dead then
                                    cxt.quest:OpinionEvent("dignitary", "bad_impression")
                                end

                            end)
                            :GoTo("STATE_CROWD_LEFT")
            end)
        end)
    
    :State("STATE_CROWD_LEFT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    The path's clear now. You can do your thing.
                dignitary:
                    !right
                * {dignitary} approaches the Beaching.
                * {dignitary} draws a thin blade and takes a sample.
                dignitary:
                    !right
                    I'm done.
                player:
                    Wow. We better keep that on the down-low.
                    I think some Heshians would be real upset to learn you're doing the science on that thing.
                dignitary:
                    Yet you seem unfazed. Are you perhaps a man of science yourself?
                player:
                    I'm a man of I-do-what-I-want. 
                    And what I want is to wrap things up with {liason} so you can get home safe.
                    !exit

            ]]
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Complete("go_to_beaching")
            StateGraphUtil.AddLeaveLocation(cxt)
        end)


QDEF:AddConvo("go_to_meeting")
    :ConfrontState("MEET", function(cxt) return cxt.location == cxt.quest:GetCastMember("liason_meeting_location") end)
        :Loc{
            DIALOG_INTRO = [[
                * You reach the meeting location--a very fancy restaurant that has been emptied of patrons.
                liason:
                    !right
                    There you are.  {did_secret_meeting? What took you so long? |Right on time.}
                dignitary:
                    !left
                    Let's get this over with, Havarian.
                left:
                    !exit
                right:
                    !exit
                    * The two diplomats have a long and boring discussion.
                    * You're about to fall asleep when you're startled from your stupor by a change in tenor.
                dignitary:
                    !left
                    !angry
                liason:
                    !right
                    !angry
                    This meeting is over!
            ]],

            OPT_INTERVENE = "Intervene and cool down the situation",
            DIALOG_INTERVENE = [[
                player:
                    !left
                    !placate
                    Now, now. Is that anyway to treat our guest?
            ]],
            DIALOG_INTERVENE_SUCCESS = [[
                liason:
                    Fine. We will continue the discussions.
                    !exit
                player:
                    !exit
                * The talks drag on, subdued but tense.
                * Finally...
                dignitary:
                    !right
                    !happy
                liason:
                    !left
                    !happy
                    I'm glad we could come to an agreement!
                    !exit
                * The liaison leaves. {dignitary} turns to you.
                player:
                    !left
                dignitary:
                    Thank you for the assistance, Havarian.
                    I was not expecting one such as yourself to have such a nuanced grasp of statecraft.
                player:
                    Hey, I got grasps on all sorts of things!

            ]],
            DIALOG_INTERVENE_FAILURE = [[
                liason:
                    !angry
                    No! We will not tolerate such blatant disrespect!
                dignitary:
                    !right
                liason:
                    Get out of here, before I declare war on Rentoria!
                dignitary:
                    !threaten
                    You wouldn't dare!
                    !exit
                * You have no choice but to follow {dignitary} outside.
            ]],

            OPT_LET_THE_MEETING_END = "Let the interminable meeting come to a merciful end",
            DIALOG_LET_THE_MEETING_END = [[
                * You decide that statecraft is above your pay grade.
                liason:
                    Get out of here, before I declare war on Rentoria!
                dignitary:
                    You wouldn't dare!
                    !exit
                * You have no choice but to follow {dignitary} outside.
            ]]
            
        }
        :Fn(function(cxt) 
            LocationUtil.SendPatronsAway(cxt.location)
            cxt:TalkTo("liason")

            cxt:Dialog("DIALOG_INTRO")
            cxt:BasicNegotiation("INTERVENE")
                :OnSuccess()
                    :ReceiveOpinion("good_impression",nil, "dignitary")
                    :CompleteQuest("go_to_meeting")
                    :Travel()
                :OnFailure()
                    :CompleteQuest("go_to_meeting")
                    :Fn(function() cxt.quest.meeting_went_poorly = true end)
                    :Travel()

            cxt:Opt("OPT_LET_THE_MEETING_END")
                :Dialog("DIALOG_LET_THE_MEETING_END")
                :Fn(function() cxt.quest.meeting_went_poorly = true end)
                :CompleteQuest("go_to_meeting")
                :Travel()

        end)

QDEF:AddConvo("go_to_secret_meeting")
    :ConfrontState("MEET", function(cxt) return cxt.location == cxt.quest:GetCastMember("secret_meeting_location") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find the secret meeting.
                dignitary:
                    !left
                    I hope you weren't waiting too long.
                    The diplomacy of the situation is delicate, you understand.
                smuggler:
                    !right
                    Let's get down to brass tacks, huh? 
                    We don't want anyone getting suspicious of where you've gone.
                dignitary:
                    !exit
                smuggler:
                    !exit
                * They do just that. You spend your time kicking rocks, and counting the critters that skitter out from under.
                player:
                    !left
                * Unfortunately you're soon joined by the worst critters of all: the Admiralty.
            ]],
            DIALOG_ADMIRALTY = [[
                
                agent:  
                    !right
                    Excuse you, citizen. We have a few questions.
                    {used_violence_at_beaching?
                        We heard there was a kerfuffle at the Beaching.
                    }
                    
                    {not used_violence_at_beaching?
                        Are you an associate of the smuggler known as {smuggler}?
                    }
                    

            ]],

            OPT_ATTACK = "Attack the Admiralty",
            DIALOG_ATTACK = [[
                player:
                    You'll never take us alive, switch!
                dignitary:
                    !left
                    !placate
                    Stop this at once! I have diplomatic immunity!
                player:
                    !left
                    !fight
                    I don't!
            ]],
            DIALOG_WON = [[
                agent:
                    !exit
                * The fight is over, but somehow {dignitary} doesn't look pleased.
            ]],
            DIALOG_WON_SMUGGLER_DEAD = [[
                agent:
                    !exit
                * {smuggler} is dead, and {dignitary} looks exhausted. 
                
                dignitary:
                    !right
                    Your recklessness has caused both our nations to lose today.
                    I hope, in the future, you find the strength to control yourself.
            ]],

            OPT_CONVINCE_LEAVE = "Convince the Admiralty to leave",
            DIALOG_CONVINCE_LEAVE = [[
                player:
                    Oh, this?
                    {dignitary} over there just has a thing for wildlife. 
                    {dignitary.HeShe}'s just stomping around looking for grubs to collect or something.
            ]],
            DIALOG_CONVINCE_LEAVE_FAILURE = [[
                player:
                    It's probably gonna take a while, so there's no point in you sticking around.
                agent:
                    Is that right? Then we'll just have to interrupt. 
            ]],
            DIALOG_CONVINCE_LEAVE_SUCCESS = [[
                player:
                    I'm holding on to some samples for {dignitary.himher}. You want any?
                * You show the Admiralty the bugs crawling up and down your legs.
                agent:
                    N-no, that's alright. 
                    Ugh, Rentorians are disgusting.
                * The Admiralty leave before you can catch their cooties for {dignitary}'s "collection", too.
                agent:
                    !exit
            ]],

            DIALOG_DONE = [[
                dignitary:
                    !right
                    I'm done.
                    Let's go!
            ]],

            SIT_MOD = "Investigating violence at the Great Beaching",

            
        }
        :Fn(function(cxt) 

            cxt.quest.param.did_secret_meeting = true

            cxt:Dialog("DIALOG_INTRO")

            local admiralty = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank(), cxt.location, true, function(agent) return agent:GetRelationship() < RELATIONSHIP.LIKED end)            
            cxt:TalkTo(admiralty[1])
            local sit_mods
            if cxt.quest.param.used_violence_at_beaching then
                sit_mods =  { { value = -20, text = cxt:GetLocString("SIT_MOD") } } 
            end
            cxt:Dialog("DIALOG_ADMIRALTY")

            
            cxt:RunLoop(function() 

                cxt:BasicNegotiation("CONVINCE_LEAVE", {situation_modifiers = sit_mods})
                    :OnSuccess()
                        :CompleteQuest("go_to_secret_meeting")
                        :Travel()

                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{ no_oppo_limit = true, noncombatants = {"dignitary"}, allies = {"smuggler"}}
                        :OnWin()
                            :Dialog(cxt.quest:GetCastMember("smuggler"):IsDead() and "DIALOG_WON_SMUGGLER_DEAD" or "DIALOG_WON")
                            :CompleteQuest("go_to_secret_meeting")
                            :Travel()
            end)
        end)


QDEF:AddConvo("go_to_departure")
    :ConfrontState("MEET", function(cxt) return cxt.location == cxt.quest:GetCastMember("departure") end)
        :Loc{
            DIALOG_INTRO = [[
                * You're about to help {dignitary} back onto the boat when you're stopped by a group of Heshians.
                zealot: 
                    !right
                    !angry
                    Not so fast, heathen!
                dignitary:
                    !left
                    I'm sorry, are you addressing me?
                    As I mentioned to my escort here, I walk in the shallows.
                zealot:
                    Hah! Hesh wouldn't extend a tentacle to a Rentorian, anyway.
                    You're a pretender, nothing more.
                    And while the Cult elders might not see through you, we do. 
                    You won't leave Havaria alive. 
            ]],

            OPT_CONVINCE_LEAVE = "Convince {agent} to let {dignitary} leave",
            DIALOG_CONVINCE_LEAVE = [[
                player:
                    !left
                    Let's take a step back, huh?
                    The war was the Admiralty's, not the Cult's. 
                    You really want to risk starting another one?
            ]],
            DIALOG_CONVINCE_LEAVE_FAILURE = [[
                player:
                    Is that what Hesh is telling you to do?
                zealot:
                    I look at the Rent and my gut curdles with loathing.
                    So yes!
            ]],
            DIALOG_CONVINCE_LEAVE_SUCCESS = [[
                player:
                    Just who do you think that benefits, huh?
                    Not your church, that's for sure.
                zealot:
                    ...
                    I suppose the Admiralty has finally stepped in line. 
                    It would be a shame to do anything that might give them more power. 
                    You can pass through for now, Rent, but we're watching you. 
                    !exit
            ]],

            OPT_LET_TAKE = "Let {zealot} take {dignitary}",
            DIALOG_LET_TAKE = [[
                player:
                    !left
                    !shrug
                    Sorry, {dignitary}. This day's gone on long enough, and I'm gettin' sleepy.
                    They can have you as far as I'm concerned.
                dignitary:
                    !right
                    !angry
                    Interesting to see what your protection is worth, then.
                    You have yet to see how dark the sky can really get, {player}. 
                    !exit
                zealot: 
                    !right
                    Good to see you're Hesh-fearing, {player}. May you walk in the shallows.
                    !exit
            ]],

            OPT_DEFEND = "Defend {dignitary}",
            DIALOG_DEFEND = [[
                player:
                    !left
                    Not on my watch!
                    !fight
            ]],
            DIALOG_WON = [[
                {dead?
                    * {zealot} lies dead in a pile of {zealot.hisher} own self-righteousness.
                }
                {not dead?
                    zealot:
                        !injured
                        Hesh will gnaw on the bones of your soul, grifter!
                        !exit
                }
            ]],

            DIALOG_DEAD_DIGNITARY = [[
                * Unfortunately, {dignitary} is dead. You'll have to tell {tei} that you messed up your babysitting job.
            ]],
            DIALOG_DIGNITARY_THANK = [[
                dignitary:
                    !right
                    Thank you for your assistance, {player}.
                    
                    {went_to_secret_meeting?
                        Tell no-one of our dealings with {smuggler}.
                    }

                    !exit
            ]],

        }
        :Fn(function(cxt) 
            cxt:TalkTo("zealot")
            CreateCombatBackup(cxt:GetAgent(), {"LUMINITIATE", "LUMICYTE"})
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:RunLoop(function()
                cxt:BasicNegotiation("CONVINCE_LEAVE")
                    :OnSuccess()
                        
                        :CompleteQuest("go_to_departure")
                        :Dialog("DIALOG_DIGNITARY_THANK")
                        :Fn(function() 
                            cxt:Opinion(cxt.quest.param.did_secret_meeting and "succeeded_mission_extra" or "good_impression", "dignitary")
                            cxt:GetCastMember("dignitary"):Retire() 
                        end)
                        :Travel()

                
                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{no_oppo_limit = true, allies = {"dignitary"}}
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :Fn(function() 
                                if cxt.quest:GetCastMember("dignitary"):IsDead() then
                                    cxt:Dialog("DIALOG_DEAD_DIGNITARY")
                                else
                                    cxt:Dialog("DIALOG_DIGNITARY_THANK")
                                    cxt:Opinion(cxt.quest.param.did_secret_meeting and "succeeded_mission_extra" or "good_impression", "dignitary")
                                    cxt:GetCastMember("dignitary"):Retire() 
                                end
                                cxt.quest:Complete("go_to_departure")
                            end)
                            :Travel()
                
                cxt:Opt("OPT_LET_TAKE")
                    :Dialog("DIALOG_LET_TAKE")
                    :ReceiveOpinion("betrayed_to_cult", nil, "dignitary")
                    :ReceiveOpinion("handed_over_rentorian")
                    :CompleteQuest("go_to_departure")
                    :Fn(function() cxt:GetCastMember("dignitary"):Retire()  end)
                    :Travel()
            end)

        end)

--this can happen any time after the beaching, if the dignitary likes you.
QDEF:AddConvo()
    :TravelConfront("MEET", function(cxt) return cxt.quest:GetCastMember("dignitary"):IsInPlayerParty() and cxt.quest:IsComplete("go_to_meeting") and cxt.quest:GetCastMember("dignitary"):GetRelationship() >= RELATIONSHIP.LIKED and not cxt.quest.param.offered_secret_meeting end)
        :Loc{
            DIALOG_INTRO = [[
                * {dignitary} stops.
                player:
                    !left
                dignitary:
                    !right
                    There is one last task I'd like to achieve while on Havarian soil, {player}.
                    I hope that I'm not wrong in my assessment of your character.
                    Can I trust you to accompany me to one more meeting?
                    It hasn't been approved by the Cult, and it may be dangerous.
                player:
                    Dangerous? Heshin' <i>finally</i>.
                    Beginning to think this job would put me in a coma.
                    Sure, Rento. Count me in.
            ]],
        }
        :Fn(function(cxt) 
            cxt.quest.param.offered_secret_meeting = true
            cxt.quest:Activate("go_to_secret_meeting")
            cxt:Dialog("DIALOG_INTRO")
            StateGraphUtil.AddLeaveLocation(cxt)
        end)




-- we also have  "meeting_went_poorly" as a variable to use here, too. The idea is to make it look like tei meant for everything to happen than did.

QDEF:AddConvo( "return_to_tei", "tei" )
    :Loc{
        OPT_HAND_IN = "Give {tei} a report",
        
        DIALOG_DINGITARY_DEAD = [[
            player:
                So, I have bad news.
            agent:
                The dignitary is dead?
            player:
                Maybe? Seized by some angry Heshians, anyway.
            agent:
                I figured that was a possibility.
                If Hesh wills it, so be it. I'm glad you were there all the same.
            player:
                Wait... you were?
            agent:
                Yes? That's why I asked you.
            player:
                Right but... normally when I screw up, folks have a little more to say than "you did your best".
            agent:
                You aren't a Luminari, Smith. You're just a--forgive the phrase--"regular schmuck".
                I didn't ask you to help assuming you'd keep the Rentorian alive.
                I asked you because even if {dignitary} did die, the Rentorians would be able to blame <i>you</i>, and not Havaria as a whole.
            player:
                What?! So I'm you're Heshin' patsy?
            agent:
                Well, yes. That's the advantage you provide.
        ]],
        
        DIALOG_HAND_IN = [[
            player:
                {dignitary} did the thing at the place... and then went back home.
            agent:
                Really?
                Unharmed?
            player:
                Why are you so surprised?
            agent:  
                Apologies. It is, of course, exactly how I expected it to go.
                !shrug
            player:
                Uh huh. 
                Well, you're welcome.
        ]],
        DIALOG_SECRET_MEETING = [[
            agent:
                Oh. One more thing. Did {dignitary} meet with anyone else while {dignitary.heshe} was here?
            player:
                ...
                I'm not supposed to say.
            agent:
                Cute.
                But thank you for your discretion. 
        ]],
    }
    :Hub(function(cxt)
            cxt:Opt("OPT_HAND_IN")
                
                :Fn(function() 
                    if cxt:GetCastMember("dignitary"):IsDead() then
                        cxt:Dialog("DIALOG_DINGITARY_DEAD")
                    else
                        cxt:Dialog("DIALOG_HAND_IN")
                    end
                    
                    if cxt.quest.param.did_secret_meeting then
                        cxt:Dialog("DIALOG_SECRET_MEETING")
                    end

                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:Complete()
                end)
                :DoneConvo()
    end)



QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}

