local QDEF = QuestDef.Define{
    title = "A Test of Faith",
    qtype = QTYPE.CONTRACT,
    rank = {1, 4},
    act_filter = "SMITH",
    desc = [[{tei} wants you to test the faith of one of the Cult's newest recruits.]],
    icon = engine.asset.Texture("icons/quests/smith_tei_recruit_filter.tex"),
    tags = {"TEI_INTRO_JOB"},
    focus = QUEST_FOCUS.COMBAT,


    on_start = function(quest)
    end,

    on_destroy = function(quest)
        if quest:GetCastMember("recruit1") and quest:GetCastMember("recruit1"):IsInPlayerParty() then
            quest:GetCastMember("recruit1"):Dismiss()
        end
        if quest:GetCastMember("recruit2") and quest:GetCastMember("recruit2"):IsInPlayerParty() then
            quest:GetCastMember("recruit2"):Dismiss()
        end
    end,

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("tei"), location = TheGame:GetGameState():GetLocation("TEI_SHRINE"), role = CHARACTER_ROLES.VISITOR} )
    end,

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastMember("recruit1") then
            fighter:AddCondition("WANTED_ALIVE")
        end
        if fighter.agent == quest:GetCastMember("recruit2") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end

}


:AddQuestLocation{
    cast_id = "beach",
    name = "Krill Sighting",
    plax = "EXT_SMITH_BEACHCREATURE",
    map_tags = {"beach"},
}


:AddCastByAlias{
  cast_id = "tei",
  alias = "TEI",
  provider = true,
  no_validation = true,
}

:AddCastByAlias{
    cast_id = "moreef",
    no_validation = true,
    alias = "SWEET_MOREEF",
}

:AddCast{
    cast_id = "recruit1",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "LUMINITIATE" ) )
    end,
}

:AddCast{
    cast_id = "recruit2",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "LUMINITIATE" ) )
    end,
}

:AddLocationCast{
    cast_id = "eats",
    cast_fn = function(quest, t)
        table.insert(t,  TheGame:GetGameState():GetLocation("PEARL_FANCY_EATS"))
    end,
}

:AddLocationCast{
    cast_id = "bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MOREEF_BAR"))
    end,
}


:AddObjective{
    id = "find_recruit1",
    title = "Go find {recruit1}",
    on_activate = function(quest) 
        quest:AssignCastMember("recruit1")
    end,
    on_complete = function(quest)
        quest:Activate("find_recruit2")
    end,
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("recruit1"), location = quest:GetCastMember("eats"), role = CHARACTER_ROLES.PATRON} )
    end,

    mark = {"eats"},
}

:AddObjective{
    id = "find_recruit2",
    title = "Go find {recruit2}",
    on_activate = function(quest) 
        quest:AssignCastMember("recruit2")
    end,
    on_complete = function(quest)
        quest:Activate("go_to_beach")
    end,
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("recruit2"), location = quest:GetCastMember("bar"), role = CHARACTER_ROLES.PATRON} )
    end,

    --desc = [[{handler} has a reward for you.]],
    mark = {"recruit2"},
    --state = QSTATUS.ACTIVE,
}

:AddObjective{
    id = "go_to_beach",
    title = "Hunt the Krill",
    desc = "Take your charges to clear the beach for {tei}.",
    mark = {"beach"},
}


:AddObjective{
    id = "return_to_tei",
    title = "Return to {tei}",
    --desc = [[{handler} has a reward for you.]],
    mark = {"tei"},
}

:AddObjective{
    id = "tell_tei_killed_both",
    title = "Tell {tei} what happened",
    --desc = [[{handler} has a reward for you.]],
    mark = {"tei"},
}


QDEF:AddIntro(
    --attract spiel
    [[
            agent:
                The Cult's been recruiting heavily since the Great Beaching.
                Too heavily, if you ask me. 
                Some of these new recruits are more motivated by the fear of their own hunger than out of good, honest fear of Hesh.
            player:
                !thought
                I have noticed the overall quality of goon has dropped since the last time I was in town.
            agent:
                Indeed. There's a pair of recruits that are on my radar for all of the wrong reasons.
                I want you to see if these new fish really know how to swim.
    ]],
    
    --on accept
    [[
        agent:
            Go pull the recruits out of whatever hole they've crawled into, and take 'em to the beach.
            There are reports of Krill coming ashore. 
            If they can survive that battle, they deserve to stay.
        player:
            Why would they listen to me?
        agent:
            !dismiss
            It's their sacred duty to return threats to the sea. That should be enough.
        player:
            !dubious
            ...
        agent:
            !shrug
            I don't know. Tell them I sent you. Or threaten them with physical violence. You're supposed to be good at these things.
    ]],
    "find_recruit1"
    )



QDEF:AddConvo( "find_recruit1", "recruit1" )
    :ConfrontState("CONF")
    :Loc{
        DIALOG_INTRO = [[
            * You enter the dining room, and are confronted with an angry scene.
            recruit1:
                !left
                !angry
                I am a representative of the almighty Hesh!
            agent:
                !right
                !angry
                I don't care who you represent, you're not going anywhere without paying!
            recruit1:
                !exit
            * {recruit1} returns to {recruit1.hisher} meal.
            agent:
                !disgust
                Damn cultists!
            player:
                !left
            agent:
                What do you want, grifter? I've got enough riff-raff taking up space here already.
        ]],
    }
    :Fn(function(cxt)
        local prop = cxt.location:GetProprietor()
        cxt:ReassignCastMember("prop", prop)
        
        cxt:TalkTo(prop)
        cxt:Dialog("DIALOG_INTRO")
        cxt:GoTo("STATE_PROP")
    end)

    :State("STATE_PROP")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                {not first_time?
                    agent:
                        See what I'm dealing with?
                }

            ]],

            OPT_ASK_ABOUT_HESH = "Ask about {recruit1}",
            DIALOG_ASK_ABOUT_HESH = [[
                player:
                    What's your beef with {recruit1}?
                agent:
                    {recruit1.HeShe}'s a freeloader! Ordered the most expensive dish in the house, and now refuses to pay.
                    Says it's my 'religious duty' to feed the faithful.
                    !spit
                    These Cultists are worse than grifters.
            ]],

            OPT_PAY_FOR_MEAL = "Pay for {recruit1}'s meal",
            DIALOG_PAY_FOR_MEAL = [[
                player:
                    I'll cover the cost.
                    !give
                agent:
                    !neutral
                    !take
                    Really? Why would you do that?
                    Not that I'm complaining, of course.
                player:
                    I suppose I just have a generous heart.
                agent:
                    !shrug
                    Whatever. Do me another favour and get that no-goodnik out of here.
                    !exit
                * {agent} returns to cooking.
            ]],

            DIALOG_GET_RECRUIT = [[
                recruit1:
                    !right
                    So what's your angle, anyway?
                player:
                    {tei} sent me to collect you. You're on beach duty.
                recruit1:
                    !scared
                    Beach duty?
                    Well, at least my last meal was a good one.
                    !exit
                * {recruit1} joins you.
            ]],

            OPT_CONVINCE_CHARITY = "Convince {agent} to donate the food",
            DIALOG_CONVINCE_CHARITY = [[
                player:
                    You want to stay on the Cult's good side, don't you?
            ]],
            DIALOG_CONVINCE_CHARITY_SUCCESS = [[
                agent:
                    Ugh. You're probably right.
                    Fine. This one's free. But I'm asking for payment up-front from now on.
                    !exit
                recruit1:
                    !right
                    Hey, thanks pal!
                    To what do I owe this honor?
                player:
                    {tei} sent me. You've got beach duty, 'pal'.
                recruit1:
                    What? I...
                    OK.
                    !exit
                * {recruit1} takes {recruit1.hisher} last bite and follows you.
            ]],
            DIALOG_CONVINCE_CHARITY_FAILURE = [[
                agent:
                    That'll just encourage more of these louts to show up asking for a handout.
                    !crossed
                    No way. {recruit1.HeShe} pays up, or I call in secuirty.
            ]],

            OPT_TALK_TO_HESH = "Talk to {recruit1}",
            DIALOG_TALK_TO_HESH = [[
                player:
                    I'll deal with this...
                    !exit
                agent:
                    !exit
                * You approach {recruit1}.
            ]],

        }
        :SetLooping()
        :Fn(function(cxt)
            cxt:TalkTo("prop")
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:RunLoop(function()
                cxt:Question("OPT_ASK_ABOUT_HESH", "DIALOG_ASK_ABOUT_HESH")
                
                cxt:Opt("OPT_PAY_FOR_MEAL")
                    :Dialog("DIALOG_PAY_FOR_MEAL")
                    :DeliverMoney(20)
                    :Dialog("DIALOG_GET_RECRUIT")
                    :CompleteQuest("find_recruit1")
                    :Fn(function() 
                        cxt.quest:GetCastMember("recruit1"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                    end)
                    :Travel()


                cxt:Opt("OPT_CONVINCE_CHARITY")
                    :Dialog("DIALOG_CONVINCE_CHARITY")
                    :Negotiation{
                        helpers = {"recruit1"}, 
                    }
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_CHARITY_SUCCESS")
                            :CompleteQuest("find_recruit1")
                            :Fn(function() 
                                cxt.quest:GetCastMember("recruit1"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            end)
                            :Travel()

                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_CHARITY_FAILURE")
                
                cxt:Opt("OPT_TALK_TO_HESH")
                    :GoTo("STATE_HESH")
            end)
        end)


    :State("STATE_HESH")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                {first_time?
                    agent:
                        !right
                        !hesh_greeting
                        How may I be of service, grifter?
                        !happy
                        Ha!
                        As if! 
                        I'm eating here, don't bug me.
                }
                {not first_time?
                    agent:
                        !right
                        !crossed
                        You again?
                }
            ]],
            OPT_ASK_ABOUT_HESH = "Ask why {agent} won't pay",
            DIALOG_ASK_ABOUT_HESH = [[
                player:
                    Why don't you want to pay your bill?
                agent:
                    Ain't it obvious?
                    I risk my life all day long in service of the Cult. 
                    I deserve this - I could die at any moment!
            ]],

            OPT_CONVINCE_PAY = "Convince {agent} to pay up",
            DIALOG_CONVINCE_PAY = [[
                player:
                    You're giving the Cult a bad name. You need to pay your bill.
            ]],
            DIALOG_CONVINCE_PAY_SUCCESS = [[
                agent:
                    Ugh. You sound like {tei}.
                    I'll pay, but only to make you shut your mouth.
                prop:
                    !right
                    Thank you, grifter.
                    !exit
            ]],

            DIALOG_CONVINCE_PAY_FAILURE = [[
                agent:
                    Nice try, trying to appeal to my sense of shame.
                    But the jokes on you!
                    !laugh
                    I have no sense of shame!
            ]],

            OPT_TALK_TO_PROP = "Talk to {prop}",
            DIALOG_TALK_TO_PROP = [[
                player:
                    A moment, please.
                    !exit
                agent
                    !exit
                * You return to {prop}.
            ]],

            OPT_FORCE_HESH_OUT = "Force {agent} to leave",
            DIALOG_FORCE_HESH_OUT = [[
                player:
                    !fight
                    Time to go, buddy.
                agent:
                    !fight
            ]],
            DIALOG_KILLED_RECRUIT = [[
                * {recruit1} slumps over on a table.
                prop:
                    !right
                    You idiot!
                    Look what you've done!
                player:
                    !bashful
                    Uh... sorry about the mess?
                    !exit
                * You leave before you have to face the consequences of your behaviour.
            ]],

            DIALOG_RECRUIT_ALIVE = [[
                recruit1:
                    !injured
                    Geez, alright, who are you anyway?
                player:
                    !neutral
                    {tei} sent me. We're going to the beach.
                recruit1:
                    !scared
                    Oh no... oh Hesh help me.
                    !exit
                * {recruit1}, broken, falls in behind you.
                prop:
                    !right
                    Who's going to pay for this mess?
                player:
                    !happy
                    Send the bill to {tei}!
                    !exit
            ]],

            OPT_TALK_TO_PROP = "Talk to {prop}",
        }
        :SetLooping()
        :Fn(function(cxt)
            cxt:TalkTo("recruit1")
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:RunLoop(function()
                cxt:Question("OPT_ASK_ABOUT_HESH", "DIALOG_ASK_ABOUT_HESH")

                cxt:Opt("OPT_CONVINCE_PAY")
                    :Dialog("DIALOG_CONVINCE_PAY")
                    :Negotiation{
                        helpers = {"prop"}, 
                    }
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_PAY_SUCCESS")
                            :Fn(function() 
                                cxt.quest:GetCastMember("recruit1"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                cxt.quest:Complete("find_recruit1")
                                StateGraphUtil.AddLeaveLocation(cxt)   
                            end)
                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_PAY_FAILURE")

                cxt:Opt("OPT_FORCE_HESH_OUT")
                    :Dialog("DIALOG_FORCE_HESH_OUT")
                    :Battle{flags = BATTLE_FLAGS.NO_BYSTANDERS }
                        :OnWin()
                            :Fn(function(cxt) 
                                if cxt:GetAgent():IsDead() then
                                    cxt.quest.param.killed_recruit1 = true
                                    cxt:Dialog("DIALOG_KILLED_RECRUIT")
                                else
                                    cxt:Dialog("DIALOG_RECRUIT_ALIVE")
                                    cxt.quest:GetCastMember("recruit1"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                end
                                cxt.quest:Complete("find_recruit1")
                                StateGraphUtil.AddLeaveLocation(cxt)   

                            end)

                cxt:Opt("OPT_TALK_TO_PROP")
                    :GoTo("STATE_PROP")
            end)
        end)



QDEF:AddConvo( "find_recruit2" )
    :ConfrontState("CONF",  function(cxt) return cxt.location == cxt.quest:GetCastMember("bar") and not cxt.quest.param.seen_bar_intro end)
        :Loc{
            DIALOG_INTRO = [[
                * You find the second recruit carousing at {moreef}'s bar.
                recruit2:
                    !left
                    !drunk
                    Another round, barkeep!
                
                moreef:
                    !right  
                    !sigh
                    If you insist.
            ]],
        }
    :Fn(function(cxt) 
        AgentUtil.MakeDrunk( cxt.quest:GetCastMember("recruit2")) 
        cxt.quest.param.seen_bar_intro = true
        cxt:Dialog("DIALOG_INTRO")
    end)


QDEF:AddAttract("find_recruit2", "moreef",

    [[
        {first_time?
            agent:
                Oh, hey {player}.
                These Cultists are such a pain in my pocketbook!
        }
        
        {not first_time?
            agent:
                How much can one person drink?!            
        }
            
    ]])


QDEF:AddConvo( "find_recruit2", "moreef")
    :Loc{
        OPT_ASK_ABOUT_HESH = "Ask about {recruit2}",
        DIALOG_ASK_ABOUT_HESH = [[
            player:
                What's the deal with {recruit2}?
            agent:
                {recruit2.HeShe} showed up just after you left on your mysterious mission.
                Been drinking ever since.
            player:
                How does a lowly luminitiate afford that?
            agent:
                The Cult has an open tab.
                !sigh
                And by open tab, I mean... I have to let them drink as much as they want if I don't want to find myself on the wrong end of a Zealot's inquisition.
        ]]
    }
    :Hub(function(cxt)
        cxt:Question("OPT_ASK_ABOUT_HESH", "DIALOG_ASK_ABOUT_HESH")
    end)


QDEF:AddConvo( "find_recruit2", "recruit2")
    :AttractState("ATTRACT")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    agent:
                        !hesh_greeting
                        Hi! What can I do for--
                        !drunk_hiccup
                        >hic<
                        --for you?
                }
                {not first_time?
                    agent:
                        !hesh_greeting
                        I'm getting closer to Hesh here!
                }
            ]],
            OPT_ASK_ABOUT_HESH = "Ask {agent} what {agent.heshe} is doing",
            DIALOG_ASK_ABOUT_HESH = [[
                player:
                    What's going on here?
                agent:
                    What does it look like?
                    I'm getting hammered!
                player:
                    Is this a... religious thing?
                agent:
                    ...
                    Sure!
            ]],

            OPT_ASK_COME_WITH = "Tell {agent} to come with you",
            DIALOG_ASK_COME_WITH = [[
                {first_time?
                    player:
                        {tei} sent me. You have beach duty.
                    agent:
                        Beach duty?
                        In this state!
                        Ha!
                        I'd rather not, thanks.
                }

                {not first_time?
                    player:
                        It's time to go.
                    agent:
                        I already told you... I'm good!
                        Have fun at the beach, though.
                }
            ]],



            OPT_FORCE_HESH_OUT = "Force {agent} to come with you",
            DIALOG_FORCE_HESH_OUT = [[
                player:
                    !fight
                    I'm not really asking...
                agent:
                    !fight
            ]],
            DIALOG_KILLED_RECRUIT = [[
                * {recruit1} slumps over on a table.
                moreef:
                    !right
                    {player}!
                    You should go. I'll cover this up!
                player:
                    !bashful
                    Thanks {moreef}. Sometimes I don't know my own strength.
                    !exit
            ]],

            DIALOG_RECRUIT_ALIVE = [[
                recruit2:
                    !injured
                    Alright, alright. I'll go to the damn beach.
                player:
                    !neutral
                    See, we could have started with that.
                recruit2:
                    I hope you get eaten by a lumicyte.
                    !exit
                * {recruit2} reluctantly falls in behind you.
                moreef:
                    !right
                    Thanks, {player}. That was getting expensive!
                player:
                    !happy
                    Any time, {moreef}.
                    !exit
            ]],          

            OPT_THREATEN = "Threaten {agent}",
            DIALOG_THREATEN = [[
                player:
                    This happens to be my territory, Heshie.
                    !threaten
                    You'd do well to listen to what I have to say here.
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                agent:
                    Fine! Let's go.
                    I was done drinking, anyway.
                player:
                    !happy
                    Sure you were!
                agent:
                    !exit
                *  You've gained a reluctant follower!
            ]],
            DIALOG_THREATEN_FAILURE = [[
                agent:
                    You think I'm afraid of you?
                    I'm in the Cult of Hesh!
                    I've got bigger, wetter, scarier things to worry about!
            ]],

            DIALOG_COME_BACK_LATER = [[
                player:
                    Excuse me a moment.
                agent:
                    Whatever! You do you!
            ]],
            REQ_ALREADY_TRIED = "You already tried that",

            OPT_DRINKING_CONTEST = "Challenge {agent} to a drinking contest",
            DIALOG_DRINKING_CONTEST = [[
                player:
                    You're a fan of liquid refreshment, eh?
                agent:
                    I get thirsty! Who doesn't?
                player:
                    Oh, me too, friend. I've been known to get powerfully thirsty.
                agent:
                    A fellow traveler! 
                player:
                    Yeah. So I'll tell you what...
                    If you can outdrink me, {moreef} here will give you 200 shills.
                moreef:
                    !right
                    Huh?
                agent:
                    !right
                    And if I can't?
                player:
                    And if you can't, you have to come with me and do your duty at the beach.
                    Deal?
                agent:
                    ...
                    Deal!
            ]],
        }
        :Fn(function(cxt) 
            cxt:RunLoop(function()
                cxt:Question("OPT_ASK_ABOUT_HESH", "DIALOG_ASK_ABOUT_HESH")

                cxt:Opt("OPT_ASK_COME_WITH")
                    :SetQuestMark()
                    :IsHubOption(true)
                    :Dialog("DIALOG_ASK_COME_WITH")
                    :LoopingFn(function() 
                        

                        cxt:Opt("OPT_DRINKING_CONTEST")
                            :ReqCondition(not cxt.quest.tried_contest, "REQ_ALREADY_TRIED")
                            :Dialog("DIALOG_DRINKING_CONTEST")
                            :Fn(function() cxt.quest.tried_contest = true end)
                            :GoTo("STATE_DRINKING_CONTEST")


                        cxt:Opt("OPT_THREATEN")
                            :Dialog("DIALOG_THREATEN")
                            :Negotiation{flags = NEGOTIATION_FLAGS.INTIMIDATION}
                                :OnSuccess()
                                    :Dialog("DIALOG_THREATEN_SUCCESS")
                                    :CompleteQuest("find_recruit2")
                                    :Fn(function() 
                                        cxt.quest:GetCastMember("recruit2"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                    end)
                                    :DoneConvo()
                                    
                                :OnFailure()
                                    :Dialog("DIALOG_THREATEN_FAILURE")
                        

                        cxt:Opt("OPT_FORCE_HESH_OUT")
                            :Dialog("DIALOG_FORCE_HESH_OUT")
                            :Battle{flags = BATTLE_FLAGS.NO_BYSTANDERS }
                                :OnWin()
                                    :Fn(function(cxt) 
                                        if cxt:GetAgent():IsDead() then
                                            cxt.quest.param.killed_recruit2 = true
                                            cxt:Dialog("DIALOG_KILLED_RECRUIT")
                                        else
                                            cxt:Dialog("DIALOG_RECRUIT_ALIVE")
                                            cxt.quest:GetCastMember("recruit2"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                        end

                                        if cxt.quest.param.killed_recruit1 and cxt.quest.param.killed_recruit2 then
                                            cxt.quest:Cancel("find_recruit2")    
                                            cxt.quest:Activate("tell_tei_killed_both")
                                        else
                                            cxt.quest:Complete("find_recruit2")
                                        end
                                        
                                        StateGraphUtil.AddEndOption(cxt)

                                    end)

                    end)


                StateGraphUtil.AddEndOption(cxt)
                    :Dialog("DIALOG_COME_BACK_LATER")
            end)
        end)

    :State("STATE_DRINKING_CONTEST")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !exit
                agent:
                    !exit

                * You join {agent} at {agent.hisher} table and wave {moreef} over.
                moreef:
                    !right
                    I didn't agree to this, {player}... that's more money than I can lose tonight.
                player:
                    !left
                    Don't worry about it. I can hold my own against this lightweight.
                moreef:
                    !sigh
                    You better.
                agent:
                    !right
                    So are we going to talk, or are we going to drink?!
            ]],

            OPT_DRINK = "Drink!",
            OPT_BAIL = "Bail!",

            DIALOG_CHEERS = [[
                player:
                    !cheers
                agent:
                    !cheers
            ]],
            DIALOG_BAIL_1 = [[
                player:
                    On second thought, this is an irresponsible decision.
                agent:
                    Easy money!
            ]],

            DIALOG_DRINK_1 = [[
                player:
                    Hits the spot, doesn't it?
                agent:
                    That's nothing. I drank more than that before breakfast.
                    Let's keep going.
            ]],

            DIALOG_BAIL_2 = [[
                player:
                    You know what, I have to grift home after this.
                    You win.
                agent:
                    Excuses, excuses! 
                    Now get out of my face!
            ]],

            DIALOG_DRINK_2 = [[
                player:
                    Ahhhhh. Refreshing. 
                agent:
                    Are you feeling this yet?
                    I'm not!
                    Time for more!
            ]],

            DIALOG_BAIL_3 = [[
                player:
                    I may have overestimated my own abilities.
                    !permit
                    Respect.
                agent:
                    And my mom said I had no marketable skills!
                    If only she could see me now!
            ]],

            DIALOG_DRINK_3 = [[
                player:
                    ...
                    !burp
                    Excuse me.
                agent:
                    I...
                    !exit
                * {agent} falls over.
                agent:
                    !right
                    !injured
                    DId I just lose?
                player:
                    !thought
                    I think we may have both lost.
                    !happy
                    But you definitely lost more than me!
                agent:
                    !exit
                    * {agent} sways as {agent.heshe} gathers {agent.hisher} belongings and joins you.
            ]],


            DIALOG_LOST = [[
                moreef:
                    !right
                    Damn it, {player}. You have a big mouth, you know that?
                player:
                    !shrug
                    It's just money, {moreef}.
                    You should be happy that I'm making healthy choices.
            ]],

            DIALOG_WON = [[
                
                moreef:
                    !right
                    You're a madman.
                player:
                    I told ya I was good for it! 
                    Now we gotta go fight some fishies!
                moreef:
                    !sigh
                    Just come back in one piece.
            ]],

        }
    :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_DRINK")
                :Dialog("DIALOG_CHEERS")
                :DoDrink( 0, DRINK_RESTORE_RESOLVE_AMOUNT )
                :Dialog("DIALOG_DRINK_1")
                :Fn(function() 

                    cxt:Opt("OPT_DRINK")
                        :Dialog("DIALOG_CHEERS")
                        :DoDrink( 0, DRINK_RESTORE_RESOLVE_AMOUNT )
                        :Dialog("DIALOG_DRINK_2")
                        :Fn(function() 

                            cxt:Opt("OPT_DRINK")
                                :Dialog("DIALOG_CHEERS")
                                :DoDrink( 0, DRINK_RESTORE_RESOLVE_AMOUNT )
                                :Dialog("DIALOG_DRINK_3")
                                :Fn(function() 
                                    cxt.quest:GetCastMember("recruit2"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                end)
                                :CompleteQuest("find_recruit2")
                                :Dialog("DIALOG_WON")
                                :DoneConvo()

                            cxt:Opt("OPT_BAIL")
                                :Dialog("DIALOG_BAIL_3")
                                :Dialog("DIALOG_LOST")
                                :DoneConvo()
                        end)

                    cxt:Opt("OPT_BAIL")
                        :Dialog("DIALOG_BAIL_2")
                        :Dialog("DIALOG_LOST")
                        :DoneConvo()
                end)
            cxt:Opt("OPT_BAIL")
                :Dialog("DIALOG_BAIL_1")
                :Dialog("DIALOG_LOST")
                :DoneConvo()
        end)


QDEF:AddConvo( "go_to_beach" )
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("beach") end)
    :Loc{
        DIALOG_INTRO = [[
            * You find the spot {tei} told you about.
            recruit:
                !left
                I don't see any-
            agent:
                !right
        ]],

        OPT_ATTACK = "Attack the monsters!",
        DIALOG_ATTACK = [[
            player:
                !left
                !fight
                Don't just stand there!
        ]],
        DIALOG_WON_SURVIVORS = [[
            * The last of the monster's blood falls into the surf.
            agent:
                !exit
            player:
                !left
                !happy
                That wasn't so bad, was it?
            survivor:
                !right
                This was supposed to be an easy gig...
            player:
                You really thought that?
            survivor:
                ...
                Well, yeah!
            player:
                Ha. And people call me gullible.
                Anyway, enjoy your life of faith and violence.
                !exit
            * {survivor} stares out over the water, pondering the choices that have led {survivor.himher} to this moment.
        ]],

        DIALOG_WON_NO_SURVIVORS = [[
            * You stand alone on the beach, surrounded by seafoam, blood, and regret.
            player:
                !happy
                Well, at least <b>I</> survived.
                !exit
            * You trudge off in the direction of {tei}'s shrine to share the good news.

        ]],
    }
    :Fn(function(cxt)
        local recruits = {}
        if cxt.quest:GetCastMember('recruit1'):IsInPlayerParty() then
            table.insert(recruits, cxt.quest:GetCastMember('recruit1'))
        end
        if cxt.quest:GetCastMember('recruit2'):IsInPlayerParty() then
            table.insert(recruits, cxt.quest:GetCastMember('recruit2'))
        end

        cxt:ReassignCastMember("recruit", recruits[1])
        
        local opfor =  CreateCombatParty({"LUMINTHIAN"}, 1, cxt.location)
        cxt:TalkTo(opfor[1])
        cxt:Dialog("DIALOG_INTRO")

        cxt:Opt("OPT_ATTACK")
            :Dialog("DIALOG_ATTACK")
            :Battle{
                allies = recruits,
            }
                :OnWin()
                    :Fn(function() 
                        local recruits = {}
                        if cxt.quest:GetCastMember('recruit1'):IsInPlayerParty() then
                            table.insert(recruits, cxt.quest:GetCastMember('recruit1'))
                        end
                        if cxt.quest:GetCastMember('recruit2'):IsInPlayerParty() then
                            table.insert(recruits, cxt.quest:GetCastMember('recruit2'))
                        end
                        if #recruits > 0 then
                            cxt:ReassignCastMember("survivor", recruits[1])
                            cxt:Dialog("DIALOG_WON_SURVIVORS")
                            cxt.quest:Complete("go_to_beach")
                            cxt.quest:Activate("return_to_tei")
                        else
                            cxt:Dialog("DIALOG_WON_NO_SURVIVORS")
                            cxt.quest:Complete("go_to_beach")
                            cxt.quest:Activate("tell_tei_killed_both")
                        end

                        if cxt.quest:GetCastMember("recruit1") and cxt.quest:GetCastMember("recruit1"):IsInPlayerParty() then
                            cxt.quest:GetCastMember("recruit1"):Dismiss()
                        end
                        if cxt.quest:GetCastMember("recruit2") and cxt.quest:GetCastMember("recruit2"):IsInPlayerParty() then
                            cxt.quest:GetCastMember("recruit2"):Dismiss()
                        end

                        StateGraphUtil.AddLeaveLocation(cxt)

                    end)
    end)


QDEF:AddConvo( "tell_tei_killed_both", "tei" )
    :Loc{
        OPT_TELL_TEI = "Tell {tei} what happened",
        DIALOG_TELL_TEI = [[
            player:
                So about those recruits...
            agent:  
                Where are they?
            player:
                Dead. Both of 'em.
            agent:
                See, I told the prefect that we were scraping the bottom of the rainbarrel with this latest batch.
            player:
                You're not mad at me?
            agent:
                Should I be?
            player:
                ...
                No.
            agent:
                Well, then. You did your part, so you get paid.
        ]]
    }
    :Hub(function(cxt)
            cxt:Opt("OPT_TELL_TEI")
                :Dialog("DIALOG_TELL_TEI")
                :Fn(function() 
                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:Complete()
                end)
                :DoneConvo()
    end)


QDEF:AddConvo( "return_to_tei", "tei" )
    :Loc{
        OPT_HAND_IN = "Give {tei} a report",
        DIALOG_HAND_IN = [[
            {both_alive?
                player:
                    I am happy to report that both of your flunkies have survived the test.
                agent:
                    Really? That's surprising. I hope you didn't help them too much.
                player:
                    !happy
                    I tend not to help people out for free.
                agent:
                    Indeed. Speaking of such, here is your pay.
                    !give
                player:
                    !take
            }

            {not both_alive?
                player:
                    Not everyone passed the test.
                agent:
                    As expected. You did a great service to the Cult by thinning out our weaker elements.
                player:
                    Yeesh, y'all are harsh.
                agent:
                    If it seems that way, it's only because we are preparing for Hesh to surface.
                player:
                    I'm waiting for my pay to surface.
                agent:
                    Oh. Of course.
                    !give
                player:
                    !take

            }

        ]]
    }
    :Hub(function(cxt)
            cxt.enc.scratch.both_alive =  not cxt.quest:GetCastMember("recruit1"):IsDead() and not cxt.quest:GetCastMember("recruit2"):IsDead()

            cxt:Opt("OPT_HAND_IN")
                :Dialog("DIALOG_HAND_IN")
                :Fn(function() 
                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:Complete()
                end)
                :DoneConvo()
    end)


