local QDEF = QuestDef.Define{
    title = "Shoot the Messenger",
    qtype = QTYPE.CONTRACT,
    rank = {1, 4},
    act_filter = "SMITH",
    desc = [[{tei} needs you to interfere with the diplomatic process.]],
    icon = engine.asset.Texture("icons/quests/smith_tei_shoot_the_messenger.tex"),
    tags = {"TEI_INTRO_JOB"},
    focus = QUEST_FOCUS.COMBAT,


    on_start = function(quest)
    end,

    on_debug_start = function(quest)
        TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("TEI_SHRINE"))
        UIHelpers.DoSpecificConvo( quest:GetCastMember("tei"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
    end,

    on_destroy = function(quest)
        if quest:GetCastMember("messenger") and quest:GetCastMember("messenger"):IsInPlayerParty() then
            quest:GetCastMember("messenger"):Dismiss()
        end
    end,
    
    avoids = {
        {"target", "messenger"}
    }
}

:AddOpinionEvents{
    gave_away_message = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Gave their message to the Spree",
    },
}


:AddCastByAlias{
  cast_id = "tei",
  alias = "TEI",
  provider = true,
  no_validation = true,
}

:AddDefCastSpawn("forger", "FOREMAN")
:AddDefCastSpawn("messenger", "JAKES_SMUGGLER")
:AddDefCastSpawn("target", "ADMIRALTY_CLERK")


:AddExistingLocation("station", "PEARL_ADMIRALTY_STATION")
:AddExistingLocation("oshnudrome", "PEARL_OSHNUDROME")
:AddExistingLocation("bar", "MOREEF_BAR")

:AddQuestLocation{
    cast_id = "forger_workshop",
    name = "Forger's Workshop",
    plax = "INT_SMITH_FORGERYWORKSHOP",
    map_tags = {"quarters"},
}


:AddObjective{
    id = "get_message",
    title = "Get the message",
    negotiation_targets = {"messenger"},
    combat_targets = {"messenger"},

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("messenger"), location = quest:GetCastMember("oshnudrome")} )
    end,
    mark = {"messenger"},
    on_complete = function(quest)
        quest:Activate("modify_the_message")
    end,

}

:AddObjective{
    id = "modify_the_message",
    title = "Change the message",
    
    on_complete = function(quest)
        quest:Activate("deliver_the_message")
        if not quest:GetCastMember("messenger"):IsDead() then
            quest:Activate("pick_up_messenger")
        end
    end,

    is_in_hiding = function(quest, agent)
        if agent == quest:GetCastMember("messenger") then
            return true
        end
    end,

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("forger"), location = quest:GetCastMember("forger_workshop")} )
    end,
    mark = {"forger"},
    
}

:AddObjective{
    id = "pick_up_messenger",
    title = "(Optional) Give the message back to {messenger}",
    desc = "It'll be a lot easier to convince {target} to take the message if it's coming from {messenger}.",
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("messenger"), location = quest:GetCastMember("bar")} )
    end,
    mark = {"messenger"},
}

:AddObjective{
    id = "deliver_the_message",
    title = "Deliver the message",
    desc = "Bring the message to {target}.",
    mark = {"target"},
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("target"), location = quest:GetCastMember("station")} )
    end,
    on_complete = function(quest)
        if quest:IsActive("pick_up_messenger") then
            quest:Cancel("pick_up_messenger")
        end
        quest:Activate("return_to_tei")
    end
}

:AddObjective{
    id = "return_to_tei",
    title = "Tell {tei} what happened",
    --desc = [[{handler} has a reward for you.]],
    mark = {"tei"},
}


QDEF:AddIntro(
    --attract spiel
    [[
        tei:
            The task I have for you is all about bureaucracy, I'm afraid.
            The Admiralty in New Deltree has sent a messenger requesting back-up. Seems the criminal elements in Murder Bay are getting out of hand.
            However, with the Great Beaching...
        player:
            Hah! The Cult doesn't have any Admiralty to spare, is that it?
        tei:
            It isn't the Cult's decision, of course. The Admiralty is autonomous. 
            But I'd like you to intercept the message before it reaches Admiralty command. 
    ]],
    
    --on accept
    [[
        agent:
            The courier is a jake, of course, and this buys us time--
            They picked the wrong jake, though. {messenger} hit the Oshnu track as soon as {messenger.heshe} arrived in town. 
            Go get the missive from {messenger} and take it to {forger} for alterations.
            But leave {messenger} alive--you'll need {messenger.himher} to deliver the altered request.

    ]],
    "get_message"
    )



QDEF:AddConvo( "get_message", "messenger" )
    :Loc{
        OPT_ASK_ABOUT_MESSAGE = "Ask {messenger} about the message...",
        DIALOG_ASK_ABOUT_MESSAGE = [[
            {first_time?
                player:
                    You're {messenger}, right?
                agent:
                    Who's askin'?
                player:
                    You have a package intended for the local Admiralty. I need to, uh... borrow... it for a bit.
                agent:
                    How do you know that?
                    I'll have you know that I am duty-bound to deliver this message unadulterated!
            }

            {not first_time?
                player:
                    So, the message...
                agent:
                    I already told you:
                    Duty. Bound.
            }
        ]],

        OPT_ASK_MESSENGER = "Ask about the messenger business",

        DIALOG_ANSWER_MESSENGER = [[
            player:
                What's it like being a messenger?
            agent:
                The pay's alright. 
                You get to see a lot of different places.
                It's not bad, all things considered.
        ]],
        OPT_ASK_OSHNUDROME = "Ask why {messenger} hasn't delivered the message yet",
        DIALOG_ASK_OSHNUDROME = [[
            player:
                So if you're a duty-bound messenger, why are you here and not, you know...
                Delivering the message?
            agent:
                I like gambling!
        ]],

        OPT_CONVINCE = "Convince {agent} to let you borrow the message",
        DIALOG_CONVINCE = [[
            player:
                I don't want to interfere with your delivery.
                I just want to borrow the message while you're here watching the snails.
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            agent:
                OK, I'll let you hold on to the message for me.
                But I don't want to know what you're doing with it!
                If I don't know, it technically ain't lying!
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            agent:
                No offense, I don't trust you, grifter.
        ]],

        OPT_ATTACK = "Beat up {agent}",
        DIALOG_ATTACK = [[
            player:
                !fight
                Sorry, but I need that message.
        ]],
        DIALOG_ON_WIN = [[
            {dead?
                * {messenger} is dead. You take the package from {messenger.hisher} corpse.
                * You have not failed in your mission, but you just complicated the heck out of it.
            }
            {not dead?
                messenger: 
                    !injured
                    Ow, stop it!
                    Here. Take the message.
                    Just bring it back when you're done, OK?
            }
         ]],

        OPT_RENT = "'Rent' the message for a while",
        DIALOG_RENT = [[
            player:
                How about I rent the message for a couple hours.
            agent:
                Rent it?
            player:
                Yeah. I pay you, you hand me the message, and you don't ask me any more questions about it until I bring it back.
                Deal?
            agent:
                !give
                I never could say no to money!
        ]],

        
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_ABOUT_MESSAGE")
            :Dialog("DIALOG_ASK_ABOUT_MESSAGE")
            :SetQuestMark()
            :IsHubOption(true)
            :Fn(function(cxt) 

                cxt:RunLoop(function() 
                    cxt:Question("OPT_ASK_MESSENGER", "DIALOG_ANSWER_MESSENGER")
                    cxt:Question("OPT_ASK_OSHNUDROME", "DIALOG_ASK_OSHNUDROME")

                    cxt:BasicNegotiation("CONVINCE")
                        :OnSuccess()
                            :CompleteQuest("get_message")
                            :DoneConvo()

                    cxt:Opt("OPT_ATTACK")
                        :Dialog("DIALOG_ATTACK")
                        :Fn(function() cxt.quest.param.attacked_messenger = true end)
                        :Battle{}
                            :OnWin()
                                :Dialog("DIALOG_ON_WIN")
                                :CompleteQuest("get_message")
                                :DoneConvo()

                    cxt:Opt("OPT_RENT")
                        :Dialog("DIALOG_RENT")
                        :DeliverMoney(50)
                        :CompleteQuest("get_message")
                        :DoneConvo()
                end)

            end)
    end)

QDEF:AddConvo( "modify_the_message", "forger" )
    :Loc{
        DIALOG_LINE_1 = [[
            agent:
                'Dear Commander {target}. I hope this message finds <b>{1}</>.'
        ]],
        DIALOG_LINE_2 = [[
            agent:
                'With the upcoming auction, the security situation in Murder Bay is <b>{1}.</b>'
        ]],
        DIALOG_LINE_3 = [[
            agent:
                'Please send <b>{1}</> immediately, before things get any worse.'
        ]],

        WORD_1_ORIG = "its way to you without interference",
        WORD_1_A = "you well",
        WORD_1_B = "your desk",
        WORD_1_C = "your butt",

        WORD_2_ORIG = "tenuous, at best",
        WORD_2_A = "under control",
        WORD_2_B = "completely normal, thank you very much",
        WORD_2_C = "like your butt",

        WORD_3_ORIG = "your reserve troops",
        WORD_3_A = "your thoughts and prayers",
        WORD_3_B = "a shipment of lumin",
        WORD_3_C = "your smelly butt",
    }
    :ConfrontState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    Hey, {tei} sent me. I need you to doctor this letter.
                    Think you can do that?
                    !give 
                agent:
                    !take 
                    Let me see. 
                    Hmmm. Admiralty stationery, standard requisition.
                    A rather chaotic hand. 
                    And a somewhat unprofessional cadence. Unusual for Admiralty. 
                player:
                    So... you can do it?
                agent:
                    Of course I can. What do you want it to say?
                player:
                    Uh, I dunno? Never been much good at writing letters, and {tei} didn't say.
                agent:
                    Then I suppose all that matters is that it's just not the original.
                    Here, I'll cut out the pertinent info. You can just... fill in the blanks.
            ]],
            DIALOG_FIRST = [[
                agent:
                    Alright. Here's the introduction:
            ]],

            DIALOG_A = [[
                agent:
                    Very polite. That's not at all like the Admiralty.
            ]],

            DIALOG_B = [[
                agent:
                    That sounds appropriately bland.
            ]],
            DIALOG_C = [[
                agent:
                    ...
                    That's an interesting choice.
            ]],
        }
        :Fn(function(cxt) 
            
            cxt.enc.scratch.word1 = cxt:GetLocString("WORD_1_ORIG")
            cxt.enc.scratch.word2 = cxt:GetLocString("WORD_2_ORIG")
            cxt.enc.scratch.word3 = cxt:GetLocString("WORD_3_ORIG")

            cxt:Dialog("DIALOG_INTRO")

            cxt:Dialog("DIALOG_FIRST")
            cxt:Dialog("DIALOG_LINE_1", cxt.enc.scratch.word1)
            
            cxt:Opt("WORD_1_A")
                :Dialog("DIALOG_A")
                :Fn(function() cxt.enc.scratch.word1 = cxt:GetLocString("WORD_1_A") end)
                :GoTo("STATE_2")
            cxt:Opt("WORD_1_B")
                :Dialog("DIALOG_B")
                :Fn(function() cxt.enc.scratch.word1 = cxt:GetLocString("WORD_1_B") end)            
                :GoTo("STATE_2")
            cxt:Opt("WORD_1_C")
                :Dialog("DIALOG_C")
                :Fn(function() 

                    cxt.enc.scratch.word1 = cxt:GetLocString("WORD_1_C") 
                    cxt.quest.param.mention_butt = true
                    print ("!!!!", cxt:GetLocString("WORD_1_C"), cxt.enc.scratch.word1)
                end)
                :GoTo("STATE_2")
        end)
    :State("STATE_2")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    The letter continues:
            ]],
            
            DIALOG_A = [[
                agent:
                    To the point. That'll probably pass muster.
            ]],

            DIALOG_B = [[
                agent:
                    You sound like you're overcompensating.
            ]],
            DIALOG_C = [[
                agent:
                    I'm not sure you're taking this exercise seriously.
            ]],
        }
        :Fn(function(cxt) 
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:Dialog("DIALOG_LINE_2", cxt.enc.scratch.word2)
            
            cxt:Opt("WORD_2_A")
                :Dialog("DIALOG_A")
                :Fn(function() cxt.enc.scratch.word2 = cxt:GetLocString("WORD_2_A") end)
                :GoTo("STATE_3")
            cxt:Opt("WORD_2_B")
                :Dialog("DIALOG_B")
                :Fn(function() cxt.enc.scratch.word2 = cxt:GetLocString("WORD_2_B") end)            
                :GoTo("STATE_3")
            cxt:Opt("WORD_2_C")
                :Dialog("DIALOG_C")
                :Fn(function() 
                    cxt.enc.scratch.word2 = cxt:GetLocString("WORD_2_C") 
                    cxt.quest.param.mention_butt = true
                end)
                :GoTo("STATE_3")
        end)
    
    :State("STATE_3")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    And now the ending...
            ]],
            
            DIALOG_A = [[
                agent:
                    Appropriately useless.
            ]],

            DIALOG_B = [[
                agent:
                    They might even send that, too.
            ]],
            DIALOG_C = [[
                agent:
                    You <i>do</i> want this letter to be believed, right?
                player:
                    Enh. Not as much as I want this, apparently.
            ]],
        }
        :Fn(function(cxt) 
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:Dialog("DIALOG_LINE_3", cxt.enc.scratch.word3)
            
            cxt:Opt("WORD_3_A")
                :Dialog("DIALOG_A")
                :Fn(function() cxt.enc.scratch.word3 = cxt:GetLocString("WORD_3_A") end)
                :GoTo("STATE_DONE")
            cxt:Opt("WORD_3_B")
                :Dialog("DIALOG_B")
                :Fn(function() cxt.enc.scratch.word3 = cxt:GetLocString("WORD_3_B") end)            
                :GoTo("STATE_DONE")
            cxt:Opt("WORD_3_C")
                :Dialog("DIALOG_C")
                :Fn(function() 
                    cxt.enc.scratch.word3 = cxt:GetLocString("WORD_3_C") 
                    cxt.quest.param.mention_butt = true
                end)
                :GoTo("STATE_DONE")

        end)

    :State("STATE_DONE")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    So here's what we wrote:
            ]],

            DIALOG_INTRO_2 = [[
                agent:
                    Signed, Lt. Oolo Ollowano of New Deltree.
                player: 
                    !happy
                    Ollowano?
                    I know that family. 
                    Pretty sure I used to beat up the youngest brother. 
                    Anyway, that sounds good to me!
                agent:
                    If you say so.
                * {agent} hands you the forged documents.
            ]]
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Dialog("DIALOG_LINE_1", cxt.enc.scratch.word1)
            cxt:Dialog("DIALOG_LINE_2", cxt.enc.scratch.word2)
            cxt:Dialog("DIALOG_LINE_3", cxt.enc.scratch.word3)

            cxt:Dialog("DIALOG_INTRO_2")
            
            cxt.quest:Complete("modify_the_message")
            StateGraphUtil.AddLeaveLocation(cxt)
        end)







QDEF:AddConvo( "pick_up_messenger", "messenger" )
    :ConfrontState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                * You find {messenger} wrapped around a bottle at Moreef's.
                agent:
                    !right
                player:
                    !left
                    I have your message.
                agent:
                    Great. I was beginning to worry.
                    Hey, the seal is off a bit.
                player:
                    !shrug
                    It was like that when I got it?
                agent:
                    Right. Well, I can fix that.
                * {agent} fixes the seal on the message with a special tool.
                player:
                    I'll never trust a seal again!
                agent:
                    Ha. You don't know the half of it.
                    Well, I'm off.
                    !exit
                player:
                    !exit
                * {agent} is about to leave, when a burly Spree stops {agent.himher} in {agent.hisher} tracks.
            ]],
            DIALOG_SPREE = [[
                agent:
                    !right
                    Hold it right there, Jake.
                messenger:
                    !left
                    Y-yes?
                agent:
                    We've been following you since you left Murder Bay.
                    That message is going to make our lives harder. I'm here to collect it.
                messenger:
                    You'll have to speak to my associate!
                player:
                    !left
                    Huh?
                agent:
                    Hand over the message, or someone's getting hurt.
            ]],

            OPT_HAND_OVER = "Hand over the message",
            DIALOG_HAND_OVER = [[
                player:
                    You heard {agent.himher}, {messenger}. Hand it over.
                messenger:
                    !left
                    But, my duty!
                agent:
                    Stuff your duty back where it came from, Jake!
                player:
                    !left
                * {agent} takes the message and destroys it.
                agent:
                    Pleasure doing business with you, grifter.
                    !exit
                messenger:
                    !angry
                    You've made an enemy today!
            ]],

            OPT_THREATEN = "Scare off the Spree",
            DIALOG_THREATEN = [[
                player:
                    You're far from home, Spree. I suggest you go back.
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                * {agent} eyes the rest of the bar suspiciously.
                agent: 
                    You'll hear from us again.
                    !exit
                * The Spree leave.
                messenger:
                    !right
                    Thank you, grifter.
                    You're going to need to help me deliver this now, if you want to see it done.
                player:
                    !shrug
                    Figures.
                    Lead the way.
            ]],
            DIALOG_THREATEN_FAILURE = [[
                agent:
                    Ha!
                    You don't scare me, grifter.
            ]],

            OPT_DEFEND = "Defend {messenger}",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Put 'em up!
            ]],
            DIALOG_WON_MESSENGER_ALIVE = [[
                messenger:
                    !right
                    That was hairy!
                    You're going to help me actually deliver this, right? There may be more of them.
                player:
                    !sigh
                    Of course.
                    Lead the way.
            ]],
            DIALOG_WON_MESSENGER_DEAD = [[
                right:
                    !exit
                * {messenger} is dead. You take the message and pocket it. 
                * Looks like you'll have to deliver it yourself.
            ]],


        }
        :Fn(function(cxt) 

            cxt:Dialog("DIALOG_INTRO")
            local bandits = CreateCombatParty("BANDIT_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(bandits[1])
            cxt:Dialog("DIALOG_SPREE")
            cxt.quest.param.good_seal = true
            
            cxt:RunLoop(function(cxt) 

                cxt:BasicNegotiation("THREATEN", {helpers = {"messenger"}})
                    :OnSuccess()
                        :Fn(function() 
                            cxt.quest:Complete("pick_up_messenger")
                            cxt.quest:GetCastMember("messenger"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            for k,v in pairs(bandits) do if not v:IsDead() then v:MoveToLimbo() end end 
                        end)
                        :DoneConvo()

                cxt:Opt("OPT_HAND_OVER")
                    :Dialog("DIALOG_HAND_OVER")
                    :ReceiveOpinion("gave_away_message", nil, "messenger")
                    :Fn(function() 
                        cxt.quest.param.no_message = true
                        cxt.quest:Cancel("pick_up_messenger")
                        cxt.quest:Cancel("deliver_the_message")
                        cxt.quest:Activate("return_to_tei")
                    end)
                    
                    :DoneConvo()

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        allies = {"messenger"}
                    }
                        :OnWin()
                            :Fn(function() 
                                cxt.quest:Complete("pick_up_messenger")
                                if cxt.quest:GetCastMember("messenger"):IsDead() then
                                    cxt:Dialog("DIALOG_WON_MESSENGER_DEAD")
                                else
                                    cxt:Dialog("DIALOG_WON_MESSENGER_ALIVE")
                                    cxt.quest:GetCastMember("messenger"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                end

                                StateGraphUtil.AddEndOption(cxt)
                            end)
                            

            end)

        end)


QDEF:AddConvo( "deliver_the_message", "target" )
    :ConfrontState("STATE")
        :Loc{
            DIALOG_INTRO_MESSENGER = [[
                * You find {target} working at the Admiralty station.
                target:
                    !right
                messenger:
                    !left
                    I have a message here for you from Murder Bay.
                    !give
                target:
                    !take
                    Let me see this.
                * {target} reads the message.
                target:
                    {mention_butt?
                        What's this about my butt?!
                    }
                    {not mention_butt?
                        This doesn't sound like Oolo.
                        Where did you get this message?
                    }
                messenger:  
                    Uh...
                player:
                    !left
            ]],

            DIALOG_INTRO_ALONE = [[
                * You find {target} working at the Admiralty station.
                target:
                    !right
                player:
                    !left
                    Hey switch. Got a message for you.
                    !give
                target:
                    !take
                    You're not the usual messenger.
                {not good_seal?
                    This seal's been tampered with.
                }
                {good_seal?
                    The seal looks alright, at least.
                }                
                * {target} reads the message.
                
                target:
                    {mention_butt?
                        What's this about my butt?!
                    }
                    {not mention_butt?
                        This doesn't sound like Oolo.
                        Where did you get this message, grifter?
                    }
            ]],
            SIT_MOD_BAD_SEAL = "The seal was bad",
            SIT_MOD_BAD_MESSENGER = "You're not the usual messenger",
            SIT_MOD_BUTT = "The Admiralty doesn't usually talk about their butts so much",

            OPT_CONVINCE = "Convince {agent} the message is real",
            DIALOG_CONVINCE = [[
                player:
                    Hey, don't blame the messenger!
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    If you don't like the content of the letter, take it up with the letter-writer!
                agent:
                    Hmph. I'll do just that.
                    I'll be writing to Lieutenant Ollowano immediately, and I will strongly suggest she not use you as her messenger. 
                !exit
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    I just deliver the message, man. 
                    You should be grateful: Murder Bay's a long ways away.
                agent:
                    Yes, I suppose so. We simply haven't the troops to spare right now as messengers.
                    Thank you for your service, grifter. 
                    You're dismissed.
                    !exit
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog(cxt.quest:GetCastMember("messenger"):IsInPlayerParty() and "DIALOG_INTRO_MESSENGER" or "DIALOG_INTRO_ALONE")
            local sit_mods = {}
            
            if not cxt.quest.param.good_seal then
                table.insert(sit_mods, { value = 10, text = cxt:GetLocString("SIT_MOD_BAD_SEAL") })
            end

            if not cxt.quest:GetCastMember("messenger"):IsInPlayerParty() then
                table.insert(sit_mods, { value = 10, text = cxt:GetLocString("SIT_MOD_BAD_MESSENGER") })
            end
            
            if cxt.quest.param.mention_butt then
                table.insert(sit_mods, { value = 10, text = cxt:GetLocString("SIT_MOD_BUTT") })
            end
            local helpers = {}
            if cxt.quest:GetCastMember("messenger"):IsInPlayerParty() then
                table.insert(helpers, cxt.quest:GetCastMember("messenger"))
            end


            cxt:BasicNegotiation("CONVINCE", {situation_modifiers = sit_mods, helpers = helpers})
                :OnSuccess()
                    :CompleteQuest("deliver_the_message")
                    :Fn(function() cxt.quest.param.message_delivered = true end)
                    :Travel()
                
                :OnFailure()
                    :CompleteQuest("deliver_the_message")
                    :Fn(function() cxt.quest.param.message_rejected = true end)
                    :Travel()
        end)




QDEF:AddConvo( "return_to_tei", "tei" )
    :Loc{
        OPT_HAND_IN = "Give {tei} a report",
        DIALOG_HAND_IN = [[
            {message_delivered?
                player:
                    The message has been delivered.
                agent:
                    Wow, they accepted that?
                    Great.
            }
            {message_rejected?
                player:
                    The Admiralty didn't like the altered message.
                agent:
                    Not surprising.
                    At least we've delayed their plans.
            }
            {no_message?
                player:
                    I got the message, but the Spree intercepted it.
                agent:
                    Well, that's a wrinkle.
                    At least it's not in the hands of the Admiralty.
                    Partial credit, Smith.
            }
        ]]
    }
    :Hub(function(cxt)
            cxt:Opt("OPT_HAND_IN")
                :Dialog("DIALOG_HAND_IN")
                :Fn(function() 
                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:Complete()
                end)
                :DoneConvo()
    end)


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}
