local fun = require "util/fun"
local daily = require "daily_util"

local RESOLVE_REQ = { 40, 50, 65, 70, 75 }

local character_options =
{
    "SAL", "ROOK", "SMITH"
}

local boss_options =
{
    {"DRONE_GOON", "HESH_BOSS"},
    {"JAKES_ASSASSIN2", "AUTOMECH_BOSS"},
    {"SHROOG"},
    {"RISE_SECOND", "MURDER_BAY_BANDIT_CONTACT"},
    {"KASHIO", "BOG_BURR_BOSS"},
}

local function DoCardDraft(cxt, difficulty, subtitle_text)
    local function OnDone()
        cxt.encounter:ResumeEncounter()
    end
    local draft_popup = Screen.DraftPackPopup()
    draft_popup:DraftPacks( cxt.player, Negotiation.Card, difficulty, OnDone, subtitle_text, cxt:GetRNG( "CARDS" ))
    TheGame:FE():InsertScreen( draft_popup )
    cxt.enc:YieldEncounter()
end

local function DoGraftDraft( cxt, difficulty, special )

    local rng = cxt:GetRNG( "GRAFTS" )
    if rng then
        push_random( function( n, m )
            return rng:Random( n, m )
        end )
    end
    local grafts
    if special then
        grafts = cxt.quest.param.special_grafts:Generate(3)
    else
        grafts = RewardUtil.GetPooledGrafts( difficulty, 3, cxt.quest.param.graft_pool )
    end

    if rng then
        pop_random()
    end

    local popup = Screen.PickGraftScreen(grafts, false, function(...) cxt.enc:ResumeEncounter(...) end)
    popup:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
    TheGame:FE():InsertScreen( popup )

    local chosen_graft = cxt.enc:YieldEncounter()
end

local follower_options =
{
    "BARTENDER", "SPARK_BARON_GOON", "BANDIT_GOON", "PRIEST"
}

local boon_options = 
{
    remove_card = function(cxt)
            cxt:Opt( "OPT_REMOVE_NEGOTIATION_CARD" )
                :PreIcon( global_images.removenegotiation )
                :Fn( function( cxt )
                    cxt:Wait()
                    AgentUtil.RemoveNegotiationCard( cxt.player, function( card )
                        cxt.enc:ResumeEncounter(card)
                    end)
                    local card = cxt.enc:YieldEncounter()
                    if card then
                        cxt:GoTo("STATE_ENCOUNTER")
                    end
                end)
    end,
    graft_slot = function( cxt )
        local txt = "OFFER_NEGOTIATION_SLOT"
        cxt:Opt( txt )
            :PreIcon( global_images.graft )
            :Quip( cxt:GetAgent(), "chum_bonus", "graft" )
            :Fn( function( cxt )
                cxt.player.graft_owner:IncreaseMaxGrafts( GRAFT_TYPE.NEGOTIATION, 1 )
                cxt:GoTo("STATE_ENCOUNTER")
            end )
    end,
    draft_card = function(cxt)
            cxt:Opt( "OPT_NEGOTIATION_DRAFT" )
                :PreIcon( global_images.buynegotiation )
                :Fn( function( cxt )
                    cxt:Wait()

                    DoCardDraft(cxt, 3)
                    cxt:GoTo("STATE_ENCOUNTER")
                end )
    end,
    upgrade_card = function(cxt)
            cxt:Opt( "OPT_UPGRADE_NEGOTIATION_CARD" )
                :PreIcon( global_images.upgradenegotiation )
                :Fn( function( cxt )
                    cxt:Wait()
                    AgentUtil.UpgradeNegotiationCard(function( card )
                        cxt.enc:ResumeEncounter( card )
                    end)
                    
                    local card = cxt.enc:YieldEncounter()
                    if card then
                        cxt:GoTo("STATE_ENCOUNTER")
                    end
                end )
    end,
    gain_follower = function(cxt)
            local agent = cxt.quest.param.follower_options and cxt.quest.param.follower_options[1]
            if not agent then
                return
            end

            cxt:Opt( "OPT_GAIN_FOLLOWER", agent )
                :PreIcon( global_images.hire )
                :Fn( function( cxt )
                    table.remove( cxt.quest.param.follower_options, 1)
                    cxt.enc:SetPrimaryCast(agent)
                    cxt.quest.param.help_agent = agent
                    cxt:Dialog("ACCEPTED_FOLLOWER")
                    cxt:GoTo("STATE_ENCOUNTER")
                end )
    end,
}

local QDEF = QuestDef.Define
{
    title = "Boss Rush Negotiation",
    icon = engine.asset.Texture("icons/quests/daily_escape_cave.tex"),
    desc = [[Defeat a series of bosses in negotiations.]],
    qtype = QTYPE.DAILY,
    on_complete = function( quest )
        quest:AddScore("WON_DAILY")
        TheGame:Win()
    end,
    on_fail = function( quest )
        TheGame:Lose()
    end,
    ChooseCharacter = function( self, rng )
        return character_options[rng:Random(#character_options)]
    end,
    
    GenerateMutatorsAndFeats = function(self, rng, character_id)
        
        local arg = {
            negotiation_draft = {
                "negotiation_bundle",
                "picky_draft_negotiation",
                "charisma",
            },
            always_take = {
                "brilliance",
            },
            mutators = {
                "roll_with_it",
                "mutator",
                "mental_overload",
                "starved",
            },
            feats = {
                {"brain_rack", "ice_cold", "absolute_focus"},
                {"blow_out", "to_the_point", "take_your_shot"},
                {"the_backup","ice_cold", "take_your_shot"},
                {"brain_rack", "cheapskate", "take_your_shot"},
                {"endless_ideas", "tall_tale", "filibuster"},
                {"blow_out", "record_breaker", "quick_wit"},
            },
            min_mutators = 1,
            max_mutators = 3,
        }

        return daily.GenerateMutatorsAndFeats(rng, character_id, arg)
    end,


    plax = "EXT_Bog_Forest_01",


    on_init = function(quest)
        TheGame:GetGameState():SetDifficulty( 2 ) -- For better initial draft.
        TheGame:GetGameState():GetCaravan():MoveToLocation(quest:GetCastMember("the_forest"))
    end,

    GenerateParams = function( self, rng )
        local param = {}
        
        param.available_encounters = {}
        for i,options in ipairs(boss_options) do
            table.insert(param.available_encounters, options[rng:Random(#options)])
        end

        param.available_bonuses = {}
        for i=1, (#param.available_encounters - 1) do
            local temp_options = {}
            for k,v in pairs(boon_options) do
                table.insert(temp_options, k)
            end
            table.seeded_shuffle(temp_options, rng)
            for i=1,3 do
                table.insert(param.available_bonuses, table.remove(temp_options, 1))
            end
        end

        param.all_encounters = shallowcopy(param.available_encounters)
        param.follower_options = table.shuffle( shallowcopy(follower_options) )

        param.step = 0
        return param
    end,


    GenerateProgressionData = function(qdef, param)
        local current_day = {}
        local next_step = param.step
        
        if param.all_encounters then
            for step,def in ipairs(param.all_encounters) do
                table.insert(current_day, {id="BOSS", boss = def})
                if next_step == step then
                    current_day[#current_day].right_now = true
                elseif next_step > step then
                    current_day[#current_day].success = true
                end
            end
        end
        return {current_day}
    end,


    get_progression = function(quest)
        return quest:GetQuestDef().GenerateProgressionData(quest:GetQuestDef(), quest.param)
    end,



}

:AddScoreDefs{
    
    SKIPPED_BONUS = 
    {
        name = "Skipped Bonus",
        value = 30,
    },
    SKIPPED_RESOLVE = 
    {
        name = "Skipped Meditation",
        value = 40,
    },
    WON_DAILY = {
        name = "Defeated All Bosses",
        value = 500,
    },
}

:AddObjective{
    id = "start",
    title = "Defeat the Bosses in Negotiation",
    on_activate = function( quest)
        quest.param.location = quest:SpawnTempLocation("GROUT_BOG_FOREST", "the_forest" )
    end,
    state = QSTATUS.ACTIVE,

    events = {
        resolve_negotiation = function( quest, minigame, repercussions )
            if repercussions and (repercussions.result == minigame.RESULT.LOSE or #quest.param.available_encounters == 0) then
                -- Game Over, so don't show any loot, upgrades, etc.
                repercussions.no_loot_screen = true
            end
        end,
    }

}

:AddCast{
    cast_id = "the_forest",
    when = QWHEN.MANUAL,
}

:AddCast{
    cast_id = "pet",
    when = QWHEN.MANUAL,
}

local function FindNewOptions(cxt)
        if #cxt.quest.param.available_encounters > 0 then
            cxt.quest.param.step = cxt.quest.param.step + 1
            cxt.quest.param.next_boss = table.remove(cxt.quest.param.available_encounters, 1)
            cxt.quest:NotifyChanged()
            return true
        else
            return false
        end
end

local function InitializeRun( cxt )
    local function Filter( graft_def )
        return graft_def.type == GRAFT_TYPE.NEGOTIATION
    end
    cxt.quest.param.graft_pool = GraftCollection.GraftPool( cxt.player, Filter, cxt:GetRNG( "GRAFTS" ) )
    cxt.quest.param.special_grafts = GraftCollection(function(graft_def) return graft_def.type == GRAFT_TYPE.NEGOTIATION end):NotInstalled(cxt.player):NotRestricted(cxt.player):Rarity(CARD_RARITY.COSMIC)

    for i, id in ipairs(cxt.quest.param.follower_options) do
        cxt.quest.param.follower_options[i] = AgentUtil.GetFreeAgent(id)
    end

    cxt.encounter:DoLocationTransition( cxt.quest.param.location )
    cxt.quest:SetRank(2)
    cxt.quest:NotifyChanged()
end

local function OfferBonuses(cxt)
    local bonuses = {}
    for i=1, 2 do
        table.insert( bonuses, table.remove(cxt.quest.param.available_bonuses))
    end

    cxt:RunLoopingFn(function()
        for i, id in ipairs( bonuses ) do
            boon_options[id](cxt)
        end
        cxt:Opt("OPT_SKIP")
            :AddScore("SKIPPED_BONUS")
            :GoTo("STATE_ENCOUNTER")
    end)
end


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
            ]],
            OPT_GET_GRAFT = "Choose a graft",
            OPT_DO_DRAFT = "Draft some cards",
        }
        :Fn(function(cxt) 

            if StateGraphUtil.RestoreDaily( cxt ) then
                return
            end

            InitializeRun(cxt)

            cxt:Dialog("DIALOG_INTRO")

            DoGraftDraft(cxt, 1, true)

            cxt:GoTo("STATE_ENCOUNTER") 
        end)

    :State("STATE_ENCOUNTER")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !right
                    !angryHostile
                player:
                    !left
                {first_time?
                    * A challenger arrives!
                }
                {not first_time?
                    * The next challenger arrives!
                }
                
            ]],
            OPT_NEGOTIATE = "Debate!",
            OPT_CONTINUE = "Continue",
            DIALOG_CLEAR = [[
                player:
                    !exit
                agent:
                    !exit
            ]],
            DIALOG_LOST = [[
                agent:
                    Enough of this.
                * You see {agent} signal to someone behind you but you don't have time to turn before you're knocked out cold.
            ]],
        }
        :Fn(function(cxt) 
                FindNewOptions(cxt)
                local boss = cxt.quest:CreateSkinnedAgent( cxt.quest.param.next_boss )
                cxt.enc:SetPrimaryCast(boss)
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_NEGOTIATE")
                    :Negotiation{
                        helpers = {cxt.quest.param.help_agent},
                        difficulty = math.min( cxt.quest.param.step, 5),
                        enemy_resolve_required = RESOLVE_REQ[math.min(cxt.quest.param.step, #RESOLVE_REQ)],
                        flags = NEGOTIATION_FLAGS.WORDSMITH | NEGOTIATION_FLAGS.NO_IMPATIENCE,
                    }
                    :OnSuccess()
                        :Fn(function()
                            cxt:Opt("OPT_CONTINUE")
                                :Fn(function(cxt)
                                    cxt:Dialog("DIALOG_CLEAR")
                                    cxt.quest.param.help_agent = nil
                                    if #cxt.quest.param.available_encounters == 0 then
                                        cxt.quest:Complete()
                                    else
                                        local diff_table = {3, 4, 4, 5}
                                        local diff = diff_table[math.min(cxt.quest.param.step, #diff_table)]
                                        DoGraftDraft(cxt, diff)
                                        DoCardDraft(cxt, diff)
                                        cxt:GoTo("STATE_HEALING")
                                    end
                                end)
                        end)
                    :OnFailure()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_LOST")
                            cxt.quest:Fail()
                            StateGraphUtil.AddEndOption(cxt)
                        end)
            end)

    :State("STATE_HEALING")
        :Loc{
            DIALOG_INTRO = [[
                * You have some time to clear your mind before the next debate.
            ]],
            OPT_SKIP = "Skip",
            OPT_REOVER = "Meditate",
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.enc:SetPrimaryCast(nil)

            local pet = cxt.enc:GetCastAgent("pet")

            cxt:Opt("OPT_REOVER")
                :PreIcon( global_images.restoreresolve )
                :DeltaResolve(15)
                :Fn(function()
                    cxt:GoTo("STATE_BONUS")
                end)

            cxt:Opt("OPT_SKIP")
                :AddScore("SKIPPED_RESOLVE")
                :GoTo("STATE_BONUS")
        end)

    :State("STATE_BONUS")
        :Loc{
            DIALOG_INTRO = [[
                * Choose a bonus reward before starting the next fight.
            ]],
            OPT_SKIP = "Skip",
            OPT_CONTINUE_TO_FIGHT = "Continue to the next fight",
            OPT_REMOVE_NEGOTIATION_CARD = "Remove a negotiation card",
            OPT_NEGOTIATION_DRAFT = "Draft a pack of negotiation cards.",
            OPT_GAIN_FOLLOWER = "Gain {1#agent} to help you in your next negotiation.",
            ACCEPTED_FOLLOWER = [[
                player:
                    !left
                agent:
                    !right
                    I've got your back, {player}!
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.enc:SetPrimaryCast(nil)
            cxt:RunFn(OfferBonuses)
        end)