local fun = require "util/fun"
local daily = require "daily_util"

local encounter_options =
{
    {id = "STATE_KRILL", dial = "KRILL_HINT"},
    {id = "STATE_BANDITS", dial = "BANDITS_HINT"},
    {id = "STATE_YOTES", dial = "YOTE_HINT"},
    {id = "STATE_FLEADS", dial = "FLEAD_HINT"},
    {id = "STATE_ADMIRALTY", dial = "ADMIRALTY_HINT"},
    {id = "STATE_ERCHIN", dial = "ERCHIN_HINT"},
    {id = "STATE_RISE", dial = "RISE_HINT"},
    {id = "STATE_JAKES", dial = "JAKES_HINT"}
}

local boon_options = 
{
    {id = "STATE_PLANTS", dial = "PLANTS_HINT"},
    {id = "STATE_TRAPPED", dial = "TRAPPED_HINT"},
    {id = "STATE_STASH", dial = "STASH_HINT"},
    {id = "STATE_FREE_GRAFT", dial = "GRAFT_HINT"}
}



local character_options =
{
    "ROOK", "SAL", "SMITH"
}

local search_items = {"lumin_grenade", "spark_grenade", "gaff", "healing_vapors", "salve", "tincture", "token_of_hesh", "nano_lattice", "replicator"}

local BANDIT_PATROL =
{
    -- Difficulty 1
    {
        {"BANDIT_RAIDER"},
        {"BANDIT_GOON2"},
        {"BANDIT_GOON", "FLEAD"}
    },
    -- Difficulty 2
    {
        {"BANDIT_GOON", "FLEAD", "FLEAD"},
        {"BANDIT_RAIDER", "FLEAD"},
    },
}

local ADMIRALTY_PATROL = 
{
    -- Difficulty 1
    {
        {"ADMIRALTY_GUARD"},
        {"ADMIRALTY_GOON", "ADMIRALTY_CLERK"},
    },
    -- Difficulty 2
    {
        {"ADMIRALTY_GOON", "VROC", "VROC"},
        {"ADMIRALTY_PATROL_LEADER", "ADMIRALTY_CLERK"},
    },
}

local MONSTER_PARTY =
{
    -- Difficulty 1
    {
        {"CRAYOTE", "CRAYOTE"},
        {"ERCHIN", "ERCHIN"},
        {"CRAYOTE", "FLEAD"},
    },
    -- Difficulty 2
    {
        {"CRAYOTE_SPITTER", "CRAYOTE", "CRAYOTE"},
        {"LUMINTHIAN", "LUMICYTE"},
        {"GRAWKIT"},
    },
}

local RISE_PATROL =
{
    -- Difficulty 1
    {
        {"RISE_REBEL", "RISE_REBEL"},
        {"RISE_RADICAL"},
    },
    -- Difficulty 2
    {
        {"RISE_PAMPHLETEER", "RISE_AUTOMECH"},
        {"RISE_RADICAL", "RISE_REBEL"},
    },
}


local JAKES_PATROL =
{
    -- Difficulty 1
    {
        {"JAKES_RUNNER", "JAKES_RUNNER"},
        {"JAKES_SMUGGLER", "VROC"},
    },
    -- Difficulty 2
    {
        {"JAKES_LIFTER", "JAKES_SMUGGLER"},
        {"JAKES_SMUGGLER", "JAKES_SMUGGLER", "VROC"},
    },
}

local boss_options =
{
    "MERCENARY_BOSS","HESH_BOSS","RENTORIAN_BOSS", "JAKES_ASSASSIN", "JAKES_ASSASSIN2"
}





local QDEF = QuestDef.Define
{
    title = "Escape the Caves",
    icon = engine.asset.Texture("icons/quests/daily_escape_cave.tex"),
    desc = [[Fight your way out of a sprawling cave system.]],
    qtype = QTYPE.DAILY,
    on_complete = function( quest )
        
        quest:AddScore("WON_DAILY")
        TheGame:Win()
    end,
    ChooseCharacter = function( self, rng )
        return character_options[rng:Random(#character_options)]
    end,
    

    GenerateMutatorsAndFeats = function(self, rng, character_id)
        
        local arg = {
            negotiation_draft = {
                    "picky_draft_negotiation",
                    "havarian_mulligan_negotiation",    
                    "negotiation_bundle",
            },
            battle_draft = {
                "havarian_mulligan_battle",
                "picky_draft_battle",
                "battle_bundle",
            },
            always_take = {
                "brilliance",
            },
            mutators = {
                "parasitology",
                "roll_with_it",
                "land_of_plenty",
                "mental_overload",
                "pet_owner",
                "contaminated_equipment",
                "score_card_mutator",
                "keepsake_mutator",
                "mutator",
            },
            feats = {
                {"vicious_feat", "no_sweat", "not_a_scratch"},
                {"smashing", "no_sweat", "take_your_shot"},
                {"smashing", "thorough_search", "clean_fight"},
                {"vicious_feat", "play_dead", "quick_work"},
                {"smashing", "tank_up", "take_your_shot"},
            },
            min_mutators = 2,
            max_mutators = 3,
        }

        return daily.GenerateMutatorsAndFeats(rng, character_id, arg)
    end,
    
    GenerateParams = function(self, rng)
        local param = {}
        param.step = 0

        param.available_encounters = {}
        local temp_encounters = ObtainWorkTable()
        table.arrayadd(temp_encounters, encounter_options)
        table.seeded_shuffle(temp_encounters, rng)
        for i=1,8 do
            table.insert(param.available_encounters, table.remove(temp_encounters))
        end
        ReleaseWorkTable(temp_encounters)

        param.available_boons = {}
        local temp_boons = ObtainWorkTable()
        table.arrayadd(temp_boons, boon_options)
        table.seeded_shuffle(temp_boons, rng)
        for i=1,2 do
            table.insert(param.available_boons, table.remove(temp_boons))
        end
        ReleaseWorkTable(temp_boons)
        
        param.boss_id = boss_options[rng:Random(#boss_options)]
        return param
    end,

    GenerateProgressionData = function(qdef, param)

        local current_day = {}

        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_boon.tex"), tt = LOC( qdef:GetLocKey("TT_BOON"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_boss.tex"), tt = LOC( qdef:GetLocKey("TT_BOSS"))} )

        local current_step = param.step or 0
        for k,v in ipairs(current_day) do
            if k == current_step then
                v.right_now = true
            elseif k < current_step then
                v.success = true
            end
        end

        return {current_day}
    end,

    get_progression = function(quest)
        return quest:GetQuestDef().GenerateProgressionData(quest:GetQuestDef(), quest.param)
    end,


    plax = "INT_CaveDeep_MurderBay_1",

    on_init = function(quest)
        TheGame:GetGameState():GetCaravan():MoveToLocation(quest:GetCastMember("the_cave"))
    end,



}

:Loc{
    TT_ENCOUNTER = "This is an encounter. You will have to play your cards right to progress!",
    TT_BOON = "You have the feeling that something good will happen to you here!",
    TT_BOSS = "A powerful enemy is waiting for you."
}

:AddLocationDefs{
    CAVE_ENTRANCE = {
        name = "Partok Caves",
        icon = engine.asset.Texture("UI/location_cave.tex"),
        plax = "EXT_Cave_Murderbay_1",
        indoors = false,
        map_tags = {"cave"},
    },

    OUTER_CAVES = 
    {
        name = "Cave interior",
        plax = "INT_Cave_MurderBay_1",
        indoors = true,
        map_tags = {"cave"},
    },
    DEEP_CAVES = 
    {
        name = "Deep Cave interior",
        plax = "INT_CaveDeep_MurderBay_1",
        indoors = true,
        map_tags = {"cave"},
    }
}

:AddCast{
    cast_id = "the_cave",
    when = QWHEN.MANUAL,
}

:AddScoreDefs{
    WON_DAILY = {
        name = "Escaped the Cave",
        value = 500,
    },
    SEARCH_CRATES = {
        name = "Searched Crates",
        value = 50,
    },

    SKIPPED_HEALING = 
    {
        name = "Skipped Healing",
        value = 40,
    },

    SKIPPED_BONUS = 
    {
        name = "Skipped Bonus",
        value = 30,
    },

    DECLINED_PET =
    {
        name = "Decline the pet",
        value = 50,
    },
}


:AddObjective{
    id = "start",
    title = "Escape the Caves",
    on_activate = function( quest)
        quest:SpawnTempLocation("CAVE_ENTRANCE", "the_cave" )
        quest.param.cave_loc = quest:SpawnTempLocation("OUTER_CAVES")
        quest.param.cave_loc:SetFlags( LOCFLAGS.HIDDEN )
        quest.param.cave_loc_deep = quest:SpawnTempLocation("DEEP_CAVES")
        quest.param.cave_loc_deep:SetFlags( LOCFLAGS.HIDDEN )
    end,
    state = QSTATUS.ACTIVE,
}

local function EnterCave(cxt) 
    if cxt.location ~= cxt.quest.param.cave_loc then
        cxt.encounter:DoLocationTransition( cxt.quest.param.cave_loc )
        cxt.encounter:PresentAgent(nil, SIDE.LEFT)
        cxt.encounter:PresentAgent(nil, SIDE.RIGHT)
    end
end

local function EnterCaveDeep(cxt) 
    if cxt.location ~= cxt.quest.param.cave_loc_deep then
        cxt.encounter:DoLocationTransition( cxt.quest.param.cave_loc_deep )
        cxt.encounter:PresentAgent(nil, SIDE.LEFT)
        cxt.encounter:PresentAgent(nil, SIDE.RIGHT)
    end
end

local function FindNewOptions(cxt)
    if cxt.quest.param.step >= 3 and cxt.quest:GetRank() < 2 then
        cxt.quest:SetRank(2)
    end
    if cxt.quest.param.step % 3 == 0 and #cxt.quest.param.available_boons >= 2 then
        cxt.enc.scratch.right_option = table.remove(cxt.quest.param.available_boons, #cxt.quest.param.available_boons)
        cxt.enc.scratch.left_option = table.remove(cxt.quest.param.available_boons, #cxt.quest.param.available_boons)
        return true
    elseif #cxt.quest.param.available_encounters >= 2 then
        cxt.enc.scratch.right_option = table.remove(cxt.quest.param.available_encounters, #cxt.quest.param.available_encounters)
        cxt.enc.scratch.left_option = table.remove(cxt.quest.param.available_encounters, #cxt.quest.param.available_encounters)
        return true
    else
        return false
    end
end

local function DoGraftDraft( cxt, difficulty, special )

    local rng = cxt:GetRNG( "GRAFTS" )
    if rng then
        push_random( function( n, m )
            return rng:Random( n, m )
        end )
    end
    local grafts
    if special then
        grafts = cxt.quest.param.special_grafts:Generate(3)
    else
        grafts = RewardUtil.GetPooledGrafts( difficulty, 3, cxt.quest.param.graft_pool )
    end

    if rng then
        pop_random()
    end

    local popup = Screen.PickGraftScreen(grafts, false, function(...) cxt.enc:ResumeEncounter(...) end)
    popup:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
    TheGame:FE():InsertScreen( popup )

    local chosen_graft = cxt.enc:YieldEncounter()
end

local function InitializeRun( cxt )
    local function Filter( graft_def )
        return graft_def.type == GRAFT_TYPE.COMBAT
    end
    cxt.quest.param.graft_pool = GraftCollection.GraftPool( cxt.player, Filter, cxt:GetRNG( "GRAFTS" ) )
    cxt.quest.param.special_grafts = GraftCollection(function(graft_def) return graft_def.type == GRAFT_TYPE.COMBAT end):NotInstalled(cxt.player):NotRestricted(cxt.player):Rarity(CARD_RARITY.COSMIC)
    cxt.quest:SetRank(1)
    EnterCaveDeep(cxt)
end


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You wake up deep inside a sprawling cave system with no sense of how you got here or how to get out.
                * You try to recall how you ended up here.
            ]],
            OPT_GRAFT = "Choose a graft",
        }
        :Fn(function(cxt) 

            if StateGraphUtil.RestoreDaily( cxt ) then
                return
            end

            InitializeRun(cxt)

            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_GRAFT")
                :Fn(function(cxt)
                    DoGraftDraft(cxt, 1, true)
                    cxt:GoTo("STATE_CHOICE")
                end)           
        end)
    
    :State("STATE_CHOICE")
        :Loc{
            DIALOG_CLEAR = [[
                player:
                    !exit
                agent:
                    !exit
            ]],
            DIALOG_INTRO_FIRST = [[
                * After getting your bearings, you explore the area around you and notice two tunnels leading away.
            ]],
            DIALOG_INTRO = [[
                * You walk along the tunnel for a while, keeping an eye out for any danger.
                * The tunnel splits in two ahead of you. You take a moment to consider your options.
            ]],
            DIALOG_INTRO_BOSS = [[
                * You walk along the tunnel for a while, keeping an eye out for any danger.
            ]],
            DIAL_RIGHT = [[
                * Right:
            ]],
            DIAL_LEFT = [[
                * Left:
            ]],
            OPT_CHOOSE_RIGHT = "Head down the right path",
            OPT_CHOOSE_LEFT = "Head down the left path",
            KRILL_HINT = [[
                * {1*Right |Left }: There are deep claw marks in the wall but you hear no sounds and see no movement. Maybe whatever was there is gone?
            ]],
            BANDITS_HINT = [[
                * {1*Right |Left }: The tunnel smells of smoke and you can see a faint red glow. Someone must be there.
            ]],
            YOTE_HINT = [[
                * {1*Right |Left }: The cave is quiet and there are no signs of life. There is, however, a fairly distinct musk in the air.
            ]],
            PLANTS_HINT = [[
                * {1*Right |Left }: Spilling around a bend in the cave, you can see traces of what appears to be lumin vapors.
            ]],
            TRAPPED_HINT = [[
                * {1*Right |Left }: Parts of the cave seem to have collapsed and something that could be a muffled scream echoes down the passage.
            ]],
            STASH_HINT = [[
                * {1*Right |Left }: There are no signs of danger, in fact this tunnel seems completely quiet.
            ]],
            GRAFT_HINT = [[
                * {1*Right |Left }: You can see a single set of footsteps heading down this tunnel.
            ]],
            FLEAD_HINT = [[
                * {1*Right |Left }: An ever so slight breeze of hot air seems to be coming from that direction.
            ]],
            ADMIRALTY_HINT = [[
                * {1*Right |Left }: Off in the distance you can hear what sounds like people talking.
            ]],
            ERCHIN_HINT = [[
                * {1*Right |Left }: You can hear a lot of commotion coming from further down the tunnel. It's hard to tell what's making the sounds.
            ]],
            RISE_HINT = [[
                * {1*Right |Left }: A thin veil of leaves and twigs covers the entrance to a tunnel. It's clearly an attempt to disguise the tunnel, albeit a bad one.
            ]],
            JAKES_HINT = [[
                * {1*Right |Left }: Along the floor of the tunnel, you can see lines in the dirt as if something has been dragged through here.
            ]],
        }
        :Fn(function(cxt) 

            cxt:Dialog("DIALOG_CLEAR")
            if cxt.quest.param.step == 0 then
                cxt:Dialog("DIALOG_INTRO_FIRST")
            elseif cxt.quest.param.step <= 4 then
                cxt:Dialog("DIALOG_INTRO")
            else
                cxt:Dialog("DIALOG_INTRO_BOSS")
            end
            cxt.quest.param.step = cxt.quest.param.step + 1
            cxt.quest:NotifyChanged()
            if cxt.quest.param.step <= 5 and FindNewOptions(cxt) then
                cxt:Dialog(cxt.enc.scratch.right_option.dial, 1)
                cxt:Dialog(cxt.enc.scratch.left_option.dial, 2)
                cxt:Opt("OPT_CHOOSE_RIGHT")
                    :GoTo(cxt.enc.scratch.right_option.id)

                cxt:Opt("OPT_CHOOSE_LEFT")
                    :GoTo(cxt.enc.scratch.left_option.id)

            else
                cxt:GoTo("STATE_BOSS")
            end
        end)

    :State("STATE_KRILL")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * As you walk along the path, you hear a snarl and spot something moving out of the corner of your eye.
                * You whip around and find yourself face-to-face with a Krill.
                agent:
                    !right
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_ON_WIN = [[
                * You're safe. For now. You decide to keep following the tunnel, there's no point turning back.
            ]],
        }
        :Fn(function(cxt) 
            local krill = Agent( "LUMINTHIAN" )
            cxt.enc:SetPrimaryCast(krill)
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                    on_win = function(cxt) 
                        cxt:Dialog("DIALOG_ON_WIN")
                        cxt:GoTo("STATE_CHOICE")
                    end,
                }
        end)
    
    :State("STATE_BANDITS")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * The smell of smoke and freshly cooked food grows stronger as you make your way through the tunnel.
                agent:
                    !right
                    Hey! Who are you?
                player:
                    I'm just trying to find my way out of here, I didn't mean to bother you.
                agent:
                    I can't let you leave, you've seen too much!
            ]],
            OPT_NEGOTIATE = "Convince {agent} to let you go",
            DIAL_NEGOTIATE = [[
                player:
                    Wait, wait, wait! No need to do anything rash.
                agent:
                    Oh yeah? And why's that?
            ]],
            DIAL_NEGOTIATION_WIN = [[
                agent:
                    Fine, fine. I guess you haven't seen anything incriminating.
                player:
                    Precisely! I'll leave and we can both just pretend I was never here.
                agent:
                    You better keep quiet, now scram!
                * You move along before {agent} changes {agent.hisher} mind.
                player:
                    !exit
                agent:
                    !exit
            ]],
            DIAL_NEGOTIATION_FAIL = [[
                agent:
                    No amount of smooth talking is going to get you out of this one!
                    !fight
                player:
                    !fight
            ]],
            OPT_DEFEND = "Defend yourself",
            OPT_ATTACK = "Attack {agent}",
            DIAL_ATTACK = [[
                player:
                    Oh yeah? Let's see what you've got then!
                agent:
                    !fight
                player:
                    !fight
            ]],
            DIAL_SURVIVORS = [[
                agent:
                    !exit
                player:
                    !left
                * As soon as you lower your weapons, the remaining bandits run in the opposite direction.
                * Satisfied to let them leave, you stay behind and search through their makeshift camp. There's still some food on the fire, enough for a small meal.
            ]],
            DIAL_ALL_DEAD = [[
                agent:
                    !exit
                player:
                    !left
                * With all the bandits dead, you feel safe to stick around and search their makeshift camp. There's still some food on the fire, enough for a small meal.
            ]],
            OPT_EAT = "Eat the food they left",
            DIAL_EAT = [[
                * You sit down for a bit and eat the remainders of their food.
            ]],
            DIAL_EAT2 = [[
                * Once you're done, you put out the last embers and continue through the cave.
            ]],
            OPT_LEAVE = "Leave without eating",
            DIAL_LEAVE_NO_EAT = [[
                * You decide not to take the risk of eating strange food and continue through the cave.
            ]],
        }
        :Fn(function(cxt) 
            local bandit_table = BANDIT_PATROL[math.min(cxt.quest:GetRank(), #BANDIT_PATROL)]
            local guards = CreateCombatParty(bandit_table[cxt:GetRNG():Random(#bandit_table)], cxt.quest:GetRank(), cxt.location)
            cxt.enc:SetPrimaryCast(guards[1])
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_FIGHT")
                :Dialog("DIAL_ATTACK")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                    on_win = function(cxt) 
                        local survivors = {}
                        for k,v in ipairs(guards) do
                            if not v:IsDead() then
                                if v:IsSentient() then
                                    table.insert(survivors, v)
                                end
                                v:MoveToLimbo()
                            end
                        end
                        if #survivors > 0 then
                            cxt:Dialog("DIAL_SURVIVORS")
                        else
                            cxt:Dialog("DIAL_ALL_DEAD")
                        end
                        cxt:Opt("OPT_EAT")
                            :Dialog("DIAL_EAT")
                            :DeltaHealth(15)
                            :Dialog("DIAL_EAT2")
                            :GoTo("STATE_CHOICE")

                        cxt:Opt("OPT_LEAVE")
                            :Dialog("DIAL_LEAVE_NO_EAT")
                            :GoTo("STATE_CHOICE")
                    end,

            cxt:Opt("OPT_NEGOTIATE")
                :Dialog("DIAL_NEGOTIATE")
                :Negotiation{
                    on_success = function() 
                        cxt:Dialog("DIAL_NEGOTIATION_WIN")
                        cxt:GoTo("STATE_CHOICE")
                    end,
                    on_fail = function() 
                        cxt:Dialog("DIAL_NEGOTIATION_FAIL")
                        cxt:Opt("OPT_DEFEND")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                                on_win = function(cxt) 
                                    local survivors = {}
                                    for k,v in ipairs(guards) do
                                        if not v:IsDead() then
                                            if v:IsSentient() then
                                                table.insert(survivors, v)
                                            end
                                            v:MoveToLimbo()
                                        end
                                    end
                                    if #survivors > 0 then
                                        cxt:Dialog("DIAL_SURVIVORS")
                                    else
                                        cxt:Dialog("DIAL_ALL_DEAD")
                                    end
                                    cxt:Opt("OPT_EAT")
                                        :Dialog("DIAL_EAT")
                                        :DeltaHealth(15)
                                        :GoTo("STATE_CHOICE")

                                    cxt:Opt("OPT_LEAVE")
                                        :Dialog("DIAL_LEAVE_NO_EAT")
                                        :GoTo("STATE_CHOICE")
                                end,
                            }
                    end,
                }
            }
        end)

    :State("STATE_YOTES")
        :Loc{
            DIALOG_INTRO = [[
                * You hear strange sounds coming from the darkness and ready your weapons.
                agent:
                    !right
                    $angryHostile
                    <i>Gurrrraauggh</i>.
                player:
                    !left
                    !sigh
                    $scaredFearful
                    Aw, crap.
            ]],
            OPT_FIGHT = "Defend yourself",
            DIALOG_ON_WIN = [[
                * With that dealt with, you continue through the caves.
            ]]
        }
        :Fn(function(cxt) 
                local monster_table = MONSTER_PARTY[math.min(cxt.quest:GetRank(), #MONSTER_PARTY)]
                local creatures = CreateCombatParty(monster_table[cxt:GetRNG():Random(#monster_table)], cxt.quest:GetRank(), cxt.location)
                cxt.enc:SetPrimaryCast(creatures[1])
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                        on_win = function(cxt) 
                            for k,v in ipairs(creatures) do
                                if not v:IsDead() then
                                    v:MoveToLimbo()
                                end
                            end
                            cxt:Dialog("DIALOG_ON_WIN")
                            cxt:GoTo("STATE_CHOICE")
                        end,
                    }
            end)

    :State("STATE_FLEADS")
        :Loc{
            DIALOG_INTRO = [[
                * Eggs of various sizes cover the ground. Things pulsate and churn inside their luminescent shells. As you make your way past the eggs, you start to feel like maybe you shouldn't be here.
                agent:
                    !right
                    $angryHostile
                    <i>Gurrrraauggh</i>.
                player:
                    !left
                    !sigh
                    $scaredFearful
                    Aw, crap.
            ]],
            OPT_FIGHT = "Defend yourself",
            DIALOG_ON_WIN = [[
                * The creatures guarding the eggs are gone, but you'd rather not stick around long enough for the eggs to hatch.
                * Before you move on, you grab a single egg to take with you. It might come in handy.
            ]]
        }
        :Fn(function(cxt) 
                local monster_table = MONSTER_PARTY[math.min(cxt.quest:GetRank(), #MONSTER_PARTY)]
                local creatures = CreateCombatParty(monster_table[cxt:GetRNG():Random(#monster_table)], cxt.quest:GetRank(), cxt.location)
                cxt.enc:SetPrimaryCast(creatures[1])
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                        on_win = function(cxt) 
                            for k,v in ipairs(creatures) do
                                if not v:IsDead() then
                                    v:MoveToLimbo()
                                end
                            end
                            cxt:Dialog("DIALOG_ON_WIN")
                            cxt:ForceTakeCards{"flead_egg"}
                            cxt:GoTo("STATE_CHOICE")
                        end,
                    }
            end)

    :State("STATE_PLANTS")
        :Loc{
            DIALOG_INTRO = [[
                * As you turn a corner, you spot a fissure in the ground, fuming with toxic vapors.
                * All around the fissure you can see a rare healing wort growing.
            ]],
            OPT_HEAL_LITTLE = "Pick a small leaf to chew (Heals <#HEALTH>{1} HP</>)",
            DIALOG_HEAL_LITTLE = [[
                * You pick one of the leaves and nibble on it. It doesn't taste right, but you feel stronger all the same.
            ]],
            OPT_HEAL_BIG = "Pick one of the weird fruits to eat (Heals <#HEALTH>{1} HP</>, receive {2#card})",
            DIALOG_HEAL_BIG = [[
                * You pick one of the plump, black fruits and take a bite. The flesh inside glows brilliant green.
                * You feel stronger, but your stomach turns from the rancid sweet taste. 
            ]],
            OPT_HEAL_BIG_TT = "You manage to keep it down, but you're not sure if it was for the best.",

            OPT_INCREASE_MAX = "Eat until you can't eat any more (<#HEALTH>Max HP +{1}</>, take <#HEALTH>{2} HP</> damage)",
            DIALOG_INCREASE_MAX = [[
                * You pound back fruit after fruit, pausing only to devour mouthfuls of leaves.
            ]],
            DIALOG_INCREASE_MAX_2 = [[
                * You pass out, alien chemicals coursing through your system, and alien thoughts coursing through your head.
            ]],
            DIALOG_INCREASE_MAX_3 = [[
                * Max HP raised by <#HEALTH>{1}</>.
                player:
                    !left
                    !dust_off
                * You wake up in a puddle of fruit juice and your own filth.
            ]],

            DIALOG_DONE = [[
                * The plant shudders as if hurt by the harvest. To your amazement, it pulls up roots and slithers away into the bush. 
                player:
                    !left
                    !surprise
                    Uh.
                    !palm
                * You hope it was just a hallucination. 
            ]],
            DIALOG_DONE_NO_HEAL = [[
                * You decide against touching the luminized plant. The plant shudders as if stung by your rejection. To your amazement, it pulls up roots and slithers away into the bush. 
                player:
                    !left
                    !surprise
                    Uh.
                    !palm
                * You hope it was just a hallucination. 
            ]],

            REQ_HEALTH_TOO_LOW = "That would kill you!",
            OPT_CONTINUE = "Continue through the cave",
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")

            local little_pts = 10
            local big_pts = 20
            
            local health, max_health = cxt.player.health:Get()
            local increase_pts = 10
            local hurt_pts = 15

            cxt:Opt("OPT_HEAL_LITTLE", little_pts)
                :PreIcon(global_images.heallittle)
                :Dialog("DIALOG_HEAL_LITTLE")
                :DeltaHealth(little_pts)
                :Dialog("DIALOG_DONE")
                :Fn(function(cxt)
                    cxt:Opt("OPT_CONTINUE")
                        :GoTo("STATE_CHOICE")
                end)
                
            cxt:Opt("OPT_HEAL_BIG", big_pts, "drunk")
                :PreIcon(global_images.heal)
                :Dialog("DIALOG_HEAL_BIG")
                :PostText("OPT_HEAL_BIG_TT")
                :DeltaHealth(big_pts)
                :Fn(function()
                    cxt:ForceTakeCards{"drunk"}
                    cxt:Dialog("DIALOG_DONE")
                    cxt:Opt("OPT_CONTINUE")
                        :GoTo("STATE_CHOICE")
                end)

            cxt:Opt("OPT_INCREASE_MAX", increase_pts, hurt_pts)
                :ReqCondition(health > hurt_pts, "REQ_HEALTH_TOO_LOW")
                :Dialog("DIALOG_INCREASE_MAX")
                :DeltaHealth(-hurt_pts)
                :Dialog("DIALOG_INCREASE_MAX_2")
                :FadeOut()
                :Fn(function() 
                    cxt.player.health:AddStatModifier( "EVENT_INCREASE", increase_pts ) 
                    cxt:Dialog("DIALOG_INCREASE_MAX_3", increase_pts)
                    cxt:FadeIn()
                    cxt:Opt("OPT_CONTINUE")
                        :GoTo("STATE_CHOICE")
                end)


            cxt:Opt("OPT_LEAVE"):MakeUnder()
                :PreIcon( global_images.close)
                :AddScore("SKIPPED_HEALING")
                :Dialog("DIALOG_DONE_NO_HEAL")
                :GoTo("STATE_CHOICE")
        end)

    :State("STATE_TRAPPED")
        :Loc{
            DIALOG_INTRO = [[
                * You come into a clearing where a hapless spelunker is injured, trapped under heavy stones, {agent.hisher} spoils strewn about the ground.
                player:
                    !left
                agent:
                    !right
                    !injured
                    Oh thank Hesh! I knew someone would show up eventually! There was a cave in, now get this rubble off me and I’ll pay you back after we get out of here!
            ]],
            OPT_HELP_STRANGER = "Help the stranger out of the rubble",
            DIALOG_HELP_STRANGER = [[
                player:
                    Of course, we can’t just leave you here. Give me a hand.
                * You lift away some of the heavier rocks, and together free their leg. They limp up, grimacing in pain but seem to be able to stand on their own.
                agent:
                    Wow, does it feel good to be out from under there. Let’s get out of here!
            ]],
            OPT_ROB_STRANGER = "Steal the stranger's things and leave them",
            DIALOG_ROB_STRANGER = [[
                * You walk up to the supplies scattered the ground and start collecting the least broken ones and putting them in your pack.
                agent: 
                    Wait… what are you doing!?
                player: 
                    What does it look like? I’m robbing you and leaving you here to die.
                agent: 
                    You’re horrible! Please don’t do this!
                * You shoulder your new supplies and set off.
            ]],
            OPT_CONTINUE = "Continue through the cave",
            DIALOG_LEAVE = [[
                * You begin to slowly walk away from the stranger.
                agent:
                    Wait! No! Please!
            ]],
        }
        :Fn(function(cxt)
            local stranger = TheGame:GetGameState():AddSkinnedAgent( "BRAVE_MERCHANT" )
            stranger:GetAspect("health"):SetPercent(.6)
            cxt.enc:SetPrimaryCast(stranger)
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_HELP_STRANGER")
                :Dialog("DIALOG_HELP_STRANGER")
                :Fn(function(cxt)
                    cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.HIRED )
                    cxt:Opt("OPT_CONTINUE")
                        :GoTo("STATE_CHOICE")
                end)
                
            cxt:Opt("OPT_ROB_STRANGER")
                :Dialog("DIALOG_ROB_STRANGER")
                :Fn(function(cxt)
                    cxt:ForceTakeCards{"salve", "lumin_grenade", "spark_grenade", "tincture", "vial_of_slurry"}
                    cxt:Opt("OPT_CONTINUE")
                        :GoTo("STATE_CHOICE")
                end)


            cxt:Opt("OPT_LEAVE"):MakeUnder()
                :PreIcon( global_images.close)
                :Dialog("DIALOG_LEAVE")
                :GoTo("STATE_CHOICE")
        end)

    :State("STATE_STASH")
        :Loc{
            DIALOG_INTRO = [[
                * You find a small box obscured by some rocks in a nook in the wall.
                player:
                    !left
                * Inside the box, you find several items that could be useful.
            ]],
            OPT_TAKE_CARD = "Take {1#card}",
            OPT_SKIP = "Continue through the cave",
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            local items = {}
            for i=1,4 do
                table.insert(items, search_items[cxt:GetRNG():Random(#search_items)])
            end
            cxt:RunLoop(function( ... )
                if #items > 0 then
                    for i,v in ipairs(items) do
                        cxt:Opt("OPT_TAKE_CARD", v)
                            :GainCard(v)
                            :Fn(function(cxt)
                                table.remove(items, i)
                            end)
                    end
                    cxt:Opt("OPT_SKIP")
                        :GoTo("STATE_CHOICE")
                else
                    cxt:GoTo("STATE_CHOICE")
                end
            end) 
        end)

    :State("STATE_FREE_GRAFT")
        :Loc{
            DIALOG_INTRO = [[
                * You turn a corner and find {agent} tinkering with something.
                player:
                    !left
                agent:
                    !right
                    !surprised
                    Oh! I didn't see you coming, {player}.
                player:
                    Uh what are you doing here?
                agent:
                    Uhm. Tell you what, if you just pretend you never saw me, you can have one of these on the house.
            ]],
            OPT_CHOOSE_GRAFT = "Choose a graft",
            DIALOG_CHOSE_GRAFT = [[
                player:
                    !graft
                    Thanks! This'll help me out.
                agent:
                    Always a pleasure.
            ]],
            DIALOG_SKIPPED_GRAFT = [[
                player:
                    Uh, no thanks. I won't tell anyone though, I just want to get out of here.
                agent:
                    Oh, alright then. I trust you.
            ]],
        }
        :Fn(function(cxt)
            local plocka = cxt.quest:CreateSkinnedAgent( "PLOCKA" )
            cxt.enc:SetPrimaryCast(plocka)

            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_CHOOSE_GRAFT")
                :Fn(function(cxt)
                    local rng = cxt:GetRNG( "GRAFTS" )
                    if rng then
                        push_random( function( n, m )
                            return rng:Random( n, m )
                        end )
                    end

                    local grafts = RewardUtil.GetPooledGrafts( 4, 3, cxt.quest.param.graft_pool )

                    if rng then
                        pop_random()
                    end

                    local popup = Screen.PickGraftScreen(grafts, false, function(...) cxt.enc:ResumeEncounter(...) end)
                    TheGame:FE():InsertScreen( popup )

                    local chosen_graft = cxt.enc:YieldEncounter()
                    if chosen_graft then
                        cxt:Dialog("DIALOG_CHOSE_GRAFT")
                    else
                        cxt:Dialog("DIALOG_SKIPPED_GRAFT")
                    end
                    cxt:GoTo("STATE_CHOICE")
                end)
        end)

    :State("STATE_ADMIRALTY")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * The sound of chatter gets louder and just before you can begin to make out the words, you're confronted.
                agent:
                    !right
                    Hey! You there!
                    You're coming with us!
                player:
                    What's going on?
                agent:
                    Don't play dumb with me. We already know you guys are keeping illicit goods in these caves.
            ]],
            OPT_NEGOTIATE = "Convince {agent} to let you pass",
            DIAL_NEGOTIATE = [[
                player:
                    Wait, wait, wait! I'm not the one you're looking for. I have no idea about any stolen goods.
                agent:
                    And why would I take your word for it?
            ]],
            DIAL_NEGOTIATION_WIN = [[
                agent:
                    I guess you don't look like any of the other smugglers...
                player:
                    I'm really just trying to get back out of this place.
                agent:
                    Alright, alright, just get out of here before I change my mind.
                * You hurriedly get on your way.
            ]],
            DIAL_NEGOTIATION_FAIL = [[
                agent:
                    I don't buy it for a second!
                    !fight
                player:
                    !fight
            ]],
            OPT_DEFEND = "Defend yourself",
            OPT_ATTACK = "Attack {agent}",
            DIAL_ATTACK = [[
                player:
                    I don't have time for arguing.
                agent:
                    !fight
                player:
                    !fight
            ]],
            DIAL_SURVIVORS = [[
                agent:
                    !exit
                player:
                    !left
                * As soon as you lower your weapons, the remaining Admiralty run away in the opposite direction.
            ]],
            DIAL_ALL_DEAD = [[
                agent:
                    !exit
                player:
                    !left
                * All the Admiralty are dead. You just hope you don't run into those smugglers they mentioned.
            ]],
            OPT_LEAVE = "Leave",
            DIAL_LEAVE = [[
                * Not wanting to wait around for anymore Admiralty, you leave their camp behind.
            ]],
        }
        :Fn(function(cxt) 
            local guard_table = ADMIRALTY_PATROL[math.min(cxt.quest:GetRank(), #ADMIRALTY_PATROL)]
            local guards = CreateCombatParty(guard_table[cxt:GetRNG():Random(#guard_table)], cxt.quest:GetRank(), cxt.location)
            cxt.enc:SetPrimaryCast(guards[1])
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_FIGHT")
                :Dialog("DIAL_ATTACK")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                    on_win = function(cxt) 
                        local survivors = {}
                        for k,v in ipairs(guards) do
                            if not v:IsDead() then
                                if v:IsSentient() then
                                    table.insert(survivors, v)
                                end
                                v:MoveToLimbo()
                            end
                        end
                        if #survivors > 0 then
                            cxt:Dialog("DIAL_SURVIVORS")
                        else
                            cxt:Dialog("DIAL_ALL_DEAD")
                        end

                        cxt:Opt("OPT_LEAVE")
                            :Dialog("DIAL_LEAVE")
                            :GoTo("STATE_CHOICE")
                    end,

            cxt:Opt("OPT_NEGOTIATE")
                :Dialog("DIAL_NEGOTIATE")
                :Negotiation{
                    on_success = function() 
                        cxt:Dialog("DIAL_NEGOTIATION_WIN")
                        cxt:GoTo("STATE_CHOICE")
                    end,
                    on_fail = function() 
                        cxt:Dialog("DIAL_NEGOTIATION_FAIL")
                        cxt:Opt("OPT_DEFEND")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                                on_win = function(cxt) 
                                    local survivors = {}
                                    for k,v in ipairs(guards) do
                                        if not v:IsDead() then
                                            if v:IsSentient() then
                                                table.insert(survivors, v)
                                            end
                                            v:MoveToLimbo()
                                        end
                                    end
                                    if #survivors > 0 then
                                        cxt:Dialog("DIAL_SURVIVORS")
                                    else
                                        cxt:Dialog("DIAL_ALL_DEAD")
                                    end

                                    cxt:Opt("OPT_LEAVE")
                                        :Dialog("DIAL_LEAVE")
                                        :GoTo("STATE_CHOICE")
                                end
                            }
                    end,
                }
            }
        end)

    :State("STATE_ERCHIN")
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !right
            * You hear a scuffle as you round a corner.
            * Drawing closer, you see a couple of erchins circle around a lone vroc, preparing to attack.
        ]],
        OPT_ATTACK_THE_ERCHINS = "Attack the erchins",
        DIALOG_ATTACK_THE_ERCHINS = [[
            player:
                !left
                !fight
            agent:
                !right
            * You draw weapons and charge!
            agent:
                !exit
        ]],
        DIALOG_BEAT_ERCHINS = [[
            player:
                !left
            agent:
                !right
            * The threat of erchin attack is over. As the dust settles, {agent} limps over and cuddles up against your leg.
            player:
                Uhh, you're welcome?
            * It seems like {agent} wants to tag along.
        ]],
        OPT_TAKE_PET = "Take {agent} with you",
        DIALOG_TAKE_PET = [[
            player:
                Alright, come on! I'll take care of you.
            * You pet {agent} and continue down the cave, a little less lonely than before.
        ]],
        OPT_LEAVE_NO_PET = "Leave without {agent}",
        DIALOG_LEAVE_WITHOUT_PET = [[
            player:
                Sorry, I just don't think it'll be safe for you.
            * As you leave {agent} looks at you with confused eyes, but eventually finds a comfortable spot to lie down.
        ]],
        OPT_LEAVE_NO_FIGHT = "Leave before the erchins see you",
        OPT_LEAVE = "Leave",
        DIALOG_LEAVE = [[
            * You decide not to risk fighting the erchins, and leave as they close in on the vroc.
        ]],
    }
    :Fn(function(cxt)
        cxt.quest.param.vroc = Agent("VROC")
        TheGame:GetGameState():AddAgent(cxt.quest.param.vroc)
        cxt.quest.param.erchins = CreateCombatParty({"ERCHIN", "ERCHIN"}, cxt.quest:GetRank(), cxt.location) 
        cxt.enc:SetPrimaryCast(cxt.quest.param.erchins[1])
        cxt:Dialog("DIALOG_INTRO")

        cxt:Opt("OPT_ATTACK_THE_ERCHINS")
            :Robot("ATTACK")
            :Dialog("DIALOG_ATTACK_THE_ERCHINS")
            :Battle{
                enemies = cxt.quest.param.erchins,
                flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                allies = {cxt.quest.param.vroc}
            }
                :OnWin()
                    :Fn(function() 
                        if not cxt.quest.param.vroc:IsRetired() then
                            cxt.enc:SetPrimaryCast(cxt.quest.param.vroc)
                            cxt:Dialog("DIALOG_BEAT_ERCHINS")

                            cxt:Opt("OPT_TAKE_PET")
                                :Fn(function(cxt)
                                    cxt.quest.param.vroc:GenerateRandomName()
                                    cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.CREW)
                                    cxt:Dialog("DIALOG_TAKE_PET")
                                    cxt:Opt("OPT_LEAVE")
                                        :GoTo("STATE_CHOICE")
                                end)

                            cxt:Opt("OPT_LEAVE_NO_PET")
                                :Dialog("DIALOG_LEAVE_WITHOUT_PET")
                                :AddScore("DECLINED_PET")
                                :GoTo("STATE_CHOICE")
                        else
                            cxt:GoTo("STATE_CHOICE")
                        end
                    end)        

        cxt:Opt("OPT_LEAVE_NO_FIGHT")
            :Dialog("DIALOG_LEAVE")
            :GoTo("STATE_CHOICE")
    end)

    :State("STATE_RISE")
        :Loc{
            DIALOG_INTRO = [[
                * You push aside the branches and leaves enough to make your way through the tunnel.
                player:
                    !left
                agent:
                    !right
                * Despite your attempts to move silently, {agent} sees you before you see {agent.himher}.
                agent:
                    !surprised
                    Huh? 
                    !angry
                    You shouldn't be here. 
                player:
                    I couldn't agree more, I'd love to get the Hesh out of here.
                agent:
                    I'm not joking around here. You made a big mistake coming here!
            ]],
            OPT_NEGOTIATE = "Convince {agent} to let you go",
            DIAL_NEGOTIATE = [[
                player:
                    No really, I have no idea what's happening. I'll just leave and we can both get on with our lives.
            ]],
            DIAL_NEGOTIATION_WIN = [[
                agent:
                    Fine... But only because you haven't seen anything. I'm not afraid of you!
                player:
                    Of course not! I'll be on my way now.
                * You hurriedly make your exit.
            ]],
            DIAL_NEGOTIATION_FAIL = [[
                agent:
                    Alright, I'm sick of talking. 
                    !fight
                player:
                    !fight
            ]],
            OPT_DEFEND = "Defend yourself",
            OPT_ATTACK = "Attack {agent}",
            DIAL_ATTACK = [[
                player:
                    I don't have time for your nonsense.
                agent:
                    !fight
                player:
                    !fight
            ]],
            DIAL_SURVIVORS = [[
                agent:
                    !exit
                player:
                    !left
                * As soon as you lower your weapons, the remaining Rise members back off and let you pass.
            ]],
            DIAL_ALL_DEAD = [[
                agent:
                    !exit
                player:
                    !left
                * All the Rise are dead.
            ]],
            OPT_LEAVE = "Leave",
            DIAL_LEAVE = [[
                * The area looks like they were preparing it to be used for storage, maybe they were smuggling in weapons?
            ]],
        }
        :Fn(function(cxt) 
            local guard_table = RISE_PATROL[math.min(cxt.quest:GetRank(), #RISE_PATROL)]
            local guards = CreateCombatParty(guard_table[cxt:GetRNG():Random(#guard_table)], cxt.quest:GetRank(), cxt.location)
            cxt.enc:SetPrimaryCast(guards[1])
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_FIGHT")
                :Dialog("DIAL_ATTACK")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                    on_win = function(cxt) 
                        local survivors = {}
                        for k,v in ipairs(guards) do
                            if not v:IsDead() then
                                if v:IsSentient() then
                                    table.insert(survivors, v)
                                end
                                v:MoveToLimbo()
                            end
                        end
                        if #survivors > 0 then
                            cxt:Dialog("DIAL_SURVIVORS")
                        else
                            cxt:Dialog("DIAL_ALL_DEAD")
                        end

                        cxt:Opt("OPT_LEAVE")
                            :Dialog("DIAL_LEAVE")
                            :GoTo("STATE_CHOICE")
                    end,

            cxt:Opt("OPT_NEGOTIATE")
                :Dialog("DIAL_NEGOTIATE")
                :Negotiation{
                    on_success = function() 
                        cxt:Dialog("DIAL_NEGOTIATION_WIN")
                        cxt:GoTo("STATE_CHOICE")
                    end,
                    on_fail = function() 
                        cxt:Dialog("DIAL_NEGOTIATION_FAIL")
                        cxt:Opt("OPT_DEFEND")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                                on_win = function(cxt) 
                                    local survivors = {}
                                    for k,v in ipairs(guards) do
                                        if not v:IsDead() then
                                            if v:IsSentient() then
                                                table.insert(survivors, v)
                                            end
                                            v:MoveToLimbo()
                                        end
                                    end
                                    if #survivors > 0 then
                                        cxt:Dialog("DIAL_SURVIVORS")
                                    else
                                        cxt:Dialog("DIAL_ALL_DEAD")
                                    end

                                    cxt:Opt("OPT_LEAVE")
                                        :Dialog("DIAL_LEAVE")
                                        :GoTo("STATE_CHOICE")
                                end,
                            }
                    end,
                }
            }
        end)

    :State("STATE_JAKES")
        :Loc{
            DIALOG_INTRO = [[
                * You follow the marks in the floor and eventually find several crates stacked against the wall of the cave.
                player:
                    !left
                    !thought
                    I wonder who left these here.
            ]],
            OPT_SEARCH_CRATES = "Search the crates",
            OPT_SEARCH_CRATES_AGAIN = "Continue searching the crates",
            TT_SEARCH = "The longer you spend here, the better chance there is of getting caught.",
            DIALOG_SEARCH_SUCCESFUL = [[
                * You rummage through the crates, making sure to not make too much noise.
                * You find a {1#card}.
            ]],
            DIALOG_SEARCH_SUCCESFUL_2 = [[
                * You continue looking around in the crates and find {1#card}.
            ]],
            DIALOG_SEARCH_FAIL = [[
                * You've got your head practically inside one of the crates when you hear a sound behind you.
                player:
                    !left
                agent:
                    !right
                    Well, well, well. Looks like someone's looking where they ought not to.
                player:
                    Oh crap.
            ]],
            OPT_ATTACK = "Defend yourself",
            DIAL_ATTACK = [[
                agent:
                    !fight
                player:
                    !fight
            ]],
            DIAL_SURVIVORS = [[
                agent:
                    !exit
                player:
                    !left
                * The remaining Jakes make a run for it, deciding the merchandise isn't worth their lives.
            ]],
            DIAL_ALL_DEAD = [[
                agent:
                    !exit
                player:
                    !left
                * All the Jakes are dead, but it's probably better to leave before anyone else arrives.
            ]],
            OPT_LEAVE = "Leave",
        }
        :Fn(function(cxt) 
            cxt.enc.scratch.attempts = 0
            local guard_table = JAKES_PATROL[math.min(cxt.quest:GetRank(), #JAKES_PATROL)]
            local guards = CreateCombatParty(guard_table[cxt:GetRNG():Random(#guard_table)], cxt.quest:GetRank(), cxt.location)
            cxt.enc:SetPrimaryCast(guards[1])
            cxt:Dialog("DIALOG_INTRO")
            local succeses = cxt:GetRNG():Random(1, 3)
            print(succeses)
            cxt:RunLoopingFn(function()
                cxt:Opt(cxt.enc.scratch.attempts == 0 and "OPT_SEARCH_CRATES" or "OPT_SEARCH_CRATES_AGAIN")
                    :AddScore("SEARCH_CRATES")
                    :PostText("TT_SEARCH")
                    :Fn(function(cxt)
                        cxt.enc.scratch.attempts = cxt.enc.scratch.attempts + 1
                        if cxt.enc.scratch.attempts <= succeses then
                            local item = search_items[math.random(#search_items)]
                            cxt:Dialog(cxt.enc.scratch.attempts == 1 and "DIALOG_SEARCH_SUCCESFUL" or "DIALOG_SEARCH_SUCCESFUL_2", item)
                            cxt:ForceTakeCards{ item }
                        else
                            cxt:Dialog("DIALOG_SEARCH_FAIL")
                            cxt:Opt("OPT_FIGHT")
                                :Dialog("DIAL_ATTACK")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                                    on_win = function(cxt) 
                                        local survivors = {}
                                        for k,v in ipairs(guards) do
                                            if not v:IsDead() then
                                                if v:IsSentient() then
                                                    table.insert(survivors, v)
                                                end
                                                v:MoveToLimbo()
                                            end
                                        end
                                        if #survivors > 0 then
                                            cxt:Dialog("DIAL_SURVIVORS")
                                        else
                                            cxt:Dialog("DIAL_ALL_DEAD")
                                        end

                                        cxt:Opt("OPT_LEAVE")
                                            :GoTo("STATE_CHOICE")
                                    end,
                                }
                        end
                    end)
                cxt:Opt("OPT_LEAVE")
                    :GoTo("STATE_CHOICE")
            end)       
        end)

    :State("STATE_BOSS")
        :Loc{
            DIALOG_INTRO = [[
                * You can finally see daylight at the end of the tunnel.
                player:
                    !left
                    About time! I feel like I've been walking in circles in here.
                * As you exit the cave, your eyes take a second to adjust to the light. At this point you notice a figure skulking in the shadow.
                agent:
                    !right
                    !angryHostile
                    I gotta admit, I didn't expect you to actually make it out.
                    I guess I have to finish this myself!
                    !fight
                player:
                    !fight
            ]],
            OPT_FIGHT = "Defend yourself",
            OPT_WIN = "Continue",
        }
        :Fn(function(cxt) 
    
                EnterCave(cxt)
                local boss = cxt.quest:CreateSkinnedAgent(cxt.quest.param.boss_id)
                cxt.enc:SetPrimaryCast(boss)
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.NO_REWARDS,
                        on_win = function(cxt) 
                            cxt:Opt("OPT_WIN")
                                :Fn(function(cxt)
                                    cxt.quest:Complete()
                                end)
                        end,
                    }
            end)