local daily = require "daily_util"

local character_options =
{
    "SAL", "ROOK", "SMITH"
}

local ally_options = 
{
    "BRAVE_MERCHANT"
}

local easy_fights =
{
    {"CRAYOTE_SPITTER", "CRAYOTE"},
    {"ERCHIN", "ERCHIN"},
    {"LUMICYTE", "LUMICYTE"},
    {"ADMIRALTY_GUARD"},
    {"BANDIT_RAIDER"},
    {"BANDIT_GOON2"},
}

local regular_fights =
{
    {"LUMINTHIAN"},
    {"CRAYOTE_SPITTER", "CRAYOTE", "CRAYOTE"},
    {"GRAWKIT"},
    {"ADMIRALTY_GOON", "VROC", "VROC"},
    {"ADMIRALTY_PATROL_LEADER", "ADMIRALTY_CLERK"},
    {"JAKES_RUNNER", "JAKES_RUNNER"},
    {"BANDIT_GOON", "FLEAD", "FLEAD"},
    {"BANDIT_RAIDER", "FLEAD"},
}

local QDEF = QuestDef.Define
{
    title = "Protection",
    icon = engine.asset.Texture("icons/quests/daily_escape_cave.tex"),
    desc = [[Protect your injured ally from waves of enemies.]],
    qtype = QTYPE.DAILY,
    on_complete = function( quest )
        quest:AddScore("WON_DAILY")
        TheGame:Win()
    end,
    on_fail = function( quest )
        TheGame:Lose()
    end,
    ChooseCharacter = function( self, rng )
        return character_options[rng:Random(#character_options)]
    end,

    GenerateMutatorsAndFeats = function(self, rng, character_id)
        
        local arg = {
            battle_draft = {
                "havarian_mulligan_battle",
                "picky_draft_battle",
                "battle_bundle",
            },
            always_take = {
                "brilliance",
            },
            mutators = {
                "parasitology",
                "roll_with_it",
                "mental_overload",
                "pet_owner",
                "contaminated_equipment",
                "score_card_mutator",
                "keepsake_mutator",
            },
            feats = {
                {"vicious_feat", "no_sweat", "not_a_scratch"},
                {"smashing", "no_sweat", "take_your_shot"},
                {"smashing", "thorough_search", "clean_fight"},
                {"vicious_feat", "play_dead", "quick_work"},
                {"smashing", "tank_up", "take_your_shot"},
            },
            min_mutators = 2,
            max_mutators = 3,
        }

        return daily.GenerateMutatorsAndFeats(rng, character_id, arg)
    end,
    
    GenerateParams = function(self, rng)
        local param = {}
        param.secondary = ally_options[rng:Random(#ally_options)]
        return param
    end,

    GenerateProgressionData = function(qdef, param)

        local current_day = {}

        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_encounter.tex"), tt = LOC( qdef:GetLocKey("TT_ENCOUNTER"))} )
        table.insert(current_day, { icon = engine.asset.Texture("UI/daily_boss.tex"), tt = LOC( qdef:GetLocKey("TT_BOSS"))} )

        local current_step = param.step or 0
        for k,v in ipairs(current_day) do
            if k == current_step then
                v.right_now = true
            elseif k < current_step then
                v.success = true
            end
        end

        return {current_day}
    end,

    get_progression = function(quest)
        return quest:GetQuestDef().GenerateProgressionData(quest:GetQuestDef(), quest.param)
    end,

    plax = "EXT_Bog_Forest_01",

    on_init = function(quest)
        TheGame:GetGameState():GetCaravan():MoveToLocation(quest:GetCastMember("the_forest"))
    end,
}

:Loc{
    TT_ENCOUNTER = "This is an encounter. You will have to play your cards right to progress!",
    TT_BOSS = "A powerful enemy is waiting for you."
}

:AddObjective{
    id = "start",
    title = "Defend {friend}",
    on_activate = function( quest)
        quest.param.location = quest:SpawnTempLocation("GROUT_BOG_FOREST", "the_forest" )
        
    end,
    state = QSTATUS.ACTIVE,
}


:AddCast{
    cast_id = "the_forest",
    when = QWHEN.MANUAL,
}
:AddCast{
    cast_id = "friend",
    when = QWHEN.MANUAL,
}

:AddScoreDefs{   
    SKIPPED_HEALING = 
    {
        name = "Skipped Healing",
        value = 40,
    },
}

local function FindNewOptions(cxt)
        if #cxt.quest.param.available_encounters > 0 then
            cxt.combat_party = table.remove(cxt.quest.param.available_encounters, 1)
            return true
        else
            return false
        end
end

local function DoGraftDraft( cxt, difficulty, special )

    local rng = cxt:GetRNG( "GRAFTS" )
    if rng then
        push_random( function( n, m )
            return rng:Random( n, m )
        end )
    end
    local grafts
    if special then
        grafts = cxt.quest.param.special_grafts:Generate(3)
    else
        grafts = RewardUtil.GetPooledGrafts( difficulty, 3, cxt.quest.param.graft_pool )
    end

    if rng then
        pop_random()
    end

    local popup = Screen.PickGraftScreen(grafts, false, function(...) cxt.enc:ResumeEncounter(...) end)
    popup:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
    TheGame:FE():InsertScreen( popup )

    local chosen_graft = cxt.enc:YieldEncounter()
end

local function InitializeRun( cxt )
    cxt.encounter:DoLocationTransition( cxt.quest.param.location )
    local time = os.time()
    time = time - (time % (60*60*24))
    cxt.quest.rng = cxt.quest.rng or engine.Random(time)
    cxt.quest.param.available_encounters = {}
    
    --Shuffle easy then add 2 to available_encounters
    local chosen_easy = ObtainWorkTable()
    table.arrayadd(chosen_easy, easy_fights)
    if cxt.quest.rng then
        table.seeded_shuffle(chosen_easy, cxt.quest.rng)
    else
        table.shuffle(chosen_easy )
    end
    for i=1,2 do
        table.insert(cxt.quest.param.available_encounters, table.remove(chosen_easy, 1))
    end
    ReleaseWorkTable(chosen_easy)

    --Shuffle regular then add 4 to available_encounters
    local chosen_regular = ObtainWorkTable()
    table.arrayadd(chosen_regular, regular_fights)
    if cxt.quest.rng then
        table.seeded_shuffle(chosen_regular, cxt.quest.rng)
    else
        table.shuffle(chosen_regular)
    end
    for i=1,4 do
        table.insert(cxt.quest.param.available_encounters, table.remove(chosen_regular, 1))
    end
    ReleaseWorkTable(chosen_regular)
    cxt.quest.param.boss_id = cxt.quest.rng:Random() < .5 and "RISE_SECOND" or "SPARK_SECOND"
    cxt.quest:SetRank(2)
    local friend = cxt.quest:CreateSkinnedAgent(cxt.quest.param.secondary or "ADMIRALTY_GUARD")
    friend:GetAspect("health"):SetPercent(.45)
    cxt.quest:AssignCastMember("friend", friend)
    cxt.quest.param.step = 1
    cxt.quest:NotifyChanged()
    local function Filter( graft_def )
        return graft_def.type == GRAFT_TYPE.COMBAT
    end
    cxt.quest.param.graft_pool = GraftCollection.GraftPool( cxt.player, Filter, cxt:GetRNG( "GRAFTS" ) )
    cxt.quest.param.special_grafts = GraftCollection(function(graft_def) return graft_def.type == GRAFT_TYPE.COMBAT end):NotInstalled(cxt.player):NotRestricted(cxt.player):Rarity(CARD_RARITY.COSMIC)
end


QDEF:AddConvo("start")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You've stumbled upon {friend} in the forest, injured and afraid.
                friend:
                    !injured
                    !right
                    Please, you've got to help me! I think there might be more people coming.
                * You get ready for any enemies that may be lurking about.
            ]],
            OPT_CONTINUE = "Continue",
            OPT_GRAFT = "Choose a starting graft",
        }
        :Fn(function(cxt) 
            if StateGraphUtil.RestoreDaily( cxt ) then
                return
            end

            InitializeRun(cxt)
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:GetCastAgent("friend"):Recruit(PARTY_MEMBER_TYPE.CREW)
            cxt:GainCards{"bodyguard_contract"}
            cxt:Opt("OPT_GRAFT")
                :Fn(function(cxt)
                    DoGraftDraft(cxt, 1, true)
                    cxt:GoTo("STATE_ENCOUNTER")
                end)          
        end)
    
    :State("STATE_ENCOUNTER")
        :Loc{
            DIALOG_CLEAR = [[
                player:
                    !exit
                agent:
                    !exit
            ]],
            DIALOG_SENTIENT = [[
                * Looks like {friend} was right about more people coming.
                player:
                    !left
                agent:
                    !right
                player:
                    !fight
                agent:
                    !fight
            ]],
            DIALOG_NOT_SENTIENT = [[
                * Wild animals must have caught your scent and are out for an easy meal.
                player:
                    !left
                agent:
                    !right
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_ON_WIN = [[
                * You and {friend} are safe for now.
            ]],
            OPT_CONTINUE = "Continue",
        }
        :Fn(function(cxt) 
            if FindNewOptions(cxt) then
                local enemies = {}
                for k,id in pairs(cxt.combat_party) do
                    local agent = Agent(id)
                    table.insert(enemies, agent)
                    TheGame:GetGameState():AddAgent(agent)
                end

                cxt.enc:SetPrimaryCast(enemies[1])
                local flags = BATTLE_FLAGS.NO_FLEE
                if enemies[1]:IsSentient() then
                    cxt:Dialog("DIALOG_SENTIENT")
                    flags = flags | BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_SURRENDER
                else
                    cxt:Dialog("DIALOG_NOT_SENTIENT")
                    flags = flags | BATTLE_FLAGS.ISOLATED
                end
                cxt:Opt("OPT_DEFEND")
                    :Battle{
                        enemies = enemies,
                        flags = flags,
                        on_runaway = function( cxt, battle )
                            cxt:GoTo("STATE_LOST")
                        end,
                        on_start_battle = function(battle) 
                            local fighter = battle:GetFighterForAgent(cxt.enc:GetCastAgent("friend"))
                            fighter:AddCondition("protected_ally", 1)
                        end,
                        on_win = function(cxt)
                            cxt:Dialog("DIALOG_CLEAR")
                            cxt:Dialog("DIALOG_ON_WIN")
                            cxt:Opt("OPT_CONTINUE")
                                :GoTo("STATE_RESPITE")
                        end,
                    }
            else
                cxt:GoTo("STATE_BOSS")
            end
            
        end)

    :State("STATE_RESPITE")
        :Loc{
            DIALOG_INTRO = [[
                * You have some time to tend to your wounds.
            ]],
            OPT_SKIP = "Skip",
            OPT_HEAL = "Tend to your injuries",
            OPT_HEAL_FRIEND = "Heal your ally",
            TT_HEAL = "This also heals your allies",
        }
        :Fn(function(cxt) 
            cxt.quest.param.step = cxt.quest.param.step + 1
            cxt:Dialog("DIALOG_INTRO")
            cxt.enc:SetPrimaryCast(nil)

            local friend = cxt.enc:GetCastAgent("friend")

            cxt:Opt("OPT_HEAL")
                :PreIcon(global_images.heal)
                :PostText("TT_HEAL")
                :DeltaHealth(20)
                :Fn(function()
                    for i,agent in TheGame:GetGameState():GetCaravan():Members() do
                        if not agent:IsPlayer() then
                            agent:GetAspect("health"):Delta(20)
                        end
                    end
                    cxt:GoTo("STATE_ENCOUNTER")
                end)

            if cxt.player:GetAspect("health"):GetPercent() == 1 and friend and not friend:IsRetired() then
                cxt:Opt("OPT_HEAL_FRIEND")
                    :PreIcon(global_images.heal)
                    :Fn(function()
                        for i,agent in TheGame:GetGameState():GetCaravan():Members() do
                            if not agent:IsPlayer() then
                                agent:GetAspect("health"):Delta(20)
                            end
                        end
                        cxt:GoTo("STATE_ENCOUNTER")
                    end)
            end

            cxt:Opt("OPT_SKIP")
                :AddScore("SKIPPED_HEALING")
                :GoTo("STATE_ENCOUNTER")
        end)

    :State("STATE_LOST")
        :Loc{
            DIALOG_LOST = [[
                * You failed to keep {friend} alive.
            ]],
            OPT_ACCEPT = "Accept",
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_LOST")
            cxt:Opt("OPT_ACCEPT")
                :Fn(function(cxt)
                    cxt.quest:Fail()
                end)
        end)

    :State("STATE_BOSS")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !angryHostile
                    I guess I'll have to do this myself!
                    !fight
                player:
                    !fight
            ]],
            OPT_FIGHT = "Defend yourself",
            OPT_WIN = "Continue",
        }
        :Fn(function(cxt) 
                local boss = cxt.quest:CreateSkinnedAgent(cxt.quest.param.boss_id)
                cxt.enc:SetPrimaryCast(boss)
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.NO_REWARDS,
                        on_win = function(cxt) 
                            cxt:Opt("OPT_WIN")
                                :Fn(function(cxt)
                                    cxt.quest:Complete()
                                end)
                        end,
                        on_start_battle = function(battle) 
                            local fighter = battle:GetFighterForAgent(cxt.enc:GetCastAgent("friend"))
                            fighter:AddCondition("protected_ally", 1)
                        end,
                        on_runaway = function( cxt, battle )
                            cxt:GoTo("STATE_LOST")
                        end,
                    }
            end)

local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.EVENT
local CARD_FLAGS = battle_defs.CARD_FLAGS

Content.AddBattleCondition( "protected_ally", {
    name = "Protected Ally",
    desc = "If this fighter dies, you lose the battle.",
    persist_on_death = true,

    event_handlers =
    {
        [ BATTLE_EVENT.FIGHTER_KILLED ] = function( self, fighter )
            if fighter == self.owner then
                self.battle:Runaway()
            end
        end
    },
})

local cards = 
{
    bodyguard_contract =
    {
        name = "Bodyguard Contract",
        anim = "taunt",
        desc = "Use special tools to protect your ally.",
        flavour = "’*Sigh* They’re with me.’",
        series = CARD_SERIES.GENERAL,
        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.TOOLBOX | CARD_FLAGS.SKILL,
        target_type = TARGET_TYPE.SELF,

        support_ids = {"reflector_shield", "single_out", "misdirection", "get_down"},
        pool_cards = {"reflector_shield", "single_out", "misdirection", "get_down"},
        refreshing = true, -- This card regains its tools after each fight

        OnInit = function( self )
            self.userdata.stored_cards = self.userdata.stored_cards or {}
            for i,id in ipairs(self.support_ids) do
                table.insert(self.userdata.stored_cards, Battle.Card(id, self.owner))
            end
        end,

        on_added = function( self, battle )
            self.userdata.stored_cards = {}
            for i,id in ipairs(self.support_ids) do
                table.insert(self.userdata.stored_cards, Battle.Card(id, self.owner))
            end
        end,
    },

    reflector_shield =
    {
        name = "Reflector Shield",
        anim = "taunt",
        desc = "Apply {1} {DEFEND} and {2} {RICOCHET} to an ally.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self:CalculateDefendText(self.defend_amt), self.ricochet_amt)
        end,
        flavour = "’It’s reliable enough. My face is proof that there’s going to be some collateral damage.’",

        cost = 0,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.FRIENDLY_OR_SELF,

        defend_amt = 10,
        ricochet_amt = 3,

        OnPostResolve = function( self, battle, attack )
            attack:AddCondition("DEFEND", self.defend_amt, self)
            attack:AddCondition("RICOCHET", self.ricochet_amt, self)
        end
    },

    single_out =
    {
        name = "Single Out",
        anim = "taunt",
        desc = "Force an enemy to target you this turn.",
        flavour = "’Hey! Nobody makes fun of Moreef but ME!’",

        cost = 0,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,

        OnPostResolve = function( self, battle, attack)
            local con = self.owner:AddCondition("single_out", 1, self)
            if con then
                con.affected = con.affected or {}
                for i,hit in ipairs(attack.hits) do
                    table.insert(con.affected, hit.target)
                end
            end
        end,

        condition =
        {
            hidden = true,

            CanBeTargetted = function( self, card, fighter )
                if self.affected and card:IsAttackCard() and table.contains(self.affected, card.owner) and not card:IsFlagged( CARD_FLAGS.SPECIAL ) then
                    if fighter:GetTeam() ~= card:GetOwner():GetTeam() and fighter:GetTeam() == self.owner:GetTeam() then
                        -- ... if the target is not us.
                        if fighter ~= self.owner and self.owner:IsAlive() then
                            return false, "Another enemy has Targeted"
                        end
                    end
                end
                return true
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if self.owner == fighter then
                        self.owner:RemoveCondition(self.id, self.stacks, self)
                    end
                end
            },
        },
    },

    misdirection =
    {
        name = "Misdirection",
        desc = "Force all enemies to target you this turn.",
        flavour = "’Oh they’re really mad now.’",

        cost = 0,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_mod = TARGET_MOD.TEAM,

        OnPostResolve = function( self, battle, attack)
            local con = self.owner:AddCondition("single_out", 1, self)
            if con then
                con.affected = con.affected or {}
                for i,hit in ipairs(attack.hits) do
                    table.insert(con.affected, hit.target)
                end
            end
        end,

        condition =
        {
            hidden = true,

            CanBeTargetted = function( self, card, fighter )
                if self.affected and card:IsAttackCard() and table.contains(self.affected, card.owner) and not card:IsFlagged( CARD_FLAGS.SPECIAL ) then
                    if fighter:GetTeam() ~= card:GetOwner():GetTeam() and fighter:GetTeam() == self.owner:GetTeam() then
                        -- ... if the target is not us.
                        if fighter ~= self.owner and self.owner:IsAlive() then
                            return false, "Another enemy has Targeted"
                        end
                    end
                end
                return true
            end,

            event_handlers =
            {
                [ BATTLE_EVENT.BEGIN_TURN ] = function( self, fighter )
                    if self.owner == fighter then
                        self.owner:RemoveCondition(self.id, self.stacks, self)
                    end
                end
            },
        },
    },

    get_down =
    {
        name = "Get Down!",
        desc = "Remove all of your {DEFEND} and apply that much {DEFEND} to an ally.",
        flavour = "’I’ll be fine, just stay here.’",

        cost = 0,

        flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
        target_type = TARGET_TYPE.FRIENDLY,

        OnPostResolve = function( self, battle, attack )
            local count = self.owner:GetConditionStacks("DEFEND")
            if count and count > 0 then
                self.owner:RemoveCondition("DEFEND", count, self)
                attack:AddCondition("DEFEND", count, self)
            end
        end
    },
}

for i, id, data in sorted_pairs(cards) do
    data.series = CARD_SERIES.GENERAL
    data.rarity = CARD_RARITY.UNIQUE

    Content.AddBattleCard( id, data )
end