local function SetNewPrestige( prestige )
    TheGame:GetGameState():SetAdvancementLevel(prestige)
    local agent = TheGame:GetGameState():GetPlayerAgent()
    agent:UpdateMaxHealth()
    local max = agent.def.max_resolve
    local advancement_mod = GetAdvancementModifier( ADVANCEMENT_OPTION.LOWER_MAX_RESOLVE )
    if advancement_mod then
        max = math.round(max * advancement_mod)
    end
    TheGame:GetGameState():GetCaravan():SetMaxResolve( max )
end

local QDEF = QuestDef.Define{
        title = "Rook Day 2 Boss Test",
        qtype = QTYPE.EXPERIMENT,
        disabled = true,
        icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
        rank = 3,
        act_filter = "ROOK",
        -- dev_only = true,
    }

    :AddLocationDefs{
        FLEAD_STAGING_SITE = {
            name = "Staging Site",
            plax = "EXT_Bog_Outcrop_01",
            indoors = false,
            map_tags = {"bog"},
        },
    }
    :AddObjective{
        id = "test",
        title = "Fight the boss, and provide feedback!",
        state = QSTATUS.ACTIVE,
    }

local PRE_FIGHT = function(cxt)
    local party
    
    if cxt.quest.param.handler_faction == "RISE" then
        party = {"RISE_PAMPHLETEER", "RISE_RADICAL", "RISE_REBEL", "RISE_AUTODOG", "RISE_AUTOMECH"}
    else
        party = {"SPARK_BARON_TASKMASTER", "SPARK_BARON_PROFESSIONAL", "SPARK_BARON_GOON", "AUTODOG", "SPARK_BARON_AUTOMECH"}
    end

    --spawn some rise
    cxt.quest.param.helpers = CreateCombatParty( party, cxt.quest:GetRank(), cxt.location )
    cxt:TalkTo(cxt.quest.param.helpers[1])
    cxt:Dialog("DIALOG_INTRO")

    local fight_allies = {}
    for k,v in ipairs(cxt.quest.param.helpers) do
        if k > 1 then
            table.insert(fight_allies, v)
        end
    end

    cxt:RunLoop(function()
        cxt:Opt("OPT_CONVINCE_HELP")
            :Negotiation{
                flags = NEGOTIATION_FLAGS.ALLY_GRAB | NEGOTIATION_FLAGS.NO_CORE_RESOLVE,
                fight_allies = fight_allies,
                reason_txt = cxt:GetLocString("NEGOTIATION_REASON")
            }
                :OnFailure()
                    :Dialog("DIALOG_FAIL_NEGOTIATION")
                :OnSuccess()
                    :Dialog("DIALOG_SUCCESS_NEGOTIATION")
                    :Fn(function(cxt, minigame) 
                        
                        cxt.quest.param.fight_allies = {}
                        for i, modifier in minigame:GetPlayerNegotiator():Modifiers() do
                            if modifier.id == "FIGHT_ALLY_WON" and modifier.ally_agent then
                                table.insert( cxt.quest.param.fight_allies, modifier.ally_agent )
                            end
                        end
                        cxt:Opt("OPT_ATTACK_THE_BOSS")
                            :Dialog("DIALOG_ATTACK_BOSS_TEAM")
                            :GoTo("STATE_DO_FIGHT")
                    end)

        cxt:Opt("OPT_ATTACK_THE_BOSS")
            :Dialog("DIALOG_ATTACK_BOSS_ALONE")
            :GoTo("STATE_DO_FIGHT")
    end)
end

QDEF:AddConvo("test")
    :ConfrontState("INTRO")
        :Fn(function(cxt)
            cxt.quest:SetRank(1)
            TheGame:GetGameState():SetDifficulty(3)
            cxt.quest.param.seed = 0
            cxt.encounter:DoLocationTransition(  cxt.quest:SpawnTempLocation("FLEAD_STAGING_SITE") )
            cxt:GoTo("STATE_INTRO") 
        end)

    :State("STATE_INTRO")
        :Loc{
            DIALOG_INTRO = [[* This is a balancing test for the day 2 alternate boss of Rook's story. The audio and visuals are not complete, and there are no story elements.]],
            OPT_SET_SEED = "Enter seed",
            OPT_USE_RANDOM_SEED = "Use a random seed",
            DIALOG_BAD_SEED = [[That is not a valid seed (seeds are numbers)]],
            OPT_SET_PRESITGE = "Set a new prestige",
            DIALOG_NEW_PRESTIGE = "Prestige set to: {1}",
            DIALOG_BAD_PRESTIGE = [[That is not a valid prestige. (prestige must be a number between 0 and 11)]]
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):SetPercent(1)
                TheGame:GetGameState():GetCaravan():SetResolvePercent(1)
                cxt.enc:GetScreen():ClearHistory()
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_USE_RANDOM_SEED")
                :Fn(function() 
                    cxt.quest.param.seed = os.time()
                    cxt:GoTo("STATE_PRE_FIGHT")
                end)
            cxt:Opt("OPT_SET_PRESITGE")
                :Fn(function(cxt) 
                    UIHelpers.EditString( 
                        "Enter a prestige", "", "", 
                        function( val )
                            val = val and tonumber(val)
                            if val then
                                cxt.quest.param.prestige = val
                            end
                            cxt.enc:ResumeEncounter(val)
                         end )
                    local prestige = cxt.enc:YieldEncounter()
                    if prestige and prestige < 12 then
                        cxt.quest.param.prestige = prestige
                        SetNewPrestige(prestige)
                        cxt:Dialog("DIALOG_NEW_PRESTIGE", prestige)
                    else
                        cxt:Dialog("DIALOG_BAD_PRESTIGE")
                    end
                end)

            cxt:Opt("OPT_SET_SEED")
                :Fn(function(cxt) 
                    UIHelpers.EditString( 
                        "Enter a seed", "", "", 
                        function( val )
                            val = val and tonumber(val)
                            if val then
                                cxt.quest.param.seed = val
                            end
                            cxt.enc:ResumeEncounter(val)
                         end )
                    local seed = cxt.enc:YieldEncounter()
                    if seed then
                        cxt.quest.param.seed = seed
                        cxt:GoTo("STATE_PRE_FIGHT")
                    else
                        cxt:Dialog("DIALOG_BAD_SEED")
                    end
                end)

        end)

    :State("STATE_PRE_FIGHT")
        :Loc{
            DIALOG_INTRO = [[
                * Random Seed: {1}
            ]],
            DIALOG_DECK = [[ * Test Deck Selected: #{1}]],
            OPT_REROLL = "Try a new seed",
            OPT_USE = "Use this seed",
            DIALOG_SOCIAL = [[
                * <#BONUS>Social Boons: {1#graft_list}</>
                * <#PENALTY>Social Banes: {2#graft_list}</>
            ]],
        }
        :Fn(function(cxt)
            local DECKS = require "content/quests/experiments/rook_game_won_decks"
            math.randomseed(cxt.quest.param.seed)
            local deck_idx = math.random(#DECKS)
            local deck = DECKS[deck_idx]
            TheGame:GetGameState():SetDecks(deck)


            cxt:TalkTo(cxt.quest.param.boss_agent_id)
            cxt:Dialog("DIALOG_INTRO", cxt.quest.param.seed)
            cxt:Dialog("DIALOG_DECK", deck_idx)
            
            local negotiation, battle, boons, banes = {}, {}, {}, {}
            for k,v in ipairs( cxt.player.graft_owner.grafts) do
                if v:GetType() == GRAFT_TYPE.SOCIAL then 
                    if v:GetDef().is_boon then
                        table.insert(boons, v)
                    elseif v:GetDef().is_bane then
                        table.insert(banes, v)
                    end
                elseif v:GetType() == GRAFT_TYPE.COMBAT then 
                    table.insert(battle, v)
                elseif v:GetType() == GRAFT_TYPE.NEGOTIATION then 
                    table.insert(negotiation, v)
                end

            end

            table.sort(boons, function(a,b) return a:GetName() < b:GetName() end)
            table.sort(banes, function(a,b) return a:GetName() < b:GetName() end)
            cxt:Dialog("DIALOG_SOCIAL", boons, banes)

        
            cxt:RunLoopingFn(function() 
                        cxt:Opt("OPT_USE")
                            :Fn(function() 
                                cxt:GoTo("STATE_DO_FIGHT_OR_NEGOTIATION")
                            end)        
                        cxt:Opt("OPT_REROLL")
                            :Fn(function() 
                                cxt.enc:GetScreen():ClearHistory()
                                cxt.quest.param.seed = os.time() + math.random(10000)
                                cxt:GoTo("STATE_PRE_FIGHT") 
                            end)        
                    end)                                    
        end)

    :State("STATE_DO_FIGHT_OR_NEGOTIATION")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !fight
                agent:
                    !right
                    !fight
            ]],
            OPT_FIGHT = "Fight!",
            DIALOG_DONE_FIGHT = [[
                left:
                    !happy
                right:
                    !greeting
                * Thanks for testing! Now would be a great time to submit some feedback about that fight! (Your seed was: {1})
            ]],
            OPT_RETRY = "Retry this exact fight",
            OPT_AGAIN = "Start over",
            DIALOG_AGAIN = [[
                player:
                    !exit
                agent:
                    !exit
            ]],
            OPT_FEEDBACK = "Tell us how that went!",

            OPT_ATTACK_THE_BOSS = "Attack the Boss",
            NEGOTIATION_REASON = "Collect allies and then play the ending card to start the fight!",
            OPT_CONVINCE_HELP = "Convince {agent} to spare some fighters",
            DIALOG_SUCCESS_NEGOTIATION = [[
                agent:
                    Fine, I can spare some fighters.
                    But <b>I'm</> not going down there.
            ]],
            DIALOG_FAIL_NEGOTIATION = [[
                agent:
                    No way. Our job is up here. You go do yours.
            ]],

            DIALOG_ATTACK_BOSS_ALONE = [[
                player:
                    !exit
                agent:
                    !cruel
                    Good luck!
            ]],

            DIALOG_ATTACK_BOSS_TEAM = [[
                * You signal to the 'volunteers', and head into battle.
                player:
                    !exit
                agent:
                    Good luck!
            ]],
        }
        :Fn(PRE_FIGHT)

    :State("STATE_DO_FIGHT")
        :Loc{
            OPT_FIGHT = "Fight!",
            DIALOG_DONE_FIGHT = [[
                left:
                    !happy
                right:
                    !greeting
                * Thanks for testing! Now would be a great time to submit some feedback about that fight! (Your seed was: {1})
            ]],
            OPT_RETRY = "Retry this exact fight",
            OPT_AGAIN = "Start over",
            DIALOG_AGAIN = [[
                player:
                    !exit
                agent:
                    !exit
            ]],
            OPT_FEEDBACK = "Tell us how that went!",
            OPT_START_NEGOTIATION = "Search for a weakness",
        }
        :Fn(function(cxt) 
            if cxt.quest.param.boss then
                cxt.quest.param.boss:Kill()
            end
            local boss = TheGame:GetGameState():AddAgent(Agent(math.random() < .5 and "AUTOMECH_BOSS" or "FLEAD_QUEEN"))
            cxt.quest.param.boss = boss
            cxt:TalkTo(boss)
            cxt:Opt("OPT_FIGHT")
                :Battle{ 
                    allies = cxt.quest.param.fight_allies,
                    flags = BATTLE_FLAGS.BOSS_FIGHT,
                    IS_EXPERIMENT = true, 

                    on_experiment_done = function( cxt, battle )
                        local player_team = {}
                        local enemy_team = {}
                        for k,v in ipairs(battle:GetScenario().teams[TEAM.BLUE]) do
                            table.insert(player_team, v:GetContentID())
                        end

                        for k,v in ipairs(battle:GetScenario().teams[TEAM.RED]) do
                            table.insert(enemy_team, v:GetContentID())
                        end

                        local player_state = TheGame:GetGameState():GetPlayerState()
                        local json_t = {
                            PLAYER_DATA = player_state,
                            BATTLE_DATA = {
                                TURNS = battle.turns,
                                CONTENT_ID = battle:GetScenario().content_id,
                                HEALTH_DELTA = battle:GetPlayerFighter():GetHealth() - battle:GetPlayerFighter():GetMaxHealth(), -- Player always start with full HP
                                PLAYER_TEAM = player_team,
                                ENEMY_TEAM = enemy_team,
                            },
                            EXPERIMENT_DATA = {
                                ENEMY_HEALTH_PERCENTAGE = battle:GetEnemyTeam():Primary():GetHealthPercent() * 100,
                            }
                        }
                        local evt_type = (battle:GetBattleResult() == BATTLE_RESULT.WON) and "BOSSBIT_EXPERIMENT_BATTLE_WON" or "BOSSBIT_EXPERIMENT_BATTLE_LOST"
                        SendMetricsData(evt_type, json_t )
                    end,
                }
                
                
                :Fn(function() 
                    cxt:Dialog("DIALOG_DONE_FIGHT", cxt.quest.param.seed)

                    cxt:RunLoopingFn(function() 
                        cxt:Opt("OPT_FEEDBACK")
                            :Fn(function() TheGame:StartFeedback() end)
                        cxt:Opt("OPT_AGAIN")
                            :Fn(function(cxt)
                                if cxt.quest.param.boss and cxt.quest.param.boss:IsDead() then
                                    cxt.quest.param.boss:Kill()
                                end
                            end)
                            :Dialog("DIALOG_AGAIN")
                            :GoTo("STATE_INTRO")
                        cxt:Opt("OPT_RETRY")
                            :GoTo("STATE_DO_FIGHT_OR_NEGOTIATION")
                    end)
                end)
        end)