local function SetNewPrestige( prestige )
    TheGame:GetGameState():SetAdvancementLevel(prestige)
    local agent = TheGame:GetGameState():GetPlayerAgent()
    agent:UpdateMaxHealth()
    local max = agent.def.max_resolve
    local advancement_mod = GetAdvancementModifier( ADVANCEMENT_OPTION.LOWER_MAX_RESOLVE )
    if advancement_mod then
        max = math.round(max * advancement_mod)
    end
    TheGame:GetGameState():GetCaravan():SetMaxResolve( max )
end

local QDEF = QuestDef.Define{
        title = "Rook Day 3 Boss Test",
        qtype = QTYPE.EXPERIMENT,
        disabled = true,
        icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
        rank = 1,
        act_filter = "ROOK",
        -- dev_only = true,
    }
    :AddLocationDefs{

        AUCTION_HOUSE =
        {
            name = "Auction House",
            desc = "The room where the auction is taking place.",
            show_agents = true,
            indoors = true,
            no_exit = true,
            plax = "EXT_NadanFight",
        }
    }
    :AddCastByAlias{
        cast_id = "arint",
        alias = "SPARK_SECOND",
        when = QWHEN.MANUAL,
    }
    :AddCastByAlias{
        cast_id = "flekfis",
        alias = "RISE_SECOND",
        when = QWHEN.MANUAL,
    }
    :AddObjective{
        id = "test",
        title = "Fight one of the Lieutenants, and provide feedback!",
        state = QSTATUS.ACTIVE,
    }

QDEF:AddConvo("test")
    :ConfrontState("INTRO")
        :Fn(function(cxt)
            cxt.quest:SetRank(1)
            TheGame:GetGameState():SetDifficulty(1)
            cxt.quest.param.seed = 0
            cxt.encounter:DoLocationTransition(  cxt.quest:SpawnTempLocation("AUCTION_HOUSE") )
            cxt:GoTo("STATE_INTRO") 
        end)

    :State("STATE_INTRO")
        :Loc{
            DIALOG_INTRO = [[* This is a balancing test for the day 3 boss of Rook's story. The experiment randomly chooses one of the two bosses. The audio and visuals are not complete, and there are no story elements.]],
            OPT_SET_SEED = "Enter seed",
            OPT_USE_RANDOM_SEED = "Use a random seed",
            DIALOG_BAD_SEED = [[That is not a valid seed (seeds are numbers)]],
            OPT_SET_PRESITGE = "Set a new prestige",
            DIALOG_NEW_PRESTIGE = "Prestige set to: {1}",
            DIALOG_BAD_PRESTIGE = [[That is not a valid prestige. (prestige must be a number between 0 and 11)]]
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.enc:GetScreen():ClearHistory()
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_USE_RANDOM_SEED")
                :Fn(function() 
                    cxt.quest.param.seed = os.time()
                    cxt:GoTo("STATE_PRE_FIGHT")
                end)
            cxt:Opt("OPT_SET_PRESITGE")
                :Fn(function(cxt) 
                    UIHelpers.EditString( 
                        "Enter a prestige", "", "", 
                        function( val )
                            val = val and tonumber(val)
                            if val then
                                cxt.quest.param.prestige = val
                            end
                            cxt.enc:ResumeEncounter(val)
                         end )
                    local prestige = cxt.enc:YieldEncounter()
                    if prestige and prestige < 12 then
                        cxt.quest.param.prestige = prestige
                        SetNewPrestige(prestige)
                        cxt:Dialog("DIALOG_NEW_PRESTIGE", prestige)
                    else
                        cxt:Dialog("DIALOG_BAD_PRESTIGE")
                    end
                end)

            cxt:Opt("OPT_SET_SEED")
                :Fn(function(cxt) 
                    UIHelpers.EditString( 
                        "Enter a seed", "", "", 
                        function( val )
                            val = val and tonumber(val)
                            if val then
                                cxt.quest.param.seed = val
                            end
                            cxt.enc:ResumeEncounter(val)
                         end )
                    local seed = cxt.enc:YieldEncounter()
                    if seed then
                        cxt.quest.param.seed = seed
                        cxt:GoTo("STATE_PRE_FIGHT")
                    else
                        cxt:Dialog("DIALOG_BAD_SEED")
                    end
                end)

        end)

    :State("STATE_PRE_FIGHT")
        :Loc{
            DIALOG_INTRO = [[
                * Random Seed: {1}
            ]],
            DIALOG_DECK = [[ * Test Deck Selected: #{1}]],
            OPT_REROLL = "Try a new seed",
            OPT_USE = "Use this seed",
            DIALOG_SOCIAL = [[
                * <#BONUS>Social Boons: {1#graft_list}</>
                * <#PENALTY>Social Banes: {2#graft_list}</>
            ]],
        }
        :Fn(function(cxt)
            local DECKS = require "content/quests/experiments/rook_game_won_decks"
            math.randomseed(cxt.quest.param.seed)
            local deck_idx = math.random(#DECKS)
            local deck = DECKS[deck_idx]
            TheGame:GetGameState():SetDecks(deck)


            cxt:TalkTo(cxt.quest.param.boss_agent_id)
            cxt:Dialog("DIALOG_INTRO", cxt.quest.param.seed)
            cxt:Dialog("DIALOG_DECK", deck_idx)
            
            local negotiation, battle, boons, banes = {}, {}, {}, {}
            for k,v in ipairs( cxt.player.graft_owner.grafts) do
                if v:GetType() == GRAFT_TYPE.SOCIAL then 
                    if v:GetDef().is_boon then
                        table.insert(boons, v)
                    elseif v:GetDef().is_bane then
                        table.insert(banes, v)
                    end
                elseif v:GetType() == GRAFT_TYPE.COMBAT then 
                    table.insert(battle, v)
                elseif v:GetType() == GRAFT_TYPE.NEGOTIATION then 
                    table.insert(negotiation, v)
                end

            end

            table.sort(boons, function(a,b) return a:GetName() < b:GetName() end)
            table.sort(banes, function(a,b) return a:GetName() < b:GetName() end)
            cxt:Dialog("DIALOG_SOCIAL", boons, banes)

        
            cxt:RunLoopingFn(function() 
                        cxt:Opt("OPT_USE")
                            :Fn(function() 
                                cxt:GoTo("STATE_PICK_BOSS")
                            end)        
                        cxt:Opt("OPT_REROLL")
                            :Fn(function() 
                                cxt.enc:GetScreen():ClearHistory()
                                cxt.quest.param.seed = os.time() + math.random(10000)
                                cxt:GoTo("STATE_PRE_FIGHT") 
                            end)        
                    end)                                    
        end)

    :State("STATE_PICK_BOSS")
        :Loc{
            DIALOG_INTRO = [[
                * Would you like to use a specific boss or play against a random one?
            ]],
            OPT_PICK_RANDOM = "Fight a random boss",
            OPT_PICK_ARINT = "Fight Arint",
            OPT_PICK_FLEKFIS = "Fight Flekfis",
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_PICK_RANDOM")
                :Fn(function(cxt)
                    cxt.quest.param.boss_agent_id = math.random() < .5 and "arint" or "flekfis"
                    cxt.quest:AssignCastMember(cxt.quest.param.boss_agent_id)
                    cxt:GoTo("STATE_DO_FIGHT_OR_NEGOTIATION")
                end)

            cxt:Opt("OPT_PICK_ARINT")
                :Fn(function(cxt)
                    cxt.quest.param.boss_agent_id = "arint"
                    cxt.quest:AssignCastMember(cxt.quest.param.boss_agent_id)
                    cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("arint"))
                    cxt:GoTo("STATE_DO_FIGHT_OR_NEGOTIATION")
                end)

            cxt:Opt("OPT_PICK_FLEKFIS")
                :Fn(function(cxt)
                    cxt.quest.param.boss_agent_id = "flekfis"
                    cxt.quest:AssignCastMember(cxt.quest.param.boss_agent_id)
                    cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("flekfis"))
                    cxt:GoTo("STATE_DO_FIGHT_OR_NEGOTIATION")
                end)
        end)

    :State("STATE_DO_FIGHT_OR_NEGOTIATION")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !fight
                agent:
                    !right
                    !fight
            ]],
            OPT_FIGHT = "Fight!",
            DIALOG_DONE_FIGHT = [[
                left:
                    !happy
                right:
                    !greeting
                * Thanks for testing! Now would be a great time to submit some feedback about that fight! (Your seed was: {1})
            ]],
            OPT_RETRY = "Retry this exact fight",
            OPT_AGAIN = "Start over",
            DIALOG_AGAIN = [[
                player:
                    !exit
                agent:
                    !exit
            ]],
            OPT_FEEDBACK = "Tell us how that went!",
            OPT_START_NEGOTIATION = "Search for a weakness",
            DIALOG_WON_CARDS = "* You won the negotiation and discovered secret weaknesses.",
            DIALOG_WON_NOTHING = "* You won the negotiation but were unable to discover any weaknesses.",
            DIALOG_LOSE_NEGOTIATION = "* You lost the negotiation.",
            NEGOTIATION_REASON = "Search for weaknesses",
        }
        :Fn(function(cxt) 
            cxt:TalkTo(cxt.quest.param.boss_agent_id)
            TheGame:GetGameState():GetCaravan():SetResolvePercent(1)
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest.param.cards_won = {}
            cxt:Opt("OPT_START_NEGOTIATION")
                -- :Dialog("DIALOG_START_NEGOTIATION")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.NO_CORE_RESOLVE,
                    reason_fn = function(minigame)
                        return loc.format(cxt:GetLocString("NEGOTIATION_REASON") )
                    end,
                    on_start_negotiation = function(minigame)
                        minigame.opponent_negotiator:FindCoreArgument():SetCards( cxt.enc:GetPrimaryCast().def.weakness_cards, cxt.quest.param.cards_won )
                        minigame:DealCard( Negotiation.Card("start_battle", minigame:GetPlayerNegotiator().agent), minigame:GetDrawDeck())
                    end,
                    on_success = function(cxt, minigame) 
                        cxt:Dialog(cxt.quest.param.cards_won and "DIALOG_WON_CARDS", cxt.quest.param.cards_won or "DIALOG_WON_NOTHING")
                        cxt:GoTo("STATE_DO_FIGHT")
                    end,

                    on_fail = function() 
                        cxt.quest.param.neg_lost = true
                        cxt:Dialog("DIALOG_LOSE_NEGOTIATION")
                        cxt:GoTo("STATE_DO_FIGHT")
                    end,
                }

            cxt:Opt("OPT_FIGHT")
                :Battle{
                    flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE,
                    IS_EXPERIMENT = true,
                    advantage = cxt.quest.param.neg_lost and TEAM.RED or nil,
                    on_player_turn_start = function( battle )
                        if cxt.quest.param.cards_won and not battle.cards_given then
                            local cards = {}
                            for i,id in ipairs(cxt.quest.param.cards_won) do
                                table.insert(cards, Battle.Card(id, battle:GetPlayerFighter()))
                            end
                            battle:DealCards(cards, battle:GetDrawDeck())
                            battle.cards_given = true
                        end
                    end,
                    on_experiment_done = function( cxt, battle )
                        local player_team = {}
                        local enemy_team = {}
                        for k,v in ipairs(battle:GetScenario().teams[TEAM.BLUE]) do
                            table.insert(player_team, v:GetContentID())
                        end

                        for k,v in ipairs(battle:GetScenario().teams[TEAM.RED]) do
                            table.insert(enemy_team, v:GetContentID())
                        end

                        local player_state = TheGame:GetGameState():GetPlayerState()
                        local json_t = {
                            PLAYER_DATA = player_state,
                            BATTLE_DATA = {
                                TURNS = battle.turns,
                                CONTENT_ID = battle:GetScenario().content_id,
                                HEALTH_DELTA = battle:GetPlayerFighter():GetHealth() - battle:GetPlayerFighter():GetMaxHealth(), -- Player always start with full HP
                                PLAYER_TEAM = player_team,
                                ENEMY_TEAM = enemy_team,
                            },
                            EXPERIMENT_DATA = {
                                ENEMY_HEALTH_PERCENTAGE = battle:GetEnemyTeam():Primary():GetHealthPercent() * 100,
                            }
                        }

                        local evt_type = (battle:GetBattleResult() == BATTLE_RESULT.WON) and "LIEUTENANT_EXPERIMENT_BATTLE_WON" or "LIEUTENANT_EXPERIMENT_BATTLE_LOST"
                        SendMetricsData(evt_type, json_t)
                    end,
                }
                
                
                :Fn(function() 
                    cxt:Dialog("DIALOG_DONE_FIGHT", cxt.quest.param.seed)

                    cxt:RunLoopingFn(function() 
                        cxt:Opt("OPT_FEEDBACK")
                            :Fn(function() TheGame:StartFeedback() end)
                        cxt:Opt("OPT_AGAIN")
                            :Fn(function(cxt)
                                cxt.quest:UnassignCastMember(cxt.quest.param.boss_agent_id)
                            end)
                            :Dialog("DIALOG_AGAIN")
                            :GoTo("STATE_INTRO")
                        cxt:Opt("OPT_RETRY")
                            :GoTo("STATE_DO_FIGHT_OR_NEGOTIATION")
                    end)
                end)
        end)

        :State("STATE_DO_FIGHT")
        :Loc{
            OPT_FIGHT = "Fight!",
            DIALOG_DONE_FIGHT = [[
                left:
                    !happy
                right:
                    !greeting
                * Thanks for testing! Now would be a great time to submit some feedback about that fight! (Your seed was: {1})
            ]],
            OPT_RETRY = "Retry this exact fight",
            OPT_AGAIN = "Start over",
            DIALOG_AGAIN = [[
                player:
                    !exit
                agent:
                    !exit
            ]],
            OPT_FEEDBACK = "Tell us how that went!",
            OPT_START_NEGOTIATION = "Search for a weakness",
        }
        :Fn(function(cxt) 

            cxt:Opt("OPT_FIGHT")
                :Battle{
                    flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE,
                    IS_EXPERIMENT = true,
                    advantage = cxt.quest.param.neg_lost and TEAM.RED or nil,
                    on_player_turn_start = function( battle )
                        if cxt.quest.param.cards_won and not battle.cards_given then
                            local cards = {}
                            for i,id in ipairs(cxt.quest.param.cards_won) do
                                table.insert(cards, Battle.Card(id, battle:GetPlayerFighter()))
                            end
                            battle:DealCards(cards, battle:GetDrawDeck())
                            battle.cards_given = true
                        end
                    end,
                    on_experiment_done = function( cxt, battle )
                        local player_team = {}
                        local enemy_team = {}
                        for k,v in ipairs(battle:GetScenario().teams[TEAM.BLUE]) do
                            table.insert(player_team, v:GetContentID())
                        end

                        for k,v in ipairs(battle:GetScenario().teams[TEAM.RED]) do
                            table.insert(enemy_team, v:GetContentID())
                        end

                        local player_state = TheGame:GetGameState():GetPlayerState()
                        local json_t = {
                            PLAYER_DATA = player_state,
                            BATTLE_DATA = {
                                TURNS = battle.turns,
                                CONTENT_ID = battle:GetScenario().content_id,
                                HEALTH_DELTA = battle:GetPlayerFighter():GetHealth() - battle:GetPlayerFighter():GetMaxHealth(), -- Player always start with full HP
                                PLAYER_TEAM = player_team,
                                ENEMY_TEAM = enemy_team,
                            },
                            EXPERIMENT_DATA = {
                                ENEMY_HEALTH_PERCENTAGE = battle:GetEnemyTeam():Primary():GetHealthPercent() * 100,
                            }
                        }
                        local evt_type = (battle:GetBattleResult() == BATTLE_RESULT.WON) and "LIEUTENANT_EXPERIMENT_BATTLE_WON" or "LIEUTENANT_EXPERIMENT_BATTLE_LOST"
                        SendMetricsData(evt_type, json_t )
                    end,
                }
                
                
                :Fn(function() 
                    cxt:Dialog("DIALOG_DONE_FIGHT", cxt.quest.param.seed)

                    cxt:RunLoopingFn(function() 
                        cxt:Opt("OPT_FEEDBACK")
                            :Fn(function() TheGame:StartFeedback() end)
                        cxt:Opt("OPT_AGAIN")
                            :Fn(function(cxt)
                                cxt.quest:UnassignCastMember(cxt.quest.param.boss_agent_id)
                            end)
                            :Dialog("DIALOG_AGAIN")
                            :GoTo("STATE_INTRO")
                        cxt:Opt("OPT_RETRY")
                            :GoTo("STATE_DO_FIGHT_OR_NEGOTIATION")
                    end)
                end)
        end)