
local BAR_DRINK_COST = 20
local BAR_FOOD_COST = 30
local HIRE_COST = 60
local WAVE_LENGTH = 5
local MAX_STAGE = 20

local function GetRewardForStage(stage)
    local reward = 15 + math.floor(stage/2)*5
    return reward
end


local QDEF = QuestDef.Define{
        title = "Smith's Bar Fight",
        qtype = QTYPE.EXPERIMENT,
        --icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
        --rank = 1,
        act_filter = "SMITH",
        --BRANCH = "dev",
        --dev_only = true,
        disabled = true,
    }

    :AddLocationDefs{
        SMITH_BAR = {
            name = "Sweet Moreef's Bar",
            plax = "INT_SMITHBAR",
            show_agents = true,
            indoors = true,
            work = 
            {
                bartender = CreateMerchantJob( PHASE_MASK_ALL, "Bartender", CHARACTER_ROLES.PROPRIETOR, "BARTENDER", "BARTENDER_ITEMS"),    
            },
        },
    }

    :AddCastByAlias{
        cast_id = "bartender",
        alias = "SWEET_MOREEF",
    }

    
    :AddObjective{
        
        id = "test",
        title = "You are on round {1}",
        title_fn = function(quest, str)
            return loc.format(str, (quest.param.stage or 0)+1)
        end,

        desc = "Current difficulty scale: {1}\nCurrent Payout:{2}x{3}={4}",
        desc_fn = function(quest, str) 
            return loc.format(str, TheGame:GetGameState():GetCurrentBaseDifficulty(),quest.param.current_payout, quest.param.current_multiplier, quest.param.current_payout * quest.param.current_multiplier)
        end,
        state = QSTATUS.ACTIVE,
        on_activate = function(quest) 
            local location = quest:SpawnTempLocation("SMITH_BAR")
            AgentUtil.TakeJob(quest:GetCastMember("bartender"), location, "bartender")
            TheGame:GetGameState():GetCaravan():MoveToLocation(location)
        end,
    }

local function AddPatron(cxt, id, ...)
    local agent = TheGame:GetGameState():AddSkinnedAgent( id )
    agent:GetBrain():SendToPatronize(cxt.location)
    return agent
end


QDEF:AddConvo("test")

    :ConfrontState("INTRO")
        :Fn(function(cxt)
            cxt.quest:SetRank(1)
            TheGame:GetGameState():SetDifficulty(1)
            TheGame:GetGameState():SetActProgress(1)
            UIHelpers.PassTime(DAY_PHASE.NIGHT)
            cxt.quest.param.stage = 0
            cxt.quest.param.current_multiplier = 1  
            cxt.quest.param.current_payout = 0

            cxt.quest.param.helper_timeouts = {}
            

            cxt:GoTo("STATE_INTRO") 
        end)
        

    :State("STATE_INTRO")
        :Loc{
            DIALOG_DRAFT = [[
                * After a long day of grifting, you reflect upon your day. You learned a thing or two, you think.        
            ]],
            OPT_DO_DRAFT = "Draft some memories",
            DIALOG_INTRO = [[
                * So there you were, minding your own business, when all of a sudden...
                agent:
                    !right
                    Hey! Rich kid!
                player:
                    !left
                    Are you talking to me?
                agent:
                    You're the only one here with more than two shills to rub together, ain't ya?
                player:
                    It's the jacket, right? It always give me away.
                agent:
                    Yeah, this ain't a bar for tourists.
                    !angry
                    Get. Out.
                * Here we go again...
            ]],
            OPT_DEFUSE = "Talk {agent} down",
            DIALOG_DEFUSE_THE_SITUATION = [[
                player:
                    Now listen, friend-
            ]],
            DIALOG_DEFUSE_FAIL = [[
                agent:
                    Blah blah blah. 
                    !fight
                    Shut your mouth and put up your dukes.
            ]],
            DIALOG_DEFUSE_SUCCESS = [[
                agent:
                    Fine.
                    Maybe I'll just leave.
                    !exit
            ]],

            OPT_DEFEND = "Teach {agent} some manners",
            DIALOG_DEFEND = [[
                player:
                    I'll leave when I'm good and ready.
                    !fight
                    You'll leave when they carry you outta here.
            ]],

            DIALOG_WON_FIGHT = [[
                {dead ?
                    * {agent}'s corpse slumps to the ground.
                }
                {not dead?
                    agent:
                        !exit
                    * {agent} makes for the door.
                }
            ]],

            OPT_PAY_OFF = "Pay off {agent}",
            DIALOG_PAY_OFF = [[
                player:
                    You're right. I am rich. You want a piece?
                agent:
                    ...
                    I wouldn't say no.
                player:
                    Here. Take this, and go buy a clue somewhere else.
                    !give
                agent:
                    !take
                    I will. But only because I want to.
                    !exit
            ]]
        }
        :Fn(function(cxt) 
            local goon = AddPatron(cxt, table.arraypick{"BANDIT_GOON", "ADMIRALTY_GOON", "LABORER"})
            
            cxt:Dialog("DIALOG_DRAFT")
            
            cxt:Opt("OPT_DO_DRAFT")
            :Fn(function() 
                
                local function OnDone()
                    cxt.encounter:ResumeEncounter()
                end
                
                for i = 1, 2 do
                    local draft_popup = Screen.DraftChoicePopup()
                    local cards = RewardUtil.GetBattleCards( 1, 3, cxt.player )
                    draft_popup:DraftCards( cxt.player, Battle.Card, cards, OnDone )
                    TheGame:FE():InsertScreen( draft_popup )
                    cxt.enc:YieldEncounter()
                end

                for i = 1, 2 do
                    local draft_popup = Screen.DraftChoicePopup()
                    local cards = RewardUtil.GetNegotiationCards( 1, 3, cxt.player )
                    draft_popup:DraftCards( cxt.player, Negotiation.Card, cards, OnDone )
                    TheGame:FE():InsertScreen( draft_popup )
                    cxt.enc:YieldEncounter()
                end

                
                cxt:TalkTo(goon)
                cxt:Dialog("DIALOG_INTRO")

                cxt:RunLoop(function() 
                    
                    cxt:Opt("OPT_PAY_OFF")
                        :Dialog("DIALOG_PAY_OFF")
                        :DeliverMoney(20)
                        :GoTo("STATE_BARTENDER")

                    cxt:Opt("OPT_DEFUSE")
                        :Negotiation{}
                            :OnSuccess()
                                :Dialog("DIALOG_DEFUSE_SUCCESS")
                                :GoTo("STATE_BARTENDER")
                            
                            :OnFailure()
                                :Dialog("DIALOG_DEFUSE_FAIL")
                                

                    cxt:Opt("OPT_DEFEND")
                        :Dialog("DIALOG_DEFEND")
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE,
                            noncombatants = {cxt.quest:GetCastMember("bartender")}
                        }
                            :OnWin()
                                :Dialog("DIALOG_WON_FIGHT")
                                :GoTo("STATE_BARTENDER")
                end)
            end)
        end)


    :State("STATE_AFTER_FIGHT")
        :Loc{
            DIALOG_HELPER_LEAVE = [[
                agent:
                    !right
                    Alright, I've had enough.
                    See you around, Smith.
                    !exit
            ]],
            DIALOG_BONUS_CHOICE = [[
                * You can talk to {bartender} and get paid, or you can keep facing threats.
                * Your current payout is {1}, and your multiplier is {2}.
                * Skipping the bartender will increase your multiplier to {3}.
            ]],
            OPT_TALK_TO_BARTENDER = "Talk to {bartender}",
            OPT_NEXT_CHALLENGE = "Take on the next challenge",

        }
        :Fn(function(cxt) 

            if cxt.quest.param.stage >= MAX_STAGE then
                cxt:GoTo("STATE_DONE")
                return
            end

            LocationUtil.SendPatronsAway(cxt.location)        

            for k,v in pairs( cxt.quest.param.helper_timeouts ) do
                cxt.quest.param.helper_timeouts[k] = cxt.quest.param.helper_timeouts[k] - 1
            end
            
            for k,v in pairs( cxt.quest.param.helper_timeouts ) do
                if k:IsInPlayerParty() and v <= 0 then
                    cxt:TalkTo(k)
                    cxt:Dialog("DIALOG_HELPER_LEAVE")
                    k:Dismiss()
                    k:MoveToLimbo()
                end
            end


            cxt.quest.param.current_payout = cxt.quest.param.current_payout + GetRewardForStage(cxt.quest.param.stage)
            local new_multiplier = cxt.quest.param.current_multiplier + .15
            cxt:Dialog("DIALOG_BONUS_CHOICE", cxt.quest.param.current_payout, cxt.quest.param.current_multiplier, new_multiplier)

            cxt:Opt("OPT_TALK_TO_BARTENDER")
                :GoTo("STATE_BARTENDER")

            if cxt.quest.param.stage % 3 ~= 0 then
                cxt:Opt("OPT_NEXT_CHALLENGE")
                    :Fn(function() cxt.quest.param.current_multiplier = new_multiplier end)
                    :GoTo("STATE_NEXT_EVENT")
            end
                

        end)

    :State("STATE_BARTENDER")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                {first_time?
                    bartender:
                        !right
                        !crossed
                        Are you causing trouble in my bar again, Smith?
                    player:
                        !thumb
                        Me? Trouble? Never!
                        !happy
                        Maybe a little?
                    bartender:
                        I have enough trouble already tonight. 
                        Waves of it, if you will.
                        Tell you what, I'll pay you to keep the riff-raff out of the bar.
                    player:
                        Sounds like fun. I'm in.
                    bartender:
                        Great. After you deal with each batch of low-lifes, come back here and I'll pay you and patch you up best I can.
                        But the less you bug me, the more I'll pay out. I got dishes to spit-clean, you know?
                }
                {not first_time?
                    bartender:
                        !right
                        You need anything?

                }
            ]],
            OPT_DRINK = "Order a drink",
            DIALOG_DRINK = [[
                agent:
                    !give
                player:
                    !drink
                    Ahhhh.
            ]],

            OPT_FOOD = "Order a snack",
            DIALOG_FOOD = [[
                agent:
                    !give
                player:
                    !take
                    That hits the spot.
            ]],

            OPT_BUY_CARDS = "Buy cards...",

            OPT_DONE = "Back into the fray!",
            DIALOG_DONE = [[
                agent:
                    Looks like someone's coming through the door.
                player:
                    Well. Back to the grindstone.
                    !exit
                agent:
                    !exit
            ]],
            DIALOG_GIVE_MONEY = [[
                player:
                    !left
                bartender:
                    !right
                    Here's your pay.
            ]],
            DIALOG_BARTENDER_ENTER = [[
                * You retreat to the bar.
                player:
                    !left
                    !happy
                bartender:
                    !right
            ]],
            DIALOG_BARTENDER_ENTER_2 = [[
                player:
                    !left
                    !happy
                bartender:
                    !right
            ]],
            DIALOG_GIVE_GRAFT = [[
                bartender:
                    I have something that might help...
            ]],
            OPT_SHOP = "Buy some cards...",
            
        }

        :Fn(function(cxt) 
            LocationUtil.SendPatronsAway(cxt.location)            
            
            cxt:TalkTo("bartender")
            cxt:Dialog("DIALOG_INTRO")
            local gave_graft = false
            local gave_money = false
            cxt:RunLoop(function() 
                local HEAL = 15
                local RESOLVE = 20

                local health_percent = TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):GetPercent()
                local resolve, resolve_max = TheGame:GetGameState():GetCaravan():GetResolve()
                
                if cxt.quest.param.stage >= 1 then
                    --cxt:Dialog("DIALOG_BARTENDER_ENTER")
                    if not gave_money then
                        cxt:Dialog("DIALOG_GIVE_MONEY")
                        cxt.encounter:GainMoney( cxt.quest.param.current_payout * cxt.quest.param.current_multiplier )
                        cxt.quest.param.current_payout = 0
                        gave_money = true
                    end

                    if cxt.quest.param.stage % 3 == 0 and not gave_graft then
                        cxt:Dialog("DIALOG_GIVE_GRAFT")
                        ConvoUtil.GiveGraftChoice(cxt, RewardUtil.GetGrafts(TheGame:GetGameState():GetCurrentBaseDifficulty(), TheGame:GetGameState():GetGraftDraftDetails().count))
                            :Fn(function() gave_graft = true end)

                    end

                end

                cxt:Opt("OPT_DRINK")
                    :Dialog("DIALOG_DRINK")
                    :ReqCondition(resolve < resolve_max, "REQ_MAX_RESOLVE")
                    :DoDrink( BAR_DRINK_COST, RESOLVE )

                cxt:Opt("OPT_FOOD")
                    :ReqCondition(health_percent < 1, "REQ_AT_FULL_HEALTH")
                    :DeliverMoney(BAR_FOOD_COST, { is_shop = true })
                    :Dialog( "DIALOG_FOOD" )
                    :DeltaHealth(HEAL)
                    :GainCards{"bloated"}
                    
                cxt:Opt("OPT_SHOP")
                    :IsHubOption(true)
                    :PreIcon( global_images.shop, UICOLOURS.MONEY_LIGHT )
                    :Fn(function()
                        cxt.enc:WaitOnLine()
                        local screen = Screen.CardShopScreen( cxt:GetAgent(), function() cxt.enc:ResumeEncounter() end )
                        TheGame:FE():InsertScreen( screen )
                        cxt.enc:YieldEncounter()
                    end)

                cxt:Opt("OPT_DONE")
                    :Dialog("DIALOG_DONE")
                    :GoTo("STATE_NEXT_EVENT")

            end)
        end)


    :State("STATE_NEXT_EVENT")
        :Fn(function(cxt) 

            if cxt.quest.param.stage % WAVE_LENGTH == WAVE_LENGTH -1  then
                cxt:GoTo("STATE_BOSS_FIGHT")
            else

                cxt.quest.monster_counter = cxt.quest.monster_counter or 1 + math.random(3)
                if cxt.quest.monster_counter then
                    cxt.quest.monster_counter = cxt.quest.monster_counter - 1
                    if cxt.quest.monster_counter <= 0 then
                        cxt.quest.monster_counter = nil
                        cxt:GoTo("STATE_MONSTER_ROUND")
                        return
                    end
                end
                
                cxt:GoTo("STATE_FIGHT")

            end
        end)

    :State("STATE_DONE")
        :Loc{
            DIALOG_INTRO = [[
                bartender:
                    !right
                player:
                    !left
                    That looks like the last of 'em.
                bartender:
                    Thanks, Smith. Good to know I can count on you. Drinks are on the house for the rest of the night.
                player:  
                    !happy
                    I'm going to make you regret that!
                    !exit
                bartender:
                    !exit
                * Thanks for playing this experiment!

            ]],
            OPT_DONE = "Back to main menu",
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_DONE")
            :Fn(function() 
                cxt:End()
                TheGame:Win()
            end)
        
        end)

    :State("STATE_FIGHT")
        :Quips{
            {
                tags = "enter_the_bar",
                "Let's tear this place apart!",
                "I'm itching for a fight!",
                "WHO DARE STANDS AGAINST ME?",
            },
        }

        :Loc{
            DIALOG_INTRO = [[
                * Trouble walks through the door.
                agent:
                    !right
                    !angry
                    %enter_the_bar
                player:
                    !left
                    !happy
            ]],

            OPT_NEGOTIATION = "Convince {agent} to leave",
            DIALOG_NEGOTIATION = [[
                player:
                    Bar's closed. Keep moving, pal.
            ]],
            DIALOG_NEGOTIATION_WON = [[
                agent:
                    Let's get out of here. Not worth the trouble.
                    !exit
                * The troublemakers leave.
            ]],
            DIALOG_NEGOTIATION_LOST = [[
                agent:
                    !threaten
                    Yeah? You going to make me leave?
            ]],
            OPT_FIGHT = "Fight {agent}",
            DIALOG_FIGHT = [[
                player:
                    !fight
                    Let's rumble!
            ]],
            DIALOG_WON = [[
                {dead?
                    * {bartender}'s staff takes out the trash.
                }
                {not dead?
                    agent:
                        !exit
                    * The troublemakers scatter.
                }
            ]],

            OPT_HIRE = "Hire {agent} to help you",
            DIALOG_HIRE = [[
                player:
                    Why fight when you could make money?
                    I'll pay you to help me out, what do you say?
                agent:
                    Deal. But I won't stick around forever.
                    !exit
            ]]

        }
        :Fn(function(cxt) 
            cxt.quest.param.stage = cxt.quest.param.stage + 1
            cxt.quest:NotifyChanged()


            local generator = table.arraypick{
                "BOGGER_PATROL",
                "ADMIRALTY_PATROL",
                "JAKES_PATROL",
                "BANDIT_PATROL",
                "SPARK_BARON_PATROL",
                "RISE_PATROL",
                "CIVILIAN_PATROL",
            }

            LocationUtil.SendPatronsAway(cxt.location)

            local opfor = CreateCombatParty(generator, TheGame:GetGameState():GetCurrentBaseDifficulty(), cxt.location)
            for k,v in pairs(opfor) do
                v:SetLocationRole(CHARACTER_ROLES.PATRON)
                v:GetAspect("health"):SetPercent(1)
            end
            cxt:TalkTo(opfor[1])
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:RunLoop(function(cxt) 

                cxt:Opt("OPT_HIRE")
                    :PreIcon( global_images.hire )
                    :Dialog("DIALOG_HIRE")
                    :DeliverMoney(cxt:GetAgent():GetCombatStrength()*HIRE_COST)
                    :RecruitMember(PARTY_MEMBER_TYPE.CREW, opfor[1])
                    :Fn(function() 
                        opfor[1]:SetLocationRole(nil)
                        cxt.quest.param.helper_timeouts[opfor[1]] = 1 + math.random(3) 
                    end)
                    :GoTo("STATE_AFTER_FIGHT")

                cxt:Opt("OPT_NEGOTIATION")
                    :Negotiation()
                        :OnSuccess()
                            :Dialog("DIALOG_NEGOTIATION_WON")
                            :GoTo("STATE_AFTER_FIGHT")
                        :OnFailure()
                            :Dialog("DIALOG_NEGOTIATION_LOST")

                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            no_oppo_limit = true,
                            noncombatants = {cxt.quest:GetCastMember("bartender")}
                        }
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :GoTo("STATE_AFTER_FIGHT")
            end)
            
        end)    

    :State("STATE_MONSTER_ROUND")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * Monsters burst through the doors! What kind of bar is this?!
                }
                {not first_time?
                    * The night is thick with monsters, it seems.
                }
                agent:
                    !right
                player:
                    !left
            ]],

            OPT_FIGHT = "Fight {agent}",
            DIALOG_FIGHT = [[
                player:
                    !fight
                    Let's rumble!
            ]],
            DIALOG_WON = [[
                * The monster threat has been dealt with.
            ]],
        }
        :Fn(function(cxt) 
            cxt.quest.param.stage = cxt.quest.param.stage + 1
            cxt.quest:NotifyChanged()

            local generator = table.arraypick{
                "MONSTERS",
                "BOG_MONSTERS",
            }
            LocationUtil.SendPatronsAway(cxt.location)

            local opfor = CreateCombatParty(generator, TheGame:GetGameState():GetCurrentBaseDifficulty(), cxt.location)
            cxt:TalkTo(opfor[1])
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE,
                        no_oppo_limit = true,
                        noncombatants = {cxt.quest:GetCastMember("bartender")}
                    }
                    :OnWin()
                        :Dialog("DIALOG_WON")
                        :Fn(function() 
                            for k,v in pairs(opfor) do
                                if not v:IsDead() then
                                    v:MoveToLimbo()
                                end
                            end
                        end)
                        :GoTo("STATE_AFTER_FIGHT")
            
        end)    

    :State("STATE_BOSS_FIGHT")
        :Quips{
            {
                tags = "enter_the_bar",
                "I hear there's a party going on in here!",
                "Where's Smith?",                
            },
        }

        :Loc{
            DIALOG_INTRO = [[
                * A really tough customer bursts through the door!
                agent:
                    !right
                    !angry
                    %enter_the_bar
                player:
                    !left
                    !fight
            ]],
            OPT_FIGHT = "Fight {agent}",
            DIALOG_FIGHT = [[
                player:
                    !fight
                    I'll take you down, too!
            ]],
            DIALOG_WON = [[
                {dead?
                    * {bartender}'s staff takes out the trash.
                }
                {not dead?
                    agent:
                        !exit
                }
            ]],
            DIALOG_GET_HARDER = [[
                * Things are going to get more difficult now. Be ready.
            ]],
            DIALOG_BONUS = [[
                * {bartender} adds a {1#money} bonus to your payout for dealing with a boss. 
            ]]

        }
        :Fn(function(cxt) 
            cxt.quest.param.stage = cxt.quest.param.stage + 1
            local bosses = {
                {"SPARK_BARON_BOSS", "DRONE_GOON", "HESH_BOSS", "RENTORIAN_BOSS"},
                {"JAKES_ASSASSIN", "JAKES_ASSASSIN2", "FLEAD_QUEEN", "AUTOMECH_BOSS"},
                {"SPARK_SECOND", "RISE_SECOND", "SHROOG", "DRUSK_1"},
                {"MURDER_BAY_BANDIT_CONTACT", "MURDER_BAY_ADMIRALTY_CONTACT"},
            }
            
            local bossdef = table.arraypick(bosses[math.min( TheGame:GetGameState():GetCurrentBaseDifficulty(), #bosses)])
            local boss = TheGame:GetGameState():AddSkinnedAgent(bossdef)
            boss:SetLocationRole(CHARACTER_ROLES.PATRON)
            cxt:TalkTo(boss)
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.NO_FLEE,
                        no_oppo_limit = true,
                        noncombatants = {cxt.quest:GetCastMember("bartender")},
                    }
                    :OnWin()
                        :Fn(function() ConvoUtil.GiveBossRewards(cxt) end)
                        
                        :Dialog("DIALOG_WON")
                        :Fn(function()
                            
                            if TheGame:GetGameState():GetCurrentBaseDifficulty() < 5 then
                                TheGame:GetGameState():SetDifficulty(TheGame:GetGameState():GetCurrentBaseDifficulty() + 1 )
                                TheGame:GetGameState():SetActProgress(TheGame:GetGameState():GetCurrentBaseDifficulty())
                                cxt:Dialog("DIALOG_GET_HARDER")
                            end
                            local bonus = TheGame:GetGameState():GetCurrentBaseDifficulty() * 100
                            cxt.quest.param.current_payout = cxt.quest.param.current_payout + bonus
                            cxt:Dialog("DIALOG_BONUS", bonus)
                            cxt.quest.monster_counter = math.random(WAVE_LENGTH)

                        end)
                        :GoTo("STATE_AFTER_FIGHT")
        end)            