local function SetNewPrestige( prestige )
    TheGame:GetGameState():SetAdvancementLevel(prestige)
    local agent = TheGame:GetGameState():GetPlayerAgent()
    agent:UpdateMaxHealth()
    local max = agent.def.max_resolve
    local advancement_mod = GetAdvancementModifier( ADVANCEMENT_OPTION.LOWER_MAX_RESOLVE )
    if advancement_mod then
        max = math.round(max * advancement_mod)
    end
    TheGame:GetGameState():GetCaravan():SetMaxResolve( max )
end

local QDEF = QuestDef.Define{
        title = "Smith Final Boss Experiment",
        qtype = QTYPE.EXPERIMENT,
        icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
        rank = 5,
        act_filter = "SMITH",
        -- BRANCH = "dev",
        -- dev_only = true,
        -- disabled = true,
    }

    :AddLocationDefs{
        BOG_MINE = {
            name = "Bog Mine",
            plax = "EXT_SMITH_FINALBOSS1",
            indoors = false,
        },
    }
    :AddLocationDefs{
        FINAL_ARENA = {
            name = "Belly of the Beast",
            plax = "INT_SMITH_FINALBOSS2",
            indoors = true,
        },
    }
    :AddObjective{
        id = "test",
        title = "Fight the boss, and provide feedback!",
        state = QSTATUS.ACTIVE,
    }



local function AddVixFight(cxt)
    
    cxt:Opt("OPT_FIGHT_NORMAL_VIX")
        :Dialog("DIALOG_FIGHT_NORMAL_VIX")
        :Battle{
            flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.BOSS_FIGHT,
            IS_EXPERIMENT = true,
            on_experiment_done = function( cxt, battle )
                local player_team = {}
                local enemy_team = {}
                for k,v in ipairs(battle:GetScenario().teams[TEAM.BLUE]) do
                    table.insert(player_team, v:GetContentID())
                end

                for k,v in ipairs(battle:GetScenario().teams[TEAM.RED]) do
                    table.insert(enemy_team, v:GetContentID())
                end

                local player_state = TheGame:GetGameState():GetPlayerState()
                local json_t = {
                    PLAYER_DATA = player_state,
                    BATTLE_DATA = {
                        TURNS = battle.turns,
                        CONTENT_ID = battle:GetScenario().content_id,
                        HEALTH_DELTA = battle:GetPlayerFighter():GetHealth() - battle:GetPlayerFighter():GetMaxHealth(), -- Player always start with full HP
                        PLAYER_TEAM = player_team,
                        ENEMY_TEAM = enemy_team,
                    },
                    EXPERIMENT_DATA = {
                        ENEMY_HEALTH_PERCENTAGE = battle:GetEnemyTeam():Primary():GetHealthPercent() * 100,
                    }
                }
                local evt_type = (battle:GetBattleResult() == BATTLE_RESULT.WON) and "EXPERIMENT_BATTLE_WON" or "EXPERIMENT_BATTLE_LOST"
                SendMetricsData(evt_type, json_t )
                if battle:GetBattleResult() == BATTLE_RESULT.WON then
                    cxt:Dialog("DIALOG_WON_FIGHT")
                    cxt:GoTo("STATE_FIGHT")
                else
                    cxt:GoTo("STATE_DONE")
                end
            end,

        }

end


QDEF:AddConvo("test")

    :ConfrontState("INTRO")
        :Fn(function(cxt)
            cxt.quest:SetRank(5)
            TheGame:GetGameState():SetDifficulty(5)
            cxt.quest.param.seed = 0
            cxt.encounter:DoLocationTransition(  cxt.quest:SpawnTempLocation("BOG_MINE") )
            cxt:GoTo("STATE_INTRO") 
        end)

    :State("STATE_INTRO")
        :Loc{
            DIALOG_INTRO = [[* This is a balancing test for the final boss of Smith's story. The audio and visuals are not complete, and there are no story elements.]],
            OPT_SET_SEED = "Enter seed",
            OPT_USE_RANDOM_SEED = "Use a random seed",
            DIALOG_BAD_SEED = [[That is not a valid seed (seeds are numbers)]],
            OPT_SET_PRESITGE = "Set a new prestige",
            DIALOG_NEW_PRESTIGE = "Prestige set to: {1}",
            DIALOG_BAD_PRESTIGE = [[That is not a valid prestige. (prestige must be a number between 0 and 11)]]
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.quest:SetRank(5)
                TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):SetPercent(1)
                TheGame:GetGameState():GetCaravan():SetResolvePercent(1)
                cxt.enc:GetScreen():ClearHistory()
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_USE_RANDOM_SEED")
                :Fn(function() 
                    cxt.quest.param.seed = os.time()
                    cxt:GoTo("STATE_PRE_FIGHT")
                end)
            cxt:Opt("OPT_SET_PRESITGE")
                :Fn(function(cxt) 
                    UIHelpers.EditString( 
                        "Enter a prestige", "", "", 
                        function( val )
                            val = val and tonumber(val)
                            if val then
                                cxt.quest.param.prestige = val
                            end
                            cxt.enc:ResumeEncounter(val)
                         end )
                    local prestige = cxt.enc:YieldEncounter()
                    if prestige and prestige < 12 then
                        cxt.quest.param.prestige = prestige
                        SetNewPrestige(prestige)
                        cxt:Dialog("DIALOG_NEW_PRESTIGE", prestige)
                    else
                        cxt:Dialog("DIALOG_BAD_PRESTIGE")
                    end
                end)

            cxt:Opt("OPT_SET_SEED")
                :Fn(function(cxt) 
                    UIHelpers.EditString( 
                        "Enter a seed", "", "", 
                        function( val )
                            val = val and tonumber(val)
                            if val then
                                cxt.quest.param.seed = val
                            end
                            cxt.enc:ResumeEncounter(val)
                         end )
                    local seed = cxt.enc:YieldEncounter()
                    if seed then
                        cxt.quest.param.seed = seed
                        cxt:GoTo("STATE_PRE_FIGHT")
                    else
                        cxt:Dialog("DIALOG_BAD_SEED")
                    end
                end)

        end)

    :State("STATE_PRE_FIGHT")
        :Loc{
            DIALOG_INTRO = [[
                * Random Seed: {1}
            ]],
            DIALOG_DECK = [[ * Test Deck Selected: #{1}]],
            OPT_REROLL = "Try a new seed",
            OPT_USE = "Use this seed",
            DIALOG_SOCIAL = [[
                * <#BONUS>Social Boons: {1#graft_list}</>
                * <#PENALTY>Social Banes: {2#graft_list}</>
            ]],
        }
        :Fn(function(cxt)
            local DECKS = require "content/quests/experiments/smith_day_4_decks"
            math.randomseed(cxt.quest.param.seed)
            local deck_idx = math.random(#DECKS)
            local deck = DECKS[deck_idx]
            TheGame:GetGameState():SetDecks(deck)

            cxt:Dialog("DIALOG_INTRO", cxt.quest.param.seed)
            cxt:Dialog("DIALOG_DECK", deck_idx)
            
            local negotiation, battle, boons, banes = {}, {}, {}, {}
            for k,v in ipairs( cxt.player.graft_owner.grafts) do
                if v:GetType() == GRAFT_TYPE.SOCIAL then 
                    if v:GetDef().is_boon then
                        table.insert(boons, v)
                    elseif v:GetDef().is_bane then
                        table.insert(banes, v)
                    end
                elseif v:GetType() == GRAFT_TYPE.COMBAT then 
                    table.insert(battle, v)
                elseif v:GetType() == GRAFT_TYPE.NEGOTIATION then 
                    table.insert(negotiation, v)
                end

            end

            table.sort(boons, function(a,b) return a:GetName() < b:GetName() end)
            table.sort(banes, function(a,b) return a:GetName() < b:GetName() end)
            cxt:Dialog("DIALOG_SOCIAL", boons, banes)

        
            cxt:RunLoopingFn(function() 
                        cxt:Opt("OPT_USE")
                            :Fn(function() 
                                cxt:GoTo("STATE_START")
                            end)        
                        cxt:Opt("OPT_REROLL")
                            :Fn(function() 
                                cxt.enc:GetScreen():ClearHistory()
                                cxt.quest.param.seed = os.time() + math.random(10000)
                                cxt:GoTo("STATE_PRE_FIGHT") 
                            end)        
                    end)                                    
        end)

    :State("STATE_START")
        :Loc{
            DIALOG_START = [[
                player:
                    !left
                agent:
                    !right
            ]],

            OPT_FIGHT_NORMAL_VIX = "Fight Vix",
            DIALOG_FIGHT_NORMAL_VIX = [[
                player:
                    !fight
                agent:
                    !fight
            ]],
            OPT_START_NEGOTIATION = "Reconcile with your brother",
            OPT_SKIP = "Skip To Mutant Vix",
            DIALOG_WON_FIGHT = [[
                * You beat Vix! But now you have to fight MUTANT VIX!
            ]],
            DIALOG_LOST_FIGHT = [[
            ]]
        }
        :Fn(function(cxt) 
            if cxt.quest.param.boss then
                local cur, max = TheGame:GetGameState():GetCaravan():GetResolve()
                TheGame:GetGameState():GetCaravan():DeltaResolve( max-cur )
                TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):SetPercent(0.8)
                cxt.quest.param.boss:Kill()
            end
            local boss = TheGame:GetGameState():AddSkinnedAgent("VIXMALLI")
            cxt.quest.param.boss = boss
            cxt:TalkTo(boss)
            cxt:Dialog("DIALOG_START")

            local RESOLVE = {70, 100, 120, 140}
            local resolve_required = RESOLVE[GetAdvancementModifier( ADVANCEMENT_OPTION.NPC_BOSS_DIFFICULTY )]

            AddVixFight(cxt)

            cxt:Opt("OPT_START_NEGOTIATION")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.NO_IMPATIENCE | NEGOTIATION_FLAGS.NO_LOOT,
                    enemy_resolve_required = resolve_required,
                }
                    :OnSuccess()
                        :GoTo("STATE_FIGHT")
                    :OnFailure()
                        :Fn(function() 
                            AddVixFight(cxt)
                        end)
                        

            cxt:Opt("OPT_SKIP")
                :GoTo("STATE_FIGHT")
            --AddAttackOption(cxt)

        end)

    :State("STATE_FIGHT")
        :Loc{
            DIALOG_WON_NEGOTIATION = [[
                * You reconciled with your brother, time for the battle!
                agent:
                    !exit
                player:
                    !exit
            ]],
            DIALOG_PREFIGHT = [[
                player:
                    !left
                agent:
                    !right
                    !vix_transform
                player:
                    !fight
            ]],
            OPT_START_BATTLE = "Fight Mutant Vix",
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_WON_NEGOTIATION")
            cxt.encounter:DoLocationTransition(  cxt.quest:SpawnTempLocation("FINAL_ARENA") )
            cxt.quest.param.boss:Kill()
            local boss = TheGame:GetGameState():AddSkinnedAgent("MUTANT_VIX")
            cxt.quest.param.boss = boss
            cxt:TalkTo(boss)
            cxt:Dialog("DIALOG_PREFIGHT")
            cxt:Opt("OPT_START_BATTLE")
                :Battle{
                    flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.BOSS_FIGHT,
                    IS_EXPERIMENT = true,
                    on_experiment_done = function( cxt, battle )
                        local player_team = {}
                        local enemy_team = {}
                        for k,v in ipairs(battle:GetScenario().teams[TEAM.BLUE]) do
                            table.insert(player_team, v:GetContentID())
                        end

                        for k,v in ipairs(battle:GetScenario().teams[TEAM.RED]) do
                            table.insert(enemy_team, v:GetContentID())
                        end

                        local player_state = TheGame:GetGameState():GetPlayerState()
                        local json_t = {
                            PLAYER_DATA = player_state,
                            BATTLE_DATA = {
                                TURNS = battle.turns,
                                CONTENT_ID = battle:GetScenario().content_id,
                                HEALTH_DELTA = battle:GetPlayerFighter():GetHealth() - battle:GetPlayerFighter():GetMaxHealth(), -- Player always start with full HP
                                PLAYER_TEAM = player_team,
                                ENEMY_TEAM = enemy_team,
                            },
                            EXPERIMENT_DATA = {
                                ENEMY_HEALTH_PERCENTAGE = battle:GetEnemyTeam():Primary():GetHealthPercent() * 100,
                            }
                        }
                        local evt_type = (battle:GetBattleResult() == BATTLE_RESULT.WON) and "EXPERIMENT_BATTLE_WON" or "EXPERIMENT_BATTLE_LOST"
                        SendMetricsData(evt_type, json_t )
                    end,
                }
                :Fn(function() 
                    cxt:GoTo("STATE_DONE")
                end)
        end)

    :State("STATE_DONE")
        :Loc{
            DIALOG_DONE_FIGHT = [[
                left:
                    !happy
                right:
                    !greeting
                * Thanks for testing! Now would be a great time to submit some feedback about that fight! (Your seed was: {1})
            ]],
            OPT_RETRY = "Retry this exact scenario",
            OPT_AGAIN = "Start over",
            DIALOG_AGAIN = [[
                player:
                    !exit
                agent:
                    !exit
            ]],
            OPT_FEEDBACK = "Tell us how that went!",
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_DONE_FIGHT", cxt.quest.param.seed)

            cxt:RunLoopingFn(function() 
                cxt:Opt("OPT_FEEDBACK")
                    :Fn(function() TheGame:StartFeedback() end)
                cxt:Opt("OPT_AGAIN")
                    :Fn(function(cxt)
                        if cxt.quest.param.boss and cxt.quest.param.boss:IsDead() then
                            cxt.quest.param.boss:Kill()
                        end
                    end)
                    :Dialog("DIALOG_AGAIN")
                    :GoTo("STATE_INTRO")
                cxt:Opt("OPT_RETRY")
                    :GoTo("STATE_START")
            end)
        end)