require "content/boon_services"

local default_texture = engine.asset.Texture("icons/quests/oppo_quiet_meditation_spot.tex")

local function PickBoonForAgent( agent )
--[[    do 
        local service_id = "SPECIAL_NOODLES"
        local service = BOON_SERVICES[ service_id ]
        if service and (service.can_offer == nil or service.can_offer( agent )) then
            return service_id
        end
        return
    end
--]]
    local relationship = agent:GetRelationship()
    local services = agent.social_boons or table.empty
    for i = relationship, RELATIONSHIP.HATED, -1 do
        if services[ i ] then
            local t = shallowcopy( services[ i ] )
            while next(t) ~= nil do
                local service_id = weightedpick( t )
                local service = BOON_SERVICES[ service_id ]
                if service and (service.can_offer == nil or service.can_offer( agent )) then
                    return service_id
                else
                    t[ service_id ] = nil -- Junk this one.
                end
            end
        end
    end
end


local QDEF = QuestDef.Define{
    title = "Meet up with {friend#agent}",
    title_fn = function(quest, str) 
        local service = BOON_SERVICES[quest.param.boon_to_offer]
        if service and service.name then
            return service.name(quest)
        end
        return str
    end,

    icon = function(quest) 
        local service = BOON_SERVICES[quest.param.boon_to_offer]
        return service and service.icon or default_texture
    end,

    desc = "{1}",
    desc_fn = function(quest, str)
        
        local service = BOON_SERVICES[quest.param.boon_to_offer]
        return service and service.desc and loc.format(str, service.desc(quest) or "") or "?!?@#!?@"
    end,

    on_init = function(quest)
        local service = BOON_SERVICES[quest.param.boon_to_offer]
        if service and service.on_init then
            service.on_init(quest)
        end
    end,

    collect_tooltip_items = function(quest, t) 
        local service = BOON_SERVICES[quest.param.boon_to_offer]
        if service and service.collect_tooltip_items then
            service.collect_tooltip_items(quest, t)
        end
    end,

    qtype = QTYPE.OPPORTUNITY,
}

:AddCast{
    cast_id = "friend",
    cast_fn = function(quest, t) 
        local candidates = {}
        local player_location = TheGame:GetGameState():GetPlayerAgent():GetLocation()
        local mem_length = GameState:GetDayPhase() == DAY_PHASE.DAY and 1 or 2
        
        for other_agent, rel in TheGame:GetGameState():GetPlayerAgent().social_connections:Relationships() do
            
            if rel > RELATIONSHIP.NEUTRAL and
                not other_agent:IsRetired() and
                other_agent:GetLocation() ~= player_location and 
                not other_agent:HasActiveQuestMembership() and
                not AgentUtil.HasPlotArmour(other_agent) and
                not other_agent:HasMemory("OFFERED_BOON", mem_length) and
                AgentUtil.CanAct(other_agent) then
                    local boon = PickBoonForAgent(other_agent)
                    if boon then
                        table.insert(candidates, {agent = other_agent, boon = boon} )
                    end
            end
        end

        if #candidates > 0 then
            local boon = table.arraypick(candidates)
            quest.param.boon_to_offer = boon.boon

            table.insert(t, boon.agent)
        end
    end,
    
    events = {
        
        relationship_changed = function(quest)
            if quest:GetCastMember("friend"):GetRelationship() <= RELATIONSHIP.NEUTRAL then
                quest:Fail()
            end
        end,
        agent_retired = function(quest)
            quest:Fail()
        end,
    }


}


:AddObjective{
    id = "visit",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"friend"},
    terminal = true
}


local convo = QDEF:AddConvo("visit", "friend")
    :AttractState("ATTRACT")
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !happy
            * {agent} is happy to see you.
        ]],
    }
    :Fn(function(cxt)
        local service = BOON_SERVICES[cxt.quest.param.boon_to_offer]
        local dest = cxt.quest.param.boon_to_offer
        if not service or (service.can_offer and not service.can_offer( cxt:GetAgent() ) ) then
            dest = "SOCIALIZE"
            service = BOON_SERVICES[ dest ]
            service.on_init( cxt.quest )
        end    
        
        cxt.quest:Complete()
        cxt:Dialog("DIALOG_INTRO")
        cxt:GetAgent():Remember("OFFERED_BOON")
        cxt:GoTo( dest )
    end)

for k,v in pairs(BOON_SERVICES) do
    assert(v.fn)
    local state = convo:State(k):Fn(v.fn)
    if v.txt then
        state:Loc(v.txt)
    end
end

