
AddOpinionEvent("STOPPED_MURDER", 
{
    delta = OPINION_DELTAS.DIMINISH,
    txt = "Kept them from killing someone",
})

local QDEF = QuestDef.Define{
    title = "Friend In Trouble",
    icon = engine.asset.Texture("icons/quests/oppo_liked_in_trouble.tex"),
    qtype = QTYPE.OPPORTUNITY,
    on_fail = function( quest )
        TheGame:GetGameState():LogNotification( NOTIFY.FRIEND_KILLED, quest:GetCastMember("friend")) 
        quest:GetCastAgent("friend"):Kill()
    end
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation(TheGame:GetGameState():GetWorldRegion():GetOutdoorLocationID()))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Rescue {friend}",
    desc = "Your friend {friend} is in trouble. You have a chance to save {friend.himher} before it's too late!",
    mark = {"location"},
    terminal = true
}

:AddCast{
    cast_id = "friend",
    condition = function( agent, quest )
        return agent:GetRelationship() >= RELATIONSHIP.LIKED and agent:GetLocation() ~= TheGame:GetGameState():GetPlayerAgent():GetLocation()
    end,
    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}

:AddCast{
    cast_id = "attacker",
    condition = function( agent, quest )
        return agent:GetRelationship() <= RELATIONSHIP.NEUTRAL and 
                QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) and
                (agent:GetRelationship(quest:GetCastMember("friend")) <= RELATIONSHIP.NEUTRAL or agent:GetRelationship(quest:GetCastMember("friend")) == nil)

    end,
    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
        agent.social_connections:SetRelationship(RELATIONSHIP.HATED, quest:GetCastMember("friend"))
    end,
}


QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            friend:
                !left
                !injured
            attacker:
                !right
                !threaten
            * You find {friend} under {attacker}'s heel, looking one blow away from the worst day of {friend.hisher} life. 
        ]],
        OPT_WAIT = "Wait and see how it plays out",
        DIALOG_WATCH = [[
            friend:
                !exit
            attacker:
                !exit
            * You stay hidden, and sure enough, that last blow falls—and {friend} falls lifeless to the ground. You slink away, telling yourself there was nothing you could've done. 
        ]],
        OPT_INTERVENE = "Save {friend}",
        DIALOG_INTERVENE = [[
            * You make enough of a racket to draw {attacker}'s attention, and {friend} looks up at you with wide eyes. 
            player:
                !left
            friend:
                !right
                !scared
                {player}?
            attacker:
                !right
                !angry_accuse
                This has nothing to do with you, grifter. Stay out of it.
        ]],
        OPT_CONVINCE = "Convince {attacker} to leave",
        DIALOG_CONVINCE = [[
            player:
                I'm afraid it does. See, {friend} is my friend.
            attacker:
                Yeah? You pick 'em poorly, then.
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player: 
                !angry_accuse
                You step off now, and be content with the beating you gave {friend.himher}. Otherwise...
            attacker:
                ... 
                !angry_shrug
                Fine. But remember this, {friend}. Remember I woulda killed you without a second thought. 
                !exit
            * {attacker} stalks off into the shadows as you help {friend} to {friend.hisher} feet.
            friend:
                !right
                !injured
                Thanks, {player}. Y-you actually saved my skin there. 
            player:
                Least I could do. Come on, let's get you somewhere safe. 
                !exit
            friend:
                !exit
            * You take {friend} to shelter and, after dressing {friend.hisher} wounds, get back on the road.
        ]],
        DIALOG_CONVINCE_FAIL = [[
            player:
                !thumb
                So you want {friend}? You need to go through me.
            attacker:
                Sure. Fine. Let's do this. 
                !fight 
        ]],
        OPT_ATTACK = "Attack {attacker}",
        DIALOG_ATTACK = [[
            player:
                !angry
                You're gonna regret messing with any of mine, pal. 
                !fight
        ]],
        DIALOG_POST_FIGHT_DIED = [[
            player:
                !left
            friend:
                !right
                !injured
                Thanks, {player}. Y-you actually saved my skin there. {attacker} has always hated me.
            player:
                {attacker.HeShe} can't hate anything now. Come on, let's get you somewhere safe. 
                !exit
            * You take {friend} to shelter and, after dressing {friend.hisher} wounds, get back on the road.
        ]],
        DIALOG_POST_FIGHT_SURVIVED = [[
            player:
                !left
            attacker:
                !right
                !injured
                Stop! I won't go anywhere near you or {friend} again, all right? My grudge dies here.
            player:
                !angry_accuse
                That's a deal. Break it, and I'll break your neck. Now get lost.
            * {attacker} slinks off. {attacker.HeShe}'ll be lucky if {attacker.heshe} makes it back to town. 
            friend:
                !right
                !injured
                Thanks, {player}. Y-you actually saved my skin there. 
            player:
                Least I could do. Come on, let's get you somewhere safe. 
                !exit
            * You take {friend} to shelter and, after dressing {friend.hisher} wounds, get back on the road.
        ]],
        OPT_DEFEND_SELF = "Defend yourself",
    }
    :SetLooping(true)
    :Fn(function(cxt)
        if cxt:FirstLoop() then
            local attacker = cxt.quest:GetCastAgent("attacker")
            if not attacker or attacker:IsRetired() then
                cxt.quest:Complete()
                cxt.enc:End()
                return
            end
            cxt.quest:Complete()
            cxt.enc:SetPrimaryCast(attacker)
            cxt:Dialog("DIALOG_INTRO")
        end

        if not cxt.quest.param.intervened then
            cxt:Opt("OPT_INTERVENE")
                :PreIcon( global_images.like )
                :Fn(function(cxt)
                    cxt.quest.param.intervened = true
                    cxt:Dialog("DIALOG_INTERVENE")
                end)

            cxt:Opt("OPT_WAIT")
                :PreIcon( global_images.wait )
                :Fn(function(cxt)
                    cxt:Dialog("DIALOG_WATCH")
                    cxt.quest:GetCastAgent("friend"):Kill()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        else
            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    noncombatants = {cxt.quest:GetCastAgent("friend")},
                    on_win = function()
                        local attacker = cxt.quest:GetCastAgent("attacker")
                        if attacker:IsDead() then
                            cxt:Dialog("DIALOG_POST_FIGHT_DIED")
                        else
                            cxt:Dialog("DIALOG_POST_FIGHT_SURVIVED")
                        end
                        cxt.quest:GetCastAgent("friend"):OpinionEvent( OPINION.SAVED_LIFE )
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                }

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    suppressed = {cxt.quest:GetCastAgent("friend")},
                    on_success = function()
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest:GetCastAgent("friend"):OpinionEvent( OPINION.SAVED_LIFE )
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function()
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                        cxt:Opt("OPT_DEFEND_SELF")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                noncombatants = {cxt.quest:GetCastAgent("friend")},
                                on_win = function()
                                    local attacker = cxt.quest:GetCastAgent("attacker")
                                    if attacker:IsDead() then
                                        cxt:Dialog("DIALOG_POST_FIGHT_DIED")
                                    else
                                        cxt:Dialog("DIALOG_POST_FIGHT_SURVIVED")
                                    end
                                    cxt.quest:GetCastAgent("friend"):OpinionEvent( OPINION.SAVED_LIFE )
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end
                            }
                    end
                }
        end
    end)
