local QDEF = QuestDef.Define{
    title = "Aerostat Contact",
    icon = engine.asset.Texture("icons/quests/oppo_aerostat_contact.tex"),
    qtype = QTYPE.OPPORTUNITY,
    precondition = function(quest) 
        local met_old_friend = false
        local player_rel = TheGame:GetGameState():GetPlayerAgent():GetAspect("social_connections")
        for agent, rel in player_rel:Relationships() do
            if player_rel:FindOpinionEvent(agent, {id ="EVENT_OLD_DRINKING_BUDDY_drank_with_them"}) or player_rel:FindOpinionEvent(agent, {id ="EVENT_OLD_DRINKING_BUDDY_paid_back_debt"}) or player_rel:FindOpinionEvent(agent, {id ="EVENT_OLD_DRINKING_BUDDY_defended_them"}) then
                met_old_friend = true
                if player_rel:FindOpinionEvent(agent, quest:GetQuestDef():GetOpinionEvent("offered_coilgun")) then
                    return false
                end
                break
            end
        end

        return met_old_friend
    end,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("ROAD_LOCATION"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Aerostat Contact",
    desc = "Your friend {friend} has a gift for you.",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    ROAD_LOCATION = {
        name = "Bog Road",
        map_tags = {"road"},
        tags = {"road"},
        plax = "EXT_Bog_Road_01",
    },
}

:AddCast{
    cast_id = "friend",
    condition = function( agent, quest )
        local player_rel = TheGame:GetGameState():GetPlayerAgent():GetAspect("social_connections")
        return player_rel:FindOpinionEvent(agent, {id ="EVENT_OLD_DRINKING_BUDDY_drank_with_them"}) or player_rel:FindOpinionEvent(agent, {id ="EVENT_OLD_DRINKING_BUDDY_paid_back_debt"}) or player_rel:FindOpinionEvent(agent, {id ="EVENT_OLD_DRINKING_BUDDY_defended_them"})
    end,
    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}

:AddOpinionEvents{
    offered_coilgun =  
    {
        delta = OPINION_DELTAS.NEUTRAL,
        txt = "Was offered a gift.",
    },
}

QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
        :Loc{
            DIALOG_INTRO_DRANK = [[
                agent:
                    !right
                player:
                    !left
                    !wave
                    {agent}! I feel like I'm running into you everywhere I go now!
                agent:
                    Ha! Just like the good old days, eh?
                    Thanks for joining me in that drink last time.
                    I'm actually glad I ran into you.
                player:
                    Oh yeah? What's on your mind?
                agent:
                    Well I found this old thing and I don't need it these days, but I figured maybe you would appreciate it.
                * {agent} reaches into {agent.hisher} bag and pulls out a {1#card}.
                player:
                    Wow, that sure is nice.
                agent:
                    It's yours if you want it. For the good old days.
            ]],
            DIALOG_INTRO_DEFENDED = [[
                player:
                    !left
                agent:
                    !right
                    !wave
                    There you are!
                player:
                    Have you been looking for me?
                agent:
                    Yeah, I wanted to thank you for helping me out last time we met.
                    I found this amongst my old things and I thought you might like it.
                * {agent} reaches into {agent.hisher} bag and pulls out a {1#card}.
                agent:
                    What do you think? Nice, right?
            ]],
            DIALOG_INTRO_PAID_DEBT = [[
                player:
                    !left
                agent:
                    !right
                    Hey! Listen, I appreciate your help last we met and I wanted to make it right.
                    The whole money situation still isn't great but I did find this.
                * {agent} reaches into {agent.hisher} bag and pulls out a {1#card}.
                player:
                    Well that certainly takes me back to the good old days. Mint condition, too.
                agent:
                    You can have it if you want. As a token of my gratitude for helping me out.
            ]],
            OPT_TAKE_COILGUN = "Accept the {1#card}",
            DIALOG_TAKE_COILGUN = [[
                player:
                    Sure! Thanks for this.
                    !take
                agent:
                    !give
                player:
                    I'll make good use of it.
                agent:
                    I don't doubt it! I hope I'll see you around again.
            ]],
            OPT_DECLINE = "Politely decline the gift",
            DIALOG_DECLINE = [[
                player:
                    You know what? Don't worry about it. You don't owe me anything.
                agent:
                    Are you sure?
                player:
                    Yeah, we are friends after all.
            ]],
        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("friend"))
                local player_rel = TheGame:GetGameState():GetPlayerAgent():GetAspect("social_connections")
                if player_rel:FindOpinionEvent(cxt:GetAgent(), {id ="EVENT_OLD_DRINKING_BUDDY_paid_back_debt"}) then
                    cxt:Dialog("DIALOG_INTRO_PAID_DEBT", "aerostat_coilgun")
                elseif player_rel:FindOpinionEvent(cxt:GetAgent(), {id ="EVENT_OLD_DRINKING_BUDDY_drank_with_them"}) then
                    cxt:Dialog("DIALOG_INTRO_DRANK", "aerostat_coilgun")
                elseif player_rel:FindOpinionEvent(cxt:GetAgent(), {id ="EVENT_OLD_DRINKING_BUDDY_defended_them"}) then
                    cxt:Dialog("DIALOG_INTRO_DEFENDED", "aerostat_coilgun")
                end
            end
            cxt:Opt("OPT_TAKE_COILGUN", "aerostat_coilgun")
                :Dialog("DIALOG_TAKE_COILGUN")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("offered_coilgun"))
                :Fn( function(cxt)
                    cxt:ForceTakeCards{"aerostat_coilgun"}
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_DECLINE")
                :Dialog("DIALOG_DECLINE")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("offered_coilgun"))
                :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)
        end)