local negotiation_defs = require "negotiation/negotiation_defs"
local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

local VALID_PET_DEFS = {"AUTODOG", "RISE_AUTODOG", "FERAL_AUTODOG"}
local BARON_AUTODOG_CARD = "autodog_blaster"
local RISE_AUTODOG_CARD = "auto_plating"
local FERAL_AUTODOG_CARD = "swamp_schematics"

Content.AddBattleCard( "auto_plating", 
{
    name = "Auto-Plating",
    desc = "While this is in your hand, gain 1 {DEFEND} whenever you draw a card.",
    flavour = "'It telescopes right out into place! You can't even feel it under your armor!'",
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    local_item = LOCAL_ITEMS.BOG,

    cost = 0,

    flags = battle_defs.CARD_FLAGS.ITEM | battle_defs.CARD_FLAGS.UNPLAYABLE,

    deck_handlers = { DECK_TYPE.IN_HAND },

    event_handlers =
    {
        [ BATTLE_EVENT.DRAW_CARD ] = function( self, battle, card, start_of_turn )
            if card.owner == self.owner and self ~= card then
                self.owner:AddCondition("DEFEND", 1, self)
            end
        end,
    },
})

Content.AddBattleCard( "autodog_blaster", 
{
    name = "AutoDog Blaster",
    anim = "shoot",
    flavour = "'TAMPERING WITH THE AUTODOG WEAPONRY IS NOT RECOMMENDED.'",
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    local_item = LOCAL_ITEMS.BOG,
    anims = { "anim/weapon_blaster_autodog.zip"},
    hit_tags = { "spark" },

    cost = 1,
    

    flags = battle_defs.CARD_FLAGS.RANGED | battle_defs.CARD_FLAGS.ITEM | battle_defs.CARD_FLAGS.PIERCING,
    max_charges = 3,

    min_damage = 6,
    max_damage = 10,
})

Content.AddNegotiationCard("swamp_schematics", 
{
    name = "Swamp Schematics",
    desc = "Gain 1 {INFLUENCE}.",
    flavour = "'A lot of things went missing in the bog, it's like the landscape changes every night.'",
    cost = 0,
    max_xp = 6,
    hatch = true,
    flags = negotiation_defs.CARD_FLAGS.ITEM | negotiation_defs.CARD_FLAGS.HATCH,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    hatch_fn = function( self, minigame )
        self:TransferCard( minigame.trash_deck )
        self:Consume()
        local card = Negotiation.Card( "swamp_schematics_hatch", self.owner )
        minigame:DealCard( card, minigame:GetHandDeck( ) )
        card:TransferCard( minigame.trash_deck )
        card:Consume()
        QuestUtil.SpawnQuest("SWAMP_CACHE")
    end,

    OnPostResolve = function( self, minigame )
        self.negotiator:AddModifier("INFLUENCE", 1, self)
    end,
})

Content.AddNegotiationCard("swamp_schematics_hatch", 
{
    name = "Combined Map",
    desc = "A large chip of dirt falls off the schematic, revealing the location of a weapons cache.",
    flavour = "'Collecting enough of the mapped areas can pay off in a big way.'",
    cost = 0,
    flags = negotiation_defs.CARD_FLAGS.ITEM | negotiation_defs.CARD_FLAGS.UNPLAYABLE,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
})

local QDEF = QuestDef.Define{
    title = "AutoDog Salvager",
    icon = engine.asset.Texture("icons/quests/oppo_autodog_salvager.tex"),
    qtype = QTYPE.OPPORTUNITY,
    precondition = function(quest) 
        for k,pet in pairs(TheGame:GetGameState():GetCaravan():GetPets()) do
            if table.contains(VALID_PET_DEFS, pet:GetContentID()) then
                return true
            end
        end
        return false
    end,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("ROAD_LOCATION"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "AutoDog Salvager",
    desc = "A tech salvager has opened an impromptu shop. They can disassemble your AutoDog for valuable parts.",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    ROAD_LOCATION = {
        name = "Bog Road",
        map_tags = {"road"},
        tags = {"road"},
        plax = "EXT_Bog_Road_01",
    },
}

:AddCast{
    cast_id = "salvager",
    condition = function( agent, quest )
        return agent:GetLocation() ~= TheGame:GetGameState():GetPlayerAgent():GetLocation() and agent:GetFactionID() == "RISE" end,

    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}

QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !wave
                    Greetings! That's a fine specimen you've got there!
                    If you sell it to me, I'll give you the best part of it!
            ]],
            OPT_BARON_AUTODOG = "Sell {1#agent}",
            TT_BARON_AUTODOG = "Gain {1#card_list}",
            DIALOG_BARON_AUTODOG = [[
                player:
                    Sure, sounds like a good deal.
                    !give
                agent:
                    !take
                    Ah, fantastic! These ones are hard to come by. The Spark Barons don't exactly hand them out for free.
            ]],
            OPT_RISE_AUTODOG = "Sell {1#agent}",
            TT_RISE_AUTODOG = "Gain {1#card_list}",
            DIALOG_RISE_AUTODOG = [[
                player:
                    You've got yourself a deal!
                    !give
                agent:
                    Excellent! Here's your payment, and the part I promised.
            ]],
            OPT_FERAL_AUTODOG = "Sell {1#agent}",
            TT_FERAL_AUTODOG = "Gain {1#card_list}",
            DIALOG_FERAL_AUTODOG = [[
                player:
                    I don't know how much use you'll get out of this one, but you've got yourself a deal.
                agent:
                    You'd be surprised how useful its spare parts can be!
                    !take
                player:
                    !give
            ]],
            OPT_DECLINE = "Decline the offer",
            DIALOG_DECLINE = [[
                player:
                    Sorry, I'm not interesting in selling anything right now.
                agent:
                    Ah, that's a shame. If you change your mind, come find me.
                player:
                    !exit
                agent:
                    !exit
            ]],
        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("salvager"))
                cxt:Dialog("DIALOG_INTRO")
                cxt.enc.scratch.owned_pets = {}
                for k,pet in pairs(TheGame:GetGameState():GetCaravan():GetPets()) do
                    if table.contains(VALID_PET_DEFS, pet:GetContentID()) then
                        cxt.enc.scratch.owned_pets[pet:GetContentID()] = pet
                    end
                end
            end
            
            if cxt.enc.scratch.owned_pets["AUTODOG"] then
                cxt:Opt("OPT_BARON_AUTODOG", cxt.enc.scratch.owned_pets["AUTODOG"])
                    :PostText("TT_BARON_AUTODOG", {BARON_AUTODOG_CARD})
                    :PostCard( BARON_AUTODOG_CARD )
                    :Dialog("DIALOG_BARON_AUTODOG")
                    :ReceiveMoney(100)
                    :Fn(function( cxt )
                        cxt.enc.scratch.owned_pets["AUTODOG"]:Dismiss()
                        cxt:GainCards{BARON_AUTODOG_CARD}
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end

            if cxt.enc.scratch.owned_pets["RISE_AUTODOG"] then
                cxt:Opt("OPT_RISE_AUTODOG", cxt.enc.scratch.owned_pets["RISE_AUTODOG"])
                    :PostText("TT_RISE_AUTODOG", {RISE_AUTODOG_CARD})
                    :PostCard( RISE_AUTODOG_CARD )
                    :Dialog("DIALOG_RISE_AUTODOG")
                    :ReceiveMoney(65)
                    :Fn(function( cxt )
                        cxt.enc.scratch.owned_pets["RISE_AUTODOG"]:Dismiss()
                        cxt:GainCards{RISE_AUTODOG_CARD}
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end

            if cxt.enc.scratch.owned_pets["FERAL_AUTODOG"] then
                cxt:Opt("OPT_FERAL_AUTODOG", cxt.enc.scratch.owned_pets["FERAL_AUTODOG"])
                    :PostText("TT_FERAL_AUTODOG", {FERAL_AUTODOG_CARD})
                    :PostCard( FERAL_AUTODOG_CARD )
                    :Dialog("DIALOG_FERAL_AUTODOG")
                    :ReceiveMoney(20)
                    :Fn(function( cxt )
                        cxt.enc.scratch.owned_pets["FERAL_AUTODOG"]:Dismiss()
                        cxt:GainCards{FERAL_AUTODOG_CARD}
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end        

            cxt:Opt("OPT_DECLINE")
                :Dialog("DIALOG_DECLINE")
                :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)


        end)


