local SPARK_PRICE = 175
local RISE_PRICE = 100
local FERAL_PRICE = 30

local QDEF = QuestDef.Define{
    title = "Autodog Road Stall",
    icon = engine.asset.Texture("icons/quests/oppo_autodog_dealer.tex"),
    qtype = QTYPE.OPPORTUNITY,
    precondition = function(quest) 
        return TheGame:GetGameState():GetCaravan():GetMoney() >= FERAL_PRICE and TheGame:GetGameState():GetCaravan():CanOwnPet()
    end,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("ROAD_LOCATION"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Autodog Stall",
    desc = "A travelling AutoDog salesman has set up a makeshift stall.",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    ROAD_LOCATION = {
        name = "Bog Road",
        map_tags = {"road"},
        tags = {"road"},
        plax = "EXT_Bog_Road_01",
    },
}

:AddCast{
    cast_id = "seller",
    condition = function( agent, quest )
        return agent:GetLocation() ~= TheGame:GetGameState():GetPlayerAgent():GetLocation() and agent:GetContentID() == "WEALTHY_MERCHANT" end, -- Change to a different def

    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}

QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !wave
                    !happy
                    Greetings! Welcome to my humble shop.
                player:
                    !wave
                    What kind of wares do you have?
                agent:
                    I've got the best deals on Autodogs in all of the bog.
                    My collection varies from pristine Autodogs straight from the factory, to modified versions to the downright feral ones.
                    Of course, they're all loyal and obedient. Trained the feral one myself.
                    All of them are guaranteed to fulfill your needs!
            ]],
            OPT_BUY_BARON_AUTODOG = "Buy the Spark Baron Autodog",
            DIALOG_BUY_BARON_AUTODOG = [[
                player:
                    Hmm, I'll take the best you've got.
                    !take
                agent:
                    !give
                    Excellent choice! Definitely the most value for your money!
            ]],
            OPT_BUY_RISE_AUTODOG = "Buy the modified Rise Autodog",
            DIALOG_BUY_RISE_AUTODOG = [[
                player:
                    You certain those modifications are reliable?
                agent:
                    Of course! Everything I sell is of the highest quality.
                player:
                    Alright, I'll take that one then!
                    !take
                agent:
                    !give
                    Perfect!
            ]],
            OPT_BUY_FERAL_AUTODOG = "Buy the 'tame' Feral Autodog",
            DIALOG_BUY_FERAL_AUTODOG = [[
                player:
                    I'll buy that one.
                agent:
                    Th- the feral one? Oh, uhm of course!
                    !give
                player:
                    !take
            ]],
            OPT_DECLINE = "Decline the offer",
            DIALOG_DECLINE = [[
                player:
                    Sorry bud, not interested right now.
                agent:
                    Well, perhaps another time. Swing by next time I have my stall up if you change your mind.
            ]],
        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("seller"))
                cxt:Dialog("DIALOG_INTRO")
            end
            cxt:Opt("OPT_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")
            
            local can_own, reason = cxt.caravan:CanOwnPet()

            cxt:Opt("OPT_BUY_BARON_AUTODOG")
                :ReqCondition( can_own, reason )
                :Dialog("DIALOG_BUY_BARON_AUTODOG")
                :DeliverMoney(SPARK_PRICE, { is_shop = true })
                :Fn(function() 
                    local pet = TheGame:GetGameState():AddAgent(Agent("AUTODOG"))
                    pet:Recruit(PARTY_MEMBER_TYPE.CREW)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_BUY_RISE_AUTODOG")
                :ReqCondition( can_own, reason )
                :Dialog("DIALOG_BUY_RISE_AUTODOG")
                :DeliverMoney(RISE_PRICE, { is_shop = true })
                :Fn(function() 
                    local pet = TheGame:GetGameState():AddAgent(Agent("RISE_AUTODOG"))
                    pet:Recruit(PARTY_MEMBER_TYPE.CREW)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_BUY_FERAL_AUTODOG")
                :ReqCondition( can_own, reason )
                :Dialog("DIALOG_BUY_FERAL_AUTODOG")
                :DeliverMoney(FERAL_PRICE, { is_shop = true })
                :Fn(function() 
                    local pet = TheGame:GetGameState():AddAgent(Agent("FERAL_AUTODOG"))
                    pet:Recruit(PARTY_MEMBER_TYPE.CREW)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_DECLINE")
                :Dialog("DIALOG_DECLINE")
                :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)


        end)

        :AskAboutHub("STATE_QUESTIONS", 
        {

            "Ask {agent} about the Spark Baron Autodog",
            [[
                player:
                    That's quite a price tag on that one.
                agent:
                    Not at all! This bad boy is in pristine condition. No damage, no modifications.
                    You really won't find another Autodog like it. Unless of course you're looking to join the Barons.
            ]],
            "Ask {agent} about the modified Rise Autodog",
            [[
                player:
                    Are those modifications going to cause problems?
                agent:
                    !hips
                    The Rise steal Autodogs and modify them all the time. It hardly ever goes wrong and I've thoroughly tested this one myself!
            ]],
            "Ask {agent} about the Feral Autodog",
            [[
                player:
                    That one doesn't exactly look safe, let alone reliable.
                agent:
                    Looks can be deceiving, my friend! I tamed this one myself, modified it to be good as new.
            ]],
        })

