local QDEF = QuestDef.Define{
    title = "Deeper in the Bog",
    desc = "{explorer} is looking to hire a guide for an expedition into the Bog.",
    icon = engine.asset.Texture("icons/quests/event_deeper_in_the_bog.tex"),
    qtype = QTYPE.OPPORTUNITY,
    on_destroy = function(quest) 
        if quest:GetCastMember("explorer") and quest:GetCastMember("explorer"):IsInPlayerParty() then
            quest:GetCastMember("explorer"):Dismiss()
        end
    end,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("BOG_TRAIL_HEAD"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"location"},
}

:AddLocationDefs{
    BOG_TRAIL_HEAD = {
        name = "Bog Trailhead",
        map_tags = {"bog"},
        plax = "EXT_Bog_Road_01",
    },
    BOG = {
        name = "Bog",
        plax = "EXT_Bog_Forest_01",
    },
    BOG_OUTCROP = {
        name = "Bog",
        plax = "EXT_Bog_Outcrop_01",
    },
    BOG_GOD = {
        name = "Bog God",
        plax = "EXT_Bog_HedgeGod_01",
    },
}

:AddCast{
    cast_id = "explorer",
    cast_fn = function(quest, t) --new person every time
        table.insert( t, quest:CreateSkinnedAgent( "JAKES_RUNNER" ) )
    end,
}


:AddOpinionEvents{

    abandoned = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Abandoned them in the Bog.",
    },
    did_not_help_them =  
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Did not help them explore the Bog.",
    },
    found_secret =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Helped them find the 'secret' of the Bog.",
    },
}

local function GoToNewLocation(cxt, id)
    local location = cxt.quest:SpawnTempLocation(id)
    location:SetMapPos( cxt:GetCastMember("location"):GetMapPos())
    cxt.encounter:DoLocationTransition( location )
    cxt.encounter:PresentAgent(nil, SIDE.LEFT)
    cxt.encounter:PresentAgent(nil, SIDE.RIGHT)
end

QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find a lone Jake poring over a crudely-drawn map.
                agent:
                    !right
                    !happy
                    You there? Are you looking for the opportunity of a lifetime?
                player:
                    !left
                    Always. What are you offering?
                agent:
                    The Bog. It holds riches - I'm sure of it!
                player:
                    In my limited experience, the Bog holds mostly danger, slime, and a quick death.
                agent:
                    But also: riches!
                    My research proves it!
                    I just need a bit of protection to help me avoid the 'quick death' part.
                    You follow me as I follow this map, and I'll share the spoils. 
                    Are you in?
                *** {agent} wants you to protect {agent.himher} as {agent.heshe} ventures deep into the bog.
            ]],

            OPT_ASK_ABOUT_RESEARCH = "Ask about {agent}'s research",
            DIALOG_ASK_ABOUT_RESEARCH = [[
                player:
                    Tell me about this research.
                agent:
                    Why? So you can steal it!
                    !crossed
                    No way.
            ]],
            OPT_AGREE = "Agree to accompany {agent} into the Bog",
            DIALOG_AGREE = [[
                player:
                    I'll do it. 
                    But this better pay off!
                agent:
                    !clap
                    Great!
                    You might have to kill an erchin or ten, but other than that this will be the easiest money you ever earned!
                    !exit
                player:
                    !shrug
                * You follow {agent} into the Bog, guns at the ready.
            ]],

            OPT_DISAGREE = "Don't agree",
            DIALOG_DISAGREE = [[
                player:
                    Sounds too dangerous. No deal.
                agent:
                    !angry
                    You're throwing away a fortune!
                    !spit
                    What kind of grifter are you anyway?
                    !exit
                * {agent} stomps off into the Bog unaccompanied.
            ]],
        }
        
        :Fn(function(cxt)
            cxt.quest:GetCastMember('explorer'):MoveToLocation(cxt.location)
            cxt:TalkTo( cxt.quest:GetCastMember('explorer') )
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                
                cxt:Opt("OPT_ASK_ABOUT_RESEARCH")
                    :Dialog("DIALOG_ASK_ABOUT_RESEARCH")

                cxt:Opt("OPT_AGREE")
                    :Dialog("DIALOG_AGREE")
                    :Fn(function() cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT) end)
                    :GoTo("STATE_BOG_STAGE_1")

                cxt:Opt("OPT_DISAGREE")
                    :Dialog("DIALOG_DISAGREE")
                    :ReceiveOpinion("did_not_help_them")
                    :CompleteQuest()
                    :Travel()
            end)


        end)
    

    --you get the insert card.
    :State("STATE_BOG_STAGE_1")
        :Loc{
            DIALOG_INTRO = [[
                * {explorer} leads you through the Bog. The terrain gets rougher the farther you get from the road, and you eventually come to a rocky outcrop.
               
            ]],
            DIALOG_SPOOKED = [[
                * Your heart fills with dread as your boots fill with bogwater.
            ]],
            DIALOG_WEARY = [[
                * Each step sinks deeper into the bog. It's all you can do to keep walking forward.
            ]],
            DIALOG_INTRO_2 = [[
                agent:
                    !right
                    !scared
                    Wait! Did you hear that?
                player:
                    !left
                    !fight
                    What is it?
                agent:
                    !cagey
                    It's...
                    Nothing. Let's keep going.
                    !neutral
            ]],
            OPT_KEEP_FOLLOWING = "Keep going",
            DIALOG_KEEP_FOLLOWING = [[
                agent:
                    !exit
                * You follow, surrounded by Bog on all sides.
            ]],
            OPT_ABANDON = "Abandon {agent}",
            DIALOG_ABANDON = [[
                player:
                    No, this is more than I bargained for.
                    Sorry, {agent}, but you're on your own.
                agent:
                    !angry
                    You can't quit now! 
                player:
                    !thought
                    Well, actually...
                    !exit
                agent:
                    We had an agreement!
                    !threaten
                    Heshin' grifter!

                * You find your way out of the Bog, leaving {agent} behind.
            ]],
        }
        :Fn(function(cxt) 
            GoToNewLocation(cxt, "BOG_OUTCROP")
            cxt:TalkTo(cxt.quest:GetCastMember("explorer"))
            cxt:Dialog("DIALOG_INTRO")
            if math.random() < .5 then
                cxt:Dialog("DIALOG_SPOOKED")
                cxt:ForceTakeCards{"spooked"}
            else
                cxt:Dialog("DIALOG_WEARY")
                cxt:ForceTakeCards{"weary"}
            end
            
            cxt:Dialog("DIALOG_INTRO_2")

            cxt:Opt("OPT_KEEP_FOLLOWING")
                :Dialog("DIALOG_KEEP_FOLLOWING")
                :GoTo("STATE_BOG_STAGE_2")
            
            cxt:Opt("OPT_ABANDON")
                :Dialog("DIALOG_ABANDON")
                :ReceiveOpinion("abandoned")
                :CompleteQuest()
                :Travel()
        end)

    --you get the insert card.
    :State("STATE_BOG_STAGE_2")
        :Loc{
            DIALOG_INTRO = [[
                * You press on. The Bog presses in. Your nose experiences things that language can not yet describe.
                agent:
                    !right
                    !cagey
                    I hear it again.
                player:
                    !left
                    Are you sure it's not-
            ]],
            
            DIALOG_MONSTER_FIGHT = [[
                agent:
                    !right
                * You are set upon by the local fauna!                    
            ]],
            DIALOG_BOGGER_FIGHT = [[
                * A band of boggers emerges!
                agent:
                    !right
                    !threaten
                    Outsiders! Kill them! Return their essence to the Bog!
            ]],
            OPT_SCARE_OFF_THE_BOGGERS = "Scare off the boggers",
            
            DIALOG_SCARE_OFF_THE_BOGGERS = [[
                player:
                    Stand down, bogger, or you might end up feeding the Bog yourself!
            ]],
            DIALOG_SCARE_OFF_THE_BOGGERS_SUCCESS = [[
                agent:
                    Mulch this!
                    !exit
                * The Boggers retreat.
                explorer:
                    !right
                    I sure am glad I brought you along!
                    Let's keep going!
                    !exit
                * You follow {explorer}.
            ]],
            DIALOG_SCARE_OFF_THE_BOGGERS_FAILURE = [[
                agent:
                    We do not fear the Bog! We are one with the Bog!
            ]],

            OPT_RUN_AWAY = "Run away, and abandon {explorer}",
            DIALOG_RUN_AWAY = [[
                player:
                    Sorry, {explorer}, this ain't worth it!
                explorer:
                    !right
                    !flinch
                    What?! But I hired you!
                player:
                    !exit
                * You leave {explorer} to {explorer.hisher} fate, and retrace your steps out of the Bog.
            ]],

            DIALOG_WON_FIGHT_EXPLORER_DEAD = [[
                right:
                    !exit
                * {explorer} is dead, and so is your contract. 
            ]],

            OPT_READ_NOTES = "Read {agent}'s notes",
            DIALOG_READ_NOTES = [[
                * You pick {explorer}'s notes of {explorer.hisher} corpse, but only find mad scrawlings.
            ]],

            DIALOG_WON_FIGHT_EXPLORER_ALIVE = [[
                explorer:
                    !right
                    Wow! What a hairball!
                    !happy
                    There's nothing in my notes about that.
                    Time to carry on!
                    !exit
                * You follow {explorer}.
            ]],
            DIALOG_DEFEND = [[
                player:
                    !fight
                    We can do this, {explorer}!
            ]]
        }
        :Fn(function(cxt) 
            GoToNewLocation(cxt, "BOG")
            cxt:TalkTo(cxt.quest:GetCastMember("explorer"))
            cxt:Dialog("DIALOG_INTRO")
            local opfor
            if math.random() < .5 then
                opfor = CreateCombatParty("BOGGER_PATROL", cxt.quest:GetRank(), cxt.location)

            else
                opfor = CreateCombatParty("BOG_MONSTERS", cxt.quest:GetRank(), cxt.location)
            end
            cxt:TalkTo(opfor[1])

            if opfor[1]:IsSentient() then
               cxt:Dialog("DIALOG_BOGGER_FIGHT") 
            else
                cxt:Dialog("DIALOG_MONSTER_FIGHT") 
            end

            cxt:RunLoop(function() 
                if opfor[1]:IsSentient() then
                   cxt:Opt("OPT_SCARE_OFF_THE_BOGGERS")
                        :Dialog("DIALOG_SCARE_OFF_THE_BOGGERS")
                        :Negotiation{}
                            :OnSuccess()
                                :Dialog("DIALOG_SCARE_OFF_THE_BOGGERS_SUCCESS")
                                :GoTo(math.random() < .5 and "STATE_BOG_STAGE_3a" or "STATE_BOG_STAGE_3b")
                            :OnFailure()
                                :Dialog("DIALOG_SCARE_OFF_THE_BOGGERS_FAILURE")
                end

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        allies = {cxt.quest:GetCastMember("explorer")},
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.ISOLATED}
                        :OnWin()
                            
                            :Fn(function() 
                                 
                                if cxt.quest:GetCastMember("explorer"):IsDead() then
                                    cxt:Dialog("DIALOG_WON_FIGHT_EXPLORER_DEAD")
                                    cxt.quest:Complete()
                                    cxt:Opt("OPT_READ_NOTES")
                                        :Dialog("DIALOG_READ_NOTES")
                                        :GainCards{"deep_bog_battle"}
                                        :Travel()
                                else
                                    cxt:Dialog("DIALOG_WON_FIGHT_EXPLORER_ALIVE")
                                    cxt:GoTo(math.random() < .5 and "STATE_BOG_STAGE_3a" or "STATE_BOG_STAGE_3b")
                                end
                            end)

                cxt:Opt("OPT_RUN_AWAY")
                    :Dialog("DIALOG_RUN_AWAY")
                    :ReceiveOpinion("abandoned", nil, cxt.quest:GetCastMember("explorer"))
                    :Fn(function() if math.random() < .5 then cxt.quest:GetCastMember("explorer"):Kill() end end)
                    :CompleteQuest()
                    :Travel()
            end)    

        end)
    
    :State("STATE_BOG_STAGE_3a")
        :Loc{
            DIALOG_INTRO = [[
                * You come to yet another identical clearing.
                explorer:
                    !right
                    !happy
                    Here it is! We found it!
                player:
                    !left
                    Found what?
                explorer:
                    !overthere
                    The secret of the bog, of course!
                    Look! It's all around you!
                player:
                    !dubious
                explorer:
                    !permit
                    Take as much as you want in payment.
                * {explorer}'s mind has clearly succumbed to the Bog.
            ]],

            OPT_CONVINCE_PAYMENT = "Convince {agent} to pay you in money",
            DIALOG_CONVINCE_PAYMENT = [[
                player:
                    Eh...
                    I wouldn't want to take the riches of the Bog from you. You earned it, friend.
                    How about a couple of shills for my time, instead?
            ]],
            DIALOG_CONVINCE_PAYMENT_SUCCESS = [[
                agent:
                    You're right. I have no need of money any more.
                    !give
                    Take it all!
                    !exit
                * {agent} runs off into the Bog. You stoop to pick up the coins, and then turn to leave.
            ]],
            DIALOG_CONVINCE_PAYMENT_FAILURE = [[
                agent:
                    I would give you all of my money if I had any.
                    But I don't! I spent it all doing my research.
                    Thank the Bog it all paid off!
                    !exit
                player:
                    !sigh
                * {agent} runs off into the Bog, leaving you with immense, invisible 'riches'.
            ]],

            OPT_LEAVE = "Leave {agent} to {agent.hisher} 'riches'",
            DIALOG_LEAVE = [[
                * You decide it unwise to provoke {agent}, and play along.
                player:
                    You're so very generous. Thank you.
                    !exit
                agent:
                    No, thank <b>you</>!
            ]]
        }
        :Fn(function(cxt) 
            GoToNewLocation(cxt, "BOG")
            cxt:TalkTo(cxt.quest:GetCastMember('explorer'))
            
            cxt:Dialog("DIALOG_INTRO")
            

            cxt:Opt("OPT_CONVINCE_PAYMENT")
                :Dialog("DIALOG_CONVINCE_PAYMENT")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_CONVINCE_PAYMENT_SUCCESS")
                        :ReceiveMoney(cxt.quest:GetRank()*75)
                        :ReceiveOpinion("found_secret")
                        :CompleteQuest()
                        :Travel()
                    :OnFailure()
                        :Dialog("DIALOG_CONVINCE_PAYMENT_FAILURE")
                        :ReceiveOpinion("found_secret")
                        :CompleteQuest()
                        :Travel()
            
            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :ReceiveOpinion("found_secret")
                :CompleteQuest()
                :Travel()

        end)
    
    :State("STATE_BOG_STAGE_3b")
        :Loc{
            DIALOG_INTRO = [[
                * You find a strange structure.
                explorer:
                    !right
                    We found it!
                    The Heart of the Bog!
                player:
                    !left
                    Where's the money part of it?
                explorer:
                    It requires sacrifice, first.
                    Quickly, lay down on that platform. 
                    I'll plunge my knife through your heart, and then your spirit will join with the Bog and unlock its secrets!
                player:
                    Excuse me?
                explorer:
                    Don't worry, it won't hurt. And you will be reborn in the spirit of the Bog!
                player:
                    Uh...
            ]],

            OPT_CONVINCE_NOT = "Convince {agent} that {agent.hisher} plan is insane",
            DIALOG_CONVINCE_NOT = [[
                player:
                    Can you even hear yourself talking anymore?
            ]],
            DIALOG_CONVINCE_NOT_SUCCESS = [[
                explorer:
                    Fine. I get it. The Bog is not cut out for everyone.
                    Now that I know where the Heart is, I can come back with someone with a more open mind.
                    !give
                    Take these shills, and leave.
                    !exit
                * {explorer} wanders off to inspect the ruins. You decide to leave quietly.
            ]],
            DIALOG_CONVINCE_NOT_FAILURE = [[
                explorer:
                    I get it. It's scary. You've not yet opened your eyes to the potential of the Bog.
                    !happy
                    I'm here to <b>help</> you, friend.
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    What the Hesh, {agent}?
            ]],
            DIALOG_POST_FIGHT = [[
                * {agent} is dead. You rummage through {agent.hisher} notes before leaving.
            ]],


        }
        :Fn(function(cxt) 
            GoToNewLocation(cxt, "BOG_GOD")
            cxt:Dialog("DIALOG_INTRO")
            cxt:TalkTo(cxt.quest:GetCastMember("explorer"))

            cxt:RunLoop(function() 
                cxt:Opt("OPT_CONVINCE_NOT")
                    :Dialog("DIALOG_CONVINCE_NOT")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_NOT_SUCCESS")
                            :CompleteQuest()
                            :ReceiveMoney(50*cxt.quest:GetRank())
                            :Travel()
                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_NOT_FAILURE")
                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                        :Battle{flags = BATTLE_FLAGS.NO_SURRENDER | BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.SELF_DEFENCE}
                            :OnWin()
                                :Dialog("DIALOG_POST_FIGHT")
                                :GainCards{"deep_bog_battle", "deep_bog_battle"}
                                :CompleteQuest()
                                :Travel()
            end)


            --
        end)

