local QDEF = QuestDef.Define{
    title = "Healing Supplies",
    icon = engine.asset.Texture("icons/quests/oppo_healing_spring.tex"),
    desc = "There's a cache of healing supplies hidden in the bog.",
    qtype = QTYPE.OPPORTUNITY,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("CACHE_LOCATION"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    CACHE_LOCATION = {
        name = "Healing Cache",
        map_tags = {"forest"},
        plax = "EXT_Bog_Forest_01",
        indoors = false,
    },
}


QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            * You find the cache of supplies half submerged in the bog.
            * Upon opening it, most of the supplies are dry, but some of them look damaged by the bog water.
        ]],
        OPT_HEAL_LITTLE = "Use the dry supplies (Heals <#HEALTH>{1} HP</>)",
        DIALOG_HEAL_LITTLE = [[
            * You use the clean supplies and put aside the damaged ones.
        ]],
        OPT_HEAL_BIG = "Use all of the supplies in the cache (Heals <#HEALTH>{1} HP</>, receive {2#card})",
        DIALOG_HEAL_BIG = [[
            * You empty out all of the supplies onto a patch of grass and begin treating your ailments.
            * You feel better, but the bog water in the supplies has made you nauseous.
        ]],
        DIALOG_DONE_NO_HEAL = [[
            * Not wanting to risk getting sick from the bog water, you leave the cache behind.
        ]]
    }
    :Fn(function(cxt)
        cxt:Dialog("DIALOG_INTRO")

        local little_pts = 15
        local big_pts = 30

        cxt:Opt("OPT_HEAL_LITTLE", little_pts)
            :PreIcon(global_images.heallittle)
            :Dialog("DIALOG_HEAL_LITTLE")
            :DeltaHealth(little_pts)
            :CompleteQuest()
            :Fn(function() StateGraphUtil.AddLeaveLocation(cxt) end)
            
        cxt:Opt("OPT_HEAL_BIG", big_pts, "sick")
            :PreIcon(global_images.heal)
            :Dialog("DIALOG_HEAL_BIG")
            :DeltaHealth(big_pts)
            :Fn(function()
                cxt:ForceTakeCards{"sick"}
            end)
            :CompleteQuest()
            :Fn(function() StateGraphUtil.AddLeaveLocation(cxt) end)

        cxt:Opt("OPT_LEAVE"):MakeUnder()
            :PreIcon( global_images.close)
            :CompleteQuest()
            :Dialog("DIALOG_DONE_NO_HEAL")
            :Travel()
    end)


