local RESOLVE_GAIN = 5
local HEALTH_LOSS = 6

local QDEF = QuestDef.Define{
    title = "Bog Shaman",
    icon = engine.asset.Texture("icons/quests/oppo_bog_shaman.tex"),
    qtype = QTYPE.OPPORTUNITY,
    precondition = function(quest) 
        return TheGame:GetGameState():GetPlayerAgent():GetAspect("negotiator"):HasCardID("horrible_rash") and TheGame:GetGameState():GetPlayerAgent():GetHealth() > HEALTH_LOSS
    end,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("ROAD_LOCATION"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Bog Shaman",
    desc = "Word has it that a Bog Shaman can cure your Horrible Rash.",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    ROAD_LOCATION = {
        name = "Bog Road",
        map_tags = {"road"},
        tags = {"road"},
        plax = "EXT_Bog_Road_01",
    },
}

:AddCast{
    cast_id = "shaman",
    cast_fn = function(quest, t)
        local def = "PRIEST"
        local agent = quest:CreateSkinnedAgent( def )
        table.insert( t, agent )
    end,
    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}

:AddOpinionEvents{
    offered_coilgun =  
    {
        delta = OPINION_DELTAS.NEUTRAL,
        txt = "Was offered a gift.",
    },
}

QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !greeting
                    Well hello! How might I help you today?
                player:
                    I got myself in some trouble messing with plants from the bog.
                    I've heard that you might be able to help me.
                agent:
                    Of course! Though I warn you, while it strengthens your mind, it may hurt your body.
            ]],
            OPT_LET_THEM_HEAL_YOU = "Allow {agent} to heal you",
            OPT_HEAL_TT = "Upgrade your max resolve by {1}",
            DIALOG_LET_THEM_HEAL_YOU = [[
                player:
                    People seem to trust you. Let's do this.
                agent:
                    Alright. Hold still and you'll feel better soon.
                * {agent} pulls out a small jar with a salve in it. {agent.HeShe} applies it to your rash in a thick layer.
                * The salve slowly begins to sting, building up to a searing pain, but soon it's over and you feel nothing where your rash used to be.
                player:
                    !wince        
            ]],
            DIALOG_LET_THEM_HEAL_YOU_2 = [[
                player:
                    You weren't lying about the pain.
                agent:
                    Ha! I'm sure you've had worse. The rash is all gone now. Your body will no longer fight the properties of the flower.
            ]],
            OPT_DECLINE = "Decline their help",
            DIALOG_DECLINE = [[
                player:
                    Uhh, on second thought, I think I'll be okay.
                agent:
                    I promise you it wont be that bad, just give me a chance.
            ]],
            OPT_INSIST = "Insist that you don't want the help",
            DIALOG_INSIST = [[
                player:
                    No, really it's fine. It doesn't even itch that much anymore.
                agent:
                    That's up to you.
                    !shrug
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("shaman"))
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_LET_THEM_HEAL_YOU")
                :Dialog("DIALOG_LET_THEM_HEAL_YOU")
                :PreIcon( global_images.restoreresolve )
                :PostText( "OPT_HEAL_TT", RESOLVE_GAIN )
                :DeltaHealth(- HEALTH_LOSS)
                :Fn(function( cxt )
                    cxt:Dialog("DIALOG_LET_THEM_HEAL_YOU_2")
                    cxt.caravan:UpgradeResolve( RESOLVE_GAIN )
                    local negotiator = TheGame:GetGameState():GetPlayerAgent():GetAspect("negotiator")
                    local card = negotiator:FindCardByID("horrible_rash")
                    if card then
                        negotiator:RemoveCard(card)
                    end
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            if not cxt.enc.scratch.declined then
                cxt:Opt("OPT_DECLINE")
                    :Dialog("DIALOG_DECLINE")
                    :Fn(function(cxt) 
                        cxt.enc.scratch.declined = true
                    end)
            else
                cxt:Opt("OPT_INSIST")
                    :Dialog("DIALOG_INSIST")
                    :Fn(function(cxt) 
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end
        end)