

local QDEF = QuestDef.Define{
    title = "Enemy Exposed",
    icon = engine.asset.Texture("icons/quests/oppo_eliminate_enemy.tex"),
    qtype = QTYPE.OPPORTUNITY,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("FOREST_LOCATION"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Eliminate {enemy}",
    desc = "{enemy} is 'secretly' meeting with a contact. You have a chance to eliminate them quietly before the meeting.",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    FOREST_LOCATION = {
        name = "In The Woods",
        map_tags = {"forest"},
        plax = "EXT_Bog_Forest_01",
        indoors = false,
        show_agents = true,
    },
}

:AddCast{
    cast_id = "enemy",
    condition = function( agent, quest )
        return agent:GetRelationship() == RELATIONSHIP.HATED and agent:GetLocation() ~= TheGame:GetGameState():GetPlayerAgent():GetLocation()
    end,
    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}

:AddCast{
    cast_id = "contact",
    condition = function( agent, quest )
        return (agent:GetRelationship(quest:GetCastMember("enemy")) >= RELATIONSHIP.NEUTRAL or agent:GetRelationship(quest:GetCastMember("enemy")) == nil)

    end,
}


QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            * You spot {agent} standing alone in a clearing, waiting for someone else.
            * You might not have that much time until their contact arrives.
        ]],
        OPT_ATTACK = "Attack {enemy}",
        DIALOG_ATTACK = [[
            * You sneak as close to {enemy} as you can while staying hidden and then jump out, guns at the ready.
        ]],
        DIALOG_ATTACK_DEAD_NO_CONTACT = [[
            * {agent} is dead on the ground and {agent.hisher} contact could be here any minute.
            * You get rid of any evidence that could lead to your misdeed and leave the area before anyone sees you.
        ]],
        DIALOG_ATTACK_BOTH_DEAD = [[
            * That fight was more than you had anticipated but no one will ever find out you were here.
            * You leave the area, making sure not to leave a trail.
        ]],
        OPT_GATHER_INFORMATION = "Wait and gather intel",

        CONTACT_ARRIVED = "<#PENALTY>ENEMY CONTACT HAS ARRIVED!</>",
        CONTACT_ARRIVING = "CONTACT ARRIVING IN: <#PENALTY>{1} {1*TURN|TURNS} </>",
    }
    :SetLooping(true)
    :Fn(function(cxt)
        if cxt:FirstLoop() then
            cxt.quest:Complete()
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("enemy"))
            cxt:Dialog("DIALOG_INTRO")
        end
        local skip_turns = 6
        local contact_arrived = false
        local reason = nil
        
        cxt:Opt("OPT_ATTACK")
            :Dialog("DIALOG_ATTACK")
            :Battle{
                advantage = TEAM.BLUE,
                flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_SURRENDER,
                on_win = function(cxt) 
                    
                    if contact_arrived then
                        cxt:Dialog("DIALOG_ATTACK_BOTH_DEAD")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    else
                        cxt:Dialog("DIALOG_ATTACK_DEAD_NO_CONTACT")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                end,

                reason_fn = function(battle)
                    return reason
                end,
                
                on_player_turn_start = function(battle)
                    skip_turns = skip_turns - 1
                    if skip_turns <= 0 then
                        if not contact_arrived then
                            local agent = cxt.quest:GetCastAgent("contact")
                            local fighter = Fighter.CreateFromAgent( agent, battle:GetScenario():GetEnemyScale() )
                            battle:GetTeam(TEAM.RED):AddFighter( fighter )
                            battle:GetTeam(TEAM.RED):ActivateNewFighters()
                        end
                        reason = cxt:GetLocString( "CONTACT_ARRIVED" )
                        contact_arrived = true
                    else
                        reason = loc.format( cxt:GetLocString( "CONTACT_ARRIVING", skip_turns ))
                    end 
                end,
            }

        cxt:Opt("OPT_GATHER_INFORMATION")
            :GoTo("STATE_WAITED")
        
    end)

:State("STATE_WAITED")
    :Loc{
        DIALOG_WAIT = [[
            * You make sure you're well concealed and wait for {contact} to arrive.
            agent:
                !right
            contact:
                !left
                Did anyone see you?
            agent:
                Of course not. I'm careful.
            * You listen as {agent} and {contact} discuss the details of an upcoming weapons sale.
            * At one point you see {contact} handing {agent} a personal combat drone.
            * After they're done, {contact} leaves first. {agent} then takes off in the opposite direction.
            contact:
                !exit
            agent:
                !exit
        ]],
        OPT_CONFRONT = "Confront {agent} about their crimes",
        DIALOG_CONFRONT = [[
            * You move out of your cover and stop {agent}.
            player:
                !left
            agent:
                !right
                Huh- What the hell are <b>you</> doing here?
            player:
                I would ask you the same, but of course your little conversation there was all the information I needed.
            agent:
                What are you trying to say?
            player:
                I'm saying if I ever see you around again, I'm turning you in.
            agent:
                Oh yeah?
        ]],
        DIALOG_CONFRONT_WON = [[
            agent:
                Alright, alright. I've got family. If you tell anyone, it could hurt them.
                If you just keep quiet, you'll never have to see me again.
                !exit
        ]],
        DIALOG_CONFRONT_LOSS = [[
            agent:
                Ha! You're trying to blackmail me? You can't squeal if you're dead.
                !fight
            player:
                !fight
        ]],
        OPT_DEFEND_SELF = "Defend yourself",
        OPT_ATTACK_ALONE = "Attack {agent}",
        DIALOG_ATTACK_ALONE = [[
            player:
                !left
            agent:
                !right
            * As {agent} is about to leave, you jump out from your cover and attack.
            player:
                !fight
            agent:
                !fight
        ]],
        DIALOG_ATTACK_WON = [[
            * {agent} is dead. You got lucky that {contact} didn't hear all that noise and come back.
            * You get rid of any evidence that could lead to your misdeed and leave the area before anyone sees you.
        ]],
    }
    :Fn(function(cxt)
        local drone = Agent("COMBAT_DRONE")
        drone:JoinParty(cxt.quest:GetCastAgent("enemy"))
        cxt.quest:GetCastAgent("contact"):MoveToLocation(cxt.quest:GetCastMember("location"))
        cxt:Dialog("DIALOG_WAIT")
        cxt.quest:GetCastAgent("contact"):MoveToLimbo()
        cxt:Opt("OPT_CONFRONT")
            :Dialog("DIALOG_CONFRONT")
            :Negotiation{
                on_success = function(cxt)
                    cxt:Dialog("DIALOG_CONFRONT_WON")
                    cxt.quest:GetCastAgent("enemy"):Retire()
                    TheGame:GetGameState():GetPlayerAgent().graft_owner:RemoveSocialGraft(cxt.quest:GetCastAgent("enemy"))
                    StateGraphUtil.AddLeaveLocation(cxt)
                end,
                
                on_fail = function(cxt)
                    cxt:Dialog("DIALOG_CONFRONT_LOSS")
                    cxt:Opt("OPT_DEFEND_SELF")
                        :Battle{
                            flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_SURRENDER | BATTLE_FLAGS.SELF_DEFENCE,
                            on_win = function(cxt )
                                cxt:Dialog("DIALOG_ATTACK_WON")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        }
                end
            }
        cxt:Opt("OPT_ATTACK_ALONE")
            :Dialog("DIALOG_ATTACK_ALONE")
            :Battle{
                advantage = TEAM.BLUE,
                flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_SURRENDER,
                on_win = function(cxt) 
                        cxt:Dialog("DIALOG_ATTACK_WON")
                        StateGraphUtil.AddLeaveLocation(cxt)
                end,
            }
    end)
