local QDEF = QuestDef.Define{
    title = "The Gun Show",
    desc = "An Admiralty weaponsmith that Rook knows is here in the bog.",
    icon = engine.asset.Texture("icons/quests/oppo_gun_show.tex"),
    qtype = QTYPE.OPPORTUNITY,
    cooldown = EVENT_COOLDOWN.LONG,
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("weaponsmith"), location = quest:GetCastMember("location")} )
    end

}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("NICE_DIGS"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    desc = "",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    NICE_DIGS = {
        name = "Rented Room",
        map_tags = {"barracks"},
        plax = "INT_SB_Res_1",
    },
}

:AddCastByAlias{
    cast_id = "weaponsmith",
    alias = "WEAPONSMITH",
}

:AddOpinionEvents{
--[[    offered_coilgun =  
    {
        delta = OPINION_DELTAS.NEUTRAL,
        txt = "Was offered a gift.",
    },
--]]    
}

 QDEF:AddAttract("investigate", "weaponsmith")

QDEF:AddConvo("investigate", "weaponsmith")
    :ConfrontState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    {player}!?
                    !happy
                    They said you were dead!
                player:
                    !happy
                    They say a lot of things, {weaponsmith}. Some of them are even true!
                agent:
                    !point
                    But never the ones you want, am I right?
                    What are you doing here?
                player:
                    Working private security. Pays better than the Admiralty ever did, and I set my own hours.
                agent:
                    I'm here looking for freelance clients. Know anyone local who can pay top dollar for good weapons?
            ]],
            OPT_ASK_ABOUT_ADMIRALTY = "Ask about the Admiralty",
            DIALOG_ASK_ABOUT_ADMIRALTY = [[
                player:
                    !interest
                    I hope the Admiralty's still treating you well.
                agent:
                    !shrug
                    $neutralResigned
                    It is what it is. Never changes, does it?
                    You put in your time, bust the heads you're asked to, and at the end of the day you're lucky if all you get is scars.
                player:
                    I imagine that wouldn't do well on the recruitment pamphlets.
                agent:
                    It does even worse in the mess hall.    
            ]],

            OPT_ASK_ABOUT_BOG = "Ask why {agent} is in the Bog",
            DIALOG_ASK_ABOUT_BOG = [[
                player:
                    And why are <i>you</i> here? Unlike the Barons to allow outside influence.
                agent:
                    I'm on leave, actually. So I'm here in an unofficial capacity.
                player:
                    !dubious
                    $neutralDubious
                    Of all the places you could choose to spend your leave, you pick <i>Grout Bog</>?
                agent:
                    !shrug
                    It's a working holiday. 
                    I make weapons. There's a lot of demand for that here.
            ]],

            OPT_ASK_ABOUT_PISTOLS = "Ask {agent} to tweak your pistols",
            DIALOG_ASK_ABOUT_PISTOLS = [[
                player:
                    !fight
                    Remember these?
                agent:
                    !interest
                    You're still using those old things, eh?
                player:
                    !happy
                    Us old things have to stick together.
                agent:
                    I'm surprised they're still intact. 
                    I was practically a child when I built those. 
                player:
                    Think you could do better now?
                agent:
                    Certainly. With the right parts, and access to my tools. And about a week.
                player:
                    What could you do with five minutes and diminished expectations?
                    !give
                agent:
                    !take
                    I could make a couple of adjustments, I suppose...
            ]],

            OPT_INCREASE_CHARGES = "Increase charge chambers",
            DIALOG_INCREASE_CHARGES = [[
                player:
                    Could you increase the number of charges?
                agent:
                    !thought
                    Yeah, it's possible.
                    Tell you what. I'll add one for free. I can add two, but it'll cost you.
                player:
                    Always the mercenary, aren't you?
                agent:
                    A girl's gotta eat!                    
            ]],

            OPT_INCREASE_ONE = "Add one charge chamber",
            OPT_INCREASE_TWO = "Add two charge chambers",


            OPT_DECREASE_CHARGES = "Decrease charge chambers",
            DIALOG_DECREASE_CHARGES = [[
                player:
                    Could you decrease the number of charges?
                agent:
                    !thought
                    Yeah, that's easy.
                    I can take one off for free. I can take two off if you pay for the parts. 
            ]],
            OPT_DECREASE_ONE = "Remove one charge chamber",
            OPT_DECREASE_TWO = "Remove two charge chambers",

            OPT_BACK = "Ask for something else",
            DIALOG_BACK = [[
                player:
                    Not quite what I'm looking for.
            ]],

            DIALOG_DO_WORK = [[
                agent:
                    Alright, give me a minute.
                    !exit
                * {agent} opens up a toolkit and spends some time working on the pistols.
                agent:
                    !right
                    There. Good as new.
                    Better, even! 
                    !give
                player:
                    !take
                    Thanks, {agent}. This'll come in handy.
                agent:
                    It was for old time's sake.
            ]],

            OPT_KEEP_THE_SAME = "Take back your pistols",
            DIALOG_KEEP_THE_SAME = [[
                player:
                    Actually, I'm quite used to the way they handle now.
                agent:
                    Suit yourself. It's not like you were going to pay me, anyway.
            ]],

            OPT_SAY_GOODBYE = "Leave {agent}",
            DIALOG_LEAVE = [[
                player:
                    It's been nice seeing you, {agent}.
                agent:
                    Likewise, {player}. 
                    My leave's up, so I'm back to Murder Bay.
                    !happyGreeting
                    Don't get yourself killed out there!
                player:
                    It's usually the other people we have to worry about, anyway.
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest:Complete()
            cxt:Dialog("DIALOG_INTRO")
            local gun_graft = cxt.player.graft_owner:GetGrafts(GRAFT_TYPE.BLASTER)[1]
            local COST = 250

            cxt:RunLoop(function()
                cxt:Question("OPT_ASK_ABOUT_ADMIRALTY", "DIALOG_ASK_ABOUT_ADMIRALTY")
                cxt:Question("OPT_ASK_ABOUT_BOG", "DIALOG_ASK_ABOUT_BOG")

                if not cxt.quest.param.did_upgrade then
                    cxt:Opt("OPT_ASK_ABOUT_PISTOLS")
                        :Dialog("DIALOG_ASK_ABOUT_PISTOLS")
                        :LoopingFn(function() 
                            cxt:Opt("OPT_INCREASE_CHARGES")
                                :Dialog("DIALOG_INCREASE_CHARGES")
                                :Fn(function()  
                                    cxt:Opt("OPT_INCREASE_ONE")
                                        :Dialog("DIALOG_DO_WORK")
                                        :Fn(function() 
                                            cxt.quest.param.did_upgrade = true
                                            gun_graft.userdata.max_charges = 5
                                        end)
                                        :Pop()
                                    cxt:Opt("OPT_INCREASE_TWO")
                                        :DeliverMoney(COST, { is_shop = true })
                                        :Dialog("DIALOG_DO_WORK")
                                        :Fn(function() 
                                            cxt.quest.param.did_upgrade = true
                                            gun_graft.userdata.max_charges = 6
                                        end)
                                        :Pop()
                                    cxt:Opt("OPT_BACK")
                                        :Dialog("DIALOG_BACK")
                                end)
                            cxt:Opt("OPT_DECREASE_CHARGES")
                                :Dialog("DIALOG_DECREASE_CHARGES")
                                :Fn(function()  
                                    cxt:Opt("OPT_DECREASE_ONE")
                                        :Dialog("DIALOG_DO_WORK")
                                        :Fn(function() 
                                            cxt.quest.param.did_upgrade = true
                                            gun_graft.userdata.max_charges = 3
                                        end)
                                        :Pop()
                                    cxt:Opt("OPT_DECREASE_TWO")
                                        :DeliverMoney(COST, { is_shop = true })
                                        :Dialog("DIALOG_DO_WORK")
                                        :Fn(function() 
                                            cxt.quest.param.did_upgrade = true
                                            gun_graft.userdata.max_charges = 2
                                        end)
                                        :Pop()
                                    cxt:Opt("OPT_BACK")
                                        :Dialog("DIALOG_BACK")

                                end)
                            cxt:Opt("OPT_KEEP_THE_SAME")
                                :Dialog("DIALOG_KEEP_THE_SAME")
                                :Pop()
                        end)
                end

                cxt:Opt("OPT_SAY_GOODBYE")
                    :Dialog("DIALOG_LEAVE")
                    :Travel()
                
                    
            end)
        end)