local QDEF = QuestDef.Define{
    title = "Count Your Winnings",
    desc = "A mysterious gambler has been seen plying {gambler.hisher} trade in the Bog.",
    icon = engine.asset.Texture("icons/quests/event_count_your_winnings.tex"),
    qtype = QTYPE.OPPORTUNITY,
}

:AddLocationDefs{
    STREET_CORNER = {
        name = "Street Corner",
        plax = "INT_WorkCamp_Res",
        map_tags = {"quarters"},
    },
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"location"},
}

:AddCastByAlias{
    cast_id = "gambler",
    alias = "GAMBLER",
}


:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("STREET_CORNER"))
    end,
}

:AddOpinionEvents{

    demanded_money = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Took all of their money."
    },
    gave_money = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Gave them money when they were broke."
    },
    gave_warning =
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Didn't punish them for trying to grift you."
    },
    traded_coin =
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Taught them how to do sweet trick flips."
    },
}



QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
        :Loc{
            DIALOG_INTRO = [[
                * You see a kra'deshi trying to get the attention of the crowd.
                agent:
                    !right
                    Step right up! Test your luck!
                * Disinterested workers stream past {agent.himher}, too absorbed with their own troubles to take on {agent.hisher}.
                player:
                    !left
                    What are you selling here, friend?
                agent:
                    !surprised
                    Oh!
                    You startled me, grifter. Nobody ever stops to talk to me here in the Bog.
                    But, to business! Yes!
                    I am offering a moment's respite from the drudgery of daily life. 
                    
                * There's a hungry glint in {agent}'s eyes that you recognize from many a mirror. 
                agent:
                    Test your luck with my game of chance! You might just win!
                * The kid's fingers twitch nervously, anticipating the grift.
                *** {agent} is a gambler, and is offering to play a game with you.
            ]],

            OPT_ASK_ABOUT_GAME = "Ask about the game",
            DIALOG_ASK_ABOUT_GAME = [[
                player:
                    What game are you offering here?
                agent:
                    !neutral
                    The ancient and noble game of the kra'deshi kings and queens of old...
                    The storied pastime of pampered aristocrats and dashing rogues...
                    A contest of danger and wits and-
                player:
                    !dubious
                    You're going to toss that coin up in the air, and let me guess how it lands, right?
                agent:
                    !happy
                    Oh! You've played before! Excellent!
            ]],
            OPT_PLAY_THE_GAME = "Play the game",
            DIALOG_PLAY_THE_GAME = [[
                player:
                    Sounds good to me. Show me what you've got.
                agent:
                    !clap
                    Alright! Keep your eye on the coin.
                    Here we go...
                    !coin_toss
                    Call it!
                * Your practiced eye tells you that the coin is weighted to come up heads.
            ]],
            OPT_DECLINE = "Don't play",
            DIALOG_DECLINE = [[
                player:
                    I don't have time for this, sorry.
                    !exit
                agent:
                    !shrug
                    Nobody ever does.
            ]],
            
            OPT_HEADS = "Call 'Heads'",
            DIALOG_HEADS = [[
                player:
                    !point
                    Heads!
                agent:
                    Uh... yes. You win!
                    You're on a hot streak! You should go double or nothing!
            ]],
            OPT_SNAILS = "Call 'Snails'",
            DIALOG_SNAILS = [[
                player:
                    !thought
                    Snails.
                agent:
                    Oh, too bad! It's heads!
                    !shrug
                    Can't win them all, friend. 
            ]],
        }
        
        :Fn(function(cxt)
            cxt.quest:Complete()
            cxt.quest:GetCastMember('gambler'):MoveToLocation(cxt.location)
            cxt:TalkTo( cxt.quest:GetCastMember('gambler') )
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                cxt:Opt("OPT_ASK_ABOUT_GAME")
                    :Dialog("DIALOG_ASK_ABOUT_GAME")

                cxt:Opt("OPT_PLAY_THE_GAME")
                    :Dialog("DIALOG_PLAY_THE_GAME")
                    :DeliverMoney(10, {no_scale = true})                    
                    :Fn(function() 
                        cxt:Opt("OPT_HEADS")
                            :Dialog("DIALOG_HEADS")
                            :GoTo("STATE_AFTER")
                        cxt:Opt("OPT_SNAILS")
                            :Dialog("DIALOG_SNAILS")
                            :GoTo("STATE_AFTER")
                    end)

                cxt:Opt("OPT_DECLINE")
                    :Dialog("DIALOG_DECLINE")
                    :Travel()
            end)
            

        end)
    :State("STATE_AFTER")
        :Loc{
            OPT_SHOW_OFF_YOUR_SKILLS = "Show {agent} how it's done",

            DIALOG_SHOW_OFF_YOUR_SKILLS = [[
                player:
                    Your technique is good, but not great. 
                    Observe:
                    
                    !coin_toss
                    Heads.
                    
                    !coin_toss
                    Snails.
                    
                    !coin_toss
                    Heads.

                agent:
                    !scared
                player:
                    You can keep it up all day, if you use your wrist properly.
                agent:
                    Listen, man, I don't want any trouble. 
                    I'm just trying to make enough money to get back to Murder Bay.

            ]],
            
            OPT_TRADE_COIN = "Offer to trade {1#graft} for {2#graft}",
            DIALOG_TRADE_COIN = [[
                player:
                    !placate
                    Tell you what...
                    Trade coins with me and we'll call it even. The weight on that one's getting in your way.
                    Practice flipping with your wrist, and you'll earn your scratch in no-time.
                agent:
                    !happy
                    Really? Thanks, grifter. 
                    I was afraid you were going to tear me a new one for a moment there.
                    !give
                player:
                    !take
                    I'll only do that if you try to cheat me again.
                agent:
                    !exit
                * {agent} scuttles off, practicing with {agent.hisher} new coin.
            ]],
            OPT_GIVE_MONEY = "Give {agent} some money",
            DIALOG_GIVE_MONEY = [[
                player: 
                    Here. A donation.
                    !give
                agent:
                    !take
                    !happy
                    Wow! Really? Why?
                player:
                    Don't make me think too hard about it, or I might change my mind. Now be more careful with your flips, {agent}.
                agent:
                    Will do, sir!
                    !exit
                * {agent} scuttles off, pockets full of fresh new shills.
            ]],
            OPT_LET_OFF_WITH_A_WARNING = "Let {agent} off with a warning",
            DIALOG_LET_OFF_WITH_A_WARNING = [[
                player:
                    I'll let you go, this time.
                    But don't try grifting out of your league again, or you might find yourself with fewer limbs.
                agent:
                    !happy
                    Y-yes sir! Thank you!
                    !exit
                * {agent} scuttles off, nervously glancing over {agent.hisher} shoulder at you.
            ]],
            OPT_DEMAND_ALL_MONEY = "Demand all of {agent}'s money",
            DIALOG_DEMAND_ALL_MONEY = [[
                player:
                    You got caught trying to out-grift a grifter.
                    !threaten
                    Now you're going to pay up. Empty your pockets.
                    !neutral
                agent:
                    !scared
                    But- but that's all I have? 
                    !neutral
                    I- fine. Take it.
                    !give
                * {agent} presses a few measly shills into your hand.
            ]],
            DIALOG_DEMAND_ALL_MONEY_2 = [[
                player:
                    I'm teaching you an important lesson, here!
                agent:
                    !shrug
                    OK, grifter.
                    !exit
                * {agent} melts back into the crowd.
            ]],
            TT_TRADE_COIN = "Trade {1#graft} for {2#graft}",
        }
        :Fn(function(cxt) 
            cxt:Opt("OPT_SHOW_OFF_YOUR_SKILLS")
                :Dialog("DIALOG_SHOW_OFF_YOUR_SKILLS")
                :Fn(function() 
                    
                    local graft_owner = cxt.player.graft_owner
                    local existing_coin = graft_owner:FindGraft(function(graft) return graft:GetDef().type == GRAFT_TYPE.COIN end)

                    cxt:Opt("OPT_TRADE_COIN", existing_coin, "krogs_weighted_coin")
                        :Dialog("DIALOG_TRADE_COIN")
                        :Fn(function() 
                            graft_owner:ReplaceGraft( existing_coin, GraftInstance("krogs_weighted_coin") )
                        end)
                        :ReceiveOpinion("traded_coin")
                        :Travel()

                    cxt:Opt("OPT_LET_OFF_WITH_A_WARNING")
                        :Dialog("DIALOG_LET_OFF_WITH_A_WARNING")
                        :ReceiveOpinion("gave_warning")
                        :Travel()
                        
                    cxt:Opt("OPT_GIVE_MONEY")
                        :DeliverMoney(50, {no_scale =true})
                        :Dialog("DIALOG_GIVE_MONEY")
                        :ReceiveOpinion("gave_money")
                        :Travel()

                    cxt:Opt("OPT_DEMAND_ALL_MONEY")
                        :Dialog("DIALOG_DEMAND_ALL_MONEY")
                        :ReceiveMoney(45, {no_scale =true})
                        :Dialog("DIALOG_DEMAND_ALL_MONEY_2")
                        :ReceiveOpinion("demanded_money")
                        :Travel()

                end)
        end)

