local QDEF = QuestDef.Define{
    title = "Quiet Clearing",
    icon = engine.asset.Texture("icons/quests/oppo_quiet_meditation_spot.tex"),
    desc = "The tide has revealed the path to a peaceful clearing in the bog.",
    qtype = QTYPE.OPPORTUNITY,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("QUIET_SPOT"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    QUIET_SPOT = {
        name = "Quiet Clearing",
        map_tags = {"forest"},
        plax = "EXT_Bog_Forest_01",
        indoors = false,
    },
}


QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            * You follow the path until you get to the clearing.
            * You take a seat on a soft patch of grass, taking in the soft sounds of the critters in the bog.
        ]],
        
        OPT_REGAIN_LITTLE = "Take a short nap",
        
        DIALOG_REGAIN_LITTLE = [[
            * You lie down, close your eyes and quickly drift off to sleep.
        ]],
        DIALOG_REGAIN_LITTLE_2 = [[
            * When you wake, you feel refreshed and ready to head out.
        ]],        
        OPT_REGAIN_BIG = "Take a long nap",
        
        DIALOG_REGAIN_BIG = [[
            * You bunch up a small mound of moss to use as a pillow and find a comfortable position to drift off in.
        ]],
        DIALOG_REGAIN_BIG_2 = [[
            * When you finally wake up, you feel completely refreshed but also a bit sore from lying on the ground.
        ]],
        OPT_REGAIN_BIG_TT = "This will hurt you for <#HEALTH>{1} HP</>",
     
        REQ_WOULD_KILL_YOU = "This would kill you!",
        DIALOG_LEAVE = [[
            * You decide that you're too busy for a nap and head back the way you came.
        ]],
        OPT_INCREASE_PERMANENT = "Meditate",
        TT_INCREASE_PERMANENT = "This will increase your max resolve by <#BONUS>{1}</>",
        DIALOG_INCREASE_PERMANENT = [[
            * You've heard that meditation is healthy, but you've never really bought into it.
            * You close your eyes and try to focus your mind on the sounds around you.
            * Eventually you fall into a deep trance that seems to broaden your mind.
        ]],
        DIALOG_MAX_CHANGE = "Gained <#RESOLVE>{1} Max Resolve</>",

    }
    :Fn(function(cxt)
        cxt:Dialog("DIALOG_INTRO")

        local little_pts = 10 

        local damage = 5
        local big_pts = 25

        local cur, max = TheGame:GetGameState():GetCaravan():GetResolve()

        cxt:Opt("OPT_REGAIN_LITTLE", little_pts)
            :PreIcon( global_images.restoreresolvelittle )
            :Dialog("DIALOG_REGAIN_LITTLE")
            :DeltaResolve(little_pts)
            :Dialog("DIALOG_REGAIN_LITTLE_2")
            :CompleteQuest()
            :Travel()
            
        cxt:Opt("OPT_REGAIN_BIG", big_pts)
            :PreIcon( global_images.restoreresolve )
            :Dialog("DIALOG_REGAIN_BIG")
            :DeltaResolve(big_pts)
            :Dialog("DIALOG_REGAIN_BIG_2")
            :DeltaHealth(-damage)
            :CompleteQuest()
            :Travel()

        local inc_amt = 5
        local inc_cost = 15

        cxt:Opt("OPT_INCREASE_PERMANENT")
            --:PreIcon( global_images.restoreresolvelittle )
            :PostText("TT_INCREASE_PERMANENT", inc_amt)
            :Dialog("DIALOG_INCREASE_PERMANENT")
            :ReqCondition(cur >= inc_cost)
            :DeltaResolve(-inc_cost)            
            :Dialog("DIALOG_MAX_CHANGE", inc_amt)
            :Fn(function() TheGame:GetGameState():GetCaravan():UpgradeResolve(inc_amt, false, true) end)
            :CompleteQuest()
            :Travel()

        cxt:Opt("OPT_LEAVE"):MakeUnder()
            :Dialog("DIALOG_LEAVE")
            :PreIcon( global_images.close)
            :CompleteQuest()
            :Travel()
    end)
