local minor_items = {"combat_gauze", "salve", "healing_vapors", "tincture", "vial_of_slurry"}
local major_items = {"combat_drone", "vroc_whistle", "lumin_grenade"}

local MAJOR_ITEM_CHANCE = .6


local QDEF = QuestDef.Define{
    title = "Desolate Battlefield",
    qtype = QTYPE.OPPORTUNITY,
    on_init = function( quest )
        quest:SetParameter( "minor", minor_items[math.random(#minor_items)] )
        quest:SetParameter( "major", major_items[math.random(#major_items)] )
        quest:SetParameter( "graft", RewardUtil.GetGrafts(quest:GetRank(), 1, TheGame:GetGameState():GetPlayerAgent())[1] )
    end,
    on_post_load = function( quest )
        if not quest.param.minor or not quest.param.major or not quest.param.graft then
            quest:GetQuestDef().on_init( quest )
        end
    end,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("BATTLEFIELD"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Search the battlefield",
    desc = "Looks like the remains of a recent skirmish. Perhaps there's still something to loot.",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    BATTLEFIELD = {
        name = "Battlefield",
        map_tags = {"forest"},
        plax = "EXT_Battlefield_1",
        indoors = false,
    },
}


QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            player:
                !left
            * You come across the signs of a recent battle. If there were any winners, they weren't the type to loot the battlefield.
            * The bodies are still fresh. Best to grab something and run, before any beasts are drawn to the bloodshed. 
        ]],
        OPT_GATHER_UP_LOOT_MINOR = "Grab {1#card}",
        TT_MINOR = "You can do this quickly",
        DIALOG_GATHER_LOOT_MINOR = [[
            * You pick up {1#card} and make tracks before you become a part of the buffet.
        ]],
        OPT_GATHER_UP_LOOT_MAJOR = "Search the crates for {1#card}",
        TT_MAJOR = "This will take some time",
        DIALOG_GATHER_LOOT_MAJOR_ATTACKED =[[
            * You struggle with the crate, but you're soon joined by the first of the scavengers.
            agent:
                !right
        ]],
        DIALOG_GATHER_LOOT_MAJOR_SAFE = [[
            * You struggle with the crate, and the lid eventually pops off. Sure enough, you find what you're looking for, and make tracks while you still can. 
        ]],
        OPT_TAKE_GRAFT = "Pull {1#graft} out of the mud",
        TT_GRAFT = "This will make you vulnerable",
        DIALOG_TAKE_GRAFT = [[
            * You wrestle with the loot lodged in the mud even as a mournful howl cuts through the air. It isn't long before you're joined by the pack. 
            agent:
                !right
        ]],
        DIALOG_POST_FIGHT = [[
            agent:
                !exit
            * The bodies are growing ripe, and more scavengers are sure to be on their way. Best to cut while you're ahead. 
        ]],
        OPT_LEAVE = "Leave empty handed",
        DIALOG_LEAVE = [[
            * Rather than take any risks, you leave the battlefield behind.
        ]],
        OPT_DEFEND_SELF = "Defend yourself",
    }
    :Fn(function(cxt)
        if cxt:FirstLoop() then
            cxt.quest:Complete()
            cxt:Dialog("DIALOG_INTRO")
        end

        cxt:Opt("OPT_GATHER_UP_LOOT_MINOR", cxt.quest.param.minor)
            :PreIcon(global_images.item)
            :PostText("TT_MINOR")
            :Dialog("DIALOG_GATHER_LOOT_MINOR", cxt.quest.param.minor)
            :Fn(function(cxt)
                AgentUtil.LearnCard(cxt.quest.param.minor)
                StateGraphUtil.AddLeaveLocation(cxt)
            end)

        cxt:Opt("OPT_GATHER_UP_LOOT_MAJOR", cxt.quest.param.major)
            :PreIcon(global_images.item)
            :PostText("TT_MAJOR", cxt.quest.param.major)
            :Fn(function(cxt)
                AgentUtil.LearnCard(cxt.quest.param.major)
                if math.random() < MAJOR_ITEM_CHANCE then
                    local animals = CreateCombatParty("MONSTERS", cxt.quest:GetRank() - 1, cxt.location)
                    cxt.enc:SetPrimaryCast(animals[1])
                    cxt:Dialog("DIALOG_GATHER_LOOT_MAJOR_ATTACKED")
                    cxt:Opt("OPT_DEFEND_SELF")
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            on_win = function()
                                cxt:Dialog("DIALOG_POST_FIGHT")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end,
                        }
                else
                    cxt:Dialog("DIALOG_GATHER_LOOT_MAJOR_SAFE")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            end)

        if not cxt.player.graft_owner:CanInstallGraft( cxt.quest.param.graft.id ) or cxt.player.graft_owner:HasGraft( cxt.quest.param.graft.id ) then
            print( "Re-assigning invalid graft", cxt.quest.param.graft )
            cxt.quest:SetParameter( "graft", RewardUtil.GetGrafts(cxt.quest:GetRank(), 1, cxt.player)[1] )
        end

        cxt:Opt("OPT_TAKE_GRAFT", cxt.quest.param.graft)
            :PreIcon(global_images.item)
            :PostText("TT_GRAFT", cxt.quest.param.graft)
            :Fn(function(cxt)
                ConvoUtil.GiveGraft(cxt, cxt.quest.param.graft)
                local animals = CreateCombatParty("MONSTERS", cxt.quest:GetRank() + 1, cxt.location)
                cxt.enc:SetPrimaryCast(animals[1])
                cxt:Dialog("DIALOG_TAKE_GRAFT")
                cxt:Opt("OPT_DEFEND_SELF")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function()
                            cxt:Dialog("DIALOG_POST_FIGHT")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }
            end)

        cxt:Opt("OPT_LEAVE")
            :Dialog("DIALOG_LEAVE")
            :Travel()
    end)
