local PLAY_HEALTH_RECOVERY = 1
local PLAY_RESOLVE_AMOUNT = 10
local PET_SUPPORT_UNUPGRADED = {"untrained_pet"}

local QDEF = QuestDef.Define{
    title = "Treat your pet",
    icon = engine.asset.Texture("icons/quests/oppo_dog_event.tex"),
    qtype = QTYPE.OPPORTUNITY,
    precondition = function(quest) 
        if TheGame:GetGameState():GetCaravan():GetPet() ~= nil then
            local pets = TheGame:GetGameState():GetCaravan():GetPets()
            for k,pet in pairs(pets) do
                if table.contains(PET_SUPPORT_UNUPGRADED, pet.support_negotiation) then
                    return true
                end
            end
            return false

        else
            return false
        end
    end,
    events = 
    {
        caravan_member_remove = function(quest, agent) 
            if not quest:GetQuestDef().precondition( quest ) then
                quest:Cancel()
            end
        end,
    }    
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("FOREST_LOCATION"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Treat your pet",
    desc = "There's a perfect spot for grooming or training your pet, if you can get there before someone else stakes a claim to it.",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    FOREST_LOCATION = {
        name = "Clearing in the Woods",
        map_tags = {"forest"},
        plax = "EXT_Forest_1",
        indoors = false,
    },
}

QDEF:AddConvo():ConfrontState("CONFRONT",
    function(cxt)
        return cxt.location == cxt.quest:GetCastMember("location")
    end):
    Loc{
        DIALOG_INTRO = [[
            * You arrive at a clearing in the woods that isn't too overgrown with wild brush, and {agent} takes off to roll around in the grass.
            Agent:
                !right
            player:
                !left
                !happy
                Heh. Cute.
        ]], 

        OPT_PLAY_WITH_PET = "Play with {agent}",
        TT_PLAY = "Fully heal {agent}",
        DIALOG_PLAY_WITH_PET = [[
            player:
                !exit
            Agent:
                !exit
            * {agent} saunters up to you, a ragged stick in its mouth. You throw it a couple of times, and {agent} dashes in pursuit.            
            player:
                !left
                !interest
                $miscMischievous
                What's that you got there?
            agent:
                !right
            * {agent} drops it at your feet. It appears to be a ragged old femur. A strip of clothing is still attached to it.
            player:
                !surprised
                $neutralTakenAback
                Oh. Well... fetch!
            agent:
                !exit
            * You toss the bone across the clearing, and {agent} bounds after it in delight. 
            Agent:
                !right
                Yap!
            * {agent} regained {1} health.
        ]],
        OPT_TRAIN_PET = "Train with {agent}",
        TT_TRAIN = "Increases {agent}'s health and damage. {agent} will also help you in negotiations once trained.",
        DIALOG_TRAIN_PET = [[
            player:
                !exit
            agent:
                !exit
            * You put together an impromptu obstacle course from branches and dead trees, leading to a dummy made from putrid mulch.
            player:
                !left
                !point
                $neutralDirect
                Up and over! That's right. 
                !placate
                $miscHushedUrgent
                Low to the ground now, real quiet—
                !cruel
                $angryHostile
                Now <i>sic</i>!
            agent:
                !right
                Grr!
            * {agent}'s max health increased by {1}, base damage increased.
        ]],

        OPT_CALL_PET = "Call {1#agent}",
        DIALOG_CALL_PET = [[
            * You call to {1#agent}, who bounds over and sits eagerly at attention.
        ]],

        REQ_ALREADY_TRAINED = "{agent} is already trained.",

    }:SetLooping( true ):Fn(function( cxt )
        
        if cxt:FirstLoop() then
            local pet = cxt.caravan:GetPet()
            if pet then
                cxt.enc:SetPrimaryCast( pet )
                AUDIO:PlayPetSounds( pet, "AFFIRMATIVE_YAP")
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete()
            else
                cxt:End()
                cxt.quest:Complete()
                return
            end
        end

        local pet = cxt:GetAgent()
        local hp, max_hp = pet:GetAspect("health"):Get()
        local hp_gain = math.ceil( PLAY_HEALTH_RECOVERY * max_hp )

        cxt:Opt("OPT_PLAY_WITH_PET")
            :Fn(function(cxt) AUDIO:PlayPetSounds(pet, "EXCITED_BORK") end)
            :Dialog("DIALOG_PLAY_WITH_PET", hp_gain, PLAY_RESOLVE_AMOUNT)
            :DeltaResolve(PLAY_RESOLVE_AMOUNT)
            :PostText("TT_PLAY")
            :Fn(function(cxt)
                pet:GetAspect("health"):Restore( 1.0 )
                StateGraphUtil.AddLeaveLocation(cxt)
            end)

        cxt:Opt("OPT_TRAIN_PET")
            :ReqCondition( table.contains(PET_SUPPORT_UNUPGRADED, pet.support_negotiation), "REQ_ALREADY_TRAINED" )
            :Fn(function(cxt)
                AUDIO:PlayPetSounds(pet, "EXCITED_BORK")
                pet:TrainPet()
            end)
            :Dialog("DIALOG_TRAIN_PET", 10)
            :Fn(function(cxt) AUDIO:PlayPetSounds(pet, "ANGRY_GROWL") end)
            :PostText("TT_TRAIN")
            :Fn(function(cxt)
                StateGraphUtil.AddLeaveLocation(cxt)
            end)

        for i, other_pet in ipairs( cxt.caravan:GetPets() ) do
            if other_pet ~= pet then
                --
                cxt:Opt( "OPT_CALL_PET", other_pet ):
                    Dialog( "DIALOG_CALL_PET", other_pet )
                    :Fn( function( cxt )
                        cxt.encounter:SetPrimaryCast( other_pet )
                        cxt.encounter:PresentRight( other_pet )
                    end )
                end
        end

    end )

