
local QDEF = QuestDef.Define{
    title = "Highway Robbery",
    icon = engine.asset.Texture("icons/quests/oppo_highway_robbery.tex"),
    qtype = QTYPE.OPPORTUNITY,
}

:AddOpinionEvents{

    did_not_help_rob =  
    {
        delta = OPINION_DELTAS.DIMINISH,
        txt = "Passed up a robbery opportunity",
    },
    gave_big_share =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Gave them a bigger cut of the loot",
    },
    took_all_loot =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Stole their share of the loot",
    },
}


:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("ROAD_LOCATION"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Highway robbery",
    desc = "Your friend {friend} has found a juicy target to rob.",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    ROAD_LOCATION = {
        name = "Highway Site",
        map_tags = {"road"},
        plax = "EXT_Road01",
    },
}

:AddCast{
    cast_id = "friend",
    condition = function( agent, quest )
        return agent:GetRelationship() >= RELATIONSHIP.LIKED and agent:GetLocation() ~= TheGame:GetGameState():GetPlayerAgent():GetLocation() and agent:GetFactionID() == "BANDITS"
    end,

    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}

QDEF:AddCast{
    cast_id = "target",
    condition = function(agent, quest)
        return agent:GetContentID() == "FOREMAN" and agent:GetRelationship() <= RELATIONSHIP.NEUTRAL
    end,
}

:AddCastFallback
{
    cast_fn = function(quest, t)
        table.insert(t, quest:CreateSkinnedAgent("FOREMAN"))
    end,
}



QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
        :Loc{
            DIALOG_INTRO = [[
                friend:
                    !right
                    !greeting
                    {player}! You want work?
                player:
                    !left
                friend:
                    !cagey
                    $miscHushedUrgent
                    A little birdie told me that a nice, fat target is headed this way, with enough shills to line both our pockets.
                    Better still: apparently the idiot cheaped out on guards.
                player:
                    !interest
                    Sounds like easy pickings. Why loop me in?
                friend:
                    !dubious
                    $neutralDubious
                    Uh, 'cause you're my pal? What kinda question is that.
                    $miscAlluring
                    Besides, you got a way with words. And if it comes to blows, you got a way with your fists, too.
                    $miscPersuasive
                    !permit
                    Whaddya say?   
                
            ]],
            OPT_ACCEPT = "Agree to the plan",
            OPT_DECLINE = "Back out",
            DIALOG_DECLINE = [[
                player:
                    !shrug
                    $neutralDubious
                    Sorry, pal. I gotta be in the <i>mood</i> for highway robbery, you know?
                friend:
                    !crossed
                    $angryReluctant
                    Huh. And here I thought you never passed up a good profit. Guess I was wrong.
                    !overthere
                    $miscMocking
                    Then if you don't mind, please kindly scram. I still gotta make a living even if you don't. 
                * You leave {friend} lurking in the shadows, waiting for {friend.hisher} target.
            ]],
            DIALOG_ACCEPT = [[
                player:
                    !handwring
                    $happyDefault
                    Hesh yeah. I love mixin' business with pleasure.
                friend:
                    !happy
                    $happyDefault
                    Right? I'll follow your lead. 
                    !exit
                * {friend} makes for the shadows at the edge of the road, and you quickly follow.
                player:
                    !exit
            ]],
            DIALOG_TARGET_APPEARS = [[
                * The target doesn't look too rich, though—despite what {friend} said, {target.heshe}'s nothing more than bedraggled.
                * All the same, you step onto the path once you have {target.himher} where you want {target.himher}.
                player:
                    !left
                    !cruel
                    Hold it, there, friend.
                target:
                    !right
                    !scared
                    W-what is this?
            ]],
            OPT_SCARE = "Convince {target} to give up {target.hisher} money",
            OPT_ATTACK = "Attack {target}",
            OPT_LET_PASS = "Let {target} pass",
            DIALOG_LET_PASS = [[
                player:
                    Oh, uh...
                * You look over {target}, quivering in {target.hisher} hole-ridden boots. 
                player:
                    !bashful
                    Hm. My bad—I thought you were someone else.
                target:
                    !permit
                    Sure. Easy mistake to make. 
                    !exit
                * {target} hurries down the road, casting nervous glances over {target.hisher} shoulder.
                friend:
                    !right
                    !angry_shrug
                    $angryDefault
                    What the Hesh, {player}?!
                player:
                    !dubious
                    $neutralDubious
                    Did that person look rich to you?
                friend:
                    !disappoint
                    I told you they were <i>cheap</i>! If rags is all it takes to keep goons off my back, I'd wear 'em, too.
                    $angryReluctant
                    And you fell for it like a soft-boiled egg. Pathetic.
                    !exit
                * {friend} skulks into the bush, likely to catch up to their prey. But at least you didn't waste your time on the poor and down-trodden.
                    !exit
            ]],
            DIALOG_SCARE = [[
                player:
                    !salute
                    $happyDefault
                    Hail, citizen. We're just some friendly tax-collectors, lookin' for contributions to road maintenance.
                    $miscAlluring
                    You sick of all this mud? Well, we're here to change that.
                target:
                    !dubious
                    $scaredDefault
                    You're going to get rid of the mud?
                player:
                    !cruel
                    One squelch at a time. It'll only cost... approximately all your money.
            ]],
            DIALOG_SCARE_SUCCESS = [[
                target:
                    !scared
                player:
                    !cruel
                    $angryCruel
                    So whaddya say, helpful citizen? Fancy contributing to the cause?
                    !throatcut
                    $angryCruel
                    Because if not... who's to say the mud won't swallow you up. And that'd be a damn shame.
                target:
                    !scared
                    $scaredDefault
                    H-hey, I get it. I'm no fool. Enough's been said. 
                    !give
                    $scaredDefault
                    Take it. And I'll be on my way. Assuming... assuming that's the deal.
                player:
                    $miscPersuasive
                    !take
                    Why of <i>course</i>. I'd never impede someone so helpful and accomodating as yourself.
                    !overthere
                    Off with you, now. And watch out for mud!
                target:
                    !exit
                * {target} makes a hasty exit.
                friend:
                    !right
                    !happy
                    $happyDefault
                    Damn, {player}. That was <i>way</i> easier than stabbing them in the eye. 

            ]],
            DIALOG_SCARE_FAIL = [[
                player:
                    !point
                    And if you don't pay up, well, I'll just have to hurt you. 
                target:
                    !angry
                    $angryHostile
                    There it is—the threats. I knew those were coming.
                    Forget it, chump. I'm not going to respond to some no-good thug looking for an easy shill.
            ]],
            DIALOG_LET_PASS_AFTER_NEGOTIATION_FAIL = [[
                player:
                    !palm
                    All right. Fine. Just go. 
                target:
                    !crossed
                    $angryDefault
                    That's what I thought!
                    !exit
                * {target} marches down the road away from you, dignity intact. Yours, on the other hand...
                friend:
                    !right
                    !interest
                    Huh. {target.HeShe} was a feisty one! But probably not worth the trouble.
                    !thought
                    I think I might've been misled. {target.HeShe} didn't <i>look</i> very rich, huh?
                target:
                    !sigh
                    Nope. 
                friend:
                    !greeting
                    Ah well. We'll laugh about this later. Be seeing you, {player}.
                    !exit
                * {friend} leaves you contemplate the choices that brought you to this moment. 
            ]],
            DIALOG_ATTACK_AFTER_NEGOTIATION_FAIL = [[
                player:
                    !fight
                    Words failed. So I guess we fall back to trusty, reliable fists.
                target:
                    !fight
            ]],

            DIALOG_ATTACK = [[
                player:
                    !cruel
                    Just wondering—if I punch you, will shills pop out? 
                    !fight
                    Guess we'll find out!
                target:
                    !fight
            ]],
        }
        
        :Fn(function(cxt)
            cxt.quest:Complete()
            cxt.quest.param.your_share = round_zero( 100 + cxt.quest:GetRank()*20, 5)

            cxt.quest:GetCastMember("friend"):MoveToLocation(cxt.location)
            cxt.quest:GetCastMember("target"):MoveToLocation(cxt.location)
            
            
            cxt.quest.param.goons = CreateCombatBackup(cxt.quest:GetCastMember("target"), "MERCENARY_BACKUP", cxt.quest:GetRank() )
            
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("friend"))
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_ACCEPT")
                :Dialog("DIALOG_ACCEPT")
                :Fn(function() 
                    cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("target"))
                    cxt:Dialog("DIALOG_TARGET_APPEARS")
                    cxt:Opt("OPT_SCARE")
                        :Dialog("DIALOG_SCARE")
                        :Negotiation{
                            flags = NEGOTIATION_FLAGS.INTIMIDATION,                            
                            on_success = function() 
                                cxt:Dialog("DIALOG_SCARE_SUCCESS")
                                cxt:GoTo("STATE_TAKE_MONEY")
                            end,

                            on_fail = function()
                                cxt:Dialog("DIALOG_SCARE_FAIL")

                                cxt:Opt("OPT_ATTACK")
                                    :Dialog("DIALOG_ATTACK_AFTER_NEGOTIATION_FAIL")
                                    :Battle{
                                        allies = {cxt.quest:GetCastMember("friend")},
                                        on_win = function() cxt:GoTo("STATE_POST_BATTLE") end,
                                    }

                                cxt:Opt("OPT_LET_PASS")
                                    :Dialog("DIALOG_LET_PASS_AFTER_NEGOTIATION_FAIL")
                                    :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)
                                
                            end,
                        }

                    cxt:Opt("OPT_ATTACK")
                        :ReceiveOpinion(OPINION.ROBBED)
                        :Dialog("DIALOG_ATTACK")
                        :Battle{
                            allies = {cxt.quest:GetCastMember("friend")},
                            on_win = function() cxt:GoTo("STATE_POST_BATTLE") end,
                        }

                    cxt:Opt("OPT_LET_PASS")
                        :Dialog("DIALOG_LET_PASS")
                        :Fn(function() cxt.quest:GetCastMember("friend"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("did_not_help_rob")) end)
                        :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)

                end)
                
                

            cxt:Opt("OPT_DECLINE")
                :Dialog("DIALOG_DECLINE")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("did_not_help_rob"))
                :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)


        end)

    :State("STATE_TAKE_MONEY")
        :Loc{
            OPT_TAKE_MONEY = "Take the loot",
            DIALOG_TAKE_MONEY = [[
                * You take all the money. At least it won't go to waste.
            ]],

            OPT_TAKE_SMALL_SHARE = "Take a small share",
            DIALOG_TAKE_SMALL_SHARE = [[
                friend:
                    !right
                    !bashful
                    $miscEmbarrassed
                    Now to count out the profits, huh?
                player:
                    !overthere
                    This was your mark. You take a bigger cut.
                friend:
                    !happy
                    Hesh, not every day you work with someone honest in this line of work.
                    Keep your head above water, {player}, 'cause it'd be a shame if you ever went under.
                    !exit
                * {friend} heads off, whistling a tune you almost recognize.
            ]],
            OPT_TAKE_FAIR_SHARE = "Take your fair share",
            DIALOG_FAIR_SHARE = [[
                friend:
                    !right
                    !interest
                    Well, on to the spoils. What d'we got?
                player:
                    !permit
                    $neutralDirect
                    The take is good. Let's split it fifty-fifty—you had the lead, but I did the legwork.
                friend:
                    !happy
                    $happyDefault
                    You got it, {player}. Always a pleasure workin' with you.
                    I'll see you on the burnt side, friend. 
                    !exit
                * {friend} heads down the road, whistling a tune you're sure is off-key. 
            ]],
            OPT_DEMAND_ALL = "Demand all of the loot",
            DIALOG_DEMAND_ALL = [[
                friend:
                    !right
                    !dubious
                    $neutralDubious
                    All right there, {player}? You got a look in your eye.
                player:
                    !crossed
                    $neutralDirect
                    Enh, it's just a look of greed. The money stays with me.
                friend:
                    !angry
                    $angryTakenAback
                    What?! You can't do that!
                player:
                    I sure can. Next time, don't leave me to do all the leg work. 
                * You head down the road, leaving {friend} sputtering behind you. That's probably the last time you'll work together, so at least you made it profitable.
                    !exit
            ]],
        }
        :Fn(function(cxt) 
            if cxt.quest:GetCastMember("friend"):IsDead() then
                cxt:Opt("OPT_TAKE_MONEY")
                    :ReceiveMoney(cxt.quest.param.your_share*2)
                    :Dialog("DIALOG_TAKE_MONEY")
                    :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)
            else
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("friend"))
                cxt:Opt("OPT_TAKE_FAIR_SHARE")
                    :ReceiveMoney(cxt.quest.param.your_share)
                    :Dialog("DIALOG_FAIR_SHARE")
                    :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)
                
                cxt:Opt("OPT_TAKE_SMALL_SHARE")
                    :ReceiveMoney(math.floor( cxt.quest.param.your_share*.25))
                    :Dialog("DIALOG_TAKE_SMALL_SHARE")
                    :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("gave_big_share"))
                    :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)
                
                cxt:Opt("OPT_DEMAND_ALL")
                    :ReceiveMoney(math.floor( cxt.quest.param.your_share*2))
                    :Dialog("DIALOG_DEMAND_ALL")
                    :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("took_all_loot"))
                    :Fn(function()  StateGraphUtil.AddLeaveLocation(cxt) end)

            end
        end)

    :State("STATE_POST_BATTLE")
        :Loc{
            DIALOG_FOUGHT_KILLED_FRIEND_ALIVE = [[
                * {target} is dead, leaving the job well and done.
                friend:
                    !right
                    !dubious
                    $neutralDubious
                    Enh, it gets results. I won't complain. 
                    But we should probably scatter before we draw any attention. 
                * {friend} ducks into the trees, making off with {friend.hisher} takings from the corpse.
            ]],
            DIALOG_FOUGHT_ALIVE_FRIEND_ALIVE = [[
                player:
                    !angry_threaten
                    $angryHostile
                    Still got some fight in you, bub, or are you ready to pay up?
                target:
                    !right
                    !injured
                    Take it! Take it all! Just leave me alive.
                * You gather up the riches while {target} clutches {target.hisher} wounds.
                player:
                    That'll do. Go on, then. Get out of here.
                target:
                    !exit
                * {target} makes for the bushes. {target.HeShe}'ll be lucky to make it to town before a vroc hunts {target.himher} down in that state.
            ]],
            DIALOG_POST_FIGHT_FRIEND_ALIVE = [[
                friend:
                    !right
                    !tired
                    $happyResigned
                    Nothing get the blood pumping like a good tussle, eh? Especially when there's a pay day attached.
            ]],
            DIALOG_FOUGHT_KILLED_FRIEND_DEAD = [[
                * Both {target} and {friend} are dead.
                * You gather your profits and head for the trees, before any law-abiding individual should happen by. 
            ]],

            DIALOG_FOUGHT_ALIVE_FRIEND_DEAD = [[
                player:
                    !angry_threaten
                    $angryHostile
                    Still got some fight in you, bub, or are you ready to pay up?
                target:
                    !right
                    !injured
                    Take it! Take it all! Just leave me alive.
                * You gather up the riches while {target} clutches {target.hisher} wounds.
                player:
                    That'll do. Go on, then. Get out of here.
                target:
                    !exit
                * {target} makes for the bushes. {target.HeShe}'ll be lucky to make it to town before a vroc hunts {target.himher} down in that state.
                * You take all of the money for yourself. {friend} doesn't seem to mind, on account of being dead.
            ]],

        }
        :Fn(function(cxt)
            if cxt.quest:GetCastMember("target"):IsDead() then
                if cxt.quest:GetCastMember("friend"):IsDead() then
                    cxt:Dialog("DIALOG_FOUGHT_KILLED_FRIEND_DEAD")
                else
                    cxt:Dialog("DIALOG_FOUGHT_KILLED_FRIEND_ALIVE")
                end
            else
                if cxt.quest:GetCastMember("friend"):IsDead() then
                    cxt:Dialog("DIALOG_FOUGHT_ALIVE_FRIEND_ALIVE")
                else
                    cxt:Dialog("DIALOG_FOUGHT_ALIVE_FRIEND_ALIVE")
                end
            end
            cxt:GoTo("STATE_TAKE_MONEY")
        end)
