local QDEF = QuestDef.Define{
    title = "Healing Spring",
    icon = engine.asset.Texture("icons/quests/oppo_healing_spring.tex"),
    desc = "A rare lumin fissure has opened in the bedrock here. Funny things happen when raw lumin vapors go untapped.",
    qtype = QTYPE.OPPORTUNITY,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("HEALING_SPRING"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    HEALING_SPRING = {
        name = "Healing Spring",
        map_tags = {"forest"},
        plax = "EXT_Forest_LuminFissure_1",
        indoors = false,
    },
}


QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            * You find a fissure in the ground, fuming with toxic vapors.
            * On the far side you spot a rare healing wort, grown fat with broad leaves under the lumin's influence.
        ]],
        OPT_HEAL_LITTLE = "Pick a small leaf to chew (Heals <#HEALTH>{1} HP</>)",
        DIALOG_HEAL_LITTLE = [[
            * You pick one of the leaves and nibble on it. It doesn't taste right, but you feel stronger all the same.
        ]],
        OPT_HEAL_BIG = "Pick one of the weird fruits to eat (Heals <#HEALTH>{1} HP</>, receive {2#card})",
        DIALOG_HEAL_BIG = [[
            * You pick one of the plump, black fruits and take a bite. The flesh inside glows brilliant green.
            * You feel stronger, but your stomach turns from the rancid sweet taste. 
        ]],
        OPT_HEAL_BIG_TT = "You manage to keep it down, but you're not sure if it was for the best.",

        OPT_INCREASE_MAX = "Eat until you can't eat any more (<#HEALTH>Max HP +{1}</>, take <#HEALTH>{2} HP</> damage)",
        DIALOG_INCREASE_MAX = [[
            * You pound back fruit after fruit, pausing only to devour mouthfuls of leaves.
            
        ]],
        DIALOG_INCREASE_MAX_2 = [[
            * You pass out, alien chemicals coursing through your system, and alien thoughts coursing through your head.
        ]],
        DIALOG_INCREASE_MAX_3 = [[
            * Max HP raised by <#HEALTH>{1}</>.
            player:
                !left
                !dust_off
            * You wake up in a puddle of fruit juice and your own filth.
        ]],

        DIALOG_DONE = [[
            * The plant shudders as if hurt by the harvest. To your amazement, it pulls up roots and slithers away into the bush. 
            player:
                !left
                !surprise
                Uh.
                !palm
            * You hope it was just a hallucination. 
        ]],
        DIALOG_DONE_NO_HEAL = [[
            * You decide against touching the luminized plant. The plant shudders as if stung by your rejection. To your amazement, it pulls up roots and slithers away into the bush. 
            player:
                !left
                !surprise
                Uh.
                !palm
            * You hope it was just a hallucination. 
        ]],

        REQ_HEALTH_TOO_LOW = "That would kill you!",
    }
    :Fn(function(cxt)
        cxt:Dialog("DIALOG_INTRO")

        local little_pts = 15
        local big_pts = 30
        
        local health, max_health = cxt.player.health:Get()
        local increase_pts = 10
        local hurt_pts = 15

        cxt:Opt("OPT_HEAL_LITTLE", little_pts)
            :PreIcon(global_images.heallittle)
            :Dialog("DIALOG_HEAL_LITTLE")
            :DeltaHealth(little_pts)
            :Dialog("DIALOG_DONE")
            :CompleteQuest()
            :Fn(function() StateGraphUtil.AddLeaveLocation(cxt) end)
            
        cxt:Opt("OPT_HEAL_BIG", big_pts, "drunk")
            :PreIcon(global_images.heal)
            :Dialog("DIALOG_HEAL_BIG")
            :PostText("OPT_HEAL_BIG_TT")
            :DeltaHealth(big_pts)
            :Fn(function()
                cxt:ForceTakeCards{"drunk"}
            end)
            :Dialog("DIALOG_DONE")
            :CompleteQuest()
            :Fn(function() StateGraphUtil.AddLeaveLocation(cxt) end)

        cxt:Opt("OPT_INCREASE_MAX", increase_pts, hurt_pts)
            :ReqCondition(health > hurt_pts, "REQ_HEALTH_TOO_LOW")
            :Dialog("DIALOG_INCREASE_MAX")
            :DeltaHealth(-hurt_pts)
            :Dialog("DIALOG_INCREASE_MAX_2")
            :FadeOut()
            :Fn(function() 
                cxt.player.health:AddStatModifier( "EVENT_HEALING_SPRING", increase_pts )
                -- Adding a stat modifier increases current health, so we have to remove it again
                cxt.player.health:Delta(-increase_pts)
            end)
            :Dialog("DIALOG_INCREASE_MAX_3", increase_pts)
            :FadeIn()
            :CompleteQuest()
            :Travel()


        cxt:Opt("OPT_LEAVE"):MakeUnder()
            :PreIcon( global_images.close)
            :CompleteQuest()
            :Dialog("DIALOG_DONE_NO_HEAL")
            :Travel()
    end)


