local QDEF = QuestDef.Define{
    title = "Meditation Spot",
    icon = engine.asset.Texture("icons/quests/oppo_quiet_meditation_spot.tex"),
    desc = "There's something special about this place. You could go here to gather your thoughts and strengthen your resolve.",
    qtype = QTYPE.OPPORTUNITY,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("QUIET_SPOT"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    QUIET_SPOT = {
        name = "Quiet Spot",
        map_tags = {"forest"},
        plax = "EXT_MeditationSpot_1",
        indoors = false,
    },
}


QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            *The woods fall silent. There is nobody around. You can take a moment to gather your thoughts.
        ]],
        
        OPT_REGAIN_LITTLE = "Meditate for a short while",
        
        DIALOG_REGAIN_LITTLE = [[
            * You consider the events of the last few days, and the challenges to come.
            * You're going to get through this!
        ]],
        
        OPT_REGAIN_BIG = "Meditate for a long while",
        
        DIALOG_REGAIN_BIG = [[
            * You slow your breathing, and fall into a deep trance.
            
        ]],
        DIALOG_REGAIN_BIG_2 = [[
            * When you're done, your back aches, but you have achieved a clarity you haven't felt in weeks.
        ]],
        OPT_REGAIN_BIG_TT = "This will hurt you for <#HEALTH>{1} HP</>",
     
        REQ_WOULD_KILL_YOU = "This would kill you!",
        DIALOG_LEAVE = [[
            * You decide to leave your thoughts safely locked up inside.
        ]],
        OPT_INCREASE_PERMANENT = "Meditate on the nature of meditation",
        TT_INCREASE_PERMANENT = "This will increase your max resolve by <#BONUS>{1}</>",
        DIALOG_INCREASE_PERMANENT = [[
            * You empty your mind, and ponder the nature of the void.
            * ...
            * You finish the session exhausted, but you learned a thing or two about your own insignificance.
        ]],
        DIALOG_MAX_CHANGE = "Gained <#RESOLVE>{1} Max Resolve</>",

    }
    :Fn(function(cxt)
        cxt:Dialog("DIALOG_INTRO")

        local little_pts = 10 

        local damage = 5
        local big_pts = 30

        local cur, max = TheGame:GetGameState():GetCaravan():GetResolve()

        cxt:Opt("OPT_REGAIN_LITTLE", little_pts)
            :PreIcon( global_images.restoreresolvelittle )
            :Dialog("DIALOG_REGAIN_LITTLE")
            :DeltaResolve(little_pts)
            :CompleteQuest()
            :Travel()
            
        cxt:Opt("OPT_REGAIN_BIG", big_pts)
            :PreIcon( global_images.restoreresolve )
            :Dialog("DIALOG_REGAIN_BIG")
            :DeltaResolve(big_pts)
            :Dialog("DIALOG_REGAIN_BIG_2")
            :DeltaHealth(-damage)
            :CompleteQuest()
            :Travel()

        local inc_amt = 5
        local inc_cost = 20

        cxt:Opt("OPT_INCREASE_PERMANENT")
            --:PreIcon( global_images.restoreresolvelittle )
            :PostText("TT_INCREASE_PERMANENT", inc_amt)
            :Dialog("DIALOG_INCREASE_PERMANENT")
            :ReqCondition(cur >= inc_cost)
            :DeltaResolve(-inc_cost)            
            :Dialog("DIALOG_MAX_CHANGE", inc_amt)
            :Fn(function() TheGame:GetGameState():GetCaravan():UpgradeResolve(inc_amt, false, true) end)
            :CompleteQuest()
            :Travel()

        cxt:Opt("OPT_LEAVE"):MakeUnder()
            :Dialog("DIALOG_LEAVE")
            :PreIcon( global_images.close)
            :CompleteQuest()
            :Travel()
    end)
