local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS

local QDEF = QuestDef.Define{
    title = "Comedy Club",
    icon = engine.asset.Texture("icons/quests/oppo_comedy_club.tex"),
    desc = "Sweet Moreef’s bar is hosting an open mic event. Good time to practice your charm.",
    qtype = QTYPE.OPPORTUNITY,
    precondition = function( quest )
        return TheGame:GetGameState():GetPlayerAgent():GetLocation() ~=  TheGame:GetGameState():GetLocation("MOREEF_BAR")
    end,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  TheGame:GetGameState():GetLocation("MOREEF_BAR"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"location"},
    terminal = true
}

:AddCast{
    cast_id = "rando",
    condition = function(agent, quest) 
        return agent:GetContentID() == "WEALTHY_MERCHANT" and agent:GetRelationship() == RELATIONSHIP.NEUTRAL
    end,
    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}:AddCastFallback{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "WEALTHY_MERCHANT" ) )
    end,
}

QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            * You enter the bar and see a small group of people sitting around a small stage where someone is telling jokes.
            player:
                !left
            agent:
                !right
                {player}! Here to give a little performance?
        ]],
        OPT_UPGRADE = "Tell some jokes",
        TT_UPGRADE = "Upgrade {1} random negotiation {1*card|cards}",
        DIALOG_UPGRADE = [[
            * You get up on stage and tell a couple of jokes, with varying success.
            ** {1#card_list} upgraded.
        ]],
        OPT_REMOVE = "Refine your skills",
        TT_REMOVE = "Remove a negotiation card",
        DIALOG_REMOVE = [[
            player:
                Uh, think you could give me some tips.
            agent:
                Of course, this is how I write most of my stuff...
            * {agent} pulls out a notebook and shows you {agent.hisher} process.
        ]],
        DIALOG_REMOVE_PST = [[
            agent:
                So you see, sometimes you just gotta know what to cut.
            player:
                Huh, thanks!
        ]],
        OPT_LEAVE = "Leave",
        DIALOG_LEAVE = [[
            player:
                Uhh, on second thought, I think I'll pass.
            agent:
                !shrug
                It's not for everyone, I guess.
            player:
                !exit
            agent:
                !exit
        ]],
        REQ_NEED_UPGRADABLE = "No cards to upgrade!",
    }
    :Fn(function(cxt)
        local upgrade_options = {}
        local maneuver_options = {}
        for i,card in ipairs(TheGame:GetGameState():GetPlayerAgent().negotiator:GetCards()) do
            if card:GetMaxXP() ~= nil and not card:UpgradeReady() and card:HasUpgrades() and not card:IsHatched() then
                table.insert(upgrade_options, card)
            end
        end
        cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("rando"))
        cxt:Dialog("DIALOG_INTRO")

        cxt:Opt("OPT_UPGRADE")
            :PostText("TT_UPGRADE", math.min(#upgrade_options, 2))
            :PreIcon( global_images.upgradenegotiation )
            :ReqCondition(#upgrade_options > 0, "REQ_NEED_UPGRADABLE")
            :Fn(function(cxt)
                local cards = table.multipick(upgrade_options, 2)
                for i,card in ipairs(cards) do
                    card:AddXP(card:GetMaxXP())
                end
                cxt:Dialog("DIALOG_UPGRADE", cards)
                cxt.quest:Complete()
                StateGraphUtil.AddLeaveLocation(cxt)
            end)
            
        cxt:Opt("OPT_REMOVE")
            :PostText("TT_REMOVE")
            :PreIcon( global_images.removenegotiation )
            :Dialog("DIALOG_REMOVE")
            :Fn(function(cxt)
                cxt:Wait()
                AgentUtil.RemoveNegotiationCard( cxt.player, function( card )
                    cxt.enc:ResumeEncounter( card )
                end)
                local card = cxt.enc:YieldEncounter()
                if card then
                    cxt:Dialog("DIALOG_REMOVE_PST")
                    cxt.quest:Complete()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            end)
            

        cxt:Opt("OPT_LEAVE"):MakeUnder()
            :Dialog("DIALOG_LEAVE")
            :PreIcon( global_images.close)
            :CompleteQuest()
            :Travel()
    end)