local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS

local QDEF = QuestDef.Define{
    title = "Fight Club",
    icon = engine.asset.Texture("icons/quests/oppo_fight_club.tex"),
    desc = "You've heard about a pop-up fighting ring around here. Could be a good place to hone your skills.",
    qtype = QTYPE.OPPORTUNITY,
    precondition = function(quest) 
        local attacks_ready = 0
        local maneuvers_ready = 0
        for i,card in ipairs(TheGame:GetGameState():GetPlayerAgent().battler:GetCards()) do
            if card:IsAttackCard() and card:GetMaxXP() ~= nil and not card:UpgradeReady() and card:HasUpgrades() and not card:IsHatched() then
                attacks_ready = attacks_ready + 1
            end
            if card:IsFlagged(CARD_FLAGS.SKILL) and card:GetMaxXP() ~= nil and not card:UpgradeReady() and card:HasUpgrades() and not card:IsHatched() then
                maneuvers_ready = maneuvers_ready + 1
            end
        end
        return attacks_ready >= 2 or maneuvers_ready >= 2
    end,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("FIGHTING_RING"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    FIGHTING_RING = {
        name = "Fighting Ring",
        plax = "EXT_Smith_Park",
        indoors = false,
    },
}

:AddCast{
    cast_id = "fighter",
    cast_fn = function(quest, t)
        local def = "JAKES_RUNNER"
        local agent = quest:CreateSkinnedAgent( def )
        table.insert( t, agent )
    end,
    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}

QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            * Before you see anybody, you can hear the sounds of people cheering and the occasional thud of punches landing.
            player:
                !left
            agent:
                !right
                Hey there! You up for a little scrap?
            player:
                Always!
        ]],
        OPT_PUNCHES = "Practice your punches",
        TT_PUNCHES = "Upgrade {1} random attack {1*card|cards}",
        DIALOG_PUNCHES = [[
            player:
                !fight
            agent:
                !fight
                Alright, give it your best shot!
            player:
                !exit
            agent:
                !exit
            * {agent} raises their arms and lets you take a few swings, ducking and weaving from your strikes.
            agent:
                !right
            player:
                !left
                !tired
        ]],
        OPT_BLOCKS = "Practice your blocks and dodges",
        TT_BLOCKS = "Upgrade {1} random maneuver {1*card|cards}",
        DIALOG_BLOCK = [[
            player:
                !fight
            agent:
                !fight
                Alright, get your guard up!
            player:
                !exit
            agent:
                !exit
            * You raise your hands and prepare to deflect {agent}'s punches.
            agent:
                !right
            player:
                !left
                !tired
        ]],
        DIALOG_UPGRADE = [[** {1#card_list} upgraded.]],

        OPT_LEAVE = "Leave",
        DIALOG_LEAVE = [[
            player:
                Uhh, on second thought, I think I'll pass.
            agent:
                !shrug
                Your loss, buddy.
            player:
                !exit
            agent:
                !exit
        ]],
    }
    :Fn(function(cxt)
        local attack_options = {}
        local maneuver_options = {}
        for i,card in ipairs(TheGame:GetGameState():GetPlayerAgent().battler:GetCards()) do
            if card:IsAttackCard() and card:GetMaxXP() ~= nil and not card:UpgradeReady() and card:HasUpgrades() then
                table.insert(attack_options, card)
            end
            if card:IsFlagged(CARD_FLAGS.SKILL) and card:GetMaxXP() ~= nil and not card:UpgradeReady() and card:HasUpgrades() then
                table.insert(maneuver_options, card)
            end
        end
        cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("fighter"))
        cxt:Dialog("DIALOG_INTRO")

        if #attack_options > 0 then
            cxt:Opt("OPT_PUNCHES")
                :PreIcon( global_images.upgradecombat )
                :PostText("TT_PUNCHES", math.min(#attack_options, 2))
                :Dialog("DIALOG_PUNCHES")
                :Fn(function(cxt)
                    local cards = table.multipick(attack_options, 2)
                    for i,card in ipairs(cards) do
                        card:AddXP(card:GetMaxXP())
                    end
                    cxt:Dialog("DIALOG_UPGRADE", cards )
                end)
                :CompleteQuest()
                :Travel()
        end
            
        if #maneuver_options > 0 then
            cxt:Opt("OPT_BLOCKS")
                :PreIcon( global_images.upgradecombat )
                :PostText("TT_BLOCKS", math.min(#maneuver_options, 2))
                :Dialog("DIALOG_BLOCK")
                :Fn(function(cxt)
                    local cards = table.multipick(maneuver_options, 2)
                    for i,card in ipairs(cards) do
                        card:AddXP(card:GetMaxXP())
                    end
                    cxt:Dialog("DIALOG_UPGRADE", cards )
                end)
                :CompleteQuest()
                :Travel()
        end

        cxt:Opt("OPT_LEAVE"):MakeUnder()
            :Dialog("DIALOG_LEAVE")
            :PreIcon( global_images.close)
            :CompleteQuest()
            :Travel()
    end)
