
AddOpinionEvent("ENCOURAGED_FAITH", 
{
    delta = OPINION_DELTAS.OPINION_UP,
    txt = "Helped them consolidate their faith",
})

local QDEF = QuestDef.Define{
    title = "Crisis of Faith",
    icon = engine.asset.Texture("icons/quests/oppo_hesh_help_me.tex"),
    qtype = QTYPE.OPPORTUNITY,
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("FOREST_LOCATION"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Help {friend} see the light",
    desc = "Your friend {friend} is a little lost and confused, and could use a soundboard.",
    mark = {"location"},
    terminal = true
}

:AddLocationDefs{
    FOREST_LOCATION = {
        name = "Beach",
        map_tags = {"beach"},
        plax = "EXT_SMITH_BEACHCREATURE",
        indoors = false,
    },
}

:AddCast{
    cast_id = "friend",
    condition = function( agent, quest )
        return agent:GetRelationship() == RELATIONSHIP.LIKED and agent:GetFactionID() == "CULT_OF_HESH"
    end,
    on_assign = function(quest, agent) 
        agent:MoveToLocation(quest:GetCastMember("location"))
    end,
}


QDEF:AddConvo()
    :SetFlag( CONVO_FLAGS.NO_PRESENT)
    :ConfrontState("CONFRONT", function(cxt, who) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            * {friend} looks startled as you enter the clearing, and rises from {friend.hisher} kneeling posture.
            player:
                !left
                !greeting
                Hey, it's {friend}!
                !permit
                Listen, don't let me disturb you. If nature calls, you gotta answer.
            friend:
                !right
                !scared
                Noo—noo, I'm just out here praying to Hesh.
                It's just... well... I'm not sure if the Almighty Tentacle is listening...
            player:
                Oh hey, don't feel so bad.
                Our great jellyfish friend ignores me <i>all</i> the time.
            friend:
                !disappoint
                I just wish I knew what to do.
        ]],

        OPT_SUGGEST_LEAVING = "Suggest moving to Murder Bay",
        DIALOG_SUGGEST_LEAVING = [[
            player:
                You know, when I'm struggling, I find it helps to just drop everything.
                All those cares and duties, who needs 'em?
                !shrug
                Why not see the sights of Murder Bay? Heard they're great this time of year.
            friend:
                !thought
                I <i>have</i> been thinking of just leaving it all behind...
                The Cult usually doesn't take well to defectors, but what's the worse that could happen?
                !happy
                You know, you're right. Thanks {player}.
                Take this, I won't be needing it anymore.
                !give
            player:
                !take
            friend:
                !exit
        ]],

        OPT_ENCOURAGE = "Encourage {friend} to keep the faith",
        DIALOG_ENCOURAGE = [[
            player:
                !point
                Hey, you just keep doing what you're doing.
                And I promise: one day, you'll be down in that dark abyss with Hesh.
            friend:
                !happy
                You're right. This is just a test of faith!
                Thanks {player}!
                !exit
            * {friend} goes back to kneeling in prayer as if the exchange never happened at all.
        ]],

        OPT_OFFER_PRAY = "Offer to pray a bit with {friend}",
        DIALOG_OFFER_PRAY = [[
            player:
                !shrug
                Some might say I'm no proper devotee, but why don't I join you in prayer?
            friend:
                !dubious
                Uhhh, that's alright, I'm not sure Hesh would... uh, approve of that.
            player:
                !thought
                Whaddya mean? I offer thanks to Hesh every time I pop a new bottle of Snellicks.
            friend:
                !laugh
                Thanks for the laugh {player}. Be seeing you.
        ]],

    }
    :Fn(function(cxt)
        cxt.quest:Complete()
        cxt:TalkTo("friend")
        
        cxt:Dialog("DIALOG_INTRO")

        cxt:Opt( "OPT_ENCOURAGE" )
            :Dialog( "DIALOG_ENCOURAGE" )
            :ReceiveOpinion("ENCOURAGED_FAITH")
            :Travel()

        cxt:Opt( "OPT_SUGGEST_LEAVING" )
            :Dialog( "DIALOG_SUGGEST_LEAVING" )
            :GainCard( "heshian_amulet" )
            :Fn(function(cxt)
                cxt:GetAgent():Retire()
            end )
            :Travel()

        cxt:Opt( "OPT_OFFER_PRAY" )
            :Dialog( "DIALOG_OFFER_PRAY" )
            :DeltaResolve( 15 )
            :Travel()
    end)
