local METTLE_POINTS = 5

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,
    priority = QEVENT_PRIORITY.HIGH,
    precondition = function(quest) 
        if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.NIGHT then

            if not TheGame:GetGameProfile():HasMettleUnlocked(TheGame:GetGameState():GetPlayerAgent():GetContentID()) and
                not TheGame:GetGameState():GetPlayerAgent().graft_owner:HasGraft("bare_mettle") then
                local thresh = 2
                if TheGame:GetGameState():GetActProgress() >= thresh then
                    return true
                end
            end
        end
    end,
}

QDEF:AddCastByAlias{
    cast_id = "stranger",
    alias = "DODGY_SCAVENGER",
}

QDEF:AddConvo()
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("FOO", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                {player_sal?
                        * You think you'd be used to the constant interruptions, but you're still surprised when a stranger approaches out of the shadows.
                    player:
                        !left
                        !dubious
                    agent:
                        !right
                        How's it faring, grifter? Fancy a test of your Mettle?
                    player:
                        $neutralDubious
                        Uh, I don't usually go in for random brawls unless I'm getting paid for it.
                    agent:
                        It's not that kind of test, hunter. That's Mettle with a capital M. 
                        !permit
                        Only some in Havaria are fit to wield it. 
                }
                {player_rook?
                        * A stranger approaches you—foolish of them, given your track record so far.
                    player:
                        !left
                    agent:
                        !right
                        !salute
                        Hail, soldier. You seem like the type who needs to spend their Mettle.  
                    player:
                        I wasn't aware I had any. 
                    agent:
                        Humility looks good on you. But we both know it's only skin-deep. 
                }
                {player_smith?
                    * The streets of the Foam are full enough with strangers that you see no reason to react when one blocks your path.
                    player:
                        !left
                        !dubious
                        Alright, there? You need directions or something?
                    agent:
                        !right
                        I should ask you the same. You look ripe with Mettle, eager to test it. 
                    player:
                        $neutralDubious
                        Normally I prefer a round or two at the bar before I start brawlin'.
                        !shrug
                        But if you <i>really</i> want a kra'deshi knuckle sandwich, I guess I can oblige.
                    agent:
                        !chuckle
                        Hah! Your bravado is a mask, but you wear it well.
                }
                {not player_smith and not player_rook and not player_sal?
                    * A stranger approaches you.
                    player:
                        !left
                    agent:
                        !right
                        You're here for Mettle.
                }

            ]],
            OPT_ASK_ABOUT_METTLE = "Ask about Mettle",
            DIALOG_ASK_ABOUT_METTLE = [[
                {player_sal?
                    player:
                        What exactly do you mean by Mettle?
                    agent:
                        It's the only currency truly native to the griftlands. 
                        !miscPersuasive
                        Exert your influence, and you will draw in Mettle as you draw breath. 
                    player:
                        !eureka
                        Aha!
                        $neutralJoke
                        Sounds like nonsense.
                    agent:
                        Your skepticism serves you well in combat, but not here. 
                        Here—find this shrine, and see for yourself.
                    * {agent} pulls out a map and points to a location that is seemingly just a cave.
                        !give
                        And take this to get you started—you'll soon see what I mean. 
                    * Gained {1} Mettle.
                }
                {player_rook?
                    player:
                        !thought
                        I've heard of this. A sort of hedge religion, yes? 
                    agent:
                        !quaint
                        It's more than quaint superstition, friend. Mettle is a force to be reckoned with.
                        Great deeds and greater purposes draw Mettle to you—and some will trade handsomely for it, once gathered.
                    player:
                        I see. Fortune favors the bold, as they say.
                    agent:
                        Here—find this shrine, and see for yourself.
                    * {agent} pulls out a map and points to a location that is seemingly just a cave.
                        !give
                        And take this to get you started—you'll soon see what I mean. 
                    * Gained {1} Mettle.
                }
            {player_smith?
                    player:
                        !crossed
                        Okay, I'll bite. Whaddya mean by Mettle?
                    agent:
                        You were raised to know only Hesh, but there are other forces at work in this world—ones far more mysterious than the Abyss.
                        !eureka
                        Do enough—fight, take risks, and stand victorious—and you'll soon feel its influence.
                    player:
                        So it's hocus pocus? Wiggledy-piggledy? 
                    agent:
                        It's your untapped potential. Unlock it and your enemies will tremble. 
                        Here—find this shrine, and see for yourself.
                    * {agent} pulls out a map and points to a location that is seemingly just a cave.
                        !give
                        And take this to get you started—you'll soon see what I mean. 
                    * Gained {1} Mettle.
                }
                
                {not player_smith and not player_rook and not player_sal?
                    player:
                        Sorry, what now?
                    agent:
                        Here—find this shrine, and see for yourself.
                    * {agent} pulls out a map and points to a location that is seemingly just a cave.
                        !give
                        And take this to get you started—you'll soon see what I mean. 
                    * Gained {1} Mettle.
                }

            ]],
            OPT_DONE = "Leave {agent}",

        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("stranger"))
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_ASK_ABOUT_METTLE")
                    :Dialog("DIALOG_ASK_ABOUT_METTLE", METTLE_POINTS)
                    :Fn(function( cxt )
                        local character_id = cxt.player:GetContentID()
                        TheGame:GetGameProfile():UnlockMettle( character_id )
                        TheGame:GetGameProfile():AddMettlePoints( character_id, METTLE_POINTS, "EVENT_FIRST_METTLE" )
                        cxt.quest:Complete()
                    end)
            else
                cxt:Opt("OPT_DONE")
                    :Travel()
            end
        end)
