local BASE_PRICE = 50
local PRICE_PER_COMBAT_STRENGTH = 30
local HAGGLE_DISCOUNT = .3
local CHANCE_TO_ROB = .2

local function CalculatePrice( agent, discount )
    local price = BASE_PRICE + (PRICE_PER_COMBAT_STRENGTH * agent:GetCombatStrength())
    if discount then
        price = round_zero(price * (1 - HAGGLE_DISCOUNT), 5)
    end
    return price
end

----------------------------------------------------------------------

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest)
        return TheGame:GetGameState():GetCaravan():HasHireling() == false and 
            TheGame:GetGameState():GetDayPhase() ~= DAY_PHASE.NIGHT and
            TheGame:GetGameState():IsHiringAvailable()
    end,
}

:AddCast{
    cast_id = "mercenary",
    condition = function(agent, quest)
        if agent:GetRelationship() <= RELATIONSHIP.DISLIKED then
            return false, "Disliked"
        end
        if not agent:HasTag("mercenary") then
            return false, "Not mercenary"
        end
        local price = CalculatePrice( agent, true )
        if TheGame:GetGameState():GetCaravan():GetMoney() < price then
            return false, "Can't afford"
        end

        if agent:GetBrain():GetWorkPosition() then
            return false
        end
        return true
    end,
}

QDEF:AddConvo()
    :ConfrontState("STATE_HIRE")
        :Loc{
            DIALOG_INTRO = [[
                * It's dangerous out on the roads, but not every encounter is an ambush. {agent} approaches you with empty hands and an honest eye.
                player:
                    !left
                agent:
                    !right
                    !placate
                    Hi there, hunter. You lookin' for reinforcement?
                    I may not be licensed but I've got a strong arm and steady aim. And I prefer jobs on the level when I can get 'em.
                    !thumb
                    You want me at your flank, I can be there for {1#money}.
            ]],
            DIALOG_ACCEPT = [[
                player:
                    !give
                    Yeah, sure. No sense breaking a sweat when I can employ the locals, right?
                agent:
                    !take
                    You got it, boss. Glad we could do business.
                    Just lead the way.
            ]],
            DIALOG_HAGGLE = [[
                player:
                    !thought
                    $angryReluctant
                    That's a bit more of my take than I'd like to give.          
            ]],
            DIALOG_LOWERED_FEE = [[
                Player:
                    !permit
                    $miscAlluring
                    Be reasonable—I gotta keep some money for myself, don't I?
                agent:
                    !shrug
                    $neutralResigned
                    Yeah, guess we all gotta make a living. I'll call it {1#money}, but only 'cause you're a hunter.
                    I expect you to have my back as much as I have yours.
            ]],
            DIALOG_NO_DISCOUNT = [[
                player:
                    Can't you lower it? Just a little?
                    !pray
                    $miscAlluring
                    <i>Please</i>?
                agent:
                    !crossed
                    $neutralDirect
                    The price is fixed. Take it or leave it. 
            ]],
            DIALOG_DECLINE = [[
                player:
                    !shrug
                    Thanks for the offer, but I prefer to split my profits one way. 
                    !greeting
                    Good luck with the job hunt, freelancer.
                agent:
                    !shrug
                    $neutralResigned
                    So it goes, I guess. Watch yourself out there, hunter.        
            ]],
            DIALOG_DECLINE_BROKE = [[
                player:
                    !shrug
                    I can't afford that. I'm as broke as you are, bub.
                agent:
                    !shrug
                    $neutralResigned
                    So it goes, I guess. Watch yourself out there, hunter.        
            ]],
            OPT_HIRE = "Hire {agent} to assist you",
            OPT_HAGGLE = "Negotiate the price",
            OPT_DECLINE = "Decline the offer",
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.quest:GetCastAgent("mercenary"):MoveToLocation(cxt.location)
                cxt.encounter:SetPrimaryCast(cxt.quest:GetCastAgent("mercenary"))
                cxt.enc.scratch.price = CalculatePrice( cxt:GetAgent() )

                local amount = CalculatePayment( cxt:GetAgent(), cxt.enc.scratch.price)
                cxt:Dialog("DIALOG_INTRO", amount)
            end

            cxt:Opt("OPT_HIRE")
                :PreIcon( global_images.hire )
                :DeliverMoney(cxt.enc.scratch.price)
                :RecruitMember( PARTY_MEMBER_TYPE.HIRED )
                :Dialog("DIALOG_ACCEPT")
                :Travel()

            cxt:Opt("OPT_HAGGLE")
                :Dialog("DIALOG_HAGGLE")
                :Negotiation{
                    on_success = function( cxt ) 
                        cxt.enc.scratch.price = CalculatePrice( cxt:GetAgent(), true )
                        local amount = CalculatePayment( cxt:GetAgent(), cxt.enc.scratch.price)
                        cxt:Dialog("DIALOG_LOWERED_FEE", amount)
                    end,
                    on_fail = function( cxt ) cxt:Dialog("DIALOG_NO_DISCOUNT") end 
                }

            


            cxt:Opt("OPT_DECLINE")
                :PreIcon( global_images.close )
                :Dialog(cxt.enc.scratch.price > cxt.caravan:GetMoney() and "DIALOG_DECLINE_BROKE" or "DIALOG_DECLINE")
                :Travel()

        end)
