local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
}

:AddOpinionEvents{
    
    helped_them_get_package = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Helped them apprehend a smuggler",
    },
    sold_them_out = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Sold them out to the Spark Barons",
    },
    kept_their_package = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Kept their package of contraband",
    },
    lied_to_them = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Lied to them about a smuggler",
    },
    covered_for_them =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Helped them evade the Spark Barons",
    },
    defended_them =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Defended them from the Spark Barons",
    },
}


local function SpawnBarons(cxt)
    cxt.enc.scratch.barons = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location, true)
    cxt:ReassignCastMember("baron", cxt.enc.scratch.barons[1])
    --cxt.enc.scratch.barons[1].social_connections:SetRelationship(RELATIONSHIP.LIKED, cxt.player)
    cxt.enc:SetPrimaryCast(cxt.enc.scratch.barons[1])
end

local function BaronsLeave(cxt)
    for k,v in pairs(cxt.enc.scratch.barons) do
        v:MoveToLimbo()
    end

end


local function OpenPackage(cxt)
    local loot = {money = 0, items = {}}
    for i, card in ipairs(BattleCardCollection.AllLocalItems():Pick(4)) do
        table.insert( loot.items, card.id)
    end
    cxt.enc:AwardLoot(loot, { found=true } )
end


QDEF:AddConvo()
    :Loc{
        DIALOG_REVEAL_MARKET = [[
            jake:
                Hey, if you need any supplies, check out the Jakes market. Tell them I sent you if they give you a hard time.
        ]],
        DIALOG_JAKE_LEAVES = [[
            jake:
                !exit
            * {jake} heaves the package, and carries on {jake.hisher} way.
        ]],

    }
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                * A Jake nervously approaches you.
                player:
                    !left
                jake:
                    !right
                    Hey. Do you want to make a quick buck?
                    Hold this package for me. I'll pay you {1#money}.
            ]],
            DIALOG_REINTRO = [[
                jake:
                    C'mon, man, I don't have time for this!
            ]],
            OPT_ASK_WHY = "Ask what {agent.heshe} is afraid of",
            DIALOG_ASK_WHY = [[
                player:
                    What are you running from?
                jake:
                    There's a Baron patrol coming around the corner any second now.
                    They'll search me for sure - I'm a Jake.
                    But you look... er... respectable enough.
            ]],
            OPT_ASK_ABOUT_PACKAGE = "Ask what's in the package",
            DIALOG_ASK_ABOUT_PACKAGE = [[
                player:
                    What's in the package?
                jake:
                    It's better if you don't know.
                    Just don't let them find it on you, alright?
            ]],

            OPT_AGREE = "Take the package",
            DIALOG_AGREE = [[
                player: 
                    Alright, I'll do it.
                jake:
                    !give
                    Oh thank Hesh. Here.
                    !exit
                * You hide the package just as the Baron patrol rounds the corner.
            ]],

            OPT_DISAGREE = "Refuse the package",
            DIALOG_DISAGREE = [[
                player:
                    I don't want to get involved.
                jake:
                    Damnit, then don't waste my time!
                    You didn't see me, OK?
                    !exit
                * {jake} hides in the ditch just in time to avoid the arriving patrol.
            ]],
        }

        :SetLooping()
        :Fn(function(cxt)
            local PAYMENT = 100
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.enc.scratch.jake = TheGame:GetGameState():AddSkinnedAgent( "JAKES_RUNNER" ) --new guy, not relationship
                cxt:ReassignCastMember("jake", cxt.enc.scratch.jake)
                cxt.enc:SetPrimaryCast(cxt.enc.scratch.jake)
                cxt:Dialog("DIALOG_INTRO", PAYMENT)
            else
                cxt:Dialog("DIALOG_REINTRO")
            end

            cxt:Question("OPT_ASK_WHY", "DIALOG_ASK_WHY")
            cxt:Question("OPT_ASK_ABOUT_PACKAGE", "DIALOG_ASK_ABOUT_PACKAGE")

            cxt:Opt("OPT_AGREE")
                :Dialog("DIALOG_AGREE")
                :ReceiveMoney(PAYMENT)
                :GoTo("STATE_HOLDING_PACKAGE")

            cxt:Opt("OPT_DISAGREE")
                :Dialog("DIALOG_DISAGREE")
                :GoTo("STATE_NOT_HOLDING_PACKAGE")
        end)

    :State("STATE_NOT_HOLDING_PACKAGE")
        :Loc{
            DIALOG_INTRO = [[
                baron:
                    !right
                * {baron} eyes you suspiciously.
                baron:
                    Have you seen a Jake pass by this way?
            ]],
            DIALOG_INTRO_FRIEND = [[
                baron:
                    !right
                    Hey {player}. You didn't happen to see a Jake come by this way, did you?
            ]],
            DIALOG_COVER_FOR_JAKE = [[
                player:
                    Nope, I haven't seen anything.
            ]],
            DIALOG_COVER_FOR_JAKE_FAIL_FRIEND = [[
                baron:
                    That doesn't quite add up, friend...
                * {baron} notices {jake}'s head peeking out of the ditch.
                baron:
                    !disappoint
                    You <i>lied</> to me.
                    But I have more important things to deal with right now.
                    !baron
                * {baron} turns to {jake} and draws {baron.hisher} weapons.
            ]],
            DIALOG_COVER_FOR_JAKE_FAIL = [[
                baron:
                    Nice try, grifter. But I was just seeing what you would say.
                    {jake} is as obvious as the shell on an oshnu.
                    !exit
                * {baron} turns to {jake} and draws {baron.hisher} weapons.
            ]],

            DIALOG_COVER_FOR_JAKE_SUCCESS = [[
                baron:
                    I guess I'll have to keep looking, then.
                    !exit
                * When the coast is clear, {jake} emerges.
                jake:
                    !right
                    !happy
                    Hey, thanks for that.
            ]],

            OPT_COVER_FOR_JAKE = "Convince {baron} you didn't see anything",
            
            OPT_LEAVE_JAKE_BEHIND = "Leave {jake} to {jake.hisher} fate",
            DIALOG_LEAVE = [[   
                * You leave.
            ]],
            OPT_SELL_OUT_JAKE = "Sell out {jake}",
            DIALOG_SELL_OUT_JAKE = [[
                player:
                    Yeah, {jake.heshe} is hiding in the ditch.
                baron:
                    Thank you for your help.
                jake:
                    !right
                    !angry
                    You rat!
                    !exit
                * The Barons march {jake} away.
            ]],
            OPT_DEFEND_THE_JAKE = "Defend {jake}"
        }
        :Fn(function(cxt) 
            SpawnBarons(cxt)
            local friend = cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL
            
            cxt:Dialog(friend and "DIALOG_INTRO_FRIEND" or "DIALOG_INTRO")

            cxt:Opt("OPT_COVER_FOR_JAKE")
                :Dialog("DIALOG_COVER_FOR_JAKE")
                :Negotiation{
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_COVER_FOR_JAKE_SUCCESS")
                        cxt.enc.scratch.jake:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("covered_for_them"))
                        cxt:Dialog("DIALOG_JAKE_LEAVES")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,

                    on_fail = function(cxt)
                        cxt.enc.scratch.barons[1]:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("lied_to_them"))
                        cxt:Dialog(friend and "DIALOG_COVER_FOR_JAKE_FAIL_FRIEND" or "DIALOG_COVER_FOR_JAKE_FAIL")
                        
                        
                        cxt:Opt("OPT_DEFEND_THE_JAKE")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                allies = {cxt.enc.scratch.jake},
                                on_win = function(cxt) 
                                    cxt:GoTo("STATE_FOUGHT_BARONS")
                                end
                            }
                        cxt:Opt("OPT_LEAVE_JAKE_BEHIND")
                            :Dialog("DIALOG_LEAVE")
                            :Travel()
                    end,
                }

                cxt:Opt("OPT_SELL_OUT_JAKE")
                    :Dialog("DIALOG_SELL_OUT_JAKE")
                    :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent("sold_them_out"), nil, cxt.enc.scratch.jake )
                    :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent("helped_them_get_package"), nil, cxt.enc.scratch.barons[1] )
                    :Travel()

        end)

    :State("STATE_HOLDING_PACKAGE")
        :Loc{
            DIALOG_INTRO = [[
                baron:
                    !right
                    Out of the way, grifter. We've got a smuggler to deal with.
                jake:
                    !left
                    I don't have anything, you rent-a-cop.
                * {baron} searches {jake}, and comes up empty.
                jake:
                    Ha! Told you!
                baron:
                    You're next, grifter. Empty your pockets.
                player:
                    !left
            ]],
            DIALOG_INTRO_FRIEND = [[
                * Your friend {baron}'s face brightens when {baron.heshe} recognises you.
                baron:
                    !happy
                    !right
                    Hey {player}. Watch out for that Jake.
                    We've been tracking {jake.hisher} little smuggling operation for weeks now.
                    {jake.HeShe}'s been supplying the Rise with weapons!
                jake:
                    !left
                    Go ahead and search me, {baron}. I'm clean.
                * {baron} searches {jake}, and comes up empty.
                jake:
                    Ha! Told you!
                baron:
                    I don't understand. I saw the goods with my own eyes.
                player:
                    !left
            ]],

            OPT_REFUSE = "Refuse",
            DIALOG_REFUSE = [[
                player:
                    !fight
                    I'd rather not.
            ]],
            OPT_CONVINCE = "Convince {agent} to skip you",
            DIALOG_CONVINCE = [[
                player:
                    That's not really neccessary, now is it?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                baron:
                    Yeah, you're right. You don't fit the description.
                    You should watch yourself, {jake}. I know I will be.
                    !exit

            ]],
            DIALOG_CONVINCE_FAIL = [[
                baron:
                    Last chance, grifter. Empty those pockets.
            ]],
            OPT_SELL_OUT_JAKE = "Sell {jake} out",
            DIALOG_SELL_OUT_JAKE = [[
                * You hand the package over to {baron}.
                player:
                    This Jake forced me to take it.
                    I don't want trouble.
                jake:
                    !right
                    !spit
                    You Hesh-sucking rat.
                baron:
                    !right
                    I knew it!
                    You're coming with me, {jake}.
                    As you were, grifter.
                    !exit
                * The Barons haul {jake} off, leaving you alone and emptyhanded.
            ]],
            
            DIALOG_SELL_OUT_JAKE_TO_FRIEND = [[
                * You hand the package over to {baron}.
                player:
                    This oshnu wanted me to hide this from you.
                jake:
                    !right
                    !spit
                    You Hesh-sucking rat.
                baron:
                    !right
                    Thanks, {player}, you're a real life-saver.
                    You're coming with me, {jake}.
                    !exit
                * The Barons haul {jake} off, leaving you alone and emptyhanded.
            ]],
            OPT_COVER_FOR_JAKE = "Cover for {jake}",
            DIALOG_COVER_FOR_JAKE = [[
                player:
                    I think I saw another Jake back the way I came from.
                    Maybe that's your guy?
                baron:
                    I'll go check it out.
                    Thanks for the tip {player}.
                    !exit
                * The Barons leave.
            ]]
        }
        :SetLooping()
        :Fn(function(cxt) 
                if cxt:FirstLoop() then
                    cxt.quest.param.has_package = true
                    SpawnBarons(cxt)
                    cxt.enc.scratch.want_to_search = cxt:GetAgent():GetRelationship() <= RELATIONSHIP.NEUTRAL --in case relationship changes in a loop
                    cxt:Dialog(cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_INTRO_FRIEND" or "DIALOG_INTRO")
                end
                
                if cxt.enc.scratch.want_to_search then
                    cxt:Opt("OPT_REFUSE")
                        :Dialog("DIALOG_REFUSE")
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            allies = {cxt.enc.scratch.jake},
                            on_win = function() 
                                cxt:GoTo("STATE_FOUGHT_BARONS")
                            end,
                        }
                    
                    cxt:Opt("OPT_CONVINCE")
                        :Dialog("DIALOG_CONVINCE")
                        :Negotiation{
                            helpers = { cxt.enc.scratch.jake },
                            on_success = function(cxt) 
                                cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                                cxt.enc.scratch.jake:OpinionEvent( cxt.quest:GetQuestDef():GetOpinionEvent("covered_for_them"))
                                BaronsLeave(cxt)
                                cxt:GoTo("STATE_HAS_PACKAGE")
                            end,
                            
                            on_fail = function(cxt)
                                cxt:Dialog("DIALOG_CONVINCE_FAIL")
                            end,
                        }

                    cxt:Opt("OPT_SELL_OUT_JAKE")
                        :Dialog("DIALOG_SELL_OUT_JAKE")
                        :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent("sold_them_out"), nil, cxt.enc.scratch.jake )
                        :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent("helped_them_get_package"), nil, cxt.enc.scratch.barons[1] )
                        :Travel()

                else
                    
                    cxt:Opt("OPT_COVER_FOR_JAKE")
                        :Dialog("DIALOG_COVER_FOR_JAKE")
                        :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent("covered_for_them"), nil, cxt.enc.scratch.jake )
                        :Fn(function(cxt) BaronsLeave(cxt) end)                        
                        :GoTo("STATE_HAS_PACKAGE")

                        
                    cxt:Opt("OPT_SELL_OUT_JAKE")
                        :Dialog("DIALOG_SELL_OUT_JAKE_TO_FRIEND")
                        :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent("sold_them_out"), nil, cxt.enc.scratch.jake )
                        :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent("helped_them_get_package"), nil, cxt.enc.scratch.barons[1] )
                        :Travel()
                end
                
            end)

    :State("STATE_FOUGHT_BARONS")
        :Loc{
            DIALOG_FOUGHT_BARONS_JAKE_DIED = [[
                right:
                    !exit
                * {jake} is dead. Out of curiosity, you open the package.
            ]],
            DIALOG_FOUGHT_BARONS_JAKE_LIVED = [[
                jake:
                    !right
                    Wow, thanks for that.
                    You really saved my mucus!
            ]]
        }
    
        :Fn(function(cxt) 

            if cxt.enc.scratch.jake:IsDead() then
                cxt:Dialog("DIALOG_FOUGHT_BARONS_JAKE_DIED")
                OpenPackage(cxt)
                StateGraphUtil.AddLeaveLocation(cxt)
            else
                
                cxt:Dialog("DIALOG_FOUGHT_BARONS_JAKE_LIVED")
                cxt.enc.scratch.jake:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("defended_them"))

                if cxt.quest.param.has_package then
                    cxt:GoTo("STATE_HAS_PACKAGE")
                else

                    cxt:Dialog("DIALOG_JAKE_LEAVES")

                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            end

        end)


    :State("STATE_HAS_PACKAGE")
        :Loc{
            DIALOG_ASK_FOR_PACKAGE = [[
                jake:
                    !right
                    Can I have the package back now?
            ]],
            OPT_GIVE_BACK_PACKAGE = "Give the package back to {jake}",
            DIALOG_GIVE_BACK_PACKAGE = [[
                player:
                    Of course.
                    !give
                jake:
                    Thanks for that. I'm late enough with this delivery already.
            ]],

            OPT_DEMAND_MORE_MONEY = "Extort {jake} for more money",
            DIALOG_DEMAND_MORE_MONEY = [[
                player:
                    If it's that valuable to you, you can pay a little more.
                jake:
                    This is blackmail!
                player:
                    Yes. Yes it is.
                jake:
                    Fine. You arse. 
                    !exit
                * You're here to make money, not friends.
            ]],

            OPT_KEEP_THE_PACKAGE = "Keep the package for yourself",
            DIALOG_KEEP_THE_PACKAGE = [[
                player:
                    I think I'll be keeping this for myself, actually.
            ]],

            DIALOG_KEEP_THE_PACKAGE_LEAVE = [[
                jake:
                    You- 
                    !angry
                    You arse.
                    !exit
                * You open the package.
            ]],
            DIALOG_KEEP_THE_PACKAGE_FIGHT = [[
                jake:
                    !angry
                    You crossed the wrong jake.
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND_JAKE_DEAD = [[
                * {jake} is dead. You open the package.
            ]],
            DIALOG_DEFEND_JAKE_ALIVE = [[
                jake:
                    !exit
                * {jake} runs off.
                * You open the package.
            ]]
        }
        :Fn(function(cxt) 
            
            cxt.enc:SetPrimaryCast(cxt.enc.scratch.jake)

            cxt:Dialog("DIALOG_ASK_FOR_PACKAGE")
            
            cxt:Opt("OPT_GIVE_BACK_PACKAGE")
                :Dialog("DIALOG_GIVE_BACK_PACKAGE")
                :Fn(function() 
                    cxt:Dialog("DIALOG_JAKE_LEAVES")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
            
            cxt:Opt("OPT_DEMAND_MORE_MONEY")
                :Dialog("DIALOG_DEMAND_MORE_MONEY")
                :ReceiveOpinion(OPINION.EXTORTION)
                :ReceiveMoney(50, {no_scale = true})
                :Travel()

            cxt:Opt("OPT_KEEP_THE_PACKAGE")
                :Dialog("DIALOG_KEEP_THE_PACKAGE")
                :ReceiveOpinion( "kept_their_package" )
                :Fn(function(cxt) 
                    -- cxt.enc.scratch.jake:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("kept_their_package"))
                    if math.random() < .5 then
                        cxt:Dialog("DIALOG_KEEP_THE_PACKAGE_LEAVE")
                        OpenPackage(cxt)
                        StateGraphUtil.AddLeaveLocation(cxt)
                    else
                        cxt:Dialog("DIALOG_KEEP_THE_PACKAGE_FIGHT")
                        cxt:Opt("OPT_DEFEND")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                on_win = function(cxt) 
                                    cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_DEFEND_JAKE_DEAD" or "DIALOG_DEFEND_JAKE_ALIVE")
                                    OpenPackage(cxt)
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end
                            }
                    end
                end)
        end)
