local negotiation_defs = require "negotiation/negotiation_defs"
Content.AddNegotiationCard("horrible_rash", 
{
    name = "Horrible Rash",
    flavour = "'Uh, might I suggest a scarf?'",
    desc = "When this card is drawn, gain 1 {FLUSTERED}.",
    cost = 1,
    quip = "sick_card",
    flags = negotiation_defs.CARD_FLAGS.STATUS | negotiation_defs.CARD_FLAGS.EXPEND,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    event_handlers =
    {
       [ negotiation_defs.EVENT.DRAW_CARD ] = function( self, minigame, card, start_of_turn )
             if card == self then
                self:NotifyTriggeredPre()
                self.negotiator:AddModifier("FLUSTERED", 1, self)
                self:NotifyTriggeredPost()
            end
        end,
    }
})

--------------------------------------------------------------------

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    on_init = function(quest)
        quest.param.try_limit = quest.param.try_limit or math.random(3)
    end,
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_FIND_PLANT"
    end)

    :State("STATE_FIND_PLANT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * Walking down the road you come across a plant that you recognize from a book about foraging in Grout Bog.
                * The plant's leaves have fine markings on them, the flowers are an exuberant shade of red, and the small berries have a deep blue colour.
                    The leaves are supposed to be delicious!
                    !thought
                    Or was it the berries?
                    !forgetful
                    Maybe the flowers?
            ]],
            OPT_TASTE_THE_LEAVES = "Taste the leaves",
            DIALOG_TASTE_THE_LEAVES = [[
                * You snap a leaf off at its stem and chew on it a bit.
                player:
                    !drink
                * The leaf is soft and releases a sweet, slightly sticky sap.
            ]],
            DIALOG_TASTE_THE_LEAVES_BAD = [[
                * You snap a leaf off at its stem and chew on it a bit.
                player:
                    !drink
                * The leaf is soft and releases a sweet, slightly sticky sap.
                * At first your mind feels clear, but as you chew, you feel a tingling sensation on your skin.
                * The tingling quickly turns to pain as you develop a horrible rash.
                player:
                    !wince
            ]],
            OPT_TASTE_THE_BERRIES = "Taste the berries",
            DIALOG_TASTE_THE_BERRIES = [[
                * You pick a couple of berries, careful not to squish them.
                player:
                    !drink
                * The berries burst as you bit into them, fresh and delicious.
            ]],
            DIALOG_TASTE_THE_BERRIES_BAD = [[
                * You pick a couple of berries, careful not to squish them.
                player:
                    !drink
                * The berries burst as you bit into them, fresh and delicious.
                * The berries reinvigorated you, but as you swallow them, a rash begins to break out on your skin.
                player:
                    !wince
            ]],
            OPT_TASTE_THE_FLOWERS = "Taste the flowers",
            DIALOG_TASTE_THE_FLOWERS = [[
                * You pick a single flower and bring it up to your nose.
                * The smell clears your mind and your senses seem to focus.
            ]],
            DIALOG_TASTE_THE_FLOWERS_BAD = [[
                * You pick a single flower and bring it up to your nose.
                * The smell clears your mind and your senses seem to focus.
                * As you put the flower down, your skin starts to sting where the flower brushed against it.
                player:
                    !wince
                    $miscEmbarrassed
                    Ah. Definitely not the flowers.
            ]],
            OPT_LEAVE_THE_PLANT_NO_TASTE = "Leave without tasting the plant",
            DIALOG_KEEP_MOVING_NO_TASTE = [[
                * You keep moving. Eating a strange plant seems risky.
            ]],
            OPT_LEAVE_TRIED_SOME = "Leave without trying the rest",
            DIALOG_LEAVE_TRIED_SOME = [[
                * You've satisfied your curiosity and don't want to test fate more than necessary.
            ]],
            DIALOG_TRIED_ALL = [[
                * Having tasted everything, with varying degrees of success, you leave the plant behind, scratching your face where the flower grazed it.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")
                cxt.enc.scratch.number_tried = 0
            end

            local function CheckTriedAll( cxt )
                if cxt.enc.scratch.number_tried >= 3 then
                    cxt:Dialog("DIALOG_TRIED_ALL")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            end

            local function ChanceForRash(cxt)
                return cxt.enc.scratch.number_tried+1 >= cxt.quest.param.try_limit
                --return math.random() < .34 + (cxt.enc.scratch.number_tried * 0.33)
            end

            cxt:Opt("OPT_TASTE_THE_LEAVES")
                :ReqCondition(not cxt.enc.scratch.tried_leaves)
                :Fn(function(cxt)
                    if ChanceForRash(cxt) then
                        cxt:Dialog("DIALOG_TASTE_THE_LEAVES_BAD")
                        ConvoUtil.DoResolveDelta(cxt, 5)
                        cxt:ForceTakeCards{"horrible_rash"}
                        StateGraphUtil.AddLeaveLocation(cxt)
                    else
                        cxt:Dialog("DIALOG_TASTE_THE_LEAVES")
                        ConvoUtil.DoResolveDelta(cxt, 5)
                        cxt.enc.scratch.tried_leaves = true
                        cxt.enc.scratch.number_tried = cxt.enc.scratch.number_tried + 1
                        CheckTriedAll(cxt)
                    end
                end)

            cxt:Opt("OPT_TASTE_THE_BERRIES")
                :ReqCondition(not cxt.enc.scratch.tried_berries)
                :Fn(function(cxt)
                    if ChanceForRash(cxt) then
                        cxt:Dialog("DIALOG_TASTE_THE_BERRIES_BAD")
                        ConvoUtil.DoHealthDelta(cxt, 5)
                        cxt:ForceTakeCards{"horrible_rash"}
                        StateGraphUtil.AddLeaveLocation(cxt)
                    else
                        cxt:Dialog("DIALOG_TASTE_THE_BERRIES")
                        ConvoUtil.DoHealthDelta(cxt, 5)
                        cxt.enc.scratch.tried_berries = true
                        cxt.enc.scratch.number_tried = cxt.enc.scratch.number_tried + 1
                        CheckTriedAll(cxt)
                    end
                end)

            cxt:Opt("OPT_TASTE_THE_FLOWERS")
                :ReqCondition(not cxt.enc.scratch.tried_flowers)
                :Fn(function(cxt)
                    if ChanceForRash(cxt) then
                        cxt:Dialog("DIALOG_TASTE_THE_FLOWERS_BAD")
                        ConvoUtil.DoResolveDelta(cxt, 5)
                        cxt:ForceTakeCards{"horrible_rash"}
                        StateGraphUtil.AddLeaveLocation(cxt)
                    else
                        cxt:Dialog("DIALOG_TASTE_THE_FLOWERS")
                        ConvoUtil.DoResolveDelta(cxt, 5)
                        cxt.enc.scratch.tried_flowers = true
                        cxt.enc.scratch.number_tried = cxt.enc.scratch.number_tried + 1
                        CheckTriedAll(cxt)
                    end
                end)

            if cxt.enc.scratch.number_tried == 0 then
                cxt:Opt("OPT_LEAVE_THE_PLANT_NO_TASTE")
                    :Dialog("DIALOG_KEEP_MOVING_NO_TASTE")
                    :Fn(function(cxt) 
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            else
                cxt:Opt("OPT_LEAVE_TRIED_SOME")
                    :Dialog("DIALOG_LEAVE_TRIED_SOME")
                    :Fn(function(cxt) 
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end
        end)


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}