local battle_defs = require "battle/battle_defs"
Content.AddBattleCard( "numbness", 
{
    name = "Numbness",
    anim = "taunt",
    flavour = "'I can't even hold onto my blaster, what's happening to me?'",
    desc = "While this is in your hand, you deal 25% less damage.",

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    cost = 2,
    target_type = TARGET_TYPE.SELF,

    flags = battle_defs.CARD_FLAGS.STATUS | battle_defs.CARD_FLAGS.CONSUME,
    event_handlers =
    {
        [ battle_defs.BATTLE_EVENT.CALC_DAMAGE ] = function( self, card, target, dmgt )
            if card.owner == self.owner then
                dmgt:AddDamage( -math.floor(dmgt.min_damage * 0.25), -math.floor(dmgt.max_damage * 0.25), self )
            end
        end,
    },
})

local BUGS_DAMAGE_MIN, BUGS_DAMAGE_MAX = 6, 17
local NUMBNESS_CHANCE = .3
local LOOT_OPTIONS = {"jammed_blaster", "lock_box", "learning_drone", "overloaded_spark_hammer"}

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    precondition = function(quest)
        return TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):Get() > BUGS_DAMAGE_MAX
    end,
}

--------------------------------------------------------------------

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_FIND_STUFF"
    end)

    :State("STATE_FIND_STUFF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !crossed
                    * You come across a small cavern partly submerged in the bog.
                    * Inside it are the remains of an unfortunate traveler with a small box of assorted items.
                    * The entrance is swarming with nasty looking insects that look ready to attack anything that comes near them.
            ]],
            OPT_KEEP_MOVING = "Leave the remains behind",
            OPT_GATHER_LOOT = "Move through the swarm and take the loot",
            TT_GATHER_LOOT = "Venturing through the swarm of insects could be harmful.",
            DIALOG_KEEP_MOVING = [[
                * You keep moving. You're not going to give the bugs a free meal today.
            ]],
            DIALOG_GATHER_LOOT_HURT = [[
                * You push through the swarm as quickly as you can, shielding your face as best as you can. A few bugs manage to get some bites in.
                * Once past the bugs, you grab as much as you can and rush out. 
                player:
                    !tired
                    Phew, wouldn't want to do that again.
            ]],
            DIALOG_GATHER_LOOT_NUMB = [[
                * You push through the swarm as quickly as you can, shielding your face as best as you can. The bugs don't seem to bite you.
                * Once past the bugs, you grab as much as you can and rush out. 
                player:
                    !tired
                    Huh, not a single bite.
                * Looking down at your arm you see a growing patch of red skin and feel a spreading numbness.
                player:
                    !wince
                    Oh...
            ]],
        }

        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")
                cxt.enc.scratch.damage = math.random( BUGS_DAMAGE_MIN, BUGS_DAMAGE_MAX )
                cxt.quest.param.bog_loot = LOOT_OPTIONS[math.random(#LOOT_OPTIONS)]
            end

            if math.random() < NUMBNESS_CHANCE then
                cxt:Opt("OPT_GATHER_LOOT")
                    :PostText("TT_GATHER_LOOT")
                    :Dialog("DIALOG_GATHER_LOOT_NUMB")
                    :GainCard(cxt.quest.param.bog_loot)
                    :Fn(function( cxt )
                        cxt:ForceTakeCards{"numbness"}
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            else
                cxt:Opt("OPT_GATHER_LOOT")
                    :PostText("TT_GATHER_LOOT")
                    :Dialog("DIALOG_GATHER_LOOT_HURT")
                    :DeltaHealth(-cxt.enc.scratch.damage)
                    :GainCard(cxt.quest.param.bog_loot)
                    :Fn(function( cxt )
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end
            
            cxt:Opt("OPT_KEEP_MOVING")
                :Dialog("DIALOG_KEEP_MOVING")
                :Fn(function(cxt) 
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
        end)

