local fun = require "util/fun"

local negotiation_defs = require "negotiation/negotiation_defs"
local battle_defs = require "battle/battle_defs"

local function GetParasiteCard()
    local negotiation_card = fun(TheGame:GetGameState():GetPlayerAgent().negotiator:GetCards())
                                :filter(function(card) return CheckBits( card.def.flags, negotiation_defs.CARD_FLAGS.PARASITE) end)
                                :randomPick()
    
    local battle_card = fun(TheGame:GetGameState():GetPlayerAgent().battler:GetCards())
                                :filter(function(card) return CheckBits( card.def.flags, battle_defs.CARD_FLAGS.PARASITE) end)
                                :randomPick()

    if battle_card and negotiation_card then
        return math.random() < .5 and battle_card or negotiation_card
    elseif negotiation_card then
        return negotiation_card
    elseif battle_card then
        return battle_card
    end
end



local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    precondition = function(quest) 
        quest.param.parasite = GetParasiteCard()
        return quest.param.parasite ~= nil
    end,

}

-------------------------------------------------------------------
QDEF:AddConvo()
    :ConfrontState("CONF", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    You there. Stop.
                player:
                    Can I help you?
                
                * The bogger stares at you, looking for something. After an uncomfortable moment, they speak.
                
                {has_parasite?
                    agent:
                        The bog grows within you, but you are not worthy of it.
                        I will take it out.
                    player:
                        Excuse me?
                    agent:
                        The process will be painful, but the bog must be made whole again.
                        You <i>will</> comply.
                }
                {not has_parasite?
                    agent:
                        You are within the bog, but the bog is not within you.
                    player:
                        Oooookay.
                    agent:
                        Tread lightly through the bog, and one day you may become worthy of its embrace!
                }
            ]],

            OPT_LET_BOGGER_TAKE_PARASITE = "Let {agent} remove {1#card}",
            DIALOG_LET_BOGGER_TAKE_PARASITE = [[
                player:
                    Be my guest.
                    !exit
                agent:
                    !exit
                * {agent} removes {1#card}. 
                * The process is <i>excruciating</>.
            ]],
            DIALOG_LET_BOGGER_TAKE_PARASITE_2 = [[
                player:
                    !left
                    !injured
                agent:
                    !right
                    The bog has been satisfied. Some day you may be worthy of its embrace!
                player:
                    ...
                agent:
                    !exit
                * {agent} leaves you to tend to your wounds.
            ]],

            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    You're not touching me, bogger!
            ]],
            DIALOG_ON_WIN = [[
                {dead ?
                    * {agent} lies dead, {agent.hisher} corpse slowly sinking into the bog. 
                    * If you didn't know better, you'd swear {agent.heshe} was smiling.
                }
                {not dead?
                    agent:
                        !injured
                        You can not escape the bog's will! 
                    player:
                        !sigh
                        It never ends with you guys, does it?
                    agent:
                        It will consume you! Your body will rot and nourish it! It will-
                    player: 
                        !exit
                    agent:
                        !threaten
                        Hey! Come back! I'm not done telling you what the bog will do to you yet!
                        !neutral
                        ...
                    * You don't stick around for the rest of the sermon.

                }
            ]]

        }
        :Fn(function(cxt)
            cxt.quest:Complete()
            
            if  not cxt.player.negotiator:HasCard(cxt.quest.param.parasite) and
                not cxt.player.battler:HasCard(cxt.quest.param.parasite) then
                    --this will rarely happen, but I guess it could, so we need to cover the case where the card has been removed. If you have removed ALL of your parasites since the quest validated, the bogger just lectures you and leaves.
                    cxt.quest.param.parasite = GetParasiteCard()
            end
            cxt.quest.param.has_parasite = cxt.quest.param.parasite ~= nil

            local opfor = CreateCombatParty("BOGGER_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(opfor[1])
            cxt:Dialog("DIALOG_INTRO")

            if cxt.quest.param.parasite then
                cxt:Opt("OPT_LET_BOGGER_TAKE_PARASITE", cxt.quest.param.parasite, cxt.quest.param.parasite:GetName())
                    :Dialog("DIALOG_LET_BOGGER_TAKE_PARASITE", cxt.quest.param.parasite)
                    :Fn(function() 
                        AUDIO:PlayEvent("event:/sfx/general/conversation/damage_taken_gen")
                        cxt.enc:GetScreen():FireHealthOverlay( 0xee5d5faf ) -- punch to red

                        if cxt.player.negotiator:HasCard(cxt.quest.param.parasite) then
                            cxt.player.negotiator:RemoveCard(cxt.quest.param.parasite)
                        elseif cxt.player.battler:HasCard(cxt.quest.param.parasite) then
                            cxt.player.battler:RemoveCard(cxt.quest.param.parasite)
                        end
                    end)
                    :Dialog("DIALOG_LET_BOGGER_TAKE_PARASITE_2")
                    :Travel()

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                    }
                        :OnWin()
                            :Dialog("DIALOG_ON_WIN")
                            :Travel()

            else
                StateGraphUtil.AddLeaveLocation(cxt)
            end
        end)




