local battle_defs = require "battle/battle_defs"

Content.AddBattleCard( "deep_bog_battle", 
{
    name = "Deep Bog Secret",
    anim = "taunt",
    flavour = "'I just need a little more time out there. I'll find out what's going on.'",
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    cost = 2,
    max_xp = 5,
    hatch = true,
    target_type = TARGET_TYPE.SELF,
    flags = battle_defs.CARD_FLAGS.SKILL | battle_defs.CARD_FLAGS.HATCH,
    hatch_fn = function( self, battle )
        self:TransferCard( battle.trash_deck )
        self:Consume()
        local card = Battle.Card( self.userdata.hatch_id or (math.random() < .5 and "deep_bog_battle_hatch_1" or  "deep_bog_battle_hatch_2"), self.owner:GetAgent() )
        self.userdata.hatched_card = card
        self.owner.agent.battler:AddCard(card)
        local battle_card = card:Clone()
        battle_card.owner = self.owner
        battle:DealCard(battle_card, battle:GetHandDeck())
    end,

    precondition = function(quest)
        return not TheGame:GetGameState():GetPlayerAgent().battler:HasCardID("deep_bog_battle_hatch_1") and
                not TheGame:GetGameState():GetPlayerAgent().battler:HasCardID("deep_bog_battle_hatch_2") and
                not TheGame:GetGameState():GetPlayerAgent().battler:HasCardID("deep_bog_battle")

    end,
})



Content.AddBattleCard( "deep_bog_battle_hatch_1", 
{
    name = "Bog Down",
    desc = "Apply {STUN}.",
    flavour = "'The vessel squirmed and blinked at me maliciously. There was nothing left to do but raise it to my lips.'",
    cost = 1,
    flags = battle_defs.CARD_FLAGS.EXPEND | battle_defs.CARD_FLAGS.SKILL,
    series = CARD_SERIES.GENERAL,
    anim = "taunt",
    rarity = CARD_RARITY.UNIQUE,
    OnPostResolve = function( self, battle, attack )
        attack:AddCondition( "STUN", 1, self )
    end,
})

Content.AddBattleCard( "deep_bog_battle_hatch_2", 
{
    name = "Bog Doom",
    anim = "punch",
    flavour = "'Please don't, I can't...'",
    desc = "Doubles the target's {SURRENDER} meter.",
    cost = 1,
    series = CARD_SERIES.GENERAL,
    flags = battle_defs.CARD_FLAGS.MELEE | battle_defs.CARD_FLAGS.EXPEND,
    rarity = CARD_RARITY.UNIQUE,

    min_damage = 1,
    max_damage = 6,
    
    OnPostResolve = function( self, battle, attack )
        for i, hit in attack:Hits() do
            if hit.target:HasMorale() then
                hit.target:DeltaMorale( hit.target:GetMorale() )
            end
        end
    end        
})


local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    is_in_hiding = function(quest, agent)
        if agent == quest:GetCastMember("bogger") then
            return true
        end
    end,
}
:AddOpinionEvents{
    
    removed_cards = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Rejected the secrets of the Bog",
    },

    refuse_give_back =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Refused to give back Bog Secrets",
    },
    
    hatched_secrets =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Hatched Bog Secrets for them",
    },
}

:AddCastByAlias{
    cast_id = "bogger",
    alias = "DEEP_BOGGER",
}

:AddObjective{
    id = "intro",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    on_complete = function(quest)
        quest:Activate("stage_2")
    end
}

:AddObjective{
    id = "stage_2",
    hide_in_overlay = true,
    on_activate = function(quest) 
        quest.param.travel_counter = math.random(8, 16)
    end,

    events = 
    {
        player_traveled = function( quest, agent ) 
            quest.param.travel_counter = quest.param.travel_counter - 1
        end,
    },
}



-------------------------------------------------------------------
QDEF:AddConvo("intro")
    :ConfrontState("CONF", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                bogger:
                    !right
                    Have you been to the Deep Bog?
                player:
                    !left
                    Sorry, did you ask me something?
                bogger:
                    $miscMocking
                    Have you been... 
                    To the Deep...
                    !dubious
                    Bog?
            ]],

            OPT_YES = "Say 'yes'",
            DIALOG_YES = [[
                player:
                    I have.
                bogger:
                    !angry
                    $angryPatienceLost
                    Liar! 
                    I would see it on you if you had.
                    The bog knows itself! It sees itself!
                    !give
                    You are not in the deep bog, but you soon will be!
            ]],
            DIALOG_YES_TRUTH = [[
                player:
                    I have.
                bogger:
                    !impressed
                    $miscHushedUrgent
                    I can see this is the truth.
                    The bog has touched you! The bog knows!
                    !give
                    Receive the blessing of the bog.
            ]],

            OPT_NO = "Say 'no'",
            DIALOG_NO = [[
                player:
                    No, I have not.
                bogger:
                    !angry
                    $angryPatienceLost
                    Fool!
                    You are already standing there.
                    You just can't see what's in front of your face.
                    !give
                    But you'll see it soon enough.
            ]],
            DIALOG_NO_LIES = [[
                player:
                    No, I have not.
                bogger:
                    !angry
                    $angryPatienceLost
                    Lies!
                    I see it on you! The bog knows!
                    You will learn your fate eventually.
                    !give
            ]],
            DIALOG_LEAVE = [[
                player:
                    Er...
                bogger:
                    !exit
                * You have the uneasy feeling that this is not over.
            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            
            if cxt.player.battler:HasCardID( "deep_bog_battle_hatch_1" ) or cxt.player.battler:HasCardID( "deep_bog_battle_hatch_2" ) or cxt.player.battler:HasCardID( "deep_bog_battle" ) then
                cxt:Opt("OPT_YES")
                    :Dialog("DIALOG_YES_TRUTH" )
                    :Fn( function(cxt)
                        cxt.caravan:UpgradeHealth( 4, "EVENT_BOG_TRUTH" )
                    end )
                    :CompleteQuest()

                cxt:Opt("OPT_NO")
                    :Dialog("DIALOG_NO_LIES")
                    :GainCard("deep_bog_battle", function(card) 
                        card.userdata.hatch_id = "deep_bog_battle_hatch_1" 
                        cxt.quest.param.inserted_card = card 
                    end)
                    :CompleteQuest()

            else
                cxt:Opt("OPT_YES")
                    :Dialog("DIALOG_YES")
                    :GainCard("deep_bog_battle", function(card) 
                        card.userdata.hatch_id = "deep_bog_battle_hatch_1" 
                        cxt.quest.param.inserted_card = card 
                    end)
                    :Dialog("DIALOG_LEAVE")
                    :CompleteQuest("intro")
                    
                cxt:Opt("OPT_NO")
                    :Dialog("DIALOG_NO")
                    :GainCard("deep_bog_battle", function(card) 
                        card.userdata.hatch_id = "deep_bog_battle_hatch_2" 
                        cxt.quest.param.inserted_card = card  
                    end)
                    :Dialog("DIALOG_LEAVE")
                    :CompleteQuest("intro")
            end
        end)


-------------------------------------------------------------------
QDEF:AddConvo("stage_2")
    :ConfrontState("CONF", function(cxt) return ConvoUtil.CanInterruptTravel(cxt) and cxt.quest.param.travel_counter <= 0 end)
        :Loc{
            DIALOG_INTRO_UNHATCHED = [[
                agent:
                    !right
                    Do you not appreciate the Deep Bog?
                player:
                    !left
                    What?
                agent:
                    The Deep Bog. It sits with you, but you do not sit with it.
                    Is it wasted upon you?
            ]],
            DIALOG_INTRO_HATCHED = [[
                agent:
                    !right
                    The Deep Bog has found you!
                player:
                    !left
                    Oh, it's you again.
                agent:
                    You have heard the Voice of the Bog!
                    What did it say to you?
            ]],
            DIALOG_INTRO_REMOVED = [[
                agent:
                    !right
                    Where is it?
                player:
                    !left
                    That boggy mess you thrust on me last time? I got rid of it.
                agent:
                    !angry
                    The Bog has touched you! But you rejected it's touch!
                    !exit
                * {agent} steps back, and a hulking beast takes {agent.hisher} place!
            ]],

            OPT_GIVE_BACK = "Give back {1#card}",
            DIALOG_GIVE_BACK = [[
                player:
                    !give
                    Here, take it.
                agent:
                    The Deep Bog is never wrong.
                    It's just sometimes less right than usual.
                    !exit
            ]],
            OPT_REFUSE_GIVE_BACK = "Refuse",
            DIALOG_REFUSE_GIVE_BACK = [[
                player:
                    No, I'm keeping it.
                agent:
                    !angry
                    May the Deep Bog consume you.
                    !exit
            ]],
            OPT_NEGOTIATE = "Tell {agent} about the Deep Bog",
            DIALOG_NEGOTIATE = [[
                player:
                    The, uh, Deep Bog told me many things...
            ]],
            DIALOG_WON_NEGOTIATION = [[
                agent:
                    !happy
                    You and I and the Deep Bog are all One now.
                    !exit
                player:
                    ...Ooooookay.
            ]],
            DIALOG_LOST_NEGOTIATION = [[
                agent:
                    No. You have not heard what the Deep Bog is saying.
                    You need to keep listening.
                    !exit
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    I'm getting sick of this Bogshite!
            ]],
            DIALOG_BEAT_MONSTER = [[
                * The beast is dead, but {agent} is nowhere to be seen.
            ]],
        }
        :Fn(function(cxt) 
            cxt.quest:GetCastMember("bogger"):MoveToLocation(cxt.location)
            cxt:TalkTo(cxt.quest:GetCastMember("bogger"))
            local has_original, has_hatched

            for k,v in ipairs(cxt.player.battler:GetCards()) do
                if v == cxt.quest.param.inserted_card then
                    has_original = true
                    break
                elseif cxt.quest.param.inserted_card.userdata.hatched_card and cxt.quest.param.inserted_card.userdata.hatched_card == v then
                    has_hatched = true
                    break
                end
            end
            
            if has_original then
                cxt:Dialog("DIALOG_INTRO_UNHATCHED")

                cxt:Opt("OPT_GIVE_BACK", cxt.quest.param.inserted_card.id)
                    :Dialog("DIALOG_GIVE_BACK")
                    :Fn(function()
                        cxt.player.battler:RemoveCard( cxt.quest.param.inserted_card )
                    end)
                    :CompleteQuest()
                    :Travel()

                cxt:Opt("OPT_REFUSE_GIVE_BACK")
                    :Dialog("DIALOG_REFUSE_GIVE_BACK")
                    :ReceiveOpinion("refuse_give_back")
                    :CompleteQuest()
                    :Travel()


            elseif has_hatched then
                cxt:Dialog("DIALOG_INTRO_HATCHED")
                cxt:Opt("OPT_NEGOTIATE")
                    :Dialog("DIALOG_NEGOTIATE")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_WON_NEGOTIATION")
                            :ReceiveOpinion("hatched_secrets")
                            :CompleteQuest()
                            :Travel()
                        :OnFailure()
                            :Dialog("DIALOG_LOST_NEGOTIATION")
                            :CompleteQuest()
                            :Travel()
            else
                cxt:Dialog("DIALOG_INTRO_REMOVED")
                local monster = Agent( "GRAWKIT" )
                cxt:ReassignCastMember("monster", monster)
                cxt:GetAgent():MoveToLimbo() 
                cxt:Opt("OPT_DEFEND")               
                    :Dialog("DIALOG_DEFEND")
                    :CompleteQuest()
                    :ReceiveOpinion("removed_cards")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        enemies = {monster},
                    }
                        :OnWin()
                            :Dialog("DIALOG_BEAT_MONSTER")
                            :Travel()
            end


        end)-------------------------------------------------------------------
