local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest) 
        return TheGame:GetGameState():GetCaravan():CanOwnPet()
    end,
}


local convo = QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit")  end)
        :Loc{
            
            DIALOG_INTRO = [[
                * You hear sounds of a scuffle up ahead.
                autodog:
                    !left
                vroc:
                    !right
                * A feral vroc and a Cobbledog are circling each other, growling or grinding gears, as might befit the muzzle.
                * If you watch and wait, it's clear they plan to tear each other apart.
            ]],
            OPT_HELP_VROC = "Help the vroc",
            DIALOG_HELP_VROC = [[
                * You attack the Cobbledog. The vroc must be smarter than it looks, because it jumps into the fray at your side. 
            ]],
            OPT_HELP_AUTODOG = "Help the Cobbledog",
            DIALOG_HELP_AUTODOG = [[
                * You attack the vroc. The Cobbledog's lens flashes green as it joins in by your side.
            ]],

            DIALOG_WON_PARTNER_FIGHT_DEAD = [[
                player:
                    !right
                * You win the fight, but your only reward is carnage.
            ]],
            DIALOG_WON_PARTNER_FIGHT_VROC_ALIVE = [[
                vroc:
                    !right
                player:
                    !left
                * The vroc licks its wounds, then looks up at you with uncanny understanding.
            ]],

            OPT_TAKE_VROC = "Let the vroc join you",
            DIALOG_TAKE_VROC = [[
                player:
                    !interest
                    Well, beast? We'd make an odd pair, but I don't mind if you don't.
                * The vroc makes happy noises, its tongue licking contentedly at its snout. 
                player:
                    !happy
                    I will take that as acceptance. Come along, then.
            ]],
            OPT_LEAVE_VROC = "Scare the vroc away",
            DIALOG_LEAVE_VROC = [[
                player:
                    !dubious
                    $neutralDubious
                    You must be joking. I have no need for a pet. 
                    Begone, beast. 
                * When words fail, you scatter rocks at the vroc's feet to drive it away.
                vroc:
                    !exit
                * It gets the hint, and retreats through the swamp. 
            ]],

            DIALOG_WON_PARTNER_FIGHT_AUTODOG_ALIVE = [[
                autodog:
                    !right
                player:
                    !left
                * The Cobbledog enters a restful state, and focuses its lens on your face.
            ]],

            OPT_TAKE_AUTODOG = "Assume command of the Cobbledog",
            DIALOG_ATAKE_AUTODOG = [[
                player:
                    $neutralDubious
                    Have you no master already, Cobbledog?
                * Its gears only grind in response.
                player:
                    !crossed
                    $neutralDirect
                    Very well. Authorization protocol K9-826. My name is Captain Rook, and you are now under my command.
                * The Cobbledog beeps an affirmative, then falls in line at your heel.
            ]],
            OPT_LEAVE_AUTODOG = "Leave the Cobbledog",
            DIALOG_LEAVE_AUTODOG = [[
                player:
                    !dubious
                    $neutralDubious
                    Well... good luck, I suppose.
                * You leave the Cobbledog in its deactivated state, to await some different fortune.
            ]],

            OPT_HELP_AUTODOG = "Help the Cobbledog",
            OPT_WATCH_THE_FIGHT = "Watch the fight",
            
            DIALOG_WATCH_THE_FIGHT = [[
                * You keep your distance, and watch as the two go in for the kill.
            ]],
            DIALOG_WATCHED_BOTH_DEAD = [[
                vroc:
                    !exit
                autodog:
                    !exit
                * The vroc tears the main articulator out of the Cobbledog, but not before taking a mean rip to its side.
                * They both collapse in a heap of fur and twisted metal.
                * You scavenge through the mess.
            ]],

            DIALOG_WATCHED_AUTODOG_WINS = [[
                vroc:
                    !exit
                autodog:
                    !exit
                * The Cobbledog lunges at the vroc, snapping its neck in one fluid motion. 
                autodog:
                    !right
                player:
                    !left
                * Its attention then swivels around to you.
            ]],
            DIALOG_WATCHED_VROC_WINS = [[
                vroc:
                    !exit
                autodog:
                    !exit
                * The vroc keeps the Cobbledog off-balance, and manages a lucky strike that disables its rear leg.
                * From there, it takes its time, slowly reducing the Cobbledog to a pile of twitching scrap.
                vroc:
                    !right
                player:
                    !left
                * While the Cobbledog performs its last dying calculations, the vroc finally notices you. 
                * It growls, ready to defend its kill.
            ]],

            OPT_DEFEND = [[
                Defend yourself!
            ]],

            DIALOG_DEFEND = [[
                player:
                    !fight
                You reach for your guns.
            ]],
            DIALOG_WON_SOLO_FIGHT = [[
                * This isn't the first time you've found yourself top dog. But it is the first time it's been a pun.
            ]],

            OPT_WALK_AWAY = "Keep walking",
            DIALOG_WALK_AWAY = [[
                * You keep walking, taking little interest in the wanton violence on display.
            ]],
            DIALOG_TAKE_AUTODOG = [[
                player:
                    $neutralDubious
                    Have you no master already, Cobbledog?
                * Its gears only grind in response.
                player:
                    !crossed
                    $neutralDirect
                    Very well. Authorization protocol K9-826. My name is Captain Rook, and you are now under my command.
                * The Cobbledog beeps an affirmative, then falls in line at your heel.
            ]]

        }
        :Fn(function(cxt)
                cxt.quest:Complete()
                
                local vroc = Agent("VROC")
                vroc:GenerateRandomName()
                TheGame:GetGameState():AddAgent( vroc )
                cxt:ReassignCastMember("vroc", vroc)
                
                local autodog = Agent("RISE_AUTODOG")
                autodog:GenerateRandomName()
                TheGame:GetGameState():AddAgent( autodog )
                cxt:ReassignCastMember("autodog", autodog)
            
                cxt:Dialog("DIALOG_INTRO")

                
                cxt:Opt("OPT_HELP_AUTODOG")
                    :Dialog("DIALOG_HELP_AUTODOG")
                    :Battle{
                        allies = {autodog},
                        enemies = {vroc},
                        on_win = function(cxt)
                            if autodog:IsDead() or autodog:InLimbo() then
                                
                                cxt:Dialog("DIALOG_WON_PARTNER_FIGHT_DEAD")
                                StateGraphUtil.AddLeaveLocation(cxt)

                            else
                                cxt:Dialog("DIALOG_WON_PARTNER_FIGHT_AUTODOG_ALIVE")
                                
                                cxt:Opt("OPT_TAKE_AUTODOG")
                                    :Dialog("DIALOG_TAKE_AUTODOG")
                                    :RecruitMember(PARTY_MEMBER_TYPE.CREW, autodog )
                                    :Travel()
                                
                                cxt:Opt("OPT_LEAVE_AUTODOG")
                                    :Dialog("DIALOG_LEAVE_AUTODOG")
                                    :Travel()

                            end
                        end,
                    }

                cxt:Opt("OPT_HELP_VROC")
                    :Dialog("DIALOG_HELP_VROC")
                    :Battle{
                        allies = {vroc},
                        enemies = {autodog},
                        on_win = function(cxt)
                            if vroc:IsDead() or vroc:InLimbo() then
                                
                                cxt:Dialog("DIALOG_WON_PARTNER_FIGHT_DEAD")
                                StateGraphUtil.AddLeaveLocation(cxt)

                            else
                                cxt:Dialog("DIALOG_WON_PARTNER_FIGHT_VROC_ALIVE")
                                
                                cxt:Opt("OPT_TAKE_VROC")
                                    :Dialog("DIALOG_TAKE_VROC")
                                    :RecruitMember(PARTY_MEMBER_TYPE.CREW, vroc )
                                    :Travel()
                                
                                cxt:Opt("OPT_LEAVE_VROC")
                                    :Dialog("DIALOG_LEAVE_VROC")
                                    :Travel()

                            end
                        end,
                    }

                cxt:Opt("OPT_WATCH_THE_FIGHT")
                    :Dialog("DIALOG_WATCH_THE_FIGHT")
                    :Fn(function(cxt) 
                        if math.random() < 0.5 then
                            cxt:Dialog("DIALOG_WATCHED_BOTH_DEAD")
                            local loot = {money = 0, items = {}}
                            AgentUtil.CollectDeathLoot(vroc, loot)
                            AgentUtil.CollectDeathLoot(autodog, loot)
                            autodog:Kill()
                            vroc:Kill()

                            cxt.enc:AwardLoot(loot, { found=true } )

                            StateGraphUtil.AddLeaveLocation(cxt)
                        else
                            if math.random() < 0.5 then
                                autodog:Kill()
                                cxt:Dialog("DIALOG_WATCHED_VROC_WINS")
                                cxt:Opt("OPT_DEFEND")
                                    :Dialog("DIALOG_DEFEND")
                                    :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        enemies = {vroc},
                                        on_win = function(cxt) 
                                            cxt:Dialog("DIALOG_WON_SOLO_FIGHT")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end

                                    }
                            else
                                vroc:Kill()
                                cxt:Dialog("DIALOG_WATCHED_AUTODOG_WINS")
                                cxt:Opt("OPT_DEFEND")
                                    :Dialog("DIALOG_DEFEND")
                                    :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        enemies = {autodog},
                                        on_win = function(cxt) 
                                            cxt:Dialog("DIALOG_WON_SOLO_FIGHT")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end

                                    }
                            end
                        end
                    end)
                
                cxt:Opt("OPT_WALK_AWAY")
                    :Dialog("DIALOG_WALK_AWAY")
                    :Travel()

        end)
    
