local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
}


local BERRIES = {

    {
        opt = "Eat the blue one",
        txt = [[
            * You eat the blue bogberry.
            * Your stomach cramps like its caught in a vice, and you have enough time to curse your rotten idea...
            * But then it's over. 
            * Wiping the sweat from your brow, you think you might even feel a bit better than you did before.
        ]],
        fn = function(cxt) 
            ConvoUtil.DoResolveDelta(cxt, 5)
            ConvoUtil.DoHealthDelta(cxt, 5)
            cxt:ForceTakeCards{"bloated"}
        end,
    },
    {
        opt = "Eat the striped one",
        txt = [[
            * You eat the striped bogberry.
            * With a zap like someone stuck a live wire in your ear, you feel a rush of mental clarity.
        ]],
        fn = function(cxt) 
            ConvoUtil.DoResolveDelta(cxt, 5)
        end,
    },
    {
        opt = "Eat the purple one",
        txt = [[
            * You eat the purple bogberry.
            * You watch as your wounds stitch back together. It's a bit eerie, and it itches like bogrash.
            * But wouldn't you know it, you're not dead.
        ]],
        fn = function(cxt) 
            ConvoUtil.DoHealthDelta(cxt, 5)
        end,
    },
    {
        opt = "Eat the spotted one",
        txt = [[
            * You eat the spotted bogberry.
            * It's not long before your thoughts cloud over like morning fog.
            * By the taste in your mouth you'd just guess you ate something tart... but you can't seem to remember what it was.
        ]],
        fn = function(cxt) 
            ConvoUtil.DoResolveDelta(cxt, -5)
        end,
    },
    {
        opt = "Eat the red one",
        txt = [[
            * You eat the red bogberry.
            * It goes down like a hot coal, leaving blisters all down your gullet. 
            * You suspect that's bad.
        ]],
        fn = function(cxt) 
            local hp = cxt.player:GetHealth()
            local delta = -math.min(5, hp-1)
            if delta ~= 0 then
                ConvoUtil.DoHealthDelta(cxt, delta)
            end
        end,
    },
    
    {
        opt = "Eat the rotten one",
        txt = [[
            * You eat the rotten bogberry.
            * The juices bubble like a fermented brine, and you have a feeling you'll regret it later... 
            * But you know what they say: whatever doesn't kill you just makes you stronger.
            * You hope this is one of those times. 
        ]],
        fn = function(cxt) 
            local perm_diff = 5
            local temp_hurt = 10

            cxt.player.health:AddStatModifier( "EVENT_BERRIES", perm_diff )
            TheGame:GetGameState():LogNotification( NOTIFY.HEALTH_UPGRADED, perm_diff )

            local hp = cxt.player:GetHealth()
            local delta = -math.min(temp_hurt, hp-1)
            if delta ~= 0 then
                ConvoUtil.DoHealthDelta(cxt, delta)
            end
                

        end,
    },
}




local convo = QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit")  end)
        :Loc{
            
            DIALOG_INTRO = [[
                * You find a small patch of bogberries growing by the wayside.
                * They don't grow much elsewhere. You know some are poisonous, but you've heard rumors that others have powerful benefits. 
                * Now if you can only recall which one's which...
            ]],

            DIALOG_ATE_ALL_BERRIES = [[
                player:
                    !exit
                * Your belly is bursting, and the bogberry bush has become barren. So you bail. 
            ]],
            
            OPT_LEAVE = "Leave the berries",
            OPT_LEAVE_REST = "Leave the rest of the berries",
            
            DIALOG_NO_EAT_BOGBERRY = [[
                player:
                    !exit
                * You decide to leave the berries where they are.
            ]],

            DIALOG_DONE_EAT_BOGBERRY = [[
                player:
                    !exit
                * You leave the rest of the berries.
            ]]

        }
        :SetLooping()
        :Fn(function(cxt)
            
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")


                cxt.enc.scratch.berries_eaten = cxt.enc.scratch.berries_eaten or 0
                cxt.enc.scratch.berries = {}
                for k, v in ipairs(BERRIES) do
                    table.insert(cxt.enc.scratch.berries, v)
                end
                table.shuffle(cxt.enc.scratch.berries)

                while #cxt.enc.scratch.berries > 3 do
                    table.remove(cxt.enc.scratch.berries)
                end
            end

            
            for k, berry in ipairs(cxt.enc.scratch.berries) do
                cxt:Opt(berry.opt)
                    :Dialog(berry.txt)
                    :Fn(function(cxt) 
                        cxt.enc.scratch.berries_eaten = cxt.enc.scratch.berries_eaten + 1

                        berry.fn(cxt)
                        table.remove(cxt.enc.scratch.berries, k)
                        if #cxt.enc.scratch.berries == 0 then
                            cxt:Dialog("DIALOG_ATE_ALL_BERRIES")                        
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    end)    
            end
            cxt:Opt(cxt.enc.scratch.berries_eaten == 0 and "OPT_LEAVE" or "OPT_LEAVE_REST")
                :Dialog(cxt.enc.scratch.berries_eaten == 0 and "DIALOG_NO_EAT_BOGBERRY" or "DIALOG_DONE_EAT_BOGBERRY")
                :Travel()
        end)
    

--hook these up to the loc tables, so we can just type them normally.
local loctable = {}
for k, v in ipairs(BERRIES) do
    local opt_id = "BERRY_OPT"..k
    loctable[opt_id] = v.opt
    v.opt = opt_id

    local text_id = "BERRY_TEXT"..k
    loctable[text_id] = v.txt
    v.txt = text_id
end
convo:Loc(loctable)
