local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    on_init = function(quest)
        if quest.param.wait_attacked == nil then
            quest.param.wait_attacked = math.random() < .5
        end

    end
}

--------------------------------------------------------------------

QDEF:AddConvo()
    :ConfrontState("CONF", function(cxt) return cxt.location:HasTag("in_transit") end)
    :Loc{
        DIALOG_INTRO = [[
            * You hear a scuffle off the main path.
            * Drawing closer, you see a couple of erchins circle around a prone figure.
            * {victim} is unconscious and will be torn to shreds if you don't intervene. 
        ]],
        OPT_ATTACK_THE_ERCHINS = "Attack the erchins",
        DIALOG_ATTACK_THE_ERCHINS = [[
            player:
                !left
                !fight
            erchin:
                !right
            * You draw weapons and charge!
        ]],
        DIALOG_BEAT_ERCHINS = [[
            * The threat of erchin attack is over. {victim} sits up, and rubs {victim.hisher} head.
            victim:
                !right
                !injured
                Wh-what happened?
            * {victim} notices the smoldering remains of an erchin.
            victim:
                !sigh
                I chose the wrong place for a nap.
                Thank you, grifter, for helping me out.
        ]],
        OPT_ACCEPT_THANKS = "Accept {victim}'s gratitude",
        DIALOG_ACCEPT_THANKS = [[
            player:
                All in a day's work.
                Be more careful next time!
            victim:
                !happy
                Will do. See you around, friend.
                !exit
        ]],
        OPT_GET_PAID = "Accept {victim}'s money",
        DIALOG_ACCEPT_MONEY = [[
            player:
                I appreciate your gratitude, but it doesn't pay the bills.
            victim:
                Oh. Of course. I guess I do owe you.
            player:
                !take
            victim:
                !give
                Here.
                May the road take you forward.
                !exit
        ]],


        OPT_WAIT = "Watch what happens next",
        DIALOG_WAIT_ATTACKED = [[
            * The erchins start feeding.
            player:
                !wince
            * {victim} barely wakes up before it's over.
            
            erchin:
                !right
            * The erchins, whipped into a frenzy by the meat, turn their attention to you!
        ]],
        OPT_DEFEND = "Defend yourself!",
        DIALOG_DEFEND = [[
            player:
                !fight
            * You draw your weapons.
        ]],
        DIALOG_DEFEND_POST = [[
            erchin:
                !exit
            * You find some inedible items left over from the erchins' meal.
        ]],
        
        DIALOG_WAIT_NOT_ATTACKED = [[
            * The erchins start feeding.
            player:
                !wince
            * {victim} barely wakes up before it's over.
            * Groggy from the meal, the erchins wander off.
            * You find some inedible items left behind.
        ]],

        OPT_LEAVE = "Leave before the erchins see you",
        DIALOG_LEAVE = [[
            * You decide not to risk provoking the erchins, and leave as they start to feed.
        ]],
    }
    :Fn(function(cxt)
        

        local defs = {
            "RISE_REBEL",
            "SPARK_BARON_GOON",
            "LABORER",
            "HEAVY_LABORER",
        }
        cxt.quest:Complete()

        cxt.quest.param.victim = AgentUtil.GetFreeAgent(table.arraypick(defs), function(agent) return agent:GetRelationship() == RELATIONSHIP.NEUTRAL end)
        cxt.quest.param.erchins = CreateCombatParty("ERCHINS", cxt.quest:GetRank(), cxt.location) 
        cxt:ReassignCastMember("erchin", cxt.quest.param.erchins[1])
        cxt:ReassignCastMember("victim", cxt.quest.param.victim)
        local DEATH_MONEY = 50 + 20*cxt.quest:GetRank()

        cxt:Dialog("DIALOG_INTRO")

        cxt:Opt("OPT_ATTACK_THE_ERCHINS")
            :Robot("ATTACK")
            :Dialog("DIALOG_ATTACK_THE_ERCHINS")
            :Battle{
                enemies = cxt.quest.param.erchins,
                flags = BATTLE_FLAGS.NO_BYSTANDERS,
            }
                :OnWin()
                    :Dialog("DIALOG_BEAT_ERCHINS")
                    :Fn(function() 
                        
                        cxt:TalkTo(cxt.quest.param.victim)
                        cxt:Opt("OPT_ACCEPT_THANKS")
                            :ReceiveOpinion(OPINION.SAVED_LIFE)
                            :Travel()

                        cxt:Opt("OPT_GET_PAID")
                            :ReceiveMoney(DEATH_MONEY)
                            :Travel()
                    end)

        cxt:Opt("OPT_WAIT")
            :Robot("WAIT")
            :Fn(function()
    
                local loot = {money = DEATH_MONEY, items = {}}
                AgentUtil.CollectDeathLoot(cxt.quest.param.victim, loot)
                cxt.quest.param.victim:Kill()

                if cxt.quest.param.wait_attacked then
                    cxt:Dialog("DIALOG_WAIT_ATTACKED")
                    cxt:Opt("OPT_DEFEND")
                        :Battle{
                            enemies = cxt.quest.param.erchins,
                            flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.SELF_DEFENCE,
                        }
                            :OnWin()
                                :Dialog("DIALOG_DEFEND_POST")
                                :Fn(function() cxt.enc:AwardLoot(loot, { found=true } ) end)
                                :Travel()

                else
                    cxt:Dialog("DIALOG_WAIT_NOT_ATTACKED")
                    cxt.enc:AwardLoot(loot, { found=true } )
                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            end)
        

        cxt:Opt("OPT_LEAVE")
            :Robot("LEAVE")
            :Dialog("DIALOG_LEAVE")
            :Fn(function() cxt.quest.param.victim:Kill() end)
            :Travel()
    end)
       

QDEF:AddTest(TestCreator()
                :AddBoolParams{"wait_attacked"}
                :AddChoices{"WAIT", "ATTACK", "LEAVE"})
