local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local PET_SUPPORT_UNUPGRADED = {"untrained_pet"}
local CROP_PRICE = 150
local EGG_PRICE = 75
local TREAT_PRICE = 75
local PET_OPTIONS = {"VROC_UPGRADED", "CRAYOTE_UPGRADED", "ERCHIN_UPGRADED", "SHROOGLET_UPGRADED"}

Content.AddBattleCard( "crop", 
{
    name = "Sic 'em!",
    anim = "taunt",

    desc = "Order your pet to take their action and prepare a new one.",
    flavour = "'Try clicking your tongue twice.'",
    cost = 1,

    flags = CARD_FLAGS.EXPEND | CARD_FLAGS.ITEM,
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    local_item = LOCAL_ITEMS.BOG,

    target_type = TARGET_TYPE.FRIENDLY,

    CanPlayCard = function( self, battle, target )
        return target == nil or target:GetAgent():IsPet()
    end,

    OnPostResolve = function( self, battle, attack )
        for i, hit in attack:Hits() do
            if hit.target:GetAgent():IsPet() then
                hit.target:PlayBehaviour()
                hit.target.last_turn = battle:GetTurns() - 1
                hit.target:PrepareTurn()
            end
        end
    end,
})

Content.AddBattleCard( "courage_crunch", 
{
    name = "Courage Crunch",
    anim = "taunt",
    flavour = "'Keeping your crayotes confident and your vrocs valiant!'",
    desc = "Heal your pet {1} and give them {2} {POWER}.",
    desc_fn = function( self, fmt_str )
        return loc.format(fmt_str, self.heal_amt, self.power_amt)
    end,

    cost = 1,

    flags = CARD_FLAGS.EXPEND | CARD_FLAGS.ITEM,
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    local_item = LOCAL_ITEMS.BOG,

    target_type = TARGET_TYPE.FRIENDLY,
    heal_amt = 8,
    power_amt = 2,

    CanPlayCard = function( self, battle, target )
        return target == nil or target:GetAgent():IsPet()
    end,

    OnPostResolve = function( self, battle, attack )
        for i, hit in attack:Hits() do
            if hit.target:GetAgent():IsPet() then
                hit.target:HealHealth(self.heal_amt)
                hit.target:AddCondition("POWER", self.power_amt, self)
            end
        end
    end,
})

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    precondition = function(quest) 
        return TheGame:GetGameState():GetCaravan():GetPet()
    end,
}
--------------------------------------------------------------------
:AddCastByAlias{
    cast_id = "trainer",
    alias = "BEASTMASTER",
}

:AddCast{
    cast_id = "pet",
    when = QWHEN.MANUAL,
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_START"
    end)
    :State("STATE_START")
        :Loc{
            DIALOG_INTRO_MECH = [[
                player:
                    !left
                agent:
                    !right
                    !wave
                    Hey! That's a mighty fine mech you've got there!
                    Might you be interested in making a little trade?
                player:
                    A trade? What are you offering in return?
                agent:
                    Your mech for my beast.
                    Are you interested?
            ]],
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !wave
                    Hey there! What a handsome beast you have!
                player:
                    Thanks! It also comes in handy in a tight spot.
                agent:
                    I bet! I can help you train it if you'd like.
                    I would also be willing to take it off your hands in exchange, if you're willing to part with it.
            ]],
            OPT_TRADE_MECH = "Trade {pet} for {2#agent}",
            DIALOG_TRADE_MECH = [[
                player:
                    Sounds like an upgrade, so sure!
                agent:
                    Well I'd like to think so, yeah.
                    I can probably get more value out of that mech than you get having it as a fighting partner.
                player:
                    It's a win-win deal then. 
                    !give
                agent:
                    !take
            ]],
            OPT_ASK_ABOUT_TRADE = "Ask what they will do with {pet}",
            DIALOG_ASK_ABOUT_TRADE = [[
                player:
                    So what exactly are you planning to do with {pet} if we trade?
                agent:
                    Oh don't worry, I'm not going to disassemble {pet.himher} or anything.
                    I love animals, even the mechanical kind! I'll make some improvements to it and maybe teach it a few new tricks.
                    After that I'll find someone to sell it to, find it a new home.
                player:
                    Hmm, that doesn't sound as bad as I was expecting.
            ]],
            OPT_TRAIN_PET = "Train {pet}",
            DIALOG_TRAIN_PET = [[
                player:
                    You think you can teach it some new tricks?
                agent:
                    Oh definitely!
                * {agent} helps you run through some exercises with {pet}
            ]],
            REQ_ALREADY_TRAINED = "Your pet is already trained.",
            OPT_LOOK_AT_WARES = "Look at {agent}'s wares",
            DIALOG_LOOK_AT_WARES = [[
                player:
                    What kinds of things do you have for sale?
                agent:
                    Well I don't have the biggest selection, but I assure you everything is of the highest quality.
            ]],
            OPT_BUY_CROP = "Buy {1#card}",
            DIALOG_BUY_CROP = [[
                player:
                    That thing seems useful.
                agent:
                    It sure is! It'll help you keep your pet under control.
                player:
                    I'll take it!
                    !take
                agent:
                    !give
            ]],
            OPT_BUY_EGG = "Buy {1#card}",
            DIALOG_BUY_EGG = [[
                player:
                    I'll take the egg.
                agent:
                    Perfect! You never know when they'll hatch, so be prepared.
            ]],
            OPT_BUY_TREATS = "Buy {1#card}",
            DIALOG_BUY_TREATS = [[
                player:
                    I'll take the treats. I'm sure that will make {pet} happy.
                agent:
                    I've never met a creature that didn't just love those things!
            ]],
            OPT_DONATE_PET = "Give {pet} to {agent}",
            TT_DONATE = "You will receive {1#card}.",
            DIALOG_DONATE_PET = [[
                player:
                    Hmm, yeah I'd be willing to give you {pet}.
                    !give
                agent:
                    !take
                    Perfect! I promise you I'll take care of {pet.himher}!
                * Your pet looks up at you with a confused look as {agent} puts a leash on {pet.himher}.
            ]],
            OPT_DECLINE = "Leave",
            DIALOG_DECLINE = [[
                player:
                    Ehh, sorry I'm not really looking to buy anything today.
                agent:
                    Ah, very well then. Perhaps another time!
            ]],
            DIALOG_DECLINE_BOUGHT_ITEM = [[
                player:
                    I think I've got everything I need right now.
                agent:
                    Alright, well thank you for your business. I hope we meet again!
            ]],
        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("trainer"))
                cxt.enc.scratch.pet = TheGame:GetGameState():GetCaravan():GetPet()
                cxt.quest:AssignCastMember("pet", cxt.enc.scratch.pet)
                if cxt.enc.scratch.pet:GetSpecies() == SPECIES.MECH then
                    local animal = Agent(PET_OPTIONS[math.random(#PET_OPTIONS)])
                    TheGame:GetGameState():AddAgent( animal )
                    animal:GenerateRandomName()
                    cxt.enc.scratch.animal = animal
                    cxt:Dialog("DIALOG_INTRO_MECH")
                else
                    cxt:Dialog("DIALOG_INTRO")
                end
            end

            if cxt.enc.scratch.pet:GetSpecies() == SPECIES.MECH then
                cxt:Opt("OPT_TRADE_MECH", cxt.enc.scratch.pet:GetName(), cxt.enc.scratch.animal)
                    :Dialog("DIALOG_TRADE_MECH")
                    :Fn(function(cxt)
                        cxt.enc.scratch.pet:Dismiss()
                        cxt.enc.scratch.animal:Recruit(PARTY_MEMBER_TYPE.CREW)
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)

                cxt:Opt("OPT_ASK_ABOUT_TRADE")
                    :Dialog("DIALOG_ASK_ABOUT_TRADE")
                    :ReqCondition(not cxt.enc.scratch.asked_about_trade)
                    :Fn(function(cxt)
                        cxt.enc.scratch.asked_about_trade = true
                    end)
            else
                cxt:Opt("OPT_TRAIN_PET")
                    :ReqCondition( table.contains(PET_SUPPORT_UNUPGRADED, cxt.enc.scratch.pet.support_negotiation), "REQ_ALREADY_TRAINED" )
                    :Dialog("DIALOG_TRAIN_PET")
                    :Fn(function(cxt)
                        cxt.enc.scratch.pet:TrainPet()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)

                cxt:Opt("OPT_LOOK_AT_WARES")
                    :Dialog("DIALOG_LOOK_AT_WARES")
                    :LoopingFn(function(cxt)
                        cxt:Opt("OPT_BUY_CROP", "crop")
                            :ReqCondition(not cxt.enc.scratch.bought_crop)
                            :Dialog("DIALOG_BUY_CROP")
                            :DeliverMoney(CROP_PRICE)
                            :GainCards{"crop"}
                            :Fn(function(cxt)
                                cxt.enc.scratch.bought_crop = true
                                cxt.enc.scratch.bought_item = true
                            end)

                        cxt:Opt("OPT_BUY_EGG", "flead_egg")
                            :ReqCondition(not cxt.enc.scratch.bought_egg)
                            :Dialog("DIALOG_BUY_EGG")
                            :DeliverMoney(EGG_PRICE)
                            :GainCards{"flead_egg"}
                            :Fn(function(cxt)
                                cxt.enc.scratch.bought_egg = true
                                cxt.enc.scratch.bought_item = true
                            end)

                        cxt:Opt("OPT_BUY_TREATS", "courage_crunch")
                            :ReqCondition(not cxt.enc.scratch.bought_crunch)
                            :Dialog("DIALOG_BUY_TREATS")
                            :DeliverMoney(TREAT_PRICE)
                            :GainCards{"courage_crunch"}
                            :Fn(function(cxt)
                                cxt.enc.scratch.bought_crunch = true
                                cxt.enc.scratch.bought_item = true
                            end)

                        StateGraphUtil.AddBackButton(cxt)
                    end)

               

                cxt:Opt("OPT_DONATE_PET")
                    :PostText("TT_DONATE", "emergency_shield_generator")
                    :PostCard( "emergency_shield_generator", true )
                    :Dialog("DIALOG_DONATE_PET")
                    :GainCards{"emergency_shield_generator"}
                    :Fn(function(cxt)
                        cxt.enc.scratch.pet:Dismiss()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end

            cxt:Opt("OPT_DECLINE")
                :Fn(function(cxt)
                    if cxt.enc.scratch.bought_item then
                        cxt:Dialog("DIALOG_DECLINE_BOUGHT_ITEM")
                    else
                        cxt:Dialog("DIALOG_DECLINE")
                    end
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
    end)

