local RESOLVE_LOSS = 10

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    precondition = function(quest)
        local resolve, max_resolve = TheGame:GetGameState():GetCaravan():GetResolve()
        return max_resolve > RESOLVE_LOSS
    end,
}

--------------------------------------------------------------------

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_FOG"
    end)

    :State("STATE_FOG")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * While walking through the bog, you enter a particularly misty patch of woods.
                * Navigation is hard and you feel as if you've been walking in circles.
                * The mist is making you dizzy and staying focused is a struggle.
                player:
                    !palm
                    I really need to get out of here.
            ]],
            OPT_FORGET_CARD = "Focus only on the path forward (Remove <!card_1>{1}</>)",
            DIALOG_FORGET_CARD = [[
                * You clear your mind of everything and focus solely on getting out of the fog.
                * You eventually make it out but you feel like you left a piece of you behind.
                player:
                    I hope I never have to do that again.
            ]],
            OPT_GET_TIPSY = "Run until you're out (Gain 2 {1#card})",
            DIALOG_GET_TIPSY = [[
                * Concerned about the effects of the mist, you begin running and don't stop until you're out of the mist.
                * Once in the clear, you stop to catch your breath and feel a bit queasy.
                player:
                    !tired
                    Maybe running wasn't such a great idea.
            ]],
            OPT_LOSE_RESOLVE = "Exert yourself to get out (Lose {1} Resolve)",
            DIALOG_LOSE_RESOLVE = [[
                * You calm yourself and try to stay fully alert despite the mist.
                * When you eventually make it out, you feel tired and drained.
                player:
                    !tired
                    I could use a break right about now.
            ]],
        }

        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")
                local battle_cards = TheGame:GetGameState():GetPlayerAgent().battler:GetCards()
                cxt.enc.scratch.forget_card = battle_cards[math.random(#battle_cards)]
            end

            local opt = cxt:Opt("OPT_FORGET_CARD", cxt.enc.scratch.forget_card:GetName())
                :PostCard( cxt.enc.scratch.forget_card )
                :Dialog("DIALOG_FORGET_CARD")
                :Fn(function(cxt)
                    TheGame:GetGameState():GetPlayerAgent().battler:RemoveCard(cxt.enc.scratch.forget_card)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
            opt:SetHoverLinkParameters( cxt.enc.scratch.forget_card )


            cxt:Opt("OPT_GET_TIPSY", "drunk_player")
                :Dialog("DIALOG_GET_TIPSY")
                :Fn(function(cxt)
                    cxt:ForceTakeCards( {"drunk_player", "drunk_player"} )
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_LOSE_RESOLVE", RESOLVE_LOSS)
                :Dialog("DIALOG_LOSE_RESOLVE")
                :Fn(function(cxt)
                    TheGame:GetGameState():GetCaravan():DeltaResolve(-RESOLVE_LOSS)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)

