local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
}

local CODES = {
    ATTACK = "0x41.0x54.0x54.0x41.0x43.0x4B",
    EXPLODE = "0x45.0x58.0x50.0x4C.0x4F.0x44.0x45",
    FOLLOW = "0x46.0x4F.0x4C.0x4C.0x4F.0x57",
    SCRAP = "0x53.0x43.0x52.0x41.0x50",
    BEACON = "0x42.0x45.0x41.0x43.0x4F.0x4E",
}

QDEF:AddConvo()
    :ConfrontState("CONFRONT", function() return true end)
        :Loc{
            DIALOG_INTRO = [[
                * You find evidence of a fight, and a damaged Enforcer. 
                player:
                    !left
                mech:
                    !injured
                    !right

                * It seems to be stuck in some kind of a diagnostic loop. 
                * Its maintainance panel is open, exposing its override controls.
            ]],
        }

        :Fn(function(cxt)
            cxt.quest:Complete()
            
            local mech = Agent("SPARK_BARON_AUTOMECH")
            TheGame:GetGameState():AddAgent( mech )
            cxt:ReassignCastMember("mech", mech)
            mech:MoveToLocation(cxt.location)
            cxt.enc:SetPrimaryCast(mech)
            mech:GetAspect("health"):SetPercent(.3)

            cxt:Dialog("DIALOG_INTRO")
            cxt.enc.scratch.options = table.shuffle{"ATTACK", "EXPLODE", "FOLLOW", "SCRAP", "BEACON"}
            cxt.enc.scratch.idx = 1
            cxt:GoTo("STATE_MENU")
        end)
    :State("STATE_MENU")
        :SetLooping()
        :Loc{
            OPT_PREV = "Press 'PRV'",
            OPT_NEXT = "Press 'NXT'",            
            OPT_SELECT = "Press 'SEL'",
            DIALOG_SELECT = [[
                player:
                    !give
                * You press the SEL button.
            ]],
            OPT_LEAVE = "Leave the automech alone",
            DIALOG_LEAVE = [[
                * You decide to leave the automech alone.
            ]],
            DIALOG_SHOW_CODE = [[* The small output window reads: '<b>{1}</>']],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_SHOW_CODE", CODES[cxt.enc.scratch.options[cxt.enc.scratch.idx]])

            cxt:Opt("OPT_PREV")
                :Fn(function() 
                    cxt.enc.scratch.idx = cxt.enc.scratch.idx > 1 and cxt.enc.scratch.idx - 1 or #cxt.enc.scratch.options
                end)

            cxt:Opt("OPT_SELECT")
                :Dialog("DIALOG_SELECT")
                :Fn(function()
                    cxt:GoTo("STATE_".. cxt.enc.scratch.options[cxt.enc.scratch.idx])
                end)

            cxt:Opt("OPT_NEXT")
                :Fn(function() 
                    cxt.enc.scratch.idx = cxt.enc.scratch.idx < #cxt.enc.scratch.options and cxt.enc.scratch.idx + 1 or 1
                    
                end)

            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :Travel()
        end)

    :State("STATE_ATTACK")
        :Loc{
            DIALOG_INTRO = [[
                * The automech springs to life.
                agent:
                    !fight
                    ATTACK ROUTINE SELECTED
                    TARGET LOCKED IN
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[  
                player:
                    !fight
                * You draw your guns just in time.
            ]],
            DIALOG_WON = [[
                * You leave the heap of scrap metal where you found it.
            ]]
        }
        :Fn(function(cxt) 
            
            local HEALTH_PERCENT = {.25, .33, .5, .67, .75}
            local percent_health = HEALTH_PERCENT[cxt.quest:GetRank()] or HEALTH_PERCENT[#HEALTH_PERCENT]
            cxt:GetAgent().health:SetPercent(percent_health)
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function() 
                        cxt:Dialog("DIALOG_WON")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }
        end)

    :State("STATE_EXPLODE")
        :Loc{
            DIALOG_INTRO = [[
                * The automech starts beeping.
                agent:
                    DESTRUCT CHARGES ACTIVATED
                    3..2..1...
                    !exit
                player:
                    !flinch
                * The automech explodes, sending shrapnel in your direction.
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            local hp = cxt.player:GetHealth()
            ConvoUtil.DoHealthDelta(cxt, -math.min(10, hp-1))
            StateGraphUtil.AddLeaveLocation(cxt)
        end)

    :State("STATE_FOLLOW")
        :Loc{
            DIALOG_INTRO = [[
                * The automech spools up.
                agent:
                    !neutral
                    FOLLOW MODE ACTIVATED
                    AWAITING FURTHER INSTRUCTION

                * Looks like you just made a friend.
            ]],
            DIALOG_NOPE = [[
                * The automech sputters.
                agent:
                    ERROR
                * Odd.
            ]]
        }
        :Fn(function(cxt) 

            local can_have = cxt.caravan:CanOwnPet()
            if can_have then
                cxt:Dialog("DIALOG_INTRO")
                cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.CREW )
                StateGraphUtil.AddLeaveLocation(cxt)
            else
                cxt:Dialog("DIALOG_NOPE")
                cxt:GoTo("STATE_MENU")
            end
        end)

    :State("STATE_SCRAP")
        :Loc{
            DIALOG_INTRO = [[
                * The automech emits a high-pitched whine.
                agent:
                    SCRAPPING INTERNAL COMPONENTS FOR RECOVERY
                    !exit
                * The 'mech falls into its constituent parts. You examine the pile, looking for useful bits.
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            local loot = {money = 0, items = {"spark_grenade", "portable_bunker"}}
            AgentUtil.CollectDeathLoot(cxt:GetAgent(), loot)
            cxt:GetAgent():Kill()
            cxt.enc:AwardLoot(loot, { found=true } )
            StateGraphUtil.AddLeaveLocation(cxt)

        end)

    :State("STATE_BEACON")
        :Loc{
            DIALOG_INTRO = [[
                * A small antenna extends from the back of the robot.
                mech:
                    BEACON ACTIVATED
                * You hear a shout from the surrounding bog.
            ]],

            DIALOG_FRIEND = [[
               baron:
                    !right
                    There it is! 
                    !happy
                    Hey {player}, what are you doing here?
            ]],
            DIALOG_NEUTRAL = [[
                baron:
                    !right
                    There it is! 
                    Step back from the hardware, grifter. That's Spark Baron property.
            ]],
            DIALOG_ENEMY = [[
                baron:
                    !right
                    There it is! 
                    What did you do here? There's blood on your hands, grifter.
            ]],

            OPT_NEGOTIATION_LET_GO = "Convince {agent} to let you go",
            DIALOG_NEGOTIATION_LET_GO = [[
                player:
                    I found it like this, I swear!
            ]],
            DIALOG_CAN_GO = [[
                agent:
                    I don't believe you, but I've got a lot of paperwork to do.
                    Go on, get out of here!
                player:
                    !exit
                * You don't stick around to press your case further.
            ]],
            DIALOG_CAN_NOT_GO = [[
                agent:
                    A likely story.
                    I'm going to bring in your bloody corpse for the reward.
                    !fight
            ]],
            DIALOG_DEFEND_PST = [[
                player:
                    !fight
                    You should have listened to me.
            ]],
            DIALOG_DEFEND = [[
                player:
                    !fight
                * You ready your best defense.
            ]],
            DIALOG_WON_FIGHT = [[
                player:
                    !exit
                    You leave, fearing reinforcements.
            ]],
            OPT_NEGOTIATION_REWARD = "Negotiate for a reward",

            DIALOG_NEGOTIATION_REWARD = [[
                player:
                    I found this mech and activated the beacon for you to come pick it up.
                    That has to be worth something, doesn't it?
            ]],
            
            DIALOG_GIVE_REWARD = [[
                agent:
                    Yeah, you're right. You do deserve the finder's fee.
                    !give
                    Here you go.
                player:
                    Thanks, friend. See you around.
                    !exit
                * You leave before {agent} changes {agent.hisher} mind.
            ]],
            DIALOG_NO_REWARD = [[
                agent:
                    We were right over there. We would have found it anyway.
                    That reward is mine, grifter.
            ]],
            DIALOG_LEAVE = [[
                * You leave, rather than debate.
            ]]



        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            local barons = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank(), cxt.location, true)
            cxt:ReassignCastMember("baron", barons[1])
            ConvoUtil.RelationshipDialog( cxt, barons[1], "DIALOG_NEUTRAL", "DIALOG_FRIEND", "DIALOG_ENEMY")
            cxt.enc:SetPrimaryCast(barons[1])


            if barons[1]:GetRelationship() < RELATIONSHIP.NEUTRAL then
                
                cxt:Opt("OPT_NEGOTIATION_LET_GO")
                    :Dialog("DIALOG_NEGOTIATION_LET_GO")
                    :Negotiation{
                        on_success = function() 
                            cxt:Dialog("DIALOG_CAN_GO")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function() 
                            cxt:Dialog("DIALOG_CAN_NOT_GO")
                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND_PST")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    bystanders = {cxt.quest.param.robot},
                                    on_win = function(cxt) 
                                        cxt:Dialog("DIALOG_WON_FIGHT")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }
                        
                        end,

                    }
                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        advantage = TEAM.BLUE,
                        bystanders = {cxt.quest.param.robot},
                        on_win = function(cxt) 
                            cxt:Dialog("DIALOG_WON_FIGHT")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    }

            else
                cxt:Opt("OPT_NEGOTIATION_REWARD")
                    :Dialog("DIALOG_NEGOTIATION_REWARD")
                    :Negotiation{
                        on_success = function() 
                            cxt:Dialog("DIALOG_GIVE_REWARD")
                            cxt.caravan:AddMoney(100)
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function() 
                            cxt:Dialog("DIALOG_NO_REWARD")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }
                
                cxt:Opt("OPT_LEAVE")
                    :Dialog("DIALOG_LEAVE")
                    :Travel()
            end

        end)
