
local function GetParasiteCards()

    local negotiation_defs = require "negotiation/negotiation_defs"
    local battle_defs = require "battle/battle_defs"
    local fun = require "util/fun"

    local negotiation_cards = fun(TheGame:GetGameState():GetPlayerAgent().negotiator:GetCards())
                                :filter(function(card) return CheckBits( card.def.flags, negotiation_defs.CARD_FLAGS.PARASITE) end)
                                :value()
    
    local battle_cards = fun(TheGame:GetGameState():GetPlayerAgent().battler:GetCards())
                                :filter(function(card) return CheckBits( card.def.flags, battle_defs.CARD_FLAGS.PARASITE) end)
                                :value()

    local ret = {}
    for k,v in ipairs(negotiation_cards) do
        table.insert(ret, v)
    end
    for k,v in ipairs(battle_cards) do
        table.insert(ret, v)
    end
    return ret
end


local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    rank = { 2, 5 },
    precondition = function(quest) 
        return #GetParasiteCards() > 0 and TheGame:GetGameState():GetCaravan():CanOwnPet()
    end,
}


QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit")  end)
        :Loc{
            DIALOG_INTRO = [[
                
                * A lumicyte comes bounding forward.
                lumicyte:
                    !right
                player:
                    !left
                    !placate
                    Whoa there!

                * Its shimmering, irisless eyes fixate upon you, but it doesn't seem to be aggressive.
                hesh:
                    !right
                    What did you find, {lumicyte}?
                    !crossed
                    A grifter? What's so special about {player.himher}?
                * The zealot stares through you. After a moment, {hesh.heshe} recoils as if slapped.
                hesh:
                    !hesh_greeting
                    Hesh be silenced!
                    You're corrupted by the bog!
                    What have you done to yourself?
            ]],

            OPT_EXPLAIN_BITE = "Explain the bite",
            DIALOG_EXPLAIN_BITE = [[
                player:
                    It was just a little insect bite. I'm sure it will pass.
                hesh:
                    No. The corruption must be burned out.
                player:
                    That sounds painful.
                hesh:
                    It's less painful than facing the Wrath of Hesh. You are endangering the entire region by harboring that parasite.
            ]],

            OPT_INSULT = "Tell the cultist to mind {hesh.hisher} own business",
            DIALOG_INSULT = [[
                player:
                    That is a very personal question, 'friend'. 
                    !cruel
                    Dangerously personal.
                hesh:
                    Not as dangerous as the Wrath of Hesh. You are endangering the entire region by harboring that parasite.

            ]],
            DIALOG_CONTINUE = [[
                hesh:
                    I will purge the corruption. Please be completely still.

                * The cultist draws out a thin, lumin-tipped blade. The Lumicyte gurgles excitedly.
            ]],



        }
        :Fn(function(cxt)
            cxt.quest:Complete()

            local lumicyte = TheGame:GetGameState():AddSkinnedAgent("LUMICYTE_UPGRADED")
            local hesh = TheGame:GetGameState():AddSkinnedAgent(cxt.quest:GetRank() <= 3 and "PRIEST" or "ZEALOT")
            
            
            lumicyte:JoinParty(hesh)
            lumicyte:AddTag("NOT_WILD")
            lumicyte:GenerateRandomName()

            cxt:ReassignCastMember("lumicyte", lumicyte)
            cxt:ReassignCastMember("hesh", hesh)

            cxt:TalkTo(hesh)
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_EXPLAIN_BITE")
                :Dialog("DIALOG_EXPLAIN_BITE")
                :Dialog("DIALOG_CONTINUE")
                :GoTo("STATE_REMOVAL")
            
            cxt:Opt("OPT_INSULT")
                :Dialog("DIALOG_INSULT")
                :Dialog("DIALOG_CONTINUE")
                :GoTo("STATE_REMOVAL")
                :GoTo("STATE_REMOVAL")
        end)

    :State("STATE_REMOVAL")
        :Loc{
            OPT_ASK_ABOUT_REMOVAL = "Ask about removing the parasite",
            DIALOG_ASK_ABOUT_REMOVAL = [[
                player:
                    You want to cut it out of me?
                hesh:
                    Yes. And then I will destroy it safely, so that Hesh may not see it.
                player:
                    Where will this leave me?
                hesh:
                    In pain. And perhaps a little bit lighter.
                    But at least you will face Hesh alone, and not infested with impurity.
            ]],
            TT_LOSE_HEALTH = "<#PENALTY> You will lose {1#card_list} and {2} Max HP! </>",
            OPT_ALLOW_REMOVE = "Allow {hesh} to remove the parasites",
            DIALOG_ALLOW_REMOVE = [[
                player:
                    ...
                    Do it.
                    !exit
                hesh:
                    !exit
                * The cultist goes about {hesh.hisher} work.
                * It hurts. 
                * <i>A lot.</>
            ]],

            OPT_REFUSE = "Refuse",
            DIALOG_REFUSE = [[
                player:
                    No, I think I'll keep myself intact, thank you very much.
                hesh:
                    !fight
                    It wasn't a question.
                player:
                    !fight
                * As weapons are drawn, the lumicyte jumps to your side!
            ]],

            DIALOG_CAN_TAKE_LUMICYTE = [[
                lumicyte:
                    !right
                * The lumicyte wags its tail and looks up at you, expectantly.
            ]],

            DIALOG_WON = [[
                {dead?
                    * The cultist slumps the ground, and the bog starts to digest {hesh.himher}.
                }
                {not dead?
                    hesh:
                        !exit
                    * The cultist flees into the bog, clutching {hesh.hisher} wounds.
                }
            ]],


            OPT_CONVINCE_MISTAKE = "Convince {hesh} that {hesh.heshe} is mistaken",
            DIALOG_CONVINCE_MISTAKE = [[
                player:
                    I can assure you that I am fine. 
                    There's no bog riding around in my veins.
            ]],
            DIALOG_CONVINCE_MISTAKE_SUCCESS = [[
                agent:
                    Perhaps you are right.
                    This lumicyte is, perhaps, defective.
                    It shall be destroyed.
            ]],
            DIALOG_CONVINCE_MISTAKE_FAILURE = [[
                agent:
                    No, the lumicytes are never wrong. They have an eighth sense for these things.
            ]],
            DIALOG_POST_REMOVAL = [[
                player:
                    !left
                    !injured
                hesh:
                    !right
                    it is done.
                    !give
                * The cultist tosses the removed material to the lumicyte, who slurps it down greedily.
                lumicyte:
                    !right
                    Guuuuuuuuuuuurgh.
                hesh:
                    !right
                    Now if you will excuse me, I must destroy this beast.
                player:
                    !neutral
            ]],


        }
        :Fn(function(cxt)
            local parasites = GetParasiteCards()
            cxt:RunLoop(function() 
                cxt:Question("OPT_ASK_ABOUT_REMOVAL", "DIALOG_ASK_ABOUT_REMOVAL")
                
                cxt:Opt("OPT_ALLOW_REMOVE")
                    :Dialog("DIALOG_ALLOW_REMOVE")
                    :PostText("TT_LOSE_HEALTH", parasites, #parasites*PARASITE_HEALTH_LOSS)
                    :Fn(function() 
                        
                        AUDIO:PlayEvent("event:/sfx/general/conversation/damage_taken_gen")
                        for k,v in pairs(parasites) do
                            cxt.player.negotiator:RemoveCard(v)
                            cxt.player.battler:RemoveCard(v)
                        end
                        cxt.enc:GetScreen():FireHealthOverlay( 0xee5d5faf ) -- punch to red

                        cxt.quest.param.beast_has_parasite = true
                    end)
                    :Dialog("DIALOG_POST_REMOVAL")
                    :GoTo("STATE_SAVE_LUMICYTE")
                
                cxt:Opt("OPT_CONVINCE_MISTAKE")
                    :Dialog("DIALOG_CONVINCE_MISTAKE")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_MISTAKE_SUCCESS")
                            :GoTo("STATE_SAVE_LUMICYTE")
                        
                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_MISTAKE_FAILURE")


                cxt:Opt("OPT_REFUSE")
                    :Dialog("DIALOG_REFUSE")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        allies = {cxt.enc:GetCastAgent("lumicyte")}
                    }
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :Fn(function() 
                                if cxt.enc:GetCastAgent("lumicyte"):IsRetired() then
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                else
                                    cxt:Dialog("DIALOG_CAN_TAKE_LUMICYTE")
                                    cxt:GoTo("STATE_KEEP_PET")
                                end
                            end)

            end)

        end)


    :State("STATE_SAVE_LUMICYTE")
        :Loc{

            OPT_QUESTION_BEAST_DESTROY = "Ask why lumicyte needs to be destroyed",
            DIALOG_QUESTION_BEAST_DESTROY = [[
                player:
                    Wait. Why must it be destroyed?
                
                {beast_has_parasite?
                    hesh:
                        To guarantee that the corruption will not return. 
                        It's an ancient and holy rite.
                        You wouldn't understand.
                }
                {not beast_has_parasite?
                    hesh:
                        This beast is just a tool in the Cult's toolbox.
                        Broken tools must be cast back into the sea, so that Hesh may renew them.
                    player:
                        ...
                    hesh:
                        It's a metaphor!
                }
            ]],

            OPT_PROTECT_THE_BEAST = "Protect the lumicyte",
            DIALOG_PROTECT_THE_BEAST = [[
                player:
                    !fight
                    I'm not going to let you do that.
            ]],
            DIALOG_WON = [[
                {dead?
                    * The cultist slumps the ground, and the bog starts to digest {hesh.himher}.
                }
                {not dead?
                    hesh:
                        !exit
                    * The cultist flees into the bog, clutching {hesh.hisher} wounds.
                }
            ]],
            DIALOG_LUMICYTE_DIED = [[
                {beast_has_parasite ?
                    player:
                        !disgust
                    * The lumicyte's body twitches as the parasites work their way back out. You turn away in disgust.
                }
                {not beast_has_parasite?
                    * The lumicyte is already sinking into the bog. You wonder what Hesh would think of that.

                }
            ]],
            DIALOG_LUMICYTE_GONE = [[
                * The lumicyte is nowhere to be seen. Only Hesh can say what will become of it now.
            ]],

            OPT_CONVINCE_GIVE_BEAST = "Convince {hesh} to hand over the lumicyte",
            DIALOG_CONVINCE_GIVE_BEAST = [[
                player:
                    Let me take the beast. I will see that it is properly dealt with.
            ]],
            DIALOG_CONVINCE_GIVE_BEAST_SUCCESS = [[
                hesh:
                    !sigh
                    Thank you. I hate that part of the job.
                    !exit
                * The cultist packs up {hesh.hisher} tools and leaves.
            ]],
            DIALOG_CONVINCE_GIVE_BEAST_FAILURE = [[
                hesh:
                    !angry_accuse
                    This is the work of Hesh, not some back alley grift to be done with soiled hands!
            ]],

            OPT_LEAVE = "Leave the cultist",
            DIALOG_LEAVE = [[
                player:
                    I'm not staying to watch this.
                    !exit
                lumicyte:
                    !left
                * The noises you hear as you leave the scene will revisit you in your weaker moments for years to come.
            ]],


            OPT_BUY_THE_LUMICYTE = "Offer a 'donation' for the lumicyte",
            DIALOG_BUY_THE_LUMICYTE = [[
                player:
                    What if I offered you a 'donation'?
                agent:
                    Are you trying to bribe a member of the Cult of Hesh to subvert their holy duties?!
                player:
                    Well...
                    Yeah.
                agent:
                    Great. Just making sure.
                    !take
                player:
                    !give
                agent:
                    Wouldn't want there to be a misunderstanding.
                    !exit
                * The cultist leaves the lumicyte in your care.
                lumicyte:
                    !right
            ]],

            DIALOG_KEEP_PET = [[
                {beast_has_parasite?
                    lumicyte:
                        !right
                        Guuuugh.
                    * The lumicyte lists from side to side, laboring under the effort to digest the parasite.
                    * It coughs up a writing tendril still covered in your blood, stares at it quizzically, and then eats it again.
                    player:
                        Charming.
                }
                {not beast_has_parasite?
                    lumicyte:
                        !right
                    * The lumicyte seems to have taken a liking to you. Or maybe it just wants to eat the parasites that are running rampant through your body.
                    * Either way, it looks up at you, expectantly.
                }
            ]],


        }  
    :Fn(function(cxt) 

        cxt:RunLoop(function() 

            cxt:Question("OPT_QUESTION_BEAST_DESTROY", "DIALOG_QUESTION_BEAST_DESTROY")
            
            cxt:Opt("OPT_BUY_THE_LUMICYTE")
                :Dialog("DIALOG_BUY_THE_LUMICYTE")
                :DeliverMoney(cxt.quest:GetRank()*50)
                :GoTo("STATE_KEEP_PET")

            cxt:Opt("OPT_PROTECT_THE_BEAST")
                :Dialog("DIALOG_PROTECT_THE_BEAST")
                :Battle{
                    allies = {cxt.enc:GetCastAgent("lumicyte")}
                }
                    :OnWin()
                        :Dialog("DIALOG_WON")
                        :Fn(function(cxt) 
                            if cxt.enc:GetCastAgent("lumicyte"):IsDead() then
                                cxt:Dialog("DIALOG_LUMICYTE_DIED")
                                StateGraphUtil.AddLeaveLocation(cxt) 
                            elseif cxt.enc:GetCastAgent("lumicyte"):IsRetired() then
                                cxt:Dialog("DIALOG_LUMICYTE_GONE")
                                StateGraphUtil.AddLeaveLocation(cxt) 
                            else
                                cxt:Dialog("DIALOG_KEEP_PET")
                                cxt:GoTo("STATE_KEEP_PET")
                            end
                        end)

            cxt:Opt("OPT_CONVINCE_GIVE_BEAST")
                :Dialog("DIALOG_CONVINCE_GIVE_BEAST")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_CONVINCE_GIVE_BEAST_SUCCESS")
                        :GoTo("STATE_KEEP_PET")
                    :OnFailure()
                        :Dialog("DIALOG_CONVINCE_GIVE_BEAST_FAILURE")

            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :Travel()
        end)


    end)

    :State("STATE_KEEP_PET")
        :Loc{

            OPT_TAKE_LUMICYTE = "Take the lumicyte",
            
            DIALOG_TAKE_LUMICYTE = [[
                player:
                    Alright, come along, then.
                * You have a new friend!
            ]],
            OPT_DRIVE_OFF_LUMICYTE = "Drive off the lumicyte",
            DIALOG_DRIVE_OFF_LUMICYTE = [[
                player:
                    !threaten
                    Get out of here!
                lumicyte:
                    !exit
                * The lumicyte runs away, tail between its legs.
            ]],
            REQ_CANT_HAVE_A_PET = "You can't have another pet right now",

        }
        :Fn(function(cxt) 
            cxt:Opt("OPT_TAKE_LUMICYTE")
                :Dialog("DIALOG_TAKE_LUMICYTE")
                :ReqCondition(TheGame:GetGameState():GetCaravan():CanOwnPet(), "REQ_CANT_HAVE_A_PET")
                :RecruitMember(PARTY_MEMBER_TYPE.CREW, cxt:GetCastAgent("lumicyte"))
                :Travel()

            cxt:Opt("OPT_DRIVE_OFF_LUMICYTE")
                :Dialog("DIALOG_DRIVE_OFF_LUMICYTE")
                :Travel()

        end)


