local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    rank = { 2, 5 },
}

QDEF:AddParameter{
    cast_id = "ambush",
    cast_fn = function( quest, t )
        table.insert( t, true )
        table.insert( t, false )
    end,
}

QDEF:AddCast{
    cast_id = "traveller",
    condition = function( agent, quest )
        return true
    end,
}


QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit")  end)
        :Loc{
            DIALOG_INTRO = [[
                * You encounter {traveller} near paralyzed with fear by the side of the bog.
                Traveller:
                    !scared
                    There's something back there, lying in the bog!
                    Something... big!
                Player:
                    !dubious
                    Big? And hiding?
                    Maybe the swamp gas has gotten to you...
                Traveller:
                    No, really!
                    I saw it after stumbling across this treasure buried in the mud...
                Player:
                    Right, treasure...
            ]],

            DIALOG_LIKED = [[
                Traveller:
                    C'mon, {player}. Help out a friend will ya? We can split what I found!
            ]],

            OPT_INVESTIGATE = "Investigate alone",
            DIALOG_INVESTIGATE = [[
                Player:
                    Fine. Stay here or be gone, I'll go see this <i>treasure</i> for myself.
                Traveller:
                    Yeah, it's all yours!
                    !exit
                Player:
                    !sigh
                * After some time searching the area, you do indeed find what seem to be some ancient Rentorian relics.
            ]],

            OPT_INVESTIGATE_TOGETHER = "Convince {traveller} to come with you",
            DIALOG_INVESTIGATE_TOGETHER = [[
                Player:
                    Look, I'll come see what you're on about but you're coming with me.
            ]],

            DIALOG_WIN_NEGOTIAITON = [[
                Traveller:
                    I really hope I don't regret this...
                    !sigh
                * {traveller} leads you, ever so slowly, to the area {traveller.heshe} indicated.
                * You discover some long-forgotten Rentorian relics half-buried in the underbrush!
            ]],

            DIALOG_LOSE_NEGOTIAITON = [[
                Traveller:
                    There's <i>no</i> way I'm going back there!
                    Haven't you heard about the grawkits?
                    Well I saw 'em, and I'm going to live to tell the tale.
                    !exit
                Player:
                    !shrug
                * You spend some time investigating the area, and do indeed find what seem to be some ancient Rentorian relics!
            ]],

            OPT_LEAVE = "Leave, and make a wide berth",
            DIALOG_LEAVE = [[
                Player:
                    My advice is to just walk the other direction then.
                    !point
                    Like I'm about to do. Farewell.
                    !exit
                * You make a wide berth around the suspicious section of bog. No point in tempting fate.
                Traveller:
                    !crossed
                    A Hesh's pox on you!
            ]],
            SIT_MOD = "{agent} is really scared!",
        }
        :Fn(function(cxt)

            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("traveller"))
            cxt.quest:Complete()

            cxt.encounter:PresentDefault()
            cxt:Dialog( "DIALOG_INTRO" )

            if cxt:GetAgent():GetRelationship() >= RELATIONSHIP.LIKED then
                cxt:Dialog( "DIALOG_LIKED" )
            end

            cxt:Opt( "OPT_INVESTIGATE" )
                :Dialog( "DIALOG_INVESTIGATE" )
                :Fn( function( cxt )
                    cxt:GetAgent():MoveToLimbo()
                end )
                :GoTo( "STATE_SEARCH" )

            cxt:Opt( "OPT_INVESTIGATE_TOGETHER" )
                :Dialog( "DIALOG_INVESTIGATE_TOGETHER" )
                :Negotiation{
                    situation_modifiers =
                    {
                        { value = 10, text = cxt:GetLocString("SIT_MOD") }
                    },

                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_WIN_NEGOTIAITON")
                        local allies = { cxt:GetAgent() }
                        cxt:GoTo("STATE_SEARCH", allies )
                    end,
                    
                    on_fail = function(cxt)
                        cxt:GetAgent():MoveToLimbo()
                        cxt:Dialog("DIALOG_LOSE_NEGOTIAITON")
                        cxt:GoTo("STATE_SEARCH")
                    end 
                }

            cxt:Opt( "OPT_LEAVE" )
                :Dialog( "DIALOG_LEAVE" )
                :ReceiveOpinion( OPINION.DID_NOT_HELP )
                :Travel()

        end)

    :State( "STATE_SEARCH" )
        :Loc{
            DIALOG_FIGHT = [[
                * Suddenly, a brief rustling in the ferns alerts you to the emerging shadow of an enormous lizard!
                player:
                    Aw, crap.
            ]],

            DIALOG_FIGHT_ALLIES = [[
                Traveller:
                    A-ha! What'd I tell you?
                    !cagey
                    Now where's that giant thing I saw..?
                * Suddenly, a brief rustling in the ferns alerts you to the emerging shadow of an enormous lizard!
                Traveler:
                    !sigh
                    There it is. I knew I'd regret this.
            ]],

            DIALOG_WIN_FIGHT = [[
                Player:
                    !happy
                * With the grawkit dispatched, you happily leave the area with your new findings.
            ]],

            DIALOG_WIN_FIGHT_ALLIES = [[
                Traveler:
                    !happy
                    Well that ended not horribly after all.
                    Be seeing you, {player}.
                    !exit
                * You help yourself to the rest of {traveller}'s lucrative discovery.
            ]],

            DIALOG_SHARE = [[
                Traveller:
                    Ah-ha, you see! Treasure!
                Player:
                    But no hiding monstrosity.
                Traveller:
                    !dubious
                    Are you really complaining?
                    !thumbs
                    Now let's split this loot and get the Hesh out of here!
            ]],

            DIALOG_ALL_YOURS = [[
                * You suppose it's a good thing {traveller} chickened out. Now these relics are all yours.
            ]]

        }
        :Fn(function(cxt, allies)

            if cxt.quest:GetCastMember( "ambush" ) then
                if allies then
                    cxt:Dialog( "DIALOG_FIGHT_ALLIES" )
                else
                    cxt:Dialog( "DIALOG_FIGHT" )
                end

                local monster = Agent( "GRAWKIT" )

                cxt:DoBattle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = allies,
                    enemies = { monster },
                    on_win = function( cxt )
                        if allies and not allies[1]:IsRetired() then
                            cxt:Dialog( "DIALOG_WIN_FIGHT_ALLIES" )
                            allies[1]:OpinionEvent( OPINION.FOUGHT_WITH )
                            allies[1]:MoveToLimbo()
                        else
                            cxt:Dialog( "DIALOG_WIN_FIGHT" )
                        end
                    end,
                }
            else
                if allies then
                    cxt:Dialog( "DIALOG_SHARE" )
                else
                    cxt:Dialog( "DIALOG_ALL_YOURS" )
                end
            end


            local loot
            if allies and not allies[1]:IsRetired() then
                loot =
                {
                    money = 50
                }

            else
                local items = BattleCardCollection.AllItems():Pick(2)
                for i = 1, #items do
                    items[i] = items[i].id
                end
                loot =
                {
                    money = 50,
                    items = items,
                }
            end
            cxt.encounter:AwardLoot( loot, { found = true })

        end )
