local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
}

:AddCast{
    cast_id = "admiralty",
    cast_fn = function(quest, t)
        local def = "ADMIRALTY_PATROL_LEADER"
        local agent = quest:CreateSkinnedAgent( def )
        table.insert( t, agent )
    end,
}


:AddOpinionEvents{
    paid_back_debt =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Paid back an old debt",
    },
    refused_to_pay_back_debt =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Refused to pay back an old debt",
    },

    gave_cold_shoulder =  
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Pretended not to know them",
    },

    refused_a_drink =  
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Refused to drink with them",
    },
    
    drank_with_them =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Drank with them, and talked about old times",
    },
    
    defended_them =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Defended them from the consequences of their actions",
    },
    
    abandoned_them =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Left them to take a beating",
    },

    denied_justice = 
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Denied them justice",

    }
}

QDEF:AddConvo()
    :ConfrontState("CONFRONT", function() return true end)
        :Loc{
            
            DIALOG_INTRO = [[
                * A lone traveler approaches.
                admiralty:
                    !right
                    {player}? Is that you?
                player:
                    !left
                admiralty:
                    It's me, {admiralty}!
                    How long has it been, man? 20 years?

                * You remember now. {admiralty} used to be a captain when you were in the Aerostat. 
                * {admiralty} was always a bit of a goon. The kind of officer the Admiralty used when smarter or weaker candidates wouldn't do.
                * Given that {admiralty.heshe}'s been demoted to a patrol leader now, {admiralty.hisher} temper must have caught up with {admiralty.himher}.
                
                *** You've been recognized by an old colleague in the Admiralty.
            ]],

            OPT_PRETEND_NOT_TO_KNOW_THEM = "Pretend you don't remember {admiralty.himher}",
            DIALOG_PRETEND_NOT_TO_KNOW_THEM = [[
                player:
                    Sorry friend, the old memory's not what it used to be...
                admiralty:
                    Oh, I see how it is.
                    Too good for your old friends, now, eh?
                    Watch your back, {player}.
                    !exit
                * {admiralty} leaves in a huff. You hope you don't run into {admiralty.himher} again.
            ]],
            OPT_GREET = "Greet {admiralty}",
            DIALOG_GREET = [[
                player:
                    {admiralty}! I barely recognized you!
                    What brings you to these parts?
                admiralty:
                    Oh, you know, the same old Admiralty Oshdung. Just working security for a visiting stuffed shirt.
                    But today's my day off, so I'm taking in the sights.
            ]],

        }
        :Fn(function(cxt)
            cxt.quest:GetCastMember("admiralty"):MoveToLocation(cxt.location)
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("admiralty"))
            cxt.quest:Complete()
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_GREET")
                :Dialog("DIALOG_GREET")
                :Fn(function(cxt) 
                    cxt:GoTo(table.arraypick{"STATE_WANTS_MONEY", "STATE_WANTS_TO_SHARE_A_DRINK", "STATE_COVER_FOR_ME"})
                end)

            cxt:Opt("OPT_PRETEND_NOT_TO_KNOW_THEM")
                :Dialog("DIALOG_PRETEND_NOT_TO_KNOW_THEM")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("gave_cold_shoulder"))
                :Travel()
        end)
    
    :State("STATE_WANTS_TO_SHARE_A_DRINK")
        :Loc{
            DIALOG_DRINK = [[
                admiralty:
                    Man, we used to go crazy back in the day.
                    Good times.
                player:
                    Yes, indeed.
                admiralty:
                    Say, I've got a little plonk with me. How about a toast?
                *** {admiralty} wants to drink with you.
            ]],
            OPT_AGREE = "Agree to the drink",
            OPT_DISAGREE = "Decline the offer",
            DIALOG_DISAGREE = [[
                player:
                    I'd love to, but I'm working right now.
                admiralty:
                    Oh, I see how it is.
                    Too good for your old friends, now, eh?
                    Watch your back, {player}.
                    !exit
                * {admiralty} leaves in a huff. You hope you don't run into {admiralty.himher} again.
            ]],
            DIALOG_AGREE = [[
                player:
                    I suppose one drink can't hurt.
                admiralty:
                    That's the spirit!
                admiralty:
                    !give
                * {admiralty} hands you a flask.
                admiralty:
                    So what are we drinking to?
            ]],
            
            OPT_OLD_TIMES = "To old times!",
            DIALOG_OLD_TIMES = [[
                player:
                    To old times! 
                    May we collect just a couple more before we're done.
                    !cheers
                admiralty:
                    !cheers
                    Stinkin' Hesh, man. How did we get so old?
            ]],

            OPT_ADMIRALTY = "To the Admiralty!",
            DIALOG_ADMIRALTY = [[
                player:
                    To the Admiralty! 
                    It's corrupt, and it's a mess, but then again, so are we.
                    !cheers
                admiralty:
                    !cheers
                    At least it's dependably bad.
            ]],

            
            OPT_HAVARIA = "To Havaria!",
            DIALOG_HAVARIA = [[
                player:
                    To Havaria!
                    We'd leave if we could, but we can't, so we don't.
                    !cheers
                admiralty:
                    !cheers
                player:
                    Oh we'll leave some day, but I'm pretty sure Hesh's bowel is worse.
            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_DRINK")
            cxt:Opt("OPT_AGREE")
                :Dialog("DIALOG_AGREE")
                :Fn(function(cxt) 
                    cxt:Opt("OPT_OLD_TIMES")
                        :Dialog("DIALOG_OLD_TIMES")
                        :DoDrink( 0, DRINK_RESTORE_RESOLVE_AMOUNT )
                        :GoTo("STATE_DRINK_2")
                    
                    cxt:Opt("OPT_ADMIRALTY")
                        :Dialog("DIALOG_ADMIRALTY")
                        :DoDrink( 0, DRINK_RESTORE_RESOLVE_AMOUNT )
                        :GoTo("STATE_DRINK_2")

                    cxt:Opt("OPT_HAVARIA")
                        :Dialog("DIALOG_HAVARIA")
                        :DoDrink( 0, DRINK_RESTORE_RESOLVE_AMOUNT )
                        :GoTo("STATE_DRINK_2")

                end)
            
            cxt:Opt("OPT_DISAGREE")    
                :Dialog("DIALOG_DISAGREE")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("refused_a_drink"))
                :Travel()


        end)
    :State("STATE_DRINK_2")
        :Loc{
            DIALOG_ANOTHER_DRINK = [[
                * {admiralty} stares pensively into the distance.
                admiralty:
                    One more, whatdya say?
            ]],
            OPT_ONE_MORE_AGREE = "Agree to another drink",
            DIALOG_ONE_MORE_AGREE = [[
                player:
                    Just one more, and then I'm done.
                admiralty:
                    That's the spirit!
                    To us!
                    We're both bastards, but at least we got friends.
                    !cheers
                player:
                    !cheers
            ]],
            DIALOG_DONE_DRINKING = [[
                admiralty:
                    See you around, friend.
                    !exit
                * {admiralty} stumbles off.
            ]],
            
            OPT_ONE_MORE_DISAGREE = "Decline another drink",
            DIALOG_ONE_MORE_DISAGREE = [[
                player:
                    Sorry, friend, but I must decline. I have work to attend to.
                admiralty:
                    Oh <i>now</> you're too good for me?
                    You know, you always were a big jerk. 
                    I never really liked you anyway.
                    !exit
                * {admiralty} stumbles off.
            ]]
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_ANOTHER_DRINK")

            cxt:Opt("OPT_ONE_MORE_AGREE")
                :Dialog("DIALOG_ONE_MORE_AGREE")
                :DoDrink( 0, DRINK_RESTORE_RESOLVE_AMOUNT )
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("drank_with_them"))
                :Dialog("DIALOG_DONE_DRINKING")
                :Travel()
            
            cxt:Opt("OPT_ONE_MORE_DISAGREE")
                :Dialog("DIALOG_ONE_MORE_DISAGREE")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("refused_a_drink"))
                :Travel()
        end)



    :State("STATE_COVER_FOR_ME")
        :Loc{
            DIALOG_INTRO = [[
                * {admiralty} nervously looks up the road.
                admiralty:
                    So, old friend, I, uh...
                    Just back me up here, alright?
                * Before you can ask any questions, you are approached by a group of angry workers.
                worker:
                    !right
                    There {admiralty.heshe} is. We found {admiralty.himher}.
                admiralty:
                    !left
                    I told you before, it's just a little misunderstanding.
                worker:
                    The only misunderstanding here is you misunderstanding how much trouble you're in.
                player:
                    !left
                worker:
                    Take a walk, grifter. We have business with the <i>Switch</>.

                *** {admiralty} is being chased by an angry mob.
            ]],
            OPT_ASK_WHAT_THEY_DID = "Ask what {admiralty} did",
            DIALOG_ASK_WHAT_THEY_DID = [[
                player:
                    Slow down for a second, here.
                    What do you want with {admiralty.himher}, anyway?
                worker:
                    If you <b>must</> know, {admiralty.heshe}'s the reason that we're stuck in this Heshforsaken bog to begin with.
                    We were happily employed back in Murder Bay, but <b>this</> oshnu's arse got us transfered here when we wouldn't pay {admiralty.himher} for protection.
                admiralty:
                    !left
                    I was offering a legitimate service!
                worker:
                    And now it's payback time.
                player:
                    !left
            ]],
            
            OPT_DEFEND_ADMIRALTY = "Defend {admiralty}",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    I don't care what {admiralty.heshe} did back in Murder Bay. You want {admiralty.himher}, you're going to have to go through me, too.
                worker:
                    !fight
                    Works for me!
            ]],
            
            DIALOG_POST_FIGHT_FRIEND_ALIVE = [[
                admiralty:
                    !right
                    Just like the good old days, eh, {player}?
                player:
                    !happy
                    I remember it hurting less afterwards back then.
                admiralty:
                    Thanks for the help, friend. If you need anything, you just ask, alright!
                    !exit
                * {admiralty} leaves in a hurry before you can actually ask {admiralty.himher} for anything.
            ]],
            
            DIALOG_POST_FIGHT_FRIEND_DEAD = [[
                * The fight is over, and {admiralty} is dead. You rifle through {agent.hisher} pockets, but find only pocket lint and regret.
            ]],


            OPT_TALK_DOWN_WORKER = "Talk {worker} down",
            DIALOG_TALK_DOWN_WORKER = [[
                player:
                    Come now, surely we can talk this through.
            ]],
            DIALOG_TALK_DOWN_WORKER_SUCCESS = [[
                worker:  
                    You're lucky your friend was here, {admiralty}.
                    Until next time.
                * The mob leaves.
                admiralty:
                    !right
                    Wow, thanks {player}. If you need anything, you just ask, alright!
                    !exit
                * {admiralty} leaves in a hurry before you can actually ask {admiralty.himher} for anything.
            ]],
            DIALOG_TALK_DOWN_WORKER_FAIL = [[
                worker:
                    Shut your mouth, grifter.
                    If you don't want to share in {admiralty.hisher} fate, leave now.
            ]],

            OPT_WALK_AWAY = "Walk away",
            DIALOG_WALK_AWAY = [[
                player:
                    I'm sorry, {admiralty}, but your problems are your own.
                admiralty:
                    !left
                    You Hesh-stained piece of oshnu dung! I should -
                admiralty:
                    !exit
                worker:
                    !exit
                * {admiralty}'s wave of invective is cut off by the sound of blows falling. You carry on down the road, so that you don't have to hear it.
            ]],


        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.quest.param.workers = CreateCombatParty(function() return {"LABORER", "LABORER", "HEAVY_LABORER" } end, cxt.quest:GetRank(), cxt.location,nil,nil,1)
                cxt.enc:SetPrimaryCast(cxt.quest.param.workers[1])
                cxt:ReassignCastMember("worker", cxt.quest.param.workers[1])
                cxt:Dialog("DIALOG_INTRO")
            end
            
            cxt:Opt("OPT_ASK_WHAT_THEY_DID")
                :Dialog("DIALOG_ASK_WHAT_THEY_DID")

            cxt:Opt("OPT_TALK_DOWN_WORKER")
                :Dialog("DIALOG_TALK_DOWN_WORKER")
                :Negotiation{
                    target_agent = cxt.quest.param.workers[1],
                    helpers =  { cxt.quest:GetCastMember("admiralty")},
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_TALK_DOWN_WORKER_SUCCESS")
                        cxt.quest:GetCastMember("admiralty"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("defended_them"))
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,

                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_TALK_DOWN_WORKER_FAIL")
                    end,
                }

            cxt:Opt("OPT_DEFEND_ADMIRALTY")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = {cxt.quest:GetCastMember("admiralty")},
                    on_win = function() 
                        if cxt:GetCastMember("admiralty"):IsDead() then
                            cxt:Dialog("DIALOG_POST_FIGHT_FRIEND_DEAD")
                        else
                            cxt:Dialog("DIALOG_POST_FIGHT_FRIEND_ALIVE")
                            cxt.quest:GetCastMember("admiralty"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("defended_them"))
                        end

                        for k,v in pairs(cxt.quest.param.workers) do
                            if not v:IsDead() then
                                v:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("denied_justice"))
                            end
                        end

                        StateGraphUtil.AddLeaveLocation(cxt)

                    end,
                }
            
            cxt:Opt("OPT_WALK_AWAY")
                :Dialog("DIALOG_WALK_AWAY")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("abandoned_them"), nil, cxt.quest:GetCastMember("admiralty"))
                :Travel()


        end)

    :State("STATE_WANTS_MONEY")
        :Loc{
            DIALOG_MONEY = [[
                agent:
                    Say, remember that last night in Docktown? 
                player:
                    I'm <b>sure</> that Docktown still remembers that night!
                agent:
                    Yeah, I got stuck holding the bag on the damages.
                player:
                    That was 20 years ago.
                agent:
                    Yeah, I know. They weren't cheap.
                    I'm a bit skint here. You mind making good on it?
                *** {agent} wants you to settle an old debt that you owe to {agent.himher}.
            ]],
            OPT_PAY_BACK = "Pay {agent} back",
            DIALOG_PAY_BACK = [[
                player:
                    Of course! I didn't know you got stuck with it.
                agent:
                    Thanks, friend. This will really help me out.
                    Be seeing you around!
                    !exit
                * {agent} leaves, in the direction of drink.
            ]],
            OPT_CANT_PAY = "Tell {agent} you don't have that much",
            DIALOG_CANT_PAY = [[
                player:
                    Sorry, {agent}, I would, but I don't have that kind of money.
                agent:
                    Eh. I get it. I do.
                    I'm stuck here in the bog for the next couple of days. If you get enough, look me up, alright.
                    !exit
                * {agent} wanders off, probably in search of someone to shake down.
            ]],
            OPT_WONT_PAY = "Tell {agent} you won't pay",
            DIALOG_WONT_PAY = [[
                player:
                    Now hold on. That was 20 years ago. You can't possibly be still nursing that grudge.
                    I don't owe you anything.
                agent:  
                    !angry
                    You're still a miserly spawn of Hesh, aren't you.
                    You know what? Keep your money. 
                    But watch your back!
                    !exit
                * {agent} storms off.
            ]],
        }
        :Fn(function(cxt) 
            local DEBT_AMT = 50
            cxt:Dialog("DIALOG_MONEY")

            cxt:Opt("OPT_PAY_BACK")
                :Dialog("DIALOG_PAY_BACK")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("paid_back_debt"))
                :DeliverMoney(DEBT_AMT, {no_scale = true})
                :Travel()

            
            if cxt.caravan:GetMoney() < DEBT_AMT then
                cxt:Opt("OPT_CANT_PAY")
                    :Dialog("DIALOG_CANT_PAY")
                    :Travel()
            end

            cxt:Opt("OPT_WONT_PAY")
                :Dialog("DIALOG_WONT_PAY")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("refused_to_pay_back_debt"))
                :Travel()



        end)
