local MANIFESTO_COST = 50

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest) 
        return TheGame:GetGameState():GetCaravan():GetMoney() >= MANIFESTO_COST
    end,
}

:AddOpinionEvents{
    
    refused_manifesto = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Refused to buy their manifesto",
    },
}


local convo = QDEF:AddConvo()
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit")  end)
        :Loc{
            
            DIALOG_INTRO = [[
                * You recognize the colors of a member of the Rise, boldly out and about on the main roads.
                
                player:
                    !left
                agent:
                    !right
                    !interest
                    You a grifter, stranger? Under contract, maybe? 
                    I bet you're wondering how you can get outta this mess. And the Rise has an answer for you.

            ]],


            OPT_EXPRESS_SUPPORT = "Express support for the Rise",
            DIALOG_EXPRESS_SUPPORT = [[
                player:
                    No need for the sales pitch, friend. I'm already a supporter of your cause.
                agent:
                    !happy
                    $miscRelieved
                    That's a relief. Can't count how many times I've asked that question only to get written up.
                    You want some literature? For a small fee, I mean.
                    If I pay off my demerits, I can keep talking up the shifts. 
            ]],

            OPT_EXPRESS_IGNORANCE = "Pretend you don't know about the Rise",
            DIALOG_EXPRESS_IGNORANCE = [[
                player:
                    !dubious
                    The Rise? Is that a... motivational group?
                    $neutralDubious
                    Or a baking technique, perhaps?
                agent:
                    !happy
                    Motivational, sure. We're a workers' rights movement. 
                    You heard of the insurrection in the Bread Fields? Where the laborers striked against the Admiralty occupation?
                player:
                    Yes, that does sound familiar.
                agent:
                    That was the Rise. So now in Grout Bog we're trying to do the same against the Spark Barons.
                    !eureka
                    Fair pay for good work: no demerits, no bottomless contracts.
                    Of course... the overhead is killer. We need money to keep operating, and our own members are lacking. 
                    You want some literature? For a small fee, I mean.
            ]],
            
            OPT_EXPRESS_DOUBT = "Express doubt about the Rise",
            DIALOG_EXPRESS_DOUBT = [[
                player:
                    !crossed
                    Spare me the lecture, laborer. I know your movement and I can't say I care for your methods.
                agent:
                    !dubious
                    Our methods? We're just turning the system back in on itself!
                    !permit
                    If you read this manifesto, I'm sure you'll understand what we mean. Why don't you take one for yourself?
                    All I ask is a small donation. It barely covers the printing cost!
            ]],

        }
        :Fn(function(cxt)
            
            if cxt:FirstLoop() then

                cxt.quest:Complete()
                cxt.enc.scratch.rise = TheGame:GetGameState():AddSkinnedAgent( "RISE_PAMPHLETEER" )
                cxt:ReassignCastMember("rise", cxt.enc.scratch.rise)
                cxt.enc:SetPrimaryCast(cxt.enc.scratch.rise)

                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_EXPRESS_SUPPORT")
                :Dialog("DIALOG_EXPRESS_SUPPORT")
                :GoTo("STATE_BUYIT")

            cxt:Opt("OPT_EXPRESS_IGNORANCE")
                :Dialog("DIALOG_EXPRESS_IGNORANCE")
                :GoTo("STATE_BUYIT")

            cxt:Opt("OPT_EXPRESS_DOUBT")
                :Dialog("DIALOG_EXPRESS_DOUBT")
                :GoTo("STATE_BUYIT")

        end)

    :State("STATE_BUYIT")
        :Loc{
            OPT_BUY_IT = "Buy {1#card}",
            DIALOG_BUY_IT = [[
                player:
                    !agree
                    Very well. Let's see this literature.
                agent:
                    Hopefully this answers any questions you might have.
                    !cagey
                    Be careful who might see you with it, though.
                    !exit
                * {agent} continues down the road, leaving you to your reading.
            ]],
            OPT_NO_BUY = "Politely decline",
            DIALOG_NO_BUY = [[
                player:
                    No, thank you.
                agent:
                    !hips
                    Come on. You wouldn't want folks thinking you're an enemy to freedom, would you?
            ]],

            OPT_NEGOTIATE = "Convince {agent} to leave you alone",
            DIALOG_NEGOTIATE = [[
                player:
                    I said no, laborer. You'd do best to honor that. 
            ]],
            DIALOG_NEGOTIATE_WON = [[
                player:
                    I don't understand why you'd want allies you don't even respect.
                    Or are you merely using the Rise as a ruse for your highway robbery?
                agent:
                    !surprised
                    H-highway...?! 
                    Okay, fine. I see how this looks. The foreman would kill me if I made the Rise look no better than the Spree.
                    !exit
                * {agent} continues down the road, head hanging heavily. 
            ]],
            DIALOG_NEGOTIATE_LOST = [[
                player:
                    Forcing me to buy that makes you no better than the Spark Barons.
                agent:
                    !angry
                    No better? You honestly comparing this to the atrocities the Barons commit?
                    !angry_accuse
                    We call that a false equivalence, friend, and I'm not falling for it. 
            ]],

            OPT_REFUSE = "Forcefully refuse",
            DIALOG_REFUSE = [[
                player:
                    !angry
                    Once more: the answer is no.
                agent:
                    !angry
                    I see you now. Watch your back. 
                    !throatcut
                    The Rise is everywhere.
            ]]

        }

        :SetLooping()
        :Fn(function(cxt)
            
            local card_id = "rise_manifesto"
            cxt:Opt("OPT_BUY_IT", card_id)
                :Dialog("DIALOG_BUY_IT")
                :DeliverMoney(MANIFESTO_COST, {is_shop = true} )
                :GainCards{card_id}
                :Travel()

            
            if not cxt.enc.scratch_refused_once then
                cxt:Opt("OPT_NO_BUY")
                    :Dialog("DIALOG_NO_BUY")
                    :Fn(function() cxt.enc.scratch_refused_once = true end)
            else
                cxt:Opt("OPT_NEGOTIATE")
                    :Dialog("DIALOG_NEGOTIATE")
                    :Negotiation{
                        on_success = function() 
                            cxt:Dialog("DIALOG_NEGOTIATE_WON")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function() 
                            cxt:Dialog("DIALOG_NEGOTIATE_LOST")
                        end,
                    }

                cxt:Opt("OPT_REFUSE")
                    :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("refused_manifesto"))              
                    :Dialog("DIALOG_REFUSE")                 
                    :Travel()
            end

        end)
    


