local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,
    --is_negative = true,

    on_init = function(quest)
    end,
}

local STATES = {"STATE_BARONS", "STATE_RISE", "STATE_MONSTERS", "STATE_BOGGERS"}

--------------------------------------------------------------------

QDEF:AddConvo()
    :ConfrontState("CONF", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                    player:
                        !left
                    * You come across a solitary Jake trying to load spilled packages back onto an unhappy oshnu.
                    agent:
                        !right
                        Grifter! Do you want to earn a couple of quick shills?
                    player:
                        !dubious
                        Maybe. Although the quicker the shill, the less likely you're able to spend it, in my experience.
                    agent:
                        This is easy work. We have to load this cargo back on the snail. I'll pay you {pay#money} when we're done.
                        You in?
                ]],
                OPT_ASK_ABOUT_CARGO = "Ask about the cargo",
                DIALOG_ASK_ABOUT_CARGO = [[
                    player:
                        What's <i>in</> the boxes?
                    agent:
                        !crossed
                        I'm paying you to lift, not to ask questions.
                ]],

                OPT_ASK_FOR_MORE_MONEY = "Ask for more money",
                DIALOG_ASK_FOR_MORE_MONEY = [[
                    player:
                        You clearly need the help.
                        But I don't sell my services so cheaply...
                ]],
                DIALOG_ASK_FOR_MORE_MONEY_SUCCESS = [[
                    agent:
                        Fine. I'll pay {pay#money}, but that's the final offer.
                ]],
                DIALOG_ASK_FOR_MORE_MONEY_FAILURE = [[
                    agent:
                        You know what, forget it.
                        I've already wasted too much time on you.
                        !exit
                    * {agent} returns to {agent.hisher} task.
                ]],

                OPT_REJECT_THE_DEAL = "Don't take the job",
                DIALOG_REJECT_THE_DEAL = [[
                    
                    {negotiated?
                        player:    
                            On second thought, no.
                        agent:
                            After all of that?!
                            Hrmph.
                            !exit
                        * {agent} returns to {agent.hisher} task.
                    }
                    
                    {not negotiated?
                        player:    
                            I have rather more lucrative business to attend to.
                        agent:
                            Suit yourself, grifter.
                            !exit
                        * {agent} returns to {agent.hisher} task.
                    }
                ]],

                OPT_ACCEPT_THE_DEAL = "Agree to help re-load the cargo",        
                DIALOG_ACCEPT_THE_DEAL = [[
                    player:
                        I'll exchange my sweat for your shills.
                    agent:
                        That's... 
                        ...unnecessarily gross. But good. Let's get to work.
                        !exit
                    player:
                        !exit

                    * You set about lifting the mysterious boxes onto the oshnu.
                    * {agent} seems nervous, jumping at every sound the Bog has to offer.

                ]],    

        }
        :Fn(function(cxt) 
                cxt.quest:Complete()
                cxt.quest.param.pay = cxt.quest:GetRank()*30
                cxt.quest.param.oshnu = AgentUtil.GetFreeAgent("MERCHANT_SNAIL")
                cxt:ReassignCastMember("oshnu", cxt.quest.param.oshnu)

                cxt.quest.param.lifter = TheGame:GetGameState():AddSkinnedAgent("JAKES_LIFTER")
                cxt.quest.param.lifter:MoveToLocation(cxt.location)
                cxt:TalkTo(cxt.quest.param.lifter)
                cxt:ReassignCastMember("lifter", cxt.quest.param.lifter)



                cxt:Dialog("DIALOG_INTRO")

                cxt:RunLoop(function() 
    
                    cxt:Question("OPT_ASK_ABOUT_CARGO", "DIALOG_ASK_ABOUT_CARGO")

                    cxt:Opt("OPT_ASK_FOR_MORE_MONEY")
                        :Dialog("DIALOG_ASK_FOR_MORE_MONEY")
                        :Negotiation{}
                            :OnSuccess()
                                :Fn(function() 
                                    cxt.quest.param.negotiated = true
                                    cxt.quest.param.pay = cxt.quest:GetRank()*50
                                end)
                                :Dialog("DIALOG_ASK_FOR_MORE_MONEY_SUCCESS")
                            
                            :OnFailure()
                                :Dialog("DIALOG_ASK_FOR_MORE_MONEY_FAILURE")
                                :ReceiveOpinion(OPINION.INSULT)
                                :Travel()

                    
                    local valid_states = {"STATE_BARONS", "STATE_RISE", "STATE_MONSTERS"}
                    if cxt.quest:GetRank() >= 3 then
                        table.insert(valid_states, "STATE_BOGGERS")
                    end



                    cxt:Opt("OPT_ACCEPT_THE_DEAL")
                        :Dialog("DIALOG_ACCEPT_THE_DEAL")
                        :GoTo(cxt.quest.param.testing_override or table.arraypick(valid_states))
                    

                    local reject = cxt:Opt("OPT_REJECT_THE_DEAL")
                        :Dialog("DIALOG_REJECT_THE_DEAL")
                    if cxt.quest.param.negotiated then
                        reject:ReceiveOpinion(OPINION.INSULT)
                    end
                    reject:Travel()


                end)

            end)

        :State("STATE_BARONS")
            :Loc{
                DIALOG_INTRO = [[
                    * {lifter} practically jumps out of {lifter.hisher} skin when a passing Baron patrol draws near.
                    lifter:
                        !left
                    agent:
                        !right
                        Well, well, well. 
                        Let's get those boxes open and see what kind of contraband I get to confiscate.
                    * You see {agent} reaching for {agent.hisher} weapon. A fight is about to break out...
                ]],

                OPT_SNEAK_AWAY = "Slip away",
                DIALOG_SNEAK_AWAY = [[
                    lifter:
                        !exit
                    agent:
                        !exit
                    * You leave just as the violence starts.
                ]],

                OPT_NEGOTIATE = "Convince {agent} to look the other way",
                DIALOG_NEGOTIATE = [[
                    player: 
                        !left
                        {liked?
                            Hey, {agent}. I can vouch for {lifter} here...
                        }

                        {not liked?
                            We don't really need to do this, do we?
                        }
                ]],
                DIALOG_NEGOTIATE_FAILURE = [[
                    agent:
                        {liked?
                            {player}, I know we're friends, but I have a job to do.
                            Now stand aside and let me do it!
                        }

                        {not liked?
                                I have a job to do here, grifter.
                                Get out of the way.
                        }
                ]],

                DIALOG_NEGOTIATE_SUCCESS = [[
                    agent:
                        {liked?
                            You're good people, {player}, so I'll do what you said.
                            Just don't let this come back on me, alright?
                            !exit
                        }

                        {not liked?
                            You make a good point, grifter.
                            I'll let it slide. This time.
                            !exit
                        }

                        * The Barons leave.
                ]],

                OPT_DEFEND_JAKES = "Defend {lifter}",
                DIALOG_DEFEND_JAKES = [[
                    player:
                        !left
                        !fight
                    * You jump into action!
                ]],
                DIALOG_WON_FIGHT_WITH_JAKES = [[
                    {dead?
                        * {agent} is dead. The Bog gurgles with appreciation.
                    }
                    {not dead?
                        agent:
                            !injured
                            You win this time, grifter.
                            !exit
                        * The Baron retreats through the Bog.
                    }
                ]]

            }
            :Fn(function(cxt) 

                local opfor = CreateCombatParty("SPARK_BARON_PATROL", cxt.quest:GetRank() + 1, cxt.location, true)
                cxt:TalkTo(opfor[1])
                cxt:Dialog("DIALOG_INTRO")


                
                cxt:RunLoop(function() 
                    cxt:Opt("OPT_SNEAK_AWAY")
                        :Dialog("DIALOG_SNEAK_AWAY")
                        :ReceiveOpinion(OPINION.ABANDONED, nil, cxt:GetCastMember("lifter"))
                        :Travel()

                    cxt:Opt("OPT_NEGOTIATE")
                        :Dialog("DIALOG_NEGOTIATE")
                        :Negotiation{}
                            :OnSuccess()
                                :Dialog("DIALOG_NEGOTIATE_SUCCESS")
                                :Fn(function() for k,v in pairs(opfor) do v:MoveToLimbo() end end)
                                :GoTo("STATE_HELPED_JAKES")
                            
                            :OnFailure()
                                :Dialog("DIALOG_NEGOTIATE_FAILURE")
                                :Travel()
                    
                    cxt:Opt("OPT_DEFEND_JAKES")
                        :Dialog("DIALOG_DEFEND_JAKES")
                        :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                no_oppo_limit = true,
                                allies = {cxt.quest.param.lifter, cxt.quest.param.oshnu}
                            }
                            :OnWin()
                                :Dialog("DIALOG_WON_FIGHT_WITH_JAKES")
                                :GoTo("STATE_HELPED_JAKES")
                end)
            end)


        :State("STATE_RISE")
            :Loc{
                DIALOG_INTRO = [[
                    * One moment, you are alone with {lifter}. The next, you are surrounded by Rise.
                    lifter:
                        !left
                    agent:
                        !right
                        I claim this cargo in the name of the People! Step aside, parasite.
                    * You see {agent} reaching for {agent.hisher} weapon. A fight is about to break out...
                ]],

                OPT_SNEAK_AWAY = "Slip away",
                DIALOG_SNEAK_AWAY = [[
                    lifter:
                        !exit
                    agent:
                        !exit
                    * You leave just as the violence starts.
                ]],

                OPT_NEGOTIATE = "Convince {agent} to look the other way",
                DIALOG_NEGOTIATE = [[
                    player: 
                        !left
                        {liked?
                            Hey, {agent}. {lifter} is no scab. Trust me...
                        }

                        {not liked?
                            Don't you have more pressing matters to deal with?
                        }
                ]],
                DIALOG_NEGOTIATE_FAILURE = [[
                    agent:
                        {liked?
                            !dubious
                            You're not standing in the way of the Revolution, are you, {player}?
                        }

                        {not liked?
                            Grifter, this doesn't concern you.
                        }
                ]],

                DIALOG_NEGOTIATE_SUCCESS = [[
                    agent:
                        {liked?
                            Alright. I'll look the other way. But only because you asked me to, {player}.
                            !exit
                        }

                        {not liked?
                            This better not end up in the Barons' hands.
                            !exit
                        }

                        * The Rise melts back into the trees.
                ]],

                OPT_DEFEND_JAKES = "Defend {lifter}",
                DIALOG_DEFEND_JAKES = [[
                    player:
                        !left
                        !fight
                    * You jump into action!
                ]],
                DIALOG_WON_FIGHT_WITH_JAKES = [[
                    {dead?
                        * {agent} is dead. The Bog gurgles with appreciation.
                    }
                    {not dead?
                        agent:
                            !injured
                            The Rise will catch up with you.
                            !exit
                        * The Rise limp away.
                    }
                ]]

            }
            :Fn(function(cxt) 

                local opfor = CreateCombatParty("RISE_PATROL", cxt.quest:GetRank() + 1, cxt.location, true)
                cxt:TalkTo(opfor[1])
                cxt:Dialog("DIALOG_INTRO")


                
                cxt:RunLoop(function() 
                    cxt:Opt("OPT_SNEAK_AWAY")
                        :Dialog("DIALOG_SNEAK_AWAY")
                        :ReceiveOpinion(OPINION.ABANDONED, nil, cxt:GetCastMember("lifter"))
                        :Travel()

                    cxt:Opt("OPT_NEGOTIATE")
                        :Dialog("DIALOG_NEGOTIATE")
                        :Negotiation{}
                            :OnSuccess()
                                :Dialog("DIALOG_NEGOTIATE_SUCCESS")
                                :Fn(function() for k,v in pairs(opfor) do v:MoveToLimbo() end end)
                                :GoTo("STATE_HELPED_JAKES")
                            
                            :OnFailure()
                                :Dialog("DIALOG_NEGOTIATE_FAILURE")
                                :Travel()
                    
                    cxt:Opt("OPT_DEFEND_JAKES")
                        :Dialog("DIALOG_DEFEND_JAKES")
                        :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                no_oppo_limit = true,
                                allies = {cxt.quest.param.lifter, cxt.quest.param.oshnu}
                            }
                            :OnWin()
                                :Dialog("DIALOG_WON_FIGHT_WITH_JAKES")
                                :GoTo("STATE_HELPED_JAKES")
                end)
            end)         

        :State("STATE_BOGGERS")
            :Loc{
                DIALOG_INTRO = [[
                    * You are surrounded on all sides by Boggers.
                    lifter:
                        !left
                    agent:
                        !right
                        !angry
                        You have stolen from us!
                    lifter:
                        !scared
                        Uhhhh...
                    player:
                        !left
                    agent:
                        And you have mishandled the holy Bog!
                        You will both die now, and learn the error of your ways!
                ]],

                OPT_DEFEND_JAKES = "Defend yourself",
                DIALOG_DEFEND_JAKES = [[
                    player:
                        !left
                        !fight
                    * You draw your weapon.
                ]],
                DIALOG_WON_FIGHT_WITH_JAKES = [[
                    {dead?
                        * {agent} sinks into the Bog that {agent.heshe} loves so much.
                    }
                    {not dead?
                        agent:
                            !injured
                            The Bog will never forgive this!
                            !exit
                    }
                ]]

            }
            :Fn(function(cxt) 

                local opfor = CreateCombatParty("BOGGER_PATROL", cxt.quest:GetRank(), cxt.location, true)
                cxt:TalkTo(opfor[1])
                cxt:Dialog("DIALOG_INTRO")
                
                cxt:RunLoop(function() 

                    cxt:Opt("OPT_DEFEND_JAKES")
                        :Dialog("DIALOG_DEFEND_JAKES")
                        :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                no_oppo_limit = true,
                                allies = {cxt.quest.param.lifter, cxt.quest.param.oshnu}
                            }
                            :OnWin()
                                :Dialog("DIALOG_WON_FIGHT_WITH_JAKES")
                                :GoTo("STATE_HELPED_JAKES")
                end)
            end)         

        :State("STATE_MONSTERS")
            :Loc{
                DIALOG_INTRO = [[
                    * {lifter} looks almost relieved when the noises reveal themselves as animal in origin.
                    agent:
                        !right
                    lifter:
                        !left
                        Let's deal with these beasts!
                ]],

                OPT_SNEAK_AWAY = "Slip away",
                DIALOG_SNEAK_AWAY = [[
                    lifter:
                        !exit
                    agent:
                        !exit
                    * You leave just as the violence starts.
                ]],

                OPT_DEFEND_JAKES = "Defend {lifter}",
                DIALOG_DEFEND_JAKES = [[
                    player:
                        !left
                        !fight
                    * You jump into action!
                ]],
                DIALOG_WON_FIGHT_WITH_JAKES = [[
                    * You have prevailed over the local fauna.
                ]]

            }
            :Fn(function(cxt) 

                local opfor = CreateCombatParty("BOG_MONSTERS", cxt.quest:GetRank() + 1, cxt.location, true)
                cxt:ReassignCastMember("lifter", cxt.quest.param.lifter)
                cxt:TalkTo(opfor[1])
                cxt:Dialog("DIALOG_INTRO")
                
                cxt:RunLoop(function() 
                    cxt:Opt("OPT_SNEAK_AWAY")
                        :Dialog("DIALOG_SNEAK_AWAY")
                        :ReceiveOpinion(OPINION.ABANDONED, nil, cxt:GetCastMember("lifter"))
                        :Travel()
                    
                    cxt:Opt("OPT_DEFEND_JAKES")
                        :Dialog("DIALOG_DEFEND_JAKES")
                        :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                allies = {cxt.quest.param.lifter, cxt.quest.param.oshnu},
                                no_oppo_limit = true
                            }
                            :OnWin()
                                :Dialog("DIALOG_WON_FIGHT_WITH_JAKES")
                                :GoTo("STATE_HELPED_JAKES")
                end)
            end)


        :State("STATE_HELPED_JAKES")
            :Loc{
                DIALOG_THANK = [[
                    player:
                        !happy
                    agent:
                        !right
                        !happy
                        Thanks for that, Grifter. 
                        You know what? You've earned your pay already. I'll finish the job.
                        And look me up if you ever need anything, alright?
                ]],
                OPT_QUESTION_OSHNU = "Ask about the oshnu",
                DIALOG_QUESTION_OSHNU = [[
                    player:
                        Your oshnu is dead. How are you going to move these goods?
                    agent:
                        That's my problem, not yours. I'll figure something out.
                    player:
                        !dubious
                        Okay... that's evasive.
                    agent:  
                        It sure is! Don't worry about it!
                ]],

                OPT_QUESTION_BOXES = "Ask about the boxes",


                DIALOG_QUESTION_BOXES = [[
                    player:
                        Can I know what's in the boxes now?
                    agent:
                        Absolutely not.
                ]],

                DIALOG_LIFTER_DEAD = [[
                    * {lifter} is dead, so you take a peek inside the boxes.
                    * They're full to the brim with dug-up Bog.
                    * A strange thing to die for, for sure.
                    player:
                        !exit
                ]],

                OPT_GET_PAID = "Get paid",
                DIALOG_GET_PAID = [[
                    agent:
                        !give
                    player:
                        !take
                        I'll take my money now.
                    agent:
                        !exit
                    * {agent} returns to {agent.hisher} task.

                ]]
            }
            :Fn(function(cxt) 
                if cxt.quest.param.lifter:IsDead() then
                    cxt:Dialog("DIALOG_LIFTER_DEAD")
                    StateGraphUtil.AddLeaveLocation(cxt)
                else
                    cxt:TalkTo(cxt.quest.param.lifter)
                    cxt:Dialog("DIALOG_THANK")
                    
                    cxt.quest.param.lifter:OpinionEvent(OPINION.DEFENDED)
                
                    cxt:RunLoop(function()
    
                        if cxt.quest.param.oshnu:IsDead() then
                            cxt:Question("OPT_QUESTION_OSHNU", "DIALOG_QUESTION_OSHNU")
                        end
                    cxt:Question("OPT_QUESTION_BOXES", "DIALOG_QUESTION_BOXES")    
                        cxt:Opt("OPT_GET_PAID")
                            :Dialog("DIALOG_GET_PAID")
                            :ReceiveMoney(cxt.quest.param.pay, {no_scale = true})
                            :Travel()
                    end)
                end

                

            end)


QDEF:AddDebugOption("testing_override", STATES)