local SAL_MONEY = 200

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    rank = {2, 5},
}

:AddCastByAlias{
    cast_id = "sal",
    alias = "NPC_SAL",
}

:AddLocationDefs{
    FIGHT_LOCATION = {
        name = "Bog Trailhead",
        plax = "EXT_Bog_Forest_01",
    },
}

:AddOpinionEvents{

    protected_from_sal = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Defended them from a bounty hunter.",
    },
    helped_sal = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Helped them with a difficult bounty.",
    },
}

QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit") end)
    :Loc{
        DIALOG_INTRO = [[
            * A fellow grifter approaches you.
            player:
                !left
            sal:
                !right
                !greeting
                Hey, buddy, you got a minute?
            player:
                !interest
                I suppose. Are you a laborer?
            sal:
                !shrug
                $miscEmbarrassed
                Uh. No. Been there, obliterated that.
                $neutralDirect
                I'm a hunter—proper licensed, not some washout. I'm tracking a target from Murder Bay.
                $miscResigned
                Normally I'd see the job through, but to be honest the less time I spend here in the bog, the better.
                !point
                You look like a capable sort. Whaddya say to some paying work?
                $miscPersuasive
                If we split the bounty, your share would be {1#money}.
        ]],
        OPT_AGREE = "Agree to the deal",
        DIALOG_AGREE = [[
            player:
                !agree 
                $miscAlluring
                Seems reasonable. I look forward to working with a professional.
            sal:
                !handwring
                $miscRelieved
                Thank the seaslime. I've about had it up to here with the bugs in this place.
                Follow me—I've got a lead and it should be quick to see it through. 
                !exit
        ]],

        OPT_DISAGREE = "Refuse",
        DIALOG_DISAGREE = [[
            player:
                I'm afraid I'm rather busy at the moment. But good luck, hunter.
            sal:
                !disappoint
                $neutralResigned
                Aw, crud.
                Your loss, pops.
                !exit
            * {sal} stalks off in pursuit of {sal.hisher} target.
        ]]
    }
    :Fn(function(cxt)
        cxt.quest:Complete()
        
        cxt.quest:GetCastMember("sal"):MoveToLocation(cxt.location)
        cxt:TalkTo(cxt.quest:GetCastMember("sal"))

        cxt:Dialog("DIALOG_INTRO", SAL_MONEY)
        
        cxt:RunLoop(function() 
            cxt:AskAbout("STATE_QUESTIONS")
            
            cxt:Opt("OPT_AGREE")
                :Dialog("DIALOG_AGREE")
                :GoTo("STATE_FIGHT_TARGET")

            cxt:Opt("OPT_DISAGREE")
                :Dialog("DIALOG_DISAGREE")
                :Travel()
        end)

    end)
    :AskAboutHub("STATE_QUESTIONS", 
    {
        "Ask about the target",
        [[
            player:
                Just who's the target?
            agent:
                Some Spark Baron enforcer. 
                !cruel
                $happyCocky
                My intel says they're rocking some experimental modifications, so things might get spicy.
        ]],

        "Ask about {agent}",
        [[
            player:
                !crossed
                I like to know a little about the people I work with.
                $neutralDubious
                How long have you been a hunter? You look quite young.
            agent:
                $happyJoke
                Bet everyone looks that way to <i>you</i>. 
                You heard of Kashio?
            player:
                $neutralThoughtful
                !agree
                Yes, I believe so. She was a rather notorious hunter, wasn't she? Something of a legend in these parts.
            agent:
                !cruel
                $miscMocking
                Hah! Sure. 
                Then she was a Spark Baron. Then a debt-broker. Easy to change it up when you're willing to trade it all.
                !happy
                $happyCocky
                Point is—she's dead now. And I'm richer than a seabed. 
            player:
                I see. 
        ]],
        "Ask about Grout Bog",
        [[
            player:
                !dubious 
                $neutralDubious
                I admit the bog is unpleasant, but surely a hunter of your apparent caliber isn't afraid of a little mud?
            agent:
                $angryAside
                Mud? No. Reliving childhood trauma? That, I'll take a pass on.
            player:
                Ah, you grew up here.
            agent:
                Too much, too fast, I'd say.
                !bashful
                $miscEmbarrassed 
                Yeesh. Is this how others feel when I go at them with prying questions?
            player:
                !shrug
                $neutralWhatever
                You answered. I merely asked.
        ]],
    })    

    :State("STATE_FIGHT_TARGET")
        :Loc{
            DIALOG_INTRO = [[
                * {sal} takes you down a hidden bog path. It's not long before you hear what you assume is your quarry.
                goon:
                    !right
                    !tired
                    $scaredFearful
                    Damned grifter hounding my every step. If Fellemo finds out about this—
                goon:
                    !scared
                sal:  
                    !left
                    !humouring
                    $miscMocking
                    Oh, don't mind me. I don't aim to interrupt your ruminations.
                    I can slap the cuffs on while you keep muttering to yourself.
                * The Baron looks around, panicked. Seeing you, {goon.heshe} clutches a coin purse.
                goon:
                    !cagey
                player:
                    !left
                goon:
                    $angryAside
                    You! You're the one Fellemo hired, right?
                    I'll pay you double what she offered to get her off my back.
            ]],

            OPT_ATTACK = "Attack {agent}",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    $miscMocking
                    So sorry, Baron, but my services have already been engaged.
            ]],

            OPT_DEFEND = "Defend {agent}",
            
            DIALOG_DEFEND = [[
                player:
                    !fight
                    $neutralDirect
                    Deal.
                sal:
                    !right
                    !angry
                    $angryTakenAback
                    Never shoulda trusted you with that rat tail on your face!
                    Bring it on, then, you bogsuckers! I'll drop you both!
            ]],

            DIALOG_FOUGHT_SAL_BOTH_DEAD = [[
                * Both the grifter and {sal.hisher} target are dead.
                * Looks like nobody's getting paid today.
            ]],
            DIALOG_FOUGHT_SAL_SAL_DEAD = [[
                goon:
                    !right
                * {goon} considers {sal}'s still-warm corpse.
                goon:
                    You're thorough, grifter. I like that.
            ]],
            DIALOG_FOUGHT_SAL_GOON_DEAD = [[
                sal:
                    !right
                    !injured
                    $angryAside
                    Nice of you to still your hand, considering you ain't got no scruples.
                player:
                    Rather cheeky tone, considering your state, isn't it?
                * But you're not the only one capable of hiding your colors. A bubble pops somewhere in the bog, and when you turn your gaze for a moment, Sal is gone.
                player:
                    !dubious
                sal:
                    !exit
                * What's worse—she took proof of {goon}'s death with her. 
                player:
                    I suppose I deserved that. 
                
               
            ]],
            DIALOG_FOUGHT_SAL_NO_DEAD = [[
                sal:
                    !right
                    !injured
                    I ain't dying here—not to a bloody Baron, no less!
                    !exit
                * Even injured, the hunter moves quickly, leaving only a splatter of her blood behind. 
                goon:
                    !right
                    !tired
                    Thanks for the help, grifter. 
                    At least you can rely on the greed of strangers, eh?
            ]],
            DIALOG_FOUGHT_GOON_BOTH_DEAD = [[
                * Both the grifter and {sal.hisher} target are dead.
                * You whistle past the carnage.
                player:
                    !exit
            ]],
            DIALOG_FOUGHT_GOON_SAL_DEAD = [[
                goon:
                    !right
                * {goon} considers {sal}'s still-warm corpse.
                goon:
                    You're thorough, grifter. I like that.
            ]],
            DIALOG_FOUGHT_GOON_GOON_DEAD = [[
                sal:
                    !right
                * {sal} kicks at {goon}'s corpse.
                sal:
                    !dubious
                    $neutralDubious
                    Coulda brought {goon.himher} in alive, but in this terrain? 
                    Dragging a prisoner back to Murder Bay would be a nightmare. 
                    !happy
                    Thanks for the help. 
            ]],
            DIALOG_FOUGHT_GOON_NO_DEAD = [[
                goon:
                    !injured
                    !right
                    $angryAside
                    Hold your fire, damnit! I yield!
                sal:
                    !right
                    !happy
                    Funny how they never come around <i>before</i> I knock the tar outta them, huh?
                    Thanks for the help, there, pops. You're pretty good with those pistols.
                player:
                    !interest
                    $miscNostalgic
                    They're the best friends I ever had.
                sal: 
                    !humouring
                    Huh. You sound like someone else I know.
            ]],

            OPT_TAKE_MONEY = "Take the money",
            DIALOG_TAKE_MONEY_FROM_SAL = [[
                sal:
                    !give
                    Here's your pay. A bargain for the time you've saved me.
                player:
                    !take
                    A pleasure. I hope you make it back to Murder Bay as quickly as you like.
                sal:
                    Quicker, if I can manage it. 
                    If your business ever brings you to Murder Bay, look me up at the Grog n' Dog.
                player:
                    !thought 
                    $neutralThoughtful
                    Rather disreputable establishment, isn't it?
                sal:
                    !happy
                    Yeah! It's the best.
                * With her bounty in hand, Sal struts off through the mud—an impressive feat in itself.
                    !exit
            ]],
            DIALOG_TAKE_MONEY_FROM_GOON = [[
                goon:
                    Take it. 
                    !give
                player:
                    !take
                    Gladly. 
                goon:
                    Your services are worth the shills. I'll keep that in mind, if I see you again.
                * Looking shaken, but breathing, {goon} heads back down the path towards civilization.
                    !exit
            
            ]],

            OPT_TEACH_SOMETHING = "Turn down the money",
            DIALOG_TEACH_SOMETHING = [[
                player:
                    Please, dear girl. I wouldn't dream of taking shills from a hunter of your caliber.
                    You would've been excellent in the Aerostat, you know. 
                sal:
                    !dubious
                    Flattery isn't a form of currency I deal in. And I don't like being in anyone's debt.
                    $happyCocky
                    How about I repay you with some pointers, huh? Then maybe one day you can be as tough as me.
               player:
                    $miscNostalgic
                    Ah, that is the dream, isn't it? 
            ]],
            DIALOG_BYE = [[Watch yourself out there.]]

        }
        :Fn(function(cxt) 
            local location = cxt.quest:SpawnTempLocation("FIGHT_LOCATION")
            location:SetMapPos( cxt.location:GetMapPos())
            cxt.encounter:DoLocationTransition( location )
            cxt.encounter:PresentAgent(nil, SIDE.LEFT)
            cxt.encounter:PresentAgent(nil, SIDE.RIGHT)
            local options = {"RISE_TURNCOAT_BOSS", "SPARK_BARON_BOSS", "DRONE_GOON"}
            local goon = cxt.quest:CreateSkinnedAgent( table.arraypick(options) )
            TheGame:GetGameState():AddAgent( goon )
            
            goon:MoveToLocation(cxt.location)
            cxt:ReassignCastMember("goon", goon)
            cxt:TalkTo("goon")

            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_DEFEND")
                :ReceiveOpinion(OPINION.BETRAYED, nil, cxt.quest:GetCastMember("sal"))
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    allies = {goon},
                    enemies = {cxt.quest:GetCastMember("sal")},
                    flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.SELF_DEFENCE,
                }
                    :OnWin()
                        :Fn(function() 
                            local sal_dead = cxt.quest:GetCastMember("sal"):IsDead()
                            local goon_dead = goon:IsDead()
                            if sal_dead and goon_dead then
                                cxt:Dialog("DIALOG_FOUGHT_SAL_BOTH_DEAD")
                            elseif sal_dead and not goon_dead then
                                cxt:Dialog("DIALOG_FOUGHT_SAL_SAL_DEAD")
                            elseif not sal_dead and goon_dead then
                                cxt:Dialog("DIALOG_FOUGHT_SAL_GOON_DEAD")
                            elseif not sal_dead and not goon_dead then
                                cxt:Dialog("DIALOG_FOUGHT_SAL_NO_DEAD")
                            end

                            if not goon_dead then
                                cxt:TalkTo(goon)
                                cxt:Opt("OPT_TAKE_MONEY")
                                    :Dialog("DIALOG_TAKE_MONEY_FROM_GOON")
                                    :ReceiveMoney(SAL_MONEY*2, {no_scale=true})
                                    :ReceiveOpinion("protected_from_sal")
                                    :Travel()
                            end

                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    allies = {cxt.quest:GetCastMember("sal")},
                    enemies = {goon},
                    flags = BATTLE_FLAGS.ISOLATED,
                }
                    :OnWin()
                        :Fn(function() 
                            local sal_dead = cxt.quest:GetCastMember("sal"):IsDead()
                            local goon_dead = goon:IsDead()

                            if sal_dead and goon_dead then
                                cxt:Dialog("DIALOG_FOUGHT_GOON_BOTH_DEAD")
                            elseif sal_dead and not goon_dead then
                                cxt:Dialog("DIALOG_FOUGHT_GOON_SAL_DEAD")
                            elseif not sal_dead and goon_dead then
                                cxt:Dialog("DIALOG_FOUGHT_GOON_GOON_DEAD")
                            elseif not sal_dead and not goon_dead then
                                cxt:Dialog("DIALOG_FOUGHT_GOON_NO_DEAD")
                            end
                            
                            if not goon_dead then
                                goon:Retire() 
                            end

                            if not sal_dead then
                                cxt:TalkTo(cxt.quest:GetCastMember("sal"))
                                cxt:Opt("OPT_TAKE_MONEY")
                                    :Dialog("DIALOG_TAKE_MONEY_FROM_SAL")
                                    :ReceiveMoney(SAL_MONEY, {no_scale=true})
                                    :ReceiveOpinion("helped_sal")
                                    :Fn(function() cxt.quest:GetCastMember("sal"):Retire() end)
                                    :Travel()

                                cxt:Opt("OPT_TEACH_SOMETHING")
                                    :Dialog("DIALOG_TEACH_SOMETHING")
                                    :ReceiveOpinion("helped_sal")
                                    :Fn(function()
                                        local function generator( fn )
                                            if math.random() < 0.5 then
                                                return BattleCardCollection.RewardableCardsFromSeries( fn, cxt.player, "SAL" )
                                            else
                                                return NegotiationCardCollection.RewardableCardsFromSeries( fn, cxt.player, "SAL" )
                                            end
                                        end
                                        local cards = RewardUtil.GetCards( 3, generator, 5, cxt.player )
                                        cxt.enc:OfferCards( cards )
                                        cxt:Dialog( "DIALOG_BYE" )
                                        cxt.quest:GetCastMember("sal"):Retire()
                                    end )
                                    :Travel()

                            else
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end)
        end)