local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS

local CARD_PRICE = 100
local DOG_PRICE = 150

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,
    precondition = function(quest) 
        local has_item = false
        for i,card in ipairs( TheGame:GetGameState():GetPlayerAgent().battler:GetCards() ) do
            if CheckBits(card.flags, CARD_FLAGS.ITEM) then
                has_item = true
                break
            end
        end
        return TheGame:GetGameState():GetCaravan():GetPet() or has_item
    end,
}
--------------------------------------------------------------------

:AddCast{
    cast_id = "collector",
    cast_fn = function(quest, t)
        local def = "LABORER"
        local agent = quest:CreateSkinnedAgent( def )
        table.insert( t, agent )
    end,
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_START"
    end)

        :State("STATE_START")
        :Loc{
            DIALOG_INTRO_NO_DOG = [[
                player:
                    !left
                agent:
                    !right
                * A stranger approaches you. {agent.HeShe} appears to be collecting scraps by the side of the road.
                agent:
                    Greetings!
                    Got anything you would be interested in selling! I'm a trader of all goods.
            ]],
            DIALOG_INTRO_DOG = [[
                player:
                    !left
                agent:
                    !right
                * A stranger approaches you. {agent.HeShe} appears to be collecting scraps by the side of the road.
                agent:
                    Greetings!
                    What a mighty fine specimen! How much do you want for it?
                player:
                    Uhh, excuse me?
                agent:
                    The dog. I'd like to buy it! I'd also settle for any other goods you have.
            ]],
            OPT_SELL_ITEM = "Sell {1#card}",
            DIALOG_SELL_ITEM = [[
                player:
                    I'd be willing to sell {1#card} for {2} shills.
                agent:
                    Wonderful! I'll take it.
                player:
                    !give
                agent:
                    !take
                    Thank you, stranger!
            ]],
            OPT_SELL_DOG = "Sell {1#agent}",
            DIALOG_SELL_DOG = [[
                player:
                    Alright fine, but {1} shills is as low as I can go.
                agent:
                    Ah, you drive a hard bargain but I'll take it.
                player:
                    !give
                agent:
                    !take
                    This will be a nice addition to my collection.
            ]],
            OPT_DECLINE = "Decline the offer",
            DIALOG_DECLINE = [[
                player:
                    Ehh, sorry I'm not really looking to sell anything today.
                agent:
                    Ah, very well then. Perhaps another time!
            ]],
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("collector"))
                local card_options = {}
                for i,card in ipairs( TheGame:GetGameState():GetPlayerAgent().battler:GetCards() ) do
                    if CheckBits(card.flags, CARD_FLAGS.ITEM) then
                        table.insert(card_options, card)
                    end
                end
                if #card_options > 0 then
                    cxt.enc.scratch.item = card_options[math.random(#card_options)]
                end
                cxt.enc.scratch.dog = TheGame:GetGameState():GetCaravan():GetPet()
                if cxt.enc.scratch.dog then
                    cxt:Dialog("DIALOG_INTRO_DOG")
                else
                    cxt:Dialog("DIALOG_INTRO_NO_DOG")
                end
            end
            if cxt.enc.scratch.item then
                cxt:Opt("OPT_SELL_ITEM", cxt.enc.scratch.item.id)
                    :Dialog("DIALOG_SELL_ITEM", cxt.enc.scratch.item.id, CARD_PRICE)
                    :ReceiveMoney(CARD_PRICE)
                    :Fn(function(cxt)
                        TheGame:GetGameState():GetPlayerAgent().battler:RemoveCard(cxt.enc.scratch.item)
                    end)
                    :Travel()
            end

            if cxt.enc.scratch.dog then
                cxt:Opt("OPT_SELL_DOG", cxt.enc.scratch.dog)
                    :Dialog("DIALOG_SELL_DOG", DOG_PRICE)
                    :ReceiveMoney(DOG_PRICE)
                    :Fn(function(cxt )
                        cxt.enc.scratch.dog:Dismiss()
                    end)
                    :Travel()
            end

            cxt:Opt("OPT_DECLINE")
                :Dialog("DIALOG_DECLINE")
                :Travel()
        end)

