local FEE_AMOUNT = 50

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,

    on_init = function(quest)
    end,
    rank = {2, 5},
    precondition = function(quest)
        return TheGame:GetGameState():GetCaravan():HasHireling() == false and 
            TheGame:GetGameState():GetCaravan():GetMoney() >= FEE_AMOUNT and
            TheGame:GetGameState():IsHiringAvailable()
    end
}

QDEF:AddCastByAlias{
    cast_id = "merchant",
    alias = "BRAVE_MERCHANT",
}

QDEF:AddOpinionEvents{
    hired_brave_merchant =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Hired them as a body guard",
    },
    did_not_pay_shel = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Did not pay them for 'protecting' you",
    }
}

QDEF:AddConvo()
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("FOO", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                * You hear an ungodly commotion up ahead.
                monster:
                    !right
                agent:
                    !left
                    !fight
                * A lone merchant is staring down a wild grawkit with a short knife.
                agent:
                    Stay back, old man!
                    This thing is dangerous!
                agent:
                    !exit
                monster:
                    !exit
                * The merchant dodges an attack with surprising agility, and counters with a wild swing.
                * {agent.HisHer} knife throws up sparks as it scrapes ineffectively against the beast's tough hide.
                player:
                    !left
            ]],
            OPT_ASK_IF_NEED_HELP = "Ask if {agent} needs help",
            DIALOG_ASK_IF_NEED_HELP = [[
                player:
                    Do you need assistance?
                agent:
                    !right
                    !fight
                    No! I've got this!
                    !exit
            ]],

            OPT_ATTACK_THE_BEAST = "Attack the beast",
            DIALOG_ATTACK_THE_BEAST = [[
                player:
                    !fight
                    I've got a clear shot!
            ]],

            DIALOG_DEFEND_GRAWKIT = [[
                player:
                    !fight
                * You draw your weapons.
            ]],

            OPT_WATCH_THE_FIGHT = "Wait to see how this ends",
            DIALOG_WATCH_FIGHT_SHEL_WINS = [[
                player:
                    !crossed
                * You keep your pistols holstered, and watch the fight.
                * The merchant is clearly no match for the monster, but somehow {merchant.heshe} manages to avoid its claws and jaws each time.
                * The grawkit, frustrated, grows tired with the battle and tries to withdraw. The merchant throws {agent.himher}self at the beast with a renewed frenzy.
                * A single lucky strike to the beast's soft abdomen is all it takes in the end. 
                * The grawkit's face fills with inarticulate disbelief as the life slowly drains out of it.
                agent:
                    !right
                    There. It's done.
                    !hips
                    That'll be {1#money}.
                player:
                    !dubious
                    Excuse me?
                agent:
                    For protecting you. I'm a body guard, and I guarded your frail, old body.
                    Now pay up.
            ]],
            
            OPT_LEAVE = "Leave the merchant to {merchant.hisher} fate",
            DIALOG_LEAVE = [[
                player:
                    !exit
                * You decide against getting involved. Neither the grawkit nor {merchant} notice you leave.
            ]],

            DIALOG_WATCH_FIGHT_SHEL_LOSES = [[
                player:
                    !crossed
                * You keep your pistols holstered, and watch the fight.
                * The merchant is clearly no match for the monster, and while {merchant.hisher} sheer tenacity keeps {merchant.himher} in the fight longer than should be possible, {merchant.heshe} eventually falls beneath the beast's trampling claws.
                monster:
                    !right
                * Its bloodlust unsatisfied, the beast turns its attention to you!
            ]],

            DIALOG_DEFEATED_GRAWKIT_AFTER_SHEL = [[
                * The beast is dead. Surprisingly, {merchant} is not.
                merchant:
                    !right
                    !injured
                    You finished it off.
                    I'll only charge you half-price.
                player:
                    You're charging me?
                merchant:
                    !crossed
                    For protection. There's no way you would have survived without my help.
            ]],

            DIALOG_DEFEATED_GRAWKIT_WITH_SHEL = [[
                * The grawkit dies, curling up into a pathetically defensive ball.
                {dead?
                    * {merchant} didn't survive the battle. You leave.
                }
                {not dead?
                    merchant:
                        !right
                        !angry
                        You fool! I had that under control! I'm going to charge you double for interfering with a professional!
                    player:
                        Charge?
                    merchant:
                        For protecting you. From the grawkit.
                        Are you even paying attention?
                }
            ]],
            


        }
        :SetLooping(true)
        :Fn(function(cxt)
            cxt.quest.param.fee = FEE_AMOUNT
            cxt.quest.param.monster = TheGame:GetGameState():AddAgent(Agent( "GRAWKIT" ))
            cxt:ReassignCastMember("monster", cxt.quest.param.monster)
            cxt.quest.param.monster:GetAspect("health"):SetPercent(.6)

            cxt:TalkTo("merchant")
            cxt.quest:Complete()
            cxt:Dialog("DIALOG_INTRO")  

            cxt:RunLoop(function() 
                cxt:Question("OPT_ASK_IF_NEED_HELP", "DIALOG_ASK_IF_NEED_HELP")

                cxt:Opt("OPT_WATCH_THE_FIGHT")
                    :Fn(function() 
                        if cxt.quest.param.shel_wins_fight then
                            cxt:Dialog("DIALOG_WATCH_FIGHT_SHEL_WINS", cxt.quest.param.fee)
                            cxt:GoTo("STATE_DEAL_WITH_SHEL_AFTER_FIGHT")
                        else
                            cxt:TalkTo(cxt.quest.param.monster)
                            cxt:Dialog("DIALOG_WATCH_FIGHT_SHEL_LOSES")
                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND_GRAWKIT")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                }
                                    :OnWin()
                                        :Fn(function()
                                            cxt:TalkTo("merchant")
                                            cxt.quest.param.fee = math.floor(FEE_AMOUNT / 2 )
                                            cxt:Dialog("DIALOG_DEFEATED_GRAWKIT_AFTER_SHEL")
                                            cxt:GoTo("STATE_DEAL_WITH_SHEL_AFTER_FIGHT")
                                        end)
                            end
                        end)
                cxt:Opt("OPT_ATTACK_THE_BEAST")
                    :Dialog("DIALOG_ATTACK_THE_BEAST")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,                        
                        enemies = {cxt.quest.param.monster},
                        allies = {cxt.quest:GetCastMember("merchant")}
                    }
                        :OnWin()
                            :Fn(function() 
                                cxt.quest.param.fee = FEE_AMOUNT * 2
                                cxt:Dialog("DIALOG_DEFEATED_GRAWKIT_WITH_SHEL")
                                cxt:GoTo("STATE_DEAL_WITH_SHEL_AFTER_FIGHT")
                            end)
            
                cxt:Opt("OPT_LEAVE")
                    :Dialog("DIALOG_LEAVE")
                    :Travel()
            end)

        end)
    

    :State("STATE_DEAL_WITH_SHEL_AFTER_FIGHT")
        :Loc{
            OPT_PAY_FEE = "Pay the 'protection fee'",
            DIALOG_PAY_FEE = [[
                player:
                    !shrug
                    Here, I guess.
                    !give
                agent:
                    !take
                    Fair pay for services rendered.
                    You really shouldn't wander around alone out here. I won't always be there to save you!
                    !exit
                * {agent} whistles as {agent.heshe} bounds off, tossing {agent.hisher} knife in one hand.
            ]],

            OPT_REFUSE_PAY = "Refuse to pay",
            DIALOG_REFUSE_PAY = [[
                player:
                    Now listen here, I didn't-
                agent:
                    !fight
                    Shut your mouth before I open your throat!
                    Are you paying me or not?
            ]],

            OPT_HIRE = "Hire {agent} as a bodyguard",
            DIALOG_HIRE = [[
                player:
                    I could use your help, actually. 
                agent:
                    !happy
                    Really?
                    I - alright. But my help doesn't come cheaply.
                player:
                    Of course not. Here.
                    !give
                agent:
                    !take
                    Let's go!
            ]],


            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND_FROM_SHEL = [[
                player:
                    !fight
                    You got lucky once, merchant...
            ]],

            DIALOG_DEFEATED_SHEL = [[
                {dead?
                    * The merchant's body lay tangled with that of the grawkit. You wonder what the next person to come across this scene will think happened here.
                }
                {not dead?
                    agent:
                        !injured
                        Watch your back, thief.
                        !exit
                    player:
                        ...
                }
            ]],

        }
        :Fn(function(cxt) 
            local merch = cxt.quest:GetCastMember("merchant")
            if not merch or merch:IsRetired() then
                StateGraphUtil.AddLeaveLocation( cxt )
                return
            end

            cxt:RunLoop(function() 
                cxt:Opt("OPT_PAY_FEE")
                    :Dialog("DIALOG_PAY_FEE")
                    :DeliverMoney(cxt.quest.param.fee)
                    :Travel()

                cxt:Opt("OPT_HIRE")
                    :Dialog("DIALOG_HIRE")
                    :DeliverMoney(cxt.quest.param.fee*2)
                    :ReceiveOpinion("hired_brave_merchant")
                    :Fn(function(cxt) 
                        cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.HIRED )
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)

                cxt:Opt("OPT_REFUSE_PAY")
                    :Dialog("DIALOG_REFUSE_PAY")
                    :Fn(function() 
                        cxt:Opt("OPT_PAY_FEE")
                            :Dialog("DIALOG_PAY_FEE")
                            :DeliverMoney(FEE_AMOUNT)
                            :Travel()

                        cxt:Opt("OPT_DEFEND")
                            :ReceiveOpinion("did_not_pay_shel")
                            :Dialog("DIALOG_DEFEND_FROM_SHEL")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                            }
                                :OnWin()
                                    :Dialog("DIALOG_DEFEATED_SHEL")
                                    :Travel()
                    end)
            end)

        end)

QDEF:AddDebugOption("shel_wins_fight", {false, true})
