local battle_defs = require "battle/battle_defs"
local MAX_RESOLVE_GAIN = 4
local CARD_TO_GIVE = "shock_box"


local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_CONFRONT"
    end)

    :State("STATE_CONFRONT")
        :Loc{
            DIALOG_INTRO = [[
                * As you're walking down the road, you hear the sounds of fighting. You look around some trees and see a member of the Rise, stuck in the mud of the bog.
                * Around {rise.himher}, you see a couple of erchins, circling their potential prey.
            ]],
            DIALOG_HELP_ME = [[
                * Just as you're about to continue, {rise.heshe} sees you.
                player:
                    !left
                rise:
                    !right
                    Hey! You there! You have to help me!
                player:
                    I don't really HAVE to do anything...
                rise:
                    No, please! I'll pay you! I promise! Just get me out of this!
            ]],
            OPT_HELP_RISE = "Pull {rise.himher} out and protect {rise.himher}",
            DIALOG_HELP_RISE = [[
                player:
                    Urgh, why is it always me?
                * You run over to the rise soldier and pull {rise.himher} out of the mud, almost getting yourself stuck.
                rise:
                    !tired
                    Phew, thank you for doing the right thing.
                * Just as you're about to leave, the erchins make their move.
                player:
                    !exit
                rise:
                    !exit
            ]],
            DIALOG_HELPED_DIED = [[
                * Despite your valiant effort, the rise soldier didn't survive the fight.
            ]],
            OPT_LEAVE_BODY = "Leave {rise.hisher} remains alone",
            TT_LEAVE_BODY = "Gain {1} Resolve",
            DIALOG_LEAVE_BODY = [[
                player:
                    Rest in peace.
                * Not wanting to steal from a corpse, you leave {rise.himher} behind and continue on your way.
            ]],
            OPT_DIED_TAKE_PAYMENT = "Take {1#money}",
            DIALOG_TAKE_PAYMENT = [[
                player:
                    Well, {rise.heshe} won't be needing this anymore.
                * You rifle through {rise.hisher} pockets and find some shills. Not wanting to attract more animals, you continue on your way.
            ]],
            DIALOG_HELPED_LIVED = [[
                player:
                    !left
                rise:
                    !right
                    !tired
                    Thank you.. I couldn't have done that on my own.
                    I don't have much money, but I can give you what I have. Unless you would accept a {1#card} instead?
            ]],
            OPT_TAKE_MONEY = "Accept the money",
            DIALOG_TAKE_MONEY = [[
                rise:
                    !shrug
                    Business is business I guess. Thank you again. You really saved me.
                player:
                    Just another day in the bog, eh?
                * You part ways with {rise}, a little bit richer than before.
            ]],
            OPT_TAKE_ITEM = "Accept the {1#card}",
            DIALOG_TAKE_ITEM = [[
                rise:
                    Good choice, these things come in handy against those dang Spark Baron Mechs. One quick zap is all it takes.
                player:
                    I'll be sure to put this to good use then.
                * You leave {rise} behind, inspecting your new gadget as you continue down the road.
            ]],
            OPT_LET_THEM_GO = "Let {rise} go without payment",
            DIALOG_LET_THEM_GO = [[
                player:
                    Don't worry about payment, I'm just glad we both made it out alive.
                rise:
                    !happy
                    Thank you! I guess there's still a few good people left in the world.
                * You part ways with {rise} and continue on your way.
                player:
                    !exit
                rise:
                    !exit
            ]],
            OPT_BACK_OFF = "Leave in a hurry",
            DIALOG_BACK_OFF = [[
                player:
                    !angry_shrug
                    This is why they tell you not to wander off in the bog. Sorry pal, you're on your own.
                    !exit
                rise:
                    !exit
                * You leave quietly, as the erchins start circling closer and closer.
            ]],
        }
        :Fn(function(cxt)
            cxt.quest:Complete()
            local REWARD_MONEY = math.round(math.randomGauss(40, 80))
            local erchin = CreateCombatParty(
                function() 
                    local t = {}
                    local num_erchin = math.min( cxt.quest:GetRank() + 1, 3)
                    for k = 1, num_erchin do
                        table.insert(t, "ERCHIN")
                    end
                    return t 
                end, cxt.quest:GetRank(), cxt.location)

            local content_id = cxt.quest:GetDifficulty() <= 3 and "RISE_REBEL" or "RISE_RADICAL"
            local rise = cxt.quest:CreateSkinnedAgent( content_id )
            rise:MoveToLocation( cxt.location )
            TheGame:GetGameState():AddAgent( rise )

            cxt:ReassignCastMember("rise", rise)
            rise:GetAspect("health"):SetPercent(.33)
            
            cxt:Dialog("DIALOG_INTRO")

            cxt.enc:SetPrimaryCast(rise)
            
            cxt:Dialog("DIALOG_HELP_ME")

            cxt:Opt("OPT_HELP_RISE")
                :Dialog("DIALOG_HELP_RISE")
                :Battle{
                    flags = BATTLE_FLAGS.SEND_TO_LIMBO,
                    allies = {rise},
                    enemies = erchin,
                    on_win = function(cxt) 
                        if rise:IsDead() then
                            cxt:Dialog("DIALOG_HELPED_DIED")
                            cxt:Opt("OPT_LEAVE_BODY")
                                :PostText("TT_LEAVE_BODY", MAX_RESOLVE_GAIN)
                                :Dialog("DIALOG_LEAVE_BODY")
                                :Fn(function( cxt )
                                    cxt.caravan:UpgradeResolve( MAX_RESOLVE_GAIN )
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)

                            cxt:Opt("OPT_DIED_TAKE_PAYMENT", REWARD_MONEY)
                                :Dialog("DIALOG_TAKE_PAYMENT")
                                :Fn(function(cxt)
                                    cxt.encounter:GainMoney( REWARD_MONEY )
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)
                        else
                            cxt:Dialog("DIALOG_HELPED_LIVED", "shock_box")
                            rise:OpinionEvent( OPINION.SAVED_LIFE )
                            cxt:Opt("OPT_TAKE_MONEY")
                                :Dialog("DIALOG_TAKE_MONEY")
                                :ReceiveMoney( REWARD_MONEY )
                                :Fn(function(cxt)
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)

                            cxt:Opt("OPT_TAKE_ITEM", CARD_TO_GIVE)
                                :Dialog("DIALOG_TAKE_ITEM")
                                :GainCard( CARD_TO_GIVE )
                                :Fn(function(cxt)
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)

                            cxt:Opt("OPT_LET_THEM_GO")
                                :Dialog("DIALOG_LET_THEM_GO")
                                :Fn(function()
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)

                        end
                    end,
                }


            cxt:Opt("OPT_BACK_OFF")
                :Dialog("DIALOG_BACK_OFF")
                :ReceiveOpinion(OPINION.DID_NOT_HELP)
                :Fn(function()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)

