local battle_defs = require "battle/battle_defs"
local negotiation_defs = require "negotiation/negotiation_defs"

local food_options =
{
    SPARKWASP_SKEWERS =
    {
        condition = function()
            local player = TheGame:GetGameState():GetPlayerAgent()
            return player:GetAspect("health"):Get() - 4 > 0 
        end,
        effect = function(cxt)
            ConvoUtil.DoHealthDelta(cxt, -4 )
        end,
        bad = true,
    },

    GELBORE_JELLO =
    {
        condition = function()
            local current_resolve, max_resolve = TheGame:GetGameState():GetCaravan():GetResolve()
            return current_resolve < max_resolve
        end,
        effect = function(cxt)
            ConvoUtil.DoResolveDelta(cxt, 5 )
        end,
    },

    OSHNU_FILLET = 
    {
        bad = true,
        effect = function( cxt )           
            cxt:ForceTakeCards{"sick"}
        end, 
    },

    HAWB_DRUMSTICK =
    {
        effect = function(cxt)            
            cxt:ForceTakeCards{"hawb_drumstick"}
        end,
    },

    PICKLED_FLOATER =
    {
        effect = function( cxt )
            cxt:ForceTakeCards{"pickled_floater"}
        end
    },

    THRASHER_STEW = 
    {
        bad = true,
        effect = function( cxt )
            cxt:ForceTakeCards{"ulcer"}
        end
    },
}

--Card from food effects
Content.AddBattleCard( "hawb_drumstick", 
{
    name = "Hawb Drumstick",
    anim = "taunt",
    flavour = "'Hawbs were doomed from the start, being extremely large, slow moving, and delicious.'",
    desc = "{HEAL 8}.",

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    cost = 0,
    target_type = TARGET_TYPE.SELF,

    max_charges = 2,

    flags = battle_defs.CARD_FLAGS.ITEM,

    OnPostResolve = function( self, battle, attack)
        self.owner:HealHealth( 8, self )
    end,
})
Content.AddBattleCard( "pickled_floater", 
{
    name = "Pickled Floater",
    anim = "taunt",
    flavour = "'Try not to think about it, and bite the brain so it moves less on the way down.'",
    desc = "Gain 5 {CONCENTRATION}.",

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    cost = 0,
    target_type = TARGET_TYPE.SELF,

    max_charges = 2,

    flags = battle_defs.CARD_FLAGS.ITEM,

    OnPostResolve = function( self, battle, attack)
        self.owner:AddCondition("CONCENTRATION", 5, self)
    end,
})

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
}

QDEF:AddCastByAlias{
    cast_id = "chef",
    alias = "WANDERING_CHEF",
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_CONFRONT"
    end)

    :State("STATE_CONFRONT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !happy
                * You're stopped on the road by {Agent}, who sizes you up with an appetite for opportunity. You're immediately on guard.
                agent:
                    $miscAlluring
                    You look like a man with a howling stomach that only a good, road-cooked meal can satisfy. 
                player:
                    !crossed
                    $neutralThoughtful
                    Good <i>and</i> road-cooked? An ambitious combination. 
                agent:
                    !wink
                     I'm an ambitious fella. Now, let your gullet lead the way. You won't get a chance like this, lemme tell ya. 
            ]],
            DIALOG_ACCEPT = [[
                player:
                    !agree
                    One can't march on an empty stomach. I'll try the {1}.
                agent:
                    !surprised
                    W-well, hey, now, that's the right attitude!
                    One {1}, coming right up. 
            ]],
            DIALOG_DECLINE = [[
                player:
                    $neutralThoughtful
                    I think not. Hunger sharpens the mind, you know.
                agent: 
                    !dubious
                    $neutralDubious
                    Sounds like nonsense to me. But it's your nonsense, so have fun with it. 
                * {agent} casts you a pitying look before continuing on {agent.hisher} way down the road. 
            ]],            

            OPT_DECLINE = "Decline the offer",
            OPT_TRY_FOOD_SPARKWASP_SKEWERS = "Try the tart stinger tart",
            DIALOG_SPARKWASP_SKEWERS = [[
                player:
                    !drink
                * The tart scratches its way down your throat, but it's got a sour bite that tickles your taste. 
                * But that's about all that's pleasing: soon after, you feel a sharp pain in your stomach.
                player:
                    !wince2
                    Ughh...
                agent:
                    !thought 
                    $neutralThoughtful
                    Hmm. Maybe I should've milked the stingers, first.
            ]],
            OPT_TRY_FOOD_GELBORE_JELLO = "Try the sparkling oshnu aspic",
            DIALOG_GELBORE_JELLO = [[
                player:
                    !drink
                * The aspic makes for some acrobatic slurping, but it tickles your tonsils on the way down. 
                player:
                    !burp
                    Hm! Effervescent!
                agent:  
                    !happy
                    I use swamp gas for the carbonation. For buoyancy!
                * {agent} leaves you to your digestion, looking pleased as a pickle.
            ]],
            OPT_TRY_FOOD_OSHNU_FILLET = "Try the grawkit fillets",
            DIALOG_OSHNU_FILLET = [[
                player:
                    !drink
                * You bite down on the fillets, and in a blink your lips go numb. 
                player:
                    !scared
                    $neutralTakenAback
                    Wha—
                    Wha habbund? 
                agent:  
                    !bashful
                    $miscEmbarrassed
                    Ah, um... that's normal. It'll wear off. 
                    But, uh, just know the same thing'll happen on the way out, if you catch my meaning. 
            ]],
            OPT_TRY_FOOD_HAWB_DRUMSTICK = "Try the scab-roasted hawb drumstick",
            DIALOG_HAWB_DRUMSTICK = [[
                player:
                    !drink
                * The skin is crispy and charred black in all the right places, and you have to catch a bit of juice as it dribbles down your chin.
                player:
                    !burp
                    My! 
                agent:
                    !chuckle
                    Good, huh?
                player:
                    !happy
                    I'm at a loss for words. 
                * {agent} leaves you to your digestion, looking pleased as a pickle.
            ]],
            OPT_TRY_FOOD_PICKLED_FLOATER = "Try the curds & floaters",
            DIALOG_PICKLED_FLOATER = [[
                player:
                    !drink
                * You help yourself to a spoonful of the sweet, spicy pudding. It wobbles going down. 
                player:
                    !thought
                    I didn't realize floaters could be served sweet.
                agent:
                    !chuckle
                    Sweet and strong! That's the way I like 'em, personally.
                * {agent} leaves you to your digestion, looking pleased as a pickle.
            ]],
            OPT_TRY_FOOD_THRASHER_STEW = "Try the sickletickle stew",
            DIALOG_THRASHER_STEW = [[
                player:
                    !drink
                * You drink the stew and bite into something that shatters under your teeth.
                player:
                    !spit
                    Blagh!
                agent:
                    !bashful
                    No? Not to your liking?
                * You pick a splinter out of your tongue. 
                agent:
                    !placate
                    $miscEmbarrassed
                    Ah, it was an experiment in texture. 
            ]],       
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.encounter:SetPrimaryCast(cxt.quest:GetCastAgent("chef"))

                local temp_options = {}
                for i, id, data in sorted_pairs( food_options ) do
                    --if there's no condition or the condition is met, add it
                    if data.condition == nil or data.condition() then
                        table.insert(temp_options, id)
                    end
                end
                table.shuffle(temp_options)
                local length = #temp_options
                for i=1, length - 3 do
                    table.remove(temp_options, math.random(#temp_options))
                end
                cxt.quest.param.options = temp_options
                cxt:Dialog("DIALOG_INTRO")
            end

            for k,v in pairs(cxt.quest.param.options) do
                cxt:Opt("OPT_TRY_FOOD_" .. v)
                    :PreIcon(global_images.food)
                    :Fn(function(cxt)
                        cxt:Dialog("DIALOG_" .. v)
                        if food_options[v].bad then
                            cxt:GoTo("STATE_BAD_FOOD", v)
                        else
                            if food_options[v].effect then
                                food_options[v].effect(cxt)
                            end

                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    end)
            end

            cxt:Opt("OPT_DECLINE")
                :PreIcon(global_images.dislike)
                :Dialog("DIALOG_DECLINE")
                :ReceiveOpinion( OPINION.INSULT )
                :Travel()

        end)

    :State("STATE_BAD_FOOD")
        :Loc{
            DIALOG_APOLOGY = [[
                player:
                    !injured
                    Your experiment <i>failed</i>, chef. 
                agent:
                    !surprise
                    W-well... <i>you</i> try coming up with new recipes on the road, huh?
                    It ain't as easy as it looks!
                player:
                    $neutralResigned
                    I suppose I should've sussed that out myself.
                agent:
                    !scared
                    $scaredDefault
                    Please don't tell anyone about this. I might lose one of my Michelin starfishes.
            ]],
            OPT_DONT_TELL = "Promise not to tell anyone",
            TT_DONT_TELL = "You'll feel the extent of that dubious meal!",

            DIALOG_DONT_TELL = [[
                player:
                    !dubious
                    $neutralResigned
                    I won't tell anyone, for my own reputation's sake, if not yours.
                    $neutralDubious
                    But next time, might I suggest trying your recipes out on a yote, first?
                agent:
                    !thought
                    $neutralThoughtful
                    Dunno if a yote has the palate for it, but... it does get real lonely on the road. 
                * {agent} cleans up {agent.hisher} gear and leaves you to rinse the taste from your mouth.
            ]],
            OPT_INSULT = "Scold {agent}",
            TT_INSULT = "You'll feel better for scolding {agent}, but {agent.heshe} isn't likely to appreciate it.",

            DIALOG_INSULT = [[
                player:
                    !angry_accuse
                    Next time you wish to test a recipe, feed it to the dogs.
                    That's the only thing your food is fit for, anyway.
                agent:
                    !angry
                    $angryDefault
                    H-how dare you! I've tasted each and every one of my recipes. 
                    But I suppose there's no accounting for unsophisticated tastes like yours, you shiftless vagabond! 
                    !exit
                * {Agent} turns to make a haughty retreat, but not before you've plastered {agent.hisher} backside with the scraps.
                player:
                    !angry
                    You'll note my aim is truer than your palate!
            ]],
        }
        
        :Fn(function(cxt, food)
            cxt:Dialog("DIALOG_APOLOGY")

            cxt:Opt("OPT_DONT_TELL")
                :PostText( "TT_DONT_TELL" )
                :Dialog("DIALOG_DONT_TELL")
                :ReceiveOpinion( OPINION.SOCIAL_FAVOUR )
                :Fn(function(cxt)
                    if food_options[ food ] then
                        food_options[food].effect(cxt)
                    end
                end)
                :Travel()

            cxt:Opt("OPT_INSULT")
                :PostText( "TT_INSULT" )
                :Dialog("DIALOG_INSULT")
                :ReceiveOpinion( OPINION.INSULT )
                :Travel()
        end)
