local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    on_init = function(quest) 
        quest.param.wants_job = math.random() > .5
    end,
}

:AddCast{
    cast_id = "worker",

    condition = function(agent, quest)
        return agent:InLimbo() and (agent.def.id == "HEAVY_LABORER" or agent.def.id == "LABORER") and agent:GetRelationship() >= RELATIONSHIP.NEUTRAL
    end,
}
:AddCastFallback
{
    cast_fn = function(quest, t)
        local def = "HEAVY_LABORER"
        local agent = quest:CreateSkinnedAgent( def )
        table.insert( t, agent )
    end,
}
:AddOpinionEvents{
    gave_little =  
    {
        delta = 
        {
            relationship_delta = 
            {
                [RELATIONSHIP.LOVED] = RELATIONSHIP.LIKED,
                [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.LIKED,
            },
        },

        txt = "Gave spare change when they asked for help",
    },
    gave_big =  
    {
        delta = 
        {
            relationship_delta = 
            {
                [RELATIONSHIP.LIKED] = RELATIONSHIP.LOVED,
                [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.LOVED,
                [RELATIONSHIP.DISLIKED] = RELATIONSHIP.NEUTRAL,
                [RELATIONSHIP.HATED] = RELATIONSHIP.NEUTRAL,
            },
        },
        txt = "Gave a lot of money when they asked for help",
    },
    gave_nothing =  
    {
        delta = 
        {
            relationship_delta = 
            {
                [RELATIONSHIP.LOVED] = RELATIONSHIP.DISLIKED,
                [RELATIONSHIP.LIKED] = RELATIONSHIP.DISLIKED,
                [RELATIONSHIP.NEUTRAL] = RELATIONSHIP.DISLIKED,
            },
        },
        txt = "Gave nothing when they asked for help",
    },
}



local function PickDialog(rel, neut, friend, enemy)
    if rel > RELATIONSHIP.NEUTRAL then
        return friend
    elseif rel < RELATIONSHIP.NEUTRAL then
        return enemy
    else
        return neut
    end
end



QDEF:AddConvo()
    :ConfrontState("CONFRONT", function() return true end)
        :Loc{
            DIALOG_INTRO = [[
                * A lone figure is loitering at the side of the road.
                player:
                    !left
                agent:
                    !right
                    $miscDepressed
                    Hey, pal. You got a shill to spare for an out-of-work laborer?
                    $miscDepressed
                    I'm real hungry, and the Barons ain't hiring today.
            ]],
            DIALOG_INTRO_FRIEND = [[
                * You see {agent} standing morosely by the roadside. {agent.HeShe} perks up as you approach.
                player:
                    !left
                agent:
                    !right
                    $happyAmazed
                    {player}! It's so good to see you!
                    I've been out here all day trying to gather up a bit of scratch for a bite to eat.
                    $miscDepressed
                    But nobody's got the time for me.
            ]],
            DIALOG_INTRO_ENEMY = [[
                * You see {agent} standing by the road. {agent.HeShe} tenses up as you approach.
                player:
                    !left
                agent:
                    !right
                    $angryHostile
                    What are <b>you</> looking at?
                player:
                    I'm just walking here.
                agent:
                    $angryHostile
                    Yeah, well, keep walking. I'm trying to earn an honest shill from the kindness of strangers.
            ]],



            OPT_GIVE_A_LITTLE = "Give {agent} some spare change",
            DIALOG_GIVE_A_LITTLE = [[
                player:
                    Here. Go get some noodles or something.
                agent:
                    $happyResigned
                    Thanks. Every little bit helps.
            ]],
            DIALOG_GIVE_A_LITTLE_FRIEND = [[
                player:
                    Here you go. I can spare this.
                agent:
                    $neutralResigned
                    Is that all? Well, thanks, I guess.
                    Be seeing you around.                    
            ]],
            DIALOG_GIVE_A_LITTLE_ENEMY = [[
                player:
                    Here. Take this.
                agent:
                    $angryReluctant
                    Wow, thanks.
                    You oshnu's arse.
            ]],


            OPT_GIVE_A_LOT = "Give {agent} a lot",
            DIALOG_GIVE_A_LOT = [[
                player:
                    Here. Go get yourself cleaned up.
                agent:
                    $happyAmazed
                    Wow, really? You're alright, you know that?
                    If you ever need anything, just ask!
            ]],
            DIALOG_GIVE_A_LOT_FRIEND = [[
                player:
                    That's unfortunate, friend. Here, let me help.
                agent:
                    $happyThanks
                    Thanks, {player}. It's good to know you can still count on your friends, even amidst all of this.
            ]],
            DIALOG_GIVE_A_LOT_ENEMY = [[
                player:
                    I know we're not on the best of terms, but here. Take this. 
                agent:
                    $neutralDubious
                    That's a lot of money, what's the catch?
                player:
                    No catch. Just maybe remember who gave it to you when you spend it.
                agent:
                    $happyTakenAback
                    Thank you. I will.
            ]],
            
            OPT_OFFER_A_JOB = "Offer to hire {agent} as protection",
            DIALOG_OFFER_JOB = [[
                player:
                    You want work? I could use a strong set of arms at my side.
            ]],
            DIALOG_OFFER_JOB_FRIEND = [[
                player:
                    Why don't you work with me? The pay's not great, but the company's good.
            ]],
            DIALOG_WANT_JOB = [[
                agent:
                    $neutralResigned
                    Yeah, sure. Beats sitting around here.
            ]],
            
            OPT_HIRE =  "Hire {agent}",
            OPT_NO_HIRE = "Don't hire {agent}",
            DIALOG_TAKE_JOB = [[
                player:
                    Deal. Let's go.
            ]],
            DIALOG_NO_TAKE_JOB = [[
                player:
                    $neutralDubious
                    I'm having second thoughts, sorry.
            ]],
            DIALOG_NO_WANT_JOB = [[
                agent:
                    $angryHostile
                    Do I look like a grifter to you? I'm not risking my neck for a few measly shills.
            ]],
            
            DIALOG_WANT_JOB_FRIEND = [[
                agent:
                    $happyThanks
                    Yeah, let's do it. Safety in numbers, right?
            ]],
            DIALOG_NO_WANT_JOB_FRIEND = [[
                agent:
                    $neutralDirect
                    Listen, I like you, but I can't do what you do. No offense, of course!
            ]],

            OPT_SAY_NO = "Don't give {agent} anything",
            
            DIALOG_SAY_NO = [[
                player:
                    Not interested.
                agent: 
                    $miscMocking
                    Yeah, well, I hope nobody helps <b>you</>, when you're at your lowest!
            ]],
            DIALOG_SAY_NO_FRIEND = [[
                player:
                    Sorry, {agent}. You have to solve this one yourself.
                agent:
                    !spit
                    $miscMocking
                    I'll remember that, 'friend'.
            ]],
            DIALOG_SAY_NO_ENEMY = [[
                * You leave, not wanting to provoke {agent.himher}.
                player:
                    !exit
                agent:
                    $miscMocking
                    Yeah, that's what I thought!
            ]]

        }
        :SetLooping()
        :Fn(function(cxt)
            
            cxt.quest:Complete()
            if cxt:FirstLoop() then
                cxt.quest:GetCastMember("worker"):MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("worker"))
                cxt:Dialog(PickDialog(cxt:GetAgent():GetRelationship(), "DIALOG_INTRO", "DIALOG_INTRO_FRIEND", "DIALOG_INTRO_ENEMY"))
            end

            local SMALL_AMT = 5
            local LARGE_AMT = 50
            local HIRE_AMT = 50

            cxt:Opt("OPT_GIVE_A_LITTLE")
                :Dialog(PickDialog(cxt:GetAgent():GetRelationship(), "DIALOG_GIVE_A_LITTLE", "DIALOG_GIVE_A_LITTLE_FRIEND", "DIALOG_GIVE_A_LITTLE_ENEMY"))
                :DeliverMoney(SMALL_AMT, {no_scale = true})
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("gave_little"))
                :Travel()
            
            cxt:Opt("OPT_GIVE_A_LOT")
                :Dialog(PickDialog(cxt:GetAgent():GetRelationship(), "DIALOG_GIVE_A_LOT", "DIALOG_GIVE_A_LOT_FRIEND", "DIALOG_GIVE_A_LOT_ENEMY"))
                :DeliverMoney(LARGE_AMT, {no_scale = true})
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("gave_big"))
                :Travel()


            local wants_job = cxt.quest.param.wants_job and TheGame:GetGameState():IsHiringAvailable()
            cxt:Opt("OPT_OFFER_A_JOB")
                :Dialog(cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_OFFER_JOB_FRIEND" or "DIALOG_OFFER_JOB")
                :ReqRelationship( RELATIONSHIP.NEUTRAL )
                :Fn(function() 
                    if wants_job then
                        cxt:Dialog(cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_WANT_JOB_FRIEND" or "DIALOG_WANT_JOB")
                        cxt:Opt("OPT_HIRE")
                            :Dialog("DIALOG_TAKE_JOB")
                            :DeliverMoney(HIRE_AMT)
                            :RecruitMember( PARTY_MEMBER_TYPE.HIRED )
                            :Travel()

                        cxt:Opt("OPT_NO_HIRE")
                            :Dialog("DIALOG_NO_TAKE_JOB")
                    else
                        cxt:Dialog(cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_NO_WANT_JOB_FRIEND" or "DIALOG_NO_WANT_JOB")
                    end
                end)

            cxt:Opt("OPT_SAY_NO")
                :Dialog(PickDialog(cxt:GetAgent():GetRelationship(), "DIALOG_SAY_NO", "DIALOG_SAY_NO_FRIEND", "DIALOG_SAY_NO_ENEMY"))
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("gave_nothing"))
                :Travel()

        end)

