local QDEF = QuestDef.Define{
    title = "Robotics Seller",
    qtype = QTYPE.EVENT,
    cooldown = EVENT_COOLDOWN.LONG,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest) 
        return TheGame:GetGameState():GetCaravan():CanOwnPet() and TheGame:GetGameState():GetPlayerAgent():GetMoney() > 100
    end,
}

:AddCast{
    cast_id = "seller",
    cast_fn = function(quest, t)
        local def = "WEALTHY_MERCHANT"
        local agent = quest:CreateSkinnedAgent( def )
        table.insert( t, agent )
    end,
}

QDEF:AddConvo()

    :Confront(function(cxt, who)
        if cxt.location:HasTag("in_transit") then
            return "STATE_MERCHANT"
        end
    end)

    :State("STATE_MERCHANT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !cagey
                    $neutralTakenAback
                    Hey. You want to buy some robotics?
                    I assure you they are of the highest quality!
                    !shrug
                    The highest quality in the Grout Bog anyways.
                *** {agent} asks if you would like to buy some robotics.
            ]],
            OPT_BUY_DRONE = "Buy {1#card}",
            OPT_BUY_ROBODOG = "Buy an Autodog",
            OPT_BUY_GRENADES = "Buy a box of grenades ({1#card_list})",
            DIALOG_PURCHASE_DRONE = [[
                player:
                    !give
                    $neutralResigned
                    I'll take the drone.
                agent:
                    !happy
                    You wont be disappointed!
                    !give
            ]],
            DIALOG_PURCHASE_ROBODOG = [[
                player:
                    !give
                    $neutralResigned
                    What can I say, I'm a sucker for animals.
                agent:
                    !surprised
                    $happyTakenAback
                    Right... Animals... Well, it's all yours!
            ]],
            DIALOG_PURCHASE_GRENADES = [[
                player:
                    !give
                    $neutralResigned
                    A box of grenades never hurt.
                    !shrug
                agent:
                    Uhh you might be using them wrong.
                    $happyTakenAback
                    Nevertheless, here you go!
                    !give
            ]],
            OPT_ASK_ABOUT_DOG = "Ask about the robot dog.",
            DIALOG_ASK_ABOUT_DOG = [[
                player:
                    Robotic dog? What's the catch? Is it broken?
                agent:
                    No, no, it's in perfect condition and it will be fiercely loyal to you.
                    Just don't ask where it came from.
                    !wink
            ]],
            OPT_DECLINE = "Decline the offer",
            DIALOG_DECLINE = [[
                player:
                    Sorry, I'm going to have to pass this time.
                agent:
                    !disappoint
                    Oh well, can't win 'em all I guess.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                local agent = AgentUtil.GetFreeAgent("LABORER", function(agent) return agent:GetRelationship() >= RELATIONSHIP.NEUTRAL end)
                agent:MoveToLocation(cxt.location)
                cxt.encounter:SetPrimaryCast( agent )

                cxt.quest.param.grenade_cards = {}
                for i = 1, 2 do
                    local def = BattleCardCollection.AllLocalItems( function(def) return CheckBits(def.item_tags or 0, ITEM_TAGS.GRENADE) end ):Pick(1)[1]
                    if def then
                        table.insert( cxt.quest.param.grenade_cards, def.id )
                    end
                end
 
                cxt:Dialog("DIALOG_INTRO")
            end
            
            cxt:Opt("OPT_BUY_DRONE", "combat_drone")
                :DeliverMoney(100, {is_shop = true})
                :PreIcon( global_images.buying )
                :Dialog("DIALOG_PURCHASE_DRONE")
                :Travel()

            cxt:Opt("OPT_BUY_ROBODOG")
                :DeliverMoney(150, {is_shop = true})
                :PreIcon( global_images.buying )
                :Fn(function( cxt )
                    local agent = Agent("AUTODOG")
                    agent:GenerateRandomName()
                    TheGame:GetGameState():AddAgent( agent )
                    agent:Recruit(PARTY_MEMBER_TYPE.CREW)
                end)
                :Dialog("DIALOG_PURCHASE_ROBODOG")
                :Travel()
            
            cxt:Opt("OPT_BUY_GRENADES", cxt.quest.param.grenade_cards)
                :DeliverMoney(150, {is_shop = true})
                :PreIcon( global_images.buying )
                :GainCards(cxt.quest.param.grenade_cards)
                :Dialog("DIALOG_PURCHASE_GRENADES")
                :Travel()

            cxt:Opt("OPT_ASK_ABOUT_DOG")
                :Dialog("DIALOG_ASK_ABOUT_DOG")

            cxt:Opt("OPT_DECLINE")
                :PreIcon( global_images.close )
                :Dialog("DIALOG_DECLINE")    
                :Travel()
        end)