local CHANCE_FOR_ENEMY = .35

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    is_negative = true,
    precondition = function(quest)
        return Util.IsPlayerWorkingWithFaction("ADMIRALTY")
    end,
}

:AddCast{
    cast_id = "admiralty",
    condition = function ( agent, quest )
        return agent:GetFactionID() == "ADMIRALTY" and agent:GetRelationship() > RELATIONSHIP.NEUTRAL
    end
}

:AddCast{
    cast_id = "laborer",
    condition = function ( agent, quest )
        return agent:GetRelationship() >= RELATIONSHIP.NEUTRAL and agent:GetFactionID() == "FEUD_CITIZEN"
    end,
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_INTERVENE"
    end)

    :State("STATE_INTERVENE")
        :Loc{
            DIALOG_INTRO = [[
                admiralty:
                    !left
                laborer:
                    !right
                    !injured
                * You see {admiralty} trying to extort {laborer}.
                admiralty:
                    Hand over the money and you'll be fine.
                laborer:
                    Please.. I need this to feed my family.

            ]],
            DIALOG_EXTORT = [[
                player:
                    !left
                    Didn't you hear {admiralty.himher}? Hand over the money!
            ]],
            DIALOG_DEFEND = [[
                admiralty:
                    !right
                player:
                    !left
                    Hey! Let {laborer.himher} be!
                admiralty:
                    How dare you? You can't give me orders!
            ]],
            DIALOG_LEAVE = [[
                * You pretend to not have seen anything and slip away unnoticed.
            ]],
            DIALOG_KILLED = [[
                admiralty:
                    !right
                    I didn't want you to kill {laborer.himher}!
                    You just lost me a steady source of cash.
            ]],
            DIALOG_BEAT = [[
                laborer:
                    !right
                    !injured
                    Have mercy! Just take the money and go.
            ]],
            DIALOG_DEFENDED_KILL = [[
                laborer:
                    !right
                    Thank you, grifter! That scum finally got what {admiralty.heshe} deserved.
                    I won't forget this.
            ]],
            DIALOG_DEFENDED = [[
                laborer:
                    !right
                    Thank you, grifter! I won't forget this.
                admiralty:
                    !right
                    !injured
                    You'll regret this, {player}.
            ]],
            DIALOG_STALEMATE = [[
                *You defeated {admiralty}, but couldn't save {laborer}.
            ]],
            OPT_EXTORT = "Help {admiralty} extort {laborer}",
            OPT_DEFEND = "Defend {laborer}",
            OPT_LEAVE = "Leave before anyone sees you"
        }

        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_EXTORT")
                :Dialog("DIALOG_EXTORT")
                :Battle
                {
                    allies = { "admiralty" },
                    enemies = { "laborer" },
                    on_win = function(cxt)
                        if cxt:GetCastAgent("laborer"):IsDead() then
                            cxt:Dialog("DIALOG_KILLED")
                            cxt:GetCastAgent("admiralty"):OpinionEvent(OPINION.WASTED_TIME)
                        else
                            cxt:Dialog("DIALOG_BEAT")
                            local money = math.floor( SIDEQUEST_REWARD[clamp(TheGame:GetGameState():GetCurrentBaseDifficulty(), 1, #SIDEQUEST_REWARD)] * 0.5)
                            cxt:GetCastAgent("laborer"):DeltaMoney(-money)
                            cxt.encounter:GainMoney( math.floor(money / 2 ))
                            cxt:GetCastAgent("laborer"):OpinionEvent(OPINION.EXTORTION)

                        end
                        StateGraphUtil.AddLeaveLocation(cxt) 
                    end
                }

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle
                {
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = { "laborer" },
                    enemies = { "admiralty" },
                    on_win = function(cxt)
                        if cxt:GetCastAgent("laborer"):IsDead() then
                            cxt:Dialog("DIALOG_STALEMATE")
                        else
                            if cxt:GetCastAgent("admiralty"):IsDead() then
                                cxt:Dialog("DIALOG_DEFENDED_KILL")
                            else
                                cxt:Dialog("DIALOG_DEFENDED")
                                cxt:GetCastAgent("admiralty"):OpinionEvent(OPINION.REFUSED_TO_HELP)
                            end
                        end
                        StateGraphUtil.AddLeaveLocation(cxt) 
                    end
                } 

            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :Travel()

        end)