local SPREE_FAIL_FEE = 50
local backup

function CalcToll(quest)
    local discount = Util.IsPlayerWorkingWithFaction("ADMIRALTY") and .5 or 1
    local base_amount = (1 + (quest:GetRank()-1))*25
    local total_amount_to_ask_for = math.round( discount*base_amount)
    return base_amount, total_amount_to_ask_for
end


local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    
    precondition = function(quest)
        local base_amount = CalcToll(quest)
        return (Util.IsPlayerWorkingWithFaction("BANDITS") or Util.IsPlayerWorkingWithFaction("ADMIRALTY")) and TheGame:GetGameState():GetCaravan():GetMoney() >= base_amount
    end,
}

:AddCast{
    cast_id = "admiralty",
    condition = function ( agent, quest )
        return agent:GetFactionID() == "ADMIRALTY"
    end
}

:AddOpinionEvents{
    pulled_rank = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Pulled Admiralty Rank",
    },
}



QDEF:AddConvo()
    :Confront(function(cxt, who)
        cxt.quest.param.base_amount, cxt.quest.param.total_amount_to_ask_for = CalcToll(cxt.quest)
        if Util.IsPlayerWorkingWithFaction("BANDITS") then
            return "STATE_SPREE"
        else
            return "STATE_ADMIRALTY"
        end
    end)

    :State("STATE_ADMIRALTY")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * The road ahead is blocked by a group of Admiralty officers.
                agent:
                    !placate
                    $neutralDirect
                    Hold it. This here's a toll road.
                player:
                    $neutralDubious
                    It is? Since when?
                agent:
                    Since the ink-spitters at headquarters said so.
                    The toll is {1#money}.
                player:
                    $miscPersuasive
                    Come on. I work for Oolo. That must count for something.
                agent:
                    $neutralThoughtful
                    Oolo, eh? Okay, let's call it {2#money}.
                player:
                    !angry_shrug
                    $neutralResigned
                    Wow. What a perk.
            ]],
            DIALOG_PAY = [[
                player:
                    $neutralResigned
                    Take it, then.
                    !give
                agent:
                    !happy
                    !take
                player:
                    No chance I can claim this as an expense, huh?
                agent:
                    $happyDefault
                    Not Heshin' likely. 
            ]],
            DIALOG_ARGUE = [[
                player:
                    {1#money}? Hrm, Oolo said I was deputized.
                    !thought
                    I wonder what kind of authority that gives me...
            ]],
            DIALOG_CONVINCED = [[
                player:
                    !overthere
                    You know what, lemme just run back to Oolo and ask her.
                    I got time. 
                agent:
                    !bashful
                    Now, hang on. Ain't no reason to involve the brass.
                    Look, you don't wanna pay the toll? 
                    Fine. I'll let it go. Just this once, mind.
                player:
                    !cruel
                    Awful kind of you. 
                * You continue down the road, kindly ignoring the grumbling that follows in your wake.
            ]],
            DIALOG_NEGOTIATION_FAIL = [[
                player:
                    !eureka
                    Maybe <i>you</i> should even be paying <i>me</i>. 
                agent:
                    Maybe I should...
                    !angry_accuse
                    Listen here, you. There's only one of us wearing Admiralty blues. 
                    I changed my mind. You <i>don't</i> qualify for the Admiralty discount. 
                    !crossed
                    That'll be {1#money}.
            ]],
            DIALOG_FIGHT = [[
                player:
                    $neutralResigned
                    !fight
                    Just remember you brought this on yourself when you tried to cheat a hunter, okay?
            ]],
            DIALOG_LEADER_KILLED = [[
                * With the mastermind taken care of, the rest of the Admiralty thugs head for the hills.
                player:
                    !angry_threaten
                    And stay gone!
            ]],
            DIALOG_EVERYONE_KILLED = [[
                player:
                    !tired
                * You drag the bodies off the road for the scavengers to feast on, and get back to what you were doing before the interruption.
            ]],
            DIALOG_BEAT = [[
                agent:
                    !right
                    !injured
                    $scaredStammering
                    S-stop, stop. It was just a bit of fun.
                    Hazing, you know? Welcome aboard. 
                    !injured_shrug
                    We're all friends here.
                player:
                    !happy
                    Oh, I know. You're alive, aren't you?
                    S'cause I used my <i>friendly</i> knives. 
                * You continue down the road, leaving the wounded behind. 
            ]],
            OPT_PAY = "Pay the toll",
            OPT_ARGUE = "Convince {agent} to leave you alone",
            OPT_FIGHT = "Attack {agent}",

            OPT_TALK_TO_MANAGER = "Threaten to talk to Oolo",
            DIALOG_TALK_TO_MANAGER = [[
                player:
                    !point
                    Did you miss the part where I mentioned that I'm a close personal associate of Oolo?
                    You know... your boss.
                    You wouldn't want this getting back to her, would you?
                agent:
                    !dubious
                    Are you pulling rank on me?
                player:
                    !crossed
                    Yes.
                agent:
                    Listen, Oolo sets the quotas, and I catch holy hell if I don't meet them.
                    Consider my position!
                player:
                    ...
                    I've considered it.
                    !point 
                    Now let me through!
                agent:
                    !overthere
                    Whatever. 
                    Get out of here, grifter.
            ]],

        }

        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                backup = CreateCombatBackup(cxt.quest:GetCastAgent("admiralty"),"ADMIRALTY_PATROL", cxt.quest:GetRank())
                cxt.encounter:SetPrimaryCast(cxt.quest:GetCastAgent("admiralty"))
                cxt:Dialog("DIALOG_INTRO", cxt.quest.param.base_amount, cxt.quest.param.total_amount_to_ask_for)
            end

            cxt:Opt("OPT_PAY")
                :Dialog("DIALOG_PAY")
                :DeliverMoney(cxt.quest.param.total_amount_to_ask_for, { no_scale = true })
                :Travel()

            cxt:Opt("OPT_TALK_TO_MANAGER")
                :Dialog("DIALOG_TALK_TO_MANAGER")
                :ReceiveOpinion("pulled_rank")
                :Travel()

            cxt:Opt("OPT_ARGUE")
                :Dialog("DIALOG_ARGUE", cxt.quest.param.total_amount_to_ask_for)
                :Negotiation
                {
                    target_agent = cxt:GetCastAgent("admiralty"),
                    on_success = function(cxt, skill_used) 
                        cxt:Dialog("DIALOG_CONVINCED")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt.quest.param.total_amount_to_ask_for = cxt.quest.param.base_amount
                        cxt:Dialog("DIALOG_NEGOTIATION_FAIL", cxt.quest.param.total_amount_to_ask_for)
                        cxt:GetAgent():OpinionEvent(OPINION.WASTED_TIME)
                    end,
                } 

            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle
                {
                    on_win = function(cxt) 
                        if cxt:GetCastAgent("admiralty"):IsDead()then
                            local survivors = {}
                            for k,v in ipairs(backup) do
                                if not v:IsDead() then
                                    if v:IsSentient() then
                                        table.insert(survivors, v)
                                    end
                                end
                            end

                            if #survivors == 0 then
                                cxt:Dialog("DIALOG_EVERYONE_KILLED")
                            else
                                cxt:Dialog("DIALOG_LEADER_KILLED")
                            end
                        else
                            cxt:Dialog("DIALOG_BEAT")
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }           
        end)

    :State("STATE_SPREE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * The road ahead is blocked by a group of Admiralty officers.
                agent:
                    !placate
                    $neutralDirect
                    Hold it. This here's a toll road.
                player:
                    $neutralDubious
                    It is? Since when?
                agent:
                    Since the ink-spitters up at headquarters said so.
                    The toll is {1#money}.
            ]],
            DIALOG_PAY = [[
                player:
                    $neutralResigned
                    Lucky for you, even hunters have to play by the rules.
                    !give
                agent:
                    !take
                    And it's mighty pleasant when you do.
                    You need a receipt?
                player:
                    !dubious
                    No?
                agent:
                    $neutralDirect
                    Good, 'cause I ain't got one for you.
                * You continue down the road, making a note to avoid this route next time.
            ]],
            DIALOG_ARGUE = [[
                player:
                    {1#money}? Do I look like I'm made of shills?
                agent:
                    !crossed
                    You wanna use the road, you gotta pay. 
                    Doesn't matter what you look like. I'm not prejudiced.
                player:
                    !hips
                    And what if I don't want to use the road?
            ]],
            DIALOG_CONVINCED = [[
                player:
                    !overthere
                    $miscPersuasive
                    What if I just stepped a little bit over there, off the path, and into the trees?
                agent:
                    !dubious
                    What, in the brush?
                    There's bugs and stuff in there.
                player:
                    $neutralJoke
                    Terrifying. Will I have to pay the toll then?
                agent:
                    !thought 
                    $neutralThoughtful
                    Huh. I suppose if I can't see you, then you never really crossed.
                    And it saves me some paperwork besides.
                    !exit
                player:
                    !exit
                * {agent} looks the other way while you slink into the trees. You travel through the brush for a spell before coming back onto the main road. 
                * You check yourself over, and find a cuttlebeetle trying to lay its egg sac inside the cuff of your sleeve.
                player:
                    !left
                    !wince
                    Ugh. 
                * You wipe the slime out as best you can and keep going.  

            ]],
            DIALOG_NEGOTIATION_FAIL = [[
                player:
                    !overthere
                    What if I just, you know, walked <i>beside</i> the road?
                agent:
                    !angry
                    What if I just shoved your face into the dirt?
                    The toll is {1#money}!
            ]],
            DIALOG_FIGHT = [[
                player:
                    $neutralResigned
                    !fight
                    Just remember you brought this on yourself when you tried to cheat a hunter, okay?
            ]],
            DIALOG_LEADER_KILLED = [[
                * With the mastermind taken care of, the rest of the Admiralty thugs head for the hills.
                player:
                    !angry_threaten
                    And stay gone!
            ]],
            DIALOG_EVERYONE_KILLED = [[
                player:
                    !tired
                * You drag the bodies off the road for the scavengers to feast on, and get back to what you were doing before the interruption.
            ]],
            DIALOG_BEAT = [[
                agent:
                    !right
                    !injured
                    $scaredStammering
                    S-stop, that's enough.
                    This isn't a post worth dying for. You want the road? Take it. 
                    It's yours, potholes and all.
                player:
                    !happy
                    Mighty generous of you, officer.
                    See you around.
                    * You continue down the road, leaving the wounded behind. 
            ]],
            OPT_PAY = "Pay the toll",
            OPT_ARGUE = "Convince {agent} to leave you alone",
            OPT_FIGHT = "Attack {agent}"
        }

        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                backup = CreateCombatBackup(cxt.quest:GetCastAgent("admiralty"),"ADMIRALTY_PATROL", cxt.quest:GetRank())
                cxt.encounter:SetPrimaryCast(cxt.quest:GetCastAgent("admiralty"))
                cxt:Dialog("DIALOG_INTRO", cxt.quest.param.total_amount_to_ask_for)
            end

            cxt:Opt("OPT_PAY")
                :Dialog("DIALOG_PAY")
                :DeliverMoney(cxt.quest.param.total_amount_to_ask_for, { no_scale = true })
                :Travel()

            cxt:Opt("OPT_ARGUE")
                :Dialog("DIALOG_ARGUE", cxt.quest.param.total_amount_to_ask_for)
                :Negotiation
                {
                    target_agent = cxt:GetCastAgent("admiralty"),
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_CONVINCED")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt.quest.param.total_amount_to_ask_for = cxt.quest.param.total_amount_to_ask_for + SPREE_FAIL_FEE
                        cxt:Dialog("DIALOG_NEGOTIATION_FAIL", cxt.quest.param.total_amount_to_ask_for)
                    end,
                } 

            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle
                {
                    on_win = function(cxt) 
                        if cxt:GetCastAgent("admiralty"):IsDead() then
                            local survivors = {}
                            for k,v in ipairs(backup) do
                                if not v:IsDead() then
                                    if v:IsSentient() then
                                        table.insert(survivors, v)
                                    end
                                end
                            end

                            if #survivors == 0 then
                                cxt:Dialog("DIALOG_EVERYONE_KILLED")
                            else
                                cxt:Dialog("DIALOG_LEADER_KILLED")
                            end
                        else
                            cxt:Dialog("DIALOG_BEAT")
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }           
        end)