local POSSIBLE_CONTRABAND_PLANTS = {
    "tincture",
    "vial_of_slurry",
}


local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    event_type = "faction_hassle",
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    is_negative = true,
    precondition = function(quest)
        return not Util.IsPlayerWorkingWithFaction("ADMIRALTY")
    end,
    process_fighter = function(quest, fighter)
        if fighter.agent == quest.param.fight_agent then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end

}

-- Find the Admiralty who likes you the least
local function FindAdmiralty(cxt)
    local candidates = ObtainWorkTable()

    for k,agent in pairs(TheGame:GetGameState().agents) do
        if (agent:GetContentID() == "ADMIRALTY_GOON" or agent:GetContentID() == "ADMIRALTY_GUARD")  and
            not agent:HasActiveQuestMembership() and
            not agent:GetBrain():IsOnDuty() and
            AgentUtil.CanAct(agent) then

            table.insert(candidates, {agent = agent, rel = agent:GetRelationship()})
        end
    end

    local agent
    if #candidates > 0 then
        local function SortByRel( a, b )
            return a.rel < b.rel
        end  
        table.sort(candidates, SortByRel)

        agent = candidates[1].agent
        if agent:GetParty() then
            agent:GetParty():Disband()
        end
    end

    ReleaseWorkTable(candidates)
    return agent
end

local function DoHassle(cxt)

    local opt = {}
    
    if cxt.caravan:GetMoney() >= 20 then
        table.insert(opt, "STATE_PLANT_CONTRABAND")
    end
    
    table.insert(opt, "STATE_WANT_TO_FIGHT")
    table.insert(opt, "STATE_FIGHT_SOMEONE_ELSE")

    if cxt.caravan:GetMoney() >= 100 then
        table.insert(opt, "STATE_DEMAND_MONEY")
    end

    local state = opt[math.random(#opt)]

    cxt:GoTo(state)
end

QDEF:AddConvo()

    :Loc{
        OPT_NEGOTIATE = "Convince {agent#agent} to back off",
        OPT_FIGHT = "Defend yourself",

        DIALOG_NEGOTIATE_WIN = [[
            player:
                !thumb
                I'm a hunter, officer. And I'm on the job. 
                If there's a toll I should be paying, I have to pass that expense on to my employers. 
                !crossed
                And they'll want a receipt—with your name and everything. 
            agent:
                !spit
                Tsk. It was just a bit of fun, hunter. No need to get nasty. 
                Be on your way, then. 
                * {agent#agent} steps to the side, refusing to meet your gaze, and allows you to pass. 
        ]],
        DIALOG_NEGOTIATE_FAIL = [[
            player:
                !angry_accuse
                I'm going to issue a complaint to your commanding officer!
            agent:
                !angry
                Oh, are you? How awful for me. 
                Until then, the toll just went up. It's {fine_amount#money}, now.
        ]],
        DIAL_WON_FIGHT_KILLED = [[
            * Violence doesn't solve every problem, but it certainly solved this one.
        ]],
        DIAL_WON_FIGHT= [[
            agent:
                !injured
            player:
                !fight
                Whaddya reckon, <i>officer</i>? Think you're convinced?
            agent:
                Back off, hunter! You made your point.
                Just go along your way.
                !exit
        ]],
    }
    :Confront(function(cxt, agent)
        if agent == cxt.quest:GetCastAgent("admiralty") then
            return "START"
        end
    end)


    ---takes a bunch of money that you might not have

    :State("STATE_DEMAND_MONEY")
        :Loc{
            DIALOG_DEMAND_MONEY = [[
                agent:
                    !overthere
                    This road is under renovation by the Admiralty. You want through? You'll need to pay the toll.
                    !cruel
                    Otherwise we got license to be authoritative.
            ]],


            DIALOG_TOOK_MONEY = [[
                player:
                    !give
                    Sure thing, officer. 
                agent:
                    !happy
                    !take
                    Why, thank you kindly, grifter.
                    !chuckle
                    Happy to see you obliging the law and doing your part to make Murder Bay a better place. 
                * Not even bothering to hedge {agent.hisher} snicker, {agent#agent} steps aside with a graceful sweep of {agent.hisher} arm.

            ]],
            OPT_HAND_OVER_MONEY = "Pay the toll",

            OPT_DONT_HAVE = "Tell {agent#agent} you can't afford to pay",
            DIALOG_DONT_HAVE = [[
                player:
                    !shrug
                    I haven't got money for a toll.
                agent:
                    !sigh
                    That's really too bad. 'Cause now I have to use my authority.
                * {agent#agent} brings {agent.hisher} baton down into {agent.hisher} open palm with an impressive smack. 
                agent:
                    !fight
                    This is gonna hurt, grifter.
            ]],
            DIAL_WON_FIGHT = [[
                player:
                    !fight
                    Whaddya reckon, <i>officer</i>? Think you're convinced?
                agent:
                    !injured
                    Back off, hunter! You made your point.
                    Just go along your way.
                    !exit
            ]]

        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt:Dialog("DIALOG_DEMAND_MONEY")
                cxt.quest:SetParameter("fine_amount", math.max( 100, math.ceil(cxt.caravan:GetMoney()/3) ))
            end
            

            cxt:Opt("OPT_HAND_OVER_MONEY")
                :DeliverMoney(cxt.quest:GetParameter("fine_amount"), {no_scale=true})
                :Dialog("DIALOG_TOOK_MONEY")
                
                :Fn(function(cxt) StateGraphUtil.AddLeaveLocation(cxt) end)
                
            
            if cxt.quest:GetParameter("fine_amount") > cxt.caravan:GetMoney() then
                cxt:Opt("OPT_DONT_HAVE")
                    :Dialog("DIALOG_DONT_HAVE")
            end

            cxt:Opt("OPT_NEGOTIATE")
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATE_WIN")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt.quest:SetParameter("fine_amount", math.floor( cxt.quest:GetParameter("fine_amount")*1.5))
                        cxt:Dialog("DIALOG_NEGOTIATE_FAIL")
                    end }

            cxt:Opt("OPT_FIGHT")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function(cxt) 
                        cxt:Dialog(cxt:GetAgent():IsDead() and "DIAL_WON_FIGHT_KILLED" or "DIAL_WON_FIGHT")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }

        end)

    :State("STATE_PLANT_CONTRABAND")
    :Loc{   
        DIALOG_INTRO = [[
            agent:
                !point
                You're under arrest for carrying contraband.
            player:
                !angry_shrug
                Wha—? Contraband? What contraband?!
            agent:
                !eureka
                Oh, right.
                !give
            * {agent#agent} fumbles around in {agent.hisher} pockets for a second, produces a {contraband#card}, and hands it to you.
        ]],

        DIALOG_INTRO2 = [[
            agent:
                There.
                !angry_accuse
                As I was saying, you're under arrest for carrying contraband.
                !cruel
                The fine is {fine_amount#money}.
                * {agent#agent} looks at you with smug satisfaction.

                *** {agent} is demanding a "fine" from you.
        ]],
        OPT_PAY_FINE = "Pay the ridiculous 'fine'",
        OPT_PAY_FINE_SMALL = "Pay a slightly less ridiculous 'fine'",
        DIALOG_TOOK_MONEY_LARGE = [[
            player:
                !disappoint
                $angryDefault
                Ugh. Fine. 
                !give
                $neutralResigned
                I don't want any trouble. Just take it.
            agent:
                !surprised
                Wha-<i>really</i>?
                !angry_accuse
                I mean... don't let me catch you doing it again.
        ]],
        DIALOG_TOOK_MONEY_SMALL = [[
            player:
                !facepalm
                Geez. I admire your hustle, at least.
                !give
                Consider this a tip for the entertainment, at least.
            agent:
                !take
                $happyDefault
                Well, Hesh. Didn't think I'd get away with this at all, really.
                !chuckle
                For being such a good sport, I'll even let you keep the {contraband#card}!
        ]],
        OPT_NEGOTIATE = "Negotiate a smaller 'fine'",
        DIALOG_NEGOTIATE_WIN = [[
            player:
                Really? <i>Really</i>?
            agent:
                !hips
                Alright. Since this is your first offense, I can lower the fine. <b>Just for you</>.
                It's {fine_amount#money}, now.
        ]],
        OPT_NEGOTIATE_TT = "Negotiate the fine to be {1#money}",
        DIALOG_NEGOTIATE_WIN_NO_MONEY = [[
            player:
                Really? <i>Really</i>?
            agent:
                !hips
                Alright. Since this is your first offense, I'll let you go. <b>Just this once</>.
        ]],
        OPT_FIGHT = "Refuse and defend yourself",
        DIAL_FIGHT = [[
            player:
                !fight
                I don't respect your authority!
        ]],
        DIAL_WON_FIGHT= [[
            player:
                I'm keeping the {contraband#card}. And you're not getting a shill. 
            agent:
                !injured
                F-fair enough, I guess. But hey! You can't blame me for trying.
            player:
                I can. And I do. 
            * You shoulder your way past {agent#agent}, leaving {agent.himher} to tend to {agent.hisher} wounds.
                !exit
        ]]
    }
    :SetLooping(true)
    :Fn(function(cxt) 
            if cxt:FirstLoop() then
                
                local card_id = table.arraypick( POSSIBLE_CONTRABAND_PLANTS )
                cxt.quest:SetParameter("contraband", card_id )

                cxt.quest:SetParameter("fine_amount", 250)
                cxt:Dialog("DIALOG_INTRO", card_id)

                AgentUtil.LearnCard( card_id )

                cxt:Dialog("DIALOG_INTRO2")
            end

            cxt:Opt(cxt.quest:GetParameter("negotiated") and "OPT_PAY_FINE_SMALL" or "OPT_PAY_FINE")
                :DeliverMoney(cxt.quest:GetParameter("fine_amount"), {no_scale=true})
                :Dialog(cxt.quest:GetParameter("negotiated") and "DIALOG_TOOK_MONEY_SMALL" or "DIALOG_TOOK_MONEY_LARGE")
                :Fn(function(cxt) StateGraphUtil.AddLeaveLocation(cxt) end)

            local small_fine = math.min(50, math.floor(cxt.caravan:GetMoney()))
            if not cxt.quest:GetParameter("negotiated") then
                cxt:Opt("OPT_NEGOTIATE")
                    :PostText("OPT_NEGOTIATE_TT", small_fine)
                    :Negotiation{
                        on_success = function(cxt)
                            cxt.quest:SetParameter("negotiated", true)
                            if cxt.caravan:GetMoney() == 0 then
                                cxt:Dialog("DIALOG_NEGOTIATE_WIN_NO_MONEY")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            else
                                cxt.quest:SetParameter("fine_amount", small_fine)
                                cxt:Dialog("DIALOG_NEGOTIATE_WIN")
                            end
                        end,
                        on_fail = function(cxt)
                            cxt.quest:SetParameter("fine_amount", math.floor( cxt.quest:GetParameter("fine_amount")*1.5))
                            cxt:Dialog("DIALOG_NEGOTIATE_FAIL")
                        end }
            end

            cxt:Opt("OPT_FIGHT")
                :Dialog("DIAL_FIGHT")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function(cxt) 
                        cxt:Dialog(cxt:GetAgent():IsDead() and "DIAL_WON_FIGHT_KILLED" or "DIAL_WON_FIGHT")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }
        end)

    --fights you, or someone else.
    :State("STATE_WANT_TO_FIGHT")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !angry_shrug
                    Last review my supervisor said my technique was rusty. Rusty! Can you believe it?
                    He assigns me to the boondocks every damn day for years, then says I can't get a promotion 'cause I can't fight as well as an officer.
                player:
                    !scared_shrug
                    $neutralDubious
                    That's, uh, rough?
                agent:
                    !neutral
                    !sigh
                    Yeah! Thanks for saying so!
                player:
                    !neutral
                agent:
                    !agree
                    And for agreeing to help me out.
                player:
                    I... didn't say—
                * {agent#agent} draws {agent.hisher} weapon with a purposeful glint in {agent.hisher} eye.
                player:
                    !dubious
                    Oh. 
                    !facepalm
                    Aw, Hesh.
            ]],
            OPT_RECEIVE_BEATING = "Take a beating",
            OPT_SUGGEST_OTHER_TARGET = "Suggest another target...",
            OPT_SUGGEST_AGENT = "Suggest {1#agent}",
            DIALOG_NEGOTIATE_FAIL = [[
                player:
                    !scared_shrug
                    $neutralResigned
                    Can't you just... fight someone else?
                agent:
                    !angry_shrug
                    Ain't nobody worth fighting on this road! It's career sabotage is what it is.

            ]],
            DIALOG_BEATING_PRE = [[
                agent:
                    !fight
                    I'm not gonna kill you! I promise. 
                player:
                    !fight
                    Fine! But pay attention so you might actually learn something.
                    !wait
            ]],

            DIALOG_BEATING_POST= [[
                player:
                    !injured
                * {agent} beats you soundly, but stops short of actually killing you.
                agent:
                    !tired
                    Hah! Ha HA! If only my boss were here to see <i>that</i>.
                    !happy
                    Thanks for your help, hunter. Next time I'm back at the barracks, I'll challenge him to a duel.
                    !chuckle
                    Then we'll see who's "officer material".
            ]],
            DIALOG_SUGGEST_OTHER = [[
                player:
                    !overthere
                    I'm just some shmuck. Now {agent#1} over there—<i>that's</i> someone worth cutting your teeth on.
                agent:
                    !thought
                    Hmmmm. Yeah, that's a good point.
                    !exit
                * {agent#agent} turns on {agent#1}, and wastes no time throwing the first punch.
                * The brawl is brief but intense, and {agent#1} knows you're to blame.
            ]],
            DIALOG_PST_FIGHT = [[
                * You walk it off. 
            ]]
        }
        :SetLooping(true)
        :Fn(function(cxt)   
                if cxt:FirstLoop() then
                    cxt:Dialog("DIALOG_INTRO")
                end
                local health, max = cxt.player:GetHealth()   


                cxt:Opt("OPT_RECEIVE_BEATING")
                    :Dialog("DIALOG_BEATING_PRE")
                    :DeltaHealth(-math.round(health/2))
                    :Dialog("DIALOG_BEATING_POST")
                    :Travel()

                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt) 
                            cxt:Dialog("DIALOG_PST_FIGHT")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }

        end)

    :State("STATE_FIGHT_SOMEONE_ELSE")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !eureka
                    Hey, hunter. You've come along at the right time.
                    !overthere
                    I've got a warrant to take down {fight_agent#agent} over there but, uh, I don't much fancy doing it on my own.
                    You want through this checkpoint? Then do me a solid, yeah?
                    !agree
                    Just be sure to take 'em alive.
                player:
                    !dubious
                    Uh, am I getting paid for this or is this a Murder Bay special: you hold all the power, and I dance.
                agent:
                    !placate
                    $miscEmbarrassed
                    H-hey! This is a <i>business opportunity</i>. I'll tell all my friends you do good work.
                player:
                    !crossed
                    Oh, well in <i>that</i> case.
                agent:
                    !angry_accuse
                    Just get over there, already, smart-ass. Or I'll write you up for disorderly conduct.
            ]],
            OPT_FIGHT_TARGET = "Attack {fight_agent#agent} for {agent#agent}",
            OPT_FIGHT = "Attack {agent#agent}",
            DIALOG_KILLED_TARGET= [[
                agent:
                    !right
                    !facepalm
                    What did I say? I said <i>don't kill 'em</i>.
                player:
                    !angry_shrug
                    You get what you pay for.
                agent:
                    !angry_threaten
                    Yeah? Well <i>you'll</i> be the one paying.
                    !fight
            ]],
            DIALOG_DIDNT_KILL_TARGET = [[
                agent:
                    !right
                    !happy
                    Hey, wow, that's great! 
                player:
                    !angry_shrug
                    Yep. 
                agent:  
                    !handwring
                    This is great. I'm gonna get so many compliments on this arrest.
                    !exit
                * {agent#agent} marches {fight_agent#agent} away without even a backwards glance.
                player:
                    I'm reeaaal happy for you.
                * It's easy to talk back when there's no one around. 
            ]],
            DIALOG_THANKS = [[
                player:
                    !tired
                    Right, the job's done.
                agent:
                    !right
                    !agree
                    Thanks for that. I saw that fight go down—I wouldn't have made it far compared to you.
                    !permit
                    I know I forced your hand, but I owe you one. For real. 
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :SetLooping(true)
        :Fn(function(cxt)
            
            
            if cxt:FirstLoop() then
                cxt.quest.param.fight_agent = TheGame:GetGameState():AddSkinnedAgent("LABORER")
                cxt:ReassignCastMember( "fight_agent", cxt.quest.param.fight_agent)
                cxt.quest.param.fight_agent:MoveToLocation(cxt.location)
                cxt:Dialog("DIALOG_INTRO")
            end


            cxt:Opt("OPT_FIGHT_TARGET")
                :Battle{
                    --flags = BATTLE_FLAGS.SELF_DEFENCE,
                    
                    noncombatants = {cxt:GetAgent()},
                    enemies = {cxt.quest:GetParameter("fight_agent")},
                    
                    on_start_battle = function(battle)
                        local fighter = battle:GetFighterForAgent(cxt.quest.param.fight_agent)
                        if fighter then
                            fighter:AddCondition("WANTED_ALIVE")
                        end
                    end,
                
                    on_win = function(cxt) 
                        
                        if cxt.quest:GetParameter("fight_agent"):IsDead() then
                            cxt:Dialog("DIALOG_KILLED_TARGET")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                    --flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function(cxt) 
                                        cxt:Dialog(cxt:GetAgent():IsDead() and "DIAL_WON_FIGHT_KILLED" or "DIAL_WON_FIGHT")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }
                        else
                            cxt:Dialog("DIALOG_DIDNT_KILL_TARGET")

                            cxt.quest:GetParameter("fight_agent"):MoveToLimbo()
                            cxt:GetAgent():MoveToLimbo()
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                        
                    end,
                }

            cxt:Opt("OPT_FIGHT")
                :Battle{
                    allies = {cxt.quest:GetParameter("fight_agent")},
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function(cxt) 
                        cxt:Dialog(cxt:GetAgent():IsDead() and "DIAL_WON_FIGHT_KILLED" or "DIAL_WON_FIGHT")
                        
                        if not cxt.quest:GetParameter("fight_agent"):IsDead() then
                            cxt.enc:SetPrimaryCast( cxt.quest:GetParameter("fight_agent"))
                            cxt:Dialog("DIALOG_THANKS", cxt.quest:GetParameter("fight_agent"))
                            cxt.quest:GetParameter("fight_agent"):OpinionEvent(OPINION.DEFENDED)
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }
        end)

    :State("START")
        :Quips{
            {
                tags = "get_over_here",
                "You! Grifter! Get over here.",
                "Well, look what we have here.",
                "Ah, here comes trouble.",
                "They said we could expect trouble on this road.",
                "Looks like the brass was right."
            },
        }
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !right
                player:
                    !left
                * You're stopped along the road by an Admiralty patrol, swinging their batons with some showmanship. 
            ]],
            DIALOG_INTRO_2 = [[
                agent:
                    %get_over_here
                
                {disliked?
                    * {agent} is no friend of yours. Something tells you this is not going to go your way.
                }
                {not disliked?
                    * There's a pompous curl to {agent.hisher} lip that suggests you're in for a heap of trouble. Best watch yourself.
                }
            ]],

        }

        :Fn(function(cxt)
            cxt.quest:Complete()
            local leader = FindAdmiralty(cxt)
            local rank = math.round( TheGame:GetGameState():GetCurrentBaseDifficulty())
            local party
            if leader then
                party = CreateCombatBackup(leader, "ADMIRALTY_PATROL_BACKUP", rank)
            else
                local agents
                agents, leader = CreateCombatParty( "ADMIRALTY_PATROL", rank, cxt.location )
            end

            cxt.encounter:SetPrimaryCast( leader )

            cxt:Dialog("DIALOG_INTRO")
            cxt:Dialog("DIALOG_INTRO_2")
            DoHassle(cxt)
        end)

