local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    is_negative = true,
    cooldown = EVENT_COOLDOWN.SHORT,
    precondition = function(quest)
        local location = TheGame:GetGameState():GetPlayerAgent():GetLocation()
        if TheGame:GetGameState():GetDayPhase() ~= DAY_PHASE.DAY and TheGame:GetGameState():GetCaravan():GetMoney() >= 100 then
            return true
        end
    end,
}

:AddOpinionEvents{
    pulled_rank = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Threatened to rat them out to Nadan",
    },
    pulled_rank_ad = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Threatened to bring the law down on them",
    },
}


--------------------------------------------------------------------

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_ACCOST"
    end)

    :State("STATE_ACCOST")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * A bandit emerges from the shadows like a worm slithering out of the tar.
                agent:
                    !right
            ]]
        }
        :Fn(function(cxt) 
                cxt.quest:Complete()
                local rank = TheGame:GetGameState():GetCurrentBaseDifficulty()
                local leader = AgentUtil.GetFreeAgent("BANDIT_GOON", function( agent ) return agent:GetRelationship() < RELATIONSHIP.LIKED end)
                leader:MoveToLocation( cxt.player:GetLocation() )
                local extras = CreateCombatBackup(leader, "BANDIT_PATROL_BACKUP", rank)
                cxt.encounter:SetPrimaryCast( leader )
                cxt:Dialog("DIALOG_INTRO")
                cxt:GoTo("STATE_ROB")
            end)


    :State("STATE_ROB")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !fight
                    Your shills or my shank. You make the call.
            ]],
            OPT_HAND_OVER_MONEY = "Hand over <i>ALL</> of your shills",
            OPT_CONVINCE_LESS = "Convince them to take only half of your shills",
            TT_LESS_AMOUNT = "Give {1#money}",
            OPT_CONVINCE_NONE = "Convince {agent} to leave you alone",
            OPT_FIGHT = "Defend yourself",
            
            DIALOG_CONVINCE_LESS = [[
                player:
                    !shrug
                    You have to leave me with something!
            ]],
            DIALOG_CONVINCE_NONE = [[
                player:
                    !crossed
                    It's dark, so maybe you didn't notice exactly who you decided to rob, pal.
            ]],
            DIALOG_GIVE_ALL = [[
                * You hand over all of your money.
                agent:
                    That's what I thought.
                    !exit
                * The bandit slips back into the night, leaving your pockets much emptier.
            ]],
            DIALOG_GIVE_LESS = [[
                player:
                    I gotta get by, same as you. Leave me alive, and I might survive long enough for you to rob me twice.
                agent:
                    That's... a weirdly compelling take. 
                    Fine. Give me half. 
                * You count out half your shills, and while {agent.hisher} eyes lap at the rest in your hand, {agent.heshe} keeps {agent.hisher} word, and leaves it to you.
                    !exit
            ]],
            DIALOG_GIVE_NONE = [[
                player: 
                    !angry
                    !thumb
                    I'm Sal ik-Derrick. And I'm a hunter. 
                    $angryHostile
                    Crooks like you? Your bounties are my pocket change. 
                agent:
                    !scared
                    $scaredDefault
                    W-wait. You're the one hunting Kashio, aren't you?
                player:
                    !cruel
                    My my. You got good ears.
                    !throatcut
                    It'd be a shame if I had to slice 'em off.
                agent:
                    !placate
                    Hey, point made, sister. I'm not fool enough to get mixed up in this.
                    Live your best life. It won't last long, but I won't be the one to end it.
                * You leave before the bandit can change {agent.hisher} mind.
            ]],
            DIALOG_WON_FIGHT = [[
                agent:
                    !exit
                * You leave before the trouble multiplies.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",

            OPT_TALK_TO_MANAGER = "Threaten to talk to Nadan",
            DIALOG_TALK_TO_MANAGER = [[
                player:
                    You can't rob me, you dolt.
                    !point
                    I'm working for Nadan.

                agent:
                    Yeah? 
                    Just 'cause you're doing a little side hustle don't make you Spree.
                player:
                    !crossed
                    I wonder what Nadan would say about that.
                agent:
                    Are you threatening me?
                player:
                    No. I'm explaining something to you.
                    There's a difference.
                agent:
                    !overthere
                    Fine. You can leave.
                    Just... don't tell Nadan about this, OK?
            ]],            

            OPT_TALK_TO_MANAGER_AD = "Threaten them with legal consequences",
            TT_TALK_TO_MANAGER_AD = "You're working for the Admiralty!",
            
            DIALOG_TALK_TO_MANAGER_AD = [[
                player:
                    You idiot. I'm working for the Admiralty.
                agent:
                    You don't look like a switch. Where's your uniform.
                player:
                    !crossed
                    I'm working for Oolo herself. Special assignment. I'm so special I don't have to wear a uniform.
                    Are you <i>sure</> you want to rob me?
                agent:
                    ...
                    You know what, you can keep the money.
                    But when your contract is over?
                    !throatcut
                player:
                    Right. I'll make a note in my schedule.
                    !exit
                agent:
                    Yeah! You better!
            ]],  
            SIT_MOD = "Too greedy",

        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt:Dialog("DIALOG_INTRO")
            end

            local FULL_AMOUNT = cxt.caravan:GetMoney()
            local LESS_AMOUNT = math.ceil( FULL_AMOUNT / 2 )

            if Util.IsPlayerWorkingWithFaction("BANDITS") then
                cxt:Opt("OPT_TALK_TO_MANAGER")
                    :Dialog("DIALOG_TALK_TO_MANAGER")
                    :ReceiveOpinion("pulled_rank")
                    :Travel()
            end

            if Util.IsPlayerWorkingWithFaction("ADMIRALTY") then
                cxt:Opt("OPT_TALK_TO_MANAGER_AD")
                    :PostText( "TT_TALK_TO_MANAGER_AD")
                    :Dialog("DIALOG_TALK_TO_MANAGER_AD")
                    :ReceiveOpinion("pulled_rank_ad")
                    :Travel()
            end

            cxt:Opt("OPT_HAND_OVER_MONEY")
                :Dialog("DIALOG_GIVE_ALL")
                :DeliverMoney( FULL_AMOUNT, {no_scale=true})
                :Travel()

            cxt:Opt("OPT_CONVINCE_LESS")
                :PostText("TT_LESS_AMOUNT", LESS_AMOUNT)
                :Dialog("DIALOG_CONVINCE_LESS")
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_GIVE_LESS") 
                        cxt.caravan:PayMoney( LESS_AMOUNT, cxt:GetAgent(), true )
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    
                    on_fail = function(cxt)
                        cxt:Opt("OPT_DEFEND_SELF")
                            :Battle{ 
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                advantage = TEAM.RED,
                                on_win = function(cxt)
                                    cxt:Dialog("DIALOG_WON_FIGHT")
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end,
                            }
                    end,
                }

            cxt:Opt("OPT_CONVINCE_NONE")
                :Dialog("DIALOG_CONVINCE_NONE")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.INTIMIDATION,
                    situation_modifiers =
                    {
                        { value = 10, text = cxt:GetLocString("SIT_MOD") }
                    },
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_GIVE_NONE")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    
                    on_fail = function(cxt)
                        cxt:Opt("OPT_DEFEND_SELF")
                            :Battle{ 
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                advantage = TEAM.RED,
                                on_win = function(cxt) 
                                    cxt:Dialog("DIALOG_WON_FIGHT")
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end,
                            }
                    end,
                }

            cxt:Opt("OPT_FIGHT")
                :Battle{ 
                    advantage = TEAM.BLUE, 
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function()
                        cxt:Dialog("DIALOG_WON_FIGHT")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    }
        end)
