local AMT_PER_LEVEL = 30


local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    event_type = "faction_hassle",
    is_negative = true,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest)
        return not Util.IsPlayerWorkingWithFaction("BANDITS")
    end,
}


:AddCast{
    cast_id = "bandit",
    condition = function( agent, quest )
        if agent:GetFactionID() == "BANDITS" and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) and agent:GetRelationship() < RELATIONSHIP.NEUTRAL and not agent:HasAspect("intimidated") then
            return true
        end
    end,        
}


QDEF:AddConvo( )
    :Quips{
        {
            tags = "get_over_here",
            "I smell money! Hand it over.",
            "This is a robbery! Pay up, or bleed out.",
            "Your pockets look heavy, friend. Let me lighten your load.",
            "You know what to do: cough up the shills."
        },
    }
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !right
            player:
                !left
            * A Spree bandit steps out onto the path, arms at the ready.
        ]],
        DIALOG_INTRO_2 = [[
            agent:
                %get_over_here
        ]],

        DIALOG_TOOK_MONEY = [[
            player:
                !give
                $miscDepressed
                Fine. Take it. I had too much, anyway.
            agent:
                !happy
                !take
                Hah, easy money! The best kind. 
            * {agent} slinks off with a bounce in {agent.hisher} heels. 
        ]],
        DIALOG_NEGOTIATE_WIN = [[
            player:
                !overthere
                $miscMischievous
                I mean, have you seen my knife? I can <i>show</i> you my knife, if that's what you want.
            agent:
                !placate
                $scaredStammering
                Uh, naw, that's all right. I've seen plenty of knives. I'll look for trouble somewhere else. 
            * {agent} slinks off, defeated. 
        ]],
        DIALOG_NEGOTIATE_FAIL = [[
            player:
                !thumb
                $miscPersuasive
                I may not look like much, but I swear I'm very deadly.
            agent:
                !chuckle
                $miscMocking
                Is that the best you got? You could've given me all your money in that time. 
                !point
                Now I'm just going to take it off your corpse instead.
        ]],
               
        OPT_HAND_OVER_MONEY = "Hand over the money",
        OPT_NEGOTIATE = "Convince {agent#agent} to back off",
        DIALOG_NEGOTIATE = [[
            player:
                !dubious
                $neutralDubious
                I'm a hunter. This can't be worth the risk for you. 
        ]],        
        OPT_FIGHT = "Defend yourself",
        DIALOG_FIGHT = [[
            player:
                !fight
                Not today, bandit.
        ]],

        DIALOG_FOUGHT_KILLED = [[
            player:
                !left
            * You wipe the bandit blood off your knife.              
        ]],
        DIALOG_FOUGHT_ALIVE = [[
            player:
                !left
            agent:
                !scared
                I was just trying to make a living!
            player:
                !angry
                Yeah? Well so was I. Now get lost!
        ]],
        OPT_DEFEND_SELF = "Defend yourself",
    }

    :Confront(function(cxt, agent)
        return "START"
    end)

    :State("START")
        :SetLooping(true)
        :Fn(function(cxt)

            if cxt:FirstLoop() then
                cxt.quest:GetCastMember("bandit"):MoveToLocation(cxt.location)
                
                cxt.quest:Complete()
                local rank = math.max(TheGame:GetGameState():GetCurrentBaseDifficulty() - 1, 1)
                local party = CreateCombatBackup(cxt.quest:GetCastMember("bandit"), "BANDIT_PATROL_BACKUP", rank)
                cxt.quest:GetCastMember("bandit"):Remember("BANDIT_THREATEN_PLAYER")
                cxt.encounter:SetPrimaryCast( cxt.quest:GetCastMember("bandit") )
                cxt:Dialog("DIALOG_INTRO")
                cxt:Dialog("DIALOG_INTRO_2")
            end

            local to_steal = TheGame:GetGameState():GetCurrentBaseDifficulty()*AMT_PER_LEVEL
            
            cxt:Opt("OPT_HAND_OVER_MONEY")
                :DeliverMoney(to_steal, {no_scale=true})
                :Dialog("DIALOG_TOOK_MONEY")
                :Travel()
            
            cxt:Opt("OPT_NEGOTIATE")
                :Dialog("DIALOG_NEGOTIATE")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.INTIMIDATION,
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATE_WIN")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATE_FAIL")
                        cxt:Opt("OPT_DEFEND_SELF")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                on_win = function(cxt) 
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end,
                            }
                    end }

            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function(cxt) 
                        cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_FOUGHT_KILLED" or "DIALOG_FOUGHT_ALIVE")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,

                }
        end)

