local HIRE_AMOUNT = 50

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,

    on_init = function(quest)
    end,

    precondition = function(quest)
        return TheGame:GetGameState():GetCaravan():HasHireling() == false and 
            TheGame:GetGameState():GetCaravan():GetMoney() >= HIRE_AMOUNT and
            TheGame:GetGameState():GetDayPhase() ~= DAY_PHASE.NIGHT and
            TheGame:GetGameState():IsHiringAvailable()

    end
}

QDEF:AddCastByAlias{
    cast_id = "stranger",
    alias = "BRAVE_MERCHANT",
    condition = function(agent, quest)
         return not agent:KnowsPlayer()
    end
}

QDEF:AddOpinionEvents{
    hired_brave_merchant =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Hired them instead of beating them up",
    },
    spared_life =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Spared their life when they tried to rob you",
    },
}

QDEF:AddConvo()
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("FOO", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !angry

                
                * Someone dressed an awful lot like a shopkeeper stops you in your tracks. 
                * {agent.HeShe} pulls a knife out of the front pocket of {agent.hisher} work apron and waves it like a bundle of burning sage. 
                
                agent:
                    !fight
                    $angryPatienceLost
                    All right, sludgesucker. Hand over all you got. 
                    !angry_accuse
                    Don't make me use this!
                * You're not even sure {agent.heshe} knows how. 
                *** {agent} is demanding all of your money.
            ]],

            OPT_ASK_ABOUT_SELF = "Ask about {agent}",
            DIALOG_ASK_ABOUT_SELF = [[
                player:
                    !dubious
                    Is this a hold up?
                agent:
                    Yes.
                player:
                    $neutralDubious
                    Really?
                agent:
                    !overthere
                    I got a knife, don't I? We're alone in the wilderness, ain't we?
                    What's so hard to get?
                player:
                    !permit
                    Well, it's just that, I'm a well-equipped bounty hunter, and you're...
                agent:
                    !cruel
                    I'm your worst nightmare! Don't doubt that for a second!
                player:
                    Yeah, sure.
                *** {agent} is extremely confident in {agent.hisher} martial abilities. You're not sure why.
            ]],

            OPT_HAND_OVER_MONEY = "Hand over all of your money",
            DIALOG_HAND_OVER = [[
                player:
                    $neutralResigned
                    Well, who am I to argue. Take it.
                    !give
                agent:
                    !take
                    R-really? 
                    !happy
                    I mean—yeah! That's right! Now go warn all your friends. 
                    !eureka
                    There's a new threat walking the streets of Murder Bay.
                player:
                    I'll do that.
                agent:
                    !exit
                * {agent} marches off in search of another target.
                *** You gave {agent} all of your money.
            ]],
            
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Just remember you brought this on yourself.
                agent:
                    Only thing getting brought... is pain!
                    !threaten
                    By me! To you!
            ]],
            DIALOG_FOUGHT_KILLED = [[
                agent:
                    !exit
                * {agent} is dead, but you're not quite sure how this could've gone any differently.
            ]],
            DIALOG_FOUGHT_LIVED = [[
                agent:
                    !injured
                player:
                    !placate
                    Come on, now. This has gone on long enough. 
                    $neutralResigned
                    What's your deal, anyway?
                agent:
                    $scaredDefault
                    I-I'm so deep in debt I dunno if I'll ever see the sky. 
                    My debt broker's threatening to sell me to the derrick. 
                    Only way to survive here in the Bay is to swim with the sharks. 
                player:
                    !point
                    You wanna swim with sharks, you need to be smarter. And maybe get a friend to watch your back, huh?
                    Go on. Get outta here.
                agent:
                    And go where?
                    !exit          
                * {agent} doesn't seem to expect an answer, as {agent.heshe} hobbles off into the wilderness.
                *** You taught {agent} a lesson, but you're pretty sure you're going to see {agent.himher} again.
            ]],
            OPT_OFFER_A_JOB = "Offer {agent} a job...",
            DIALOG_OFFER_A_JOB = [[
                player:
                    !interest
                    Listen, uh—what's your name again?
                agent:
                    {agent}.
                    !palm
                    $angryDefault
                    Ah, suckerspawn! I shouldn't'a told you that.
                player:
                    Right, well. You did. So... I don't want to fight you. 
                    !shrug
                    But how about a job? I could pay you for... protection?
                agent:
                    !chuckle
                    Protection, eh? I don't blame you. It's a scary world out there.
                player:
                    $neutralJoke
                    It can be, yes.
                agent:
                    I won't do it for less than {1#money}.
            ]],
            OPT_HIRE = "Hire {agent}",

            DIALOG_HIRE = [[
                player:
                    You drive a hard bargain, but here you go.
                    !give
                agent:
                    !take
                    Wow! This is great!
                    Never been hired muscle before.
                player:
                    !neutralJoke
                    I'm sure you'll do swimmingly.
                    !overthere
                    After you. 
                agent:
                    !happy
                * {agent} swishes {agent.hisher} knife through the air, cutting down some conjured-up ruffian. 
                *** You hired {agent} as a bodyguard.
            ]],
            OPT_REFUSE_HIRE = "Refuse the offer",
            DIALOG_REFUSE_HIRE = [[
                player:
                    !shrug
                    That's a lot for someone who shows up with no references.
                agent:
                    !threaten
                    Yeah?! Then fight me! And you'll see how strong I really am.
                player:
                    !palm
                    Ugh.
            ]],

            OPT_INTIMIDATE = "Scare {agent.himher} away",
            DIALOG_INTIMIDATE = [[
                player:
                    !interest
                    Take a second to really let this whole situation sink in, huh?
            ]],
            DIALOG_INTIMIDATE_FAIL = [[
                player:
                    Do you even know how to use that blade?
                agent:
                    !fight
                    Sharp end goes in your direction. I've got it figured out.
            ]],
            DIALOG_INTIMIDATE_SUCCESS = [[
                player:
                    I'm a <i>bounty hunter</i>. 
                agent:
                    So? I ain't got no bounty on my head! 
                player:
                    Yeah, because you're <i>not dangerous</i>.
                agent:
                    ...
                player:
                    ...
                agent:
                    ... huh?
                player:
                    !angry_threaten
                    Just get out of here before I turn your insides out and stuff you in a chum bucket!
                agent:
                    !placate
                    Okay, okay! 
                    Sheesh. Nobody ever said muggery'd be this hard.
                * {agent} shrugs off your anger and turns for the road, muttering.
            ]],
            SIT_MOD = "{agent} is <i>done</>.",

        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:GetCastMember("stranger"):MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("stranger"))
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")  
            end
            
            cxt:Opt("OPT_ASK_ABOUT_SELF")
                :Dialog("DIALOG_ASK_ABOUT_SELF")

            cxt:Opt("OPT_HAND_OVER_MONEY", cxt.caravan:GetMoney())
                :DeliverMoney(cxt.caravan:GetMoney(), {no_scale = true})
                :Dialog("DIALOG_HAND_OVER", cxt.caravan:GetMoney())
                :Travel()


            cxt:Opt("OPT_OFFER_A_JOB")
                :Dialog("DIALOG_OFFER_A_JOB", HIRE_AMOUNT)
                :IsHubOption(true)
                :PreIcon( global_images.hire )
                :Fn(function(cxt) 
                    cxt:Opt("OPT_HIRE")
                        :Dialog("DIALOG_HIRE")
                        :DeliverMoney(HIRE_AMOUNT, {no_scale = true})
                        :Fn(function(cxt) 
                            cxt.quest:OpinionEvent("stranger", "hired_brave_merchant")
                            cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.HIRED )
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)

                    cxt:Opt("OPT_REFUSE_HIRE")
                        :Dialog("DIALOG_REFUSE_HIRE")
                        :PreIcon( global_images.back )

                    end)
                

            cxt:Opt("OPT_INTIMIDATE")
                :Dialog("DIALOG_INTIMIDATE")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.INTIMIDATION,
                    situation_modifiers =
                    {
                        { value = 10, text = cxt:GetLocString("SIT_MOD") }
                    },

                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_INTIMIDATE_SUCCESS")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt) 
                        cxt:Dialog("DIALOG_INTIMIDATE_FAIL")
                    end,
                }
            
            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function(cxt) 
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_FOUGHT_KILLED")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        else
                            cxt:Dialog("DIALOG_FOUGHT_LIVED")
                            cxt.quest:OpinionEvent("stranger", "spared_life")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end

                    end,
                }
        end)
