local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest)
        local memory = TheGame:GetGameState():GetPlayerAgent():HasMemory("DANGEROUS_GOODS_SELLER")
        return memory
    end,
    
    on_init = function(quest)
        local memory = TheGame:GetGameState():GetPlayerAgent():HasMemory("DANGEROUS_GOODS_SELLER")
        TheGame:GetGameState():GetPlayerAgent():Forget("DANGEROUS_GOODS_SELLER")
        quest.param.cost = memory.cost
        quest.param.item_id = memory.item_id
    end,
}

QDEF:AddCastByAlias{
    cast_id = "stranger",
    alias = "DANGEROUS_STRANGER",
}

QDEF:AddOpinionEvents{
    gave_back_item =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Gave them back an item that they sold to you",
    },
}


QDEF:AddConvo()
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("FOO", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                * {agent} approaches you.
                player:
                    !left
                agent:
                    !right
                    Grifter! I've finally found you!
                    I need that {1#card} back.
                player:
                    The thing you sold to me?
                agent:
                    Yes. There was a... misunderstanding. 
                    It was not mine to sell.
                    You will be reimbursed.
            ]],

            OPT_GIVE_IT_BACK = "Give the {1#card} back",
            DIALOG_GIVE_IT_BACK = [[
                player:
                    Sure, you can have it back.
                    !give
                agent:
                    !take
                    Thank you very much, grifter.
                    !happy
                    I appreciate this, truly.
                    !exit
            ]],


            OPT_DONT_HAVE_IT_ANY_MORE = "Tell {agent} you don't have it",
            DIALOG_DO_NOT_HAVE_IT = [[
                player:
                    I don't have it any more.
                agent:
                    You don't?
                    That is... unfortunate.
                    Good bye, Grifter.
                    !exit
                player:
                    Well what the Hesh was that all about?
            ]],


            OPT_EXTORT = "Extort {agent} for {1#money}",

            DIALOG_EXTORT = [[
                player:
                    I think I deserve consideration for my time. 
                    Give me {1#money} for it, and we have a deal.
            ]],

            DIALOG_EXTORT_FAIL = [[
                agent:
                    Your greed is testing my patience.
                    Please do not test my patience further.
            ]],

            DIALOG_EXTORT_SUCCESS = [[
                agent:
                    That is an acceptable sum.
                player:
                    Here you go.
                    !give
                agent:
                    !take
                    Good bye, grifter.
                    !exit
                * {agent} leaves quickly.
            ]],

            OPT_REFUSE = "Refuse",
            DIALOG_REFUSE = [[
                player:
                    No. You sold it to me, now it's mine.
                agent:
                    I'm afraid I have to insist.
                    Hand it over. Or die.
            ]],
            OPT_HAND_OVER = "Hand it over",
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Your customer service is <b>terrible!</>
            ]],
            DIALOG_HAND_OVER = [[
                player:
                    Fine, if it means that much to you.
                agent:
                    It does. 
                    Here is your payment.
                    !give
                    Goodbye.
                    !exit
            ]],
            DIALOG_WON_FIGHT_DEAD = [[
                agent:
                    !exit
                * {agent} is dead. You wonder why they chose to die for so little.
            ]],

            DIALOG_WON_FIGHT_ALIVE = [[
                agent:
                    !injured
                player:
                    !angry
                    Now scram! Don't make me hurt you more!
                agent:
                    You will live to regret this, grifter.
                    !exit
            ]],
        }

        :SetLooping(true)
        :Fn(function(cxt)
                
                if cxt:FirstLoop() then
                    cxt.quest:GetCastMember("stranger"):MoveToLocation(cxt.location)
                    cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("stranger"))
                    cxt.quest:Complete()
                    cxt:Dialog("DIALOG_INTRO", cxt.quest.param.item_id)
                end

                local has_item = cxt.player.battler:HasCardID(cxt.quest.param.item_id)
                local extort_amt = cxt.quest.param.cost*5

                if has_item then
                    
                    local opt = cxt:Opt("OPT_GIVE_IT_BACK", cxt.quest.param.item_id)
                        :Dialog("DIALOG_GIVE_IT_BACK")
                        :ReceiveMoney(cxt.quest.param.cost)
                    if not cxt.enc.scratch.extorted then
                        opt:ReceiveOpinion( "gave_back_item" )
                    end
                    opt:Fn(function(cxt)
                            cxt.player.battler:RemoveCardByID(cxt.quest.param.item_id)
                        end)
                        :Travel()

                    cxt:Opt("OPT_EXTORT", extort_amt)
                        :Dialog("DIALOG_EXTORT", extort_amt)
                        :ReceiveOpinion( OPINION.EXTORTION )
                        :Negotiation{
                            on_success = function(cxt) 
                                cxt:Dialog("DIALOG_EXTORT_SUCCESS")
                                cxt.encounter:GainMoney( extort_amt )
                                cxt.player.battler:RemoveCardByID(cxt.quest.param.item_id)
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end,
                            on_fail = function(cxt) 
                                cxt.enc.scratch.extorted = true
                                cxt:Dialog("DIALOG_EXTORT_FAIL")
                            end,

                        }

                    cxt:Opt("OPT_REFUSE")
                        :Dialog("DIALOG_REFUSE")
                        :Fn(function(cxt) 
                            cxt:Opt("OPT_HAND_OVER")
                                :Dialog("DIALOG_HAND_OVER")
                                :ReceiveMoney(cxt.quest.param.cost)
                                :Fn(function(cxt) 
                                    cxt.player.battler:RemoveCardByID(cxt.quest.param.item_id)
                                end)
                                :Travel()

                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle{
                                    enemies =
                                    {
                                        cxt:GetAgent(), TheGame:GetGameState():AddSkinnedAgent( "CRAYOTE_UPGRADED" )
                                    },
                                    on_win = function(cxt)  
                                        cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_WON_FIGHT_DEAD" or "DIALOG_WON_FIGHT_ALIVE" )
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }
                        end)
                else
                    cxt:Opt("OPT_DONT_HAVE_IT_ANY_MORE")
                        :Dialog("DIALOG_DO_NOT_HAVE_IT")
                        :Travel()
                end
            end)
