local MERCENARY_BACKUP_CHANCE = .3
local CHANCE_TO_OFFER_SOMETHING = .75
local CHANCE_TO_DOUBLECROSS = .25
local DISCOUNT = .1

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,
}

    
QDEF:AddCast{
    cast_id = "target",

    cast_fn = function(quest, t) 
        table.insert(t, quest:CreateSkinnedAgent("WEALTHY_MERCHANT"))
    end,

    on_assign = function( quest, agent )
        local money = QuestUtil.CalculateRewardValue( quest )
        agent:DeltaMoney( money )
    end
}

--------------------------------------------------------------------

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_INTRO"
    end)

    :State("STATE_INTRO")
        :Fn( function( cxt )

            local agent = cxt.quest:GetCastMember( "target" )

            cxt.encounter:SetPrimaryCast( agent )

            if math.random() < CHANCE_TO_OFFER_SOMETHING then
                cxt.enc.scratch.graft = RewardUtil.GetGrafts(TheGame:GetGameState():GetCurrentBaseDifficulty(), 1, TheGame:GetGameState():GetPlayerAgent())[1]
            end

            if cxt.enc.scratch.graft then
                cxt:GoTo( "STATE_OFFER" )
            else
                cxt:GoTo( "STATE_MERCHANT" )
            end                
            cxt.quest:Complete()
        end )

    :State("STATE_MERCHANT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !crossed
                    * You come across a lone merchant, a rare sight on Havarian roads. Usually they're smart enough to travel with armed security. 
                agent:
                    !right
                    !happy
                    Hello, stranger. Nice day, isn't it?
                * {agent}'s clothing is rich—and rich clothing has deep pockets. 
                * And the roads are deserted, 'cept for you and your new friend.
            ]],

            OPT_ROB = "Demand their money",
            DIALOG_ROB = [[
                player: 
                    !throatcut
                    Your money or your life. I'll give you one guess which one I'd prefer.
            ]],


            OPT_LET_THEM_GO = "Be on your way",

            DIALOG_LEAVE_DISLIKED = [[
                * {agent} throws back {agent.hisher} shoulders and turns from you with a noticeable flare of {agent.hisher} nostrils.
                agent:
                    !angry
                    $angryReluctant
                    I'd bid you good day. But I don't think you deserve it.
                * {agent.heshe} struts away.
            ]],
            DIALOG_LEAVE = [[
                player:
                    !dubious
                    Uh, yeah. Balmy, even. Have a good one, fat stacks. 
                    !bashful
                    I mean... sirrah. 
                agent:
                    !salute
                    Be safe on these roads, young lady. Bandits are no stranger to these parts.
                * {agent} disappears around the bend, whistling as if the world were {agent.hisher} music box.
                    !exit
            ]],

        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt:Dialog("DIALOG_INTRO")
            end
            
            if cxt:GetAgent():GetMoney() > 0 then
                cxt:Opt("OPT_ROB")
                    :PreIcon( global_images.rob )
                    :Dialog("DIALOG_ROB")
                    :ReceiveOpinion( OPINION.ROBBED )
                    :GoTo("STATE_ROB")
            end

            cxt:Opt("OPT_LET_THEM_GO")
                :Dialog(cxt:GetAgent():GetRelationship() >= RELATIONSHIP.NEUTRAL and "DIALOG_LEAVE" or "DIALOG_LEAVE_DISLIKED")
                :PreIcon( global_images.close )
                :Fn(function(cxt)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)

        :State("STATE_OFFER")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !crossed
                    * You come across a lone merchant, a rare sight on Havarian roads. Usually they're smart enough to travel with armed security. 
                agent:
                    !right
                    !happy
                    Hey, stranger. Nice day, isn't it?
                * {agent}'s clothing is rich—and rich clothing has deep pockets. 
                * And the roads are deserted, 'cept for you and your new friend.
                agent:
                    !interest
                    $miscAlluring
                    Say, you look like someone in need of an edge. I have some goods that might interest you.
            ]],
            OPT_ROB = "Rob {agent}",
            DIALOG_ROB = [[
                player:
                    !angry
                    I think I'll just take it.
            ]],
            OPT_DECLINE = "Decline offer",
            OPT_PURCHASE = "Buy {1#graft}",
            OPT_DOUBLECROSS = "Buy {1#graft}",
            DIALOG_LEAVE_DISLIKED = [[
                * {agent} throws back {agent.hisher} shoulders and turns from you with a noticeable flare of {agent.hisher} nostrils.
                agent:
                    !angry
                    $angryReluctant
                    I'd bid you good day. But I don't think you deserve it.
                * {agent.heshe} struts away.
            ]],
            DIALOG_LEAVE = [[
                player:
                    !dubious
                    No thanks. 
                agent:
                    !shrug
                    Your loss, young lady.
                * {agent} disappears around the bend, whistling as if the world were {agent.hisher} music box.
                    !exit
            ]],
            DIALOG_DOUBLECROSS = [[
                * As you reach for your money, the merchant's eyes light up with greed. {agent.HisHer} unseen allies emerge from the brush.
                agent:
                    !cruel
                    The fish is flush, folks. Time to reel it in. 

            ]],
            OPT_GIVE_MONEY = "Give {agent} the money",
            DIALOG_GIVE_MONEY = [[
                player:
                    Whoa! Listen, just take the money and let me go!
                agent:
                    Huh, easiest money I've made in a while!
            ]],
            DIALOG_ITEM_PURCHASED = [[
                agent:
                    !happy
                    Pleasure doing business with you.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
            DIALOG_KILLED = [[
                * {agent} lies dead in a pool of treachery. Good riddance.
            ]],
            DIALOG_SURVIVED = [[
                agent:
                    !right
                    !injured
                    Mercy! Please!
                    !exit
                * {agent} scuttles off, relieved of the burden of their pockets.
            ]],

        }
        :SetLooping()
        :Fn(function(cxt)

            local price = Util.GetGraftPrice(cxt.enc.scratch.graft.id)
            local doubleCross

            if cxt:FirstLoop() then
                doubleCross = math.random() < CHANCE_TO_DOUBLECROSS
                cxt:Dialog("DIALOG_INTRO")
            end
                
            if(doubleCross) then
                cxt:Opt("OPT_DOUBLECROSS", cxt.enc.scratch.graft )
                    :RequireMoney(price)
                    :PreIcon( global_images.buying )
                    :Dialog("DIALOG_DOUBLECROSS")
                    :Fn(function()
                        local rank = cxt.quest:GetRank()

                        CreateCombatBackup(cxt.encounter:GetAgent(), "MERCENARY_BACKUP", rank)
                        cxt:Opt("OPT_GIVE_MONEY")
                            :Fn(function(cxt)
                                cxt:Dialog("DIALOG_GIVE_MONEY")
                                cxt.caravan:PayMoney( price, cxt:GetAgent() )
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)

                        cxt:Opt("OPT_DEFEND_SELF")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                on_win = function (cxt)
                                    cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_KILLED" or "DIALOG_SURVIVED")
                                    cxt:GoTo("STATE_POST_ROB")
                                end,  
                            }
                    end)
            else 
                cxt:Opt("OPT_PURCHASE", cxt.enc.scratch.graft)
                    :PreIcon( global_images.graft )
                    :RequireMoney(price)
                    :Fn( function( cxt, who )
                        if ConvoUtil.GiveGraft(cxt, cxt.enc.scratch.graft) then
                            local modified_price = CalculatePayment( cxt:GetAgent(), price )
                            cxt.caravan:PayMoney( modified_price, cxt:GetAgent() )
                            cxt:Dialog( "DIALOG_ITEM_PURCHASED" )
                            StateGraphUtil.AddLeaveLocation(cxt) 
                        end
                    end )
            end    
            
            cxt:Opt("OPT_ROB")
                :PreIcon( global_images.rob )
                :Dialog("DIALOG_ROB")
                :ReceiveOpinion( OPINION.ROBBED )
                :GoTo("STATE_ROB")

            cxt:Opt("OPT_DECLINE")
                :Dialog(cxt:GetAgent():GetRelationship() >= RELATIONSHIP.NEUTRAL and "DIALOG_LEAVE" or "DIALOG_LEAVE_DISLIKED")
                :PreIcon( global_images.close )
                :Travel()
        end)


    :State("STATE_POST_ROB")
        :Loc{
            OPT_TAKE_GRAFT = "Take {1#graft}",
        }
        :Fn(function(cxt) 
            if cxt.enc.scratch.graft then
                cxt:Opt("OPT_TAKE_GRAFT", cxt.enc.scratch.graft)
                    :PreIcon( global_images.graft )
                    :Fn(function() 
                        ConvoUtil.GiveGraft(cxt, cxt.enc.scratch.graft)
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end
            StateGraphUtil.AddLeaveLocation(cxt)
        end)
    
    :State("STATE_ROB")
        :Loc{
            DIALOG_ROBBED = [[
                agent:
                    !scared
                    Easy, f-friend. There's no need to resort to violence.
                * {agent} hands you {agent.hisher} purse as quickly as {agent.hisher} shaking hands allow.
                agent:
                    !give
                    There, you see? You're happy, I'm happy. 
                player:
                    !take
                * You count the shills out into your open palm and make a quick exit before the law should happen by.
            ]],
            DIALOG_NO_BACKUP = [[
                agent:
                    !scared
                    Please, no! That will ruin me!
                    Let me give you half of what I have, we can both walk away happy.
            ]],            
            OPT_TAKE_SMALL_LOOT = "Take half of their money",
            DIALOG_TAKE_SMALL_LOOT = [[
                player:
                    !take
                    Deal.
                agent:
                    !exit
                * {agent} makes a hasty escape as you are counting your windfall.
            ]],
            OPT_DEMAND_ALL = "Demand everything",


            DIALOG_BACKUP = [[
                agent:
                    !handwave
                    You think it's that easy, huh?
                * {agent} waves to an unseen observer. Within moments {agent.heshe} is flanked by some imposing backup.
                agent:
                    !angry
                    You were saying?
            ]],
            OPT_BACK_DOWN = "Try to back down",
            DIALOG_BACK_DOWN = [[
                player:
                    Now, now. I wasn't serious about that...
            ]],
            
            DIALOG_BACKED_DOWN = [[
                agent:
                    You're not worth the effort.
                    !exit

                * {agent} leaves with {agent.hisher} entourage. 

                player:
                    !sigh
                * You exhale, your dignity in tatters, but your hide intact.

            ]],

            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    You don't scare me!
            ]],
            DIALOG_DEMAND_ALL = 
            [[
                player:
                    I don't do anything half-arsed. I want <b>everything</>.
            ]],

            DIALOG_SUCCESS_DEMAND_ALL =
            [[
                agent:
                    !scared
                    Take it! Please! I just don't want trouble.
                    !exit
                player:
                    !take

                * You count out your ill-gotten gains, savoring every coin.
            ]],

            DIALOG_FAIL_DEMAND_ALL = 
            [[
                agent:
                    !angry
                    No. You're not being reasonable.
                    I'd rather die with dignity than live with that.
                    !scared
            ]],
            DIALOG_ATTACK = [[
                player:
                    !fight
                    OK, I'll just beat it out of you.
            ]],

            DIALOG_BEAT = [[
                agent:
                    !right
                    !injured
                    Just take it, you Heshin' grifter.
            ]],
            DIALOG_KILLED = [[
                agent:
                    !exit
                * You cut the merchant's purse from {agent.hisher} belt, rolling {agent.hisher} body into the brush, and continue along your way.
            ]],
            DIALOG_FAIL = [[
                agent:
                    !thought
                    Nice try, grifter.
                    !cruel
                    But it's my duty to honest merchants everywhere to put you underground for this.
            ]],
            OPT_ATTACK = "Beat it out of them",
            SIT_MOD = "Protecting their livelihood",

        }
        :SetLooping()
        :Fn(function(cxt) 

            if math.random() < MERCENARY_BACKUP_CHANCE then
                
                if cxt:FirstLoop() then
                    cxt:Dialog("DIALOG_BACKUP")
                    CreateCombatBackup(cxt.encounter:GetAgent(), "MERCENARY_BACKUP", TheGame:GetGameState():GetCurrentBaseDifficulty())
                end
                
                cxt:Opt("OPT_BACK_DOWN")
                    :Dialog("DIALOG_BACK_DOWN")
                    :Negotiation
                    {
                        on_success = function() 
                            cxt:Dialog("DIALOG_BACKED_DOWN")
                            cxt:GoTo("STATE_POST_ROB")
                        end,
                        on_fail = function(cxt, skill_used) 
                            
                            cxt:Dialog("DIALOG_FAIL")

                            --get attacked
                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle
                                {
                                    flags = BATTLE_FLAGS.REWARD_DEATH_LOOT  | BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function(cxt) 
                                        cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_KILLED" or "DIALOG_BEAT")
                                        cxt:GoTo("STATE_POST_ROB")
                                    end,
                                }
                        end
                    }
                
                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle
                    {
                        flags = BATTLE_FLAGS.REWARD_DEATH_LOOT | BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt) 
                            cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_KILLED" or "DIALOG_BEAT")
                            cxt:GoTo("STATE_POST_ROB")
                        end,
                    }

            else

                if cxt:FirstLoop() then
                    cxt:Dialog("DIALOG_NO_BACKUP")
                end
                
                local money = cxt:GetAgent():GetMoney()
                local small_amt = math.round( money /2 )

                cxt:Opt("OPT_TAKE_SMALL_LOOT")
                    :PreIcon( global_images.rob )
                    :ReceiveMoney(small_amt)
                    :Fn(function() 
                        cxt:GetAgent():DeltaMoney(-small_amt)
                        cxt:Dialog("DIALOG_TAKE_SMALL_LOOT")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
                
                cxt:Opt("OPT_DEMAND_ALL")
                    :Dialog("DIALOG_DEMAND_ALL")
                    :Negotiation
                    {
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        situation_modifiers =
                        {
                            { value = 5, text = cxt:GetLocString("SIT_MOD") }
                        },
                        on_success = function() 
                            cxt:Dialog("DIALOG_SUCCESS_DEMAND_ALL")
                            -- Set value of money again in case it increased during negotiation
                            money = cxt:GetAgent():GetMoney()
                            cxt:GetAgent():DeltaMoney(-money)
                            cxt.encounter:GainMoney( money )
                            cxt:GoTo("STATE_POST_ROB")
                        end,
                        on_fail = function(cxt, skill_used) 
                            cxt:Dialog("DIALOG_FAIL_DEMAND_ALL")
                        end
                    }
                
                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle
                    {
                        flags = BATTLE_FLAGS.REWARD_DEATH_LOOT,
                        on_win = function(cxt) 
                            cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_KILLED" or "DIALOG_BEAT")
                            cxt:GoTo("STATE_POST_ROB")
                        end,
                    }
            end
        end)
