
function GetLastRun()
    local current_act = TheGame:GetGameState():GetCurrentAct()
    local history = TheGame:GetRunHistory()
    for i = #history, 1, -1 do
        local run = history[i]
        if GetPlayerActData(run.current_act) == current_act and run.negotiation_deck and run.battle_deck then
            return run
        end
    end
end



local MAX_CARDS = 3
local MAX_GRAFTS = 2
local CARD_PRICE = 40
local GRAFT_PRICE = 80
local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,

    precondition = function(quest) 
        return TheGame:GetGameState():GetPlayerAgent():GetMoney() > 100 and GetLastRun()
    end,

    on_init = function(quest)
        local run = GetLastRun()
        local filtered_grafts = {}
        if run.grafts then 
            for i,graft in ipairs(run.grafts) do
                local def = Content.GetGraft(graft.id)
                if def and (def.type == GRAFT_TYPE.NEGOTIATION or def.type == GRAFT_TYPE.COMBAT) and not def.brawl then
                    table.insert(filtered_grafts, graft)
                end
            end
        end
        quest:SetParameter( "grafts", filtered_grafts )
        quest:SetParameter( "battle_cards", run.battle_deck or table.empty )
        quest:SetParameter( "negotiation_cards", run.negotiation_deck or table.empty )
    end,
}

QDEF:AddCastByAlias{
    cast_id = "stranger",
    alias = "DODGY_SCAVENGER",
}

QDEF:AddConvo()
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("FOO", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Quips{
            {
                tags = "buy_graft",
                [[
                    agent:
                        !happy
                        Feels familiar, doesn't it?
                ]],
                [[
                    agent:
                        !happy
                        Take better care of it this time.
                ]],
                [[
                    agent:
                        !happy
                        Hopefully it serves you right.
                ]],
            },
            {
                tags = "buy_card",
                [[
                    agent:
                        !happy
                        Wrench to let that one go, wasn't it?
                ]],
                [[
                    agent:
                        !happy
                        I thought you'd buy that one.
                ]],
                [[
                    agent:
                        !happy
                        Falling into old patterns? Ha! 
                ]],
                [[
                    agent:
                        !happy                    
                        Fits you like a glove.
                ]],
                [[
                    agent:
                        !happy
                        Like coming home, ain't it?
                ]],
            }
        }
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * You're greeted on the road by someone with the greedy eyes and twitchy fingers of a scavenger. 
                agent:
                    !surprised
                    $neutralTakenAback
                    It's you!
                    But you're alive. I thought I saw you—I mean, I was sure that you—
                    !shrug
                    $scaredStammering
                    Well, <i>died</i>.
                player:
                    !dubious
                    $neutralDubious
                    Uh, not last I checked. 
                agent:
                    Well, I'm sure as Hesh that I looted a corpse that looked just like yours.
                    You're welcome to browse my wares and buy anything that calls to you.

            ]],
            OPT_BUY_GRAFT = "Buy {1#graft}",
            OPT_BUY_CARD = "Buy {1#card}",
            OPT_ASK_WHERE_FOUND = "Ask where this stuff comes from",
            DIALOG_ASK_WHERE_FOUND = [[
                player:
                    You said you looted this from a corpse?
                agent:                  
                    !handwave
                    $miscMischievous  
                    This is the griftlands—I have every right to sell this property, even if you aren't really dead.
                player:
                    !agree
                    I get it. I've looted enough corpses myself.
            ]],
            OPT_DONE = "Leave {agent}",
            DIALOG_DONE = [[
                player:
                    I'm good, thanks.
                agent:
                    Don't be <i>too</> careful out there.
                    $happyJoke
                    !chuckle
                    You'll put me out of business.
                    !exit
                * You blink and {agent} is gone.
            ]]


        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:GetCastMember("stranger"):MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("stranger"))
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")  
                local cards = {}
                
                local negotiation_cards = cxt.quest:GetParameter( "negotiation_cards" ) or table.empty
                for k, t in ipairs( negotiation_cards ) do
                    local id = t.id
                    local card_def = Content.GetNegotiationCard(id)
                    if card_def then
                        local price = math.round( math.randomGauss( CARD_PRICE*.667, CARD_PRICE*1.333) )
                        local found
                        for k,v in pairs(cards) do
                            if v.card_id == id then
                                found = true
                                break
                            end
                        end
                        if not found then
                            table.insert(cards, {card_id = id, price = price, rarity = card_def.rarity, userdata = t.userdata })
                        end
                    end
                end
                
                local battle_cards = cxt.quest:GetParameter( "battle_cards" ) or table.empty
                for k, t in ipairs( battle_cards ) do
                    local id = t.id
                    local card_def = Content.GetBattleCard(id)
                    if card_def then
                        local price = math.round( math.randomGauss( CARD_PRICE*.667, CARD_PRICE*1.333) )
                        local found
                        for k,v in pairs(cards) do
                            if v.card_id == id then
                                found = true
                                break
                            end
                        end
                        if not found then
                            table.insert(cards, {card_id = id, price = price, rarity = card_def.rarity, userdata = t.userdata })
                        end
                    end
                end

                local grafts = {}
                
                local possible_grafts = cxt.quest:GetParameter( "grafts" ) or table.empty
                for k, t in ipairs( possible_grafts ) do
                    local id = t.id
                    local def = Content.GetGraft(id)
                    local price = math.round( math.randomGauss( GRAFT_PRICE*.667, GRAFT_PRICE*1.333) )
                    
                    if def and not cxt.player.graft_owner:HasGraft( id ) then
                        table.insert(grafts, {graft_id = id, price = price, rarity = def.rarity, userdata = t.userdata })
                    end
                end

                cxt.enc.scratch.cards = cards
                cxt.enc.scratch.grafts = grafts
            
                table.sort(cxt.enc.scratch.cards, function(a,b) return CARD_RARITY_ORDINALS[a.rarity] > CARD_RARITY_ORDINALS[b.rarity] end)
                table.sort(cxt.enc.scratch.grafts, function(a,b) return CARD_RARITY_ORDINALS[a.rarity] > CARD_RARITY_ORDINALS[b.rarity] end)
                cxt.enc.scratch.cards[MAX_CARDS+1] = nil
                cxt.enc.scratch.grafts[MAX_GRAFTS+1] = nil
            end

            cxt:Opt("OPT_ASK_WHERE_FOUND")
                :Dialog("DIALOG_ASK_WHERE_FOUND")

            for k,item in ipairs(cxt.enc.scratch.grafts) do
                
                if item.graft_id then
                    cxt:Opt("OPT_BUY_GRAFT", item.graft_id)
                        :PreIcon( global_images.graft )
                        :RequireMoney(item.price, { is_shop = true })
                        :Fn( function( cxt, who )
                            local installed = ConvoUtil.GiveGraft(cxt, GraftInstance(item.graft_id, item.userdata ))
                            if installed then
                                local amount = CalculatePayment(cxt:GetAgent(), item.price, 1, { is_shop = true })
                                cxt.caravan:PayMoney( amount, cxt:GetAgent() )
                                cxt:Quip( cxt:GetAgent(), "buy_graft" )
                                table.remove(cxt.enc.scratch.grafts, k )
                            end
                        end )
                end
            end

            for k,item in ipairs(cxt.enc.scratch.cards) do
                    cxt:Opt("OPT_BUY_CARD", item.card_id)
                        :DeliverMoney(item.price, {is_shop=true})
                        :GainCard(item.card_id)
                        :Fn(function() 
                                cxt:Quip( cxt:GetAgent(), "buy_card" )
                                table.remove(cxt.enc.scratch.cards, k )
                            end)
            end

            cxt:Opt("OPT_DONE")
                :MakeUnder()
                :Dialog("DIALOG_DONE")
                :Travel()
        end)
