
local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    rank = { 1, 5 },
    precondition = function(quest) 
        return TheGame:GetGameState():GetCaravan():CanOwnPet()
    end,
}



:AddCast {
    cast_id = "lumicyte",
    no_validation = true,
    cast_fn = function(quest, t)
        table.insert(t, quest:CreateSkinnedAgent("LUMICYTE_UPGRADED"))
    end,
    
    events = {
        agent_retired = function(quest, agent) 
            quest:Cancel()
        end,
        resolve_battle = function(quest)
            if not quest.param.countdown then
                quest.param.countdown = 3 + math.random(8)
            end

        end,
    }
}

:AddObjective{
    id = "pre",
    state = QSTATUS.ACTIVE,
}

:AddDormancyState("waiting", "active", false, 2, 5)
:AddObjective{
    id = "active",
}

:AddOpinionEvents{

    found_lumicyte =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Found their missing lumicyte",
    },
    kept_lumicyte =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Refused to return their missing lumicyte",
    },
    
}



QDEF:AddConvo("pre")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit")  end)
        :Loc{
            DIALOG_INTRO = [[
                * You're startled by a Cult lumicyte bounding merrily towards you.
                player:
                    !left
                    !surprised
                    Uh—
                lumicyte:
                    !right
                * It's unaccompanied, but seems friendly enough.
                player:
                    !dubious
                    $neutralDubious
                    What do you want?
                * You try to walk past, and it blocks your path, wagging its tail.
                player:
                    !permit
                lumicyte:
                    Guuuuuuuuurgh.

            ]],
            
            OPT_TAKE_LUMICYTE = "Take the lumicyte",
            
            DIALOG_TAKE_LUMICYTE = [[
                player:
                    !thought
                    I wonder if it's because I smell like lumin...?
                    Or maybe you and me have just both had enough of the Cult, huh?
                lumicyte:
                    Pbbbbbbbbth!
                * Seems you have a new friend. 
            ]],
            OPT_DRIVE_OFF_LUMICYTE = "Drive off the lumicyte",
            DIALOG_DRIVE_OFF_LUMICYTE = [[
                player:
                    !threaten
                    Go on! Git! 
                    $angryDefault
                    I had enough of you curs on the derrick. 
                lumicyte:
                    !exit
                * The lumicyte runs away, whimpering softly as it goes. 
                player:
                    Ugh. Hate those things. 
            ]],
            REQ_CANT_HAVE_A_PET = "You can't have another pet right now",

        }
        :Fn(function(cxt) 
            cxt:TalkTo("lumicyte")
            cxt:GetAgent():GenerateRandomName()
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_TAKE_LUMICYTE")
                :Dialog("DIALOG_TAKE_LUMICYTE")
                :ReqCondition(TheGame:GetGameState():GetCaravan():CanOwnPet(), "REQ_CANT_HAVE_A_PET")
                :RecruitMember(PARTY_MEMBER_TYPE.CREW, cxt.quest:GetCastMember("lumicyte"))
                :CompleteQuest("pre")
                :ActivateQuest("waiting")
                :Travel()

            cxt:Opt("OPT_DRIVE_OFF_LUMICYTE")
                :Dialog("DIALOG_DRIVE_OFF_LUMICYTE")
                :CompleteQuest()
                :Travel()
        end)


QDEF:AddConvo("active")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit") and cxt.quest:GetCastMember("lumicyte"):IsInPlayerParty() end)
        :Loc{
            DIALOG_INTRO = [[
                * A bedraggled Cultist approaches you eagerly.
                player:
                    !left
                agent:
                    !right
                    !happy_call_pet
                    $neutralRelief
                    There you are!
                player:
                    !dubious
                    $neutralDubious
                    Excuse me?
                agent:
                    !neutral
                    Oh—not you.
                lumicyte:
                    !left
                agent:
                    !point
                    {lumicyte}!
                    !happy
                    I've been looking all over for you!
                player:
                    !left
                    !crossed
                    Ahem.
                agent:
                    Thank you for finding my lumicyte!
                    Never known one to be as curious as this one—you wouldn't believe the trouble it gets into.
            ]],
            OPT_ASK_ABOUT_LUMICYTE = "Ask about {lumicyte}",
            DIALOG_ASK_ABOUT_LUMICYTE = [[
                player:
                    What do you mean <i>your</i> lumicyte?
                agent:
                    I'm a trainer. This lumicyte is my latest charge, but it's proving a challenge.
                    !thought
                    A mind of its own, this one has. Maybe the will of Hesh itself compels it. 
            ]],

            OPT_DEMAND_A_REWARD = "Demand a reward",
            DIALOG_DEMAND_A_REWARD = [[
                player:
                    $miscPersuasive
                    Sounds like I've provided a valuable service to the Cult, then.
                    One that deserves compensation, maybe?
                agent:
                    Ah—you're after a reward.  
                    $neutralResigned
                    I suppose I should expect as much. But lumicytes of this pedigree are valuable.
                    Here you are. 
                    !give
                agent:
                    Alright, {lumicyte}. Let's go!
                    !exit
                * {lumicyte} hesitates, more confusion than usual clouding {lumicyte.hisher} lumin-lit eyes.
                lumicyte:
                    !right
                    Marrrroooo?
                agent:
                    !crossed
                * You're too busy counting your new shills to notice.
                lumicyte:
                    !exit
            ]],

            OPT_HAND_OVER_LUMICYTE = "Hand over {lumicyte}",
            DIALOG_HAND_OVER_LUMICYTE = [[
                lumicyte:
                    !right
                player:
                    $miscMischievous
                    !permit
                    Whaddya reckon, {lumicyte}? You wanna go with the nice cultist?
                lumicyte:
                    Yap!
                * The lumicyte nuzzles up against {agent} affectionately.
                agent:
                    !right
                player:
                    !happy
                    I think that's a yes. 
                agent:
                    !happy
                    $happyThanks
                    Thank you, grifter. Such graciousness isn't common in the griftlands.
                    !hesh_greeting
                    May you walk in the shallows. 
                    !exit
                * {agent} leads {lumicyte} away, off to do the Cult's bidding.
            ]],

            OPT_REFUSE = "Refuse",
            DIALOG_REFUSE = [[
                lumicyte:
                    !right
                player:
                    $miscMischievous
                    !permit
                    Whaddya reckon, {lumicyte}? You wanna go back to the Cult?
                    Or have you had your fill of Hesh and its bossy tentacles?
                lumicyte:
                    Grrrrrup.
                * The lumicyte hunches its shoulders, growling a clear warning at {agent}. 
                agent:
                    !right
                player:
                    !crossed
                    $neutralDirect
                    That seems like a no.
                    {lumicyte}'s happy where {lumicyte.heshe} is, thanks.
                agent:
                    !angry
                    $neutralDirect
                    I see.
                    $angryDefault
                    Well, I know as well as anyone not to wrestle with an angry lumicyte.
                    Pray the creature doesn't drag you into the deep, grifter. You won't enjoy the experience. 
                    !exit
                * {agent} storms off. You wonder if you chose the right course of action there.
                lumicyte:
                    !right
                    Gruuuuuuu?
                player:
                    !happy
                * Of course you did!
            ]],



        }
        :Fn(function(cxt) 
        
            cxt.quest:Complete()

            cxt:TalkTo(TheGame:GetGameState():AddSkinnedAgent("PRIEST"))
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function()
                
                cxt:Question("OPT_ASK_ABOUT_LUMICYTE", "DIALOG_ASK_ABOUT_LUMICYTE")

                cxt:Opt("OPT_DEMAND_A_REWARD")
                    :Dialog("DIALOG_DEMAND_A_REWARD")
                    :ReceiveMoney(300)
                    :Fn(function() cxt.quest:GetCastMember("lumicyte"):Dismiss() end)
                    :Travel()

                cxt:Opt("OPT_HAND_OVER_LUMICYTE")
                    :Dialog("DIALOG_HAND_OVER_LUMICYTE")
                    :ReceiveOpinion("found_lumicyte")
                    :Fn(function() cxt.quest:GetCastMember("lumicyte"):Dismiss() end)
                    :Travel()

                cxt:Opt("OPT_REFUSE")
                    :Dialog("DIALOG_REFUSE")
                    :ReceiveOpinion("kept_lumicyte")
                    :Travel()
            end)

        end)
