local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,

    on_init = function(quest)
    end,
}

local COST = 90
--------------------------------------------------------------------

QDEF:AddOpinionEvents{
    turned_over_weapons = {
        delta = OPINION_DELTAS.LIKE,
        txt = "Turned over weapons to them",
    },
}

QDEF:AddConvo()
    :ConfrontState("CONF", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                    player:
                        !left
                    * You find a Jake sitting by the side of the road. {agent.HeShe} is visibly injured.
                    * {agent.HeShe} gets up, painfully, as you approach.
                    
                    agent:
                        !right
                        !injured
                        Grifter, I have an offer for you!
                    player:
                        !dubious
                        I'm listening...
                    agent:
                        I've got here a package deal. Three units of the finest improvised weaponry this side of Grout Bog.
                        They really pack a punch! I can tell you that from experience.
                ]],

            OPT_OFFER_TO_BUY_SOME = "Offer to buy <i>some</> of the weapons",

            DIALOG_OFFER_TO_BUY_SOME = [[
                player:
                    I don't need that many. How much for just one?
                agent:  
                    Sorry, I gotta move all of these. You have to take them as a package.
            ]],

            OPT_ASK_ABOUT_INJURIES = "Ask about {agent}'s injuries",
            DIALOG_ASK_ABOUT_INJURIES = [[
                player:
                    Are you alright? You don't look alright.
                agent:
                    I'm fine. I just... let's say these items were hard to come by.
            ]],

            OPT_ASK_FOR_A_BETTER_PRICE = "Ask for a better price",
            DIALOG_ASK_FOR_A_BETTER_PRICE = [[
                player:
                    You're going to have to do better on the price...
                agent:
                    Nope, can't do that. I almost died getting these items.
                    They're already priced to move.
            ]],

            OPT_BUY_THE_ITEMS = "Buy {1#card_list}",
            DIALOG_BUY_ITEMS = [[
                player:
                    I'll take them!
                agent:
                    Really? 
                    !happy
                    Great!
                    You don't know how much this means to me...
                    !give
                player:
                    !take
            ]],

            OPT_DEMAND_THE_ITEMS = "Take the items by force",
            DIALOG_DEMAND_THE_ITEMS = [[
                player:
                    You know what? I think I'll just take them all.
                    !cruel
                    For free.
                agent:  
                    Really?
                    Really?!
                    !fight
                    I'm so tired of this dreck!
            ]],
            DIALOG_WON_FIGHT = [[
                {dead?
                    * You pry your prize from {agent}'s cold, dead hands.
                }
                {not dead?
                    agent:  
                        Fine. Just take them.
                        I should have known it would end this way.
                        !exit
                    * {agent} limps off, leaving you with the goods.
                }
            ]],

            OPT_DECLINE = "Decline the offer",
            DIALOG_DECLINE = [[
                player:
                    I'm not interested.
                agent:
                    No? 
                    Oh. 
                    !sigh
                    Maybe they were right about me.
                    Maybe I should just stick to lifting heavy things.
                    I'll just... I'll be on my way now.
                    !exit
                * {agent} picks up {agent.hisher} wares, and sadly walks off into the distance.
            ]],




        }
        :Fn(function(cxt) 
                cxt.quest:Complete()
                cxt.quest.param.lifter = TheGame:GetGameState():AddSkinnedAgent("JAKES_LIFTER")
                cxt.quest.param.lifter:MoveToLocation(cxt.location)
                cxt.quest.param.lifter.health:SetPercent(.3)
                cxt:TalkTo(cxt.quest.param.lifter)
                cxt:ReassignCastMember("lifter", cxt.quest.param.lifter)

                cxt.quest.param.cost = 100
                cxt:Dialog("DIALOG_INTRO")

                local items = {"rpg", "rpg", "rpg"}
                cxt:RunLoop(function() 
                    
                    
                    cxt:Question("OPT_OFFER_TO_BUY_SOME", "DIALOG_OFFER_TO_BUY_SOME")
                    cxt:Question("OPT_ASK_ABOUT_INJURIES", "DIALOG_ASK_ABOUT_INJURIES")
                    cxt:Question("OPT_ASK_FOR_A_BETTER_PRICE", "DIALOG_ASK_FOR_A_BETTER_PRICE")

                    cxt:Opt("OPT_BUY_THE_ITEMS", items)
                        :DeliverMoney( COST, { is_shop = true } )
                        :Dialog("DIALOG_BUY_ITEMS")
                        :GainCards(items)
                        :GoTo("STATE_RISE_ARRIVES")

                    cxt:Opt("OPT_DEMAND_THE_ITEMS")
                        :Dialog("DIALOG_DEMAND_THE_ITEMS")
                        :ReceiveOpinion( OPINION.ROBBED )
                        :Battle{}
                            :OnWin()
                                :Dialog("DIALOG_WON_FIGHT")
                                
                                :Fn(function() 
                                    cxt.quest.param.fought_lifter = true
                                    if not cxt:GetAgent():IsDead() then
                                        cxt:GetAgent():MoveToLimbo()
                                    end
                                    --local loot = {money = 0, items = {"rpg","rpg","rpg"}}
                                    --cxt.encounter:AwardLoot( loot, { found = true } )
                                end)
                                :GainCards{"rpg","rpg","rpg"}
                                :GoTo("STATE_RISE_ARRIVES")

                    cxt:Opt("OPT_DECLINE")
                        :Dialog("DIALOG_DECLINE")
                        :Travel()

                end)

            end)
    
    :State("STATE_RISE_ARRIVES")
        :Loc{
            DIALOG_INTRO = [[
                {fought_lifter?
                    * As you're gathering up the loot, a group of Rise gets the drop on you.
                    agent:
                        !right
                        Hold it right there, grifter.
                        That's the rightful property of the Rise.
                        !threaten
                        Now hand it over!
                }
                {not fought_lifter?
                    lifter:
                        !exit
                    * As you take possession of the goods, the hairs stand up on the back of your neck.
                    * You're being watched!
                    agent:
                        !right
                        There are the weapons!
                        !threaten
                        Hand them over, grifter. They don't belong to you.
                }
            ]],
            OPT_ASK_ABOUT_RISE= "Ask what the Rise is doing in Murder Bay",
            DIALOG_ASK_ABOUT_RISE = [[
                player:
                    You're a little far from home, aren't you?
                agent:
                    Those weapons were stolen by {lifter}, and we followed {lifter.hisher} trail here.
                    We're headed back to the bog as soon as we've recovered our property.
            ]],
            OPT_ASK_ABOUT_LIFTER= "Ask about the Jake",
            DIALOG_ASK_ABOUT_LIFTER = [[
                player:
                    So {lifter} stole this stuff?
                agent:  
                    The weapons were in transit to Grout Bog to help us in our struggle against the Baron oppressors.
                    This Jake took them right off the docks and ran with them.
            ]],
            DIALOG_WON_FIGHT = [[
                {dead?
                    * The Rise came a long way only to find their defeat at the hands of a grifter.
                }

                {not dead?
                    * {agent} grits {agent.hisher} teeth.
                    agent:
                        !injured
                        If you ever step foot in Grout Bog, you're dead!
                        !exit
                    player:
                        But I let you live.
                        You could have thanked me!

                }
            ]],

            OPT_HAND_OVER = "Hand over the loot",
            DIALOG_HAND_OVER = [[
                * You hand over the ill-gotten goods.
                player:
                    Listen, I don't want any trouble. Take them.
                    !give
                agent:
                    Glad you see it that way!
                    If you're ever in the Bog, look us up.
                    !exit
                * The Rise retreat, weapons in tow.
                
                {not fought_lifter?
                    * You look around for {lifter}, but can't find {lifter.himher}. 
                }
            ]],

            OPT_SCARE_OFF = "Scare the Rise off",
            DIALOG_SCARE_OFF = [[
                {fought_lifter?
                    player:
                        You saw what I did to get these. Do you really want to push me?
                }
                {not fought_lifter?
                    player:
                        I bought these fair and square. You don't want to stand between a grifter and their money.
                }
            ]],
            DIALOG_SCARE_OFF_WIN = [[
                agent:
                    Rise! Away!
                    !exit
                * The Rise scatter, rather than fall to your blades.
            ]],
            DIALOG_SCARE_OFF_LOSE = [[
                agent:
                    You don't scare us, grifter.
                    We're hardened revolutionaries!
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    You're not taking what's mine!
            ]],
        }
        :Fn(function(cxt) 
            local team = CreateCombatParty("RISE_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(team[1])
            cxt:Dialog("DIALOG_INTRO")

            local allies = {}
            local helpers = {}
            if not cxt.quest.param.fought_lifter then
                table.insert (allies, cxt.quest.param.lifter)
                table.insert(helpers, cxt.quest.param.lifter)
            end

            cxt:RunLoop(function() 
                cxt:Question("OPT_ASK_ABOUT_RISE", "DIALOG_ASK_ABOUT_RISE")
                cxt:Question("OPT_ASK_ABOUT_LIFTER", "DIALOG_ASK_ABOUT_LIFTER")
            

                cxt:Opt("OPT_SCARE_OFF")
                    :Dialog("DIALOG_SCARE_OFF")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        helpers = helpers,
                    }
                        :OnSuccess()
                            :Dialog("DIALOG_SCARE_OFF_WIN")
                            :Fn(function() 
                                for k,v in pairs(team) do
                                    if not v:IsDead() then v:MoveToLimbo() end 
                                end
                                if not cxt.quest.param.lifter:IsDead() and cxt.quest.param.lifter:GetLocation() == cxt.location then
                                    cxt:GoTo("STATE_POST_WITH_LIFTER")
                                else
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end
                            end)
                        :OnFailure()
                            :Dialog("DIALOG_SCARE_OFF_LOSE")

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        allies = allies,
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        no_oppo_limit = true,
                    }
                        :OnWin()
                            :Dialog("DIALOG_WON_FIGHT")
                            :Fn(function() 
                                
                                for k,v in pairs(team) do
                                    if not v:IsDead() then v:MoveToLimbo() end 
                                end
                                if not cxt.quest.param.lifter:IsDead() and cxt.quest.param.lifter:GetLocation() == cxt.location then
                                    cxt:GoTo("STATE_POST_WITH_LIFTER")
                                else
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end

                            end)

                cxt:Opt("OPT_HAND_OVER")
                    :Dialog("DIALOG_HAND_OVER")
                    :ReceiveOpinion("turned_over_weapons")
                    :Fn(function()  
                        for k = 1, 3 do
                            local card = cxt.player.battler:FindCardByID("rpg")
                            if card then
                                cxt.player.battler:RemoveCard(card)
                            end
                        end
                    end)
                    :Travel()

            end)
        end)

    :State("STATE_POST_WITH_LIFTER")
        :Loc{
            DIALOG_LIFTER_POST_TALK = [[
                agent:
                    !right
                player:
                    You didn't tell me those weapons were that hot!
                agent:
                    I am truly sorry for that. I thought I had lost them.
                    Thank you for helping me.
            ]],

            OPT_ASK_LIFTER_ABOUT_RISE = "Ask {agent} about the Rise",
            DIALOG_ASK_LIFTER_ABOUT_RISE = [[
                player:
                    Those Rise really had it in for you. How long have they been chasing you?
                agent:
                    The Rise are more stubborn than I had accounted for. They've been following me for days.
                    They got some good hits in when I took the cache, but I gave as good as I got.
                    !shrug
                    Ain't no accounting for politics.
            ]],

            OPT_ASK_LIFTER_ABOUT_JAKES = "Ask {agent} about smuggling",
            DIALOG_ASK_LIFTER_ABOUT_JAKES = [[
                player:
                    No offense, but you aren't very good at this whole smuggling thing, are you?
                agent:
                    I am not. I joined the Jakes to get ahead in life, and for the past 7 years, all I've done is lift heavy things.
                    I want to be more than just a lifter.
                    !sigh
                    Maybe some day.
            ]],


        }
        :Fn(function(cxt) 
            cxt:TalkTo(cxt.quest.param.lifter)
            cxt:Dialog("DIALOG_LIFTER_POST_TALK")                                        
            cxt.quest.param.lifter:OpinionEvent(OPINION.DEFENDED)
            cxt:RunLoop(function() 
                cxt:Question("OPT_ASK_LIFTER_ABOUT_RISE", "DIALOG_ASK_LIFTER_ABOUT_RISE")
                cxt:Question("OPT_ASK_LIFTER_ABOUT_JAKES", "DIALOG_ASK_LIFTER_ABOUT_JAKES")
                StateGraphUtil.AddLeaveLocation(cxt)
            end)        
        end)
