local SAL_MONEY = 200

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    rank = {2, 5},
    cooldown = EVENT_COOLDOWN.LONG,    

    on_init = function(quest) 
        quest.param.reward_amount = 300
        quest.param.rook_wins_solo_fight = quest.param.rook_wins_solo_fight == nil and math.random() > .5 or quest.param.rook_wins_solo_fight
        if quest.param.hesh_boss == nil then
            quest.param.hesh_boss = math.random() < .5
        end
    end,
}

:AddCastByAlias{
    cast_id = "rook",
    alias = "NPC_ROOK",
}


QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit") end)
    :Loc{
        DIALOG_INTRO = [[
            * A figure approaches you, and as he comes into the light you quickly recognize the cunning gaze of another grifter.
            player:
                !left
            rook:
                !right
                !greeting
                Greetings, friend. I hope road and fortune are treating you well.
            player:
                $neutralDubious
                I guess that depends on how this conversation ends. 
            rook:
                !happy
                Ah, I suppose it's only natural you'd be on your guard.
                It's a cutthroat world we live in, after all. But I'm merely interested in contracting your blades.
                You see, I've unfortunately picked up a tail. An assassin, actually.
            player:
                !crossed
                $miscMocking
                You seem pretty relaxed, considering. 
            rook:
                !shrug
                It's old hat. I prefer the ones I know about to the ones I don't.
                $miscPersuasive
                I'd like to nip this in the bud rather than keep up the chase any longer. I'll give you {reward_amount#money} if you join me in the fray.
        ]],
        
        OPT_AGREE = "Agree to the deal",
        DIALOG_AGREE = [[
            player:
                $neutralThanks
                Sure. I do bodyguard work all the time.
            rook:
                Excellent. Here's your fee, then.
                !give
            player:
                !take
            rook:
                !interest
                Ah, here they come.
                I do prefer clean work, when possible, so for now simply hang back and look intimidating. 
                !exit
            player:
                !exit

        ]],

        OPT_DISAGREE = "Refuse",
        DIALOG_DISAGREE = [[
            player:
                !placate
                I've got assassins of my own to worry about.
                Good luck, though, pops.  
            rook:
                !happy
                $neutralResigned
                I haven't run out yet.  
                !exit
            * {rook} continues down the road, hands resting nonchalantly on his pistols.
        ]],

        OPT_ASK_ABOUT_TARGET = "Ask about the target",
        DIALOG_ASK_ABOUT_TARGET = [[
            player:
                Who's after you?
            rook:
                !thought
                $neutralThoughtful
                Would that I knew.
                Someone with more grudge than sense, at least. 
        ]],

        OPT_ASK_ABOUT_SELF = "Ask about {agent}",
        DIALOG_ASK_ABOUT_SELF = [[
            player:
                !interest
                So you've been on the run, then? 
                Or does something else bring you to Murder Bay?
            agent:
                $neutralWhatever
                I'm just passing through.
                !crossed
                $neutralDirect
                It's business.
            player:
                What kind of business?
            agent:
                !interest
                The kind operated under strict confidentiality. 
                If you're trying to suss out whether there's a bounty for me, the answer is no.
                !happy
                $miscMischievous
                At least, not one under any name you'd recognize.
            player:
                !humouring
                Wow. You sure know how to stimulate the curiosity, huh?
            agent:
                !neutral
                I would prefer to simply hire you, if it's all the same.
        ]],
    }
    :Fn(function(cxt)
        cxt.quest:Complete()
        
        cxt:TalkTo(cxt.quest:GetCastMember("rook"))

        cxt:Dialog("DIALOG_INTRO")
        
        cxt:RunLoop(function() 
            cxt:Question("OPT_ASK_ABOUT_TARGET", "DIALOG_ASK_ABOUT_TARGET")
            cxt:Question("OPT_ASK_ABOUT_SELF", "DIALOG_ASK_ABOUT_SELF")
            
            cxt:Opt("OPT_AGREE")
                :ReceiveMoney(cxt.quest.param.reward_amount)
                :Dialog("DIALOG_AGREE")
                :GoTo("STATE_FIGHT_TARGET")

            cxt:Opt("OPT_DISAGREE")
                :Robot("DISAGREE")
                :Dialog("DIALOG_DISAGREE")
                :Travel()
        end)

    end)

    :State("STATE_FIGHT_TARGET")
        :Loc{
            DIALOG_INTRO = [[
                * Sure enough, a lone figure soon approaches.
                rook:
                    !left
                    Alright, <i>friend</i>. That's close enough.
                enemy:
                    !right
                    !angry_accuse
                    You're going to pay for what you've done, Rook.
                rook:
                    Unlikely, given my track record.
                    !overthere
                    I'm afraid you've bit off more than you can chew. I've brought in reinforcements.
                player:
                    !left
                enemy:
                    !dubious
                    $neutralDubious
                    What, this poor sod?
                    Listen to me, grifter. You'll just as likely get a shot in the back than whatever he's promised you.
                player:
                    !crossed
                    I've been hired. You got a counter-offer I should consider?
                enemy:
                    !fight
                    Sure. Stand aside, or I'll bury you both.
            ]],

            OPT_ATTACK = "Attack {enemy}",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    Not good enough.
            ]],

            OPT_STAND_ASIDE = "Stand aside",
            DIALOG_STAND_ASIDE = [[
                player:
                    !thought
                    $neutralThoughtful
                    You know, on second thought... I think I left the electric bartop on.
                    Need to go unplug that before it zaps someone I actually like.
                enemy:
                    !cruel
                    $angryCruel
                    Wise move, grifter.
                    !exit
            ]],
            DIALOG_FOUGHT = [[
                {rook_dead and enemy_dead?
                    * The air is thick with the smell of blood and charred flesh.
                    * You riffle through the mysterious grifter's pockets, but find nothing of interest. Looks like his story will remain a mystery.
                }

                {rook_dead and not enemy_dead?
                    enemy:
                        !injured
                        !right
                        He's finally dead.
                    player:
                        You're pretty close yourself, bub. What was your beef with that guy?
                    
                    {hesh_boss?
                        enemy:
                            I'm just an agent of Hesh. It was fated that this one should die.
                            But you are just a common mercenary.
                            I won't hold your actions against you.
                        player:
                            Thanks, I guess?
                        enemy:
                            !exit
                        * {enemy} leaves you.
                    }

                    {not hesh_boss?
                        enemy:
                            Just know that there's a lot of blood on his hands.
                        player:
                            There's a lot of blood on <i>all</> of him, now.
                        enemy:
                            It's a metaphor, damnit!
                            But this isn't your war, grifter. Good bye.
                            !exit
                        * {enemy} limps off.
                    }
                }

                {not rook_dead and not enemy_dead?
                    {hesh_boss?
                        enemy:
                            !right
                            !injured
                        rook:
                            !left
                            You shouldn't have followed me again, {enemy}.
                        enemy:
                            It won't be the last time. What you did was unforgivable.
                        rook:
                            Now that's just not true. 
                            !cruel
                            I've already forgiven myself.
                            !exit
                        enemy:
                            !exit
                        * {rook} executes {enemy} in cold blood.
                        player:
                            !left
                            !surprised
                            Whoa! What the hell?
                            !neutral
                        rook:
                            !right
                            You heard {enemy.himher}. I'm not safe while {enemy.heshe}'s still drawing breath.
                        player:
                            What did you do to make {enemy.himher} so angry?
                        rook:
                            !shrug
                            I had a run-in with the Cult back in Pearl-on-the-Foam. It was just a job, but {enemy} is- er, was a zealot.
                            There's nothing worse on Hesh's blue orb than someone who takes their religion seriously.
                    }
                    {not hesh_boss?
                        enemy:
                            !right
                            !injured
                        rook:
                            !left
                            This again, {enemy}?
                        enemy:
                            You did me wrong, {rook}. We had a deal and you did me wrong.
                        rook:
                            I'll make you a new deal...
                            !exit
                        enemy:
                            !exit
                        * {rook} executes {enemy} in cold blood.
                        player:
                            !left
                            !surprised
                            Whoa! What the hell?
                            !neutral
                        rook:
                            !right
                            You don't get to be my age leaving loose ends a'frayed. 
                            That's a lesson you'd do well to learn.
                        player:
                            OK, grifter. Next are you going to tell me how much harder the world was in your day?
                        rook:
                            !shrug
                            Nah. It was indifferent then, and it's indifferent now. 
                            Just watch your back, collect your shills, and you'll do fine.
                    }

                    rook:
                        !salute
                        Until next time.
                        !exit
                    * {rook} saunters off, a wary eye betraying his collected gait.
                }
                
                {not rook_dead and enemy_dead?
                    rook:
                        !right
                        Thank you, friend. I could have handled that myself, but I would have risked breaking a sweat.
                    player:
                        Are you going to tell me what that was all about?
                    rook:
                        For you? I assume it was about getting paid.
                    player:
                        No, for you and the{hesh_boss? Hesh-head| Rentorian}.
                    rook:
                        Forgive me, but long experience has taught me the value of keeping my secrets secret.
                        It's nothing personal, of course. Just a matter of policy.
                        We're friendly, but not friends, after all.
                    player:
                        Suit yourself, then.
                    rook:
                        I do. And I will.
                        !salute
                        Until next time, grifter.
                        !exit
                    * {rook} heads off deeper into Murder Bay, carrying more baggage than a pack oshnu under that ridiculous hat.
                }

            ]],

            DIALOG_ROOK_LOST_SOLO_FIGHT = [[
                player:
                    !exit
                * The two strangers fight a pitched battle, and it slowly becomes obvious that the grifter is outmatched.
                enemy:
                    !left
                rook:
                    !right
                    !injured
                    Stop, stop. I yield! Have mercy!
                {hesh_boss?
                    enemy:
                        !fight
                        You don't deserve mercy, {rook}!
                        You don't deserve to live after what-
                }
                {not hesh_boss?
                    enemy:
                        !fight
                        Mercy? <i>You</> are asking <i>me</> for mercy?
                        You've got some nerve, {rook}. I would have thought-
                }
                enemy:
                    !surprised
                    !exit
                rook:
                    !fight
                * {rook} quick-draws his pistols and catches {enemy} square in the chest, leaving a smoldering hole in place of the smoldering hatred that until recently dwelled there.

            ]],
            DIALOG_ROOK_WON_SOLO_FIGHT = [[
                player:
                    !exit
                * The two strangers fight a pitched battle. {enemy} fight with passion, but {rook} keeps {rook.hisher} cool, deftly avoiding most blows and slowly wearing them down.
                * After a particularly reckless swing, {enemy} topples over and falls into the dirt. 
                * {rook} dispatches {enemy.himher} with a single blast through the back of the skull, with all of the feeling one usually reserves for taking out the household trash.
            ]],

            DIALOG_ROOK_TALKS_TO_YOU_AFTER_BETRAYAL = [[
                player:
                    !left
                rook:
                    !right
                    !neutral
                    You did not live up to your side of our agreement, grifter.
                    If I were the type to take things personally, I would probably strike you down.
                    But I'm tired. And my joints hurt. And they'd hurt even more if I took the time to deal with you right now.
                player:
                    I, uh...
                rook:
                    No. No more words. 
                    !threaten
                    You'll need them the next time we meet.
                    !exit
                
            ]],

            OPT_LET_HIM_GO = "Let {rook} go",
            DIALOG_LET_HIM_GO = [[
                * {rook} turns on his heel and walks away. {rook.HeShe} moves pretty quickly, considering {rook.hisher} age.
            ]],

            OPT_ATTACK_ROOK = "Attack {rook}",
            DIALOG_ATTACK_ROOK = [[
                player:
                    !fight
                * You lunge after the aging grifter, not wanting to leave any loose ends.                    
            ]],
            
            DIALOG_WON_ROOK = [[
                player:
                    !neutral
                * {rook} had a long and storied run as a grifter, but ended up dying here, alone and forgotten in the middle of nowhere. 
                * Are you looking at your own fate? Will your life end on some nameless backroad, a footnote in some other grifter's story?
                player:
                    !shrug
                * You continue. There's grifting to do.
            ]]

        }
        :Fn(function(cxt) 
            
            local def = cxt.quest.param.hesh_boss and "HESH_BOSS" or "RENTORIAN_BOSS"

            local enemy = cxt.quest:CreateSkinnedAgent( def )
            TheGame:GetGameState():AddAgent( enemy )
            cxt:ReassignCastMember("enemy", enemy)
            cxt:TalkTo("enemy")
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_ATTACK")
                :ReceiveOpinion(OPINION.HELPED, nil, cxt.quest:GetCastMember("rook"))
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    allies = {cxt:GetCastMember("rook")},
                    enemies = {cxt:GetCastMember("enemy")},
                    flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.SELF_DEFENCE,
                }
                    :OnWin()
                        :Fn(function() 
                            cxt.enc.scratch.rook_dead = cxt.quest:GetCastMember("rook"):IsDead()
                            cxt.enc.scratch.enemy_dead = enemy:IsDead()

                            cxt:Dialog("DIALOG_FOUGHT")
                            if not enemy:IsDead() then
                                enemy:Kill()
                            end
                            if not cxt.enc.scratch.rook_dead then
                                cxt:GetCastMember("rook"):Retire()
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)

            cxt:Opt("OPT_STAND_ASIDE")
                :ReceiveOpinion(OPINION.BETRAYED, nil, cxt:GetCastMember("rook"))
                :Dialog("DIALOG_STAND_ASIDE")
                :Fn(function(cxt) 
                    if cxt.quest.param.rook_wins_solo_fight then
                        cxt:Dialog("DIALOG_ROOK_WON_SOLO_FIGHT")
                        enemy:Kill()
                    else
                        cxt:Dialog("DIALOG_ROOK_LOST_SOLO_FIGHT")
                        enemy:Kill()
                    end
                    cxt.quest:GetCastMember("rook").health:SetPercent( .5 )

                    cxt:Dialog("DIALOG_ROOK_TALKS_TO_YOU_AFTER_BETRAYAL")
                    cxt:Opt("OPT_LET_HIM_GO")
                        :Dialog("DIALOG_LET_HIM_GO")
                        :Fn(function() cxt:GetCastMember("rook"):Retire() end)
                        :Travel()

                    cxt:Opt("OPT_ATTACK_ROOK")
                        :Dialog("DIALOG_ATTACK_ROOK")
                        :Battle{
                            enemies = {cxt:GetCastMember("rook")},
                            flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_SURRENDER,
                            advantage = TEAM.BLUE,
                        }
                            :OnWin()
                                :Dialog("DIALOG_WON_ROOK")
                                :Travel()

                    
                end)
        end)


QDEF:AddDebugOption("hesh_boss", {false, true})
QDEF:AddDebugOption("rook_wins_solo_fight", {false, true})
