local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
}


QDEF:AddCastByAlias{
    cast_id = "chemist",
    alias = "DANGEROUS_STRANGER",
}

QDEF:AddConvo()
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("CHEMIST_VENDOR", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * A mysterious figure approaches you.
                agent:
                    !neutralGreeting
                    Greetings, grifter. 
                    It's rare I meet a fellow traveller in these parts.
                player:
                    !dubious
                    $neutralDubious
                    Really? I feel like I'm constantly running into people.
                agent:
                    $neutralResigned
                    Then you're fortunate. The isolation has made it difficult to complete my work.
                    I'm a chemist; I've been experimenting with new ingredients, but I've only been able to test them on myself.
                    !overthere
                    $miscPersuasive
                    Would you like to try one, and contribute your own experiences to the data?
                * The stranger shows you a row of vials, their contents bubbling against the glass.
            ]],

            OPT_GREEN_VIAL = "Try the murky green vial",

            DIALOG_GREEN_VIAL = [[
                player:
                    What does the green one do?
                agent:
                    It rebalances the humors and flushes the system of toxins.
                player:
                    !thought
                    $neutralThoughtful
                    Makes sense. Green is natural, so it must be healthy.
                agent:
                    !agree
                    That's the idea. Although it took a surprising number of ingredients to get it that color.
                    !give
                player:
                    !drink
                    !wait
                    Huh. Not bad.
            ]],

            OPT_BLUE_VIAL = "Try the smoking blue vial",

            DIALOG_BLUE_VIAL = [[
                player:
                    !interest
                    That blue one looks spicy.
                agent:
                    !handwring
                    Ah yes, this is an unusual blend of three rare Havarian essences.
                    On their own, each is more toxic than the last. 
                    But together, they are a stimulating form of gastrotherapy.  
                player:
                    !dubious 
                    $neutralDubious
                    I assume that means it won't kill me.
                agent:
                    Correct.
                    !give
                agent: 
                    !give
                player:
                    !drink
                    !wince
                    Blech, the flavor!
                agent:
                    Not all gastrotherapy is pleasant, of course.

            ]],

            OPT_ORANGE_VIAL = "Try the bubbling orange vial",

            DIALOG_ORANGE_VIAL = [[
                player:
                    I'll give the orange one a try.
                agent:
                    Excellent choice.
                    It's a fermentation of ambergris and sundevil's spade. It invigorates the vitals and revitalizes the vigors.
                    !give
                player:
                    Huh. Nice.
                    !drink
                    !wait
                    !surprised
                    Phew! Invigorated is right!
            ]],
            
            OPT_BUY_WARP_VIAL = "Take the swirling purple vial",
            
            DIALOG_BUY_WARP_VIAL = [[
                player:
                    That one's interesting. Looks like the sky after a storm.
                    !take
                agent:
                    !agree
                    $neutralThoughtful
                    Hmm, and just as unpredictable. This one has different results each time—
                    But discovery favors the willing, I always say.
                    !give
                player:
                    !placate
                    $neutralDubious
                    Uh—
                agent:
                    !interest
                    Well, go on! Drink! 
                player:
                    !shrug
                    !wait
                    !drink
                    Hmm. 
            ]],

            OPT_DECLINE = "Decline the offer",
            DIALOG_DECLINE = [[
                player:
                    !shrug
                    $neutralJoke
                    Hard to resist your mystery sludge, but that's a no from me. 
                agent:
                    As you like. 
                    !exit
                * The mysterious figure leaves as silently as {agent.heshe} arrived.
            ]],

            DIALOG_POST_DRINK = [[
                * {agent} quickly jots down a note after watching your reaction.
                agent:
                    Thank you for your participation. I hope we meet again soon—
                    !eureka
                    A follow-up can only improve upon the data. 
                player:
                    !overthere
                    $neutralDubious
                    Uh, sure. Hey, this was <i>safe</i>, right?
                agent:
                    I'm fairly confident! But, of course, only time will truly tell. 
                    !exit
                * The mysterious figure leaves as silently as {agent.heshe} arrived. After a moment, {agent.heshe} disappears from view.
                player:
                    Huh.
            ]],
        }

        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:GetCastMember("chemist"):MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("chemist"))
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")  
            end

            cxt:Opt("OPT_GREEN_VIAL")
                :Dialog("DIALOG_GREEN_VIAL")
                :Fn( function( cxt )
                    cxt:Wait()
                    AgentUtil.RemoveNegotiationCard( cxt.player, function( card )
                        cxt.enc:ResumeEncounter( card )
                    end)
                    local card = cxt.enc:YieldEncounter()
                    if card then
                        cxt:Dialog("DIALOG_POST_DRINK")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                end )

            cxt:Opt("OPT_BLUE_VIAL")
                :Dialog("DIALOG_BLUE_VIAL")
                :Fn( function( cxt )
                    cxt:Wait()
                    AgentUtil.UpgradeNegotiationCard( function( card )
                        cxt.enc:ResumeEncounter( card )
                    end )
                    local card = cxt.enc:YieldEncounter()
                    if card then
                        cxt:Dialog("DIALOG_POST_DRINK")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                end )

            cxt:Opt("OPT_ORANGE_VIAL")
                :Dialog("DIALOG_ORANGE_VIAL")
                :Fn( function( cxt )
                    cxt.caravan:UpgradeHealth( 4, "EVENT_ORAGE_VIAL" )
                end )
                :Dialog("DIALOG_POST_DRINK")
                :Travel()

            cxt:Opt("OPT_BUY_WARP_VIAL", "warp_vial")
                :Dialog("DIALOG_BUY_WARP_VIAL")
                :GainCards{"warp_vial"}
                :DeliverMoney(50)
                :Dialog("DIALOG_POST_DRINK")
                :Travel()

            cxt:Opt("OPT_DECLINE")
                :Dialog("DIALOG_DECLINE")
                :Travel()
        end)
