local HEAL_PERCENT = .5

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,
    precondition = function(quest)
        return TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):GetPercent() < 0.75
    end,
}
--------------------------------------------------------------------

:AddCast{
    cast_id = "priest",
    bucket_id = "CULT_OF_HESH",
    condition = function(agent, quest)
        return agent:GetContentID() == "PRIEST"
    end,
}
:AddCastFallback
{
    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "PRIEST" )
        table.insert( t, agent )
    end,
}


QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_OFFER_HEALING"
    end)

        :State("STATE_OFFER_HEALING")
        :Loc{
            DIALOG_INTRO = [[
                priest:
                    !right
                    !happy
                player:
                    !left
                    !crossed
                * You come across {priest}, a priest of the Cult of Hesh traveling alone. 
                * {priest.HeShe} seems calm despite the lack of backup. 
                priest:
                    Greetings, traveler!
                    You look hurt. For a small donation, you may accept the blessing of Hesh and be healed!
            ]],

            OPT_ACCEPT = "Accept healing",
            OPT_ROB = "Rob {priest}",
            DIALOG_ROB = [[
                player:
                    !fight
                    How about you hand over what you've already collected?
            ]],
            OPT_LEAVE = "Decline the offer and leave",
            DIALOG_ACCEPT = [[
                * {priest} lays {priest.hisher} hands on you and heals you.
                priest:
                    That should feel better!
            ]],
            DIALOG_ATTACK = [[
                player:
                    Those are some mighty fancy robes!
            ]],
            DIALOG_LEAVE = [[
                player:
                    Uuh, I'd rather not.
            ]],
            DIALOG_LEAVE_DISLIKED = [[
                player:
                    No. Not from <b>you</>.
            ]],
            DIALOG_KILLED = [[
                priest:
                    !exit
                * You cut the priest's purse from {priest.hisher} belt, rolling {priest.hisher} body into the brush, and continue along your way.
            ]],
            DIALOG_BEAT = [[
                priest:
                    !right
                    !injured
                    W-what do you want with me, heathen?!
                player:
                    !dubious
                    All your money.
                    !shrug
                    Figured that was obvious.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)

            local item
            local price
            local doubleCross

            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.encounter:SetPrimaryCast( cxt:GetCastAgent("priest") )
                cxt:Dialog("DIALOG_INTRO")
                cxt:GetAgent():SetMoney(50)
            end
                
            local cur, max = cxt.player.health:Get()
            cxt:Opt("OPT_ACCEPT")
                :PreIcon(global_images.heal)
                :Dialog("DIALOG_ACCEPT")
                :DeltaHealth(HEAL_PERCENT * max)
                :DeliverMoney(20)
                :Travel()
            
            cxt:Opt("OPT_ROB")
                :Dialog("DIALOG_ROB")
                :PreIcon( global_images.rob )
                :ReceiveOpinion(OPINION.ROBBED)
                :Battle
                {
                    on_win = function(cxt) 
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_KILLED")
                        else
                            cxt:Dialog("DIALOG_BEAT")
                            cxt.encounter:GainMoney( cxt:GetAgent():GetMoney() )
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }

            cxt:Opt("OPT_LEAVE")
                :PreIcon(global_images.close)
                :Dialog(cxt:GetAgent():GetRelationship() >= RELATIONSHIP.NEUTRAL and "DIALOG_LEAVE" or "DIALOG_LEAVE_DISLIKED")
                :Fn(function(cxt)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)

