local CHANCE_FOR_ENEMY = .35
local CHANCE_TO_DIE = .4

local OPTIONS = {"STATE_ADMIRALTY", "STATE_SPREE"} --, "STATE_HESH"}

local QDEF = QuestDef.Define
{
    title = "Good Samaritan",
    icon = engine.asset.Texture("icons/quests/event_injured_person.tex"),
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    events = 
    {
        action_clock_advance = function(quest, location)
            if location ~= quest:GetCastMember("chemist") and quest.param.death_countdown then
                quest.param.death_countdown = quest.param.death_countdown - 1
            end
        end,
    }
}

:AddCast{
    cast_id = "injured_person",
    events = {
        agent_retired = function( quest, agent ) 
            if quest:IsActive("take_to_help") then
                quest:Fail()
            end
        end,
        dismissed = function( quest )
            if quest:IsActive("take_to_help") then
                quest:Fail()
            end
        end
    },
    condition = function(agent, quest)
        return not (agent:GetBrain():GetWorkplace() and agent:GetBrain():GetWorkplace():HasTag("chemist"))
    end,

}

:AddCast{
    cast_id = "main_agent",
    when = QWHEN.MANUAL,
}

:AddLocationCast{
    cast_id = "chemist",
    condition = function(location, quest)
        return location:HasTag("chemist")
    end,
}

:AddObjective{
    id = "intro",
    state = QSTATUS.ACTIVE,    
}

:AddObjective{
    id = "take_to_help",
    title = "Take {injured_person} to {chemist}",
    desc = "{injured_person} is hurt. Take them to {chemist} to get help.",
    mark = { "chemist" },
}


QDEF:AddConvo("take_to_help")
    :Confront(function(cxt, who)
            if cxt.location == cxt.quest:GetCastMember("chemist") then
                return "STATE_DONE"
            end
        end)
    :State("STATE_DONE")
        :Loc{
            DIALOG_INTRO = [[
                * You help {injured_person} through the door.
                player:
                    !left
                injured_person:
                    !right
                    !injured
                player:
                    !overthere
                    $miscRelieved
                    There. You should be okay here.
                injured_person:
                    $miscRelieved
                    Thank you so much. I won't forget this.
                    !exit
                * {injured_person} stumbles off to tend to {injured_person.hisher} wounds.
            ]]
        }
        :Fn(function(cxt) 
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")
                local agent = cxt.quest:GetCastMember("injured_person")
                agent:OpinionEvent(OPINION.SAVED_LIFE)
                agent:Dismiss()
                agent.health:SetPercent(math.random() * 0.2 + 0.4)
                StateGraphUtil.AddEndOption(cxt)
            end)


QDEF:AddConvo("intro")
    :Confront(function(cxt, who)
        local agent = cxt:GetCastAgent("injured_person")
        if agent:GetFactionID() == "ADMIRALTY" then
            return "STATE_SPREE"
        elseif agent:GetFactionID() == "BANDITS" then
            return "STATE_ADMIRALTY"
        end

        return OPTIONS[math.random(#OPTIONS)]
    end)
    :State("STATE_ADMIRALTY")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !injured
                * {agent} comes stumbling towards you, clutching {agent.hisher} gut like it might fall out.
                agent:
                    $scaredPowerless
                    Please! Help me!
                player:
                    !placate
                    Hold on. What's happening?
                agent:
                    $scaredDefault
                    An admiralty patrol just attacked me! But I ain't done nothing wrong!
            ]],
            OPT_HIDE = "Hide {agent} until the danger passes",
            DIALOG_HIDE = [[
                player:
                    !agree
                    Alright. Hide in the tree line and I'll keep them busy.
                agent:
                    $scaredDefault
                    Th-thank you! 
                    !exit
                * {agent} hobbles {agent.hisher} way to the tree line and gets out of sight just in time.
            ]],
            OPT_PROTECT = "Agree to protect {agent}",
            DIALOG_PROTECT = [[
                player:
                    !sigh
                    Sounds like the Admiralty, all right. Get behind me.
                agent:
                    $scaredDefault
                    Th-thank you!
                * {agent} wipes blood from their mouth and huddles in your shadow.
            ]],
            OPT_LEAVE = "Leave {agent} to {agent.hisher} fate",
            DIALOG_LEAVE = [[
                player:
                    !dubious
                    Admiralty? No offense, but I'd rather not get involved.
                agent:
                    $scaredPowerless
                    N-no! Please! You have to help me!
                player:
                    !shrug
                    Sorry, it's not my place.
                * With a panicked yelp, {agent} makes for the tree line. Likely the only help {agent.heshe}'ll get there will be from hungry yotes.
                    !exit
            ]],
        }

        :Fn(function(cxt)
            local agent = cxt.quest:GetCastAgent("injured_person")
            cxt.encounter:SetPrimaryCast(agent)
            agent:MoveToLocation(cxt.location)
            agent.health:SetPercent(math.random(15, 25)*0.01)
            
            cxt.quest:Complete("intro")

            cxt:Dialog("DIALOG_INTRO")


            cxt:Opt("OPT_HIDE")
                :PreIcon(global_images.like)
                :Dialog("DIALOG_HIDE")
                :Fn(function(cxt)
                        cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest.param.death_countdown = 3
                        cxt.quest.param.action = "hiding"
                    end)
                :GoTo("STATE_ADMIRALTY_ARRIVAL")

            cxt:Opt("OPT_PROTECT")
                :PreIcon(global_images.like)
                :Dialog("DIALOG_PROTECT")
                :Fn(function(cxt)
                        cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest.param.death_countdown = 3
                        cxt.quest.param.action = "protecting"
                    end)
                :GoTo("STATE_ADMIRALTY_ARRIVAL")

            cxt:Opt("OPT_LEAVE")
                :PreIcon(global_images.close)
                :Dialog("DIALOG_LEAVE")
                :Fn(function(cxt)
                    cxt:GetAgent():OpinionEvent(OPINION.DID_NOT_HELP)
                    if math.random() < CHANCE_TO_DIE then
                        cxt:GetAgent():Kill()
                    end
                    cxt.quest:Complete()
                end)
                :Travel()
        end)


    :State("STATE_ADMIRALTY_ARRIVAL")
        :Loc{
            DIALOG_INTRO = [[
                * Sure enough, the Admiralty patrol comes hot on {injured_person}'s tail, breathless from the chase.
                player:
                    !left
                agent:
                    !right
                    !tired
                    Hey, you! We're looking for a wanted criminal. You seen 'em?
            ]],
            OPT_GIVE_UP_PERSON = "Give up {injured_person}",
            DIALOG_GIVE_UP_PERSON = [[
                player:
                    !overthere
                    Actually, you know what? I have. {injured_person} is all yours.
                injured_person:
                    !right
                    !surprised
                    $scaredTakenAback
                    You said you'd help me!
                player:
                    !shrug
                    I did. But that's before I knew you were a <i>wanted criminal</i>, so it doesn't count.
                agent:
                    !right
                    !salute
                    Thank you for your help, citizen! This one's been a thorn in our sides too long.
            ]],
            OPT_LIE = "Trick {agent} into looking elsewhere",
            DIALOG_LIE = [[
                player:
                    !thought
                    $neutralThoughtful
                    Now that you mention it, I did see someone walk off towards the creek over there. 
                    !point
                agent:
                    !dubious
                    $neutralDubious
                    Hm, that's rough wood in that direction. Are you sure?
            ]],
            DIALOG_LIE_FAILED = [[
                player:
                    !agree
                    Yep, {injured_person} was definitely headed in that direction.
                agent:
                    !dubious
                    $neutralThoughtful
                    I didn't say {injured_person.hisher} name. 
                player:
                    !scared_shrug
                    $scaredDefault
                    Oh. Uh... oops?
                agent:
                    !angry_accuse
                    $angryHostile
                    So it's an accomplice, is it? You're under arrest!
                    !fight
            ]],
            DIALOG_LIE_SUCCESS = [[
                player:
                    !agree
                    $neutralThoughtful
                    Yeah, {injured_person.heshe} looked real roughed up. I think {injured_person.heshe} might've been concussed?
                agent:
                    !handwring
                    $happyCocky
                    Good—we'll catch {injured_person.himher} quick. Thanks for your help, grifter!
                    !exit
                player:
                    !salute
                    $neutralAside
                    No problem, Officer Brine-for-Brains.
                * {injured_person} emerges from the tree line, trembling like a beaten dog.
                injured_person:
                    !right
                    !injured
                    $happyTakenAback
                    Thank you—truly. Hesh must like me something fierce to put a hunter in my path.
                player:
                    !tired
                    $neutralDubious
                    Hesh doesn't like you much if it lets you bleed out. Let's get you some help, huh?
                * You help {injured_person} to walk as gingerly as you can, but leave a string of profanities in their wake. 
            ]],
            OPT_LIE_FAILED_DEFEND = "Defend {injured_person}",
            DIALOG_LIE_FAILED_DEFEND = [[
                player:
                    !fight
                    $angryHostile
                    Yeah, that's right. An accomplice. So come and get me, switch scum!
                agent:
                    !fight
            ]],
            OPT_LIE_FAILED_GIVE_UP = "Let them take {injured_person}",
            DIALOG_LIE_FAILED_GIVE_UP = [[
                player:
                    !placate
                    $scaredDefault
                    Hey now, I'm no accomplice⁠—I have nothing to do with this. {injured_person.HeShe} is hiding right over there.
                agent:
                    !throatcut
                    $angryHostile
                    That's more like it. Get out of here before you get yourself hurt.
                    !exit
                player:
                    !exit
            ]],
            OPT_FIGHT = "Attack {agent}",
            DIALOG_FIGHT = [[
                player:
                    !angry_accuse
                    $angryHostile
                    You're gonna pay for what you did!
                    !exit
                agent:
                    !exit
            ]],
            DIALOG_POST_FIGHT_NO_SURVIVORS = [[
                * With the patrol dead, you motion for {injured_person} to come with you and head off to seek help.
            ]],
            DIALOG_POST_FIGHT_SURVIVORS = [[
                * The last of the patrol scatter and you head off with {injured_person} to find help.
            ]],
            OPT_INTIMIDATE = "Intimidate {agent}",
            DIALOG_INTIMIDATE = [[
                player:
                    !crossed
                    I'm a hunter, and I've just been contracted by {injured_person} here to keep them alive.
                    Even if it means killing <i>you</i> to guarantee it.
            ]],
            DIALOG_INTIMIDATE_FAIL = [[
                player:
                    !point
                    So what's it gonna be, punk? Do you feel lucky? 
                agent:
                    !cruel
                    $angryCruel
                    I do, actually. After all, I've got two suckers here instead of one. 
                    !fight
                player:
                    !scared
            ]],
            DIALOG_INTIMIDATE_SUCCESS = [[
                player:
                    !left
                    !angry_accuse
                    So how do you wanna end this? Alive, or dead?
                agent:
                    ...
                    !angry_shrug
                    Yeah? All {injured_person.heshe} has is pocket lint anyway. Enjoy your <i>contract</i>, hunter.
                    !exit
                * {agent} leaves in a hurry, with a few cautionary glances over {agent.hisher} shoulder.
                injured_person:
                    !right
                    Thank you—truly. Hesh must like me something fierce to put a hunter in my path on today of all days.
                player:
                    !tired
                    He doesn't like you much if he lets you bleed out. Let's get you some help, huh?
                * You help {injured_person} to walk as gingerly as you can, but leave a string of profanities in their wake. 
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }

        :Fn(function(cxt)
            
            if cxt:FirstLoop() then
                local admiralty_agent = AgentUtil.GetFreeAgent("ADMIRALTY_GUARD")
                cxt.quest:AssignCastMember("main_agent", admiralty_agent)
                cxt.enc.scratch.patrol = CreateCombatBackup(admiralty_agent, "ADMIRALTY_PATROL_BACKUP", cxt.quest:GetRank())
                table.insert(cxt.enc.scratch.patrol, admiralty_agent)
                cxt.enc:SetPrimaryCast(admiralty_agent)
                cxt:Dialog("DIALOG_INTRO")
            end
            
            if cxt.quest.param.action == "hiding" then
                cxt:Opt("OPT_GIVE_UP_PERSON")
                    :PreIcon(global_images.dislike)
                    :Dialog("DIALOG_GIVE_UP_PERSON")
                    :Fn(function(cxt)
                        local injured = cxt:GetCastAgent("injured_person")
                        injured:Dismiss()
                        injured:OpinionEvent( OPINION.BETRAYED )
                        cxt:GetCastAgent("main_agent"):OpinionEvent( OPINION.APPROVE )
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)

                cxt:Opt("OPT_LIE")
                    :Dialog("DIALOG_LIE")
                    :Negotiation{
                        suppressed = {cxt:GetCastAgent("injured_person")},
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_LIE_SUCCESS")
                            cxt.quest:Activate("take_to_help")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_LIE_FAILED")
                            cxt:Opt("OPT_LIE_FAILED_DEFEND")
                                :Dialog("DIALOG_LIE_FAILED_DEFEND")
                                :Battle{
                                    noncombatants = {cxt:GetCastAgent("injured_person")},
                                    on_win = function(cxt)
                                        local survivors = ConvoUtil.CollectSurvivors(cxt.enc.scratch.patrol)
                                        if #survivors == 0 then
                                            cxt:Dialog("DIALOG_POST_FIGHT_NO_SURVIVORS")
                                        else
                                            cxt:Dialog("DIALOG_POST_FIGHT_SURVIVORS")
                                        end
                                        cxt.quest:Activate("take_to_help")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }

                            cxt:Opt("OPT_LIE_FAILED_GIVE_UP")
                                :Dialog("DIALOG_LIE_FAILED_GIVE_UP")
                                :Fn(function(cxt)
                                    cxt:GetAgent():OpinionEvent(OPINION.DID_NOT_HELP)
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)
                            
                        end,
                    }
            else
                cxt:Opt("OPT_INTIMIDATE")
                    :Dialog("DIALOG_INTIMIDATE")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        suppressed = {cxt:GetCastAgent("injured_person")},
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_INTIMIDATE_SUCCESS")
                            cxt.quest:Activate("take_to_help")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_INTIMIDATE_FAIL")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        noncombatants = {cxt:GetCastAgent("injured_person")},
                                        on_win = function(cxt)
                                            local survivors = ConvoUtil.CollectSurvivors(cxt.enc.scratch.patrol)
                                            if #survivors == 0 then
                                                cxt:Dialog("DIALOG_POST_FIGHT_NO_SURVIVORS")
                                            else
                                                cxt:Dialog("DIALOG_POST_FIGHT_SURVIVORS")
                                            end
                                            cxt.quest:Activate("take_to_help")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end
                                    }
                        end,
                    }

                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{
                        noncombatants = {cxt:GetCastAgent("injured_person")},
                        on_win = function(cxt)
                            local survivors = ConvoUtil.CollectSurvivors(cxt.enc.scratch.patrol)
                            if #survivors == 0 then
                                cxt:Dialog("DIALOG_POST_FIGHT_NO_SURVIVORS")
                            else
                                cxt:Dialog("DIALOG_POST_FIGHT_SURVIVORS")
                            end
                            cxt.quest:Activate("take_to_help")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    }
            end        
        end)

    :State("STATE_SPREE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !injured
                * {agent} comes stumbling towards you, clutching {agent.hisher} gut like it might fall out.
                agent:
                    $scaredPowerless
                    Please! Help me!
                player:
                    !placate
                    Hold on. What's happening?
                agent:
                    $scaredDefault
                    Spree! They're tryin' to kill me! Please, there ain't no one else who can help!
            ]],
            OPT_HIDE = "Hide {agent} until the danger passes",
            DIALOG_HIDE = [[
                player:
                    !agree
                    Alright. Hide in the tree line and I'll keep them busy.
                agent:
                    $scaredDefault
                    Th-thank you! 
                    !exit
                * {agent} hobbles {agent.hisher} way to the tree line and gets out of sight just in time.
            ]],
            OPT_PROTECT = "Agree to protect {agent}",
            DIALOG_PROTECT = [[
                player:
                    !sigh
                    They're dogged like that. All right, get behind me.
                agent:
                    $scaredDefault
                    Th-thank you!
                * {agent} wipes blood from their mouth and huddles in your shadow.
            ]],
            OPT_LEAVE = "Leave {agent} to {agent.hisher} fate",
            DIALOG_LEAVE = [[
                player:
                    Spree? Uh, I'd prefer to stay out of trouble.
                agent:
                    No! Please! You have to help me!
                player:
                    Sorry, it's not my place.
                    !exit
            ]],
        }

        :Fn(function(cxt)
            local agent = cxt.quest:GetCastAgent("injured_person")
            cxt.encounter:SetPrimaryCast(agent)
            agent:MoveToLocation(cxt.location)
            agent.health:SetPercent(math.random(15, 25)*0.01)
            
            cxt.quest:Complete("intro")

            cxt:Dialog("DIALOG_INTRO")


            cxt:Opt("OPT_HIDE")
                :Dialog("DIALOG_HIDE")
                :Fn(function(cxt)
                        cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest.param.death_countdown = 3
                        cxt.quest.param.action = "hiding"
                    end)
                :GoTo("STATE_SPREE_ARRIVAL")

            cxt:Opt("OPT_PROTECT")
                :Dialog("DIALOG_PROTECT")
                :Fn(function(cxt)
                        cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest.param.death_countdown = 3
                        cxt.quest.param.action = "protecting"
                    end)
                :GoTo("STATE_SPREE_ARRIVAL")

            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :Fn(function(cxt)
                    cxt:GetAgent():OpinionEvent(OPINION.DID_NOT_HELP)
                    if math.random() < CHANCE_TO_DIE then
                        cxt:GetAgent():Kill()
                    end
                    cxt.quest:Complete()
                end)
                :Travel()
        end)


    :State("STATE_SPREE_ARRIVAL")
        :Loc{
            DIALOG_INTRO = [[
                * The Spree are punctual, especially when there's a trail of blood to follow.
                player:
                    !left
                agent:
                    !right
                    !angry_accuse
                    Hey! You lookin' for trouble?
            ]],
            OPT_GIVE_UP_PERSON = "Give up {injured_person}",
            DIALOG_GIVE_UP_PERSON = [[
                player:
                    !thought
                    You know what? I'm really not. I'm a hunter. So how about I leave you to your business, and you leave me to mine?
                injured_person:
                    !right
                    !scared
                    $scaredPowerless
                    W-what?! N-no! You said you would help me!
                agent:
                    !right
                    !angry_shrug
                    $angryDefault
                    We don't want no trouble with a hunter. Sod off, and we'll take care of the rest.
                player:
                    !agree
                    $neutralDubious
                    Ruder than I'd like, but fair.
                * You continue down the path, ignoring the screams you leave behind you.

            ]],
            OPT_LIE = "Trick {agent} into looking elsewhere",
            DIALOG_LIE = [[
                player:
                    Not me, friend. I'm just minding my own business.
                    !point
                    $neutralThoughtful
                    But now that you mention it, I <i>did</i> see a weeping wretch head off in the other direction. 
                    Easy pickings for you, I imagine.
                agent:
                    !dubious
                    $neutralDubious
                    Is that so?
            ]],
            DIALOG_LIE_FAILED = [[
                player:
                    !agree
                    $miscPersuasive
                    Sure. They were limping and everything. Go that way, and you can't miss 'em.
                agent:
                    !angry_accuse
                    $angrySeething
                    Funny, 'cause we just came from that direction, and I didn't see no wretch.
                    You're up to something!
            ]],
            DIALOG_LIE_SUCCESS = [[
                player:

                agent:
                    !cruel
                    $angryCruel
                    I was actually just thinking I should circle back and check again. Thanks for your help!
                    !exit
                player:
                    !salute
                    $miscMocking
                    No problem, scum.
                agent:
                    !exit
                * {injured_person} comes out of hiding once {agent} has left.
                injured_person:
                    !right
                    !injured
                    $miscRelieved
                    Thank you so much! You saved my life there.
                player:
                    Let's get you somewhere safe. 
            ]],
            OPT_LIE_FAILED_DEFEND = "Defend {injured_person}",
            DIALOG_LIE_FAILED_DEFEND = [[
                player:
                    You're not getting {injured_person.himher} without going through me!
                agent:
                    Ha! Sounds like a plan!
            ]],
            OPT_LIE_FAILED_GIVE_UP = "Let them take {injured_person}",
            DIALOG_LIE_FAILED_GIVE_UP = [[
                player:
                    Okay, okay, I'm sorry. They're hiding right over there!
                agent:
                    That's more like it. Get out of here before you get hurt.
            ]],
            
            OPT_INTIMIDATE = "Intimidate {agent}",
            DIALOG_INTIMIDATE = [[
                player:
                    !crossed
                    I'm a hunter, and I've just been contracted by {injured_person} here to keep them alive.
                    Even if it means killing <i>you</i> to guarantee it.
            ]],
            DIALOG_INTIMIDATE_FAIL = [[
                player:
                    !point
                    So what's it gonna be, punk? Do you feel lucky? 
                agent:
                    !cruel
                    I do, actually. After all, I've got two suckers here instead of one. 
                    !fight
                player:
                    !scared
            ]],
            DIALOG_INTIMIDATE_SUCCESS = [[
                player:
                    !left
                    !angry_accuse
                    So how do you wanna end this? Alive, or dead?
                agent:
                    ...
                    !angry_shrug
                    Yeah? All {injured_person.heshe} has is pocket lint anyway. Enjoy your <i>contract</i>, hunter.
                    !exit
                * {agent} leaves in a hurry, with a few cautionary glances over {agent.hisher} shoulder.
                injured_person:
                    !right
                    Thank you—truly. Hesh must like me something fierce to put a hunter in my path on today of all days.
                player:
                    !tired
                    He doesn't like you much if he lets you bleed out. Let's get you some help, huh?
                * You help {injured_person} to walk as gingerly as you can, but leave a string of profanities in their wake. 
            ]],
            OPT_FIGHT = "Attack {agent}",
            DIALOG_FIGHT = [[
                player:
                    Look, we both know this ends in blood. So let's just get it over with.
                    !fight
                agent:
                    !fight
            ]],
            DIALOG_POST_FIGHT_NO_SURVIVORS = [[
                * With the Spree dead, {injured_person} emerges shakily from the sidelines.
                injured_person:
                    !right
                    Thank you—truly. Hesh must like me something fierce to put a hunter in my path on today of all days.
                player:
                    !left
                    !tired
                    He doesn't like you much if he lets you bleed out. Let's get you some help, huh?
                * You help {injured_person} to walk as gingerly as you can, but leave a string of profanities in their wake. 
            ]],
            DIALOG_POST_FIGHT_SURVIVORS = [[
                * The last of the group scatters, bleating a bit like flogged cattle.
                injured_person:
                    !right
                    Thank you—truly. Hesh must like me something fierce to put a hunter in my path on today of all days.
                player:
                    !left
                    !tired
                    He doesn't like you much if he lets you bleed out. Let's get you some help, huh?
                * You help {injured_person} to walk as gingerly as you can, but leave a string of profanities in their wake. 

            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }

        :Fn(function(cxt)
            local patrol
            if cxt:FirstLoop() then
                local spree_agent = AgentUtil.GetFreeAgent(cxt.quest:GetRank() <= 1 and "BANDIT_GOON" or "BANDIT_GOON2")
                cxt.quest:AssignCastMember("main_agent", spree_agent)
                cxt.enc.scratch.patrol = CreateCombatBackup(spree_agent, "BANDIT_PATROL_BACKUP", cxt.quest:GetRank())
                table.insert( cxt.enc.scratch.patrol, spree_agent )
                cxt.enc:SetPrimaryCast(spree_agent)
                cxt:Dialog("DIALOG_INTRO")
            end
            
            if cxt.quest.param.action == "hiding" then
                cxt:Opt("OPT_GIVE_UP_PERSON")
                    :PreIcon(global_images.dislike)
                    :Dialog("DIALOG_GIVE_UP_PERSON")
                    :Fn(function(cxt)
                        local injured = cxt:GetCastAgent("injured_person")
                        injured:Dismiss()
                        injured:OpinionEvent( OPINION.BETRAYED )
                        cxt:GetCastAgent("main_agent"):OpinionEvent( OPINION.APPROVE )
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)

                cxt:Opt("OPT_LIE")
                    :Dialog("DIALOG_LIE")
                    :Negotiation{
                        suppressed = {cxt:GetCastAgent("injured_person")},
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_LIE_SUCCESS")
                            cxt.quest:Activate("take_to_help")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_LIE_FAILED")
                            cxt:Opt("OPT_LIE_FAILED_DEFEND")
                                :Dialog("DIALOG_LIE_FAILED_DEFEND")
                                :Battle{
                                    noncombatants = {cxt:GetCastAgent("injured_person")},
                                    on_win = function(cxt)
                                        local survivors = ConvoUtil.CollectSurvivors(cxt.enc.scratch.patrol)
                                        if #survivors == 0 then
                                            cxt:Dialog("DIALOG_POST_FIGHT_NO_SURVIVORS")
                                        else
                                            cxt:Dialog("DIALOG_POST_FIGHT_SURVIVORS")
                                        end
                                        cxt.quest:Activate("take_to_help")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }

                            cxt:Opt("OPT_LIE_FAILED_GIVE_UP")
                                :Dialog("DIALOG_LIE_FAILED_GIVE_UP")
                                :Fn(function(cxt)
                                    cxt:GetAgent():OpinionEvent(OPINION.DID_NOT_HELP)
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)
                            
                        end,
                    }
            else
                cxt:Opt("OPT_INTIMIDATE")
                    :Dialog("DIALOG_INTIMIDATE")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        suppressed = {cxt:GetCastAgent("injured_person")},
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_INTIMIDATE_SUCCESS")
                            cxt.quest:Activate("take_to_help")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_INTIMIDATE_FAIL")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    noncombatants = {cxt:GetCastAgent("injured_person")},
                                    on_win = function(cxt)
                                        local survivors = ConvoUtil.CollectSurvivors(cxt.enc.scratch.patrol)
                                        if #survivors == 0 then
                                            cxt:Dialog("DIALOG_POST_FIGHT_NO_SURVIVORS")
                                        else
                                            cxt:Dialog("DIALOG_POST_FIGHT_SURVIVORS")
                                        end
                                        cxt.quest:Activate("take_to_help")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }
                        end,
                    }

                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{
                        noncombatants = {cxt:GetCastAgent("injured_person")},
                        on_win = function(cxt)
                            local survivors = ConvoUtil.CollectSurvivors(cxt.enc.scratch.patrol)
                            if #survivors == 0 then
                                cxt:Dialog("DIALOG_POST_FIGHT_NO_SURVIVORS")
                            else
                                cxt:Dialog("DIALOG_POST_FIGHT_SURVIVORS")
                            end
                            cxt.quest:Activate("take_to_help")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    }
            end        
        end)

QDEF:AddConvo("take_to_help")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.quest.param.death_countdown <= 0 end)
        :Loc{
            DIALOG_INTRO = [[
                player: 
                    !left
                injured_person:
                    !right
                    !injured
                injured_person:
                    I'm not... going to make it...
                    !exit
                * {injured_person} dies.
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest:Fail()
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:GetCastMember("injured_person"):Kill()
            StateGraphUtil.AddEndOption(cxt)
        end)
