local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest)
        return TheGame:GetGameState():GetCaravan():GetMoney() >= 50
    end,
}

QDEF:AddCastByAlias{
    cast_id = "lotto",
    alias = "GAMBLER",
}


QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_CONFRONT"
    end)
    :Loc{
        OPT_HEADS = "Call heads",
        DIALOG_HEADS = [[
            player:
                Heads!
        ]],
        OPT_TAILS = "Call snails",
        DIALOG_TAILS = [[
            player:
                Snails!
        ]],
    }

    :State("STATE_CONFRONT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !cruel
                * {agent} approaches you wearing a smile like a grease stain.
                agent:
                    $happyGreeting
                    Bet you love a good thrill. Fancy a wager?
                player:
                    !dubious
                    $neutralDubious
                    A wager? What kind?
                agent:
                    $miscMischievous
                    Just a game of chance, friend. 
                    I'll flip this shill, and you call it in the air. Double your money if the fates are on your side.
            ]],

            OPT_DO_BET = "Place a bet of {1#money}",
            DIALOG_DO_BET = [[
                player:
                    !happy
                    $happyCocky
                    I wouldn't be here if the fates weren't usually on my side. 
                agent:
                    !happy
                    $happyThanks
                    Excellent. That's the kind of overconfidence I like to hear. 
                    !wink
                * With a wink, {agent} tosses the coin high in the air...
                agent:
                    !point
                    $neutralDirect
                    Call it!
            ]],
            
            
            OPT_DECLINE = "Decline the offer",
            DIALOG_DECLINE = [[
                player:
                    !bashful
                    $neutralResigned
                    The odds and me are usually... at odds. 
                    I think I'll keep my shills, but I appreciate the offer all the same.
                agent:
                    !neutralWhatever
                    $neutralWhatever
                    Huh. Fair enough. Don't forget to live a little, though.
                    !exit
                * {agent} leaves, flicking {agent.hisher} coin as {agent.heshe} goes.
            ]]
        }
        
        :Fn(function(cxt)
            cxt.quest:GetCastMember("lotto"):MoveToLocation(cxt.location)
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("lotto"))
            cxt.quest:Complete()
            
            cxt:Dialog("DIALOG_INTRO")

            for k,bet in pairs{ 50, 75, 150} do
                cxt:Opt("OPT_DO_BET", bet)
                    :Fn(function( cxt )
                        AUDIO:PlayEvent("event:/ui/main/gen/place_bet")
                    end)
                    :Dialog("DIALOG_DO_BET", bet)
                    :RequireMoney(bet, {no_scale=true})
                    :Fn(function(cxt) 
                        cxt.enc.scratch.amt_to_win = bet
                        cxt.enc.scratch.amt_to_lose = bet
                        cxt:Opt("OPT_HEADS")
                            :Dialog("DIALOG_HEADS")
                            :GoTo("STATE_POST_BET")
                        cxt:Opt("OPT_TAILS")
                            :Dialog("DIALOG_TAILS")
                            :GoTo("STATE_POST_BET")
                    end)
            end


            cxt:Opt("OPT_DECLINE")
                :Dialog("DIALOG_DECLINE")
                :Travel()

        end)

    :State("STATE_POST_BET")
        :Loc{

            DIALOG_WIN_BET = [[
                agent:
                    !coin_toss
                * {agent} catches the coin with a flourish. {agent.HeShe} shows it to you.
                agent:
                    $happyCocky
                    Sorry, pal, but it looks like—
                player:
                    !happy
                    $happyAmazed
                    #event:/ui/main/gen/win_bet
                    I won!
                agent:
                    !surprise
                    $neutralTakenAback
                    What?
                * {agent} looks disappointedly down at the coin. Sure enough, you called it.
                agent:  
                    !angry
                    $angryTakenAback
                    What the...
                    !happy
                    $happyResigned
                    I mean, whaddya know! You <b>are</> lucky!
                    Of course... you <b>could</> take your winnings now, or you could double your haul!
                    !overthere
                    $miscAlluring
                    Double or nothing, what do you say? Chance favors the bold, ya know.
                *** You won, and now {agent} is offering you double or nothing.
            ]],
            
            OPT_TAKE_PAY_OUT = "Take your winnings",
            DIALOG_PAY_OUT = 
            [[
                player:
                    $happyThanks
                    Nah, I don't want to push my luck.
            ]],

            OPT_TAKE_DOUBLE_OR_NOTHING_WIN = "Bet again!",
            DIALOG_TAKE_DOUBLE_OR_NOTHING_WIN = [[
                player:
                    !crossed
                    $happyCocky
                    Toss again. I aim to leave this meeting richer than ever.
                agent:
                    !happy
                    $happyDefault
                    Atta girl. Ready or not...
                * {agent} flips the coin.
                agent:
                    Call it!
            ]],

            DIALOG_LOST_BET = [[
                * {agent} catches the coin with a flourish. {agent.HeShe} shows it to you.
                agent:
                    !chuckle
                    $happyCocky
                    #event:/ui/main/gen/lose_bet
                    Looks like the fates fancy <i>me</i> today.
                player:
                    !cringe
                    $angryAside
                    Bladderwrack!
                agent:
                    $miscMischievous
                    Aw, no need to look so glum. After all, we're here to have fun, ain't we?
                    $miscAlluring
                    I'll give you one more shot. Double or nothing, whaddya say?
                *** You lost, but {agent} is offering you double or nothing.
                
            ]],
            OPT_TAKE_DOUBLE_OR_NOTHING_LOSS = "Double down on your bet",
            DIALOG_TAKE_DOUBLE_OR_NOTHING_LOSS = [[
                player:
                    !crossed
                    $neutralWhatever
                    I'm a sucker for punishment, apparently. Toss it!
                agent:
                    $happyTakenAback
                    I like the way you think.
                * {agent} tosses the coin again.
                agent:
                    Call it!
            ]],


            OPT_ACCUSE_OF_CHEATING = "Accuse {agent} of cheating",

            DIALOG_ACCUSE_OF_CHEATING = [[
                player:
                    !angry
                    $angryDefault
                    Something tells me this game is rigged. You wouldn't be cheating now, would you?
                agent:
                    !angry_shrug
                    $angryDefault
                    Hey, pal, you don't like the game? Don't play it.
                    Keep your accusations. Now, if you'll excuse me, I think the fun is over.
                * {agent} slips the coin into {agent.hisher} pocket before passing you by.
            ]],    

            OPT_PAY_UP = "Pay {agent}",
            DIALOG_PAY_UP = [[
                * You pay {agent} the money.
                player:
                    !give
                    $neutralResigned
                    Fair's fair. Take it.
                agent:
                    !take
                    $happyDefault
                    Nice. I like you, {player}⁠—you say yes to the game, and pay without complaint. 
                    Let's play again sometime, huh?
                    !exit
                * {agent} skips away, counting your money.
            ]]
        }
        
        :Fn(function(cxt)
            
            if true then --math.random() < .25 then     --they're *trying* to cheat :)
                cxt:Dialog("DIALOG_WIN_BET")
                cxt:Opt("OPT_TAKE_DOUBLE_OR_NOTHING_WIN")
                    :Fn(function( cxt )
                        AUDIO:PlayEvent("event:/ui/main/gen/place_bet")
                    end)
                    :Dialog("DIALOG_TAKE_DOUBLE_OR_NOTHING_WIN")
                    :Fn(function() 
                        cxt.enc.scratch.amt_to_win = cxt.enc.scratch.amt_to_win*2
                        cxt.enc.scratch.amt_to_lose = 0
                        cxt:Opt("OPT_HEADS")
                            :Dialog("DIALOG_HEADS")
                            :GoTo("STATE_DOUBLE_WIN_PST")
    
                        cxt:Opt("OPT_TAILS")
                            :Dialog("DIALOG_TAILS")
                            :GoTo("STATE_DOUBLE_WIN_PST")
                    end)
                
                cxt:Opt("OPT_TAKE_PAY_OUT")
                    :Dialog("DIALOG_PAY_OUT")
                    :GoTo("STATE_TAKE_WINNINGS")
            else


                cxt:Dialog("DIALOG_LOST_BET")
                
                cxt:Opt("OPT_TAKE_DOUBLE_OR_NOTHING_LOSS")
                    :Fn(function( cxt )
                        AUDIO:PlayEvent("event:/ui/main/gen/place_bet")
                    end)
                    :Dialog("DIALOG_TAKE_DOUBLE_OR_NOTHING_LOSS")
                    :RequireMoney(cxt.enc.scratch.amt_to_lose*2)
                    :Fn(function() 
                        cxt.enc.scratch.amt_to_win = 0
                        cxt.enc.scratch.amt_to_lose = cxt.enc.scratch.amt_to_lose*2
                        
                        cxt:Opt("OPT_HEADS")
                            :Dialog("DIALOG_HEADS")
                            :GoTo("STATE_DOUBLE_LOSS_PST")
    
                        cxt:Opt("OPT_TAILS")
                            :Dialog("DIALOG_TAILS")
                            :GoTo("STATE_DOUBLE_LOSS_PST")
                    end)
                
                cxt:Opt("OPT_PAY_UP")
                    :Dialog("DIALOG_PAY_UP")
                    :DeliverMoney(cxt.enc.scratch.amt_to_lose, {no_scale = true})
                    :Travel()

                cxt:Opt("OPT_ACCUSE_OF_CHEATING")
                    :Dialog("DIALOG_ACCUSE_OF_CHEATING")
                    :GoTo("STATE_ACCUSE")

            end

        end)


    :State("STATE_DOUBLE_LOSS_PST")
        :Loc{
            DIALOG_WIN_BET = [[
                * {agent} catches the coin awkwardly, almost dropping it. The result is in your favor.
                agent:
                    !palm
                    $neutralResigned
                    Aw, Hesh!
                player:
                    !happy
                    #event:/ui/main/gen/win_bet
                    At least I recouped my losses.
                agent:
                    $angryReluctant
                    Yeah, at least you did. Well, that's enough fun for today.
                    !shrug
                    Maybe we'll try this again some other time, eh?
                * {agent} looks dourly at {agent.hisher} coin before slipping it back into {agent.hisher} pocket.
                    !exit
            ]],

            DIALOG_LOSE_BET = [[
                * {agent} catches the coin expertly, almost flaunting their manipulations.
                agent:
                    $miscMocking
                    #event:/ui/main/gen/lose_bet
                    'Fraid that's another win for me. 
                    If I were you, I'd stay out of trouble today, friend—the fates clearly have it out for you.
                    !cruel
                    Time to settle up.
            ]], 
            OPT_HAND_OVER_MONEY = "Pay {agent}",
            DIALOG_HAND_OVER_MONEY = [[
                * You pay {agent} the full amount.
                player:
                    !give
                    Ugh, fine.
                agent:
                    !take
                    Chin up. No need to be a sore loser, champ.
                    $miscNostalgic
                    It's always gotta be <i>someone's</i> turn to win.
                    !exit
                * {agent} skips away, counting your money.

            ]],
            OPT_ACCUSE_OF_CHEATING = "Accuse {agent} of cheating",
            DIALOG_ACCUSE_OF_CHEATING = [[
                player:
                    !angry
                    $angryDefault
                    Something tells me this game is rigged. You wouldn't be cheating now, would you?
                agent:
                    !angry_shrug
                    $angryDefault
                    Hey, pal, you don't like the game? Don't play it.
                    Keep your accusations. Now, if you'll excuse me, I think the fun is over.
                * {agent} slips the coin into {agent.hisher} pocket before passing you by.
            ]],
        }
        :Fn(function(cxt)  
            if math.random() < .25 then
                cxt:Dialog("DIALOG_WIN_BET")
                StateGraphUtil.AddLeaveLocation(cxt)

            else
                cxt:Dialog("DIALOG_LOSE_BET")

                cxt:Opt("OPT_HAND_OVER_MONEY")
                    :Dialog("DIALOG_HAND_OVER_MONEY")
                    :DeliverMoney(cxt.enc.scratch.amt_to_lose, {no_scale = true})
                    :Travel()

                cxt:Opt("OPT_ACCUSE_OF_CHEATING")
                    :Dialog("DIALOG_ACCUSE_OF_CHEATING")
                    :GoTo("STATE_ACCUSE")

            end

        end)

    :State("STATE_DOUBLE_WIN_PST")
        :Loc{
            DIALOG_WIN_BET = [[
                * {agent} catches the coin awkwardly, almost dropping it.
                agent:
                    !angry
                    $angryTakenAback
                    Heshin' slime!
                player:
                    !cruel
                    $happyCocky
                    #event:/ui/main/gen/win_bet
                    Today <b>is</> my lucky day!
            ]],

            OPT_TAKE_WINNINGS = "Take your winnings",
            
            DIALOG_TAKE_WINNINGS = [[
                player:
                    !crossed
                    I'll be taking my winnings, now.
            ]],


            DIALOG_LOSE_BET = [[
                * {agent} catches the coin expertly, almost flaunting their manipulations, and reveals your loss.
                agent:
                    $happyCocky
                    #event:/ui/main/gen/lose_bet
                    You know what they say: easy come, easy go.
                    Let's call it there, huh? Clearly the fates favor neither of us today.
            ]], 
            OPT_LET_GO = "Let {agent} go",
            DIALOG_LET_GO = [[
                player:
                    !sigh
                    $neutralResigned
                    Yeah, yeah. Kinda amounted to a waste of time, though.
                agent:
                    !eureka
                    $happyCocky
                    It's never a waste when there's odds to play.
                    !exit

                * {agent} leaves so quickly you'd almost wonder if {agent.heshe} were in a hurry.
            ]],


            OPT_ACCUSE_OF_CHEATING = "Accuse {agent} of cheating",
            DIALOG_ACCUSE_OF_CHEATING = [[
                player:
                    !angry
                    $angryDefault
                    Something tells me this game is rigged. You wouldn't be cheating now, would you?
                agent:
                    !angry_shrug
                    $angryDefault
                    Hey, pal, you don't like the game? Don't play it.
                    Keep your accusations. Now, if you'll excuse me, I think the fun is over.
                * {agent} slips the coin into {agent.hisher} pocket before passing you by.
            ]],
        }
        :Fn(function(cxt)  
            if math.random() < .25 then
                cxt:Dialog("DIALOG_WIN_BET")
                
                cxt:Opt("OPT_TAKE_WINNINGS")
                    :Dialog("DIALOG_TAKE_WINNINGS")
                    :GoTo("STATE_TAKE_WINNINGS")

            else
                cxt:Dialog("DIALOG_LOSE_BET")

                cxt:Opt("OPT_LET_GO")
                    :Dialog("DIALOG_LET_GO")
                    :Travel()

                cxt:Opt("OPT_ACCUSE_OF_CHEATING")
                    :Dialog("DIALOG_ACCUSE_OF_CHEATING")
                    :GoTo("STATE_ACCUSE")
            end
        end)

    :State("STATE_TAKE_WINNINGS")
        :Loc{
            DIALOG_TAKE_MONEY = [[
                agent:
                    !angry
                    Take your money, before I tear up sayin' goodbye.
                    !give
                player:
                    !take
                    Why thank you. And thanks for the clean, honest fun.
                agent:
                    !bashful
                    $miscEmbarrassed
                    Easy for you to say, of course.
                    !exit
                * {agent} leaves, staring ruefully at {agent.hisher} coin.
            ]],
            DIALOG_EXCUSE = [[
                player:
                    !happyThanks
                    $happyThanks
                    So, I'll just take what's mine and be on my way, huh?
                agent:
                    !bashful
                    Heh. Well, you see...
                    Now don't get mad, but...
                player:
                    !dubious
                    $neutralDubious
                    Don't tell me. You're a little short.
                agent:
                    !scared_shrug
                    $miscEmbarrassed
                    I'd say I'm pretty average in height, actually.
                player:
                    !angry
                agent:
                    $scaredStammering
                    But I can only give you {1#money}. You understand, right?
            ]],
            OPT_TAKE_SMALLER_AMOUNT = "Take {1#money}",
            DIALOG_TAKE_SMALLER_AMOUNT = [[
                player:
                    !palm
                    Guess there's no harm done if you just give me what you can.
                agent:
                    !give
                    $scaredStammering
                    R-right? No harm done! So no hard feelings, right?
                    !exit
                * {agent} scuttles away before you can change your mind.
            ]],

            OPT_BEAT_UP = "Beat up {agent}",
            DIALOG_BEAT_UP = [[
                player:
                    !angry
                    $angryDefault
                    Where I'm from, you're only as good as your word. 
                    !fight
                    What you can't pay in shills I'll take out of your hide, hm?
            ]],
            DIALOG_POST_FIGHT_DEAD = [[
                * {agent} is dead. You should probably feel at least a little bad about that.
                player:
                    !sigh
                    I hate Murder Bay.
                * Hopefully the next person who crosses your path will bring better fortunes for the both of you.
            ]],

            DIALOG_POST_FIGHT_ALIVE = [[
                agent:
                    !injured
                * You pull your last punch, but that doesn't stop {agent} from flinching.
                player:
                    !angry
                    $angryDefault
                    Whaddya know. Looks like it's <i>your</i> lucky day after all. 
                    !angry_accuse
                    Now get outta here, and don't cheat someone you can't take in a fight. 
                agent:
                    !exit
                * {agent} does indeed get outta there. Whether they'll take your words to heart, only the future can say.
            ]],
            OPT_LET_GO = "Take the smaller amount",
            DIALOG_LET_GO = [[
                player:
                    !palm
                    Guess there's no harm done if you just give me what you can.
                agent:
                    !give
                    R-right? No harm done! So no hard feelings, right?
                    !exit
                * {agent} scuttles away before you can change your mind.
            ]]

        }
        :Fn(function(cxt) 
            
            if cxt.enc.scratch.amt_to_win < 150 and math.random() < .5 then
                cxt:Dialog("DIALOG_TAKE_MONEY")
                cxt.enc:GainMoney( cxt.enc.scratch.amt_to_win )
                StateGraphUtil.AddLeaveLocation(cxt)
            else

                local small_amt = math.floor(cxt.enc.scratch.amt_to_win/2)
                cxt:Dialog("DIALOG_EXCUSE", small_amt)
                cxt:Opt("OPT_LET_GO", small_amt)
                    :ReceiveMoney(small_amt)
                    :Dialog("DIALOG_LET_GO")
                    :Travel()

                cxt:Opt("OPT_BEAT_UP")
                    :Dialog("DIALOG_BEAT_UP")
                    :Battle{
                        on_win = function(cxt) 
                            cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_POST_FIGHT_DEAD" or "DIALOG_POST_FIGHT_ALIVE" )
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }
            end
        end)

    :State("STATE_ACCUSE")
        :Loc{
            OPT_LET_GO = "Let {agent} go",
            DIALOG_LET_GO = [[
                player:
                    !angry_accuse
                    Sod off. But remember this face, 'cause I'm not gonna play your fool again.
                agent:
                    !placate
                    Hey, you got it. You're nobody's sucker.
                * {agent} leaves, looking for an easier target.
            ]],


            OPT_NEGOTIATE_PAYOFF = "Demand {agent} give you {1#money}",
            DIALOG_NEGOTIATE_PAYOFF = [[
                player:
                    !angry_accuse
                    Cut me in, and your secret's safe with me.
                    Otherwise? I got a good memory, and a penchant for sullyin' otherwise unimpeachable reputations.
            ]],
            DIALOG_NEGOTIATE_PAYOFF_FAIL = [[
                player:
                    !crossed
                    I'll tell everyone what you're up to.
                    Then the only toss up will be your butt over a cliff. 
                agent:
                    !chuckle
                    Hah! I've been husslin' for longer than you've been on dry land, derrick.
                    !handwave
                    You don't scare me. Forget it.
            ]],
            DIALOG_NEGOTIATE_PAYOFF_SUCCESS = [[
                player:
                    !neutralDirect
                    $neutralDirect
                    My word is stronger than my arm, and you <i>don't</i> wanna see how strong <i>that</i> is.
                agent:
                    !angryResigned
                    $angryResigned
                    Ach, fine! I have it too good to give this up. 
                    !give
                player:
                    !take
                    Your secret's safe with me.
            ]],


            OPT_FIGHT_PAYOFF = "Beat {agent} up",
            DIALOG_FIGHT_PAYOFF = [[
                player:
                    !fight
                    How about I just beat it out of you?
            ]],
            DIALOG_FIGHT_PAYOFF_AGENT_DEAD = [[
                * {agent} is dead. You pick up the stray change off the ground and continue on your way.
            ]],
            DIALOG_FIGHT_PAYOFF_AGENT_ALIVE = [[
                agent:
                    !scared
                    $scaredStammering
                    O-okay! I'll deal you in! Just don't hurt me no more!
                player:
                    Good move. Smarts are better than luck any day.
                agent:
                    !exit
                * {agent} scurries off to lick {agent.hisher} wounds.
            ]],


            OPT_BACK_DOWN_PAYOFF = "Let {agent} go",
            DIALOG_BACK_DOWN_PAYOFF = [[
                player:
                    !palm
                    Ugh, this is just a headache. Get lost already, will ya?
                agent:
                    !angry_shrug
                    $angryReluctant
                    My pleasure. 
                    !exit
                * {agent}'s already gone by the time you look up from your sudden headache.
            ]],

            OPT_BEAT_UP = "Beat {agent} to teach them a lesson",
            DIALOG_BEAT_UP = [[
                player:
                    How about I give you a free lesson in probability?
                    !fight
                    I'm <i>probably</i> gonna beat your head in!
            ]]
        }
        :SetLooping()

        :Fn(function(cxt)
            local PAYOFF_AMT = 100
            if cxt:FirstLoop() then
                cxt:GetAgent():SetMoney(PAYOFF_AMT*2)
            end

            cxt:Opt("OPT_LET_GO")
                :Dialog("DIALOG_LET_GO")
                :Travel()
            
            cxt:Opt("OPT_NEGOTIATE_PAYOFF", PAYOFF_AMT)
                :Dialog("DIALOG_NEGOTIATE_PAYOFF")
                :Negotiation{
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_NEGOTIATE_PAYOFF_SUCCESS")
                        cxt.encounter:GainMoney( PAYOFF_AMT )
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATE_PAYOFF_FAIL")
                    end,
                }
            cxt:Opt("OPT_BEAT_UP")
                :Dialog("DIALOG_BEAT_UP")
                :Battle{
                    on_win = function(cxt)
                        cxt:Dialog( cxt:GetAgent():IsDead() and "DIALOG_FIGHT_PAYOFF_AGENT_DEAD" or "DIALOG_FIGHT_PAYOFF_AGENT_ALIVE")
                        if not cxt:GetAgent():IsDead() then
                            cxt.encounter:GainMoney( PAYOFF_AMT )
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,    
                }

        end)


