local PIT_DAMAGE_MIN, PIT_DAMAGE_MAX = 5, 14

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest)
        return TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):Get() > PIT_DAMAGE_MAX and TheGame:GetGameState():GetCaravan():CanOwnPet()
    end,
}

:AddCast{
    cast_id = "pup",
    when = QWHEN.MANUAL,
}


QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_CONFRONT"
    end)

    :Loc{
        DIALOG_SAVED = [[
            * Back on dry land, the pup finds a clean patch of skin above your brow to lick with its scratchy tongue.
        ]],
        DIALOG_SAVED2 = [[
            pup:
                !happy_bark
                Yap!
            player:
                !tired
                Eghhh. 
            * The pup cocks its head in concern.
            player:
                Naw, it's fine. Maybe just keep it to yourself next time. 
            pup:
                !bark
                Yorp?
            player:
                !agree
                Yeah, you're coming with me. 
                $neutralDubious
                Unless you'd prefer to stay here in the tar pits?
            pup:
                Norp!
            player:
                !happy_call_pet
                That's what I thought. Let's go, then. 
        ]],        
    }

    :State("STATE_CONFRONT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * As you're passing by a bubbling tar pit, you hear a high-pitched whine from somewhere nearby.
                player:
                    !dubious
                * Looking around, you spot a lone crayote pup struggling in the clotted black mud. 
                pup:
                    !right
                    !bark
                    Awoooo.

                *** A crayote pup is struggling. Do you save it?
            ]],
            OPT_RESCUE = "Rescue the crayote pup",
            DIALOG_SAVE = [[
                player:
                    !scared
                    $scaredFearful
                    I'm coming to get you, buddy. Don't be scared.
                * You anchor yourself to a nearby stump before stepping carefully into the tar.
                * Freeing the pup is a labor, and you choke down the sour air of the tar pit under the exertion.
            ]],
            OPT_IGNORE = "Ignore the whining pup",
            DIALOG_IGNORE = [[
                * You wouldn't wanna get caught by its mother, ankle deep in tar. 
                * Once your back is turned, the yote pup gives a last, panicked yelp. 
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                local agent = Agent("CRAYOTE")
                TheGame:GetGameState():AddAgent( agent )
                cxt.enc:SetPrimaryCast(agent)
                cxt.quest:AssignCastMember("pup", agent)
                AUDIO:PlayPetSounds(cxt.quest:GetCastMember("pup"), "WHIMPER")
                cxt:Dialog("DIALOG_INTRO")
                cxt.enc.scratch.damage = math.random( PIT_DAMAGE_MIN, PIT_DAMAGE_MAX )
            end

            cxt:Opt("OPT_RESCUE", cxt.enc.scratch.damage)
                :PostText("TT_PET_TAKE")
                :PreIcon(global_images.invite)
                :Dialog("DIALOG_SAVE")
                :DeltaHealth(-cxt.enc.scratch.damage)
                :Fn(function(cxt)
                    cxt:GetAgent():GenerateRandomName()
                end)
                :Dialog( "DIALOG_SAVED" )
                :Fn(function(cxt) AUDIO:PlayPetSounds(cxt.quest:GetCastMember("pup"), "SCARED_YELP") end)
                :Dialog( "DIALOG_SAVED2" )
                :RecruitMember(PARTY_MEMBER_TYPE.CREW)
                :Fn(function(cxt) cxt.quest:Complete() end)
                :Travel()

            cxt:Opt("OPT_IGNORE")
                :PreIcon(global_images.close)
                :Dialog("DIALOG_IGNORE")
                :GoTo("STATE_PUP_IGNORED")
        end)

    :State("STATE_PUP_IGNORED")
        :Loc{
            OPT_RECONSIDER = "Go back and save the damn pup",
            DIALOG_RECONSIDER = [[
                player:
                    Okay, okay. I'm coming.
                * The pup shivers in the muck, but manages a pitiful wag of its tail as you approach.
            ]],
            OPT_ABANDON = "Abandon the pup to its fate",
            DIALOG_ABANDON = [[
                player:
                    !shrug
                pup:
                    !exit
                * The road winds, and it ain't too long before the sounds of the bubbling muck—and the distressed pup—are far behind you.
            ]],
        }

        :Fn(function(cxt)
            cxt:Opt("OPT_RECONSIDER", cxt.enc.scratch.damage)
                :PostText("TT_PET_TAKE")
                :Dialog("DIALOG_RECONSIDER")
                :DeltaHealth(-cxt.enc.scratch.damage)
                :Fn(function(cxt)
                    cxt:GetAgent():GenerateRandomName()
                end)
                :Dialog( "DIALOG_SAVED" )
                :RecruitMember(PARTY_MEMBER_TYPE.CREW) 
                :Fn(function( cxt )
                    cxt.quest:Complete()
                end)
                :Travel()

            cxt:Opt("OPT_ABANDON")
                :Dialog("DIALOG_ABANDON")
                :Fn(function(cxt) 
                    AUDIO:PlayPetSounds(cxt.quest:GetCastMember("pup"), "WHIMPER")
                    cxt.quest:Complete()
                end)
                :Travel()
        end)