local QDEF = QuestDef.Define{
    title = "Pet Seller",
    qtype = QTYPE.EVENT,
    cooldown = EVENT_COOLDOWN.LONG,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest) 
        return TheGame:GetGameState():GetCaravan():CanOwnPet()
    end,
}

:AddCast{
    cast_id = "vroc",
    when = QWHEN.MANUAL,
}

QDEF:AddConvo()

    :Confront(function(cxt, who)
        if cxt.location:HasTag("in_transit") then
            return "STATE_MERCHANT"
        end
    end)

    :State("STATE_MERCHANT")
        :Loc{
            DIALOG_INTRO = [[
            * Your path is blocked by a ragged and breathless merchant.
                player:
                    !left
                agent:
                    !right
                    !cagey
                    $neutralTakenAback
                    Hey. You wanna buy a vroc? Real cheap. Just, ah, don't ask where it came from.
                    Or if it's healthy.
                    Or trained.
                    !shrug
                    $neutralDubious
                    But it's yours if you have the shills.
                *** {agent} asks if you would like to buy a stray vroc.
            ]],
            OPT_ASK_ABOUT_VROC = "Ask about the vroc",
            OPT_TAKE_IT = "Buy the Vroc",
            DIALOG_ASK_ABOUT = [[
                player:
                    !interest
                    $neutralDubious
                    So is it...
                agent:
                    I said don't ask about it!
                player:
                    !dubious
                    But—
                agent:
                    No!
                    !crossed
                    Just take it or leave it.
            ]],
            DIALOG_PURCHASE = [[
                player:
                    !give
                    $neutralResigned
                    What can I say, I'm a sucker for animals.
                agent:
                    !surprised
                    $happyTakenAback
                    Huh. Well... great. Let me go get it.
            ]],
            OPT_DEMAND_THE_VROC = "Convince {agent} to hand over the vroc",

            OPT_DECLINE = "Decline the offer",
            DIALOG_DECLINE = [[
                player:
                    !placate
                    $miscRelieved
                    No thanks. This seems a little fishy to me.
                agent:
                    !angry_shrug
                    $angryPatienceLost
                    It's <i>not</i> a fish. It's a vroc. Thanks for wasting my time.
                    !exit
                * {agent} skulks off, muttering angrily. 
            ]],
            DIALOG_HAND_OVER = [[
                player:
                    !shrug
                    $miscPersuasive
                    Something tells me the vroc is more trouble than you bargained for. Why not just hand it over?
            ]],
            DIALOG_WON_NEGOTIATION = [[
                player:
                    !interest
                    $miscPersuasive
                    It must be giving you a lot of hassle. Is this really worth your time?
                agent:
                    !scared_shrug
                    $neutralResigned
                    To be honest, it's a menace. 
                    Honestly, you'd be doing me a favor.
            ]],
            DIALOG_FAILED_NEGOTIATION = [[
                player:
                    Really, you should be <i>paying me</i> to rid you of it. 
                agent:
                    !angry
                    $angryPatienceLost
                    No, and now it just go more expensive.
            ]],
            REASON_DEMAND = "Demand {agent} hand over the vroc",
            SIT_MOD = "Doesn't want to give it up for free",
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                local agent = AgentUtil.GetFreeAgent("LABORER", function(agent) return agent:GetRelationship() >= RELATIONSHIP.NEUTRAL end)
                agent:MoveToLocation(cxt.location)
                cxt.encounter:SetPrimaryCast( agent )

                cxt.quest.param.vroc_cost = CalculatePayment( agent, 100, 10 )
 
                cxt:Dialog("DIALOG_INTRO")
            end
            
            cxt:Question("OPT_ASK_ABOUT_VROC", "DIALOG_ASK_ABOUT")

            cxt:Opt("OPT_DEMAND_THE_VROC")
                :Dialog("DIALOG_HAND_OVER")                
                :Negotiation{
                    reason = "REASON_DEMAND",
                    situation_modifiers =
                    {
                        { value = 10, text = cxt:GetLocString("SIT_MOD") }
                    },
                    on_success = function() 
                        cxt:Dialog( "DIALOG_WON_NEGOTIATION" )
                        cxt:GoTo("STATE_GET_VROC")
                    end,
                    on_fail = function()
                        cxt:Dialog("DIALOG_FAILED_NEGOTIATION")
                        cxt.quest.param.vroc_cost = 200 
                    end }

            cxt:Opt("OPT_TAKE_IT")
                :PostText("TT_PET_TAKE")
                :DeliverMoney(cxt.quest.param.vroc_cost)
                :PreIcon( global_images.buying )
                :Dialog("DIALOG_PURCHASE")
                :GoTo("STATE_GET_VROC")

            cxt:Opt("OPT_DECLINE")
                :PreIcon( global_images.close )
                :Dialog("DIALOG_DECLINE")    
                :Travel()
        end)
    
    :State("STATE_GET_VROC")
        :Loc{
            DIALOG_TAKE_IT = [[
                agent:
                    !exit
                * {agent} kicks the covered cage to jostle the vroc, then wrestles it into a lead. When {agent.heshe} returns, both {agent} and the vroc are looking sour.
                agent:
                    !right
                    !spit
                    Here you go. One mongrel.
                vroc:
                    !right
                * The vroc looks at you quizzically, and then pees. {agent} shuffles uncomfortably as the puddle spreads toward {agent.hisher} boots.
                ]],
            DIALOG_TAKE_IT2 = [[
                agent:
                    !right
                    Well, uh. Enjoy, I guess.
                    !exit
                vroc:
                    !right
            ]],
            DIALOG_TAKE_IT3 = [[
                player:
                    !dubious
                * You wonder if this was a good idea.
            ]],
        }
        :Fn(function(cxt)
            local agent = Agent("VROC")
            agent:GenerateRandomName()
            TheGame:GetGameState():AddAgent( agent )
            cxt:ReassignCastMember("vroc", agent)
            cxt.quest:AssignCastMember("vroc", agent)
            cxt:Dialog("DIALOG_TAKE_IT")
            AUDIO:PlayPetSounds(cxt.quest:GetCastMember("vroc"), "EXCITED_BORK")
            cxt:Dialog("DIALOG_TAKE_IT2")
            AUDIO:PlayPetSounds(cxt.quest:GetCastMember("vroc"), "AFFIRMATIVE_YAP")
            cxt:Dialog("DIALOG_TAKE_IT3")
            agent:Recruit(PARTY_MEMBER_TYPE.CREW)
            cxt.quest:Complete()
            StateGraphUtil.AddLeaveLocation(cxt)

        end)