local QDEF = QuestDef.Define
{
    title = "Hesh Outpost",
    icon = engine.asset.Texture("icons/quests/sal_hesh_outpost.tex"),
    home = true, -- If this shows a yellow-colored icon, with a yellow arrow under it
    qtype = QTYPE.STORY,

    plot_armour_fn = function(quest, agent)
        if agent == quest:GetCastMember("priest") or agent == quest:GetCastMember("luminari") then
            return true
        end
    end,
    
    --Want to keep them here. maybe these should be jobs instead?
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("luminari"), location = quest:GetCastMember("home"), role = CHARACTER_ROLES.GUARD } )
        table.insert (t, { agent = quest:GetCastMember("priest"), location = quest:GetCastMember("home"), role = CHARACTER_ROLES.PROPRIETOR } )
    end,

    events = {
        phase_change = function(quest)
            if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY then
                quest.param.blessing_available = true
            end
        end,
    },
    on_init = function (quest )
        quest.param.blessing_available = true
    end


}

:AddLocationCast{
    cast_id = "home",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MURDER_BAY_HESH_OUTPOST"))
    end,
}

:AddCastByAlias{
    cast_id = "priest",
    alias = "HESH_OUTPOST_PRIEST",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "luminari",
    alias = "HESH_OUTPOST_LUMINARI",
    no_validation = true,
}

:Loc{
    VISIT = [[Visit the Hesh Outpost daily for a blessing or sparring.]],
    ACTIVITY_AVAILABLE = [[<#BONUS>Activity available!</>]],
    ACTIVITY_UNAVAILABLE = [[<#PENALTY>Activity unavailable until tomorrow.</>]],
}

:AddObjective{
    id = "visit",
    title = "Visit the Hesh Outpost",
    loc =
    {

    },
    desc = "",
    desc_fn = function(quest, str)
        local tt = { quest:GetLocalizedStr( "VISIT" ) }
        if quest.param.blessing_available then
            table.insert( tt, quest:GetLocalizedStr( "ACTIVITY_AVAILABLE" ) )
        else
            table.insert( tt, quest:GetLocalizedStr( "ACTIVITY_UNAVAILABLE" ) )
        end
        return table.concat( tt, "\n" )
    end,

    hide_in_overlay = true,
    low_priority = true,
    state = QSTATUS.ACTIVE,
    mark = {"home"}
}



QDEF:AddConvo("visit", "priest")
    :Priority(CONVO_PRIORITY_LOW)
    :Loc{

        OPT_GET_BLESSING = "Get a blessing",
        DIALOG_GET_BLESSING = [[
            player:
                !interest
                You got any holiness to spare?
            agent:
                Are you asking for a blessing?
            player:
                Yeah, why not. Couldn't hurt, right?
            
        ]],
        DIALOG_GET_BLESSING_2 = [[
            player:
                !surprised
                $scaredTakenAback
                Yeowch! Did you... pinch me?!
            agent:
                !shrug
                Hesh has the stingers of a jellyfish. Its touch doesn't come without consequence.
            player:
                Great. But... huh. 
                !thought
                $neutralThoughtful
                I <i>do</i> feel different.
                !happy
                $happyThanks
                Thanks!
        ]],
        REQ_AT_FULL_RESOLVE = "You are at full resolve",
        REQ_ALREAY_USED = "You've already performed an activity at the Hesh Outpost today.",
    }

    :Hub( function(cxt, who)

            cxt:Opt("OPT_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")

            local resolve, max_resolve = cxt.caravan:GetResolve()
            cxt:Opt("OPT_GET_BLESSING")
                :SetQuestMark(cxt.quest)
                :ReqCondition(resolve < max_resolve, "REQ_AT_FULL_RESOLVE")
                :ReqCondition(cxt.quest.param.blessing_available, "REQ_ALREAY_USED")
                :Dialog( "DIALOG_GET_BLESSING" )
                :DeltaResolve(15)
                :Dialog( "DIALOG_GET_BLESSING_2" )
                :Fn(function() 
                    cxt.quest.param.blessing_available = false
                end)
        end
    )

    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask {agent} about this place",
        [[
            player:
                !interest
                $neutralDubious
                So what is it you two are doing here anyway?
            agent:
                $neutralDirect
                That's none of your concern.
            player:
                !angry_shrug
                Ugh, I hate it when people say that. 
                $neutralResigned
                Fine, keep your secrets.
            agent:
                !dubious 
                Hesh admires a meddler, {player}. But I don't. 
                Just mind your own business and we can stay unlikely allies. 
        ]],

        "Ask about {luminari}",
        [[
            player:
                !interest
                Tell me about {luminari}.
            agent:
                !crossed
                What do you wish to know? {luminari.HeShe}'s a Luminari, one of the Cult's trained enforcers.
                {luminari} comes by {luminari.hisher} faith honestly, unlike some of the clerical members of the church. 
            player:
                !handwring
                $happyDefault
                Oooh, that sounds like you have an axe to grind.
            agent:
                I shan't name names, of course.
            player:
                !forgetful
                Tsk. 
            agent:
                Luminari are the most diligent of the throng. They go where priests cannot, and do what needs to be done.
                !dubious
                $neutralResigned
                I think {luminari} wishes, however, that {luminari.heshe} had a different lot. 
                {luminari.HeShe} wonders what {luminari.heshe} would look like in priestly robes, I think. 
            player:
                !interest
                Is that bad?
            agent:
                !shrug
                Only when it prevents {luminari.himher} from doing the work a Luminari is meant for. 
                !agree
                If we succeed here, though⁠—it might just be possible.
        ]],

        "Ask about holy relics",
        [[
            player:
                !interest
                I'm still not entirely sure what the deal is with this auction. 
                $neutralDubious
                My parents dug up relics in Grout Bog for a living. But I never once saw anything I'd call holy.
            agent:
                $angryReluctant
                The Spark Barons wish to return to the star-faring ways of the Vagrant Age.
                !angry_Shrug
                So they only rebuild what they find, rather than truly learn from it. 
                !interest
                Perhaps a small lesson is called for, if you can stomach the sermon.
            player:
                !agree
                Enh, go on, then. 
            agent:
                !overthere
                $miscAlluring
                The relics of the Vagrant Age are from a time before the Abyss.
                The waters of fate were stagnant, and the star-faring people grew fetid.
            player:
                $neutralDubious
                Seems odd, considering they were free to leave the planet. 
            agent:
                !disappoint
                Their bodies were free. But spiritually, they were bound. 
                We keep our chains close because they remind us. But also they give us power.
                !eureka
                Because, after all, it was something the star-farers did that <i>woke</i> Hesh from the Abyss in the first place.
            player:
                !dubious
                $neutralTakenAback
                What? You don't mean literally. Hesh doesn't <i>actually</i> live out in the ocean.
            agent:
                !crossed
                The ocean runs all the way to the planet's core. 
                And yet the star-farers did <i>something</i> to stir the slumbering Hesh.
                $miscAlluring
                There may be other dormant gods waiting for a similar upheaval. 
                !interest
                I, for one, would be interested in knowing how to wake them. 
            player:
                !forgetful
                Wow. That's a tall order. 
        ]],

    })


QDEF:AddConvo("visit", "luminari")
    :Priority(CONVO_PRIORITY_LOW)
    :Loc{
        OPT_DO_SPARRING = "Spar with {luminari}",
        DIALOG_DO_SPARRING = [[
            player:
                !greeting
                $neutralGreeting
                Hey, you must be getting antsy out here all alone in the woods with only Stick-in-the-Mud for company.
                $miscAlluring
                Wanna spar?
            agent: 
                !hesh_greeting
                I must be vigilant, for I never know where Hesh will see fit to deliver me.
            player:
                !dubious
                ...
            agent:
                !interest
                That's a "yes", of course.
            player:
                !exit
            agent:
                !exit
            * You and {luminari} spar across a patch of dry dirt. It's hard work, but you both learn something before it's through.                           
        ]],
        DIALOG_DO_SPARRING_2 = [[
            player:
                !left
                !injured
            agent:
                !right
                !injured
            * When it's over, you each take a moment to collect yourselves.         
            player:
                !neutral 
            agent:
                !neutral
            player:
                !happy
                $happyThanks
                Next time, you might just best me.
            agent:
                !happy
                I look forward to it. Until then, {player}.
                And I know you hate hearing this...
            player:
                !palm
                Here it comes.
            agent:
                !wink
                $miscMischievous
                But may you walk in the shallows.
                
        ]],
        REQ_HEALTH_TOO_LOW = "You are too weak. It would kill you!",
        REQ_ALREAY_USED = "You've already performed an activity at the Hesh Outpost today.",
        REQ_NOTHING_TO_UPGRADE = "All of your battle cards are already upgraded",
        CARD_TRAINED = "{1#card} gained {2} XP",
        CARD_TRAINED_READY = "{1#card} gained {2} XP <#BONUS>Upgrade ready!</>",
        TT_SPARRING = "This will gain up to {1} random battle card XP",
        CARD_TRAINED_SUMMARY = "Total XP Gained: {1}",
    }

    :Hub( function(cxt, who)

            cxt:Opt("OPT_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")

            local candidates = {}
            local cards = TheGame:GetGameState():GetPlayerAgent().battler:GetCards()
            for i=1, #cards do
                if cards[i]:HasUpgrades() and not cards[i]:UpgradeReady() and not cards[i]:IsHatched() and cards[i]:GetXP() then
                    table.insert(candidates, cards[i])
                end                        
            end
            local DMG = 5
            local NUM_UPS = 5
            local health = TheGame:GetGameState():GetPlayerAgent():GetHealth()

            local resolve, max_resolve = cxt.caravan:GetResolve()
            cxt:Opt("OPT_DO_SPARRING")
                :SetQuestMark(cxt.quest)
                :PostText("TT_SPARRING", NUM_UPS)
                :ReqCondition(health > DMG, "REQ_HEALTH_TOO_LOW")
                :ReqCondition(cxt.quest.param.blessing_available, "REQ_ALREAY_USED")
                :ReqCondition(#candidates > 0 , "REQ_NOTHING_TO_UPGRADE")
                :Dialog( "DIALOG_DO_SPARRING" )
                :DeltaHealth(-DMG)
                :Fn(function() 
                    local txt = {}
                    local results = {}
                    table.shuffle(candidates)
                    local total_xp = 0
                    for k = 1, NUM_UPS do
                        if #candidates <= 0 then
                            break
                        else
                            local card = table.arraypick(candidates)
                            card:AddXP(1)
                            if card:UpgradeReady() then
                                table.arrayremove(candidates, card)
                            end
                            results[card] = (results[card] or 0) + 1
                            total_xp = total_xp + 1
                        end
                    end
                    local locstr = cxt:GetLocString("CARD_TRAINED_SUMMARY", total_xp)
                    table.insert( txt, locstr )
                    for card, xp in pairs(results) do
                        
                        if card:UpgradeReady() then
                            local locstr = cxt:GetLocString("CARD_TRAINED_READY", card.id, xp)
                            table.insert( txt, locstr )
                        else
                            local locstr = cxt:GetLocString("CARD_TRAINED", card.id, xp)
                            table.insert( txt, locstr )
                        end
                    end
                    
                    cxt.enc:ShowPreformattedText(table.concat(txt, "\n"))


                end)
                :Dialog( "DIALOG_DO_SPARRING_2" )
                :Fn(function() 
                    cxt.quest.param.blessing_available = false
                end)
        end
    )


    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask {agent} about this place",
        [[
            player:
                !interest
                So... what's the deal with this outpost of yours? 
            agent:
                It's an outpost.
            player:
                !agree
                $neutralDubious
                Uh huh. Does this have something to do with the auction?
                Like, are you keeping a treasure hoard here or...?
            agent:
                !crossed
                $angryDefault
                Let's just say I won't suffer even the slightest Heshian trinket falling into the hands of the Spark Barons.
                Some of us keep the faith, even when a treaty would have us do otherwise. 
            player:
                !chuckle
                Ah, I see. So you're stirring the pot.
                $neutralJoke
                The pot that the Admiralty have kept at a nice simmer for twenty years or so.
            agent:
                !agree
                Ye—no. 
                $scaredStammering
                I mean no.
                !placate
                It's more than that. But also... it's nothing. 
                $neutralDirect
                Mind your own business!
            player:
                !dubious
                $neutralDubious
                Uh huh.
        ]],

        "Ask about {priest}",
        [[
            player:
                What can you tell me about {priest}? 
                !chuckle
                {priest.HeShe}'s kind of a hardass, huh?
            agent:
                !surprised
                $scaredTakenAback
                I, uh—this is not—
            player:
                !wink
                $miscAlluring
                Oh, come on, {agent}. We're friends, aren't we?
                $miscPersuasive
                If you can't vent to me, who <i>can</i> you talk to?
            agent:
                !disappoint
                ...
                $neutralDefault
                {priest} is extremely principled. I would be honored to have half {priest.hisher} conviction. 
                !eureka
                But {priest.heshe} doesn't always believe in my potential. So... I have much to prove.
            player:
                !dubious
                Huh. 
            agent:
                You want me to speak and think poorly of {priest.himher}, I know. 
                !permit
                And I understand why. You get strength from opposing those who are powerful.
                But I'm made stronger by {priest}'s resolve. 
        ]],

        "Ask about Hesh",
        [[

            player:
                !bashful
                $miscEmbarrassed
                Look, I don't believe in this Hesh thing. Kinda spent too much of my life grit-deep in lumin for that. 
                But it must be nice to have so much... guidance.
            agent:
                !cruel
                $neutralJoke
                You're not the only heathen to presume so. But Hesh is not a god of purpose.
                Hesh's whims are like tides, and we are but flotsam at their mercy.
                !agree
                Only when you accept this can you truly be free.
            player:
                !wince2
                $neutralWhatever
                Hesh's "whims" tried to keep me on the derrick, sucking up sea sludge. 
                I think it's safe to say that way of life isn't for me.
            agent:
                !interest
                Perhaps now. But maybe not always.
                Hesh is eternal, so it can afford to wait.
        ]],

             
    })
