local PAYMENT_AMT = 50

local function CleanUpFn(quest)
    if quest:GetCastMember("luminari") and quest:GetCastMember("luminari"):IsInPlayerParty() then
        quest:GetCastMember("luminari"):Dismiss()
    end
end


local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,

    precondition = function(quest)
        return QuestUtil.CanTakeSideJob()
    end,

    icon = engine.asset.Texture("icons/quests/sal_hesh_outpost.tex"),
    home = true, -- If this shows a yellow-colored icon, with a yellow arrow under it

    on_fail = CleanUpFn,
    on_complete = CleanUpFn,
    on_cancel = CleanUpFn,
}

:AddCastByAlias{
    cast_id = "luminari",
    alias = "HESH_OUTPOST_LUMINARI",
    events = 
    {
        agent_retired = function(quest) 
            quest:Fail()
        end,
    }
}

:AddCastByAlias{
    cast_id = "priest",
    alias = "HESH_OUTPOST_PRIEST",
    events = 
    {
        agent_retired = function(quest) 
            quest:Fail()
        end,
    }
}

:AddObjective{
    id = "intro",
    state = QSTATUS.ACTIVE,    
    on_complete = function(quest) 
    end
}

:AddLocationCast{
    cast_id = "outpost",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MURDER_BAY_HESH_OUTPOST"))
    end,
}


:AddObjective{
    id = "deliver_spark",
    title = "Help {luminari} deliver the spark",
    desc = "{luminari} has hired you to help carry some spark for {luminari.himher}.",
    mark = { "outpost" },
    on_activate = function(quest)
        quest:GetCastMember("luminari"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
    end,
}

:AddOpinionEvents{
    defended_spark = 
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Kept spark out of Baron hands",
    },  
    did_not_serve_hesh = 
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Did not serve the Whim of Hesh",

    },
    gave_up_spark =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Gave up spark to the Spark Barons",
    },
}


QDEF:AddConvo("intro")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("FOO", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                * You happen upon a Luminari standing off to the side of the road, deep in thought.
                player:
                    !left
                agent:
                    !right
                    !hesh_greeting
                    $neutralDirect
                    Greetings, grifter. 
                    I need your help, and am at your mercy unless you give it. 
                    !overthere
                * {agent} motions toward a pile of dirty canisters, one of them leaking a noxious orange fluid.
                agent:
                    !wince2
                    $neutralDubious
                    I require assistance carrying this... <i>spark</>. 
                    Surely you have time to assist the will of Hesh.
                    !dubious
                    $neutralDubious
                    Or don't you walk in the shallows?
                    
            ]],


            OPT_ASK_WHATS_IN_IT = "Ask what's in it for you",
            DIALOG_ASK_WHATS_IN_IT = [[
                player:
                    !hips
                    $miscMocking
                    I don't like to get my boots wet. So what's in it for me, Luminari?
                agent:
                    !disappoint
                    $neutralResigned
                    Ah, a heathen. I should've expected as much in this Heshless part of the griftlands.
                    !handwave
                    $neutralResigned
                    Very well. Hesh can also provide {1#money}.
                player:
                    !happy
                    $neutralJoke
                    So kind of Hesh to offer. 
                agent:
                    !interest
                    $miscAlluring
                    Hesh can offer enrichment of the <i>soul</i>, if you prefer.
                player:
                    $neutralDirect
                    Pass. 
            ]],

            OPT_ASK_WHY_NEED_HELP = "Ask why {agent.heshe} can't do it",
            DIALOG_ASK_WHY_NEED_HELP = [[
                player:
                    !dubious
                    $neutralDubious
                    There isn't much of it. Why do you need help at all?
                    I thought Luminari were the best and boldest of the throng. 
                agent:
                    !permit
                    Indeed. And as such, I'm prohibited from touching that which is an abomination unto Hesh.
                    !angry_accuse
                    Spark is a nefarious sludge, the fruit of a wicked land.
                player:
                    I've heard it works just as good as lumin. And comes out of the ground with a simple machine. 
                    !cruel
                    $miscMocking
                    Or is it only <i>holy</i> if it's harvested by unpaid labor?
                agent:
                    !disappoint
                    $neutralResigned
                    I've offered you a job, grifter. I don't appreciate having my time wasted.
            ]],

            OPT_AGREE_TO_CARRY = "Agree to help carry the spark",
            DIALOG_GET_ACCESS = [[
                player:
                    !thumb
                    $neutralJoke
                    Saddle me up and call me a pack mule.
                    Wouldn't be the first time.
                agent:
                    !hesh_greeting
                    Thank you—although you may be a heathen, your actions will ripple through the tides.
                    Come with me.
                * The Luminari heads for the road, leaving you to follow with the spark in tow.
                    !exit
            ]],

            OPT_DECLINE = "Refuse",
            DIALOG_DECLINE = [[
                player:
                    !angry
                    I spent years lugging lumin for the Cult.
                    !point
                    I'm through doing your dirty work, Heshian.
                agent:
                    ...
                    !sigh
                    I suppose that's—
                * Whatever {agent} was going to say, {agent.heshe} clearly thinks better of it. 
                    !angry
                    Fine. Then be gone. 
                * {agent} turns back to the spark. 
            ]],
        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:GetCastMember("luminari"):MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("luminari"))
                cxt.quest:Complete("intro")
                cxt:Dialog("DIALOG_INTRO")  
            end

            cxt:Opt("OPT_ASK_WHY_NEED_HELP")
                :Dialog("DIALOG_ASK_WHY_NEED_HELP")

            cxt:Opt("OPT_ASK_WHATS_IN_IT")
                :Dialog("DIALOG_ASK_WHATS_IN_IT", PAYMENT_AMT)

            cxt:Opt("OPT_AGREE_TO_CARRY")
                :Dialog("DIALOG_GET_ACCESS")
                :ReceiveMoney(PAYMENT_AMT)
                :Fn(function() 
                    cxt.quest:Activate("deliver_spark")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_DECLINE")
                :Dialog("DIALOG_DECLINE")
                :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent("did_not_serve_hesh"))
                :Fn(function(cxt) cxt.quest:Fail() end)
                :Travel()

        end)


QDEF:AddConvo("deliver_spark")
    --:Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("FOO", function(cxt) return not cxt.location:HasTag("in_transit") and cxt.location ~= cxt.quest:GetCastMember("outpost") end)
        :Loc{
            DIALOG_TOO_MANY_DELAYS = [[
                * {luminari} has had enough, approaching you impatiently.
                player:
                    !left
                luminari:
                    !right
                    !angry_accuse
                    Clearly you only accepted the work to further waste my time.
                player:
                    !interest
                    Hey, you never said it had to be done in a hurry—
                luminari:
                    !salute
                    $neutralDirect
                    We're done here, grifter. Good day. 
            ]],

            DIALOG_INTRO = [[
                * {luminari} approaches you, clearly agitated.
                player:
                    !left
                luminari:
                    !right
                    This is <i>not</i> the right direction.
                    !dubious
                    $neutralDubious
                    You've been tasked with a job, hunter, or have you forgotten?
            ]],
            OPT_OK = "Get moving",
            DIALOG_OK = [[
                player:
                    !eureka
                    $miscMocking
                    Right! Knew I Was forgetting something. Let's get on that.
            ]],
            OPT_ACTUALLY_NO = "Ditch the Luminari",
            DIALOG_ACTUALLY_NO = [[
                player:
                    $neutralDubious
                    Enh, you know what? I think I've had a better offer.
                agent:
                    !angry
                    What?! You—
                * The Luminari pauses, no doubt remembering {agent.hisher} piety. 
                    !palm
                    So be it, grifter.
                    Pray you <i>don't</i> get your boots wet, lest Hesh should drag you into the depths. 
                * {agent} turns again to consider the lump of spark.
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest.param.strikes = (cxt.quest.param.strikes or 0) + 1
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("luminari"))
            if cxt.quest.param.strikes > 2 then
                cxt:Dialog("DIALOG_TOO_MANY_DELAYS")
                cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("did_not_serve_hesh"))
                cxt.quest:Fail()
                StateGraphUtil.AddEndOption(cxt)
            else
                
                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_ACTUALLY_NO")
                    :Dialog("DIALOG_ACTUALLY_NO")
                    :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("did_not_serve_hesh"))
                    :Fn(function(cxt) cxt.quest:Fail() end)
                    :End()

                cxt:Opt("OPT_OK")
                    :Dialog("DIALOG_OK")
                    :Travel()
            end
            
        end)



QDEF:AddConvo("deliver_spark")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("FOO", function(cxt) return not cxt.quest.param.interrupted and cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                * You don't make it very far down the road before you're confronted by a Spark Baron, {baron.hisher} face pallid from exertion.
                baron:
                    !right
                    !tired 
                    $angryDefault
                    S-stop right there!
                player:
                    !left
                baron:
                    !angry_accuse
                    $angryDefault
                    Where'd you find that spark, grifter?
                player:
                    !shrug
                    I'm, uh, just holding it for a friend?
                baron:
                    !cruel
                    Hah! That Luminari there? Not likely!
                    That spark is stolen. I was transporting it to a supply station. 
                player:
                    !dubious
                    $neutralDubious
                    So how'd it end up with my Luminari friend here?
                baron:
                    !angry_accuse
                    $angryPatienceLost
                    Ask your slippery pal. I just got up to take a leak!
                    Now, the Luminari might wanna remember we got a truce in play.
                    The Heshians don't touch spark, and <i>we</i> don't touch lumin. 
                    $angryCruel
                    We both play by the rules, and we keep Havaria from all-out war, don't we?
                *** The Spark Barons want the spark that you are carrying for {luminari}. 
            ]],

            OPT_HAND_OVER = "Hand over the spark",
            DIALOG_HAND_OVER = [[
                player:
                    !give
                    $neutralResigned
                    Just take it, already. I'm not getting roped into your bloodfeud.
                baron:
                    !take
                    $angryCruel
                    Smart, grifter. Remember this next time a Heshian asks you to get your hands dirty. 
                    !exit
                * {baron} packs up the spark and heads off, leaving you alone with the surly Luminari.
                luminari:
                    !right
                    !angry
                player:
                    !placate
                    Hey, don't look at me like that. This was <i>your</i> mess, through and through.
                luminari:
                    $neutralDirect
                    I don't view my holy duty as a burden, grifter. 
                    But I do believe I'm owed a, ah—<b>refund</b>, for services you failed to render. 
            ]],
            OPT_GIVE_BACK_THE_MONEY = "Give back the money",
            DIALOG_GIVE_BACK_THE_MONEY = [[
                luminari:
                    !take
                player:
                    !give
                    No one to blame but myself for having anything to do with the Cult. 
                luminari:
                    !exit
                * {luminari} counts the money, and then leaves without saying another word.
            ]],


            OPT_REFUSE_TO_GIVE_BACK_MONEY = "Attack the Luminari",
            DIALOG_REFUSE_TO_GIVE_BACK_MONEY = [[
                player:
                    !angry_shrug
                    $angryPatienceLost
                    Are you <i>kidding</i> me? 
                    I did the job you asked. It's not my fault you're an entitled prig. 
                    $angryPatienceLost
                    You know what?
                    !fight
                    $angryHostile
                    Let's just do what we should've done from the start.
            ]],
            DIALOG_FOUGHT_LUMINARI_ALIVE = [[
                player:
                    !angry
                    $angryDefault
                    There, I even went and showed you <i>mercy</i>.
                    $angryReluctant
                    Get out of my face, Heshian. 
                luminari:
                    !injured
                    $angryReluctant
                    Hesh has seen you, grifter.
                    !exit
                * {luminari} limps off.
            ]],
            DIALOG_FOUGHT_LUMINARI_DEAD = [[
                * You're not sure that was worth the trouble. But it felt good!
            ]],

            OPT_NEGOTIATION = "Convince {baron} to back off",
            DIALOG_NEGOTIATION = [[
                player:
                    !cruel
                    $miscMocking
                    We're not in Grout Bog, Baron, and I don't see any Admiralty who can force us to play nice. 
                    Seems you're gonna have to play by different rules today.
                baron:
                    !crossed
                    And which rules would those be?
            ]],
            DIALOG_NEGOTIATION_SUCCESS = [[
                player:
                    !cruel
                    $angryCruel
                    You want to take on even a <i>lone</i> Luminari, Baron? 
                    !cruel
                    $angryCruel
                    We're alone out here. We make the rules. And right now, I don't see much of a reason to keep you alive.
                baron:
                    !scared
                    ...
                    F-fine. Keep the spark. 
                    !angry
                    There's more where that came from, Luminari. <i>A lot</i> more. 
                    $miscMocking
                    How's the lumin harvest faring, huh?
                    !exit
                * The Baron leaves, and {luminari} exhales.
                luminari:
                    !right
                    You have been licked by Hesh's blessed tongue, grifter.
                player:
                    !wince2
                    This is why I hate Heshian blessings.
                luminari:
                    Let's continue.
            ]],
            DIALOG_NEGOTIATION_FAIL = [[
                player:
                    !overthere
                    Because this is the griftlands, and... finders keepers?
                baron:
                    !chuckle
                    Is that it? Who do you think you're convincing?
                    !angry_accuse
                    $angryHostile
                    Now hand it over, else I'll have to use force to reclaim my legal property.
            ]],
            OPT_DEFEND_SPARK = "Defend the spark",
            DIALOG_DEFEND_SPARK = [[
                player:
                    Here I was beginning to think this job was boring.
                    !fight
                    You want it? Come and get it! 
            ]],
            DIALOG_FOUGHT_BARONS_LUMINARI_DEAD = [[
                agent:
                    !exit
                * The luminari is dead. You leave the spark where it fell.
                * If someone else wants to haul it through the muck, let them.
            ]],
            DIALOG_FOUGHT_BARONS_SURVIVOR = [[
                survivor:
                    !right
                    !injured
                    You win, grifter! 
                    But tell your Luminari that this slight won't go unreturned. 
                    !exit
                * The Baron hurries off, leaving you with the Luminari and your pile of spark.
                luminari:
                    !right
                    Impressive work. Though I'd have rather you hadn't left any witnesses. 
                    As is, word will get back that the Heshians struck, against treaty.
                player:
                    You paid me to haul spark, <i>not</i> to assassinate some Barons. 
                    Consider yourself lucky I was willing to take up the fight at all.
                luminari:
                    Luck has nothing to do with it, of course.
            ]],
            DIALOG_FOUGHT_BARONS_NO_SURVIVORS = [[
                luminari:
                    !right
                    It is the will of Hesh. If any of them had survived, they would've reported this.
                    My superiors would prefer the treaty remain intact for now. 
                player:
                    !interest
                    $neutralDubious
                    Then maybe they shouldn't send you specifically to break it, huh?
                luminari:
                    !sigh
                    Maybe. 
                    It's not my place to criticize their decisions. 
                player:
                    !forgetful
            ]],
            OPT_CONTINUE = "Continue On",
            OPT_DEMAND_MORE_MONEY = "Demand more money",
            DIALOG_DEMAND_MORE_MONEY = [[
                player:
                    !overthere
                    By the way? The terms of this arrangement have clearly changed.
                    Trouble with the Spark Barons takes this up a pay grade.
                luminari:
                    !thought
                    $neutralThoughtful
                    That's true. And I can't complain about services rendered.
                    Would {1#money} be a satisfactory increase?
                    !give
                player:
                    !take
                    It's a good start.
                luminari:
                    !happy
                    Then let's continue.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.quest.param.interrupted = true
                local opfor = CreateCombatParty("SPARK_BARON_PATROL_MB", cxt.quest:GetRank(), cxt.location)
                cxt.quest.param.opfor = opfor
                cxt:ReassignCastMember( "baron", opfor[1] )
                cxt.enc:SetPrimaryCast(opfor[1])
                cxt:Dialog("DIALOG_INTRO")   
            end
                
            cxt:Opt("OPT_HAND_OVER")
                :Dialog("DIALOG_HAND_OVER")
                :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("gave_up_spark"), nil, cxt.quest:GetCastMember("luminari"))
                :Fn(function(cxt) 
                    cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("luminari"))
                    cxt.quest:Fail()
                    cxt:Opt("OPT_GIVE_BACK_THE_MONEY")
                        :DeliverMoney( PAYMENT_AMT, {no_scale=true})
                        :Dialog("DIALOG_GIVE_BACK_THE_MONEY")
                        :Travel()

                    cxt:Opt("OPT_REFUSE_TO_GIVE_BACK_MONEY")
                        :Dialog("DIALOG_REFUSE_TO_GIVE_BACK_MONEY")
                        :Battle{
                            on_win = function(cxt) 
                                cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_FOUGHT_LUMINARI_DEAD" or "DIALOG_FOUGHT_LUMINARI_ALIVE")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end,
                        }
                    --StateGraphUtil.AddLeaveLocation(cxt)
                end)
            
            cxt:Opt("OPT_NEGOTIATION")
                :Negotiation{
                    on_success = function(cxt)
                        cxt.quest:GetCastMember("luminari"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("defended_spark"))
                        cxt:Dialog("DIALOG_NEGOTIATION_SUCCESS")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt) 
                        cxt:Dialog("DIALOG_NEGOTIATION_FAIL")                        
                    end,
                }
            cxt:Opt("OPT_DEFEND_SPARK")
                :Dialog("DIALOG_DEFEND_SPARK")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = {"luminari"},
                    on_win = function(cxt) 
                        
                        if cxt.quest:GetCastMember("luminari"):IsDead() then
                            cxt:Dialog("DIALOG_FOUGHT_BARONS_LUMINARI_DEAD")
                            StateGraphUtil.AddLeaveLocation(cxt)    
                        else
                            cxt.quest:GetCastMember("luminari"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("defended_spark"))
                            local survivor
                            for k, v in pairs( cxt.quest.param.opfor ) do
                                if not v:IsDead() and v:IsSentient() then
                                    survivor = v
                                    break
                                end
                            end

                            if survivor then
                                cxt:ReassignCastMember( "survivor", survivor )
                                cxt:Dialog("DIALOG_FOUGHT_BARONS_SURVIVOR")
                            else
                                cxt:Dialog("DIALOG_FOUGHT_BARONS_NO_SURVIVORS")
                            end

                            cxt:Opt("OPT_DEMAND_MORE_MONEY")
                                :Dialog("DIALOG_DEMAND_MORE_MONEY", PAYMENT_AMT)
                                :ReceiveMoney(PAYMENT_AMT)
                                :Travel()

                            cxt:Opt("OPT_CONTINUE")
                                :Travel()

                        end
                    end,
                }

        end)

QDEF:AddConvo("deliver_spark", "luminari")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("FOO", function(cxt) return cxt.location == cxt.quest:GetCastMember("outpost") end)
        :Loc{
            DIALOG_INTRO = [[
                * You arrive at a small outpost constructed around a strange relic.
                agent:
                    !right
                    That's it, hunter. Thank you for your assistance. 
                    !point
                    Just place the <i>spark</> under that tarp, if you don't mind.
                player:
                    !exit
                * You drop the spark and cover it with the tarp. {agent} is visibly relieved once it's out of sight.
                agent:
                    !dubious
                player:
                    !left
                    !bashful
                    What's that look for? Is there something on my face?
                agent:
                    $neutralDirect
                    Hesh's work often requires discretion.
                    !crossed
                    It occurs to me I have no way to assure yours. 
                player:
                    !placate
                    Not true. That's what you <i>paid</i> for, after all.
                *** You have completed your task for the Luminari.
            ]],
            OPT_TRY_LEAVE = "Leave",
            DIALOG_TRY_LEAVE = [[
                player:
                    !overthere
                    $neutralDubious
                    So, with our business concluded... I'll be going now.
                * Just as you're about to leave, a priest emerges from the outpost with a bark of dismay.
                priest:
                    !right
                    !angry
                    Trespasser! What do you think you're doing in this holy space?!
                luminari:
                    !left
                    !placate
                    Priest, this is a hunter I hired to carry the spar—
                priest:
                    !surprise
                    You <i>hired</i> a common grifter?!
                luminari:
                    !neutral
                    I can't carry the accursed fuel! 
                priest:
                    !angry_accuse
                    You <i>can</i>, Luminari, and you should have.
                    That's why you <i>exist</i>! To prostrate yourself before the Abyss!
                    You are no priest. You're a holy thug, the least of Hesh's reach.
                    !palm
                    $angryReluctant
                    Did anyone else see you?
                luminari:
                    !sigh
                    There were some Barons, but—
                priest:
                    !angry
                    $angryDefault
                    Barons?! How have you failed so completely?
                *** The priest and the Luminari are arguing about what to do with you.
            ]],
            OPT_SNEAK_OFF = "Try to back away slowly",
            DIALOG_SNEAK_OFF = [[
                * You decide to leave before blades come out. You take two steps back, but the priest's attention snaps to you.
                player:
                    !left
                    !overthere
                    I'll just, uh—
                priest:
                    !point
                    You'll stop right there.
                    Each breath you take threatens to release the secret of our outpost. 
                    !angry
                    $angrySeething
                    Clearly, it's Hesh's will that you should die here rather than betray us.
                *** The priest is threatening to have you killed because you know the location of their secret outpost.
            ]],
            OPT_CONVINCE = "Make a case for yourself",
            DIALOG_CONVINCE = [[
                player:
                    Who brought me to you, huh, if not Hesh itself?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !thumb
                    I got the job done. Seems to me, Hesh meant for me to be here.
                    $miscPersuasive
                    So trust in Hesh that I'll keep the secret, yeah?
                priest:
                    !thought
                    $neutralThoughtful
                    It's true that Hesh's abyssal tides can't be seen from the surface. Why Hesh would send <i>you</i> ...
                    !point
                    Keep your mouth shut, and I'll allow you to use this place for rest. 
                    At worst, you'll find the faith. At best, you'll be where I can keep an eye on you.
                *** The Heshians have agreed to let you live and use their outpost.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    Who's to say it's not the will of your fishlord, huh?
                priest:
                    <i>I</i> say, grifter. 
                    {luminari}, dispose of her, will you?
                luminari:
                    !right
                    !disappoint
                    $miscDepressed
                    I—yes, priest.
            ]],
            OPT_ATTACK = "Get the drop on the cultists",
            DIALOG_ATTACK = [[
                player:
                    If Hesh had wanted me dead, it should've done it when I was slaving away on that derrick for ten years.
                    !fight
                    Hesh had its chance! Now it's <i>my</i> turn.
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    Typical. Lift a finger for the Heshians, and they'll give you one in return.
                    !fight
                    I won't go down easy.   
            ]]
        }
        :Fn(function(cxt)
            cxt.quest:Complete()
            
            cxt.quest:GetCastMember("luminari"):Dismiss()
            cxt.quest:GetCastMember("priest"):MoveToLocation(cxt.location)
            cxt:Dialog("DIALOG_INTRO")  

            cxt:Opt("OPT_TRY_LEAVE")
                :Dialog("DIALOG_TRY_LEAVE")
                :Fn(function(cxt) 
                    cxt:Opt("OPT_SNEAK_OFF")
                        :Dialog("DIALOG_SNEAK_OFF")
                        :Fn(function(cxt) 
                            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("priest"))
                            cxt:Opt("OPT_CONVINCE")
                                :Dialog("DIALOG_CONVINCE")
                                :Negotiation{
                                    on_success = function() 
                                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                                        QuestUtil.SpawnQuest("SAL_HESH_OUTPOST")   
                                        StateGraphUtil.AddEndOption(cxt)
                                    end,
                                    on_fail = function() 
                                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                                        cxt:Opt("OPT_DEFEND")
                                            :Battle{
                                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                                bystanders = {"priest"},
                                                enemies = {"luminari"},
                                                on_win = function(cxt) 
                                                    cxt:GoTo("STATE_WON_FIGHT")
                                                end
                                            }
                                    end,
                                }
                            cxt:Opt("OPT_ATTACK")
                                :Dialog("DIALOG_ATTACK")
                                :Battle{                                   
                                    enemies = {"luminari", "priest"},
                                    advantage = TEAM.BLUE,
                                    on_win = function(cxt) 
                                        cxt:GoTo("STATE_WON_FIGHT")
                                    end
                                }
                        end)
                end)
        end)
    
    :State("STATE_WON_FIGHT")
        :Loc{
            DIALOG_FOUGHT_BOTH_DEAD = [[
            * Both the Heshians are dead. It brings you a bit of satisfaction, but not much else.
            * You make for the hills before anyone can find you in this hugely incriminating situation. 
            ]],

            DIALOG_FOUGHT_PRIEST_DEAD = [[
                luminari:
                    !left
                    !injured
                    Enough, enough! I yield!
                player:
                    !angry
                    $angryDefault
                    Good. And remember that you're the ones who forced my hand.
                luminari:
                    So be it, hunter.
                    $angryReluctant
                    Leave me be—my work is more important than any vendetta.
                player:
                    !chuckle
                    $angryCruel
                    Good luck doing it on your own. 
                * You glance back at the mystery outpost one last time. You have a feeling it'll be dismantled before you'll ever find it again.
            ]],


            DIALOG_FOUGHT_LUMINARI_DEAD = [[
                priest:
                    !right
                * {priest} stares at {luminari}'s body, terrified.
                player:
                    !cruel
                    $miscMocking
                    See? You don't take care of your toys, they end up broken.
                priest:
                    !scared
                    $scaredStammering
                    D-don't kill me. I-I have so much work left to do.
                player:
                    !angry_accuse
                    $angrySeething
                    You send anyone after me, or speak a word of what I've done here—
                priest:
                    !placate
                    $scaredDefault
                    I won't! Just go!
                player:
                    !cruel
                    Thank you. I will. 
            ]],

            DIALOG_FOUGHT_BOTH_ALIVE = [[
                priest:
                    !right
                * The priest crouches over the Luminari, both looking like they took more hits than they could handle.
                priest:
                    !injured
                    $scaredStammering
                    Enough! You've played your part, grifter.
                    $miscHopeless 
                    Now all you see are two broken believers. 
                player: 
                    !angry_accuse
                    <i>You</i> were the ones who attacked <i>me</i>, don't forget. 
                    I just did a job. It's not my fault you have trust issues.
                priest:
                    ...
                luminari:
                    !right
                    !injured
                    $miscHopeless
                    She's right. We must remember, priest, that Hesh's whims are mysterious. We challenged when we should've submitted.
                priest:
                    !right
                    !disappoint
                    Forgive us, hunter. Clearly my friend and I have much to consider. 
                player:
                    !overthere
                    $miscDepressed
                    Ugh, whatever. Just get out of here, both of you.
                    And tell your lunatic god to stop sending its slippery feelers into my business.
                * The priest and Luminari support one another carefully as you leave them behind. 
            ]],
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt.quest:GetCastMember("luminari"):IsDead() and cxt.quest:GetCastMember("priest"):IsDead() then
                cxt:Dialog("DIALOG_FOUGHT_BOTH_DEAD")
            elseif cxt.quest:GetCastMember("luminari"):IsDead() then
                cxt:Dialog("DIALOG_FOUGHT_LUMINARI_DEAD")
            elseif cxt.quest:GetCastMember("priest"):IsDead() then
                cxt:Dialog("DIALOG_FOUGHT_PRIEST_DEAD")
            else
                cxt:Dialog("DIALOG_FOUGHT_BOTH_ALIVE")
            end
            StateGraphUtil.AddLeaveLocation(cxt)

            

        end)