
local QDEF = QuestDef.Define
{
    title = "Rise and Shine",
    icon = engine.asset.Texture("icons/quests/sal_rise_outpost.tex"),
    home = true, -- If this shows a yellow-colored icon, with a yellow arrow under it
    qtype = QTYPE.SIDE,
    tags = {"manual_spawn"},
    focus = QUEST_FOCUS.NEGOTIATION,
    desc = "{rebel1} needs your help contacting local merchants who might be interested in buying stolen goods from Grout Bog.",
    plot_armour_fn = function(quest, agent)
        if agent == quest:GetCastMember("rebel1") then
            return true
        end
    end,

    on_destroy = function(quest) 
        if quest:GetCastMember("buyer1") and quest:GetCastMember("buyer1"):IsInPlayerParty() then
            quest:GetCastMember("buyer1"):Dismiss()
        end
        if quest:GetCastMember("buyer2") and quest:GetCastMember("buyer2"):IsInPlayerParty() then
            quest:GetCastMember("buyer2"):Dismiss()
        end
    end,

    --Want to keep them here. maybe these should be jobs instead?
    collect_agent_locations = function(quest, t)
        
        if quest:IsInactive() then
            table.insert (t, { agent = quest:GetCastMember("rebel1"), location = TheGame:GetGameState():GetLocation("GROG_N_DOG"), role = CHARACTER_ROLES.PATRON } )
        else
            table.insert (t, { agent = quest:GetCastMember("rebel1"), location = quest:GetCastMember("home"), role = CHARACTER_ROLES.PROPRIETOR } )
        end
    end,

    prevent_random_events = true,
    
    avoids = 
    {
        {"rebel1", "buyer1"},
        {"rebel1", "buyer2"},
        {"buyer1", "buyer2"},
    },
    on_init = function(quest)
        quest.param.should_do_early_complication = math.random() < .5
    end,

}

:AddOpinionEvents{

    sided_with_barons =  
    {
        delta = OPINION_DELTAS.DIMINISH,
        txt = "Gave up your business partners to the Spark Barons",
    },
}

:AddLocationCast{
    cast_id = "home",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MURDER_BAY_RISE_OUTPOST"))
    end,
}

:AddCast{
    cast_id = "buyer1",
    when = QWHEN.MANUAL,
    condition = function( agent, quest )
        return agent:GetContentID() == "WEALTHY_MERCHANT" and agent:GetRelationship(quest:GetCastMember("rebel1")) == RELATIONSHIP.NEUTRAL and agent:GetLocation() ~= quest:GetCastMember("rebel1"):GetLocation() 
    end,

    events = 
    {
        agent_retired = function( quest, agent ) 
            if quest:IsActive("contact_buyer_1") then
                quest:Fail("contact_buyer_1")
            end
        end,
    }


}
:AddCastFallback{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "WEALTHY_MERCHANT" ) )
    end,
}

:AddCast{
    cast_id = "buyer2",
    when = QWHEN.MANUAL,
    condition = function( agent, quest )
        return agent:GetContentID() == "WEALTHY_MERCHANT" and agent:GetRelationship(quest:GetCastMember("rebel1")) == RELATIONSHIP.NEUTRAL and agent:GetLocation() ~= quest:GetCastMember("rebel1"):GetLocation() 
    end,

    events = 
    {
        agent_retired = function( quest, agent ) 
            if quest:IsActive("contact_buyer_2") then
                quest:Fail("contact_buyer_2")
            end
        end,
    }

}

:AddCastFallback{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "WEALTHY_MERCHANT" ) )
    end,
}


:AddCastByAlias{
    cast_id = "rebel1",
    provider = true,
    alias = "RISE_OUTPOST_REBEL1",
    no_validation = true,
}


:AddObjective{
    id = "contact_buyer_1",
    title = "Convince {buyer1} to buy from the Rise",
    desc = "The Rise are looking for buyers for some stolen Spark Baron technology. Help them seal the deal.",
    mark = {"buyer1"},
    negotiation_targets = {"buyer1"},

    on_complete = function(quest)
        if quest:IsActive("contact_buyer_2") then
            quest:Cancel("contact_buyer_2")
        end
        quest:Activate("take_buyer_to_deal")
        quest.param.current_escort = quest:GetCastMember("buyer1")
    end,

    on_fail = function(quest)
        if not quest:IsActive("contact_buyer_2") then
            quest:Fail()
        end
    end,
}

:AddObjective{
    id = "contact_buyer_2",
    title = "Convince {buyer2} to buy from the Rise",
    desc = "The Rise are looking for buyers for some stolen Spark Baron technology. Help them seal the deal.",
    mark = {"buyer2"},
    negotiation_targets = {"buyer2"},

    on_complete = function(quest)
        if quest:IsActive("contact_buyer_1") then
            quest:Cancel("contact_buyer_1")
        end
        quest:Activate("take_buyer_to_deal")
        quest.param.current_escort = quest:GetCastMember("buyer2")
    end,
    
    on_fail = function(quest)
        if not quest:IsActive("contact_buyer_1") then
            quest:Fail()
        end
    end,
}


:AddObjective{
    id = "take_buyer_to_deal",
    title = "Take {1#agent} to the Rise camp",
    title_fn = function(quest, str) 
        return loc.format(str, quest.param.current_escort) 
    end,

    mark = {"home"},

    events = 
    {
        caravan_member_remove = function(quest, agent) 
            if agent == quest.param.current_escort then
                local other_objective_id = agent == quest:GetCastMember("buyer1") and "contact_buyer_2" or "contact_buyer_1"
                local other_buyer = quest:GetCastMember("buyer1") and quest:GetCastMember("buyer2") or quest:GetCastMember("buyer1")
                if quest:IsFailed(other_objective_id) or other_buyer:IsDead() then
                    quest:Fail()
                else
                    quest:Activate(other_objective_id)
                    quest:Fail("take_buyer_to_deal")
                end
            end
        end,
    }        

}


QDEF:AddConvo(nil, nil)

    :Loc{
        OPT_TELL_ABOUT_DEAL = "Tell {agent} about the Rise offer",
        DIALOG_TELL_ABOUT_DEAL_1 = [[
            player:
                !crossed
                Word is that you're in the market for some goods from Grout Bog.
                I'd like to facillitate the exchange. 
            agent:
                !thought
                Ah, I see. That means you're one of those... what are they called?
            player:
                The Rise. And no, I'm not. 
                I'm just an honest grifter, collecting a paycheque for services rendered. Are you interested or not?
            agent:
                !dubious
                How do I know this is legit? This could be a trick by the Barons.
                Hesh knows they wouldn't pass up an opportunity to make trouble. 
        ]],
        DIALOG_TELL_ABOUT_DEAL_2 = [[
            player:
                Word is that you're in the market for some goods from Grout Bog.
            agent:
                Could be. But I've been burnt by those crazies before. How do I know their goods are... good?
        ]],

        OPT_CONVINCE = "Convince {agent} to come along",
        DIALOG_CONVINCE_1 = [[
            player:
                This close to the auction? Come on. They have bigger fish to fry—ones with funny hats.
        ]],

        DIALOG_CONVINCE_2 = [[
            player:
                Because this is the stuff the Spark Barons thought was good enough to bring to the auction.
        ]],

        DIALOG_FAIL_1 = [[
            player:
                The risk might be high, sure, but the rewards are too good to pass up.
            agent:
                I disagree. And I'll prove it by passing it up. 
        ]],
        DIALOG_FAIL_2 = [[
            player:
                If the Spark Barons have vetted it, surely that's good enough?
            agent:
                !angry
                Yeah, right. Everyone knows they bring the most useless trash for the Cult to pick over.
                If I wanted garbage I'd just fish it out of the river for free.
        ]],

        DIALOG_SUCCEED_1 = [[
            player:
                $miscPersuasive
                Trust me, it's the Rise. Nothing but <i>liberate</i> this and <i>manumission</i> that.
                !sigh
                It's exhausting.
            agent:
                I suppose you have a point.
                !hips 
                Very well. Lead on.
        ]],

        DIALOG_SUCCEED_2 = [[
            player:
                $miscPersuasive
                !overthere
                You're basically getting auction-grade relics, but without having to compete with the Cult. 
                Sounds good, right?
            agent:
                Perhaps too good. But I believe you. 
                !point
                Lead the way, but don't expect me to let my guard down.
        ]],

    }
    :Hub(function(cxt) 

        local b1 = cxt.quest:IsActive("contact_buyer_1") and cxt:GetAgent() == cxt.quest:GetCastMember("buyer1")
        local b2 = cxt.quest:IsActive("contact_buyer_2") and cxt:GetAgent() == cxt.quest:GetCastMember("buyer2")
        if (b1 or b2) then
            cxt:Opt("OPT_TELL_ABOUT_DEAL")
                :Dialog(b1 and "DIALOG_TELL_ABOUT_DEAL_1" or "DIALOG_TELL_ABOUT_DEAL_2")
                :Fn(function() 
                        cxt:Opt("OPT_CONVINCE")
                            :Dialog(b1 and "DIALOG_CONVINCE_1" or "DIALOG_CONVINCE_2")
                            :Negotiation{
                                on_success= function()
                                    cxt.quest:Complete(b1 and "contact_buyer_1" or "contact_buyer_2")
                                    cxt:Dialog(b1 and "DIALOG_SUCCEED_1" or "DIALOG_SUCCEED_2")
                                    cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                    StateGraphUtil.AddEndOption(cxt)
                                end,
                                on_fail = function()
                                    cxt:Dialog(b1 and "DIALOG_FAIL_1" or "DIALOG_FAIL_2")
                                    cxt.quest:Fail(b1 and "contact_buyer_1" or "contact_buyer_2")
                                    
                                    StateGraphUtil.AddEndOption(cxt)
                                end
                            }
                        StateGraphUtil.AddBackButton(cxt)
                end)
        end
    end)


QDEF:AddConvo("take_buyer_to_deal", nil)
    :ConfrontState("confront", function(cxt) return cxt.quest.param.should_do_early_complication and not cxt.quest.param.has_been_travel_interrupted and cxt.location:HasTag("in_transit") end) 
        :Fn(function(cxt) 
            cxt.quest.param.has_been_travel_interrupted = true
            cxt:GoTo( table.arraypick{"STATE_SB_TAIL", "STATE_OFFER_TO_BETRAY"} )
        end)
    
    :State("STATE_OFFER_TO_BETRAY")
        :Loc{
            DIALOG_INTRO = [[
                * {agent} clears {agent.hisher} throat.
                player:
                    !left
                agent:
                    !right
                    So, you're a rational businessperson, right?
                    You're not caught up in the Rise's politics, are you?
            ]],
            OPT_PROFESSIONAL = "Talk about professionalism",
            DIALOG_PROFESSIONAL = [[
                player:
                    I'm a <b>grifter</>.
                    I do folks' dirty work, and they pay me for it.
                    Politics are not part of my job description.
                agent:
                    I totally respect that. 
                    That's why I'm prepared to offer you a lot of money to do a... slightly different job.
            ]],
            OPT_SYMPATHETIC = "Express sympathy for the workers",
            DIALOG_SYMPATHETIC = [[
                player:
                    I know what it's like to be stuck under someone's bootheel.
                    I'm not about to start waving their flag, but if I'm picking sides I know where I'd land.
                agent:
                    Those sure are pretty words.
                    Truly moving.
                    But I've got a pile of money that I think will change your tune.
            ]],
            OPT_DONT_TAKE_SIDES = "Feign ironic detachment",
            DIALOG_DONT_TAKE_SIDES = 
            [[
                player:
                    !shrug
                    I'm not really the joining type.
                    Besides, I'm sure they'd be just as bad as the Barons if they ever got their way.
                agent:
                    That's the spirit! The world's a flaming dung-heap. Always was, always will be.
                    May as well get rich off it while you're here, amiright?
            ]],
        }
        :Fn(function(cxt)
            cxt.enc:SetPrimaryCast(cxt.quest.param.current_escort)
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_PROFESSIONAL")
                :Dialog("DIALOG_PROFESSIONAL")
                :GoTo("STATE_OFFER_TO_BETRAY_2")

            cxt:Opt("OPT_SYMPATHETIC")
                :Dialog("DIALOG_SYMPATHETIC")
                :GoTo("STATE_OFFER_TO_BETRAY_2")
            
            cxt:Opt("OPT_DONT_TAKE_SIDES")
                :Dialog("DIALOG_DONT_TAKE_SIDES")
                :GoTo("STATE_OFFER_TO_BETRAY_2")
        end)

    :State("STATE_OFFER_TO_BETRAY_2")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    These Rise bozos have been flapping gums far and wide about their stolen goods for weeks.
                    I know a Baron who will pay top dollar for the location of that camp.
                    
                    I <b>can</> make money on the goods, but I can make a lot more on the info, and have less work to do.
                    What do you say?
            ]],
            OPT_ACCEPT = "Sell {agent} the info",
            DIALOG_ACCEPT = [[
                player:
                    Alright, you got me.
                    I can't turn down an easy offer.
                agent:
                    !happy
                    That's what I thought.
                * You trade the info for the money.
                agent:
                    Alright, if anyone asks, this never happened.
                player:
                    Of course.
                agent:
                    !exit
                * {agent} heads off to find {agent.hisher} Baron contact.
            ]],
            OPT_REFUSE = "Stick to the plan",
            DIALOG_REFUSE = [[
                player:
                    Let's just stick to the plan, alright?
                agent:
                    Of course! Of course!
                    It was just an idea.
                    !exit
                player:
                    !exit
                * As you set off again, {agent} whistles a happy tune, trying to change the topic.
            ]],
            OPT_EXTRACT = "Blackmail {agent} for {1#money}",
            DIALOG_EXTRACT = [[
                player:
                    Those Rise 'bozos' seem pretty dangerous, if you ask me. 
                    Maybe I'll tell them what you just said.
                    I wonder what would happen then?
            ]],
            DIALOG_EXTRACT_FAIL = [[
                agent:
                    Was that <i>blackmail</>? 
                    I respect the attempt, really.
                    But they're going to believe <b>my</> word over <b>yours</>, grifter.
                    Let's just pretend this conversation didn't happen.
            ]],
            DIALOG_EXTRACT_SUCCESS = [[
                agent:
                    Whoa, good point.
                    I know when I've been bested.
                    How about I give you a little something and we pretend this never happened, OK?
                    !give
                player:
                    !take
                    As you were.
            ]],
        }
        :Fn(function(cxt) 

            local betray_amt = 500
            local blackmail_amt = 200

            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_ACCEPT")
                :Dialog("DIALOG_ACCEPT")
                :ReceiveMoney(betray_amt, {no_scale = true})
                :FailQuest()
                :ReceiveOpinion(OPINION.BETRAYED, nil, cxt.quest:GetCastMember("rebel1"))
                :Travel()

            cxt:Opt("OPT_REFUSE")
                :Dialog("DIALOG_REFUSE")
                :Travel()

            cxt:Opt("OPT_EXTRACT", blackmail_amt)
                :Dialog("DIALOG_EXTRACT")
                :Negotiation{
                    on_success= function() 
                        cxt:Dialog("DIALOG_EXTRACT_SUCCESS")
                        cxt.caravan:AddMoney(blackmail_amt)
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function() 
                        cxt:Dialog("DIALOG_EXTRACT_FAIL")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }
        end)


    :State("STATE_SB_TAIL")
        :Loc{
            DIALOG_INTRO_ATTACKED_BY_BARONS = [[
                * It doesn't take a hunter as good as you to know you're being followed. You turn to face the tail.
                player:
                    !left
                    Hey, it's rude to slink around like that. 
                    I should know—I'm plenty rude myself.
                agent:
                    !right
                    !surprised
                    Shoulda kept your eyes on the road, grifter! 
                    !fight
                    Attack!
            ]],
            
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Ugh, typical.
            ]]            ,
            DIALOG_FOUGHT_MERCHANT_DEAD_BARONS_DEAD = [[
                * The fight is over, and you stand alone in a pile of carnage. 
            ]],
            DIALOG_FOUGHT_MERCHANT_ALIVE_BARONS_DEAD = [[
                * The Spark Barons are dead.
                merchant:
                    !right
                player:
                    $neutralResigned
                    Friends of yours?
                merchant:
                    No, but it only makes sense that they would want to track down their stolen goods. 
                    !handwring
                    At the very least, that confirms that the goods are worth fighting for. 
            ]],
            DIALOG_FOUGHT_MERCHANT_DEAD_BARONS_ALIVE = [[
                * {merchant} is dead, and the Rise won't have any reason to pay you.
                player:
                    !angry
                    What'd you have to attack for?! 
                survivor:
                    !right
                    !injured
                    Because I'm no dummy!
                    You're an associate of the Rise, and you're helping them steal from under our noses.
                    By rights I should've taken you out when your back was turned for being a—a bog insurrectionist! 
                player:
                    !angry_shrug
                    I'm not in the bog, Baron, and I don't plan on going back anytime soon. 
                    You want me to care about your problems? Pay me.
                survivor:
                    W-well, sure. We can do that.
                    Tell me who hired you and where we might find 'em. Then we'll see you paid. 
            ]],
            DIALOG_FOUGHT_MERCHANT_ALIVE_BARONS_ALIVE = [[
                survivor:
                    !right
                    !injured
                * The fight ends with hurt feelings and bruises.
                player:
                    !angry
                    Why are you following us?
                survivor:
                    You're an associate of the Rise, and you're helping them steal from under our noses.
                merchant:
                    !right
                    The petty squabbles of Grout Bog are hardly any concern to the good people of Murder Bay.
                    We're honest business folk, {player}. The Spark Barons acted out of line, as they well know.
                survivor:
                    !right
                    F-fine. You want to make this about a bottom line?
                    Tell us where to find the Rise agent who hired you, and we'll pay you handsomely. More than they offered, that's for sure.
            ]],

            OPT_TELL = "Tell the Barons where the Rise is hiding",
            DIALOG_TELL = [[
                player:
                    Money talks, and so do I.
                * You mark the location of the Rise encampment on {survivor}'s map.
                survivor:
                    !happy
                    The Spark Barons thank you.
                    !exit
                player:
                    !angry_shrug
                    Ugh. That left a bad taste in my mouth.
                    !neutral
                    $neutralJoke
                    But yay, shills.
            ]],

            OPT_NO_TELL = "Refuse to help the Barons",
            DIALOG_NO_TELL = [[
                player:
                    Forget it, Baron. I'll keep my nose out of bog business. 
                    !throatcut
                    Now get out of here before I kill you and take your shoes.
                survivor:
                    !exit
            ]],
            DIALOG_ANGRY_MERCHANT = [[
                merchant:
                    !right
                    I'm out of here. You're dangerous, and not to be trusted!
                    !exit
                player:
                    !thought
                    Did {merchant.heshe} realize that I'm a grifter?
            ]],
            DIALOG_HAPPY_MERCHANT = [[
                merchant:
                    !right
                    !happy
                    Let's get going. If the Barons are willing to kill for this stuff, it must be good!
            ]]

        }
        :Fn(function(cxt)
            cxt.quest.param.did_fight_already = true

            local opfor = CreateCombatParty("SPARK_BARON_PATROL_MB", cxt.quest:GetRank(), cxt.location)
            cxt.enc:SetPrimaryCast(opfor[1])
            cxt:ReassignCastMember("merchant", cxt.quest.param.current_escort)
            
            cxt:Dialog("DIALOG_INTRO_ATTACKED_BY_BARONS")

            
            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    enemies = opfor,
                    allies = {cxt.quest.param.current_escort},
                    on_win = function(cxt) 
                        local survivor
                        for k, v in pairs( opfor ) do
                            if not v:IsDead() and v:IsSentient() then
                                survivor = v
                                break
                            end
                        end
                        
                        if survivor then
                            cxt:ReassignCastMember("survivor", survivor)
                        end

                        if survivor then
                            cxt:Dialog(cxt.quest.param.current_escort:IsDead() and "DIALOG_FOUGHT_MERCHANT_DEAD_BARONS_ALIVE" or "DIALOG_FOUGHT_MERCHANT_ALIVE_BARONS_ALIVE")
                            
                            cxt:Opt("OPT_TELL")
                                :Dialog("DIALOG_TELL")
                                :ReceiveOpinion(OPINION.BETRAYED, nil, cxt.quest:GetCastMember("rebel1"))
                                :ReceiveMoney(200)
                                :FailQuest()
                                :Fn(function() if not cxt.quest.param.current_escort:IsDead() then cxt:Dialog("DIALOG_ANGRY_MERCHANT") cxt.quest.param.current_escort:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("sided_with_barons")) end end)
                                :Travel()

                            cxt:Opt("OPT_NO_TELL")
                                :Dialog("DIALOG_NO_TELL")
                                :Fn(function() if not cxt.quest.param.current_escort:IsDead() then cxt:Dialog("DIALOG_HAPPY_MERCHANT") end end)
                                :Travel()

                        else
                            if cxt.quest.param.current_escort:IsDead() then
                                cxt:Dialog("DIALOG_FOUGHT_MERCHANT_DEAD_BARONS_DEAD")
                            else
                                cxt:Dialog("DIALOG_FOUGHT_MERCHANT_ALIVE_BARONS_DEAD")
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end

                    end
                }           
        end)






QDEF:AddConvo("take_buyer_to_deal", nil)
    :ConfrontState("confront", function(cxt) return cxt.location == cxt.quest:GetCastMember("home") end)
        :Loc{
            DIALOG_INTRO = [[
                * You arrive at the Rise camp.
                merchant:
                    !left
                    So where's the stuff?
                rebel1:
                    !right
                    We meet at last. Let me show you what we have.
                    {player}, please keep an eye out for trouble, will you?
                    !exit
                merchant:
                    !exit
                * {merchant} and {rebel1} retreat into a tent. You can just barely hear their muffled deliberations through the canvas.
                player:
                    !left
                * The haggling goes on for quite some time. You scan the surrounding trees for threats.

                *** You are waiting outside while {merchant} and {rebel1} do business.
            ]],
        }
        :Fn(function(cxt)
            local escort = cxt.quest.param.current_escort
            cxt:ReassignCastMember("merchant", escort)
            cxt.quest:Complete("take_buyer_to_deal")
            cxt.quest.param.current_escort:Dismiss()
            cxt.encounter:SetPrimaryCast( escort )
            cxt:Dialog("DIALOG_INTRO")
            local opts = {"STATE_HARD_SELL"}
            if cxt.quest.param.should_do_early_complication then
                table.insert(opts, "STATE_SALE_SUCCESS")            
            end
            if not cxt.quest.param.did_fight_already then
                table.insert(opts, "STATE_BARON_AMBUSH")            
            end
            cxt:GoTo(table.arraypick(opts) )

        end)

    :State("STATE_BARON_AMBUSH")    
        :Loc{
            DIALOG_INTRO = [[
                * You see the glint of weaponry.
                player:
                    Who's there?
                * Spark Barons emerge from the gloom.
                baron:
                    !right
                    !angry
                    You have two choices, grifter.
                    Take a walk, and this never happened.
                    !throatcut
                    Or stay, and we'll kill you.
            ]],
            OPT_LEAVE = "Leave",
            DIALOG_LEAVE = [[
                player:
                    !placate
                    Hey, sure. I was never here.
                baron:
                    Good choice. Now scram.
                player:
                    !exit
                baron:
                    !exit
                * As you walk away from the camp, you hear the sounds of battle. 
                * It's rather one-sided.
            ]],

            OPT_FIGHT = "Defend the camp",
            DIALOG_FIGHT = [[
                player:
                    !fight
                    Are you sure about that?
                    {rebel1}! We've got company!
                * {rebel1} rushes out of the tent to join you!
            ]],
            DIALOG_FIGHT_REBEL_ALIVE = [[
                player:
                    !neutral
                rebel1:
                    !right
                    Thanks, {player}. I'm glad you were here for that.
                    The deal is a success, too, so the Rise will be well-supplied, thanks to you.
                    If you want to buy anything, feel free to drop in. I'll be in the Bay for a couple more days waiting for the transactions to go through.

                *** The Rise have completed their deal and have offered to sell you supplies when you need them.
            ]],
            DIALOG_FIGHT_REBEL_DEAD = [[
                right:
                    !exit
                * The Barons are defeated, but {rebel1} is dead. 
                * There's no sign of {merchant}, either. You presume {merchant.heshe} took as much loot as {merchant.heshe} could carry and ran for it. 
                * You look over what's left, and find a strange, animal-shaped automech.
            ]],
            OPT_ACTIVATE_THE_MECH = "Activate the mech",
            DIALOG_ACTIVATE_THE_MECH = [[
                * You cautiously activate the mech. It whirs to life, and stares at you.
                player:
                    !give                
                autodog:
                    !right
                
            ]],
            DIALOG_ACTIVATE_THE_MECH_FRIENDLY = [[
                player:
                    !happy
                * Looks like you have a new 'friend'.
            ]],
            DIALOG_ACTIVATE_THE_MECH_UNFRIENDLY = [[
                player:
                    !fight
                * It attacks!
            ]],
            DIALOG_FOUGHT_DOG_WON = [[
                * The mech lies in pieces.
            ]],
            OPT_LEAVE_THE_MECH = "Leave the mech",
            DIALOG_LEAVE_THE_MECH = [[
                * You decide it's probably best to leave the sleeping robots alone.
            ]],
            DIALOG_ACTIVATE_THE_MECH_ATTACK = [[
                * The mech attacks!
            ]],


        }
        :Fn(function(cxt) 
            local opfor = CreateCombatParty("SPARK_BARON_PATROL_MB", cxt.quest:GetRank(), cxt.location)
            cxt:ReassignCastMember("baron", opfor[1])
            cxt:Dialog("DIALOG_INTRO")


            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :ReceiveOpinion(OPINION.BETRAYED, nil, cxt.quest:GetCastMember("rebel1"))
                :ReceiveOpinion(OPINION.BETRAYED, nil, cxt.quest.param.current_escort)
                :FailQuest()
                :Travel()

            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                    enemies = opfor,
                    allies = {cxt.quest:GetCastMember("rebel1")},
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_BYSTANDERS,
                    on_win = function(cxt) 
                        cxt.quest.param.current_escort:MoveToLimbo()
                        for k,v in pairs(opfor) do
                            if not v:IsDead() then
                                v:MoveToLimbo() 
                            end
                        end

                        if cxt.quest:GetCastMember("rebel1"):IsDead() then
                            local autodog = cxt.quest:CreateSkinnedAgent( "AUTODOG" )
                            autodog:MoveToLocation( cxt.location )
                            TheGame:GetGameState():AddAgent( autodog )
                            cxt:ReassignCastMember("autodog", autodog)

                            cxt:Dialog("DIALOG_FIGHT_REBEL_DEAD")
                            cxt.quest:Fail()
                            cxt:Opt("OPT_ACTIVATE_THE_MECH")
                                :Dialog("DIALOG_ACTIVATE_THE_MECH")
                                :Fn(function() 
                                    local can_own_pet = cxt.caravan:CanOwnPet()
                                    if can_own_pet and math.random() < .5 then
                                        cxt:Dialog("DIALOG_ACTIVATE_THE_MECH_FRIENDLY")
                                        autodog:Recruit(PARTY_MEMBER_TYPE.CREW)
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    else
                                        cxt:Dialog("DIALOG_ACTIVATE_THE_MECH_ATTACK")
                                        cxt:Opt("OPT_DEFEND")
                                            :Dialog("DIALOG_DEFEND")
                                            :Battle{
                                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                                enemies = {autodog},
                                                on_win = function(cxt)
                                                    cxt:Dialog("DIALOG_FOUGHT_DOG_WON")
                                                    StateGraphUtil.AddLeaveLocation(cxt)
                                                end,
                                            }
                                    end
                                end)
                            cxt:Opt("OPT_LEAVE_THE_MECH")
                                :Dialog("DIALOG_LEAVE_THE_MECH")
                                :Travel()

                            
                        else
                            cxt:Dialog("DIALOG_FIGHT_REBEL_ALIVE")
                            cxt.quest:Complete()
                            QuestUtil.SpawnQuest("SAL_RISE_OUTPOST")
                            ConvoUtil.GiveQuestRewards(cxt)
                            StateGraphUtil.AddEndOption(cxt)
                        end
                    end,
                }
        end)

    :State("STATE_SALE_SUCCESS")    
        :Loc{
            DIALOG_INTRO = [[
                * After what seems like an eternity, {rebel1} and {merchant} emerge from the tent, all smiles.
                rebel1:
                    !right
                    !happy
                    A pleasure doing business with you.
                merchant:
                    !left
                    !happy
                    Don't spend it all in one place, now.
                    !exit
                * {merchant} nods in your direction as {merchant.heshe} leaves.
                player:
                    !left
                    !crossed
                    $happyAmused
                    I take it things went well.
                rebel1:
                    !eureka
                    Yes! Now we can buy the supplies that we need to really bring it to the Barons.
                    I'll be in here in the Bay for a couple of days wrapping up business. If you're looking for exotic goods from the Bog, feel free to drop by.
                *** The Rise have completed their deal and have offered to sell you supplies when you need them.                    
            ]],
        }
        :Fn(function(cxt) 
            cxt:ReassignCastMember("merchant", cxt.quest.param.current_escort)
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Complete()
            QuestUtil.SpawnQuest("SAL_RISE_OUTPOST")
            ConvoUtil.GiveQuestRewards(cxt)
            cxt.quest.param.current_escort:MoveToLimbo()
            StateGraphUtil.AddEndOption(cxt)
        end)

    :State("STATE_HARD_SELL")    
        :Loc{
            DIALOG_INTRO = [[
                * After a time, {merchant} bursts from the tent.
                merchant:
                    !right
                    !angry
                    What a waste of time!
                    They quote one price, and then charge another. I would be surprised if any of their stuff worked at all!
                *** {merchant} is walking out on the deal.
            ]],
            
            OPT_CONVINCE = "Convince {merchant} to go back in the tent",
            DIALOG_CONVINCE = [[    
                player:
                    !shrug
                    So you're just going to walk away empty-handed? 
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !overthere
                    $miscPersuasive
                    If you don't act now, the goods will get scooped up by the Cult and squirreled away in Supplicant City.
                merchant:
                    !disappoint
                    Ah, I guess that's a fair point. I suppose... I can go a <i>little</i> over budget.
                    !exit
                * {merchant} goes back in the tent. After some more shouting, {rebel1} comes out.
                rebel1:
                    !right
                    !happy
                    $happyThanks
                    Thanks for the help. I think this is going to work.
                    I'll be here in the Bay for a couple of days wrapping up business. If you're looking for exotic goods from the Bog, feel free to drop by.
            ]], 

            DIALOG_CONVINCE_FAIL = [[
                player:
                    !overthere
                    Come on. You don't want a chance like this to slip through your fingers!
                merchant:
                    !angry_accuse
                    $angryDefault
                    You're in on it too, aren't you?
                    Get outta my way.
                    !exit
                * {merchant} storms off.
                rebel1:
                    !right
                    !facepalm
                    Well, that went well.
                    We're going to have to pick up stakes and try moving this stuff somewhere else.
                    !shrug
                    Not your fault, I suppose. 
                    !exit
                * {rebel1} starts striking camp.
            ]],

            OPT_LET_GO = "Let {merchant} go",
            DIALOG_LET_GO = [[
                player:
                    Don't let me stop you.
                merchant:
                    !exit
                * {merchant} heads of in the direction of civilization.
                rebel1:
                    !right
                    !facepalm
                    Well, that went well.
                    We're going to have to pick up stakes and try moving this stuff somewhere else.
                    !shrug
                    Not your fault, I suppose. 
                    !exit
                * {rebel1} starts striking camp.
            ]],
        }
        :Fn(function(cxt) 
            cxt:ReassignCastMember("merchant", cxt.quest.param.current_escort)
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")

                :Negotiation{
                    target_agent = cxt.quest.param.current_escort,
                    on_success = function() 
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest:Complete()
                        QuestUtil.SpawnQuest("SAL_RISE_OUTPOST")
                        ConvoUtil.GiveQuestRewards(cxt)
                        StateGraphUtil.AddEndOption(cxt)
                    end,

                    on_fail = function() 
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }

            cxt:Opt("OPT_LET_GO")
                :Dialog("DIALOG_LET_GO")
                :Fn(function(cxt) 
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)



QDEF:AddConvo(nil, "rebel1")
    :Loc{
        OPT_ASK_ABOUT_JOB = "Ask about the job",
        DIALOG_ASK_ABOUT_JOB = [[
            player:
                !interest
                Let's talk. What's the job?
            agent:
                Just a simple sale. I got goods worth unloading, and leads who want to buy.
                !overthere
                Only problem is I've been made. There's Spark Barons here in Murder Bay who would definitely recognize me.
                I need you to approach the buyers on my behalf. Convince them to come to me so we can close the deal.
        ]],

        DIALOG_ACCEPT_JOB = [[
            player:
                Sure. I'm in. 
                !happy
                $neutralMischievous
                It'd be fun to stick it to the Barons, for old time's sake.
            agent:
                Sure. I mean, if you're not having fun, what's the point, right?
                !give 
                Here's where to meet me once you make contact with the buyer.
        ]],
        DIALOG_DECLINE_JOB = [[
            player:
                You know, I think I'd like to leave that chapter of my life buried deep.
                No sense digging up old trauma.
            agent:
                Please reconsider. I'll be here for the rest of the day if you do.
        ]]

    }
    :Hub(function(cxt) 

        cxt:Opt("OPT_ASK_ABOUT")
            :IsHubOption(true)
            :GoTo("STATE_QUESTIONS")


        cxt:Opt("OPT_ASK_ABOUT_JOB")
            :Dialog("DIALOG_ASK_ABOUT_JOB")
            :SetQuestMark(cxt.quest)
            :ReqCondition(QuestUtil.CanTakeSideJob(), "REQ_SIDE_JOB_LIMIT")
            :Fn(function(cxt) 
                StateGraphUtil.PresentQuestOffer(cxt, cxt.quest, nil, 
                    function() 
                        cxt:Dialog("DIALOG_ACCEPT_JOB")
                        cxt.quest:AssignCastMember("buyer1")
                        cxt.quest:AssignCastMember("buyer2")
                        cxt.quest:Activate("contact_buyer_1")
                        cxt.quest:Activate("contact_buyer_2")

                        StateGraphUtil.AddEndOption(cxt)
                    end, 
                    function() 
                        cxt:Dialog("DIALOG_DECLINE_JOB")
                        StateGraphUtil.AddEndOption(cxt)
                    end)
                end)

    end)

    :AskAboutHub("STATE_QUESTIONS", 
    {
        "Ask about smuggling",
        [[
            player:
                Is it "light" smuggling 'cause the cargo ain't heavy, or because the law-breaking aspect is kind of ambiguous?
            agent:
                The second one. First one depends on how strong you are, I guess.
            player:
                !interest
                And what brings you here? You got a day pass or something?
            agent:
                !chuckle
                Hah! Not bloody likely. 
                I've been brought along as part of the Barons' entourage. They need a laborer like me to do the heavy lifting.
            player:
                !dubious 
                $neutralJoke
                Aha. Now I see where the smuggling comes in. 

        ]],
        "Ask about the hand signal",
        [[
            player:
                So what was that weird salute all about?
            agent:
                !eureka
                The Rise is working to liberate the workers, so we have to be discreet around the Spark Barons.
                The signal helps us tell friend from foe. 
                !thought
                Or at least it does in the bog. Here it just makes us look silly.
            player:
                !dubious
                Huh. I hope that works out.
                Didn't go so well the last time, in my recollection.
            agent:
                !interest
                You were there for that?
            player:
                $neutralResigned
                I don't remember much of how it started. But I sure remember how it ended.
        ]],    
        "Ask about Grout Bog",
        [[
            player:
                You know anyone by the name of Vapin? Or Ecta? Or maybe Prindo?
            agent:
                Never heard of the first two. But how do <i>you</i> know Prindo Kalandra?
            player:
                !interest
                $miscEmbarrassed
                We were... friends, way back when. She's still alive?
            agent:
                And kicking. Often literally. 
            player:
                !placate
                $miscRelieved
                Hold it. Whatever you're involved with, don't say anything that might get her in trouble.
            agent:
                !dubious
                $neutralDubious
                I can get a message to her, if you want. When I go back.
            player:
                !bashful
                I—
                $neutralDirect
                No. I wouldn't even know what to say.
            agent:
                $neutralJoke
                "I miss you," is a good start, in my experience.
            player:
                !palm 
                Gah. Can we just change the subject please? 
            agent:
                Your call.
        ]],
        "Ask about the Spark Barons",
        [[
            player:
                So the Spark Barons still hold dominion in Grout Bog, huh?
                Kinda expected the Cult might have kicked them out by now.
            agent:
                Nope. The treaty's intact. Though the Barons certainly know they can use the Cult as a threat to keep us in line.
                !permit
                Things are bad. You know, we could probably use someone like you on our side.
            player:
                !crossed
                Once was enough, thanks. But good luck. 
                And I mean it.
        ]],

    })

    :AttractState("foo")
        :Loc{
            DIALOG_OFFER_FIRST_TIME = [[
                * {agent} flashes you a complicated hand signal, and then watches you expectantly.
                agent:
                    !risesalute
                player:
                    !dubious
                    Uh... Can I help you?
                agent:
                    !sigh
                    Sorry, I keep forgetting where I am. I take it you've never been to Grout Bog?
                player:
                    !crossed
                    Actually, I was born there.
                    $angryDefault
                    Bad memories.
                agent:
                    !agree
                    Ah, sure. Those are the primary export, after all. 
                    But you made it out, which means you might just be exactly who I'm looking for.
                    How do you feel about a bit of light smuggling?
            ]],
            DIALOG_OFFER_SUB = [[
                agent:
                    Change your mind about the job?
            ]],
        }
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                if not cxt.quest.param.has_seen_first_offer then
                    cxt.quest.param.has_seen_first_offer = true
                    cxt:Dialog("DIALOG_OFFER_FIRST_TIME")
                else
                    cxt:Dialog("DIALOG_OFFER_SUB")
                end
            end
        end)

