local BOUNTY = 200
local DEATH_PERCENTAGE = .6

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    precondition = function(quest)
        return Util.IsPlayerWorkingWithFaction("BANDITS")
    end,
}

:AddCast{
    cast_id = "spree",
    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "BANDIT_RAIDER" )
        table.insert(t, agent)

    end,
}

:AddCast{
    cast_id = "bounty_hunter",
    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "JAKES_SMUGGLER" )
        table.insert(t, agent)
    end,
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_BOUNTY"
    end)

    :State("STATE_BOUNTY")
        :Loc{
            DIALOG_INTRO = [[
                
                * This part of the road is usually pretty quiet, but you come across a bit of a ruckus all the same. 
                * Seems a bounty hunter has finally tracked down their quarry. 
                
                bounty_hunter:
                    !right
                spree:
                    !left
                bounty_hunter:
                    The bounty doesn't specify you have to be dead, {spree}. So come quietly and you won't get hurt.
                spree:
                    !crossed
                    Naw. I think I'm good with a bit of hurt, if it means putting you in the grave. 
                    You've been tailing me too long. 
                player:
                    !left
                    Having trouble here, hunter?
                bounty_hunter:
                    !right
                    Hm. I've heard of you. You're {player}, aren't you?
                    Word is you're the real deal. 
                    You want in on this bounty? Between the two of us it'll be easy pickings.
            ]],
            DIALOG_HELP = [[
                player:
                    !happy
                    Sounds fun.
                spree:
                    !right
                    !angry
                player:
                    !left
                    Well, {spree}, let's see if your big words actually count for something.
            ]],
            DIALOG_DEFEND = [[
                player:
                    'Fraid not, hunter. {spree} is with me. 
                    Walk away and there'll be no hard feelings, alright?
                bounty_hunter:
                    !angry
                    No hard feelings? I've been tracking this mark for days!
                    !angry_accuse
                    You're just trying to steal my take. Well, guess what? I can take you both on. 
            ]],
            DIALOG_LEAVE = [[
                player:
                    And deprive you of the satisfaction? Never.
                    It's all yours. Have fun, chief.
                * You quickly make your exit before you can get dragged into someone else's troubles.
            ]],
            DIALOG_DEFENDED_KILL = [[
                player:
                    !left
                    !tired
                    Phew. Not the first tussle I've had with another hunter, but they never get easier.
                spree:
                    !right
                    !happy
                    No kidding. Glad I had you there to save my skin. 
                player:
                    !point
                    Don't mention it. But try to get better at staying out of trouble.
                spree:
                    !chuckle
                    If I did that, what would we have in common?
            ]],
            DIALOG_DEFENDED_BEAT = [[
                player:
                    !left
                bounty_hunter:
                    !injured
                    !right
                    Thought you were a hunter, Sal.
                    You know the profit's going the other way, right?
                player:
                    Yeah, well. Sometimes I can afford to lose a few shills. 
                    Now get outta here. And don't let me catch you coming after {spree} again.
                bounty_hunter:
                    !exit
                    * The hunter limps away, no doubt cursing your name into the dirt.
                spree:
                    !right
                    I probably could've handled that on my own, you know. 
                player:
                    Oh yeah?
                spree:
                    Yeah. I mean, maybe. 
                    !happy
                    $happyDefault
                    So I guess what I'm saying is thanks. 
            ]],
            DIALOG_HELPED_KILL = [[
                player:
                    !left
                    !tired
                    Phew. Put up a fight, that one did.
                bounty_hunter:
                    !right
                    Made a mess <i>and</i> halved our profits. The bounty was worth more alive.
                    !give
                    All the same, here's your half. At least we made quick work of it.
                player:
                    !take
                    Appreciate the business. Good luck out there, hunter.
                bounty_hunter:
                    Same to you, hunter.
            ]],
            DIALOG_HELPED_BEAT = [[
                player:
                    !left
                    !tired
                    Well, two heads really <i>are</i> better than one. 
                bounty_hunter:
                    !right
                    No kidding. And the bounty's worth more alive.
                    !give
                    Here's your half, more than fairly earned! 
                    I'll turn in the mark to claim the rest.
                player:
                    Good luck out there, hunter.
                bounty_hunter:
                    Same to you, hunter.
            ]],
            DIALOG_STALEMATE_DEFENDING = [[
                *You defeated {bounty_hunter}, but couldn't save {spree}.
            ]],
            DIALOG_STALEMATE_HELPING = [[
                *You defeated {spree}, but couldn't save {bounty_hunter}. {spree} isn't worth anything dead.
            ]],
            OPT_HELP = "Help {bounty_hunter} capture {spree}",
            OPT_DEFEND = "Defend {spree}",
            OPT_LEAVE = "Leave them to figure it out"
        }

        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_HELP")
                :Dialog("DIALOG_HELP")
                :ReceiveOpinion( OPINION.FOUGHT_WITH, nil, cxt:GetCastAgent( "bounty_hunter" ) )
                :Battle
                {
                    allies = { "bounty_hunter" },
                    enemies = { "spree" },
                    on_win = function(cxt)
                        if cxt:GetCastAgent("bounty_hunter"):IsDead() then
                            cxt:Dialog("DIALOG_STALEMATE_HELPING")
                        else
                            if cxt:GetCastAgent("spree"):IsDead() then
                                cxt:Dialog("DIALOG_HELPED_KILL")
                                cxt.encounter:GainMoney( math.floor(BOUNTY * DEATH_PERCENTAGE) )
                            else 
                                cxt:Dialog("DIALOG_HELPED_BEAT")
                                cxt.encounter:GainMoney( BOUNTY )
                                -- positive opinion event
                            end
                        end
                        StateGraphUtil.AddLeaveLocation(cxt) 
                    end
                }

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :ReceiveOpinion( OPINION.HELPED, nil, cxt:GetCastAgent( "spree" ) )
                :Battle
                {
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = { "spree" },
                    enemies = { "bounty_hunter" },
                    on_win = function(cxt)
                        if cxt:GetCastAgent("spree"):IsDead() then
                            cxt:Dialog("DIALOG_STALEMATE_DEFENDING")
                        else
                            if cxt:GetCastAgent("bounty_hunter"):IsDead() then
                                cxt:Dialog("DIALOG_DEFENDED_KILL")
                            else
                                cxt:Dialog("DIALOG_DEFENDED_BEAT")
                            end
                        end
                        StateGraphUtil.AddLeaveLocation(cxt) 
                    end
                } 

            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :ReceiveOpinion( OPINION.DID_NOT_HELP, nil, cxt:GetCastAgent( "spree" ) )
                :ReceiveOpinion( OPINION.DID_NOT_HELP, nil, cxt:GetCastAgent( "bounty_hunter" ) )
                :Travel()

        end)