local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
}

--------------------------------------------------------------------

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_FIND_STUFF"
    end)

    :State("STATE_FIND_STUFF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !crossed
                    * You come across the remains of an unfortunate traveler.
                    * Whatever attacked him took the flesh but left the wealth: something interesting pokes out from under the corpse.
                    * Of course, whatever did this might still be around.
            ]],
            OPT_KEEP_MOVING = "Leave before anything sees you",
            OPT_GATHER_LOOT = "Gather up the loot",
            DIALOG_GATHER_LOOT = [[
                * You gather up what you can find. You'll have to study it a little more closely when it's safe.
            ]],
            DIALOG_KEEP_MOVING = [[
                * You keep moving. You feel bad for the corpse on the ground, but that doesn't mean you want to keep it company.
            ]],

            DIALOG_SAFELY_LEAVE = [[
                * You leave in a hurry, not wanting to push your luck.
            ]],

            DIALOG_MONSTERS = [[
                * Unfortunately, you loiter long enough to make yourself a target. 
                agent:
                    !right
                player:
                    !surprised
            ]],
            DIALOG_WON_FIGHT = [[
                agent:
                    !exit
                * You leave before more monsters can arrive.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }

        :SetLooping()
        :Fn(function(cxt)
            cxt.quest:Complete()
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_GATHER_LOOT")
                :Dialog("DIALOG_GATHER_LOOT")
                :Fn(function(cxt) 

                        local amt = math.round( 100 * (math.randomGauss(.5, 1.5)))
                        cxt.enc:GainMoney(amt)

                        if math.random() < .5 then
                            cxt:Dialog("DIALOG_SAFELY_LEAVE")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        else
                            
                            local rank = TheGame:GetGameState():GetCurrentBaseDifficulty()
                            local backup = CreateCombatParty("MONSTERS", rank, cxt.player:GetLocation())
                            cxt.encounter:SetPrimaryCast( backup[1] )

                            cxt:Dialog("DIALOG_MONSTERS")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    on_win = function(cxt) 
                                        cxt:Dialog("DIALOG_WON_FIGHT")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }


                        end

                    end)
            
            cxt:Opt("OPT_KEEP_MOVING")
                :Dialog("DIALOG_KEEP_MOVING")
                :Fn(function(cxt) 
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
        end)

