local battle_defs = require "battle/battle_defs"
local negotiation_defs = require "negotiation/negotiation_defs"

local food_options =
{
    CRAYOTE_CASSEROLE =
    {
        condition = function()
            local player = TheGame:GetGameState():GetPlayerAgent()
            local current_resolve, max_resolve = TheGame:GetGameState():GetCaravan():GetResolve()
            return player:GetAspect("health"):Get() < player:GetAspect("health").max or current_resolve < max_resolve
        end,
        effect = function(cxt)
            ConvoUtil.DoHealthDelta(cxt, 4 )
            ConvoUtil.DoResolveDelta(cxt, 4 )
        end,
    },

    FLEAD_STEW =
    {
        condition = function()
            local current_resolve, max_resolve = TheGame:GetGameState():GetCaravan():GetResolve()
            return current_resolve < max_resolve
        end,
        effect = function(cxt)
            ConvoUtil.DoResolveDelta(cxt, 5 )
        end,
    },

    SAUTEED_GRUBS =
    {
        condition = function()
            local current_resolve, max_resolve = TheGame:GetGameState():GetCaravan():GetResolve()
            return current_resolve < max_resolve
        end,
        effect = function(cxt)
            ConvoUtil.DoResolveDelta(cxt, 8 )
        end,
    },
    FURRUSK_LIVER_SANDWICH =
    {
        effect = function(cxt)            
            cxt:ForceTakeCards{"well_fed"}
        end,
    },

    KEYBONE_SQUARES =
    {
        effect = function(cxt)
            cxt:ForceTakeCards{"hearty"}
        end,
    },

    PICKLED_EYES = 
    {
        bad = true,
        effect = function(cxt)
            cxt:ForceTakeCards{"ulcer"}
        end,
    },

    SALTED_SNAIL_EGGS = 
    {
        bad = true,
        effect = function( cxt )           
            cxt:ForceTakeCards{"sick"}
        end, 
    },
}

--Card from food effects
Content.AddBattleCard( "well_fed", 
{
    name = "Well Fed",
    anim = "taunt",
    flavour = "''Urp'.'",
    desc = "{HEAL 5}",

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    cost = 0,
    target_type = TARGET_TYPE.SELF,

    flags = battle_defs.CARD_FLAGS.STATUS | battle_defs.CARD_FLAGS.CONSUME,

    OnPostResolve = function( self, battle, attack)
        self.owner:HealHealth( 5, self )
    end,
})
Content.AddNegotiationCard("hearty", 
{
    name = "Hearty",
    desc = "Restore 5 resolve.",
    flavour = "'A nice and juicy Vroc roast! Just like Mom used to make!'",
    
    cost = 0,
    flags = negotiation_defs.CARD_FLAGS.STATUS | negotiation_defs.CARD_FLAGS.CONSUME,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    OnPostResolve = function( self, minigame )
        self.negotiator:ModifyResolve(5, self)
    end,
})
local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    on_init = function(quest)
        if not quest.param.options then
            local temp_options = {}
            for i, id, data in sorted_pairs( food_options ) do
                --if there's no condition or the condition is met, add it
                if data.condition == nil or data.condition() then
                    table.insert(temp_options, id)
                end
            end
            table.shuffle(temp_options)
            local length = #temp_options
            for i=1, length - 3 do
                table.remove(temp_options, math.random(#temp_options))
            end
            quest.param.options = temp_options
        end
    end,
}

QDEF:AddCastByAlias{
    cast_id = "chef",
    alias = "WANDERING_CHEF",
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_CONFRONT"
    end)

    :State("STATE_CONFRONT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !happy
                * You're stopped on the road by {Agent}, whose face lights up as {agent.heshe} sees you. It's not the reaction you're used to.
                agent:
                    Now <i>you</i> look hungry. Am I right? And I got just the thing for it: <i>delicious</i>, non-toxic food. 
                player:
                    !dubious
                    Food on the road, huh? What's the catch?
                agent:
                    !thought
                    Well... it's food on the road, like you said. But I'm testing out a new recipe, and I'll cook it if you'll eat it. 
            ]],
            DIALOG_ACCEPT = [[
                player:
                    !happy
                    !handwring
                    Free food <i>is</i> my only weakness. I'll try the {1}!
                agent:
                    !surprised
                    H-huh, really? 
                    !happy
                    I mean, great choice! I'll get started now. 
            ]],
            DIALOG_DECLINE = [[
                player:
                    Tempting. But I just ate!
                * Your stomach growls and you try and muffle it with the crook of your arm.
                agent:
                    !angry
                    Huh. I'll try not to be offended. But I won't succeed. 
                * {agent} casts you a haughty look before continuing on {agent.hisher} way down the road. 
            ]],            
            OPT_DECLINE = "Decline the offer",
            OPT_TRY_FOOD_CRAYOTE_CASSEROLE = "Try the crayote courage casserole",
            DIALOG_CRAYOTE_CASSEROLE = [[
                player:
                    !drink
                * You get a piece of yote gristle stuck between your teeth. But otherwise, it's not bad!
                    agent:
                        !interest
                        So? Whaddya think?
                    player:
                        !happy_shrug
                        Pretty good, for street meat!
                    agent:
                        !happy
                * You got a free meal, and {agent} got a compliment. Seems like you both made out ahead.
            ]],
            OPT_TRY_FOOD_FLEAD_STEW = "Try the blackened flead guts",
            DIALOG_FLEAD_STEW = [[
                player:
                    !drink
                * The black sauce smells like peat, but leaves you smacking your lips all the same. 
                player:
                    !burp
                    Definitely the tastiest flead guts I ever ate. Thanks, {agent}!
                agent:  
                    !happy
                * {agent} leaves you to your digestion, looking pleased as punch.
            ]],
            OPT_TRY_FOOD_SAUTEED_GRUBS = "Try the griddled gristle",
            DIALOG_SAUTEED_GRUBS = [[
                player:
                    !drink
                * It takes a minute to chew through the gelatinous gravy, but the taste is mighty satisfying.
                player:
                    !burp
                    Filling. And strangely... <i>fulfilling</i>.
                agent:
                    !wink
                * {agent} smiles knowingly as {agent.heshe} packs up {agent.hisher} supplies. 
                    Happy travels, hunter.         
            ]],
            OPT_TRY_FOOD_FURRUSK_LIVER_SANDWICH = "Try the oshnu bile tartare",
            DIALOG_FURRUSK_LIVER_SANDWICH = [[
                player:
                    !drink
                * The tartare threatens to slip between your fingers, but you manage to shovel it in before it gets away.
                agent:
                    !happy
                    Enh? It's good, right? Be detailed with those tasting notes.
                player:
                    !burp
                    Mrff! Mmmhrrrm! <burp>
                * With a hearty laugh, {agent} leaves you to enjoy the aftertaste.
            ]],
            OPT_TRY_FOOD_KEYBONE_SQUARES = "Try the fungus and mettlenettle brownies",
            DIALOG_KEYBONE_SQUARES = [[
                player:
                    !drink
                * The brownies melt on your tongue, tasting only a little bit like fresh fertilizer.
                player:
                    !happy
                    $miscNostalgic
                    Tastes just like the ones my dad used to make.
                agent:
                    !happy
                    Yes, the misty-eyed nostalgia is intentional! 
                * {agent} leaves you to continue down the road and into memory lane.
            ]],
            OPT_TRY_FOOD_PICKLED_EYES = " Try the pickled talons",
            DIALOG_PICKLED_EYES = [[
                player:
                    !drink
                * You feel something puncture your tongue before everything from the ears down goes suddenly numb.
                player:
                    !dubious
                    Mah mowf feelf fummy.
                agent:
                    !placate
                    $neutralTakenAback
                    Well that's awful interesting, mmhmm. No refunds, though.
            ]],
            OPT_TRY_FOOD_SALTED_SNAIL_EGGS = "Try the bogleech sausage",
            DIALOG_SALTED_SNAIL_EGGS = [[
                player:
                    !drink
                    !disappoint
                * You bite down on a caramelized leech, and it explodes into spores.
                player:
                    !burp
                    <kaff!> <hack> Ughh... 
                agent:  
                    !chuckle
                    Aha! You got a good one! No need to thank me, hunter—your reaction is thanks enough.  

            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.encounter:SetPrimaryCast(cxt.quest:GetCastAgent("chef"))

                cxt:Dialog("DIALOG_INTRO")
            end

            for k,v in pairs(cxt.quest.param.options) do
                cxt:Opt("OPT_TRY_FOOD_" .. v)
                    :PreIcon(global_images.food)
                    :Fn(function(cxt)
                        cxt:Dialog("DIALOG_" .. v)
                        if food_options[v].bad then
                            cxt:GoTo("STATE_BAD_FOOD", v)
                        else
                            if food_options[v].effect then
                                food_options[v].effect(cxt)
                            end

                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    end)
            end

        
            cxt:Opt("OPT_DECLINE")
                :Robot("SKIP")
                :PreIcon(global_images.dislike)
                :Dialog("DIALOG_DECLINE")
                :ReceiveOpinion( OPINION.INSULT )
                :Travel()

        end)

    :State("STATE_BAD_FOOD")
        :Loc{
            DIALOG_APOLOGY = [[
                player:
                    !injured
                    Ugh... that was awful.
                agent:
                    !surprise
                    Wha—?! Well, not all my ideas can be keepers, I guess. 
                    !scared
                    $scaredDefault
                    You won't, uh, tell anyone about this, will ya? I'm tryin' to open my own restaurant in Murder Bay...
            ]],
            OPT_DONT_TELL = "Promise not to tell anyone",
            TT_DONT_TELL = "You'll feel the extent of that dubious meal!",

            DIALOG_DONT_TELL = [[
                player:
                    !dubious
                    Mistakes happen. Just, uh, try it out on a yote first next time.
                agent:
                    !thought
                    Dunno if a yote has the palate for it, but... it does get real lonely on the road. 
                * {agent} cleans up {agent.hisher} gear and leaves you to rinse the taste from your mouth.
            ]],
            OPT_INSULT = "Scold {agent}",
            TT_INSULT = "You'll feel better for scolding {agent}, but {agent.heshe} isn't likely to appreciate it.",

            DIALOG_INSULT = [[
                player:
                    !angry_accuse
                    You coulda poisoned me, you damn lunatic!
                agent:
                    !scared
                    W-well you didn't have to take it! There's no accounting for taste!
                    !exit
                * {Agent} kicks up {agent.hisher} heels for a hasty retreat, but not before you splatter {agent.hisher} backside with the leftovers.
                player:
                    !angry
                    And good riddance! Blech!
            ]],
        }
        
        :Fn(function(cxt, food)
            cxt:Dialog("DIALOG_APOLOGY")


            print( food )

            cxt:Opt("OPT_DONT_TELL")
                :PostText( "TT_DONT_TELL" )
                :Dialog("DIALOG_DONT_TELL")
                :ReceiveOpinion( OPINION.SOCIAL_FAVOUR )
                :Fn(function(cxt)
                    if food_options[ food ] then
                        food_options[food].effect(cxt)
                    end
                end)
                :Travel()

            cxt:Opt("OPT_INSULT")
                :Robot("INSULT")
                :PostText( "TT_INSULT" )
                :Dialog("DIALOG_INSULT")
                :ReceiveOpinion( OPINION.INSULT )
                :Travel()
        end)

----------------------------------------------------------------------------------

QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 },
        {name = "Force skip", choice_weights = {SKIP=10}},
    }
}

local foods = {}
for k,v in pairs(food_options) do
    table.insert(foods, {k})
end

QDEF:AddTest(TestCreator()
                :AddParam("options", foods)
                :AddChoices{"INSULT"})
