
local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    is_negative = true,
    on_init = function(quest)
        if quest.param.offer_pet == nil then
            quest.param.offer_pet = math.random() < .5
        end
    end,
}

--------------------------------------------------------------------

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "ACCOST_BATTLE"
    end)

    :State("ACCOST_BATTLE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * One of the downsides of Murder Bay: you can't see two feet off the road. Something rustles in the greenery.
                agent:
                    !right
                player:
                    !surprise
                * Wild animals!

            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                !fight
                
            ]],
            DIALOG_WON = [[
                * It's not your calling to mess with nature. You leave before any more beasts can bring trouble. 
            ]]
        }
        :Fn(function(cxt) 
                cxt.quest:Complete()
                local rank = TheGame:GetGameState():GetCurrentBaseDifficulty()
                local backup = CreateCombatParty("YOTES", rank, cxt.player:GetLocation())
                local leader = backup[1]
                cxt.encounter:SetPrimaryCast( leader )
                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt) 
                            
                            if cxt.quest.param.offer_pet and cxt.caravan:CanOwnPet() then
                                cxt:GoTo("STATE_BABY")
                            else
                                cxt:Dialog("DIALOG_WON")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end
                    }
            end)

    :State("STATE_BABY")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !exit
                * In the silence after the battle, you hear a pathetic yeowl just beyond the treeline.
                pup:
                    !right
                player:
                    !surprise
                * Investigation reveals a mewling yote pup, young enough to still need tending.
                * It looks up at you with big, wide eyes. It might still be able to imprint on you... or it might be too wild to bother. 
            ]],
            OPT_LEAVE_IT = "Leave it",
            OPT_TAKE_IT = "Take the pup",
            DIALOG_LEAVE_IT = [[
                * You leave the pup. Its pitiful cries are no match for your cold, cold heart.
            ]],
            DIALOG_TAKE_IT = [[
                * One look was all it took. You pet the pup and it follows readily at your heel. 
                * You're a yote mom now.
            ]],
        }
        :Fn(function(cxt) 

            local agent = Agent("CRAYOTE")
            TheGame:GetGameState():AddAgent( agent )
            cxt.enc:SetPrimaryCast(agent)
            cxt:ReassignCastMember("pup", agent)
 
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_LEAVE_IT")
                :PreIcon(global_images.close)
                :Dialog("DIALOG_LEAVE_IT")
                :Fn(function() StateGraphUtil.AddLeaveLocation(cxt) end)
            
            cxt:Opt("OPT_TAKE_IT")
                :Robot("TAKE_IT")
                :PreIcon(global_images.invite)
                :PostText("TT_PET_TAKE")
                :Dialog("DIALOG_TAKE_IT")
                :Fn(function()
                    agent:GenerateRandomName()
                    agent:Recruit( PARTY_MEMBER_TYPE.CREW)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            end)


QDEF:AddTest(TestCreator()
                :AddBoolParams{"offer_pet"}
                :AddChoices{"TAKE_IT"})
