local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
Content.AddBattleCard( "unknown_concoction", 
{
    name = "Unknown Concoction",
    anim = "drink",
    desc = "{DRINK}.{1}",
    flavour = "'Drink me.'",
    desc_fn = function( self, fmt_str )
        if type(self.userdata.chosen_effect) == "string" and self.userdata.reveal then
            return loc.format( fmt_str, self.def:GetLocalizedString( self.userdata.chosen_effect ))
        else
            return loc.format( fmt_str, self.def:GetLocalizedString( "UNKNOWN_DESC" ) )
        end
    end,
    ShowEffect = function( self )
        self.userdata.reveal = true
    end,
    OnInit = function( self )
        self.userdata.chosen_effect = self.options[math.random(#self.options)]
    end,
    loc_strings =
    {
        UNKNOWN_DESC = "\nThis concoction has an unknown effect.",
        DUPLICATE = "\nYour next card is played twice, but with a random target.",
        ACTION = "\nGain 2 actions, discard a random card.",
        FREEBIE = "\nA random card in your hand costs 0, but also has {EXPEND}."
    },

    item_tags = ITEM_TAGS.COMBAT,
    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    cost = 1,
    flags = CARD_FLAGS.SKILL | CARD_FLAGS.EXPEND,
    max_charges = 5,
    target_type = TARGET_TYPE.SELF,

    OnPostResolve = function( self, battle, attack )
        local card = Battle.Card( "empty_bottle", self.owner )
        battle:DealCards( {card}, battle:GetDiscardDeck() )
        battle:BroadcastEvent( BATTLE_EVENT.DRINK, battle, 1, {card} )
        if not self.userdata.reveal then
            self:ShowEffect()
        end
        self.effects[self.userdata.chosen_effect](self, battle, attack)
    end,

    options = {"DUPLICATE", "ACTION", "FREEBIE"},

    effects =
    {
        ["DUPLICATE"] = function( self, battle, attack )
            local con = self.owner:AddCondition("DUPLICATION_POTION", 1, self)
            con.skip_card = self
        end,
        ["ACTION"] = function( self, battle, attack )
            battle:ModifyActionCount(2)
            battle:DiscardCard()
        end,
        ["FREEBIE"] = function( self, battle, attack )
            local card = battle:GetHandDeck():PeekRandom()
            if card then
                card:SetFlags(CARD_FLAGS.FREEBIE)
                card:SetFlags(CARD_FLAGS.EXPEND)
            end
        end,
    },
})


Content.AddBattleCondition( "DUPLICATION_POTION", 
{
    name = "Duplication Potion",
    desc = "Your next card is played twice, but with a random target.",

    event_handlers =
    {
        [ BATTLE_EVENT.EXECUTES_ADDED ] = function( self )
            self.owner:RemoveCondition( self.id )
        end,

        [ BATTLE_EVENT.POST_RESOLVE ] = function( self, battle, attack )
            if attack.card ~= self.skip_card and attack.attacker == self.owner then
                self.owner:RemoveCondition( self.id, 1 )
                battle:PlayCard( attack.card )
            end
        end,

        [ BATTLE_EVENT.CALC_TARGET_MOD ] = function( self, acc, card, target )
            if card.owner == self.owner and acc.value ~= TARGET_MOD.TEAM and acc.value ~= TARGET_MOD.ALL then
                acc:ModifyValue( TARGET_MOD.RANDOM1, self )
            end
        end,
    },
})


local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    precondition = function(quest)
        local memory = TheGame:GetGameState():GetPlayerAgent():HasMemory("ABANDONED_CONCOCTION")
        return not memory
    end,
}

--------------------------------------------------------------------

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_FIND_BOTTLES"
    end)

    :State("STATE_FIND_BOTTLES")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    * You spot a conspicuous tarp on the side of the road. 
                    * Rookie mistake: anything worth covering is definitely worth stealing. 
                    * You lift the tarp for a quick peek and find a mostly empty pile of crates, save for a single glass bottle labeled "Batch No. {1}c".
            ]],
            OPT_TAKE_BOTTLE = "Take the <!card_1>{1}</> with you",
            DIALOG_TAKE_BOTTLE = [[
                player:
                    Well, whaddya know. That's my lucky number.
                    !take
                * You shove the bottle into your bag for later enjoyment.
                player:
                    !exit
            ]],
            DIALOG_POST_SAMPLE_TAKE_BOTTLE = [[
                player:
                    This stuff is too strong to just leave lying around.
                    It's my civic duty to see it, uh, disposed of.
                    !take
                * You shove the bottle into your bag for later and quickly leave the scene.
                player:
                    !exit
            ]],
            OPT_SAMPLE = "Have a little taste",
            DIALOGE_SAMPLE = [[
                player:
                    It's a hot day. Stayin' hydrated is a matter of life and death.
                    !drink
                * The concoction burns like ice and tastes like an old man's armpit.
                player:
                    !burp
                    Ooh. Yeah. That's, uh... that's a sipper.
            ]],
            OPT_LEAVE = "Leave the bottle",
            DIALOG_LEAVE = [[
                player:
                    Can see why this one might've been left untouched.
                    !exit
                * You make it two steps before turning around.
                player:
                    !left
                    !thought
                    What if they don't come back though? Then it's just littering.
            ]],
            OPT_REALLY_LEAVE = "Leave the bottle, for real this time",
            DIALOG_REALLY_LEAVE = [[
                player:
                    No! I'm not going to steal someone's stuff.
                    !exit
            ]],
            REQ_ALREADY_TRIED = "You already had a taste",
        }

        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.quest.param.tasted = false
                cxt.quest.param.batch_nr = cxt.quest.param.batch_nr or math.random(100)
                cxt:Dialog("DIALOG_INTRO", cxt.quest.param.batch_nr)
                cxt.quest.param.bottle = cxt.quest.param.bottle or Battle.Card("unknown_concoction", cxt.player)
            end

            cxt:Opt("OPT_TAKE_BOTTLE", cxt.quest.param.bottle:GetName())
                :Dialog(cxt.quest.param.tasted and "DIALOG_POST_SAMPLE_TAKE_BOTTLE" or "DIALOG_TAKE_BOTTLE")
                :Fn(function(cxt)
                    cxt.player.battler:AddCard(cxt.quest.param.bottle)
                    StateGraphUtil.AddLeaveLocation(cxt)
                    cxt.player:Remember("ABANDONED_CONCOCTION", {batch = cxt.quest.param.batch_nr})
                end)
                :SetHoverLinkParameters( cxt.quest.param.bottle )

            cxt:Opt("OPT_SAMPLE")
                :Dialog("DIALOGE_SAMPLE")
                :ReqCondition(not cxt.quest.param.tasted, "REQ_ALREADY_TRIED")
                :Fn(function(cxt)
                    cxt.quest.param.bottle.userdata.charges = cxt.quest.param.bottle.userdata.charges - 1
                    cxt.quest.param.bottle:ShowEffect()
                    cxt.quest.param.tasted = true
                end)

            if cxt.quest.param.tried_leave then
                cxt:Opt("OPT_REALLY_LEAVE")
                    :Dialog("DIALOG_REALLY_LEAVE")
                    :Travel()
            else
                cxt:Opt("OPT_LEAVE")
                    :Dialog("DIALOG_LEAVE")
                    :Fn(function( cxt )
                        cxt.quest.param.tried_leave = true
                    end)
            end
        end)

