local HEALTH_COST = 10
local SHILL_AMOUNT = 100
local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    precondition = function(quest)
        local memory = TheGame:GetGameState():GetPlayerAgent():HasMemory("ABANDONED_CONCOCTION")
        return memory
    end,

    on_init = function(quest)
        local memory = TheGame:GetGameState():GetPlayerAgent():HasMemory("ABANDONED_CONCOCTION")
        TheGame:GetGameState():GetPlayerAgent():Forget("ABANDONED_CONCOCTION")
        quest.param.batch = memory.batch
        quest.param.card = TheGame:GetGameState():GetPlayerAgent().battler:FindCardByID("unknown_concoction")
    end,
}

--------------------------------------------------------------------

QDEF:AddCastByAlias{
    cast_id = "stranger",
    alias = "DANGEROUS_STRANGER",
}

QDEF:AddOpinionEvents{
    gave_back_bottle = {
        delta = OPINION_DELTAS.OPINION_UP,
        txt = "Gave them the rest of their concoction",
    },
    gave_tissue_sample = {
        delta = OPINION_DELTAS.OPINION_UP,
        txt = "Gave them a tissue sample to study",
    },
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_INTRO"
    end)

    :State("STATE_INTRO")
        :Loc{
            DIALOG_INTRO = [[
                * You're used to being noticed, between your rugged good looks and your roguish charm.
                * But as you pass {stranger} on the road, {stranger.hisher} head snaps in your direction so fast you'd wonder {stranger.heshe} even has a spine.
                agent:
                    !right
                    Hey! You!
                player:
                    !left
                    That's my name. In most of Havaria, anyway. Whaddya need?
                agent:
                    Batch {1}c. Sound familiar?
                player:
                    !surprised
                    Uh. Weird question to ask out of the blue.
                agent:
                    Hardly. There's a faint smell of marshmint on your breath, and a dark vein of ichor creeping into your eyestalk.
                player:
                    !cough
                    There is?
                agent:
                    It's subtle, but I know what to look for. Even just having Batch {1}c on your person is enough to contaminate the system.
                    Please, do you happen to have any left? It's vital for my research.
            ]],
            OPT_GIVE_REST = "Offer {stranger} the rest",
            TT_GIVE_REST = "Lose {1#card}",
            DIALOG_GIVE_REST_DRANK_LITTLE = [[
                player:
                    Hey, I don't wanna be contaminated! Take it!
                    !give
                agent:
                    !take
                    Thank you! Likely your blood levels are low, and it should subside without further effects.
                    But, ah, since you happen to currently be at the peak of your exposure... could I trouble you for a tissue sample?
                    Paid, of course!
            ]],
            DIALOG_GIVE_REST_DRANK_LOT = [[
                player:
                    Uh, boy. I'm afraid I did a lot more than just keep it in my pocket.
                    There's a little bit left, though. 
                    !give
                agent:
                    !take
                    Oh, dear. You <i>drank</i> all this?
                player:
                    It was an acquired taste.
                agent:
                    And yet you're still standing. Fascinating.
                    Then I request a consolation prize for my studies: could I take a small tissue sample?
                    It would be as inconspicious and painless as I can manage. But I'll pay you for it!
            ]],
            OPT_LIE = "Pretend it's empty",
            DIALOG_LIE = [[
                player:
                    Oh, uh, I only found the empty bottle.
                agent:
                    !disappoint
                    Oh? That's a shame.
                    Still... for your blood levels to be so high even from just vapor exposure... 
                    Could I bother you for a tissue sample? That alone will at least advance the field.
            ]],
            OPT_NO_BOTTLE = "Explain that it's gone",
            DIALOG_NO_BOTTLE = [[
                player:
                    Wish I could help you, 'specially now that you mention, uh, contamination.
                    !bashful
                    But I drank it. The whole bottle.
                agent:
                    !surprised
                    T-that's not possible. It <i>can't</i> be possible.
                    Can it?
                player:
                    Baby, I make the impossible possible on a daily basis.
                agent:
                    Please, you have to let me take a tissue sample. You have no idea what that would mean for science.
            ]],
            REQ_NO_BOTTLE = "You no longer have the concoction",
        }
        :SetLooping()
        :Fn(function(cxt)
            cxt.quest:Complete()            
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("stranger"))
                cxt:Dialog("DIALOG_INTRO", cxt.quest.param.batch)
            end

            cxt:Opt("OPT_GIVE_REST")
                :ReqCondition(cxt.quest.param.card, "REQ_NO_BOTTLE")
                :ReceiveOpinion("gave_back_bottle")
                :PostText("TT_GIVE_REST", cxt.quest.param.card or "unknown_concoction")
                :Fn(function(cxt)
                    local cur, max = cxt.quest.param.card:GetCharges()
                    if cur <= 0.5 * max then
                        cxt:Dialog("DIALOG_GIVE_REST_DRANK_LOT")
                    else
                        cxt:Dialog("DIALOG_GIVE_REST_DRANK_LOT")
                    end
                    cxt.player.battler:RemoveCard(cxt.quest.param.card)
                    cxt.quest.param.gave_bottle = true
                    cxt:GoTo("STATE_TISSUE_SAMPLE")
                end)

            if cxt.quest.param.card then
                cxt:Opt("OPT_LIE")
                    :Dialog("DIALOG_LIE")
                    :GoTo("STATE_TISSUE_SAMPLE")

            else
                cxt:Opt("OPT_NO_BOTTLE")
                    :Dialog("DIALOG_NO_BOTTLE")
                    :GoTo("STATE_TISSUE_SAMPLE")
            end
        end)

    :State("STATE_TISSUE_SAMPLE")
        :Loc{
            OPT_GIVE_TISSUE_SAMPLE = "Allow {stranger} to take a tissue sample",
            DIALOG_SAMPLE = [[
                player:
                    Uh, yeah I suppose that couldn't hurt.
                agent:
                    Oh, well... it will. But I'll be quick!
                    !exit
                player:
                    !exit
                * {agent} gets out {agent.hisher} tools and takes a small tissue sample.
                agent:
                    !right
                player:
                    !left
                    !wince2
                    Yeowch!
                agent:
                    That's a good lad. Very brave!
                    Let me just put a little bandage on the boo-boo and...
                    Such a soldier! And you've contributed so much to progress.
            ]],
            OPT_REFUSE_SAMPLE = "Refuse",
            DIALOG_REFUSE_NO_BOTTLE = [[
                player:
                    Tissue sample? I like my tissue where it's at, thank you very much.
                agent:
                    I see. Well it's not like I can force you.
                    Although clearly, next time I'm fortunate enough to replicate the formula, I need to put fine print on the label.
                    "If you steal, I get a peel".
                player:
                    Wow. 
            ]],
            DIALOG_REFUSE_GAVE_BOTTLE = [[
                player:
                    Tissue sample? I like my tissue where it's at, thank you very much.
                agent:
                    Disappointing, but understandable. Still, even observing the effects on you will be of note.
            ]],
        }
        :Fn(function(cxt)
            cxt:Opt("OPT_GIVE_TISSUE_SAMPLE")
                :DeltaHealth(-HEALTH_COST)
                :ReceiveOpinion("gave_tissue_sample")
                :Dialog("DIALOG_SAMPLE")
                :ReceiveMoney(SHILL_AMOUNT)
                :Fn(function(cxt)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_REFUSE_SAMPLE")
                :Dialog(cxt.quest.param.gave_bottle and "DIALOG_REFUSE_GAVE_BOTTLE" or "DIALOG_REFUSE_NO_BOTTLE")
                :Fn(function(cxt)
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)

