local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
}

--------------------------------------------------------------------

QDEF:AddConvo()
    :ConfrontState("CONF", function(cxt) return cxt.location:HasTag("in_transit") end)
    :Loc{
        DIALOG_INTRO = [[
            * You hear a scuffle down one of the side roads, and follow your nose to the somewhat sharp smell of trouble.
            * Two civilians are being attacked by heshbladders.
        ]],
        OPT_LEAVE = "Leave before anyone sees you",
        DIALOG_LEAVE = [[
            * You decide that they'll probably be fine without you. Plus those things are gross and your clothes are (somewhat) clean.
        ]],
        OPT_DEFEND = "Help get rid of the heshbladders",
        DIALOG_DEFEND = [[
            player:
                !left
                !sigh
                Hey, bladders. You're full of it.
                !fight
        ]],
        DIALOG_WON_ALL_DEAD = [[
            player:
                !left
            * You've made a mess, but you figure it's someone else's job to clean up.
        ]],
        DIALOG_WON_SOME_DEAD = [[
            player:
                !left
                Yikes, those things really beat the piss outta ya, huh?
            agent:
                !right
                !injured
                Thank you. It's inspiring to see a good samaritan in these strange times.
            player:
                Wassat? Some kind of bird?
                Hopefully it's not drawn to the smell of heshbladder because, uh, you got it all over you.
        ]],
        DIALOG_WON_NONE_DEAD = [[
            agent:
                !right
            player:
                !left
                Phew! You're lucky I give as good as I get, huh?
                No need to thank me all at once.
            agent:
                Hesh brought you in our path today, for certain.
            player:
                Eh, I wouldn't go too crazy. Heshbladders are named after the big guy, after all. 
                I'm just named after my great aunt.
        ]],
        OPT_ASK_FOR_REWARD = "Ask for a reward",
        DIALOG_REWARD = [[
            player:
                Hey, uh, speakin' of your undying gratitude and all...
                Maybe you wanna thank the fates, and me, with a little financial incentive?
            agent:
                Huh?
                Oh, right. I guess some payment is fair.
                I don't have much, but here's {1#money}.
                !give
            player:
                !take
        ]],
        OPT_REJECT_REWARD = "Let them go without a reward",
        DIALOG_REJECT_REWARD = [[
            player:
                Stay safe out there. No tellin' what you might run into in the Foam these days.
            agent:
                Thank you. This won't be forgotten.
        ]],
    }
    :Fn(function(cxt)
        local SAVE_FEE = 40 + 25*cxt.quest:GetRank()
        local defs = {
            "WEALTHY_MERCHANT",
            "LUMINITIATE"
        }
        cxt.quest:Complete()

        cxt.quest.param.victims = {}
        for i=1,2 do
            local agent = AgentUtil.GetFreeAgent(table.arraypick(defs), function(agent) return agent:GetRelationship() == RELATIONSHIP.NEUTRAL end)
            agent:GetAspect("health"):SetPercent(0.4)
            table.insert(cxt.quest.param.victims, agent)
        end
        cxt.quest.param.heshbladders = CreateCombatParty({"HESHBLADDER", "HESHBLADDER"}, cxt.quest:GetRank(), cxt.location)
        cxt:TalkTo(cxt.quest.param.victims[1])

        cxt:Dialog("DIALOG_INTRO")

        cxt:Opt("OPT_DEFEND")
            :Robot("DEFEND")
            :Dialog("DIALOG_DEFEND")
            :Battle{
                no_oppo_limit = true,
                allies = cxt.quest.param.victims,
                enemies = cxt.quest.param.heshbladders,
                flags = BATTLE_FLAGS.NO_BYSTANDERS,
            }
                :OnWin()
                    :Fn(function() 
                        local alive_agents = {}
                        for i,agent in ipairs(cxt.quest.param.victims) do
                            if not agent:IsDead() then
                                table.insert(alive_agents, agent)
                            end
                        end

                        if #alive_agents == 0 then
                            cxt:Dialog("DIALOG_WON_ALL_DEAD")
                        else
                            cxt:TalkTo(alive_agents[1])
                            if #alive_agents < #cxt.quest.param.victims then
                                cxt:Dialog("DIALOG_WON_SOME_DEAD")
                            else
                                cxt:Dialog("DIALOG_WON_NONE_DEAD")
                            end
                            cxt:Opt("OPT_ASK_FOR_REWARD")
                                :Dialog("DIALOG_REWARD", SAVE_FEE)
                                :ReceiveMoney(SAVE_FEE)
                                :Travel()

                            cxt:Opt("OPT_REJECT_REWARD")
                                :Dialog("DIALOG_REJECT_REWARD")
                                :ReceiveOpinion(OPINION.SAVED_LIFE)
                                :Fn(function(cxt)
                                    for i,agent in ipairs(cxt.quest.param.victims) do
                                        if not agent:IsDead() and agent ~= cxt:GetAgent() then
                                            agent:OpinionEvent( OPINION.SAVED_LIFE )
                                        end
                                    end
                                end)
                                :Travel()
                        end
                    end)
        

        cxt:Opt("OPT_LEAVE")
            :Robot("LEAVE")
            :Dialog("DIALOG_LEAVE")
            :Fn(function() 
                for i,agent in ipairs(cxt.quest.param.victims) do
                    if not agent:IsDead() then
                        agent:Kill()
                    end
                end
            end)
            :Travel()
    end)
       

QDEF:AddTest(TestCreator()
                :AddChoices{"DEFEND", "LEAVE"})
