local battle_defs = require "battle/battle_defs"

local JUNK_PARTS = { "emergency_shield_generator", "enfeebling_ray", "nano_lattice", "fuel_canister", "spark_shot",
                "targeting_core", "energy_siphon", "replicator", "shock_box", "defective_grenade", "shock_core", "support_beacon" }
for i, card_id in ipairs( JUNK_PARTS ) do
    assert( Content.GetBattleCard( card_id ), card_id )
end

AddOpinionEvent("BROKE_THEIR_MECH", 
{
    delta = OPINION_DELTAS.OPINION_DOWN,
    txt = "Destroyed their automech",
})

Content.AddBattleCondition("MALFUNCTION",
{
    name = "Malfunction",
    desc = "Cause {1} exactly {2} damage to 'repair' the malfunction. {1} will surrender.",
    desc_fn = function( self, fmt_str )
        return loc.format( fmt_str, self.owner:GetName(), self.repair_damage )
    end,

    event_handlers =
    {
        [ battle_defs.BATTLE_EVENT.BEGIN_PLAYER_TURN ] = function( self )
            if self.repair_damage == nil then
                self.repair_damage = self.stacks * math.random( 1, 8 )
            end
        end,

        [ battle_defs.BATTLE_EVENT.DAMAGE_APPLIED ] = function( self, fighter, damage, delta, source )
            if fighter == self.owner and damage == self.repair_damage then
                fighter:RemoveCondition( "MALFUNCTION" )
                fighter:Surrender()
            end
        end,
    }

})

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    on_init = function( quest )
        quest.param.aggro = math.random() > 0.5
    end,
}

:AddCast{
    cast_id = "baron",
    condition = function(agent, quest) 
        return agent:GetContentID() == "SPARK_BARON_GOON"
    end,
}:AddCastFallback( "SPARK_BARON_GOON" )

:AddCast{
    cast_id = "mech",
    no_validation = true,
    cast_fn = function( quest, t )
        table.insert( t, quest:CreateSkinnedAgent( "SPARK_BARON_AUTOMECH" ) )
    end
}

QDEF:AddConvo()
    :ConfrontState("CONF", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                mech:
                    !left
                baron:
                    !right
                    !angry
                    Stupid mech!
                    I'll disassemble you, I will!
                player:
                    !left
                    Huh. Do threats work on those metal guys?
                    I'll have to try it next time.
                baron:
                    !sigh 
                    No, threats don't work. They're not people, no matter how stubborn they can be.
                    The machine won't move, and so I'm stuck here along with it.
                player:
                    !thought
                    When I don't move, it's usually because someone's trying to make me.
                    Have you tried not trying?
                baron:
                    ...
                    That's the dumbest thing I ever heard.
            ]],

            OPT_KICK_IT = "Kick the automech",
            OPT_KICK_IT_AGAIN = "Kick it again",

            DIALOG_FIRST_KICK = [[
                player:
                    !point
                    Kicking solves a surprising number of problems in life. 
                    Mind if I...?
                * You don't wait for permission, and give the mech a hearty kick.
                mech:
                    !right
            ]],
            DIALOG_ANOTHER_KICK = [[
                mech:
                    !right
                player:
                    Sometimes it just takes a few...
                * You widen your hips and give it another go.
            ]],
            DIALOG_KICK_IT = [[
                mech:
                    !right
                * You give the lifeless hunk of metal a solid <i>whack<i> with your foot. It hurts, but the mech doesn't move.
                !shake
                player:
                    Hmm...
            ]],

            DIALOG_KICK_IT_TO_LIFE = [[
                mech:
                    !right
                * You give the lifeless hunk of metal a solid <i>whack<i> with your foot and the mech suddenly whirrs to life.
                !shake
            ]],

            PRE_BATTLE = [[
                mech:
                    !right
                    MALFUNCTION. ATTACKING RANDOMLY.
                player:
                    !scared
                    I guess this is how I act when someone kicks me, too.
                    !fight
                    Ah well. In another life, mech!
            ]],

            DIALOG_BROKE_IT = [[
                {baron_alive?
                    baron:
                        !right
                        !disgusted
                        You... broke it.
                    player:
                        Umm.
                        !bashful
                        See ya!
                }
                {not baron_alive?
                    * You shuffle off before anyone sees the mess you made.
                }
            ]],
            DIALOG_WOW_OK = [[
                {baron_alive?
                    baron:
                        !right
                    player:
                        !shrug
                        See.
                    baron:
                        Huh.
                        Well, thanks!
                }
                {not baron_alive?
                    * {baron} is dead.
                    mech:
                        !right
                        CONFUSING FEELINGS.
                    player:
                        You and me both, buddy.
                        !exit
                }
            ]],

            OPT_HAUL_IT = "Offer to haul the mech (Receive {1#card})",
            DIALOG_HAUL_IT = [[
                player:
                    !permit
                    Well, I'm big and strong, as you can see.
                    I might regret this, but I could help you haul this hunk of metal for some coinage.
                baron:
                    !right
                    Fine.
                !fade
                * You're big, strong, and prophetic: you <i>do</i> regret this.
                player:
                    !injured
                    Gah. There. Hesh.
                    ...
                    I think I sprained something.
                mech:
                    !right
                    REBOOTING.
                baron:
                    !right
                    !happy
                    Hey, look! It's back online!
                    Perhaps it just needed a cooldown. 
                player:
                    !sigh
            ]],

            OPT_TAKE_IT = "Convince {baron} to let you take it apart",
            DIALOG_TAKE_IT = [[
                baron:
                    !right                    
                player:
                    Here's an idea. Smash the mech, sell it for parts. 
                    Now you're rich <i>and</i> travelling light.

            ]],
            DIALOG_TAKE_IT_SUCCESS = [[
                player:
                    Time is money, after all. Aren't there ruins you could be spelunking and a cult you could be thumbing your nose at?
                baron:
                    Hrm. I think you're right.
                    Let me just...
                !fade
                * {baron} makes short work of the stationery mech--impressive considering {baron.heshe} does it bare-handed.
                    Yeah I'm over this. Smashing time.
                    !fade
                    !crossed
                    Alright, take your pick of this junk.
                    As a souvenir of the times we shared, I guess.
            ]],
            DIALOG_TAKE_IT_LEAVE = [[
                player:
                    !take
                    This'll do.
                    !salute
                    Later, sparky.
            ]],
            DIALOG_TAKE_IT_FAIL = [[
                player:
                    Ain't it your robot anyway?
                baron:
                    No! It's Spark Baron property, and the taskmaster would have my head.
                    This mech has got to go where it's going in <i>one piece</i>.
            ]],

            DIALOG_LEAVE = [[
                player:
                    Good luck.
                    Machines are creepy.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("baron"))
                cxt:Dialog("DIALOG_INTRO")
            end

            if math.random() < 0.5 and (cxt.scratch.kicked or 0) < 3 then
                cxt:Opt(cxt.scratch.kicked and "OPT_KICK_IT_AGAIN" or "OPT_KICK_IT")
                    :Dialog(cxt.scratch.kicked and "DIALOG_ANOTHER_KICK" or "DIALOG_FIRST_KICK" )
                    :Dialog("DIALOG_KICK_IT")
                    :DeltaHealth(-3)
                    :Fn(function(cxt)
                        cxt.scratch.kicked = (cxt.scratch.kicked or 0) + 1
                    end )
            else
                cxt:Opt(cxt.scratch.kicked and "OPT_KICK_IT_AGAIN" or "OPT_KICK_IT")
                    :Dialog(cxt.scratch.kicked and "DIALOG_ANOTHER_KICK" or "DIALOG_FIRST_KICK" )
                    :DeltaHealth(-3)
                    :Dialog("DIALOG_KICK_IT_TO_LIFE")
                    :Fn(function(cxt)
                        cxt.scratch.kicked = (cxt.scratch.kicked or 0) + 1
                        local mech = cxt.quest:GetCastAgent( "mech" )
                        if cxt.quest.param.aggro then
                            cxt:Dialog("PRE_BATTLE")
                            cxt:DoBattle{
                                enemies = { mech },
                                on_start_battle = function(battle) 
                                    local fighter = battle:GetFighterForAgent(mech)
                                    if fighter then
                                        fighter:AddCondition("MALFUNCTION")
                                    end
                                end,
                                on_win = function( cxt )
                                    cxt.enc.scratch.baron_alive = cxt.quest:GetCastAgent("baron"):IsAlive()
                                    if cxt.quest:GetCastAgent("mech"):IsDead() then
                                        cxt:Dialog("DIALOG_BROKE_IT")
                                        cxt:GetAgent():OpinionEvent( OPINION.BROKE_THEIR_MECH )
                                    else
                                        cxt:Dialog("DIALOG_WOW_OK")
                                        cxt:GetAgent():OpinionEvent( OPINION.HELPED )
                                    end
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end
                            }

                        else
                            cxt.enc.scratch.baron_alive = true
                            cxt:Dialog("DIALOG_WOW_OK")
                            cxt:GetAgent():OpinionEvent( OPINION.HELPED )
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end

                    end )
            end

            cxt:Opt("OPT_HAUL_IT", "status_injury")
                :Dialog("DIALOG_HAUL_IT")
                :ReceiveMoney( CalculateReward( cxt:GetAgent(), 150, 10 ))
                :GainCard( "status_injury" )
                :Travel()
            
            cxt:Opt("OPT_TAKE_IT")
                :Dialog("DIALOG_TAKE_IT")
                :Negotiation{
                }
                    :OnSuccess()
                        :Fn(function( cxt, minigame )
                            cxt:Dialog( "DIALOG_TAKE_IT_SUCCESS")
                            cxt:Wait()

                            local draft_popup = Screen.DraftChoicePopup()
                            local function OnDone()
                                cxt.encounter:ResumeEncounter()
                            end
                            local cards = BattleCardCollection( function(def) return table.contains( JUNK_PARTS, def.id ) end ):GenerateRandom(3, cxt.player)
                            draft_popup:DraftCards( cxt.player, Battle.Card, cards, OnDone )
                            TheGame:FE():InsertScreen( draft_popup )

                            cxt.enc:YieldEncounter()
                            cxt:Dialog("DIALOG_TAKE_IT_LEAVE")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)
                    :OnFailure()
                        :Fn(function() 
                            cxt:Dialog( "DIALOG_TAKE_IT_FAIL")
                        end)

            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :Travel()
        end)