local SMALL_TOLL_AMOUNT = 25
local TOLL_AMOUNT = 100

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    
    _precondition = function(quest)
        return TheGame:GetGameState():GetCaravan():GetMoney() >= SMALL_TOLL_AMOUNT
    end,
}

:AddCast{
    cast_id = "priest",
    condition = function ( agent, quest )
        return agent:GetContentID() == "PRIEST"
    end
}:AddCastFallback("PRIEST")


QDEF:AddConvo()
    :Confront(function(cxt, who)
        if cxt.player:IsLiked( cxt.quest:GetCastAgent("priest")) then
            return "STATE_LIKED"
        else
            return "STATE_NEUTRAL"
        end
    end)

    :State("STATE_LIKED")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * A number of Hesh-clad figures congregate before you expectantly.
                agent:
                    !hesh_greeting
                    $happyGreeting
                    Ah, {player}, we find you once again flirting with the deep.
                player:
                    Don't think so. I'm not really the flirtin' type.
                agent:
                    And yet here you are, brought prostrate before Hesh's will once again. 
                    Would you perhaps see fit to share your favor with a small pledge? 
                    Surely Hesh would continue to honor you in these strange times if you yield, just a bit, to Hesh's influence.
            ]],

            OPT_PAY = "Give a small pledge",
            DIALOG_PAY = [[
                player:
                    Fine, fine. If it'll get you off my back.
                agent:
                    !happy
                    That is for Hesh to decide. But for now, we leave you to the sun-warmed shallows.                   
            ]],

            OPT_PAY_LARGE = "Give a large pledge",
            DIALOG_PAY_LARGE = [[
                player:
                    Sure, why not? As long as you remember who's the high roller in my family, huh?
                    !give
                agent:
                    Yes, of <i>course</i>.
            ]],

            OPT_REFUSE = "Refuse",
            DIALOG_REFUSE = [[
                player:
                    Maybe if Hesh wants shills so bad, it should crawl outta the ocean and come get 'em.
                    Or do you think that's what it tried to do already? 
                    Didn't turn out too well, if so.
                agent:
                    !disappoint
                    I will ignore your heretical flippancy. 
                    But be warned, the strong currents can pull the sand out from under you.
            ]],

        }

        :SetLooping()
        :Fn(function(cxt)
            cxt.quest:Complete()
            cxt.encounter:SetPrimaryCast(cxt.quest:GetCastAgent("priest"))
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_PAY")
                :Dialog("DIALOG_PAY")
                :DeliverMoney(SMALL_TOLL_AMOUNT, { no_scale = true })
                :Travel()

            cxt:Opt("OPT_PAY_LARGE")
                :Dialog("DIALOG_PAY")
                :ReceiveOpinion( OPINION.DEEPENED_RELATIONSHIP )
                :DeliverMoney(TOLL_AMOUNT, { no_scale = true })
                :Travel()

            cxt:Opt("OPT_REFUSE")
                :Dialog("DIALOG_REFUSE", cxt.quest.param.total_amount_to_ask_for)
                :ReceiveOpinion( OPINION.DID_NOT_HELP )
                :Travel()
        end)

    :State("STATE_NEUTRAL")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                * A number of Hesh-clad figures congregate before you menacingly.
                agent:
                    !hesh_greeting
                    $neutralDirect
                    Hail, citizen. It's come to our attention that you've been, ah, derelict in your services to Hesh.
                    Now that you're back in the Pearl, you must want to remedy that.
                player:
                    Well, hey, I'm kind of a citizen of the world, ya know? 
                    Who's to say I've been derelict when I've been, uh, giving back in my own way?
                agent:
                    !permit   
                * {agent} only smiles acidly in response.
            ]],

            OPT_PAY = "Give a pledge",
            DIALOG_PAY = [[
                player:
                    !disappoint
                    Fine. That's the most I can afford.
                    !point
                    Not like it wasn't gonna find its way to you bottom-feeders anyway.
                agent:
                    Your flattery won't go unrecognized, {player}. Nor will your cooperation.
                    
            ]],

            OPT_NEGOTIATE = "Convince {agent} you've already donated today",
            DIALOG_NEGOTIATE = [[
                player:
                    Oh c'mon, I've already donated like three times today!
            ]],
            DIALOG_NEGOTIATION_SUCCESS = [[
                player:
                    First I paid tithes to get the local paper, then I contributed to the pilgrim fund...
                priest:
                    !thought
                    Hm, I see. You <i>do</i> look sufficiently poor.
                player:
                    !hips
                    Darn right!
            ]],
            DIALOG_NEGOTIATION_FAILURE = [[
                player:
                    Bleed me dry, why don't you?
                priest:
                    !point
                    Judging by the money bag on your hip, it seems like you still have blood for the taking.
            ]],

            OPT_REFUSE = "Refuse",
            DIALOG_REFUSE = [[
                player:
                    Forget it. Ain't I got enough problems without your bottom line to boot?
                    So how about I boot your bottom.
                priest:
                    I beg your pardon?
                player:
                    !fight
                priest:
                    !angry
            ]],
        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.scratch.amount = math.max( SMALL_TOLL_AMOUNT, round_zero( cxt.caravan:GetMoney() / 2, 50 ))
                local priest = cxt.quest:GetCastAgent("priest")
                cxt.scratch.backup = CreateCombatBackup(priest,"HESH_PATROL", cxt.quest:GetRank())
                cxt.encounter:SetPrimaryCast(priest)
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_PAY")
                :Dialog("DIALOG_PAY")
                :DeliverMoney(cxt.scratch.amount)
                :Travel()

            cxt:Opt("OPT_NEGOTIATE")
                :Dialog("DIALOG_NEGOTIATE")
                :Negotiation{
                    acting_against = { table.unpack(cxt.scratch.backup) }
                }
                    :OnSuccess()
                        :Dialog("DIALOG_NEGOTIATION_SUCCESS")
                        :Travel()

                    :OnFailure()
                        :Dialog("DIALOG_NEGOTIATION_FAILURE")

            cxt:Opt("OPT_REFUSE")
                :Dialog("DIALOG_REFUSE")
                :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
                :Travel()
        end )


