local battle_defs = require "battle/battle_defs"
local HEAL_PER_CHARGE = 4
local CHARGES = 3

local MONEY_PER_CHARGE = 60

--TODO: Make this more intersting
Content.AddBattleCard("half_sandwich", 
{
    name = "Partial Sandwich",
    anim = "eat",
    desc = "{HEAL} {1} health.",
    flavour = "'It's not half bad!'",
    desc_fn = function(self, fmt_str)
        return loc.format(fmt_str, self.healing_amt )
    end,

    healing_amt = HEAL_PER_CHARGE,
    max_charges = CHARGES,
    cost = 1,
    item_tags = ITEM_TAGS.COMBAT,
    rarity = CARD_RARITY.RARE,
    flags = battle_defs.CARD_FLAGS.SKILL | battle_defs.CARD_FLAGS.ITEM,
    target_type = TARGET_TYPE.SELF,
    series = "GENERAL",

    OnPostResolve = function( self, battle, attack )
        self.owner:HealHealth(self.healing_amt)
    end
})



local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT, 
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,
    
    on_destroy = function(quest)
        if quest:GetCastMember("sandwich_person") and quest:GetCastMember("sandwich_person"):IsInPlayerParty() then
            quest:GetCastMember("sandwich_person"):Dismiss()
        end
    end

}

:AddCast{
    cast_id = "sandwich_person",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "LABORER" ) )
    end,
    events = {
        agent_retired = function( quest, agent ) 
            if quest:IsActive("recover_money_with_friend") then
                quest:Cancel("recover_money_with_friend")
                quest:Activate("get_that_sandwich")
            end
        end,
    },

}

:AddLocationCast{
    cast_id = "sandwich_spot",
    when = QWHEN.MANUAL,
}

:AddObjective{
    id = "find_sandwich",
    state = QSTATUS.ACTIVE,
}


:AddDormancyState("dormant", "meet_sandwich_person", false, 3, 5)

:AddObjective{
    id = "meet_sandwich_person",
    hide_in_overlay = true,
}

:AddObjective{
    id = "get_that_sandwich",
    title = "Go get that sandwich!",
    desc = "There's a half-eaten sandwich filled with money just waiting for the right grifter to come along and take it!",
    mark = {"sandwich_spot"},
    is_in_hiding = function(quest, agent)
        return agent == quest:GetCastMember("sandwich_person")
    end,

    events = 
    {
        phase_change = function(quest, location)
            quest:Cancel()
        end,
    }
}


:AddObjective{
    id = "recover_money_with_friend",
    title = "Take {sandwich_person} to the sandwich",
    desc = "You've agreed to take {sandwich_person} to recover {sandwich_person.hisher} lost paycheque.",
    mark = {"sandwich_spot"},
    on_activate = function(quest) 
        quest.param.death_countdown = 6
    end,

    events = 
    {

        action_clock_advance = function(quest, location)
            if location ~= quest:GetCastMember("sandwich_spot") and quest.param.death_countdown then
                quest.param.death_countdown = quest.param.death_countdown - 1
            end
        end,
    }

}


:AddDormancyState("wait_for_poop", "get_poop_money", false, 4, 7)
:AddObjective{
    id = "get_poop_money",
    hide_in_overlay = true,
}

:AddLocationDefs{
    SANDWICH_SPOT = {
        name = "Sandwich Spot",
        plax = "PLACEHOLDER",
        indoors = false,
    },
}

:AddOpinionEvents{

    returned_whole_money =
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Returned their lost paycheque",
    },
    
    returned_whole_sandwich = 
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Returned their lost lunch",
    },
    
    returned_partial_money =
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Returned most of their lost paycheque",
    },
    
    returned_partial_sandwich =
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Returned part of their lost lunch",
    },
    
    gave_directions_to_money_sandwich =
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Gave directions to their lost paycheque",
    },
    
    gave_directions_to_sandwich =
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Gave directions to their lost lunch",
    },
    
    wasted_time =
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Wasted their time",
    },

    lied_about_sandwich =
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Tried to cheat them out of their lost lunch",
    },
    
}


--------------------------------------------------------------------

QDEF:AddConvo("find_sandwich")
    :Confront(function(cxt, who)
        return "STATE_FIND_STUFF"
    end)

    :State("STATE_FIND_STUFF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You happen upon a small satchel left in the middle of the road.
                player:
                    !thought
                * Having more curiosity than sense, you open it without thinking.
                
                * There's a half-eaten sandwich inside. 
                player:
                    !happy
                    Hey! Lunch!

            ]],

            OPT_SMELL_IT = "Smell the sandwich",
            DIALOG_SMELL_IT = [[
                * You cautiously smell the sandwich.
                player:
                    Mmmmmmm.
                * You can't identify the meat, but it smells delicious.
            ]],

            OPT_TASTE_IT = "Take a small taste",
            DIALOG_TASTE_IT = [[
                * You take a tentative nibble.
                player:
                    Dang, that's good.
            ]],
            
            OPT_EAT_THE_SANDWICH = "Eat the sandwich",
            DIALOG_EAT_THE_SANDWICH = [[
                player:
                    Waste not, want not!
                    !burp
                * You gobble the sandwich. Despite being a bit stale, it really hits the spot.                    
            ]],

            OPT_TAKE_THE_SANDWICH = "Take the sandwich",
            DIALOG_TAKE_THE_SANDWICH = [[
                !cagey
                * You glance around for witnesses. Finding none, you slip the half-sandwich into a pocket for safekeeping.
                !happy
            ]],
            OPT_LEAVE_THE_SANDWICH = "Leave it for someone else",
            DIALOG_LEAVE_THE_SANDWICH = [[
                * You decide not to consume the sandwich of dubious provenance.
            ]],
        }

        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("find_sandwich")
                cxt.quest:Activate("dormant")
            end

            cxt:Question("OPT_SMELL_IT", "DIALOG_SMELL_IT")
            cxt:Question("OPT_TASTE_IT", "DIALOG_TASTE_IT")

            cxt:Opt("OPT_EAT_THE_SANDWICH")
                :Dialog("DIALOG_EAT_THE_SANDWICH")
                :Fn(function() 
                    cxt.quest.param.ate_sandwich = true 
                end)
                :DeltaHealth(HEAL_PER_CHARGE*CHARGES)
                :Travel()

            cxt:Opt("OPT_TAKE_THE_SANDWICH")
                :Dialog("DIALOG_TAKE_THE_SANDWICH")
                :Fn(function() cxt.quest.param.took_sandwich = true end)
                :GainCard("half_sandwich", function(card) cxt.quest.param.sandwich_card = card end)
                :Travel()

            cxt:Opt("OPT_LEAVE_THE_SANDWICH")
                :Dialog("DIALOG_LEAVE_THE_SANDWICH")
                :Fn(function() 
                    cxt.quest.param.left_it = true
                    cxt.quest.param.old_sandwich_spot = cxt.location
                    local location = cxt.quest:SpawnTempLocation("SANDWICH_SPOT")
                    location.plax = cxt.location:GetPlax()
                    location:SetMapPos(cxt.location:GetMapPos())
                    cxt.quest:AssignCastMember("sandwich_spot", location)
                end)
                :Travel()
        end)


QDEF:AddConvo("meet_sandwich_person", nil)
    :TravelConfront("MEET_SANDWICH_OWNER", function(cxt) return TheGame:GetGameState():CanSpawnTravelEvent() end)
        :Loc{
            DIALOG_INTRO = [[
                * A laborer approaches, stooped over, eyes glued to the ground, searching.
                * {agent.HeShe} is so fixated upon {agent.hisher} search that {agent.heshe} almost bumps into you.
                agent:
                    !right
                    !surprised
                player:
                    !left
                    !happy
                    Hey, friend. Watch where you're going.
                agent:
                    Oh. Sorry, grifter.
                    Say, you haven't seen a sandwich laying on the ground, have you?
            ]],

            OPT_ASK_ABOUT_SANDWICH = "Ask about the sandwich",
            
            DIALOG_ASK_ABOUT_SANDWICH = [[
                player:
                    A sandwich, eh?
                    Can you describe it?
                agent:
                    It's... it's a sandwich.
                    It might have been in a satchel?
                    Do you know anything about it?
            ]],

            OPT_PRESS_FOR_DETAILS = "Press for details",
            DIALOG_PRESS_FOR_DETAILS = [[
                player:
                    Listen, I'd love to help, but...
                    Why are you so concerned about half of a sandwich?
            ]],
            DIALOG_PRESS_FOR_DETAILS_SUCCESS = [[
                agent:
                    Alright, if you must know.
                    There's money hidden in it.
                    Quite a lot of money.
                player:
                    !dubious
                    You were hiding money in a sandwich?
                agent:
                    I had just collected my month's pay, and I was heading home.
                    The roads have been crawling with bandits and fee-collecting Admiralty, so I thought it would be safer in there.
                    Sure enough, I was held up. When they didn't find any money on me, they beat me up and stole my lunch!
                    I had to run, but I thought I saw them take a bite and throw the rest away.
                player:
                    That's some bad luck, there, friend.
                agent:
                    !sigh
                    Story of my life.
            ]],
            DIALOG_PRESS_FOR_DETAILS_FAILURE = [[
                agent:
                    I'm hungry, alright? Now have you seen it or not?
            ]],


            OPT_RETURN_WHOLE_MONEY = "Return money from the sandwich",
            DIALOG_RETURN_WHOLE_MONEY = [[
                * You pull out the sandwich and search between its layers.
                * Sure enough, you find {money_amt#money}.
                player:
                    Here you go!
                    !give
                agent:
                    !take
                    Really?
                    !happy
                player:
                    It's your money. I'm keeping the sandwich, though.
                agent:
                    Thanks, grifter. You're alright, you know that?
                player:
                    !shrug
                    I have my moments. Hide it somewhere better next time.
                agent:
                    !exit
                * {agent} scurries off, head on a swivel in case {agent.heshe} gets robbed again.
            ]],
            
            OPT_RETURN_WHOLE_SANDWICH = "Return the sandwich",
            DIALOG_RETURN_WHOLE_SANDWICH = [[
                player:
                    Do you mean...
                    !give
                    ...this sandwich?!
                    !happy
                agent:
                    !take
                    Yes!
                    Thank you!
                    Oh thank Hesh.
                    See you around, grifter!
                    !exit
                player:
                    !thought
                    * You're not sure you fully understand what just transpired. 
                    * It's a good, familiar feeling.
            ]],
            
            OPT_RETURN_PARTIAL_MONEY = "Return what's left of the money",
            DIALOG_RETURN_PARTIAL_MONEY = [[
                * You pull out what's left of the sandwich and search between its sticky layers.
                * Sure enough, you find {money_amt#money}.
                player:
                    Here you go!
                    !give
                agent:
                    !take
                    Really? That's... hey, there was more than this.
                player:
                    !shrug
                    I was hungry.
                agent:
                    You... ate it?
                    And you didn't notice?
                player:
                    As I said, I was hungry.
                agent:
                    Oh. Okay.
                    Well, thanks grifter. Something is better than nothing, for sure.
                agent:
                    !exit
                * {agent} scurries off, head on a swivel in case {agent.heshe} gets robbed again.

            ]],
            OPT_RETURN_PARTIAL_SANDWICH = "Return what's left of the sandwich",
            DIALOG_RETURN_PARTIAL_SANDWICH = [[
                * You produce what's left of the sandwich.
                player:
                    Here you go.
                    !give
                    I, um, may have eaten some of that already.
                agent:
                    You <i>what?</>
                    I - I mean, thanks.
                    Bye, now.
                    !exit
                player:
                    !thought
                    * You're not sure you fully understand what just transpired. 
                    * It's a good, familiar feeling.
            ]],

            OPT_LIE_ABOUT_HAVING_IT = "Lie and say you don't have it",
            DIALOG_LIE_ABOUT_HAVING_IT = [[
                player:
                    You think I would pick up some random sandwich off the ground?
                    What do you think I am?
                    If I didn't know any better, I might take that as an insult...
                agent:
                    !placate
                    No, no. Of course not.
                    I'll be going now.
                    !exit
                * {agent} runs away.
            ]],

            DIALOG_GET_PARTIAL_MONEY = [[
                * You look inside what's left of the sandwich, and lo and behold, you find {money_amt#money} wedged between the layers of deliciousness.
                * You're pretty sure you must have eaten the rest.
                player:
                    !thought
                * You make a mental note to pay careful attention the next time you relieve yourself, and continue along your way.
            ]],
            DIALOG_GET_FULL_MONEY = [[
                * You open up the sandwich and find a fat stack of sticky bills.
                player:
                    !happy
                * Money <i>and</> a snack. This is turning out to be a good day!
            ]],

            OPT_PLAY_INNOCENT = "Play innocent",
            DIALOG_PLAY_INNOCENT_MONEY = [[
                player:
                    Sorry, I haven't seen anything like that.
                agent:
                    !sigh
                    Sure, sure. As if you'd tell me anyway.
                    !exit
                * {agent} slogs off, the weight of the world bearing down on {agent.hisher} shoulders.
            ]],
            DIALOG_PLAY_INNOCENT = [[
                player:
                    That sounds delicious, but I didn't see anything like that.
                agent:
                    !sigh
                    Oh, ok. Well, thanks for your help, anyway.
                    !exit
                * {agent} slogs off, the weight of the world bearing down on {agent.hisher} shoulders.
            ]],

            OPT_GIVE_DIRECTIONS = "Give directions",
            DIALOG_GIVE_DIRECTIONS_MONEY = [[
                player:
                    I did see something like that a while ago...
                * You jot down a crude map and pass it over.
                agent:
                    !happy
                    Thank you, grifter! It's so rare to find a generous soul in these parts.
            ]],
            DIALOG_GIVE_DIRECTIONS = [[
                player:
                    I did see something like that a while ago...
                * You jot down a crude map and pass it over.
                agent:
                    !happy
                    Thank you, grifter! It's so rare to find a generous soul in these parts.
                player:
                    Think of me when you eat that sandwich, friend.
                agent:
                    I will!
                    !exit
            ]],
            OPT_PLAY_DUMB = "Play dumb",
            
            DIALOG_PLAY_DUMB = [[
                player:
                    Nope. I haven't seen anything.
                agent:
                    !sigh
                    Oh, ok. Well, thanks for your help, anyway.
                    !exit
                * {agent} slogs off, the weight of the world bearing down on {agent.hisher} shoulders.                
            ]],
            DIALOG_PLAY_DUMB_MONEY = [[
                player:
                    Nope. I haven't seen anything.
                agent:
                    !sigh
                    Riiiiight. And I totally believe you.
                    !exit
                * {agent} slogs off, the weight of the world bearing down on {agent.hisher} shoulders.                
            ]],

            OPT_OFFER_TO_HELP = "Offer to provide an escort",
            DIALOG_OFFER_TO_HELP = [[
                player:
                    Those bandits might still be around. For a little coin I could protect you.
                agent:
                    That's a good idea!
                    Let's go!
                    !exit
            ]],
            OPT_LET_THEM_GO = "Watch {agent} leave",
            DIALOG_LET_THEM_GO = [[
                player:
                    Well, have fun with that.
                agent:
                    !exit
            ]]


        }
        :Fn(function(cxt) 
            cxt.quest:AssignCastMember("sandwich_person")
            cxt.quest:Complete("meet_sandwich_person")
            local know_about_money = false
            local sandwich_card = cxt.player.battler:FindCard(function(card) return card == cxt.quest.param.sandwich_card end)

            local charges, max_charges = 0, CHARGES
            if sandwich_card then
                charges, max_charges = sandwich_card:GetCharges()
            end

            local have_partial = sandwich_card and charges < max_charges
            local have_full = sandwich_card and charges >= max_charges
            cxt.quest.param.eaten_charges = 0
            if have_partial then
                cxt.quest.param.eaten_charges = max_charges - charges
            elseif cxt.quest.param.ate_sandwich then
                cxt.quest.param.eaten_charges = max_charges
            end
            local ate_it = cxt.quest.param.ate_sandwich or (cxt.quest.param.took_sandwich and not sandwich_card)
            cxt.quest.param.money_amt = sandwich_card and charges*MONEY_PER_CHARGE or 0
            

            cxt:TalkTo("sandwich_person")
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoopingFn(function()
                
                cxt:Question("OPT_ASK_ABOUT_SANDWICH", "DIALOG_ASK_ABOUT_SANDWICH")

                
                cxt:Opt("OPT_PRESS_FOR_DETAILS")
                    :Dialog("DIALOG_PRESS_FOR_DETAILS")
                    :Negotiation()
                        :OnSuccess()
                            :Dialog("DIALOG_PRESS_FOR_DETAILS_SUCCESS")
                            :Fn(function() know_about_money = true end)
                        
                        :OnFailure()
                            :Dialog("DIALOG_PRESS_FOR_DETAILS_FAILURE")
                        

                if have_full then
                    if know_about_money then
                        cxt:Opt("OPT_RETURN_WHOLE_MONEY")
                            :Dialog("DIALOG_RETURN_WHOLE_MONEY")
                            :ReceiveOpinion("returned_whole_money")
                            :CompleteQuest()
                            :Travel()
                    else
                        cxt:Opt("OPT_RETURN_WHOLE_SANDWICH")
                            :Dialog("DIALOG_RETURN_WHOLE_SANDWICH")
                            :ReceiveOpinion("returned_whole_sandwich")
                            :Fn(function() 
                                cxt.player.battler:RemoveCard( sandwich_card )
                            end)
                            :CompleteQuest()
                            :Travel()
                    end

                elseif have_partial then
                    if know_about_money then
                        cxt:Opt("OPT_RETURN_PARTIAL_MONEY")
                            :Dialog("DIALOG_RETURN_PARTIAL_MONEY")
                            :ReceiveOpinion("returned_partial_money")
                            :Fn(function() cxt.quest:Activate("wait_for_poop") end)
                            :Travel()
                    else
                        cxt:Opt("OPT_RETURN_PARTIAL_SANDWICH")
                            :Dialog("DIALOG_RETURN_PARTIAL_SANDWICH")
                            :ReceiveOpinion("returned_partial_sandwich")
                            :Fn(function() 
                                cxt.player.battler:RemoveCard( sandwich_card )
                            end)
                            :CompleteQuest()
                            :Travel()
                    end
                end

                if have_full or have_partial then
                    
                    cxt:Opt("OPT_LIE_ABOUT_HAVING_IT")
                        :Dialog("DIALOG_LIE_ABOUT_HAVING_IT")
                        :Fn(function() 
                            if know_about_money then
                                cxt:Dialog(have_partial and "DIALOG_GET_PARTIAL_MONEY" or "DIALOG_GET_FULL_MONEY")
                                cxt.encounter:GainMoney( cxt.quest.param.money_amt  )

                                if have_partial then
                                    cxt.quest:Activate("wait_for_poop")
                                else
                                    cxt.quest:Cancel()
                                end
                            end
                        end)
                        :Travel()
                end


                if ate_it then
                    cxt:Opt("OPT_PLAY_INNOCENT")
                        :Dialog(know_about_money and "DIALOG_PLAY_INNOCENT_MONEY" or "DIALOG_PLAY_INNOCENT")
                        :Fn(function() cxt.quest:Activate("wait_for_poop") end)
                        :Travel()
                end

                if cxt.quest.param.left_it then
                    if know_about_money then

                        cxt:Opt("OPT_PLAY_DUMB")
                            :Dialog("DIALOG_PLAY_DUMB_MONEY")
                            :ActivateQuest("get_that_sandwich")
                            :Travel()
                        

                        cxt:Opt("OPT_GIVE_DIRECTIONS")
                            :Dialog("DIALOG_GIVE_DIRECTIONS_MONEY")
                            :ReceiveOpinion("gave_directions_to_money_sandwich")
                            :Fn(function() 

                                cxt:Opt("OPT_OFFER_TO_HELP")
                                    :Dialog("DIALOG_OFFER_TO_HELP")
                                    :ActivateQuest("recover_money_with_friend")
                                    :ReceiveMoney(30)
                                    :Fn(function() cxt.quest:GetCastMember("sandwich_person"):Recruit(PARTY_MEMBER_TYPE.ESCORT) end)
                                    :Travel()

                                cxt:Opt("OPT_LET_THEM_GO")
                                    :Dialog("DIALOG_LET_THEM_GO")
                                    :Travel()
                                    :CancelQuest()
                            end)
                            
                    else
                        cxt:Opt("OPT_PLAY_DUMB")
                            :Dialog("DIALOG_PLAY_DUMB")
                            :Travel()
                            :CancelQuest()

                        cxt:Opt("OPT_GIVE_DIRECTIONS")
                            :Dialog("DIALOG_GIVE_DIRECTIONS")
                            :ReceiveOpinion("gave_directions_to_sandwich")
                            :Travel()
                            :CancelQuest()
                    end
                end
            end)
        end)



QDEF:AddConvo("recover_money_with_friend")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.quest.param.death_countdown <= 0 end)
        :Loc{
            DIALOG_INTRO = [[
                * {agent} has lost their patience with you.
                agent:
                    !right
                player: 
                    !left
                    I swear it was around here somewhere...
                agent:
                    I'll just find it myself. Thanks for wasting my time, grifter.
                    !exit
            ]]
        }
        :Fn(function(cxt) 
            cxt:TalkTo("sandwich_person")
            cxt:Dialog("DIALOG_INTRO")
            cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("wasted_time"))
            cxt.quest:Cancel()
            StateGraphUtil.AddEndOption(cxt)
        end)


QDEF:AddConvo("recover_money_with_friend")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("sandwich_spot") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find the spot where you saw the sandwich. 
                sandwich_person:
                    !right
                player: 
                    !left
                    Here it is.
                sandwich_person:
                    That's my satchel!
                    But...
                * Unfortunately, the satchel is surrounded by crayotes, who were attracted by the smell of the sandwich.
                sandwich_person:
                    What are we going to do?
            ]],

            OPT_LEAVE = "Leave {sandwich_person} to sort it out",
            DIALOG_LEAVE = [[
                player:
                    I brought you here, my job is done.
                    I didn't sign up to fight a bunch of monsters!
                sandwich_person:
                    But...
                player:
                    !exit
                * Another satisfied customer.
            ]],

            OPT_ATTACK_MONSTERS = "Charge the monsters",
            DIALOG_ATTACK = [[
                player:
                    Do you want that sandwich back or not?
                    !fight
                sandwich_person:
                    !exit
                agent:
                    !right
            ]],
            DIALOG_WON_FRIEND_ALIVE = [[
                * You have successfully protected the sandwich.
                * {sandwich_person} finds {sandwich_person.hisher} money.
                sandwich_person:
                    !right
                    Thank you, grifter!
                    You really helped me out.
            ]],
            OPT_ASK_FOR_SANDWICH = "Ask for the sandwich",
            DIALOG_ASK_ABOUT_SANDWICH = [[
                player:
                    !point
                    Are you going to eat that?
                sandwich_person:
                    This? Uh, no. You can have it if you want.
                player:
                    !happy
                    I do!
                    !take
                * You pocket the sandwich, losing only the tiniest sliver of dignity as you do.
                sandwich_person:
                    !exit
            ]],


            DIALOG_WON_FRIEND_DEAD = [[
                player:
                    !happy
                * You have successfully protected the sandwich.
                player:
                    !shrug
                * You have not successfully protected {sandwich_person}, however.

                * You pick the money out of the sandwich, and pocket it.
            ]],
            OPT_TAKE_SANDWICH = "Take the sandwich",
            DIALOG_TAKE_THE_SANDWICH = [[
                * There's no sense in letting a good day-old half-sandwich go to waste, so you take it, too.
            ]],
            
            OPT_EAT_THE_SANDWICH = "Eat the sandwich",
            DIALOG_EAT_THE_SANDWICH = [[
                * There's no sense in letting a good day-old half-sandwich go to waste, so you eat it.
            ]],

        }
        :Fn(function(cxt) 
            cxt.quest:Complete("recover_money_with_friend")
            cxt:TalkTo("sandwich_person")
            cxt:Dialog("DIALOG_INTRO")
            local opfor =  CreateCombatParty("MONSTERS", TheGame:GetGameState():GetCurrentBaseDifficulty(), cxt.location)
            cxt:TalkTo(opfor[1])


            cxt:Opt("OPT_ATTACK_MONSTERS")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    allies = {cxt.quest:GetCastMember("sandwich_person")}
                }
                    :OnWin()
                        :Fn(function(cxt) 
                            cxt.quest:Complete()
                            if cxt.quest:GetCastMember("sandwich_person"):IsDead() then
                                
                                cxt:Dialog("DIALOG_WON_FRIEND_DEAD")
                                cxt:Opt("OPT_TAKE_SANDWICH")
                                    :Dialog("DIALOG_TAKE_THE_SANDWICH")
                                    :GainCard("half_sandwich")
                                    :Travel()

                                cxt:Opt("OPT_EAT_THE_SANDWICH")
                                    :Dialog("DIALOG_EAT_THE_SANDWICH")
                                    :DeltaHealth(HEAL_PER_CHARGE*CHARGES)
                                    :Travel()

                                StateGraphUtil.AddLeaveLocation(cxt) 

                            else
                                cxt:Dialog("DIALOG_WON_FRIEND_ALIVE")

                                cxt:Opt("OPT_ASK_FOR_SANDWICH")
                                    :Dialog("DIALOG_ASK_ABOUT_SANDWICH")
                                    :GainCard("half_sandwich")
                                    :Travel()
                                
                                StateGraphUtil.AddLeaveLocation(cxt) 
                            end
                        end)
            
            cxt:Opt("OPT_LEAVE")
                :CancelQuest()
                :Travel()

        end)


QDEF:AddConvo("get_that_sandwich")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("sandwich_spot") end)
        :Loc{
            DIALOG_INTRO_DEAD = [[
                * You trace back your steps to the sandwich's last known location.
                player:
                    !left
                * You find it, and recover the money.
            ]],
            DIALOG_INTRO = [[
                * You trace back your steps to the sandwich's last known location.
                player:
                    !left
                * You find it!
                * And {agent} finds you.
                agent:
                    !right
                    !angry
                    Ha! Just as I thought!
                player:
                    Fancy meeting you here.
                agent:
                    !hips
                    Cut the crap. I figured you knew what was up, so I followed you.
                    I'm done taking this world's Oshnu dung. I'm taking back my lunch.
            ]],

            OPT_LET_TAKE = "Let {agent} take the sandwich and the money",
            DIALOG_LET_TAKE = [[
                player:
                    !placate
                    Don't let me stop you.
                agent:
                    I... I won't!
                * {agent} grabs the satchel and books it down the road.
            ]],

            OPT_DEMAND_THE_SANDWICH = "Let {agent} have the money, but demand the sandwich",
            DIALOG_DEMAND_THE_SANDWICH = [[
                player:
                    You can take the money.
                    !point
                    But leave the sandwich for me!
                agent:
                    ...
                    !dubious
                    Oooookay.
                    !exit
            ]],

            OPT_BATTLE = "Fight {agent}",
            DIALOG_BATTLE = [[
                player:
                    Nope. 
                    !fight
                    Finders, keepers!
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} lies dead at your feet. 
                    * You collect the money from the sandwich.
                }

                {not dead?
                    agent:
                        !injured
                        You... bastard.
                        !exit
                    * You collect the money from the sandwich.
                }

            ]],

            OPT_TAKE_SANDWICH = "Take the sandwich",
            DIALOG_TAKE_THE_SANDWICH = [[
                * There's no sense in letting a good day-old half-sandwich go to waste, so you take it, too.
            ]],
            
            OPT_EAT_THE_SANDWICH = "Eat the sandwich",
            DIALOG_EAT_THE_SANDWICH = [[
                * There's no sense in letting a good day-old half-sandwich go to waste, so you eat it.
            ]],
        }
        :Fn(function(cxt) 
            cxt.quest:Complete()
            if cxt.quest:GetCastMember("sandwich_person"):IsDead() then
                
                
                cxt:Dialog("DIALOG_INTRO_DEAD")
                cxt.encounter:GainMoney( MONEY_PER_CHARGE * CHARGES )

                cxt:Opt("OPT_TAKE_SANDWICH")
                    :Dialog("DIALOG_TAKE_THE_SANDWICH")
                    :GainCard("half_sandwich")
                    :Travel()

                cxt:Opt("OPT_EAT_THE_SANDWICH")
                    :Dialog("DIALOG_EAT_THE_SANDWICH")
                    :DeltaHealth(HEAL_PER_CHARGE*CHARGES)
                    :Travel()

                StateGraphUtil.AddLeaveLocation(cxt) 
            else

                cxt:TalkTo("sandwich_person")
                cxt:Dialog("DIALOG_INTRO")

                cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("lied_about_sandwich"))

                cxt:Opt("OPT_LET_TAKE")
                    :Dialog("DIALOG_LET_TAKE")
                    :Travel()

                cxt:Opt("OPT_DEMAND_THE_SANDWICH")
                    :Dialog("DIALOG_DEMAND_THE_SANDWICH")
                    :GainCard("half_sandwich")
                    :Travel()

                cxt:Opt("OPT_BATTLE")
                    :Dialog("DIALOG_BATTLE")
                    :Battle{}
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :Fn(function()
                                cxt.encounter:GainMoney( MONEY_PER_CHARGE * CHARGES )
                                cxt:Opt("OPT_TAKE_SANDWICH")
                                    :Dialog("DIALOG_TAKE_THE_SANDWICH")
                                    :GainCard("half_sandwich")
                                    :Travel()

                                cxt:Opt("OPT_EAT_THE_SANDWICH")
                                    :Dialog("DIALOG_EAT_THE_SANDWICH")
                                    :DeltaHealth(HEAL_PER_CHARGE*CHARGES)
                                    :Travel()

                                StateGraphUtil.AddLeaveLocation(cxt) 
                            end)
            end
        end)


QDEF:AddConvo("get_poop_money", nil)
    :TravelConfront("POOP_MONEY", function(cxt) return TheGame:GetGameState():CanSpawnTravelEvent() end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You feel a rumbling in your gut.
                * Nature is giving you a call!
                player:
                    !exit
                * You find a discrete spot to take care of business.
                * As you are washing up, you find some money that has made the incredible journey through your digestive system.
                player:
                    !left
                    !happy
                * You continue.
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.encounter:GainMoney( MONEY_PER_CHARGE * cxt.quest.param.eaten_charges )
            cxt.quest:Complete()
            StateGraphUtil.AddLeaveLocation(cxt) 
        end)
