local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
}

QDEF:AddCast{
    cast_id = "admiralty",
    condition = function( agent, quest )
        return agent:GetContentID() == "ADMIRALTY_PATROL_LEADER"
    end,
}:AddCastFallback( "ADMIRALTY_PATROL_LEADER" )

QDEF:AddCast{
    cast_id = "bandit",
    condition = function( agent, quest )
        return agent:GetContentID() == "BANDIT_RAIDER"
    end,
}:AddCastFallback( "BANDIT_RAIDER" )

QDEF:AddCast{
    cast_id = "prisoner",
    condition = function( agent, quest )
        return agent:GetFactionID() == "BANDITS"
    end,
}:AddCastFallback( "BANDIT_GOON" )


QDEF:AddObjective{
    id = "intro",
    state = QSTATUS.ACTIVE
}

local function CreateCombatParties( cxt )
    if cxt.scratch.backup == nil then
        local admiralty = CreateCombatBackup( cxt.quest:GetCastAgent("admiralty"), "ADMIRALTY_PATROL_BACKUP", cxt.quest:GetDifficulty())
        table.insert( admiralty, 1, cxt.quest:GetCastAgent("admiralty"))
        local bandits = CreateCombatBackup( cxt.quest:GetCastAgent("bandit"), "BANDIT_PATROL_BACKUP", cxt.quest:GetDifficulty())
        table.insert( bandits, 1, cxt.quest:GetCastAgent("bandit"))
        cxt.scratch.backup = { admiralty, bandits }
    end
    return table.unpack(cxt.scratch.backup)
end

local convo = QDEF:AddConvo("intro")
    :Loc{
        DIALOG_BANDITS_WIN = [[
            bandit:
                !right
                !injured
                Those switches deserved it.
                Thanks for your help.
                !exit
        ]],
        DIALOG_ADMIRALTY_WIN = [[
            { not dead?
                admiralty:
                    !right
                    Rare to find an upstanding citizen like yourself.
                    The Admiralty thank you.
                    !exit
            }
            player:
                !thought
                Huh. Is this what it's like to be a law-abiding citizen?
                Strange feeling.
        ]],    
    }
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit")  end)
        :Loc{           
            DIALOG_INTRO = [[
                * Down the road you notice a group of Admiralty hauling a prisoner, likely en route to the Pearl Prison.
                * Some distance behind, a poorly concealed group of Spree are stalking them.
            ]],

            OPT_APPROACH_BANDITS = "Approach the bandits",
            DIALOG_APPROACH_BANDITS = [[
                player:
                    !greeting
                    Well this doesn't look suspicious. Whatcha doin?
                bandit:
                    !angry
                    Ssssh!
                    We're bustin' {prisoner} free!
                    Those switches won't even know what hit 'em.
            ]],

            OPT_APPROACH_ADMIRALTY = "Approach the admiralty",

            DIALOG_LEAVE = [[
                * Best to leave that powderkeg to blow of its own accord.
            ]],
        }
        :Fn(function(cxt)
            cxt.quest:Complete()
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt( "OPT_APPROACH_BANDITS" )
                :Fn(function(cxt)
                    cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("bandit"))
                    cxt.enc:PresentDefault()
                end )
                :Dialog("DIALOG_APPROACH_BANDITS" )
                :GoTo( "BANDITS" )

            cxt:Opt( "OPT_APPROACH_ADMIRALTY" )
                :Fn(function(cxt)
                    cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("admiralty"))
                    cxt.enc:PresentDefault()
                end )
                :GoTo( "ADMIRALTY" )

            cxt:Opt( "OPT_LEAVE" )
                :Dialog("DIALOG_LEAVE")
                :Travel()

        end)

    :State("BANDITS")
        :Loc{
            OPT_HELP = "Join the ambush",
            DIALOG_HELP = [[
                player:
                    Good times! I'm here to help.
                bandit:
                    !dubious
                    You know {prisoner}?
                player:
                    What I know is that setting {prisoner.himher} loose from a bunch of goony switches is more fun than not.
                bandit:
                    Good enough for me. Let's get 'em!
                    !fight
                    Hey {prisoner}! We're comin to save ya!
            ]],
            DIALOG_HELP_WIN = [[
                prisoner:
                    !right
                    Hey thanks for that, I owe you one.
                {dead?
                    player:
                        Uh, sorry about your friends.
                    prisoner:
                        !shrug
                        They only wanted to bust me out so I could pay them back.
                    player:
                        You owed them money?
                    prisoner:
                        Oh yeah. And two oshnus. You really saved my bacon.
                        Heck, you deserve a cut of this giant debt I no longer have to pay!
                        !give
                }
            ]],

            OPT_NEGOTIATE = "Offer to negotiate with the guards",
            DIALOG_NEGOTIATE = [[
                player:
                    Listen, you just sit tight. I got this.
                bandit:
                    !exit
                * You saunter up to the group of switches in the distance, flashing a thumbs-up back to the waiting Spree.
                admiralty:
                    !right
            ]],

            OPT_TURNIN = "Break the bandits' cover",
            DIALOG_TURNIN = [[
                player:
                    !thought
                    Not sure how you're going to do that with a crazy stranger yelling loudly and breaking your cover.
                    !wave
                    <b>Guards, guards! Over here!</b> These miscreants are planning to ambush you!
                bandit:
                    !scared
                    Gah!
                admiralty:
                    !right
                    !angry
                    Oh good. And here I was worried we wouldn't get to bust heads today.
            ]],
        }
        :Fn(function(cxt)
            local admiralty, bandits = CreateCombatParties(cxt)

            cxt:Opt( "OPT_HELP")
                :Dialog("DIALOG_HELP")
                :Battle{
                    enemies = admiralty,
                    allies = bandits,
                }
                :ReceiveOpinion( OPINION.SAVED_FROM_CAPTIVITY, nil, cxt.quest:GetCastAgent("prisoner") )
                :OnWin()
                    :Dialog("DIALOG_HELP_WIN")
                    :Fn(function(cxt)
                        if cxt:GetAgent():IsDead() then
                            cxt.caravan:AddMoney( 50 )
                        end
                    end )
                    :Travel()

            cxt:Opt( "OPT_NEGOTIATE")
                :Dialog("DIALOG_NEGOTIATE")
                :GoTo("ADMIRALTY")

            cxt:Opt( "OPT_TURNIN" )
                :Dialog("DIALOG_TURNIN")
                :Fn(function(cxt)
                    -- This is just to make the DIALOG_ADMIRALTY_WIN dialog afterwards work.
                    cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("admiralty"))
                end)
                :Battle{
                    enemies = bandits,
                    allies = admiralty,
                }
                :ReceiveOpinion( OPINION.APPROVE, nil, cxt.quest:GetCastAgent("admiralty") )
                :OnWin()
                    :Dialog("DIALOG_ADMIRALTY_WIN")                        
                    :Travel()

        end )

    :State("ADMIRALTY")
        :Loc{
            DIALOG_APPROACH_ADMIRALTY = [[
                player:
                    !greeting
                    Greetings, fine officers of the law.
                    Just wanted to let you know how I appreciate your fine work.
                admiralty:
                    !angry
                    Buzz off. Can't you see we got our hands full?
                * {admiralty} gestures to the dour-looking Spree captive.
            ]],

            OPT_TURNIN = "Blow the bandits' cover",
            DIALOG_TURNIN = [[
                player:
                    !thumb
                    Uh, so there's some Spree hiding around the corner.
                    You might wanna go take care of that before your captive here gets any bright ideas.
                admiralty:
                    Hrmph, those jellyfish think they can get the jump on us?
                    I'll teach 'em...
                    !exit
            ]],

            OPT_JOIN_BATTLE = "Join the Admiralty in the brawl",

            OPT_LETBE = "Watch the fight from the sidelines",
            DIALOG_LETBE = [[
                * You let the two groups smash it up while standing on the sidelines.
                * Eventually, the victor walks over to you none worse for wear.
            ]],

            OPT_NAMEDROP = "Mention an Admiralty friend of yours...",
            OPT_MENTION_NAME = "{1#agent}",
            DIALOG_NAMEDROP = [[
                player:
                    !thumb
                    Hey, {1.name}'s orders. I gotta take this prisoner back to uh, base.
                admiralty:
                    Good riddance. I'm tired of dragging this meatsack around.
                    !exit
                prisoner:
                    !right
                    !dubious
                    What's going on?
                player:
                    Just a little skirtin' the law for my own benefit.
                    You know how it goes.
                    !thumb
                    Your friends are back there.
                prisoner:
                    !happy
                    It's my lucky day!
            ]],

            OPT_NEGOTIATE = "Negotiate for release",
            DIALOG_NEGOTIATE = [[
                player:
                    This prisoner isn't worth your time.
            ]],
            DIALOG_NEGOTIATE_WIN = [[
                admiralty:
                    Yeah, you're right. Screw this, let's head to the Dirty Pearl.
                    !exit
                prisoner:
                    !right
                    Guess I owe you one.
            ]],
            DIALOG_NEGOTIATE_FAIL = [[
                admiralty:
                    Scram, before I end up with two prisoners instead of one.
            ]],

            REQ_NO_FRIENDS = "You don't have any Admiralty friends.",
            TT_WILL_LOVE = "{1} will love you."
        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                local admiralty = cxt.quest:GetCastAgent("admiralty")
                cxt.enc.scratch.names_to_drop = cxt.player.social_connections:GetConnections(
                        function(agent, rel) return rel > RELATIONSHIP.NEUTRAL and agent:GetFactionID() == "ADMIRALTY" and agent ~= admiralty end)
                cxt.enc:SetPrimaryCast(admiralty)
                cxt:Dialog("DIALOG_APPROACH_ADMIRALTY")
            end

            cxt:Opt("OPT_TURNIN")
                :Dialog("DIALOG_TURNIN")
                :Fn(function(cxt)
                    local admiralty, bandits = CreateCombatParties(cxt)
                    cxt:Opt("OPT_JOIN_BATTLE")
                        :Battle{
                            allies = admiralty,
                            enemies = bandits
                        }
                        :ReceiveOpinion( OPINION.APPROVE, nil, cxt.quest:GetCastAgent("admiralty") )
                        :OnWin()
                            :Dialog("DIALOG_ADMIRALTY_WIN")
                            :Travel()

                    cxt:Opt("OPT_LETBE" )
                        :Dialog("DIALOG_LETBE")
                        :Fn(function(cxt)
                            if math.random() < 0.5 then
                                cxt:Dialog("DIALOG_ADMIRALTY_WIN")
                            else
                                cxt:Dialog("DIALOG_BANDITS_WIN")
                            end
                        end )
                        :Travel()
                end )

            local friend = 


            cxt:Opt("OPT_NAMEDROP")
                :ReqCondition(#cxt.enc.scratch.names_to_drop > 0, "REQ_NO_FRIENDS")
                :LoopingFn(function() 
                    for _, agent in ipairs(cxt.enc.scratch.names_to_drop) do
                        cxt:Opt("OPT_MENTION_NAME", agent)
                            :Dialog("DIALOG_NAMEDROP", agent:LocTable())
                            :ReceiveOpinion( OPINION.SAVED_FROM_CAPTIVITY, nil, cxt.quest:GetCastAgent("prisoner"))
                            :Travel()

                    end
                    StateGraphUtil.AddBackButton(cxt)
                end)

            cxt:Opt("OPT_NEGOTIATE")
                :Dialog("DIALOG_NEGOTIATE")
                :PostText("TT_WILL_LOVE", cxt.quest:GetCastAgent("prisoner"))
                :Negotiation{
                    target_agent = cxt.quest:GetCastAgent("admiralty"),
                    on_success = function(cxt)
                        cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("prisoner"))
                        cxt:Dialog("DIALOG_NEGOTIATE_WIN")
                        cxt.quest:GetCastAgent("prisoner"):OpinionEvent( OPINION.SAVED_FROM_CAPTIVITY)
                        StateGraphUtil.AddLeaveLocation(cxt)

                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATE_FAIL")
                    end,
                }
        end )
        