local SNAIL_DEF = Content.GetCharacterDef("RACING_SNAIL")
local DEF =
    CharacterDef("LONELY_SNAIL",
    {
        base_def = "RACING_SNAIL",
        fight_data = table.extend( SNAIL_DEF.fight_data )
        {
            MAX_HEALTH = 30,
            damage_mult = 0.6,
        },
        aspects = { social_connections = true },
    })
Content.AddCharacterDef( DEF )
DEF:InheritBaseDef()

--------------------------------------------------------------

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.LONG,
    precondition = function( quest )
        return TheGame:GetGameState():GetCaravan():CanOwnPet()
    end,
    on_init = function(quest)
        quest.param.countdown = math.random( 3, 5 )
    end,
    events = 
    {
        action_clock_advance = function(quest, location)
            quest.param.countdown = (quest.param.countdown or 0) - 1
        end,
    }
}

:AddCast{
    cast_id = "oshnu",
    no_validation = true,
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "LONELY_SNAIL" ) )
    end,
    events =
    {
        agent_retired = function( quest, agent )
            quest:Cancel()
        end,
        dismissed = function( quest, agent )
            quest:Cancel()
        end,
    }
}

:AddCast{
    cast_id = "straggler",
    condition = function(agent, quest)
        return agent.combat_backup ~= nil and agent:GetFactionID() ~= "RISE"
    end,
}


:AddExistingLocation( "oshnudrome", "PEARL_OSHNUDROME" )

:AddObjective{
    id = "intro",
    state = QSTATUS.ACTIVE,
    on_complete = function(quest)
        quest:Activate("wrangling")
    end
}

:AddObjective{
    id = "wrangling",
}

:AddObjective{
    id = "head_to_the_drome",
    title = "Escort {oshnu}",
    desc = "{oshnu} seems to want you to take {oshnu.himher} to the Oshnudrome.",
    mark = { "oshnudrome" },
}


QDEF:AddConvo("intro")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                oshnu:
                    !right
                    !sad
                player:
                    !greeting
                    Well, hello there little fella.
                    Where's your wrangler?
                    !dubious
                    This is some kind of trap, right?
                oshnu:
                    Ssshmmu?
            ]],
            
            OPT_WAIT_IN_HIDING = "Wait in hiding to see if somebody comes by",
            DIALOG_WAIT_IN_HIDING = [[
                player:
                    You stay right there fella.
                    You might be cute but ol' {player} isn't tricked so easily!
                    !exit
                * You wait around a good hour, before a crowd comes straggling by.
            ]],

            OPT_TAKE_REINS = "Take the oshnu's reins",
            DIALOG_TAKE_REINS = [[
                player:
                    !thought
                    I've always wanted a racing oshnu.
                    I think I'll call you {oshnu}.
                    !hips
                    Come with me my friend!
                ]],
            DIALOG_JOINED = [[
                player:
                    Ever had Snellicks before?
                    !exit
                oshnu:
                    Sssshmmuu?
                    !exit
            ]],
        }
        :Fn(function(cxt)
            cxt.enc:SetPrimaryCast( cxt.quest:GetCastMember( "oshnu" ))
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt( "OPT_WAIT_IN_HIDING" )
                :Dialog("DIALOG_WAIT_IN_HIDING")
                :GoTo( "STATE_STRAGGLERS" )

            cxt:Opt( "OPT_TAKE_REINS" )
                :Fn(function(cxt)
                    cxt:GetAgent():GenerateRandomName()
                end)
                :Dialog( "DIALOG_TAKE_REINS" )
                :RecruitMember( PARTY_MEMBER_TYPE.CREW )
                :Dialog( "DIALOG_JOINED" )
                :CompleteQuest( "intro" )
                :Travel()

                StateGraphUtil.AddLeaveLocation( cxt )
                    :Fn(function() 
                        cxt.quest:Complete()
                    end)
        end )

    :State("STATE_STRAGGLERS")
        :Loc{
            DIALOG_STRAGGLERS = [[
                straggler:
                    !left
                    Well what have we here!
                    !dubious
                    Hey, is this some kind of trap or something?
            ]],

            OPT_AMBUSH = "Ambush them",
            DIALOG_AMBUSH = [[
                player:
                    !left
                    !point
                    You bet it's a trap!
                    Walked right into it, ya goof!
                straggler:
                    !right
                    Who's the goof? You're just one kra'deshi!
                    !fight
                    Get him!
            ]],
            DIALOG_AMBUSH_WIN = [[
                player:
                    !left
                    !laugh
                    I guess it was a trap after all.
                    C'mon, {oshnu}. You're my new drinkin buddy.
                oshnu:
                    !disappoint
                    Shmmu...
            ]],
            OPT_LEAVE = "Leave the incident behind",
            DIALOG_LEAVE = [[
                left:
                    !happy
                    Free oshnu! Must be my lucky day!
                    !exit
                right:
                    !exit
                * You sneak off, leaving the oshnu to its new owners.
            ]]
        }
        :Fn(function(cxt)
            local leader = cxt.quest:GetCastMember("straggler")
            leader:MoveToLocation(cxt.location)
            cxt.scratch.opfor = CreateCombatBackup( leader, nil, cxt.quest:GetDifficulty() + 1)
            table.insert( cxt.scratch.opfor, 1, leader )

            local oshnu = cxt.quest:GetCastMember("oshnu")

            cxt:Dialog( "DIALOG_STRAGGLERS" )

            cxt:Opt( "OPT_AMBUSH" )
                :ReceiveOpinion( OPINION.DEFENDED, { only_show = true }, oshnu )
                :Battle{
                    enemies = cxt.scratch.opfor,
                    on_win = function( cxt )
                        oshnu:GenerateRandomName()
                        cxt:Dialog("DIALOG_AMBUSH_WIN")
                        cxt.quest.param.countdown = (cxt.quest.param.countdown or 0) * 2
                        oshnu:Recruit( PARTY_MEMBER_TYPE.CREW )
                        cxt.quest:Complete("intro")
                        oshnu:OpinionEvent( OPINION.DEFENDED )
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                }


            cxt:Opt( "OPT_LEAVE" )
                :Dialog("DIALOG_LEAVE")
                :CompleteQuest()
                :Travel()
end
        )


QDEF:AddConvo("wrangling")
    :Loc{
        OPT_RELEASE = "Release {oshnu}",
    }
    :Hub_Location( function( cxt, who )
        if cxt.location == cxt.quest:GetCastMember("oshnudrome") then
            cxt:Opt("OPT_RELEASE")
                :Fn( function(cxt)
                    UIHelpers.DoSpecificConvo( cxt.quest:GetCastMember("oshnu"), cxt.convodef.id, "STATE_LEAVE" ,nil,nil,cxt.quest)
                end )
        end
    end )
    :State("STATE_LEAVE")
        :Loc{
            DIALOG_LEAVE = [[
                player:
                    Well, here we are.
                    Go live your dreams {oshnu}!
                    Make me proud.
                    I promise not to bet against you.
                    !thought
                    Unless you're terrible at racing, in which case, I'll probably bet against you.
                oshnu:
                    Shmoo!
                    !exit
                * {oshnu} leaves something for you on the ground.
            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_LEAVE")
            cxt:GetAgent():Dismiss()
            cxt:GetAgent():MoveToLimbo()
            cxt.quest:Complete()
            cxt:GainCards( { table.arraypick{"gluey_antennae", "oshnu_glue"} } )
            cxt:End()
        end)

-- Your oshnu gets depressed after a while.
QDEF:AddConvo("wrangling")
    :Confront( function( cxt, who )
        if who == cxt.quest:GetCastMember("oshnu") and cxt.location:HasTag("in_transit") then
            if (cxt.quest.param.countdown or 0) <= -3 then
                return "STATE_SHMOO_OFF"
            elseif not cxt.quest.param.shmooed and (cxt.quest.param.countdown or 0) <= 0 then
                return "STATE_SHMOO"
            end
        end
    end )
    :State( "STATE_SHMOO" )
        :Loc{
            DIALOG_SAY_THE_WORD = [[
                * You notice {oshnu} acting a little down in the dumps.
                player:
                    !left
                    What's wrong there fella? Dreams of glory at the Oshnudrome?
                * {oshnu} perks up immediately at the sound of the word.
                player:
                    Got ambitions, huh?
            ]],

            OPT_PROMISE = [[Promise to take {oshnu} to the Oshnudrome]],
            DIALOG_PROMISE = [[
                player:
                    Don't worry, we'll hit up the 'drome soon enough.
            ]],

            OPT_DENY = [[Carry on as normal]],
            DIALOG_DENY = [[
                !thought
                Wouldn't know what that feels like.
            ]],
        }
        :Fn(function(cxt)
            cxt.quest.param.shmooed = true

            cxt:Dialog( "DIALOG_SAY_THE_WORD" )
            cxt:Opt( "OPT_PROMISE" )
                :Dialog("DIALOG_PROMISE")
                :ActivateQuest("head_to_the_drome")
                :Travel()

            cxt:Opt( "OPT_DENY" )
                :Dialog("DIALOG_DENY")
                :Travel()


        end )

    :State( "STATE_SHMOO_OFF" )
        :Loc{
            DIALOG_LEAVE = [[
                player:
                    !left
                    Just wait there, {oshnu}.
                    !thumb
                    There's some uh, bushes here need tending.
                    !exit
                right:
                    Shmmoo?
                    ... Shuuu Shmooo...
                    !exit
                player:
                    !left
                    !dubious
                    {oshnu}?
                    ...
                    Where'd you go?
                    ...
                    !sigh
                    But we're friends!
                * It seems this friendship is over.
            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog( "DIALOG_LEAVE" )
            cxt:GetAgent():Dismiss()
            cxt:GetAgent():MoveToLimbo()
            cxt.quest:Complete()
            cxt:End()
        end )