local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    rank = {2, 2},
}

:AddCastByAlias{
    cast_id = "sal",
    alias = "NPC_SAL",
}

:AddOpinionEvents{

    convinced_and_hired = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Joined forces with them against Vixmalli",
    },
    hired = 
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Joined forces with them against Vixmalli",
    },
}

QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit") and TheGame:GetGameState():GetDayPhase() == DAY_PHASE.NIGHT end)
    :Loc{
        DIALOG_INTRO = [[
            player:
                !left
            * Out of the shadows, a grifter emerges.
            agent:
                !right
                You are Smith Banquod, correct?
            player:
                You ask that like you already know the answer.
            agent:
                Yes, but I need to be sure.
            player:
                !dubious
                Yeah, I'm Smith. 
                And I'm a Banquod, too, despite the best efforts of my kin.
                !happy
                Now, are you going to tell me I won a prize or something?
            agent:
                !fight
                It's nothing personal, Mr. Banquod, but my employer wants you dead.
            player:
                !neutral
        ]],

        OPT_ASK_ABOUT_BOSS = "Ask who sent {sal}",
        DIALOG_ASK_ABOUT_BOSS = [[
            player:
                Who is this employer, exactly?
            agent:
                Sorry, but they paid extra for discretion.
        ]],

        OPT_GRIFTER_IN_PEARL = "Ask why {sal} is in the Pearl",
        DIALOG_GRIFTER_IN_PEARL = [[
            player:
                I don't know you, grifter, and I know <b>all</> of the dangerous types from the Pearl.
                So either you ain't from here, or you ain't dangerous.
                Which is it?
            agent:
                ...
                Why are we still talking?
                I just told you that I'm here to kill you, and you take this as an opportunity to have a chat?
            player:
                !shrug
                I'm a people person, what can I say?
        ]],

        OPT_DEFEND = "Defend yourself!",
        DIALOG_DEFEND = [[
            player:
                !fight
                Nothing personal to you, either, grifter, but I don't take kindly to assassination attempts.
        ]],
        
        OPT_TURN_SIDES = "Convince {agent} not to attack you",
        DIALOG_TURN_SIDES = [[
            player:
                !point
                You're a grifter.
                !thumb
                I'm a grifter.
                We both know how this works. Somebody pays one of us, and the other one bleeds.
                But maybe...
                Just maybe... 
                !happy
                ...it doesn't have to be that way this time.
        ]],
        DIALOG_TURN_SIDES_SUCCESS = [[
            agent:
                You know what? 
                !neutral
                Screw it.
                I don't want that pompous slug's money.
            player:
                Would I be correct in assuming that the 'pompous slug' you're talking about is one Vixmalli Banquod?
            agent:
                Yeah, you would.
                I've run grifts for some unsavoury characters in my time, but that one takes the cake.
            player: 
                He's not going to be happy with you coming back empty-handed.
        ]],
        DIALOG_TURN_SIDES_FAILURE = [[
            agent:
                No. I took the money, so I'm doing the job.
                !fight
                It's the grifter code.
            player:
                We have a code now?!
        ]],

    }
    :Fn(function(cxt)
        cxt.quest:Complete()

        local possible_pets = {"FLEAD_UPGRADED", "ERCHIN_UPGRADED", "CRAYOTE_UPGRADED", "VROC_UPGRADED", "SHROOGLET_UPGRADED", "LUMICYTE_UPGRADED"}
        cxt.quest.param.pet = TheGame:GetGameState():AddSkinnedAgent(table.arraypick(possible_pets))
        cxt.quest.param.pet:GenerateRandomName()
        cxt:TalkTo("sal")
        cxt:Dialog("DIALOG_INTRO")


        cxt:RunLoop(function() 
            
            cxt:QST("ASK_ABOUT_BOSS")
            cxt:QST("GRIFTER_IN_PEARL")

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    enemies = {"sal", cxt.quest.param.pet}
                }
                    :OnWin()
                        :Fn(function() cxt.quest.param.fought = true end)
                        :GoTo("STATE_POST_FIGHT")

            cxt:BasicNegotiation("TURN_SIDES")
                :OnSuccess()
                    :GoTo("STATE_POST_CONFRONT")
        end)

    end)

    :State("STATE_POST_FIGHT")
        :Loc{
            DIALOG_WON = [[
                {dead?
                    player:
                        !thought
                    * The grifter is dead. You'll never know who sent {agent.himher}.
                    player:
                        !shrug
                    * Luckily for you, you're good at living with uncertainty!
                }
                {not dead?
                    agent:
                        !injured
                    agent:
                        Ugh. You're tougher than you look.
                    player:
                        Yeah, and smarter, too. 
                        I spared your life-- time to spill the beans. 
                        Who sent you?
                    agent:
                        Y-your brother.
                    player:
                        That only slightly narrows it down. Which one?
                    agent:
                        The Heshy one with the money.
                    player:
                        !thought
                        That makes sense.
                        If I know Vix, he's not going to be very happy with you.
                }
            ]],

            DIALOG_PET_RETURN = [[
                * The grifter's pet returns, and nuzzles the body.
                agent:
                    !right
                * It turns to you, and the fight goes out of its beady little eyes.
            ]],
            
            OPT_LEAVE_IT = "Leave {agent}",
            DIALOG_LEAVE_IT = [[
                player:
                    !exit
                * You walk away, leaving the animal to its own fate.
            ]],

            OPT_TAKE_IT = "Take {agent} with you",
            DIALOG_TAKE_IT = [[
                player:
                    !point
                    Don't look at me like that!
                    ...
                    !point
                    Stop it!
                    ...
                    !sigh
                    Fine. You can come.
                player:
                    !exit
                agent:
                    !exit
                * Your new best friend frolics about your heels as you walk away.
            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_WON")
            if cxt:GetAgent():IsAlive() then
                 cxt:GoTo("STATE_POST_CONFRONT")   
            else

                if not cxt.quest.param.pet:IsRetired() and TheGame:GetGameState():GetCaravan():CanOwnPet() then
                    cxt:TalkTo(cxt.quest.param.pet)
                    cxt:Dialog("DIALOG_PET_RETURN")
    
                    cxt:Opt("OPT_LEAVE_IT")
                        :PreIcon(global_images.close)
                        :Dialog("DIALOG_LEAVE_IT")
                        :Travel()
                    
                    cxt:Opt("OPT_TAKE_IT")
                        :PreIcon(global_images.invite)
                        :PostText("TT_PET_TAKE")
                        :Dialog("DIALOG_TAKE_IT")
                        :Fn(function()
                            cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.CREW)
                        end)
                        :Travel()

                else
                    StateGraphUtil.AddLeaveLocation(cxt)                 
                end
            end
        end)

    :State("STATE_POST_CONFRONT")
        :Loc{
            OPT_LET_GO = "Let {agent} go",
            DIALOG_LET_GO = [[
                player:
                    !neutral
                    Get out of here, and don't come after me again!
                * The grifter fixes you in a steely gaze, then responds with a curt nod and a grunt before melting back into the shadows.
                agent:
                    !exit
            ]],

            OPT_HIRE_FOR_FUTURE = "Hire {agent} to lie to Vix",
            DIALOG_HIRE_FOR_FUTURE = [[
                player:
                    !give
                    I have a new job for you.
                    I want you to wait a day, and then go tell Vix that you chased me out of town.
                    You can be convincing, right?
                agent:
                    I can be <b>very</> convincing.
                player:
                    Great. I want him to be surprised when I show up at his doorstep.
                agent:
                    That it? You don't want me to finish him off for you?
                player:
                    No, that's a family matter. I want to handle that job myself.
                agent:
                    !shrug
                    Suit yourself.
                    !exit
            ]],

            OPT_ASK_VIX = "Ask about {agent}'s contract with Vixmalli",
            DIALOG_ASK_VIX = [[
                player:
                    What did Vix hire you for, exactly?
                agent:
                    !neutral
                    He said to make you disappear.
                    Silently. But with "gratuitous violence", if at all possible.
                player:
                    Yeah, that sure sounds like Vix.
                    How much was he paying?
                agent:
                    Enough to get back to Murder Bay.
                    One thousand shills.
                player: 
                    That <i>is</> a lot!
                    !happy
                    I think I'm flattered!
            ]],

            OPT_ASK_PLANS = "Ask {agent} about {agent.hisher} plans",
            DIALOG_ASK_PLANS = [[
                player:
                    !thought
                    So you kill me, collect the money, and then what?
                agent:
                    I have business in Murder Bay.
                    Personal business.
                player:
                    Ooooh, mysterious!
                agent:
                    !shrug
                    You're not the only one with messed up family stuff to work out.
                    Now I have to find another paycheck before I can leave.

            ]],
        }
        :Fn(function(cxt) 

            cxt:RunLoop(function(cxt)

                cxt:QST("ASK_VIX")
                cxt:QST("ASK_PLANS")

                cxt:Opt("OPT_LET_GO")
                    :Dialog("DIALOG_LET_GO")
                    :Travel()

                local opt = cxt:Opt("OPT_HIRE_FOR_FUTURE")
                    :Dialog("DIALOG_HIRE_FOR_FUTURE")
                    :DeliverMoney(200)
                    :ReceiveOpinion(cxt.quest.param.fought and "hired" or "convinced_and_hired" )
                    :Fn(function() 
                        TheGame:GetGameState():GetMainQuest().param.sal_hired_to_lie = true
                    end)
                    :Travel()

            end)
        end)