local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
    on_init = function( quest )
        quest.param.stole_wallet = math.random() < 0.5
    end,
    events = 
    {
        action_clock_advance = function(quest, location)
            if quest.param.patience_timer and location ~= quest:GetCastMember("merch_home") then
                quest.param.patience_timer = quest.param.patience_timer - 1
            end
        end,
    }
}

:AddCast{
    cast_id = "merchant",
    condition = function(agent, quest) 
        return agent:GetContentID() == "WEALTHY_MERCHANT"
    end,
}:AddCastFallback{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "WEALTHY_MERCHANT" ) )
    end,
}

:AddCast{
    cast_id = "bandit",
    condition = function(agent, quest) 
        return agent:GetContentID() == "BANDIT_GOON2"
    end,
}:AddCastFallback{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "BANDIT_GOON2" ) )
    end,
}

:AddOpinionEvents{
    play_along =  
    {
        delta = OPINION_DELTAS.OPINION_UP,
        txt = "Let them get away with a crime",
    },
}

:AddObjective{
    id = "intro",
    state = QSTATUS.ACTIVE,    
}

:AddLocationCast{
    cast_id = "merch_home",
    when = QWHEN.MANUAL,
}

:AddObjective{
    id = "escort_home",
    title = "Escort {merchant} home",
    desc = "{merchant} is worried about bandits, escort {merchant.himher} home.",
    mark = { "merch_home" },
}

QDEF:AddConvo("intro")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("CONF", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                bandit:
                    !left
                merchant:
                    !right
                    I know it was you! There was no one else at the bar!
                bandit:
                    I didn't do anything! You think I need to be stealing from the likes of you?
                merchant:
                    Judging by your fashion choices, yeah. I do.
            ]],
            OPT_ASK_MERCHANT = "Ask {merchant} what's going on",
            DIALOG_ASK_MERCHANT = [[
                merchant:
                    !right
                player:
                    !left
                    Woah, calm down a bit. What's going on?
                merchant:
                    What's going on is that this thug stole my wallet!
                player:
                    Oh. Is that true? Did you steal {merchant.hisher} wallet?
                bandit:
                    !right
                    No! {merchant.HeShe}'s making it up, it's all a con!
            ]],
            OPT_ASK_BANDIT = "Ask {bandit} what's going on",
            DIALOG_ASK_BANDIT = [[
                bandit:
                    !right
                player:
                    !left
                    Hold on, what did you do?
                bandit:
                    Me? Nothing! This fool is trying to frame me.
                merchant:
                    !left
                    Frame you? I've never heard anything so ridiculous!
                    {bandit.HeShe} stole my wallet! Teach {bandit.himher} a lesson for me and I'll make it worth your time.
            ]],
            OPT_BEAT_UP_BANDIT = "Beat up {bandit}",
            TT_BEAT_UP = "{merchant} has promised to pay you",
            DIALOG_BEAT_UP = [[
                bandit:
                    !right
                player:
                    You really shouldn't steal from people.
                bandit:
                    I'm telling you, I didn't!
                player:
                    Save it for someone who belives you!
                    !fight
            ]],
            DIALOG_BEAT_UP_WON = [[
                {bandit_dead?
                    * {bandit} lies dead on the ground.
                    player:
                        !left
                    merchant:
                        !right
                        I told you to rough {bandit.himher} up, not kill {bandit.himher}!
                }
                {not bandit_dead?
                    player:
                        !left
                    bandit:
                        !right
                        !injured
                    {stole_wallet?
                        bandit:
                            Alright, alright! I did it.
                            Here! Just take it.
                            !give
                        player:
                            !take
                            That's more like it.
                        * You hand the wallet back to {merchant} who seems grateful.
                    }
                    {not stole_wallet?
                        bandit:
                            Okay, okay! I'll pay, just please let me live!
                            !give
                        * {bandit} hands you a wad of money, but no wallet.
                        player:
                            !take
                            That's more like it.
                        bandit:
                            !exit
                        * {bandit} stumbles off, clearly wanting to get out of this situation.
                    }
                }
            ]],
            OPT_CONVINCE_MERCHANT = "Convince {merchant} to let {bandit} go",
            DIALOG_CONVINCE = [[
                merchant:
                    !right
                player:
                    !left
                    This seems like a misunderstanding, maybe someone else took your wallet?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !left
                merchant:
                    !right
                    I guess I could be wrong... I still don't trust {bandit}, but you do make some solid points, {player}.
                    Maybe I should go back and double check at the bar.
                    !exit
                bandit:
                    !right
                    Phew! Really saved me there.
                { stole_wallet?
                    bandit:
                        Just as I was thinking I wouldn't get away with it.
                    player:
                        Wait, you did steal it?
                    bandit:
                        Oh. Uh... No?
                }
                { not stole_wallet?
                    bandit:
                        I appreciate you proving my innocence.
                        !exit
                    player:
                        !exit
                }
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    !left
                merchant:
                    !right
                    No no no, I'm certain it was {bandit}. You can't smooth talk your way out of this one.
                bandit:
                    !right
                    You're just making things worse. Nobody asked for your opinion.
                player:
                    No one ever does...
                * Clearly you're not needed here, so you decide to let them figure it out by themselves.
            ]],
            OPT_LEAVE = "Leave",
            DIALOG_LEAVE = [[       
                { asked?
                    * On one hand, {merchant} could be telling the truth. On the other hand, {merchant.heshe} could be trying to frame {bandit}.
                    * That sure is a lot of options. You decide it's probably best to let them figure it out by themselves.
                }
                { not asked?
                    * You seem to have stumbled into something of a mess, probably best to just leave it be.
                }
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt:Dialog("DIALOG_INTRO")
                cxt.enc.scratch.stole_wallet = cxt.quest.param.stole_wallet
            end
            if not cxt.enc.scratch.asked then
                cxt:Opt("OPT_ASK_MERCHANT")
                    :Dialog("DIALOG_ASK_MERCHANT")
                    :Fn(function( cxt )
                        cxt.enc.scratch.asked = true
                    end)
                
                cxt:Opt("OPT_ASK_BANDIT")
                    :Dialog("DIALOG_ASK_BANDIT")
                    :Fn(function( cxt )
                        cxt.enc.scratch.asked = true
                    end) 
            else
                cxt:Opt("OPT_CONVINCE_MERCHANT")
                    :Negotiation{
                        target_agent = cxt.quest:GetCastAgent("merchant"),
                        helpers = {cxt.quest:GetCastAgent("bandit")},
                    }
                        :OnSuccess()
                            :Fn(function( cxt )
                                cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                                if cxt.enc.scratch.stole_wallet then
                                    cxt:GoTo("STATE_BANDIT_LIED")
                                else
                                    cxt.quest:Complete()
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end
                            end)

                        :OnFailure()
                            :Fn(function( cxt )
                                cxt:Dialog("DIALOG_CONVINCE_FAIL")
                                cxt.quest:Complete()
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)

                cxt:Opt("OPT_BEAT_UP_BANDIT")
                    :Battle{
                        enemies = {cxt.quest:GetCastAgent("bandit")},
                    }
                        :OnWin()
                            :Fn(function() 
                                cxt.enc.scratch.bandit_dead = cxt.quest:GetCastAgent("bandit"):IsDead()
                                cxt:Dialog("DIALOG_BEAT_UP_WON")
                                if cxt.quest.param.stole_wallet then
                                    cxt:GoTo("STATE_PROPOSE_ESCORT")
                                else
                                    cxt:GoTo("STATE_MERCHANT_LIED")
                                end
                            end)
            end

            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :CompleteQuest()
                :Travel()
        end)

    :State("STATE_BANDIT_LIED")
        :Loc{
            OPT_EXTORT = "Extort {bandit}",
            DIALOG_EXTORT = [[
                bandit:
                    !right
                player:
                    !left
                    If you're going to make me an accomplice and want me to keep quiet, it's only fair that you share the loot.
                bandit:
                    Urgh, fine. Just promise you'll keep it to yourself.
                * {bandit} hands you {1#money} to keep you quiet.
            ]],
            OPT_PLAY_ALONG = "Play along",
            DIALOG_PLAY_ALONG = [[
                bandit:
                    !right
                player:
                    !left
                    Well... I'm glad I could help {merchant} see the truth.
                    !wink
                bandit:
                    !happy
                    I knew I could count on you {player}.
            ]]
        }
        :Fn(function(cxt)
            cxt:TalkTo(cxt.quest:GetCastAgent("bandit"))
            local extort_amt = 45 + 25*cxt.quest:GetRank()
            cxt:Opt("OPT_EXTORT")
                :Dialog("DIALOG_EXTORT", extort_amt)
                :ReceiveMoney(extort_amt)
                :Fn(function(cxt)
                    cxt.quest:Complete()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_PLAY_ALONG")
                :Dialog("DIALOG_PLAY_ALONG")
                :ReceiveOpinion("play_along")
                :Fn(function(cxt)
                    cxt.quest:Complete()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

        end)

    :State("STATE_MERCHANT_LIED")
        :Loc{
               OPT_ASK_ABOUT_WALLET = "Ask about the wallet",
               DIALOG_ASK = [[
                   merchant:
                       !right
                   player:
                       !left
                       So, if {bandit.heshe} stole your wallet, why did {bandit.heshe} only give you money?
                   merchant:
                       Huh? What do you mean?
                   player:
                       Well don't you want the wallet itself back?
                   * {merchant} clearly wasn't being honest.
               ]],
               OPT_EXTORT_MERCHANT = "Extort {merchant}",
               DIALOG_EXTORT_MERCHANT = [[
                   player:
                       You know. I don't like being tricked. Maybe I should go find an officer and tell them about your little scheme.
                   merchant:
                       Surely that won't be necessary. What if I just gave you a little tip for your help?
                   player:
                       Try a big tip and you've got yourself a deal.
                   merchant:
                       I suppose that's fair.
               ]],
               OPT_LET_IT_GO = "Let it go",
               DIALOG_LET_IT_GO = [[
                   player:
                       I guess the money is the most important part of a wallet.
                   merchant:
                       Exactly! I was thinking of getting a new wallet anyway.
                   player:
                       Of course, it's not like you would lie.
                       !wink
               ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            local extort_amt = 20 + 20 *cxt.quest:GetRank()
            if cxt.enc.scratch.ask_about_wallet then
                cxt:TalkTo(cxt.quest:GetCastAgent("merchant"))
                cxt:Opt("OPT_EXTORT_MERCHANT")
                    :Dialog("DIALOG_EXTORT_MERCHANT")
                    :ReceiveMoney(extort_amt)
                    :Fn(function(cxt)
                        cxt.quest:Complete()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)

                cxt:Opt("OPT_LET_IT_GO")
                    :Dialog("DIALOG_LET_IT_GO")
                    :ReceiveOpinion("play_along")
                    :Fn(function(cxt)
                        cxt.quest:Complete()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            else
                cxt:Opt("OPT_ASK_ABOUT_WALLET")
                    :Dialog("DIALOG_ASK")
                    :Fn(function(cxt)
                        cxt.enc.scratch.ask_about_wallet = true
                    end)
            end
            cxt:Opt("OPT_LEAVE")
                :CompleteQuest()
                :End()
        end)

    :State("STATE_PROPOSE_ESCORT")
        :Loc{
           DIALOG_PROPOSAL = [[
                player:
                    !left
                merchant:
                    !right
                    I appreciate your help, but I am a bit worried that there might be other bandits waiting for me.
                    Would you escort me home? For a fee, of course!
           ]],
           OPT_ACCEPT = "Accept the offer",
           DIALOG_ACCEPT = [[
                   player:
                       Of course! I could always use an extra shill or two.
                   merchant:
                       !happy
                       Perfect! Let's get going then.
           ]],
           OPT_DECLINE = "Decline the offer",
           DIALOG_DECLINE = [[
                   player:
                       Look, I'm glad I could help out but I'm a busy guy! I'm sure you'll be fine though.
                   merchant:
                       !sigh
                       Oh, well alright then.
           ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_PROPOSAL")

            cxt:Opt("OPT_ACCEPT")
                :Dialog("DIALOG_ACCEPT")
                :Fn(function( cxt )
                    cxt.quest:AssignCastMember("merch_home", cxt.quest:GetCastAgent("merchant"):GetHomeLocation())
                    cxt.quest:Activate( "escort_home" )
                    cxt.quest:Complete( "intro" )
                    cxt.quest.param.patience_timer = 3
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_DECLINE")
                :Dialog("DIALOG_DECLINE")
                :Fn(function(cxt)
                    cxt.quest:Complete()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)



QDEF:AddConvo("escort_home")
    :Confront(function(cxt, who)
            if cxt.location == cxt.quest:GetCastMember("merch_home") then
                return "STATE_DONE"
            end
            if cxt.location:HasTag("in_transit") and not cxt.quest.param.done_interrupt then
                return "STATE_INTERRUPT"
            end
        end)
    :State("STATE_INTERRUPT")
        :Loc{
            DIALOG_INTRO = [[
                * {merchant} seems incredibly nervous as you're walking down the road, and rightfully so.
                * Before you know it, a group of bandits block your way.
                player:
                    !left
                agent:
                    !right
                    !angry_accuse
                    There {merchant.heshe} is! Listen {player}, you should get out of here.
                    We have some unfinished business with {merchant} here.
            ]],
            OPT_DEFEND = "Defend {merchant}",
            DIALOG_DEFEND = [[
                player:
                    If you want {merchant.himher}, you're going to have to go through me.
                    !fight
                agent:
                    !fight
            ]],
            DIALOG_DEFEND_WON = [[
                {merchant_dead?
                    * You may have won the battle, but {merchant} didn't make it.
                    player:
                        Aw shucks...
                }
                {not merchant_dead?
                    player:
                        !left
                    merchant:
                        !right
                        Well now I'm really glad you came!
                    player:
                        Don't thank me just yet. We should keep moving.
                }
            ]],
            OPT_ABANDON = "Abandon {merchant}",
            DIALOG_ABANDON = [[
                player:
                    !left
                    Uhh you know what? I totally forgot I had plans today.
                    I really gotta get going. Can't just show up late, you know?
                merchant:
                    !right
                    What?! No, you said you would help!
                * You turn on your heel and hurry away. As you're leaving you see the bandits surround {merchant}.
            ]],
        }
        :Fn(function(cxt) 
            cxt.quest.param.done_interrupt = true
            local bandits = CreateCombatParty("BANDIT_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(bandits[1])

            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle()
                    :OnWin()
                        :Fn(function() 
                            cxt.enc.scratch.merchant_dead = cxt.quest:GetCastAgent("merchant"):IsDead()
                            cxt:Dialog("DIALOG_DEFEND_WON")
                            if cxt.enc.scratch.merchant_dead then
                                cxt.quest:Fail()
                            else
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end)

            cxt:Opt("OPT_ABANDON")
                :Dialog("DIALOG_ABANDON")
                :ReceiveOpinion( OPINION.ABANDONED, nil, cxt.quest:GetCastAgent("merchant"))
                :Fn(function(cxt)
                    cxt.quest:Fail()
                    cxt:Travel()
                end)

        end)

    :State("STATE_DONE")
        :Loc{
            DIALOG_INTRO = [[
                * You arrive at {merchant}'s home, with {merchant} thankfully still in one piece.
                player:
                    !left
                merchant:
                    !right
                    Phew! Good to be home after all that.
                    Thanks for the help! You earned this!
            ]],

            OPT_REWARD = "Receive your reward",
        }
        :Fn(function(cxt) 
                local HELP_AMT = 40 + 25*cxt.quest:GetRank()
                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_REWARD")
                    :CompleteQuest()
                    :ReceiveMoney( HELP_AMT )
                    :Fn(function(cxt)
                        cxt.quest:GetCastMember("merchant"):Dismiss()
                    end )
                    :Travel()
            end)

QDEF:AddConvo("escort_home")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.quest.param.patience_timer <= 0 end)
        :Loc{
            DIALOG_IMPATIENT = [[
                player: 
                    !left
                merchant:
                    !right
                    !crossed
                    I don't have time for this. I'll take my chances with bandits.
                    !exit
                * {merchant} leaves, annoyed with your tardiness.
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest:Fail()
            cxt:Dialog("DIALOG_IMPATIENT")
            cxt.quest:GetCastMember("merchant"):Dismiss()
            StateGraphUtil.AddEndOption(cxt)
        end)