local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    cooldown = EVENT_COOLDOWN.SHORT,
}

:AddCast{
    cast_id = "wrangler",
    condition = function(agent, quest) 
        return agent:GetContentID() == "OSHNU_WRANGLER"
    end,
}:AddCastFallback{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "OSHNU_WRANGLER" ) )
    end,
}

:AddCast{
    cast_id = "officer",
    condition = function(agent, quest) 
        return agent:GetContentID() == "ADMIRALTY_GOON"
    end,
}:AddCastFallback{
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "ADMIRALTY_GOON" ) )
    end,
}

:AddCast{
    cast_id = "oshnu",
    when = QWHEN.MANUAL,
}

:AddOpinionEvents{
    helped_with_oshnu=  
    {
        delta = OPINION_DELTAS.OPINION_UP,
        txt = "Helped them with their stubborn oshnu",
    },
    attack_oshnu =  
    {
        delta = OPINION_DELTAS.BAD,
        txt = "Attacked their oshnu",
    },
}

QDEF:AddConvo()
    :Priority(CONVO_PRIORITY_HIGHEST)
    :ConfrontState("CONF", function(cxt) return cxt.location:HasTag("in_transit") end)
        :Loc{
            DIALOG_INTRO = [[
                wrangler:
                    !left
                officer:
                    !right
                    I won't tell you again, {wrangler}! Move that oshnu or I'll have it towed straight to the noodle shop.
                wrangler:
                    I'm trying! {oshnu.HeShe}'s just anxious--never seen this many people on the road before.
                    Give me a second to calm {oshnu.himher} down and we'll get out of your way.
                officer:
                    You've had more than enough seconds. I'm calling the butcher.
            ]],
            OPT_ASK_WRANGLER = "Ask {wrangler} what's going on",
            DIALOG_ASK_WRANGLER = [[
                wrangler:
                    !right
                player:
                    !left
                    What's the hold up, huh? I got places to be.
                wrangler:
                    Aw, Hesh. It's my oshnu, {oshnu}. {oshnu.HeShe} won't come out of {oshnu.hisher} shell and it's causing a jam.
                officer:
                    !right
                    !angry_accuse
                    Yeah, well, once the butcher gets here, that oshnu will <i>become</i> jam.
                player:
                    Huh?
                officer:
                    ...
                    I mean we're gonna kill it.
                    And move it out of the way.
                player:
                    Weird threat, but okay.
            ]],
            OPT_ASK_OFFICER = "Ask {officer} what's going on",
            DIALOG_ASK_OFFICER = [[
                officer:
                    !right
                player:
                    !left
                    Hey, an oshnu blocking the road ain't exactly news. 
                    What's the problem?
                officer:
                    The problem is <i>this</i> oshnu. That's the third time this week it's caused a problem, and {wrangler} refuses to discipline it.

            ]],
            OPT_COAX_OSHNU = "Coax the oshnu into moving",
            DIALOG_COAX_OSHNU = [[
                player:
                    Alright, stitch your britches. I got this.
                * You approach the oshnu, who sticks an eyestalk out just enough to get a look at you.
                oshnu:
                    !right
                player:
                    Hey there, champ. What do you say we move you out of the way?
                    Then you can have your meltdown in peace, without all these people bothering you, huh?
                * The oshnu eyes you suspiciously.
            ]],
            DIALOG_COAX_SUCCESS = [[
                player:
                    !left
                    Just move to the side, there, and then you can hunker right back down again. From one layabout to another, huh?
                oshnu:
                    !right
                * You seem to have made a connection with the oshnu, who emerges fully from its shell and moves off to the side of the road.
                wrangler:
                    !right
                    Good oshnu, {oshnu}! Thank Hesh!
                    And thank you, {player}. You certainly have a way with the snails, huh? 
                    Here, for your troubles. And for saving my {oshnu}.
                    !give
                * {wrangler} hands you {1#money}.
            ]],
            DIALOG_COAX_FAIL = [[
                player:
                    !left
                    Look, you wanna lay around doing nothing, I got nothing but support for you. 
                    But when you block my way? Well, then I take issue. 
                    So come on, you slimebucket. Get movin'.
                oshnu:
                    !right
                * The oshnu retracts its head fully into its shell and seals itself inside.
                player:
                    Sheesh. Try to ask nicely...
            ]],
            OPT_ATTACK_OSHNU = "Attack the oshnu",
            DIALOG_ATTACK_OSHNU = [[
                player:
                    Why wait for the butcher? Seems to me that there's a pretty straightforward way to resolve this.
                wrangler:
                    !fight
                    Don't you dare!
                player:
                    !fight
                    I dare all the time, bud.
            ]],
            DIALOG_ATTACK_WON = [[
                {oshnu_dead?
                * With the oshnu killed, its anchor grip on the ground releases, and shoving it to the side is an easy if unglamourous task.

                }
                {not oshnu_dead?
                * The oshnu gets the message, and moves to the side if only to get away from you.

                }
            ]],
            REQ_ALREADY_TRIED = "You already tried this",
            OPT_LEAVE = "Leave",
            DIALOG_LEAVE = [[
                { asked?
                    player:
                        Huh, well, uh...
                        Wish I could help, but, uh. 'Scuse me.
                    * Ignoring the grunts of protest from all parties, you squeeze your way around the oshnu's shell, making your exit.
                }
                { not asked?
                * Ignoring the grunts of protest from all parties, you squeeze your way around the oshnu's shell, making your exit.
            }
            ]],
            SIT_MOD = "The oshnu is stubborn."
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.quest.param.coax_amt = 25 + 15 * cxt.quest:GetRank()
                local snail = cxt.quest:CreateSkinnedAgent("MERCHANT_SNAIL")
                snail:GenerateRandomName()
                cxt.quest:AssignCastMember("oshnu", snail)
                cxt:Dialog("DIALOG_INTRO")
            end
            if not cxt.quest.param.asked then
                cxt:Opt("OPT_ASK_WRANGLER")
                    :Dialog("DIALOG_ASK_WRANGLER")
                    :Fn(function( cxt )
                        cxt.quest.param.asked = true
                        cxt.enc.scratch.asked = true
                    end)
                
                cxt:Opt("OPT_ASK_OFFICER")
                    :Dialog("DIALOG_ASK_OFFICER")
                    :Fn(function( cxt )
                        cxt.quest.param.asked = true
                        cxt.enc.scratch.asked = true
                    end) 
            else
                cxt:Opt("OPT_COAX_OSHNU")
                    :ReceiveOpinion("helped_with_oshnu", nil, cxt.quest:GetCastAgent("wrangler"))
                    :ReqCondition(not cxt.quest.param.failed_coax, "REQ_ALREADY_TRIED")
                    :Negotiation{
                        target_agent = cxt.quest:GetCastAgent("oshnu"),
                        helpers = {cxt.quest:GetCastAgent("wrangler")},
                        situation_modifiers = {
                                { value = 5*math.ceil(cxt.quest:GetRank()/2), text = cxt:GetLocString("SIT_MOD") }                    
                            },
                    }
                        :OnSuccess()
                            :Fn(function( cxt )
                                cxt:Dialog("DIALOG_COAX_SUCCESS", cxt.quest.param.coax_amt)
                                TheGame:GetGameState():GetCaravan():AddMoney(cxt.quest.param.coax_amt)
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)

                        :OnFailure()
                            :Fn(function( cxt )
                                cxt:Dialog("DIALOG_COAX_FAIL")
                                cxt.quest.param.failed_coax = true
                            end)

                cxt:Opt("OPT_ATTACK_OSHNU")
                    :ReceiveOpinion("attack_oshnu", nil, cxt.quest:GetCastAgent("wrangler"))
                    :Battle{
                        enemies = {cxt.quest:GetCastAgent("oshnu")},
                    }
                        :OnWin()
                            :Fn(function() 
                                cxt.enc.scratch.oshnu_dead = cxt.quest:GetCastAgent("oshnu"):IsDead()
                                cxt:Dialog("DIALOG_ATTACK_WON")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)
            end
            cxt:Opt("OPT_LEAVE")
                :Dialog("DIALOG_LEAVE")
                :Travel()
        end)