
local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    on_init = function(quest)
        quest.param.max_daggers = math.random(1, 3)
        quest.param.daggers = 0
    end,
}

:AddCast{
    cast_id = "leader",
    condition = function(agent, quest)
        return agent:GetFactionID() == "BANDITS"
    end,
}
:AddCastFallback( "BANDIT_GOON" )


:AddObjective{
    id = "intro",
    state = QSTATUS.ACTIVE,
}


QDEF:AddConvo("intro")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * You notice a jagged dagger laying conspicuously on the trail.
            ]],
        }
        :Fn(function(cxt)
            cxt.quest:Complete()
            cxt:Dialog("DIALOG_INTRO")
            cxt:GoTo( "ANOTHER_ONE" )
        end )

    :State("ANOTHER_ONE")
    :SetLooping(true)
        :Loc{
            OPT_PICK_UP = "Pick up a {1#card}",

            OPT_LEAVE = "Leave the dagger, and give it a wide berth",
            DIALOG_LEAVE = [[
                player:
                    !laugh
                    Like I'm falling for that one.
            ]],

            DIALOG_ANOTHER_ONE = [[
                * Further down the lane, you find yet another dagger.
            ]],

            DIALOG_PICK_UP1 = [[
                !take
                Hammers are more my thing, but they don't fit in your pants nearly as well.
            ]],
            DIALOG_PICK_UP2 = [[
                !dubious
                Another one? Guess this is my lucky dagger.
                * You laugh alone at your joke.
            ]],
            DIALOG_PICK_UP3 = [[
                !happy
                Alright, free stabby thing. Might come in handy.
            ]],
            DIALOG_PICK_UP4 = [[
                !thought
                This trail seems kind of suspicious... but at least I'll be heavily armed.
            ]],
            DIALOG_PICK_UP5 = [[
                Now this is just irresponsible. And I'd know all about it.
            ]],
            DIALOG_PICK_UP6 = [[
                Ain't seen this many daggers since the time I used Mullifee's dagger collection to make my way out of the hedge maze.
            ]],
            DIALOG_PICK_UP7 = [[
                !take
                This is too much good fortune. Maybe I should donate these.
            ]],

            OPT_LEAVE = "Leave the dagger, and steer clear of the whole avenue",
            DIALOG_ABORT = [[
                player:
                    Yeah, no.
            ]],
            DIALOG_LEAVE = [[
                player:
                    !laugh
                    Like I'm falling for that one.
            ]],
        }
        :Fn(function(cxt)
            if (cxt.quest.param.daggers or 0) > 0 then
                cxt:Dialog("DIALOG_ANOTHER_ONE")
            end

            cxt:Opt( "OPT_PICK_UP", "makeshift_dagger" )
                :Fn(function() 
                    if cxt.quest.param.quips == nil then
                        cxt.quest.param.quips = { "DIALOG_PICK_UP1", "DIALOG_PICK_UP2", "DIALOG_PICK_UP3", "DIALOG_PICK_UP4", "DIALOG_PICK_UP5", "DIALOG_PICK_UP6", "DIALOG_PICK_UP7" }
                    end
                    local pickup_dialog = table.remove( cxt.quest.param.quips, math.random( #cxt.quest.param.quips ) ) or "DIALOG_PICK_UP1"
                    cxt:Dialog(pickup_dialog)
                end)
                :GainCards( { "makeshift_dagger" } )
                :Fn(function(cxt)
                    cxt.quest.param.daggers = (cxt.quest.param.daggers or 0) + 1
                    if cxt.quest.param.daggers >= cxt.quest.param.max_daggers then
                        cxt:GoTo("BATTLE")
                    end
                end )

            cxt:Opt( "OPT_LEAVE" )
                :Dialog( cxt.quest.param.daggers > 0 and "DIALOG_ABORT" or "DIALOG_LEAVE")
                :Travel()
            end )

    :State("BATTLE")
    :Loc{
        PRE_BATTLE = [[
            * As you turn the corner, you see some ruffians engaged in target practice with an array of shattered glass bottles.
            leader:
                !right
                Careful, stranger. You wouldn't wanna cut yourself.
                Heh.
            player:
                !dubious
                You throwing <i>knives</i> at <i>glass</i>?
                That's stupid. I mean, how do you even know you got the blade the right way around?
                You need something squishy to stab for proper practice. Like wood, or a straw dummy, or, hmmm...
            leader:
                A <i>real</i> dummy?
            player:
                Well, sure, if you can find--
                Hey, wait a second.
        ]],
        LIKED = [[
            * As you turn the corner, you see some ruffians engaged in target practice with an array of shattered glass bottles.
            leader:
                !right
                !greeting
                Well look who it is!
                Picking up after me, {player}?
            player:
                !happy
                Just watching over my buddy.
                You children have fun now!
                !wave
        ]],
        POST_BATTLE =[[
            player:
                !tired
                Well that was fun. And dangerous.
                !happy
                But mostly fun.
        ]]
    }
    :Fn(function(cxt)

            local leader = cxt.quest:GetCastMember("leader")
            cxt.enc:SetPrimaryCast(leader)
            cxt.scratch.opfor = CreateCombatBackup( leader, nil, cxt.quest:GetDifficulty() + 1)
            table.insert( cxt.scratch.opfor, 1, leader )

            if leader:IsLiked() then
                cxt:Dialog("LIKED")
                cxt:Travel()
            else
                cxt:Dialog("PRE_BATTLE")

                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        enemies = cxt.scratch.opfor,
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt)
                            cxt.enc:ExitAgent( cxt:GetAgent() )
                        end,
                    }
                    :OnWin():Fn( function( cxt ) cxt.encounter:ExitAgent( cxt:GetAgent() ) end ):Dialog( "POST_BATTLE")
                    :Travel()
            end

        end )
