local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
    on_init = function( quest )
        quest.param.scoundrel = math.random() < 0.5
    end,
}

QDEF:AddCast{
    cast_id = "vip",
    condition = function( agent, quest )
        return agent:GetCombatStrength() <= 2 and not agent:KnowsPlayer()
    end,
}

QDEF:AddObjective{
    id = "intro",
    state = QSTATUS.ACTIVE
}

QDEF:AddObjective{
    id = "vip",
    title = "VIP",
    icon = engine.asset.Texture("icons/quests/secret_shopper.tex"),
    desc = "You have a VIP token you can use to access lounge drinks.",
    mark = function(quest, t, in_location)
        if in_location then
            local location = TheGame:GetGameState():GetLocation( "PEARL_OSHNUDROME" )
            if location and location:GetProprietor() then
                table.insert(t, location:GetProprietor())
            end
        end
    end,
}

local convo = QDEF:AddConvo("intro")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit")  end)
        :Loc{           
            DIALOG_INTRO = [[
                * A stumbling passerby collides with you suddenly. Seems like an accident, but you never know in the Foam.
                player:
                    !left
                vip:
                    !right
                    !drunk
                player:
                    Hey, watch it!
                vip:
                    !surprise
            ]],

            OPT_DEMAND = "Demand an apology",
            DIALOG_DEMAND = [[
                player:
                    !friend
                    The next words I hear better be sweet and apologetic-like.
                vip:
                    !scared
                    Ok, ok!
                    I'm <i>—hic—</i> ssshhoory! Won't happen again!
                    !exit
            ]],

            DIALOG_DEMAND_SCOUNDREL = [[
                player:
                    !friend
                    The next words I hear better be sweet and apologetic-like.
                vip:
                    !drunk_hiccup
                * The {vip.desc} burps loudly in your face.
                * Oh, <i>it's on.</i>
            ]],

            DIALOG_POST_DEMAND = [[
                player:
                    !hips
                * You feel a surge of self-satisfaction.                    
            ]],

            DIALOG_POST_BATTLE = [[
                vip:
                    !exit
                player:
                    {dead?
                        Well that escalated a little quickly...
                        !exit
                    }
                    {not dead?
                        Maybe that wasn't necessary...
                        !thought
                        That's probably why it was so fun.
                    }
            ]],

            OPT_APOLOGIZE = "Make an apology",
            DIALOG_APOLOGIZE = [[
                player:
                    !placate
                    Listen, sorry.
                    Could've been my fault, friend.
                vip:
                    A fine gentleman you are!
                    Ever heard of the Oshnudrome?
                player:
                    Well of course.
                    I never go there though, they'll cut you off after two orders o' plonk...
                vip:
                    Not with this VIP token they won't!
                    Perhaps a distinguished one like yourself could use it.
                    !drunk_hiccup
                    Enjoy!
                    !wave
                player:
                    !wave
            ]],

            DIALOG_APOLOGIZE_SCOUNDREL = [[
                player:
                    !placate
                    Pardon me! My large, powerful arms sometimes get in the way.
                vip:
                    !cagey
                    Uh... right.
                    Think nothing of it!
                    !exit
                * {vip} scurries off quickly, and only when {vip.heshe} disappears from view do you notice your pockets hanging open.
                player:
                    Rats.
            ]],
        }
        :Fn(function(cxt)
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("vip"))
            cxt:Dialog("DIALOG_INTRO")

            if cxt.quest.param.scoundrel then            
                cxt:Opt("OPT_DEMAND")
                    :Dialog("DIALOG_DEMAND_SCOUNDREL")
                    :CompleteQuest()
                    :Fn(function(cxt)
                        cxt:DoBattle()
                    end )
                    :Dialog("DIALOG_POST_BATTLE")
                    :Travel()
            else
                cxt:Opt("OPT_DEMAND")
                    :Dialog("DIALOG_DEMAND")
                    :CompleteQuest()
                     :Fn(function(cxt)
                        cxt:GetAgent():OpinionEvent( OPINION.DISAPPROVE_MINOR )
                        ConvoUtil.DoResolveDelta(cxt, 10)
                    end )    
                    :Dialog("DIALOG_POST_DEMAND")
                    :Travel()
            end

            if cxt.quest.param.scoundrel then
                cxt:Opt("OPT_APOLOGIZE")
                    :Dialog("DIALOG_APOLOGIZE_SCOUNDREL")
                    :CompleteQuest()
                    :Fn(function(cxt)
                        cxt.caravan:PayMoney( math.random(1, 5) * 10)
                    end )
                    :Travel()
            else
                cxt:Opt("OPT_APOLOGIZE")
                    :Dialog("DIALOG_APOLOGIZE")
                    :CompleteQuest("intro")
                    :ActivateQuest("vip")
                    :Travel()
            end
        end)


QDEF:AddConvo("vip")
    :Loc{
        OPT_VIP = "Show your VIP token",
        DIALOG_VIP = [[
            player:
                Serve me the good stuff!
            * {agent} looks at the token you've provided with a dubious expression.
            agent:
                Where'd you get this?
            player:
                !shrug
                C'mon, it's not like I just picked it up from a drunken stranger in an alley somewhere.
            agent:
                ...
                Alright, what'll it be?
        ]],

        OPT_ORDER = [[Order {1#card}]],
    }
    :Hub( function(cxt, who)
        if cxt.location:GetContentID() ~= "PEARL_OSHNUDROME" or cxt.location:GetProprietor() ~= who then
            return
        end

        if not cxt.quest.param.showed_vip then
            cxt:Opt("OPT_VIP")
                :Dialog("DIALOG_VIP")
                :Fn(function(cxt)
                    cxt.quest.param.showed_vip = true
                end)
        else
            cxt:Opt("OPT_ORDER", "snellecks_finest")
                :GainCard( "snellecks_finest" )
                :DeliverMoney( 100 )

            cxt:Opt("OPT_ORDER", "lumin_wine")
                :GainCard( "lumin_wine" )
                :DeliverMoney( 50 )

            cxt:Opt("OPT_ORDER", "fleadshell_brandy")
                :GainCard( "fleadshell_brandy" )
                :DeliverMoney( 150 )
        end
    end)
