local battle_defs = require "battle/battle_defs"
local negotiation_defs = require "negotiation/negotiation_defs"

local food_options =
{
    HESHBLADDER_SURPRISE =
    {
        condition = function()
            local player = TheGame:GetGameState():GetPlayerAgent()
            return player:GetAspect("health"):Get() - 10 > 0 
        end,
        effect = function(cxt)
            ConvoUtil.DoHealthDelta(cxt, -10 )
            local player = TheGame:GetGameState():GetPlayerAgent()
            player.health:AddStatModifier("EVENT_FOOD_HESHBLADDER_SURPRISE", 5)
        end,
    },

    MUD_JUMPER_MUFFINS =
    {
        condition = function()
            local current_resolve, max_resolve = TheGame:GetGameState():GetCaravan():GetResolve()
            return current_resolve < max_resolve
        end,
        effect = function(cxt)
            ConvoUtil.DoResolveDelta(cxt, 5 )
        end,
    },

    BOGROOT_ECLAIR = 
    {
        condition = function()
            local player = TheGame:GetGameState():GetPlayerAgent()
            for i,card in ipairs(player.battler:GetCards()) do
                if card:HasUpgrades() and not card:UpgradeReady() and not card:IsHatched() then
                    return true
                end
            end
            for i,card in ipairs(player.negotiator:GetCards()) do
                if card:HasUpgrades() and not card:UpgradeReady() and not card:IsHatched() then
                    return true
                end
            end
        end,
        effect = function( cxt )           
            local options = {}
            local player = TheGame:GetGameState():GetPlayerAgent()
            for i,card in ipairs(player.battler:GetCards()) do
                if card:HasUpgrades() and not card:UpgradeReady() and not card:IsHatched() and card:GetXP() ~= nil then
                    table.insert(options, card)
                end
            end
            for i,card in ipairs(player.negotiator:GetCards()) do
                if card:HasUpgrades() and not card:UpgradeReady() and not card:IsHatched() and card:GetXP() ~= nil then
                    table.insert(options, card)
                end
            end
            if #options > 0 then
                local choice = table.arraypick(options)
                choice:AddXP(choice:GetMaxXP())
                cxt:Dialog("DIALOG_XP_GAIN", choice)
            end
        end, 
    },

    FLEADSHELL_BRANDY =
    {
        effect = function(cxt)            
            cxt:ForceTakeCards{"fleadshell_brandy"}
        end,
    },

    TIDEPOOL_PODS =
    {
        condition = function()
            local player = TheGame:GetGameState():GetPlayerAgent()
            return player:GetAspect("health"):Get() - 8 > 0 
        end,
        effect = function( cxt )
            ConvoUtil.DoHealthDelta(cxt, -8 )
            cxt:ForceTakeCards{"tidepool_pods"}
        end,
        bad = true,
    },

    SPICED_SCUTTLERS = 
    {
        bad = true,
        effect = function( cxt )
            cxt:ForceTakeCards{"ulcer"}
        end
    },

    BASTED_BARNACLES = 
    {
        bad = true,
        effect = function( cxt )
            cxt:ForceTakeCards{"sick"}
        end
    },
}

--Card from food effects
Content.AddBattleCard( "fleadshell_brandy", 
{
    name = "Fleadshell Brandy",
    anim = "drink",
    flavour = "'Is that a hint of tallowfish that I’m tasting? Really strengthens the overall bouquet.'",
    desc = "{DRINK}.\nGain 1 action.",

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    cost = 0,
    target_type = TARGET_TYPE.SELF,

    max_charges = 3,

    flags = battle_defs.CARD_FLAGS.ITEM,

    OnPostResolve = function( self, battle, attack)
        local card = Battle.Card( "empty_bottle", self.owner )
        card.show_dealt = false
        battle:DealCards( {card}, battle:GetDiscardDeck() )
        battle:BroadcastEvent( battle_defs.BATTLE_EVENT.DRINK, battle, 1, {card} )
        battle:ModifyActionCount(1)
    end,
})
Content.AddBattleCard( "tidepool_pods", 
{
    name = "Tidepool Pods",
    anim = "throw",
    icon = "battle/rugs_tidepool_pods.tex",
    flavour = "'Ok, we definitely shouldn’t be eating these.'",
    desc = "Hits all enemies.",

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    cost = 1,

    max_charges = 5,

    flags = battle_defs.CARD_FLAGS.ITEM | battle_defs.CARD_FLAGS.RANGED,
    target_mod = TARGET_MOD.TEAM,

    min_damage = 3,
    max_damage = 6,
})

local QDEF = QuestDef.Define
{
    qtype = QTYPE.EVENT,
    spawn_event_mask = QEVENT_TRIGGER.TRAVEL,
}

QDEF:AddCastByAlias{
    cast_id = "chef",
    alias = "WANDERING_CHEF",
}

QDEF:AddConvo()
    :Confront(function(cxt, who)
        return "STATE_CONFRONT"
    end)

    :State("STATE_CONFRONT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !happy
                * You're used to seeing strange characters on the road, but the fellow who stops you up short would look out of place just about anywhere.
                agent:
                    $miscAlluring
                    Why, what's this? You're no pilgrim, that's for sure. 
                    But you also don't quite look like a Pearly.
                player:
                    Best of both worlds, is what I am. Born and bred from the Foam, but seasoned by the road.
                agent:
                    Then perhaps your palate is equally adventurous, hm? Fancy trying one of my amuse-bouches? 
                    Rare ingredients sourced from the Bog and local delicacies, fused into one unique cuisine.
            ]],
            DIALOG_DECLINE = [[
                player:
                       I hear "fused" and all I can think is that it'll gum up my insides.
                       Thanks, but no thanks. 
                agent: 
                    !dubious
                    $neutralDubious
                    Fine. That's fine. I don't even care.
                * {agent} definitely cares.
            ]],            

            OPT_DECLINE = "Decline the offer",
            OPT_TRY_FOOD_HESHBLADDER_SURPRISE = "Try the heshbladder au vin",
            DIALOG_HESHBLADDER_SURPRISE = [[
                player:
                    !shrug
                    That sounds fancy. Means it's got booze in it, right?
                agent:
                    !permit
                    The whole is greater than the sum of its parts.
                player:
                    !drink
                * You throw back the tiny serving, barely remembering to extend your pinkie.
                * It doesn't taste like much, but then you didn't give it much time in your mouth.
                player:
                    Yeah, feels like I put food in there.
                    Surprisingly filling, actually.
                agent:
                    I, ah, was hoping you might have something to say about the taste.
                    But at least I know it satisfies.
            ]],
            OPT_TRY_FOOD_MUD_JUMPER_MUFFINS= "Try the mud jumper muffins",
            DIALOG_MUD_JUMPER_MUFFINS= [[
                player:
                    !shrug
                    Bog muffins, huh? Sounds risky, but I guess no one ever got hurt by a muffin.
                agent:
                    !agree
                    You would think!
                    !give
                player:
                    !take
                    !drink
                * You eat the muffin, deftly catching a crumb as it tries to escape down your collar.
                player:
                    !burp
                    Hey, pretty good!
                agent:  
                    !happy
            ]],
            OPT_TRY_FOOD_BOGROOT_ECLAIR = "Try the deep-eel eclair",
            DIALOG_BOGROOT_ECLAIR = [[
                player:
                    !shrug
                    Last time I had deep-eel it was for my birthday.
                    Didn't realize you could make it into a sweet?
                agent:
                    !give
                    Only time will tell. Let's see, shall we?
                player:
                    !drink
                * You bite down on the eclair and are slightly taken aback by the wriggling insides.
                  agent:
                      Turns out when you eat them live, they're quite sugary!
                player:
                    !surprised
                    That's, uh, got a real kick to it.
                agent:  
                       Well that <i>is</i> odd, considering they don't have feet.
            ]],
            OPT_TRY_FOOD_FLEADSHELL_BRANDY = "Try the fleadmilk brandy pudding",
            DIALOG_FLEADSHELL_BRANDY = [[
                player:
                    !shrug
                    Brandy and pudding, what's not to like?
                agent:
                    Excellent choice!
                    !give 
                player:
                    !take
                    * The pudding execretes an off-white foam when you apply any pressure.
                agent:
                    Careful! If you bruise it, it destabilizes the flavors.
                player:
                    !dubious 
                    * Not sure how you're supposed to chew a pudding without bruising it, so you throw your head back and swallow it whole, raisins and all.
                    !drink
                   agent:
                       !interest 
                       Well?
                player:
                    !burp
                    I can taste it... <i>in</i> my stomach?
                agent:
                    !chuckle
                    And?
                player:
                    My stomach... is pleased.
            ]],
            OPT_TRY_FOOD_TIDEPOOL_PODS = "Try the tidepool pods",
            DIALOG_TIDEPOOL_PODS = [[
                player:
                    !shrug
                    You sure these are food? The colors look unnatural.
                agent:
                    !happy
                    Nothing is food unless it's eaten! 
                player:
                    I guess that's true.
                    !drink 
                * You bite down on the thin membrane of the pod, which promptly explodes from the pressure.
                player:
                    !wince2 
                    Gah!
                * The inside of your mouth erupts in itchy sores.
                agent:
                    !bashful
                    Oh my... I'm terribly sorry. I guess that one wasn't ripe yet. 
                    Please, take the rest as an apology. They might not ripen, but at the very least you can use them on your enemies. 
            ]],
            OPT_TRY_FOOD_SPICED_SCUTTLERS = "Try the truffled scuttlers",
            DIALOG_SPICED_SCUTTLERS = [[
                player:
                    !shrug
                    Didn't know scuttlers were even edible!
                agent:
                    Prepare to be surprised.
                    !give
                player:
                    !drink
                * The scuttlers don't go down easy. You swear one of them clings to your esophagus.
                player:
                    !spit
                    Blagh!
                    Thought you said they were edible?!
                agent:
                    !bashful
                       Actually, I said you'd be surprised.
                    $miscEmbarrassed 
                    But I admit that was an unintended reading.
            ]],
            OPT_TRY_FOOD_BASTED_BARNACLES = "Try the fire-shucked barnacles",
            DIALOG_BASTED_BARNACLES = [[
                player:
                    !shrug
                    Fire-shucked? Ain't that a Rentorian curse?
                agent:
                    !surprised
                       Ah, you <i>are</i> well-travelled.
                       Well, let's say this dish is... divisive.
                player:
                    !drink
                * The barnacles have a pleasing char, but the texture makes your throat constrict involuntarily.
                player:
                    !spit
                    No.
                    !thought 
                    Well...
                    !neutral
                    No. Naw. 
                agent:
                    You still don't look too sure.
                    But if food is a journey, your face told the whole story.
            ]],
            DIALOG_XP_GAIN = [[
                *** XP granted on {1#card}.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:Complete()
                cxt.encounter:SetPrimaryCast(cxt.quest:GetCastAgent("chef"))

                local temp_options = {}
                for i, id, data in sorted_pairs( food_options ) do
                    --if there's no condition or the condition is met, add it
                    if data.condition == nil or data.condition() then
                        table.insert(temp_options, id)
                    end
                end
                table.shuffle(temp_options)
                local length = #temp_options
                for i=1, length - 3 do
                    table.remove(temp_options, math.random(#temp_options))
                end
                cxt.quest.param.options = temp_options
                cxt:Dialog("DIALOG_INTRO")
            end

            for k,v in pairs(cxt.quest.param.options) do
                cxt:Opt("OPT_TRY_FOOD_" .. v)
                    :PreIcon(global_images.food)
                    :Fn(function(cxt)
                        cxt:Dialog("DIALOG_" .. v)
                        if food_options[v].bad then
                            cxt:GoTo("STATE_BAD_FOOD", v)
                        else
                            if food_options[v].effect then
                                food_options[v].effect(cxt)
                            end

                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    end)
            end

            cxt:Opt("OPT_DECLINE")
                :PreIcon(global_images.dislike)
                :Dialog("DIALOG_DECLINE")
                :ReceiveOpinion( OPINION.INSULT )
                :Travel()

        end)

    :State("STATE_BAD_FOOD")
        :Loc{
            DIALOG_APOLOGY = [[
                player:
                    !injured
                    Thanks for the grub, but maybe stick to the normal stuff, huh?
                agent:
                    !surprise
                    "Normal stuff"? That's a bit loaded. 
                    There is no discovery without risk.
                    However, it's hard to find willing taste-testers as it is. I'd prefer nobody knew about your, ah, bad taste.
            ]],
            OPT_DONT_TELL = "Promise not to tell anyone",
            TT_DONT_TELL = "You'll feel the extent of that dubious meal!",
            -- Below is old dialogue from Rook, needs replacing
            DIALOG_DONT_TELL = [[
                player:
                    !dubious
                    $neutralResigned
                    Who would I tell? It's just food.
                    You think my life is really so boring that <i>this</i> is what I choose to relive?
                agent:
                    !angry_shrug
                    Here I thought my food would be an inspiration. 
                    But if we're agreed, that's enough for me. 
                * {agent} cleans up {agent.hisher} gear and leaves you to rinse the taste from your mouth.
            ]],
            OPT_INSULT = "Scold {agent}",
            TT_INSULT = "You'll feel better for scolding {agent}, but {agent.heshe} isn't likely to appreciate it.",

            DIALOG_INSULT = [[
                player:
                    !angry_accuse
                    Word of advice? If you wanna keep your head in the Foam, maybe don't go around trying to poison the locals.
                       Pilgrims, on the other hand...
                agent:
                    I thought you were worldly, but it seems you're just as bad as the rest of the Pearl. 
                    All waste, no taste. 
                    At least pilgrims are open to new experiences.
                    !exit
                * {Agent} leaves you alone with the failings of your palate.
            ]],
        }
        
        :Fn(function(cxt, food)
            cxt:Dialog("DIALOG_APOLOGY")

            cxt:Opt("OPT_DONT_TELL")
                :PostText( "TT_DONT_TELL" )
                :Dialog("DIALOG_DONT_TELL")
                :ReceiveOpinion( OPINION.SOCIAL_FAVOUR )
                :Fn(function(cxt)
                    if food_options[ food ] then
                        food_options[food].effect(cxt)
                    end
                end)
                :Travel()

            cxt:Opt("OPT_INSULT")
                :PostText( "TT_INSULT" )
                :Dialog("DIALOG_INSULT")
                :ReceiveOpinion( OPINION.INSULT )
                :Travel()
        end)
