local ITEM_CARDS = {"spear_head", "letter_opener", "lumin_grenade"}

AddOpinionEvent("GAVE_SECRET_DOCUMENTS", 
{
    delta = OPINION_DELTAS.MAJOR_GOOD,
    txt = "Gave them secret documents for free",
})

local QDEF = QuestDef.Define{
    title = "Return Intel to the Admiralty",
    icon = engine.asset.Texture("icons/quests/hatch_envelope_admiralty.tex"),
    qtype = QTYPE.OPPORTUNITY,
    tags = {"manual_spawn"},   
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Secret Admiralty Envelope",
    desc = "The Admiralty would almost certainly like to have their documents back.",
    mark = {"clerk"},
    terminal = true,
    on_complete = function(quest)
        for id,quest in TheGame:GetGameState():ActiveQuests() do
            if quest:GetContentID() == "HATCH_ENVELOPE_SPREE" then
                TheGame:GetGameState():RemoveActiveQuest(quest)
                break
            end
        end
    end
}
:AddCast{
    cast_id = "clerk",
    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "ADMIRALTY_CLERK" )
        table.insert(t, agent)
    end,
}

QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastAgent("clerk"):GetLocation() end)
    :Loc{
        DIALOG_INTRO = [[
            player:
                !left
            clerk:
                !right
                What can I help you with?
            player:
                I came across some classified Admiralty documents and I figured you might want them back.
            clerk:
                Classified documents? I'll take those off your hands. Don't worry, I'll sort this out.
        ]],
        DIALOG_INTRO_SECOND = [[
            player:
                !left
            clerk:
                !right
                Ah, so you changed your mind?
        ]],
        OPT_ASK_FOR_PAYMENT = "Ask for payment",
        DIALOG_ASK_FOR_PAYMENT = [[
            player:
                Of course you can have them. As long as I get something in return.
            clerk:
                Ah. I see.
                !crossed
                I probably shouldn't offer this but what if I trade you some of our equipment for the documents?
        ]],
        OPT_CHOOSE_ITEM = "Choose an item",
        DIALOG_CHOOSE_ITEM = [[
            player:
                Seems like a reasonable deal to me.
                !give
            clerk:
                !take
                Good. Let's just keep it quiet, yeah?
            player:
                My lips are sealed.
        ]],
        OPT_GIVE_FOR_FREE = "Give {clerk} the documents for free",
        TT_GIVE_FOR_FREE = "{clerk} will remember this.",
        DIALOG_GIVE_FREE = [[
            player:
                Sure, you can have them.
                !give
            agent:
                !take
                Thank you so much, {player}! My boss is really going to appreciate this.
        ]],
        OPT_DECLINE = "Decline their offer",
        DIALOG_DECLINE = [[
            player:
                Actually, I need to attend to something else.
            clerk:
                Oh.. Well come back if you change your mind. I really would like those documents.
            player:
                !exit
            clerk:
                !exit
        ]],
        CARD_PICK_TITLE = "Choose Equipment",
        CARD_PICK_SUBTITLE = "You can pick 1",
    }
    :Fn(function(cxt)
        if cxt:FirstLoop() then
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("clerk"))
            if not cxt.enc.scratch.done_intro then
                cxt:Dialog("DIALOG_INTRO")
                cxt.enc.scratch.done_intro = true
            else
                cxt:Dialog("DIALOG_INTRO_SECOND")
            end
        end

        cxt:Opt("OPT_ASK_FOR_PAYMENT")
            :Dialog("DIALOG_ASK_FOR_PAYMENT")
            :Fn(function(cxt)
                cxt:Opt("OPT_CHOOSE_ITEM")
                    :Dialog("DIALOG_CHOOSE_ITEM")
                    :Fn(function(cxt )
                        local cards = {}
                        for i,id in ipairs(ITEM_CARDS) do
                            table.insert(cards, Battle.Card(id, cxt.player))
                        end
                        cxt.enc:OfferCards(cards, cxt:GetLocString("CARD_PICK_TITLE"), cxt:GetLocString("CARD_PICK_SUBTITLE"), true)
                        StateGraphUtil.AddLeaveLocation(cxt)
                        cxt.quest:Complete()
                    end)
            end)

        cxt:Opt("OPT_GIVE_FOR_FREE")
            :Dialog("DIALOG_GIVE_FREE")
            :Fn(function(cxt)
                cxt:GetAgent():OpinionEvent(OPINION.GAVE_SECRET_DOCUMENTS)
                StateGraphUtil.AddLeaveLocation(cxt)
                cxt.quest:Complete()
            end)       

        cxt:Opt("OPT_DECLINE")
            :Dialog("DIALOG_DECLINE")
            :Travel()
    end)
