local INITIAL_PRICE = 100
local BONUS_PAYMENT = 75

local QDEF = QuestDef.Define{
    title = "Sell Intel to the Spree",
    icon = engine.asset.Texture("icons/quests/hatch_envelope_spree.tex"),
    qtype = QTYPE.OPPORTUNITY,
    tags = {"manual_spawn"},
    
}

:AddLocationCast{
    cast_id = "location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("BANDIT_CAMP"))
    end,
}

:AddObjective{
    id = "investigate",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    title = "Secret Admiralty Envelope",
    desc = "The Spree would most likely be interested in the secret Admiralty Papers.",
    mark = {"location"},
    terminal = true,
    on_complete = function(quest)
        for id,quest in TheGame:GetGameState():ActiveQuests() do
            if quest:GetContentID() == "HATCH_ENVELOPE_ADMIRALTY" then
                TheGame:GetGameState():RemoveActiveQuest(quest)
                break
            end
        end
    end
}
:AddCast{
    cast_id = "bandit",
    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "BANDIT_GOON" )
        table.insert(t, agent)
    end,
}

:AddLocationDefs{
    BANDIT_CAMP = {
        name = "Bandit Camp",
        plax = "EXT_SpreeCampsite_1",
        desc = "You can probably find some Spree members to sell the documents to here.",
        indoors = false,
        map_tags = {"tents"},
    },
}
QDEF:AddConvo():ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("location") end)
    :Loc{
        DIALOG_INTRO = [[
            player:
                !left
            bandit:
                !right
                !dubious
                Can I help you with something?
            player:
                I came across some classified Admiralty documents and I figured you might be interested.
            bandit:
                !happy
                Well certainly! Let's see them.
            player:
                Sure, for a price.
            bandit:
                Ah. I see. I can do {1#money}. Deal?
        ]],
        DIALOG_INTRO_SECOND = [[
            player:
                !left
            bandit:
                !right
                Ah, so you changed your mind?
                I'm still willing to pay {1#money} for those papers.
        ]],
        OPT_ACCEPT_PRICE = "Accept the initial offer",
        DIALOG_ACCEPT_PRICE = [[
            player:
                Sounds good to me!
                !give
            bandit:
                !take
                Excellent! We'll be sure to put these to good use.
        ]],
        OPT_DEMAND_MORE = "Demand an additional {1#money}",
        DIALOG_DEMAND_MORE = [[
            player:
                Uh that's not quite what I had in mind.
                Throw in another {1#money} and you've got yourself a deal.
            bandit:
                !crossed
                What makes you think they're worth that much?
        ]],
        DIALOG_DEMAND_SUCCESS = [[
            bandit:
                Alright fine. Just hand over the papers and get out of here.
                !give
            player:
                !take
                Glad we could make a deal.
        ]],
        DIALOG_DEMAND_FAIL = [[
            player:
                So what do you say? Are we in agreement?
            bandit:
                No way. The price is {1#money} and that's final.
        ]],
        OPT_DECLINE = "Decline their offer",
        DIALOG_DECLINE = [[
            player:
                Actually, I need to attend to something else.
            bandit:
                Oh.. Well come back if you change your mind.
            player:
                !exit
            bandit:
                !exit
        ]],
    }
    :Fn(function(cxt)
        if cxt:FirstLoop() then
            local bandit = cxt.quest:GetCastAgent("bandit")
            bandit:MoveToLocation(cxt.quest:GetCastMember("location"))
            cxt.enc:SetPrimaryCast(bandit)
            if not cxt.enc.scratch.done_intro then
                cxt:Dialog("DIALOG_INTRO", INITIAL_PRICE)
                cxt.enc.scratch.done_intro = true
            else
                cxt:Dialog("DIALOG_INTRO_SECOND", INITIAL_PRICE)
            end
        end

        cxt:Opt("OPT_ACCEPT_PRICE")
            :Dialog("DIALOG_ACCEPT_PRICE")
            :ReceiveMoney(INITIAL_PRICE)
            :Fn(function(cxt)
                cxt:GetAgent():OpinionEvent(OPINION.APPROVE)
                StateGraphUtil.AddLeaveLocation(cxt)
                cxt.quest:Complete()
            end)

        cxt:Opt("OPT_DEMAND_MORE", BONUS_PAYMENT)
            :ReqCondition(not cxt.enc.scratch.failed_negotiation)
            :Dialog("DIALOG_DEMAND_MORE", BONUS_PAYMENT)
            :Negotiation{
                on_success = function(cxt)
                    cxt:Dialog("DIALOG_DEMAND_SUCCESS")
                    cxt.enc:GainMoney( INITIAL_PRICE + BONUS_PAYMENT )
                    cxt.quest:Complete()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end,
                on_fail = function(cxt)
                    cxt.enc.scratch.failed_negotiation = true
                    cxt:Dialog("DIALOG_DEMAND_FAIL", INITIAL_PRICE)
                end,
            }        

        cxt:Opt("OPT_DECLINE")
            :Dialog("DIALOG_DECLINE")
            :Travel()
    end)
