local QDEF = QuestDef.Define{
        title = "Another Day Older and Deeper in Debt",
        qtype = QTYPE.SIDE,
        desc = [[{giver} has gotten {giver.himher}self into some money trouble with {target}. Find a way for {giver} to either make money or pay it back slowly.]],
        icon = engine.asset.Texture("icons/quests/side_another_day_older_and_deeper_in_debt.tex"),
        --icon = engine.asset.Texture("icons/quests/special_delivery.tex"),
        avoids = 
        {
            {"giver", "target"},
        },

    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent, quest)
            return agent:GetWorkBoss() and ( agent:GetContentID() == "LABORER" or agent:GetContentID() == "HEAVY_LABORER" )
        end,
    }

    :AddCast{
        cast_id = "target",

        condition = function(agent, quest)
            local valid, reason = QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true)
            if valid then
                return not AgentUtil.IsFriend(agent, quest:GetCastMember("giver"))
            end

        end,
        on_assign = function(quest, agent)
            if quest:GetCastMember("giver"):GetRelationship(agent) == RELATIONSHIP.NEUTRAL then
                quest:GetCastMember("giver").social_connections:SetRelationship(RELATIONSHIP.DISLIKED, agent)
            end
        end,

        events = {
            agent_retired = function(quest) 
                
                if quest:IsActive("talk_to_boss") then
                    quest:Cancel("talk_to_boss")
                end
                
                if quest:IsActive("talk_to_lender") then
                    quest:Complete("talk_to_lender")
                end

                if not quest:IsActive("return_to_giver") then
                    quest:Activate("return_to_giver")
                end
            end,
        }
    }

    :AddCast{
        cast_id = "boss",

        cast_fn = function(quest, t) 
            table.insert(t, quest:GetCastMember("giver"):GetWorkBoss())
        end,

        events = {
            agent_retired = function(quest) 
                
                if quest:IsActive("talk_to_boss") then
                    quest:Fail("talk_to_boss")
                end
                
                if quest:IsActive("talk_to_lender") then
                    quest:Fail("talk_to_lender")
                end

                if not quest:IsActive("return_to_giver") then
                    quest:Activate("return_to_giver")
                end
            end,
        }
    }

    :AddObjective{
        id = "talk_to_lender",
        title = "Talk to {target}",
        desc = [[{giver} has asked you to convince {target} to give {giver.himher} more favorable repayment terms.]],
        mark = { "target" },
        negotiation_targets = {"target"},
        on_activate = function(quest)
            quest:Activate("talk_to_boss")
        end,
        on_complete = function(quest)
            quest:Activate("return_to_giver")
            if quest:IsActive("talk_to_boss") then
                quest:Cancel("talk_to_boss")
            end
        end,
    }

    
    :AddObjective{
        id = "talk_to_boss",
        title = "(Optional) Talk to {boss}",
        desc = [[Alternately, if you can convince {boss} to give {giver} more shifts at the dig site, they'll be able to keep up with their payments.]],
        mark = { "boss" },
        
        negotiation_targets = {"boss"},

        on_complete = function(quest)
            quest:Activate("return_to_giver")
            
            if quest:IsActive("talk_to_lender") then
                quest:Cancel("talk_to_lender")
            end
        end,
    }

    :Loc{
        DEAD_BOSS_NO_JOB = [[Tell {giver} that {boss} is dead, so they no longer have a job.]],
        TARGET_DEAD_DEBT_CLEARED = [[Tell {giver} that {target} is dead, so they no longer have a debt.]],
        DEBT_RENEGOTIATED = [[Tell {giver} that their debt has been renegotiated.]],
        MORE_SHIFTS = [[Tell {giver} that you got them more shifts.]],
    }
    :AddObjective{
        id = "return_to_giver",
        title = "Talk to {giver}",
        
        desc = "",
        desc_fn = function(quest) 
            if quest:GetCastMember("boss"):IsDead() then
                return quest:GetLocalizedStr("DEAD_BOSS_NO_JOB")
            elseif quest:GetCastMember("target"):IsDead() then
                return quest:GetLocalizedStr("TARGET_DEAD_DEBT_CLEARED")
            elseif quest:IsComplete("talk_to_lender") then
                return quest:GetLocalizedStr("DEBT_RENEGOTIATED")
            elseif quest:IsComplete("talk_to_boss") then
                return quest:GetLocalizedStr("MORE_SHIFTS")
            end
        end, 
        ready_for_completion = true,
        mark = { "giver" },
    }

    :AddOpinionEvents{
        killing_in_the_name_of =  
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Killed someone in their name",
        },
    }


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("talk_to_boss", "boss")
    :Loc{
        OPT_NEGOTIATE = "Ask {boss} to give {giver} more work",
        DIALOG_NEGOTIATE = [[
            player:
                !interest
                $neutralGreeting
                You're {giver}'s boss, right? What would you say to giving {giver.himher} a bit more work?
            agent:
                !angry
                $neutralDubious
                That lazy sod? Why in Hesh's bubbly name would I do that?
        ]],
        DIALOG_NEGOTIATE_WIN = [[
            player:
                !permit
                $miscPersuasive
                {giver}'s desperate. And I hear desperation works like a charm for productivity.
                !eureka
                Don't you want employees who are too afraid to talk back or complain?
            agent:
                !thought
                $neutralResigned
                Huh. That <i>would</i> be nice. 
                We got an extra shipment that needs taking care of but most won't work after dark. Say it's "too dangerous" or some such. 
                !agree
                Sure. Tell {giver} to go get started. {giver.HeShe} can take triple shifts until we meet quota.
            player:
                !happy
                Excellent. I'm, er, sure {giver.heshe}'ll be very grateful.
        ]],
        DIALOG_NEGOTIATE_LOSE = [[
            player:
                !shrug
                {giver}'s got debts to pay. If {giver.heshe} can't pay them back, then you'll be short one worker completely.
            agent:
                !cruel
                $angryPatienceLost
                You kiddin'? That's the best news I've heard all week. 
                !handwring
                $angryCruel
                {giver}'s my worst worker. With {giver.himher} gone I'll be able to requisition another. 
        ]],
        OPT_BEAT_UP = "Beat up {agent}",
        DIALOG_BEAT = [[
            player:
                !angry
                $angryCruel
                How about this: you give {giver} more work, or I put <i>you</i> out of commission.
            agent:
                !angry
                $neutralDubious
                And how are you going to do that?
            player:
                !fight
                $angryHostile
                With violence, obviously. Figured that was clear. 
        ]],
        DIALOG_BEATEN = [[
            agent:
                !injured
                $scaredFearful
                Cut it out, grifter, you think I can stand up to you?!
                Fine. {giver} gets the work. All the work. I hope {giver.heshe} chokes on it. 
            player:
                !angry_accuse
                Next time, cooperate <i>before</i> the fists come out. 
        ]],
        DIALOG_DEAD = [[
            * {agent} is dead. Somehow you don't think that'll help {giver}'s situation much.
        ]],

    }
    
    :Hub(function(cxt) 
            local helpers = {}
            if cxt.quest:GetCastMember("giver"):GetLocation() == cxt.location then
                table.insert(helpers, cxt.quest:GetCastMember("giver"))
            end
            cxt:Opt("OPT_NEGOTIATE")
                    :Dialog("DIALOG_NEGOTIATE")
                    :Negotiation{
                        helpers = helpers,
                        on_success = function(cxt) 
                            cxt.quest.param.convinced_boss = true
                            cxt:Dialog("DIALOG_NEGOTIATE_WIN")
                            cxt.quest:Complete("talk_to_boss")
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_NEGOTIATE_LOSE")
                        end,

                    }
            
            cxt:Opt("OPT_BEAT_UP")
                :Dialog("DIALOG_BEAT")
                :Battle{
                    noncombatants = {cxt.quest:GetCastMember("giver")},
                    on_win = function( cxt )
                        if cxt:GetAgent():IsDead() then
                            cxt.quest.param.killed_boss = true
                            cxt:Dialog("DIALOG_DEAD")
                        else
                            cxt.quest.param.beat_boss = true
                            cxt:Dialog("DIALOG_BEATEN")
                            cxt.quest:Complete("talk_to_boss")
                        end          
                        StateGraphUtil.AddEndOption(cxt)              
                    end
                }
        end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


QDEF:AddConvo("talk_to_lender", "target")
    :Loc{
        OPT_NEGOTIATE = "Ask {target} to give {giver} an extension",
        DIALOG_NEGOTIATE = [[
            player:
                !interest
                $neutralGreeting
                I hear you're an up-and-coming debt-broker. You lent money to {giver}, right?
            agent:
                !crossed
                $angryPatienceLost
                Maybe I did. Maybe I am. What's it to you, hunter?
            player:
                !happy
                $miscPersuasive
                Just wondering what your enforcement schedule is like. Maybe you could tolerate a late payment?
            agent:
                !spit
                $miscMocking
                Why, because I'm a sucker?
        ]],
        DIALOG_NEGOTIATE_WIN = [[
            player:
                !permit
                $miscPersuasive
                You're just getting started. You think Kashio became the crimelord she is overnight? 
                Even she pretended to have a soft side once. That's how she ended up with so many contracts in the first place.
                !overthere
                $miscAlluring
                "Borrow from Kashio," they said. "Her rates are low and she doesn't mind if you make a late payment."
            agent:
                !thought
                $neutralResigned
                Huh. I <i>am</i> having trouble finding more, err, "customers".
                !agree
                $neutralThanks
                All right. You tell {giver} that {giver.heshe} can have another ten days before {giver.hisher} first payment. Just make sure {giver.heshe} spreads the word.
            player:
                !happy
                $happyThanks
                Smart thinking. I'll be sure to pass that along.
        ]],
        DIALOG_NEGOTIATE_LOSE = [[
            player:
                !shrug
                $neutralTakenAback
                You don't wanna get a reputation for being <i>too mean</i>, do you?
            agent:
                !dubious
                ...
                !angry
                $angryDefault
                <i>Yes</i>. That's how people flippin' pay me back!
                Now if you're done wastin' my time, you've reminded me that I need to check in with {giver} about {giver.hisher} payment. 
            player:
                !surprised
                $scaredStammering
                Ah. Are you sure?
            agent:
                $angryDefault
                Yes.
        ]],
        OPT_BEAT_UP = "Beat up {agent} to send a message",
        DIALOG_BEAT = [[
            player:
                !angry_threat
                $angryDefault
                You're a crook, just like all the other debt-brokers. And {giver} is sick of it.
            agent:
                !angry_shrug
                $angryDefault
                Oh really? And {giver.heshe} sent you to beg for lenience, huh?
            player:
                $angryCruel
                Beg, beat.
                !fight
                The methods were left up to me.
        ]],
        DIALOG_BEATEN = [[
            agent:
                !injured
                $angryDefault
                All right, all right! Stop! Hesh, this line of work ain't for me.
                $angryReluctant
                I'll give {giver} more time. I'll even get out of the debt-brokerin' business completely. Just don't kill me!
            player:
                !cruel
                $miscMocking
                I'm glad I could broaden your horizons. 
                !angry_threaten
                $angryCruel
                Don't you forget this revelation of yours, or I'll be back. 
        ]],
        DIALOG_DEAD = [[
            * {agent} is dead, and {giver}'s deadline is extended indefinitely.
        ]],

    }
    :Hub(function(cxt) 
            cxt:Opt("OPT_NEGOTIATE")
                    :Dialog("DIALOG_NEGOTIATE")
                    :Negotiation{
                        on_success = function(cxt) 
                            cxt.quest.param.convinced_lender = true
                            cxt:Dialog("DIALOG_NEGOTIATE_WIN")
                            cxt.quest:Complete("talk_to_lender")
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_NEGOTIATE_LOSE")
                        end,

                    }
            
            cxt:Opt("OPT_BEAT_UP")
                :Dialog("DIALOG_BEAT")
                :Battle{
                    noncombatants = {cxt.quest:GetCastMember("giver")},
                    on_win = function( cxt )
                        
                        if cxt:GetAgent():IsDead() then
                            cxt.quest.param.killed_lender = true
                            cxt:Dialog("DIALOG_DEAD")
                        else
                            cxt.quest.param.beat_lender = true
                            cxt:Dialog("DIALOG_BEATEN")
                            cxt.quest:Complete("talk_to_lender")
                        end

                        StateGraphUtil.AddEndOption(cxt)              
                    end
                }
        end)


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


QDEF:AddConvo("return_to_giver", "giver")
    :Loc{
        OPT_BOSS_IS_DEAD = "Tell {giver} that {boss} is dead",
        DIALOG_BOSS_DEAD = [[
            player:
                $miscPersuasive
                So that job of yours. Do you actually like it?
            agent:
                $neutralTakenAback
                Like it? You can <i>like</i> it? 
                I pull stuff outta the ground. I'm lucky if I don't lose an arm doing it. Then I hand it over. 
            player:
                $neutralDubious
                Well, the good news is I killed your boss. So... maybe you'll be in line for a promotion? 
            agent:
                !facepalm
                Or maybe they'll just shut down the whole dig until they can get someone else on it!
            player:
                $miscEmbarrassed
                Hey, I paved the way for your success! I'm not culpable for your lack of ambition. 
            agent:
                You should go.
            player:
                Yeah. Probably.
                !exit

        ]],
        
        OPT_LENDER_IS_DEAD = "Tell {giver} that {target} is dead",
        DIALOG_LENDER_DEAD = [[
            player:
                !greeting
                $happyGreeting
                Good news! You don't have to pay off your debt.
            agent:
                !interest
                $neutralTakenAback
                What, at <i>all</i>? How'd you manage that?
            player:
                !thumb
                $happyCocky
                I killed {target}. Or, if you prefer, I held out my knife and {target.heshe} ran into it an unsafe number of times.
            agent:
                !surprised
                $scaredStammering
                You WHAT?! You weren't supposed to KILL anybody!
            player:
                $neutralWhatever
                Hire a bounty hunter, get bountiful results, I say. 
                !dubious
                $neutralDubious
                Anyway, you're welcome.
            agent:
                !facepalm

        ]],
        
        OPT_MORE_SHIFTS = "Tell {giver} you got {giver.himher} more shifts",
        DIALOG_MORE_SHIFTS = [[
            player:
                !happy
                $happyGreeting
                Hey, I solved your problem! {boss} has agreed to give you more shifts. 
                Not only will you be able to make your payments, but you won't have <i>time</i> to rack up any more debts!
            agent:
                !scared
                $neutralResigned
                Ah... that's great, but for some reason I don't... feel happy.
            player:
                !shrug
                $neutralAside
                Can't imagine why not. 
            agent:
                !neutral
                !agree
                $happyDefault
                Well, thanks all the same. I'd better report to {boss} before {boss.heshe} gives my shifts away, though. 
        ]],

        OPT_BETTER_TERMS = "Tell {giver} you got {giver.himher} better terms",
        DIALOG_BETTER_TERMS = [[
            player:
                !greeting
                $happyGreeting
                I talked to {target}. {target.HeShe} has agreed to give you more time between payments.
                Should give you some room to breathe now, right?
            agent:
                !interest
                $happyThanks
                Holy Hesh, no kidding. How'd you manage that?
            player:
                !happy
                $miscPersuasive
                I'm very persuasive! So, uh... about my fee?
            agent:
                !bashful
                $miscEmbarrassed
                Oh! Right, err. It's not much—I gotta shave the shills, you know? But I hope it'll do.
        ]],

   }
    :Hub(function(cxt) 
            if cxt.quest:GetCastMember("boss"):IsDead() then
                cxt:Opt("OPT_BOSS_IS_DEAD")    
                    :Dialog("DIALOG_BOSS_DEAD")
                    :Fn(function(cxt) 
                        cxt.quest:OpinionEvent("giver", "killing_in_the_name_of")
                        cxt.quest:Fail() end)
                    :DoneConvo()
            elseif cxt.quest:GetCastMember("target"):IsDead() then
                cxt:Opt("OPT_LENDER_IS_DEAD")    
                    :Dialog("DIALOG_LENDER_DEAD")
                    :Fn(function(cxt) 
                            cxt.quest:OpinionEvent("giver", "killing_in_the_name_of")
                            cxt.quest:Complete() 
                            ConvoUtil.GiveQuestRewards(cxt)
                        end)
                    :DoneConvo()
            elseif cxt.quest:IsComplete("talk_to_boss") then
                cxt:Opt("OPT_MORE_SHIFTS")    
                    :Dialog("DIALOG_MORE_SHIFTS")
                    :Fn(function(cxt) cxt.quest:Complete() ConvoUtil.GiveQuestRewards(cxt) end)
                    :DoneConvo()
            elseif cxt.quest:IsComplete("talk_to_lender") then
                cxt:Opt("OPT_BETTER_TERMS")    
                    :Dialog("DIALOG_BETTER_TERMS")
                    :Fn(function(cxt) cxt.quest:Complete() ConvoUtil.GiveQuestRewards(cxt) end)
                    :DoneConvo()
            end

        end)


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            player:
                !Dubious 
                $neutralDubious
                You all right, there? You look a little stressed.
            agent:
                !scared
                $scaredGreeting
                Do I? It's probably just 'cause I'm gonna be dead in a few hours.
            player:
                !interest
                $neutralDubious
                Uhh... really?
            agent:
                !scared_shrug
                $scaredDefault
                Maybe. I dunno. Probably? I borrowed money from {target}, but the boss hasn't given me enough shifts to pay it back!
                Now I gotta convince {target} to give me more time... assuming {target.heshe} doesn't kill me, first.
                $scaredDefault
                Unless... <i>you</i> could help me out? Whaddya think?
        ]],
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "talk_to_lender", "DIALOG_OFFER")
    end)


