
local LOCATION_DEF =
{
    id = "ASSET_RECOVERY_LOCATION",
    name = "Buried Goods",
    desc = "A clearing in the woods",
    map_tags = {"forest"},
    plax = "EXT_Forest_1",
}

if not Content.GetLocationContent(LOCATION_DEF.id) then
    Content.AddLocationContent(LOCATION_DEF)
end

AddOpinionEvent("ASSET_RECOVERY_DISCOVERED_THEIR_THEFT", 
{
    delta = OPINION_DELTAS.MAJOR_BAD,
    txt = "Uncovered their theft",
})


local QDEF = QuestDef.Define{
        title = "Asset Recovery",
        qtype = QTYPE.SIDE,
        rank = {1, 4},
        focus = QUEST_FOCUS.COMBAT,

        desc = "{giver} is trying to track down some stolen goods, and needs you to track down some leads.",
        icon = engine.asset.Texture("icons/quests/side_asset_recovery.tex"),

        on_destroy = function( quest )
            if quest:GetCastMember("the_woods") then
                TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("the_woods"))
            end
        end,

        avoids = 
        {
            {"giver", "clue_1"},
            {"giver", "clue_2"},
            {"clue_2", "clue_1"},
        },
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = QuestUtil.CanDoRichCivvieQuest,
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "WEALTHY_MERCHANT" ) )
        end,
    }

    :AddCast{
        cast_id = "the_woods",
        when = QWHEN.MANUAL,
        no_validation = true,
    }

    :AddCast{
        cast_id = "clue_1",
        condition = function(agent, quest)
            if QuestUtil.FilterAgentForDifficulty(quest:GetRank() - 1, agent, true, true) then

                return not AgentUtil.IsFriend(agent, quest:GetCastAgent("giver")) and not agent:GetFaction():IsLawful() and not AgentUtil.IsCoworker(agent, quest:GetCastMember("giver"))
            end
        end,

        events = {
            agent_retired = function( quest, agent )
                if quest:IsActive("investigate_1") then
                    quest:Fail("investigate_1")
                end

                if not quest:IsActive("investigate_2") then
                    if quest:IsActive("find_the_goods") then
                        quest:Fail("find_the_goods")
                    end
                    quest:Activate("return_to_explain")
                end

            end,
        }
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"BANDIT_GOON", "BANDIT_GOON2", "JAKES_RUNNER", "JAKES_SMUGGLER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }

    :AddCast{
        cast_id = "clue_2",
        condition = function(agent, quest)
            if QuestUtil.FilterAgentForDifficulty(quest:GetRank() - 1, agent, true, true) then
                return not AgentUtil.IsFriend(agent, quest:GetCastAgent("giver")) and not agent:GetFaction():IsLawful() and not AgentUtil.IsCoworker(agent, quest:GetCastMember("giver"))
            end
        end,

        events = {
            agent_retired = function( quest, agent )
                if quest:IsActive("investigate_2") then
                    quest:Fail("investigate_2")
                end

                if not quest:IsActive("investigate_1") then
                    if quest:IsActive("find_the_goods") then
                        quest:Fail("find_the_goods")
                    end
                    quest:Activate("return_to_explain")
                end

            end,
        }
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"BANDIT_GOON", "BANDIT_GOON2", "JAKES_RUNNER", "JAKES_SMUGGLER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }


    :AddObjective{
        id = "find_the_goods",
        title = "Find the stolen goods",
        desc = "Follow up on {giver}'s leads to find the location of the stolen goods.",
        on_activate = function(quest) 

            quest.param.guilty_agent = math.random() > .5 and quest:GetCastAgent("clue_1") or quest:GetCastAgent("clue_2")
        end,
        --mark = { "clue_1"},
    }
    :AddObjective{
        id = "investigate_1",
        title = "Confront {clue_1}",
        desc = "See if {clue_1} knows anything about the stolen goods.",
        mark = { "clue_1"},
        negotiation_targets = {"clue_1"},
        combat_targets = {"clue_1"},
    }

    :AddObjective{
        id = "investigate_2",
        title = "Confront {clue_2}",
        desc = "See if {clue_2} knows anything about the stolen goods.",
        mark = { "clue_2"},
        negotiation_targets = {"clue_2"},
        combat_targets = {"clue_2"},
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Tell {giver} about the goods",
        desc = "You found the goods. Go tell {giver} where they are, so {giver.heshe} can send a recovery team",
        ready_for_completion = true,
        mark = {"giver"},
    }

    :AddObjective{
        id = "return_to_explain",
        title = "Explain the situation",
        desc = "You killed your last lead. Explain the situation to {giver}",
        mark = {"giver"},
    }

    :AddObjective{
        id = "go_to_woods",
        title = "Investigate the burial site",
        desc = "{guilty_agent} buried the goods in the woods. Go see if they are still there.",
        on_activate = function( quest)

            if quest:IsActive("investigate_1") then
                quest:Cancel("investigate_1")
            end
            quest:UnassignCastMember( "clue_1" )

            if quest:IsActive("investigate_2") then
                quest:Cancel("investigate_2")
            end
            quest:UnassignCastMember( "clue_2" )

            local location = Location( LOCATION_DEF.id )
            assert(location)
            TheGame:GetGameState():AddLocation(location)
            quest:AssignCastMember("the_woods", location )
        end,
        mark = {"the_woods"},
    }


QDEF:AddConvo("return_for_reward", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_GET_PAID = "Tell {giver} where the goods are",
        DIALOG_INTRO = [[
            player:
                $neutralDefault
                The goods are buried in the woods. {guilty_agent} took them.
            agent:
                $happyThanks
                Good work. I'll send my people to dig them up.
                I have to be more careful about who I hire in the future.
        ]]
    }
    
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_INTRO")
            :CompleteQuest()
            :DoneConvo()
    end)

QDEF:AddConvo("return_to_explain", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_INTRO = [[

        ]],
        OPT_ASK_FOR_PAYMENT = "Convince {agent} to pay you",
        DIALOG_ASK_FOR_PAYMENT = [[
            player:
                $neutralGreeting
                So here's the situation...
        ]], 
        DIALOG_FAIL = [[
            agent:
                $angrySeething
                They're both dead?! You're useless!
        ]],
        DIALOG_SUCCES = [[
            agent:
                $neutralResigned
                Oh, I see. Well I guess you did your best.
        ]],
        OPT_ADMIT_FAILURE = "Admit your failure",
        DIALOG_ADMIT_FAILURE = [[
            player:
                $miscPersuasive
                Look, things got out of hand and they're both dead.
            agent:
                $angrySeething
                Dead?! You're useless!
        ]],
    }
    
    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_FOR_PAYMENT")
            :Dialog("DIALOG_ASK_FOR_PAYMENT")
            :Negotiation{
                on_success = function(cxt)
                    cxt:Dialog("DIALOG_SUCCES")
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                    StateGraphUtil.AddEndOption(cxt)
                end,
                on_fail = function(cxt)
                    cxt:Dialog("DIALOG_FAIL")                        
                    cxt.quest:Fail()
                    cxt:GetAgent():OpinionEvent(OPINION.JOB_FAILED_MINOR)
                    StateGraphUtil.AddEndOption(cxt)
                end 
            }

        cxt:Opt("OPT_ADMIT_FAILURE")
            :Dialog("DIALOG_ADMIT_FAILURE")
            :Fn(function( cxt )
                cxt.quest:Fail()
                cxt:GetAgent():OpinionEvent(OPINION.JOB_FAILED_MINOR)
                StateGraphUtil.AddEndOption(cxt)
            end)
            
    end)


QDEF:AddConvo("go_to_woods", nil)
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("the_woods") then
                return "STATE_WOODS"
            end
        end)
    

    :State("STATE_WOODS")
        :Loc{
            DIALOG_INTRO = [[
                * The clearing is deserted. You look around a bit, and find evidence of digging.
                * Brushing aside the top layers of dirt, you find the top of a wooden shipping crate with {giver}'s mark stamped on it.
                player:
                    !left
                    $happyAmazed
                    Paydirt!
            ]],
            DIALOG_LEAVE = [[
                * The crate is too big to carry, so you cover it up again.
                * You should go tell {giver} that you've confirmed the location.
            ]]

        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("find_the_goods")
                cxt.quest:Complete("go_to_woods")

                if not cxt.quest.param.guilty_agent:IsDead() then

                    cxt.enc:SetPrimaryCast(cxt.quest.param.guilty_agent)
                    cxt.quest.param.guilty_agent:MoveToLocation(cxt.location)
                    if cxt.quest:GetRank() > 1 then
                        cxt.quest.param.goons = CreateCombatBackup(cxt.quest.param.guilty_agent, "MERCENARY_BACKUP", (cxt.quest:GetRank()-1))
                    end
                    cxt:GoTo("STATE_JUMPED")
                else
                    cxt:GoTo("STATE_LEAVE")    
                end
            end)
    :State("STATE_JUMPED")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    * You are interrupted by {guilty_agent}.
                    !right
                    !cruel
                    $angrySeething
                    Find what you were looking for?
                    * You try to think of a witty reply.

            ]],
            DIALOG_FIGHT_START = [[
                player:
                    $miscMocking
                    Uh, yes, actually?
                agent:
                    $angrySeething
                    You should have kept your nose out of my business!
                    Now I'm going to kill you and bury you, too!
            ]],
            DIALOG_WON = [[
                agent:
                    !exit
                * You've defended the dig site.
            ]],
            OPT_DEFEND_YOURSELF = "Defend yourself",

        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_DEFEND_YOURSELF")
                    :Dialog( "DIALOG_FIGHT_START" )
                    :Battle
                    {
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.ISOLATED,
                        
                        on_runaway = function(cxt) 
                            cxt.quest.param.ran_away = true
                            cxt.quest:Activate("return_for_reward")
                        end,
                        on_win = function(cxt) 
                            cxt:Dialog("DIALOG_WON")
                            cxt:GoTo("STATE_LEAVE")
                        end,
                    }
            end)


    :State("STATE_LEAVE")
        :Loc{
            DIALOG_LEAVE = [[
                * The crate is too big to carry, so you cover it up again.
                * You should go tell {giver} that you've confirmed the location.
            ]]
        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_LEAVE")
                cxt.quest:Activate("return_for_reward")
                StateGraphUtil.AddLeaveLocation(cxt)
            end)


local on_success_fn = function(cxt, obj_id) 
    cxt.quest:Complete(obj_id)

    if cxt:GetAgent() == cxt.quest.param.guilty_agent then
        cxt:Dialog("DIALOG_SUCCESS_GUILTY")
        cxt:GetAgent():OpinionEvent(OPINION.ASSET_RECOVERY_DISCOVERED_THEIR_THEFT)
        cxt.quest:Activate("go_to_woods")
        StateGraphUtil.AddEndOption(cxt)
    else
        cxt:Dialog("DIALOG_SUCCESS_INNOCENT")
        if cxt.quest.param.guilty_agent:IsDead() then
            cxt:Dialog("DIALOG_FAIL_KILLED_GUILTY")
            cxt.quest:Fail("find_the_goods")
            cxt.quest:Activate("return_to_explain")
        end
        StateGraphUtil.AddEndOption(cxt)
    end
end


QDEF:AddConvo()
    :Priority(CONVO_PRIORITY_HIGHEST)
    :RequireFlags(CONVERSATION_FLAGS.BUSINESS)
    :Loc{
        OPT_ASK_ABOUT_THEFT = "Press {agent} for information about the theft",
        DIALOG_INTRO_1 = [[
            player:
                $miscPersuasive
                You were working security for {giver} when the goods got stolen.
                I'm no detective, but that seems mighty suspicious.
            agent:
                $neutralDubious
                I don't know what you mean...
        ]],
        DIALOG_INTRO_2 = [[
            player:
                $neutralDubious
                So, you were working for {giver} when {giver.hisher} goods got stolen.
                That's pretty suspicious...
            agent:
                $angryTakenAback
                What are you insinuating?
        ]],
        
        DIALOG_SUCCESS_GUILTY = [[
            * {agent} glances around furtively.
            agent:
                $miscHushedUrgent
                Shhhh, keep your voice down!
            * {agent} plasters on a fake smile. It's not very convincing.
                !happy
                $happyDefault
                I took the goods, yeah.
                But they're worthless! Bunch of weird stuff from the Roaloch. Nobody would buy any of it!
                I buried the lot out in the woods. 
            * {agent} marks a point on your map.
                $neutralResigned
                I just want to walk away from this one, right? 
        ]],
        
        DIALOG_SUCCESS_INNOCENT = [[
            agent:
                $angryTakenAback
                It wasn't me, I swear!
                {guilty_agent#agent} did it, and threatened to kill me if I talked.
                Please, just let me go.
            * You're pretty sure {agent.heshe} is telling the truth.
        ]],
        
        DIALOG_FAIL = [[
            agent:
                $angryHostile
                I'm not saying nothing about that!
            * If you want more information, you're going to have to beat it out of {agent.himher}.
        ]],

        OPT_BEAT_INFO_OUT = "Beat the info out of {agent}",
        DIALOG_BEAT_INFO_OUT = [[
            player:
                $miscPersuasive
                I have a proposition for you.
            agent:
                $neutralDubious
                I'm listening...
            player:
                !fight
                $angryGreeting
                You're going to tell me what I want to know, or you're going to die.
        ]],
        DIALOG_FAIL_KILLED_GUILTY = [[
            * Looks like you killed the only person who knew the truth.
            player:
                !facepalm.
            * Good job, {player}. Real solid detective work.
        ]],
        DIALOG_DEAD = [[
            * {agent} is dead. There's still one more person who might know the truth.
        ]],
    }
    :Hub(function(cxt, who)

        local obj_id, other_obj_id
        if who == cxt.quest:GetCastAgent("clue_1") then
            obj_id, other_obj_id = "investigate_1", "investigate_2"
        elseif who == cxt.quest:GetCastAgent("clue_2") then
            obj_id, other_obj_id = "investigate_2", "investigate_1"
        else
            return
        end

        if not cxt.quest:IsActive( obj_id ) then
            return
        end

        cxt:Opt("OPT_ASK_ABOUT_THEFT")
            :Dialog(cxt:GetAgent() == cxt.quest:GetCastAgent("clue_1") and "DIALOG_INTRO_1" or "DIALOG_INTRO_2")
            :Negotiation{
                on_success = function() on_success_fn(cxt, obj_id) end,
                on_fail = function()
                    cxt:Dialog("DIALOG_FAIL")                        
                    cxt:Pop()
                end 
            }
        cxt:Opt("OPT_BEAT_INFO_OUT")
            :Dialog("DIALOG_BEAT_INFO_OUT")
            :Battle{
                on_win = function(cxt) 

                    if cxt:GetAgent():IsDead() then
                        cxt.quest:Fail(obj_id)

                        if not cxt.quest:IsActive(other_obj_id) then
                            cxt:Dialog("DIALOG_FAIL_KILLED_GUILTY")
                            cxt.quest:Fail("find_the_goods")
                            cxt.quest:Activate("return_to_explain")
                        else
                            cxt:Dialog("DIALOG_DEAD")
                        end

                        StateGraphUtil.AddEndOption(cxt)
                    else
                        on_success_fn(cxt, obj_id)
                    end
                end,
            }


    end)    

QDEF:AddConvo(nil, "giver")
    :Loc{
        DIALOG_OFFER = [[
            agent:
                $neutralDirect
                I'm looking for a thief.
            player:
                $happyCocky
                Great. What do you want stolen?
            agent:
                !dubious
                $neutralDubious
                No, I mean, I'm looking for someone who has stolen from <i>me</>.
            player:
                $neutralResigned
                Oh, right. Of course.
            agent:
                $neutralDirect
                Some very valuable goods that were part of a highly complex three-way speculative deal went missing from my warehouse.
                I can salvage the deal if I get them back soon, but I don't know exactly who took them.
                I'll give you a cut if you can locate them.
        ]],

        DIALOG_FOLLOWUP = [[
            agent:
                $neutralDirect
                Some ruffians that I hired to run security are the most likely culprits. 
                !thought
                $neutralThoughtful
                {clue_1}, and {clue_2}, I think their names were.
                Start by asking them some questions. I'm sure one of them will know something.
        ]]
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "find_the_goods", "DIALOG_OFFER", function(cxt) 
                    cxt:Dialog("DIALOG_FOLLOWUP") 
                    cxt.quest:Activate("investigate_1")
                    cxt.quest:Activate("investigate_2")
                end)
    end)


