local LOCATION_DEF =
{
    id = "CROSSROADS_BANDIT_AMBUSH_POINT",
    name = "Crossroads Ambush Spot",
    desc = "The junction of two dangerous backroads deep in the wilderness around Murder Bay",
    icon = engine.asset.Texture("icons/quests/at_the_crossroad.tex"),
    map_tags = {"road"},
    plax = "EXT_Road01",
    indoors = false,
}

if not Content.GetLocationContent(LOCATION_DEF.id) then
    Content.AddLocationContent(LOCATION_DEF)
end

local function LimboOpFor(cxt)
    for k, v in pairs(cxt.enc.scratch.opfor) do
        if not v:IsDead() then
            v:MoveToLimbo()
        end
    end
end


local QDEF = QuestDef.Define{
        title = "Meet me at the Crossroads",
        qtype = QTYPE.SIDE,
        rank = {2, 5},
        focus = QUEST_FOCUS.COMBAT,

        desc = [[Join {giver} at an ambush point and help choose robbery targets. You'll get a cut of the loot, if you both survive.]],
        icon = engine.asset.Texture("icons/quests/at_the_crossroad.tex"),
        reward_mod = 0,
        on_destroy = function( quest )
            if quest:GetCastMember("crossroads") then
                TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("crossroads"))
            end
        end,
        on_init = function(quest)
            quest:SetParameter("player_share", 0)
        end,
    }
   
   :AddCast{
        cast_id = "giver",
        provider = true,
        condition = QuestUtil.CanDoBanditGoonQuest,
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"BANDIT_GOON", "BANDIT_GOON", "BANDIT_GOON2", "BANDIT_GOON2", "BANDIT_RAIDER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }


    :AddLocationCast{
        cast_id = "crossroads",
        when = QWHEN.MANUAL,
        no_validation = true,
    }

    :AddObjective{
        id = "go_to_crossroads",
        title = "Go to {crossroads#location}",
        desc = "Meet {giver} at {crossroads#location}, so you can start robbing people.",
        mark = { "crossroads"},
        
        on_activate = function( quest)
            local location = Location( LOCATION_DEF.id )
            assert(location)
            TheGame:GetGameState():AddLocation(location)
            quest:AssignCastMember("crossroads", location )
        end,
    }

    :AddObjective{
        id = "rob_some_things",
        title = "Help {giver} rob passersby",
        desc = [[This is a busy road. Your job is to evaluate each target, and pick one to steal from.]],
        mark = { "giver" },
        terminal = true,
        on_activate = function(quest)
        end,
    }


local function CreateAttackState(loot_modifier, party_generator)
    return function (cxt) 
        cxt.enc.scratch.opfor = CreateCombatParty(party_generator, cxt.quest:GetRank(), cxt.location)
        cxt.enc:SetPrimaryCast(cxt.enc.scratch.opfor[1])
        local giver = cxt.quest:GetCastMember("giver")

        cxt:Dialog("DIALOG_INTRO")
        
        cxt:Opt("OPT_LET_PASS")
            :Dialog("DIALOG_PASS")
            :Fn(function() LimboOpFor(cxt) end)
            :Pop()


        cxt:Opt("OPT_STEP_OUT")
            :Dialog("DIALOG_ACCOST")
            :Battle{                
                allies = {giver},
                enemies = cxt.enc.scratch.opfor,

                on_win = function(cxt) 

                    cxt.quest:SetParameter("player_share", cxt.quest:GetParameter("player_share") + math.floor( 40*cxt.quest:GetRank()*loot_modifier) )

                    if giver:IsDead() then
                        cxt:Dialog("DIALOG_GIVER_DEAD", cxt.quest:GetParameter("player_share") * 2)
                        cxt.caravan:AddMoney(cxt.quest:GetParameter("player_share") * 2)
                        StateGraphUtil.AddLeaveLocation(cxt)
                    else
                        LimboOpFor(cxt)

                        cxt.enc:SetPrimaryCast(giver)

                        cxt.enc.scratch.admiralty_chance = cxt.enc.scratch.admiralty_chance + .2
                        
                        cxt:Dialog("DIALOG_ON_WON")
                        
                        cxt:Opt("OPT_DIVVY")
                            :Dialog("DIALOG_DIVVY")
                            :Fn(function(cxt)
                                    cxt.caravan:AddMoney(cxt.quest:GetParameter("player_share"))
                                    cxt.quest:Complete()
                                    ConvoUtil.GiveQuestRewards(cxt)
                                    giver:MoveToLimbo()
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)
                        
                        cxt:Opt("OPT_HIDE")
                            :Dialog("DIALOG_HIDE")
                            :Pop()
                    end
                end,
            }
    end
end

QDEF:AddConvo("go_to_crossroads")
    :Quips{
        {
            tags = "accosted_on_road",
            "What do you want?"
        },
        {
            tags = "accosted_on_road, merchant",
            "Who- who's there?"
        },
        {
            tags = "accosted_on_road, cult_of_hesh",
            "Out of the way. We are attending to Hesh's business.",
        },
        {
            tags = "accosted_on_road, laborer",
            "What's this, then?",
        },
        {
            tags = "accosted_on_road, jakes",
            "We don't have any kindness for strangers met on the road. Move along.",
        }

    }
    :Loc{
        OPT_START_BANDITRY = "Tell {agent} you are ready to start",
        DIALOG_GIVER_DEAD = [[
            agent:
                !exit
            * {giver} is dead. You take twice your share, totalling {1#money}. Maybe crime does pay, afterall.
        ]],
        OPT_STEP_OUT = "Step out of the bush",
        OPT_LET_PASS = "Let them pass",
        DIALOG_PASS = [[
            agent:
                !exit
            * They pass.
        ]],
        DIALOG_ACCOST = [[
            player:
                !left
            agent:
                %accosted_on_road
            player:
                !fight
            * You signal to {giver}.
        ]],        
        DIALOG_ON_WON = [[
            * The dust settles. {agent} tallies up the loot.
            giver:
                !right
                $happyCocky
                Well, that went alright. I'd say we get about {player_share#money} each if we call it now.
                Whaddya think? Quit while we're ahead? 
                !cruel
                $angryCruel
                Or keep on trucking?

        ]],
        DIALOG_FAIL_CONVINCE = [[
            * {agent}, unconvinced, waves you away and hurries down the road. 
            * You rejoin {giver} in the bush, avoiding the disapproving sneer {giver.heshe} casts your way.
        ]],

        OPT_DIVVY = "Divvy up the loot and part ways",
        OPT_HIDE = "Hide in the bushes and wait for another mark",
        DIALOG_HIDE = [[
            player:
                $angryCruel
                I'm feelin' lucky. And greedy.
            agent:
                $happyCocky
                You and me both, friend. Let's get back to it then.
                !exit
        ]],
        DIALOG_DIVVY = [[
            player:
                $miscPersuasive
                Think I'd rather hedge my bets. I'm careful like that.
            agent:
                $neutralResigned
                I've been at this game long enough to see the wisdom in that, even if my belly disagrees.
                !give
                $happyCocky
                Here's your share. Be seeing you, grifter.
                !exit
            * {agent} skulks back into the bushes to make a low-profile exit. 
        ]]

    }
    
    :Hub(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("crossroads") and cxt.quest:GetCastMember("crossroads") == cxt.location and cxt:GetAgent() == cxt.quest:GetCastMember("giver") then
                cxt:Opt("OPT_START_BANDITRY")
                    :GoTo("STATE_START_BANDITRY")
            end
            
        end)
    
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("crossroads") then --and cxt.quest:GetCastMember("giver"):GetLocation() ~= cxt.location then
                return "STATE_BANDIT_ARRIVE"
            end
        end)
    
    :State("STATE_BANDIT_ARRIVE")
        :Loc{
            DIALOG_INTRO = [[
                * You find the crossroads that {giver} told you about.
                player:
                    !left
                giver:
                    !right
                    $happyCocky
                    Ah, there you are! You ready for some action?
            ]],

            DIALOG_LATER = [[
                giver:
                    $angryPatienceLost
                    Ugh, thought you were supposed to be a professional. Get your head in the game and come back as soon as you're ready.
            ]],
            OPT_START = "Get into position",
            OPT_LATER = "Attend to something else first",
        }
        :Fn(function(cxt) 
                local giver = cxt.quest:GetCastMember("giver")
                if giver:GetLocation() ~= cxt.location then
                    giver:MoveToLocation(cxt.location)
                end
                cxt.enc:SetPrimaryCast(giver)
                
                cxt:Dialog("DIALOG_INTRO")
                
                cxt:Opt("OPT_START")
                    :GoTo("STATE_START_BANDITRY")


                cxt:Opt("OPT_LATER")
                    :Dialog("DIALOG_LATER")
                    :Travel()


            end)

    :State("STATE_START_BANDITRY")
        :Loc{
            
            DIALOG_START = [[
                player:
                    $neutralDirect
                    What's the plan here?
                agent:
                    $neutralDirect
                    You're gonna skulk around those bushes. Stay outta sight, but keep an eye on the road.
                    When something juicy comes past, you go flag them down.
                    I'll come around from behind, and we'll get the drop on them.
                    Now, let's get to business.
                    !exit
                player:
                    !exit
                * You and {agent} get into position, squatting glamorously in the mud.
            ]],
            DIALOG_CONTINUE = [[
                player:
                    !exit
                agent:
                    !exit

                * You resume your watch.
            ]]
        }
        :SetLooping()
        :Fn(function(cxt) 
                cxt:Dialog(cxt:FirstLoop() and "DIALOG_START" or "DIALOG_CONTINUE")
                if cxt:FirstLoop() then
                    cxt.enc.scratch.target_options = {
                        "STATE_MERCHANT",
                        "STATE_JAKES",
                        "STATE_HESH",
                        "STATE_LABORERS",
                    }

                    --remove one at random
                    table.remove(cxt.enc.scratch.target_options, math.random(#cxt.enc.scratch.target_options))
                    cxt.enc.scratch.admiralty_chance = 0
                end

                if #cxt.enc.scratch.target_options == 0 or math.random() < cxt.enc.scratch.admiralty_chance then
                    cxt:GoTo("STATE_ADMIRALTY_BUST")
                else
                    local state = table.remove(cxt.enc.scratch.target_options, math.random(#cxt.enc.scratch.target_options))
                    cxt:GoTo(state)
                end
                
            end)


    :State("STATE_MERCHANT")
        :Loc{
            DIALOG_INTRO = [[
                * Before long, a merchant comes down the road.
                agent:
                    !right
            ]],
            DIALOG_PASS = [[
                agent:
                    !exit
                * You let them pass.
                giver:
                    !right
                    $miscDepressed
                    That would have been a good one.
                    !exit
            ]]
        }
        :Fn(CreateAttackState(
            1.0,
            function(level) 
                local PARTY_TABLE = 
                {
                    {"WEALTHY_MERCHANT", "COMBAT_DRONE"},
                    {"WEALTHY_MERCHANT", "JAKES_RUNNER", "COMBAT_DRONE"},
                    {"WEALTHY_MERCHANT", "JAKES_SMUGGLER", "COMBAT_DRONE"},
                    {"WEALTHY_MERCHANT", "JAKES_SMUGGLER", "JAKES_RUNNER", "COMBAT_DRONE"},
                }
                return PARTY_TABLE[ math.min(#PARTY_TABLE, level) ]
            end))
 
    :State("STATE_JAKES")
        :Loc{
            DIALOG_INTRO = [[
                * Before long, you spot a small contingent of Jakes, their wagons fat with cargo.
                agent:
                    !right
            ]],        
            DIALOG_PASS = [[
                agent:
                    !exit
                * You let them pass.
                giver:
                    !right
                    $miscRelieved
                    That would have been good, but a lot of work.
                    !exit
            ]]
        }
        :Fn(CreateAttackState( 
            1.25,
            function(level) 
                local PARTY_TABLE = 
                {
                    {"JAKES_RUNNER"},
                    {"JAKES_RUNNER", "JAKES_RUNNER"},
                    {"JAKES_SMUGGLER", "JAKES_RUNNER"},
                    {"JAKES_SMUGGLER", "JAKES_RUNNER", "JAKES_RUNNER"},
                    {"JAKES_SMUGGLER", "JAKES_SMUGGLER", "JAKES_RUNNER", "JAKES_RUNNER"},
                }
                return PARTY_TABLE[ math.min(#PARTY_TABLE, level) ]
            end))
 
    :State("STATE_HESH")
        :Loc{
            DIALOG_INTRO = [[
                * A Hesh cultist approaches, pockets laden with tithe money.
                agent:
                    !right
            ]],
            DIALOG_PASS = [[
                agent:
                    !exit
                * You let them pass.
                giver:
                    !right
                    $miscDepressed
                    I could have used that cash.
                    Not so sure about the everlasting damnation, though.
                    !exit
            ]]

        }
        :Fn(CreateAttackState( 
            1.0,
            function(level) 
                local PARTY_TABLE = 
                {
                    {"PRIEST"},
                    {"PRIEST", "PRIEST"},
                    {"PRIEST", "LUMINARI"},
                    {"PRIEST", "PRIEST", "LUMINARI"},
                    {"PRIEST", "PRIEST", "LUMINARI", "LUMINARI"},
                }
                return PARTY_TABLE[ math.min(#PARTY_TABLE, level) ]
            end))

    :State("STATE_LABORERS")
        :Loc{
            DIALOG_INTRO = [[
                * A group of laborers is walking hurriedly toward your ambush point.
                agent:
                    !right
            ]],        
            DIALOG_PASS = [[
                agent:
                    !exit
                * You let them pass.
                giver:
                    !right
                    $miscRelieved
                    Good call. Probably wouldn't have had much money, anyway.
                    !exit
            ]]

        }
        :Fn(CreateAttackState(
            .6,
            function(level) 
                local PARTY_TABLE = 
                {
                    {"LABORER", "LABORER"},
                    {"LABORER", "LABORER", "LABORER"},
                    {"HEAVY_LABORER", "HEAVY_LABORER"},
                    {"HEAVY_LABORER", "HEAVY_LABORER", "HEAVY_LABORER"},
                    {"HEAVY_LABORER", "HEAVY_LABORER", "HEAVY_LABORER", "HEAVY_LABORER"},
                }
                return PARTY_TABLE[ math.min(#PARTY_TABLE, level) ]
            end))



    :State("STATE_ADMIRALTY_BUST")
        :Loc{
            DIALOG_INTRO = [[
                * An Admiralty patrol appears on the path.
                agent:
                    !right
                    $angryPatienceLost
                    You there! Get out of the bushes!
                player:
                    !left
                agent:
                    $angryHostile
                    You're the one that's been causing trouble around here. 
                    I'm bringing you in!
            ]],        
            DIALOG_GIVER_DIED = [[
                * {giver} is dead. With nothing more to do, you take all the loot for yourself, a cool {1#money}.

            ]],
            DIALOG_GIVER_LIVED = [[
                giver:
                    !right
                    $happyThanks
                    I'm glad I brought you along!
                    We should divvy up the loot and split.
                * {giver} hands over {player_share#money} and runs off into the wilderness.
            ]],
            OPT_NEGOTIATE = "Convince {agent} you aren't who they're looking for",
            DIALOG_NEGOTIATE_SUCCESS = [[
                agent:
                    $neutralResigned
                    Hmmm. Guess we have to keep looking.
                    !exit
                * {agent} leaves.
            ]],
            DIALOG_NEGOTIATE_FAIL = [[
                agent:
                    $angryCruel
                    Nice try, Grifter!
                    I'll pass that one on when I'm telling the story of how I killed you.
            ]],
            DIALOG_LOOT = [[
                giver:
                    $happyThanks
                    Your share is {player_share#money}.
                    Be seeing you around, grifter.
                    !exit
            ]],
            DIALOG_NO_LOOT = [[
                giver:
                    $angryPatienceLost
                    We didn't earn any money at all.
                    Thanks for wasting my time.
                    !exit
            ]],
            DIALOG_CLOSE_CALL = [[
                giver:
                    !right
                    $miscRelieved
                    Hesh, that was close!
                    We can't use this ambush spot again any time soon.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }
        :Fn(function(cxt)
                
                local giver = cxt.quest:GetCastMember("giver")
                local battle_params = 
                {
                    allies = {giver},
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function(cxt) 

                        LimboOpFor(cxt)
                        if giver:IsDead() then
                            cxt:Dialog("DIALOG_GIVER_DIED", cxt.quest:GetParameter("player_share") * 2)
                            cxt.caravan:AddMoney(cxt.quest:GetParameter("player_share") * 2)
                        elseif cxt.quest:GetParameter("player_share") <= 0 then
                            cxt.enc:PresentRight( giver )
                            cxt:Dialog("DIALOG_NO_LOOT")
                            cxt.quest:Fail()
                        else
                            cxt:Dialog("DIALOG_GIVER_LIVED")
                            cxt.caravan:AddMoney(cxt.quest:GetParameter("player_share"))
                            cxt.quest:Complete()
                            ConvoUtil.GiveQuestRewards(cxt)
                            giver:MoveToLimbo()
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)

                    end,
                }

                cxt.enc.scratch.opfor = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.enc:SetPrimaryCast(cxt.enc.scratch.opfor[1])
                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_NEGOTIATE")
                    :Negotiation{
                        helpers = {giver},
                        on_success = function(cxt) 
                            cxt:Dialog("DIALOG_NEGOTIATE_SUCCESS")
                            
                            LimboOpFor(cxt)

                            cxt:Dialog("DIALOG_CLOSE_CALL")
                            if cxt.quest:GetParameter("player_share") <= 0 then
                                cxt:Dialog("DIALOG_NO_LOOT")
                                cxt.quest:Fail()
                            else
                                cxt:Dialog("DIALOG_LOOT")
                                cxt.caravan:AddMoney(cxt.quest:GetParameter("player_share"))
                                cxt.quest:Complete()
                                ConvoUtil.GiveQuestRewards(cxt)
                            end
                            giver:MoveToLimbo()
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt) 
                            cxt:Dialog("DIALOG_NEGOTIATE_FAIL")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle(battle_params)
                        end,
                    }

                cxt:Opt("OPT_FIGHT")
                    :Battle(battle_params)

            end)

QDEF:AddConvo(nil, "giver")
    :Loc{
        DIALOG_OFFER = [[
            player: 
                $neutralDirect
                You got work for me?
        * {agent} gives you a quick once-over, clearly running the numbers in {agent.hisher} mind. You must have passed some test, because {agent.heshe} leans in close.
            agent:
                !cagey
                $neutralDubious
                There's a golden ambush spot out round that new trade route the Admiralty opened.
                Only now with the auction coming up they're too preoccupied to keep it nice and safe.
                !wink
                $happyCocky
                A couple of opportunistic folks could really make a killing.
                Sound like your kinda thing?
        ]],
        DIALOG_TOOK_JOB = [[
            agent:
                !crossed
                $happyCocky
                Get your stuff in order and meet me up there.
                Don't keep me waiting! I'm not too patient.
        ]]
    }
    :Hub(function(cxt)
        StateGraphUtil.OfferSideJob(cxt, "go_to_crossroads", "DIALOG_OFFER", function(cxt) 
                cxt:Dialog("DIALOG_TOOK_JOB")
            end)
    end)

