local QDEF = QuestDef.Define{
        title = "Silence a Rival",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{giver} is offering you money to lay a beat-down on {target}. Just be careful not to take it too far.]],
        icon = engine.asset.Texture("icons/quests/attack_rival.tex"),
        focus = QUEST_FOCUS.COMBAT,
        
        avoids = 
        {
            {"giver", "target"},
        },
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent)
            return agent.social_connections ~= nil
        end,
    }
    
    :AddCast{
        cast_id = "target",
        
        cast_fn = function( quest, t )

            local giver = quest:GetCastMember("giver")
            for agent,rel in pairs(giver.social_connections.relationships) do
                if not AgentUtil.IsCoworker(agent, quest:GetCastMember("giver")) then
                    if rel < RELATIONSHIP.NEUTRAL then
                        if QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, false) then
                            table.insert(t, agent)
                        end
                    end
                end
            end
        end,
        events = 
        {
            agent_retired = function( quest, agent )
                if quest:IsActive("beat_target") then
                    quest:Complete( "beat_target" )
                    quest:Activate( "return_for_reward" )
                end
            end,
            
            aspects_changed = function( quest, agent, added, aspect )
                if added and is_instance( aspect, Aspect.Intimidated ) then
                    if quest:IsActive("beat_target") then
                        quest:Complete( "beat_target" )
                        quest:Activate( "return_for_reward" )
                    end
                end
            end
        },

        on_assign = function(quest, agent)
            if quest:GetCastMember("giver"):GetRelationship(agent) == RELATIONSHIP.NEUTRAL then
                quest:GetCastMember("giver").social_connections:SetRelationship(RELATIONSHIP.DISLIKED, agent)
            end
        end
    }
    :AddCastFallback
    {
        condition = function(agent, quest)
            if quest:GetCastMember("giver"):GetRelationship(agent) <= RELATIONSHIP.NEUTRAL and not AgentUtil.IsCoworker(agent, quest:GetCastMember("giver")) then
                if QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, false) then
                    return true
                end
            end
        end,
    }



    :AddObjective{
        id = "beat_target",
        title = [[Find {target}]],
        desc = "Beat up {target} for {giver}. Remember: no killing!",
        combat_targets = {"target"},
        mark = { "target" },
        
        on_activate = function( quest )
            --We really don't want to do this, because the party will follow the person around and cause weird distortions in location populations. It's better for it to end up too easy, rather than too hard.
            --local target = quest:GetCastAgent( "target" )
            --CreateCombatBackup( target, nil, quest:GetRank() )
        end,

        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("target") then
                fighter:AddCondition("WANTED_ALIVE")
            end
        end,


        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Get reward from {giver#agent}",
        desc = [[You dispensed with violence, it's time for {giver} to dispense with cash.]],
        ready_for_completion = true,
        mark = { "giver" },
        terminal = true,
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
    }


QDEF:AddConvo("beat_target", "target")
    :Loc{
        OPT_BEAT_UP = "Beat up {target}, like {giver} asked",
        DIALOG_BEAT_UP = [[
            player:
                !forgetful
                $neutralWhatever
                Sorry, champ. This isn't personal. But I've been hired to make your life hell.
            agent:
                $neutralTakenAback
                !surprised
                W-what? You're joking, right? Who'd want that?
            player:
                !sigh
                $neutralResigned
                Client confidentiality. Just close your eyes up real tight and I promise it'll be over quick. 
                !fight
        ]],
        DIALOG_KILLED = [[
            agent:
                !exit
            * Against {giver}'s wishes, you killed {target}. You're not even sure why. Hopefully the consequences won't be too severe.
        ]],

        DIALOG_SURRENDERED = [[
            * You pull up short before your beating takes a turn for the fatal.
            agent:
                !injured
                $scaredAnguished
                Why are you even doing this??
            player:
                !shrug
                $neutralResigned
                A job's a job. Anyway, that's all I was asked to do. 
                Want my advice? Hire protection. 
            * You leave {target} to nurse {target.hisher} wounds and confusion alike.
        ]]

    }
    :Hub(function(cxt)
            
        cxt:Opt("OPT_BEAT_UP")
            :Dialog("DIALOG_BEAT_UP")
            :Battle{
                on_win = function() 
                    if cxt:GetCastAgent("target"):IsDead() then
                        cxt:Dialog("DIALOG_KILLED")
                    else
                        cxt:Dialog("DIALOG_SURRENDERED")
                    end

                    if cxt.quest:IsActive("beat_target") then
                        cxt.quest:Complete( "beat_target" )
                        cxt.quest:Activate( "return_for_reward" )
                    end
                    
                    StateGraphUtil.AddEndOption(cxt)
                end,
            }
        
        end)

QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_TELL_KILLED = "Tell {agent} that {target} is dead",
        OPT_GET_PAID = "Get paid",
        DIALOG_KILLED = [[
            player:
                !bashful
                $miscEmbarrassed
                Well, I roughed {target} up all right, as requested. Unfortunately they, uh, were softer than I expected.
            agent:
                !dubious
                $neutralWhatever
                You killed 'em?
                !angry
                $neutralWhatever
                Normally I wouldn't mind, 'cept I'd taken a liking to the idea of stringing this on a little longer. 
                Well, considering you breached the terms of our agreement, I consider it fair that I don't pay you. 
            player:
                !palm
                $angryReluctant
                I never shoulda taken this job. 
            agent:
                !shrug
                $neutralWhatever
                Maybe. But that's your problem. 
        ]],

        DIALOG_PAID = [[
            player:
                !permit
                $neutralDirect
                You got what you wanted, and {target}'s got the scars to prove it.
            agent:
                !interest
                $happyDefault
                Hah! Really? That's all it took?
                Next time I wanna make someone's life hell, all I gotta do is pay for it, and there are no consequences?
                !give
                Shoulda done that a lot sooner. Here's your pay, hunter.
            player:
                !take
                $neutralDubious
                What did {target} do to deserve it, anyway?
            agent:
                !dubious
                $neutralTakenAback
                Huh? <i>Deserve</i> it? 
                $neutralWhatever
                Where's the fun in that? Better if they don't even know what they did wrong, ain't it?
            player:
                !dubious
                $neutralDubious
                Uh...
            agent:
                !agree
                $neutralWhatever
                Thanks for your services, {player}. 
        ]],

    }
    :Quips{
        {
            tags = "give_payment",
            "Money well spent!",
            "I wish I could have seen the look on {target.hisher} face.",
            "You told them that <b>I</> sent you, right?",
            "That'll teach {target.himher}!",
        },
    }
    :Hub(function(cxt) 
        
        local target = cxt.quest:GetCastMember("target")
        if target:IsDead() then
            
            cxt:Opt("OPT_TELL_KILLED")
                :SetQuestMark()
                :Dialog("DIALOG_KILLED")
                :Fn( function( cxt ) cxt.quest:Fail() end )
        
        else
        
            cxt:Opt("OPT_GET_PAID")
                :PreIcon( global_images.hire )
                :SetQuestMark()
                --:Quip(cxt:GetAgent(), "give_payment")
                :Dialog("DIALOG_PAID")
                :CompleteQuest()

        end

    end)


QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            player:
                !interest
                You hiring? I'm looking for work.
            agent:
                !placate
                What? No, I don't got—
                !neutral
                $neutralWhatever
                Hey, wait. You're muscle, right? You like to bring the hurt for those that ask for it?
            player:
                !shrug
                Whether or not I <i>like</i> it doesn't always factor. Sometimes it just is what it is. 
            agent:
                !thought
                $neutralWhatever
                Yeah, yeah. You make chaos—that's all I need to know.
                $neutralDirect
                I want you to rough up {target}. In exchange, I'll pay you. Sound like a deal?
                
        ]],
        DIALOG_ACCEPTED = [[
            player:
                !interest
                $neutralDubious
                Uh, sure. Normally I like to know <i>why</i> I'm roughing someone up.
            agent:
                !angry_shrug
                $angryDefault
                You need a reason? Why, you gonna play judge, jury, <i>and</i> executioner?
                That's not what I hired you for. Do the job, or don't get paid. Simple as that.
            player:
                !angry
                $neutralDubious
                Do you... want {target} killed?
            agent:
                !thought
                $neutralThoughtful
                Hmmm.
                $neutralDirect
                No. Not this time. But thanks for clarifying. 
            player:
                !dubious 
                $neutralDubious
                Yeesh. 
        ]]

    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "beat_target", "DIALOG_OFFER", function() cxt:Dialog("DIALOG_ACCEPTED") end)
    end)

