local DEST_STATE_OVERRIDE = nil -- make true or false to debug-force an option in the final bog confrontation
local TALK_SUCCESS_OVERRIDE = nil

local CleanUpFn = function(quest) 
    if quest:GetCastMember("giver"):IsInPlayerParty() then
        quest:GetCastMember("giver"):Dismiss()
    end

    if quest:IsFailed() and not quest:GetCastMember("giver"):IsDead() then
        quest:GetCastMember("giver"):OpinionEvent(OPINION.JOB_FAILED_MINOR, { quest = quest})
    end

end


local QDEF = QuestDef.Define{
        title = "Back Channel Negotiations",
        qtype = QTYPE.SIDE,
        desc = [[{giver} is looking for a business partner amongst the Spree.]],
        icon = engine.asset.Texture("icons/quests/side_back_channel_negotiation.tex"),
        --icon = engine.asset.Texture("icons/quests/special_delivery.tex"),

        on_fail = CleanUpFn,
        on_complete = CleanUpFn,
        on_cancel = CleanUpFn,
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent, quest)
            return agent:GetFactionID() == "ADMIRALTY" and agent:GetBrain():GetWorkPosition() == nil
        end,
    }
    :AddCastFallback("ADMIRALTY_CLERK")


    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_ADMIRALTY_CONTACT",
        limit = QLIMIT.NOLIMIT,
    }

    :AddCast{
        cast_id = "target",
        condition = function(agent, quest)
            if agent:GetFactionID() == "BANDITS" and agent:GetRelationship(quest:GetCastMember("giver")) >= RELATIONSHIP.NEUTRAL and agent:GetBrain():GetWorkPosition() == nil then
                return true
            end
        end,
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"BANDIT_GOON","BANDIT_GOON","BANDIT_GOON2","BANDIT_RAIDER","BANDIT_CAPTAIN"}
            local def = defs[math.min( quest:GetRank(), #defs)]
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }


    :AddLocationDefs{
        AMBUSH_SITE = {
            plax = "EXT_Forest_1",
            name = "Ambush Site",
            map_tags = {"forest"},
        },
    }

    :AddLocationCast{
        cast_id = "ambush_site",
        when = QWHEN.MANUAL,
        cast_fn = function(quest, t)
            table.insert(t,  quest:SpawnTempLocation("AMBUSH_SITE"))
        end,
    }

    :AddObjective{
        id = "talk_to_target",
        title = "Talk to {target}",
        desc = [[{giver} has a business proposition for {target}. Convince {target} that it's in their best interest to agree.]],
        mark = { "target" },
        
        negotiation_targets = {"target"},
        on_activate = function(quest)
            if Util.IsPlayerWorkingWithFaction("ADMIRALTY") then
                quest:Activate("rat_out_giver")
            end
        end,

        on_fail = function(quest)
            quest:Activate("return_to_giver")
        end,

        on_complete = function(quest)
            quest:Activate("return_to_giver")
        end,
    }

    :AddObjective{
        id = "rat_out_giver",
        title = "(Optional) Tell {handler} about {giver}'s shady dealings",
        desc = [[{handler} would probably want to know that one of {handler.hisher} people is working out side-deals with the enemy.]],
        mark = { "handler" },
        
        on_complete = function(quest) 
            if quest:IsActive("talk_to_target") then
                quest:Cancel("talk_to_target")
            end

            quest:Activate("lure_giver")
        end,
    }
    
    :AddObjective{
        id = "lure_giver",
        title = "Convince {giver} to come with you",
        desc = "{handler} wants you to lure {giver} to the ambush point for a little disciplinary 're-education'. Convince them to follow you.",
        mark = { "giver"},
        negotiation_targets = {"giver"},
        on_complete = function(quest) 
            quest:Activate("beat_up_giver")
        end,
    }
    

    :AddObjective{
        id = "beat_up_giver",
        title = "Take {giver} into the forest",
        desc = "Take {giver} to the {ambush_site} and beat them up.",
        mark = { "ambush_site" },
        on_activate = function(quest) 
            if not quest:GetCastMember( "ambush_site" ) then
                quest:AssignCastMember("ambush_site")
            end
            if quest:IsActive("return_to_giver") then
                quest:Cancel("return_to_giver")
            end

        end,
        on_complete = function(quest) 
            quest:Activate("return_to_handler")
        end,
    }

    :AddObjective{
        id = "attend_meeting",
        title = "Take {giver} to the meeting",
        desc = "Take {giver} to the {ambush_site} and protect them during the meeting with {target}.",
        mark = { "ambush_site" },
        on_activate = function(quest) 
            quest:Cancel("rat_out_giver")
            if not quest:GetCastMember( "ambush_site" ) then
                quest:AssignCastMember("ambush_site")
            end
        end,
    }

    :AddObjective{
        id = "return_to_handler",
        title = "Return to {handler}",
        desc = "{handler} promised to pay you for beating up {giver}. Go collect your blood money.",
        ready_for_completion = true,
        mark = { "handler" },
    }

    :Loc{
        RETURN_TO_GIVER_AGREED = [[Tell {giver} that {target} agreed]],
        RETURN_TO_GIVER_NOHARM = [[Tell {giver} that {target} won't come to terms]],
    }

    :AddObjective{
        id = "return_to_giver",
        title = "Return to {giver}",
        
        desc_fn = function(quest, str)
            if quest.param.agreed then
                return quest:GetLocalizedStr( "RETURN_TO_GIVER_AGREED" )
            else
                return quest:GetLocalizedStr( "RETURN_TO_GIVER_NOHARM" )
            end

        end,
        desc = "{1}",
        mark = { "giver" },
    }

    --[[:AddOpinionEvents{
        killing_in_the_name_of =  
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Killed someone in their name",
        },
    }--]]

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("talk_to_target", "target")
    :Loc{
        OPT_NEGOTIATE = "Present {giver}'s offer",
        DIALOG_NEGOTIATE = [[
            player:
                !interest
                $miscPersuasive
                I've been sent to recruit you for an enticing venture. It's secretive, but profitable. 
            agent:
                !dubious
                $neutralDubious
                Don't be cute, hunter. What's the job? 
            player:
                !placate
                $neutralThoughtful
                All right, I'll give it to you straight.
                
        ]],
        DIALOG_NEGOTIATE_WIN = [[
            player:
                !shrug
                $neutralResigned
                So you can see why the officer wants to keep it quiet. 
            agent:
                !thought
                $neutralThoughtful
                Sounds like {giver} has more to lose than I do, if this goes south. You go tell {giver} that I want to meet. Discreetly, of course.
                If {giver.heshe} doesn't like that, well, too bad. {giver.HeShe} can find another stooge. 
            player:
                !wince2
                $neutralResigned
                {giver.HeShe} <i>won't</i> like it, but I'll pass it on.
        ]],
        DIALOG_NEGOTIATE_LOSE = [[
            player:
                !bashful
                $miscPersuasive
                I mean, everyone loves a dirty cop. What could possibly go wrong?
            agent:
                !angry_shrug
                $neutralWhatever
                The problem is they don't keep their dirt to themselves. So the answer is no. {giver} needs to find another stooge. 
            player:
                !dubious
                $neutralDubious
                Ah, I'll, uh, let {giver.himher} know.
        ]],

    }
    
    :Hub(function(cxt) 
            cxt:Opt("OPT_NEGOTIATE")
                    :Dialog("DIALOG_NEGOTIATE")
                    :Negotiation{
                        on_success = function(cxt) 
                            cxt:Dialog("DIALOG_NEGOTIATE_WIN")
                            cxt.quest:Complete("talk_to_target")
                            cxt.quest.param.agreed = true
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                        
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_NEGOTIATE_LOSE")
                            cxt.quest:Fail("talk_to_target")
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                    }
        end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("rat_out_giver", "handler")
    :Loc{
        OPT_RAT = "Tell {handler} about {giver}'s shady dealings",
        DIALOG_RAT = [[
            player:
                !interest
                $miscPersuasive
                The Admiralty rewards snitches, yeah? That's a thing you do?
            agent:
                !dubious
                $neutralDubious
                Sure, if you're offering. Who do you want to—
            player:
                !eureka
                $happyCocky
                {giver}. {giver.HeShe}'s trying to set up shop with the Spree. <i>Very</i> naughty. I heartily disapprove.
            agent:
                !disappoint
                $miscDepressed
                Yeah, that checks out. But {giver.heshe}'s got friends in high places, sadly—too high for retribution.
                !thought
                How about this: set up a meeting for {giver}'s little enterprise. Then when you get {giver.himher} alone... 
                You can see to a little off-the-books discipline.
            player:
                $neutralTakenAback
                So my reward for telling you something important is... more work?
            agent:
                !wink
                $happyCocky
                Hey, a pro like you wants to see things through. No need to thank me. 
            *** {handler} wants you to rough up {giver} as punishment. 
                
        ]],
    }
    :Hub(function(cxt) 
            cxt:Opt("OPT_RAT")
                :Dialog("DIALOG_RAT")
                :Fn(function(cxt) 
                        cxt.quest:Complete("rat_out_giver")
                        cxt.quest:SetProvider(cxt:GetAgent())
                        
                    end)
        end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("lure_giver", "giver")
    :Loc{
        OPT_TRICK = "Convince {giver} to come with you",
        DIALOG_TRICK = [[
            player:
                !interest
                $happyGreeting
                So the good news is that {target} is <i>very</i> interested. {target.HeShe} just wants a meeting to make it official.
            agent:
                !surprised
                $angryTakenAback
                What?! I hired you so I wouldn't <i>have</i> to meet {target.himher}.
            player:
                !shrug
                $neutralWhatever
                I know it's not ideal, but it's what you got.
        ]],
        DIALOG_NEGOTIATE_WIN = [[
            player:
                !permit
                $miscPersuasive
                This way you'll be able to feel {target} out, too. Plus, you can always say you were meeting an informant.
            agent:
                !point
                $neutralResigned
                I—ugh, maybe you're right. But if this goes sour, I expect you to have my back. 
        ]],
        DIALOG_NEGOTIATE_LOSE = [[
            player:
                !thought
                $neutralThoughtful
                The woods are isolated, and no one will be around to see you, uh—
            agent:
                !angry
                $angryTakenAback
                W-wait a minute! You're trying to trick me, aren't you? Did you sell me out?!
            player:
                !placate
                $miscPersuasive
                Of course not! Do I look like a snitch?
            agent:
                $angryHostile
                You won't when I'm through with you!
                !attack
        ]],
        DIALOG_DEAD = [[
            agent:
                !exit
            * {agent} is dead. Probably best for you and {handler} both if you keep your mouth shut.
        ]],
        DIALOG_LESSON = [[
            agent:
                !injured
                $scaredAnguished
                You've made your point, hunter. So what happens now?
            player:
                !point
                $angrySeething
                Fortunately for you, nobody wants you dead. But you're to behave from now on. 
            agent:
                $scaredFearful
                Fine, I hear you. Shame on me for dreaming of a better life, huh.
            player:
                !crossed
                $angryHostile
                I'm sure you won't make that mistake twice.
        ]],
        OPT_DEFEND = "Defend yourself!",
    }
    :Hub(function(cxt) 
            cxt:Opt("OPT_TRICK")
                :Fn(function(cxt) 
                    if cxt.quest:IsActive("return_to_giver") then
                        cxt.quest:Cancel("return_to_giver")
                    end
                end)
                :Dialog("DIALOG_TRICK")
                :Negotiation{
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_NEGOTIATE_WIN")
                        cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest:Complete("lure_giver")
                        StateGraphUtil.AddEndOption(cxt)
                    end,
                    
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATE_LOSE")
                        cxt:Opt("OPT_DEFEND")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                on_win = function(cxt) 
                                    if cxt:GetAgent():IsDead() then
                                        cxt:Dialog("DIALOG_DEAD")
                                        cxt.quest:Fail()
                                    else
                                        cxt:Dialog("DIALOG_LESSON")
                                        cxt.quest:Cancel("lure_giver")
                                        cxt.quest:Activate("return_to_handler")
                                    end
                                    StateGraphUtil.AddEndOption(cxt)
                                end,
                            }
                    end,
                }
        end)

QDEF:AddConvo("beat_up_giver")
    :Confront(function(cxt) 
            if cxt.quest:GetCastMember("giver"):IsInPlayerParty() and cxt.location == cxt:GetCastMember("ambush_site") then
                return "STATE_CONFRONT"
            end
        end)
    :State("STATE_CONFRONT")
        :Loc{
            DIALOG_INTRO = [[
                * As you enter the forest, the looming tree tops seem to spook {giver}.
                player:
                    !left
                giver:
                    !right
                    !cagey
                    $scaredDefault
                    You sure this is the right place? You sure this isn't—
                player:
                    !happy
                    $happyDefault
                    A trap? Afraid so, pal. See, I told {handler} about your little side hustle and she ain't too pleased. 
                giver:
                    !angry
                    $angryTakenAback
                    You—you fink! 
                    $angryDefault
                    I won't make the mistake of hiring a lumin-stained outsider like you again.
                    !attack
            ]],
            DIALOG_KILLED = [[
                * {agent} is dead. You're going to pretend that didn't happen, and hope that {handler} doesn't bring it up.
            ]],
            DIALOG_SURVIVED = [[
                agent:
                    !injured
                    $scaredAnguished
                    You've made your point, hunter. So what happens now?
                player:
                    !point
                    $angrySeething
                    Fortunately for you, nobody wants you dead. But you're to behave from now on. 
                agent:
                    $scaredFearful
                    Fine, I hear you. Shame on me for dreaming of a better life, huh.
                player:
                    !crossed
                    $angryHostile
                    I'm sure you won't make that mistake twice.
                * {agent} limps into the bush, likely to look for a new leaf to turn over.
            ]],
            OPT_BEAT_UP = "Beat up {giver}",

            }
        :Fn(function(cxt) 

                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("giver"))
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_BEAT_UP")
                    :ReceiveOpinion( OPINION.BETRAYED )
                    :Battle{
                        on_win = function(cxt) 
                            if cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_KILLED")
                                cxt.quest:Fail()
                                
                            else
                                cxt:Dialog("DIALOG_SURVIVED")
                                cxt.quest:Complete("beat_up_giver")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end,
                    }

            end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("attend_meeting")
    :Confront(function(cxt) 
            if cxt.quest:GetCastMember("giver"):IsInPlayerParty() and cxt.location == cxt:GetCastMember("ambush_site") then
                return "STATE_CONFRONT"
            end
        end)
    
    :State("STATE_CONFRONT")
        :Loc{
                DIALOG_INTRO = [[
                    * You enter the meeting site. {giver} looks around nervously, jumping at every shadow.
                    giver:
                        !left
                        $neutralDubious
                        So this is the place, huh? Where's {target}?
                ]],


            }
        :Fn(function(cxt) 
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("giver"))
                cxt:Dialog("DIALOG_INTRO")
                
                local possibles = {"STATE_NPC_TALKS"}
                if cxt.quest:GetCastMember("target"):GetRelationship() <= RELATIONSHIP.NEUTRAL then
                    table.insert(possibles, "STATE_AMBUSH")
                end


                local dest_state = DEST_STATE_OVERRIDE
                if not dest_state then
                    dest_state = table.arraypick(possibles)
                end
                cxt:GoTo(dest_state)
            end)

    :State("STATE_AMBUSH")
        :Loc{
            DIALOG_INTRO = [[
                * The unnatural silence is your only warning of the ambush. 
                target:
                    !right
                    $angryHostile
                    There's the switch. Get 'em!
                    !fight
                giver:
                    !angry
                    $scaredFearful
                    Aw, Hesh! I knew it!
            ]],

            DIALOG_WON_AMBUSH_GIVER_ALIVE = [[
                player:
                    !left
                    !tired
                giver:
                    !right
                    $angrySeething
                    {target} set us up! I knew I never should've trusted a Spree. 
                    !give
                    Still, you did the job. And I can't afford to burn any more bridges. So you'll still get paid. 
                player:
                    !take
                    Finally. 
            ]],

            DIALOG_WON_AMBUSH_GIVER_DEAD = [[
                player:
                    !left
                agent:
                    !exit
                * {giver} is dead, and you've lost your payout. But at least you got to stretch your legs.
            ]],
            OPT_DEFEND = "Defend yourself!",
        }
        :Fn(function(cxt) 
                CreateCombatBackup(cxt.quest:GetCastMember("target"), "BANDIT_PATROL_BACKUP", cxt.quest:GetRank())
                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        allies = {cxt.quest:GetCastMember("giver")},
                        enemies = {cxt.quest:GetCastMember("target")},
                        on_win = function(cxt) 
                            if cxt.quest:GetCastMember("giver"):IsDead() then
                                cxt:Dialog("DIALOG_WON_AMBUSH_GIVER_DEAD")
                                cxt.quest:Fail()
                            else
                                cxt:Dialog("DIALOG_WON_AMBUSH_GIVER_ALIVE")
                                cxt.quest:GetCastMember("giver"):Dismiss()
                                cxt.quest:Complete()
                                ConvoUtil.GiveQuestRewards(cxt)
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }
            end)

    :State("STATE_NPC_TALKS")
        :Loc{
            DIALOG_INTRO = [[
                * {target} shows up alone.
                target:
                    !right
                    $neutralDubious
                    All right. I'm here. So if you're on the level, let's talk.
                giver:
                    $neutralDirect
                    Well...
                    !exit
                target:
                    !exit
                * The two discuss their common ground, and unconvincingly promise to never stab each other in the back. 
                player:
                    !left
                    !crossed
                * You wait for things to get interesting, but they remain oblivious to your boredom. 
                player:
                    !sigh 
                    !exit
            ]],

            DIALOG_AMBUSH = [[
                target:
                    !left
                target:
                    $happyDefault
                    All right, you've convinced me. Let's get rich.
                * As if summoned by the word, the bushes part, revealing a surly-looking bandit.
                agent:
                    !right
                    $happyCocky
                    Knew you were up to something, {target}. Never dreamed you'd be working for the Admiralty, though.
                    !throatcut 
                    'Course, that means you'll be dying with them, too. 
            ]],
            DIALOG_AMBUSH_EVERYONE_DEAD = [[
                * Both {target} and {giver} are dead—so you're definitely not getting paid. So it goes.
            ]],
            DIALOG_AMBUSH_TARGET_DEAD = [[
                giver:
                    !right
                    $miscDepressed
                    Hesh. We survived, but {target} didn't. And I just got {target.himher} exactly where I wanted.
                    !give
                    Still, you did the job. And I can't afford to burn any more bridges. So you'll still get paid. 
                player:
                    !take
                    Finally. 
            ]],

            DIALOG_AMBUSH_GIVER_DEAD = [[
                target:
                    !right
                    $miscDepressed
                    Hesh. Thanks for killing them, grifter. At least I can still head back to camp without expecting a executioner.
                    !shrug
                    Too bad {giver}'s dead. Guess neither of us are getting paid today.
                    !exit
                * {target} leaves, taking care to cover their tracks.
            ]],

            DIALOG_AMBUSH_BOTH_ALIVE = [[
                player:
                    !exit
                agent:
                    !exit
                * {target} and {giver} look at each other across the battlefield, breathing heavily.
                target:
                    !left
                giver:
                    !right
                    $neutralDubious
                    So I guess we're in this together now.
                target:
                    $miscDepressed
                    I already regret it.
                player:
                    !left
                giver:
                    $neutralDirect
                    Thanks for the protection, {player}. We'll take it from here.

            ]],

            DIALOG_TALK_ARGUMENT = [[
                giver:
                    !right
                target:
                    !left
                    $angryReluctant
                    Thanks for the offer, but I'm gonna have to say no. It's too risky.
                giver:
                    $miscDepressed
                    !disappoint
                    I'm sorry you feel that way. It's not profit without a little risk. 
                    !throatcut
                    {player}, clean up here. We can't afford the loose end.
                player:
                    !left
                    !dubious
                target:
                    !right
                    !placate
                    $scaredFearful
                    Wait, grifter. Don't be so hasty! I'll pay more for the protection. 
            ]],
            
            OPT_PICK_GIVER = "Attack {target} for {giver}",
            DIALOG_PICK_GIVER = [[
                player:
                    $neutralResigned
                    Sorry, {target}, but it's my reputation that pays the bills. 
            ]],
            OPT_PICK_TARGET = "Betray {giver}",
            DIALOG_PICK_TARGET = [[
                giver:
                    !right
                    !scared
                player:
                    $neutralResigned
                    Sorry, {giver}, but when the money's good, I take it. 
            ]],


            DIALOG_EVERYBODY_DEAD = [[
                * Both sides are dead, and you're left picking the lint of your pockets. Well played. 
            ]],

            DIALOG_WON_WITH_TARGET = [[
                target:
                    !right
                    $happyDefault
                    !give
                    Thanks for that, grifter. If it weren't for your bottom line, I'd be dead.
                player:
                    !take
                    Happy to help, when the price is right.
            ]],

            DIALOG_WON_WITH_GIVER = [[
                giver:
                    !right
                    $miscRelieved
                    Well, that didn't turn out the way I wanted. But nothing ventured, nothing gained.
                    !chuckle
                    Speaking of which, take your blood money, hunter. 
                player:
                    !agree
                    Thanks. I will. 
            ]],


            DIALOG_SIDED_TARGET_DEAD = [[
                 * Both {target} and {giver} are dead—so you're definitely not getting paid. So it goes.
            ]],
            
            DIALOG_SIDED_GIVER_DEAD = [[
                 * Both {target} and {giver} are dead—so you're definitely not getting paid. So it goes.
            ]],
            OPT_DEFEND = "Defend yourself",
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            local talk_success = math.random() < .5
            if TALK_SUCCESS_OVERRIDE ~= nil then
                talk_success = TALK_SUCCESS_OVERRIDE
            end

            if talk_success then
                local opfor = CreateCombatParty("BANDIT_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.enc:SetPrimaryCast(opfor[1])

                cxt:Dialog("DIALOG_AMBUSH")            
                cxt:Opt("OPT_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        allies = {cxt.quest:GetCastMember("giver"), cxt.quest:GetCastMember("target")},
                        on_win = function(cxt) 

                            if cxt.quest:GetCastMember("giver"):IsDead() and cxt.quest:GetCastMember("target"):IsDead() then
                                cxt:Dialog("DIALOG_AMBUSH_EVERYONE_DEAD")
                                cxt.quest:Fail()
                            elseif cxt.quest:GetCastMember("target"):IsDead() then
                                cxt:Dialog("DIALOG_AMBUSH_TARGET_DEAD")
                                cxt.quest:Complete()
                                ConvoUtil.GiveQuestRewards(cxt)
                            elseif cxt.quest:GetCastMember("giver"):IsDead() then
                                cxt:Dialog("DIALOG_AMBUSH_GIVER_DEAD")
                                cxt.quest:Fail()
                            else
                                cxt:Dialog("DIALOG_AMBUSH_BOTH_ALIVE")
                                cxt.quest:GetCastMember("giver"):Dismiss()
                                cxt.quest:Complete()
                                ConvoUtil.GiveQuestRewards(cxt)
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }

            else
                cxt:Dialog("DIALOG_TALK_ARGUMENT")
                
                cxt:Opt("OPT_PICK_TARGET")
                    :Dialog("DIALOG_PICK_TARGET")
                    :Fn(function(cxt) 
                            cxt:GetCastAgent("giver"):OpinionEvent(OPINION.BETRAYED)
                            cxt.quest:SetProvider(cxt.quest:GetCastMember("target"))
                    end)
                    :Battle{
                        allies = {cxt.quest:GetCastMember("target")},
                        enemies = {cxt.quest:GetCastMember("giver")},
                        on_win = function(cxt) 
                            if cxt.quest:GetCastMember("target"):IsDead() then
                                cxt:Dialog("DIALOG_SIDED_TARGET_DEAD")
                                cxt.quest:Fail()
                            else
                                cxt:Dialog("DIALOG_WON_WITH_TARGET")
                                cxt.enc:GainMoney( cxt.quest:GetRank() * 50 )                                        
                                cxt.quest:Complete()
                                ConvoUtil.GiveQuestRewards(cxt)
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    }
                cxt:Opt("OPT_PICK_GIVER")
                    :Dialog("DIALOG_PICK_GIVER")
                    :Battle{
                        allies = {cxt.quest:GetCastMember("giver")},
                        enemies = {cxt.quest:GetCastMember("target")},
                        on_win = function(cxt) 
                            if cxt.quest:GetCastMember("giver"):IsDead() then
                                cxt:Dialog("DIALOG_SIDED_GIVER_DEAD")
                                cxt.quest:Fail()
                            else
                                cxt:Dialog("DIALOG_WON_WITH_GIVER")
                                cxt.quest:Complete()
                                ConvoUtil.GiveQuestRewards(cxt)
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                        
                    }
            end
        end)


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("return_to_giver", "giver")
    :Loc{
        OPT_AGREED = "Tell {giver} the deal is done",
        DIALOG_AGREED = [[
            player:
                !interest
                $happyGreeting
                So the good news is that {target} is <i>very</i> interested. {target.HeShe} just wants a meeting to make it official.
            agent:
                !surprised
                $angryTakenAback
                What?! I hired you so I wouldn't <i>have</i> to meet {target.himher}.
                !palm
                But it is what it is, I guess. So let's go. Your job was to be my proxy—so I'll keep you proximate. 
            player:
                !disappoint
                $neutralTakenAback
                Ugh, I knew you'd say that. 
        ]],
        DIALOG_ADMIRALTY =
        [[
            agent:
                $angryAside
                'Course, I could tell {handler} {handler.hisher} new pet has been seen running around in bandit camps. 
            player:
                !angry
                I could tell {handler} <i>you're</i> looking for a new line of work.
            agent:
                $neutralWhatever
                See? We have fun, you and I. Let's go. 
        ]],
        DIALOG_NON_ADMIRALTY =
        [[
            agent:
                $miscPersuasive
                You wanna get paid, don't you? Then let's go.
        ]],
        OPT_DISAGREED = "Tell {giver} {target} isn't interested",
        DIALOG_DISAGREED = [[
            player:
                $neutralDubious
                !shrug
                Sorry, {giver}. {target} wasn't interested.
            agent:
                !dubious
                $neutralTakenAback
                Your <i>job</i> was to be convincing!
            player:
                $miscDepressed
                I was, if you ask me. Just wasn't enough. 
            agent:
                !handwave
                $angryThanks
                Then I have nothing to pay you. Get lost—none of this ever happened. 
        ]],

    }
    :Hub(function(cxt) 
            if cxt.quest.param.agreed then
                cxt:Opt("OPT_AGREED")
                    :Dialog("DIALOG_AGREED")
                    :Dialog(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "DIALOG_ADMIRALTY" or "DIALOG_NON_ADMIRALTY")
                    :CompleteQuest("return_to_giver")
                    :ActivateQuest("attend_meeting")
                    :Fn(function(cxt) 

                            cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        end)
                    :DoneConvo()
            else
                cxt:Opt("OPT_DISAGREED")
                    :Dialog("DIALOG_DISAGREED")
                    :FailQuest()
                    :DoneConvo()
            end            
        end)    

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("return_to_handler", "handler")
    :Loc{
        OPT_SAY_DONE = "Tell {handler} {giver} has been punished",
        DIALOG_DONE = [[
            player:
                $happyCocky
                {giver} won't step out of line again. 
            agent:
                $neutralThanks
                Great! Now shut your mouth about it. Can't let our little secret get out, now, can we?
        ]],
    }
    :Hub(function(cxt) 
            cxt:Opt("OPT_SAY_DONE")
                :Dialog("DIALOG_DONE")
                :CompleteQuest()
                :DoneConvo()
        end)    

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo(nil, "giver")
    :Loc{
        DIALOG_OFFER = [[
            agent:
                !offer
                $happyGreeting
                Hey, do you wanna make some money?
                I've got a great opportunity, but there's a hitch: I need a partner in the Spree to pull it off.
            player:
                !dubious
                $neutralDubious
                Pretty sure you're supposed to <i>arrest</i> the Spree—not work with 'em.
            agent:  
                !agree
                $miscPersuasive
                Exactly. I need some insulation from blowback if I'm gonna make this work. So how about <i>you</i> be my proxy?
                Go talk to {target}, and convince {target.himher} this is a good deal {target.heshe} shouldn't refuse. 
        ]],
        DIALOG_PST = [[
            agent:
                !cagey
                $miscMischievous
                Great! Just, uh, don't let my bosses find out what we're doing. Can't spend the money if we're dead, right?
        ]]
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "talk_to_target", "DIALOG_OFFER", function(cxt)
                if Util.IsPlayerWorkingWithFaction("ADMIRALTY") then
                    cxt:Dialog("DIALOG_PST")
                end
            end)
    end)


