local fail_clues_fn = function(quest)
    if  not quest:IsActive("investigate_associate_1") and
        not quest:IsActive("investigate_associate_2") then
            quest:Activate("explain_your_failure")
    end
end

local complete_clues_fn = function(quest) 
    if quest:IsActive("investigate_associate_1") then
        quest:Cancel("investigate_associate_1")
    end

    if quest:IsActive("investigate_associate_2") then
        quest:Cancel("investigate_associate_2")
    end

    quest:Activate("apprehend_target")
end

local QDEF = QuestDef.Define{
        title = "Bounty Hunt",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = "The Admiralty has issued a bounty on {target}. There's a {alive_bonus#money} bonus if you bring {target.himher} in alive.",
        icon = engine.asset.Texture("icons/quests/bounty_hunt.tex"),
        focus = QUEST_FOCUS.COMBAT,
        
        on_init = function(quest)
            quest.param.alive_bonus = quest:GetRank()*25 
            quest.extra_reward = EXTRA_QUEST_REWARD.FREE_ITEM
            quest.extra_reward_data = "bounty_hunter"
        end,
        
        on_post_load = function(quest)
            if quest:IsActive( "investigate_associate_1") and (quest:GetCastMember( "associate_1" )  == nil or quest:GetCastMember( "associate_1" ):IsDead()) then
                quest:Fail( "investigate_associate_1" )
            end
            if quest:IsActive( "investigate_associate_2") and (quest:GetCastMember( "associate_2" )  == nil or quest:GetCastMember( "associate_2" ):IsDead()) then
                quest:Fail( "investigate_associate_2" )
            end
        end,

        avoids = {
            {"giver", "target"},
            {"giver", "associate_1"},
            {"giver", "associate_2"},
            {"target", "associate_1"},
            {"target", "associate_2"},
            {"associate_1", "associate_2"},
        },

        is_in_hiding = function(quest, agent)
            if (quest:IsActive("investigate_associate_1") or quest:IsActive("investigate_associate_2"))
                and agent == quest:GetCastMember("target") then
                --
                return true
            end
        end,

    }

    :AddCast{
        cast_id = "giver",
        provider = true,

        condition = function(agent, quest) 
            if agent:GetFactionID() == "ADMIRALTY" and agent:GetContentID() == "ADMIRALTY_CLERK" then
                return true
            end
        end,
    }

    :AddCast{
        cast_id = "target",
        condition = function(agent, quest) 
            return agent:GetFactionID() ~= "ADMIRALTY" and agent:GetContentID() ~= "LUMINARI" and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, false) and agent:GetBrain():GetWorkPosition() == nil
        end,
        events = 
        {
            agent_retired = function( quest, agent ) 
                if quest:IsActive("take_target_to_giver") or quest:IsActive("apprehend_target") then
                    
                    if quest:IsActive("take_target_to_giver") then
                        quest:Cancel("take_target_to_giver")
                    end

                    if quest:IsActive("apprehend_target") then
                        quest:Cancel("apprehend_target")
                    end

                    if agent:IsDead() and not quest:IsActive("tell_target_giver_dead") then
                        quest:Activate("tell_target_giver_dead")
                    end
                end

            end,
        }

    }
    :AddCastFallback{
        cast_fn = function(quest, t)
            
            local options = {}
            local rank = quest:GetRank() 
            if rank <= 1 then
                table.insert(options, "LABORER")
                table.insert(options, "HEAVY_LABORER")
            elseif rank <= 3 then
                table.insert(options, "BANDIT_GOON")
                table.insert(options, "BANDIT_GOON2")
                table.insert(options, "JAKES_RUNNER")
            else --4, 5
                table.insert(options, "JAKES_SMUGGLER")
                table.insert(options, "WEALTHY_MERCHANT")
                table.insert( options, "BANDIT_RAIDER" )
            end

            if #options > 0 then
                local def = options[math.random(#options)]
                table.insert( t, quest:CreateSkinnedAgent( def ) )
            end
        end,
    }

    :AddCast{
        cast_id = "associate_1",
        condition = function(agent, quest) 
            return agent:GetFactionID() ~= "ADMIRALTY" and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, false)
        end,
        events =
        {
            agent_retired = function( quest, agent ) 
                if quest:IsActive("investigate_associate_1") then
                    quest:Fail("investigate_associate_1")
                end
            end,
        }
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"LABORER", "BANDIT_GOON", "BANDIT_GOON2", "JAKES_RUNNER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }


    :AddCast{
        cast_id = "associate_2",
        condition = function(agent, quest) 
            return agent:GetFactionID() ~= "ADMIRALTY" and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, false, true)
        end,
        events =
        {
            agent_retired = function( quest, agent ) 
                if quest:IsActive("investigate_associate_2") then
                    quest:Fail("investigate_associate_2")
                end
            end,
        }
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"HEAVY_LABORER", "BANDIT_GOON2", "JAKES_RUNNER", "JAKES_SMUGGLER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }

    
    :AddObjective{
        id = "investigate_associate_1",
        title = "Talk to {associate_1}",
        desc = "{associate_1} might know where {target} is. Go ask them nicely for some information.",
        mark = { "associate_1"},

        on_complete = complete_clues_fn,
        on_fail = fail_clues_fn,
        negotiation_targets = {"associate_1"},
        combat_targets = {"associate_1"},

    }

    :AddObjective{
        id = "investigate_associate_2",
        title = "Talk to {associate_2}",
        desc = "{associate_2} might know where {target} is. Go ask them nicely for some information.",
        mark = { "associate_2"},
        on_complete = complete_clues_fn,
        on_fail = fail_clues_fn,
        negotiation_targets = {"associate_2"},
        combat_targets = {"associate_2"},
    }

    :AddObjective{
        id = "apprehend_target",
        title = "Confront {target}",
        desc = "You have found {target}'s current location. Go confront {target.himher}.",
        mark = { "target"},
        
        combat_targets = {"target"},

    }

    :AddObjective{
        id = "explain_your_failure",
        title = "Return to {giver}",
        desc = "None of the clues panned out. You should probably tell {giver}.",
        mark = { "giver"},
    }

    :AddObjective{
        id = "take_target_to_giver",
        title = "Bring {target} to the Admiralty",
        desc = "You've captured {target}. Now bring them in and collect your pieces of silver.",
        mark = { "giver"},
    }

    :AddObjective{
        id = "tell_target_giver_dead",
        title = "Collect bounty from {giver}",
        desc = "You killed {target}. Go collect your blood money.",
        mark = { "giver"},
    }

    :AddObjective{
        id = "convince_giver_target_dead",
        title = "Convince {giver}",
        desc = "You let {target} live. Go convince {giver} that they're dead.",
        mark = { "giver"},
        negotiation_targets = {"giver"},

    }


local function on_win_associate_battle(cxt)
    cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_WON_FIGHT_DEAD" or "DIALOG_WON_FIGHT_ALIVE")
    local assoc1 = cxt:GetAgent() == cxt.quest:GetCastMember("associate_1") and cxt.quest:IsActive("investigate_associate_1")
    
    if not cxt:GetAgent():IsDead() then
        cxt.quest:Complete(assoc1 and "investigate_associate_1" or "investigate_associate_2")
    end

    StateGraphUtil.AddEndOption(cxt)
end


local function on_win_target_battle(cxt)
    cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_KILLED_TARGET" or "DIALOG_CAPTURED_TARGET")
    if not cxt:GetAgent():IsDead() then
        if cxt.quest:IsActive( "apprehend_target" ) then
            cxt.quest:Complete( "apprehend_target" )
        end
        cxt.quest:Activate("take_target_to_giver")
        cxt.quest:GetCastAgent("target"):Recruit(PARTY_MEMBER_TYPE.CAPTIVE)
        TheGame:GetGameState():AddPendingEvent()
    end
    
    StateGraphUtil.AddEndOption(cxt)
end

QDEF:AddConvo("explain_your_failure", "giver")
    :Loc{
        OPT_EXPLAIN_FAILURE = "Tell {giver} that {target} got away",
        DIALOG_INTRO = [[
            player:
                $miscEmbarrassed
                {target} got away. I couldn't figure out where to find {target.himher}.
            giver:
                $angryReluctant
                I should have your bounty hunting license revoked.
                You're lucky we're understaffed.
        ]]
    }
    :Hub(function(cxt)
        cxt:Opt("OPT_EXPLAIN_FAILURE")
            :Dialog("DIALOG_INTRO")
            :FailQuest()
            :DoneConvo()
    end)

QDEF:AddConvo("convince_giver_target_dead", "giver")
    :Loc{
        OPT_TELL_TARGET_GIVER_DEAD = "Convince {giver} that {target} is dead",
        DIALOG_TELL = [[
            player:
                $miscPersuasive
                {target} is dead.
            giver:
                $neutralTakenAback
                Uh uh. And do you have any proof of that?
        ]],
        DIALOG_FAILURE = [[
            giver:
                $angryTakenAback
                Just stop talking.
                I don't know what you're up to, but I want no part of it.
                Get out of here before I have <b>you</> arrested instead.
        ]],
        DIALOG_SUCCESS = [[
            giver:
                $happyThanks
                That sounds about right.
                Here's your reward.
        ]]
    }
    :Hub(function(cxt)
        cxt:Opt("OPT_TELL_TARGET_GIVER_DEAD")
            :Dialog("DIALOG_TELL")
            :Negotiation{
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_SUCCESS")
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                    StateGraphUtil.AddEndOption(cxt)
                end,
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_FAILURE")
                    cxt.quest:Fail()
                    StateGraphUtil.AddEndOption(cxt)
                end,
            }
    end)

QDEF:AddConvo("take_target_to_giver", "giver")
    :Loc{
        OPT_HAND_OVER = "Hand over {target}",
        DIALOG_HAND_OVER = [[
            player:
                $happyGreeting
                Here's {target}, as promised.
            giver:
                $happyThanks
                You actually brought {target.himher} in alive? Nice job, grifter. You surprise me.
            target:
                !left
                !angry
            giver:
                $angryCruel
                Come along, {target}. We have a <b>lot</> to talk about.
            target:
                $angryDefault
                This is an outrage! I been set up, I tell ya!
            giver:
                !cruel
                $angryCruel
                Save it for the magistrate.

            *** You successfully bring {target} back for the Admiralty.
        ]]


    }
    :Hub(function(cxt)
        cxt:Opt("OPT_HAND_OVER")
            :Fn(function(cxt) 
                    cxt:Dialog("DIALOG_HAND_OVER")
                    cxt.quest:GetCastMember("target"):OpinionEvent(OPINION.SOLD_OUT_TO_ADMIRALTY)
                    cxt.quest:GetCastMember("target"):Dismiss()
                    cxt.quest:GetCastMember("target"):MoveToLimbo()
                    cxt.quest:GetCastMember("target"):Retire()
                    cxt.enc:GainMoney(cxt.quest.param.alive_bonus)
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                    StateGraphUtil.AddEndOption(cxt)
                end)
    end)


QDEF:AddConvo("tell_target_giver_dead", "giver")
    :Loc{
        OPT_TELL_TARGET_GIVER_DEAD = "Tell {giver} that {target} is dead",
        DIALOG_TELL = [[
            player:
                $neutralGreeting
                {target} is dead. Put up too much of a fight to be brought in alive, unfortunately.
            giver:
                !shrug
                $happyResigned
                Shame, but I guess that was too much to ask for.
                Still, bounty said either way was fine. So you get yours. 
                !give
            player:
                !take
                We all do, in the end.
        ]]
    }
    :Hub(function(cxt)
        cxt:Opt("OPT_TELL_TARGET_GIVER_DEAD")
            :Dialog("DIALOG_TELL")
            :CompleteQuest()
    end)

QDEF:AddConvo("apprehend_target", "target")
    :RequireFlags(CONVERSATION_FLAGS.BUSINESS)
    :Loc{
        OPT_ASK_ABOUT_BOUNTY = "Confront {target} about their bounty",
        DIALOG_BOUNTY = [[
            player:
                $neutralGreeting
                You know how when a bounty hunter catches their quarry, they say something like...
                !angry_accuse 
                $neutralJoke
                "You gonna come quiet, or are we gonna do this the hard way?"
            agent:
                !dubious
                Uh, yeah?
            player:
                !neutral
                ...
            agent:
                ...
            player:
                !happy
                ...
            agent:
                !palm
                ... oh.
        ]],

        DIALOG_FIGHT = [[
            agent:
                $happyCocky
                Seems you can't decide whether you're a hunter or a comedian. 
                But I know what you really are: 
                !fight
                $angryHostile
                Dead.

            *** {agent} resists being taken in for a bounty!
        ]],

        OPT_DEFEND = "Defend yourself",
        OPT_BACK_OFF = "Back off",
        DIALOG_BACK_OFF = [[
            player:
                !placate
                Whoa now, let's not get hasty.
                I don't actually much like the hard way, myself. 
                $miscPersuasive
                Consider this settled, alright?
            * You back off, leaving {agent} glowering.
        ]],

        DIALOG_OFFER = [[
            agent:
                !placate
                Now hold on! You got a bottom line to consider, here.
                You want to collect, and I want to put a stop to this bounty nonsense.
            player:
                !crossed
                I take it you have something in mind.
            agent:
                "Dead or alive", right? You tell 'em I died in the fight. 
                You get paid, and no more bounty. We both win.
            player:
                $neutralDubious
                Except you're worth more alive.
            agent:
                Not if I pay you, too.
                !wink 
                $miscPersuasive
                Think about it, eh?
                *** {agent} wants to cut you a deal to betray {giver}.
        ]],
        OPT_ACCEPT_OFFER = "Accept the deal",
        OPT_REJECT_OFFER = "Reject the deal and take {target} by force",
        DIALOG_ACCEPT = [[
            player:
                $neutralDirect
                !point
                You disappear. Completely. 
                My livelihood is my reputation, and you're asking me to risk both. 
            agent:
                !overthere
                Hey, I'll be a complete ghost.
                It'll be just like you really killed me.
            player:
                !agree
                $neutralDirect
                Deal.
            agent:
                $happyThanks
                Hey, you're not so bad, hunter. 
                Glad you could see reason.
                !give
                !exit
            * {agent} hightails it out of here. You can only hope this "ghost" doesn't come back to haunt you.
            *** You let {agent} go, and now have to convince {giver} that you did your job.
        ]],
        DIALOG_REJECT = [[
            player:
                $angryAside
                And piss off the Admiralty? They're my biggest client. 
            agent:
                !disappoint
                $neutralResigned
                I'm sorry to hear that. 
                'Cause there's only one way this ends, now.
                !fight
        ]],
        DIALOG_KILLED_TARGET = [[
            * {target} is dead. At least you'll still get paid.
        ]],
        DIALOG_CAPTURED_TARGET = [[
            target:
                !injured
                $scaredAnguished
                All right! Enough. 
                I'll take my chances with the Admiralty. 
            player:
                !tired
                That's more like it. 
                Just wish you coulda realized that before we went through all this rigamarole. 
            * You bind {target}'s wrists, and {target.heshe} falls in line behind you.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_ABOUT_BOUNTY")
            :Dialog("DIALOG_BOUNTY")
            :Fn(function(cxt) 
                local offer = QuestUtil.CalculateRewardValue(cxt.quest)
                if cxt:GetAgent():GetRenown() > 2 and cxt:GetAgent():GetRenown() >= cxt:GetAgent():GetCombatStrength() then
                    cxt:Dialog("DIALOG_OFFER")
                    cxt:Opt("OPT_ACCEPT_OFFER")
                        :ReceiveMoney(offer)
                        :Fn(function() 
                                cxt:Dialog("DIALOG_ACCEPT")
                                cxt:GetAgent():MoveToLimbo()
                                cxt:GetAgent():Retire()
                                cxt.quest:Activate("convince_giver_target_dead")
                                StateGraphUtil.AddEndOption(cxt)
                            end)
                    cxt:Opt("OPT_REJECT_OFFER")
                        :Dialog("DIALOG_REJECT")
                        :Battle{
                            on_win = on_win_target_battle,
                        }

                else
                    cxt:Dialog("DIALOG_FIGHT")
                    cxt:Opt("OPT_DEFEND")
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            on_win = on_win_target_battle,
                        }                    
                    
                    cxt:Opt("OPT_BACK_OFF")
                        :Dialog("DIALOG_BACK_OFF")
                        :ReceiveOpinion(OPINION.THREATENED_TO_COLLECT_BOUNTY)
                        :Fn(function() StateGraphUtil.AddEndOption(cxt) end)
                end
            end)
    end)


QDEF:AddConvo()
    :RequireFlags(CONVERSATION_FLAGS.BUSINESS)
    :Loc{
            OPT_ASK_ABOUT_TARGET = "Ask about {target}",
            OPT_CONVINCE_THEM_TO_TALK = "Convince {agent} to talk",
            OPT_PAY_UP = "Pay for the information",
            OPT_BEAT_THEM_UP = "Beat the information out of {agent}",

            
            DIALOG_INTRO = [[
                player:
                    $miscPersuasive
                    Hey, pal. You wouldn't happen to know where I might find {target}, wouldja?
            ]],
            DIALOG_AFRAID = [[
                agent:
                    !cagey
                    {target}? No, no idea. I don't even know {target.himher}.
                    And if I did, {target.heshe} woulda told me to keep my mouth shut.
                    $scaredStammering
                    And if {target.heshe} didn't, I would know enough not to talk. 
                player:
                    !dubious
                    $neutralDubious
                    You sure? You seem kinda scared of {target.himher}.
                    Considering {target.heshe}'s someone you've never met and all.
            ]],
            DIALOG_TRY_CONVINCE_AFRAID = [[
                player:
                    $miscPersuasive
                    !overthere
                    Look, I'm a hunter, all right?
                    You tell me where {target} is, and I take {target.himher} in.
                    Then you have nothing to be scared of. 
                agent:
                    !scared_shrug
                    $scaredFearful
                    Oh yeah? And what if you fail, huh? Then {target} will be extra ornery.
                    I mean, uh, or so I think. I don't know {target.himher} personally.
            ]],
            DIALOG_CONVINCE_AFRAID = [[
                player:
                    You wanna spend your whole life living in fear?
                    !eureka
                    Or do you wanna seize the day and make something happen?
                    $miscPersuasive
                    Tell me where to go, and I make your problem go away. 
                agent:
                    !dubious
                    You could really do that? 
                    I guess you're a hunter. I guess that's what people pay you for.
                    !point
                    Okay. I'll show you. But you gotta do the job right!
                * {agent} marks your map in a hurry, then turns away as if you never even spoke.
                agent:
                    !exit
            ]],
            DIALOG_NO_CONVINCE_AFRAID = [[
                player:
                    !thumb
                    I'm the one taking all the risk, ain't I?
                    You're just being a baby.
                agent:
                    !scared_shrug
                    $scaredPowerless
                    You know what they say about babies in Havaria?
                    It's the one thing we don't turn into fodder for the yotes.
                    Forget it. I ain't tellin' you squat. 
            ]],
            DIALOG_ATTACK_AFRAID = [[
                player:
                    !angry_accuse
                    $angryHostile
                    Look, this is an Admiralty contract.
                    Which <i>means</i> you're obstructing justice.
                    And I got a hunter's license that says I can use force.
                    !fight
            ]],

            DIALOG_PAID_UP = [[
                player:
                    Maybe this will afford you some courage, huh?
                    !give
                agent: 
                    !take
                    What's this for?
                player:
                    Call it a protection fund. You tell me where to find {target}, then use that to lay low.
                    It's enough to keep you safe until the job is done. 
                agent:
                    !happy
                    Hey, now that's not a bad idea!
                    $happyThanks
                    Maybe this'll work out just fine. 
                * {agent} marks your map in a hurry, then turns away as if you never even spoke.
            ]],

            

            DIALOG_DEFENSIVE = [[
                agent:
                    !angry
                    $angryDefault 
                    Why? What for?
                    I ain't telling you nothing. 
            ]],

            DIALOG_TRY_CONVINCE_DEFENSIVE = [[
                player:
                    You sure about that, pal?
                    Seems to me you're caught between a knife and a hard place.
            ]],
            DIALOG_CONVINCE_DEFENSIVE = [[
                player:
                    !thumb
                    In case it weren't clear, I'm the knife.
                    The Admiralty issued a bounty for your friend {target}.
                    $angryCruel
                    As a hunter, I'm licensed to collect. 
                agent:
                    $neutralResigned
                    Wha—a real hunter? Not just some wannabe merc?
                    Didn't know {target} had dug up that kind of trouble.
                    !placate
                    Look, I'll tell you where to find {target.himher}. Just leave me out of it.
                * {agent} marks your map, then turns away.
                agent:
                    !exit
            ]],
            DIALOG_NO_CONVINCE_DEFENSIVE = [[
                player: 
                    I got a bounty from the Admiralty, you know.
                    What you're doing is against the rules.
                agent:
                    !cruel
                    $angryCruel
                    Hah! Whose rules? The switches'?
                    Don't give a flying fishcake. 
            ]],
            DIALOG_ATTACK_DEFENSIVE = [[
                player:
                    !fight
                    $angryHostile
                    Fine. Let's just skip to the interrogation, alright?
            ]],
            
            DIALOG_WON_FIGHT_DEAD = [[
                * You won't be learning anything more from {agent}.
            ]],
            
            DIALOG_WON_FIGHT_ALIVE = [[
                agent:
                    !injured
                    $scaredAnguished
                    You won, okay? I'll tell you. 
                player:
                    $angryThanks
                    Better be worth it. 
                    !forgetful
                    Think I pulled a muscle in my shoulder.
                * {agent} scribbles the location on your map in a scrawl weakened by your beating.
                *** You get the information you were looking for.
            ]],


            DIALOG_SELLING = [[
                agent:
                    !crossed
                    $miscPersuasive
                    You after that bounty, hunter?
                    That means you stand to profit. Seems only fair that I should, too.
            ]],
            DIALOG_DENY = [[
                agent:
                    $scaredFearful
                    I, uh, don't know who you're talking about.
                    So stop asking!
                * {agent} is clearly hiding something.
            ]],
            DIALOG_ATTACK_SELLING = [[
                player:
                    !fight
                    $angryDefault
                    How'd you like to profit from a learning experience, hm?
            ]],
            
            DIALOG_ATTACK_DENY = [[
                player:
                    !agree
                    Fair enough. Stand still. 
                    !fight
                    $angryCruel
                    I know a procedure that can jog your memory.
            ]],

            DIALOG_FRIENDLY = [[
                agent:
                    !cagey
                    $miscHushedUrgent
                    Hey, not too loud! If word gets out I'm friends with the hunter after {target}...
                player:
                    What, I'm not good for your reputation?
                agent:
                    !placate
                    You don't play well with all audiences. 
                    But I wouldn't do wrong by you, Sal. I'll tell you.
                * {agent} marks {target}'s location on your map.
                agent:
                    !bashful
                    $miscEmbarrassed
                    Just keep this between us, okay?
                player:
                    !wink 
                    It's our little secret.
            ]]
        }
    :Hub(function(cxt) 
        local assoc1 = cxt:GetAgent() == cxt.quest:GetCastMember("associate_1") and cxt.quest:IsActive("investigate_associate_1")
        local assoc2 = cxt:GetAgent() == cxt.quest:GetCastMember("associate_2") and cxt.quest:IsActive("investigate_associate_2")
        if assoc1 or assoc2 then
            cxt:Opt("OPT_ASK_ABOUT_TARGET")   
                :Dialog("DIALOG_INTRO")
                :Fn(function(cxt) 


                        if cxt:GetAgent():GetRelationship() >= RELATIONSHIP.LIKED then
                            cxt:Dialog("DIALOG_FRIENDLY")
                            cxt.quest:Complete(assoc1 and "investigate_associate_1" or "investigate_associate_2")
                        elseif cxt:GetAgent():GetRelationship() <= RELATIONSHIP.DISLIKED then
                            cxt:Dialog("DIALOG_DEFENSIVE")
                            cxt:Opt("OPT_CONVINCE_THEM_TO_TALK")
                                :Dialog("DIALOG_TRY_CONVINCE_DEFENSIVE")
                                :Negotiation{
                                    on_success = function(cxt) 
                                        cxt:Dialog("DIALOG_CONVINCE_DEFENSIVE")
                                        cxt.quest:Complete(assoc1 and "investigate_associate_1" or "investigate_associate_2")
                                        StateGraphUtil.AddEndOption(cxt)
                                    end,
                                    on_fail = function(cxt) 
                                        cxt:Dialog("DIALOG_NO_CONVINCE_DEFENSIVE")
                                    end,
                                }
                            cxt:Opt("OPT_BEAT_THEM_UP")
                                :Dialog("DIALOG_ATTACK_DEFENSIVE")
                                :Battle{
                                    on_win = on_win_associate_battle
                                }
                        elseif cxt:GetAgent():GetCombatStrength() < cxt.quest:GetCastMember("target"):GetCombatStrength() then
                            cxt:Dialog("DIALOG_AFRAID")
                            cxt:Opt("OPT_CONVINCE_THEM_TO_TALK")
                                :Dialog("DIALOG_TRY_CONVINCE_AFRAID")
                                :Negotiation{
                                    on_success = function(cxt) 
                                        cxt:Dialog("DIALOG_CONVINCE_AFRAID")
                                        cxt.quest:Complete(assoc1 and "investigate_associate_1" or "investigate_associate_2")
                                        StateGraphUtil.AddEndOption(cxt)
                                    end,
                                    on_fail = function(cxt) 
                                        cxt:Dialog("DIALOG_NO_CONVINCE_AFRAID")
                                    end,
                                }
                            cxt:Opt("OPT_BEAT_THEM_UP")
                                :Dialog("DIALOG_ATTACK_AFRAID")
                                :Battle{
                                    on_win = on_win_associate_battle
                                }
                        elseif cxt:GetAgent():GetRenown() >= cxt:GetAgent():GetCombatStrength() then
                            cxt:Dialog("DIALOG_SELLING")
                            
                            cxt:Opt("OPT_PAY_UP")
                                :DeliverMoney(BRIBE_MEDIUM)
                                :Dialog("DIALOG_PAID_UP")
                                :Fn(function(cxt)
                                    cxt.quest:Complete(assoc1 and "investigate_associate_1" or "investigate_associate_2")
                                    StateGraphUtil.AddEndOption(cxt)
                                end)

                            cxt:Opt("OPT_BEAT_THEM_UP")
                                :Dialog("DIALOG_ATTACK_SELLING")
                                :Battle{
                                    on_win = on_win_associate_battle
                                }

                        else
                            cxt:Dialog("DIALOG_DENY")

                            cxt:Opt("OPT_BEAT_THEM_UP")
                                :Dialog("DIALOG_ATTACK_DENY")
                                :Battle{
                                    on_win = on_win_associate_battle
                                }
                        end

                        StateGraphUtil.AddBackButton(cxt)
                    end)         
        end
    end)

QDEF:AddConvo(nil, "giver")
    :Loc{
        DIALOG_OFFER = [[
            player:
                $neutralGreeting
                I hear you're looking for hunters.
            agent: 
                $neutralDirect
                Always. There's never a shortage of bounties in Havaria.
                !dubious
            * {agent} appraises you, paying careful attention to the knives in your belt.
                Seems like you might be a good fit for one target in particular.
            player:
                $neutralDubious
                What did they do?
            agent:
                $neutralResigned
                !sigh
                Honestly, I don't even know. I just have a list to clear.
                Bring them in dead or alive. I'll pay more for alive if you can manage it, but I'm not picky.
        ]],
        DIALOG_TOOK_JOB = [[
            player:
                !interest
                $neutralDirect
                So where is this person supposed to be?
            agent:
                $neutralJoke
                That's supposed to be <i>your</> job, innit?
                !notebook
                According to the dossier, {target.heshe} is a known associate of {associate_1} and {associate_2}.
                I'd start by talking to one of them. See what you can scare up.
            player:
                !salute
                Aye aye, sarge.
            agent:
                $neutralWhatever
                That ain't my rank.
                !crossed
                But your enthusiasm is noted.
        ]]
    }
    :Hub(function(cxt)
        StateGraphUtil.OfferSideJob(cxt, nil, "DIALOG_OFFER", function(cxt) 
                cxt.quest:Activate("investigate_associate_1")
                cxt.quest:Activate("investigate_associate_2")
                cxt:Dialog("DIALOG_TOOK_JOB")
            end)
    end)
