
local LOCATION_DEF =
{
    id = "INFESTED_SMUGGLE_BEACH",
    name = "Secluded Beach",
    desc = "This beach is overrun with hostile wildlife.",
    map_tags = {"beach"},
    plax = "EXT_Beach_MurderBay_1",
    indoors = false,
}

if not Content.GetLocationContent(LOCATION_DEF.id) then
    Content.AddLocationContent(LOCATION_DEF)
end

local QDEF = QuestDef.Define{
        title = "Hit the Beach",
        qtype = QTYPE.SIDE,
        rank = {1, 4},
        icon = engine.asset.Texture("icons/quests/clear_out_smuggle_point.tex"),
        focus = QUEST_FOCUS.COMBAT,
        
        desc = [[{giver} wants you to clear out some beasties from a desolate beach so that the Jakes can use it for smuggling.]],
        get_fail_string = function(quest) 
            if quest:GetParameter("ran_away") then
                return "Ran away from the fight"
            else
                return "Didn't go to the beach in time"
            end
        end,
        
        on_destroy = function( quest )
            if quest:GetCastMember("the_beach") then
                TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("the_beach"))
            end
        end,
    }

    :AddCast{
        -- giver
        cast_id = "giver",
        provider = true,
        condition = QuestUtil.CanDoJakesGoonQuest,
    }

    :AddCast{
        cast_id = "the_beach",
        when = QWHEN.MANUAL,
        no_validation = true,
    }

    :AddObjective{
        id = "clear_out_the_beach",
        title = "Go to {the_beach#location}",
        desc = "Clear the hostile wildlife from {the_beach#location}.",
        mark = { "the_beach" },
        
        on_activate = function( quest)
            local location = Location( LOCATION_DEF.id )
            assert(location)
            TheGame:GetGameState():AddLocation(location)
            quest:AssignCastMember("the_beach", location )
        end,


        on_complete = function(quest) 
            quest:Activate("return_for_reward")
        end,

        on_fail = function(quest)
            quest:Fail()
        end,

    }

    :AddObjective{
        id = "return_for_reward",
        
        title = "Get reward from {giver}",
        desc = "You did the work, now get your due.",
        ready_for_completion = true,
        mark = { "giver" },
    }


QDEF:AddConvo("return_for_reward", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_TELL = "Tell {agent} the beach is clear",
        DIALOG_DONE = [[
            player:
                !permit
                $happyGreeting
                Your beach is clear. More fetid than ever, but monster-free all the same.
            agent:
                !thought
                $neutralThoughtful
                That's not bad. Whatever keeps the lollygaggers away.
                !crossed
                $neutralDirect
                We've got important business to conduct there. Which I expect you'll keep to yourself.
            player:
                !salute
                $happyThanks
                Hunter's honor.
            agent:
                Hrm.
        ]]
    }
    
    :Hub(function(cxt) 
            cxt:Opt("OPT_TELL")
                    :Dialog("DIALOG_DONE")
                    :CompleteQuest()
        end)
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("clear_out_the_beach")
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("the_beach") then
                return "STATE_BEACH_FIGHT"
            end
        end)
    :State("STATE_BEACH_FIGHT")
        :Loc{
            DIALOG_INTRO_DARK = [[
                * The beach might be an ugly scratch of shore, but at night the stars twinkle prettily all the same.
                player:
                    !left
                * And wouldn't you know it: the glint of eyes and fangs and whatever else might be out there, that's mighty lovely, too.
            ]],
            DIALOG_INTRO_BRIGHT = [[
                * The beach is easy to find, having exactly the unique pungency of {giver}'s description.
                * Other than a couple of broken crates stamped with Deltrean seals, there's not much here, as far as you can tell.
                player:
                    !left
                    !dubious
                    $neutralDubious
                    Hello? Monsters? You wanna come out and get stabbed?
                * There's a snuffling from within one of the shattered crates, which almost distracts you from the creatures cresting the bluff above you.
            ]],
            DIALOG_SHOW_UP = [[
                agent:
                    !right
                * You grip your weapon, pointy end outward. 
            ]],
            DIALOG_REMATCH = [[
                player:
                    !left
                * You're ready this time, and spoiling for a rematch.
                agent:
                    !right
                player:
                    !fight
            ]],
            DIALOG_LEFT = [[
                player:
                    !left
                * You scour the beach, but you come up empty. No more beasties, as far as you can tell. Maybe you scared them off.
                * If it's good enough for you, it'll have to be good enough for {agent}.

            ]],
            DIALOG_WON_FIGHT = [[
                agent:
                    !exit
                * You make good work of the monsters. {giver} didn't say nothing about cleaning up the mess, though. 
            ]],
            OPT_DEFEND = "Defend yourself",

        }

        :Fn(function(cxt) 
                local dark = TheGame:GetGameState():GetDayPhase() == DAY_PHASE.NIGHT
                
                
                if cxt.quest:GetParameter("monsters") then
                    local monsters = cxt.quest:GetParameter("monsters")
                    local found = false
                    for k,v in ipairs(monsters) do
                        if not v:IsDead() then
                            cxt.encounter:SetPrimaryCast(v)
                            cxt:Dialog("DIALOG_REMATCH")
                            found = true
                            break
                        end
                    end
                    if not found then
                        cxt:Dialog("DIALOG_LEFT")
                        cxt.quest:Complete("clear_out_the_beach")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                else
                    cxt:Dialog( dark and "DIALOG_INTRO_DARK" or "DIALOG_INTRO_BRIGHT")
                    local monsters = CreateCombatParty("MONSTERS", cxt.quest:GetRank(), cxt.location)
                    cxt.quest:SetParameter("monsters", monsters)
                    cxt.encounter:SetPrimaryCast(monsters[1])
                    cxt:Dialog("DIALOG_SHOW_UP")
                end
                
                cxt:Opt("OPT_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt) 
                            cxt:Dialog("DIALOG_WON_FIGHT")
                            cxt.quest:Complete("clear_out_the_beach")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_runaway = function(cxt)
                            cxt.quest:SetParameter("ran_away", true)
                            cxt.quest:Fail()
                        end,
                    }
            end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            player:
                !crossed
                $neutralDirect
                I hear you've got work for me. I'm here to help with that.
            agent:
                !interest
                $neutralThoughtful
                Huh. You certainly look like you can handle it. 
                There's a beach nearby—kinda ugly, with a sludge smell you can't find nowhere else.
                !cagey
                $neutralDubious
                Us Jakes use it for... stuff that ain't none of your concern. 
                Only it's been overrun with beasts.
            player:
                !happy
                $neutralDubious
                Is that a euphemism?
            agent:
                !placate
                $neutralTakenAback
                What? Naw, I mean, <i>actual</i> beasts. 
                We picked that place 'cause no one else wanted it, only it turns out it's just right for those monsters.
                !permit
                $neutralDirect
                Clear 'em out, and we'll consider the job done.
        ]]
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "clear_out_the_beach", "DIALOG_OFFER")
    end)


