local QDEF = QuestDef.Define{
        title = "Competing Bids",
        qtype = QTYPE.SIDE,
        --disabled = true,
        rank = {1, 5},
        desc = "{giver} wants you to track down and capture {target}, but has been cagey about the details.",
        icon = engine.asset.Texture("icons/quests/competing_bids.tex"),
        focus = QUEST_FOCUS.COMBAT,

        on_destroy = function( quest )
            if quest:GetCastMember("other_hunter") and quest:GetCastMember("other_hunter"):IsInPlayerParty() then
                quest:GetCastMember("other_hunter"):Dismiss()
            end

            if quest:GetCastMember("target") and quest:GetCastMember("target"):IsInPlayerParty() then
                quest:GetCastMember("target"):Dismiss()
            end
        end,

        on_init = function(quest) 
            if quest:GetCastMember("other_hunter"):GetFactionID() == "ADMIRALTY" then
                quest.param.hunter_admiralty = true
            elseif quest:GetCastMember("other_hunter"):GetFactionID() == "BANDITS" then
                quest.param.hunter_bandits = true
            elseif quest:GetCastMember("other_hunter"):GetFactionID() == "JAKES" then
                quest.param.hunter_jakes = true
            end

            if quest:GetCastMember("target"):GetFactionID() == "BILEBROKERS" then
                quest.param.target_bilebroker = true
            elseif quest:GetCastMember("target"):GetFactionID() == "RISE" then
                quest.param.target_rise = true
            elseif quest:GetCastMember("target"):GetFactionID() == "BOGGER" then
                quest.param.target_bogger = true
            end

            quest.param.hideout_type = quest.param.hideout_type or table.arraypick{"URBAN", "RURAL"}
            quest.param.is_city = quest.param.hideout_type == "URBAN"

        end
    }

    :AddOpinionEvents{
        killed_bystander_together =  
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "You accidentally killed an informant together",
        },
        killed_bystander_alone =  
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Killed the only lead in their bounty case",
        },
        
        gave_generous_cut =  
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "You gave them a generous cut from your shared adventure",
        },
        gave_normal_cut =  
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "You worked together to bring in a bounty",
        },
        gave_no_cut =  
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "You cheated them out of their cut after working with them to bring in a bounty",
        },
        turned_them_in =  
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Turned them in as a bounty",
        },

    }

    :AddLocationDefs{
        
        SAFEHOUSE = {
            name = "The Safehouse",
            map_tags = {"residence"},
            plax = "EXT_door_slums1",
        },
        
        CAVE = {
            name = "The Sheltered Cave",
            map_tags = {"cave"},
            plax = "EXT_Cave_Murderbay_1",
        },
    }    

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent, quest) 
            return agent:GetContentID() == "ADMIRALTY_CLERK"
        end,
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "ADMIRALTY_CLERK" ) )
        end,
    }


    --we don't know this person.
    :AddCast{
        cast_id = "other_hunter",
        cast_fn = function(quest, t)
            local def = quest.param.other_hunter_def
            if not def then
                --higher ranked quests have higher ranked hunters
                if quest:GetRank() <= 2 then
                    def = table.arraypick{"ADMIRALTY_GOON", "JAKES_RUNNER", "BANDIT_GOON2"}
                else
                    def = table.arraypick{"ADMIRALTY_PATROL_LEADER", "JAKES_SMUGGLER", "BANDIT_RAIDER"}
                end
            end
            table.insert( t, quest:CreateSkinnedAgent( def ))
        end,
    }


    --we do not know this person.
    :AddCast{
        cast_id = "bystander",
        cast_fn = function(quest, t)
            local def = quest.param.bystander_def
            local opts = quest:GetRank() <= 2 and {"HEAVY_LABORER", "LABORER"} or {"FOREMAN", "WEALTHY_MERCHANT"}
            table.insert( t, quest:CreateSkinnedAgent( table.arraypick( opts )))
        end,
    }

    :AddLocationCast{
        cast_id = "hiding_spot",
        when = QWHEN.MANUAL,
        cast_fn = function(quest, t)
            table.insert( t, quest:SpawnTempLocation( quest.param.hideout_type == "RURAL" and "CAVE" or "SAFEHOUSE"))
        end,
    }

    :AddLocationCast{
        cast_id = "noodle_shop",
        cast_fn = function(quest, t)
            table.insert(t, TheGame:GetGameState():GetLocation("MURDERBAY_NOODLE_SHOP"))
        end,
    
    }

    --we don't know this person, and they are not from around here    
    :AddCast{
        cast_id = "target",
        cast_fn = function(quest, t) 
            local def = quest.param.target_def or table.arraypick{"CHEMIST", "RISE_PAMPHLETEER"}
            table.insert( t, quest:CreateSkinnedAgent( def ))
        end,

        events = {
            agent_retired = function(quest, agent) 
                if quest:IsActive("take_target_in") then 
                    quest:Cancel("take_target_in")
                    quest:Activate("return_for_payment")
                end
            end,
        }

    }

    :AddObjective{
        id = "investigate_noodle_shop",
        title = "Investigate {noodle_shop}",
        desc = "Ask around at {noodle_shop} to see if anyone has a lead on {target}.",
        mark = { "noodle_shop", "other_hunter", "bystander"},
        
        on_complete = function(quest) 
            --quest:Activate("defend_the_store")
        end,

        is_in_hiding = function(quest, agent)
            if agent == quest:GetCastMember("target") then
                return true
            end
        end,

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("other_hunter"), location = quest:GetCastMember("noodle_shop")} )
            table.insert (t, { agent = quest:GetCastMember("bystander"), location = quest:GetCastMember("noodle_shop")} )
        end,

        negotiation_targets = {"bystander", "other_hunter"},
        combat_targets = {"bystander"},
    }
    

    :AddObjective{
        id = "go_to_hideout",
        title = "Investigate {target}'s hideout",
        desc = "{target} is rumored to be holed up nearby. See if {target.heshe} is still there.",
        mark = { "hiding_spot"},
        on_activate = function(quest)
            quest:AssignCastMember("hiding_spot")
        end,

        on_complete = function(quest) 
            --quest:Activate("defend_the_store")
        end,

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("target"), location = quest:GetCastMember("hiding_spot")} )
        end,
    }

    :AddObjective{
        id = "other_hunter_ambush",
    }

    :AddObjective{
        id = "take_target_in",
        title = "Return {target} to {giver}",
        desc = [[Turn over {target} for your reward.]],
        ready_for_completion = true,
        mark = { "giver" },
        --terminal = true,
    }
    :AddObjective{
        id = "return_for_payment",
        title = "Get paid for killing {target}",
        desc = [[{giver} will pay you for proof of killing {target}.]],
        ready_for_completion = true,
        mark = { "giver" },
        --terminal = true,
    }
    
    :AddObjective{
        id = "settle_with_other_hunter",
    }

    
---------------------------------------------------------------------------------------------------------------------------------------------------------------

--There is a lot of variation in the casting in this quest. I've set up dialog variables, so that you can do special lines for specific people or combinations thereof.
--[[
    other_hunter:
        {hunter_admiralty?
            I'm admiralty!
        }
        {hunter_jakes?
            I'm jakes!
        }
        {hunter_bandits?
            I'm bandit
        }
    target:
        {target_bilebroker?
            I'm a dredger!
        }
        {target_rise?
            I'm rise!
        }
        {target_bogger?
            I'm bogger!
        }

    player:

        {target_bogger and hunter_bandits?
            A bandit is hunting a bogger! 
        }
--]]


--This is the job offer.
QDEF:SideJobOffer("investigate_noodle_shop", 
    --the job offer
    [[
        player:
            You post a bounty?
        agent:
            !impressed
            Y-yeah. Wow. Didn't expect it to actually get any interest.
        player:
            !dubious
            $neutralDubious
            Well, I'm a hunter. It's my job. 
            You put up a posting, I respond to it, we talk pay...
        agent:
            Right. Right, of course. Yeah.
            I need you to take down {target}—{target.heshe}'s{target_bilebroker? one of them Roaloch-crawlin' weirdos}{target_bogger? some kind of cultist}{target_rise? a debt dodger}. 
    ]],
    
    --you take the job
    [[
        agent:
            Dead or alive. Whatever's easier.
            !shrug
            But, uh, you need to show me proof, else it's no deal, right? Don't want you claiming credit for someone else's work.
        player:
            !suspicious
            Someone else? Has another hunter already taken this bounty?
        agent:
            Maybe...? They didn't say they were a hunter, but...
            $neutralDubious
            That's fine, right? Two folks can go after the same bounty at once?
            !shrug
            I've never done this before. 
        player:
            !cruel
            It's fine. Just means they're going to be a mite disappointed when I scoop 'em.
        agent:
            !agree
            Sure, I guess. Makes no difference to me, as long as it gets done.
            Head on over to {noodle_shop}. Word is {target} likes the taste of their oshnu bile. 
    ]])


--You arrive at the noodleshop, and find another bounty hunter is already investigating! This is a quick non-interactive intro, to introduce the characters.
-- The player will 
QDEF:AddConvo("investigate_noodle_shop", nil)
    :ConfrontState("STATE_CONFRONT", function(cxt) return not cxt.quest.param.has_played_intro and cxt.location == cxt.quest:GetCastMember("noodle_shop") end)
    :Loc{
        DIALOG_INTRO = [[
            * You recognize the tension when you step into {noodle_shop}, like the oshnu mucus is coagulating in the air.
            * It's the feeling a room gets when a hunter is working it.
            bystander:
                !left
            other_hunter:
                !right
                !angry_accuse
                Now I've said it once, and I don't much like the idea of repeating myself.
                Where is {target}?
            bystander:
                !angry
                And <i>I've</i> said it a dozen times—shove off. 
            * It's clear {other_hunter} won't get anywhere with {bystander} at this rate. 
            player:
                !left
                !happy
                $happyCocky
                This looks like quite the impasse you've reached.
                Maybe take a breather, jog around the block. 
            other_hunter:
                !crossed
                $neutralDirect
                Yeah? And who might you be?
            player:
                A hunter, same as you.
                !thought
                Though I gotta say—it doesn't look like you're doing too well. 
            * The patrons, sensing trouble, bury their heads in their noodles. 
            * {other_hunter} watches you with a shrewd eye, keeping {other_hunter.hisher} distance for the time being.
        ]],
    }
    :Fn(function(cxt) 
        cxt.quest.param.has_played_intro = true
        cxt.quest:GetCastMember("bystander"):MoveToLocation(cxt.location)
        cxt.quest:GetCastMember("bystander"):SetLocationRole(CHARACTER_ROLES.PATRON)
        cxt:TalkTo("other_hunter")
        cxt:Dialog("DIALOG_INTRO")
        StateGraphUtil.AddEndOption(cxt)
    end)



-- Talk to the other hunter, maybe convince them to work together?
QDEF:AddConvo("investigate_noodle_shop", "other_hunter")
    :Loc{
        DIALOG_ATTRACT = [[
            agent:
                !cruel
                Competition, eh?
                Hrmph. Murder Bay has enough contracts without you needing to edge in on mine, don't it?
        ]],

        OPT_CONVINCE_WORK_TOGETHER = "Convince {agent} to work with you",
        DIALOG_CONVINCE_WORK_TOGETHER = [[
            player:
                !placate
                No need for that. 
                Look, you're having a tough time cracking this crowd.
                Why don't we work together? 
        ]],
        DIALOG_CONVINCE_WORK_TOGETHER_SUCCESS = [[
            player:
                $miscPersuasive
                We split the rewards, sure, but we also split the effort. 
            agent:
                !thought
                $neutralThoughtful
                Hrm. Ain't often we get to network in our line of work.
                !shrug
                $neutralWhatever
                What the Hesh. Sure. 
                First order of business, though, you gotta prove your worth. 
                !point
                I think that {bystander} knows something, but {bystander.heshe}'s not talking.
                Fix it, then we'll call this mutually beneficial.
        ]],
        DIALOG_CONVINCE_WORK_TOGETHER_FAILURE = [[
            player:
                !overthere
                Say, for seventy-thirty split—
            agent:
                Hah! Who taught you to negotiate, ya rube?
                !crossed
                Forget it. I'll be keeping <i>a hundred</i> percent of the profits, when I get the bounty.
        ]],

    }
    --you are not working together
    :Questions("QUESTIONS_NO_AGREEMENT", {
        "Ask about {target}",
        [[
            player:
                So, you a licensed hunter? Or just a hobbyist?
            agent:
                !hips
                What I am is none of your business.
            player:
                !dubious
                $neutralDubious
                That's literally not true. 
                Like I said: I'm a hunter. And I want this contract.
                $miscMocking
                Whether or not you're licensed just alters what I'm going to do about it.
            agent:
                Information is power. And I'm not inclined to give you any.
        ]]
    })

    -- you are working together
    :Questions("QUESTIONS_AGREEMENT", {
        "Ask about {target}",
        [[
            player:
                $neutralThoughtful
                You got a headstart on me. Any leads?
            agent:
                $neutralResigned
                Afraid not much. {bystander} knows something, I'm sure of that. But {bystander.heshe}'s clammed up tight.
                $miscPersuasive
                You pry {bystander.himher} open and I'll be mighty impressed.
        ]],
    })

    :AttractState("ATTRACT")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_ATTRACT")
        end)

    :Hub(function(cxt) 
        cxt:InsertQuestions(cxt.quest.param.working_with_other_hunter and "QUESTIONS_AGREEMENT" or "QUESTIONS_NO_AGREEMENT")

        cxt:Opt("OPT_CONVINCE_WORK_TOGETHER")
            :Dialog("DIALOG_CONVINCE_WORK_TOGETHER")
            :Negotiation{}
                :OnSuccess()
                    :Dialog("DIALOG_CONVINCE_WORK_TOGETHER_SUCCESS")
                    :Fn(function() 
                        cxt.quest.param.working_with_other_hunter = true
                        cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                    end)
                :OnFailure()
                    :Dialog("DIALOG_CONVINCE_WORK_TOGETHER_FAILURE")
    end)

--talk to the bystander. You're going to need to force them to talk, one way or another. It will be easier if you're working with the other hunter
QDEF:AddConvo("investigate_noodle_shop", "bystander")
    :Loc{
        DIALOG_ATTRACT = [[
            agent:
                $angryDefault
                Listen, I'll tell you what I told that jerk.
                I don't know nothing, and I don't want trouble.
        ]],
        OPT_QUESTION = "Ask about {target}",
        DIALOG_QUESTION = [[
            player:
                $miscPersuasive
                !interest
                Easy. I'm just looking for {target}. No reason this has to go down sour.
            agent:
                $neutralDirect
                !crossed
                Too late. I've had it up to here with you, and I just met you.
        ]],

        OPT_INTIMIDATE = "Intimidate {agent} to get the info",
        DIALOG_INTIMIDATE = [[
            {working_with_other_hunter?
                player:
                    !cruel
                    Feeling's mutual, trust me. 
                    Unfortunately, my friend and I aren't leaving until we're satisfied.
                    One way or another, information's gonna come outta you.
                    !angry_threaten
                    $angryCruel
                    Beaten or stirred—your choice.
            }
            {not working_with_other_hunter?
                player:
                    !cruel
                    Feeling's mutual, trust me.
                    Unfortunately for you, I'm much better at my job than that bozo.
                    !angry_threaten
                    $angryCruel
                    And I know what to do with folks who don't cooperate.
            }
        ]],
        DIALOG_INTIMIDATE_SUCCESS = [[
            player:
                !cruel
                $angryCruel
                So what'll it be? You clamp down on that tongue of yours too hard and you're likely to lose it.
            {working_with_other_hunter?
                agent:
                    !scared
                    $scaredDefault
                    Okay, okay. One of you was bad enough. {is_city? {target}'s hiding in the slums! | {target}'s holed up in a cave!}
                other_hunter:
                    !right
                    !impressed
                    Not bad, {player}. At least you can deliver. 
                    Now let's see you on the follow through, eh?
            }
            
            {not working_with_other_hunter?
                agent:
                    !scared
                    $scaredDefault
                    Y-you make a compelling argument, you know? 
                    I-I reckon I like you better than that other guy. {is_city? {target}'s moved to a safehouse! | {target}'s in a cave!}
                player:
                    !angry 
                    Keep it down!
                * Too late. {other_hunter} slips out of the shop, no doubt having heard {agent}'s change of heart.
                player:
                    !palm
                    Great. 
            }
        ]],
        DIALOG_INTIMIDATE_FAILURE = [[
            player:
                Let's just say if you don't talk, I'm gonna have to be mean.
            {working_with_other_hunter?
                agent:
                    !handwave
                    Go ahead. I could use a laugh.
                    Just so happens you two are a couple of clowns.
                other_hunter:
                    !right
                    !disappoint 
                    $neutralResigned
                    Smooth move, snail tracks.
                    Got any other bright ideas?
                agent:
                    !right
                    Bet not.
            }
            
            {not working_with_other_hunter?
                agent:
                    !dubious
                    $neutralDubious
                    No offense, pal, but that other hunter's got grit. 
                    If {other_hunter.heshe} couldn't scare me, you don't stand a chance.
            }
        ]],

        FIGHT_REASON_BEAT_UP = "Beat up {agent} for information",
        OPT_BEAT_UP = "Beat the info out of {agent}",
        DIALOG_BEAT_UP = [[
            player:
                !fight
                Alright, I guess we do this the hard way.

            {working_with_other_hunter?
                other_hunter:
                    !left
                    Lacks finesse, but I guess it gets results.
                    !fight
            }
        ]],
        DIALOG_FOUGHT_EVERYONE_DEAD = [[
            * You might have eliminated the competition, but your lead's gone cold. Literally.
        ]],
        DIALOG_FOUGHT_BYSTANDER_DEAD= [[
            * {bystander}'s gone cold, and so has any chance of finding {target}.
            {working_with_other_hunter?
                other_hunter:
                    !right
                    !angry 
                    $angryReluctant
                    Well, thanks for nothing, "partner".
                    Now we're <i>both</i> out of a pay day.
                    Next time, how about you leave the work to the professionals?
                    !spit
                * {other_hunter} stalks off, leaving you to contemplate your failings.
                    !exit
            }
            {not working_with_other_hunter?
                other_hunter:
                    !right
                    !angry
                    $angryDefault
                    What the Hesh? What'd you do that for?
                player:
                    !thought
                    Woops.
                other_hunter:
                    Yeah, right. "Woops" she says.
                    !angry_point
                    You did that on purpose. Just so I wouldn't beat you to the target.
                    Grow up.
                * {other_hunter} stalks off, leaving you to contemplate your failings.  
                    !exit
            }
        ]],
        DIALOG_FOUGHT_WON_ALL_ALIVE = [[
            bystander:
                !injured
            player:
                Now, about {target}...
            bystander:
                {target} is in {hiding_spot}! You win, all right?
                But you better actually bring {target.himher} in, 'cause I don't want the fallout from this.
            
            {working_with_other_hunter?
                other_hunter:
                    !right
                    !impressed
                    Huh. You're good in a fight, at least.
                    Means it should be easy to bring {target} in. 
                    Now let's go find out.
                    !exit
            }
            {not working_with_other_hunter?
                * Last you noticed, {other_hunter} was off to the side, watching you duke it out. 
                * Seems awful suspicious that {other_hunter.heshe}'s nowhere to be seen now.
            }

        ]]

    }
    :AttractState("ATTRACT"):Fn( function(cxt) 
        cxt:Dialog("DIALOG_ATTRACT")
    end )

    :Hub(function(cxt) 
        local helpers, hinders, noncombatants, allies = {}, {}, {}, {}
        
        
        if cxt.quest.param.working_with_other_hunter then
            table.insert(helpers, cxt.quest:GetCastMember("other_hunter"))
            table.insert(allies, cxt.quest:GetCastMember("other_hunter"))
        else
            table.insert(hinders, cxt.quest:GetCastMember("other_hunter") )
            table.insert(noncombatants, cxt.quest:GetCastMember("other_hunter"))
        end

        cxt:Question("OPT_QUESTION", "DIALOG_QUESTION")
        
        cxt:Opt("OPT_INTIMIDATE")
            :Dialog("DIALOG_INTIMIDATE")
            :Negotiation{
                flags = NEGOTIATION_FLAGS.INTIMIDATION,
                helpers = helpers, 
                hinders = hinders,
            }
                :OnSuccess()
                    :Fn(function() 
                        cxt.quest:Complete("investigate_noodle_shop")
                        cxt.quest:Activate("go_to_hideout")
                        cxt:Dialog("DIALOG_INTIMIDATE_SUCCESS")
                        if not cxt.quest.param.working_with_other_hunter then
                            cxt.quest:GetCastMember("other_hunter"):MoveToLimbo()
                        end
                        StateGraphUtil.AddEndOption(cxt)
                    end)
                    
                :OnFailure()
                    :Dialog("DIALOG_INTIMIDATE_FAILURE")



        cxt:Opt("OPT_BEAT_UP")
            :Dialog("DIALOG_BEAT_UP")
            :Battle{
                reason_txt = cxt:GetLocString("FIGHT_REASON_BEAT_UP"),
                allies = allies,
                noncombatants = noncombatants,
            }
                :OnWin()
                    :Fn(function() 
                        if cxt:GetAgent():IsDead() then
                            cxt.quest:Fail()
                            
                            if cxt.quest:GetCastMember("other_hunter"):IsDead() then
                                cxt:Dialog("DIALOG_FOUGHT_EVERYONE_DEAD")
                            else
                                cxt:Dialog("DIALOG_FOUGHT_BYSTANDER_DEAD")        
                                if cxt.quest.param.working_with_other_hunter then
                                    cxt.quest:GetCastMember("other_hunter"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("killed_bystander_together"))
                                else
                                    cxt.quest:GetCastMember("other_hunter"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("killed_bystander_alone"))
                                end
                                cxt.quest:GetCastMember("other_hunter"):MoveToLimbo()
                            end
                        else
                            cxt.quest:Complete("investigate_noodle_shop")
                            cxt.quest:Activate("go_to_hideout")

                            cxt:Dialog("DIALOG_FOUGHT_WON_ALL_ALIVE")
                            if not cxt.quest.param.working_with_other_hunter then
                                cxt.quest:GetCastMember("other_hunter"):MoveToLimbo()
                            end
                        end
                        StateGraphUtil.AddEndOption(cxt)

                    end)
    end)


--arrive at the hideout, and take the bounty, alive or dead. The bounty fight is going to be relatively easy. The other hunter is the real problem.
QDEF:AddConvo("go_to_hideout")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("hiding_spot") end)
    :SetLooping()
    :Loc{
        DIALOG_INTRO = [[
            {is_city? 
                * You find the address that {bystander} gave you.
            }
            {not is_city? 
                * You find the cave where {bystander} said it would be.
            }

            {working_with_other_hunter?
                player:
                    !left
                other_hunter:
                    !right
                    Here we are. You ready?
                    !exit
            }
            
            {not working_with_other_hunter?
                player:
                    !left
            }

            * You rap once on the door, and {target} emerges like a wary anemone. 
            target:
                !right
                !dubious
                $scaredDefault
                Who are you? Who sent you?
        ]],
        OPT_INTIMIDATE = "Convince {target} to come along peacefully",
        DIALOG_INTIMIDATE = [[
            player:
                !crossed
                $neutralDirect
                I'm pretty sure you already know, so I ain't gonna expend the effort of telling you.
                Now—you going to come along quietly?
        ]],
        DIALOG_INTIMIDATE_SUCCESS = [[
            agent:
                !scared
                Okay! I'll go with you. Please don't kill me!

            {working_with_other_hunter?
                other_hunter:
                    !right
                    !happy
                    $happyCocky
                    Nice work! Of course, two hunters is more daunting than one.
                    If only all my jobs were this easy. 
            }
        ]],
        DIALOG_INTIMIDATE_FAILURE = [[
            agent:
                Are <i>you</i> going to take me somewhere nice?
                I don't think so.
            player:
                The hard way, then.
        ]],

        OPT_ATTACK = "Attack {agent}",
        DIALOG_ATTACK = [[
            player:
                !fight
                Alright, let's do this!
        ]],
        DIALOG_ATTACKED = [[
            {dead?
                * {target} is dead. You take a token as proof.
            
                {working_with_other_hunter and not other_hunter_dead?
                    other_hunter:
                        !right
                        Well, the job's done. Let's go collect our pay.
                }
            }
            
            {not dead?

                target:
                    !injured
                    Stop! I'll come quiet!

                {working_with_other_hunter and not other_hunter_dead ?
                    other_hunter:
                        !right
                        Alright! Bonus pay! Let's take {target.himher} in.
                }
            }
        ]],

        DIALOG_AMBUSHED_BY_OTHER_HUNTER = [[
            other_hunter:
                !right
                A fine effort.
                !clap
                You're good at murdering. Not so good at covering your tracks.
                Now hand over the trophy and you won't meet the same fate.
        ]],

        DIALOG_DEFEND_SELF = [[
            player:
                You assume I am concerned with whether you track me or not.
                Let's dance.
        ]],

        DIALOG_DEFEND_SELF_WON = [[
            {not other_hunter_dead ?
                player:
                    Sorry for your wasted efforts, but at least you came up with your life.
                other_hunter:
                    !scared
                    Gah!
                    !exit
            }
            {other_hunter_dead ?
                player:
                    !sigh
                    Well that was entirely unnecessary.
            }
        ]],

        OPT_HAND_OVER_PROOF = "Hand over the proof",

        DIALOG_HAND_OVER_PROOF = [[
            player:
                !angry
                Have it. This job isn't worth any more bloodshed.
            other_hunter:
                !take
                I am glad we can come to this agreement.
                See you around, killer.
        ]],


        OPT_ATTACK_OTHER_HUNTER = "Attack {other_hunter}",
        DIALOG_ATTACK_OTHER_HUNTER = [[
            player:
                !fight
                Actually...
        ]],
        DIALOG_ATTACK_OTHER_HUNTER_WON = [[
            {dead?
                * {other_hunter} is dead. You get to keep all of the sweet, sweet money.
            }
            {not dead?
                other_hunter:
                    !injured
                    You're a rat bastard, {player}!
                player:
                    Get out of here before I finish you off!
                other_hunter:
                    !exit
            }
        ]],
        OPT_HEAD_OUT = "Leave with {other_hunter}", 
        DIALOG_HEAD_OUT = [[
            player:
                !agree
                Let's go.
        ]],
        FIGHT_REASON = "Kill {agent} or take {agent.himher} captive.",
        FIGHT_REASON_HUNTER = "Defend yourself from {other_hunter}.",
        FIGHT_REASON_HUNTER_BETRAY = "Betray {other_hunter}."
    }
    :Fn(function(cxt) 
        local allies = {}
        if cxt.quest.param.working_with_other_hunter then
            table.insert(allies, cxt.quest:GetCastMember("other_hunter"))
        end

        if cxt:FirstLoop() then
            cxt.quest:Complete("go_to_hideout")
            cxt:TalkTo("target")
            cxt:Dialog("DIALOG_INTRO")
        end

        cxt:Opt("OPT_INTIMIDATE")
            :Dialog("DIALOG_INTIMIDATE")
            :Negotiation{flags = NEGOTIATION_FLAGS.INTIMIDATION}
                :OnSuccess()
                    :Fn(function() 
                        cxt:Dialog("DIALOG_INTIMIDATE_SUCCESS")
                        cxt.quest:Activate("take_target_in")
                        cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.CAPTIVE)
                        if not cxt.quest.param.working_with_other_hunter then
                            cxt.quest:Activate("other_hunter_ambush")
                        end

                        StateGraphUtil.AddLeaveLocation(cxt)                        
                    end)
                :OnFailure()
                    :Dialog("DIALOG_INTIMIDATE_FAILURE")

        cxt:Opt("OPT_ATTACK")
            :Dialog("DIALOG_ATTACK")
            :Battle{
                reason_txt = cxt:GetLocString("FIGHT_REASON"),
                allies = allies,
            }
                :OnWin()
                    :Fn(function() 
                        cxt.enc.scratch.other_hunter_dead = cxt.quest:GetCastMember("other_hunter"):IsDead()
                        cxt:Dialog("DIALOG_ATTACKED")
                        

                        --target is dead
                        if cxt:GetAgent():IsDead() then
                            cxt.quest:Activate("return_for_payment")
                            
                            --you worked together
                            if cxt.quest.param.working_with_other_hunter then

                                --other hunter dead
                                if cxt.quest:GetCastMember("other_hunter"):IsDead() then
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                else 
                                    --other hunter alive. you can betray them here if you want.
                                    cxt:TalkTo("other_hunter")
                                    
                                    cxt:Opt("OPT_ATTACK_OTHER_HUNTER")
                                        :Dialog("DIALOG_ATTACK_OTHER_HUNTER")
                                        :ReceiveOpinion(OPINION.BETRAYED)
                                        :Battle{
                                            reason_txt = cxt:GetLocString("FIGHT_REASON_HUNTER_BETRAY")
                                        }
                                            :OnWin()
                                                :Dialog("DIALOG_ATTACK_OTHER_HUNTER_WON")
                                                :Travel()

                                    cxt:Opt("OPT_HEAD_OUT")
                                        :Dialog("DIALOG_HEAD_OUT")
                                        :Travel()
                                end
                            --you are not working together. Get jumped by other hunter
                            else
                                cxt:Dialog("DIALOG_AMBUSHED_BY_OTHER_HUNTER")
                                cxt:TalkTo("other_hunter")
                                
                                cxt:Opt("OPT_DEFEND")
                                    :Dialog("DIALOG_DEFEND_SELF")
                                    :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        reason_txt = cxt:GetLocString("FIGHT_REASON_HUNTER")
                                    }
                                        :OnWin()
                                            :Fn( function() cxt.enc.scratch.other_hunter_dead = cxt.quest:GetCastMember("other_hunter"):IsDead() end )
                                            :Dialog( "DIALOG_DEFEND_SELF_WON" )
                                            :Travel()

                                cxt:Opt("OPT_HAND_OVER_PROOF")
                                    :Dialog("DIALOG_HAND_OVER_PROOF")
                                    :Fn(function() 
                                        cxt.quest:Fail() 
                                        cxt.quest:GetCastMember("other_hunter"):MoveToLimbo()
                                    end)
                                    :Travel()
                            end

                        else
                            if not cxt.quest.param.working_with_other_hunter then
                                cxt.quest:Activate("other_hunter_ambush")
                            end
                            cxt.quest:Activate("take_target_in")
                            cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.CAPTIVE)
                            StateGraphUtil.AddLeaveLocation(cxt)                        
                        end
                    end)
    end)


-- you have not been working with the other hunter, and they ambush you on your way to turn in the live bounty
QDEF:AddConvo( "other_hunter_ambush" )
    :ConfrontState("CONFRONT", function(cxt) return cxt.location:HasTag("in_transit") end)
    :Loc{
        DIALOG_INTRO = [[
            * You know from experience that there's a twist in the air. You're being followed.
            player:
                !left
                You can just come on out, now, {other_hunter}.
            other_hunter:
                !right
                Thanks for the elbow grease, {player}. I was fresh out. 
                I'll be taking {target.himher} now.
        ]],
        OPT_DEFEND = "Defend your bounty",
        DIALOG_DEFEND = [[
            player:
                !fight
                Not likely.
        ]],
        DIALOG_DEFEND_WON = [[
            {dead?
                * {other_hunter} is dead. The bounty is all yours now.
            }
            {not dead?
                other_hunter:
                    !right
                    You know what, you can keep it.
                player:
                    Generous of you. 
            }
        ]],

        OPT_NEGOTIATE_PAYMENT = "Negotiate a payoff",
        DIALOG_NEGOTIATE_PAYMENT = [[
            player:
                It was your lead, but I did all the work.
                I'll hand over {target} if you cough up enough shills.
        ]],
        DIALOG_NEGOTIATE_PAYMENT_FAILURE = [[
            agent:
                Nah, I'd rather keep all the money for myself.
                You understand, of course?
        ]],

        DIALOG_NEGOTIATE_PAYMENT_SUCCESS = [[
            agent:
                Fine. I'll give you a cut.
                It's cheaper than a fight.
        ]],
        OPT_TAKE_CUT = "Take the money",
        DIALOG_TAKE_CUT = [[
            player:
                I'll take it.
                !take
            agent:
                !give
                Now, {target}. Come with me.
            target:
                !left
                !scared
                Oh no...
                !exit
            agent:
                !exit

            * You count your new shills as {other_hunter} leads {target} back to {giver}.
        ]],

        OPT_REFUSE_CUT = "Refuse the deal",
        DIALOG_REFUSE_CUT = [[
            player:
                No. Not worth it.
            agent:
                Why are you wasting my time with this? Hand over {target}. Now.
        ]],

        OPT_HAND_OVER_BOUNTY = "Hand over {target}",
        DIALOG_HAND_OVER_BOUNTY = [[
            player:
                Just take {target.himher}.
            agent:
                Easiest bounty I ever hunted!
            target:
                !left
                Oh no!
                !exit
            agent:
                !exit
            * You watch your payday walk away.
        ]],
        FIGHT_REASON_DEFEND = "Defend yourself from {other_hunter}!"

    }
    :Fn(function(cxt) 
        cxt.quest:Complete("other_hunter_ambush")
        cxt:TalkTo("other_hunter")
        cxt:Dialog("DIALOG_INTRO")
        local offer = math.round( QuestUtil.CalculateRewardValue(cxt.quest)*.4 )

        cxt:RunLoop(function() 
            cxt:Opt("OPT_NEGOTIATE_PAYMENT")
                :Dialog("DIALOG_NEGOTIATE_PAYMENT")
                :Negotiation{}
                    :OnSuccess()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_NEGOTIATE_PAYMENT_SUCCESS")
                            cxt:Opt("OPT_TAKE_CUT")
                                :ReceiveMoney(offer)
                                :FailQuest()
                                :Travel()

                            cxt:Opt("OPT_REFUSE_CUT")
                                :Dialog("DIALOG_REFUSE_CUT")

                        end)
                    :OnFailure()
                        :Dialog("DIALOG_NEGOTIATE_PAYMENT_FAILURE")


            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    reason_txt = cxt:GetLocString("FIGHT_REASON_DEFEND"),
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    noncombatants = {cxt.quest:GetCastMember("target")},
                }
                    :OnWin()
                        :Dialog("DIALOG_DEFEND_WON")
                        :Travel()
            
            cxt:Opt("OPT_HAND_OVER_BOUNTY") 
                :Dialog("DIALOG_HAND_OVER_BOUNTY")
                :FailQuest()
                :Travel()
        end)
    end)


--
QDEF:AddConvo("settle_with_other_hunter")
    :ConfrontState("STATE_CONFRONT")  
        :Loc{
            DIALOG_PAYMENT_DETAILS = [[
                player:
                    !left
                other_hunter:
                    !right
                    Alright, {player}. Where's my share?
            ]],

            OPT_GIVE_LARGE_SHARE = "Give {agent} a generous share",
            DIALOG_GIVE_LARGE_SHARE = [[
                player:
                    Here. You earned it.
                    !give
                agent:
                    !take
                    $happyThanks
                    Wow, that's more than I expected.
                    !happy
                    You're all right, {player}.
                    I'll see you around, huh? 
                    !exit
            ]],

            OPT_GIVE_SMALL_SHARE= "Give {agent} a fair share",
            DIALOG_GIVE_SMALL_SHARE = [[
                player:
                    Here. This is fair.
                    !give
                agent:
                    $neutralThanks
                    Yeah. And not a shill more.
                    Until next time, {player}.
                    !exit
            ]],
            
            OPT_KEEP_ALL_THE_MONEY = "Keep all of the money",
            DIALOG_KEEP_ALL_THE_MONEY = [[
                player:
                    I think I'll just keep all of it.
                    !cruel
                    $miscMocking
                    Ha!
                agent:
                    !angry
                    You cheating—
                    !angry_accuse
                    You'll regret this!
                    !exit
            ]],        
        }
        :Fn(function(cxt) 
            cxt:TalkTo("other_hunter")
            cxt:Dialog("DIALOG_PAYMENT_DETAILS")

            cxt:Opt("OPT_GIVE_LARGE_SHARE")
                :Dialog("DIALOG_GIVE_LARGE_SHARE")
                :DeliverMoney(math.floor( cxt.quest.param.split_amt*.67) )
                :ReceiveOpinion("gave_generous_cut")
                :Fn(function() cxt.quest:GetCastMember("other_hunter"):MoveToLimbo() StateGraphUtil.AddEndOption(cxt) cxt.quest:Complete() end)
            
            cxt:Opt("OPT_GIVE_SMALL_SHARE")
                :Dialog("DIALOG_GIVE_SMALL_SHARE")
                :DeliverMoney(math.floor( cxt.quest.param.split_amt*.33) )
                :ReceiveOpinion("gave_normal_cut")
                :Fn(function() cxt.quest:GetCastMember("other_hunter"):MoveToLimbo() StateGraphUtil.AddEndOption(cxt) cxt.quest:Complete() end)
            
            cxt:Opt("OPT_KEEP_ALL_THE_MONEY")
                :Dialog("DIALOG_KEEP_ALL_THE_MONEY")
                :ReceiveOpinion("gave_no_cut")
                :Fn(function() cxt.quest:GetCastMember("other_hunter"):MoveToLimbo() StateGraphUtil.AddEndOption(cxt) cxt.quest:Complete() end)

        end)


-- Bring back proof of death.
QDEF:AddConvo("return_for_payment", "giver")
    :Loc{
        DIALOG_ATTRACT = [[
            agent:
                You're back.
        ]],

        OPT_CLAIM_PAYMENT = "Claim your bounty",
        DIALOG_CLAIM_PAYMENT = [[
            player:
                I tracked down {target}.
            agent:
                ...Where is {target.heshe} now?
            player:
                Dead!
            agent:
                Eh. Good enough.
        ]],
        DIALOG_CLAIM_PAYMENT_WITH_OTHER_HUNTER = [[
            player:
                We tracked down {target}.
            other_hunter:
                !left
                And killed {target.himher} dead!
            player:
                !left
            agent:
                Aw, ain't that cute. You don't often see grifters working together.
                Here's your payment. Sort out the details yourselves.
        ]],
    }
    :AttractState("ATTRACT", function(cxt) 
        cxt:Dialog("DIALOG_ATTRACT")
    end)

    :Hub(function(cxt) 
        if cxt.quest:GetCastMember("other_hunter"):IsInPlayerParty() then
            cxt:Opt("OPT_CLAIM_PAYMENT")
                :SetQuestMark()
                :Dialog("DIALOG_CLAIM_PAYMENT_WITH_OTHER_HUNTER")
                :Fn(function() 
                    cxt.quest.param.split_amt = math.round( QuestUtil.CalculateRewardValue(cxt.quest) )
                    cxt.quest:Activate("settle_with_other_hunter")
                    ConvoUtil.GiveQuestRewards(cxt)
                    StateGraphUtil.AddEndOption(cxt)
                end) 
        else
            cxt:Opt("OPT_CLAIM_PAYMENT")
                :SetQuestMark()
                :Dialog("DIALOG_CLAIM_PAYMENT")
                :Fn(function() 
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                    StateGraphUtil.AddEndOption(cxt)
                end) 
        end
    end)


QDEF:AddConvo("take_target_in", "giver")
    :Loc{
        DIALOG_ATTRACT = [[
            agent:
                Ah. I see you have {target} with you! Good!
        ]],

        OPT_CLAIM_PAYMENT = "Claim your bounty",
        DIALOG_CLAIM_PAYMENT = [[
            player:
                Here's {target}.
            agent:
                Excellent!
            target:
                !right
                Eat me, pig!
            agent:
                !right
                Charming.
                Thank you for your service. Here's your pay.
                !give
        ]],
        DIALOG_CLAIM_PAYMENT_WITH_OTHER_HUNTER = [[
            player:
                We have a delivery for you.
            other_hunter:
                !left
                !eureka
                A special delivery!
            player:
                !left
                !embarrassed
                Er, yeah. What {other_hunter} said.
            agent:
                !right
                !interest
                Hey, that's neat. You don't often see grifters working together.
                'Course, I'm only paying the once, mind. Sort out the details yourselves.
        ]],
    }
    :AttractState("ATTRACT")
        :Fn( function( cxt )
            cxt:Dialog("DIALOG_ATTRACT")
        end)

    :Hub(function(cxt) 
        if cxt.quest:GetCastMember("other_hunter"):IsInPlayerParty() then
            cxt:Opt("OPT_CLAIM_PAYMENT")
                :SetQuestMark()
                :Dialog("DIALOG_CLAIM_PAYMENT_WITH_OTHER_HUNTER")
                :Fn(function() 
                    cxt.quest.param.split_amt = math.round( QuestUtil.CalculateRewardValue(cxt.quest) )
                    cxt.quest:Activate("settle_with_other_hunter")
                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:GetCastMember("target"):Retire()
                end) 
                :ReceiveOpinion("turned_them_in", nil, "target")
                :Fn(function()
                    StateGraphUtil.AddEndOption(cxt)
                end)

        else
            cxt:Opt("OPT_CLAIM_PAYMENT")
                :SetQuestMark()
                :Dialog("DIALOG_CLAIM_PAYMENT")
                :Fn(function() 
                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:GetCastMember("target"):Retire()
                    cxt.quest:Complete() -- MSUT COME LAST, can trigger AdvanceTime.
                end) 
                :ReceiveOpinion("turned_them_in", nil, "target")
                :Fn(function()
                    StateGraphUtil.AddEndOption(cxt)
                end)
        end
    end)

QDEF:AddDebugOption("target_def", {"CHEMIST", "BOGGER_CULTIVATOR", "RISE_PAMPHLETEER"})
QDEF:AddDebugOption("other_hunter_def", {"ADMIRALTY_GOON", "JAKES_RUNNER", "BANDIT_GOON2", "ADMIRALTY_PATROL_LEADER", "JAKES_SMUGGLER", "BANDIT_RAIDER"})
QDEF:AddDebugOption("hideout_type", {"URBAN", "RURAL"})


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 50 },
        
    }
}
