local QDEF = QuestDef.Define{
    title = "Debt Collection",
    qtype = QTYPE.SIDE,
    rank = {1, 5},
    desc = [[{giver} lent {target} {debt_amount#money}. Retrieve the money if you can, or "make an example" of {target} to make {giver} happy.]],
    icon = engine.asset.Texture("icons/quests/debt_collection.tex"),
    --weight_fn = function( quest ) return OPPORTUNITY.HIGH_SPAWN_WEIGHT*100 end,
    focus = QUEST_FOCUS.BALANCED,

    avoids = 
    {
        --giver and target will avoid each other during all objectives
        {"giver", "target"},
        --giver and target will avoid each other during confront_target
        --{"giver", "target", {"confront_target"}}
    },

    on_init = function( quest )
        local money = quest:GetParameter("rewards")
        -- the debt is for between 185% and 210% of the reward value 
        local random_var = math.random(185, 210) * 0.01
        local debt_amount = math.ceil(money * random_var)

        quest:SetParameter("debt_amount", debt_amount)
        if quest.param.has_money == nil then
            quest:SetParameter("has_money", math.random() > 0.5)
        end
        return true
    end,
    }

    :AddCast{
        -- giver
        cast_id = "giver",
        provider = true,
    }

    :AddCast{
        -- target
        cast_id = "target",
        condition = function(other_agent, quest)
            
            if not AgentUtil.IsCoworker(other_agent, quest:GetCastMember("giver")) and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), other_agent, true, true) then
                local giver = quest:GetCastAgent( "giver" )
                if giver:GetRelationship(other_agent) > RELATIONSHIP.NEUTRAL then
                    return false
                end

                return true
            end
        end,
        
        events =
        {
            agent_retired = function( quest, agent )
                if quest:GetParameter("has_money") then
                    quest:Activate("return_the_debt")
                else
                    quest:Activate("report_beating")
                end
            end,
        }
    }

    :AddObjective{

        id = "confront_target",
        title = "Confront {target} about their debt",
        desc = "Find {target} and deliver an ultimatum: pay up, or get ready to get hurt",
        mark = { "target" },
        on_activate = function( quest )
            local target = quest:GetCastAgent( 'target' )
            if quest:GetParameter("has_money") then
                target:DeltaMoney( quest:GetParameter( "debt_amount" ))    
            end
        end,
        is_negotiation_target = function( self, agent )
            if self.param.has_money then
                return agent == self:GetCastAgent("target")
            end
        end,
        combat_targets = {"target"},
    }



    :AddObjective{
        id = "pay_back_debt",
        title = "(Optional) Pay off {target}'s debt to {giver}",
        desc = "You can choose to pay off {target}'s debt out of the goodness of your rational self-interest.",
        mark = { "giver" },
    }

    :AddObjective{
        id = "tell_target_settled",
        title = "Tell {target} that you settled {target.hisher} debt",
        desc = "Make sure that {target} knows what you did for {target.himher}.",
        mark = { "target" },
        ready_for_completion = true,
        terminal = true,
    }

    
    :AddObjective{
        -- return the debt
        id = "report_beating",
        title = "Tell {giver} about beating {target}",
        desc = "You didn't get the money, but at least you scared the oshnu dung out of {target}.",
        ready_for_completion = true,
        mark = { "giver" },
        terminal = true,

        on_activate = function (quest)
            if quest:IsActive("pay_back_debt") then
                quest:Cancel( "pay_back_debt" )
            end
            if quest:IsActive( "confront_target" ) then
                quest:Complete( "confront_target" )
            end
        end,
    }

    :AddObjective{
        id = "return_the_debt",
        title = "Return {debt_amount#money} to {giver}",
        desc = "You have {giver}'s money. You better return it before they get suspicious.",
        ready_for_completion = true,
        negotiation_targets = { "giver" },
        mark = { "giver" },
        on_activate = function (quest)
            
            if quest:IsActive("pay_back_debt") then
                quest:Cancel( "pay_back_debt" )
            end
            
            if quest:IsActive("report_beating") then
                quest:Cancel( "report_beating" )
            end

            if quest:IsActive( "confront_target" ) then
                quest:Complete( "confront_target" )
            end
        end,
    }


QDEF:AddConvo("tell_target_settled", "target")
    :Priority( CONVO_PRIORITY_HIGHEST )
    :Loc{
        OPT_TELL = "Tell {target} {target.hisher} debt is settled",
        DIAL_DONE = [[
            player:
                $happyGreeting
                !greeting
                You're off the hook, {target}. I paid your debts for you.
            target:
                !dubious
                $happyAmazed
                You serious? What kind of hunter <i>settles</i> debts?
            player:
                !shrug
                $happyDefault
                I learned from the best. Besides, maybe I like you.
            target:
                !interest
                $happyAmazed
                Maybe you do. I won't take it for granted. 
                If I can repay it, I will. 
        ]],
    }
    :Hub(function(cxt)
        
        cxt:Opt("OPT_TELL")
            :Dialog("DIAL_DONE")
            :ReceiveOpinion(OPINION.SETTLED_DEBT)
            :CompleteQuest()
    end)


QDEF:AddConvo( "confront_target", "target")
    :Priority( CONVO_PRIORITY_HIGHEST )
    :Loc{
        OPT_COLLECT_DEBT = "Confront {target} about {target.hisher} debt to {giver}",
        OPT_WARN_DEBT = "Warn {target} about shirking their debt to {giver}",

        DIALOG_NOPE = [[
            player:
                player:
                !interest
                $neutralDirect
                You know {giver}? {giver.HeShe}'s hired me to collect. 
                !shrug
                $neutralWhatever
                Personally, I'd like this to go easy. But I'll leave it up to you. 
            Agent:
                !spit
                $angrySeething
                Yeah? Well I <i>told</i> {giver} {giver.heshe} can shuck it. 
                I already paid {giver.himher} twice what {giver.heshe} lent me, and I'm sick of {giver.hisher} 'added interest'.
                !angry_accuse
                $angryHostile
                Now {giver.heshe}'s paying bounty hunters just to squeeze me for more? Forget it. 
        ]],
        --REQ_NO_MONEY = "{agent} doesn't have any money",
        OPT_NEGOTIATIE_FOR_IT = "Convince {agent} to hand it over easy",
        OPT_ATTACK_FOR_IT = "Beat {agent} to convince them to hand it over",

        DIALOG_HAVE_MONEY = [[
            player:
                !interest
                $neutralGreeting
                You know {giver}? {giver.HeShe}'s hired me to collect. 
                !shrug
                $neutralWhatever
                Personally, I'd like this to go easy. But I'll leave it up to you. 
            agent:
                !dubious
                $neutralDubious
                I have it. I was planning on takin' it to {giver} direct as soon as I got off shift. 
                Why should I give it to <i>you</i>? What guarantee do I have that you're on the level?
        ]],
        DIALOG_INTIMIDATED_TAKE_IT = [[
            player:
                !angry_shrug
                $angryDefault
                I was given two instructions, {agent}: get the shills, or leave you broken. 
                You don't wanna hand it over? That's your prerogative. But then I fall back on option number two. 
                !cruel
                $angryCruel
                So if that's what you want...
            agent:
                !flinch
                $scaredFearful
                It's not! Fine. Take it!
                !give
                $scaredStammering
                B-but I want a receipt. 
            player:
                !take
                $angryThanks
                Sure, pal. No problem. I'm a professional, after all. 

        ]],
        DIALOG_HAVE_MONEY_LOVED_RELATIONSHIP = [[
            player:
                !interest
                $neutralGreeting
                Hey, {agent}. You know {giver}?
            agent:
                !facepalm
                $neutralResigned
                Ugh, do I ever. I borrowed money from {giver.himher} months ago and the interest has been bleeding me dry.
            player:
                !shrug
                $neutralResigned
                That's why I'm here. {giver.HeShe} wants me to collect. Sorry.
                !neutral
            agent:
                $neutralResigned
                Naw, that's not your fault. Easier to hand it over to you anyway. 
                !give
                This is the last of the payments—I should be in the clear after this.
            player:
                !take
                $miscRelieved
                Well, happy day.

        ]],

        DIALOG_NO_MONEY_LOVED_RELATIONSHIP = [[
            player:
                !interest
                $neutralGreeting
                Hey, {agent}. You know {giver}?
            agent:
                !facepalm
                $neutralResigned
                Ugh, do I ever. I borrowed money from {giver.himher} months ago and the interest has been bleeding me dry.
            player:
                !shrug
                $neutralResigned
                That's why I'm here. {giver.HeShe} wants me to collect. Sorry.
                !neutral    
            agent:
                !disappoint
                $miscDepressed
                Aw, Hesh. If I had the money, I'd pay it! But I don't!
                !scared_shrug
                $neutralResigned
                You believe me, right?
        ]],

        DIALOG_OFFERED_TO_PAY = [[
            agent: 
                !overthere
                $happyTakenAback
                If you <i>do</i> pay {giver} back for me, I'd—well, I dunno. I don't think I've ever had a friend that good.
            player:
                !thumb
                $happyCocky
                Watch and weep, bud. I'm about to do you one Hesh of a favor. 
        ]],

        DIALOG_KILLED_THEM = [[
            * Sure enough, you find the shills in {agent}'s pockets when you check. {giver} will still be expecting {giver.hisher} money.
        ]],
        DIALOG_DONT_HAVE_IT = [[
            player:
                !interest
                $neutralGreeting
                You know {giver}? {giver.HeShe}'s hired me to collect. 
                !shrug
                $neutralWhatever
                Personally, I'd like this to go easy. But I'll leave it up to you. 
            agent:
                !angry_shrug
                $angryPatienceLost
                Yeah? Well I'll make it as easy as I can: I ain't got it. And {giver} sending hired goons to breathe down my neck won't change that.
        ]],
        OPT_OFFER_TO_PAY_IT = "Offer to pay the debt off for them ({1#money})",
        DIALOG_PAY_BACK = [[
            player:
                !permit
                $miscPersuasive
                Look, I'll spot you, all right?
                !shrug
                $neutralWhatever
                There's no reason you gotta bleed for {giver}'s return.
            agent:
                !dubious
                $neutralDubious
                Wha—really? Forgive me for bein' skeptical, but I'll believe it when I see it. 
        ]],

        DIALOG_PAY_BACK_LOVED = [[
            player:
                !permit
                $happyDefault
                Relax. I'll spot you.
            agent:
                !sigh 
                $miscRelieved
                I don't know what I did to deserve you, bud. 
                But I'll make it up to you. Just watch. 
            player:
                !wink
                $miscPersuasive
                Yeah, yeah. I'll hold you to that.
        ]],
        OPT_ATTACK_PUNISHMENT = "Rough {target.himher} up",
        DIALOG_BEATEN = [[
            agent:
                !injured
            player:
                $angryCruel
                {giver} sends {giver.hisher} regards.
            agent:
                !injured
                $scaredAnguished
                You've done what you came to do. Run off to your master and tell {giver.himher} the job's done, already.
                !exit
            * {agent} limps away, leaving you to enjoy the satisfaction of a job well done.
        ]],
        DIALOG_KILLED_THEM_NO_MONEY = [[
            * {giver} will want to know about what happened here.
        ]],
        DIALOG_HAND_OVER = [[
            player:
                $neutralResigned
                Just pay up and this'll all go away.
            agent:
                !give
                $angrySeething
                Fine! Take it, and give it back to {giver}. I just want this to be over.
            player:
                !take
                $happyThanks
                See? Nothing to it. 
        ]],
        DIALOG_NO_DONT_PAY_IT = [[
            player:
                !permit
                $happyDefault
                Look, I'll spot you, all right?
                !shrug
                $neutralWhatever
                There's no reason you gotta bleed for {giver}'s return.
            agent:
                !dubious
                $neutralDubious
                What? Then {giver.heshe}'s just getting what {giver.heshe} wants out of <i>your</i> pocket instead of mine.
                !agree
                $neutralResigned
                {giver}'s scum. Let {giver.himher} suffer without the coin, I say. Do <i>that</i>, if you're feeling charitable towards me.
        ]]

    }
    :RequireFlags(CONVERSATION_FLAGS.BUSINESS)
    :Hub(function(cxt)
        
        if not cxt.quest:GetParameter("has_money") and cxt:GetAgent():HasAspect("intimidated") then
            cxt:Opt( "OPT_WARN_DEBT" )
                :Fn( function() 
                    cxt:Dialog("DIALOG_BEATEN")
                    cxt.quest:Activate("report_beating")
                    StateGraphUtil.AddEndOption(cxt)
                end )
            return
        end

        cxt:Opt("OPT_COLLECT_DEBT")
            :SetQuestMark()
            :Fn(function(cxt)
                if cxt.quest.param.has_money then
                    if cxt:GetAgent():GetRelationship() >= RELATIONSHIP.LOVED then
                        cxt:Dialog("DIALOG_HAVE_MONEY_LOVED_RELATIONSHIP")
                        cxt.enc:GainMoney(cxt.quest:GetParameter('debt_amount'))
                        cxt.quest:Activate("return_the_debt")

                    elseif cxt:GetAgent():HasAspect("intimidated") then
                        cxt:Dialog("DIALOG_INTIMIDATED_TAKE_IT")
                        cxt.enc:GainMoney(cxt.quest:GetParameter('debt_amount'))
                        cxt.quest:Activate("return_the_debt")

                    else
                        cxt:Dialog("DIALOG_HAVE_MONEY")
                        cxt:Opt("OPT_NEGOTIATIE_FOR_IT")
                            :Negotiation{
                                on_success = function( cxt ) 
                                    cxt:Dialog("DIALOG_HAND_OVER")
                                    cxt.enc:GainMoney(cxt.quest:GetParameter('debt_amount'))
                                    cxt.quest:Activate("return_the_debt")
                                end,
                                on_fail = function( cxt ) cxt:Dialog("DIALOG_NOPE") end 
                            }


                        cxt:Opt("OPT_ATTACK_FOR_IT")
                            :Robot("FIGHT")
                            :Battle{
                                on_win = function(cxt) 
                                    cxt.quest:Activate("return_the_debt")

                                    if cxt:GetAgent():IsDead() then
                                        cxt:Dialog("DIALOG_KILLED_THEM")
                                        cxt.enc:GainMoney(cxt.quest:GetParameter('debt_amount'))
                                        StateGraphUtil.AddEndOption(cxt)
                                    else
                                        cxt:Dialog("DIALOG_INTIMIDATED_TAKE_IT")
                                        cxt.enc:GainMoney(cxt.quest:GetParameter('debt_amount'))
                                    end
                                end
                            }


                        StateGraphUtil.AddBackButton(cxt)
                    end
                    
                else
                    if cxt:GetAgent():HasAspect("intimidated") then
                        cxt:Dialog("DIALOG_BEATEN")
                        if not cxt.quest:IsActive("pay_back_debt") then
                            cxt.quest:Activate("report_beating")
                        end
                    else
                        if cxt.quest.param.offered_to_pay then
                            cxt:Dialog("DIALOG_OFFERED_TO_PAY")
                        else
                            if cxt:GetAgent():GetRelationship() >= RELATIONSHIP.LOVED then
                                cxt:Dialog("DIALOG_NO_MONEY_LOVED_RELATIONSHIP")
                            else
                                cxt:Dialog("DIALOG_DONT_HAVE_IT")
                            end
                        end

                        if cxt.quest.param.offered_to_pay ~= true then
                            cxt:Opt("OPT_OFFER_TO_PAY_IT", cxt.quest:GetParameter("debt_amount"))  
                                :Robot("OFFER_TO_PAY")
                                :PreIcon(global_images.hire)
                                :Fn(function(cxt) 
                                        if cxt:GetAgent():GetRelationship() == RELATIONSHIP.LOVED then
                                            cxt:Dialog("DIALOG_PAY_BACK_LOVED")
                                        else
                                            cxt:Dialog("DIALOG_PAY_BACK")
                                        end
                                        cxt.quest:Activate("pay_back_debt")
                                        cxt.quest.param.offered_to_pay = true
                                        cxt.enc:End()
                                    end)

                            cxt:Opt("OPT_ATTACK_PUNISHMENT")
                                :Robot("FIGHT")
                                :Battle{
                                    on_win = function(cxt) 

                                        cxt.quest:Activate("report_beating")

                                        if not cxt:GetAgent():IsDead() then
                                            cxt:Dialog("DIALOG_BEATEN")
                                        else
                                            cxt:Dialog("DIALOG_KILLED_THEM_NO_MONEY")
                                        end
                                        StateGraphUtil.AddEndOption(cxt)
                                    end,
                                }
                        else
                            cxt:Opt("OPT_ATTACK_PUNISHMENT")
                                :Robot("FIGHT")
                                :Battle{
                                    on_win = function(cxt) 

                                        cxt.quest:Activate("report_beating")

                                        if not cxt:GetAgent():IsDead() then
                                            cxt:Dialog("DIALOG_BEATEN")
                                        else
                                            cxt:Dialog("DIALOG_KILLED_THEM_NO_MONEY")
                                        end
                                        StateGraphUtil.AddEndOption(cxt)
                                    end,
                                }
                        end
                        StateGraphUtil.AddBackButton(cxt)
                    end
                end
            end)
    end)


QDEF:AddConvo("report_beating", "giver")
    :Priority( CONVO_PRIORITY_HIGHEST )
    :Loc{
        OPT_REPORT = "Tell {giver} what happened with {target}",
        DIAL_KILLED = [[
            player:
                !hips
                $miscPersuasive
                {target} didn't have the shills. But {target.hisher} corpse will be a, uh, <i>gentle</i> reminder to anyone else who doesn't feel like paying you back.
            agent:
                !surprised
                $happyTakenAback
                Uh, wow. You actually <i>killed</i> {target.himher}?
                Guess that's one way to guarantee I wouldn't dream of stiffing you. 
                !give
                $happyThanks
                Here's your payment. Next time, maybe ease up a little, huh?
        ]],
        DIAL_BEAT = [[
            player:
                !hips
                $miscPersuasive
                {target} didn't have the shills, but I made sure to pass on your, uh, <i>message</i>.
            agent:
                !chuckle
                $happyJoke
                Yeah, I figured as much. I'd bled {target.himher} pretty dry already. 
                Wish I coulda been there to see it, but I'm a busy person. 
                !give
                $happyThanks
                Your payment, as promised. Thanks for the job well done. 
        ]],
    }
    :Hub(function(cxt)
        cxt:Opt("OPT_REPORT")
            :Dialog(cxt.quest:GetCastAgent("target"):IsDead() and "DIAL_KILLED" or "DIAL_BEAT")
            :CompleteQuest()
    end)


QDEF:AddConvo(nil, "giver")
    :Priority( CONVO_PRIORITY_HIGHEST )
    :Loc{
        DIALOG_OFFER = [[
            player:
                !interest
                $neutralDirect
                Word is you got work for me?
                I'm happy to help, if the price is right. 
            agent:
                !thought
                $neutralThoughtful
                Hmm. I lent {target} {debt_amount#money}, and {target.heshe}'s past due in paying me back.
                Do you want to see if you can go collect on it?
            player:
                $happyDefault
                No problem. What do you want me to do if they don't have it, though?
            agent:
                !angry
                !eureka
                $angrySeething
                Make an example out of them, so folks don't start thinking I'm a charitable operation.
                !NEUTRAL
                !agree
                $neutralThanks
                Either way, I'll see you get paid—'cause I know you're no charity, yourself.
        ]],

        OPT_GIVE_UP = "Tell {giver} you can't recoup {target}'s debt",
        DIAL_GIVE_UP = [[
            player:
                !shrug
                $neutralWhatever
                There's no money to be had. {target}'s pockets are empty.
            giver:
                !angry
                $angryTakenAback
                Yeah? Forgive me if I don't believe you, but I can smell a rat.
                You'll <i>both</i> pay the price for cheating me.
        ]],

        OPT_RETURN_MONEY = "Deliver {target}'s money",
        OPT_GIVE_MONEY = "Pay off {target}'s debt",
        DIAL_GET_MONEY = [[
            player:
                !give
                $happyGreeting
                Shills collected, as promised.
            agent:
                !take
                $happyThanks
                You're kidding. Huh.
                !dubious
                $neutralDubious
                I thought for sure you'd come back with bruised knuckles.
            player:
                $neutralDirect
                You disappointed?
            agent:
                !thought
                $neutralThoughtful
                Hmm. Naw, I guess not! Money's good. I like it, and it spends well.
        ]],
        OPT_LIE = "Convince {giver} you sent {target} a message",
        DIALOG_LIE = [[
            player:
                $miscPersuasive
                {target} didn't have the money. But I made sure {target.heshe} got the message.
            agent:
                !dubious
                $angryCruel
                Oh yeah? Paint me a picture. I want <i>all</i> the details.
        ]],

        DIALOG_FORGIVEN = [[
            player:
                !fight
                $angryCruel
                Then I smacked {target.himher} frontwise <i>and</i> backwise, and left {target.himher} to bleed out in the sand, gurgling in the surf. 
            agent:
                !placate
                $neutralResigned
                All right, all right. Yeesh. Didn't realize you'd paint that picture so viscerally.
                Pity about the money, though.
        ]],
        DIALOG_ANGRY = [[
            player:
                !bashful
                $miscEmbarrassed
                Well I, er, you know, just got very angry and—
            agent:
                !angry_accuse
                $angrySeething
                What are you trying to pull, grifter? Did you go soft on me?
                I don't hire weaklings. Now I'll have to confront {target} myself just to make sure the job was done.
                !angry_shrug
                Get outta my face. I've got work to do. 
        ]],

        DIAL_RETURN_THE_DEBT = [[
            agent:
                !give
                $happyThanks
                Thanks for your services, pal! Here's your cut. I'll be sure to mention you if anybody else needs work doin'.
            player:
                !take
                $happyJoke
                Just tell 'em to look for Sal ik-Derrick. They'll know me by the trail of satisfied customers I leave in my wake. 
        ]],
    }
    :Hub(function(cxt) 
        
        StateGraphUtil.OfferSideJob(cxt, "confront_target", "DIALOG_OFFER" )

        if cxt.quest:IsActive( "pay_back_debt" ) then
            cxt:Opt("OPT_GIVE_MONEY", cxt.quest:GetCastAgent("giver"))
                :PreIcon(global_images.hire)
                :DeliverMoney( cxt.quest:GetParameter('debt_amount'), {no_scale=true} )
                :Fn(function(cxt) 
                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:Complete("pay_back_debt")
                    cxt.quest:Complete("confront_target")
                    cxt.quest:Activate("tell_target_settled")
                end)
                :Dialog("DIAL_GET_MONEY")
        end

        if cxt.quest:IsActive( "return_the_debt" ) then
            cxt:Opt("OPT_RETURN_MONEY", cxt.quest:GetCastAgent("giver"))
                :DeliverMoney( cxt.quest:GetParameter('debt_amount'), {no_scale=true} )
                :CompleteQuest()
                :Dialog("DIAL_RETURN_THE_DEBT")
        end

        if cxt.quest:IsActive( "return_the_debt" ) or cxt.quest:IsActive( "pay_back_debt" ) then
            cxt:Opt("OPT_LIE")
                :Robot("LIE_TO_KEEP")
                :Dialog("DIALOG_LIE")
                :Negotiation{
                    on_success = function( cxt ) 
                        cxt:Dialog("DIALOG_FORGIVEN")
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end,
                    on_fail = function( cxt ) 
                        cxt:Dialog("DIALOG_ANGRY")
                        cxt.quest:Fail()
                        StateGraphUtil.AddEndOption(cxt)
                    end 
                }


            cxt:Opt("OPT_GIVE_UP" )
                :PreIcon(global_images.dislike)
                :Dialog( "DIAL_GIVE_UP" )
                :Fn(function(cxt)
                    cxt.quest:Fail()
                end)
        end
    end)



QDEF:AddTest(TestCreator()
                :AddRelationshipSwitch("target")
                :AddBoolParams{"has_money"}
                :AddQuestFn("target_is_intimidated", function(quest) quest:GetCastMember("target"):GainAspect("intimidated") end)
                :AddChoices{"LIE_TO_KEEP", "OFFER_TO_PAY", "FIGHT"})
