local negotiation_defs = require "negotiation/negotiation_defs"

Content.AddNegotiationCard("summons_card", 
{
    name = "Deliver Summons",
    flavour = "'You’ve been served.'",
    desc = "{INCEPT} 2 {planted_summons}. Target gains 1 {IMPATIENCE}.",
    
    cost = 1,
    flags = negotiation_defs.CARD_FLAGS.MANIPULATE | negotiation_defs.CARD_FLAGS.EXPEND,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,
    show_dealt = true,

    OnPostResolve = function( self, minigame, card )
        self.anti_negotiator:InceptModifier("planted_summons", 2, self )
        self.anti_negotiator:AddModifier("IMPATIENCE")
    end,
})

Content.AddNegotiationModifier( "planted_summons",
{
    name = "Summons",
    desc = "Remove {1} {1*stack|stacks} at the end of your turn.\nWhen destroyed, returns to your draw pile.\n\n<#PENALTY>This must be attached to your opponent at the end of the negotiation to succeed!</>",
    modifier_type = MODIFIER_TYPE.INCEPTION,
    
    desc_fn = function(self, fmt_str)
        return loc.format(fmt_str, self.decay_rate )
    end,

    decay_rate = 1,
    --self_attack_amt = 10,

    OnUnapply = function( self )
        local card = Negotiation.Card( "summons_card", self.anti_negotiator.agent )
        card:TransferCard(self.engine:GetDrawDeck())
        --battle:DealCards( cards, battle:GetDrawDeck() ) --is there a way 
    end,

    event_handlers =
    {

        [ negotiation_defs.EVENT.END_PLAYER_TURN ] = function( self, minigame )
            self.negotiator:DeltaModifier(self, -self.decay_rate)
            AUDIO:PlayEvent("event:/sfx/battle/cards/neg/bonus_tick_down")
        end

    },

})


AddOpinionEvent("DELIVERED_SUMMONS", 
{
    delta = OPINION_DELTAS.DISLIKE,
    txt = "Delivered an Admiralty Summons",
})

local QDEF = QuestDef.Define{
        title = "The Summoning",
        qtype = QTYPE.SIDE,
        desc = [[{giver} needs you to deliver an Admiralty summons to {target}. {target.HeShe} isn't going to take it willingly.]],
        icon = engine.asset.Texture("icons/quests/deliver_summons.tex"),
        rank = {1, 5},
        focus = QUEST_FOCUS.NEGOTIATION,
        avoids = 
        {
            {"giver", "target"},
        },


    }

QDEF:AddCast{
    cast_id = "giver",
    provider = true,
    condition = QuestUtil.CanDoAdmiraltyGoonQuest,
}:AddCastFallback
{
    cast_fn = function(quest, t)
        local defs = {"ADMIRALTY_GOON", "ADMIRALTY_GOON", "ADMIRALTY_GUARD"}
        local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
        table.insert( t, agent )
    end,
}


:AddCast{
    cast_id = "target",
    condition = function( agent, quest )

        if QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) and not agent:GetFaction():IsLawful() and not AgentUtil.IsCoworker(agent, quest:GetCastMember("giver")) then
            return true
        end
    end,
}:AddCastFallback
{
    cast_fn = function(quest, t)
        local defs = {"BANDIT_GOON", "BANDIT_GOON2", "JAKES_RUNNER", "BANDIT_RAIDER"}
        local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
        table.insert( t, agent )
    end,
}



:AddObjective{
    id = "deliver_summons",
    title = "Give {target} the summons",
    --desc = "Deliver summons to {target}",
    mark = { "target" },
    negotiation_targets = {"target"},
    combat_targets = {"target"},
    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end,
    is_combat_target = function(quest, agent)
        if agent == quest:GetCastMember("target") and quest.param.failed_negotiation then
            return true
        end
    end,

}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver#agent}",
    desc = [[You've earned it!]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}

------------------------------------------------------------------

QDEF:AddAttract("deliver_summons", "target",
    
    [[
        player:
            !greeting
            $neutralGreeting
            Hey, are you—
        agent:
            !interest
            $scaredDefault
            You're not Admiralty, right?
        player:
            !dubious
            $neutralDubious
            Do I <i>look</i> like—
        agent:
            !point
            $scaredFearful
            You have to tell me if I ask!
    ]])


QDEF:AddConvo("deliver_summons", "target")
    :RequireFlags(CONVERSATION_FLAGS.BUSINESS)
    :Loc{
        OPT_TRY_PLANT_SUMMONS = "Try to plant the summons on {target}",
        OPT_TRY_TO_GIVE = "Convince {target} to take the summons",
        DIALOG_PLANTED_FAIL = [[
            
            * {target} notices your awkward fumbling and marks you as a thief.
            target:
                !angry
                $angryTakenAback
                Hey, what are you trying to pull?
        ]],
        DIALOG_PLANTED = [[
            * You deftly place the summons on {target}'s person.
            player:
                !point
                $miscPersuasive
                Hey, what's that in your pocket?
            
            * {target} is surprised to discover the summons in their possessions.
            target:
                $angryTakenAback
                Oh Hesh. 
                Where did this come from?!
                !angry
                $angrySeething
                This is <b>your</> doing.
        ]],
        DIALOG_LEAVE = [[
            target:
                $angrySeething
                We're done here.
        ]],
        DIALOG_NEGOTIATION = [[ 
            target:
                $angrySeething
                I <b>know</> the Admiralty is trying to get me. 
                Why would I make their job easy?
        ]],
        DIALOG_TAKE_IT = [[
            target:
                !sigh
                $neutralResigned
                This is going to cost me a fortune in bribes.
        ]],
        DIALOG_REFUSE = [[
            target:
                $angrySeething
                No, I know my rights.
                So long as I don't take that thing, they can't touch me!
        ]],
        DIALOG_TAKE_IT_FAIL = [[
            * Just as you're about to leave the summons with {agent}, {agent.heshe} throws it back in your face.
            target:
                $angrySeething
                Not so fast, you!
                So long as I don't take that thing, they can't touch me!
        ]],

        OPT_FORCE_TAKE_SUMMONS = "Force {target} to take the summons",
        DIALOG_TRY_FORCE_TAKE = [[
            player:
                $angryHostile
                You're going to take this summons if I have to shove it down your throat.
            agent:
                $angryHostile
                So that's how this is going to be, then?
        ]],
        DIALOG_FORCE_TAKE_IT = [[
            agent:
                $scaredPowerless
                I yield! 
                I'll take the damn thing.

            player:
                $angryAside
                That's what you get for making me work for it.
        ]],
        DIALOG_FORCE_KILLED = [[
            agent:
                !exit
            * You half-heartedly place the summons on {agent}'s still-warm corpse.
            player:
                $neutralWhatever
                Close enough, I hope.
        ]],
        NEGOTIATION_REASON_END_IT = "End the negotiation while {agent} has the summons!",
        NEGOTIATION_REASON_PLAY_IT = "Play {1#card} to give {agent} the summons.",


    }
    :Hub(function(cxt)
            
            cxt:Opt("OPT_TRY_TO_GIVE")
                :Dialog("DIALOG_NEGOTIATION")
                :Negotiation{
                    on_start_negotiation = function(minigame)
                        local card = Negotiation.Card( "summons_card", minigame.player_negotiator.agent )
                        card:TransferCard(minigame:GetHandDeck())

                    end,

                    on_success = function(_, minigame) 
                        if minigame.opponent_negotiator:HasModifier("planted_summons") then
                            cxt:Dialog("DIALOG_TAKE_IT")
                            cxt.quest:Complete("deliver_summons")
                            cxt:GetAgent():OpinionEvent(OPINION.DELIVERED_SUMMONS)
                        else
                            cxt.quest.param.failed_negotiation = true
                            cxt:Dialog("DIALOG_TAKE_IT_FAIL")
                        end
                    end,
                    on_fail = function() 
                        cxt.quest.param.failed_negotiation = true
                        cxt:Dialog("DIALOG_REFUSE")
                    end,
                    reason_fn = function(minigame)
                        if minigame.opponent_negotiator:HasModifier("planted_summons") then
                            return cxt:GetLocString("NEGOTIATION_REASON_END_IT") 
                        else
                            return loc.format( cxt:GetLocString("NEGOTIATION_REASON_PLAY_IT"), "summons_card")
                        end
                    end,

                }
            if cxt.quest.param.failed_negotiation then
                cxt:Opt("OPT_FORCE_TAKE_SUMMONS")
                    :Dialog("DIALOG_TRY_FORCE_TAKE")
                    :Battle{
                        on_win = function(cxt) 
                            if cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_FORCE_KILLED")
                            else
                                cxt:Dialog("DIALOG_FORCE_TAKE_IT")
                                cxt:GetAgent():OpinionEvent(OPINION.DELIVERED_SUMMONS)
                            end
                            cxt.quest:Complete("deliver_summons")
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                    }

            end
        end)



QDEF:AddConvo("return_for_reward", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_REPORT = "Tell {agent} the summons has been delivered",
        DIALOG_REPORT_DEAD = [[
            agent:
                $neutralResigned
                Yeah, I heard how you did it.
                !shrug
                $neutralWhatever
                Either way, <i>my</> problem is solved. 
        ]],
        DIALOG_REPORT = [[
            agent:
                $neutralDubious
                So they got the summons?
                !sigh
                $neutralResigned
                Yaaaaaay.
                Wake me up when it's time to go beat someone senseless.

        ]]
    }

    :Hub(function(cxt) 
    
        cxt:Opt("OPT_REPORT")
            :Dialog(cxt.quest:GetCastMember("target"):IsDead() and "DIALOG_REPORT_DEAD" or "DIALOG_REPORT")
            :CompleteQuest()
    end)



QDEF:AddConvo(nil, "giver")
    :Priority( CONVO_PRIORITY_HIGHEST )
    :Loc{
        DIALOG_OFFER = [[
            player:
                !interest
                $neutralGreeting
                You got work for me?
            agent:
                $angrySeething
                Can you believe this? I have to deliver a summons to {target}.
                !spit
                $angrySeething
                A summons!
                <b>I</> joined the Admiralty to crack heads and take names, not to serve <i>paperwork</>.
                !dubious
                $neutralDubious
                You look like you could use a couple shills, though.
        ]],
        DIALOG_GIVE = [[
            agent: 
                !give
                $neutralDirect
                Here's the summons. Make sure it's served nice and proper, got it?
        ]]
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "deliver_summons", "DIALOG_OFFER", function() 
                cxt:Dialog("DIALOG_GIVE")
            end)
    end)


