local ACCEPTABLE_IDS = {
    LABORER= true,
    HEAVY_LABORER= true,
    MERCHANT= true,
    BARTENDER= true,
    SHOPKEEPER= true,
}


local function is_valid_target(quest, giver, agent)
    
    --for agent,rel in pairs(giver.social_connections.relationships) do
    if not agent:IsRetired() and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) then
        local rel = giver:GetRelationship(agent)
        if rel <= RELATIONSHIP.NEUTRAL and agent:GetBrain():GetWorkPosition() then
            local boss = agent:GetWorkBoss()
            if boss and boss ~= giver then
                return true
            end
        end
    end
    --end
end

local QDEF = QuestDef.Define{
        title = "Early Retirement",
        desc = [[{giver} is hoping to get {target} fired so {giver.hisher} friend can take over. {giver} just needs your help making it happen.]],
        icon = engine.asset.Texture("icons/quests/early_retirement.tex"),
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        focus = QUEST_FOCUS.NEGOTIATION,

        avoids = { {"target", "giver" }},


        on_init = function(quest)
            local target = quest:GetCastMember("target")
            quest:SetParameter("wanted_job", target:GetBrain():GetWorkPosition())
            return true
        end,

        on_destroy = function(quest)
            quest:GetParameter("wanted_job"):SetPaused(false)
        end

    }

    :AddCast{
        cast_id = "target",
        condition = function(agent, quest)
            if agent:GetBrain():GetWorkPosition() and agent:GetWorkBoss() then
                return QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true)
            end
        end,

        events = 
        {
            agent_retired = function( quest, agent ) 
                if not quest:IsActive() then
                    quest:Cancel()
                else
                    if quest:IsActive("get_target_fired") then
                        quest:Complete("get_target_fired")
                    end
                end
            end,
            
            work_changed = function( quest, agent )
                if agent:GetBrain():GetWorkPosition() == nil then
                    --spawn a buddy, and set them as the next hire
                    --quest:GetParameter("wanted_job"):SetNextHire(quest:GetCastMember("giver"))
                    if quest:IsActive("get_target_fired") then
                        quest:Complete("get_target_fired")
                    end

                end
            end,
        }
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent, quest)
            local target = quest:GetCastMember("target")
            if not AgentUtil.IsFriend(agent, target) and agent:GetWorkBoss() ~= target and target:GetWorkBoss() ~= agent then
                return true
            end
        end,

    }
    

    :AddCast{
        cast_id = "boss",
        optional = true,
        when = QWHEN.MANUAL,
        cast_fn = function(quest,t)
            
            local boss = quest:GetCastMember("target"):GetWorkBoss()
            if boss then
                table.insert(t, boss)
            end
        end,
    }

    :AddObjective{
        id = "get_target_fired",
        title = "Make {target} lose their job",
        desc = [[Get {target} fired, kill {target.himher}, or convince {target.himher} to leave {target.hisher} position willingly.]],
        mark = { "target", "boss" },
        combat_targets = {"target", "boss"},
        negotiation_targets = {"target", "boss"},
        on_activate = function(quest)
            quest:AssignCastMember("boss")
        end,
        on_complete = function(quest)
            quest:Activate("return_for_reward")
        end
    }
    :AddObjective{
        id = "return_for_reward",
        title = "Get reward from {giver#agent}",
        desc = [[Tell {giver} that the position has been vacated for {giver.hisher} friend.]],
        terminal = true,
        ready_for_completion = true,
        mark = { "giver" },
        cast = 
        {
            giver = 
            {
                provider = true
            },
        },
    }



QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        DIALOG_THANKS = [[
            player:
                Job's done. {target}'s out of work.
            agent:
                $neutralThanks
                Well that's great news!
        ]],
        DIALOG_INTRODUCE = [[
            agent:
                $neutralGreeting
                Hey, meet {friend.name}.
                {friend.HeShe}'s gonna do a real bang-up job. 
                
                {boss_cast ?
                    Maybe {boss} will finally be able to get some rest and not have to clean up after others anymore.
                }
            friend:
                !right
                $happyGreeting
                Nice to meet you. Thanks for the job!
            agent:
                !left
                $neutralThanks
                Now go get to work, you lazy oshnu!
            friend:
                !exit
            agent:
                !exit

        ]],
    }
    :Hub(function(cxt) 
        cxt:QuestOpt()
            
            :Dialog("DIALOG_THANKS")
            :Fn( function( cxt ) 
                    --do the hiring, introduce the buddy
                    if cxt.quest:GetParameter("wanted_job"):GetAgent() == nil then
                        cxt.quest:GetParameter("wanted_job"):TryHire()
                        local new_agent = cxt.quest:GetParameter("wanted_job"):GetAgent()
                        if new_agent then
                            new_agent.social_connections:SetRelationship(RELATIONSHIP.LOVED, cxt:GetAgent())
                            new_agent:MoveToLocation(cxt.location)
                            cxt:ReassignCastMember("friend", new_agent)
                            cxt.enc.scratch.boss_cast = cxt.quest:GetCastMember("boss") ~= nil
                            cxt:Dialog("DIALOG_INTRODUCE")
                            new_agent:OpinionEvent(OPINION.FOUND_A_JOB)
                        end
                    end
                end)
            --:Dialog("DIALOG_INTRODUCE")
            :CompleteQuest()
            :Fn(function() StateGraphUtil.AddEndOption(cxt) end)
    end)


QDEF:AddConvo("get_target_fired", "target")
    :Loc{
        OPT_CONVINCE_TO_LEAVE_JOB = "Convince {target} to leave their job",
        DIALOG_KEEP_JOB = [[  
            player:
                This job is boring. I'm bored just standing here.
                Come on, don't wanna do something fun?
            agent:
                $neutralWhatever
                If your job is so stimulating, why are you here bothering me?
                I got bills to pay, and besides, my boss loves me.
                I convinced 'em to dump their loser ex, and now they act like we're besties.
                Show up late, skim from the till—you name it, I can pull it off. 
                Go bother someone else now.
        ]],
        DIALOG_LEAVE_JOB = [[
            player:
                $miscPersuasive
                Someone of your skillset can do better elsewhere, right?
                There's gotta be loads of opportunities available.
                !eureka
                You could get a job with benefits, or heck, even a raise!
            agent:
                !thought
                H-hey, you know what? I could! Yeah!
                !handwave
            * {agent} throws {agent.hisher} work badge on the ground. 
                I'm desirable! I'm skilled! I don't gotta stand for this.
                The future's the limit, baby!
            * {agent} saunters off to enjoy {agent.hisher} newfound freedom, but not before throwing a rude gesture towards {agent.hisher} colleagues.
        ]],
        OPT_ROUGH_THEM_UP = "Attack {target}",
        DIALOG_ROUGH_UP = [[
            player:
                $miscPersuasive
                Look at you. All that tension in your shoulders.
                !eureka
                You look like you need a vacation.
            agent:
                !chuckle
                $neutralDubious
                Oh yeah, sure. Just hop a ship to Deltree, why don't I?
                I can't afford a vacation. 
            player:
                Sure you can. 
                !overthere
                In your <i>dreams!</i>
            agent:
                Hah!
                !dubious
                ... wait, what?
            player:
                !fight
            agent:
                !surprised
        ]],
        DIALOG_WON = [[
            agent:
                !injured
                Why are you doing this?! What'd I do to you, huh?
            player:
                !angry_threaten
                $angryCruel
                Nothing. But you're unemployed now, got it?
                You go find work somewhere else. But you don't come back here.
            agent:
                $scaredPowerless
                I... I don't understand. But fine, I'll do what you say.
            * {agent} sadly removes {agent.hisher} work badge and hobbles weakly out the door.
        ]],
        DIALOG_WON_KILLED = [[
            agent:
                !exit
            player:
                Well, look at that. A position has just opened up.
            * No one laughs, but then, you didn't invite anyone to. 
        ]],
        DIALOG_CONVINCE_TO_LEAVE_JOB = [[
            player:
                !wince
                Man, you <i>work</i> here? It's kind of a dump, ain't it?
            target:
                $neutralTakenAback
                Is it?
        ]],        
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE_TO_LEAVE_JOB")
            :Dialog("DIALOG_CONVINCE_TO_LEAVE_JOB")
            :Negotiation{
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_KEEP_JOB")
                end,
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_LEAVE_JOB")
                    if cxt.quest:GetParameter("wanted_job"):GetAgent() then
                        cxt.quest:GetParameter("wanted_job"):Fire()
                    end
                    

                    cxt.quest:GetParameter("wanted_job"):SetPaused(true)
                    cxt:GetAgent():MoveToLimbo()
                    StateGraphUtil.AddEndOption(cxt)
                end,
            }   
        cxt:Opt("OPT_ROUGH_THEM_UP")
            :Dialog("DIALOG_ROUGH_UP")
            :Battle{
                on_win = function(cxt) 
                    if cxt:GetAgent():IsDead() then
                        cxt:Dialog("DIALOG_WON_KILLED")
                    else
                        cxt:Dialog("DIALOG_WON")
                        if cxt.quest:GetParameter("wanted_job"):GetAgent() then
                            cxt.quest:GetParameter("wanted_job"):GetAgent():OpinionEvent(OPINION.LOST_A_JOB)
                            cxt.quest:GetParameter("wanted_job"):Fire()
                        end
                        cxt.quest:GetParameter("wanted_job"):SetPaused(true)
                    end
                    StateGraphUtil.AddEndOption(cxt)
                end
            }
    end)

QDEF:AddConvo("get_target_fired", "boss")
    :Loc{
        OPT_CONVINCE_FIRE_TARGET = "Convince {agent} to fire {target}",
        DIALOG_KEEP_JOB = [[
            player: 
                $miscPersuasive
                So you see, I'm very displeased with {target}'s service and I—
            agent:
                !crossed
                $neutralDirect
                {giver} put you up to this, didn't {giver.heshe}?
                You know {giver.heshe} just wants {target} fired because {giver} and I used to be a thing.
                {target} was the one who convinced me I could do better. 
                !thumb
                $angryDefault
                I ain't gonna fire a friend. Tell {giver} {giver.heshe} is still a slimeball.
        ]],
        DIALOG_LEAVE_JOB = [[
            player:
                I know folks need to talk to Bilebrokers often enough, but {target} didn't even wash {target.hisher} hands after.
                Who knows what Roaloch spores {target} could have in all {target.hisher} crevices?
                It's a danger to everyone else here.
            agent:
                !scared
                $scaredStammering
                I knew {target} was a bit lax on hygiene but... even after dealing with Roaloch goods?
                That's... serious.
                !palm
                Hesh, I hate this, but I am gonna have to let {target} go. For {target.hisher} good and mine. 
            * {agent} heads off to tell {target} the bad news. 
        ]],
        OPT_ROUGH_THEM_UP = "Convince {agent}... <i>with your fists</>",
        DIALOG_ROUGH_UP = [[
            player:
                !angry_accuse
                $angryHostile
                I asked nicely. Now I'm telling.
            agent:
                !angry
                $angryTakenAback
                It's a shake down, is it? 
                Why didn't you say so instead of wasting my time!
                !fight
        ]],
        
        DIALOG_WON = [[
            player:
                Come to heel, now, and let {target} go.
            agent:
                !injured
                $scaredFearful
                Fine! Fine! I'll fire {target}.
                Just stop hurting me!
            * {agent} heads off to go give {target} the bad news.
        ]],
        DIALOG_WON_KILLED = [[
            agent:
                !exit
            * {agent} is dead. You'll have to try talking to {target} directly.
        ]],
        DIALOG_FIRED = [[
            agent:
                !right
                !angry
                $angryCruel
                Apparently I just got fired. Why does it seem to me <i>you</i> must've had something to do with it?
            player:
                $neutralWhatever
                Nothing personal, pal. Maybe just wash your hands, huh?
            agent:
                I don't even know what that means! Go suck sludge!
                !exit
        ]],
        DIALOG_CONVINCE_TO_FIRE_TARGET = [[
        player:
              Hey, you the boss here?
              I'd like to file a complaint about {target}.
        agent:
            !palm
            $neutralResigned
            Ugh, not this again. 
            Let's hear it. What did {target} "do" now?
        ]], 
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE_FIRE_TARGET")
            :Dialog("DIALOG_CONVINCE_TO_FIRE_TARGET")
            :Negotiation{
                subject = cxt.quest:GetCastMember("target"),
                
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_KEEP_JOB")
                    cxt.quest.param.failed_boss_neg = true
                end,

                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_LEAVE_JOB")
                    if cxt.quest:GetParameter("wanted_job"):GetAgent() then
                        cxt.quest:GetParameter("wanted_job"):GetAgent():OpinionEvent(OPINION.LOST_A_JOB)
                        cxt.quest:GetParameter("wanted_job"):Fire()
                    end
                    cxt.quest:GetParameter("wanted_job"):SetPaused(true)
                    cxt.encounter:SetPrimaryCast(cxt.quest:GetCastAgent("target"))
                    cxt:Dialog("DIALOG_FIRED")

                    StateGraphUtil.AddEndOption(cxt)
                    cxt.quest:GetCastAgent("target"):MoveToLimbo()
                end,
            }   
        
        if cxt.quest.param.failed_boss_neg then
            cxt:Opt("OPT_ROUGH_THEM_UP")
                :Dialog("DIALOG_ROUGH_UP")
                :Battle{
                    on_win = function(cxt) 
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_WON_KILLED")
                        else
                            cxt:Dialog("DIALOG_WON")
                            
                            if cxt.quest:GetParameter("wanted_job"):GetAgent() then
                                cxt.quest:GetParameter("wanted_job"):GetAgent():OpinionEvent(OPINION.LOST_A_JOB)
                                cxt.quest:GetParameter("wanted_job"):Fire()
                            end
                            
                            cxt.quest:GetParameter("wanted_job"):SetPaused(true)
                            
                        end
                        StateGraphUtil.AddEndOption(cxt)
                    end
                }
        end


    end)


QDEF:AddConvo(nil, "giver")
    :Loc{
        DIALOG_OFFER = [[
            player:
                I'm here on business, actually. You wanted a hunter?
            agent:
                Ah, right. 
                $neutralDirect
                A friend of mine is coming to town, and they need a job.
                Unfortunately all the best jobs are taken, often by unqualified dreck. 
                !angry_shrug
                {target} in particular... I've written letters of complaint to {target.hisher} superiors. But nothing's been done!
            player:
                !crossed
                $neutralJoke
                Wow, after you wrote letters of complaint and everything.
            agent:
                Right?! 
                My friend needs that job. So I want you to get {target} fired.
                With force, if necessary.
        ]],
        DIALOG_TRY_BOSS = [[
            player:
                Sure, why not. Should be an easy task.
            agent:
                $miscPersuasive
                {boss} is the site supervisor. {boss.HeShe}'s the one you'll want to convince.
            player:
                I'll keep that in mind.
        ]],
        DIALOG_NO_BOSS = [[
            player:
                Sure, why not. Should be an easy task.
            agent:
                $neutralThanks
                Yes, well, it hasn't been anything my shrewd business reviews could fix.
                But hopefully you'll fare better.
        ]],
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "get_target_fired", "DIALOG_OFFER", function() 
                if cxt.quest:GetCastMember("boss") then
                    cxt:Dialog("DIALOG_TRY_BOSS")
                else
                    cxt:Dialog("DIALOG_NO_BOSS")
                end
            end)
    end)

