local LOCATION_DEF =
{
    id = "ADMIRALTY_ROUTE",
    name = "Admiralty Road",
    plax = "EXT_Road01",
    desc = "The Admiralty Convoy will be travelling along this road at some point today.",
    indoors = false,
    map_tags = {"road"},
}

if not Content.GetLocationContent(LOCATION_DEF.id) then
    Content.AddLocationContent(LOCATION_DEF)
end

local QDEF = QuestDef.Define{
        title = "Friend or Fanatic",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        icon = engine.asset.Texture("icons/quests/free_priest.tex"),
        desc = [[{giver} has offered you a reward to rescue {target} from bandits.]],
        focus = QUEST_FOCUS.BALANCED,

    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent)
            return agent.def.id == "PRIEST"
        end,
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent( "PRIEST" )
            table.insert( t, agent )
        end,
    }

    
    :AddCast{
        cast_id = "target",
        
        condition = function(agent)
            return agent.def.id == "LUMINARI"
        end,
        events = 
        {          
            dismissed = function( quest )
                quest:Fail()
            end
        }
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent( "LUMINARI")
            table.insert( t, agent )
        end,
    }

    :AddLocationCast{
        cast_id = "roadside",
        when = QWHEN.MANUAL,
        no_validation = true,
    }

    :AddObjective{
        id = "intercept_convoy",
        title = [[Intercept the convoy transporting {target}]],
        desc = "Intercept the convoy and free {target}.",
        combat_targets = {"patrol_leader"},
        mark = { "roadside" },
        
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },

        on_activate = function( quest)
            local location = Location( LOCATION_DEF.id )
            assert(location)
            TheGame:GetGameState():AddLocation(location)
            quest:AssignCastMember("roadside", location )
        end,
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Get reward from {giver#agent}",
        desc = [[Return to {giver} with {target} to claim your reward.]],
        ready_for_completion = true,
        mark = { "giver" },
        terminal = true,
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
    }

QDEF:AddConvo("intercept_convoy")
    :Confront(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("roadside") then
                return "STATE_PREPARE"
            end
        end)

    :State("STATE_PREPARE")
        :Loc{
            DIALOG_INTRO = [[
                * {target}'s trail leads you here—judging by the prints, {target.heshe} likely lost {target.hisher} footing in a scuffle with bandits. 
                * You spot the tell-tale signs of a kidnapping and follow it off the trail. 
            ]],
            OPT_PREPARE = "Proceed carefully",
            DIALOG_PREPARE = [[
                * You hear voices and melt back into the bush like a stalking yote.
            ]],
        }
        :Fn(function(cxt)
            local team = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank(), cxt.location, true)
            cxt.encounter:SetPrimaryCast(team[1])

            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_PREPARE")
                :Dialog("DIALOG_PREPARE")
                :GoTo("STATE_CONVOY_ARRIVAL")
        end)

    :State("STATE_CONVOY_ARRIVAL")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !right
                * To your surprise, you don't find a bandit camp—but rather the Admiralty, with shoulders as stiff as their starched collars.
                agent:
                    !shrug
                    $miscHopeless
                    It's nothing personal, Luminari. I don't wanna hurt you. Not least because that would really piss in Hesh's waters.
                    We just have a point to make is all. 
                    The Spree do this to travelers all the time. But the annex can stop it. 
                target:
                    !left
                    !angry
                    $angryDefault
                    You chose your path, soldier. The Cult will hear of this, whatever you do to me.
                agent:
                    !scared_shrug
                    $miscHopeless
                    Sorry to say that that's the idea, pal. They'll just think it's the Spree who did it.
                * This isn't quite what you expected, but a rescue mission is a rescue mission. Better get to it.
            ]],
            OPT_TRICK = "Convince {agent} to hand over {target}",
            DIALOG_TRICK = [[
                * You step out of hiding and approach the guard, trying your best to look like you belong.
                agent:
                    !right
                    !surprised
                player:
                    !left
                    !interest
                    $neutralDirect
                    Hey, this the gig? I have orders to come take the Luminari off your hands.
                agent:
                    !dubious
                    $neutralDubious
                    I hadn't heard of this.
            ]],
            DIALOG_TRICK_UNKNOWN = [[
                agent:
                    Who are you, anyway?
                player:
                    Why, I'm {player}. Your friendly neighborhood unsavory.
            ]],
            DIALOG_TRICK_SUCCESS = [[
                player:
                    !angry_shrug
                    $neutralWhatever
                    I was sent to deal with the Luminari so that <i>you</i> don't have to. 
                    Hesh won't know you've got lumin-lit blood on your hands if I'm the one to do the deed.
                agent:
                    !agree
                    $miscRelieved
                    Yeah, yeah. That's great. I don't want nothing to do with killing a Luminari. 
                    Then we won't be lying when we claim to have nothing to do with it, too. 
                    !overthere
                    $miscRelieved
                    Go on, take the Luminari. The less I know of this, the better. 
                * The Admiralty pack up and leave, eager to be out of earshot when the violence begins.
                target:
                    !right
                    !angry
                    $angryDefault
                    Well, brigand? Are the waters of your soul so muddy that Hesh won't notice your deeds?
                player:
                    !angry_shrug
                    $neutralWhatever
                    I couldn't care less, but can the sermon. I ain't got the stomach for it.
                * You untie the Luminari and help dust the dirt from their robes.
                target:
                    !dubious
                    $neutralDubious
                    What is this?
                player:
                    {giver} sent me to rescue you. So you're rescued. 
                target:
                    $neutralResigned
                    Hesh truly does move in mysterious ways.
            ]],
            DIALOG_TRICK_FAIL = [[
                player:
                    $miscPersuasive
                    You trying to pin this on the Spree, right? Well, I look more like a bandit than you do.
                agent:
                    !angry_accuse
                    $angryPatienceLost
                    <i>Too much</i>, even. I dunno who you are, but I got enough on my mind without you stepping in.
                player:
                    !disappoint
                    $neutralWhatever
                    It was worth a shot.
                    !fight
                * Looks like the Luminari isn't getting free without some bloodshed.
            ]],
            OPT_INTIMIDATE = "Threaten {agent}",
            DIALOG_INTIMIDATE = [[
                * You step out of hiding and approach the guard, muscles flexing. 
                agent:
                    !right
                    !surprise
                player:
                    !left
                    !cruel
                    $angryCruel
                    Now, what was it you just said? "It's nothing personal, I don't want to hurt you." 
                agent:
                    !angry
                    $angryDefault
                    What the Hesh are you doing here?!
                player:
                    !crossed
                    $angryDefault
                    I'm about to serve up some divine retribution. Unless you want to hand over the Luminari.
            ]],
            DIALOG_CONVINCED = [[
                player:
                    !cruel
                    $angryCruel
                    They say the tendrils of Hesh move in mysterious ways. Seems to me like you're about to get throttled by them.
                agent:
                    !scared
                    $scaredDefault
                    N-no, I'm not! I was against this idea in the first place!
                    The Luminari's all yours. I'm a good Heshian, I am!
                * The Admiralty book it for the hills, like the tentacles of Hesh itself will come snapping at their heels.
                * You untie the Luminari and help {target.himher} find {target.hisher} footing.
                target:
                    !right
                    !dubious
                    $neutralDubious
                    Didn't expect rescue to come from someone who looks like you, grifter.
                player:
                    !crossed
                    $miscMocking
                    Well I didn't expect manners from a Luminari, so at least one of us knows where we stand.
                    Let's go. {giver} is waiting for you.
            ]],
            DIALOG_INTIMIDATE_FAIL = [[
                player:
                    !angry_accuse
                    $angryDefault
                    Seems you're being naughty! I'm here to put an end to it.
                agent:
                    !angry
                    $angryHostile
                    You and what army, grifter? I was paid to dispose of someone.
                    Now I'll just have to dispose of you, too.
                    !fight
            ]],
            OPT_ATTACK = "Take {target} by force",
            DIALOG_ATTACK = [[
                * Before {agent} has a chance to react, you emerge from hiding with your weapons ready.
                player:
                    !exit
                agent:
                    !exit
            ]],
            DIALOG_WON_BATTLE = [[
                player:
                    !left
                target:
                    !right
                    !dubious
                    $neutralDubious
                    Who <i>are</i> you, really?
                player:
                    {giver} sent me.
                target:
                    !happy
                    $miscRelieved
                    You mean <i>Hesh</i> sent you. Thank the deep waters. 
                player:
                    !bashful
                    $neutralResigned
                    Sure, fella. Whatever.
                * You grab {target}, remove {target.hisher} cuffs and leave before anyone else arrives.
            ]],

            SIT_MOD = "Under orders",
        }
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    advantage = TEAM.BLUE,
                    on_win = function(cxt)
                        cxt:Dialog("DIALOG_WON_BATTLE")
                        cxt.quest:GetCastAgent("target"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest:Complete("intercept_convoy")
                        cxt.quest:Activate("return_for_reward")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                }

            if Util.IsPlayerWorkingWithFaction("ADMIRALTY") then
                cxt:Opt("OPT_TRICK")
                    :Dialog("DIALOG_TRICK")
                    :Fn( function( cxt )
                        if cxt:GetAgent():GetRelationship() == RELATIONSHIP.NEUTRAL then
                            cxt:Dialog("DIALOG_TRICK_UNKNOWN" )
                        end
                    end )
                    :Negotiation{
                        situation_modifiers =
                        {
                            { value = 10, text = cxt:GetLocString("SIT_MOD") }
                        },
                        on_success =  function(cxt)
                            cxt:Dialog("DIALOG_TRICK_SUCCESS")
                            cxt.quest:GetCastAgent("target"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            cxt.quest:Complete("intercept_convoy")
                            cxt.quest:Activate("return_for_reward")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_TRICK_FAIL")
                            cxt:Opt("OPT_ATTACK")
                                :Battle{
                                    on_win = function(cxt)
                                        cxt:Dialog("DIALOG_WON_BATTLE")
                                        cxt.quest:GetCastAgent("target"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                        cxt.quest:Complete("intercept_convoy")
                                        cxt.quest:Activate("return_for_reward")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }
                        end,
                    }
            else
                cxt:Opt("OPT_INTIMIDATE")
                    :Dialog("DIALOG_INTIMIDATE")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        situation_modifiers =
                        {
                            { value = 10, text = cxt:GetLocString("SIT_MOD") }
                        },
                        on_success =  function(cxt)
                            cxt:Dialog("DIALOG_CONVINCED")
                            cxt.quest:GetCastAgent("target"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            cxt.quest:Complete("intercept_convoy")
                            cxt.quest:Activate("return_for_reward")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_INTIMIDATE_FAIL")
                            cxt:Opt("OPT_ATTACK")
                                :Battle{
                                    on_win = function(cxt)
                                        cxt:Dialog("DIALOG_WON_BATTLE")
                                        cxt.quest:GetCastAgent("target"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                                        cxt.quest:Complete("intercept_convoy")
                                        cxt.quest:Activate("return_for_reward")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }
                        end,
                    }
            end
        end)


QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_PAYMENT = "Return {target} to {giver}",
        DIALOG_COMPLETE = [[
            player:
                !overthere
                $happyCocky
                Here you go. One Luminari, alive and... well, alive.
            target:
                !left
                This one leaves puddles in her footsteps, I think, {giver}. 
                I'm unharmed.
            agent:
                !happy
                $happyThanks
                Ah, that's wonderful news! 
            player:
                !left
            agent:
                $happyThanks
                Thank you, {player}. Hesh's tendrils truly quiver where you walk.
            player:
                !wince2
                $neutralDubious
                That's... yikes. 
                What about my payment?
            agent:
                !agree
                $happyDefault
                Yes, yes, of course. Profits to those who embrace the Abyss.                 
        ]],

    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_PAYMENT")
            :Dialog("DIALOG_COMPLETE")
            :CompleteQuest()
            :Fn(function(cxt)
                local saved = cxt.quest:GetCastAgent("target")
                saved:Dismiss()
                saved:OpinionEvent( OPINION.SAVED_LIFE )
                cxt.quest:GetCastAgent("giver"):OpinionEvent( OPINION.APPROVE )

            end)
    end)


QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            agent:
                !interest
                $neutralGreeting
                You're a hunter, right? So you do jobs for pay?
            player:
                !crossed
                $neutralDubious
                Not <i>all</i> jobs. And only for certain amounts of pay. But you've got the idea.
            agent:
                !overthere
                $neutralDirect
                My friend {target} was on a mission through the more rugged parts of Murder Bay, but I haven't heard from {target.himher} in days.
                $scaredDefault
                {target.HeShe}'s a Luminari, trained in combat. It's hard to believe some miscreants managed to get the jump on {target.himher}.
                But {target.heshe}'s missing all the same. I've petitioned the priests in Supplicant City to investigate but I worry they'll come too late.
                !permit
                Will you look for {target.himher}?                
        ]],
        DIALOG_ACCEPTED = [[
            player:
                !agree
                $neutralDirect
                A job for pay. You got yourself a deal.
            agent:
                $miscRelieved
                Thank you, hunter. This is a huge relief. 
                !salute
                May your path keep to the shallows.  
        ]]

    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "intercept_convoy", "DIALOG_OFFER", function() cxt:Dialog("DIALOG_ACCEPTED") end)
    end)

