local QDEF = QuestDef.Define{
        title = "Internal Affairs",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{giver} wants you to beat up {target}. But keep the violence covert, and be careful not to kill {target.himher}.]],
        icon = engine.asset.Texture("icons/quests/internal_affair.tex"),
        focus = QUEST_FOCUS.COMBAT,
        --don't get involved in fights directly

        get_fight_support = function(quest, agent, target_agent)
            if agent == quest:GetCastMember("giver") and target_agent == quest:GetCastMember("target") then
                return SUPPORT_ACTION.STAY_OUT, SUPPORT_REASON.QUEST
            end
        end,
        
        avoids = 
        {
            {"giver", "target"},
        },

        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("target") then
                fighter:AddCondition("WANTED_ALIVE")
            end
        end,

    }

QDEF:AddCast{
    cast_id = "giver",
    provider = true,

    condition = function(agent, quest)
        return QuestUtil.CanDoAdmiraltyGoonQuest(agent)
    end,
}

:AddCast{
    cast_id = "target",
    cast_fn = function( quest, t )
        local agent = quest:CreateSkinnedAgent( "ADMIRALTY_GOON" )
        quest:GetCastMember("giver").social_connections:SetRelationship(RELATIONSHIP.HATED, agent)
        table.insert(t, agent)

    end,
    events = 
    {
        agent_retired = function( quest, agent )
            if not quest:IsActive() then
                quest:Fail()
            end
        end,

        --[[aspects_changed = function( quest, agent, added, aspect )
            if added and is_instance( aspect, Aspect.Intimidated ) then
                if quest:IsActive("beat_target") then
                    quest:Complete( "beat_target" )
                    quest:Activate("return_and_choose")
                end
            end
        end
        --]]
    }
}


:AddObjective{
    id = "beat_target",
    title = "Beat up or intimidate {target}",
    desc = [[Intimidate with your words or with your fists, whichever is sharper.]],
    mark = { "target" },
    combat_targets = {"target"},
    negotiation_targets = {"target"},
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver#agent}",
    desc = [[Get paid for your work, or you may as well have never done it.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}

:AddObjective{
    id = "return_and_choose",
    title = "Get reward from {giver#agent} or get them to talk",
    desc = [[Get paid for your work or get {giver} talking about their dealings.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}



QDEF:AddAttract("beat_target", "target",
    [[
        agent:
            !dubious
            $neutralDubious
            What? Why are you looking at me all funny like that? 
            I got something on my face?
    ]])

QDEF:AddAttract("beat_target", "giver",
    [[
        agent:
            !eureka
            $miscNostalgic
            Can't wait for you to teach {target} some respect.
    ]])


QDEF:AddConvo("beat_target", "target")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_ATTACK = "Beat up {target}",
        DIALOG_ATTACK = [[
            player:
                !angry_accuse
                $angryDefault
                Seems I've been hired to put you in your place. So ... I'm gonna do that.
            agent:
                !placate
                $neutralTakenAback
                What? Did {giver} put you up to this?
                You know that sorry sod is nothing but a dishonorable cheat.
            player:
                !fight
                $angryHostile
                   Don't care. Now, duke's up.
        ]],
        DIALOG_WON_KILLED= [[
            agent:
                !exit
            * That got a little out of hand. You had better keep a low profile around {giver}, now.
        ]],
        DIALOG_WON_SURRENDERED = [[
            agent:
                !injured
            player:
                !cruel
                $angryCruel
                You've had enough?
            agent:
                $angrySeething
                Tell {giver} to watch {giver.hisher} back. You too, grifter.
                You gave me a beating to remember, and remember I will.
        ]],
        OPT_INTIMIDATE = "Intimidate {target} for {giver}",
        DIALOG_INTIMIDATE = [[
            player:
                !crossed
                $neutralDirect
                You've made a few enemies, you know that? 
            agent:
                !chuckle
                $angryCruel
                Sure I have. Wouldn't be worth my salt otherwise. What's it to you, hunter?
        ]],
        DIALOG_INTIMIDATED = [[
            player:
                $angryCruel
                Consider this a friendly warning. Someone <i>soft</i> hired me to sort you out.
                !throatcut
                But I guarantee they aren't the worst of your concerns. So you behave now, or the next time you see me?
                $angrySeething
                I'll be looking down on you while you bleed out. Understand?
            agent:
                !scared
                $scaredFearful
                H-hey, sure, I get it. Burned one too many bridges, and I have no interest in swimming to shore. 
                But an enterprising person like yourself oughtta know: {giver} is a real piece of work. 
                I've been trying to expose {giver.hisher} corruption for ages. There'd be a big reward in it, if someone were to do it.
                But if you really want me to drop it, your message has been received. I'll drop it. 
            player:
                !angry_accuse
                $angryDefault
                Good. And don't forget it.
            agent:
                !exit
            * The job is done, but it might be worth considering that extra reward.
        ]],
        DIALOG_NOT_INTIMIDATED = [[
            player:
                !interest
                $neutralDirect
                Someone is <i>very angry</i>, and thinks you should be nicer from now on.
            agent:
                !chuckle
                $angryCruel
                Oh yeah? Sounds like <i>someone</i> can't handle a little conflict. I'm not afraid of anyone like that.
                You run on back to {giver} and tell {giver.himher} that {giver.heshe}'ll have to do better than that if {giver.heshe} wants me out of the running.
                !interest
                $happyCocky
                That is who hired you, isn't it? {giver}?
            player:
                !forgetful
                $neutralTakenAback
                I keep strict client confidentiality.
            agent:
                !cruel
                $happyDefault
                Uh huh. Sure you do.
        ]]
    }
    :Attract(function(cxt)     
            return "STATE_CONFRONT"
        end)
    :State("STATE_CONFRONT")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !dubious
                    $neutralDubious
                    What? Why are you looking at me all funny like that? 
                    I got something on my face?
            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)
        :Hub(function(cxt) 
                cxt:Opt("OPT_INTIMIDATE")
                    :Dialog("DIALOG_INTIMIDATE")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.HOSTILE | NEGOTIATION_FLAGS.INTIMIDATION,
                        on_success = function() 
                            cxt:Dialog("DIALOG_INTIMIDATED")
                            cxt:GetAgent():AddAspect("intimidated")
                            
                            cxt.quest:Complete("beat_target")
                            cxt.quest:Activate("return_and_choose")
                            
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                        on_fail = function() 
                            cxt:Dialog("DIALOG_NOT_INTIMIDATED")
                        end,
                    }


                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{
                        on_win = function(cxt) 
                            if cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_WON_KILLED")
                            else
                                cxt:Dialog("DIALOG_WON_SURRENDERED")
                            end
                            cxt.quest:Complete("beat_target")
                            cxt.quest:Activate("return_for_reward")
                            StateGraphUtil.AddEndOption(cxt)
                        end,

                    }    
        end)


QDEF:AddConvo("return_for_reward", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_REPORT = "Tell {agent} about {target}",
        DIALOG_REPORT = [[
            player:
                !happy
                !greeting
                $happyGreeting
                I think your {target} problems are in the past, friend.
            agent:
                !handwring
                $happyAmazed
                {target.HeShe} got the message, did {target.heshe}? Tell me all the details.
            player:
                !overthere
            * You tell {agent} about your encounter with {target}, making sure to gently embellish some of the more savory details.
            player:
                !agree
                $happyCocky
                Let's just say {target}'s gonna be laundering their skivvies where no one can see.
            agent:
                !happy
                $happyThanks
                That's a gift. Thanks for deliverin', hunter. You do good work.
        ]],

        DIALOG_REPORT_DEAD = [[
            * As you approach, {agent}'s shoulders gather against {agent.hisher} neck like a thunderhead looming on the horizon.
            player:
                !bashful
                $miscEmbarrassed
                Hey, so, uh—
            agent:
                !angry
                $angrySeething
                You flea-headed sandworm! What the <i>Hesh</i> did I tell you?!
            player:
                !scared_shrug
                $miscEmbarrassed
                Well—
            agent:
                !angry_accuse
                $angrySeething
                Everyone's already talkin' about the hunter who went and got {target} killed! And then you come straight to me?!
                I gotta get outta here—before anyone sees us talking. Don't let me catch you looking for me again!
                !exit

        ]],
        DIALOG_FORGIVE = [[
            agent:
                $neutralResigned
                Fine. You had your reasons.
                !point
                $angryPatienceLost
                Don't cross me again.
            * {agent} turns and leaves.
        ]],
        DIALOG_FAIL = [[
            agent:
                $angryHostile
                You brought this on yourself, hunter, by going and acting the fool! Now hands up—'cause you're under arrest!
                !fight
            * {agent} attacks you!
        ]],
    }

    :Hub(function(cxt) 
        
        local target = cxt.quest:GetCastMember("target")
        if target:IsDead() then
            cxt:Opt("OPT_REPORT")
                :Dialog("DIALOG_REPORT_DEAD")
                :Fn( function( cxt )
                    cxt.quest:Fail()
                    StateGraphUtil.AddEndOption(cxt)
                    -- cxt:DoNegotiation{
                    --     on_success = function() 
                    --         cxt:Dialog("DIALOG_FORGIVE")
                    --         cxt.quest:Complete()
                    --         StateGraphUtil.AddEndOption(cxt)
                    --     end,
                    --     on_fail = function()
                    --         cxt:Dialog("DIALOG_FAIL")
                    --         cxt.quest:Fail()
                    --         cxt:DoBattle{
                    --             flags = BATTLE_FLAGS.SELF_DEFENCE,
                    --         }
                    --     end 
                    -- }
                end )
        else
            cxt:Opt("OPT_REPORT")
                :Dialog("DIALOG_REPORT")
                :CompleteQuest()
        end
    end)

QDEF:AddConvo("return_and_choose", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_REPORT = "Tell {agent} about {target}",
        DIALOG_REPORT = [[
            player:
                !happy
                !greeting
                $happyGreeting
                I think your {target} problems are in the past, friend.
            agent:
                !handwring
                $happyAmazed
                {target.HeShe} got the message, did {target.heshe}? Tell me all the details.
            player:
                !overthere
            * You tell {agent} about your encounter with {target}, making sure to gently embellish the parts that strike {agent}'s fancy.
            player:
                !agree
                $neutralThoughtful
                Oh, {target.heshe} did mention that {target.heshe} was investigating you and trying to frame you for corruption. 
                With {target} lying low, you won't know where your enemy is at.
            agent:
                !thought
                Hm. Good point.
                !give
                Still, my old problem is solved. And you've a little bonus for going above and beyond.
        ]],
        OPT_EXPOSE = "Convince {agent} to expose {agent.hisher} secrets",
        DIALOG_FAIL = [[
            agent:
                !angry
                Sounds like you got a lot of questions, but nothing to show for it.
                We're done here.
        ]],
        DIALOG_EXPOSE = [[
            player:
                !greeting
                $neutralGreeting
                I hear rumors that you have some side dealings going on...
            agent:
                !cagey
                $neutralWhatever
                Might do. Not saying I do, but I might. Why would you be worth telling?
        ]],
        DIALOG_ARREST = [[
            player:
                $miscPersuasive
                I do good work, don't I? If you've got an angle, maybe I can help it along.
            agent:
                !dubious
                $neutralResigned
                Yeah? Maybe you can. 
                And Hesh knows I've been hoarding my secrets long enough. Might even be a relief to let some of them go.
                * Unburdened, {agent} explains all the details of {agent.hisher} operations, down to the smallest detail.
            target:
                !left
                !point
                $happyCocky
                Ha! I've finally caught you red-handed. {agent}, you're under arrest!
            player:
                !left
                !bashful
                $miscEmbarrassed
                Whoops.
            agent:
                $angrySeething
                {player}! You backstabbing liar!
            player:
                !shrug
                $neutralWhatever
                Just doing my civic duty and stuff.
            target:
                !right
                $happyThanks
                !give
                Thanks for your help, {player}. A deal's a deal—here's your compensation.
            player:
                !take
        ]],
    }

    :Hub(function(cxt) 

        cxt:Opt("OPT_REPORT")
            :Dialog("DIALOG_REPORT")
            :Fn(function(cxt)
                cxt.quest.param.reward_negotiated_bonus = cxt.quest.param.reward_negotiated_bonus and cxt.quest.param.reward_negotiated_bonus + 30 or 30
                cxt.quest:VerifyRewards()
            end)
            :CompleteQuest()

        cxt:Opt("OPT_EXPOSE")
            :Dialog("DIALOG_EXPOSE")
            :Negotiation{
                on_success = function() 
                    local target = cxt.quest:GetCastAgent("target")
                    target:MoveToLocation(cxt.location)
                    cxt:Dialog("DIALOG_ARREST")
                    cxt.quest.quest_provider = target
                    target:OpinionEvent(OPINION.HELPED)
                    target:RemoveAspect("intimidated", true)
                    local old_giver = cxt.quest:GetCastAgent("giver")
                    old_giver:OpinionEvent(OPINION.BETRAYED)
                    old_giver:MoveToLimbo()
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                    StateGraphUtil.AddEndOption(cxt)
                end,

                on_fail = function()
                    cxt:Dialog("DIALOG_FAIL")
                    cxt.quest:Fail()
                    StateGraphUtil.AddEndOption(cxt)
                end 
            }

    end)


QDEF:AddConvo(nil, "giver")
    :Loc{
        DIALOG_OFFER = [[
            player:
                !interest
                $neutralGreeting
                I hear you have a problem that needs rectifyin'.
                !thumb
                $happyCocky
                I'm a hunter. I can help with that.
            agent:
                !thought
                $neutralThoughtful
                Yeah? This <i>could</i> use a professional touch. 
                I have this... colleague. {target}. {target.HeShe}'s always undermining me in front of the officers.
                !angry_shrug
                $angryDefault
                I think {target.heshe}'d be happiest if I wound up dead in a ditch and {target.heshe} wound up claiming my bunk. 
            player:
                !dubious
                $neutralDubious
                Is it a nice bunk?
            agent:
                !neutral
                !agree
                $neutralThoughtful
                It's next to the window. Has a nice view of the less sludgy part of the bay.
                I'd like to shut {target} down myself but there are <i>rules</> about in-fighting, and a punishment to match.
                !overthere
                But those rules don't apply to <i>you</>...
        ]],
        DIALOG_POST = [[
            player:
                !happy
                $happyCocky
                I can offer you the basic "blisters & bruises" package for a steal. 
                That kind of violence falls well within my job description. 
            agent: 
                !agree
                $neutralDirect
                Just give 'em a good scare. You can rough {target.himher} up a little, too. 
                But <b>do not</b> kill {target.himher}. You do that, there'll be an inquiry at <i>best</i>. 
                !palm
                $miscDepressed
                And at worst, I'll have to take {target}'s patrols. Ugh. 

        ]]
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "beat_target", "DIALOG_OFFER", function() 
                cxt:Dialog("DIALOG_POST")
            end)
    end)


