
local LOCATION_DEF =
{
    id = "BUYER_HOUSE",
    name = "Meeting point",
    plax = "INT_RichHouse_1",
    indoors = false,
    map_tags = {"residence"},
    show_agents= true,
}

if not Content.GetLocationContent(LOCATION_DEF.id) then
    Content.AddLocationContent(LOCATION_DEF)
end

--Mults for the rewards
local SMUGGLER_BOUNTY = 2.5
local SPREE_OFFER = 1.25
local BUYER_EXTORT_AMOUNT =  0.25

local QDEF = QuestDef.Define{
        title = "Travel Insurance",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        icon = engine.asset.Texture("icons/quests/jake_escort.tex"),
        desc = [[{giver} needs you to safely escort one of {giver.hisher} smugglers to the buyer.]],
        focus = QUEST_FOCUS.COMBAT,

        -- icon = engine.asset.Texture("icons/quests/attack_rival.tex"),
        on_destroy = function( quest )
            local smuggler = quest:GetCastMember("smuggler")
            if smuggler and smuggler:IsInPlayerParty() then
                smuggler:Dismiss()
            end
        end,
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent)
            return agent:GetFactionID() == "JAKES"
        end,
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent( "JAKES_SMUGGLER" )
            table.insert( t, agent )
        end,
    }
    
    :AddCast{
        cast_id = "smuggler",
        
        condition = function(agent)
            return agent:GetFactionID() == "JAKES" and not agent:GetBrain():IsOnDuty() and agent:GetCombatStrength() <= 2
        end,
        events = 
        {          
            dismissed = function( quest )
                if not quest.param.betrayed then
                    quest:Fail()
                end
            end
        }
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent( "JAKES_RUNNER" )
            table.insert( t, agent )
        end,
    }

    :AddCast{
        cast_id = "buyer",
        condition = function(agent)
            return agent:GetContentID() == "WEALTHY_MERCHANT"
        end,
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent( "WEALTHY_MERCHANT" )
            table.insert( t, agent )
        end,
    }

    :AddLocationCast{
        cast_id = "buyer_house",
        when = QWHEN.MANUAL,
        no_validation = true,
    }

    :AddObjective{
        id = "meet_smuggler",
        title = [[Meet up with {smuggler}]],
        desc = "Find {smuggler} and begin the escort.",
        mark = { "smuggler" },
        
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
    }

    :AddObjective{
        id = "escort_to_buyer",
        title = [[Escort {smuggler}]],
        desc = "Travel with {smuggler} to the buyer.",
        mark = { "buyer" },
        
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },

        on_activate = function( quest)
            local location = Location( LOCATION_DEF.id )
            assert(location)
            TheGame:GetGameState():AddLocation(location)
            quest:AssignCastMember("buyer_house", location )
            quest:GetCastAgent("buyer"):MoveToLocation(location)
        end,
    }

    :AddObjective{
        id = "trade_alone",
        title = [[Bring the package to {buyer}]],
        desc = "Complete the trade with {buyer} on your own.",
        mark = { "buyer" },
        
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
    }

    :AddObjective{
        id = "return_alone",
        title = "Give payment to {giver}",
        desc = [[Return to {giver} with the payment to claim your reward.]],
        ready_for_completion = true,
        mark = { "giver" },
        terminal = true,
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
    }

    :AddObjective{
        id = "return_together",
        title = "Give payment to {giver}",
        desc = [[Return to {giver} with the payment to claim your reward.]],
        ready_for_completion = true,
        mark = { "giver" },
        terminal = true,
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
    }

QDEF:AddConvo("meet_smuggler", "smuggler")
    :Loc{
        DIALOG_INTRO = [[
            agent:
                $miscMocking
                !crossed
                You the escort? Hope you're sturdy. The last one only made it a week!
            player:
                $neutralJoke
                !happy
                Encouraging words. I'm sure we'll be fine.
        ]],
        OPT_BEGIN_ESCORT = "Tell {agent} you're ready to go",
        DIALOG_BEING_ESCORT = [[
            player:
                Let's get going.
        ]],    
    }

    :Hub(function(cxt) 
        if not cxt.quest.param.intro_played then
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest.param.intro_played = true
        end
        
        cxt:Opt("OPT_BEGIN_ESCORT")
            :PreIcon(global_images.invite)
            :Dialog("DIALOG_BEING_ESCORT")
            :Fn(function(cxt)
                if Util.IsPlayerWorkingWithFaction("BANDITS") then
                    cxt.quest.param.use_spree = true
                elseif Util.IsPlayerWorkingWithFaction("ADMIRALTY") then
                    cxt.quest.param.use_spree = false
                else
                    if math.random() < .5 then
                        cxt.quest.param.use_spree =  true
                    else
                        cxt.quest.param.use_spree =  false
                    end
                end
                cxt.quest:GetCastAgent("smuggler"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                cxt.quest:Complete("meet_smuggler")
                cxt.quest:Activate("escort_to_buyer")
            end)
    end)
    
    
QDEF:AddConvo("escort_to_buyer")
    :Confront(function(cxt)
        if not cxt.quest.param.has_been_stopped then
            if cxt.location:HasTag("in_transit") then
                if cxt.quest.param.use_spree then
                    return "STATE_SPREE_INTERRUPT"
                else
                    return "STATE_ADMIRALTY_INTERRUPT"
                end
            end
        end

        if cxt.location == cxt.quest:GetCastMember("buyer_house") or (cxt.location and cxt.location == cxt.quest:GetCastMember("buyer"):GetLocation()) then
            return "STATE_BUYER"
        end
    end)

    :State("STATE_SPREE_INTERRUPT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    $miscPersuasive
                    Listen up, Grifter. We don't care about you, we just want the package.
                player:
                    $neutralDubious
                    And why would I give it to you?
                agent:
                    $miscPersuasive
                    What are they paying you? I guarantee we'll pay more.
                    How does {1#money} sound?

                *** {agent} wants to buy the package from you.
            ]],
            OPT_CONVINCE = "Pay {agent} to back off",
            DIALOG_CONVINCE = [[
                player:
                    $neutralWhatever
                    How about I pay <i>you</> instead, and you just forget about the package?
                agent:
                    !thought
                    $neutralDubious
                    Let me heft that bag of shills and I'll tell you if it's heavy enough.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                agent:
                    !take
                    $neutralResigned
                    Fine, but don't think we couldn't take it if we wanted to.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                agent:
                    !point
                    $angryPatienceLost
                    Nah, not heavy enough. I think we want that package.
            ]],
            OPT_FIGHT = "Defend {smuggler}",
            DIALOG_FIGHT = [[
                player:
                    $neutralThanks
                    Sorry, I've got a contract that I'd like to honour.
                agent:
                    $angryHostile
                    Fine, we'll do it the hard way.
            ]],
            DIALOG_BEAT = [[
                agent:
                    !injured
                    $scaredAnguished
                    Fine, go! That stupid delivery isn't worth dying for.
            ]],
            DIALOG_KILLED = [[
                * With the Spree thugs defeated, you hastily get on with your trip.
            ]],
            OPT_ACCEPT = "Kill {smuggler} and accept their offer",
            DIALOG_ACCEPTED = [[
                * You swiftly pull out your dagger and kill {smuggler} before {smuggler.heshe} can react.
                player:
                    $happyCocky
                    Sorry about the mess. Let's talk business.
                agent:
                    $happyAmazed
                    Fantastic! Here's the money. I think you'll find it fair.
            ]],            
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest.param.has_been_stopped = true
                cxt.quest.param.scaled_spree_offer = math.floor(SPREE_OFFER * cxt.quest:GetParameter("rewards"))
                local thugs = CreateCombatParty("BANDIT_PATROL", cxt.quest:GetRank(), cxt.location, true, function(agent) return agent:GetRelationship() < RELATIONSHIP.LIKED end)
                cxt.encounter:SetPrimaryCast(thugs[1])
                cxt:Dialog("DIALOG_INTRO", cxt.quest.param.scaled_spree_offer)
            end

            local price = CalculatePayment( cxt:GetAgent(), cxt.quest:GetRank()^2 * 10 + 10 )

            if not cxt.quest.param.tried_convince then
                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :RequireMoney( price, { no_scale = true } )
                    :Negotiation{
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                            cxt.caravan:PayMoney( price, cxt:GetAgent() )
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_FAIL")
                            cxt.quest.param.tried_convince = true
                        end
                    }
            end

            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = {"smuggler"},
                    on_win = function(cxt)
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_KILLED")
                        else
                            cxt:Dialog("DIALOG_BEAT")
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                }

            cxt:Opt("OPT_ACCEPT")
                :Dialog("DIALOG_ACCEPTED")
                :ReceiveMoney(cxt.quest.param.scaled_spree_offer)
                :Fn(function(cxt)
                    cxt.quest:GetCastAgent("giver"):OpinionEvent(OPINION.BETRAYED)
                    AgentUtil.CommitMurder(cxt.quest:GetCastAgent("smuggler"))
                    cxt.quest:GetCastAgent("smuggler"):Kill()
                    cxt.quest:Fail()
                end)
                :Travel()
            

        end)

    :State("STATE_ADMIRALTY_INTERRUPT")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !angry_accuse
                    $angryGreeting
                    Stop right there! Your pal {smuggler} is a wanted criminal. We're taking {smuggler.himher} in.
                player:
                    $neutralDubious
                    !placate
                    Whoa, hold up. I was just hired to do a job—and I need {smuggler} to do it.
                agent:
                    $miscPersuasive
                    !cruel
                    Is that so? How about this: the bounty is {1} shills. Pay it, and we'll consider it a fair trade.
            ]],
            OPT_CONVINCE = "Convince {agent} to back off",
            DIALOG_CONVINCE = [[
                player:
                    $miscPersuasive
                    !interest
                    Listen, we're just going about our business. Just look the other way, would you?
                agent:
                    $neutralTakenAback
                    !crossed
                    And why would we do that?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    $miscPersuasive
                    Because the bounty's not going anywhere, but right now, {smuggler}'s under my protection. 
                    !overthere
                    So you can fight us both, or wait until I'm not around and your odds are more favorable. 
                agent:
                    $neutralResigned
                    !thought
                    Hrm. That's a fair point. Fine, you can go along your way. 
                player:
                    $neutralThanks
                    Thank you for seeing things my way.
                * You and {smuggler} leave the Admiralty officers behind—to wait for {smuggler} on another day.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player: 
                    !shrug
                    {smuggler} and I are both very dangerous! 
                agent:
                    $angryHostile
                    !angry_accuse
                    Uh huh. So you want me to let you both go? I don't think so.
            ]],
            OPT_FIGHT = "Defend {smuggler}",
            DIALOG_FIGHT = [[
                player:
                    !angry
                    $angryHostile
                    {smuggler}'s under my protection. Since you can't take a <i>hint</i>, how about you take a beating?
                    !fight
            ]],
            DIALOG_BEAT = [[
                player:
                    $angryPatienceLost
                    !angry_accuse
                    Had enough?
                agent:
                    !injured
                    $miscEmbarrassed
                    Fine, fine! You made your point! {smuggler} is outta our league. Take {smuggler.himher} and go.
                 * You and {smuggler} make a hasty exit, before reinforcements can arrive to sour your getaway.
            ]],
            DIALOG_KILLED = [[
                smuggler:
                    !right
                    !tired 
                    $happyTakenAback
                    Huh! Didn't expect that kind of performance outta you, to be honest. I'm impressed!
                player:
                    !dubious
                    $miscRelieved
                    Any more surprises I should know about?
                smuggler:
                    !thought
                    I don't... think so? But maybe we should make tracks, just in case.
                * You grab {smuggler} and leave the bodies behind before anyone can catch you misbehaving. 
            ]],
            DIALOG_SMUGGLER_KILLED = [[
                * You may have won the fight, but with {smuggler} dead, you'll have to carry on alone.
            ]],
            OPT_ACCEPT = "Pay them off",
            DIALOG_ACCEPTED = [[
                player:
                    $angryDefault
                    I'm not sure {smuggler.heshe}'s worth it, but fine, here's the money.
                    !give
                agent:
                    !take
                    !happy
                    $happyCocky
                    Don't bounty hunters normally <i>collect</i> on bounties? Thank you kindly, hunter. 
                * The officers leave you to the dusty road, laughing as they go.
                smuggler:
                    !right
                    Well, can't say I expected—
                player:
                    !handwave
                    $angryDefault
                    Stow it. We've got work to do. Don't make me regret my investment. 
                * You continue along your route.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest.param.has_been_stopped = true
                cxt.quest.param.scaled_bounty = math.floor(SMUGGLER_BOUNTY * QuestUtil.CalculateReward( cxt.quest ))
                local guards = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank(), cxt.location, true, function(agent) return agent:GetRelationship() < RELATIONSHIP.LIKED end)
                cxt.encounter:SetPrimaryCast(guards[1])
                cxt:Dialog("DIALOG_INTRO", cxt.quest.param.scaled_bounty)
            end

            if not cxt.quest.param.tried_convince then
                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation{
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_FAIL")
                            cxt.quest.param.tried_convince = true
                        end
                    }
            end

            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = {"smuggler"},
                    on_win = function(cxt)
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_KILLED")
                        else
                            cxt:Dialog("DIALOG_BEAT")
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                }

            cxt:Opt("OPT_ACCEPT")
                :PreIcon(global_images.hire)
                :Dialog("DIALOG_ACCEPTED")
                :RequireMoney(cxt.quest.param.scaled_bounty)
                :Fn(function(cxt)
                    cxt.caravan:PayMoney( cxt.quest.param.scaled_bounty, cxt:GetAgent() )
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)

    -- :State("STATE_ACCEPTED_BOUNTY")
    --     :Loc{
    --         OPT_ASK_FOR_PACKAGE = "Ask for the package",
    --         DIALOG_ASK = [[
    --             player:
    --                 $miscPersuasive
    --                 Before you go, {smuggler} has something I need. If you could hand me that package, I'll be on my way.
    --             agent:
    --                 $neutralThoughtful
    --                 !thought
    --                 Oh yeah? And why shouldn't we just confiscate it?
    --         ]],
    --         DIALOG_ASK_SUCCESS = [[
    --             player:
    --                 !shrug
    --                 Because I've been cooperating just fine until now, haven't I?
    --             agent:
    --                 $neutralDubious
    --                 !sigh
    --                 Alright, alright, just take the package and get out of the way.
    --             * {agent} tosses the package at your feet before leaving with {smuggler}.
    --         ]],
    --         DIALOG_ASK_FAIL = [[
    --             agent:
    --                 $miscPersuasive
    --                 This is property of a wanted criminal. We'll be taking it with us.
    --             * Before you can get another word in, {agent} leaves with {smuggler}.
    --         ]],
    --         OPT_LEAVE = "Leave",
    --         DIALOG_LEAVE = [[
    --             player:
    --                 $happyCocky
    --                 Glad we could help each other out.
    --             * You leave before any trouble can break out.
    --         ]],   
    --     }
    --     :Fn(function(cxt)
    --         cxt:Opt("OPT_ASK_FOR_PACKAGE")
    --             :Negotiation{
    --                 on_success = function(cxt)
    --                     cxt:Dialog("DIALOG_ASK_SUCCESS")
    --                     cxt.quest:Complete("escort_to_buyer")
    --                     cxt.quest:Activate("trade_alone")
    --                     StateGraphUtil.AddLeaveLocation(cxt)
    --                 end,
    --                 on_fail = function(cxt)
    --                     cxt.quest:Fail()
    --                     StateGraphUtil.AddLeaveLocation(cxt)
    --                 end
    --             }

    --         cxt:Opt("OPT_LEAVE")
    --             :Dialog("DIALOG_LEAVE")
    --             :Fn(function(cxt)
    --                 cxt.quest:Fail()
    --                 StateGraphUtil.AddLeaveLocation(cxt)
    --             end)
    --     end)

    :State("STATE_BUYER")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !dubious
                    $neutralDubious
                    What can I help you with?
                player:
                    $neutralGreeting
                    {giver} sent us. We have a delivery for you.
                agent:
                    $neutralThanks
                    I thought that might be it. Can't be too careful, though.
                * {smuggler} hands the package to {agent}, who responds with a sack of jingling shills in return.
            ]],
            OPT_EXTORT = "Extort {buyer} for more money",
            DIALOG_EXTORT = [[
                player:
                    !shrug
                    $miscPersuasive
                    Of course, the roads are dangerous out there. Extra precautions had to be taken. 
                    So there's gonna be a transportation fee, too. 
                agent:
                    $neutralDubious
                    Oh yeah? And how much would that be?
                player:
                    !happy
                    Well, since you're a loyal customer, you get a preferred rate.
                    {1#money}, because we like you. 
            ]],
            DIALOG_EXTORT_SUCCESS = [[
                player:
                    $miscPersuasive
                    After all, how often do you get service with a smile in the griftlands, huh?
                agent:
                    $neutralResigned
                    Easy to smile when you're serving it with a grift, more like. 
                    All right, I'll pay it. But tell {giver} this is the last time I pay any hidden fees.
                player:
                    $happyCocky
                    Gladly. I'll even deliver the message for free.
                * You pocket the extra cash, still smiling as you leave.
            ]],
            DIALOG_EXTORT_FAIL = [[
                player:
                    !interest
                    Besides, I hear a tip is customary on the mainland.
                agent:
                    !angry
                    $angryPatienceLost
                    This look like the mainland to you, grifter?
                    Keep your fees, and tell {giver} the next delivery that comes with a mark-up will <i>get</i> marked-up. 
                player:
                    $neutralResigned
                    Yeah, you got it. Message received.
                * You hurriedly make your exit before {agent} gets any angrier.
            ]],
            OPT_LEAVE = "Take the money and leave",
            DIALOG_TAKE_MONEY = [[
                player:
                    !give
                agent:
                    !take
                    $neutralGreeting
                    Well, it might've taken a while. But at least it's here.
                    Now sod off, I got plans for this that need some privacy.
                player:
                    !dubious 
                    Uh, sure. Enjoy.  
                    !exit
                * With the transaction complete, you make a bee-line for the exit. 
            ]],
            DIALOG_EXTORT_TT = "Extort an extra {1#money} for yourself.",
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.encounter:SetPrimaryCast(cxt.quest:GetCastAgent("buyer"))
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest.param.scaled_extortion_amount = math.max (math.floor(BUYER_EXTORT_AMOUNT * QuestUtil.CalculateReward( cxt.quest )), 40)
            end

            cxt:Opt("OPT_LEAVE")
                :PreIcon(global_images.hire)
                :Dialog("DIALOG_TAKE_MONEY")
                :Fn(function(cxt)
                    cxt.quest:Complete("escort_to_buyer")
                    cxt.quest:Activate("return_together")
                end)
                :Travel()

            cxt:Opt("OPT_EXTORT")
                :PostText("DIALOG_EXTORT_TT", cxt.quest.param.scaled_extortion_amount)
                :Dialog("DIALOG_EXTORT", cxt.quest.param.scaled_extortion_amount)
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_EXTORT_SUCCESS")
                        cxt.enc:GainMoney(cxt.quest.param.scaled_extortion_amount)
                        cxt.quest:Complete("escort_to_buyer")
                        cxt.quest:Activate("return_together")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_EXTORT_FAIL")
                        cxt:GetAgent():OpinionEvent(OPINION.TRIED_TO_SWINDLE)
                        cxt.quest:Complete("escort_to_buyer")
                        cxt.quest:Activate("return_together")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                }
            
        end)


QDEF:AddConvo("trade_alone", "buyer")
    :Loc{
        DIALOG_INTRO = [[
            player:
                !left
            agent:
                !right
                !dubious
                $neutralDubious
                What can I help you with?
            player:
                $neutralGreeting
                {giver} sent me to deliver a package for you.
            agent:
                $neutralDubious
                Hmm, I expected one of the usual guys. Are you new?
            player:
                $miscPersuasive
                Uhh, {giver} needed someone with a special skillset this time.
            agent:
                !shrug
            * {smuggler} hands the package to {agent}. {agent} places a sack of money on the table.
        ]],
        OPT_EXTORT = "Extort {buyer} for more money",
        DIALOG_EXTORT = [[
            player:
                $miscPersuasive
                Great, now I just need the transportation fee.
            agent:
                $neutralTakenAback
                {giver.heshe} said what now?
        ]],
        DIALOG_EXTORT_SUCCESS = [[
            agent:
                $neutralResigned
                Hmph, well it's hard to argue with {giver} when {giver.heshe} isn't here, isn't it?
            player:
                $neutralThanks
                Sorry, I'm just the messenger.
        ]],
        DIALOG_EXTORT_FAIL = [[
            agent:
                $angryPatienceLost
                Do you think I was born yesterday? Get out of here!
        ]],
        OPT_LEAVE = "Take the money",
        DIALOG_TAKE_MONEY = [[
            * You take the money.
        ]]
    }
    :Hub(function(cxt)
        if cxt:FirstLoop() then
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest.param.scaled_extortion_amount = math.floor(BUYER_EXTORT_AMOUNT * QuestUtil.CalculateReward( cxt.quest ))
        end

        cxt:Opt("OPT_EXTORT")
            :Dialog("DIALOG_EXTORT")
            :Negotiation{
                on_success = function(cxt)
                    cxt:Dialog("DIALOG_EXTORT_SUCCESS")
                    cxt.enc:GainMoney( cxt.quest.param.scaled_extortion_amount )
                    cxt.quest:Complete("trade_alone")
                    cxt.quest:Activate("return_alone")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end,
                on_fail = function(cxt)
                    cxt:Dialog("DIALOG_EXTORT_FAIL")
                    cxt:GetAgent():OpinionEvent(OPINION.TRIED_TO_SWINDLE)
                    cxt.quest:Complete("trade_alone")
                    cxt.quest:Activate("return_alone")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end
            }

        cxt:Opt("OPT_LEAVE")
            :Dialog("DIALOG_TAKE_MONEY")
            :Fn(function(cxt)
                cxt.quest:Complete("trade_alone")
                cxt.quest:Activate("return_alone")
            end)
            :Travel()
    end)

QDEF:AddConvo("return_alone", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_CONVINCE = "Explain {smuggler}'s tragic demise",
        DIALOG_CONVINCE = [[
            player:
                $miscPersuasive
                So about {smuggler}...
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            agent:
                $neutralResigned
                Well, we knew it was a dangerous job. I'm sure you did your best.
        ]],
        DIALOG_CONVINCE_FAIL = [[
            agent:
                $neutralWhatever
                This all sounds fishy, but hey, you got the job done. 
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_CONVINCE")
            :Dialog("DIALOG_CONVINCE")
            :Negotiation{
                on_success = function(cxt)
                    cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                    StateGraphUtil.AddEndOption(cxt)
                end,
                on_fail = function(cxt)
                    cxt:Dialog("DIALOG_CONVINCE_FAIL")
                    cxt.quest:GetCastAgent("giver"):OpinionEvent(OPINION.DISAPPROVE_MINOR)
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                    StateGraphUtil.AddEndOption(cxt)
                end
            }
    end)

QDEF:AddConvo("return_together", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_REWARD = "Receive payment",
        DIALOG_REWARD = [[
            agent:
                Thanks for your help today, Sal.
        ]],
    }
    :Hub(function(cxt)   
        cxt:Opt("OPT_REWARD")
            :PreIcon(global_images.receiving)
            :Dialog("DIALOG_REWARD")
            :CompleteQuest()
            :Fn(function(cxt)
                cxt.quest:GetCastAgent("smuggler"):Dismiss()
            end)
            :DoneConvo()
    end)


QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            player:
                !interest
                $neutralGreeting
                I heard you have need of a hunter?
            agent:  
                !agree
                $happyGreeting
                You heard right. I got a big delivery today that's gonna make my courier the target of every bandit in Murder Bay.
                !point
                $happyThanks
                So I want you to go with {agent.himher} and keep {agent.himher} <i>and</i> the delivery nice and safe. 
                
        ]],
        DIALOG_ACCEPTED = [[
            player:
                !salute
                $happyCocky
                You got it.
            agent:
                $neutralThanks
                If it comes to it, the courier is worth more to me than the delivery. But the client probably won't agree.
                !shrug
                $neutralWhatever
                So... good luck. 
        ]]

    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "meet_smuggler", "DIALOG_OFFER", function() cxt:Dialog("DIALOG_ACCEPTED") end)
    end)

