local QDEF = QuestDef.Define{
        title = "A Matter of Faith",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{giver} needs your help returning {target} to the Heshian flock, one way or another.]],
        icon = engine.asset.Texture("icons/quests/side_matter_of_faith.tex"),
        focus = QUEST_FOCUS.BALANCED,

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("target"), location = quest:GetCastMember("street_corner") } )
        end,

        on_destroy = function(quest)
            if quest:GetCastMember("giver") and quest:GetCastMember("giver"):IsInPlayerParty() then
                quest:GetCastMember("giver"):Dismiss()
            end
        end
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent)
            return agent:GetContentID() == "ZEALOT" and agent:GetRelationship() >= RELATIONSHIP.NEUTRAL
        end,
    }
    :AddCastFallback{
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "ZEALOT" ) )
        end,
    }

    :AddCast{
        cast_id = "target",
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "PRIEST" ) )
        end,
    }
    
    :AddObjective{
        id = "confront_target",
        title = [[Confront {target}]],
        desc = "Confront {target} with {giver}.",
        mark = { "target" },
    }

    :AddQuestLocation{
        cast_id = "street_corner",
        name = "Street Corner",
        plax = "EXT_door_slums1",
        map_tags = {"city"},
        tags = {"city"},
        indoors = false,
    }


QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            player:
                !interest
                I hear you're looking for some muscle.
            agent:
                Tell me—do you walk in the shallows?
            player:
                !shrug
                I walk where I walk. 
                Listen, if this is going to turn into a sermon—
            agent:
                $neutralResigned
                I suppose you'll say that's not what I'm paying you for?
            player:
                !shrug
                You haven't paid me anything yet. 
                If you've got the shills I'll do what you need done. Just don't expect me to believe in it.
            agent:
                !thought
                Perhaps that's all one can expect in these trying times. 
                {target} has been telling lies about Hesh, and people are listening. I need your help talking some sense into {target.himher}.
        ]],
        DIALOG_ACCEPTED = [[
            player:
                Fine. I'll do it. 
                !happy
                For the money.
                !neutral
            agent:
                Good. I know where {target} is holding court. I'll take you there, and you back me up while I talk some sense into {target.himher}.
            player:
                $neutralDubious
                We're just going to talk?
            agent:
                If we can. If that fails, well, I assume you're armed.
            player:
                !dubious
                Thought priests could only die by drowning.
            agent:
                If {target} can't be turned, then {target.heshe} is drowned already.
                Metaphorically speaking, of course.
        ]]

    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "confront_target", "DIALOG_OFFER", 
            function() 
                cxt:Dialog("DIALOG_ACCEPTED") 
                cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
            end)
    end)


QDEF:AddConvo("confront_target", "target")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !exit
                * You find {target} addressing a crowd of onlookers.
                target:
                    !hesh_greeting
                    !right
                    The Cult would have you believe Hesh is a loveless being—a creature of will and power alone.
                    But does Hesh not make its presence felt in all our lives? And is that not in itself an act of love?
                    !exit
                * The noise from the crowd suggests {target} has given them something to think about. {giver} pulls you aside.
                player:
                    !left
                giver:
                    !right
                    $angryDefault
                    !spit
                    These are not Hesh's teachings! You must stop {target.himher} at once.
            ]],

            OPT_ASK_WHY_YOU = "Ask why {giver} doesn't lead the way",
            DIALOG_ASK_WHY_YOU = [[
                player:
                    !placate
                    Hold up. Why would I be the one doing the talking? You're the Heshian, remember?
                giver:
                    {target} plies the crowd with promises of an accepting god. If I openly forbid it, the notion becomes only more tantalizing.
                    You must undermine the sermon—lead them to doubt.
                player:
                    What does it matter, anyway? What's the harm in letting them believe the jelly loves them?
                giver:
                    $angrySeething
                    !eureka
                    The Abyss cannot be courted by its supplicants! 
                    Hesh is unknowable. To fool themselves only takes them further from enlightenment.
                player:
                    !amused
                    Perish the thought.
            ]],

            OPT_TALK_TO_TARGET = "Talk to {target}",


        }
        :Fn(function(cxt) 
            cxt:TalkTo("giver")            
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function()
                cxt:Question("OPT_ASK_WHY_YOU", "DIALOG_ASK_WHY_YOU")
                cxt:Opt("OPT_TALK_TO_TARGET")
                    :GoTo("STATE_TALK_TO_TARGET")
            end)

        end)

    :State("STATE_TALK_TO_TARGET")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !exit
                giver:
                    !exit
                * You make your way to the front of the crowd.
                player:
                    !left
                target:
                    !right
                    !happy
                    Yes? I can see from your face that you have questions.
            ]],

            OPT_ATTACK = "Sucker punch {target}",
            DIALOG_ATTACK = [[
                player:
                    Not a question, per se.
                    !fight
                    More of a statement.
                target:
                    !scared
                    
            ]],
            DIALOG_WON_AGAINST_PRIEST = [[
                {dead?
                    * {target} slumps to the ground, and the gathered crowd shrieks with outrage.
                    * Looks like you've turned the congregation into a mob.
                }
                {not dead?
                    target:
                        !injured
                    giver:
                        !left
                        {target}! See reason. It's not too late for you to disavow these pungent lies. 
                    target:
                        The only lie is the one in Supplicant City. 
                        Hesh belongs to the people! And they <i>will</i> seize this chance!
                        !exit
                    giver:
                        !exit
                    * {target}'s not wrong. Looks like you've turned the congregation into a mob.
                }
            ]],

            OPT_THREATEN_PRIEST = "Contradict {target}",
            DIALOG_THREATEN_PRIEST = [[
                player:
                    If Hesh is loving, what's up with the derricks?
                target:
                    The derricks, child? 
                    Lumin is a gift. It brings light into the shadows.
                player:
                    !crossed
                    Right, but the folks who plumb it don't exactly get a say, do they?
                    I was sold to the derricks when I was a kid.
            ]],

            DIALOG_THREATEN_PRIEST_SUCCESS = [[
                target:
                    Hesh doesn't task us with anything we're not strong enough to handle, of course.
                player:
                    Uh, tell that to the old-timers who are gonna die out there before they work off their debts.
                * The soft mumbling of the crowd has turned into something more like indigestion. 
                * The preacher has lost {target.hisher} hold on the audience, and they begin to disperse.
                giver:
                    !left
                    !cruel
                    Come with me, {target}. We have a lot to talk about.
                target:
                    {giver}! I told you not to follow me. This is the path Hesh has set for me!
                giver:
                    If you think Hesh wants you hanged for heresy, well... who am I to deny your fate?
                target:
                    We'll see what Hesh truly intends for me!
                    !fight
                * Surprisingly, the priest lashes out. Guess it's a fight or nothing.
            ]],
            DIALOG_THREATEN_PRIEST_FAILURE = [[
                target:
                    And look at you now. Strong. Capable. 
                    What the Cult tells us is Hesh's whims, are actually signs of Hesh's devotion—to <i>us</i>.
                    Tell us truthfully, child: {giver} send you, didn't {giver.heshe}?
                player:
                    ...
                target:
                    !disappoint
                    You see? The only one manipulating our fates is the Cult itself. 
                    These people now know they have the power to choose, that Hesh's love supports them. 
                    And what will they do?
                    !exit
                * The crowd surrounds you and {giver}, weapons drawn. 
                * Seems they've accepted Hesh's love. Too bad they're not the type to pay it forward.
            ]],

            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
            ]],


        }
        :Fn(function(cxt) 
            cxt:TalkTo(cxt.quest:GetCastMember("target"))
            cxt:Dialog("DIALOG_INTRO")            
            
            local rabble = CreateCombatParty("CIVILIAN_PATROL", cxt.quest:GetRank()+1, cxt.location)
            local hinders = {}
            for k,v in pairs(rabble) do
                table.insert(hinders, v)
            end


            cxt:Opt("OPT_THREATEN_PRIEST")
                :Dialog("DIALOG_THREATEN_PRIEST")
                :Negotiation{
                    hinders = hinders
                }
                    :OnSuccess()
                        :Dialog("DIALOG_THREATEN_PRIEST_SUCCESS")
                        :Fn(function() 
                            
                            for k,v in pairs(rabble) do
                                v:MoveToLimbo()
                            end

                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle{
                                    allies = {cxt.quest:GetCastMember("giver")},
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                }
                                    :OnWin()
                                        :GoTo("STATE_POST_FIGHT")

                        end)
                    :OnFailure()
                        :Dialog("DIALOG_THREATEN_PRIEST_FAILURE")
                        :Fn(function() 
                            local enemies = shallowcopy(rabble)
                            table.insert(enemies, 1, cxt.quest:GetCastMember("target"))
                            cxt.enc.scratch.fought_rabble = true

                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle{
                                    no_oppo_limit = true,
                                    enemies = enemies,
                                    allies = {cxt.quest:GetCastMember("giver")},
                                }
                                    :OnWin()
                                        :GoTo("STATE_POST_FIGHT")
                        end)

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                
                :Battle{ 
                    flags = BATTLE_FLAGS.NO_BYSTANDERS,
                    on_win = function(cxt) 
                        cxt:Dialog("DIALOG_WON_AGAINST_PRIEST")
                        cxt.enc.scratch.fought_rabble = true
                        cxt:Opt("OPT_DEFEND")
                            :Dialog("DIALOG_DEFEND")
                            :Battle{
                                enemies = rabble,
                                allies = {cxt.quest:GetCastMember("giver")},
                                noncombatants = {cxt.quest:GetCastMember("target")},
                            }
                                :OnWin()
                                    :GoTo("STATE_POST_FIGHT")
                    end,
                }

        end)    

    :State("STATE_POST_FIGHT")
        :Loc{
            DIALOG_WON_AGAINST_RABBLE = [[
                player:
                    !left
                target:
                    !exit
                {zealot_dead?
                    * {giver} is dead, and so is your chance of getting paid. 
                player:
                    Hesh can get shucked.
                }

                {not zealot_dead?
                    giver:
                        !right
                        Well. That's one way to enforce the orthodoxy.
                    {not priest_dead?
                        I'll take {target} back to the Cult. {target.HeShe} will have a great deal of recanting to do.
                    }
                    {priest_dead?
                        It's unfortunate that {target} had to die. I'm worried it only made a martyr in the eyes of those gathered here.
                        However... you saved me having to listen to {target}'s heresy any longer. 
                        So I'll take the victory.
                    }

                    !hesh_greeting
                    May you walk in the shallows, grifter.
                    Even if you prefer your flotsam life.
                }
            ]],
            
            DIALOG_WON_AGAINST_PRIEST_SOLO = [[
                player:
                    !left
                target:
                    !exit
                {zealot_dead?
                    * {giver} is dead, and so is your chance of getting paid. 
                player:
                    Hesh can get shucked.
                }

                {not zealot_dead?
                    giver:
                        !right
                        Well done, grifter. I'm glad you were able to discredit {target.himher} before the bloodshed.
                    
                    {not priest_dead?
                        I'll take {target} back to the Cult. {target.HeShe} will have a great deal of recanting to do.
                    }

                    {priest_dead?
                        It's unfortunate that {target} had to die. But at least {target.heshe} didn't play the martyr.
                    }
                    !hesh_greeting
                    May you walk in the shallows, grifter.
                    Even if you prefer your flotsam life.
                }
            ]],
        }
        :Fn(function(cxt)

                cxt.enc.scratch.zealot_dead = cxt.quest:GetCastMember("giver"):IsDead()
                cxt.enc.scratch.priest_dead = cxt.quest:GetCastMember("target"):IsDead()
                cxt:Dialog(cxt.enc.scratch.fought_rabble and "DIALOG_WON_AGAINST_RABBLE" or "DIALOG_WON_AGAINST_PRIEST_SOLO")    

                if not cxt.enc.scratch.zealot_dead then
                    if cxt.quest:GetCastMember("target"):IsAlive() then
                        cxt.quest:GetCastMember("target"):Retire()
                    end
                    ConvoUtil.GiveQuestRewards(cxt)
                    cxt.quest:Complete()
                    StateGraphUtil.AddLeaveLocation(cxt)
                else
                    cxt.quest:Fail()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end

        end)