local QDEF = QuestDef.Define{
        title = "Middle Man",
        qtype = QTYPE.SIDE,
        rank = {1, 3},
        icon = engine.asset.Texture("icons/quests/middle_man.tex"),
        desc = [[Convince {target} to complete {target.hisher} transaction with {giver}.]],
        focus = QUEST_FOCUS.NEGOTIATION,
        avoids = 
        {
            {"giver", "target"},
        },
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = QuestUtil.CanDoJakesGoonQuest,        
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"JAKES_RUNNER", "JAKES_RUNNER", "JAKES_SMUGGLER", "JAKES_SMUGGLER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }

    :AddCast{
        cast_id = "target",
        condition = function(agent, quest)
            return agent:GetFactionID() ~= "ADMIRALTY" and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, false, true) and not AgentUtil.IsCoworker(agent, quest:GetCastMember("giver"))
        end,
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"LABORER", "HEAVY_LABORER", "BANDIT_GOON2", "WEALTHY_MERCHANT"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }

    :AddObjective{
        id = "do_convince",
        title = "Convince {target} to pay up",
        desc = [[Track down {target} and convince them to hold up their end of the bargain with {giver}.]],
        mark = { "target" },
        negotiation_targets = {"target"},
        combat_targets = {"target"},
    }

    :AddObjective{
        id = "convince_giver",
        title = "Convince {giver} to drop the deal",
        desc = [[Convince {giver} to leave {target} alone.]],
        mark = { "giver" },
        negotiation_targets = {"giver"},
        combat_targets = {"giver"},
    }

    :AddObjective{
        id = "report_back_to_giver",
        title = "Tell {giver} the deal is back on",
        desc = [[Tell {giver} that you successfully convinced {target}.]],
        terminal = true,
        ready_for_completion = true,
        mark = { "giver" },
    }
    :AddOpinionEvents{
        stood_up_for = 
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "Stood up for them in a business deal",

        },
    }

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddAttract("do_convince", "target",
    [[
        agent:
            $neutralDubious
            You haven't been talking to {giver}, have you?
    ]])


QDEF:AddConvo("do_convince", "target")
    :Loc{
        OPT_ASK_ABOUT_GOODS = "Ask about {giver}'s deal",
        DIALOG_OFFER = [[
            player:
                $neutralDirect
                So {giver} tells me you two had a deal, and you lost your nerve.
            agent:
                $neutralDirect
                Yeah, like I told {giver.himher} already, I don't want second-rate goods.
            player:
                $neutralTakenAback
                Second rate goods?
            agent:
                $angrySeething
                The last batch I got from {giver.himher} was a total bust. 
                $angryPatienceLost
                Unusable trash!
            player:
                $neutralDirect
                Last batch of <i>what</>, exactly?
            agent:
                $miscEmbarrassed
                !bashful
                I... uh, I'd rather not say.
            *** {agent} is refusing to pay {giver}.
        ]],
        OPT_OFFER_TO_CONVINCE_GIVER_TO_BACK_OFF = "Offer to convince {giver} to back off",
        DIALOG_SWITCH_SIDES = [[
            player:
                $neutralDubious
                So the goods weren't... good?
            agent:
                $neutralDirect
                No. They were not.
            player:
                $neutralThoughtful
                I'll go talk to {giver}. Maybe make them back off.
            agent:
                $happyAmazed
                You will? I really appreciate that. Thanks!
        ]],

        OPT_CONVINCE_THEM_TO_PAY = "Convince {target} to pay up",
        DIALOG_CONVINCE = [[
            player:
                $neutralWhatever
                I don't care about the details. {giver} wants {giver.hisher} money.
            agent:  
                $angryHostile
                I won't pay!
        ]],
        DIALOG_CONVINCED = [[
            agent:
                $scaredPowerless
                Fine! I'll do it!
                Go tell {giver} I'll take the bloody shipment. 
                But this is the last time!
        ]],
        DIALOG_NOT_CONVINCED = [[
            agent:
                $angrySeething
                No, I told you. I don't want {giver}'s trash.
                Now stop bothering me!
        ]],
        OPT_ATTACK = "Rough up {target}",
        DIALOG_ATTACK = [[
            player:
                !fight
                $angryHostile
                This is on you.
            agent:
                !fight
                $angryHostile
                I'm going to ship you back to {giver} in a box!

        ]],
        DIALOG_WON_KILLED = [[
            * {agent} is dead. Better go tell {giver} they aren't going to get their money.
        ]],
        DIALOG_WON_SURVIVED = [[
            agent:
                !injured
                $scaredAnguished
                Stop! Stop!
                I'll do it.
                Just leave me alone.
        ]],

    }
    :RequireFlags(CONVERSATION_FLAGS.BUSINESS)
    :Hub(function(cxt, who)
        cxt:Opt("OPT_ASK_ABOUT_GOODS")
            :Fn(function(cxt) 
                    cxt:Dialog("DIALOG_OFFER")
                    cxt:Opt("OPT_OFFER_TO_CONVINCE_GIVER_TO_BACK_OFF")
                        :PreIcon( global_images.like )
                        :ReceiveOpinion(cxt.quest:GetQuestDef():GetOpinionEvent("stood_up_for"))
                        :Fn(function(cxt) 
                                cxt:Dialog("DIALOG_SWITCH_SIDES")
                                cxt.quest:Complete("do_convince")
                                cxt.quest:Activate("convince_giver")

                            end)

                    cxt:Opt("OPT_CONVINCE_THEM_TO_PAY")
                        :Dialog("DIALOG_CONVINCE")
                        :Negotiation{
                            on_success = function() 
                                cxt:Dialog("DIALOG_CONVINCED")
                                cxt.quest:Complete("do_convince")
                                cxt.quest:Activate("report_back_to_giver")
                            end,
                            on_fail = function() 
                                cxt:Dialog("DIALOG_NOT_CONVINCED")
                            end,
                        }

                    cxt:Opt("OPT_ATTACK")
                        :Dialog("DIALOG_ATTACK")
                        :Battle{
                            on_win = function(cxt) 
                                if cxt.quest:GetCastMember("target"):IsDead() then
                                    cxt:Dialog("DIALOG_WON_KILLED")
                                else
                                    cxt:Dialog("DIALOG_WON_SURVIVED")
                                end
                                cxt.quest:Complete("do_convince")
                                cxt.quest:Activate("report_back_to_giver")
                                StateGraphUtil.AddEndOption(cxt)
                            end,
                        }
                    
                    StateGraphUtil.AddBackButton(cxt)
                end)


        end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddAttract("report_back_to_giver", "giver",
    [[
        agent:
            $neutralGreeting
            So did you knock some sense into {target.himher}?
    ]])


QDEF:AddConvo("report_back_to_giver", "giver")
    :Loc{
        OPT_REPORT = "Tell {giver} about {target}",
        DIALOG_IS_PAYING = [[
            player:
                $neutralDirect
                The deal is back on.
            agent:
                $happyAmazed
                I should use you for all of my negotiations!
                Thanks!
        ]],
        DIALOG_IS_DEAD = [[
            player:
                $miscEmbarrassed
                Things got heated, and {target} got dead.
            agent:
                $angryPatienceLost
                You killed them?!
                Now I'm down one deal and one customer.
                Get out of my sight.
        ]]
    }
    :Hub(function(cxt, who)
            cxt:Opt("OPT_REPORT")
                :Fn(function(cxt)
                    if cxt.quest:GetCastMember("target"):IsDead() then
                        cxt:Dialog("DIALOG_IS_DEAD")
                        cxt.quest:Fail()
                        StateGraphUtil.AddEndOption(cxt)
                    else
                        cxt:Dialog("DIALOG_IS_PAYING")
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end
                end)
    end)


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddAttract("convince_giver", "giver",
    [[
        agent:
            $neutralDubious
            I'm still waiting to move that shipment...
    ]])


QDEF:AddConvo("convince_giver", "giver")
    :Loc{
        OPT_CONVINCE_GIVER = "Convince {giver} to back off",
        DIALOG_CONVINCE = [[
            player:
                $miscPersuasive
                The way I see it, {target} has a legit complaint.
            agent:
                $angryTakenAback
                What did they tell you?! 
        ]],
        DIALOG_CONVINCED = [[
            agent:
                $neutralResigned
                Maybe you're right.
                I'll cut them some slack this time.
        ]],
        DIALOG_NOT_CONVINCED = [[
            agent:
                $angryTakenAback
                Don't tell me how to run my business!
                You're fired, {player}.
        ]],

        SIT_MOD = "Feels betrayed",
    }
    :Hub(function(cxt, who)
            cxt:Opt("OPT_CONVINCE_GIVER")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    situation_modifiers =
                    {
                        { value = 10, text = cxt:GetLocString("SIT_MOD") },
                    },
                    on_success = function() 
                        cxt:Dialog("DIALOG_CONVINCED")
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end,
                    on_fail = function() 
                        cxt:Dialog("DIALOG_NOT_CONVINCED")
                        cxt.quest:Fail()
                        StateGraphUtil.AddEndOption(cxt)
                    end,
                }

    end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            player:
                !interest
                $neutralGreeting
                I hear you're looking for help.
            agent:
                !agree
                $happyGreeting
                You heard right. I run a little import-export business.
                I find things people want, and they pay me accordingly.
            player:
                !dubious
                $neutralDubious
                What kind of things?
            agent:
                !shrug
                $neutralWhatever
                Imports from the mainland, mostly. Unmentionables, occasionally. 
                Does the hunter have scruples, or will you be taking the job?
            player:
                !crossed
                $miscPersuasive
                I assume you'll be paying well for the "ask me no questions" vibe.
            agent:
                !sigh
                $neutralResigned
                A client ordered some very expensive items, then backed out at the last minute, leaving me with the bag.
                Now I'm sitting on a shipment with no buyer. 
                !angry_shrug
                $angryPatienceLost
                I don't want the goods. It takes time and money to find a new buyer. I need you to go <i>remind</i> {target.himher} the meaning of "all sales are final".
        ]],
        DIALOG_GIVE = [[
            player:
                !agree
                $neutralDirect
                Consider it taken care of.
            agent:
                !point
                $miscRelieved
                Don't come back to me without a buyer, ya hear?
        ]],
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "do_convince", "DIALOG_OFFER", function() 
                cxt:Dialog("DIALOG_GIVE")
            end)
    end)


