local function GetGuardAtLocation(location)
    for k,wp in location:WorkPositions() do
        if wp:GetRole() == CHARACTER_ROLES.GUARD then
            return wp:GetAgent()
        end
    end
end

local QDEF = QuestDef.Define{
        title = "Private Security",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = "{giver} has heard that bandits are coming to rob {giver.hisher} store, and is looking for help defending {giver.hisher} property.",
        icon = engine.asset.Texture("icons/quests/private_security.tex"),
        focus = QUEST_FOCUS.COMBAT,

        precondition = function(quest) return TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY end,
        on_destroy = function( quest )
            if quest.param.bandits then
                for k,v in pairs(quest.param.bandits) do
                    v:MoveToLimbo()
                end
            end
        end,
        
    }

    :AddCast{
        cast_id = "giver",
        provider = true,

        condition = function(agent, quest) 
            
            local wp = agent:GetBrain():GetWorkPosition()
            if wp and wp:GetRole() == CHARACTER_ROLES.PROPRIETOR and wp:GetLocation():HasTag("shop") then
                local guard = GetGuardAtLocation(wp:GetLocation())
                if guard and not guard:HasQuestMembership() then
                    return true
                end
            end
        end,
    }

    :AddCast{
        cast_id = "bandit",
        when = QWHEN.MANUAL,
    }
    :AddLocationCast{
        cast_id = "shop",
        cast_fn = function(quest, t)
            table.insert(t, quest:GetCastMember('giver'):GetBrain():GetWorkPosition():GetLocation())
        end,
    }
    
    :AddCast{
        cast_id = "guard",
        cast_fn = function(quest, t) 
            table.insert(t, GetGuardAtLocation( quest:GetCastMember('giver'):GetBrain():GetWorkPosition():GetLocation() ) )
        end,
    }

    :AddObjective{
        id = "go_to_shop",
        title = "Go to {shop}",
        desc = "Get over to {shop} before the Spree does",
        mark = { "shop"},
        
        on_complete = function(quest) 
            quest:Activate("defend_the_store")
        end,
    }
    :AddObjective{
        id = "defend_the_store",
        title = "Wait for the Spree to arrive",
        desc = "{giver} wants to spring an ambush on the bandits, so you're going to have to hide before they arrive.",
        mark = { "giver"},
        on_activate = function(quest)
            quest:Activate("convince_guard_to_help")
            if quest:GetCastMember("giver"):HasAspect("cardshop") and quest:GetCastMember("giver"):GetAspect("cardshop"):HasItemsForSale() then
                quest:Activate("get_supplies_from_giver")
            end
        end,
    }

    :AddObjective{
        id = "convince_guard_to_help",
        title = "(Optional) Convince {guard} to help",
        desc = "{guard} is supposed to keep {giver} safe, but something has them spooked. Go bolster their confidence, or you're going to be facing the bandits by yourself.",
        negotiation_targets = { "guard"},
        mark = { "guard"},
    }

    :AddObjective{
        id = "get_supplies_from_giver",
        title = "(Optional) Get supplies from {giver}",
        desc = "If {giver} wants your help with a battle, they might be willing to part with some of their goods.",
        negotiation_targets = { "giver" },
        mark = { "giver"},
    }

QDEF:AddConvo("go_to_shop", "giver")
    :ConfrontState("STATE_GREETING", function(cxt) return cxt.location == cxt.quest:GetCastMember("shop") end)    
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    $happyGreeting
                    You're here! Great.
                    The Spree will be here soon. I've got an ambush planned, but before we set that up you should talk some sense into {guard}.
                    {guard.HeShe}'s been acting weird since their first visit. Just make sure {guard.heshe}'s not going to bail.
            ]]
        }
        :Fn(function(cxt) 
                cxt.quest:Complete("go_to_shop")
                cxt:Dialog("DIALOG_INTRO")
                StateGraphUtil.AddEndOption(cxt)
        end)    




QDEF:AddConvo("convince_guard_to_help", "guard")
    :Loc{
        OPT_CONVINCE_HELP = "Convince {guard} to help you defend {giver}",
        
        DIALOG_INTRO = [[
            player:
                $miscPersuasive
                So, how about those bandits, eh?
            agent:
                !angry_shrug
                $neutralResigned
                Don't you start on me, too.
                I don't know how to—I only took this job 'cause—
            player:
                !interest
                Because...?
            agent:
                !crossed
                Give me one good reason to stay and risk my life for this job, huh?
            player:
                I'll give you three!
        ]],
        DIALOG_NEG_FAIL = [[
            player:
                !eureka
                And thirdly—
            agent:
                $neutralWhatever
                Cram it, hunter. This is your job, not mine. 
                !exit
            * {agent} leaves, {agent.hisher} shadow barely lingering in the doorway.
            giver:
                !right
                !scared
                $miscHopeless
                Well... that didn't go well.
        ]],
        DIALOG_NEG_SUCCESS = [[
            player:
                !overthere
                And thirdly, look at {giver}. Look at that face.
                $miscPersuasive
                Is that a face that can handle another betrayal?
                Or is that a face you want to hold gently, and promise you'll always be there for.
            agent:
                !scared
                $neutralTakenAback
                Uh, well, I—uh...
                !embarrassed
                $miscEmbarrassed
                O-okay. I'll stay. I don't wanna abandon {giver}—or anyone! I wouldn't wanna abandon anyone!
        ]],
    }
    
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE_HELP")
            :Dialog("DIALOG_INTRO")
            :Negotiation{
                helpers = {"giver"},
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_NEG_SUCCESS")
                    cxt.quest:Complete("convince_guard_to_help")
                    StateGraphUtil.AddEndOption(cxt)
                end,
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_NEG_FAIL")

                    cxt:GetAgent():GetBrain():GetWorkPosition():Fire()
                    cxt:GetAgent():MoveToLimbo()

                    cxt.quest:Fail("convince_guard_to_help")
                    StateGraphUtil.AddEndOption(cxt)
                end,
            }
    end)


QDEF:AddConvo()
    :Loc{
        OPT_TELL_GIVER_READY = "Tell {giver} you're ready",
        OPT_GET_SUPPLIES = "Get some supplies from {giver}",
        DIALOG_INTRO = [[
            player:
                $miscPersuasive
                I'm putting my neck on the line to protect you. Do you have anything that might help with that behind the counter?
            agent:
                !dubious
                $miscMocking
                You want something for free? 
        ]],
        DIALOG_NEG_FAIL = [[
            player:
                Come on. Just a little something—
            agent:
                !hips
                $neutralDirect
                No. Compensation has already been settled. If you didn't come prepared, that's your problem.
        ]],
        DIALOG_NEG_SUCCESS = [[
            player:
                $miscPersuasive
                You want me to succeed, don't you?
            agent:
                !sigh
                $neutralResigned
                Fine. You can pick <i>one thing</i> from my stock.
        ]],
        OPT_PICK_ITEM = "Pick an item",
        DIALOG_PICKED_ITEM = [[
            agent:
                $neutralResigned
                You'd better be worth it.
        ]],

        DIALOG_OFFER = [[
            player:
                $neutralGreeting
                Word is you need some help?
            agent:
                !agree
                Huh. Yeah, and you look like you could cut it.
                $neutralDirect
                One of those Spree types was sniffing around my shop yesterday, like they were casing the joint, you know?
                I got my guard to chase them out, but I know how those scum operate. They'll be back.
        ]],
        DIALOG_PST = [[
            agent:
                !thought
                $neutralThoughtful
                Before the Spree arrive, you probably want to check in with my guard, {guard}.
                I'm not sure what they said to {guard.himher} yesterday, but {guard.heshe}'s been acting strange ever since.
        ]],
        DIALOG_GO_TO_THERE = [[
            agent:
                $neutralThanks
                I'll meet you at the shop. Don't take too long!
        ]]
    }

    :Hub(function(cxt) 
        if cxt:GetAgent() == cxt.quest:GetCastMember("giver") then


            StateGraphUtil.OfferSideJob(cxt, nil, "DIALOG_OFFER", function(cxt) 
                        if cxt.location ~= cxt.quest:GetCastMember("shop") then
                            cxt:Dialog("DIALOG_GO_TO_THERE")
                            cxt.quest:Activate("go_to_shop")
                        else
                            cxt:Dialog("DIALOG_PST")     
                            cxt.quest:Activate("defend_the_store")
                        end
                    end)


            if cxt.quest:IsActive("get_supplies_from_giver") then
                cxt:Opt("OPT_GET_SUPPLIES")
                    :Dialog("DIALOG_INTRO")
                    :Negotiation{
                        on_success = function(cxt) 
                            cxt.quest:Complete("get_supplies_from_giver")
                            cxt:Dialog("DIALOG_NEG_SUCCESS")
                            cxt:Opt("OPT_PICK_ITEM")
                                :PreIcon( global_images.shop, UICOLOURS.MONEY_LIGHT )
                            :Fn(function()
                                    cxt.enc:WaitOnLine()
                                    local screen = Screen.CardShopScreen( cxt:GetAgent(), function() cxt.enc:ResumeEncounter() end, 1 )
                                    TheGame:FE():InsertScreen( screen )
                                    cxt.enc:YieldEncounter()
                                    cxt:Dialog("DIALOG_PICKED_ITEM")
                                end)

                        end,
                        on_fail = function(cxt) 
                            cxt:Dialog("DIALOG_NEG_FAIL")
                            cxt.quest:Fail("get_supplies_from_giver")
                            StateGraphUtil.AddEndOption(cxt)
                        end,

                        }
            end
            if cxt.quest:IsActive("defend_the_store") then
                cxt:Opt("OPT_TELL_GIVER_READY")
                    :GoTo('STATE_DEFEND')
            end
        end
    end)


    :State("STATE_DEFEND")
        :Loc{
            
            DIALOG_INTRO = [[
                player:
                    !salute
                    $miscPersuasive
                    I'm ready!
                agent:
                    $neutralThanks
                    Alright. You hide under the counter here.
                    !eureka
                    When I give the word, come out stabbing!
                player:
                    I won't take that literally, but you got it.
                    !exit
            ]],
            DIALOG_GUARD_LEAVING = [[
                * {guard} approaches {giver}.
                guard:
                    !left
                    !scared_shrug
                    $neutralDubious
                    This is crazy, {giver}. We can't beat the Spree, even with this grifter's help.
                    I'm outta here. And if you had any sense at all... you would be, too.
                    !exit
                * {guard} makes for the exit, leaving nothing but a grim feeling behind.
                giver:
                    !palm
                    $neutralResigned
                    Figures. I knew {guard} wasn't right for the job. 

                *** {guard} has left. You're going to have to face the Spree alone.

            ]],
            OPT_WAIT = "Wait",

            DIALOG_IN_THE_DARK = [[
                * You wedge yourself under the counter, and {agent} pulls a sheet over you.
                * Time passes. You hear customers come and go, the wheels of commerce spinning above your head.
                * Just as your leg is starting to cramp, you hear something loud and angry come through the door.
            ]],

            DIALOG_BANDIT_HERE = [[
                bandit:
                    !right
                    $happyGreeting
                    Did you miss me?
                agent:
                    !left
                    $angryPatienceLost
                    
                bandit:
                    $happyCocky
                    Aw, see what I mean, guys? This shopkeeper is <b>incorrigible</>.
                    Looks like we are going to have to teach {giver.himher} a lesson.
                agent:
                    $angryPatienceLost
                    Yeah, well <b>I've</> got a lesson for <b>you!</>
                * {giver} kicks you under the counter. This would be your cue.

                *** You are hiding under the counter and can't see anything. The Spree are here, and spoiling for a fight. Are you going to fulfill your obligation to {giver}?
            ]],

            OPT_JUMP_OUT = "Jump out and confront the bandits",
            OPT_STAY_HIDDEN = "Stay quiet and wait for the danger to pass",
            DIALOG_STAY_HIDDEN = [[
                * You ignore {agent}, and stay hidden.
                * The silence sits heavily upon the shop. {agent} kicks you a couple more times, with increasing urgency.
                * The bandit clears {bandit.hisher} throat.
                bandit:
                    $miscMischievous
                    Ooookay. Got a problem with your leg there?
                agent:
                    $scaredFearful
                    No, I just- uh...
                bandit:
                    $angrySeething
                    How about I give you a problem?
                    !exit

                * You stay quiet as the bandits trash the place. {agent} catches the worst of the violence. 
                * When they finally leave, you emerge.
            ]],
            
            DIALOG_STAYED_HIDDEN = [[
                player:
                    !right
                agent:
                    !left
                    !injured
                    $scaredAnguished
                    Get the hell out of here, grifter.
                player:
                    !exit
                * You take your leave.
            ]],
            DIALOG_MEET_BANDITS_WORKING_WITH = [[
                * You come out from behind the counter.
                bandit:
                    !right
                player:
                    !left
                bandit:
                    $angryTakenAback
                    Whoa, what are <b>you</> doing back there?
                    You want a piece of this action, friend? 
                    Spree's gotta stick together, right?

                *** {bandit} is offering to cut you in on the robbery.

            ]],
            OPT_ASK_BANDITS_TO_LEAVE = "Ask {bandit} to leave",
            DIALOG_ASK_LEAVE = [[
                player:
                    $angryHostile
                    This is my turf, {bandit}.
                    You wouldn't want to encroach on my turf, would you?
                bandit:
                    $angryTakenAback
                    Since when do <i>you</> have turf?
            ]],
            DIALOG_ASK_SUCCESS = [[
                bandit:
                    $neutralResigned
                    This is your turf, and I respect that.
                    Didn't mean no disrespect.
                    !exit
                * The Spree leaves, looking for an unclaimed prize.
            ]],
            DIALOG_QUEST_WIN_WITH_SPREE = [[
                giver:
                    !right
                    $miscMocking
                    You <i>work</> with these people?
                    I appreciate the help, but...
                * {giver} thinks twice about questioning your life decisions.
                giver:
                    $neutralTakenAback
                    Here's your pay, grifter.
            ]],
            
            DIALOG_ASK_FAILURE = [[
                bandit:
                    $angryHostile
                    You should only take turf you can hold, grifter!
                    Didn't they tell you that when you signed up?
                * The Spree attacks!
            ]],
            DIALOG_DECLINE_OFFER = [[
                player:
                    $angrySeething
                    This place is off limits. It's time you leave {giver} alone.
                bandit:
                    $angryHostile
                    Ha! You don't get to tell me what to do!
            ]],
            DIALOG_QUEST_LOSE_GIVER_DEAD = [[
                * {giver} is dead. You have failed in your duty.
                * You leave before the Admiralty arrives with questions.
            ]],

            DIALOG_MEET_BANDITS = [[
                * You come out from behind the counter.
                bandit:
                    !right
                player:
                    !left
                bandit:
                    $miscMocking
                    Aw, look. {giver.HeShe} bought {giver.himher}self a bodyguard.
                    Cute.
                    Scram, grifter, or we'll gut you where you stand.
            ]],
            OPT_ATTACK_THE_BANDITS = "Defend the shop",
            DIALOG_ATTACK_THE_BANDITS = [[
                player:
                    $neutralDirect
                    I'm not going anywhere.
                bandit:
                    $angryHostile
                    Well, then.
                    It's your funeral.
                    !fight
            ]],
            DIALOG_QUEST_WIN = [[
                giver:
                    !right
                    $happyThanks
                    Good job, grifter.
                    Take this. You earned every coin.
            ]],
            OPT_RUN_AWAY = "Leave the shop",
            DIALOG_RUN_AWAY = [[
                * You size up the opposition and decide you'd rather not fight.
                player:
                    $scaredTakenAback
                    Sorry, {giver}. You're on your own.
                giver:
                    !right
                    $angrySeething
                    You rat fink!
                bandit:
                    !left
                    $happyCocky
                    Now where were we?
                * You high-tail it, not waiting to see what happens next.
            ]],
            OPT_HELP_THE_BANDITS = "Join the Spree in robbing {giver}",
            

            --make this branch an actual fight
            DIALOG_HELP_BANDITS = [[
                player:
                    $miscPersuasive
                    Sure! I softened this one up for us already.
                bandit:
                    $happyCocky
                    That's what I like to hear!
                giver:
                    !right
                    $angryHostile
                    You! You bastard!
                    !fight

            ]],
            DIALOG_WON_BETRAYAL_BANDIT_ALIVE = [[
                bandit:
                    !right
                    $happyThanks
                    Here's your share, friend. Be seeing you around!
                player:
                    !exit
                * You take the money and leave.
            ]],
            
            DIALOG_WON_BETRAYAL_BANDIT_DEAD = [[
                giver:
                    !exit
                * You gather up a bunch of loose shills and sneak away.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
            OPT_ATTACK = "Attack",
            SIT_MOD = "You are working with the Spree",
        }
         
    :Fn(function(cxt)
        cxt:Dialog("DIALOG_INTRO")

        if cxt.quest:IsActive("convince_guard_to_help") then
            cxt:Dialog("DIALOG_GUARD_LEAVING")
            cxt.quest:GetCastMember("guard"):GetBrain():GetWorkPosition():Fire()
            cxt.quest:GetCastMember("guard"):MoveToLimbo()
            cxt.quest:Fail("convince_guard_to_help")
        end

        if cxt.quest:IsActive("get_supplies_from_giver") then
            cxt.quest:Fail("get_supplies_from_giver")
        end


        cxt.enc:GetScreen():ClearHistory()
        cxt.enc:GetScreen():Fade( 1, false )
        cxt.enc:Wait(.25)
        cxt:Dialog("DIALOG_IN_THE_DARK")

        --this is supposed to be a hard fight, so boost the difficulty. 
        cxt.quest.param.bandits = CreateCombatParty("BANDIT_PATROL", cxt.quest:GetRank() + 1, cxt.location, false)
        cxt.quest:AssignCastMember("bandit", cxt.quest.param.bandits[1])
        LocationUtil.SendPatronsAway( cxt.location, function( agent ) return not agent:HasAspect( "bribed" ) end )

        local allies = {"giver"}
        if cxt.quest:IsComplete("convince_guard_to_help") then
            table.insert( allies, "guard") 
        end

        cxt:Dialog("DIALOG_BANDIT_HERE")

        cxt:Opt("OPT_JUMP_OUT")
            :Fn(function(cxt) 
                cxt.enc:GetScreen():ClearHistory()
                cxt.enc:GetScreen():Fade( 1, true )
                cxt.enc:Wait(.25)
                cxt.enc:SetPrimaryCast( cxt.quest:GetCastMember( "bandit") )

                if Util.IsPlayerWorkingWithFaction("BANDITS") then
                    cxt:Dialog("DIALOG_MEET_BANDITS_WORKING_WITH")
                    cxt:Opt("OPT_ASK_BANDITS_TO_LEAVE")
                        :Dialog("DIALOG_ASK_LEAVE")
                        :Negotiation{
                            situation_modifiers =
                            {
                                { value = -10, text = cxt:GetLocString("SIT_MOD") }
                            },

                            on_success = function(cxt) 
                                cxt:Dialog("DIALOG_ASK_SUCCESS")
                                cxt:Dialog("DIALOG_QUEST_WIN_WITH_SPREE")
                                cxt.quest:Complete()
                                ConvoUtil.GiveQuestRewards(cxt)
                                StateGraphUtil.AddEndOption(cxt)
                            end,

                            on_fail = function(cxt) 
                                cxt:Dialog("DIALOG_ASK_FAILURE")
                                cxt:Opt("OPT_DEFEND_SELF")
                                    :Battle{
                                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                                        allies = allies,
                                        enemies = cxt.quest.param.bandits,
                                        on_win = function(cxt) 
                                            if cxt:GetCastMember("giver"):IsDead() then
                                                cxt.quest:Fail()
                                                cxt:Dialog("DIALOG_QUEST_LOSE_GIVER_DEAD")
                                                StateGraphUtil.AddLeaveLocation(cxt)
                                            else
                                                cxt:Dialog("DIALOG_QUEST_WIN_WITH_SPREE")
                                                cxt.quest:Complete()
                                                ConvoUtil.GiveQuestRewards(cxt)
                                                StateGraphUtil.AddEndOption(cxt)
                                            end
                                        end
                                    }
                            end,
                        }

                    cxt:Opt("OPT_ATTACK_THE_BANDITS")
                        :Dialog("DIALOG_DECLINE_OFFER")
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            allies = allies,
                            enemies = cxt.quest.param.bandits,
                            on_win = function(cxt) 
                                if cxt:GetCastMember("giver"):IsDead() then
                                    cxt.quest:Fail()
                                    cxt:Dialog("DIALOG_QUEST_LOSE_GIVER_DEAD")
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                else
                                    cxt:Dialog("DIALOG_QUEST_WIN")
                                    cxt.quest:Complete()
                                    ConvoUtil.GiveQuestRewards(cxt)
                                    StateGraphUtil.AddEndOption(cxt)
                                end
                            end,
                        }

                    cxt:Opt("OPT_HELP_THE_BANDITS")
                        :Fn(function(cxt) 
                            
                            cxt.quest:GetCastMember("giver"):OpinionEvent(OPINION.BETRAYED)
                            cxt:Dialog("DIALOG_HELP_BANDITS")
                            cxt:Opt("OPT_ATTACK")
                                :Battle{
                                    allies = cxt.quest.param.bandits,
                                    enemies = allies,
                                    
                                    on_win = function(cxt) 
                                        cxt.quest:Fail()
                                        if cxt:GetCastMember("bandit"):IsDead() then
                                            cxt.enc:GainMoney(2*cxt.quest.param.rewards or 100)
                                            cxt:Dialog("DIALOG_WON_BETRAYAL_BANDIT_DEAD")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        else
                                            cxt:Dialog("DIALOG_WON_BETRAYAL_BANDIT_ALIVE")
                                            cxt.enc:GainMoney(cxt.quest.param.rewards or 100)
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end
                                    end
                                }
                        end)
                else
                    cxt:Dialog("DIALOG_MEET_BANDITS")
                    cxt:Opt("OPT_ATTACK_THE_BANDITS")
                        :Dialog("DIALOG_ATTACK_THE_BANDITS")
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            allies = allies,
                            enemies = cxt.quest.param.bandits,
                            on_win = function(cxt) 
                                if cxt:GetCastMember("giver"):IsDead() then
                                    cxt.quest:Fail()
                                    cxt:Dialog("DIALOG_QUEST_LOSE_GIVER_DEAD")
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                else
                                    cxt:Dialog("DIALOG_QUEST_WIN")
                                    cxt.quest:Complete()
                                    ConvoUtil.GiveQuestRewards(cxt)
                                    StateGraphUtil.AddEndOption(cxt)
                                end
                            end,
                        }
                    cxt:Opt("OPT_RUN_AWAY")
                        :Fn(function(cxt) 
                            cxt:Dialog("DIALOG_RUN_AWAY")
                            cxt.quest:GetCastMember("giver"):OpinionEvent(OPINION.BETRAYED)
                            cxt.quest:Fail()
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)

                end
                

            end)
        cxt:Opt("OPT_STAY_HIDDEN")
            :Fn(function(cxt) 
                cxt:Dialog("DIALOG_STAY_HIDDEN")
                cxt.enc:GetScreen():ClearHistory()
                cxt.enc:GetScreen():Fade( 1, true )
                cxt.enc:Wait(.25)
                cxt.quest:GetCastMember("giver"):OpinionEvent(OPINION.BETRAYED)
                cxt:Dialog("DIALOG_STAYED_HIDDEN")

                cxt.quest:Fail()
                StateGraphUtil.AddLeaveLocation(cxt)
            end)
    end)







