AddOpinionEvent("KILLED_BETRAYAL", 
{
    delta = OPINION_DELTAS.LIKE,
    txt = "Killed someone that betrayed them",
})

local function CleanUpFn(quest)
    if quest:GetCastMember("target") and quest:GetCastMember("target"):IsInPlayerParty() then
        quest:GetCastMember("target"):Dismiss()
    end
end

local QDEF = QuestDef.Define{
        title = "Repo Person",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        
        focus = QUEST_FOCUS.NEGOTIATION,

        desc = [[{giver}'s business partner, {target}, has gone missing with a buyer's money. Find {target} and complete the sale before the buyer gets angry.]],
        icon = engine.asset.Texture("icons/quests/repo_person.tex"),
        avoids = {
            {"giver", "target"},
            {"giver", "client"},
            {"target", "client"},
        },
        on_fail = CleanUpFn,
        on_complete = CleanUpFn,
        on_cancel = CleanUpFn,
        on_init = function(quest)
            quest.param.has_money = math.random() > .5
        end,
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = QuestUtil.CanDoJakesGoonQuest,
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"JAKES_RUNNER","JAKES_RUNNER", "JAKES_SMUGGLER", "JAKES_SMUGGLER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }

    :AddCast{
        cast_id = "target",
        condition = function( agent, quest )
            if QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) then
                return agent:GetFaction() ~= quest:GetCastMember("giver"):GetFaction() and not AgentUtil.IsCoworker(agent, quest:GetCastMember("giver")) and agent:GetFactionID() ~= "BANDITS"
            end
        end,
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"LABORER", "BANDIT_GOON", "HEAVY_LABORER", "ADMIRALTY_GUARD", "ADMIRALTY_PATROL_LEADER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }

    :AddCast{
        cast_id = "client",
        condition = function(agent, quest)
            return QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) and not AgentUtil.IsCoworker(agent, quest:GetCastMember("target"))
        end
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"HEAVY_LABORER", "BANDIT_GOON2", "FOREMAN", "WEALTHY_MERCHANT", "ADMIRALTY_PATROL_LEADER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }


    :AddLocationCast{
        cast_id = "bandit_camp",
        when = QWHEN.MANUAL,
        no_validation = true,
    }

    :AddLocationDefs{
        BANDIT_CAMP = {
            name = "Bandit Camp",
            plax = "EXT_SpreeCampsite_1",
            desc = "The bandits that stole the goods should be here.",
            indoors = false,
            map_tags = {"tents"},
        }
    }

    :AddObjective{
        id = "find_target",
        title = "Find {target}",
        desc = "Confront {target} about the missing money.",
        negotiation_targets = {"target"},
        combat_targets = {"target"},
        mark = { "target" },
    }

    :AddObjective{
        id = "convince_client",
        title = "Meet with {client}",
        desc = "Meet with {client} to broker a deal.",
        negotiation_targets = {"client"},
        combat_targets = {"client"},
        mark = { "client" },
    }

    :AddObjective{
        id = "meet_with_client_target_attacked",
        title = "Meet with {client}",
        desc = "Meet with {client} and tell them about {target}.",
        negotiation_targets = {"client"},
        combat_targets = {"client"},
        mark = { "client" },
    }


    :AddObjective{
        id = "recover_package",
        title= "Recover the package",
        desc = "{target} lost the package, so you'll have to retrieve it",
        mark = {"bandit_camp"},
        on_activate = function( quest)
            local location = quest:SpawnTempLocation("BANDIT_CAMP")
            quest:AssignCastMember("bandit_camp", location )
        end,
    }

    :AddObjective{
        id = "deliver_package",
        title = "Deliver the package to {client}",
        desc = "With the package recovered, you can finally deliver it to {client}",
        mark = {"client"},
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Return to {giver}",
        desc = "You've managed to salvage the deal, return to {giver} for your reward..",
        ready_for_completion = true,
        terminal = true,
        mark = { "giver" },
    }
    
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


QDEF:AddConvo("find_target", "target")
    :Loc{
        OPT_ASK_ABOUT_MONEY = "Ask {target} about the money", 
        OPT_COME_BACK_LATER = "Come back later",
        DIALOG_COME_BACK_LATER = [[
            player:
                !point
                Don't go any where. I'll be back.
        ]]
    }
    :Hub(function ( cxt )
        cxt:Opt("OPT_ASK_ABOUT_MONEY")
            :GoTo("STATE_TARGET_INTRO")
    end)

    :State("STATE_TARGET_INTRO")
        :Loc{            
            DIALOG_INTRO = [[
                agent:
                    !right
                player:
                    !left
                    $neutralGreeting
                    You {agent}? {giver} sent me. Wants to know where {giver.hisher} money got to. 
            ]],
            DIALOG_LOST_PACKAGE = [[
                agent:
                    !scared
                    It's come to that, has it?
                    !placate
                    $scaredDefault
                    I got the goods just like {giver} asked me to, but I got mugged. Now the bandits are sitting pretty on <i>our</i> stock.
            ]],
            DIALOG_SPENT_MONEY = [[
                agent:
                    !palm
                    $scaredDefault
                    Aw, Hesh. Look, the fact is... I mighta dipped into the money to pay off some debts. 
                    !scared_shrug
                    $scaredStammering
                    I had no choice! My debt collector was gonna kill me. But now I guess the client's gonna kill me and {giver} both if we don't deliver, huh?
            ]],
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("target"))
                cxt:Dialog("DIALOG_INTRO")
                if cxt.quest.param.has_money == nil then
                    cxt.quest.param.has_money = math.random() > .5
                end

                if cxt.quest.param.has_money then
                    cxt:Dialog("DIALOG_LOST_PACKAGE")
                    cxt:GoTo("STATE_BANDITS") 
                else
                    cxt:Dialog("DIALOG_SPENT_MONEY")
                    cxt:GoTo("STATE_NO_MONEY") 
                end
                
            end
        end)

    :State("STATE_BANDITS")
        :Loc{
            OPT_OFFER_HELP = "Offer to help retrieve the goods",
            DIALOG_OFFER_HELP = [[
                player:
                    !thumb
                    $neutralDubious
                    That's where I come in. Know where to find these bandits?
                agent:
                    !agree
                    $happyThanks
                    I do. And with a mug as funny-lookin' as yours, we might just be able to get the stuff back.
                player:
                    !dubious
                    $neutralDubious
                    Uh... maybe let <i>me</i> do the talking, though.
            ]],
            OPT_OFFER_TALK = "Offer to talk to {client} for {target.himher}",
            DIALOG_OFFER_TALK = [[
                player:
                    !crossed
                    $neutralThoughtful
                    I wasn't paid to risk my neck, so those bandits can keep their prize.
                    !interest
                    Your client is {client}, yeah? Let's go see if we can get you and {giver} out of this mess.
                agent: 
                    !agree
                    $miscRelieved
                    Guess I had to face the music sooner or later. Let's go. 
            ]],
            OPT_ATTACK = "Attack {target} to set an example",
            DIALOG_ATTACK = [[
                 player:
                    !fight
                    $angryHostile
                    Sorry, {target}, but someone's gonna have to explain your mistake to the client. 
                    This'll sting, but it might keep you and {giver} alive. At least until the next screw up. 
                agent:
                    !flinch
                    $scaredPowerless
                    W-what?! But—
                * You pull out your weapons, and {target} falls silent. 
            ]],
            DIALOG_ATTACKED_DEAD = [[
                * {target} is dead. Hopefully that'll at least keep {giver} safe from the client.
            ]],
            DIALOG_ATTACKED_ALIVE = [[
                agent:
                    !injured
                    $scaredFearful
                    Please! Don't kill me! I-I just made a mistake, that's all.
                player:
                    !disappoint
                    I'm not gonna kill you. But the client might've. Now, at least, I can say you've learned your lesson.
                    Don't screw up again. You can't afford the consequences.
                * You leave {target} to contemplate {target.hisher} fate. Now you have to explain the situation to {client}.
            ]],
        }
        :Fn(function(cxt)
            cxt:Opt("OPT_OFFER_HELP")
                :Dialog("DIALOG_OFFER_HELP")
                :RecruitMember( PARTY_MEMBER_TYPE.ESCORT )
                :Fn(function(cxt)
                    cxt.quest:Complete("find_target")
                    cxt.quest:Activate("recover_package")
                end)

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    on_win = function()
                        cxt.quest:Complete("find_target")
                        if cxt.quest:GetCastAgent("target"):IsDead() then
                            cxt:Dialog("DIALOG_ATTACKED_DEAD")
                        else
                            cxt:Dialog("DIALOG_ATTACKED_ALIVE")
                        end
                        cxt.quest:Activate("meet_with_client_target_attacked")
                        StateGraphUtil.AddEndOption(cxt)
                    end
                }
            
            cxt:Opt("OPT_COME_BACK_LATER")
                :Dialog("DIALOG_COME_BACK_LATER")
                :Fn(function() StateGraphUtil.AddEndOption(cxt) end)
        end)

    :State("STATE_NO_MONEY")
        :Loc{
            OPT_PAY_BACK_DEBT = "Offer to settle their debt ({1#money})",
            DIALOG_SETTLE_DEBT = [[
                player:
                    !palm
                    $neutralResigned
                    You're dead if you can't pay those debts back. So I'll reimburse {client} for you.
                agent:
                    !surprised
                    $happyAmazed
                    Really? You'd do that for {giver} and me? That's—thank you!
                player:
                    !handwave
                    $neutralWhatever
                    Yeah, yeah. Your sob story worked. But don't screw up again, all right? 
                    $neutralAside
                    You clearly can't afford the consequences.
            ]],
            OPT_BETRAY_GIVER = "Propose to betray {giver} for a share of the money",
            DIALOG_BETRAY_GIVER = [[
                player:
                    $miscPersuasive
                    !thought
                    Under those terms you're dead. But maybe we tell the client it's not your fault. Maybe we pin it on {giver} instead.
                    $miscAlluring
                    !handwring
                    Pay me the money you have left, and we'll call it a deal.
                agent:
                    !dubious
                    $miscRelieved
                    You'd sell {giver} up the river so easily? Well... I won't complain. It's my neck you're saving.
                    It's a deal.
            ]],
            OPT_ATTACK = "Attack {target} to set an example",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    $angryHostile
                    Sorry, {target}, but someone's gonna have to explain your mistake to the client. 
                    This'll sting, but it might keep you and {giver} alive. At least until the next screw up. 
                agent:
                    !flinch
                    $scaredPowerless
                    W-what?! But—
                * You pull out your weapons, and {target} falls silent. 
            ]],
            DIALOG_ATTACKED_DEAD = [[
                * {target} is dead. Hopefully that'll at least keep {giver} safe from the client.
            ]],
            DIALOG_ATTACKED_ALIVE = [[
                agent:
                    !injured
                    $scaredFearful
                    Please! Don't kill me! I-I just made a mistake, that's all.
                player:
                    !disappoint
                    $neutralResigned
                    I'm not gonna kill you. But the client might've. Now, at least, I can say you've learned your lesson.
                    $angryDefault
                    Don't screw up again. You can't afford the consequences.
                * You leave {target} to contemplate {target.hisher} fate. Now you have to explain the situation to {client}.
            ]],
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest.param.debt = 50 + (cxt.quest:GetRank() * 75)
            end
            cxt:Opt("OPT_PAY_BACK_DEBT", cxt.quest.param.debt)
                :PreIcon(global_images.hire)
                :Dialog("DIALOG_SETTLE_DEBT")
                :Fn(function(cxt)
                    cxt.quest:Complete("find_target")
                    cxt.quest:Activate("convince_client")
                    StateGraphUtil.AddEndOption(cxt)
                end)

            cxt:Opt("OPT_BETRAY_GIVER")
                :Dialog("DIALOG_BETRAY_GIVER")
                :Fn(function(cxt)
                    cxt.quest:Complete("find_target")
                    cxt.quest:Activate("convince_client")
                    StateGraphUtil.AddEndOption(cxt)
                end)

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    on_win = function()
                        cxt.quest:Complete("find_target")
                        if cxt.quest:GetCastAgent("target"):IsDead() then
                            cxt:Dialog("DIALOG_ATTACKED_DEAD")
                        else
                            cxt:Dialog("DIALOG_ATTACKED_ALIVE")
                        end
                        cxt.quest:Activate("meet_with_client_target_attacked")
                        StateGraphUtil.AddEndOption(cxt)
                    end
                }
            cxt:Opt("OPT_COME_BACK_LATER")
                :PreIcon(global_images.close)
                :Dialog("DIALOG_COME_BACK_LATER")
                :Fn(function() StateGraphUtil.AddEndOption(cxt) end)

        end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


QDEF:AddConvo("recover_package")
    :Confront(function ( cxt )
        if cxt.location == cxt.quest:GetCastMember("bandit_camp") then
            return "STATE_BANDIT_CAMP"
        end
    end)
    :State("STATE_BANDIT_CAMP")
        :Loc{            
            DIALOG_INTRO = [[
                * Finding the bandits is the easy part; getting the goods back will likely be harder.
                agent:
                    !right
                    !crossed
                player:
                    !left
                    !greeting
                    $neutralGreeting
                    Hate to interrupt you in your place of business, but it seems you stole something from a friend of mine.
                agent:
                    !laugh
                    $happyCocky
                    Could be. I have a habit of collecting other people's stuff. Whatcha propose to do about it, grifter?
            ]],
            OPT_CONVINCE = "Convince {agent} to hand over the package",
            DIALOG_CONVINCE = [[
                player:
                    !permit
                    $miscPersuasive
                    I don't think you know exactly who you've stolen from.
                agent:
                    !sigh
                    $neutralWhatever
                    And I guess you mean to enlighten me. 
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !overthere
                    $miscPersuasive
                    This package is meant for an extremely important debt-broker. You heard of Kashio?
                agent:
                    $neutralResigned
                    Oh. <i>Oh</i>.
                    Yeah, there's no world where you could pay me to stand in the way of that. 
                    !bashful
                    $miscEmbarrassed
                    Hesh. No hard feelings, all right? Just take the stuff and go. 
                player:
                    !happy
                    $miscRelieved
                    I'm just <i>glad</i> this was all a misunderstanding.
                    !exit
                * You take the package and hurry away before {agent} can see through your bluff. 
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    !overthere
                    $miscPersuasive
                    These goods are meant for {target}'s sick ma, you see, and she doesn't have much longer to—
                agent:
                    !cruel
                    $angryCruel
                    Oh. You didn't mention it was <i>urgent</i>. In that case, these must be very valuable. 
                    So how about you pay up and we can call this a happy little transaction?
            ]],
            OPT_PAY = "Pay {agent} for the package",
            DIALOG_PAY = [[
                player:
                    !permit
                    $miscPersuasive
                    This is all just business to you. So why don't we pay you for the goods, and we keep this hassle-free.
                agent: 
                    !chuckle
                    $happyThanks
                    Well, sure. Like you said, this is just business. And I'm happy to help.
            ]],
            OPT_ATTACK = "Attack {agent} to get the package",
            DIALOG_ATTACK = [[
                player:
                    !angry
                    $angryPatienceLost
                    Fine. There are are other ways to settle this.
                    !fight
                agent:
                    !fight
            ]],
            DIALOG_WON_BATTLE = [[
                * You find the goods in the bandits' things. 
            ]],

        }
        :SetLooping(true)
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                local bandits = CreateCombatParty("BANDIT_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.enc:SetPrimaryCast(bandits[1])
                cxt.quest.param.package_price = 50 + (cxt.quest:GetRank() * 75)
                cxt:Dialog("DIALOG_INTRO")                
            end

            cxt:Opt("OPT_CONVINCE")
                :ReqCondition(cxt.quest.param.tried_convince ~= true)
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    on_success = function()
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest:Complete("recover_package")
                        cxt.quest:Activate("deliver_package")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function()
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                        cxt.quest.param.package_price = math.floor(cxt.quest.param.package_price * 1.2)
                        cxt.quest.param.tried_convince = true
                    end,
                    }

            cxt:Opt("OPT_PAY")
                :DeliverMoney(cxt.quest.param.package_price)
                :Dialog("DIALOG_PAY")
                :Fn(function(cxt)
                    cxt.quest:Complete("recover_package")
                    cxt.quest:Activate("deliver_package")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    allies = {cxt.quest:GetCastMember("target")},
                    on_win = function()
                        cxt:Dialog("DIALOG_WON_BATTLE")
                        cxt.quest:Complete("recover_package")
                        cxt.quest:Activate("deliver_package")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                }
        end)

QDEF:AddConvo("deliver_package", "client")
    :Loc{            
        OPT_DELIVER_PACKAGE = "Give {client} the goods",
        DIALOG_DELIVER_PACKAGE = [[
            agent:
                !right
            player:
                !left
                !permit
                $neutralGreeting
                You {client}? {giver} sent me to deliver the goods, and to apologize for the delay. 
                It won't happen again.  
            agent:
                !dubious
                $neutralDubious
                Huh. I was about to send someone to collect—but I can see {giver} hired a professional to get it done.
                I'll assume this was just a hiccup. But you tell {giver} I won't be so charitable next time. 
        ]],
    }
    :Hub(function(cxt, who)
        cxt:Opt("OPT_DELIVER_PACKAGE")
            :Dialog("DIALOG_DELIVER_PACKAGE")
            :CompleteQuest()
    end)

QDEF:AddConvo("meet_with_client_target_attacked", "client")
    :Loc{
        OPT_TELL_CLIENT = "Tell {client} about {target}",
        DIALOG_TELL_DEAD = [[
            player:
                !interest
                You {client}? {giver} hired me to look into your missing goods. 
                $neutralDirect
                Turns out {target} cheated you and {giver} both, so I dealt with it.
            agent:
                !dubious
                $neutralDubious
                Dealt with how? {target.HeShe}'s dead?
            player:
                !agree
                $happyCocky
                As the dickens. {giver} won't be working with {target.himher} again. 
        ]],
        DIALOG_TELL_ALIVE = [[
            player:
                !interest
                You {client}? {giver} hired me to look into your missing goods. 
                $neutralDirect
                Turns out {target} cheated you and {giver} both, so I dealt with it.
            agent:
                !dubious
                $neutralDubious
                Dealt with how? {target.HeShe}'s dead?
            player:
                !crossed
                $neutralWhatever
                {target.HeShe}'s alive. Barely. But nobody will try to cheat you or {giver} again, believe me.
        ]],
        OPT_CONVINCE_TO_FORGIVE = "Convince {client.himher} to let {giver} off the hook",
        DIALOG_CONVINCE_TO_FORGIVE = [[
            player:
                !point
                $miscPersuasive
                I wanna stress that this really isn't {giver}'s fault. You're not planning any retribution, are you?
            agent:
                !angry_shrug
                $angryDefault
                Hah! Wouldn't you, in my position? How do I know {giver} wasn't trying to cheat me, too?
        ]],
        DIALOG_CONVINCE_FAIL = [[
            player:
                !scared_shrug
                $miscDepressed
                {giver}'s just honest folk trying to make a living in this hard, cold world.
            agent:
                !spit
                $angryDefault
                Yeah, by taking me for a ride. Forget it—you tell {giver} that {giver.heshe} better watch {giver.hisher} back. 
                If {giver.heshe} wanted to avoid nasty consequences, {giver.heshe} should've been more careful about {giver.hisher} business partners.
            player:
                !disappoint
                $neutralResigned
                Nice of you to at least give warning.
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                !shrug
                $miscPersuasive
                {giver} is trying to set up an honest—er, a reputable... uh, a long and profitable venture here. {giver.HeShe} got just as cheated as you did by {target}.
            agent:
                $neutralThoughtful
                I suppose that's true. And {giver.heshe} hired you to make it right. All right, I won't bring the hammer down. 
                You served up enough retribution for my tastes. 
            player:
                !happy
                $happyThanks
                Exactly! Glad we could come to an agreement.
        ]],
        SIT_MOD = "Killed {target}",
    }

        :Hub(function(cxt, who)
            cxt:Opt("OPT_TELL_CLIENT")
                :Fn(function()
                    local client = cxt.quest:GetCastAgent("client")
                    if cxt.quest:GetCastAgent("target"):IsDead() then
                        cxt:Dialog("DIALOG_TELL_DEAD")
                        client:OpinionEvent( OPINION.KILLED_BETRAYAL )
                        cxt.enc.scratch.mod =
                        {
                            { value = -10, text = cxt:GetLocString("SIT_MOD") }
                        }
                    else
                        cxt:Dialog("DIALOG_TELL_ALIVE")
                    end

                    cxt:Opt("OPT_CONVINCE_TO_FORGIVE")
                        :Dialog("DIALOG_CONVINCE_TO_FORGIVE")
                        :Negotiation{
                            situation_modifiers = cxt.enc.scratch.mod,
                            on_success = function()
                                cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                                cxt.quest:Complete()
                                ConvoUtil.GiveQuestRewards(cxt)
                            end,
                            on_fail = function()
                                cxt:Dialog("DIALOG_CONVINCE_FAIL")
                                cxt.quest:Fail()
                            end
                        }
                end )
        end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("convince_client", "client")
    :Confront(function(cxt)
        return "STATE_CONVINCE_CLIENT"
    end)
    :State("STATE_CONVINCE_CLIENT")
        :Loc{
            DIALOG_INTRO = [[
                * As you arrive, you see {client} in a heated debate with {giver} and {target}.
                player:
                    !left
                client:
                    !right
                    $angrySeething
                    And who are you? Did you <i>also</i> get cheated by these two idiots?
                player:
                    !crossed
                    $neutralDirect
                    I'm a hunter. And like it or not, I have answers for you.
            ]],
            OPT_BLAME_GIVER = "Blame everything on {giver}",
            DIALOG_BLAME_GIVER = [[
                player:
                    $miscPersuasive
                    Look, this is just a simple misunderstanding. {target} is as blameless as you are.
                    You wanna know who's responsible? It was {giver}—{giver.heshe} intended to cheat you from the very start. 
                giver:
                    !right
                    !angry
                    $angryTakenAback
                    What?! Y-you're lying! I hired you to <i>fix</i> this!
                client:
                    !right
                    !palm
                    $angryReluctant
                    Look, all I know, {giver}, is that you're the one I made the deal with.
                    !angry
                    Who you hire, and how it all shakes out—that's <i>your</i> responsibility. 
                * Dismissed, you and {target} head for the door, leaving {giver} behind to deal with the consequences. 
            ]],
            OPT_BLAME_TARGET = "Blame everything on {target}",
            DIALOG_BLAME_TARGET = [[
                player:
                    $neutralDirect
                    Things were going according to plan until {target} dropped the ball. Seems only fair {target.heshe} should be the one to pay the price.
                target:
                    !right
                    What?! N-no! {giver}'s the one in charge! I'm just a nobody!
                client:
                    !right
                    $angryThanks
                    Oh, really? Good. So killing you won't burn any important bridges.
                    !angry_shrug
                    $angryDefault
                    I'll take this blood price, {giver}. You and your grifter friend are free to go. But don't screw me again, you understand?
                * {giver} doesn't look happy with this turn of events, but {giver.heshe}'s smart enough to keep {giver.hisher} mouth shut about it as you both make for the door.
            ]],
            OPT_PAY_DEBT = "Offer to pay their debt",
            DIALOG_PAY_DEBT = [[
                player:
                    $miscPersuasive
                    !placate
                    I've been brought in to make this right. {giver} and {target} made a mistake—but it was an innocent one.
                    $miscPersuasive
                    Let me pay you, and we can all leave it in the past. 
                client:
                    !agree
                    $neutralDirect
                    I only wanted profit. I see no reason to make this bloody if I'm being respected. 
                    $neutralThanks
                    Thank you, grifter. Glad to see {giver} has integrity. 
                * {giver} and {target} both relax, visibly, now that the trouble has passed. The three of you quickly make a graceful exit.
            ]],
            OPT_ATTACK = "Attack {client}",
            DIALOG_ATTACK = [[
                player:
                    !angry
                    $angryReluctant
                    We all know this is gonna come to blows, so let's just get it over with, all right?
                    !fight
                client:
                    !fight
            ]],
            DIALOG_ATTACKED_ALIVE = [[
                player:
                    !left
                client:
                    !right
                    !injured
                    $scaredAnguished
                    I yield, all right?! You robbed me and won. There's no retribution coming if you just let me walk away.
                player:
                    $angryThanks
                    Sounds like an understanding to me. 
                * With {client} beaten and browbeaten, {target} and {giver} are safe. But they don't look too happy about it.
            ]],
            DIALOG_ATTACKED_DEAD = [[
                player:
                    !left
                    !shrug
                    $neutralJoke
                    Well... at least {client} won't want payback.
                giver:
                    !right
                    !palm
                    $angryReluctant
                    Aw, Hesh. Nobody will wanna do business with me now. 
                target:
                    !right
                    $miscRelieved
                    Nobody wants to do business with a corpse, neither. {player} really saved our skin. 
                giver:
                    !left
                    !angry_accuse
                    $angryDefault
                    Flog your skin! You're the reason this all went south anyway!
                target:
                    !angry
                    $angryDefault
                    You weren't complaining when I was the only one willing to do the grunt work, bucko!
                * You leave the two partners to their renegotiations. 
            ]],
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest:GetCastAgent("giver"):MoveToLocation(cxt.location)
                cxt.quest:GetCastAgent("target"):MoveToLocation(cxt.location)
                cxt.quest:GetCastAgent("client"):MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("client"))
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_BLAME_GIVER")
                :Dialog("DIALOG_BLAME_GIVER")
                :Fn(function(cxt)
                    cxt.quest:GetCastAgent("giver"):OpinionEvent( OPINION.BETRAYED )
                    cxt.quest:GetCastAgent("client"):OpinionEvent( OPINION.APPROVE )
                    local target = cxt.quest:GetCastAgent("target")
                    target:OpinionEvent( OPINION.HELPED )
                    cxt.quest:SetProvider(target)
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                end)

            cxt:Opt("OPT_BLAME_TARGET")
                :Dialog("DIALOG_BLAME_TARGET")
                :Fn(function(cxt)
                    cxt.quest:GetCastAgent("target"):OpinionEvent( OPINION.BETRAYED )
                    cxt.quest:GetCastAgent("giver"):OpinionEvent( OPINION.DISAPPROVE_MINOR )
                    cxt.quest:GetCastAgent("client"):OpinionEvent( OPINION.APPROVE )
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                end)

            cxt:Opt("OPT_PAY_DEBT")
                :DeliverMoney(cxt.quest.param.debt, { no_scale = true })
                :Dialog("DIALOG_PAY_DEBT")
                :Fn(function(cxt)
                    cxt.quest:GetCastAgent("target"):OpinionEvent( OPINION.SETTLED_DEBT )
                    cxt.quest:GetCastAgent("giver"):OpinionEvent( OPINION.SETTLED_DEBT )
                    cxt.quest:Complete()
                    ConvoUtil.GiveQuestRewards(cxt)
                end)

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    on_win = function()
                        if cxt.quest:GetCastAgent("client"):IsDead() then
                            cxt:Dialog("DIALOG_ATTACKED_DEAD")
                        else
                            cxt:Dialog("DIALOG_ATTACKED_ALIVE")
                        end
                        cxt.quest:GetCastAgent("target"):OpinionEvent( OPINION.HELPED )
                        cxt.quest:GetCastAgent("giver"):OpinionEvent( OPINION.DISAPPROVE_MINOR )
                        cxt.quest:Complete()
                        ConvoUtil.GiveQuestRewards(cxt)
                    end
                }
        end)

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            player:
                !greeting
                $happyGreeting
                I'm offering my arm for pay. It slices, it dices—you won't find a better offer, I tell ya.
            agent:
                !interest
                $neutralThoughtful
                Maybe just a firm handshake will do the trick.
                !thought
                $neutralThoughtful
                I've got a client who, uh, "hired" my business partner and I to "acquire" some "goods" "legally".
            player:
                !dubious
                $neutralDubious
                I'm detecting a lot of euphemisms.
            agent:
                !disappoint
                $miscDepressed
                Well, my business partner took off with the funds. I <i>hope</i> {target.heshe} just went to get the goods, like {target.heshe} was supposed to. 
                But I haven't seen {target.himher} in days, and the client is breathing blisters onto my neck.
                !shrug
                $neutralWhatever
                I need you to confront {target}, find out what the hold up is. Hopefully it's a quick fix.
            player:
                !crossed
                $neutralDubious
                Yeah, calling it now: it's not.
            agent:
                $neutralResigned
                Just... find {target}, get the goods, then deliver them to our client. I'll pay you extra just to make sure this all gets smoothed over, okay?
        ]],
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "find_target", "DIALOG_OFFER")
    end)


