local function MakeLocation(quest, location_id)
    quest:SetParameter("location_id" , location_id)
    local location = Location( location_id )
    TheGame:GetGameState():AddLocation(location)
    quest:AssignCastMember("the_point", location )
end


local QDEF = QuestDef.Define{
        title = "Recon in Force",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc =[[Investigate a wilderness location for signs of illegal activity]],
        focus = QUEST_FOCUS.COMBAT,

        icon = engine.asset.Texture("icons/quests/scout_out_patrol_spot.tex"),
        on_destroy = function( quest )
            if quest:GetCastMember("the_point") then
                TheGame:GetGameState():MarkLocationForDeletion(quest:GetCastMember("the_point"))
            end
        end,
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = QuestUtil.CanDoAdmiraltyGoonQuest,
    }

    :AddLocationDefs{
        LOCATION_DEF_BEACH =
        {
            id = "PATROL_SPOT_BEACH",
            name = "Beach Patrol Point",
            desc = "Jakes can often be found smuggling goods along the shores of Murder Bay",
            
            plax = "EXT_Beach_MurderBay_1",
            map_tags = {"beach"},
            indoors = false,
        },
        LOCATION_DEF_WOODS =
        {
            id = "PATROL_SPOT_WOODS",
            name = "Woods Patrol Point",
            desc = "Bandits often make their home in the wilderness of Murder Bay",
            plax = "EXT_Forest_1",
            map_tags = {"forest"},
            indoors = false,
        }
    }

    :AddLocationCast{
        cast_id = "the_beach",
        when = QWHEN.MANUAL,
    }
    :AddLocationCast{
        cast_id = "the_woods",
        when = QWHEN.MANUAL,
    }

    :Loc{
        PICK_INVESTIGATION_LOCATION = "Pick a location to investigate",
    }


    :AddObjective{
        id = "scout_the_beach",
        title = "Investigate {the_beach#location}",
        desc = "Patrol for suspicious activity at {the_beach#location}",
        mark = { "the_beach" },
        on_activate = function(quest)
            quest:SpawnTempLocation("LOCATION_DEF_BEACH", "the_beach" )
        end,
        on_complete = function(quest) 
            quest:Activate("report_back_to_giver")
            quest:Cancel("scout_the_woods")
        end,
        on_fail = function(quest)
            quest:Fail()
        end,
    }

    :AddObjective{
        id = "scout_the_woods",
        title = "Investigate {the_woods#location}",
        desc = "Patrol for suspicious activity at {the_woods#location}",
        mark = { "the_woods" },
        on_activate = function(quest)
            quest:SpawnTempLocation("LOCATION_DEF_WOODS", "the_woods" )
        end,
        on_complete = function(quest) 
            quest:Activate("report_back_to_giver")
            quest:Cancel("scout_the_beach")
        end,
        on_fail = function(quest)
            quest:Fail()
        end,
    }


    :AddObjective{
        id = "report_back_to_giver",
        --time_limit = 1,
        title = "Return to {giver}",
        ready_for_completion = true,
        desc = "Report your findings to {giver}",
        mark = { "giver" },
    }


QDEF:AddConvo("report_back_to_giver", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_TELL = "Tell {agent} what you found",
        DIALOG_DONE = [[
            player:
                Well, officer, the job is done, and I can tell you all ab—
            agent:
                !palm
                $angryDefault
                Aw, crud! The paperwork!
                Look, keep your report to yourself. We'll just say it was quiet. 
                Right? You're alive. <i>Must've</i> been quiet.
            player:
                !dubious
                $neutralWhatever
                Uh. Sure. 
            agent:
                !happy
                Nice. Easiest money you ever made, huh?
        ]]
    }
    
    :Hub(function(cxt) 
            cxt:Opt("OPT_TELL")
                    :Dialog("DIALOG_DONE")
                    :CompleteQuest()
        end)
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("scout_the_beach")
    :Confront(function(cxt) 
        if cxt.location == cxt.quest:GetCastMember("the_beach") then
            if math.random() > .5 then
                return "STATE_BEACH_MONSTERS"
            else
                return "STATE_BEACH_PEOPLE"
            end
        end
    end)

    :State("STATE_BEACH_PEOPLE")
        :Loc{
            DIALOG_INTRO = [[
                * It's clear you don't have much practice sneaking across the sand, because the Jakes are waiting for you when you find them.
                player:
                    !left
                agent:
                    !right
                    !angry
                    $angryTakenAback
                    You <i>lost</i>? It's not exactly the weather for bathing.
                player:
                    !placate
                    Oh, uh... 
            ]],
            DIALOG_TALK_WAY_OUT = [[
                player:
                    Look, I was just trying to keep away from the Admiralty, alright?
                    !bashful
                    I'm not exactly on the good side of the law.
                agent:
                    !suspicious
            ]],
            DIALOG_TALKED_OUT = [[
                player:
                    I'll just head that way, and I never saw you, all right?
                    !interest
                    And you never saw me. Sound fair?
                agent:
                    !dubious
                    $neutralDubious
                    What about the Admiralty? Should we expect them?
                player:
                    $miscPersuasive
                    Well, I <i>heard</i> the officer who was supposed to patrol this way took the day off.
                agent:
                    !cruel
                    Hah! Lucky for all of us.
                    Now scram. And remember, none of us saw anything.
                * You leave before the Jakes can change their mind.
            ]],
            DIALOG_WIN = [[
                * You leave before more Jakes make their way ashore.
            ]],
            
            DIALOG_FAIL_TALK = [[
                player:
                    I didn't know you Jakes would be here!
                    So I'll just—
                agent:
                    !angry
                    Oh, no. You get caught by the Admiralty now, you'll just turn us in.
                    It's nothing personal, grifter.
                    !reach_weapon
            ]],
            OPT_TALK_WAY_OUT = "Talk your way out of trouble",
            OPT_ATTACK = "Attack them",
            OPT_DEFEND_SELF = "Defend yourself",
        }

        :Fn(function(cxt) 
                local monsters = CreateCombatParty("JAKES_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.encounter:SetPrimaryCast(monsters[1])
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("scout_the_beach")
                
                cxt:Opt("OPT_TALK_WAY_OUT")
                    :Dialog("DIALOG_TALK_WAY_OUT")
                    :Negotiation{
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_TALKED_OUT")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_FAIL_TALK")
                            cxt:Opt("OPT_DEFEND_SELF")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE, 
                                    advantage = TEAM.RED,
                                    on_win = function(cxt) 
                                        cxt:Dialog("DIALOG_WIN")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end
                                }
                        end
                    }
                cxt:Opt("OPT_ATTACK")
                    :Battle{
                        on_win = function(cxt) 
                            cxt:Dialog("DIALOG_WIN")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    }
            end)

    :State("STATE_BEACH_MONSTERS")
        :Loc{
            DIALOG_INTRO = [[
                * You arrive at the beach, and start looking around.
                * Just as you've about decided the coast is clear, you hear an ominous keening coming from the water.
                monster:
                    !right
                * You're going to have to fight your way out of this one.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }

        :Fn(function(cxt) 
                local monsters = CreateCombatParty("MONSTERS", cxt.quest:GetRank(), cxt.location)
                cxt.encounter:SetPrimaryCast(monsters[1])
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("scout_the_beach")
                cxt:Opt("OPT_DEFEND_SELF")
                    :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
            end)



QDEF:AddConvo("scout_the_woods")
    :Confront(function(cxt) 
            
            if cxt.location == cxt.quest:GetCastMember("the_woods") then
                if math.random() > .5 then
                    return "STATE_WOODS_MONSTERS"
                else
                    return "STATE_WOODS_PEOPLE"
                end
            end
        end)

    :State("STATE_WOODS_PEOPLE")
        :Loc{
            DIALOG_INTRO = [[
                
                * You pause a spell to catch your breath and try to pick the burrs out of your hair.
                * Good thing you do, because you can make out the sounds of voices hissing in the woods. 

                agent:
                    !right
                * Bandits!
                * They haven't seen you yet. You could try to sneak away, or get the drop on them, burrs and all.
            ]],
            
            DIALOG_TRY_SNEAK = [[
                * You decide that you've seen enough, and turn to leave.
            ]],
            DIALOG_SNEAK_AWAY = [[
                * You manage to sneak away without the bandits noticing you.
            ]],


            OPT_ATTACK = "Attack them",
            OPT_SNEAK_AWAY = "Try to sneak away quietly",
            DIALOG_WIN = [[
                * You leave before reinforcements can arrive.
            ]],
            DIALOG_BANDIT_FIGHT = [[
            * Unfortunately, you don't go unnoticed for long.
                agent:
                    !angry
                    $angryHostile
                    Hey! What are you doing here?
                player:
                    Oh, uh, I was just...
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }

        :Fn(function(cxt) 
                local monsters = CreateCombatParty("BANDIT_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.encounter:SetPrimaryCast(monsters[1])
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("scout_the_woods")
                
                cxt:Opt("OPT_ATTACK")
                    :Battle{ 
                        advantage = TEAM.BLUE, 
                        on_win = function(cxt) 
                            cxt:Dialog("DIALOG_WIN")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }
                cxt:Opt("OPT_SNEAK_AWAY")
                    :Dialog("DIALOG_TRY_SNEAK")
                    :Fn(function() 
                            if math.random() < 0.5 then
                                cxt:Dialog("DIALOG_BANDIT_FIGHT")
                                cxt:Opt("OPT_DEFEND_SELF")
                                    :Battle{
                                        on_win = function(cxt) 
                                            cxt:Dialog("DIALOG_WIN")
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end,

                                        flags = BATTLE_FLAGS.SELF_DEFENCE, 
                                        advantage = TEAM.RED,
                                    }                                
                            else
                                cxt:Dialog("DIALOG_SNEAK_AWAY")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end)

            end)

    :State("STATE_WOODS_MONSTERS")
        :Loc{
            DIALOG_INTRO = [[
                
                * You stumble through the underbrush to the designated clearing.
                * There's nothing here but whispering trees and stoic rocks.
                agent:
                    !right
                * That is, until the ravenous pack of yotes comes slobbering out of the bush.
                * Maybe those whispering trees had tried to warn you. 
            ]],
            DIALOG_WIN = [[
                * The location is now clear. You should report back to {giver}.
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
        }

        :Fn(function(cxt) 
                local monsters = CreateCombatParty("YOTES", cxt.quest:GetRank(), cxt.location)
                cxt.encounter:SetPrimaryCast(monsters[1])
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("scout_the_woods")
                cxt:Opt("OPT_DEFEND_SELF")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE,
                        on_win = function(cxt) 
                            cxt:Dialog("DIALOG_WIN")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,

                    }

            end)


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            agent:
                $neutralWhatever
                I spend half of my life on patrol, looking for who-knows-what in this Hesh-forsaken land.
                !overthere
                $neutralResigned
                "Go to Havaria," the posters said. "Experience adventure!" 
            player:
                !humouring
                The adventuring life ain't how you pictured it?
            agent:
                !shrug
                I thought the criminals would be killing me. Instead it's my feet.
                I'll pay you to do my patrol, just for a day off. Should be easy—nothing ever happens, anyway.
        ]],
        
        DIALOG_CHOICE = [[
            player:
                Sure, I can do that. 
            agent:
                $neutralThanks
                I've got seniority so you get your pick. 
                You can defend the woods against the Spree, or check for Jakes smuggling down on the beach. 
                !wink
                Easy Spree-sy, or Knock-Knock Jakes. Take your pick. 
            player: 
                !disgust
                Do I get hazard pay for that joke?
            agent:
                Nope. 

        ]],

    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, nil, "DIALOG_OFFER", function() 
                cxt.quest:Activate("scout_the_beach")
                cxt.quest:Activate("scout_the_woods")
                cxt:Dialog("DIALOG_CHOICE")
            end)
    end)


