local QDEF = QuestDef.Define{
        title = "Secret Shopper",
        qtype = QTYPE.SIDE,
        rank = {1, 3},
        desc = [[{giver} has a package ready at {shop#location}, but can't pick it up.]],
        icon = engine.asset.Texture("icons/quests/secret_shopper.tex"),
        on_init = function(quest)
            local package_value = quest:GetRank()*50
            quest:SetParameter("package_value", package_value )
            quest:SetParameter("debt_amount", math.floor( package_value * 2 ))
            quest:SetParameter("money_give", math.floor( package_value* .67) )
        end,
        focus = QUEST_FOCUS.NEGOTIATION,

        avoids = {
            {"giver", "shop"},
            {"giver", "shopkeeper"},
        },

        collect_agent_locations = function(quest, t)
            if quest:IsActive("obtain_the_package") then
                table.insert (t, { agent = quest:GetCastMember("shopkeeper"), location = quest:GetCastMember('shop'), role = CHARACTER_ROLES.PROPRIETOR } )
            end
        end,

    }

   :AddCast{
        cast_id = "giver",
        provider = true,
        condition = QuestUtil.CanDoBanditGoonQuest,
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"BANDIT_GOON","BANDIT_GOON","BANDIT_GOON2","BANDIT_RAIDER","BANDIT_CAPTAIN"}
            local def = defs[math.min( quest:GetRank(), #defs)]
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }


    :AddLocationCast{
        cast_id = "shop",
        condition = function (loc, quest) 
            return loc:HasTag("shop") and loc:GetFactionID() ~= quest:GetCastAgent("giver"):GetFactionID() and (quest:GetCastMember("giver"):GetLocation() ~= loc)
        end,
    }
    :AddCast{
        cast_id = "shopkeeper",
        cast_fn = function(quest, t)
            table.insert( t, quest:GetCastMember("shop"):GetProprietor())
        end
    }

    :AddObjective{
        id = "deliver_package",
        title = "Give the goods to {giver}",
        desc = "Return to {giver} and give them the goods",
        mark = { "giver"},
    }

    :AddObjective{
        id = "obtain_the_package",
        title = "Go to {shop#location}",
        desc = "Get {giver}'s package from {shop#location}. You may have to do some convincing and/or stealing.",

        is_negotiation_target = function(quest, agent) 
            if agent:GetLocation() == quest:GetCastMember("shop") and agent and AgentUtil.IsProprietor(agent) then
                return true
            end
        end,

        is_combat_target = function(quest, agent) 
            if agent:GetLocation() == quest:GetCastMember("shop") and agent and AgentUtil.IsProprietor(agent) then
                return true
            end
        end,
        --negotiation_targets = {""},
        mark = { "shopkeeper" },

        on_activate = function(quest)
            quest:GetCastMember("shopkeeper"):MoveToLocation(quest:GetCastMember('shop'))
        end,

        on_complete = function(quest) 
            quest:Activate("deliver_package")
        end
    }

QDEF:AddConvo("obtain_the_package", "shopkeeper")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_INQUIRE_ABOUT_PACKAGE = "Ask about {giver}'s package",
    }
       
    :Hub(function(cxt) 
            cxt:Opt("OPT_INQUIRE_ABOUT_PACKAGE")
                :GoTo("STATE_ASK_ABOUT_PACKAGE")
        end)

    :State("STATE_ASK_ABOUT_PACKAGE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !interest
                    $neutralGreeting
                    Hey, I'm here to pick up a package. You know anything about that?
                agent:
                    !overthere
                    $neutralThoughtful
                    All I got here is a package for {giver}. And {giver.heshe}'s not getting it until {giver.heshe} pays off {giver.hisher} tab.
                player:
                    $neutralDubious
                    {giver.HisHer}... tab?
                agent:
                    !agree
                    Yep. I let {giver} run it up long enough—now it's {1#money} deep.  
            ]],
            OPT_PAY_TAB = "Pay off {giver}'s tab",
            DIALOG_PAID_TAB = [[
                player:
                    !permit
                    What if I paid it off instead?
                agent:
                    !dubious
                    $neutralDubious
                    You'd do that for {giver}? Not sure you'd know what you're getting into. But you pay me {1#money}, and its yours.
                    !give
                    You can tell {giver} if {giver.heshe} ever wants back in this place {giver.heshe} still has to pay me the difference.
            ]],

            OPT_OFFER_TO_BUY_PACKAGE = "Convince {agent} to sell you the package",
            DIALOG_PRE_BUY = [[
                player:
                    !thumb
                    I just wanna buy the package for myself, actually.
                agent:
                    !dubious
                    $neutralDubious
                    Sounds suspicious. How do I know you aren't just picking it up for {giver}?
            ]],
            DIALOG_BOUGHT = [[
                player:
                    !point
                    $miscPersuasive
                    All I know is you've been stiffed, and I have shills to dress the wound. 
                agent: 
                    !shrug
                    Yeah, fair enough. At least I'm not stuck holding the bag.
                * {agent} slides the package across the counter towards you. 
            ]],
            
            OPT_SMASH_AND_GRAB = "Attack {agent} to get the package",
            DIALOG_SMASH_AND_GRAB = [[
                player:
                    !angry_threat
                    $angryHostile
                    I can tell you one thing: you want to hand that over, before I have to hurt you to get it.
                agent:
                    !angry
                    $angryTakenAback
                    You prepared to back up those threats, hunter? 'Cause you're gonna have to.
                    !fight
            ]],

            DIALOG_WON_KILLED = [[
                agent:
                    !exit
                player:
                    !take
                * You lean over the counter to grab the paper-wrapped package. It's a little bloodsplattered, but {giver} can't complain.
            ]],
            DIALOG_WON_SURRENDERED = [[
                player:
                    !cruel
                    $angryCruel
                    Now you're backed up, {agent.honorific}. You gonna hand over that package?
                agent:
                    !injured
                    $angryReluctant
                    Fine! Take it and go! But you tell {giver} {giver.heshe}'s not welcome here.
            ]],
            DIALOG_REFUSE_DEMAND = [[
                player:
                    !angry_shrug
                    $angryPatienceLost
                    You want my money or not?
                agent:
                    !handwave
                    $neutralWhatever
                    Not. You tell {giver} to solve {giver.hisher} own problems.
            ]]
        }
        :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO", cxt.quest.param.debt_amount)

                cxt:Opt("OPT_PAY_TAB")
                    :DeliverMoney(cxt.quest:GetParameter("debt_amount"), {no_scale = true})
                    :Dialog("DIALOG_PAID_TAB", cxt.quest.param.debt_amount)
                    :Fn(function(cxt) 
                            cxt.quest:Complete("obtain_the_package")
                        end)
                
                cxt:Opt("OPT_OFFER_TO_BUY_PACKAGE")
                    :RequireMoney(cxt.quest:GetParameter("package_value"))
                    :Dialog("DIALOG_PRE_BUY")
                    :Negotiation{
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_BOUGHT")
                            local amount = CalculatePayment( cxt:GetAgent(), cxt.quest:GetParameter("package_value") )
                            cxt.caravan:PayMoney( amount, cxt:GetAgent() )
                            cxt.quest:Complete("obtain_the_package")
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                    }


                cxt:Opt("OPT_SMASH_AND_GRAB")
                    :Dialog("DIALOG_SMASH_AND_GRAB")
                    :Battle{
                        no_oppo_limit = true,
                        on_win = function(cxt) 
                            if cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_WON_KILLED")
                            else
                                cxt:Dialog("DIALOG_WON_SURRENDERED")
                            end
                            cxt.quest:Complete("obtain_the_package")
                            StateGraphUtil.AddEndOption(cxt)
                        end,

                    }

                StateGraphUtil.AddBackButton(cxt)

            end)

QDEF:AddConvo("deliver_package", "giver")
    :Loc{
        OPT_GIVE_PACKAGE = "Give {giver} their package",
        DIALOG_THANKS = [[
            player:
                !give
                Here you go. One package delivered. 
            agent:
                !happy
                !take
                $happyTakenAback
                Wow, really? You're a lifesaver! Here's your bonus, as promised.
            player: 
                $happyCocky
                A pleasure. 
        ]]
    }
    :Hub(function(cxt)
        cxt:Opt("OPT_GIVE_PACKAGE")
            :Dialog("DIALOG_THANKS")
            :CompleteQuest()
    end)


QDEF:AddConvo(nil, "giver")
    :Loc{
        DIALOG_OFFER = [[
            player:
                !interest
                $neutralGreeting
                You need something?
            agent:
                !thought
                $neutralThoughtful
                Yeah, actually. I have a special package on order at {shop#location}. Only problem is I'm banned from the premises.
                You pick it up and bring it back for me, and we'll call it a job done. 
        ]],
        DIALOG_TOOK_JOB = [[
            agent:
                !give
                Here's the cash I owe for the package. This <i>ain't</i> your pay. 
            player:
                !take
            agent:
                $neutralDirect
                Bring me back what's mine and there'll be more than this waiting for you, understood?
            player:
                !agree
                Sure. No problem.
        ]]
    }
    :Hub(function(cxt)
        StateGraphUtil.OfferSideJob(cxt, "obtain_the_package", "DIALOG_OFFER", function(cxt) 
                cxt:Dialog("DIALOG_TOOK_JOB")
                cxt.caravan:AddMoney(cxt.quest:GetParameter("money_give"))
            end)
    end)

