local QDEF = QuestDef.Define{
        title = "Sing for Your Supper",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = "{target} is trying to get {giver} to turn informant. {giver} is looking for help to get out of this arrangement.",
        icon = engine.asset.Texture("icons/quests/side_sing_for_your_supper.tex"),
        focus = QUEST_FOCUS.NEGOTIATION,
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = QuestUtil.CanDoPoorCivvieQuest,
    }

    :AddCast{
        cast_id = "target",
        condition = function(agent, quest)
            if QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) then
                return not AgentUtil.IsFriend(agent, quest:GetCastAgent("giver")) and agent:GetFactionID() == "ADMIRALTY" and not agent:IsWithAgent( quest:GetCastAgent("giver") )
            end
        end,
        
        on_assign = function(quest, agent)
            if quest:GetCastMember("giver"):GetRelationship(agent) == RELATIONSHIP.NEUTRAL then
                quest:GetCastMember("giver").social_connections:SetRelationship(RELATIONSHIP.DISLIKED, agent)
            end
        end,

        events =
        {
            agent_retired = function( quest, agent )
                if quest:IsActive("confirm_turnabout") then
                    quest:Fail()
                    return
                else
                    
                    if quest:IsActive("turnabout") then
                        quest:Cancel("turnabout")
                    end

                    if quest:IsActive("confront_admiralty") then
                        quest:Complete("confront_admiralty")
                        quest:Activate("return_for_reward")
                        return
                    else
                        quest:Fail()
                    end
                end


            end,
        }
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"ADMIRALTY_GOON","ADMIRALTY_GUARD","ADMIRALTY_CLERK", "ADMIRALTY_PATROL_LEADER"}
            local def = defs[math.min( quest:GetRank(), #defs)]
            local agent = quest:CreateSkinnedAgent( def )
            table.insert( t, agent )
        end,
    }

    :AddObjective{
        id = "confront_admiralty",
        title = "Convince {target} to leave {giver} alone",
        desc = "{target} has been harassing {giver}, trying to turn them into an informant. {giver} has asked you to make it stop.",
        mark = { "target"},

        negotiation_targets = {"target"},
        combat_targets = {"target"},

        on_complete = function(quest)
            quest:Activate("return_for_reward")
        end,

    }

    :AddObjective{
        id = "turnabout",
        title = "Convince {giver} to be an informant",
        desc = "{target} will pay you if you convince {giver} to be an informant.",
        mark = { "giver", "target"},

        negotiation_targets = {"giver"},
        combat_targets = {"giver"},

        on_complete = function(quest)
            quest:Activate("confirm_turnabout")
        end,

    }

    :AddObjective{
        id = "confirm_turnabout",
        title = "Return to {target}",
        desc = "Go tell {target} that {giver} has agreed to talk.",
        mark = { "target"},
        on_activate = function(quest) 
            if quest:IsActive("confront_admiralty") then
                quest:Cancel("confront_admiralty")
            end
        end
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Return to {giver} for your reward",
        desc = "{target} won't be bothering {giver} again. Go collect your due.",
        mark = { "giver" },
        ready_for_completion = true,
        on_activate = function(quest) 
            if quest:IsActive("turnabout") then
                quest:Cancel("turnabout")
            end
        end
    }


QDEF:AddConvo("turnabout", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_CONVINCE_BE_INFORMANT = "Convince {giver} to turn informant",
        DIALOG_CONVINCE = [[
            player:
                !overthere
                $miscPersuasive
                You know, it might just be easier if you were to do what {target} wants.
                If you got intel, I think you should spill. 
            agent:
                !surprised
                $neutralTakenAback
                What are you talking about? That's how I end up dead!
        ]],
        DIALOG_NEGOTIATION_SUCCESS = [[
            player:
                !dubious
                $neutralDubious
                It's <i>Murder Bay</i>. The thing's kind of in the name.
                If you got intel on dangerous people, then it won't be long before those dangerous people see you taken care of.
                $miscPersuasive
                But if you tell {target} what {target.heshe} wants to know, the Admiralty can at least protect you.
            agent:
                $neutralResigned
                Maybe you're right. 
                !shrug
                $miscDepressed
                You work your whole life just trying to have a little slice of something you can keep to yourself.
                $neutralResigned
                But there's always someone holding the leash. 
                !disappoint
                I'll talk to {target}. Like you say, I'm probably dead either way. 
        ]],
        DIALOG_NEGOTIATION_FAIL = [[
            player:
                !overthere
                $miscPersuasive
                Look, {target.heshe}'ll just keep hassling you until you give {target.himher} what {target.heshe} wants.
            agent:
                !angry
                $angryTakenAback
                Huh! And how much is {target} paying you to say so?!
                Fool on me for thinking you were on my side just cause you were a Derrick and all.
                Well, screw you. 
        ]],
        OPT_FORCE_BE_INFORMANT = "Force {agent} to turn informant",
        
        DIALOG_FORCE_BE_INFORMANT = [[
            player:
                $miscPersuasive
                Look, you were worried that snitchin' would get you killed right?
                !angry
                $angryDefault
                So how about this: snitch, or I'll kill you. 
                Or at least make you wish you were dead. Sound fair?
            agent:
                !scared
                $neutralTakenAback
                W-why are you doing this?!
            player:
                Don't matter. Just know that I'm doing it. 
                !fight
        ]],
        DIALOG_FORCED = [[
            agent:
                !injured
                $scaredAnguished
                I'll do it! I'll do it!
            player:
                !cruel
                $happyCocky
                Excellent. I'll tell {target} the good news!
                Happy we could all see eye-to-eye-to-eye on this. 
        ]],
        DIALOG_KILLED = [[
            * {agent} chose death, apparently.
        ]]
    }
    :Hub(function(cxt)
            cxt:Opt("OPT_CONVINCE_BE_INFORMANT")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{ 
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATION_SUCCESS")
                        cxt.quest:Complete("turnabout")
                        cxt.quest:SetProvider(cxt:GetCastAgent("target"))
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATION_FAIL")
                        cxt:GetAgent():OpinionEvent(OPINION.BETRAYED)
                        cxt.quest:Fail()
                        StateGraphUtil.AddEndOption(cxt)
                    end }

            cxt:Opt("OPT_FORCE_BE_INFORMANT")
                :Dialog("DIALOG_FORCE_BE_INFORMANT")
                :Battle{ 
                    on_win = function(cxt)
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_KILLED")
                            cxt.quest:Fail()
                        else
                            cxt:Dialog("DIALOG_FORCED")
                            cxt.quest:Complete("turnabout")
                            cxt.quest:SetProvider(cxt:GetCastAgent("target"))
                        end
                        StateGraphUtil.AddEndOption(cxt)
                    end,
                }
        end)

QDEF:AddConvo("confront_admiralty", "target")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_GIVER = [[
            player:
                $neutralGreeting
                {giver} says you've been harassing {giver.himher}.
                !dubious
                $neutralDubious
                Trying to get {giver.himher} to snitch on... something?
            agent:
                $miscAlluring
                Oh, on "something" all right. What {giver} knows could put a lot of people behind bars.
                !overthere
                I thought {giver.heshe} might turn informant with a little pressure—do the right thing, you know how it is.
                $neutralThoughtful
                Why? You got some leverage I could use to make it stick?
        ]],

        OPT_ASK_ABOUT_GIVER = "Ask about {giver}",
        OPT_CONVINCE_LEAVE_ALONE = "Convince {target} to leave {giver} alone",
        DIALOG_CONVINCE_LEAVE_ALONE_START = [[
            player:
                !interest
                $miscPersuasive
                What good's an informant if you have to strongarm them?
                I think it's in your best interest to back off. 
            agent:
                !crossed
                $neutralDubious
                Is that so? And why the Hesh would I do that?
        ]],
        DIALOG_NEGOTIATION_SUCCESS = [[
            player:
                You pressure {giver}, and all that'll do is get you some rushed, bad intel. 
                $neutralResigned
                !shrug
                And a dead informant, once word gets out that {giver.heshe} snitched.
                $miscPersuasive
                But you step back, then {giver} thinks that you're okay. That you can be trusted, when the time comes.
            agent:  
                !dubious
                $neutralDubious
                "The time comes"? What time?
            player:
                It's Murder Bay. {giver}'s no crook—{giver.heshe} just has the bad luck of being in the wrong place <i>all the time</i>.
                !happy
                $happyCocky
                Eventually, things will get too hot. And <i>then</i> {giver} will come to you with all the juicy gossip you could want.
            agent:
                $neutralThoughtful
                Hrm. Patience isn't my particular virtue. But you're not wrong. 
                Fine. I can wait. Tell {giver} I'll respect {giver.hisher} decision. 
        ]],
        DIALOG_NEGOTIATION_FAIL = [[
            player:
                !point
                $neutralDirect
                {giver} said no, and you should respect that. That's all there is to it.
            agent:
                !angry
                $angrySeething
                Meanwhile, crooks run amok and there's nothing I can do to stop it. 
                Don't tell me how to do my job, hunter—your motivations are suspect, anyway.
        ]],
        OPT_OFFER_TO_HELP_CONVINCE_GIVER = "Offer to help convince {giver}",
        DIALOG_TURNAROUND = [[
            player:
                $miscPersuasive
                Sounds like you have your reasons. I can get {giver} to talk to you, if you're willing to pay.
            agent:
                !agree
                Of course. There's always pay involved. 
                You get {giver} talking, and you'll get what's coming to you.
            player:
                !dubious
                $neutralDubious
                Err—
            agent:
                !palm
                $neutralResigned
                Sorry, I'm used to talking in threats. What I mean is, you'll get paid. 
        ]],

        OPT_BEAT_UP = "Beat up {target} to make them back off",
        DIALOG_BEAT_UP = [[
            player:
                !point
                $angryHostile
                You're going to leave {giver} alone.
            agent:
                !hips
                $angryHostile
                No, I'm not. 'Cause it's my job. 
                So you wanna stop me? You'll have to <i>stop me</i>, get it?
            player:
                !fight
                $angryHostile
                Oh, I get it. Thanks for the invite.
        ]],
        DIALOG_KILLED = [[
            agent:
                !exit
            * Well, that's one way to solve a problem.
        ]],

        DIALOG_SURRENDERED = [[
            agent:
                !injured
                $scaredAnguished
                Alright, I give up. Tell {giver} I'll bark up a different tree from now on.
                But {giver} knows too much. And one day? Murder Bay will come to collect. 
                {giver.HeShe} will <i>wish</i> {giver.heshe} had gone along when I gave {giver.himher} the chance.
            player:
                !Cruel
                $happyCocky
                Ooh, I love the sound of a hollow threat.
                I'll be sure to pass it along, though. 
        ]]

    }
    :Hub(function(cxt)
            
            cxt:Opt("OPT_ASK_ABOUT_GIVER")

                :Dialog("DIALOG_GIVER")
                :LoopingFn(function(cxt) 
                        
                        cxt:Opt("OPT_CONVINCE_LEAVE_ALONE")
                            :Dialog("DIALOG_CONVINCE_LEAVE_ALONE_START")
                            :Negotiation{ 
                                on_success = function(cxt)
                                    cxt:Dialog("DIALOG_NEGOTIATION_SUCCESS")
                                    cxt.quest:Complete("confront_admiralty")
                                    cxt:Pop()
                                end,
                                on_fail = function(cxt)
                                    cxt:Dialog("DIALOG_NEGOTIATION_FAIL")
                                end }

                        if not cxt.quest.param.counteroffer then
                            cxt:Opt("OPT_OFFER_TO_HELP_CONVINCE_GIVER")
                                :Dialog("DIALOG_TURNAROUND")
                                :Fn(function(cxt) 
                                        cxt.quest.param.counteroffer = true
                                        cxt.quest:Activate("turnabout")
                                        cxt:Pop()
                                    end)
                        end

                        cxt:Opt("OPT_BEAT_UP")
                            :Dialog("DIALOG_BEAT_UP")
                            :Battle{
                                on_win = function() 
                                    if cxt:GetCastAgent("target"):IsDead() then
                                        cxt:Dialog("DIALOG_KILLED")
                                    else
                                        cxt:Dialog("DIALOG_SURRENDERED")
                                    end
                                    cxt.quest:Complete("confront_admiralty")
                                    StateGraphUtil.AddEndOption(cxt)
                                end,
                        }

                        StateGraphUtil.AddBackButton(cxt)                        
                    end)
        end)

QDEF:AddConvo("return_for_reward", "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_GET_PAID = "Tell {giver} that {target} won't bother {giver.himher} anymore",
        DIALOG_INTRO_KILLED = [[
            player:
                !interest
                Well, the good news is that you don't gotta worry about {target} anymore.
            giver:
                !crossed
                $neutralDubious
                Saying it like that suggests there's bad news.
            player:
                $miscEmbarrassed
                You're <i>technically</> an accessory to murder, now. 
                'Cause {target}'s dead.
            giver:
                !scared_shrug
                $scaredDefault
                Aw, Hesh. Everyone knows {target} was trying to get me to talk! 
                Guess I shouldn't be surprised this is how it turned out.
                !disappoint
                $neutralResigned
                They don't call it Murder Bay for nothing.
        ]],
        DIALOG_INTRO = [[
            player:
                $neutralDirect
                It's done. You don't need to worry about {target} any more.
            giver:
                !happy
                $happyThanks
                Geez, {player}, you really did that? You sure do deliver.
                Thanks a million. Now I can just... live my life, ya know?
            player:
                !happy
                Murder Bay be damned.
            giver:
                Yeah. Murder Bay be damned. 
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog(cxt.quest:GetCastAgent("target"):IsDead() and "DIALOG_INTRO_KILLED" or "DIALOG_INTRO")
            :CompleteQuest()
    end)

QDEF:AddConvo("confirm_turnabout", "target")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        OPT_GET_PAID = "Tell {target} that {giver} is ready to talk",
        DIALOG_INTRO = [[
            player:
                $happyCocky
                {giver} has had a change of heart. You should be getting the answers you're looking for soon.
            target:
                $happyAmazed
                Nice job! I hope you didn't have to hurt {giver.himher} too much.
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_INTRO")
            :CompleteQuest()
    end)


QDEF:AddConvo(nil, "giver")
    :Loc{
        DIALOG_OFFER = [[
            player:
                !hips
                Hey, do you—
            agent:
                $scaredFearful
                !scared
                I'm no rat! I swear!
            player:
                I—
            agent:
                !angry_accuse
                $angryTakenAback
                I won't talk, you hear me?! 
            player:
                !dubious
                That's—
            agent:
                !angry
                $scaredAnguished
                You can tell {target} {target.hisher} threats won't make no difference!
            player:
                !placate
                $neutralDubious
                Hold on. What's {target} threatening you for?
            agent:
                $neutralDubious
                You aren't one of {target}'s goons? 
                {target.HeShe} wants me to snitch, but I ain't suicidal. Back on the derrick we learn to keep our mouths <i>shut</i>.
            player:
                Want me to get {target.himher} off your back for you? I can offer that kind of service.
                !happy
                At a good rate, even.
            agent:
                !interest
                Hesh, you'd do that?
        ]],
        DIALOG_TAKEN = [[
            agent:
                !happy
                $happyAmazed
                You get {target} to lay off, and I'll pay you what you're worth, I swear.
                I just wanna live and let live, you know? 
            player:
                !agree
                Sure. Leave it to me. 
        ]],
    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "confront_admiralty", "DIALOG_OFFER", function() cxt:Dialog("DIALOG_TAKEN") end)
    end)