local QDEF = QuestDef.Define{
        title = "Termination of Command",
        qtype = QTYPE.SIDE,
        rank = {2, 5},
        desc = [[{giver} wants you to deliver {target} an official reprimand.]],
        icon = engine.asset.Texture("icons/quests/side_termination_of_command.tex"),
        focus = QUEST_FOCUS.BALANCED,

        on_destroy = function(quest)
            if quest:GetCastMember("support") and quest:GetCastMember("support"):IsInPlayerParty() then
                quest:GetCastMember("support"):Dismiss()
            end
        end,

        avoids = 
        {
            {"giver", "support"}
        },

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("target"), location = quest:GetCastMember("camp")} )
            if quest:IsActive("find_support") then
                table.insert (t, { agent = quest:GetCastMember("support"), location = quest:GetCastMember("lighthouse")} )
            end
        end,

    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent)
            return agent:GetContentID() == "ADMIRALTY_CLERK" and agent:GetRelationship() >= RELATIONSHIP.NEUTRAL
        end,
    }
    :AddCastFallback{
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "ADMIRALTY_CLERK" ) )
        end,
    }


    :AddCast{
        cast_id = "support",
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "ADMIRALTY_GOON" ) )
        end,
    }


    :AddCast{
        cast_id = "target",
        condition = function(agent)
            return agent:GetContentID() == "ADMIRALTY_CLERK" and not agent:GetBrain():GetWorkPosition()
        end,
    }
    :AddCastFallback{
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "ADMIRALTY_CLERK" ) )
        end,
    }


    :AddQuestLocation{
        cast_id = "camp",
        name = "Camp",
        plax = "EXT_ForestShrine_01",
        map_tags = {"forest"},
        indoors = false,
    }

    :AddLocationCast{
        cast_id = "lighthouse",
        cast_fn = function(quest, t)
            table.insert(t, TheGame:GetGameState():GetLocation("LIGHTHOUSE"))
        end,
    }

    :AddObjective{
        id = "find_support",
        title = [[Find {support}]],
        mark = {"support"},
        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("support") then
                fighter:AddCondition("WANTED_ALIVE")
            end
        end,
        
        on_complete = function(quest) 
            quest:Activate("terminate_command")
        end
    }

    :AddObjective{
        id = "terminate_command",
        title = [[Terminate {target}]],
        mark = {"target"},
        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("target") then
                fighter:AddCondition("WANTED_DEAD")
            end
        end,
    }


    :AddObjective{
        id = "convince_dead",
        title = [[Convince {giver} {target} is dead]],
        mark = {"giver"},
    }

    :AddObjective{
        id = "return_for_reward",
        title = [[Return to {giver} for your reward]],
        mark = {"giver"},
    }


QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            * {giver} looks up from {giver.hisher} clipboard as you approach.
            player:
                !shrug
                !dubious
                Word is you're looking for a grifter.
            agent:
                The <i>Admiralty</> needs something taken care of, yes.
            player:
                Oh, sounds official.
                !dubious
                Don't you people have goons for that?
            agent:
                We do. But this mission is of a sensitive nature. I can't trust our people to do it.
            player:
                !happy
                Now this sounds lucrative!
            agent:
                One of our own, {target}, has gone rogue out in the wilderness.
                We sent an Admiralty team to bring {target.himher} back in, but they were compromised. 
                Now {target} is holed up somewhere out there in the wilderness, operating beyond the pale of any acceptable conduct, still commanding troops.
                !throatcut
                I want you to find {target} and terminate {target.hisher} command.
                
        ]],
        DIALOG_ACCEPTED = [[
            player: 
                I'll do it.
            agent:
                Good.
                A member of the compromised team has been spotted back in Murder Bay.
                Find {support.himher}, and get {target}'s location, and infiltrate it.
            player:
                And bring {target} back?
            agent:
                No. Just bring back {target.hisher} badge. {target.HisHer} remains can belong to the wilderness.
            player:
                Ah, this is a take-no-prisoners situation.
                Got it.
        ]]

    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "find_support", "DIALOG_OFFER", 
            function() 
                cxt:Dialog("DIALOG_ACCEPTED") 
            end)
    end)


    


QDEF:AddConvo("find_support", "support")
    :ConfrontState("STATE_CONFRONT", function(cxt) return not cxt.quest.param.saw_support end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !exit
                * You see {support}, the rogue Admiralty agent you were sent to look out for. {support.HeShe} hasn't noticed you yet.
            ]]
        }
        :Fn(function(cxt)
           cxt.quest.param.saw_support = true
           cxt:Dialog("DIALOG_INTRO")
           StateGraphUtil.AddEndOption(cxt)
        end)

    :AttractState("STATE_ATTRACT")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * You find {agent} standing with the other hired meat at the beach.
                    * {agent.HisHer} furtive sideways glances and jumpiness have not found {agent.himher} any work so far today.
                    agent:
                        !cagey
                        Hey, you have work for me? I'm affordable. 
                    player:
                        You're Admiralty, aren't you? Don't you already have a job?
                    agent:
                        What are you, Internal Corrections? What does it matter?
                        !placate
                        The uniform is a feature, not a detriment. Means I can move with impunity.
                        So are you hiring or not?
                    player:
                        $miscPersuasive
                        Well, sure, I do have some work that needs doing.
                        I'm looking for somebody. {target}. Think you could find {target.himher} for me?
                    agent:
                        !scared
                        Oh. Oh, no.
                        No, sorry. I, uh, can't help you with that. 
                        !bashful
                        Never got missing persons training, see.
                }
                {not first_time?
                    agent:
                        I told you I can't help you find {target}.
                        !suspicious
                        O-or whatever name it was you said.

                }
            ]],

            OPT_HIRE = "Hire {agent} as a guide",
            DIALOG_HIRE = [[
                player:
                    I'm a grifter, and so are <i>you</> now, apparently.
                    As one grifter to another, let me give you two pieces of advice.
                    First: don't leave loose ends. If you and {target} have unfinished business, it's going to come back on you, eventually.
                    And second: don't pass up an opportunity to make money.
                    I'm going to do you the professional courtesy of offering you a way to follow both of those rules at once.
                    I'll pay you some red hot shills, you'll take me to {target}, and we'll take care of {target.himher}. Deal?
                agent:
                    ...
                    I'm going to regret this, aren't I?
                player:
                    Absolutely not. Because it's <i>very</i> good advice.
                agent:
                    !sigh
                    Deal.
            ]],

            OPT_CONVINCE = "Convince {agent} that {target} must be stopped",
            DIALOG_CONVINCE = [[
                player:
                    !permit 
                    $miscPersuasive
                    Clearly something's got you rattled. Talk to me.
            ]],
            
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    $miscPersuasive
                    You're facing something down alone, and it doesn't have to be that way. 
                    I'm here to help.
                agent: 
                    $miscRelief
                    Y-yeah. Maybe you <i>can</i> help.
                    !shrug
                    And if you can't, well. Maybe throwing you in {target.hisher} direction will buy me some time.
                player:
                    !interest
                    Ominous! But I'm glad we've come to an agreement.

            ]],
            
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    !interest
                    If {target}'s so scary, don't you think {target.heshe}'ll come for you sooner or later?
                agent:
                    !crossed
                    $neutralDirect
                    Yep. And there's nothing I can do, nowhere I can go, that could change that.
            ]],

            OPT_FORCE = "Force {agent} to lead you to {target}",
            DIALOG_KILLED_SUPPORT = [[
                * {agent}'s blood seeps into the sand, and it doesn't do much to point you in the right direction. Seems you'll never find {target} now.
            ]],
            
            DIALOG_FORCE = [[
                player:
                    !fight
                    You haven't got your head on straight. Let me knock it around a little until you can think clearly.
            ]],

            DIALOG_FORCED = [[
                player:
                    Now, are you gonna play nice and show me the way to {target}?
                agent:
                    !injured
                    Clearly fate has decided it should be so. 
                    I don't think you're going to like what we find when we meet {target}.
                player:
                    What we'll find is the end of a job, and I like getting paid.
                    Now lead the way.
            ]],


            OPT_ASK_ABOUT_TARGET = "Ask about {target}",
            DIALOG_ASK_ABOUT_TARGET = [[
                player:
                    {target}'s name had an effect on you, that's for sure.
                agent:
                    Don't know what you mean. 
                player:
                    !dubious
                    You're trembling. I can see the ripples in the mud at your feet.
                agent:
                    Maybe I'm just cold.
                player:
                    Look, I get it. {target} went rogue. {target.HeShe} probably found some bankroll or another to exploit.
                    But {target.heshe} can't be more dangerous than any of your other superiors.
                agent:
                    Only one superior more dangerous, and I don't make a habit of contemplating the Abyss.
                player:
                    Wait, what?
                * {agent} looks off into the distance, {agent.hisher} mouth set in a thin line. 
            ]]
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:RunLoop(function()


                cxt:Question("OPT_ASK_ABOUT_TARGET", "DIALOG_ASK_ABOUT_TARGET")
                cxt:Opt("OPT_HIRE")
                    :Dialog("DIALOG_HIRE")        
                    :DeliverMoney(cxt.quest:GetRank()*50)
                    :RecruitMember(PARTY_MEMBER_TYPE.ESCORT)
                    :CompleteQuest("find_support")
                    :Fn(function() cxt.quest.param.hired_support = true end)
                    :DoneConvo()

                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_SUCCESS")
                            :RecruitMember(PARTY_MEMBER_TYPE.ESCORT)
                            :CompleteQuest("find_support")
                            :Fn(function() cxt.quest.param.convinced_support = true end)
                            :DoneConvo()

                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_FAILURE")

                cxt:Opt("OPT_FORCE")
                    :Dialog("DIALOG_FORCE")
                    :Battle{}
                        :OnWin()
                            :Fn(function() 
                                if cxt:GetAgent():IsDead() then
                                    cxt:Dialog("DIALOG_KILLED_SUPPORT")
                                    cxt.quest:Fail()
                                    StateGraphUtil.AddEndOption(cxt) 
                                else
                                    cxt:Dialog("DIALOG_FORCED")   
                                    cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)                                 
                                    cxt.quest.param.forced_support = true
                                    cxt.quest:Complete("find_support")
                                    StateGraphUtil.AddEndOption(cxt) 
                                end
                            end)

                StateGraphUtil.AddEndOption(cxt)
            end)

        end)


local function AddBattleOption( cxt, opt )
    local allies = {}
    if not cxt.quest.param.forced_support then
        table.insert(allies, cxt.quest:GetCastMember("support"))
    end

    cxt:Opt( opt )
        :Battle{
            flags = BATTLE_FLAGS.ISOLATED,
            no_oppo_limit = true,
            allies = allies,
        }
            :OnWin()
                :Fn(function() 
                    cxt.quest:Complete("terminate_command")

                    cxt.enc.scratch.support_here = not cxt.quest.param.forced_support and not cxt.quest:GetCastMember("support"):IsDead()
                    cxt:Dialog("DIALOG_WIN")
                    if cxt.enc.scratch.support_here then
                        cxt.quest:GetCastMember("support"):Dismiss()
                    end

                    if not cxt:GetAgent():IsDead() then
                        cxt:GetAgent():Retire()
                        cxt.quest:Activate("convince_dead")
                    else
                        cxt.quest:Activate("return_for_reward")
                    end
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
end

QDEF:AddConvo("terminate_command", "target")
    :SetFlag( CONVO_FLAGS.NO_PRESENT)
    :Loc
    {
        OPT_ATTACK = "Attack {agent}",
        OPT_ATTACK_FOLLOWERS = "Attack {agent} and {agent.hisher} followers",
        DIALOG_WIN = [[
            {dead?
                * {agent} is dead. Despite {agent.hisher} conviction, you don't see {agent.himher} coming back to life anytime soon.
            }

            {not dead?
                agent:
                    You... have left me alive. 
                    Why? Why deny me the chance to do better next time?
                player:
                    Enh, sounds like your life is punishment enough.
                agent:
                    ...
                    This does not bode well.
                    If I'm not the victor, and if I don't die... am I still making progress?
                    Or am I just losing resolve while standing still?
                player:
                    Guess you don't know everything, huh? Why not go back to the drawing board, leave the Admiralty alone.
                    Do your weird spooky hermit thing until you have all the answers.
                    I can always come back and kill you later.
                agent:
                    I'd like that. 
                    !exit
                * {agent} fades silently into the tree line, leaving you to contemplate the vastness of your confusion.
                    
            }

            {support_here?

                player:
                    Alright, {support}? That wasn't so bad.
                    What exactly was it that blew your mind so much, anyway?
                support:
                    !right
                    What, you're just gonna learn that the only person responsible for your own happiness is yourself, and shrug it off like it's no big deal?
                player:
                    Huh. That <i>is</i> a bummer.
                    ...
                    Still, if I have agency, then I choose to still blame others when I feel like it.
                    As a treat.
                support:
                    Whoa. WHOA.
                    So you're saying I can choose to continue livin' my life just the way I did before this all happened?
                player:
                    That's free will, bud. 
                support:
                    I need to have a think.
                    !exit
                * {support} staggers off into the camp, massaging {support.hisher} temples.

            }
        ]],
    }
    :ConfrontState("STATE_ATTRACT")
        :Loc{
            DIALOG_INTRO = [[

                * {support} leads you on a winding path through the forest, and you sharpen your wits as you walk, memorizing the twists and turns.
                * At least you'll be able to find your way back easily enough: the trail is marked with char and strange, off-colour stains you can't place.
                support:
                    !right
                    It's just up ahead.
                player:
                    !left

                {hired_support?
                    support:
                        I can't believe the things I'll do for a couple of shills.
                    player:
                        Enh, you get used to it. After you.
                    support:
                        !exit
                    player:
                        !exit

                    * You follow {support} into a campsite. Everything looks in order, but there's a stink like blood and something you might find under a sewer grate.                     
                }

                {forced_support?
                    support:
                        I'd say good luck, but luck has no place here.
                        You want my advice: just accept whatever's coming. It'll go down easier.
                        !exit
                    * {support} slips away like oil in the water, so quick you don't even realize you're alone.

                    * You turn and continue into a campsite. Everything looks in order, but there's a stink like blood and something you might find under a sewer grate. 
                }

                {convinced_support?
                    support:
                        !placate
                        $scaredStammering
                        H-hey, maybe you should go on without me, yeah?
                        I don't know if I can do this.
                    player:
                        $miscAnnoyed
                        What? You flaking out on me?
                    support:
                        !scared_shrug
                        I wouldn't wish a head-to-head confrontation with {target} on anybody.
                        But you still don't get it. I don't think I can handle what {target.heshe} has to say.
                        Not again.
                    player:
                        !crossed
                        I need your arms, not your ears. So stuff 'em full of wad for all I care.
                        If it gets real bad, I promise to start shrieking every work song I know.
                        And we had a lot of them on the derrick.
                    support:
                        !sigh
                        $neutralResigned
                        I'm gonna regret this.
                    player:
                        !agree
                        $neutralJoke
                        Definitely. I wasn't kidding when I said "shrieking". Now let's go.
                    support:
                        !exit
                    player:
                        !exit
                    * You follow {support} into a campsite. Everything looks in order, but there's a stink like blood and something you might find under a sewer grate. 

                }

            ]],


            DIALOG_MEET_TARGET = [[
                target:
                    !right
                    Hello, {player}. I've been expecting you.
                player:
                    !left
                    Is that so? You must have a good network for someone who's ghosted {target.hisher} whole faction.

                {forced_support?
                    target:
                        I feel the suffering of my children—even wayward ones like {support}.
                        The violence you inflicted upon {support.himher} lanced the boil of expectation that had been growing under my skin for quite some time.
                        
                }
                {hired_support?
                    target:
                        I feel the excitement of my children—even wayward ones like {support}.
                        The opportunity to earn shills, while base, sends tremors through the energies that bind us together.
                        And likewise I feed back my own excitement—my own opportunity to meet <i>you</i>.
                }
                {convinced_support?
                    target:
                       I feel {support}'s doubt as if it were my own. I let it grow, to see what fruit the vine might yield.
                       And here you are.
                }

                player:
                    I've spent enough time with the Cult to know mumbo-jumbo when I hear it.
                    Spare me your energies or whatever woo-woo you've got spinning.
                target:
                    And yet more questions gather, perched at the tip of your mind.
                    I'll let you ask them, if you'd like. Before it's too late.
                    I am my own master and, whether you accept it or not, so are you.

            ]],

            OPT_ASK_ABOUT_METHODS = "Ask {agent} what {agent.heshe}'s doing",
            DIALOG_ASK_ABOUT_METHODS = [[
                player:
                    I've heard of folks being discharged from the Admiralty, but what I've been hired to do is kind of extreme.
                    I'd be lying if I said I wasn't piqued.
                target:
                    The wilderness allows me to meditate on the true nature of this world.
                    When I share my insights, however, it leaves people feeling a bit... bare.
                    Stripped of purpose. Of title. Of mental fortitude, even.
                player:
                    So if you tell me, I'll run raving into the depths of the wilderness, huh?
                target:
                    That's up to you. The Heshians would have you believe that your actions are prescribed.
                    You are tethered to the Abyss, pushed and pulled by the tides.
                    What they don't tell you... is that you are the true master. 
                    We were not destined for this conflict. 
                    Somewhere in the strands, we were allies. Even friends. 
                    A reality where it was <i>I</i> who sent you to eliminate {giver}.
                player:
                    Ah, I get it. This is your counter-offer.
                target:
                    !sigh 
                    No. You still only see the veneer, and not the grain.
                    Perhaps you weren't ready for this message.
            ]],

            OPT_ASK_ABOUT_ADMIRALTY = "Ask why the Admiralty wants {agent} terminated",
            DIALOG_ASK_ABOUT_ADMIRALTY = [[
                player:
                    So you've gone a little off your rocker.
                    Happens a lot in the griftlands. Still doesn't explain why the Admiralty wants you dead.
                agent:
                    Because with this knowledge I have grown powerful.
                    Though the right often bickers with the left, the Admiralty and the Cult are two tentacles of their beloved god.
                    And I sit on the cusp of a chasm even greater than their Abyss, and my mind alone can plumb its depths. 
                    They send you after me because they see themselves as extensions of a greater will.
                    You accept "the job" because you see yourself as an extension of theirs. 
                    But reach weakens the grip. You are diluted—a vessel of someone else's intent.
                    While I am my own master.
            ]],

            OPT_SCARE_AWAY_FOLLOWERS = "Scare away {agent}'s followers",
            DIALOG_SCARE_AWAY_FOLLOWERS = [[
                player:
                    And what about your followers, then? Are they their own masters?
            ]],
            NEGOTIATION_REASON = "Scare off {agent}'s supporters",
            DIALOG_SUCCESS = [[
                player:
                    What about your followers? Do they have free will?
                    Will it be their choice to die when they stay and fight for you?
                agent:
                    You assume death is a consequence, when really it is a new beginning.
                    We all play the cards we're dealt. And my hand has grown powerful.
                * Perhaps {agent}'s followers weren't as confident in {agent.hisher} teachings as {agent.heshe} thought. 
                * But they exercise their free will enough to slink into the trees, leaving {agent} to stand alone.
                player:
                    Great. Then you won't be put out when I win.   
                    !fight
            ]],
            DIALOG_FAILURE = [[
                player:
                    That's a good line you got. "Join my weird philosophy, where you can choose to die!"
                    Because that is what they're choosing. Death.
                {scared_goons?
                * You've struck fear into the hearts of {agent}'s followers, but {agent.heshe} quells it with a stern word.
                    agent:
                        Of course you're all free to leave, if that is your wish. 
                        But you will lose the chance for a fresh start, as you've begged for since the beginning.
                }
                agent:
                    We live, we die. It's all progress.
                    But I do not intend to retire just yet.
                    !fight
            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            local helpers = {}

            if cxt.quest.param.forced_support then
                cxt.quest:GetCastMember("support"):Retire()
            else
                table.insert(helpers, cxt.quest:GetCastMember("support"))
            end

            cxt:TalkTo("target")

            local defs = {
                    {"ADMIRALTY_GOON","VROC", "VROC"},
                    {"ADMIRALTY_GUARD","ADMIRALTY_GOON", "VROC"},
                    {"ADMIRALTY_GOON","ADMIRALTY_GUARD","ADMIRALTY_GOON"},
                    {"ADMIRALTY_PATROL_LEADER","ADMIRALTY_GUARD","ADMIRALTY_GOON"},
                    {"ADMIRALTY_PATROL_LEADER","ADMIRALTY_GUARD","ADMIRALTY_GOON", "ADMIRALTY_GOON"},
                }

            local fight_allies = CreateCombatBackup(cxt:GetAgent(), defs[math.min(cxt.quest:GetRank(), #defs)] , cxt.quest:GetRank())

            cxt:Dialog("DIALOG_MEET_TARGET")

            cxt:RunLoop(function() 

                cxt:Question("OPT_ASK_ABOUT_METHODS", "DIALOG_ASK_ABOUT_METHODS")
                cxt:Question("OPT_ASK_ABOUT_ADMIRALTY", "DIALOG_ASK_ABOUT_ADMIRALTY")

                cxt:Opt("OPT_SCARE_AWAY_FOLLOWERS")
                    :Dialog("DIALOG_SCARE_AWAY_FOLLOWERS")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.ALLY_SCARE,
                        enemy_resolve_required = 5 + cxt.quest:GetRank()*5,
                        fight_allies = fight_allies,
                        reason_txt = cxt:GetLocString("NEGOTIATION_REASON"),
                        helpers = helpers,
                    }
                        :OnSuccess()
                            :Fn(function(cxt, minigame) 
                                
                                local keep_allies = {}
                                for i, modifier in minigame:GetOpponentNegotiator():Modifiers() do
                                    if modifier.id == "FIGHT_ALLY_SCARE" and modifier.ally_agent then
                                        table.insert( keep_allies, modifier.ally_agent )
                                    end
                                end

                                for k,v in pairs(fight_allies) do
                                    if not table.arrayfind(keep_allies, v) then
                                        v:MoveToLimbo()
                                    end
                                end
                            end)
                            :Dialog("DIALOG_SUCCESS")
                            :GoTo("STATE_FIGHT")

                        :OnFailure()
                            :Fn(function(cxt, minigame) 
                                cxt.enc.scratch.scared_goons = not (minigame:GetOpponentNegotiator():FindCoreArgument() and minigame:GetOpponentNegotiator():FindCoreArgument():GetShieldStatus())
                            end)
                            :Dialog("DIALOG_FAILURE")
                            :GoTo("STATE_FIGHT")

                AddBattleOption( cxt, "OPT_ATTACK_FOLLOWERS" )

            end)
        end)



        :State("STATE_FIGHT")
            :Fn(function(cxt) 
                
                AddBattleOption( cxt, "OPT_ATTACK" )

            end)


QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_GET_PAID = "Get paid",
        DIALOG_GET_PAID = [[
            player:
                I've dealt with {target}.
            agent:
                Good. Now never speak of this again.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :SetQuestMark()
            :Dialog("DIALOG_GET_PAID")
            :Fn(function() 
                ConvoUtil.GiveQuestRewards(cxt)
            end)
            :CompleteQuest()
    end)

QDEF:AddConvo("convince_dead", "giver")
    :Loc{
        OPT_CONVINCE = "Convince {giver} that {target} is dead",
        DIALOG_CONVINCE = [[
            player:
                Mission accomplished. {target} has been eliminated.
                {target.HeShe} won't be bothering anyone with {target.hisher} heretical flaff anymore.
            agent:
                Wait, you spoke with {target.himher}?
                Anyone else who spoke with {target.himher} turned on their positions or ended up dead.
                How do I know you haven't been incepted with some psychic conditioning?
            player:
                !dubious 
                $neutralDubious
                What-a-whatting?
        ]],

        DIALOG_CONVINCE_SUCCESS = [[
            player: 
                Look, {target} said something about free will and yadda yadda yadda...
                If you'd wanted me to take notes, you should've specified.
            agent:
                !point 
                No! No notes!
                {target} already has the grunts spinning stories about mind-control and subliminal messaging.
                It'll take a while for those superstitions to die out, but at least it's taken care of.
                !give
                Thank you for your work.
            player:
                Hey, no problem. 

        ]],

        DIALOG_CONVINCE_FAILURE = [[
            player:
                Look, if this is about the free will stuff, I don't really think {target.heshe} was at full mast, you know?
                I took the wind out of {target.himher} sails. {target.HeShe}'s no longer a threat.
            agent:
                !angry
                So {target.heshe} <i>hasn't</i> been killed.
                Then you won't be paid. It's as simple as that. 
                And I'll have to find some other way to neutralize {target.himher}.
                !exit
            * Well, that was one way to waste part of the day.
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE")
            :SetQuestMark()
            :Dialog("DIALOG_CONVINCE")
            :Negotiation{}
                :OnSuccess()
                    :Dialog("DIALOG_CONVINCE_SUCCESS")
                    :Fn(function() 
                        ConvoUtil.GiveQuestRewards(cxt)
                    end)
                    :CompleteQuest()
                    :DoneConvo()
                :OnFailure()
                    :Fn(function() cxt.quest:Fail() end)
                    :DoneConvo()

    end)    