local LOCATION_DEF =
{
    id = "THIEF_CAMP",
    name = "Campsite",
    desc = "Makeshift campsite",
    plax = "EXT_SpreeCampsite_1",
    indoors = false,
    map_tags = {"tents"},
}

if not Content.GetLocationContent(LOCATION_DEF.id) then
    Content.AddLocationContent(LOCATION_DEF)
end

local BRIBE_AMOUNT = 100

local QDEF = QuestDef.Define{
        title = "Ice Cold",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{giver} is offering a reward for hunting down a thief.]],
        icon = engine.asset.Texture("icons/quests/thief_bounty.tex"),
        avoids = 
        {
            {"giver", "thief"},
            {"giver", "contact"},
            {"thief", "contact"},
        },
        focus = QUEST_FOCUS.BALANCED,

        is_in_hiding = function(quest, agent)
            if not quest:IsActive("capture_thief") and agent == quest:GetCastMember("thief") then
                return true
            end
        end,
        -- icon = engine.asset.Texture("icons/quests/attack_rival.tex"),
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent)
            return agent:GetFactionID() == "BANDITS"
        end,
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            local agent = quest:CreateSkinnedAgent( "BANDIT_RAIDER" )
            table.insert( t, agent )
        end,
    }


    
    :AddCast{
        cast_id = "thief",
        
        condition = function(agent, quest)
            return agent:GetFactionID() == "BANDITS" and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) and not AgentUtil.IsCoworker(agent, quest:GetCastMember("giver")) and not AgentUtil.IsFriend(agent, quest:GetCastMember("giver"))
        end,
        events = 
        {
            agent_retired = function( quest, agent )
                
            end,
        },
        
        on_assign = function(quest, agent)
            if quest:GetCastMember("giver"):GetRelationship(agent) == RELATIONSHIP.NEUTRAL then
                quest:GetCastMember("giver").social_connections:SetRelationship(RELATIONSHIP.DISLIKED, agent)
            end
        end

    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"BANDIT_GOON", "BANDIT_GOON", "BANDIT_GOON2", "BANDIT_RAIDER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }


    :AddCast{
        cast_id = "contact",
        condition = function( agent, quest )
            local thief = quest:GetCastMember("thief")
            if agent.social_connections:GetRelationship(thief) >= RELATIONSHIP.NEUTRAL and QuestUtil.FilterAgentForDifficulty(quest:GetRank(), agent, true, true) then
                return true
            end
        end,
        on_assign = function(quest, agent)
            if quest:GetCastMember("thief"):GetRelationship(agent) == RELATIONSHIP.NEUTRAL then
                quest:GetCastMember("thief").social_connections:SetRelationship(RELATIONSHIP.LIKED, agent)
            end
        end
    }:AddCastFallback
    {
        cast_fn = function(quest, t)
            local defs = {"LABORER", "BANDIT_GOON", "HEAVY_LABORER", "JAKES_SMUGGLER", "HEAVY_LABORER"}
            local agent = quest:CreateSkinnedAgent( defs[math.min( quest:GetRank(), #defs)])
            table.insert( t, agent )
        end,
    }

    :AddLocationCast{
        cast_id = "campsite",
        when = QWHEN.MANUAL,
        no_validation = true,
    }

    :AddObjective{
        id = "find_contact",
        title = [[Find the thief's contact]],
        desc = "Rumor has it {contact} knows how to find {thief}.",
        mark = { "contact" },
        
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
        negotiation_targets = {"contact"},
        combat_targets = {"contact"},
    }

    :AddObjective{
        id = "capture_thief",
        title = [[Find and Capture]],
        desc = "Find the thief and bring them in.",
        mark = { "thief" },
        
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },

        on_activate = function( quest)
            local location = Location( LOCATION_DEF.id )
            assert(location)
            TheGame:GetGameState():AddLocation(location)
            quest:AssignCastMember("campsite", location )
            quest:GetCastAgent("thief"):MoveToLocation(location)
        end,

        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("thief") then
                fighter:AddCondition("WANTED_ALIVE")
            end
        end,
    }

    :AddObjective{
        id = "return_with_proof",
        title = "Get reward from {giver}",
        desc = [[Tell {giver} about {thief}'s fate and receive your reward.]],
        ready_for_completion = true,
        mark = { "giver" },
        terminal = true,
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
    }

    :AddObjective{
        id = "return_with_captive",
        title = "Get reward from {giver}",
        desc = [[Return to {giver} with {thief} to claim your reward.]],
        ready_for_completion = true,
        mark = { "giver" },
        terminal = true,
        cast = 
        {
            giver = 
            {
                provider = true
            }
        },
    }

QDEF:AddConvo("find_contact", "contact")
    :Loc{
        DIALOG_INTRO = [[
            agent:
                $neutralThoughtful
                What's that look about?
        ]],
        OPT_ASK_ABOUT = "Ask {agent} about {thief}",
        DIALOG_ASK_ABOUT = [[
            player:
                $neutralDirect
                I'll be straight with you. I'm only here 'cause I'm looking for {thief}.
                You know anything about where {thief.heshe} might be?
            agent:
                $neutralTakenAback
                Might do. Mightn't, too. 
                !dubious
                None of your business either way.
        ]],
        
    }
    :Hub(function(cxt) 
        cxt:Dialog("DIALOG_INTRO")
        cxt:Opt("OPT_ASK_ABOUT")
            :Dialog("DIALOG_ASK_ABOUT")
            :GoTo("STATE_GET_INFO")

    end)
    :State("STATE_GET_INFO")
        :Loc{
            OPT_CONVINCE = "Convince {agent} to tell you",
            DIALOG_CONVINCE = [[
                player:
                    !placate
                    $miscPersuasive
                    No need for hardball, pal. I just want to talk to {thief.himher}, that's all.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !overthere
                    I've got good news, actually. A big windfall.
                    You wouldn't want to stand in the way of that, would you?
                agent:
                    !dubious
                    $neutralTakenAback
                    A windfall?
                    Hm. {thief} is hiding out at a campsite. I'll mark it for you.
                    Maybe the ingrate will see fit to share some of that good fortune.
                player:
                    !happy
                    $miscMocking
                    Why, {thief.heshe} just might!
            ]],
            DIALOG_CONVINCE_FAIL = [[
                agent:
                    $angryPatienceLost
                    I know what kind of "talking" you typically do, hunter.
                    Leave me out of it.
            ]],
            OPT_BRIBE = "Bribe {agent} to butter {agent.himher} up",
            DIALOG_BRIBED = [[
                player:
                    $neutralThanks
                    Here, take this. Think of it as a token of goodwill.
                agent:
                    $neutralDubious
                    Uhh, okay. What's the catch?
            ]],
            OPT_ATTACK = "Beat the information out of {agent}",
            DIALOG_ATTACK = [[
            
            ]],
            DIALOG_BEAT = [[
                agent:
                    !injured
                    $scaredAnguished
                    Alright, alright, I'll tell you! Just let me live.
                    {thief} is hiding at a campsite, I'll show you the way.
                player:
                    $angryCruel
                    The hard way always works.
            ]],
            DIALOG_KILLED = [[
                * {agent} was your only hope of finding {thief}.
            ]],
            TT_BRIBED = "This will reduce {agent}'s negotiation resolve by <#RESOLVE>{1}</>",
            SIT_MOD = "Bribed",
        }
        :SetLooping()
        :Fn(function(cxt)
            local BRIBE_RESOLVE_AMT = 10
            if cxt:FirstLoop() then
                cxt.encounter:SetPrimaryCast(cxt:GetCastAgent("contact"))
                cxt.enc.scratch.negotiation_tried = false
            end

            if cxt.quest.param.bribed then
                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation{
                        situation_modifiers =
                        {
                            { value = -BRIBE_RESOLVE_AMT, text = cxt:GetLocString("SIT_MOD") }
                        },
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                            cxt.quest:Complete("find_contact")
                            cxt.quest:Activate("capture_thief")
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_FAIL")
                            cxt.enc.scratch.negotiation_tried = true
                        end,
                    }
            else
                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation{
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                            cxt.quest:Complete("find_contact")
                            cxt.quest:Activate("capture_thief")
                            StateGraphUtil.AddEndOption(cxt)
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIALOG_CONVINCE_FAIL")
                            cxt.enc.scratch.negotiation_tried = true
                        end,
                    }
            end
            

            if not cxt.enc.scratch.negotiation_tried and not cxt.quest.param.bribed then
                cxt:Opt("OPT_BRIBE")
                    :PreIcon( global_images.giving )
                    :DeliverMoney( BRIBE_AMOUNT )
                    :Dialog("DIALOG_BRIBED")
                    :PostText("TT_BRIBED", BRIBE_RESOLVE_AMT)
                    :Fn(function(cxt)
                        cxt.quest.param.bribed = true
                    end)
            end

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    on_win = function(cxt)
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_KILLED")
                            cxt.quest:Fail()
                        else
                            cxt:Dialog("DIALOG_BEAT")
                            cxt.quest:Complete("find_contact")
                            cxt.quest:Activate("capture_thief")
                        end
                        StateGraphUtil.AddEndOption(cxt)
                    end
                }

            StateGraphUtil.AddBackButton(cxt)
        end)
    
QDEF:AddConvo("capture_thief")
    :Loc{
        DIALOG_INTRO = [[
            * You find the camp, though it looks as if it's seen better days.
            * {thief} looks as downtrodden as the campground. 

            agent:
                !right
            player:
                !left
                {thief}, I presume? 
            agent:
                $neutralDubious
                Depends. What's it to you?
            player:
                $miscPersuasive
                I've got something real special for whoever owns that name.
            * {thief}'s hand twitches towards {thief.hisher} weapon. 
        ]],
        OPT_CONVINCE = "Convince {agent} to come with you",
        DIALOG_CONVINCE = [[
            player:
                !placate
                $miscPersuasive
                Hold on, there! This ain't bad news, friend. 
                You've got an inheritance, even. Your great-great-grandparent, Addicus, has died and left you everything.
            agent:
                !interest
                $neutralTakenAback
                My great-great-what?
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                !thought 
                $happyAmazed
                Thousands of shills, actually. Ready to be claimed by the only known heir: {thief} Mossi. 
                That's you, right?
            agent:
                $neutralDubious
                Mossi? That ain't my—
                $scaredStammering
                I mean, uh... 
                !agree
                $neutralDirect
                Yeah. {thief} Mossi. That's me.
                !scared_shrug
                Absolutely <i>devastated</i> that Grandparent Addicus is dead.  
                !happy
            * {agent} follows behind, trying and failing to keep the skip out of {agent.hisher} step.
        ]],
        DIALOG_CONVINCE_FAIL = [[
            player:
                !scared_shrug
                $neutralResigned
                Grandparent. Addicus. You know, the one with all the money?
            agent:
                !angry
                $angryPatienceLost
                My family ain't ever had <i>any</i> money. Why else would I be in <i>Murder Bay</i>?
                Now enough with the lies. 
        ]],
        OPT_ATTACK = "Attack and capture {agent}",
        DIALOG_ATTACK = [[
            player:
                !fight
                $angryHostile
                Ugh, fine. Let's do this the hard way, then. 
            agent:
                !angry
                As if there was ever an easy way. 
        ]],
        DIALOG_BEAT = [[
            agent:
                !injured
                $scaredAnguished
                Alright, alright! I'll go with you!
                I don't know what this is about. You a hunter? Why would anyone have a bounty on me?
            player:
                !cruel
                $miscMocking
                I'm sure you'll figure it out before too long.
                
        ]],
        DIALOG_KILLED = [[
            * With {thief} dead, you can say goodbye to that bonus. You haul the corpse over your shoulder and make your way back to {giver}.
        ]],
    }
    
    :ConfrontState("STATE_HELLO", function(cxt, who) return who == nil and cxt.location == cxt.quest:GetCastAgent("thief"):GetLocation()  end)
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastAgent("thief"))
                cxt:Dialog("DIALOG_INTRO")
            end


            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest:Complete("capture_thief")
                        cxt.quest:Activate("return_with_captive")
                        cxt.quest:GetCastAgent("thief"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        
                        StateGraphUtil.AddLeaveLocation(cxt)
                        
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                    end,

                }

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    on_win = function(cxt)
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_KILLED")
                            cxt.quest:Complete("capture_thief")
                            cxt.quest:Activate("return_with_proof")
                        else
                            cxt:Dialog("DIALOG_BEAT")
                            cxt.quest:GetCastAgent("thief"):Recruit(PARTY_MEMBER_TYPE.CAPTIVE)
                            cxt.quest:Complete("capture_thief")
                            cxt.quest:Activate("return_with_captive")
                        end
                        
                        StateGraphUtil.AddLeaveLocation(cxt)
                        
                    end
                }
        end)

QDEF:AddConvo("return_with_captive", "giver")
    :Loc{
        OPT_PAYMENT = "Return {thief} to {giver}",
        DIALOG_COMPLETE = [[
            player:
                !left
                Here you go. One {thief}, coming in hot.
            agent:
                !right
                !neutral
                $neutralWhatever
                Heh. This will be good.
            thief:
                !left
                !surprise
                What the—
                {agent}?! What the hell?
            agent:
                Thought I'd forgotten about what you stole, huh?
                !angry
                Thought I'd just <i>let it slide</i>? 
                !angry_accuse
                Well surprise, slugsucker. 
            thief: 
                !angry_shrug
                What I stole? You mean the—the dang <i>pickled talons</i>?
                !interest
                There were only two left in the whole jar!
            agent:
                I was saving them! And I was <i>hungry</i>!
                You're gonna pay, you rotten, thievin'—
            player:
                !left
                !placate
                Uh, don't mean to interrupt, but...
            agent:
                Yeah, yeah. You did your part, and delivered this traitor.
                Here, take your payment, and bonus besides. 
                I'll deal with {thief.himher}.
            player:
                !dubious 
                Good luck with that.         
        ]],
        DIALOG_LEAVE = [[
            * As you walk away you hear the first punch land. But it's none of your business now.    
        ]]

    }
    :Hub(function(cxt) 
        
        local opt = cxt:Opt("OPT_PAYMENT")
            :PreIcon( global_images.giving )
            :Dialog("DIALOG_COMPLETE")

        if not cxt.quest:GetCastAgent("thief"):IsCaptiveMember() then
            opt:ReceiveOpinion(OPINION.BETRAYED, nil, cxt.quest:GetCastAgent("thief"))
        end

        opt:Fn(function(cxt)
                cxt.quest:GetCastAgent("thief"):Dismiss()
                -- ADD BONUS HERE
            end)
            :CompleteQuest()
            :Dialog("DIALOG_LEAVE")
            :Travel()

    end)

QDEF:AddConvo("return_with_proof", "giver")
    :Loc{
        OPT_PAYMENT = "Show {giver} proof of {thief}'s death",
        DIALOG_COMPLETE = [[
            player:
                !shrug 
                $neutralResigned
                {thief} was suspicious, even with three years to cool off.
                Maybe you're not the only person {thief.heshe} had to fear. 
            agent:
                !angry
                $neutralWhatever
                Not surprising. {thief} was the worst roommate I ever had.
            player:
                !interest
                Roommate? Really?
            agent:
                !disappoint
                $neutralResigned
                Yeah... we were close, too, before—
                !angry
                Well. What's done is done.
                Here's your pay.
            * You leave {agent} to {agent.hisher} brooding.               
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_PAYMENT")
            :Dialog("DIALOG_COMPLETE")
            :CompleteQuest()
            :Travel()
    end)


QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            player:
                !interest
                You {agent}? I hear you're looking to hire a hunter for a job.
            agent:
                Really? What strange providence. 
                I do indeed need some hunting done.
                !cross
                {thief} stole something from me then crawled under a rock to hide. Likely hoping I'd grow forgetful.
            player:
                $neutralDubious
                I take it you haven't.
            agent:
                !angry
                $angryCruel
                It's been three years, so I'd say not. Time to have my revenge, I think.
            player:
                !dubious
                $neutralDubious
                Wait, you waited <i>three years</i>?
                You don't even seem all that mad anymore.
            agent:
                !agree
                $neutralWhatever
                Yeah. {thief.HeShe}'ll never see it coming.
        ]],
        DIALOG_ACCEPTED = [[
            agent:
                !point
                $neutralDirect
                Bring {thief.himher} to me. There's a bonus if {thief.heshe}'s alive.
            player:
                $miscPersuasive
                Three years is a long time to sniff out a lead.
            agent:
                $neutralThoughtful
                Fortunately {thief} stinks. Go find {contact}, maybe you can get something out of {contact.himher}.
        ]]

    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "find_contact", "DIALOG_OFFER", function() cxt:Dialog("DIALOG_ACCEPTED") end)
    end)

