local QDEF = QuestDef.Define{
        title = "Travelling Salesgrifter",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{giver} needs some help making deliveries.]],
        icon = engine.asset.Texture("icons/quests/side_travelling_salesgrifter.tex"),
        focus = QUEST_FOCUS.BALANCED,

        on_destroy = function(quest)
            if quest:GetCastMember("giver") and quest:GetCastMember("giver"):IsInPlayerParty() then
                quest:GetCastMember("giver"):Dismiss()
            end
            if quest:GetCastMember("snail") and quest:GetCastMember("snail"):IsInPlayerParty() then
                quest:GetCastMember("snail"):Dismiss()
            end
        end
    }

    :AddCast{
        cast_id = "giver",
        provider = true,
        condition = function(agent)
            return agent:GetContentID() == "JAKES_LIFTER" and agent:GetRelationship() >= RELATIONSHIP.NEUTRAL
        end,
    }
    :AddCastFallback{
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "JAKES_LIFTER" ) )
        end,
    }


    :AddCast{
        cast_id = "snail",
        no_validation = true,
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "MERCHANT_SNAIL" ) )
        end,
    }


    :AddQuestLocation{
        cast_id = "street_corner",
        name = "Street Corner",
        plax = "EXT_door_slums1",
        map_tags = {"city"},
        tags = {"city"},
        indoors = false,
    }

    :AddQuestLocation{
        cast_id = "clearing",
        name = "Clearing",
        plax = "EXT_Forest_1",
        map_tags = {"forest"},
        indoors = false,
    }

    :AddQuestLocation{
        cast_id = "beach",
        name = "Desolate Beach",
        plax = "EXT_Beach_MurderBay_1",
        map_tags = {"beach"},
        indoors = false,
    }

    :AddObjective{
        id = "first_delivery",
        title = [[Make the first delivery]],
        mark = {"clearing"},
        
        on_complete = function(quest) 
            quest:Activate("second_delivery")
        end
    }

    :AddObjective{
        id = "second_delivery",
        title = [[Make the second delivery]],
        mark = {"beach"},
    }
    
    :AddObjective{
        id = "third_delivery",
        title = [[Make the third delivery]],
        mark = {"street_corner"},
    }

    :AddOpinionEvents{
        helped_sell = {
            delta = OPINION_DELTAS.LIKE,
            txt = "Helped them make a sale",
        },
        lost_sale = {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Lost them a sale",
        },
    }


QDEF:AddConvo("first_delivery")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("clearing") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find a lone {admiralty?Admiralty|Spree} waiting in clearing.
                player:
                    !left
                giver:
                    !right 
                    I'll do the talking. This should only take a minute.
                    !exit
                * {giver} does the talking, alright—but as the minutes pass, it's clear things are getting a bit heated.
                agent:
                    !right
                    !crossed
                giver:
                    !left
                    !angry_accuse
                    Now you just walk that tongue of yours right back where it came from. We agreed to a price.
                agent:
                    We also agreed to a delivery window. And you missed it.
                giver:
                    So? The goods are stable, they ain't turned. 
                agent:
                    !angry
                    My mood has, though. So I say: half price. Or no deal.
            ]],

            OPT_CONVINCE = "Help convince {agent} to pay the full price",
            DIALOG_CONVINCE = [[
                player:
                    !left
                    !bashful
                    Easy, friend.
                    Seems to me you're neglecting the implications of a late oshnu.
                agent:
                    And what would those implications be?
                player:
                    !placate
                    You see this slime? 
                agent:
                    !disgust
                    Ugh!
            ]],
            DIALOG_FAILURE = [[
                player:
                    !overthere
                    This slime means the oshnu is extra special. So we had to take it extra slow. 
                    $miscPersuasive
                    You wouldn't pay half price for an extra-special delivery, would you?
                agent:
                    And now my delivery's <i>extra</i> especially covered in slime, is that it?
                giver:
                    !left
                    Don't listen to this grifter, {agent}. She doesn't know an eyestalk from a love dart.
                    !exit
                * {giver} and {agent} finish their negotiation. You get the impression {giver} did not come out ahead.
                player:
                    !left
                giver:
                    !right
                    !palm
                    I hired you for your muscle, not your mouth. 
                    Let's go. And just keep a hold on the oshnu, all right?
                    !exit
            ]],

            DIALOG_SUCCESS = [[
                player:
                    !overthere
                    $miscPersuasive
                    This slime is the result of a rare crossbreed, an oshnu known for its mucus, but not for its speed.
                    $happyAmazed
                    Apply this mucus beneath the eyes and you'll shed years—Hesh, even decades!—while you sleep. 
                    !crossed
                    And best of all? Your goods are covered in it. So we're throwing it in for free. Provided the deal stands.
                agent:
                    !suspicious
                    $neutralDubious
                    What, you serious?
                player:
                    !wink
                    $miscMischievous
                    As the grave. Which you won't be seeing for a long time yet, thanks to this slime. 
                    !exit
                giver:
                    !left
                    !scared_shrug
                    $neutralDubious
                    So, uh... full price, right?
                agent:
                    !thought
                    Full price, sure. If I can't outwit my enemies, I may as well outlast them. 
                    !exit
                giver:
                    !exit
                * {giver} and {agent} wrap up their transaction with a somewhat slippery handshake.  
                player:
                    !left
                giver:
                    !right
                    !happy
                    Thanks for the help, {player}. Where'd you get the idea for that, huh? Anti-aging mucus?
                player:
                    One of the clergy on the derricks swore by it. 
                    !shrug 
                    $neutralJoke
                    'Course, the lumin made his teeth fall out, so it was pretty hard to gauge the results.
                giver:
                    Well, that was good thinking, alright.
                    !overthere
                    Let's do the next delivery.
            ]],

            OPT_SAY_NOTHING = "Don't get involved",
            DIALOG_SAY_NOTHING = [[
                left:
                    !exit
                right:
                    !exit
                * You watch as {agent} talks {giver} down to a price so low it's a wonder {giver.heshe} doesn't need a shovel.
                player:
                    !left
                giver:
                    !right
                    !palm
                    $neutralResigned
                    Well, that didn't go as well as I hoped it would.
                    I'll have to make up the difference on volume.
                    !overthere
                    Let's go to the next delivery.
            ]]
        }
        :Fn(function(cxt) 

            local agent = AgentUtil.GetFreeAgent(table.arraypick{"BANDIT_GOON", "ADMIRALTY_GOON"})
            cxt:TalkTo(agent)
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{helpers = {"giver"}}
                    :OnSuccess()
                        :Dialog("DIALOG_SUCCESS")
                        :ReceiveOpinion("helped_sell", nil, "giver")
                        :CompleteQuest("first_delivery")
                        :Travel()
                    :OnFailure()
                        :Dialog("DIALOG_FAILURE")
                        :ReceiveOpinion("lost_sale", nil, "giver")
                        :CompleteQuest("first_delivery")
                        :Travel()


            cxt:Opt("OPT_SAY_NOTHING")
                :Dialog("DIALOG_SAY_NOTHING")
                :CompleteQuest("first_delivery")
                :Travel()
        end)



QDEF:AddConvo(nil, "giver")
    :Priority(CONVO_PRIORITY_HIGHEST)
    :Loc{
        DIALOG_OFFER = [[
            * Folks give the Jake a wide berth, likely on account of the mucus staining {giver.hisher} leathers.
            player:
                !placate
                I hear you're looking for help—but if it's a laundress you're after, I heard wrong.
            agent:
                !dubious
                $neutralDubious
                A laundress? What?
                Naw. I'm looking for someone who's got a way with oshnus. 
            player:
                Dunno if I'd claim that much. Not to someone like you, anyway.
            agent:
                !suspicious
                Eh? Oh. You mean the mucus.
                The slimy bastards dump gallons of it on the regular. That's why I need your help.
            player:
                !dubious
                Uh...
            agent:
                I have some deliveries to make, and I need someone to help me guide the oshnu on the way. 
                It's easy enough. Though <i>you</i> might need some laundry done yourself when it's done.
                
        ]],
        DIALOG_ACCEPTED = [[
            player:
                !neutral
                $miscMischievous
                What can I say? I have a soft spot for slimeballs. 
            agent:
                !angry_shrug
                That's uncalled for, innit?
            player:
                ...
                !dubious
                I was talking about the oshnu.
            agent:
                !interest
                Oh!
                Right.
                Well, here. Take the lead.
            snail:
                !right
            * {agent} hands you the end of a leather lead, slippery with slime. The other end is attached to an ornery-looking oshnu.
            player:
                !happy
                $happyAmazed
                Aw, who's a cuti—
            * The lead threatens to slip from your grip at even the slightest pressure.
            player:
                !surprise
                Gah!
            agent:
                !right
                $angryReluctant
                You see? At least your gloves aren't already saturated with it.
                Now let's get moving. We got two stops to make before the deed is done.
        ]]

    }
    :Hub(function(cxt) 
        StateGraphUtil.OfferSideJob(cxt, "first_delivery", "DIALOG_OFFER", 
            function() 
                cxt:Dialog("DIALOG_ACCEPTED") 
                cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                cxt.quest:GetCastMember("snail"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
            end)
    end)





QDEF:AddConvo("second_delivery")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("beach") end)
        :Loc{
            DIALOG_INTRO = [[
                * As you approach the beach, the oshnu bucks in alarm, mucus flying.
                snail:
                    !right
                player:
                    !left
                    !flinch
                    Whoa, there, slimy. What's'a matter?
                snail:
                    !exit
                * The oshnu retracts into its shell as a group of Jakes approach. And by the look on {giver}'s face, you suspect {giver.heshe} wishes {giver.heshe} could crawl right in there with it.
                agent:
                    !right
                    !angry
                    There {giver.heshe} is! And there's our supper!
                giver:
                    !left
                    !bashful
                    Hey, {agent}. Funny running into you here, of all places.
                agent:
                    !threaten
                    I ain't laughin', thief. But I will be once I have you <i>and</i> that oshnu up on the spit.
                player:
                    !left
                    !placate
                    Hang on. The oshnu's yours? And you were gonna eat it?
                    But it's... mighty slimy.
                agent:
                    Because we been fattening it up, haven't we? Until {giver} here stole it right from our larder.
                giver:
                    !right
                    !angry_accuse
                    Ol' Luthie's still got years left in 'im. I won't let you guzzle down a fine pack steed.
                agent:
                    !right

            ]],


            OPT_THREATEN = "Scare off the Jakes",
            DIALOG_THREATEN = [[
                player:
                    !left
                    !angry
                    Ol' Luthie's under my protection, you understand?
            ]],
            DIALOG_FAILURE = [[
                agent:
                    Are you... threatening me?
                player:
                    !thought
                    Was that not clear?
            ]],
            DIALOG_FAILURE_2 = [[
                * {agent} sucker punches you in the gut, and then draws steel.
                player:
                    !injured
                agent:
                    !fight
                    !exit
            ]],
            DIALOG_SUCCESS = [[
                player:
                    !throatcut
                    You just turn yourselves right around, else I'll make a shishkabob outta your organs.
                agent:
                    !placate
                    Hey, now—no need for threats.
                    {giver}'s right. If the oshnu's got a few more deliveries in it, then we can wait.
                    $neutralResigned
                    There's always tomorrow's supper, after all.
                    !exit
                * The Jakes leave the beach in a hurry, but not without leaving a few choice mutters in their wake.
                giver:
                    !right
                    Over my dead body. This oshnu's been with me for years. 
                player:
                    !neutral
                    $neutralResigned
                    Seems they've worked up an appetite, though. Not sure how much time that buys you and Luthie.
                giver:
                    Yeah, well. That's why we need to make these deliveries. 
                    !eureka
                    Enough shills, and I can buy us both a ticket into Pearl-on-the-Foam.
                player:
                    !interest
                    That's a rich neighborhood. What would a Jake and a slimy oshnu do there?
                giver:
                    Enter the races, of course! With all this mucus, Luthie's extra aerodynamic. 
                player:
                    !dubious
                    Uh... huh.

            ]],


            OPT_DEFEND = "Defend the oshnu",
            DIALOG_DEFEND = [[
                player:
                    !fight
            ]],



        }
        :Fn(function(cxt) 

            local opfor =  CreateCombatParty("JAKES_PATROL", cxt.quest:GetRank()+1, cxt.location)
            
            cxt:TalkTo(opfor[1])
            
            cxt:Dialog("DIALOG_INTRO")
            
            local current_health, max_health = TheGame:GetGameState():GetPlayerAgent():GetHealth()

            
            cxt:Opt("OPT_THREATEN")
                :Dialog("DIALOG_THREATEN")
                :Negotiation{flags = NEGOTIATION_FLAGS.INTIMIDATION, helpers = {"giver"}}
                    :OnSuccess()
                        :Dialog("DIALOG_SUCCESS")
                        :GoTo("STATE_OPTIONAL_OFFER")
                    
                    :OnFailure()
                        :Dialog("DIALOG_FAILURE")
                        :DeltaHealth(math.max( -10, -current_health+1))
                        :Dialog("DIALOG_FAILURE_2")
                        :Fn(function(cxt) 
                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle{
                                    no_oppo_limit = true,
                                    allies = {cxt.quest:GetCastMember("giver"), cxt.quest:GetCastMember("snail")},
                                    flags = BATTLE_FLAGS.SELF_DEFENCE
                                }
                                    :OnWin()
                                        :GoTo("STATE_POST_FIGHT")
                        end)
                        :Travel()

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        no_oppo_limit = true,
                        allies = {cxt.quest:GetCastMember("giver"), cxt.quest:GetCastMember("snail")},
                        flags = BATTLE_FLAGS.SELF_DEFENCE
                        }   
                        :OnWin()
                            :GoTo("STATE_POST_FIGHT")        
        end)

        :State("STATE_POST_FIGHT")
            :Loc{
                DIALOG_WON_FIGHT_ALL_ALIVE = [[
                    player:
                        !left
                        !neutral
                    giver:
                        !right
                        Those scumbags! Luthie's been by my side since he was as big as my finger.
                        Nobody's eating him, slime or no.
                        Thanks for your stickin' your neck out, grifter. It's more than some of the other Jakes would'a done.
                    player:
                        Yeah, well. Luthie's grown on me. Or at least his mucus has.
                ]],
                
                DIALOG_WON_FIGHT_SNAIL_DEAD = [[
                    * The dust settles, and it's clear that all that remains of the oshnu is its rapidly liquefying corpse and the slime on your hands.
                    player:
                        !left
                        !neutral
                    giver:
                        !right
                        !scared
                        L-Luthie?
                    player:
                        !bashful
                        I, uh...
                        I'm sorry for your loss.
                    giver:
                        !tired 
                        At least we killed the bastards.
                        Could you... could you say a few words?
                    player:
                        !scaredStammering
                        Me?
                    giver:
                        It's what Luthie would've wanted.
                    player:
                        !forgetful
                    * You string together as much of a memorial as you can manage, given the short notice and the growing smell.
                ]],

                DIALOG_WON_FIGHT_GIVER_DEAD = [[
                    right:
                        !exit
                    player:
                        !left
                        !neutral
                    * You survived, but {giver} didn't make it. Seems your part here is at an end. 

                ]],
            }
            :Fn(function(cxt) 
                if not cxt.quest:GetCastMember("giver"):IsDead() and not cxt.quest:GetCastMember("snail"):IsDead() then
                    cxt:Dialog("DIALOG_WON_FIGHT_ALL_ALIVE")
                    cxt:GoTo("STATE_OPTIONAL_OFFER")
                else
                    if cxt.quest:GetCastMember("giver"):IsDead() then
                        cxt:Dialog("DIALOG_WON_FIGHT_GIVER_DEAD")
                        cxt.quest:Fail()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    else
                        cxt:Dialog("DIALOG_WON_FIGHT_SNAIL_DEAD")
                        ConvoUtil.GiveQuestRewards(cxt)
                        cxt.quest:Complete()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end
                end
            end)

        :State("STATE_OPTIONAL_OFFER")
            :Loc{
                DIALOG_INTRO = [[
                    agent:
                        So, this didn't work out like it was supposed to. I have an idea, though. 
                        Are you good for one more delivery?
                    player:
                        !crossed
                        Is this going to be another ambush?
                    agent:
                        !placate
                        No! Of course not!
                        Probably not.
                        !sigh
                        Maybe. But it's the only way I'm going to turn a profit.
                        !happy
                        I'll give you an upfront bonus.
                ]],

                OPT_TAKE_OFFER = "Take the offer",
                DIALOG_TAKE_OFFER = [[
                    player:
                        !agree
                        I'm game.
                    agent:
                        Really? That's great!
                        Okay. Keep your wits about you. This one might go south.
                    player:
                        $neutralJoke
                        Bound to happen, after the other two went so well.
                        Lead on. Luthie and I are ready.
                        !exit
                ]],

                OPT_REJECT_OFFER = "Reject the offer",
                DIALOG_REJECT_OFFER = [[
                    player:
                        I don't think so, pal. I wish you two the best, but this is more than we agreed.
                    agent:
                        Fair's fair, I guess. Here's your pay.
                        !give
                    player:
                        !take
                    agent:
                        !exit
                    * You leave {giver} staring out across the water, contemplating {giver.hisher} fate.
                    
                ]],

            }
            :Fn(function(cxt) 
                cxt:TalkTo(cxt.quest:GetCastMember("giver"))
                
                cxt:Dialog("DIALOG_INTRO")
                
                cxt:Opt("OPT_TAKE_OFFER")
                    :ReceiveMoney(cxt.quest:GetRank()*50)
                    :Dialog("DIALOG_TAKE_OFFER")
                    :Fn(function() 
                        cxt.quest:Complete("second_delivery")
                        cxt.quest:Activate("third_delivery")
                    end)
                    :Travel()
                
                cxt:Opt("OPT_REJECT_OFFER")
                    :Dialog("DIALOG_REJECT_OFFER")
                    :Fn(function() 
                        ConvoUtil.GiveQuestRewards(cxt)
                        cxt.quest:Complete()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)

            end)




QDEF:AddConvo("third_delivery")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("street_corner") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                * {giver} finds a dredger deep in the warrens of the city.
                agent:
                    !right
                    $miscHushedUrgent
                    What are <i>you</> doing here?
                    !cagey
                    This place is crawling with Admiralty—
            ]],
            DIALOG_INTRO_2 = [[
                * As if summoned by the name, a cadre of switches appears from around the corner.
                giver:
                    !scared
                agent:
                    !right
                    Well, well. Looks like a smuggling operation unfolding right before our very eyes.
                    !cruel
                    Unless, of course, that oshnu is carrying alms for the poor.
            ]],

            OPT_ATTACK = "Attack the Admiralty",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    Oh, we're carrying, alright. 
                    Get 'em!
            ]],

            OPT_NEGOTIATE = "Convince {agent} to look the other way",
            DIALOG_NEGOTIATE = [[
                {liked?
                    player:
                        !left
                        Hey, {agent}. You got the wrong idea.
                    agent:
                        Sal? Let's hear it, then.
                }
                {not liked?
                    player:
                        !left
                        !placate
                        Now, now—let's not jump to conclusions.
                }
            ]],

            DIALOG_SUCCESS = [[
                player:
                    This oshnu's on its way to the Heshians. They're to turn it into a soup for hungry bellies.
                agent:
                    !crossed
                    Charity? From the Heshians? That'll be a first.
                player:
                    !forgetful
                    Well, this one's got... psychotropic pheromones. They wanna dope the masses, I think. 
                agent:
                    $neutralWhatever
                    Hm. That checks out. Carry on, then. 
            ]],

            DIALOG_FAILURE = [[
                player:
                    !overthere
                    This oshnu is carrying, uh, rags. And... medicines.
                    But not very good ones. Mostly just oils, you know. The medicines that don't work.
                    $scaredStammering
                    'Cause if they worked, we'd be giving them to rich folks. And not... the poor.
                agent:
                    !crossed
                    $angryPatienceLost
                    Excuse me if I don't find this rambling of yours all that convincing. 
                    Let's see some proof, huh?
            ]],

            OPT_BRIBE = "Offer a bribe",
            DIALOG_BRIBE = [[
                player:
                    !left
                    Oh, you said <i>alms for the poor</i>. Yes, indeed! We got alms right here, in fact. 
                    Here, try some.
                    !give
                agent:
                    !take
                    Well, wouldn't ya know? That's mighty charitable of you.
                    If anyone else catches you, mind, then this never happened. 
                    Be gone by the time I come around this way again. 
                    !exit
            ]],
            DIALOG_AD_LEAVE = [[
                * The Admiralty leave.
                giver:
                    !right
                    $miscRelieved
                    Quick thinking, {player}. Not sure I could've handled that half so well.
                    Now stand guard, if you don't mind, while I handle the negotiations.
                    !exit
                * {giver} bends {giver.hisher} head in conversation with the dredger and proceeds to get fleeced.
                * You're polite enough not to mention it. 
            ]],

            DIALOG_WON_FIGHT = [[
                giver:
                    !right
                    Wasn't expecting that much trouble, but I guess that's just the business we choose to be in, right?
                    Right?
                    Anyway, thanks for the help. I'll take it from here.
                    !exit
                * You give it at most two days before {giver} has lost every last shill.
            ]],
            DIALOG_GIVER_DEAD = [[
                * {giver} is dead, and more trouble is on its way. You leave before it finds you.
            ]]

        }
        :Fn(function(cxt) 

            local agent = AgentUtil.GetFreeAgent(table.arraypick{"CHEMIST"})
            cxt:TalkTo(agent)
            cxt:Dialog("DIALOG_INTRO")
            local opfor =  CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank()+1, cxt.location)
            cxt:TalkTo(opfor[1])
            cxt:Dialog("DIALOG_INTRO_2")

            cxt:RunLoop(function() 
                cxt:Opt("OPT_NEGOTIATE")
                    :Dialog("DIALOG_NEGOTIATE")
                    :Negotiation{
                        helpers = {"giver", agent}
                        }
                        :OnSuccess()
                            :Dialog("DIALOG_SUCCESS")
                            :Dialog("DIALOG_AD_LEAVE")
                            :Fn(function() 
                                ConvoUtil.GiveQuestRewards(cxt)
                                cxt.quest:Complete()
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)
                        :OnFailure()
                            :Dialog("DIALOG_FAILURE")


                cxt:Opt("OPT_BRIBE")
                    :DeliverMoney(cxt.quest:GetRank()*60)
                    :Dialog("DIALOG_BRIBE")
                    :Dialog("DIALOG_AD_LEAVE")
                    :Fn(function() 
                        ConvoUtil.GiveQuestRewards(cxt)
                        cxt.quest:Complete()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)

                
                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{
                        allies = {cxt.quest:GetCastMember("giver"), cxt.quest:GetCastMember("snail"), agent},
                        no_oppo_limit = true,
                    }
                        :OnWin()
                            :Fn(function(cxt) 
                                if cxt.quest:GetCastMember('giver'):IsDead() then
                                    cxt:Dialog("DIALOG_GIVER_DEAD")
                                    cxt.quest:Fail()
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                else
                                    cxt:Dialog("DIALOG_WON_FIGHT")
                                    ConvoUtil.GiveQuestRewards(cxt)
                                    cxt.quest:Complete()
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end

                            end)

            
            end)
                
        end)

