local QDEF = QuestDef.Define{
        title = "Bounced Out",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        icon = engine.asset.Texture("icons/quests/side_smith_bounced_out.tex"),
        desc = [[{moreef}'s bouncer {bouncer} has been detained by the Admiralty for disorderly conduct. Get {bouncer.himher} out!]],
        focus = QUEST_FOCUS.BALANCED,
        tags = {"MOREEF_INTRO_JOB"},
        act_filter = "SMITH",
        on_destroy = function(quest)
            if quest:GetCastMember("bouncer") and quest:GetCastMember("bouncer"):IsInPlayerParty() then
                quest:GetCastMember("bouncer"):Dismiss()
            end

            if quest:GetCastMember("lawyer") and quest:GetCastMember("lawyer"):IsInPlayerParty() then
                quest:GetCastMember("lawyer"):Dismiss()
            end
        end,

        --this is a little bit of boilerplate that lets us test the quest directly from the quest menu
        on_debug_start = function(quest)
            TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("MOREEF_BAR"))
            UIHelpers.DoSpecificConvo( quest:GetCastMember("moreef"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
        end,

        is_in_hiding = function(quest, agent)
            return agent == quest:GetCastMember("enemy")
        end,
        
        collect_agent_locations = function(quest, t)
            table.insert(t, { agent = quest:GetCastMember("guard"), location = quest:GetCastMember("jail"), role = CHARACTER_ROLES.PROPRIETOR} )
        end,
    }



    :AddCastByAlias{
        cast_id = "moreef",
        alias = "SWEET_MOREEF",
        provider = true,
        no_validation = true,
    }


    :AddDefCastSpawn("bouncer", "JAKES_SMUGGLER")
    :AddDefCastSpawn("lawyer", "PEARLIE")
    :AddDefCastSpawn("guard", "ADMIRALTY_PATROL_LEADER")
    :AddDefCastSpawn("enemy", "ADMIRALTY_GOON")
    :AddDefCastSpawn("ruffian", "JAKES_SMUGGLER")
    :AddDefCastSpawn("forger", "FOREMAN")
    :AddDefCastSpawn("clerk", "ADMIRALTY_CLERK")


    :AddExistingLocation("jail", "PEARL_JAIL")

    :AddQuestLocation{
        cast_id = "ruffians",
        name = "The Easies' Hangout Spot",
        plax = "EXT_Ship_Graveyard",
        map_tags = {"cliffs"},

    }
    :AddExistingLocation("station", "PEARL_ADMIRALTY_STATION")
    
    :AddQuestLocation{
        cast_id = "forger_workshop",
        name = "Forger's Workshop",
        plax = "INT_SMITH_FORGERYWORKSHOP",
        map_tags = {"quarters"},
    }


    :AddObjective{
        id = "start",
        on_activate = function(quest) 
            quest:Activate("bouncer_in_jail")
            quest:Activate("talk_to_bouncer")
            quest:Complete('start')
        end,
    }

    :AddObjective{
        id = "talk_to_bouncer",
        title = "Talk to {bouncer}",
        desc = [[See why {bouncer} has been detained.]],
        mark = { "jail" },
        on_complete = function(quest)
            quest:Activate("enlist_ruffians")
            quest:Activate("talk_to_lawyer")
        end,
    }

    :AddObjective{
        id = "enlist_ruffians",
        title = "Enlist The Easies",
        desc = [[Enlist {bouncer}'s friends 'The Easies' to help free {bouncer.himher}.]],
        mark = { "ruffians" },
        on_complete = function(quest)
            if quest:IsActive("talk_to_lawyer") then
                quest:Cancel("talk_to_lawyer")
            end
            
            if quest:IsActive("get_forgery") then
                quest:Cancel("get_forgery")
            end

            if quest:IsActive("get_release_order") then
                quest:Cancel("get_release_order")
            end
        end,
    }

    :AddObjective{
        id = "talk_to_lawyer",
        title = "Talk to {lawyer}",
        desc = [[{bouncer} has a lawyer on call to get {bouncer.himher} out of jams just like the one {bouncer.heshe} is currently in.]],
        mark = { "lawyer" },

        collect_agent_locations = function(quest, t)
            table.insert(t, { agent = quest:GetCastMember("lawyer"), location = quest:GetCastMember("lawyer"):GetBrain():GetHome() })
        end,

    }

    :AddObjective{
        id = "get_forgery",
        title = "Talk to {forger}",
        desc = [[{forger} can make you a relatively convincing release order, for a price.]],
        mark = { "forger" },

        collect_agent_locations = function(quest, t)
            table.insert(t, { agent = quest:GetCastMember("forger"), location = quest:GetCastMember("forger_workshop") })
        end,
    }

    :AddObjective{
        id = "get_release_order",
        title = "Take {lawyer} to {clerk}",
        desc = [[{clerk} can be persuaded to produce a release order for {bouncer}.]],
        mark = { "clerk" },
        on_activate = function(quest)
            
            quest:Complete("talk_to_lawyer")
            
            if quest:IsActive("get_forgery") then
                quest:Cancel("get_forgery")
            end
            
            if quest:IsActive("enlist_ruffians") then
                quest:Cancel("enlist_ruffians")
            end

        end,

        collect_agent_locations = function(quest, t)
            table.insert(t, { agent = quest:GetCastMember("clerk"), location = quest:GetCastMember("station") })
        end,
    }

    
    :AddObjective{
        id = "bouncer_in_jail",
        title = "Free {bouncer} from the jail",
        --hide_in_overlay = true,
        collect_agent_locations = function(quest, t)
            table.insert(t, { agent = quest:GetCastMember("bouncer"), location = quest:GetCastMember("jail"), role = CHARACTER_ROLES.PRISONER} )
        end,
        mark = {"guard"},

        on_complete = function(quest)

            if quest:IsActive("enlist_ruffians") then
               quest:Cancel("enlist_ruffians") 
            end
            if quest:IsActive("talk_to_lawyer") then
               quest:Cancel("talk_to_lawyer") 
            end
            if quest:IsActive("get_forgery") then
               quest:Cancel("get_forgery") 
            end
            if quest:IsActive("get_release_order") then
               quest:Cancel("get_release_order") 
            end

            quest:GetCastMember("bouncer"):OpinionEvent(quest:GetQuestDef():GetOpinionEvent("got_out_of_jail"))
            quest:Activate("return_to_moreef")
        end,
    }


    :AddObjective{
        id = "return_to_moreef",
        title = "Take {bouncer} back to {moreef}",
        desc = [[{bouncer} is free! Tell {moreef} the good news!]],
        mark = { "moreef" },
        
        on_activate = function(quest)
            quest:GetCastMember("bouncer"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
            quest.param.enemy_pending = true
        end,

        on_complete = function(quest)
        end,
    }

    :AddObjective{
        id = "return_to_moreef_fail",
        title = "Tell {moreef} the bad news",
        desc = [[{moreef} will want to hear it from you.]],
        mark = { "moreef" },
        
        on_activate = function(quest)
            if quest:IsActive("return_to_moreef") then
                quest:Fail("return_to_moreef")
            end
            if quest:IsActive("bouncer_in_jail") then
                quest:Fail("bouncer_in_jail")
            end
            if quest:IsActive("get_forgery") then
                quest:Fail("get_forgery")
            end
            if quest:IsActive("enlist_ruffians") then
                quest:Fail("enlist_ruffians")
            end
        end,


        on_complete = function(quest)
        end,
    }

    :AddOpinionEvents{
    
        held_back_in_fight =  
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "You held back in the jail fight",
        },
        
        got_out_of_jail =  
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "Got them out of jail",
        },
        
        abandoned =  
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Abandoned them to their enemies",
        },
        
        killed_enemy =  
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Helped them kill their nemesis",
        },
        
        helped_bouncer =  
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Helped their nemesis get out of jail",
        },
    }



QDEF:AddIntro(
    [[
        
        player:
            I've been meaning to ask - why don't you have a bouncer?
        moreef:
            I have one. {bouncer.HisHer} name is {bouncer}. 
        player:
            Is {bouncer.heshe} tiny?
            Because I don't see {bouncer.himher} here.
        agent:
            {bouncer.HeShe}'s in an Admiralty cooling cell. Again.
            !shrug
            It happens pretty often - {bouncer.heshe}'s a bit of a hothead. 
        player:
            I appreciate an unbounced bar as much as the next ruffian, but that doesn't sound like it's very good for business.
        agent:
            Oh it's terrible! But what are you going to do?
        player:
            !thought
            Indeed...
    ]],
    [[
        player:
            I'll go bust {bouncer.himher} out. Sound good?
        agent:
            !dubious
            You're just going to walk in there and demand {bouncer}'s release?
        player:
            No, of course not! I'm way more subtle than that.
        agent:
            ...
        player:
            !shrug
            I'll come up with something on the walk over.

    ]],
    "start")


QDEF:AddConvo("talk_to_bouncer")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember('jail') end)
        :Loc{
            DIALOG_INTRO = [[
                * You ask to talk to {bouncer}.
                guard:
                    !right
                    You have 5 minutes. No more.
                    !exit
                bouncer:
                    !right
                player:
                    !left
                    Hope you talk fast, {bouncer}, 'cause Sweets sent me to get you outta here and I don't have a lot of ideas.
                bouncer:
                    Moreef sent you? For me?
                    Didn't realize he had that kinda clout.
                player:
                    Naw, he just has that kinda <i>lout</i>. 
                    Now talk to me.
                bouncer:
                    Look, it's all a misunderstanding. {enemy}'s had it out for me for years.
                    But I'm not stupid. I lawyered up before this went down. Go talk to {lawyer}, my lawyer, and maybe you two can crack me outta this.
                player:
                    Ugh, bureaucracy? One of the reasons why I left the Foam in the first place.
                bouncer:
                    Well, if you're that sort, there's also The Easies. 
                player:
                    The... what?
                bouncer:
                    My guys. The gang I roll with, when I'm not on the job at the Hideaway.
                    Look, that's our five minutes, and that's all I got. 
                    I'd wish you luck but... I'm gonna need it more than you, I think.
                player:
                    Hmm.
            ]],
            

            DIALOG_LEAVE = [[
                guard:
                    !right
                    Time's up. Say your goodbyes before {bouncer} disintegrates.
                bouncer:
                    !right
                    I'm countin' on you, {player}! Don't leave me hanging!
                    !exit
            ]]

        }
        :Questions("QUESTIONS", {
            "Ask what {bouncer} did to {enemy}",
            [[
                player:
                    What exactly did you do to land in this mess?
                bouncer:
                    Got in a fight. Knocked a few heads. Broke some stuff.
                player:
                    Ah, I know that tango. Pretty good at it myself.
                bouncer:
                    Yeah. Unfortunately, one of the heads I knocked is {enemy}'s, and {enemy.heshe} has better friends than I do.
                player:
                    Hrm. We'll see about that.
            ]],
            "Ask about {enemy}",
            [[
                player:
                    This {enemy} gonna be a problem for me if I'm trying to get you out?
                bouncer:
                    Probably. {enemy.HeShe}'s a problem just for breathin' the same air as the rest of us.
                    Be careful, because you're not much good to me or Moreef if you get thrown in here, too.
            ]],
            "Ask about the Easies",
            [[
                player:
                    Who are the Easies, again?
                agent:
                    !happy
                    They're my gang! We've been rolling together since we were kicked out of the creche.
                    We kinda go our own ways most of the time, what with me being career-minded and them being more, um, street.
                player:
                    Lemme guess. They're not so much "your gang" as the gang you wish you were part of?
                agent:
                    Well... I am a bit behind in my dues.
                    But I'm sure they'll still help me out!
            ]],

            "Ask about {lawyer}",
            [[
                player:
                    How's a shlub like you end up with a lawyer?
                bouncer:
                    Wow. Gonna choose not to take offense to that. 
                    But it's the Pearl. <i>Everyone's</i> got a lawyer.
                    And you can't skip a stone without hitting some newly licensed greenhorn.
                    Let's just say my guy was willing to work real cheap, so I could afford the retainer.
                player:
                    Oh yeah?
                    Real cheap doesn't usually mean real good.
                bouncer:
                    Thanks, that thought's real good for my morale.

            ]],
        })
        :Fn(function(cxt) 
            
            cxt.quest:GetCastMember("guard"):MoveToLocation(cxt.location)
            
            if not cxt.quest.param.guard_backup then
                cxt.quest.param.guard_backup = CreateCombatBackup(cxt.quest:GetCastMember("guard"), { "ADMIRALTY_GUARD", "ADMIRALTY_GUARD", "ADMIRALTY_GOON", "VROC" }, 1)
                table.insert(cxt.quest.param.guard_backup, 1, cxt.quest:GetCastMember("guard"))
            end

            cxt:TalkTo("bouncer")
            cxt:Dialog("DIALOG_INTRO")
            
            cxt.quest:Complete("talk_to_bouncer")

            cxt:RunLoop(function()

                local inserted = cxt:InsertQuestions("QUESTIONS")
                if inserted == 0 then
                    cxt:Dialog("DIALOG_LEAVE")
                    cxt:End()
                else
                    StateGraphUtil.AddEndOption(cxt)
                        :Dialog("DIALOG_LEAVE")
                end
            end)
        end)


QDEF:AddConvo("bouncer_in_jail", nil)
    :TravelConfront("MEET_CHALLENGER", function(cxt) return not cxt.quest.param.met_challenge and cxt.quest.param.have_order and cxt.quest:GetCastMember("lawyer"):IsInPlayerParty() end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !angry
                I'm here for {lawyer}! {lawyer.HeShe} was supposed to get me off without a charge, instead I'm indebted to the biggest dunce in the Foam.
                I want payback <i>now</i>. No way I'm working this off for the next three years.
            ]],

            OPT_THREATEN = "Scare {agent} away",
            DIALOG_THREATEN = [[
                player:
                    Ah, sod off. Unless you want some bruises to go with your bad luck.
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                player:
                    Seriously, you see these arms? You think these are just for show?
                agent:
                    Hrm.
                    !threaten
                    Next time, {lawyer}!
                    !exit

                lawyer:
                    !right
                    Thanks, grifter. Let's continue.
            ]],
            
            DIALOG_THREATEN_FAILURE = [[
                agent:
                    !threaten
                    No! I'm tired of being afraid!
            ]],

            OPT_DEFEND = "Defend {lawyer}",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    I don't have time for this crap!
            ]],
            DIALOG_WON_LAWYER_DEAD = [[
                agent:
                    !exit
                * {lawyer} is dead. You still have the order, though.
            ]],
            DIALOG_WON_LAWYER_ALIVE = [[
                lawyer:
                    !right
                    Thanks, grifter. Let's continue.
            ]],

            OPT_LEAVE_LAWYER = "Leave lawyer to {lawyer.hisher} fate",
            DIALOG_LEAVE_LAWYER = [[
                player:
                    Sorry, I'm going to continue on my own.
                    !exit
                agent:  
                    Ha!
                lawyer:
                    !left
                    !scared
                    Uhhh... can we talk?

            ]],
        }
        :Fn(function(cxt) 
            local baron = AgentUtil.GetFreeAgent("BANDIT_RAIDER", function(agent) return agent:GetRelationship() == RELATIONSHIP.NEUTRAL end)
            cxt:TalkTo(baron)
            cxt.quest.param.met_challenge = true
            
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 
                
                cxt:BasicNegotiation("THREATEN", {flags = NEGOTIATION_FLAGS.INTIMIDATION, helpers = {"lawyer"}})
                    :OnSuccess()
                        :Travel()

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE, allies = {"lawyer"}}
                        :OnWin()
                            :Fn(function() 
                                cxt:Dialog(cxt.quest:GetCastMember("lawyer"):IsDead() and "DIALOG_WON_LAWYER_DEAD" or "DIALOG_WON_LAWYER_ALIVE")
                            end)
                            :Travel()

                cxt:Opt("OPT_LEAVE_LAWYER")
                    :Dialog("DIALOG_LEAVE_LAWYER")
                    :ReceiveOpinion("abandoned", nil, "lawyer")
                    :Fn(function() 
                        cxt.quest:GetCastMember("lawyer"):Dismiss()
                    end)
                    :Travel()
            end)
        end)


-- this is if you convince the ruffians to show up.
QDEF:AddConvo("bouncer_in_jail")
    :ConfrontState("STATE", function(cxt) return cxt.quest.param.ruffians_joined and cxt.location == cxt.quest:GetCastMember("jail") end)
        :Loc{
            DIALOG_INTRO = [[
                * You enter the jail, followed by the Easies.
                ruffian:
                    !left
                    We're here for {bouncer}!
                guard:
                    !right
                    To arms!
            ]],
            OPT_FIGHT = "Join the attack",
            DIALOG_FIGHT = [[
                player:
                    !left
                    !fight
                    * You charge into the fray!
            ]],

            OPT_THREATEN = "Convince {guard} to run away",
            DIALOG_THREATEN = [[
                player:
                    !left
                    You're outnumbered, {guard}.
                    I suggest you run away.
            ]],
            
            DIALOG_THREATEN_SUCCESS = [[
                guard:
                    !scared
                    I'm not paid enough for this!
                    !exit
            ]],

            DIALOG_THREATEN_FAILURE = [[
                guard:
                    !angry
                    We're not afraid of your dumb gang, grifter.
            ]],

            OPT_WATCH_FIGHT = "Watch the fight play out",
            DIALOG_WATCH_FIGHT = [[
                * You hold back, waiting for a good opening.
                left:
                    !exit
                right:
                    !exit
                * Fists and worse fly, and eventually it's down to {ruffian} and {guard}.
                guard:
                    !right
                    !fight
                ruffian:
                    !left
                    !injured
                    What are you waiting for, grifter?!
            ]],
            OPT_JOIN_LATER = "Join the fight",
            DIALOG_JOIN_LATER = [[
                player:
                    !right
                    !fight
                    I'm here now.
            ]],
        }
        :Fn(function(cxt) 

            cxt:Dialog("DIALOG_INTRO")
            cxt:TalkTo('guard')
            
            cxt:RunLoop(function() 
                
                cxt:BasicNegotiation("THREATEN", {flags = NEGOTIATION_FLAGS.INTIMIDATION, helpers = {"bouncer", "ruffian"}})
                    :OnSuccess()
                        :GoTo("STATE_WON")

                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{
                        allies = cxt.quest.param.the_ruffians,
                        enemies = cxt.quest.param.guard_backup,
                        no_oppo_limit = true,
                    }
                        :OnWin()
                            :GoTo("STATE_WON")

                cxt:Opt("OPT_WATCH_FIGHT")
                    :Dialog("DIALOG_WATCH_FIGHT")
                    :ReceiveOpinion("held_back_in_fight", nil, "ruffian")
                    :Fn(function() 

                        cxt.quest:GetCastMember("ruffian"):GetAspect("health"):SetPercent( .33 )
                        cxt.quest:GetCastMember("guard"):GetAspect("health"):SetPercent( .5 )

                        cxt:Opt("OPT_JOIN_LATER")
                            :Dialog("DIALOG_JOIN_LATER")
                            :Battle{
                                flags = BATTLE_FLAGS.NO_BYSTANDERS,
                                allies = {"ruffian"},
                                enemies = {"guard"},
                            }
                                :OnWin()
                                    :GoTo("STATE_WON")
                    end)
            end)
        end)
    :State("STATE_WON")
        :Loc{
            DIALOG_RUFFIAN_DEAD = [[
                * {ruffian} is dead, and {bouncer} doesn't look like this was the win {bouncer.heshe} wanted.
                bouncer:
                    !right
                    O-oh. Aw, geez.
                    If I'd'a known this would be the cost...
                    Woulda just stayed in jail.
                player:
                    None of that. {ruffian} knew the risks.
                    You gotta get out there and make {ruffian.himher} proud.
                bouncer:
                    Yeah. Yeah, I guess.
                    I'll see you back at the bar, grifter.
                    !exit
            ]],
            DIALOG_RUFFIAN_ALIVE = [[
                * When the dust settles, {bouncer} hastily wipes some grime from {bouncer.hisher} eyes.
                ruffian:
                    !left
                    All right, {bouncer}? What's this I hear about you getting in hot water?
                    The Easies don't boil so easy, remember?
                bouncer:
                    !right
                    Don't go down, don't go over, don't go anywhere easy. Yeah, I remember.
                    To be honest, {ruffian}, I thought maybe you wouldn't come for me. On account of, you know.
                ruffian:
                    It was just a mistake, {bouncer}. The stains came out ages ago. 
                    No harm, no foul.
                    Your dues on the other hand... they're way, way over.
                bouncer:
                    Y-yeah. I guess I owe you big for this one, huh?
                ruffian:
                    You'll work it off. Especially if you're at the Hideaway.
                    Something tells me business is about to be booming, huh?
                player:
                    !left 
                    Alright, enough with the heart-to-heart. We drinking, or what?
                bouncer:
                    You got it, {player}! See you back at Sweet Moreef's.
                    !exit
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog(cxt.quest:GetCastMember("ruffian"):IsDead() and "DIALOG_RUFFIAN_DEAD" or "DIALOG_RUFFIAN_ALIVE")
            cxt.quest:Complete("bouncer_in_jail")
            StateGraphUtil.AddLeaveLocation(cxt) 
        end)



QDEF:AddConvo("bouncer_in_jail", "bouncer")
    :AttractState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                guard:
                    !right
                    No talking to the prisoners!
                    !exit
            ]]
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            StateGraphUtil.AddEndOption(cxt)
        end)


QDEF:AddConvo("bouncer_in_jail", "guard")
    :AttractState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    agent:
                        Why are you still here?
                }
                {not first_time?
                    agent:
                        You're starting to get on my nerves, grifter.
                }
            ]],
            
            OPT_BOUNCER = "Ask about {bouncer}",
            DIALOG_BOUNCER = [[
                * {guard} expresses their disdain for you, {bouncer}, the world, and pretty much everything in it.
                * It's quite charming.
            ]],

            OPT_ATTACK = "Free {bouncer} by force",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    Let {bouncer} out!
                agent:
                    You crazy?
                    This is an Admiralty building. Full of armed guards.
                    And you're... just a guy.
                player:
                    Yeah! A guy who's gonna break your face!
                agent:
                    !shrug
                    Your funeral.
                    !fight
            ]],

            DIALOG_WON = [[
                agent:
                    !exit
                player:
                    !happy
                * You unclip the ostentatiously large keyring from the downed guard's belt and open {bouncer}'s cell with a flourish.
                player:
                    Ta da.
                bouncer:
                    !right
                    Holy Hesh, did you just do that?!
                    You took down a whole platoon!
                player: 
                    Sure did!
                bouncer:
                    W-we gotta get outta here! I-I can't get caught up in this.
                    You don't know me! We never met!
                    !exit
                player:
                    ...
                player:
                    You're welcome!
            ]],

            
            OPT_FORGED_ORDER = "Show {agent} the forged release order",
            DIALOG_FORGED_ORDER = [[
                player:
                    !give
                    I have a completely legitimate release order for {bouncer}.
                agent:
                    Is that so?
            ]],
            DIALOG_FORGED_ORDER_SUCCESS = [[
                player:
                    You'll know it's real on account of the glossy paper. And the very red, red wax on the seal.
                agent:
                    !dubious
                    Uh-huh. 
                    Orders are orders, I guess.
                    You're free to go, {bouncer}.
                bouncer:
                    !right
                    Thanks, {player}!
                    Feels good to get the wind on my face again.
                player:
                    We ain't even outside yet.
                bouncer:
                    I meant metaphorically. 
                    Let's go back to the bar!
                    !exit
            ]],
            DIALOG_FORGED_ORDER_FAILURE = [[
                player:
                    Pretty sure all the words are spelled right, but don't look too close, just in case.
                agent:  
                    Sure, that's what all the forgers say.
                    You trying to pull a jailbreak, grifter? Or are you just stupid?
            ]],

            OPT_ORDER = "Show {agent} the release order",
            DIALOG_ORDER = [[
                player:
                    I have an order from your commanding officer!
                    !give
                agent:
                    From my--are you serious? Let me see that!
            ]],
            DIALOG_ORDER_SUCCESS = [[
                player:
                    Straight from the top, officer. Or do you need to run it up the flagpole, first?
                agent:
                    Hesh-damned paperwork.
                    You're free to go, {bouncer}!
                bouncer:
                    !right
                    Thanks, {player}!
                    Feels good to get the wind on my face again.
                player:
                    We ain't even outside yet.
                bouncer:
                    I meant metaphorically. 
                    Let's get back to the bar!
                    !exit
            ]],
            DIALOG_ORDER_FAILURE = [[
                player:
                    Yessiree, all very official and legal, a thing I know all about.
                agent:
                    Hah. Bet you don't even know what this says, not really.
                    Thing is? It's a bunch of jargon to me.
                    And I don't much feel smart enough to understand it all. 
                    Meaning {bouncer} stays.
            ]],

            OPT_LAWYER = "Let {lawyer} do {lawyer.hisher} thing",
            DIALOG_LAWYER = [[
                lawyer:
                    !left
                    I demand you release my client immediately!
                    I have an order from your superiors.
                    !give
                guard:
                    Oh, not you again.
                    Fine. Take {bouncer.himher}. I'm tired of staring at {bouncer.hisher} ugly face, anyway.
                    !exit
                * {guard} begrudgingly opens the cell and lets {bouncer} out.
                bouncer:
                    !right
                    Thanks, {lawyer}!
                lawyer:
                    Don't thank me, just stop getting yourself into such stupid scrapes.
                bouncer:
                    Awww, but then you'd have nothing to do!
                player:
                    !left
                bouncer:
                    Let's go back to the bar!
                    !exit
            ]],

            OPT_NO_ORDER = "Demand {agent} release {bouncer}",
            DIALOG_NO_ORDER = [[
                player:
                    I demand you release {bouncer} immediately!
                agent:
                    And who are you, making such demands, huh?
            ]],
            DIALOG_NO_ORDER_SUCCESS = [[
                player:
                    I'm Smith Banquod, spawn of a line tracing back all the way to the Deltrean Tree. 
                    Progeny of the root that was, vessel of the tentacle that is. 
                    My great, great grandfather was second only to the emperor. My great aunt was an Admiral on the Rentorian frontlines.
                    You want pedigree? I have it coming out the wazoo.
                agent:
                    Alright, alright. Geez.
                    You didn't mention you were going to bore me to death.
                    {bouncer}'s just a rotten hoodlum. You want {bouncer.himher} so bad, take {bouncer.himher}.
                    But if I see either of you back in here again, I'll stop up my ears with wax and leave you both to rot.
                    !exit
                * {guard} begrudgingly opens the cell and lets {bouncer} out.
                bouncer:
                    !right
                    Wow, that was some fancy talking!
                    You should consider going into law!
                player:
                    But I just spent all that time getting <i>out</i> of it.
                    Come on. Let's go back to the bar, huh? Tell Sweets what we did.
                    !exit
            ]],
            DIALOG_NO_ORDER_FAILURE = [[
                player:
                    Uh, 'cause I asked, and my family's rich and important?
                agent:
                    Best thing about you rich brats is that every single one of you thinks you're the eponymous Pearl.
                    But all you are is a bubble in the foam.
                player:
                    ...
                agent:
                    That's a <i>no</i>.

            ]],

            SIT_MOD_NO_ORDER = "Why would {guard} even consider that request?",
            SIT_MOD_LAWYER = "{lawyer} is present",
            SIT_MOD_FORGERY = "The release order is a forgery",

            OPT_ABANDON_BOUNCER = "Give up on freeing {bouncer}",
            DIALOG_ABANDON_BOUNCER = [[
                player:
                    You know what?
                    You're on your own, {bouncer}.
                    I can't help you.
                    !exit
            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:RunLoop(function()
                
                cxt:QST("BOUNCER")


                if cxt.quest.param.have_order then

                    cxt.enc.scratch.lawyer_here = cxt.quest:GetCastMember('lawyer'):IsInPlayerParty()
                    
                    if cxt.enc.scratch.lawyer_here then
                        cxt:Opt("OPT_LAWYER")
                            :SetQuestMark()
                            :Dialog("DIALOG_LAWYER")
                            :CompleteQuest("bouncer_in_jail")
                            :Fn(function() cxt.quest:GetCastMember("lawyer"):Dismiss() end)
                            :Travel()
                    else
                        cxt:BasicNegotiation("ORDER", {situation_modifiers = { { value = -10, text = cxt:GetLocString("SIT_MOD_LAWYER")} }, helpers = {"bouncer"} })
                            :SetQuestMark()
                            :OnSuccess()
                                :CompleteQuest("bouncer_in_jail")
                                :Travel()
                    end
                elseif cxt.quest.param.have_forged_order then

                    cxt:BasicNegotiation("FORGED_ORDER", {situation_modifiers = { { value = 15, text = cxt:GetLocString("SIT_MOD_FORGERY") } }, helpers = {"bouncer"} })
                        :SetQuestMark()
                        :OnSuccess()
                            :CompleteQuest("bouncer_in_jail")
                            :Travel()

                else
                    cxt:BasicNegotiation("NO_ORDER", {situation_modifiers = { { value = 50, text = cxt:GetLocString("SIT_MOD_NO_ORDER") } }, helpers = {"bouncer"} })
                        :SetQuestMark()
                        :OnSuccess()
                            :CompleteQuest("bouncer_in_jail")
                            :Travel()
                end


                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{
                        noncombatants = {"bouncer"},
                        enemies = cxt.quest.param.guard_backup,
                        no_oppo_limit = true,
                    }
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :CompleteQuest("bouncer_in_jail")
                            :Fn(function() 
                                cxt.quest.param.fought_law = true
                                cxt.quest:Complete("bouncer_in_jail")
                            end)
                            :Travel()
                

                cxt:Opt("OPT_ABANDON_BOUNCER")
                    :Dialog("DIALOG_ABANDON_BOUNCER")
                    :Fn(function() 
                        cxt.quest:Activate("return_to_moreef_fail")
                    end)
                    :Travel()


                StateGraphUtil.AddEndOption(cxt)

            end)
        end)        


QDEF:AddConvo("talk_to_lawyer", "lawyer")
    :ConfrontState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        Hey, you the lawyer? I got one of your clients, {bouncer}, who needs your help?
                    agent:
                        !sigh
                        What has {bouncer.heshe} done this time?
                    player:
                        Sounds to me like it's just a wrong place, wrong time situation.
                        Got in trouble with {enemy} and needs to cool off in an Admiralty cell.
                    agent:
                        Well, I'm afraid {enemy}'s lawyer is a bit more, ah, accredited than I am.
                        If we're going to get {bouncer} out we'll need an order of pardon. 
                    player:
                        Ohhh, I know those. My parents used to have them sent express to me all the time.
                    agent:
                        I... see. Well, I know an Admiralty clerk who owes me a favor. 
                        This is a bit of a tall order, but I could potentially call it in with this.
                        Alternatively... and I dare say this might be a route more familiar to you...
                    player:
                        Forgery?
                    agent:
                        You didn't hear it from me. And we never had this conversation.
                }
                {not first_time?
                    agent:
                        So what's it going to be? Are we going to get that release order?
                }
            ]],
            OPT_CLERK = "Ask about the clerk",
            DIALOG_CLERK = [[
                player:
                    Who's this clerk, then?
                agent:
                    Someone who has themselves needed to consult the boundaries of the law.
                    That's all you need to know--client confidentiality and all.
                player:
                    Ah, dirty cop. I get ya.
            ]],
            OPT_FORGERY = "Ask about the forgery",
            DIALOG_FORGERY = [[
                player:
                    Say I <i>didn't</i> want a forgery. Who would I definitely not want to talk to in order to avoid, uh, accidentally getting one?
                agent:
                    !sigh
                    These days it seems you can't trip without "accidentally" procuring a forgery.
                    But I know a place, and I will happily mark it out for you so that you can <i>avoid</i> it.
            ]],


            OPT_HEAD_TO_CLERK = "Take {agent} to {clerk} to get a release order",
            DIALOG_HEAD_TO_CLERK = [[
                player:
                    Well, since you're on the level, we may as well go all the way.
                    Let's go talk to this clerk you know, huh?
                agent:
                    !angry_shrug
                    Sure, I'll just <i>clear my afternoon</i>.
                player:
                    Great!
                agent:
                    That was sarcasm, but since you seem to be immune--
                player:
                    Not immune! Just indifferent. Lead the way!
                    !exit
            ]]

        }
        :Fn(function(cxt) 
            

            cxt:Dialog("DIALOG_INTRO")
            if cxt.quest:IsInactive("get_forgery") then
                cxt.quest:Activate("get_forgery")
            end
            
            cxt:RunLoop(function() 
                cxt:QST("CLERK")
                cxt:QST("FORGERY")

                cxt:Opt("OPT_HEAD_TO_CLERK")
                    :Dialog("DIALOG_HEAD_TO_CLERK")
                    :SetQuestMark()
                    :Fn(function() 
                        cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                        cxt.quest:Activate("get_release_order")
                    end)
                    :Travel()

                StateGraphUtil.AddLeaveLocation(cxt) 
                
            end)
        end)



QDEF:AddConvo("get_release_order", "clerk")
    :Loc{
        OPT_GET_ORDER = "Ask for a release order",
        DIALOG_GET_ORDER = [[
            player:
                My friend and I need a release order for {bouncer}...
        ]],
        DIALOG_GET_ORDER_SUCCESS = [[
            agent:
                Fine. I'll do it. But only because {lawyer} asked so nicely.
                !give
            lawyer:
                !right
                Now, we have to go present this to the guards at the jail. 
                Should be easy!
        ]],
        DIALOG_GET_ORDER_FAILURE = [[
            agent:
                No, I'm not sticking my neck out for you.
                !exit
            lawyer:
                !right
                Well, that's it for me. You're going to have to try something else to get {bouncer} out.
                !exit
        ]],
    }
    :Hub(function(cxt) 
        cxt:BasicNegotiation("GET_ORDER", {helpers = {"lawyer"}} )
            :OnSuccess()
                :CompleteQuest("get_release_order")
                :Fn(function() 
                    cxt.quest.param.have_order = true
                end)
                :DoneConvo()

            :OnFailure()
                :Fn(function() 
                    cxt.quest:GetCastMember("lawyer"):Dismiss()
                    cxt.quest:Fail("get_release_order")
                    
                    if not cxt.quest:IsFailed("get_forgery") then
                        cxt.quest:Activate("get_forgery")
                    end

                    if not cxt.quest:IsFailed("enlist_ruffians") then
                        cxt.quest:Activate("enlist_ruffians")
                    end

                end)
                :DoneConvo()
    end)


QDEF:AddConvo("get_forgery", "forger")
    :Loc{
        OPT_GET_ORDER = "Ask about release orders...",
        DIALOG_GET_ORDER = [[
            {first_time?
                player:
                    I need a release order.
                agent:
                    Yeah, I can do that.
                    It's not cheap, though.
            }
            {not first_time?
                player:
                    So, about the release order...
            }
        ]],

        OPT_PAY = "Pay for the order",
        DIALOG_PAY = [[
            player:
                !give
                Better be good, then, if I get what I pay for.
            forger:
                I've never had any complaints.
        ]],

        DIALOG_DO_WORK = [[
            forger:
                !neutral_notepad
            * {forger} gets to work.
            forger:
                !give
                And this is yours.
            player:
                !thought
                That doesn't take very long to do, does it?
            forger:
                No, but learning <i>how</> to do it takes a lifetime.
        ]],

        OPT_FREE_ORDER = "Use {bouncer}'s credit",
        DIALOG_FREE_ORDER = [[
            player:
                {bouncer} will pay you back when {bouncer.heshe} is out.
                I promise!
        ]],
        DIALOG_FREE_ORDER_SUCCESS = [[
            agent:
                I don't usually do spec work, but fine.
        ]],
        DIALOG_FREE_ORDER_FAILURE = [[
            agent:
                No money, no papers.
                That's my policy.
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_ORDER")
            :Dialog("DIALOG_GET_ORDER")
            :SetQuestMark( cxt.quest )
            :IsHubOption(true)
            :LoopingFn(function() 
                
                cxt:Opt("OPT_PAY")
                    :Dialog("DIALOG_PAY")
                    :DeliverMoney(50)
                    :Dialog("DIALOG_DO_WORK")
                    :CompleteQuest("get_forgery")
                    :Fn(function() 
                        cxt.quest.param.have_forged_order = true
                    end)
                    :DoneConvo()
                
                cxt:BasicNegotiation("FREE_ORDER")
                    :OnSuccess()
                        :Dialog("DIALOG_DO_WORK")
                        :CompleteQuest("get_forgery")
                        :Fn(function() 
                            cxt.quest.param.have_forged_order = true
                        end)
                        :DoneConvo()
                StateGraphUtil.AddBackButton(cxt)
            end)
    end)    


QDEF:AddConvo("enlist_ruffians")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("ruffians") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                ruffian:
                    !right
                {first_time?
                    * It's easy enough to find the gang {bouncer} was talking about. They look like the kind of scraps you used to boss around when you were younger.
                    * Confident, you walk up to them like you own the place.
                player:
                    Hey, you the Easies? Funny thing, this used to be <i>my</i> turf when I was a spawn.
                    I like what you've done with the place.
                ruffian:
                    Yeah? Seems to me like you're trying to ingratiate yourself with us by acting like you're our superior.
                    Thing is, we gravitate towards this life because we hate all authority.
                    And that includes washed-up has-beens like you acting like you're still relevant.
                player:
                    Wow. <i>Wow</i>. Ouch.
                    I'm, uh, here for {bouncer}, if that makes a difference.
                ruffian:
                    Maybe it does, maybe it doesn't. 
                }

                {not first_time?
                player:
                    So, about {bouncer}...
                }
            ]],

            OPT_RUFFIANS = "Ask about the gang",
            DIALOG_RUFFIANS = [[
                player:
                    So you call yourself the "Easies", huh? What's that for?
                agent:
                    'Cause we take what we want, and it's always easy.
                player:
                    Right. And not because you're... an easy target?
                agent:
                    No. 
                player:
                    'Cause that's maybe how I would interpret it.
                agent:
                    You'd be wrong.
                    And stupid. 

            ]],

            OPT_BOUNCER = "Ask about {bouncer}",
            DIALOG_BOUNCER = [[
                player:
                    Look, {bouncer} sent me to ask you for help...
                    Any reason why that might be?
                agent:
                    {bouncer.HeShe}'s one of us, isn't {bouncer.heshe}?
                player:
                    Well, that checks out, I guess.
                    Since {bouncer} was arrested... pretty easily.
            ]],

            OPT_JOIN = "Ask about joining the gang",
            DIALOG_JOIN = [[
                player:
                    Can I be an Easy, too?
                agent:
                    !crossed
                    ...
                player:
                    No?
                agent:
                    No.
            ]],

            OPT_FIGHT = "Enlist the Easies to free {bouncer}",
            DIALOG_FIGHT = [[
                player:
                    Your comrade-in-arms is rotting away in a jail cell, and you're just sitting here?
                    That's not very punk of you. What do the Easies even stand for, huh?
            ]],
            DIALOG_FIGHT_SUCCESS = [[
                player:
                    I could go back and tell {bouncer} that the Easies aren't up to the task, though it would break {bouncer.hisher} little heart.
                agent:
                    Stop, stop. You're embarrassing yourself with how ignorant you sound.
                    Of course we're gonna help {bouncer}.
                    Now if you're not a total has-been, you'll join us, yeah?
                    Meet us at the jail!
                    !exit
            ]],
            DIALOG_FIGHT_FAILURE = [[
                player:
                    You gotta help your own, right?
                agent:
                    {bouncer} never apologized for ruining my favorite shirt.
                    And {bouncer.hisher} dues aren't even paid up. We had to foreclose on the clubhouse.
                    So... no. 
                    Maybe this'll be a good lesson for {bouncer}, and for you.
                player:
                    Man, we just met.
                agent:
                    And yet the contempt comes so easy!
            ]],

            OPT_PAY_OFF = "Pay off the gang",
            DIALOG_PAY_OFF = [[
                player:
                    Look, I have shills. And if one thing never changes, it's that money moves mountains.
                    !give
                agent:
                    It doesn't even need to move mountains.
                    Just us.
                    And that it does.
                    Let's roll!
                    !exit
            ]],

        }
        :Fn(function(cxt) 

            cxt:TalkTo("ruffian")
            if not cxt.quest.param.the_ruffians then
                cxt.quest.param.the_ruffians = CreateCombatBackup(cxt.quest:GetCastMember("ruffian"), { "JAKES_RUNNER", "JAKES_LIFTER" }, 1)
                table.insert(cxt.quest.param.the_ruffians, 1, cxt.quest:GetCastMember("ruffian"))
            end
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop( function(cxt) 

                cxt:QST("RUFFIANS")
                cxt:QST("BOUNCER")
                cxt:QST("JOIN")

                cxt:Opt("OPT_PAY_OFF")
                    :Dialog("DIALOG_PAY_OFF")
                    :DeliverMoney(100)
                    :CompleteQuest("enlist_ruffians")
                    :Fn(function() cxt.quest.param.ruffians_joined = true end)
                    :Travel()

                cxt:BasicNegotiation("FIGHT")
                    :OnSuccess()
                        :CompleteQuest("enlist_ruffians")
                        :Fn(function() cxt.quest.param.ruffians_joined = true end)
                        :Travel()
                    :OnFailure()
                        :FailQuest("enlist_ruffians")
                        :Travel()

                StateGraphUtil.AddLeaveLocation(cxt) 


            end)

        end)


QDEF:AddConvo("return_to_moreef", nil)
    :TravelConfront("MEET_CHALLENGER", function(cxt) return cxt.quest.param.enemy_pending and cxt.quest:GetCastMember("bouncer"):IsInPlayerParty() end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    Hold up. Someone's following us.
                enemy:
                    !right
                    !angry
                    I see you've got a guardian angel, now, {bouncer}?
                bouncer:
                    !left
                    So what if I do?
                enemy:
                    Just know that I'm not going anywhere.
                    !cruel
                    I'm going to get you back eventually.
                player:
                    !left
                enemy:
                    I don't know you, but you're on my list now, too, grifter.
                    !threaten
                    Watch your step!
                    !exit
                * {enemy} turns to leave.
                bouncer:
                    !right
                    Are we going to let {enemy} talk to us that way?!
            ]],

            OPT_ATTACK = "Attack {enemy}",
            DIALOG_ATTACK = [[
                player:
                    No, I am not.
                bouncer:
                    !exit
                player:
                    Hey!
                enemy:
                    !right
                    !angry
                    What?
                player:
                    !fight
                    This!
            ]],
            DIALOG_WON_BOUNCER_DEAD = [[
                {dead?
                    * Well, that was messy.
                    !exit
                }
                
                {not dead?
                    enemy:
                        !injured
                        I'm glad {bouncer} is dead. 
                        I just hope that you're next!
                        !exit
                }
            ]],
            DIALOG_WON_BOUNCER_ALIVE = [[
                {dead?
                    bouncer:
                        !right
                        Wow, we did it!
                    player:
                        !neutral
                        Yeah, I guess we did.
                        I think you owe me one for that.
                    bouncer:
                        More like two, friend.
                        Let's go.
                }
                {not dead?
                    enemy:
                        !injured
                        I'll get you next time!
                        !exit
                    * {enemy} limps off.
                    bouncer:
                        !right
                    player:
                        !neutral
                        Is {enemy.heshe} always so pleasant?
                    bouncer:
                        You don't know the half of it.
                        Let's go.
                }
            ]],

            OPT_LET_GO = "Let {enemy} go",
            DIALOG_LET_GO = [[
                player:
                    Haven't you had enough action yet for one day?
                bouncer:
                    !disappoint
                    This is going to come back to bite us.
                    !exit
            ]],

        }
        :Fn(function(cxt) 
            cxt.quest.param.enemy_pending = false
            cxt:TalkTo("enemy")
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opinion("helped_bouncer", "enemy")


            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{allies = {"bouncer"}}
                    :OnWin()
                        :Fn(function() 
                            if cxt.quest:GetCastMember("bouncer"):IsDead() then
                                cxt:Dialog("DIALOG_WON_BOUNCER_DEAD")
                                cxt.quest:Fail("return_to_moreef")
                                cxt.quest:Activate("return_to_moreef_fail")
                            else
                                cxt:Dialog("DIALOG_WON_BOUNCER_ALIVE")
                                if cxt:GetCastMember("enemy"):IsDead() then
                                    cxt:Opinion("killed_enemy", "bouncer")
                                end
                            end
                        end)
                        :Travel()

                        

            cxt:Opt("OPT_LET_GO")
                :Dialog("DIALOG_LET_GO")
                :Travel()

        end)






QDEF:AddConvo("return_to_moreef", "moreef")
    :Loc{

        OPT_TELL_MOREEF = "Tell {moreef} that {bouncer} is free",
        DIALOG_TELL_MOREEF = [[
            * You return {bouncer} to {moreef}.
            bouncer:
                !right
                Guess it's back to work, huh? I'll get punched in.
            agent:
                !right
                Hey, {bouncer} doesn't look too bad!
                To be honest I kinda wasn't expecting to ever see {bouncer.himher} again.
            player:
                Sometimes I surprise myself, even. So I won't hold it against you, Sweets.
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_TELL_MOREEF")
            :SetQuestMark()
            :Dialog("DIALOG_TELL_MOREEF")
            :CompleteQuest()
            :Fn(function() 
                ConvoUtil.GiveQuestRewards(cxt)
                AgentUtil.TakeJob(cxt.quest:GetCastMember("bouncer"), cxt.location, "bouncer")
                cxt.quest:GetCastMember("bouncer"):GetBrain():SendToWork()
            end)
    end)


QDEF:AddConvo("return_to_moreef_fail", "moreef")
    :Loc{
        OPT_TELL_MOREEF_DEAD = "Tell {moreef} that {bouncer} is dead",
        DIALOG_TELL_MOREEF_DEAD = [[
            player:
                Sorry, Sweets. {bouncer} just had too many close calls.
                One of them had to claim {bouncer.himher} sooner or later.
            agent:
                Ah, typical. That's just my luck these days, hm?
                Still, at least I don't have to pay out {bouncer}'s last two weeks.
                !sigh
                It's so hard to find good help.
        ]],
        OPT_TELL_MOREEF = "Tell {moreef} that you can't get {bouncer} out",
        
        DIALOG_TELL_MOREEF = [[
            player:
                {bouncer}'s locked up tighter than my old gran's jaw. 
                I'm good, but I'm not a miracle worker.
            agent:
                Ah, typical. That's just my luck these days, hm?
                Still, at least I don't have to pay out {bouncer}'s last two weeks.
                !sigh
                It's so hard to find good help.
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt(cxt.quest:GetCastMember("bouncer"):IsDead() and "OPT_TELL_MOREEF_DEAD" or "OPT_TELL_MOREEF")
            :SetQuestMark()
            :Dialog(cxt.quest:GetCastMember("bouncer"):IsDead() and "DIALOG_TELL_MOREEF_DEAD" or "DIALOG_TELL_MOREEF")
            :FailQuest()
    end)    

QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}
