local QDEF = QuestDef.Define{
        title = "Delicate Negotiations",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{andwanette} has a long-standing rivalry with {rival}. {rival} is doing business with both the Barons and the Cult, and will lose both contracts if {rival.hisher} partners find out. Make that happen.]],
        icon = engine.asset.Texture("icons/quests/side_smith_delicate_negotiations.tex"),
        focus = QUEST_FOCUS.NEGOTIATION,
        tags = {"ANDWANETTE_JOB"},
        act_filter = "SMITH",

        --this is a little bit of boilerplate that lets us test the quest directly from the quest menu
        on_debug_start = function(quest)
            TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("PEARL_FANCY_EATS"))
            UIHelpers.DoSpecificConvo( quest:GetCastMember("andwanette"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
        end,

        collect_agent_locations = function(quest, t)
            --table.insert (t, { agent = quest:GetCastMember("assassin"), location = quest:IsActive("meet_assassin_post_kill") and quest:GetCastMember("cliffside") or quest:GetCastMember("park"), role = CHARACTER_ROLES.VISITOR} )
        end,

        avoids = {
            {"andwanette", "priest"},
            {"andwanette", "baron"},
            {"rival", "priest"},
            {"rival", "baron"},
            {"priest", "baron"},
        },

        on_destroy = function(quest) 
            if quest:GetCastMember("rival") and quest:GetCastMember("rival"):GetParty()  then
                quest:GetCastMember("rival"):Dismiss()
            end
        end,

        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("rival") then
                fighter:AddCondition("WANTED_ALIVE")
            end
        end,

    }

    :AddCastByAlias{
        cast_id = "andwanette",
        alias = "ANDWANETTE",
        provider = true,
        no_validation = true,
    }


    :AddDefCastSpawn("rival", "WEALTHY_MERCHANT")
    :AddDefCastSpawn("priest", "PRIEST")
    :AddDefCastSpawn("baron", "SPARK_BARON_TASKMASTER")


    :AddExistingLocation("meeting_place", "PEARL_FANCY_EATS")
    
    :AddObjective{
        id = "start",
        hide_in_overlay = true,
        
        on_activate = function(quest) 
            quest:Activate("arrange_meeting_with_priest")
            quest:Activate("arrange_meeting_with_baron")
        end,

        events = {
            quests_changed = function(quest, event_quest) 
                if event_quest == quest then
                    if quest:IsComplete("arrange_meeting_with_priest") and quest:IsComplete("arrange_meeting_with_baron") then
                        quest:Complete('start')
                    end
                end
            end           
        },
        
        on_complete = function(quest)
            quest:Activate("collect_rival")
        end,

        is_in_hiding = function(quest, agent)
            if agent == quest:GetCastMember("rival") then
                return true
            end
        end,

    }

    :AddObjective{
        id = "arrange_meeting_with_priest",
        title = "Invite {priest} to the meeting",
        desc = "Convince {priest} that {rival} needs to talk to them.",
        mark = {"priest"},
        negotiation_targets = {"priest"},
    }
    :AddObjective{
        id = "arrange_meeting_with_baron",
        title = "Invite {baron} to the meeting",
        desc = "Convince {baron} that {rival} needs to talk to them.",
        mark = {"baron"},
        negotiation_targets = {"baron"},
    }

    :AddObjective{
        id = "collect_rival",
        title = "Convince {rival}",
        desc = "Convince {rival} that {andwanette} wants to meet.",
        mark = {"rival"},

        on_complete = function(quest) 
            quest:Activate("escort_rival_to_meeting")
        end,
        
        collect_agent_locations = function(quest, t)
            local rival = quest:GetCastMember("rival")
            if rival and not rival:IsRetired() then
                table.insert (t, { agent = rival, location = rival:GetBrain():GetHome() } )
            end
            table.insert (t, { agent = quest:GetCastMember("baron"), location = quest:GetCastMember("meeting_place") } )        
            table.insert (t, { agent = quest:GetCastMember("priest"), location = quest:GetCastMember("meeting_place") } )        
        end,
    }

    :AddObjective{
        id = "escort_rival_to_meeting",
        title = "Bring {rival} to the meeting",
        desc = "Get {rival} to the meeting, and watch the fireworks.",
        mark = {"meeting_place"},
        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("baron"), location = quest:GetCastMember("meeting_place") } )        
            table.insert (t, { agent = quest:GetCastMember("priest"), location = quest:GetCastMember("meeting_place") } )        
        end,

        on_complete = function(quest)
            quest:Activate("return_to_andwanette")
        end,
    }

    :AddObjective{
        id = "rival_died",
        title = "Tell {andwanette} that {rival} is dead",
        desc = "You let {rival} die, robbing {andwanette} of {andwanette.hisher} revenge. Try to make it up to {andwanette.himher}.",
        mark = {"andwanette"},
    }

    :AddObjective{
        id = "return_to_andwanette",
        title = "Return to {andwanette}",
        desc = "Tell {andwanette} what happened with {andwanette.hisher} rival.",
        mark = {"andwanette"},
        on_activate = function(quest) 
            quest.param.confront_pending = true
        end,
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------

    :AddOpinionEvents{
        picked_side = {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Helped them fight their enemies",
        },
        raised_suspicion = 
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Raised their suspicion",
        },
        saved_from_bandits = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Saved them from bandits",
        },
        betrayed = 
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "They know you set them up",
        },
        suspicious = 
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "They suspect you set them up",
        }
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddIntro(
    --attract spiel
    [[

        andwanette:
            Now you <i>just</i> caught me in the middle of completely destroying my rival's life.
            I'm committed to your cause against your brother, of course, and I intend to see to that first.
            But it would be the sweetest wedding present to me if you could follow up.
        player:
            What kinda destruction we talkin' here? Like a hit?
            I'm not the best at hits. More of an open brawler, really.
        andwanette:
            No, no! This is your standard sabotage.
            You see, my nemesis, {rival}, has trade channels open with both the Barons and the Heshians simultaneously.
            {rival.HeShe} has built a house of cards and I simply can't resist giving it a good knock.
        player:
            You got something in mind?
        andwanette:
            The Cult doesn't know about {rival}'s double dipping, and will surely be very upset when they find out.
            Meanwhile, I know that the Barons are <i>only</i> interested in trading with {rival} <i>because</i> of {rival.hisher} Cult connections.
        player:
            Huh. Seems like all it needs is a little nudge, yeah?
        andwanette:
            Precisely! Just a teensy-weensy little nudge. Practically a puff of air.
        player:
            Well, I'm full of air, at least.
        andwanette: 
            Quite.
    ]],
    
    --on accept
    [[
        andwanette:
            Now there isn't much point to a debacle unless it's done entirely in public.
            So I've made a reservation for two tables at a popular bistro here in the Pearl.
            Invite both of {rival}'s contacts for lunch--and {rival} too, of course.
    ]],
    "start")


QDEF:AddConvo("arrange_meeting_with_priest", "priest")
    :Loc{
        OPT_TELL_MEETING = "Tell {agent} about the meeting...",
        DIALOG_TELL_MEETING = [[
            {first_time?
                player:
                    'Scuse me, your brininess. I'm a representative of {rival}.
                    {rival.HeShe} was wondering if you had time to discuss some trade.
                agent:
                    Excuse you, indeed. {rival} knows very well that I summon {rival.himher} at my convenience, not the other way around.
                    Why should I acquiesce to this audacity?
            }
            
            {not first_time?
                player:
                    Just checkin' in. You getting hungry yet?
                agent:
                    Not enough to submit so easily to {rival}'s impertinence.
            }

        ]],
        OPT_CONVINCE = "Convince {agent} it's important",
        DIALOG_CONVINCE = [[
            player:
                Oh, {rival} completely understands that it's a massive disrespect.
                That's why {rival.heshe}'s made it worth your while.
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                An exclusive reservation at the Bistro Rentori awaits you for your troubles.
            agent:
                Hm, foreign food, is it?
                I'm not a fan, but I understand that the restaurant is extremely exclusive.
                So tell {rival} I will come along shortly. At <i>my</i> leisure, of course.
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            player:
                You like Rentorian grub? 'Cause {rival}'s gonna treat you to lunch if you want.
            agent:
                I prefer my food quivering, the Deltrean way, as {rival} well knows.
                !sigh 
                Very well. Tell {rival} I'll be along shortly, but I will be bringing my personal entourage, including my personal chef.
        ]],
        DIALOG_BACK = [[
            player:
                Hold that thought, your brininess. I'll be right back.
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_TELL_MEETING")
            :Dialog("DIALOG_TELL_MEETING")
            :Fn(function() 
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("arrange_meeting_with_priest")
                    :OnFailure()
                        :Fn(function() cxt.quest.param.priest_backup = true end)
                        :ReceiveOpinion("raised_suspicion")
                        :CompleteQuest("arrange_meeting_with_priest")

                StateGraphUtil.AddBackButton(cxt)
                    :Dialog("DIALOG_BACK")
            end)
    end)

QDEF:AddConvo("arrange_meeting_with_baron", "baron")
    :Loc{
        OPT_TELL_MEETING = "Tell {agent} about the meeting...",
        DIALOG_TELL_MEETING = [[
            {first_time?
                player:
                    Hope I haven't kept you waiting. {rival} sent me to arrange your next meeting.
                agent:
                    So soon? But we just met yesterday.
                    And who are you, anyway?
            }
            
            {not first_time?
                player:
                    {rival}'s gettin' antsy, you know. {rival.HeShe} needs to talk to you real bad.
                agent:
                    And why would that be, huh?
            }

        ]],
        OPT_CONVINCE = "Convince {agent} to come along",
        DIALOG_CONVINCE = [[
            player:
                !happy
                {rival} sent me because {rival.heshe}'s being watched. By the Cult.
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                All I'm allowed to tell you is that {rival.heshe} wants to meet in the Bistro Rentori. 
            agent:
                That place? It's pretty crowded.
                {rival} must be worried about Cult retribution. I wonder if {rival.heshe} learned something about the Beaching...
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            player:
                So at the very least you'll get a good meal out of it.
            agent:
                Or a Heshian ambush. 
                Fine. I'll go to the stupid meeting, but I'm not going alone.

        ]],
        DIALOG_BACK = [[
            player:
                Hold that thought. I'll be right back.
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_TELL_MEETING")
            :Dialog("DIALOG_TELL_MEETING")
            :Fn(function() 
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("arrange_meeting_with_baron")
                    :OnFailure()
                        :Fn(function() cxt.quest.param.baron_backup = true end)
                        :ReceiveOpinion("raised_suspicion")
                        :CompleteQuest("arrange_meeting_with_baron")
                        
                StateGraphUtil.AddBackButton(cxt)
                    :Dialog("DIALOG_BACK")
            end)
    end)


QDEF:AddConvo("collect_rival", "rival")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                * {rival} isn't hard to find, because {rival.hisher} home is almost as ostentatious as Andwanette's.
                * You weren't quite expecting to see {rival.heshe} already has company, though.
                rival:
                    !left
                    Please, we can talk about this!
                bandit:
                    !right
                    I said hand over what you owe us!
                    Waste more of my time and I'm going to start breaking things.
                
                player:
                    !left
                    Hey.
                bandit:
                    Who the hell are you? What are you doing here?
                rival:
                    !left
                    This is, uh, my business partner!
                    You better get out of here before he gets angry!
                    You don't want to see him when he's angry!
                player:
                    !left
                bandit:
                    This grifter? Ha!
                    I'm going to kill him in front of you to make a point.
            ]],
            OPT_BATTLE = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Wait a minute, I get a say in this too!
            ]],
            DIALOG_WON = [[
                agent:
                    !exit
                * The bandit threat is over, and {rival} comes out of hiding.
            ]],

            OPT_CONVINCE = "Convince {bandit} to leave",
            DIALOG_CONVINCE = [[    
                player:
                    Wait just a minute, now.
                    My, uh, partner and I have important business to discuss.
                    We'll have your money, soon, but we need a little more time!
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                agent:
                    No more extensions!
                    !fight
                    Kill the grifter!
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                agent: 
                    Fine. One more day.
                    But you better come through with money, {rival}, or...
                    !thoatcut
                    No grifter will be able to save you next time.
            ]],
            DIALOG_RIVAL_DIED = [[
                * {rival} is dead. This isn't what {andwanette} wanted, but maybe you can salvage the situation by talking to {andwanette.himher} personally.
            ]]

        }
        :Fn(function(cxt) 
            cxt.enc.scratch.bandits = CreateCombatParty("BANDIT_PATROL", 2, cxt.location)
            cxt:ReassignCastMember('bandit', cxt.enc.scratch.bandits[1])
            cxt:TalkTo(cxt.enc.scratch.bandits[1])
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :GoTo("STATE_POST")

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE, allies = {"rival"}}
                        :OnWin()
                            :Fn(function() 
                                if cxt.quest:GetCastMember("rival"):IsDead() then
                                    cxt:Dialog("DIALOG_RIVAL_DIED")

                                    cxt.quest:Fail("collect_rival")
                                    cxt.quest:Activate("rival_died")
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                else
                                    cxt:GoTo("STATE_POST")
                                end

                            end)
            end)
        end)

        :State("STATE_POST")
            :Loc{
                DIALOG_POST = [[
                    player:
                        !happy
                    rival:
                        !right
                        You're an unexpected blessing. To what do I owe this little miracle, hm?
                    player:
                        Actually, Andwanette sent me.
                        She's, uh, lookin' to get out of the game, and was wondering if you might want to inherit some of her hustles.
                    rival:
                        I must admit that's surprising. Andwanette and I haven't always seen eye to eye...
                    player:
                        She respects you, though. So I think she wants to see you set up to, you know, carry on her legacy.
                        She's getting married, you see. 
                    rival:
                        She <i>is</i>? Really? Presumably she must be marrying up to be so tempted.
                        I suppose it's best to bury the hatchet, especially because, as you could see, I could really use the shills.
                    player:
                        Miracle upon miracle, right?
                        I'll take you to the Bistro Rentori--and don't worry about the price. It'll be her treat!
                ]],


                --OPT_Q1 = "Ask about the bandits",
                --DIALOG_A1 = [[
                    --agent:
                        --Blech.
                --]],
                --OPT_Q2 = "Ask about {andwanette}",
                --DIALOG_A2 = [[
                    --agent:
                        --Not my favorite person. Nope.
                --]],

                --OPT_LEAVE = "Take {rival} to '{andwanette}'",
                --DIALOG_LEAVE = [[
                    --rival:
                        --Let's go!
                --]],
                DIALOG_LEAVE = [[
                    rival:
                        Ooh, fancy.
                        !happy
                        Lead the way, moneybags!
                        !exit
                ]],

            }
            :Fn(function(cxt) 

                for k,v in pairs(cxt.enc.scratch.bandits) do
                    if not v:IsDead() then v:MoveToLimbo() end
                end


                cxt:TalkTo("rival")
                cxt:Dialog("DIALOG_POST")
                cxt:Opinion("saved_from_bandits")
                
                cxt:Opt("OPT_LEAVE")
                    :PreIcon( global_images.close)
                    :Dialog("DIALOG_LEAVE")
                    :Fn(function() 
                        cxt.quest:GetCastMember("rival"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                    end)
                    :CompleteQuest("collect_rival")
                    :Travel()
            end)




QDEF:AddConvo("escort_rival_to_meeting")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("meeting_place") end)
        :Loc{
            DIALOG_INTRO = [[
                * You arrive at the meeting place.
                rival:
                    !left
                    She's certainly seeking to impress, isn't she?
                    But I've actually eaten here once before. So where's {andwanette}?

                {not baron_backup?
                    * {baron} is already here.
                }
                {baron_backup?
                    * {baron} is already here, and {baron.heshe} brought backup.
                }
                baron:
                    !right
                    {rival}. I got your message. What was so important that you needed to discuss?
                
                rival:
                    My message?
                    I'm not sure what you mean. Were we supposed to meet?
                
                {not priest_backup?
                    * {priest} approaches, looking haughty and inconvenienced. 
                }
                {priest_backup?
                    * {priest} approaches, flanked by muscle.
                }

                priest:
                    !right
                    !angry
                    {rival}! This better be good. I will not make a habit of responding to your summons.
                    * {priest} suddenly notices {baron}, and {priest.hisher} eyes shrink to pinpricks of hate.
                    Is this... a <i>Spark Baron</i>, {rival}? What under Hesh's bell is this about?
                player:
                    !left
                    Oh, no worries, your brininess. {rival} just has to finish up a quick business chat with {baron}, here, and then {rival.heshe}'ll be right with you.
                priest:
                    <i>Business chat?!</i> Do you mean to tell me you do business with Barons?
                rival:
                    !left
                    I--what! No! 
                    I mean, that is... caravans are very expensive, and the lumin alone isn't enough to finance my, uh--
                priest: 
                    You transport lumin alongside the heathen fuel?! How dare you!
                    That is heresy punishable by death!
                rival:
                    !scared
                    W-what?! No! I was just--
                    {baron}, my friend, surely you'll help me!

                baron:
                    !right
                    !angry
                    This is your screw-up, {rival}! If the jelly-lover doesn't deal with you, I'll do it myself.
                
                rival:
                    !scared
            
                
                {baron_backup and not priest_backup?
                    baron:
                        !threaten
                        And if one priest goes down, too, then that's just a freebie.

                }
                
                {not baron_backup and priest_backup?
                    
                    priest:
                        !right
                        !fight
                        Fortunately we're well prepared to take down you <i>and</i> the Baron.

                }

                {baron_backup and priest_backup?
                    baron:
                        This is a bit earlier than we wanted to make our move...
                        But it's also a long time coming.
                        !fight
                }

                {not baron_backup and not priest_backup?
                    baron:
                        !fight
                        Wonder if either of you can throw a punch, huh?
                }

                * Looks like a fight is about to break out!
                * {rival} scuttles out of the door in midst of the confusion.
            ]],

            OPT_FIGHT_WITH_BARON = "Join {baron} in the fight!",
            DIALOG_FIGHT_WITH_BARON = [[
                * You dive into the brawl on the side of {baron}!
            ]],
            DIALOG_WON_WITH_BARON = [[
                player:
                    !left
                    !happy
                baron:
                    !right
                    Not sure what role you had to play in all this. But when it came down to it, you helped <i>me</i> out.
                    I'll remember that. 
                player:
                    That's all a guy can ask, really. 
                baron:
                    !exit
            ]],

            OPT_FIGHT_WITH_PRIEST = "Join {priest} in the fight!",
            DIALOG_FIGHT_WITH_PRIEST = [[
                * You dive into the brawl on the side of {priest}!
            ]],
            DIALOG_WON_WITH_PRIEST = [[
                player:
                    !left
                    !happy
                priest:
                    !right
                    I'm not sure I approve of your methods, young man, but you exposed a terrible wickedness.
                    And became one with the reach of Hesh when I was threatened. 
                    The Cult will take note of your service.
                player:
                    Aw, that's nice.
                priest:
                    !exit
            ]],

            DIALOG_WON_WITH_CHAOS = [[
                player:
                    !left
                    !thought
                agent:
                    !exit
                * You spilled a lot of blood on the nice tablesettings.
                player:
                    !exit
                * Seems like a good idea to leave before anyone asks you to pay the tab.
            ]],

            OPT_LEAVE_THE_FIGHT = "Leave {baron} and {priest} to fight it out",
            DIALOG_LEAVE_THE_FIGHT = [[
                * This isn't your fight.
                * You sneak out while everyone is distracted.
            ]],
 
        }
        :Fn(function(cxt) 
            
            local barons = CreateCombatBackup(cxt.quest:GetCastMember('baron'), cxt.quest.param.baron_backup and {"SPARK_BARON_GOON", "SPARK_BARON_GOON"} or {})
            table.insert(barons, 1, cxt.quest:GetCastMember('baron'))

            local cultists = CreateCombatBackup(cxt.quest:GetCastMember('priest'), cxt.quest.param.priest_backup and {"LUMINITIATE", "LUMINITIATE"} or {} )
            table.insert(cultists, 1, cxt.quest:GetCastMember('priest'))
            
            cxt:Dialog("DIALOG_INTRO")
            
            cxt.quest:GetCastMember("rival"):Dismiss()
            cxt.quest:GetCastMember("rival"):MoveToLimbo()



            cxt:Opt("OPT_FIGHT_WITH_BARON")
                :ReceiveOpinion("picked_side", "baron")
                :Dialog("DIALOG_FIGHT_WITH_BARON")
                :CompleteQuest("escort_rival_to_meeting")
                :Battle{
                        allies = barons,
                        enemies = cultists,
                        on_runaway = StateGraphUtil.DoRunAwayNoFail,
                        no_oppo_limit = true,
                }
                    :OnWin()
                        :Fn(function() 
                            if cxt.quest:GetCastMember("baron"):IsDead() then
                                cxt:Dialog("DIALOG_WON_WITH_CHAOS")
                            else
                                cxt:Dialog("DIALOG_WON_WITH_BARON")
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)  
                        end)


            cxt:Opt("OPT_FIGHT_WITH_PRIEST")
                :ReceiveOpinion("picked_side", "priest")
                :Dialog("DIALOG_FIGHT_WITH_PRIEST")
                :CompleteQuest("escort_rival_to_meeting")
                :Battle{
                        allies = cultists,
                        enemies = barons,
                        on_runaway = StateGraphUtil.DoRunAwayNoFail,
                        no_oppo_limit = true,
                }
                    :OnWin()
                        :Fn(function() 
                            if cxt.quest:GetCastMember("priest"):IsDead() then
                                cxt:Dialog("DIALOG_WON_WITH_CHAOS")
                            else
                                cxt:Dialog("DIALOG_WON_WITH_PRIEST")
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)  
                        end)
                        

            cxt:Opt("OPT_LEAVE_THE_FIGHT")
                :ReceiveOpinion("raised_suspicion", "priest")
                :ReceiveOpinion("raised_suspicion", "baron")
                :Dialog("DIALOG_LEAVE_THE_FIGHT")
                :CompleteQuest("escort_rival_to_meeting")
                :Travel()


        end)


QDEF:AddConvo("return_to_andwanette", nil)
    :TravelConfront("STATE", function(cxt) return cxt.quest.param.confront_pending end)
        :Loc{
           
            DIALOG_INTRO = [[
                * {rival} finds you.
                player:
                    !left
                rival:
                    !right
                    {player}?
                    What happened back there? Where was {andwanette}?
                    I can't believe {priest} and {baron} were both there at the same time.
                player:
                    Uh, yeah. What are the odds of that, huh?
                    Crazy!
                rival:
                    They both seemed to be expecting me.
                    Wait a second...
            ]],

            OPT_CONVINCE = "Convince {rival} of your innocence",
            DIALOG_CONVINCE = [[
                player:
                    You seem to play the field a bit, if you don't mind me saying so.
                    Maybe you got more enemies than you think.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    If I were you, I'd keep your friends close until you can sort it all out.
                    After all, you aren't exactly flush right now.
                agent:
                    That is very good advice.
                    !happy
                    Well, at least I know I have one friend in this harsh world.
                    Thank you, {player}.
                    I'm going to go lay low for a while. See you around.
                    !exit
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    Like, more than just Andwanette, I mean.
                agent:
                    I thought you said Andwanette wanted to bury the hatchet?
                    Hesh, I should've known! You set me up!
                    !fight
                    You bastard!
            ]],

            OPT_BRUSH_OFF = "Brush off {rival}'s concerns",
            DIALOG_BRUSH_OFF = [[
                player:
                    Think you've been through enough today without getting paranoid.
                agent:
                    Hah! It's not paranoia if I'm right.
                    I'm on to you. 
                    !exit
            ]],

            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    This is going to hurt you more than it hurts me!
            ]],

            DIALOG_WON = [[
                {dead?
                    * {andwanette} wanted {rival} to survive. You have some explaining to do.
                }
                {not dead?
                    agent:
                        !injured
                    player:
                        Shouldn't pick fights you know you can't win.
                        !point
                        Now scram!
                    agent:
                        !exit
                }
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest.param.confront_pending = false

            cxt:TalkTo("rival")
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:BasicNegotiation("CONVINCE")
                :OnSuccess()
                    :Travel()
                :OnFailure()
                    :ReceiveOpinion("betrayed")
                    :Fn(function() 

                        cxt:Opt("OPT_DEFEND")
                            :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
                                :OnWin()
                                    :Dialog("DIALOG_WON")
                                    :Fn(function() 
                                        if cxt:GetAgent():IsDead() then
                                            cxt.quest:Cancel("return_to_andwanette")
                                            cxt.quest:Activate("rival_died")
                                        end
                                    end)
                                    :Travel()
                    end)

            
            cxt:Opt("OPT_BRUSH_OFF")
                :Dialog("DIALOG_BRUSH_OFF")
                :ReceiveOpinion("suspicious")
                :Travel()
        end)


QDEF:AddConvo("return_to_andwanette", "andwanette")
    :Loc{

        OPT_GIVE_REPORT = "Tell {andwanette} about the meeting",
        DIALOG_GIVE_REPORT = [[
            player:
                It's done. {rival}'s business is dead in the water.
                Not to mention {rival.heshe} seems to be in trouble with some bandits, so...
                No money, no prospects. Things don't look good.
            agent:
                That's so magnificent I could cry. 
            player:
                {rival} sure did.
            agent:
                You're truly a gift, Smith. 
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_GIVE_REPORT")
            :SetQuestMark()
            :Fn(function() 
                cxt:Dialog("DIALOG_GIVE_REPORT")
                ConvoUtil.GiveQuestRewards(cxt)
                cxt.quest:Complete()
            end)
    end)


QDEF:AddConvo("rival_died", "andwanette")
    :Loc{

        OPT_TELL_DIED = "Tell {andwanette} that {rival} is dead.",
        DIALOG_TELL_DIED = [[
            player:
                It all went just the way you wanted, 'cept that {rival} picked a fight {rival.heshe} shouldn't have.
            agent:
                What are you saying?
        ]],
        DIALOG_TELL_DIED_SUCCESS = [[
            player:
                I'm saying your plan was perfect, but {rival} was too stupid to live.
                {rival.HeShe}'s dead. Sorry.
            agent:
                That <i>is</i> disappointing. But I take some satisfaction in thinking {rival} was the master of {rival.hisher} own demise.
                You give complicated gifts, grifter.
        ]],
        DIALOG_TELL_DIED_FAILURE = [[
            player:
                I'm sayin' that if you really want to concoct grand schemes, maybe remember your target is an idiot.
            agent:
                !angry
                My agent was an idiot, more like.
                !palm
                {rival} was the only one who pushed me to the spotlight. 
                Someday I wonder if you'll know this kind of solitude.
            player:
                Uh, my siblings cut me outta the family.
            agent:
                Hardly the same!
        ]],

        SIT_MOD = "You really messed up!",

    }
    :Hub(function(cxt) 
        
        
        cxt:BasicNegotiation("TELL_DIED", {{situation_modifiers = {value = 10, text = cxt:GetLocString("SIT_MOD")}}})
            :SetQuestMark()
            :OnSuccess()
                :CompleteQuest()
            :OnFailure()
                :FailQuest()

    end)    

QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}



    



