local GOODS = {
    {id = "WEAPONS", seller_def = "ADMIRALTY_CLERK", buyer_defs = {"RISE_RADICAL", "BANDIT_GOON2"}},
    {id = "SPARK", seller_def = "SPARK_BARON_TASKMASTER", buyer_defs = {"CHEMIST", "ADMIRALTY_GUARD"}},
    {id = "LUMIN", seller_def = "PRIEST", buyer_defs = {"RISE_REBEL", "SPARK_BARON_PROFESSIONAL"}},
    {id = "MONSTER_BITS", seller_def = "CHEMIST", buyer_defs = {"WEALTHY_MERCHANT", "ZEALOT"}},
}

local function GetGoods(id)
    for k,v in ipairs(GOODS) do
        if v.id == id then
            return v
        end
    end
end



local QDEF = QuestDef.Define{
        title = "Demand and Supply",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{denikus} is busy preparing for the wedding contract, and needs you to run his distribution network.]],
        icon = engine.asset.Texture("icons/quests/side_smith_demand_and_supply.tex"),
        --disabled = true,
        focus = QUEST_FOCUS.BALANCED,
        tags = {"DENIKUS_JOB"},
        act_filter = "SMITH",

        --this is a little bit of boilerplate that lets us test the quest directly from the quest menu
        on_debug_start = function(quest)
            TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("PEARL_OSHNUDROME"))
            UIHelpers.DoSpecificConvo( quest:GetCastMember("denikus"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
        end,

        on_destroy = function(quest)
            if quest:GetCastMember("lifter") and quest:GetCastMember("lifter"):IsInPlayerParty() then
                quest:GetCastMember("lifter"):Dismiss()
            end

            if quest:GetCastMember("oshnu") and quest:GetCastMember("oshnu"):IsInPlayerParty() then
                quest:GetCastMember("oshnu"):Dismiss()
            end
        end,
        
        is_in_hiding = function(quest, agent)
            if agent == quest:GetCastMember("lifter") then
                return true
            end
        end,

        avoids = {
            {"seller1", "seller2"},
            {"seller1", "denikus"},
            {"seller2", "denikus"},

            {"buyer1", "buyer2"},
            {"buyer1", "denikus"},
            {"buyer2", "denikus"},

            {"chosen_seller", "buyer1"},
            {"chosen_seller", "buyer2"},
        }

    }

    :AddCastByAlias{
        cast_id = "denikus",
        alias = "DENIKUS",
        provider = true,
        no_validation = true,
    }

    :AddCast{
        cast_id = "seller1",
        when = QWHEN.MANUAL,
    }
    
    :AddCast{
        cast_id = "seller2",
        when = QWHEN.MANUAL,
    }

    :AddCast{
        cast_id = "chosen_seller",
        when = QWHEN.MANUAL,
    }
    
    :AddCast{
        cast_id = "other_seller",
        when = QWHEN.MANUAL,
    }

    :AddCast{
        cast_id = "buyer1",
        when = QWHEN.MANUAL,
    }
    
    :AddCast{
        cast_id = "buyer2",
        when = QWHEN.MANUAL,
    }
    :AddCast{
        cast_id = "chosen_buyer",
        when = QWHEN.MANUAL,
    }
    :AddCast{
        cast_id = "other_buyer",
        when = QWHEN.MANUAL,
    }


    :AddDefCastSpawn("lifter", "JAKES_LIFTER")

    :AddCast{
        cast_id = "oshnu",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, quest:CreateSkinnedAgent("MERCHANT_SNAIL"))
            t[1]:GenerateRandomName()
        end,
    }


    :AddQuestLocation{
        cast_id = "pickup_spot",
        name = "Pickup Spot",
        plax = "EXT_SMITH_ISOLATED_DESERT",
        map_tags = {"port"},
    }

    :AddQuestLocation{
        cast_id = "dropoff_spot",
        name = "Dropoff Spot",
        plax = "EXT_Ship_Graveyard",
        map_tags = {"cliffs"},
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------
    
    :AddObjective{
        id = "pick_a_seller",
        title = "Find a seller",
        desc = [[{denikus} has a couple of leads on commodities entering the Pearl today. Find the most promising one, and arrange a sale.]],
        mark = { "seller1", "seller2" },

        negotiation_targets = {"seller1", "seller2"},

        on_activate = function(quest)
            local goods = table.shuffle(shallowcopy(GOODS))
            local GOOD1, GOOD2 = goods[1], goods[2]
            quest.param.good1 = GOOD1.id
            quest.param.good2 = GOOD2.id

            quest:AssignCastMember("seller1", AgentUtil.GetFreeAgent(GOOD1.seller_def, function(agent) return agent:GetRelationship() <= RELATIONSHIP.NEUTRAL end))
            quest:AssignCastMember("seller2", AgentUtil.GetFreeAgent(GOOD2.seller_def, function(agent) return agent:GetRelationship() <= RELATIONSHIP.NEUTRAL end))
        end,

        on_complete = function(quest)
            quest:Activate("tell_denikus_about_seller")
            quest.param.other_seller_pending = true
        end,
    }


    :AddObjective{
        id = "tell_denikus_about_seller",
        title = "Tell {denikus} about {chosen_seller}",
        desc = [[You've found a seller. Report back to {denikus}.]],
        mark = { "denikus"},
        on_complete = function(quest) 
            quest:Activate("pick_a_buyer")
        end,
    }


    :AddObjective{
        id = "pick_a_buyer",
        title = "Pick a buyer",
        desc = [[Find someone to buy the goods you bought from {chosen_seller}.]],
        mark = { "buyer1", "buyer2" },

        negotiation_targets = {"buyer1", "buyer2"},

        on_activate = function(quest)
            
            local good = GetGoods(quest.param.seller_good)
            quest:AssignCastMember("buyer1", AgentUtil.GetFreeAgent(good.buyer_defs[1]))
            quest:AssignCastMember("buyer2", AgentUtil.GetFreeAgent(good.buyer_defs[1]))
        end,

        on_complete = function(quest)
            quest:Activate("meet_lifter")
        end,
    }


    :AddObjective{
        id = "meet_lifter",
        title = "Meet up with {lifter}",
        desc = [[{lifter} will meet you at the pickup spot.]],
        mark = { "pickup_spot"},
    }


    :AddObjective{
        id = "deliver_goods",
        title = "Deliver the goods",
        desc = [[Escort the shipment to the dropoff spot. Keep an eye out for trouble!]],
        mark = { "dropoff_spot"},
        
        on_activate = function(quest)
            quest.param.deliver_interruption_pending = true
            quest:GetCastMember("lifter"):Recruit( PARTY_MEMBER_TYPE.ESCORT )
            quest:GetCastMember("oshnu"):Recruit( PARTY_MEMBER_TYPE.ESCORT )
        end,

        on_complete = function(quest)
             if quest:GetCastMember("lifter") and quest:GetCastMember("lifter"):IsInPlayerParty() then
                quest:GetCastMember("lifter"):Dismiss()
            end

            if quest:GetCastMember("oshnu") and quest:GetCastMember("oshnu"):IsInPlayerParty() then
                quest:GetCastMember("oshnu"):Dismiss()
            end

            quest:Activate("return_to_denikus")
        end,
     }


    :AddObjective{
        id = "return_to_denikus",
        title = "Return to {denikus}",
        desc = [[Tell {denikus} how the transaction went.]],
        mark = { "denikus"},
    }


    :AddOpinionEvents{
        bought_at_high_price = {
            delta = OPINION_DELTAS.LIKE,
            txt = "Bought from them at a high price",
        },
        sold_at_a_low_price = {
            delta = OPINION_DELTAS.LIKE,
            txt = "Sold to them at a low price",
        },
        backed_out_of_deal = {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Backed out of a deal",
        },
        refused_deal = {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Refused to do business with them",
        },
        switched_deal = {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Switched business partners for them",
        },
        helped_load = {
            delta = OPINION_DELTAS.LIKE,
            txt = "Helped them load an Oshnu",
        },
        not_helped_load = {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Refused to help them",
        }
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddIntro(
    --attract spiel
    [[
        agent: 
            The wheels of romance are in motion, but I still have business to see to.
            That's where you come in, soon-to-be brother-in-law.
        player:
            Business? Yeah, sure. I can take care of your "business".
            Just show me who to punch.
        agent:
            Easy, there. That's not what I had in mind.
            It'd help to convince Theroux of my proposal if I had some trade certificates to show for it.
            Haven't you ever heard that you catch more flies with honey?
        player:
            Pretty sure you catch more flies with a flyswatter.
            But I'll try things your way, if that's what you wanna hear.
    ]],
    
    --on accept
    [[
        agent:
            It's not that hard. You find a seller, then you find a buyer.
            Put two and two together and--well, it's not much different than matchmaking, since that's the game you're playing.
            I have a few leads on folks who might be selling, so choose one and chase it. 
        player:
            And then what?
        agent:
            Then you--well, actually, maybe then you come back to me, yeah?
            Best I have a look at what you've arranged before we settle on the buyer.
        player:
            Kinda figured you maybe had a few more shortcuts up your sleeve.
        agent:
            This <i>is</i> the shortcut.
            Have at it. And I'll see you back in a bit.
    ]],
    "pick_a_seller")

---------------------------------------------------------------------------------------------------------------------------------------------------

local find_a_seller_fn = function(cxt)
    cxt:Dialog("DIALOG_INTRO")
    cxt:RunLoop(function() 

        cxt:Question("OPT_Q1", "DIALOG_A1")
        
        cxt:Opt("OPT_TAKE_DEAL")
            :Dialog("DIALOG_TAKE_DEAL")
            :ReceiveOpinion("bought_at_high_price")
            :Fn(function() 
                
                cxt.quest.param.seller_good = cxt.enc.scratch.GOOD.id
                cxt.quest.param.other_seller_good = cxt.enc.scratch.OTHER_GOOD.id
                cxt.quest:AssignCastMember("chosen_seller", cxt:GetAgent())
                cxt.quest:AssignCastMember("other_seller", cxt:GetAgent() == cxt.quest:GetCastMember("seller1") and cxt.quest:GetCastMember("seller2") or cxt.quest:GetCastMember("seller1"))

            end)
            :CompleteQuest("pick_a_seller")
            :DoneConvo()

        cxt:Opt("OPT_CONVINCE_SELL_BETTER_DEAL")
            :Dialog("DIALOG_CONVINCE_SELL_BETTER_DEAL")
            :Negotiation{}
                :OnSuccess()
                    :Dialog("DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS")
                    :Fn(function() 
                        cxt.quest.param.got_good_price_from_seller = true
                        cxt.quest.param.seller_good = cxt.enc.scratch.GOOD.id
                        cxt.quest.param.other_seller_good = cxt.enc.scratch.OTHER_GOOD.id
                        cxt.quest:AssignCastMember("chosen_seller", cxt:GetAgent())
                        cxt.quest:AssignCastMember("other_seller", cxt:GetAgent() == cxt.quest:GetCastMember("seller1") and cxt.quest:GetCastMember("seller2") or cxt.quest:GetCastMember("seller1"))
                    end)
                    :CompleteQuest("pick_a_seller")
                    :DoneConvo()
                :OnFailure()
                    :Dialog("DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE")


            StateGraphUtil.AddBackButton(cxt)
                :Dialog("DIALOG_BACK")
    end)
    
end

QDEF:AddConvo("pick_a_seller")
    :Loc{
        OPT_ASK_ABOUT_GOODS = "Ask about a trade deal...",
    }
    :Hub(function(cxt) 
        
        if  cxt:GetAgent() == cxt.quest:GetCastMember("seller1") or
            cxt:GetAgent() == cxt.quest:GetCastMember("seller2") then

                local GOOD1 = GetGoods(cxt.quest.param.good1)
                local GOOD2 = GetGoods(cxt.quest.param.good2)
                cxt.enc.scratch.GOOD = cxt:GetAgent() == cxt.quest:GetCastMember("seller1") and GOOD1 or GOOD2
                cxt.enc.scratch.OTHER_GOOD = cxt:GetAgent() == cxt.quest:GetCastMember("seller1") and GOOD2 or GOOD1

                cxt:Opt("OPT_ASK_ABOUT_GOODS")
                    :SetQuestMark()
                    :GoTo("STATE_"..cxt.enc.scratch.GOOD.id)
        end

    end)

    :State("STATE_WEAPONS")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        Hey, I'm, uh, looking to start up some trade arrangements for a friend of mine.
                        {denikus}. A legit trader, ya know? Just looking for some new product to move.
                        I hear you're selling?
                    agent:
                        !cagey
                        A legit trader, huh? 
                        That actually sounds all right. Because I have, uh, legitimate product.
                        Definitely on the level, definitely not slightly illegal.
                    player:
                        I'm definitely okay with that.
                        But how "not illegal" are we talkin', here?
                    agent:
                        Well, these weapons have been safety tested, certified, and are ready for use.
                        By... licensed Admiralty.
                        I <i>assume</i> the buyer will be licensed members of the Admiralty, and that I needn't ask any follow-up questions.  
                }
                {not first_time?
                    player:
                        So about those weapons...
                    agent:
                        "Legitimate product". But go on.
                }
            ]],
            OPT_Q1 = "Ask why {agent} is selling weapons",
            DIALOG_A1 = [[
                player:
                    So, you're Admiralty.
                agent:
                    Obviously.
                player:
                    And you're selling these weapons to--
                agent:
                    <i>Other Admiralty</i>. Theoretically. As far as I know.
                player:
                    Right, but, uh, say those other Admiralty ain't quite as Admiralty as you expect.
                    Aren't you worried about them using one of those weapons on you?
                agent:
                    In the Pearl? Are you kidding?
                    The most dangerous thing to happen in the Pearl was a Heshian miracle.
                    Wherever these weapons end up, it'll be some Murder Bay grunt's problem. Not mine.
            ]],

            OPT_TAKE_DEAL = "Agree to the asking price",
            DIALOG_TAKE_DEAL = [[
                player:
                    Great, so, uh... let's do it?
                agent:
                    At the asking price?
                player:
                    Oh. I guess that's bad, huh? Can I try again?
                agent:
                    Hah! Tell {denikus} I'm ready to move when {denikus.heshe} is.
            ]],
            OPT_CONVINCE_SELL_BETTER_DEAL = "Negotiate for a better offer",
            DIALOG_CONVINCE_SELL_BETTER_DEAL = [[
                player:
                    You <i>know</i> this is a hard sell, especially in the Pearl.
                    Let's work out some kinda discount, huh?
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS = [[
                player:
                    Security on the streets is extra tight. So we'll have to sit on the product for a while.
                    There'll be warehousing fees, bribes, the works.
                agent:
                    Twist my arm, why don't you!
                player:
                    Hey, sure. If you're offering.
                agent:
                    What? No! 
                    But {denikus} can have {denikus.hisher} discount.
                    Tell {denikus.himher} I'm ready when {denikus.heshe} is.

            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE = [[
                player:
                    That's dangerous product to sit on, especially now with all the pilgrims and grifters, huh?
                agent:
                    Then that sounds like a compelling reason for the product to stay with me. 
                    The price already accounts for the risk. If you don't like it, don't buy it.
            ]],
            DIALOG_BACK = [[
                player:
                    I need to think about this.
                agent:
                    Don't take too long. You're not the only buyer in the Pearl.
            ]]
        }
        :Fn(find_a_seller_fn) 
        
    :State("STATE_SPARK")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        Hey, you {agent}? {denikus} sent me.
                        I hear you got a proposition: product in need of movement.
                    agent:
                        Sure. The finest spark, straight from the refinery. 
                    player:
                        Whoa, spark, huh? 
                        That's gonna be hard to sneak around with luminari everywhere.
                    agent:
                        Yep. That's what makes it valuable.
                        You got a product everyone wants but nobody can get? Then you're in the money.
                }
                {not first_time?
                    agent:
                        Better than lumin, I guarantee it.
                }
            ]],
            OPT_Q1 = "Ask about the spark",
            DIALOG_A1 = [[
                player:
                    How much spark are we talking about?
                agent:
                    Enough to power a city block for a year, if the Heshians would ever allow it.
                    And enough to do just about whatever you'd like to do, if the Heshians won't know about it.
                player:
                    That's... a lot of spark.
                    It won't go off, right? Or explode on me?
                agent:
                    It doesn't go off. And it only explodes in the hands of idiots.
                player:
                    Er, right.
            ]],

            OPT_TAKE_DEAL = "Take it at the asking price",
            DIALOG_TAKE_DEAL = [[
                player:
                    Well, uh, sign me up for the spark, I guess.
                agent:
                    Excellent choice. A premium price for a premium product.
                    Arrange the pickup with {denikus}. I'll wait for you.
            ]],
            OPT_CONVINCE_SELL_BETTER_DEAL = "Negotiate for a better offer",
            DIALOG_CONVINCE_SELL_BETTER_DEAL = [[
                player:
                    Spark's a hard sell in the Foam. How about a little first-time discount, huh? 
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS = [[
                player:
                    Think of it like, ya know, an investment.
                    We find a buyer, forge that supply line...
                    Then we can come back and talk more business later.
                agent:
                    Hm. I suppose the Beaching has people doubling down on lumin.
                    I'll give {denikus} an "early-adopter" discount.
                player:
                    Mighty charitable of you, pal.
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE = [[
                player:
                    I mean, it's not like spark is as good as lumin...
                agent:
                    Not like ...?
                    If you want to be an ignorant shellshucker, do it on your own time. 
                    On mine? I charge a premium.
            ]],
            DIALOG_BACK = [[
                player:
                    Gimme a bit, will you?
                agent:
                    I'll wait. But not too long.
            ]]
        }
        :Fn(find_a_seller_fn) 

    :State("STATE_LUMIN")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        {denikus} is buying. You selling?
                    agent:
                        I... I am.
                        I have a small supply of lumin that I need to liquidate.
                    player:
                        I thought lumin was a liquid already.
                    agent:
                        You know what I mean.
                        Do you want it?
                }
                {not first_time?
                    player:
                        Is the lumin still for sale?
                    agent:
                        Yes. Please, take it.
                }
            ]],
            OPT_Q1 = "Ask why {agent} needs to get rid of the lumin",
            DIALOG_A1 = [[
                player:
                    Ain't lumin usually worth <i>more</i> through the official channels?
                    You know, what with the whole Cult of Hesh holdin' a monopoly on it?
                agent:
                    Ah, yes, well, you see... <i>this</i> lumin is, um, unofficial.
                player:
                    Is it spoiled or something?
                agent:
                    No, no. Of course not. It's perfectly fine for most purposes.
                player:
                    Most?
                agent:
                    It's just not fit for, uh... liturgical uses.
                player:
                    Uh. What?
                agent:
                    The creatures on the beach, you know, give off lumin.
                    Well, after some study, we determined that this lumin might have been excreted from...
                    An unsavory part of the animal.
                player:
                    Uh. 
                    Is this...
                    Poop lumin?
                * {agent}'s eyelids flutter with dismay at the very mention, but {agent.heshe} doesn't deny it.
                player:
                    But, uh, technically speakin'...
                    Since you don't actually know where regular lumin comes from...
                agent:
                    Don't you dare say it!
                    Don't even suggest it!
                    <i>This</i> lumin is different, and we'll leave it at that.

            ]],

            OPT_TAKE_DEAL = "Take it at the asking price",
            DIALOG_TAKE_DEAL = [[
                player:
                    Seems like what we don't acknowledge can't hurt us.
                    So we'll take the poo--the, uh, unsavory lumin.
                agent:
                    R-really? I mean, that's excellent news, of course!
                player:
                    I'll go talk to {denikus}, and come back later to complete the deal.                    
            ]],
            OPT_CONVINCE_SELL_BETTER_DEAL = "Negotiate for a better offer",
            DIALOG_CONVINCE_SELL_BETTER_DEAL = [[
                player:
                    Buddy. Pal. Come on.
                    It's poop lumin.
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS = [[
                player:
                    I'm gonna need a big discount on that. Like, Hesh-big.
                agent:
                    Alright, alright.
                    I'll give {denikus} a discount. I just want this ordeal to be over.
                player:
                    !shrug
                    In this business, ain't none of us got clean hands.
                    But, uh, your hands are maybe dirtier than most.
                agent:
                    Come back with an oshnu. It's... rather a lot.

            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE = [[
                player:
                    You really gonna make me pay premium for--
                agent:
                    Lumin, yes! It may be of, uh, questionable providence, but it is still the aether of Hesh. 
                    To go any lower would be, well, blasphemy.
            ]],
            DIALOG_BACK = [[
                player:
                    I'm going to need to think this one through.
                agent:
                    !hesh_greeting
                    Walk in the shallows until you return.
            ]]
        }
        :Fn(find_a_seller_fn) 


    :State("STATE_MONSTER_BITS")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        {denikus} says you have something for sale?
                    agent:
                        I do. Exotic compounds straight from the Roaloch.
                        Excretions scraped from the pustules of the Pulpous Tongue itself.
                        Mucus from the wild snail pits of Tharr.
                        Even the concentrated droppings of an engorged ghall fly swarm.
                    player:
                        Honestly, that all sounds kinda gross.
                    agent:
                        Oh it is. Moreso than you could ever imagine.
                        But also quite effective, in the right hands. 
                }
                {not first_time?
                    player:
                        You still have those gross monster-bits for sale?
                    agent:
                        Yes. Although I would prefer if you used their proper names. 
                }
            ]],
            OPT_Q1 = "Ask about the Roaloch",
            DIALOG_A1 = [[
                player:
                    So... I've never seen the Roaloch. Does this stuff just like... grow out of the ground?
                agent:
                    There is no "ground" on the Roaloch. Not as you mean, anyway.
                    The Roaloch is ecosystem unto itself, forging symbiotic relationships with the very geology.
                    A seedling latches onto land and then grows, fusing itself to the bedrock and eroding the soil. 
                    Any creatures caught in its path are assimilated and changed.
                player:
                    You serious? So this stuff could turn everyone into monsters?
                agent:
                    By the time we've harvested a new species, the mutagen is inert.
                    The fruits of the Roaloch aren't always safe--they may be highly virulent, toxic, or predacious--but that's why we study them so closely. 

            ]],

            OPT_TAKE_DEAL = "Take it at the asking price",
            DIALOG_TAKE_DEAL = [[
                player:
                    Alright, I'll take your nightmare stew of monster bits.
                agent:
                    Since you're paying full price, I will overlook that insulting description of my wares.
                player:
                    That's fair.
                    I'll contact you again for the pickup.
            ]],
            OPT_CONVINCE_SELL_BETTER_DEAL = "Negotiate for a better offer",
            DIALOG_CONVINCE_SELL_BETTER_DEAL = [[
                player:
                    Look, you might <i>say</i> it's safe, but as a first time customer I'm cagey.
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS = [[
                player:
                    Let's build a rapport, huh? Earn my trust with good prices, then I'll be a repeat customer.
                agent:
                    You are a skilled negotiator.
                    I will give you a lower price, in the interest of hastening the deal.
                    My wares are perishable, after all.
                player:
                    Ew.
                    Anyway, I'll come by with an oshnu to get the stuff.

            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE = [[
                player:
                    How do I know you're on the level?
                agent:
                    I am alive, am I not?
                    Were I incompetent or ignorant, the Roaloch would have killed me long ago.
                    My presence here is a testimony to my ability. And my ability deserves compensation.
            ]],
            DIALOG_BACK = [[
                player:
                    I have other sellers to talk to. I'll be back in a bit.
                agent:
                    !permit
                    As you will.
            ]]
        }
        :Fn(find_a_seller_fn) 






QDEF:AddConvo("tell_denikus_about_seller", nil)
    :TravelConfront("STATE", function(cxt) return cxt.quest.param.other_seller_pending end)
        :Loc{
            DIALOG_INTRO = [[
                * {agent} approaches you.
                player:
                    !left
                agent:
                    !right
                    I hear you're dealing with {chosen_seller}.
                    Listen, I need this sale more than {chosen_seller.heshe} does.
                    Please, reconsider my offer. I'll give you a better deal.
            ]],
            OPT_SWITCH_DEAL = "Pick {agent} over {chosen_seller}",
            DIALOG_SWITCH_DEAL = [[
                player:
                    Yeah, fine. I'll do it.
                agent:
                    Thank you! You're really helping me out of a tight spot here.
                    !exit
                * {agent} leaves, to go ready {agent.hisher} goods for transport.
            ]],

            OPT_REFUSE = "Refuse to switch your deal",
            DIALOG_REFUSE = [[
                player:
                    No, I gave {chosen_seller} my word.
                agent:
                    !spit
                    Thanks for nothing, grifter.
                    !exit
            ]],
        }
        :Fn(function(cxt) 
            cxt.quest.param.other_seller_pending = false
            cxt:TalkTo("other_seller")
            cxt:Dialog("DIALOG_INTRO")

            
            cxt:Opt("OPT_SWITCH_DEAL")
                :Dialog("DIALOG_SWITCH_DEAL")
                :ReceiveOpinion("backed_out_of_deal", nil, "chosen_seller" )
                :ReceiveOpinion("switched_deal", nil, "other_seller")
                :Fn(function() 
                    
                    cxt.quest.param.got_good_price_from_seller = true
                    cxt.quest.param.switched_seller = true

                    cxt.quest.param.other_seller_good, cxt.quest.param.seller_good = cxt.quest.param.seller_good, cxt.quest.param.other_seller_good
                    local new_seller = cxt.quest:GetCastMember("other_seller")
                    local new_other_seller = cxt.quest:GetCastMember("chosen_seller")
                    
                    cxt.quest:UnassignCastMember("other_seller")
                    cxt.quest:UnassignCastMember("chosen_seller")

                    cxt.quest:AssignCastMember("other_seller", new_other_seller)
                    cxt.quest:AssignCastMember("chosen_seller", new_seller)

                end)
                :Travel()

            cxt:Opt("OPT_REFUSE")
                :Dialog("DIALOG_REFUSE")
                :ReceiveOpinion("refused_deal", nil, "other_seller")
                :Travel()

        end)


QDEF:AddConvo("tell_denikus_about_seller", "denikus")
    :AttractState("ATTRACT")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    {player}! Have you arranged the sale?
            ]],
            DIALOG_WEAPONS = [[
                player:
                    Yeah. A load of weapons from the Admiralty.
                
                { got_good_price_from_seller?
                    I got you a good deal, too!
                }
                {not got_good_price_from_seller?
                    They're not cheap, but they're plenty deadly.
                }
                agent:
                    It's your first day on the trading beat and you're already an arms dealer?
                    You move fast, kid, I like it.
                    Now I know a couple of folks in town who could use that kind of hardware. See if you can arrange a good deal with one of 'em.
            ]],
            DIALOG_SPARK = [[
                player:
                    We're buying spark from the Barons. 
                    Or at least from one particular Baron. I don't know if {chosen_seller.heshe} is acting in the interest of {chosen_seller.hisher} faction.
                
                { got_good_price_from_seller?
                    Especially at the price I got from {chosen_seller.himher}!
                }
                {not got_good_price_from_seller?
                    Although that would explain the markup I paid.
                }
                agent:
                    Spark is dangerous and hard to move, but that's why we get paid to do it.
                    I know some folks who'll want that kind of power. You should talk to 'em.
            ]],
            DIALOG_LUMIN = [[
                player:
                    You're the proud owner of a whole load of possibly blasphemous lumin.
                
                { got_good_price_from_seller?
                    It's unholy, but it was cheap.
                }
                {not got_good_price_from_seller?
                    I probably shouldn't have paid full price, now that I hear that part that out loud.
                }
                
                agent:
                    If it's blue and it burns, it's lumin enough for most.
                    The Cult keeps the local supply locked up tight. I know a couple of folks who could really use some off the books.

            ]],
            DIALOG_MONSTER_BITS = [[
                player:
                    I talked to a Bilebroker, and I bought us a big squishy pile of decaying monster parts.
                    {chosen_seller.HeShe} tells me that's worth something.

                { got_good_price_from_seller?
                    At least I didn't pay very much for it?
                }
                {not got_good_price_from_seller?
                    And it better be, considering what I agreed to pay.
                }
                
                agent:
                    Oh, Roaloch materials. Those usually get a good price if you can move them before they rot.
                    Or spontaneously combust.
                    Or come back to life.
                    Anyway, I know a couple of folks who might be interested. Go talk to them and pick one.
            ]],


            DIALOG_PST = [[
                agent:

                    { got_good_price_from_seller?
                        Now you already have a good deal going, but there's nothing stopping you from squeezing the buyer, too.
                    }
                    {not got_good_price_from_seller?
                        You've signed us up to pay a lot for this stuff. See if you can get a better deal from the buyer. We have a bottom line to watch!
                    }
                player:
                    Will do. What happens once I've found a buyer?
                agent:
                    I'll send my people to meet you at the pickup spot. The Beaching has put a lot more eyeballs in the Pearl, so I need you to escort the goods to the drop off spot.
                    Just make sure nothing interferes, right?
                    And then come back, and I'll give you your cut!
                player:
                    This all sounds dangerously close to honest labor. I'm reminded of why I fell into a life of grifting and carousing.

            ]],

        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            cxt:Dialog("DIALOG_".. cxt.quest.param.seller_good)
            cxt:Dialog("DIALOG_PST")

            cxt.quest:Complete("tell_denikus_about_seller")

            StateGraphUtil.AddEndOption(cxt)
        end)
        





local find_a_buyer_fn = function(cxt)
    cxt:Dialog("DIALOG_INTRO")
    cxt:RunLoop(function() 

        cxt:Opt("OPT_TAKE_DEAL")
            :Dialog("DIALOG_TAKE_DEAL")
            :ReceiveOpinion("sold_at_a_low_price")
            :Fn(function() 
                cxt.quest:AssignCastMember("chosen_buyer", cxt:GetAgent())
                cxt.quest:AssignCastMember("other_buyer", cxt:GetAgent() == cxt.quest:GetCastMember("buyer1") and cxt.quest:GetCastMember("buyer2") or cxt.quest:GetCastMember("buyer1"))
            end)
            :CompleteQuest("pick_a_buyer")
            :DoneConvo()

        cxt:Opt("OPT_CONVINCE_SELL_BETTER_DEAL")
            :Dialog("DIALOG_CONVINCE_SELL_BETTER_DEAL")
            :Negotiation{}
                :OnSuccess()
                    :Dialog("DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS")
                    :Fn(function() 
                        cxt.quest.param.got_good_price_from_buyer = true
                        cxt.quest:AssignCastMember("chosen_buyer", cxt:GetAgent())
                        cxt.quest:AssignCastMember("other_buyer", cxt:GetAgent() == cxt.quest:GetCastMember("buyer1") and cxt.quest:GetCastMember("buyer2") or cxt.quest:GetCastMember("buyer1"))
                    end)
                    :CompleteQuest("pick_a_buyer")
                    :DoneConvo()
                :OnFailure()
                    :Dialog("DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE")


            StateGraphUtil.AddBackButton(cxt)
                :Dialog("DIALOG_BACK")
    end)
    
end

QDEF:AddConvo("pick_a_buyer")
    :Loc{
        OPT_PRESENT_OFFER = "Present {denikus}'s offer...",
    }
    :Hub(function(cxt) 
        
        if  cxt:GetAgent() == cxt.quest:GetCastMember("buyer1") or
            cxt:GetAgent() == cxt.quest:GetCastMember("buyer2") then

                cxt:Opt("OPT_PRESENT_OFFER")
                    :SetQuestMark()
                    :GoTo("STATE_"..cxt.quest.param.seller_good)
        end

    end)


    --selling to bandits or rise
    :State("STATE_WEAPONS")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        I hear you might be in the market for a little firepower.
                    
                    {rise?
                        agent:
                            How did you know?
                        player:
                            I mean, you're dressed like a revolutionary...
                        agent:
                            Right. That. I've been meaning to talk to the leadership about the wisdom of wearing uniforms undercover.
                            But yeah, I'm here in the Pearl looking for support and materiel for the cause.
                    }

                    {bandits?
                        agent:
                            I hear you might be in the market for a beating.
                        player:
                            Whoa, that's a bit agressive, don't you think?
                        agent:
                            Sorry, yeah. It's the training.
                            You're right about the weapons, though. The Spree's always looking for an edge.
                    }
                }
                {not first_time?
                    player:
                        Are you still interested in weapons?
                }
            ]],
            OPT_TAKE_DEAL = "Sell for the standard price",
            DIALOG_TAKE_DEAL = [[
                player:
                    They're yours, for the standard price.
                agent:
                    Excellent!
                player:
                    I'll go arrange the dropoff.
            ]],
            
            OPT_CONVINCE_SELL_BETTER_DEAL = "Negotiate for a higher price",
            DIALOG_CONVINCE_SELL_BETTER_DEAL = [[
                player:
                    Now these are quality Admiralty goods.
                    Very pointy. You'll have to pay a premium for stuff this deadly.
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS = [[
                agent:
                    {rise?
                        The Revolution isn't cheap. We'll pay your price, grifter.
                    }
                    {bandits?
                        A good weapon is worth a good price.
                        Really, it's an investment in capital goods.
                    }
                player:
                    Great. Be ready at the dropoff spot.

            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE = [[
                agent:
                    You're asking too much. I won't pay that.
            ]],
            DIALOG_BACK = [[
                player:
                    I have other buyers who are interested. I might be back.
                agent:
                    Uh huh.
            ]]
        }
        :Fn(find_a_buyer_fn) 
        
    --admiralty or bilebrokers
    :State("STATE_SPARK")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        {denikus} tells me that you're in the market for spark.
                    
                    agent:
                        {admiralty?
                            Yeah, I might be. The brass don't outfit us proper. My outfit could really use a little extra for when the going gets tough, if you know what I mean.
                        }
                        {bilebrokers?
                            I may be. Spark would be beneficial to my research.
                        }
                }
                {not first_time?
                    player:
                        I still have that spark, if you're game.
                }
            ]],
            OPT_TAKE_DEAL = "Sell for a low price",
            DIALOG_TAKE_DEAL = [[
                player:
                    I'll even give you a deal on it.
                agent:
                    Works for me.
                    Thanks, grifter.
                player:
                    I'll bring it to the dropoff spot. Be ready.
            ]],
            OPT_CONVINCE_SELL_BETTER_DEAL = "Negotiate for a higher price",
            DIALOG_CONVINCE_SELL_BETTER_DEAL = [[
                player:
                    Now, spark isn't easy to come by. We're not just giving it away...
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS = [[
                agent:
                    Alright. I'll pay what you're asking.
                player:
                    Nice. I'll bring it to the drop off. Be ready.
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE = [[
                agent:
                    Are you trying to rip me off?
            ]],
            DIALOG_BACK = [[
                player:
                    Hold that thought...
            ]]
        }
        :Fn(find_a_buyer_fn) 


    --rise or spark_barons
    :State("STATE_LUMIN")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        You're in the Pearl. You must be here to find lumin.
                        !point
                        Tell me I'm wrong!
                    agent:
                        {rise?
                            You're not wrong. I'm here gathering support and materiel for the cause.
                            If you've got lumin, I would like to buy it.
                        }
                        {spark_barons?
                            !sigh
                            All day long back home it's just 'spark this' and 'spark that'.
                            I came to the Pearl to get my hands on a sample of lumin.
                            !shrug
                            I'm curious, alright!
                        }
                }
                {not first_time?
                    player:
                        You still interested in lumin?
                    agent:
                        I am. Are you still selling?
                }
            ]],
            OPT_TAKE_DEAL = "Sell for market price",
            DIALOG_TAKE_DEAL = [[
                player:
                    I'll sell you a lot for market price. How about that?
                agent:
                    Wow. Really? That's generous.
                    I'll take it.
                player:
                    Be ready to accept delivery when I come back.
            ]],
            OPT_CONVINCE_SELL_BETTER_DEAL = "Try to get a better price",
            DIALOG_CONVINCE_SELL_BETTER_DEAL = [[
                player:
                    Now, this isn't just any lumin.
                    This is official lumin, straight from the Cult of Hesh!
                    That's gotta be worth a little something extra, right?
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS = [[
                agent:
                    You know what? I'll pay.
                    Mostly just so I don't have to hear that spiel again.
                player:
                    Works for me.
                    Be ready to accept delivery soon.
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE = [[
                agent:
                    No, I'm not paying extra. This is the Pearl. There's lumin everywhere you look.
            ]],
            DIALOG_BACK = [[
                player:
                    Give me a moment, please.
            ]]
        }
        :Fn(find_a_buyer_fn) 

    --feud_citizen or cult_of_hesh
    :State("STATE_MONSTER_BITS")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        You look like someone who would enjoy a big vat of horrific monster parts.
                
                    {cult_of_hesh?
                        agent:
                            What?
                            Oh, are you referring to Roaloch product?
                            Yes, I have been putting out inquiries about that.
                            I have certain <i>theories</> about the nature of Hesh that can only be tested empirically.
                        player:
                            Hey what you do with this stuff is your business.
                            !cough
                        agent:
                            It's all very scientific, I assure you.
                    }
                }
                {not first_time?
                    player:
                        I'm still looking for a buyer for these guts...
                    agent:
                        I'm listening.
                }
            ]],
            OPT_TAKE_DEAL = "Sell at a reasonable price",
            DIALOG_TAKE_DEAL = [[
                player:
                    You know what, I'm going to give you deal.
                    !point
                    Just for you.
                agent:
                    I'm not complaining. I'll take it.
                player:
                    !happy
                    Great!
                    I'll contact you again for the pickup.
            ]],
            OPT_CONVINCE_SELL_BETTER_DEAL = "Sell at an unreasonable price",
            DIALOG_CONVINCE_SELL_BETTER_DEAL = [[
                player:
                    Now, these monster guts have traveled very far to make their way to you.
                    That's gotta be worth something.
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_SUCCESS = [[
                agent:
                    Fine. I'll pay your ridiculous price.
                    But I won't like it!
                player:
                    !happy
                    You don't have to!
                    I'll contact you again for delivery.
            ]],
            DIALOG_CONVINCE_SELL_BETTER_DEAL_FAILURE = [[
                agent:
                    I know what these things are worth grifter. 
                    And it's less than that.
            ]],
            DIALOG_BACK = [[
                player:
                    Hold on, I'll be right back.
            ]]
        }
        :Fn(find_a_buyer_fn) 




QDEF:AddConvo("meet_lifter")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("pickup_spot") end)
        :Loc{
            
            DIALOG_INTRO = [[
                * You find a Jake loading the goods onto an Oshnu at the spot {denikus} pointed out.
                player:
                    !left
                lifter:
                    !right
                    You're {denikus}' grifter?
                player:
                    I'm my own grifter!
                    But yeah, I'm working with {denikus}.
                lifter:
                    I'm glad you finally made it. 
                    {chosen_seller} dropped the stuff off here, but made a mess of it.
                    My back's killing me - would you mind helping me load the rest?
                    
            ]],

            OPT_AGREE = "Agree to help",
            DIALOG_AGREE = [[
                player:
                    Yeah, I'll help you.
                    !exit
                agent:
                    !exit
                * You help load the Oshnu. It is extremely hard work, and by the end, you're feeling it.
            ]],

            DIALOG_AGREE_2 = [[
                player:
                    !left
                    That really hurts!
                agent:
                    !right
                    !sigh
                    Tell me about it.
                    !happy
                    Thanks for the help, grifter.
                    Let's go.
                    !exit
                player:
                    !exit
            ]],
            OPT_REFUSE = "Refuse to help",
            DIALOG_REFUSE = [[
                player:
                    Uh... How about I stand watch?
                agent:
                    !spit
                    Whatever.
                    !exit
                * You watch {agent} load the heavy boxes onto the Oshnu. It really does look like hard work.
                agent:
                    !right
                player:
                    All done?
                agent:
                    No thanks to you.
                    Let's go.
                    !exit
                player:
                    !exit
            ]]

        }
        :Fn(function(cxt)
            cxt.quest:Complete("meet_lifter")
            
            cxt:TalkTo("lifter")
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_AGREE")
                :Dialog("DIALOG_AGREE")
                :DeltaHealth(-15)
                :Dialog("DIALOG_AGREE_2")
                :ReceiveOpinion("helped_load")
                :ActivateQuest("deliver_goods")
                :Travel()

            cxt:Opt("OPT_REFUSE")
                :Dialog("DIALOG_REFUSE")
                :ReceiveOpinion("not_helped_load")
                :ActivateQuest("deliver_goods")
                :Travel()

        end)


local function MakeInterruptionFn(can_negotiate, can_bribe, can_defend)
    local fn = function(cxt) 
            cxt:TalkTo(cxt.enc.scratch.opfor[1])
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 

                if can_bribe then
                    cxt:Opt("OPT_BRIBE")
                        :Dialog("DIALOG_BRIBE")
                        :DeliverMoney(60)
                        :Fn(function() for k,v in pairs(cxt.enc.scratch.opfor) do v:MoveToLimbo()end end)
                        :Travel()
                end

                if can_negotiate then

                    cxt:Opt("OPT_NEGOTIATION")
                        :Dialog("DIALOG_NEGOTIATION")
                        :Negotiation{
                            helpers = {"lifter"},
                        }
                            :OnSuccess()
                                :Dialog("DIALOG_NEGOTIATION_SUCCESS")
                                :Fn(function() for k,v in pairs(cxt.enc.scratch.opfor) do v:MoveToLimbo()end end)
                                :Travel()
                            :OnFailure()
                                :Dialog("DIALOG_NEGOTIATION_FAILURE")
                end

                if can_defend then

                    cxt:Opt("OPT_DEFEND")
                        :Dialog("DIALOG_DEFEND")
                        :Battle{
                            allies = {"lifter", "oshnu"},
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                        }
                            :OnWin()    
                                :Dialog("DIALOG_ON_WIN")
                                :GoTo("STATE_POST_FIGHT")
                end
            end)
    end

    return fn
end

QDEF:AddConvo("deliver_goods")
    :Loc{
        OPT_BRIBE = "Give {agent} a bribe",
        OPT_DEFEND = "Defend yourself!",
        OPT_NEGOTIATION = "Convince {agent} to let you pass",
    }
    :TravelConfront("STATE", function(cxt) return cxt.quest.param.deliver_interruption_pending end)
        :Fn(function(cxt)
            cxt.quest.param.deliver_interruption_pending = false 
            cxt:GoTo("STATE_"..cxt.quest.param.seller_good)
        end)
    :State("STATE_WEAPONS")
        :Loc{
            DIALOG_INTRO = [[
                * You are stopped by an Admiralty patrol.
                player:
                    !left
                agent:
                    !right
                    That's far enough, grifter.
                    We have a report of a load of stolen weapons being moved through the Pearl.
                    You're going to have to show me what's in that Oshnu.
            ]],

            DIALOG_BRIBE = [[
                player:
                    I can't show you what's in the Oshnu, but I can show you what's in my hand.
                    !give
                    It's money.
                    For you.
                    If you leave us alone.
                agent:
                    That's probably the least graceful attempt at bribery I've ever encountered.
                player:
                    But you're still going to take though, right?
                agent:
                    ...
                    Yes. I'm not stupid.
                    !exit
                * The Admiralty leave, divvying the money amongst themselves.
                lifter:
                    !right
                    Life sure is easy when you walk around with fat stacks like that.
                player:
                    !happy
                    I make no apologies for who I am!
            ]],

            DIALOG_DEFEND = [[
                player:
                    !fight
                    Go ahead and try, friend.
            ]],

            DIALOG_NEGOTIATION = [[
                player:
                    We already cleared this with your, uh, superior officer.
            ]],
            DIALOG_NEGOTIATION_FAILURE = [[
                agent:
                    Nice try, but my superior officer hasn't shown up for work in weeks.
                    I think he's dead!
            ]],
            DIALOG_NEGOTIATION_SUCCESS = [[
                agent:
                    Oh, so you already have the paperwork.
                player:
                    Yeah, just... not with me.
                    !shrug
                    In my other pants, maybe?
                    Do I look like I'd lie to you?
                agent:
                    ...
                    Alright, I'll trust you.
                    !exit
                * The Admiralty leave. You think a large part of their motivation is so that they don't have to talk to you any more.
                lifter:
                    !right
                    How often does something like that work for you?
                player:
                    More often than it should, probably.
            ]],
            DIALOG_ON_WIN = [[
                {dead?
                    player:
                        !thought
                    * You stare down at another dead switch and almost feel bad.
                    player:
                        !happy
                    * Eh. They probably deserved it, somehow.
                }
                {not dead?
                    agent:
                        !injured
                        C-carry on, grifter.
                        !exit
                    player:
                        You're welcome!
                    * The admiralty scuttle away to lick their wounds.
                }
            ]],

        }
        :Fn(function(cxt) 
            cxt.scratch.opfor = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank(), cxt.location)
            local fn = MakeInterruptionFn(true, true, true)
            fn(cxt)
        end)



    :State("STATE_SPARK")
        :Loc{
            DIALOG_INTRO = [[
                * From out of nowhere, a bunch of Rise surround you.
                player:
                    !left
                agent:
                    !right
                    Hold up, grifter!
                    We're here to take back our spark.
                player:
                    I bought it fair and square from {chosen_seller}.
                agent:
                    But it was mined by us.
                    We followed {chosen_seller} all the way from the Bog to get it back.
                    It's ours by rights!
            ]],

            DIALOG_BRIBE = [[
                player:
                    I need these weapons, but...
                    Maybe I could give you a little something towards the cause?
                    !give
                agent:
                    !thought
                * {agent} mulls over your offer. 
                agent:
                    We'll take it.
                    But only because we don't want to waste our ammunition on you.
                * The Rise depart as quickly as they arrived.
                lifter:
                    !right
                    Can't say you'll get far paying for everything twice.                    
                player:
                    !happy
                    It's just money. There's always more where that came from.
            ]],

            DIALOG_DEFEND = [[
                player:
                    !fight
                    This is why nobody likes you.
            ]],

            DIALOG_NEGOTIATION = [[
                player:
                    You don't want these weapons. They're defective.
            ]],
            DIALOG_NEGOTIATION_FAILURE = [[
                agent:
                    A likely story. How about we test those 'defective' weapons out on you to be sure?
            ]],
            DIALOG_NEGOTIATION_SUCCESS = [[
                player:
                    I bought 'em cheap because they're the Admiralty's rejects. I'm making a killing selling them off.
                    And I'll be out of town before anyone's the wiser.
                agent:
                    ...
                    That's a clever scheme.
                    Remind me not to buy anything from you, grifter.
                    !exit
                * The Rise leave, in search of other weapons to feed back into the Bog.
                lifter:
                    !right
                    You're pretty quick with your words.
                player:
                    !happy
                    I've been told I'm sometimes too quick!
            ]],
            DIALOG_ON_WIN = [[
                {dead?
                    player:
                        !thought
                    * {agent}'s revolution is over.
                }
                {not dead?
                    agent:
                        !injured
                        You're no better than the Barons, grifter.
                        !exit
                }
            ]],
        }
        :Fn(function(cxt) 
            cxt.scratch.opfor = CreateCombatParty("RISE_PATROL", cxt.quest:GetRank(), cxt.location)
            local fn = MakeInterruptionFn(true, true, true)
            fn(cxt)
        end)



    :State("STATE_LUMIN")
        :Loc{
            DIALOG_INTRO = [[
                * You are set upon by the Cult.
                player:
                    !left
                agent:
                    !right
                    Grifter. By authority of the Cult, you will hand over that tainted lumin to us.
                player:
                    Hey, I bought it fair and square from {chosen_seller}!
                agent:
                    {chosen_seller} was wrong to sell it to you.
                    It is unclean, and must be destroyed!
            ]],

            DIALOG_BRIBE = [[
                player:
                    How about I tell you that I'm going to destroy it.
                    And then I pay you to believe me.
                    !give
                agent:
                ...
                * {agent} hesitates. 
                agent:
                    That is acceptable.
                * The Cultists leave, no doubt working out their justifications for what just happened.
                lifter:
                    !right
                    I always knew they were as corrupt as the rest of us.
                player:
                    !happy
                    Oftentimes, moreso.
            ]],

            DIALOG_DEFEND = [[
                player:
                    !fight
                    I hope your affairs are in order!
            ]],

            DIALOG_NEGOTIATION = [[
                player:
                    This lumin? I'm taking it to be destroyed, on the order of the Cardinal.
            ]],
            DIALOG_NEGOTIATION_FAILURE = [[
                agent:
                    The Cardinal wouldn't stoop to talk to a work like you.
                    Hand over the lumin, now.
            ]],
            DIALOG_NEGOTIATION_SUCCESS = [[
                agent:
                    Far be it from me to interfere with the Cardinal's business.
                    !exit
                * The Cultists leave to darken some other chump's day.
                lifter:
                    !right
                    I can't believe they thought that someone like you would know a Cardinal.
                player:
                    You'd be surprised.

            ]],
            DIALOG_ON_WIN = [[
                {dead?
                    player:
                        !thought
                    * {agent} has gone to meet Hesh.
                }
                {not dead?
                    agent:
                        !injured
                        May your bones get stuck in Hesh's teeth.
                        !exit
                }
            ]],            
        }
        :Fn(function(cxt) 
            cxt.scratch.opfor = CreateCombatParty("HESH_PATROL", cxt.quest:GetRank(), cxt.location)
            local fn = MakeInterruptionFn(true, true, true)
            fn(cxt)
        end)
    
    :State("STATE_MONSTER_BITS")
        :Loc{
            DIALOG_INTRO = [[
                lifter:
                    !right
                    Whatever we're moving, it really smells.
                player:
                    !left
                    You're probably happier not knowing.
                    Let's just get moving before it attracts-
                
                * It's too late - the smell of the Roaloch has attracted the local fauna.
                agent:
                    !right
            ]],
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Bad monsters!
            ]],
            DIALOG_ON_WIN = [[
                player:
                    !neutral

            ]],
        }
        :Fn(function(cxt) 
            cxt.scratch.opfor = CreateCombatParty("MONSTERS", cxt.quest:GetRank(), cxt.location)
            local fn = MakeInterruptionFn(false, false, true)
            fn(cxt)
        end)

    :State("STATE_POST_FIGHT")
        :Loc{
            DIALOG_OSHNU_DEAD = [[
                lifter:
                    !right
                    This Oshnu ain't going any further.
                player:
                    What do you suggest we do?
                lifter:
                    You go ahead to the drop-point and tell the buyer they'll have to meet us here, instead.
                    I'll guard the goods.
            ]],
            DIALOG_LIFTER_DEAD = [[
                * {lifter} is dead.
                * You grab the reins of the Oshnu and continue towards the drop-point.
            ]],
            DIALOG_BOTH_DEAD = [[
                * You've done a bad job keeping your charges alive.
                * You hide the goods at the side of the road, and continue along to the drop-point.
            ]],
            DIALOG_BOTH_ALIVE = [[
                lifter:
                    !right
                    Good thing you were here for that.
                    Let's go.
                    !exit
            ]],
        }
        :Fn(function(cxt) 

            if cxt.quest:GetCastMember("lifter"):IsDead() and cxt.quest:GetCastMember("oshnu"):IsDead() then
                cxt:Dialog("DIALOG_BOTH_DEAD")
            elseif cxt.quest:GetCastMember("lifter"):IsDead() then
                cxt:Dialog("DIALOG_LIFTER_DEAD")
            elseif cxt.quest:GetCastMember("oshnu"):IsDead() then
                cxt:Dialog("DIALOG_OSHNU_DEAD")
                cxt.quest:GetCastMember("lifter"):Dismiss()
            else
                cxt:Dialog("DIALOG_BOTH_ALIVE")
            end

            StateGraphUtil.AddLeaveLocation(cxt)
        end)


QDEF:AddConvo("deliver_goods")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("dropoff_spot") end)
        :Loc{
            DIALOG_NO_LIFTER = [[
                * You arrive at the drop-point with {oshnu}.
                player:
                    !left
                agent:
                    !right
                    Took you long enough, grifter.
                player:
                    We ran into trouble.
                agent:
                    That's not my problem.
                * {agent} inspects the goods.
                agent:
                    Tell {denikus} I'm charging a 5% late fee.
            ]],
            DIALOG_NO_SNAIL = [[
                * You arrive at the drop-point empty-handed.
                player:
                    !left
                agent:
                    !right
                    Where are the goods I'm paying for?
                player:
                    We got jumped. They're back a ways. 
                agent:
                    !hips
                player:
                    It's not far!
                agent:
                    Incompetence!
                    Tell {denikus} that I'm withholding 15% for the inconvenience.
            ]],

            OPT_PROTEST = "Convince {agent} to pay the full amount",
            DIALOG_PROTEST = [[
                player:
                    Hey, it's not my fault. There were extenuating circumstances!
            ]],
            DIALOG_PROTEST_SUCCESS = [[
                agent:
                    Fine, fine. I won't hold it against you.
                    Just stop whining.
                    !exit
            ]],
            DIALOG_PROTEST_FAILURE = [[
                agent:
                    It's not my fault that you're bad at your job.
                    !exit
            ]],

            OPT_ACCEPT = "Accept the penalty",
            DIALOG_ACCEPT = [[
                player:
                    !shrug
                    Go ahead. I'll still get paid.
                agent:
                    Hrmmph!
                    !exit
            ]],

            DIALOG_ARRIVE = [[
                *You arrive at the drop-point with the goods.
                player:
                    !left
                agent:
                    !right
                    You're on time, grifter, I respect that.
                * {agent} inspects the goods.
                agent:
                    And everything's here. 
            ]],


            OPT_PRESS_FOR_A_TIP = "Ask for a tip",
            DIALOG_PRESS_FOR_A_TIP = [[
                player:
                    !cough
                    Good service deserves consideration, doesn't it?
            ]],
            DIALOG_PRESS_FOR_A_TIP_SUCCESS = [[
                agent:
                    Forgive my manners.
                    !give
                * {agent} presses a couple clammy shills into your and {lifter}'s hands and leaves.
                player:
                    Gee, thanks.
                lifter:
                    !right
                    !shrug
                    It's more than I usually get.
            ]],
            DIALOG_PRESS_FOR_A_TIP_FAILURE = [[
                agent:
                    I consider it your job to provide good service.
                    !exit
                * {agent} leaves with {oshnu}.
                lifter:
                    !right
                    !shrug
                    Don't take it personally. 
            ]],

            OPT_SAY_GOODBYE = "Bid {agent} farewell",
            DIALOG_SAY_GOODBYE = [[
                player:
                    !happy
                    Glad to be of service.
                agent:
                    !exit
                * {agent} takes {oshnu} and leaves.
                lifter:
                    !right
                    These rich arseholes are all the same.
                    See you around, grifter.
                    !exit
            ]],
        }
        :Fn(function(cxt) 
            cxt:TalkTo("chosen_buyer")
            if cxt.quest:GetCastMember("oshnu"):IsDead() or cxt.quest:GetCastMember("lifter"):IsDead() then
                
                cxt:Dialog(cxt.quest:GetCastMember("oshnu"):IsDead() and "DIALOG_NO_SNAIL" or "DIALOG_NO_LIFTER")
                cxt:Opt("OPT_PROTEST")
                    :Dialog("DIALOG_PROTEST")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_PROTEST_SUCCESS")
                            :CompleteQuest("deliver_goods")
                            :Travel()
                        :OnFailure()
                            :Dialog("DIALOG_PROTEST_FAILURE")
                            :CompleteQuest("deliver_goods")
                            :Fn(function() cxt.quest.param.unhappy_buyer = true end)
                            :Travel()

                
                cxt:Opt("OPT_ACCEPT")
                    :Dialog("DIALOG_ACCEPT")
                    :CompleteQuest("deliver_goods")
                    :Fn(function() cxt.quest.param.unhappy_buyer = true end)
                    :Travel()
            else
                cxt:Dialog("DIALOG_ARRIVE")

                cxt:Opt("OPT_PRESS_FOR_A_TIP")
                    :Dialog("DIALOG_PRESS_FOR_A_TIP")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_PRESS_FOR_A_TIP_SUCCESS")
                            :ReceiveMoney(10)
                            :CompleteQuest("deliver_goods")
                            :Travel()
                        :OnFailure()
                            :Dialog("DIALOG_PRESS_FOR_A_TIP_FAILURE")
                            :CompleteQuest("deliver_goods")
                            :Travel()


                cxt:Opt("OPT_SAY_GOODBYE")
                    :Dialog("DIALOG_SAY_GOODBYE")
                    :CompleteQuest("deliver_goods")
                    :Travel()
            end
        end)




QDEF:AddConvo("return_to_denikus", "denikus")
    :Loc{

        OPT_GET_PAID = "Tell {denikus} about the deal",
        DIALOG_PAID = [[
            player:
                The deal is done.

            {unhappy_buyer?
                agent:
                    Yeah, I heard from {chosen_buyer}.
                    {chosen_buyer.HeShe} isn't happy.
                    But {chosen_buyer.heshe} still paid. I guess that's good enough.
            }

            {not unhappy_buyer and (got_good_price_from_seller or got_good_price_from_buyer)?
                agent:
                    And you did a good job with those negotiations. You're a natural at this!
                    Here's your cut.
            }

            {not unhappy_buyer and not (got_good_price_from_seller or got_good_price_from_buyer)?
                agent:
                    You didn't exactly drive a hard bargain with {chosen_buyer} or {chosen_seller}, did you?
                    I'll make it up on volume, but I'm not giving you an extra cut for that performance.
            }

        ]],

        DIALOG_BONUS = [[
            agent:
                !give
            * {agent} gives you {1#money} as a bonus.
        ]]
    }
    :Hub(function(cxt) 
    
        --should get a cut, which is bigger, the more you succeeded in negotiating with the buyer and the seller.

        cxt:Opt("OPT_GET_PAID")
            :SetQuestMark()
            :PreIcon( global_images.hire )
            :Dialog("DIALOG_PAID")
            :Fn(function() 
                local bonus = 0
                if not cxt.quest.param.unhappy_buyer then
                    if cxt.quest.param.got_good_price_from_buyer then
                        bonus = bonus + 100 
                    end
                    if cxt.quest.param.got_good_price_from_seller then
                        bonus = bonus + 100 
                    end
                end
                ConvoUtil.GiveQuestRewards(cxt, cxt.quest, bonus > 0 and "DIALOG_BONUS" or nil) 

            end)
            :CompleteQuest()
  

    end)


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}



    

