local QDEF = QuestDef.Define{
        title = "Deep Undercover",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        icon = engine.asset.Texture("icons/quests/side_smith_eden_deep_undercover.tex"),
        focus = QUEST_FOCUS.BALANCED,
        tags = {"EDEN_JOB"},
        act_filter = "SMITH",
        --disabled = true,

        on_debug_start = function(quest)
            quest:GetCastMember("eden"):MoveToLocation(TheGame:GetGameState():GetLocation("MOREEF_BAR"))
            TheGame:GetGameState():GetCaravan():MoveToLocation(quest:GetCastMember("eden"):GetLocation())
            UIHelpers.DoSpecificConvo( quest:GetCastMember("eden"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
        end,

        collect_agent_locations = function(quest, t)
            --table.insert (t, { agent = quest:GetCastMember("assassin"), location = quest:IsActive("meet_assassin_post_kill") and quest:GetCastMember("cliffside") or quest:GetCastMember("park"), role = CHARACTER_ROLES.VISITOR} )
        end,

        is_in_hiding = function(quest, agent)
            return agent == quest:GetCastMember("target")
        end,

        avoids = {
        },

        on_destroy = function(quest) 
            --table.arrayfunction(members, function(cast_id) if quest:GetCastMember(cast_id) and quest:GetCastMember(cast_id):GetParty() then quest:GetCastMember(cast_id):Dismiss() end end)
        end,


        on_post_load = function(quest)
            if quest:IsActive('start') and not quest:GetCastMember("hat_prop") then
                if quest:GetCastMember("shop"):GetProprietor() then
                    quest:AssignCastMember("hat_prop", quest:GetCastMember("shop"):GetProprietor())
                else
                    quest:Fail()
                end
            end
        end,


        process_fighter = function(quest, fighter)
            --[[if table.any(members, function(id) return fighter.agent == quest:GetCastAgent(id) end) then
                fighter:AddCondition("WANTED_ALIVE")
            end--]]
        end,    
    }

    :AddCastByAlias{
        cast_id = "eden",
        alias = "EDEN",
        provider = true,
        no_validation = true,
    }

    :AddCastByAlias{
        cast_id = "seemli",
        alias = "SEEMLI",
        no_validation = true,
    }

    :AddCastByAlias{
        cast_id = "mullifee",
        alias = "MULLIFEE",
        no_validation = true,
    }

    :AddObjective{
        id = "start",
        title = "Investigate the hat shop",
        desc = "Someone at the hat shop knows something about the Rise. Extract that information.",
        
        mark = {"hat_prop"},

        on_activate = function(quest) 
            quest:GetCastMember("shop"):DoHiring()
            quest:AssignCastMember("hat_prop", quest:GetCastMember("shop"):GetProprietor())
        end,

        on_complete = function(quest)
            quest:Activate("go_to_beach")
        end,
        
        on_fail = function(quest)
        end,
    }

    :AddObjective{
        id = "return_to_eden",
        title = "Return to {eden}",
        desc = "Tell {eden} what happened.",
        mark = {"eden"},
    }
    
    :AddObjective{
        id = "go_to_beach",
        title = "Collect your 'hat'",
        desc = "The front told you to collect your special hat at the beach. You're starting to think there may not be an actual hat on offer, here.",
        mark = {"the_beach"},
        on_complete = function(quest)
            quest:Activate("kill_target")
        end,
    }

    :AddObjective{
        id = "kill_target",
        title = "Kill {target}",
        desc = "If you want to meet the leader of the Rise and find their secret headquarters, you're going to have to kill {target}, or at least make it look like you did.",
        mark = {"target_spot"},
        on_complete = function(quest)
            quest:Activate("infiltrate_the_hideout")
        end,
    }

    :AddObjective{
        id = "infiltrate_the_hideout",
        title = "Meet {seemli}",
        desc = "Time to meet {seemli} and officially join the Rise.",
        mark = {"clubhouse"},
        on_complete = function(quest)
            quest:Activate("return_to_eden")
        end,
    }

    :AddObjective{
        id = "tell_fail",
        title = "Return to {eden}",
        desc = "Tell {eden} what happened.",
        mark = {"eden"},
    }


    :AddQuestLocation{
        cast_id = "clubhouse",
        name = "Rise Headquarters",
        plax = "INT_SMITH_RISE_CLUBHOUSE",
        show_agents = true,
        tags = {"building"},
        indoors = true,
    }

    :AddQuestLocation{
        cast_id = "target_spot",
        name = "Patrol Point",
        plax = "EXT_SMITH_Nightlife_Street",
        show_agents = true,
        tags = {"street"},
    }

    :AddExistingLocation("shop", "PEARL_HAT_SHOP")
    :AddDefCastSpawn("rise_scout", "RISE_RADICAL")
    :AddDefCastSpawn("target", "ADMIRALTY_PATROL_LEADER")
    
    
    :AddCast{
        cast_id = "hat_prop",
        when = QWHEN.MANUAL,
        no_validation = true,
    } 

    :AddQuestLocation{
        cast_id = "the_beach",
        name = "Rise Beach Party",
        plax = "EXT_SMITH_BEACHCREATURE",
        map_tags = {"beach"},
    }


------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------

    :AddOpinionEvents{
        suspicious = {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "They think you're trying to infiltrate the Rise",
        },
        hostile = {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "They know you were trying to infiltrate the Rise",
        }
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------



QDEF:AddIntro(
    --attract spiel
    [[
        eden:
            It's hardly a secret that the Rise are gaining traction here in Pearl-on-the-Foam.
            Technically, {mullifee} has been assigned the case of sussing it out. 
            But I'm not in the business of letting her take credit for anything.
        player:
            Yeah, neither was Vix.
        eden:
            I'm sure these names you say are meaningful to someone, but not me. Keep your personal life to yourself.
            At any rate, I want you to infiltrate the cell and confirm their headquarters.
    ]],
    
    --on accept
    [[
        agent:
            Doubtless you'll need to break a law or two in your quest to blend in.
            I don't really care. Keep my name out of it, and we'll both be happy.
        player:
            You got a lead for me in addition to all this love and care?
        agent:
            The Fancy Millinery is reportedly lousy with discontented servants. 
            Likely it's a front.
            Buy some hats, get some answers.
            If you see the leader, {seemli}, then you know you've found the headquarters.
    ]],
    "start")



QDEF:AddConvo("start", "hat_prop")
    :Loc{
        OPT_HATS = "Ask about hats",
        DIALOG_HATS = [[
            player:
                Wow. A whole shop, just for hats?
            agent:
                And veils. And occasionally novelty fascinators, when they're in season.
            player:
                When are they <i>not</i>?
            agent:
                Quite regularly. 
            player:
                Cool.
                Not much of a hat person, myself.
                On account of the eye stalks.
            agent:
                I see. Then what brings you here?
            player:
                A guy can dream, right?
        ]],
        OPT_ASK_ABOUT_THE_RISE = "Ask {agent} about the Rise",
        DIALOG_ASK_ABOUT_THE_RISE = [[
            {first_time?
                player:
                    So I hear you might know something about the Rise here in the Pearl.
                agent:
                    !suspicious
                    $miscHushedUrgent
                    Keep your voice down!
                    !crossed
                    The Rise is a dangerous criminal organization! I don't know anything about that!
                player:
                    It's like that, is it?
                    !nudge_nudge
                    What if I had a lot of money to spend on a hat? You got any <i>special</i> hats?
                    Ones that might fit a rebel like me?
                agent:
                    Well... I suppose I might have a <i>special</i> hat that would fit you.
                    Let's just say it helps you to "Rise" to any occasion.
            }
            
            {not first_time?
                player:
                    My "friend" is still interested in learning more about a certain dangerous criminal organization.
                agent:
                    Look, are you going to buy the special hat or not?
            }
        ]],

        OPT_BUY_THE_SPECIAL_HAT = "Buy the special hat",
        DIALOG_BUY_THE_SPECIAL_HAT = [[
            player:
                Alright. I'll buy this special hat.
                Better whistle a tune and make my head look less fat for what I'm paying.
                !give
            agent:
                No guarantees. 
            player:
                ...
                So where's the hat?
            agent:
                It's not here. You pick it up at the beach.
            player:
                This all sounds very suspicious.
            agent:
                !dubious
                Yes, just like every other part of this conversation, since you stepped through that door.
        ]],

        OPT_LET_IN = "Convince {agent} to give you the special hat",
        DIALOG_LET_IN = [[
            player:
                I don't really have that kind of scratch right now.
                How about you just give me that hat, and I pay you back later?
        ]],

        DIALOG_LET_IN_SUCCESS = [[
            player:
                Sometimes the hat needs the head more than the head needs the hat.
                You get what I'm saying?
                !thumb
                This head is going places. The hat might wanna hitch a ride.
            agent:
                Fine. You can pick up the <i>hat</i> on the beach.
                But you'll have to work out a payment schedule with my associates.
            player:
                !dismiss
                $neutralWhatever
                I'm sure I'll be fine. I always am!
                !point
                Catch you on the flippitty-flip!
        ]],
        DIALOG_LET_IN_FAILURE = [[
            player:
                What's the point of a hat nobody can wear, anyway?
            agent:
                !crossed    
                The whole point of the hat is fundraising.
                No funds?
                No hat.
        ]],

        OPT_FORCE_TELL = "Force {agent} to give you the hat",
        DIALOG_FORCE_TELL = [[
            player:
                You're going to give me that hat!
        ]],
        DIALOG_KILLED_LEAD = [[
            * {agent} is dead, and you don't know which hat is the special one.
            * You're going to have to tell {eden} you killed {eden.hisher} only lead.
        ]],
        DIALOG_WON = [[
            agent:
                !scared
                Stop! Please!
                There isn't actually a literal hat! It's all just cloak and dagger nonsense.
                The Rise vets new recruits down at the beach. I can draw you a map!
            player:
                !happy
                Much obliged. 
                Although I am sad that there's no real hat.
                !point
                That's false advertising!
        ]],

    }
    :Hub(function(cxt) 
    
        if cxt:GetAgent() == cxt.quest:GetCastMember("shop"):GetProprietor() then
            cxt:Opt("OPT_ASK_ABOUT_THE_RISE")
                :Dialog("DIALOG_ASK_ABOUT_THE_RISE")
                :LoopingFn(function() 

                    cxt:Opt("OPT_BUY_THE_SPECIAL_HAT")
                        :Dialog("DIALOG_BUY_THE_SPECIAL_HAT")
                        :DeliverMoney(300, {is_shop = true})
                        :CompleteQuest("start")
                        :Fn(function() 
                            cxt.quest.param.bought_hat = true
                        end)
                        :DoneConvo()

                    cxt:BasicNegotiation("LET_IN")
                        :OnSuccess()
                            :CompleteQuest("start")
                            :Fn(function() 
                                cxt.quest.param.talked_way_in = true
                            end)
                            :DoneConvo()

                    cxt:Opt("OPT_FORCE_TELL")
                        :Dialog("DIALOG_FORCE_TELL")
                        :Battle{
                            on_start_battle = function(battle) 
                                local fighter = battle:GetFighterForAgent(cxt:GetAgent())
                                if fighter then
                                    fighter:AddCondition("WANTED_ALIVE")
                                end
                            end

                        }
                            :OnWin()
                                :Fn(function() 
                                    if cxt:GetAgent():IsDead() then
                                        cxt:Dialog("DIALOG_KILLED_LEAD")
                                        cxt.quest:Fail("start")
                                        cxt.quest:Activate("tell_fail")
                                    else
                                        cxt.quest:Complete("start")
                                        cxt:Dialog("DIALOG_WON")
                                        cxt.quest.param.fought_way_in = true

                                    end
                                end)
                                :DoneConvo()
                    
                    StateGraphUtil.AddBackButton(cxt)
                end)
        end

    end)


QDEF:AddConvo("go_to_beach")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("the_beach") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find a lone Rise waiting for you at the beach.
                agent:
                    !right
                player:
                    !left
                    I was told there would be a hat here for me.
                {has_prop?
                    agent:
                        Oh, you're the one {prop} was talking about.
                    player:
                        Yeah, sure.
                        But where's my hat?
                }
                agent:
                    !cruel
                    Oh, I've got a hat for you, grifter.
                    A real fancy one!
                    !exit
                * {agent} steps back, and fiddles with a small control box.
                * A shambling figure rises out of the sand!
            ]],
            DIALOG_ROBOT_INTRO = [[
                agent:
                    !right
                    COMBAT PROTOCOL ACTIVATED
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    It's not even wearing a damn hat. 
                    I think this shop of yours might be a front!
            ]],
            
            DIALOG_WON = [[
                * The automaton collapses into a pile of broken parts.
                agent:
                    !right
                    !happy
                    Congratulations!
                player:
                    !neutral
                    Thanks! What for?
                agent:
                    The report was that you weren't very wealthy, nor were you good at subterfuge.
                    This proves, at least, that you know how to fight. So it seems the Rise could use you after all.
            ]],

            OPT_ROBOT = "Ask about the robot",
            DIALOG_ROBOT = [[
                player:
                    What was that thing?
                agent:
                    Oh, you mean Shambles? It's an automech, smuggled out of Grout Bog.
                player:
                    Did I... kill it?
                agent:
                    Oh, no! I'll fix it up, good as new and ready for the next recruit looking for a hat.
                player:
                    Like flies to honey. Poor sods.
            ]],

            OPT_HAT = "Ask for the hat",
            DIALOG_HAT = [[
                player:
                    !handwring
                    I was promised a hat.
                agent:
                    !dubious
                    You're not the fastest brick in the shed, are you?
                player:
                    !happy
                    Where's my hat?
                agent:
                    There's no hat, man.
                player:
                    !neutral
                agent:
                    !shrug
                    Sorry.
            ]],

            OPT_RISE = "Ask about the Rise",
            DIALOG_RISE = [[
                player:
                    What's the Rise doing here in the Pearl?
                    Doesn't strike me as your natural habitat.
                agent:
                    Yeah, there's a lot of money here.
                    But who do you think keeps all of these fancy buildings clean?
                    Who keeps the plumbing working? Who keeps the lights on?
                    !point
                    Workers, man.
                    And we're going to teach them what their real power is.
            ]],

            OPT_CONTINUE = "Ask about the next step",
            DIALOG_CONTINUE = [[
                player:
                    So now that I'm a real, certified member of the Rise, what happens next?
                    You have any secret handshakes or bank account numbers you want to give me?
                agent:
                    Oh, you ain't Rise yet, buddy.
                    You've just completed the first step.
                player:
                    I was afraid you might say that.
                agent:
                    Yeah, now that we know you can fight, we're going to put that to use.
                    If you want to meet our leader and join the club for real, you have to take out an Admiralty, first.
                player:
                    Just any Admiralty?
                agent:
                    No, that'd be too easy. We need you to take down a specific one.
                    Let's say, {target}. {target.HeShe}'s out on patrol right now. Should be easy to find.
                    Kill {target}, and then you can meet the Boss.
                player:
                    Geez, you guys don't mess around!
                agent:
                    !exit
                * {agent} busies {agent.himher}self fixing the automech.                    
            ]]
        }
        :Fn(function(cxt) 
            cxt:ReassignCastMember( "prop", cxt.quest:GetCastMember("shop"):GetProprietor())
            cxt.quest.param.has_prop = cxt:GetCastMember("prop") ~= nil


            cxt:TalkTo("rise_scout")            
            cxt:Dialog("DIALOG_INTRO")

            local robot = TheGame:GetGameState():AddSkinnedAgent( "RISE_AUTOMECH" )

            cxt:ReassignCastMember("robot", robot)
            cxt:TalkTo(robot)
            cxt:Dialog("DIALOG_ROBOT_INTRO")
            
            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.SELF_DEFENCE,
                }
                    :OnWin()
                        :Fn(function(cxt)  
                            cxt:TalkTo("rise_scout")
                            cxt:Dialog("DIALOG_WON")
                            cxt:RunLoop(function() 

                                cxt:QST("ROBOT")
                                cxt:QST("HAT")
                                cxt:QST("RISE")

                                cxt:Opt("OPT_CONTINUE")
                                    :SetQuestMark()
                                    :Dialog("DIALOG_CONTINUE")
                                    :CompleteQuest("go_to_beach")
                                    :Travel()
                            end)

                        end)
        end)



QDEF:AddConvo("kill_target")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("target_spot") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find {target}, training some vroc.
                vroc:
                    !left
                target:
                    !right
                    Sit!
                    !placate
                    Stay!
                vroc:
                    !exit
                target:
                    !hips
                    I said "Stay"!
                * The vroc smell you coming and bound around your feet.
                player:
                    !happy
                    !left
                    Down, puppies!
                target:
                    Hey, those are working animals. 
                    Please don't distract them!
            ]],

            OPT_CONVINCE = "Convince {agent} to fake {agent.hisher} own death",
            DIALOG_CONVINCE = [[
                player: 
                    So I'm not supposed to tell anyone, but...
                    I'm working undercover!
                    And I need your help!
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    ...and that's why I need you to fake your own death.
                    !point
                    Just for a little bit!
                agent:
                    Hey, sounds fun.
                    I can take a nap in the kennels, then make up a story about some fugue state when it's done.
                player:
                    Everyone should treat themselves to a fugue state now and again, I always say.
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    It's not even that hard. I've faked my own death loads of times.
                agent:
                    Wait. Lemme get this straight.
                    The Admiralty sent you to infiltrate the Rise, and you need me to take the fall as part of your initiation?
                player:
                    Just a fake fall. Like... a stumble. A stagger, at least. 
                agent:
                    ...
                    !dubious
                    I'm an actual Admiralty officer.
                    With actual orders. That I actually intend to follow.
                    !dismiss
                    Get out of here, grifter.
            ]],

            OPT_ATTACK = "Attack {agent}",
            DIALOG_ATTACK = [[
                player:
                    Sorry it came to this, but...
                    !fight
                agent:
                    Huh?!
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead. The Rise will have no reason to question your loyalty now.
                }
                {not dead?
                    agent:
                        !injured
                    player:
                        So here's how this is going to work.
                        You're going to play dead, so I don't have to kill you.
                    agent:
                        Fine. Sure! I'll do it!
                    player:
                        !happy
                        Now why didn't we start with that attitude?
                }
            ]],

            DIALOG_OFFER_PET = [[
                agent:
                    Hey, do you want to buy one of these Vrocs?
                player:
                    Aren't they Admiralty property?
                agent:
                    Naw. Only the best and brightest make it to squad.
                    The rejects we just mince into meat for the others.
                player:
                    Whoa, really?!
                agent:
                    I mean. Metaphorically.
                    We sell them, and the money goes towards the upkeep for the others.
                player:
                    Okay. Because that was... very misleading.
            ]],
            OPT_TAKE_PET = "Buy the vroc",
            OPT_LEAVE_PET = "Leave it",
            DIALOG_TAKE_PET = [[
                player:
                    Yeah, sure. I need someone to make me look smart.
                vroc:
                    !right
                * The vroc happily follows you.
            ]],
            DIALOG_LEAVE_PET = [[
                player:
                    !refuse
                    I'm a lone wolf who plays by his own rules.
                    There's only room in this cockpit for one pilot.
                    Kra'deshi got 360-degree vision. I watch my own back.
                agent:
                    Okay, one trite aphorism was enough, thanks.
                    Forget I asked. I'll just see myself into hiding now.
                    !exit
            ]],
        }
        :Fn(function(cxt) 
            cxt:TalkTo("target")

            local vroc1 = TheGame:GetGameState():AddSkinnedAgent("VROC")
            local vroc2 = TheGame:GetGameState():AddSkinnedAgent("VROC")
            local vroc3 = TheGame:GetGameState():AddSkinnedAgent("VROC")

            vroc1:GenerateRandomName()
            
            local opfor = {cxt:GetAgent(), vroc1, vroc2, vroc3}
            cxt:ReassignCastMember("vroc", vroc1)


            cxt:TalkTo("target")            
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("kill_target")
                        :Fn(function() 
                            if cxt.caravan:CanOwnPet() then
                                cxt:Dialog("DIALOG_OFFER_PET")
                                
                                cxt:Opt("OPT_TAKE_PET")
                                    :DeliverMoney(200)
                                    :Dialog("DIALOG_TAKE_PET")
                                    :RecruitMember(PARTY_MEMBER_TYPE.CREW, vroc1 )
                                    :Travel()

                                cxt:Opt("OPT_LEAVE_PET")
                                    :Dialog("DIALOG_LEAVE_PET")
                                    :Travel()
                            else
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end)
                        

                cxt:Opt("OPT_ATTACK")   
                    :Dialog("DIALOG_ATTACK")
                    :Battle{enemies = opfor}
                        :OnWin()
                            :CompleteQuest("kill_target")
                            :Fn(function() 
                                cxt:Dialog("DIALOG_WON")
                                cxt.quest.param.killed_admiralty = cxt:GetAgent():IsDead()
                            end)
                            :Travel()
            end)
        end)


QDEF:AddConvo("infiltrate_the_hideout")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("clubhouse") end)
        :Loc{
            DIALOG_INTRO = [[
                * You follow {rise_scout}'s directions to the Rise hideout.
                player:
                    !left
                    Knock knock?
                seemli:
                    !right
                    You must be the new recruit I've heard so much about.
                    I did feel something in the wind today during my morning meditations.
                    A portent of things to come.
                player:
                    !happy
                    Oh, yeah. I have good things come out of my port end sometimes, too.
                
                {killed_admiralty?
                    seemli:
                        Yes.
                        {target}'s body was found, murdered most foul.
                        Some might take that as a sign of your dedication to the cause.
                    player:
                        Nice.
                    seemli:
                        But I'm not so sure. Convince me.
                }

                {not killed_admiralty?
                    seemli:
                        Reports of {target}'s death seem, perhaps, exaggerated. 
                        I can't work with a man I can't trust.
                        And if that man is a spy, well... I can't abide him, either.
                    player:
                        Is that all a coy way of saying you wanna kill me?
                    seemli:
                        !dismiss
                        Yes, I'm afraid so.
                        !exit
                    * You are set upon by Rise guards. 
                }
            ]],

            OPT_CONVINCE = "Convince {agent} you are loyal to the Rise",
            DIALOG_CONVINCE = [[
                player:
                    Look, is this the face of a status quo-enforcer?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    I've never accepted things as they are once in my whole life.
                    My name might be Banquod, but I'm all about upending tables.
                agent:
                    Hm. Well put.
                    And welcome to the Rise.
                    !point
                    Now get out of here, and await further instructions.
                    !exit
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    I even got disowned by my own parents!
                agent:
                    Yes, and that's sort of the problem, isn't it?
                    You're only here because you've been dealt a bad hand this round.
                    But the Pearlian privilege that runs in your veins will give you opportunities aplenty.
                    You aren't ready to join us. Not yet.
                player:
                    H-hey, my life ain't so easy, ya know!
                agent:
                    I do. But that's not what this is about.
                * {agent} signals to {agent.hisher} guards, and they seize you.
                agent:
                    Nothing personal. In fact, I rather like your aura.
                    But don't come back.
                player:
                    !exit
                * The guards manhandle you something fierce, and toss you out.
            ]],
            
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Well, alright.
            ]],
            DIALOG_WIN = [[
                * There's a lull in the battle, and you manage to escape.
            ]]
        }
        :Fn(function(cxt) 
            cxt:TalkTo("seemli")
            cxt:Dialog("DIALOG_INTRO")
            if cxt.quest.param.killed_admiralty then

                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("infiltrate_the_hideout") 
                        :Travel()
                    
                    :OnFailure()
                        :CompleteQuest("infiltrate_the_hideout") 
                        :DeltaHealth( -20 )
                        :Fn(function() 
                            cxt:Opinion("suspicious", "seemli")
                            cxt.quest.param.suspicious = true
                            local runaway_loc = TheGame:GetGameState():AddLocation( Location( TheGame:GetGameState():GetWorldRegion():GetOutdoorLocationID() ) )
                            runaway_loc:SetFlags( LOCFLAGS.TRANSIENT )
                            cxt.encounter:DoLocationTransition( runaway_loc )
                        end)
                        :Travel()

            else
                local opfor = CreateCombatParty({"RISE_RADICAL", "RISE_REBEL"}, cxt.quest:GetRank(), cxt.location) 
                cxt:Opinion("hostile", "seemli")
                cxt.quest.param.suspicious = true
                
                cxt:Opt("OPT_DEFEND")
                    :Battle{
                        enemies = opfor,
                        flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.SELF_DEFENCE,    
                    }
                        :OnWin()
                            :Dialog("DIALOG_WIN")
                            :CompleteQuest("infiltrate_the_hideout")
                            :Travel()
            end
        end)


QDEF:AddConvo("return_to_eden", "eden")
    :Loc{

        OPT_GIVE_REPORT = "Tell {agent} about the cell",
        DIALOG_GIVE_REPORT = [[
            player:
                I can confirm that {seemli} is at the Rise hideout.
            
            {suspicious?
                player:
                    But {seemli.heshe} probably won't be there for long. I... wasn't very good at being undercover.
                agent:
                    Alright, we'll have to move fast then.
            }
            {not suspicious?
                player:
                    {seemli.HeShe} didn't suspect a thing!
                agent:
                    Surprisingly excellent work, grifter.
            }
            
            agent:
                What did you have to do to convince them?

            {killed_admiralty?
                player:
                    Were you friends with {target}?
                agent:
                    The vroc trainer? Not particularly.
                player:
                    Good.
            }
            {not killed_admiralty?
                player:
                    They wanted me to kill {target}, but I just faked it.
                agent:
                    Eh. The intelligence was more important than a vroc trainer.
                player:
                    Remind me never to join the Admiralty.
            }
        ]],
    }
    :Hub(function(cxt)
        
        cxt:Opt("OPT_GIVE_REPORT")
            :SetQuestMark()
            :Fn(function() 
                cxt:Dialog("DIALOG_GIVE_REPORT")
                ConvoUtil.GiveQuestRewards(cxt)
                cxt.quest:Complete()
            end)
    end)


QDEF:AddConvo("tell_fail", "eden")
    :Loc{

        OPT_GIVE_REPORT = "Tell {agent} you messed up",
        DIALOG_GIVE_REPORT = [[
            player:
                So. The Rise.
            agent:  
                Have you confirmed their situation?
            player:
                Er... no.
                Actually I may have messed up entirely.
            agent:
                !facepalm
                Why do I even try?
        ]],
    }
    :Hub(function(cxt)
        
        cxt:Opt("OPT_GIVE_REPORT")
            :SetQuestMark()
            :Fn(function() 
                cxt:Dialog("DIALOG_GIVE_REPORT")
                cxt.quest:Fail()
            end)
    end)

QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}



    



