local targets = {
    "lifter",
    "pearlie",
    "wrangler",
    "hesh",
}
local function AddClue(quest)
    if not quest.param.uncovered_clerk then
        quest.param.clues = (quest.param.clues or 0) + 1
        if quest.param.clues >= 3 then
            quest.param.uncovered_clerk = true
            quest:Activate("confront_clerk")
        end
    end
end

local QDEF = QuestDef.Define{
        title = "From the Top",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{eden} wants you to clamp down on Rise sympathizers in Pearl-On-Foam.]],
        --disabled = true,
        icon = engine.asset.Texture("icons/quests/side_smith_eden_from_the_top.tex"),
        focus = QUEST_FOCUS.BALANCED,
        tags = {"EDEN_JOB"},
        act_filter = "SMITH",
        --disabled = true,

        on_debug_start = function(quest)
            quest:GetCastMember("eden"):MoveToLocation(TheGame:GetGameState():GetLocation("MOREEF_BAR"))
            TheGame:GetGameState():GetCaravan():MoveToLocation(quest:GetCastMember("eden"):GetLocation())
            UIHelpers.DoSpecificConvo( quest:GetCastMember("eden"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
        end,

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("pearlie"), location = quest:GetCastMember("shop"), role = CHARACTER_ROLES.VISITOR} )
            table.insert (t, { agent = quest:GetCastMember("wrangler"), location = quest:GetCastMember("snail_pen"), role = CHARACTER_ROLES.VISITOR} )
            table.insert (t, { agent = quest:GetCastMember("hesh"), location = quest:GetCastMember("corner"), role = CHARACTER_ROLES.VISITOR} )
            table.insert (t, { agent = quest:GetCastMember("lifter"), location = quest:GetCastMember("dock"), role = CHARACTER_ROLES.VISITOR} )

            if quest:IsActive("confront_clerk") then
                table.insert (t, { agent = quest:GetCastMember("clerk"):GetHomeLocation(), location = quest:GetCastMember("clerk"), role = CHARACTER_ROLES.VISITOR} )
            end
        end,

        is_in_hiding = function(quest, agent)
            return not quest:IsActive("confront_clerk") and agent == quest:GetCastMember("clerk")
        end,

        avoids = {
        },

        on_destroy = function(quest) 
            --table.arrayfunction(members, function(cast_id) if quest:GetCastMember(cast_id) and quest:GetCastMember(cast_id):GetParty() then quest:GetCastMember(cast_id):Dismiss() end end)
        end,
        
        process_fighter = function(quest, fighter)
            --[[if table.any(members, function(id) return fighter.agent == quest:GetCastAgent(id) end) then
                fighter:AddCondition("WANTED_ALIVE")
            end--]]
        end,    
    }

    :AddCastByAlias{
        cast_id = "eden",
        alias = "EDEN",
        provider = true,
        no_validation = true,
    }

    :AddObjective{
        id = "return_to_eden",
        title = "Return to {eden}",
        desc = "Tell {eden} what happened.",
        mark = {"eden"},
    }

    :AddObjective{
        id = "start",
        --title = "",
        --desc = "Tell {eden} what happened.",
        --mark = {"eden"},
        on_activate = function(quest)
            local objectives = Iter.array(targets):map(function(v) return "example_".. v end):shuffle()
            quest.param.not_objective = objectives:next()
            quest.param.objectives = {}
            objectives:for_each(function(obj) 
                table.insert(quest.param.objectives, obj)
                quest:Activate(obj)
            end)
            quest.param.clues = 0
            quest.param.examples = 0
            
        end,

        events = {
            quests_changed = function(quest, event_quest) 
                if event_quest == quest and quest.param.objectives then
                    local all_done = Iter.array(quest.param.objectives):all(function(obj) return quest:IsComplete(obj) or quest:IsFailed(obj) end)
                    if all_done then
                        quest:Complete("start")
                    end
                end
            end           
        },

        on_complete = function(quest)
            quest:Activate("return_to_eden")
        end,
    }
    
    :AddObjective{
        id = "example_lifter",
        title = "Confront {lifter}",
        desc = "{lifter} is a member of the Jakes. You'll find {lifter.himher} working at the docks.",
        mark = {"lifter"},
    }
    
    :AddObjective{
        id = "example_pearlie",
        title = "Confront {pearlie}",
        desc = "{pearlie} is a well-to-do civilian. {pearlie.HeShe} is currently shopping.",
        mark = {"pearlie"},
    }

    :AddObjective{
        id = "example_hesh",
        title = "Confront {hesh}",
        desc = "{hesh} is a low-level member of the Cult. They're spreading the word in town.",
        mark = {"hesh"},
    }

    :AddObjective{
        id = "example_wrangler",
        title = "Confront {wrangler}",
        desc = "{wrangler} is an Oshnu Wrangler. You'll find {wrangler.himher} wrangling Oshnus.",
        mark = {"wrangler"},
    }

    :AddObjective{
        id = "confront_clerk",
        title = "(Optional) Confront {clerk}",
        desc = "You've determined that {clerk} is working with the Rise. That could prove to be a lucrative secret.",
        mark = {"clerk"},
        on_activate = function(quest) 
            quest:GetCastMember("clerk"):GetBrain():MoveToHome()
        end,
    }


--[[
    :AddQuestLocation{
        cast_id = "clubhouse",
        name = "Rise Headquarters",
        plax = "INT_SMITH_RISE_CLUBHOUSE",
        show_agents = true,
        tags = {"building"},
        indoors = true,
    }
    :AddExistingLocation("shop", "PEARL_HAT_SHOP")
    :AddDefCastSpawn("rise_scout", "RISE_RADICAL")
--]]
    
    :AddDefCastSpawn("wrangler", "OSHNU_WRANGLER")
    :AddDefCastSpawn("pearlie", "PEARLIE")
    :AddDefCastSpawn("hesh", "LUMINITIATE")
    :AddDefCastSpawn("lifter", "JAKES_LIFTER")
    :AddDefCastSpawn("clerk", "ADMIRALTY_CLERK")


    :AddExistingLocation("shop", "PEARL_HAT_SHOP")
    
    :AddQuestLocation{
        cast_id = "snail_pen",
        name = "Oshnu Pen",
        desc = "{wrangler}'s place of work.",
        plax = "INT_SMITH_SNAILPEN",
        map_tags = {"building"},
        indoors = true,
    }

    :AddQuestLocation{
        cast_id = "corner",
        name = "Street corner",
        desc = "{hesh} is preaching here.",
        plax = "EXT_SMITH_Nightlife_Street",
        show_agents = true,
        tags = {"street"},
    }

    :AddQuestLocation{
        cast_id = "dock",
        name = "Loading Dock",
        desc = "{lifter} is working a shift here.",
        plax = "EXT_SMITH_WARF",
        map_tags = {"port"},
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------

    :AddOpinionEvents{
        paid_for_info = {
            delta = OPINION_DELTAS.LIKE,
            txt = "Let them off with a warning",
        },
        extorted = {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Extorted them",
        },
        keep_secret = {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Promised to keep their secret",
        },
        betrayed = {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Betrayed them to the Admiralty",
        },
        uncovered_clerk = {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Uncovered a traitor for them",
        }
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddIntro(
    --attract spiel
    [[
        Eden:
            You've probably heard about the Rise sympathizers here in the Pearl.
            It's so typical of our domestics: aspire to grandeur but cast their lot with the debt-dodgers and contractors in the rest of Havaria.
        player:
            We got dibs on wanting a better life?
        eden:
            The only better life for them is in Deltree, and they're welcome to buy themselves a one-way ticket.
            I have a list of the suspects here. I want you to confront them, put an end to their plans.
            And make an example of them, for anyone who might be getting tetchy.
    ]],
    
    --on accept
    [[
        player:
            You got specifics? That was a lot of vague talk for some rough work.
        agent:
            These are domestics we're talking about. If any of their employers complain, we can offer recompense.
            Otherwise it's fair game. Treat them as you would a pest.
            I doubt any of them have information that I'd value, but if they do... well, that's just a bonus. 
        player:
            Okay, I guess.
    ]],
    "start")



QDEF:AddConvo("example_wrangler")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("wrangler"):GetLocation() end)
        :Loc{
            DIALOG_INTRO = [[
                * You find {agent} tending to a racing Oshnu.
                * There's nobody else here.
                agent:
                    !right
                    Hey! You're not allowed back here.
                player:
                    !left
                    I'll go, but first I have a couple of questions.
                agent:
                    !dubious
                    Make it quick, grifter. I have to get this oshnu ready for today's race.
            ]],
            OPT_RISE = "Ask about the Rise",
            DIALOG_RISE = [[
                player:
                    Word on the street is you're cozy with the Rise.
                agent:
                    !cagey
                    I- I don't know nothing about that!
                player:
                    !dubious
                    Yeah? We'll see.
            ]],

            OPT_CONVINCE = "Convince {agent} that {agent.heshe} is in danger",
            DIALOG_CONVINCE = [[
                player:
                    We both know you just wanna go back to living your life, right?
                    
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    This Rise business--you've gotten your fill and you want out.
                    So tell me who your boss is, and I'll swear up and down you're clean.
                agent:
                    I'm not even an official member of the Rise. I just make the snails faster or slower when I'm told.
                player:
                    And who's doing the asking?
                agent:
                    They just leave me notes. Anonymous ones.
                    Here, take this. I got it this morning.
                    !give
                player:
                    !take                   
                    Alright. Now keep your head down and ignore any other notes you might get.
                    Then the Admiralty won't have to send someone less friendly.
                agent:
                    Thank you!
                    !exit
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    This Rise business is too much work. Don't you wanna take it easy?
                agent:
                    You calling me lazy?
                    That's so typical of you noble-types. You think any discontent means we're just looking to loaf. 
                    I'm no snitch.
            ]],

            OPT_ATTACK = "Beat up {agent}",
            DIALOG_ATTACK = [[
                player:
                    I have a message for you from the Admiralty.
                agent:
                    What is it?
                player:
                    !fight
                    This.
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead. That's a pretty clear message.
                }
                {not dead?
                    agent:
                        !injured
                        Stop! Please!
                        I'm done with the Rise! I swear!
                    player:
                        Who was giving you orders?
                    agent:
                        I dunno. All I got was notes. Like this one.
                        !give
                    player:
                        !take
                        Alright. Now get out of my sight.
                    agent:
                        !exit
                }
            ]],
            DIALOG_GET_CLUE = [[
                * You inspect the note. It's written on the back of an old Admiralty requisition slip. 
                
                {uncovered_clerk?
                    player:
                        !eureka
                    * You put the clues together. {clerk} is working both sides.
                }

                {not uncovered_clerk?
                    player:
                        !thought
                    * The Rise might have someone working inside the Admiralty. Interesting.
                }
            ]]

        }
        :Fn(function(cxt)

            local snail = cxt.quest:CreateSkinnedAgent("RACING_SNAIL") 
            cxt:TalkTo("wrangler")
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                
                cxt:QST("RISE")
                
                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{
                            enemies = {cxt:GetAgent(), snail},
                        }
                        :OnWin()
                            :CompleteQuest("example_wrangler")
                            :Dialog("DIALOG_WON")
                            :Fn(function() 
                                cxt.quest.param.examples = cxt.quest.param.examples + 1
                                if cxt:GetAgent():IsAlive() then
                                    AddClue(cxt.quest)
                                    cxt:Dialog("DIALOG_GET_CLUE")
                                end
                            end)
                            :Travel()

    
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("example_wrangler")
                        :Fn(function() 
                            AddClue(cxt.quest)
                            cxt:Dialog("DIALOG_GET_CLUE")
                        end)
                        :Travel()

            end)
        end)


QDEF:AddConvo("example_pearlie")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("pearlie"):GetLocation() end)
        :Loc{
            DIALOG_INTRO = [[
                * You find {agent} doing some luxury shopping.
                agent:
                    !right
                player:
                    !left
                    How's it going, friend?
                agent:
                    !dubious
                    Are you talking to me?
                    Can you even afford anything in here?
            ]],
            OPT_RISE = "Ask about the Rise",
            DIALOG_RISE = [[
                player:
                    What would your Rise buddies say if they saw you throwing around this kind of scratch?
                agent:
                    !angry
                    The Rise?!
                    Do I <b>look</> like a member of the Rise to you?!
            ]],

            OPT_CONVINCE = "Convince {agent} that {agent.heshe} is in danger",
            DIALOG_CONVINCE = [[
                player:
                    Hey, I get the appeal of slumming it with the wretches. 
                    I've been doing it so long you wouldn't know I'm not wretched to look at me!
                    !shrug
                    But you really wanna live like the common people, bud? 'Cause my order is to beat the snot outta ya, if so.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    Speaking from experience: it gets old, sooner or later.
                    And then you come home and find out your family disowned you for bein' a tourist. 
                agent:
                    !sigh
                    Disowned? Really? My own family has been, ah, a little hostile, as of late.
                    Let's just say that I... sympathise. And that I acknowledge, given recent events in the Bread Fields...
                    Well, I'd prefer to be on the right side of history. 
                player:
                    !dubious
                    Buddy, I'm just trying to keep you on the right side of this very moment.
                    You got a name for your Rise contact or not?
                agent:
                    Just a codename. "LEDGER"
                player:
                    Great. Super. Now get outta here before I decide to get rough with you anyway.
                agent:
                    H-how dare you!
                player:
                    Easily, my bud. I darest most easily.
                agent:
                    !exit
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                agent:
                    You think it's all drunken shanties and lewd jokes, but then you got debts, and friends with debts, and friends with knives...
                    And not-friends with knives.
                player:
                    You know the secret to keeping your friends with knives friendly?
                    !crossed
                    Loyalty.  
            ]],

            OPT_ATTACK = "Beat up {agent}",
            DIALOG_ATTACK = [[
                player:
                    Alright, this should be easy.
                agent:
                    What should be easy?
                player:
                    !fight
                    This.
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead. The shop cleared out when the fight started, but you're sure your message was heard.
                }
                {not dead?
                    agent:
                        !injured
                        Alright! Enough!
                        I <b>AM</> a Rise agent.
                        A really important one, too!
                        I'm editor-in-chief of the weekly newsletter.
                    player:
                        Wait, you <i>write</i> their propaganda? 
                        That actually is pretty important.
                    agent:
                        Oh, n-no. I don't actually write it. 
                        I just, ah, check it for errors.
                        Typographical ones, mostly.
                        I get the manuscripts from my handler. I don't know their name, but their codename is "LEDGER".
                    player:
                        Well, they're going to have to deal with a couple of spelling mistakes in the next one, right?
                        'Cause you don't work for them anymore.
                    agent:
                        O-oh, right, sure, fine. Only... it's just a bit of a bugbear of mine--misplaced punctuation, I mean--and I really feel much better if I can--
                    player:
                        {agent}?
                        Beat it.
                    agent:
                        Ah. Yes, sir.
                        !exit
                }
            ]],
            DIALOG_GET_CLUE = [[
                * 'LEDGER' sounds like something administrative.
                
                {uncovered_clerk?
                    player:
                        !eureka
                    * It's clear now. {clerk} the clerk is working both sides.
                }

                {not uncovered_clerk?
                    player:
                        !thought
                    * That narrows it down a bit, but you don't have a positive ID yet.
                }
            ]],

            OPT_EXTORT = "Extort a payment from {agent}",
            DIALOG_EXTORT = [[
                player:
                    Listen, bud. I don't care one way or the other about the Admiralty and the Rise.
                    But I have been sent here to make an example out of you.
                    I'm getting paid to do it, so I could also be paid not to do it.
                    Whaddya say?
            ]],
            
            DIALOG_EXTORT_SUCCESS = [[
                agent:
                    Alright, alright.
                    Keep your voice down! I have a reputation to maintain.
                player:
                    Now tell me who your contact is.
                agent:
                    They never let me know their name, but their codename is "LEDGER".
                player:
                    That'll have to do. Now go back to your rich friends and keep your head down.
                agent:
                    ...
                    !exit

            ]],
            DIALOG_EXTORT_FAILURE = [[
                agent:
                    No! I'm not giving you anything!
            ]],
        }
        :Fn(function(cxt)

            cxt:TalkTo("pearlie")
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                cxt:QST("RISE")
                
                cxt:BasicNegotiation("EXTORT", {flags = NEGOTIATION_FLAGS.INTIMIDATION})
                    :ReceiveOpinion("extorted")
                    :OnSuccess()
                        :ReceiveMoney(100)
                        :CompleteQuest("example_pearlie")
                        :Fn(function() 
                            AddClue(cxt.quest)
                            cxt:Dialog("DIALOG_GET_CLUE")
                        end)
                        :Travel()

                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{}
                        :OnWin()
                            :CompleteQuest("example_pearlie")
                            :Dialog("DIALOG_WON")
                            :Fn(function() 
                                cxt.quest.param.examples = cxt.quest.param.examples + 1
                                if cxt:GetAgent():IsAlive() then
                                    AddClue(cxt.quest)
                                    cxt:Dialog("DIALOG_GET_CLUE")
                                end
                            end)
                            :Travel()

    
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("example_pearlie")
                        :Fn(function() 
                            AddClue(cxt.quest)
                            cxt:Dialog("DIALOG_GET_CLUE")
                        end)
                        :Travel()
            end)
        end)

QDEF:AddConvo("example_lifter")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("lifter"):GetLocation() end)
        :Loc{
            DIALOG_INTRO = [[
                * You find {agent} laughing with some coworkers beside a pile of unpacked freight.
                agent:
                    !right
                player:
                    !left
                    Hey, shouldn't you be working?
                agent:
                    We're on break, grifter.
                    What's your excuse?
            ]],
            OPT_RISE = "Ask about the Rise",
            DIALOG_RISE = [[
                player:
                    You might hurt the Rise's cause if folks see you leaning on your shovel like that. Just sayin'.
                agent:
                    !angry
                    As I said before, we're on break.
                    And why would I care what the Rise thinks?
            ]],

            OPT_CONVINCE = "Convince {agent} that {agent.heshe} is in danger",
            DIALOG_CONVINCE = [[
                player:
                    Listen, I get it. Lifting things is hard. S'why I rarely even lift a finger.
                    !shrug
                    But the Admiralty has your number, and they sent me to collect it. 
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    Whatever the Rise is giving you can't be worth the hassle, can it?
                    Just gimme a name and we can both forget all about it.
                agent:
                    Hrm. Fine. 
                    The Rise promise better pay in the future, but I can't buy food with promises.
                player: 
                    Yeah, yeah, I didn't ask for your life story. Just tell me who your contact is.
                agent:
                    Beats me. I just have part of an address.
                    !give
                player:
                    !take
                    That'll have to do. Now keep quiet about this, or I'll be back.
                agent:
                    Sure. Whatever.
                    !exit
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    You don't want the threat of Admiralty hanging over you all your life, do you?
                agent:
                    !angry
                    No. I don't want that.
                player:
                    So...
                agent:
                    So get bent.
            ]],

            OPT_ATTACK = "Beat up {agent}",
            DIALOG_ATTACK = [[
                player:
                    I guess it's come to this.
                agent:
                    To what?
                player:
                    !fight
                    This.
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead. You kick the body over into the water.
                }
                {not dead?
                    agent:
                        !injured
                        Stop!
                    player:
                        So are you ready to talk?
                    agent:
                        Yes. I steal things from shipments for the Rise.
                        They pay well, and I need to eat.
                    player:
                        Who's your contact?
                    agent:
                        I don't know. All I know is where the goods end up.
                        !give
                    player:
                        !take
                        It's a clue, at least. Now keep quiet, or I'll be back.
                    agent:
                        !exit
                }
            ]],
            DIALOG_GET_CLUE = [[
                * The address is in the richer part of town.
                
                {uncovered_clerk?
                    player:
                        !eureka
                    * It's clear now. {clerk} the clerk is working both sides.
                }

                {not uncovered_clerk?
                    player:
                        !thought
                    * You're getting closer, but you need more clues.
                }
            ]],
            OPT_PAY_FOR_INFO = "Pay {agent} for information about {agent.hisher} handler",
            DIALOG_PAY_FOR_INFO = [[
                player:
                    I get it. You gotta eat.
                    How about I do you a favor and pay you for the identity of your handler?
                    !give
                agent:
                    !take
                    ...
                    That is acceptable.
                    I don't know who I've been dealing with, but I know where the shipments end up. Here.
                    !give
                player:
                    !take
                    That'll do. Now keep quiet about this.
                agent:
                    Understood.
                    !exit
            ]],
        }
        :Fn(function(cxt)

            CreateCombatBackup(cxt:GetCastMember("lifter"), "JAKES_BACKUP", cxt.quest:GetRank()-1)
            cxt:TalkTo("lifter")
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function()

                cxt:QST("RISE")

                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{}
                        :OnWin()
                            :CompleteQuest("example_lifter")
                            :Dialog("DIALOG_WON")
                            :Fn(function() 
                                cxt.quest.param.examples = cxt.quest.param.examples + 1
                                if cxt:GetAgent():IsAlive() then
                                    AddClue(cxt.quest)
                                    cxt:Dialog("DIALOG_GET_CLUE")
                                end
                            end)
                            :Travel()
    

                cxt:Opt("OPT_PAY_FOR_INFO")
                    :Dialog("DIALOG_PAY_FOR_INFO")
                    :DeliverMoney(50)
                    :ReceiveOpinion("paid_for_info")
                    :CompleteQuest("example_lifter")
                    :Fn(function() 
                        AddClue(cxt.quest)
                        cxt:Dialog("DIALOG_GET_CLUE")
                    end)
                    :Travel()

                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("example_lifter")
                        :Fn(function() 
                            AddClue(cxt.quest)
                            cxt:Dialog("DIALOG_GET_CLUE")
                        end)
                        :Travel()
            end)
        end)


QDEF:AddConvo("example_hesh")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("hesh"):GetLocation() end)
        :Loc{
            DIALOG_INTRO = [[
                * You find {hesh} addressing a small crowd of the faithful.
                agent:
                    !right
                    ...and so we must consider Hesh's indifference to be not only a <i>divine</> gift, but also a <i>temporary</> one. Hesh will notice us all in time.
                player:
                    !left
                    Wait, so Hesh hasn't noticed me yet?
                agent:
                    If Hesh had noticed you, you would know, grifter.
            ]],
            OPT_RISE = "Ask about the Rise",
            DIALOG_RISE = [[
                player:
                    Tell me this - has Hesh noticed you palling around with the Rise?
                agent:
                    What?
                    No! I mean, I'm not 'palling around' with the Rise.
                    The Rise are a criminal organization, and I'm a law abiding member of the Cult of Hesh!
            ]],

            OPT_CONVINCE = "Convince {agent} that {agent.heshe} is in danger",
            DIALOG_CONVINCE = [[
                player:
                    The Admiralty knows you've been going off-message in your little sermons.
                    They don't like the parts about "oppression" and "injustice" and whatnot.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                agent:
                    Alright, I hear what you are saying.
                    I <i>have</> been recruiting for the Rise, because that's what's right.
                    Hesh isn't going to help anyone in this life, so I figure someone might as well.
                player:
                    Yeah yeah, down with the system.
                    Who's your handler?
                agent:
                    Don't got one. They leave me dead drops and I raise or lower the blinds on my window. 
                player:
                    Not much of a conversation, is it?
                agent:
                    Yeah, well. Means that nobody can give the game away when goons like you come to call.
                    !neutral_notepad
                    Here. It's my cipher.
                    !give
                player:
                    !take
                    The cipher for your... blinds?
                agent:
                    Hey, it's more complicated than you'd think!
                    Now leave me alone. 
                    !exit
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                agent:
                    You're accusing me of heresy, grifter.
                    !threaten
                    You should know better than that.
            ]],

            OPT_ATTACK = "Beat up {agent}",
            DIALOG_ATTACK = [[
                player:
                    Let's see if we can get Hesh to notice this.
                agent:
                    To notice what?
                player:
                    !fight
                    This.
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead, and so are {agent.hisher} heretical beliefs.
                }
                {not dead?
                    agent:
                        !injured
                        Enough!
                    player:
                        Alright, you ready to spill the beans?
                    agent:
                        I recruit for the Rise. Because it's the right thing to do.
                    player:
                        Oooh, a true believer. Cute. 
                        Who's your contact?
                    agent:
                        I don't have one. I use a cipher to communicate with them. Here, take it.
                        !give
                    player:
                        !take
                        If there's one thing I'm good at, it's solving complex logical problems. This thing is as good as cracked.
                    agent:
                        ...
                    player:
                        Whatever. Now get out of here!
                    agent:
                        !exit
                }
            ]],
            DIALOG_GET_CLUE = [[
                * The code is, indeed, very complex. But you manage to glean some information from it.
                
                {uncovered_clerk?
                    player:
                        !eureka
                    * It's clear now. {clerk} the clerk is working both sides.
                }

                {not uncovered_clerk?
                    player:
                        !thought
                    * You're almost at a breakthrough, but you'll need more clues to really break it.
                }
            ]]
        }
        :Fn(function(cxt)

            CreateCombatBackup(cxt:GetCastMember("hesh"), {"LABORER", "BANDIT_GOON"} )
            cxt:TalkTo("hesh")
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function()

                cxt:QST("RISE")
                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{}
                        :OnWin()
                            :CompleteQuest("example_hesh")
                            :Dialog("DIALOG_WON")
                            :Fn(function() 
                                cxt.quest.param.examples = cxt.quest.param.examples + 1
                                if cxt:GetAgent():IsAlive() then
                                    AddClue(cxt.quest)
                                    cxt:Dialog("DIALOG_GET_CLUE")
                                end
                            end)
                            :Travel()
    
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("example_hesh")
                        :Fn(function() 
                            AddClue(cxt.quest)
                            cxt:Dialog("DIALOG_GET_CLUE")
                        end)
                        :Travel()
            end)
        end)



QDEF:AddConvo("confront_clerk", "clerk")
    :ConfrontState("CONFRONT")
        :Loc{
            DIALOG_INTRO = [[
                * You bust down {clerk}'s door, and catch {clerk.himher} reading a Rise pamphlet.
                agent:
                    !right
                    !surprised
                    Who the Hesh are you?!
                player:
                    !left
                    !happy
                    Just a concerned citizen.                    
                agent:
                    !neutral
                    You can't just barge into my home, grifter! Get out!
                player:
                    Say, is that seditious material you're reading there?
                agent:
                    What? No! It's-
                player:
                    Spare me. I've talked to all your puppets already. I know what's up.
                agent:
                    !angry
                    Alright, then. What do you want?
            ]],

            OPT_RISE = "Ask why {agent} is working with the Rise",
            DIALOG_RISE = [[
                player:
                    You have a cushy job with the Admiralty.
                    Why are you risking that to work with the Rise?
                agent:  
                    I've got my reasons. And they're my own.
                player:
                    OK, the Rise works in mysterious ways. I get it.
                    So I guess you think that makes ME the bad guy here, huh?
                agent:
                    !interest
                    ...Yes?
                player:
                    !neutral
            ]],

            OPT_EXTORT = "Extort {agent}",
            DIALOG_EXTORT = [[
                player:
                    !placate
                    I don't care what you get up to in your spare time.
                    But I bet there are some people who would.
                    !thought
                    It would be a pity if your secret got out...
            ]],
            DIALOG_EXTORT_FAILURE = [[
                agent:
                    !fight
                    You're no good, grifter!
            ]],
            DIALOG_EXTORT_SUCCESS = [[
                agent:
                    I'll pay! I'll pay!
                    Just know that if you betray me, the Rise will kill you while you sleep.
                player:
                    Fair deal.
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Alright.
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead. You hope {eden} appreciates your extra effort.
                }
                {not dead?
                    agent:
                        !injured
                    
                    player:
                        !point
                        You started that.
                        I'm leaving now.
                        !exit
                }
            ]],

            OPT_PROMISE_TO_KEEP_SECRET = "Promise to keep {agent}'s secret",
            DIALOG_PROMISE_TO_KEEP_SECRET = [[
                player:
                    !placate
                    Don't worry!
                    I was just tracking you down to warn you to be more careful!
                    !happy
                    Your secret is safe with me.
                agent:
                    I- I don't know what to say.
                    Thank you, grifter, you are a true friend of the Rise.
                    Now please leave my house, and never come back again.
                player:
                    !exit
            ]],

            OPT_LEAVE = "Leave",
            DIALOG_LEAVE = [[
                player:
                    You know what?
                    I have the wrong address.
                    !exit
            ]],

        }
        :Fn(function(cxt)

            cxt.quest:Complete("confront_clerk")
            
            cxt:TalkTo("clerk")
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function()

                cxt:QST("RISE")

                cxt:BasicNegotiation("EXTORT")
                    :ReceiveOpinion("extorted")
                    :OnSuccess()
                        :ReceiveMoney(300)
                        :Fn(function() cxt.quest.param.promised_clerk = true end)
                        :Travel()
                    
                    :OnFailure()
                        :Fn(function() 
                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
                                    :OnWin()
                                        :Dialog("DIALOG_WON")
                                        :Travel()
                        end)
                
                cxt:Opt("OPT_PROMISE_TO_KEEP_SECRET")
                    :Dialog("DIALOG_PROMISE_TO_KEEP_SECRET")
                    :Fn(function() cxt.quest.param.promised_clerk = true end)
                    :ReceiveOpinion("keep_secret")
                    :Travel()

                cxt:Opt("OPT_LEAVE")
                    :Dialog("DIALOG_LEAVE")
                    :MakeUnder()
                    :Travel()

            end)
        end)



QDEF:AddConvo("return_to_eden", nil)
    :TravelConfront("STATE", function(cxt) 
            return not cxt.quest.param.confronted and cxt.caravan.last_travel_destination == cxt.quest:GetCastMember("eden"):GetLocation()
        end)
        :Loc{
            DIALOG_INTRO = [[
                * You are set upon by a small group of Rise.
                agent:
                    !right
                    Hold up right there.
                    !crossed
                player:
                    !left
                agent:
                    You've been asking dangerous questions, grifter.
                    Now I know you people only do things for the money, so I'm not going to take personal offense.
                    I'll even cut you a deal: 
                    Give us a "donation", and tell us who's pulling your strings, and we'll let you live.
            ]],

            OPT_TELL_ALL = "Give the 'donation' and spill the beans",
            DIALOG_TELL_ALL = [[
                player:
                    You're right. I am in this for the money.
                    And the money ain't worth taking all of you on in a fight.
                    !interest
                    Although I totally could.
                agent:
                    !take
                    We'll put this to a good cause.
                    Who put you up to this?
                player:
                    {eden}. Obviously.
                agent:
                    !thought
                    It comes right from the top. We need to change our tactics.
                    Farewell, grifter.
                    !exit
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Nah, I'm just going to force my way through you.
            ]],
            DIALOG_WON = [[
                agent:
                    !exit
                player:
                    !neutral
                * The path is now clear.
            ]],

            OPT_TRICK = "Convince {agent} that you're actually protecting the Rise",
            DIALOG_TRICK = [[
                player:
                    Hey, I'm protecting your interests with my 'dangerous questions'!
            ]],
            DIALOG_TRICK_SUCCESS = [[
                player:
                    The Admiralty wanted me to break some Rise bones. I'm giving them just enough to keep the heat off of you guys.
                agent:
                    !thought
                    I guess that's noble?
                player:
                    You're damn right it is! You should be paying ME a donation!
                agent:
                    Let's not get ahead of ourselves.
                    Carry on, grifter. But know that we're watching you.
                    !exit
            ]],
            DIALOG_TRICK_FAILURE = [[
                player:
                    I'm was hired to beat up you lot, but I'm just collecting info instead. You should be thanking me!
                agent:
                    So instead of delivering some punishment to a couple low-level functionaries, you're instead compromising our entire network?
                player:
                    Yeah! I mean... sort of?
                agent:
                    !hips
                    Yeah, no. We can do without that kind of help.
            ]],

        }
        :Fn(function(cxt) 
            cxt.quest.param.confronted = true

            local rise = CreateCombatParty("RISE_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(rise[1])

            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                
                if cxt.quest.param.examples == 0 then
                    cxt:BasicNegotiation("TRICK")
                        :OnSuccess()
                            :Travel()
                end

                cxt:Opt("OPT_TELL_ALL")
                    :Dialog("DIALOG_TELL_ALL")
                    :DeliverMoney(math.min( math.max(50, cxt.caravan:GetMoney()), 200) )
                    :Travel()

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{no_oppo_limit = true, flags = BATTLE_FLAGS.SELF_DEFENCE, on_runaway = StateGraphUtil.DoRunAwayNoFail}
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :Travel()
            end)

        end)


QDEF:AddConvo("return_to_eden", "eden")
    :Loc{
        OPT_GIVE_REPORT = "Tell {agent} what happened",
        DIALOG_GIVE_REPORT_GOOD = [[
            player:
                !happy
                I'm done working through the list.
            agent:
                Was the message received?
            player:
                Loud and clear.
            agent:
                Excellent. Good work, Grifter.
        ]],
        DIALOG_GIVE_REPORT_BAD = [[
            player:
                I'm back.
            agent:
                Was the message understood?
            player:
                !shrug
                Eh... does anyone really ever understand anyone?
            agent:
                !facepalm
                Incompetence.
                Utter incompetence.
            player:
                I put forth my best effort, that has to count for something, right?
            agent:
                If that was your best effort, I feel sorry for you.
        ]],

        OPT_TELL_ABOUT_CLERK = "Tell {agent} about {clerk}",
        OPT_TELL_ABOUT_CLERK_BETRAY = "Betray {clerk}'s secret",

        DIALOG_TELL_ABOUT_CLERK = [[
            player:
                You need to clean your house.
                {clerk} has been working with the Rise.
            agent:
                !surprised
                No! It can't be true!
                We play poker every Thursday night!
            player:
                !shrug
                I have all kinds of proof.
            agent:
                !thought
                I have a lot to consider now. Thank you, grifter. You've done a great service to the Admiralty.
        ]],
        DIALOG_TELL_ABOUT_CLERK_TO_SAVE = [[
            player:
                Wait. I did figure out something you don't know.
                {clerk} has been working with the Rise.
            agent:
                !thought
                I suspected as much. You have proof?
            player:
                Pffft. Yeah.
            agent:
                !thought
                Alright, you redeemed yourself. But don't go off-book again, grifter.
        ]],

        OPT_DONT_TELL = "Keep {clerk}'s secret",
        DIALOG_DONT_TELL = [[
            player:
                !salute
                Glad to be of service.
            agent:
                You are dismissed.            
        ]],
    }
    :Hub(function(cxt)
        
        cxt:Opt("OPT_GIVE_REPORT")
            :SetQuestMark()
            :Fn(function() 
                
                local success = cxt.quest.param.examples >= 2
                cxt:Dialog(success and "DIALOG_GIVE_REPORT_GOOD" or "DIALOG_GIVE_REPORT_BAD")

                if cxt.quest.param.uncovered_clerk and cxt.quest:GetCastMember("clerk"):IsAlive() then
                    
                    local opt = cxt:Opt(cxt.quest.param.promised_clerk and "OPT_TELL_ABOUT_CLERK_BETRAY" or "OPT_TELL_ABOUT_CLERK")
                                    :Dialog(success and "DIALOG_TELL_ABOUT_CLERK" or "DIALOG_TELL_ABOUT_CLERK_TO_SAVE")
                    
                    if cxt.quest.param.promised_clerk then
                        opt:ReceiveOpinion("betrayed", nil, "clerk")
                    end
                    if success then
                        opt:ReceiveOpinion("uncovered_clerk")
                    end

                    
                    opt:CompleteQuest()
                        :Fn(function() 
                            ConvoUtil.GiveQuestRewards(cxt)
                        end)
                    
                    cxt:Opt("OPT_DONT_TELL")
                        :Dialog("DIALOG_DONT_TELL")
                        :Fn(function() 
                            if success then
                                ConvoUtil.GiveQuestRewards(cxt)
                                cxt.quest:Complete()
                            else
                                cxt.quest:Fail()
                            end
                        end)
                else
                    if success then
                        ConvoUtil.GiveQuestRewards(cxt)
                        cxt.quest:Complete()
                    else
                        cxt.quest:Fail()
                    end
                end


            end)
    end)

QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}



    



