local QDEF = QuestDef.Define{
        title = "The Great Lumin Heist",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{denikus} has a heist arranged that is in danger of falling off the rails. Make sure it happens.]],
        icon = engine.asset.Texture("icons/quests/side_smith_great_lumin_heist.tex"),
        focus = QUEST_FOCUS.BALANCED,
        tags = {"DENIKUS_JOB"},
        act_filter = "SMITH",
        
        on_debug_start = function(quest)
            TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("PEARL_OSHNUDROME"))
            UIHelpers.DoSpecificConvo( quest:GetCastMember("denikus"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
        end,
        
        avoids = {
            {"zealot", "insider"},
            {"other_seller", "buyer"},
            {"other_seller", "zealot"},
        },

    }

    :AddCastByAlias{
        cast_id = "denikus",
        alias = "DENIKUS",
        provider = true,
        no_validation = true,
    }

    :AddDefCastSpawn("insider", "PRIEST")
    :AddDefCastSpawn("muscle", "BANDIT_RAIDER")
    :AddDefCastSpawn("transporter", "JAKES_SMUGGLER")
    :AddDefCastSpawn("buyer", "CHEMIST")
    
    :AddDefCastSpawn("zealot", "ZEALOT")
    :AddDefCastSpawn("other_seller", "SPARK_BARON_TASKMASTER")

    --:AddDefCast("buyer", "PRIEST")
    
------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------
    
    :AddObjective{
        id = "talk_to_insider",
        title = "Compromise the Security",
        desc = [[{insider} is supposed to make sure the security detail is light during the heist. Go make sure that happens.]],
        mark = { "insider" },

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("insider"), location =  quest:GetCastMember("insider_temple")} )
        end,

        on_complete = function(quest)
            if quest:IsActive('deal_with_zealot') then
                quest:Cancel('deal_with_zealot')
            end
        end,

    }
    
    :AddObjective{
        id = "deal_with_zealot",
        title = "(Optional) Deal with {zealot}",
        desc = [[Throw {zealot} off of {insider}'s trail.]],
        mark = { "zealot" },

        on_complete = function(quest)
            if quest:IsActive('talk_to_insider') then
                quest:Complete('talk_to_insider')
            end
        end,
    }


    :AddObjective{
        id = "talk_to_muscle",
        title = "Get the muscle ready",
        desc = [[{muscle} is arranging the muscle needed for the heist. Make sure {muscle.heshe} has enough goons.]],
        mark = { "muscle" },

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("muscle"), location = quest:GetCastMember("muscle_beach")} )
        end,
    }


    :AddObjective{
        id = "talk_to_transporter",
        title = "Arrange transportation",
        desc = [[{transporter} is going to move the stolen Lumin out of the Pearl. Tell {transporter.himher} to get ready.]],
        mark = { "transporter" },

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("transporter"), location = quest:GetCastMember("transporter_place")} )
        end,
    }

    :AddObjective{
        id = "talk_to_buyer",
        title = "Secure the buyer",
        desc = [[{buyer} is having second thoughts about {buyer.hisher} purchase. Go seal the deal.]],
        mark = { "buyer" },

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("buyer"), location = quest:GetCastMember("buyer_place")} )
        end,

        on_complete = function(quest)
            if quest:IsActive('deal_with_other_seller') then
                quest:Cancel('deal_with_other_seller')
            end
        end,
    }

    :AddObjective{
        id = "deal_with_other_seller",
        title = "(Optional) Scare off {other_seller}",
        desc = [[{other_seller} is offering {buyer} a load of spark instead of your lumin. Make that not happen.]],
        mark = { "other_seller" },

        on_complete = function(quest)
            if quest:IsActive('talk_to_buyer') then
                quest:Complete('talk_to_buyer')
            end
        end,
    }


    :AddObjective{
        id = "start",
        on_activate = function(quest) 
            quest:Activate("talk_to_insider")
            quest:Activate("talk_to_buyer")
            quest:Activate("talk_to_muscle")
            quest:Activate("talk_to_transporter")
        end,
        events = {
            quests_changed = function(quest, event_quest) 
                if event_quest == quest then
                    if quest:IsComplete("talk_to_insider") and 
                        quest:IsComplete("talk_to_buyer") and 
                        quest:IsComplete("talk_to_muscle") and 
                        quest:IsComplete("talk_to_transporter") then
                            quest:Complete('start')    
                            quest:Activate('do_the_heist')
                    end
                end
            end           
        },

    }

    :AddObjective{
        id = "do_the_heist",
        title = "Do the heist!",
        desc = [[The pieces are all set up. Go liberate some lumin!]],
        mark = { "heist_spot" },
        on_complete = function(quest)
            quest:Activate("return_to_denikus")
        end,
    }

    :AddObjective{
        id = "return_to_denikus",
        title = "Return to {denikus}",
        desc = [[Tell {denikus} how the heist went down.]],
        mark = { "denikus" },
        on_activate = function(quest)
            quest.param.pending_interruption = (quest.param.other_seller_angry and not quest:GetCastMember("other_seller"):IsDead()) or ( (quest.param.zealot_suspicious or quest.param.zealot_angry ) and not quest:GetCastMember("zealot"):IsDead())
        end,
        
        on_complete = function(quest) 
            
            if quest.param.side_deal then
                local overrides = { cast = { transporter = quest:GetCastMember("transporter") } }
                local quest = QuestUtil.SpawnQuest("SIDE_SMITH_GREAT_LUMIN_HEIST_LATER", overrides)
            end
        end
    }

    :AddQuestLocation{
        cast_id = "insider_temple",
        name = "Hesh Temple",
        plax = "INT_SMITH_HESHTEMPLE",
        map_tags = {"building"},
    }

    :AddQuestLocation{
        cast_id = "muscle_beach",
        name = "Beach",
        plax = "EXT_SMITH_BEACHCREATURE",
        map_tags = {"beach"},
    }

    :AddQuestLocation{
        cast_id = "transporter_place",
        name = "Wharf",
        plax = "EXT_SMITH_WARF",
        map_tags = {"port"},
    }

    :AddQuestLocation{
        cast_id = "buyer_place",
        name = "Streetcorner",
        plax = "EXT_SMITH_Nightlife_Street",
        map_tags = {"street"},
    }

    :AddExistingLocation("heist_spot", "PEARL_LUMIN_TANK")


    :AddOpinionEvents{
        let_off_hook = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Let them off the hook for the heist",
        },--]]
        mistrust = 
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Suspects you cheated them (you totally did)",
        }
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddIntro(
    --attract spiel
    [[
        denikus:
            Ever participated in a heist, Smith?
        player:
            I wish! Why, you want me to do a heist?
            Should I be the face? Or the muscle? Ooh, or maybe the mastermind, huh?
            Suffice it to say I've got the skills for all three.
        denikus:
            Sorry, bud. This time, you're the handler. Need you to coordinate with the others and get them ready.
            Then you'll be on hand if anything goes wrong.
        player:
            Aw. I'm the fixer? 
            What if nothing goes wrong?
        denikus:
            That's what we <i>want</i> to happen. And learning that will be good experience for your next heist. 
            Chin up, champ. There's always opportunities for a dreamer.
    ]],
    
    --on accept
    [[
        player:
            Sit tight, I guess. I'll make sure your crew gets to port safely.
        agent:
            See? You're getting in the spirit already.
            There might be an 'I' in 'heist', but that doesn't mean it's a solo operation.
        ]],
    "start")


-------------------

QDEF:AddConvo("talk_to_insider", "insider")
    :AttractState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * You find {insider}. According to the brief, {insider} is working on the inside, trying to pull the guard from the shipment.
                    player:
                        Hey, uh. The swamps of the Bog sure have changed.
                    insider:
                        At least the Barons aren't here.
                        I take it {denikus} sent you? 
                    player:
                        Yeah. {denikus} has another job he's trying to pull, so I'm his fixer.
                    insider:
                        Well, I'm sorry to say it, but you might want to call the whole thing off.
                        I'm having second thoughts about my part in it, okay?
                        {zealot}'s been watching me a little too close, lately. I think {zealot.heshe} suspects.
                }
                {not first_time?
                    * You find {insider} again, looking just as anxious as before.
                }
            ]],

            OPT_THREATEN = "Threaten {agent} to do {agent.hisher} job",
            DIALOG_THREATEN = [[
                player: 
                    Who you more afraid of, huh? 
                    {zealot}, or me?
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                player:
                    Hint: I'm the one with the big fists.
                insider:
                    O-okay, okay! I'll do it!
                    But I want some kind of protection from {zealot}. My cut might not be enough to buy my way out of this mess.
            ]],
            DIALOG_THREATEN_FAILURE = [[
                player:
                    {zealot} can't break your face open like I can, can {zealot.heshe}?
                insider:
                    Yes. And then some. All with the consent of the Cult, no less.
                    I'm sorry, but I just can't take the risk!
            ]],

            OPT_GO_DEAL_WITH_ZEALOT = "Leave to go deal with {zealot}",
            DIALOG_GO_DEAL_WITH_ZEALOT = [[
                player:
                    So {zealot}'s all that's standing in your way? If {zealot.heshe} were outta the picture, you'd be back in action?
                insider:
                    Well, yes, b-but--
                player:
                    Keep your shoes warm. I'll be right back.
            ]],

            OPT_LET_OFF_HOOK = "Let {agent} off the hook",
            DIALOG_LET_OFF_HOOK = [[
                player:
                    Okay, fine. Forget it. We'll adjust the plan accordingly.
                    Just stay outta our way, and don't you dare squeal.
                insider:
                    Hey, don't worry--I'm not suicidal. 
                    Thank you for understanding. Good luck. 

            ]]
        }
    :Fn(function(cxt) 
        cxt:Dialog("DIALOG_INTRO")

        cxt:RunLoop(function() 
            
            cxt:BasicNegotiation("THREATEN", {flags = NEGOTIATION_FLAGS.INTIMIDATION})
                :OnSuccess()    
                    :CompleteQuest("talk_to_insider")
                    :Fn(function() cxt.quest.param.zealot_suspicious = true end)
                    :DoneConvo()

            cxt:Opt("OPT_LET_OFF_HOOK")
                :Dialog("DIALOG_LET_OFF_HOOK")
                :ReceiveOpinion("let_off_hook")
                :Fn(function()  
                    cxt.quest.param.full_guard = true
                    cxt.quest.param.zealot_angry = true
                end)
                :CompleteQuest("talk_to_insider")
                :End()

            cxt:Opt("OPT_GO_DEAL_WITH_ZEALOT")
                :PreIcon( global_images.close)
                :Dialog("DIALOG_GO_DEAL_WITH_ZEALOT")
                :Fn(function()
                    if cxt.quest:IsInactive("deal_with_zealot") then
                        cxt.quest:Activate("deal_with_zealot")
                    end
                end)
                :DoneConvo()
            end)
    end)


QDEF:AddConvo("deal_with_zealot", "zealot")
    :AttractState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * Finding {zealot} is as simple as following the smell of smug superiority.
                    player:
                        Hey, you {zealot}?
                        I've got an important message for you.
                    zealot:
                        Is that so? And from whom, I wonder?
                }
                
                {not first_time?
                    zealot:
                        You again? Why do you insist on interrupting my contemplations?
                }
            ]],

            OPT_THREATEN = "Scare {agent} off of the investigation",
            
            DIALOG_THREATEN = [[
                player: 
                    Seems you've been hassling my buddy {insider}. 
                    You got a reason for that? Only 'cause, you know, {insider.hisher} job is pretty dangerous.
                    Lots of pure lumin getting moved around. Lots of opportunity to get a face full of lumin burn, too.
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                player:
                    Lumin shipping is a dangerous business. 
                    If you don't stop making {insider} so nervous, you might end up on the wrong end of the fluid valve.
                zealot:
                    O-oh. I suppose that's a valid concern.
                    If I've been interfering with {insider}'s work, I apologize.
                    I'll be sure to give {insider.himher} plenty of space.
                player:
                    Good. It's safest that way.
            ]],
            DIALOG_THREATEN_FAILURE = [[
                player:
                    You wouldn't want to bring down the danger, would you?
                zealot:
                    I take it this has something to do with that worm's suspicious behavior lately, hm?
                    Suffice it to say, I won't be scared off so easily.
            ]],

            OPT_CONVINCE = "Convince {agent} that {insider} is innocent",
            
            DIALOG_CONVINCE = [[
                player: 
                    Look, I get why you've been on to {insider.himher}.
                    But the reason {insider}'s been acting so suspicious lately is 'cause {insider.heshe}'s been having... weird feelings.
                zealot:
                    What kind of feelings?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    Feelings like {insider.heshe}'s in the wrong line of work. Like that maybe {insider.heshe}'s got a calling elsewhere.
                zealot:
                    Ah--a summoning from Hesh, perhaps?
                player:
                    Maybe! {insider} wants to spend the day in deep, uninterrupted contemplation to see what Hesh wants.
                zealot:
                    I see.
                    Well, it would be against my nature to stand in the way of someone's faith... and Hesh itself.
                    I'm happy for {insider}. Though many prefer to walk in the shallows, it is unlike anything else to be embraced by Hesh in the deep.

            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    Feelings like maybe Hesh thinks you're being a pain in the butt.
                zealot:
                    Ah. 
                    I see.
                    That's... not how this works.
                    Perhaps {insider} is having a crisis of faith, yes? And I should keep {insider.himher} under <i>very</i> close observation.
            ]],

            OPT_BATTLE = "Attack {zealot}",
            DIALOG_BATTLE = [[
                player:
                    Aw, forget it. I wanted to be the muscle anyway!
                    !fight
            ]],
            DIALOG_WON = [[
                {dead?
                    * {zealot} is out of the picture, though you suspect {insider} won't be too happy with this method.
                }
                {not dead?
                    player:
                        You know what? I think you're overdue for a vacation.
                        Why don't you get outta here, take care of yourself after this unfortunate workplace accident, huh?
                    zealot:
                        Y-yes, I agree.
                        I think I'm meant to visit my aunt in Supplicant City.
                    player:
                        Aw, that's nice. Give her my love, yeah?
                    zealot:
                        !exit
                }
            ]],

        }
    :Fn(function(cxt) 
        cxt:Dialog("DIALOG_INTRO")

        cxt:RunLoop(function()

            cxt:BasicNegotiation("THREATEN", {flags = NEGOTIATION_FLAGS.INTIMIDATION})
                :OnSuccess()    
                    :CompleteQuest("deal_with_zealot")
                    :DoneConvo()

            cxt:BasicNegotiation("CONVINCE")
                :OnSuccess()    
                    :CompleteQuest("deal_with_zealot")
                    :DoneConvo()

            cxt:Opt("OPT_BATTLE")
                :Dialog("DIALOG_BATTLE")
                :Battle{}
                    :OnWin()
                        :Dialog("DIALOG_WON")
                        :CompleteQuest("deal_with_zealot")
                        :DoneConvo()
            
            StateGraphUtil.AddEndOption(cxt)
        end)
    end)

----------------


QDEF:AddConvo("talk_to_muscle", "muscle")
    :AttractState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * You find {muscle}, the muscle, lounging about looking like anything but.
                    player:
                        Hey, {muscle}? {denikus} sent me. Just checking in to make sure we're good to go.
                    muscle:
                        What, he added one more to the roster? How small is my take gonna get, huh?
                        The way I see it, I'm a vital part of this operation. You can't make headway without my goons at your side. 
                        I want a bigger cut, or we walk.
                }
                {not first_time?
                    muscle:
                        Unless you come offering shills, I'm not your guy.
                }
            ]],


            OPT_CONVINCE = "Convince {muscle} to honor the deal",
            
            DIALOG_CONVINCE = [[
                player: 
                    Hey, I'm not cutting into your take. You'll still get what you were offered.
                agent:
                    And what if I want more?
                player:
                    Yeah? And what if you ever wanna work again, huh?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    Denikus is moving up in the world. You back out now, and that mistake'll haunt you like a bad smell.
                    But sure, worth it just to make a point, huh? Why not?
                agent:  
                    ...
                    Fine. But I mean it! I want my full cut, no matter how many more partners Denikus brings in.
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    You think you'll survive a blow to your reputation like that?
                agent:
                    Why not? If anything it'll teach people not to lowball me.
            ]],

            OPT_GIVE_MONEY = "Give {agent} a kickback",
            DIALOG_GIVE_MONEY = [[
                player:
                    How about I just pay you a little extra, and you fall in line?
                    !give
                agent:
                    !take
                    That'll do it. Appreciate you being so expedient. 
            ]],
            
            OPT_ACCEPT_REDUCED_MUSCLE = "Accept fewer hired goons",
            DIALOG_ACCEPT_REDUCED_MUSCLE = [[
                player:
                    Look, I don't got the money to increase your pay. 
                    Just give us the folks you can spare for the price, alright?
                agent:
                    Oh, sure. I have a couple of new recruits you could use some whetting. 
                    Have fun cutting their teeth.
            ]],            
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 

                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :CompleteQuest("talk_to_muscle")
                        :DoneConvo()

                cxt:Opt("OPT_GIVE_MONEY")
                    :DeliverMoney(80)
                    :Dialog("DIALOG_GIVE_MONEY")
                    :CompleteQuest('talk_to_muscle')
                    :DoneConvo()

                cxt:Opt("OPT_ACCEPT_REDUCED_MUSCLE")
                    :Dialog("DIALOG_ACCEPT_REDUCED_MUSCLE")
                    :Fn(function() 
                        cxt.quest.param.reduced_muscle = true
                    end)
                    :CompleteQuest('talk_to_muscle')
                    :DoneConvo()


                StateGraphUtil.AddEndOption(cxt)

            end)
        end)


----------------



QDEF:AddConvo("talk_to_transporter", "transporter")
    :AttractState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * {transporter} eyes you up as you arrive; clearly {transporter.heshe} was expecting you.
                    transporter:
                        You from Denikus, yeah? The transport's all ready to go.
                        Once we load up the lumin, it'll be smooth sailing.
                        Of course... that's if you <i>want</i> it to be.
                    player:
                        Oh yeah? And why would I want anything else?
                    transporter:
                        Because maybe we hit a pothole in the road, and some of the lumin slips out.
                        And maybe that lumin slips into someone else's hands.
                        Denikus won't be happy... but that's where you come in.
                        Talk him down, and I'll make sure you get a nice, fat cut.
                }
                
                {not first_time?
                    transporter:
                        Alright, Smith? My offer still stands.
                }
            ]],

            OPT_TAKE_THE_DEAL = "Take the deal",
            DIALOG_TAKE_THE_DEAL = [[
                player:
                    All I gotta do is smooth talk him, huh?
                    I reckon I can handle that. You got yourself a deal.
                agent:
                    Then I'm very much lookin' forward to working with you, bud.
            ]],

            OPT_REFUSE_THE_DEAL = "Refuse the deal",
            DIALOG_REFUSE_THE_DEAL = [[
                player:
                    Naw. No judgment against you or anything, but I got more riding on this than pay.
                    I prefer to stay on the level with Denikus. I won't rat you out, though.
                agent:
                    Enh, no harm in asking, then.
                    Thanks for being cool about it, at least.
            ]],

       
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_TAKE_THE_DEAL")
                :Dialog("DIALOG_TAKE_THE_DEAL")
                :Fn(function() cxt.quest.param.side_deal = true end)
                :CompleteQuest("talk_to_transporter")
                :DoneConvo()

            cxt:Opt("OPT_REFUSE_THE_DEAL")
                :Dialog("DIALOG_REFUSE_THE_DEAL")
                :Fn(function() cxt.quest.param.side_deal = false end)
                :CompleteQuest("talk_to_transporter")
                :DoneConvo()
        end)


-------------



QDEF:AddConvo("talk_to_buyer", "buyer")
    :AttractState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * You approach {buyer}, confident that this will be the least problematic step of the process.
                    * You are wrong.
                    player:
                        The plan's a go. You ready to receive the lumin?
                    buyer:
                        Hrm. I'm afraid not. 
                        I have the facilities for receiving and processing, of course, but I've been reading more about fuel sources...
                        Did you know spark is cheaper <i>and</i> more explosive than lumin?
                        Seems to me that if I invest in a large quantity of lumin, I'll be on the tail end of a dying trade.
                }
                {not first_time?
                    buyer:
                        I'm still waiting to hear why I should go forward with this lumin operation.
                }
            ]],

            OPT_SALES = "Give your best sales pitch",
            
            DIALOG_SALES = [[
                player: 
                    Hey, these are weird times, bud. You have no idea what the future holds for lumin.
                    
            ]],
            DIALOG_SALES_SUCCESS = [[
                player:
                    Between the Great Beaching and the extra hostility between Barons and the Cult...
                    I dunno. If you were in Murder Bay I might tell you something different.
                    But here in the Pearl, switching to spark would make you more enemies than sales.
                    Not to mention the tithe hike the Cult would put you through...
                agent:  
                    Ah, the backend. I hadn't thought of that. Of <i>course</i> the tithes would increase.
                    Hm, well, thank you for the bigger picture. 
                    I'm ready to go ahead as planned, then.
            ]],
            DIALOG_SALES_FAILURE = [[
                player:
                    Besides, spark is still kinda new. You don't know how dangerous it can be.
                agent:
                    "New" and "dangerous" is the line the Cult keeps sputtering.
                    But spark has been around for decades, now! And more and more businesses are adopting it.
                    I won't be considered backwater by both Deltreans <i>and</i> Rentorians at once. 
            ]],

            OPT_GO_TALK_TO_OTHER_SELLER = "Go have a 'chat' with {other_seller}",
            DIALOG_GO_DEAL_WITH_OTHER_SELLER = [[
                player:
                    I'll, uh, be back. You just sit tight.
                agent:
                    Where are you going?
                player:
                    Nowhere! Just, uh, maybe I wanna see what all the fuss is about.
                    With the spark.
                agent:
                    Hah! Wondering if you should switch tracks as well, hm? 
                    I look forward to hearing your decision.
            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 

                cxt:BasicNegotiation("SALES")
                    :OnSuccess()    
                        :Fn(function() cxt.quest.param.other_seller_angry = true end)
                        :Fn(function() 
                            if cxt.quest:IsActive("deal_with_other_seller") then
                                cxt.quest:Cancel("deal_with_other_seller")
                            end
                        end)
                        :CompleteQuest("talk_to_buyer")
                        :DoneConvo()

                cxt:Opt("OPT_GO_TALK_TO_OTHER_SELLER")
                    :PreIcon( global_images.close)
                    :Dialog("DIALOG_GO_DEAL_WITH_OTHER_SELLER")
                    :Fn(function()
                        if cxt.quest:IsInactive("deal_with_other_seller") then
                            cxt.quest:Activate("deal_with_other_seller")
                        end
                    end)
                    :DoneConvo()
                end)
            end)




QDEF:AddConvo("deal_with_other_seller", "other_seller")
    :AttractState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * It's easy to find the Spark Barons in the Pearl, because all the Heshians give them a wide berth.
                    player:
                        Hey, you the one selling the spark?
                    other_seller:
                        I am. How much you want?
                }
                {not first_time?
                    other_seller:
                        Still got spark for sale, if you're interested.
                }
            ]],

            OPT_CONVINCE = "Convince {agent} that {buyer} is broke",
            DIALOG_CONVINCE = [[
                player:
                    I'm not buyin', friend. Just wanna give you a friendly warning.
                    {buyer} is broke. {buyer.HeShe}'s planning to stiff you once the sale is done.
            ]],
            
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    If you wanna go forward with the deal, do yourself a favor and charge extra <i>up front</i>.
                agent:
                    Hah! Thanks for the heads up. 
                    I've wasted enough of my time on {buyer} already to spend any more.
            ]],

            DIALOG_CONVINCE_FAILURE = [[
                player:
                    I would cancel the sale if I were you.
                agent:
                    You would, would you? Excuse my skepticism.
                    Especially considering {buyer} agreed to pay up-front.
            ]],

            OPT_THREATEN = "Scare off {agent}",
            DIALOG_THREATEN = [[
                player:
                    Back off of {buyer}!
                    Or I'll hurt you!
            ]],
            
            DIALOG_THREATEN_SUCCESS = [[
                player:
                    This is lumin territory, bud. Find some other sucker to take your earthspit. 
                agent:
                    !scared
                    Okay, okay! I'll find someone else to buy my spark.
            ]],

            DIALOG_THREATEN_FAILURE = [[
                agent:
                    I'm not afraid of you!
            ]],

            OPT_BATTLE = "Beat up {agent}",
            DIALOG_BATTLE = [[
                player:
                    I'll just beat you senseless, now, OK?
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead. {buyer} won't be buying {agent.hisher} spark now.
                }
                {not dead?
                    agent:
                        !injured
                        Fine! Take my customer! I don't care!
                }
            ]]


        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 

                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()    
                        :CompleteQuest("deal_with_other_seller")
                        :DoneConvo()

                cxt:BasicNegotiation("THREATEN", {flags = NEGOTIATION_FLAGS.INTIMIDATION})
                    :OnSuccess()    
                        :CompleteQuest("deal_with_other_seller")
                        :DoneConvo()

                cxt:Opt("OPT_BATTLE")
                    :Dialog("DIALOG_BATTLE")
                    :Battle{}
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :CompleteQuest("deal_with_other_seller")
                            :DoneConvo()
                
                StateGraphUtil.AddEndOption(cxt)

            end)
        end)

-------------
QDEF:AddConvo("do_the_heist")
    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("heist_spot") end)
        :Loc{
            DIALOG_DO_HEIST = [[
                * You arrive at the heist spot, just as the lumin is arriving.
                {full_guard?
                    * It is heavily guarded.
                }
                {not full_guard?
                    * {insider} did {insider.hisher} job, and the guard is sparse.
                }
                guard:
                    !right
                * The signal is given, and {muscle}'s goons jump out of cover.
                goon:
                    !left

                {reduced_muscle?
                    * {muscle} didn't send {muscle.hisher} best.
                }
                {not reduced_muscle?
                    * Looks like {muscle} sent {muscle.hisher} most impressive goons.
                }


                {full_guard and reduced_muscle?
                    goon:
                        Uhhh... stick 'em up?
                    guard:
                        Is this some kind of joke?
                }

                {not full_guard and not reduced_muscle?
                    guard:
                        !scared
                        Oh no.
                    goon:
                        You're outnumbered, Hesher. Just walk away, and nobody gets hurt.
                }

                {full_guard and not reduced_muscle?
                    goon:
                        Back away from the lumin, and you won't get hurt.
                    guard:
                        !fight
                        To arms! We're under attack!

                }
                {not full_guard and reduced_muscle?
                    goon:
                        !fight
                        I think we can take 'em!    
                    guard:
                        !angry
                        Is that all you got?
                }

                * You step forward to try to take control of the situation.
                
                player:
                    !left


            ]],
            SIT_MOD_OUTNUMBER = "Your side is stronger!",
            SIT_MOD_OUTNUMBERED = "Their side is stronger!",

            OPT_THREATEN = "Scare off the guards",
            DIALOG_THREATEN = [[
                player:
                    !left
                    You don't want to die for your bosses' lumin, do you?
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                agent:
                    The grifter has a point. 
                    We don't get paid enough for this.
                    !exit
                * The guard leaves.
            ]],
            DIALOG_THREATEN_FAILURE = [[
                agent:
                    We aren't afraid of you! 
                    Hold the line!
                goon:
                    !right
                    This is getting out of control. Run!
                    !exit
                guard:
                    !right
                    !cruel
                    Kill the grifter.
            ]],

            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Ok, we're doing it this way, now?
            ]],
            DIALOG_WON_FIGHT_ALONE = [[
                agent:
                    !exit
                * You make short work of the guard, and are left with the lumin.
                player:
                    !happy
            ]],

            OPT_ATTACK = "Lead the attack!",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    Charge!
            ]],
            DIALOG_WON_FIGHT = [[
                agent:
                    !exit
                * The guards fall to your combined strength.
                player:
                    !happy
            ]],

            DIALOG_TRANSPORT = [[
                player:
                    !left
                transporter:
                    !right
                    We'll take it from here.
                
                {side_deal?
                    Remember our agreement. I'll find you later to give you your payment.
                }
                player:
                    !exit

            ]]

        }
        :Fn(function(cxt) 
            cxt.enc.scratch.hesh = CreateCombatParty(cxt.quest.param.full_guard and {"LUMINARI", "LUMINITIATE", "LUMINITIATE", "LUMICYTE"} or {"LUMINITIATE", "LUMINITIATE", "LUMICYTE"}, 1, cxt.location)
            cxt.enc.scratch.goons = CreateCombatParty(cxt.quest.param.reduced_muscle and {"BANDIT_GOON", "BANDIT_GOON2"} or {"BANDIT_RAIDER", "BANDIT_GOON2"}, 1, cxt.location)
            cxt:ReassignCastMember("goon", cxt.enc.scratch.goons[1])
            cxt:ReassignCastMember("guard", cxt.enc.scratch.hesh[1])
            cxt:Dialog("DIALOG_DO_HEIST")

            local sit_mods = {}
            cxt:TalkTo(cxt.enc.scratch.hesh[1])

            if cxt.quest.param.full_guard and cxt.quest.param.reduced_muscle then
                table.insert(sit_mods, { value = 10, text = cxt:GetLocString("SIT_MOD_OUTNUMBERED") })
            elseif not cxt.quest.param.full_guard and not cxt.quest.param.reduced_muscle then
                table.insert(sit_mods, { value = -10, text = cxt:GetLocString("SIT_MOD_OUTNUMBER") })    
            end

            
            cxt:BasicNegotiation("THREATEN", {flags = NEGOTIATION_FLAGS.INTIMIDATION, helpers = cxt.enc.scratch.goons, situation_modifiers = sit_mods })
                :OnSuccess()
                    :Dialog("DIALOG_TRANSPORT")
                    :CompleteQuest("do_the_heist")
                    :Travel()
                
                :OnFailure()
                    :Fn(function() 
                        
                        for k, v in pairs(cxt.enc.scratch.goons) do v:MoveToLimbo() end
                        cxt:Opt("OPT_DEFEND")
                            :Dialog("DIALOG_DEFEND")
                            :Battle{no_oppo_limit = true}
                                :OnWin()
                                    :Dialog("DIALOG_WON_FIGHT_ALONE")
                                    :Dialog("DIALOG_TRANSPORT")
                                    :CompleteQuest("do_the_heist")
                                    :Travel()
                    end)

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{no_oppo_limit = true,
                        allies = cxt.enc.scratch.goons}
                    :OnWin()
                        :Dialog("DIALOG_WON_FIGHT")
                        :Dialog("DIALOG_TRANSPORT")
                        :CompleteQuest("do_the_heist")
                        :Travel()
            
        end)

-------------

QDEF:AddConvo("return_to_denikus")
    :TravelConfront("STATE", function(cxt) return cxt.quest.param.pending_interruption end)
        :Fn(function(cxt) 
            cxt.quest.param.pending_interruption = false
            local can_be_other_seller = (cxt.quest.param.other_seller_angry and not cxt.quest:GetCastMember("other_seller"):IsDead()) 
            local can_be_zealot = ( (cxt.quest.param.zealot_suspicious or cxt.quest.param.zealot_angry ) and not cxt.quest:GetCastMember("zealot"):IsDead())

            if not can_be_other_seller then
                cxt:GoTo("STATE_ZEALOT")
            elseif not can_be_zealot then
                cxt:GoTo("STATE_OTHER_SELLER")
            else
                cxt:GoTo(table.arraypick{"STATE_OTHER_SELLER", "STATE_ZEALOT"})
            end
        end)
        :State("STATE_OTHER_SELLER")
            :Loc{
                DIALOG_INTRO = [[
                    * Of course, most problems don't ever have a simple solution. 
                    other_seller:
                        !right
                    player:
                        !left
                    other_seller:
                        You're the one who stole {buyer}'s business, are you?
                        I don't take kindly to competition, even if it is in lumin territory.
                ]],

                OPT_PAY_OFF = "Pay off {other_seller}",
                DIALOG_PAY_OFF = [[
                    player:
                        Relax, bud. This was bigger than just one sale.
                        Here, have a slice. No harm done, see?
                        !give
                    other_seller:
                        !take
                        Hm. I suppose so.
                        But next time you have a big scheme? It better not come anywhere near me.
                ]],

                OPT_DEFEND = "Defend yourself",
                DIALOG_DEFEND = [[
                    player:
                        !fight
                        Fine. Guess I'll knock your head in, then, since you insist on sticking it where it don't belong.
                ]],
                DIALOG_WON = [[
                    {dead?
                        * {agent} is dead. Problem solved.
                    }
                    {not dead?
                        agent:
                            !injured
                            You've made a big mistake. Spark is the future, and there's no place for you in it.
                            !exit
                    }
                ]]
            }
            :Fn(function(cxt) 
                cxt:TalkTo("other_seller")
                cxt:Dialog("DIALOG_INTRO")

                cxt:Opt("OPT_PAY_OFF")
                    :DeliverMoney(50)
                    :Dialog("DIALOG_PAY_OFF")
                    :Travel()

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE, on_runaway = StateGraphUtil.DoRunAwayNoFail}
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :Travel()

            end)
        :State("STATE_ZEALOT")
            :Loc{
                DIALOG_INTRO = [[
                    * Of course, most problems don't ever have a simple solution. 
                    zealot:
                        !right
                    player:
                        !left
                    
                    {zealot_suspicious?
                        zealot:
                            Stop right there! You've been working with {insider} to steal lumin, haven't you?
                    }
                    {zealot_angry?
                        zealot:
                            Stop, thief! I know you took the lumin. 
                    }
                ]],

                OPT_CONVINCE = "Convince {agent} you are innocent",
                DIALOG_CONVINCE = [[
                    player:
                        {zealot_angry?
                            Lumin? Do I look like the kinda guy who knows what to do with lumin?
                        }
                        {zealot_suspicious?
                            Calm down, bud. I dunno about lumin, and I don't know no {insider} either.
                        }
                ]],
                DIALOG_CONVINCE_SUCCESS = [[
                    player:
                        There's a lot of guys around here who look just like me. 
                        Maybe you got me confused with someone else?
                    agent:
                        Hm. All I have is my suspicions, and I don't discredit them easily.
                        But since I have no proof... then I'll let you go, for now.
                        !exit
                ]],
                DIALOG_CONVINCE_FAILURE = [[
                    player:
                        You think I got nothing better to do than be a thief?
                    agent:
                        I suppose that's something you'll have to consider while you're rotting in an Admiralty brig.
                ]],

                OPT_DEFEND = "Defend yourself!",
                DIALOG_DEFEND = [[
                    player:
                        !fight
                        You're mighty rude. And that makes me angry.
                ]],
                DIALOG_WON = [[
                    {dead?
                        * {zealot} won't be bothering you any more.
                    }
                    {not dead?
                        agent:
                            !injured
                        player:
                            !happy
                            So, did I pass the test? Am I innocent now?
                        agent:
                            !exit
                        player:
                            Yeah, that's what I thought!
                    }
                ]]
            }
            :Fn(function(cxt) 
                cxt:TalkTo("zealot")
                cxt:Dialog("DIALOG_INTRO")

                cxt:RunLoop(function() 
                    cxt:BasicNegotiation("CONVINCE")
                        :OnSuccess()
                            :Travel()

                    cxt:Opt("OPT_DEFEND")
                        :Dialog("DIALOG_DEFEND")
                        :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE, on_runaway = StateGraphUtil.DoRunAwayNoFail}
                            :OnWin()
                                :Dialog("DIALOG_WON")
                                :Travel()
                end)
            end)


QDEF:AddConvo("return_to_denikus", "denikus")
    :Loc{

        OPT_GET_PAID = "Get paid",
        DIALOG_PAID = [[
            player:
                The heist is done!
            agent:
                Everything went smoothly.
                Good to know I can trust you.
        ]],

        OPT_CONVINCE = "Tell {agent} about the heist",
        DIALOG_CONVINCE = [[
            player:
                The heist is done!
            agent:
                And all of the lumin is accounted for?
            player:
                Well, most of it. You know how it is.
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            player:
                The derricks are sloppy these days, shipping off tanks at half capacity.
            agent:
                Is that right?
                You know, considering the delicate nature of the business we're in together, you'd think you wouldn't want to risk losing my trust.
                But here you are. 
            player:
                Uh, I don't--
            agent:
                Don't lie to me again. It hurts my feelings, and makes me doubt you as an ally.
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                One of the tanks had a leak. We noticed before we got too far, managed to patch it up.
                But if it's short, that's why.
            agent:
                See? You made a good fixer after all.
                Keep this up and you'll be a heist expert in no time.
        ]],

    }
    :Hub(function(cxt) 
        
        --make sure you account for the kickback here. You might have to convince denikus to complete the quest successfully
        if cxt.quest.param.side_deal then

            cxt:BasicNegotiation("CONVINCE")
                :OnSuccess()
                    :Fn(function() ConvoUtil.GiveQuestRewards(cxt) end)
                    :CompleteQuest()
                :OnFailure()
                    :ReceiveOpinion("mistrust")
                    :Fn(function() 
                        cxt.quest.param.poor_performance = true
                        ConvoUtil.GiveQuestRewards(cxt) 
                    end)
                    :CompleteQuest()

        else
            cxt:Opt("OPT_GET_PAID")
                :SetQuestMark()
                :PreIcon( global_images.hire )
                :Dialog("DIALOG_PAID")
                :Fn(function() ConvoUtil.GiveQuestRewards(cxt) end)
                :CompleteQuest()
        end


    end)


--- add a delayed followup with the transporter, where you get your kickback.


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}







local QDEF = QuestDef.Define{
        id = "SIDE_SMITH_GREAT_LUMIN_HEIST_LATER",
        qtype = QTYPE.STORY,
        is_in_hiding = function(quest, agent)
            return agent == quest:GetCastMember("transporter")
        end,   
    }

    :AddCast{
        cast_id = "transporter",
    }

    :AddDormancyState("wait", "follow_up", true, 4, 10, true)
    :AddObjective{
        id = "follow_up",
    }

    :AddOpinionEvents{
        good_deal = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Had profitable dealings with them",
        },--]]
        bad_deal = 
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Shook them down for more money in a business deal",
        }
    }

    :AddConvo("follow_up")
        :ConfrontState("CONF", function(cxt) 
                                    return TheGame:GetGameState():CanSpawnTravelEvent() and 
                                            cxt.location:HasTag("in_transit")
                                end)
        :Loc{
            DIALOG_INTRO = [[
                
                * {transporter} flags you down.
                player:
                    !left
                agent:
                    !right
                    !greeting
                    {player}! I've been looking all over for you.
                    I have your share of the money from the heist!
            ]],

            OPT_TAKE_MONEY = "Take the money",
            DIALOG_TAKE_MONEY = [[
                player:
                    Thank you!
                agent:
                    Pleasure doing business with you, grifter.
                    !exit
            ]],
            OPT_DEMAND_MORE = "Demand a bigger cut",
            DIALOG_DEMAND_MORE = [[
                player:
                    I think I deserve a little more than that.
                    Eh?
                agent:
                    Well, I guess since you are a terrifying grifter and all, I'll give you a little more.
                    !give
                    But I'm mighty insulted!
                    !exit
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest:Complete()
            cxt:TalkTo("transporter")
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_TAKE_MONEY")
                :Dialog("DIALOG_TAKE_MONEY")
                :ReceiveOpinion("good_deal")
                :ReceiveMoney(100)
                :Travel()
            cxt:Opt("OPT_DEMAND_MORE")
                :Dialog("DIALOG_DEMAND_MORE")
                :ReceiveOpinion("bad_deal")
                :ReceiveMoney(200)
                :Travel()

        end)


