local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT
local FOLLOW_UP



local QDEF = QuestDef.Define{
        title = "Rare Imports",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{moreef} is looking to get some special stock for the bar.]],
        icon = engine.asset.Texture("icons/quests/side_smith_rare_imports.tex"),
        focus = QUEST_FOCUS.BALANCED,
        tags = {"MOREEF_INTRO_JOB"},
        act_filter = "SMITH",
        on_init = function(quest) 
            
            quest.param.lumin_wine = true
        end,

    }

    :AddCastByAlias{
        cast_id = "moreef",
        alias = "SWEET_MOREEF",
        provider = true,
        no_validation = true,
    }

    :AddCast{
        cast_id = "jake",
        when = QWHEN.MANUAL,
    }

    :AddCast{
        cast_id = "priest",
        when = QWHEN.MANUAL,
    }


    :AddQuestLocation{
        cast_id = "skiff",
        name = "Jakes Dock",
        plax = "EXT_SMITH_WARF",
        map_tags = {"port"},
        indoors = false,
    }


    :AddObjective{
        id = "go_to_skiff",
        title = "Get the Goods",
        mark = {"skiff"},
        on_complete = function(quest)
            quest:Activate("return_goods_to_moreef")
        end,
    }

    :AddObjective{
        id = "return_goods_to_moreef",
        title = "Deliver the goods to {moreef}",
        mark = {"moreef"}
    }
    :AddObjective{
        id = "return_to_moreef_empty",
        title = "Tell {moreef} the bad news",
        mark = {"moreef"}
    }


    :AddOpinionEvents{

        bribed_priest =  
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Paid a bribe to the cult for them",
        },
        killed_priest =  
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Killed a member of the Cult on their property",
        },
        annoyed_priest =  
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Brought the authorities down on their business",
        },
    }



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            {lumin_wine?
                agent:
                    You ever had aqua lumina?
                player:
                    That stuff the priests drink? Of course not.
                    Vix had it when he was initiated. Made his eyeballs glow for a month.
                agent:
                    W-well, the pilgrims want it. The nobles want it. 
                    Pretty much everyone wants to drink the blood of Hesh, or whatever they think it is. 
                    And these days, with lumin pouring off that thing on the beach, I have a lead on some unlicensed distillation.
                player:
                    Hah! Is that all? 
                    I've had a friend or two in the jakes. I know something about moving contraband.
                agent:
                    Well, I could use it. And if we're "friends" and all, I look f-forward to seeing you d-deliver.
                }

        ]] 
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                The jakes are laying low but I have a contact who can get you in touch.
            
            {lumin_wine?
                player:
                    What d'ya want me to do with the stuff? Should I sample it, make sure it's legit?
                agent:
                    Good point. Could be diluted lumin, made to look drinkable. 
                    Drinking lumin will make you go blind. And sterile. And probably strip half your brain cells. 
                player:
                    Maybe just a sip, then.
            }
        ]] 
    }
    :State("START")

        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Activate("go_to_skiff")
        end)


                                

--you go to the skiff, but the jakes don't want to deal with you, because they don't know you.
QDEF:AddConvo("go_to_skiff")                                
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("skiff") end)
        :Loc{
            DIALOG_INTRO = [[
                * The jakes have always had a foothold in the Foam, providing goods from the mainland that don't go through Supplicant City tithing. 
                * Now, though, they've really come into their element, what with the glut of pilgrims clogging up the supply lines. 
                * Likely they know this position makes them even bigger targets than usual, and they're jumpy when you arrive.
                agent:
                    !right
                    Who's there?
                player:
                    !left
                    Hey, bud, relax. Just coming to talk business.
                agent:
                    !crossed
                    We don't got business with a grifter like you.
                    Shove off.
            ]],

            OPT_THREATEN = "Threaten {agent}",
            DIALOG_THREATEN = [[
                player:
                    I'm here for product, bud. Sweet Moreef's got his name in and I'm here to pick up.
                    And if you don't want to give it to me, I'm gonna have to take it. 
                    Which would you prefer?
            ]],
            DIALOG_THREATEN_FAILURE = [[
                player:
                    I'm not leaving until I get what I want.
                agent:
                    !fight
                    Then I guess you're not leaving. 'Cause Sweet Moreef doesn't work with oozers like you.
            ]],

            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !sigh
                    ...
                    !fight
                    Let's go!
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead, so you help yourself to the goods.
                }
                {not dead?
                    agent:
                        !injured
                    player:
                        Now, as I was about to explain, I'm here on behalf of my good friend {moreef}. 
                        I'll take a crate of your finest seashine.
                    agent:
                        !give
                        Th-that's worth a lot of shills. I can't just hand it over to someone I don't know.
                    player:
                        What, we don't know each other? We just fought, didn't we?
                        We may as well be broodlings. 
                    * You pick up the crate, glass bottles clattering heavily within.
                }
            ]],

            DIALOG_THREATEN_SUCCESS = [[
                player:
                    Of course, I could always summon a Luminari or two. I'm a Banquod, after all.
                    We're <i>holy</i>.
                agent:
                    !placate
                    H-hey, you never said nothing about no nobility.
                    We don't want any trouble with the Cult. And Moreef's tab is paid, so these goods are yours.
                player:
                    S'what I thought. You should really introduce yourself up front, you know.
                    It's good manners.
                agent:
                    !exit

                * {agent} scurries off to sort through the piled crates.
            ]],

            OPT_CONVINCE = "Convince {agent} you're representing {moreef}",
            DIALOG_CONVINCE = [[
                player:
                    Hey, I'm just the pack oshnu. All I want is to pick up a package for {moreef}.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    He sent me here 'cause he thought you'd have it.
                    Do I gotta go back and report otherwise?
                agent:
                    No, no, we have it. You're working for {moreef}, huh?
                    Wouldn't expect him to trust something like this to someone else.
                player:
                    I'm special.
                agent:
                    Uh huh. You and everyone else in the Foam.
                    Lemme find that order for you.
                    !exit
                    * {agent} disappears behind crates and paperwork.
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    C'mon, I came all this way to pick up the order. You gonna send me back without it?
                agent:
                    You bet. {moreef} didn't say he was sending someone in his stead.
                    So I'm disinclined to believe you.
            ]],
        }
        :Fn(function(cxt) 
                local jakes = CreateCombatParty("JAKES_PATROL", cxt.quest:GetRank(), cxt.location)
                cxt.quest:AssignCastMember("jake", jakes[1])
                
                cxt:TalkTo(jakes[1])
                cxt:Dialog("DIALOG_INTRO")

                cxt:RunLoop(function() 
                    
                    cxt:Opt("OPT_CONVINCE")
                        :Dialog("DIALOG_CONVINCE")
                        :Negotiation{}
                            :OnSuccess()
                                :Dialog("DIALOG_CONVINCE_SUCCESS")
                                :GoTo("STATE_HESH_SHAKEDOWN")
                            
                            :OnFailure()
                                :Dialog("DIALOG_CONVINCE_FAILURE")

                    cxt:Opt("OPT_THREATEN")
                        :Dialog("DIALOG_THREATEN")
                        :Negotiation{ flags = NEGOTIATION_FLAGS.INTIMIDATION }
                            :OnSuccess()
                                :Dialog("DIALOG_THREATEN_SUCCESS")
                                :Fn(function() 
                                    cxt.quest.param.threatened_jakes = true
                                end)
                                :GoTo("STATE_HESH_SHAKEDOWN")
                            
                            :OnFailure()
                                :Dialog("DIALOG_THREATEN_FAILURE")
                                :Fn(function() 

                                    cxt:Opt("OPT_DEFEND")
                                        :Dialog("DIALOG_DEFEND")
                                        :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
                                            :OnWin()
                                                :Dialog("DIALOG_WON")
                                                
                                                :Fn(function() 
                                                    cxt.quest.param.fought_jakes = true
                                                    cxt.quest:Complete("go_to_skiff")
                                                    StateGraphUtil.AddLeaveLocation(cxt) 
                                                end)
                                end)
                end)

        end)
    
    --the jakes are in the back, when a priest shows up threatening to bust the operation. 
    :State("STATE_HESH_SHAKEDOWN")
        :Loc{
            DIALOG_INTRO = [[
                * You hum an old school song to pass the time.
                * Just when you're getting to the good part, your performance is interrupted by the dour presence of the Cult.
                agent:  
                    !right
                    Ah, I thought the jakes might be proliferating.
                    And what are you, grifter? A jake in training? Or a client?
                    Show me the contraband.
                player:
                    Aw, come on.
                    These aren't mine. I'm just... standing next to them.
                agent:
                    !dubious
                    This isn't going to go well for you.
            ]],
            OPT_BRIBE = "Bribe {agent}",
            DIALOG_BRIBE = [[
                player:
                    Surely you have riffraff to police, hmm?
                    What's the matter with letting me barter for a few silks? My tithes are paid and my family's in good standing.
                agent:
                    Are you... a noble? 
                    Forgive me for my skepticism. Most don't deal with the jakes directly.
                player:
                    Sure, you're just doing your job. How about a contribution, to show my, uh, devotion?
                    You know. For the Cult. To build an orphanage or whatever. 
                    !give
            ]],
            DIALOG_POST_BRIBE = [[
                agent:
                    Ah, that would be... welcome.
                    The Cult has increased expenses during these unusual times. 
                * {agent} counts the money.
                agent:
                    May you walk in the shallows, sire. 
                    !exit
            ]],
            OPT_THREATEN = "Scare {agent} away",
            DIALOG_THREATEN = [[
                player:
                    !threaten
                    How about you just turn around and walk right back into the ocean, squidlicker?
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                player:
                    I'm Smith Banquod. You wanna mess with me?
                agent:
                    Banquod? Vixmalli's ignoble brother?
                player:
                    Hey! He can disown me, but my blood's still as blue as his.
                    And those were my parents Hesh chose to crush, too. You wanna mess with this?
                agent:
                    !scared
                    I would need to consult the tides before I knew if it was my purpose.
                    So I'll leave you be, for now. May Hesh guide my hand tomorrow.
                    !exit
                player:
                    !happy
                * {agent} runs off, lumin-loving tail between {agent.hisher} robes.
            ]],
            DIALOG_THREATEN_FAILURE = [[
                agent:
                    You think you're scary?
                    Hesh sees right through you, grifter.
                    And so do I.
            ]],
            OPT_BEAT_UP = "Beat some sense into {agent}",
            DIALOG_BEAT_UP = [[
                player:
                    I've got 1, 2, 3, 4, 5, 6 reasons for you to get out of here!
                agent:
                    That's a very specific numb—
                player:
                    !fight
                agent:
                    Oh, you're counting up your fingers. 
                    !fight
                    Cute.

            ]],
            DIALOG_WON = [[
                {dead?
                    * You see a brief flash of terror in {agent}'s dying eyes as {agent.heshe} meets Hesh.
                }
                {not dead?
                    agent:
                        !injured
                        Blasphemy!
                    player:
                        Uh huh. 
                        You were shaking me down in the name of your god.
                    agent:
                        !exit
                    * {agent} limps off.
                }
            ]]
        }
        :Fn(function(cxt) 

                local priest = AgentUtil.GetFreeAgent("PRIEST", function(agent) return agent:GetRelationship() <= RELATIONSHIP.NEUTRAL end)
                cxt.quest:AssignCastMember("priest", priest)

                cxt:TalkTo(priest)
                cxt:Dialog("DIALOG_INTRO")

                cxt:RunLoop(function(cxt)
                    
                    cxt:Opt("OPT_BRIBE")
                        :Dialog("DIALOG_BRIBE")
                        :DeliverMoney(cxt.quest:GetRank()*50)
                        :Dialog("DIALOG_POST_BRIBE")
                        :Fn(function() 
                            cxt.quest.param.bribed_priest = true
                        end)
                        :GoTo("STATE_GET_GOODS")

                    cxt:Opt("OPT_THREATEN")
                        :Dialog("DIALOG_THREATEN")
                        :Negotiation{ flags = NEGOTIATION_FLAGS.INTIMIDATION }
                            :OnSuccess()
                                :Dialog("DIALOG_THREATEN_SUCCESS")
                                :Fn(function() 
                                    cxt.quest.param.scared_priest = true 
                                    
                                    cxt:GetAgent():MoveToLimbo()
                                end)
                                :GoTo("STATE_GET_GOODS")
                            :OnFailure()
                                :Dialog("DIALOG_THREATEN_FAILURE")

                    cxt:Opt("OPT_BEAT_UP")
                        :Dialog("DIALOG_BEAT_UP")
                        :Fn(function() cxt.quest.param.fought_priest = true end)
                        :Battle{advantage = TEAM.BLUE}
                            :OnWin()
                                :Dialog("DIALOG_WON")
                                :Fn(function() 
                                    if cxt:GetAgent():IsDead() then
                                        cxt.quest.param.priest_body_present = true
                                    else
                                        cxt:GetAgent():MoveToLimbo()
                                    end
                                end)
                                :GoTo("STATE_GET_GOODS")

                end)
        end)


    -- the jakes come back with the goods. There may or may not be a dead priest lying on the floor here.
    :State("STATE_GET_GOODS")
        :Loc{
            DIALOG_INTRO = [[
                
                {priest_body_present?
                    * {agent} comes back, carrying a small crate.
                    agent:
                        !right
                        What was all of that noise—
                    * {agent.HeShe} almost drops the crate when {agent.heshe} sees the body of the priest lying at your feet.
                        !surprised
                        What the hell?!
                    player:
                        Huh?
                        Oh, this?
                        I'm pretty sure that was here when I got here.
                    agent:
                        We have a deal with the local authorities, you idiot!
                        We pay them a small bribe every so often, and they don't bring the power of the Cult down on us.
                    player:
                        !happy
                        Now you don't have to pay any more! You should thank me!
                    agent:
                        !angry
                        Take the goods, and leave.
                        And tell {moreef} that {moreef.hisher} unit price just doubled.
                        !exit
                    * {agent} storms off to fetch something to clean up the mess.
                }

                {not priest_body_present?

                    * {agent} comes back, carrying a small crate.
                    agent:
                        !right
                        !agent
                        Who were you talking to?
                    player:
                        Some priest bozo. Don't worry. I took care of it.
                    agent:
                        You...what? We have an arrangement with the Cult. 
                    
                    {bribed_priest?

                        player:
                            Don't worry. I covered your bribe this time.
                        agent:
                            Oh. Thank you.
                            Be seeing you, grifter.
                    }
                    
                    {fought_priest?

                        player:
                            Well, I arranged {priest.hisher} face a little.
                            No permanent damage!
                        agent:
                            ...
                        player:
                            !happy
                            You're welcome?
                        agent:
                            Take the goods, and get out.
                            !exit
                    }

                    {scared_priest?
                        player:
                            I used my words!
                        agent:
                            !sigh
                            This is going to come back on us.
                            Just take the stuff and get out of here before you cause more problems.
                            !exit
                    }

                }
            ]]
        }
        :Fn(function(cxt)
            cxt:TalkTo("jake")
            cxt:Dialog("DIALOG_INTRO")
            
            if cxt.quest.param.bribed_priest then
                cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("bribed_priest") )
            elseif cxt.quest.param.priest_body_present then
                cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("killed_priest") )
            else
                cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("annoyed_priest") )
            end
            cxt.quest:Complete("go_to_skiff")
            StateGraphUtil.AddLeaveLocation(cxt) 
        end)



QDEF:AddConvo("return_goods_to_moreef")
    :ConfrontState("CONF", function(cxt) return not cxt.quest.param.mugged_yet  end)
        :Loc{
            DIALOG_INTRO = [[
                * You are minding your own business, carrying a valuable crate of contraband goods, when you are are approached by some riff-raff.
                player:
                    !left
                agent:
                    !right
                    You there!
                    Hand over the box, or you're going to get hurt.
            ]],

            OPT_GIVE_SOME = "Convince {agent} to take just some of the goods",
            DIALOG_GIVE_SOME = [[
                player:
                    I get it. You're a bandit. You do banditry.
                    But these don't belong to me. They belong to a friend of mine.
                    So if I give them to you, I'll be letting my friend down.
                    And that will make them very sad. Which will also make me sad, because I'm a good friend.
                agent:
                    ...
                player:
                    How about I give you just a little bit, so that nobody is the wiser?
            ]],
            DIALOG_GIVE_SOME_SUCCESS = [[
                agent:
                    Well, Ok. That's easier than cracking your head open.
                player:
                    Aw, what a charmer. Now get out of here, ya little scamp.
                agent:
                    !exit
                * You tighten your grip on the remaining goods.
            ]],
            DIALOG_GIVE_SOME_FAILURE = [[
                agent:
                    No, that's not how banditing works.
                    I demand your stuff. You give it to me.
                    It's easy!
            ]],

            OPT_GIVE_ALL = "Hand over the goods",
            DIALOG_GIVE_ALL = [[
                player:
                    You know what? This isn't worth the trouble.
                    Take it.
                agent:
                    !happy
                    Really?
                    Wow, usualy people complain, or put up a fight, or something.
                    This was so easy!
                    Thanks, sucker!
                    !exit
                * {agent} leaves, along with whatever remained of your dignity.
            ]],

            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Aw, c'mon! I worked hard for this!
            ]],
            DIALOG_WON = [[
                {dead?
                    * You ponder the fragility of life, and question some of the choices that have led to this moment.
                    player:
                        !burp
                        BURP!
                    * It passes.
                }
                {not dead?
                    agent:
                        !injured
                    player:
                        Get out of here before you make me angry, you lousy bandit!
                    agent:
                        !exit
                    * The old neighborhood isn't what is used to be.
                }
            ]]

        }
        :Fn(function(cxt) 
            local bandits = CreateCombatParty("BANDIT_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(bandits[1])
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                    
                cxt.quest.param.mugged_yet = true

                cxt:Opt("OPT_GIVE_SOME")
                    :Dialog("DIALOG_GIVE_SOME")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_GIVE_SOME_SUCCESS")
                            :Fn(function() cxt.quest.param.gave_some_goods = true end)
                            :Travel()
                        
                        :OnFailure()
                            :Dialog("DIALOG_GIVE_SOME_FAILURE")
                        


                cxt:Opt("OPT_GIVE_ALL")
                    :Dialog("DIALOG_GIVE_ALL")
                    :Fn(function() 
                        cxt.quest:Cancel("return_goods_to_moreef")
                        cxt.quest:Activate("return_to_moreef_empty")
                    end)
                    :Travel()
                
                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :Travel()
            end)

        end)




QDEF:AddConvo("return_to_moreef_empty", "moreef")
    :Loc{
        OPT_TELL = "Tell {moreef} what happened",
        DIALOG_TELL = [[
            agent:
                Where are the goods?
            player:
                So, there was a complication...
            agent:
                You know what, forget it.
                I'll just have to figure something else out.
                Try harder next time, {player}.
        ]]
    }
    

    :Hub(function(cxt) 

        cxt:Opt("OPT_TELL")
            :SetQuestMark()
            :Dialog("DIALOG_TELL")
            :FailQuest()

    end)        

QDEF:AddConvo("return_goods_to_moreef", "moreef")
    :Loc{
        OPT_GIVE_GOODS = "Give {moreef} the goods",
        DIALOG_GIVE_GOODS = [[
            player:
                !give
                {gave_some_goods? I have absolutely 100% of the goods for you! | I have the goods for you! }
            agent:
                I hope you didn't run into trouble.
            

            {fought_jakes and not killed_priest?
                player:
                    You might want to send a fruit basket over to the Jakes.
                agent:
                    Why?
                player:
                    No reason. Just... send the basket.
                agent:
                    Hrmmm.
            }

            {killed_priest and not fought_jakes?
                player:
                    Nah, I just had to kill a priest. Nothing too hairy.
                agent:
                    Ha. Good one.
                    Wait. You're joking, right?
                player:
                    ...
                    Yes?
                player:
                    Hrmmm.
            }

            {not killed_priest and not fought_jakes?
                player:
                    !happy
                    Nothing I couldn't handle!
            }

            {lumin_wine?
                agent:
                    Well, the good news is that we're stocked with Lumin wine now. I'll let you buy them at cost, because you're so helpful.
                player:
                    I have to pay? What is this, the Grout Bog?
                agent:
                    I know better than to give you an open bar.
            }

        ]]
    }
    

    :Hub(function(cxt) 

        cxt:Opt("OPT_GIVE_GOODS")
            :SetQuestMark()
            :Dialog("DIALOG_GIVE_GOODS")
            :CompleteQuest()
            :Fn(function() 
                ConvoUtil.GiveQuestRewards(cxt)
                local good
                if cxt.quest.param.lumin_wine then
                    good = "lumin_wine"
                end
                QuestUtil.SpawnQuest( "MOREEF_SELL_SPECIAL_GOODS", {parameters = { goods = good } } )
            end)

    end)                                




local QDEF = QuestDef.Define{
        id = "MOREEF_SELL_SPECIAL_GOODS",
        qtype = QTYPE.STORY,
        on_init = function(quest)
            quest.param.stock = 3
        end,
        
        events = {
            phase_change = function(quest)
                quest.param.stock = 1 + math.random(3)
            end,
        },
    }
    :AddCastByAlias{
        cast_id = "moreef",
        alias = "SWEET_MOREEF",
        no_validation = true,
    }
    :AddConvo(nil, "moreef")
        :Loc{

            OPT_BUY_GOOD = "Buy {1#card}",
            DIALOG_BUY_GOODS = [[
                player: 
                    !point
                    I'll take one of those.
                agent:
                    !give
                    My pleasure.
            ]],
            REQ_OUT_OF_STOCK = "{moreef} is out of stock for now. Try again later.",
            TT_STOCK_LEFT = "Remaining stock: {1}",

        }
        :Hub(function(cxt) 

            if cxt.quest.param.goods then
                cxt:Opt("OPT_BUY_GOOD", cxt.quest.param.goods)
                    :ReqCondition( cxt.quest.param.stock > 0, "REQ_OUT_OF_STOCK")
                    :PostText("TT_STOCK_LEFT", cxt.quest.param.stock)
                    :Dialog("DIALOG_BUY_GOODS")
                    :DeliverMoney(20, { is_shop = true } )
                    :GainCards{cxt.quest.param.goods}
                    :Fn(function() cxt.quest.param.stock = cxt.quest.param.stock - 1 end)
            end

        end)


local battle_defs = require "battle/battle_defs"
--TODO: Make this more intersting
Content.AddBattleCard("lumin_wine", 
{
    name = "Lumin Wine",
    anim = "drink_fancy",
    desc = "{DRINK} and {HEAL} {1} health.",
    flavour = "'I don’t know where it came from, but this was a very good year.'",

    desc_fn = function(self, fmt_str)
        return loc.format(fmt_str, self.healing_amt )
    end,

    cost = 1,
    
    item_tags = ITEM_TAGS.COMBAT,
    rarity = CARD_RARITY.RARE,
    flags = battle_defs.CARD_FLAGS.SKILL | battle_defs.CARD_FLAGS.CONSUME | battle_defs.CARD_FLAGS.ITEM,
    target_type = TARGET_TYPE.SELF,
    series = "SMITH",

    healing_amt = 6,

    OnPostResolve = function( self, battle, attack )
        self.owner:HealHealth(self.healing_amt)
        local cards = { }
        local card = Battle.Card( "empty_bottle", self.owner )
        card.show_dealt = false
        table.insert(cards, card)
        battle:DealCards( cards, battle:GetDiscardDeck() )
        battle:BroadcastEvent( BATTLE_EVENT.DRINK, battle, 1, cards )
    end
})

