local battle_defs = require "battle/battle_defs"
local BATTLE_EVENT = battle_defs.BATTLE_EVENT


local QDEF = QuestDef.Define{
        title = "Render Unto Hesh",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{moreef} needs you to get one of two rival tithe-collectors off {moreef.hisher} back.]],
        icon = engine.asset.Texture("icons/quests/side_smith_render_unto_hesh.tex"),
        focus = QUEST_FOCUS.BALANCED,
        tags = {"MOREEF_INTRO_JOB"},
        act_filter = "SMITH",
        --disabled = true,
        on_init = function(quest) 
        end,
    
        --this is a little bit of boilerplate that lets us test the quest directly from the quest menu
        on_debug_start = function(quest)
            TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("MOREEF_BAR"))
            --UIHelpers.DoSpecificConvo( quest:GetCastMember("moreef"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
            quest:Activate("start")
        end,

    }


    :AddDefCastSpawn("luminitiate", "LUMINITIATE")
    :AddDefCastSpawn("priest", "PRIEST")
    :AddDefCastSpawn("zealot", "ZEALOT")
    :AddDefCastSpawn("luminari", "LUMINARI")
    :AddDefCastSpawn("merchant", "WEALTHY_MERCHANT")

    
    :AddCastByAlias{
        cast_id = "moreef",
        alias = "SWEET_MOREEF",
        provider = true,
        no_validation = true,
    }


    :AddLocationCast{
        cast_id = "compound",
        cast_fn = function(quest, t)
            table.insert(t,  TheGame:GetGameState():GetLocation("PEARL_CULT_COMPOUND"))
        end,
    }

    :AddLocationCast{
        cast_id = "training",
        cast_fn = function(quest, t)
            table.insert(t,  TheGame:GetGameState():GetLocation("PEARL_CULT_TRAINING"))
        end,
    }

    :AddQuestLocation{
        cast_id = "ambush_site",
        name = "Ambush Spot",
        plax = "EXT_SMITH_ISOLATED_DESERT",
        map_tags = {"port"},
    }

    :AddQuestLocation{
        cast_id = "zealot_loc",
        name = "Zealot's last known location",
        plax = "EXT_SMITH_BEACHCREATURE",
        map_tags = {"cliffs"},
    }



    :AddObjective{
        id = "start",
        hide_in_overlay = true,
        on_activate = function(quest)
            quest:Activate("talk_to_priest")
            quest:Activate("talk_to_luminitiate")
            quest:Complete("start")
        end,
    }

    :AddObjective{
        id = "talk_to_priest",
        title = "Talk to {priest}",
        desc = "{priest} is very comfortably ensconced in the Cult hierarchy.",
        mark = {"priest"},
        on_complete = function(quest)
            quest:Activate("convince_zealot")
        end,

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("priest"), location = quest:GetCastMember("compound")} )
        end,

    }

    :AddObjective{
        id = "convince_zealot",
        title = "Convince {zealot} of {luminitiate}'s heresy",
        desc = "{luminitiate} is getting too powerful for someone so low in the Cult hierarchy. If {zealot} sees this, {zealot.heshe} might attempt an arrest.",
        mark = {"zealot"},

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("priest"), location = quest:GetCastMember("compound")} )
            table.insert (t, { agent = quest:GetCastMember("zealot"), location = quest:GetCastMember("zealot_loc")} )
        end,

        on_complete = function(quest)
            quest:Cancel("talk_to_luminitiate")
            quest:Cancel("set_up_ambush")
            quest:Activate("arrest_luminitiate")
        end,
    }

    :AddObjective{
        id = "arrest_luminitiate",
        title = "Entrap {luminitiate}",
        desc = "{zealot} is going to watch you entrap {luminitiate}, and then spring out to arrest {luminitiate.himher}.",
        mark = {"luminitiate"},
        on_complete = function(quest)
            quest:Activate("return_to_moreef")
        end,
        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("luminitiate"), location = quest:GetCastMember("training")} )
        end,
    }


    :AddObjective{
        id = "talk_to_luminitiate",
        title = "Talk to {luminitiate}",
        desc = "{luminitiate} is leading a band of rogue luminitiates who have been terrorizing the local populace.",
        mark = {"luminitiate"},
        on_complete = function(quest)
            quest:Activate("set_up_ambush")
        end,
    
        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("luminitiate"), location = quest:GetCastMember("training")} )
        end,
    }

    :AddObjective{
        id = "set_up_ambush",
        title = "Set up the ambush",
        desc = "{priest} will be collecting from {merchant} soon. Convince {merchant} to help you set up an ambush.",
        mark = {"merchant"},
        on_activate = function(quest) 
            quest:GetCastMember("merchant"):GetBrain():MoveToHome()
        end,

        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("merchant") then
                fighter:AddCondition("WANTED_ALIVE")
            end
        end,


        on_complete = function(quest)
            quest:Cancel("convince_zealot")
            quest:Cancel("talk_to_priest")
            quest:Activate("ambush_priest")
        end,
    }

    :AddObjective{
        id = "ambush_priest",
        title = "Ambush {priest}",
        desc = "{merchant} has set the trap. Now go teach {priest} a lesson.",
        mark = {"ambush_site"},
        on_complete = function(quest)
            quest:Activate("return_to_moreef")
        end,

        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("priest") then
                fighter:AddCondition("WANTED_DEAD")
            end
        end,
    }
    
    :AddObjective{
        id = "return_to_moreef",
        title = "Return to {moreef}",
        mark = {"moreef"}
    }

    :AddObjective{
        id = "failure_case",
        title = "Return to {moreef}",
        desc = "You messed this one up. Time to break the bad news to {moreef}.",
        mark = {"moreef"}
    }


    :AddOpinionEvents{
        got_confession = {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Rooted out heresy in the Cult with them",
        },
        entrapped = {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Entrapped them with a Zealot",
        },
        did_not_get_confession= {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Botched an entrapment scheme",
        },
    }


QDEF:AddIntro(
    --attract spiel
    [[
        agent:
            I got Heshians coming out the wazoo. It's bad enough under the best of times keeping up with the tithes, but after the Beaching? 
            F-forget about it.
            Now I've got <i>two</i> Heshians coming round to bother me for tithes, and they aren't comparing notes.
            One wants it for the church, the other wants it for a beach clean-up operation.  
        player:
            You tell one about the other?
        agent:
            Yeah! But they tell me to ignore the other and pay them.
            I'm stuck between them, with no way forward that doesn't mean spending my last shill.
    ]],
    
    --on accept
    [[
        agent:
            So there's {priest}, and then {luminitiate}. 
            {priest} is corrupt enough to keep the tithes to {priest.himher}self, so it doesn't surprise me the Luminitiate doesn't want {priest.himher} paid. 
            {luminitiate} is just a Luminitiate, but they can be scarier than the higher ranks because they got something to prove.
        player:
            I was almost one of those, ya know. Before I said "Hesh no, don't ask me again" and split. 
        agent:
            You as a Luminari? Now that's a scary thought.
        player:
            Enh, don't waste too much brainpower on it. I wouldna made the cut even if I tried.

    ]],
    "start"
    )




QDEF:AddConvo("talk_to_luminitiate")
    :ConfrontState("MEET", function(cxt) return cxt.location == cxt.quest:GetCastMember("training") end)
        :Fn(function(cxt) 
            if not cxt.quest.param.has_access_to_training_facility then
                cxt:GoTo("STATE_FIGHT")
            else
                cxt:GoTo("STATE_ACCESS")
            end
        end)
    
    :State("STATE_FIGHT")
        :Loc{
            DIALOG_INTRO = [[
                * You arrive at the training facility. It smells like effort and fervor.
                luminitiate:
                    !right
                    Hey! Recruit! You're late!
                player:
                    !left
                    Wha--?
                    Who, me?
                luminitiate:
                    !angry
                    <b>Did I say you could talk</b>?
                    !neutral
                    Hesh demands you hold your tongue, unless you've proven yourself worthy of speaking in its name.
                player:
                    !shrug
                agent:
                    !right
                * The assembled Luminitiates form a circle around you. There's no escape!
            ]],

            DIALOG_ON_WIN = [[
                * You catch your breath.
                player:
                    Seems kinda unfair... you made your pets do it.
            ]],
        }
        :Fn(function(cxt)
            local monsters = CreateCombatParty({"LUMICYTE", "LUMICYTE"})
            cxt:TalkTo(monsters[1])
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_DEFEND")
                :Battle{
                    no_oppo_limit = true,
                    enemies = monsters,
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.NO_BYSTANDERS
                }
                    :OnWin()
                        :Fn(function(cxt) cxt.quest.param.has_access_to_training_facility = true end)
                        :Dialog("DIALOG_ON_WIN")
                        :GoTo("STATE_ACCESS")
        end)
    :State("STATE_ACCESS")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        !left
                    agent:
                        !right
                        You did pretty well for a greenhorn!
                    player:
                        I'm not a recruit, ya doof. I just had some questions.
                    agent:
                        No kiddin'? Then you got off easy.
                        I would've made you fight <i>two</i> rounds if I'd known you were just a gawker.
                }

                {not first_time?
                    player:
                        !left
                    agent:
                        !right
                        You're back? I'd offer you another lumicyte fight, but we're fresh out.
                }
            ]],

            OPT_ASK_HERE = "Ask about this place",
            DIALOG_ASK_HERE = [[    
                player:
                    What is this place, anyway?
                agent:
                    This is the future of the Cult, of course.
                    Each recruit you see is fighting for the chance to be a servant to the tides.
                    We abandon all ego, and accept only purpose.
                player:
                    That true?
                agent:
                    Of course! A Luminari has to be a vessel without pride.
                    Mind, I'm better at it than most.

            ]],
            OPT_ASK_MOREEF = "Ask about {moreef}'s tithes",
            DIALOG_ASK_MOREEF = [[
                player:
                    Why are you charging {moreef} a tithe?
                    He already pays his tithes. He can't afford another.
                agent:
                    {moreef} pays his tithes to a pretender! 
                    If Hesh doesn't exert its will through {priest}, it's because {priest.heshe} is too bloated and lazy to move.
                    <i>We</i> are the ones doing beach patrols, honoring the beast. 
                    We bathe in its blubber and cleanse ourselves with the foamy brine at its mouth.
                    Or what we think is its mouth.
                player:
                    Do the fancy bishops in Supplicant City know you're double-dipping like this?
                agent:
                    They aren't here. The Great Beaching is. 
                    Hesh's will seems clear.
            ]],


            OPT_PROPOSAL = "Offer to deal with {priest}",
            DIALOG_PROPOSAL = [[
                player:
                    So what do you want us to do about {priest}?
                    'Cause {priest.heshe}'s just as insistent as you are that {moreef} keeps paying.
                agent:
                    Hm. Perhaps Hesh has brought you here for a reason then. Perhaps your purpose is to... dispose of {priest}.
                player:
                    Uh, goodie?
                agent:
                    {priest} will be collecting from {merchant} later today. 
                    Convince {merchant} to change the meeting location to somewhere out of the way.
                    Then you can...
                    !throatcut
                    ...
                player:
                    And what will you be doing while I do all of your dirty work?
                agent:
                    Training for a glorious future.
                    Now go! Embody Hesh's will!
                * {agent} shoos you out the door, clearly thinking you should embody it somewhere else.
            ]],
        }

        :Fn(function(cxt) 
            cxt:TalkTo("luminitiate")
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 

                cxt:Question("OPT_ASK_HERE", "DIALOG_ASK_HERE")
                cxt:Question("OPT_ASK_MOREEF", "DIALOG_ASK_MOREEF")
                --cxt:Question("OPT_ASK_PRIEST", "DIALOG_ASK_PRIEST")

                cxt:Opt("OPT_PROPOSAL")
                    :SetQuestMark()
                    :Dialog("DIALOG_PROPOSAL")
                    :CompleteQuest("talk_to_luminitiate")
                    :Travel()

                StateGraphUtil.AddLeaveLocation(cxt) 
            end)
        end)        




QDEF:AddConvo("talk_to_priest")
    :ConfrontState("MEET", function(cxt) return cxt.location == cxt.quest:GetCastMember("compound") and not cxt.quest.param.has_access_to_priest_building end)
        :Loc{
            DIALOG_INTRO = [[
                
                {first_time?
                    * The Hesh Compound is, in a word, stuffy.
                    * You are stopped by a dour-looking luminari.
                    player:
                        !left
                    agent:
                        !right
                        The compound is closed to non-members, by order of the Bishop.
                }
                {not first_time?
                    player:
                        !left
                    agent:
                        !right
                        I told you before, the compound is closed to the likes of you.
                }
            ]],

            OPT_BRIBE = "Bribe {agent}",
            DIALOG_BRIBE = [[
                player:
                    Come on, pal. I got a desperate need to speak to a priest, and a couple shills for you if you'll help it happen.
                    !give
                agent:
                    ...
                    Very well.
                    
                * {agent} lets you in.
            ]],

            OPT_CONVINCE = "Convince {agent} you have urgent business inside",
            DIALOG_CONVINCE = [[
                player:
                    Please, I have to talk to {priest}.
                    It's urgent Cult business!
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                agent:
                    If you're on official business, you may enter.
                player:
                    Great!
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                agent:
                    No. I have very strict orders.
                player:
                    But--
                priest:
                    !left
                    What's all this commotion?
                agent:
                    This grifter is trying to get in to see you.
                    But he's being rather unconvincing about it.
                priest:
                    Hmmm. Interesting.
                    Well, my day has been boring so far. Let them in. I'll hear them out.
                    !exit
                * You can practically see the waves of annoyance radiating off of the Luminari.
                player:
                    !left
                    !happy
                luminari:
                    Go on in. But don't try anything funny!

            ]],
        }
        :Fn(function(cxt) 
            
            cxt:TalkTo("luminari")
            
            cxt:Dialog("DIALOG_INTRO")
        
            cxt:RunLoop(function() 

                cxt:Opt("OPT_BRIBE")
                    :Dialog("DIALOG_BRIBE")
                    :DeliverMoney(50)
                    :Fn(function() cxt.quest.param.has_access_to_priest_building = true end)
                    :DoneConvo()
                
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :Fn(function() cxt.quest.param.has_access_to_priest_building = true end)
                        :DoneConvo()
                    :OnFailure()
                        :Fn(function() cxt.quest.param.has_access_to_priest_building = true end)
                        :DoneConvo()
            
                StateGraphUtil.AddLeaveLocation(cxt)
            end)

        end)


QDEF:AddConvo("talk_to_priest", "priest")
    :ConfrontState("MEET", function(cxt) return cxt.quest.param.has_access_to_priest_building end)
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    agent:
                        !right
                        You're not supposed to be in here.
                    player:
                        Why does everyone keep saying that?
                        I'm here on behalf of {moreef}.
                    agent:
                        Ah, yes, the barkeep. A very lucrative parishioner.
                    player:
                        {moreef.HeShe} won't be if {luminitiate} keeps double-dipping from {moreef.hisher} pockets.
                    agent:
                        Ah, yes. The new recruits are always overzealous, but the Great Beaching has whipped them into a frenzy.
                        Youth will be youth, eh?
                }

                {not first_time?
                    agent:
                        Are you here to pester me with more worldly concerns?
                }
            ]],

            OPT_ASK_LUMINITIATE = "Ask about {luminitiate}",
            DIALOG_ASK_LUMINITIATE = [[
                player:
                    Why is the Cult letting {luminitiate} collect tithes?
                agent:
                    It's not. But with the pilgrims about, it's difficult to keep track of the ledgers.
                    Resources are stretched thin, and some of the initiates are taking advantage of the lapsed supervision.
                    Not to mention allowing dangerous ideology to incubate and... fester.
                    There's no doubt that the Great Beaching is a sign. But the ranks are torn asunder trying to interpret its meaning.
                    {luminitiate} in particular is espousing dangerous idolatry, bordering on heresy.
            ]],

            OPT_PROPOSAL = "Offer to deal with {luminitiate}",
            DIALOG_PROPOSAL = [[
                player:
                    Maybe I could do something to help with your {luminitiate} problem?
                agent:
                    Perhaps...
                    If you could convince {zealot} to open an investigation, I'm sure {zealot.heshe} could find something.
                    With their heresy exposed, the Cult hierarchy will have no choice but to act.
                player:
                    I meant maybe I could just hit them really hard or something? This all sounds very dry.
                agent:
                    They're too strong to attack directly. Trust me, this is the only way.
                    Now go, I have shills to count.
                * {agent} signals to {agent.hisher} security, who pushes you out the door.
            ]],
        }

        :Fn(function(cxt) 
            cxt:TalkTo("priest")
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 

                cxt:Question("OPT_ASK_LUMINITIATE", "DIALOG_ASK_LUMINITIATE")

                cxt:Opt("OPT_PROPOSAL")
                    :SetQuestMark()
                    :Dialog("DIALOG_PROPOSAL")
                    :CompleteQuest("talk_to_priest")
                    :Travel()

                StateGraphUtil.AddLeaveLocation(cxt) 
            end)
        end)                



QDEF:AddConvo("set_up_ambush", "merchant" )
    
    :Loc{
        OPT_ASK_ABOUT_PRIEST = "Ask {agent} about {priest}...",
        DIALOG_ASK_ABOUT_PRIEST = [[    
            {first_time?
                player:
                    I hear you've got a meeting with {priest} today.
                agent:
                    Yeah, what's it to you?
            }

            {not first_time?
                player:
                    So, about {priest}...
                agent:
                    Your interest is sort of suspicious, you know.
            }
        ]],

        DIALOG_BACK = [[
            player:
                Let's talk about something else.
        ]],
        OPT_BRIBE = "Bribe {agent}",
        DIALOG_BRIBE = [[
            player:
                Thing is, {priest} is about to have a fall from grace. It'll be a good idea for you to establish, uh, new connections.
                If you hold the meeting somewhere else, somewhere I choose--and take these shills for your trouble--then you'll have friends on the other side.
                !give
            agent:
                !take 
                I was just thinking that the old meeting point was feeling a little stale.
                And I have been feeling like I could treat myself, lately.
        ]],

        DIALOG_NEW_LOCATION = [[
            agent:
                Where do you want want the meeting to happen?
            player:
                Somewhere...
                !thought
                ...
                ...secluded.
            
            player:
                !exit
            agent:
                I have regrets.
        ]],

        OPT_CONVINCE = "Convince {agent} to move the meeting site",
        DIALOG_CONVINCE = [[
            player:
                Thing is, {priest} is about to have a fall from grace. It'll be a good idea for you to establish, uh, new connections.
                Only thing you gotta do is move the meeting. Simple, right?
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                It'll be good for your future profits. 
            agent:
                Hm, and my future prospects, I suspect. 
                Fine, I'll play.
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            player:
                But you just need to move it somewhere nice and secluded, where there can be no witnesses.
                Easy-peasy!
            agent:
                I'm going to say... that's a no.
        ]],

        OPT_THREATEN = "Bully {agent} into changing the meeting site",
        DIALOG_THREATEN = [[
            player:
                !threaten
                Listen, dweeb. You're going to move the site!
        ]],
        DIALOG_THREATEN_SUCCESS = [[
            agent:
                !scared
                Fine! I'll do it! Just stop yelling!
        ]],
        DIALOG_THREATEN_FAILURE = [[
            agent:
                You can't push me around like that!
                I have more money than you!
        ]],

        OPT_FORCE = "Beat up {agent}",
        DIALOG_FORCE = [[
            player:
                !fight
                I'm going to move your facial features around, and then you're going to move the meeting site!
        ]],
        DIALOG_WON = [[
            agent:
                !injured
                Stop! I'll do what you want!
        ]],
        DIALOG_KILLED = [[
            * {agent} is dead. And so is your chance of getting {priest} out into the open.
        ]],
        FIGHT_REASON_BEAT_UP = "Intimidate {merchant}!"

    }


    :Hub(function(cxt) 

        cxt:Opt("OPT_ASK_ABOUT_PRIEST")
            :IsHubOption(true)
            :SetQuestMark()
            :Dialog("DIALOG_ASK_ABOUT_PRIEST")
            :LoopingFn(function() 
                
                cxt:Opt("OPT_BRIBE")
                    :Dialog("DIALOG_BRIBE")
                    :DeliverMoney(50)
                    :CompleteQuest("set_up_ambush")
                    :Dialog("DIALOG_NEW_LOCATION")
                    :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                    :DoneConvo()

                if not cxt.quest.param.tried_to_ask_merchant then
                    cxt:BasicNegotiation("CONVINCE")
                        :Fn(function() cxt.quest.param.tried_to_ask_merchant = true end)
                        :OnSuccess()
                            :CompleteQuest("set_up_ambush")
                            :Dialog("DIALOG_NEW_LOCATION")
                            :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                            :DoneConvo()

                    cxt:BasicNegotiation("THREATEN", {flags = NEGOTIATION_FLAGS.INTIMIDATION})
                        :Fn(function() cxt.quest.param.tried_to_ask_merchant = true end)
                        :OnSuccess()
                            :CompleteQuest("set_up_ambush")
                            :Dialog("DIALOG_NEW_LOCATION")
                            :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                            :DoneConvo()

                end

                cxt:Opt("OPT_FORCE")
                    :Dialog("DIALOG_FORCE")
                    :Battle{
                        reason_txt = cxt:GetLocString("FIGHT_REASON_BEAT_UP"),
                    }
                        :OnWin()
                            :Fn(function() 
                                if cxt:GetAgent():IsDead() then
                                    cxt:Dialog("DIALOG_KILLED")
                                    cxt.quest:Fail("set_up_ambush")
                                    if cxt.quest:IsFailed("convince_zealot") then
                                        cxt.quest:Activate("failure_case")
                                    end
                                    StateGraphUtil.AddEndOption(cxt)
                                else
                                    cxt:Dialog("DIALOG_WON")
                                    cxt.quest:Complete("set_up_ambush")
                                    cxt:Dialog("DIALOG_NEW_LOCATION")
                                    cxt:GetAgent():MoveToLimbo() 
                                    StateGraphUtil.AddEndOption(cxt)
                                end
                            end)

                StateGraphUtil.AddBackButton(cxt)
                    :Dialog("DIALOG_BACK")
            end)

    end)

QDEF:AddConvo("ambush_priest")
    :ConfrontState("MEET", function(cxt) return cxt.location == cxt.quest:GetCastMember("ambush_site") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find {priest} and {merchant}.
                merchant:
                    !left
                priest:
                    !right
                    Why did you want to meet here this time?
                merchant:
                    I'm so sorry!
                player:
                    !left
                    Hey!
                priest:
                    What is the meaning of this?!
                player:
                    {luminitiate} sends {luminitiate.hisher} warmest regards.
                priest:
                    !hesh_greeting
            ]],

            OPT_DEMAND = "Demand {agent} leave the Pearl",
            DIALOG_DEMAND = [[
                player:
                    Look, pal, it's honestly easiest if I just kill you.
                    But because I'm a such nice guy, I'll let you live if you skip town!
            ]],
            DIALOG_DEMAND_SUCCESS = [[
                priest:
                    I know when I'm beat. I'll leave. 
                    I've always wanted to retire somewhere nice.
                    !exit
                * {luminitiate} might not fully accept that outcome, but at least {moreef} will only have one pocket to fill.
            ]],
            DIALOG_DEMAND_FAILURE = [[
                priest:
                    !angry
                    I'm a priest of Cult of Hesh. I have gazed upon the troubled waters and faced down that which gazed back!
                    I'm not afraid of a grifter like you!
            ]],

            OPT_FIGHT = "Attack {agent}",
            DIALOG_FIGHT = [[
                player:
                    !fight
                    Nothing personal!
            ]],
            DIALOG_WON = [[
                {dead?
                    * {priest} is dead. You wonder what this will do to local Cult politics.
                }
                {not dead?
                    priest:
                        !injured
                        Why didn't you finish me off?
                    player:
                        !shrug
                        I have my reasons.
                        You're going to leave town now, and never come back. Got it?
                    priest:
                        I- I will.
                        !exit
                    * {luminitiate} might not fully accept that outcome, but at least {moreef} will only have one pocket to fill.
                }
            ]],
            FIGHT_REASON = "Kill {priest}!",

        }
        :Fn(function(cxt)
            cxt:TalkTo("merchant")
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:TalkTo("priest")
            

            cxt:RunLoop(function(cxt)

                cxt:BasicNegotiation("DEMAND", {flags = NEGOTIATION_FLAGS.INTIMIDATION})
                    :OnSuccess()
                        :Fn(function() 
                            cxt:GetAgent():Retire()
                            cxt.quest:Complete("ambush_priest")
                            StateGraphUtil.AddLeaveLocation(cxt) 
                        end)
                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{
                        reason_txt = cxt:GetLocString("FIGHT_REASON"),                        
                    }
                        :Dialog("DIALOG_WON")
                        :CompleteQuest("ambush_priest")
                        :Travel()
            end)
        end)


QDEF:AddConvo("convince_zealot", "zealot" )
    :ConfrontState("MEET")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?

                    * You find {zealot} communing with the Beaching, holding tightly to a lumin-drenched piece of ambergris.
                    player:
                        Alright, there, {zealot}? 
                    zealot:
                        !right
                        Are you a pilgrim? What brings you to this holiest of places?
                    player:
                        Well... for one, my parents died here.
                        Or maybe it was a little ways further down.
                        Hard to tell on account of how huge that thing is.
                    zealot:
                        Oh, I see. You're one of the Banquod brood, are you?
                        Hesh's will has exerted itself upon you. Truly a blessing.
                    player:
                        Oh yeah, I feel proper blessed.
                        The other thing is {priest} sent me to find you.
                    zealot:
                        Oh?
                }
                
                {not first_time?
                    * You find {zealot}, still in the same place.
                    zealot:
                        !right
                        Have you come to commune once more with Hesh's emissary?
                }

            ]],

            --OPT_ASK_ABOUT_BEACHING = "Ask about the Beaching",
            --DIALOG_ASK_ABOUT_BEACHING =[[
                --player:
                    --What?
                --agent:
                    --Yup.
            --]],
            --OPT_ASK_ABOUT_BEING_A_ZEALOT = "Ask {agent} about their role in the Cult.",
            --DIALOG_ASK_ABOUT_BEING_A_ZEALOT =[[
                --player:
                    --What?
                --agent:
                    --Yup.
            --]],

            OPT_CONVINCE = "Convince {zealot} to investigate {luminitiate}",
            DIALOG_CONVINCE = [[
                player:
                    {priest} has reason to think {luminitiate} might be playing fast and loose with the Cult's teachings.
                    Getting a little too close to the heresy department.
                    You got any thoughts on that?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    I went by and visited {luminitiate} myself, ya know. And something felt... off.
                    I couldn't say what it was, but I just really wanted to punch {luminitiate.himher} right in the face.
                agent:
                    Is that right? 
                    That impulse may very well have been Hesh's will moving through you. 
                player:
                    No! Ya don't say? Should I have acted on it, then?
                agent:
                    Yes! For whatever reason, Hesh moves through you. Do not fight it, or you will surely drown in the Abyss.
                    For my part, I feel compelled to caution. I would like some evidence of {luminitiate}'s heresy before I act.
                player:
                    Well... <i>I</i> feel compelled to help you with that!
                agent:
                    Aha! Yes! Hesh's power is great. 
                    It is... so fulfilling, submitting oneself to Hesh's will and seeing the tendrils align.
                    I'll be waiting for the fates to give us our moment. May you soon again walk in the shallows, but for now--may Hesh hold you aloft.
                    !exit
            ]],

            DIALOG_CONVINCE_FAILURE = [[
                player:
                    I just don't like what {luminitiate}'s saying. It's too radical!
                agent:
                    I see. Change is difficult, it's true. But look around you.
                    We cannot deny that these are strange times. One thing is clear: Hesh is asking us to listen. 
                    It's unclear who will speak for Hesh, so we must watch, and wait.
                player:
                    Aw, man.
                agent:
                    Unless, of course, you wish to be tested.
                    I can grant you a chance to see if your will is an extension of Hesh's:
                    Trial by bladder!
                player:
                    Eh?
                agent:
                    I will summon a pair of heshbladders, and you will dispatch them as a test of faith!
                player:
                    Oh, that's not what I was thinking. 
                    Yours is better.
                    But not by a lot.
            ]],
            OPT_DEFEAT_BLADDERS = "Attempt trial by bladder",
            DIALOG_DEFEAT_BLADDERS = [[
                player:
                    Alright. Lemme at those bladders!
                agent:
                    Yes! Feel yourself possessed by the will of Hesh!
            ]],
            DIALOG_WON = [[
                * You defeat the bladders, and now you smell like pee. This seems to get {agent} pretty excited, though.
                agent:
                    For whatever reason, Hesh moves through you. Do not fight it, or you will surely drown in the Abyss.
                    For my part, I feel compelled to caution. I would like some evidence of {luminitiate}'s heresy before I act.
                player:
                    Well... <i>I</i> feel compelled to help you with that!
                agent:
                    Aha! Yes! Hesh's power is great. 
                    It is... so fulfilling, submitting oneself to Hesh's will and seeing the tendrils align.
                    I'll be waiting for the fates to give us our moment. May you soon again walk in the shallows, but for now--may Hesh hold you aloft.
                    !exit
            ]]

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
            
                --cxt:Question("OPT_ASK_ABOUT_BEACHING", "DIALOG_ASK_ABOUT_BEACHING")
                --cxt:Question("OPT_ASK_ABOUT_BEING_A_ZEALOT", "DIALOG_ASK_ABOUT_BEING_A_ZEALOT")

                if not cxt.quest.param.failed_zealot_negotiation then
                    cxt:BasicNegotiation("CONVINCE")
                        :OnSuccess()
                            :CompleteQuest("convince_zealot")
                            :Travel()

                        :OnFailure()
                            :Fn(function() cxt.quest.param.failed_zealot_negotiation = true end)
                            :Fn(function() cxt.quest.param.bladders = CreateCombatParty({"HESHBLADDER", "HESHBLADDER"}, 1, cxt.location) end)
                else
                    cxt:Opt("OPT_DEFEAT_BLADDERS")
                        :Dialog("DIALOG_DEFEAT_BLADDERS")
                        :Battle{
                            enemies = cxt.quest.param.bladders,
                            noncombatants = {"zealot"},
                            flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.NO_BYSTANDERS
                        }
                            :OnWin()
                                :Dialog("DIALOG_WON")
                                :CompleteQuest("convince_zealot")
                                :Travel()
                end
                
                StateGraphUtil.AddLeaveLocation(cxt) 
            end)

        end)




QDEF:AddConvo("arrest_luminitiate", "luminitiate" )
    :ConfrontState("MEET")
        :Loc{
            DIALOG_INTRO = [[
                * You find {luminitiate}, deep in training that looks more furious than it is effective.
                luminitiate:
                    !right
                    Are you here to bulk up for Hesh?
                player:
                    !left
                    Oh yeah, I want to get ripped. For... Hesh?
                    Sure.
            ]],

            OPT_ENTRAP = "Entrap {luminitiate}",
            DIALOG_ENTRAP = [[
                player:
                    So, that stuff you were saying earlier. About the priests being too lazy for Hesh to move...
            ]],
            DIALOG_ENTRAP_SUCCESS = [[
                player:
                    Can you maybe go into detail on that? Like shouldn't Hesh be powerful enough to move anybody?
                agent:
                    Perhaps. Or perhaps not.
                    Why send the beast, hmm, unless it was an attempt to reconnect?
                    Perhaps we've lost our way. Perhaps Hesh only moves a select few.
                    Like myself, for instance.
                zealot:
                    !left
                    Hold your tongue, {luminitiate}, before you say something that only the Abyss can swallow.
                agent:
                    {zealot}?! I was just... I mean... 
                    It was a test! To see if this pilgrim was worthy of our teachings!
                zealot:
                    Perhaps that's true. My gut tells me... that only an interrogation will appease my doubts.
                    Rejoice, {luminitiate}, for your chance to prove you are afloat in the currents.
                    !exit
                agent:
                    !exit
                * {zealot} quickly incapacitates {agent}, moving with a speed that leaves the other luminitiates rubbing their eyes.
                player:
                    !left
                zealot:
                    !right
                    We've done it, {player}. For your sake, we may hope that this was the only task Hesh had planned for you.
                    But I would be honored if our paths crossed again.
                    !exit
            ]],

            DIALOG_ENTRAP_FAILURE = [[
                player:
                    Can you explain that to me? Loudly enough for everyone to hear?
                agent:
                    I don't think so, pilgrim. 
                    Since we last spoke I have contemplated my gains, and it has humbled me.
                    I am prostrate before the will of Hesh.
            ]],

            OPT_ATTACK = "Attack",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    Change of plans!
                zealot:
                    !left
                    !happy
                    Oooh! Are we sparring?
            ]],

            DIALOG_WON_ZEALOT_DEAD = [[
                {dead?
                    * There are a whole lot of dead bodies on the ground. You decide to leave.
                    player:
                        !exit
                }
                {not dead?
                    player:
                        Whatcha gotta say about your ego now, heretic?
                    agent:
                        !right
                        !injured
                        The zealots are on to us! We gotta go underground!
                        !exit
                    * The remaining Luminitiates run away.
                }
            ]],
            DIALOG_WON_ZEALOT_ALIVE = [[
                {dead?
                    zealot:
                        !right
                        That was... abrupt. I thought you planned to get a confession first?
                    player:
                        What can I say? Hesh's will overcame me. 
                    zealot:
                        !sigh
                        Then only time will tell the results.
                        !exit
                }
                {not dead?
                    agent:
                        !injured
                        You beat me?! Impossible!
                    zealot:
                        !right
                        That was... abrupt. I thought you planned to get a confession first?
                    player:
                        What can I say? Hesh's will overcame me. 
                    zealot:
                        !sigh
                        Then only time will tell the results.
                        And {luminitiate} remains. 
                        !exit
                }
            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 
                cxt:BasicNegotiation("ENTRAP",{suppressed = {cxt:GetCastAgent("zealot")}})
                    :OnSuccess()
                        :ReceiveOpinion("entrapped")
                        :ReceiveOpinion("got_confession", nil, "zealot")
                        :Fn(function() cxt:GetAgent():Retire() end)
                        :CompleteQuest("arrest_luminitiate")
                        :Travel()
                
                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{allies = {"zealot"}}
                        :OnWin()
                            :Fn(function() 
                                cxt:Dialog(cxt.quest:GetCastMember("zealot"):IsDead() and "DIALOG_WON_ZEALOT_DEAD" or "DIALOG_WON_ZEALOT_ALIVE")

                                if not cxt:GetAgent():IsDead() then
                                    cxt:GetAgent():Retire()
                                end

                                cxt.quest:Complete("arrest_luminitiate")
                                StateGraphUtil.AddLeaveLocation(cxt) 
                            end)
                            :ReceiveOpinion("did_not_get_confession", nil, "zealot")
            end)
        end)




QDEF:AddConvo("return_to_moreef", "moreef")
    :ConfrontState("MEET")
        :Loc{
            DIALOG_INTRO_PRIEST = [[
                moreef:
                    !left
                priest:
                    !right
                    The, ah, clerical error has been corrected, and you should only be paying the regular tithes once again.
                moreef:
                    Can I have next month's off to make up for it? 
                priest:
                    Ah, normally, I would allow it. But, uh, Hesh is telling me I cannot.
                    Sorry.
                player:
                    !left
                priest:
                    Ah! {player}! Thank you for dealing with {luminitiate}!
                    As thanks, I'd like to deputize you as a formal tithe collector. 
                    You may claim one tithe per day--to spend however Hesh compels you, of course.
                player:
                    Hey, not bad!
                priest:
                    !exit
                moreef:
                    !right
                    How does Hesh compel you?
                player:
                    Guess we'll see tomorrow! But I kinda think booze will be involved.
                moreef:
                    Thanks for fixing the tithe situation. Kinda wish I could get a refund, but fool on me to wish for anything, huh?
            ]],
            DIALOG_INTRO_LUMINITIATE = [[
                moreef:
                    !left
                luminitiate:
                    !right
                    Now that the corruption has been uprooted, you don't have to pay that old crook anymore.
                    You just have to pay me!
                moreef:
                    Oh, okay! Do you think I could start getting receipts for my tithes, or...
                luminitiate:
                    I wanna say yes, but Hesh is saying... no.
                player:
                    !left
                luminitiate:
                    Hey {player}! That old priest is gone. That's great!
                    If you ever need backup, swing by the center. I'll lend you a recruit.
                player:
                    Sounds good. I can whip 'em into shape for you.
                luminitiate:
                    !exit
                moreef:
                    !right
                    Thanks, {player}! You just halved my tithes!
            ]],
        }
    :Fn(function(cxt) 
        if cxt.quest:IsComplete("arrest_luminitiate") then
            cxt:Dialog("DIALOG_INTRO_PRIEST")
            QuestUtil.SpawnQuest( "SMITH_DAILY_TITHE" )
        else
            cxt:Dialog("DIALOG_INTRO_LUMINITIATE")
            QuestUtil.SpawnQuest( "SMITH_DAILY_LUMINITIATE" )
        end
        
        ConvoUtil.GiveQuestRewards(cxt)
        cxt.quest:Complete()
        StateGraphUtil.AddEndOption(cxt)        
    end)


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}





local QDEF = QuestDef.Define{
        id = "SMITH_DAILY_TITHE",
        qtype = QTYPE.STORY,
        icon = engine.asset.Texture("icons/quests/smith_daily_tithe.tex"),
        on_init = function(quest)
            quest.param.ready = true
        end,
        
        events = {
            phase_change = function(quest)
                if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY then
                    quest.param.ready = true
                end
                
            end,
        },
    }
    :AddObjective{
        id = "start",
        state = QSTATUS.ACTIVE,
    }
    :AddConvo("start")
        :Loc{
            REQ_DAILY = "You already did this today",
            OPT_COLLECT_TITHE = "Collect a tithe",
            DIALOG_COLLECT_TITHE = [[
                player:
                    Pay up. It's for Hesh.
            ]],
            DIALOG_COLLECT_TITHE_FAILURE = [[
                agent:
                    What? I paid my tithe today already.
                    I think you're mistaken.
            ]],
            DIALOG_COLLECT_TITHE_SUCCESS = [[
                agent:
                    !sigh
                    Again?
                    !give
            ]],
        }
        :Hub(function(cxt) 
            local amount = cxt:GetAgent():GetRenown() * 20
            
            cxt:Opt("OPT_COLLECT_TITHE")
                :ReqCondition(cxt.quest.param.ready, "REQ_DAILY")
                :Dialog("DIALOG_COLLECT_TITHE")
                :PostText( "RECEIVE_MONEY", amount )
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_COLLECT_TITHE_SUCCESS")
                        :ReceiveMoney(amount)
                        :Fn(function() cxt.quest.param.ready = false end)

                    :OnFailure()
                        :Dialog("DIALOG_COLLECT_TITHE_FAILURE")
                        :Fn(function() cxt.quest.param.ready = false end)
                
        end)



local QDEF = QuestDef.Define{
        id = "SMITH_DAILY_LUMINITIATE",
        icon = engine.asset.Texture("icons/quests/smith_daily_luminitiate.tex"),
        qtype = QTYPE.STORY,
        on_init = function(quest)
            quest.param.ready = true
        end,
        
        events = {
            phase_change = function(quest)
                if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY then
                    quest.param.ready = true
                end
                
            end,
            quests_changed = function( quest )
                if TheGame:GetGameState():IsHiringAvailable() then
                    if quest:GetObjectiveState("shop_open") == QSTATUS.INACTIVE then
                        quest:SetObjectiveState("shop_open", QSTATUS.ACTIVE)
                    end
                else
                    if quest:GetObjectiveState("shop_open") == QSTATUS.ACTIVE then
                        quest:SetObjectiveState("shop_open", QSTATUS.INACTIVE)
                    end
                end
            end,

        },
    }
    :AddObjective{
        id = "shop_open",
        state = QSTATUS.ACTIVE,
        title = "Luminitiates",
        desc = "Luminitiates are waiting for jobs here.",
        
        mark = function(quest, t, in_location)
            if quest.param.ready then
                table.insert(t, quest:GetCastMember("training"))
            end
        end,
    }
    :AddLocationCast{
        cast_id = "training",
        cast_fn = function(quest, t)
            table.insert(t,  TheGame:GetGameState():GetLocation("PEARL_CULT_TRAINING"))
        end,
    }
    :AddConvo("shop_open")
        :Loc{
            REQ_DAILY = "You already did this today",
            
            OPT_HIRE_MERC = "Take {agent} on your next job",
            DIALOG_HIRED = [[
                player:
                    !permit
                    $miscAlluring
                    You want some real-world experience?
                agent:
                    !shrug
                    Let's roll!
            ]],
            OPT_REPLACE_MERC = "Replace {old_merc} with {agent}",
            DIALOG_REPLACED_MERC = [[
                player:
                    !permit
                    $miscAlluring
                    I hope you can do better than {old_merc}.
                agent:
                    !hesh_greeting
                    Hesh will wake!
            ]],
        }
        :Hub(function(cxt) 
            if cxt.location == cxt.quest:GetCastMember("training") and cxt:GetAgent():GetContentID() == "LUMINITIATE" then
                
                local hireling = TheGame:GetGameState():GetCaravan():GetHireling()
                cxt:ReassignCastMember("old_merc", hireling)

                if hireling == nil then
                    cxt:Opt("OPT_HIRE_MERC")
                        :ReqCondition(cxt.quest.param.ready, "REQ_DAILY")
                        :Fn(function(cxt) 
                            cxt.quest.param.ready = false
                            cxt:Dialog("DIALOG_HIRED")
                            cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.HIRED )
                            StateGraphUtil.AddEndOption(cxt)
                        end)

                else
                    cxt:Opt("OPT_REPLACE_MERC")
                        :ReqCondition(cxt.quest.param.ready, "REQ_DAILY")
                        :Fn(function(cxt) 
                            cxt.quest.param.ready = false
                            cxt:Dialog("DIALOG_REPLACED_MERC")
                            hireling:Dismiss()
                            cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.HIRED )
                            StateGraphUtil.AddEndOption(cxt)
                        end)
                end

            end
                
        end)

