local QDEF = QuestDef.Define{
        title = "New Deal, Same as the Old Deal",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[]],
        --disabled = true,
        --icon = engine.asset.Texture("icons/quests/side_smith_seemli_triangular_firing_squad.tex"),
        focus = QUEST_FOCUS.BALANCED,
        tags = {"SEEMLI_JOB"},
        act_filter = "SMITH",
        --disabled = true,

        on_debug_start = function(quest)
            local friend = TheGame:GetGameState():AddSkinnedAgent( "ADMIRALTY_CLERK" )
            friend.social_connections:SetRelationship(RELATIONSHIP.LIKED, TheGame:GetGameState():GetPlayerAgent())
            quest:GetCastMember("seemli"):MoveToLocation(TheGame:GetGameState():GetLocation("MOREEF_BAR"))
            TheGame:GetGameState():GetCaravan():MoveToLocation(quest:GetCastMember("seemli"):GetLocation())
            UIHelpers.DoSpecificConvo( quest:GetCastMember("seemli"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
        end,

        collect_agent_locations = function(quest, t)
            --table.insert (t, { agent = quest:GetCastMember("assassin"), location = quest:IsActive("meet_assassin_post_kill") and quest:GetCastMember("cliffside") or quest:GetCastMember("park"), role = CHARACTER_ROLES.VISITOR} )
        end,

        avoids = {
            {"new_supplier", "supporter"},
            {"new_supplier", "ad_friend"},
            {"supporter", "ad_friend"},
        },

        on_destroy = function(quest) 
            local member_ids = {"supporter"}

            Iter.array(member_ids)
                :for_each(function(cast_id) 
                        if quest:GetCastMember(cast_id) and quest:GetCastMember(cast_id):GetParty() then quest:GetCastMember(cast_id):Dismiss() end
                    end)
        end,

        is_in_hiding = function(quest, agent)
            return agent == quest:GetCastMember("old_supplier")
        end,
    }

    :AddCastByAlias{
        cast_id = "seemli",
        alias = "SEEMLI",
        provider = true,
        no_validation = true,
    }

    :AddDefCastSpawn("new_supplier", "JAKES_SMUGGLER")
    :AddDefCastSpawn("old_supplier", "JAKES_SMUGGLER")
    :AddDefCastSpawn("supporter", "JAKES_LIFTER")
    
    :AddCast{
        cast_id = "ad_friend",
        when = QWHEN.MANUAL,
        optional = true,
        condition = function(agent, quest) 
            print( agent, agent:GetRelationship() > RELATIONSHIP.NEUTRAL, agent:GetFactionID() )
            return agent:GetRelationship() > RELATIONSHIP.NEUTRAL and agent:GetFactionID() == "ADMIRALTY"
        end,
    }

    :AddObjective{
        id = "start",
        hide_in_overlay = true,
        on_activate = function(quest) 
            quest:Activate("talk_to_new_supplier")
            quest:Complete("start")
        end,
    }

    :AddObjective{
        id = "talk_to_new_supplier",
        title = "Contact {new_supplier}",
        desc = "Convince {new_supplier} to work with the Rise.",
        mark = {"new_supplier"},
        on_complete = function(quest) 
            quest:Activate("get_support")
            quest:AssignCastMember("ad_friend")
            if quest:GetCastMember("ad_friend") then
                quest:Activate("talk_to_friend")
            end
            quest:Activate("seize_cache")
        end,
    }


    :AddObjective{
        id = "get_support",
        title = "Gather support",
        desc = "Convince {supporter} to help out in the coming battle.",
        mark = {"supporter"},
    }
    
    :AddObjective{
        id = "talk_to_friend",
        title = "Talk to {ad_friend} about the cache",
        desc = "Convince your friend {ad_friend} to reduce the guard on the cache.",
        mark = {"ad_friend"},
    }

    :AddObjective{
        id = "seize_cache",
        title = "Seize the weapons cache",
        desc = "Take back the Rise's cache of weapons from the Admiralty. Be forewarned, though - it will be heavily guarded!",
        mark = {"cache"},
        on_complete = function(quest) 
            quest:Activate("deliver_to_safehouse")
        end,
    }

    :AddObjective{
        id = "deliver_to_safehouse",
        title = "Deliver the codes",
        desc = "Tell the Rise that the weapons are ready for pickup.",
        mark = {"safehouse"},
        on_complete = function(quest) 
            quest:Activate("return_to_seemli")
        end,

    }

    :AddObjective{
        id = "rescue_old_supplier",
        title = "(Optional) Rescue {old_supplier}",
        desc = "The Admiralty is moving {old_supplier} out of the region. Stop them!",
        mark = {"rescue_point"},
        on_complete = function(quest) 
            if not quest:IsDone() then
                quest:OpinionEvent("old_supplier", "rescued")            
                quest:OpinionEvent("new_supplier", "rescued_rival")            
                quest.param.rescued_old_supplier = true
            end
        end,
    }

    :AddObjective{
        id = "return_to_seemli",
        title = "Return to {seemli}",
        desc = "Tell {seemli} that the new deal is in place.",
        mark = {"seemli"},
    }

    :AddQuestLocation{
        cast_id = "safehouse",
        name = "Rise Safehouse",
        plax = "INT_SMITH_RISE_CLUBHOUSE",
        show_agents = true,
        tags = {"building"},
        indoors = true,
    }

    :AddQuestLocation{
        cast_id = "rescue_point",
        name = "Departure point",
        plax = "EXT_Ship_Graveyard",
        map_tags = {"cliffs"},
        indoors = false,
    }

    :AddQuestLocation{
        cast_id = "cache",
        name = "Seized Goods",
        plax = "EXT_SMITH_SEIZED_GOODS",
        tags = {"building"},
    }        
------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------

    :AddOpinionEvents{
        killed_ads = 
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Killed Admiralty against their wishes",
        },--]]
        stole_from_rise = 
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "They think you stole from the Rise",
        },--]]
        explained_kill = 
        {
            delta = OPINION_DELTAS.NEUTRALIZING,
            txt = "Convinced them you were justified in killing their friends",
        },
        helped_get_back = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Helped them get back in the good graces of the Jakes",
        },
        took_side_deal = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Helped them turn a nice profit at the Rise's expense",
        },
        rejected_side_deal = 
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Refused to help them cheat the Rise",
        },
        rescued_rival = 
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Rescued their rival",
        },
        rescued = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Rescued them from the Admiralty",
        }
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------
QDEF:AddIntro(
    --attract spiel
    [[
        agent:
            The spirit is willing, but the body is wanting.
            As noble as our cause may be, we can't have a revolution without a chain of command. 
            Our jakes contact, {old_supplier}, has been captured by the Admiralty, with the supplies impounded.
            I need you to go where your gifts take you: retrieve my supplies, and find me a new supplier.
    ]],
    
    --on accept
    [[
        player:
            Where am I supposed to find you a new supplier?
        agent:
            Why, with the jakes, of course. 
            As much as I'd like to believe they can be compelled by the fervor of our cause, you'll likely need a compelling offer.
            Seek out {new_supplier} and see if you can't entice {new_supplier.himher} to sign on.

    ]],
    "start")

QDEF:AddConvo(nil, "new_supplier")
    :Loc{

        OPT_ASK_WEAPONS = "Ask {agent} about {seemli}'s weapons",
        DIALOG_ASK_WEAPONS = [[
            player:
                I'm supposed to talk to you about reclaiming some waylaid goods. You know what I mean?
            agent:
                Oh, that freedom fighter sent you, did she?
                I was expecting to see one of you much sooner. You wait too long in this business and you lose your shot.
                Well, ever since {old_supplier} got captured, I'm in charge. You want to do business?
                Then get those goods out of impound, and <i>pay for them</i>. 
            player:
                You mean pay for them again, right? They musta been paid for up front.
            agent:
                Actually, no. {old_supplier} was a softy and agreed to a handshake. 
                But we're gonna do things differently, aren't we?
                I'll mark your map with the impound lot. And for what it's worth, I think I saw {supporter} slumming it around there.
                {supporter.HeShe}'s another softy. Maybe you can convince {supporter.himher} to help.
        ]],
        DIALOG_ADMIRALTY_FRIEND = [[
            player: 
                I have some strings I can pull with the Admiralty.
            agent:
                Good for you.
                I don't care how it gets done, just get it done.
        ]]
    }
    :Hub(function(cxt) 

        if cxt.quest:IsActive("talk_to_new_supplier") then        
            cxt:Opt("OPT_ASK_WEAPONS")
                :Dialog("DIALOG_ASK_WEAPONS")
                :SetQuestMark()
                :Fn(function() 
                    cxt.quest:Complete("talk_to_new_supplier")

                    if cxt.quest:IsActive("talk_to_friend") then
                        cxt:Dialog("DIALOG_ADMIRALTY_FRIEND")
                    end

                end)
        else
           cxt:Opt("OPT_ASK_ABOUT")
               :IsHubOption(true)
               :GoTo("STATE_QUESTIONS")     
        end
    end)

    :AskAboutHub("STATE_QUESTIONS", 
             {

               "Ask about {old_supplier}",
               [[
                   player:
                        !dubious
                        You don't seem concerned about {old_supplier}'s wellbeing.
                   agent:
                        !handwave
                        Why should I be? {old_supplier.HeShe} and I didn't see eye-to-eye on the bottom line.
                    player:
                        Yeah, but I thought jakes looked out for each other. Nobody holds a jake in debt. 
                    agent: 
                        Sure, that's true in the rest of Havaria. But here in the Pearl? The money's too good.
                        I run my business my way. And now, {old_supplier}'s business is my business.
               ]],
               "Ask about Rise",
               [[
                   player:
                        I figured the jakes would be a little more sympathetic to the Rise.
                    agent:
                        Oh yeah? Because we're free-range laborers, you mean?
                    player:
                        More or less. Ain't that what the Rise is after, too? 
                    agent:
                        Dunno. Don't really care. 
                        But if the Rise all get what they want and then try to step in on the shipping business?
                        The jakes will shut them down right quick, you have my word on that.
               ]],
               "Ask about {supporter}",
               [[
                    player:
                        What's the deal with {supporter}?
                    agent:
                        {old_supplier}'s right hand. The two of them used to run a route between the Bread Fields and Lakespit.
                    player:
                        Huh. Talk about a rock and a hard place.
                    agent:
                        Yeah, well. Between the Admiralty and the Spark Barons, they saw enough to get soft on the Rise. 
                        Of course, they also couldn't have been farther from Pearl-on-the-Foam. 
                        When they landed here, the two of them had no idea what kind of a different beast the Admiralty could be.
                        I heard {old_supplier} tried to bribe the patrol. As if someone like {old_supplier} could've pulled that off here.
               ]]
        })  

QDEF:AddAttract("get_support", "supporter",
    [[
        {first_time?
            player:
                Hey, you {supporter}? {new_supplier} told me to talk to you.
            agent:
                Why? You here to rub it in? 
                It's not enough I've been kicked outta the jakes?
            player:
                That's the first I'd heard of it. {new_supplier} just mentioned you might wanna help me.
                I'm trying to get the seized goods outta the impound for the Rise. 
            agent:
                Huh. 

        }
        {not first_time?
            agent:
                We're not getting the goods back that easy.
                If you want to sit out here and sulk with me, though, I won't stop you.
        }
    ]])

QDEF:AddConvo("get_support", "supporter")
    :Loc{
        OPT_CONVINCE_HELP = "Convince {agent} to help you regain the cache",
        DIALOG_CONVINCE_HELP = [[
            player:
                Seems to me you got kicked outta the jakes trying to help the Rise. Is that right?
                Goods impounded, and you lose your cred?
            agent:
                Hey, goods get seized all the time. This ain't Deltree with its fancy taxes and trade laws.
                This is the frontier, and we're entrepreneurs.
                Naw. I'm in trouble because I didn't get paid in advance. That goes against the jakes code of conduct.
            player:
                So seems to me like we should work together on this.
        ]],
        DIALOG_CONVINCE_HELP_SUCCESS = [[
            player:
                {new_supplier} obviously wants me to help you. We get the goods, you get paid, and then you're back in the jakes.
            agent:
                Huh. Yeah. Yeah, actually, that just might work.
                It didn't kill me the first time, so let's give it a second shot.
            player:
                I'll stop you before we go for three, how about that?
        ]],
        DIALOG_CONVINCE_HELP_FAILURE = [[
            player:
                You help me and I swear the Rise will be good for the money.
            agent:
                No way, bud. That's how I got in this mess in the first place.
                Keep your promises outta my wallet. I need to move on and find new opportunities.
                !exit
            * {agent} leaves to start a new life, though you can't imagine the prospects are good.
        ]],

    }
    :Hub(function(cxt) 
        cxt:BasicNegotiation("CONVINCE_HELP")
            :OnSuccess()
                :Fn(function() 
                    cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                    cxt.quest:Complete("get_support")
                end)
            :OnFailure()
                :Fn(function() 
                    cxt:GetAgent():MoveToLimbo()
                    cxt.quest:Fail("get_support")
                end)
                :DoneConvo()
    end)


QDEF:AddAttract("talk_to_friend", "ad_friend",
    [[
        agent:
            !greeting
            Well hey, {player}. What brings you to my humble station?
            Not breaking any laws, I hope?
        player:
            Who, me?
    ]])

QDEF:AddConvo("talk_to_friend", "ad_friend")
    :Loc{
        OPT_CONVINCE_HELP = "Ask {agent} for help with the cache",
        DIALOG_CONVINCE_HELP = [[
            player:
                Maybe you could help me out with a little problem.
            * You express your conundrum with the seized goods in a very roundabout way, using more insinuation than fact.
                
        ]],
        DIALOG_CONVINCE_HELP_SUCCESS = [[
            player:
                See, those goods were supposed to be a present for my family. 
            agent:
                !dubious
                So you're telling me, an officer of the Admiralty, that you want me to help you steal contraband weapons?
            player:
                That's not so bad, is it? We cool?
            agent:
                ...
                Yeah, alright. But if anybody asks, we never spoke.
                I'll find other work for the guards to do. That should give you a chance to sneak in unnoticed.
            player:
                Hey, you're a real stand-up pal, you know that?
            agent:
                Just don't get any of my people killed, or we're going to have problems.
            player:
                Sure, sure. I won't hurt nobody.
        ]],
        DIALOG_CONVINCE_HELP_FAILURE = [[
            player:
                It was a bunch of weapons, maybe about yay big.
                They're just mine, y'see, and I want 'em.
            agent:
                Right, well... I'm afraid I can't help you there.
                Friend or no, those are catalogued dangerous contraband.
                We're already under a lot of pressure from New Deltree. Last thing I need is some military auditor on my tail.
            player:
                Ah, yeah. I guess it was a bit of a stretch, huh?
                Thanks all the same, I guess.
        ]],

    }
    :Hub(function(cxt) 
        cxt:BasicNegotiation("CONVINCE_HELP")
            :OnSuccess()
                :Fn(function() 
                    cxt.quest:Complete("talk_to_friend")
                    cxt.quest.param.deal_with_admiralty = true
                end)
                :DoneConvo()

            :OnFailure()
                :Fn(function() 
                    cxt:GetAgent():MoveToLimbo()
                    cxt.quest:Fail("talk_to_friend")
                end)
                :DoneConvo()
    end)


QDEF:AddConvo("seize_cache")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("cache") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find the seized cache.
                * It's in a bunch of access-locked crates. Each one opens with a special access code, which the Admiralty has in the manifest.
                
                {deal_with_admiralty?
                    * {ad_friend} was good to {ad_friend.hisher} word, and it seems lightly guarded.
                }

                {not deal_with_admiralty?
                    * There's a heavy guard presence.
                }
                
                agent:
                    !right
                    !angry
                    Hey!
                    Who goes there!?
                player:
                    !left
                    !greeting
                    I'm in the market for some black market weapons. Have you seen any around?
                agent:
                    You funny, or just stupid? Either way I don't have time for you. 
                    Shove off, and hope I don't see you again.
            ]],

            OPT_SCARE_OFF = "Scare off the Admiralty",
            DIALOG_SCARE_OFF = [[
                player:
                    Take a walk, switch.
                    !threaten
                    Or someone's going to get hurt!
            ]],
            DIALOG_SCARE_OFF_SUCCESS = [[
                agent:
                    !scared
                    This ain't worth getting killed over!
                    !exit
                * {agent} throws the shipment manifest at you and runs.
            ]],
            DIALOG_SCARE_OFF_FAILURE = [[
                agent:
                    !threaten
                    Kill the grifter!
            ]],

            OPT_ATTACK = "Attack!",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    Graaaaaaaah!
            ]],
            DIALOG_WON = [[
                agent:
                    !exit
                * You pick up the manifest and inspect the codes.
                {betrayed_admiralty? 
                    * You hope that {ad_friend} isn't the type to hold a grudge about a couple of dead coworkers.
                }
            ]],
            DIALOG_SUPPORTER_DONE = [[
                player:
                    There you have it. Easy peasy erchin squeezy.
                supporter:
                    !right
                    Y-you did it? You got the cache? So does this mean...
                player:
                    You got it, bud. You know where to find Seemli?
                    Tell her I sent you. She'll see you settled up.
                !exit
            ]],
            DIALOG_DONE = [[
                * The cache is too big to carry, even if you had help, so you're going to have to take the codes to the Rise.
                * You do your best to hide the crates before making your exit.
                player:
                    !exit
            ]]


        }
        :Fn(function(cxt) 
            local opfor
            

            if cxt.quest:IsActive("get_support") then
                cxt.quest:Cancel("get_support")
            end

            if cxt.quest.param.deal_with_admiralty then
                opfor = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank() + 1, cxt.location)
            else
                opfor = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank() - 1, cxt.location)
            end
            local allies = {}
            
            if cxt.quest:GetCastMember("supporter"):IsInPlayerParty() then
                table.insert(allies, "supporter")
            end
            cxt:TalkTo(opfor[1])
            local advantage = TEAM.BLUE

            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 
                    
                    cxt:BasicNegotiation("SCARE_OFF", {helpers = allies, flags = NEGOTIATION_FLAGS.INTIMIDATION})
                        :OnSuccess()
                            :Fn(function() 

                                if cxt.quest:GetCastMember("supporter"):IsInPlayerParty() then
                                    cxt:Dialog("DIALOG_SUPPORTER_DONE")
                                    cxt.quest:GetCastMember("supporter"):Dismiss()
                                    cxt.quest:OpinionEvent("supporter", "helped_get_back")
                                end
                                cxt.quest:Complete("seize_cache")
                                cxt:Dialog("DIALOG_DONE")
                            end)

                            :Travel()

                        :OnFailure()
                            :Fn(function() 
                                advantage = TEAM.RED
                            end)

                    cxt:Opt("OPT_ATTACK")
                        :Dialog("DIALOG_ATTACK")
                        :Battle{
                            advantage = advantage,
                            enemies = opfor,
                            allies = allies,
                            no_oppo_limit = true,
                        }
                            :OnWin()
                                :Fn(function()
                                    
                                    if cxt.quest.param.deal_with_admiralty then
                                        local killed_someone = Iter.array(opfor):any(function(enemy) return enemy:IsSentient() and enemy:IsDead() end)
                                        if killed_someone then
                                            cxt.quest.param.betrayed_admiralty = true
                                        end
                                    end
                                    cxt:Dialog("DIALOG_WON")

                                    if cxt.quest:GetCastMember("supporter"):IsInPlayerParty() then
                                        cxt:Dialog("DIALOG_SUPPORTER_DONE")
                                        cxt.quest:GetCastMember("supporter"):Dismiss()
                                        cxt.quest:OpinionEvent("supporter", "helped_get_back")
                                    end
                                    cxt.quest:Complete("seize_cache")
                                    cxt:Dialog("DIALOG_DONE")
                                end)
                                :Travel()

                end)

        end)

QDEF:AddConvo( "return_to_seemli" )
    :TravelConfront("CONFRONT", function(cxt) return cxt.quest.param.betrayed_admiralty and not cxt.quest.param.betrayed_admiralty_followup and not cxt.quest:GetCastMember("ad_friend"):IsRetired() end)
        :Loc{
            DIALOG_INTRO = [[
                * You're stopped by {ad_friend}, goons in tow.
                ad_friend:
                    !right
                    !angry
                    There he is!
                player:
                    !left
                    Here I am!
                    What's up, {ad_friend}? Staying frosty?
                ad_friend:
                    One rule, Smith! I told you not to kill any Admiralty.
                    Now you've gone and forced my hand. 
            ]],
            OPT_CONVINCE = "Convince {agent} you did what you had to do",
            DIALOG_CONVINCE = [[
                player:
                    Listen! I didn't mean to kill anyone! Things just got out of control!
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    My fists... are so big and powerful. I'm scared of them myself sometimes, ya know?
                ad_friend:
                    Just shut up.
                    I don't want to hear any more excuses.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    These are weird times, and corruption's gone crazy.
                    You know my own sister's in the Admiralty. I wouldn't hurt nobody on purpose. 
                    Who's to say those guards weren't on the take anyway, huh?
                ad_friend:
                    !sigh
                    I think you're just talking, but you actually have a point.
                    There's been reason to suspect...
                    At any rate, I can fudge the paperwork and put this to bed.
                    But you--I don't wanna see you asking me any more favors, understand? 
                    A friend that makes trouble is no friend of mine.
                player:
                    Fair enough.
                    !exit
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    I'm a very dangerous man!
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is as dead as your one-time friendship.
                    * You move on.
                }
                {not dead?
                    agent:
                        !injured
                        I'll remember this, grifter.
                    player:
                        See that you do. I don't like to beat in the same face twice.
                        !exit
                }

            ]],

        }
        :Fn(function(cxt) 
            cxt.quest.param.betrayed_admiralty_followup = true
            cxt:TalkTo("ad_friend")
            local party = CreateCombatBackup(cxt:GetAgent(), "ADMIRALTY_PATROL_BACKUP", cxt.quest:GetRank())
            
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:OpinionEvent("ad_friend", "killed_ads")

            cxt:RunLoop(function() 
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :Fn(function() 
                            cxt.quest:OpinionEvent("ad_friend", "explained_kill")
                        end)
                        :Travel()

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{ enemies = cxt.enc.scratch.bandit_survivors, flags = BATTLE_FLAGS.SELF_DEFENCE}
                        :OnWin()
                            :Dialog("DIALOG_WON")
                            :Travel()
            end)

        end)


QDEF:AddConvo("rescue_old_supplier")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("rescue_point") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find the Admiralty moving {old_supplier}.
                
                agent:
                    !right
                    Be careful! There's a grifter skulking about.
                    
                player:
                    !left
                    !greeting
                    Hi there!
                    I'm interested in your prisoner.
                agent:
                    Keep your distance! This is Admiralty business!
            
            ]],

            OPT_CONVINCE = "Convince {agent} to release {old_supplier}",
            DIALOG_CONVINCE = [[
                player:
                    'Scuse me, officer, but I'm on an important mission for the Admiralty.
                    How about you let {old_supplier} go?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    This is covert stuff, comes from the top. Captain Mullifee's orders, in fact.
                    With {old_supplier}'s help I'll be able to flush out some rats that need exterminating.
                agent:
                    Ah. You're her brother, aren't you? I think I've seen your picture on her desk.
                    This is a little unorthodox, but these are unorthodox times. And I have bigger things to worry about than a common smuggler.
                    !exit
                * The Admiralty move off, leaving {old_supplier} behind.
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    My sister should be okay with it. Just tell Captain Mullifee that Smith says hi.
                agent:
                    Do you think I'm stupid?
                player:
                    ...
                    No?
                    Hard to tell. You just got one of those faces.
                agent:
                    Shove off!
            ]],

            OPT_PAY_FOR_RELEASE = "Pay for {old_supplier}'s release",
            DIALOG_PAY_FOR_RELEASE = [[
                player:
                    How about I just pay you, and you pretend you lost {old_supplier.himher}?
                agent:
                    You know {old_supplier.heshe}'s only being held for a fine, right?
                    Unlike the jakes to pay it. This one must've run afoul of some code or another.
                player:
                    Oh. So this ain't even technically a bribe?
                agent:
                    Not if I do up the paperwork and make it official.
                    If I <i>don't</i> make it official... then yes. It's a bribe.
                player:
                    !give
                    I'll leave that part up to you, pal.
                agent:
                    !take
                    That'll do. Have fun with your misfit jake. 
                    !exit
                * The Admiralty saunter off with shills in hand. {old_supplier} looks at you expectantly.
                old_supplier:
                    !right 
                    !dubious
            ]],

            OPT_ATTACK = "Attack the Admiralty",
            DIALOG_ATTACK = [[
                player:
                    !attack
                    Graaaaaaaah!
            ]],
            DIALOG_WON_NEW_SUPPLIER_DEAD = [[
                agent:
                    !exit
                * The Admiralty threat collapses, but so does {old_supplier}.
                * Your rescue attempt wasn't worth much.
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead. You free {old_supplier} from {old_supplier.hisher} bonds.
                }
                {not dead?
                    agent:
                        !injured
                        Run!
                        !exit
                    * You turn to {old_supplier}, who has escaped from {old_supplier.hisher} bonds.
                }
            ]],
            DIALOG_RESCUED = [[
                player:
                    !happy
                old_supplier:
                    !right
                    That's mighty kind of you, grifter.
                    {supporter} and I aren't from these parts. We ran afoul of the Admiralty most cataclysmically.
                    Afraid the jakes won't take me back, though, seeing as I let the Rise pay on delivery.
                player:
                    Yeah, about that. I think it'll all work out in the end.
                    Why not pay {new_supplier} a visit? You should have good news coming. 
                old_supplier:
                    Curious. Well, I eagerly await your surprise. 
                    Thank you again. 
                    !exit
            ]]


        }
        :Fn(function(cxt) 
            local opfor = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank(), cxt.location)

            cxt:TalkTo(opfor[1])

            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function()

                cxt:BasicNegotiation("CONVINCE", {helpers = "old_supplier"})
                    :OnSuccess()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_RESCUED")
                            cxt.quest:Complete("rescue_old_supplier")
                        end)
                        :Travel()

                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{
                        noncombatants = {"new_supplier"},
                    }
                        :OnWin()
                            :Fn(function()
                                if cxt.quest.param.deal_with_admiralty then
                                    local killed_someone = Iter.array(opfor):any(function(enemy) return enemy:IsSentient() and enemy:IsDead() end)
                                    if killed_someone then
                                        cxt.quest.param.betrayed_admiralty = true
                                    end
                                end
                                
                                if cxt.quest:GetCastMember("old_supplier"):IsAlive() then
                                    cxt:Dialog("DIALOG_WON")
                                    cxt:Dialog("DIALOG_RESCUED")
                                    cxt.quest:Complete("rescue_old_supplier")
                                else
                                    cxt:Dialog("DIALOG_WON_NEW_SUPPLIER_DEAD")
                                    cxt.quest:Fail("rescue_old_supplier")
                                end
                            end)
                            :Travel()

                cxt:Opt("OPT_PAY_FOR_RELEASE")
                    :DeliverMoney(200)
                    :Dialog("DIALOG_PAY_FOR_RELEASE")
                    :Dialog("DIALOG_RESCUED")
                    :Fn(function() 
                        cxt.quest:Complete("rescue_old_supplier")
                    end)
                    :Travel()
            end)

        end)


QDEF:AddConvo( "deliver_to_safehouse" )
    :TravelConfront("CONFRONT", function(cxt) return not cxt.quest.param.offered_side_deal and not cxt.quest:GetCastMember("new_supplier"):IsRetired() end)
        :Loc{
            DIALOG_INTRO = [[
                * You are surprised to see {new_supplier} waiting for you.
                player:
                    !left
                agent:
                    !right
                    You did it, eh?
                    I didn't expect you to pull it off.
                player:
                    Thank you for your vote of confidence!
                agent:
                    Anyway, {old_supplier} was a fool, and gave those Rise weirdos too good of a deal.
                    I'd like to buy back half of the codes from you. My people will get in there and be out with the goods before anyone's the wiser.
                    You'll have to smooth things over with the buyers, but you can just say the Admiralty took 'em or something.
                    Whaddya say, pal? Are we in business?
            ]],

            OPT_ACCEPT_DEAL = "Accept the deal",
            DIALOG_ACCEPT_DEAL = [[
                player: 
                    Sounds good to me!
                    !give
                agent:
                    !take
                    Great!
                    Let this be the start of a very profitable relationship!
                    !exit
            ]],

            OPT_REJECT_DEAL = "Reject the deal",
            DIALOG_REJECT_DEAL = [[
                player:
                    Nah. I can't risk {seemli} getting mad and or murderous at me.
                agent:
                    Really?
                    This was basically free money, grifter.
                    Your loss.
                    !exit
            ]],

        }
        :Fn(function(cxt) 
            cxt.quest.param.offered_side_deal = true
            cxt:TalkTo("new_supplier")
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_ACCEPT_DEAL")
                :Dialog("DIALOG_ACCEPT_DEAL")
                :ReceiveMoney(200)
                :ReceiveOpinion("took_side_deal")
                :Fn(function() cxt.quest.param.side_deal = true end)
                :Travel()

            cxt:Opt("OPT_REJECT_DEAL")
                :Dialog("DIALOG_REJECT_DEAL")
                :ReceiveOpinion("rejected_side_deal")
                :Travel()

        end)


QDEF:AddConvo("deliver_to_safehouse")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("safehouse") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    Yo, Riseguys. Special delivery.
                    The cache was too big for lil ol' me, but I got the access codes here, and a guarantee it'll be guard-free for a while.
                agent:
                    !right
                    Wait, this is for the weapons cache? Good work. 
                    We were just discussing whether or not to rescue {old_supplier}. 
                    Now we don't need to!
            ]],
            DIALOG_SUPPLIER_DEAL = [[
                * You hand over the codes that you have left.
                agent:
                    Only three access codes? What, did they double up or something?
                    There should be at least six crates.
            ]],
            DIALOG_NO_SUPPLIER_DEAL = [[
                * You hand over the codes with a flourish.
                agent:
                    !happy
                    This should check out. We'll do the pick up and report back to {seemli}.
                    Thanks for your help, grifter!
            ]],

            
            OPT_CONVINCE = "Convince {agent} that was all there was",
            DIALOG_CONVINCE = [[
                player:
                    Hey, man, I never got a manifest.
                    "Find the cache," Seemli said. "Talk to the supplier."
                    I did both those things.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    I can't be responsible for someone else's screw-ups, can I?
                agent:
                    Yeah, okay, I believe you.
                    Things must've gotten moved around. It's a shame, but we'll make it work.
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    And those codes that were ripped out? They were like that when I got 'em.
                    So don't even think of sayin' I sold them back to the jakes!
                agent:
                    ...
                    You sold them back to the jakes, didn't you?
                player:
                    Maybe.
                agent:
                    {seemli} will hear about this. That's all I can say.
            ]],

            OPT_IGNORE = "Ignore {agent}'s complaints",
            DIALOG_IGNORE = [[
                player:
                    Eh. Take it or leave it.
                    I did my job. I don't really care what you think.
                agent:
                    {seemli} will hear about this!
            ]],
            
            DIALOG_RESCUE_CHANCE = [[
                agent:
                    Wait.
                    Before you go--
                    We have a lead on {old_supplier}'s location.
                    You might want to look into that. {old_supplier.HeShe} gave us a good deal, and it'd be a shame to burn that bridge.
                player:
                    Is this a suggestion, or an order? Seemli didn't mention nothing about a rescue.
                agent:
                    A suggestion. It's up to you. 
                player:
                    Hrm. I'll think about it.
                    !exit
            ]],

        }
        :Fn(function(cxt) 
            local agent = AgentUtil.GetFreeAgent("RISE_PAMPHLETEER")
            cxt:TalkTo(agent)

            cxt:Dialog("DIALOG_INTRO")
            if cxt.quest.param.side_deal then
                cxt:Dialog("DIALOG_SUPPLIER_DEAL")
                cxt:BasicNegotiation("CONVINCE")
                    :OnSuccess()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_RESCUE_CHANCE")
                            cxt.quest:Activate("rescue_old_supplier")
                            cxt.quest:Complete("deliver_to_safehouse")
                        end)
                        :Travel()
                    :OnFailure()
                        :Fn(function() 
                            cxt.quest.param.rise_suspicious = true
                            cxt.quest:Complete("deliver_to_safehouse")
                        end)
                        :ReceiveOpinion("stole_from_rise")
                        :Travel()

                cxt:Opt("OPT_IGNORE")
                    :Dialog("DIALOG_IGNORE")
                    :ReceiveOpinion("stole_from_rise")
                    :Fn(function() 
                        cxt.quest.param.rise_suspicious = true
                        cxt.quest:Complete("deliver_to_safehouse")
                    end)
                    :Travel()

            else
                cxt:Dialog("DIALOG_NO_SUPPLIER_DEAL")
                cxt:Dialog("DIALOG_RESCUE_CHANCE")
                cxt.quest:Activate("rescue_old_supplier")
                cxt.quest:Complete("deliver_to_safehouse")
                StateGraphUtil.AddLeaveLocation(cxt)    
            end

        end)



QDEF:AddConvo("return_to_seemli", "seemli")
    :Loc{

        OPT_GIVE_REPORT = "Tell {seemli} about the weapons",
        DIALOG_GIVE_REPORT_SUSPICION = [[
            player:
                The weapon codes have been delivered.
            agent:
                All of them?
            player:
                Er... yeah. Of course.
            agent:
                Hm. You're not a very good liar, you know.
            player:
                People tell me I have an honest face.
            agent:
                I suppose. But you also have a purple aura, which is a sure sign of deceit.
                A therapeutic mud cleanse with exfoliating quartz will clarify the energies, should you wish to.
            player:
                Huh. So you're not mad?
            agent:
                Oh, I'm very upset! I'm just practicing tantric moodblasting. 
                You will, of course, only be paid for what you returned.
            player:
                That's fair.
        ]],
        DIALOG_GIVE_REPORT = [[
            player:
                The revolution is armed and dangerous.
            agent:
                Excellent.
                I don't trust this {new_supplier}, yet--our energies are in conflict, I think.
                But at least we have our hardware.
            player:
                Er, yeah. Good thing the illegal weapons don't mess with your... whatever.
            agent:
                Oh, no. I'm very much attuned to the harmonic resonance of a military-grade pistol. But thank you for your consideration.
        ]],
        DIALOG_GIVE_REPORT_RESCUED = [[
            player:
                I have good news, and I have better news.
            agent:
                Oh? Should I have my pendulum ready for this, in case the energies wish to weigh in?
            player:
                Uh, no. I was just trying to... you know, reframe the negatives.
                I recovered your weapons!
            agent:
                Excellent!
            player:
                I recovered your old supplier, too!
            agent:
                You <i>did</i>? I'm glad that {old_supplier} could be spared.
                I always felt like our spirits knew each other in our Vagrant lives. 
                And the discounts were a bonus, too.
            player:
                !shrug
                What can I say? I'm a pleaser.
        ]]
    }

    --rise_suspicious
    --rescued_old_supplier
    :Hub(function(cxt) 
        

        
        cxt:Opt("OPT_GIVE_REPORT")
            :SetQuestMark()
            :Fn(function() 
                if cxt.quest.param.rise_suspicious then
                    cxt.quest.param.poor_performance = true
                    cxt:Dialog("DIALOG_GIVE_REPORT_SUSPICION")
                elseif cxt.quest.param.rescued_old_supplier then
                    cxt:Dialog("DIALOG_GIVE_REPORT_RESCUED")
                else
                    cxt:Dialog("DIALOG_GIVE_REPORT")
                end
                ConvoUtil.GiveQuestRewards(cxt)
                cxt.quest:Complete()
            end)
    end)


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}



    



