local objectives = {"collect_dilettante", "collect_opportunist", "collect_true_believer"}
local members = {"dilettante", "opportunist", "true_believer"}

local QDEF = QuestDef.Define{
        title = "Triangular Firing Squad",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[]],
        icon = engine.asset.Texture("icons/quests/side_smith_seemli_triangular_firing_squad.tex"),
        focus = QUEST_FOCUS.BALANCED,
        tags = {"SEEMLI_JOB"},
        act_filter = "SMITH",
        --disabled = true,

        on_debug_start = function(quest)
            quest:GetCastMember("seemli"):MoveToLocation(TheGame:GetGameState():GetLocation("MOREEF_BAR"))
            TheGame:GetGameState():GetCaravan():MoveToLocation(quest:GetCastMember("seemli"):GetLocation())
            UIHelpers.DoSpecificConvo( quest:GetCastMember("seemli"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
        end,

        collect_agent_locations = function(quest, t)
            --table.insert (t, { agent = quest:GetCastMember("assassin"), location = quest:IsActive("meet_assassin_post_kill") and quest:GetCastMember("cliffside") or quest:GetCastMember("park"), role = CHARACTER_ROLES.VISITOR} )
        end,

        avoids = {
            {"seemli", "opportunist"},
            {"seemli", "true_believer"},
            {"seemli", "dilettante"},

            {"dilettante", "opportunist"},
            {"dilettante", "true_believer"},
            {"opportunist", "true_believer"},
        },

        on_destroy = function(quest) 
            table.arrayfunction(members, function(cast_id) if quest:GetCastMember(cast_id) and quest:GetCastMember(cast_id):GetParty() then quest:GetCastMember(cast_id):Dismiss() end end)
        end,
        
        process_fighter = function(quest, fighter)
            if table.any(members, function(id) return fighter.agent == quest:GetCastAgent(id) end) then
                fighter:AddCondition("WANTED_ALIVE")
            end
        end,    

    }

    :AddCastByAlias{
        cast_id = "seemli",
        alias = "SEEMLI",
        provider = true,
        no_validation = true,
    }

    :AddDefCastSpawn("dilettante", "WEALTHY_MERCHANT")
    :AddDefCastSpawn("opportunist", "JAKES_SMUGGLER")
    :AddDefCastSpawn("true_believer", "RISE_RADICAL")
    
    :AddObjective{
        id = "start",
        hide_in_overlay = true,
        
        on_activate = function(quest) 
            table.arrayfunction(objectives, function(o) quest:Activate(o) end)
        end,

        events = {
            quests_changed = function(quest, event_quest) 

                if event_quest == quest then
                    local all_done = table.all(objectives, function(obj) return quest:IsDone(obj) end)
                    local all_failed = table.all(objectives, function(obj) return quest:IsFailed(obj) end)
                    if all_done then
                        if all_failed then
                            quest:Fail("start")
                            quest:Activate("return_failure")
                        else
                            quest:Complete("start")
                        end
                    end
                end
            end           
        },
        
        on_complete = function(quest)
            quest:Activate("regain_clubhouse")
        end,
        
        on_fail = function(quest)
            table.arrayfunction(objectives, function(o) if quest:IsActive(o) then quest:Cancel(o) end end)
            table.arrayfunction(members, function(cast_id) if quest:GetCastMember(cast_id) and quest:GetCastMember(cast_id):GetParty() then quest:GetCastMember(cast_id):Dismiss() end end)
            quest:Activate("return_failure")
        end,
    }

    :AddObjective{
        id = "collect_dilettante",
        title = "Collect {dilettante}",
        desc = "Convince {dilettante} to rejoin the cell.",
        mark = {"dilettante"},
        negotiation_targets = {"dilettante"},

        on_complete = function(quest)
            quest:GetCastMember("dilettante"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
        end,
    }

    :AddObjective{
        id = "collect_true_believer",
        title = "Collect {true_believer}",
        desc = "Convince {true_believer} to rejoin the cell.",
        mark = {"true_believer"},
        negotiation_targets = {"true_believer"},

        on_complete = function(quest)
            quest:GetCastMember("true_believer"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
        end,
    }

    :AddObjective{
        id = "collect_opportunist",
        title = "Collect {opportunist}",
        desc = "Convince {opportunist} to rejoin the cell.",
        mark = {"opportunist"},
        negotiation_targets = {"opportunist"},

        on_complete = function(quest)
            quest:GetCastMember("opportunist"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
        end,
    }

    :AddObjective{
        id = "return_failure",
        title = "Tell {seemli} you messed up",
        desc = "You failed to convince any of the cell members to rejoin.",
        mark = {"seemli"},
    }

    :AddObjective{
        id = "regain_clubhouse",
        title = "Regain the cell's base",
        desc = "Take the cell members to the base, and re-establish the Rise's presence in the Pearl.",
        mark = {"clubhouse"},
        
        on_activate = function(quest)
            quest.param.attempted_regain = true
        end,

        on_complete = function(quest)
            quest:Activate("return_to_seemli")
        end,
        
        on_fail = function(quest)
            quest:Activate("return_failure")
        end,
    }

    :AddQuestLocation{
        cast_id = "clubhouse",
        name = "Rise Headquarters",
        plax = "INT_SMITH_RISE_CLUBHOUSE",
        show_agents = true,
        tags = {"building"},
        indoors = true,
    }

    :AddObjective{
        id = "return_to_seemli",
        title = "Return to {seemli}",
        desc = "Tell {seemli} that the cell has been re-established.",
        mark = {"seemli"},
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------

    :AddOpinionEvents{
        restarted_revolution = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "You resurrected the Rise in the Pearl with them",
        },
        abandoned_revolution = 
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "You abandoned the Rise",
        }
    }

------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------



QDEF:AddIntro(
    --attract spiel
    [[
        agent:
            This isn't the first time a Rise cell has convened here in the Pearl. 
            We actually had a collective about a year ago.
        player:
            What happened?
        agent:
            Oh, well. The servant's ball was particularly good that year, and some lost their discontent.
            But the bonds that bind remain strong. And I believe it's time for us to meet again.
        player:
            Right, so... I'll round up the gang, then? Just gimme the names.
        agent:
            Round up the gang <i>and</i> give them purpose. Perhaps you could inspire them to action.
            Our old headquarters are currently in Admiralty hands. Reunite us with our home--the energies are good there.
    ]],
    
    --on accept
    [[
        agent:
            The best of the old crew were {dilettante}, {opportunist}, and {true_believer}. 
            In my meditations, I've felt their unrest. I believe they are once again ready to walk with me.
            {dilettante} is a noble, actually, but enjoys slumming it with us. Not unlike you, actually!
        player:
            A gus after my own pus. 
        agent:
            {opportunist} has connections with the jakes, which adds value.
            Lastly, {true_believer} is a servant, and usually miserable. It should be simple to get {true_believer.himher} back on board.
            Just one thing to bear in mind: their auras aren't exactly in sync with one another.
        player:
            Their whatty what what? 
        agent:
            We were once united by purpose, but they were never united by affection. The three of them despise one another.
            I trust you'll find the resonance that brings them amplitude. 
        player:
            Sure. And if not, I can at least make their ears ring.
    ]],
    "start")


local wants_in_party = {
    dilettante = "true_believer",
    true_believer = "opportunist",
    opportunist = "dilettante",
}

local dislikes_in_party = {
    dilettante = "opportunist",
    true_believer = "dilettante",
    opportunist = "true_believer",
}

local function CreateNegotiation(cxt, obj, cast_id)
    local helpers = {}
    local hinders = {}
    local situation_modifiers = {}

    if cxt.quest:GetCastMember(wants_in_party[cast_id]):IsInPlayerParty() then
        cxt.enc.scratch.wants_in_party = true
        table.insert(situation_modifiers, { value = -10, text = loc.format( cxt:GetLocString("WANTS_IN_PARTY"), cxt.quest:GetCastMember(wants_in_party[cast_id])) })  
    end        

    if cxt.quest:GetCastMember(dislikes_in_party[cast_id]):IsInPlayerParty() then
        cxt.enc.scratch.dislikes_in_party = true
        table.insert(situation_modifiers, { value = 20, text = loc.format( cxt:GetLocString("DISLIKES_IN_PARTY"), cxt.quest:GetCastMember(dislikes_in_party[cast_id])) })  
    end

    for wanter, wants in pairs(wants_in_party) do
        if cast_id == wants then
            if cxt.quest:GetCastMember(wanter):IsInPlayerParty() then
                cxt.enc.scratch.wanter_in_party = true
                table.insert(helpers, wanter)
            end
        end
    end

    for disliker, dislikes in pairs(dislikes_in_party) do
        if cast_id == dislikes then
            if cxt.quest:GetCastMember(disliker):IsInPlayerParty() then
                cxt.enc.scratch.disliker_in_party = true
                table.insert(hinders, disliker)
            end
        end
    end

    cxt:BasicNegotiation("CONVINCE_REJOIN", {situation_modifiers = situation_modifiers, hinders = hinders, helpers = helpers})
        :SetQuestMark()
        :OnSuccess()
            :CompleteQuest(obj)
            :DoneConvo()
        :OnFailure()
            :FailQuest(obj)
            :Fn(function() 
                cxt:GetAgent():MoveToLimbo()
            end)
            :DoneConvo()
end


-- the person you are talking to quips about your party composition up-front
-- the people in your party quip on the results at the end
QDEF:AddConvo("collect_dilettante", "dilettante")
    :Loc{

        OPT_ASK_ABOUT_RISE = "Ask {agent} about the Rise",
        DIALOG_ASK_ABOUT_RISE = [[
            player:
                Hey, ever think about getting back in on the Rise?
                It's pretty cool these days.
            agent:
                Cool?
            player:
                Er, yeah. I hear you and me got stuff in common.
                Thought you might like to know I endorse it.
            agent:
                But you're old.
            player:
                I ain't old!
            agent:
                Hrm. I guess I can join up, like, ironically.

        ]],
        OPT_ASK_ABOUT_TRUE_BELIEVER = "Ask {agent} about {true_believer}",
        DIALOG_ASK_ABOUT_TRUE_BELIEVER = [[
            player:
                Seemli mentioned you maybe didn't think much of {true_believer}. That true?
            agent:
                What? No! Not at all. 
                {true_believer} was the real deal. {true_believer.HeShe} brought authenticity to the whole experiment.
                If {true_believer} is back on board--now that would be a compelling lure.
        ]],

        OPT_ASK_ABOUT_OPPORTUNIST = "Ask {agent} about {opportunist}",
        DIALOG_ASK_ABOUT_OPPORTUNIST = [[
            player:
                I'm gonna be approaching {opportunist}, too. That gonna be a problem?
            agent:
                Ugh. 
                I learned last time that being a revolutionary meant keeping the company of posers like that.
                {opportunist} is only in it for the money, and I was the biggest target.
        ]],

        DISLIKES_IN_PARTY = "Doesn't think {1.name} is really Rise",
        WANTS_IN_PARTY = "Looks up to {1.name}",

        OPT_CONVINCE_REJOIN = "Convince {agent} to rejoin the Rise cell",
        DIALOG_CONVINCE_REJOIN = [[
            player:
                So whaddya say? You want back in?
            {wants_in_party?
                agent:
                    !happy
                    Well, if {true_believer} is in, then I can't refuse.
                    I'm too addicted to the sincerity, you know? It's like really living.
            }
            {dislikes_in_party?
                agent:
                    !hips
                    You got {opportunist} with you? Why should I link arms with that swindler?
            }
            
            {not dislikes_in_party and not wants_in_party?
                agent:
                    !happy
                    Have other people been talking about me? Do they think I'm in the Rise?
            }
        ]],
        DIALOG_CONVINCE_REJOIN_SUCCESS = [[
            player:
                Think of the big picture, kid. The Rise has taken off in the Bread Fields, in Grout Bog...
                You wanna be the last one to join the revolution? Or do you wanna be its mascot?
            agent:
                Hm. It's a good point. 
                And why should the laborers get all the esteem, anyway?
                They always get all the credit. It's time they share the spotlight.

            {wanter_in_party?
                opportunist:
                    !right
                    !handwring
                    Well, you know what they say about a fool and {dilettante.hisher} shills.
            }
            {disliker_in_party?
                
                true_believer:
                    !right
                    !sigh
                    Yes, fine. Let the blue-blooded pig into our midst. 
                    At least it'll give me the chance to prove this is a cause worth dying for.
            }

            {not wanter_in_party and not disliker_in_party?
                * {agent} falls in line behind you.
                * {agent} joins your group.
            }

        ]],


        DIALOG_CONVINCE_REJOIN_FAILURE = [[
            player:
                The Rise is about to be the hottest action in town.
            agent:
                Uh, look around you, old man. 
                <i>Hesh</i> is the hottest thing in town. I've already decided to become a pilgrim.
                My tailor is designing a set of boutique tattered rags as we speak, and promises the salt stains will be very authentic.
            
            {disliker_in_party?
                true_believer:
                    !right
                    !happy
                    Hah! There's no such thing as fair-weather justice, coward. Let your whims carry you straight into the Abyss.
            }
            
            {wanter_in_party?
                opportunist:
                    !right
                    !disappoint
                    Hrm. Almost not worth it, if we ain't flush enough to fund any group activities.
            }

        ]],
    }
    :Hub(function(cxt) 
        cxt:QST("ASK_ABOUT_RISE")
        cxt:QST("ASK_ABOUT_OPPORTUNIST")
        cxt:QST("ASK_ABOUT_TRUE_BELIEVER")
        CreateNegotiation(cxt, "collect_dilettante", "dilettante")
    end)

QDEF:AddConvo("collect_true_believer", "true_believer")
    :Loc{

        DISLIKES_IN_PARTY = "Thinks {1.name} is a poser",
        WANTS_IN_PARTY = "Wants access to {1.name}'s weapons",

        OPT_ASK_ABOUT_RISE = "Ask {agent} about the Rise",
        DIALOG_ASK_ABOUT_RISE = [[
            player:
                What's the deal with you and the Rise, huh?
            agent:
                The Rise? A cause that cut to my marrow. 
                But Seemli wasn't cutthroat enough. She believed a transition could be peaceful.
                And after our cell fizzled out, I lost heart.
                How I'd yearn to be part of that Grout Bog faction, killing Spark Barons with mud up to their ears...
            player:
                Oh yeah. That sounds... great.
        ]],
        OPT_ASK_ABOUT_DILETTANTE = "Ask {agent} about {dilettante}",
        DIALOG_ASK_ABOUT_DILETTANTE = [[
            player:
                Whaddya make of {dilettante}?
            agent:
                That gutter-diving snipe? {dilettante.HeShe} treated us like spectacles in a travelling show.
                Our woes, our struggles were all baubles for {dilettante.hisher} amusement. 
                I suspect {dilettante.hisher} insincerity was the reason our movement failed. 
        ]],

        OPT_ASK_ABOUT_OPPORTUNIST = "Ask {agent} about {opportunist}",
        DIALOG_ASK_ABOUT_OPPORTUNIST = [[
            player:
                How about {opportunist}? Think they'd be an asset to the Rise again?
            agent:
                {opportunist} and I don't always see eye to eye, but such people are necessary cogs in the machine of change.
                With {opportunist.hisher} jake connections, our cell can be properly outfitted.
                And with the right equipment, we can really wreak havoc. 
        ]],


        OPT_CONVINCE_REJOIN = "Convince {agent} to rejoin the Rise cell",
        
        DIALOG_CONVINCE_REJOIN = [[
            player:
                You know, you should get back on board. Be the change you wanna exact on the world.
            
            {wants_in_party?
                agent:
                    !happy
                    You have {opportunist} with you? That suggests there's at least opportunity.
                    Give me a position! Show me where the ammunition is! 
            }
            {dislikes_in_party?
                agent:
                    !hips
                    And what about {dilettante}? Is that riff-raff indicative of the kind of change we can expect?
            }

            {not dislikes_in_party and not wants_in_party?
                agent:
                    The cause calls. The question is: do I submit?
            }

        ]],
        DIALOG_CONVINCE_REJOIN_SUCCESS = [[
            player:
                Seemli's changed. She's much more ruthless now. That stuff in Grout Bog really inspired her.
                You want action? Seemli will make sure you see it.
            agent:
                And if a blue-blood or two falls in my path? All the better. I'm in.

            {wanter_in_party?
                dilettante:
                    !right
                    !eureka
                    That passion! That fervor! This is the perfect inspiration for my verse novel about Deltinean the First.
            }
            {disliker_in_party?
                opportunist:
                    !right
                    !sigh
                    We're going to have to listen to speeches now, ain't we?
            }

            {not wanter_in_party and not disliker_in_party?
                * {agent} falls in line behind you.
                * {agent} joins your group.
            }

        ]],


        DIALOG_CONVINCE_REJOIN_FAILURE = [[
            player:
                Besides, it's probably pretty lonely being all grumpy on your lonesome.
            agent:
                My beliefs aren't for want of <i>company</i>. I would rather stand alone than stand for nothing.
                I refuse. When the revolution comes, I will be ready. But I will not throw my lot in with Seemli again.
            
            {disliker_in_party?
                opportunist:
                    !right
                    !happy
                    Well, at least we won't have to read {agent.hisher} manifestos any more.
            }
            
            {wanter_in_party?
                dilettante:
                    !right
                    !bashful
                    God, {true_believer.heshe} makes being a lone wolf sound so exceptional.
                    We're still the Rise, right? Even though we don't have {agent}?
            }
        ]],
    }
    :Hub(function(cxt) 
        cxt:QST("ASK_ABOUT_RISE")
        cxt:QST("ASK_ABOUT_OPPORTUNIST")
        cxt:QST("ASK_ABOUT_DILETTANTE")
        CreateNegotiation(cxt, "collect_true_believer", "true_believer")
    end)

QDEF:AddConvo("collect_opportunist", "opportunist")
    :Loc{


        OPT_ASK_ABOUT_RISE = "Ask {agent} about the Rise",
        DIALOG_ASK_ABOUT_RISE = [[
            player:
                You used to be in the Rise, right? Fancy taking it up again?
            agent:
                Hesh, yeah. Believe it or not, there's not a big market for contraband weapons from the nobility.
                When the Rise was active, they were big, big spenders. 
                Almost coulda bought my own way into the nobility, if I'd wanted.
        ]],
        OPT_ASK_ABOUT_DILETTANTE = "Ask {agent} about {dilettante}",
        DIALOG_ASK_ABOUT_DILETTANTE = [[
            player:
                What about your old teammate, {dilettante}?
            agent:
                !laugh
                Shill-shucker! Oh, that kid was the best. 
                Footing the bill on every expense, ordering doubles of everything. 
                Half the stuff I sold {dilettante.himher} wasn't even real.
        ]],

        OPT_ASK_ABOUT_TRUE_BELIEVER = "Ask {agent} about {true_believer}",
        DIALOG_ASK_ABOUT_TRUE_BELIEVER = [[
            player:
                You work closely with {true_believer} at all?
            agent:
                Ugh, too often. 
                {true_believer.HeShe} did nothing but yammer on and on about revolution and change.
                As if there's ever been a point in history when things were <i>fair</i>.
                You think the Vagrants had justice and equality all figured out?
                Then how'd we end up stuck down here, on this diseased mudball, with only scraps to pick at?
            player:
                Sounds like you got some ideas of your own that need yammering.
            agent:
                ...
                Forget I said anything.
        ]],


        DISLIKES_IN_PARTY = "Thinks {1.name} is a fanatic",
        WANTS_IN_PARTY = "Admires {1.name}'s money",

        OPT_CONVINCE_REJOIN = "Convince {agent} to rejoin the Rise cell",
        
        DIALOG_CONVINCE_REJOIN = [[
            player:
                The Rise could use your connections. You wanna come back?
            
            {wants_in_party?
                agent:
                    !happy
                    Is that {dilettante} you got with you? Didn't realize you were bankrolled.
            }
            {dislikes_in_party?
                agent:
                    !hips
                    You expect me to follow {true_believer} around again? Time is money, and that bloviator wastes both.
            }

            {not dislikes_in_party and not wants_in_party?
                agent:
                    The Rise ain't going nowhere without access to weapons. But what's in it for me?
            }

        ]],
        DIALOG_CONVINCE_REJOIN_SUCCESS = [[
            player:
                With the pilgrims in town and the news from Grout Bog, the Rise is gonna be more flush than ever.
                Someone's gotta profit, right?
            agent:
                Business calls, it seems. I'm in.

            {wanter_in_party?
                true_believer:
                    !right
                    !eureka
                    Excellent. Now let's talk supply lines--I have ideas about exactly which heads should roll, and in what direction.
            }
            {disliker_in_party?
                dilettante:
                    !right
                    !sigh
                    So now we're just letting anyone join? 
            }

            {not wanter_in_party and not disliker_in_party?
                * {agent} falls in line behind you.
                * {agent} joins your group.
            }

        ]],

        DIALOG_CONVINCE_REJOIN_FAILURE = [[
            player:
                Whaddya say? It'll be just like old times.
            agent:
                Huh. Thanks for reminding me--we disbanded for a reason.
                I think I'll ply my trade elsewhere. I already rung that sponge dry.
            
            {disliker_in_party?
                dilettante:
                    !right
                    !happy
                    Excellent. The more exclusive the club, the better.
            }
            
            {wanter_in_party?
                dilettante:
                    !right
                    !angry_shrug
                    Without that weapons connection, we're already dead in the water. But I suppose that's the fate I agreed to.
            }
        ]],
    }
    :Hub(function(cxt) 
        cxt:QST("ASK_ABOUT_RISE")
        cxt:QST("ASK_ABOUT_DILETTANTE")
        cxt:QST("ASK_ABOUT_TRUE_BELIEVER")
        CreateNegotiation(cxt, "collect_opportunist", "opportunist")
    end)



QDEF:AddConvo("regain_clubhouse")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("clubhouse") end)
        :Loc{
            DIALOG_INTRO = [[
                * You approach the Rise's erstwhile headquarters here in the Pearl. It is crawling with Admiralty.
                player:
                    !left
                    Alright, this is our first order of business as a team. Retaking your old headquarters.
                    It won't be easy, but we're in this together. And Seemli gave her orders.
                * Your "team", such as it is, stands ready.
            ]],

            OPT_USE_DILETTANTE = "Get {dilettante} to bribe the Admiralty",
            DIALOG_USE_DILETTANTE = [[
                * You get {dilettante} to approach the Admiralty alone, and pay some of them to go away. 
                * One of the guards recognizes {dilettante} and exchanges a fraternal handshake before dispersing. 
                dilettante:
                    !right
                    I only had to promise my tickets to the most exclusive event in the Pearl.
                    Do you know how hard it was to get those? Not just anyone gets to poke the Great Beaching with an anointed stick!
                    !exit
            ]],

            OPT_USE_TRUE_BELIEVER = "Get {true_believer} to rally the troops",
            DIALOG_USE_TRUE_BELIEVER = [[
                player:
                    {true_believer}, why don't you say something to get everyone feelin' the mojo, huh?
                true_believer:
                    !right
                    I shan't bore you with unnecessary words. Just know that this will be the first victory of many.
                player:
                    Huh. Effective.
            ]],

            OPT_USE_OPPORTUNIST = "Get weapons from {opportunist}",
            DIALOG_USE_OPPORTUNIST = [[
                player:
                    {opportunist}, whaddya got for us, huh? Someone like you is always carrying.
                opportunist:
                    !right
                    !dubious
                    Uh, yes. In exchange for shills.
                player:
                    That's an Admiralty squat now! It's loaded with goods.
                    You'll be able to take what you want once we're inside.
                opportunist:
                    Fine. I can spare these...
                    !exit
            ]],
            OPT_TAKE_ITEMS = "Take {1#card_list}",
            OPT_LEAVE_ITEMS = "Leave the items",

            OPT_ATTACK_THE_ADMIRALTY = "Attack the Admiralty",
            DIALOG_ATTACK_THE_ADMIRALTY = [[
                * You charge the Admiralty position!
                player:
                    !left
                    !fight
                agent:
                    !right
                    !fight
            ]],

            DIALOG_EVERYONE_DEAD = [[
                agent:
                    !exit
                * The battle is over, and you won. But at the cost of your team.
                * {seemli} is going to have mixed feelings about this.
            ]],
            DIALOG_REGAINED = [[
                agent:
                    !exit
                player:
                    !happy
                * You have driven off the Admiralty, and regained the headquarters.
                survivor:
                    !right
                    !happy
                    Hesh, that wasn't as simple as it looked.
                    Go tell {seemli} we're back in business!
            ]],

            OPT_LEAVE_AND_FAIL = "Abandon the Rise",
            DIALOG_LEAVE_AND_FAIL = [[
                * You decide that playing revolutionary isn't all it's cracked up to be, and leave.
            ]],
        }
        :Fn(function(cxt)
            
            cxt:Dialog("DIALOG_INTRO")

            local dilettante_used = false
            local true_believer_used = false
            local opportunist_used = false

            local allies = {}
            if cxt.quest:GetCastMember("dilettante"):IsInPlayerParty() then
                table.insert(allies, cxt.quest:GetCastMember("dilettante"))
            end

            if cxt.quest:GetCastMember("true_believer"):IsInPlayerParty() then
                table.insert(allies, cxt.quest:GetCastMember("true_believer"))
            end

            if cxt.quest:GetCastMember("opportunist"):IsInPlayerParty() then
                table.insert(allies, cxt.quest:GetCastMember("opportunist"))
            end

            local enemy_defs = {"ADMIRALTY_INVESTIGATOR", "ADMIRALTY_PATROL_LEADER", "ADMIRALTY_GOON", "VROC", "VROC", "ADMIRALTY_GUARD", "ADMIRALTY_GUARD"}

            local opfor = CreateCombatParty(enemy_defs, cxt.quest:GetRank(), cxt.location)

            cxt:TalkTo(opfor[1])
            cxt:RunLoop(function() 
                
                if cxt.quest:GetCastMember("dilettante"):IsInPlayerParty() and not dilettante_used then
                    cxt:Opt("OPT_USE_DILETTANTE")
                        :Dialog("DIALOG_USE_DILETTANTE")
                        :Fn(function() 
                            dilettante_used = true 
                            table.remove(opfor):MoveToLimbo()
                            table.remove(opfor):MoveToLimbo()
                        end)
                end


                if cxt.quest:GetCastMember("true_believer"):IsInPlayerParty() and not true_believer_used then
                    cxt:Opt("OPT_USE_TRUE_BELIEVER")
                        :Dialog("DIALOG_USE_TRUE_BELIEVER")
                        :Fn(function() 
                            true_believer_used = true 
                        end)

                end

                if cxt.quest:GetCastMember("opportunist"):IsInPlayerParty() and not opportunist_used then
                    cxt:Opt("OPT_USE_OPPORTUNIST")
                        :Dialog("DIALOG_USE_OPPORTUNIST")
                        :Fn(function() 
                            opportunist_used = true 
                            local cards = {"rpg","rpg","rpg"}
                            cxt:Opt("OPT_TAKE_ITEMS", cards)
                                :GainCards(cards)
                            cxt:Opt("OPT_LEAVE_ITEMS")                                
                        end)
                end

                cxt:Opt("OPT_ATTACK_THE_ADMIRALTY")
                    :Dialog("DIALOG_ATTACK_THE_ADMIRALTY")
                    :Battle{
                        allies = allies,
                        enemies = opfor,
                        on_start_battle = function(battle)
                            if true_believer_used then
                                for _, fighter in pairs( battle:GetTeam(TEAM.BLUE):GetAllFighters()) do
                                    if not fighter.agent:IsPlayer() then
                                        fighter:AddCondition("DAUNTLESS")
                                    end
                                end
                            end
                        end,

                    }
                        :OnWin()
                            :Fn(function() 
                                local alive = {}
                                for k,v in ipairs(allies) do
                                    if v:IsAlive() then
                                        table.insert(alive, v)
                                    end
                                end
                                
                                if #alive == 0 then
                                    cxt:Dialog("DIALOG_EVERYONE_DEAD")
                                    cxt.quest:Fail("regain_clubhouse")
                                else
                                    cxt:ReassignCastMember("survivor", table.arraypick(alive))
                                    cxt:TalkTo("survivor")
                                    cxt:Dialog("DIALOG_REGAINED")

                                    for k,v in pairs(alive) do
                                        cxt:Opinion(v, "restarted_revolution")
                                    end

                                    cxt.quest:Complete("regain_clubhouse")
                                end
                                StateGraphUtil.AddLeaveLocation(cxt) 


                            end)

                local opt = cxt:Opt("OPT_LEAVE_AND_FAIL")                            

                for k,v in pairs(allies) do
                    opt:ReceiveOpinion("abandoned_revolution", nil, v)
                end

                opt:Dialog("DIALOG_LEAVE_AND_FAIL")
                    :Fn(function() 
                        for k,v in pairs(allies) do
                            v:Dismiss()
                        end
                        cxt.quest.param.abandoned = true 
                    end)
                    :FailQuest("regain_clubhouse")
                    :Travel()


                    
            end)
        end)

QDEF:AddConvo("return_failure", "seemli")
    :Loc{

        OPT_GIVE_REPORT = "Tell {seemli} what happened",
        DIALOG_GIVE_REPORT = [[
            {attempted_regain?
                player:
                    I think The Man may have won this round.
                agent:
                    I worried as much. A ragtag band could hardly take on the Admiralty.
                    But we've demonstrated our intentions to the universe.
                    We can only hope the universe hears our plea.
                player:
                    Uh. Sure.
            }
            {abandoned?
                player:
                    That place was crawling with Switches!
                    It looked too hard, so I bailed.
                agent:
                    !facepalm
                    What kind of message does that send, {player}? 
                    How can we court victory if you can't even channel good vibrations?
            }
            {not attempted_regain and not abandoned?
                player:
                    Your old <i>comrades</i> left me high and dry.
                    Not a single one of them wanted back on board. 
                agent:
                    I hoped you'd be the connective tissue that would stitch them together.
                    The forces of oppression are driving us apart.
                    I--I need to write a new pamphlet about this.
            }
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_GIVE_REPORT")
            :SetQuestMark()
            :Fn(function() 
                cxt:Dialog("DIALOG_GIVE_REPORT")
                cxt.quest:Fail()
            end)
    end)

QDEF:AddConvo("return_to_seemli", "seemli")
    :Loc{

        OPT_GIVE_REPORT = "Tell {seemli} about the cell",
        DIALOG_GIVE_REPORT = [[
            player:
                Good news, Seemli. The revolution is back on!
            seemli:
                That's wonderful! Of course, I've managed to double our recruits among the pilgrims...
                But your efforts are very important, as well. 
                Our spiritual family grows apace!
            player:
                Whatever you say, boss.
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_GIVE_REPORT")
            :SetQuestMark()
            :Fn(function() 
                cxt:Dialog("DIALOG_GIVE_REPORT")
                ConvoUtil.GiveQuestRewards(cxt)
                cxt.quest:Complete()
            end)
    end)


QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}



    



