
local QDEF = QuestDef.Define{
        title = "The Fastest Oshnu",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{moreef} made a sizable bet on the next race at the Oshnu track. You need to make it a winner.]],
        icon = engine.asset.Texture("icons/quests/side_smith_snail_racing.tex"),
        focus = QUEST_FOCUS.BALANCED,
        tags = {"MOREEF_INTRO_JOB"},
        act_filter = "SMITH",
        avoids = {
            {"rider", "oshnudrome"},
            {"rider", "moreef"},
            {"chemist", "moreef"},
            {"chemist", "oshnudrome"}
        }

    }

    :AddCastByAlias{
        cast_id = "moreef",
        alias = "SWEET_MOREEF",
        provider = true,
        no_validation = true,
    }

    :AddCast{
        cast_id = "snail1",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, quest:CreateSkinnedAgent("RACING_SNAIL"))
            t[1]:GenerateRandomName()
        end,
    }

    :AddCast{
        cast_id = "snail2",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, quest:CreateSkinnedAgent("RACING_SNAIL"))
            t[1]:GenerateRandomName()
        end,
    }

    :AddCast{
        cast_id = "rider",
        cast_fn = function(quest, t) 
            table.insert(t, quest:CreateSkinnedAgent("OSHNU_WRANGLER"))
        end,
    }


    :AddCast{
        cast_id = "chemist",
        cast_fn = function(quest, t) 
            table.insert(t, quest:CreateSkinnedAgent("CHEMIST"))
        end,
    }

    :AddQuestLocation{
        cast_id = "snail_pen",
        name = "Oshnu Pen",
        desc = "Oshnus are kept apart before a race, because they can't stand each other.",
        plax = "INT_SMITH_SNAILPEN",
        map_tags = {"building"},
        indoors = true,
    }

    :AddLocationCast{
        cast_id = "oshnudrome",
        cast_fn = function(quest, t)
            table.insert(t, TheGame:GetGameState():GetLocation("PEARL_OSHNUDROME"))
        end,
    }

    :AddObjective{
        id = "start",
        on_activate = function(quest) 
            quest:Activate("talk_to_chemist")
            quest:Activate("intimidate_rider")
            quest:Complete('start')
        end,
    }

    :AddObjective{
        id = "talk_to_chemist",
        title = "Talk to {chemist}",
        desc = [[{chemist} might be able to give you something to affect one of the snails in the race.]],
        mark = { "chemist" },
        on_complete = function(quest)
            quest:Cancel("intimidate_rider")
        end
    }

    :AddObjective{
        id = "intimidate_rider",
        title = "Get {rider} to throw the race",
        desc = [[It might be easiest just to convince {rider} to throw the race...]],
        mark = { "rider" },
        on_complete = function(quest)
            quest:Cancel("talk_to_chemist")
            quest:Cancel("make_snail1_slower")
            quest:Cancel("make_snail2_faster")
            
            quest:Activate('attend_the_race')
        end
    }

    :AddObjective{
        id = "make_snail1_slower",
        title = "Compromise {snail1}",
        desc = [[{snail1} will not be able to race effectively with a hangover. You know what to do.]],
        mark = { "snail_pen" },
        on_complete = function(quest)
            quest.param.snail1_slow = true
            quest:Cancel("intimidate_rider")
            quest:Activate('attend_the_race')
        end

    }

    :AddObjective{
        id = "make_snail2_faster",
        title = "Enhance {snail2}",
        desc = [[{chemist} gave you a concoction to make {snail2} race like {snail2.heshe}'s never raced before. Administer it <i>discreetly</>.]],
        mark = { "snail_pen" },
        on_complete = function(quest)
            quest.param.snail2_fast = true
            quest:Cancel("intimidate_rider")
            quest:Activate('attend_the_race')
        end
    }

    :AddObjective{
        id = "attend_the_race",
        title = "Attend the race",
        desc = [[It's time for the race! Go watch the outcome and collect {moreef}'s winnings.]],
        mark = { "oshnudrome" },
        
        on_complete = function(quest)
            quest:Activate("return_to_moreef")
        end,

    }

    :AddObjective{
        id = "return_to_moreef",
        title = "Tell {moreef} {moreef.heshe} won",
        desc = [[{moreef}'s oshnu won!]],
        mark = { "moreef" },
        on_complete = function(quest)
            local overrides = { cast = { oshnu = quest:GetCastMember("snail1") } }
            local quest = QuestUtil.SpawnQuest("SMITH_STORY_GET_RACING_SNAIL_LATER", overrides)
        end,
    }

    :AddOpinionEvents{

        old_frenemy_chemist =  
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "You may have gotten them busted once upon a time",
        },
        good_tipper = 
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "Gave them a good tip at the Oshnudrome",
        },
        sketchy_bet =  
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Made a suspicious bet at the Oshnudrome",
        },
    }



QDEF:AddIntro(
    [[
        agent:
            !interest
            Not sure they have them wherever you've been, but have you been following the races?
        player:
            !thought
            The oshnus? Naw. 
            Everything moves too quick. No drama. 
        agent:
            !placate
            Normally I don't mess with gambling. Too risk averse.
            But these are strange times and... well, I just happen to have a lead on the next race.
            !cagey
            $miscHushedUrgent
            Everyone expects {snail1} to win, but {snail1.heshe}'s apparently had an injury. One race away from the noodlehouse, they said. 
            So those in the know are actually putting shills on {snail2}.
        player:
            !dubious
            Those in the know, huh? And who told you?
        agent:
            Nobody. I overheard a patron talking about it with a friend.
        player:
            !point
            Oh yeah? They were sitting, what, here at the bar?
        agent:
            !agree
            Right in front of me.
            They were keeping quiet, but some folks think the bartender is part of the scenery.
        player:
            $neutralWhatever
            Uh huh. I've run that scam before, too.
        agent:
            !surprised
            $neutralTakenAback
            Scam?
        player:
            Sure. Puts more shills in circulation, and buddy wins 'em all.
            Take my advice and just forget it.
        agent:
            !scared_shrug
            I would, but...
        player:
            !sigh
            $neutralDisappointed
            You already made your wager. How much?
        agent:
            More than I can afford to lose.
    ]],
    [[
        player:
            !thought
            Let me think. Been a while since I thought about oshnus.
            Other than, you know. For eatin'.
        agent:
            I'm washed. How could I be so stupid?
            There's no getting my money back.
        player:
            Sure there is, old bud. And then some. You just need to make sure your snail wins. 
            When's the race?
        agent:
            In a couple of hours.
        player:
            Alright, no problem. Plenty of time to turn the odds in your favor.
            Sit pretty. I'll be back.
    ]],
    "start")



QDEF:AddConvo("talk_to_chemist", "chemist")
    :Loc{
        OPT_ASK_ABOUT_SNAILS = "Ask {agent} about oshnu physiology",
        DIALOG_ASK_ABOUT_SNAILS = [[
            {first_time ?
                player:
                    !thought
                    Hey, whaddya know about, uh, mucus membranes?
                agent:
                    !crossed
                    I know that if you want yours clogged, you're going to have to find somebody else.
                    I don't sell uppers to riff-raff.
                player:
                    Not <i>mine</i>, ya loon. An oshnu's. 
                agent:  
                    Oh, well. Hm.
                    I don't typically offer veterinary services—chemically speaking, oshnus are very complicated.
                player:
                    Yeah? Must be hard to make 'em go fast. Or slow 'em down.
                agent:
                    !dubious
                    Well, yes. That's why there's a whole industry built up around it. 
                    I do sell a few substances that, well, can substitute for certain pheromones...
                player:
                    How much?
                agent:
                    Ah. I see. 
                    I suspect you won't just be paying for my expertise, but my discretion, as well.
            }
            {not first_time?
                player:
                    So, about those oshnu uppers...
                agent:
                    Back again? The issue remains the same.
                    You get what you pay for, and you're asking for a lot.
            }
        ]],
        

        DIALOG_ASK_ABOUT_SNAILS_CONVINCED = [[
            player:
                Let's talk business.
            agent:
                Somehow I suspect that's not what I would call it.
        ]],
        
        OPT_PAY = "Pay money",
        DIALOG_PAY = [[
            player:
                No discount for old time's sake?
                !give
            agent: 
                !take
                You're lucky I'm not charging you triple.
        ]],

        OPT_CONVINCE = "Convince {agent} to waive {agent.hisher} fee",
        DIALOG_CONVINCE = [[
            player:
                Huh, seems to me there's money to be made in maybe studying oshnus a little more closely.
                Especially swanky, pedigree oshnus like they got here in the Foam.
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                If only you had an agent who could get close to the thoroughbreds, then you could just sit back and observe.
                None of the risk, all of the reward.
            agent:
                You're proposing I conduct doping experiments on the Pearl's prize oshnus?
            player:
                Naw. I'm proposing you just keep an eye out and, you know, <i>pay attention</i>.
                Whatever you learn could probably be pretty valuable. 
            agent:  
                ...
                I suppose that's true. And if my dosage kills the snail, that's a lesson learned, as well.
                Very well.
        ]],
        DIALOG_CONVINCE_FAILURE = [[
            player:
                You want to know what makes these oshnus tick, don't ya?
            agent:
                No. Doing business with the Pearl is hard enough as it is. 
                I'll do work for pay, or not at all.
        ]],

        OPT_GIVE_SAMPLE = "Give {agent} a tissue sample",
        DIALOG_GIVE_SAMPLE = [[
            player:
                You bilebrokers like, uh, <i>stuff</i>, right? To do your experiments on?
            agent:
                Stuff?
            player:
                You know. Bits. And stuff.
            agent:
                Ah. You mean living tissue. 
                Yes, it's always valuable for testing reagents from the Roaloch. 
                If that's the arrangement you'd like to make, I promise not to take anything that won't grow back.
            player:
                !shrug
                Then sure. Yeah. What's the harm?
            agent:
                Quite a bit, actually.
                * The chemist takes enough to leave you feeling just a bit gutted. 
        ]],
        DIALOG_GIVE_SAMPLE_2 = [[
            player:
                !left
                Hesh's tentacles, that stings.
                !give
            agent:
                !take
            * {agent} takes the tissue, wraps it in an oilcloth and stashes it away.
            agent:
                That will keep my research going for a while.
                Now, what do you need?
        ]],

    }

    :AttractState("STATE_ATTRACT")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * Bilebrokers aren't exactly welcome in the Pearl, but the old and young alike often seek out their concoctions.
                    agent:
                        !angry_accuse
                        Hey! You're not allowed in here.
                    * The face isn't familiar, but the tone is. 
                    player:
                        !dubious
                        Huh? Who, me? 
                    * {agent} points to a rough sketch behind the counter that looks more or less like you--only fifteen years younger.
                    agent:
                        Banned for life, I believe that says. Sound familiar?
                    player:
                        !placate
                        $neutralTakenAback
                        Ah, no. I've, uh, never been here before. I'm just a pilgrim, right?
                        First time in the Pearl. 
                        !embarrassed
                        Oh, wow! It's so... so nice here. Real fancy.
                        Never in all my dreams did I imagine me, Jo McNobody, would end up in Pearl-on-the-Film.
                    agent:
                        <i>Foam</i>.
                    player:
                        Right, that's the kinda mistake a nobody like me would make for sure.
                    agent:
                        !crossed
                        ...
                        Business is slow. So you can browse.
                        $neutralDirect
                        But keep your hands where I can see them, and don't touch anything. 
                        These aren't beauty supplies.
                }

                {not first_time?
                    agent:
                        !angry
                        I'm watching you.
                    player:
                        !shrug
                        I can tell.
                }
            ]]
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            if not cxt.quest.param.met_chemist then
                cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("old_frenemy_chemist"))
            end
        end)

    :Hub(function(cxt) 

        
        if cxt.quest.param.bribed_chemist or cxt.quest.param.convinced_chemist or cxt.quest.param.gave_sample_to_chemist then
            cxt:Opt("OPT_ASK_ABOUT_SNAILS")
                :SetQuestMark()
                :Dialog("DIALOG_ASK_ABOUT_SNAILS_CONVINCED")
                :GoTo("STATE_CHOOSE")
        else

            cxt:Opt("OPT_ASK_ABOUT_SNAILS")
                :SetQuestMark()
                :Dialog("DIALOG_ASK_ABOUT_SNAILS")
                :Fn(function() 

                    cxt:Opt("OPT_PAY")
                        :Dialog("DIALOG_PAY")
                        :DeliverMoney(cxt.quest:GetRank()*50)
                        :Fn(function() 
                            cxt.quest.param.bribed_chemist = true
                        end)
                        :GoTo("STATE_CHOOSE")
                    
                    cxt:Opt("OPT_GIVE_SAMPLE")
                        :Dialog("DIALOG_GIVE_SAMPLE")
                        :DeltaHealth(-20)
                        :Dialog("DIALOG_GIVE_SAMPLE_2")
                        :Fn(function() cxt.quest.param.gave_sample_to_chemist = true end)
                        :GoTo("STATE_CHOOSE")

                    cxt:Opt("OPT_CONVINCE")
                        :Dialog("DIALOG_CONVINCE")
                        :Negotiation{}
                            :OnSuccess()
                                :Fn(function() cxt.quest.param.convinced_chemist = true end)
                                :Dialog("DIALOG_CONVINCE_SUCCESS")
                                :GoTo("STATE_CHOOSE")
                            :OnFailure()
                                :Dialog("DIALOG_CONVINCE_FAILURE")

                end)
        end

    end)

    :State("STATE_CHOOSE")
        :Loc{

            DIALOG_INTRO = [[
                {first_time?
                    player:
                        All I need is some simple oshnu drugs. 
                        Something to make 'em kick up their heels. Or drag 'em. Either works.
                        Whichever's easier.
                    agent:
                        Neither of them are what I'd consider <i>easy</i>.
                        Oshnus aren't of this world.
                    player:
                        So? Neither are we. I mean, strictly speaking.
                    agent:
                        And oshnus are more alien still. 
                        All the same, it's all pheromones, one way or the other.
                        I can make a perfume, but it'll be costly whichever way you decide.
                }

                {not first_time?
                    agent:
                        So which perfume should I prepare? A stimulant, or a sedative?
                }
            ]],
            
            OPT_ACCEPT = "Agree to this plan",
            OPT_DECLINE = "Think it over a while longer",
            
            OPT_GO_FASTER = "Ask about speeding up an oshnu",
            DIALOG_DIALOG_GO_FASTER = [[
                player:
                    You know what? I'm impatient. Let's make the oshnu <i>fast</i>.
                agent:
                    Hm, yes. Well a mating pheromone would certainly provide the motivation, but you'll need something physiological for the thrust.
                player:
                    Uh—
                agent: 
                    Just a little bit of spark delivered to the amygdala will trigger a metabolic cascade. 
                    I'll have to add a time-release inhibitor or its hearts will all explode, of course.
                player:
                    Is this... safe?
                agent:
                    For the oshnu? Yes.
                    For you? 
                    ...
            ]],
            DIALOG_ACCEPT_FASTER = [[
                player:
                    Well I ain't getting any younger so...
                    I guess we'll go for that one.
            ]],
            DIALOG_DECLINE_FASTER = [[
                player:
                    I, uh, maybe wanna consider my other options here.
            ]],

            OPT_GO_SLOWER = "Ask about slowing down an oshnu",
            DIALOG_DIALOG_GO_SLOWER = [[
                player:
                    I know a snail that could use a rest. How do we slow it down?
                agent:
                    Ah, well, to that end we simply target the beast's spatio-temporal perception. 
                    Lacing a sedative with a recent discovery from the Roaloch—endothelial-penetrating spores, nothing too alarming—will tether that area of the creatures' brain to its cranial sheath.
                    Its sense of time will be pretty much literally suspended. 
                player:
                    You said "simply".
                agent:
                    Well, in comparison.
                player:
                    To what?
                agent:
                    Metaphysical nanotechnology?
                player:
                    Now you're just making things up. 
            ]],
            DIALOG_ACCEPT_SLOWER = [[
                player:
                    All I need to know is that it works.
                agent:
                    Alarmingly well.
            ]],
            DIALOG_DECLINE_SLOWER = [[
                player:
                    I think you're having me on with that one.
            ]],

            DIALOG_MAKE_POTION = [[
                agent:
                    Give me a moment.
                    !exit
                * {agent} lines up a row of bubbling beakers before you can bark. You let your eyes glaze over just long enough to get past all the science stuff.
                agent:
                    !right
                    !give
                    Here. It's done.
                player:
                    Hey, that looks... kinda tasty, actually?
                agent:
                    Do not eat it.
                    Unless you want to regurgitate your digestive lining. Out both ends at once.
                player:
                    That's a hard pass.
                agent:
                    Yes. It is.

            ]],

            OPT_NEED_TO_THINK = "Reconsider your options",
            DIALOG_NEED_TO_THINK = [[
                player:
                    This is starting to sound way more complicated than it needs to be.
                    How hard can it be to rig a race, huh?
                    I'm overthinking this. 
                agent:
                    Something tells me that's not likely.
                    {bribed_chemist?
                        But I have your shills, and I'll be keeping them.
                        So do what you want.
                    }
                    {not bribed_chemist?
                        I'll be here when you come to your senses.
                    }
                    
            ]],
        }
        :Fn(function(cxt) 

            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function()


                cxt:Opt("OPT_GO_FASTER")
                    :Dialog("DIALOG_DIALOG_GO_FASTER")
                    :Fn(function() 

                        cxt:Opt("OPT_ACCEPT")
                            :Dialog("DIALOG_ACCEPT_FASTER")
                            :Dialog("DIALOG_MAKE_POTION")
                            :Fn(function() 
                                cxt.quest:Complete("talk_to_chemist")
                                cxt.quest:Activate("make_snail2_faster")
                            end)
                            :DoneConvo()

                        cxt:Opt("OPT_DECLINE")
                            :Dialog("DIALOG_DECLINE_FASTER")

                    end)

                cxt:Opt("OPT_GO_SLOWER")
                    :Dialog("DIALOG_DIALOG_GO_SLOWER")
                    :Fn(function() 

                        cxt:Opt("OPT_ACCEPT")
                            :Dialog("DIALOG_ACCEPT_SLOWER")
                            :Dialog("DIALOG_MAKE_POTION")
                            :Fn(function() 
                                cxt.quest:Complete("talk_to_chemist")
                                cxt.quest:Activate("make_snail1_slower")
                            end)
                            :DoneConvo()

                        cxt:Opt("OPT_DECLINE")
                            :Dialog("DIALOG_DECLINE_SLOWER")

                    end)

                cxt:Opt("OPT_NEED_TO_THINK")
                    :Dialog("DIALOG_NEED_TO_THINK")
                    :DoneConvo()
            end)
        end)


QDEF:AddConvo("intimidate_rider", "rider")
    :Loc{
        OPT_ASK_ABOUT_SNAILS = "Ask about the upcoming race",
        DIALOG_ASK_ABOUT_SNAILS = [[
            player:
                Hey, aren't you that {rider.gender:guy|gal}? That {rider.gender:guy|gal} with the famous snail?
                {snail1}'s rider, right? That you?
            agent:
                Excuse me? {snail1} is the snail with the famous <i>rider</i>. 
                And yes, that rider is me.
            player:
                Oh yeah, sure. My kid's a big fan.
                {snail1} this and {snail1} that. I bet she'd be thrilled to learn I met the real life rider.
            agent:
                !sigh
                $neutralResigned
                I suppose you want an autograph?
        ]],

        OPT_BRIBE = "Bribe {agent} to take a dive",
        DIALOG_BRIBE = [[
            player:
                How much is an autograph?
            agent:
                Three shills. Six if you want me to draw a little happy snail with it.
                It's not a very good snail, mind, but if your kid's an idiot then she might not mind.
            player:
                That sounds great. I'll take...
                !thought
                er...
                {1} of them. But you can skip the drawing. And the autographing, too.
            agent:
                I—what? I don't understand.
            player:
                I'm gonna pay you money, yeah? We're agreed on that part. 
                And in exchange, you're going to do me a favor. A very slow favor. 
            agent:  
                Oh. Oh! You're <i>bribing</i> me. 
            player:
                !dubious
                Yep. You got a problem with that?
            agent:
                !sigh
                Honestly, no. I'm starting to feel neglected, you know? Like I've accomplished so much, but I'm completely replaceable.
                Would {snail1} even notice if I were to drop dead? Replaced with another rider?
                At least this way it's a form of controlled self-sabotage. An end to my career that I, {agent}, could truly say—
            player:
                !handwave
                Yeah, yeah, jabber jabber. Let's just make it happen, huh? 
        ]],
        OPT_THREATEN = "Threaten {agent} into losing",
        DIALOG_THREATEN = [[
            player:
                You as fast on your feet as you are in the saddle?
            agent:  
                What? No, not particularly.
            player:
                Then I guess you got nowhere to run, huh?
        ]],
        DIALOG_THREATEN_SUCCESS = [[
            player:
                See, I need you to lose that race today. No matter what.
                And if you don't agree to it like civilized folk should, then I'll have to kill you.
            agent:
                !scared
                $scaredStammering
                N-no! No you won't!
                I'll agree! I'm civilized!
            player:
                Happy to hear it.            
                Just remember, if you win that race, you lose.
                Like, after. 
                !throatcut
                'Cause I'll hurt you.
            agent:
                Yes, I understand that part!
        ]],
        DIALOG_THREATEN_FAILURE = [[
            player:
                Because you're gonna need to run away from me if you win that race.
            agent:
                Was that... a threat?
            player:
                Yeah. Yeah! Obviously.
            agent:
                Well, not really. It was kind of difficult to follow and I'm still not sure I get the thread.
                Suffice it to say... I'm not afraid of you.
                And I have every intention of winning the race.
        ]],
        OPT_BEAT_UP = "Beat up {agent}",
        DIALOG_BEAT_UP = [[
            player:
                !fight
            agent:
                !surprised
                Hey, what are you doing?
            player:
                Skipping to the good part.
        ]],
        DIALOG_WIN = [[
            {dead?
                * {agent} is dead.
                !thought
                * You briefly consider the logistics of propping {agent.hisher} corpse up on top of the snail and letting it run the race.
                * But a murder has a tendency to get things, like races, cancelled. 
                * Spoilsport. 
            }
            {not dead?
                agent:
                    !injured
                    Stop it! I'll throw the race!
                    Not as if my integrity is of value in these times, anyway.
                player:
                    Integrity? Like you've never taken a bribe?
                agent:
                    Of course I have! Bribing is sportsmanlike!
                    You're just a thug. You wouldn't understand.
                player:
                    !exit
                * You leave {agent} to tend to {agent.hisher} bruised ego.
            }
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_ABOUT_SNAILS")
            :SetQuestMark()
            :Dialog("DIALOG_ASK_ABOUT_SNAILS")
            :Fn(function() 

                cxt:RunLoop(function() 
                    
                    local bribe_amount = 100*cxt.quest:GetRank()
                    cxt:Opt("OPT_BRIBE")
                        :DeliverMoney(bribe_amount, {no_scale = true})
                        :Dialog("DIALOG_BRIBE", math.ceil(bribe_amount / 3 ))
                        :CompleteQuest("intimidate_rider")
                        :Fn(function() cxt.quest.param.bribed_rider = true end)
                        :DoneConvo()

                    cxt:Opt("OPT_THREATEN")
                        :Dialog("DIALOG_THREATEN")
                        :Negotiation{flags = NEGOTIATION_FLAGS.INTIMIDATION}
                            :OnSuccess()
                                :Dialog("DIALOG_THREATEN_SUCCESS")
                                :CompleteQuest("intimidate_rider")
                                :Fn(function() cxt.quest.param.intimidated_rider = true end)
                                :DoneConvo()

                            :OnFailure()
                                :Dialog("DIALOG_THREATEN_FAILURE")

                    cxt:Opt("OPT_BEAT_UP")
                        :Dialog("DIALOG_BEAT_UP")
                        :Battle{}
                            :OnWin()
                                :Dialog("DIALOG_WIN")
                                :CompleteQuest("intimidate_rider")
                                :Fn(function() 
                                    if cxt:GetAgent():IsDead() then
                                        cxt.quest.param.killed_rider = true
                                    else
                                        cxt.quest.param.intimidated_rider = true
                                    end
                                end)
                                :DoneConvo()

                    StateGraphUtil.AddEndOption(cxt)
                end)

            end)
    end)


QDEF:AddConvo("make_snail1_slower")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("snail_pen")  end)
        :Loc{
            DIALOG_INTRO = [[
                * You find {snail1}'s pen, and wait for the coast to clear.
                player:
                    !left
                agent:
                    !right
                player:
                    Hey there, buddy.
                * The oshnu stares back at you, unmoved by your friendly address.
            ]],
            OPT_DRUG_SNAIL = "Feed the potion to {snail1}",
            DIALOG_DRUG_SNAIL = [[
                player:
                    !give
                    I've got a treat for ya. You want it? Huh? 
                * The Oshnu sniffs at the concoction and snorts a disapproval. 
                * Looks like it's going to take some convincing.
            ]],
            OPT_FORCE_THE_SNAIL = "Force {agent} to drink the concoction",
            DIALOG_FORCE_THE_SNAIL = [[
                * You take a deep breath, and then reach for the oshnu's head.
                player:
                    !take
                    !exit
                agent:
                    !exit
            ]],
            DIALOG_FORCE_THE_SNAIL_2 = [[
                * After several painful bites, you manage to get most of the potion down the oshnu's throat.
                agent:
                    !right
                player:
                    !left
                    !injured
                    Calm down, ya baby! You act like nobody's tried to dope you before.
                * The oshnu huffs and retracts into its shell.
                agent:
                    !exit
                * You decide to leave before anyone comes to investigate the commotion.
            ]],

            OPT_DEMONSTRATE = "Drink some of the concoction yourself to prove its safety",
            DIALOG_DEMONSTRATE = [[
                player:
                    Oooh, look. S'got pretty colors. And it smells so tasty. See?
                    !drink
                * You take a slug of the concoction, which slides down your throat like a silk slipper.
                player:
                    !drunk
            ]],
            DIALOG_DEMONSTRATE_2 = [[
                player:
                    That's... that's not half bad, actually.
                * The oshnu's curiosity has been piqued. It guzzles the remainder of the brew when offered.
                player:
                    You know, you're alright, {snail1}!
                    Hey don't—don't be sad when you lose later, alright? It's not you. You're a good snail.
                    !exit
                * You stumble out of the pen, the harsh sun making your eyestalks ache.
            ]],
        }
        :Fn(function(cxt) 
            cxt:TalkTo("snail1")
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_FORCE_THE_SNAIL")
                :Dialog("DIALOG_FORCE_THE_SNAIL")
                :DeltaHealth(-15)
                :Dialog("DIALOG_FORCE_THE_SNAIL_2")
                :CompleteQuest("make_snail1_slower")
                :Travel()

            cxt:Opt("OPT_DEMONSTRATE")
                :Dialog("DIALOG_DEMONSTRATE")
                :Fn(function() AgentUtil.MakeDrunk( cxt.player ) end)
                :Dialog("DIALOG_DEMONSTRATE_2")
                :CompleteQuest("make_snail1_slower")
                :Travel()
        end)


QDEF:AddConvo("make_snail2_faster")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("snail_pen")  end)
        :Loc{
            DIALOG_INTRO = [[
                * Getting into the stables is easier than it should be, which makes you wonder why you've never rigged a race before now.
                * While most of the pens are labeled with elaborate, filigreed plaques, {snail2}'s is nothing but chipped paint and old plywood.
                player:
                    !left
                snail2:
                    !right
                player:
                    I like to keep it simple too, old slimer. 
                    Whaddya say to a little treat, huh? 
                * {snail2} is a big one. Which explains why you didn't notice the stable hand with a fork full of fewmets lurking just behind the snail.
                player:
                    !surprised
                agent:
                    !right
                    !dubious  
                    {not met?
                        Hey! No treats for the oshnu before a race.
                    }
                    {met?
                        {player}? Didn't realize you had business in the stables.
                    }

            ]],

            OPT_CONVINCE = "Convince {agent} you're here to help",
            DIALOG_CONVINCE = [[
                player:
                    Well, uh, y'see, I'm a veterinary student. And this oshnu's looking a little rough, ya know?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !point
                    It's the kinda funky colour around the, uh—that part.
                agent:
                    The mantle? But... {snail2.heshe}'s had those markings since {snail2.heshe} was a whelk.
                player:
                    Well, geez, you been putting off treatment that long? 
                    You want me to write you up for animal endangerment? 
                    'Cause I can. As a veterinary student. That's a legit power I have.
                agent:
                    No! Just... give {snail2} the medicine. 
                snail2:
                    !right
                player:
                    !give
                * You tip the concoction into the oshnu's trough, where it slurps it up with a shudder.
                agent:
                    !right
                    Is {snail2.heshe} going to be okay? Should I pull {snail2.himher} from the race?
                player:
                    No! No, no.
                    {snail2}'s gotta, you know, sweat out the sickness.
                    Excretions, toxins, yadda yadda.
                    The race is <i>very important</i>. More important than the medicine, even.
                    !exit
                * {agent} looks confused, so you make a hasty exit before {agent.heshe} can doubt your clinical experience.
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                agent:

                player:
                    Last time I saw an oshnu this bad, it was already a noodle!
                agent:
                    Is that right? If you're an oshnu doctor, where's your hat?
                player:
                    !palm 
                    I've never really looked good in hats.
                agent:
                    !crossed
                    Then it's hard to imagine how you would've passed your veterinary exam.
            ]],

            OPT_INTIMIDATE = "Scare {agent} away",
            DIALOG_INTIMIDATE = [[
                player:
                    !threaten
                    Did you ever think you'd die as you lived?
                    Shoveling snail turds for a D-rank oshnu?
                agent:
                    W-what do you mean by that, exactly?
            ]],
            DIALOG_INTIMIDATE_SUCCESS = [[
                player:
                    Just think there are probably better ways to die than choking on droppings face down in a snailpit, ya know?
                agent:
                    !scared
                    I... I didn't see anything.
                    !exit
                * {agent} leaves, and lives to shovel another stable.
                snail2:
                    !right
                player:
                    !give
                * You tip the concoction into the oshnu's trough, where it slurps it up with a shudder.
                player:
                    Hopefully your race goes as well as your appetite, huh?
                    !exit
                * You leave before {agent} finds {agent.hisher} spine and returns.

            ]],
            DIALOG_INTIMIDATE_FAILURE = [[
                player:
                    I mean you could die out here and nobody would know. 
                agent:
                    !crossed
                    Except this loyal, unwavering oshnu, who, by the way, comes from Admiralty stock.
            ]],
            
            OPT_FIGHT = "Beat up {agent}",
            DIALOG_FIGHT = [[
                player:
                    !fight
                    Whatever.
            ]],
            DIALOG_WIN = [[
                {dead?
                    * You toss {agent}'s body into the piled fewmets.
                }
                {not dead?
                    agent:
                        !injured
                    player:
                        Some oshnu, huh? Didn't even spit mucus while you were taking a beating.
                        Now get outta here, before I decide you've seen too much.
                    agent:
                        !exit
                }
                snail2:
                    !right
                player:
                    Sorry for the delay, you fancy slug. Allow me to serve you.
                * You tip the concoction into the oshnu's trough, where it slurps it up with a shudder.                
                player:
                    Hopefully your race goes as well as your appetite, huh?
                    !exit
                * You leave before anyone comes to investigate the commotion.
            ]],
        }
        :Fn(function(cxt) 
            
            local agent = AgentUtil.GetFreeAgent("OSHNU_WRANGLER")
            cxt:TalkTo(agent)
            cxt:Dialog("DIALOG_INTRO")

            local tried_negotiation = false
            
            cxt:RunLoop(function() 
                
                if not tried_negotiation then
                    cxt:Opt("OPT_CONVINCE")
                        :Dialog("DIALOG_CONVINCE")
                        :Fn(function() tried_negotiation = true end)
                        :Negotiation{}            
                            :OnSuccess()
                                :Dialog("DIALOG_CONVINCE_SUCCESS")
                                :CompleteQuest("make_snail2_faster")
                                :Travel()

                            :OnFailure()
                                :Dialog("DIALOG_CONVINCE_FAILURE")

                    cxt:Opt("OPT_INTIMIDATE")
                        :Fn(function() tried_negotiation = true end)
                        :Dialog("DIALOG_INTIMIDATE")
                        :Negotiation{flags = NEGOTIATION_FLAGS.INTIMIDATION}
                            :OnSuccess()
                                :Dialog("DIALOG_INTIMIDATE_SUCCESS")
                                :CompleteQuest("make_snail2_faster")
                                :Travel()
                            :OnFailure()
                                :Dialog("DIALOG_INTIMIDATE_FAILURE")
                end

                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{}
                        :OnWin()
                            :Dialog("DIALOG_WIN")
                            :CompleteQuest("make_snail2_faster")
                            :Travel()

            end)

        end)



QDEF:AddConvo("attend_the_race")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("oshnudrome")  end)
        :Loc{
            DIALOG_INTRO = [[
                * You arrive at the Oshnudrome just before the beginning of the race.
                agent:
                    !right
                    Last call for wagers! Place your bets before the lights dim!
                player:
                    !left
            ]],

            OPT_PLACE_BET = "Place a bet on {snail2}",
            DIALOG_PLACE_BET = [[
                player:
                    !left
                    Hey, pal. Be a sport and put my money on {snail2}, wouldja?
                agent:
                    A fan of the long shot, eh?
                    Well, we all must have our quirks.
                    Good luck!
                    !exit
                * {agent} takes your money and hands you a receipt.
            ]],

            OPT_ASK_ABOUT_SNAILS = "Ask about the odds",
            DIALOG_ASK_ABOUT_SNAILS = [[
                player:
                    What odds am I looking at, here? I ain't usually one for the races.
                agent:
                    {snail1} is the favorite—descended from Supplicant City pedigree, a whelk from the same clutch is said to have been sent to the Emperor themself.
                {intimidated_rider?
                    !thought
                    Although between you and me, {snail1.hisher} rider's been acting a bit strangely today. 
                    Perhaps too much celebrating after their last win? 
                }
                {snail2_fast?
                    !thought
                    Surprisingly, people on the ground tell me that {snail2}'s been frothing at the glands—an unusual display of vigor for the beast.
                    Maybe it knows something we don't, hm?
                }
                {snail1_slow?
                    !thought
                    But reports have it that {snail1.heshe} was looking a bit sluggish today.
                    Err—no pun intended, of course.
                }
                {killed_rider?
                    !thought
                    But {snail1.hisher} usual rider hasn't shown up today, so the sponsors have sent in a replacement. That might upset the usual dynamics of the field.
                }
                player:
                    Well, ya don't say? That <i>is</i> interesting.
            ]],


            OPT_SKIP_BET = "Skip the bet",
            DIALOG_SKIP_BET = [[
                player:
                    I think I'll watch and wait for the next heat.
                agent:
                    Very well. But remember: a wager is never wasted as long as you had fun.
                    !exit
            ]],

        }
        :Fn(function(cxt) 
            
            local agent = AgentUtil.GetFreeAgent("WEALTHY_MERCHANT")
            cxt:TalkTo(agent)
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function(cxt) 
                cxt:Question("OPT_ASK_ABOUT_SNAILS", "DIALOG_ASK_ABOUT_SNAILS")
                
                local bet_amt = 50
                
                cxt:Opt("OPT_PLACE_BET")
                    :Dialog("DIALOG_PLACE_BET")
                    :DeliverMoney(bet_amt, {no_scale = true})
                    :Fn(function() cxt.quest.param.bet_amt = bet_amt end)
                    :GoTo("STATE_RACE")

                cxt:Opt("OPT_SKIP_BET")
                    :Dialog("DIALOG_SKIP_BET")
                    :GoTo("STATE_RACE")

            end)
        end)

    :State("STATE_RACE")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !exit
                agent:
                    !exit

                * You park yourself on a stool at the viewing window and flag down a server for some deep-fried fungiforms.
                * The plate arrives just in time for the oshnus to explode out of the starting gate.
                
                {snail1_slow?
                    * All except for {snail1}, that is. Despite {snail1.hisher} rider kicking heels into the poor oshnu's slippery gut, {snail1.heshe} seems content to take it slow.
                }
                {snail2_fast?
                    * {snail2} soon pulls ahead of the pack, {snail2.hisher} surprised rider yelping loud enough to pick it up on the speakers.
                }
                {killed_rider?
                    * {snail1} starts off strong, but bucks when {snail1.hisher} replacement rider jerks the yoke too hard. 
                }
                {intimidated_rider?
                    * {snail1} starts off strong before mysteriously falling behind, the rider's spurs hanging limp at the oshnu's flanks.
                }

                * The three laps are over just as you finish your food, and {snail2} emerges victorious.
                * The mood in the Oshnudrome has taken a sharp dive. Good thing you're clearing out on a high note.
            ]],
            
            OPT_TAKE_WINNINGS = "Collect your winnings",
            DIALOG_TAKE_WINNINGS = [[
                agent:
                    !right
                player:
                    !left
                    Whaddya know? Looks like I'm a winner, huh?
                agent:
                    Seems so. A very select few will be very happy with this turn of events.
                    I wonder what you all knew that even the tides of Hesh couldn't seem to predict.
            ]],

            OPT_TIP_WELL = "Give {agent} a good tip",
            DIALOG_TIP_WELL = [[
                player:
                    !shrug
                    I can't speak for the others, but for me it's just beginner's luck.
                    But I know what goes around comes around. So why don't you share a bit of my fortune.
                    !give
                agent:
                    !take
                    I'm very pleased to do so. May your luck continue, hm?
                player:
                    I've got a hunch it will.
                    !exit
            ]],
            
            OPT_LEAVE = "Leave without tipping",
            DIALOG_LEAVE = [[
                player:
                    Hah! Sounds like a sore loser talkin'.
                    See you on the flipper side, poindexter.
                    !exit
            ]],


        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")

            cxt.quest:Complete("attend_the_race")

            if cxt.quest.param.bet_amt then
                cxt:Opt("OPT_TAKE_WINNINGS")
                    :Dialog("DIALOG_TAKE_WINNINGS")
                    :ReceiveMoney(cxt.quest.param.bet_amt*2, {no_scale = true})
                    :Fn(function() 

                        cxt:Opt("OPT_TIP_WELL")
                            :Dialog("DIALOG_TIP_WELL")
                            :ReceiveOpinion("good_tipper")
                            :DeliverMoney(20)
                            :Travel()

                        cxt:Opt("OPT_LEAVE")
                            :Dialog("DIALOG_LEAVE")
                            :ReceiveOpinion("sketchy_bet")
                            :Fn(function() cxt.quest.param.suspicious = true end)
                            :Travel()
                    
                    end)
            else
                StateGraphUtil.AddLeaveLocation(cxt)
            end
        end)





QDEF:AddConvo("return_to_moreef")
    :ConfrontState("CONF", function(cxt) return not cxt.quest.param.has_had_post_confront and cxt.location:HasTag("in_transit") end) 
        :Loc{
            DIALOG_INTRO = [[
                * You're stopped by an Admiralty patrol, and it's clear this is more than just a routine check-in.
                player:
                    !left
                agent:
                    !right
                    Hold it, grifter. We have some questions about the oshnu races.
                player:
                    I ain't much of an expert. You lookin' for tips?
                agent:
                    We have reason to believe today's race might have been the result of tampering.
                    And witnesses describe a kra'deshi much like you loitering in off-limits areas.
            ]],

            OPT_BRIBE = "Throw money at the problem",
            DIALOG_BRIBE = [[
                player:
                    !shrug
                    You might have noticed Pearl-on-the-Foam is full of kra'deshi just like me.
                    Handsome. Smart. Rich.
                    Here. Let me donate to your investigation. Maybe with this you can get a clue.
                    !give
                agent:
                    !take
                    ...
                    Indeed.
                    !cough 
                    As you were.
                    !exit
            ]],

            OPT_CONVINCE = "Convince {agent} of your (relative) innocence",
            DIALOG_CONVINCE = [[
                player:
                    Off-limits? Didn't realize it was off-limits to place a wager.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    What's more likely: that I somehow rigged an oshnu race, or that some folks are just mad they lost and I won?
                    Or are you new to the Foam? Maybe you don't know what the locals are like.
                agent:
                    !dubious
                    ...
                    I suppose that's a fair point. 
                    Wouldn't be the first time the elite have set me chasing my own tail.
                    But if I catch you with even a whiff of trouble about you...
                player:
                    Yeah, yeah. Bluster bluster. 
                    I'm a good boy, I am. Honest.
                agent:
                    !exit
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    What's so suspicious about winning, huh? 
                agent:
                    It's not the winning, it's the facts. And they're stacked against you.
            ]],

            OPT_FIGHT = "Defend yourself",
            DIALOG_FIGHT = [[
                player:
                    !fight
                    Aw, shove it down your blowhole.
            ]],
            DIALOG_WIN = [[
                {dead?
                    * Well, that's one way to deal with your problems.
                }
                {not dead?
                    agent:
                        !injured
                        This isn't over, grifter. Pearl-on-the-Foam isn't big enough for you to make your own rules.
                    player:
                        Really? Tell that to literally everyone else.
                        !exit
                }
                * You dust yourself off and continue.
            ]]
        }
        :Fn(function(cxt)

            local patrol = CreateCombatParty("ADMIRALTY_PATROL", math.min(cxt.quest:GetRank(), 1), cxt.location)
            cxt.quest.param.has_had_post_confront = true
            cxt:TalkTo(patrol[1])
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function(cxt) 
                
                cxt:Opt("OPT_BRIBE")
                    :Dialog("DIALOG_BRIBE")
                    :DeliverMoney(50*cxt.quest:GetRank())
                    :Travel()

                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation()
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCE_SUCCESS")
                            :Travel()

                        :OnFailure()
                            :Dialog("DIALOG_CONVINCE_FAILURE")
                            :Travel()

                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
                        :OnWin()
                            :Dialog("DIALOG_WIN")
                                :Travel()


            end)
        end)


QDEF:AddConvo("return_to_moreef", "moreef")
    :Loc{
        OPT_GET_PAID = "Tell {moreef} the good news",
        DIALOG_GET_PAID = [[
            player:
                Hesh kissed you with its slippery tentacles, my friend, the day you decided to place your trust in me.
            agent:
                !neutral
                Oh, don't rub it in. I'm already out my savings. 
                I get it: my luck's shite.
            player:
                Huh? You <i>won</i>, furbutt. You get your savings back and then some.
            agent:
                W-wait, what? How?
                I thought you said that tip was a scam.
            player:
                !happy
                It was, you idiot. Lucky for you, I've got a history of flipping the odds.
            agent:
                !happy
                You rigged the race? For me?
            player:
                Well, I ain't got no one else to rig it for, do I?
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_PAID")
            :SetQuestMark()
            :Dialog("DIALOG_GET_PAID")
            :CompleteQuest()
            :Fn(function() 
                ConvoUtil.GiveQuestRewards(cxt)
            end)
    end)





local QDEF = QuestDef.Define{
        id = "SMITH_STORY_GET_RACING_SNAIL_LATER",
        qtype = QTYPE.STORY,
        is_in_hiding = function(quest, agent)
            return agent == quest:GetCastMember("oshnu")
        end,   
    }

    :AddCast{
        cast_id = "oshnu",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, quest:CreateSkinnedAgent("RACING_SNAIL"))
        end,
    }

    :AddDormancyState("wait", "follow_up", true, 4, 10, true)
    

    :AddObjective{
        id = "follow_up",
    }

    :AddConvo("follow_up")
        :ConfrontState("CONF", function(cxt) 
                                    return TheGame:GetGameState():CanSpawnTravelEvent() and 
                                            cxt.location:HasTag("in_transit")  and
                                            TheGame:GetGameState():GetCaravan():CanOwnPet()
                                end)
        :Loc{
            DIALOG_INTRO = [[
                * Your luck being what it is, you come across a handler attempting to wrestle an oshnu down the road.
                oshnu:
                    !left
                agent:
                    !right
                    !angry
                    Come <i>on</i>, you slimy scumnoodle!
                * The oshnu twists its rubbery neck and headbutts {agent} with a watery slap.
                oshnu:
                    !exit
                agent:
                    !injured
                    Ow!
                player:
                    !left
                agent:
                    !angry
                    What are <i>you</i> looking at?
            ]],

            OPT_ASK_ABOUT_SNAIL = "Ask about the oshnu",
            DIALOG_ASK_ABOUT_SNAIL = [[
                player:
                    Your buddy there doesn't seem to like you very much. 
                agent:
                    The feeling is mutual.
                    The racer's due for retirement, but {oshnu.heshe}'s putting up a fight.
                player:
                    Stupid of it. Bet retirement's much nicer than the stables.
                agent:
                    You thick? It's a racing oshnu. And I want my money's worth.
                    If I get this lurch to market, we can render it down. Fat goes into cosmetics, mucus turns into wax for <i>active</i> racing snails. 
                player:
                    !dubious
                    ...
                agent:
                    Don't look at me like that! If this oshnu wanted to live it shouldn't have gotten old. 

            ]],
            OPT_BUY_SNAIL = "Buy the oshnu",
            DIALOG_BUY_SNAIL = [[
                player:
                    You want your profits?
                    I'll pay to take {oshnu.himher} off your hands.
                    !give
                agent:
                    !take
                    No kidding? 
                    It's not as much as I'd have gotten at market, but it also frees up my time.
                    Have fun doing... whatever it is you plan on doing.
                    !exit
                * {oshnu} stares at you with moistened eyeballs.
                oshnu:
                    !right
                    Mep?
            ]],

            OPT_OFFER_TO_FINISH = "Offer to take {oshnu} to market for {agent}",
            DIALOG_OFFER_TO_FINISH = [[
                player:
                    Let me do you a favor. I'll take {oshnu} in.
            ]],
            DIALOG_OFFER_TO_FINISH_SUCCESS = [[
                player:
                    I got a way with oshnus anyway. And if I haggle you a better price, I'll keep the upsell as my take.
                agent:
                    Yeah? I suppose you come from one of those old families, huh? 
                    Probably had a stable full of them yourself growing up.
                    Well, have at it. Tell them to forward payment to my account. They'll know the one.
                    !exit
                * {oshnu} stares at you with moistened eyeballs.
                oshnu:
                    !right
                    Flurk.

            ]],
            DIALOG_OFFER_TO_FINISH_FAILURE = [[
                player:
                    I'll only take a small cut of the sale for my fee.
                agent:
                    Nice try, but the more this beast wrestles with me, the more mucus I'll have at the scale.
                    This might be a pain, but at least it's a profitable one.
            ]],
            OPT_LIBERATE_OSHNU = "Liberate the oshnu",
            DIALOG_LIBERATE_OSHNU = [[
                player:
                    You know what? I think there's something this oshnu and I have in common.
                    Neither of us like you much.
                    !fight
            ]],
            DIALOG_WON = [[
                {not dead?
                    player:
                        Whaddya make of that, huh?
                    agent:
                        !injured
                        You crazy?! All this for a lousy oshnu!
                        Keep the damn thing. I'm done with it, anyway.
                        !exit
                }
                {dead?
                    * {agent} lies dead in a puddle of mucus. 
                    player:
                        Well that's just gross.
                }
            ]],
            DIALOG_GOT_SNAIL_AFTER_BATTLE = [[
                * {oshnu} stares at you with moistened eyeballs.
                oshnu:
                    !right
                    Fnit. Fnit. Fnit.
            ]],
            DIALOG_SNAIL_DEAD = [[
                * {oshnu}'s shell is cracked and oozing. But since the beast is dead, the mucus is already starting to turn rancid.
            ]],
            DIALOG_LEAVE = [[
                player:
                    Well, you two kids have fun. 
                    !exit
            ]],

        }
        :Fn(function(cxt) 
            cxt.quest:Complete()
            
            local agent = AgentUtil.GetFreeAgent("OSHNU_WRANGLER")
            cxt:TalkTo(agent)

            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                cxt:Question("OPT_ASK_ABOUT_SNAIL", "DIALOG_ASK_ABOUT_SNAIL")
                
                cxt:Opt("OPT_BUY_SNAIL")
                    :Dialog("DIALOG_BUY_SNAIL")
                    :DeliverMoney(100)
                    :Fn(function() cxt.quest:GetCastMember("oshnu"):Recruit(PARTY_MEMBER_TYPE.CREW) end)
                    :Travel()

                cxt:Opt("OPT_OFFER_TO_FINISH")
                    :Dialog("DIALOG_OFFER_TO_FINISH")
                    :Negotiation{}
                        :OnSuccess()
                            :Dialog("DIALOG_OFFER_TO_FINISH_SUCCESS")
                            :Fn(function() cxt.quest:GetCastMember("oshnu"):Recruit(PARTY_MEMBER_TYPE.CREW) end)
                            :Travel()

                        :OnFailure()
                            :Dialog("DIALOG_OFFER_TO_FINISH_FAILURE")

                cxt:Opt("OPT_LIBERATE_OSHNU")
                    :Dialog("DIALOG_LIBERATE_OSHNU")
                    :Battle{
                        allies = {cxt.quest:GetCastMember("oshnu")},
                    }
                        :OnWin()
                            :Fn(function() 
                                if not cxt.quest:GetCastMember("oshnu"):IsDead() then
                                    cxt:Dialog("DIALOG_GOT_SNAIL_AFTER_BATTLE")
                                    cxt.quest:GetCastMember("oshnu"):Recruit(PARTY_MEMBER_TYPE.CREW)
                                else
                                    cxt:Dialog("DIALOG_SNAIL_DEAD")
                                end
                            end)
                            :Travel()

                StateGraphUtil.AddLeaveLocation(cxt)
                    :Dialog("DIALOG_LEAVE")
            end)

        end)
