local FOLLOW_UP

local QDEF = QuestDef.Define{
        title = "Sponsorship Deal",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{moreef} needs you to convince {target} to sponsor {moreef.hisher} bar.]],
        icon = engine.asset.Texture("icons/quests/side_smith_sponsorship_deal.tex"),
        focus = QUEST_FOCUS.NEGOTIATION,
        tags = {"MOREEF_INTRO_JOB"},
        act_filter = "SMITH",

        avoids = {
            {"target", "other"},
            {"moreef", "other"},
            {"moreef", "other"},
        },

        on_destroy = function(quest)
            if quest:GetCastMember("other") and quest:GetCastMember("other"):IsInPlayerParty() then
                quest:GetCastMember("other"):Dismiss()
            end
        end
    }

    :AddCastByAlias{
        cast_id = "moreef",
        alias = "SWEET_MOREEF",
        provider = true,
        no_validation = true,
    }

    --you don't know this person
    :AddCast{
        cast_id = "target",
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "WEALTHY_MERCHANT" ) )
        end,
    }

    :AddDefCast("other", "HEAVY_LABORER")

    :AddObjective{
        id = "find_target",
        title = "Find {target}",
        desc = [[You need to talk to {target} to renegotiate {target.hisher} sponsorship of {moreef}'s bar.]],
        mark = { "target" },
    }
    
    :AddObjective{
        id = "escort_other",
        title = "Take {other} to {target}",
        desc = [[You agreed to help {other} apologize to {target}.]],
        mark = { "target" },
    }

    :AddObjective{
        id = "deal_with_other",
        title = "(Optional) Deal with {other}",
        desc = [[Dissuade {other} from talking about {target}'s shady dealings.]],
        mark = { "other" },
    }

    :AddObjective{
        id = "return_to_target",
        title = "Return to {target}",
        desc = [[Tell {target} that {other} has been dealt with.]],
        mark = { "target" },
    }

    :AddObjective{
        id = "return_for_reward",
        title = "Tell {moreef} about the deal",
        desc = [[You negotiated a new deal with {target}. Tell {moreef} the good news.]],
        ready_for_completion = true,
        mark = { "moreef" },
        terminal = true,
    }

    :AddOpinionEvents{
        helped_them_apologize = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Helped them apologize for grifting",
        },
        forced_them_to_sponsor_moreef = 
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Forced them to sponsor Moreef's bar",
        },
        helped_them_with_an_extortionist  = 
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "Helped them sort out an extortionist",
        },
        wasted_their_time = 
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Wasted their time",

        }
    }



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                With all this hubbub, the Foam is up to its walls in Luminari. And my business blessings are out of date.
                My sponsor has always been a local merchant who gets blessings for cheap, but these days, getting right with the Heshians is a competitive business.
                Turns out I got out-bid.
            player:
                Can't you just, I dunno, scribble some proverb on the wall and call it a day?
            agent:
                A forgery? You kidding?
                That'd lose me my business <i>and</i> my hide. 
                I'd rather just pay the shills. If you can renegotiate with {target} for me, that is. 
        ]] 
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            player:
                Sure. How hard can it be?
            agent:
                So you say. 
                I ain't got much more to lose, so if you do this, it'll bring me back from the brink.
                If you don't... well, I'm mentally prepared.
        ]] 
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Activate("find_target")
        end)




QDEF:AddConvo("find_target", "target")
    :Loc{
        OPT_ASK_ABOUT_SPONSORSHIP = "Ask {target} about {target.hisher} sponsorship of {moreef}'s bar",
        DIALOG_ASK_ABOUT_SPONSORSHIP = [[
            {first_time?

                player:
                    Hey, you the brass buttons with the blessings for bounty?
                agent:
                    That's a rather unsavory way of describing it. 
                    My business is in good standing with the Cult, and as such I am a licensed blessings distributor. 
                player:
                    Well, one of them was for my buddy {moreef} at the Tropical Hideaway.
                    You've apparently been good to him before. So I'm gonna need you to rustle up the scripts to put the touch of Hesh on his bar.
                agent:
                    {moreef} sent you? Then he knows it's just business.
                    At the moment I can get a much better return from other businesses—ones that stand to make a tidy sum in these times.
                    Tell {moreef} that he needs to provide a better return on investment for his next sponsor.
            }
            
            {not first_time?
                player:
                    You know what, merchant? I can't leave my buddy in the lurch. I've got more to say.
                agent:
                    !sigh
                    I'm sure you do. 
            }

        ]],

        OPT_THREATEN_TO_BEAT_UP = "Threaten to beat up {agent}",
        DIALOG_THREATEN_TO_BEAT_UP = [[
            player:
                You sure you don't wanna reconsider? 'Cause if you can't sponsor Sweets, we'll need to move some assets around.
                And by "assets" I mean "the features of your face".
                !threaten
                'Cause I'm gonna rearrange it. Your face.
            agent:
                Cute. Perhaps you remember me saying I'm in good standing with the Cult.
                Suffice it to say... you'd regret even trying. 
                I'll tell you what. If you're so eager to use those fists, let me choose the target.
                If you provide that service for me, and do so discretely... then we can see about the Hideaway.
            player:   
                Huh. Say I do it. Who am I knocking down, and what for?
            agent:
                {other}. And for crossing me. It's as simple as that.
        ]],

        OPT_AGREE_TO_DEAL = "Agree to beat up {other}",
        DIALOG_AGREE_TO_DEAL = [[
            player:
                Huh. Sure, okay. I'm not picky.
            agent:  
                I suspected that was the case. 
                Find me again once you've completed the service. 
                Then we'll talk about blessings.
        ]],

        OPT_ATTACK = "Attack {target}",
       
        DIALOG_ATTACK = [[
            player:
                Man, but then I gotta go somewhere, and do a thing... you're right here.
                Seems easier to just cut out the middleman.
                !fight
            agent:  
                !surprise
                W-what?! 
                !fight 
                You'll be lucky to live to regret this!
        ]],

        OPT_CONVINCE_TARGET = "Convince {target} to give {moreef} another chance",
        DIALOG_CONVINCE_TARGET = [[
            player:
                I don't think you've considered the opportunity, here.
                The Hideaway has a rare chance to, uh, <i>distinguish</i> itself from the other bars.
            agent:
                Oh yes? And what would that look like?
        ]],
        DIALOG_CONVINCE_TARGET_SUCCESS = [[
            player:
                For one thing, it's waterfront. <i>Great</i> view of the Beaching.
                For another, it's got me.
            agent:
                And that's... a selling feature?
            player:
                I'm a Banquod. My parents are the ones that got did by the biomass.
            agent:
                Ah. So you make the Hideaway a holy destination unto itself?
                See the Great Beaching, have a drink, touch a Banquod...
            player:
                !placate
                Well, hang on, I dunno about letting everyone up in my—
            agent:
                Yes, I can see how that'd be profitable.
                Very well. Tell {moreef} I'll send along the blessings.
            player:
                Oh, well. Good. Yeah.
        ]],
        
        DIALOG_CONVINCE_TARGET_FAILURE = [[
            player:
                For one thing, it can cater to pilgrims and nobles alike. 
            agent:
                Theoretically, <i>any</i> bar could. But they don't.
                Because when this is over, the one that served the riffraff will have a bad reputation.
                I'd rather cut my losses and start a new relationship with the potential for longevity.
                The answer is no.
        ]],

        DIALOG_WON = [[
            {dead?
                * {agent} is dead, and can't procure blessings for anyone. {moreef} is probably going to be disappointed. Probably.
            }
            
            {not dead?
                player:
                    Well? You still got things to say or have we reached an agreement?
                agent:
                    !injured
                    B-blessings will be provided, if it spares my life.
                    But you're going to be dead within a day for running afoul of the Cult.
                player:
                    !happy
                    Enh, progress is progress. I'll take it!
            }
        ]],
        SIT_MOD = "{target} doesn't want to listen to you.",

    }
    :Hub(function(cxt)
            cxt:Opt("OPT_ASK_ABOUT_SPONSORSHIP")
                :SetQuestMark()
                :Dialog("DIALOG_ASK_ABOUT_SPONSORSHIP")
                :Fn(function() 

                    cxt:RunLoop(function() 

                        cxt:Opt("OPT_CONVINCE_TARGET")
                            :Dialog("DIALOG_CONVINCE_TARGET")
                            :Negotiation{
                                situation_modifiers =
                                {
                                    { value = 10, text = cxt:GetLocString("SIT_MOD") }
                                },

                            }
                                :OnSuccess()
                                    :Dialog("DIALOG_CONVINCE_TARGET_SUCCESS")
                                    :Fn(function() 
                                            cxt.quest:Complete("find_target")
                                            cxt.quest:Activate("return_for_reward")
                                        end)
                                    :DoneConvo()
                                    
                                :OnFailure()
                                    :Dialog("DIALOG_CONVINCE_TARGET_FAILURE")

                        if not cxt.quest.param.did_threaten then
                            cxt:Opt("OPT_THREATEN_TO_BEAT_UP")
                                :IsHubOption(true)
                                :Dialog("DIALOG_THREATEN_TO_BEAT_UP")
                                :Fn(function() cxt.quest.param.did_threaten = true end)
                        else

                            cxt:Opt("OPT_ATTACK")
                                :Dialog("DIALOG_ATTACK")
                                :Fn(function() cxt.quest.param.attacked_sponsor = true end)
                                :Battle{}
                                    :OnWin()
                                        :Fn(function() 
                                            cxt.quest:Complete("find_target")
                                            cxt.quest:Activate("return_for_reward")
                                            cxt:Dialog("DIALOG_WON")
                                            if cxt:GetAgent():IsAlive() then
                                                cxt:Opinion("forced_them_to_sponsor_moreef")
                                            end
                                        end)
                                        :Travel()

                            if not cxt.quest.param.agreed_to_deal then
                                cxt:Opt("OPT_AGREE_TO_DEAL")
                                    :Dialog("DIALOG_AGREE_TO_DEAL")
                                    :Fn(function()
                                            cxt.quest.param.agreed_to_deal = true
                                            cxt.quest:Activate("deal_with_other")
                                        end)
                                    :DoneConvo()
                            end
                        end

                        StateGraphUtil.AddBackButton(cxt)
                    end)

                    

                end)
    end)




QDEF:AddConvo("deal_with_other", "other")
    :Loc{
        OPT_ASK_ABOUT_TARGET = "Ask about {target}",
        DIALOG_ASK_ABOUT_TARGET = [[
            player:
                Hey, you {other}? Not to hit you with bad news but, uh...
                {target} asked me to beat you up. Or kill you.
                !thought
                I actually forgot to clarify which one.
            agent:
                W-what?!
                I figured it was a reasonable business proposal, but this is {target.hisher} response?
            player:
                !shrug
                Dunno, don't much care... I'm just tellin' you in case you have a better offer.
                You know, before I commence with the pummelling of unknown intensity.
            agent:
                Wait. Wait. 
                How about you just take me back to {target}? I can apologize for, ah, any offense.
                Offer better terms, if {target.heshe} wants. 
                We can skip the beating part, and everybody wins!
        ]],

        OPT_AGREE_TO_DEAL = "Accept the offer",
        DIALOG_AGREE_TO_DEAL = [[
            player:
                Yeah, I suppose that works. Plus then I can clarify whether I was supposed to beat you dead or not.
            agent:
                G-great. Yeah. Good for everyone.
        ]],

        OPT_INTIMIDATE = "Put the fright to {agent}",
        DIALOG_INTIMIDATE = [[
            player:
                I'm a charitable guy, so how about this:
                You stop making trouble for {target}, and I won't turn you into a barely-living pretzel.
        ]],
        DIALOG_INTIMIDATE_SUCCESS = [[
            player:
                Pretty sure I could get your head literally halfway up your own ass before I'd have to break a second rib.
                You wanna find out?
            agent:
                !scared
                I-it was a dumb idea, I get it! 
                {target} won't hear from me again, okay?
            player:
                Good.

        ]],
        DIALOG_INTIMIDATE_FAILURE = [[
            player:
                You gotta weigh the risks, here. And the risks are high.
            agent:
                So are the gains. If {target} doesn't want to work with me, then that just means I keep all the profits.
                You can't stop an entrepeneur like myself with petty threats.
        ]],

        OPT_ATTACK = "Beat up {agent}",
        DIALOG_ATTACK = [[
            player:
                !fight
                This is all too complicated for a simple reprobate like me.
        ]],
        DIALOG_ON_WIN = [[
            {dead? 
                * Hopefully, this was the result {target} wanted. Next time you should probably clarify beforehand.
            }
            {not dead?
                agent:
                    S-stop, I yield!
                    I'll drop it with {target}. {target.HeShe} will never hear from me again.
                player:
                    !happy
                    Good. Pretty sure this is the result I was supposed to aim for. 
                    But, uh, where will you be if it turns out I need to come back and kill you?
                agent:
                    Uh... h-here?
                player:
                    Great. Good to have my bases covered.
            }
        ]],
        OPT_ASK_ABOUT_PROBLEM = "Ask about the business proposal",
        DIALOG_ASK_ABOUT_PROBLEM = [[
            player:
                What kind of business proposal provokes this kind of reaction?
            agent:
                It's very straightforward. {target} sells and distributes Heshian blessings to businesses that want to be in good standing with the Cult.
                But with my help, we could expand to offer other certifications. Even more blessings.
            player:
                More blessings...
                Wait, you mean counterfeits?
            agent:
                Just slightly refurbished. Blessings with last year's date rubbed off and a new one put in.
            player:
                I can see why that kind of proposal might get under {target}'s skin.
        ]],
    }
    :Hub(function(cxt)
        
        cxt:Opt("OPT_ASK_ABOUT_TARGET")
            :SetQuestMark()
            :Dialog("DIALOG_ASK_ABOUT_TARGET")
            :Fn(function() 

                cxt:RunLoop(function() 

                    cxt:Question("OPT_ASK_ABOUT_PROBLEM", "DIALOG_ASK_ABOUT_PROBLEM")

                    cxt:Opt("OPT_AGREE_TO_DEAL")
                        :Dialog("DIALOG_AGREE_TO_DEAL")
                        :Fn(function() 
                            cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            cxt.quest:Complete("deal_with_other")
                            cxt.quest:Complete("find_target")
                            cxt.quest:Activate("escort_other")
                        end)
                        :Travel()
                    
                    cxt:Opt("OPT_INTIMIDATE")
                        :Dialog("DIALOG_INTIMIDATE")
                        :Negotiation{
                            flags = NEGOTIATION_FLAGS.INTIMIDATION
                        }
                            :OnSuccess()
                                :Dialog("DIALOG_INTIMIDATE_SUCCESS")
                                :Fn(function() 
                                    cxt.quest:Complete("find_target")
                                    cxt.quest:Complete("deal_with_other")
                                    cxt.quest:Activate("return_to_target")
                                    cxt:End()
                                end)
                            :OnFailure()
                                :Dialog("DIALOG_INTIMIDATE_FAILURE")

                    cxt:Opt("OPT_ATTACK")
                        :Dialog("DIALOG_ATTACK")
                        :Battle{}
                            :OnWin()
                                :Dialog("DIALOG_ON_WIN")
                                :Fn(function() 
                                    cxt.quest.param.other_dead = cxt:GetAgent():IsDead()
                                    cxt.quest:Complete("find_target")
                                    cxt.quest:Complete("deal_with_other")
                                    cxt.quest:Activate("return_to_target")
                                    cxt:End()
                                end)
                                :DoneConvo()
                    end)
            end)
    end)



QDEF:AddConvo("return_to_target", "target")
    :Loc{
        OPT_TELL_ABOUT_OTHER = "Tell {agent} about {other}",
        DIALOG_TELL_ABOUT_OTHER = [[
            player:
                Hey, that thing you wanted done? I did it. 
                I'm reliable like that.
            {other_dead?
                agent:
                    I heard. A bit over the top, don't you think?
                player:
                    !shrug
                    Aw, Hesh. So you didn't want {other.himher} dead?
                    You should really clarify when you're sending out muscle like me.
                agent:
                    Thank you... for that feedback.
                    As per our agreement, I'll send the appropriate blessings to the Hideaway.
                    But if the returns aren't up to snuff...
                player:
                    ...
                agent:
                    ...
                player:
                    Oh, is this a threat? You'll do to me and Sweets what I did to {other}...
                    I get it. 
                agent:
                    Glad to hear it. 
            }
            {not other_dead?
                agent:
                    And you can be confident that it's truly taken care of?
                    I don't want to just have to send another grifter later if your beating didn't leave an impression.
                player:
                    Naw, it's one-and-done. A deal's a deal.
                agent:
                    So it is. I'll give {moreef} another chance and send along the appropriate blessings.
                    But hopefully this serves as a warning for how I deal with disappointing business arrangements.
                player:
                    Hey, if there's a chance to make a quick buck, you can bet I'll find it.
                    The Hideaway's in good hands.
                agent:
                    Hm.
            }
        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_TELL_ABOUT_OTHER")
            :ReceiveOpinion("helped_them_with_an_extortionist")
            :Dialog("DIALOG_TELL_ABOUT_OTHER")
            :CompleteQuest("return_to_target")
            :ActivateQuest("return_for_reward")
    end)


QDEF:AddConvo("escort_other", "target")
    :AttractState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !angry
                    Excuse you? I believe I told you to teach {other} a lesson—not give {other.himher} a platform.
                other:
                    !left
                    !placate
                    Well, first, I realized you were a little light on the details for what that lesson should look like—
                agent:
                    I don't care what it looks like. Just do it!
                player:
                    !left
            ]],
            OPT_ATTACK_OTHER = "Attack {other}",
            DIALOG_ATTACK_OTHER = [[
                other:
                    !right   
                    !scared             
                player:
                    !fight
                    Glad we got that clarification, huh?
            ]],
            DIALOG_WON = [[
                target:
                    !right
                { other_dead ?
                    target:
                        So why, exactly, did you feel the need to bring me this mess?
                    player:
                        Well—
                    target:
                        Nevermind. I'll send blessings to {moreef} because that was the deal...
                        But tell him to never send <i>you</i> in his place again.
                    player:
                        Huh. I guess this is more or less exactly how I got out of ever doing chores as a kid.
                }
                {not other_dead ?
                    target:
                        Enough. {other} has learned {other.hisher} lesson.
                    player:
                        Sure. Sorry about bringing {other.himher} back here, I just wasn't sure—
                        Well, next time, I'll know you like 'em alive.
                    target:
                        There won't be a <i>next time</i>. 
                        I'll give {moreef} another chance, but you? Get out of my face, and pray we don't have business again.
                }
            ]],

            OPT_NEGOTIATE = "Convince {target} to show mercy",
            DIALOG_NEGOTIATE = [[
                player:
                    Look at that face. How could I say no?
                    Why not hear what {other.heshe} has to say, huh? What harm will it do?
            ]],
            DIALOG_NEGOTIATE_SUCCESS = [[
                player:
                    Fact is, none of us know how long the Beaching will last. 
                    So you need to keep your options open, right? Be up for pursuing other avenues.
                    Having a fall guy like {other} to bite the consequences seems like as good a plan as any.
                agent:
                    Hm. You're not wrong. There is a new market that needs serving. 
                    There are risks, but... aren't there always?
                    !sigh
                    Very well. I'll send the blessings along to {moreef}. 
                    {other} and I have things to discuss, so please see yourself out.
            ]],
            DIALOG_NEGOTIATE_FAILURE = [[
                player:
                    Seems if you go into business with {other} then you can control the operation.
                    Rather than let it run free like a sticky toddler.
                agent:
                    My plan was to control the operation by eliminating {other} entirely.
                    And since I can't trust you to play the part I cast you for, then kindly exit stage left. 
                player:
                    You say "kindly" but, you know, that tone of yours—
                agent:
                    Get. Out.
            ]],
            OPT_LEAVE_ALL = "Leave {other} and {target} to work it out",

            DIALOG_LEAVE_ALL = [[
                player:
                    Well, you two seem like you got this.
                    !overthere
                    I'll, uh, just...
                    !exit
                * You leave while {target} is too distracted to hold you accountable.
            ]],

            DIALOG_TALK_TO_OTHER = [[
                agent:  
                    !right
                    Thanks, {player}. I'm alive, healthy, and bound to turn a profit.
                player:
                    That mean I get a cut?
                agent:
                    Uh, not <i>contractually</i>, but...
                player:
                    Hah! I'm kidding. Sort of. 
                    Mostly. 
                    Unless you do end up making lots of money and then...
                    Well, we'll see.
                    !exit
            ]]

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_ATTACK_OTHER")
                :Dialog("DIALOG_ATTACK_OTHER")
                :Battle{
                    noncombatants = {cxt.quest:GetCastMember("target")},
                    enemies = {cxt.quest:GetCastMember("other")},
                }
                    :OnWin()
                        :Dialog("DIALOG_WON")
                        :CompleteQuest("escort_other")
                        :ActivateQuest("return_for_reward")
                        :Travel()

            cxt:Opt("OPT_NEGOTIATE")
                :Dialog("DIALOG_NEGOTIATE")
                :Negotiation{
                    helpers = {cxt.quest:GetCastMember("other")},
                }
                    :OnSuccess()
                        :Dialog("DIALOG_NEGOTIATE_SUCCESS")
                        :ReceiveOpinion("helped_them_with_an_extortionist")
                        :CompleteQuest("escort_other")
                        :ActivateQuest("return_for_reward")
                        :Fn(function() 
                            cxt:TalkTo("other")
                            cxt:Dialog("DIALOG_TALK_TO_OTHER")
                            cxt:GetCastMember("other"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("helped_them_apologize"))
                            cxt:GetCastMember("other"):Dismiss()
                            StateGraphUtil.AddEndOption(cxt)
                        end)

                    :OnFailure()
                        :ReceiveOpinion("wasted_their_time")
                        :Dialog("DIALOG_NEGOTIATE_FAILURE")
                        :FailQuest()
                        :Travel()

            cxt:Opt("OPT_LEAVE_ALL")
                :Dialog("DIALOG_LEAVE_ALL")
                :FailQuest()
                :Travel()
        end)


QDEF:AddConvo("return_for_reward", "moreef")
    :Loc{
        
        OPT_GET_PAID = "Get paid",
        DIALOG_PAID = [[
            player:
                !permit
                $neutralDirect
                Well, bud, it took all my cunning but... I got you.
                I heap blessings upon you. Blessings up the wazoo.
            {attacked_sponsor?
                agent:
                    I heard! You <i>attacked</i> {target}?
                    Is violence seriously the only way you know how to engage with the world?
                player:
                    Well, not the <i>only</i> way. But it's been pretty effective.
            }
            {not attacked_sponsor?
                agent:
                    I have them right here. {target} likes efficiency.
                    And they're legitimate and everything.
                player:
                    Well, yeah. Why wouldn't they be?
                agent:
                    You didn't use to--
                    I mean, I just--
                    Nothing. I'm glad it's done. 
            }

        ]],
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_GET_PAID")
            :PreIcon( global_images.hire )
            :SetQuestMark()
            :Dialog("DIALOG_PAID")
            :Fn(function() 
            
                if cxt.quest:GetCastMember("target"):IsAlive() then
                    cxt.quest:SpawnFollowQuest(FOLLOW_UP.id)
                end

                ConvoUtil.GiveQuestRewards(cxt)
            end)
            :CompleteQuest()

    end)






------THIS IS A FOLLOW-UP that takes place a couple of phases later.


FOLLOW_UP = QDEF:AddFollowup()

FOLLOW_UP.on_init = function(quest)
    quest.param.timeout = TheGame:GetGameState():GetDateTime() + 1 + math.random(2)
    quest:UnassignCastMember("other")
end

FOLLOW_UP.on_post_load = function(quest)
    local target = quest:GetCastMember( "target" )
    if target == nil or target:IsRetired() then
        quest:Cancel()
    end
end


for _, cast_def in pairs(FOLLOW_UP.cast) do
    if cast_def.cast_id == "target" then
        
        cast_def.events = cast_def.events or {}
        cast_def.events.agent_retired = function(quest, agent) 
            print ("AGENT DIED", agent)
            if agent == quest:GetCastMember("target") then
                if quest:IsActive() then
                    quest:Cancel()
                end
            end
        end    
        break
    end
end



FOLLOW_UP
    :AddObjective{
        id = "wait",
        state = QSTATUS.ACTIVE,
        events =
        {
            phase_change = function(quest)
                
                if not quest:GetCastMember("target"):IsAlive() then
                    quest:Cancel()
                end

                if TheGame:GetGameState():GetDateTime() >= quest.param.timeout then
                    quest:Complete('wait')
                    quest:Activate((quest.param.attacked_sponsor or quest:GetCastMember("target"):GetRelationship() == RELATIONSHIP.HATED) and 'followup_revenge' or 'followup_friendly' )
                end
            end,
            agent_retired = function( quest, agent ) 
                if agent == quest:GetCastMember("target") then
                    quest:Cancel()
                end
            end,
           
        },
    }

    :AddObjective{
        id = "followup_friendly",
        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("target"), location = quest:GetCastMember('home'), role = CHARACTER_ROLES.VISITOR } )
        end,
    }

    :AddObjective{
        id = "followup_revenge",
    }

    :AddOpinionEvents{
        good_business = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Gave them good business advice",
        },
        apologized = 
        {
            delta = OPINION_DELTAS.NEUTRALIZING,
            txt = "Apologized for misdeeds"
        }
    }

FOLLOW_UP:AddConvo("wait", "target")
        :Priority(CONVO_PRIORITY_LOW)
        :AttractState("STATE")
            :Loc{
                DIALOG_INTRO = [[
                    agent:
                        {attacked_sponsor?
                            !angry
                            Still breathing, grifter? You'll get yours soon enough.
                        }
                        {not attacked_sponsor?
                            Profits aren't much so far, {player}. But I'll be patient.
                        }
                ]]
            }
            :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
            end)



FOLLOW_UP:AddConvo("followup_friendly", "target")
        :Priority(CONVO_PRIORITY_LOW)
        :ConfrontState("STATE", function(cxt) return cxt.quest:GetCastMember("target"):IsAlive() end)
            :Loc{
                DIALOG_INTRO = [[
                    agent:
                        {player}. I have something for you.
                    player:
                        Is that so? 
                        Or—am I supposed to refuse it first? I can never remember which classes get which customs.
                    agent:
                        We're business partners, so you take what's yours.
                        You were right about {moreef}'s bar. Profits have been... unexpected.
                        I feel like you deserve a cut.
                    player:
                        Hey, that's what I'm talkin' about!
                        You grease my back, I grease yours. 
                        And together we slide our way into fortune, yeah?
                    agent:
                        I... suppose?
                        !give

                ]],
                OPT_TAKE_MONEY = "Take the money",

            }
            :Fn(function(cxt) 
                
                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_TAKE_MONEY")
                    :ReceiveMoney(100)
                    :ReceiveOpinion("good_business")
                    :CompleteQuest()
            end)

FOLLOW_UP:AddConvo("followup_revenge", nil)
        :TravelConfront("MEET_PATROL", function(cxt) return cxt.quest:GetCastMember("target"):IsAlive() end)
            :Loc{
                DIALOG_INTRO = [[
                    * You're pulled up short by {target}.
                    player:
                        !left
                    target:
                        !right
                        I told you, {player}, that this would end poorly for you.
                        The Cult didn't take kindly to tales of your antics. 
                        Yield, and you might only be sent to the derricks for a year or two.
                ]],
                OPT_DEFEND = "Defend yourself!",
                DIALOG_DEFEND = [[
                    player:
                        Naw. I never was big on boats.
                        Now you gonna fight or do you gotta bloviate a bit more first?
                        !fight
                ]],
                DIALOG_WON = [[
                    {dead?
                        * {target} is dead. You hope that doesn't cause trouble for {moreef}.
                    }
                    {not dead?
                        target:
                            !injured
                            Next time, grifter, you won't be so lucky.
                            !exit
                        player:
                            !angry
                            Yeah?
                            Well...
                            You too, buddy!
                            !happy
                        * You sure showed {target.himher}.
                    }
                ]],

                OPT_APOLOGISE = "Apologize",
                DIALOG_APOLOGISE = [[
                    player:
                        Hey, look. It was a mistake, alright?
                ]],
                DIALOG_APOLOGISE_SUCCESS = [[
                    player:
                        I came to the Foam with no friends but {moreef}. 
                        Bad calls got made. Mostly by me.
                        But that don't mean you gotta keep the course. 
                        My fortune's on the up. You let bygones be, and you stand a chance to reap that someday.
                    agent:
                        Everyone knows you've been disowned, Banquod.
                        But it's clear your fortunes <i>are</i> on the mend.
                        These vague promises of yours earn you time, at least. But I'll be watching.
                        !exit
                ]],
                DIALOG_APOLOGISE_FAILURE = [[
                    player:
                        It won't happen again. Just get over it!
                    agent:
                        I'm sure I'll have all the closure I could ever need.
                        Once you're dead, of course.
                ]],

            }
            :Fn(function(cxt) 
                    cxt:TalkTo("target")
                    cxt.quest:Complete()
                    CreateCombatBackup(cxt:GetAgent(), "MERCENARY_BACKUP", cxt.quest:GetRank())
                    cxt:Dialog("DIALOG_INTRO")

                    cxt:RunLoop(function(cxt) 
                        cxt:Opt("OPT_APOLOGISE")
                            :Dialog("DIALOG_APOLOGISE")
                            :Negotiation{}
                                :OnSuccess()
                                    :ReceiveOpinion("apologized")
                                    :Dialog("DIALOG_APOLOGISE_SUCCESS")
                                    :Travel()

                                :OnFailure()
                                    :Dialog("DIALOG_APOLOGISE_FAILURE")


                        cxt:Opt("OPT_DEFEND")
                            :Battle{flags = BATTLE_FLAGS.SELF_DEFENCE}
                                :OnWin()
                                    :Dialog("DIALOG_WON")
                                    :Travel()
                    end)
                end)

