local function CheckForDeath(battle)
    local team = battle:GetScenario():GetTeam(TEAM.RED)
    for k,v in ipairs(team) do
        if v:IsSentient() and v:IsDead() then
            return true
        end
    end
    team = battle:GetScenario():GetTeam(TEAM.BLUE)
    for k,v in ipairs(team) do
        if v:IsSentient() and v:IsDead() then
            return true
        end
    end
    return false
end


local QDEF = QuestDef.Define{
        title = "Three is a Party",
        qtype = QTYPE.SIDE,
        rank = {1, 5},
        desc = [[{moreef} needs you to drum up some business for {moreef.hisher} bar.]],
        icon = engine.asset.Texture("icons/quests/side_smith_three_is_a_party.tex"),
        focus = QUEST_FOCUS.NEGOTIATION,
        tags = {"MOREEF_INTRO_JOB"},
        act_filter = "SMITH",
        on_init = function(quest) 
            quest.param.party_people = {}
        end,

        on_debug_start = function(quest)
            TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("MOREEF_BAR"))
            UIHelpers.DoSpecificConvo( quest:GetCastMember("moreef"), "DEBUG_SIDE_JOB", "STATE_RUN_JOB", nil, quest )
        end,

        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("wrangler") or
                fighter.agent == quest:GetCastAgent("merchant") or
                fighter.agent == quest:GetCastAgent("luminitiate") or
                fighter.agent == quest:GetCastAgent("admiralty") then

                    fighter:AddCondition("WANTED_ALIVE")
            end
        end,


    }

    :AddCastByAlias{
        cast_id = "moreef",
        alias = "SWEET_MOREEF",
        provider = true,
        no_validation = true,
    }

    :AddDefCastSpawn("wrangler", "OSHNU_WRANGLER")
    :AddDefCastSpawn("luminitiate", "LUMINITIATE")
    :AddDefCastSpawn("admiralty", "ADMIRALTY_GOON")
    :AddDefCastSpawn("merchant", "WEALTHY_MERCHANT")


    :AddExistingLocation("station", "PEARL_ADMIRALTY_STATION")
    :AddExistingLocation("fancy_dinner", "PEARL_FANCY_EATS")
    :AddQuestLocation{
        cast_id = "snail_pen",
        name = "Oshnu Pen",
        plax = "INT_SMITH_SNAILPEN",
        map_tags = {"building"},
        indoors = true,
    }
    :AddQuestLocation{
        cast_id = "street_corner",
        name = "Busy Streetcorner",
        plax = "EXT_SMITH_Nightlife_Street",
        map_tags = {"street"},
        indoors = false,
    }


    :AddObjective{
        id = "collect_three",
        hide_in_overlay = true,
        on_activate = function(quest)
            quest:Activate("get_wrangler")
            quest:Activate("get_admiralty")
            quest:Activate("get_merchant")
            quest:Activate("get_luminitiate")
        end,
        events = 
        {
            quests_changed = function(quest, event_quest) 
                if event_quest == quest then
                    local num_complete = (quest:IsComplete("get_wrangler") and 1 or 0) +
                                            (quest:IsComplete("get_luminitiate") and 1 or 0) +
                                            (quest:IsComplete("get_merchant") and 1 or 0) +
                                            (quest:IsComplete("get_admiralty") and 1 or 0)
                    local num_failed = (quest:IsFailed("get_wrangler") and 1 or 0) +
                                            (quest:IsFailed("get_luminitiate") and 1 or 0) +
                                            (quest:IsFailed("get_merchant") and 1 or 0) +
                                            (quest:IsFailed("get_admiralty") and 1 or 0)

                    if num_complete >= 3 then
                        quest:Complete("collect_three")
                        quest:Activate("return_to_moreef_party")
                    elseif num_failed >= 2 then
                        quest:Cancel("collect_three")
                        quest:Activate("return_to_moreef_no_party")
                    end

                end
            end,
        },
    }
    
    :AddObjective{
        id = "return_to_moreef_party",
        title = "See how the party is going",
        desc = "{moreef} needs you to chaperone the party and make sure nobody breaks the bar.",
        mark = {"moreef"},

        on_activate = function(quest)
            if quest:IsActive("get_wrangler") then
                quest:Cancel("get_wrangler")
            end
            if quest:IsActive("get_merchant") then
                quest:Cancel("get_merchant")
            end
            if quest:IsActive("get_luminitiate") then
                quest:Cancel("get_luminitiate")
            end
            if quest:IsActive("get_admiralty") then
                quest:Cancel("get_admiralty")
            end

        end
    }
    
    :AddObjective{
        id = "return_to_moreef_no_party",
        title = "Tell {moreef} the party is cancelled",
        mark = {"moreef"},
        
        on_activate = function(quest)
            if quest:IsActive("get_wrangler") then
                quest:Cancel("get_wrangler")
            end

            if quest:IsActive("get_merchant") then
                quest:Cancel("get_merchant")
            end
            if quest:IsActive("get_luminitiate") then
                quest:Cancel("get_luminitiate")
            end
            if quest:IsActive("get_admiralty") then
                quest:Cancel("get_admiralty")
            end

        end
    }

    :AddObjective{
        id = "get_wrangler",
        title = "Invite {wrangler}",
        desc = "{wrangler} is an oshnu wrangler, and could use a break.",
        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("wrangler"), location = quest:GetCastMember("snail_pen")} )
        end,
        mark = {"wrangler"},
        on_complete = function(quest) 
            table.insert(quest.param.party_people, quest:GetCastMember("wrangler"))
            quest:GetCastMember("wrangler"):MoveToLimbo()
        end
    }

    :AddObjective{
        id = "get_luminitiate",
        title = "Invite {luminitiate}",
        desc = "{luminitiate} is part of the Cult, and has an air of respectability.",
        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("luminitiate"), location = quest:GetCastMember("street_corner")} )
        end,
        mark = {"luminitiate"},
        on_complete = function(quest) 
            table.insert(quest.param.party_people, quest:GetCastMember("luminitiate"))
            quest:GetCastMember("luminitiate"):MoveToLimbo()
        end
    }

    :AddObjective{
        id = "get_merchant",
        title = "Invite {merchant}",
        desc = "{merchant} drives a hard bargain, but parties harder.",
        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("merchant"), location = quest:GetCastMember("fancy_dinner")} )
        end,
        mark = {"merchant"},
        on_complete = function(quest) 
            table.insert(quest.param.party_people, quest:GetCastMember("merchant"))
            quest:GetCastMember("merchant"):MoveToLimbo()
        end
    }

    :AddObjective{
        id = "get_admiralty",
        title = "Invite {admiralty}",
        desc = "{admiralty} may be part of the authorities, but that doesn't mean {admiralty.heshe} doesn't know a good time.",
        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("admiralty"), location = quest:GetCastMember("station")} )
        end,
        mark = {"admiralty"},
        on_complete = function(quest) 
            table.insert(quest.param.party_people, quest:GetCastMember("admiralty"))
            quest:GetCastMember("admiralty"):MoveToLimbo()
        end
    }


    :AddOpinionEvents{
        helped_with_paperwork = 
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "Helped them finish their paperwork",
        },
        killed_snail = 
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Killed their Oshnu!",
        },
        sealed_deal = 
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "Sealed their tricky business deal",
        },
        good_party = 
        {
            delta = OPINION_DELTAS.LIKE,
            txt = "Invited them to a good party",
        },
        bad_party = 
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Invited them to a bad party",
        }
    }

QDEF:AddIntro(
    --attract spiel
    [[
        moreef:
            !shrug
            I figure, maybe I'm not honoring the Beaching enough.
            It's a pretty big deal, after all. Having a holy visitation from the Abyss.
        player:
            Yeah, I mean, I've trashed parties for less.
            !thought
            Like that one you had way back when! What was it?
        moreef:
            M-my thirteenth birthday?
        player:
            !chuckle
            Good times.
        moreef:
            Anyway, if I can reach out to some of my best customers, let them know it's an exclusive event.
            I'll be trying out new cocktails. Pretty sure I got a recipe book around here somewhere I can dust off.
        player:
            Hey, folks never turn down a free drink.
        moreef:
            !angry_shrug
            What? No. No free drinks! 
            The whole point is to pay my bills.
    ]],
    
    --on accept
    [[
        agent:
            If even just three customers show up, I can keep the lights on. 
            You want to help? Then deliver the invites and come back here to cover the door.
        player:
            !thumb
            You got it. I'm nothing if not persuasive.
        agent:
            ... 
            !sigh
            Yeah, well. Don't rough anyone up too bad, or it won't be much help to business.
    ]],
    "collect_three")


QDEF:AddConvo("get_admiralty", "admiralty")
    :AttractState("ATTRACT")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * You find {admiralty} on the job, looking like {admiralty.heshe} could use a break.
                    player:
                        !left
                        !greeting
                        Hey, officer. 
                    agent:
                        !right
                        Can I help you, citizen?
                    player:
                        Hear you're a regular at the Hideaway. 
                        I'm here to invite you to a VIP shindig. Only VIPs allowed.
                        That's you. You're a VIP.
                    agent:
                        I'm flattered. But my status at the Hideaway doesn't turn into a promotion in the ranks.
                        In fact, I've got enough VIP of my own over here: very important paperwork.
                        !chuckle
                    * {admiralty.HeShe} chuckles at {admiralty.hisher} own joke, but the pile of paperwork doesn't find it amusing.
                }
                {not first_time?
                    player:
                        You ready to party yet?
                    agent:
                        My paperwork hasn't gone anywhere, and so neither have I.
                }
            ]],

            OPT_CONVINCE_SKIP = "Convince {agent} to procrastinate",
            DIALOG_CONVINCE_SKIP = [[
                player:
                    Last I checked, papers aren't alive.
                    And you? You need to be living.
            ]],
            DIALOG_CONVINCE_SKIP_SUCCESS = [[
                player:
                    Here's how I figure: no way you can finish all that paperwork today, right?
                    So there's no point even getting started! 
                    Take a break, relax, clear your head with a decent amount of booze.
                agent:
                    Hrm. That's pretty compelling.
                    I'm sure these numbers will all be a little more accessible after a night of R & R, too.
                    See you at the Hideaway, citizen!
                    !exit
                * {agent} heads off to get ready, leaving you alone with the paperwork.
                player:
                    ...
                    Don't look at me like that.
            ]],

            DIALOG_CONVINCE_SKIP_FAILURE = [[
                player: 
                    Those papers aren't going anywhere, right?
                agent:
                    Correct. And they'll continue to go nowhere as long as I don't do them.
                    It's unfortunate, but please pass on my apologies to {moreef}.
                    I wish I could, but I can't.
            ]],

            OPT_HELP_WITH_PAPERWORK = "Offer to help with the paperwork",
            DIALOG_HELP_WITH_PAPERWORK = [[
                player:
                    You just need one of those whatchamacallits. 
                    A clerk!
                agent:
                    Hesh knows I've asked for one.
                player:
                    Well, Hesh is answering your prayers. 'Cause today? I'm your guy.
                agent:
                    Is that right? Hrm. I suppose these forms are simple enough. 
                    Come with me, then, and I'll set you up in an empty corner. 
                    !exit
                player:
                    !exit
                * {agent} clears a space and leaves you with half the stack of documents.
                * Three sheets in, your eyes start to cross.
            ]],
            DIALOG_HELP_WITH_PAPERWORK_2 = [[
                * Hours later, you slap down the last sheet. Your eyelids feel like they've been fixed with gum.
                player:
                    !injured
                    !left
                    It-it's done. 
                    That's the most work I've ever done. Can't believe you do that every day.
                agent:
                    !happy
                    !right
                    Yes, it's pretty grueling. But with your help we were done in under an hour!
                player:
                    What? That can't be right.
                agent:
                    Thanks again, {player}. Let's spill drinks together at the Hideaway later!
                    !exit
                * {agent} leaves to get ready.

            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:BasicNegotiation("CONVINCE_SKIP")
                :OnSuccess()
                    :CompleteQuest("get_admiralty")
                    :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                    :DoneConvo()
                :OnFailure()
                    :FailQuest("get_admiralty")

            cxt:Opt("OPT_HELP_WITH_PAPERWORK")
                :Dialog("DIALOG_HELP_WITH_PAPERWORK")
                :DeltaResolve(-15)
                :Dialog("DIALOG_HELP_WITH_PAPERWORK_2")
                :ReceiveOpinion("helped_with_paperwork")
                :CompleteQuest("get_admiralty")
                :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                :DoneConvo()

        end)


QDEF:AddConvo("get_wrangler", "wrangler")
    :ConfrontState()
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * You find {wrangler} dealing with an oshnu. Or, rather, it seems the oshnu is the one dealing with {wrangler}.
                    snail:
                        !left
                    wrangler:
                        !right
                        !angry
                        Bad snail! Bad!
                    player:
                        !left
                        Somebody call my name?
                    wrangler:
                        What? No.
                        It's this damned oshnu. It won't move out of its pen so that I can muck it out.
                    player:
                        !happy
                        Well, I've got something to ease your stress. There's an <i>exclusive</i> party at the Hideaway, and you're invited!
                        You gotta leave the oshnu here, though.
                    wrangler:
                        Hesh, I'd love to. But I've kinda got my hands full.
                }
                {not first_time?
                    snail:
                        !left
                    wrangler:
                        !right
                        Please! Just get out of the way!
                    player:
                        !left
                        Still dealing with the oshnu, eh?
                    wrangler:
                        !sigh
                        Always, and forever.
                        
                }
            ]],

            OPT_FIGHT_OSHNU = "Offer to teach {snail} a lesson",
            DIALOG_FIGHT_OSHNU = [[
                player:
                    Oshnus like that only understand one thing: brute force!
                    Want me to show that beast who's boss?
                wrangler:
                    Hey, you wanna give it a shot? Knock yourself out.
                    But {snail} here is a valuable thoroughbred, so be careful, alright?
                player:
                    Trust me, I know what I'm doing.
                snail:
                    !right
            ]],
            DIALOG_OSHNU_DEAD = [[
            * You asserted your dominance so well that the oshnu rolled over and died.
                wrangler:
                    !right
                    !angry
                    Look what you did!
                player:
                    Uhhh, sorry?
                    I guess I don't know my own strength!
                wrangler:
                    Get out of here before I call the Admiralty!
                    Oshnu-killer!
                player:
                    !exit
            ]],
            DIALOG_OSHNU_ALIVE = [[
                snail:
                    !right
                    Meeerp.
                    !exit
                * The oshnu submits to your will and oozes out of the way.
                wrangler:
                    !right
                    Hey, you weren't lyin'. You really speak their language, huh?
                player:
                    I'm a natural.
                    Now you can do your thing and have time to come to the party, yeah?
                    Just, uh, be sure to clean up first.
                wrangler:
                    What, you think I go around smelling like oshnu droppings for fun?
                    See you later!
                    !exit
            ]],

            OPT_CONVINCE_OSHNU = "Convince {snail} to move",
            DIALOG_CONVINCE_OSHNU = [[
                player:
                    Let me talk to {snail}.
                    I have a way with animals.
                wrangler:
                    !shrug
                    Worth a shot, I guess?
                    !exit
                snail:
                    !right
                    Blem?
            ]],
            DIALOG_CONVINCE_OSHNU_SUCCESS = [[
                player:
                    So that's why you gotta do what {wrangler} here says. You get it?
                snail:
                    Mrrrrrown.
                    !exit
                * The oshnu ducks out of the way, letting {wrangler} into the pen.
                wrangler:
                    !right
                    Hey, you weren't lyin'. You really speak their language, huh?
                player:
                    I'm a natural.
                    Now you can do your thing and have time to come to the party, yeah?
                    Just, uh, be sure to clean up first.
                wrangler:
                    What, you think I go around smelling like oshnu droppings for fun?
                    See you later!
                    !exit
            ]],
            DIALOG_CONVINCE_OSHNU_FAILURE = [[
                player:
                    So that's why you gotta do what {wrangler} here says. You get it?
                snail:
                    Rarrf!
                player:
                    !threaten
                    Don't you take that tone with me!
                snail:
                    !exit
                * {snail} clearly is having none of what you're offering.
                wrangler:
                    !right
                    !placate
                    Hey, hey, let's simmer down.
                    These things start biting when they get too angry.
                player:
                    !hips
                    The Oshnu started it.
                wrangler:
                    Well, now I have to finish it. Good-bye.
                    !exit
            ]],

        }
        :Fn(function(cxt) 
            
            cxt.quest.param.snail = cxt.quest.param.snail or AgentUtil.GetFreeAgent("MERCHANT_SNAIL")
            cxt:ReassignCastMember('snail', cxt.quest.param.snail)
            cxt:TalkTo(cxt.quest.param.snail)
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_FIGHT_OSHNU")
                :Dialog("DIALOG_FIGHT_OSHNU")
                :Battle{
                    flags = BATTLE_FLAGS.NO_BYSTANDERS,
                    on_start_battle = function(battle)
                        local fighter = battle:GetFighterForAgent(cxt.quest.param.snail)
                        if fighter then
                            fighter:AddCondition("WANTED_ALIVE")
                        end
                    end,
                }
                    :OnWin()
                        :Fn(function() 
                            
                            if cxt.quest.param.snail:IsDead() then
                                cxt:Dialog("DIALOG_OSHNU_DEAD")
                                cxt:Opinion("killed_snail", "wrangler")
                                cxt.quest:Fail("get_wrangler")
                            else
                                cxt:Dialog("DIALOG_OSHNU_ALIVE")
                                cxt.quest:Complete("get_wrangler")
                            end
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end)

            cxt:BasicNegotiation("CONVINCE_OSHNU")
                :OnSuccess()
                    :CompleteQuest("get_wrangler")
                    :Travel()
                :OnFailure()
                    :FailQuest("get_wrangler")
                    :Travel()

            StateGraphUtil.AddLeaveLocation(cxt)

        end)



QDEF:AddConvo("get_merchant", "merchant")
    :ConfrontState()
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    target:
                        !left
                    merchant:
                        !right
                    * You find {merchant} trying to conduct business.
                    merchant:
                        It's a solid deal. You'd be a fool to turn it down.
                    player:
                        !left
                        !happy
                        Oh, I agree.
                    merchant:
                        !dubious
                        Excuse me? 
                        Can't you see we're doing business here, grifter?
                    player:
                        I'm in business, too.
                        !chuckle
                        The business of partying down!
                        Party at Moreef's! You're invited! Very exclusive!
                    merchant:
                        I'm <i>clearly</i> busy.                         
                }
                {not first_time?
                    * {merchant} and {target} are still in discussions.
                    merchant:
                        !right
                    target:
                        !left
                        I'm still not sure...
                    player:
                        !happy
                        !left
                    merchant:
                        You again? I told you, I'm busy!
                }
            ]],


            OPT_SEAL_DEAL = "Convince {target} to agree to the deal",
            DIALOG_SEAL_DEAL = [[
                player:
                    All you gotta do is make 'em reflect.
                target:
                    !right
                player:
                    !point
                    You! {target}! What are you, some no-good slob?
                    This deal is only for smartypants and go-getters!
            ]],
            DIALOG_SEAL_DEAL_SUCCESS = [[
                player:
                    !point
                    You wanna be a plankton? Or a barracuda?
                target:
                    !interest
                    The lifecycle of a plankton is actually extremely interesting.
                    But... I don't think I'd be very good at it.
                player:
                    Exactly! So we're in business, yeah?
                target:
                    !bashful
                    Yes! I mean... I think? Y-yes!
                    The deal is on, {merchant}! I'll send along the paperwork shortly.
                    !exit
                merchant:
                    !right
                    Well, I'll be. I've been working that deal for weeks.
                player:
                    !left
                    Yikes. Embarrassing for you, huh? 
                    What can I say? I'm a pro.
                merchant:
                    !right
                    And you'll be at that party later, yes? Seems like a good networking opportunity.
                    Count me in!
                    !exit
            ]],
            DIALOG_SEAL_DEAL_FAILURE = [[
                player:
                    You might look like a dummy, but you got a chance to prove us otherwise by just agreeing to... whatever it is {merchant}'s asking.
                target:
                    !angry
                    Is this how you conduct business? With insults?
                    Forget it, {merchant}. We're done here.
                    !exit
                merchant:
                    !right
                    W-wait! {target}!
                    I assure you this grifter isn't a licensed representative of my interests!
                * {merchant} chases after {target}, leaving you alone with your failures.
                    !exit
                player:
                    !dubious
                    So I guess that's a no on the party, huh?
            ]],

            OPT_BRIBE = "Bribe {target} to close the deal",
            DIALOG_BRIBE = [[
                player:
                    Now, I may not be much of a merchant, but even I know it takes money to make money.
                target:
                    !right
                player:
                    Here. I'm gonna give you this bribe, and then you'll agree to whatever malarky {merchant} is proposing.
                    Deal?
                target:
                    !dubious
                    I—well...
                    This puts me in the black, at least. So... yes?
                    I'll send along the paperwork shortly, {merchant}.
                    !shrug
                    You have an interesting arrangement with your partner here.
                    !exit
                
                merchant:
                    !right
                player:
                    We good? Party time?
                merchant:
                    It may have been your shills to start, but now {target.heshe}'s going to expect cash up front.
                    !sigh
                    Whatever. I need a drink. I'll go to your stupid party.
                player:
                    It's only as stupid as the guest list! So cheer up. 
                    See ya later!
            ]]

        }
        :Fn(function(cxt) 
            
            cxt.quest.param.business_target = cxt.quest.param.business_target or AgentUtil.GetFreeAgent("FOREMAN")
            cxt:ReassignCastMember('target', cxt.quest.param.business_target)
            cxt:TalkTo(cxt.quest.param.business_target)
            cxt:Dialog("DIALOG_INTRO")

            cxt:BasicNegotiation("SEAL_DEAL", {helpers = {"merchant"}})
                :OnSuccess()
                    :CompleteQuest("get_merchant")
                    :ReceiveOpinion("sealed_deal",nil, "merchant")
                    :Travel()

                :OnFailure()
                    :FailQuest("get_merchant")
                    :Travel()


            cxt:Opt("OPT_BRIBE")
                :Dialog("DIALOG_BRIBE")
                :DeliverMoney(40)
                :CompleteQuest("get_merchant")
                :Travel()
            
            StateGraphUtil.AddLeaveLocation(cxt)
        end)


QDEF:AddConvo("get_luminitiate", "luminitiate")
    :ConfrontState()
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    * You find {luminitiate} lecturing passersby about the will of Hesh.
                    luminitiate:
                        !right
                        People often ask why does Hesh not show itself to us?
                        Why must we only feel the machinations of its will as it twists us from the inside?
                        !overthere
                        Well, now Hesh has presented itself upon our very shores!
                    heckler:
                        !left
                        !handwave
                        Hah! It's still twisting my insides, though! The smell gets worse every day.
                        !exit
                    * The flock doesn't seem to be receptive to {luminitiate}'s advances.
                    player:
                        !left
                        !crossed
                        Seems like you're having difficulty gaining traction, huh?
                    luminitiate:
                        !angry_point
                        That miscreant keeps heckling me! At first I thought it was because I'm still a novice...
                        But now I wonder if {heckler.heshe} might be a heathen!
                    player:
                        It happens. Anyway, why not let off some steam, eh?
                        !interest
                        There's a party going down at the Hideaway and you should come.
                    luminitiate:
                        !angry_shrug
                        I can't <i>party</i> while the word goes unspoken! 
                        These people are ignoring the pull of Hesh's tides. 
                }
                {not first_time?
                    * {luminitiate} and {heckler} are still arguing.
                    luminitiate:
                        !right
                    heckler:
                        !left
                        !angry_shrug
                        How about Hesh check out <i>my</i> Abyss, huh?
                    player:
                        !happy
                        !left
                    luminitiate:
                        !angry
                        Do you see what I'm contending with?!
                }
            ]],

            OPT_THREATEN = "Preach to {heckler}",
            DIALOG_THREATEN = [[
                player:
                    Here. I'm not up on my Luminology, but I know a thing or two about bluster—I mean, preaching.
                heckler:
                    !right
                    !surprise
                    W-wait! I think I feel Hesh's influence moving within me. 
                    It wants me to... to...
                    !cruel
                    * {heckler} bends over and lets loose a gust of flatulence so impressive you have to stimy your applause.
                player:
                    !impressed
                    If you had that in you, how do you know it <i>wasn't</i> Abyssal influence, huh?
            ]],
            
            DIALOG_THREATEN_SUCCESS = [[
                player:
                    Look, maybe you don't like the idea of Hesh. But what's the alternative?
                    No boss? No order? The rich get richer, the poor change hands...
                    And every day of your worthless existence is just stretchin' out the inevitable, waiting to die.
                heckler:
                    !scared
                    ...
                    Geez, that's depressing.
                player:
                    !overthere
                    But hey, {luminitiate} here is the dumb one for thinking Hesh fills {luminitiate.himher} with purpose, right?
                    And if {luminitiate.heshe}'s wrong, the worst {luminitiate.heshe} did was live each day like it meant something.
                    Sucks to be {luminitiate.himher}, right?
                heckler:
                    !sad 
                    Y-yeah, well... 
                    I've got purpose, too. Just need to find it... is all.
                    * Chastised, {heckler} melts into the crowd, likely off to search for meaning. 
                    !exit
                luminitiate:
                    !right
                    That was... very impressive!
                    !thought
                    I'm not sure I appreciate the way you "hedged your bets" as a tactic but I can't argue with results.
                    That heckler may very well be a convert.
                player:
                    !hips
                    I'll drink to that, eh?
                luminitiate:
                    !agree
                    Yes, I think so! 
                    You've convinced me that in order to speak to the world, I must spend more time in it, anyway.
                    !exit
            ]],
            DIALOG_THREATEN_FAILURE = [[
                player:
                    Maybe you've been doin' Hesh's bidding this whole time. Maybe Hesh is the one who wanted you to come here and act like a prig.
                    You know, to motivate {luminitiate} to preach better.
                heckler:
                    Hah! Well if that's the case, it doesn't seem to be working.
                    Guess Hesh picked the wrong person for the job.
                    <i>Again</i>.
                    !exit
                * {heckler} decides to make an exit, and the small crowd leaves with {heckler.himher}.
                luminitiate:
                    !right
                    Well, I've failed—no thanks to you. 
                player:
                    So is Hesh inspiring you to drink it off?
                luminitiate:
                    No. Hesh is inspiring me to sulk in my bunk all night and wish you'd never been born.
                    Good day.
                    !exit
            ]],

            OPT_BEAT = "Beat up {heckler}",
            DIALOG_BEAT = [[
                player:
                    Watch this...
                right:
                    !exit
                player:
                    Hey, {heckler}?
                heckler:
                    !right
                    Yeah?
                player:
                    !fight
                    Boo!
            ]],
            DIALOG_WON = [[
                luminitiate:
                    !right
                    Behold! The will of Hesh!
                    It plucks you from the shallows and fills you with intent! 
                    Submit yourself and you, like {player}, will always know which way the current pulls you. 
                * The crowd definitely seems moved by your display. A couple passersby make the symbol of Hesh upon their brow.
                luminitiate:
                    That is certainly one way to inspire the flock, {player}.
                    I suppose there will be others at this party tonight? Count me in—perhaps it'll be easier to ply my craft on the inebriated.
                    !exit
            ]]


        }
        :Fn(function(cxt) 
            
            cxt.quest.param.heckler = cxt.quest.param.heckler or AgentUtil.GetFreeAgent("LABORER")
            cxt:ReassignCastMember('heckler', cxt.quest.param.heckler)
            cxt:TalkTo(cxt.quest.param.heckler)
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:BasicNegotiation("THREATEN", {flags = NEGOTIATION_FLAGS.INTIMIDATION, helpers = {"luminitiate"}})
                :OnSuccess()
                    :CompleteQuest("get_luminitiate")
                    :Travel()
                :OnFailure()
                    :FailQuest("get_luminitiate")
                    :Travel()

            cxt:Opt("OPT_BEAT")
                :Dialog("DIALOG_BEAT")
                :Battle{noncombatants = {"luminitiate"}}
                    :OnWin()
                        :Dialog("DIALOG_WON")
                        :CompleteQuest("get_luminitiate")
                        :Travel()
            
            StateGraphUtil.AddLeaveLocation(cxt)
        end)



QDEF:AddConvo("return_to_moreef_party", "moreef")
    :ConfrontState()
    :Loc{
        DIALOG_INTRO = [[
            * You must've been able to slap on the ooze, because {1#agent_list} all showed up. Even you didn't think you could be so persuasive.
            moreef:
                !right
            player:
                !left
                !happy
                How's it going? I see everyone made it, thanks to me, of course.
            moreef:
                !shrug
                Well, you were always popular in school. So I figured this had to be easy for you.
                Looks like I'm gonna need your other skillset soon, though. There's trouble brewing.
                !point
                Look at those two.
            player:
                !exit
            moreef:
                !exit

        ]],

    }
    :Fn(function(cxt) 

        for k,v in pairs(cxt.quest.param.party_people) do
            v:MoveToLocation(cxt.location)
        end
        cxt:Dialog("DIALOG_INTRO", cxt.quest.param.party_people)

        
        local W = table.arrayfind( cxt.quest.param.party_people, cxt.quest:GetCastMember("wrangler")) ~= nil
        local M = table.arrayfind( cxt.quest.param.party_people, cxt.quest:GetCastMember("merchant")) ~= nil
        local A = table.arrayfind( cxt.quest.param.party_people, cxt.quest:GetCastMember("admiralty")) ~= nil
        local L = table.arrayfind( cxt.quest.param.party_people, cxt.quest:GetCastMember("luminitiate")) ~= nil
        
        if L and M and A then
            cxt:GoTo("STATE_LMA")
        elseif M and W and L then
            cxt:GoTo("STATE_MWL")
        elseif A and L and W then
            cxt:GoTo("STATE_ALW")
        else
            cxt:GoTo("STATE_WAM")
        end

    end)
    --luminitiate tries to extract a titlhe from the merchant
    :State("STATE_LMA")
        :Loc{
            DIALOG_INTRO = [[
                merchant:
                    !right
                    !angry
                luminitiate:
                    !left
                    !angry
                * You see {luminitiate} hassling {merchant}, spittle flying between them.
                luminitiate:
                    !threaten
                    You've been profiting more than ever with the glut of pilgrims.
                    The tithes you pay should reflect that! It should be a percentage, not a contracted amount!
                merchant:
                    Why? If Hesh saw fit to line my pockets, why shouldn't I keep the change, huh?
                    I've paid my tithes, and you won't get another shill from me, novice.
            ]],

            OPT_CONVINCE_LUMINITIATE = "Convince {luminitiate} to back down",
            DIALOG_CONVINCE_LUMINITIATE = [[
                luminitiate:
                    !right
                player:
                    !left
                    Hey, this isn't gonna win you over any devotees, {luminitiate}.
            ]],
            DIALOG_CONVINCE_LUMINITIATE_SUCCESS = [[
                player:
                    If Hesh compels people, you gotta put yourself in their head.
                    {merchant} here has made a lot of money in a scary time. But nobody knows what will happen tomorrow.
                    Supplies are hard to come by, and some businesses are going under. 
                    {merchant} probably feels like {merchant.heshe}'s gotta have a floodfund for when the tides change.
                    And if that's how {merchant} feels, then that's probably what Hesh wants {merchant.himher} to feel, right?
                luminitiate:
                    Hrm. I doubt {merchant} is in touch with the will of Hesh enough to know <i>what</i> Hesh wants of {merchant.himher}.
                    But... since I'm still a novice... I can't claim to know for certain.
                    Do what you will, merchant. But remember: those who fight against the tides will drown in them.
                    !exit
                * {merchant} already seems to be reaching for another drink. Whatever insults were traded are soon forgotten in the fog.
            ]],
            DIALOG_CONVINCE_LUMINITIATE_FAILURE = [[
                player:
                    Sometimes you gotta be gentle, alright?
                luminitiate:
                    Gentle? Hesh is not a gentle god!
                    Hesh demands! Hesh manipulates!
                    And those who fight their purpose will choke on the brine. 
                    I refuse to associate with you bloated floaters. 
                    !exit
                * Unfortunately, {luminitiate}'s outburst was a real mood-killer. The drinking stops, and soon everyone begins to trickle out.
            ]],


            OPT_CONVINCE_MERCHANT = "Convince {merchant} to buy a round",
            DIALOG_CONVINCE_MERCHANT = [[
                merchant:
                    !right
                player:
                    !left
                    With that new deal of yours, maybe you wanna buy the next round, huh?
            ]],
            DIALOG_CONVINCE_MERCHANT_SUCCESS = [[
                player:
                    After all, {luminitiate} will never see the kind of shills you have.
                    Surely Hesh moves you to flaunt your wealth.
                merchant:
                    !cruel
                    Hm. I suppose that's true. 
                    Luminari aren't known for their lavish lifestyles, and failed recruits don't go on to become Bishops.
                    Very well, novice. Let me buy you a drink, and may it be as salty as you hope your future to be.
                    !exit
                * {luminitiate} seems content with the offer, and by the next round both {luminitiate.heshe} and {merchant} are drunkenly singing torch songs at the bar.
            ]],
            DIALOG_CONVINCE_MERCHANT_FAILURE = [[
                player:
                    Share the shills, bud. Don't be a cheapskate.
                merchant:
                    Hah! You think because I <i>have</i> shills that means they're yours to spend indiscriminately.
                    I built my business very well before you came along today, and I did so by choosing carefully how I use my money.
                    And I think I've spent enough of it here today, thank you. I bid you adieu.
                    !exit
                * With the merchant's tirade, the other partiers don't seem inspired to spend any more of their coin. 
            ]],

            OPT_DO_NOTHING = "Stay out of it",
            DIALOG_DO_NOTHING = [[  
                * You step back.
                left:
                    !exit
                right:
                    !exit
                * {merchant} and {luminitiate} yell at each other for a while longer, and thoroughly ruin everyone's night.
            ]]
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:BasicNegotiation( "CONVINCE_LUMINITIATE",
                {
                    target_agent = cxt.quest:GetCastMember("luminitiate"),
                    helpers = {"merchant"}
                })
                    :OnSuccess()
                        :GoTo("STATE_DONE")
                    :OnFailure()
                        :Fn(function() cxt.quest.param.bad_party = true end)
                        :GoTo("STATE_DONE")

            cxt:BasicNegotiation( "CONVINCE_MERCHANT",
                {
                    target_agent = cxt.quest:GetCastMember("merchant"),
                    helpers = {"luminitiate"}
                })
                    :OnSuccess()
                        :GoTo("STATE_DONE")
                    :OnFailure()
                        :Fn(function() cxt.quest.param.bad_party = true end)
                        :GoTo("STATE_DONE")

            cxt:Opt("OPT_DO_NOTHING")
                :Dialog("DIALOG_DO_NOTHING")
                :Fn(function() cxt.quest.param.bad_party = true end)
                :GoTo("STATE_DONE")
        
        end)

    --merchant complains about wrangler's smell
    :State("STATE_MWL")
        :Loc{
            DIALOG_INTRO = [[
                merchant:
                    !right
                    !angry
                    It smells like an oshnu in here!

                wrangler:
                    !left
                    !angry
                    H-hey! It's not me—I made sure to bathe twice before coming.
                merchant:
                    I wasn't told a <i>laborer</i> would be here. I thought the guest list was more exclusive. 
                    But if this is the kind of riff-raff I'm expected to associate with, I'll just leave.
                wrangler:
                    !threaten
                    Not without apologizing first, you ain't!
            ]],

            OPT_CONVINCE_MERCHANT = "Convince {merchant} to apologize",
            DIALOG_CONVINCE_MERCHANT = [[
                merchant:
                    !right
                player:
                    !left
                    !placate
                    If the Great Beaching has taught us anything, it's that it doesn't matter what your station is. 
                    You can still get squashed by Hesh's will.
            ]],
            DIALOG_CONVINCE_MERCHANT_SUCCESS = [[
                player:
                    !thumb
                    Take my word for it: high places, low places. It don't matter. 
                    A friend is a friend, and can save your skin when you least expect it.
                merchant:
                    !thought
                    I suppose that's true.
                    After all, you walked through my door earlier and secured that deal.
                    Perhaps I should be more open-minded.
                wrangler:
                    !left
                merchant:
                    !sigh
                    {wrangler}, I apologize for commenting on your odor.
                    If you wish to purchase some perfumes, I do carry them and would be happy to extend a... four-percent discount.
                wrangler:
                    !dubious
                    Thanks, I guess?
                    !exit
                * {wrangler} makes for the far side of the bar, but seems to enjoy {wrangler.himher}self all the same.
            ]],
            DIALOG_CONVINCE_MERCHANT_FAILURE = [[
                player:
                    You could die tomorrow, so why not be nice today?
                merchant:
                    !spit
                    I would rather die tomorrow than associate with the likes of {wrangler}.
                    !exit
                * {merchant} leaves, and then {wrangler} follows. You hear the sounds of a brief scuffle outside.
                * The dust settles, but the mood is ruined.
            ]],

            OPT_DEFEND_MERCHANT = "Defend {merchant}",
            DIALOG_DEFEND_MERCHANT = [[
                player:
                    !left
                    !fight
                    Hey, no fighting in the bar! {wrangler}, you're pushin' it!
            ]],

            DIALOG_WON_NO_DEATHS = [[
                wrangler:
                    !right
                    !injured
                player:
                    !happy
                    You get it out of your system? 
                wrangler:
                    Yeah... yeah, I guess so.
                merchant:
                    !left
                    I must say, you put up quite the fight.
                    Perhaps I could interest you in the private security sector? 
                    At any rate your smell would serve as a warning before we set you loose, hm?
                    Let me buy you a drink, and we can discuss.
                    !exit
                wrangler:
                    !exit
                * {wrangler}, surprisingly, seems flattered by this offer. The bar collectively exhales, and gets back to spending money.
            ]],
            DIALOG_WON_DEATHS = [[
                right:
                    !exit
                moreef:
                    !right
                    Hesh, Smith! Nobody told you to kill anyone!
                * But you did, and the bar soon empties as the blood creeps across the floor.
            ]],

            OPT_DO_NOTHING = "Stay out of it",
            DIALOG_DO_NOTHING = [[  
                * You step back.
                left:
                    !exit
                right:
                    !exit
                * {merchant} and {luminitiate} yell at each other for a while longer, and thoroughly ruin everyone's night.
            ]]        
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:BasicNegotiation( "CONVINCE_MERCHANT",
                {
                    target_agent = cxt.quest:GetCastMember("merchant"),
                    helpers = {"wrangler"}
                })
                    :OnSuccess()
                        :GoTo("STATE_DONE")
                    :OnFailure()
                        :Fn(function() cxt.quest.param.bad_party = true end)
                        :GoTo("STATE_DONE")


            cxt:Opt("OPT_DEFEND_MERCHANT")
                :Dialog("DIALOG_DEFEND_MERCHANT")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = {"merchant"},
                    enemies = {"wrangler"},
                }
                    :OnWin()
                        :Fn(function(cxt, battle) 
                            local death = CheckForDeath(battle)
                            if death then
                                cxt.quest.param.bad_party = true
                                cxt:Dialog("DIALOG_WON_DEATHS")
                            else
                                cxt:Dialog("DIALOG_WON_NO_DEATHS")

                            end
                        end)
                        :GoTo("STATE_DONE")


            cxt:Opt("OPT_DO_NOTHING")
                :Dialog("DIALOG_DO_NOTHING")
                :Fn(function() cxt.quest.param.bad_party = true end)
                :GoTo("STATE_DONE")


        end)

    --Admiralty tries to pick a fight with the luminitiate
    :State("STATE_ALW")
        :Loc{
            DIALOG_INTRO = [[
                admiralty:
                    !right
                    !fight
                luminitiate:
                    !left
                    !fight
                * Everything is going great, until {admiralty} and {luminitiate} come to blows.
                admiralty:
                    The Admiralty is the true Deltrean order! Heshians have been inching in on the state for years.
                    But not today, little cultist. And not on my watch.
                luminitiate:
                    The Admiralty is the least of Hesh's reach, and will be dropped as a dead limb before too long!
                    You, however, will likely die first!
            ]],
            OPT_JOIN_ADMIRALTY = "Jump in on {admiralty}'s side",
            DIALOG_JOIN_ADMIRALTY = [[
                player:
                    !left
                    !fight
                luminitiate:
                    !right
                * You enter the fray to break up the fight.
            ]],
            OPT_JOIN_LUMINITIATE = "Jump in on {luminitiate}'s side",
            DIALOG_JOIN_LUMINITIATE = [[
                player:
                    !left
                    !fight
                admiralty:
                    !right
                * You enter the fray to break up the fight.
            ]],
            DIALOG_WON_DEATHS = [[
                right:
                    !exit
                * You let the bodies hit the floor—and with that, the party's over.
            ]],
            DIALOG_WON_NO_DEATHS = [[
                right:
                    !injured
                player:
                    Everyone, just settle down!
                    !happy
                    This is supposed to be a party!
                    !exit
                right:
                    !exit
                * The pressure that was in the room has been released, and everyone goes back to nursing their drinks and their grudges.
            ]],
            OPT_DO_NOTHING = "Don't get involved",
            DIALOG_DO_NOTHING = [[
                * You decide they're not worth the trouble.
                left:
                    !exit
                right:
                    !exit
                * A fight starts, and soon the air is thick with fists and bottles. You sit calmly as the chaos swirls around you, until it eventually spills out into the street.
            ]]

        }
        :Fn(function(cxt) 

            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_JOIN_ADMIRALTY")
                :Dialog("DIALOG_JOIN_ADMIRALTY")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = {"admiralty"},
                    enemies = {"luminitiate"},
                }
                    :OnWin()
                        :Fn(function(cxt, battle) 
                            if CheckForDeath(battle) then
                                cxt.quest.param.bad_party = true
                                cxt:Dialog("DIALOG_WON_DEATHS")
                            else
                                cxt:Dialog("DIALOG_WON_NO_DEATHS")

                            end
                        end)
                        :GoTo("STATE_DONE")

            cxt:Opt("OPT_JOIN_LUMINITIATE")
                :Dialog("DIALOG_JOIN_LUMINITIATE")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = {"luminitiate"},
                    enemies = {"admiralty"},
                }
                    :OnWin()
                        :Fn(function(cxt, battle) 
                            if CheckForDeath(battle) then
                                cxt.quest.param.bad_party = true
                                cxt:Dialog("DIALOG_WON_DEATHS")
                            else
                                cxt:Dialog("DIALOG_WON_NO_DEATHS")

                            end
                        end)
                        :GoTo("STATE_DONE")

            cxt:Opt("OPT_DO_NOTHING")
                :Dialog("DIALOG_DO_NOTHING")
                :Fn(function() cxt.quest.param.bad_party = true end)
                :GoTo("STATE_DONE")

        end)

    --Wrangler wants revenge against the admiralty
    :State("STATE_WAM")
        :Loc{
            DIALOG_INTRO = [[
                * As the party drags on, {wrangler} gets more and more sullen.
                admiralty:
                    !left
                wrangler:
                    !right
                    !angry
                    It's you! 
                admiralty:
                    !surprised
                wrangler:
                    You don't remember me, do you?
                    You planted slurry on me and busted me a while back.
                    I spent three months doing hard labor because of you!
                admiralty:
                    You're right.
                    !shrug
                    I don't remember everyone I've ever busted.
                    !cruel
                    But if I did bust you, you probably deserved it.
            ]],

            OPT_CONVINCE_WRANGLER = "Convince {wrangler} {wrangler.heshe} has the wrong Admiralty",
            DIALOG_CONVINCE_WRANGLER = [[
                wrangler:
                    !right
                player:
                    !left
                    Are you sure you have the right switch? They all wear the same uniform.
            ]],
            DIALOG_CONVINCE_WRANGLER_SUCCESS = [[
                player:
                    And {admiralty} has a pretty boring face. 
                wrangler:
                    Hm. I suppose that's true.
                    And I'm not really eager to get in trouble with the law again.
                admiralty:
                    !left
                    Hey, this is a party, so I'll let it slide.
                    Go get yourself a drink. I promise I won't arrest you tonight.
                    !exit 
                wrangler:
                    !exit
                * An uneasy calm falls back on the room, and the party continues.

            ]],
            DIALOG_CONVINCE_WRANGLER_FAILURE = [[
                player:
                    Besides, what are you gonna do? Beat up an officer?
                wrangler:
                    {admiralty.HeShe}'s off duty. I'm not beating up an officer.
                    !cruel
                    I'm getting in a bar brawl.
                    !exit
                player:
                    !exit
                    
                    * {wrangler} attacks {admiralty}. {admiralty} easily deflects the blows, and counters with {admiralty.hisher} own.
                    * The two of them spill out of the bar, and the party is ruined.

            ]],

            OPT_BEAT_UP_ADMIRALTY = "Beat up {admiralty}",
            DIALOG_BEAT_UP_ADMIRALTY = [[
                admiralty:
                    !right
                player:
                    !left
                    You like picking on the little guy, eh?
                    !fight
                    How about I teach you what that feels like?
            ]],
            DIALOG_WON_DEATHS = [[
                right:
                    !exit
                left:
                    !exit
                * What was supposed to be a simple beating turned into something far more deadly. The party is over.
            ]],
            DIALOG_WON_NO_DEATHS = [[
                admiralty:
                    !injured
                player:
                    You learned your lesson?
                admiralty:
                    Y-yes sir!
                player:
                    !happy
                    Great. Now drink up!
                    !exit
                admiralty:  
                    !exit
                * Moreef makes a fortune on rounds once the crowd resumes drinking.
            ]],

            OPT_DO_NOTHING = "Don't intervene",
            DIALOG_DO_NOTHING = [[
                * You let them go at it.
                left:
                    !exit
                right:
                    !exit
                * It's not a fair fight, and {wrangler} takes the worst of it, ending up under arrest.
                * The party is over.
            ]]
            
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")


            cxt:BasicNegotiation( "CONVINCE_WRANGLER",
                {
                    target_agent = cxt.quest:GetCastMember("wrangler"),
                    helpers = {"admiralty"}
                })
                    :OnSuccess()
                        :GoTo("STATE_DONE")
                    :OnFailure()
                        :Fn(function() cxt.quest.param.bad_party = true end)
                        :GoTo("STATE_DONE")




            cxt:Opt("OPT_BEAT_UP_ADMIRALTY")
                :Dialog("DIALOG_BEAT_UP_ADMIRALTY")
                :Battle{
                    allies = {"wrangler"},
                    enemies = {"admiralty"},
                }
                    :OnWin()
                        :Fn(function(cxt, battle) 
                            if CheckForDeath(battle) then
                                cxt.quest.param.bad_party = true
                                cxt:Dialog("DIALOG_WON_DEATHS")
                            else
                                cxt:Dialog("DIALOG_WON_NO_DEATHS")

                            end
                        end)
                        :GoTo("STATE_DONE")


            cxt:Opt("OPT_DO_NOTHING")
                :Dialog("DIALOG_DO_NOTHING")
                :Fn(function() cxt.quest.param.bad_party = true end)
                :GoTo("STATE_DONE")
        end)

    :State("STATE_DONE")
        :Loc{
            DIALOG_GOOD_PARTY = [[
                moreef:
                    !right
                player:
                    !left
                * When everyone leaves, you approach {moreef}.
                player:
                    How's the register?
                moreef:
                    !happy
                    Overflowing, believe it or not. Didn't think I could milk so much out of such a small guest list.
                    But you always had a way with people.
                    When you weren't... shaving off a chunk of their fur, shaping it into a witch doll, burying the witch doll in rancid oshnu bile, and then dunking their head in the cistern.
                player:
                    Weirdly specific thing to remember, considering I don't. Who'd I do that to?
                moreef:
                    !sigh 
                    Nobody important.
                    !happy
                    What's important is these profits, eh? 
            ]],
            DIALOG_BAD_PARTY = [[
                moreef:
                    !right
                player:
                    !left
                * {moreef} is not happy.
                moreef:
                    !angry
                    You were supposed to keep things calm! Now I have bills <i>and</i> damages to pay for.
                    And I'll be lucky if any of my regulars ever come back.
                player:
                    !shrug
                    I'm more a party goer than a party thrower, {moreef}.
                moreef:
                    Ugh. Whatever. Now I have to clean up this mess.
            ]]
        }
        :Fn(function(cxt) 
            
            if cxt.quest.param.bad_party then
                cxt:Dialog("DIALOG_BAD_PARTY")
                for k,v in pairs(cxt.quest.param.party_people) do
                    if not v:IsDead() then
                        cxt:Opinion("bad_party", v)
                        v:MoveToLimbo()
                    end
                end

                cxt.quest:Fail()
            else
                cxt:Dialog("DIALOG_GOOD_PARTY")
                ConvoUtil.GiveQuestRewards(cxt)
                cxt.quest:Complete()
                for k,v in pairs(cxt.quest.param.party_people) do
                    if not v:IsDead() then
                        cxt:Opinion("good_party", v)
                        v:MoveToLimbo()
                    end
                end
            end

            StateGraphUtil.AddEndOption(cxt)

        end)


QDEF:AddConvo("return_to_moreef_no_party", "moreef")
    :Loc{
        OPT_TELL_MOREEF = "Tell {moreef} the party is cancelled",
        DIALOG_TELL_MOREEF = [[
            player:
                Sorry, Sweets, I couldn't make it happen.
                Everyone's just got too much going on, ya know?
            agent:
                !sigh
                Yeah, I guess. I shouldn't be surprised.
                Just figured if anyone could make a party happen, it'd be you.
            player:
                Well, we can still party just the two of us, if you want!
                Like old times?
            agent:
                !sigh
                I never went to the same parties as you in the old times, Smith.
                Don't see a reason to start now.
        ]]
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_TELL_MOREEF")
            :SetQuestMark()
            :Dialog("DIALOG_TELL_MOREEF")
            :FailQuest()
    end)    

QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}
