


local START_DIFFICULTY =1

local MAX_DIFFICULTY = 5


local DAY_SCHEDULE = {
    {quest = "ROOK_STORY_DAY_1", difficulty = 1},  
    {quest = "ROOK_STORY_DAY_2", difficulty = 2},  
    {quest = "ROOK_STORY_DAY_3", difficulty = 3},  
    {quest = "ROOK_STORY_DAY_4", difficulty = 4},  
}


local QDEF = QuestDef.Define
{
    --title = "The Bog Job",
    qtype = QTYPE.STORY,

    on_init = function(quest)
        TheGame:GetGameState():SetMainQuest(quest)
        TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"))
        TheGame:GetGameState():SetDifficulty(START_DIFFICULTY)
        QuestUtil.StartDayQuests(DAY_SCHEDULE, quest)
        
        quest.param.faction_vendor_quest = QuestUtil.SpawnQuest( "ROOK_STORY_FACTION_VENDORS")
        quest.param.coin_trader_quest = QuestUtil.SpawnQuest( "ROOK_STORY_COIN_TRADER")
        
        if not TheGame:GetGameState():GetPlayerAgent().graft_owner:HasGraft("bare_mettle") then
            quest.param.mettle_quest = QuestUtil.SpawnQuest("ROOK_STORY_METTLE_SHOP")
        end
        

        if quest.param.start_on_day and quest.param.start_on_day > 2 then
            quest.param.coin_trader_quest:Complete("first_meeting")
            QuestUtil.SpawnQuest("ROOK_STORY_LIVING_AT_BAR")            
            QuestUtil.SpawnQuest( "ROOK_STORY_BLACK_MARKET")
            
            if quest.param.force_faction_alliance == "RISE" then
                quest:GetQuestDef().SetPrimaryHandlerRise(quest)
            else
                quest:GetQuestDef().SetPrimaryHandlerBarons(quest)
            end

            quest:GetQuestDef().SetVendorEnabled(quest, "SPARK_BARONS", true) 
            quest:GetQuestDef().SetVendorEnabled(quest, "RISE", true) 
        end
        if quest.param.start_on_day and quest.param.start_on_day >= 4 then
            quest.param.expdition_faction = quest.param.force_faction_alliance or "RISE"
            if quest.param.expdition_faction == "RISE" then
                quest:GetCastMember("spark_second"):Kill()
            else
                quest:GetCastMember("rise_second"):Kill()
            end
        end

        QuestUtil.DoNextDay(DAY_SCHEDULE, quest, quest.param.start_on_day  )
        
        TheGame:GetGameState():GetPlayerAgent().graft_owner:AddGraft(GraftInstance("grout_bog_burrs"))
        DoAutoSave()
    end,

    max_day = 4,
    on_post_load = function(quest)
        QuestUtil.StartDayQuests(DAY_SCHEDULE, quest)        
    end,

    get_narrative_progress = function(quest)
        
        local total_days = 4
        local completed_days = (quest.param.day or 1)-1
        if quest.param.day_quest == nil then
            return 0, loc.format( LOC "CALENDAR.DAY_FMT", 1 ), ""
        end

        local percent = (completed_days) / total_days
        local title = loc.format(LOC "CALENDAR.DAY_FMT", quest.param.day or 1)
        return percent, title, quest.param.day_quest and quest.param.day_quest:GetTitle() or ""
    end,


    quest_reward_fn = function(quest, reward_quest)
        local qrank = reward_quest:GetRank()
        local val = 80 + (qrank -1)*20
        
        if reward_quest:GetRewardSize() == QREWARDSIZE.SMALLER then
            val = val * .67
        elseif reward_quest:GetRewardSize() == QREWARDSIZE.LARGER then
            val = val * 1.5
        end
        
        if reward_quest:GetQuestDef().faction == "SPARK_BARONS" then
            val = val * 1.333
        end

        val = round_zero( val * reward_quest.reward_random, 5 )
        return val
    end,


    plot_armour_fn = function(quest, agent)
        return agent:IsCastInQuest(quest)
    end,

    events = 
    {
        quests_changed = function(quest, event_quest) 
            if event_quest == quest.param.day_quest and quest.param.day_quest:IsComplete() then
                QuestUtil.DoNextDay(DAY_SCHEDULE, quest)
            end
        end,
    },


    SetVendorEnabled = function(quest, faction, enabled)
        quest.param.faction_vendor_quest:GetQuestDef().SetVendorEnabled(quest.param.faction_vendor_quest, faction, enabled)
    end,
    EnhanceVendor = function(quest, faction)
        quest.param.faction_vendor_quest:GetQuestDef().EnhanceVendor(quest.param.faction_vendor_quest, faction)
    end,
    
    PresentFactionJobChoice = function(cxt, quest_options, force_take, on_picked_fn)

        for k,v in ipairs(quest_options) do
            v:SetHideInOverlay(true)
        end


        local quest = TheGame:GetGameState():GetMainQuest()
        if #quest_options > 0 then 

            cxt:RunLoopingFn( function()
                for k,job in ipairs(quest_options) do
                    
                    cxt:QuestOpt( job )
                        :ShowQuestAsInactive()
                        :Fn(function(cxt) 
                            StateGraphUtil.PresentQuestOffer(cxt, job, nil, 
                                function() 
                                    quest.param.offered_jobs[job.content_id] = (quest.param.offered_jobs[job.content_id] or 0) + 1
                                    for k,v in pairs(quest_options) do
                                        if v ~= job then
                                            v:Cancel()
                                        end
                                    end
                                    cxt:PlayQuestConvo(job, QUEST_CONVO_HOOK.ACCEPTED)
                                    cxt:Pop()
                                    job:SetHideInOverlay(false)
                                    on_picked_fn(cxt, job)
                                end,
                                function()
                                    if not cxt:PlayQuestConvo(job, QUEST_CONVO_HOOK.DECLINED) then
                                        cxt:Quip( cxt:GetAgent(), "job_decline")
                                    end

                                end,
                                #quest_options == 1)
                        end)
                end

                if not force_take then
                    StateGraphUtil.AddBackButton(cxt)
                end
            end )
        end
        
    end,

    SetPrimaryHandlerBarons = function(quest)
        quest.param.faction_pick = "SPARK_BARONS"
        quest.param.alt_faction_pick = "RISE"
        if quest.param.all_day_quests then
            for k,v in ipairs(quest.param.all_day_quests) do
                if v:GetQuestDef():GetCast("primary_handler") then
                    v:AssignCastMember("primary_handler", quest:GetCastMember("baron_contact"))
                end
                if v:GetQuestDef():GetCast("secondary_handler") then
                    v:AssignCastMember("secondary_handler", quest:GetCastMember("rise_contact"))
                end
            end        
        end

    end,

    SetPrimaryHandlerRise = function(quest)
        quest.param.faction_pick = "RISE"
        quest.param.alt_faction_pick = "SPARK_BARONS"
        if quest.param.all_day_quests then
            for k,v in ipairs(quest.param.all_day_quests) do
                if v:GetQuestDef():GetCast("primary_handler") then
                    v:AssignCastMember("primary_handler", quest:GetCastMember("rise_contact"))
                end
                if v:GetQuestDef():GetCast("secondary_handler") then
                    v:AssignCastMember("secondary_handler", quest:GetCastMember("baron_contact"))
                end
            end        
        end

    end,

    GetPrimaryHandler = function(quest)
        if quest.param.faction_pick == "RISE" then
            return quest:GetCastMember("rise_contact")
        elseif quest.param.faction_pick == "SPARK_BARONS" then
            return quest:GetCastMember("baron_contact")
        end
    end,

    GetSecondaryHandler = function(quest)
        if quest.param.faction_pick == "RISE" then
            return quest:GetCastMember("baron_contact")
        elseif quest.param.faction_pick == "SPARK_BARONS" then
            return quest:GetCastMember("rise_contact")
        end
    end,


    SpawnFactionJobs = function(quest, num_jobs, faction, must_have_tags, can_not_have_tags, cast_overrides, rank)
        rank = rank or TheGame:GetGameState():GetCurrentBaseDifficulty()
        quest.param.offered_jobs = quest.param.offered_jobs or {}
        local attempt_quest_ids = {}
        for k, questdef in pairs( Content.GetAllQuests() ) do
            if questdef:GetType() == QTYPE.CONTRACT and questdef:FilterForAct( "ROOK" ) and questdef.faction == faction then
                local include = true
                
                if must_have_tags then
                    include = include and questdef.tags and all_tags(questdef.tags, table.unpack(must_have_tags))
                end
                
                if can_not_have_tags then
                    include = include and not (questdef.tags and any_tags(questdef.tags, table.unpack(can_not_have_tags)))
                end

                if include then
                    table.insert(attempt_quest_ids, questdef.id)
                end
            end
        end
        
        local event_id = "ROOK_CONTRACT_SPAWN_".. (faction or "NO_FACTION")
        local quest_scores = {}
        for k,v in ipairs(attempt_quest_ids) do
            quest_scores[v] = QuestUtil.CalcQuestSpawnScore(event_id, math.floor(#attempt_quest_ids/2), v)    
        end
        
        table.shuffle(attempt_quest_ids) --to mix up the case where there are a lot of ties
        table.stable_sort(attempt_quest_ids, function(a,b) return quest_scores[a] < quest_scores[b] end)

        local quest_options = {}
        local failed_quests = {}
        local new_quest
        for _, quest_id in ipairs(attempt_quest_ids) do
            
            local foo
            local overrides = {qrank = rank}
            if cast_overrides then
                overrides.cast = cast_overrides
            end
            new_quest, foo = QuestUtil.SpawnInactiveQuest( quest_id, overrides) 
            if foo then
                table.insert(failed_quests, foo)
            end
            if new_quest then
                TheGame:GetGameProfile():RecordIncident(event_id, new_quest:GetContentID())
                table.insert(quest_options, new_quest)
                if #quest_options >= num_jobs then
                    break
                end
            end
        end

        return quest_options

    end,

}

--give our principal cast their jobs
:AddCastByAlias{
    cast_id = "hebbel",
    alias = "HEBBEL",
    on_assign = function(quest, agent)
        AgentUtil.TakeJob(agent, TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"), "bartender")
    end,
}

:AddCastByAlias{
    cast_id = "baron_contact",
    alias = "SPARK_CONTACT",
}

:AddCastByAlias{
    cast_id = "rise_contact",
    alias = "KALANDRA",
}    

:AddCastByAlias{
    cast_id = "spark_second",
    alias = "SPARK_SECOND",
}

:AddCastByAlias{
    cast_id = "rise_second",
    alias = "RISE_SECOND",
}


:AddObjective{
    id = "start",
    state = QSTATUS.ACTIVE,
}   



QDEF:AddDebugOption("start_on_day", {1,2,3,4})
    :AddDebugOption("force_faction_alliance", {"RISE", "SPARK_BARONS"}, function(params) return params.start_on_day and params.start_on_day > 2 end)
    :AddDebugOption("expedition_faction", {"RISE", "SPARK_BARONS"}, function(params) return params.start_on_day and params.start_on_day == 4 end)

