
local QDEF = QuestDef.Define
{
    title = "Black Market",
    icon = engine.asset.Texture("icons/quests/rook_story_rise_black_market.tex"),
    qtype = QTYPE.STORY,
    home = true,
    plot_armour_fn = function(quest, agent)
        if agent:GetBrain():GetWorkplace() == quest:GetCastMember("home") then
            return true
        end
    end,

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("plocka"), location = quest:GetCastMember("home"), role = CHARACTER_ROLES.VENDOR } )
    end,

    events = 
    {
        caravan_member_event = function(quest, event, agent, old_loc, new_loc)
            if event == "agent_location_changed" and agent == TheGame:GetGameState():GetPlayerAgent() and new_loc == quest:GetCastMember("home") then
                if quest.param.last_graft_restock ~= TheGame:GetGameState():GetDateTime() then
                    quest.param.negotiation_grafts = GenerateGrafts(GRAFT_TYPE.NEGOTIATION)
                    quest.param.combat_grafts = GenerateGrafts(GRAFT_TYPE.COMBAT)
                    quest.param.last_graft_restock = TheGame:GetGameState():GetDateTime()
                end
            end
        end,
    },

}

:AddLocationCast{
    cast_id = "home",
    cast_fn = function(quest, t)
        table.insert(t,  TheGame:GetGameState():GetLocation("GB_BLACK_MARKET"))
    end,
}


:AddCastByAlias{
    cast_id = "plocka",
    alias = "PLOCKA",
}




:AddObjective{
    state = QSTATUS.ACTIVE,
    id = "gain_access",
    title = "Investigate the Black Market",
    desc = "You've heard that the Jakes are running a Black Market here in Grout Bog.",
    hide_in_overlay = true,
    low_priority = true,
    mark = {"home"},

    on_complete = function(quest)
        quest:Activate("visit")
    end,
}


:AddObjective{
    id = "visit",
    title = "Shop at the Black Market",
    desc = "Buy powerful negotiation and battle cards from the Jakes.",
    hide_in_overlay = true,
    low_priority = true,
    mark = {"home"}
}

QDEF:AddConvo("gain_access")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("home") end)
        :Loc{
            DIALOG_INTRO_NO_GUARD = [[
                * You arrive at the black market. There's no guard posted, so you walk straight in.
            ]],
            DIALOG_INTRO = [[
                {not liked?
                    * You approach the market, but an armed guard seeps out of the shadows to cut you off.
                    player:
                        !left
                    agent:
                        !right
                        $angryDefault
                        This establishment is for members only.
                        $angryHostile
                        You turn around, now, and go back the way you came.
                }
                
                {liked?
                    * You approach the market, but an armed guard seeps out of the shadows to cut you off.
                    player:
                        !left
                    agent:
                        !right
                        Oh, hey {player}.
                        Sorry, but I can't let you in here.
                        Members only, and all that.
                }
            ]],
            
            OPT_BUY_A_MEMBERSHIP = "Buy a 'membership'",
            DIALOG_BUY_A_MEMBERSHIP = [[
                player:
                    !give
                    $miscAlluring
                    Yes, well, I've been meaning to come by and pay my membership dues. 
                    I believe this should cover it, no?
                * {agent} considers the money for a long moment.
                agent:
                    ...
                    !happy
                    $happyThanks
                    Works for me. Welcome to the club, softside. 
                    !cruel
                    Hope you don't get fleeced.
            ]],

            OPT_NEGOTIATE_YOUR_WAY_IN = "Convince {agent} to let you in",
            DIALOG_NEGOTIATE_YOUR_WAY_IN = [[
                player:
                    !interest
                    $miscPersuasive
                    As clandestine as you make it sound, this is, ultimately, a business.
            ]],
            DIALOG_NEGOTIATE_YOUR_WAY_IN_SUCCESS = [[
                player:
                    My work takes me all over the continent, my friend. Sometimes all the way to Rentoria. 
                    I'm a member worth having, with connections in every faction. 
                agent:
                    !dubious
                    $neutralDubious
                    You a spy, then?
                player:
                    !happy
                    ...
                agent:
                    ... 
                    !point
                    Fine. Conditional membership granted. 
                    But you better have shills to spill.
            ]],
            DIALOG_NEGOTIATE_YOUR_WAY_IN_FAIL = [[
                player:   
                    !thumb  
                    I'm an important man. If you only knew—
                agent:
                    !angry_accuse
                    $angryDefault
                    What I <i>know</i> is that you stink like a yote and you got a swagger like you've been in charge your whole life. 
                    Admiralty? Spark Baron? I don't care. You're bad for business.
            ]],

            OPT_DROP_FRIEND_NAME = "Mention a friend who is a member...",
            OPT_MENTION_NAME = "{1#agent}",
            REQ_NO_FRIENDS = "You don't have any friends who are already members",

            DIALOG_NAME_DROP_FRIEND = [[
                player:
                    $miscPersuasive
                    A good friend of mine, {1.name}, told me about this place. I believe {1.heshe}'s a member.
                agent:
                    $neutralDefault
                    !thought
                    {1.name}, huh? {1.HeShe} did mention something about a referral.
                    ...
                    $angryHostile
                    !angry_accuse
                    Okay, you can come in. But don't give me a reason to revoke your access, or it'll be {1.name} who pays the price.
            ]],

            OPT_COME_BACK_LATER = "Leave for now",
            DIALOG_COME_BACK_LATER = [[
                player:
                    !overthere
                    $angryReluctant
                    I'll leave unsatisfied for now. But you'll regret this later.
                agent:
                    !shrug
                    $neutralWhatever
                    Uh huh.
                player:
                    !exit
            ]]
        }
        :SetLooping()
        :Fn(function(cxt) 

            local guard = cxt.location:GetAgentByRole(CHARACTER_ROLES.GUARD)
            if cxt:FirstLoop() then
                if not guard then
                    cxt:Dialog("DIALOG_INTRO_NO_GUARD")
                    --uhh.... ok? Can this happen?
                    -- ccc: Crash report, came in: Yes it can.
                    cxt.quest:Complete("gain_access")
                    StateGraphUtil.AddEndOption(cxt)
                    return
                end

                cxt.enc.scratch.names_to_drop = cxt.player.social_connections:GetConnections(function(agent, rel) return rel > RELATIONSHIP.NEUTRAL and agent:GetFactionID() == "JAKES" and agent ~= guard end)
                cxt.enc:SetPrimaryCast(guard)
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")

            
            cxt:Opt("OPT_NEGOTIATE_YOUR_WAY_IN")
                :Dialog("DIALOG_NEGOTIATE_YOUR_WAY_IN")
                :Negotiation{
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_NEGOTIATE_YOUR_WAY_IN_SUCCESS")
                        cxt.quest:Complete("gain_access")
                        StateGraphUtil.AddEndOption(cxt)
                    end,
                    on_fail = function(cxt) 
                        cxt:Dialog("DIALOG_NEGOTIATE_YOUR_WAY_IN_FAIL")
                    end,
                }

            
            cxt:Opt("OPT_BUY_A_MEMBERSHIP")
                :Dialog("DIALOG_BUY_A_MEMBERSHIP")
                :DeliverMoney(100)
                :CompleteQuest("gain_access")
                :End()

            cxt:Opt("OPT_DROP_FRIEND_NAME")
                :ReqCondition(#cxt.enc.scratch.names_to_drop > 0, "REQ_NO_FRIENDS")
                :LoopingFn(function() 
                    for _, agent in ipairs(cxt.enc.scratch.names_to_drop) do
                        cxt:Opt("OPT_MENTION_NAME", agent)
                            :Dialog("DIALOG_NAME_DROP_FRIEND", agent:LocTable())
                            :CompleteQuest("gain_access")
                            :End()
                    end
                    StateGraphUtil.AddBackButton(cxt)
                end)
            cxt:Opt("OPT_COME_BACK_LATER")
                :Dialog("DIALOG_COME_BACK_LATER")
                :Travel()
        end)


    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask {agent} about this place",
        [[
            player:
                $neutralGreeting
                I'm a potential customer, you know. What do you sell?
            agent:
                $neutralDirect
                Things. 
                $neutralDirect
                To people. 
                $angryDefault
                Who are members, unlike you.
        ]],
        "Ask about the Jakes",
        [[
            player:
                $neutralGreeting
                I didn't realize Jakes were allowed in Grout Bog.
            agent:
                !crossed
                Your freedom to walk away from this without trouble is growing smaller by the minute. 
        ]],
        "Ask about getting a membership",
        [[
            player:
                !placate
                $miscPersuasive
                Slow down, my friend. I <i>want</i> to play by the rules.
                How would I go about acquiring membership myself?
            agent:
                $neutralDirect
                It's easy, if you've been invited.
                $neutralDirect
                But you haven't.
                $angryHostile
                So it isn't.
        ]],
    })


QDEF:AddConvo("visit")
    :AttractState("STATE_ATTRACT", function(cxt) return cxt.location == cxt.quest:GetCastMember("home") and cxt.quest:GetCastMember("home"):GetAgentByRole(CHARACTER_ROLES.GUARD) == cxt:GetAgent() and not cxt.quest.param.guard_intro end)
        :Loc{
            DIALOG_FIRST_INTRO = [[
                agent: 
                    $neutralResigned
                    Just because I let you in doesn't mean I trust you.
                    $angryHostile
                    Break anything here and I'll break <b>you</>.
            ]],
        }

        :Fn(function(cxt) 
            cxt.quest.param.guard_intro = true
            cxt:Dialog("DIALOG_FIRST_INTRO")
        end)


QDEF:AddConvo("visit")
    :Loc{
        DIALOG_REMOVE_BATTLE_CARD = [[
            agent:
                $neutralThoughtful
                Let's see what we can do...
        ]],
        DIALOG_REMOVE_NEGOTIATION_CARD = [[
            agent:
                $neutralThoughtful
                I can lighten your load.
        ]],
    }
    :Hub(function(cxt) 
        if cxt.location == cxt.quest:GetCastMember("home") and cxt.quest:GetCastMember("home"):GetAgentByRole(CHARACTER_ROLES.PROPRIETOR) == cxt:GetAgent() then

            StateGraphUtil.AddRemoveBattleCardOption( cxt, "DIALOG_REMOVE_BATTLE_CARD" )

            StateGraphUtil.AddRemoveNegotiationCardOption( cxt, "DIALOG_REMOVE_NEGOTIATION_CARD" )
        end
    end)
    :Priority(CONVO_PRIORITY_HIGH)
    :AttractState("STATE_ATTRACT", function(cxt) return cxt.location == cxt.quest:GetCastMember("home") and cxt.quest:GetCastMember("home"):GetAgentByRole(CHARACTER_ROLES.PROPRIETOR) == cxt:GetAgent() end)
        :Loc{
            DIALOG_FIRST_INTRO = [[
                agent:
                    $neutralThoughtful
                    You got past security, eh?
                    If they're cool with you, so am I.
                    Let's get down to business, then.
            ]],
            DIALOG_INTRO = [[
                agent:
                    $happyGreeting
                    I've got the best goods this side of the Bread Fields.
                    And by best, I mean only. Ha!
            ]]
        }

        :Fn(function(cxt) 
            if not cxt.quest.param.vendor_intro then
                cxt.quest.param.vendor_intro = true
                cxt:Dialog("DIALOG_FIRST_INTRO")
            else
                cxt:Dialog("DIALOG_INTRO")
            end
        end)


------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo(nil, "plocka")
        
        :Loc{
            OPT_SELL_GRAFT = "Purchase {1#graft}",
            DIALOG_SELL_GRAFT = [[
                agent:
                    $happyThanks
                    Good choice. You won't regret it.
            ]],
            REQ_FULL = "You have too many grafts of this type",
            
            OPT_SEE_NEGOTIATION_GRAFTS = "Buy negotiation grafts...",
            OPT_SEE_COMBAT_GRAFTS = "Buy battle grafts...",
            OPT_GET_MORE_SLOTS = "Add more graft slots...",
            DIALOG_GET_MORE_SLOTS = [[
                agent:
                    I can drill you a new slot, but I'm not going to lie:
                    !interest
                    It's going to hurt like hell.
            ]],
            DIALOG_ADD_SLOT_1 = [[
                agent:
                    This is going to sting a little.
                    !exit
                player:
                    !exit
            ]],
            DIALOG_ADD_SLOT_2 = [[
                agent:
                    !right
                player:
                    !left
                agent:
                    Good as new!                    

            ]],
            OPT_ADD_NEGOTIATION_SLOT = "Add a negotiation graft slot",
            OPT_ADD_COMBAT_SLOT = "Add a battle graft slot",
            REQ_TOO_MANY = "You have the maximum allowed!",
            OPT_ASK_ABOUT_SPARK_BARONS = "Ask {agent} what {agent.heshe} is doing here",
            DIALOG_ABOUT_SPARK_BARONS = [[
                player:
                    You're a member of the Spark Barons, are you not?
                agent:
                    I am.
                player:
                    And this is an illegal, black-market operation run by the Jakes, is it not?
                agent:
                    It is.
                player:
                    How do you reconcile that? Shouldn't you be reporting this to your superiors?
                agent:
                    ...
                    I contain multitudes.
            ]]

        }
        :AttractState("STATE_ATTRACT", function(cxt) return cxt.location == cxt.quest:GetCastMember("home") end)
            :Loc{
                DIALOG_INTRO = [[
                    player:
                        !left
                    agent: 
                        !right
                ]],
                DIALOG_FIRST_TIME = [[
                    * You're surprised to see a Spark Baron here, selling grafts.
                    player:
                        !left
                    agent:
                        !right
                        !overthere
                        $miscPersuasive
                        You look like someone who could use an edge. I have several available-for a price.
                ]],
                DIALOG_RETURNING = [[
                    * {agent} is tuning up one of {agent.hisher} black-market grafts.
                    agent:
                        !overthere
                        $miscPersuasive
                        I keep telling you—these grafts are made for someone of your talents.
                ]],
                DIALOG_LEAVE = [[
                    agent:
                        $happyThanks
                        Until next time.
                ]],

            }
            :Fn(function(cxt) 
                    -- cxt:Dialog("DIALOG_INTRO")
                    if cxt.quest.param.graft_intro or cxt:GetAgent():KnowsPlayer() then
                        cxt:Dialog( "DIALOG_RETURNING" )
                    else
                        cxt:Dialog( "DIALOG_FIRST_TIME" )
                        cxt.quest.param.graft_intro = true
                    end
                end)

        :Hub( function(cxt, who)

            if not cxt.quest.param.asked_about_barons then
                cxt:Opt("OPT_ASK_ABOUT_SPARK_BARONS")
                    :Dialog("DIALOG_ABOUT_SPARK_BARONS")
                    :Fn(function() cxt.quest.param.asked_about_barons = true end)
            end

            cxt:Opt("OPT_SEE_NEGOTIATION_GRAFTS")
                :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.NEGOTIATION )
                :Fn(PresetGrafts(cxt.quest.param.negotiation_grafts))
            
            cxt:Opt("OPT_SEE_COMBAT_GRAFTS")
                :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.FIGHT )
                :Fn(PresetGrafts(cxt.quest.param.combat_grafts))

            cxt:Opt("OPT_GET_MORE_SLOTS")
                :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.OBJECTIVE )
                :Dialog("DIALOG_GET_MORE_SLOTS")
                :Fn(function(cxt)

                        ConvoUtil.OptBuyGraftSlot( cxt, GRAFT_TYPE.NEGOTIATION, "DIALOG_ADD_SLOT_1" )
                            :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.NEGOTIATION )
                            :Dialog( "DIALOG_ADD_SLOT_2" )

                        ConvoUtil.OptBuyGraftSlot( cxt, GRAFT_TYPE.COMBAT, "DIALOG_ADD_SLOT_1" )
                            :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.FIGHT )
                            :Dialog( "DIALOG_ADD_SLOT_2" )
                        
                        StateGraphUtil.AddBackButton(cxt)  
                    end)
        end )
