
local QDEF = QuestDef.Define{
    qtype = QTYPE.STORY,
    title = "Bug Hunter",
    icon = engine.asset.Texture("icons/quests/rook_story_bog_monster.tex"),
    on_init = function(quest)
        quest.param.handler_faction = math.random() > .5 and "RISE" or "SPARK_BARONS"
        quest:AssignCastMember("handler_second")
    end,
}

:AddCast{
    cast_id = "handler_second",
    when = QWHEN.MANUAL,        
    cast_fn = function(quest, t) 
        table.insert(t, TheGame:GetGameState():GetAgentByAlias(quest.param.handler_faction == "RISE" and "RISE_SECOND" or "SPARK_SECOND") )
    end,
    no_validation = true,
    proivider = true,
    on_assign = function(quest, agent)
        agent:OpinionEvent(quest:GetQuestDef():GetOpinionEvent("has_heard_rumours_about_you"))
    end,
}

:AddLocationCast{
    cast_id = "bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"))
    end,
}

:AddObjective{
    id = "introduction",
    title = "Go Home",
    desc = "The hour is late. Time to recharge the old batteries.",
    state = QSTATUS.ACTIVE,
    on_complete = function(quest) 
        quest:Activate("kill_the_flead_queen")
    end,
}


:AddCastByAlias{
    cast_id = "baron_contact",
    alias = "SPARK_CONTACT",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "rise_contact",
    alias = "KALANDRA",
    no_validation = true,
}    


:AddObjective{
    id = "kill_the_flead_queen",
    title = "Kill the Flead Queen",
    mark = {"flead_queen_staging"},
    on_activate = function(quest)
        quest:AssignCastMember("flead_queen_staging")
        quest:AssignCastMember("flead_queen_lair")
    end,
}

:AddLocationDefs{
    FLEAD_STAGING_SITE = {
        name = "Staging Site",
        plax = "EXT_Bog_Outcrop_01",
        indoors = false,
        map_tags = {"bog"},
    },


    FLEAD_QUEEN_LAIR = {
        name = "Flead Site",
        --desc = "This is a hideout that the spree uses as a staging-ground for attacks on the Admiralty.",
        plax = "INT_Bog_Cave_01",
        indoors = true,
        map_tags = {"bog"},
    },
}

:AddLocationCast{
    cast_id = "flead_queen_lair",
    when = QWHEN.MANUAL,        
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("FLEAD_QUEEN_LAIR"))
    end,
}
:AddLocationCast{
    cast_id = "flead_queen_staging",
    when = QWHEN.MANUAL,        
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("FLEAD_STAGING_SITE"))
    end,
}

:AddOpinionEvents{
    has_heard_rumours_about_you = 
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Has heard you are working with their enemies",
    },
    got_their_people_killed =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Got their people killed",
    },
    dealt_with_a_monster =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Killed a monster for them",
    },
}


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("introduction")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("bar") end)
        :Loc{
            
            DIALOG_INTRO_RISE = [[
                * {agent} emerges from the back of the bar.
                agent:
                    !right
                    Grifter, over here. I have a task for you.
                player:
                    !left
                    Last I checked, my employer is {rise_contact}, not you.
                agent:
                    Last <i>I</> checked, folks say you've been working with the Barons between jobs for {rise_contact}.
                    Now {rise_contact} is very busy, so I doubt {rise_contact.heshe}'s heard the same rumors that I have.
                    !cruel
                    Yet.
                *** {agent} is on to your double-act, and is shaking you down.
            ]],

            DIALOG_RISE_FLEAD = [[
                player:
                    So what is it you want?
                agent:
                    I need a mess cleaned up.
                    We bought some flead eggs from the Spree in Murder Bay. The plan was to slow down the Baron dig sites with the bugs. 
                    Treat the workers right, the bugs don't show up, and the quotas get filled.
                player:
                    !dubious
                    You <i>willingly</> introduced fleads to the Bog?
                agent:
                    Turns out one of those eggs was a queen.
                    !shrug
                    Honest mistake.
                    But now I need it dead before the Barons find it.
                    They'll call in the big guns if they need to deal with it, and I don't want to face those big guns afterwards.
            ]],

            DIALOG_INTRO_SPARK = [[
                * {agent} emerges from the back of the bar.
                agent:
                    !right
                    Grifter. We have reason to speak.
                player:
                    !left
                    We do?
                agent:
                    I have heard disquieting rumors that you have been less than forthright in your dealings with {baron_contact}.
                player:
                    !happy
                    Rumors are like birds, always flying about. I wouldn't pay them any-
                agent:
                    Drop the act, grifter. I am immune to your folksy charms. I put out inquiries about you and I know all about your past.
                    I know who you are, and what you've done.
                *** {agent} is on to your double-act, and is shaking you down.
            ]],

            DIALOG_SPARK_FLEAD = [[
                player:
                    !cruel
                    You might know what I've done. But you have no idea what I can do.
                    !happy
                    I do respect an honest shakedown, though. 
                    !neutral
                    What is it you need done?
                agent:
                    One of our dig sites has been experiencing disruptions. Fleads have been attacking the workers.
                    We thought we had it under control, but the last attack revealed the source - a queen.
                    I want you to go eradicate the problem. Quietly. I don't want to panic the workers and give them any more of an excuse to shirk their duty.
            ]],
            
            DIALOG_SETOUT_RISE = [[
                player:
                    I suppose I don't have much choice in the matter, do I?
                agent:
                    Ha! You know the score!
                    There's some of our brothers and sisters keeping the flead at bay at the site.
                    Maybe they'll help you. But probably not.
                    I'll find you when it's over.
            ]],
            
            DIALOG_SETOUT_SPARK = [[
                player:
                    I'll do it, but only because I have to.
                agent:
                    We have a containment team at the location. Meet with them, and they will show you how to access the queen.
                    I will find you when the task is completed.
            ]],

            OPT_SET_OUT = "Set out for the bog",
        }
        :Fn(function(cxt)
            cxt:GetCastMember("handler_second"):MoveToLocation(cxt.location)
            cxt:TalkTo("handler_second")
            cxt:Dialog(cxt.quest.param.handler_faction == "RISE" and "DIALOG_INTRO_RISE" or "DIALOG_INTRO_SPARK")
            
            
            cxt:Dialog(cxt.quest.param.handler_faction == "RISE" and "DIALOG_RISE_FLEAD" or "DIALOG_SPARK_FLEAD")

            
            cxt:RunLoop(function()
                
                cxt:AskAbout(cxt.quest.param.handler_faction == "RISE" and "RISE_QUESTIONS_FLEAD" or "BARON_QUESTIONS_FLEAD")
                
                
                cxt:Opt("OPT_SET_OUT")
                    :Dialog(cxt.quest.param.handler_faction == "RISE" and "DIALOG_SETOUT_RISE" or "DIALOG_SETOUT_SPARK")  
                    :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                    :CompleteQuest("introduction")
                    :End()
            end)

        end)

    :AskAboutHub("RISE_QUESTIONS_FLEAD", 
    {

        "Ask {agent} about the flead queen",
        [[
            player:
                I've seen flead before, but not a queen. What are we up against here?
            agent:
                It's big. And nasty. 
                !shrug
                It'll probably die if you shoot it enough.
            player:
                Very enlightening, thanks.
        ]],
        "Ask about the egg plan",
        [[
            player:
                Flead are nasty. Why would you endanger your own people with those eggs?
            agent:
                We gave the workers warning. The only folks in real danger were the Baron extermination squads.
                And now you, I guess.
        ]],

    })

    :AskAboutHub("BARON_QUESTIONS_FLEAD", 
    {

        "Ask {agent} about the flead queen",
        [[
            player:
                What exactly is a flead queen?
            agent:
                Flead live in communal nests deep underground. Each nest has a queen at its center.
                The queen produces the nest's young, and has a psionic link with all of her offspring.
                I don't know what they look like. Nobody who's seen the one at the worksite has survived to tell us.
        ]],
        "Ask how the flead got into the bog",
        [[
            player:
                Flead aren't from around here. How'd they get past your security wall?
            agent:
                The Rise are nothing if not resourceful. 
                Fleads are big, but their eggs are durable, and small enough to eat. They probably hid them internally and walked through the front gate.
            player:
                That's unsettling.
            agent:
                !shrug
                You asked.
        ]],

    })



local PRE_FIGHT = function(cxt)
    local party
    
    if cxt.quest.param.handler_faction == "RISE" then
        party = {"RISE_PAMPHLETEER", "RISE_RADICAL", "RISE_REBEL", "RISE_AUTODOG", "RISE_AUTOMECH"}
    else
        party = {"SPARK_BARON_TASKMASTER", "SPARK_BARON_PROFESSIONAL", "SPARK_BARON_GOON", "AUTODOG", "SPARK_BARON_AUTOMECH"}
    end

    --spawn some rise
    cxt.quest.param.helpers = CreateCombatParty( party, cxt.quest:GetRank(), cxt.location )
    cxt:TalkTo(cxt.quest.param.helpers[1])
    cxt:Dialog("DIALOG_INTRO")

    local fight_allies = {}
    for k,v in ipairs(cxt.quest.param.helpers) do
        if k > 1 then
            table.insert(fight_allies, v)
        end
    end

    cxt:RunLoop(function()
        cxt:AskAbout(cxt.quest.param.handler_faction == "RISE" and "RISE_QUESTIONS" or "BARON_QUESTIONS")

        cxt:Opt("OPT_CONVINCE_HELP")
            :Dialog("DIALOG_CONVINCE_HELP")
            :Negotiation{
                flags = NEGOTIATION_FLAGS.ALLY_GRAB | NEGOTIATION_FLAGS.NO_CORE_RESOLVE,
                fight_allies = fight_allies,
                reason_txt = cxt:GetLocString("NEGOTIATION_REASON")
            }
                :OnFailure()
                    :Dialog("DIALOG_FAIL_NEGOTIATION")
                :OnSuccess()
                    :Dialog("DIALOG_SUCCESS_NEGOTIATION")
                    :Fn(function(cxt, minigame) 
                        
                        cxt.quest.param.fight_allies = {}
                        for i, modifier in minigame:GetPlayerNegotiator():Modifiers() do
                            if modifier.id == "FIGHT_ALLY_WON" and modifier.ally_agent then
                                table.insert( cxt.quest.param.fight_allies, modifier.ally_agent )
                            end
                        end
                        cxt:Opt("OPT_ATTACK_THE_QUEEN")
                            :Dialog("DIALOG_ATTACK_QUEEN_TEAM")
                            :GoTo("STATE_ATTACK_FLEAD_QUEEN")
                    end)

        cxt:Opt("OPT_ATTACK_THE_QUEEN")
            :Dialog("DIALOG_ATTACK_QUEEN_ALONE")
            :GoTo("STATE_ATTACK_FLEAD_QUEEN")
    end)
end

QDEF:AddConvo("kill_the_flead_queen")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("flead_queen_staging") end)
        :Fn(function(cxt) 
            cxt:GoTo(cxt.quest.param.handler_faction == "RISE" and "STATE_PRE_FIGHT_RISE" or "STATE_PRE_FIGHT_BARONS")
        end)

    :State("STATE_PRE_FIGHT_RISE")
        :Loc{
            
            DIALOG_INTRO = [[
                * You find some Rise members guarding the entrance to a small cave.
                * Unsettling noises occasionally rise up from its mouth, to which the Rise responds by blindly firing shots into the darkness.
                player:
                    !left
                agent:
                    !right
                    Are you the relief?
                    Oh Hesh am I glad to see you. We're running low on ammo, and the thing down there's getting angrier.

            ]],
            OPT_ATTACK_THE_QUEEN = "Attack the Flead Queen",
            NEGOTIATION_REASON = "Collect allies and then play the ending card to start the fight!",
            OPT_CONVINCE_HELP = "Convince {agent} to spare some fighters",
            DIALOG_CONVINCE_HELP = [[
                player:
                    I'm going to take down that queen, but I need backup.
                agent:
                    Whoa, now - we didn't sign up for that.
            ]],
            DIALOG_SUCCESS_NEGOTIATION = [[
                agent:
                    Fine, I can spare some fighters.
                    But <b>I'm</> not going down there.
            ]],
            DIALOG_FAIL_NEGOTIATION = [[
                agent:
                    No way. Our job is up here. You go do yours.
            ]],

            DIALOG_ATTACK_QUEEN_ALONE = [[
                * You head into the cave without the help of the Rise.
                player:
                    !exit
                agent:
                    !cruel
                    Good luck in there!
            ]],

            DIALOG_ATTACK_QUEEN_TEAM = [[
                * You signal to the 'volunteers', and head into the cave.
                player:
                    !exit
                agent:
                    Good luck in there!
            ]],

        }
        :Fn(PRE_FIGHT)

    :AskAboutHub("RISE_QUESTIONS", 
    {

        "Ask {agent} about cave",
        [[
            player:
                What's down there?
            agent:
                We were told it's a Flead queen.
                !shrug
                Whatever it is, it's big. And mean.
                You're going to take care of it for us, right?
        ]],
        "Ask for help",
        [[
            player:
                Are you going to provide backup?
            agent:
                Of course.
                !laugh
                I'm going to be "back up" here!
                !neutral                
                Seriously, though. I am <b>not</> going in that cave.
        ]],

    })
    
    :State("STATE_PRE_FIGHT_BARONS")
        :Loc{
            DIALOG_INTRO = [[
                * You find the cave entrance. A small detachment of Spark Barons has established a perimeter around it.
                * Unsettling noises occasionally rise up from the depths. The barons respond with blaster fire, and the occasional grenade.
                player:
                    !left
                agent:
                    !right
                    The grifter is here!
                player:
                    !left
                agent:
                    {handler_second} said to expect you. 
                    !overthere
                    The queen is down there. Good luck.
            ]],
            OPT_ATTACK_THE_QUEEN = "Attack the Flead Queen",
            NEGOTIATION_REASON = "Collect allies and then play the ending card to start the fight!",
            OPT_CONVINCE_HELP = "Convince {agent} to provide backup",
            
            DIALOG_CONVINCE_HELP = [[
                player:
                    If it's taking all of you to keep the queen pinned down, surely it will take more than just me to destroy it.
                    I need some volunteers to go down with me.
                agent:
                    I don't get paid enough for that. 
                    I might send some of my underlings, if you make a strong enough case.
            ]],
            DIALOG_SUCCESS_NEGOTIATION = [[
                agent:
                    Take the help, and get the job done.
                    I'll be up here covering the exit.
            ]],
            DIALOG_FAIL_NEGOTIATION = [[
                agent:
                    I think I'll keep my forces above ground. There's too much paperwork to fill out if I lose anyone.
            ]],

            DIALOG_ATTACK_QUEEN_ALONE = [[
                * You stride into the cave without the help of the Barons.
                player:
                    !exit
            ]],

            DIALOG_ATTACK_QUEEN_TEAM = [[
                * You rally the volunteers, and head into the cave.
                player:
                    !exit
                agent:
                    Don't get my people killed! 
                    I hate paperwork!
            ]],

        }
        :Fn(PRE_FIGHT)
        
    :AskAboutHub("BARON_QUESTIONS", 
    {
        "Ask {agent} about cave",
        [[
            player:
                What are we up against here?
            agent:
                <i>We</> are up against a hole in the ground.
                <b>You</> are up against a flead queen.
                And probably a not insubstantial quantity of flead and flead larvae.
                But you'll do fine, I'm sure.
        ]],
        "Ask for help",
        [[
            player:
                I'm going to need some help killing this beast.
            agent:
                I think you're misunderstanding the situation here.
                You <b>are</> the help.
                So go kill the beast, will you?
        ]],

    })    

    
    :State("STATE_ATTACK_FLEAD_QUEEN")
        :Loc{
            
            DIALOG_INTRO = [[
                player:
                    !exit
                right:
                    !exit

                * You follow the low, guttural sounds deep into the cave.
                player:
                    !left
                * Turning a corner, you find yourself in the queen's brood chamber.
                    
                agent:
                    !right                    
                player:
                    !fight
                * Hundreds of hungry eyes snap to attention as the queen rises on impossibly small wings to meet you.
            ]],
            DIALOG_BEAT_FLEAD_QUEEN = [[
                right:
                    !exit
                * The flead queen collapses in a pile of chitin and ichor.
                player:
                    !exit
                * The stench of it all chases you out of the cave.
            ]],
            
            OPT_ATTACK_THE_QUEEN = "Attack the Flead Queen",

            DIALOG_TALK_TO_HELPER_SOLO_RISE = [[
                player:
                    !left
                agent:
                    !right
                    You survived that?!
                * {agent} hands a fistful of shills to one of {agent.hisher} underlings.
                agent:
                    I lost money on that bet, grifter.
                player:
                    Sorry to disappoint you with my competence!
            ]],
            DIALOG_TALK_TO_HELPER_SOLO_BARONS = [[
                player:
                    !left
                agent:
                    !right
                    You survived that?!
                * {agent} looks you up and down appraisingly as you wipe the flead ichor off of your metal leg. 
                agent:
                    Seems like I underestimated you. 
                player:
                    That's alright, I'm used to it.
            ]],
            DIALOG_TALK_TO_HELPER_SURVIVED_RISE = [[
                player:
                    !left
                agent:
                    !right
                * {agent} does a quick headcount as your party emerges from the cave.
                * Satisfied with the result, {agent.heshe} nods to you.
                agent:
                    So it's dead, then.
                player:
                    It is. And all of your people have been returned to you in one piece.
                agent:
                    Good work, grifter. I was right to bet on you.
            ]],
            DIALOG_TALK_TO_HELPER_SURVIVED_BARONS = [[
                player:
                    !left
                agent:
                    !right
                * Back topside, {agent}'s minions reassemble.
                * {agent.HeShe} does a quick roll call, and finding no-one missing, extends their hand to you.
                agent:
                    Excellent work, grifter. I couldn't have done it better myself.
                player:
                    Truer words you've never spoken.
            ]],
            DIALOG_TALK_TO_HELPER_DIED_RISE = [[
                player:
                    !left
                agent:
                    !right
                * You limp back out of the cave, leaving the bodies of the fallen behind.
                agent:
                    What happened down there?
                player:
                    The beast is dead.
                agent:
                    !angry
                    Yeah, but that's not all that's dead. You wasted what I gave you. I won't forget this.
                    !exit
                * {agent} storms off in anger.
            ]],
            DIALOG_TALK_TO_HELPER_DIED_BARONS = [[
                player:
                    !left
                agent:
                    !right
                * You limp back out of the cave, leaving the bodies of the fallen behind.
                agent:
                    Where are my people?
                player:
                    There were casualties. 
                agent:
                    !angry
                    Hesh damn it. Now I have paperwork to fill out. I told you that I hate paperwork!
                    !exit
                * {agent} storms off in anger.
            ]],

            DIALOG_HANDLER_TALK_BARONS = [[
                * {handler_second} arrives on the scene.
                handler_second:
                    !right
                    You got the job done, grifter.
                    I am thankful. 
                    But I will be watching. If the rumors of your dalliance with the Rise turn out to be more than rumors, I will be forced to execute you.
                player:
                    !dubious
                    You're welcome?
                handler_second:
                    !give
                    But I am getting ahead of myself. Here, take this as a token of our working relationship.
            ]],
            DIALOG_HANDLER_TALK_BARONS_2 = [[
                player:
                    Much appreciated.
                handler_second:
                    Don't let it go to waste.
                    !exit
                * {handler_second} leaves, leaving the rest of the Barons to clean up the mess.
            ]],
            DIALOG_HANDLER_TALK_RISE = [[
                * {handler_second} arrives on the scene.
                handler_second:
                    !right
                    It's dead?
                player:
                    It is.
                handler_second:
                    And you are not.
                    That is good, I guess.
                    But if I find out that you <i>are</> working with the Barons, I will gut you myself.
                player:
                    That's... Charming.
                handler_second:
                    !give
                    I forget my manners. You helped me help the Rise, so I owe you something. Here, take this.

            ]],
            DIALOG_HANDLER_TALK_RISE_2 = [[
                player:
                    !happy
                    That's surprisingly generous of you, {handler_second}. Are you warming up to me?
                handler_second:
                    Don't make me tear it out of you again.
                    !exit
                * {handler_second} leaves, leaving the rest of the Rise to clean up the mess.
            ]],

        }
        :Fn(function(cxt)
            
            cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("flead_queen_lair") )

            local queen = TheGame:GetGameState():AddAgent(Agent("FLEAD_QUEEN"))
            cxt:TalkTo(queen)

            cxt:Dialog("DIALOG_INTRO")
                
            cxt:Opt("OPT_ATTACK_THE_QUEEN")
                :Battle{ 
                    allies = cxt.quest.param.fight_allies,
                    flags = BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.NO_BURRS,
                }    
                :OnWin()
                    :Fn(function() 
                        local had_allies = cxt.quest.param.fight_allies and #cxt.quest.param.fight_allies > 0
                        local all_allies_survived = true
                        if had_allies then
                            for k,v in ipairs(cxt.quest.param.fight_allies) do
                                if v:IsDead() and v:IsSentient() then
                                    all_allies_survived = false
                                    break
                                end
                            end
                        end
                        cxt:Dialog("DIALOG_BEAT_FLEAD_QUEEN")
                        cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("flead_queen_staging") )
                        cxt:TalkTo(cxt.quest.param.helpers[1])
                        if not had_allies then
                            cxt:Dialog(cxt.quest.param.handler_faction == "RISE" and "DIALOG_TALK_TO_HELPER_SOLO_RISE" or "DIALOG_TALK_TO_HELPER_SOLO_BARONS")
                        elseif all_allies_survived then
                            cxt:Dialog(cxt.quest.param.handler_faction == "RISE" and "DIALOG_TALK_TO_HELPER_SURVIVED_RISE" or "DIALOG_TALK_TO_HELPER_SURVIVED_BARONS")
                        else
                            cxt:Dialog(cxt.quest.param.handler_faction == "RISE" and "DIALOG_TALK_TO_HELPER_DIED_RISE" or "DIALOG_TALK_TO_HELPER_DIED_BARONS")
                            cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("got_their_people_killed"))
                        end
                        
                        cxt:GetCastMember("handler_second"):MoveToLocation(cxt.location) 
                        cxt:Dialog(cxt.quest.param.handler_faction == "RISE" and "DIALOG_HANDLER_TALK_RISE" or "DIALOG_HANDLER_TALK_BARONS")
                        cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("dealt_with_a_monster"))
                    end)
                    
                    :Fn(function() ConvoUtil.GiveBossRewards(cxt, GRAFT_TYPE.COMBAT) end)
                    :Dialog(cxt.quest.param.handler_faction == "RISE" and "DIALOG_HANDLER_TALK_RISE_2" or "DIALOG_HANDLER_TALK_BARONS_2")

                    :CompleteQuest()
                    :Travel()
        end)
