
local QDEF = QuestDef.Define{
    qtype = QTYPE.STORY,
    title = "System Debogging",
    icon = engine.asset.Texture("icons/quests/rook_story_bog_robot.tex"),
    on_init = function(quest)
        local is_test = TheGame:GetLocalSettings().ROBOTICS or quest.param.handler_faction ~= nil
        quest.param.handler_faction = quest.param.handler_faction or ( math.random() > .5 and "RISE" or "SPARK_BARONS" )
        quest:AssignCastMember("handler_second")
        if is_test then
            TheGame:GetGameState():GetCaravan():MoveToLocation(quest:GetCastMember("bar"))
        end


    end,
}

:AddCast{
    cast_id = "handler_second",
    when = QWHEN.MANUAL,        
    cast_fn = function(quest, t) 
        table.insert(t, TheGame:GetGameState():GetAgentByAlias(quest.param.handler_faction == "RISE" and "RISE_SECOND" or "SPARK_SECOND") )
    end,
    no_validation = true,
    proivider = true,
    on_assign = function(quest, agent)
        agent:OpinionEvent(quest:GetQuestDef():GetOpinionEvent("has_heard_rumours_about_you"))
    end,
}

:AddLocationCast{
    cast_id = "bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"))
    end,
}

:AddObjective{
    id = "introduction",
    title = "Go Home",
    desc = "The hour is late. Time to recharge the old batteries.",
    state = QSTATUS.ACTIVE,
    on_complete = function(quest) 
        quest:Activate("kill_the_robot")
    end,
}

:AddCastByAlias{
    cast_id = "baron_contact",
    alias = "SPARK_CONTACT",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "rise_contact",
    alias = "KALANDRA",
    no_validation = true,
}    


:AddObjective{
    id = "kill_the_robot",
    title = "Stop the Rampaging Automech",
    mark = {"robot_dig_site"},
    on_activate = function(quest)
        quest:AssignCastMember("robot_dig_site")
    end,
}

:AddLocationDefs{
    ROBOT_DIG_SITE = {
        name = "Staging Site",
        plax = "EXT_BigWorksite",
        indoors = false,
        map_tags = {"worksite"},
    },
}

:AddLocationCast{
    cast_id = "robot_dig_site",
    when = QWHEN.MANUAL,        
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("ROBOT_DIG_SITE"))
    end,
}

:AddOpinionEvents{
    has_heard_rumours_about_you = 
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Has heard you are working with their enemies",
    },
    got_their_people_killed =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Got their people killed",
    },
    dealt_with_a_monster =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Killed a monster for them",
    },
}


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("introduction")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("bar") end)
        :Loc{

            DIALOG_INTRO = [[
                * {agent} emerges from the back of the bar.
                player:
                    !left
                agent:
                    !right
                
                {rise?
                    agent:
                        Grifter. I have a situation that needs dealing with.
                    player:
                        If <i>you</> have a situation, I would suggest that <i>you</> find a way to deal with it.
                    agent:
                        !crossed
                        I have. You will deal with the situation, or {rise_contact} will hear about all the time you're spending with the Barons.
                    player:
                        !happy
                        Are you shaking me down, {agent}?
                    agent:
                        !shrug
                        Call it what you want. 
                        But cross me, and I'll crush you.
                    *** {agent} knows you're working with the Barons, and is blackmailing you. You should play along, to maintain your cover.
                }
                {spark_barons?

                    agent:
                        Grifter. We have reason to speak.
                    player:
                        Always willing to talk to a friend. Or to you, even.
                    agent:
                        I have researched your past. I have researched your connections. 
                        I have seen reports of what you do when you are not working for {baron_contact}. 
                        Your charms may work on {baron_contact.himher}, but I am far more diligent than my employer.

                    *** {agent} knows you're working with the Rise, and is blackmailing you. You should play along, to maintain your cover.
                }
            ]],

            OPT_HEAR_DEMANDS = "Hear out {agent}'s demands",
            DIALOG_INTRO_2 = [[
                {rise?
                    player:
                        !sigh
                        Alright. What is it that you want?
                    agent:
                        The Barons are testing a new digging machine.
                        If this thing works like they say it's going to, they'll be able to cut the workforce in half.
                    player:
                        So you want me to make this test fail?
                    agent:
                        No, it already has. 
                        We have a person on the inside that modified the control routines.
                    player:
                        So what's the problem?
                    agent:
                        They were too thorough. The machine's run amok.
                        If the Barons shut it down and recover the core, they'll know what we did and they'll continue testing.
                    player:
                        So you need me to cover your tracks.
                    agent:
                        Yes. With explosions, preferably.
                    *** {agent} needs you to go kill a killer robot.
                }
                
                {spark_barons?
                    player:
                        Alright, so you've done your homework. What do you want from me?
                    agent:
                        Do you know why the Barons are in the bog?
                    player:
                        I always assumed you got your kicks watching poor folks work at digging holes.
                    agent:
                        No. The work is the point, not the workers. 
                        We would be much more profitable with fewer mouths to pay, and fewer rebellious bones to break.
                    player:
                        Naturally.
                    agent:
                        To that end, we have been testing a machine for automated digging.
                        But-
                    player:
                        There's always a but, isn't there?
                    agent:
                        -but it has malfunctioned and is destroying valuable company assets. 
                        I need you to decommission the machine. 
                        By force.
                    *** {agent} wants you to shut down an automatic digging machine.
                }

            ]],
            OPT_SET_OUT = "Set out for the bog",
            DIALOG_SET_OUT = [[
                {rise?
                    player:
                        Fine. I'll do it. 
                    agent:
                        The cause thanks you.
                        !exit
                    * {agent} leaves the bar, headed back to Rise HQ, no doubt.
                }
                {spark_barons?

                }

            ]],
            
            DIALOG_ASKED_ALL_QUESTIONS = [[
                {rise?
                    agent:
                        Enough questions. Time for you to kill the machine.
                        !exit
                    * {agent} leaves the bar, headed back to Rise HQ, no doubt.
                }
                {spark_barons?
                    agent:
                        I have spent enough time talking to you. You will go perform the task now.
                        !exit
                    * {agent} leaves the building.
                }
            ]]

        }
        :Questions("SPARK_BARONS", {
            "Ask about the machine",
            [[
                player:
                    Tell me about this machine. How big is it.
                agent:
                    It's an automech that we recovered from the bog. Probably a battle unit, judging by its design.
                player:
                    You found an ancient battlemech, and you decided to use it to dig holes?
                agent:
                    Creating holes in the ground is not that different from creating holes in an opponent. 
                    In some ways, it is even easier- the ground rarely presents a moving target.
            ]],        
            "Ask about the damage",
            [[
                player:
                    You said this machine was damaging company assets. Which assets?
                agent:
                    It terminated the full shift that was working at the site. Eight laborers, and a foreman.
                    Even worse, it damaged their equipment. It will cost hundreds of shills to replace.
            ]],
            "Ask why your help is needed",
            [[
                player:
                    You Barons have a lot more firepower than I do. Why are you sending me in to do this?
                agent:
                    I <i>could</> send a patrol in to decommission the machine, but there would be casualties, and casualties mean paperwork.
                    !interest
                    And I <i>hate</> paperwork.
            ]]
        })
        :Questions("RISE", {
            "Ask about the machine",
            [[
                player:
                    player:
                        What kind of machine is this, exactly?
                    agent:
                        It's more of an armored battlemech. 
                        With lasers.
                        And a rocket-leg.
                    player:
                        !dubious
                        And they're using it to dig holes?
                    agent:
                        !shrug
                        That's the Barons, for you.
            ]],        
            "Ask about the sabotage",
            [[
                player:
                    How did you sabotage the machine?
                agent:
                    We have leverage on one of the Baron techs.
                    They did things to it.
                    <i>Computer</> things.
                player:
                    What the Hesh is a computer?
                agent:
                    !shrug
                    Beats me. Whatever's inside an automech's head, I guess.
            ]],
            "Ask why you need to help",
            [[
                player:
                    I thought the Rise was good at making things explode. Why would you need my services?
                agent:
                    Every member of the Rise has to be individually trained, equipped, and motivated to fight.
                    They're expensive to replace, and sending them into a meat grinder is bad for morale.
                    !happy
                    I've got leverage on you. That makes you cheap.
            ]]
        })
        :Fn(function(cxt)
            cxt:GetCastMember("handler_second"):MoveToLocation(cxt.location)
            cxt:TalkTo("handler_second")
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_HEAR_DEMANDS")
                :Dialog("DIALOG_INTRO_2")
                :LoopingFn(function()
                    local inserted = cxt:InsertQuestions(cxt:GetAgent():GetFactionID())
                    if inserted == 0 then
                        cxt:Dialog("DIALOG_ASKED_ALL_QUESTIONS")
                        cxt.quest:Complete("introduction")
                        cxt:GetAgent():MoveToLimbo()    
                        cxt:End()
                    end

                    cxt:Opt("OPT_SET_OUT")
                        :Dialog("DIALOG_SET_OUT")  
                        :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                        :CompleteQuest("introduction")
                        :End()
                end)
        end)



local PRE_FIGHT = function(cxt)
    local party
    
    if cxt.quest.param.handler_faction == "RISE" then
        party = {"RISE_PAMPHLETEER", "RISE_RADICAL", "RISE_REBEL", "RISE_AUTODOG", "RISE_AUTOMECH"}
    else
        party = {"SPARK_BARON_TASKMASTER", "SPARK_BARON_PROFESSIONAL", "SPARK_BARON_GOON", "AUTODOG", "SPARK_BARON_AUTOMECH"}
    end

    --spawn some rise
    cxt.quest.param.helpers = CreateCombatParty( party, cxt.quest:GetRank(), cxt.location )
    cxt:TalkTo(cxt.quest.param.helpers[1])
    cxt:Dialog("DIALOG_INTRO")

    local fight_allies = {}
    for k,v in ipairs(cxt.quest.param.helpers) do
        if k > 1 then
            table.insert(fight_allies, v)
        end
    end

    cxt:RunLoop(function()
        cxt:Question("OPT_QUESTION_1", "DIALOG_QUESTION_1")

        cxt:Opt("OPT_CONVINCE_HELP")
            :Dialog("DIALOG_CONVINCE_HELP")
            :Negotiation{
                flags = NEGOTIATION_FLAGS.ALLY_GRAB | NEGOTIATION_FLAGS.NO_CORE_RESOLVE,
                fight_allies = fight_allies,
                reason_txt = cxt:GetLocString("NEGOTIATION_REASON")
            }
                :OnFailure()
                    :Dialog("DIALOG_FAIL_NEGOTIATION")
                :OnSuccess()
                    :Dialog("DIALOG_SUCCESS_NEGOTIATION")
                    :Fn(function(cxt, minigame) 
                        
                        cxt.quest.param.fight_allies = {}
                        for i, modifier in minigame:GetPlayerNegotiator():Modifiers() do
                            if modifier.id == "FIGHT_ALLY_WON" and modifier.ally_agent then
                                table.insert( cxt.quest.param.fight_allies, modifier.ally_agent )
                            end
                        end
                        cxt:Opt("OPT_ATTACK_THE_ROBOT")
                            :Dialog("DIALOG_ATTACK_WITH_TEAM")
                            :GoTo("STATE_ATTACK_ROBOT")
                    end)

        cxt:Opt("OPT_ATTACK_THE_ROBOT")
            :Dialog("DIALOG_ATTACK_ALONE")
            :GoTo("STATE_ATTACK_ROBOT")
    end)
end

QDEF:AddConvo("kill_the_robot")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("robot_dig_site") end)
        :Fn(function(cxt) 
            cxt:GoTo(cxt.quest.param.handler_faction == "RISE" and "STATE_PRE_FIGHT_RISE" or "STATE_PRE_FIGHT_BARONS")
        end)


    :State("STATE_PRE_FIGHT_RISE")
        :Loc{
            
            DIALOG_INTRO = [[
                * You approach the dig site, and find a small contingent of Rise hunkered down behind a dead oshnu.
                * All is quiet, but a faint charnel smell hangs in the air, hinting at recent violence.
                player:
                    !left
                agent:
                    !right
                    Oh thank Hesh, the relief is here.
                    The thing has retreated back into the hole to recharge. I don't know if we can repel it again when it comes back.
                    You need to go shut it down!

            ]],
            OPT_ATTACK_THE_ROBOT = "Attack the Automech",
            NEGOTIATION_REASON = "Collect allies and then play the ending card to start the fight!",
            OPT_CONVINCE_HELP = "Convince {agent} to spare some fighters",
            DIALOG_CONVINCE_HELP = [[
                player:
                    {handler_second} sent me to help, but I can't do this myself.
                agent:
                    We've already suffered losses! I'm not going to order any more of my people to their deaths.
                player:
                    If I attack this thing alone, it'll kill me and then come for you. Our only chance is to take it on together.
            ]],
            DIALOG_SUCCESS_NEGOTIATION = [[
                agent:
                    Fine. Take the volunteers, and take care of that thing. I'll... cover your retreat.
                    From here.
            ]],
            DIALOG_FAIL_NEGOTIATION = [[
                agent:
                    No. If you can't stop that thing, we're going to high-tail it out of the bog.
                    I don't even like this place. I'm not going to die for it.
            ]],

            DIALOG_ATTACK_ALONE = [[
                * You cast a disdainful gaze to the cowering Rise.
                player:
                    I'll handle this myself, then.
                    !exit
            ]],

            DIALOG_ATTACK_WITH_TEAM = [[
                * You signal to the 'volunteers', and approach the dig.
                player:
                    !exit
                agent:
                    Don't get my people killed!
            ]],

    
            OPT_QUESTION_1 = "Ask about the automech",
            DIALOG_QUESTION_1 = [[
                player:
                    You've fought this thing. What can you tell me about it?
                agent:
                    It's a beast!
                    We poured everything we had into it, and all we managed to do was knock its leg off.
                    Then the damned thing just started shooting flames at us out of the stump!
            ]],            

        }
        :Fn(PRE_FIGHT)

    
    :State("STATE_PRE_FIGHT_BARONS")
        :Loc{
            DIALOG_INTRO = [[
                * You approach the dig site, and find a small contingent of Barons holding a perimeter around the main pit.
                * All is quiet, but a faint charnel smell hangs in the air, hinting at recent violence.
                player:
                    !left
                agent:
                    !right
                    !salute
                    You're the attaché that {handler_second} promised?
                player:
                    That's much fancier title than I'm used to. Most folks just call me 'grifter' and try to not make eye contact.
                agent:
                    I'll call you whatever you want if you can get us out of this, er... situation.
                player:
                    I do enjoy a challenge. Where is the device?
                agent:
                    The 'device' is currently at the bottom of the pit. Probably recharging for another attack.
                    Please see to it.
            ]],
            OPT_ATTACK_THE_ROBOT = "Attack the Automech",
            NEGOTIATION_REASON = "Collect allies and then play the ending card to start the fight!",
            OPT_CONVINCE_HELP = "Convince {agent} to spare some fighters",
            DIALOG_CONVINCE_HELP = [[
                player:
                    {handler_second} sent me to help, but I can't do this myself.
                agent:
                    I've already lost three people and a 'mech. My quarterly report is going to be hard enough to square as-is. 
                    !point
                    <i>You're</> the expert. <i>You</> go do the job.
                player:
                    Let me break down the cost-benefit for you...
            ]],
            DIALOG_SUCCESS_NEGOTIATION = [[
                agent:
                    Fine. Take some of my assets. But don't break them.
            ]],
            DIALOG_FAIL_NEGOTIATION = [[
                agent:
                    No. I'm not sending good money after bad. If you can't contain the threat, we'll just fill in the pit and write the whole thing off.
            ]],

            DIALOG_ATTACK_ALONE = [[
                * You shrug, unable to argue against {agent}'s balance sheet.
                player:
                    I'll be right back, then.
                    !exit
            ]],

            DIALOG_ATTACK_WITH_TEAM = [[
                * You signal to the 'volunteers', and approach the pit.
                player:
                    !exit
                agent:
                    I'm holding you personally responsible for any losses!
            ]],

    
            OPT_QUESTION_1 = "Ask about the automech",
            DIALOG_QUESTION_1 = [[
                player:
                    So what are we up against here.
                agent:
                    It's an automech that was recovered from one of the deep bog pits, and retrofitted for digging.
                    Unfortunately, it seems to be having trouble distinguishing between the ground, and... us.
                    We've managed to contain it so far, but it's highly adaptable. Losing a leg did not slow it down - in fact it seemed to make it even more dangerous.
                player:
                    An automech after my own heart.
            ]],    
        }
        :Fn(PRE_FIGHT)
        
  
    :State("STATE_ATTACK_ROBOT")
        :Loc{
            
            DIALOG_INTRO = [[
                player:
                    !exit
                right:
                    !exit

                * You walk toward the pit in silence, and peer over the edge.
                player:
                    !left
                    !interest
                    ...
                * You see a faint glow deep in the bowels of the bog. 
                * You pick up a small rock and throw it in.
                player:
                    !give
                * There's a rush of hot air as the mech rises from the deep!
                player:
                    !flinch
                agent:
                    !right
                    NEW TARGET DETECTED
                player:
                    !fight
                agent:
                    ENGAGE DIGGING MECHANISMS
            ]],
            OPT_ATTACK_THE_ROBOT = "Attack!",
            DIALOG_ATTACK_THE_ROBOT = [[
                player:
                    Engage THIS!
            ]],

            DIALOG_BEAT_ROBOT = [[
                right:
                    !exit
                * The automech falls to the ground, its control circuits irreparably scrambled.
                {rise?
                    {solo?
                        agent:
                            !right
                            !impressed
                            Wha-?
                        player:
                            !dust_off
                            All in a day's work.
                    }
                    {not solo and lost_allies?
                        agent:
                            !right
                            !angry
                            You messed up! You spilled good Rise blood!
                        player:
                            !dust_off
                            Eh. So did you. The difference is, I got the job done.
                            !point
                            Don't make me bring that to {handler_second}'s attention.

                    }
                    {not solo and not lost_allies?
                        agent:
                            !right
                            !salute
                            You did good, grifter.
                        player:
                            !dust_off
                            They're competent troops. They just need proper leadership.
                            Try harder next time, will you?
                    }
                }
                {spark_barons?
                    {solo?
                        agent:
                            !right
                            !impressed
                            You did it! See, you didn't need our help.
                        player:
                            !dust_off
                            Perhaps I should tell {handler_second} how redundant you are?
                        agent:
                            !placate
                            Now now, let's not get hasty. We loosened the lid, so to speak.
                    }
                    {not solo and lost_allies?
                        agent:
                            !right
                            !angry
                            Look what you did! My assets are dead!
                        player:
                            !dust_off
                            I'd be more concerned with what you didn't do, if I were you.
                            !point
                            I'll bet that {handler_second} will remember that you couldn't do what needed to be done.

                    }
                    {not solo and not lost_allies?
                        agent:
                            !right
                            !salute
                            You have my respect, grifter.
                        player:
                            !dust_off
                            Thank you for lending a hand. 
                            !thought
                            Or at least the hands of others.

                    }
                }
            ]],
            DIALOG_TALK_TO_HANDLER = [[
                right:
                    !exit
                * {handler_second} arrives on the scene, as if summoned by your victory.

                {rise?
                    agent:
                        !right
                        You did good work, grifter. I can see why {rise_contact} keeps you around.
                    player:
                        I danced to your tune, so are we on good terms now?
                    agent:
                        !disagree
                        No.
                        Not yet. 
                        !crossed
                        But you've proven yourself useful enough to keep around. 
                        Here, take this...
                        !give
                }
                {spark_barons?
                    agent:
                        !right
                        I commend you on your martial skill, grifter.
                    player:
                        It'll be a cold day in the Arkcross when some creaking old automech gets the best of me.
                        Are we square now?
                    agent:
                        !crossed
                        We are 'square' enough that I will permit you to continue to draw breath.
                        !point
                        But I still do not trust you.
                    player:
                        Don't fall all over yourself expressing gratitude, now.
                    agent:
                        I am not without gratitude.
                        Here, take this trinket in payment...
                        !give
                }
            ]],
            DIALOG_TALK_TO_HANDLER_2 = [[
                {rise?
                    player:
                        My thanks.
                    agent:
                        You wouldn't thank me if you saw the strings attached to that thing.
                        My people will finish destroying the evidence. 
                        Make yourself scarce, grifter. And don't let me find you fraternizing with the Barons.
                    * {handler_second} lopes off, leaving heavy footprints in the bog.
                }
                {spark_barons?
                    player:
                        Thank you.
                    agent:
                        {baron_contact} trusts you, but we both know how ill-considered most of {baron_contact.hisher} decisions are.
                        I will be watching you, grifter. I suggest you watch yourself.
                    * {handler_second} motions for {handler_second.hisher} goons to attend to the mess, and turns to leave.
                }
            ]],


        }
        :Fn(function(cxt)
            
            local robot = TheGame:GetGameState():AddAgent(Agent("AUTOMECH_BOSS"))
            cxt:TalkTo(robot)

            cxt:Dialog("DIALOG_INTRO")
                
            cxt:Opt("OPT_ATTACK_THE_ROBOT")
                :Dialog("DIALOG_ATTACK_THE_ROBOT")
                :Battle{ 
                    allies = cxt.quest.param.fight_allies,
                    flags = BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.NO_BURRS,
                }    
                :OnWin()
                    :Fn(function() 
                        local had_allies = cxt.quest.param.fight_allies and #cxt.quest.param.fight_allies > 0
                        local all_allies_survived = true
                        if had_allies then
                            for k,v in ipairs(cxt.quest.param.fight_allies) do
                                if v:IsDead() and v:IsSentient() then
                                    all_allies_survived = false
                                    break
                                end
                            end
                        end
                        if cxt:TalkToSomeone(cxt.quest.param.helpers) then
                            cxt.enc.scratch.solo = not had_allies
                            cxt.enc.scratch.lost_allies = had_allies and not all_allies_survived
                            cxt:Dialog("DIALOG_BEAT_ROBOT")
                            if cxt.enc.scratch.lost_allies then
                                cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("got_their_people_killed"))
                            end
                        end

                        cxt:TalkTo("handler_second")
                        cxt:Dialog("DIALOG_TALK_TO_HANDLER")
                        ConvoUtil.GiveBossRewards(cxt, GRAFT_TYPE.COMBAT)
                        cxt:Dialog("DIALOG_TALK_TO_HANDLER_2")
                        cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("dealt_with_a_monster"))
                    end)
                    :CompleteQuest()
                    :Travel()
        end)

QDEF:AddDebugOption("handler_faction", {"RISE", "SPARK_BARONS"})

QDEF:AddTest{
    tests = {
        {name = "Random", reps = 10 }, --ehhh... should be enough
    }
}
