local function FindCoinGrafts( number, rarity )
    local owner = TheGame:GetGameState():GetPlayerAgent()
    local collection = GraftCollection.RewardableCoin(owner):Rarity(rarity)
    local grafts = collection:Generate(number)
    
    if grafts then
        for k,graft in ipairs(grafts) do
            TheGame:GetGameProfile():SetSeenGraft( graft.id )
        end
    end
    
    return grafts
end

local RARITY_TABLE = {
    CARD_RARITY.COMMON,
    CARD_RARITY.UNCOMMON,
    CARD_RARITY.RARE
}

local QDEF = QuestDef.Define
{
    title = "Coin Trader",
    --icon = engine.asset.Texture("icons/quests/rook_story_living_at_bar.tex"),
    qtype = QTYPE.STORY,

    collect_agent_locations = function(quest, t)
        if quest:IsActive("shop_active") or quest:IsActive("first_meeting") then
            table.insert (t, { agent = quest:GetCastMember("coin_trader"), location =  TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"), role = CHARACTER_ROLES.VENDOR} )
        end
    end,
    
    is_in_hiding = function(quest, agent)
        if agent == quest:GetCastMember("coin_trader") and not quest:IsActive("shop_active") then
            return true
        end
    end,
    
    plot_armour_fn = function(quest, agent)
        return agent == quest:GetCastMember("coin_trader")
    end,

    events = {
        phase_change = function(quest)
            if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY then
                if quest:IsActive("shop_active") then
                    quest:Cancel("shop_active")
                end
                quest:Activate("shop_active")
            end
        end,
    },
}

:AddCastByAlias{
    cast_id = "coin_trader",
    alias = "COIN_TRADER",
    limit = QLIMIT.NOLIMIT,
}

:AddObjective{
    id = "shop_active",
    hide_in_overlay = true,
    low_priority = true,
    on_activate = function(quest)
        quest.param.newest_difficulty = TheGame:GetGameState():GetCurrentBaseDifficulty()
        quest.param.grafts = FindCoinGrafts(3, RARITY_TABLE[math.min(#RARITY_TABLE, quest.param.newest_difficulty)])
    end,
}


:AddObjective{
    id = "first_meeting",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    low_priority = true,
    on_complete = function(quest) 
        quest:Activate("shop_active")
    end,
}

QDEF:AddConvo("first_meeting")                
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("coin_trader"):GetLocation() end)
        :Loc{
            DIALOG_INTRO = [[
                * A dusty old kra'deshi approaches you as you enter.
                agent:
                    !right
                    !greeting
                    You're Admiralty, yeah? I can read it 'cross your shoulders.
                player:
                    !left
                    !agree
                    I was. Once. 
                agent:
                    Aerostat, if I had to guess?
                ]],
            OPT_FRIENDLY = "Compliment {agent} on {agent.hisher} perception",
            DIALOG_FRIENDLY = [[
                player:
                    !happy
                    $neutralJoke
                    They can take our youth, eh, but they can't take our cunning.
                agent:                    
                    Oh, they would if they could, I'm sure.
            ]],
            OPT_UNFRIENDLY = "Warn {agent} to mind {agent.hisher} own business",
            DIALOG_UNFRIENDLY = [[
                player:
                    $neutralDubious
                    You're very free with your observations. 
                    !dubious
                    That's an indulgence that can get you into trouble.
                agent:
                    !wink
                    $miscAlluring
                    Oh, I don't mean nothing by it. I'm Admiralty, too, sometimes. Heh.
                    Just nice to see a friendly face so far from home.
            ]],
            DIALOG_INTRO_2 = [[
                player:
                    !salute
                    Alright. Well met, then. Enjoy your—
                agent:
                    Do you have your squadron coin? 
                    !bashful
                    $miscEmbarrassed
                    Can I... can I see it? 
                player:
                    !crossed
                    Now that's quite personal.
                agent:
                    !placate
                    No, please don't take offense. I'm a collector, you see.
                    I have several coins of my own, if you're up for a trade.
                player:
                    ...
                agent:
                    !overthere
                    I'm always here. I like the foot traffic.
                    You decide you might be up for a trade, just find me. 
                    !exit
                * {agent} backs off, and seems content to give you your space. If there's an angle, you can't guess it.
                *** {agent} has an interest in coins, and will trade with you here in the bar.

            ]],
        }
        :Fn(function(cxt)
            cxt:TalkTo(cxt.quest:GetCastMember("coin_trader"))
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_FRIENDLY")
                :Dialog("DIALOG_FRIENDLY")
                :Dialog("DIALOG_INTRO_2")
                :CompleteQuest("first_meeting")
                :DoneConvo()

            cxt:Opt("OPT_UNFRIENDLY")
                :Dialog("DIALOG_UNFRIENDLY")
                :Dialog("DIALOG_INTRO_2")
                :CompleteQuest("first_meeting")
                :DoneConvo()
         end)


QDEF:AddConvo("shop_active", "coin_trader")                
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !Greeting
                $happyGreeting
                Looking for coins? I've got some good ones for trade.
        ]],
        OPT_TRADE_COINS = "Trade Coins",
        DIALOG_TRADE_COINS = [[
            agent:
                !happy
                Always nice to chat with a fellow enthusiast!
        ]],
        DIALOG_NEW_COIN = [[
            * You trade your current coin for a {1#graft}.
            agent:
                $miscNostalgic
                Ah, yes. This one has a familiar weight. 
                A pleasure to hold it again, for sure. 
                !exit
            * {agent} studies the coin fondly before sliding it into a lined case.
        ]],
        DIALOG_NO_COIN = [[
            agent:
                !agree
                I can see why you'd want to hang on to that one.
                Perhaps later you'll be ready to part with it, hmm?
        ]],
    }
    
    :AttractState("asd")
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            if cxt.quest.param.newest_difficulty ~= TheGame:GetGameState():GetCurrentBaseDifficulty() then
                cxt.quest.param.newest_difficulty = TheGame:GetGameState():GetCurrentBaseDifficulty()
                cxt.quest.param.grafts = FindCoinGrafts(3, RARITY_TABLE[math.min(#RARITY_TABLE, cxt.quest.param.newest_difficulty)])
            end
        end)
    :Hub(function(cxt)
        local current_coin = cxt.player:GetAspect("graft_owner"):GetGrafts( GRAFT_TYPE.COIN )[1]

        cxt:Opt("OPT_TRADE_COINS")
            :PreIcon( global_images.receiving )
            :ReqCondition( current_coin ~= nil, "REQ_COIN" )
            :Fn(function() 
                cxt.enc:WaitOnLine()
                local popup = Screen.PickCoinScreen(cxt.quest.param.grafts, false, function(graft) print (graft) cxt.enc:ResumeEncounter(graft) end, true)
                TheGame:FE():InsertScreen( popup )

                local chosen_coin = cxt.enc:YieldEncounter()
                cxt.enc:Wait()
                if chosen_coin then
                    cxt:Dialog("DIALOG_NEW_COIN", chosen_coin)
                    cxt:GetAgent():MoveToLimbo()
                    cxt.quest:Cancel('shop_active')
                else
                    --cxt.enc:Next()
                    cxt:Dialog("DIALOG_NO_COIN")
                end
            end)
    end)
