
local QDEF = QuestDef.Define
{
    title = "Stepping in it",
    --desc = [[]],
    --icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    qtype = QTYPE.STORY,
    icon = engine.asset.Texture("icons/quests/rook_story_stepping_in_it.tex"),

    --put other handler into hiding once you've picked
    is_in_hiding = function(quest, agent)

        if quest:IsComplete("contact_barons") then
            if quest:GetCastMember("rise_contact") == agent then
                return true
            end
        end
        
        if quest:IsComplete("contact_rise") then
            if quest:GetCastMember("spark_contact") == agent then
                return true
            end
        end
    end,


    --put handlers in their houses before you've picked, and lock your pick in place once you make it
    collect_agent_locations = function(quest, t)
        
        table.insert (t, { agent = quest:GetCastMember("spark_second"), location = quest:GetCastMember("baron_hq"), role = CHARACTER_ROLES.VISITOR} )
        table.insert (t, { agent = quest:GetCastMember("rise_second"), location = quest:GetCastMember("rise_hq"), role = CHARACTER_ROLES.VISITOR} )

        if quest:IsActive("job_followup_barons") then
            table.insert (t, { agent = quest:GetCastMember("spark_contact"), location = quest:GetCastMember("bar"), role = CHARACTER_ROLES.CONTACT} )
        elseif quest:IsActive("contact_barons") or quest:IsComplete("contact_barons") then
            table.insert (t, { agent = quest:GetCastMember("spark_contact"), location = quest:GetCastMember("baron_hq"), role = CHARACTER_ROLES.CONTACT} )
        end


        if quest:IsActive("job_followup_rise") then
            table.insert (t, { agent = quest:GetCastMember("rise_contact"), location = quest:GetCastMember("bar"), role = CHARACTER_ROLES.CONTACT} )
        elseif quest:IsActive("contact_rise") or quest:IsComplete("contact_rise") then
            table.insert (t, { agent = quest:GetCastMember("rise_contact"), location = quest:GetCastMember("labour_office"), role = CHARACTER_ROLES.CONTACT} )
        end
    end,

    on_start = function(quest)
        quest:Activate("starting_out")
        quest:Activate("first_bar_visit")
    end,
    
    events = 
    {
    },
}

:AddLocationCast{
    cast_id = "rise_hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_RISE_HQ"))
    end,
}


:AddLocationCast{
    cast_id = "baron_hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_BARON_HQ"))
    end,
}

:AddCast{
    cast_id = "primary_handler",
    when = QWHEN.MANUAL,
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "spark_contact",
    alias = "SPARK_CONTACT",
    no_validation = true,
}


:AddLocationCast{
    cast_id = "labour_office",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_LABOUR_OFFICE"))
    end,
}

:AddCastByAlias{
    cast_id = "rise_contact",
    alias = "KALANDRA",
    no_validation = true,
}


:AddLocationCast{
    cast_id = "bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"))
    end,
}

:AddCastByAlias{
    cast_id = "hebbel",
    alias = "HEBBEL",
}

:AddCastByAlias{
    cast_id = "spark_second",
    alias = "SPARK_SECOND",
}

:AddCastByAlias{
    cast_id = "rise_second",
    alias = "RISE_SECOND",
}


--this gets you to the point where you're through the gate and you've talked to hebbel
:AddSubQuest{
    id = "starting_out",
    quest_id = "ROOK_STORY_ENTERING_GROUT_BOG",
    on_complete = function(quest)
        quest:GetCastMember("rise_contact"):SetBuildOverride("female_foreman_kalandra_build")
        quest:Activate("contact_barons")
        quest:Activate("contact_rise")
    end,
}

:AddObjective{
    id = "contact_barons",
    title = "Contact {spark_contact#agent}",
    desc = "Check in with your old friend {spark_contact} to work for the Spark Barons.",
    mark = {"spark_contact"},
    icon = engine.asset.Texture("icons/quests/rook_story_work_for_fellemo.tex"),
    on_complete = function(quest) 
        
        local main = quest.param.main_quest
        main:GetQuestDef().SetPrimaryHandlerBarons(main) 
        main:GetQuestDef().SetVendorEnabled(main, "SPARK_BARONS", true) 
        quest:Cancel("contact_rise")
        quest:Activate("get_job")
    end,
}

:AddObjective{
    id = "contact_rise",
    title = "Contact {rise_contact}",
    desc = "{rise_contact}, a local labor leader, has put out word that {rise_contact.heshe} is looking for some extra help.",
    mark = {"rise_contact"},
    icon = engine.asset.Texture("icons/quests/rook_story_work_for_kalandra.tex"),
    on_complete = function(quest)
        local main = quest.param.main_quest
        main:GetQuestDef().SetPrimaryHandlerRise(main) 
        quest:Cancel("contact_barons")
        quest:Activate("get_job")
    end,
}


:AddObjective{
    id = "get_job",
    title = "Get a job from {primary_handler}",
    mark = {"primary_handler"},
    
    on_complete = function(quest) 
        quest:Activate("do_job")
    end,
    
}

:AddObjective{
    id = "first_bar_visit",
    hide_in_overlay = true,
}


:AddObjective{
    id = "do_job",
    hide_in_overlay = true,
    events = 
    {
        quests_changed = function(quest, event_quest)
            if quest.param.current_job == event_quest and not event_quest:IsActive() then
                quest:Complete("do_job")
            end
        end
    },

    on_complete = function(quest) 
        quest.param.job_history = quest.param.job_history or {}
        table.insert(quest.param.job_history, quest.param.current_job)
        quest.param.current_job = nil
        if quest.param.job_history and #quest.param.job_history >= 2 then
            QuestUtil.SpawnOpportunities()
            quest:Activate("visit_bar")
        else
            quest:Activate("get_job")
        end
    end,

}

:AddObjective{
    id = "visit_bar",
    title = "Go to {bar} and wait for sunset",
    mark = {"hebbel"},
    on_complete = function(quest) 
        UIHelpers.PassTime(DAY_PHASE.NIGHT)
       
        if TheGame:GetGameState():GetMainQuest().param.faction_pick == "RISE" then
            quest:Activate("job_followup_rise") 
        else            
            quest:Activate("job_followup_barons")
        end
    end,
}

:AddObjective{
    id = "job_followup_rise",
    title = "Talk to {primary_handler} about getting more work",    
    mark = {"primary_handler"},
    on_activate = function(quest)
        quest:GetCastMember("primary_handler"):MoveToLocation(quest:GetCastMember("bar"))
    end,
}

:AddSubQuest{
    id = "do_infiltration_rise",
    quest_id = "ROOK_STORY_INFILTRATE_RISE",
    on_activate = function(quest)
        quest:GetCastMember("rise_contact"):SetBuildOverride("female_rise_kalandra_build")
        TheGame:GetGameState():AddPendingEvent()
        QuestUtil.SpawnOpportunities()
    end,
    
    on_complete = function(quest)
        quest:Activate("file_report")
    end,
}


:AddObjective{
    id = "job_followup_barons",
    mark = {"primary_handler"},
    on_activate = function(quest)
        quest:GetCastMember("primary_handler"):MoveToLocation(quest:GetCastMember("bar"))
    end,
}

:AddSubQuest{
    id = "do_infiltration_barons",
    quest_id = "ROOK_STORY_INFILTRATE_BARONS",
    on_activate = function(quest)
        TheGame:GetGameState():AddPendingEvent()
        QuestUtil.SpawnOpportunities()
    end,
    on_complete = function(quest)
        quest:Activate("file_report")
    end,
}


:AddSubQuest{
    id = "file_report",
    quest_id = "ROOK_STORY_DEAD_DROP_DAY_1",
    
    on_activate = function(quest)
        QuestUtil.SpawnOpportunities()
        TheGame:GetGameState():AddPendingEvent()
    end,

    on_complete = function(quest)
        quest:Activate("day_over")
    end,
}


:AddObjective{
    id = "day_over",
    title = "Return to {bar} and rest",
    desc = "{hebbel} has a room ready for you. Time to recharge.",
    mark = {"bar", "hebbel"},
    
    on_activate = function(quest) 
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true
    end,
    
    on_complete =function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
    end
}


-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

--this will trigger mid-conversation after you turn in the loyalty quest
QDEF:AddConvo("visit_bar", "primary_handler")
    :Attract(function(cxt) if not cxt.quest.param.told_to_wait then return "STATE_TALK" end end)
    :Hub(function(cxt)
        if not cxt.quest.param.told_to_wait then
            cxt:GoTo("STATE_TALK")
        end
    end)
    :State("STATE_TALK")
        :Loc{
            DIALOG_INTRO_RISE = [[
                agent:
                    Time's come. I got something extra-special that needs your skills.
                    !overthere
                    $miscPersuasive
                    That bar I mentioned earlier—it's kind of neutral ground, catering to barons and workers both.
                    Meet me there after nightfall, and we'll discuss it. 
                player:
                    !interest
                    $miscAlluring
                    Ah, and here I was wondering if you'd only hired me for the dagger, and forgotten my cloak.
                agent:
                    Later. All right?
                * {agent} leaves you alone with your questions.                    
                    !exit
            ]],
            DIALOG_INTRO_BARON = [[
                agent:
                    !suspicious
                    $neutralResigned
                    Now, {player}, you know damn well I didn't call you all the way out here just to rough up some organizers.
                    I have my own enforcers for that. I need your particular skills. 
                player:
                    !crossed
                    $neutralDubious
                    I thought you might. You want to discuss that here?
                agent:
                    !thought
                    $neutralTakenAback
                    Ah—no. Now that you mention it, I rather don't.
                    !agree
                    That watering hole I mentioned earlier—it's a good place to have a quiet conversation. 
                    Meet me there after sunset, yes? Then we'll discuss more. 
                player:
                    !dubious 
                    You all right to make it there on your own? Or will you need an armed escort.
                agent:
                    Ah, yes, that gentle ribbing again. 
                    !cruel
                    $miscMocking
                    You forget, Captain—you may have been our leader, but I was the better marksman. 
                    $neutralDirect
                    It would be a shame if that required a demonstration.
                * {agent} leaves you alone to contemplate his response. 
                !exit
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest.param.told_to_wait = true
            cxt:Dialog(TheGame:GetGameState():GetMainQuest().param.faction_pick == "RISE" and "DIALOG_INTRO_RISE" or "DIALOG_INTRO_BARON")
            cxt:GetAgent():MoveToLimbo()
            StateGraphUtil.AddEndOption(cxt)
        end)


QDEF:AddConvo("contact_barons", "spark_second")
    :AttractState("attr")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !crossed
                    If you have questions, talk to {spark_contact}.
            ]]
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            StateGraphUtil.AddEndOption(cxt)
        end)

QDEF:AddConvo("contact_barons")
    :ConfrontState("CONFRONT", function(cxt) 
            return not cxt.quest.param.access_to_spark and 
            cxt.location == cxt.quest:GetCastMember("spark_contact"):GetLocation() and 
            cxt.location:GetAgentByRole(CHARACTER_ROLES.GUARD) ~= nil end)

        :Loc{
            DIALOG_INTRO_SHOVEL = [[
                agent:
                    !right
                * A heavily-armed guard bars your way.
                player:
                    !left
                agent:
                    Halt! 
                * {agent} sees the shovel you took at the gate.
                agent:
                    !crossed
                    $neutralDirect
                    This is an administrative zone. No workers allowed.
            ]],
            DIALOG_INTRO_AGAIN = [[
                * {agent} is waiting for you.
                player:
                    !left
                agent:
                    !right
                    !angry
                    $angryPatienceLost
                    You again? Didn't I tell you to get out of here?
            ]],

            OPT_CONVINCE = "Convince {agent} that you need to talk to {spark_contact}",
            DIALOG_CONVINCE = [[
                player:
                    I have a meeting with {spark_contact}.
            ]],
            DIALOG_CONVINCED = [[
                agent:
                    Alright, you can talk to {spark_contact}. 
                    !point
                    But don't try nothing!
                    !exit
                * {agent} steps aside.
            ]],

            DIALOG_NO_CONVINCE = [[
                agent:
                    {spark_contact} doesn't have time to meet with no common laborer. Get out of here before I throw you out of here!
                player:
                    !exit
                * You decide against escalating the situation and leave.
            ]],


            OPT_LEAVE = "Leave without complaint",
            DIALOG_LEAVE = [[
                player:
                    My apologies. I must have taken a wrong turn.
                    !exit
                * You leave.
            ]],
            
            DIALOG_INTRO_LIST = [[
                * As you enter the official-looking building, you're stopped by an armed guard.
                player:
                    !left
                agent:
                    !right
                    $neutralDirect
                    !crossed
                    Stay right where you are, grifter, and state your business.
            ]],
            OPT_SHOW_LIST = "Show {agent} your papers",
            DIALOG_SHOW_LIST = [[
                player:
                    !give
                    $neutralGreeting
                    The official sort, my good soldier. Nothing to fret about.
                * You show {agent} your signed papers. {agent.HeShe} somehow studies them while still fixing you with a suspicious glare.
                agent:
                    $neutralDirect
                    This checks out.
                    !permit
                    Very well. 
                    !exit
                * You step inside.

            ]],

            DIALOG_INTRO_NEITHER = [[
                * There's no guard at the door, so you waltz straight into the administrative building.
            ]],

            DIALOG_INTRO_KICK_OUT = [[
                * {agent} is waiting for you at the door.
                agent:
                    !right
                    !threaten
                    I said get lost, grifter. You aren't welcome here.
                * You are kicked out before you can object with either words or fists.
            ]]
        }
        :Fn(function(cxt) 
            local blacklist = cxt.player.negotiator:GetCardCount("blacklist") > 0 
            local shovel = cxt.player.battler:GetCardCount("shovel") > 0
            
            cxt:TalkTo(cxt.location:GetAgentByRole(CHARACTER_ROLES.GUARD))

            if cxt.quest.param.failed_access_to_spark then
                cxt:Dialog("DIALOG_INTRO_KICK_OUT")
                StateGraphUtil.AddLeaveLocation(cxt)
            elseif shovel then
                cxt:Dialog(cxt.quest.param.has_talked_to_spark_guard and "DIALOG_INTRO_AGAIN" or "DIALOG_INTRO_SHOVEL")
                cxt.quest.param.has_talked_to_spark_guard = true
                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation()
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCED")
                            :Fn(function() cxt.quest.param.access_to_spark = true end)
                            :DoneConvo()
                        :OnFailure()
                            :Dialog("DIALOG_NO_CONVINCE")
                            :Fn(function() cxt.quest.param.failed_access_to_spark = true end)
                            :Travel()

                cxt:Opt("OPT_LEAVE")
                    :Dialog("DIALOG_LEAVE")
                    :Travel()

            elseif blacklist then
                cxt:Dialog("DIALOG_INTRO_LIST")

                cxt:Opt("OPT_SHOW_LIST")
                    :Dialog("DIALOG_SHOW_LIST")
                    :Fn(function() cxt.quest.param.access_to_spark = true end)
                    :DoneConvo()
            else
                cxt:Dialog("DIALOG_INTRO_NEITHER")
                cxt.quest.param.access_to_spark = true
                StateGraphUtil.AddEndOption(cxt)
            end

        end)


QDEF:AddConvo("contact_barons", "spark_contact")
    :AttractState("attract")
        :Loc{
            DIALOG_INTRO = [[
                * {spark_contact} is easy to recognize, even though the years have been a little too kind to him. 
                spark_contact:
                    !drink_not_drunk
                * You spot him lapping at a drink like a contented salamander. 
                player:
                    $miscMocking
                    I see you've gone up in the world since discharging, eh, rookie?
                spark_contact:
                    !neutral
                    {player}, you old scoundrel! You finally made it. 
                    !chuckle
                    Figured you'd be able to move faster than that, even with that bum leg of yours. 
                player:
                    Docktown's a long way from here, {spark_contact}.
                    !point
                    And that's Captain Rook to you—even if we're both retired. 
                spark_contact:
                    !handwave
                    $miscMischievous
                    I'll call you Your Royal Highness if you can get the job done.
                    Enough fiddlefaddle. Let's get you to work, so you can see what I'm up against. 
            ]],
            DIALOG_INTRO_ALREADY_TALKED = [[
                spark_contact:
                    So are you ready to start, {player}?
            ]],
            DIALOG_REINTRO = [[
                spark_contact:
                    !crossed
                    $neutralResigned
                    You aren't getting cold feet, are you?
                player:
                    Just being thorough. You know me.
            ]],
            DIALOG_QUESTIONS = [[
                player:
                    !crossed
                    $neutralDubious
                    You might note I haven't actually agreed to the job yet. 
                    You hardly provided much information—just a slapdash cypher that any fool could decode.
                spark_contact:
                    !cruel
                    $neutralWhatever
                    I don't have the time for codes anymore, Captain. 
                    I've gone further than the Admiralty could ever take either of us.
                    $neutralWhatever
                    All the same, very well—I'll answer your questions. The least I could do.
            ]],
            
            OPT_ACCEPT_OFFER = "Work for {spark_contact}",
            DIALOG_ACCEPT_OFFER = [[
                player:
                    $neutralThoughtful
                    You wouldn't have called me out here if it weren't for something interesting.
                    !agree
                    I'll take the job. 
                spark_contact:
                    !happy
                    $miscRelieved
                    Excellent. If there's anyone who can get to the bottom of this, it's you.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt) 
            if not cxt.quest.param.talked_to_baron then
                cxt.quest.param.talked_to_baron = true
                cxt:Dialog(cxt:FirstLoop() and "DIALOG_INTRO" or "DIALOG_REINTRO")
            else
                cxt:Dialog(cxt:FirstLoop() and "DIALOG_INTRO_ALREADY_TALKED" or "DIALOG_REINTRO")
            end

            cxt:Opt("OPT_ASK_ABOUT")
               :Dialog("DIALOG_QUESTIONS")
               :IsHubOption(true)
               :GoTo("STATE_QUESTIONS")     

            cxt:Opt("OPT_ACCEPT_OFFER")
                :Dialog("DIALOG_ACCEPT_OFFER")
                :CompleteQuest("contact_barons")
                :Pop()
        end)
    

     :AskAboutHub("STATE_QUESTIONS", 
        {
            "Ask about making contact",
            [[
                player: 
                    !thought
                    $neutralThoughtful
                    Last I heard from you, it was—
                agent:  
                    !agree
                    The Rentoria Raids. Must've been fifteen years ago.
                    $miscMischievous
                    At the time they told us you were killed in action. But we all knew better.
                player:
                    $neutralDirect
                    I should hope so. When did you hear otherwise?
                agent:
                    After I joined the Barons. One of the Rents mentioned they knew you. 
                    !interest
                    $neutralJoke
                    Well—knew your cover, but I'd have recognized it anywhere.
                    I said to my superiors, you know, "You want someone who can cut to the quick—we should recruit Rook".
                player:
                    !dubious 
                    And how did that go over?
                agent:
                    !crossed
                    $neutralJoke
                    Unfortunately, it's far easier to <i>need</i> a spy than it is to <i>trust</i> one.
                    But times have changed. And you're being given a chance.
            ]],
            "Ask about the work",
            [[
                player: 
                    !thumb
                    $neutralThoughtful
                    What service can I provide the illustrious Spark Barons?
                    $neutralResigned
                    If it's to act as a liaison with the Admiralty, I'm afraid I was dishonorably discharged.
                agent:
                    !chuckle
                    $miscMischievous
                    Oh, I heard. The whole unit did. 
                    Are you familiar with a little movement called the Rise?
                player:
                    $neutralThoughtful
                    It's out in the Bread Fields, isn't it? Labor disputes, something of that cut. 
                agent:
                    !agree
                    It's migrated. Some of the contractors have cottoned on to that scheme.
                    !point 
                    $neutralDirect
                    I want this handled in a way that doesn't leave an indiscriminate body count. 
                player:
                    Define "indiscriminate", please.
                agent:
                    !shrug
                    Use your judgment. If someone needs to be put down, so be it. 
                    $neutralDirect
                    But we can't make a profit without laborers to do the work. 
            ]],
            "Ask about your authority",
            [[
                player: 
                    !dubious
                    $neutralDubious
                    What sort of authority will I wield while I do this work?
                agent: 
                    !cruel
                    $happyCocky
                    Hah! I beg your pardon, Captain. You'll have as much authority as I see fit to lend you.
                    $neutralJoke
                    And for the moment, that means just about none.
                    !shrug
                    $miscPersuasive
                    It's nothing personal, of course. I still respect you tremendously.
                player:
                    $neutralDubious
                    Yes, well, you never were one to pass up an opportunity.
                agent:
                    $miscMischievous
                    That's exactly right, Rook. I'm honored you remember.
            ]],
            "Ask about the Rise",
            [[
                player: 
                    How dangerous are the Rise?
                agent:  
                    !palm
                    $neutralResigned
                    They're murdering my fiscal reports, certainly. 
                    But the real threat to you is the bog itself. 
                player:
                    !dubious
                    $neutralDubious
                    The bog? I had hostile terrain training. 
                agent:
                    !chuckle
                    Yes, you used to be the best. But with that leg of yours, no more, eh? 
                    Beasts and poisons prey on flesh, contaminants prey on metalworks. 
                    There's not a part of you that won't find something in the bog unpleasant.
                    !point
                    $miscMocking
                    And the Rise will use that against you, for certain. If you fail to be subtle, that is.
            ]],
        })


QDEF:AddConvo("contact_rise")
    :ConfrontState("CONFRONT", function(cxt) 
            return not cxt.quest.param.access_to_rise and 
            cxt.location == cxt.quest:GetCastMember("rise_contact"):GetLocation() and 
            cxt.location:GetAgentByRole(CHARACTER_ROLES.GUARD) ~= nil end)

        :Loc{
            DIALOG_INTRO_LIST = [[
                agent:
                    !right
                * A surly guard blocks your entrance.
                player:
                    !left
                agent:
                    What's this, then? 
                * {agent} notices the Spark Baron list you have tucked under your arm.
                agent:
                    !crossed
                    You have no business here, scab.
            ]],
            DIALOG_INTRO_AGAIN = [[
                * {agent} is waiting for you.
                player:
                    !left
                agent:
                    !right
                    !crossed
                    You ain't welcome here.
            ]],

            OPT_CONVINCE = "Convince {agent} that you need to talk to {rise_contact}",
            DIALOG_CONVINCE = [[
                player:
                    I need to talk to {rise_contact}. {rise_contact.HeShe} is expecting me.
            ]],
            DIALOG_CONVINCED = [[
                agent:
                    If {rise_contact} wants to see you, there must be a reason.
                    !point
                    Don't give me a reason to break you.
                    !exit
                * {agent} steps aside.
            ]],

            DIALOG_NO_CONVINCE = [[
                agent:
                    Spark Barons ain't welcome here. You smell like Baron. 
                    !threaten
                    Get. Out.
                player:
                    !exit
                * You decide against escalating the situation and leave.
            ]],


            OPT_LEAVE = "Leave without complaint",
            DIALOG_LEAVE = [[
                player:
                    My apologies. I must have taken a wrong turn.
                    !exit
                * You leave.
            ]],
            
            DIALOG_INTRO_SHOVEL = [[
                * You enter the Labor Office, shovel at the ready.
                player:
                    !left
                agent:
                    !right
                    You here to dig, grifter?
            ]],
            OPT_SHOW_SHOVEL = "Show {agent} your shovel",
            DIALOG_SHOW_SHOVEL = [[
                * You brandish the shovel.
                player:
                    Oh, I'm here to dig, friend. 
                    Why else would anyone end up in the Bog?
                agent:
                    Good point.
                    Talk to {rise_contact}. {rise_contact.HeShe} will set you up with a contract.
            ]],
            
            DIALOG_INTRO_NEITHER = [[
                * There's no guard at the door, so you waltz straight into the labor office.
            ]],

            DIALOG_INTRO_KICK_OUT = [[
                * {agent} is waiting for you at the door.
                agent:
                    !right
                    !threaten
                    I already told you. No Barons!
                * You are kicked out before you can object with either words or fists.
            ]]
        }
        :Fn(function(cxt) 
            local blacklist = cxt.player.negotiator:GetCardCount("blacklist") > 0 
            local shovel = cxt.player.battler:GetCardCount("shovel") > 0
            
            cxt:TalkTo(cxt.location:GetAgentByRole(CHARACTER_ROLES.GUARD))

            if cxt.quest.param.failed_access_to_rise then
                cxt:Dialog("DIALOG_INTRO_KICK_OUT")
                StateGraphUtil.AddLeaveLocation(cxt)
            
            elseif blacklist then
                cxt:Dialog(cxt.quest.param.has_talked_to_rise_guard and "DIALOG_INTRO_AGAIN" or "DIALOG_INTRO_LIST")
                cxt.quest.param.has_talked_to_rise_guard = true
                
                cxt:Opt("OPT_CONVINCE")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation()
                        :OnSuccess()
                            :Dialog("DIALOG_CONVINCED")
                            :Fn(function() cxt.quest.param.access_to_rise = true end)
                            :DoneConvo()
                        :OnFailure()
                            :Dialog("DIALOG_NO_CONVINCE")
                            :Fn(function() cxt.quest.param.failed_access_to_rise = true end)
                            :Travel()

                cxt:Opt("OPT_LEAVE")
                    :Dialog("DIALOG_LEAVE")
                    :Travel()

            elseif shovel then
                cxt:Dialog("DIALOG_INTRO_SHOVEL")

                cxt:Opt("OPT_SHOW_SHOVEL")
                    :Dialog("DIALOG_SHOW_SHOVEL")
                    :Fn(function() cxt.quest.param.access_to_rise = true end)
                    :DoneConvo()
            else
                cxt:Dialog("DIALOG_INTRO_NEITHER")
                cxt.quest.param.access_to_rise = true
                StateGraphUtil.AddEndOption(cxt)
            end

        end)

QDEF:AddConvo("contact_rise", "rise_contact")
    :AttractState("attract")
        :Loc{
            DIALOG_INTRO = [[
                * {rise_contact} holds herself like a leader in sheep's clothing. She looks up as you approach.
                player:
                    !give
                    I believe you're the foreman, yes? {rise_contact}? 
                * You pass her the work visa, quickly flashing your signet coin between your fingers when you do.
                player:
                    !coin_toss
                rise_contact:
                    !interest
                * Her eyes light up shrewdly and, maybe, with a <>spark</> of mischief.
                rise_contact:
                    $miscAlluring
                    I am. But I should warn you: this work isn't for the meek.
                    The bog will be the death of you, if you let it reach your neck.
                player:
                    !thought
                * That's the code phrase you were expecting. So you respond in kind.
                player:
                    $neutralThoughtful
                    Then I'll watch my step, and keep my neck dry. 
                * She nods, satisfied.
                rise_contact:
                    !agree
                    $miscRelieved
                    You're Rook, then? 
                    !happy
                    Your references are glowing. That's unusual for this side of the bog.
                    $neutralDirect
                    I can't give you the details now, but we're on a timeline, and I need to put you to work right away.
                    So let's get started, shall we?
            ]],
            DIALOG_REINTRO = [[
                rise_contact:
                    !crossed
                    $neutralResigned
                    So, are we in business? Or do you have some other state to undermine?
            ]],
            DIALOG_QUESTIONS = [[
                player:
                    I'm afraid I will need <i>some</i> details if we're to be in business.
                rise_contact:
                    I figured you might. Go on, then. 
                    
            ]],
            
            OPT_ACCEPT_OFFER = "Work for {rise_contact}",
            DIALOG_INTRO_ALREADY_TALKED = [[
                agent:
                    We have a deal, right?
            ]],
            DIALOG_ACCEPT_OFFER = [[
                player:
                    !agree
                    As you say, you've hired me to do a job. 
                    $miscAlluring
                    I'm nothing if not professional. 
                rise_contact:
                    !happy
                    Happy to be in business. I promise you'll find me a fair employer.
            ]],
        }
        :SetLooping()
        :Fn(function(cxt) 
            if not cxt.quest.param.talked_to_rise then
                cxt.quest.param.talked_to_rise = true
                cxt:Dialog(cxt:FirstLoop() and "DIALOG_INTRO" or "DIALOG_REINTRO")
            else
                cxt:Dialog(cxt:FirstLoop() and "DIALOG_INTRO_ALREADY_TALKED" or "DIALOG_REINTRO")
            end

            cxt:Opt("OPT_ASK_ABOUT")
               :Dialog("DIALOG_QUESTIONS")
               :IsHubOption(true)
               :GoTo("STATE_QUESTIONS")     

            cxt:Opt("OPT_ACCEPT_OFFER")
                :Dialog("DIALOG_ACCEPT_OFFER")
                :CompleteQuest("contact_rise")
                :Pop()

        
        end)

     :AskAboutHub("STATE_QUESTIONS",
        {
            "Ask about the Rise",
            [[
                player:
                    $neutralDirect
                    Let's not be cute about it. You're one of the Rise rebels, yes?
                    !overthere
                    Even the gate guard had their suspicions. 
                agent:
                    !crossed
                    $neutralResigned
                    The Rise isn't a rebellion, it's a labor movement.
                    My workers like me. That's reason enough for the Barons to cast me as the villain. 
                    !eureka
                    $neutralDirect
                    Let them. If their eyes are on me, then my agents can pass without scrutiny.
                player:
                    !dubious
                    $neutralDubious
                    You're bold, I see.
                agent:
                    !cruel
                    $miscPersuasive
                    Hah! Boldness is <i>action</i>, not words. Hold your applause.
            ]],
            "Ask about the work",
            [[
                player: 
                    I know it's frowned upon to discuss compensation, but this isn't my fight.
                agent:  
                    Which is why you're exactly the agent I need. 
                    !point
                    $miscAlluring
                    I hired you because you're competent, so I assume you don't mind that the work is dangerous.
                player:
                    !agree
                    Correct.
                agent:
                    !interest 
                    $miscAlluring
                    And I assume you don't mind that it might make you a few enemies—
                player:
                    If it does, it's hardly your fault.
                agent:
                    You do the work I pay you for—nothing more, nothing less. And I'll never bring <i>morals</i> into it. 
                    !placate
                    $miscPersuasive
                    No begging for you to see the light, no appealing to your good conscience. 
                    Fair pay for your services.  
                player:
                    !wink
                    That sounds agreeable to me. 
            ]],
            "Ask about the Barons",
            [[
                player:
                    !thought
                    $neutralThoughtful
                    How does a labor movement quantify success? When will you know you've won?
                agent:
                    $neutralDirect
                    We're not here to overthrow. We're here to stand.
                    !angry_shrug
                    $angryReluctant
                    We sign contracts with no way out—Barons control the markets, and we blow our earnings at the company store before we've finished our shift.
                    Demerits are issued if you drop a relic, if you get sick, if you get <i>others</i> sick...
                    !crossed
                    $neutralDirect
                    Folks always say the derricks are even worse. But this isn't good by any reckoning.
                player:
                    !interest
                    What if Barons end up dead to make it happen? Do your ideals cover that?
                agent:
                    !agree
                    My ideals demand it. The workers under my care are dying every day.
                    If Barons die so things can change... it's just the scales righting themselves.
            ]],
        })


QDEF:AddConvo("get_job", "primary_handler")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    I have a couple of things that need doing...
            ]],
            DIALOG_NEXT_INTRO = [[
                agent:
                    Alright, I have another job for you.
            ]],
            DIALOG_PICKED = [[
                agent:
                    Your service is appreciated.
            ]],

            DIALOG_INTRODUCE_BAR_RISE = [[
                agent:
                    !overthere
                    If you need supplies, check in at {bar}. We don't have much to spare here, but the barkeep there sells whatever you might need.
                    The bar's kind of a neutral ground for the Spark Barons who want to pretend their workers like them.
                    It's safe enough to let yourself be seen there without drawing too much suspicion.
            ]],
            
            DIALOG_INTRODUCE_BAR_BARON = [[
                agent:
                    $neutralThoughtful
                    !thought
                    Oh, and I suppose you'll be wanting to requisition some supplies...
                    But I'm afraid that's not on the table, friend. You're a contractor, after all.
                    !shrug
                    $neutralResigned
                    Try the bartender over at {bar}. It's an unsavory sort of place, and it'll serve unsavory sorts like you.
                    !happy 
                    $happyCocky
                    Otherwise I'm sure you'll find everything you need. You always were resourceful. 
            ]],
        }
        :Fn(function(cxt) 

            cxt:Dialog(cxt.quest.param.job_history and #cxt.quest.param.job_history >= 1 and "DIALOG_NEXT_INTRO" or "DIALOG_INTRO")

            local quest_options = cxt.quest.param.main_quest:GetQuestDef().SpawnFactionJobs(cxt.quest.param.main_quest, 2, cxt:GetAgent():GetFactionID(), nil, {"nasty", "late_game"}, {giver = cxt:GetAgent()})
            TheGame:GetGameState():GetMainQuest():GetQuestDef().PresentFactionJobChoice(cxt, quest_options, true, 
                    function(cxt, job) 
                        cxt.quest.param.current_job = job 
                        cxt.quest:Complete("get_job")
                        if cxt.quest.param.job_history and #cxt.quest.param.job_history == 1 then
                            cxt:Dialog(cxt:GetAgent():GetFactionID() == "RISE" and "DIALOG_INTRODUCE_BAR_RISE" or cxt:Dialog("DIALOG_INTRODUCE_BAR_BARON"))                            
                            QuestUtil.SpawnQuest("ROOK_STORY_LIVING_AT_BAR")
                        end
                        StateGraphUtil.AddEndOption(cxt)
                    end)
        end)


QDEF:AddConvo("job_followup_rise", "primary_handler")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            
            --if you did the job successfully
            DIALOG_INTRO_LIKED = [[
                agent:
                    !happy
                    !crossed
                    $happyCocky
                    Time for the real reason I hired you, I think.
                    You know it wasn't just to babysit my workers.
                player:
                    $neutralThanks
                    I assumed not. You seem capable of doing that yourself.
            ]],
            --you did the job for parital credit
            DIALOG_INTRO_NEUTRAL = [[
                agent:
                    !cagey
                    $miscMischievous
                    You weren't followed, I assume? Let's make this quick all the same.
            ]],
            --you failed the job (ran away from a fight?)
            DIALOG_INTRO_DISLIKED = [[
                agent:
                    !angryshrug
                    $neutralDubious
                    Rook.
                player:
                    Alright, {rise_contact}?
                agent:
                    ...
                    !palm
                    After the stunt you pulled earlier... I dunno.
                    $neutralResigned
                    But I don't have many options right now, so I guess we're doing this.
            ]],

            DIALOG_DESCRIBE_JOB = [[
                agent:
                    There's a reason I hired <i>you</i> and not any of the other grifters with guns just as quick as yours.
                    !permit
                    It's 'cause I know you go back with {spark_contact}. 
                    And I need you to infiltrate the Spark Barons.
                player:
                    Ah, the pieces are beginning to coalesce.
                agent:
                    There's a worker, one of mine, actually. {spark_contact.HeShe} has been stealing from Barons for a while now, but I looked the other way.
                    Until {spark_contact.heshe} started stealing from my other workers, too. 
                    Take {spark_contact.himher} with you when you go to meet the Barons, as a sign of good faith.
            ]],

            OPT_ACCEPT_JOB = "Accept the job",
            DIALOG_ACCEPT_JOB = [[
                player:
                    $happyThanks
                    I would've appreciated you giving me the details sooner, but I can't say I'm not grateful for the challenge.
                agent:
                    $neutralDirect
                    Glad to hear it. 
                    !happy
                    $miscMischievous
                    Now it must be <i>years</i> since you last saw your old pal. Don't you think it's time you paid him a visit?
                player:
                    !agree
                    Yes, and I know just the present to bring him, too. 
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest:Complete("job_followup_rise")
            if cxt:GetAgent():GetRelationship() >= RELATIONSHIP.LIKED then
                cxt:Dialog("DIALOG_INTRO_LIKED")
            elseif cxt:GetAgent():GetRelationship() <= RELATIONSHIP.DISLIKED then
                cxt:Dialog("DIALOG_INTRO_DISLIKED")
            else
                cxt:Dialog("DIALOG_INTRO_NEUTRAL")
            end
            
            cxt:Dialog("DIALOG_DESCRIBE_JOB")
            
            cxt:RunLoop(function()
                
                cxt:AskAbout("STATE_QUESTIONS")

                cxt:Opt("OPT_ACCEPT_JOB")
                    :Dialog("DIALOG_ACCEPT_JOB")
                    :Fn(function() 
                        cxt.quest:Activate("do_infiltration_barons")
                        local main_quest = TheGame:GetGameState():GetMainQuest()
                        main_quest:GetQuestDef().SetVendorEnabled(main_quest, "RISE", true) 
                    end)
                    :End()
            end)


        end)

    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask about the infiltration",
        [[
                player:
                    $neutralDubious
                    Why am I doing this?
                agent: 
                    Does it matter? I thought you didn't want any moralizing. 
                player:
                    !shrug
                    $miscPersuasive
                    Knowledge is power, as they say. So it depends on how effective you'd like me to be.
                agent: 
                    $miscMischievous
                    Fellemo has something that belongs to me. And I don't want him to break it. 
                    !eureka
                    I need you to cozy up so we can find out where it is, and get it back.
                player:
                    What is it exactly that Fellemo has?
                agent:
                    !thumb
                    An invention. <i>My</i> invention. 
                    Normally I would've been able to use it to parlay myself a place in the Barons but...
                    !angry
                    $angryDefault
                    He stole it, instead. Said I wasn't Spark Barons <i>material</i>.
                player:
                    $neutralResigned
                    That's the problem with hierarchies. They prefer their ranks subordinate. 
        ]],
        "Ask {agent} about the Spark Barons",
        [[
            player:
                !dubious
                $neutralDubious
                The Spark Barons have a strict recruitment policy.     
            agent:
                But you have a chummy past with one of the Barons' biggest wigs. 
                $miscMischievous
                And he knows your worth. I bet he's in real need of a union buster, for example.
            player:
                !agree
                Ah. Corporate sabotage is more my speed, certainly. 
        ]],

        "Ask {agent} about the work",
        [[
            player:
                !interest
                So what's the plan? After this we only speak in secret?
            agent:
                Fellemo will want you where you can do the most damage. So I'm sure I'll see you right back at my workshop tomorrow. 
                !overthere
                From this point on you'll be a double agent. You'll have to work for both of us.
                But hopefully you're good enough to sabotage the jobs he asks you to do while still staying in his good graces.
            player:
                !shrug
                $neutralResigned
                That will be very difficult.
            agent:
                !point
                $neutralDirect
                Do what you have to do. But I need to know I can trust you, or this won't work.
        ]],
    })


QDEF:AddConvo("job_followup_barons", "primary_handler")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            
            --if you did the job successfully
            DIALOG_INTRO_LIKED = [[
                agent:
                    !happy
                    $happyThanks
                    Ah, this place brings back memories. I won a lot of money playing cards at this very table.
                player:
                    !dubious
                    Is that why you brought me here? To play cards?
                agent:
                    I have something even better in store for you, old friend. 
            ]],
            --you did the job for parital credit
            DIALOG_INTRO_NEUTRAL = [[
                agent:
                    $neutralDirect
                    I see you're as punctual as always.
                    It always bored me a little, if we're honest. But I won't complain.
            ]],
            --you failed the job (ran away from a fight?)
            DIALOG_INTRO_DISLIKED = [[
                agent:
                    I'm sorry to say that you disappoint me, Rook. 
                    !dubious
                    $neutralDubious
                    The decorated officer I remember would never have delivered as poorly as you did today.
                player:
                    I'm many years older and one leg short, old friend. 
                    !permit
                    Besides, you get what you pay for. 
                agent:
                    $neutralResigned
                    Touché. Then I may as well get my money's worth.
            ]],

            DIALOG_DESCRIBE_JOB = [[
                agent: 
                    I want you to infiltrate the Rise. It's time we put an end to this worker rebellion.
                player:
                    !agree
                    I see. I'm to find out who's in charge? A member list?
                agent:
                    !angry
                    I <i>know</i> who's in charge. It's that Prindo {rise_contact}. 
                    I want you to get close to her. Make yourself invaluable. 
                    !cruel
                    $angryCruel
                    Then we'll take it from there. 
                player:
                    !crossed
                    $neutralDubious
                    And how am I to do that when I can be seen having a friendly chat with a prominent Spark Baron?
                agent:
                    !chuckle
                    $miscMischievous
                    Why, we're only talking, and you're <i>very</i> tricky.
                    The taps here loosen tongues, and I have valuable information.
                    !interest
                    $miscAlluring
                    The kind of information that would be very valuable to the Rise.
                player:
                    $neutralDirect
                    Not if the intel isn't good. 
                agent:
                    !agree
                    $neutralDirect
                    The intel's very real, I assure you. We'll take a few casualties but it's a price I'm willing to pay.
                player:
                    !thought
                    $neutralDubious
                    So I'm to convince them I'm a double agent?
                agent:
                    $angryCruel
                    You <i>are</i> a double agent. 
                    !handwave
                    Please, Rook, this is child's play compared to your Admiralty days.

                *** {agent} wants you to infiltrate the Rise by feeding them some intelligence.

            ]],

            OPT_ACCEPT_JOB = "Accept the job",
            DIALOG_ACCEPT_JOB = [[
                player:
                    $neutralDefault
                    Very well. I imagine I shouldn't just march right up to this Prindo Kalandra, of course. 
                agent:
                    Try turning yourself in at one of the cells. I happen to know the location of one. 
                    !overthere
                    $miscPersuasive
                    Take it nice and slow—as you say, you've been seen with me. It'll take some convincing that you're on the level.
                    Especially because their suspicions will be well-founded. 
                    !cruel
                    I have no plan for extracting you if things go sour, old friend, so do try not to mess this up.
                player:
                    !dubious
                    $miscMocking
                    Your loyalty is inspiring as always, Fellemo.
                * Fellemo serves you only a wink in response as he pays his tab and makes for the door. 
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest:Complete("job_followup_barons")
            if cxt:GetAgent():GetRelationship() >= RELATIONSHIP.LIKED then
                cxt:Dialog("DIALOG_INTRO_LIKED")
            elseif cxt:GetAgent():GetRelationship() <= RELATIONSHIP.DISLIKED then
                cxt:Dialog("DIALOG_INTRO_DISLIKED")
            else
                cxt:Dialog("DIALOG_INTRO_NEUTRAL")
            end
            
            cxt:Dialog("DIALOG_DESCRIBE_JOB")
            
            cxt:RunLoop(function()

                cxt:AskAbout("STATE_QUESTIONS")

                cxt:Opt("OPT_ACCEPT_JOB")
                    :Dialog("DIALOG_ACCEPT_JOB")
                    :Fn(function() 
                        cxt.quest:Activate("do_infiltration_rise")
                    end)
                    :End()

            end)

        end)

    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask {agent} about the infiltration",
        [[
            player:
                !interest
                The intel won't be enough, of course. The Rise will want me to prove my loyalty.
            agent:
                Naturally. I expect you to be shrewd. Do as much as it takes to earn their trust... 
                Otherwise, you're still working for me. 
                I imagine you'll remember that when the chips fall. 
        ]],

        "Ask about {rise_contact}",
        [[
            player:
                If you <i>know</i> this person is the ringleader, surely it's simplest just to have her eliminated?
            agent:
                Unrest in the Grout Bog is older than my tenure. 
                It happened before, as little as ten years ago. Do you remember?
            player:
                I heard something of it. Massive riots, yes? Labor revolts?
            agent:
                Yes. A woman named Fallon was the ringleader. But Prindo's father, Seban Kalandra, was her second in command.
                They were all put down, of course, but the sentiment lives on.
                I'm not interested in having this problem in another ten years. I want it dealt with for good.
            player:
                Kill the sentiment?
            agent:
                Precisely. 
        ]],

    })



QDEF:AddConvo("visit_bar", "hebbel")
        :Loc{
            OPT_WAIT_AROUND= "Wait for {primary_handler}",
            DIALOG_WAIT_AROUND = [[
                player:
                    Nothing for me, barkeep. I'm only here to meet someone.
                agent:
                    !shrug
                    $neutralDubious
                    Ain't none of my business, is it?
                player:
                    !exit
                agent:
                    !exit
                * You lean against the bar and watch the crowd as you wait for sunset.
            ]],
            DIALOG_HANDLER_ARRIVES = [[
                * With the last of the light, {primary_handler} walks through the door.
            ]]
        }
        :Hub(function(cxt) 

            cxt:Opt("OPT_WAIT_AROUND")
                :SetQuestMark()
                :Dialog("DIALOG_WAIT_AROUND")
                :Fn(function() 
                    cxt.enc:GetScreen():ForceWaitOnLine()
                    cxt.enc:DoFade(1)
                    cxt.quest:Complete("visit_bar")
                    local screen = TheGame:FE():FindScreen( Screen.LocationScreen )
                    if screen then
                        screen:UpdateDayPhase()
                    end
                    cxt.enc:GetScreen():ClearHistory()
                end)
                :Dialog("DIALOG_HANDLER_ARRIVES")
                :DoneConvo()

        end)
        


QDEF:AddConvo("day_over", "hebbel")
    :Loc{
        OPT_GET_A_ROOM = "Rest in the back",
        DIALOG_GET_ROOM = [[
            player:
                How about that room you promised?
            agent:
                I'll go wake up the cockroaches.
                !exit
            player:
                Charming.
                !exit
            * {agent} shows you to your room.
        ]],
        DIALOG_WAKE = [[
            * You wake up, ready to further your intrigue here in Grout Bog.
        ]],
    }

    :Hub(function(cxt) 
            cxt:Opt("OPT_GET_A_ROOM")
                :SetQuestMark()
                :PreIcon(global_images.sleep)
                :Fn(function(cxt) 
                    local grog = cxt.location
                    cxt:Dialog("DIALOG_GET_ROOM")
                    cxt:AwardUnlock("SMITH_PLAYABLE", "SMITH", LOC"UI.CHARACTER_UNLOCKED.SMITH_TITLE", LOC"UI.CHARACTER_UNLOCKED.SMITH_BODY")
                    
                    cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR.inn_room") )
                    grog:SetPlax()
                    ConvoUtil.DoSleep(cxt, "DIALOG_WAKE")
                    
                    cxt.quest:Complete()

                    cxt:Opt("OPT_LEAVE")
                        :MakeUnder()
                        :Fn(function() 
                            cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR") )
                            cxt:End()
                        end)

                end)
        end)    
    :AttractState("attract", function(cxt) return not cxt.quest.param.did_hebbel_recap end)
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    You ready to hit the sack?
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

        end)





QDEF:AddConvo("first_bar_visit", "hebbel")
    :AttractState("asd")
        :Loc{
            DIALOG_CHECK_IN = [[
                * The bar is well known in your network: a quiet speakeasy where folks like you can rest away from prying eyes.
                * The clients keep their heads down even as you come in. If they're curious enough to study your face, they're clever enough to hide it. 
                player:
                    !greeting
                    $happyGreeting
                    Hello, barkeep. The name is Rook and I'm simply parched.
                hebbel:
                    !dubious
                * The bartender inspects you carefully, nods once, and keeps the conversation nice and low.
                hebbel:
                    !agree
                    Word was I should expect you. I don't make a habit of helping out no more than I need to...
                    !interest
                    You keep that in mind.
            ]],

            OPT_ACK = "Thank {hebbel}",

            DIALOG_ASK_ABOUT = [[
                player:
                    $miscPersuasive
                    I don't suppose you could provide me with a little information?
                hebbel:
                    It's the job, ain't it? Whaddya need?
            ]],
            DIALOG_GET_STARTED = [[
                player:
                    !happy
                    $happyThanks
                    Thank you, barkeep. Your warmth and hospitality will keep me going today, for certain.
                agent:
                    !dubious
                    $neutralDubious
                    Hold on. I got somethin' for you. 
            * {agent} digs around under the bar, producing a lined case of unlicensed grafts.
                player:
                    !thumb
                    A gift, my fine friend? And we've only just met. 
                agent:
                    !shrug
                    $neutralWhatever
                    You call in good favors. 
            ]],
            DIALOG_GET_STARTED_2 = [[
                player:
                    Very useful. And if I wanted to call in further favors?
                agent:
                    I got drinks, and I got eats. You want anything other than that, you're going to have to talk to the Jakes.
                    They have a shop set up on the edge of the bog. And the edge of the law, for that matter.
                    But they're a cagey bunch, so you might have to prove yourself worthy of their business.
                    I'm sure you'll figure it out.
                player:
                    Well, I appreciate the information. I'm sure we'll speak again soon.
                    !exit
            ]],

        }
        
        :SetLooping()
        :Fn(function(cxt)
            
            if cxt:FirstLoop() then
                cxt.quest:Complete("first_bar_visit")
                cxt:Dialog("DIALOG_CHECK_IN")
            end

            cxt:Opt("OPT_ASK_ABOUT")
                :Dialog("DIALOG_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")

            cxt:Opt("OPT_ACK")
                :Dialog("DIALOG_GET_STARTED")
                :Fn(function(cxt) 
                    ConvoUtil.GiveGraftChoice(cxt, RewardUtil.GetGrafts(1, TheGame:GetGameState():GetGraftDraftDetails().count))
                end)
                :Dialog("DIALOG_GET_STARTED_2")
                :Fn(function() 
                    local main_quest = TheGame:GetGameState():GetMainQuest()
                    if not main_quest.param.market_quest then
                        QuestUtil.SpawnQuest( "ROOK_STORY_BLACK_MARKET" )
                    end
                end)
                :End()
        end)

     :AskAboutHub("STATE_QUESTIONS", 
        {
            
            "Ask about Grout Bog",
            [[
                player:
                    !interest
                    Can you tell me anything about the terrain here?
                    I like to be prepared.
                agent:
                    Grout Bog is proper miserable. S'why the Spark Barons could get any rights at all, I suspect.
                    $neutralResigned
                    The bog will suck up anything that falls in it, and the beasts will scavenge the rest. 
                    But it's full to the brim with Vagrant Age tech, if you know where to find it.
                player:
                    !happy
                    $happyCocky
                    Aha. Dangerous, but profitable. My kind of place, I think.
            ]],

            "Ask about the Spark Barons",
            [[
                player:
                    !wink
                    I believe what Spark Barons call tech, Heshians call antiquities.
                agent:
                    !shrug
                    A lot of fuss over rusted old refuse, if you ask me. 
                    Used to be the Cult was too mighty. But these days, the Heshians have to settle for what the Barons will trade.
                    !cruel
                    $miscMischievous
                    And pray to their tentacled friend that whatever they get is actually "holy".
                player:
                    !thought
                    That's a powerful monopoly to hold.
                agent:
                    !overthere
                    $neutralWhatever
                    Yeah, well, there's a reason the Barons are more often than not callin' the shots these days.
            ]],

            "Ask about the laborers",
            [[
                player:
                    !neutral
                    But all the digging is done by laborers, correct? Not any of the actual Spark Barons?
                agent:
                    !handwave
                    $neutralWhatever
                    What, and get their fancy tools dirty? Forget about it.
                    Labor's all contracted out. Of course, since the Barons own the land, and the borders, and all the shops and supplies—
                    $neutralResigned
                    Let's just say there's a lot of pay to dock. These days it's almost as bad as the derricks. 
                player:
                    !dubious
                    $neutralDubious
                    You mean it's indentured servitude.
                agent:
                    Might as well be. 
                    !forgetful
                    $miscPersuasive
                    Now, I don't pretend to know nothing 'bout nothing—but if I know anything about anything, that's what you're here for.
                    Some of the laborers have started organizin'. Calling themselves the Rise.
                    !agree
                    I bet you'll get familiar with them real soon, if you aren't already.
            ]],

            "Ask about the barkeep",
            [[
                player: 
                    And what about you? Where do you factor into this?
                agent:  
                    This here's the only private business in the bog. Thank my grandfather for that.
                    !thumb
                    $neutralThoughtful
                    I keep secrets—and everybody knows better than to ask me not to.
                player:
                    !cruel
                    $miscAlluring
                    And what if I need you to favor me over your other clientele?
                    Feed me a little intel, maybe even about their own objectives?
                agent:
                    !angry_accuse
                    $neutralDirect
                    That's not a service I offer. 
                player:
                    !placate
                    $neutralTakenAback
                    Come now—
                agent:
                    !angrySeething
                    $angrySeething
                    I'd heard you were getting too old for this game. 
                    You wanna stick your nose in other people's business? That's your blood in the pipe.
                    !thumb
                    $angryHostile
                    I don't rat out my customers—not to you, and not to them.
                    Be grateful. 
                player:
                    $neutralThanks
                    Good to know. And thank you for your frankness. 
            ]],
        })     
