

local QDEF = QuestDef.Define
{
    title = "Working Both Sides",
    --desc = [[]],
    --icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    qtype = QTYPE.STORY,
    icon = engine.asset.Texture("icons/quests/rook_story_working_both_sides.tex"),

    --put other handler into hiding once you've picked
    is_in_hiding = function(quest, agent)
        return quest:GetCastMember("rise_contact") == agent or quest:GetCastMember("spark_contact") == agent
    end,


    --put handlers in their houses before you've picked, and lock your pick in place once you make it
    collect_agent_locations = function(quest, t)
        
        table.insert (t, { agent = quest:GetCastMember("spark_contact"), location = quest:GetCastMember("baron_hq"), role = CHARACTER_ROLES.CONTACT} )
        if quest.param.main_quest.param.faction == "SPARK_BARONS" then
            table.insert (t, { agent = quest:GetCastMember("rise_contact"), location = quest:GetCastMember("rise_hq"), role = CHARACTER_ROLES.CONTACT} )
        else
            table.insert (t, { agent = quest:GetCastMember("rise_contact"), location = quest:GetCastMember("labour_office"), role = CHARACTER_ROLES.CONTACT} )
        end
    end,

   
    events = 
    {
    },
    
    on_start = function(quest)
        if quest.param.main_quest.param.faction_pick == "RISE" then
            quest:GetCastMember("rise_contact"):SetBuildOverride("female_foreman_kalandra_build")
        end

        quest:Activate("pick_first_job_primary")
        quest:Activate("pick_first_job_secondary")
    end,
}



:AddCast{
    cast_id = "primary_handler",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert(t, quest.param.main_quest:GetQuestDef().GetPrimaryHandler(quest.param.main_quest))
    end,
}
:AddCast{
    cast_id = "secondary_handler",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        table.insert(t, quest.param.main_quest:GetQuestDef().GetSecondaryHandler(quest.param.main_quest))
    end,
}

:AddCast{
    cast_id = "first_job_handler",
    when = QWHEN.MANUAL,
}
:AddCast{
    cast_id = "second_job_handler",
    when = QWHEN.MANUAL,
}

-------------
:AddLocationCast{
    cast_id = "baron_hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_BARON_HQ"))
    end,
}

:AddCastByAlias{
    cast_id = "spark_contact",
    alias = "SPARK_CONTACT",
    no_validation = true,

}


-------------
:AddLocationCast{
    cast_id = "labour_office",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_LABOUR_OFFICE"))
    end,
}

:AddLocationCast{
    cast_id = "rise_hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_RISE_HQ"))
    end,
}

:AddCastByAlias{
    cast_id = "rise_contact",
    alias = "KALANDRA",
    no_validation = true,
}


-------------
:AddLocationCast{
    cast_id = "bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"))
    end,
}

:AddCastByAlias{
    cast_id = "hebbel",
    alias = "HEBBEL",
}

:AddLocationCast{
    cast_id = "dead_drop_spot",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_DEAD_DROP_SPOT"))
    end,
}


-----------------------------
:AddObjective{
    id = "pick_first_job_primary",
    title = "Report for duty with {primary_handler}",
    desc = [[See what covert jobs {primary_handler} has that need doing today.]],
    mark = { "primary_handler"},
    on_complete = function(quest)
        quest.param.did_primary_first = true
        quest:Cancel("pick_first_job_secondary")
        quest:Activate("do_job")
    end,
    on_activate = function(quest)
        QuestUtil.SpawnQuest("EVENT_INSECT_BITE")
    end,
}

:AddObjective{
    id = "pick_first_job_secondary",
    title = "See what {secondary_handler} wants",
    desc = [[You are moonlighting with {secondary_handler}. See if they need any skulduggery taken care of.]],
    mark = { "secondary_handler"},
    on_complete = function(quest)
        quest:Cancel("pick_first_job_primary")
        quest:Activate("do_job")
    end,
}

-----------------------------
:AddObjective{
    id = "pick_followup_secondary",
    title = "See what {secondary_handler} needs",
    mark = { "secondary_handler"},
    on_complete = function(quest)
        quest:Activate("do_job")
    end,
}

:AddObjective{
    id = "pick_followup_primary",
    title = "See what {primary_handler} needs",
    mark = { "primary_handler"},
    on_complete = function(quest)
        quest:Activate("do_job")
    end,
}

:AddObjective{
    id = "store_offer_barons",
}

:AddObjective{
    id = "store_offer_rise",
}

:AddSubQuest{
    id = "boss_fight",
    title = "Return to {bar}",
    quest_id = "ROOK_STORY_BOG_BOSS",
    on_activate = function(quest)
        TheGame:GetGameState():AddPendingEvent()
    end,
    mark = {"bar"},
    on_complete = function(quest)
        quest:Activate("day_over")
    end,
}

:AddSubQuest{
    id = "boss_fight_2",
    title = "Return to {bar}",
    quest_id = "ROOK_STORY_BOG_ROBOT",
    on_activate = function(quest)
        TheGame:GetGameState():AddPendingEvent()
    end,
    mark = {"bar"},
    on_complete = function(quest)
        quest:Activate("day_over")
    end,
}
-------------------------


:AddObjective{
    id = "do_job",
    hide_in_overlay = true,
    events = 
    {
        quests_changed = function(quest, event_quest)
            if quest.param.current_job == event_quest and not event_quest:IsActive() then

                quest:Complete("do_job")
                
                if quest.param.job_history and #quest.param.job_history == 1 then
                    if event_quest:IsComplete() then
                        local faction = quest.param.did_primary_first and quest.param.main_quest.param.faction_pick or quest.param.main_quest.param.alt_faction_pick
                        if faction == "SPARK_BARONS" then
                            quest:Activate("store_offer_barons")
                        else
                            quest:Activate("store_offer_rise")
                        end
                    end
                end

            end
        end
    },

    on_complete = function(quest) 
        quest.param.job_history = quest.param.job_history or {}
        if quest.param.current_job then
            table.insert(quest.param.job_history, quest.param.current_job)
        end
        quest.param.current_job = nil
        
        if #quest.param.job_history == 1 then
            quest:Activate(quest.param.did_primary_first and "pick_followup_secondary" or "pick_followup_primary")
        elseif #quest.param.job_history == 2 then
            quest:Activate("check_drop_point")
        elseif #quest.param.job_history == 3 then            


            local boss_def = TheGame:GetGameProfile():GetNoStreakRandom("ROOK_DAY_2_BOSS_PICK", {"boss_fight", "boss_fight_2"}, 2)
            quest:Activate(boss_def)
        end
    end,
}


:AddObjective{
    id = "check_drop_point",
    title = "Check the drop point",
    desc = "File your daily report at the drop point.",
    on_activate = function(quest)
        UIHelpers.PassTime(DAY_PHASE.NIGHT)
        TheGame:GetGameState():AddPendingEvent()
        QuestUtil.SpawnOpportunities()
    end,
    mark = {"dead_drop_spot"}

}


:AddObjective{
    id = "day_over",
    title = "Return to {bar} and rest.",
    desc = "{hebbel} has a room ready for you. Time to recharge.",
    mark = {"bar", "hebbel"},
    on_activate = function(quest) 
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true
    end,
    
    on_complete =function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
    end

}


QDEF:AddConvo("day_over", "hebbel")
    :Loc{
        OPT_GET_A_ROOM = "Rest in the back",
        DIALOG_GET_ROOM = [[
            player:
                !handwring
                $miscPersuasive
                Now, about that fine accommodation you so graciously offered...
            agent:
                $neutralResigned
                I'll go wake up the cockroaches.
                !exit
            player:
                Charming.
                !exit
            * {agent} shows you to your room.
        ]],
        DIALOG_WAKE = [[
            * You open your eyes to a new day. At least this one doesn't come with a new name and identity, too.
        ]],
    }

    :Hub(function(cxt) 
            cxt:Opt("OPT_GET_A_ROOM")
                :SetQuestMark()
                :PreIcon(global_images.sleep)
                :Fn(function(cxt) 
                    local grog = cxt.location
                    cxt:Dialog("DIALOG_GET_ROOM")
                    cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR.inn_room") )
                    grog:SetPlax()
                    ConvoUtil.DoSleep(cxt, "DIALOG_WAKE")
                    cxt.quest:Complete()

                    cxt:Opt("OPT_LEAVE")
                        :MakeUnder()
                        :Fn(function() 
                            cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR") )
                            cxt:End()
                        end)                    

                end)
        end)    
    
    :AttractState("attract", function(cxt) return not cxt.quest.param.did_hebbel_recap end)
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    You ready to hit the sack?
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

        end)


----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- You do your first job for your primary handler 
QDEF:AddConvo("pick_first_job_primary", "primary_handler")
    :Loc{
        OPT_GET_WORK = "Get work",
        DIALOG_GET_WORK = [[
            player:
                !interest
                So, shall we begin? I imagine there's a great deal of work to be done yet, undermining the enemy.
        ]],

        DIALOG_RISE_OFFER = [[
            agent:
                We've been laying the groundwork since the small hours. Now it's your turn.

        ]],
        DIALOG_SPARK_OFFER = [[
            agent:
                Ah, yes, no time like the present, eh?
        ]],
    }

    :Hub(function(cxt) 

            cxt:Opt("OPT_GET_WORK")
                :Dialog("DIALOG_GET_WORK")
                :Dialog(cxt:GetAgent() == cxt.quest:GetCastMember("rise_contact") and "DIALOG_RISE_OFFER" or "DIALOG_SPARK_OFFER")
                :Fn(function()
                    local main_quest = cxt.quest.param.main_quest
                    cxt.quest.param.current_job = main_quest:GetQuestDef().SpawnFactionJobs( main_quest, 1, main_quest.param.faction_pick, nil, {"late_game"}, {giver = cxt:GetAgent()} )[1]

                    main_quest:GetQuestDef().PresentFactionJobChoice(cxt, {cxt.quest.param.current_job}, true, 
                            function(cxt, job) 
                                cxt.quest:Complete("pick_first_job_primary")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)
                end)
        end)    

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- You do your first job for your primary handler 
QDEF:AddConvo("pick_first_job_secondary", "secondary_handler")
    :Loc{
        OPT_GET_WORK = "Do some dirty work",
        DIALOG_GET_WORK = [[
            player:
                You have my assignment, I take it? 
        ]],

        DIALOG_RISE_OFFER = [[
            agent:
                !agree
                Hope you're ready for a challenge.

        ]],
        DIALOG_SPARK_OFFER = [[
            agent:
                Ah, how the tables have turned, eh?
        ]],
    }

    :Hub(function(cxt) 

            cxt:Opt("OPT_GET_WORK")
                :Dialog("DIALOG_GET_WORK")
                :Dialog(cxt:GetAgent() == cxt.quest:GetCastMember("rise_contact") and "DIALOG_RISE_OFFER" or "DIALOG_SPARK_OFFER")
                :Fn(function() 
                    local main_quest = cxt.quest.param.main_quest
                    cxt.quest.param.current_job = main_quest:GetQuestDef().SpawnFactionJobs( main_quest, 1, main_quest.param.alt_faction_pick, nil, {"late_game"}, {giver = cxt:GetAgent()} )[1]

                    main_quest:GetQuestDef().PresentFactionJobChoice(cxt, {cxt.quest.param.current_job}, true, 
                            function(cxt, job) 
                                cxt.quest:Complete("pick_first_job_secondary")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)
                end)
        end)    



----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- You do your second job for your secondary handler 
QDEF:AddConvo("pick_followup_secondary", "secondary_handler")
    :Loc{
        OPT_GET_WORK = "Do some dirty work",
        DIALOG_GET_WORK = [[
            player:
                What next?
        ]],

        DIALOG_RISE_OFFER = [[
            agent:
                Right back at it, huh? You <i>are</i> a professional.

        ]],
        DIALOG_SPARK_OFFER = [[
            agent:
                Cut to the chase, don't you, old chap?
        ]],
    }

    :Hub(function(cxt) 

            cxt:Opt("OPT_GET_WORK")
                :Dialog("DIALOG_GET_WORK")
                :Dialog(cxt:GetAgent() == cxt.quest:GetCastMember("rise_contact") and "DIALOG_RISE_OFFER" or "DIALOG_SPARK_OFFER")
                :Fn(function() 
                    local main_quest = cxt.quest.param.main_quest
                    cxt.quest.param.current_job = main_quest:GetQuestDef().SpawnFactionJobs( main_quest, 1, main_quest.param.alt_faction_pick, nil, {"late_game"}, {giver = cxt:GetAgent()} )[1]

                    main_quest:GetQuestDef().PresentFactionJobChoice(cxt, {cxt.quest.param.current_job}, true, 
                            function(cxt, job) 
                                cxt.quest:Complete("pick_followup_secondary")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)
                end)
        end)    


----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------            
-- You do your second job for your primary handler 
QDEF:AddConvo("pick_followup_primary", "primary_handler")
    :Loc{
        OPT_GET_WORK = "Do some dirty work",
        DIALOG_GET_WORK = [[
            player:
                Once more unto the breach, I suppose?
        ]],

        DIALOG_RISE_OFFER = [[
            agent:
                Afraid so. We're almost done for the day, though.

        ]],
        DIALOG_SPARK_OFFER = [[
            agent:
                Tireless as always, I see. 
        ]],
    }

    :Hub(function(cxt) 

            cxt:Opt("OPT_GET_WORK")
                :Dialog("DIALOG_GET_WORK")
                :Dialog(cxt:GetAgent() == cxt.quest:GetCastMember("rise_contact") and "DIALOG_RISE_OFFER" or "DIALOG_SPARK_OFFER")
                :Fn(function() 
                    local main_quest = cxt.quest.param.main_quest
                    cxt.quest.param.current_job = main_quest:GetQuestDef().SpawnFactionJobs( main_quest, 1, main_quest.param.faction_pick, nil, {"late_game"}, {giver = cxt:GetAgent()} )[1]

                    main_quest:GetQuestDef().PresentFactionJobChoice(cxt, {cxt.quest.param.current_job}, true, 
                            function(cxt, job) 
                                cxt.quest:Complete("pick_followup_primary")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)
                end)
        end)    



QDEF:AddConvo("check_drop_point")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("dead_drop_spot") end)
        :Loc{
            DIALOG_INTRO = [[
                * You arrive at the drop point, this time without ambush.
                player:
                    !left
                * You jot quickly in a bird scratch only you and a few others understand, detailing what you've learned of the Rise and the Spark Barons.
                * You file your report, and find a message left by your counterpart on the other side of the wall.
            ]],

        }
        :Fn(function(cxt) 
            
            cxt:Dialog("DIALOG_INTRO")

            cxt.quest.param.drop_job = cxt.quest.param.main_quest:GetQuestDef().SpawnFactionJobs(cxt.quest.param.main_quest, 1, nil, {"dead_drop"} )[1]
            TheGame:GetGameState():GetMainQuest():GetQuestDef().PresentFactionJobChoice(cxt, {cxt.quest.param.drop_job}, true, 
                    function(cxt, job) 
                        cxt.quest:Complete("check_drop_point")
                        
                        cxt.quest.param.current_job = cxt.quest.param.drop_job
                        cxt.quest:Activate("do_job")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)


        end)



QDEF:AddConvo("store_offer_barons", "spark_contact")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            
            --if you did the job successfully
            DIALOG_INTRO = [[
                agent:
                    Thanks for coming here first today.
                    If you need a little help getting things done out there, you can talk to my quartermaster over there about hiring out some automechs.

            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest.param.main_quest:GetQuestDef().EnhanceVendor(cxt.quest.param.main_quest, "SPARK_BARONS")
            cxt.quest:Complete("store_offer_barons")
        end)        



QDEF:AddConvo("store_offer_rise", "rise_contact")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            
            --if you did the job successfully
            DIALOG_INTRO = [[
                agent:
                    I appreciate you came here first this morning. It shows dedication to the cause.
                    The quartermaster at the HQ might also have some spare cobbledogs you can use.
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest.param.main_quest:GetQuestDef().EnhanceVendor(cxt.quest.param.main_quest, "RISE")
            cxt.quest:Complete("store_offer_rise")
        end)                