local QDEF = QuestDef.Define
{
    title = "Bogged Down",
    --desc = [[]],
    --icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    qtype = QTYPE.STORY,
    icon = engine.asset.Texture("icons/quests/rook_story_working_both_sides.tex"),

    --put other handler into hiding once you've picked
    is_in_hiding = function(quest, agent)
        return quest:GetCastMember("rise_contact") == agent or quest:GetCastMember("spark_contact") == agent
    end,

    --put handlers in their houses before you've picked, and lock your pick in place once you make it
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("spark_contact"), location = quest:GetCastMember("baron_hq"), role = CHARACTER_ROLES.CONTACT} )
        table.insert (t, { agent = quest:GetCastMember("rise_contact"), location = quest:GetCastMember("rise_hq"), role = CHARACTER_ROLES.CONTACT} )
    end,
   
    events = 
    {
    },
    
    on_start = function(quest)
        quest:GetCastMember("rise_contact"):SetBuildOverride("female_rise_kalandra_build")
        quest:Activate("talk_to_hebbel")
    end,
}

-------------
:AddLocationCast{
    cast_id = "baron_hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_BARON_HQ"))
    end,
}

:AddCastByAlias{
    cast_id = "spark_contact",
    alias = "SPARK_CONTACT",
    no_validation = true,
}


-------------
:AddLocationCast{
    cast_id = "rise_hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_RISE_HQ"))
    end,
}

:AddCastByAlias{
    cast_id = "rise_contact",
    alias = "KALANDRA",
    no_validation = true,
}


--ie the one you give the intel.
:AddCast{
    cast_id = "first_handler",
    when = QWHEN.MANUAL,
}

:AddCast{
    cast_id = "second_handler",
    when = QWHEN.MANUAL,
}


-------------
:AddLocationCast{
    cast_id = "bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR"))
    end,
}

:AddCastByAlias{
    cast_id = "hebbel",
    alias = "HEBBEL",
}

-----------------------------

-------------------------

:AddObjective{
    id = "talk_to_hebbel",
    title = "Check with {hebbel}",
    desc = "See if {hebbel} has any information for you.",
    mark = {"hebbel"},
    on_complete = function(quest) 
        quest:Activate('deliver_info_to_rise')
        quest:Activate('deliver_info_to_barons')
        TheGame:GetGameState():AddPendingEvent()
    end,
}

:AddObjective{
    id = "deliver_info_to_rise",
    title = "Give the info to {rise_contact}",
    desc = "If you can convince {rise_contact} to act on the intelligence, you might be able to tag along.",
    mark = {"rise_contact"},
    icon = engine.asset.Texture("icons/quests/rook_story_work_for_kalandra.tex"),
    on_complete = function(quest)
        quest:Cancel("deliver_info_to_barons")

    end

}
:AddObjective{
    id = "deliver_info_to_barons",
    title = "Give the info to {spark_contact}",
    desc = "If you can convince {spark_contact} to act on the intelligence, you might be able to tag along.",
    mark = {"spark_contact"},
    icon = engine.asset.Texture("icons/quests/rook_story_work_for_fellemo.tex"),
    on_complete = function(quest)
        quest:Cancel("deliver_info_to_rise")
    end
}


:AddObjective{
    id = "do_job_1",
    hide_in_overlay = true,
    events = 
    {
        quests_changed = function(quest, event_quest)
            if quest.param.current_job == event_quest and not event_quest:IsActive() then
                quest:Complete("do_job_1")
            end
        end
    },

    on_complete = function(quest) 
        quest.param.current_job = nil
        quest:Activate("done_job_1")
        QuestUtil.SpawnOpportunities()
        TheGame:GetGameState():AddPendingEvent()
    end,
}

:AddObjective{
    id = "done_job_1",
    hide_in_overlay = true,
    on_complete = function(quest) 
        quest:Activate("check_in_with_second_handler")
    end,
    mark = {"first_handler"}
}

:AddObjective{
    id = "check_in_with_second_handler",
    title = "Work for {second_handler}",
    desc = "Keep up appearances by doing some work for {second_handler}",
    mark = {"second_handler"},
}


:AddObjective{
    id = "do_job_2",
    hide_in_overlay = true,
    events = 
    {
        quests_changed = function(quest, event_quest)
            if quest.param.current_job == event_quest and not event_quest:IsActive() then
                quest:Complete("do_job_2")
            end
        end
    },
    
    on_complete = function(quest) 
        quest.param.current_job = nil
        quest:Activate("done_job_2")
        QuestUtil.SpawnOpportunities()
    end,
}

:AddObjective{
    id = "done_job_2",
    hide_in_overlay = true,
    on_complete = function(quest) 
        quest:Activate("get_bog_briefing")
    end,
    mark = {"second_handler"}
}


:AddObjective{
    id = "get_bog_briefing",
    title = "Talk to {first_handler} about the Boggers",
    desc = "Night has fallen, so {first_handler} should be ready to deal with the Bogger cache. Report to them to find out what they decided to do.",
    on_activate = function(quest) 
        UIHelpers.PassTime(DAY_PHASE.NIGHT)
    end,
    on_complete = function(quest)
        quest:Activate("do_bogger_quest")
    end,
    mark = {"first_handler"},

}

:AddSubQuest{
    id = "do_bogger_quest",
    quest_id = "ROOK_STORY_BOGGER_RAID",

    addCast = function(quest, cast)
        cast.giver = quest:GetCastMember("first_handler")
    end,

    on_complete = function(quest)
        quest:Activate("file_report")    
        TheGame:GetGameState():AddPendingEvent()
    end,
}

:AddSubQuest{
    id = "file_report",
    quest_id = "ROOK_STORY_DEAD_DROP_DAY_3",
    addParams = function(quest, params)
        params.working_with = quest:GetCastMember("first_handler"):GetFactionID()
    end,

    on_complete = function(quest)
        quest:Activate("day_over")    
    end,
}


:AddObjective{
    id = "day_over",
    title = "Return to {bar} and rest",
    desc = "{hebbel} has a room ready for you. Time to recharge.",
    mark = {"hebbel"},
    on_activate = function(quest) 
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true
    end,
    
    on_complete =function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
    end

}


--you wake up, and hebbel gives you a package that was delivered during the night.
QDEF:AddConvo("talk_to_hebbel", "hebbel")
    :ConfrontState("STATE_TALK")
        :Loc{
            DIALOG_INTRO = [[
                * {hebbel} grunts like a grease fire as you approach. 
                player:
                    !left
                hebbel:
                    !right
                    $neutralDirect
                    Package came for you.
                    !give
                player:
                    Who left it?
                    !take
                hebbel:
                    !handwave
                    $neutralWhatever
                    Didn't leave a name. Just a package.
                player:
                    !thought
                    You have a shrewd eye in there somewhere, {hebbel}, I'm sure of it.
                hebbel:
                    !crossed
                    Just take it before it explodes. That'd be bad for business.
                    !exit
                * {hebbel} resumes {hebbel.hisher} duties, making sure to keep the bar between {hebbel.himher}self and your package.
            ]],
            OPT_OPEN_MESSAGE = "Open the package",

            DIALOG_OPEN_MESSAGE = [[
                * You hold your breath and pull the string.
                !wince
                $miscRelieved
                player:
                    $neutralThoughtful
                    Hmmm. No explosion. 
                    !happy
                    $miscMischievous
                    Must be my lucky day.
                * Inside you find a bundle of loose papers and a note addressed to you in a complex cipher.
                * Nevertheless, years of training have allowed you to grasp most of the work at a glance:
                ** <i>Candidate device... located</>
                ** <i>Deep bog... rumored hermetic faction</i>
                * The next portion requires a more clear translation, but after a moment of study its meaning unfurls before you:
                ** <i>Secure the loyalty of one of your contacts using the enclosed intelligence. Either one of them should know more about these "Boggers".</>
                * The intelligence file itself reveals the location of a Bogger encampment, and implies that they've found a large cache of ancient technology.
                * This is the kind of lead that either the Barons or the Rise would literally kill for.
                *** You have received instructions to give a bundle of intelligence to one of your contacts.
            ]]
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:Opt("OPT_OPEN_MESSAGE")
                :Dialog("DIALOG_OPEN_MESSAGE")
                :CompleteQuest("talk_to_hebbel")
                :Fn(function() StateGraphUtil.AddEndOption(cxt) end)
        end)


local function PresentJobChoice(cxt, tracking_objective)
    local quest_options = cxt.quest.param.main_quest:GetQuestDef().SpawnFactionJobs(cxt.quest.param.main_quest, 2, cxt:GetAgent():GetFactionID(), {"late_game"}, nil, {giver = cxt:GetAgent()})
    TheGame:GetGameState():GetMainQuest():GetQuestDef().PresentFactionJobChoice(cxt, quest_options, true, 
            function(cxt, job) 
                cxt.quest.param.current_job = job 
                cxt.quest:Activate(tracking_objective)
                StateGraphUtil.AddEndOption(cxt)
            end)
end

--you give the info to kalandra
QDEF:AddConvo("deliver_info_to_rise", "rise_contact")
     :Loc{
        OPT_GIVE_INTELLIGENCE = "Give {agent} the dossier",
        DIALOG_GIVE_INTELLIGENCE = [[
            player:
                I have something for you.
                !give
            agent:
                !take
            * {agent} leafs through the papers.
            agent:
                !thought
                So the Boggers found themselves a weapons cache...
                Those bog-addled fools are more likely to worship them than put them to good use.
                In the right hands, those weapons could topple the Barons.
            player:
                So you want me to retrieve them?
            agent:
                No, it's too big a job for just one person. And I don't trust this report. Not yet at least.
                We'll discuss this again soon. I need to consider our options.
        ]],
        OPT_ASK_ABOUT_BOGGERS = "Ask about the Boggers",
        DIALOG_ASK_ABOUT_BOGGERS = [[
            player:
                !crossed
                Who are these Boggers?
            agent:
                $neutralResigned
                They worship the bog like it was some kind of living thing.
                Most of them are ex-Baron. Too much free time, I guess, and they get philosophical.
                !shrug
                They mostly keep to the deeper parts of the bog, so we don't encounter them much.
        ]],
        OPT_GET_WORK = "Get work",
        DIALOG_GET_WORK = [[
            player:
                How can I be of service to the Rise while you're busy hatching plans?
            agent:
                !agree
                There's always work to be done.
        ]],

        OPT_ASK_ABOUT_PLAN = "Press {agent} for details on {agent.hisher} plan",
        DIALOG_ASK_ABOUT_PLAN = [[
            player:
                So what are you going to do about the cache?
            agent:
                $neutralDirect
                If it's real, I intend to take it.
            player:
                !interest
                By force?
            agent:
                If it comes to that. But there might be a better way. 
                !thought
                $neutralResigned
                The Boggers are a hard bunch to read.
        ]]

    }
    :AttractState("STATE_TALK")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !salute
                    Good morning, {player}. Are you ready to stand up for what's right?
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
        end)
    :Hub(function(cxt)
        cxt:Opt("OPT_GIVE_INTELLIGENCE")
            :SetQuestMark()
            :Fn(function()
                    cxt.quest:AssignCastMember("first_handler", cxt:GetAgent())
                    cxt.quest:AssignCastMember("second_handler", cxt.quest:GetCastMember("spark_contact"))
                    TheGame:GetGameState():GetMainQuest().param.expedition_faction = "RISE"
                    cxt:Dialog("DIALOG_GIVE_INTELLIGENCE")
                    cxt.quest:Complete("deliver_info_to_rise")
                    
                    cxt:RunLoop(function() 
                        
                        cxt:Question("OPT_ASK_ABOUT_PLAN", "DIALOG_ASK_ABOUT_PLAN")
                        cxt:Question("OPT_ASK_ABOUT_BOGGERS", "DIALOG_ASK_ABOUT_BOGGERS")
                        
                        cxt:Opt("OPT_GET_WORK")
                            :Fn(function() 
                                cxt:Dialog("DIALOG_GET_WORK")
                                PresentJobChoice(cxt, "do_job_1")
                            end)
                            
                    end)

                    
                end)
    end)


--you give the info to fellemo
QDEF:AddConvo("deliver_info_to_barons", "spark_contact")
    :Loc{
        OPT_GIVE_INTELLIGENCE = "Give {agent} the dossier",

        DIALOG_GIVE_INTELLIGENCE = [[
            player:
                I have intelligence to deliver.
                !give
            agent:
                !take
            * You hand {agent} the dossier. {agent.HeShe} pretends to skim it.
            agent:
                !neutralDubious
                So... what am I looking at?
            player:
                $miscPersuasive
                The Boggers have apparently found a large cache of artifacts.
            agent:
                !handwave
                Those loons, eh? I'll dispatch a patrol to scare them off and collect the loot. It'll be a nice bump in this quarter's earnings.
            player:
                $neutralDubious
                Yes, but...
                $miscPersuasive
                The loot in question seems to be a pile of rare weaponry. A regular patrol would only end up fertilizing the bog if you sent them.
            agent:
                !interest
                $neutralWhatever
                I see. 
                !thought                    
                Er...
            * You are reminded of a particularly dull Aerostat recruit you knew in a past life.
            * {agent} never was good at lateral thinking.
            agent:
                ...I'll schedule a meeting with Arint, and we'll set something up.

        ]],
        OPT_ASK_ABOUT_BOGGERS = "Ask about the Boggers",
        DIALOG_ASK_ABOUT_BOGGERS = [[
            player:
                Who are these Boggers?
            agent:
                $neutralDirect
                Crackpots who worship the bog.
                !spit
                $angryAside
                Most of them are workers who wandered off the job. Lazy sods.
                They mostly keep to themselves, but we have had to execute a few for stealing from our dig sites.
                !shrug
                $neutralWhatever
                Cost of doing business, I suppose.
        ]],
        OPT_GET_WORK = "Get work",
        DIALOG_GET_WORK = [[
            player:
                !interest
                While you're consulting with Arint, do you have anything else that needs to be done?
            agent:
                Of course. The usual. 
        ]],

        OPT_ASK_ABOUT_PLAN = "Press {agent} for details on {agent.hisher} plan",
        DIALOG_ASK_ABOUT_PLAN = [[
            player:
                So what are you going to do about the cache?
            agent:
                !happy
                Eager for action, eh, old man? You always were a ruthless tactician.
                But this is above your paygrade, I'm afraid. You've kicked it upstream—now sit tight and await your orders.
            player:
                !dubious
                $neutralDubious
                If you wish my help in planning the assault—
            agent:
                !neutral
                $neutralDirect
                I said what I said, Rook. At ease.
                I'm sure Arint will think of something.
        ]]
    }
    :AttractState("STATE_TALK")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    !happy
                    Ah, {player}. You caught me reminiscing on our Aerostat days.
                    $miscMischievous
                    I wouldn't trade my Spark Baron commission for the poor life of reconnaissance, but there was something about those days, eh?
                    $miscNostalgic
                    The brisk Rentorian air. The allure of danger in every shadow.
                    !laugh
                    Good times.
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
        end)
    :Hub(function(cxt)
        
        cxt:Opt("OPT_GIVE_INTELLIGENCE")
            :SetQuestMark()
            :Fn(function() 
                    cxt.quest:AssignCastMember("first_handler", cxt:GetAgent())
                    cxt.quest:AssignCastMember("second_handler", cxt.quest:GetCastMember("rise_contact"))
                    TheGame:GetGameState():GetMainQuest().param.expedition_faction = "SPARK_BARONS"
                    cxt:Dialog("DIALOG_GIVE_INTELLIGENCE")
                    cxt.quest:Complete("deliver_info_to_barons")
                    
                    cxt:RunLoop(function() 
                        
                        cxt:Question("OPT_ASK_ABOUT_PLAN", "DIALOG_ASK_ABOUT_PLAN")
                        cxt:Question("OPT_ASK_ABOUT_BOGGERS", "DIALOG_ASK_ABOUT_BOGGERS")
                        
                        cxt:Opt("OPT_GET_WORK")
                            :Fn(function() 
                                cxt:Dialog("DIALOG_GET_WORK")
                                PresentJobChoice(cxt, "do_job_1")
                            end)
                            
                    end)

                end)
    end)


--bridge conversation that plays after you hand in the quest. the handler gives you an update about the boggers (ask abouts?), and then tells you to go maintain your cover
QDEF:AddConvo("done_job_1", "first_handler")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            DIALOG_INTRO = [[
                {spark_barons ? 
                    agent:
                        !dubious 
                        $miscRelieved
                        Things are progressing quite well. I don't know how to say it but...
                        Even when I fought to have you brought on board, old man, I never suspected you would deliver so thoroughly.
                    player:
                        !interest
                        You know I take pride in my work.
                    agent:
                        !agree
                        Yes, yes. And you should!
                        $happyAmazed
                        The Boggers are proving too fortuitous a find. 
                        Not only have they unearthed rare treasures, but I have it on good authority that the Rise are interested in their resources, as well.
                        !thought
                        $neutralThoughtful
                        Perhaps we can set a trap that might see Kalandra finished off once and for all—it almost makes the bounty negligible.
                        <i>Almost</i>. 
                    player:
                        !crossed
                        $neutralDirect
                        I see. And what does this compounding entail?
                    agent:
                        We need to lure that Prindo girl into a trap, of course. 
                        !cruel 
                        $miscMischievous
                        Pay her a visit. See if she doesn't need your, ah, <i>assistance</i> in any matter. 

                }
                
                {rise ? 
                    agent:
                        !happy
                        $happyDefault
                        I can't believe our luck. You know, at first I thought you were as likely to sabotage as help me. 
                        But desperate times call for desperate measures. And so far? You're a ray of hope. 
                    player:
                        !humouring
                        $happyJoke
                        It's not like you to speak so optimistically, Kalandra. I'll take that as a compliment.
                    agent:
                        $miscMischievous
                        Good. 'Cause that's how I meant it, and it'll take another miracle to make me repeat it.
                        !overthere
                        Thanks to you and those Boggers, we've got a chance to end the Barons once and for all.
                        Or at least end their tyranny. 
                    player:
                        !agree 
                        $neutralThoughtful
                        Ah, two oshnus with one saltlick, I take it?
                    agent:
                        !eureka
                        Got it in one. But what we <i>need</i> is for Fellemo to follow your lead.
                        Time to milk your old war buddy for what he's worth, {player}. 

                }

            ]],


            DIALOG_ON_LEAVE = [[
                {spark_barons ? 
                    agent:
                        It's very important you have her trust, so you have my permission to behave accordingly.
                    player:
                        $miscAlluring
                        As always I appreciate the flexibility. 
                    agent:
                        !agree
                        I'm sure. Just be certain you don't appreciate it <i>too</i> freely, eh?
                }
                
                {rise ? 
                    agent:
                        Whatever you gotta do to make him sure you're his loyal friend, do it. 
                        All of the workers under me would gladly die for the cause. A small sacrifice if it meant freedom for us all.
                    player:
                        !interest
                        That's quite the commitment.
                    agent:
                        !suspicious
                        We signed contracts that will follow us to our graves, if not see us there all the sooner.
                        $neutralDirect
                        None of us are afraid of permanence. 
                }

            ]],

        }
        :Fn(function(cxt) 
            --we can structure this however... depends on what kind of witty reparte rook has with each handler.
            cxt.quest:Complete("done_job_1")
            cxt:Dialog("DIALOG_INTRO")
            StateGraphUtil.AddEndOption(cxt)                
                :Dialog("DIALOG_ON_LEAVE")
        end)


--you go to check in with the handler you didn't give the intel to. They maybe mention something about bogger rumors? 
QDEF:AddConvo("check_in_with_second_handler", "second_handler")
    :AttractState("STATE_TALK")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                {spark_barons ?
                    !angry_shrug
                    $angryTakenAback
                    {player}! Where in the deepest trenches have you been? 
                    I've heard numerous reports of the Rise having some lead on the Boggers, but nothing has come back to me. 
                    $neutralTakenAback
                    Surely you must have heard something?
                }
                {rise ?
                    !dubious
                    Look who finally turns up. We don't have any time to spare.
                    Folks are saying the Barons have some trick up their sleeve—something to do with the Boggers that supposedly live out in the wilds. 
                    !overthere
                    $miscResigned
                    Whatever it is they're up to, it can't be good. 
                    So tell me what you know about it.
                }
            ]],
            OPT_TELL_ABOUT_BOGGER_CACHE = "Tell {agent} about the cache",
            DIALOG_TELL_ABOUT_BOGGER_CACHE = [[
                player:
                    !agree
                    It's true. {first_handler} has a lead on a cache of Bogger weapons.
                
                {spark_barons ?
                    agent:
                        Aha. 
                        !thought
                        $neutralThoughtful
                        I suppose I shouldn't have had that messenger issued five demerits for falsehood...
                        $neutralDirect
                        Ah well! No sense ruminating over past transgressions. We can only move forward, eh?
                    player:
                        !hips
                        Yes—and so I await your orders. What would you have me do?
                    agent:
                        !humouring
                        $happyAmazed
                        My dear Captain—those words are truly the music of the deep. 
                        What I would have you do is follow Kalandra along on her little mission... and, when the time is right...
                        !throatcut
                        $angryCruel
                        See that those weapons get put to good use.
                }
                
                {rise ?
                    agent:
                        !interest 
                        $happyAmazed
                        You're joking. 
                        $miscNostalgic
                        Finding a payload like that is something every Rise dreams of. 
                        It's the way they tell us we can buy our way out of our contracts—even become a Baron ourselves. 
                        Only one person ever found a cache that big. 
                    player:
                        !suspicious
                        You mean Kashio, I take it.
                    agent:
                        $neutralResigned
                        I suppose everyone's heard of her by now. Not like anyone else ever discovered automechs. 
                        !point
                        Look... with that cache, we could negotiate better terms for all of us. The Barons would have no choice but to listen.
                        $miscPersuasive
                        You need to tag along, learn what you can, and make sure those weapons get back to us. 
                }
            ]],

            OPT_KEEP_SECRET = "Keep the cache a secret",
            DIALOG_KEEP_SECRET = [[
                player:
                    !shrug 
                    I can't say I've heard anything of the sort. I'm sorry to disappoint. 
                
                {spark_barons ?
                    agent:
                        !disappoint
                        $neutralResigned
                        Seaspit. Well, keep your ear to the ground, and tell me if you learn anything.
                }
                
                {rise ?
                    agent:
                        !disappoint
                        $neutralResigned
                        Blast it. Keep your ear to the ground, and tell me if you learn anything.
                }
            ]]
        }
        :Fn(function(cxt) 
            
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_TELL_ABOUT_BOGGER_CACHE")
                :Dialog("DIALOG_TELL_ABOUT_BOGGER_CACHE")
                :Fn(function() cxt.quest.param.told_second_handler_about_cache = true end)
                :GoTo("STATE_GET_WORK")


            cxt:Opt("OPT_KEEP_SECRET")
                :Dialog("DIALOG_KEEP_SECRET")
                :GoTo("STATE_GET_WORK")
        end)

    :State("STATE_GET_WORK")
        :Loc{
            OPT_ASK_FOR_WORK = "Ask for work",
            DIALOG_ASK_FOR_WORK = [[
                player:
                    Idle hands become slippery tentacles, as they say. 
                    What do you have to keep a fellow of my skill occupied, lest the tides catch me up in their whims?
                
                {spark_barons ?
                    agent:
                        There's always something. Let me look through the day's reports. 
                }

                {rise ?
                    agent:
                        Hm. We did just get some fresh intel from one of the scouts. 
                }
            ]]
        }
    
        :Fn(function(cxt) 
            cxt.quest:Complete("check_in_with_second_handler")
            cxt:Opt("OPT_ASK_FOR_WORK")
                :SetQuestMark()
                :Dialog("DIALOG_ASK_FOR_WORK")
                
                :Fn(function()
                    cxt:RunLoop(function() 
                        PresentJobChoice(cxt, "do_job_2")
                    end)
                end)
        end)
            

--bridge conversation that plays after you're done with the second handler's quest.
QDEF:AddConvo("done_job_2", "second_handler")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            DIALOG_INTRO = [[
                { told_second_handler_about_cache ?
                    {spark_barons ? 
                        agent:
                            !handwring
                            $angryCruel
                            Ah, the anticipation moistens the skin, doesn't it? 
                        player:
                            !agree
                            Yes, though in humans that's considered an unappealing feature.
                        agent:
                            Humans! I suppose that's what makes that Kalandra trust you so much. 
                            !chuckle
                            The fool. She's making a move on the cache, I'm sure of it. 
                            And undoubtedly she'll send you to recover it. 
                            !neutral 
                            $neutralDirect
                            Remember our arrangement, Captain. It would be truly painful to have to kill you. 
                    }
                    {rise ? 
                        agent:
                            !permit
                            $miscHushedUrgent
                            You feel that? The tension's so thick even the flies can't slice through it. 
                            Hope you're ready to move. I want to see the far side of this. 
                        player:
                            !hips
                            $neutralDirect
                            Always.
                        agent:
                            Whatever history you have with Fellemo is the best weapon we could've asked for. 
                            And he's a fool if he doesn't appreciate that.
                            !point
                            When the time comes... I hope you remember who really believes in you. 
                    }
                }
                { not told_second_handler_about_cache ?

                    {spark_barons ? 
                        agent:
                            !angry 
                            $angryPatienceLost
                            Confound it!
                        player:
                            Problems?
                        agent:
                            !angry_shrug
                            I've chased every rumor on that supposed Bogger cache, and they all go up like so much swamp gas. 
                        player:
                            !interest
                            $miscPersuasive
                            Perhaps they truly were just rumors, then.
                        agent:
                            $angryAside
                            So it would be easy to believe.
                        * {agent} turns from you, {agent.hisher} expression shrouded. 
                    }
                    {rise ? 
                        * {agent} riffles through the maps in front of {agent.himher}, surrounded by a nest of crinkled paper. You step carefully over a discarded wad on the floor.
                        agent:
                            !angry
                            $angryPatienceLost
                            No, no, no, no. 
                        player:
                            Have you misplaced something, my dear?
                        agent:
                            !angry_shrug
                            $angryAside
                            It's the blasted Barons! And those damn rumors!
                            They <i>might</i> have found a lead on a stash of Vagrant tech. That stash <i>might</i> be incredibly dangerous weapons.
                            Ifs and maybes don't help me. Don't help any of us.
                        player:
                            !dubious 
                            $miscPersuasive
                            Could it be that Fellemo is merely trying to goad you into recklessness? 
                            Perhaps they aren't so much rumors as purposefully-seeded doubts.
                        agent:
                            Hm. And you'd know best, I suppose. 
                        * {agent} turns from you, {agent.hisher} expression shrouded. 
                    }
                }
            ]],

        }
        :Fn(function(cxt) 
            --we can structure this however... depends on what kind of witty reparte rook has with each handler.
            cxt.quest:Complete("done_job_2")
            cxt:Dialog("DIALOG_INTRO")
            StateGraphUtil.AddEndOption(cxt)                
        end)



--you return to the first handler, and they set you up with some goons to go take on the boggers in an "active negotiation"
QDEF:AddConvo("get_bog_briefing", "first_handler")
    :Loc{
        OPT_READY = "Report for duty",
    }
    :AttractState("attr")
        :Loc{
            DIALOG_INTRO = [[
                {spark_barons? 
                    agent:
                        Are you ready?
                }
                {rise? 
                    agent:
                        Are you ready?
                }
            ]],
        }
        :Fn(function(cxt)
                cxt:Dialog("DIALOG_INTRO")
            end)
    
    :Hub(function(cxt) 
        cxt:Opt("OPT_READY")
            :GoTo("STATE_START_JOB")
    end)

    :State("STATE_START_JOB")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    I'm ready.
            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Complete("get_bog_briefing")
            cxt:PlayQuestConvo( cxt.quest.param["sub_quest_ROOK_STORY_BOGGER_RAID"], QUEST_CONVO_HOOK.INTRO )
        end)


QDEF:AddConvo("day_over")
    :ConfrontState("CONF", function(cxt) return cxt.quest.param.waiting_for_finale end)
        :Loc{
            DIALOG_WAKE = [[
                * You wake with the sense the world is ready for you.
            ]],
        }
        :Fn(function(cxt) 
            ConvoUtil.DoSleep(cxt, "DIALOG_WAKE")
            cxt.quest:Complete()

            cxt:Opt("OPT_LEAVE")
                :MakeUnder()
                :Fn(function() 
                    cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR") )
                    cxt:End()
                end)
        end)

QDEF:AddConvo("day_over", "hebbel")
    :Loc{
        OPT_GET_A_ROOM = "Rest in the back",
        DIALOG_GET_ROOM = [[
            player:
                No rest for the wicked, eh, Hebbel?
            * {agent} shows you to your room.
            agent:
                !exit
            player:
                !exit
        ]],
        DIALOG_WAKE = [[
            * You wake with the taste of blood in your mouth. You're not sure who it belongs to.
        ]],
    }

    :Hub(function(cxt) 
            cxt:Opt("OPT_GET_A_ROOM")
                :SetQuestMark()
                :PreIcon(global_images.sleep)
                :Fn(function(cxt) 
                    cxt:Dialog("DIALOG_GET_ROOM")
                    cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR.inn_room") )
                    ConvoUtil.DoSleep(cxt, "DIALOG_WAKE")
                    cxt.quest:Complete()

                    cxt:Opt("OPT_LEAVE")
                        :MakeUnder()
                        :Fn(function() 
                            cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GB_NEUTRAL_BAR") )
                            cxt:End()
                        end)

                end)
        end)    
  