--this is the day 1 boss fight
local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    title = "Dead Drop",
    icon = engine.asset.Texture("icons/quests/rook_story_dead_drop.tex"),
    on_init = function(quest)

        if not quest.param.working_with then
            quest.param.working_with = "RISE"
        end

        if quest.param.working_with == "RISE" then
            quest:AssignCastMember("target", quest:GetCastMember("spark_second"))
        else
            quest:AssignCastMember("target", quest:GetCastMember("rise_second"))
        end
    end,
}

:AddCastByAlias{
    cast_id = "spark_second",
    alias = "SPARK_SECOND",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "rise_second",
    alias = "RISE_SECOND",
    no_validation = true,
}

:AddCast{
    cast_id = "target",
    when = QWHEN.MANUAL,
}



:AddLocationCast{
    cast_id = "dead_drop_spot",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_DEAD_DROP_SPOT"))
    end,
}

:AddObjective{
    id = "file_report",
    title = "Report on the Boggers",
    desc = "File a report about your experience with the boggers and the movement of the device.",
    mark = {"dead_drop_spot"},
    state = QSTATUS.ACTIVE,
}


local function DoWeaknessFight(cxt)
    cxt:Opt("OPT_DEFEND_SELF")
        :Dialog("DIALOG_DEFEND_SELF")
        :Battle{
            flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE,
            advantage = cxt.quest.param.neg_lost and TEAM.RED or nil,
            on_player_turn_start = function( battle )
                if cxt.quest.param.cards_won and not battle.cards_given then
                    local cards = {}
                    for i,id in ipairs(cxt.quest.param.cards_won) do
                        table.insert(cards, Battle.Card(id, battle:GetPlayerFighter()))
                    end
                    battle:DealCards(cards, battle:GetDrawDeck())
                    battle.cards_given = true
                end
            end,
        }
        :OnWin()
            :GoTo("STATE_POST_FIGHT")
end


QDEF:AddConvo("file_report")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("dead_drop_spot") end)
        :Loc{
            DIALOG_INTRO = [[
                * You notice long before you reach the drop point that someone has been here before you, and likely they're still lying in wait.
                * It seems you'll have to draw them out if you want an introduction. 
                player:
                    !left
                    !neutral_notepad
                * You make yourself look preoccupied, and sure enough, your guest reveals themselves. 
            ]],
            DIALOG_CONFRONTATION = [[
                {spark_barons? 
                    agent:
                        !right
                        !angry
                        Should've known you weren't worth your weight in peat. 
                    player:
                        You shouldn't be so hard on yourself, Arint. 
                    agent:
                        All your slick talk and mustache twirling. 
                        Fellemo might think he has you under heel but I know how far a spy can be trusted.
                        You've stolen our dirty little secrets long enough. Time to turn you into one. 
                        !fight
                }
                
                {rise? 
                    agent:
                        !right
                        !disappoint
                        Huh. So you <i>are</i> a traitor. 
                    player:
                        !crossed
                        Ah, Flekfis. I was wondering when you'd take up my scent.
                    agent:
                        I didn't wanna believe it. I thought, you know, you're not so bad...
                        !angry
                        But then all the loose ends just kept tying themselves into knots. 
                        Don't worry. Kalandra won't be quite as disappointed once she realizes I've had you taken care of.
                        !fight
                }
            ]],

            OPT_DEFEND_SELF = "Defend yourself",
            DIALOG_DEFEND_SELF = [[
                player:
                    !fight
            ]],

            NEGOTIATION_REASON = "Probe for weaknesses",

            DIALOG_WON_CARDS = [[
                player:
                    You've been weighed, measured, and cataloged. 
                    To be honest... it wasn't very interesting.
                agent:
                     !fight
                     I-I'm not scared of you! Shut up and finish this!
            ]],
            DIALOG_WON_NOTHING = [[
                player:
                    !interest
                    Is this how you die? What would your parents think?
                agent:
                    !dubious
                    What would my...? Is that your attempt to get under my skin?
                    Pathetic. 
                
            ]],
            DIALOG_LOSE_NEGOTIATION = [[
                player:
                    Can you still hear the lambs, {agent}?
                agent:
                    Can I still hear the <i>what</i>? 
                    I think you've wasted enough of my time. Let's get this over with.
                    !fight
            ]],

            OPT_ATTACK = "Attack!",
            DIALOG_ATTACK = [[
                player:
                    !cruel
                    I'd be lying if I said I hadn't been looking forward to this, actually.
                    You only made it easier by isolating yourself away from the herd, as it were.
                    !fight
            ]],

            OPT_START_NEGOTIATION = "Probe for weaknesses",
            DIALOG_START_NEGOTIATION = [[
                player:
                    Are you sure this is how you want to proceed?
                    I'm a spy, after all. I'm trained to look for weaknesses.
            ]]


        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt:TalkTo("target")
            cxt:Dialog("DIALOG_CONFRONTATION")
            cxt.quest.param.cards_won = {}
            

            cxt:Opt("OPT_START_NEGOTIATION")
                :Dialog("DIALOG_START_NEGOTIATION")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.NO_CORE_RESOLVE,

                    reason_fn = function(minigame)
                        return loc.format(cxt:GetLocString("NEGOTIATION_REASON") )
                    end,
                    on_start_negotiation = function(minigame)
                        local core = minigame.opponent_negotiator:FindCoreArgument()
                        if core and core.SetCards then
                            core:SetCards( cxt.enc:GetPrimaryCast().def.weakness_cards, cxt.quest.param.cards_won )
                        end
                        minigame:DealCard( Negotiation.Card("start_battle", minigame:GetPlayerNegotiator().agent), minigame:GetDrawDeck())
                    end,
                    
                    on_success = function(cxt, minigame) 
                        
                        cxt:Dialog(cxt.quest.param.cards_won and "DIALOG_WON_CARDS" or "DIALOG_WON_NOTHING")
                        DoWeaknessFight(cxt)
                    end,

                    on_fail = function() 
                        cxt.quest.param.neg_lost = true
                        cxt:Dialog("DIALOG_LOSE_NEGOTIATION")
                        DoWeaknessFight(cxt)
                    end,
                }            

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE,
                    advantage = TEAM.BLUE,
                    }
                    :OnWin()
                        :GoTo("STATE_POST_FIGHT")
        end)


    :State("STATE_POST_FIGHT")
        :Loc{
            DIALOG_WON_FIGHT = [[
                {dead?
                    * You feel a pang as you hide the body. This is one death that won't go unnoticed, no matter how easily it might sink beneath the muck.
                }

                {not dead?
                    {spark_barons?
                        agent:
                            !injured
                            Go on. Finish what you started.
                            I would do it to you.
                    }
                    {rise?
                        agent:
                            !injured
                            Go on. Finish me off. 
                            This is exactly the kind of thing I'd die for.
                    }
                }
            ]],

            OPT_FINISH_OFF = "Finish {agent} off",
            DIALOG_FINISH_OFF = [[
                player:
                    I'm afraid I will simply have to. I've come too far to let you get in my way.
                    For what it's worth, I truly wish you could've just minded your own business.
                agent:
                    !exit
                * You make it quick, but you still feel a pang as you hide the body. This is one death that won't go unnoticed, no matter how easily it might sink beneath the muck.
            ]],

            OPT_LET_GO = "Let {agent} go",
            DIALOG_LET_GO = [[
                player:
                    No. I've done enough.
                    Frankly it doesn't matter if you tell your superior. My work here is almost done. 
                    If you've any sense, though, I won't see you again—I don't give a fig about your petty labor squabbles. 
                    So keep to yourself from here on out.
                agent:
                    You're a damn fool if you think I'll do anything of the sort.
                    !throatcut
                    See you later, {player}.
                    !exit
                * {agent} disappears into the trees before you have a chance to regret your decision. 
            ]],
        }
        :Fn(function(cxt) 

            cxt:Dialog("DIALOG_WON_FIGHT")
            
            if cxt:GetAgent():IsDead() then
                cxt.quest:Complete()
                StateGraphUtil.AddLeaveLocation(cxt)
            else
                cxt:Opt("OPT_FINISH_OFF")
                    :Dialog("DIALOG_FINISH_OFF")
                    :Fn(function() 
                        cxt.quest:Complete()
                        local death_item = cxt:GetAgent():GetDeathItem()
                        cxt:GainCards{death_item}
                        cxt:GetAgent():Kill()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
                cxt:Opt("OPT_LET_GO")
                    :Dialog("DIALOG_LET_GO")
                    :Fn(function() 
                        cxt:GetAgent():OpinionEvent(OPINION.DISAPPROVE_MAJOR)
                        cxt.quest:Complete()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end)
            end

        end)

QDEF:AddDebugOption("working_with", {"RISE", "SPARK_BARONS"})


           