
local CHUM_BONUSES = StateGraphUtil.ValidateChumBonuses(
{
    {
        fn = StateGraphUtil.OfferSpareChange( 100 ),
    },
    {
        fn = StateGraphUtil.OfferUpgradeNegotiation(),
    },
    {
        fn = StateGraphUtil.OfferUpgradeBattle()
    },
    {
        fn = StateGraphUtil.OfferCard( "amnesiator" )
    },
    {
        fn = StateGraphUtil.OfferCompressionGear()
    },
    {
        fn = StateGraphUtil.OfferTriageKit()
    },
    {
        fn = StateGraphUtil.OfferBattleDraft( 2 )
    },
    {
        fn = StateGraphUtil.OfferNegotiationDraft( 2 )
    },
    {
        fn = StateGraphUtil.OfferBoomBox( 2 )
    },
    {
        fn = StateGraphUtil.OfferGraftSlot( GRAFT_TYPE.COMBAT, 1 )
    },
    {
        fn = StateGraphUtil.OfferGraftSlot( GRAFT_TYPE.NEGOTIATION, 1 )
    },
})


local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    on_init = function(quest)
        TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("GB_GATE"))
    end,
}

:AddCastByAlias{
    cast_id = "kalandra",
    alias = "KALANDRA",
    no_validation = true
}

:AddCastByAlias{
    cast_id = "spark_contact",
    alias = "SPARK_CONTACT",
    no_validation = true
}

:AddCast{
    cast_id = "gate_guard",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_GATE"):GetProprietor())
    end,
}

:AddObjective{
    id = "get_through_gate",
    title = "Enter Grout Bog",
    desc = "Grout Bog is a protected enclave. You're going to have to get past the gate to get to your job.",
    state = QSTATUS.ACTIVE,
}


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


QDEF:AddConvo("get_through_gate")
    :Confront(function(cxt)
        if TheGame:GetLocalSettings().FAST_STARTUP then
            cxt.quest:Complete()
        else
            return "STATE_START"
        end
    end)

    :State("STATE_START")
        :Loc{
            DIALOG_INTRO = [[
            * The land around Grout Bog squelches with mud hungry for legs as well as boots. 
            * The only road leads to a gate, flanked by armed security. A Spark Baron guard steps out into the path as you approach.
                player:
                    !left
                gate_guard:
                    !right
                    $neutralDirect
                    Stop right there. The bog is closed. 
                    !point
                    $angryDefault
                    Especially to grifters like you.
                    !cruel
                    $angryCruel
                    Unless you pay the, uh, <i>processing</i> fee.
                    Then maybe I could see to considering your petition for entry.
                *** {gate_guard} is demanding a bribe to get through the gate.
            ]],
            DIALOG_REINTRO = [[
                gate_guard:
                    $angryDefault
                    Enough chatter. You either got a reason to be here, or it's time to leave.
            ]],
            

            OPT_PAY_BRIBE = "Pay the bribe",
            DIALOG_PAY_BRIBE = [[
                player:
                    $neutralResigned
                    Fine. I suppose your lavish lifestyle needs supplementing. 
                    !give
                agent:
                    !take
                * You press your hard-earned shills into {gate_guard}'s hands.
            ]],
            OPT_NEGOTIATE = "Convince {agent} to let you in without paying",
            DIALOG_NEGOTIATE = [[
                player:
                    Forgive me for being skeptical of this <i>fee</i>. 
                    !crossed
                    $angryDefault
                    In fact, I'd like to speak to your supervisor. 
            ]],
            DIALOG_SUCCESS = [[
                player:
                    !permit
                    $angryDefault
                    Well? Do you want to get your superiors involved?
                agent:
                    !chuckle
                    Hah! They'd just double the fee. 
                    $neutralResigned
                    But fine, grifter, have it your way.
            ]],
            DIALOG_FAIL = [[
                player:
                    !crossed
                    $angryDefault
                    What's more, I don't appreciate your tone. 
                agent:
                    !hips
                    $angryCruel
                    Well, I don't appreciate the fact that you're a cheapskate.  
            ]],
            SIT_MOD = "{agent} isn't good at {agent.hisher} job",

        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("gate_guard"))
                cxt:Dialog("DIALOG_INTRO")
            else
                cxt:Dialog("DIALOG_REINTRO")
            end

            cxt:Opt("OPT_PAY_BRIBE")
                :Dialog("DIALOG_PAY_BRIBE")
                :DeliverMoney(20)
                :GoTo("STATE_PICK_SIDE")

            cxt:Opt("OPT_NEGOTIATE")
                :Dialog("DIALOG_NEGOTIATE")
                :Negotiation{
                    situation_modifiers =
                    {
                        { value = -6, text = cxt:GetLocString("SIT_MOD") }
                    },
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_SUCCESS")
                        cxt:GoTo("STATE_PICK_SIDE")
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_FAIL")
                    end,
                }
        end)

    :State("STATE_PICK_SIDE")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !point
                    But you better have some work papers, else the only thing I'm authorized to give you is a kick in the seat.
            ]],
            DIALOG_REINTRO = [[
                agent:
                    So are you going to give me your papers, or am I going to give you a very bad day?
            ]],
            OPT_GIVE_PAPERS = "Hand over your papers",
            
            DIALOG_GIVE_PAPERS = [[
                player:
                    !happy
                    $happyCocky
                    No need for hostilities, friend. I'm here by invitation.
                    !give 
                    See for yourself, if you like.
            * You pull the signed work visa from an oilskin wallet you carry for such purposes. 
                gate_guard:
                    !dubious
                    $neutralDubious
                    Huh. Looks legit.
                    Who exactly are you meant to report to?
            ]],
            
            OPT_RISE = "You're here to dig (You will receive {1#card})",
            TT_RISE = "You'll report to {kalandra}, a foreman",
            DIALOG_RISE = [[
                player:
                    A foreman in the workers' camp. 
                    !thought 
                    $neutralThoughtful
                    Someone named... Prindo Kalandra, I believe.
                gate_guard:
                    $neutralResigned
                    {kalandra}? Lucky break. She won't work you to death, that's for sure. 
                    !cruel
                    I think she's too soft on the workers, if you ask me. 
                player:
                    !interest
                    I don't think I did.
                gate_guard:
                    !agree
                    $neutralWhatever
                    Rude of you. 
            ]],

            OPT_BARONS = "You're here to work security (You will receive {1#card})",
            TT_BARONS = "You'll report to {spark_contact}, the Spark Baron",
            DIALOG_BARONS = [[
                player:
                    !crossed
                    Do I look like a common laborer to you?
                    $neutralDirect
                    I'm reporting to Lellyn Fellemo directly.
                gate_guard:
                    !interest
                    $neutralDubious
                    {spark_contact}?! No way someone like you—
                * {agent.HisHer} eyes glance down at your work visa again—and this time, {agent.heshe} reads the fine print.
                    !scared_shrug
                    $scaredDefault
                    U-uh, I mean, of course you are. Sir!
                    !neutral
                    $scaredDefault
                    We better hurry this along. {spark_contact} doesn't like to be kept waiting.
            ]],

            DIALOG_ASK_ABOUT = [[
                player:
                    !placate
                    $miscPersuasive
                    I've been on the road a long time. Let me rest these old bones a minute while I get my bearings, hm?
                gate_guard:
                    !crossed
                    $neutralDirect
                    You got five minutes. We have a strict no loiterin' policy.  
            ]],
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("gate_guard"))
                cxt:Dialog("DIALOG_INTRO")
            else
                cxt:Dialog("DIALOG_REINTRO")
            end

            cxt:Opt("OPT_ASK_ABOUT")
                :Dialog("DIALOG_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")

            cxt:Opt("OPT_GIVE_PAPERS")                
                :Dialog("DIALOG_GIVE_PAPERS")
                :Fn(function(cxt)
                    cxt:Opt("OPT_BARONS", "blacklist")
                        :Dialog("DIALOG_BARONS")
                        :PostText("TT_BARONS")
                        :GainCards{"blacklist"}
                        :Fn(function() TheGame:GetGameState():GetMainQuest().param.faction_pick = "SPARK_BARONS" end)
                        :GoTo("STATE_SEARCH")
                    
                    cxt:Opt("OPT_RISE", "shovel")
                        :Dialog("DIALOG_RISE")
                        :PostText("TT_RISE")
                        :GainCards{"shovel"}
                        :Fn(function() TheGame:GetGameState():GetMainQuest().param.faction_pick = "RISE" end)
                        :GoTo("STATE_SEARCH")
                end)

        end)

    :State("STATE_SEARCH")
        :Loc{
            DIALOG_SEARCH = [[
                gate_guard:
                    I still gotta search you, though. You got anything to declare?
                * You rummage through your backpack, showing {agent} the contents. You've only got one thing worth declaring, though.                     
            ]],

        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_SEARCH")
            cxt:RunLoopingFn( StateGraphUtil.OfferChumBonuses, CHUM_BONUSES)
            cxt:GoTo("STATE_LEAVE")
        end)
    :State("STATE_LEAVE")
        :Loc{
            DIALOG_LEAVE = [[
                player:
                    So am I free to go?
                gate_guard:
                    !agree
                    $neutralResigned
                    Yeah, whatever.
                player:
                    !salute
                    $neutralGreeting
                    Thank you, officer. It's been a pleasure.
                gate_guard:
                    !dubious
                player:
                    !exit
                gate_guard:
                    !point
                    $angryDefault
                    But don't try anything funny in there!
                * You go through the gate, but you feel like {agent} will miss you when you're gone.
            ]],
            OPT_ASK_ABOUT_BARONS = "Ask more about {spark_contact}",            
            DIALOG_ASK_ABOUT_BARONS = [[
                player:
                    !thought
                    $neutralThoughtful
                    I don't suppose you know what kind of work {spark_contact} might have for me?
                agent:
                    !shrug
                    $neutralTakenAback
                    How the Hesh should I know?
                    Things are getting rough with the workers, though. They want more rights, better pay—
                    Maybe {spark_contact} wants you to work security and shut down their union before it starts.
                player:
                    !agree
                    $neutralThoughtful
                    I see. That makes sense.
                    And what if I would prefer to find other work?
                agent:
                    !placate
                    $miscEmbarrassed
                    If {spark_contact} is expecting you, I wouldn't wanna disappoint him.
                    But there's always more work to be had in the bog. 
                    You could check in with a foreman named {kalandra}—she's not too strict, and might take pity on an old fogey like you.
                player:
                    !angryReluctant
                    Charming. 
            ]],
            OPT_ASK_ABOUT_RISE = "Ask more about {kalandra}",            
            DIALOG_ASK_ABOUT_RISE = [[
                player:
                    !thought
                    $neutralThoughtful
                    You say {kalandra} is easy to work for? Should I be pleased?
                agent:
                    !cruel
                    $miscMischievous
                    Depends on how honest you are.
                player:
                    !permit
                    And what does that foreboding little statement mean?
                agent:
                    !shrug
                    $angryReluctant
                    Look, I don't know nothing about nothing. But... 
                    I've heard rumors {kalandra} might actually be one of <i>them</i>. The Rise.
                    $neutralResigned
                    Saying so to the wrong person could get her killed, though, and I don't know for sure.
                player:
                    !dubious 
                    $neutralDubious
                    What does this have to do with me?
                agent:
                    $neutralResigned
                    Like I said, depends. Maybe nothing. 
                    Maybe she's just soft and you get a nice, cushy bog job. For what that's worth.
                    !throatcut
                    $neutralDirect
                    Or maybe you fall in with the wrong crowd and it's <i>your</i> neck on the block when {spark_contact} finally puts a stop to it.
                player:
                    $neutralResigned
                    !interest
                    And what if that fate sounds... unappetizing?
                agent:
                    $neutralThoughtful
                    You could always talk to the Spark Barons. If there's a job that needs doin', they'd know.
            ]],

            OPT_ASK_TO_LEAVE = "Take your leave",

        }
        :SetLooping()
        :Fn(function(cxt)

            if TheGame:GetGameState():GetMainQuest().param.faction_pick == "SPARK_BARONS" then
                cxt:Question("OPT_ASK_ABOUT_BARONS", "DIALOG_ASK_ABOUT_BARONS")
            else
                cxt:Question("OPT_ASK_ABOUT_RISE", "DIALOG_ASK_ABOUT_RISE")
            end

            cxt:Opt("OPT_ASK_TO_LEAVE")
                :PreIcon( global_images.close)
                :Dialog("DIALOG_LEAVE")
                :CompleteQuest()
                :Travel()

        end)


     :AskAboutHub("STATE_QUESTIONS", 
        {
            "Ask about Grout Bog",
            [[
                player: 
                    !interest
                    As I recall, Grout Bog was once a prospector's dream. This seems very bureaucratic.
                agent:  
                    !dubious
                    $neutralDubious
                    You kiddin'? It's been under Spark Baron regulation for a good twenty years at least.
                    A little piece of paradise away from the Admiralty and the Cult.
                player:
                    !crossed
                    $neutralDubious
                    Paradise for whom?
                agent:
                    $miscMischievous
                    Well, the Barons, of course. Not so much the contracted labor.
                    !overthere
                    <i>They</i> have to dig up the tech, and the bog isn't a forgiving place. 
                    $neutralResigned
                    Between the pitfalls and the grawkits, most don't make it to the end of their terms.
                player:
                    !agree
                    Ah. That sounds unpleasant. 
                agent:
                    !point
                    $miscMocking
                    No kidding. The road outta here is right behind you, grifter. 
            ]],
            "Ask about the gate",
            [[
                player: 
                    !interest
                    So the gate is to keep unlicensed rogues from plundering the dig sites?
                agent:  
                    !shrug 
                    $neutralDubious
                    Sure. Not that most would have the guts to try.
                    The digs are for <i>old</i> tech, from the Vagrant Age. Half of it explodes the second it oxidizes.
                    !happy
                    $happyCocky
                    Only Barons got the know-how to dig it up safely.
                player:
                    $miscAlluring
                    And weaponize it.
                agent:
                    $neutralWhatever
                    Gotta profit somehow. 
                    But the gates are mostly here to keep laborers <i>in</i>.
                player:
                    $neutralResigned
                    I see. That's slightly worrying. 
            ]],
            "Ask about the Spark Barons",
            [[
                player:
                    !crossed
                    I assume the Spark Barons are the law of the land, then?
                agent:
                    !agree
                    They're the one paying everyone's bills, aren't they?
                    $miscMischievous
                    Grout Bog's operations manager is a former Admiralty officer, even, so he knows a thing or two about authority.
                player:
                    !interest
                    That would be Lellyn Fellemo, yes? The kra'deshi?
                agent:
                    !dubious
                    $neutralDubious
                    Huh. Yeah, that's right. You heard of him?
                player:
                    !overthere
                    $miscNostalgic
                    He and I are old friends, actually. Though we were each twenty years younger when our friendship took root. 
                    $neutralResigned
                    Time will tell how well it's weathered. 
                agent:
                    $neutralDubious
                    Huh. If you say so, grifter.
            ]],
            "Ask about the laborers",
            [[
                player:
                    And what about the laborers, then? 
                    !dubious 
                    $neutralDubious
                    I've heard whispers of some worker rebellions rising up in the Bread Fields.
                agent:
                    $neutralResigned
                    Yeah, it's a rot, that is. A real sickness.
                    And it's spread to Grout Bog. Half the workers are a sneeze away from open revolt. 
                    !angry_accuse
                    $angryReluctant
                    So we don't need folks like you coming in and stirring up trouble.
                player:
                    Then who should I avoid?
                agent:
                    That's the problem—they keep their cards close. Wily-like. 
                    !shrug
                    $miscHopeless
                    The leader could even be a foreman, for all we know.]]
        })        


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
