local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local BATTLE_EVENT = battle_defs.BATTLE_EVENT

local function spawn_next_quest( quest )
    local quest_num = quest.param.quests_completed + 1
    local id = quest.param.quest_list[quest_num]
    
    if id then
        local spawned_quest, err = QuestUtil.SpawnQuest(id, { parameters = { parent_quest = quest, working_with = quest.param.working_with } })
        if spawned_quest then
            quest:SetParameter("current_quest", spawned_quest)
            return spawned_quest
        end
    end
end


local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    title = "The Expedition",
    on_init = function(quest)
        TheGame:GetGameState():SetDifficulty(4)
        quest.param.working_with = quest.param.working_with or "RISE"
    end,
    icon = engine.asset.Texture("icons/quests/rook_story_expedition.tex"),
    restrict_exit = function(quest, location)
        if quest:IsActive("expedition") then
            return true
        end
    end,
}

:AddCast{
    cast_id = "faction_second",
    no_validation = true,
    cast_fn = function(quest, t) 
        table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "RISE_SECOND" or "SPARK_SECOND"))
    end
}

:AddCast{
    cast_id = "handler",
    no_validation = true,
    cast_fn = function(quest, t) 
        table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "KALANDRA" or "SPARK_CONTACT"))
    end
}

:AddObjective{
    state = QSTATUS.ACTIVE,
    id = "start",
    title = "Follow {handler} into the bog",
    desc = "{handler} is waiting for you to accompany them on an expedition into the depths of the bog to find the Boggers' inner sanctum.",
    mark = {"handler"},
    on_complete = function(quest)
        quest:Activate("expedition")
        TheGame:GetGameState():GetPlayerAgent():GetLocation():ClearAgents()
    end,
}

:AddObjective{
    --state = QSTATUS.ACTIVE,
    id = "expedition",
    title = "Recover the device",
    desc = "Use all of your connections in order to get to the Bogger's inner sanctum and recover the mysterious device for your <i>real</> employers.",
    on_activate = function(quest) 
        quest.param.quest_list = {"ROOK_STORY_EXPEDITION_STEP_1_WORKERS", "ROOK_STORY_EXPEDITION_MONSTERS", "ROOK_STORY_EXPEDITION_PICNIC", "ROOK_STORY_EXPEDITION_BOGGERS", "ROOK_STORY_EXPEDITION_OTHER_HANDLER", "ROOK_STORY_EXPEDITION_BOSS_FIGHT"}
        quest.param.quests_completed = 0
        spawn_next_quest(quest)
    end,

    events = 
    {
        quests_changed = function(quest, event_quest)
            if event_quest == quest.param.current_quest then
                if event_quest:IsComplete() or event_quest:IsFailed() then
                    quest.param.quests_completed = quest.param.quests_completed + 1
                    spawn_next_quest(quest)
                end
            end
        end,
    },
}


--You meet your Lt. Helper and their backup at the trailhead.

QDEF:AddConvo("start", "handler")
    :ConfrontState("CONF")

        :Questions("RISE", {
            "Ask about {faction_second}",
            [[
                player:
                    Just you? Where's {faction_second}?
                agent:
                    Keeping the Barons distracted. Besides, this is dangerous, and I wouldn't ask {faction_second} to do something I wouldn't do myself.
                    !humouring
                    He asked me to send his regards, though. Apparently you impressed him yesterday. 
                    Flekfis can be a bit stubborn but there's never been anyone more loyal.
                player:
                    I'm honored to have earned his esteem.
            ]]

        })
        :Questions("SPARK_BARONS", {
            "Ask about {faction_second}",
            [[
                player:
                    Why, old fellow, I didn't expect to see you getting your boots dirty.
                    Where's {faction_second}? Isn't this precisely the sort of thing you keep her around for?
                agent:
                    !hips
                    I never trust too good of a thing, my friend. {faction_second} is holding the fort in case the Rise get any conniving ideas. 
                    She sends her regrets, though. I imagine she was looking forward to fighting at your side.
                player:
                    !thought
                    $neutralDubious
                    Was she? I got the impression she wasn't too pleased with the work you've had me doing.
                agent:
                    $miscMischievous
                    She does chafe a bit, doesn't she? But I've regaled her with enough stories of your heroics that I think she was eager to see for herself.
                    Some other time, perhaps.
            ]]

        })

        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                handler:
                    !right
                
                {first_time?

                    {rise?
                        * You're not shocked to see {handler} herself at the rendezvous point, flanked by a small cadre of Rise. 
                        * Something about these freedom-fighting types leads to acts of heroism among the leadership.
                        handler:
                            $miscRelieved
                            It's about time. I was beginning to worry Hebbel hadn't delivered the message.
                        player:
                            !agree
                            He did. He was rather displeased, in fact.
                        handler: 
                            Guess he's the superstitious sort. 
                            Well, thanks to your intel, we spent all night studying the known maps of this part of the bog.
                            !overthere
                            $neutralResigned
                            Most of it is unpassable, as likely to eat your leg off as let you pass.
                            $neutralThoughtful
                            But judging by the terrain, and what we know about the Boggers' movements...
                            This map is our best guess.
                            !give
                        player:
                            !take
                            I've charted my fair share of unmapped territory. This should be simple.
                    }
                    
                    {spark_barons?
                        * You're rather surprised to find {handler} himself has made an appearance, along with a number of Spark Baron guards.
                        handler:
                            $happyGreeting
                            Ah, you made it. I wondered if your leg had slowed you down.
                        player:
                            $neutralResigned
                            I'm afraid Hebbel was rather reticent about your message.
                        handler:
                            Was he? Hm. 
                            I hope you aren't, however. After we've spent so many resources making you a map.
                            !give
                        player:
                            !take
                            $neutralDubious
                            A map? I thought this part of the bog was uncharted.
                        handler:
                            !agree
                            Oh, it is. Treacherous terrain. 
                            But we rigged an automech retinal scanner onto a gas balloon and were able to get some readings. 
                            It's not fool-proof, of course. Requires configuration. But between it and your insight, you should be able to find the way.
                    }
                }
                
                {not first_time? 

                {spark_barons?
                    handler:
                        Do hurry up, Rook. The daylight dwindles while you dilly-dally. 
                    }
                {rise?
                    handler:
                        You double-booked, Rook? We haven't got all day. }
                }
            ]],
            OPT_START = "Head out",
            DIALOG_START = [[
            {spark_barons?
                player:
                    Yes, yes, I'm ready. 
                    We soldier on, eh?
                agent:
                    !humouring
                    $happyJoke
                    You do the soldiering, old chap. That's below my paygrade.
                    The Barons I've brought can provide support, but we have limited resources.
            }
            {rise?
                player:
                    I'm ready, certainly. Can you and your people keep up?
                    $neutralDubious
                    I can't imagine there's much combat experience between the lot of you.
                agent:
                    !crossed
                    $neutralDirect
                    We can handle ourselves. But the movement has made me promise to stay out of danger.
                    If you get into trouble the Rise can help. But there's only so much we can do.
            }

            ]],
            DIALOG_START_2 = [[
                agent:
                    Remember: no reinforcements. We're on our own.
                    !exit
                player:
                    !exit
                * The party falls in line, protecting {handler} in the middle. You lead the way.
            ]],

            OPT_NOT_YET = "Take care of something else first",
            DIALOG_NOT_YET = [[
                player:
                    Give me a moment. I need to prepare.
                agent:
                    !angry
                    $miscMocking
                    Oh, certainly. The bog's not going anywhere.
                player:
                    !exit                    
            ]]

        }
        :Fn(function(cxt) 
            cxt:TalkTo("handler")
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 
                cxt:InsertQuestions(cxt:GetAgent():GetFactionID())
                cxt:Opt("OPT_NOT_YET")
                    :Dialog("DIALOG_NOT_YET")
                    :Travel()
                cxt:Opt("OPT_START")
                    :SetQuestMark()
                    :Dialog("DIALOG_START")
                    :GainCards{cxt:GetAgent():GetFactionID() == "RISE" and "rise_expedition" or "baron_expedition"}
                    :Dialog("DIALOG_START_2")
                    :CompleteQuest("start")
                    :DoneConvo()
            end)
        end)



QDEF:AddDebugOption("working_with", {"RISE", "SPARK_BARONS"})



------------------------------------------------------------------------------------------
--you go talk to your handler, and get your marching orders



Content.AddBattleCard( "rise_expedition", {
    name = "Rise Forces",
    anim = "taunt",
    desc = "Call on your Rise followers for help.",
    flavour = "'Geared up and good to go.'",
    series = CARD_SERIES.GENERAL,
    cost = 0,
    rarity = CARD_RARITY.UNIQUE,
    flags = CARD_FLAGS.TOOLBOX | CARD_FLAGS.SKILL,
    target_type = TARGET_TYPE.SELF,

    support_ids = {"call_in_rise_rebel", "call_in_rise_pamphleteer", "call_in_rise_automech", "call_in_rise_radical"},
    pool_cards = {"call_in_rise_rebel", "call_in_rise_pamphleteer", "call_in_rise_automech", "call_in_rise_radical"},

    OnInit = function( self )
        self.userdata.stored_cards = self.userdata.stored_cards or {}
        for i,id in ipairs(self.support_ids) do
            table.insert(self.userdata.stored_cards, Battle.Card(id, self.owner))
        end
    end,
} )

Content.AddBattleCard( "baron_expedition", {
    name = "Baron Forces",
    anim = "taunt",
    desc = "Call on your Baron followers for help.",
    flavour = "'Suit up and head out!'",
    
    series = CARD_SERIES.GENERAL,
    cost = 0,
    rarity = CARD_RARITY.UNIQUE,
    flags = CARD_FLAGS.TOOLBOX | CARD_FLAGS.SKILL,
    target_type = TARGET_TYPE.SELF,

    support_ids = {"call_in_baron_goon", "call_in_baron_taskmaster", "call_in_baron_professional", "call_in_baron_automech"},
    pool_cards = {"call_in_baron_goon", "call_in_baron_taskmaster", "call_in_baron_professional", "call_in_baron_automech"},

    OnInit = function( self )
        self.userdata.stored_cards = self.userdata.stored_cards or {}
        for i,id in ipairs(self.support_ids) do
            table.insert(self.userdata.stored_cards, Battle.Card(id, self.owner))
        end
    end,
} )

local SUPPORT_CARDS = 
{
    -- Rise
    call_in_rise_rebel = 
    {
        name = "Call in Rise Rebel",
        anim = "taunt",
        desc = "Call in a Rise Rebel to make an impairing attack against your target.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        max_charges = 3,
        hide_in_cardex = true,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("RISE_REBEL"), 4 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST+12)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("rebel_slash_call_in", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    call_in_rise_pamphleteer = 
    {
        name = "Call in Rise Pamphleteer",
        anim = "taunt",
        desc = "Call in a Rise Pamphleteer to make an attack against your target that applies {DEFECT}.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        max_charges = 2,
        hide_in_cardex = true,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("RISE_PAMPHLETEER"), 4 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST+12)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("can_opener_call_in", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    call_in_rise_automech = 
    {
        name = "Call in Rise Automech",
        anim = "taunt",
        desc = "Call in a Rise Automech to make a powerful attack against your target.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        max_charges = 2,
        hide_in_cardex = true,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("RISE_AUTOMECH"), 4 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST+12)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("mech_cross_call_in", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    call_in_rise_radical =
    {
        name = "Call in Rise Radical",
        anim = "taunt",
        desc = "Call in a Rise Radical to make an attack against all enemies.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        target_mod = TARGET_MOD.TEAM,
        max_charges = 2,
        hide_in_cardex = true,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("RISE_RADICAL"), 4 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("fire_in_the_hole_call_in", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    call_in_kalandra =
    {
        name = "Call in Kalandra",
        anim = "taunt",
        desc = "Call in Kalandra to attack your enemies.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        max_charges = 2,
        hide_in_cardex = true,
        target_mod = TARGET_MOD.TEAM,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("KALANDRA"), 1 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("kalandra_tag_in_attack", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    -- Baron
    call_in_baron_goon = 
    {
        name = "Call in Spark Baron Goon",
        anim = "taunt",
        desc = "Call in a Spark Baron Goon to make a impairing attack against your target.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        max_charges = 3,
        hide_in_cardex = true,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("SPARK_BARON_GOON"), 4 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST+12)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("ai_spark_baron_billy_club_call_in", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    call_in_baron_taskmaster = 
    {
        name = "Call in Spark Baron Taskmaster",
        anim = "taunt",
        desc = "Call in a Spark Baron Taskmaster to make a multi-hit attack against your target.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        max_charges = 3,
        hide_in_cardex = true,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("SPARK_BARON_TASKMASTER"), 4 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("master_blaster_call_in", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    call_in_baron_professional = 
    {
        name = "Call in Spark Baron Professional",
        anim = "taunt",
        desc = "Call in a Spark Baron Professional to make an attack against all enemies.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        target_mod = TARGET_MOD.TEAM,
        max_charges = 2,
        hide_in_cardex = true,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("SPARK_BARON_PROFESSIONAL"), 4 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("professional_blast_call_in", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    call_in_baron_automech = 
    {
        name = "Call in Spark Baron Automech",
        anim = "taunt",
        desc = "Call in a Spark Baron Automech to make a wounding attack against your target.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        max_charges = 2,
        hide_in_cardex = true,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("SPARK_BARON_AUTOMECH"), 4 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("enforcer_shot_call_in", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },

    call_in_fellemo =
    {
        name = "Call in Fellemo",
        anim = "taunt",
        desc = "Call in Fellemo to attack your target.",

        cost = 0,
        rarity = CARD_RARITY.UNIQUE,
        flags = CARD_FLAGS.SKILL,
        max_charges = 2,
        hide_in_cardex = true,

        OnPostResolve = function( self, battle, attack )
            local fighter = Fighter.CreateFromAgent( Agent("SPARK_CONTACT"), 1 )
            fighter.team = self.owner:GetTeam()
            fighter.call_in = true
            fighter.home_x = (FIGHT_MAX_DIST)*(self.owner:GetTeam() == TEAM.RED and 1 or -1)
            fighter:OnActivateFighter( battle )
            local card = Battle.Card("fellemo_tag_in_attack", fighter)
            battle:PlayCard(card, attack.target)
        end,
    },
}

for i, id, data in sorted_pairs( SUPPORT_CARDS ) do
    data.series = CARD_SERIES.GENERAL
    Content.AddBattleCard( id, data )
end



