local QDEF = QuestDef.Define{
        title = "Bogger Encounter",
        qtype = QTYPE.STORY,
        on_start = function(quest)
            if quest.param.working_with == "RISE" then
                quest.param.working_with_rise = true
            else
                quest.param.working_with_spark_barons = true
            end
        end,
        on_complete = function(quest) 
            TheGame:GetGameState():GetPlayerAgent():GetLocation():ClearAgents()
        end
    }

    :AddCast{
        cast_id = "handler",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "KALANDRA" or "SPARK_CONTACT"))
        end
    }

    :AddObjective{
        hide_in_overlay = true,
        id = "do_event",
        state = QSTATUS.ACTIVE,
        terminal = true,
    }
    
    :AddLocationDefs{
        MY_LOCATION = {
            name = "Road",
            plax = "EXT_Bog_Forest_01",
        },
    }


QDEF:AddConvo("do_event")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                * You're deep in the bog now. A fly buzzes next to your ear, loud enough to almost crack your skull.
                * The waters are thick and murky around your legs. There's no use denying it—you'll have to stop to clear the debris from out of your false joint. 
                player:
                    !left
                    $neutralDirect
                    Hold. Just a moment.
                handler:
                    !right
                    What is it? 
                player:
                    !permit
                    $neutralResigned
                    One of the charming idiosyncracies of these artificial legs.
                    They need to be maintained.
                {working_with_rise?
                handler:
                    ...  
                player:
                    !dubious
                    I suppose you're contemplating leaving me behind.
                handler:
                    We need to move quickly. 
                player:
                    $miscMocking
                    I thought you were all about teamwork. No one left behind.
                handler:
                    That takes energy and resources, and I'm a finite source of both.
                    !placate
                    $neutralResigned
                    Believe me, I feel for you—lost your leg in service only to have them cast you aside.
                    But you're still better off than most of the workers here. 
                    You can leave. You can make choices. And you can take care of yourself.
                    My efforts are better placed elsewhere.
                }
                {working_with_spark_barons?
                handler:
                    ...
                    !permit
                * Suddenly, {handler} crouches by your side, pulling a kit from {handler.hisher} satchel.
                    I developed these cloths for our weapons, but I imagine it'll do just as well on that bum leg of yours.
                * {handler.HeShe} begins to clean the joint of your artificial knee with military efficiency.
                player:
                    !dubious
                    $neutralDubious
                    I can handle that.
                handler:
                    !humouring
                    $happyCocky
                    You forget, I always had commendations for field maintenance.
                    We need to be quick, and I trust myself to do the job properly.
                    ...
                    !tired
                    There. Should do the trick now, old boy. Give it a bend.
                * Sure enough, your leg works better than it has in a while.
                player:
                    !humouring
                    $neutralThanks
                    Thank you.
                }
                * You both fall silent as your ears prick. Something about the bog has shifted. Even the buzzing flies have gone.
                bogger:
                    !right
                    The Bog hungers, outsiders. 
                    You should sink beneath its waters so it can feed.
            ]],

            OPT_DEFEND = "Defend {handler}",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Fall back, {handler}! 
                    I'll handle this. 
            ]],
            DIALOG_DEFEND_SUCCESS = [[
                
                {dead?  
                    * The battle is won. And with the fresh bodies, the flies return. Whatever tension clogged the air seems to have dissipated.
                }
                
                {not dead?
                    player:
                        $neutralDirect
                        Yield, and I'll show mercy. 
                    bogger:
                        Mercy has no place out here. 
                    * Too quick for comfort, the Bogger flees into the dense tree line.
                        !exit
                }
                
                {working_with_rise?
                    handler:
                        !right
                        We must be getting close.
                        You all right?
                    player:
                        !shrug
                        None worse for wear. 
                    handler:
                        Then let's press on. This'll be over soon.
                }
                
                {working_with_spark_barons?
                    handler:
                        !right
                        Well, we certainly know we're on the right track.
                        $happyDefault
                        !nudge_nudge
                        How did the leg perform? Better than expected, eh?
                        No excuse for slacking now, I'll say.
                    player:
                        We can likely expect more.
                    handler: 
                        !agree
                        Then I'm glad you took the time to get yourself in order.
                }
            ]],

            OPT_INTIMIDATE = "Scare off the Boggers",
            DIALOG_INTIMIDATE = [[
                player:
                    !angry_threaten
                    Burn the bog! Best you slink back where you came from.
            ]],
            
            DIALOG_INTIMIDATE_FAILURE = [[
                player:
                    !angry
                    $angryPatienceLost
                    We don't fear your beloved Deep Bog. It's the Bog that should fear <i>us</i>.
                bogger:
                    !suspicious
                    ...
                * The Boggers stare unsettlingly at you, and you notice, too late, that you've been surrounded.
            ]],
            DIALOG_INTIMIDATE_SUCCESS = [[
                player:
                    The bog has been gutted for decades. You're fools if you think we have anything to fear.
                bogger:
                    !suspicious
                    ...
                    !exit
                * The Boggers watch you shrewdly before vanishing, one by one, back into the tree line.
                handler:
                    !right
                    !interest
                    $miscHushedUrgent
                    We must be getting close.
                player:
                    Indeed. Let's press on.
            ]],

            
        }
        :Fn(function(cxt) 
            cxt.encounter:DoLocationTransition( cxt.quest:SpawnTempLocation("MY_LOCATION") )
            
            local boggers = CreateCombatParty({"BOGGER_CULTIVATOR", "BOGGER_CLOBBER", "BOGGER_CLOBBER"}, 4, cxt.location)
            cxt:ReassignCastMember("bogger", boggers[1])
            cxt:Dialog("DIALOG_INTRO")

            
            local allies = cxt.quest.param.working_with == "RISE" and 
                {TheGame:GetGameState():AddSkinnedAgent( "RISE_REBEL"), TheGame:GetGameState():AddSkinnedAgent( "RISE_REBEL") } or
                {TheGame:GetGameState():AddSkinnedAgent( "SPARK_BARON_GOON"), TheGame:GetGameState():AddSkinnedAgent( "SPARK_BARON_GOON") }

            cxt:TalkTo("bogger")
            cxt:RunLoop(function() 

                cxt:Opt("OPT_INTIMIDATE")
                    :Dialog("DIALOG_INTIMIDATE")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        helpers = {allies[1]},
                    }
                        :OnSuccess()
                            :Dialog("DIALOG_INTIMIDATE_SUCCESS")
                            :CompleteQuest()
                            :DoneConvo()
                        
                        :OnFailure()
                            :Dialog("DIALOG_INTIMIDATE_FAILURE")

                
                local handler_card = cxt.quest.param.working_with == "RISE" and "call_in_kalandra" or "call_in_fellemo"


                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.ISOLATED,
                        no_oppo_limit = true,
                        noncombatants = {cxt.quest:GetCastMember("handler")},
                        allies = allies,
                        
                        on_start_battle = function(battle) 
                            if handler_card then
                                local card = Battle.Card( handler_card, battle.player_team:Primary() )
                                card.show_dealt = true
                                battle:DealCard( card, battle:GetDrawDeck() )
                            end
                        
                        end
                    }
                        :OnWin()
                            :Fn(function() 
                                cxt:Dialog("DIALOG_DEFEND_SUCCESS")
                                cxt.quest:Complete()
                                StateGraphUtil.AddEndOption(cxt)
                            end)
            end)


        end)

QDEF:AddDebugOption("working_with", {"RISE", "SPARK_BARONS"})

QDEF:AddTest(TestCreator()
                :SetReps(5)
                :SetParams{working_with= "RISE"})

QDEF:AddTest(TestCreator()
                :SetReps(5)
                :SetParams{working_with= "SPARK_BARONS"})
