local QDEF = QuestDef.Define{
        title = "Recover the Artifact",
        qtype = QTYPE.STORY,
        rank = 5,
        on_init = function(quest)
            -- table that each quest will have passed in so we can share data between quests
            TheGame:GetGameState():SetDifficulty(5)
        end,

    }
    
    :AddCastByAlias{
        cast_id = "bog_boss",
        alias = "BOGGER_BOSS",
        no_validation = true,
    }

    :AddObjective{
        id = "fight_boss",
        hide_in_overlay = true,
        state = QSTATUS.ACTIVE,
        terminal = true,
    }

    :AddLocationDefs{
        MY_LOCATION = {
            name = "The Bogger Sanctum",
            plax = "INT_BOG_MINE",
            indoors = true,
        },
    }
    
    :AddCast{
        cast_id = "handler",
        no_validation = true,
        optional = true,
        cast_fn = function(quest, t) 
            if quest.param.working_with then
                table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "KALANDRA" or "SPARK_CONTACT"))
            end
        end
    }


local function AddAttackOption(cxt)

    local handler_card = nil
    
    if cxt.enc.scratch.handler_helping then
        handler_card = cxt.quest.param.working_with == "RISE" and "call_in_kalandra" or "call_in_fellemo"
    end

    local opt = cxt:Opt("OPT_ATTACK")
        :Dialog("DIALOG_ATTACK")
        :Battle{
                flags = BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.NO_REWARDS | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.NO_BURRS,
                
                on_start_battle = function(battle) 
                    local fighter = battle:GetFighterForAgent(cxt.quest:GetCastMember("bog_boss"))
                    if fighter then
                        fighter.behaviour.suppress_conditions = {}
                        
                        if cxt.quest.param.boss_fight_modifiers_to_remove then
                            for _,condition_id in ipairs(cxt.quest.param.boss_fight_modifiers_to_remove) do
                                fighter.behaviour.suppress_conditions[condition_id] = true
                            end
                        end
                        
                        if fighter.behaviour.AddBossConditions then
                            fighter.behaviour:AddBossConditions(fighter)
                        end

                        if handler_card then
                            local card = Battle.Card( handler_card, battle.player_team:Primary() )
                            card.show_dealt = true
                            battle:DealCard( card, battle:GetDrawDeck() )
                        end
                        
                        if cxt.quest.param.boss_fight_cards_to_win then
                            for _,won_card in ipairs( cxt.quest.param.boss_fight_cards_to_win ) do
                                local card = Battle.Card( won_card, battle.player_team:Primary() )
                                card.show_dealt = true
                                battle:DealCard( card, battle:GetDrawDeck() )
                            end
                        end
                    end
                end

            }
            :OnWin()
                :Fn(function()
                    cxt.quest:Complete()
                    cxt:Dialog("DIALOG_BEAT_BOSS")

                    if cxt.quest:GetCastMember("handler") then
                        cxt:TalkTo("handler")
                        cxt:Dialog("DIALOG_TALK_TO_HANDLER")
                    end

                    cxt:RunLoop(function() 
                        --if cxt.quest:GetCastMember("handler") then
                          --  cxt:InsertQuestions(cxt.quest:GetCastMember("handler"):GetFactionID())
                        --end
                        cxt:Opt("OPT_LEAVE")
                            :Fn(function() 
                                local flags = {}

                                cxt:FadeOut()
                                local kalandra = TheGame:GetGameState():GetAgent("KALANDRA")
                                local fellemo = TheGame:GetGameState():GetAgent("SPARK_CONTACT")
                                local flags = {
                                    kalandra_alive = kalandra and not kalandra:IsDead(),
                                    fellemo_alive = fellemo and not fellemo:IsDead(),
                                }
                                    
                                    local slide_show
                                    if cxt.quest.param.working_with == "RISE" then
                                        slide_show = "rook_ending_rise"
                                    elseif cxt.quest.param.working_with == "SPARK_BARONS" then
                                        slide_show = "rook_ending_barons"
                                    else
                                        slide_show = "rook_ending_nobody"
                                    end

                                TheGame:GetGameState():AddScore("PROGRESS", 4)

                                cxt:AwardUnlock("MUTATORS_UNLOCKED", "ROOK", LOC"UI.CHARACTER_UNLOCKED.MUTATORS_TITLE", LOC"UI.CHARACTER_UNLOCKED.MUTATORS_BODY")
                                cxt:AwardUnlock("ROOK_BRAWL_PLAYABLE", "ROOK", LOC"UI.CHARACTER_UNLOCKED.ROOK_BRAWL_TITLE", LOC"UI.CHARACTER_UNLOCKED.ROOK_BRAWL_BODY")

                                cxt.enc:ShowSlides(slide_show, flags, function() TheGame:Win() end )
                            end)
                    end)
                        
                end)
    return opt
end


QDEF:AddConvo("fight_boss")
    :ConfrontState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                * Eventually the road leads you to a strange outcropping of rock, sticking out of the Bog like a snaggletooth. 
                * Just inside the entrance you find signs of civilization, such as it is: glowing relics and some discarded bones have been shaped into what might be an altar.
                {handler_helping?
                    player:
                        !left
                        We're here.
                    handler:
                        !right
                        !dubious
                        Are we? 
                        $neutralDubious
                        Then where's the cache?
                    bog_boss:
                        !right
                        You'll find what you seek in the heart of the Bog.
                    player:
                        !humouring
                        Oh, hello.
                        $neutralResigned 
                        I suppose you're going to offer resistance if we say we're here to claim an ancient treasure?
                    bog_boss:
                        !overthere
                        There's nothing to resist. You coming here was foretold by the Bog.
                        It's been salivating with the utmost anticipation.
                    player:
                        That sounds rather unpleasant.
                    bog_boss:
                        I assure you, there is nothing but bliss in the embrace of the Bog.
                        $angryCruel
                        Let it feast upon you and know deliverance.
                }
                {not handler_helping?
                    player:
                        !left
                    bog_boss:
                        !right
                        You have come.
                    player:
                        Oh, hello. 
                        $neutralResigned
                        I suppose you're going to offer a resistance if I say I'm here to claim an ancient treasure?
                    bog_boss:
                        !overthere
                        There's nothing to resist. You coming here was foretold by the Bog.
                        It's been salivating with the utmost anticipation.
                    player:
                        That sounds rather unpleasant.
                    bog_boss:
                        I assure you, there is nothing but bliss in the embrace of the Bog.
                        $angryCruel
                        Let it feast upon you and know deliverance.
                }
            ]],
            OPT_ATTACK_FAITH = "Attack {bog_boss}'s faith",
            DIALOG_ATTACK_FAITH = [[
                player:
                    $miscMocking
                    Ah, the Bog foretold this, did it?
            ]],
            DIALOG_ATTACK_FAITH_SUCCESS = [[
                player:
                    !cruel 
                    $miscMocking
                    Did it also foretell my desecration? How I sullied it as I came through?
                    How I spilled blood that wasn't mine to spill?
                    Is that what your Bog wanted?
                agent:
                    !scared
                    $angryTakenAback
                    You—you know nothing of what the Bog wants!
                player:
                    And I dare say, neither do you.
                agent:
                    !angry 
                    $angrySeething
                    Enough!
            ]],
            DIALOG_ATTACK_FAITH_FAILURE = [[
                player:
                    !crossed
                    It seems to me you're just putting words in the Bog's mouth.
                agent:
                    !disappoint 
                    $neutralResigned
                    The Bog speaks in a language only we can hear.
                    Your heretic's tongue wouldn't begin to grapple with it.
            ]],

            OPT_ATTACK = "Attack!",
            DIALOG_ATTACK = [[
                player:
                    !angry
                    Then I suppose this ends only one way. 

                {handler_helping?
                    handler:
                        !left
                        Careful, Rook. Who knows what these Boggers are capable of.
                    player:
                        !left
                    bog_boss:
                        !right
                        The Bog consumes. We merely provide.
                        !fight
                }
            ]],


            DIALOG_BEAT_BOSS = [[
                * The Bog—or whatever it was—sinks slowly into the earth. 
            ]],
            DIALOG_TALK_TO_HANDLER = [[
                handler:
                    !right
                    It's done.
                player:
                    So it is. 
                * {handler} gets to work examining the area. There is work to be done, and you only started it.
                * {handler.HeShe} doesn't notice the strange device poking out of the mud.
            ]],

            OPT_LEAVE = "Take the device",
            DIALOG_LEAVE = [[
                * You take the device.
                player:
                    !take
                    !exit
            ]]

        }
        :Questions("RISE", {
            "Ask about {handler}",
            [[
                player:
                    !interest
                    $neutralDirect
                    This is goodbye, you know. 
                    The job is done, and I don't make a habit of lingering in one place too long.
                handler:
                    I figured as much. I don't suppose there's any convincing you to stay?
                    !overthere
                    We could use someone like you here. 
                    $miscPersuasive
                    When was the last time you settled in a place and fought for something beyond your next pay?
                player:
                    !crossed
                    $neutralDirect
                    A long time ago. It didn't go as planned. 
                    With any luck, the same thing won't happen to you.
                handler:
                    !happy
                    $happyThanks
                    Less luck needed now, at any rate.
                    Goodbye, Rook. Don't let the mud bog you down.

            ]],

        })
        :Questions("SPARK_BARONS", {
            "Ask about {handler}",
            [[
                player:
                    !humouring
                    Well, old bean. It's certainly been a grand reunion, hasn't it?
                handler:
                    !happy 
                    $happyTakenAback
                    Time enough for more of those, eh? Don't tell me you're thinking of leaving already?
                player:
                    $neutralDirect
                    Ah—
                handler:
                    !overthere
                    Because if it's the insubordination, Captain, I understand.
                    $miscPersuasive 
                    A man of your caliber doesn't deserve these sly remarks.
                    Stay, and I'll see you instated as an equal.
                    $miscPersuasive 
                    The Barons can provide ample protection from whatever ghosts dog your steps. 
                player:
                    !placate
                    $neutralDirect
                    No, thank you. 
                    There are some things that can only be outrun. 
                handler:
                    !neutral
                    ...
                    Seems like a coward's excuse, if you ask me.
                    $neutralResigned
                    But I'll no doubt find you if I have need of you again.
                    So long, chum. Mind the potholes on your way out. 
            ]],

        })

        :Fn(function(cxt) 
            cxt.encounter:DoLocationTransition( cxt.quest:SpawnTempLocation("MY_LOCATION") )
            
            cxt.enc.scratch.handler_helping = cxt.quest:GetCastMember("handler") ~= nil
            
            local core_arg = nil

            cxt:Dialog("DIALOG_INTRO")
            cxt:TalkTo("bog_boss")
            cxt:RunLoop(function() 
                cxt:Opt("OPT_ATTACK_FAITH")
                    :Dialog("DIALOG_ATTACK_FAITH")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.NO_IMPATIENCE | NEGOTIATION_FLAGS.NO_LOOT,
                        enemy_resolve_required = 30,
                        on_start_negotiation = function(minigame)
                            core_arg = minigame.opponent_negotiator:FindCoreArgument()
                        end,
                        
                        on_success = function(cxt, minigame) 
                            cxt.quest.param.boss_fight_cards_to_win = core_arg.cards_to_win
                            cxt.quest.param.boss_fight_modifiers_to_remove = core_arg.modifiers_to_remove
                            cxt:Dialog("DIALOG_ATTACK_FAITH_SUCCESS")
                            AddAttackOption(cxt)
                        end,
                    }
                        
                    :OnFailure()
                        :Dialog("DIALOG_ATTACK_FAITH_FAILURE")
                
                AddAttackOption(cxt)

            end)
        end)



QDEF:AddDebugOption("working_with", {"RISE", "SPARK_BARONS"})
