local QDEF = QuestDef.Define{
        title = "Monster Encounter",
        qtype = QTYPE.STORY,
        on_start = function(quest)
            if quest.param.working_with == "RISE" then
                quest.param.working_with_rise = true
            else
                quest.param.working_with_spark_barons = true
            end
        end,
        on_complete = function(quest) 
            TheGame:GetGameState():GetPlayerAgent():GetLocation():ClearAgents()
        end
    }

    :AddCast{
        cast_id = "handler",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "KALANDRA" or "SPARK_CONTACT"))
        end
    }

    :AddObjective{
        hide_in_overlay = true,
        id = "do_event",
        state = QSTATUS.ACTIVE,
        terminal = true,
    }
    
    :AddLocationDefs{
        MY_LOCATION = {
            name = "Road",
            plax = "EXT_Bog_Outcrop_01",
        },
    }


QDEF:AddConvo("do_event")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                * The bog presses in on all sides.
                {working_with_rise?
                handler:
                    !right
                    !cagey
                    $miscHushedUrgent
                    I've lived in this bog my whole life...
                    And I know when we're being stalked.
                player:
                    !left
                    !suspicious
                    Someone's following us?
                handler:
                    $miscHushedUrgent
                    Some<i>thing</i>. 
                    Quiet.
                * You hold your breath, straining your hearing.
            }
            {working_with_spark_barons?
                handler:
                    !right
                    !interest
                    $neutralDubious
                    All right, old chap? You seem a bit tense.
                player:
                    !left
                    !suspicious
                    $miscHushedUrgent
                    Something feels off.
                handler:
                    !thought
                    $neutralThoughtful
                    Does it? I don't believe we're being followed.
                    I flatter myself that I would notice if we were.
                player:
                    $miscHushedUrgent
                    Not if our tail had an evolutionary advantage.
                    !placate
                * You hold out your hand to silence the group, ignoring Fellemo's dubious look.
                monster:
                    !right
            }

            ]],

            OPT_DEFEND = "Defend {handler}",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    $angryDefault
                    Give the beasts space, {handler}. I'll take care of this.
            ]],
            DIALOG_DEFEND_SUCCESS = [[
                * With the beasts taken care of, you turn to check on {handler}.
                
                {working_with_rise?
                    handler:
                        !right
                        $miscRelieved
                        Believe me. You learn to trust your danger sense real quick when you work out here.
                        Imagine being a child and having to deal with that.
                    player:
                        $neutralDubious
                        Quite honestly, I can't imagine it at all.
                    handler:
                        Lucky you. 
                        Anyway, we best head out before the scavengers catch wind of the blood. 
                }
                
                {working_with_spark_barons?
                    handler:
                        !right
                        !surprised
                        $scaredTakenAback
                        By Hesh's reach! What <i>were</i> those things?
                    player:
                        !tired
                        You haven't encountered them before?
                    handler:
                        !neutral 
                        Of course not. Threats like that are contained well before they reach headquarters.
                        Let's press on before the stench of the corpses draws some other nefarious creature.
                }
            ]],
            
        }
        :Fn(function(cxt) 
            cxt.encounter:DoLocationTransition( cxt.quest:SpawnTempLocation("MY_LOCATION") )

            local monster_parties = {
                {"ERCHIN", "ERCHIN", "ERCHIN", "ERCHIN"},
                {"GRAWKIT"},
                {"FERAL_AUTODOG", "FERAL_AUTODOG", "FERAL_AUTODOG","FERAL_AUTODOG", "FERAL_AUTODOG","FERAL_AUTODOG", "FERAL_AUTODOG"},
            }

            
            local monsters = CreateCombatParty(monster_parties[math.random(#monster_parties)], 4, cxt.location)
            cxt:ReassignCastMember("monster", monsters[1])
            cxt:Dialog("DIALOG_INTRO")

            local flags
            if monsters[1]:GetContentID() == "FERAL_AUTODOG" then
                flags = BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_DEATH_LOOT
            else
                flags = BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.SELF_DEFENCE
            end
            
            local allies = cxt.quest.param.working_with == "RISE" and 
                {TheGame:GetGameState():AddSkinnedAgent( "RISE_AUTOMECH") } or
                {TheGame:GetGameState():AddSkinnedAgent( "AUTODOG"), TheGame:GetGameState():AddSkinnedAgent( "AUTODOG") }

            cxt:TalkTo("monster")
            local handler_card = cxt.quest.param.working_with == "RISE" and "call_in_kalandra" or "call_in_fellemo"

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = flags,
                    
                    no_oppo_limit = true,
                    noncombatants = {cxt.quest:GetCastMember("handler")},
                    allies = allies,
                    on_start_battle = function(battle) 
                        if handler_card then
                            local card = Battle.Card( handler_card, battle.player_team:Primary() )
                            card.show_dealt = true
                            battle:DealCard( card, battle:GetDrawDeck() )
                        end
                    
                    end

                }
                    :OnWin()
                        :Fn(function() 
                            cxt:Dialog("DIALOG_DEFEND_SUCCESS")
                            cxt.quest:Complete()
                            StateGraphUtil.AddEndOption(cxt)
                        end)


        end)

QDEF:AddDebugOption("working_with", {"RISE", "SPARK_BARONS"})

QDEF:AddTest(TestCreator()
                :SetReps(5)
                :SetParams{working_with= "RISE"})

QDEF:AddTest(TestCreator()
                :SetReps(5)
                :SetParams{working_with= "SPARK_BARONS"})
