


local QDEF = QuestDef.Define{
        title = "Deal with {other_handler}",
        qtype = QTYPE.STORY,

        on_init = function(quest)
            TheGame:GetGameState():SetDifficulty(5)
            quest.param.working_with = quest.param.working_with or "RISE"
            UIHelpers.PassTime(DAY_PHASE.NIGHT)
        end,
        on_start = function(quest) 
            
            if quest.param.killed_lt then
                quest:UnassignCastMember("faction_second")
            end
            
            if quest.param.killed_other_lt then
                quest:UnassignCastMember("other_faction_second")
            end

        end,
        on_complete = function(quest) 
            TheGame:GetGameState():GetPlayerAgent():GetLocation():ClearAgents()
        end
    }

    :AddObjective{
        hide_in_overlay = true,
        id = "deal_with_handler",
        title = "Defeat {bog_boss}",
        state = QSTATUS.ACTIVE,
        terminal = true,
    }

    :AddLocationDefs{
        MY_LOCATION = {
            name = "The Bogger Sanctum",
            plax = "EXT_BOG_DEEPBOG",
        },
    }
    

    :AddCast{
        cast_id = "handler",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "KALANDRA" or "SPARK_CONTACT"))
        end
    }
    :AddCast{
        cast_id = "faction_second",
        no_validation = true,
        optional = true,
        cast_fn = function(quest, t) 
            table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "RISE_SECOND" or "SPARK_SECOND"))
        end
    }
    
    :AddCast{
        cast_id = "other_handler",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "SPARK_CONTACT" or "KALANDRA" ))
        end
    }

    :AddCast{
        cast_id = "other_faction_second",
        no_validation = true,
        optional = true,
        cast_fn = function(quest, t) 
            table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "SPARK_SECOND" or "RISE_SECOND"))
        end
    }
    
    :AddOpinionEvents{
        did_not_gain_trust =  
        {
            delta = OPINION_DELTAS.DIMINISH,
            txt = "Gained their suspicion",
        },
        
        gained_trust =  
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Gained their trust by defeating their rival",
        },
    }


local function DoSwitchFight(cxt, opt)
    
        local allies = cxt.quest.param.working_with == "RISE" and 
            {TheGame:GetGameState():AddSkinnedAgent( "SPARK_BARON_GOON"), TheGame:GetGameState():AddSkinnedAgent( "SPARK_BARON_PROFESSIONAL") } or
            {TheGame:GetGameState():AddSkinnedAgent( "RISE_REBEL"), TheGame:GetGameState():AddSkinnedAgent( "RISE_RADICAL") }

        local handler_card = cxt.quest.param.working_with == "RISE" and "call_in_fellemo" or "call_in_kalandra"
        opt:Fn(function(cxt) 

            local remove_card = cxt.player.battler:FindCardByID( cxt.quest.param.working_with == "RISE" and "rise_expedition" or "baron_expedition")
            if remove_card then
                cxt.player.battler:RemoveCard( remove_card )
            end
        end)
        :Battle{
            flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.NO_FLEE,
            allies = allies,
            enemies = cxt.quest.param.working_with == "RISE" and cxt.enc.scratch.rise_oppo or cxt.enc.scratch.baron_oppo,
            
            on_start_battle = function(battle) 
                if handler_card then
                    local card = Battle.Card( handler_card, battle.player_team:Primary() )
                    card.show_dealt = true
                    battle:DealCard( card, battle:GetDrawDeck() )
                end
            
            end

        }
            :OnWin()
                :Fn(function() 
                    
                    --swap!
                    local old_handler = cxt.quest:GetCastMember("handler")
                    local new_handler = cxt.quest:GetCastMember("other_handler")
                    cxt.quest:UnassignCastMember("handler")
                    cxt.quest:UnassignCastMember("other_handler")
                    cxt.quest:AssignCastMember("other_handler", old_handler)
                    cxt.quest:AssignCastMember("handler", new_handler)

                    cxt.encounter.scratch.killed_lt, cxt.encounter.scratch.killed_other_lt = cxt.encounter.scratch.killed_other_lt, cxt.encounter.scratch.killed_lt  
                    local new_boss_faction = cxt.quest.param.working_with == "RISE" and "SPARK_BARONS" or "RISE"
                    cxt.enc.scratch.working_with_rise = new_boss_faction == "RISE"
                    cxt.enc.scratch.working_with_spark_barons = new_boss_faction == "SPARK_BARONS"
                    cxt.enc.scratch.handler_dead = new_handler:IsDead()
                    cxt.enc.scratch.other_handler_dead = old_handler:IsDead()
                    
                    cxt.quest.param.working_with = new_boss_faction

                    if cxt.quest.param.parent_quest then
                        cxt.quest.param.parent_quest.param.working_with = new_boss_faction
                        cxt.quest.param.parent_quest:UnassignCastMember("handler")
                        cxt.quest.param.parent_quest:AssignCastMember("handler", cxt.quest:GetCastMember("other_handler"))
                    end
                    
                    cxt:GainCards{new_boss_faction == "RISE" and "rise_expedition" or "baron_expedition"}                    

                    cxt:TalkTo("handler")
                    cxt:Dialog("DIALOG_WON")
                    cxt:GoTo("STATE_CONVINCE_TRUST")
                end)
end


QDEF:AddConvo("deal_with_handler")
    :ConfrontState("STATE")
        :Loc{
            DIALOG_INTRO = [[
                * The light dims as you march deeper into the bog.
                * It's not just the daylight that's dwindling--the air grows thicker, congealing into a miasma of spores and swamp gas. 
                player:
                    !left
                    Something's not right. 
                * The air is thick enough to dampen sounds, but you know all the same when you've walked into a trap.
                other_handler:
                    !right
                    !hips
                player:
                    !left

                {killed_lt? 
                    {working_with_rise?
                        other_handler:
                            Arint's body was uncovered this morning by a routine patrol.
                            $angrySeething
                            And here you are—not just fraternizing but favoring the enemy.
                            $angryDefault
                            I suppose this is what I deserve, eh? For trusting a spy.
                            Foolish of me to assume the bonds of war could keep us close.
                            $angryCruel
                            Perhaps your dishonorable discharge was well-earned after all.
                    }
                    {working_with_spark_barons?
                        other_handler:
                            You think I wouldn't know something was up when Flek didn't report?
                            $angrySeething
                            I've known him since we were kids. And you left him to rot in a ditch.
                            $angryDefault
                            Guess it makes sense the Rise could never satisfy your bottom line.
                            You're nothing but a parasite.
                    }
                }
                {not killed_lt?
                    {working_with_rise?
                        other_handler:
                            Ah, Captain. There you are.
                            $miscAlluring
                            I was beginning to worry {handler} might have caught wind of your duplicity and left you dead in the bog. 
                            But I think it's time this charade came to an end.
                            !hips
                            $miscPersuasive 
                            {handler}, come peacefully and I'll see to it you're executed cleanly.
                            And that the workers under your influence are treated fairly. 
                        handler:
                            !left
                            !angry_accuse
                            $angryDefault
                            Oh, no, you don't, slimesucker. Rook? Now's your time to prove your word. 
                            With {other_handler} out of the picture, labor negotiations are as good as won. 
                        player:
                            !left
                    }
                    
                    {working_with_spark_barons?
                        other_handler:
                            Alright, Rook? For a second I wondered if Fellemo had maybe hooked you for insubordination.
                            !cruel 
                            But you're only fashionably late. 
                            $neutralDirect
                            Lay down your arms, {handler}, and come quiet, and we can negotiate a fair deal.
                            Otherwise I'll have to kill you. And I'm afraid your old war buddy won't do nothing to stop it.
                        handler:
                            !left
                            !angry_accuse
                            My old "war buddy" would rather lay down his life than betray a comrade. 
                            I followed this man into more danger than even you're in right now, young lady.
                            !crossed
                            Captain? Show this woman how we dealt with threats in the Aerostat.
                        player:
                            !left
                    }

                }

            ]],

            OPT_DEFEND_HANDLER = "Defend {handler}",
            DIALOG_DEFEND_HANDLER = [[
                player:
                    !fight
                    Get back, {handler}. I'll handle this.
            ]],

            DIALOG_WON = [[
                {other_handler_dead?
                    {working_with_rise? 
                        handler:
                            !right
                            $miscRelieved
                            Fellemo's dead. You really did it.
                            !happy
                            If only Flek could be here. He would scold me for not believing in you.
                        
                        {killed_lt?
                            player:
                                !permit
                                $neutralResigned
                                I am truly sorry about Flekfis. His loyalty to you was striking.
                            handler:
                                !palm
                                $miscDepressed
                                In a weird way, he would've been satisfied knowing this is what it led to.
                                With Fellemo dead, and our victory almost guaranteed.
                        }

                        handler:
                            !crossed
                            Assuming it wasn't just a ruse, of course.    
                        player:
                            !dubious
                            Meaning... ?
                        handler:
                            $neutralResigned
                            After all this... I gotta say, Rook. It's hard to have faith.
                            So much double dealing, so much lying.
                            I can't shake the feeling that there's a long con in this somewhere.
                    }

                    {working_with_spark_barons?
                        handler:
                            !right 
                            Ah, well. I would've preferred a public execution, but perhaps that would've just made a martyr out of her.
                            I must say, old boy, you did have me worried for a second there.
                            $neutralDubious
                            {other_handler} seemed quite convinced of your performance.
    
                        {not killed_lt?
                            player:
                                You did hire a spy, remember. I had to maintain cover.
                            handler:
                                Yes, so I told Arint. She might forgive you, with time, but she'll never forget.
                                I hope it's a lesson she takes to heart. I'd be well-served if she took a leaf out of your book.
                                !suspicious
                                $neutralThoughtful
                                That said... if dealing with Arint were part of the plan, why didn't you tell me of it?
                        }
                        
                        player:
                            Bee in your bonnet, Fellemo?
                        handler: 
                            You always were five steps ahead, Rook. It makes me wonder: if I seem to be keeping pace, perhaps I should be suspicious.
                    }
                }

                {not other_handler_dead?
                
                    {working_with_rise? 
                        other_handler:
                            !right
                            !injured
                            $angrySeething
                            You fool. You damn, dishonorable maggot. 
                            Years of my life were dedicated to following you behind enemy lines.
                            And this is how I'm repaid? 
                        handler:
                            !left
                            !angry 
                            $angrydefault
                            The only maggot here is you, {other_handler}. And it's about time everyone knew it.
                            You're going to run back to headquarters, and you're going to sign our workers' constitution.
                            Because you know I'm going to get the cache, and what comes after that is up to you.
                            !cruel
                            $miscMocking
                            Play nice and maybe you'll be allowed to peaceably retire to the Foam or wherever it is rich slugs like you go. 
                        other_handler:
                            !exit
                        * Kalandra sends Fellemo back to civilization with a kick to the backside.
                        player:
                            !left
                            No need to kick a man when he's down, you know.
                        handler:
                            !right
                            About that. After all this... I gotta say, Rook. It's hard to have faith.
                            So much double dealing, so much lying.
                            I can't shake the feeling that there's a long con in this somewhere.
                    }

                    {working_with_spark_barons?
                        other_handler:
                            !right
                            !injured
                            Finish me off, Rook. If there's any sense of decency in you that isn't curdled by deceit.
                            I refuse to be a puppet to his propaganda.
                        handler:
                            !left
                            !cruel
                            $angryCruel
                            I think not, Kalandra. 
                            $miscMocking
                            Hobble along back to your work camp. I'm sure you'll find the workers already quelled and quarantined.
                            They await only my word. 
                            If your Rise aren't rounded up by the time I return, then the whole work camp will be flushed. 
                            And I'll have the means to do it.
                        other_handler:
                            $scaredPowerless
                            Y-you wouldn't. You can't afford to lose all your laborers.
                        handler:
                            $miscMocking
                            Oh, after we recover the cache I'll be able to afford almost everything.
                            And I understand that the Cult of Hesh is desperate for trade after what happened in Murder Bay.
                            A worthy supply of relics in exchange for a derrick worth of labor seems reasonable.
                        other_handler:
                            !exit
                        * Kalandra gets up and starts running, her injuries overwhelmed by urgency.
                        player:
                            !left
                            $neutralDubious
                            Well, I'm glad that worked out as you liked.
                        handler:
                            !right
                            !suspicious
                            Yes. Convenient, isn't it?
                        player:
                            Bee in your bonnet, Fellemo?
                        handler: 
                            You always were five steps ahead, Rook. It makes me wonder: if I seem to be keeping pace, perhaps I should be suspicious.
                    }
                }

            ]],



            OPT_CONVINCE_OTHER_HANDLER = "Try to switch sides and join {other_handler}",
            DIALOG_CONVINCE_OTHER_HANDLER = [[
                player:
                    A turnabout like this requires sacrifices.
                    You couldn't expect me to pull this kind of deep cover without taking risks.
                    I'm sorry, but your lieutenant got in the way.
            ]],

            DIALOG_CONVINCE_OTHER_HANDLER_SUCCESS = [[
                player:
                    !shrug
                    $miscPersuasive
                    It's regretful, but the circumstances forced my hand. 
                    I assure you that my loyalty is truly to you.
                other_handler:
                    !angry
                    You'll prove it by turning your sights on {handler}.
                handler:
                    !right
                    !angry
                    $angryTakenAback
                    You can't be serious. 
            ]],
            DIALOG_CONVINCE_OTHER_HANDLER_FAILURE = [[
                player:
                    !crossed
                    It's the risk of the business. If you didn't like it, you shouldn't have hired me.
                other_handler:
                    !angry
                    On that we both agree.
            ]],

            OPT_DEFEND_OTHER_HANDLER = "Switch sides and attack {handler}",
            DIALOG_DEFEND_OTHER_HANDLER = [[
                handler:
                    !right
                    !scared
                player:
                    Unfortunately, this is where the chips, and you, both fall.
                    !fight

            ]],


            OPT_CONTINUE_WITH_NEW_HANDLER = "Continue with {handler}",
            DIALOG_CONTINUE_WITH_NEW_HANDLER = [[
                player:
                    I believe there's a treasure waiting for you somewhere in this bog.
                    Now, shall we use my map to claim it?
            ]],
            DIALOG_CONTINUE_WITH_NEW_HANDLER_2 = [[
                other_handler:
                    Lead on.
            ]],

            OPT_ABANDON_EVERYONE = "Leave {handler} and {other_handler} to fight each other",
            DIALOG_ABANDON_EVERYONE = [[
                player:
                    !exit
                * With the two occupied, you slip into the bog, the memory of the map burned into your mind.
                * You couldn't have asked for a better distraction. But then, that's why you planned it this way.
                * Behind you is the serenade of pistolfire. 
            ]],
        }
        :Fn(function(cxt) 
            cxt.encounter.scratch.killed_lt = cxt.quest:GetCastMember("faction_second") == nil
            cxt.encounter.scratch.killed_other_lt = cxt.quest:GetCastMember("other_faction_second") == nil
            cxt.encounter.scratch.working_with_spark_barons =  cxt.quest.param.working_with == "SPARK_BARONS"
            cxt.encounter.scratch.working_with_rise =  cxt.quest.param.working_with == "RISE"

            cxt.encounter:DoLocationTransition( cxt.quest:SpawnTempLocation("MY_LOCATION") )

            cxt.enc.scratch.rise_oppo = {cxt.quest:GetCastMember(cxt.quest.param.working_with == "RISE" and "handler" or "other_handler")}
            
            for k,v in ipairs({"RISE_REBEL", "RISE_RADICAL", "RISE_AUTOMECH"}) do
                local agent = TheGame:GetGameState():AddAgent(Agent(v))
                table.insert(cxt.enc.scratch.rise_oppo, agent)
            end

            cxt.enc.scratch.baron_oppo = {cxt.quest:GetCastMember(cxt.quest.param.working_with == "RISE" and "other_handler" or "handler")}
            for k,v in ipairs({"SPARK_BARON_PROFESSIONAL", "SPARK_BARON_AUTOMECH", "AUTODOG"}) do
                local agent = TheGame:GetGameState():AddAgent(Agent(v))
                table.insert(cxt.enc.scratch.baron_oppo, agent)
            end


            cxt:TalkTo("other_handler")
            cxt:Dialog("DIALOG_INTRO")
    
            local allies = cxt.quest.param.working_with == "RISE" and 
                {TheGame:GetGameState():AddSkinnedAgent( "RISE_REBEL"), TheGame:GetGameState():AddSkinnedAgent( "RISE_RADICAL") } or 
                {TheGame:GetGameState():AddSkinnedAgent( "SPARK_BARON_GOON"), TheGame:GetGameState():AddSkinnedAgent( "SPARK_BARON_PROFESSIONAL") }


            cxt:RunLoop(function() 
                local handler_card = cxt.quest.param.working_with == "RISE" and "call_in_kalandra" or "call_in_fellemo"

                cxt:Opt("OPT_DEFEND_HANDLER")
                    :Dialog("DIALOG_DEFEND_HANDLER")
                    :ReceiveOpinion(OPINION.BETRAYED, nil, "other_handler")
                    :Battle{
                            flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.NO_FLEE,
                            allies = allies,
                            enemies = cxt.quest.param.working_with == "RISE" and cxt.enc.scratch.baron_oppo or cxt.enc.scratch.rise_oppo,
                        
                            on_start_battle = function(battle) 
                                if handler_card then
                                    local card = Battle.Card( handler_card, battle.player_team:Primary() )
                                    card.show_dealt = true
                                    battle:DealCard( card, battle:GetDrawDeck() )
                                end
                            
                            end

                        }
                        :OnWin()
                            :Fn(function() 
                                cxt.enc.scratch.handler_dead = cxt.quest:GetCastMember("handler"):IsDead()
                                cxt.enc.scratch.other_handler_dead = cxt.quest:GetCastMember("other_handler"):IsDead()

                                cxt:Dialog("DIALOG_WON")
                                cxt:TalkTo("handler")
                                cxt:GoTo("STATE_CONVINCE_TRUST")

                            end)


                if cxt.encounter.scratch.killed_other_lt then
                    
                    cxt:Opt("OPT_CONVINCE_OTHER_HANDLER")
                        :Dialog("DIALOG_CONVINCE_OTHER_HANDLER")
                        :Negotiation{
                            target_agent = cxt.quest:GetCastMember("other_handler"),
                            suppressed = {cxt.quest:GetCastMember("handler")},
                        }
                            :OnSuccess()
                                :Dialog("DIALOG_CONVINCE_OTHER_HANDLER_SUCCESS") 
                                :ReceiveOpinion(OPINION.BETRAYED, nil, "handler")

                                :Fn(function() 
                                    local opt = cxt:Opt("OPT_DEFEND_OTHER_HANDLER")
                                        :Dialog("DIALOG_DEFEND_OTHER_HANDLER")
                                    DoSwitchFight(cxt, opt)
                                end)

                            
                            :OnFailure()
                                :Dialog("DIALOG_CONVINCE_OTHER_HANDLER_FAILURE") 
                
                else

                    local opt = cxt:Opt("OPT_DEFEND_OTHER_HANDLER")
                                    :ReceiveOpinion(OPINION.BETRAYED, nil, "handler")
                                    :Dialog("DIALOG_DEFEND_OTHER_HANDLER")
                        DoSwitchFight(cxt, opt)
                end


                cxt:Opt("OPT_ABANDON_EVERYONE")
                    :Dialog("DIALOG_ABANDON_EVERYONE")
                    :ReceiveOpinion(OPINION.BETRAYED, nil, "other_handler")
                    :ReceiveOpinion(OPINION.BETRAYED, nil, "handler")
                    :Fn(function() 
                        local rise_card = cxt.player.battler:FindCardByID("rise_expedition")
                        local baron_card = cxt.player.battler:FindCardByID("baron_expedition")
                        if rise_card then
                            cxt.player.battler:RemoveCard( rise_card )
                        end
                        if baron_card then
                            cxt.player.battler:RemoveCard( baron_card )
                        end
                        
                        if cxt.quest.param.parent_quest then
                            cxt.quest.param.parent_quest.param.working_with = nil
                        end

                    end)
                    :CompleteQuest()
                    :DoneConvo()

            end)
    end)

    :State("STATE_CONVINCE_TRUST")
        :Loc{
            OPT_CONVINCE_TRUST = "Convince {handler} you are on their side",
            DIALOG_CONVINCE_TRUST = [[
                player:
                    I suppose I have only my actions to speak for me.
            ]],
            DIALOG_CONVINCE_TRUST_SUCCESS = [[
                player:
                    !overthere
                    $miscPersuasive
                    You asked me to walk an impossible line, serving two masters, earning the trust of both.
                    I believe I did so to the best of my ability. And now here I stand, by <i>your</i> side. 
                    $neutralDirect
                    No one else's.
                {working_with_rise? 
                    handler:
                        !sigh
                        $neutralResigned
                        You're right. Of course. 
                        Believe it or not, I haven't always been able to trust people.
                        But I'm choosing to trust you. 
                    player:
                        I appreciate it, Kalandra.
                        !permit 
                        When we began this journey you said you'd never ask me to take the moral high ground.
                        $miscNostalgic
                        But I have done. And the view is fine. 
                        Now... that cache awaits. 
                }

                {working_with_spark_barons?
                    handler: 
                        !agree
                        $miscNostalgic
                        Yes, indeed. Just like I stood next to you in Rentoria. 
                    player:
                        And the Lakespit intrigue. And the Roaloch.
                    handler: 
                        !happy 
                        $miscNostalgic
                        Operation Controlled Burn! How could one forget.
                        Of course, Rook. Captain. 
                        Spies we might have been. But we were also brothers-in-arms. 
                        $miscNostalgic
                        And that bond doesn't decay.
                    player:
                        Our mission isn't yet complete. We still have a weapon cache to secure.
                    handler:
                        !permit
                        Then lead the way, Captain. 
                }
            ]],
            DIALOG_CONVINCE_TRUST_FAILURE = [[
            player:
                    I've done only as you asked. Earned {other_handler}'s trust only at your behest.
                {working_with_rise?
                    handler: 
                        !crossed
                        $neutralDirect
                        Maybe so. But we have no connection. You don't know me, and I don't know you.
                        I hired you through a shadow, and retained you with shills.
                        Those kinds of loyalties are about as stable as the bog we stand in. 
                        So if it's all the same to you... I'll be keeping a close eye on you as we finish the deal.
                    player:
                        $neutralResigned
                        As you like, of course.
                        Shall we continue?
                }
                    
                {working_with_spark_barons?
                    handler:
                        !thought
                        It's the "only" part I struggle with. 
                        You see, I have some contacts in the Admiralty still, and I'm not one to let a detail go overlooked. 
                        So I wondered why you were dishonorably discharged.
                    player:
                        !crossed
                        I was smuggling Admiralty secrets to a rebel group in the Bread Fields. 
                        Is that what this is about?
                    handler:
                        !cruel
                        $miscMocking
                        Ah yes, an unnamed "rebel group". 
                        Farmers who were keen to throw off their Admiralty yokes. 
                        Of course, it wasn't long before those Farmers named their movement, and allowed it to spread insidiously to other regions.
                        $angryCruel
                        You must know what those farmers call themselves <i>now</i>.
                    player:
                        ...
                        $neutralDirect
                        The Rise.
                    handler:
                        So you understand why I might be skeptical of your true purpose.
                        But time will tell, eh? And the bog ahead is very dangerous.
                        So lead on, my Captain.
                        $angryCruel 
                        And I will follow from behind. 
                }
            ]],
            OPT_CONTINUE_NO_TRUST = "Brush off {handler}'s concerns",

            DIALOG_CONTINUE_NO_TRUST = [[
                player:
                    !angry_shrug
                    $angryPatienceLost
                    I've done only as you asked. That you now have doubts is none of my concern.
                    But there is a job I've been hired to do and I intend to finish it.
                    Unless I'm being dismissed from duty?
                handler:
                    $neutralDirect
                    No. You aren't.
                    You're just being watched.
                    !exit
                * You continue through the bog, doing your best to navigate the treacherous terrain.
            ]],
            SIT_MOD = "Wants to trust you",
        }
        :Fn(function(cxt) 

            if not cxt.quest:GetCastMember("other_handler"):IsDead() then
                cxt.quest:GetCastMember("other_handler"):MoveToLimbo()
            end

            cxt:Opt("OPT_CONVINCE_TRUST")
                :Dialog("DIALOG_CONVINCE_TRUST")
                :Negotiation{
                    situation_modifiers =
                    {
                        { value = -15, text = cxt:GetLocString("SIT_MOD")  }
                    },
                }
                    :OnSuccess()
                    :Dialog("DIALOG_CONVINCE_TRUST_SUCCESS")
                    :ReceiveOpinion("gained_trust")
                    :CompleteQuest()
                    :DoneConvo()

                :OnFailure()
                    :Dialog("DIALOG_CONVINCE_TRUST_FAILURE")
                    :ReceiveOpinion("did_not_gain_trust")
                    :CompleteQuest()
                    :DoneConvo()

            cxt:Opt("OPT_CONTINUE_NO_TRUST")
                :Dialog("DIALOG_CONTINUE_NO_TRUST")
                :ReceiveOpinion("did_not_gain_trust")
                :CompleteQuest()
                :DoneConvo()

        end)


QDEF:AddDebugOption("working_with", {"RISE", "SPARK_BARONS"})
QDEF:AddDebugOption("killed_lt", {false, true})
QDEF:AddDebugOption("killed_other_lt", {false, true})