local QDEF = QuestDef.Define{
        title = "Monster Encounter",
        qtype = QTYPE.STORY,
        on_start = function(quest)
            if quest.param.working_with == "RISE" then
                quest.param.working_with_rise = true
            else
                quest.param.working_with_spark_barons = true
            end
        end,
        on_complete = function(quest) 
            TheGame:GetGameState():GetPlayerAgent():GetLocation():ClearAgents()
        end
    }

    :AddCast{
        cast_id = "handler",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "KALANDRA" or "SPARK_CONTACT"))
        end
    }

    :AddObjective{
        hide_in_overlay = true,
        id = "do_event",
        state = QSTATUS.ACTIVE,
        terminal = true,
    }
    
    :AddLocationDefs{
        MY_LOCATION = {
            name = "Ruins",
            plax = "EXT_Bog_HedgeGod_01",
        },
    }


QDEF:AddConvo("do_event")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                * The expedition reaches an abandoned ruin, easily defensible and good enough for a rest.
                player:
                    !left
                {working_with_rise?
                handler:
                    !right
                    Hold up. Let's take a breather.
                    !thought
                    Might do to assess our resources going forward, too. Make sure we're prepared for... whatever we might find.
                player:
                    !interest
                    Feeling apprehensive, are we?
                handler:
                    $neutralDubious
                    You know how many people have made it into the Deep Bog and survived?
                player:
                    I do not.
                handler:
                    $neutralDirect
                    Me neither. Only a fool goes in cocksure and blindsided.
                }
                {working_with_spark_barons?
                handler:
                    !right
                    !hips
                    A moment, Rook. This is a good moment to rest before we make the final approach.
                    You humans are rather short-winded, after all.
                player:
                    $miscMischievous
                    I suppose that means kra'deshi are therefore long-winded?
                handler:
                    !chuckle
                    $happyJoke
                    Hah! Quite. 
                }
                handler:
                    !exit
                * The troops spread out, leaving you to snatch at a couple quiet moments to yourself.

            ]],

            OPT_RESTORE_HEALTH = "Tend to your wounds",
            DIALOG_RESTORE_HEALTH = [[
                * You take a moment to catalog your inventory of injuries, both new and old.
                * You clean what you can, and apply salves from the expedition's supplies.
                * You're not quite good as new, but then again you weren't quite good when you were new anyway.
            ]],

            OPT_RESTORE_RESOLVE = "Meditate",
            DIALOG_MEDITATE = [[
                * You close your eyes and release your regrets into the wind.
                * The process always leaves you feeling sharper—after all, once all the recriminations and bad memories are gone, there isn't much left to take up space.
            ]],

            OPT_PRACTICE_FIGHTING = "Practice your combat techniques",
            DIALOG_PRACTICE_FIGHTING = [[
                * You checked your pistols last night, but a good soldier is always prepared.
                * After giving them a quick but thorough clean, you practice unholstering them with subtlety. 
            ]],


            OPT_PRACTICE_RHETORIC = "Practice your rhetorical skills",
            DIALOG_PRACTICE_RHETORIC = [[
                * It's uncertain how useful persuasion will be with these Boggers when you find them, but a sharp mind is never wasted.
                !agree
                * You sit in silent practice, the only movement a flurry of feigned emotions across your face, and the cogs spinning between your ears. 
                * The rest of the expedition is polite enough to pretend not to notice the strange display.
            ]],

            OPT_RALLY_THE_TROOPS = "Rally the troops",
            DIALOG_RALLY_THE_TROOPS = [[
                * You walk among the troops, critiquing their stance and offering advice on proper maintenance of their weapons. 
                * Soothed by your confidence, a few of them seem to muster strength for the road ahead.
            ]],

            OPT_EXPLORE_THE_TEMPLE = "Explore the temple",
            DIALOG_EXPLORE_THE_TEMPLE = [[
                player:
                    I'll check the perimeter.
                    !exit
                * You leave the group behind to explore the dusty old ruins. Though the bog has eaten away at their foundations, it's clear no one has passed this way in quite some time.
                * Though you're no archaeologist, your keen eye finds something interesting all the same. 
            ]],

            DIALOG_GAINED_CHARGE = "* Gained Charge: {1#card}",
            TT_WILL_GAIN_CHARGES = "Will regain charges: {1#card_list}",


            TT_PRACTICE_FIGHTING = "Upgrade a battle card of your choice.",
            TT_PRACTICE_RHETORIC = "Upgrade a negotiation card of your choice."
        }
        :Fn(function(cxt) 
            cxt.encounter:DoLocationTransition( cxt.quest:SpawnTempLocation("MY_LOCATION") )

            cxt:Dialog("DIALOG_INTRO")

            local special_card = cxt.player.battler:FindCardByID("rise_expedition") or cxt.player.battler:FindCardByID("baron_expedition")
            local restore_cards = {}
            if special_card then
                local all_spent_cards = {}
                
                for k,v in ipairs(special_card.userdata.stored_cards) do
                    local charges, max_charges = v:GetCharges()
                    if charges and charges < max_charges then
                        for n = 1, max_charges-charges do
                            table.insert(all_spent_cards, v)
                        end
                    end
                end
                
                if #all_spent_cards > 0 then
                    table.shuffle(all_spent_cards)
                end

                for k = 1, math.min(#all_spent_cards, 3) do
                    table.insert(restore_cards, all_spent_cards[k])
                end
            end

            cxt:RunLoopingFn( function()

                cxt:Opt("OPT_RESTORE_HEALTH")
                    :PreIcon(global_images.heal)
                    :Dialog("DIALOG_RESTORE_HEALTH")
                    :DeltaHealth( 15 )
                    :GoTo("STATE_DONE")

                local resolve, max_resolve = cxt.caravan:GetResolve()

                cxt:Opt("OPT_RESTORE_RESOLVE")
                    :PreIcon(global_images.restoreresolve)
                    :Dialog("DIALOG_MEDITATE")
                    :ReqCondition(resolve < max_resolve, "REQ_AT_FULL_RESOLVE")
                    :DeltaResolve( 25 )
                    :GoTo("STATE_DONE")

                cxt:Opt("OPT_PRACTICE_FIGHTING")
                    :Dialog("DIALOG_PRACTICE_FIGHTING")
                    :PostText("TT_PRACTICE_FIGHTING")
                    :PreIcon( global_images.upgradecombat )
                    :Fn( function( cxt )
                        cxt:Wait()
                        AgentUtil.UpgradeBattleCard( function( card )
                            cxt.enc:ResumeEncounter( card )
                        end )
                        local card = cxt.enc:YieldEncounter()
                        if card then
                            cxt:GoTo("STATE_DONE")
                        end
                    end )


                cxt:Opt("OPT_PRACTICE_RHETORIC")
                    :Dialog("DIALOG_PRACTICE_RHETORIC")
                    :PostText("TT_PRACTICE_RHETORIC")
                    :PreIcon( global_images.upgradenegotiation )
                    :Fn( function( cxt )
                        cxt:Wait()
                        AgentUtil.UpgradeNegotiationCard( function( card )
                            cxt.enc:ResumeEncounter( card )
                        end )
                        local card = cxt.enc:YieldEncounter()
                        if card then
                            cxt:GoTo("STATE_DONE")
                        end
                    end )


                if #restore_cards > 0 then
                    cxt:Opt("OPT_RALLY_THE_TROOPS")
                        :Dialog("DIALOG_RALLY_THE_TROOPS")
                        :PostText("TT_WILL_GAIN_CHARGES", restore_cards)
                        :Fn(function() 
                            for k,card in ipairs(restore_cards) do
                                card:RestoreCharges( 1 )
                                cxt:Dialog("DIALOG_GAINED_CHARGE", card)
                            end
                        end)
                        :GoTo("STATE_DONE")
                end

                cxt:Opt("OPT_EXPLORE_THE_TEMPLE")
                    :Dialog("DIALOG_EXPLORE_THE_TEMPLE")
                    :Fn(function() 

                        local loot = {money = 0, items = {"spark_grenade", "portable_bunker", "aerostat_coilgun", "bog_scimitar"}}
                        cxt.enc:AwardLoot(loot, { found=true } )

                    end)
                    :GoTo("STATE_DONE")
            end)


        end)
    :State("STATE_DONE")
        :Loc{
            DIALOG_INTRO = [[
                * {handler} raises {handler.hisher} voice for all to hear.
                handler:
                    !right
                    Time to go. 
                * You and the troops fall into step. The Deep Bog awaits.
            ]]
        }
    
    :Fn(function(cxt) 
        cxt:Dialog("DIALOG_INTRO")
        cxt.quest:Complete()
        StateGraphUtil.AddEndOption(cxt)
    end)


QDEF:AddDebugOption("working_with", {"RISE", "SPARK_BARONS"})

