local QDEF = QuestDef.Define{
        title = "Worker Encounter",
        qtype = QTYPE.STORY,
        
        on_start = function(quest)
            if quest.param.working_with == "RISE" then
                quest.param.working_with_rise = true
            else
                quest.param.working_with_spark_barons = true
            end
        end,
    }

    :AddCast{
        cast_id = "handler",
        no_validation = true,
        cast_fn = function(quest, t) 
            table.insert(t, TheGame:GetGameState():GetAgent(quest.param.working_with == "RISE" and "KALANDRA" or "SPARK_CONTACT"))
        end
    }

    :AddObjective{
        hide_in_overlay = true,
        id = "do_event",
        state = QSTATUS.ACTIVE,
        terminal = true,
        on_complete = function(quest) 
            TheGame:GetGameState():GetPlayerAgent():GetLocation():ClearAgents()
        end
    }
    
    :AddLocationDefs{
        MY_LOCATION = {
            name = "Road",
            plax = "EXT_Bog_Road_01",
        },
    }

    :AddOpinionEvents{
        did_not_help_worker = {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Did not help a worker",
        },
        saved_worker_from_barons = {
            delta = OPINION_DELTAS.LIKE,
            txt = "Saved a worker from the barons",
        },
        saved_from_barons = {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Saved them from the barons",
        },

        did_not_get_workers_to_work = {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Did not force workers back to work",
        },

        got_all_workers_to_work= {
            delta = OPINION_DELTAS.LIKE,
            txt = "Got a bunch of slacking workers to work again",
        },

        paid_bribe = {
            delta = OPINION_DELTAS.LIKE,
            txt = "Paid a big, fat bribe"
        }

    }

QDEF:AddConvo("do_event")
    :ConfrontState("CONF")
        :Fn(function(cxt) if cxt.quest.param.working_with == "RISE" then cxt:GoTo("STATE_RISE_VERSION") else cxt:GoTo("STATE_BARON_VERSION") end end)
    :State("STATE_RISE_VERSION")
        :Loc{
            DIALOG_INTRO = [[
                * The sounds of muddy footfall cause your whole party to pull up short.
                * You don't expect to find a worker this deep in the bog, but you do. The Barons at {worker.hisher} heels might explain why.
                baron:
                    !right
                    !angry 
                    $neutralWhatever
                    Keep it moving, you sludgepile.
                    You'll stop walking when I say so. 
                worker:
                    !left
                    !scared
                    $scaredStammering
                    Y-you've made your point, {baron}. I get it. I won't talk back again.
                    We don't want to r-run into anything dangerous out here, right?
                baron:
                    !cruel
                    "We" won't. But you might.
                * Your group has gone unnoticed in the bush. 
                player:
                    !left
                handler:
                    !right
                handler:
                    $miscHushedUrgent
                    You gotta save that worker, Rook.
                player:
                    $miscHushedUrgent
                    We're on a covert operation, you might recall.
                handler:
                    !agree
                    $miscHushedUrgent
                    Agreed. So the more covert, the better. 
                    We'll stay here. You go do your thing. 
            ]],
            OPT_CONFRONT_BARON = "Confront the Baron",
            DIALOG_CONFRONT_BARON = [[
                player:
                    Very well. 
                handler:
                    !exit
                * With {handler} and the other Rise out of sight, you approach the Barons. 
                baron:
                    !right
                player:
                    !greeting
                    $happyGreeting
                    Ah, hello. I was hoping I might run into someone.
                    I'm a bit lost—I don't suppose you could point me back towards civilization, eh?
                baron:
                    !suspicious
                    Sure—it's right back where you came from.
                worker:
                    !right
                    !scared
                    W-wait, please! This Baron is going to leave me out here to die!
                    You have to help me. I can show you the way back if you take me with you.
                baron:
                    !right
                    !angry
                    Shaddup! 
                    Ignore {worker.himher}. This is just a rudimentary patrol, and {worker}'s trying to shirk responsibility. 
            ]],

            OPT_CONVINCE_LET_GO = "Convince {baron} to let the worker go",
            DIALOG_CONVINCE_LET_GO = [[
                player:
                {liked?
                    Be reasonable, {baron}. We both know what this is really about.
                }
                {not liked?
                    I think not, Baron. Seems to me you're more interested in squandering company resources.
                }

            ]],
            DIALOG_CONVINCE_LET_GO_SUCCESS = [[
                {liked?
                    player:
                        !left
                        $miscPersuasive
                        Is your pride worth the scolding you'll get for wasting valuable resources?
                        !permit 
                        This worker looks healthy, fit—no doubt there are many profitable years left on {worker.hisher} contract. 
                        I'm just looking out for you, my friend. 
                    baron:
                        ... 
                        $neutralResigned
                        I suppose you're right.
                    worker:
                        !left
                        !scared
                    baron: 
                        !angry_accuse
                        You're getting a reprieve, {worker}, because my pal Rook here's got some sense.
                        $angryAside
                        But you sass me again, and there won't be any argument I'll listen to, understand?
                        !exit
                    * {baron} nods to {baron.hisher} entourage and they disperse back into the trees. 
                }
                {not liked?
                    player:
                        !left
                        Whatever petty grudge you're trying to play out will get back to Fellemo, I assure you.
                        !crossed
                        And then you'll have to explain yourself to <i>him</i>, if not me. 
                    baron:
                        !angry_shrug
                        $angryReluctant
                        Fine. Have it your way.
                        $angryCruel
                        But you watch yourself, grifter. You won't always be in Fellemo's good books.
                        And when you aren't...
                        !throatcut
                        ...
                        !exit
                    * The Barons skulk off, throwing bitter looks over their shoulders. 
                }

                player:
                    !left
                worker:
                    !right
                    !happy
                    $miscRelieved
                    Thank you! I dunno what I did to deserve this good turn.
                player:
                    You can find your way back from here, I assume.
                worker:
                    !bashful
                    $miscEmbarrassed
                    Yeah, I... think so?
                player:
                    !permit
                    Here, let me sketch out a route for you.
                * You spend a few moments relaying safe passage back through the bog. 
                player:
                    Keep to that road and you'll likely be fine.
                worker:
                    $happyThanks
                    You're a lifesaver, truly. Next time you're back at the workcamps, I'll do my best to make it up to you.
                    !exit
                handler:
                    !right
                    !humouring
                    Well, whaddya know. That was mighty kind of you, {player}.
                player:
                    $neutralWhatever
                    I did what you asked. I believe those are the terms of my employment.
                handler:
                    You went above and beyond what I asked. If it were me, I would've brought {worker} with us.
                    That would've given you more muscle to work with.
                    !happy 
                    Still, you are who you are. So let's get back to it.
            ]],
            DIALOG_CONVINCE_LET_GO_FAILURE = [[
                {liked?
                    player:
                        !permit
                        This can't be worth the paperwork. I know how much you loathe administrative duties.
                    baron:
                        !cruel
                        $angryCruel
                        Hah! {worker} here has a file as thick as {worker.hisher} skull.
                        I'll be <i>saving</i> paperwork in the long run if I get {worker.himher} outta my skin.
                }
                {not liked?
                    player:
                        !hips
                        $angryDefault
                        I can't stand idly by and watch you recklessly endanger this {worker} to satisfy your squabble.
                        Yield at once, or I'll be forced to take action.
                    baron:
                        !cruel
                        $angryCruel
                        And what action would that be? Kissing my ass?
                }
            ]],

            OPT_FIGHT = "Attack the Barons",
            DIALOG_FIGHT = [[
                {liked?
                    player:
                        I'm afraid you leave me no choice.
                        !fight
                    baron:
                        !surprised
                        $neutralTakenAback
                        Wha—?
                        Are you serious? You'd fight me over a worm like {worker}?
                        !angry
                        $angryPatienceLost
                        Fine. Have it your way.
                        !fight
                }
                {not liked?
                    player:
                        I'm afraid you leave me no choice.
                        !fight
                    baron:
                        Hah! This should be good. 
                        !fight
                }
            ]],
            DIALOG_FIGHT_WON = [[
                {dead and worker_dead?
                    * You've killed the Baron, but you were unable to save the worker.
                    player:
                        !sigh
                        $miscEmbarrassed
                        Ah. That didn't go quite as planned.
                    handler:
                        !right
                        !angry_shrug
                        $angryReluctant
                        No kidding. 
                        At least you stood up to them, though.
                        $neutralResigned 
                        Once we're done with this, we'll honor {worker} and all the others who've fallen.
                } 
                {not dead and worker_dead?
                    * You stand over {baron} victorious... but {worker} is dead.
                    baron:
                        Y-you killed {worker.himher}, not me!
                        !exit
                    * {baron} runs blindly into the bog. Whether or not {baron.heshe}'ll make it back safely is uncertain.
                    player:
                        !sigh
                        Ah. That didn't go quite as planned.
                    handler:
                        !right
                        !angry_shrug
                        $angryPatienceLost
                        No kidding. 
                        Now the Baron might report what {baron.heshe} saw. 
                        We better get moving before this comes back to haunt us.
                } 
                {not dead and not worker_dead?
                    baron:
                        !injured
                        $scaredStammering
                        H-hey, I dunno whose side you're on but I want out. 
                        You can have {worker} for all I care.
                        !exit
                    * {baron} runs blindly into the bog.
                    worker:
                        !right
                        !happy
                        $happyThanks
                        Thank you! I dunno what I did to deserve this good turn.
                    player:
                        Go on, then. If you're smart, you'll follow {baron} and let {baron.himher} be your guide. 
                        !point
                        From a distance, of course.
                    worker:
                        !agree 
                        $happyThanks
                        Okay. Yeah. Sure. 
                        !exit
                    * You watch {worker} head warily into the bog.
                    handler:
                        !right
                        !facepalm
                        $neutralResigned
                        Feeling compassionate, are we?
                    player:
                        $neutralDubious
                        I take it you disapprove.
                    handler: 
                        !overthere
                        That Baron was the enemy. If {baron.heshe} makes it back {baron.heshe} might report you. 
                        So let's get moving in case they get on our tail. 
                } 
                {dead and not worker_dead?
                    * {baron}'s corpse hits the ground with a sickening squelch.
                    worker:
                        !right
                        !scared 
                        $scaredTakenAback
                        Y-you killed {baron.himher}? What am I gonna tell the others?
                    player:
                        !point
                        Nothing. You'll head back to camp and pretend you merely lost your way in the bog.
                        I doubt anyone worth fearing will ever find {baron.hisher} body.
                    worker:
                        $scaredStammering
                        R-right. Okay.
                        !exit
                    * {worker} heads warily into the bog, eager to put the memory behind {worker.himher}.
                    handler:
                        !right
                        $neutralDirect
                        Good thinking. Best nobody knows we were here. 
                        Now let's get a move on. We've already lost enough daylight. 
                        
                } 
            ]],

            OPT_BACK_DOWN = "Back down",
            DIALOG_BACK_DOWN = [[
                player:
                    You make a compelling point. 
                    !overthere
                    So by all means, continue.
                    !exit
                baron:
                    !cruel 
                    $angryCruel
                    Oh, I intend to.
                    !exit
                * The Baron and {baron.hisher} entourage converge on {worker}. You make sure to avoid {worker.hisher} gaze.
                * They vanish deeper into the bog with a savage slurp of the mud at their ankles. If you recall the map correctly, it's very unlikely any of them will make it back alive.
                player:
                    !left
                handler:
                    !right
                    !angry
                    $angryAside
                    You were <i>supposed</i> to help {worker.himher}.
                player:
                    !crossed
                    $neutralWhatever
                    I was supposed to remain covert, so I chose to disengage rather than escalate.
                    You know as well as I do that change requires sacrifice, my dear Kalandra.
                handler:    
                    I'll keep that in mind the next time <i>you're</i> the one in hot water.
                    Now let's keep going, before any of those Barons double back.
                    !exit

            ]],
            
            OPT_KEEP_WALKING = "Keep walking",
            DIALOG_KEEP_WALKING = [[
                player:
                    !dubious
                    You're telling me you wish to jeopardize everything for one worker?
                handler:
                    $miscHushedUrgent
                    That one worker has a family. Friends. 
                player:
                    $neutralDirect
                    So do the Barons you're always content to cut down. But that's an easier sacrifice to make, I suppose.
                handler:
                    !suspicious
                    ... 
                    $angryReluctant
                    Fine. We keep walking.
                    !exit
                * You lead the party around the disturbance, but {handler} falls into a sullen silence.
            ]],
            OPT_PAY_OFF = "Bribe {baron}",
            DIALOG_PAY_OFF = [[
                player:
                    !placate
                    I have no doubt that {worker} has caused you some grief. 
                    $miscPersuasive
                    But surely a little financial incentive would serve as a suitable balm?
                baron:
                    !suspicious
                    You bribing me? For <i>this</i> waste of gristle?
                player:
                    It's a fair price, is it not?
                    !give
                baron:
                    !take
                    If you ask me, you overpaid.
                    {worker} can't afford any more debts, but I'll gladly leave you to sort that out.
                    !exit
                * Chuckling, the Barons take their leave. You fail to call out to them when you notice them heading in the wrong direction.
                * They'll figure it out eventually. Perhaps when they're neck-deep in a sinkhole. 
                worker:
                    !right
                    !scared
                    $miscRelieved
                    Thank you, truly. But I don't got any way to pay you back.
                player:
                    No, I suspected as much. Somehow I think my bottom line will still be met.
                    !point
                    Ah, be sure you go <i>that</i> way when leaving.
                worker:
                    $happyThanks
                    Oh—right! Thanks again!
                * {worker} hurries off, more or less in the right direction. 
                handler:
                    !right
                    !crossed
                    I'd pay you back, but I'm not exactly flush at the moment.
                player:
                    We'll settle up at the end of the day, I assure you.
            ]]
        }
        :Fn(function(cxt) 
            cxt.encounter:DoLocationTransition( cxt.quest:SpawnTempLocation("MY_LOCATION") )
            
            local worker = TheGame:GetGameState():AddSkinnedAgent( "HEAVY_LABORER")
            worker:MoveToLocation(cxt.location)
            
            local barons = CreateCombatParty("SPARK_BARON_PATROL", 4, cxt.location, true)
            cxt:ReassignCastMember("baron", barons[1])
            cxt:ReassignCastMember("worker", worker)
            
            cxt:Dialog("DIALOG_INTRO")

            local allies = {TheGame:GetGameState():AddSkinnedAgent( "RISE_REBEL") }
            
            cxt:TalkTo("handler")
            cxt:Opt("OPT_CONFRONT_BARON")
                :Dialog("DIALOG_CONFRONT_BARON")
                :Fn(function() 

                    cxt:TalkTo("baron")
                    cxt:RunLoop(function() 

                        cxt:Opt("OPT_PAY_OFF")
                            :DeliverMoney(100)
                            :Dialog("DIALOG_PAY_OFF")
                            :ReceiveOpinion("paid_bribe", nil, cxt:GetCastMember("worker"))                            
                            :ReceiveOpinion("saved_worker_from_barons", nil, cxt:GetCastMember("handler") )
                            :ReceiveOpinion("saved_from_barons", nil, cxt:GetCastMember("worker"))
                            :CompleteQuest()
                            :DoneConvo()

                        cxt:Opt("OPT_CONVINCE_LET_GO")
                            :Dialog("DIALOG_CONVINCE_LET_GO")
                            :Negotiation{
                                flags = cxt:GetCastMember("baron"):GetRelationship() > RELATIONSHIP.NEUTRAL and NEGOTIATION_FLAGS.INTIMIDATION,
                                helpers = {allies[1]},
                                suppressed = {cxt.quest:GetCastAgent("handler") },
                            }
                                :OnSuccess()
                                    :Dialog("DIALOG_CONVINCE_LET_GO_SUCCESS")
                                    :ReceiveOpinion("saved_worker_from_barons", nil, cxt:GetCastMember("handler") )
                                    :ReceiveOpinion("saved_from_barons", nil, cxt:GetCastMember("worker"))
                                    :CompleteQuest()
                                    :DoneConvo()

                                
                                :OnFailure()
                                    :Dialog("DIALOG_CONVINCE_LET_GO_FAILURE")

                        cxt:Opt("OPT_FIGHT")
                            :Dialog("DIALOG_FIGHT")
                            :Battle{
                                flags = BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                                no_oppo_limit = true,
                                allies = allies,
                                noncombatants = {cxt.quest:GetCastMember("handler")},
                            }
                                :OnWin()
                                    :Fn(function() 
                                        cxt.enc.scratch.worker_dead = cxt:GetCastMember("worker"):IsDead()
                                        if not cxt.enc.scratch.worker_dead then
                                            cxt:GetCastMember("worker"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("saved_from_barons"))
                                            cxt:GetCastMember("handler"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("saved_worker_from_barons"))
                                        end

                                        cxt:Dialog("DIALOG_FIGHT_WON")
                                        cxt.quest:Complete()
                                        StateGraphUtil.AddEndOption(cxt)
                                    end)

                        cxt:Opt("OPT_BACK_DOWN")
                            :Dialog("DIALOG_BACK_DOWN")
                            :ReceiveOpinion("did_not_help_worker")
                            :CompleteQuest()
                            :DoneConvo()

                    end)

                end)
            
            cxt:Opt("OPT_KEEP_WALKING")
                :Dialog("DIALOG_KEEP_WALKING")
                :ReceiveOpinion("did_not_help_worker")
                :CompleteQuest()
                :DoneConvo()
        end)

    :State("STATE_BARON_VERSION")
        :Loc{
            DIALOG_INTRO = [[
                * You're deep in the bog when you hear the sounds of struggle ahead of you. 
                * Fellemo's Barons hang back while you two creep forward to investigate and discover a few workers heaving salvage through the wet earth.
                player:
                    !left
                handler:
                    !right
                    !angry
                    $miscHushedUrgent
                    Thieves! Stealing valuable relics from beneath our very feet.
                    It won't stand. Rook, be a good fellow and deal with them quickly, yes?
            ]],
            QUESTION_WHY_NOT_YOU = "Ask why {agent} doesn't handle it",
            DIALOG_WHY_NOT_YOU = [[
                player:
                    Me? Why not you, or one of your Barons?
                    $neutralDubious
                    You never saw fit to grant me that kind of authority, you might recall.
                agent:
                    $miscHushedUrgent
                    This is a covert operation, and you're the special agent prized for his tactical skill.
                    I'm the commanding officer. And therefore, I command. 
                    !point
                    Go deal with it. 
            ]],
            OPT_CONFRONT_WORKERS = "Confront the workers",
            DIALOG_CONFRONT_WORKERS = [[
                player:
                    !salute
                    Aye aye. 
                handler:
                    !exit
                * You step out of the shrub line, careful to draw attention away from the lurking Barons.
                worker:
                    !right
                    !tired
                player:
                    $miscMocking
                    My, my, my. This is quite the haul.
                worker:
                    !placate
                    It's ours, grifter. We risked our necks to make this claim. 
                    Now we're gonna sell the profits and buy out our contracts.
            ]],

            OPT_CONVINCE_WORK = "Convince {worker} to deliver it to the Barons",
            DIALOG_CONVINCE_WORK = [[
                player:
                    !thought
                    $neutralThoughtful
                    My understanding is that you're required, per your contract, to report all finds to the Barons, no?

            ]],
            DIALOG_CONVINCE_WORK_SUCCESS = [[
                {liked?
                    player:
                        I say this out of concern for your well-being. 
                        !permit
                        If you're found in breach of contract, the Barons can bury you in even greater debt.
                        Whereas if you deliver the materials you'll be eligible for a sizeable bonus.
                    worker:
                        !scared 
                        $scaredTakenAback
                        B-but... we'll be lucky if the bonus covers half our debts. 
                    player:
                        I know, friend. But that's still a large chunk, and you'll be able to cover the rest.
                        $miscPersuasive
                        The risks are just too great otherwise.
                    worker:
                        $neutralResigned
                        I suppose you're right.
                        !neutral 
                        We'll deliver this to the Barons. It's better than getting buried in demerits.
                        !exit
                    * The workers continue to haul their salvage through the bog, this time with much less enthusiasm.
                }
                {not liked?
                    player:
                        !dubious
                        Besides, isn't there a <i>reward</i> for informants?
                        $neutralDubious
                        How well can you truly trust your compatriots? Would any of you turn each other in to avoid retribution?
                    worker:
                        !cagey
                        I, uh...
                        $neutralTakenAback
                        I'd never. And these guys would never!
                        ...
                        But maybe we'd all just better turn this in to the Barons. 
                        Since, uh, we wouldn't wanna be found in breach of contract.
                        !exit
                    * The workers continue to haul their salvage through the bog, this time with much less enthusiasm.
                }
                
                handler:
                    !right
                    $neutralResigned
                    I suppose that's your idea of "dealing with it", is it?
                    !disappoint
                    Now I see why Kalandra was inclined to trust you.
                player:
                    And why is that?
                handler:
                    You've a soft hand, Rook. I must say it surprises me. 
                    !eureka
                    I remember you being much stricter. But I did have you on a bit of a pedestal, didn't I?
                    Perhaps with age I've grown wiser. Or you've grown softer.
                player:
                    $neutralDirect
                    Or perhaps I just know when a subtle touch is required.
                    Shall we continue on? Your treasure awaits. 
                handler:
                    That it does. 
            ]],
            DIALOG_CONVINCE_WORK_FAILURE = [[
                player:
                        You wouldn't want to be in breach of contract, would you?
                    worker:
                        !angry_shrug
                        $angryReluctant
                        We don't want to be in contract at all. 
                        And if we gotta drag this salvage all the way to the Jakes to buy our way out, we will. Gladly.
            ]],

            OPT_FIGHT = "Attack the workers",
            DIALOG_FIGHT = [[
                {liked?
                    player:
                        I'm sorry, but I'm forced to take disciplinary action.
                        This is, I believe, for your own good.
                        !fight
                    worker:
                        !surprise 
                        $neutralTakenAback
                        Y-you for real? 
                        ...
                        !angry
                        Fine. I got too much on the line to go easy on you. 
                        !fight
                }
                {not liked?
                    player:
                        Let's just cut straight to the disciplinary action then, shall we?
                        !fight
                    worker:
                        !angry
                        $angryPatienceLost
                        Buddy, you got another thing coming if you think we're just gonna go down easy.
                        We've risked way too much for that.
                        !fight
                }
            ]],
            DIALOG_FIGHT_WON = [[
                {killed_all_workers?
                    * The corpses gather around the relic they were hauling through the bog. 
                    handler:
                        !right
                        Hrm. 
                        $neutralResigned
                        I suppose at least this relic will make up the lost profits from their labor.
                        Good work, Rook. 
                        I'll have my Barons mark this location so we can double back for this salvage later.
                    player:
                        The real treasure awaits, yes?
                    handler:
                        Indeed it does. 
                }

                {not killed_all_workers and killed_any_workers?
                    {dead?
                        * You pause long enough for the surviving worker to disappear into the bog, no doubt fleeing back to their work camps.
                    }
                    worker:
                        !exit
                    handler:
                        !right
                        Hrm. 
                        I suppose at least this relic will make up the lost profits from their labor.
                        Good work, Rook. 
                        I'll have my Barons mark this location so we can double back for this salvage later.
                    player:
                        The real treasure awaits, yes?
                    handler:
                        Indeed it does. 
                } 
                {not killed_any_workers?
                    player:
                        There. Next time, be sure to play by the rules.
                        And the worst you'll suffer is some demerits.
                    worker:
                        !injured
                        Sure. <i>That's</i> the worst we'll suffer.
                        $angryCruel
                        Have fun stooging for the Barons, or whatever it is you're doing all the way out here.
                        Pig.
                    * The workers abandon their salvage to limp through the foliage, leaning against one another for support.
                    * Clearly they aren't cowed by your savagery—not as long as they're together.
                        !exit
                    handler:
                        !right
                        Hrm.
                    player:
                        You sound disatisified.
                    handler:
                        Oh, the resolution was good enough. The workers are demoralized but not decommissioned, and this salvage looks fairly profitable.
                        But they were a little too cunning. I worry what they might tell the others, back at camp.
                        Perhaps it'd set the Rise on the hunt.
                    player:
                        Then we better keep moving, lest we linger long enough to leave our scent.
                }
            ]],

            OPT_BACK_DOWN = "Back down",
            DIALOG_BACK_DOWN = [[
                player:
                    !placate
                    Fair enough. Enjoy your good fortune—I hope it works in your favor.
                    !exit
                worker:
                    !exit
                * The workers eye you suspiciously but hurry along with their salvage in tow.
                player:
                    !left
                handler:
                    !right
                    !angry
                    $angryDefault
                    Blast it, Rook, that was a poor show. 
                    What in the deep do you think you're playing at?
                player:
                    $miscPersuasive
                    Those workers were too shrewd. If I'd forced the issue they might've gotten suspicious.
                handler:
                    Then you could have killed them, of course. That salvage looked valuable enough to make up for the lost labor.
                    Ah well. Too late to dwell on your mistakes now.
                    Let's press on. The true treasure awaits.
                    !exit
                * You lead the party around the workers, and deeper into the bog.

            ]],
            
            OPT_KEEP_WALKING = "Keep walking",
            DIALOG_KEEP_WALKING = [[
                player:
                    !dubious
                    $neutralDubious
                    Perhaps you've been out of the field too long, old friend.
                    This is merely a distraction. That relic will look like sodden refuse in comparison to the real bounty that awaits us.
                handler:
                    !angry_shrug
                    And I suppose you're not capable of securing both?
                    Fine. Prioritize as you see fit. 
                    But the Captain I used to know would seize the stars and established a perimeter around the horizon. 
                    !exit
                * You do your best to ignore his disappointment, and lead the party further into the bog. 
            ]],
            OPT_PAY_OFF = "Bribe the workers",
            DIALOG_PAY_OFF = [[
                player:
                    This relic might make a pretty penny for the Jakes, but they'll know you're desperate and price you accordingly.
                    !dubious
                    And you'll have to carry it all the way there, through dangerous bog.
                    Allow me to buy the relic off you. 
                    $miscAlluring
                    The profits might not be as strong, but the convenience can't be knocked.
                    !give
                worker:
                    !take
                    $neutralTakenAback
                    You serious? I mean...
                    I won't lie, this thing was hard enough to dig outta the ground. 
                    Carting it across the bog was gonna be a risk. 
                    !agree 
                    $miscRelieved
                    You have a deal, grifter. Enjoy your haul, however you see fit. 
                    !exit
                * The workers amble off into the bush, already discussing the drinks they plan to down when they're back at the camp.
                handler:
                    !right
                    We discover thieving workers and you decide to reward them?
                player:
                    $neutralResigned
                    You have the relic, which is surely worth more than they could've dreamed.
                    And they'll be hungover and piss poor again by this time tomorrow, ready to crawl back into submission, as you like.
                handler:
                    $angryReluctant
                    I suppose. But I would've liked to have seen some punishment.
                player:
                    Plenty of time for that when you're properly armed and dangerous, Fellemo.
                * You continue on your way through the deep bog. 
            ]]
            
        }
        :Fn(function(cxt) 
            cxt.encounter:DoLocationTransition( cxt.quest:SpawnTempLocation("MY_LOCATION") )
            cxt:Dialog("DIALOG_INTRO")
            
            
            local workers = CreateCombatParty({"HEAVY_LABORER", "LABORER", "LABORER", "HEAVY_LABORER"}, 4, cxt.location, true)
            cxt:ReassignCastMember("worker", workers[1])

            cxt:TalkTo("handler")
            local allies = { TheGame:GetGameState():AddSkinnedAgent( "SPARK_BARON_GOON") }
            cxt:RunLoop(function()  
                cxt:Question("QUESTION_WHY_NOT_YOU", "DIALOG_WHY_NOT_YOU")
                cxt:Opt("OPT_CONFRONT_WORKERS")
                    :Dialog("DIALOG_CONFRONT_WORKERS")
                    :Fn(function() 

                        cxt:TalkTo("worker")
                        cxt:RunLoop(function() 


                            cxt:Opt("OPT_PAY_OFF")
                                :DeliverMoney(100)
                                :Dialog("DIALOG_PAY_OFF")
                                :ReceiveOpinion("paid_bribe", nil, cxt:GetCastMember("worker"))
                                :CompleteQuest()
                                :DoneConvo()

                            cxt:Opt("OPT_CONVINCE_WORK")
                                :Dialog("DIALOG_CONVINCE_WORK")
                                :Negotiation{
                                    flags = cxt:GetCastMember("worker"):GetRelationship() > RELATIONSHIP.NEUTRAL and NEGOTIATION_FLAGS.INTIMIDATION,
                                    suppressed = { cxt:GetCastMember("handler") },
                                    helpers = {allies[1]},
                                }
                                    :OnSuccess()
                                        :Dialog("DIALOG_CONVINCE_WORK_SUCCESS")
                                        :ReceiveOpinion("got_all_workers_to_work", nil, cxt:GetCastMember("handler") )
                                        :CompleteQuest()
                                        :DoneConvo()

                                    
                                    :OnFailure()
                                        :Dialog("DIALOG_CONVINCE_WORK_FAILURE")

                            cxt:Opt("OPT_FIGHT")
                                :Dialog("DIALOG_FIGHT")
                                :Battle{
                                    flags = BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                                    allies = allies,
                                    no_oppo_limit = true,
                                    noncombatants = {cxt.quest:GetCastMember("handler")},
                                }
                                    :OnWin()
                                        :Fn(function() 
                                            local survivors = ConvoUtil.CollectSurvivors(workers)
                                            cxt.enc.scratch.killed_all_workers = #survivors == 0
                                            cxt.enc.scratch.killed_any_workers = #survivors < #workers
                                            cxt:Dialog("DIALOG_FIGHT_WON")
                                            if not cxt.enc.scratch.killed_any_workers then
                                                cxt:GetCastMember("handler"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("got_all_workers_to_work"))
                                            end
                                            cxt.quest:Complete()
                                            StateGraphUtil.AddEndOption(cxt)
                                        end)

                            cxt:Opt("OPT_BACK_DOWN")
                                :Dialog("DIALOG_BACK_DOWN")
                                :ReceiveOpinion("did_not_get_workers_to_work")
                                :CompleteQuest()
                                :DoneConvo()

                        end)

                    end)
                
                cxt:Opt("OPT_KEEP_WALKING")
                    :Dialog("DIALOG_KEEP_WALKING")
                    :ReceiveOpinion("did_not_get_workers_to_work")
                    :CompleteQuest()
                    :DoneConvo()
            end)
        end)


QDEF:AddDebugOption("working_with", {"RISE", "SPARK_BARONS"})

QDEF:AddTest(TestCreator()
                :SetReps(10)
                :SetParams{working_with= "RISE"})

QDEF:AddTest(TestCreator()
                :SetReps(10)
                :SetParams{working_with= "SPARK_BARONS"})
