local FOOD_COST = 40
local HEAL_AMOUNT = 20


CARD_SHOP_DEFS.SPARK_BARON_SHOP_ITEMS = function(stock)
    AddShopItems(stock, 3, {"targeting_core", "robone", "energy_siphon", "robodex", "nano_lattice", "oshnu_glue", "skimmer"})
end

CARD_SHOP_DEFS.RISE_SHOP_ITEMS = function(stock)
    AddShopItems(stock, 3, {"shock_box", "coil_grenade", "oshnu_bile", "portable_bunker", "replicator", "jump_cables", "fuel_canister"})
end

local function working_for_faction(q, faction)
    for k,quest in TheGame:GetGameState():ActiveQuests() do
        if quest:GetQuestDef().faction == faction then
            return true
        end

        if quest:GetQuestDef().WorkingForFaction then
            if quest:GetQuestDef().WorkingForFaction(quest, faction) then
                return true
            end
        end
    end
end


local QDEF = QuestDef.Define
{
    title = "Provisions",
    --icon = engine.asset.Texture("icons/quests/rook_story_living_at_bar.tex"),
    qtype = QTYPE.STORY,
    desc = "You can buy factional items here when you're on the clock.",

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("baron_vendor"), location = quest:GetCastMember("baron_hq"), role = CHARACTER_ROLES.VENDOR} )
        table.insert (t, { agent = quest:GetCastMember("rise_vendor"), location = quest:GetCastMember("rise_hq"), role = CHARACTER_ROLES.VENDOR} )
    end,

    plot_armour_fn = function(quest, agent)
        return agent:IsCastInQuest(quest)
    end,

    SetVendorEnabled = function(quest, faction, enabled)
        
        local obj_id = faction == "SPARK_BARONS" and "baron_buy" or "rise_buy"
        if enabled then
            if not quest:IsActive(obj_id) then
                quest:Activate(obj_id)
            end
        else
            if quest:IsActive(obj_id) then
                quest:Cancel(obj_id)
            end
        end
    end,

    EnhanceVendor = function(quest, faction)
        if faction == "SPARK_BARONS" then
            quest.param.baron_enhanced = true
        end
        
        if faction == "RISE" then
            quest.param.rise_enhanced = true
        end

    end,

}

:AddLocationCast{
    cast_id = "baron_hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_BARON_HQ"))
    end,
}

:AddLocationCast{
    cast_id = "rise_hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_RISE_HQ"))
    end,
}

:AddCast{
    cast_id = "baron_vendor",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():AddSkinnedAgent( "SPARK_BARON_TASKMASTER" ))
    end,
    on_assign = function(quest, agent)
        agent:GainAspect("cardshop")
        agent:GetAspect("cardshop"):SetStockID("SPARK_BARON_SHOP_ITEMS")
    end,
}

:AddCast{
    cast_id = "rise_vendor",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():AddSkinnedAgent( "RISE_PAMPHLETEER" ))
    end,

    on_assign = function(quest, agent)
        agent:GainAspect("cardshop")
        agent:GetAspect("cardshop"):SetStockID("RISE_SHOP_ITEMS")
    end,
}

:AddObjective{
    id = "baron_buy",
    title = "Buy Spark Baron equipment",
    icon = engine.asset.Texture("icons/quests/rook_story_work_for_fellemo.tex"),
    --desc = "{hebbel} sells refreshment and healing.",
    --state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    low_priority = true,
    mark = function(quest, t, in_location)
        if working_for_faction(quest, "SPARK_BARONS") then
            table.insert(t, quest:GetCastMember("baron_hq"))        
        end
    end,
    --mark = {"baron_hq"},
}

:AddObjective{
    id = "rise_buy",
    title = "Buy Rise equipment",
    icon = engine.asset.Texture("icons/quests/rook_story_work_for_kalandra.tex"),
    --desc = "{hebbel} sells refreshment and healing.",
    --state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    low_priority = true,
    mark = function(quest, t, in_location)
        if working_for_faction(quest, "RISE") then
            table.insert(t, quest:GetCastMember("rise_hq"))        
        end
    end,
}

QDEF:AddConvo("baron_buy", "baron_vendor")                
    :Loc{
        DIALOG_INTRO = [[
            agent:
                You looking for some equipment?
        ]],
        OPT_SHOP = "Buy Spark Baron equipment",

        
        OPT_BUY_AN_AUTODOG = "Buy an autodog",
        DIALOG_BUY_AUTODOG = [[
            player:
                I need an autodog.
            agent:  
                These things don't grow on trees, you know.
            autodog:
                !right
            * The autodog activates and starts following you.
            autodog:
                WOOF
            agent:
                !right
        ]],
    }
    :AttractState("asd")
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
        end)
    :Hub(function(cxt)

            cxt:Opt("OPT_SHOP")
                :IsHubOption(true)
                :PreIcon( global_images.shop, UICOLOURS.MONEY_LIGHT )
                :Fn(function()
                    cxt.enc:WaitOnLine()
                    local screen = Screen.CardShopScreen( cxt:GetAgent(), function() cxt.enc:ResumeEncounter() end )
                    TheGame:FE():InsertScreen( screen )
                    cxt.enc:YieldEncounter()
                end)
            
            if cxt.quest.param.baron_enhanced then
                local can_own, reason = cxt.caravan:CanOwnPet()
                cxt:Opt("OPT_BUY_AN_AUTODOG")
                    :DeliverMoney(250, { is_shop = true })
                    :ReqCondition( can_own, reason )
                    :Fn(function() 
                        local autodog = cxt.quest:CreateSkinnedAgent( "AUTODOG" )
                        autodog:MoveToLocation( cxt.location )
                        TheGame:GetGameState():AddAgent( autodog )
                        autodog:Recruit(PARTY_MEMBER_TYPE.CREW)
                        cxt:ReassignCastMember("autodog", autodog)
                    end)
                    :Dialog("DIALOG_BUY_AUTODOG")
            end

    end)

QDEF:AddConvo(nil, "baron_vendor")                    
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !crossed
                I only sell to members.
        ]],
    }
    :AttractState("asd", function(cxt) return not cxt.quest:IsActive('baron_buy') end)
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
        end)


QDEF:AddConvo("rise_buy", "rise_vendor")                
    :Loc{
        DIALOG_INTRO = [[
            agent:
                You need stuff? I have stuff, but you have to pay!
        ]],
        OPT_SHOP = "Buy Rise equipment",
     
        OPT_BUY_A_COBBLEDOG = "Buy a Cobbledog",
        DIALOG_BUY_COBBLEDOG = [[
            player:
                I need a cobbledog.
            agent:  
                I can throw another one together.
            autodog:
                !right
            * The cobbledog shudders pathetically.
            autodog:
                WO- O. O...OF
            agent:
                !right
        ]],
    }
    :AttractState("asd")
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
        end)
    :Hub(function(cxt)

            cxt:Opt("OPT_SHOP")
                :IsHubOption(true)
                :PreIcon( global_images.shop, UICOLOURS.MONEY_LIGHT )
                :Fn(function()
                    cxt.enc:WaitOnLine()
                    local screen = Screen.CardShopScreen( cxt:GetAgent(), function() cxt.enc:ResumeEncounter() end )
                    TheGame:FE():InsertScreen( screen )
                    cxt.enc:YieldEncounter()
                end)
            if cxt.quest.param.rise_enhanced then
                local can_own, reason = cxt.caravan:CanOwnPet()
                cxt:Opt("OPT_BUY_A_COBBLEDOG")
                    :DeliverMoney(150, { is_shop = true })
                    :ReqCondition( can_own, reason )
                    :Fn(function() 
                        local autodog = cxt.quest:CreateSkinnedAgent( "RISE_AUTODOG" )
                        autodog:MoveToLocation( cxt.location )
                        TheGame:GetGameState():AddAgent( autodog )
                        autodog:Recruit(PARTY_MEMBER_TYPE.CREW)
                        cxt:ReassignCastMember("autodog", autodog)
                    end)
                    :Dialog("DIALOG_BUY_COBBLEDOG")                
            end
    end)

QDEF:AddConvo(nil, "rise_vendor")                    
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !crossed
                Rise only, buddy.
        ]],
    }
    :AttractState("asd", function(cxt) return not cxt.quest:IsActive('rise_buy') end)
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
        end)
