
local QDEF = QuestDef.Define{
    qtype = QTYPE.STORY,
    title = "Infiltrate the Barons",
    icon = engine.asset.Texture("icons/quests/rook_story_infiltrate_barons.tex"),
    collect_agent_locations = function(quest, t)
        if quest:IsActive("capture_prisoner") and not quest:GetCastMember("prisoner"):IsDead() then
            table.insert (t, { agent = quest:GetCastMember("prisoner"), location = quest:GetCastMember("prisoner"):GetBrain():GetHome() , role = CHARACTER_ROLES.VISITOR} )
        end
        table.insert (t, { agent = quest:GetCastMember("second"), location = quest:GetCastMember("hq"), role = CHARACTER_ROLES.CONTACT} )

    end,

    is_in_hiding = function(quest, agent)
        if agent == quest:GetCastMember("other_handler") then
            return true
        end
    end,

    on_destroy = function(quest) 
        if quest:GetCastMember("prisoner"):IsInPlayerParty() then
            quest:GetCastMember("prisoner"):Dismiss()
        end

    end,



    process_fighter = function(quest, fighter)
        if quest:IsActive('capture_prisoner') then
            if fighter.agent == quest:GetCastAgent("prisoner") then
                fighter:AddCondition("WANTED_ALIVE")
            end
        end
    end,


    avoids = {
        "prisoner", "kalandra",
        "prisoner", "hq",
    }
}

:AddCastByAlias{
    cast_id = "kalandra",
    alias = "KALANDRA",
    no_validation = true,
}

:AddCast{
    cast_id = "prisoner",

    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "FOREMAN" )
        table.insert( t, agent )
        agent:GetBrain():GenerateHome()
    end,
}

:AddCastByAlias{
    cast_id = "other_handler",
    alias = "SPARK_CONTACT",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "second",
    alias = "SPARK_SECOND",
    no_validation = true,
}

:AddLocationCast{
    cast_id = "hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_BARON_HQ"))
    end,
}


:AddObjective{
    id = "capture_prisoner",
    title = "Capture {prisoner}",
    desc = "Find {prisoner} at {prisoner.hisher} home and take {prisoner.himher} prisoner.",
    state = QSTATUS.ACTIVE,
    mark = {"prisoner"},
    on_complete = function(quest)
        quest:Activate("go_to_hq")
    end,
    negotiation_targets = {"prisoner"},
    combat_targets = {"prisoner"},

}

:AddObjective{
    id = "go_to_hq",
    title = "Go to {hq}",
    desc = "You need to convince the Barons that you are a grifter who wants to work for them.",
    mark = {"hq"}
}

:AddObjective{
    id = "do_job",
    events = 
    {
        quests_changed = function(quest, event_quest)
            if quest.param.loyalty_job == event_quest and not event_quest:IsActive() then
                quest:Complete("do_job")

            end
        end
    },
    
    on_activate = function(quest)
        TheGame:GetGameState():AddPendingEvent()
    end,

    on_complete = function(quest) 
        quest:Activate("job_followup")
    end,
}

:AddObjective{
    id = "job_followup",
    title = "Talk to {second} about the job",
    mark = {"second"}
}



------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("capture_prisoner", "prisoner")
    :Loc{
        OPT_DEMAND_SURRENDER = "Demand {prisoner} come with you",
        DIALOG_INTRO = [[
            player: 
                $angryHostile
                You're coming with me!
            agent:
                $scaredTakenAback
                You're not a Baron, are you? You're just working with them, right?
                $scaredFearful
                We can work out a deal that'll help us both!
                $miscPersuasive
                Just hear me out!
        ]],
        DIALOG_REINTRO = [[
            agent:
                $angryHostile
                I'm not going with you!
        ]],
        OPT_FIGHT = "Beat up {prisoner}",
        DIALOG_FIGHT = [[
            player:
                $angryHostile
                Ok, I'll fight you.
        ]],
        DIALOG_FIGHT_LIVED = [[
            agent:
                !injured
                $scaredPowerless
                I give up!
        ]],
        DIALOG_FIGHT_DIED = [[
            * You weren't supposed to kill {agent}!
            * You take {agent.hisher} papers and proof that you killed {agent.himher}.
            * Ew.
        ]],

        OPT_NEGOTIATE = "Convince {prisoner} to give up",
        DIALOG_NEGOTIATE = [[
            player:
                $angryHostile
                Don't make this hard!
        ]],
        DIALOG_NEGOTIATE_WON =[[
            agent:
                $neutralResigned
                OK I'll go with you!
        ]],
        DIALOG_NEGOTIATE_LOST =[[
            agent:
                $angrySeething
                No way dude. Make me.
        ]],
        
        OPT_TAKE_PRISONER = "Take {agent} prisoner",
        DIALOG_TAKE_PRISONER = [[
            * You take {agent} prisoner.
        ]],
        DIALOG_ASK_ABOUT = [[
            player:
                $neutralDirect
                Just a few quick questions, first.
            agent:
                $miscMocking
                Are you kidding? 
        ]],
        OPT_ASK_ABOUT_DEAL = "Ask about {agent}'s proposal",
        
        DIALOG_ASK_ABOUT_DEAL = [[
            player:
                I'm listening...
            agent:
                $scaredDefault
                They think I'm Rise, but I'm not.
                $scaredDefault
                I just don't like working!
                $scaredDefault
                I've got a way out, though. I can leave!
                $scaredDefault
                I'll give you my ID and then disappear forever.
                $scaredDefault
                All you have to do is convince them that I'm dead. How does that sound?
        ]],
        OPT_ACCEPT_DEAL = "Accept the offer",
        DIALOG_REJECT_DEAL = [[
            player:
                $neutralDirect
                No, I don't want to cross the Barons. I hear they're pretty ruthless.
            agent:  
                $neutralResigned
                Tell me about it.
        ]],
        DIALOG_ACCEPT_DEAL = [[
            player:
                $neutralThoughtful
                That sounds easier than fighting you.
            agent:
                $happyThanks
                Great! Here's my ID. Just tell them that the rest of me got disintegrated.
                !exit
            * You suspect it will be more difficult than that.
        ]],
        OPT_REJECT_DEAL = "Reject the offer",
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_DEMAND_SURRENDER")
            :ReceiveOpinion( OPINION.TRIED_TAKE_CAPTIVE) 
            :Dialog("DIALOG_INTRO")
            :LoopingFn(function(cxt) 
                if not cxt:FirstLoop() then
                    cxt:Dialog("DIALOG_REINTRO")
                end

                cxt:Opt("OPT_ASK_ABOUT_DEAL")
                    :Dialog("DIALOG_ASK_ABOUT_DEAL")
                    :Fn(function(cxt) 
                        cxt:Opt("OPT_ACCEPT_DEAL")
                            :Dialog("DIALOG_ACCEPT_DEAL")
                            :ReceiveOpinion(OPINION.HELPED_THEM_DISAPPEAR)
                            :Fn(function() 
                                cxt:GetAgent():Retire()
                                cxt.quest:Complete("capture_prisoner")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end)

                        cxt:Opt("OPT_REJECT_DEAL")
                            :Dialog("DIALOG_REJECT_DEAL")
                    end)


                --[[cxt:Opt("OPT_ASK_ABOUT")
                    :Dialog("DIALOG_ASK_ABOUT")
                    :IsHubOption(true)
                    :GoTo("STATE_QUESTIONS")
                --]]

                cxt:Opt("OPT_NEGOTIATE")
                    :Dialog("DIALOG_NEGOTIATE")
                    :Negotiation{
                    
                        on_success = function(cxt)  
                            cxt:Dialog("DIALOG_NEGOTIATE_WON")
                            cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.CAPTIVE, cxt.quest )
                            cxt.quest:Complete("capture_prisoner")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,

                        on_fail = function(cxt) 
                            cxt:Dialog("DIALOG_NEGOTIATE_LOST")
                        end,

                    }
                cxt:Opt("OPT_FIGHT")
                    :Dialog("DIALOG_FIGHT")
                    :Battle{
                        on_win = function(cxt) 
                            if cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_FIGHT_DIED")
                            else
                                cxt:Dialog("DIALOG_FIGHT_LIVED")
                                cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.CAPTIVE, cxt.quest )
                            end
                            cxt.quest:Complete("capture_prisoner")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }
                StateGraphUtil.AddBackButton(cxt)
            end)
    end)
    :AskAboutHub("STATE_QUESTIONS", 
    {
        "Ask about {kalandra}",
        [[
            player: 
                $scaredPowerless
                {kalandra.HeShe}'s mean!
        ]],
    })



QDEF:AddConvo("go_to_hq")
    :ConfrontState("state", function(cxt) return cxt.location == cxt.quest:GetCastMember("hq") end)
        :Loc{
            DIALOG_INTRO_WITH_PRISONER = [[
                * At the gate to the Baron enclave, you leave a message for Fellemo, hoping your name still carries enough weight to get you through the door.
                * Sure enough, you're escorted inside—but it's not Fellemo who greets you.
                player:
                    !left
                second:
                    !right
                    $angryHostile
                    You have five minutes to state your case, grifter. Then we trigger the attack protocols.
                player:
                    Let's not be hasty, now. Fellemo should be expecting me.
                    I have here a member of the Rise—a wanton miscreant he'll no doubt want to question.
            ]],
            DIALOG_INTRO_WITH_PROOF = [[
                * At the gate to the Baron enclave, you leave a message for Fellemo, hoping your name still carries enough weight to get you through the door.
                * Sure enough, you're escorted inside—but it's not Fellemo who greets you.
                player:
                    !left
                second:
                    !right
                    $angryHostile
                    You have five minutes, grifter. Then we trigger the attack protocols.
                player:
                    !interest
                    $miscPersuasive
                    Fellemo wants me here, I assure you. I have sensitive information I know he'll want to see.
                second:
                    And two pistols. 
                    Witnesses have seen you talking with leading members of the Rise. 
                    We know what's happening in the Bread Fields—it's only a matter of time before the Rise start to deal in assassins.
            ]],
            OPT_CONVINCE = "Convince {agent} you're telling the truth",
            DIALOG_CONVINCE = [[
                player:
                    $miscPersuasive
                    Easy, there, friend. I can assure you that I have only Fellemo's best interests at heart.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    I came here of my own free will to offer assistance. Will you at least hear me out?
                agent:
                    $neutralDirect
                    I suppose so. 
                    Remember, though, that you're surrounded by armed Barons. 
                    And it will be my decision whether you ultimately may speak with Fellemo.
                player:
                    $happyCocky
                    Then I look forward to convincing you of my good intentions. 
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    If I wanted Fellemo dead, believe me, he would be dead.
                * If you were hoping to convince {second} of your good intentions, you failed. {second.HeShe} snarls openly at you.
                agent:
                    $neutralDubious
                    Kill the spy. We can use his corpse for field tests. 
                player:
                    $neutralTakenAback
                    Hah! I'd like to see you—oh. 
                    !exit
                * The Spark Baron levels a weapon at you that makes the air in front of it crackle like a storm. 
                * You're just about to ask what it does... when you find out. 
            ]],
            OPT_ACCEPT_YOUR_FATE = "You are dead",
        }
    
    :Fn(function(cxt) 
        cxt.quest:Complete("go_to_hq")
        cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("second"))
        if cxt.quest:GetCastMember("prisoner"):IsInPlayerParty() then
            cxt:Dialog("DIALOG_INTRO_WITH_PRISONER")
            cxt.quest:GetCastMember("prisoner"):Dismiss():Retire()
            cxt:GoTo("STATE_GET_JOB")
        else
            cxt:Dialog("DIALOG_INTRO_WITH_PROOF")
            cxt:Opt("OPT_CONVINCE")
                :Negotiation{
                    on_success = function() 
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt:GoTo("STATE_GET_JOB")
                    end,
                    on_fail = function() 
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                        cxt:Opt("OPT_ACCEPT_YOUR_FATE")
                            :Fn(function() 
                                cxt.enc:PlayerDeath(cxt:GetAgent())
                            end)
                    end,
                }
            end
        end)
    :State("STATE_GET_JOB")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    I simply wish to help my old friend Fellemo with his Rise problem. 
                    As he should know, I'm an experienced agent. I can infiltrate, assimilate, and sabotage with precision.
                agent:
                    And this is supposed to make me trust you? 
                    Hm. I <i>do</i> have one way to test your loyalties, perhaps. 
                player:
                    I'm all ears. 
            ]],
            DIALOG_PICKED = [[
                agent:
                    $happyThanks
                    Don't bother returning if you can't complete the job. 
                    I won't give you a second chance. 
                player:
                    I understand you completely. 
            ]],
            OPT_ASK_ABOUT = "Ask some questions first...",
            DIALOG_ASK_ABOUT = [[
                player:
                    Just a moment, my good lieutenant. I have some questions. 
            ]]
        }
        :Fn(function(cxt) 
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("second"))
            cxt:Dialog("DIALOG_INTRO")

            local main_quest = cxt.quest.param.parent_quest.param.main_quest
            
            local quest_options = main_quest:GetQuestDef().SpawnFactionJobs(main_quest, 1, cxt:GetAgent():GetFactionID(), {"nasty"}, {"late_game"}, {giver = cxt.quest:GetCastMember("second")})
            
            TheGame:GetGameState():GetMainQuest():GetQuestDef().PresentFactionJobChoice(cxt, quest_options, true,
                    function(cxt, job) 
                        cxt.quest.param.loyalty_job = job
                        cxt.quest:Activate("do_job")
                        cxt:Dialog("DIALOG_PICKED")
                        main_quest:GetQuestDef().SetVendorEnabled(main_quest, "SPARK_BARONS", true) 
                        StateGraphUtil.AddEndOption(cxt)
                    end)
        end)


--this will trigger mid-conversation after you turn in the loyalty quest
QDEF:AddConvo("job_followup", "second")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            
            
            DIALOG_INTRO = [[
                agent:
                    $neutralResigned
                    Hm. I guess you did the job as well as could be expected. 
                    Very well. Fellemo will see you.
                * {other_handler} enters the room, and {second} steps to the side with a head bowed in deference, eyes still cunningly locked on yours.
                other_handler:
                    !right
                    So. 
                    !point
                    $angryDefault
                    I don't see you in years, Rook, and you show up at my door with enemy ties, expecting a warm welcome?
                player:
                    I'm ever the optimist, I suppose. It's good to see you, old friend.
                other_handler:
                    Can it. 
                    Everyone out. I need a moment alone with this slippery eel.
                * The room empties, though {second} mouths threats at you as {second.heshe} goes.
            ]],

            DIALOG_INTRO_2 = [[
                other_handler:
                    Well?
                * Within seconds, {other_handler} melts into the relaxed stance you remember. 
                * He reaches under a desk for two polished glasses, and passes you a drink poured from his hip flask. 
                other_handler:
                    !give
                player:
                    !take
                    It went just as you suspected. {kalandra} has asked me to infiltrate the Barons.
                    Apparently you took something she wants back?
                other_handler:
                    !drink
                    $happyCocky
                    Ah yes—I knew that would be the ice pick in her temple. 
                    And she doesn't suspect your true allegiances?
                player:
                    Not as far as I can tell. What next?
                other_handler:
                    You'll do as she anticipates—come see me every day, and check in with her as she requires.
                    Once you have her trust, I'll tell you the next step. 
                    You'll have to forgive the ruse, of course, as I continue to distrust you in front of my underlings.
                player:
                    Naturally.
                other_handler: 
                    All the same, it's good to see you again. You've found a place to lay your head?
                player:
                    Some bar called 'The Last Stand'. You've heard of it?
                other_handler: 
                    It'll do. 
                    Now sod off, grifter. We'll have more work for you tomorrow.
                * With one last swig of his drink, {other_handler} leaves you alone to your scheming. 
                other_handler:
                    !exit
                *** {other_handler} and you are in cahoots! Or are you!?
            ]],
            DIALOG_INTRO_JOB_FAILED = [[
                agent:
                    $angrySeething
                    I told you not to come back here a failure. 
                    Seems the only solution is to put you out of your misery. 
                * The buzz of {second}'s weapon makes the air sizzle, but {other_handler} enters the room before you can see it used.
                other_handler:
                    !right
                    !point
                    $angryPatienceLost
                    {second}, enough! 
                    My time with Rook has earned him at least one more chance. I'll handle this.
                    $angryHostile
                    Everyone out. 
                * The room empties, though {second} leaves unwillingly and mouths threats at you as {second.heshe} goes.

            ]],
            DIALOG_INTRO_JOB_FAILED_2 = [[
                * Alone with {other_handler}, you can feel the tension in the room, as if {second}'s weapon were still charging nearby.
                other_handler:
                    Well?
                player:
                    It went just as you suspected. {kalandra} has asked me to infiltrate the Barons.
                other_handler:
                    And how were you going to do that, exactly, when you failed at the task that my lieutenant gave you?
                player:
                    What would you have me do? I'm supposed to be a double agent. 
                    Would you like me to jeopardize what little trust {kalandra} has in me already?
                other_handler:
                    I'd like you to be effective. 
                    But you haven't ruined this yet. 
                player:
                    So, what next?
                other_handler:
                    You'll play it just as she suspects—come see me every day, and check in with her as she requires.
                    Once you have her trust, I'll tell you the next step. 
                    !exit
                * {other_handler} leaves you alone to your scheming. 
                *** {other_handler} and you are in cahoots! Or are you!?
            ]]

        }
        :Fn(function(cxt) 
            
            cxt:Dialog(cxt.quest.param.loyalty_job:IsComplete() and "DIALOG_INTRO" or "DIALOG_INTRO_JOB_FAILED")


            local to_leave = {}
            for k,v in ipairs(cxt.location.agents) do
                if v ~= cxt.quest:GetCastMember("other_handler") and v ~= cxt.player and not v:IsInPlayerParty() then
                    table.insert(to_leave, v)
                end
            end
            for k,v in ipairs(to_leave) do
                v:MoveToLimbo()
            end

            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("other_handler"))
            cxt:Dialog(cxt.quest.param.loyalty_job:IsComplete() and "DIALOG_INTRO_2" or "DIALOG_INTRO_JOB_FAILED_2")
            
            cxt.quest:GetCastMember("other_handler"):MoveToLimbo()
            
            --this requires a double click to exit, because of the way this convo started. I'm... not sure why yet.
            StateGraphUtil.AddEndOption(cxt)
            cxt.quest:Complete()

        end)
