
local QDEF = QuestDef.Define{
    qtype = QTYPE.STORY,
    title = "Infiltrate the Rise",
    icon = engine.asset.Texture("icons/quests/rook_story_infiltrate_rise.tex"),
    is_in_hiding = function(quest, agent)
        if agent == quest:GetCastMember("kalandra") then
            return true
        end
    end,
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("second"), location = quest:GetCastMember("hq"), role = CHARACTER_ROLES.CONTACT} )
    end,

}

:AddCastByAlias{
    cast_id = "kalandra",
    alias = "KALANDRA",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "spark_handler",
    alias = "SPARK_CONTACT",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "second",
    alias = "RISE_SECOND",
    no_validation = true,
}


:AddLocationCast{
    cast_id = "hq",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_RISE_HQ"))
    end,
}

:AddLocationDefs{
    AMBUSH_SITE = {
        name = "Rise Ambush Site",
        map_tags = {"bog"},
        plax = "EXT_Bog_Outcrop_01",
        indoors = false,
    },
}

:AddLocationCast{
    cast_id = "ambush_site",
    cast_fn = function(quest, t)
        table.insert(t, quest:SpawnTempLocation("AMBUSH_SITE"))
    end,
}


:AddObjective{
    id = "find_the_rise",
    title = "Make contact with the Rise",
    desc = "Rise patrols are often spotted in the bog at night. It shouldn't be too hard to find one.",
    state = QSTATUS.ACTIVE,
    mark = {"ambush_site"},
    on_complete = function(quest) 
        quest:Activate("give_the_intel")
    end,
}

:AddObjective{
    id = "give_the_intel",
    title = "Take the intel to Rise HQ",
    desc = "Deliver {spark_handler}'s intel to {kalandra} to gain {kalandra.hisher} trust.",
    mark = {"hq"},
    
    on_complete = function(quest) 
        local main_quest = quest.param.parent_quest.param.main_quest
        main_quest:GetQuestDef().SetVendorEnabled(main_quest, "RISE", true) 
    end,

}


:AddObjective{
    id = "do_job",
    events = 
    {
        quests_changed = function(quest, event_quest)
            if quest.param.loyalty_job == event_quest and not event_quest:IsActive() then
                quest:Complete("do_job")
            end
        end
    },
    
    on_activate = function(quest)
        TheGame:GetGameState():AddPendingEvent()
    end,

    on_complete = function(quest) 
        quest:Activate("job_followup")
    end,
}

:AddObjective{
    id = "job_followup",
    title = "Talk to {second} about the job",
    mark = {"second"}
}

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("find_the_rise")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("ambush_site") end)
        :Loc{
            DIALOG_INTRO = [[
                * You head to the location {spark_handler} mentioned—but as far as you can tell, it's nothing but a desolate patch of swamp.
                * You kick up some moss, looking for signs of buried fires, and eventually you see a small mark scratched into the lichen of a large stone.
                agent:
                    !right
                    $angryGreeting
                    You <i>lost</i>, grifter?
                player:
                    !left
                * {agent.HisHer} eyes flame when {agent.heshe} sees you in the light. 
                agent:
                    !surprised
                    $angrySeething
                    <i>You're</i> the one contracting for the Barons. Fair warning: you should leave while you still can.
                    $angryHostile
                    There's going to be a reckoning soon, and nobody will care if you get caught in the crossfire. 
                player:
                    You're a member of the Rise, then?
                    I don't suppose you know where I'd find {kalandra} this time of night, do you? 
                agent:
                    !threaten
                    I'll kill you before I let you get to her, bootlicker. Be sure of that.
            ]],
            OPT_CONVINCE_TURN = "Convince {agent} you want to switch sides",
            DIALOG_CONVINCE_TURN = [[
                player:
                    $happyCocky
                    Let's take a breath, shall we? 
                    Yes, I'm on the payroll. But my loyalties lie with {kalandra}.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    I'm risking my life to be here, with valuable intel, I might add. 
                agent:
                    !happy
                    $happyDefault
                    W-well. If you're on the level...
                    That's a windfall, make no mistake. 
                    This could really make a difference, pops—I hope you know that.
                * {agent} gestures for your map, and makes a quick mark on it.
                agent:
                    $miscPersuasive
                    You won't find {kalandra} there—she's too well protected for that. 
                    But that's how you can find someone who can vet you.
                player:
                    Why, thank you for your trust. 
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    But if you don't want my valuable information, fine—I'll just return to the Spark Barons, shall I?
                agent:
                    !fight
                    $neutralWhatever
                    Do that. But let me scratch a warning in your hide, first. 
                    We won't be bullied by a buster like you.
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    $neutralResigned
                    Remember that I <i>tried</i> to play nice. 
            ]],
            OPT_FIGHT = "Beat up {agent} to establish your dominance",
            
            DIALOG_FIGHT = [[
                player:
                    !fight
                    $angryHostile
                    I see you have a problem with authority. Perhaps I should instill some awe in you, hm?
            ]],
            DIALOG_FOUGHT_RISE_ALIVE = [[
                agent:
                    !injured
                    $scaredPowerless
                    S-stop! Don't hurt me, please!
                player:
                    I never wished to, you foolish oaf. 
                    As I was saying, I'm simply looking for {kalandra}. 
                    I have a valuable opportunity for her, despite your best efforts to squander it.
                agent:
                    I don't know where she is! She keeps her agenda secret. 
                    But I know where you can find someone who can pass it along. 
                player:
                    $angryThanks
                    That will do. Thank you kindly—I hope you have better manners when next we meet. 
                agent:
                    !exit
            ]],
            DIALOG_FOUGHT_RISE_DEAD = [[
                * {agent} is dead, but you frisk {agent.hisher} corpse and find a crude map labeled "FRPERG ONFR". 
                player:
                    $neutralThanks
                    Amateurs.
            ]]

        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                local rise = CreateCombatParty(function() return {"RISE_REBEL", "RISE_AUTOMECH"} end ,1, cxt.location)
                cxt.enc:SetPrimaryCast(rise[1])
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_CONVINCE_TURN")
                :Dialog("DIALOG_CONVINCE_TURN")
                :Negotiation{
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest:Complete("find_the_rise")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function(cxt) 
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                        cxt:Opt("OPT_DEFEND")
                            :Dialog("DIALOG_DEFEND")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                on_win = function(cxt) 
                                    cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_FOUGHT_RISE_DEAD" or "DIALOG_FOUGHT_RISE_ALIVE")
                                    cxt.quest:Complete("find_the_rise")
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end,
                            }
                        
                    end,
                }

            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                    on_win = function(cxt) 
                        cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_FOUGHT_RISE_DEAD" or "DIALOG_FOUGHT_RISE_ALIVE")
                        cxt.quest:Complete("find_the_rise")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }

        end)


QDEF:AddConvo("give_the_intel")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("hq") end)
        :Loc{
            DIALOG_INTRO = [[
                * You find a ramshackle structure hidden in the bog right where you were expecting. Thinking it best to put a bold foot forward, you barge in.
                second:
                    !right
                    $angryGreeting
                    Who the Hesh is that?
                player:
                    !left
                    !greeting
                    $miscPersuasive
                    I'm looking for {kalandra}. Heard of {kalandra.himher}?
                second:
                    !crossed
                    $angryHostile
                    Drop the act, grandad. I'm two seconds away from blasting you back to wherever you came from.
                    Explain yourself. Now.
                player:
                    !permit
                    $miscPersuasive
                    I work with the Spark Barons, but I find them... distasteful. 
                    I'm here to offer my services.
                second:
                    $angryReluctant
                    And you expect us to trust you?
                player:
                    !happy
                    $happyCocky
                    Of course not. That's why I've come bearing gifts.
                * You hand over the intel. {second}'s eyes grow wide when {second.heshe} reads it.
                second:
                    !interest
                    $neutralTakenAback
                    I'm going to need some time to go over this.
                    ...
                    How about this: I send you out on a job while I check this info. 
                    !point
                    When you come back—if you come back—maybe then you can talk to the boss. 
                
            ]],
            DIALOG_PICKED = [[
                second:
                    $angryThanks
                    Now go do it!
            ]],
            DIALOG_ASK_ABOUT = [[
                player:
                    $neutralDirect
                    Before I do your dirty work, I have questions.
            ]]
        }
        :Fn(function(cxt) 
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("second"))
            cxt:Dialog("DIALOG_INTRO")
            
            local main_quest = cxt.quest.param.parent_quest.param.main_quest
            local quest_options = main_quest:GetQuestDef().SpawnFactionJobs(main_quest, 1, cxt:GetAgent():GetFactionID(), {"nasty"}, {"late_game"}, {giver = cxt.quest:GetCastMember("second")})
            TheGame:GetGameState():GetMainQuest():GetQuestDef().PresentFactionJobChoice(cxt, quest_options, true, 
                    function(cxt, job) 
                        cxt.quest.param.loyalty_job = job
                        cxt.quest:Complete("give_the_intel")
                        cxt.quest:Activate("do_job")
                        cxt:Dialog("DIALOG_PICKED")
                        
                        main_quest:GetQuestDef().SetVendorEnabled(main_quest, "RISE", true) 
                        StateGraphUtil.AddEndOption(cxt)
                    end)
        end)


QDEF:AddConvo("job_followup", "second")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !dubious
                    $neutralDubious
                    It's done, huh? Well...
                * Before {agent} can voice any more disapproval, {kalandra} enters the room. 
                * It's clear she's the leader by the way Flekfis rolls back his shoulders, trying to undo years of back-breaking labor.
                * Her eyes lock heavily on yours, and she steps forward, fearless. 
                kalandra:
                    !right
                    !point
                    $angryGreeting
                    Enough with the third degree. I want to meet the turncoat.
                    Give us the room, everyone. I'll take it from here.
                * The room empties, and you're left alone with an eager {kalandra}, who turns to you with a glint in her eye.
            ]],

            DIALOG_INTRO_2 = [[
                kalandra:
                    !interest
                    $neutralDirect
                    Nice to meet you face-to-face, {player}. I have to say, one day in the bog and you've already made some progress.
                    I have it on good authority that {spark_handler} is very pleased to have finally gotten you on-site.
                player:
                    $neutralResigned
                    Fellemo often gives mixed signals. But yes, he has me on the payroll. 
                    !eureka
                    Old friends and comrades, reunited by the threat of a labor union. 
                    I went to him first, as your letter proposed. Excellent cipher, by the way. 
                kalandra:
                    !happy
                    I take naturally to this cloak and dagger stuff. 
                    Right, well—I won't pretend I trust you completely. I hired you to <i>spy</i>, after all. 
                player:
                    !dubious
                    $neutralDubious
                    Ah yes. I'm to look for... what was it, again?
                kalandra:
                    !placate
                    That'll come later. For now, just stay on {spark_handler}'s good side. 
                    Fellemo thinks you're spying on me—we can't give him any indication it's the other way around.
                    $neutralResigned
                    But I want you to check in with me every day. For what I'm paying you, you should make time for both.
                player:
                    !salute
                    $neutralDirect
                    As you like.
                kalandra:
                    !exit
                * {kalandra} leaves you to your ruminations. It's been a long day, and it seems like tomorrow will be even longer.
                *** {kalandra} was your true client all along. Or so it might seem.
            ]],
            DIALOG_INTRO_JOB_FAILED = [[
                agent:
                    !angry_shrug
                    $angrySeething
                    Surprise, surprise. You weren't able to deliver, huh?
                    So either you're a liar, or you're incompetent. 
                    !angry_accuse
                    Either way, we don't got time to waste with bootlickers like you.
                * Before you can respond to {agent}'s wagging finger by breaking it, {kalandra} enters the room.
                kalandra:
                    !right
                    !point
                    $angryHostile
                    I'll handle this.
                    Everyone out. Except for the grifter.
                * The room empties, and you are alone with {kalandra}.

            ]],
            DIALOG_INTRO_JOB_FAILED_2 = [[
                agent:
                    !angry
                    $angryPatienceLost
                    You really aren't making this easy for me, {player}.
                player:
                    !crossed
                    Ah, {kalandra}, I presume. I have to say, your letters were most impressively encrypted.
                agent:
                    Can it with the flattery. I have it on good authority you're a slippery old eel.
                    Now I know I hired you to spy on {spark_handler}, and I know that means you can't betray the Barons too openly...
                    !overther
                    But unless you want anyone catching wise to who you're <i>really</i> working for, you need to at least pretend to be competent.
                player:
                    !interest
                    Ah, but you see, Fellemo expects the same of me. He's hired me to spy on <i>you</i>.
                agent:
                    !agree
                    I figured he would. Now, at least, you can come and go through both our camps. 
                    Check in with me tomorrow. I'll have something else for you.
                    And <i>hopefully</i> this time, you won't screw it up.
                    !exit
                * {kalandra} leaves you to your ruminations. It's been a long day, and it seems like tomorrow will be even longer.
                *** {kalandra} was your true client all along. Or so it might seem.
            ]]

        }
        :Fn(function(cxt) 
            
            cxt:Dialog(cxt.quest.param.loyalty_job:IsComplete() and "DIALOG_INTRO" or "DIALOG_INTRO_JOB_FAILED")

            local to_leave = {}
            for k,v in ipairs(cxt.location.agents) do
                if v ~= cxt.quest:GetCastMember("kalandra") and v ~= cxt.player and not v:IsInPlayerParty() then
                    table.insert(to_leave, v)
                end
            end
            for k,v in ipairs(to_leave) do
                v:MoveToLimbo()
            end
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("kalandra"))

            cxt:Dialog(cxt.quest.param.loyalty_job:IsComplete() and "DIALOG_INTRO_2" or "DIALOG_INTRO_JOB_FAILED_2")
            
            cxt.quest:GetCastMember("kalandra"):MoveToLimbo()
            
            --this requires a double click to exit, because of the way this convo started. I'm... not sure why yet.
            StateGraphUtil.AddEndOption(cxt)
            cxt.quest:Complete()

        end)
