local QDEF = QuestDef.Define
{
    title = "Mettle Vendor",
    icon = engine.asset.Texture("icons/quests/grit_shop.tex"),
    qtype = QTYPE.STORY,
    home = true,
    
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("stranger"), location = quest:GetCastMember("shop"), role = CHARACTER_ROLES.VENDOR } )
    end,

    on_init = function(quest)
        quest.param.started_unlocked = TheGame:GetGameProfile():HasMettleUnlocked(TheGame:GetGameState():GetPlayerAgent():GetContentID())
        if TheGame:GetGameState():CanDoMettle() then
            if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.NIGHT then
                quest:Activate("visit")
            end
        end
    end,

    events = 
    {
        phase_change = function(quest)
            
            if TheGame:GetGameState():CanDoMettle() then
                if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY then
                    quest.param.bought_today = false
                    quest:Cancel("visit")
                else
                    quest:Activate("visit")
                end
            end
        end,
        
        quests_changed = function(quest, event_quest) 
            if not quest.param.started_unlocked and not quest:IsActive("visit") and TheGame:GetGameState():CanDoMettle() then
                quest:Activate("visit")
            end
        end,
    },}

:AddExistingLocation( "shop", "GB_METTLE_SHOP" )

:AddCastByAlias{
    cast_id = "stranger",
    alias = "DODGY_SCAVENGER",
}

:AddObjective{
    state = QSTATUS.INACTIVE,
    id = "visit",
    title = "Shop for Mettle upgrades",
    desc = "You have <b>{1} Mettle</> to spend on upgrades, and <b>{2} Prestige Points</> to spend on Flourishes.",
    desc_fn = function(quest, str) 
        return loc.format(str, TheGame:GetGameProfile():GetMettlePoints(TheGame:GetGameState():GetPlayerAgent():GetContentID()), TheGame:GetGameProfile():GetPrestigePoints())
    end,
    hide_in_overlay = true,
    low_priority = true,
    mark = {"shop"},
}

QDEF:AddConvo("visit")
    :ConfrontState("FOO", function(cxt) return cxt.location == cxt.quest:GetCastMember("shop") end)
        :ProgressionLoc{
            DIALOG_INTRO = {
                [[
                    {first_time?
                        player:
                            !left
                        agent:
                            !right
                        player:
                            You again?!
                            What kind of scam are you running here, friend?
                        agent:
                            !placate
                            No scam, I swear!
                            I'm just here to help you be your best.
                            With the power of mettle!
                    }
                    {not first_time?
                        player:
                            !left
                        agent:
                            !right
                            !greeting
                            Ready to test your mettle again?
                    }
                ]],
                [[
                    {first_time?
                        agent:
                            !right
                        player:
                            !left
                            I'm having the most intense deja-vu right now.
                            !dubious
                            Do we know each other?
                        agent:
                            Sometimes.
                            Sometimes not.
                            Depends on how far you get each time.
                        player:
                            Each time I what?
                        agent:
                            You probably don't want to know the details yet.
                    }
                    {not first_time?
                        player:
                            !left
                        agent:
                            !right
                            Welcome back, grifter.
                    }
                ]],
                [[
                    {first_time?
                        agent:
                            !right
                        player:
                            !left
                        agent:
                            !happy
                            Can I help you?
                        player:
                            I... don't know. Can you?
                        agent:
                            Yes. Over time. 
                            How ever long it takes.
                    }
                    {not first_time?
                            player:
                                !left
                            agent:
                                !right
                                Can't stay away, can you?
                    }
                ]],
                [[
                    {first_time?
                        agent:
                            !right
                        player:
                            !left
                            I've come in search of Mettle.
                        agent:
                            I know you have.
                    }
                    {not first_time?
                            player:
                                !left
                            agent:
                                !right
                                Let's make you stronger!
                    }
                ]],
            }
        }

        :Loc{


            OPT_FLOURISHES = "Unlock Flourishes",
            DIALOG_FLOURISHES = [[
                player:
                    I want to learn something new.
            ]],
            DIALOG_POST_FLOURISHES = [[
                agent:
                    It's important to end things well.
            ]],

            OPT_SHOP = "Spend Mettle",
            DIALOG_SHOP = [[
                player:
                    Let's see what you have.
            ]],
            DIALOG_POST_SHOP = [[
                agent:
                    You have a cautious eye, grifter. What lessons has life taught you?
            ]],
            OPT_LEAVE = "Stop Shopping",
            DIALOG_LEAVE = [[
                player:
                    I'll be back.
                    !exit
                agent:
                    I know you will!
            ]],
            DIALOG_ASK_ABOUT = [[
                player:
                    I'm curious about something.
            ]],

            OPT_BUY_METTLE = "Purchase {1} Mettle {1*Point|Points}",
            DIALOG_BUY_METTLE = [[
                agent:
                    I see you're not above throwing your coin around to get ahead.
                    I like that!
            ]],
            TT_BUY_METTLE_ALREADY = "You already bought mettle today! Try again tomorrow.",
        }   
        :Fn(function(cxt) 
            local stranger = cxt.quest:GetCastAgent("stranger")
            stranger:MoveToLocation(cxt.quest:GetCastMember("shop"))
            cxt.enc:SetPrimaryCast(stranger)
            cxt:ProgressDialog("ROOK_METTLE_VISITS", "DIALOG_INTRO")

            local day = TheGame:GetGameState():GetActProgress()
            


            local METTLE_POINTS = {1, 3, 9, 18}
            local COST = {200, 400, 600, 800}
            local num_points = METTLE_POINTS[math.min(day, #METTLE_POINTS)]
            local mettle_cost = COST[math.min(day, #COST)]

            cxt:RunLoop(function() 
                cxt:Opt("OPT_SHOP")
                    :Dialog("DIALOG_SHOP")
                    :PreIcon(global_images.mettle)
                    :IsHubOption(true)
                    :Fn(function()
                        cxt.enc:WaitOnLine()
                        local screen = Screen.MettleScreen( cxt.player, function() cxt.enc:ResumeEncounter() end )
                        TheGame:FE():PushScreen( screen )
                        cxt.enc:YieldEncounter()
                        cxt:Dialog("DIALOG_POST_SHOP")
                    end)


                cxt:Opt("OPT_BUY_METTLE", num_points)
                    :Dialog("DIALOG_BUY_METTLE")
                    :DeliverMoney(mettle_cost, {is_shop = true})
                    :ReqCondition(not cxt.quest.param.bought_today, "TT_BUY_METTLE_ALREADY")
                    :PreIcon(global_images.mettle)
                    :Fn(function() 
                        AUDIO:PlayEvent("event:/ui/mettle_shop/buy_mettlepoints")
                        TheGame:GetGameProfile():AddMettlePoints( TheGame:GetGameState():GetPlayerAgent():GetContentID(), num_points, "BUY_METTLE_SHOP") 
                        cxt.quest.param.bought_today = true
                        DoAutoSave()
                    end)

                cxt:Opt("OPT_FLOURISHES")
                    :Dialog("DIALOG_FLOURISHES")
                    :PreIcon(global_images.flourish)
                    :IsHubOption(true)
                    :Fn(function()
                        cxt.enc:WaitOnLine()
                        local screen = Screen.FlourishScreen( cxt.player, function() cxt.enc:ResumeEncounter() end )
                        TheGame:FE():PushScreen( screen )
                        cxt.enc:YieldEncounter()
                        cxt:Dialog("DIALOG_POST_FLOURISHES")
                    end)

                cxt:Opt("OPT_ASK_ABOUT")
                    :Dialog("DIALOG_ASK_ABOUT")
                    :IsHubOption(true)
                    :GoTo("STATE_QUESTIONS")

                cxt:Opt("OPT_LEAVE")
                    :MakeUnder()
                    :Dialog("DIALOG_LEAVE")
                    :PreIcon(global_images.close)
                    :Travel()
            end)
        end)


    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask about mettle",
        [[
            player:
                So what is this mettle, anyway?
            agent:
                It's the measure of how much you've applied yourself.
                In this life, and all previous ones.
            player:
                !dubious
                What do you mean, previous ones?
            agent:
                !laugh
                Ha. You don't know, do you?
                Well, I'm not going to spoil it for you grifter.
                Maybe some day you'll figure it out.
        ]],
        "Ask how to get mettle",
        [[
            player:
                So how do I get more mettle?
            agent:
                Mettle will find you, if you are worthy.
                If you see an opportunity - seize it!
            player:
                !thought
                Hmmm. Can I just buy it from you?
            agent:
                Of course!
        ]],
        "Ask about flourishes",
        [[
            player:
                What is a flourish?
            agent:
                It's an advanced technique. 
                Only usable under certain conditions.
                You can't learn them in the usual way, but you never forget them once you know them.
            player:
                Odd.
            agent:
                Indeed!
        ]],
    })