local AuctionUtil = class( "AuctionUtil" )
AuctionUtil.can_reload = true

function AuctionUtil.HandleGraft(cxt, lots, graft, after_fn)
    cxt:RunLoop(function() 
        cxt:Opt("OPT_TAKE_GRAFT", graft)
            :PreIcon( global_images.graft )
            :Fn( function( cxt, who )
                local installed = ConvoUtil.GiveGraft(cxt, graft)
                if installed then
                    cxt:Pop()
                    after_fn(cxt)
                end
        end )

        cxt:Opt("OPT_SMASH_GRAFT", graft)
            :Dialog("DIALOG_SMASH_GRAFT")
            :Fn(function() 
                cxt:Pop()
                after_fn(cxt)
            end)
    end)
end

function AuctionUtil.HandleItem(cxt, lots, item_id, after_fn)
    cxt:Opt("OPT_TAKE_ITEM", item_id)
        :GainCards{item_id}
        :Fn(function() 
            after_fn(cxt)
        end)

    cxt:Opt("OPT_SMASH_ITEM", item_id)
        :Dialog("DIALOG_SMASH_ITEM", item_id)
        :Fn(function() 
            after_fn(cxt)
        end)
    
end

function AuctionUtil.PresentLots(cxt, lots, after_fn, extra_fn)
    cxt:RunLoop(function(cxt) 
        for k,v in ipairs(lots.grafts) do
            local idx = k
            local graft = GraftInstance(v)
            
            cxt:Opt("OPT_TAKE_GRAFT", graft)
                :PreIcon( global_images.graft )
                :Fn( function( cxt, who )
                    local installed = ConvoUtil.GiveGraft(cxt, graft)
                    if installed then
                        table.remove(lots.grafts, idx)
                        after_fn(cxt)
                    end
                end)
        end

        for k,v in ipairs(lots.items) do
            local idx = k
            cxt:Opt("OPT_TAKE_ITEM", v)
                :GainCards{v}
                :Fn(function() 
                    table.remove(lots.items, idx)
                    after_fn(cxt)
                end)
        end

        if extra_fn then
            extra_fn(cxt)
        end

    end)
end


function AuctionUtil.PresentLotsInBoxes(cxt, lots, after_fn)
        for k,v in ipairs(lots.grafts) do
            local idx = k
            local graft_item = GraftInstance(v)
            cxt:Opt("OPT_GRAFT_BOX", graft_item:GetName())
                :Dialog("DIALOG_OPEN_BOX")
                :Fn(function() 
                    table.remove(lots.grafts, idx)
                    AuctionUtil.HandleGraft(cxt, lots, GraftInstance(v), after_fn )
                end)
        end

        for k,v in ipairs(lots.items) do
            local idx = k
            local carddef = Content.GetBattleCard( v )
            cxt:Opt("OPT_ITEM_BOX", carddef:GetLocalizedName())
                :Dialog("DIALOG_OPEN_BOX")
                :Fn(function() 
                    table.remove(lots.items, idx)
                    AuctionUtil.HandleItem(cxt, lots, v, after_fn )
                end)
        end

end
