local AUCTION_DATA = require "content/grafts/kashio_boss_fight_defs"
require "content/quests/story/Sal/Finale/auction_util"



local QDEF = QuestDef.Define{
        title = "Kashio Test",
        disabled= true,
        qtype = QTYPE.EXPERIMENT,
        icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
        rank = 5,
    }
    :AddLocationDefs{

        AUCTION_HOUSE =
        {
            name = "Auction House",
            desc = "The room where the auction is taking place.",
            show_agents = true,
            indoors = true,
            no_exit = true,
            plax = "EXT_NadanFight",
        }
    }

    :AddCastByAlias{
        cast_id = "auctioneer",
        alias = "HESH_AUCTIONEER",
    }

    :AddCastByAlias{
        cast_id = "kashio",
        alias = "KASHIO",
    }

    :AddObjective{
        id = "test",
        title = "Fight Kashio, and provide feedback!",
        state = QSTATUS.ACTIVE,
    }


QDEF:AddConvo("test")
    :ConfrontState("INTRO")
        :Fn(function(cxt)
            cxt.quest:SetRank(5)
            TheGame:GetGameState():SetDifficulty(5)
            cxt.quest.param.seed = 0
            cxt.encounter:DoLocationTransition(  cxt.quest:SpawnTempLocation("AUCTION_HOUSE") )
            cxt:GoTo("STATE_INTRO") 
        end)

    :State("STATE_INTRO")
        :Loc{
            DIALOG_INTRO = [[* This is a balancing test for the Kashio encounter. The audio and visuals are not complete, and there are no story elements.]],
            OPT_SET_SEED = "Enter seed",
            OPT_USE_RANDOM_SEED = "Use a random seed",
            DIALOG_BAD_SEED = [[That is not a valid seed (seeds are numbers)]],
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.enc:GetScreen():ClearHistory()
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_USE_RANDOM_SEED")
                :Fn(function() 
                    cxt.quest.param.seed = os.time()
                    cxt:GoTo("STATE_PRE_FIGHT")
                end)
            cxt:Opt("OPT_SET_SEED")
                :Fn(function(cxt) 
                    UIHelpers.EditString( 
                        "Enter a seed", "", "", 
                        function( val )
                            val = val and tonumber(val)
                            if val then
                                cxt.quest.param.seed = val
                            end
                            cxt.enc:ResumeEncounter(val)
                         end )
                    local seed = cxt.enc:YieldEncounter()
                    if seed then
                        cxt.quest.param.seed = seed
                        cxt:GoTo("STATE_PRE_FIGHT")
                    else
                        cxt:Dialog("DIALOG_BAD_SEED")
                    end
                end)

        end)

    :State("STATE_PRE_FIGHT")
        :Loc{
            DIALOG_INTRO = [[
                * Random Seed: {1}
            ]],

            OPT_TAKE_ITEM = "Take {1#card}",
            OPT_TAKE_GRAFT = "Take {1#graft}",
            
            OPT_SMASH_ITEM = "Smash it",
            OPT_SMASH_GRAFT = "Smash it",

            OPT_LEAVE_ITEM = "Leave it",
            OPT_LEAVE_GRAFT = "Leave it",


            DIALOG_LEAVE_GRAFT = [[
                * You decide to leave the graft where it is.
            ]],
            DIALOG_SMASH_GRAFT = [[
                * You smash the graft.
            ]],
            DIALOG_TAKE_GRAFT = [[
                * You take the graft.
            ]],

            DIALOG_LEAVE_ITEM = [[
                * You decide to leave the {1#card} where it is.
            ]],
            DIALOG_SMASH_ITEM = [[
                * You smash the {1#card}.
            ]],
            DIALOG_TAKE_ITEM = [[
                * The thought of using this on Kashio brings a smile to your face.
            ]],

            DIALOG_OPEN_BOX = [[
                player:
                    !take
            ]],

            DIALOG_PICK = [[
                * You now have the opportunity to pick some items and grafts. Kashio will get whatever you don't pick, and use it against you.
                * The first item will be picked from your story choices. Choose now:
            ]],

            OPT_GRAFT_BOX = "Pick Graft '{1}'",
            OPT_ITEM_BOX = "Pick Item '{1}'",
            OPT_CONTINUE = "",
            OPT_SKIP = "Skip the rest",
            DIALOG_DECK = [[ * Test Deck Selected: #{1}]],
            OPT_REROLL = "Try a new seed",

            DIALOG_SOCIAL = [[
                * <#BONUS>Social Boons: {1#graft_list}</>
                * <#PENALTY>Social Banes: {2#graft_list}</>
            ]]
        }
        :Fn(function(cxt)
            local DECKS = require "content/quests/story/Sal/Finale/kashio_fight_test_decks"
            math.randomseed(cxt.quest.param.seed)
            local lots = AUCTION_DATA.PickAuctionLots(2, 3)
            cxt.quest.param.lots = lots
            cxt.quest.param.presented_lots = deepcopy(lots) 
            local deck_idx = math.random(#DECKS)
            local deck = DECKS[deck_idx]
            TheGame:GetGameState():SetDecks(deck)
            local money_amt = 100 + math.random(60)*5
            cxt.caravan:SetMoney(money_amt)


            cxt:TalkTo("kashio")
            cxt:Dialog("DIALOG_INTRO", cxt.quest.param.seed)
            cxt:Dialog("DIALOG_DECK", deck_idx)
            
            local negotiation, battle, boons, banes = {}, {}, {}, {}
            for k,v in ipairs( cxt.player.graft_owner.grafts) do
                if v:GetType() == GRAFT_TYPE.SOCIAL then 
                    if v:GetDef().is_boon then
                        table.insert(boons, v)
                    elseif v:GetDef().is_bane then
                        table.insert(banes, v)
                    end
                elseif v:GetType() == GRAFT_TYPE.COMBAT then 
                    table.insert(battle, v)
                elseif v:GetType() == GRAFT_TYPE.NEGOTIATION then 
                    table.insert(negotiation, v)
                end

            end

            --table.sort(battle, function(a,b) return a:GetName() < b:GetName() end)
            --table.sort(negotiation, function(a,b) return a:GetName() < b:GetName() end)
            table.sort(boons, function(a,b) return a:GetName() < b:GetName() end)
            table.sort(banes, function(a,b) return a:GetName() < b:GetName() end)
            cxt:Dialog("DIALOG_SOCIAL", boons, banes)
            cxt:Dialog("DIALOG_PICK")
            local num_picked = 0 
            
            cxt:RunLoopingFn(function() 
                AuctionUtil.PresentLots(cxt, lots, 
                    function() 
                        num_picked = num_picked + 1
                        if num_picked >= 1 then
                            cxt:GoTo("STATE_PRE_AUCTION")
                        end
                    end,
                    function()  
                        cxt:Opt("OPT_SKIP")
                            :Fn(function() 
                                --cxt:GoTo("STATE_PRE_AUCTION")
                                cxt:GoTo("STATE_DO_FIGHT")
                            end)        
                        cxt:Opt("OPT_REROLL")
                            :Fn(function() 
                                cxt.enc:GetScreen():ClearHistory()
                                cxt.quest.param.seed = os.time() + math.random(10000)
                                cxt:GoTo("STATE_PRE_FIGHT") 
                            end)        
                    end)                        
                    
            end)
            
        end)

        :State("STATE_DO_FIGHT")
            :Loc{
                DIALOG_INTRO = [[
                    player:
                        !left
                        !fight
                    kashio:
                        !right
                        !fight
                    * Kashio takes the remaining items!
                ]],
                OPT_FIGHT = "Fight!",
                DIALOG_DONE_FIGHT = [[
                    left:
                        !happy
                    right:
                        !greeting
                    * Thanks for testing! Now would be a great time to submit some feedback about that fight! (Your seed was: {1})
                ]],
                OPT_RETRY = "Retry this exact fight",
                OPT_AGAIN = "Start over",
                DIALOG_AGAIN = [[
                    player:
                        !exit
                    kashio:
                        !exit
                ]],
                OPT_FEEDBACK = "Tell us how that went!"
            }
            :Fn(function(cxt) 
                local handler_card = table.arraypick{"nadan_signal", "oolo_signal"}
                cxt:TalkTo("kashio")
                cxt:Dialog("DIALOG_INTRO")
                cxt:Opt("OPT_FIGHT")
                    :Battle{
                        flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE,
                        IS_EXPERIMENT = true,
                        on_start_battle = function(battle) 
                            local fighter = battle:GetFighterForAgent(cxt.quest:GetCastMember("kashio"))
                            if fighter then
                                for k,v in ipairs(cxt.quest.param.lots.items) do
                                    local condition = AUCTION_DATA.CONDITION_LOOKUPS[v]
                                    if condition then
                                        fighter:AddCondition(condition)
                                    end
                                end
                                for k,v in ipairs(cxt.quest.param.lots.grafts) do
                                    local condition = AUCTION_DATA.CONDITION_LOOKUPS[v]
                                    if condition then
                                        fighter:AddCondition(condition)                                    
                                    end
                                end
                                fighter:ClearPreparedCards()
                                fighter:PrepareTurn()
                            end

                            if handler_card then
                                local card = Battle.Card( handler_card, battle.player_team:Primary() )
                                card.show_dealt = true
                                battle:DealCard( card, battle:GetDrawDeck() )
                            end
                        end,

                        on_experiment_done = function( cxt, battle )
                            local player_team = {}
                            local enemy_team = {}
                            for k,v in ipairs(battle:GetScenario().teams[TEAM.BLUE]) do
                                table.insert(player_team, v:GetContentID())
                            end

                            for k,v in ipairs(battle:GetScenario().teams[TEAM.RED]) do
                                table.insert(enemy_team, v:GetContentID())
                            end

                            local player_state = TheGame:GetGameState():GetPlayerState()
                            local kashio_items = {}
                            local player_items = {}
                            for i,item in ipairs(cxt.quest.param.presented_lots.items) do
                                if table.contains(cxt.quest.param.lots.items, item) then
                                    table.insert(kashio_items, item)
                                else
                                    table.insert(player_items, item)
                                end
                            end
                            for i,graft in ipairs(cxt.quest.param.presented_lots.grafts) do
                                if table.contains(cxt.quest.param.lots.grafts, graft) then
                                    table.insert(kashio_items, graft)
                                else
                                    table.insert(player_items, graft)
                                end
                            end
                            local json_t = {
                                PLAYER_DATA = player_state,
                                BATTLE_DATA = {
                                    TURNS = battle.turns,
                                    CONTENT_ID = battle:GetScenario().content_id,
                                    HEALTH_DELTA = battle:GetPlayerFighter():GetHealth() - battle:GetPlayerFighter():GetMaxHealth(), -- Player always start with full HP
                                    PLAYER_TEAM = player_team,
                                    ENEMY_TEAM = enemy_team,
                                },
                                EXPERIMENT_DATA = {
                                    REMAINING_ITEMS = kashio_items,
                                    CHOSEN_ITEMS = player_items,
                                    ENEMY_HEALTH_PERCENTAGE = battle:GetEnemyTeam():Primary():GetHealthPercent() * 100,
                                }
                            }
                            local evt_type = (battle:GetBattleResult() == BATTLE_RESULT.WON) and "KASHIO_EXPERIMENT_BATTLE_WON" or "KASHIO_EXPERIMENT_BATTLE_LOST"
                            SendMetricsData(evt_type, json_t)
                        end,
                    }
                    
                    
                    :Fn(function() 
                        cxt:Dialog("DIALOG_DONE_FIGHT", cxt.quest.param.seed)

                        cxt:RunLoopingFn(function() 
                            cxt:Opt("OPT_FEEDBACK")
                                :Fn(function() TheGame:StartFeedback() end)
                            cxt:Opt("OPT_AGAIN")
                                :Dialog("DIALOG_AGAIN")
                                :GoTo("STATE_INTRO")
                            cxt:Opt("OPT_RETRY")
                                :GoTo("STATE_DO_FIGHT")
                        end)
                    end)
            end)
    
    :State("STATE_PRE_AUCTION")
        :Loc{
            DIALOG_INTRO = [[
                * You now have the chance to perform a special negotiation to get a second item. OR you can skip it, and get an ambush modifier instead.
            ]],
            OPT_AUCTION = "Try the auction",
            OPT_DO_AUCTION = "Start the auction!",
            DIALOG_AUCTION = [[
                auctioneer:
                    !right
                    !happy
                    Who's going to get these fine items?
            ]],
            
            OPT_AMBUSH = "Ambush Kashio",
            DIALOG_FAIL_NEGOTIATION = [[
                * Oh dear, you didn't get anything. And now Kashio is extra mad at you!
            ]],
            DIALOG_SUCCESS = [[
                * You won something at the auction! Now to fight Kashio with it!
            ]],
            PST_FIGHT_WIN_ITEM = [[
                * You won {1#card}.
            ]],
            PST_FIGHT_WIN_GRAFT = [[
                * You won {1#graft}.
            ]],
            PST_FIGHT_LOSE = [[
                * You didn't win anything.
            ]],
            NEGOTIATION_REASON = [[Capture an auction item before the other bidders.]],


        }
        :Fn(function(cxt) 
            cxt.caravan:SetResolvePercent(1)
            cxt.quest.param.doing_ambush = false
            cxt:Dialog("DIALOG_INTRO")
            cxt:TalkTo("auctioneer")
            cxt:Opt("OPT_DO_AUCTION")
                :Dialog("DIALOG_AUCTION")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.NO_CORE_RESOLVE | NEGOTIATION_FLAGS.NO_IMPATIENCE,
                    reason_txt = cxt:GetLocString("NEGOTIATION_REASON"),
                    on_start_negotiation = function(minigame)
                        minigame.opponent_negotiator:FindCoreArgument():SetItems(cxt.quest.param.lots.grafts, cxt.quest.param.lots.items)
                    end,
                    on_success = function(cxt, minigame) 
                        local core = minigame.opponent_negotiator:FindCoreArgument()
                        if core.won_item then
                            table.arrayremove(cxt.quest.param.lots.items, core.won_item)
                            cxt:GainCards{core.won_item}
                        elseif core.won_graft then
                            table.arrayremove(cxt.quest.param.lots.grafts, core.won_graft)
                            ConvoUtil.GiveGraft(cxt, GraftInstance(core.won_graft))
                        end 
                        cxt:Dialog(core.won_item and "PST_FIGHT_WIN_ITEM" or "PST_FIGHT_WIN_GRAFT", core.won_item or core.won_graft)
                        cxt:GoTo("STATE_DO_FIGHT") 
                    end,

                    on_fail = function() 
                        cxt:Dialog("PST_FIGHT_LOSE")
                        cxt:GoTo("STATE_DO_FIGHT") 
                    end,
                }

            cxt:Opt("OPT_AMBUSH")
                :Fn(function() cxt.quest.param.doing_ambush = true end)
                :GoTo("STATE_DO_FIGHT")
        end)
