local AUCTION_DATA = require "content/grafts/kashio_boss_fight_defs"
local kashio_stinger = "event:/music/pre_kashio_cnv"

local QDEF = QuestDef.Define{
        title = "The Auction",
        qtype = QTYPE.STORY,
        icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
        rank = 5,

        on_init = function(quest) 
            quest.param.cross_quest_data = quest.param.cross_quest_data or {}
            quest.param.cross_quest_data.auction_lots = quest.param.cross_quest_data.auction_lots or AUCTION_DATA.PickAuctionLots(2, 3)
            if quest.param.requisition then
                quest.param.cross_quest_data.requisition = true
            end

            
            if quest.param.force_faction_alliance then
                Util.SetPlayerWorkingWithFaction(quest.param.force_faction_alliance)
            end

        end,
    }
    :AddLocationDefs{

        AUCTION_HOUSE =
        {
            name = "Auction House",
            desc = "The room where the auction is taking place.",
            show_agents = true,
            indoors = true,
            no_exit = true,
            plax = "INT_AuctionHouse_01",
        }
    }

    --one or both handlers might be dead at this point, so they are nonvalidated (we want to cast the corpses)
    :AddCastByAlias{
        cast_id = "oolo",
        alias = "MURDER_BAY_ADMIRALTY_CONTACT",
        no_validation = true,
        optional = true,
    }

    :AddCastByAlias{
        cast_id = "nadan",
        alias = "MURDER_BAY_BANDIT_CONTACT",
        no_validation = true,
        optional = true,
    }

    :AddCastByAlias{
        cast_id = "kashio",
        alias = "KASHIO",
        no_validation = true,
    }

    :AddCast{
        cast_id = "handler",
        when = QWHEN.MANUAL,
        optional = true,
    }

    
    :AddObjective{
        id = "pre_auction",
        title = "Confront {kashio}",
        state = QSTATUS.ACTIVE,
    }

    :AddCastByAlias{
        cast_id = "auctioneer",
        alias = "HESH_AUCTIONEER",
    }


QDEF:AddConvo("pre_auction")
    :ConfrontState("CONF")
        :Fn(function(cxt) 
            local location = cxt.quest:SpawnTempLocation("AUCTION_HOUSE")
            
            cxt:GetCastMember("auctioneer"):MoveToLocation(location)
            cxt:GetCastMember("auctioneer"):SetLocationRole(CHARACTER_ROLES.PROPRIETOR)
            cxt.quest.param.crowd = {}            
            local people = {
                "JAKES_SMUGGLER",
                "JAKES_SMUGGLER",
                "JAKES_SMUGGLER",
                "ADMIRALTY_PATROL_LEADER",
                "ADMIRALTY_PATROL_LEADER",
                "WEALTHY_MERCHANT",
                "WEALTHY_MERCHANT",
                "WEALTHY_MERCHANT",
                "LUMINARI",
                "LUMINARI",
                "LUMINARI",
                "PRIEST",
                "PRIEST",
            }

            
            if cxt.quest.param.cross_quest_data.friend_inside then
                cxt.quest.param.cross_quest_data.friend_inside:MoveToLocation(location)
            end



            for _, id in ipairs(people) do
                local agent = AgentUtil.GetFreeAgent(id)
                agent:MoveToLocation(location)
                table.insert( cxt.quest.param.crowd, agent ) 
            end
            cxt.encounter:DoLocationTransition( location  )
--]]
            cxt:GoTo(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "STATE_FRONT_DOOR" or "STATE_SNEAK_IN")
        end)
    
    :State("STATE_SNEAK_IN")
        :Loc{
            DIALOG_INTRO = [[
                * The auction is starting. You can participate and maybe gain an item, or hide in the shadows and hope to catch {kashio} by surprise.
            ]],
            OPT_HIDE_IN_THE_SHADOWS = "Stay hidden in the shadows",
            OPT_AUCTION = "Approach the auction",
            DIALOG_AUCTION = [[
                * You see an empty spot at the front of the auction, and take it.
            ]],



        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_HIDE_IN_THE_SHADOWS")
                :GoTo("STATE_DO_AMBUSH")

            cxt:Opt("OPT_AUCTION")
                :GoTo("STATE_DO_AUCTION")

        end)
    
    :State("STATE_FRONT_DOOR")
        :Loc{
            DIALOG_INTRO = [[
                * You pass through the door and are confronted by both the stink of incense and the inner guard.
                agent:
                    !right
                player:
                    !left
                    !disgust
                
                { disliked ? 
                    agent:
                        !crossed
                        $angryDefault
                        What are <i>you</> doing here?
                    player:
                        !cruel
                        {oolo} sent me. If you have a problem, I'm pretty sure there's a flagpole you can run it up.
                        $miscMocking
                        Or sit on. Your call. 
                    agent:
                        Well, this is one sign the auction is sure losing its prestige. 
                }
                { liked ? 
                    agent:
                        !happy
                        {player}! I didn't expect to see you here.
                    player:
                        !happy
                        $happyGreeting
                        I'm working for {oolo}. Has its perks, being on the Admiralty's arm.
                    agent:
                        $neutralJoke
                        Yeah, well, it's a cheap date, sometimes. 
                        !permit
                        Glad to have you aboard. I heard there might be trouble today.
                }
                { neutral ? 
                    agent:
                        !placate
                        Hold it, grifter. This auction is invitation-only. 
                    player:
                        I'm working for {oolo}. I've got an extremely shiny badge to show for it, too. 
                    agent:
                        !dubious
                        Oh, I heard about you. A weird move, but these are weird times.
                    player:
                        $neutralJoke
                        Nice of you to say the times are weird, and not Oolo herself.
                    agent:
                        I didn't <i>not</i> say it. 
                        !permit
                        Go on in. 
                }
            ]],
            OPT_GIVE_REQUISITION = "Hand over your requisition",
            DIALOG_GIVE_REQUISITION = [[
                player:
                    !handwring
                    I wanna have a look at the auction items before any go up on the block. 
                    {oolo} says I'm allowed to take one as a reward.

                {disliked ? 
                    agent:
                        !crossed
                        You expect me to believe that?
                    player:
                        $neutralDirect
                        You don't have to. She gave me a requisition chit.
                        !give
                    agent:
                        !take
                        $miscSurprised
                        She gave you a—
                        Hrmph. 
                        !neutral_notepad
                        Which one are you taking?
                }
                {not disliked ?
                    agent:
                        !interest
                        That's a big ask, {player}. You got any proof?
                    player:
                        !give
                        Just a little requisition chit. 
                    agent:
                        !take
                        Legit and everything. Then step right up, hunter.
                        !neutral_notepad
                        Which one will you be taking?
                }
                player:
                    !thought
                * You browse the items on display, each one carefully positioned on mildew-repelling fabrics. 
            ]],

            DIALOG_DONE_REQUISITION = [[
                agent:
                    Is that it? Are we done here?
            ]],

            OPT_TAKE_SEAT_AT_AUCTION = "Take a place at the auction",
            DIALOG_TAKE_SEAT_AT_AUCTION = [[
                player:
                    !point
                    I'm needed up at the front. That way I can make sure nobody gets up to any funny business.
                
                {liked ? 
                    agent:
                        Hard to imagine anyone would be so bold.
                        But then I guess the auction attracts all kinds. 
                }
                
                {not liked ? 
                    agent:
                        !angry_shrug
                        Fine. But don't interfere with the bidding.
                        There are Cult bigwigs here who would gladly make an example of you.
                        !cruel
                        Not that I'd stop them. 
                }

                {neutral ?
                    agent: 
                        !agree
                        Sure. Should be an easy posting.
                        On your way, then.
                }

                player:
                    !exit
            ]],

            
            OPT_HANG_BACK_WITH_SECURITY = "Hang back with the security forces",
            OPT_HANG_BACK_WITH_SECURITY_TT = "Face Kashio in combat immediately!",
            DIALOG_HANG_BACK = [[
                player:
                    I'm here to back up the guard.
                {liked ? 
                    agent:
                        !disappoint
                        $neutralResigned
                        So Oolo <i>is</i> expecting things to go bad, huh?
                        I suppose it wouldn't be the first time. But it'd break a good long streak. 
                        Good luck, then. 
                }
                {neutral ? 
                    agent:
                        !shrug
                        Not sure why Oolo thinks we can't handle this ourselves, but I suppose the annex has folks on edge. 
                }
                {disliked ? 
                    agent:
                        !angry_shrug                      
                        Yeah, because grifters aren't security risks in and of themselves.
                        I'll have my eye on you. 
                }
                player:
                    !exit
            ]],
            
            DIALOG_PICK = [[
                agent:
                    !neutral_notepad
                    Fine, fine.
                    You got a post?
            ]],
            DIALOG_NO_PICK = [[
                player:
                    You know what? I think I'm good. 
                    She who travels lightly carries only peace of mind, right?
                agent:
                    !shrug
                    If you say so.
                    You got a post?
            ]],
            OPT_TAKE_ITEM = "Take {1#card}",
            OPT_TAKE_GRAFT = "Take {1#graft}",
            
            OPT_SMASH_ITEM = "Smash it",
            OPT_SMASH_GRAFT = "Smash it",

            OPT_SKIP = "Don't take anything"

        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                local guard = AgentUtil.GetFreeAgent("ADMIRALTY_GUARD", function(agent) return agent ~= cxt.quest.param.cross_quest_data.friend_outside and agent ~= cxt.quest.param.cross_quest_data.friend_inside end)
                cxt:ReassignCastMember("guard", guard)
                if cxt.quest.param.set_guard_relationship then
                    guard.social_connections:SetRelationship(cxt.quest.param.set_guard_relationship, cxt.player)
                end
                cxt:TalkTo("guard")
                cxt:Dialog("DIALOG_INTRO")
            end

            if cxt.quest.param.cross_quest_data.requisition then
                cxt:Opt("OPT_GIVE_REQUISITION")
                    :Dialog("DIALOG_GIVE_REQUISITION")
                    :Fn(function() 
                        cxt.quest.param.cross_quest_data.requisition = false
                        AuctionUtil.PresentLots(cxt, cxt.quest.param.cross_quest_data.auction_lots, 
                            function() 
                                cxt:Dialog("DIALOG_PICK")
                                cxt:Pop()
                            end,
                            function()  
                                cxt:Opt("OPT_SKIP")
                                    :Dialog("DIALOG_NO_PICK")       
                                    :Pop()
                            end)                        
                    end)
            end

            cxt:Opt("OPT_TAKE_SEAT_AT_AUCTION")
                :Dialog("DIALOG_TAKE_SEAT_AT_AUCTION")
                :GoTo("STATE_DO_AUCTION")

            cxt:Opt("OPT_HANG_BACK_WITH_SECURITY")
                :PostText("OPT_HANG_BACK_WITH_SECURITY_TT")
                :Dialog("DIALOG_HANG_BACK")
                :GoTo("STATE_DO_AMBUSH")
        end)

    :State("STATE_DO_AUCTION")
        :Loc{
            DIALOG_INTRO = [[
                
                * The auctioneer takes the stage and a hush falls over the crowd.
                auctioneer:
                    !right
                    !hesh_greeting
                    $happyGreeting
                    Welcome, all, to the Auction of Heshian Antiquities and Subservience!
                    As always, you've been gathered here by Hesh's will, drawn to this maelstrom of opulence. 
                    A reminder that acts of violence are not an acceptable form of payment.
                    ...
                    !clap
                    Now let's get down to business!
                    This next lot is a cache of powerful artifacts pulled from the depths of Grout Bog...
            ]],
            OPT_START_NEGOTIATION = "Join the auction",
            DIALOG_START_NEGOTIATION = [[   
                player:
                    !happy
                    !left
                    I'm in! Put me down for whatever!
                * You don't have the shills to <i>actually</> purchase any of these items, but these fools don't need to know that.
            ]],
            DIALOG_LOSE_NEGOTIATION = [[
                player:
                    I bid, uh... ten thousand shills!
                auctioneer:
                    !angry
                    Ten thousa—
                    You aren't supposed to be here, grifter!
                    !angry_accuse
                    Security!
                
                player:
                    $scaredPowerless
                    Uh...

                auctioneer:
                    !exit
                * The crowd parts and a bold, familiar figure strides toward you.
                kashio:
                    !right
                    !hips
                    Not sure how <i>you</> got into the auction. 
            ]],

            DIALOG_LOSE_NEGOTIATION_2 = [[
                
                player:
                    !angry
                kashio:
                    But rest assured the folks here can take one look at you and know you don’t belong.
                player:
                    $angryPatienceLost
                    I'm not here for <i>them</>, Kashio. I'm here for <i>you</>.

            ]],
            DIALOG_WIN = [[
                auctioneer:
                    !point
                    Sold! To the grifter in the front row!
                player:
                    !take
                {items?
                    * You gather up the {card_table#card_list}.
                }
                {grafts?
                    * You gather up the {graft_table#graft_list}.
                }
                auctioneer:
                    !exit
            ]],
            DIALOG_KASHIO_APPEARS = [[

                * It takes you a second to realize the hall has fallen quiet. Too quiet. 
                player:
                    !angry
                * While you were getting your bid on, Kashio and her goons closed rank around you. 
            ]],
            DIALOG_KASHIO_APPEARS_2 = [[

                kashio:
                    !right
                    !hips

                kashio:
                    !dubious
                    $angryGreeting
                    Not sure how a hunter like you got into the auction. 
                    But rest assured the folks here can take one look at you and know you don’t belong.

                player:
                    $angrySeething
                    You gonna start a fight here, Kashio? 

                kashio:
                    !cruel
                    $miscMocking
                    Why not? I’ve been asked to.

                * Sure enough, you look around and see more than a few disapproving stares from the gathered elite. 
                * Guess they didn’t take too kindly to you muscling in and claiming their prize. 
            ]],
        }
        :Fn(function(cxt) 
            cxt.enc:PresentSides(nil,nil)
            cxt:TalkTo("auctioneer")
            cxt:Dialog("DIALOG_INTRO")   
            cxt.quest.param.sal_is_ambushing = false
            
            local lots = cxt.quest.param.cross_quest_data.auction_lots

            cxt:Opt("OPT_START_NEGOTIATION")
                :Dialog("DIALOG_START_NEGOTIATION")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.NO_CORE_RESOLVE | NEGOTIATION_FLAGS.NO_IMPATIENCE | NEGOTIATION_FLAGS.NO_LOOT,
                    reason_txt = cxt:GetLocString("NEGOTIATION_REASON"),
                    on_start_negotiation = function(minigame)
                        local core = minigame.opponent_negotiator:FindCoreArgument()
                        core:SetItems(lots.grafts, lots.items)
                        core:AddAuctionItem()
                        
                        minigame.opponent_negotiator:CreateModifier( "BIDDER" )
                        minigame.opponent_negotiator:CreateModifier( "BIDDER" )
                    end,
                    on_success = function(cxt, minigame) 
                        local core = minigame.opponent_negotiator:FindCoreArgument()
                        if not core.won_item and not core.won_graft and (#lots.items + #lots.grafts) > 0 then
                            --get one at random I guess?
                            local idx = math.random( #lots.items + #lots.grafts )
                            print( "Random picked a lot:", idx, #lots.items, #lots.grafts )
                            if idx <= #lots.items then
                                core.won_item = { table.remove(lots.items, idx) }
                            else
                                core.won_graft = { table.remove(lots.grafts, idx - #lots.items) }
                            end
                        end
                        cxt.enc.scratch.items = core.won_item and #core.won_item > 0
                        cxt.enc.scratch.grafts = core.won_graft and #core.won_graft > 0
                        cxt.quest.param.card_table = core.won_item or {}
                        cxt.quest.param.graft_table = core.won_graft or {}
                        cxt:Dialog("DIALOG_WIN")
                        if core.won_item then
                            for i,item in ipairs(core.won_item) do
                                table.arrayremove(lots.items, item)
                                cxt:GainCards{item}
                            end
                        end
                        if core.won_graft then
                            for i,graft in ipairs(core.won_graft) do
                                table.arrayremove(lots.grafts, graft)
                                ConvoUtil.GiveGraft(cxt, GraftInstance(graft))
                            end
                        end 
                        
                        cxt:Dialog("DIALOG_KASHIO_APPEARS")
                        cxt.encounter:SetMusicEvent( kashio_stinger )
                        cxt:Dialog("DIALOG_KASHIO_APPEARS_2")
                        cxt:GoTo("STATE_FIGHT_KASHIO")
                    end,

                    on_fail = function() 
                        cxt:Dialog("DIALOG_LOSE_NEGOTIATION")
                        cxt.encounter:SetMusicEvent( kashio_stinger )
                        cxt:Dialog("DIALOG_LOSE_NEGOTIATION_2")
                        cxt:GoTo("STATE_FIGHT_KASHIO")
                    end,
                }

        end)

    :State("STATE_DO_AMBUSH")
        :Loc{
            DIALOG_INTRO = [[
                left:
                    !exit
                right:
                    !exit
                * The auction starts. 
                * You let the items pass you by. You’re in this for an even bigger prize.
                * And it isn’t long before you’re rewarded. You get so used to the same few oiled elbows making their bids that new movement at the edge of the room draws your attention.

                kashio:
                    !right
                    !happy

                * It’s Kashio, looking none the wiser that you might be in the crowd. 
                * She’s confident. Her friends part to socialize with the crowd, leaving Kashio’s back exposed. 
                * Now’s your chance. You pass through the shadows like a thread, weaving your way to Kashio’s side.
            ]],
            DIALOG_INTRO_2 = [[
                kashio:
                    !angry

                player:
                    !left
                    !threaten
                    $angryGreeting
                    No sudden movements.

                * Your blade presses gently into her side, just below her ribs.

                player:
                    $angrySeething
                    I can’t even count as high as the number of reasons I got for wanting you dead.
                    But I’ll still turn you in alive, if you play nice.
            ]],
        }
        :Fn(function(cxt) 
            cxt.quest.param.sal_is_ambushing = true
            cxt:Dialog("DIALOG_INTRO")
            cxt.encounter:SetMusicEvent( kashio_stinger )
            cxt:Dialog("DIALOG_INTRO_2")
            cxt:GoTo("STATE_FIGHT_KASHIO")
        end)

    :State("STATE_FIGHT_KASHIO")
        :Loc{
            DIALOG_INTRO = [[

                kashio:
                    $miscMocking
                    !cruel
                    I knew a grubby little bogling like you would grow up too greedy for your own good.

                player:
                    $angrySeething
                    I hate you enough to almost fall for that. But ain’t nobody greedier than you, Kashio. 
                    $angryAside
                    You sold out your wit, your friends—heck, even me. 

                kashio:
                    ...
                
                player:
                    I bet my contract didn’t even go for that high a price. You coulda made more off me as your apprentice.     
                    But I bet lookin’ at my face just made you hate your freedom a little too much.

                {sal_is_ambushing ? 
                    * Seems you’re not the only action on the floor. Admiralty guards pour in from the door, roughly accosting some merchants who, judging by the scars, are anything but. 
                    * You’ve been in enough brawls to know what’s about to go down. But so has Kashio. 
                    kashio:
                        !fight
                    * She thrusts her elbow into the crook of your blade arm, throwing you off balance. By the time you recover she’s got her weapon drawn.
                }

                {not sal_is_ambushing ? 
                    * You notice Admiralty guards approaching from the door, and see the specters of Spree goons coalescing out of the shadows.
                    * You’ve been in enough brawls to know what’s about to go down.
                    kashio:
                        !fight
                    * Kashio’s answer is to raise her weapon, her goons convene to block your escape from either side.  
                }
            ]],
            

            OPT_FIGHT = "Kill Kashio",
            DIALOG_FIGHT_PRE = [[
                * The lights go out.
                * You hear the shouts, bangs, and sound of the great Lumin tank under the stage breaking.
                player:
                    !fight
                * The emergency lighting comes back on, revealing a ruined auction hall.
            ]],
            DIALOG_FIGHT_PST = [[
                kashio:
                    !fight
                    $angryCruel
                    Well, ik-Derrick?
            ]],
            
            
            DIALOG_WON_FIGHT = [[
                player:
                    !left
                    !neutral
                    !tired
                
                kashio:
                    !exit
                
                * The dust settles. Those that could took cover out of harm’s reach. Heads reappear like crabs crawling out of the sludge after the rain.

                { nadan_friend ? 
                    * Once you and Kashio threw the first blows, the Spree hiding in the rafters made their move. The Admiralty never saw it coming, and cultists fled before their own heads could roll.

                    nadan:
                        !right
                        Never seen a knife move as fast as yours.
                        That’s good. Efficient.

                    {dead ?
                        player:
                            !cruel
                            I got these blades with Kashio in mind. 

                        * Your words come light, but the presence of Kashio's corpse weighs heavy.

                        nadan:
                            !agree
                            Then keep them close. They’re loyal and effective.
                            A weapon like that is better than a friend. 

                        player:
                            !dubious
                            Dunno if I agree with that.

                        nadan:
                            $neutralDirect
                            You don’t have to. 
                            See you around, Sal. Remember what I said.
                            !exit

                        * Nadan melts back into the shadows.
                        * You look down one last time at Kashio before taking what you need to claim the bounty. 
                        * This feeling you get ain't exactly peace—but it's reassuring, all the same. 
                        * You leave her on the floor without a backwards glance. No more looking back.
                    }
                    {not dead ?
                        nadan:
                            !right
                            What about <i>her</i>?

                        kashio: 
                            !right
                            !injured
                            Neither of you are the type to do me any favors.

                        player:
                            $neutralResigned
                            All due respect, Kashio—which is none, by the way—
                            !angry_shrug
                            $angryAside
                            You have no idea what “type” I am.
                            I’ll turn you in alive. Let the client decide what to do with you.
                            !cruel
                            The bounty’s pretty steep. Maybe they’ll recoup their losses—sell you to the derrick.

                        kashio:
                            $angrySeething
                            If you could work your way out, so can I.

                        player:
                            $neutralDirect
                            If you do, you’ll deserve a fresh start. 

                        nadan:
                            !right
                            !angry
                            !angryDefault
                            This isn’t what we agreed.
                            She was supposed to die.

                        player:
                            !dubious
                            I made my choices. Thought you said that was alright.

                        nadan:
                            $neutralResigned
                            I did. And if I wanted it done my way, I should’ve done it myself.
                            !thought
                            ...
                            You said you’d never take the bounty to come after me.

                        player:
                            !crossed
                            Didn’t say never. Just said I wouldn’t want to.

                        nadan:
                            ...
                            Something to think about, anyway.
                            See you around, Sal.
                            !exit

                        * Nadan ducks out of the tent before reinforcements can arrive.
                        * You leave by the front door, bounty in hand. 
                    }
                }
            ]],

            DIALOG_WON_FIGHT_2 = [[

                {oolo_friend ? 
                    * Once you and Kashio threw the first blows it wasn’t long before the Spree hiding in the rafters made their move, and the Admiralty closed in to finish them off. There aren’t many people left standing.

                    oolo:
                        !right
                        You still breathing, Sal? 
                    player:
                        !left
                        Seem to be.
                    oolo:
                        $neutralJoke
                        I dunno. I wouldn’t put it past you to be an extremely cunning and duplicitous corpse.
                        !permit
                    * She pats down your shoulders, checking you for bumps and bruises.
                        
                    oolo:
                        !shrug
                        Enh, you’ve been through worse. 

                    {dead ?
                        oolo:
                            You killed her, huh? Figured the bounty would be worth more if she were alive.

                            She had valuable Spark Baron intel, I’ll bet. Inventions, discoveries—ancient ruins worth plundering.

                        player:
                            !disappoint
                            $neutralRelieved
                            She didn’t really give me much of a choice.

                            Besides, there was more than shills at stake here.

                        oolo:
                            Hesh forbid I should show sympathy for you or anyone else, but—
                            You don’t look as jubilant as you should.

                        player:
                            !dubious
                            You know what’s stupid?

                            I would’ve forgiven her. Five years in, seven years, even. All she had to do was come get me.

                        oolo:
                            Yeah.
                            That <i>is</i> kinda stupid.
                            But I get it. Family baggage. I don’t have any but I’ve heard of it. 

                        player:
                            Kashio wasn’t my family.

                        oolo:
                            !disapproving
                            No, that’s for sure. Family wouldn’t sell you to the derrick, for a start.
                            Get out of here, Sal. Get your bounty. 
                            !happy
                            And when you need something to keep you busy, come find me.
                            
                        * Oolo heads to the door, barking orders as a flood of Admiralty reinforcements arrive.
                        * You look down one last time at Kashio before taking what you need to claim the bounty.
                        * This feeling you get ain't exactly peace—but it's reassuring, all the same.
                        * You leave her on the floor without a backwards glance. No more looking back.
                    }
                    {not dead?
                        oolo:
                            !clap
                            $happyAmused
                            Don’t tell me you’ve left me a present.

                        player:
                            !crossed
                            There’s no Admiralty bounty for Kashio. But I figured you’d like to say goodbye before I haul her off for good.

                        kashio:
                            !right
                            !injured
                            $angrySeething
                            As if you haven’t just invited more yotes into your den.
                            $angryDefault
                            There are Admiralty officers up the ranks who won’t be happy at your initiative, switch. And they won’t make life any easier for either of you.
 
                        oolo:
                            !left
                            !thumb
                            Hey, I welcome the change of pace.
                            And when has life ever been easy for Sal, huh? You ever seen a derrick?
                            $neutralJoke
                            I bet Sal’s teeth still glow in the dark.

                        player:
                            !left
                            It isn’t about what’s easy.
                            It’s just about putting things right.

                        kashio:
                            !exit

                        * Kashio turns away from you, so all you can see is the stubborn curve of her brow.

                        oolo:
                            !right
                            Hesh speed, Sal. I’m sure I’ll see you very soon.
                            Won’t be long before I’ll need someone with a bit of flexibility.

                        player:
                            With this bounty, I’ll never need to work again.

                        oolo:
                            $neutralJoke
                            Hah! As if you’d retire now. 
                            !wink
                            $miscMischievous
                            I’ll see you when you’re bored, then. 

                        * Oolo heads to the door, barking orders as a flood of Admiralty reinforcements arrive.
                        * You head out the back door, to keep your bounty from too many prying eyes. 
                    }
                }
            ]],

            DIALOG_WON_FIGHT_3 = [[

                {not oolo_friend and not nadan_friend ?
                    {dead ?
                        * You look down one last time at Kashio, before taking what you need to claim the bounty.
                        * You leave her on the floor without a backwards glance. No more looking back.
                    }
                    {not dead?
                        kashio:
                            !right
                            !injured
                            You’re about an inch short, kid.
                            $neutralDirect
                            Revenge isn’t satisfied by mercy.

                        player:
                            !crossed 
                            $neutralResigned
                            Only satisfaction I need is mine.  
                            I spent too many years trying to be like you. 

                        kashio:
                            $angryDefault
                            And you learned better, huh? So what. You’re still standing here alone.

                        * She ain’t wrong, but no doubt she’d have found something to be smug about. 
                        * You cuff her and walk her out the door. No one stops you, because who would dare?
                    }

                }
            
            ]],

            OPT_WIN = "Leave the auction",
        }
        :Fn(function(cxt)
            local handler_card            
            cxt:TalkTo("kashio")
            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_FIGHT")
                :Fn(function() 
                    cxt.location:SetPlax("INT_AuctionHouse_01_Damage") 
                    cxt.enc:GetScreen():Fade( .2, false )
                    local loopsound = AUDIO:CreateEventInstance("event:/sfx/ambience/scene/indoor/auction_house_brawl")
                    loopsound:Start()
                    AUDIO:PlayEvent("event:/sfx/ambience/story/auction_luminburst")
                    
                    for k,v in ipairs(cxt.quest.param.crowd) do
                        v:MoveToLimbo()
                    end
                    
                    if Util.IsPlayerWorkingWithFaction("BANDITS") and cxt:GetCastMember("nadan") and not cxt:GetCastMember("nadan"):IsDead() and cxt:GetCastMember("nadan"):GetRelationship() >= RELATIONSHIP.NEUTRAL then
                        handler_card = "nadan_signal"
                    end

                    if Util.IsPlayerWorkingWithFaction("ADMIRALTY") and cxt:GetCastMember("oolo") and not cxt:GetCastMember("oolo"):IsDead() and cxt:GetCastMember("oolo"):GetRelationship() >= RELATIONSHIP.NEUTRAL then
                        handler_card = "oolo_signal"
                    end

                    cxt:Dialog("DIALOG_FIGHT_PRE")
                    cxt.enc:GetScreen():Fade( 1, true )
                    loopsound:Stop()
                    loopsound:Release()

                    cxt:Dialog("DIALOG_FIGHT_PST")
                end)
                :Battle{
                    flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.NO_REWARDS | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.BOSS_FIGHT,
                    advantage = cxt.quest.param.sal_is_ambushing and TEAM.BLUE,
                    on_start_battle = function(battle) 
                        local fighter = battle:GetFighterForAgent(cxt.quest:GetCastMember("kashio"))
                        if fighter then
                            for k,v in ipairs(cxt.quest.param.cross_quest_data.auction_lots.items) do
                                local condition = AUCTION_DATA.CONDITION_LOOKUPS[v]
                                if condition then
                                    fighter:AddCondition(condition)
                                end
                            end
                            for k,v in ipairs(cxt.quest.param.cross_quest_data.auction_lots.grafts) do
                                local condition = AUCTION_DATA.CONDITION_LOOKUPS[v]
                                if condition then
                                    fighter:AddCondition(condition)                                    
                                end
                            end
                            fighter:ClearPreparedCards()
                            fighter:PrepareTurn()

                            if handler_card then
                                local card = Battle.Card( handler_card, battle.player_team:Primary() )
                                card.show_dealt = true
                                battle:DealCard( card, battle:GetDrawDeck() )
                            end

                        end
                    end,
                }
                    :OnWin()
                        :Fn(function() 
                            
                            cxt.enc.scratch.nadan_friend = Util.IsPlayerWorkingWithFaction("BANDITS") and cxt:GetCastMember("nadan") and not cxt:GetCastMember("nadan"):IsDead() and cxt:GetCastMember("nadan"):GetRelationship() >= RELATIONSHIP.NEUTRAL
                            cxt.enc.scratch.oolo_friend = Util.IsPlayerWorkingWithFaction("ADMIRALTY") and cxt:GetCastMember("oolo") and not cxt:GetCastMember("oolo"):IsDead() and cxt:GetCastMember("oolo"):GetRelationship() >= RELATIONSHIP.NEUTRAL
                            
                            cxt:Dialog("DIALOG_WON_FIGHT")
                            cxt:Dialog("DIALOG_WON_FIGHT_2")
                            cxt:Dialog("DIALOG_WON_FIGHT_3")
                            
                            cxt:Opt("OPT_WIN")
                                :Fn(function() 
                                    cxt.quest:Complete()
                                    cxt:End()
                                    if not TheGame:GetLocalSettings().ROBOTICS then
                                        local flags = {
                                            kashio_is_alive = not cxt:GetCastMember("kashio"):IsDead(),
                                            kashio_is_dead = cxt:GetCastMember("kashio"):IsDead(),
                                            nadan_is_alive = Util.IsPlayerWorkingWithFaction("BANDITS") and cxt:GetCastMember("nadan") and not cxt:GetCastMember("nadan"):IsDead(),
                                            oolo_is_alive = Util.IsPlayerWorkingWithFaction("ADMIRALTY") and cxt:GetCastMember("oolo") and not cxt:GetCastMember("oolo"):IsDead(),
                                        }
                                        TheGame:GetGameState():AddScore("PROGRESS", 5)
                                        
                                        cxt:AwardUnlock("SAL_BRAWL_PLAYABLE", "SAL", LOC"UI.CHARACTER_UNLOCKED.SAL_BRAWL_TITLE", LOC"UI.CHARACTER_UNLOCKED.SAL_BRAWL_BODY")

                                        cxt.enc:ShowSlides("sal_ending_slideshow", flags, function() TheGame:Win() end )
                                    end
                                end)
                        end)
        end)


QDEF:AddDebugOption("force_faction_alliance", {"ADMIRALTY", "BANDITS"})
    :AddDebugOption("requisition", {false, true}, function(param) return param.force_faction_alliance == "ADMIRALTY" end)
    :AddDebugOption("set_guard_relationship", RELATIONSHIP_ARRAY, function(param) return param.force_faction_alliance == "ADMIRALTY" end)    

QDEF:AddTest(TestCreator()
                :AddParam("force_faction_alliance", {"ADMIRALTY"})
                :AddParam("requisition", {true, false}))

QDEF:AddTest(TestCreator()
                :AddParam("force_faction_alliance", {"BANDITS"}))
                
