local QDEF = QuestDef.Define{
        title = "Guard Duty",
        qtype = QTYPE.STORY,
        icon = engine.asset.Texture("icons/quests/sal_story_auction_house_admiralty.tex"),
        on_start = function(quest) 

            quest.param.cross_quest_data = quest.param.cross_quest_data or {}
            
            if quest.param.set_target_relationship == "FRIEND" then
                quest:GetCastMember("assassination_target").social_connections:SetRelationship(RELATIONSHIP.LIKED, TheGame:GetGameState():GetPlayerAgent())
            elseif quest.param.set_target_relationship == "ENEMY" then
                quest:GetCastMember("assassination_target").social_connections:SetRelationship(RELATIONSHIP.DISLIKED, TheGame:GetGameState():GetPlayerAgent())
            elseif quest.param.set_target_relationship == "NEUTRAL" then
                quest:GetCastMember("assassination_target").social_connections:SetRelationship(RELATIONSHIP.NEUTRAL, TheGame:GetGameState():GetPlayerAgent())
            end
        end,
    }

    :AddLocationDefs{
        BACK_ROOM =
        {
            name = "Back Room",
            indoors = true,
            no_exit = true,
            plax = "INT_Auction_Backroom_1",
        },
    }

    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_ADMIRALTY_CONTACT",
        no_validation = true,
    }

    :AddCast{
        cast_id = "other_guard",
        cast_fn = function(quest, t) 
            table.insert(t, quest.param.cross_quest_data and quest.param.cross_quest_data.other_guard or AgentUtil.GetFreeAgent("ADMIRALTY_GUARD"))
        end,
    }

    :AddCast{
        cast_id = "assassination_target",
        score_fn = function(agent, quest)
            local score = 1
            if agent:GetRelationship() ~= RELATIONSHIP.NEUTRAL then
                score = score + 10
            end
            return score
        end,

        condition = function(agent)
            return agent:GetFactionID() ~= "ADMIRALTY" and agent:GetRenown() <= 3 and agent:GetCombatStrength() <= 3
        end,
    }
    :AddCastFallback{
        cast_fn = function(quest, t)
            table.insert( t, AgentUtil.GetFreeAgent("FOREMAN") )
        end,
    }

    :AddObjective{
        id = "assassinate_the_target",
        title = "Kill {assassination_target}",
        on_complete = function(quest)
            quest:Activate("enter_the_auction")
        end,

        process_fighter = function(quest, fighter)
            if fighter.agent == quest:GetCastAgent("assassination_target") then
                fighter:AddCondition("WANTED_DEAD")
            end
        end,
        
        state = QSTATUS.ACTIVE,
    }

    :AddLocationCast{
        cast_id = "entrance",
        when = QWHEN.MANUAL,
    }
    :AddObjective{
        id = "enter_the_auction",
        title = "Enter the Auction",
        terminal = true,
    }

    :AddOpinionEvents{
        let_go_after_beating = 
        {
            delta = OPINION_DELTAS.NEUTRALIZING,
            txt = "Helped them escape from an assassination attempt",
        },
        provoke_attempt = 
        {
            delta = OPINION_DELTAS.DISLIKE,
            txt = "Tried to provoke them into a fight",
        },
        warned_them_of_assassination = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Warned them about an assassination attempt",
        },
    }


QDEF:AddConvo("assassinate_the_target")
    :ConfrontState("ASSASSINATION")
        :Loc{
            DIAL_ASSASSINATION_INTRO = [[
                player:
                    !left
                    !overthere
                    $neutralResigned
                    Okay, I'm ready. Anything I should know before I get started?
                handler:
                    !right
                    !agree
                    One thing, yeah. Mostly you just need to check papers, make sure everyone's on the guest list. Except for someone named {assassination_target}.
                    !throatcut
                    $miscAlluring
                    Unfortunately {assassination_target.heshe}'s gonna have an accident before {assassination_target.heshe} can make {assassination_target.hisher} way inside, if you understand my meaning.
                    But this isn't some criminal or debt-dodger, so your hunter rights don't apply.
                    !point
                    Get {assassination_target.himher} away from any witnesses—or, heck, I don't know, make it look like self-defense. Then the witnesses work in your favor. 
                    ...
                    !dubious
                    $neutralDubious
                    You look apprehensive. Something wrong?
                player:
                    !bashful
                    $miscEmbarrassed
                    Uh, nope. This is just my resting face. Constantly apprehensive.
                handler:
                    !point
                    Well... this is the last task. Get this done and all our debts are settled. Heck, I might even owe <i>you</i>.
                    !exit
                * {handler} heads off to take care of business elsewhere.
                player:
                    !exit
                * You start your shift.
                * You process person after person, doing searches, checking documents, and generally being a pain in the arse.
                * Finally, your mark arrives.
            ]],

            DIAL_TARGET_ARRIVES_FRIEND = [[
                player:
                    !left
                    !scared
                assassination_target:
                    !right
                    !greeting
                    $happyDefault
                    Wha—hey, Sal! Imagine meeting you here. Didn't expect to find you working this kind of gig. Hope that means you can get into the auction afterwards. 
                    !wink
                    $happyDefault
                    I'll buy you a drink, for old time's sake.
                player:
                    !facepalm
                    $scaredDefault
                    Oh, geeze.
            ]],

            DIAL_TARGET_ARRIVES_ENEMY = [[
                player:
                    !left
                    !cruel
                assassination_target:
                    !right
                    !angry_shrug
                    $angryHostile
                    What the—what the Hesh are <i>you</i> doing here, lumin stain?
                player:
                    !cruel
                    $happyCocky
                    Guarding the doors. Why, you wanna get through?
                assassination_target:
                    !angry
                    $angryHostile
                    You couldn't dream of keeping me out, miscreant. My name's on the list. 
                player:
                    !chuckle
                    $happyCocky
                    Oh, it sure is. 
            ]],
            DIAL_TARGET_ARRIVES_NEUTRAL = [[
                player:
                    !left
                assassination_target:
                    !right
                    !happy
                    Hello, officer. I'm here for the auction. 
                    I believe you'll find all my equipment meets entry requirements.
                player:
                    I see. Name?
                assassination_target:
                    {assassination_target}. I believe I'm on the list. 
                player:
                    Yeah, I believe you're right.
            ]],

            OPT_PROVOKE = "Provoke {assassination_target} into attacking you",

            DIAL_PROVOKE_INTRO_ENEMY = [[
                player:
                    !cruel
                    $angryHostile
                    You gonna back up those insults with any kind of muscle? Or are you too scared to come at me when I've got a badge?
            ]],
            DIAL_PROVOKE_INTRO_2_ENEMY = [[
                assassination_target:
                    !angry
                    $angryHostile
                    I should have expected as much from you.
            ]],

            DIAL_PROVOKE_SUCCESS_ENEMY = [[
                assassination_target:
                    !angry_shrug
                    $angryHostile
                    I don't care what you're doing—I'll gut you where you stand if you play me wrong.
                player:
                    !chuckle
                    $happyCocky
                    Oh yeah? Prove it. You don't have the guts.
                assassination_target:
                    !fight 
                    $angryHostile
                    I have the guts, all right. And I'm gonna leave <i>yours</i> strewn across the dirt when I'm done.
            ]],
            DIAL_PROVOKE_FAIL_ENEMY = [[
                assassination_target:
                    !dubious
                    $happyCocky
                    Is that the best you got? Please. I've been more insulted by a vroc. Now enough with the hold up. 
            ]],


            DIAL_PROVOKE_INTRO_FRIEND = [[
                player:
                    !dubious
                    $angryCruel
                    You know, I've always thought you were uglier than you smell, but then I realized I was just congested.
            ]],
            DIAL_PROVOKE_INTRO_2_FRIEND = [[
                assassination_target:
                    !surprised
                    $scaredTakenAback
                    Wha—what's that for?
            ]],
            DIAL_PROVOKE_SUCCESS_FRIEND = [[
                player:
                    $angryHostile
                    Look at you. Can't even think up a good comeback. I never should've let you ride on my coat tails. 
                    !throatcut
                    But once we're done here? I'll be glad to be rid of you.
                assassination_target:
                    !fight
                    $angrySeething
                    Is that so, <i>friend</i>? Well excuse me while I leave a scar or two for you to remember me by. 
            ]],
            DIAL_PROVOKE_FAIL_FRIEND = [[
                assassination_target:
                    !handwave
                    $happyDefault
                    You should be careful joking around like that. Not everyone gets that kind of humor, Sal.
                    !wink
                    Now, if you'll excuse me, I do have some money I'm looking forward to spending.
            ]],
            
            DIAL_PROVOKE_INTRO_NEUTRAL = [[
                player:
                    We don't let people like <i>you</> into this fine establishment.
            ]],
            DIAL_PROVOKE_INTRO_2_NEUTRAL = [[
                assassination_target:
                    !hips
                    Excuse me? What did you say?!
            ]],
            DIAL_PROVOKE_SUCCESS_NEUTRAL = [[
                assassination_target:
                    I'm not going to stand for this!
            ]],
            DIAL_PROVOKE_FAIL_NEUTRAL = [[
                assassination_target:
                    Why are you treating me this way?
            ]],


            OPT_SEARCH = "Convince {assassination_target} you need to search {assassination_target.hisher} bags",
            

            DIAL_SEARCH_INTRO_ENEMY = [[
                player:
                    $neutralDirect
                    !shrug
                    Just 'cause you hate me doesn't mean the rules don't apply. The Cult of Hesh demands tight security.
            ]],
            DIAL_SEARCH_SUCCESS_ENEMY = [[
                assassination_target:
                    !faceplam
                    $neutralResigned
                    Fine. Bet you're just having a <i>day</i> with this authority you have over me, huh?
                    !angry_threaten
                    $angryHostile
                    Just don't forget it's only temporary. And I'll remember the smug look on your face the next time we meet. 
            ]],
            DIAL_SEARCH_FAIL_ENEMY = [[
                assassination_target:
                    !chuckle
                    Hah! Not a chance. My bag is my business, and if you don't like it, you can suck my stale air.
            ]],


            DIAL_SEARCH_INTRO_FRIEND = [[
                player: 
                    !placate
                    Unfortunately I gotta check your bag before I can let you in.
                assassination_target:
                    !cagey
                    Oh! Er... here? For everyone to see?
                player:
                    !wink
                    No worries, friend. We can take it inside. Away from prying eyes.
            ]],
            DIAL_SEARCH_SUCCESS_FRIEND = [[
                assassination_target:
                    !happy
                    Thanks for the discretion, Sal. Of course everything in my bag is totally legal. Just some of it might give less understanding folks the wrong idea.
            ]],

            DIAL_SEARCH_FAIL_FRIEND = [[
                assassination_target:
                    !chuckle
                    C'mon, Sal, we're a little past that, aren't we? If I say my bag is clear, it's clear, right?
            ]],
            
            DIAL_SEARCH_INTRO_NEUTRAL = [[
                player:
                    I have to check your bag in the secondary inspection area.
            ]],
            DIAL_SEARCH_SUCCESS_NEUTRAL = [[
                assassination_target:
                    I - uh, OK. There's nothing amiss in there, I assure you!
            ]],
            DIAL_SEARCH_FAIL_NEUTRAL = [[
                assassination_target:
                    !angry
                    No, I know my rights! I don't have to listen to some grifter!
            ]],

            OPT_WARN_OF_ATTACK = "Warn {assassination_target} of {handler}'s plan",

            DIAL_WARNED_OF_ATTACK_ENEMY = [[
            * You check to make sure the other guards can't overhear you and drop your voice to an urgent hush.
                player:
                    !cagey
                    Hey, look—we don't share no love for each other, but I don't want you dead. This is a trap.
                    You've been set up for a hit. And I'm the one meant to do it.
                assassination_target:
                    !spit
                    Please. You're pathetic, you know that? Get outta my way.
                * {assassination_target.HeShe} brushes past you with a sharp shoulder to yours. 
            ]],

             DIAL_WARNED_OF_ATTACK_FRIEND = [[
            * You check to make sure the other guards can't overhear you and drop your voice to an urgent hush.
                player:
                    !cagey
                    Hey, {assassination_target}—you gotta get outta here. Someone wants you dead, and they've hired me to do it.
                    But just cus I won't doesn't mean they don't have a back-up plan. You walk in there, you might end up dead.
                assassination_target:
                    !thought
                    Yeah? Maybe... but not if you're watching my back.
                player:
                    !interest
                    But—
                assassination_target:
                    !thumb
                    I'm not giving up the auction just cus someone's got it out for me. I'm going in there. You come find me and we'll take care of it, yeah?
                    !greeting
                    But hey—thanks for the warning. You really are a pal. 
            ]],

            DIAL_WARNED_OF_ATTACK_NEUTRAL = [[
                * You wait for the other guards to be distracted, and then warn {assassination_target} in hushed tones.
                player:
                    Word on the street is that someone powerful wants you dead. This is not a safe place for you.
                assassination_target:
                    !surprised
                    What? Really? I have no idea why!
            ]],

            DIAL_LET_PASS_WARNED_ENEMY = [[
                player:
                    !angry_shrug
                    Fine. I tried to warn you. It's your funeral.
                assassination_target:
                    !handwave
                    <i>Shut up already</i>. I'm heading inside.
            ]],

            DIAL_LET_PASS_WARNED_FRIEND = [[
                player:
                    !disappoint
                    Still think you shouldn't go in there. But it's your call, I guess.
                assassination_target:
                    !thumb
                    Hey, I'll feel safer surrounded by witnesses. Good luck out here!
            ]],
            DIAL_LET_PASS_WARNED_NEUTRAL = [[
                player:
                    Alright, be careful in there. Don't forget what I said.
                assassination_target:
                    Thank you, {player}. I won't forget this.
            ]],

            DIAL_LET_PASS_ENEMY = [[
                player:
                    !handwave
                    You know what? You're not worth the hassle I could cook up to give you.
                    !overthere
                    You go right on ahead, buddy.
                assassination_target:
                    !cruel
                    As if you could stop me.
                * {assassination_target.HeShe} heads inside without so much as a backwards glance.
            ]],

            DIAL_LET_PASS_FRIEND = [[
                player:
                    !greeting
                    Hey, {assassination_target}! Your name's on the list, so come on in.
                assassination_target:
                    !happy
                    Thanks, pal!
                * {assassination_target.HeShe} heads inside with a wink, tossing you a wink like it's a tip. 
            ]],
            DIAL_LET_PASS_NEUTRAL = [[
                player:
                    This all checks out, go on in.
                assassination_target:
                    Thank you, grifter.
            ]],
            OPT_LET_PASS = "Let {assassination_target} enter the auction",
            OPT_FIGHT = "Fight",
            NEGOTIATION_REASON = "Insult {agent} to provoke an attack!",
        }
        :SetLooping()
        :Fn(function(cxt)
            
            local target = cxt.quest:GetCastMember("assassination_target")


            if cxt:FirstLoop() then

                cxt:TalkTo(cxt.quest:GetCastMember("handler"))
                cxt:Dialog("DIAL_ASSASSINATION_INTRO")
                cxt.quest:GetCastMember("handler"):MoveToLimbo()

                cxt.quest:AssignCastMember("entrance", cxt.location)
                cxt.quest.param.relationship_suffix = "_NEUTRAL"            
                if target:GetRelationship() < RELATIONSHIP.NEUTRAL then
                    cxt.quest.param.relationship_suffix = "_ENEMY"
                elseif target:GetRelationship() > RELATIONSHIP.NEUTRAL then
                    cxt.quest.param.relationship_suffix = "_FRIEND"            
                end

                cxt:TalkTo(target)
                cxt:Dialog("DIAL_TARGET_ARRIVES"..cxt.quest.param.relationship_suffix)
            end


            cxt:Opt("OPT_PROVOKE")
                :Robot("PROVOKE")
                :Dialog("DIAL_PROVOKE_INTRO"..cxt.quest.param.relationship_suffix)
                :ReceiveOpinion("provoke_attempt")
                :Dialog("DIAL_PROVOKE_INTRO_2"..cxt.quest.param.relationship_suffix)
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.HOSTILE | NEGOTIATION_FLAGS.PROVOCATION,

                    reason_fn = function(minigame)
                        return loc.format(cxt:GetLocString("NEGOTIATION_REASON") )
                    end,

                    on_success = function( cxt, minigame )
                        cxt:Dialog("DIAL_PROVOKE_SUCCESS"..cxt.quest.param.relationship_suffix)
                        cxt:Opt("OPT_FIGHT")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE,
                                allies = {"other_guard"},
                                on_win = function(cxt)
                                    if target:IsDead() then
                                        cxt:GoTo("ENTER_AUCTION")
                                    else
                                        cxt:GoTo("POST_FIGHT_TARGET_ALIVE")
                                    end
                                end,
                            }

                    end,
                    on_fail = function( cxt )
                        cxt:Dialog("DIAL_PROVOKE_FAIL"..cxt.quest.param.relationship_suffix)
                    end,
                }

            cxt:Opt("OPT_SEARCH")
                :Robot("SEARCH")
                :Dialog("DIAL_SEARCH_INTRO"..cxt.quest.param.relationship_suffix)
                :Negotiation{
                    on_success = function( cxt )
                        
                        cxt:Dialog("DIAL_SEARCH_SUCCESS"..cxt.quest.param.relationship_suffix)
                        local new_location = cxt.quest:SpawnTempLocation("BACK_ROOM")
                        cxt.encounter:DoLocationTransition( new_location )
                        target:MoveToLocation(new_location)
                        cxt:GoTo("ALONE_WITH_ASSASSIN_TARGET")
                    end,
                    on_fail = function( cxt )
                        cxt:Dialog("DIAL_SEARCH_FAIL"..cxt.quest.param.relationship_suffix)
                    end,
                }

            if not target:HasMemory("WARNED_OF_ATTACK") then
                cxt:Opt("OPT_WARN_OF_ATTACK")
                    :Robot("WARN")
                    :Dialog("DIAL_WARNED_OF_ATTACK"..cxt.quest.param.relationship_suffix)
                    :ReceiveOpinion("warned_them_of_assassination")                    
                    :Fn(function(cxt)
                        target:Remember("WARNED_OF_ATTACK")
                    end)
            end

            if target:HasMemory("WARNED_OF_ATTACK") then
                cxt:Opt("OPT_LET_PASS")
                    :Robot("LET_PASS")
                    :Dialog("DIAL_LET_PASS_WARNED"..cxt.quest.param.relationship_suffix)
                    :GoTo("ASSASSINATION_FAILED")
            else
                cxt:Opt("OPT_LET_PASS")
                    :Robot("LET_PASS")
                    :Dialog("DIAL_LET_PASS"..cxt.quest.param.relationship_suffix)
                    :GoTo("ASSASSINATION_FAILED")
            end
        end)

    :State("ALONE_WITH_ASSASSIN_TARGET")
        :Loc{
            DIAL_ALONE_WITH_ASSASSIN_TARGET_ENEMY = [[
             * Though {assassination_target.heshe} huffs like a buffalo, you lead {assassination_target.himher} into a darkened room and gesture for {assassination_target.himher} to prop {assassination_target.hisher} satchel up on the table.
                player:
                    !left
                    !angry_shrug
                    Gotta confiscate any weapons you might have, too. None allowed in the auction.
                assassination_target:
                    !facepalm
                    You gotta be kidding me. Trust me when I say I'll be filing a very rigorous complaint with whoever put <i>you</i> on guard duty.
            ]],

            DIAL_ALONE_WITH_ASSASSIN_TARGET_FRIEND = [[
             * {assassination_target} follows you from your post outside the door into a darkened room, clearly unsuspecting. They good-naturedly put their satchel up the table for you to inspect.
                player:
                    !left
                assassination_target:
                    !right
                    !chuckle
                    Have a look. There's nothing <i>too</i> illegal in there, I should think.
            ]],

            DIAL_ALONE_WITH_ASSASSIN_TARGET_NEUTRAL = [[
                * {assassination_target} follows you from your post outside the door into a darkened room, clearly unsuspecting. 
                player:
                    !left
                assassination_target:
                    !right
                    So now what?
            ]],
            OPT_ATTACK = "Attack",
            DIAL_ATTACK = [[
                player:
                    !fight
                    Stay quiet and I'll make this quick.
            ]],
            DIAL_TARGET_DEAD = [[
                * {assassination_target} is dead. You stuff the body behind some crates, straighten your clothes, and go back outside.
            ]]
        }
        :Fn(function(cxt)
            cxt:Dialog("DIAL_ALONE_WITH_ASSASSIN_TARGET"..cxt.quest.param.relationship_suffix)
            cxt:Opt("OPT_ATTACK")
                :Dialog("DIAL_ATTACK")
                :Battle{
                    flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE,
                    allies = {},
                    enemies = {"assassination_target"},
                    on_win = function(cxt)
                        if cxt.quest:GetCastMember("assassination_target"):IsDead() then
                            cxt:Dialog("DIAL_TARGET_DEAD")
                            
                            cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("entrance") )
                            cxt:GoTo("ENTER_AUCTION")
                        else
                            cxt:GoTo("POST_FIGHT_TARGET_ALIVE")
                        end
                    end,
                }
        end)

    :State("POST_FIGHT_TARGET_ALIVE")
        :Loc{
            DIAL_POST_FIGHT_TARGET_ALIVE_ENEMY = [[
            * The dust settles, but you've yet to land the killing blow. 
                assassination_target:
                    !injured
                    So? You won. What more do you want? A medal for mercy?
            ]],

            DIAL_POST_FIGHT_TARGET_ALIVE_FRIEND = [[
            * The dust settles, and you have one less chance to reconcile before you say goodbye for good.
                assassination_target:
                    !injured
                    I don't understand! I thought we were on the level, Sal.
            ]],
            DIAL_POST_FIGHT_TARGET_ALIVE_NEUTRAL = [[
            * The dust settles.
                assassination_target:
                    !injured
                    What the Hesh?!
            ]],

            OPT_FINISH_OFF = "Finish {assassination_target.himher} off",
            DIAL_FINISH_OFF_ENEMY = [[
                player:
                    !cruel
                    Folks advise against mixing business with pleasure. But I dunno, it's working for me.
                assassination_target:
                    !exit
                * Like always, your final strike is clean and effective.
            ]],

            DIAL_FINISH_OFF_FRIEND = [[
                player:
                    !shrug
                    It's just business. 
                assassination_target:
                    !exit
                * You're not one for sloppy kills, but you make sure to keep this one especially painless.
            ]],
            DIAL_FINISH_OFF_NEUTRAL = [[
                player:
                    !shrug
                    I said I'd make it quick.
                assassination_target:
                    !exit
            ]],

            OPT_LET_GO = "Let {assassination_target} go",
            DIAL_LET_GO_FRIEND = [[
                player:
                    !hips
                    I was <b>ordered</> to kill you, but I'm not good at following orders.
                    Run out of here like you're on fire, and I can keep up my cover.
                assassination_target:
                    Ugh.
                    !exit
            ]],
            DIAL_LET_GO_ENEMY = [[
                player:
                    !hips
                    I was <b>ordered</> to kill you, but I'm not good at following orders.
                    Run out of here like you're on fire, and I can keep up my cover.
                assassination_target:
                    Ugh.
                    !exit
            ]],
            DIAL_LET_GO_NEUTRAL = [[
                player:
                    !hips
                    I was <b>ordered</> to kill you, but I'm not good at following orders.
                    Run out of here like you're on fire, and I can keep up my cover.
                assassination_target:
                    Ugh.
                    !exit
            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIAL_POST_FIGHT_TARGET_ALIVE"..cxt.quest.param.relationship_suffix)

            cxt:Opt("OPT_FINISH_OFF")
                :Robot("FINISH_OFF")
                :Dialog("DIAL_FINISH_OFF"..cxt.quest.param.relationship_suffix)
                :Fn(function(cxt)
                    cxt.quest:GetCastMember("assassination_target"):Kill()
                    cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("entrance") )
                    cxt:GoTo("ENTER_AUCTION")
                end)

            cxt:Opt("OPT_LET_GO")
                :Robot("LET_GO")
                :Dialog("DIAL_LET_GO"..cxt.quest.param.relationship_suffix)
                :ReceiveOpinion("let_go_after_beating")
                :Fn(function(cxt)
                    
                    cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("entrance") )
                    cxt:GoTo("ASSASSINATION_FAILED")
                end)
        end)

    :State("ASSASSINATION_FAILED")
        :Loc{
            DIAL_ASSASSINATION_FAILED = [[
            * You had your chance, and you let it get away. {handler} might've tolerated some failure in the past, but {handler.heshe}'s sure made it clear this job mattered.
                player:
                    !left
                handler:
                    !right
                    !angry_shrug
                    What the Hesh, Sal. You had one job.
                player:
                    !angry_accuse
                    If it were so easy, why didn't you do it yourself?
                handler:
                    Because I pay others to keep my hands clean. And you're supposed to provide a service.
                    !facepalm
                    I don't have the time or energy to be disappointed in you. Just... go inside already. I need to clean up your mess.
            ]],
        }
        :Fn(function(cxt)
            cxt:TalkTo(cxt.quest:GetCastMember("handler"))
            cxt.quest:GetCastMember("handler"):OpinionEvent(OPINION.JOB_FAILED_MINOR, {quest = cxt.quest})
            cxt:Dialog("DIAL_ASSASSINATION_FAILED")
            cxt.quest:Fail()
            StateGraphUtil.AddEndOption(cxt)
        end)


    :State("ENTER_AUCTION")
        :Loc{
            DIAL_DONE_GUARDING = [[
            * You can't help but feel a sense of pride for a job well done. When {handler} returns to relieve you, {handler.heshe} has a hint of a satisfied smile tugging at {handler.hisher} poker face.
                player:
                    !left
                handler:
                    !right
                    Well done, Sal. Consider this the end of your contract. You do good work, so I'm honestly sorry to see you go.
                    Here, take this for your troubles.
                    !give
                player:
                    !take
                    Another present for your favorite recruit?
                handler:
                    !wink
                    You know it. It's a requisition chit. We get a stack from the Cult of Hesh whenever the Admiralty is playing nice.
                    You can use it to exchange for one item on the block. You know, in the pursuit of duty.
                player:
                    !interest
                    Well... isn't that a nice surprise.
                handler:
                    !chuckle
                    They do happen, occasionally. 
            ]],
        }
        :Fn(function(cxt)
            cxt:TalkTo(cxt.quest:GetCastMember("handler"))
            cxt.quest:GetCastMember("handler"):OpinionEvent(OPINION.JOB_MINOR, {quest = cxt.quest})
            cxt:Dialog("DIAL_DONE_GUARDING")
            cxt.quest.param.cross_quest_data.requisition = true
            cxt:End()
            UIHelpers.DoSpecificConvo( cxt.quest:GetCastMember("handler"), "SAL_STORY_OOLO_CONVOS", "STATE_GET_CHIT",nil,nil,cxt.quest)
        end)


QDEF:AddTest(TestCreator()
    :AddChoiceSet{"PROVOKE", "WARN", "LET_PASS"} --does one of each
    :AddParam("set_target_relationship", {"FRIEND", "NEUTRAL", "ENEMY"})
    :AddNegotiationDimension()
    :AddBattleDimension())


QDEF:AddTest(TestCreator()
    :AddParam("set_target_relationship", {"FRIEND", "NEUTRAL", "ENEMY"})
    :SetChoices{"SEARCH"} --constant
    :AddNegotiationDimension()
    :AddBattleDimension())

QDEF:AddDebugOption("set_target_relationship", {"FRIEND", "NEUTRAL", "ENEMY"})




