local QDEF = QuestDef.Define{
        title = "Guard duty",
        qtype = QTYPE.STORY,
        icon = engine.asset.Texture("icons/quests/sal_story_auction_house_admiralty.tex"),
        on_init = function(quest) 
            quest.param.cross_quest_data = quest.param.cross_quest_data or {}
        end,
    }

    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_ADMIRALTY_CONTACT",
        no_validation = true,
    }

    :AddCast{
        cast_id = "other_guard",
        cast_fn = function(quest, t) 
            table.insert(t, quest.param.cross_quest_data and quest.param.cross_quest_data.other_guard or AgentUtil.GetFreeAgent("ADMIRALTY_GUARD"))
        end,
    }

    --this is going to be someone who doesn't like you! And is high level! Hopefully.
    :AddCast{
        cast_id = "hater",
        condition = function(agent) return agent:GetRelationship() < RELATIONSHIP.NEUTRAL and agent:GetFactionID() ~= "ADMIRALTY" end,
        score_fn = function(agent, quest)
            local score = agent:GetCombatStrength()*2 + agent:GetRenown()
            if agent:GetRelationship() == RELATIONSHIP.HATED then
                score = score*2
            end
            return score
        end,
    }
    :AddCastFallback{
        cast_fn = function(quest, t)
            --fall back on some generated rando who hates admiralty, I guess.
            local agent = quest:CreateSkinnedAgent( "BANDIT_RAIDER" )
            table.insert( t, agent )
        end,
    }

    :AddObjective{
        id = "auction_guard_door",
        title = "Keep the riff-raff out of the auction",
        state = QSTATUS.ACTIVE,
        terminal = true,
    }


QDEF:AddConvo("auction_guard_door")
    :ConfrontState("HATER_ARRIVES")
        :Loc{
            OPT_NEGOTIATE = "Try to talk {hater.himher} down",
            OPT_FIGHT = "Crack some heads",
            OPT_ASK_GUARD_FOR_HELP = "Appeal to {other_guard} for help",

            DIAL_INTRO = [[
            * The job goes smoothly enough with {other_guard} on duty next to you. But you've never met the plan you couldn't wrinkle.
            player:
                !left
                !neutral_notepad
            * You're checking some big wig's name off the list when you look up to see {hater} shouldering {hater.hisher} way through the line.
                    !palm
                    Ugh, here we go.
                hater:
                    !right
                    !angry_accuse
                    Hey! Scumbag!
                player:
                    !dubious
                    I assume you mean me.
                hater:
                    !cagey
                    I don't see anyone else here who fits the description. 
                    You, on the other hand, are just a boil waiting to be popped.
                player:
                    !angry
                other_guard:
                    !right
                    You causin' trouble, Sal? {handler} won't like that. 
            * And {handler}'s bound to find out if you don't handle this quick—{hater}'s angry spittle has attracted attention you can't afford.
                hater:
                    !right
                    I've had enough of you, coming to Havaria and acting already like you own the place.
                    We're gonna settle this. No one will help you here.
                player:
                    !sigh
                    It's always something.
            ]],

            DIAL_PRE_NEGOTIATE = [[
                player:
                    !placate
                    $miscPersuasive
                    Look, you really don't wanna do this.
                hater:
                    !cruel
                    $angryDefault
                    Oh, I'm pretty positive I do.
            ]],

            DIAL_NEGOTIATE_SUCCESS = [[
                player:
                    !thumb
                    I got friends here. And if you try to take me down, you'll have to face them, too.
                    !angry
                    Walk away, bud. Or you won't live to regret it.
                hater:
                    !angry
                    Y-yeah?! Well—
                player:
                    !angry_accuse
                    Move it.
                hater:
                    !handwave
                    Fine. But don't think I hate you any less, just 'cause you might be right.
                    You better watch your back, hunter. 'Cause you haven't seen the last of me. 
                    !exit
            ]],

            DIAL_NEGOTIATE_FAIL = [[
                player:
                    !hips
                    You think you can mess with me? 
                hater:
                    I <i>know</i> I can. And it's gonna feel so damn good, too. 
                    !angry_accuse
                    Get ready to face the reaper, hunter. 
            ]],

            OPT_TEACH_A_LESSON = "Defend yourself",
            DIAL_PRE_FIGHT_ALLY = [[
                player:
                    $angrySeething
                    I warned you, I have friends here.
                hater:
                    !fight
                    Less talking! More bleeding!
            ]],
            DIAL_PRE_FIGHT_ALONE = [[
                player:
                    !sigh
                    Let's get this over with.
                hater:
                    See! Nobody likes you!
            ]],

            --so this is new. it conditionally builds the string based on the state of the game, without extra lua code. 
            -- { expression ? TRUE STRING | FALSE STRING}
            -- the false string is optional, and is more useful for quick inline asides.
            -- the expression syntax is kind of like lua. 
            --  It uses:  (), and, or, not 
            -- the variables in it are defined by the quest AND ALSO anything you can use in a quip.
            
            DIAL_WON_FIGHT = [[
                { dead ? 
                    * {hater} is dead.
                }

                { not dead ? 
                    hater:
                        !injured
                        Stop! Don't hurt me any more!
                    player:
                        Get out of here before I change my mind about killing you.
                    hater:
                        !exit
                    * {hater} beats feet. You hope you've seen the last of {hater.himher}.
                }
                
                { not got_help ?
                    other_guard:
                        !right
                        I guess you can take care of yourself.
                    player:
                        No thanks to you.
                }

                { dead and other_guard_dead ?
                    * So is your backup.
                }

                { not dead and other_guard_dead ?
                    * {other_guard} didn't survive the encounter. Oh well.
                }

                { got_help and not other_guard_dead ?
                    other_guard:
                        !right
                    player:
                        Thanks for the assist.
                    other_guard:
                        Any time, Sal.
                }
                
                right:
                    !exit
            ]],

            DIAL_ASK_FOR_HELP = [[
                other_guard:
                    !right
                player:
                    Can I get a little help here?
            ]],
            DIAL_ASK_FOR_HELP_SUCCESS = [[
                other_guard:
                    !sigh
                    Fine. If trouble starts, I've got your back.
                hater:
                    !right
                    Hey! I was talking to you!
            ]],
            DIAL_ASK_FOR_HELP_FAIL = [[
                other_guard:
                    !crossed
                    No way. I'm not paid to solve your problems.
                hater:
                    !right
                    Hey! I was talking to you!
            ]],

            GET_HELP_TT = "{other_guard} will help you deal with {hater}.",

        }
        :SetLooping()
        :Fn(function(cxt)
            
            if cxt:FirstLoop() then
                cxt.quest:GetCastMember("handler"):MoveToLimbo()
                cxt:Dialog("DIAL_INTRO")
                cxt:TalkTo("hater")

                local backup_party = "MERCENARY_BACKUP"
                if cxt:GetCastMember("hater"):GetFactionID() == "JAKES" then
                    backup_party = "JAKES_BACKUP"
                elseif cxt:GetCastMember("hater"):GetFactionID() == "BANDITS" then
                    backup_party = "BANDIT_PATROL_BACKUP"
                end

                cxt.quest.param.goons = CreateCombatBackup(cxt:GetCastMember("hater"), backup_party, 3 )
                
                if cxt:GetAgent():GetRelationship() >= RELATIONSHIP.NEUTRAL then
                    cxt:GetAgent():OpinionEvent(OPINION.HATE_WORKING_FOR_ADMIRALTY)
                end
            end

            
            local helpers =  {}
            if cxt.enc.scratch.got_help then
                table.insert( helpers, "other_guard" )
            end

            cxt:Opt("OPT_NEGOTIATE")
                :Robot("NEGOTIATE")
                :Dialog("DIAL_PRE_NEGOTIATE")
                :Negotiation{
                    flags = NEGOTIATION_FLAGS.NO_BYSTANDERS,
                    helpers = helpers,
                    on_success = function(cxt)
                        cxt:Dialog("DIAL_NEGOTIATE_SUCCESS")
                        cxt:GoTo("STATE_ENTER_AUCTION")
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIAL_NEGOTIATE_FAIL")
                    end,
                }

            local allies, noncombatants = {}, {}

            if cxt.enc.scratch.got_help then
                table.insert( allies, cxt.quest:GetCastMember("other_guard"))
            else
                table.insert( noncombatants, cxt.quest:GetCastMember("other_guard"))
            end

            cxt:Opt("OPT_TEACH_A_LESSON")
                :Robot("FIGHT")
                :Fn(function() cxt:Dialog(cxt.enc.scratch.got_help and "DIAL_PRE_FIGHT_ALLY" or "DIAL_PRE_FIGHT_ALONE") end)
                :Battle{    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE,
                            noncombatants = noncombatants, 
                            allies = allies
                        }
                    :OnWin()
                        :Fn(function() 
                            cxt.enc.scratch.other_guard_dead = cxt.quest:GetCastMember("other_guard"):IsDead() --used by the strings
                            cxt.enc.scratch.hater_dead = cxt.quest:GetCastMember("hater"):IsDead() --used by the strings
                            cxt:Dialog("DIAL_WON_FIGHT") 
                        end)
                        :GoTo("STATE_ENTER_AUCTION")

            if not cxt.enc.scratch_asked_for_help then
                cxt:Opt("OPT_ASK_GUARD_FOR_HELP")
                    :Dialog("DIAL_ASK_FOR_HELP")
                    :Fn(function() cxt.enc.scratch_asked_for_help = true end)
                    :Robot("GET_HELP")
                    :PostText( "GET_HELP_TT")
                    :Negotiation{
                        target_agent = cxt.quest:GetCastMember("other_guard"),
                        on_success = function(cxt)
                            cxt:Dialog("DIAL_ASK_FOR_HELP_SUCCESS")
                            cxt.enc.scratch.got_help = true
                        end,
                        on_fail = function(cxt)
                            cxt:Dialog("DIAL_ASK_FOR_HELP_FAIL")
                        end,
                    }
            end
        end)

    :State("STATE_ENTER_AUCTION")
        :Loc{
            DIAL_DONE_GUARDING = [[
            * It's not long before {handler} comes to relieve you from your shift.
                player:
                    !left
                    !salute 
                    Hey there, {handler}. Mission accomplished, it seems.
                handler:
                    !right
                    I take it things went well? I haven't heard any complaints, anyway.
                    So... a deal's a deal. You get into the auction. 
                    And here, for good luck.
                    !give
                player:
                    !take
                    A tip? You shouldn't have.
                handler:
                    !happy
                    It's a requisition chit. You can use it to seize one item from the auction block.
                    !wink
                    Consider it the Cult's gratitude as well as mine. 

            ]],
        }
        :Fn(function(cxt)
            
            cxt:Dialog("DIAL_DONE_GUARDING")
            cxt:TalkTo(cxt:GetCastMember("handler"))
            cxt.quest.param.cross_quest_data.requisition = true
            cxt:End()
            UIHelpers.DoSpecificConvo( cxt.quest:GetCastMember("handler"), "SAL_STORY_OOLO_CONVOS", "STATE_GET_CHIT",nil,nil,cxt.quest)

        end)


QDEF:AddTest(TestCreator()
    :AddChoices{"FIGHT"}
    :AddBattleDimension())

QDEF:AddTest(TestCreator()
    :AddChoices{"NEGOTIATE"}
    :AddNegotiationDimension())

QDEF:AddTest(TestCreator()
    :AddChoices{"GET_HELP"}
    :AddNegotiationDimension()
    :AddBattleDimension())
    
