local QDEF = QuestDef.Define{
        title = "Guard Duty",
        icon = engine.asset.Texture("icons/quests/sal_story_auction_house_admiralty.tex"),
        qtype = QTYPE.STORY,

        on_start = function(quest)
            
            local new_target 
            if quest.param.recast_smuggle_target == "BANDIT" then
                new_target = quest:CreateSkinnedAgent( "BANDIT_GOON" ) 
            elseif quest.param.recast_smuggle_target == "ADMIRALTY" then
                new_target = quest:CreateSkinnedAgent( "ADMIRALTY_GUARD" ) 
            elseif quest.param.recast_smuggle_target == "OTHER" then
                new_target = quest:CreateSkinnedAgent( "HEAVY_LABORER" ) 
            end
            if new_target then
                new_target.social_connections:SetRelationship(RELATIONSHIP.LIKED, TheGame:GetGameState():GetPlayerAgent())
                quest:UnassignCastMember("smuggle_target")                
                quest:AssignCastMember("smuggle_target", new_target)                
            end
            quest.param.cross_quest_data = quest.param.cross_quest_data or {}
        end,
    }

    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_ADMIRALTY_CONTACT",
        no_validation = true,
    }

    :AddCast{
        cast_id = "other_guard",
        
        cast_fn = function(quest, t) 
            if quest.param.cross_quest_data and quest.param.cross_quest_data.other_guard then
                table.insert(t, quest.param.cross_quest_data.other_guard)
            else
                table.insert(t, AgentUtil.GetFreeAgent("ADMIRALTY_GUARD"))
            end

        end,
    }

    :AddCast{
        cast_id = "smuggle_target",
        score_fn = function(agent, quest)
            local score = 1
            if agent:GetFactionID() == "BANDITS" then
                score = score + 10
            end
            if agent:GetRelationship() > RELATIONSHIP.NEUTRAL then
                score = score + 10
            end
            return score
        end,
        condition = function(agent)
            return not agent:HasActiveQuestMembership() and agent:GetRelationship() >= RELATIONSHIP.NEUTRAL
        end,
    }
    :AddCastFallback
    {
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "HEAVY_LABORER" ) )
        end,
    }

    :AddObjective{
        id = "sneak_in_friend",
        title = "Keep the riff-raff out of the auction",
        state = QSTATUS.ACTIVE,
        terminal = true,
    }
    :AddOpinionEvents{
        did_not_let_in = 
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Didn't let them into the auction",
        },
    }


QDEF:AddConvo("sneak_in_friend")
    :ConfrontState("smuggle_target")
        :Loc{
            DIALOG_INTRO_BANDIT = [[
                * You're not long on the job before you catch a suspicious figure slinking towards you, like a cat looking for a mouse hole.
                * It takes you a moment to realize it's {smuggle_target}—who looks thrilled to see you, and you don't need many guesses as to why.
                    player:
                        !left
                        !dubious
                    smuggle_target:
                        !right
                        $happyDefault
                        {player.name}?!
                        What are you doing in Admiralty blues, huh?
                        !surprise
                        $neutralDubious
                        You ain't gone legit, have you?
                    player:
                        !wink
                        $neutralJoke
                        Only when the mood strikes. 
                        !overthere
                        This is temporary. I'm doing a job, then it's back to my freelancing ways.
                    smuggle_target:
                        !thought
                        Well, heck, this works out perfect. I just wanna get inside.
                        But you and me, we have an understanding, right?
                        !thumb
                        So whaddya say? Can you get me in?
                    player:
                        !shrug
                        Sure, I don't see why not.
                    smuggle_target:
                        !chuckle
                        $happyDefault
                        Nice. I thought I was gonna have to knock a few heads just to get through the door.
                        You've really saved me a body count, pal.
                ]],
            DIALOG_INTRO_ADMIRALTY = [[
                * You're not long on the job before you're approached by another admiralty guard, looking you up and down with surprise.
                * It takes you a moment to realize it's {smuggle_target}, on account of the way they keep fidgeting in their uniform.
                    player:
                        !left
                        !dubious
                    smuggle_target:
                        !right
                        $happyDefault
                        {player.name}? Didn't expect to see you here.
                        When did <i>you</i> enlist?
                    player:
                        !wink
                        $neutralJoke
                        Maybe I just wanted to follow in your foot steps, eh, pal?
                        !overthere
                        This is temporary. I'm doing a job, then it's back to my freelancing ways.
                    smuggle_target:
                        !thought
                        Well, heck, this works out perfect. I just wanna get inside.
                        But I couldn't get on security detail—{handler} says I don't clean up nice, if you can believe it.
                        !thumb
                        So whaddya say? Can you get me in?
                    player:
                        !shrug
                        Sure, I don't see why not.
                    smuggle_target:
                        !chuckle
                        $happyDefault
                        Nice. I thought I was going to have to call in all the favors just to get through the door.
                        Fortunately it's just one, huh?
                ]],
            DIALOG_INTRO_OTHER = [[
                * You're not long on the job before you're approached by a shifty looking character, neck craning to peer through the auction doors.
                * It takes you a moment to realize it's {smuggle_target}, come to gawk.
                    player:
                        !left
                        !dubious
                    smuggle_target:
                        !right
                        $happyDefault
                        {player.name}? Hesh, someone said you'd be here, but I wasn't sure I'd be so lucky.
                    player:
                        !wink
                        But here we are. You need something, {smuggle_target}?
                    smuggle_target:
                        !agree
                        I need <i>in</i>, {player}. Into the auction. 
                        I can't afford nothing on the block but I got my own reasons.
                        !thumb
                        So whaddya say? Can you get me in?
                    player:
                        !shrug
                        Sure, I don't see why not.
                    smuggle_target:
                        !chuckle
                        $happyDefault
                        Nice. I thought I was going to have to call in all the favors just to get through the door.
                        Fortunately it's just one, huh?
                ]],
        }
        :Fn(function(cxt) 
            cxt.quest:GetCastMember("handler"):MoveToLimbo()
            cxt:TalkTo(cxt.quest:GetCastMember("smuggle_target"))
            if cxt.quest:GetCastMember("smuggle_target"):GetFactionID() == "BANDITS" then
                cxt:Dialog("DIALOG_INTRO_BANDIT")
            elseif cxt.quest:GetCastMember("smuggle_target"):GetFactionID() == "ADMIRALTY" then
                cxt:Dialog("DIALOG_INTRO_ADMIRALTY")
            else
                cxt:Dialog("DIALOG_INTRO_OTHER")
            end

            cxt:GoTo("CONVINCE_OTHER_GUARD")
        end)


    :State("CONVINCE_OTHER_GUARD")
        :Loc{
            DIALOG_INTRO = [[
                other_guard:
                    !right
                    !angry_accuse
                    Hold on, buddy.
                    $angryDefault
                    What are you trying to pull? Just 'cause you're on guard duty doesn't mean you get to override the guest list.
            ]],
            OPT_ATTACK_GUARD = "Attack {other_guard}",
            DIALOG_ATTACK_GUARD = [[
                player:
                    !left
                    !angry_threaten
                    $angryHostile
                    Maybe you should keep your eyes to yourself, huh?
                    Or else I might pluck them out for you.
                other_guard:
                    !right
                    !flinch
                    $scaredStammering
                    Wha—
                    !angry
                    $angryDefault
                    Gotta say, {player}, I didn't think you could be this dumb.
                    Guess you're the kind who likes to prove folks wrong.
            ]],

            DIALOG_POST_FIGHT_GUARD_ALIVE = [[
                other_guard:
                    !right
                    !injured
                    $angryDefault
                    You just made a big mistake, {player}. {handler} won't let this go for nothing.
            ]],
            DIALOG_POST_FIGHT_GUARD_DEAD = [[
                * {other_guard} lies dead at your feet. That's going to be hard to explain.
            ]],
            DIALOG_FOUGHT_FRIEND_SURVIVED = [[
                player:
                    $neutralResigned
                    Well, that didn't go as smooth as I'd have liked.
                smuggle_target:
                    !right
                    !embarrassed
                    No kiddin'. Sorry I caused you so much trouble.
                player:
                    $neutralWhatever
                    Enh. I made my choices.
                smuggle_target:
                    !happy
                    $happyDefault
                    And I'm grateful for it. Catch you on the flippitty-flip, Sal.
                    !exit
                * {smuggle_target} slips through the entrance.
            ]],

            OPT_CONVINCE = "Convince {other_guard} to look the other way",
            DIALOG_CONVINCE = [[
                player:
                    !interest
                    Come on, {other_guard}. What's the point of being guards if we can't flex our authority a little?
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    !interest
                    It's just <i>one friend</i>. 
                other_guard:
                    !faceplam
                    That's how it starts, sure. And then {smuggle_target.heshe} opens the back door and lets in half the Spree.
                    What am I, some kind of moron? 
                    !crossed
                    $neutralDirect
                    We have a job to do, and that's to keep people like {smuggle_target.himher} <i>out</i>.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !interest
                    $miscPersuasive
                    Come on, {other_guard}. What's the point of being guards if we can't flex our authority a little?
                other_guard:
                    !right
                    !thought
                    Enh, I suppose it can't do much harm. 
                    As long as {smuggle_target.heshe} doesn't cause too much trouble inside, it's probably all right.
                smuggle_target:
                    !right
                    $miscMocking
                    Me? Trouble? Perish the thinker! 
                    !wink
                    You can trust me, <i>officer</i>.
                    !handwave
                    $happyDefault
                    See you inside!
                * With a dry chuckle, {smuggle_target} slips through the door before either of you can change your mind.
            ]],
            OPT_DECLINE_FRIEND = "Tell {smuggle_target} you've changed your mind",
            DIALOG_DECLINE = [[
                smuggle_target:
                    !right
                player:
                    !placate
                    Sorry, {smuggle_target}. You heard {other_guard.himher}. 
                    I'd let you in if I could but the uniform says otherwise.
                smuggle_target:
                    !dubious
                    $neutralDubious
                    Sounds like quitter's talk to me. I thought you had more gumption than that.
                    !angry
                    $angryDefault
                    Still, I'll remember this next time <i>you</i> come asking <i>me</i> to work a miracle. 
                * With a shake of {smuggle_target.hisher} head, {smuggle_target} slips back into the crowd. 
            ]],

        }

        :SetLooping()
        :Fn(function(cxt)
            local friend = cxt.quest:GetCastMember("smuggle_target")
            
            if cxt:FirstLoop() then
                cxt:TalkTo(cxt.quest:GetCastMember("other_guard"))
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Robot("NEGOTIATE")
                :Negotiation{
                    on_success = function( cxt )
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        friend:OpinionEvent(OPINION.HELPED)
                        
                        cxt.quest.param.cross_quest_data.friend_inside = friend
                        friend:MoveToLimbo()

                        cxt.quest:SetParameter("helped_friend", true)
                        cxt:GoTo("STATE_ENTER_AUCTION")
                    end,
                    on_fail = function( cxt )
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                    end,
                }

            cxt:Opt("OPT_ATTACK_GUARD")
                :Robot("FIGHT")
                :Dialog("DIALOG_ATTACK_GUARD")
                :Battle{
                    flags = BATTLE_FLAGS.NO_FLEE,
                    allies = {"smuggle_target"},
                    enemies = {"other_guard"},
                    on_win = function( cxt )
                        cxt:Dialog(cxt.quest:GetCastMember("other_guard"):IsDead() and "DIALOG_POST_FIGHT_GUARD_DEAD" or "DIALOG_POST_FIGHT_GUARD_ALIVE")
                        cxt.quest:SetParameter("fought_guard", true)
                        
                        if not friend:IsDead() then
                            cxt:Dialog("DIALOG_FOUGHT_FRIEND_SURVIVED")
                            friend:OpinionEvent(OPINION.HELPED)
                            cxt.quest.param.cross_quest_data.friend_inside = friend
                            friend:MoveToLimbo()
                            
                            cxt.quest:SetParameter("helped_friend", true)
                        end
                        cxt:GoTo("STATE_ENTER_AUCTION")
                    end,
                }

            cxt:Opt("OPT_DECLINE_FRIEND")
                :Robot("DECLINE")
                :Dialog("DIALOG_DECLINE")
                :ReceiveOpinion("did_not_let_in", nil, "smuggle_target")
                :Fn(function(cxt)
                    cxt.quest.param.cross_quest_data.friend_outside = friend
                    cxt:GoTo("STATE_ENTER_AUCTION")
                end)
        end)

    :State("STATE_ENTER_AUCTION")
        :Loc{
            DIALOG_DONE_GUARDING_NO_SMUGGLE = [[
            * The rest of the shift proves short. The guest list is still only a quarter checked off when {handler} cuts back through the crowd.
                player:
                    !left
                handler:
                    !right
                    Well, seems your time is up. No problems?
                player:
                    !shrug
                    $neutralWhatever
                    None worth reporting.
                handler:
                    !happy
                    $happyDefault
                    Good enough for me. All right, you earned your way in. Enjoy the auction. 
                    And for what it's worth, you've earned a little something extra.
                    !give
                    Take this requisition chit. The Cult lets us claim a limited number of items in exchange for our cooperation. 
                    One of those items can now be yours. 
                player:
                    !take
                    $happyDefault
                    Huh. Praise Hesh, I guess.
                handler:    
                    Amazing what a windfall will do for the ol' conviction, right? 
                * With a last nod to {handler}, you head inside.
                    
            ]],

            DIALOG_SMUGGLED_FOUGHT_GUARD_DEAD = [[
                * The rest of the shift proves short—if only because you left a stinkin' corpse just sitting in the street. It's not long before {handler} comes to investigate.
                handler:
                    !right
                    $neutralDirect
                    {player}. Do you wanna tell me what I'm looking at?
                player:
                    I wanna say a spleen? Don't reckon I actually know what a spleen looks like, though.
                    !shrug
                    Look, {other_guard.heshe} had it coming. Tried to sneak one of {other_guard.hisher} Spree friends inside, wanted me to look the other way.
                    I said "No way—that's not what we were hired for."
                    One thing led to another, and...
                handler:
                    !dubious
                    $neutralDubious
                    And you've gone ahead and made a little entryway potpourri for the guests.
                player:
                    That's a nice way of looking at it.
                handler:
                    !disappoint
                    $neutralResigned
                    I can't argue with your results, {player}. But my bosses sure wish your tactics weren't so indiscreet.
                    Anyway, a deal's a deal. You did your job, so now you get to go inside. 
                    I'll stay out here and take care of <i>this</i>.
                    !exit
                * You saunter inside, only out of place 'cause of the blood stains on your uniform.
            ]],

            DIALOG_SMUGGLED_FOUGHT_GUARD_ALIVE = [[
                * The rest of the shift proves short—but the sight of {handler}'s face when {handler.heshe} comes to relieve you makes you wish it could've lasted a little bit longer.
                handler:
                    !right
                player:
                    !left
                    !embarrassed
                    How's it going, {handler}?
                handler:
                    !angry
                    $angryDefault
                    You tell me, Sal. Word is you've been doing less <i>standing guard</i> and more <i>helping yourself</i>.
                player:
                    !placate
                    Well, hey, that's just hearsay, if you ask me.
                handler:
                    $angryDefault
                    I'm not asking you. I'm having second thoughts about having asked you for <i>anything</i>.
                    !disappoint
                    $neutralResigned
                    Still, no use regretting my decisions now. A deal's a deal. And a hunter's a hunter.
                    Go do what you needed to do. And make sure I don't regret <i>this</i>, at least.
                    !exit
                * You make your way into the auction, trying to ignore the bitter sigh {handler} sends after you.
            ]],

            DIALOG_SMUGGLED_NEGOTIATED = [[
                * By the end of the shift, you and {other_guard} have resorted to exchanging dry commentary on the ridiculous outfits of the auction guests.
                * When {handler} arrives, you're mid chuckle.
                handler:
                    !right
                    $miscMocking
                    Glad to see you two are <i>chums</i>. I've heard no complaints from the guests so I guess you did good work.
                player:
                    I live to serve. Or that's what the Cult kept telling me back on the derrick, anyway.
                handler:
                    Well, not anymore. A deal's a deal. You have your ticket into the auction.
                    $neutralJoke
                    Good luck in there. Get who you need—but try not to make too much of a mess, all right?
                player:
                    $miscMischievous
                    Me? Never a trace. Take care, {handler}. 
                * You duck through the entrance into the darkened foyer and quickly slip into the shadows. You may be allowed, but that doesn't mean you should make yourself known. 

            ]],
        }
        :Fn(function(cxt)
            cxt.quest:GetCastMember("handler"):MoveToLocation(cxt.location)

            if cxt.quest:GetParameter("helped_friend") then
                if cxt.quest:GetCastMember("other_guard"):IsDead() then
                    cxt:Dialog("DIALOG_SMUGGLED_FOUGHT_GUARD_DEAD")
                else
                    if cxt.quest:GetParameter("fought_guard") then
                        cxt:Dialog("DIALOG_SMUGGLED_FOUGHT_GUARD_ALIVE")
                    else
                        cxt:Dialog("DIALOG_SMUGGLED_NEGOTIATED")
                    end
                end
                StateGraphUtil.AddEndOption(cxt)
                cxt.quest:Complete()
            else
                cxt:Dialog("DIALOG_DONE_GUARDING_NO_SMUGGLE")
                cxt.quest.param.cross_quest_data.requisition = true
                cxt:End()
                UIHelpers.DoSpecificConvo( cxt.quest:GetCastMember("handler"), "SAL_STORY_OOLO_CONVOS", "STATE_GET_CHIT",nil,nil,cxt.quest)

            end
        end)


QDEF:AddTest(TestCreator()
    :AddChoices{"FIGHT", "NEGOTIATE", "DECLINE"}
    :AddParam("recast_smuggle_target", {"BANDIT", "ADMIRALTY", "OTHER"}))

QDEF:AddDebugOption("recast_smuggle_target", {"BANDIT", "ADMIRALTY", "OTHER"})
