local AUCTION_DATA = require "content/grafts/kashio_boss_fight_defs"
require "content/quests/story/Sal/Finale/auction_util"

local QDEF = QuestDef.Define{
        qtype = QTYPE.STORY,
        rank = 5,
        on_init = function(quest)

            if quest.param.cross_quest_data then
                quest.param.has_key = quest.param.cross_quest_data.has_key
                quest.param.has_lockmelter = quest.param.cross_quest_data.has_lockmelter
            end

            quest.param.cross_quest_data = quest.param.cross_quest_data or {}
            quest.param.cross_quest_data.auction_lots = quest.param.cross_quest_data.auction_lots or AUCTION_DATA.PickAuctionLots(2, 3)


            if not quest.param.story_beat then
                quest.param.story_beat = table.arraypick{"arrest", "ambush"}
            end

        end,

    }
    :AddLocationDefs{
        BACK_ROOM =
        {
            name = "Back Room",
            indoors = true,
            no_exit = true,
            plax = "INT_Auction_Backroom_1",
        },
    }

    :AddObjective{
        id = "back_room",
        state = QSTATUS.ACTIVE,
    }



QDEF:AddConvo("back_room")
    :ConfrontState("INTRO")
        :Loc{
            DIALOG_INTRO = [[
                * You emerge into a dusty back room of the auction house. It seems as though you've stumbled across the storage room for the auction lots. 
                player:
                    !left

                {has_key ? 
                    * You fidget with the small key. There's no indication which box it will fit into, but you have a feeling it will fit into one of them.
                }
                
                {has_lockmelter ? 
                    * You pull out the lockmelter. You have your pick of the artifacts, it seems.
                }

                {not has_key and not has_lockmelter ? 
                    * The lots are all locked in strong boxes, and you can hear the crowd through the door already settling down for the start of the auction.
                    * There's no way you can open them up without attracting attention.
                }
            ]],

            OPT_FIND_LOCK = "Find the lock for the key",
            DIALOG_FIND_LOCK = [[
                * You frantically try the lock in the various boxes. Just as you are about to give up, you find the one that fits!
            ]],
            DIALOG_FIND_GRAFT = [[ 
                * Inside, you find a {1#graft}.
            ]],
            DIALOG_FIND_ITEM = [[ 
                * Inside, you find a {1#card}.
            ]],
            OPT_INSPECT_THE_LOTS = "Inspect the lots",
            DIALOG_INSPECT_THE_LOTS = [[
                * You inspect the locked boxes, reading the labels.
                * The lockmelter will only open one of them, so you need to pick carefully.
            ]],
            OPT_CONTINUE = "Continue to the auction",
            
            DIALOG_CONTINUE = [[
                player:
                    !exit
                * You carefully open the door and slip into the auction.
            ]],
            
            OPT_TAKE_ITEM = "Take {1#card}",
            OPT_TAKE_GRAFT = "Take {1#graft}",
            
            OPT_SMASH_ITEM = "Smash it",
            OPT_SMASH_GRAFT = "Smash it",

            DIALOG_SMASH_GRAFT = [[
                * You smash the graft on the table, and stash the pieces back in the box.
            ]],
            DIALOG_TAKE_GRAFT = [[
                * Luckily, nobody heard that.
            ]],

            DIALOG_SMASH_ITEM = [[
                * You smash the {1#card} on the table, and stash the pieces back in the box.
            ]],
            DIALOG_TAKE_ITEM = [[
                * The thought of using this on Kashio brings a smile to your face.
            ]],

            DIALOG_OPEN_BOX = [[
                player:
                    !take
                * The lockmelter sizzles as it opens the box, and you inspect the contents.
            ]],

            OPT_GRAFT_BOX = "Contents: Graft '{1}'",
            OPT_ITEM_BOX = "Contents: Item '{1}'",
        }
        :Fn(function(cxt) 
            
            local after_fn = function() 
                cxt:Dialog("DIALOG_CONTINUE")
                cxt.quest:Complete()
                cxt:End()
            end


            cxt.encounter:DoLocationTransition( cxt.quest:SpawnTempLocation("BACK_ROOM") ) 
            cxt:Dialog("DIALOG_INTRO")

            if cxt.quest.param.has_key then
                cxt:Opt("OPT_FIND_LOCK")
                    :Dialog("DIALOG_FIND_LOCK")
                    :Fn(function() 
                        if math.random() < .5 then
                            --get a graft
                            local graft_id = table.remove( cxt.quest.param.cross_quest_data.auction_lots.grafts, math.random(#cxt.quest.param.cross_quest_data.auction_lots.grafts))
                            local graft = GraftInstance(graft_id)
                            cxt:Dialog("DIALOG_FIND_GRAFT", graft_id)
                            AuctionUtil.HandleGraft(cxt, cxt.quest.param.cross_quest_data.auction_lots, graft, after_fn)                        
                        else
                            --get an item
                            local item_id = table.remove( cxt.quest.param.cross_quest_data.auction_lots.items, math.random(#cxt.quest.param.cross_quest_data.auction_lots.items))
                            cxt:Dialog("DIALOG_FIND_ITEM", item_id)
                            AuctionUtil.HandleItem(cxt, cxt.quest.param.cross_quest_data.auction_lots, item_id, after_fn)

                        end

                    end)
            end

            if cxt.quest.param.has_lockmelter then
                cxt:Opt("OPT_INSPECT_THE_LOTS")
                    :Dialog("DIALOG_INSPECT_THE_LOTS")
                    :Fn(function() 
                        
                        AuctionUtil.PresentLots(cxt, cxt.quest.param.cross_quest_data.auction_lots, after_fn)

                        cxt:Opt("OPT_CONTINUE")
                            :Dialog("DIALOG_CONTINUE")
                            :CompleteQuest()
                            :End()
                    end)
                    
            end

            cxt:Opt("OPT_CONTINUE")
                :Dialog("DIALOG_CONTINUE")
                :CompleteQuest()
                :End()


        end)


QDEF:AddDebugOption("has_lockmelter", {true, false})
    :AddDebugOption("has_key", {true, false})
    




