local QDEF = QuestDef.Define{
        title = "Back Door Plan",
        desc = "{handler} is waiting for you at the secret back entrance to the auction.",
        qtype = QTYPE.STORY,
        rank = 5,
        icon = engine.asset.Texture( "icons/quests/sal_story_auction_house_spree.tex" ),
        is_in_hiding = function(quest, agent)
            if agent == quest:GetCastMember("handler") then
                return true
            end
        end,

        on_init = function(quest)
            quest.param.cross_quest_data = quest.param.cross_quest_data or {}
            if not quest.param.story_beat then
                quest.param.story_beat = table.arraypick{"arrest", "ambush"}
            end

            if quest.param.set_escort_relationship then
                quest:GetCastMember("bandit_escort").social_connections:SetRelationship(quest.param.set_escort_relationship, TheGame:GetGameState():GetPlayerAgent())
            end

            if quest.param.set_nadan_relationship then
                quest:GetCastMember("handler").social_connections:SetRelationship(quest.param.set_nadan_relationship, TheGame:GetGameState():GetPlayerAgent())
            end

        end,

    }

    :AddOpinionEvents{
        abandoned = 
        {
            delta = OPINION_DELTAS.MAJOR_BAD,
            txt = "Abandoned them to the Admiralty",
        },
        rescued = 
        {
            delta = OPINION_DELTAS.MAJOR_GOOD,
            txt = "Rescued them from the Admiralty",
        },
        betrayed_you = 
        {
            delta = OPINION_DELTAS.BAD,
            txt = "Used you as bait to get into the auction",
        }
    }


    :AddLocationDefs{
        ENTRANCE = {
            name = "Hidden Entrance",
            plax = "EXT_Forest_1",
            --plax = "INT_AuctionSpreeEntrance_1",
        },
        TUNNEL = {
            name = "Secret Access Tunnel",
            indoors = true,
            plax = "INT_AuctionSpreeEntrance_1",
        },
        BACK_ROOM =
        {
            name = "Back Room",
            indoors = true,
            no_exit = true,
            plax = "INT_Auction_Backroom_1",
        },
    }

    :AddLocationCast{
        cast_id = "entrance",
        cast_fn = function(quest, t)
            table.insert(t,  quest:SpawnTempLocation("ENTRANCE"))
        end,
    }

    :AddCastByAlias{
        cast_id = "handler",
        alias = "MURDER_BAY_BANDIT_CONTACT",
        no_validation = true,
    }

    :AddCast{
        cast_id = "bandit_escort",
        condition = function(agent, quest)
            return agent:GetContentID() == "BANDIT_RAIDER"
        end,
        score_fn = function(agent, quest)
            if agent:GetRelationship() ~= RELATIONSHIP.NEUTRAL then
                return 10
            end
            return 1
        end,
    }
    :AddCastFallback{
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( "BANDIT_RAIDER" ) )
        end,
    }

    :AddObjective{
        id = "entrance_encounter",
        title = "Meet {handler} to get access to the auction",
        mark = {"entrance"},
        state = QSTATUS.ACTIVE,
    }

QDEF:AddConvo("entrance_encounter")
    :ConfrontState("ARREST_INTRO", function(cxt) return cxt.location == cxt.quest:GetCastMember("entrance") and cxt.quest.param.story_beat == "arrest" end)
        :Loc{
            DIAL_ARRIVE = [[
                bandit_escort:
                    !right
                { liked ? 
                    * You find your friend {bandit_escort} waiting at the meeting spot.
                    player:
                        !left                    
                    bandit_escort:
                        !happy
                        Sal! Good to see you. Nadan's waiting for you inside.
                    player:
                        !dubious
                        Inside of what?
                    bandit_escort:
                        This way!
                        !exit
                }
                { disliked ? 
                    * {bandit_escort} is waiting at the meeting spot. {bandit_escort.HeShe} groans when {bandit_escort.heshe} sees you approaching.
                    player:
                        !left                    
                    bandit_escort:
                        !spit
                        Grifter.
                    player:
                        The feeling's mutual, pal. Where's your boss?
                    bandit_escort:
                        !angry
                        This way.
                        !exit
                }
                { neutral ?
                    * {bandit_escort} is nervously waiting for you at the meeting spot. {bandit_escort.HisHer} hand is never far from {bandit_escort.hisher} weapon as you talk.
                    player:
                        !left                    
                    bandit_escort:
                        You must be the grifter that {handler} was talking about.
                    player:
                        Where <i>is</> {handler}?
                    bandit_escort:
                        This way.
                        !exit
                }
                * {bandit_escort} reaches into the underbrush and fiddles with something. A hidden panel slowly slides open in a nearby rockface, exposing an entrance to an underground tunnel.
                bandit_escort:
                    !right
                    !overthere
                    Follow me. And watch your step.
            ]],

            OPT_ENTER_TUNNEL = "Enter the tunnel",
            DIALOG_ENTER_TUNNEL = [[
                bandit_escort:
                    !exit
                * You weigh your options, and decide that a shot at Kashio is worth the chance of an ambush.
                
                player:
                    !shrug
                    Here goes.
                    !exit
            ]]
        }
        :Fn(function(cxt)
            cxt:TalkTo("bandit_escort")
            cxt:Dialog("DIAL_ARRIVE")
            cxt:Opt("OPT_ENTER_TUNNEL")
                :Dialog("DIALOG_ENTER_TUNNEL")
                :Fn(function() cxt.encounter:DoLocationTransition(  cxt.quest:SpawnTempLocation("TUNNEL") ) end )
                :GoTo("STATE_IN_TUNNEL")
        end)

    :State("STATE_IN_TUNNEL")
        :Loc{
            DIALOG_INTRO = [[
                left:
                    !exit
                right:
                    !exit

                * You follow {bandit_escort} through the entrance. As you go deeper underground, the sound of your footfalls is echoed by shouting up ahead. 
                * {bandit_escort} grips {bandit_escort.hisher} weapon tightly and motions for you to be quiet. The two of you peer into the opening of a large underground chamber.
                guard_1:
                    !left
                handler:
                    !right
                    !injured
                    Let me go, and you might survive.
                guard_1:
                    !left
                    !throatcut
                    Keep flapping those lips, bandit, and you'll see what happens to you.
                    !happy
                    Imagine our luck, happening upon the leader of the Heshin' Spree waiting all by his lonesome.
                guard_2:
                    !right
                    !happy
                    We're getting promotions for this!
                guard_1:
                    !thumb
                    $neutralWhatever
                    You can keep the promotions for yourself! I just want enough shills that I don't have to wear this uniform any more.
                    !exit
                guard_2:
                    !exit
                * {bandit_escort} turns to you.
                player:
                    !left
                bandit_escort:
                    !right
                    !scared
                    $miscHushedUrgent
                    The Admiralty's on to us! We have to get out of here!
            ]],

            DIALOG_CONVINCE_HELP = [[
                player:
                    $miscHushedUrgent
                    We can't just bail! We gotta help {handler}.
            ]],
            DIALOG_NEG_SUCCESS = [[
                bandit_escort:
                    !sigh 
                    $angryReluctant
                    You're right. We can do this!
                * Good thing you got {bandit_escort} on side, because your heated debate just gave you away.
                guard_1:
                    !right
                    !angryPatienceLost
                    Who's there?
            ]],
            DIALOG_NEG_FAILURE = [[
                bandit_escort:
                    !angry_shrug
                    $miscHushedUrgent
                    I'm not getting myself killed for this!
                    !exit
                * {bandit_escort} runs noisily back down the tunnel, leaving you the parting gift of being found out.
                guard_1:
                    !right
                    !fight
                    What's this, then? More Spree for us to arrest?
            ]],
            DIALOG_CHARGE_IN = [[
                player:
                    !angry_shrug
                    $angryAside
                    Fine, ya baby. Hold your breath for all I care.
                    !exit
                guard_1:
                    !exit
                * You charge into the chamber, catching the Admiralty patrol by surprise.
                player:
                    !left
                    !fight
                guard_1:
                    !right
                    !surprised
                    Wha?!
            ]],
            DIALOG_SNEAK_PAST = [[
                player:
                    $miscHushedUrgent
                    Agreed. Let's stick to the shadows. 
                bandit_escort:
                    !exit
                * {bandit_escort} doesn't heed your advice, panicking and breaking into a sprint. 
                player:
                    !wince_critical
                * They cut {bandit_escort.himher} down in a hail of blasterfire.
            ]],
            OPT_CONVINCE_HELP = "Convince {bandit_escort} to help fight the Admiralty",
            OPT_CHARGE_IN = "Charge in without {bandit_escort}",
            OPT_SNEAK_PAST = "Try to sneak past the Admiralty",
        }
        :Fn(function(cxt) 
            cxt.quest.param.admiralty_patrol = CreateCombatParty("ADMIRALTY_PATROL", 5, cxt.location)
            cxt:ReassignCastMember('guard_1', cxt.quest.param.admiralty_patrol[1])
            cxt:ReassignCastMember('guard_2', cxt.quest.param.admiralty_patrol[2])
            cxt:TalkTo('bandit_escort')
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_CONVINCE_HELP")
                :Robot("CONVINCE")
                :Dialog("DIALOG_CONVINCE_HELP")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_NEG_SUCCESS")
                        :Fn(function() cxt.quest.param.escort_fought = true end)
                        :GoTo("STATE_FIGHT_ADMIRALTY")
                    :OnFailure()
                        :Dialog("DIALOG_NEG_FAILURE")
                        :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                        :GoTo("STATE_FIGHT_ADMIRALTY")

            cxt:Opt("OPT_CHARGE_IN")
                :Robot("CHARGE")
                :Dialog("DIALOG_CHARGE_IN")
                :Fn(function() cxt:GetAgent():MoveToLimbo() cxt.quest.param.charge_bonus = true end)
                :GoTo("STATE_FIGHT_ADMIRALTY")

            cxt:Opt("OPT_SNEAK_PAST")
                :Robot("SNEAK")                
                :Dialog("DIALOG_SNEAK_PAST")
                :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                :GoTo("STATE_SNEAK_PAST")

        end)

    :State("STATE_FIGHT_ADMIRALTY")
        :Loc{
            OPT_DEFEND = "Defend yourself",
            OPT_ATTACK = "Attack!",
            DIALOG_ATTACK = [[
                player:
                    I've got you, {handler}!
            ]],
            DIALOG_DEFEND = [[
                right:
                    !angry_accuse
                    $angryDefault
                    More Spree. Kill them!
            ]]
        }
        :Fn(function(cxt)
            local attack = cxt.quest.param.charge_bonus
            
            local allies = {}
            if cxt.quest.param.escort_fought then
                table.insert(allies, cxt.quest:GetCastMember("bandit_escort"))
            end

            cxt:Opt(attack and "OPT_ATTACK" or "OPT_DEFEND")
                :Dialog(attack and "DIALOG_ATTACK" or "DIALOG_DEFEND")
                :Battle{
                    allies = allies,
                    no_oppo_limit = true,
                    enemies = cxt.quest.param.admiralty_patrol,
                    bystanders = {cxt.quest:GetCastMember("handler")},
                    advantage = attack and TEAM.BLUE,
                    flags = attack and BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED or (BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED)
                }
                    :OnWin()
                        :GoTo("STATE_POST_BATTLE")
        end)


    :State("STATE_SNEAK_PAST")
        :Loc{
            DIALOG_INTRO = [[
                * You take the opportunity to skulk through the chamber.
                player:
                    !exit
                * You're half-way through when {handler} notices you.
                player:
                    !left
                handler:
                    !right
                    !injured
                    Sal! You made it. Deal with these goons, and we'll continue on to the auction.
                player:
                    !bashful
                    Uh...
                * {handler}'s not in a state to move quickly or help in a fight. 
                * The guards are busy inspecting their handiwork, but they'll be back momentarily. You <i>could</> get the jump on them, if you acted first.
            ]],
            OPT_ATTACK = "Attack the guards",
            OPT_ABANDON_NADAN = "Leave {handler} and head for the auction",
            DIALOG_ABANDON = [[
                player:
                    Sorry, {handler}, you're on your own.
                    !exit
                handler:
                    Never trust a grifter.
                * You rush through the tunnels, hoping that the Admiralty won't leave their prize to follow you.

            ]],
            DIALOG_ATTACK = [[
                guard_1:
                    !right
                    !fight
                    Kill the grifter!
                player:
                    !fight
            ]]
        }
        :Fn(function(cxt)
            cxt:TalkTo("handler")
            cxt:Dialog("DIALOG_INTRO") 

            cxt:Opt("OPT_ATTACK")
                :Robot("SNEAK_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    enemies = cxt.quest.param.admiralty_patrol,
                    bystanders = {cxt.quest:GetCastMember("handler")},
                    no_oppo_limit = true,
                    advantage = TEAM.BLUE,
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED
                }
                    :OnWin()
                        :GoTo("STATE_POST_BATTLE")
            
            cxt:Opt("OPT_ABANDON_NADAN")
                :Robot("SNEAK_ABANDON")
                :Dialog("DIALOG_ABANDON")
                :ReceiveOpinion("abandoned")
                :Fn(function()
                    StateGraphUtil.AddEndOption(cxt)
                        :CompleteQuest()
                end)
        end)
    :State("STATE_POST_BATTLE")
        :Loc{
            DIALOG_WON_BATTLE = [[
                right:
                    !exit
                
                {survivors ? 
                    * The surviving guards lock eyes with you, and start backing out of the chamber.
                    * {handler}, having freed {handler.himher}self from {handler.hisher} restraints, dispatches them from behind with a single blow.
                    handler:
                        !right
                        Can't have them raising the alarm, now can we?
                }
                
                {not survivors ? 
                    * {handler} wriggles free from {handler.hisher} restraints, and inspects the Admiralty corpses.
                    handler:
                        !right
                        Good work, Sal. Dead switches can't give us away.
                }

                {not escort_fought ?
                    
                    * {escort_dead ? {handler} kicks {bandit_escort}'s body. | {handler} stares off in the direction that {bandit_escort} fled. }

                    handler:
                        If only all of my minions were as loyal as you.
                }

                {escort_fought and escort_dead ? 

                    handler:
                        Shame about {bandit_escort}. {bandit_escort.HeShe} was a good Spree, up to the end.
                    * You don't bother to correct {handler.himher}.
                }

                {escort_fought and not escort_dead ? 
                    handler:
                        $neutralDirect
                        !point
                        {bandit_escort}, go secure the entrance. Make sure no more switches surprise us down here.
                    bandit_escort:
                        !left
                        !salute
                        $neutralDirect
                        Yes sir!
                        !exit
                    * {bandit_escort} slinks away, clearly wondering if {handler} witnessed {bandit_escort.hisher} moment of cowardice.
                    player:
                        !left
                    handler:
                        $neutralThoughtful
                        You can pay them, but you can never really count on them, eh?
                        !happy
                        I could learn a thing or two about persuasiveness from you.
                }
            ]],
            DIALOG_ONWARD = [[
                player:
                    So, the auction?
                handler:
                    It's down that tunnel.
                player:
                    You're not coming?
                handler:
                    !disappoint
                    $neutralResigned
                    Not this way. If the Admiralty knows about it, I have some tracks to cover.
                    But you stuck your neck out for me, and I'm not ungrateful. Here, take this.
                    !give
                player:
                    !take
                handler:
                    It's a lockmelter. Might come in handy in there.
                    I'll find my own way in—I intend to see the look on Kashio's face when you deal with her. 
                player:
                    $neutralThanks
                    Thanks. I'll remember that.
                handler:
                    !crossed
                    Damn right you will. If you survive your deathmatch, I expect a kickback.
                    $neutralDirect
                    Now get going, before all this blabbing gives us away.

            ]]

        }
        :Fn(function(cxt) 
            local survivors = ConvoUtil.CollectSurvivors(cxt.quest.param.admiralty_patrol)
            cxt.enc.scratch.survivors = #survivors > 0 
            cxt.enc.scratch.escort_dead = cxt:GetCastMember("bandit_escort"):IsDead()
            cxt:TalkTo(cxt.quest:GetCastMember("handler")) 

            cxt:Dialog("DIALOG_WON_BATTLE")
            cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("rescued"))
            for k,v in ipairs(survivors) do
                v:Kill()
            end

            cxt:Dialog("DIALOG_ONWARD")
            cxt.quest.param.cross_quest_data.has_lockmelter = true
            StateGraphUtil.AddEndOption(cxt)
                :CompleteQuest()
        end)

--------------------

QDEF:AddConvo("entrance_encounter")
    :ConfrontState("ARREST_INTRO", function(cxt) return cxt.location == cxt.quest:GetCastMember("entrance") and cxt.quest.param.story_beat == "ambush" end)
        :SetLooping(true)
        :Loc{
            DIALOG_INTRO = [[
                * It's quiet as you approach the meeting point, but your trained eyes spot Nadan crouching in the darkness, a finger pushed to his twisted lips.
                handler:
                    !right
                player:
                    !left
                    $miscHushedUrgent
                    We hiding from something?
                handler:
                    !overthere
                    $miscHushedUrgent
                    Admiralty ambush. On the far side. They're camouflaged but not good enough.
                    $miscHushedUrgent
                    Ain't no way through that's not through them. I need you to distract 'em—draw them out so I can get the drop on them.
            ]],

            OPT_ACCEPT = "Distract the ambush party",
            DIALOG_ACCEPT = [[
                player:
                    !agree
                    Sure. I'll distract 'em, you come in blades drawn. Sounds good. 
                handler:
                    !point
                    $neutralDirect
                    Be smart and careful. There's no prize for being the first one dead. 
                player:
                    !thumb
                    $neutralJoke
                    That's why they call me Sal "Smart & Careful" ik-Derrick.
                    !exit
                handler:
                    !exit
            ]],
            OPT_CONVINCE_NADAN = "Convince {handler} to distract them, instead",
            DIALOG_CONVINCE_NADAN = [[
                player:
                    !point
                    $miscHushedUrgent
                    Counterpoint: <i>you</i> distract 'em, and I'll be the cavalry.
                handler:
                    !crossed
                    $neutralAside
                    And why in Sog's name would I agree to that?

            ]],
            DIALOG_CONVINCE_NADAN_SUCCESS = [[
                player:
                    !shrug
                    $miscPersuasive
                    You're <i>the Scourge</i> of Murder Bay. That's way more distracting than me—Sal Nobody from the Derrick. 
                handler:
                    !angry_accuse
                    $angryAside
                    You're saying I got the face for it. But it's not a bad point.
                    $neutralDirect
                    Okay, fine. But get your blades ready—I put 'em to use quick.
                * Nadan slinks through the bushes until there's enough distance that he could make his entrance. And he does. Noisily. 
                    !exit
            ]],
            DIALOG_CONVINCE_NADAN_FAIL = [[
                player:
                    !scared_shrug
                    It sounds... hard?
                handler:
                    $angryPatienceLost
                    Don't make the mistake of thinking we're partners here, Sal. You still work for me. 
                    !angry_accuse
                    And I'm telling you to go distract them. So <b>do it</b>.
                    !exit
            ]],
            OPT_SUGGEST_OTHER_WAY = "Tell {handler} to go around the ambush", -- Just so people can get the dialog telling them there's no way
            DIALOG_REJECT_SUGGESTION = [[
                player:
                    !dubious
                    Can't we just... sneak past?
                handler:
                    !palm
                    They're right in front of the entrance. No way through that doesn't make mincemeat outta us both.
                    There's my way, or no way. Take your pick.
            ]],
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt.quest.param.admiralty_patrol = CreateCombatParty("ADMIRALTY_PATROL", 5, cxt.location, true)
                cxt:ReassignCastMember('guard_1', cxt.quest.param.admiralty_patrol[1])
                
                if cxt.quest.param.set_guard_relationship then
                    cxt.quest.param.admiralty_patrol[1].social_connections:SetRelationship(cxt.quest.param.set_guard_relationship, TheGame:GetGameState():GetPlayerAgent())
                end


                cxt:TalkTo("handler")
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_ACCEPT")
                :Robot("CHARGE_AMBUSH")                
                :Dialog("DIALOG_ACCEPT")
                :GoTo("ATTACK_THE_AMBUSH")

            cxt:Opt("OPT_CONVINCE_NADAN")
                :Dialog("DIALOG_CONVINCE_NADAN")
                :Robot("CONVINCE_NADAN")
                :Negotiation{
                    suppressed = {cxt:GetCastAgent("guard_1")},
                    on_success = function()
                        cxt:Dialog("DIALOG_CONVINCE_NADAN_SUCCESS")
                        cxt:GoTo("NADAN_BAIT")
                    end,
                    on_fail = function()
                        cxt:Dialog("DIALOG_CONVINCE_NADAN_FAIL")
                        cxt:GoTo("ATTACK_THE_AMBUSH")
                    end
                }

            if not cxt.enc.scratch.suggested_other_way then
                cxt:Opt("OPT_SUGGEST_OTHER_WAY")
                    :Dialog("DIALOG_REJECT_SUGGESTION")
                    :Fn(function() cxt.enc.scratch.suggested_other_way = true end)
            end
        end)

    :State("ATTACK_THE_AMBUSH")
        :Loc{
            DIALOG_AMBUSH = [[
                * You emerge from the shadows, pretending to be sneaky about it. You're joined by a smug officer soon after.
                player:
                    !left
                agent:
                    !right
                    Looks like our tip was right. Stop right there, grifter. You're in Admiralty territory, as you damn well know.
                player:
                    !placate
                    Ahh! Ooh, darn! You got me!
                    
                {liked ? 
                    player:
                        !dubious
                        Wait, is that you, {agent}?
                    agent:
                        !surprised
                        Sal?! What are you doing here? This is off-limits!
                }

                {disliked ?
                    player:
                        Oh Hesh, it's <i>you</>.
                    agent:
                        !cruel
                        Ha! I've been looking for a chance to rough you up.
                }

                {neutral ? 
                    agent:
                        !hips
                        Cut the act. You have some explaining to do.
                }
            ]],

            OPT_CONVINCE_FRIEND = "Convince {agent} to get out of here",
            DIALOG_CONVINCE_FRIEND = [[
                player:
                    !overthere
                    {agent}, buddy, the best thing for you right now is to round up your troops and walk away.
            ]],
            DIALOG_CONVINCE_FRIEND_SUCCESS = [[
                player:
                    !cagey
                    I'm here to stop a dangerous man. Nadan Undar.
                agent:
                    !scared
                    $scaredTakenAback
                    N-Nadan? The Scourge of Murder Bay?
                player:
                    !thought
                    $neutralAside
                    Wow. Didn't think anyone actually called him that. 
                    !agree
                    Word is, he wants into that auction—and he'll <i>kill</i> anyone in his path. 
                    !thumb
                    Let me save your neck. Get outta here and I promise... I'll be the one to stop him. 
                agent:
                    !tired
                    $scaredThanks
                    I trust you, Sal. Good luck and stay safe. 
                    !exit
                * With a snap of {agent.hisher} fingers, {agent} rounds up {agent.hisher} soldiers and make their way out of harm's way.
            ]],
            DIALOG_CONVINCE_FRIEND_FAIL = [[
                player:
                    !scared_shrug
                    $neutralWhatever
                    I'm having a party at the, uh, Grog n' Dog, and you're invited!
                agent: 
                    Uh, Sal? This is my job. I'm not going to a party. 
                handler:
                    !right 
                    Too bad. {agent.HeShe} would've been the life of it. 
                * With a serpent-like hiss, Nadan slips from the shadows and makes quick work of {agent}'s troops.
                agent:
                    !right
                    H-hey! Drop your weapon! Oh, Hesh!
                    !fight

            ]],
        }
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt:TalkTo("guard_1")
                cxt:Dialog("DIALOG_AMBUSH")
            end

            if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL then
                cxt:Opt("OPT_CONVINCE_FRIEND")
                    :Dialog("DIALOG_CONVINCE_FRIEND")
                    :Negotiation{
                        suppressed = {cxt:GetCastAgent("handler")},
                        on_success = function( )
                            cxt:Dialog("DIALOG_CONVINCE_FRIEND_SUCCESS")
                            cxt:GoTo("STATE_LET_BY")
                        end,
                        on_fail = function()
                            cxt:Dialog("DIALOG_CONVINCE_FRIEND_FAIL")
                            cxt.enc.scratch.failed_friend_negotiation = true
                            cxt:GoTo("STATE_DO_ATTACK")
                        end
                    }
                
            else
                cxt:GoTo("STATE_DO_ATTACK")
            end
        end)

    :State("STATE_DO_ATTACK")
        :Loc{
            DIALOG_NADAN_INTERRUPTION = [[
                player:
                    So, you see, everything's connected. Everything's in its place. And you? You're just in the wrong place at the right time.
                agent: 
                    What the Hesh are you talking about?
                * With a serpent-like hiss, Nadan slips from the shadows and eliminates {agent}'s troops.
                agent:
                    H-hey, what—
                    !scared
                player:
                    !fight
                * {handler} leaves {agent} to you.
            ]],
            DIALOG_ATTACK_ALONE = [[
                player:
                    So, you see, officer—
                * You stall, waiting for Nadan to jump in and finish off the guards, but he never does. 
                * Instead, you see the whisper of his shadow slink between the trees and into the hidden entrance... leaving you behind to deal with the guards.
                player:
                    !palm
                    Aw, sea spit. Fine!
                    !angry
                    Put 'em up, already, and let's get this over with!
                    !fight
                agent:
                    !fight
            ]],
            OPT_DEFEND_SELF = "Defend yourself",
            OPT_ATTACK = "Attack",
            DIALOG_ATTACK = [[
                player:
                    You're mine!
            ]],
            
            DIAL_FOLLOW_NADAN = [[
                * You see {handler}'s tracks leading to the secret entrance. It will be easy to follow {handler.himher}.
            ]],

        }
        :Fn(function(cxt) 
                if cxt.quest.param.nadan_betrays_you == nil then
                    local betray_odds = {
                        [RELATIONSHIP.HATED] = 1,
                        [RELATIONSHIP.DISLIKED] = .5,
                        [RELATIONSHIP.NEUTRAL] = .1,
                        [RELATIONSHIP.LIKED] = 0,
                        [RELATIONSHIP.LOVED] = 0,
                    }
                    cxt.quest.param.nadan_betrays_you = math.random() < betray_odds[cxt.quest:GetCastMember("handler"):GetRelationship()]
                end

                    
                if not cxt.enc.scratch.failed_friend_negotiation and cxt.quest.param.nadan_betrays_you then
                    cxt:Dialog("DIALOG_ATTACK_ALONE")
                    cxt.quest:GetCastMember("handler"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("betrayed_you"))
                    cxt:Opt("OPT_ATTACK")
                        :Battle{
                            flags = BATTLE_FLAGS.NO_BYSTANDERS| BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                            on_win = function()
                                cxt:Dialog("DIAL_FOLLOW_NADAN")
                                cxt:GoTo("STATE_BETRAYED_BY_NADAN")
                            end
                        }
                else
                    if not cxt.enc.scratch.failed_friend_negotiation then
                        cxt:Dialog("DIALOG_NADAN_INTERRUPTION")
                    end

                    for k,v in ipairs( cxt.quest.param.admiralty_patrol ) do
                        if k > 1 then
                            v:Kill()
                        end
                    end
                    cxt.quest.param.admiralty_patrol = {cxt.quest.param.admiralty_patrol[1]}


                    cxt:Opt("OPT_ATTACK")
                        :Dialog("DIALOG_ATTACK")
                        :Battle{
                            no_oppo_limit = true,
                            flags = BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED,
                            on_win = function()
                                cxt:GoTo("STATE_BEAT_GUARDS_WITH_NADAN")
                            end
                        }
                end
        end)


    :State("NADAN_BAIT")
        :Loc{
            DIALOG_INTRO = [[
                handler:
                    !left
                    Come on out, you soppy switches. I know you're here.
                guard_1:
                    !right
                    Then you know this ain't gonna end well for you, Nadan. 
                * Three admiralty officers emerge from the tree line, all clearly eager for a fight.
            ]],
            OPT_HELP_NADAN = "Ambush the Admiralty Leader (Help Nadan)",
            DIALOG_HELP_NADAN = [[
                * You slip between the trees and slit the throat of the closest officer while the other two are fixated on Nadan. 
                * Nadan tackles another officer, blade drawn, leaving one last target for you.
                player:
                    !left
                    Time's up, bootlicker. 
                    !fight
                    !exit
            ]],
            DIALOG_HELPED_NADAN = [[
                right:
                    !exit
                * {handler}, clearly enjoying {handler.himher}self, dispatches the last of the surviving Admiralty.
                handler:
                    !right
                    Hah! Forgot how fun it could be to play with 'em, first.
            ]],
            OPT_SNEAK_BY = "Use the distraction to sneak past them (Abandon Nadan)",
            DIALOG_SNEAK_BY = [[
                * With the guards distracted, you sneak past the fracas. The hidden entrance should be just ahead.
                handler:
                    Hey! Where are you going?!
            ]],
            OPT_CONTINUE = "Make a break for it"

        }
        :Fn(function(cxt)
            cxt:TalkTo("handler")
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest.param.admiralty_patrol[1]:GetParty():RemoveMember(cxt.quest.param.admiralty_patrol[1])

            cxt:Opt("OPT_HELP_NADAN")
                :Dialog("DIALOG_HELP_NADAN", cxt.quest.param.admiralty_patrol[1])
                :Battle{
                    enemies = {cxt.quest.param.admiralty_patrol[1]},
                    advantage = TEAM.BLUE,
                    flags = BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_BYSTANDERS,
                    on_win = function()
                        cxt:Dialog("DIALOG_HELPED_NADAN")
                        cxt:GoTo("STATE_BEAT_GUARDS_WITH_NADAN")
                    end
                }

            cxt:Opt("OPT_SNEAK_BY")
                :Dialog("DIALOG_SNEAK_BY")
                :ReceiveOpinion("abandoned")
                :Fn(function(cxt)
                    cxt:Opt("OPT_CONTINUE")
                        :GoTo("STATE_BETRAYED_NADAN")
                end)
        end) 


    :State("STATE_BEAT_GUARDS_WITH_NADAN")
        :Loc{
            DIAL_HELPED_FIGHT = [[
                player:
                    !left
                handler:
                    !right
                    Looks like you <b>do</> know how to use those things.
                player:
                    !happy
                    I picked a couple things up over the years.
                handler:
                    Let's go, before more switches come sniffing.
                    !exit
            ]],

            OPT_FOLLOW = "Follow {handler}",
            DIAL_LEAVE_TOGETHER = [[
                * Nadan leads the way, and you proceed together in familiar silence. 
                player:
                    !exit
            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIAL_HELPED_FIGHT")
            cxt.enc.scratch.nadan_fought = true
            cxt:Opt("OPT_FOLLOW")
                :Dialog("DIAL_LEAVE_TOGETHER")
                :GoTo("STATE_WITH_NADAN")
        end)

    :State("STATE_LET_BY")
        :Loc{
            DIAL_INTRO = [[
                handler:
                    !right
                    !dubious
                    Huh. Guess that was easier than stabbing them in the neck and waiting for the blood to stop spurting.
                    Well, let's head on in, shall we?
                    !exit
            ]],
            OPT_FOLLOW = "Follow {handler}",
            DIAL_LEAVE_TOGETHER = [[
                
                * Nadan leads the way, and you proceed together in familiar silence. 
                player:
                    !exit
            ]],
        }
        :Fn(function(cxt)
            cxt:Dialog("DIAL_INTRO")
            cxt:Opt("OPT_FOLLOW")
                :Dialog("DIAL_LEAVE_TOGETHER")
                :GoTo("STATE_WITH_NADAN")
        end)



    :State("STATE_BETRAYED_BY_NADAN")
        :Loc{
            DIALOG_INTRO = [[
                * You catch up with {handler} in the tunnels behind the secret entrance. Watching from the shadows, you see {handler.himher} conferring with some of {handler.hisher} Spree.
                bandit_escort:
                    !left
                handler:    
                    !right
                    Excuse me a moment, {bandit_escort}.
                bandit_escort:
                    Sure thing, boss.
                    !exit
                handler:
                    I can hear you, {player}. You're not half as subtle as you think you are.
                player:
                    !left
                    I could say the same for you. You were supposed to help me back there?
                handler:
                    !shrug
                    I saw the opportunity, and I took it. I'm not going to risk my life helping some grifter.
                    Besides, you can take care of yourself.
            ]],
            OPT_ATTACK_NADAN = "Attack {handler}",
            DIALOG_ATTACK_NADAN = [[
                player:
                    !fights
                    I pay my debts.
                handler:
                    !sigh
                    Really? You're going to do this?
            ]],
            DIALOG_WON_FIGHT = [[
                {dead ? 
                    * {handler} lies dead at your feet. You quickly rifle through {handler.hisher} pockets, and find a small key, which you pocket.
                }
                { not dead ? 
                    handler:
                        !injured
                        I underestimated you, grifter. 
                    player:
                        !neutral
                        People tend to do that.
                        
                    * You quickly rifle through {handler}'s possessions, and find a small key, which you pocket.
                    player:
                        !take

                    handler:
                        If you survive the next few hours, I might just have to kill you.
                    player:
                        Yeah yeah yeah. I haven't heard that before.
                }
            ]],
            OPT_ENTER_THE_AUCTION = "Continue to the auction",
            DIALOG_ENTER_THE_AUCTION = [[
                player:
                    !exit
                * You continue to the auction.
            ]],


            OPT_DEMAND_RESTITUTION = "Demand restitution",
            DIALOG_DEMAND = [[
                player:
                    You left me for dead, {handler}!
                    After all we've been through, you owe me something for that.
            ]],
            DIALOG_DEMAND_SUCCESS = [[
                handler:
                    !shrug
                    Fine. If it means I don't have to worry about you coming for revenge, take this.
                    !give
                * {handler} hands you a small key.
                handler:
                    It will open one of the artifact crates in the back room. Maybe you can use the contents.
                    Now get out of here before I make you disappear.
            ]],
            DIALOG_DEMAND_FAILURE = [[
                handler:
                    !crossed
                    I'm letting you live so that you can march on up to your death at the auction.
                    That's enough.
            ]],

        }
        :SetLooping()
        :Fn(function(cxt) 
            
            if cxt:FirstLoop() then
                cxt.encounter:DoLocationTransition(  cxt.quest:SpawnTempLocation("TUNNEL") ) 
                cxt.quest:GetCastMember("bandit_escort"):MoveToLocation(cxt.location)
                cxt:TalkTo("handler")
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_ATTACK_NADAN")
                :Dialog("DIALOG_ATTACK_NADAN")
                :Battle{flags = BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED}
                    :OnWin()
                        :Dialog("DIALOG_WON_FIGHT")
                        :Fn(function() 
                            cxt.quest.param.cross_quest_data.has_key = true
                            cxt:Opt("OPT_LEAVE")
                                :CompleteQuest()
                                :End()
                        end)

            cxt:Opt("OPT_DEMAND_RESTITUTION")
                :Dialog("DIALOG_DEMAND")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_DEMAND_SUCCESS")
                        :Fn(function() 
                            cxt.quest.param.cross_quest_data.has_key = true
                            cxt:Opt("OPT_LEAVE")
                                :CompleteQuest()
                                :End()
                        end)
                    :OnFailure()
                        :Dialog("DIALOG_DEMAND_FAILURE")

            cxt:Opt("OPT_ENTER_THE_AUCTION")
                :Dialog("DIALOG_ENTER_THE_AUCTION")
                :Fn(function() 
                    StateGraphUtil.AddEndOption(cxt)
                        :CompleteQuest()

                end)
        end)

    :State("STATE_BETRAYED_NADAN")
        :Loc{
            DIALOG_INTRO = [[
                right:
                    !exit
                left:
                    !exit
                * You enter the tunnels, and set about finding the way through.
                * Eventually, you come across a small group of Spree.
                bandit_escort:
                    !right
                    Who's there?
                player:
                    !left
                bandit_escort:
                    Oh, it's {handler}'s grifter.
                    If <i>you're</> here, where's {handler}?
            ]],

            OPT_ATTACK_SPREE = "Charge the Spree",
            DIALOG_ATTACK_SPREE = [[
                player:
                    !fight
                    I'll send you to him!
            ]],
            DIALOG_WON_FIGHT_AGAINST_SPREE = [[
                right:
                    !exit
                * You leave the battle behind, and plunge deeper into the tunnels.
            ]],

            OPT_CONVINCE = "Convince {agent} that {handler} sent you",
            DIALOG_CONVINCE = [[
                player:
                    {handler} sent me to find you. There are switches at the entrance of the tunnel.
                    You need to go help {handler.himher}!
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                bandit_escort:
                    Alright. You guard this junction. Don't let anyone go down that tunnel.
                player:
                    !point
                    That one, there?
                bandit_escort:
                    Yeah. It heads straight to the auction. We don't want anyone going through there.
                player:
                    Gotcha. Now go save poor {handler}!
                bandit_escort:
                    !exit
                player:
                    !happy
                * The Spree rush back to the tunnel entrance.
            ]],
            OPT_LEAVE = "Head down the forbidden tunnel",
            DIALOG_LEAVE = [[
                player:
                    !exit
                * You head to the auction.
            ]],
            DIALOG_CONVINCE_FAILURE = [[
                bandit_escort:
                    !point
                    I see {handler} coming this way!
                
                handler:
                    !right
                    !injured
                player:
                    {handler}! You... you made it!
                handler:
                    !angry
                    ...
                    !fight
                    Leave the grifter to me.
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                * {handler}'s guards block your escape. {handler} may be injured from {handler.hisher} previous fight, but {handler.heshe} angry enough to compensate.
            ]],
            DIALOG_WON_FIGHT_VS_NADAN = [[
                { dead ?
                    * {handler} falls to the ground. The assembled Spree don't seem to know what to do.
                    player:
                        !exit
                    * You take the opportunity to make a break for it and rush deeper into the tunnel.
                }
                {not dead? 
                    handler:
                        !injured
                        Stop. I'll let Kashio finish you off.
                    player:
                        How generous of you.
                    handler:
                        Go. Before I change my mind.
                    player:
                        !exit
                    * You heed {handler}'s advice.
                }
            ]]
        }
        :Fn(function(cxt) 
            cxt.encounter:DoLocationTransition(  cxt.quest:SpawnTempLocation("TUNNEL") ) 
            cxt.quest:GetCastMember("bandit_escort"):MoveToLocation(cxt.location)
            CreateCombatBackup(cxt.quest:GetCastMember("bandit_escort"), "BANDIT_PATROL", 3)
            cxt:TalkTo(cxt.quest:GetCastMember("bandit_escort"))
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_ATTACK_SPREE")
                :Dialog("DIALOG_ATTACK_SPREE")
                :Battle {
                            no_oppo_limit = true,
                            flags = BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED, 
                            advantage = TEAM.BLUE
                        }
                    :OnWin()
                        :Dialog("DIALOG_WON_FIGHT_AGAINST_SPREE")
                        :Fn(function() 
                            cxt:Opt("OPT_LEAVE")
                                :Dialog("DIALOG_LEAVE")
                                :CompleteQuest()
                                :End()
                        end)

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{}
                    :OnSuccess()
                        :Dialog("DIALOG_CONVINCE_SUCCESS")
                        :Fn(function() 
                            cxt:Opt("OPT_LEAVE")
                                :Dialog("DIALOG_LEAVE")
                                :CompleteQuest()
                                :End()
                        end)

                    :OnFailure()
                        :Fn(function() 
                            cxt:TalkTo("handler")
                            cxt:Dialog("DIALOG_CONVINCE_FAILURE")
                            cxt:GetAgent().health:SetPercent( .4 )
                            cxt:Opt("OPT_DEFEND")
                                :Dialog("DIALOG_DEFEND")
                                :Battle{
                                    flags = BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_BYSTANDERS, 
                                    advantage = TEAM.BLUE
                                }
                                    :OnWin()
                                        :Dialog("DIALOG_WON_FIGHT_VS_NADAN")
                                        :Fn(function() 
                                            cxt:Opt("OPT_LEAVE")
                                                :CompleteQuest()
                                                :End()
                                        end)

                        end)
        end)

    :State("STATE_WITH_NADAN")
        :Loc{
            DIALOG_INTRO = [[
                * You follow {handler} through the twisting tunnels and find a small contingent of Spree guarding a junction.
                bandit_escort:
                    !left
                    {handler}'s here!
                handler:
                    !right
                    Look alive, you scabs. It's time to get down to business.
                player:
                    !left
                    Is this the entrance to the auction?
                handler:
                    Yeah. You go up ahead and do your business.
                player:
                    Can I convince you to come with?
                
                {nadan_fought ? 
                    handler:
                        I'm not going to lie, that fight took the wind out of me.
                        We're going to hold up tight here, and wait for the fireworks to start. Once the assembled hoi-polloi panic and make a runner their shill-purses will be easy pickings.

                }
                {not nadan_fought ? 
                    handler:
                        Ha! You could probably convince me to change my ways and join the Admiralty, from what I've seen.
                        We're going to wait here untill the fireworks start, and then get down to some good old-fashioned looting.
                }

                handler:
                    Here—take this.
                    !give
                
                player:
                    !take
                * {handler} passes you a small device.
                handler:
                    It's a lockmelter. It should open one of the artifact crates. Might be something useful inside. Might help you survive today.
                player:
                    !dubious
                    Why are you just giving me this?
                
                handler:
                    If you <b>do</> manage to take down Kashio, remember who helped you.
                    Don’t give her a chance to get the first blow.
                    Cut hard, and fast—and you’ll get out of this alive.

                player:
                    What about you? Where will you be?

                handler:
                    I want her dead as much as you do. It’s why we’re in this together.
                    But once Kashio’s dealt with… we’ll go our separate ways.
                    It’s better that way. Won’t be long until the bounty comes calling for me. 
            ]],


            OPT_ASK_ABOUT_BOUNTY = "Ask Nadan about his bounty",
            DIALOG_ASK_ABOUT_BOUNTY = [[
                player:
                    !interest
                    I know there’s a bounty out on you, but what makes you think I’ll take it?

                agent:
                    $neutralDirect
                    Because you’ve got honor. And you clean up messes.
                    Sooner or later I’ll make a mess you can’t leave standing.

                player:
                    !shrug 
                    $miscPersuasive
                    I can choose my jobs. You did me a solid. We’re friends, even.

                agent:
                    When you come for me, I won’t hold back. I’ll fight to kill.
                    But I won’t hold it against you, either.

                player:
                    !dubious
                    $neutralDubious
                    What the Hesh are you talking about?

                agent:
                    !placate
                    What I mean is we’re good. If our paths ever cross for the worse, I won’t think you a traitor.
                    You live your life your way, and it’ll be alright.
            ]],

            OPT_ASK_NADAN_ABOUT_PLANS = "Ask Nadan about afterwards",
            DIALOG_ASK_NADAN_ABOUT_PLANS = [[
            player:
                What’ll you do with Kashio gone?

            agent:
                !suspicious
                $neutralThoughtful
                Expand my operation. But just enough to keep the others in line.
                When someone like Kashio gets too much power, they need pruning.

            player:
                Right. Well… I hope your shears are sharp. Because weeds tend to grow.
            ]],


            OPT_THANK_NADAN = "Thank {agent}",
            DIALOG_THANK_NADAN = [[
                player:
                    I know you’re not going to like to hear this, but thank you.

                agent:
                    Why wouldn’t I like to hear it?

                player:
                    Uh. Because you don’t seem… very touchy-feely?

                agent:
                    I just don’t make a show of it. 
                    ...
                    You’re welcome.
            ]],

        }
        :Fn(function(cxt) 
            cxt.encounter:DoLocationTransition( cxt.quest:SpawnTempLocation("TUNNEL") ) 
            cxt.quest:GetCastMember("bandit_escort"):MoveToLocation(cxt.location)
            cxt:TalkTo("handler")
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest.param.cross_quest_data.has_lockmelter = true

            cxt:RunLoop(function() 
                if not cxt.enc.bounty_ask then
                    cxt:Opt("OPT_ASK_ABOUT_BOUNTY")
                        :Dialog("DIALOG_ASK_ABOUT_BOUNTY")
                        :Fn(function() cxt.enc.bounty_ask = true end)
                end

                if not cxt.enc.plans_ask then
                    cxt:Opt("OPT_ASK_NADAN_ABOUT_PLANS")
                        :Dialog("DIALOG_ASK_NADAN_ABOUT_PLANS")
                        :Fn(function() cxt.enc.plans_ask = true end)
                end

                cxt:Opt("OPT_THANK_NADAN")
                    :Dialog("DIALOG_THANK_NADAN")
                    :Fn(function() 
                        cxt.quest:Complete()
                        StateGraphUtil.AddEndOption(cxt)
                    end)
            end)
        end)

--------------------
QDEF:AddTest(TestCreator()
    :SetParams{story_beat = "arrest"}
    :AddParam("set_escort_relationship",  RELATIONSHIP_ARRAY)
    :SetChoices{"CHARGE"} --constant
    :AddBattleDimension())

QDEF:AddTest(TestCreator()
    :SetParams{story_beat = "arrest"}
    :AddParam("set_escort_relationship",  RELATIONSHIP_ARRAY)
    :SetChoices{"CONVINCE"} --constant
    :AddNegotiationDimension()
    :AddBattleDimension())

QDEF:AddTest(TestCreator()
    :SetParams{story_beat = "arrest"}
    :AddParam("set_escort_relationship",  RELATIONSHIP_ARRAY)
    :SetChoices{"SNEAK"} --constant
    :AddChoiceSet{"SNEAK_ATTACK", "SNEAK_ABANDON"}
    :AddNegotiationDimension()
    :AddBattleDimension())

--eh... it'll catch 'em all eventually.
QDEF:AddTest(TestCreator()
    :SetParams{story_beat = "ambush"}
    :SetReps(20))


QDEF:AddDebugOption("story_beat", {"arrest", "ambush"})
    :AddDebugOption("set_nadan_relationship",  RELATIONSHIP_ARRAY)
    :AddDebugOption("set_escort_relationship",  RELATIONSHIP_ARRAY, function(params) return params.story_beat == "arrest" end )
    :AddDebugOption("set_guard_relationship",  RELATIONSHIP_ARRAY, function(params) return params.story_beat == "ambush" end )
    :AddDebugOption("nadan_betrays_you", {true, false}, function(params) return params.story_beat == "ambush" end )
    




