local AUCTION_DATA = require "content/grafts/kashio_boss_fight_defs"


local ADMIRALTY_VARIATIONS = {
    "SAL_STORY_AUCTION_ADMIRALTY_SMUGGLE", 
    "SAL_STORY_AUCTION_ADMIRALTY_PERSONAL_TROUBLE", 
    "SAL_STORY_AUCTION_ADMIRALTY_ASSASSINATION",
}

local function spawn_next_quest( quest )
    local quest_num = quest:GetParameter("quests_completed") + 1
    local id = quest.param.quest_list[quest_num]
    
    if id then
        local spawned_quest, err = QuestUtil.SpawnQuest(id, { parameters = { cross_quest_data = quest.param.cross_quest_data } })
        if spawned_quest then
            quest:SetParameter("current_quest", spawned_quest)
            return spawned_quest
        end
    end
end

local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    reward_mod = 0,
    on_init = function(quest)


        -- table that each quest will have passed in so we can share data between quests
        quest.param.cross_quest_data = {}
        quest.param.cross_quest_data.auction_lots = AUCTION_DATA.PickAuctionLots(2, 3)

        TheGame:GetGameState():SetDifficulty(5)

        if quest.param.force_faction_alliance then
            Util.SetPlayerWorkingWithFaction(quest.param.force_faction_alliance)
        end
        local working_with_admiralty = Util.IsPlayerWorkingWithFaction("ADMIRALTY")
        quest.param.quests_completed = 0

        quest.param.quest_list = {}
        if working_with_admiralty then
            table.insert( quest.param.quest_list, "SAL_STORY_AUCTION_ADMIRALTY")
            table.insert( quest.param.quest_list, quest.param.force_variation or table.arraypick(ADMIRALTY_VARIATIONS))
        else
            table.insert(quest.param.quest_list, "SAL_STORY_AUCTION_BANDIT")
            table.insert(quest.param.quest_list, "SAL_STORY_AUCTION_BACK_ROOM")
        end

        table.insert(quest.param.quest_list, "SAL_STORY_AUCTION")

        spawn_next_quest(quest)
    end,

    events = 
    {
        quests_changed = function(quest, event_quest)
            if event_quest == quest.param.current_quest then
                if event_quest:IsComplete() or event_quest:IsFailed() then
                    quest.param.quests_completed = quest.param.quests_completed + 1
                    spawn_next_quest(quest)
                end
            end
        end,
    },
}

:AddObjective{
    hidden = true,
    state = QSTATUS.ACTIVE,
    id = "auction_day",
    title = "Attend the auction",
    desc = "Auction day has come.",
}

QDEF:AddTest(TestCreator()
                :AddParam("force_faction_alliance", {"ADMIRALTY"})
                :AddParam("force_variation", ADMIRALTY_VARIATIONS))

QDEF:AddTest(TestCreator()
                :AddParam("force_faction_alliance", {"BANDITS"}))
                

QDEF:AddDebugOption("force_faction_alliance", {"ADMIRALTY", "BANDITS"})
QDEF:AddDebugOption("force_variation", ADMIRALTY_VARIATIONS, function(params) return params.force_faction_alliance == "ADMIRALTY" end)
