
AddOpinionEvent("FACTION_RIVALS", 
{
    delta = OPINION_DELTAS.MAJOR_BAD,
    txt = "Faction Rivals",
})

AddOpinionEvent("AIDED_THE_ENEMY_FACTION", 
{
    delta = OPINION_DELTAS.DISLIKE,
    txt = "You joined up with the {2#faction}",
    txt_fn = function( txt, agent, data )
        return loc.format(txt, agent:LocTable(), TheGame:GetGameState():GetFaction(data.faction_id))
    end
})

AddOpinionEvent("GOES_WAY_BACK", 
{
    delta = OPINION_DELTAS.MAJOR_GOOD,
    txt = "{1.HeShe} was friends with your parents",
})

AddOpinionEvent("OLD_ENEMY", 
{
    delta = OPINION_DELTAS.MAJOR_BAD,
    txt = "{1.HeShe} betrayed your family, and now views you as a threat",
})



local DAY_SCHEDULE = {
    {quest = "SAL_STORY_DAY_1", difficulty = 1},  -- 2 sides, meet merchant, meet kasio
    {quest = "SAL_STORY_DAY_2", difficulty = 2},  -- pick handler, do handler job 1, do side quest, go shopping
    {quest = "SAL_STORY_DAY_3", difficulty = 3},  -- handler job 2, side job, assassin attempt
    {quest = "SAL_STORY_DAY_4", difficulty = 4},  -- handler boss fight, side job 
    {quest = "SAL_STORY_DAY_5", difficulty = 5},  -- The auction
}


local QDEF = QuestDef.Define
{
    title = "Hunting Kashio",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    qtype = QTYPE.STORY,
    desc = [[You've made it to Havaria: next step is to find {kashio}. Word is she'll be showing her face at a Murder Bay auction. You've got a couple of days to find a way in.]],

    max_day = 5,
    get_narrative_progress = function(quest)
        
        local total_days = 5
        local completed_days = (quest.param.day or 1)-1

        local percent = completed_days / total_days
        local title = loc.format(LOC "CALENDAR.DAY_FMT", quest.param.day or 1)
        return percent, title, quest.param.day_quest and quest.param.day_quest:GetTitle() or ""
    end,

    on_init = function(quest)

        TheGame:GetGameState():SetMainQuest(quest)

        quest.param.merchants_quest = QuestUtil.SpawnQuest("SAL_STORY_MERCHANTS")
        quest.param.grog_quest = QuestUtil.SpawnQuest("SAL_STORY_LIVING_AT_GROG")
        if not TheGame:GetGameState():GetPlayerAgent().graft_owner:HasGraft("bare_mettle") then
            quest.param.mettle_quest = QuestUtil.SpawnQuest("SAL_STORY_METTLE_SHOP")
        end
        assert(TheGame:GetGameState())
        
        TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("GROG_N_DOG"))
        
        quest:SetParameter("home_base", TheGame:GetGameState():GetLocation("GROG_N_DOG"))
        quest:GetCastMember("bandit_contact"):OpinionEvent(OPINION.FACTION_RIVALS, nil, quest:GetCastMember("admiralty_contact"))

        quest:GetCastMember("fssh"):OpinionEvent(OPINION.GOES_WAY_BACK)
        quest:GetCastMember("fssh"):OpinionEvent(OPINION.GOES_WAY_BACK, nil, quest:GetCastMember("admiralty_contact"))
        quest:GetCastMember("fssh"):OpinionEvent(OPINION.GOES_WAY_BACK, nil, quest:GetCastMember("bandit_contact"))
        quest:GetCastMember("fssh"):OpinionEvent(OPINION.OLD_ENEMY, nil, quest:GetCastMember("kashio"))

        QuestUtil.StartDayQuests(DAY_SCHEDULE, quest)

        
        if quest.param.start_on_day and quest.param.start_on_day > 2 then
            if quest.param.force_faction_alliance == "BANDITS" then
                quest:GetQuestDef().SelectBanditHandler(quest)
            else
                quest:GetQuestDef().SelectAdmiraltyHandler(quest)
            end
        end

        QuestUtil.DoNextDay(DAY_SCHEDULE, quest, quest.param.start_on_day )
        DoAutoSave()

    end,

    plot_armour_fn = function(quest, agent)

        if agent == quest:GetCastMember("admiralty_contact") or agent == quest:GetCastMember("bandit_contact") then
            return true
        end

        if agent == quest:GetCastMember("fssh") then
            return true
        end
    end,

    quest_reward_fn = function(quest, reward_quest)

        local qrank = reward_quest:GetRank()
        local val = 0
        if reward_quest:GetType() == QTYPE.CONTRACT then
            val = CONTRACT_REWARD[clamp(qrank, 1, #CONTRACT_REWARD)]
        else
            val = SIDEQUEST_REWARD[clamp(qrank, 1, #SIDEQUEST_REWARD)]
        end
        val = round_zero( val * reward_quest.reward_random, 5 )
        return val

    end,

    events = 
    {
        quests_changed = function(quest, event_quest) 
            if event_quest == quest.param.day_quest and quest.param.day_quest:IsComplete() then
                QuestUtil.DoNextDay(DAY_SCHEDULE, quest)
                
                if quest.param.rise_outpost_quest == nil and quest.param.day > 1 then
                    if math.random() < .25 then
                        quest.param.rise_outpost_quest = QuestUtil.SpawnQuest("SAL_RISE_OUTPOST_ENTRY_1")
                    end
                end
            end
        end,
    },

    is_in_hiding = function(quest, agent)
        if quest:IsActive("kashio_in_hiding") and agent == quest:GetCastMember("kashio") then
            return true
        end
        
        if quest.param.other_handler and quest.param.other_handler == agent then
            return true
        end

    end,

    
    ---------------------------these are utility functions that are called by the sub-quests.
    

    TrySpawnHandlerQuest = function(quest, rank)
        rank = rank or TheGame:GetGameState():GetCurrentBaseDifficulty()
        local faction_id = Util.IsPlayerWorkingWithFaction("BANDITS") and "BANDITS" or "ADMIRALTY"
        local attempt_quest_ids = {}
        for k, questdef in pairs( Content.GetAllQuests() ) do
            if questdef:GetType() == QTYPE.CONTRACT and questdef:FilterForAct( "SAL" ) and questdef.faction == faction_id then
                table.insert(attempt_quest_ids, questdef.id)
            end
        end

        
        local event_id = "SAL_CONTRACT_SPAWN_" .. faction_id
        local quest_scores = {}
        for k,v in ipairs(attempt_quest_ids) do
            quest_scores[v] = QuestUtil.CalcQuestSpawnScore(event_id, math.floor( #attempt_quest_ids / 2), v)    
        end
        table.shuffle(attempt_quest_ids)
        table.stable_sort(attempt_quest_ids, function(a,b) return quest_scores[a] < quest_scores[b] end)
        print (tostr(quest_scores))

        local failed_quests = {}
        local new_quest
        for _, quest_id in ipairs(attempt_quest_ids) do
            local err
            new_quest, err = QuestUtil.SpawnInactiveQuest( quest_id, {cast = {handler = quest:GetCastMember("handler")}, qrank = rank}) 
            if new_quest then
                break
            else
                table.insert(failed_quests, err)
            end
        end

        if new_quest then
            TheGame:GetGameProfile():RecordIncident(event_id, new_quest:GetContentID())
        end
        

        return new_quest
    end,

    SelectBanditHandler = function(quest)
        Util.SetPlayerWorkingWithFaction("BANDITS")
        quest:GetCastMember("admiralty_contact"):OpinionEvent(OPINION.AIDED_THE_ENEMY_FACTION, {faction_id = "BANDITS"})
        quest:AssignCastMember("handler", quest:GetCastMember("bandit_contact"))
        quest.param.other_handler = quest:GetCastMember("admiralty_contact")

        --[[local grog_room = TheGame:GetGameState():GetLocation( "GROG_N_DOG.inn_room" )
        if grog_room then
            grog_room:SetPlaxElementVisiblity( "SPREE_TROPHY", true )
        end
        --]]

        if quest.param.all_day_quests then
            for k,v in ipairs(quest.param.all_day_quests) do
                v.param.handler = quest:GetCastMember("bandit_contact")
                if v:GetQuestDef():GetCast("handler") then
                    v:AssignCastMember("handler", v.param.handler)
                end
            end        
        end
    end,


    SelectAdmiraltyHandler = function(quest)
        Util.SetPlayerWorkingWithFaction("ADMIRALTY")
        quest:GetCastMember("bandit_contact"):OpinionEvent(OPINION.AIDED_THE_ENEMY_FACTION, {faction_id = "ADMIRALTY"})
        quest:AssignCastMember("handler", quest:GetCastMember("admiralty_contact"))
        quest.param.other_handler = quest:GetCastMember("bandit_contact")

        --[[local grog_room = TheGame:GetGameState():GetLocation( "GROG_N_DOG.inn_room" )
        if grog_room then
            grog_room:SetPlaxElementVisiblity( "ADMIRALTY_TROPHY", true )
        end--]]

        if quest.param.all_day_quests then
            for k,v in ipairs(quest.param.all_day_quests) do
                v.param.handler = quest:GetCastMember("admiralty_contact") 
                if v:GetQuestDef():GetCast("handler") then
                    v:AssignCastMember("handler", v.param.handler)
                end
            end        
        end
    end,
}



:AddCastByAlias{
    cast_id = "kashio",
    alias = "KASHIO",
    on_assign = function(quest, agent)
        agent:OpinionEvent(OPINION.OLD_ENEMY)
    end
}

:AddCast{
    cast_id = "handler",
    when = QWHEN.MANUAL,
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    on_assign = function(quest, agent)
        AgentUtil.TakeJob(agent, TheGame:GetGameState():GetLocation("GROG_N_DOG"), "bartender")
    end,
}


:AddCastByAlias{
    cast_id = "admiralty_contact",
    alias = "MURDER_BAY_ADMIRALTY_CONTACT",
    on_assign = function(quest, agent)
        local location = TheGame:GetGameState():GetLocation("ADMIRALTY_BARRACKS")
        AgentUtil.TakeJob(agent, location, "faction_contact")
    end
}

:AddCastByAlias{
    cast_id = "bandit_contact",
    alias = "MURDER_BAY_BANDIT_CONTACT",
    on_assign = function(quest, agent)
        local location = TheGame:GetGameState():GetLocation("SPREE_INN")
        AgentUtil.TakeJob(agent, location, "faction_contact")
    end
}

:AddObjective{
    id = "kashio_in_hiding",
    title = "Find {kashio}",
    desc = "{kashio} is in hiding. If you're going to confront {kashio.himher} you're going to have to get access to the upcoming auction.",
    hide_in_overlay = true,
    state = QSTATUS.ACTIVE,
}

QDEF:AddDebugOption("start_on_day", {1,2,3,4,5})
    :AddDebugOption("force_faction_alliance", {"ADMIRALTY", "BANDITS"}, function(params) return params.start_on_day and params.start_on_day > 2 end)


