
local CleanUpFn = function(quest) 
    if quest:GetCastMember("noodle_chef"):IsInPlayerParty() then
        quest:GetCastMember("noodle_chef"):Dismiss()
    end
    if quest:GetCastMember("snail") and quest:GetCastMember("snail"):IsInPlayerParty() then
        quest:GetCastMember("snail"):Dismiss()
    end
end


local QDEF = QuestDef.Define
{
    title = "Capture Nadan",
    icon = engine.asset.Texture("icons/quests/sal_story_kill_nadan.tex"),
    
    qtype = QTYPE.STORY,
    desc = [[{handler} is setting a trap to capture {other_handler}, and you're the bait.]],
    block_side_jobs = true,
    on_fail = CleanUpFn,
    on_complete = CleanUpFn,
    on_cancel = CleanUpFn,

    

    prevent_random_events = function(quest)
        return quest:IsActive("go_to_ambush_point")     --the snail must live
    end,


    is_in_hiding = function(quest, agent)
        if quest:IsActive("get_snail") and agent == quest:GetCastMember("noodle_chef") then
            return true
        end
    end,

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("other_handler") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,
}

:AddCastByAlias{
    cast_id = "other_handler",
    alias = "MURDER_BAY_BANDIT_CONTACT",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "handler",
    alias = "MURDER_BAY_ADMIRALTY_CONTACT",
}

:AddCast{
    cast_id = "snail",
    when = QWHEN.MANUAL,
}

:AddOpinionEvents{
    no_capture_nadan = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Did not capture Nadan",
    },
}




:AddCast{
    cast_id = "noodle_chef",
    cast_fn = function(quest, t) 
        if TheGame:GetGameState():GetLocation("MURDERBAY_NOODLE_SHOP"):GetProprietor() then
            table.insert(t, TheGame:GetGameState():GetLocation("MURDERBAY_NOODLE_SHOP"):GetProprietor())
        end
    end,
    events = 
    {
        agent_retired = function( quest, agent )
            if quest:IsActive( "get_snail" ) then
                -- If noodle chef died before we even got the snail, cast someone new.
                quest:UnassignCastMember( "noodle_chef" )
                quest:AssignCastMember( "noodle_chef" )
            end
        end,
    }
}
:AddCastFallback    
{
    --juuuuuuust in case. this shouldn't happen, but who knows?
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "BARTENDER" ) )
    end,
}


:AddCast{
    cast_id = "goon_leader",
    when = QWHEN.MANUAL,
    score_fn = function(agent, quest)
        if agent:GetRelationship() ~= RELATIONSHIP.NEUTRAL then
            return 10
        end
        return 1

    end,
    condition = function(agent, quest) 
        return agent:GetContentID() == "BANDIT_RAIDER" or agent:GetContentID() == "BANDIT_CAPTAIN"
    end
}
:AddCastFallback    
{
    --juuuuuuust in case. this shouldn't happen, but who knows?
    cast_fn = function(quest, t)
        table.insert( t, quest:CreateSkinnedAgent( "BANDIT_CAPTAIN" ) )
    end,
}

:AddObjective{
    id = "get_snail",
    title = "Get the oshnu",
    mark = {"back_alley"},
    state = QSTATUS.ACTIVE,
    on_activate = function(quest)
        quest:SpawnTempLocation("BACK_ALLEY", "back_alley" )
    end,
    on_complete = function(quest)
        quest:Activate("go_to_ambush_point")
    end,
}

:AddObjective{
    id = "go_to_ambush_point",
    title = "Go to the ambush",
    mark = {"ambush_point"},
    on_activate = function(quest)
        quest:SpawnTempLocation("AMBUSH_SPOT", "ambush_point" )
    end,
}

:AddObjective{
    id = "capture_other_handler",
    title = "Capture {other_handler}",
    mark = {"ambush_point"},
}

:AddObjective{
    id = "return_to_handler",
    title = "Bring {other_handler} to {handler}",
    mark = {"handler"},
}

:AddObjective{
    id = "return_to_handler_empty",
    title = "Tell {handler} that {other_handler} is dead",
    mark = {"handler"},
}

:AddObjective{
    id = "return_to_handler_lying",
    title = "Convince {handler} that {other_handler} escaped",
    mark = {"handler"},
}

:AddLocationDefs{
    BACK_ALLEY = {
        name = "Noodle Shop Back Door",
        plax = "EXT_door_slums1",
        map_tags = {"city"},
    },

    AMBUSH_SPOT = {
        name = "Ambush Point",
        map_tags = {"road"},
        plax = "EXT_NadanFight",
        indoors = false,
        show_agents = false,
    }
}

:AddLocationCast{
    cast_id = "ambush_point",
    when = QWHEN.MANUAL,
}

:AddLocationCast{
    cast_id = "back_alley",
    when = QWHEN.MANUAL,
    on_assign = function(quest) 
        quest:GetCastMember("back_alley"):SetMapPos( TheGame:GetGameState():GetLocation("MURDERBAY_NOODLE_SHOP"):GetMapPos() )
        
    end,
}


QDEF:AddConvo("get_snail")
    :ConfrontState("THE_STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("back_alley") end)
    :Loc{
        DIALOG_INTRO = [[
            * You find the alleyway that {handler} indicated, and someone you take for {noodle_chef}.
            noodle_chef:
                !right
            player:
                !left
                !dubious
                {handler} sent me to pick up an oshnu. You don't... keep 'em here, do you?
            noodle_chef:
                !thumb
                $neutralWhatever
                Over here.
                !exit
                * {noodle_chef} pulls a tarp off of what looks like a nondescript pile of refuse, revealing a drowsy oshnu. It blinks its eyestalks at you. 
            snail:
                !right 
                Wok.
            player:
                !handwring
                $miscMischievous
                Aw. Hey, slimy girl! You want walkies? Huh? Walkies?
            noodle_chef:
                !right
                !spit
                $neutralWhatever
                This one's a lazy so-and-so. Better off as noodles, if you ask me. Not sure what the Admiralty wants with it.
            player:
                !angry
                $angryDefault
                She's a she and she's beautiful.
            noodle_chef:
                !handwave
                $neutralWhatever
                Not my oshnu, not my snail race, hunter. Tell {handler} she owes me the next pedigree that goes lame. 
            *** {noodle_chef} has given you an oshnu to use in your decoy caravan.
        ]],

        OPT_TAKE_OSHNU = "Take the oshnu",
        DIALOG_TAKE_OSHNU = [[
            player:
                !thought
                $neutralThoughtful
                I'm horrified, but then, the noodles are damn good. 
            noodle_chef:
                !permit
                $neutralWhatever
                Somehow I suspect you sleep just fine at night. 
                !shrug
                Good luck out there, hunter. Not that I care much.
            * {noodle_chef} heads back inside the restaurant, leaving you with your new mucousy friend. 
        ]],
        OPT_CONVINCE_COME = "Convince {noodle_chef} to come with you",
        DIALOG_CONVINCE_COME = [[
            player:
                !overthere
                $scaredTakenAback
                What, on my own? I dunno this majestic creature's antennae from its love dart. 
                {handler} was under the impression that you'd come with me.
        ]],
        DIALOG_CONVINCE_FAIL = [[
            player:
                $neutralDirect
                Angelica needs a handler! 
            noodle_chef:
                !forgetful
                $neutralWhatever
                I agree. But I'm not a handler—I'm a chef.
                If "Angelica" needs a marinade, you know where to find me. 
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                $miscPersuasive
                {handler} has plans to take good care of all the regimental oshnus. 
                !crossed
                $miscMocking
                If you want a pedigree to go "lame", you'll want to be a better ally to the Admiralty.
            noodle_chef:
                !angry
                $angryDefault
                Tsk, you kidding me? Next thing you know they're just gonna keep 'em around for ceremonies.
                Waste of good meat, if you ask me.
                !disappoint 
                $neutralResigned
                Fine. Lead the way, grifter.
        ]],
    }
    :Fn(function(cxt) 
        
        cxt.quest:AssignCastMember("snail", TheGame:GetGameState():AddSkinnedAgent( "MERCHANT_SNAIL" ))
        cxt.quest:GetCastMember("snail"):MoveToLocation(cxt.location)

        cxt.quest:GetCastMember("noodle_chef")
        cxt.quest:GetCastMember("noodle_chef"):MoveToLocation(cxt.location)
        cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("noodle_chef"))

        cxt:Dialog("DIALOG_INTRO")

        cxt:Opt("OPT_TAKE_OSHNU")
            :Dialog("DIALOG_TAKE_OSHNU")
            :Fn(function(cxt) 
                cxt.quest:GetCastMember("snail"):Recruit(PARTY_MEMBER_TYPE.CREW) 
                cxt.quest:Complete("get_snail")
            end)
            :Travel()

        cxt:Opt("OPT_CONVINCE_COME")
            :Dialog("DIALOG_CONVINCE_COME")
            :Negotiation{
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                    cxt.quest:GetCastMember("snail"):Recruit(PARTY_MEMBER_TYPE.CREW)
                    cxt:GetAgent():Recruit()
                    cxt.quest:Complete("get_snail")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end,
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_CONVINCE_FAIL")
                    cxt.quest:GetCastMember("snail"):Recruit(PARTY_MEMBER_TYPE.CREW)
                    cxt.quest:Complete("get_snail")
                    StateGraphUtil.AddLeaveLocation(cxt)
                end,
            }
    end)



QDEF:AddConvo("go_to_ambush_point")
    :ConfrontState("THE_STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("ambush_point") end)
    :Loc{
        DIALOG_INTRO = [[
            * You find the ambush location that {handler} marked on your map.
            player:
                !left
                !happy
                $happyDefault
                Well, this is cozy.
        ]],
        DIALOG_INTRO_CHEF = [[
            noodle_chef:
                !right
                !agree
                
                {snail_in_party ?
                    noodle_chef:
                        $neutralResigned
                        The oshnu's good and moist. That keeps it happy. 
                }

                noodle_chef:
                What are we doing out here, anyway?
            player:
                !interest
                Just act natural.
            noodle_chef:
                !exit
        ]],
        DIALOG_SPREE_ARRIVAL = [[
            * It's not long before you hear the crunch of feet through the underbrush. You pretend not to notice. 
        ]],
        DIALOG_GOON_FRIEND = [[
            * A group of Spree bandits emerge from the tree line. You recognize your friend, {goon_leader}, among their ranks. 
            goon_leader:
                !right
                !surprised
                $neutralTakenAback
                {player}? You sure have a habit of turning up in the worst places, you know that?
        ]],
        DIALOG_GOON_NOT_FRIEND = [[
            * When they finally emerge from the tree line, it's with confidence, blades drawn.
            goon_leader:
                !right
                Here it is, boss—another gift from your old friend {handler}.
        ]],
        DIALOG_NADAN_INTRO = [[
            other_handler:
                !right
                !dubious
                $angryDefault
                You don't look like Admiralty. Did they finally wise up and stop sending their own on this suicide run?
            player:
                !happy
                $happyCocky
                Something like that. You're {other_handler}, right? 
                I mean, with a face like that, I can't imagine who else you'd be.
            
            {snail_in_party ?
                * {other_handler} eyes you warily. He raises his hands, and {goon_leader} stops approaching the oshnu.
                other_handler:
                    !placate
                    $angryReluctant
                    You might wanna hold back, {goon_leader}. It's clear this ain't like the other caravans.
            }

            {not snail_in_party ?
                * {other_handler} eyes you warily. He raises his hands, and {goon_leader} stops short.
                other_handler:
                    !placate
                    $angryReluctant
                    You might wanna hold back, {goon_leader}. This was supposed to be a caravan, but I don't see any oshnus.
            }

            player:
                !shrug
                $miscMocking
                'Fraid not. You're under arrest, {other_handler}. And I've got orders to bring you in alive, so I'd appreciate if you play nice. 
                !cruel
                $angryCruel
                I'd hate to do to your backside what was done to your face. 
            other_handler:
                !angry
                $angrySeething

                {snail_in_party?
                    {goon_leader}? Kill the hunter. And her slimy little friend, too. 
                }
                
                {not snail_in_party?
                    {goon_leader}? Kill the hunter for wasting our time. 
                }

            *** {other_handler} fell for the trap, and you attempt to arrest him.
        ]],


    }
    :Fn(function(cxt) 
        local snail = cxt.quest:GetCastMember("snail")
        cxt.enc.scratch.snail_in_party = snail and snail:IsInPlayerParty()
        cxt.quest:Complete("go_to_ambush_point")
        cxt.quest:GetCastMember("other_handler"):MoveToLocation(cxt.location)
        cxt.quest:AssignCastMember("goon_leader")
        cxt.quest:GetCastMember("goon_leader"):MoveToLocation(cxt.location)
        cxt.quest.param.goons = CreateCombatBackup(cxt.quest:GetCastMember("goon_leader"), "BANDIT_PATROL_BACKUP", (cxt.quest:GetRank()-1))


        cxt:Dialog("DIALOG_INTRO")
        if cxt.quest:GetCastMember("noodle_chef"):IsInPlayerParty() then
            cxt:Dialog("DIALOG_INTRO_CHEF")
        end

        cxt:Dialog("DIALOG_SPREE_ARRIVAL")
        cxt:Dialog(cxt.quest:GetCastMember("goon_leader"):GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_GOON_FRIEND" or "DIALOG_GOON_NOT_FRIEND")
        cxt:Dialog("DIALOG_NADAN_INTRO")
        cxt:GoTo("STATE_DEAL_WITH_GOONS")
    end)

    :State("STATE_DEAL_WITH_GOONS")
        :Loc{
            DIALOG_GOON_CONFRONT_FRIEND = [[
                goon_leader:
                    !right
                    !disappoint
                    $angryReluctant
                    I'm sorry, {player}, but I've got my orders and I don't intend to refuse them. 
            ]],
            DIALOG_GOON_CONFRONT_NOT_FRIEND = [[
                goon_leader:
                    !right
                    !cruel
                    $angryCruel
                    Nice knives. Mind if I take them off your corpse?
            ]],
            OPT_NEGOTIATE_GOON = "Convince {goon_leader} to walk away",
            DIALOG_NEGOTIATE_GOON = [[
                player:
                    !overthere
                    $neutralDirect
                    {other_handler} is coming with me. I don't have to kill you first, but I will if you stand in my way.
            ]],

            OPT_FIGHT_GOONS = "Defend yourself!",
            DIALOG_FIGHT_GOONS = [[
                player:
                    !fight
                    $angrySeething
                    I've got orders to keep {other_handler} alive. But if you got a death wish, I'm happy to oblige.
            ]],


            DIALOG_POST_GOON_FIGHT = [[
                agent:
                    !exit
                * {other_handler} watches impassively as you cut through {other_handler.hisher} goons.
                player:
                    !angry_threaten
                    $angryDefault
                    Now how about you, {other_handler}? You gonna come quiet?
                other_handler:
                    !right
                    !disappoint
                    $angryReluctant
                    You ain't half bad, for a butcher. Too bad—in another life, maybe you could've come and worked for me. 
                    <i>This</i> life will soon be over, though. 
                    !fight
            ]],

            DIALOG_NEGOTIATE_GOON_FAIL = [[
                player:
                    !shrug
                    $neutralDubious
                    You have a lot to live for, don't you?
                goon_leader:
                    !neutralDirect
                    I do. S'why I'm on the side that'll win. 
                    !fight
            ]],
            DIALOG_NEGOTIATE_GOON_SUCCESS = [[
                goon_leader:
                    !palm
                    $neutralResigned
                    Fine. I can't take up arms against you, {player}.
                    But you better do this right, or I'm dead.
                    !exit
                * The Spree goons melt into the shadows, leaving {other_handler} alone.
                other_handler:
                    !right
                    $neutralDubious
                    Well, that was impressive. But I'm afraid {goon_leader} ain't got long for this world once I'm done with you.
                    !fight
                    Let's see if you fight as good as you weaselword.
            ]],
            OPT_DEFEND = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    It didn't have to go like this!
            ]]
        }
        :Fn(function(cxt) 
            cxt.enc:SetPrimaryCast(cxt:GetCastMember("goon_leader"))
            cxt:Dialog(cxt:GetCastMember("goon_leader"):GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_GOON_CONFRONT_FRIEND" or "DIALOG_GOON_CONFRONT_NOT_FRIEND")

            cxt:Opt("OPT_NEGOTIATE_GOON")
                :Dialog("DIALOG_NEGOTIATE_GOON")
                :Negotiation{
                    
                    on_success = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATE_GOON_SUCCESS")
                        cxt:GetCastMember("goon_leader"):MoveToLimbo()
                        cxt:GoTo("STATE_FACE_NADAN")
                    end,
                    
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NEGOTIATE_GOON_FAIL")
                        cxt:Opt("OPT_DEFEND")
                            :Dialog("DIALOG_DEFEND")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                noncombatants = {cxt.quest:GetCastMember("other_handler")},
                                on_win = function(cxt) 
                                    cxt:Dialog("DIALOG_POST_GOON_FIGHT")
                                    cxt:GoTo("STATE_FACE_NADAN")
                                end
                            }
                    end,
                }

            cxt:Opt("OPT_FIGHT_GOONS")
                :Dialog("DIALOG_FIGHT_GOONS")
                :Battle{
                    noncombatants = {cxt.quest:GetCastMember("other_handler")},
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function(cxt) 
                        cxt:Dialog("DIALOG_POST_GOON_FIGHT")
                        cxt:GoTo("STATE_FACE_NADAN")
                    end
                }
        end)
    :State("STATE_FACE_NADAN")
        :Loc{
            DIALOG_POST_NADAN_FIGHT_DEAD = [[
                other_handler:
                    !exit
                    * {other_handler} lies dead at your feet, against {handler}'s orders.
                    * Your only hope now is that {handler} feels generous enough to offer partial credit.
            ]],
            DIALOG_POST_NADAN_FIGHT_ALIVE = [[
                other_handler:
                    !injured
                    $angryReluctant
                    All right, grifter. I'm beat. You win.
                player:
                    !angry
                    $neutralDirect
                    You going to come quietly now?
                other_handler:
                    !injured_shrug
                    $neutralResigned
                    Sure. I'm a reasonable man—I'll cooperate.
                    $angryCruel
                    After all, it's a long way back to headquarters. 
                    * There's a venom in his glance that leaves you wondering if maybe the hard part is still to come.
            ]],

            DIALOG_NOODLE_CHEF_LEAVE_SNAIL = [[
                * Before you leave, {noodle_chef} turns to you.
                noodle_chef:
                    !right
                    $angryReluctant
                    I'm done. You tell {handler} that I'm not for hire. The next time she sends a grifter, they better have a pedigree in hand.
                    !angry_accuse
                    <i>And</i> I'm taking my oshnu back.
                * {noodle_chef} leads the oshnu back in the direction of {noodle_chef.hisher} shop. 
                player:
                    !scared
                    $neutralResigned
                    Goodbye, my slimy friend.
            ]],
            DIALOG_NOODLE_CHEF_LEAVE_NO_SNAIL = [[
                * Before you leave, {noodle_chef} turns to you.
                noodle_chef:
                    !right
                    $angryReluctant
                    I'm done. You tell {handler} that I'm not for hire. The next time she sends a grifter, they better have a pedigree in hand.
                    !exit
                * {noodle_chef} runs back in the direction of {noodle_chef.hisher} shop.
            ]],
            DIALOG_SNAIL_NO_CHEF = [[
                agent:
                    !exit
                * Upset by all of the violence, {snail} retracts back into her shell. Nothing you do will convince her to come back out, so you have no choice but to leave her behind.
                player:
                    !disappoint
                    $miscDepressed
                    Be free, my friend.
            ]],
            OPT_FIGHT_NADAN = "Capture {other_handler}",
            NADAN_FIGHT_INSTRUCTIONS = "Capture {other_handler} alive!",
        }
        :Fn(function(cxt) 
            for k,v in pairs(cxt.quest.param.goons) do
                if not v:IsDead() then
                    v:MoveToLimbo()
                end
            end
            cxt.enc:SetPrimaryCast(cxt:GetCastMember("other_handler"))
            cxt:Opt("OPT_FIGHT_NADAN")
                :Battle{
                    reason_fn = function(battle)
                        return cxt:GetLocString("NADAN_FIGHT_INSTRUCTIONS")
                    end,

                    flags = BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function(cxt) 
                        if cxt:GetCastMember("other_handler"):IsDead() then
                            cxt:Dialog("DIALOG_POST_NADAN_FIGHT_DEAD")
                            cxt.quest:Activate("return_to_handler_empty")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        else
                            cxt:Dialog("DIALOG_POST_NADAN_FIGHT_ALIVE")
                            cxt:GetCastMember("other_handler"):Recruit( PARTY_MEMBER_TYPE.CAPTIVE )

                            local snail = cxt.quest:GetCastMember("snail")
                            if cxt.quest:GetCastMember("noodle_chef"):IsInPlayerParty() then
                                if snail and snail:IsInPlayerParty() then
                                    cxt:Dialog("DIALOG_NOODLE_CHEF_LEAVE_SNAIL")
                                    snail:Dismiss():MoveToLimbo()
                                else
                                    cxt:Dialog("DIALOG_NOODLE_CHEF_LEAVE_NO_SNAIL")
                                end
                                cxt.quest:GetCastMember("noodle_chef"):Dismiss():MoveToLimbo()

                            elseif snail and snail:IsInPlayerParty() then
                                cxt:Dialog("DIALOG_SNAIL_NO_CHEF")
                                snail:Dismiss():MoveToLimbo()
                            end
                            cxt.quest:Activate("return_to_handler")

                            StateGraphUtil.AddLeaveLocation(cxt)
                        end
                    end,
                }
        end)


QDEF:AddConvo("return_to_handler", "handler")
    :Loc{
        OPT_HAND_OVER = "Hand over {other_handler}",
        
        DIALOG_HAND_OVER = [[
            player:
                !salute
                $happyCocky
                Take it in, {handler}! I, Sal, have captured the second deadliest criminal in Murder Bay.
                $miscEmbarrassed
                And it only resulted in... maybe a few deaths. 
            other_handler:
                !left
            handler:
                !chuckle
                $happyCocky
                Oh, this is good. When was the last time I saw that beautiful face of yours, {other_handler}?
                !cruel
                $angryCruel
                You must be parched, with all that walking, and your face all full of holes. 
                $miscAlluring
                How about we get you a nice, long drink of water? And then maybe you and I can have a chat.
                !point
            * {handler} gestures for {other_handler} to be taken away. 
            player:
                !left
                $happy
                I take it you're satisfied?
            handler:
                !happy
                $happyDefault
                You did good, {player}. Better than most of my own have done. 
                $miscMischievous
                Tomorrow's the auction. Get a good rest tonight, because it's a big day for us both.
        ]],
        DIALOG_ASKED_ALL_QUESTIONS = [[
            agent:
                !handwring
                Now, I have a prisoner to interrogate. And you need sleep. 
                $neutralDirect
                I'll be having you work security, so show up looking presentable, and we'll get you through the door.
        ]],
    }
    :Questions("QUESTIONS", {
        "Ask about Nadan",
        [[
            player:
                !interest
                What'll you do with him, now that you've got him?
            handler:
                $neutralWhatever
                Standard interrogation. Find out what he was up to, what his feelers found out.
                $neutralThoughtful
                Maybe I'll even be able to learn something about Kashio.
                !eureka
                The two were a unit, once. He must have something up his sleeve we can exploit.
            player:
                !dubious
                $neutralDubious
                I didn't take him for much of a talker.
            handler:
                !humorous
                $happyJoke
                With that hole in his face?
                $neutralJoke
                It's literally impossible for his lips to be sealed.
        ]],
        "Ask about the auction",
        [[
            player:
                So tomorrow's the big day. You still good for your end of the bargain?
            handler:
                !handwave
                $miscAlluring
                Sal, please. A deal's a deal. I certainly don't see any advantage in cheating you now. 
                You've proven yourself an ally better kept happy. 
            {liked?
            handler:
                !happy
                $happyJoke
                Besides, you deserve your shot. And I plan to level your scope. 
            }]]
    })
    :Hub(function(cxt) 
    
        cxt:Opt("OPT_HAND_OVER")
            :Dialog("DIALOG_HAND_OVER")
            :ReceiveOpinion(OPINION.DID_LOYALTY_QUEST)
            :Fn(function() 
                cxt.quest:GetCastMember("other_handler"):Dismiss()
                cxt.quest:GetCastMember("other_handler"):MoveToLimbo()
                cxt.quest:Complete()

                cxt:RunLoop(function() 
                    local inserted = cxt:InsertQuestions("QUESTIONS")
                    if inserted == 0 then
                        cxt:Dialog("DIALOG_ASKED_ALL_QUESTIONS")
                        cxt:End()
                    else
                        StateGraphUtil.AddEndOption(cxt)
                    end
                end)
            end)
    end)    



    
   

QDEF:AddConvo("return_to_handler_empty", "handler")
    :Loc{
        OPT_TELL_DEAD = "Tell {handler} that {other_handler} is dead",
        DIALOG_TELL_DEAD = [[
            player:
                Sorry, {handler}. I know you wanted {other_handler.himher} alive, but {other_handler} is dead.
            handler:
                !palm
                ...
                $angrySeething
                Do you recall, {player}, that I specifically asked you to bring {other_handler.himher} in alive?
                {other_handler.HeShe} had information. Extremely valuable information. 
            player:
                !overthere
                $angryTakenAback
                Hey, at least your enemy is dead, right?
            handler:
                !angry_accuse
                $angryDefault
                I'd have let {other_handler} destroy the whole Bay just to increase the satisfaction of finally one-upping {other_handler.himher}. 
                $miscMocking
                I thought we were on the same wavelength, {player}. But it seems you're just a mindless goon.
                !crossed
                $neutralDirect
                Tomorrow's the auction. I'll get you in, as promised, but then our dealings are over. 
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_TELL_DEAD")
            :Dialog("DIALOG_TELL_DEAD")
            :ReceiveOpinion("no_capture_nadan")
            :FailQuest()
            :Travel()
    end)    


QDEF:AddConvo("return_to_handler_lying", "handler")
    :Loc{
        OPT_CONVINCE_ESCAPED = "Convince {handler} that {other_handler} escaped",
        
        DIALOG_CONVINCE_ESCAPED = [[
            player:
                !bashful
                $miscEmbarrassed
                Sorry, boss. But {other_handler} got away. 
            agent:
                !angry
                $angryDefault
                Of course he did. Go on, then. Give me your report.
        ]],
        
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                !overthere
                $neutralResigned
                We had to take a detour to avoid Admiralty patrols, and he slipped off while I slept. 
                I'd triple bound him and everything.
            agent:
                !disappoint
                $neutralResigned
                It wouldn't be the first time {other_handler} got the slip on a skilled professional.
                Some opportunities are just too good to be true. 
            player:
                !interest
                $miscPersuasive
                We're still good for tomorrow right? You'll help me get into the auction?
            agent:
                !agree
                My word's my word. We're good, {player}. You've done your part, and we need to take down Kashio.
                !eureka
                Besides, this just means I still get a chance to catch {other_handler} myself. 
        ]],

        DIALOG_CONVINCE_FAIL = [[
            player:
                !forgetful
                $neutralTakenAback
                He knocked me out when I wasn't looking! A real clock to the old noggin. It's a miracle I'm still alive, really!
            agent:
                !angry
                $neutralDirect
                Really? Because it sounds to me like you just screwed up and don't want to admit it.
                $angryDefault
                Fool of me to trust a grifter. I won't make the same mistake again. 
            player:
                !shrug
                $scaredDefault
                What about the auction tomorrow?
            agent:
                !angry_accuse
                $angryDefault
                Hah! And trust <i>you</i> to take on Kashio? 
                ...
                !disappoint
                $neutralResigned
                Fortunately for you, there's no one else. So we're still on for tomorrow.
                $angry
                $angryCruel
                But screw this up and I guarantee the only place you'll find work is back on the derrick. 
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_CONVINCE_ESCAPED")
            :Dialog("DIALOG_CONVINCE_ESCAPED")
            :Negotiation{
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                    cxt.quest:Complete()
                    StateGraphUtil.AddEndOption(cxt)
                end,

                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_CONVINCE_FAIL")
                    cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("no_capture_nadan"))
                    cxt.quest:Fail()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end,
            }
    end)    


QDEF:AddConvo("return_to_handler")
    :ConfrontState("THE_STATE", function(cxt) return not cxt.quest.param.has_offered_deal and cxt.location:HasTag("in_transit") end)   
    :Loc{
        DIALOG_INTRO = [[
            * {other_handler} makes the journey as difficult as possible, and laces it with vague threats.
            * But when it's clear there's nobody coming to rescue {other_handler.himher}, his mood shifts. 
            * He sulks for a time, but eventually he breaks the silence open like a fresh egg.
            player:
                !left
            agent:
                !right
                !dubious
                $neutralDubious
                You got nerve, hunter. You gonna be content to stay in Murder Bay workin' for a lead-pusher like {handler}?
                $miscAlluring
                She doesn't have the gumption to lift you to the heights you really want.
            player:
                !happy
                $happyCocky
                   Actually, she's getting me exactly where I wanna be. At Kashio's throat.
            agent:
                !interest
                $neutralThoughtful
                You're after Kashio? Huh. You shoulda come to <i>me</i>, hunter. I coulda made it real.
                !agree
                $miscPersuasive
                You want powerful people who owe you a favor. After Kashio, then what?
                So let me go. Tell {handler} I got away. 
            player:
                !crossed
                $neutralDubious
                And I would do that because... ?
            agent:
                !cruel
                $miscMischievous
                I got know-how in this noggin worth more than any Heshian relic. 
                You want it? It's yours.
            *** {other_handler} is offering to give you a Spree faction card if you let {other_handler.himher} go, and convince {handler} that {other_handler.heshe} escaped.

        ]],
        OPT_ACCEPT = "Accept the offer",
        
        DIALOG_ACCEPT = [[
            player:
                !agree
                $neutralThanks
                Alright. Impart away.
            agent:
                !neutralDirect
                See? You're too smart to waste it on the Admiralty.
            * You untie {other_handler}, and he whispers secrets on the Spree that give you an advantage you never expected.
        ]],
        OPT_DECLINE = "Decline the offer",
        DIALOG_DECLINE = [[
            player:
                !hips
                $neutralDirect
                Nope. Believe it or not, my word means more than just a paycheck.
            agent:
                !spit
                $angryCruel
                Then I mistook you for shrewd. Guess you're just another schmuck, after all.
            * He falls back into sullen silence for the remainder of the trek.
        ]],
        CARD_PICK_TITLE = "Choose a Spree Card",
        CARD_PICK_SUBTITLE = "",
        DIALOG_GOOD_BYE = [[
            agent:
                !tired
                Don't die before we meet again, hunter. I reckon we both have a lot to accomplish.
                !exit
        ]],

    }
    :Fn(function(cxt) 
        cxt.quest.param.has_offered_deal = true
        cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("other_handler"))
        cxt:Dialog("DIALOG_INTRO")

        cxt:Opt("OPT_ACCEPT")
            :Dialog("DIALOG_ACCEPT")
            :ReceiveOpinion(OPINION.SPARED_LIFE)
            :Fn(function() 
                    cxt.enc:OfferCards(GetUniqueFactionCards(cxt.player, "BANDITS", 3, 0), cxt:GetLocString("CARD_PICK_TITLE"), cxt:GetLocString("CARD_PICK_SUBTITLE")) 
                end)
            :Dialog("DIALOG_GOOD_BYE")
            :Fn(function() 
                cxt.quest:GetCastMember("other_handler"):Dismiss()
                cxt.quest:GetCastMember("other_handler"):MoveToLimbo()
                cxt.quest:Cancel("return_to_handler")
                cxt.quest:Activate("return_to_handler_lying")
            end)


        cxt:Opt("OPT_DECLINE")
            :Dialog("DIALOG_DECLINE")
            :Travel()

    end)






QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            * {agent}'s expression is unusually stormy. 
            agent:
                !angry
            player:
                !greeting
                All right, {handler}? You look like you found the gristle.
            agent:
                !eureka
                $neutralResigned
                Just a bad idea percolating. But hey, you're the perfect agent for all my bad ideas. 
                $miscAlluring
                How'd you like to go after the <i>second</i> biggest bounty in Murder Bay?
            player:
                !thought
                $neutralThoughtful
                Second biggest, huh? Might be good practice for Kashio. 
                Is this about that Spree bigwig? {other_handler}?
            agent:
                !agree
                $neutralDirect
                Got it in one. Intel says he's been <i>personally</i> attacking all our caravans escorting holy relics for the auction. 
                $miscMischievous
                Well, I have one caravan left. And it makes sense to guard the honey with my sharpest stinger. 
                !cruel
                Especially if the stinger's yours.
            player:
                !dubious
                $neutralJoke
                Flattery doesn't pay the bills, {handler}—but it does keep the lights on.
        ]],
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


 
QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_ON_TAKEN = [[
            agent:
                !point 
                $neutralDirect
                You do this job and you'll get into that auction with VIP treatment, if I have my way. 
                But remember: I want Nadan captured, not killed. He's worth far more to my career alive.
            player:
                !crossed
                $neutralResigned
                He seems like he'd be hard to kill, anyway.
            agent:
                $neutralDirect
                Go get an oshnu from {noodle_chef}, and then take it on the merchant route. You'll look just like the other caravans.
                !cruel
                $miscMischievous
                I'll make sure word gets back to the Spree, and maybe hint that <i>this</i> caravan is my personal favorite. {other_handler} won't be able to resist.
        ]],
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_ON_TAKEN")

            cxt:Opt("OPT_ASK_ABOUT")
               :IsHubOption(true)
               :GoTo("STATE_QUESTIONS")     

            StateGraphUtil.AddEndOption(cxt)
                --:Dialog("DIALOG_TELL_ABOUT_HELPER")
        end)

    :AskAboutHub("STATE_QUESTIONS", 
             {

               "Ask about the caravans",
               [[
                   player:
                        !dubious
                        $neutralThoughtful
                        What were the caravans transporting?
                   agent:
                        !handwave
                        $neutralWhatever
                        Relics for the Cult, mostly. You know the Admiralty's trying to get the Heshians on side for the annex. 
                        We had "offerings" we were transporting. Best security in the Bay, you figure.
                        !disappoint
                        $angryReluctant
                        {other_handler} must've figured the blow would sting twice as hard—on our coffers and our authority. 
                        Pretty sure he knew I was overseeing it, too, which would just be the meringue on the seaslug for him.
                    player:
                        !wince_critical
                        $neutralDubious
                        Is that... meant to be delicious?
                    agent:
                        !chuckle
                        $miscMischievous
                        It's a Havarian delicacy, slick! Best enjoyed alongside a dish of cold, spiteful revenge, of course.
                        $miscAlluring
                        You do this job and I'll treat you to it. 
                    player:
                        !dubious
                        $neutralDubious
                        Hm.
               ]],
               "Ask about {other_handler}",
                [[
                    player:
                        !crossed
                        You think {other_handler} would make this personal? 
                    agent:
                        !happy
                        $happyDefault
                        Oh, most definitely. {other_handler.HeShe} <i>hates</i> me. 
                        All because I maybe might've blown him up once.
                    player:
                        !interest 
                        $neutralTakenAback
                        Right, so he <i>is</i> hard to kill, then. What happened?
                    agent:
                        !overthere
                        $neutralWhatever
                        I got a tip that Kashio was moving slurry into Murder Bay. We caught up, but the bandits fought back <i>hard</i>. 
                        Took out half my squad all over some measly contraband. I figured if the goods were destroyed, they might cut their losses and run. 
                    player:
                        !wink
                        $miscMischievous
                        Sounds like giving up.
                    agent:
                        !point
                        $neutralResigned
                        Sounds like surviving to regroup, you mean. Only problem was... they weren't carting slurry. It was explosives. 
                        Killed all the bandits except Nadan. And Kashio, because it turns out she wasn't even there. 
                        !wince
                        $angryAnnoyed
                        I think Kashio might've been the one to feed me the intel.
                    player:
                        !agree
                        So it was a... mistake? Blowing him up?
                    agent:
                        !handwave
                        $neutralWhatever
                        The only mistake was I didn't blow his whole head off. Sloppy. 
                        $miscPersuasive
                        But you get him, then tomorrow you get Kashio. Then we'll be sitting pretty. 
                ]]
             })  