local NUMBER_OF_JOBS_TO_DO = 2
local NUMBER_OF_JOBS_TO_SPAWN = 3

local CHUM_BONUSES = StateGraphUtil.ValidateChumBonuses(
{
    {
        fn = StateGraphUtil.OfferSpareChange( 100 ),
    },
    {
        fn = StateGraphUtil.OfferUpgradeNegotiation(),
    },
    {
        fn = StateGraphUtil.OfferUpgradeBattle()
    },
    {
        fn = StateGraphUtil.OfferCard( "amnesiator" )
    },
    {
        fn = StateGraphUtil.OfferCompressionGear()
    },
    {
        fn = StateGraphUtil.OfferTriageKit()
    },
    {
        fn = StateGraphUtil.OfferBattleDraft( 2 )
    },
    {
        fn = StateGraphUtil.OfferNegotiationDraft( 2 )
    },
    {
        fn = StateGraphUtil.OfferBoomBox( 2 )
    },
    {
        fn = StateGraphUtil.OfferGraftSlot( GRAFT_TYPE.COMBAT, 1 )
    },
    {
        fn = StateGraphUtil.OfferGraftSlot( GRAFT_TYPE.NEGOTIATION, 1 )
    },
})


local QDEF = QuestDef.Define
{
    title = "Welcome to Havaria",
    desc = [[ Make contact with Fssh, and establish yourself in the world]],
    icon = engine.asset.Texture("icons/quests/sal_story_act1_huntingkashio.tex"),
    qtype = QTYPE.STORY,
    
    on_start = function(quest)
        quest:Activate("meet_fssh")
    end,
    
    avoids_fn = function(quest, agent, location)
        if location == TheGame:GetGameState():GetLocation("GROG_N_DOG") and agent:IsQuestProvider() then
            return true
        end
    end,

    events = 
    {
        quests_changed = function(quest, event_quest) 

            --kind of gross to do this this way, but the quest mechanisms we want to look at are all commonized. 
            if quest:IsActive("pick_market_job") then
                if quest.param.market_quests then
                    for k,v in pairs(quest.param.market_quests) do
                        if v == event_quest then
                            if event_quest:IsComplete("offer_job") then
                                quest:Complete("pick_market_job")
                                quest.param.market_quest = event_quest
                                for k,v in pairs(quest.param.market_quests) do
                                    if v ~= event_quest then
                                        v:Cancel()
                                    end
                                end

                            end
                            break
                        end
                    end
                end
            end

            if quest:IsActive("do_market_job") and event_quest == quest.param.market_quest then
                if not event_quest:IsActive() then
                    quest:Complete("do_market_job")
                    if event_quest:IsComplete() then
                        QuestUtil.SpawnQuest("ROAD_SIDE_MERCHANTS")
                    end
                end
            end
        end                
    },
    
}


:AddCastByAlias{
    cast_id = "kashio",
    alias = "KASHIO",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}

:AddCast{
    cast_id = "guard",
    when = QWHEN.MANUAL,
}

:AddCast{
    cast_id = "kashio_goon",
    when = QWHEN.MANUAL,
}

QuestUtil.AddSideJobChain(QDEF, NUMBER_OF_JOBS_TO_SPAWN, NUMBER_OF_JOBS_TO_DO, "talk_to_fssh", true, true)


QDEF:AddObjective{
    id = "meet_kashio",
    title = "Return to the Grog",
    desc = "You've completed Fssh's job. Return to the Grog to see if she's found any leads for you.",
    mark = {"fssh"},

    on_activate = function(quest)
        --QuestUtil.SpawnOpportunities()
    end,

    on_complete = function(quest)
        quest:Activate("sleep_day_one")
    end,
}

:AddObjective{
    id = "talk_to_fssh",
    mark = {"fssh"},
    title = "Talk to {fssh}",
    desc = "Fssh has a special job for you",
    on_activate = function(quest)
        UIHelpers.PassTime(DAY_PHASE.NIGHT)
        QuestUtil.SpawnOpportunities()
    end,
    on_complete = function(quest)
        quest:Activate("pick_market_job")
    end,
}

:AddObjective{
    id = "pick_market_job",
    title = "Talk to one of the merchants",
    desc = "The merchants both need a favour. Choose one and help them out.",
    on_complete = function(quest)
        quest:Activate("do_market_job")
    end,
}

:AddObjective{
    id = "do_market_job",
    title = "Help one of the merchants",
    on_complete = function(quest)
        quest.param.boss_id = TheGame:GetGameProfile():GetNoStreakRandom("SAL_DAY_1_BOSS_PICK", {"DRONE_GOON", "SPARK_BARON_BOSS", "RISE_TURNCOAT_BOSS"}, 2)
        quest:Activate("meet_kashio")
    end,
}

:AddObjective{
    id = "sleep_day_one",
    title = "Rest to regain your strength",
    desc = "It's been a long day for you! Sleeping will give you a chance to heal and recharge your abilities",
    mark = {"fssh"},
    on_activate = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true
        --makes debugging the day out a lot easier
        if quest:IsActive("pick_a_side_job") then
            quest:Cancel("pick_a_side_job")
        end
    end,
    on_complete = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
    end,
}

:AddObjective{
    id = "meet_fssh",
    --state = QSTATUS.ACTIVE,
    on_complete = function(quest)
        LocationUtil.SendPatronsAway( TheGame:GetGameState():GetLocation("GROG_N_DOG") )        
        quest:Activate("pick_a_side_job")
    end,
}


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("meet_fssh")
    :Confront(function(cxt)
        
        if TheGame:GetLocalSettings().FAST_STARTUP then
            cxt.quest:Complete("meet_fssh")
        else
            return "STATE_TALK"
        end
    end)

    :State("STATE_TALK")
        
        :Loc{

            DIALOG_INTRO = [[
                * At first sniff the Grog n' Dog smells like stale hops and dried blood. 
                fssh:
                    !right
                    !dubious
                    $neutralResigned
                    Well, look what the tide dredged up. 
                player:
                    !left
                    !greeting
                    $happyDefault
                    {fssh}! By the holy brine. I'd tear up if I weren't dead inside. 
                fssh:
                    $happyDefault
                    It's damn good to see you looking well. You have a plan to get on your feet?
                player:
                    !handwring
                    $happyCocky
                    I do: that <i>bounty</i> on {kashio}. How do we—
                Fssh:
                    !crossed
                    $neutralDirect
                    Forget it. You might have grown up strong but you ain't <b>that</> strong.
                player:
                    !scared
                Fssh:
                    !sigh
                    But that's not going to stop you from trying, is it? 
                    !point
                    $neutralDirect 
                    If you want a <i>chance</> at surviving this, you better make yourself some friends, fast. 
                    !thumb
                    I can help you with that.
                player:
                    !cruel
                    $happyDefault
                    Knew I could count on you, {fssh}.
            ]],



            DIALOG_FSSH_CONVO_REPLAY_1 = [[  
                * The Grog n' Dog has its charm: the patrons look lively, and the floorboards creak like a well-fed toad. 
                player:
                    !left
                fssh:
                    !right
                    !greeting
                    $miscRelieved
                    There you are. I started to think you mighta gotten mugged before you even made it through the door.
                player:
                    !interest
                    $neutralJoke
                    Your faith moves me, {fssh}. But I <i>do</i> feel ready for a fight. Like my blood is pumping, ya know? 
                    !eureka 
                    $happyDefault
                    Should serve me well up against {kashio}.       
                fssh:
                    !chuckle
                    $neutralJoke
                    First breath of fresh air in ten years, and you get straight to the point. 
                    But you won't get nowhere without making some friends. Get your bearings, and maybe tomorrow we can talk about {kashio}.
                    !permit
                    $miscRelieved
                    Let me give you something. People leave the damnedest things behind in the bar.

                ** {fssh} will give you something every time you start over.
            ]],

            DIALOG_FSSH_CONVO_REPLAY_2 = [[  
                fssh:
                    !right
                * {fssh} is waiting for you, her hip cocked against the bar. 
                player:
                    !left
                fssh:
                    !greeting
                    $happyDefault
                    Long time, Sal. But it feels like I just saw you.
                player:
                    !bashful
                    $happyResigned
                    Guess I ain't changed that much, huh? 
                fssh:
                    !crossed
                    $neutralResigned
                    Some things don't. And knowing you, there's one thing you're after: that bounty.
                    You do some jobs today, and I promise I'll help you out tomorrow. 
                    !give
                    $neutralDirect
                    For now, take this—to welcome you home. 
            ]],


            DIALOG_FSSH_CONVO_REPLAY_3a = [[  
                fssh:
                    !right
                * {fssh} welcomes you, and it's back to the hunt for another day. 
                player:
                    !left
                fssh:
                    !sigh
                    $neutralResigned
                    You look like you got plans to take on the biggest crime lord in the Bay, no matter what I might say to stop you.
                    !give 
                    Just promise me you won't do it without help. 
            ]],
            DIALOG_FSSH_CONVO_REPLAY_3b = [[  
                fssh:
                    !right
                * {fssh} has a drink waiting for you. Even though it's been years, she remembered your favorite.
                player:
                    !left
                fssh:
                    !offer
                    $neutralDirect
                    You're not the only one who wants {kashio} out of the picture.
                    !agree
                    $happyDefault
                    Play your cards right, and that revenge you want so bad? It'll deliver itself.

            ]],
            DIALOG_FSSH_CONVO_REPLAY_3c = [[  
                fssh:
                    !right
                * You're back in Havaria, and {fssh} welcomes you like you were only gone for a wink.
                player:
                    !left
                fssh:
                    !eureka
                    $neutralDirect
                    Welcome back, Sal. Let's not waste much time, huh? You got work to do, and I got things to help you do it. 
            ]],

            OPT_GET_WORK = "Get work from {agent}",
            DIALOG_TALK_ABOUT_GIVERS = [[
                agent:
                    !shrug
                    $neutralDirect
                    I got people in my pocket who owe me a favor: {1#agent_list}. They're easy people to find.
                    !permit
                    $neutralDirect
                    Tell 'em I sent you, and they can settle their tabs by giving you work. Get that done, and I'll get you a plate of popped oshnu eyes on the house.
                player: 
                    !wink
                    $miscNostalgic
                    Bless your warty hide, Fssh. I'll hold you to that.
                *** Fssh has some jobs you can do to get strong enough to take on {kashio}.
            ]],


            OPT_REMIND_ME = "Let's chat",
            

            DIALOG_REMIND_ME = [[
                player:
                    !thought
                    $miscNostalgic
                    I can take a second to catch up. It's been long enough.
                Fssh:
                    !permit
                    $miscNostalgic
                    Good to see you in one piece, after what you've been through. But could you maybe try to blend in a little?
                    !dubious
                    $neutralResigned
                    Bounty hunters are bad for business, and this place feels emptier already.
                * Sure enough, a couple of patrons make for the exit, their drinks still bubbling in their mugs.
                player:
                    !overthere
                    $neutralTakenAback
                    I don't wanna cause you trouble. You want me gone, I'm gone.
                Fssh:
                    !point
                    $neutralDirect
                    Suckerspit. It took you ten years to earn your freedom. I'd be a piss-poor friend to your folks if I let you throw it in the gutter now.
                    $neutralThanks
                    You're staying. At least until you get on your feet.
                player:
                    !wince2
                    Kindness makes me uncomfortable. But... thanks.
            ]],

            DIALOG_REMIND_ME2 = [[
                player:
                    !thought
                    $miscNostalgic
                    It's been a long time, Fssh. Last I saw you... Hesh, that must've been⁠—
                fssh:
                    !agree
                    $miscRelieved
                    It was before your folks, uh... Well, before they died, anyway.
                player:
                    $neutralWhatever
                    Yeah. 
                fssh:
                    !interest
                    $miscNostalgic
                    Yeah. Well. You look like your dad, ya know.
                player:
                    !bashful
                    $miscEmbarrassed
                    Livelier, I hope. So, uh... you got work for me?

            ]],

            DIALOG_REMIND_ME3 = [[
                fssh: 
                    !thought
                    $neutralThoughtful
                    Funny, you know. It feels like you never even left.
                player:
                    !happy
                    $neutralJoke
                    Wish I could say the same. 
                fssh:
                    !agree
                    $neutralResigned
                    Enh. You're home where you belong, now. That's enough for today.
                player:
                    !dubious
                    !neutralJoke
                    Hesh, {fssh}. That's very sentimental of you.
                fssh:
                    !bashful
                    !miscEmbarrassed
                    Right. Back to shop talk, then. 
            ]],

            DIALOG_REMIND_ME4 = [[
                player:
                    !thought
                    $neutralThoughtful
                    This bar looks real familiar. Have I... been here before?
                fssh:
                    !shrug
                    To be honest, I mighta decorated it like the one in Grout Bog. 
                    !chuckle
                    $miscMischievous
                    You know the one. You'd always have to come in looking for your Ma and me, 'cause we'd be playing cards so long the sun rose. 
                player:
                    !agree
                    $neutralResigned
                    They closed that bar off after you left the Bog. They said it was a spawning ground for dissent.
                fssh:
                    !dubious
                    $neutralDubious
                    Huh. But plenty of new memories to make here, am I right?
                    $happyDefault
                    Let's get you to work on that bright, shiny future of yours.

            ]],

            DIALOG_REMIND_ME5 = [[
                player:
                    !scared_shrug
                    $neutralResigned
                    I feel like I've dreamt about this a hundred times, but in my dreams it always turns sour.
                    $neutralResigned
                    I end up on the wrong end of someone's grudge and it all goes to hell. 
                fssh:
                    !permit
                    You're home, kid. But I'd be lyin' if I said life here was gonna be much easier than life on the derrick. 
                    $neutralDirect
                    The big difference is you can make progress. You can move forward.  
                    !point
                    Might take brains, and flex, and a bit of luck. But you got all three. 
                player:
                    !dubious
                    $neutralJoke
                    Well, geez, Fssh, that was... actually a decent pep talk.  
                    !eureka
                    $happyDefault
                    So... let's get started, I guess.
            ]],


        }

        :SetLooping()
        :Fn(function(cxt) 

            if cxt:FirstLoop() then

                cxt.quest:Complete("meet_fssh")
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("fssh"))

                --we can also do more complex logic, manually:
                -- local progress = cxt:ProgressionCheck("fssh_first_convo")
                -- this will get the current value of the given progression, and increment it

                local replay_num = TheGame:GetGameProfile():GetNarrativeProgress( "heard_fssh_introduction" )
                if replay_num == 0 then
                    cxt:Dialog("DIALOG_INTRO")
                
                elseif replay_num == 1 then
                    cxt:Dialog("DIALOG_FSSH_CONVO_REPLAY_1")

                elseif replay_num == 2 then
                    cxt:Dialog("DIALOG_FSSH_CONVO_REPLAY_2")

                else
                    local opts = {
                        "DIALOG_FSSH_CONVO_REPLAY_3a",
                        "DIALOG_FSSH_CONVO_REPLAY_3b",
                        "DIALOG_FSSH_CONVO_REPLAY_3c",
                    }
                    cxt:Dialog(table.arraypick(opts))
                end
                
                if (replay_num or 0) > 0 then
                    cxt:RunLoopingFn( StateGraphUtil.OfferChumBonuses, CHUM_BONUSES)
                end
            end
            
            cxt:Opt("OPT_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")
            
            
            cxt:Opt("OPT_GET_WORK")
                :PreIcon( global_images.quest )
                :Fn(function() 
                    local givers = {}
                    for k, v in ipairs(cxt.quest.param.job_options) do
                        local provider = v:GetProvider()
                        if provider then
                            table.insert(givers, provider)
                        end
                    end
                    cxt:Dialog("DIALOG_TALK_ABOUT_GIVERS", givers)
                end)
                :Fn(function() 
                    StateGraphUtil.AddLeaveLocation(cxt)
                        :Fn(function()
                            TheGame:GetGameProfile():IncNarrativeProgress("heard_fssh_introduction")
                        end)
                end)
        end)



    --These can have complex requirements, like specific playthrough numbers, or whatever. Or we can make an entirely different sets for the first time vs intermediate times vs the steady-state time
    :AskAboutHubConditions("STATE_QUESTIONS", 
    {
        function(cxt)
            return true
        end,
        "Ask about Havaria",
        [[
            player:
                !permit
                $neutralDefault
                Help me get my bearings. It's been a long time since I was last in Havaria.
            agent:
                !shrug
                $angryReluctant
                It's dirty and broken, like always. If debt brokering was bad when you were a kid, it's worse now. 
                Half the population is indentured, and them's that hold contracts sell them to pay off their own. 
            player:
                !thought
                $neutralThoughtful
                No wonder customs was so uptight on my way back in. Must be a lot of fraud. 
            agent:
                !permit
                $angryDefault
                That ain't the half of it. The Admiralty is pushing for an official annex. 
                They wanna make everyone real Deltrean citizens, with papers and everything.
            player:
                !dubious
                $neutralDubious
                In Havaria? You're joking.
            Fssh:
                $neutralResigned
                !disappoint
                They say it'll stop illegal debt brokering. Now anyone with a hand in labor contracts is real ornery—from the poor to the powerful.
                !chuckle
                $neutralJoke
                You can imagine the reaction.
            player:
                !wince
                $miscHopeless
                I can.
        ]],
        function ( cxt )
            cxt.quest.param.asked_about_havaria = true
        end,

        function(cxt)
            return cxt.quest.param.asked_about_havaria ~= nil
        end,
        "Ask about the annex",
        [[
            player:
                $neutralDubious
                The annex can't be a popular move. Most come here to get <i>away</i> from the Admiralty.
            agent:
                $angryDefault
                It ain't. Both the Spree <i>and</i> the Cult are in arms about it. The Spree 'cause they're criminals, the Cult 'cause they profit off the labor more than anybody.
            player: 
                $miscDepressed
                !sigh
                No need to tell me that.
            agent:
                !interest
                $neutralDubious
                The Cult's holding a big shindig in a few days—their usual wheelin' and dealin' in the name of the divine.
                Word is, the Admiralty's gonna use it to make their proposition: <i>mandatory citizenship</i>. 
                !disappoint
                $neutralResigned
                It's gonna need to be a mighty good pitch to convince the Cult. And I wouldn't put it past the Spree to set fire to the whole affair, if they can.
            * {fssh} casts a dour eye over the bar, like it's a garden on the near side of winter.
            agent:
                !tired
                $miscDepressed
                Suffice it to say, kid, tensions are <i>real</i> high. You'd be smart to keep your head <i>down</i> instead of looking for trouble.
        ]],
        function() end,

        function(cxt)
            return true
        end,
        "Ask about Fssh",
        [[
            player:
                !interest
                $neutralDubious
                So what do I owe you for this, Fssh? A cut of my takings?
            agent:
                !wave
                $neutralDirect
                Hesh no. I left my bounty hunting days behind me. I owed your parents, once. I don't like leaving debts unsettled. Even to dead folks.
                $miscNostalgic
                They were decent people, Sal Fallon. Watch you don't end up like them.
            player:
                !dubious
                $miscDepressed
                You know even when you pay off the debts, the derrick still owns your name? I'm an ik-Derrick now. Or so they tell me.
            agent:
                !spit
                $angryDefault
                Yeah. That's some messed up bladderwrack, though.
            player: 
                !disappoint
                $miscDepressed
                Yeah.
        ]],
        nil,

        nil,
        "Ask about Kashio",
        [[
            player:
                $neutralThanks
                You lined up work for me real fast. You some kind of mover, here?
            agent:
                !handwave
                $neutralWhatever
                Naw, I just know people, and people know me. 
                You don't need to go after Kashio, kid. I have enough work to keep you busy.
            player:
                !dubious
                $neutralTakenAback
                I spent my childhood in a lumin pipe because of her. And now she's worth more money than either of us can count.
            fssh:
                !shrug
                $angryAside
                And she's <i>more dangerous</i> than either of us can figure. You go after her, and she'll clap back with a knife for each kidney.
                !permit
                $neutralThoughtful
                I may have extras to spare, but you only got two.
            player:
                !thumb
                $angryDefault
                I worked off a lifetime's debt in ten years just so I could get back here and settle the score. 
                !crossed
                If you were my friend, you wouldn't be tellin' me to drop it.
            Fssh:
                !scared_shrug
                $neutralResigned
                Aw, Hesh, that ain't what I meant. You know it ain't.  
            player:
                !sigh
            fssh:
                !neutral
                I'll give you a roof, a bed, and at least one good meal a day. You live your life the way you want. It's yours.
            player:
                !neutral
                !agree
                $miscRelieved
                I'll pay you back. 
            fssh:
                !agree
                $miscRelieved
                I know. Let's get you to work.
        ]],
        function() end,


    })        

------------------------------------------------------------------------------------------------------ THIS IS ABOUT SLEEPING AT THE GROG

QDEF:AddConvo("meet_kashio", "fssh")
    :ConfrontState("STATE_KASHIO_STUFF")    
        :ProgressionLoc{
            DIALOG_INTRO = {  
                --1
                [[ 
                    kashio:
                        !angry
                        !right
                    fssh:
                        !left
                        !injured
                    * It's late when you get back to the Grog n' Dog, but you can spot trouble before you even step onto its porch.
                    * You slip quietly behind an overturned table before anyone notices.
                    kashio:
                        !angry
                        $angryHostile
                        I hear you've been asking after my bounty. But you're outta the game, {fssh}. So you must be askin' for someone else.
                        I don't hold it against you. But whatever hunter you're hiding, you need to hand them over.
                    fssh:
                        $angryHostile
                        There's no hunter here, Kashio. But we both know that won't stop you from being a mudsucker.
                    kashio:
                        !angry_accuse
                        $angryHostile
                        I've a business to run, same as you. There was a time when you respected that.
                    * The slimy underside of Kashio's voice is enough to draw you out of your hiding spot, blade at the ready.
                    player:
                        !left
                        !fight
                    kashio:
                        !surprised
                    player:
                        $angrySeething
                        !angry_threaten
                        Don't be so smug, {kashio}. Nobody gets a bounty like yours without enemies.
                    kashio:
                        !dubious
                        $happyTakenAback
                        <i>Sal</i>? You're the hunter? I'd have thought the derrick would've beaten the spirit out of you.
                    player:
                        !angry_shrug
                        $angrySeething
                        It didn't have to be this way, Kashio. You could've bought me out at any time. I'd have forgiven you.
                    kashio:
                        !interest
                        With that look in your eye? I don't think so. 
                        $neutralResigned
                        Don't mistake this for condescension. I'm <i>proud</i> you made it off the derrick. I should've known you would. 
                        $neutralResigned
                        Excuse me if I don't stick around to see this done. It'll hurt too much to see you go down.
                    * Kashio tosses a fistful of shills onto the soiled bar counter.
                    kashio:
                        !give
                        $angryCruel
                        For the damages, Fssh, if you manage to survive. It ain't nothing personal, after all.
                        !exit
                    * Not even the floorboards creak as Kashio leaves. For a moment her goons hang there, watching to see what you'll do.
                    *** Kashio leaves her goons to finish you off.
                ]],
                --2
                [[
                
                * Something in the Grog n' Dog isn't right, but you've learned to expect trouble. You approach from the shadows.
                kashio:
                    !angry
                    !right
                fssh:
                    !left
                    !injured
                kashio:
                    !cruel
                    $miscMocking
                    Why, <i>Fssh</i>. This is a lot of hostility comin' from someone I used to think was a friend.
                fssh:
                    $angryAside
                    And how many friends do you got left these days? That bounty on your head didn't come from nowhere.
                kashio: 
                    !angry
                    $angryDefault
                    About that⁠—
                * You know an entrance when you see it. 
                player:
                    !left
                    !cruel
                    $angryCruel
                    Yeah. <i>About</i> that. 
                kashio:
                    !interest
                    ...
                    !chuckle
                    $neutralJoke
                    Thought it might be you, Sal. I always knew you'd either die on that derrick or come back for my jugular. 
                    !disappoint
                    $angryAside
                    Once was a time when you said you wanted to grow up to be just like me. But <i>I</i> would've been smart enough to keep my head on my shoulders.
                * Kashio jerks her thumb in your direction, speaking to the two goons at her side.
                kashio:
                    $angrySeething
                    Kill her. And do it right.
                    !exit
                agent:
                    !right
                player:
                    !fight
                * You make to pursue Kashio as she leaves, but the goon blocks your path. No way to her unless it's through the wall of hired muscle.
                *** Kashio leaves her goons to finish you off.
            ]],
            --3
            [[
                kashio:
                    !angry
                    !right
                fssh:
                    !left
                    !injured
                * Your gut tells you that something will be amiss even before you reach the front door. You wait, just within earshot.
                kashio:
                    !angry
                    $angryHostile
                    Just tell me where the hunter is, Fssh, and we can both go back to our lives. 
                fssh:
                    $angryCruel
                    That's the thing, Kash. <i>Your</i> life has a price, now. Sooner or later, someone's gonna claim it.
                kashio:
                    !angry_accuse
                * Kashio raises her hand as if to strike, but you peel out of the shadow with your dagger glinting.
                player:
                    !left
                    !threaten
                    $angrySeething
                    You wanna watch where that fist comes down, Kashio. You aim it reckless like that and someone might cut it off.
                Kashio:
                    !cruel
                    $miscAlluring
                    <i>There</i> you are. I thought that might draw you out. 
                    $miscMocking
                    That's the reason I sold you off, you know, rather than training you like I said I would. I knew you'd only grow up soft.
                * But her demeanor is changed. She makes for the door real quick, tossing an order to her goon. 
                    $angrySeething
                    Bash her head in. I promise her head is as soft as the rest of her.
                    !exit
                *** Kashio leaves her goons to finish you off.
            ]],
            --4
            [[
                kashio:
                    !angry
                    !right
                fssh:
                    !left
                    !injured
                * You know what you'll find in the Grog n' Dog: Kashio always had a way of turning up, like a splinter. 
                kashio:
                    !angry_accuse
                    $angryHostile
                    Where's the hunter, Fssh? Where's Sal?!
                player:
                    !left
                    !cruel
                    $angryCruel
                    Halfway to putting a blade to your neck, Kashio. 
                * You rush forward to see the job done, but she sidesteps you easily, the brunt of her elbow coming down on the base of your skull.
                * You blink the stars out of your eyes and recover, like she taught you to, once—but she's at the door. She lingers just long enough to see her goon close in.
                agent:
                    !right
                player:
                    $angryHostile
                    !angry
                    Kashio!
                * But she's already gone.
                *** Kashio leaves her goons to finish you off.
            ]],
            --5 and final
            [[
                kashio:
                    !angry
                    !right
                fssh:
                    !left
                    !injured
                * It's late when you get back to the Grog n' Dog, but you can spot trouble before you even step onto its porch.
                * You slip quietly behind an overturned table before anyone notices... or so you think.
                player:
                    !left
                    !surprised
                * Kashio's hand grabs you by the collar, yanking you out into the weak lumin light of the bar. 
                player:
                    !injured
                    Urk!
                kashio:
                    !fight
                * She kicks you hard in the ribs, but you manage to keep your breath. You're gonna need it.
                kashio:
                    Stay. Down. 
                    !exit
                *** Kashio leaves her goons to finish you off.

            ]],
            },
         
        }
        :Loc{

            DIALOG_GOON_THREATEN = [[
                agent:
                    !right
                agent:
                    !throatcut
                    $angryGreeting
                    Go down quiet, hunter. I promise it'll be just like goin' to sleep.
            ]],
            OPT_REQUEST_BACKUP = "Convince {1#agent} to help you",
            DIALOG_REQUEST_BACKUP = [[
                guard:
                    !right
                player:
                    {guard}, you're the bouncer!
                guard:
                    I'm hired to keep ruffians out. Kashio's a class of her own. 
            ]],
            DIALOG_CONVINCED = [[
                player:
                    You gonna make me do <i>your</i> job for you?
                guard:
                    Naw, I guess not. All right.
                * {guard} steps up to your side, ready to defend the Grog n' Dog. Better late than never, you suppose. 
                    !exit
                agent:
                    !right
            ]],
            DIALOG_NOT_CONVINCED = [[
                player:
                    What are you, a coward?
                guard:
                    You know what? I just might be.
                * {guard} makes for the door, and Kashio's goons allow {guard.himher} to pass with a chuckle.
                    !exit
                agent:
                    !right
            ]],
            -- DIALOG_NOT_CONVINCED =
            -- [[
            --     player:
            --         !angry_accuse
            --         $angryHostile
            --         I put down dogs like you for <i>breakfast</i>.
            --     agent:
            --         !fight
            --         $angryCruel
            --         You sure you wanna go with <i>that</i> for your last words, hunter? Embarrassing.
            -- ]],

            -- DIALOG_CONVINCED =
            -- [[
            --     player:
            --         !crossed
            --         $angrySeething
            --         Kashio thinks you're disposable. It'd be a shame if you agreed with her.
            --     agent:
            --         !angry_shrug
            --         $angryPatienceLost
            --         I ain't nobody's lapdog. And it just so happens I have places to be. But if {kashio} asks, I beat you and left you for dead.
            --     player:
            --         !cruel
            --         $angryCruel
            --         Your secret's safe with me.
            --     agent:
            --         !exit
            --     * The goon leaves you with nothing but a trashed bar and a chip on your shoulder.
            -- ]],
            OPT_DEFEND = "Defend yourself!",
            OPT_FIGHT = "Attack Kashio's goon",
            DIALOG_FIGHT = [[
                player:
                    !angry
                    $angryHostile
                    Nice to see Kashio cares about as much for her allies as she does for her enemies. 
                    You lot have <i>no idea</i> what she's signed you up for.
                    !fight
                agent:
                    !right
                    !fight
            ]],
            DIALOG_POST_FIGHT_DEAD = [[

            ]],
            NEGOTIATION_REASON = "Convince {1#agent} to help defend the bar",
            DIALOG_POST_FIGHT_ALIVE = [[

            ]],
            SIT_MOD = "Afraid of Kashio",
        }
        :Fn(function(cxt) 
            local guard
            
            cxt.quest.param.done_asking_about_kashio = true
            cxt.location:SetPlax("INT_GrogNDog_Damaged_1")
            
            TheGame:SubmitLog("DAY 1 BOSS")
            
            for _, agent in cxt.location:Agents() do
                if agent:GetRoleAtLocation() == CHARACTER_ROLES.GUARD then
                    guard = agent
                    cxt.quest:AssignCastMember("guard", guard)
                    break
                end
            end

            local boss = cxt.quest:CreateSkinnedAgent( cxt.quest.param.boss_id )

            cxt.quest:AssignCastMember("kashio_goon", boss)
            --
            local kashio = cxt.quest:GetCastMember("kashio")
            kashio:MoveToLocation(cxt.location)
            
            
            cxt:ProgressDialog("kashio_meeting", "DIALOG_INTRO")
            

            kashio:MoveToLimbo()
            cxt.enc:SetPrimaryCast(boss)

            cxt:Dialog("DIALOG_GOON_THREATEN")

            
            if not cxt.enc.scratch.tried_negotiate and guard then
                cxt:Opt("OPT_REQUEST_BACKUP", guard)
                    :Fn(function() cxt.enc:SetPrimaryCast(guard) end)
                    :Dialog("DIALOG_REQUEST_BACKUP")
                    :Negotiation{
                        difficulty = cxt.quest:GetRank(),
                        target_agent = guard,
                        helpers = { "fssh" },
                        situation_modifiers =
                        {
                            { value = 10, text = cxt:GetLocString("SIT_MOD") }
                        },
                        reward_difficulty = QUEST_MAX_RANK,
                        flags = NEGOTIATION_FLAGS.NO_BYSTANDERS,
                        reason = "NEGOTIATION_REASON",

                        on_success = function(cxt) 
                                cxt.enc:SetPrimaryCast(boss)
                                cxt:Dialog("DIALOG_CONVINCED")
                                cxt:Opt("OPT_DEFEND")
                                    :Battle{
                                        allies = {"guard"},
                                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.BOSS_FIGHT,
                                        noncombatants = {cxt.quest:GetCastMember("fssh")},
                                        enemies = {boss},
                                        on_win = function(cxt) 
                                            boss:MoveToLimbo()
                                            cxt:GoTo("STATE_REWARD")
                                        end,                
                                    }
                            end,
                        on_fail = function(cxt) 
                                cxt.enc:SetPrimaryCast(boss)
                                cxt:Dialog("DIALOG_NOT_CONVINCED")
                                cxt:Opt("OPT_DEFEND")
                                    :Battle{
                                        --advantage = TEAM.RED,
                                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.BOSS_FIGHT,
                                        noncombatants = {cxt.quest:GetCastMember("fssh")},
                                        enemies = {boss},
                                        on_win = function(cxt) 
                                            boss:MoveToLimbo()
                                            cxt:GoTo("STATE_REWARD")
                                        end,                
                                    }
                            end,
                    }
            end

            cxt:Opt("OPT_FIGHT")
                :Fn(function() cxt.enc:SetPrimaryCast(boss) end)
                :Dialog("DIALOG_FIGHT")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_BYSTANDERS | BATTLE_FLAGS.BOSS_FIGHT,
                    reward_difficulty = QUEST_MAX_RANK,
                    noncombatants = {cxt.quest:GetCastMember("fssh"), cxt.quest:GetCastMember("kashio")},
                    enemies = {boss},
                    on_win = function(cxt) 
                        boss:MoveToLimbo()
                        cxt:GoTo("STATE_REWARD")
                    end,                
                }
        end)

        :State("STATE_REWARD")
            :Loc{
                DIALOG_WON_FIGHT = [[
                    agent:
                        !exit
                    * Fssh carefully navigates the mess of her bar, favoring her side. 
                    * She doesn't look too badly injured, but she's not as young as she used to be.
                    fssh:
                        !right
                        !injured
                        $miscRelieved
                        Well, that shucked a rotten oyster. But we've been through worse, and we're both still standing.
                    player:
                        !angry
                        ...
                        I can't believe she came for you like that. <i>You</i>. 
                        She doesn't care about anyone, does she? She's just a monster all the way down.
                    fssh:
                        !neutral
                        $neutralResigned
                        What's done is done. The fire's only good if it stokes the engine, kid.
                    *** You and Fssh have both survived the fight.
                ]],


                DIALOG_DONE = [[
                    player:
                        !disappoint
                        $miscDepressed
                        I should've gone for Kashio as soon as I arrived. Now you're dragged into it, too. 
                    fssh:
                        $miscDepressed
                        None of that. You've got a bit of grit in your eye. You'll wink it out once you get some sleep.
                        I got a room set up for you in the back. 
                    * With Kashio gone, the few patrons that remain return to their drinks—them that weren't spilled, anyway.
                    *** Fssh suggests you talk to her when you're ready to sleep the day off.
                ]],
                OPT_LOOK_AT_CARDS = "Look at Fssh's Cards",
                CARD_PICK_TITLE = "Fssh's Cards",
                CARD_PICK_SUBTITLE = "You must pick 1",
            }
            :Fn(function(cxt) 
                    ConvoUtil.GiveBossRewards(cxt, GRAFT_TYPE.NEGOTIATION)
                    cxt:Dialog("DIALOG_WON_FIGHT")
                    cxt:Dialog("DIALOG_DONE")

                    StateGraphUtil.AddEndOption(cxt)

                    cxt.quest:Complete("meet_kashio")
                    cxt.quest:Activate("sleep_day_one")                    
                end)

------------------------------------------------------------------------------------------------------ 

QDEF:AddAttract("sleep_day_one", "fssh",
    [[
        * Fssh didn't get to the top of her field by indulging her wounds. She barely flinches as she spitshines an old mug.
        agent:
            !spit
            $miscRelieved
            I bet your blood's boilin', kid, but we survived the day.
            !overthere
            Cool your feet, get some rest, and we'll come up with a plan in the morning. 
    ]])


QDEF:AddConvo("sleep_day_one", "fssh")
    :Loc{
        OPT_GET_A_ROOM = "Rest in the back",
        REQ_NOT_ALONE = "You need to get rid of your company first",
        DIALOG_GET_ROOM = [[
            player:
                !sigh
                $neutralResigned
                A good night's rest can't hurt. Unless I wake up dead, that is.
            agent:
                !thumb
                $neutralThoughtful
                Kashio won't come back on my watch. The back's barricaded. And the front has me.
                $miscNostalgic
                G'night, kid.
                !exit
            player:
                !exit
            * {fssh} leads you through a curtain behind the bar, nodding towards a cozy room at the back. 
        ]],
        DIALOG_WAKE = [[
            * You slept through the night with one hand gripping the knife under your pillow. But then, you've slept that way every night since you were twelve.
        ]],
        OPT_END_DEMO = "Finish Demo",
    }

    :Hub(function(cxt) 

            if cxt.quest:IsActive("sleep_day_one") then
                cxt:Opt("OPT_GET_A_ROOM")
                    :SetQuestMark()
                    :PreIcon(global_images.sleep)
                    :Fn(function(cxt) 
                        local grog = cxt.location
                        cxt:Dialog("DIALOG_GET_ROOM")
                        
                        cxt:AwardUnlock("ROOK_PLAYABLE", "ROOK", LOC"UI.CHARACTER_UNLOCKED.ROOK_TITLE", LOC"UI.CHARACTER_UNLOCKED.ROOK_BODY")

                        cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GROG_N_DOG.inn_room") )
                        grog:SetPlax()
                        cxt:FadeIn()
                        ConvoUtil.DoSleep(cxt, "DIALOG_WAKE")
                        
                        cxt.quest:Complete()

                       
                        if TheGame:IsDemo() then
                            TheGame:DeleteSave(TheGame:GetGameState().save_file_name, function() print ("DELETED", TheGame:GetGameState().save_file_name)  end)
                            cxt:Opt("OPT_END_DEMO")
                                :CheckDemo()
                        else
                            cxt:Opt("OPT_LEAVE")
                                :MakeUnder()
                                :Fn(function() 
                                    cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GROG_N_DOG") )
                                    cxt:End()
                                end)
                        end

                        --StateGraphUtil.AddLeaveLocation(cxt)

                    end)

            end
        end)    


QDEF:AddConvo("sleep_day_one")
    :Priority(CONVO_PRIORITY_LOWEST)
    :AttractState("STATE_ATTRACT", function(cxt) return cxt.location == TheGame:GetGameState():GetLocation("GROG_N_DOG") and cxt:GetAgent() and cxt:GetAgent() ~= cxt.quest:GetCastMember("fssh") and ( cxt.quest.param.talked_about_attack  == nil or cxt.quest.param.talked_about_attack[cxt:GetAgent()] == nil)end)
        :Quips{
            {
                tags = "talk_about_kashio",
                "That was intense!",
                "Kashio is so scary!",
                "Kashio is going to come after you now!",
            },
        }

        :Fn(function(cxt) 
            cxt.quest.param.talked_about_attack = cxt.quest.param.talked_about_attack or {}
            cxt.quest.param.talked_about_attack[cxt:GetAgent()] = true
            cxt:Quip(cxt:GetAgent(), "talk_about_kashio")
        end)


QDEF:AddConvo("talk_to_fssh")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == TheGame:GetGameState():GetLocation("GROG_N_DOG") and not cxt.quest.param.fssh_intro_market_job end)
        :Loc{
            DIALOG_MERCHANT_INTROS = [[
                * {fssh} greets you enthusiastically.
                fssh:
                    !right
                    !greeting
                    {player}! I've lined up a great opportunity for you. 
                player:
                    !left
                fssh:
                    !overthere
                    Those two over there are <i>night merchants</i>. They sell rare goods you can't get nowhere else. And it just so happens they need <i>your</i> help.
                    You can only be in one place at a time—but either will get you a leg up you sorely need, kid. 
                    !point
                    Go tell 'em I sent you, and ingratiate yourself.
                    Just be warned that they're both persnickety so-and-so's. You're probably going to cheese off the one you don't pick.
                player:
                    !agree
                    Huh. Sounds neat. Thanks, Fssh.

                *** You can only help one of the night merchants. They will remember your decision.
            ]],
        }
    :Fn(function(cxt) 

            local options = {"SAL_STORY_MERCHANT_COMBAT", "SAL_STORY_MERCHANT_NEGOTIATION","SAL_STORY_MERCHANT_GRAFTS", "SAL_STORY_MERCHANT_PETS"}
            while #options > 2 do
                table.remove(options, math.random(#options))
            end
            cxt.quest.param.market_quests = {}

            for k,v in ipairs(options) do
                local q = QuestUtil.SpawnQuest(v)
                if q then
                    table.insert( cxt.quest.param.market_quests, q )
                end
            end

            cxt.quest.param.market_quests[1]:AssignCastMember( "other_merchant", cxt.quest.param.market_quests[2]:GetCastMember("night_merchant") )
            cxt.quest.param.market_quests[2]:AssignCastMember( "other_merchant", cxt.quest.param.market_quests[1]:GetCastMember("night_merchant") )

            cxt.quest:Complete("talk_to_fssh")

            cxt.quest.param.fssh_intro_market_job = true

            cxt:Dialog("DIALOG_MERCHANT_INTROS")
            
            StateGraphUtil.AddEndOption(cxt) 
        end)
