local NUMBER_OF_JOBS_TO_SPAWN = 3
local NUMBER_OF_JOBS_TO_DO = 1


local function ActivateNewJob(cxt)
    cxt.quest:Activate("do_a_handler_job")
    local new_job = cxt.quest.param.main_quest:GetQuestDef().TrySpawnHandlerQuest(cxt.quest.param.main_quest)
    assert(new_job)
    cxt.quest.param.handler_job = new_job
    
    StateGraphUtil.PresentQuestOffer(cxt, new_job, nil, 
        function() 
            cxt:PlayQuestConvo(new_job, QUEST_CONVO_HOOK.ACCEPTED)
        end, nil, true)
end

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

local QDEF = QuestDef.Define
{
    title = "Picking Sides",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_picking_sides.tex"),

    qtype = QTYPE.STORY,
    desc = [[ Pick a side, and start working for one of the factions in Murder Bay.]],

    
    is_hiring_available = function(quest)
        return false--quest:IsActive("pick_a_side_job")
    end,

    
    on_start = function(quest) 
        quest:Activate("start")
    end,

    --we changed the structure of some days
    on_post_load = function(quest)
        if quest:IsActive() and TheGame:GetGameState():GetCreationBuild() < 444763 then
            if Iter.values(quest.objectives):all(function(q) return not quest:IsActive(q.id) end ) and quest:IsInactive("go_home") then
                quest:Activate("go_home")
            end
        end
    end,

    events = 
    {
        quests_changed = function(quest, event_quest) 
            if quest:IsActive("do_a_handler_job") and event_quest == quest.param.handler_job then
                if event_quest:IsComplete() or event_quest:IsFailed() then
                    quest:Complete("do_a_handler_job")
                end
            end
        end,
    },
}

:AddOpinionEvents{
    spared_assassin = 
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Spared their life",
    }
}

:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}


:AddCastByAlias{
    cast_id = "admiralty_contact",
    alias = "MURDER_BAY_ADMIRALTY_CONTACT",
}

:AddCastByAlias{
    cast_id = "bandit_contact",
    alias = "MURDER_BAY_BANDIT_CONTACT",
}


:AddCast{
    cast_id = "handler",
    when = QWHEN.MANUAL,
    no_validation = true,
}


:AddObjective{
    id = "pick_admiralty",
    title = "Meet with {admiralty_contact}",
    desc = "{fssh} suggests that {admiralty_contact} can get you in the front door of the auction, but you're going to have to stooge for Admiralty to get {admiralty_contact.hisher} help.\n\nThe Admiralty is what passes for government around here, but they comport themselves more like an occupying army.\n\nDon't expect to make a lot of friends amongst the people if you pick this side.",
    icon = engine.asset.Texture("icons/quests/sal_story_chooseadmiralty.tex"),

    mark = {"admiralty_contact"},
    
    on_complete = function(quest)
        TheGame:GetGameProfile():SetCustomAgentUnlock( quest:GetCastMember("admiralty_contact"):GetUniqueID(), "picked_admiralty" )
        quest:Cancel("pick_spree")
        quest.param.main_quest:GetQuestDef().SelectAdmiraltyHandler(quest.param.main_quest)
        TheGame:GetGameState():AddPendingEvent("faction_hassle")
    end,
}

:AddObjective{
    id = "pick_spree",
    title = "Meet with {bandit_contact}",
    icon = engine.asset.Texture("icons/quests/sal_story_choosespree.tex"),
    desc = "{fssh} says that {bandit_contact} can sneak you into the auction if you do some work for the Spree.\n\nThe Spree is a loosely organized band of outlaws and raiders that survive on the outskirts of Admiralty territory. They rob from pretty much everyone, and give mostly to themselves.\n\nBe prepared to run afoul of the law if you choose this path.",
    mark = {"bandit_contact"},

    on_complete = function(quest)
        TheGame:GetGameProfile():SetCustomAgentUnlock( quest:GetCastMember("bandit_contact"):GetUniqueID(), "picked_spree" )
        quest:Cancel("pick_admiralty")
        quest.param.main_quest:GetQuestDef().SelectBanditHandler(quest.param.main_quest)
        TheGame:GetGameState():AddPendingEvent("faction_hassle")
    end,
}


:AddObjective{
    id = "do_a_handler_job",
    title = "Do a job for {handler}",
    hide_in_overlay = true,
    --desc = "You've completed Fssh's jobs. Return to the Grog to see if she's found any leads for you.",

    on_activate = function(quest)
        TheGame:GetGameState():AddPendingEvent()
    end,
    
    on_complete = function(quest)
        quest:Activate("go_home")
        QuestUtil.SpawnOpportunities()
    end,
}

:AddObjective{
    id = "go_home",
    title = "Return to the Grog to Sleep",
    --desc = "You've completed Fssh's jobs. Return to the Grog to see if she's found any leads for you.",
    mark = {"fssh"},
    on_activate = function(quest) 
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true
        UIHelpers.PassTime(DAY_PHASE.NIGHT)
        TheGame:GetGameState():AddPendingEvent()
        QuestUtil.SpawnQuest("ROAD_SIDE_MERCHANTS")
        quest.param.boss_id = TheGame:GetGameProfile():GetNoStreakRandom("SAL_DAY_2_BOSS_PICK", {"JAKES_ASSASSIN", "JAKES_ASSASSIN2"}, 2)
    end,
    
    on_complete =function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
        quest:Activate("do_sleep")
    end
}
:AddObjective{
    id = "do_sleep",
    title = "Get Some Shut-eye",
    --desc = "You've completed Fssh's jobs. Return to the Grog to see if she's found any leads for you.",
}


:AddObjective{
    id = "start",
    on_activate = function(quest)
        quest.param.merc_quest = QuestUtil.SpawnQuest("SAL_STORY_MERCS")
    end,
}

--QuestUtil.AddSideJobChain(QDEF, NUMBER_OF_JOBS_TO_SPAWN, NUMBER_OF_JOBS_TO_DO, "go_home")

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


QDEF:AddConvo("go_home", "handler")
    :Loc{
        DIALOG_INTRO = [[
            {admiralty?
            agent:
                I'm done with you for the day.
                But this isn't an exclusive arrangement. If you have other work that needs doing, go for it.
            }
           {bandits?
            agent:
                We're done here. 
                You got other things to do, that's fine.
                But don't take risks. You're no good to me dead.
            }
        ]],
        DIALOG_ASKED_ALL_QUESTIONS = [[
            {bandits?
                agent:
                    Enough questions.
                player:
                    !agree
                    Sure. You're the boss.
            }
            {admiralty?
                agent:
                    Someone in my position might be suspicious of a hunter asking so many personal questions.
                player:
                    Hey, no offense meant. 
                agent:
                    None taken. Not a lot of friends on the derrick, right? 
                    You wanna get friendly on the job, that's your call.
                    But just keep in mind I prefer you ask <i>questions</i> rather than <i>favors</i>.
                    Enjoy the rest of your day, hunter. I have a feeling you'll make the most of it. 
            }
        ]],
    }
    :Questions("SPREE", {
        "Ask about Murder Bay",
        [[
            player:
                !interest
                So are you from here?
            agent:
                !agree
                $neutralWhatever
                Yeah. There's murder in my blood, and my blood in the bay. 
            player:
                Grim, but evocative. 
                Must be hard to defend your territory. There's a lot of competition.
            agent:
                $neutralWhatever
                Sometimes folks get outta line. 
                $neutralDirect
                But not for long.
            player:
                !dubious
                $neutralDubious
                Ah... hah.
        ]],
        "Ask about Nadan's thugs",
        [[
            player:
                How do your people feel about you working with a hunter?
            agent:
                !shrug
                $neutralWhatever
                Didn't ask.
                If they got feelings, that's on them.
            player:
                $neutralDubious
                You didn't talk about it with them?
            agent:
                $neutralDirect
                I don't talk.
            player:
                Yeah, so I see.
        ]],
        "Ask about Oolo",
        [[
            player:
                !thought
                $neutralThoughtful
                You know, there was someone else who said they could get me into the auction.
                In the Admiralty.
            agent:
                !angry
                ...
            player:
                I think she wanted to get Kashio as much as you do. 
                !shrug
                Maybe I should reach out? I mean, if you're willing to work with a <i>hunter</i>, maybe...
            agent:
                $angryPatienceLost
                No.
                I don't work with switches.
                $angrySeething
                Least of all <i>that</i> switch.
            player:
                You mean Oolo? That was her name, I think.
            agent:
                $neutralDirect
                It is. Don't say it again.
            player:
                !dubious
                $neutralDubious
                Okay, then.
        ]],
    })
    :Questions("ADMIRALTY", {
        "Ask about Murder Bay",
        [[
            player:
                !interest
                Mind chatting for a bit? It's been a while since I had colleagues. 
                Were you born in Murder Bay?
            agent:
                !agree
                I was, actually, though I'm supposed to pretend otherwise.
                $neutralResigned
                Two of my siblings were born in Deltree, and my parents would've preferred that for all of us.
            player:
                That sounds loaded.
            agent:
                !shrug
                $neutralWhatever
                Enh, my parents are minor nobility. They offended the delicate sensibilities of some mainland oozer or another. 
                Suddenly they find themselves in soft, <i>voluntary</i> exile to Havaria. 
                !overthere
                Now the entire brood is supposed to redeem them so we can all go home.
            player:
                Seems like a lot of pressure.
            agent:
                !crossed
                $neutralJoke
                Not if you don't keep it under a lid. 
                I have my outlets. I just remind myself I'm here because I want to be.
        ]],
        "Ask about Oolo's underlings",
        [[
            player:
                How do the other officers feel about you working with a hunter?
            agent: 
                !chuckle
                $neutralJoke
                Oh, they had a few choice words.
                Some because of their pride, others because maybe they don't want you figuring out what they're up to.
            player:
                We had dirty cops on the derrick, too.
                !handwring 
                $miscMocking
                Getting the upper hand on them was a specialty of mine.
            agent:
                Is that right? Guess you needed to be wily to work your debt off as fast as you did.
                !interest
                Well, if you see a chance to put that skill to use, I'm all ears.
            player:
                I'll keep that in mind.
        ]],
        "Ask about Nadan",
        [[
            player:
                !thought
                $neutralThoughtful
                If you're willing to cut corners, maybe we should recruit some help?
                There was another guy who wanted to help me get into the auction. 
                $neutralThoughtful
                What was his name... Nadan?
            agent:
                !chuckle
                Hah! Nadan! 
            player:
                You know him?
            agent:
                !humour
                Oh, sure. He's only tried to kill me about five times.
                It's a good thing you came to me. Because I'm gonna take him down.
                !point
                And if you were working with him, I'd take you down along with him.
                Just something to keep in mind.
        ]],
    })
    :Hub(function(cxt)

        if not cxt.quest.param.handler_spoke_about_sides then
            cxt.quest.param.handler_spoke_about_sides = true
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                local inserted = cxt:InsertQuestions(cxt:GetAgent():GetFactionID() == "BANDITS" and "SPREE" or "ADMIRALTY")
                if inserted == 0 then
                    cxt:Dialog("DIALOG_ASKED_ALL_QUESTIONS")
                    cxt:End()
                else
                    StateGraphUtil.AddEndOption(cxt)
                end

            end)
        end
end)





QDEF:AddConvo("start", "fssh")
    :ConfrontState("START")
        :Loc{
            DIALOG_INTRO = [[
                * It's early morning, but the Grog n' Dog's patrons are already hard at work on their drinks.
                player:
                    !left
                    !forgetful
                    So I know you thought I should hold off on that bounty, but after last night... 
                    $miscMischievous
                    If Kashio wants me to be her worst nightmare, I'd like to oblige.
                fssh:
                    !chuckle
                    $happyJoke
                    And how you gonna do that? She knows you're here, and she's got money and goons to keep her safe. What do <i>you</i> got?
                player:
                    !thought
                    $miscAlluring
                    Hm. You?
                fssh:
                    !agree
                    $neutralDirect
                    You're damn right. Lucky for you, I have a couple of contacts who want Kashio dead almost as much as you do, if you can convince them to work with you.
                player:
                    $neutralDubious
                    Will they need convincing?
                fssh:
                    'Fraid so. The Admiralty doesn't trust hunters as a rule, but talk to my contact, and see if you can convince her otherwise.  
                    !permit
                    $neutralTakenAback
                    But taking out Kashio would make a space the Spree'd be more than happy to fill. I know somebody who'll talk to you, especially if you can bring him Kashio's head on a platter.
                *** {fssh} knows some people who can help you with your Kashio problem. You're going to have to choose one to work with if you want access to the auction.
            ]],

            DIALOG_FAST_INTRO = [[
                fssh:
                    !crossed
                    $neutralDirect
                    You look like you want revenge. And I know just the folks who can help you get it, if you're ready.
                    There's people in both the Spree and the Admiralty who want to see Kashio taken down. Dangerous friends—so pick wisely. 
            ]],

            OPT_REMIND_ME = "Let's chat",
            DIALOG_REMIND_ME = [[
                player:
                    !disappoint
                    $miscDepressed
                    I didn't think I could get angry like that anymore. You spend years on the derrick, you learn to tamp it down, you know?
                Fssh:
                    !interest
                    $neutralThoughtful
                    Kashio has that effect on people. And you have more reason than most. 'Cept for all the other folks she sent to the derrick, maybe.
                player:
                    !shrug
                    $neutralDefault
                    They're not here, though. I am.
            ]],

            DIALOG_GIVE = [[
                fssh: 
                    $neutralThanks
                    Before you go, I've got something for you.
                    !give
            ]],

            DIALOG_POST = [[
                player:
                    !take
                    $neutralTakenAback
                    Thanks. What is it?
                fssh:
                    $miscAlluring
                    I don't know what kind of work they'll have for you, but I have a feeling that will be useful.
                player:
                    !happy
                    $happyThanks
                    I like any present that keeps me alive, or helps me make someone else dead. 
            ]],
            OPT_HEAD_OUT = "Head out",
        }
        :SetLooping()
        :Fn(function(cxt)   
            if cxt:FirstLoop() then
                cxt.quest:Complete("start")
                
                if TheGame:GetGameProfile():HasUnlock( "HAS_HEARD_FSSH_INTRODUCTION_2" ) then
                    cxt:Dialog("DIALOG_FAST_INTRO")
                else
                    cxt:Dialog("DIALOG_INTRO")
                end
                cxt.quest:Activate("pick_spree")
                cxt.quest:Activate("pick_admiralty")
            end

            
            if TheGame:GetGameProfile():HasUnlock( "HAS_HEARD_FSSH_INTRODUCTION_2" ) and not cxt.enc.scratch.has_done_intro then
                cxt:Opt("OPT_REMIND_ME")
                    :Dialog("DIALOG_REMIND_ME")
                    :Fn(function() 
                        cxt.enc.scratch.has_done_intro = true
                        cxt:Dialog("DIALOG_INTRO")
                    end)
            end

            cxt:Opt("OPT_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")

            cxt:Opt("OPT_HEAD_OUT")
                :PreIcon(global_images.close)
                :Fn(function(cxt) 
                        TheGame:GetGameProfile():AcquireUnlock("HAS_HEARD_FSSH_INTRODUCTION_2")
                    end)
                :End()

        end)

    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask about the Admiralty",
        [[
            player:
                !dubious
                $neutralDubious
                You think I should play nice with the law, huh?
            agent:
                $neutralWhatever
                Why not? Lotsa folks think bounty hunters are on the Admiralty's side.
            player:
                !sigh
                $neutralDubious
                Lotsa folks don't, too. The Admiralty, for example.
            agent:
                {admiralty_contact} will at least give you the time. The Admiralty acts like the tough guys round here but they're as broke and corrupt as the rest. 
                !shrug
                $neutralWhatever
                You don't like it? There's always the Spree.
        ]],
        "Ask about the Spree",
        [[
            player: 
                !interest
                $neutralTakenAback
                Lots of unclaimed bounties running with the Spree, I take it?
            agent:
                $happyCocky
                They're small fry and you're a barracuda. But everyone knows Kashio's bounty is on the line. You've got your sights set higher.
                !point
                Convince {bandit_contact} of that and {bandit_contact.heshe}'ll give you the leg up you need. Once you've paid your dues, of course.
            player:
                !dubious
                $neutralDubious
                And if I don't?
            agent:
                $happyJoke
                Then maybe {bandit_contact.heshe}'ll wanna kill you. Or maybe {bandit_contact.heshe}'ll just wanna be left alone. 
                !chuckle
                Peoples is peoples. 
            player:
                !agree
                $happyDefault
                That's very true.
        ]],

        "Ask about Fssh",
        [[
            player:
                !bashful
                $neutralThoughtful
                You really okay with this after last night? 
            agent:
                $neutralResigned
                !shrug
                We're in it now, whether we like it or not. Leave my grudges where I like 'em, as long as the hunt's still on.
            player:
                !cruel
                $angryCruel
                Absolutely. There's blood in the water.
            agent:
                !throatcut
                $angryCruel
                Then do it. You won't get no complaints from me. 
        ]],
        "Ask about Kashio",
        [[
            agent:
                !angry_shrug
                $angryDefault
                Figures Kashio'd come after you no matter what you did. 
            player:
                !agree
                Well, it's not like I came back as a lowly lumin sludger. 
                She probably has an alert out for any bounty hunter who comes into port.
                !happy
                $happyDefault
                Plus you were asking around for info, huh? Thought you said I should keep my head down?
            agent:
                $neutralDirect
                Yeah, well. I never forgot what Kashio did, neither.
                Didn't seem right that you'd come home just to let bygones be bygones.
                !handwring
                $miscNostalgic
                And that <i>bounty</i> ...
            player:
                !thought
                $miscNostalgic
                It <i>is</i> a big bounty.
            agent:
                $happyDefault
                The biggest. With that money we could open <i>three</i> bars.
            player:
                !chuckle
                $happyJoke
                You got it, Fssh. Dream big. 
        ]]

    })        


------------------------------------------------------------------------------------------------------ 


QDEF:AddConvo("pick_spree", "bandit_contact")
    :AttractState("STATE_INTRO")
        :Loc{
            OPT_INTRODUCE_YOURSELF = "Introduce yourself",
            DIALOG_INTRO = [[
            * The man meets your gaze with a face like barbed wire.
            player:
                !permit
                $neutralThoughtful
                Nadan, right? I'm Sal. I hear you're a guy worth knowing.  
            agent: 
                !cruel
                $miscAlluring
                And I hear you've got an axe to grind. You wanna bury it in a few skulls?
                !point
                You won't get anywhere near Kashio without my help.
                !crossed
                But my help don't come cheap or easy. 
                You do good work and survive to reap the benefits? Then maybe we're in business.
            *** Nadan has agreed to work with you, though he remains hostile.
            ]],
            OPT_LATER = "Maybe later",
            DIALOG_LATER = [[
                player:
                    $neutralThanks
                    I'm not ready to make a decision quite yet.
            ]],
            OPT_AGREE = "Work for {agent}",
            OPT_PICK_FACTION_CARD = "Pick your Spree card",
            DIALOG_AGREE = [[
                player:
                    $miscAlluring
                    !agree
                    A contract's a contract. I'm happy to be in business with you, Nadan.
                agent:
                    !shrug
                    $neutralWhatever
                    Glad you're convinced so easy. You do good on those favors, then maybe we're as copacetic as you say. And Kashio will be all yours.
            ]],
            DIALOG_TAKE_GRAFT = [[
                agent:
                    $neutralDirect
                    First order of business? Take this.                   
                    !give
                player:
                    !take
            ]],
            DIALOG_TOOK_GRAFT = [[
                player:
                    $neutralDubious
                    What is it?
                agent:
                    !thumb
                    $miscPersuasive
                    This will tell people that you're under my protection.
                player:
                    !happy
                    $happyDefault
                    Nice. I've always wanted to learn a secret handshake. 
                *** Nadan gives you a card identifying you as one of his allies.
            ]],
            OPT_GET_WORK = "Get work",
            DIALOG_GET_WORK = [[
                player:
                    !permit
                    $happyCocky
                    So, put me to work, boss. I'm here to impress, ain't I?
                handler:
                    !agree
                    $neutralDirect
                    You're hungry. That's good. 
            ]],
            CARD_PICK_TITLE = "Spree Cards",
            CARD_PICK_SUBTITLE = "You must pick 1",
        }
    
        :SetLooping()
        :Fn(function(cxt)
            if cxt:FirstLoop() then
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_AGREE")
                :Dialog("DIALOG_AGREE")
                :PreIcon( engine.asset.Texture( "UI/ic_nav_spree.tex" ) )
                :ReqCondition(QuestUtil.CanTakeSideJob(), "REQ_SIDE_JOB_LIMIT")
                :CompleteQuest("pick_spree")
                    :Fn(function(cxt) 
                            cxt:Dialog("DIALOG_TAKE_GRAFT")

                            Util.SetPlayerWorkingWithFaction("BANDITS")
                            cxt:Opt("OPT_PICK_FACTION_CARD")
                                :Fn(function() 
                                    cxt.enc:OfferCards(GetUniqueFactionCards(cxt.player, "BANDITS", 2, 0), cxt:GetLocString("CARD_PICK_TITLE"), cxt:GetLocString("CARD_PICK_SUBTITLE"), true)
                                    cxt:Dialog("DIALOG_TOOK_GRAFT")
                                    
                                    cxt:Opt("OPT_GET_WORK")
                                        :PreIcon(global_images.quest)
                                        :Dialog("DIALOG_GET_WORK")
                                        :Fn(function() 
                                                ActivateNewJob(cxt)
                                            end)
                                        :DoneConvo()
                                end)
                        end)


            cxt:Opt("OPT_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")


            cxt:Opt("OPT_LATER")
                :Dialog("DIALOG_LATER")
                :DoneConvo()

        end)    
     :AskAboutHub("STATE_QUESTIONS", 
        {
            "Ask about Kashio",
            [[
                player: 
                    !placate
                    $neutralDubious
                    Hold up. What exactly is it that you're offering? 
                agent:  
                    !dubious
                    $neutralDirect
                    You want Kashio. I can get you Kashio.
                    !interest
                    In a few days, the Cult will be rolling into Murder Bay for their annual <i>antiquities</i> auction.
                    They sell relics and holy pardons to them that can afford it in exchange for labor contracts and other illicits.
                player:
                    !thought
                    $neutralThoughtful
                    That does sound like Kashio's market. But auctions like that have exclusive guest lists.
                agent:
                    !thumb
                    $miscPersuasive
                    And that's where our business arrangement comes in. I can get you into that auction—but only if you can prove you deserve it.
            ]],

            "Ask about the auction",
            [[
                player:
                    !crossed
                    $neutralDubious
                    You sure Kashio will be there herself? She might just send her goons to do her shopping for her.
                agent:
                    !shrug
                    It ain't just about the goods this year. Big decisions are getting made. The Admiralty wants to annex, and they need the Cult on side to do it. 
                    !spit
                    "Citizenship" would make most sane Havarians choke. But it goes down smoother with some Heshian <i>wisdom</i>. 
                    !angry_shrug
                    $miscAlluring
                    And guess whose life gets a whole lot harder if labor contracts get regulated?
                player:
                    !thought
                    $miscMischievous
                    The biggest debt broker in the griftlands, I suppose.
                agent:
                    The annex is a wrench in her whole operation. She'll be there to make sure it doesn't happen. 
                    So take that wrench and yank it. 
            ]],


            "Ask about Nadan",
            [[
                player:
                    !dubious
                    $neutralDubious
                    I get you're the one with demands, pal, but I like to know who I'm working with.
                agent:
                    $neutralWhatever
                    Yeah? Can't say I blame you, with that fish-outta-water stink.
                    !throatcut
                    $neutralDirect
                    If you were from here you'd have heard of me already. I'm the Scourge of Murder Bay.
                player:
                    !interest
                    $neutralTakenAback
                    Folks actually call you that?
                agent:
                    !shrug
                    $neutralWhatever
                    I dunno, I never asked. I'm not much of a conversationalist.
                player:
                    $neutralDubious
                    Huh.
            ]],

            "Ask about the work",
            [[
                player:
                    !interest
                    $neutralThoughtful
                    So this work you're having me do...
                agent:
                    $neutralWhatever
                    Favors.
                player:
                    !crossed
                    $neutralDirect
                    Yeah, right. It's illegal, I assume?
                agent:
                    !dubious
                    $neutralDubious
                    That's up to you, ain't it? I only care about results. 
                    !shrug
                    $neutralWhatever
                    If you find a way of getting me what I want while playing nice with the law... That's your time wasted, not mine.
            ]],

            "Ask about the Admiralty",
            [[
                player:
                    !agree
                    $neutralDubious
                    I take it you're not a fan of the annex?
                agent:
                    !dubious
                    $neutralDubious
                    You're an ik-Derrick, ain't ya? What has the Admiralty ever done for you?
                player:
                    !permit
                    $neutralResigned
                    It's the <i>Cult</i> that runs the derricks.
                agent:
                    !angry_shrug
                    $angryPatienceLost
                    And it's the Admiralty that plays their lapdog, turning a blind eye.
                    $angryDefault
                    They don't care for law and order in Havaria—they only care to please their fishlords back across the sea.
                    !spit
                    $angrySeething
                    Even if it means throwing good folk to the Abyss. 
                player:
                    !scared
                    $neutralResigned
                    Heavy.
                agent:
                    $angrySeething
                    I didn't make it that way. I'm just livin' with the consequences.  
                player:
                    !neutral  
                    
            ]],
        })  


QDEF:AddConvo("pick_admiralty", "admiralty_contact")
    :AttractState("STATE_INTRO")
        :Loc{
            DIALOG_INTRO = [[
            * You spot a young kra'deshi sitting at the desk. She doesn't look like your average officer, though.
            * Her uniform has a modified fit, and her boots are too slick for requisition.
            player:
                !permit
                $happyGreeting
                Are you Oolo? I'm Sal. Fssh mentioned we might be able to help each other out.
            agent:
                !crossed
                $miscMischievous
                You're the bounty hunter, then! I've been waiting to meet someone with just your particular set of skills. 
                !point
                You want Kashio dead, and it just so happens I do, too. Do me a solid, and I'll get you where you need to be to make it happen.
                !happy
                $miscAlluring
                Play by my rules, and the killing blow will be yours, I promise. What do you say, hunter?
            *** Oolo has agreed to work with you, though her motives are suspicious.
            ]],
            OPT_LATER = "Maybe later",
            DIALOG_LATER = [[
                player:
                    !placate
                    $neutralDefault
                    I'm not ready to make a decision quite yet.
                agent:
                    !dubious
                    $neutralDubious
                    Well, sure, it's a big decision for you. But neither of us are getting any younger, and Kashio's only getting richer.
            ]],

            OPT_AGREE = "Work for {agent}",
            DIALOG_AGREE = [[
                player:
                    !point
                    $neutralDubious
                    I'm in. But if I end up taking a fall so you can get a promotion or whatever—
                agent:
                    !handwave
                    $miscPersuasive
                    Please. If anything goes wrong it'll be your own fault, I promise. 
                player:
                    !shrug
                    $miscMocking
                    Oh good. I was worried you wouldn't have anything reassuring to say.
            ]],
            DIALOG_TAKE_GRAFT = [[
                agent:
                    $neutralDefault
                    First up, you'll need this. Take it.
                    !give
                player:
                    !take
                    $neutralDubious
                    What is it?
            ]],
            DIALOG_TOOK_GRAFT = [[
                agent:
                    !agree
                    $miscAlluring
                    Folks here take any edge they can get. If you can't compete, you're surf slime. Consider this... your deputy's badge.
                    !wink
                    $miscMischievous
                    A gift for my favorite investment.
                player:
                    !dubious
                    $happyJoke
                    That sounds extremely suspicious.
                agent:
                    !thumb
                    $neutralDirect
                    Impossible. I'm wearing a badge. 
                *** {handler} gives you a card identifying you as one of her allies.
            ]],
            OPT_GET_WORK = "Get work",
            DIALOG_GET_WORK = [[
                player:
                    !handwring
                    $neutralResigned
                    All right, so when can I start? 
                handler:
                    !thought
                    $neutralThoughtful
                    Hmm. What would be a good use of your time...
            ]],
            DIALOG_GOT_WORK = [[
                handler:
                    !overthere
                    $miscPersuasive
                    Got it? Now scoot. I have schemes to incubate, plots to hatch.
            ]],
            CARD_PICK_TITLE = "Admiralty Cards",
            CARD_PICK_SUBTITLE = "You must pick 1",
            OPT_PICK_FACTION_CARD = "Pick an Admiralty card",

        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_AGREE")
                :Dialog("DIALOG_AGREE")
                :PreIcon( engine.asset.Texture( "UI/ic_nav_admiralty.tex" ) )
                :ReqCondition(QuestUtil.CanTakeSideJob(), "REQ_SIDE_JOB_LIMIT")
                :CompleteQuest("pick_admiralty")
                :Fn(function(cxt) 
                        cxt:Dialog("DIALOG_TAKE_GRAFT")
                        cxt:Opt("OPT_PICK_FACTION_CARD")
                        :Fn(function()
                            cxt.enc:OfferCards(GetUniqueFactionCards(cxt.player, "ADMIRALTY", 0, 2), cxt:GetLocString("CARD_PICK_TITLE"), cxt:GetLocString("CARD_PICK_SUBTITLE"), true)
                            Util.SetPlayerWorkingWithFaction("ADMIRALTY")
                            cxt:Dialog("DIALOG_TOOK_GRAFT")
                            cxt:Opt("OPT_GET_WORK")
                                :PreIcon(global_images.quest)
                                :Dialog("DIALOG_GET_WORK")
                                :Fn(function() 
                                        ActivateNewJob(cxt)
                                    end)
                                :Dialog("DIALOG_GOT_WORK")
                                :DoneConvo()
                        end)
                    end)
            

            cxt:Opt("OPT_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")

            cxt:Opt("OPT_LATER")
                :Dialog("DIALOG_LATER")
                :DoneConvo()

        end)

     :AskAboutHub("STATE_QUESTIONS", 
        {

            "Ask about Kashio",
            [[
                player:
                    !dubious
                    $neutralDubious
                    If the Admiralty's so tough, why don't you have Kashio in custody already?
                agent:
                    $neutralWhatever
                    Kashio pays well for allies in the upper ranks. We've got implicit orders to leave her alone. 
                    $miscEmbarrassed
                    I break those orders, and I may as well be breaking my own neck. 
                player:
                    $miscMocking
                    So... you're no help at all, then?
                agent:
                    !crossed
                    $neutralDirect
                    In a few days, the Cult will be hosting an auction in Murder Bay. They send their starchiest collars to trade relics for contracts.
                player:
                    !thought
                    $neutralThoughtful
                    And Kashio's got the most labor contracts in the Bay.
                agent:
                    $miscAlluring
                    Of course, a hunter like you has no hope of an invite. But it so happens I pick the security detail. 
                    !thumb
                    $neutralDirect
                    You want your opportunity, hunter? Stick with me, and I promise to get you into that auction. 
            ]],

            "Ask about the auction",
            [[
                player:
                    !crossed
                    $neutralDubious
                    If Kashio's so important, who's to say she'll show up to the auction herself?
                agent:
                    !handwring
                    $neutralDubious
                    Because she has competition. This year, the Admiralty wants to negotiate an annex: official citizenship for everyone.
                    !agree
                    $neutralThoughtful
                    The Cult wants to say no, of course, but with mainland support, a deal <i>will</i> get made all the same.
                    !cruel
                    $neutralWhatever
                    Which means more laws, more rules, and no more legal gray area for Kashio to wash her feet in. 
                player:
                    !thought
                    $miscMischievous
                    So she'll be there to stop it. 
                agent:
                    !wink
                    $miscMischievous
                    With blades drawn if she must. And no one who'll want to be seen protecting her.  
            ]],


            "Ask about Oolo",
            [[
                player:
                    !crossed
                    $neutralDubious
                    No offense, but I got no love for the Admiralty. Why should I work with you?
                agent:
                    $neutralTakenAback
                    !overthere
                    Because I'm <i>not</i> the Admiralty. I'm me. 
                    The upper ranks are full of inflated egos and blue-blooded ingrates who take their privilege as a matter of course.
                    !handwave
                    $angryAside
                    If they aren't in Kashio's pocket, they're in the Cult's, or some other rich scumbag's. 
                player:
                    !happy
                    $neutralJoke
                    Oh, so you're the <i>good guy</i>? Righting wrongs in a corrupt world?
                agent:
                    !chuckle
                    $miscPersuasive
                    Hesh, no. But I don't like Kashio. And I won't let the brass tell me to look the other way.
                    !offer 
                    <i>And</i> I'm willing to work with an ik-Derrick. That should be all you need to know.

            ]],

            "Ask about the work",
            [[
                player:
                    !interest
                    $happyCocky
                    So what am I supposed to do for you, officer?
                agent:
                    !sigh
                    $miscDepressed
                    The brass have too many corrupt ties, and it keeps me from doing my job. 
                    I could stop half the crime in Murder Bay if the crooks would stop paying off my superiors.
                    !point
                    $miscRelieved
                    But you aren't on Admiralty payroll. And it would wipe the smug look off their faces if you went around cleaning up <i>their</i> mess.
                player:
                    !dubious
                    $neutralDubious
                    So I'm... what? An office prank?
                agent:
                    !happy
                    !handwring
                    $happyDefault
                    You're a lesson they need to learn. But I might have been planning on documenting their reaction, sure.
            ]],

            "Ask about the Spree",
            [[
                player:
                    !interest
                    $neutralDirect
                    So I take it you're a fan of the annex?
                agent:
                    !shrug
                    $neutralWhatever
                    Ugh, I dunno. That's politics. 
                    But too many people take their cue from Kashio, and I'm sick of not being able to do my job.
                    !eureka
                    $neutralDirect
                    The Spree make life interesting, but only if I actually get to <i>arrest them</i> from time to time.
                player:
                    !happy
                    $happyJoke
                    Sounds like you should quit the Admiralty and become a hunter.
                agent:
                    !wink
                    $happyJoke
                    On your salary? I'll keep my commission, thanks. Besides, that's what delegation is for. 
            ]],
        })  


QDEF:AddConvo("go_home", "handler")
    :Loc{
        DIALOG_NOW_WHAT_SPREE = [[
            agent:
                $neutralWhatever
                !overthere
                That's all I have for today. I'll give word to Fssh when I need you again.
            player:
                $neutralDirect
                What should I do until then?
            agent:
                $neutralAside
                Keep your blades sharp. The next job I give you will be harder, so you best get ready.
                !crossed
                You'll make better muscle if you got a reputation that precedes you. 
                !spit
                So go work one up, and let folks know you're not one to be messed with.
        ]],
        DIALOG_NOW_WHAT_ADMIRALTY = [[
            agent:
                $neutralWhatever
                Well, the job's done. You can tell Fssh she was right: you'll suit my needs just fine.
            player:
                $neutralDirect
                That's it? You must have more work for me.
            agent:
                !placate
                $miscMischievous
                Points for enthusiasm, but cool your jets, hunter. We don't want to ruin a good thing. 
                !overthere
                Go... be a hunter. Do whatever hunters do. Collateral always makes a job go easier. 
                I'll send word to Fssh when I'm ready to work with you again. 
        ]],
    }
    :Hub(function(cxt)
            
            if not cxt.quest.param.has_talked_post_quest then
                if Util.IsPlayerWorkingWithFaction("BANDITS") then
                    cxt:Dialog("DIALOG_NOW_WHAT_SPREE")
                else
                    cxt:Dialog("DIALOG_NOW_WHAT_ADMIRALTY")
                end
                cxt.quest.param.has_talked_post_quest = true
            end
        end)

QDEF:AddConvo("go_home", "fssh")
    :Loc{
    
        DIALOG_INTRO_SPREE = [[
            agent:
                !crossed
                $miscMocking
                How'd it go? You and Nadan get all riled up by your hatred of the Admiralty?
            player:
                !dubious
                $neutralDubious
                Mostly he just seems like an opportunist to me. 
            agent:
                !chuckle
                $happyJoke
                Yeah? Well then I'm glad you found each other. There's opportunities aplenty for a crook like him and an arm like yours.
                It's gettin' late. Anything you need before you turn in?
        ]],
        DIALOG_INTRO_ADMIRALTY = [[
            agent:
                !crossed
                $miscMocking
                So, are you and Oolo as chummy as a tin of bogworms now?
            player:
                !dubious
                $neutralDubious
                She's clever. But I think she'd sell me up the river the second I weighed down her canoe.
            agent:
                !chuckle
                $happyJoke
                Yeah? Well, then you two have that in common. Just stay on her good side and she'll stay on yours. 
        ]],

        DIALOG_ASSASSIN_WARNING = [[
            agent:
                !cagey
                $scaredDefault
                Say... someone came in looking for you earlier. I sent them packing, of course.
                !interest
                You make any enemies you might be expecting? They didn't look friendly. 
            player:
                !shrug
                $miscMocking
                I'm a hunter, Fssh. I literally make enemies for a living.
            agent:
                !point
                I'm just saying you might want to keep your guard up. <i>Extra</i> up. Tomorrow I'm gonna look into getting some boobytraps for the bar. 
                !disappoint
                $neutralResigned
                Ambush me once, shame on me. Ambush me twice—
            * {fssh} shakes her head. 
            *** {fssh} warned you that someone was looking for you earlier. 
        ]],
        DIALOG_ASSASSIN_FAST_WARNING = [[
            player:
                !tired
                Time to hit the sack.
            agent:
                !cagey
                $scaredDefault
                Say... someone came in looking for you earlier. They didn't look friendly. Maybe sleep with one eye open tonight, huh?
            *** {fssh} warned you that someone was looking for you earlier. 
        ]],
        OPT_LETS_CHAT = "",
        DIALOG_UNIQUE = [[

        ]],
        OPT_GET_A_ROOM = "Rest in the back",
        REQ_NOT_ALONE = "You need to get rid of your company first",
        
        
        OPT_GO_TO_BED = "Head to bed",
        OPT_LATER = "Do something else first",
        DIALOG_LATER = [[
            player:
                Actually, I've got business to attend to first.
        ]],

        DIALOG_GET_ROOM = [[
            player:
                I'll take my chances.    
                !exit
            agent:
                !exit
            * {fssh} steps aside and you head down to your room for the night. 
        ]],

        OPT_ASK_ABOUT_STRANGER = "Ask about the stranger",
        DIALOG_ASK_ABOUT_STRANGER = [[
                player:
                    !interest
                    $neutralThoughtful
                    Asking after me? What'd they want?
                fssh:
                    Said they wanted to hire you, but if you ask me they looked like trouble.
                    I sent 'em packing, said you were out on the road and wouldn't be back for days.
                    !shrug
                    $neutralWhatever
                    Still, I've got my blades under the bar if we need 'em. 
        ]],
        OPT_ASK_ABOUT_FSSH = "Ask about Fssh",
        DIALOG_ASK_ABOUT_FSSH = [[
                player:
                    $happyCocky
                    Wish I'd been here sooner yesterday. Would've liked to have seen you give Kashio a walloping. 
                fssh:
                    !dubious 
                    $neutralDubious
                    You mean get walloped by Kashio, more like.
                    I ain't as young as I used to be, Sal. You been gone a long time.
                    !forgetful
                    $neutralDubious
                    And Kashio... well, she could've laid me out in our youth all the same.
                player:
                    !interest
                    Should I be worried? You were the one who taught me how to fight.
                fssh:
                    What, you rewritin' history? Kashio taught you, too. 
                    You know her moves better than she knows yours. You got this.
                    !thought
                    $neutralThoughtful
                    Huh. But remind me later to teach you a different way to throw a punch. 
                    $neutralResigned
                    I taught you when you were little, of course, but I'm feeling it now. Maybe that technique wasn't so good for longevity.
                player:
                    !happy 
                    $neutralJoke
                    I don't really expect to make it to your age, anyway.
                fssh:
                    !angry_permit
                    $neutralDirect
                    You Heshin' better.
        ]],        
    }
    :Hub(function(cxt) 
        if not cxt.quest.param.intro_done then
            if Util.IsPlayerWorkingWithFaction("BANDITS") then
                cxt:Dialog("DIALOG_INTRO_SPREE")
                --cxt:Dialog("DIALOG_ASSASSIN_WARNING")
            elseif Util.IsPlayerWorkingWithFaction("ADMIRALTY") then
                cxt:Dialog("DIALOG_INTRO_ADMIRALTY")
                --cxt:Dialog("DIALOG_ASSASSIN_WARNING")
            end
            cxt.quest.param.intro_done = true  
        end


        cxt:Opt("OPT_GET_A_ROOM")
            :PreIcon(global_images.sleep)
            :Dialog("DIALOG_ASSASSIN_FAST_WARNING")
            :LoopingFn(function(cxt) 

                cxt:Question("OPT_ASK_ABOUT_STRANGER", "DIALOG_ASK_ABOUT_STRANGER")
                cxt:Question("OPT_ASK_ABOUT_FSSH", "DIALOG_ASK_ABOUT_FSSH")

                cxt:Opt("OPT_GO_TO_BED")
                    :SetQuestMark()
                    :Dialog("DIALOG_GET_ROOM")
                    :Fn(function() DoAutoSave() end)
                    :GoTo("STATE_ASSASSINATION_ATTEMPT")
                
                cxt:Opt("OPT_LATER")
                    :Dialog("DIALOG_LATER")
                    :Pop()
                
            end)
        end)    


    :State("STATE_ASSASSINATION_ATTEMPT")
    :Loc{
        DIALOG_INTRO = [[
            * Something's not right.
            * In the dim light, some of the shadows look darker than usual. 
            player:
                !left
                !angry
                $angryHostile
                Who's there?
                !surprised
        ]],
        DIALOG_INTRO_2 = [[                
            agent:
                !right
            * A figure steps out of the shadows, heavily armed. If it thinks like an assassin and slinks like an assassin...
            agent:
                $miscAlluring
                This woulda been easier if you'd gone to sleep, hunter. Not too late to shut your eyes, if you prefer.

            *** Kashio has sent an assassin for you. Defend yourself!
        ]],
        
        OPT_ATTACK = "Attack the assassin",
        DIALOG_ATTACK = [[
            player:
                $angryDefault
                You shoulda stayed in the shadows, bud. This ain't gonna end well for you.
                !fight
            agent:
                !fight
                I was just hoping you'd make the job interesting.
        ]],

        OPT_DEMORALIZE = "Demoralize the assassin before attacking",
        DIALOG_DEMORALIZE = [[
            player:
                !angry_shrug
                Do you even know who you're working for?
            agent:
                Caring isn't part of the job description.
            player:
                Well maybe you should reevaluate that. Kashio ain't like other clients...
        ]],
        OPT_ATTACK_DEMORALIZED = "Rush the assassin!",
        DIALOG_DEMORALIZED = [[
            player:
                Just remember: she wouldn't want to have me killed if she didn't consider me a sizable threat.
                !cruel
                $miscMocking
                But no problem—<i>you'll</i> handle what Kashio couldn't, right?
            * The assassin doesn't answer, but you can tell you've left them off-balance. 
        ]],
        DIALOG_ATTACK_DEMORALIZED = [[
            player:
                !fight
                Hah!

        ]],

        DIALOG_NOT_DEMORALIZED = [[
            player:
                So? Do you really want to work for someone like that?
            agent:
                !fight
                I should get paid extra for having to listen to this.
        ]],
        DIALOG_ATTACK_NOT_DEMORALIZED = [[
            player:
                !fight
                Can't blame me for trying!
        ]],

        NEGOTIATION_REASON = "Give {agent} an {battle.EXISTENTIAL_CRISIS} to help in the coming battle ({1} {1*stack|stacks} on win)",

       
        OPT_DEFEND_SELF = "Defend yourself",
    }
    :Fn(function(cxt) 
            if cxt:FirstLoop() then
                TheGame:SubmitLog("DAY 2 BOSS")
                cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GROG_N_DOG.inn_room") )
                TheGame:GetGameState():GetPlayerAgent():MoveToLocation( TheGame:GetGameState():GetLocation("GROG_N_DOG.inn_room") )
                cxt.enc:GetScreen():ClearHistory()
                cxt.enc:GetScreen():SetBlur(false)
                cxt:Dialog("DIALOG_INTRO")
                
                if cxt.quest.param.boss_id == nil then
                    cxt.quest.param.boss_id = TheGame:GetGameProfile():GetNoStreakRandom("SAL_DAY_2_BOSS_PICK", {"JAKES_ASSASSIN", "JAKES_ASSASSIN2"}, 2)
                end
                local assassin = cxt.quest:CreateSkinnedAgent( cxt.quest.param.boss_id )

                TheGame:GetGameState():AddAgent( assassin )
                assassin:MoveToLocation(cxt.location)
                cxt.enc:SetPrimaryCast(assassin)
                cxt:Dialog("DIALOG_INTRO_2")
            end

            local won_bonuses = {}

            cxt:Opt("OPT_DEMORALIZE")
                :Dialog("DIALOG_DEMORALIZE")
                :Negotiation{
                    on_start_negotiation = function(minigame)
                        local mod = minigame.opponent_negotiator:CreateModifier( "IMPENDING_DOOM", 2 )
                        mod.result_table = won_bonuses

                        local mod = minigame.opponent_negotiator:CreateModifier( "IMPENDING_DOOM", 3 )
                        mod.result_table = won_bonuses

                        local mod = minigame.opponent_negotiator:CreateModifier( "IMPENDING_DOOM", 5 )
                        mod.result_table = won_bonuses
                    end,

                    reason_fn = function(minigame)
                        local total_amt = 0
                        for k,v in pairs(won_bonuses) do
                            total_amt = total_amt + v
                        end
                        return loc.format(cxt:GetLocString("NEGOTIATION_REASON"), total_amt )
                    end,

    
                    on_success = function(cxt) 

                        cxt:Dialog("DIALOG_DEMORALIZED")
                        cxt:Opt("OPT_ATTACK_DEMORALIZED")
                            :Dialog("DIALOG_ATTACK_DEMORALIZED")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.ISOLATED,
                                on_start_battle = function(battle) 
                                    
                                    local total = 0
                                    for k,v in pairs(won_bonuses) do
                                        total = total + v
                                    end

                                    if total > 0 then
                                        for k,v in pairs (battle:GetTeam(TEAM.RED):GetFighters()) do
                                            v:AddCondition("EXISTENTIAL_CRISIS", total)
                                        end
                                    end

                                end,
                                on_win = function(cxt)
                                    cxt:GoTo(cxt:GetAgent():IsDead() and "STATE_POST_FIGHT_DEAD" or "STATE_POST_FIGHT_ALIVE")
                                end,
                            }
                    end,
                    on_fail = function(cxt)
                        cxt:Dialog("DIALOG_NOT_DEMORALIZED")
                        cxt:Opt("OPT_DEFEND_SELF")
                            :Dialog("DIALOG_ATTACK_NOT_DEMORALIZED")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.ISOLATED,
                                advantage = TEAM.RED,
                                on_win = function(cxt) 
                                    cxt:GoTo(cxt:GetAgent():IsDead() and "STATE_POST_FIGHT_DEAD" or "STATE_POST_FIGHT_ALIVE")
                                end,
                            }

                    end,
                }

            cxt:Opt("OPT_DEFEND_SELF")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.ISOLATED,
                    advantage = TEAM.BLUE,
                    -- noncombatants = cxt.quest.param.party_members,
                    on_win = function(cxt) 
                        if cxt:GetAgent():IsDead() then
                            cxt:GoTo("STATE_POST_FIGHT_DEAD")
                        else
                            cxt:GoTo("STATE_POST_FIGHT_ALIVE")
                        end
                    end,
                }
        end)    
    
    :State("STATE_POST_FIGHT_ALIVE")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !injured
                * {fssh} charges into the room, moving spryly for a kra'deshi her age. Her jaw locks when she sees the mess.
                fssh:
                    !right
                    !angry
                    $angryTakenAback
                    That's the stranger I told you about. You all right, kid?
                player:
                    !crossed
                    $angryAside
                    Kashio sent them. 
                fssh:
                    !spit
                    $angryDefault
                    Typical Kashio, leaving others to clean up her mess.
            ]],
            OPT_DISPOSE = "Get {fssh} to dispose of the assassin",
            OPT_LET_GO = "Let the assassin go",
            DIALOG_DISPOSE = [[
                player:
                    !palm
                    $neutralResigned
                    Can you take care of this, {fssh}?
                fssh:
                    !right
                    !angry
                    !agree
                    $angryDefault
                    No problem. I've got a reputation to uphold, after all.
                agent:
                    !left
                    !injured
                    $angryAside
                    You used to be someone to fear, {fssh}. Now you're just old. 
                fssh:
                    $happyResigned
                    !overthere
                    No "used to be" about it. Allow me to demonstrate.
                    !exit
                agent:
                    !exit
                * The deed is done quicker than you can bat an eye. If anything, your room is cleaner than it was before.
                player:
                    !left
                    Good reminder not to mess with you.
                fssh:
                    !right
                    !wink
                    Did you need one? You get some sleep, {player}. I'll stand guard.
                    !exit

            ]],
            DIALOG_LET_GO = [[
                agent:
                    !injured
                    !right
                player:
                    !cruel
                    $angryCruel
                    Go back to Kashio and tell her this only whets my appetite.
                agent:
                    $angrySeething
                    It's your funeral. Even if it's been postponed.
                    !exit
                * {agent} leaves by the door, but you make a note to reinforce the barricades on all the windows.
                fssh:
                    !right
                    !dubious
                    I gotta say, kid, this seems like a bad idea.
                player:
                    !happy
                    $happyCocky
                    Just imagine the look on Kashio's face when she finds out her assassin failed.
                fssh:
                    !shrug
                    $neutralDubious
                    Sure. Satisfyin'. I just hope this doesn't come back to haunt us. Now I gotta go back to boobytrappin' the bar in earnest. 
                * {fssh} stands guard by your door for the rest of the night, leaving you to catch up on your shut-eye. 
                    !exit
            ]],
        }
        :Fn(function(cxt)
                cxt:Dialog("DIALOG_INTRO")
                
                cxt:Opt("OPT_DISPOSE")
                    :Fn(function() 
                            cxt.quest:Complete("go_home")
                            cxt:GetAgent():Kill()
                            DoAutoSave()
                            cxt:Dialog("DIALOG_DISPOSE")           
                            cxt:End()
                        end)

                cxt:Opt("OPT_LET_GO")
                    --we really have to make this come up again. Maybe the assassin will be in the finale if they're alive and were spared.
                    :ReceiveOpinion( cxt.quest:GetQuestDef():GetOpinionEvent("spared_assassin" ))
                    :Fn(function() 
                            cxt:GetAgent():MoveToLimbo()
                            cxt.quest:Complete("go_home")
                            DoAutoSave()
                            cxt:Dialog("DIALOG_LET_GO")            
                            cxt:End()
                        end)
            end)        

    :State("STATE_POST_FIGHT_DEAD")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !exit
                * {agent} lies dead at your feet. {fssh} charges into the room, moving spryly for a kra'deshi her age.
                fssh:
                    !right
                    !angry
                    $angryTakenAback
                    That's the stranger I told you about. You all right, kid?
                player:
                    !crossed
                    $angryAside
                    Kashio sent them. 
                fssh:
                    !spit
                    $angryDefault
                    Typical Kashio, leaving others to clean up her mess.
                player:
                    !exit
                fssh:
                    !exit
                * It takes you both a while to deal with the mess, but when you're done it's cleaner than it was to start.
                player:
                    !left
                fssh:
                    !right
                    !tired
                    $neutralWhatever
                    There. Complimentary house cleaning. Now you should sleep soundly. In theory.
                * {fssh} stands guard by your door for the rest of the night, leaving you to catch up on your shut-eye. 
                    !exit

            ]],

        }
        :Fn(function(cxt)
                cxt.quest:Complete("go_home")
                DoAutoSave()
                cxt:Dialog("DIALOG_INTRO")
                cxt:End()
                --cxt:GoTo("STATE_BACK_TO_SLEEP")         
            end)


QDEF:AddConvo("do_sleep")
    :ConfrontState("STATE_BACK_TO_SLEEP")
        :Loc{
            DIALOG_BACK_TO_SLEEP = [[
                player:
                    !exit
                * You spend time slapping together some makeshift security; if anyone comes through the windows now, you'll know about it.  
            ]],
            
            DIALOG_WAKE = [[
                * When you wake up, you'd almost think the Grog is beginning to look homey—if you hadn't almost been murdered in it.
            ]],
        }
        :Fn(function(cxt)
            --cxt.enc:GetScreen():SetBlur(true)
            ConvoUtil.GiveBossRewards(cxt, GRAFT_TYPE.COMBAT)
            cxt:Dialog("DIALOG_BACK_TO_SLEEP")
            --cxt.enc:GetScreen():SetBlur(false)

            ConvoUtil.DoSleep(cxt, "DIALOG_WAKE" )
            cxt.quest:Complete()
            --DoAutoSave()

           
            cxt:Opt("OPT_LEAVE")
                :MakeUnder()
                :Fn(function() 
                    cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GROG_N_DOG") )
                    cxt:End()
                end)
        end)   
