
local NUMBER_OF_JOBS_TO_DO = 1
local NUMBER_OF_JOBS_TO_SPAWN = 2
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


local QDEF = QuestDef.Define
{
    title = "Getting Settled",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_workin.tex"),
    qtype = QTYPE.STORY,
    desc = [[Continue working for {handler}.]],

    on_start = function(quest) 
        quest:Activate("start")
    end,
    
    --we changed the structure of some days
    on_post_load = function(quest)
        if quest:IsActive() and TheGame:GetGameState():GetCreationBuild() < 444763 then
            if Iter.values(quest.objectives):all(function(q) return not quest:IsActive(q.id) end ) and quest:IsInactive("return_to_grog") then
                quest:Activate("return_to_grog")
            end
        end
    end,

    events = 
    {
        quests_changed = function (quest, event_quest)
            if quest:IsActive("do_a_handler_job") and event_quest == quest.param.handler_job then
                if event_quest:IsComplete() or event_quest:IsFailed() then
                    quest:Complete("do_a_handler_job")
                end
            end

            if quest:IsActive("do_fsshcake_job") and event_quest == quest.param.fsshcake_job then
                if not event_quest:IsActive() then
                    quest:Complete("do_fsshcake_job")
                end

            end

        end,
    },

}

QDEF:AddObjective{
    id = "contact_handler",
    title = "Visit {handler}",
    desc = "{handler} has another special mission for you. Go do {handler.hisher} dirty work.",
    --desc = "You've completed Fssh's jobs. Return to the Grog to see if she's found any leads for you.",
    mark = {"handler"},
}

QDEF:AddObjective{
    id = "do_a_handler_job",
    title = "Do a job for {handler}",
    hide_in_overlay = true,
    --desc = "You've completed Fssh's jobs. Return to the Grog to see if she's found any leads for you.",
    
    on_complete = function(quest)
        quest:Activate("return_to_grog")
    end,
}


--QuestUtil.AddSideJobChain(QDEF, NUMBER_OF_JOBS_TO_SPAWN, NUMBER_OF_JOBS_TO_DO, "return_to_grog")
--QuestUtil.AddSideJobChain(QDEF, NUMBER_OF_JOBS_TO_SPAWN, NUMBER_OF_JOBS_TO_DO, "sleep_at_grog")


QDEF:AddCast{
    cast_id = "handler",
    when = QWHEN.MANUAL,
    no_validation = true,
}

QDEF:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}



QDEF:AddObjective{
    id = "return_to_grog",
    title = "Return to the Grog",
    mark = {"fssh"},
    on_activate = function(quest) 
        QuestUtil.SpawnOpportunities()
    end,
}

QDEF:AddObjective{
    id = "do_fsshcake_job",
    on_activate = function(quest) 
        UIHelpers.PassTime(DAY_PHASE.NIGHT)
        quest.param.merchant_job = QuestUtil.SpawnQuest("ROAD_SIDE_MERCHANTS")
        quest.param.fsshcake_job = QuestUtil.SpawnQuest("SAL_STORY_FSSHCAKES")
        
        --quest.param.main_quest.param.grog_quest.param.no_cakes = true --this is not a good way to do things, but here we are.

        -- ccc: is this really necessary?  its nice to have the merchant available.        
        -- if quest.param.merchant_job and quest.param.merchant_job:IsActive() then
        --     quest.param.merchant_job:Cancel()
        -- end

    end,

    on_complete = function(quest)
        quest:Activate("sleep_at_grog")
    end,
}

QDEF:AddObjective{
    id = "sleep_at_grog",
    title = "Sleep at the Grog",
    desc = "Time to get some rest.",
    mark = {"fssh"},
    on_activate = function(quest) 
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true
    end,
    on_complete = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
    end,
}

QDEF:AddObjective{
    id = "start",
    on_activate = function(quest)
        quest.param.merc_quest = QuestUtil.SpawnQuest("SAL_STORY_MERCS")
    end,

}

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("start", "fssh")
    :ConfrontState("START")
        :Loc{
            DIALOG_FAST_INTRO = [[
                fssh:
                    Morning, Sal. Why don't you go see what {handler} has for you? I'm a little busy, here.
            ]],
            DIALOG_LONG_INTRO = [[
            * You find {fssh} behind the bar, rigging the bartop to snap with electricity. She turns off the current as you approach.
                player:
                    !dubious
                    $neutralJoke
                    You know customers drink at this bar, right?
                fssh:
                    !right
                    !tired
                    $neutralDirect
                    Customers ain't got nothing to worry about. But no more assassins on my watch—that's a guarantee.
                    This'll take a while to set up, so why don't you go bother {handler}, see what you can do to get closer to Kashio.
                    !interest
                    $miscAlluring
                    I'll have a job for you later, if you're interested. 
                * She turns back to her bar and flips the switch. Clearly, you've been dismissed.
                *** {fssh} advised you to check in with {handler}. 
            ]],

        }
        :Fn(function(cxt)   
            cxt.quest:Complete("start")
            -- cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Activate("contact_handler")
            cxt:Dialog("DIALOG_LONG_INTRO")
            StateGraphUtil.AddEndOption(cxt)
        end)



QDEF:AddConvo("return_to_grog", "handler")
    :Loc{
        DIALOG_INTRO = [[
            agent:
                I'm done with you for the day.
                Keep your head down out there. Word is you've made some enemies.
        ]],
        DIALOG_ASKED_ALL_QUESTIONS = [[
            {bandits?
                agent:
                    That all?
                player:
                    I suppose so.
            }
            {admiralty?
                agent:
                    $miscMischievous
                    With an interrogation like this, you might've missed your calling in the Admiralty.
                player:
                    All right, we'll call it there.
                    !humouring
                    For now.
            }
        ]],
    }
    :Questions("SPREE", {
        "Ask about Nadan",
        [[
            player:
                $neutralDubious
                You're not exactly how I expected a bandit kingpin to act.
            agent:
                $neutralDirect
                Who says I'm a kingpin?
            player:
                !humouring
                Well, that's what you'll be once we take out Kashio, right?
            agent:
                ... 
                !shrug
                Naw. Not my style.
            player:
                !overthere
                Okay, so that's what I mean—
                $neutralDubious
                What <i>is</i> your style?
            agent:
                !sigh
                ... 
                Class.
                Principles.
                !eureka
                Integrity.
            player:
                !overthere
                You think the stuff you're having me do gives you integrity?
            agent:
                No.
                $neutralJoke
                That's why <i>you're</i> doing it.
        ]],
        "Ask about the Spree",
        [[
            player:
                To hear folks on the street talk, you have this kind of... bandit prince vibe.
                Most of them are scared, others think you provide legit protection.
                !overthere
                But almost every Spree I come across is just a straight-up thug.
            agent:
                !dubious
                So?
            player:
                $neutralDubious
                So you don't think maybe you're at odds with your own goons?
            agent:
                I'm me. 
                They're them.
                !thumb
                The folks under my wing do what I say, when I say it.
                What they get up to in their own time is their business.
                As long as they don't stick my name on it.
            player:
                !interest
                Huh.
            agent:
                Listen.
                If I'd done it any different, I'd be dead.
                Kashio woulda seen to it.
            player:
                Really? Now that's a story I'd like to hear.
            agent:
                !crossed
                No.  
            ]],
        "Ask about Oolo",
        [[
            player:
                !bashful
                I know I'm not supposed to say her name...
                But I'm too curious not to ask.
                What's the deal with you and that Admiralty officer?
            agent:
                $angryDefault
                She's Admiralty. I'm Spree. 
                She tried to kill me. I want to return the favor.
            player:
                !dubious
                $neutralDubious
                Come on, it can't be that simple.
                You're too practical for that. 
            agent:
                $angrySeething
                It <i>is</i> that simple. 
                !thumb
                And if you need clarity, just look at my face.
            player:
                !interest
                Your face? Oolo did tha—
            agent:
                !angry_accuse
                Not another word. 
        ]],
    })
    :Questions("ADMIRALTY", {
        "Ask about Kashio",
        [[
            player:
                !interest
                So why've you got it out for Kashio?
            agent:
                Because I'm an officer of the law. Why else?
            player:
                $miscMischievous
                I just reckon your definition of "the law" is personally motivated.
            agent:
                Well, fine. You asked. 
                !crossed
                It's not just my superiors who take her bribes.
                $neutralResigned
                My parents do, too.
            player:
                !suspicious
                $neutralDubious
                Your parents are in business with Kashio?
            agent:
                !agree
                They got into contract trading to gain more clout with the local oozers.
                But Kashio has a way of bending gravity around her.
                !overthere
                My parents are up to their eyestalks in it.
            player:
                $neutralDubious
                So you're trying to set them free?
            agent:
                !cruel
                $neutralJoke
                Hah! Nothing so doting, believe me. I just want to pull the rug out from under them. 
                $neutralWhatever
                Because clearly the message didn't get through the first time.
        ]],
        "Ask about Oolo's parents",
        [[
            player:
                What's the deal with your folks, anyway?
            agent:
                !sigh
                $angryReluctant
                They're entitled oozers. 
                !shrug
                You know the sort—they came of age when the seas were fruitful, overfished, and now wonder why their brood can't catch a measly anchovy.
            player:
                Huh.
            agent:
                I suppose your parents were laborers? Must've been nice.
                $neutralJoke
                They knew what they were setting you up for: nothing good. 
            player:
                !shrug
                Well... they tried to strike. Demanded fair pay, proper treatment. 
                $neutralWhatever
                And then they died.
            agent:
                !impressed
                $miscAlluring
                And you're here taking nothing for granted except what you can take for yourself.
                The self-sufficiency looks good on you. 
        ]],
        "Ask about Nadan",
        [[
            player:
                $neutralDubious
                So about that Nadan guy...
                Maybe I could arrange a truce?
                !thought
                Unless you think he'd betray us to Kashio.
            agent:
                !chuckle
                Oh, not bogging likely. 
                If there's anyone Nadan hates more than me, it's Kashio.
                His interest in getting her is legit.
                !throatcut
                $neutralJoke
                But if he can get a knife between my eyestalks at the same time, that'd be his lucky day.
            player:
                !interest
                What's Kashio to him? A rival?
            agent:
                !neutral
                He was her protégé once. Until she decided she didn't want to cut him into the profits anymore.
                But at least for a while he was a "legitimate businessman", as far as the law goes.
            player:
                Ain't the law the only thing that matters?
            agent:
                You and I both know it's just slavery by a different name.
                When you run out of debts to broker, you make new ones.
                !overthere
                $neutralWhatever
                Burn down a business, ransack a caravan, and suddenly you got yourself a contract for free labor.
            player:
                !angry_shrug
                Sure. Kashio even writes in children as collateral.
                !neutral
                I didn't realize you were actually against debt brokering.
            agent:
                $neutralWhatever
                Just feels a little bit too much like how kra'deshi parents treat their brood.
                I <i>like</i> arresting scumbags. But if someone does something illegal because they're desperate...
                Takes the fun out of it. Just a bit.
                !happy
                $happyJoke
                If everyone's equal, then my job only gets better.
        ]],
    })
    :Hub(function(cxt)

        if not cxt.quest.param.handler_spoke_about_sides then
            cxt.quest.param.handler_spoke_about_sides = true
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                local inserted = cxt:InsertQuestions(cxt:GetAgent():GetFactionID() == "BANDITS" and "SPREE" or "ADMIRALTY")
                if inserted == 0 then
                    cxt:Dialog("DIALOG_ASKED_ALL_QUESTIONS")
                    cxt:End()
                else
                    StateGraphUtil.AddEndOption(cxt)
                end

            end)
        end
end)


QDEF:AddConvo("return_to_grog")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == TheGame:GetGameState():GetLocation("GROG_N_DOG") end)
        :Loc{
            DIALOG_INTRO = [[
                * {fssh} looks worried—but you'd be worried, too, if <i>your</i> bar was smoking.
                player:
                    !left
                    !dubious
                    $neutralJoke
                    All right, Fssh?
                fssh:
                    !right
                    !tired
                    $miscDepressed
                    We have a problem. Some of the supplies I need to finish this up are contraband. Apparently.
                player:
                    $neutralDubious
                    I see where this is going but I don't know how to stop it from getting there.
                fssh:
                    $neutralTakenAback
                    You wanna be gently shook from your slumber by a new assassin each night? 
                    !overthere
                    I need you to head down to the docks and grease the wheels, else I'll never get this done. 
                *** {fssh} needs you to grab some supplies from the docks, where the goods have been seized.
            ]],
            OPT_TAKE_JOB = "Head out for the dock",
            DIALOG_TAKE_JOB_BOUNCER = [[
                player:
                    !happy
                    $happyResigned
                    I guess I'm headed to the docks, then.
                fssh:
                    !handwave
                    $neutralWhatever
                    Feel free to take my bouncer with you. Might take some convincing, though.
                *** {fssh} suggests you take her bouncer with you, if you can convince them.
            ]],
            DIALOG_TAKE_JOB_NO_BOUNCER = [[
                player:
                    !happy
                    $happyResigned
                    I guess I'm headed to the docks, then.
                fssh:
                    $neutralWhatever
                    Don't take too long. 
                *** You head out.
            ]],
        }
    :SetLooping()
    :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("fssh"))
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:Complete("return_to_grog")
            end

            cxt:Opt("OPT_ASK_ABOUT")
                :IsHubOption(true)
                :GoTo("STATE_QUESTIONS")

            cxt:Opt("OPT_TAKE_JOB")
                :PreIcon(global_images.close)
                :ActivateQuest("do_fsshcake_job")
                :Fn(function(cxt)
                    if cxt.quest.param.fsshcake_job:GetCastMember("bouncer") then
                        cxt:Dialog("DIALOG_TAKE_JOB_BOUNCER")
                    else
                        cxt:Dialog("DIALOG_TAKE_JOB_NO_BOUNCER")
                    end

                end)
                :End()
        end)

    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask about booby trap",
        [[
            player:
                !permit
                $neutralDubious
                Is this really necessary, Fssh? We've done all right so far.
            agent:
                $neutralWhatever
                Sure, if you don't count last night. 
            player:
                !interest
                I'm alive, aren't I?
            agent:
                $neutralResigned
                Might not have gone that way. Might've gone <i>very</i> different. 
        ]],
        "Ask about Admiralty",
        [[
            player:
                !dubious
                $neutralDubious
                Can't you get your friends in the Admiralty to help you with this?
            agent:
                $neutralWhatever
                Calling in favors for any old problem is exactly how you run out of favors to call in. 
                !interest
                $neutralResigned
                Besides, with the auction getting closer I don't exactly know <i>who</i> my friends are. 
                !throatcut
                $neutralDirect
                Winds change. Opportunities knock. Heads roll. 
        ]],
    })
------------------------------------------------------------------------------------------------------ 


QDEF:AddConvo("sleep_at_grog", "fssh")
    :Loc{
        OPT_GET_A_ROOM = "Rest in the back",
        REQ_NOT_ALONE = "You need to get rid of your company first",
        DIALOG_GET_ROOM = [[
            player:
                !exit
            agent:
                !exit
            * {fssh} disables the alarm long enough for you to slip through the curtain, home at last.
        ]],
        DIALOG_WAKE = [[
            * Something about the sunlight here feels murky and thick, like a sludge tide.
            * But there's no use sleeping in. You only just got back in the griftlands, and already you've made yourself an enemy too powerful to ignore.
            * Nothing for it but to forge ahead, and hope you can make some allies to match. 
        ]],
    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_GET_A_ROOM")
            :PreIcon(global_images.sleep)
            :SetQuestMark()
            :Fn(function(cxt) 
                
                cxt:Dialog("DIALOG_GET_ROOM")
                
                cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GROG_N_DOG.inn_room") )

                ConvoUtil.DoSleep(cxt, "DIALOG_WAKE" )

                if cxt.quest.param.fsshcake_job and cxt.quest.param.fsshcake_job:IsComplete("return_to_fssh") then
                    cxt.quest.param.main_quest.param.grog_quest.param.no_cakes = nil
                end
                cxt.quest:Complete()

                cxt:Opt("OPT_LEAVE")
                    :MakeUnder()
                    :Fn(function() 
                        cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GROG_N_DOG") )
                        cxt:End()
                    end)

            end)
        end)            




QDEF:AddAttract("contact_handler", "handler")
    :Loc{
        DIALOG_ADMIRALTY = [[
            player:
                !greeting
                $happyGreeting
                How are the gears of government today, Oolo? Still grinding?
            agent:
                !happy
                !crossed
                Jammed, I'm afraid. Which means I have plenty of need for your skills. I've got two jobs I could use you for. 
                !wink
                $neutralJoke
                And since I don't hate you, I'll even let you pick.
        ]],
        DIALOG_SPREE = [[
            player:
                !greeting
                $neutralJoke
                Well, Nadan, what extralegal shenanigans do you want me to get up to today?
            agent:
                !crossed
                $angryDefault
                Don't know if I appreciate you makin' light of the work we do here, but as long as you get the job done, I guess I can't complain.
                $neutralDirect
                I got two jobs for you today. Pick the one where you'd be the most effective.
        ]],
        OPT_JOB_PICK = "Job: <b>{1}</>",
        DIALOG_PICKED = [[
            player:
                !salute
                $neutralJoke
                Mission received. I'll see you when the job is done.
        ]],
        DIALOG_PICK_SOMETHING = [[
            agent:
                !dubious
                $neutralDubious
                You're going to have to pick <i>something</>.
            player:
                !interest
                I'm thinkin' about it! 
        ]]

    }
    :SetLooping()
    :Fn(function(cxt) 
        if cxt:FirstLoop() then
            cxt:Dialog(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "DIALOG_ADMIRALTY" or "DIALOG_SPREE")
            
            cxt.quest.param.job_choices = {}
            for k = 1, 2 do
                local job = cxt.quest.param.main_quest:GetQuestDef().TrySpawnHandlerQuest(cxt.quest.param.main_quest)
                job:SetHideInOverlay(true)
                table.insert(cxt.quest.param.job_choices, job)
            end
        end

        if cxt.quest.param.job_choices then
            for k, quest in ipairs(cxt.quest.param.job_choices) do
                cxt:QuestOpt( quest )
                    :ShowQuestAsInactive()
                    :Fn(function(cxt) 
                            StateGraphUtil.PresentQuestOffer(cxt, quest, nil, 
                            function() 
                                cxt.quest:Complete("contact_handler")                
                                cxt.quest.param.handler_job = quest
                                for k, v in pairs(cxt.quest.param.job_choices) do
                                    if v ~= quest then
                                        v:Cancel()
                                    end
                                end

                                cxt.quest.param.job_choices = nil
                                cxt.quest:Activate("do_a_handler_job")
                                
                                if not cxt:PlayQuestConvo(quest, QUEST_CONVO_HOOK.ACCEPTED) then
                                    cxt:Dialog("DIALOG_PICKED")
                                end
                                quest:SetHideInOverlay(false)

                                StateGraphUtil.AddEndOption(cxt)

                            end, 
                            function() 
                                
                                if not cxt:PlayQuestConvo(quest, QUEST_CONVO_HOOK.DECLINED) then
                                    cxt:Dialog("DIALOG_PICK_SOMETHING")
                                end
                            end,
                            true ) -- MUST ACCEPT, as invokikng the option spawns the quest
                        end)
            end
        else
            cxt:End()
        end

    end)    


