local NUMBER_OF_JOBS_TO_SPAWN = 3
local NUMBER_OF_JOBS_TO_DO = 1

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


local QDEF = QuestDef.Define
{
    title = "Final Pieces",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_kill_a_big.tex"),
    qtype = QTYPE.STORY,
    desc = [[It is the day before the auction. {handler} has a special job for you.]],

    on_start = function(quest) 
        quest:Activate("start")
    end,

    avoids_fn = function(quest, agent, location)
        if location == TheGame:GetGameState():GetLocation("GROG_N_DOG") and agent:IsQuestProvider() then
            return true
        end
    end,

    events = 
    {
        quests_changed = function(quest, event_quest) 
            if quest:IsActive("do_handler_job") and event_quest == quest.param.handler_job then
                if event_quest:IsComplete() or event_quest:IsFailed() then
                    quest:Complete("do_handler_job")
                end
            end
        end,
    },

    collect_agent_locations = function(quest, t)
        if quest:IsActive("do_handler_job") then
            if not quest:GetCastMember("other_handler"):IsInPlayerParty() then
                table.insert (t, { agent = quest:GetCastMember("other_handler"), location = quest:GetCastMember('trap_location')} )
            end
        end
    end,
}

:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}

:AddCast{
    cast_id = "handler",
    when = QWHEN.MANUAL,
    no_validation = true,
}

:AddCast{
    cast_id = "other_handler",
    when = QWHEN.MANUAL,
    no_validation = true,
}


:AddCastByAlias{
    cast_id = "admiralty_contact",
    alias = "MURDER_BAY_ADMIRALTY_CONTACT",
}

:AddCastByAlias{
    cast_id = "bandit_contact",
    alias = "MURDER_BAY_BANDIT_CONTACT",
}


:AddObjective{
    id = "go_home",
    title = "Return to the Grog",
    desc = [[Go back to the Grog and rest before the auction.]],
    mark = {"home_base", "fssh"},
    on_activate = function(quest) 
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true
        TheGame:GetGameState():AddPendingEvent("faction_hassle")
        --QuestUtil.SpawnQuest("ROAD_SIDE_MERCHANTS")
    end,
    on_complete = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
    end,
}

:AddObjective{
    id = "meet_with_handler",
    title = "Meet with {handler}",
    mark = {"handler"},
    on_activate = function(quest) 
        UIHelpers.PassTime(DAY_PHASE.NIGHT)
        QuestUtil.SpawnQuest("ROAD_SIDE_MERCHANTS")
        QuestUtil.SpawnOpportunities()
    end,
    on_complete = function(quest)
        quest:Activate("do_handler_job")
    end,

}

:AddObjective{
    id = "do_handler_job",
    title = "Do {handler}'s mission",
    hide_in_overlay = true,
    on_activate = function(quest)
        quest.param.handler_job = QuestUtil.SpawnInactiveQuest(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "SAL_STORY_CAPTURE_NADAN" or "SAL_STORY_KILL_OOLO")
        if quest.param.handler_job == nil then
            quest:Complete("do_handler_job")
        end

    end,
    on_complete = function(quest)
        quest:Activate("go_home")
    end,
}


:AddObjective{
    id = "start",
    on_activate = function(quest)
        quest.param.merc_quest = QuestUtil.SpawnQuest("SAL_STORY_MERCS")
    end,
}

QuestUtil.AddSideJobChain(QDEF, NUMBER_OF_JOBS_TO_SPAWN, NUMBER_OF_JOBS_TO_DO, "meet_with_handler")


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("start", "fssh")
    :ConfrontState("START")
        :Loc{
            DIALOG_INTRO = [[
            * When you come out of your room in the morning, {fssh} looks noticeably uncomfortable. 
                fssh:
                    !forgetful
                    Hey, {player}? You have a visitor.
                player:
                    !dubious
                    $neutralTakenAback
                    Oh? 
            ]],

            DIALOG_ADMIRALTY = [[
                handler:
                    !right
                    So, this is where you lay your head, is it?
                    Can you recommend anything on the menu?
                player:
                    !humouring
                    $happyTakenAback
                    {handler}? What brings you here?
                    Didn't think you'd want to slum it with me here at the Grog.
                handler:
                    Why not? 'Specially after I heard you were running an errand for me last night. Down at the docks?
                    Only I don't recall asking you to run an errand...
                player:
                    !forgetful
                    Ah, about that—
                {liked?
                handler:
                    !handwave
                    Hah! It's fine. You have your reasons.
                    Besides, give it a few more days and <i>I'll</i> be name-dropping <i>you</i>.
                }
                {not liked?
                handler:
                    $neutralWhatever
                    It's fine. This is an arrangement of convenience, after all.
                    !crossed
                    But if I catch you using my name to make trouble for my officers again, then maybe I won't find you quite as useful as before.
                }
                player:
                    Is that why you came? To make sure I wasn't being naughty?
                handler:
                    $miscHushedUrgent
                    I've got an opportunity for you. A big one. I can't talk too much about it here—besides, the clandestine angle is more fun.
                    Come talk to me tonight when you're done with your other affairs. 
                    !wink
                    $happyCocky
                    Do this one last thing for me, and I'll get you into the auction tomorrow.
            ]],
            DIALOG_SPREE = [[
                handler:
                    !right
                    !dubious
                    This is a good set-up you have here. 
                    Booby traps. And Fssh with her eye on the door. 
                    !agree 
                    Not bad for a derrick. 
                player:
                    $neutralJoke
                    Nice to see you too, Nadan. You want a drink?
                {liked?
                handler:
                    Hrm. 
                    !humouring
                    Maybe next time.
                }
                {not liked?
                handler:
                    I ain't here for camaraderie.
                    $angryDefault
                    And I wouldn't get it from you, besides.
                player:
                    !dubious
                    Offense taken.
                }
                handler:
                    !point
                    $neutralDirect
                    Time for you to prove your worth. I've got a big job for you tonight. 
                    It needs your full attention, so come to me when you're ready to focus. 
                    !eureka
                    $neutralResigned
                    You do this, and do it right—Hesh, I'll help you take down Kashio myself. 
            ]],
            DIALOG_PST = [[
                player:
                    !interest
                    $happyTakenAback
                    So tomorrow's the big day, is it?
                fssh:
                    !right
                    You better get ready for it. I've got a couple more jobs you can do to prepare.
                handler:
                    !right
                    $neutralDirect
                    Find me when the sun goes down, and I'll tell you what you need to know.
                    !exit
                *** {handler} is hatching a plan for you to capture {other_handler} tonight. Do jobs to prepare for a tough fight.
            ]],
        }
        :Fn(function(cxt)   
            cxt.quest:AssignCastMember("other_handler", cxt.quest:GetCastMember(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "bandit_contact" or "admiralty_contact"))
            
            cxt.quest:Complete("start")
            cxt:Dialog("DIALOG_INTRO")
            --cxt.quest:GetCastMember("handler"):MoveToLocation(cxt.location)

            if Util.IsPlayerWorkingWithFaction("ADMIRALTY") then
                cxt:Dialog("DIALOG_ADMIRALTY")
            else
                cxt:Dialog("DIALOG_SPREE")
            end

            cxt:Dialog("DIALOG_PST")
            cxt:GetCastMember("handler"):MoveToLimbo()
            cxt.quest:Activate("pick_a_side_job")
            StateGraphUtil.AddEndOption(cxt)
        end)

------------------------------------------------------------------------------------------------------ 




QDEF:AddConvo("go_home", "fssh")
    :Loc{
        OPT_GET_A_ROOM = "Rest in the back",
        DIALOG_GET_ROOM = [[
            player:
                !exit
            agent:
                !exit
            * {fssh} shows you to your room.
        ]],
        DIALOG_WAKE = 
        [[
            * The auction is today. You hope you're ready for it.
        ]]
    }
    :Hub(function(cxt) 

        cxt:Opt("OPT_ASK_ABOUT")
           :IsHubOption(true)
           :GoTo(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "STATE_QUESTIONS_OOLO" or "STATE_QUESTIONS_NADAN")     

        cxt:Opt("OPT_GET_A_ROOM")
            :PreIcon(global_images.sleep)
            :SetQuestMark()
            :Fn(function(cxt) 
                
                cxt:Dialog("DIALOG_GET_ROOM")
                cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GROG_N_DOG.inn_room") )
                ConvoUtil.DoSleep(cxt, "DIALOG_WAKE")
                cxt.quest:Complete()
                cxt:Opt("OPT_LEAVE")
                    :MakeUnder()
                    :Fn(function() 
                        cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GROG_N_DOG") )
                        cxt:End()
                    end)
                

            end)
    end)    
    :AskAboutHub("STATE_QUESTIONS_OOLO", 
            {

               "Ask about Grout Bog",
               [[
                    player:
                        You ever think about going back?
                    agent:
                        !dublious
                        $neutralTakenABack
                        To Grout Bog?! What the Hesh for?
                    player:
                        !interest
                        I hear they're starting it up again—the strikes, I mean. Fighting for equal pay.
                    agent:
                        $angryReluctant
                        Yeah, well, some folks got short memories.
                        I don't need to see that go down again.
                        !thumb 
                        At least last time I had two eyes.
                    player:
                        $neutralResigned
                        Yeah. You're right. 
                    agent:
                        ... 
                        I guess there's one thing that's different about it this time around.
                        You're not a little kid anymore.
               ]],

               "Ask about Fssh",
               [[
                    player:
                        !bashful
                        Hey. You got plans for tomorrow?
                        In case, you know—things don't work out the way we hope?
                    agent:
                        !suspicious
                        What? Like if you die?
                    player:
                        Kashio will come for you if I'm not able to stop her.
                    agent:
                        !agree
                        Easy fix for that. 
                        Don't die.
                    player:
                        !humouring
                        I'll do my best.
                    agent:
                        !accuse
                        I mean it, Sal. It's not an option. 
                ]],
            })
        
    :AskAboutHub("STATE_QUESTIONS_NADAN", 
            {

               "Ask about Grout Bog",
               [[
                    player:
                        You ever think about going back?
                    agent:
                        !dublious
                        $neutralTakenABack
                        To Grout Bog?! What the Hesh for?
                    player:
                        !interest
                        I hear they're starting it up again—the strikes, I mean. Fighting for equal pay.
                    agent:
                        $angryReluctant
                        Yeah, well, some folks got short memories.
                        I don't need to see that go down again.
                        !thumb 
                        At least last time I had two eyes.
                    player:
                        $neutralResigned
                        Yeah. You're right. 
                    agent:
                        ... 
                        I guess there's one thing that's different about it this time around.
                        You're not a little kid anymore.
               ]],

               "Ask about Fssh",
               [[
                    player:
                        !bashful
                        Hey. You got plans for tomorrow?
                        In case, you know—things don't work out the way we hope?
                    agent:
                        !suspicious
                        What? Like if you die?
                    player:
                        Kashio will come for you if I'm not able to stop her.
                    agent:
                        !agree
                        Easy fix for that. 
                        Don't die.
                    player:
                        !humouring
                        I'll do my best.
                    agent:
                        !accuse
                        I mean it, Sal. It's not an option. 
                ]],
            })
        


QDEF:AddAttract("meet_with_handler", "handler")
    :Loc{
        DIALOG_ADMIRALTY = [[
            agent:
                !greeting
                Ah, there you are. 
                Ready to change the course of history, my best and shiniest hunter?
        ]],
        DIALOG_SPREE = [[
            agent:
                !happy
                Good. You're here.
                Glad I didn't have to send someone to drag you in. 
        ]],
    }
    :Fn(function(cxt) 
            cxt:Dialog(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "DIALOG_ADMIRALTY" or "DIALOG_SPREE")
            cxt.quest:Complete("meet_with_handler")
            if cxt.quest.param.handler_job then
                StateGraphUtil.PresentQuestOffer(cxt, cxt.quest.param.handler_job, nil, 
                    function() 
                        cxt:PlayQuestConvo(cxt.quest.param.handler_job, QUEST_CONVO_HOOK.ACCEPTED)
                    end, nil, true)
            end
    end)    


--after talking to your handler
QDEF:AddConvo("pick_a_side_job" , "fssh")

    :Hub(function(cxt) 
        cxt:Opt("OPT_ASK_ABOUT")
           :IsHubOption(true)
           :GoTo(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "STATE_QUESTIONS_OOLO" or "STATE_QUESTIONS_NADAN")     
    end)

    :AskAboutHub("STATE_QUESTIONS_OOLO", 
            {

               "Ask about {handler}",
               [[
                    player:
                        !interest
                        You all right, Fssh? You seemed kinda nervous. 
                        $miscThoughtful
                        I thought Oolo was a friend of yours.
                    agent:
                        !handwave
                        $neutralResigned
                        Oh, Oolo's alright. It's just her family that's the problem.
                        So much as gleek at their darling daughter and you invite a world of hurt.
                    player:
                        !thought
                        $miscThoughtful
                        Oolo doesn't seem the type to solve problems by slinging her name around.
                    agent:
                        !humouring
                        Sometimes I forget you aren't kra'deshi. You don't know what it's like, being the only daughter in a brood.
                        On the one side you gotta carry the good weight of the family's name, but on the other, you aren't allowed to take any risks.
                        !point
                        It's a squeeze <i>you'd</i> push back on, and Oolo's no different.
                    player:
                        $neutralResigned
                        I guess my folks aren't around to treat me like a power play.
                    agent:
                        !placate
                        They'd've never done. Your folks were the type to make starlight outta broken glass. 
                        $angryReluctant
                        Kashio didn't deserve to walk in their shadow, let alone hold their debts. 
                    player:
                        !thumb
                        $miscEmbarrassed
                        I, uh, guess I should get moving, huh? 
                        Got a big day ahead. 
               ]],
            })
         
    :AskAboutHub("STATE_QUESTIONS_NADAN", 
            {

               "Ask about {handler}",
               [[
                    player:
                        What was that? Never seen you squirm so much.
                    agent:
                        !cringe
                        You <i>seen</i> that face? 
                        All humans are a little bit gross, but that's a whole 'nother level. 
                    player:
                        $miscEmbarrassed
                        Ouch.
                    agent: 
                        !chuckle
                        Don't act like you didn't know. You were always goofy-lookin'.
                        You made up for it by being the smartest damn kid in the whole bog. 
                    player:
                        You miss being on the Spark Barons' payroll?
                    agent:
                        $angryReluctant
                        Hesh no. The drinks were flat and the rations were stale.
                        Plus I hated hunting truants. They were just honest folk trying to get by.
                    player:
                        Kashio hated it, too. 
                    agent:
                        Not the same. What I hated was stepping on the workers. What <i>she</i> hated was being stepped on by the Barons.
                        $neutralDisappointed
                        If I'da known that at the time, well. Maybe I would've tried harder to get your parents out of the bog before things got outta hand.
                        Or at least bought your bond myself.
                    player:
                        !humouring
                        $neutralJoke
                        Hm. I guess paying <i>you</i> back for my parents' debts would've been a bit better than the derrick.
                    agent:
                        !handwave
                        $neutralWhatever
                        Ach, nevermind. I woulda fed you to the yotes. 
                        Then you'd be even uglier. 

               ]],
            })
        