local CleanUpFn = function(quest) 
    if quest:GetCastMember("bouncer") and quest:GetCastMember("bouncer"):IsInPlayerParty() then
        quest:GetCastMember("bouncer"):Dismiss()
    end
end


local QDEF = QuestDef.Define
{
    title = "Roaloch and Load",
    desc = [[Fssh has asked you to help get a shipment unstuck.]],
    icon = engine.asset.Texture("icons/quests/sal_story_merchant_combat.tex"),
    qtype = QTYPE.STORY,
    
    block_side_jobs = true,
    on_fail = CleanUpFn,
    on_complete = CleanUpFn,
    on_cancel = CleanUpFn,
}

:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}



:AddCastByAlias{
    cast_id = "oolo",
    alias = "MURDER_BAY_ADMIRALTY_CONTACT",
    no_validation = true,
}

:AddLocationCast{
    cast_id = "docks",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MURDER_BAY_HARBOUR"))
    end,
}

:AddCast{
    cast_id = "fssh_contact",
    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "JAKES_SMUGGLER" )
        table.insert(t, agent)
    end,
}

:AddCast{
    cast_id = "bouncer",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GROG_N_DOG"):GetWorkPosition("bouncer"):GetAgent())
    end,
    optional = true,
}

:AddCast{
    cast_id = "the_ship",
    when = QWHEN.MANUAL,
    no_validation = true,
}



:AddObjective{
    id = "go_to_docks",
    title = "Go to the docks",
    desc = "Fssh's package is being held up at the docks",
    state = QSTATUS.ACTIVE,
    on_activate = function(quest)
        TheGame:SubmitLog("DAY 3 BOSS QUEST START")
        quest.param.boss_id = quest.param.boss_id or TheGame:GetGameProfile():GetNoStreakRandom("SAL_DAY_3_BOSS_PICK", {"SHROOG", "DRUSK_1"}, 2)
        if quest:GetCastMember("bouncer") then
            quest:Activate("get_bouncer")
        end
    end,
    on_complete = function(quest)
        quest:Activate("get_the_package")
    end,
    mark = {"docks"},
}

:AddObjective{
    id = "get_the_package",
    title = "Get {fssh}'s package",
    desc = "Get on the ship, and find the package",
    on_complete = function(quest)
        quest:Activate("return_to_fssh")
    end,
    mark = {"docks"},
}

:AddObjective{
    id = "return_to_fssh",
    title = "Bring the package back to {fssh}",
    desc = "You have the package, now return it to {fssh}",
    mark = {"fssh"},
}

:AddObjective{
    id = "get_bouncer",
    -- state = QSTATUS.ACTIVE,
    title = "(Optional) convince {bouncer} to help you",
    desc = "{fssh} said {fssh.heshe} could spare {bouncer} for a while.",
    mark = {"bouncer"},
    negotiation_targets = {"bouncer"},
}


:AddLocationDefs{
    SHIPDECK = {
        name = "Ship Deck",
        plax = "INT_Shipdeck_1",
        indoors = false,
    },
}


:AddOpinionEvents{

    got_ship_out_of_quarantine =  
    {
        delta = OPINION_DELTAS.LOVED,
        txt = "Got their ship out of quarantine",
    },

    stole_a_package =  
    {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Stole a package from them",
    },
    
}



QDEF:AddConvo("go_to_docks")
    :ConfrontState("STATE_CONFRONT_ADMIRALTY", function(cxt) return cxt.location == cxt.quest:GetCastMember("docks") end)
        :Loc{
            DIALOG_INTRO = [[
                * It's dark when you arrive. The docks should be closed, but they shimmer with activity.
                * It's not long before you spot why: a ship from the Roaloch is moored in the harbor. No doubt there's a quarantine in place.
                * The guards on duty approach you.
            ]],
            DIALOG_GREETING_ADMIRALTY = [[
                agent:
                    !right
                    !dubious
                    $neutralDubious
                    Hey, ain't you {oolo}'s little plaything? The hunter she keeps on a soft leash?
                player:
                    !left
                    !thought
                    $neutralThoughtful
                    Is that the way <i>she</i> describes me?
                agent:
                    !neutralResigned
                    Dock's closed, grifter. Even if {oolo} sent you. 
                    $miscMocking
                    We got a Roaloch shipment, and I bet you'd hate to be the one to test its quarantine. 

                *** You need to get past {agent} to find the shipment Fssh is waiting for.
            ]],

            DIALOG_GREETING = [[
                agent:
                    !right
                    !dubious
                    $neutralDubious
                    Who's that? Show yourself!
                player:
                    !left
                    !salute
                    $happyDefault
                    Howdy, officers. It's just me, your friendly neighborhood hunter.
                agent:
                    !neutralResigned
                    Oh yeah? Dock's <i>closed</i>, hunter. Especially to you.
                    !cruel
                    $miscMocking
                    We got a Roaloch shipment, and I bet you'd hate to be the one to test its quarantine. 

                *** You need to get past {agent} to find the ship.
            ]],


            OPT_ASK_ABOUT_CLOSURE = "Ask about the quarantine",
            DIALOG_ASK_ABOUT_CLOSURE = [[
                player:
                    !interest
                    When does the quarantine lift? 
                agent:
                    !overthere
                    $neutralWhatever
                    No clue. Something's off with the boat. The skipper's not responding to signals. 
                    $neutralResigned
                    Wouldn't be the first time the Roaloch shipped a plague with the manifest. 
            ]],
            DIALOG_ASK_ABOUT_CLOSURE_ADMIRALTY = [[
                player:
                    !interest
                    $miscPersuasive
                    At least give me an estimate. What am I supposed to tell {oolo}?
                agent:
                    !chuckle
                    $miscMocking
                    Tell her not to send her pets face-first into an outbreak.
                    The Roaloch ship isn't responding to signals, so who knows what they might be carrying.
            ]],


            OPT_NEGOTIATION = "Talk your way through",
            
            DIALOG_NEGOTIATION_ADMIRALTY = [[
                player:
                    !crossed
                    {oolo}'s no dummy, and I'm not a child. She trusts me for a reason.
            ]],
            DIALOG_NEGOTIATION = [[
                player:
                    !angry_shrug
                    What's it to you if I get sick? Just look the other way, switch.  
            ]],


            DIALOG_NEGOTIATE_SUCCESS = [[
                player:
                    !overthere
                    $miscPersuasive
                    I'm a hunter. I can take care of myself, and this is a waste of all our time.
                agent:
                    !handwave
                    $angryReluctant
                    You know what? Fine. Go on through. 
                    With any luck you'll break out in putrid warts. Then I'll be laughing. 
                    !exit
                player:
                    !disgust
                    $neutralDubious
                    Blech. I hope not. 
                    !exit

                *** You convinced {agent} to let you through.                    
            ]],
            DIALOG_NEGOTIATE_FAIL = [[
                player:
                    !overthere
                    $miscPersuasive
                    It's late. Do you get paid enough for this?
                agent:
                    !angry
                    $angryDefault
                    No, I don't. So slag off.
            ]],

            OPT_PAY_BRIBE = "Bribe your way through",
            DIALOG_BRIBE = [[
                player:
                    !permit
                    Maybe I didn't make myself clear. 
                * You shake out a few shills into your palm and fan them out prettily. 
                agent:
                    !interest
                    $happyTakenAback
                    Oh, I see we've just had a little misunderstanding. 
                    !take
                    $happyDefault
                    Everything's clear as rainwater now, hunter. You head right on in.
                    !exit
                player:
                    !exit
                *** You pay off {agent} and {agent.heshe} lets you through.
            ]],

            OPT_FIGHT_THROUGH = "Attack the guards",
            DIALOG_FIGHT_THROUGH = [[
                player:
                    !disappoint
                    $neutralResigned
                    Sorry, switches. It ain't nothing personal. You're just... in my way.
                    !fight
            ]],

            DIALOG_FOUGHT_KILLED = [[
                * {agent} is dead. {oolo} might have something to say about that, and it probably won't be nice.
            ]],
            
            DIALOG_FOUGHT_SURVIVED = [[
                player:
                    !left
                    Back down, switch! It doesn't have to be your life on the line.
                agent:
                    !right
                    !injured
                    I'm down, I'm down! I'll curl up in a corner and nobody has to know you didn't beat me unconscious. 
                    Just go in, and leave me alone.
                    !exit
                * {agent} does exactly as {agent.heshe} promised, and leaves you to your work.

                *** You beat up {agent} to get past {agent.himher}.
            ]],

        }
    :SetLooping()
    :Fn(function(cxt) 
            
            local BRIBE_AMT = 100 --enough that you might not want to pay it, but you probably can

            if cxt:FirstLoop() then
                cxt.location:ClearAgents()
                cxt.quest:Complete("go_to_docks")
                cxt.quest.param.guard = TheGame:GetGameState():AddSkinnedAgent("ADMIRALTY_PATROL_LEADER") --this is a new guy, because I want the dialog to be based on your factional relationship.
                local extras = CreateCombatBackup(cxt.quest.param.guard, "ADMIRALTY_PATROL_BACKUP", cxt.quest:GetRank()+1)

                --AgentUtil.SpawnTransientParty({"ADMIRALTY_GUARD", "ADMIRALTY_GUARD"}, cxt.quest.param.guard)
                cxt.enc:SetPrimaryCast(cxt.quest.param.guard)
                cxt:Dialog("DIALOG_INTRO")
                cxt:Dialog(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "DIALOG_GREETING_ADMIRALTY" or "DIALOG_GREETING")
                if cxt.quest:IsActive("get_bouncer") then
                    cxt.quest:Cancel("get_bouncer")
                end

            end
            
            cxt:Opt("OPT_NEGOTIATION")
                :Dialog(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "DIALOG_NEGOTIATION_ADMIRALTY" or "DIALOG_NEGOTIATION")
                :Negotiation{
                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_NEGOTIATE_SUCCESS")
                        cxt:GoTo("STATE_SHIPDECK_TRANSITION")
                    end,

                    on_fail = function(cxt) 
                        cxt:Dialog("DIALOG_NEGOTIATE_FAIL")
                    end,
                }


            cxt:Opt("OPT_PAY_BRIBE")
                :PreIcon( global_images.giving )
                :DeliverMoney(BRIBE_AMT)
                :Dialog("DIALOG_BRIBE")
                :GoTo("STATE_SHIPDECK_TRANSITION")


            cxt:Opt("OPT_FIGHT_THROUGH")
                :Dialog("DIALOG_FIGHT_THROUGH")
                :Battle{
                    on_win = function(cxt) 
                        if cxt:GetAgent():IsDead() then

                            if Util.IsPlayerWorkingWithFaction("ADMIRALTY") then
                                cxt.quest:GetCastMember("oolo"):OpinionEvent(OPINION.WORKED_AGAINST_THE_ADMIRALTY)
                            end
                            
                            cxt:Dialog("DIALOG_FOUGHT_KILLED")
                        else
                            cxt:Dialog("DIALOG_FOUGHT_SURVIVED")
                        end

                        cxt:GoTo("STATE_SHIPDECK_TRANSITION")
                    end,
                }

            cxt:Opt("OPT_ASK_ABOUT_CLOSURE")
                :Dialog(Util.IsPlayerWorkingWithFaction("ADMIRALTY") and "DIALOG_ASK_ABOUT_CLOSURE_ADMIRALTY" or "DIALOG_ASK_ABOUT_CLOSURE")

            
        end)



    :State("STATE_FIGHT_BEAST")
        :Loc{
            DIALOG_INTRO = [[
                left:
                    !exit
                right:
                    !exit
                * You almost make it when you hear a gurgling from across the bow.
            ]],
            DIALOG_SHROOG = [[
                agent:
                    !right
                * A hulking beast heaves itself out of the water, shaking off droplets that sizzle on contact. 
            ]],
            DIALOG_CLERK_FIGHT = [[
                clerk:
                    !left
                * {clerk} rushes to your side, {clerk.hisher} mouth twisted in anger.
                clerk:
                    !fight
                    I knew it! Hunter, you're officially deputized until this monster is destroyed!
            ]],
            DIALOG_JAKE_FIGHT = [[
                jake:
                    !left
                * {jake} rushes to your side, looking suitably slack-jawed.
                jake:
                    !scared
                    My dock! No, no! 
                    !fight
                    I'll die first before you ruin me, you Roalochian nightmare!
            ]],
            DIALOG_NO_ONE_FIGHT = [[
                * With a quick glance, {jake} and {clerk} both dive overboard to avoid the beast. It turns its attention to you.
            ]],
            OPT_DEFEND = "Defend yourself!",
            DIALOG_ATTACK = [[
                player:
                    !left
                    !fight
                * You charge!
            ]],

            DIALOG_PST_FIGHT_CLERK = [[
                player:
                    !left
                    !tired
                    $neutralTakenAback
                    Is this normal for the Roaloch?!
                clerk:
                    !right
                    !disappoint
                    $neutralResigned
                    Never seen a Roalochian aberration quite like that. I was lucky you were here, hunter.
                    You can keep that parcel as payment—I'll turn a blind eye to its contents just this once. 
                    !dubious
                    $neutralDubious
                    As long as it's not from the Roaloch, of course.
                    !exit
                * {clerk} rallies {clerk.hisher} troops for a clean-up brigade.
                *** You have defeated the monster, and {clerk} lets you leave with {fssh}'s package.
            ]],
            DIALOG_PST_FIGHT_JAKE = [[
                player:
                    !left
                    !tired
                    $neutralResigned
                    Blech. Did any of its blood get on you? Someone said something about... warts.
                jake:
                    !right
                    !scared_shrug
                    $scaredDefault
                    I hate to say it, but that switch had a point. Imagine that thing running loose in Murder Bay.
                    !dubious 
                    Speakin' of—you're not supposed to be here, hunter. What was it you wanted?
                player:
                    Oh, ah—just that package, there.
                jake:
                    !give
                    That all? Here, take it. 
                    You better push off before the reinforcements arrive.
                player:
                    !agree 
                    Good idea. See you around! Hopefully wart-free.
                    !exit
                *** You have defeated the monster with {jake}'s help.
            ]],
            DIALOG_PST_FIGHT = [[
                player:
                    !left
                * The ship deck is covered with ichor. You gingerly pick up the package and leave before anyone shows up to stop you.

            ]]
        }
        :Fn(function(cxt)
            
            cxt:Dialog("DIALOG_INTRO")
            
            if not cxt.quest.param.jake_help then
                cxt.quest.param.ship_jake:MoveToLimbo()
            end

            if not cxt.quest.param.clerk_help then
                cxt.quest.param.ship_clerk:MoveToLimbo()
            end
            cxt.quest.param.boss_id = cxt.quest.param.boss_id or TheGame:GetGameProfile():GetNoStreakRandom("SAL_DAY_3_BOSS_PICK", {"SHROOG", "DRUSK_1"}, 2)
            cxt.quest.param.beast = TheGame:GetGameState():AddSkinnedAgent(cxt.quest.param.boss_id) 
            cxt.enc:SetPrimaryCast(cxt.quest.param.beast)

            local allies = {}
            cxt:Dialog("DIALOG_SHROOG")
            if cxt.quest.param.clerk_help then
                cxt:Dialog("DIALOG_CLERK_FIGHT")
                table.insert(allies, cxt.quest.param.ship_clerk)
            elseif cxt.quest.param.jake_help then
                cxt:Dialog("DIALOG_JAKE_FIGHT")
                table.insert(allies, cxt.quest.param.ship_jake)
            else
                cxt:Dialog("DIALOG_NO_ONE_FIGHT")
            end

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    flags = BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.SELF_DEFENCE,
                    allies = allies,
                    on_win = function()
                        if cxt.quest.param.clerk_help and not cxt.quest.param.ship_clerk:IsDead() then
                            cxt:Dialog("DIALOG_PST_FIGHT_CLERK")
                        elseif cxt.quest.param.jake_help and not cxt.quest.param.ship_jake:IsDead() then
                            cxt:Dialog("DIALOG_PST_FIGHT_JAKE")
                        else
                            cxt:Dialog("DIALOG_PST_FIGHT")
                        end
                        cxt.quest:Complete("get_the_package")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }


        end)

    :State("STATE_SHIPDECK_TRANSITION")
        :Loc{
            
            OPT_MOVE = "Find the ship",
            DIALOG_MOVE = [[
                * You make your way onto the deck of the Roaloch ship.
            ]],

        }
        :Fn(function(cxt) 
            cxt:Opt("OPT_MOVE")
                :Fn(function()
                    local ship = cxt.quest:SpawnTempLocation("SHIPDECK", "the_ship" )
                    ship:SetMapPos(cxt.location:GetMapPos())
                    cxt.encounter:DoLocationTransition( ship )
                end)
                :Dialog("DIALOG_MOVE")
                :GoTo("STATE_SHIPDECK_CONFRONTATION")

        end)
    :State("STATE_SHIPDECK_CONFRONTATION")
        :Loc{
            DIALOG_INTRO = [[
                * You come across an Admiralty officer, neck-deep in debate with a Jakes stevedore. 
                * The glint of a small box catches your eye from a pile of contraband. You recognize it as the goods Fssh needs.
                clerk:
                    !right
                    !angry
                jake:
                    !left
                    !angry
                clerk:
                    !angry_accuse
                    $angryDefault
                    I don't care what schedule you're on, that ship stays in quarantine.
                jake:
                    !overthere
                    $angryDefault
                    I got other ships to unload! What am I supposed to tell the Heshians?
                    That they should <i>delay</i> the auction 'cause their goods are held up?
                clerk:
                    !angry_shrug
                    $angryPatienceLost
                    You let more ships in, and that's exactly how infection spreads. Or are you too much of an ignoramus to follow?
                * You figure this is a good time to break the tension. 
                player:
                    !left
                clerk:
                    !palm
                    $angryReluctant
                    What are you doing here, grifter? This is a secure area.
                *** {clerk} and {jake} are arguing about releasing the ship. You can see {fssh}'s package on the dock.                   
            ]],

            OPT_OFFER_TO_ATTACK_CLERK = "Offer to attack the Admiralty",
            DIALOG_OFFER_TO_ATTACK_CLERK = [[
                * You pull the stevedore aside for a quick private chat, though of course the Admiralty clerk is immediately suspicious.
                jake:
                    !right
                player:
                    !cagey
                    $miscHushedUrgent
                    You want this switch gone? I can handle that for you. At a discount, even.
                jake:
                    !handwave
                    $neutralTakenAback
                    What?! No! If I start putting switches in the ground every time we disagree, I'd be locked up. 
                    If you wanna help, do it... graciously. You know, diplomacy-style. I ain't got much of that myself.
                clerk:
                    !right
                    !dubious
                    $neutralDubious
                    Excuse me, but I asked you a question, hunter. What are you doing here?
                *** {jake} knows that violence won't solve this problem.
            ]],

            OPT_REMIND_BRIBE = "Remind {clerk} of your recent bribe",

            OPT_CUT_A_SIDE_DEAL = "Cut a side deal with {clerk} to get the package",
            DIALOG_CUT_A_SIDE_DEAL = [[
                player:
                    !forgetful
                    I'm, uh, just here to get a package. I don't care about the quarantine or... whatever.
                clerk:
                    !crossed
                    $neutralDirect
                    Bit too late for that, considering you've breached it. 
                player:
                    !handwave
                    $miscPersuasive
                    Enh, I'm fine. But that package right there... what do I have to do to snag it off you?
            ]],

            DIALOG_CUT_A_SIDE_DEAL_SUCCESS = [[
                player:
                    !interest
                    $miscAlluring
                    You've had a long day saying no. Just say yes for this one thing—you'll feel better, I promise.
                clerk:
                    !palm
                    $neutralResigned
                    Ugh, you grifters have a lot of nerve. But if one package is all you want, it's easy enough to give it a simple inspection.
                jake:
                    !right
                    Typical! Exceptions abound when someone with power and influence walks through. 
                player:
                    !dubious
                    $neutralTakenAback
                    Power and influence? You mean <i>me</i>?
                clerk:
                    !right
                    !crossed
                    $neutralWhatever
                    This package is unusual, but I can release it to you, hunter—assuming your license is in order.
                player:
                    !give
                    Oh, sure. Sometimes I forget I have this.
                clerk:
                    !dubious
                    Looks above board. Be sure to register if you construct this into any, I don't know, weapons or traps.
                    $neutralDirect 
                    Those require different paperwork.
                player:
                    !embarrassed
                    $neutralDubious
                    Uh... yeah. Will do. No problem.
                *** You convinced {clerk} to release your package.
            ]],

            DIALOG_CUT_A_SIDE_DEAL_FAIL = [[
                player:
                    !shrug 
                    $miscPersuasive
                    Come on, it's just one little box. What's the harm?
                clerk:
                    !angry_accuse
                    $neutralResigned
                    This dock is in quarantine. The box is on the dock. Ergo, the box is quarantined.
                    And so are <i>you</i>, hunter. Why doesn't anyone understand what that means?
                *** {clerk} won't cut you a special deal.
            ]],

            OPT_GRAB_YOUR_PACKAGE_AND_RUN = "Grab the package and make a break for it",
            DIALOG_GRAB_AND_RUN = [[
                player:
                    !placate
                    Oh! I was just looking for the privy. I'll show myself out.
                    !exit
                * You nonchalantly walk past the pile of goods, {clerk} and {jake} both watching you intently.
                jake:
                    !left
                clerk:
                    !crossed
                    $neutralDubious
                    Where does she think she's going?
                * You <i>discreetly</i> pick up the package and hold it behind your back.
                jake:
                    !thought
                    $neutralResigned
                    Pretty sure that box has explosives in it.
                clerk:
                    !angry_accuse
                    $angryDefault
                    Hey! We can see what you're doing!
                * You make a run for it.
                * It's the perfect crime.
                *** You steal the package, in the most obvious way possible, and make a break for it.
            ]],

            OPT_RUN_FOR_IT = "Run for it!",

            OPT_CONVINCE_END_QUARANTINE = "Convince {clerk} to end the quarantine",
            DIALOG_CONVINCE_END_QUARANTINE = [[
                player:
                    !overthere
                    $miscPersuasive
                    Listen, {agent}, you gotta think clearly here. The auction's in two days.
                clerk:
                    !angry 
                    Do you think I don't know that?
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !interest
                    $miscPersuasive
                    You haven't even let the Roaloch ship dock yet. So just keep it out there until you hear from the skipper.
                    In the meantime, let the Heshian ships come and go as they please. 
                    !happy
                    $miscPersuasive
                    Quarantine is preserved, and so is your sanity. 
                clerk:
                    !palm
                    $neutralResigned
                    It's disorderly, but with the auction in two days, I suppose it can't be helped.
                    Fine, {jake}. The quarantine is lifted. 
                    !angry_accuse
                    $neutralDirect
                    But if that Roaloch ship drifts even an inch, I want to be told immediately!
                    !exit
                * {clerk} wanders off to sign some documents.
                jake:
                    !right
                    !happy
                    Thanks for that, grifter. My jaw was starting to ache from all that chin-wagging. 
                
                *** You convinced {clerk} to release the ship. {jake} is impressed with you.
            ]],
            
            DIALOG_CONVINCE_FAILURE = [[
                player:
                    !shrug 
                    I don't get how anyone can get sick if the ship hasn't even docked yet.
                clerk:
                    !palm
                    $neutralResigned
                    That's because you're ignorant. It's not just infectious diseases that hail from the Roaloch, hunter. 
                    If just <i>one</i> shrooglet has found its way on board... 
                * {clerk} shudders in open horror.
                *** {clerk} refuses to release the ship.
            ]],
            SIT_MOD = "Going against orders",
            

        }   
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.quest.param.ship_jake = AgentUtil.GetFreeAgent("JAKES_SMUGGLER")
                cxt.quest.param.ship_clerk = AgentUtil.GetFreeAgent("ADMIRALTY_CLERK")
                cxt.quest.param.ship_jake:MoveToLocation(cxt.location)
                cxt.quest.param.ship_clerk:MoveToLocation(cxt.location)

                cxt:ReassignCastMember("clerk", cxt.quest.param.ship_clerk)
                cxt:ReassignCastMember("jake", cxt.quest.param.ship_jake)

                cxt:Dialog("DIALOG_INTRO")
                cxt.enc:SetPrimaryCast(cxt.quest.param.ship_clerk)
            end

            if cxt.quest.param.ship_clerk.bribed then
                cxt:Opt("OPT_REMIND_BRIBE")
                    :Dialog("DIALOG_CUT_A_SIDE_DEAL_SUCCESS")
                    :Fn( function( cxt )
                        cxt.quest.param.clerk_help = true
                    end )
                    :GoTo("STATE_FIGHT_BEAST")

            else
                cxt:Opt("OPT_CUT_A_SIDE_DEAL")
                    :Dialog("DIALOG_CUT_A_SIDE_DEAL")
                    :Negotiation{
                        on_success = function() 
                            cxt.quest.param.clerk_help = true
                            cxt:Dialog("DIALOG_CUT_A_SIDE_DEAL_SUCCESS")
                            cxt:GoTo("STATE_FIGHT_BEAST")
                        end,

                        on_fail = function() 
                            cxt:Dialog("DIALOG_CUT_A_SIDE_DEAL_FAIL")
                            cxt:GoTo("STATE_FIGHT_BEAST")
                        end,
                    }
            end

            cxt:Opt("OPT_OFFER_TO_ATTACK_CLERK")
                :Dialog("DIALOG_OFFER_TO_ATTACK_CLERK")

            cxt:Opt("OPT_CONVINCE_END_QUARANTINE")
                :Dialog("DIALOG_CONVINCE_END_QUARANTINE")
                :Negotiation{
                    helpers = { cxt.quest.param.ship_jake },
                    situation_modifiers =
                    {
                        { value = 30, text = cxt:GetLocString("SIT_MOD") }
                    },

                    on_success = function() 
                        cxt.quest.param.jake_help = true
                        cxt.quest.param.ship_jake:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("got_ship_out_of_quarantine"))
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt:GoTo("STATE_FIGHT_BEAST")
                    end,

                    on_fail = function() 
                        cxt:Dialog("DIALOG_CONVINCE_FAILURE")
                        cxt:GoTo("STATE_FIGHT_BEAST")
                    end,
                }
            

            cxt:Opt("OPT_GRAB_YOUR_PACKAGE_AND_RUN")
                :Dialog("DIALOG_GRAB_AND_RUN")
                :Fn(function() 
                    cxt.quest.param.ship_jake:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("stole_a_package"))
                    cxt.quest.param.ship_clerk:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("stole_a_package"))
                end)
                :GoTo("STATE_FIGHT_BEAST")
        end) 


QDEF:AddConvo("get_bouncer", "bouncer")

    :Loc{
        OPT_ASK_TO_COME = "Ask {agent} to help you",
        DIALOG_ASK_TO_COME = [[
            player:
                !greeting 
                $neutralGreeting
                Hey there, {agent}. {fssh} said you might be able to help me solve {fssh.hisher} supply problem.
            agent:
                $neutralDubious
                Did {fssh.heshe}, now?
            player:
                !dubious
                $neutralDubious
                You're {fssh.hisher} <i>employee</i>, right? So you do jobs for {fssh} and in exchange she pays you?
                This would be another one of those times.
            agent:
                !crossed
                $neutralWhatever
                {fssh} and I have a very specific understanding of what is or isn't my job. 
                Make it worth my while, and we can talk.                
            *** {agent} works for {fssh}, not you. This will take some convincing.
        ]],
        OPT_PAY = "Pay {agent} to help",
        DIALOG_PAY = [[
            player:
                We're talking about money, right? I have money.
                !give
                Are we good now?
            agent:
                !salute
                Lead the way, boss.
            ** You hired {agent}.
        ]],


        OPT_NEGOTIATION = "Convince {agent} to do it for free",
        DIALOG_NEGOTIATION = [[
            player:
                !eureka
                $miscAlluring
                Think big picture, {agent}. Imagine how much this would improve your lot.
            agent:
                !interest
                $neutralWhatever
                This should be good. Go on.
        ]],
        DIALOG_NEGOTIATION_WIN = [[
            player:
                !interest
                $miscPersuasive
                We get that stuff for {fssh}, then the bar is safer. If the bar is safer, then your job is easier.
                A little bit of effort now for a whole lot less effort later. Make sense?
            agent:
                !thought
                $neutralThoughtful
                Hrm. Yeah, all right. That makes sense. 
                !point
                $neutralDirect
                Although I gotta say, too much of my job these days involves keeping your scrawny butt safe.
            player:
                !happy
                $neutralJoke
                That must be very hard for you.
            agent:
                $neutralWhatever
                Whatever. Lead the way. 
            ** You convinced {agent} to work for free.
        ]],
        DIALOG_NEGOTIATION_LOSS = [[
            player:
                Picture this—
            agent:
                $neutralDirect
                No. 
            player: 
                !interest
                But you didn't even let me—
            agent:
                !crossed
                $neutralDirect
                Still no. I'm going to stay right here and protect the <i>bar</i>. You know, my actual job. 
            ** You failed to convince {agent}.
        ]],


    }
    :Hub(function(cxt) 
            
            local HIRE_BOUNCER_AMOUNT = 200

            cxt:Opt("OPT_ASK_TO_COME")
                :SetQuestMark()
                :Dialog("DIALOG_ASK_TO_COME")
                :LoopingFn(function(cxt) 
                    
                    cxt:Opt("OPT_PAY")
                        :DeliverMoney(HIRE_BOUNCER_AMOUNT)
                        :Dialog("DIALOG_PAY")
                        :Fn(function() 
                            cxt.quest:Complete("get_bouncer")
                            cxt:GetAgent():Recruit()
                            StateGraphUtil.AddEndOption(cxt)
                        end)

                    cxt:Opt("OPT_NEGOTIATION")
                        :Dialog("DIALOG_NEGOTIATION")
                        :Negotiation{
                            on_success = function(cxt) 
                                cxt.quest:Complete("get_bouncer")
                                cxt:Dialog("DIALOG_NEGOTIATION_WIN")
                                cxt:GetAgent():Recruit()
                                StateGraphUtil.AddEndOption(cxt)
                            end,
                            on_fail = function(cxt) 
                                cxt:Dialog("DIALOG_NEGOTIATION_LOSS")
                            end,
                        }

                    StateGraphUtil.AddBackButton(cxt)

                end)
        end)
    

QDEF:AddConvo("return_to_fssh", "fssh")

    :Loc{
        OPT_GIVE_FSSH_PACKAGE = "Give {fssh} the package",
        DIALOG_GIVE_PACKAGE = [[
            player:
                !give
                $happyDefault
                Here it is, {fssh}. One box of pure contraband. Just, uh, make sure to wash your hands after plugging it in.
            fssh:
                !dubious
                $neutralDubious
                Wash my hands? Why?
            player:
                !thought
                $neutralThoughtful
                Enh... don't worry about it. It's probably fine. 
                !happy
                $happyDefault
                You'd look good with a few extra warts anyway.
            fssh:
                !happy
                $happyDefault
                You're a good kid, Sal. Now go on, get some rest. I'll get to work on this bar. 
        ]],
        DIALOG_GIVE_PACKAGE_BOUNCER_DEAD = [[
            player:
                !give
                $happyDefault
                Here it is, {fssh}. One box of pure contraband. 
            fssh:
                Hey, not bad! Where's {bouncer}?
            player:
                !embarrassed
                I'm, uh, afraid {bouncer.heshe} didn't make it. I'm sorry.
            fssh:
                !sad 
                Ah. Well. 
                Then I'll name this booby-trapped bartop in {bouncer.hisher} honor. 
                !eureka
                The {bouncer} Memorial Death Bar. 
            player:
                !agree
                $neutralDubious
                That's... nice. 
            fssh:
                !happy
                $miscNostalgic
                Yeah. Well, you better go on and get some rest. The {bouncer} Memorial Death Bar isn't gonna build itself.
        ]],
    }
    :Hub(function(cxt) 
            cxt:Opt("OPT_GIVE_FSSH_PACKAGE")
                :SetQuestMark()
                :Fn(function()
                    
                    cxt:Dialog(cxt.quest:GetCastMember("bouncer") and cxt.quest:GetCastMember("bouncer"):IsDead() and "DIALOG_GIVE_PACKAGE_BOUNCER_DEAD" or "DIALOG_GIVE_PACKAGE")
                    cxt.quest:Complete("return_to_fssh")
                    cxt.quest:Complete()
                    cxt:Pop()

                end)
        end)
    