local CleanUpFn = function(quest) 
    if quest.param.hireling and quest.param.hireling:IsInPlayerParty() then
        quest.param.hireling:Dismiss()
    end
end

local QDEF = QuestDef.Define
{
    title = "Kill Oolo",
    icon = engine.asset.Texture("icons/quests/sal_story_kill_oolo.tex"),
    qtype = QTYPE.STORY,
    desc = [[Kidnap {other_handler}'s brother, and use {sibling.himher} as bait to draw {other_handler} into a fight. Kill {other_handler}, and {handler} will get you into the auction.]],
    focus = QUEST_FOCUS.COMBAT,
    block_side_jobs = true,
    
    on_start = function(quest) 
    end,

    on_fail = CleanUpFn,
    on_complete = CleanUpFn,
    on_cancel = CleanUpFn,

    collect_agent_locations = function(quest, t)
        --table.insert (t, { agent = quest:GetCastMember("other_handler"), location = quest:GetCastMember('trap_location')} )
    end,

    restrict_exit = function(quest, location)
        if location == quest:GetCastMember("safehouse") then
            return true
        end
    end,

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("other_handler") then
            fighter:AddCondition("WANTED_DEAD")
        end

    end,

}

:AddCastByAlias{
    cast_id = "other_handler",
    alias = "MURDER_BAY_ADMIRALTY_CONTACT",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "handler",
    alias = "MURDER_BAY_BANDIT_CONTACT",
}

:AddCastByAlias{
    cast_id = "kashio",
    alias = "KASHIO",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "sibling",
    alias = "MURDER_BAY_ADMIRALTY_CONTACT_SIBLING",

    on_assign = function(quest, agent)
        quest:GetCastMember("other_handler").social_connections:SetRelationship(RELATIONSHIP.LOVED, agent)
    end,
}

:AddLocationCast{
    cast_id = "spree_bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("SPREE_INN"))
    end,
}


:AddObjective{
    state = QSTATUS.ACTIVE,
    id = "compromise_guard",
    title = "Compromise {sibling}'s welcome party",
    mark = {"beach"},
    on_activate = function(quest) 
        TheGame:SubmitLog("KILL OOLO BOSS QUEST START")
        quest:SpawnTempLocation("BEACH", "beach" )
        quest:Activate("get_spree_help")
    end,

    on_complete = function(quest)
        
        if quest:IsActive("get_spree_help") then
            quest:Cancel("get_spree_help")
        end

        quest:Activate("take_sibling_to_safehouse")
    end,
}

:AddObjective{
    id = "get_spree_help",
    title = "(Optional) Pick a Spree member to help you",
    desc = "{handler} said you could take one of his Spree to help you, if you can convince them.",
    
    
    on_activate = function(quest) 
        local num_possibles = 0
        for _, agent in TheGame:GetGameState():GetLocation("SPREE_INN"):Agents() do
            if not agent:IsInPlayerParty() and not agent:HasQuestMembership() and agent:GetFactionID() == "BANDITS" then
                num_possibles = num_possibles + 1
            end
        end
        if num_possibles < 3 then
            local extra = AgentUtil.GetFreeAgent("BANDIT_RAIDER")
            extra:MoveToLocation(TheGame:GetGameState():GetLocation("SPREE_INN"))

        end

    end,

    mark = function(quest, t, in_location)

        table.insert( t, quest:GetCastMember("spree_bar"))
        if in_location and TheGame:GetGameState():GetPlayerAgent():GetLocation() == TheGame:GetGameState():GetLocation("SPREE_INN") then
            for _, agent in TheGame:GetGameState():GetLocation("SPREE_INN"):Agents() do
                if not agent:IsInPlayerParty() and not agent:HasQuestMembership() and agent:GetFactionID() == "BANDITS" then
                    table.insert(t, agent)
                end
            end
        end
    end,

}

:AddObjective{
    id = "take_sibling_to_safehouse",
    title = "Go to the safehouse",
    mark = {"safehouse"},
    on_activate = function(quest) 
        quest:SpawnTempLocation("SAFEHOUSE", "safehouse" )
    end,
    on_complete = function(quest)
        quest:Activate("kill_other_handler")
    end,
}

:AddOpinionEvents{
    no_kill_oolo = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Did not kill Oolo",
    },
    
    killed_brother = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Killed their brother",
    },
    killed_sister = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Killed their sister",
    },
}



:AddObjective{
    id = "kill_other_handler",
    title = "Kill {other_handler}",
    mark = {"other_handler"},
    on_activate = function(quest) 
        local ext = quest:SpawnTempLocation("SAFEHOUSE_EXT", "safehouse_ext" )
        ext:SetMapPos(quest:GetCastMember("safehouse"):GetMapPos())
    end,

    on_complete = function(quest)
        quest:Activate("return_to_handler")
    end,
}

:AddObjective{
    id = "explain_handler_escape",
    title = "Convince {handler} that {other_handler} escaped",
    mark = {"handler"},
}


:AddObjective{
    id = "return_to_handler",
    title = "Tell {handler} that {other_handler} is dead",
    mark = {"handler"},
}

:AddLocationDefs{
    SAFEHOUSE = {
        name = "Spree Safehouse",
        --desc = "{other_handler} is here",
        plax = "INT_SpreeShack_1",
        indoors = true,
        map_tags = {"forest"},
    },

    SAFEHOUSE_EXT = {
        name = "Spree Safehouse Exterior",
        --desc = "{other_handler} is here",
        plax = "EXT_SpreeCampsite_2",
        indoors = false,
        map_tags = {"forest"},
    },

    BEACH = {
        name = "Secret Cove",
        --desc = "{other_handler} is here",
        plax = "EXT_Beach_MurderBay_1",
        indoors = false,
        map_tags = {"beach"},
    },
}

:AddLocationCast{
    cast_id = "beach",
    when = QWHEN.MANUAL,
}

:AddLocationCast{
    cast_id = "safehouse",
    when = QWHEN.MANUAL,
}

:AddLocationCast{
    cast_id = "safehouse_ext",
    when = QWHEN.MANUAL,
}



QDEF:AddConvo("return_to_handler", "handler")
    :Loc{
        OPT_TELL_DEAD = "Tell {handler} that {other_handler} is dead",
        DIALOG_TELL_DEAD = [[
            player:
                {other_handler} is dead, as you requested. 
                !shrug
                $miscDepressed
                Gotta say, it got a bit messy there. You not worried about the wrath of a kra'deshi dynasty?
            handler:
                Ha. 
                Do I look like I worry much?
            player:
                I, uh...
                !suspicious 
                It's... hard to say.
            handler:
                Oolo was no good. 
                A dirty cop, spiteful and reckless because her parents would buy her out of trouble.
                $angryAside
                Her whole brood's in Kashio's pocket.
                $happyCocky
                !eureka
                And once we deal with Kashio, there won't be anyone to protect that "dynasty" no more.
                ...
                Thanks for your hard work. 
                I'll send word tomorrow when it's Kashio's turn.
        ]],
        DIALOG_ASKED_ALL_QUESTIONS = [[
            agent:
                ...
            player:
                What?
            agent:
                Tired. You got me talking too much.
                Get some shut eye. Or tomorrow you won't see Kashio comin'.
        ]],
    }
    :Questions("QUESTIONS", {
        "Ask about Oolo",
        [[
            player:
                !interest
                You ever gonna tell me what Oolo did?
            handler:
                Naw. She just ain't got principles.
                $neutralWhatever
                Gets under my skin.
            player:
                !dubious
                Literally or...?
            handler:
                !shrug
                $neutralDubious
                She punched a hole in my face, and you wanna know the details?
                Don't you wanna read some poetry or something? Enhance yourself?
            player:
                !happy
                You read poetry?
            handler:    
                !agree
                Sure I do. I'll lend you a book sometime.
        ]],
        "Ask about tomorrow",
        [[
            player:
                !crossed
                $neutralDubious
                And what about tomorrow? You ready for Kashio?
            handler:
                I am.
                !thought
                $neutralThoughtful
                The whole soul's gonna wash off after this. Like a pearl on the foam.
                This time tomorrow, where do you see yourself?
            player:
                !placate
                Let's not get ahead of ourselves. I'm just taking it as it comes, you know?
            handler:
                !dubious
                $miscAlluring
                Don't tell me you can't picture it. You came all this way.
                I'll manifest it for ya, take it out of the abstract.
                !overthere
                Kashio will be down. And you'll be standing. 
                Taller than you've ever been. And that memory you got of Kashio in your head, of her being powerful and unstoppable...
                That memory won't be there no more.
                !point
                And you'll have room for something new in its place.
            player:
                !interest
                I've <i>never</i> heard you talk so much.
            handler:
                $happyJoke
                I'm damn loquacious, you get to know me well enough.
        ]],
    })

    :Hub(function(cxt) 
            cxt:Opt("OPT_TELL_DEAD")
                :Dialog("DIALOG_TELL_DEAD")
                :ReceiveOpinion(OPINION.DID_LOYALTY_QUEST)
                :CompleteQuest()
                :Fn(function() 
                    cxt:RunLoop(function() 
                        local inserted = cxt:InsertQuestions("QUESTIONS")
                        if inserted == 0 then
                            cxt:Dialog("DIALOG_ASKED_ALL_QUESTIONS")
                            StateGraphUtil.AddEndOption(cxt)
                        else
                            StateGraphUtil.AddEndOption(cxt)
                        end
                    end)
                end)
    end)    


QDEF:AddConvo("explain_handler_escape", "handler")
    :Loc{
        OPT_TELL_GOT_AWAY = "Tell {handler} that {other_handler} escaped",
        DIALOG_TELL_GOT_AWAY = [[
            player:
                !greeting
                $miscEmbarrassed
                So, uh. Funny story, actually—
            handler:
                !angry
                $angryPatienceLost
                {player}, you know damn well I ain't got no sense of humor.
                So if you're trying to delay pissin' me off, you've already failed.
            player:
                !bashful
                $miscEmbarrassed
                {other_handler} got away.
            handler:
                !angry_shrug
                S'what I figured. Shoulda known better than to leave it to the likes of you.
                I'll get you into that damn auction. But don't expect we'll work together again.
            player:
                !angry
                If {other_handler} was so important to you, maybe you should've handled it yourself.
            handler:
                !spit
                $neutralWhatever
                Clearly. 
                Now get outta my face. You got a big day tomorrow and it'd be a shame if I felt like using you for target practice, first.
        ]],

    }
    :Hub(function(cxt) 
        cxt:Opt("OPT_TELL_GOT_AWAY")
            :Dialog("DIALOG_TELL_GOT_AWAY")
            :ReceiveOpinion("no_kill_oolo")
            --cxt:GetCastMember("handler"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("no_capture_nadan"))
            :CompleteQuest()
            :End()
    end)    





QDEF:AddConvo("get_spree_help" )
    :Loc{
        OPT_CONVINCE_TO_HELP = "Convince {agent} to come with you",
        DIALOG_CONVINCE_TO_HELP = [[
            player:
                !greeting
                $happyGreeting
                Hey, it's {agent}! My buddy! My chum!
                I'm doing a big job for {handler}. I could use some help.
        ]],
        DIALOG_CONVINCE_TO_HELP_SUCCESS = [[
            player:
                !interest
                $miscAlluring
                This is an important gig. If we pull it off, we'll be {handler}'s favorites. Easily.
            agent:
                !thought
                $neutralThoughtful
                Hrm. I <i>could</i> use the leverage. {handler}'s been giving me the stink eye a bit more than usual.
                !agree
                Sure, I'll tag along. 
        ]],

        DIALOG_CONVINCE_TO_HELP_FAILURE = [[
            player:
                !overthere
                $miscAlluring
                It's a big job! Big rewards! Don't you want in?
            agent:
                !dubious
                $neutralDubious
                You kiddin'? No way. Everyone knows we're going after {other_handler}. 
                Boss sent you 'cause you're <i>disposable</i>. You're on your own.
            player:
                !disappoint
                $miscDepressed
                Aw, sea spit. 
        ]],

        OPT_PAY_TO_HELP = "Pay {agent} to help you",
        DIALOG_PAY = [[
            player:
                $miscAlluring
                Look, you don't have to do it for free. I can pay you—good money, too.
                !give
            agent:
                !take
                $neutralDirect
                Now we're talkin'. I'll never say no to your money, hunter.
        ]],

    }
    :Hub(function(cxt) 

        if cxt.location == TheGame:GetGameState():GetLocation("SPREE_INN") and not cxt:GetAgent():IsInPlayerParty() and not cxt:GetAgent():HasQuestMembership() and cxt:GetAgent():GetFactionID() == "BANDITS" then

            cxt:Opt("OPT_CONVINCE_TO_HELP")
                :Dialog("DIALOG_CONVINCE_TO_HELP")
                
                :Negotiation{

                    on_success = function(cxt) 
                        cxt:Dialog("DIALOG_CONVINCE_TO_HELP_SUCCESS")
                        cxt:GetAgent():Recruit()
                        cxt.quest.param.hireling = cxt:GetAgent()
                        cxt.quest:Complete("get_spree_help")
                        StateGraphUtil.AddEndOption(cxt)
                    end,


                    on_fail = function(cxt) 
                        cxt:Dialog("DIALOG_CONVINCE_TO_HELP_FAILURE")
                    end,
                }
            
            cxt:Opt("OPT_PAY_TO_HELP")
                :Dialog("DIALOG_PAY")
                :PreIcon( global_images.hire )
                :DeliverMoney(200)
                :Fn(function(cxt)
                    cxt:GetAgent():Recruit()
                    cxt.quest.param.hireling = cxt:GetAgent()
                    cxt.quest:Complete("get_spree_help")
                    StateGraphUtil.AddEndOption(cxt)
                end)
        end

    end)    

---------------------------------------------------------------------------

QDEF:AddConvo("take_sibling_to_safehouse")
    :ConfrontState("THE_STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("safehouse") end)
    :Loc{
        DIALOG_INTRO_CAPTIVE = [[

            * You find the Spree safehouse. {sibling} stares at it, dumbstruck, and you take the opportunity to push {sibling.himher} roughly inside.
            player:
                !left
            sibling:
                !right
                !injured
                $scaredPowerless
                Ow! Stop manhandling me, peasant! 
                $angryAside
                Your criminal lifestyle choices are no excuse for poor breeding. 
            player:
                $neutralAside
                Really? It's done all right so far. 
            * {agent} greets you, casting a disdainful look in {sibling}'s direction.
            agent:
                !right
                $neutralWhatever
                Was he like that the whole way here?
            player:
                !disappoint
                $neutralResigned
                He <i>is</i> a Heshian, after all.
        ]],

        DIALOG_INTRO_ESCORT = [[
            * You find the Spree safehouse and usher {sibling} safely inside.
            player:
                !left
                !overthere
                $miscMocking
                Watch your head, my liege. 
            sibling:
                !right
                !agree
                $neutralDirect
                Why, thank you. I must say, for a hired gun you have shown impeccable respect and—
            * Finally, {sibling} takes in the squalor of the safehouse... and the Spree thug approaching with a length of coarse rope.
            sibling:
                !surprised
                $angryTakenAback
                Why—wait just a moment. You aren't a <i>hoodlum</i>, are you?!
            * {sibling} is quickly bound and gagged by {agent}, who manages it with professional ease.
        ]],
        DIALOG_INTRO_ESCORT_2 = [[
            agent:
                !right
                !tired
                You made it. He give you any trouble?
            player:
                !forgetful
                $neutralResigned
                You know Heshians. They do all the talking and let the tentacles do all the thinking.
        ]],

        DIALOG_INTRO_DEAD = [[
            * You find the Spree safehouse and step inside.
            player:
                !left
            * {agent} greets you, confused.
            agent:
                !right
                Where's the bait?
            player:
                !placate
                $scaredDefault
                Uh, dead, unfortunately. But I left a clear trail for {other_handler} to follow.
            agent:
                !palm
                $neutralResigned
                You better hope that's good enough. I don't wanna be the one to explain this to {handler} if it all goes wrong. 
        ]],

        DIALOG_SPREE_CONVO = [[
            player:
                So what have you heard?
            agent:
                $neutralWhatever
                Apparently {other_handler} was spittin' bricks when {sibling} didn't show up. Ransom note should've gotten there by now. 
                !interest
                One thing, though—word is {other_handler} has one of those pedigree battle oshnus. So don't expect her to go down easy.
            * {agent} begins to pack up {agent.hisher} things, heading for the door.
            player:
                !dubious
                $neutralDubious
                Wait—where are <i>you</i> going?
            agent:
                !shrug
                $neutralWhatever
                {handler} wants me back on the tail end in case anything goes wrong. And, uh, no offense, but it probably will. 
            *** {other_handler} is coming. You can try to convince {agent} to stay and help you fight.
        ]],

        OPT_CONVINCE_TO_STAY = "Convince {agent} to stay and fight",
        OPT_PAY_TO_STAY = "Pay {agent} to stay and fight",
        OPT_LET_GO = "Let {agent} go",
        DIALOG_CONVINCE = [[
            player:
                !dubious
                $neutralTakenAback
                Hang on, I actually need your help here!
        ]],
        DIALOG_CONVINCE_SUCCESS = [[
            player:
                !overthere
                $miscPersuasive
                If you stay, our chances against {other_handler} double. It's simple math.
            agent: 
                $neutralDubious
                Simple math is that {handler} kills me if I don't follow orders.
            player:
                !interest
                $miscPersuasive
                He won't if we win. And if we don't, you'll already be dead.
            agent:
                !thought
                $neutralThoughtful
                Well that—huh. I guess that does make a weird sort of sense. Fine. I'll help you here. But you take the oshnu, all right?
                $miscDepressed
                Those things creep me out. 
        ]],
        DIALOG_CONVINCE_FAIL = [[
            player:
                !overthere
                $miscPersuasive
                If you stay, we can take {other_handler} down together.
            agent:
                $neutralDirect
                Naw. That ain't my problem. YOU want to complain, take it up with the boss.
                !cruel
                $neutralJoke
                But you won't, 'cause you need to stay in his good books as much as I do.
                $neutralJoke
                See you on the surf side, {player}. If you survive, of course.
                !exit
        ]],

        DIALOG_PAY_TO_STAY = [[
            player:
                $miscAlluring
                {handler} isn't paying as well as I am. 
                !give
            agent:
                !take
                Ah, the highest bidder shows her head. Suddenly I'm intrigued.
                !cringe
                $miscEmbarrassed
                Just... keep the oshnu away from me.
        ]],
        DIALOG_LET_GO = [[
            player:
                !hips
                $miscMocking
                Well, didn't realize the odds were so stacked against me, here. I'd say I'm doing pretty well if you ask me.
            agent:
                !shrug
                $neutralWhatever
                Nothing personal, grifter. You survive, and you can tell me you told me so.
                !exit
        ]]
    }

    :SetLooping()
    :Fn(function(cxt) 
        if cxt:FirstLoop() then

            local bandit = AgentUtil.GetFreeAgent("BANDIT_RAIDER")
            bandit:MoveToLocation(cxt.location)
            cxt.enc:SetPrimaryCast(bandit)
            cxt.quest.param.spree_staying = bandit

            cxt.quest:Complete("take_sibling_to_safehouse")
            
            if cxt.quest:GetCastMember("sibling"):IsDead() then
                cxt:Dialog("DIALOG_INTRO_DEAD")
            elseif cxt.quest:GetCastMember("sibling"):IsCaptiveMember() then
                cxt:Dialog("DIALOG_INTRO_CAPTIVE")
                cxt.quest:GetCastMember("sibling"):Dismiss()
            else
                cxt:Dialog("DIALOG_INTRO_ESCORT")
                cxt.quest:GetCastMember("sibling"):OpinionEvent(OPINION.BETRAYED)
                cxt.quest:GetCastMember("sibling"):Dismiss()
                cxt:Dialog("DIALOG_INTRO_ESCORT_2")
            end
            cxt:Dialog("DIALOG_SPREE_CONVO")
        end

        cxt:Opt("OPT_CONVINCE_TO_STAY")
            :Dialog("DIALOG_CONVINCE")
            :Negotiation{
                suppressed = { cxt.quest:GetCastAgent("sibling") },
                on_success = function(cxt) 
                    cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                    cxt:GoTo("STATE_FIGHT_OOLO")
                end,
                
                on_fail = function(cxt) 
                    cxt:Dialog("DIALOG_CONVINCE_FAIL")
                end,
            }

        cxt:Opt("OPT_PAY_TO_STAY")
            :Dialog("DIALOG_PAY_TO_STAY")
            :DeliverMoney(200)
            :GoTo("STATE_FIGHT_OOLO")

        cxt:Opt("OPT_LET_GO")
            :Dialog("DIALOG_LET_GO")
            :Fn(function() 
                cxt.quest.param.spree_staying:MoveToLimbo()
                cxt.quest.param.spree_staying = nil
            end)
            :GoTo("STATE_FIGHT_OOLO")
    end)

    :State("STATE_FIGHT_OOLO")
        :Loc{
            DIALOG_INTRO = [[
                * You sit and wait, and it's not long before you hear something large cleaving its way through the bush outside. 
                player:
                    !left
                    Well, here we go.
            ]],
            DIALOG_SIBLING_PRESENT_INTRO = [[
                * {sibling} wrestles his mouth out of his gag, bleating smugly.
                sibling:
                    !right
                    !chuckle
                    $angryCruel
                    Hah! Now you'll see, miscreant. No one interferes with the Ollowano family. 
                    $angryCruel
                    For what it's worth, I plan on having your skull preserved as a trophy. 
                    !exit
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            if not cxt.quest:GetCastMember("sibling"):IsDead() then
                cxt:Dialog("DIALOG_SIBLING_PRESENT_INTRO")
            end
            StateGraphUtil.AddEndOption(cxt)
        end)


QDEF:AddConvo("kill_other_handler")
    :Loc{
        OPT_FACE_OOLO = "Face {other_handler}",
    }
    :Hub_Location( function(cxt) 
        if cxt.quest:IsActive("kill_other_handler") and cxt.location == cxt.quest:GetCastMember("safehouse") then
            cxt:Opt("OPT_FACE_OOLO")
                --:Dialog("DIALOG_LEAVE")
                :Fn(function(cxt) 
                    TheGame:FE():FadeTransition(function() 
                        cxt.caravan:MoveToLocation(cxt.quest:GetCastMember("safehouse_ext") )
                    end)
                end)
                :End()
        end
    end)

    :ConfrontState("THE_STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("safehouse_ext") end)
        :Loc{
            DIALOG_PRE_ARRIVE = [[
                * You exit the safehouse, ready to face {other_handler}.
                player:
                    !left
                * {other_handler} cuts a daunting figure, perched atop a fully-equipped battle oshnu. You've never seen a snail that frightening.
            ]],
            
            DIALOG_OOLO_SIBLING_DEAD = [[
                other_handler:
                    !right
                    !angry
                    $angrySeething
                    You have no idea what kind of grave you've dug for yourself, grifter. 
                    !angry_accuse
                    $angrySeething
                    My family's gonna have a lot of demands for what we do with you. But I'll just keep it short and simple: you're dead.
            ]],
            DIALOG_OOLO_SIBLING_ALIVE = [[
                other_handler:
                    !right
                    !cruel
                    $angryCruel
                    This was a fun joke, grifter. But all you got for yourself was a day in the company of my least interesting sibling. 
                    !angry_shrug
                    $neutralResigned
                    Hope it was worth it. Because this'll be a sorry way to die.
            ]],
            DIALOG_FIGHT = [[
                player:
                    !fight
                    $angryPatienceLost
                    Yeah, yeah. I have my own score to settle. So let's just get this over with. 
            ]],
            DIALOG_WON_OOLO_DEAD_SIBLING_DEAD = [[
                * Both {other_handler} and {sibling} are dead by your hand. You hope that the rest of the dynasty doesn't come looking for revenge.
            ]],
            DIALOG_WON_OOLO_ALIVE_SIBLING_DEAD = [[
                other_handler:
                    !injured
                    $angryDefault
                    So this is it, huh? Bested by some two-bit thug.
                player:
                    !crossed
                    $neutralWhatever
                    I'd like to take full credit, but this is a bit bigger than my own needs.
                    $miscMocking
                    Let's just say {handler} sends his regards.
                other_handler:
                    !injured
                    $neutralResigned
                    Ah. Hah. Surprised he didn't wanna do the job himself.
                    $miscMocking
                    But {handler}'s big secret is that he always was a bit sentimental. You can tell him I said that.
            ]],

            OPT_FINISH_OLOO_OFF = "Finish {other_handler} off",
            DIALOG_FINISH_OLOO_OFF = [[
                player:
                    That's enough for final words. Goodbye, what's-your-name. 
                other_handler:
                    !surprised
                    !exit
                * With {other_handler} dead at your feet, you sheathe your knives. Time to report back to {handler}.
            ]],

            OPT_SPARE_OLOO = "Spare {other_handler}",
            DIALOG_SPARE_OLOO_SIBLING_DEAD = [[
                player:
                    !thought
                    $neutralThoughtful
                    Actually... I changed my mind.
                other_handler:
                    !injured
                    $angryTakenAback
                    What? You just killed my brother and beat me half to death... and now you're letting me go?
                    Is this some long con of {handler}'s?
                player:
                    !shrug
                    $neutralWhatever
                    Maybe. Or maybe I just don't appreciate being the tool he uses for this particular job.
                    I don't know you, {other_handler}. You never did wrong by me. 
                    !bashful
                    $miscEmbarrassed
                    Besides, I—I dunno. You remind me of someone. Maybe killing you would just bug me. 
                other_handler:
                    Fool notion to let me go, grifter. You've done wrong by me. 
                player:
                    !happy
                    $miscMocking
                    Maybe. But I'm not scared of you. Run along now, champ. I'll handle {handler}.
                other_handler:
                    !injured_shrug
                    $angryCruel
                    Something tells me I won't need to wait long for you to get yours. 
                    !exit
                * {other_handler} skulks off. 
            ]],

            DIALOG_WON_OOLO_DEAD_SIBLING_ALIVE = [[
                * {other_handler}'s corpse lies in the dirt.
                * You hear a scuffle behind you. {sibling} has struggled from {sibling.hisher} bonds and emerged from the safehouse.
                sibling:
                    !right
                    !scared
                    {other_handler}! No!
                player:
                    !palm
                    $angryReluctant
                    You really should have stayed in there, {sibling}.
                sibling:
                    !placate
                    $scaredDefault
                    Hold on, grifter. I can see I was just a pawn. You got what you wanted out of {other_handler}, didn't you?
                    $scaredPowerless
                    I—I can pay. I'm the favorite; my mother will want to see me safely returned. But nobody will care about {other_handler}.
                    !interest
                    What's more, I can promise you our family won't seek retribution. 
                player:
                    Hm.
            ]],
            OPT_FINISH_OFF_SIBLING = "Finish off {sibling}",
            DIALOG_FINISH_OFF_SIBLING = [[
                player:
                    !cruel
                    $angryCruel
                    I've never had an entire kra'deshi dynasty after me. Sounds kinda fun.
                sibling:
                    !scared
                    $scaredPowerless
                    B-but!
                player:
                    $miscMocking
                    I'd say it's nothing personal but... I did spend time with you. 
                sibling:
                    !exit
                * {sibling} goes down easily. Clearly {other_handler} inherited all the cunning in that family. 
            ]],

            OPT_LET_SIBLING_GO = "Let {sibling} go",
            DIALOG_LET_SIBLING_GO = [[
                player:
                    !handwave
                    $neutralWhatever
                    Alright, get out of here. You're not what {handler} wanted anyway.
                    I want that money though, {sibling}. I caught you once, I can catch you again. 
                sibling:
                    !exit
                * {sibling} turns around. After much effort, and some rather unpleasant noises, {sibling} produces a fistful of slimy shills.
                sibling:
                    !right
                    !give
                player:
                    !take
                sibling:
                    !bashful
                    $miscEmbarrassed
                    I, uh, don't suppose you know the way back to civilization? The Admiralty headquarters, perhaps?
                player:
                    !angry
                    ...
                sibling:
                    !scared
                * With nothing but a steely stare in response, {sibling} gets the hint and disappears into the wilderness.
                    I'll be going now.
                    !exit
            ]],
            
            DIALOG_WON_OOLO_ALIVE_SIBLING_ALIVE = [[
                player:
                    !crossed
                    $neutralDubious
                    This is {handler}'s vendetta, not mine. You got a better offer?
                other_handler:
                    $neutralDirect
                    I'm an officer in the Admiralty. What do you think? 
                    !overthere
                    $miscPersuasive
                    Let me and {sibling} go. We'll leave town and never come back. In exchange, I can teach you some Admiralty tricks.
                    $miscAlluring
                    Ones you'd never learn on your own, no matter who you charmed. What do you say?
                *** {other_handler} is offering you a special Admiralty card if you let {other_handler.himher} escape. You'll have to smooth things over with {handler}, of course.
            ]],

            OPT_TAKE_OOLO_DEAL = "Exchange {other_handler}'s life for an Admiralty faction card",
            DIALOG_TAKE_OOLO_DEAL = [[
                player:
                    !agree
                    Deal.
            ]],
            DIALOG_TAKE_OOLO_DEAL_2 = [[
                other_handler:
                    !interest
                    There. This is the kind of information that it takes a lifetime to earn.
                player:
                    !happy
                    $happyDefault
                    I've always been a fan of shortcuts. Now, take your brother and get outta here. 
                other_handler:
                    !disappoint
                    $miscDepressed
                    Too bad you didn't come to me from the start, hunter. I could've used someone pragmatic like you.
                    Next time, pick better friends than people like {handler}.
                player:
                    !dubious
                    $neutralDubious
                    I'll pick my friends as I like 'em, thanks.
                    !exit
                * {other_handler} gathers {sibling} and they both escape into the wilderness. You're going to have to smooth things over with {handler}.
            ]],
        OPT_FINISH_OFF_OLOO_AND_SIBLING = "Finish off {other_handler} and {sibling}",
        DIALOG_FINISH_OFF_OLOO_AND_SIBLING = [[ 
            player:
                $angryCruel
                !cruel
                Sorry, {other_handler}, but you've failed to tempt me. 
            other_handler:
                !exit
            * It's dirty, brutal work, but you finish off your targets. All that's left is your report to {handler}.
        ]],
        CARD_PICK_TITLE = "Admiralty Cards",
        CARD_PICK_SUBTITLE = "You can pick 1",


        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_PRE_ARRIVE")
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("other_handler"))
            if cxt.quest:GetCastMember("sibling"):IsDead() then
                cxt:Opinion("killed_brother", "other_handler")
                cxt:Dialog("DIALOG_OOLO_SIBLING_DEAD")
            else
                cxt:Dialog("DIALOG_OOLO_SIBLING_ALIVE")
            end

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                    allies = {cxt.quest.param.spree_staying},
                    
                    flags = BATTLE_FLAGS.BOSS_FIGHT | BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function() 
                        
                        cxt.quest.param.spree_allies_alive = {}

                        if cxt.quest.param.spree_staying and not cxt.quest.param.spree_staying:IsDead() then
                            table.insert( cxt.quest.param.spree_allies_alive, cxt.quest.param.spree_staying)
                        end

                        if cxt.quest.param.hireling and not cxt.quest.param.hireling:IsDead() then
                            table.insert( cxt.quest.param.spree_allies_alive, cxt.quest.param.hireling)
                        end 



                        if cxt.quest:GetCastMember("sibling"):IsDead() then
                            
                            if cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_WON_OOLO_DEAD_SIBLING_DEAD")
                                cxt.quest:Complete("kill_other_handler")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            else
                                cxt:Dialog("DIALOG_WON_OOLO_ALIVE_SIBLING_DEAD")
                                
                                cxt:Opt("OPT_FINISH_OLOO_OFF")
                                    :Dialog("DIALOG_FINISH_OLOO_OFF")
                                    :Fn(function() 
                                        local loot = {money = 0, items = {}}
                                        AgentUtil.CollectDeathLoot(cxt.quest:GetCastMember("other_handler"), loot)
                                        cxt.quest:GetCastMember("other_handler"):Kill()
                                        AgentUtil.CommitMurder(cxt.quest:GetCastMember("other_handler"))
                                        cxt.enc:AwardLoot(loot, { found=true } )
                                    end)
                                    :CompleteQuest("kill_other_handler")
                                    :Travel()

                                cxt:Opt("OPT_SPARE_OLOO")
                                    :Dialog("DIALOG_SPARE_OLOO_SIBLING_DEAD")
                                    :Fn(function() 
                                        cxt.quest:Cancel("kill_other_handler")
                                        cxt.quest:Activate("explain_handler_escape")
                                        cxt.quest:GetCastMember("other_handler"):Retire()

                                        if #cxt.quest.param.spree_allies_alive == 0 then
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        else
                                            cxt:GoTo("STATE_DEAL_WITH_SPREE_WITNESSES")
                                        end
                                    end)
                            end
                        else
                            if cxt:GetAgent():IsDead() then

                                cxt:Opinion("killed_sister", "sibling")
                                cxt:Dialog("DIALOG_WON_OOLO_DEAD_SIBLING_ALIVE")
                                cxt.quest:Complete("kill_other_handler")

                                cxt:Opt("OPT_LET_SIBLING_GO")
                                    :ReceiveMoney(300)
                                    :Dialog("DIALOG_LET_SIBLING_GO")
                                    :Fn(function() 
                                        cxt.quest:GetCastMember("sibling"):OpinionEvent(OPINION.LET_LEAVE_TOWN)
                                        cxt.quest:GetCastMember("sibling"):Retire()
                                    end)
                                    :Travel()

                                cxt:Opt("OPT_FINISH_OFF_SIBLING")
                                    :Dialog("DIALOG_FINISH_OFF_SIBLING")
                                    :Fn(function() 
                                        local loot = {money = 0, items = {}}
                                        AgentUtil.CollectDeathLoot(cxt.quest:GetCastMember("sibling"), loot)
                                        cxt.quest:GetCastMember("sibling"):Kill()
                                        AgentUtil.CommitMurder(cxt.quest:GetCastMember("sibling"))                                        
                                        cxt.enc:AwardLoot(loot, { found=true } )
                                    end)
                                    :Travel()

                            else
                                cxt:Dialog("DIALOG_WON_OOLO_ALIVE_SIBLING_ALIVE")

                                cxt:Opt("OPT_TAKE_OOLO_DEAL")
                                    :Dialog("DIALOG_TAKE_OOLO_DEAL")
                                    :Fn(function() cxt.enc:OfferCards(GetUniqueFactionCards(cxt.player, "ADMIRALTY", 0, 3), cxt:GetLocString("CARD_PICK_TITLE"), cxt:GetLocString("CARD_PICK_SUBTITLE")) end)
                                    :Dialog("DIALOG_TAKE_OOLO_DEAL_2")
                                    :Fn(function()
                                        cxt.quest:Cancel("kill_other_handler")
                                        cxt.quest:Activate("explain_handler_escape")
                                        
                                        cxt.quest:GetCastMember("sibling"):OpinionEvent(OPINION.LET_LEAVE_TOWN)
                                        cxt.quest:GetCastMember("other_handler"):OpinionEvent(OPINION.LET_LEAVE_TOWN)

                                        cxt.quest:GetCastMember("sibling"):Retire()
                                        cxt.quest:GetCastMember("other_handler"):Retire()
                                        if #cxt.quest.param.spree_allies_alive == 0 then
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        else
                                            cxt:GoTo("STATE_DEAL_WITH_SPREE_WITNESSES")
                                        end
                                    end)


                                cxt:Opt("OPT_FINISH_OFF_OLOO_AND_SIBLING")
                                    :Dialog("DIALOG_FINISH_OFF_OLOO_AND_SIBLING")
                                    :Fn(function() 
                                        local loot = {money = 0, items = {}}
                                        AgentUtil.CollectDeathLoot(cxt.quest:GetCastMember("sibling"), loot)
                                        AgentUtil.CollectDeathLoot(cxt.quest:GetCastMember("other_handler"), loot)
                                        cxt.quest:GetCastMember("other_handler"):Kill()
                                        cxt.quest:GetCastMember("sibling"):Kill()
                                        AgentUtil.CommitMurder(cxt.quest:GetCastMember("other_handler"))                                        
                                        AgentUtil.CommitMurder(cxt.quest:GetCastMember("sibling"))                                        
                                        cxt.enc:AwardLoot(loot, { found=true } )
                                        cxt.quest:Complete("kill_other_handler")
                                    end)
                                    :Travel()

                            end
                        end

                    end,

                }
             --cxt.quest.param.spree_staying

        end)
    :State("STATE_DEAL_WITH_SPREE_WITNESSES")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !right
                    Hey, what's the deal? {handler} is not going to like this!
                player:
                    !left
            ]],
            OPT_BUY_SILENCE = "Pay {agent} to look the other way",
            DIALOG_BUY_SILENCE = [[
                player:
                    They just got away, right. There was nothing we could do.
                    !give
                agent:
                    !take
                    Alright. I'll keep quiet.
                    !exit
                * {agent} leaves, probably to go spend {agent.hisher} newfound wealth.
            ]],

            OPT_CONVINCE_SILENCE = "Convince {agent} to shut {agent.hisher} mouth",
            DIALOG_CONVINCE_SILENCE = [[
                player:
                    I suggest you close your mouth, if you know what's good for you.
            ]],

            DIALOG_CONVINCE_FAIL = [[
                agent:
                    !angry
                    How about I close your <b>face</> instead, traitor?
            ]],

            DIALOG_CONVINCE_SUCCESS = [[
                agent:
                    Never mind! I'll shut up.
                    !exit
                * {agent} leaves.
            ]],
            
            OPT_ATTACK_WITNESSES = "Attack {agent}",
            OPT_DEFEND = "Defend yourself!",

            DIALOG_DEFEND = [[
                player:
                    !fight
                    Stupid Spree!
            ]],
            DIALOG_ATTACK = [[
                player:
                    !fight
                    You're right. So I need to cover my tracks!
            ]],

            DIALOG_POST_FIGHT = [[
                agent:
                    !exit
                * You've made your point. Time to talk to {handler}.
            ]],

        }

        :Fn(function(cxt)
                cxt.enc:SetPrimaryCast(cxt.quest.param.spree_allies_alive[1])
                cxt:Dialog("DIALOG_INTRO")
                CleanUpFn(cxt.quest)

                cxt:Opt("OPT_BUY_SILENCE")
                    :Dialog("DIALOG_BUY_SILENCE")
                    :DeliverMoney(100*#cxt.quest.param.spree_allies_alive)
                    :Travel()
                
                cxt:Opt("OPT_CONVINCE_SILENCE")
                    :Dialog("DIALOG_CONVINCE_SILENCE")
                    :Negotiation{
                        on_success = function(cxt) 
                            cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                            
                            StateGraphUtil.AddLeaveLocation(cxt)
                            
                        end,
                        on_fail = function(cxt) 

                            cxt:Dialog("DIALOG_CONVINCE_FAIL")
                            cxt:Opt("OPT_DEFEND")
                                :Battle{
                                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                                    enemies = cxt.quest.param.spree_allies_alive,
                                    on_win = function(cxt) 
                                        cxt:Dialog("DIALOG_POST_FIGHT")
                                        StateGraphUtil.AddLeaveLocation(cxt)
                                    end,
                                }
                        end,
                    }
                
                cxt:Opt("OPT_ATTACK_WITNESSES")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{
                        enemies = cxt.quest.param.spree_allies_alive,
                        on_win = function(cxt) 
                            cxt:Dialog("DIALOG_POST_FIGHT")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                    }
        end)
    

QDEF:AddConvo("compromise_guard")
    :ConfrontState("THE_STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("beach") end)
    :Loc{
        DIALOG_INTRO = [[
            * You arrive at the beach to find two Admiralty guards, their heads hung in boredom.
            guard_1:
                !right
                !spit
                $neutralResigned
                When is this Heshian brat supposed to show up, anyway?
            guard_2:
                !left
                $neutralWhatever
                Should be soon, shouldn't it? High tides don't wait for nobody, Heshian or not.
            guard_1:
                !dubious
                $neutralDubious
                Ugh. I wanna get my plonk on. 
            guard_2:
                !crossed
                $neutralWhatever
                {other_handler} gave us our orders. And no way I'm gonna let you bail without me.

            * So far, you remain unnoticed.
        ]],
        DIALOG_INTRO_SINGLE = [[
            * You arrive at the beach, where you find a rather impatient Admiralty.
            guard_1:
                !right
                !spit
                $neutralResigned
                When is this Heshian brat supposed to show up, anyway?
                !crossed
                $neutralWhatever
                If I wasn't terrified of {other_handler}, I'd go get my plonk on.
            * So far, you remain unnoticed.
        ]],
        OPT_CONVINCE_TO_LEAVE = "Trick {guard_1} into leaving",
        OPT_ATTACK = "Attack the guards",

        DIALOG_TRICK_HAS_SPREE = [[
            player:
                !left
                !salute
                $neutralDirect
                Hail and well met, fellow Admiralty. I am here to, uh, relieve you. From your duty.
            agent:
                !angry
                $angryTakenAback
                With a damn <i>Spree</i> thug in tow? Guess {other_handler} was right to be careful—foul play <i>is</i> afoot. 
        ]],
        
        DIALOG_TRICK = [[
            player:
                !left
                !salute
                $neutralGreeting
                Ahoy. I'm your midnight relief, here to relieve.
            agent:  
                !dubious
                $neutralDubious
                Oh yeah? That seems too good to be true. 
            player:
                !wink
                $miscAlluring
                Sometimes the stars align, friend. I'm here on {other_handler}'s orders.
        ]],

        DIALOG_SUCCESS_TRICK = [[
            player:
                !interest
                $miscPersuasive
                {other_handler} thought an Admiralty guard would draw too much attention, so {other_handler.heshe} hired me. 
                I can guard the Heshian and get them where they need to go safely—all under the radar.
            agent:
                !happy
                $happyDefault
                Aha. That's clever. And that makes this our lucky day, don't it? 
                !chuckle
                Thanks, hunter—have fun babysitting.
                !exit
            * The guards leave. You breathe a sigh of relief.
            player:
                !cruel
                $neutralAside
                Lucky day indeed.
        ]],

        DIALOG_FAIL_TRICK = [[
            player:
                !forgetful
                $neutralDubious
                {other_handler} wanted me to do the guardin' on account of how you two can't tell your ears from your elbows.
            agent:
                !dubious
                $neutralDubious
                Oh yeah? That might <i>sound</i> like {other_handler}... but something's not right.
                $neutralDirect
                Just to play it safe, how about we arrest you and let {other_handler} sort it all out in the end, huh?
            player:
                !placate
                $scaredPowerless
                Oh, no—no, you definitely don't want to—
            agent:
                !fight
                $angryDefault
                Seize her!
        ]],
        OPT_DEFEND = "Defend yourself!",
        DIALOG_ATTACK = [[
            player:
                !left
                !fight
                $happyCocky
                Aw, well. I'm not going down easy, you switchy bastards!
            agent:
                !fight
        ]],
        DIALOG_WON_FIGHT = [[
            agent:
                !exit
            player:
                !neutral
            * You've made a real mess of the beach. {sibling} is going to know something is up.    
        ]]

    }
    :Fn(function(cxt)
        
        if cxt.quest:IsActive("get_spree_help") then
            cxt.quest:Cancel("get_spree_help")
        end

        local opfor = CreateCombatParty("ADMIRALTY_PATROL", cxt.quest:GetRank()-1, cxt.location,nil,nil,1)

        cxt:ReassignCastMember( "guard_1", opfor[1] )
        cxt:ReassignCastMember( "guard_2", opfor[2] )
        cxt.enc:SetPrimaryCast(opfor[1])

        cxt:Dialog(opfor[2] and opfor[2]:IsSentient() and "DIALOG_INTRO" or "DIALOG_INTRO_SINGLE")
        cxt:Opt("OPT_CONVINCE_TO_LEAVE")
            :Fn(function()
                if cxt.quest.param.hireling and cxt.quest.param.hireling:IsInPlayerParty() then
                    cxt:Dialog("DIALOG_TRICK_HAS_SPREE")
                else
                    cxt:Dialog("DIALOG_TRICK")
                end
            end )
            :Negotiation{
                on_success = function()
                    cxt:Dialog("DIALOG_SUCCESS_TRICK")
                    for k,v in pairs( opfor ) do
                        v:MoveToLimbo()
                    end

                    cxt:GoTo("STATE_SIBLING_ARRIVE")
                end,

                on_fail = function()
                    cxt:Dialog("DIALOG_FAIL_TRICK")
                    cxt:Opt("OPT_DEFEND")
                        :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE,
                            on_win = function()
                                for k,v in pairs( opfor ) do
                                    if not v:IsDead() then
                                        v:MoveToLimbo()
                                    end
                                end
                                cxt:Dialog("DIALOG_WON_FIGHT")
                                cxt.quest.param.sibling_upset = true
                                cxt:GoTo("STATE_SIBLING_ARRIVE")
                            end,

                        }
                end,
            }

        cxt:Opt("OPT_ATTACK")
            :Dialog("DIALOG_ATTACK")
            :Battle{
                advantage = TEAM.BLUE,
                on_win = function()
                    cxt:Dialog("DIALOG_WON_FIGHT")
                    cxt.quest.param.sibling_upset = true
                    cxt:GoTo("STATE_SIBLING_ARRIVE")
                end,
            }
    end)   
    
    :State("STATE_SIBLING_ARRIVE")
        :Loc{
            DIALOG_INTRO = [[
                agent:
                    !exit
                * Out over the water, a light cuts its way through the fog.
                * Soon a small dinghy inserts itself on the shore, and out of it steps a kra'deshi dressed in flamboyant finery.
                sibling:
                    !right
            ]],

            DIALOG_INTRO_FOUGHT = [[
                * {sibling} quickly takes in the signs of combat, and {sibling.hisher} face creases in growing panic.
                * But when {sibling.heshe} spots you, {sibling.hisher} demeanor changes to reflect {sibling.hisher} pedigree.
                sibling:
                    !point
                    $neutralDirect
                    You there. What's the meaning of all this? I demand an explanation.
            ]],
            DIALOG_INTRO_DID_NOT_FIGHT = [[
                sibling:
                    !interest
                * {sibling} looks you over with open distaste. 
                sibling:
                    !dubious
                    $angryDefault
                    Who are you? Where's {other_handler}? 
                    {other_handler.HeShe} had clear instructions from mother to meet me here.
            ]],

            OPT_CONVINCE_TO_COME = "Convince {sibling} that you are {sibling.hisher} escort",
            OPT_THREATEN = "Demand {sibling} come with you",
            OPT_SUBDUE = "Subdue {sibling}",
            DIALOG_THREATEN = [[
                player:
                    !angry_accuse
                    $angryCruel
                    Yeaaaah, I got bad news. This is a kidnapping. So I'm gonna need you to shut up and come with me.  
                agent:
                    !angry
                    $angryTakenAback
                    W-what?! Is this some sort of joke?
            ]],

            DIALOG_THREATEN_SUCCESS = [[
                player:
                    !angry_accuse
                    $angrySeething
                    You wanna test me? Go on—see if Hesh saves you.
                sibling:
                    !scared
                    $scaredPowerless
                    I'll come! But don't expect a ransom to be paid if I'm hurt!
                    !exit
                * You take {sibling} into custody. Roughly.
            ]],
            DIALOG_THREATEN_FAIL = [[
                player:
                    !eureka
                    $angryCruel
                    Sure, it's a joke. The punchline is that I actually punch you.
                sibling:
                    !chuckle
                    $angryCruel
                    Pathetic! If you're the kind of common thug that proliferates Murder Bay, my sister should be transferred elsewhere.
                    If you lay a finger on me, she'll see you hanged—or someone will, at any rate.
            ]],
            DIALOG_CONVINCE = [[
                player:
                    !greeting
                    $miscPersuasive
                    I'm your guard, of course.
                sibling:
                    !crossed
                    $neutralDubious
                    Hrm. You don't <i>look</> like one of her tin soldiers.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    !overthere
                    $miscPersuasive
                    The bandits in the area have been keeping a close eye on the Admiralty. 
                    {other_handler} thought you'd be safer with off-the-books protection.
                sibling:
                    $neutralDirect
                    She <i>knows</i> I prefer to travel with fanfare.
                    !disappoint
                    $neutralResigned
                    But this cloak-and-dagger nonsense is very typical for {other_handler.himher}. Well, lead on.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    !overthere
                    $miscAlluring
                    I'm your personal escort, hired by {other_handler} herself to see you to safety. 
                sibling:
                    !angry
                    $angryDefault
                    Dressed like that? Whatever racket you're angling, I suggest you pick a new target.
                    !throatcut
                    You don't want to see the Abyss my sister will tip you into if you make trouble for our family. 
            ]],
            DIALOG_SUBDUE = [[
                sibling:
                    !scared
                player:
                    !angry_accuse
                    $angryCruel
                    You gonna come quiet? Or do I need to remind you where you've landed?
                sibling:
                    !cruel
                    $angryCruel
                    My geography has always been the best of the family. 
                    !angry_threaten
                    $angryHostile
                    Perhaps you're the one who needs to be taught a lesson, hmm?
            ]],
            DIALOG_KILLED_SIBLING = [[
                * You killed {sibling}, but you leave the evidence where it is in hopes of salvaging the mission. 
                * With any luck, {other_handler} will still come to you—if only for revenge. 
            ]],

            DIALOG_CAPTURE_SIBLING = [[
                player:
                    !fight
                    Lucky for you, I need you alive. But alive doesn't mean the same as "not dismembered", so keep quiet.
                sibling:    
                    !injured
                    ...
                    I-I'm cooperating. You win. 
                * You shackle {sibling} and {sibling.heshe} falls in line behind you.

            ]]
        }
        :SetLooping()
        :Fn(function(cxt) 
            if cxt:FirstLoop() then
                cxt.quest:Complete("compromise_guard")
                cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("sibling"))
                cxt:Dialog("DIALOG_INTRO")
                cxt.quest:GetCastMember("sibling"):MoveToLocation(cxt.location)
                cxt:Dialog(cxt.quest.param.sibling_upset and "DIALOG_INTRO_FOUGHT" or "DIALOG_INTRO_DID_NOT_FIGHT")        
            end

            if cxt.quest.param.sibling_upset then
                cxt:Opt("OPT_THREATEN")
                    :Dialog("DIALOG_THREATEN")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        on_success = function(cxt)
                            cxt:Dialog("DIALOG_THREATEN_SUCCESS")
                            cxt.quest:GetCastAgent("sibling"):Recruit(PARTY_MEMBER_TYPE.CAPTIVE)
                            cxt.quest:Activate("take_sibling_to_safehouse")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt) 
                            cxt:Dialog("DIALOG_THREATEN_FAIL")
                        end,
                    }
            else
                cxt:Opt("OPT_CONVINCE_TO_COME")
                    :Dialog("DIALOG_CONVINCE")
                    :Negotiation{
                        on_success = function(cxt) 
                            cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                            cxt.quest:GetCastAgent("sibling"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            cxt.quest:Activate("take_sibling_to_safehouse")
                            StateGraphUtil.AddLeaveLocation(cxt)
                        end,
                        on_fail = function(cxt) 
                            cxt:Dialog("DIALOG_CONVINCE_FAIL")
                        end,
                    }
            end

            cxt:Opt("OPT_SUBDUE")
                :Dialog("DIALOG_SUBDUE")
                :Battle{
                    on_start_battle = function(battle)
                        local fighter = battle:GetFighterForAgent(cxt.quest:GetCastMember("sibling"))
                        if fighter then
                            fighter:AddCondition("WANTED_ALIVE")
                        end
                    end,

                    on_win = function(cxt)
                        if cxt:GetAgent():IsDead() then
                            cxt:Dialog("DIALOG_KILLED_SIBLING")
                        else
                            cxt:Dialog("DIALOG_CAPTURE_SIBLING")
                            cxt.quest:GetCastAgent("sibling"):Recruit(PARTY_MEMBER_TYPE.CAPTIVE)
                        end
                        cxt.quest:Activate("take_sibling_to_safehouse")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }
        end)

QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                !happy
            player:
                !dubious
                $neutralDubious
                What's up, {handler}? You look... happy? Is that your happy face?
            agent:
                !interest
                You know {other_handler}? 
            player:
                !agree
                $neutralJoke
                Name rings a bell. Pretty sure you forbade me from saying it.
                She's an Admiralty officer, right?
            agent:
                !handwring
                That's her. Kra'deshi. Slippery as Hesh itself. Only one person living I hate as much as {kashio}—and that's {other_handler}. 
                But like most of her kind, she's got a big family, lots of obligations. 
                !permit
                Turns out one of her brothers is in the Cult and {sibling.heshe}'s coming to town for the auction.
                !cruel
                You see where I'm going with this? I've got a big barbed hook—all I need is the worm. 
        ]],
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)


 
QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_ON_TAKEN = [[
            handler:
                !overthere
                $neutralDirect
                {sibling}, Oolo's kin, is arriving at the beach tonight. {sibling.HeShe}'s hired private protection. 
                Go insinuate yourself, then set the trap. Once you're in position, we can send the "ransom note".
                !point
                $angryDefault
                Only there's not gonna be no ransom. You'll kill {other_handler} the second she shows her face.
                $miscAlluring
                Do that, and I promise you, you've proven yourself to me. I'll get you into that auction on my word. 
            *** Nadan wants you to capture {sibling} to lure {other_handler} into a trap.
        ]],
        DIALOG_TELL_ABOUT_HELPER = [[
            handler:
                !hips
                $neutralDirect
                Alright. If you need a little extra muscle, feel free to bring one of the Spree.
                Just know that you won't be able to talk your way through any Admiralty with a Spree in tow.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_ON_TAKEN")

            cxt:Opt("OPT_ASK_ABOUT")
               :IsHubOption(true)
               :GoTo("STATE_QUESTIONS")     

            StateGraphUtil.AddEndOption(cxt)
                :Dialog("DIALOG_TELL_ABOUT_HELPER")
        end)

    :AskAboutHub("STATE_QUESTIONS", 
             {

               "Ask about {handler}",
               [[
                    player:
                        !interest
                        $neutralDubious
                        All this just for revenge on one person?
                    agent:
                        !angryAside
                        Bit rich comin' from you, ain't it? I thought you and I were on the same level.
                    player:
                        !shrug
                        $neutralResigned
                        Fair enough. But is there really no easier way to get close to {other_handler}?
                    agent:
                        !spit 
                        $angryDefault
                        I'd recount the ways I'd tried, but that'd just make my cheeks turn red.
                        $neutralResigned 
                        She has a knack for leaping from the pot the moment you turn on the heat. She's smart, at least—I'll give her that. 
                        !cruel
                        $angryCruel
                        But family politics make people like her easy to manipulate, if you find the right string and give it a good tug.
                ]],
                "Ask about {other_handler}",
                [[
                    player:
                        !interest
                        Why do you have it out for {other_handler}, anyway?
                    agent:
                        !shrug
                        $neutralResigned
                         We've been deadlocked since she was a rookie. I robbed her once. She arrested me once. We got over it, made our own waves. 
                         !agree
                         She never came to heel like a lot of other Admiralty would—never took a bribe. So I could respect that.
                    player:
                        $neutralDubious
                        So... then what?
                    agent:
                        !crossed
                        $neutralDirect
                        I told you {kashio} sold me out. {other_handler}'s the one who was there to profit. 
                        I was carrying explosive contraband. She set a trap to light it up. 
                        $angrySeething
                        !thumb
                        Only way she could've known was if she took the tip from Kashio. 
                    player:
                        !dubious
                        $neutralTakenAback
                        She <i>blew you up</i>? Okay, maybe that warrants a heap or two of revenge.
                    agent:
                        !angry_shrug
                        $miscMocking
                        Well, now that I have your <i>approval</i>, do you wanna stop yabbering and get to work?
                ]]
             })  