local QDEF = QuestDef.Define
{
    title = "The Grog n' Dog",
    icon = engine.asset.Texture("icons/quests/sal_story_act1_living_at_grog.tex"),
    home = true, -- If this shows a yellow-colored icon, with a yellow arrow under it
    qtype = QTYPE.STORY,
    --desc = [[This is your home base]],
    events = {
        phase_change = function(quest)
            if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY then
                quest.param.fssh_cake_available = true
            end
        end,
    },
    on_init = function (quest )
        quest.param.fssh_cake_available = true
    end

}

:AddLocationCast{
    cast_id = "home",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GROG_N_DOG"))
    end,
}

:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}

:Loc{
    DRINKS_AVAILABLE = "Fssh has drinks available for sale",
    NO_CAKES = "Buy drinks to restore resolve.\n<#PENALTY>Fssh can not bake healing items.</>",
    YES_CAKES = "Buy drinks to restore resolve.\nHealing items available from Fssh.",
    BUY_DRINKS = "Buy drinks to restore resolve.\n<#PENALTY>Fssh is out of healing items for the day.</>",
}

:AddObjective{
    id = "get_healing",
    title = "Get Help From {fssh}",
    desc = "",
    desc_fn = function(quest, str) 
        local tt = { quest:GetLocalizedStr( "DRINKS_AVAILABLE" ) }
        if quest.param.no_cakes then
            return quest:GetLocalizedStr( "NO_CAKES" )
        elseif quest.param.fssh_cake_available then
            return quest:GetLocalizedStr( "YES_CAKES" )
        else
            return quest:GetLocalizedStr( "BUY_DRINKS" )            
        end
    end,
    hide_in_overlay = true,
    low_priority = true,
    state = QSTATUS.ACTIVE,
    mark = {"fssh"}
}

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
QDEF:AddConvo("get_healing", "fssh")
    :Priority(CONVO_PRIORITY_LOW)

    :Loc{
        OPT_BUY_FOOD = "Eat one of Fssh's famous Fsshcakes",

        DIALOG_BUY_FOOD = [[
            player:
                $neutralResigned
                I'm hurt, Fssh.
            agent:
                !give
                $happyGreeting
                Here. Have a Fsshcake.
            player:
                !take
                $happyThanks
                Thanks.
        ]],

        DIALOG_PST = [[
            player:
                $neutralTakenAback
                Ugh, that's terrible.
            agent:
                $neutralResigned
                Nobody eats them for the taste, that's for sure.
        ]],
        --REQ_ALREADY_ATE_ONE = "You already ate a Fsshcake today!",
        REQ_NO_CAKES = "Fssh can't make cakes right now!",

        REQ_TOO_FULL = "You have too many {1#card} cards in your deck already!",

        TT_CAKES = "Restore {1} health.\nInsert {2#card} to your battle deck.",

        OPT_CHANGE_OUTFIT = "Go to your room",

        DIALOG_CHANGE_OUTFIT = [[
            player:
                !point
                I'll just pop into my room.
            agent:
                !handwave
                Fine.
                !exit
        ]],

        TT_OUTFIT = "Change into another of {player}'s outfits.",
        TT_PET = "Play with {1#agent}.",
        TT_PETS = "Play with your pets.",
        TT_OUTFITPET = "Change into another of {player}'s outfits or play with {1#agent}.",
        TT_OUTFITPETS = "Change into another of {player}'s outfits or play with your pets.",
    }

    :Hub( function(cxt, who)

            cxt:Opt("OPT_BUY_FOOD")
                :PreIcon( cxt.quest:GetMarkData() )
                :ReqCondition(not cxt.quest.param.no_cakes, "REQ_NO_CAKES")
                :Dialog( "DIALOG_BUY_FOOD" )
                :DoEat(FSSHCAKE_COST, FSSHCAKE_HEAL)
                :Dialog("DIALOG_PST")
            
            if not TheGame:GetGameState():GetMainQuest().param.restrict_room then

                local unlocked_outfits = 1
                for k,v in ipairs( Content.GetOutfitsForCharacter(cxt.player.id) ) do
                    if v.unlocked or TheGame:GetGameProfile():HasUnlock( v.id ) then
                        unlocked_outfits = unlocked_outfits + 1
                    end
                end
                local num_pets = #cxt.caravan:GetPets()
                local tt = ""

                if unlocked_outfits > 1 and num_pets == 1 then
                    tt = loc.format( cxt:GetLocString( "TT_OUTFITPET" ), cxt.caravan:GetPets()[1] )
                elseif unlocked_outfits > 1 and num_pets > 1 then
                    tt = loc.format( cxt:GetLocString( "TT_OUTFITPETS" ) )
                elseif unlocked_outfits > 1 then
                    tt = cxt:GetLocString( "TT_OUTFIT" )
                elseif num_pets == 1 then
                    tt = loc.format( cxt:GetLocString( "TT_PETS" ) )
                elseif num_pets > 0 then
                    tt = loc.format( cxt:GetLocString( "TT_PET" ), cxt.caravan:GetPets()[1] )
                end
                if unlocked_outfits > 1 or num_pets > 0 then
                    cxt:Opt("OPT_CHANGE_OUTFIT")
                        :PreIcon( global_images.backpack )
                        :PostText( tt )
                        :Dialog( "DIALOG_CHANGE_OUTFIT" )
                        :Fn(
                            function()
                                cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("GROG_N_DOG.inn_room") )
                                cxt:End()
                            end
                        )
                end
            end
                
        end
    )


