local CleanUpFn = function(quest) 
    if quest:GetCastMember("night_merchant"):IsInPlayerParty() then
        quest:GetCastMember("night_merchant"):Dismiss()
    end

    if quest:IsFailed() and not quest:GetCastMember("night_merchant"):IsDead() then
        quest:GetCastMember("night_merchant"):OpinionEvent(OPINION.JOB_FAILED_MINOR, { quest = quest})
    end
end

local QDEF = QuestDef.Define
{
    title = "Blood from Stone",
    desc = [[{night_merchant} needs some help getting a loan extension.]],
    icon = engine.asset.Texture("icons/quests/sal_story_merchant_combat.tex"),
    qtype = QTYPE.STORY,
    block_side_jobs = true,    
    events = 
    {
        caravan_member_remove = function(quest, agent) 
            if agent == quest:GetCastMember("night_merchant") then
                quest:Fail()
            end
        end,
    },
    on_fail = CleanUpFn,
    on_complete = CleanUpFn,
    on_cancel = CleanUpFn,

    collect_agent_locations = function(quest, t)
        if quest:IsActive("offer_job") then
            table.insert (t, { agent = quest:GetCastMember("night_merchant"), location = TheGame:GetGameState():GetLocation("GROG_N_DOG"), role = CHARACTER_ROLES.VISITOR } )
        end
    end,

}

:AddCastByAlias{
    cast_id = "night_merchant",
    alias = "RAKE",
    provider = true,
    no_validation = true,
    no_cleanup = true,
    events = 
    {
        agent_retired = function(quest) 
            quest:Fail()
        end
    }
}

:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}

:AddLocationCast{
    cast_id = "market",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MARKET_STALL"))
    end,
}

:AddCast{
    cast_id = "the_meet_point",
    when = QWHEN.MANUAL,
    no_validation = true,
}

:AddCast{
    cast_id = "other_merchant",
    when = QWHEN.MANUAL,
}

:AddObjective{
    id = "offer_job",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    
    on_activate = function(quest)
        quest:GetCastMember("night_merchant"):MoveToLocation(TheGame:GetGameState():GetLocation("GROG_N_DOG"))
    end,

    mark = {"night_merchant"},
    on_complete = function(quest)
        quest:Activate("go_to_meet")
    end,
}

:AddObjective{
    id = "go_to_meet",
    title = "Go to the meeting",
    desc = "{night_merchant} needs you to convince {night_merchant.hisher} creditors to back off",
    
    mark = {"the_meet_point"},

    on_activate = function(quest)
        quest:GetCastMember("night_merchant"):Recruit( PARTY_MEMBER_TYPE.ESCORT )
        quest:SpawnTempLocation("MEET_BEACH", "the_meet_point" )
    end,

    on_complete = function(quest)
        quest:Activate("go_to_market")
    end,
    
}

:AddObjective{
    id = "go_to_market",
    title = "Take {night_merchant} to the Market",
    desc = "{night_merchant} has asked you to take {night_merchant.himher} to the night market.",
    
    mark = {"market"},

    on_activate = function(quest)
    end,
}


:AddLocationDefs{
    MEET_BEACH = {
        name = "The Beach",
        plax = "EXT_Beach_MurderBay_1",
        indoors = false,
        map_tags = {"beach"},
    },

}
---------------These are shown between jobs. maybe fssh should tell you more stuff (optionally?) Or even give you things, if it feels arduous for the player to make the trek back to the grog


QDEF:AddConvo("offer_job", "night_merchant")
       :Loc{
            OPT_ASK_ABOUT_JOB = "Ask about special job",
            DIALOG_STRANGER_INTRO = [[
                player:
                    !permit
                    $neutralGreeting
                    Hey, stranger. {fssh} says you're looking for help.
                night_merchant:
                    !right
                    !bashful
                    $miscEmbarrassed
                    Guess I am. Y'see, normally I just punch my problems until they stop moving. But some problems are too big to punch. 
                player:
                    !interest
                    So what can I do for you?
                night_merchant:
                    I run a little combat training operation, but I need an extension on my debts, or I might have to close up shop.
                    !burp
                    If you help me buy a little more time, I'll take you to our secret market—you can get your hands on good... goods.
                    !neutral
                    
                *** {night_merchant} asks if you'll help them buy some time for their debt.
            ]],
            
            OPT_DISAGREE = "Not right now",
            DIALOG_DISAGREE = [[
                player:
                    !tired
                    Sorry, pal, but I'm kinda beat. First day on the continent and all that.
                    Good luck with your problem, though.
                night_merchant:
                    !dubious
                    Huh. Fssh made it sound like you could use the edge. Guess not. 
            ]],
            OPT_AGREE = "Accept the job",
            DIALOG_AGREE = [[
                player:
                    !agree
                    Sounds like we have a deal. Lead the way, and I'll do the talking.
                night_merchant:
                    !happy
                    Beauty! Thanks, hunter.
                other_merchant:
                    !right
                    %merchant_rejected

                * {night_merchant} joins you.
            ]],
        }
    
    :Hub(function(cxt) 
            cxt:Opt("OPT_ASK_ABOUT_JOB")
                :SetQuestMark()
                :LoopingFn(function(cxt)
                    if cxt:FirstLoop() then
                        cxt:Dialog("DIALOG_STRANGER_INTRO")
                    end
                    

                    cxt:Opt("OPT_AGREE")
                        :CompleteQuest("offer_job")
                        :PreIcon( global_images.accept )
                        :ReceiveOpinion( OPINION.DID_NOT_HELP, nil, cxt.quest:GetCastMember( "other_merchant" ))
                        :Dialog("DIALOG_AGREE")
                        :Fn(function() StateGraphUtil.AddEndOption(cxt) end )
            
                    cxt:Opt("OPT_DISAGREE")
                        :PreIcon( global_images.reject )
                        :Dialog("DIALOG_DISAGREE")
                        :Pop()

                    cxt:Opt("OPT_ASK_ABOUT")
                        :IsHubOption(true)
                        :GoTo("STATE_QUESTIONS")

                end)
        end)


    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask about debt",
        [[
            player:
                !dubious
                $neutralDubious
                How much do you owe, anyway?
            night_merchant:
                $neutralWhatever
                I've lost track, to be honest. I just try to make the minimum payments.
                !scared_shrug
                $neutralWhatever
                But it's never enough. Kashio just keeps cranking up the interest. If I had a head for numbers, I might be able to figure it out. 
            player:
                !left
                !surprised
                $neutralDubious
                Wait, the debt broker is <i>Kashio</i>?
            night_merchant:
                !agree
                Yeah—mine and every other sad sucker's in Murder Bay. I only ever talk to her proxy though. Why, you know her?
            player:
                !tired
                She, uh, has a reputation. Even back on the derrick.
            night_merchant:
                !overthere
                Then you know I'm in big trouble if I don't pay her back. 
        ]],

    })        




QDEF:AddConvo("go_to_meet", nil)
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("the_meet_point") end)
        :Loc{
            DIALOG_INTRO = [[
                * {night_merchant} leads you to a desolate beach.
                night_merchant:
                    !left
                    !overthere
                    Here's the place. Remember, we can't fight our way through this. Trust me—I've tried.
                * With that warning, you're joined by a merchant with a stiff collar and a sneer to match. 
                creditor: 
                    !right
                    I'm here for the money, {night_merchant}. Kashio's tired of excuses.
                player:
                    !left
                creditor:
                    !angry
                    And who the Hesh are you? Hired muscle?
                player:
                    !placate
                    Let's not be hasty.
                *** {creditor} is trying to collect money from {night_merchant}. Help {night_merchant} get an extension.                    
            ]],
            
            OPT_CONVINCE = "Convince {creditor} to extend the loan",
            DIALOG_CONVINCE = [[
                player:
                    {night_merchant} needs just a little more time.
            ]],
            DIALOG_CONVINCE_FAIL = [[
                player:
                    So that's why, err—you see, if you just—
                creditor:
                    <i>Enough</i>. I've heard every excuse there is, and you're wasting my time.
                    Sleep lightly, {night_merchant}. Our "collections" agents will be in touch—with interest added for this inconvenience.
                    !exit
                * {creditor} slinks into the night, leaving nothing but {creditor.hisher} threat behind.
                night_merchant:
                    !right
                    !scared
                player:
                    So, uh, that didn't go great, huh?
                night_merchant:
                    !angry
                    Huh. Ya think?!
                    !angry_accuse
                    Tell {fssh} {fssh.heshe} has terrible taste in goons. I need to go arm up. 
                    !exit
                 * If you had to guess, you'd say your invitation to the secret market has been revoked. Nothing for it but to head back home.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    A <i>living</i> customer is a paying customer, and that's what you want, isn't it?
                creditor:
                    Yeah, provided {night_merchant.heshe}'ll actually pay. 
                    !sigh
                    Fine. I'll give {night_merchant} another week.
                    !throatcut
                    But after that—
                * {creditor} slinks into the night, leaving nothing but {creditor.hisher} threat behind.
                night_merchant:
                    !right
                    !happy
                    A week! Hesh, that really saves my skin. You have no idea.
                player:
                    !happy
                    So we're satisfied?
                   night_merchant:
                       !eureka
                       Satisfied, relieved, ready to party—you name it. Let's get to the night market, huh? 
                       We have something to celebrate, you and I!
                *** You helped {night_merchant}, and now {night_merchant.heshe} will take you to the market. 
            ]],
            OPT_BEAT_UP = "Offer to beat {agent} up",
            DIALOG_BEAT_UP = [[
                player:
                    !fight
                * You reach for your blade. {night_merchant} stops you.
                night_merchant:
                    !right
                    !scared
                    I <b>told</> you, I've tried that.
                    Sure, we could beat this one up, but Kashio will send more.
                    I hired you for your words, not your knives.
                player:
                    !neutral
                creditor:
                    !right
                    !happy
                    Ha. Typical.
                    !neutral
                    Now where's the money?
                *** {night_merchant} stops you. Kashio will keep coming even if you resort to violence.
            ]],
            OPT_BACK_DOWN = "Back down",
            DIALOG_BACK_DOWN = [[
                player:
                     Look, I—I don't want nothing to do with this. Not if Kashio's the broker.
                creditor:
                    Yeah? Smart move. 
                    {night_merchant}, I'll deal with you <b>shortly</>.
                    !exit
                * {creditor} slinks into the night, leaving nothing but {creditor.hisher} threat behind.
                night_merchant:
                    !right
                player:
                    Sorry, {night_merchant}. It's complicated.
                night_merchant:
                    Oh <i>really</i>? Well thanks for telling me!
                    !angry_accuse
                    I need to go arm up. You can find your own way back to your hidey hole. 
                    !exit
                * If you had to guess, you'd say your invitation to the secret market has been revoked. Nothing for it but to head back home.
            ]],
            SIT_MOD = "It's about business",

        }
    :SetLooping()
    :Fn(function(cxt) 
            --we want someone we don't know
            if cxt:FirstLoop() then
                local creditor = TheGame:GetGameState():AddSkinnedAgent( "WEALTHY_MERCHANT" )
                cxt:ReassignCastMember("creditor", creditor)
                cxt.enc:SetPrimaryCast(creditor)
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_CONVINCE")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    situation_modifiers =
                    {
                        { value = 10, text = cxt:GetLocString("SIT_MOD") }
                    },
                    on_success = function() 
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest:Complete("go_to_meet")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    
                    on_fail = function()
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                        cxt.quest:Fail()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,

                }

            cxt:Opt("OPT_BEAT_UP")
                :ReqTimer( "DIALOG_BEAT_UP", 1 )
                :Dialog("DIALOG_BEAT_UP")
                :Fn( function( cxt )
                    cxt:GetAgent():Remember("DIALOG_BEAT_UP")
                end )

            cxt:Opt("OPT_BACK_DOWN")
                :Dialog("DIALOG_BACK_DOWN")
                :Fn(function() 
                    cxt.quest:Fail()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)

        end)


QDEF:AddConvo("go_to_market", "night_merchant")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == TheGame:GetGameState():GetLocation("MARKET_STALL") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !dubious
                    $miscMocking
                    This is it, huh? Pretty modest.
                night_merchant:
                    !right
                    !chuckle
                    $happyCocky
                    Hey, you want fancy, or you want cheap? This is where folks come to get an edge for a good price.
                    !agree
                    $happyThanks
                    And now, you can be one of those people. 
                ** {night_merchant} now likes you, and will offer you discounts in {night_merchant.hisher} shop.

            ]],
        }
    :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:GetCastMember("night_merchant"):OpinionEvent(OPINION.JOB_MINOR, { quest = cxt.quest })
            cxt.quest:Complete()
            StateGraphUtil.AddEndOption(cxt)
        end)



