local CleanUpFn = function(quest) 
    if quest:GetCastMember("night_merchant"):IsInPlayerParty() then
        quest:GetCastMember("night_merchant"):Dismiss()
    end

    if quest:IsFailed() and not quest:GetCastMember("night_merchant"):IsDead() then
        quest:GetCastMember("night_merchant"):OpinionEvent(OPINION.JOB_FAILED_MINOR, { quest = quest})
    end
end


local QDEF = QuestDef.Define
{
    title = "In Default",
    desc = [[ A customer is late on their payments to {night_merchant}. Time to make them pay up.]],
    icon = engine.asset.Texture("icons/quests/sal_story_merchant_graft.tex"),
    qtype = QTYPE.STORY,
    block_side_jobs = true,    
    events = 
    {
        caravan_member_remove = function(quest, agent) 
            if agent == quest:GetCastMember("night_merchant") then
                quest:Fail()
            end
        end,
    },


    on_fail = CleanUpFn,
    on_complete = CleanUpFn,
    on_cancel = CleanUpFn,

    collect_agent_locations = function(quest, t)

        if quest:IsActive("confront_target") then
            table.insert (t, { agent = quest:GetCastMember("target"), location = quest:GetCastMember("target"):GetHomeLocation(), role = CHARACTER_ROLES.PATRON } )
        end

        if quest:IsActive("offer_job") then
            table.insert (t, { agent = quest:GetCastMember("night_merchant"), location = TheGame:GetGameState():GetLocation("GROG_N_DOG"), role = CHARACTER_ROLES.VISITOR } )
        end

    end,

}

:AddCastByAlias{
    cast_id = "night_merchant",
    alias = "PLOCKA",
    provider = true,
    no_validation = true,
    no_cleanup = true,
    events = 
    {
        agent_retired = function(quest) 
            quest:Fail()
        end
    }
}

:AddOpinionEvents{

    extracted_a_graft =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Extracted a graft from them",
    },
    stood_up_for_them =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Bought them more time to pay off their graft",
    },
    refused_to_extract =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Refused to extract their graft",
    },
}


:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}

:AddCast{
    cast_id = "target",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():AddSkinnedAgent( "HEAVY_LABORER" ))
    end,
    when = QWHEN.MANUAL,
}

:AddCast{
    cast_id = "other_merchant",
    when = QWHEN.MANUAL,
}


:AddLocationCast{
    cast_id = "market",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MARKET_STALL"))
    end,
}


:AddObjective{
    id = "offer_job",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    
    on_activate = function(quest)
        quest:GetCastMember("night_merchant"):MoveToLocation(TheGame:GetGameState():GetLocation("GROG_N_DOG"))
    end,

    mark = {"night_merchant"},
    on_complete = function(quest)
        quest:Activate("confront_target")
    end,
}


:AddObjective{

    id = "confront_target",
    title = "Take {night_merchant} to confront {target}",
    desc = "{night_merchant} needs your help confronting {target}",
    
    mark = {"target"},

    on_activate = function(quest)
        quest:GetCastMember("night_merchant"):Recruit( PARTY_MEMBER_TYPE.ESCORT )
        quest:AssignCastMember("target")
        quest:GetCastMember("target"):GetBrain():MoveToHome()
        quest:GetCastMember("night_merchant"):MoveToLocation(TheGame:GetGameState():GetLocation("GROG_N_DOG"))
    end,

    on_complete = function(quest)
        quest:Activate("go_to_market")
    end,
}

:AddObjective{
    id = "go_to_market",
    title = "Take {night_merchant} to the Market",
    desc = "{night_merchant} has asked you to take them to the night market.",
    
    mark = {"market"},

    on_activate = function(quest)
    end,
}


---------------These are shown between jobs. maybe fssh should tell you more stuff (optionally?) Or even give you things, if it feels arduous for the player to make the trek back to the grog


QDEF:AddConvo("offer_job", "night_merchant")
       :Loc{
            OPT_ASK_ABOUT_JOB = "Ask about special job",
            DIALOG_STRANGER_INTRO = [[
                player:
                    !permit
                    {fssh} says you're looking for some hired muscle.
                * {night_merchant} looks you up and down. You feel like an oshnu on market day.
                night_merchant:
                    Yes, you'll do.
                player:
                    Wait, do for what? What is the actual job?
                night_merchant:
                    You just need to stand behind me and look intimidating. 
                    You can handle that, right?
                player:
                    Depends. What's standing in front of you?
                night_merchant:
                    Don't worry about that part.
                    You do this job, and I'll let you in to the night market.

                *** {night_merchant} asks if you'll be hired muscle tonight.
            ]],
            
            OPT_DISAGREE = "Not right now",
            DIALOG_DISAGREE = [[
                player:
                    !tired
                    Nah, something doesn't sound right about this.
                night_merchant:
                    !dubious
                    Now I'm not going to say you're wrong for suspecting something.
                    But you are wrong for passing up a good deal.


            ]],
            OPT_AGREE = "Accept the job",
            DIALOG_AGREE = [[
                player:
                    !agree
                    The details are fuzzy, but I'm in.
                night_merchant:
                    !happy
                    Excellent. Let's go - I'll lead the way.
                * {night_merchant} joins you.

                other_merchant:
                    !right
                    %merchant_rejected
            ]],
        }
    
    :Hub(function(cxt) 
            cxt:Opt("OPT_ASK_ABOUT_JOB")
                :SetQuestMark()
                :LoopingFn(function(cxt)
                    if cxt:FirstLoop() then
                        cxt:Dialog("DIALOG_STRANGER_INTRO")
                    end

                    cxt:Opt("OPT_AGREE")
                        :CompleteQuest("offer_job")
                        :PreIcon( global_images.accept )
                        :ReceiveOpinion( OPINION.DID_NOT_HELP, nil, cxt.quest:GetCastMember( "other_merchant" ))
                        :Dialog("DIALOG_AGREE")
                        :Fn(function() StateGraphUtil.AddEndOption(cxt) end )
            
                    cxt:Opt("OPT_DISAGREE")
                        :PreIcon( global_images.reject )
                        :Dialog("DIALOG_DISAGREE")
                        :Pop()

                    cxt:Opt("OPT_ASK_ABOUT")
                        :PreIcon(global_images.question)
                        :IsHubOption(true)
                        :GoTo("STATE_QUESTIONS")

                end)
        end)


    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask for details",
        [[
            player:
                !dubious
                $neutralDubious
                No, really, I need to know more about what I'm agreeing to.
            night_merchant:
                $neutralWhatever
                I'm hiring you to do work, not to ask annoying questions.
                You can handle it, or I wouldn't hire you.
                It gets you closer to your goal, and you'll earn my gratitude.
                Take it, or leave it, grifter.
        ]],

    })      


QDEF:AddConvo("confront_target", nil)
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("target"):GetLocation() end)
        :Loc{
            DIALOG_INTRO = [[
                * {night_merchant} opens the door to a nondescript hovel and ushers you through.
                night_merchant:
                    !left
                    Say nothing, {player}, until I tell you to say something.
                target: 
                    !right
                    !angry
                    I was just looking for you!
                    The graft you sold me... it stopped working!
                night_merchant:
                    !sigh
                    The graft I so graciously extended credit for you to buy?
                    I disabled it when you fell behind on your payments.
                    Again.
                target:
                    !flinch
                    I can pay! I have the money!
                night_merchant:
                    !disappoint
                    It's too late for that. I'm losing money just taking the time to talk to you.
                    !cruel
                    I'm going to need the graft back.
                target:
                    But it's - it's installed! They don't just come out unless you...
                    ...unless you cut them out. You can't make me do that! Please!
                target:
                    !neutral
                night_merchant:
                    !permit
                    I wouldn't dream of asking you to do that to yourself. 
                night_merchant:                    
                    Which is why I brought my friend here. 

                target:
                    !scared
                night_merchant:                                        
                    {player}, extract the graft from {target}, if you please.
                player:
                    !left   
                    !wince
                    Uh...
                *** {night_merchant} has asked you to forcefully remove a graft from {target}.
            ]],
            
            OPT_DO_IT = "Extract the graft",
            DIALOG_DO_IT = [[
                player:
                    I'm sorry, but this is my job.
                    Try not to move. I'll try to make this as painless as possible.
                target:
                    You monster!
                player:
                    !exit
                target:
                    !exit
                * You hack into {target}'s flesh, searching for the graft.
                * It takes a while.
                * You eventually find it, and dig it out. The last biomechanical tendril pulls out with a sickening pop.
                * {target} survives the process. Barely.
                player:
                    !left
                target:
                    !right
                    !injured
                    I won't forget this.
                    !exit
                night_merchant:
                    !right
                    !happy
                    Good job, {player}. I was worried for a moment that you might have a conscience.
                player:
                    !give
                    Ugh. Here. Take it.
                night_merchant:
                    Nah, you keep it. You <b>earned</> it.

            ]],
            
            OPT_TAKE_GRAFT = "Take {1#graft}",
            DIALOG_TAKE_GRAFT = [[
                player:
                    Ow. That tingles.
                night_merchant:
                    Ha! You could have at least washed it off first!
            ]],
            OPT_SKIP_GRAFT = "Skip the graft",
            
            DIALOG_SKIP = [[
                player:
                    I'll pass.
                night_merchant:
                    Make sense. You've seen where it's been.
            ]],

            DIALOG_GO_TO_MARKET = [[
                night_merchant:
                    And now, as promised, I'll show you the way to market.
            ]],


            OPT_CONVINCE_MERCHANT = "Convince {night_merchant} to reconsider",
            DIALOG_CONVINCE = [[
                night_merchant:
                    !right
                player:
                    Wait, surely you could give {target} another chance...
            ]],
            DIALOG_CONVINCE_FAIL = [[
                night_merchant:
                    !point
                    You're soft, grifter.
                    !angry
                    I'll come back later with someone else to get this job done.
                    !exit
                * {night_merchant} storms out.
                target:
                    !right
                    !happy
                    Thank you, friend! I don't know how I'll ever repay you!
                    All I can give you right now is my gratitude.
                player:
                    !shrug
                    Yeah, that's just as good as money.
                    I'll see you around.
            ]],
            DIALOG_CONVINCE_SUCCESS = [[
                night_merchant:
                    Fine. I'll give {target} one last chance. If {target.heshe} disappoints me again I will have {target.himher} fed to the crayotes.
                target:
                    !right
                    !happy
                    Oh thank Hesh! I won't waste this chance!
                night_merchant:
                    !right
                    I suppose I'll take you to the market, now, grifter.
            ]],
            OPT_REFUSE = "Refuse",
            DIALOG_REFUSE = [[
                player:
                    I can't be a part of this.
                night_merchant:
                    !right
                    Oh, so now you grow a conscience?
                    How disappointing.
                    I'll be back, {target}. Don't think this is over.
                    !exit
                * {night_merchant} marches off into the night.
                target:
                    !right
                    Now {night_merchant.heshe} is even more angry.
                    What am I going to do?
                player:
                    !shrug
                * Lacking answers, you leave too.
            ]],
            SIT_MOD = "Bad for business",

        }

    :Fn(function(cxt) 
            cxt.enc:SetPrimaryCast(cxt.quest:GetCastMember("night_merchant"))
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_DO_IT")
                :PreIcon(global_images.rob)
                :LoopingFn(function(cxt) 
                    if cxt:FirstLoop() then
                        cxt:Dialog("DIALOG_DO_IT")
                        cxt.quest:Complete("confront_target")
                        cxt.quest:GetCastMember("target"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("extracted_a_graft"))
                        cxt.enc.scratch.graft = RewardUtil.GetGrafts(TheGame:GetGameState():GetCurrentBaseDifficulty(), 1, TheGame:GetGameState():GetPlayerAgent())[1]
                    end
                        
                    
                    cxt:Opt("OPT_TAKE_GRAFT", cxt.enc.scratch.graft)
                        :PreIcon(global_images.receiving)
                        :Fn(function() 
                            
                            local installed = ConvoUtil.GiveGraft(cxt, cxt.enc.scratch.graft)
                            if installed then
                                cxt:Dialog("DIALOG_TAKE_GRAFT")
                                cxt:Dialog("DIALOG_GO_TO_MARKET")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end)
                    
                    cxt:Opt("OPT_SKIP_GRAFT")
                        :PreIcon(global_images.close)
                        :Dialog("DIALOG_SKIP")
                        :Dialog("DIALOG_GO_TO_MARKET")
                        :Travel()
                end)

            cxt:Opt("OPT_CONVINCE_MERCHANT")
                :Dialog("DIALOG_CONVINCE")
                :Negotiation{
                    --helpers = {cxt.quest:GetCastMember("target")},
                    situation_modifiers =
                    {   
                        { value = 20, text = cxt:GetLocString("SIT_MOD") }
                    },
                    on_success = function() 
                        cxt:Dialog("DIALOG_CONVINCE_SUCCESS")
                        cxt.quest:GetCastMember("target"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("stood_up_for_them"))
                        cxt.quest:Complete("confront_target")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    
                    on_fail = function()
                        cxt:Dialog("DIALOG_CONVINCE_FAIL")
                        cxt.quest:GetCastMember("target"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("refused_to_extract"))
                        cxt.quest:Fail()
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }
            cxt:Opt("OPT_REFUSE")
                :PreIcon(global_images.reject)
                :Dialog("DIALOG_REFUSE")
                :Fn(function(cxt) 
                    cxt.quest:GetCastMember("target"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("refused_to_extract"))
                    cxt.quest:Fail()
                    StateGraphUtil.AddLeaveLocation(cxt)
                end)
        end)


QDEF:AddConvo("go_to_market", "night_merchant")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == TheGame:GetGameState():GetLocation("MARKET_STALL") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !dubious
                    $miscMocking
                    This is it, huh? Pretty modest.
                night_merchant:
                    !right
                    !chuckle
                    $happyCocky
                    People don't come here for the ambiance.
                    They come here for the goods they can't get anywhere else.
                ** {night_merchant} now likes you, and will offer you discounts on {night_merchant.hisher} shop.
            ]],
        }
    :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:GetCastMember("night_merchant"):OpinionEvent(OPINION.JOB_MINOR, { quest = cxt.quest })
            cxt.quest:Complete()
            StateGraphUtil.AddEndOption(cxt)
        end)



