local CleanUpFn = function(quest) 
    if quest:GetCastMember("night_merchant"):IsInPlayerParty() then
        quest:GetCastMember("night_merchant"):Dismiss()
    end

    if quest:IsFailed() and not quest:GetCastMember("night_merchant"):IsDead() then
        quest:GetCastMember("night_merchant"):OpinionEvent(OPINION.JOB_FAILED_MINOR, { quest = quest})
    end

end

local QDEF = QuestDef.Define
{
    title = "Night Monsters",
    desc = [[Help {night_merchant} with some wildlife management]],
    icon = engine.asset.Texture("icons/quests/sal_story_escort_merchant.tex"),
    qtype = QTYPE.STORY,
    block_side_jobs = true,    
    
    events = 
    {
        caravan_member_remove = function(quest, agent) 
            if agent == quest:GetCastMember("night_merchant") then
                quest:Fail()
            end
        end,
    },
    on_fail = CleanUpFn,
    on_complete = CleanUpFn,
    on_cancel = CleanUpFn,

    collect_agent_locations = function(quest, t)
        if quest:IsActive("offer_job") then
            table.insert (t, { agent = quest:GetCastMember("night_merchant"), location = TheGame:GetGameState():GetLocation("GROG_N_DOG"), role = CHARACTER_ROLES.VISITOR } )
        end
    end,

}

:AddCastByAlias{
    cast_id = "night_merchant",
    alias = "ENDO",
    provider = true,
    no_validation = true,
    no_cleanup = true,
    events = 
    {
        agent_retired = function(quest) 
            quest:Fail()
        end
    }
}

:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}

:AddLocationCast{
    cast_id = "market",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MARKET_STALL"))
    end,
}

:AddCast{
    cast_id = "the_cave",
    when = QWHEN.MANUAL,
    no_validation = true,
}

:AddCast{
    cast_id = "other_merchant",
    when = QWHEN.MANUAL,
}

:AddObjective{
    id = "offer_job",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    
    on_activate = function(quest)
        quest:GetCastMember("night_merchant"):MoveToLocation(TheGame:GetGameState():GetLocation("GROG_N_DOG"))
    end,
    mark = {"night_merchant"},
    on_complete = function(quest)
        quest:Activate("go_to_cave")
    end,
}

:AddObjective{
    id = "go_to_cave",
    title = "Clear out the monsters",
    desc = "{night_merchant} has tasked you with purging a monster nest.",
    
    mark = {"the_cave"},

    on_activate = function(quest)
        quest:GetCastMember("night_merchant"):Recruit( PARTY_MEMBER_TYPE.ESCORT )
        quest:SpawnTempLocation("MONSTER_CAVE", "the_cave" )
    end,

    on_complete = function(quest)
        quest:Activate("go_to_market")
    end,
    
}

:AddObjective{
    id = "go_to_market",
    title = "Take {night_merchant} to the Market",
    desc = "{night_merchant} has asked you to take them to the night market.",
    
    mark = {"market"},

    on_activate = function(quest)
    end,
}


:AddLocationDefs{
    MONSTER_CAVE = {
        name = "Monster Cave",
        plax = "EXT_Cave_Murderbay_1",
        indoors = false,
        map_tags = {"forest"},
    },

}
---------------These are shown between jobs. maybe fssh should tell you more stuff (optionally?) Or even give you things, if it feels arduous for the player to make the trek back to the grog


QDEF:AddConvo("offer_job", "night_merchant")
        :Loc{
            OPT_ASK_ABOUT_JOB = "Ask about special job",
            DIALOG_STRANGER_INTRO = [[
                player:
                    !permit
                    Hey, I'm a friend of Fssh's. How can I help?
                night_merchant:
                    !right
                    !interest
                    {fssh} tells me you're pretty good with a blade.
                player:
                    !crossed
                    I get by. Sounds like you need something stabbed. Is it something with four legs or two?
                night_merchant:
                    Hmm. 
                    !agree
                    Yes.
                player:
                    !dubious
                night_merchant:
                    !overthere
                    Let's just call it an infestation, and it's getting in the way of my good deeds. 
                    I'll get you there, and you... flash your steel, or whatever it is hunters do. Agreed?

                *** {night_merchant} asks if you'll clear out an infestation for {night_merchant.himher}.
            ]],
            
            OPT_DISAGREE = "Not right now",
            DIALOG_DISAGREE = [[
                player:
                    !shrug
                    It's been a long day, and that sounds like a lot of work.
                    Rain check?
                night_merchant:
                    !disappoint
                    I don't have that luxury. But you'll go your way, and I'll go mine.
            ]],
            OPT_AGREE = "Accept the job",
            DIALOG_AGREE = [[
                player:
                    !happy
                    Sure. Plus you made it sound so exciting.
                night_merchant:
                    !agree
                    You're the thrillseeker, and I'm the client. Let's go.

                * {night_merchant} joins you.

                other_merchant:
                    !right
                    %merchant_rejected
            ]],
        }
    
    :Hub(function(cxt) 
            cxt:Opt("OPT_ASK_ABOUT_JOB")
                :SetQuestMark()
                :LoopingFn(function(cxt)
                    if cxt:FirstLoop() then
                        cxt:Dialog("DIALOG_STRANGER_INTRO")
                    end
                    
                    cxt:Opt("OPT_AGREE")
                        :CompleteQuest("offer_job")
                        :PreIcon( global_images.accept )
                        :ReceiveOpinion( OPINION.DID_NOT_HELP, nil, cxt.quest:GetCastMember( "other_merchant" ))
                        :Dialog("DIALOG_AGREE")
                        :Fn(function() StateGraphUtil.AddEndOption(cxt) end )

                    cxt:Opt("OPT_DISAGREE")
                        :PreIcon( global_images.reject )
                        :Dialog("DIALOG_DISAGREE")
                        :Pop()

                    cxt:Opt("OPT_ASK_ABOUT")
                        :IsHubOption(true)
                        :GoTo("STATE_QUESTIONS")
                    
                end)
        end)


    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask about the market",
        [[
            player:
                !dubious
                $neutralDubious
                So what's so special about this market, anyway?
            night_merchant:
                $neutralWhatever
                It operates, ah, outside of Admiralty supervision. So you get quality goods with minimal taxation. 
            player:
                And how does the Admiralty feel about this?
            night_merchant: 
                !handwave
                Hunter, please. I'm a Heshian. I don't give a squat what the Admiralty feels about anything.
            player:
                !chuckle
                Eloquently put, priest. 
        ]],

    })        




QDEF:AddConvo("go_to_cave", nil)
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("the_cave") end)
        :Loc{
            DIALOG_INTRO = [[
                * {night_merchant} leads you to the cave, which emanates a stink you can't quite place.
                player:
                    !left
                night_merchant:
                    !right
                    Here it is.
                player:
                    So what now? You want me to—
                agent: 
                    !right
                player:
                    !surprised
                    $angryTakenAback
                    Aw, Hesh!
                    !fight

                * At least the monsters got respect for your time.
            ]],
            OPT_DEFEND = "Defend yourself",
            
            DIALOG_MERCHANT_SURVIVED = [[
                player:
                    !tired
                    Heshin' Hesh spit. 
                night_merchant:
                    !right
                    !angry_accuse
                    Enough! The god of salt didn't slither forth so you could squander its holy name.
                    !disappoint
                    All the same, you've done what I asked—and I'm grateful. 
                    The market awaits. Let's be off. 
            ]],
            
            DIALOG_MERCHANT_DIED = [[
                agent:
                    !exit
                * The battle's won, but unfortunately {night_merchant} didn't make it, and it plucks at you like a pointed finger. 
                * Nothing for it but to retrace your steps back to the Grog n' Dog, and try to tell Fssh what happened.
            ]],
        }
    :Fn(function(cxt) 
            
            local monsters = CreateCombatParty("MONSTERS", 3, cxt.location)
            cxt.quest:SetParameter("monsters", monsters)
            cxt.encounter:SetPrimaryCast(monsters[1])

            cxt:Dialog("DIALOG_INTRO")
            cxt:Opt("OPT_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    allies = {cxt.quest:GetCastMember("night_merchant")},
                    
                    -- --this is technically a story quest, so by default you aren't allowed to run away.
                    -- This handler permits it.
                    on_runaway = StateGraphUtil.DoRunAway,
                    
                    on_win = function(cxt) 
                        if cxt.quest:GetCastMember("night_merchant"):IsDead() then
                            cxt:Dialog("DIALOG_MERCHANT_DIED")
                            cxt.quest:Fail()
                        else
                            cxt:Dialog("DIALOG_MERCHANT_SURVIVED")
                            cxt.quest:Complete("go_to_cave")
                        end
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                }
        end)


QDEF:AddConvo("go_to_market", "night_merchant")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == TheGame:GetGameState():GetLocation("MARKET_STALL") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !dubious
                    $miscMocking
                    This is it, huh? Pretty modest.
                night_merchant:
                    !right
                    !dubious
                    $neutralJoke
                    It doesn't exist for the decor. The goods here are unbound by tax or law. 
                    And now you're welcome to do your shopping.
                    !agree
                    $happyThanks
                    Thanks, Sal. Fssh was right: you do good work.
                ** {night_merchant} now likes you, and will offer you discounts on {night_merchant.hisher} shop.

            ]],
        }
    :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:GetCastMember("night_merchant"):OpinionEvent(OPINION.JOB_MINOR, { quest = cxt.quest })
            cxt.quest:Complete()
            StateGraphUtil.AddEndOption(cxt)
        end)



