local QDEF = QuestDef.Define
{
    title = "Invasive Species",
    desc = [[{night_merchant} needs help trapping an elusive beast.]],
    
    icon = engine.asset.Texture("icons/quests/sal_story_merchant_pet.tex"),
    qtype = QTYPE.STORY,
    block_side_jobs = true,    
    
    events = 
    {
        caravan_member_remove = function(quest, agent) 
            if agent == quest:GetCastMember("night_merchant") then
                quest:Fail()
            end
        end,
    },

    on_destroy = function(quest) 
        quest:GetCastMember("night_merchant"):Dismiss()
        if quest:IsFailed() and not quest:GetCastMember("night_merchant"):IsDead() then
            quest:GetCastMember("night_merchant"):OpinionEvent(OPINION.JOB_FAILED_MINOR, { quest = quest})
        end

    end,

    collect_agent_locations = function(quest, t)
        if quest:IsActive("offer_job") then
            table.insert (t, { agent = quest:GetCastMember("night_merchant"), location = TheGame:GetGameState():GetLocation("GROG_N_DOG"), role = CHARACTER_ROLES.VISITOR } )
        end
    end,
    
    process_fighter = function(quest, fighter)
        if fighter.agent == quest.param.monster then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end

}

:AddCastByAlias{
    cast_id = "night_merchant",
    alias = "BEASTMASTER",
    provider = true,
    no_validation = true,
    no_cleanup = true,
    events = 
    {
        agent_retired = function(quest) 
            quest:Fail()
        end
    }
}

:AddCast{
    cast_id = "other_merchant",
    when = QWHEN.MANUAL,
}


:AddCastByAlias{
    cast_id = "fssh",
    alias = "FSSH",
    no_validation = true,
}

:AddLocationCast{
    cast_id = "market",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MARKET_STALL"))
    end,
}

:AddObjective{
    id = "offer_job",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    mark = {"night_merchant"},
    on_complete = function(quest)
        quest:Activate("hunt_the_beast")
    end,
}

:AddObjective{
    id = "hunt_the_beast",
    title = "Hunt the beast",
    desc = "{night_merchant} needs your help to capture a ferocious beast.",
    mark = {"hunt_location"},

    on_activate = function(quest)
        quest:GetCastMember("night_merchant"):Recruit( PARTY_MEMBER_TYPE.ESCORT )
    end,

    on_complete = function(quest)
        quest:Activate("go_to_market")
    end,
}

:AddLocationCast{
    cast_id = "hunt_location",
    cast_fn = function(quest, t)
        table.insert(t,  quest:SpawnTempLocation("HUNT_LOCATION"))
    end,
}

:AddObjective{
    id = "go_to_market",
    title = "Follow {night_merchant} to the Market",
    desc = "{night_merchant} has asked you to take them to the night market.",
    mark = {"market"},
}

:AddLocationDefs{
    HUNT_LOCATION = {
        name = "Hunt location",
        plax = "EXT_Forest_1",
        indoors = false,
        map_tags = {"forest"},
    },

}
---------------These are shown between jobs. maybe fssh should tell you more stuff (optionally?) Or even give you things, if it feels arduous for the player to make the trek back to the grog


QDEF:AddConvo("offer_job", "night_merchant")
       :Loc{
            OPT_ASK_ABOUT_JOB = "Ask about special job",
            DIALOG_STRANGER_INTRO = [[
                player:
                    !interest
                    {fssh} says you've got work? 
                agent:  
                    !crossed
                    You're the grifter {fssh.heshe} was talking so highly of.
                    Tell me, are you good with animals?
                player:
                    {has_vroc?
                        !joke
                        Vrocs are pretty alright. Once you get past the smell.
                    }
                    {has_crayote?
                        !overthere
                        I'm kind of a crayote person. Can't you tell?
                    }
                    {not has_vroc and not has_crayote?
                        !shrug
                        Depends on the animal, I suppose. 
                    }
                agent:
                    The one I'm concerned with is a bit more exotic than you're probably used to.
                    !thumb
                    There's a grawkit loose in Murder Bay, and I intend to catch it.
                    I just need the right bait.
                player:
                    What kind of bait?
                agent:
                    !interest
                    Well...
                player:
                    !disappoint
                    $neutralResigned
                    Oh. I see.
                agent:
                    !overthere
                    It's building a nest nearby. It won't take too long.

                *** {night_merchant} asks you to help {night_merchant.himher} track down a particularly nasty beast.
            ]],
            
            OPT_DISAGREE = "Not right now",
            DIALOG_DISAGREE = [[
                player:
                    Let me think about it.
                agent:
                    Sure. But don't think too long. Once the nest is built it'll start looking for a mate.
                    You won't want to be within twenty miles when <i>that</> happens.
            ]],
            OPT_AGREE = "Accept the job",
            DIALOG_AGREE = [[
                agent:
                    !neutral
                player:
                    !agree
                    I'll do it.
                agent:
                    !happy
                    Great! I know where it's trying to build its nest.
                    It'll be looking for tough, stringy materials. 
                player:
                    ...
                    !angry
                    Hold up.
                agent:
                    !handwave
                    Just keep your sinews where they belong and you'll be fine.
                * Seems your conversation was overheard. {other_merchant} steps in to interrupt.
                other_merchant:
                    !right
                    %merchant_rejected

                * {night_merchant} joins you.
            ]],

            OPT_ASK_ABOUT_BEAST = "Ask about grawkits",
            DIALOG_ASK_ABOUT_BEAST = [[
                player:
                    What is a grawkit, exactly?
                agent:  
                    The good news is that it's an herbivore.
                player:
                    So it's not dangerous?
                agent:
                    Oh no, it's deadly. It weighs three tons and has a penchant for bellyflopping on anything it sees as a threat.
                    It'll kill you. It just won't eat you after.
            ]],
            OPT_HOW_GOT_HERE = "Ask how the grawkit got here",
            DIALOG_ASK_HOW_GOT_HERE = [[
                player:
                    So this grawkit's not from around here?
                agent:  
                    !sigh
                    No, they're from Grout Bog, mostly. This one was probably a pet that got released.
                player:
                    !dubious
                    People keep these things as pets?
                agent:
                    Oh, yeah, they're cute little things when they're young.
                    !throatcut
                    But keep it too long and next thing you know it's built a nest out of your sweaters. With your arms still inside.
                    !happy
            ]],
        }
    
    :Hub(function(cxt) 
            local q2 = false
            cxt:Opt("OPT_ASK_ABOUT_JOB")
                :Fn(function() 

                    for k,pet in pairs(TheGame:GetGameState():GetCaravan():GetPets()) do
                        local id = pet:GetContentID()
                        if id == "VROC" then
                            cxt.enc.scratch.has_vroc = true
                        end
                        if id == "CRAYOTE" or id == "CRAYOTE_SPITTER" then
                            cxt.enc.scratch.has_crayote = true
                        end
                    end

                end)
                :SetQuestMark()
                :Dialog("DIALOG_STRANGER_INTRO")
                :LoopingFn(function(cxt)
                    
                    
                    cxt:Question("OPT_ASK_ABOUT_BEAST", "DIALOG_ASK_ABOUT_BEAST", function() q2 = true end)
                    if q2 then
                        cxt:Question("OPT_HOW_GOT_HERE", "DIALOG_ASK_HOW_GOT_HERE")
                    end

                    cxt:Opt("OPT_AGREE")
                        :CompleteQuest("offer_job")
                        :PreIcon( global_images.accept )
                        :ReceiveOpinion( OPINION.DID_NOT_HELP, nil, cxt.quest:GetCastMember( "other_merchant" ))
                        :Dialog("DIALOG_AGREE")
                        :Fn(function() StateGraphUtil.AddEndOption(cxt) end )
            
                    cxt:Opt("OPT_DISAGREE")
                        :PreIcon( global_images.reject )
                        :Dialog("DIALOG_DISAGREE")
                        :Pop()

                end)
        end)


QDEF:AddConvo("hunt_the_beast", nil)
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("hunt_location") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                night_merchant:
                    !right
                    Can you smell that?
                player:
                    !thought
                    It smells like cabbage flatulence and old leather.
                night_merchant:
                    !agree
                    Grawkit. Ain't no doubt.
                    We've got the advantage of bein' downwind, so let's not squander it.
                    Now, you're the bait, mind. So you're in charge of sending it my way.
                player:
                    And where are you going to be?
                night_merchant:
                    !point
                    Over yonder, hiding in the bushes.                 
                player:
                    Shouldn't we try to flank it? Make it—
                night_merchant:
                    !placate
                    Naw, too dangerous for me. 
                    I ain't paying you to take issue with my strategy. But if you're gonna be a baby about it, I have some cracklers you can use. They might help.
            ]],

            OPT_TAKE_CRACKLER = "Take {1#card}",
            DIALOG_TAKE_CRACKLER = [[
                player:
                    !take
            ]],
            OPT_ATTACK = "Attack the beast",
            DIALOG_ATTACK = [[
                player:
                    I'm ready.
                night_merchant:
                    Remember. Don't kill it!
                    !exit
                * Soon enough, the grawkit comes lumbering by. You wave your gangly limbs from side to side, hoping they look tough and alluring.
                player:
                    !overthere
                    %angryHostile
                    Oy, farts-for-brains! You want some of this? 
                    How are you gonna have babies if you can't even provide a home, huh?
                monster:
                    !right
                player:
                    !fight
                * Hard to tell if a grawkit is angry, but at least it seems your taunt was super effective.
            ]],
            DIALOG_KILLED_BEAST = [[
                night_merchant:
                    !right
                    !angry
                    You killed it!
                    It was a majestic beast and you let it die!
                player:
                    !angry_shrug
                    You mighta noticed I was fighting for my life back there. 
                    Maybe you coulda stepped in while I had it distracted, if you wanted it so bad.
                night_merchant:
                    Aw, shove it, hunter. We're done here.
                    !exit
                * {night_merchant} goes to sort through the grawkit's entrails, leaving you with only the grawkit's stink to show for your efforts.
            ]],
            DIALOG_CAPTURED_BEAST = [[
                right:
                    !exit
                    * You drive the grawkit toward {night_merchant}, and it flees into the bushes.
                    * Sounds of a scuffle, punctuated by the occasional squawk, give way to sudden silence.
                    * {night_merchant} joins you in the clearing, dragging a tattered net. 
                player:
                    !dubious
                    You alright? Did it get away?
                night_merchant:
                    !right
                    It did. 
                    !sigh 
                    Magnificent creature, that grawkit. All resilience and odor, absolutely no weakness whatsoever.
                    I can only hope to grow stronger and catch it next time.
                player:
                    You're strangely okay with this. You didn't <i>let</i> it get away, did you?
                night_merchant:
                    O-of course not! That grawkit has eluded me for months.
                player:
                    !happy
                    It can be pretty thrilling, having a worthy rival.
                night_merchant:
                    ...
                    We best move on. You've earned your pay, and your reward.
            ]]
        }

    :Fn(function(cxt) 

            cxt.quest.param.monster = TheGame:GetGameState():AddAgent(Agent( "GRAWKIT" ))
            cxt:ReassignCastMember("monster", cxt.quest.param.monster)
            cxt.quest.param.monster:GetAspect("health"):SetPercent(.8)

            local took_1, took_2, took_3 = false, false, false
            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoop(function() 

                if not took_1 then
                    cxt:Opt("OPT_TAKE_CRACKLER", "crackler")
                        :Dialog("DIALOG_TAKE_CRACKLER")
                        :GainCards{"crackler"}
                        :Fn(function() took_1 = true end)
                end

                if not took_2 then
                    cxt:Opt("OPT_TAKE_CRACKLER", "crackler")
                        :Dialog("DIALOG_TAKE_CRACKLER")
                        :GainCards{"crackler"}
                        :Fn(function() took_2 = true end)
                end

                if not took_3 then
                    cxt:Opt("OPT_TAKE_CRACKLER", "crackler")
                        :Dialog("DIALOG_TAKE_CRACKLER")
                        :GainCards{"crackler"}
                        :Fn(function() took_3 = true end)
                end

                cxt:Opt("OPT_ATTACK")
                    :Dialog("DIALOG_ATTACK")
                    :Battle{
                        enemies = {cxt.quest.param.monster},
                        noncombatants = {cxt.quest:GetCastMember("night_merchant")}

                    }
                        :OnWin()
                            :Fn(function() 
                                if cxt.quest.param.monster:IsDead() then
                                    cxt:Dialog("DIALOG_KILLED_BEAST")
                                    cxt.quest:Fail()
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                else
                                    cxt:Dialog("DIALOG_CAPTURED_BEAST")
                                    cxt.quest:Complete("hunt_the_beast")
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end
                            end)
            end)

        end)


QDEF:AddConvo("go_to_market", "night_merchant")
    :ConfrontState("STATE_CONFRONT", function(cxt) return cxt.location == TheGame:GetGameState():GetLocation("MARKET_STALL") end)
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                    !dubious
                    $miscMocking
                    <i>This</> is the market?
                night_merchant:
                    !right
                    !chuckle
                    $happyCocky
                    It's how we keep the prices low. And the Admiralty off our backs.
                    Take a look around, and if you're in the market for a furry little friend I can hook you up.
                ** {night_merchant} now likes you, and will offer you discounts on {night_merchant.hisher} shop.

            ]],
        }
    :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:GetCastMember("night_merchant"):OpinionEvent(OPINION.JOB_MINOR, { quest = cxt.quest })
            cxt.quest:Complete()
            StateGraphUtil.AddEndOption(cxt)
        end)



