
local PET_HEAL_COST = 100
local PET_COST = 200
local PET_UPGRADE_COST = 250


local PETS = {
    FLEAD = {def = "FLEAD", dialog = "DIALOG_ASK_ABOUT_FLEAD", cost = .8, upgrade_def = "FLEAD_UPGRADED"},
    CRAYOTE = {def = "CRAYOTE", dialog = "DIALOG_ASK_ABOUT_CRAYOTE", cost = 1, upgrade_def = "CRAYOTE_UPGRADED"},
    VROC = {def = "VROC", dialog = "DIALOG_ASK_ABOUT_VROC", cost = 1.2, upgrade_def = "VROC_UPGRADED"},
    SHROOGLET = {def = "SHROOGLET", dialog = "DIALOG_ASK_ABOUT_SHROOGLET", cost = 1.25, upgrade_def = "SHROOGLET_UPGRADED"},
    ERCHIN = {def = "ERCHIN", dialog = "DIALOG_ASK_ABOUT_ERCHIN", cost = 0.8, upgrade_def = "ERCHIN_UPGRADED"},
    LUMICYTE = {def = "LUMICYTE", dialog = "DIALOG_ASK_ABOUT_LUMICYTE", cost = 1.25, upgrade_def = "LUMICYTE_UPGRADED"},
    -- The below are simply used as prices when selling them
    AUTODOG = {def = "AUTODOG", cost = 1.2},
    FERAL_AUTODOG = {def = "FERAL_AUTODOG", cost = 0.5},
    RISE_AUTODOG = {def = "RISE_AUTODOG", cost = .8},
}

function GeneratePets()
    local pets = {}
    for k,v in pairs(PETS) do
        if v.dialog ~= nil then
            table.insert(pets, v.def)
        end
    end
    table.shuffle(pets) 
    local ret = {}
    
    for k = 1, 2 do
        --print( pets[k] )
        local pet = TheGame:GetGameState():AddAgent(Agent(pets[k]))
        pet:AddTag("NO_AUTO_CULL")
        pet:GenerateRandomName()
        table.insert(ret, pet)
    end
    return ret
end
    


--this quest sets up the merchants, and has the shopping conversations in it. 

local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY, 

    on_init = function(quest)
        AgentUtil.TakeJob(quest:GetCastMember("negotiation_shop"), quest:GetCastMember("market"), "negotiation_shop")
        AgentUtil.TakeJob(quest:GetCastMember("battle_shop"), quest:GetCastMember("market"), "battle_shop")
        AgentUtil.TakeJob(quest:GetCastMember("graft_shop"), quest:GetCastMember("market"), "graft_shop")
        AgentUtil.TakeJob(quest:GetCastMember("beastmaster_shop"), quest:GetCastMember("market"), "beastmaster_shop")
    end,

    events = 
    {
        caravan_member_event = function(quest, event, agent, old_loc, new_loc)
            if event == "agent_location_changed" and agent == TheGame:GetGameState():GetPlayerAgent() and new_loc == quest:GetCastMember("market") then
                if quest.param.last_graft_restock ~= TheGame:GetGameState():GetDateTime() then
                    quest.param.negotiation_grafts = GenerateGrafts(GRAFT_TYPE.NEGOTIATION)
                    quest.param.combat_grafts = GenerateGrafts(GRAFT_TYPE.COMBAT)
                    quest.param.last_graft_restock = TheGame:GetGameState():GetDateTime()
                    if quest.param.beasts then
                        for k,v in pairs(quest.param.beasts) do
                            v:Retire()
                        end
                    end
                    quest.param.beasts = GeneratePets()
                end
            end
        end,
    },
}


:AddObjective{
    id = "shop_open",
    state = QSTATUS.ACTIVE,
}


QDEF:AddCastByAlias{
    cast_id = "battle_shop",
    alias = "RAKE",
}

QDEF:AddCastByAlias{
    cast_id = "graft_shop",
    alias = "PLOCKA",
}

QDEF:AddCastByAlias{
    cast_id = "negotiation_shop",
    alias = "ENDO",
}

QDEF:AddCastByAlias{
    cast_id = "beastmaster_shop",
    alias = "BEASTMASTER",
}

:AddLocationCast{
    cast_id = "market",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MARKET_STALL"))
    end,
}


QDEF:AddConvo(nil, "battle_shop")
        :Loc{
            OPT_REMOVE_CARD = "Remove a battle card",
            
            DIAL_REMOVE_CARD = [[
                agent:
                    $miscPersuasive
                    The real secret to expert combat is to know your strengths, and remove your weaknesses.
            ]],
        }    
        :AttractState("STATE_ATTRACT", function(cxt) return cxt:GetAgent():GetBrain():IsOnDuty() end)
            :Loc{
                DIALOG_INTRO = [[
                    player:
                        !left
                    agent: 
                        !right
                    
                ]],
                DIALOG_FIRST_TIME = [[
                    * There's a rough combat ring scratched into the sand. {agent} stands guard over it, scrutinizing your approach.
                    player:
                        !left
                    agent:
                        !right
                        !happy
                        $happyGreeting
                        You look tough for a lumin worker. 
                    player:
                        !crossed
                        $neutralWhatever
                        Don't let the stains fool you. I didn't work lumin for long.
                    agent:
                        $happyDefault
                        Yeah? Then you're probably big on self improvement.
                        !thumb
                        $miscPersuasive
                        Train with me, and I'll teach you to fight Havarian-style. 
                        !permit
                        $happyCocky
                        They don't call it Murder Bay for nothing. 
                ]],
                DIALOG_RETURNING = [[
                    * You find {agent} tending to {agent.hisher} ring.
                    agent:
                        !happy
                        $happyGreeting
                        Knew I'd see you again. Nothing quite as thrilling as the taste of your own blood in your mouth, huh?
                    player:
                        !dubious
                        $neutralDubious
                        I can think of a few things.
                    agent:
                        !wink
                        $miscPersuasive
                        You say that, but here you are.
                ]],
                DIALOG_LEAVE = [[
                    agent:
                        !wink
                        $happyThanks
                        See you soon, hunter! I'm countin' on it.
                ]]
            }
            :Fn(function(cxt) 
                    cxt:Dialog("DIALOG_INTRO")
                    if cxt.quest.param.battle_intro or cxt:GetAgent():KnowsPlayer() then
                        cxt:Dialog( "DIALOG_RETURNING" )
                    else
                        cxt:Dialog( "DIALOG_FIRST_TIME" )
                        cxt.quest.param.battle_intro = true
                    end
                end)

        :Hub(function(cxt)
            if not cxt:GetAgent():GetBrain():IsOnDuty() then
                return
            end
            
            StateGraphUtil.AddRemoveBattleCardOption( cxt, "DIAL_REMOVE_CARD" )
        end)


----------------------------------------------------------------------

QDEF:AddConvo(nil, "negotiation_shop")
        :Loc{
            OPT_ANNIHILATION = "Remove a Negotiation card",
            DIALOG_ANNIHILATION = [[
                agent:
                    $miscPersuasive
                    Hesh annihilates all, but those in Hesh's favor choose the path of their annihilation.
            ]],

        }
        :AttractState("STATE_ATTRACT", function(cxt) return cxt:GetAgent():GetBrain():IsOnDuty() end)
            :Loc{
                DIALOG_INTRO = [[
                    player:
                        !left
                    agent: 
                        !right
                    
                ]],
                DIALOG_FIRST_TIME = [[
                    
                    * A follower of Hesh is meditating here.
                    player:
                        !left
                        $neutralGreeting
                        And what are <i>you</i> peddling, Heshite?
                    agent:
                        !right
                        $neutralThoughtful
                        Do you seek enlightenment, child?
                        Hesh distributes it freely... for a price.
                ]],
                DIALOG_RETURNING = [[
                    agent:
                        $neutralThoughtful
                        The bell of Hesh has room to shelter all.
                    player:
                        !wink
                        $miscPersuasive
                        For a price.
                ]]
            }
            :Fn(function(cxt) 
                    cxt:Dialog("DIALOG_INTRO")
                    if cxt.quest.param.negotiation_intro or cxt:GetAgent():KnowsPlayer() then
                        cxt:Dialog( "DIALOG_RETURNING" )
                    else
                        cxt:Dialog( "DIALOG_FIRST_TIME" )
                        cxt.quest.param.negotiation_intro = true
                    end
                end)

        :Hub(function(cxt)
            if not cxt:GetAgent():GetBrain():IsOnDuty() then
                return
            end

            StateGraphUtil.AddRemoveNegotiationCardOption( cxt, "DIALOG_ANNIHILATION" )
          end)


------------------------------------------------------------------------------------------------------------------------------
--make this person a shop keeper, so that this convo can outlive the quest. 

QDEF:AddConvo(nil, "graft_shop")
        
        :Loc{
            OPT_SELL_GRAFT = "Purchase {1#graft}",
            DIALOG_SELL_GRAFT = [[
                agent:
                    $happyThanks
                    Good choice. You won't regret it.
            ]],
            REQ_FULL = "You have too many grafts of this type",
            
            OPT_SEE_NEGOTIATION_GRAFTS = "Buy negotiation grafts...",
            OPT_SEE_COMBAT_GRAFTS = "Buy battle grafts...",
            OPT_GET_MORE_SLOTS = "Add more graft slots...",
            DIALOG_GET_MORE_SLOTS = [[
                agent:
                    I can drill you a new slot, but I'm not going to lie:
                    !interest
                    It's going to hurt like hell.
            ]],
            DIALOG_ADD_SLOT_1 = [[
                agent:
                    This is going to sting a little.
                    !exit
                player:
                    !exit
            ]],
            DIALOG_ADD_SLOT_2 = [[
                agent:
                    !right
                player:
                    !left
                agent:
                    Good as new!                    

            ]],
            OPT_ADD_NEGOTIATION_SLOT = "Add a negotiation graft slot",
            OPT_ADD_COMBAT_SLOT = "Add a battle graft slot",
            REQ_TOO_MANY = "You have the maximum allowed!",
            
        }
        :AttractState("STATE_ATTRACT", function(cxt) return cxt:GetAgent():GetBrain():IsOnDuty() end)
            :Loc{
                DIALOG_INTRO = [[
                    player:
                        !left
                    agent: 
                        !right
                    
                ]],
                DIALOG_FIRST_TIME = [[
                    * Like all good markets, there's a Spark Baron selling tech of questionable provenance (and safety).
                    player:
                        !left
                    agent:
                        !right
                        !overthere
                        $miscPersuasive
                        You need an edge, hunter? A good graft can warn off your enemies—or lure them to the grave. 
                ]],
                DIALOG_RETURNING = [[
                    * {agent} is tuning up one of {agent.hisher} black-market grafts.
                    agent:
                        !overthere
                        $miscPersuasive
                        I keep telling you—these grafts are made for someone of your talents.
                ]],
                DIALOG_LEAVE = [[
                    agent:
                        $happyThanks
                        Until next time.
                ]],

            }
            :Fn(function(cxt) 
                    -- cxt:Dialog("DIALOG_INTRO")
                    if cxt.quest.param.graft_intro or cxt:GetAgent():KnowsPlayer() then
                        cxt:Dialog( "DIALOG_RETURNING" )
                    else
                        cxt:Dialog( "DIALOG_FIRST_TIME" )
                        cxt.quest.param.graft_intro = true
                    end
                end)

        :Hub( function(cxt, who)
            if not cxt:GetAgent():GetBrain():IsOnDuty() then return end

            cxt:Opt("OPT_SEE_NEGOTIATION_GRAFTS")
                :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.NEGOTIATION )
                :Fn(PresetGrafts(cxt.quest.param.negotiation_grafts))
            
            cxt:Opt("OPT_SEE_COMBAT_GRAFTS")
                :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.FIGHT )
                :Fn(PresetGrafts(cxt.quest.param.combat_grafts))

            cxt:Opt("OPT_GET_MORE_SLOTS")
                :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.OBJECTIVE )
                :Dialog("DIALOG_GET_MORE_SLOTS")
                :Fn(function(cxt)
                        ConvoUtil.OptBuyGraftSlot( cxt, GRAFT_TYPE.NEGOTIATION, "DIALOG_ADD_SLOT_1" )
                            :Dialog( "DIALOG_ADD_SLOT_2" )

                        ConvoUtil.OptBuyGraftSlot( cxt, GRAFT_TYPE.COMBAT, "DIALOG_ADD_SLOT_1" )
                            :Dialog( "DIALOG_ADD_SLOT_2" )

                        StateGraphUtil.AddBackButton(cxt)  
                    end)
        end )

----------------------------------------------------------------------------------------------------------------------------------------------------------------



QDEF:AddConvo(nil, "beastmaster_shop")
        :Loc{
            OPT_ASK_ABOUT_PET = "Ask to see {1#agent}",
            OPT_BUY_PET = "Buy {1#agent}",
            OPT_DONT_BUY_PET = "Look at something else",
            DIALOG_BUY_PET = [[
                player:
                    I'll take it!
                beastmaster_shop:
                    !right
                    Excellent! Try not to break it, alright?
            ]],

            DIALOG_DONT_BUY_PET = [[
                beastmaster_shop:
                    !right
                    No? Something else?
            ]],


            OPT_HEAL_PET = "Heal {1#agent}",
            
            DIALOG_ASK_ABOUT_FLEAD = [[
                player:
                    You're selling fleads? Aren't they just vermin?
                monster:
                    !right
                beastmaster_shop:
                    Oh, no. Fleads are noble beasts!
                    Very loyal, if you earn their respect.
                    Tasty, too, if it comes to it.

            ]],

            DIALOG_ASK_ABOUT_CRAYOTE = [[
                player:
                    Tell me about the crayote.
                monster:
                    !right
                beastmaster_shop:
                    Crayotes are relentless scavengers.
                    Nature's ultimate survivors. Almost nothing can stop them.
                    Except for lasers. They aren't so good against those.
            ]],
            
            DIALOG_ASK_ABOUT_VROC = [[
                player:
                    Vrocs, eh? I've seen the Admiralty use these.
                monster:
                    !right
                beastmaster_shop:
                    With good reason! Vrocs are smart! Like birds!
                    And they're tough, like birds!
                    But they don't fly away, so they're easier to catch.

            ]],
            DIALOG_ASK_ABOUT_SHROOGLET = [[
                player:
                    What the heck is that thing?
                monster:
                    !right
                beastmaster_shop:
                    This is a juvenile shroog.
                    Fully-grown, these things are terrors of the Roaloch. Apex predators in a land of apex predators.
                    But they start out like this lil' cutie.
            ]],
            
            DIALOG_ASK_ABOUT_LUMICYTE = [[
                player:
                    That thing reeks of lumin.
                monster:
                    !right
                beastmaster_shop:
                    It sure does!
                    That's a lumicyte. The Cult harvests them as larvae from the Heshian Sea, and force feeds 'em lumin as they grow.
                    Makes 'em mean. And tough. 
                    And smelly.
            ]],
            DIALOG_ASK_ABOUT_ERCHIN = [[
                player:
                    What is that?
                monster:
                    !right
                beastmaster_shop:
                    This is an erchin.
                    Native to Grout Bog, these walking mouths reproduce faster than any other species in Havaria.
                    But don't worry about this one! I fixed it!
            ]],
            REQ_AT_FULL_HEALTH = "Your pet is at full health already",     
            DIALOG_HEAL_PET = [[
                agent: 
                    A little worse for wear?
                    I can take care of that.
                pet:
                    !right
                * {1.The_desc} healed for {2} HP.
            ]],
            OPT_UPGRADE_PET = "Upgrade {1#agent}",
            DIALOG_UPGRADE_PET = [[
                player:
                    Can you improve {pet} at all?
                agent:
                    Yeah, I think I can.
                    !exit
                * {agent} takes {pet} aside, and leads {pet.himher} behind a curtain.
                * You try not to listen as {agent} gets to work.
            ]],

            DIALOG_UPGRADE_PET_2 = [[
                * Finally, they emerge.
                pet:
                    !right
                * {pet} looks healthy, if a bit wary.
                agent:
                    !right
                    !happy
                    Here you go, better than new!
            ]],
            OPT_SELL_PET = "Sell {1#agent}",
            OPT_REALLY_SELL_PET = "Really sell {1#agent}?",

            DIALOG_SELL_PET = [[
                player:
                    Want to buy {pet}?
                agent:
                    !thought
                    Yeah. I can put it to use.
                pet:
                    !right
                * You hand {pet} over to {agent}. Your pet looks up at you one last time with uncomprehending eyes before being led behind the curtain.
                agent:
                    !right
                    !happy
                    I'm always looking for spare parts!


            ]]


        }    
        :AttractState("STATE_ATTRACT", function(cxt) return cxt:GetAgent():GetBrain():IsOnDuty() end)
            :Loc{
                DIALOG_INTRO = [[
                    player:
                        !left
                    agent: 
                        !right
                    
                ]],
                DIALOG_FIRST_TIME = [[
                    * {agent} stands before you, surrounded by various animal parts and accessories.
                    player:
                        !left
                    agent:
                        !right
                        !happy
                        $happyGreeting
                        Quick - what's the most dangerous beast in Havaria?
                    player:
                        !dubious
                        $neutralWhatever
                        Uh... is it people?
                    agent:
                        $happyCocky
                        Ha! Not even close!
                        The most dangerous animal in Havaria is...
                        !thought
                        $neutralThoughtful
                        Er, yeah, you got it. It <i>is</> people.
                        !happy
                        Anyway, if you want to add to your pack, you've come to the right place!
                ]],
                DIALOG_RETURNING = [[
                    * {agent} is affectionately dressing a crayote carcass.
                    agent:
                        !happy
                        $happyGreeting
                        Hello there, {player}. Have you come back to partake in the splendor of nature?
                ]],
                DIALOG_LEAVE = [[
                    agent:
                        !wink
                        $happyThanks
                        Be careful out there. Everything is hungry!
                ]],
            }
            :Fn(function(cxt) 
                    cxt:Dialog("DIALOG_INTRO")
                    if cxt.quest.param.beast_intro or cxt:GetAgent():KnowsPlayer() then
                        cxt:Dialog( "DIALOG_RETURNING" )
                    else
                        cxt:Dialog( "DIALOG_FIRST_TIME" )
                        cxt.quest.param.beast_intro = true
                    end
                end)

        :Hub(function(cxt)
            if not cxt:GetAgent():GetBrain():IsOnDuty() then return end

                for k,v in ipairs(cxt.quest.param.beasts) do

                    local data = PETS[v:GetContentID()]
                    if data then
                        cxt:Opt("OPT_ASK_ABOUT_PET", v)
                            :Fn(function() 
                                cxt:ReassignCastMember("monster", v)
                                cxt:Dialog(data.dialog)

                                local can_own, reason = cxt.caravan:CanOwnPet()
                                cxt:Opt("OPT_BUY_PET", v)
                                    :ReqCondition( can_own, reason )
                                    :Dialog("DIALOG_BUY_PET", v)
                                    :DeliverMoney(math.round( PET_COST*(data.cost or 1)),{ is_shop = true })
                                    :Fn(function() 
                                        table.arrayremove(cxt.quest.param.beasts, v)
                                        v:Recruit(PARTY_MEMBER_TYPE.CREW)
                                    end)

                                cxt:Opt("OPT_DONT_BUY_PET")
                                    :Dialog("DIALOG_DONT_BUY_PET")
                            end)
                    end
                end

                

                local pets = TheGame:GetGameState():GetCaravan():GetPets()
                for k, pet in ipairs (pets) do
                    local DATA = PETS[pet:GetContentID()]
                    
                    local sell_cost = DATA and DATA.cost
                    if not sell_cost then
                        for k,v in pairs(PETS) do
                            if v.upgrade_def == pet:GetContentID() then
                                sell_cost = v.cost*2
                                break
                            end
                        end
                    end

                    if sell_cost then
                        cxt:Opt("OPT_SELL_PET", pet)
                            :Fn(function() 
                                cxt:ReassignCastMember("pet", pet)

                                cxt:Opt("OPT_REALLY_SELL_PET", pet)
                                    :Dialog("DIALOG_SELL_PET")
                                    :ReceiveMoney(math.round( .5*PET_COST*(sell_cost or 1)))
                                    :Fn(function() 
                                        pet:Dismiss()
                                        pet:Retire()
                                    end)

                                cxt:Opt("OPT_BACK_BUTTON")
                            end)
                    end

                    local current, max, min = pet.health:Get()
                    cxt:Opt("OPT_HEAL_PET", pet)
                        :Fn(function() 
                            cxt:ReassignCastMember("pet", pet)
                        end)
                        :ReqCondition(pet.health:GetPercent() < 1, "REQ_AT_FULL_HEALTH")
                        :DeliverMoney(math.round(PET_HEAL_COST*(1-pet.health:GetPercent())), { is_shop = true })
                        :Dialog("DIALOG_HEAL_PET", pet:LocTable(), math.round( (1-pet.health:GetPercent())*max))
                        :Fn(function() 
                            pet.health:SetPercent(1)
                        end)

                    if DATA and DATA.upgrade_def then
                        cxt:Opt("OPT_UPGRADE_PET", pet)
                            :PostText("TT_PET_UPGRADE")
                            :Fn(function() 
                                cxt:ReassignCastMember("pet", pet)
                            end)
                            :Dialog("DIALOG_UPGRADE_PET")
                            :DeliverMoney(math.round( PET_UPGRADE_COST*(DATA.cost or 1)),{ is_shop = true })
                            :Fn(function() 
                                local upgrade_def = Content.GetCharacterDef( DATA.upgrade_def )
                                    pet:ReinitializeAgent( upgrade_def )
                            end)
                            :Dialog("DIALOG_UPGRADE_PET_2")
                    end

                end
            end)

         
