--this quest sets up the merchants, and has the shopping conversations in it. 

local QDEF = QuestDef.Define
{
    qtype = QTYPE.WANDERING_SHOP, 
    title = "Meat Market",
    desc = "Mercenaries are available for hire at the Murder Torch",
    icon = engine.asset.Texture("icons/quests/sal_story_mercs.tex"),
    hide_in_overlay = true,

    collect_agent_locations = function(quest, t)
        local location = quest:GetCastMember( "lighthouse" )
        if quest:IsActive("shop_open") and location then
            local agent = quest:GetCastMember( "merc" )
            if agent then
                table.insert (t, { agent = agent, location = location, role = CHARACTER_ROLES.VISITOR } )
            end
            local agent = quest:GetCastMember( "weak_merc" )
            if agent then
                table.insert (t, { agent = agent, location = location, role = CHARACTER_ROLES.VISITOR } )
            end
        end
    end,    

    events = {
        quests_changed = function( quest )
            if TheGame:GetGameState():IsHiringAvailable() then
                if quest:GetObjectiveState("shop_open") == QSTATUS.INACTIVE then
                    quest:SetObjectiveState("shop_open", QSTATUS.ACTIVE)
                end
            else
                if quest:GetObjectiveState("shop_open") == QSTATUS.ACTIVE then
                    quest:SetObjectiveState("shop_open", QSTATUS.INACTIVE)
                end
            end
        end,

        phase_change = function(quest, event_quest) 
            if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.NIGHT then
                quest:Cancel()
            end
        end,
    } 

}


:AddObjective{
    id = "shop_open",
    state = QSTATUS.ACTIVE,
    hide_in_overlay = true,
    --mark = {"lighthouse"},
    mark = function(quest, t, in_location)
        table.insert(t, quest:GetCastMember("lighthouse"))      
        if in_location then
            local merc = quest:GetCastMember("merc")
            if merc and merc:GetLocation() == quest:GetCastMember("lighthouse") then
                table.insert(t, merc)
            end

            local merc = quest:GetCastMember("weak_merc")
            if merc and merc:GetLocation() == quest:GetCastMember("lighthouse") then
                table.insert(t, merc)      
            end

            --[[if quest:GetCastMember("strong_merc"):GetLocation() == quest:GetCastMember("lighthouse") then
                table.insert(t, quest:GetCastMember("strong_merc"))      
            end--]]

        end
    end,
}

QDEF:AddCast{
    cast_id = "merc",
    cast_fn = function(quest, t)
        local id = table.arraypick{"BANDIT_GOON", "BANDIT_GOON2", "JAKES_RUNNER"}
        table.insert(t, AgentUtil.GetFreeAgent(id, function(agent) return not AgentUtil.HasJob(agent) and not agent:HasQuestMembership() end))
    end,
    events =
    {
        agent_retired = function( quest, agent )
            quest:UnassignCastMember( "merc" )
            quest:AssignCastMember( "merc" )
        end,
    }
}

QDEF:AddCast{
    cast_id = "weak_merc",
    cast_fn = function(quest, t)
        local id = table.arraypick{"HEAVY_LABORER", "LABORER"}
        table.insert(t, AgentUtil.GetFreeAgent(id, function(agent) return not AgentUtil.HasJob(agent) and not agent:HasQuestMembership() end))
    end,
    events =
    {
        agent_retired = function( quest, agent )
            quest:UnassignCastMember( "weak_merc" )
            quest:AssignCastMember( "weak_merc" )
        end,
    }
}


:AddLocationCast{
    cast_id = "lighthouse",
    
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("LIGHTHOUSE"))
    end,
}


QDEF:AddConvo("shop_open")
    :Loc{
        DIALOG_INTRO = [[
            * A motley of opportunists have gathered here looking for an honest day's work.
            * Prices vary, but so does the skill. 
        ]],
        OPT_HIRE_MERC = "Hire {agent} for your next job",
        DIALOG_HIRED = [[
            player:
                !permit
                $miscAlluring
                I'm hiring, if you're not afraid of trouble.
            agent:
                !shrug
                $neutralWhatever
                Better than sitting here all day. Lead the way, boss. 
        ]],
        OPT_REPLACE_MERC = "Replace {old_merc} with {agent}",
        DIALOG_REPLACED_MERC = [[
            player:
                !permit
                $miscAlluring
                I hope you can do better than {old_merc}.
            agent:
                Best in the business.
        ]],
        REQ_ALREADY_HAVE_HIRELING = "You already have a hireling",
    }
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("lighthouse") and not cxt.quest.param.intro end)
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest.param.intro = true
        end)

    :Hub(function(cxt)
            if cxt.location == cxt.quest:GetCastMember("lighthouse") then
                local is_merc = false
                local cost= 1

                if cxt:GetAgent() == cxt.quest:GetCastMember("weak_merc") then
                    is_merc = true
                    cost = 100
                elseif cxt:GetAgent() == cxt.quest:GetCastMember("merc") then
                    is_merc = true
                    cost = 200
                --[[elseif cxt:GetAgent() == cxt.quest:GetCastMember("strong_merc") then
                    is_merc = true
                    cost = 400--]]
                end

                if is_merc then
                    local hireling = TheGame:GetGameState():GetCaravan():GetHireling()
                    cxt:ReassignCastMember("old_merc", hireling)

                    if hireling == nil then
                        cxt:Opt("OPT_HIRE_MERC")
                            :SetQuestMark()
                            :DeliverMoney(cost + (cxt:GetAgent():GetCombatStrength() - 1)*50)
                            :Fn(function(cxt) 
                                cxt.quest:Complete()
                                cxt:Dialog("DIALOG_HIRED")
                                cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.HIRED )
                                StateGraphUtil.AddEndOption(cxt)
                            end)

                    else
                        cxt:Opt("OPT_REPLACE_MERC")
                            :DeliverMoney(cost + (cxt:GetAgent():GetCombatStrength() - 1)*50)
                            :Fn(function(cxt) 
                                cxt.quest:Complete()
                                cxt:Dialog("DIALOG_REPLACED_MERC")
                                hireling:Dismiss()
                                cxt:GetAgent():Recruit( PARTY_MEMBER_TYPE.HIRED )
                                StateGraphUtil.AddEndOption(cxt)
                            end)
                    end
                end
            end
    end)
