local QDEF = QuestDef.Define
{
    title = "Mettle Vendor",
    icon = engine.asset.Texture("icons/quests/grit_shop.tex"),
    qtype = QTYPE.STORY,
    home = true,
    
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("stranger"), location = quest:GetCastMember("shop"), role = CHARACTER_ROLES.VENDOR } )
    end,

    on_init = function(quest)
        quest.param.started_unlocked = TheGame:GetGameProfile():HasMettleUnlocked(TheGame:GetGameState():GetPlayerAgent():GetContentID())
        if TheGame:GetGameState():CanDoMettle() then
            if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.NIGHT then
                quest:Activate("visit")
            end
        end
    end,

    events = 
    {
        phase_change = function(quest)
            
            if TheGame:GetGameState():CanDoMettle() then
                if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY then
                    quest.param.bought_today = false
                    quest:Cancel("visit")
                else
                    quest:Activate("visit")
                end
            end
        end,
        
        quests_changed = function(quest, event_quest) 
            if not quest.param.started_unlocked and not quest:IsActive("visit") and TheGame:GetGameState():CanDoMettle() then
                quest:Activate("visit")
            end
        end,
    },
}

:AddExistingLocation( "shop", "MB_METTLE_SHOP" )
:AddCastByAlias{
    cast_id = "stranger",
    alias = "DODGY_SCAVENGER",
}

:AddObjective{
    state = QSTATUS.INACTIVE,
    id = "visit",
    title = "Shop for Mettle upgrades",
    desc = "You have <b>{1} mettle</> to spend on upgrades, and <b>{2} Prestige Points</> to spend on Flourishes.",
    desc_fn = function(quest, str) 
        return loc.format(str, TheGame:GetGameProfile():GetMettlePoints(TheGame:GetGameState():GetPlayerAgent():GetContentID()), TheGame:GetGameProfile():GetPrestigePoints())
    end,
    hide_in_overlay = true,
    low_priority = true,
    mark = {"shop"},
}

QDEF:AddConvo("visit")
    :ConfrontState("FOO", function(cxt) return cxt.location == cxt.quest:GetCastMember("shop") end)
        :ProgressionLoc{
            DIALOG_INTRO = {
                [[
                    {first_time?
                        player:
                            !left
                        agent:
                            !right
                        player:
                            Wait... did you seek me out just to recommend your own store?
                        agent:
                            !greeting
                            You've heard of "word of mouth" advertising, haven't you?
                            Just have a look at my wares and I think you'll agree it's a solid recommendation.
                    }
                    {not first_time?
                        player:
                            !left
                        agent:
                            !right
                            !greeting
                            You return!
                    }
                ]],
                [[
                    {first_time?
                        agent:
                            !right
                        player:
                            !left
                            Are you always here?
                        agent:
                            Only when you are, technically speaking.
                            But there are a lot of 'yous' going on, usually.
                        player:
                            Huh?
                        agent:
                            Nevermind. You're here to progress, not solve riddles!
                    }
                    {not first_time?
                            player:
                                !left
                            agent:
                                !right
                                Welcome back, grifter.
                    }
                ]],
                [[
                    {first_time?
                        agent:
                            !right
                        player:
                            !left
                            Wait... why did I come here?
                        agent:
                            Oh come now, you know that I know that you know what's going on.
                            Just roll with it!
                    }
                    {not first_time?
                            player:
                                !left
                            agent:
                                !right
                                Can't stay away, can you?
                    }
                ]],
                [[
                    {first_time?
                        agent:
                            !right
                        player:
                            !left
                            I'm here for mettle.
                        agent:
                            You always are!
                    }
                    {not first_time?
                            player:
                                !left
                            agent:
                                !right
                                Let's make you stronger!
                    }
                ]],
            }
        }
        :Loc{

            OPT_FLOURISHES = "Unlock Flourishes",
            DIALOG_FLOURISHES = [[
                player:
                    I want to learn something new.
            ]],
            DIALOG_POST_FLOURISHES = [[
                agent:
                    It's important to end things well.
            ]],

            OPT_SHOP = "Spend Mettle",
            DIALOG_SHOP = [[
                player:
                    Alright, I'll have a look.
            ]],
            DIALOG_POST_SHOP = [[
                agent:
                    May you grow stronger each day.
            ]],
            OPT_LEAVE = "Stop Shopping",
            DIALOG_LEAVE = [[
                player:
                    Well, I have to go now.
                    !exit
                agent:
                    Until we meet again.
            ]],
            DIALOG_ASK_ABOUT = [[
                player:
                    I have so many questions...
            ]],

            OPT_BUY_METTLE = "Purchase {1} Mettle {1*Point|Points}",
            DIALOG_BUY_METTLE = [[
                agent:
                    Money doesn't buy everything, but it can buy a bit of mettle every day.
            ]],
            TT_BUY_METTLE_ALREADY = "You already bought mettle today! Try again tomorrow.",
        }   
        :Fn(function(cxt) 
            local stranger = cxt.quest:GetCastAgent("stranger")
            stranger:MoveToLocation(cxt.quest:GetCastMember("shop"))
            cxt.enc:SetPrimaryCast(stranger)
            cxt:ProgressDialog("SAL_METTLE_VISITS", "DIALOG_INTRO")
            
            local day = TheGame:GetGameState():GetActProgress()
            local METTLE_POINTS = {2, 4, 6, 8, 16}
            local COST = {100, 200, 300, 400, 600}
            local num_points = METTLE_POINTS[math.min(day, #METTLE_POINTS)]
            local mettle_cost = COST[math.min(day, #COST)]

            cxt:RunLoop(function() 
                cxt:Opt("OPT_SHOP")
                    :Dialog("DIALOG_SHOP")
                    :PreIcon(global_images.mettle)
                    :IsHubOption(true)
                    :Fn(function()
                        cxt.enc:WaitOnLine()
                        local screen = Screen.MettleScreen( cxt.player, function() cxt.enc:ResumeEncounter() end )
                        TheGame:FE():PushScreen( screen )
                        cxt.enc:YieldEncounter()
                        cxt:Dialog("DIALOG_POST_SHOP")
                    end)


                cxt:Opt("OPT_BUY_METTLE", num_points)
                    :Dialog("DIALOG_BUY_METTLE")
                    :DeliverMoney(mettle_cost, {is_shop = true})
                    :ReqCondition(not cxt.quest.param.bought_today, "TT_BUY_METTLE_ALREADY")
                    :PreIcon(global_images.mettle)
                    :Fn(function() 
                        AUDIO:PlayEvent("event:/ui/mettle_shop/buy_mettlepoints")
                        TheGame:GetGameProfile():AddMettlePoints( TheGame:GetGameState():GetPlayerAgent():GetContentID(), num_points, "BUY_METTLE_SHOP") 
                        cxt.quest.param.bought_today = true
                        DoAutoSave()
                    end)

                cxt:Opt("OPT_FLOURISHES")
                    :Dialog("DIALOG_FLOURISHES")
                    :PreIcon(global_images.flourish)
                    :IsHubOption(true)
                    :Fn(function()
                        cxt.enc:WaitOnLine()
                        local screen = Screen.FlourishScreen( cxt.player, function() cxt.enc:ResumeEncounter() end )
                        TheGame:FE():PushScreen( screen )
                        cxt.enc:YieldEncounter()
                        cxt:Dialog("DIALOG_POST_FLOURISHES")
                    end)


                cxt:Opt("OPT_ASK_ABOUT")
                    :Dialog("DIALOG_ASK_ABOUT")
                    :IsHubOption(true)
                    :GoTo("STATE_QUESTIONS")

                cxt:Opt("OPT_LEAVE")
                    :MakeUnder()
                    :Dialog("DIALOG_LEAVE")
                    :PreIcon(global_images.close)
                    :Travel()
            end)
        end)


    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask about mettle",
        [[
            player:
                What is mettle, anyway?
            agent:
                Mettle is a measure of how tough you are. 
                Once you have it, nobody can take it away from you.
                Not even death.
            player:
                !dubious
                Oh, so it's a spiritual thing?
            agent:
                !laugh
                If only you were so lucky!
        ]],
        "Ask how to get mettle",
        [[
            player:
                How do I get mettle?
            agent:
                You need to earn it.
                Do hard things. Prove yourself.
                Mettle will follow.
            player:
                Can't I just, like, buy it?
            agent:
                Yeah, that works, too.
                Parting with shills is a hard thing to do, after all.
        ]],        
        "Ask about flourishes",
        [[
            player:
                So what's a flourish?
            agent:
                It's a power that builds up within you, to be used in times of great crisis.
            player:
                And I can learn this from you?
            agent:
                No, I can't teach you anything.
                I just help you unlock what's already there.
        ]],

    })