local battle_defs = require "battle/battle_defs"
local CARD_FLAGS = battle_defs.CARD_FLAGS
local STATUS = battle_defs.STATUS

Content.AddBattleCard( "smith_day_1_boss_fight_heckle", table.extend(NPC_BUFF)
{
    name = "Heckle",
    hide_in_cardex = true,
    anim = "taunt",
    cost = 0,
    series = CARD_SERIES.NPC,
    rarity = CARD_RARITY.UNIQUE,
    flags = CARD_FLAGS.SKILL | CARD_FLAGS.CONSUME,
    target_type = TARGET_TYPE.ENEMY,
    OnPostResolve = function( self, battle, attack )
        
        --do something negative to the target:
        local options = {"wound", "impair"}
        if attack.target:HasMorale() then
            table.insert(options, "moraledown")
        end
        local action = table.arraypick(options)

        self.owner:SayQuip("smithday1boss", action)
        battle:Delay(1.5)

        if action == "wound" then
            attack.target:AddCondition("WOUND", 3, self)
        elseif action == "impair" then
            attack.target:AddCondition("IMPAIR", 1, self)
        elseif action == "moraledown" then
            attack.target:ApplyMoraleDamage(5, self)
        end
        
        battle:Delay(.5)

        self.owner:Depart()
    end
} )    

Content.AddBattleCard( "smith_day_1_boss_fight_encourage", table.extend(NPC_BUFF)
{
    name = "Encourage",
    hide_in_cardex = true,
    anim = "taunt",
    cost = 0,
    series = CARD_SERIES.NPC,
    target_type = TARGET_TYPE.FRIENDLY,
    rarity = CARD_RARITY.UNIQUE,
    flags = CARD_FLAGS.SKILL | CARD_FLAGS.CONSUME,
    OnPostResolve = function( self, battle, attack )

        --do something negative to the target:
        local options = {"power", "evasion", "defense"}
        
        if attack.target:GetHealthPercent() < 1 and not attack.target:IsPlayer() then
            table.insert(options, "healthup")
        end
        local action = table.arraypick(options)
        
        self.owner:SayQuip("smithday1boss", action)
        battle:Delay(1.5)

        if action == "power" then
            attack.target:AddCondition("POWER", 3, self)
            attack.target:AddCondition("POWER_LOSS", 3, self)
        elseif action == "healthup" then
            attack.target:HealHealth(5, self)
        elseif action == "evasion" then
            attack.target:AddCondition("EVASION", 1, self)
        elseif action == "defense" then
            attack.target:AddCondition("DEFEND", 10, self)
        end
        
        battle:Delay(.5)
        self.owner:Depart()
    end
} )    


local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    title = "The Bar Hits Back",
    
    on_init = function(quest) 
    end,
    icon = engine.asset.Texture("icons/quests/smith_story_homecoming.tex"),

    events = 
    {
        quests_changed = function(quest, event_quest) 
        end,
    },
}

:AddObjective{
    id = "start",
    title = "Return to Sweet Moreef's",
    desc = "",
    state = QSTATUS.ACTIVE,
    mark = { "moreef" },
    
    on_complete = function(quest)
        quest:Activate("day_over")
    end,

    process_fighter = function(quest, fighter)
        if fighter.agent == quest:GetCastAgent("boss") then
            fighter:AddCondition("WANTED_ALIVE")
        end
    end,

}

:AddObjective{
    id = "day_over",
    title = "Talk to {moreef} about your room",
    desc = "{moreef} has a room ready for you. Time to recharge.",
    mark = {"moreef"},

    on_activate = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true
    end,

    on_complete = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
    end,
}

:AddOpinionEvents{

    killed_their_boss =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Killed their boss in front of them",
    },
    
    won_them_over =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Won them over from their crew",
    },

    spared_them_after_fight  =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Spared them after a bar brawl",
    },
    killed_in_bar  =  
    {
        delta = OPINION_DELTAS.DIMINISH,
        txt = "Killed someone in their bar",
    },

    cleaned_up_bar  =  
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Cleared out some trouble in their bar",
    },
}



:AddLocationCast{
    cast_id = "moreef_bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MOREEF_BAR"))
    end,
}

:AddCastByAlias{
    cast_id = "moreef",
    alias = "SWEET_MOREEF",
}


:AddCast{
    cast_id = "boss",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        local boss_def = TheGame:GetGameProfile():GetNoStreakRandom("SMITH_DAY_1_BOSS_PICK", {"WEEZIL", "BRUT"}, 2)
        table.insert( t, quest:CreateSkinnedAgent( boss_def ) )
    end,
}

:AddCast{
    cast_id = "goon1",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "WEALTHY_MERCHANT" )
        table.insert( t, agent )
    end,
}


:AddCast{
    cast_id = "goon2",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "HEAVY_LABORER" )
        table.insert( t, agent )
    end,
}

:AddCast{
    cast_id = "goon3",
    when = QWHEN.MANUAL,
    cast_fn = function(quest, t)
        local agent = quest:CreateSkinnedAgent( "JAKES_RUNNER" )
        table.insert( t, agent )
    end,
}




--probably going to do a thing where you have to convince a bunch of the other people in the bar to help with a special negotiation (where you give away moreef's booze)

QDEF:AddConvo("start", "moreef")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                moreef:
                    !right
                * When you come back the bar is empty save for a table occupied by the kind of rowdy miscreants you generally enjoy.
                * Not tonight though—too much on your mind. But the good pal that you are, you still notice {moreef} seems distracted.
                player:
                    !greeting
                    $neutralJoke
                    Hey, Sweets. You ain't got no love for your ol' friend and savior?
                moreef:
                    !scared_shrug
                    $scaredStammering
                    N-no, it's just—
                    We got customers—
                player:
                    You're welcome.
                moreef:
                    !point
                    Right, but they're causing trouble. See that big lug over there?
                * {moreef} directs your attention to the table—not that you'd need help finding the only other people in the joint.
                moreef:
                    Really less of a customer and more of a parasite. They've scared off all the other clientele.
                    {no_bouncer?
                        And I don't have a bouncer to deal with the riffraff!
                    }
                player:
                    !cruel
                    $miscMocking
                    Fortunately for you, you got riffraff fit to make someone bounce.
                    Lemme handle it.
                    !exit
                moreef:
                    ...
                    !interest
                    $scaredStammering
                    W-wait! Please don't break anything!
                    $miscHushedUrgent
                    And don't kill anyone!
                * You hear the request, and you tell yourself you'll seriously consider it. 
                    !exit
            ]],

            DIALOG_BOSS_INTRO = [[
                * You approach the close-knit group surrounding a mountain of empty and broken glasses.
                * These folks are familiar—the kind of bar slugs you find all across Havaria. 
                * Most of them seem the sort that would melt under an open tab or a closed fist, whichever way you went.
                * But for one. Liquor has put a soft, buttery glaze in {boss.hisher} eyes, but it's only a marinade for a meat too tough to rend.
                boss:   
                    !drunk
                    !right
                player:
                    !left
                    !happy
                    Well hey, fellas. You lookin' to add to your company?
                    Seems awful lonely in here, just you lot and nobody else.
                boss:
                    That's by design. We're not in the mood for socializing.
                    So scram or get scrambled.
            ]],

            OPT_SUCKER_PUNCH = "Sucker-punch {agent}",
            DIALOG_SUCKER_PUNCH = [[
                player:
                    Hey, that's pretty clever, huh? 
                    I got a brother who's good with words like you. 'Cept I plan on killing him tomorrow.
                    !fight
                * You let loose your spring-coiled arm and catch {boss.himher} square between the eyes. {boss.HisHer} buddies holler like wounded yotes, their instincts dulled by the grog.
                boss:
                    !surprised
                    Hey!
                    !angry
                    It's gonna be one of those nights, is it?
                    !fight 
                    Fine by me. I was starting to itch anyway.
            ]],

            OPT_ASK_ABOUT_PARTY = "Ask about the group",
            DIALOG_ASK_ABOUT_PARTY = [[ 
                player:
                    !overthere
                    No socializin'? But there's so many of you!
                    There's gotta be some special occasion that brings you out to the Hideaway.
                    Or, oh—you got sorrows to drown? These fellas your emotional support?
                goon2:
                    !right
                    !angry
                    It's the boss' birthday, if you have to know.
                boss:
                    !right
                    No it ain't! And no he don't!
                    Now I <i>know</i> you heard me the first time, pal. 
                    But since I'm feeling charitable, I'll say it one more time:
                    Go away. 
            ]],

            OPT_UNDERMINE_BOSS = "Undermine {agent}'s cultivated air",
            DIALOG_UNDERMINE_BOSS = [[
                player:
                    Must take a lot of work, getting your voice to grumble like that.
                boss:
                    What?
                player:
                    C'mon, tell me you don't practice your "tough" look in front of a mirror as you brush your teeth.
                goon2:
                    !right
                    Shows what you know! The boss don't even brush {boss.hisher} teeth!
                boss:
                    !right
                    Yes I do, I—
                player:
                    !chuckle
                    Guess that explains the aroma!

            ]],

            DIALOG_FAILURE = [[
                {scared_goons?
                    agent:
                        Is that all you got, smart guy?
                    * The goons that were flagging return to {boss}'s side.
                }
                {not scared_goons?
                    agent:
                        You aren't convincing anyone.
                }
                agent:
                    !fight
                    Stand back, guys. I'll teach this grifter a lesson.
            ]],

            DIALOG_SUCCESS = [[
                convinced_goon:
                    !left
                    You know, I wasn't gonna say anything, boss, but you <i>could</i> reassess your personal hygiene habits...
                agent:
                    What?!
                convinced_goon:
                    Just maybe don't shoot the messenger, you know? This guy did you a favor by telling you. 
                    Think of it like... a birthday present!
                player:
                    !left
                    !happy
                    Yeah! A birthday present!
                agent:
                    !fight
                    You wanna give me a present, grifter? How about one of your eyestalks, for good luck?
            ]],
            DESTROY_MODIFIER_1 = [[
                goon1:
                    !left
                    I think this grifter's tougher than you!
                agent:
                    !angry
                    Fool! Who needs you anyway?
                player:
                    !left
            ]],
            DESTROY_MODIFIER_2 = [[
                goon2:
                    !left
                    He's got your number, boss!
                agent:
                    !angry
                    What do you know?
                player:
                    !left
            ]],
            DESTROY_MODIFIER_3 = [[
                goon3:
                    !left
                    I'm on the wrong side!
                agent:
                    !angry
                    I never liked you anyway!
                player:
                    !left
            ]],
            NEGOTIATION_SHIELD = "Defeat a supporter argument to drop the shield!",
            NEGOTIATION_REASON = "Steal {boss}'s goons and then destroy {boss.hisher} resolve",
            NEGOTIATION_REASON_CONTINUE = "Destroy {boss}'s entourage",
            NEGOTIATION_REASON_END_IT = "Demoralize {boss}!",

        }
        :Fn(function(cxt) 
            LocationUtil.SendPatronsAway( cxt.location )        
            cxt.quest:AssignCastMember("boss")
            cxt.quest:AssignCastMember("goon1")
            cxt.quest:AssignCastMember("goon2")
            cxt.quest:AssignCastMember("goon3")

            cxt.quest:GetCastMember('boss'):MoveToLocation(cxt.location)
            cxt.quest:GetCastMember('goon1'):MoveToLocation(cxt.location)
            cxt.quest:GetCastMember('goon2'):MoveToLocation(cxt.location)
            cxt.quest:GetCastMember('goon3'):MoveToLocation(cxt.location)
            cxt.enc.scratch.no_bouncer = cxt.location:GetWorkPosition("bouncer") == nil or ( cxt.location:GetWorkPosition("bouncer"):GetAgent() == nil )
            cxt:Dialog("DIALOG_INTRO")

            cxt:TalkTo("boss")

            cxt:Dialog("DIALOG_BOSS_INTRO")
            local goons = {cxt.quest:GetCastMember("goon1"), cxt.quest:GetCastMember("goon2"), cxt.quest:GetCastMember("goon3")}
            cxt.quest.param.boss_goons = {}
            cxt.quest.param.your_side_goons = {}

            cxt:RunLoopingFn(function() 

                
                cxt:Question("OPT_ASK_ABOUT_PARTY", "DIALOG_ASK_ABOUT_PARTY")

                cxt:Opt("OPT_UNDERMINE_BOSS")
                    :Dialog("DIALOG_UNDERMINE_BOSS")
                    :Negotiation{
                        --flags = NEGOTIATION_FLAGS.ALLY_SCARE, 
                        enemy_resolve_required = 20,
                        reason_fn = function(minigame)
                            local num = 0
                            for i, modifier in minigame:GetOpponentNegotiator():Modifiers() do
                                if modifier.id == "FIGHT_SUPPORTER" then
                                    num = num + 1
                                end
                            end
                            if num == 0 then
                                return cxt:GetLocString("NEGOTIATION_REASON_END_IT")
                            else
                                return cxt:GetLocString("NEGOTIATION_REASON_CONTINUE")
                            end
                        end,

                        on_start_negotiation = function(minigame)
                            local core = minigame.opponent_negotiator:FindCoreArgument()
                            core:SetShieldStatus( true, cxt:GetLocString("NEGOTIATION_SHIELD"))
                            for k, ally in ipairs(goons) do
                                local mod = Negotiation.Modifier( "FIGHT_SUPPORTER", minigame:GetOpponentNegotiator() )
                                mod.destroy_dialog = cxt:GetLocString("DESTROY_MODIFIER_"..k)
                                mod:SetAgent(ally)
                                minigame:GetOpponentNegotiator():CreateModifier( mod )
                            end
                        end,
                    }
                        :OnSuccess()
                            :Fn(function(cxt, minigame) 
                                
                                for i, modifier in minigame:GetOpponentNegotiator():Modifiers() do
                                    if modifier.id == "FIGHT_SUPPORTER" and modifier.ally_agent then
                                        table.insert( cxt.quest.param.boss_goons, modifier.ally_agent )
                                    end
                                end
                                for k,v in pairs(goons) do
                                    if not table.arrayfind(cxt.quest.param.boss_goons, v) then
                                        table.insert(cxt.quest.param.your_side_goons, v)    
                                    end
                                end
                                cxt:ReassignCastMember("convinced_goon", cxt.quest.param.your_side_goons[1])
                            end)

                            :Dialog("DIALOG_SUCCESS")
                            :GoTo("STATE_FIGHT")

                        :OnFailure()
                            :Fn(function(cxt, minigame) 
                                cxt.enc.scratch.scared_goons = not (minigame:GetOpponentNegotiator():FindCoreArgument() and minigame:GetOpponentNegotiator():FindCoreArgument():GetShieldStatus())
                                cxt.quest.param.boss_goons = goons
                                cxt.quest.param.your_side_goons = {}
                                cxt.quest.param.advantage = TEAM.RED
                            end)
                            :Dialog("DIALOG_FAILURE")
                            :GoTo("STATE_FIGHT")



                cxt:Opt("OPT_SUCKER_PUNCH")
                    :Dialog("DIALOG_SUCKER_PUNCH")
                    :Fn(function() 
                        cxt.quest.param.boss_goons = goons
                        cxt.quest.param.advantage = TEAM.BLUE
                    end)
                    :GoTo("STATE_FIGHT")
            end)


        end)

    :State("STATE_FIGHT")
        :Loc{
            OPT_FIGHT = "Defend the bar",
            DIALOG_FIGHT = [[
                player:
                    !fight
                agent:
                    !fight
                
                {some_convinced?
                    * {boss}'s goons are too busy fighting amongst themselves to provide anything more than moral support.
                }
                {none_convinced?
                    * {boss}'s goons surround the fight, laughing and jeering and blocking all escape routes.
                }
                {all_convinced?
                    * {boss}'s erstwhile goons hang back and jeer at their former companion.
                }
            ]],
            DIALOG_WON_KILLED = [[
                * {agent} slumps over, gasping as the light goes out in {agent.hisher} eyes.
                goon1:
                    !right
                    !scared
                    Holy Hesh-spit!
                goon2:
                    !right
                    !angry
                    You killed {agent.himher}!
                goon3:
                    !right
                    !threaten
                    Watch your back, grifter!
                * The remnants of {agent}'s crew make for the exits, leaving you, {moreef} and the stiff to work out your feelings.
            ]],

            DIALOG_WON_KILLED_2 = [[
                moreef:
                    !angry
                    !right
                    {player}! Do you know what you've done?!
                player:
                    You had a problem. I fixed it for you.
                moreef:
                    !facepalm
                    I'm trying to run a reputable business here. 
                    I can't expect reputable customers to show up if you're washing my floorboards in blood.
                player:
                    !shrug
                    Sorry, I guess.
                    !dismiss
                    I was just trying to help.
                moreef:
                    Just... try harder.
                    I'll deal with the body. You probably need some rest. When you're ready I've made up a room.
                    Grab something from the lost and found, maybe it will help you do better next time.
                player:
                    !happy
                    Thanks buddy!
            ]],
            DIALOG_WON_SPARED = [[
                agent:
                    !injured
                    Stop! Please!
                player:
                    !cruel
                    Had enough, eh?
                agent:
                    We- we just got carried away.
                    No hard feelings?
                player:
                    All feelings are hard, friend.
                    !happy
                    But I'm not one to hold a grudge.
                agent:
                    !overthere
                    We'll see ourselves out.
                    !exit
                * The party moves outside. You sense you haven't seen the last of {agent}.
            ]],
            DIALOG_WON_SPARED_2 = [[
                moreef:
                    !right
                    Thanks, {player}. That could have got ugly.
                player:
                    Please. I'm a professional!
                    I had everything under control the entire time.
                moreef:
                    I've made up a room for you in the back, when you're ready to hit the sack.
                    Oh, and one more thing...
                    Since you handled those goons so well, I'll let you take something from the lost and found.
                player:
                    !happy
                    Just like old times!
            ]],
            DIALOG_POST_REWARDS = [[
                player:
                    People sure lose some good stuff in here!
                    Thanks, Moreef!
                    !exit
            ]],
            FIGHT_REASON = "Get {boss} to surrender!",
        }
        :Fn(function(cxt) 

            cxt.enc.scratch.all_convinced= #cxt.quest.param.boss_goons == 0
            cxt.enc.scratch.none_convinced = #cxt.quest.param.your_side_goons == 0
            cxt.enc.scratch.some_convinced = not cxt.enc.scratch.all_convinced and not cxt.enc.scratch.none_convinced
            
            local goon_idx = 1
            local goons = {cxt.quest:GetCastMember("goon1"), cxt.quest:GetCastMember("goon2"), cxt.quest:GetCastMember("goon3")}
            table.shuffle(goons)

            
            cxt:Opt("OPT_FIGHT")
                :Dialog("DIALOG_FIGHT")
                :Battle{
                    advantage = cxt.quest.param.advantage,
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.BOSS_FIGHT,
                    
                    reason_fn = function(minigame)
                        return loc.format(cxt:GetLocString("FIGHT_REASON"))
                    end,

                    -- make this "on_player_turn_start" instead....
                    on_player_turn_start = function(battle)
                        
                        if not battle:GetEnemyTeam():IsSurrendering() then
                            battle:YieldPlay( STATUS.SYNC )

                            local goon = goons[goon_idx]
                            goon_idx = goon_idx + 1
                            
                            local on_player_side = table.arrayfind(cxt.quest.param.your_side_goons, goon)
                            
                            local fighter = Fighter.CreateFromAgent( goon, 1 )
                            fighter.team = on_player_side and battle:GetTeam(TEAM.BLUE) or battle:GetTeam(TEAM.RED)
                            fighter.call_in = true
                            fighter.home_x = (FIGHT_MAX_DIST)*(on_player_side and -1 or 1)
                            fighter:OnActivateFighter( battle )
                            
                            local boss_fighter = battle:GetFighterForAgent(cxt:GetAgent())
                            local player_fighter = battle:GetFighterForAgent(cxt.player)
                            
                            if math.random() < .5 then
                                local card = Battle.Card("smith_day_1_boss_fight_heckle", fighter)
                                card.show_dealt = false
                                battle:PlayCard(card, on_player_side and boss_fighter or player_fighter)
                            else
                                local card = Battle.Card("smith_day_1_boss_fight_encourage", fighter)
                                card.show_dealt = false
                                battle:PlayCard(card, on_player_side and player_fighter or boss_fighter)
                            end

                            if goon_idx > #goons then
                                goon_idx = 1
                                table.shuffle(goons)
                            end
                        end
                    end,
                }
                    :OnWin()
                        
                        :CompleteQuest("start")
                        :Fn(function() 
                            cxt:Dialog(cxt:GetAgent():IsDead() and "DIALOG_WON_KILLED" or "DIALOG_WON_SPARED")
                            if cxt:GetAgent():IsDead() then
                                for k,v in pairs(cxt.quest.param.your_side_goons) do
                                    v:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("killed_their_boss"))
                                end
                                for k,v in pairs(cxt.quest.param.boss_goons) do
                                    v:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("killed_their_boss"))
                                end
                            else
                                cxt:GetAgent():OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("spared_them_after_fight"))

                                for k,v in pairs(cxt.quest.param.your_side_goons) do
                                    v:OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("won_them_over"))
                                end
                                
                                local overrides = { cast = { boss = cxt:GetAgent() } }
                                QuestUtil.SpawnQuest("SMITH_LOW_FRIENDS_IN_PLACES", overrides)                                

                            end

                            cxt:GetAgent():MoveToLimbo()
                            cxt.quest:GetCastMember("goon1"):MoveToLimbo()
                            cxt.quest:GetCastMember("goon2"):MoveToLimbo()
                            cxt.quest:GetCastMember("goon3"):MoveToLimbo()
                            
                            cxt:Dialog(cxt.quest:GetCastAgent("boss"):IsDead() and "DIALOG_WON_KILLED_2" or "DIALOG_WON_SPARED_2")
                            
                            if cxt.quest:GetCastAgent("boss"):IsDead() then
                                cxt.quest:GetCastMember("moreef"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("killed_in_bar"))
                                ConvoUtil.GiveGraftDraft(cxt, CARD_RARITY.RARE)
                            else
                                cxt.quest:GetCastMember("moreef"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("cleaned_up_bar"))
                                ConvoUtil.GiveBossRewards(cxt, GRAFT_TYPE.NEGOTIATION)
                            end
                            cxt:Dialog("DIALOG_POST_REWARDS")
                        end)
                        :DoneConvo()
        end)


QDEF:AddConvo("day_over", "moreef")
    :Loc{
        
        OPT_RECAP_DAY = "Talk to {moreef} about your plans",
        DIALOG_RECAP_DAY = [[
            {first?
                player:
                    Well, it's been an interesting first day back!
                moreef:
                    You're not exactly keeping a low profile.
                    I appreciate the help with the bar, but you have to know that your siblings are going to make a move against you.
                player:
                    Oh I'm going to deal with them next.
                moreef:
                    What do you mean, 'deal with'?
                player:
                    !happy
                    Nothing specific. Just going to start with friendly conversation, and see where the whim takes us.
                moreef:
                    !sigh
                    I don't like the sound of that.
                    You ready to hit the sack?
            }
            {not first?
                moreef:
                    Your room's ready, if you are.
            }
        ]],
        OPT_GET_A_ROOM = "Head to your room",
        DIALOG_GET_ROOM = [[
            player:
                I'm ready to pass out.
                May as well do it indoors!
            moreef: 
                !exit
            player:
                !exit
            * {moreef} opens a door behind the bar for you.
        ]],


        DIALOG_WAKE = [[
            * You wake up, feeling groggy in the morning light.
        ]],
    }

    :Hub(function(cxt)
            cxt:Opt("OPT_RECAP_DAY")
                :SetQuestMark()
                :Dialog("DIALOG_RECAP_DAY")
                :Fn(function(cxt) 

                    cxt:RunLoopingFn(function() 
                        --local inserted = cxt:InsertQuestions("QUESTIONS")
                        
                        cxt:Opt("OPT_GET_A_ROOM")
                            :Dialog("DIALOG_GET_ROOM")
                            :PreIcon(global_images.sleep)
                            :Fn(function()


                                local grog = cxt.location
                                cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("MOREEF_BAR.inn_room") )
                                grog:SetPlax()
                                cxt:FadeIn()
                                ConvoUtil.DoSleep(cxt, "DIALOG_WAKE")
                                cxt.quest:Complete()
                                cxt:Opt("OPT_LEAVE")
                                    :MakeUnder()
                                    :Fn(function() 
                                        cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("MOREEF_BAR") )
                                        cxt:End()
                                    end)

                            end)
                        StateGraphUtil.AddEndOption(cxt)                            
                    end)    
                end)
        end)
    --[=[:Questions("QUESTIONS", {
        "Q1",
        [[
            * A1
        ]],

        "Q2",
        [[
            * A2
        ]]

    })--]=]
