local CHUM_BONUSES = StateGraphUtil.ValidateChumBonuses(
{
    {
        fn = StateGraphUtil.OfferSpareChange( 100 ),
    },
    {
        fn = StateGraphUtil.OfferUpgradeNegotiation(),
    },
    {
        fn = StateGraphUtil.OfferUpgradeBattle()
    },
    {
        fn = StateGraphUtil.OfferCard( "amnesiator" )
    },
    {
        fn = StateGraphUtil.OfferCompressionGear()
    },
    {
        fn = StateGraphUtil.OfferTriageKit()
    },
    {
        fn = StateGraphUtil.OfferBattleDraft( 2 )
    },
    {
        fn = StateGraphUtil.OfferNegotiationDraft( 2 )
    },
    {
        fn = StateGraphUtil.OfferBoomBox( 2 )
    },
    {
        fn = StateGraphUtil.OfferGraftSlot( GRAFT_TYPE.COMBAT, 1 )
    },
    {
        fn = StateGraphUtil.OfferGraftSlot( GRAFT_TYPE.NEGOTIATION, 1 )
    },
})

local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    on_init = function(quest) 
        TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("SMITH_HOME"):GetDoorLocation())
    end,
    icon = engine.asset.Texture("icons/quests/smith_story_homecoming.tex"),
}

:AddObjective{
    id = "start",
    title = "Go Home",
    desc = "You're not getting into the family home, that's for sure.",
    state = QSTATUS.ACTIVE,
}


:AddDefCast("gate_guard", "ADMIRALTY_GOON")


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


QDEF:AddConvo("start")
    :Confront(function(cxt)
        --if TheGame:GetLocalSettings().FAST_STARTUP then
            --cxt.quest:Complete()
        --else
            return "STATE_START"
        --end
    end)



    :State("STATE_START")
        :Loc{
            DIALOG_INTRO = [[
                * This isn't the first time you wake up in a gutter. But it is the first time it hurts so bad.
                * You knew you weren't the family favorite. But to be kicked out of your own parents' funeral? 

                player:
                    !left
                player:
                    !facepalm
            ]],

            OPT_THINK_OF_BAD_TIMES = "Check yourself for bruises (draft a combat card)",
            DIALOG_THINK_OF_BAD_TIMES = [[
                * A fella like you hasn't combed every bar in Havaria without knowing how to throw a punch, but your sister knocked you down the moment you spoke disrespect.
                * Your bruises will heal. But you'll need to keep your arm springloaded if you don't wanna be caught off-guard again.
            ]],

            OPT_THINK_OF_GOOD_TIMES = "Go over what happened (draft a negotiation card)",
            DIALOG_THINK_OF_GOOD_TIMES = [[
                * You've talked your way out of retribution before. But your baby brother talked twisters around you.
                * Might be a good idea to plate your tongue with silver, at least.
            ]],

        }
        :Fn(function(cxt) 


            cxt:Dialog("DIALOG_INTRO")

            local function OnDone()
                cxt.encounter:ResumeEncounter()
            end


            cxt:Opt("OPT_THINK_OF_GOOD_TIMES")
                :Dialog("DIALOG_THINK_OF_GOOD_TIMES")
                :Fn(function() 
                    local draft_popup = Screen.DraftChoicePopup()
                    local cards = RewardUtil.GetNegotiationCards( 1, 3, cxt.player )
                    draft_popup:DraftCards( cxt.player, Negotiation.Card, cards, OnDone )
                    TheGame:FE():InsertScreen( draft_popup )
                    cxt.enc:YieldEncounter()
                end)
                :GoTo("STATE_GATE")
            
            cxt:Opt("OPT_THINK_OF_BAD_TIMES")
                :Dialog("DIALOG_THINK_OF_BAD_TIMES")
                :Fn(function() 
                    local draft_popup = Screen.DraftChoicePopup()
                    local cards = RewardUtil.GetBattleCards( 1, 3, cxt.player )
                    draft_popup:DraftCards( cxt.player, Battle.Card, cards, OnDone )
                    TheGame:FE():InsertScreen( draft_popup )
                    cxt.enc:YieldEncounter()
                end)
                :GoTo("STATE_GATE")


        end)

    :State("STATE_GATE")
        :Loc{

            DIALOG_MEET_GUARD = [[
                player:
                    !suspicious
                * Before, you entered by the front entrance, elbowing your way through the crowd. This time, though, you head around the back.
                agent:
                    !right
                    !crossed
                    Hold it right there, grifter. The captain has given me license to beat you down again, if you need it.
                player:
                    !angry_accuse
                    If you know who I am, you know I ain't no grifter. This is my house!              
                agent:
                    $neutralWhatever
                    Not anymore. Besides, I've been stationed here for five years, and this is the first time I've seen you.
                player:
                    !angry_shrug
                    I had better things to do than let my glands harden around this dump, didn't I?
                agent: 
                    !cruel
                    Kra'deshi politics are never kind, but to be disowned by your own brood? That must sting. 
                * It'd take time and self-reflection to notice where your feelings are at, and you'd rather avoid it altogether.
                * You notice a box just inside the estate gates, a glint of precious metals catching your eye. 
                * Looks like some of the heirlooms from your parents' vacation home.
            ]],

            OPT_DEMAND_STUFF = "Demand one of the heirlooms",
            DIALOG_DEMAND_STUFF = [[
                    player:
                        !overthere
                        You expect me to walk away empty-handed?
                        My parents are dead, and I've got brood rights. 
                        !point
                        $angryReluctant
                        Rights to that box, for instance. 
                    agent:
                        !dubious
                    * {agent} glances over {agent.hisher} shoulder in surprise—seems the box was overlooked. 
                    agent:
                        No one here cares about your damn rights. I'm doing you a favor by turning you away like this.
            ]],

            OPT_BACK_DOWN = "Back down",
            DIALOG_BACK_DOWN = [[
                player:
                    !angry_accuse
                    $angrySeething
                    You've made a big mistake, switch. I'm the <i>last</i> Banquod you want to make angry.
                    And once I get my footing? I'll be back for you.
                agent:
                    !crossed
                    $miscMocking
                    The other Banquods are the most powerful people in Pearl-on-the-Foam, and they've washed their hands of you.
                * You skulk off, noticing, for the first time, the mud baked into the fine embroidery you wear. 
            ]],

            OPT_THREATEN = "Threaten {agent}",
            DIALOG_THREATEN = [[
                player:
                    !angry_shrug
                    I've tussled with switches before, you know. Only they weren't cozied up in the suburbs, and they knew how to throw a punch.
            ]],
            DIALOG_THREATEN_SUCCESS = [[
                player:
                    !cruel
                    $angryCruel
                    You wanna see what your blood looks like here on the nice flagstone? 
                    I tell ya from experience, that stuff is a pain to get out. 
                * Maybe you are that intimidating—or maybe {agent.heshe} doesn't want to risk a riot. 
                agent:
                    !palm
                    $neutralResigned
                    Fine! But if the Banquods notice anything missing, they'll learn you're a thief as well as a disgrace.
                * You rummage through the box. Most of it's forgettable, but there is one heirloom you'd like to keep...
            ]],
            DIALOG_THREATEN_FAILURE = [[
                player:
                    Of course, if you're itching for a fight—
                agent:
                    I served in the expeditionary forces. You're not fooling anyone with your bluster.
            ]],

            OPT_ATTACK = "Beat up {agent}",
            DIALOG_ATTACK = [[
                player:
                    !fight
                    Didn't expect to have to cave someone's face in on <i>this</i> side of the gates.
                    But at least I'm used to it.
            ]],
            
            DIALOG_WON_FIGHT = [[
                {dead?
                    * This isn't the first time you took a fight too far, but you don't want this to be the last, either.
                    * When you hear shouts from further into the estate, you have to assume reinforcements are on their way.
                    * You open the box.
                }
                {not dead?
                    player:
                        All I wanted was what's rightfully mine. 
                        !shrug
                        $miscMocking
                        "Not on the guestlist," you said. 'Cept my fist was on the guestlist to your face.
                    agent:
                        !injured
                        Hah! All this for a bauble. Take it, then. But I'll be telling your siblings about this.
                        They'll reclaim what's theirs, if they need to.
                    * You open the box. There's not much in here, and most of it is useless.
                }
            ]],

            DIALOG_POST_KILL = [[
                * You grab blindly at whatever fits into your hand, and quickly weave your way out through the crowd. 
                * A stealthy escape isn't your usual style. Fortunately, with the streets so full of grifters, you manage to slip away unnoticed.
            ]],

            OPT_LEAVE = "Leave",
            DIALOG_LEAVE = [[
                player:
                    Watch yourself, switch.
                agent:
                    !shrug
                    <i>I</i> haven't been disowned.
                * Touché.
            ]],
        }
        :Fn(function(cxt) 


            cxt:TalkTo("gate_guard")
            cxt:Dialog("DIALOG_MEET_GUARD")
            
            cxt:RunLoop(function() 

                cxt:Opt("OPT_DEMAND_STUFF")
                    :Dialog("DIALOG_DEMAND_STUFF")
                    :Fn(function() 
                        
                        cxt:Opt("OPT_THREATEN")
                            :Dialog("DIALOG_THREATEN")
                            :Negotiation{flags = NEGOTIATION_FLAGS.INTIMIDATION}
                                :OnSuccess()
                                    :Dialog("DIALOG_THREATEN_SUCCESS")
                                    :Fn(function() 
                                        cxt:RunLoopingFn( StateGraphUtil.OfferChumBonuses, CHUM_BONUSES)
                                        cxt:GoTo("STATE_POST_CHUM")
                                    end)

                                :OnFailure()
                                    :Dialog("DIALOG_THREATEN_FAILURE")

                        cxt:Opt("OPT_ATTACK")
                            :Fn(function() cxt.enc.scratch.did_fight = true end)                            
                            :Dialog("DIALOG_ATTACK")
                                :Battle{}
                                    :OnWin()
                                        :Dialog("DIALOG_WON_FIGHT")
                                        :Fn(function() 
                                            cxt:RunLoopingFn( StateGraphUtil.OfferChumBonuses, CHUM_BONUSES)
                                            
                                            if cxt:GetAgent():IsDead() then
                                                cxt:Dialog("DIALOG_POST_KILL")
                                                cxt.quest:Complete()
                                                StateGraphUtil.AddLeaveLocation(cxt)
                                            else
                                                cxt:GoTo("STATE_POST_CHUM")
                                            end
                                            
                                        end)

                        cxt:Opt("OPT_BACK_DOWN")
                            :Dialog("DIALOG_BACK_DOWN")

                    end)

                cxt:Opt("OPT_LEAVE")
                    :CompleteQuest()
                    :Dialog("DIALOG_LEAVE")
                    :Travel()

            end)
        end)

    :State("STATE_POST_CHUM")
        :Loc{
            DIALOG_INTRO = [[
                * You take what's yours—or at least, the only thing you're allowed.
                agent:
                    Now leave. 
            ]],
            DIALOG_ASK_ABOUT = [[
                player:
                    Wait a second. I got questions.
                agent:
                    !shrug
                    This isn't your funeral... but if you want it to be, I suppose I can't stop you from sticking around.
                    Ask, then.
            ]],
            OPT_LEAVE = "Attempt to leave with a mote of dignity",
            DIALOG_LEAVE = [[
                player:
                    !handwave
                    Fine. I don't need this dump. I've traveled the world!
                    See you on the flipper side, chump.
                    !exit
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")            

            cxt:RunLoop(function()
                
                cxt:Opt("OPT_ASK_ABOUT")
                    :Dialog("DIALOG_ASK_ABOUT")
                    :IsHubOption(true)
                    :GoTo("STATE_QUESTIONS")

                cxt:Opt("OPT_LEAVE")
                    :SetQuestMark()
                    :CompleteQuest()
                    :Dialog("DIALOG_LEAVE")
                    :Travel()
            end)

        end)
    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask about the funeral",
        [[
            player:
                !overthere
                What's with all the peasants, huh?
                No way they're here to give a damn about this rotten family.
            agent:
                They're pilgrims. Or so they claim.
                !shrug
                The Cult has allowed them access. That's all I need to know.
            player:
                Pilgrims? For what?
            agent:
                !impressed
                Why, for <i>you</i>, of course. The prodigal son returned.
            player:
                !angry
                You got a lot of sass for someone at the top of my shucklist. 
        ]],

        "Ask about your parents",
        [[
            player:
                !dubious
                This is all some big scam, ain't it?
                At least tell me how they died, if I don't get to see the bodies.
            agent:
                You aren't alone there—no one has.
            player:
                !eureka
                Ah-hah! Somethin' fishy after all, eh?!
            agent:
                Something fishy and large, more like. 
                Not sure how you made it all the way here without hearing the news, but I shan't spoil it for you.
                You'll have to find someone else to share the grisly details of their demise. 
            player:
                !suspicious
                You hear that? That's the sound of a plot thickening.
            agent: 
                !dubious
                The plot, or perhaps just the blood in your brain. 
        ]],

        "Ask about your siblings",
        [[
            player:
                !sigh
                Still don't make sense. 
                Why'd Mullifee even tell me about the funeral if I wasn't gonna be allowed in, huh?
            agent:
                A mystery you'll have to uncover for yourself.
                There's no love for you here, I'll tell you that much.
            player:
                !angry
                I didn't ask you!
            agent:
                !dubious
                Actually, you did.
        ]],
    })
