local QDEF = QuestDef.Define{
        qtype = QTYPE.STORY,
        icon = engine.asset.Texture("icons/quests/smith_low_friends_in_places.tex"),
        is_in_hiding = function(quest, agent)
            return agent == quest:GetCastMember("boss")
        end,        
    }


    --gets passed in
    :AddCast{
        cast_id = "boss",
    }


    :AddLocationCast{
        cast_id = "moreef_bar",
        cast_fn = function(quest, t)
            table.insert(t, TheGame:GetGameState():GetLocation("MOREEF_BAR"))
        end,
    }

    :AddCastByAlias{
        cast_id = "moreef",
        alias = "SWEET_MOREEF",
    }


    :AddObjective{
        id = "wait",
        state = QSTATUS.ACTIVE,
        on_activate = function(quest)
            quest.param.start_time = TheGame:GetGameState():GetDateTime()
        end,

        is_in_hiding = function(quest, agent)
            if agent == quest:GetCastMember("boss") then
                return true
            end
        end,
        
        events = 
        {
            phase_change = function(quest, new_phase)
                if TheGame:GetGameState():GetDateTime() > quest.param.start_time + 2 then
                    quest:Complete("wait")
                    quest:Activate("hireable")
                end
            end,
        },
    }

    :AddObjective{
        id = "hireable",

        collect_agent_locations = function(quest, t)
            table.insert (t, { agent = quest:GetCastMember("boss"), location = quest:GetCastMember("moreef_bar") } )
        end,

        on_activate = function(quest)
            quest.param.start_time = TheGame:GetGameState():GetDateTime()
        end,

        events = 
        {
            phase_change = function(quest, new_phase)
                if TheGame:GetGameState():GetDateTime() > quest.param.start_time + 4 then
                    quest:Cancel()
                    quest:GetCastMember("boss"):Retire()
                end
            end,
        },
    }


QDEF:AddAttract("hireable", "boss",
    [[
        {not has_heard_offer?
            agent:
                Hello {player}.
                I thought I could find you here.
            player:
                You're not going to start any trouble are you?
            agent:
                No! of course not!
                I just wanted to tell you that I'll be around for a while, and if you need some help, I'm available to hire.
            player:
                How generous of you!
            agent:
                I'm giving you the friends-and-family rate, friend!

        }
        {has_heard_offer?
            agent:
                Offer's still open, grifter.
        }
    ]])

QDEF:AddConvo("hireable", "boss")
        :Loc{
            OPT_HIRE = "Hire {agent}",
            DIALOG_ACCEPT = [[
                player:
                    So you're looking for work, eh?
                agent:
                    I have bills to pay, and scores to settle, just like anyone else.
                player:
                    Alright, I could use the help.
                    !give
                agent:
                    !take
                * {agent} joins you, for now. The pained expression on {agent.hisher} face as {agent.heshe} takes orders is worth every shill.
            ]],
            REQ_CAN_HIRE = "You can't hire help right now",

        }
        :Hub(function(cxt) 
            cxt.quest.param.has_heard_offer = true

            
            cxt:Opt("OPT_HIRE")
                :PreIcon( global_images.hire )
                :Dialog("DIALOG_ACCEPT")
                :ReqCondition(TheGame:GetGameState():IsHiringAvailable(), "REQ_CAN_HIRE")
                :RecruitMember( PARTY_MEMBER_TYPE.HIRED )
                :DeliverMoney(TheGame:GetGameState():GetCurrentBaseDifficulty()*50)
                :CompleteQuest()
                :DoneConvo()

        end)

