
local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    title = "Meet Tei",
    icon = engine.asset.Texture("icons/quests/smith_story_tei.tex"),
    
    on_init = function(quest) 
    end,

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("tei"), location =  quest:GetCastMember("meeting_spot"), role = CHARACTER_ROLES.VISITOR} )
    end,

}


:AddObjective{
    id = "start",
    title = "Go to the meeting spot",
    desc = "Attend the mysterious meeting.",
    state = QSTATUS.ACTIVE,
    mark = { "tei" },
}

:AddObjective{
    id = "do_job",
    hide_in_overlay = true,
    events = 
    {
        quests_changed = function(quest, event_quest)
            if quest.param.job == event_quest and not event_quest:IsActive() then
                quest:Complete()
            end
        end
    },
}


:AddLocationCast{
    cast_id = "meeting_spot",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("TEI_SHRINE"))
    end,
}

:AddCastByAlias{
    cast_id = "tei",
    alias = "TEI",
}

:AddCastByAlias{
    cast_id = "vixmalli",
    alias = "VIXMALLI",
    no_validation = true,
}

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("start", "tei")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                * You figure you'd found the right place when you see the Luminari. You always did have the knack for escalating the drama.
                player:
                    Huh. You the one askin' for me at the Hideaway?
                agent:
                    I am. 
                    Though from what I hear, it won't be too long before you're often asked for.
                player:
                    Uh-huh. I figure you got about five minutes before the intrigue wears off and I get bored.
                    So whaddya want? 
                    I reckon I'm about ten years late on my Cult dues. But you'd have better luck wringing brine from a stone.
                agent: 
                    Luminari aren't tax collectors. 
                    My name is Tei Utaro. And I'm here to propose a partnership.
            ]],
            DIALOG_ASK_ABOUT = [[
                {first_time?
                    player:
                        Uh, look, bud. I'm flattered and all. And I get that I'm a real catch.
                        But I'm not the marrying type, ya know?
                    agent:
                        And I suppose you never will be, now that you've been disowned.
                        No title, no inheritance—a broodling flung from the reef. 
                    player:
                        Huh. Never been neg'd before. 
                        Feels bad, actually.
                    agent:
                        ...
                        I meant a <i>business</i> arrangement, Banquod. 
                        One that sees you restored to your former... glory.
                    player:
                        That sounds pretty underhanded for a Luminari.
                        Naturally I got questions.
                    agent:
                        Naturally.

                }
            ]],
            OPT_START_WORK = "Accept the job",
            DIALOG_START_WORK = [[
                player:
                    So doing this will get me back my inheritance?
                agent:
                    That I can't guarantee. But I do know that whatever happened to you was masterminded by {vixmalli}.
                    Uproot {vixmalli}...
                player:
                    And it's fertile soil underneath. I get it.
                    Seems like we're in business, Luminari. 
                agent:
                    I was confident that would be the case.
            ]],
            OPT_ACCEPT_JOB = "Accept the job",
        }

        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function()
                cxt:Opt("OPT_ASK_ABOUT")
                    :Dialog("DIALOG_ASK_ABOUT")
                    :IsHubOption(true)
                    :GoTo("STATE_QUESTIONS")

                cxt:Opt("OPT_START_WORK")
                    :Dialog("DIALOG_START_WORK")
                    :CompleteQuest("start")
                    :Fn(function() 
                        cxt.quest.param.job = TheGame:GetGameState():GetMainQuest():DefFn("SpawnPoolJob", "TEI_INTRO_JOB")
                        if cxt.quest.param.job then
                            cxt.quest:Activate("do_job")
                            cxt:PlayQuestConvo(cxt.quest.param.job, QUEST_CONVO_HOOK.INTRO)
                            
                            cxt:Opt("OPT_ACCEPT_JOB")
                                :Fn(function() 
                                    cxt:PlayQuestConvo(cxt.quest.param.job, QUEST_CONVO_HOOK.ACCEPTED)
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)
                        end
                        
                    end)
            end)
        end)

        :AskAboutHub("STATE_QUESTIONS", 
        {

            "Ask about the partnership",
            [[
                player:
                    What would a Luminari need with someone like me?
                    Ain't you got the licence to do whatever it is the Cult wants you doin'?
                agent:
                    Luminari are the right hand of the Cult, it's true. But occasionally, we're also the left.
                    The Beaching has introduced the Cult to interesting times. There's blood in the water, and the sharks are making their move.
                    There's corruption in the Cult.
                player:
                    Well, duh.
                agent:
                    ...
                    And I'm seeking to purge it. Covertly.
                    I need someone who can act as my agent.
                    Someone no one would believe is an agent of the Cult.
                player:
                    Ah, the silt settles.
            ]],
            "Ask about the Beaching",
            [[
                player:
                    So that bloated biomass on the beach—the one that killed my folks...
                agent:
                    A message from Hesh, no doubt. The creature's veins run rich with lumin.
                    But what the message means... has stirred a dangerous current in the Cult's ranks.
                    If we don't act, that current becomes a vortex—and vortices tend to drag everything down.
                player:
                    ... 
                    You don't really think <i>my parents</i> were chosen, do you? 
                    Sure, they were Heshians, but only 'cause the neighbors were.
                agent:
                    I'm not sure what to believe. 
                    Until Hesh's intentions are known, I can only go where the waters send me.
            ]],
        })

