local QDEF = QuestDef.Define
{
    title = "Party Store",
    icon = engine.asset.Texture("icons/quests/smith_story_party_store.tex"),
    qtype = QTYPE.STORY,
    home = true,
    
    plot_armour_fn = function(quest, agent)
        if agent:GetBrain():GetWorkplace() == quest:GetCastMember("back_room") then
            return true
        end
    end,

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("plocka"), location = quest:GetCastMember("back_room"), role = CHARACTER_ROLES.VENDOR } )
    end,

    events = 
    {
        caravan_member_event = function(quest, event, agent, old_loc, new_loc)
            if event == "agent_location_changed" and agent == TheGame:GetGameState():GetPlayerAgent() and new_loc == quest:GetCastMember("back_room") then
                if quest.param.last_graft_restock ~= TheGame:GetGameState():GetDateTime() then
                    quest.param.negotiation_grafts = GenerateGrafts(GRAFT_TYPE.NEGOTIATION)
                    quest.param.combat_grafts = GenerateGrafts(GRAFT_TYPE.COMBAT)
                    quest.param.last_graft_restock = TheGame:GetGameState():GetDateTime()
                end
            end
        end,
    },

}

:AddLocationCast{
    cast_id = "shop",
    cast_fn = function(quest, t)
        table.insert(t,  TheGame:GetGameState():GetLocation("PEARL_PARTY_STORE"))
    end,
}

:AddLocationCast{
    cast_id = "back_room",
    cast_fn = function(quest, t)
        table.insert(t,  TheGame:GetGameState():GetLocation("PEARL_PARTY_STORE.back_market"))
    end,
}

:AddCastByAlias{
    cast_id = "plocka",
    alias = "PLOCKA",
}

:AddCastByAlias{
  cast_id = "steven",
  alias = "STEVEN",
  no_validation = true,
}

:AddCastByAlias{
    cast_id = "moreef",
    alias = "SWEET_MOREEF",
}



:AddObjective{
    state = QSTATUS.ACTIVE,
    id = "gain_access",
    title = "Visit the Party Store",
    desc = "The Party Store is more than it seems. Talk to the proprietor about the special reserve.",
    low_priority = true,
    mark = {"shop"},

    on_complete = function(quest)
        quest:Activate("visit")
    end,
}


:AddObjective{
    id = "visit",
    title = "Shop at the Party Store",
    desc = "The business is up front. The party is in the back room.",
    hide_in_overlay = true,
    low_priority = true,
    mark = {"shop"}
}

QDEF:AddConvo("gain_access")
    :ConfrontState("CONFRONT", function(cxt) return cxt.location == cxt.quest:GetCastMember("shop") end)
        :Loc{

            DIALOG_INTRO_LIKED = [[
                * You enter the Party Store which, unsurprisingly, looks like it sells party supplies.
                * The proprietor turns out to be your friend {agent}.
                agent:
                    !right
                    Oh! {player}!
                    !happy
                    What are you doing here?
            ]],
            DIALOG_INTRO = [[
                * You enter the Party Store which, unsurprisingly, looks like it sells party supplies.
                * The proprietor doesn't look too impressed with you, but {agent.heshe}'s likely used to a finer sort of clientele in the Foam.
                player:
                    !left
                agent:
                    !right
                    Can I, ah, <i>help</i> you? 
                    I'm afraid we're not hiring party clowns today.
            ]],

            DIALOG_ASK_ABOUT_SPECIAL_RESERVE_LIKED = [[
                player:
                    !left
                    I heard you maybe had a better selection than your window lets on.
                    That true?
                agent:
                    Did {moreef} tell you about that?
                    !thought
                    {moreef.HeShe}'s supposed to be more discreet about that.
                player:
                    Yes, but {moreef.heshe}'s not. So can I see the goods?
                agent:
                    Yes. They're in the back.
            ]],
            
            OPT_ASK_ABOUT_SPECIAL_RESERVE = "Ask to see the special reserve",
            DIALOG_ASK_ABOUT_SPECIAL_RESERVE = [[
                player:
                    I heard you maybe had a better selection than your window lets on.
                    That true?
                agent:
                    We have the finest selection in the Foam. 
                    Finer than some can afford, I might add.
                player:
                    You might also add a bit more respect, but that's just a suggestion.
                    {moreef} sent me. Told me you'd settle up by letting me see the wares.
                agent:
                    Sweet Moreef isn't the sort to give favors so freely.
                    How do I know you're being truthful?
            ]],

            OPT_CONVINCE = "Convince {agent} that you're cool",
            DIALOG_CONVINCE = [[
                player:
                    Sweets and me go way back, tail to tadpole. 
            ]],

            DIALOG_CONVINCE_SUCCESS = [[
                player:
                    Soon as I heard my buddy was in trouble? You bet I came back to the Foam, just to see him through.
                    He figures I've dusted my knuckles enough helping him out, time we upgrade the hardware.
                    So whaddya say? You gonna cooperate?
                agent:
                    Not like Moreef to let his plights known to just anyone, so I suppose that's legitimate.
                    I'll show you what we have for our more discriminating customers. 
                    But for obvious reasons it's a little easier to show the best stock after dark, so you'll want to check back often to see what we have.
                player:
                    What, really? Life's too short to spend it shoppin'.
                agent:
                    I daresay it'll be shorter still if you don't keep an edge.
                player:
                    Huh. Fair enough.
            ]],

            DIALOG_CONVINCE_FAILURE = [[
                player:
                    In fact, one time, I hung him out the window by his Heshian hood. 
                    He hollered and yelped for the headmaster, bawlin' like the sea's source.
                    I told him he was getting saltier and holier by the second.
                agent: 
                    Doesn't much sound like you were friends, does it?
                player:
                    What? 'Course we were.
                agent:
                    I'm afraid I can't take your word for it.
                    Kindly leave my shop, before I alert the Luminari there's a self-proclaimed heathen within.
                player:
                    Guess you don't know a joke when you see one.
                    Party supplier? More like Party pooper.
                * You're not sure that was your best zinger, so you leave before he can tell you so. 
                * You'll have to try again tomorrow.
            ]],

            OPT_COME_BACK_LATER = "Come back later",
            DIALOG_COME_BACK_LATER = [[
                player:
                    This whole thing suddenly seems like too much work.
                    Maybe I'll come back later when I'm feelin' up for punishment.
                * In your experience, it's better to make an exit than a fool.
                    !exit
            ]]
        }

        :SetLooping()
        :Fn(function(cxt) 

            cxt:TalkTo(cxt.location:GetWorkPosition("proprietor"):GetAgent())

            cxt:Dialog(cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL and "DIALOG_INTRO_LIKED" or "DIALOG_INTRO")

            if cxt:GetAgent():GetRelationship() > RELATIONSHIP.NEUTRAL then
                cxt:Opt("OPT_ASK_ABOUT_SPECIAL_RESERVE")
                    :Dialog("DIALOG_ASK_ABOUT_SPECIAL_RESERVE_LIKED")
                    :SetQuestMark()
                    :Fn(function() 
                        cxt.quest:Complete("gain_access")
                        cxt.quest:Activate("visit")
                        cxt:End()
                        cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("back_room") )
                    end)                    
            else
                cxt:Opt("OPT_ASK_ABOUT_SPECIAL_RESERVE")
                    :Dialog("DIALOG_ASK_ABOUT_SPECIAL_RESERVE")
                    :SetQuestMark()
                    :Fn(function() 
                        cxt:Opt("OPT_CONVINCE")
                            :Dialog("DIALOG_CONVINCE")
                            :Negotiation{timer_id = "GET_INTO_PARTY_STORE"}
                                :OnSuccess()
                                    :Dialog("DIALOG_CONVINCE_SUCCESS")
                                    :Fn(function() 
                                        cxt.quest:Complete("gain_access")
                                        cxt.quest:Activate("visit")
                                        cxt:End()
                                        cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("back_room") )
                                    end)
                                
                                :OnFailure()
                                    :Dialog("DIALOG_CONVINCE_FAILURE")
                                    :Travel()

                        cxt:Opt("OPT_COME_BACK_LATER")
                            :PreIcon( global_images.close)
                            :Dialog("DIALOG_COME_BACK_LATER")
                            :Travel()
                    end)   
    
                cxt:Opt("OPT_COME_BACK_LATER")
                    :PreIcon( global_images.close)
                    :Dialog("DIALOG_COME_BACK_LATER")
                    :Travel()
            end



        end)



QDEF:AddConvo("visit", "steven")
    :AttractState("ATTR")
        :Loc{
            DIALOG_INTRO = [[
                {first_time?
                    player:
                        I've been told the real party's going on back here.
                    agent:
                        !greeting
                        Indeed!
                        This is the place for the partying!
                    player:
                        It doesn't look like much...
                        It's just two of you and some dusty old crates.
                    agent:
                        !point
                        I can assure you, we know how to party.
                        Those crates?
                        Full of party.
                }
                {not first_time?
                    agent:
                        !greeting
                        Are you ready for more of the partying?
                        I certainly am!
                }
            ]],
            OPT_LEAVE = "Stop Partying",
            DIALOG_LEAVE = [[
                player:
                    I think I've had enough partying for now.
                    !exit
                agent:
                    The party will be waiting for you when you get back!
            ]],
            DIALOG_REMOVE_BATTLE_CARD = [[
                player:
                    I have a bad habit I can't seem to shake.
                agent:
                    $neutralThoughtful 
                    Perhaps you have a buggy subroutine?
            ]],
            DIALOG_REMOVE_NEGOTIATION_CARD = [[
                player:
                    Ever just get tongue-tied?
                agent:
                    $neutralThoughtful
                    No. That would require a tongue.
            ]],
            DIALOG_ASK_ABOUT = [[
                player:
                    I have so many questions...
            ]],
            OPT_SHOP = "See what {agent} has for sale...",
            DIALOG_POST_SHOP = [[
                agent:
                    !happy
                    Party every day!
            ]]
        }   
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            
            cxt:RunLoop(function() 
                StateGraphUtil.AddRemoveBattleCardOption( cxt, "DIALOG_REMOVE_BATTLE_CARD" )
                StateGraphUtil.AddRemoveNegotiationCardOption( cxt, "DIALOG_REMOVE_NEGOTIATION_CARD" )

                cxt:Opt("OPT_SHOP")
                    :IsHubOption(true)
                    :PreIcon( global_images.shop, UICOLOURS.MONEY_LIGHT )
                    :Fn(function()
                        cxt.enc:WaitOnLine()
                        local screen = Screen.CardShopScreen( cxt:GetAgent(), function() cxt.enc:ResumeEncounter() end )
                        TheGame:FE():InsertScreen( screen )
                        cxt.enc:YieldEncounter()
                        cxt:Dialog("DIALOG_POST_SHOP")
                    end)


                cxt:Opt("OPT_ASK_ABOUT")
                    :Dialog("DIALOG_ASK_ABOUT")
                    :IsHubOption(true)
                    :GoTo("STATE_QUESTIONS")

                cxt:Opt("OPT_LEAVE")
                    :MakeUnder()
                    :Dialog("DIALOG_LEAVE")
                    :PreIcon(global_images.close)
                    :DoneConvo()
            end)
        end)


    :AskAboutHub("STATE_QUESTIONS", 
    {

        "Ask when the party starts",
        [[
            player:
                So when does the party start?
            agent:
                Look around you, friend. The party has already started!
        ]],
        "Ask what the party is about",
        [[
            player:
                What's the reason for this party?
            agent:
                Parties do not need a reason, friend.
                Parties <b>are</> the reason!
            player:
                Is there a theme?
            agent:
                The theme is... 
                !laugh
                Partying!
        ]],
        "Ask when the party ends",
        [[
            player:
                How long is this party going to last?
            agent:
                This party will never end!
            player:
                Never?
            agent:
                It will outlast you, at least!
        ]],
    })



QDEF:AddConvo("visit")
    :Loc{
        OPT_GO_BACK = "Go to the back room",
    }
    :Hub_Location( function(cxt) 
        if (cxt.location == cxt.quest:GetCastMember("shop") and (cxt.location:GetProprietor() == nil or cxt.location:GetProprietor():GetLocation() ~= cxt.location)) then
            cxt:Opt("OPT_GO_BACK")
                :SetQuestMark()
                :Fn(function() 
                    cxt:End()
                    cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("back_room") )
                end)
        end
    end)

QDEF:AddConvo("visit")
        :Loc{
            OPT_VISIT_THE_PARTY = "Get access to the back room",
            DIALOG_VISIT_THE_PARTY = [[
                {first_time?
                    player:
                        There's too much business here up front!
                        !happy
                        I need to see that party in the back!
                    agent:
                        Gee, I've never heard that one before.
                        !permit
                        Go on in.
                    player:
                        !exit
                }
                {not first_time?
                    agent:
                        Don't upset {steven}.
                    player:
                        !exit
                }
            ]],
            ASK_ABOUT_STEVEN = "Ask about {steven}",
            DIALOG_ASK_ABOUT_STEVEN = [[
                player:
                    So what's the deal with the automech?
                agent:
                    His name is Steven.
                player:
                    Ok, what's the deal with Steven?
                agent:
                    !shrug
                    He's a hard worker. 
                    He takes the whole 'party' motif a bit too literally, perhaps.
                    But he sure does move that product!
            ]]
        }

    :Hub(function(cxt) 
        if (cxt.location == cxt.quest:GetCastMember("shop") and cxt:GetAgent() == cxt.location:GetProprietor()) then
            cxt:Question("ASK_ABOUT_STEVEN", "DIALOG_ASK_ABOUT_STEVEN")
            cxt:Opt("OPT_VISIT_THE_PARTY")
                :SetQuestMark()
                :Dialog("DIALOG_VISIT_THE_PARTY")
                :Fn(function() 
                    cxt:End()
                    cxt.encounter:DoLocationTransition( cxt.quest:GetCastMember("back_room") )

                end)
        end
        
    end)


QDEF:AddConvo("visit", "plocka")
        :AttractState("STATE_ATTRACT")
            :Loc{
                DIALOG_INTRO = [[
                    {first_time?
                        * You're surprised to see a Spark Baron in the Foam, but you suppose enough has changed since you left.
                        player:
                            !left
                            A Baron, huh? Did one of you finally earn your way into the upper crust?
                        agent:
                            !right
                            Money opens doors, and the Spark Barons increasingly have more to offer.
                        player:
                            Yeah? Let's see it, then.
                    }
                    {not first_time?

                        * {agent} is tuning up one of {agent.hisher} black-market grafts.
                        agent:
                            !overthere
                            $miscPersuasive
                            How's it going, choir boy? Ready to see things on the Spark side?

                    }
                ]],
                DIALOG_LEAVE = [[
                    agent:
                        $happyThanks
                        Until next time.
                ]],
                OPT_SELL_GRAFT = "Purchase {1#graft}",
                DIALOG_SELL_GRAFT = [[
                    agent:
                        $happyThanks
                        Ah, a fine choice. I see your taste is finer than your appearance.
                ]],
                REQ_FULL = "You have too many grafts of this type",
                
                OPT_SEE_NEGOTIATION_GRAFTS = "Buy negotiation grafts",
                OPT_SEE_COMBAT_GRAFTS = "Buy battle grafts",
                OPT_GET_MORE_SLOTS = "Add more graft slots",
                DIALOG_GET_MORE_SLOTS = [[
                    player:
                        Hey, folks out there got more upgrades than I remember.
                        How do I get in on that?
                    agent:
                        I am a licensed graft surgeon.
                        !interest
                        But I'm not licensed to administer anaesthesia.
                ]],
                DIALOG_ADD_SLOT_1 = [[
                    agent:
                        This may sting a little. It will likely sting a lot.
                        !exit
                    player:
                        !exit
                ]],
                DIALOG_ADD_SLOT_2 = [[
                    agent:
                        !right
                    player:
                        !left
                    agent:
                        Hm, yes. You appear to be patched up quite nicely.
                        Let me know if anything feels... knocked loose.                    

                ]],
                OPT_ADD_NEGOTIATION_SLOT = "Add a negotiation graft slot",
                OPT_ADD_COMBAT_SLOT = "Add a battle graft slot",
                REQ_TOO_MANY = "You have the maximum allowed!",
                DIALOG_SEE_NEGOTIATION_GRAFTS = [[
                    agent:
                        Need to get the last word in?

                ]],
                DIALOG_SEE_COMBAT_GRAFTS = [[
                    agent:
                        !cruel
                        I can turn you into a weapon!
                ]],


            }
            :Fn(function(cxt) 
                cxt:Dialog("DIALOG_INTRO")
            
                cxt:RunLoop(function() 
                
                    cxt:Opt("OPT_SEE_NEGOTIATION_GRAFTS")
                        :Dialog("DIALOG_SEE_NEGOTIATION_GRAFTS")
                        :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.NEGOTIATION )
                        :LoopingFn(PresetGrafts(cxt.quest.param.negotiation_grafts))
                    
                    cxt:Opt("OPT_SEE_COMBAT_GRAFTS")
                        :Dialog("DIALOG_SEE_COMBAT_GRAFTS")
                        :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.FIGHT )
                        :LoopingFn(PresetGrafts(cxt.quest.param.combat_grafts))

                    cxt:Opt("OPT_GET_MORE_SLOTS")
                        :PreIcon( engine.asset.Texture( "UI/ic_graftscompendium.tex"), UICOLOURS.OBJECTIVE )
                        :Dialog("DIALOG_GET_MORE_SLOTS")
                        :IsHubOption(true)
                        :LoopingFn(function()  
                            ConvoUtil.OptBuyGraftSlot( cxt, GRAFT_TYPE.NEGOTIATION, "DIALOG_ADD_SLOT_1" )
                                :Dialog( "DIALOG_ADD_SLOT_2" )

                            ConvoUtil.OptBuyGraftSlot( cxt, GRAFT_TYPE.COMBAT, "DIALOG_ADD_SLOT_1" )
                                :Dialog( "DIALOG_ADD_SLOT_2" )
                            
                            StateGraphUtil.AddBackButton(cxt)  
                        end)
                    
                    cxt:Opt("OPT_LEAVE")
                        :MakeUnder()
                        :Dialog("DIALOG_LEAVE")
                        :PreIcon(global_images.close)
                        :DoneConvo()

                end)

        end )
