local START_DIFFICULTY =1
local MAX_DIFFICULTY = 5

local DAY_SCHEDULE = {
    {quest = "SMITH_STORY_DAY_1", difficulty = 1},  
    {quest = "SMITH_STORY_DAY_2", difficulty = 2},  
    {quest = "SMITH_STORY_DAY_3", difficulty = 4},  
}

if not IsEarlyAccess() then
    table.insert(DAY_SCHEDULE, {quest = "SMITH_STORY_DAY_4", difficulty = 5} )
end


local QDEF = QuestDef.Define
{
    --title = "The Bog Job",
    qtype = QTYPE.STORY,

    on_init = function(quest)

        local friend_params = {"day_one_friend", "day_two_friend", "day_three_friend", "day_three_friend_2"}
        for _, friend in ipairs(friend_params) do
            if quest.param[friend] and quest.param[friend] ~= "NONE" then
                local agent = AgentUtil.GetOrSpawnAgentbyAlias(quest.param[friend])
                if agent then
                    if not agent.guid then
                        TheGame:GetGameState():AddAgent(agent)
                    end
                    agent.social_connections:SetRelationship(RELATIONSHIP.LOVED, TheGame:GetGameState():GetPlayerAgent())
                end
            end
        end

        TheGame:GetGameState():SetMainQuest(quest)
        TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("MOREEF_BAR"))
        TheGame:GetGameState():SetDifficulty(START_DIFFICULTY)
        QuestUtil.StartDayQuests(DAY_SCHEDULE, quest)
        QuestUtil.DoNextDay(DAY_SCHEDULE, quest, quest.param.start_on_day)
        if not TheGame:GetGameState():GetPlayerAgent().graft_owner:HasGraft("bare_mettle") then
            quest.param.mettle_quest = QuestUtil.SpawnQuest("SMITH_STORY_METTLE_SHOP")
        end

        if (quest.param.start_on_day or 0) > 1 then
            QuestUtil.SpawnQuest("SMITH_LIVING_AT_BAR")
        end

        DoAutoSave()
    end,

    max_day = 4,
    
    on_post_load = function(quest)
        QuestUtil.StartDayQuests(DAY_SCHEDULE, quest)        
    end,

    get_narrative_progress = function(quest)
        
        local total_days = 4
        local completed_days = (quest.param.day or 1)-1
        if quest.param.day_quest == nil then
            return 0, loc.format( LOC "CALENDAR.DAY_FMT", 1 ), ""
        end

        local percent = (completed_days) / total_days
        local title = loc.format(LOC "CALENDAR.DAY_FMT", quest.param.day or 1)
        return percent, title, quest.param.day_quest and quest.param.day_quest:GetTitle() or ""
    end,


    plot_armour_fn = function(quest, agent)
        return agent:IsCastInQuest(quest)
    end,

    events = 
    {
        quests_changed = function(quest, event_quest) 
            if event_quest == quest.param.day_quest and quest.param.day_quest:IsComplete() then
                QuestUtil.DoNextDay(DAY_SCHEDULE, quest)
            end
        end,
    },

    SpawnPoolJob = function(quest, pool_name, default_giver)
        local event_id = pool_name
        local attempt_quest_ids = {}
        for k, questdef in pairs( Content.GetAllQuests() ) do
            if questdef:FilterForAct( "SMITH" ) and questdef:HasTag(pool_name) then
                table.insert(attempt_quest_ids, questdef.id)
            end
        end

        local quest_scores = {}
        for k,v in ipairs(attempt_quest_ids) do
            quest_scores[v] = QuestUtil.CalcQuestSpawnScore(event_id, math.floor(#attempt_quest_ids/2), v)    
        end
        table.shuffle(attempt_quest_ids) --to mix up the case where there are a lot of ties
        table.stable_sort(attempt_quest_ids, function(a,b) return quest_scores[a] < quest_scores[b] end)
        local new_quest
        for _, quest_id in ipairs(attempt_quest_ids) do
            local overrides = {qrank = TheGame:GetGameState():GetCurrentBaseDifficulty()}
            new_quest = QuestUtil.SpawnQuest( quest_id, overrides) 
        
            if new_quest then
                TheGame:GetGameProfile():RecordIncident(event_id, new_quest:GetContentID())
                return new_quest
            end
        end

        if not new_quest then
            local overrides = { cast = {giver = default_giver} }
            local quest_id = "PLACEHOLDER_SMITH_JOB"
            new_quest = QuestUtil.SpawnQuest( quest_id, overrides) 
            assert(new_quest)
        end

        return new_quest
    end,


}

--give our principal cast their jobs
:AddCastByAlias{
    cast_id = "moreef",
    alias = "SWEET_MOREEF",
    on_assign = function(quest, agent)
        AgentUtil.TakeJob(agent, TheGame:GetGameState():GetLocation("MOREEF_BAR"), "bartender")
    end,
}

:AddObjective{
    id = "start",
    state = QSTATUS.ACTIVE,
}   


QDEF:AddDebugOption("start_on_day", {1,2,3,4})
    :AddDebugOption("day_one_friend", {"BRUT", "WEEZIL", "NONE"}, function(params) return params.start_on_day and params.start_on_day > 1 end)
    :AddDebugOption("day_two_friend", {"THEROUX", "DENIKUS", "ANDWANETTE", "NONE"}, function(params) return params.start_on_day and params.start_on_day > 2 end)
    :AddDebugOption("day_three_friend", {"SEEMLI", "EDEN", "MULLIFEE", "NONE"}, function(params) return params.start_on_day and params.start_on_day > 3 end)
    :AddDebugOption("day_three_friend_2", {"BISHOP_OF_FOAM", "TEI", "NONE"}, function(params) return params.start_on_day and params.start_on_day > 3 end)
    :AddDebugOption("skip_to_party", {false, true}, function(params) return params.start_on_day and params.start_on_day == 4 end)





