local QDEF = QuestDef.Define
{
    title = "The Ambush",
    qtype = QTYPE.STORY,
    on_init = function(quest) 
        --TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("SMITH_HOME"):GetDoorLocation())
    end,
    icon = engine.asset.Texture("icons/quests/smith_story_day_3_mullifee.tex"),

    avoids = {"partner", "mullifee"},

    on_post_load = function(quest)
        -- soft-lock backward compatability fix.
        if quest:IsComplete( "go_empty") then
            quest:Complete()
        end
    end,

    on_destroy = function( quest )
        if quest:GetCastMember( "partner" ) and quest:GetCastMember( "partner" ):IsInPlayerParty( ) then
            quest:GetCastMember( "partner" ):Dismiss( )
        end
    end,    
}

:AddCastByAlias{
    cast_id = "moreef",
    alias = "SWEET_MOREEF",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "mullifee",
    alias = "MULLIFEE",
    no_validation = true,
}

:AddCast{
    cast_id = "partner",
    cast_fn = function(quest, t) 
        if quest.param.parent_quest then
            table.insert( t, quest.param.parent_quest:GetCastMember("partner"))
            
            if quest.param.parent_quest:GetCastMember("partner"):GetContentID() == "SEEMLI" then
                quest.param.seemli = true
            else
                quest.param.eden = true
            end

        end
    end,
    no_validation = true,
}

:AddObjective{
    id = "start",
    title = "Meet Mullifee at the bar",
    --desc = "Time t",
    state = QSTATUS.ACTIVE,
    mark = {"mullifee"},

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("mullifee"), location = TheGame:GetGameState():GetLocation("MOREEF_BAR"), role = CHARACTER_ROLES.PATRON} )
    end,

    on_complete = function(quest)
        quest:Activate("get_partner")
    end,
}


:AddObjective{
    id = "get_partner",
    title = "Go to {partner}",
    --desc = "Time t",
    mark = {"partner"},

    is_in_hiding = function(quest, agent)
        if agent == quest:GetCastMember("mullifee") then
            return true
        end
    end,

    on_complete = function(quest)
        quest:Activate("do_trap")
    end,
    on_fail = function(quest)
        quest:Activate("go_empty")
    end,
}

:AddObjective{
    id = "do_trap",
    title = "Take {partner} to the trap",
    --desc = "Time t",
    mark = {"ambush_spot"},

    is_in_hiding = function(quest, agent)
        if agent == quest:GetCastMember("mullifee") then
            return true
        end
    end,

    on_complete = function(quest)
    end,
}


:AddObjective{
    id = "go_empty",
    title = "Tell {mullifee} the trap didn't work",
    --desc = "Time t",
    mark = {"ambush_spot"},

    is_in_hiding = function(quest, agent)
        if agent == quest:GetCastMember("mullifee") then
            return true
        end
    end,
}

:AddQuestLocation{
    cast_id = "ambush_spot",
    name = "Ambush Spot",
    plax = "EXT_SMITH_AMBUSH_SITE",
    map_tags = {"building"},
}

:AddOpinionEvents{
    saved_life =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "'Saved' them from an ambush",
    },
    
    set_up =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Set them up for an ambush",
    },
    convinced_work = 
    {
        delta = OPINION_DELTAS.LIKE,
        txt = "Convinced them to work with you"
    },
    helped_them = 
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Helped them further their career"
    },
    betrayed_deal = 
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Betrayed them in a betrayal plot",
    }
}

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------



QDEF:AddConvo("start")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("mullifee"):GetLocation() end )
        :Loc{
            DIALOG_INTRO = [[
                mullifee:
                    !right
                player:
                    !left
                    Alright, Fee? Ready for schemin'?
                mullifee:
                    !crossed
                    $neutralDirect
                    I don't scheme. I anticipate.
                player:
                    !humouring
                    $happyJoke
                    Sure. Just like you "anticipated" Vix wouldn't like lumicytes in his pajamas. 
                
                {seemli?
                mullifee:
                    !neutral_notepad
                    I've cleared the arrangement with {partner}'s employers. They have a statement prepared tomorrow for how they had already fired her for unrelated matters.
                player:
                    !agree
                    Sure. Sounds believable.
                mullifee:
                    If {partner} has your trust, then tell her that you've arranged a meeting with one of the Rise leaders from the Bog.
                    I've intercepted messages that suggest she's been trying to contact them. 
                player:
                    You got a name I can drop?
                mullifee:
                    !agree
                    Flekfis. There's rumors he might be dead, but nothing substantiated.
                    It should be appropriately tempting.
                    !give
                    We'll prepare the ambush at this location. All you have to do is get {partner} there. 
                player:
                    !thumb
                    $happyCocky
                    Hey, the bare minimum is my specialty.
                mullifee:
                    !dubious
                    $miscMocking
                    Yes. I remember.
                }
                {eden?
                mullifee:
                    !eureka
                    $neutralDirect
                    The trap for {partner} needs to be perfect. I've worked too hard for my position to have it mired in controversy.
                player:
                    !humouring
                    You know, Dad woulda said there was no mud you couldn't bribe your way out of.
                mullifee:
                    $miscResigned
                    Tell that to the Great Beaching. 
                    As for {partner}, she won't be able to resist an opportunity to see me thoroughly disgraced.
                    So that's probably the opportunity to draw her into ambush.
                player:
                    !thought
                    $neutralThought
                    No problem. I'll just tell her 'bout the time you swapped Vix's gill gel for pepper rub.
                mullifee: 
                    $miscResigned
                    I doubt that would be sufficient.
                    !give
                    Here's the location where we'll set the ambush. I trust you'll think up something more enticing.

                }
                
            ]],
            --OPT_Q1_SEEMLI = "Q1 Seemli",
           -- DIALOG_Q1_SEEMLI = [[
               --* !
            --]],
            --OPT_Q2_SEEMLI = "Q2 Seemli",
            --DIALOG_Q2_SEEMLI = [[
                --* !
            --]],

           -- OPT_Q1_EDEN = "Q1 Eden",
            --DIALOG_Q1_EDEN = [[
               -- * !
           -- ]],

            --OPT_Q2_EDEN = "Q2 Eden",
            --DIALOG_Q2_EDEN = [[
                --* !
            --]],

            OPT_TELL_READY = "Tell {mullifee} you're ready",
            DIALOG_TELL_READY = [[
                player:
                    Let's go!
            ]],

        }
        :Fn(function(cxt) 
            cxt:TalkTo("mullifee")
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoopingFn(function() 
                
                --[[if cxt.quest.param.seemli then
                    cxt:QST("Q1_SEEMLI")
                    cxt:QST("Q2_SEEMLI")
                else
                    cxt:QST("Q1_EDEN")
                    cxt:QST("Q2_EDEN")
                end--]]

                cxt:Opt("OPT_TELL_READY")
                    :SetQuestMark()
                    :Dialog("DIALOG_TELL_READY")
                    :CompleteQuest('start')
                    :Fn(function() cxt:GetAgent():MoveToLimbo() end)
                    :DoneConvo()
            end)                
            

        end)


QDEF:AddConvo("get_partner", "partner")
    :Loc{
        OPT_TRICK_GO = "Convince {partner} to follow you to the ambush",
        DIALOG_TRICK_GO = [[
            player:
                $miscNostalgic
                So, {partner}. We've had some good times, you and me. 
                But we gotta move on to bigger, better things, right?
                !overthere
                $miscAlluring
                And I got a little present for you, to show how much I care.

        {seemli?
            agent:
                !dubious
                $neutralThoughtful
                A gift? I'd rather not take anything from the upper class.
                Even if you're broke, you're noble-born. And I wouldn't want the other Rise to think I was accepting bribes.
            player:
                !placate
                Hey, no sweat. This is the kind gift that all the Rise can enjoy.
                You heard of a guy named Flekfis?
                !miscAlluring
                He's gonna be in town, and I've arranged a little rendezvous. 
        }

        {eden?
            agent: 
                !disgust
                $neutralWhatever
                I'd rather not have any evidence that we ever associated, when all this is over.
            player:
                !agree
                And that's the gift. Turns out, I got evidence Mullifee ain't so clean.
                She's my twin, after all. 
            agent:
                Hm. I'm listening.
        }
        ]],
        DIALOG_TRICK_GO_SUCCESS = [[

        {seemli?
            player:
                !hips
                $happyCocky
                Turns out, Flekfis and I know a few people in common. So it was easy enough to arrange a meet and greet.
            agent:
                !interest
                $happyTakenAback
                Wait, <i>the</i> Flekfis is here, in the Pearl? 
                $neutralThoughtful
                We haven't heard much out of Grout Bog lately. Perhaps this is why?
            player:
                $neutralDubious
                Time's a wasting, ya know. Not sure how long he'll wait.
            agent:
                $happyTakenAback
                !impressed
                O-of course! We'll go right away! 
                Lead on! 
}
        {eden?
            player:
                !cruel
                $miscMocking
                Funny thing, when your parents die you find out all sorts of dirty secrets.
                Turns out Mullifee isn't as self-made as she likes to claim. 
                !overthere
                I know exactly where she's gonna be, trying to clean up the mess our parents left for her.
            agent:
                !suspicious
                You mean we'll actually catch her in the act, hm?
                $neutralThoughtful
                I suppose that'll cover all the bases. Evidence of her misdeeds, and a reason to strike...
                !cruel
                <i>If</i> she resists arrest.
            player:
                $miscAlluring
                Pretty sure we can make that happen.
            agent:
                $miscMischievous
                Anything's possible. 
                Very well, Smith. Let's chase this plan of yours.
                Take me to your sister. Just don't expect me to tolerate any sentimentality when the time comes.
}
        ]],
        DIALOG_TRICK_GO_FAILURE = [[ 
        {seemli?
            player:
                !agree
                Yep. Flekfis himself, stalks and all. Saw him with my own two eyes.
            agent:
                !suspicious
                Wait, stalks? Flekfis is a jarackle. 
            player:
                $neutralTakenAback
                Right. But... he's such a good jarackle, I think of him like a kra'deshi, you know?
            agent:
                $neutralDirect
                No. I don't know.
                !angry
                And I'm not quite sure what to believe <i>you</i> know, either.
}
        {eden?
            player:
                !overthere
                Gambling debts. Huge ones. Bigger than mine!
            agent:
                !suspicious
                Is that right?
            player:
                Yep. She can't be stopped.
            agent:
                $neutralResigned
                You understand this is the Pearl, yes?
                Gambling is a requirement, not a vice.
            player:
                !bashful
                Not the way I do it, apparently.

}        ]],
    }
    :Hub(function(cxt) 
        cxt:BasicNegotiation("TRICK_GO")
            :SetQuestMark()
            :OnSuccess()
                :RecruitMember( PARTY_MEMBER_TYPE.ESCORT )
                :CompleteQuest("get_partner")
                :DoneConvo()
            :OnFailure()
                :FailQuest("get_partner")
                :DoneConvo()
    end)
        

QDEF:AddConvo("go_empty")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("ambush_spot") end)
        :Loc{
            DIALOG_INTRO = [[
                * Mullifee melts out of the darkness when she sees you're alone.
                mullifee:
                    !right
                * You'd like to say she looks disappointed, but...
                mullifee:
                    $neutralResigned
                    Failed, I see.
                player:
                    !left
                    $angryReluctant
                    !point
                    You see nothin', sis. I'm not here to get played by you.
                    You had me jumpin' through hoops just to get what's mine. 
                mullifee:
                    !disappoint
                    $angryAside
                    Vixmalli grew up bigger than the rest of us combined. Any sibling affection is dwarfed by his influence.
                    You've proven yourself enough of a bother that Vix wants you brought in and dealt with properly.
                    $neutralDirect
                    If you're not the ally I need, then I'll have to turn you over. 
            ]],
            OPT_CONVINCE_WORK = "Convince {agent} to work with you",
            DIALOG_CONVINCE_WORK = [[
                player:
                    !angry_shrug
                    $angryDefault
                    What influence, huh?! We <i>made</i> Vix. You, Roux—even me. 
                    Vix'd be nothing but pond scum if he hadn't had our shoulders to step on.
                    You wanna get one up on him? Work <i>with</i> me.
            ]],
            DIALOG_CONVINCE_WORK_SUCCESS = [[
                player:
                    !disappoint
                    $angryPatienceLost
                    I don't know what's gotten into the lot of you, but if Vix wants to cut me out so bad...
                    What says he won't do the same to you someday, huh?
                    At least you know I am what I am, even if you don't much like me.
                agent:
                    ...
                    !crossed
                    Don't be stupid. Of course I like you.
                    $neutralResigned
                    It's just never been about that. You didn't want to play the game, so we removed you from the board.
                    $neutralThoughtful
                    If you've changed your mind... then I suppose we did once make a good team.
                player:
                    !happy 
                    $happyTakenAback
                    Now you're talking, Fee. Knew you wouldn't let me dry out.
                agent:
                    $miscNostalgic
                    I suppose my sentimentality once again has gotten the best of me.

            ]],
            DIALOG_CONVINCE_WORK_FAILURE = [[
                player:
                    !angry
                    $scaredStammering
                    I'm twice the kra'deshi Vix is!
                agent:
                    $neutralResigned
                    That's simply not true. 
                    !fight
                    I'd ask you to come quietly, but of course we both know that's not in the cards.
            ]],

            OPT_DEFEND = "Defend yourself!",
            DIALOG_DEFEND = [[
                player:
                    !angryPatienceLost
                    Whatcha gonna do, Fee? Kill me?!
                agent:
                    That's up to you.
            ]],
            DIALOG_WON = [[
                {dead?
                    * You'd like Mullifee to get a proper burial, but it'll have to wait. There's no telling who might come across you like this.
                    * Somehow, you doubt anyone would believe you killed your own sister in self-defense.
                }
                {not dead?
                agent:
                    !injured
                    Huh. Not sure I recognized that fighting technique.
                player:
                    !tired
                    That's cause there isn't none. I just grit my teeth and hope I come out the other side alive.
                agent:
                    $neutralDubious
                    It's effective.
                    I suppose, for once, it's I who've disappointed <i>you</i>.
                player:
                    !disappoint
                    $neutralResigned
                    Just get outta my way. And keep your head down.
                    I didn't kill you, but you got enough folks who woulda.

                }
            ]],
        }
    :Fn(function(cxt)
        cxt:TalkTo("mullifee")
        cxt:Dialog("DIALOG_INTRO")
        cxt:BasicNegotiation("CONVINCE_WORK")
            :OnSuccess()
                :ReceiveOpinion("convinced_work", nil, "mullifee")
                :CompleteQuest()
                :Travel()

            :OnFailure()
                :Fn(function() 
                    cxt:Opt("OPT_DEFEND")
                        :Dialog("DIALOG_DEFEND")
                        :Battle{}
                            :OnWin()
                                :Dialog("DIALOG_WON")
                                :CompleteQuest()
                                :Travel()
                    end)


            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{}
                    :OnWin()
                        :Dialog("DIALOG_WON")
                        :CompleteQuest()
                        :Travel()
    end)

QDEF:AddConvo("do_trap")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("ambush_spot") end)
        :Loc{
            DIALOG_INTRO = [[
                * Whether it's impatience or doubt, Mullifee wastes no time springing her trap.
                * She slips into view like a predator, ready to spill blood.
                mullifee:
                    !right
            ]],

            OPT_SAVE_PARTNER = "Protect {partner}",
            DIALOG_SAVE_PARTNER = [[
                * Maybe you're sentimental. Or not sentimental enough.
                * But at the last minute, you can't go through with it. Blood be damned.
                player:
                    !left
                    !fight
                    $angryTakenAback
                    {partner}, look out! We been set up!
            ]],

            OPT_SAVE_PARTNER_BATTLE = "Defend {partner}",
            DIALOG_SAVE_PARTNER_BATTLE = [[
                mullifee:
                    !right
                    $neutralResigned
                    You always were unreliable, {player}.
                    !right
                    But no worry—I brought friends.
                    
            ]],
            

            DIALOG_WON_WITH_PARTNER = [[
                {mul_dead and partner_dead?
                    * When the dust settles, there's no one left to tell the tale. 'Cept you, of course. And you have a feeling you'll be changing the details on retell.
                player:
                    !dubious
                    $neutralResigned
                    The cheese stands alone. 
                    $angryReluctant
                    Won't be the last time, neither.
                }
                {mul_dead and not partner_dead?
                    * When the dust settles, you try not to look too hard at your sister's fallen body.
                    player:
                        !shrug
                        $miscEmbarrassed
                        Guess that didn't go exactly as planned, huh?
                    partner:
                        !right 
                    {seemli?
                        !tired
                        I never expected this evening to come to violence but...
                        $miscRelieved
                        Defeating Mullifee is a major coup. She was one of the most effective Admiralty officers in the region.
                        We can only hope Flekfis got away safely. 
                    player:
                        !bashful
                        $neutralTakenAback
                        Oh, uh, yeah. I'm sure he did. He's a ... stand-up guy, that Flekfis.
                    partner:
                        !permit
                        I doubt this was the ending you had in mind. But if there's anything I can do to help you in your own quest, let me know.
                        $neutralThanks
                        The Rise repay their debts. 
                        !point
                    * {partner} picks a piece of dust from your collar with propriety before disappearing.
                        !exit
                    }
                    {eden?
                        !dust_off
                        $miscRelieved
                        Bit of an upset at the start, sure, but we got the results we were hoping for.
                        Didn't even have to bait her into attacking first, luckily enough.
                    player:
                        !disappoint
                        $miscResigned
                        Yeah. Lucky.
                    partner:
                        !point
                        Now, I warned you I wouldn't tolerate any sentimentality. But if you need a moment with the body, I shan't wait up.
                        And if you lay claim to your inheritance after all, well...
                        !crossed
                        $miscAlluring
                        The Banquods are one of the most powerful families in the Pearl. I won't reject an alliance.
                    * You're not sure whether she expects a thank you, but she leaves before you can offer one.
                        !exit
                }
                }
                {not mul_dead and partner_dead?
                    * The dust settles, but tempers are still flying high.
                    mullifee:   
                        !right
                        !injured
                        $angryReluctant
                        Figures. I offer you a hand, and you drag me down into the mud with you.
                        Is this what you always had in mind? Working with my enemy?
                    player:
                        !angry_shrug
                        $angryCruel
                        {partner}'s dead, ain't she? What more do you want?
                    mullifee:
                        $angryPatienceLost
                        To know which side you're on!
                    player:
                        When I figure it out, you'll be the first to know.
                        !angry_accuse
                        Don't expect things to be chummy after what you did. I'm sore, and I've made it known.
                    mullifee:
                        Clearly. 
                        So what now? 
                    player:
                        $angryDefault
                        Just stay outta my way. I got one last score to settle, and it ain't got nothing to do with you anymore.
                    * Dismissed, Mullifee makes for the exit, leaving you to brood on your own.
                    mullifee:
                        !exit
                }
                {not mul_dead and not partner_dead?
                    mullifee:
                        !right
                        !injured
                        $angryAside
                        This is your problem, Smith. You only ever do things in half measures.
                    partner:
                        !right
                        !injured
                        $miscHushedUrgent
                        What is the meaning of this, exactly? 
                    {seemli?
                        $scaredTakenAback
                        Flekfis was never here, was he?
                        This was all a trap!
                    }
                    {eden?
                        !injured
                        $angryTakenAback
                        What's going on, Banquod?
                        Do you have a reasonable explanation for why the enemy we were supposed to ambush was instead waiting for <i>us</i>.
                    }
                }
            ]],

            
            OPT_ATTACK_PARTNER = "Side with {mullifee}",
            DIALOG_ATTACK_PARTNER = [[
                    player:
                        !left
                        !humouring
                        $miscMischievous
                        Been a long time since we fought back to back, huh?
                        Just make sure not to stab me in it.
                    mullifee:
                        $neutralWhatever
                        That would bring down my accuracy stats.
                    partner:
                        !right
                        $angryTakenAback
                        This was a trap?!
                    player:
                        !happy
                        Oh, yeah. Sorry about that.
                        $miscMocking
                        I woulda told you sooner, except that's not how traps work.
                        !fight
            ]],

            OPT_ATTACK_PARTNER_BATTLE = "Attack {partner}",
            DIALOG_ATTACK_PARTNER_BATTLE = [[
                partner:
                    !right
                    !angry
                    I'm smart enough not to trust a grifter, partner.
                    I had backup trail us.
                    $angryDefault
                    Now you're going to die!
            ]],

            DIALOG_WON_WITH_MULLIFEE = [[
                {mul_dead and partner_dead?
                    * You pat yourself down as the dust settles, checking for any lasting injuries.
                    player:
                        !dust_off
                        $miscRelieved
                        Woo, that was better than a high holy day at the bar. 
                        Y'all right, Fee?
                    * {mullifee} doesn't answer. You look down and see she sustained enough damage for the both of you.
                    player:
                        !scared 
                        $scaredTakenAback
                        Oh. 
                        $miscDepressed
                        Hesh, Fee. I didn't want it to go down like this.
                        $neutralDirect
                        This is all Vix's fault. And I'll make him pay, I swear.
                    * There's no time for a proper burial, so you make do with your promise, for now.
                }
                
                {mul_dead and not partner_dead?
                    * The dust settles, but not without a sacrifice.
                    partner:
                        !right
                        !injured
                    player:
                        !angry 
                        $angrySeething
                        You... you coward rat bastard! You killed my sister!
                    partner:
                        $angryTakenAback
                        <i>I'm</i> the coward?
                    {seemli?
                        $angryReluctant
                        This was all a trap! You played my good nature against me.
                        I never wanted this to come to violence, but you forced my hand.
                    }
                    {eden?
                        $angryCruel
                        You think I wouldn't come prepared? Of course this was a trap.
                        That's one thing you Banquods have in common—skulking incompetence.
                    }
                    player:
                        !angry_accuse
                        You don't like the way we play the game? Then stay out of it.
                        If I see you meddling anywhere in my family again, I'll use your head like a kickball.
                    partner:
                        !exit
                    * {partner} limps off, leaving you alone to clean up the body. You can't give Mullifee a proper burial, but you can at least make sure her collar's straight.
                }
                {not mul_dead and partner_dead?  
                    * Mullifee checks the body before nodding, once, in satisfaction.
                    mullifee:
                        !right
                        !agree
                        $neutralThoughtful
                        Good work. Cleaner than I expected.
                    player:
                        You lot underestimate me too much.
                        Whatcha gonna do now, anyway? You got your enemy outta the way, but it won't be long before you make more.
                        !shrug 
                        No offense, Fee, but you're kinda hard to get along with these days.
                        You used to be fun.
                    mullifee:
                        !crossed
                        $neutralDubious
                        And you used to be present. 
                        But you've done the favor you promised. And I'll return it in kind.
                        !interest 
                        $neutralThanks
                        When you approach Vixmalli, you can trust that I'll support you.
                    * It ain't much—not from this grown-up that's what's left of your twin sister—but it's a start.
                }
                {not mul_dead and not partner_dead?
                    mullifee:
                        !right
                    partner:
                        !left
                        !injured
                    * You've left {partner} alive enough to flinch as Mullifee approaches.
                    mullifee:
                        !dubious
                        $neutralDubious
                        This situation you've placed yourself in. Is it worth it?
                    {seemli?
                    partner:
                        $angryReluctant
                        F-freedom is always worth it.
                    mullifee:
                        $neutralResigned
                        Ah, but that's the one thing you've truly lost.
                    * Mullifee secures the cuffs around {partner}'s wrists, making sure to cinch them extra tight. 
                    }
                    {eden?
                    partner:
                        $angryCruel
                        Ask me again in a year, when you've run afoul of someone even higher up the chain than me.
                        You think this is over? You've attacked a superior officer. 
                    mullifee:
                        No, I've arrested a criminal. 
                        While you were busy babysitting, I searched your office.
                    * {partner}, for once, looks speechless, only wincing when Mullifee tightens the handcuffs.
                    }
                    player:
                        !left
                        !salute
                        $miscMocking
                        Mission accomplished, officer.
                    mullifee:
                        Yes. You showed a lot more restraint than I would expect.
                        $neutralThoughtful
                        If this is the same care you can bring to the table in family matters...
                        !agree
                        Then you have my support against Vixmalli. However events may unfold.
                    * You weren't sure you expected today to end with such a win. But here you are. 
                }
            ]],

            OPT_CONVINCE_PARTNER = "Convince {partner} that you didn't know about the ambush",
            DIALOG_CONVINCE_PARTNER = [[
                player:
                    !scared_shrug
                    $neutralTakenAback
                    Geez, that came outta nowhere, am I right?
            ]],
            DIALOG_CONVINCE_PARTNER_SUCCESS = [[
                player:
                    !bashful
                    $miscEmbarrassed
                    Now you see why there's bad blood with the rest of my family, huh?
                    Imagine coming home to three folks just like her, each of them wanting you out.
                {seemli?
                partner:
                    !permit
                    $miscNostalgic
                    My deepest sympathies, truly.
                    You're always welcome with the Rise. And when the time comes, the Rise pays its debts. Remember that.
                * {partner} picks a piece of dust from your collar with propriety before disappearing.
                    !exit
                }
                {eden?
                partner: 
                    !dust_off
                    $neutralWhatever
                    Yes, well. Not everyone sees the benefit of marching to your own beat.
                    I always have use for someone who thinks outside the box, Smith.
                    If you do end up back at your family's table, let me know, and I'll be sure to repay the favor.
                    !exit
                    * You're not sure whether she expects a thank you, but she leaves before you can offer one.
                }
            ]],
            DIALOG_CONVINCE_PARTNER_FAILURE = [[
                player:
                    !placate
                    $scaredStammering
                    Who knew that Mullifee would be waiting in ambush for you, one of her very worst enemies!
                partner:
                    !angry
                    ...
                    Family politics are your own business.
                    !angry_accuse
                    $angryAside
                    You stay away from me and mine, and hope that we don't meet again.
            ]],
        }
    :Fn(function(cxt)
        
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_ATTACK_PARTNER")
                :Dialog("DIALOG_ATTACK_PARTNER")
                :Fn(function() 

                    local backup = CreateCombatBackup(cxt.quest:GetCastMember("partner"), cxt.quest:GetCastMember("partner").combat_backup, cxt.quest:GetRank()-1)
                    table.insert(backup, 1, cxt.quest:GetCastMember("partner"))
                    cxt:Opt("OPT_ATTACK_PARTNER_BATTLE")
                        :Dialog("DIALOG_ATTACK_PARTNER_BATTLE")
                        :Battle{
                            no_oppo_limit = true,
                            allies = {"mullifee"},
                            enemies = backup
                        }
                            :OnWin()
                                :Fn(function() 
                                    cxt.enc.scratch.mul_dead = cxt:GetCastMember("mullifee"):IsDead()
                                    cxt.enc.scratch.partner_dead = cxt:GetCastMember("partner"):IsDead()
                                end)
                                :Dialog("DIALOG_WON_WITH_MULLIFEE")
                                :ReceiveOpinion("helped_them", nil, "mullifee")
                                :CompleteQuest()
                                :Travel()
                end)
            
            cxt:Opt("OPT_SAVE_PARTNER")
                :ReceiveOpinion("betrayed_deal", nil, "mullifee")
                :Dialog("DIALOG_SAVE_PARTNER")
                :Fn(function() 

                    local backup = CreateCombatBackup(cxt.quest:GetCastMember("mullifee"), "ADMIRALTY_PATROL", cxt.quest:GetRank()-1)
                    table.insert(backup, 1, cxt.quest:GetCastMember("mullifee"))
                    cxt:Opt("OPT_SAVE_PARTNER_BATTLE")
                        :Dialog("DIALOG_SAVE_PARTNER_BATTLE")
                            :Battle{ 
                                no_oppo_limit = true,
                                flags = BATTLE_FLAGS.SELF_DEFENCE,
                                allies = {"partner"},
                                enemies = backup
                            }
                                :OnWin()
                                    :Fn(function() 
                                        cxt.enc.scratch.mul_dead = cxt:GetCastMember("mullifee"):IsDead()
                                        cxt.enc.scratch.partner_dead = cxt:GetCastMember("partner"):IsDead()
                                    end)
                                    :Dialog("DIALOG_WON_WITH_PARTNER")
                                    :Fn(function() 

                                        if cxt.quest:GetCastMember("mullifee"):IsAlive() and cxt.quest:GetCastMember("partner"):IsAlive() then
                                            cxt:TalkTo("partner")
                                            cxt:BasicNegotiation("CONVINCE_PARTNER", {hinders = {"mullifee"} })
                                                :OnSuccess()
                                                    :ReceiveOpinion("saved_life", nil, "partner")
                                                    :CompleteQuest()
                                                    :Fn(function() cxt:GetCastMember("mullifee"):Retire() end)
                                                    :Travel()
                                                    --you get partner as an ally
                                                :OnFailure()
                                                    :ReceiveOpinion("set_up", nil, "partner")
                                                    :CompleteQuest()
                                                    :Fn(function() cxt:GetCastMember("mullifee"):Retire() end)
                                                    :Travel()
                                                    --partner hates you
                                        else
                                            if cxt.quest:GetCastMember("partner"):IsAlive() then
                                                cxt:Opinion("saved_life", "partner")
                                            end
                                            cxt.quest:Complete()
                                            if cxt:GetCastMember("mullifee"):IsAlive() then
                                                cxt:GetCastMember("mullifee"):Retire()
                                            end
                                            StateGraphUtil.AddLeaveLocation(cxt)
                                        end
                                        
                                    end)                
                    end)

    end)

