require "ui/screens/WIPpopup"

local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    title = "The Marriage Contract",
    icon = engine.asset.Texture("icons/quests/smith_story_marriage_contract.tex"),

    on_init = function(quest) 
        quest:AssignCastMember("suitor", quest.param.parent_quest:GetCastMember("suitor"))
        quest.param.boss_def = quest.param.boss_def or TheGame:GetGameProfile():GetNoStreakRandom("SMITH_DAY_2_BOSS_PICK", {"MARK_NINE_NINE", "TWIN_BOSS_1"}, 2)
    end,

    on_destroy = function(quest)

        if quest:GetCastMember("suitor") and quest:GetCastMember("suitor"):IsInPlayerParty() then
            quest:GetCastMember("suitor"):Dismiss()
        end
    end,
}


:AddObjective{
    id = "start",
    title = "Meet up with {suitor}",
    desc = "{suitor} is going to meet you at {moreef}'s bar.",
    state = QSTATUS.ACTIVE,
    mark = { "bar" },

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("suitor"), location =  quest:GetCastMember("bar"), role = CHARACTER_ROLES.VISITOR} )
    end,
    on_complete = function(quest)
        quest:Activate("go_to_the_chapel")
    end,

}

:AddObjective{
    id = "go_to_the_chapel",
    title = "Get to the chapel",
    desc = "Get {suitor} to the marriage chapel in one piece.",
    mark = {"chapel"},
    on_complete = function(quest)
        quest:Activate("sleep")
    end,
}

:AddObjective{
    id = "sleep",
    title = "Head back to the bar to sleep",
    desc = "",
    mark = {"moreef"},
    on_activate = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true

        if quest:GetCastMember("suitor") and quest:GetCastMember("suitor"):IsInPlayerParty() then
            quest:GetCastMember("suitor"):Dismiss()
        end

    end,
    on_complete = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
    end,
}



:AddLocationCast{
    cast_id = "bar",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("MOREEF_BAR"))
    end,
}

:AddLocationCast{
    cast_id = "chapel",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("CONTRACT_CHAPEL"))
    end,
}

:AddCastByAlias{
    cast_id = "mullifee",
    alias = "MULLIFEE",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "theroux",
    alias = "THEROUX",
    no_validation = true,
}

:AddCast{
    cast_id = "suitor",
    no_validation = true,
    when = QWHEN.MANUAL,
    on_assign = function(quest, agent)
        if agent:GetContentID() == "DENIKUS" then
            quest.param.denikus = true
        else
            quest.param.andwanette = true
        end

    end
}

:AddCastByAlias{
    cast_id = "seemli",
    alias = "SEEMLI",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "eden",
    alias = "EDEN",
    no_validation = true,
}


:AddCast{
    cast_id = "bodyguard",
    when = QWHEN.MANUAL,
    no_validation = true,
    cast_fn = function(quest, t)
        quest.param.boss_def = quest.param.boss_def or TheGame:GetGameProfile():GetNoStreakRandom("SMITH_DAY_2_BOSS_PICK", {"MARK_NINE_NINE", "TWIN_BOSS_1"}, 2)
        table.insert( t, quest:CreateSkinnedAgent( quest.param.boss_def ) )
    end,
}


:AddCastByAlias{
    cast_id = "moreef",
    alias = "SWEET_MOREEF",
}


:AddOpinionEvents{

    abandoned =  
    {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Abandoned them to their enemies",
    },
    altered_deal = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Didn't hold up your end of the bargain to kill your brother",
    },
    tried_to_alter_deal = {
        delta = OPINION_DELTAS.DISLIKE,
        txt = "Tried to change the terms of your deal",
    },
    
    ruined_special_day = {
        delta = OPINION_DELTAS.MAJOR_BAD,
        txt = "Ruined their special day!",
    },

    reconciled = {
        delta = OPINION_DELTAS.LIKE,
        txt = "You reconciled your differences",

    },
    made_rich_widow = {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Made them into a rich widow",

    }
    
}


QDEF:AddConvo("start", "moreef")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                {denikus?
                * You find {suitor} leaning over the bar, talking to an increasingly frazzled {moreef}. 
                moreef:
                    !right
                suitor:
                    !left
                    !permit
                    $neutralResigned
                    I gave you twenty shills to break into smaller chips, <i>and</i> the shills to pay my tab, but you only gave me the change back--
                moreef:
                    !angry
                    $angryTakenAback
                    N-no, I gave you what you're owed! I don't think you gave me the twenty?
                player:
                    !left
                    Don't give him nothin', Sweets. 
                    I've whistled that tune before and it's not as catchy as he thinks.
                suitor:
                    !right
                    !humouring
                    $neutralWhatever
                    I'm getting married tonight, you know. Least you could do is get me drunk first. 
                }
                {andwanette?
                    * You find {suitor} perched on a bar stool looking like a preening seabird.
                moreef:
                    !right
                suitor:
                    !left 
                    !dust_off
                    $miscAlluring
                    Of course, I can't even think of the last time I actually paid for a drink.
                    Normally my hosts are outbidding each other for the chance to serve me.
                moreef:
                    !dubious
                    $neutralResigned
                    Well, that's all well and good, {suitor}, but I should hope a woman of your stature would be able to pay her tab...
                player:
                    !left 
                    Sorry, Sweets, but {suitor} here isn't used to slumming it. 
                    Ask her for an autograph. You can sell it later.
                suitor:
                    !right
                    !happy
                    $happyCocky
                    I'll be sure not to personalize it, then.
                }
            ]],
            QUESTION_DENIKUS = "Ask about the ceremony",
            DIALOG_ANSWER_DENIKUS = [[
                player:
                    !point
                    Whatever game Theroux's playing, he expects you to be on the level.
                    You can drink when it's over. 
                suitor:
                    !thought
                    $neutralDubious
                    I've never married a kra'deshi before. Is drinking part of the ceremony?
                player:
                    !agree
                    $miscRelieved
                    Oh, yeah. First you sign the forms in triplicate, then you down the rest of the ink in one go.
                    It'll mess you up good but you'll see in pretty colors for the rest of the night.
                suitor:
                    !dubious 
                    $neutralDubious
                    Are you having me on?
                player:
                    !hips
                    Nope. If you don't wanna drink squid juice, just make your signature real big and loopy.
                    You ready?
                suitor:
                    !overthere
                    Never enough time to be anything else. After you.

            ]],
            QUESTION_ANDWANETTE = "Ask about the ceremony",
            DIALOG_ANSWER_ANDWANETTE = [[
                player:
                    !dubious 
                    $neutralDubious
                    You look... exactly the same. Thought you had to get ready?
                suitor:
                    $miscAlluring
                    Emotionally, my dear Smith. Perhaps not a facet of being you're familiar with.
                    Now, is there anything I should know? Preferences Theroux might have?
                    !hips
                    I aim to seem perfect, but not <i>too</i> perfect, lest it make him suspicious.
                player:
                    !forgetful
                    $neutralTakenAback
                    Uh, he likes... himself? And... people who make him feel good about himself?
                suitor:
                    !suspicious
                    $neutralDubious
                    I suspect you mean that to be a vice, but I'd feel sorry for the person who <i>didn't</i> like those things.
                    $miscAlluring
                    All the same, I'll widen my eyes so he sees himself reflected within.
            ]],
            OPT_HEAD_OUT_WITH_SUITOR = "Take {suitor} to the chapel",
            DIALOG_HEAD_OUT_WITH_SUITOR = [[
                player:
                    Alright. Say goodbye to your old life. You're about to get Banquod.
                suitor:
                    !salute
                    Goodbye, old life.
                    !exit
                player:
                    !exit
                moreef:
                    !right
                    !dubious
                    $neutralDubious
                    ... Bye?
                    !exit
            ]],
        }
        :Fn(function(cxt) 


            cxt:Dialog("DIALOG_INTRO")
            cxt:RunLoopingFn(function() 
                if cxt.quest.param.denikus then
                    cxt:Question("QUESTION_DENIKUS", "DIALOG_ANSWER_DENIKUS")
                else
                    cxt:Question("QUESTION_ANDWANETTE", "DIALOG_ANSWER_ANDWANETTE")
                end

                cxt:Opt("OPT_HEAD_OUT_WITH_SUITOR")
                    :SetQuestMark()
                    :Dialog("DIALOG_HEAD_OUT_WITH_SUITOR")
                    :Fn(function() 
                        cxt.quest:GetCastMember("suitor"):Recruit(PARTY_MEMBER_TYPE.ESCORT)
                    end)
                    :DoneConvo()
                    :CompleteQuest("start")
            end)

        end)



----------------- A good ol' travel interrupt. The encounter is different, depending up on the suitor.

QDEF:AddConvo("go_to_the_chapel")
    :TravelConfront("STATE", function(cxt) return not cxt.quest.param.did_travel_event end)
        :Loc{
            DIALOG_INTRO = [[
                * The route to the magistrate's office is familiar enough to you, what with being dragged in and out of it enough times in your disgruntled youth.
                * So with your feet going on muscle memory, your mind is free to wander.
                * Theroux was always annoying—you can easily picture his smug glee when he heard you were being cast out, happy enough to be elevated on his own.
                * You've spent your whole life waiting for a chance to clock him and get away with it. But that was just to rough him up a little.
                * Killing him's a different story.
                agent:
                    !right
                    Hey, bud, you better watch where you're going.
                player:
                    !left
                    !dubious
                    $neutralWhatever
                    Huh? Oh. Sorry.
                    Got something on my mind. Nothing personal.
                suitor:
                    !left
                    !placate
                    I think it probably <i>is</i> personal, Smith.
                
                {denikus?
                    agent:
                        !angry 
                        $angryAside
                        Should've kept laying low, Cassando. Or—what are you calling yourself now? Denikus?
                        I wasn't sure I had your trail. Maybe another month or two, I might've given up.
                        But you had to get greedy and betrothe yourself to the only person in Havaria more greasy than you.
                    suitor:
                        $neutralResigned
                        What is it you want, {agent}? Shills?
                        !shrug
                        This time tomorrow I'll have five times what you gave me. If you want me to pay you back, it'll be simple.
                    agent:
                        !angry_accuse
                        You broke my heart, and you offer me shills?
                        I would've paid twice that much to get you to stay.
                    player:
                        !left
                        !dubious
                        $neutralTakenAback
                        Shuckin' Hesh, are you serious?
                        He's a two-bit con-man! I could floss my teeth with his act. 
                    agent:
                        And who are you? Hired muscle?
                        !crossed
                        I'll kill you, too, if I have to. 
                }
                {andwanette?
                    agent:
                        !cruel
                        $miscMocking
                        'Fraid so, Andwanette. You got quite the bounty on you, you know that?
                    suitor:
                        !hips
                        I should hope so. Though I suppose I was rather insulted it was so low.
                        I may be self-made, but my work is artisanal. 
                    player:
                        !left
                        !placate
                        $neutralTakenAback
                        Wait, wait, wait. Hold up. What bounty?
                    agent:
                        Your friend's got enemies, pal. Nobody likes an upstart.
                        Some Murder Bay debt-broker's got big plans to throw her out with the bathwater. 
                        !cutthroat
                        Maybe ship her to a derrick, where she won't be a thorn no more.
                }
            ]],

            OPT_NEGOTIATE = "Convince {agent} to let {suitor} pass",
            DIALOG_NEGOTIATE = [[
                player:
                    !crossed
                    You maybe want to think again, there, stranger.
                    Theroux Banquod isn't the sort of kra'deshi you take on lightly.
            ]],
            DIALOG_NEGOTIATE_SUCCESS = [[
                player:
                    !shrug
                    He may be a slimy suck-up, but that's my baby brother, and neither of us like to lose.
                    I dunno what he sees in {suitor}, here, but he's decided he wants to get hitched, and it's my job to get {suitor} to the magistrate.
                    Now you might feel like messing with one Banquod. But two?
                {denikus?
                agent:
                    !angry 
                    $angryReluctant
                    I only want {suitor} dead. I don't take issue with you, or your brother.
                    ... but it won't be long before Theroux feels the same way as I do.
                    !cruel
                    $miscMocking
                    And when that happens? I'll know exactly where to find you, <i>my love</i>.
                    !exit
                * With an ominous gesture, {agent} retreats, leaving you in the company of a very unconcerned-looking {suitor}.
                suitor:
                    !right
                    $miscMischievous
                    Well, well, well. Maybe there is a bit of the romantic in you after all.
                player:
                    !overthere
                    Bit rich coming from a guy who messes his marks up <i>that</i> bad.
                suitor:
                    $miscAlluring
                    You know why my performance is always so convincing?
                    Because I make myself believe it. Just a bit. 
                    The way you talked about Theroux, there... what's your excuse?
                player:
                    !handwave
                    $angryAside
                    Ah, shaddup and keep walking. I wanna get this over with so I can see the last of you.
                    !exit
                }

                {andwanette?
                agent:
                    !dubious
                    $neutralTakenAback
                    So the rumors <i>were</i> true. Hesh. 
                    I don't wanna tangle with a dynasty. If she's marrying up, then it'll take someone harder than me to claim the bounty.
                    !suspicious
                    Congrats on your nuptials, upstart. You better hope that'll be enough to protect you.
                    * Shaking {agent.hisher} head, {agent} slinks back into the shadows, leaving you to escort Andwanette to safety.
                suitor:
                    !right
                    !interest
                    $miscAlluring
                    Such passion! My, I didn't know you had it in you, Smith.
                    If you can summon that pathos once Theroux is dead, then this might be even more fun than I'd hoped.
                player:
                    !sigh
                    Let's keep going, huh?
                    Beginning to sting, realizing I don't got any siblings in my corner.
                suitor:
                    We'll sort that out, darling. 
                }
                
            ]],
            DIALOG_NEGOTIATE_FAILURE = [[
                player:
                    !crossed
                    You want to get in the way of kra'deshi marriage contracts? 
                    That's sabotage, and the courts don't look too kindly on it. 
                    At worst you'll have to pay damages. And at best? Well... we're a spiteful bunch.
                agent:
                    $miscMocking
                    I'll take my chances. Somehow I'm not worried about anyone noticing or caring <i>too</i> much.
            ]],
            OPT_DEFEND = "Defend {suitor}",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    Fine. I could use a good fight to help me get my head on straight.
            ]],
            DIALOG_SUITOR_DEAD = [[
                * Despite your efforts, {suitor} is dead. And you don't think your ventriloquism is up to the task of fooling Theroux for as long as a contract takes to finalize.
                * You'll have to do your best on your own to get close, and do what needs doing.    
            ]],
            DIALOG_WON = [[
                {dead?
                    * {agent} is dead, leaving you both looking only lightly dissheveled.
                }
                {not dead?
                    agent:
                        !injured
                    player:
                        This plan of yours? Best reconsider.
                    agent:
                        $angryReluctant
                        This isn't over. Only one of you Banquods has any real clout, and it's not you.
                    player:
                        Yeah? Your skull says otherwise.
                    agent:
                        !exit
                    * {agent} retreats as quick as {agent.hisher} injuries allow. 
                }
                suitor:
                    !right
                    Well, that was certainly dramatic.
                    !dust_off
                    Do I still look fit to be married?
                player:
                    !shrug
                    Yeah, yeah. A real peach. 
                    Let's keep going, huh?
            ]],

            OPT_ABANDON = "Abandon {suitor}",
            DIALOG_ABANDON = [[
                player:
                    $neutralResigned
                    This officially just got to be more trouble than you're worth.
                    !dubious
                    Sorry, {suitor}. If I'd known you were this much of a liability, I coulda saved you some paperwork.
                suitor:
                    !right
                {denikus?
                suitor:
                    !interest
                    $neutralTakenAback
                    You're just going to leave me here? At {agent}'s mercy?
                    That's cold, man. I didn't take you for being that ruthless.
                player:
                    !shrug
                    Then you seriously misread the room, guy.
                }
                {andwanette?
                suitor:
                    !interest
                    $scaredTakenAback
                    Y-you can't be serious! After all we've gone through?
                    And I swore to be your loyal sister!
                    $angryCruel
                    No wonder your siblings cast you out. It's all transparent, now.
                player:
                    !humouring
                    Have fun on the derrick, Andwanette.
                    $miscMischievous
                    If it's any consolation, the value of your autographs will probably go up.
                }
                * You leave before you have to witness the ugly aftermath. 
                player:
                    !exit
                * Now you just have to think of what you'll say to get close to Theroux...
            ]]
        }
        :Fn(function(cxt) 
            cxt.quest.param.did_travel_event = true

            local opfor =  CreateCombatParty(cxt.quest.param.denikus and "ADMIRALTY_PATROL" or "HESH_PATROL", cxt.quest:GetRank(), cxt.location)
            cxt:TalkTo(opfor[1])
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function() 
                cxt:Opt("OPT_NEGOTIATE")
                    :Dialog("DIALOG_NEGOTIATE")
                    :Negotiation{
                        helpers = {"suitor"}
                    }
                        :OnSuccess()
                            :Dialog("DIALOG_NEGOTIATE_SUCCESS")
                            :Travel()
                        
                        :OnFailure()
                            :Dialog("DIALOG_NEGOTIATE_FAILURE")

                cxt:Opt("OPT_DEFEND")
                    :Dialog("DIALOG_DEFEND")
                    :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE,
                        allies = {"suitor"},
                        on_run = function(cxt, battle)
                            cxt.quest.param.suitor_abandoned = true
                            cxt.quest:GetCastMember("suitor"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("abandoned"))
                            StateGraphUtil.DoRunAwayNoFail(cxt, battle)
                        end
                    }
                        :OnWin()
                            :Fn(function() 
                                if cxt.quest:GetCastMember("suitor"):IsDead() then
                                    cxt.quest.param.suitor_dead = true
                                    cxt:Dialog("DIALOG_SUITOR_DEAD")
                                else
                                    cxt:Dialog("DIALOG_WON")
                                end

                            end)
                        :Travel()

                cxt:Opt("OPT_ABANDON")
                    :Dialog("DIALOG_ABANDON")
                    :ReceiveOpinion("abandoned", nil, "suitor")
                    :Fn(function() 
                        cxt.quest.param.suitor_abandoned = true
                        cxt.quest:GetCastMember("suitor"):Dismiss() 
                    end)
                    :Travel()

                end)

        end)



QDEF:AddConvo("go_to_the_chapel")

    :ConfrontState("STATE", function(cxt) return cxt.location == cxt.quest:GetCastMember("chapel") end)
        :Fn(function(cxt) 
            if cxt.quest:GetCastMember("suitor"):IsInPlayerParty() then
                cxt:GoTo("STATE_SUITOR")
            else
                cxt:GoTo("STATE_NO_SUITOR")
            end
        end)
    
    :State("STATE_SUITOR")
        :Loc{
            DIALOG_INTRO = [[
                * The magistrate's office is calm and quiet--seems Theroux might've booked it off specifically for the signing. 
                * You stay out of sight once your brother appears, hiding in the vestibule and poking one eye out to watch.
                * Theroux looks... happy. The happiest you've ever seen him, to be honest. He's bouncing on his heels as he reaches his hands out to {suitor}'s, taking them in his own.
                * The magistrate notarizes the paperwork and seals it inside a metal tube before leaving the newlyweds alone to contemplate their future together.
                theroux:
                    !left
                    !happy
                    $happyAmazed
                    I can't tell you how thrilled I am to meet you, and thank you for agreeing to this frankly <i>rushed</i> courtship.
                suitor:
                    !right
                    !happy 
                    $miscAlluring
                    Well, I certainly can't complain.
                theroux:
                    Oh, I'm sure you can. Especially when you meet my family. I'd disown all of them if I could.
                player:
                    !left
                    Yeah... speaking of.
                theroux:
                    !right
                    !surprise
                    $neutralTakenaback
                    S-Smith?! 
                    !angry
                    I should've known you'd show up. You can't stand to have any crisis not be your making.
                player:
                    !angry
                    $neutralWhatever
                    Don't quite get what that means... but if there's a party, I'll crash it. 
                theroux:
                    !angry_shrug
                    Nobody wants you here! Just go back to whatever lousy dive you call home and let the parasites eat you.
                player:
                    $angryPatienceLost
                    This <i>is</i> my home, you brat! I only left 'cause the lot of you made me feel like I didn't belong!
                    !hips
                    But that's gonna change. Because I'm gonna get back what's mine. 
                * To your surprise, Theroux throws out an arm to shield {suitor}. It catches you off-guard enough to give him time to call for back-up.
                theroux:
                    !scared
                    $scaredAnguished
                    Assassin! Thief! Someone help!
                bodyguard:
                    !right
                * Turns out when you're rich and powerful, there's always "someone" on hand.
            ]],

            DIALOG_DEFEND = [[
                player:
                    !fight
                    First, I'm gonna smash this guard of yours into the filigreed floor.
                    Then, baby brother, you and I are gonna have words. And they won't be nice.
            ]],

            DIALOG_TALK_TO_THEROUX = [[
                * All that's left is Theroux, trembling in his very expensive boots. {suitor} stands by, watching you closely.
                theroux:
                    !right
                    !angry
                    $scaredTakenAback
                    Well, you've done what you always do.
                    So what next, Smith? You kill me, and then what?
                    Vix, if you can even get to him? And <i>Mullifee</i>? I'd like to see you try.
            ]],



            OPT_KILL = "Kill {theroux}",
            DIALOG_KILL = [[
                player:
                    !angry
                    Thing is, Theroux—you disowned me.
                    So that means we're not even actually brothers anymore.
                theroux:
                    !exit
                * It doesn't feel good, but he does go down easy.
                suitor:
                    !right
                    !dubious
                    Romance really is dead, I see.
                player:
                    !neutral
                    Like that's what you signed on for?
                
                {denikus?
                    suitor:
                        I dunno. Hard not to think, after what happened earlier.
                        !shrug
                        Theroux seemed nice enough. Maybe I coulda stuck around.
                    player:
                        Maybe you ought to. With that old flame of yours after you... no offense, but you could probably use more friends than you got.
                    suitor:
                        $neutralThoughtful
                        Hm. I thought you wanted me gone? Now you offer protection?
                    player:
                        Maybe I'm just pragmatic, yeah? 
                        I'm down a brother. I could use a stand-in.
                    suitor:
                        $neutralDubious
                        Speaking of... how are you going to frame this?
                    player:
                        !overthere
                        $miscMocking
                        He came at me, didn't he? I just wanted to wish him well on his wedding.
                    suitor: 
                        I'll do my best to sell it. But not sure the excuse matches the mark.
                        !exit 
                }
                {andwanette?
                    suitor:
                        Indeed. I'm not usually standing up close when my plans come to fruition.
                    player:
                        !crossed
                        You usually get your friends to do your dirty work, right?
                    suitor:
                        !interest
                        $miscAlluring
                        Oh, darling, people like me don't have friends. Just... temporary alliances of convenience.
                    player:
                        $neutralDubious
                        Speaking of such, we're solid now, right?
                    suitor:
                        Assuming our interests continue to align? Yes.
                        !eureka
                        Solid as the ground that catches the blood we spill together.
                    player:
                        And you'll make sure to tell everyone it was self-defence, right? Theroux came at me?
                    suitor:
                        !agree
                        Of course. I'll come up with something far more interesting than that, besides.
                    player:
                        !happy
                        Good luck with that!
                        !exit
                }
            ]],

            OPT_RECONCILE = "Reconcile with {theroux}",
            DIALOG_RECONCILE = [[
                player:
                    What was I supposed to do, huh? You lot wouldn't even <i>see</i> me. 
                    !overthere
                    Just kicked me out and had some Admiralty stooge tell me I wasn't welcome.
                    If you wanna talk it out, let's talk it out.
            ]],
            DIALOG_RECONCILE_SUCCESS = [[
                player:
                    I mean, come on, slugger. All three of you cut me out? So easy?
                theroux:
                    $angryReluctant
                    You never made anything easy for us.
                    It took me ages to prove I wasn't just another Smith, all bang and no flash.
                    !eureka
                    But the more success I had, the more Vixmalli took it under control. 
                    He said it was a reserve for when <i>you</i> inevitably dragged us through the silt yet again. 
                player:
                    !shrug 
                    So you got rid of me? In exchange for, what, a ten-year marriage contract?
                theroux:
                    !wince
                    $miscEmbarrased
                    ... Four, actually.
                player:
                    $miscMocking
                    Hah! You that desperate?
                theroux:
                    Believe it or not, I might actually <i>like</i> to be married.
                    It's lonely work, being under Vix's thumb and Mullifee's notice. 
                    You have your freedom, at least. 
                player:
                    !suspicious
                    $miscHushedUrgent
                    Then how about this: <i>help me get back in</i>. Get me reinstated.
                    You do that, and I'll fight Vix slime and stalk to help you out, huh?
                theroux:
                    !scared_shrug
                    $neutralTakenAback
                    I... suppose I would like this marriage to last longer than four years.
                player:
                    Right. Uh. About that...
                {denikus?
                    suitor:
                      !right 
                        !dubious
                        $neutralResigned
                        Oh, don't mind me. This isn't even the most interesting wedding I've ha—ah, <i>attended</i>.
                        Just regretting some of the negotiations I neglected to lean in on. 
                        But a lot can happen in four years, I suppose.
                    player:
                        $neutralDubious
                        I, uh, guess?
                    * You're not sure what to make of this turn of events, but at least they make a cute couple.
                }
                {andwanette?
                    suitor:
                        !right 
                        !angry 
                        $angryAside
                        Yes. <i>About that</i>. Am I to call you <i>brother</i>, dear? After such open hostility towards my.. <i>beloved husband</i>?
                    player:
                        !scared_shrug
                        $miscEmbarrassed
                        It's, uh, nice to see you're so protective of him, yeah?
                        Maybe you can help him stand up to the others at family meetings.
                    * Andwanette smiles devotingly at Theroux, but the second his back is turned her needlepoint gaze turns on you.
                    * The curses she mouths silently in your direction aren't very sisterly, but then, you've never been very brotherly.
                }
            ]],
            DIALOG_RECONCILE_FAILURE = [[
                player:
                    !angry
                    $angryDefault
                    You left me out of the will!
                agent:
                    !angry_accuse
                    $angryPatienceLost
                    You spent more than your inheritance on gambling debts and bribes! 
                    I should know, I managed the family ledgers. Eleven hundred shills to pay off a single bar tab. 
                    Fourteen <i>thousand</i> to expedite forged diplomatic papers. 
                    $angryCruel
                    If you came here expecting me to grovel for forgiveness, I'm afraid you've vastly overestimated your own worth.
                player:
                    !threaten
                    $angrySeething
                    Why, you snot-nosed, superior little—

                * You weren't sure what insult could carry your anger, but it doesn't matter—{suitor} sticks him with his own bodyguard's blade, and Theroux slumps lifelessly to the floor.
                agent:
                    !exit
                {denikus?
                    suitor:
                        !right
                        $neutralDirect
                        There. Problem solved.
                    player:
                        !surprise
                        $scaredStammering
                        I—what—
                        $neutralTakenAback
                        I thought you didn't kill?
                    suitor:
                        !shrug 
                        $neutralResigned
                        You saved my life back there. Figured I owed you more than I'd promised.
                        But, uh, you won't be seeing me again after this.
                        And my name? You don't know it.
                    player:
                        !forgetful
                        $neutralTakenAback
                        Right... yeah. I, uh... yeah.
                        * {suitor} leaves, and you're left staring at the body.
                }
                {andwanette?
                    suitor:
                        !right
                        $neutralTakenAback
                        Hesh's bell, Smith! I thought we had an arrangement.
                        If you hadn't fumbled it so completely I could've orchestrated my reaction for the best coverage.
                        !dust_off
                        $neutralResigned
                        Now look—there's blood on me!
                    player:
                        !shrug
                        Nobody told you to stab him! You said you weren't gonna stab him!
                    suitor:
                        Yes, but it was clear <i>you</i> weren't going to, either.
                        $miscAlluring
                        And since you saved me from those bounty hunters, I was feeling sentimental.
                        Don't expect that kind of preferential treatment again. We may be siblings now, but I'm <i>very</i> displeased with you.
                        !crossed
                        I imagine I'll find it much harder to covincingly argue to the others that you should be reinstated.
                        Especially since you <i>killed</i> my husband in a duel for my heart.
                    player:
                        !placate
                        H-hey, wait just a minute!
                    suitor:
                        Now as your loving sister, I'll give you time to leave before I report this crime. 
                        !angry_accuse
                        But do not contact me again. It will only mean trouble for you.
                    * You don't always make the best decisions when you're confused, so you make for the door before you do something you'll regret.
                    }
            ]],

        }
    
        :Fn(function(cxt) 
            
            cxt.quest:AssignCastMember("bodyguard")
            local enemies = {cxt.quest:GetCastAgent("bodyguard")}
            
            cxt:TalkTo("bodyguard")
            cxt:Dialog("DIALOG_INTRO")

            cxt.quest:UnassignCastMember("bodyguard" ) -- This allows this agent to become Hated via Murder, as is the cases for TWIN_BOSS.
            
            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    noncombatants = {cxt.quest:GetCastAgent("suitor"), cxt.quest:GetCastAgent("theroux")},
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.BOSS_FIGHT,
                    enemies = enemies,
                }
                    :OnWin()
                        :Fn(function() 
                            ConvoUtil.GiveBossRewards(cxt, GRAFT_TYPE.COMBAT)
                            cxt:TalkTo("theroux")
                            cxt:Dialog("DIALOG_TALK_TO_THEROUX")

                            cxt:RunLoop(function(cxt) 
                                
                                    --cxt:Question("OPT_THEROUX_Q_1", "DIALOG_THEROUX_Q_1")
                                    --cxt:Question("OPT_THEROUX_Q_2", "DIALOG_THEROUX_Q_2")

                                    cxt:Opt("OPT_KILL")
                                        :Dialog("DIALOG_KILL")
                                        :Fn(function() 
                                            AgentUtil.CommitMurder(cxt:GetAgent())
                                            cxt:GetAgent():Kill()
                                            cxt.quest:GetCastMember("suitor"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("made_rich_widow"))
                                        end)
                                        :CompleteQuest("go_to_the_chapel")
                                        :Travel()

                                    cxt:Opt("OPT_RECONCILE")
                                        :Negotiation{suppressed = {cxt.quest:GetCastMember("suitor")}}
                                            :OnSuccess()
                                                :Dialog("DIALOG_RECONCILE_SUCCESS")
                                                :CompleteQuest("go_to_the_chapel")
                                                :ReceiveOpinion("reconciled")
                                                :Fn(function() 
                                                    cxt.quest:GetCastMember("suitor"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("altered_deal"))
                                                end)

                                                :Travel()
                                            
                                            :OnFailure()
                                                :Dialog("DIALOG_RECONCILE_FAILURE")
                                                :Fn(function() 
                                                    AgentUtil.CommitMurder(cxt:GetAgent())
                                                    cxt:GetAgent():Kill()
                                                    cxt.quest:GetCastMember("suitor"):OpinionEvent(cxt.quest:GetQuestDef():GetOpinionEvent("tried_to_alter_deal"))
                                                end)
                                                :CompleteQuest("go_to_the_chapel")
                                                :Travel()
                                end)
                        end)
        end)


    :State("STATE_NO_SUITOR")
        :Loc{
            DIALOG_INTRO = [[
                * You get to the magistrate's office and let yourself in. 
                theroux:
                    !right
                    !surprise
                    S-Smith?!
                    $neutralTakenaback 
                    What are you doing here? 
                player:
                    !left
                    !shrug 
                    $miscMocking
                    What, I can't come to wish my baby brother the best of joy on his wedding day?
                theroux:
                    How would <i>you</i> know anything about that?
                    My betrothed isn't even here yet. And I want you gone before they—
                * His eyes grow dull as he puts two and two together.
                theroux:
                    !suspicious
                    $neutralWhatever
                    They're... not coming. Because it was just a trick.
                player:
                    !chuckle
                    Never thought I'd see a flash of smarts go through that brittle skull of yours.
                theroux:
                    !angry
                    Just go back to whatever lousy dive you call home and let the parasites eat you.
                player:
                    !angry
                    This <i>is</i> my home, you brat! I only left 'cause the lot of you made me feel like I didn't belong!
                    But that's gonna change. Because I'm gonna get back what's mine. 
                    And you never beat me in a fight before. I doubt you're gonna start now.
                theroux:
                    You think I don't travel with a bodyguard, you idiot? 
                bodyguard:
                    !right
                * Clearly, Theroux's grown up enough to hire someone else to fight his battles for him.
            ]],

            DIALOG_DEFEND = [[
                player:
                    !fight
                    First, I'm gonna smash this guard of yours into the filigreed floor.
                    $angryAside
                    Then, baby brother, you and I are gonna have words. And they won't be nice.
            ]],

            DIALOG_THEROUX_DIED_IN_FIGHT = [[
                * Turns out, it's a lot harder to pull your punches now than it used to be.
                * At first you think Theroux is just faking. But he could never stay still for that long.
                player:
                    !scared_shrug
                    $miscDepressed
                    Aw, Hesh.
                    Didn't have to go this way, Theroux.
                    $neutralDubious
                    But what else was I supposed to do, huh?
            ]],

            DIALOG_TALK_TO_THEROUX = [[
                theroux:
                    !right
                    !injured
                    Y-you set this whole thing up, for what?
            ]],

            OPT_KILL = "Kill {theroux}",
            DIALOG_KILL = [[
                player:
                    !shrug
                    Petty revenge, I guess.
                * Theroux goes down easy, just like he did when you were little.
                * Only this time, he's not getting back up.
            ]],
            
            OPT_LET_GO = "Let {theroux} go",
            DIALOG_LET_GO = [[
                player:
                    I could ask you the same thing.
                    But I know I don't really wanna hear the answer.
                    ... It's hard, you know? Realizing all your siblings hate you.
                    !shrug
                    $neutralResigned
                    I mean, I kinda knew before. But I thought you'd at least pretend otherwise.
                theroux:
                    $angryPatienceLost
                    Of course I hate you! Look what you've done!
                player:
                    !agree
                    Yeah, I get that. It's a flead queen-egg scenario.
                    ...
                    !handwave
                    Get out of here, Theroux. I got thinking to do.
                    !angry
                    But tell Vix I'm coming for him. I know he's the real mastermind.
                agent:
                    !exit

                * Theroux scarpers off, his wedding outfit mottled with blood. You can bet Vix will get the message.
            ]]
        }
    
        :Fn(function(cxt) 
            
            cxt.quest:AssignCastMember("bodyguard")
            cxt:TalkTo("bodyguard")
            cxt:Dialog("DIALOG_INTRO")

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    enemies = {"bodyguard", "theroux"},
                    flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.BOSS_FIGHT,
                }
                    :OnWin()
                        :Fn(function() 
                            if cxt.quest:GetCastMember("theroux"):IsDead() then
                                cxt:Dialog("DIALOG_THEROUX_DIED_IN_FIGHT")
                                cxt.quest:Complete("go_to_the_chapel")
                                StateGraphUtil.AddLeaveLocation(cxt)

                            else
                                cxt:TalkTo("theroux")
                                cxt:Dialog("DIALOG_TALK_TO_THEROUX")

                                cxt:RunLoop(function(cxt) 
                                
                                    -- cxt:Question("OPT_THEROUX_Q_1", "DIALOG_THEROUX_Q_1")
                                    -- cxt:Question("OPT_THEROUX_Q_2", "DIALOG_THEROUX_Q_2")

                                    cxt:Opt("OPT_KILL")
                                        :Dialog("DIALOG_KILL")
                                        :Fn(function() 
                                            AgentUtil.CommitMurder(cxt:GetAgent())
                                            cxt:GetAgent():Kill()
                                        end)
                                        :CompleteQuest("go_to_the_chapel")
                                        :Travel()
                                    
                                    
                                    cxt:Opt("OPT_LET_GO")
                                        :Dialog("DIALOG_LET_GO")
                                        :ReceiveOpinion("ruined_special_day")
                                        :CompleteQuest("go_to_the_chapel")
                                        :Travel()

                                end)
                            end
                        end)
        end)




QDEF:AddConvo("sleep", "moreef")
    :Loc{

        
        --  Possible states:
        --    theroux is alive and married, and reconciled with you. Suitor doesn't like you
        --    theroux is alive, and doesn't like you. Suitor is dead or captured
        --    theroux is dead, suitor is rich now and owes you a favour
        --    theroux is dead, suitor is rich but doesn't like you
        

        OPT_GET_A_ROOM = "Head to your room",
        
        DIALOG_GET_ROOM = [[
            * Back at the Hideaway, you and Moreef catch up. 
            * {moreef} opens a door behind the bar for you.
            moreef:
                !exit
        ]],

        DIALOG_WAKE = [[
            * You wake up to the sharp jab of a finger, right between the only two ribs you haven't broken at some point or another.
            player:
                !left
                !tired
                $neutralWhatever
                Sleepin.
                $neutralWhatever
                Fi-mor-mints.
            * You try to roll over, but now there's an armored boot in the bed, and the toe digs into your spleen.
            mullifee:
                !right
                No more minutes. 
            player:
                !suspicious
                What the—
                ...
                !surprise
                $neutralTakenAback
                Mullifee?!
                Didn't expect a visit from you. Not after you made it so clear we were done.
            mullifee:
                It wasn't personal. But if you're going to sulk about it...
            player:
                !angry_shrug
                $neutralTakenAback
                You disowned me!
            mullifee:
                $miscMischievous
                I thought you liked to "roll with the punches". So adapt.
                Two days ago, you were kicked out of the family. Today, I'm here with an offer. 
                $miscPersuasive
                What happens next is up to you, but I guarantee whining about it won't change a thing. 
            player:
                !dubious
                $neutralDubious
                ... What sort of an offer?
            mullifee:
                You want back in to the family. And that's not happening unless we drown out Vixmalli.
            player:
                !agree
                Oh, I'll drown him, alright. Gimme the chance.
            mullifee:
                !point
                Exactly. With mom gone, the family needs new muscle. This is your chance to prove yourself useful.
                I've got two targets. But you only need to take out one.
                Do that for me, and I'll help you out against Vix.  
            player:
                !interest
                $neutralDubious
                Yeah? And just who are these targets?
            mullifee:
                Seemli Arammy. Known connections to the Rise, wanted for inciting unrest in the servants' quarters of many local families.
                The other is ... an Admiralty officer. I'd prefer to keep the details of that one to myself, unless you buy in.
            player:
                !humouring
                $miscMocking
                A lack of trust? In <i>this</i> family?
        ]],

        OPT_SEEMLI = "Ask about {seemli}",
        DIALOG_SEEMLI = [[
            player:
                Who's this jarackle you mentioned?
            mullifee:
                For now, a relative nobody. I'd like to nip it in the bud now before the problem grows.
                 {seemli.HeShe}'s stirring up support for a Rise movement here in the Pearl among the domestic staff.
             player:
                !forgetful
                 Domestic staff? Like, butlers and stuff?
                 That doesn't sound so scary.
             mullifee:
                 And it wasn't, before the glut of pilgrims. Now there's plenty of hoodlums to add to the swell.
                 The Rise have already upset the status quo in Spark Baron lands and the Bread Fields.
                 !disappoint
                 $neutralResigned 
                 I'd rather not underestimate them here.
            player:
                !overthere
                So arrest her, then. Whaddya need me for?
                $miscMocking
                Or aren't you a big, tough Admiralty officer no more?
            mullifee:
                Seemli is technically under a domestic contract with a rather wealthy family.
                !dubious 
                They would <i>prefer</i> that I deal with her covertly.
               player:
               !chuckle
                Ah, yeah. That kinda thing is sure fodder for the gossip rags.
               mullifee:
                Quite.
        ]],

        OPT_EDEN = "Ask about {eden}",
        DIALOG_EDEN= [[
            player:
                !cruel 
                $miscMocking
                And this <i>superior officer</i>? Don't tell me you're trying to kill your way through the ranks.
                When Pops offered to buy you a bigger commission you sounded so proud turning him down. 
            mullifee:
                !crossed
                It's not opportunism. It's precaution. 
                My performance reviews have been exemplary. 
            player:
                $miscMocking
                Oh, <i>congrats</i>.
            mullifee:
                !dubious
                I <i>was</i> due for a promotion, but then I discovered one of my superiors is accepting bribes from the Barons.
                And unfortunately I wasn't very subtle about it.
            player:
                !cruel
                Hah! That was stupid.
            mullifee:
                Yes, well... My promotion has been stalled. 
                But I have reason to believe {eden} won't stop there, and will likely try to have me killed.
            player:
                So you shoot this {eden} first, is that it?
            mullifee:
                !point
                Incorrect. <i>You</i>'ll do the shooting.
        ]],

        OPT_SET_OUT = "Set out for the day",
        DIALOG_SET_OUT = [[
            
            player:
                Well, guess the day waits for no one. 
                Least of all me.
                !exit
            
            mullifee:
                !exit
        ]],   
    }

    :Hub(function(cxt)
        cxt:Opt("OPT_GET_A_ROOM")
            :SetQuestMark()
            :Dialog("DIALOG_GET_ROOM")
            :PreIcon(global_images.sleep)
            :Fn(function()
                local grog = cxt.location
                cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("MOREEF_BAR.inn_room") )
                grog:SetPlax()
                
                ConvoUtil.DoSleep(cxt, nil, function() 
                    cxt:Dialog("DIALOG_WAKE")
                    cxt:TalkTo("mullifee")
                    cxt:RunLoopingFn(function() 
                        
                        cxt:QST("EDEN")
                        cxt:QST("SEEMLI")
                        
                        cxt:Opt("OPT_SET_OUT")
                            :SetQuestMark()
                            :Dialog("DIALOG_SET_OUT")
                            :Fn(function() 
                                if cxt:GetAgent() then
                                    cxt:GetAgent():MoveToLimbo() 
                                end
                                cxt.quest:Complete()
                                cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("MOREEF_BAR") )
                                cxt:End()
                            end)
                            
                    end)
                end)
            end)
        end)
