local QDEF = QuestDef.Define
{
    title = "Mettle Vendor",
    icon = engine.asset.Texture("icons/quests/grit_shop.tex"),
    qtype = QTYPE.STORY,
    home = true,
    
    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("stranger"), location = quest:GetCastMember("shop"), role = CHARACTER_ROLES.VENDOR } )
    end,

    on_init = function(quest)
        quest.param.started_unlocked = TheGame:GetGameProfile():HasMettleUnlocked(TheGame:GetGameState():GetPlayerAgent():GetContentID())
        if TheGame:GetGameState():CanDoMettle() then
            if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.NIGHT then
                quest:Activate("visit")
            end
        end
    end,

    events = 
    {
        phase_change = function(quest)
            
            if TheGame:GetGameState():CanDoMettle() then
                if TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY then
                    quest.param.bought_today = false
                    quest:Cancel("visit")
                else
                    quest:Activate("visit")
                end
            end
        end,
        
        quests_changed = function(quest, event_quest) 
            if not quest.param.started_unlocked and not quest:IsActive("visit") and TheGame:GetGameState():CanDoMettle() then
                quest:Activate("visit")
            end
        end,
    },}

:AddExistingLocation( "shop", "PEARL_METTLE_SHOP" )

:AddCastByAlias{
    cast_id = "stranger",
    alias = "DODGY_SCAVENGER",
}

:AddObjective{
    state = QSTATUS.INACTIVE,
    id = "visit",
    title = "Shop for Mettle upgrades",
    desc = "You have <b>{1} Mettle</> to spend on upgrades, and <b>{2} Prestige Points</> to spend on Flourishes.",
    desc_fn = function(quest, str) 
        return loc.format(str, TheGame:GetGameProfile():GetMettlePoints(TheGame:GetGameState():GetPlayerAgent():GetContentID()), TheGame:GetGameProfile():GetPrestigePoints())
    end,
    hide_in_overlay = true,
    low_priority = true,
    mark = {"shop"},
}

QDEF:AddConvo("visit")
    :ConfrontState("FOO", function(cxt) return cxt.location == cxt.quest:GetCastMember("shop") end)
        :ProgressionLoc{
            DIALOG_INTRO = {
                [[
                {first_time?
                    player:
                        !left
                    agent:
                        !right
                    player:
                        Hey, I recognize you!
                        You're the one who told me to come here!
                        !happy
                        You can't fool me that easily!
                    agent:
                        !placate
                        I'm not trying to fool anyone.
                        I'm trying to test your mettle!
                }
                {not first_time?
                    player:
                        !left
                    agent:
                        !right
                        !greeting
                        I knew you'd be back.
                }
                ]],
                [[
                    {first_time?
                        agent:
                            !right
                        player:
                            !left
                            !thought
                            Wait. Have I been here before?
                        agent:
                            Not in this lifetime.
                        player:
                            The ol' noggin gets fuzzy sometimes.
                            !happy
                            Occupational hazard!
                        agent:
                            You call what you do an occupation, now? Interesting.
                    }
                    {not first_time?
                            player:
                                !left
                            agent:
                                !right
                                Welcome back, grifter.
                    }
                ]],
                [[
                    {first_time?
                        agent:
                            !right
                        player:
                            !left
                            Um...
                        agent:
                            Looking for something?
                        player:
                            Yeah, I guess I am.
                        agent:
                            Then you came to the right place!
                    }
                    {not first_time?
                            player:
                                !left
                            agent:
                                !right
                                Can't stay away, can you?
                    }
                ]],
                [[
                    {first_time?
                        agent:
                            !right
                        player:
                            !left
                            I'm ready to get stronger!
                        agent:
                            Persistence pays off!
                    }
                    {not first_time?
                            player:
                                !left
                            agent:
                                !right
                                Let's make you stronger!
                    }
                ]],
            }
        }

        :Loc{


            OPT_FLOURISHES = "Unlock Flourishes",
            DIALOG_FLOURISHES = [[
                player:
                    I want to learn something new.
            ]],
            DIALOG_POST_FLOURISHES = [[
                agent:
                    It's important to end things well.
            ]],

            OPT_SHOP = "Spend Mettle",
            DIALOG_SHOP = [[
                player:
                    Let me see what you're selling.
            ]],
            DIALOG_POST_SHOP = [[
                agent:
                    I knew you'd like what I have on offer.
            ]],
            OPT_LEAVE = "Stop Shopping",
            DIALOG_LEAVE = [[
                player:
                    I'm going to leave now. You're creeping me out.
                    !exit
                agent:
                    !happy
                    I'm here every night!
            ]],
            DIALOG_ASK_ABOUT = [[
                player:
                    !thought
                    So. Mettle, eh?
            ]],

            OPT_BUY_METTLE = "Purchase {1} Mettle {1*Point|Points}",
            DIALOG_BUY_METTLE = [[
                agent:
                    !happy
                    You of all people know how shills can open doors.
            ]],
            TT_BUY_METTLE_ALREADY = "You already bought mettle today! Try again tomorrow.",
        }   
        :Fn(function(cxt) 
            local stranger = cxt.quest:GetCastAgent("stranger")
            stranger:MoveToLocation(cxt.quest:GetCastMember("shop"))
            cxt.enc:SetPrimaryCast(stranger)

            cxt:ProgressDialog("SMITH_METTLE_VISITS", "DIALOG_INTRO")
            
            local day = TheGame:GetGameState():GetActProgress()
            


            local METTLE_POINTS = {1, 3, 9, 18}
            local COST = {200, 400, 600, 800}
            local num_points = METTLE_POINTS[math.min(day, #METTLE_POINTS)]
            local mettle_cost = COST[math.min(day, #COST)]

            cxt:RunLoop(function() 
                cxt:Opt("OPT_SHOP")
                    :Dialog("DIALOG_SHOP")
                    :PreIcon(global_images.mettle)
                    :IsHubOption(true)
                    :Fn(function()
                        cxt.enc:WaitOnLine()
                        local screen = Screen.MettleScreen( cxt.player, function() cxt.enc:ResumeEncounter() end )
                        TheGame:FE():PushScreen( screen )
                        cxt.enc:YieldEncounter()
                        cxt:Dialog("DIALOG_POST_SHOP")
                    end)


                cxt:Opt("OPT_BUY_METTLE", num_points)
                    :Dialog("DIALOG_BUY_METTLE")
                    :DeliverMoney(mettle_cost, {is_shop = true})
                    :ReqCondition(not cxt.quest.param.bought_today, "TT_BUY_METTLE_ALREADY")
                    :PreIcon(global_images.mettle)
                    :Fn(function() 
                        AUDIO:PlayEvent("event:/ui/mettle_shop/buy_mettlepoints")
                        TheGame:GetGameProfile():AddMettlePoints( TheGame:GetGameState():GetPlayerAgent():GetContentID(), num_points, "BUY_METTLE_SHOP") 
                        cxt.quest.param.bought_today = true
                        DoAutoSave()
                    end)

                cxt:Opt("OPT_FLOURISHES")
                    :PreIcon(global_images.flourish)
                    :Dialog("DIALOG_FLOURISHES")
                    :IsHubOption(true)
                    :Fn(function()
                        cxt.enc:WaitOnLine()
                        local screen = Screen.FlourishScreen( cxt.player, function() cxt.enc:ResumeEncounter() end )
                        TheGame:FE():PushScreen( screen )
                        cxt.enc:YieldEncounter()
                        cxt:Dialog("DIALOG_POST_FLOURISHES")
                    end)

                cxt:Opt("OPT_ASK_ABOUT")
                    :Dialog("DIALOG_ASK_ABOUT")
                    :IsHubOption(true)
                    :GoTo("STATE_QUESTIONS")

                cxt:Opt("OPT_LEAVE")
                    :MakeUnder()
                    :Dialog("DIALOG_LEAVE")
                    :PreIcon(global_images.close)
                    :Travel()
            end)
        end)


    :AskAboutHub("STATE_QUESTIONS", 
    {
        "Ask about {agent}",
        [[
            player:
                I feel like I've met you before.
            agent:
                I get that a lot.
            player:
                <i>Have</> I met you before?
            agent:
                I honestly don't keep track of who's met me.
                But I've met you before, in one form or another!
            player:
                Creepy.
        ]],
        "Ask about mettle",
        [[
            player:
                Is this stuff fake? It sounds fake.
            agent:
                I assure you it's not fake. Anything you buy here is guaranteed for this life, and all others.
            player:
                !dubious
                Others?
            agent:
                !laugh
                Don't worry about that part!
                Just know that you're getting your value's worth.
        ]],
        "Ask how to get mettle",
        [[
            player:
                Alright, how do I get more of this mettle?
            agent:
                Hard work, luck, and perseverance.
            player:
                !thought
                Not really my style.
                How about I just give you money?
            agent:
                That works too!
        ]],        
    })