local QDEF = QuestDef.Define
{
    title = "Pick a Side",
    qtype = QTYPE.STORY,
    on_init = function(quest) 
        --TheGame:GetGameState():GetCaravan():MoveToLocation(TheGame:GetGameState():GetLocation("SMITH_HOME"):GetDoorLocation())
    end,
    icon = engine.asset.Texture("icons/quests/smith_story_pick_a_side.tex"),
    collect_agent_locations = function(quest, t)

        if not quest.param.seemli then
            table.insert (t, { agent = quest:GetCastMember("eden"), location = quest:GetCastMember("eden_office"), role = CHARACTER_ROLES.PATRON} )
        end

        if not quest.param.eden then
            table.insert (t, { agent = quest:GetCastMember("seemli"), location = quest:GetCastMember("seemli_hideout"), role = CHARACTER_ROLES.PATRON} )
        end
    end,

    plot_armour_fn = function(quest, agent)
        if agent == quest:GetCastMember("partner") then
            return true
        end
    end,
}

:AddCastByAlias{
    cast_id = "moreef",
    alias = "SWEET_MOREEF",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "mullifee",
    alias = "MULLIFEE",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "seemli",
    alias = "SEEMLI",
    no_validation = true,
}

:AddCastByAlias{
    cast_id = "eden",
    alias = "EDEN",
    no_validation = true,
}

:AddCast{
    cast_id = "partner",
    no_validation = true,
    when = QWHEN.MANUAL,
}

:AddObjective{
    id = "start",
    title = "Talk to Moreef",
    desc = "Figure out a course of action.",
    state = QSTATUS.ACTIVE,
    on_activate = function(quest)
        quest:Activate("pick_eden") 
        quest:Activate("pick_seemli") 
        quest:Complete("start")
    end,
}

------------------

:AddObjective{
    id = "pick_eden",
    title = "Contact {eden}",
    desc = "Convince {eden} that you want to help the Admiralty.",
    mark = {"eden"},
    on_complete = function(quest)
        quest:Cancel("pick_seemli")
        quest.param.eden = true
        quest.param.parent_quest:AssignCastMember('partner', quest:GetCastMember("eden"))
        quest:AssignCastMember('partner', quest:GetCastMember("eden"))
    end,

}

:AddObjective{
    id = "pick_seemli",
    title = "Contact {seemli}",
    desc = "Convince {seemli} that you want to join the Rise.",
    mark = {"seemli"},    
    on_complete = function(quest)
        quest:Cancel("pick_eden")
        quest.param.seemli = true
        quest.param.parent_quest:AssignCastMember('partner', quest:GetCastMember("seemli"))
        quest:AssignCastMember('partner', quest:GetCastMember("seemli"))
    end,
}

:AddObjective{
    id = "do_job",
    hide_in_overlay = true,
    events = 
    {
        quests_changed = function(quest, event_quest)
            if quest.param.current_job == event_quest and not event_quest:IsActive() then
                quest:Complete("do_job")
                quest:Activate("done_job")
            end
        end
    },
}

:AddObjective{
    id = "done_job",
    hide_in_overlay = true,
    on_complete = function(quest) 
        quest:Complete()
    end,
    mark = {"partner"}
}


:AddQuestLocation{
    cast_id = "seemli_hideout",
    name = "Seemli's Hideout",
    plax = "INT_SMITH_SEEMLISHIDEOUT",
    map_tags = {"quarters"},
    indoors = true,
}

:AddQuestLocation{
    cast_id = "eden_office",
    name = "Eden's Office",
    plax = "INT_SMITH_EDENOFFICE",
    map_tags = {"building"},
    indoors = true,
}


------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------

QDEF:AddConvo("pick_seemli", "seemli")
    :AttractState("STATE_ATTRACT")    
    :Loc{
        DIALOG_INTRO = [[
            * If {seemli}'s afraid of the authorities, {seemli.heshe} doesn't show it. 
            seemli:
                Hello, hello. Welcome! Would you like a pamphlet?
            player:
                I wouldn't get much use out of it, to be honest.
            seemli:
                Ah, you can't read? We have pictographic ones for your convenience!
                The Rise is a peoples' movement, so we try to move people.
                Spiritually, I mean.
            player:
                Not sure I, uh, get it...
                Look, I just heard you maybe had some work? And I could use some work.
                I've been disowned by my family, you see. The Banquods? 
            seemli:
                The Banquods? That wealthy estate with the sea views?
                 Wait—that would make you related to Mullifee Banquod, wouldn't it? The Admiralty officer?
             player:
                 It woulda—but like I said, she disowned me.
                 Now I'm just trying to get by. And I sure wouldn't mind taking a crack at her skull, while I'm at it.
             seemli:
                 I see. That's... good timing.
                 In fact, I <i>do</i> have a job for you, ex-Banquod.     
        ]],
        OPT_LATER = "Consider your options",
        DIALOG_LATER = [[
            player:
                I need to ask around a bit. No offense, but I don't got a lot of friends right now.
            seemli:
                Oh, none taken. All I'm doing is gathering tinder. 
                I'm happy to wait until the spark of revolution strikes.
        ]],


        OPT_AGREE = "Agree to help",
        DIALOG_AGREE = [[
            player:
                You know what? That sounds grand.
                I always was a black sheep. Anything I can do to stick it to my betters sounds good to me.
            seemli:
                That's great! Happy to have you on board.
        ]],

        OPT_Q1 = "Ask about Mullifee",
        DIALOG_Q1 = [[
               player:
                   You mentioned my sister. She a problem to you?
               seemli:
                   Oh, only that she represents the strict enforcement of the status quo.
                   I don't have anything against her <i>personally</i>—except, of course, her staggering privilege.
               player:
                   Okay, I'll bite. What's that mean?
               seemli:
                   As if you don't know! She's the only female heir to a kra'deshi dynasty. 
                   Everything was handed to her, from her commission to her secured future.
               player:
                   Yeah. Plus, you know, she didn't even hesitate to knock me outta the will.
                   We're twins, even! We shared an egg sac! You'd think that'd mean something.
               seemli:
                   You <i>would</i> think that.
        ]],
    }

    :Fn(function(cxt, who)
        cxt:Dialog("DIALOG_INTRO")
        cxt:RunLoopingFn(function() 

            cxt:QST("Q1")
            cxt:Opt("OPT_AGREE")
                :SetQuestMark()
                :Dialog("DIALOG_AGREE")
                :Fn(function()
                    cxt.quest.param.current_job = TheGame:GetGameState():GetMainQuest():DefFn("SpawnPoolJob", "SEEMLI_JOB", cxt:GetAgent())
                    --cxt.quest.param.current_job = QuestUtil.SpawnQuest("PLACEHOLDER_SMITH_JOB", {cast = {giver = cxt:GetAgent()} })
                    cxt:PlayQuestConvo(cxt.quest.param.current_job, QUEST_CONVO_HOOK.INTRO)
                    cxt:PlayQuestConvo(cxt.quest.param.current_job, QUEST_CONVO_HOOK.ACCEPTED)


                    cxt.quest:Complete("pick_seemli")
                    cxt.quest:Activate("do_job")
                    StateGraphUtil.AddEndOption(cxt)
                end)
                :DoneConvo()

            cxt:Opt("OPT_LATER")
                :PreIcon(global_images.back)
                :Dialog("DIALOG_LATER")
                :DoneConvo()

        end)
    end)


QDEF:AddConvo("pick_eden", "eden")
    :AttractState("STATE_ATTRACT")    
    :Loc{
        DIALOG_INTRO = [[
            * Finding {eden} is easy enough, what with {eden.himher} being an important figure and all.
            player:
                Hey, you {eden}? Word is you might be looking for some help.
            eden:
                And whose word would that be? I don't deal with thugs.
            player:
                Naw, I hear you. You prefer, uh, well-<i>equipped</i> allies.
                You know what I mean? Well-equipped?
            eden:
                I'm not sure that I—
            player:
                Folks who got lots of neat tools. And know how to use 'em.
            eden:
                Look, I don't much like your tone, and I—
            player:
                Tools that go <i>pow</i> 'cause they got spark coming outta them.
            eden:
                Oh. 
                <i>Oh</i>. 
                It doesn't seem like subterfuge is your skillset, exactly.
            player:
                Hey, don't sell me short! You think I'm just here for the shills?
                I want revenge on a certain someone. First name rhymes with Bullybee, last name with Lame...quod.
                And my friends, the, uh, tools—they told me you'd maybe help me get that. 
            eden:
                Huh. I might have work for you, after all.

        ]],
        OPT_LATER = "Consider your options",
        DIALOG_LATER = [[
            player:
                Alright, alright. Good to know.
                I just, uh, gotta go check on something first.
            eden:
                Dumb <i>and</i> indecisive? What could possibly go wrong.
            player:
                Whassat?
            eden:
                Nothing. 
        ]],


        OPT_AGREE = "Agree to help",
        DIALOG_AGREE = [[
            player:
                You know what, officer? I think you and I might just be in business.
            eden:
                Thrilling.
        ]],

        OPT_Q1 = "Ask about Mullifee",
        DIALOG_Q1 = [[
            player:
                So, uh. You hate... Bullybee too, right?
                'Cause that's why I'm here. To get at Bullybee.
            eden:
                I don't know what you're talking about.
                What I do know is that you're going to do a job for me. Then you'll return here for a reward.
                And if you play your cards right, that reward will include some information on where you might find...
            player:
                Bullybee.
            eden:
                !sigh
                Bullybee.
        ]],

    }

    :Fn(function(cxt, who)
        cxt:Dialog("DIALOG_INTRO")
        cxt:RunLoopingFn(function() 

            cxt:QST("Q1")

            cxt:Opt("OPT_AGREE")
                :SetQuestMark()
                :Dialog("DIALOG_AGREE")
                :Fn(function()
                    cxt.quest.param.current_job = TheGame:GetGameState():GetMainQuest():DefFn("SpawnPoolJob", "EDEN_JOB", cxt:GetAgent())
                    --cxt.quest.param.current_job = QuestUtil.SpawnQuest("PLACEHOLDER_SMITH_JOB", {cast = {giver = cxt:GetAgent()} })
                    cxt:PlayQuestConvo(cxt.quest.param.current_job, QUEST_CONVO_HOOK.INTRO)
                    cxt:PlayQuestConvo(cxt.quest.param.current_job, QUEST_CONVO_HOOK.ACCEPTED)
                    cxt.quest:Complete("pick_eden")
                    cxt.quest:Activate("do_job")
                    StateGraphUtil.AddEndOption(cxt)
                end)
                :DoneConvo()

            cxt:Opt("OPT_LATER")
                :PreIcon(global_images.back)
                :Dialog("DIALOG_LATER")
                :DoneConvo()

        end)
    end)

----------------------------------------------------------------------------


QDEF:AddConvo("done_job", "partner")
    :Attract(function(cxt) return "STATE_TALK" end)
    :Hub(function(cxt)
        cxt:GoTo("STATE_TALK")
    end)
    :State("STATE_TALK")
        :Loc{
            DIALOG_INTRO = [[
                {seemli?
                    seemli:
                        The universe will correct itself. 
                    player:
                        You seem, uh, awful chipper for a revolutionary.
                    seemli:
                        There's no reason why we can't achieve autonomy with a smile!
                        Bloodshed is only a worst-case scenario.
                    player:
                        Right. So, uh, speakin' of bloodshed...
                    seemli:
                        Right! Right. We'll, uh, discuss it later, hm?
                        I've got cupcakes I should be baking for a Cult function this afternoon...
                    player:
                        Cupcakes?!
                    seemli:
                        Oh, twist my arm! I'll save one for you when we meet later.
                }
                {eden?
                    eden:
                        Well. Seems you're not a complete ignoramus.
                        Just so happens... we could be in business.
                    player:
                        Hey, that's what I like to hear!
                    eden:
                        Of course, I did some digging. I like to know who I'm working with.
                        You didn't mention <i>Bullybee</i> is your sister.
                    player:
                        She ain't. Or did you not dig that part up?
                        I got disowned.
                    eden:
                        Well, it complicates things. Makes it messier. And I don't like mess.
                        So kindly leave before anyone sees us talking too long. 
                        We'll discuss this situation more later, when we can speak plainly. 
                }
            ]],
        }
        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Complete("done_job")
            StateGraphUtil.AddLeaveLocation(cxt)
        end)
