local negotiation_defs = require "negotiation/negotiation_defs"
local EVENT = negotiation_defs.EVENT
Content.AddNegotiationCard("shifting_mutation", 
{
    name = "Shifting Mutation",
    desc = "At the end of your turn, deal 2 damage to your core argument and incept 2 {intimidated}.",
    flavour = "'Why is everyone looking at me like that?'",
    
    cost = 1,
    flags = negotiation_defs.CARD_FLAGS.STATUS | negotiation_defs.CARD_FLAGS.EXPEND,

    rarity = CARD_RARITY.UNIQUE,
    series = CARD_SERIES.GENERAL,

    event_handlers =
    {
        [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
            self:NotifyTriggeredPre()
            minigame:ApplyPersuasion(self, self.negotiator, 2, 2)
            self.anti_negotiator:AddModifier("intimidated", 2, self )
            self:NotifyTriggeredPost()
        end,
    },
})

local QDEF = QuestDef.Define
{
    title = "Sampling Error",
    qtype = QTYPE.STORY,
    on_init = function(quest) 
    end,
    
    icon = engine.asset.Texture("icons/quests/smith_story_take_a_sample.tex"),
    collect_agent_locations = function(quest, t)
    end,
    
    plot_armour_fn = function(quest, agent)
        if agent == quest:GetCastMember("bilebroker") and quest:IsActive("contact_bilebroker") then
            return true
        end

    end,

    on_destroy = function(quest)
        if quest:GetCastMember("bilebroker") and quest:GetCastMember("bilebroker"):IsInPlayerParty() then
            quest:GetCastMember("bilebroker"):Dismiss()
        end
    end,

    is_in_hiding = function(quest, agent)
        if agent == quest:GetCastMember("bogger") then
            return true
        end
    end,
}

:AddCastByAlias{
    cast_id = "moreef",
    alias = "SWEET_MOREEF",
    no_validation = true,
}

:AddDefCastSpawn("bilebroker", "CHEMIST")
:AddDefCastSpawn("bogger", "BOGGER_CULTIVATOR")

:AddObjective{
    id = "start",
    title = "Talk to Moreef",
    desc = "You have questions for Moreef. Chiefly, why did he let Vixmalli into your room?!",
    state = QSTATUS.ACTIVE,
    on_complete = function(quest)
        quest:Activate("contact_bilebroker") 
    end,
}

:AddObjective{
    id = "contact_bilebroker",
    title = "Contact {bilebroker}",
    desc = "Talk to the mysterious bilebroker.",
    mark = {"bilebroker"},
    
    on_complete = function(quest)
        quest:Activate("collect_sample")
    end,
}

:AddObjective{
    id = "collect_sample",
    title = "Collect a sample",
    desc = "Collect a sample of the Great Beaching.",
    mark = {"sampling_site"},
    on_complete = function(quest)
        quest.param.smith_did_sample = not quest:GetCastMember("bilebroker"):IsInPlayerParty()
        quest:Activate("deliver_sample")
    end,
}

:AddObjective{
    id = "deliver_sample",
    title = "Deliver the sample",
    desc = "Take the sample to the rendezvous point.",
    mark = {"rendezvous_point"},
        on_complete = function(quest)
    end,
}

:AddQuestLocation{
    cast_id = "sampling_site",
    name = "Sampling Site",
    plax = "EXT_SMITH_BEACHCREATURE", -- to be replaced
    map_tags = {"beach"},
    indoors = false,
}

:AddQuestLocation{
    cast_id = "rendezvous_point",
    name = "Rendezvous Point",
    plax = "EXT_SMITH_Nightlife_Street",
    map_tags = {"street"},
    indoors = false,
}

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("start", "moreef")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                    Alright, Smith? You look a little shaken.
                player:
                    Did you just see Vix go by?
                agent:
                    Your brother was here?! 
                    Sheesh. Least he could've done was take the front entrance. It'd do the business good if folks thought Vixmalli Banquod himself drank here.
                player:
                    Not so sure about that, bud. He looked a little under the weather.
                agent:
                    Well... speaking of, there was a bilebroker looking for you earlier.
                    I know you have big plans today, but it sounded like maybe something you'd at least want to take a look at. 
                    !give 
                    Here's the meet-up. Might wanna mosey--not sure how long they'll wait for you.
            ]],

            OPT_VIX = "Ask about Vixmalli",
            DIALOG_VIX = [[
                player:
                    So Vix just let himself in?
                agent:
                    Musta done. I wouldn't have let him in without at least giving you a warning.
                    What... what'd you do, huh? Did you guys fight?
                player:
                    ... No. 
                agent:
                    So you made up?
                player:
                    No, we didn't do that neither. 
                    He was acting funny. Saying some weird sludge, too. 
                agent:
                    And you're sure he was actually here?
                    Maybe it was some kinda dream. You've been pretty fixated the past few days, after all.
                player:
                    Nightmare, more like. But yeah, maybe so.
            ]],

            OPT_BROKER = "Ask about the bilebroker",
            DIALOG_BROKER = [[
                player:
                    What's a bilebroker want with me?
                agent:
                    Who knows? Word's got out you do good work, and anyone who can navigate the Foam these days is probably someone worth knowing.
                    Assuming you still need to confront Vix like you planned, you've got some time until the Beaching event. 
                player:
                    Bilebrokers squick me out, to be honest.
                agent:
                    I don't blame you. But if you ask me, it's better to count one as a friend than not. 
            ]],

            OPT_SET_OUT = "Leave",
            DIALOG_SET_OUT = [[
                player:
                    Right, well... 
                    Guess if this day is gonna be weird as hell, I should just go for it.
                    I'll go talk to that broker. Meet back here later?
                agent:
                    You got it. And good luck.
            ]]
        }
        :Fn(function(cxt) 
            
            cxt:Dialog("DIALOG_INTRO")
            cxt.quest:Complete("start")

            cxt:RunLoopingFn(function() 
                
                cxt:QST("VIX")
                cxt:QST("BROKER")
                
                cxt:Opt("OPT_SET_OUT")
                    :SetQuestMark()
                    :Dialog("DIALOG_SET_OUT")
                    :Fn(function() cxt.quest:Complete('start') end)
                    :DoneConvo()
            end)
        end)


QDEF:AddConvo("contact_bilebroker", "bilebroker")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                player:
                    !left
                agent:
                    !right
                player:
                    You were lookin' for me? I gotta say, not sure what you'd want with a guy like me. 
                agent:
                    I am. I hear you've been close to the Great Beaching--not once, but many times. Is that true?
                player:
                    "Close" might be a little strong. I've been within whiffing distance, but I wouldn't say we're on a first name basis. 
                agent:
                    I have a client who wishes to procure a genetic sample. 
                    It's dangerous work, and the Heshians would find it heretical. I require the assistance of someone who doesn't fear the Cult.
                player:
                    Well, hey, then maybe I can help you after all. 
                    I'm, uh, feeling a little iffy on the Beaching right now, though. What's your client want with the sample, anyway?
                agent:
                    A great and terrible truth. Their words. 
                    My payment, in exchange for this service, is information: they've promised to share what they know about this behemoth.
                    You may receive shills, however.
                player:
                    Shills are always good. But... you know, I wouldn't hate learning a little more about the Beaching, too. 
                agent:
                    Information is addictive. 
                    !give
                    Here, take this vial. I need you to fill it with as uncontaminated a sample as you can manage. 
                    Once you've done so, head to the rendezvous point to meet with the buyer. 
                    Is that clear?
            ]],

            OPT_BEACHING = "Ask about Beaching",
            DIALOG_BEACHING = [[
                player:
                    So I guess you're telling me the Beaching ain't just some bloated puffer eel, huh?
                agent:
                    At that size? That would be marvelous in itself, wouldn't it?
                player:
                    Eh, sure.
                agent: 
                    The behemoth is a creature unknown. If it's truly a sending from Hesh... then its biology must contain a great many secrets.
                    Perhaps the buyer knows. That's certainly what I'm depending on. 
            ]],

            OPT_SAMPLE = "Ask about the sample",
            DIALOG_SAMPLE= [[
                player:
                    How do I get the goods anyway? Just... scoop up some snot?
                agent:
                    It's not complicated, but you do wish to get as natural a source as you can. 
                    Make a small incision to siphon off some of the more malodorous fluid.
                player:
                    Somehow that's even grosser than snot.
            ]],

            OPT_COME_WITH = "Convince {agent} to come with you",
            DIALOG_COME_WITH = [[
                player:
                    Hey, uh, you're gonna come with me, ain'tcha? 
                    I can watch your back and get you close, but you're the real expert when it comes to this science stuff. 
            ]],
            DIALOG_COME_WITH_SUCCESS = [[
                player:
                    Most folks agree: I'm an idiot. And if I screw it up then neither of us get any answers, huh?
                agent:
                    Hm, I suppose so, if you are as idiotic as you claim.
                    Though at least knowing your weaknesses speaks to your wisdom, if not your cunning.
                player:

                    Oh, yeah. I'm hella wise. And my wisdom suggests we work together on this.
                agent:
                    Very well. Lead on, then. And please be sure to keep me safe, as you promised.
                    Bilebrokers aren't welcome on the beach. 
            ]],

            DIALOG_COME_WITH_FAILURE = [[
                player:
                    Besides, it sounds gross and I don't wanna touch it.
                agent:
                    I'm afraid it's required. Bilebrokers have been banned from the beach.
                    Going with you will only present a greater challenge and compromise our success.
                    There's a reason I wished to hire you, after all, rather than simply gather the collection myself.
            ]],


            OPT_SET_OUT = "Accept the deal",
            DIALOG_SET_OUT = [[
                {not come_with?
                    player:
                        Alright, bud. Guess I better get going before that party later makes the beach too crowded.
                    agent:
                        Good luck. 
                }
                {come_with?
                    player:
                        Alright, bud. You ready to mosey?
                    agent:
                        Yes, I believe so. This is exciting! I love the sea, when the Heshians tolerate me to be near it. 
                }
            ]]
        }
        :Fn(function(cxt) 
            
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoopingFn(function() 
                
                cxt:QST("SAMPLE")
                cxt:QST("BEACHING")
                cxt:BasicNegotiation("COME_WITH")
                    :OnSuccess()
                        :Fn(function()
                            cxt:GetAgent():Recruit(PARTY_MEMBER_TYPE.ESCORT)
                            cxt.enc.scratch.come_with = true
                        end)
                
                cxt:Opt("OPT_SET_OUT")
                    :SetQuestMark()
                    :Dialog("DIALOG_SET_OUT")
                    :Fn(function() cxt.quest:Complete('contact_bilebroker') end)
                    :DoneConvo()
            end)
        end)


QDEF:AddConvo("collect_sample")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("sampling_site") end)
        :Loc{
            DIALOG_INTRO = [[
                * It's early in the day, but Admiralty and laborers are busy preparing the beach for the Deltrean visitors. 
                * Fortunately, you're good at pretending you belong. You puff out your chest and walk around to the back of the behemoth, looking down your nose at any laborers who glance your way.
                player:
                        !left
                        !hips
                        Ah hem hem hem! Wot wot! I'm a very pompous Pearly, wot!
                * Unsurprisingly, you're soon approached.
                {broker_here?
                    bilebroker:
                        !right
                    bilebroker:
                        !point
                        One too many "wots", perhaps. Someone's coming.
                        !exit
                    agent:
                        !right
                        !angry_accuse
                        Excuse you, sir, but the bilebrokers have been prohibited from accessing this site!
                        
                }
                {not broker_here?
                    player:
                        !left
                    agent:
                        !right
                        !placate
                        Hold on there, sir. The event doesn't begin for several hours.
                        Until then, I'm afraid you're trespassing.
                }
            ]],

            OPT_CONVINCE = "Convince {agent} you're here on official business",
            DIALOG_CONVINCE = [[
                player:
                    !thumb
                    Excuse you, indeed! Don't you know who I am?!
                ]],
            DIALOG_CONVINCE_FAILURE = [[
                {broker_here?
                player:
                    This bilebroker doesn't give a damn about the Beaching. {bilebroker.HeShe}'s my personal whatchacallit.
                    !eureka
                    Apothecary! That thing.
                agent:
                    Good. 'Cause you'll need some remedies once the guards are through with you.
                    !exit
                }
                {not broker_here?
                player:
                    Trespassing, shmpespassing! What do I care about some damn Deltreans, huh?
                agent:
                    Ah. That complicates matters.
                    Unfortunately, you had to make this political. Now we'll see you properly dealt with.
                    Guards!
                    !exit
                }
            ]],

            DIALOG_CONVINCE_SUCCESS = [[
                {broker_here?
                player:
                    !happy
                    Relax, bud! This here's my physician. I got a bad case of the blues, see.
                    On account of how my parents got crushed by this big blob over here.
                    }
                {not broker_here?
                player:
                    Trespassing? I'm just here to pay my respects. My parents died right on this beach, after all.
                    Or don't that make me a guest of honor?
                }
                agent:
                    !surprise
                    Y-you mean...
                player:
                    !hips
                    That's right. I'm a Banquod.
                agent:
                    !hesh_salute
                    Of course, sir. I'll just, ah, inform the high priests that you're here, shall I?
                    We'll have someone show you around in a, uh, sanctioned manner.
                    !exit
                * The priest scuttles off--appropriate, considering it's the beach.
            ]],

            DIALOG_GET_SAMPLE = [[
                {broker_here?
                    bilebroker:
                        !right
                        Well, if I'd had known you'd have made it so easy, I might have brought more vials.
                    player:
                        Yeah, well, no time for regrets. Looks like you got about three minutes to do the thing.
                    bilebroker:
                        On it.
                        !exit
                    * The bilebroker moves quickly, unwrapping some tools from a wax-treated envelope and digging deep into the behemoth's blubber.
                    bilebroker:
                        !right
                        I have it! 
                        Now to see about getting some answers. 
                }
                {not broker_here?
                    player:
                        !exit
                    * You approach the beast and try to spot an appropriate sample. It all looks festering to you.
                    player:
                        Well, uh... 
                        That bilebroker said I was fit to do it myself.
                        !shrug 
                        So I guess we do this my way.
                    * Throwing from the hip, you plunge your fist deep into the behemoth's blubber. Fluid squelches between your knuckles.
                    player:
                        !left
                        !tired
                        Ugh! 
                        But I guess this don't smell as bad as the rest of it. 
                        !exit
                    * With a fistful of behemoth bile, you quickly make your way off the beach.
                }
            ]],

            OPT_DEFEND = "Defend yourself!",

            DIALOG_WON_FIGHT = [[
                {broker_died?
                    agent:
                        !exit
                    * {bilebroker} is dead. You're going to have to collect the sample yourself.
                }
                {broker_here and not broker_died?
                    bilebroker:
                        !right
                        You see? I told you bringing me along would only lead to trouble.
                    player:
                        Yeah, yeah. The coast is clear, ain't it?
                    bilebroker:
                        For now. We shouldn't loiter, however.
                }
                {not broker_here?
                    * The coast is clear, but it won't be for long.
                }
            ]],

            OPT_RETREAT = "Leave the beach",
            DIALOG_RETREAT = [[
                * You leave before more trouble arrives.
            ]],

            OPT_OFFER_DONATION = "Offer a donation",
            DIALOG_OFFER_DONATION = [[
                player:
                    Where do you put the money in?
                agent:
                    I beg your pardon?
                player:
                    I got tithes here to pay to the behemoth, but I can't find the shill slot. 
                    How about I hand it to you, and you can make sure it gets somewhere... safe?
                    !give
                agent:
                    !take
                    Ah, yes, of course! Thank you for this offering.
                    I suppose it can't hurt for you to, ah, witness the Beaching up close. 
                    From this exact distance, mind. And no closer.
                player:
                    Oh, you have my word. And my shills, for that matter.
                agent:
                    Quite.
                    !exit
            ]],

            OPT_ATTACK = "Just get rid of {agent.himher}",
            DIALOG_ATTACK = [[
                player:
                    Oh, yeah, okay. See, the thing is...
                    !fight
                agent:
                    !scared
            ]],

        }
        :Fn(function(cxt) 
            cxt.enc.scratch.broker_here = cxt.quest:GetCastMember("bilebroker"):IsInPlayerParty()
            local allies = {}
            if cxt.enc.scratch.broker_here then
                table.insert(allies, "bilebroker")
            end

            local priest = AgentUtil.GetFreeAgent("PRIEST")
            cxt:TalkTo(priest)
            cxt:Dialog("DIALOG_INTRO")
            
            
            
            cxt:BasicNegotiation("CONVINCE")
                :OnSuccess()
                    :Fn(function()
                        cxt:Dialog("DIALOG_GET_SAMPLE")
                        cxt.quest:Complete("collect_sample")
                        StateGraphUtil.AddLeaveLocation(cxt)  
                    end)
                :OnFailure()
                    :Fn(function() 
                        CreateCombatBackup(priest, "ADMIRALTY_PATROL_BACKUP", cxt.quest:GetRank())
                        cxt:Opt("OPT_DEFEND")
                            :Battle{
                                allies = allies,
                                flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE,
                            }
                                :OnWin():Fn(function() 
                                    cxt.enc.scratch.broker_died = cxt.enc.scratch.broker_here and cxt.quest:GetCastMember("bilebroker"):IsDead()
                                    if cxt.enc.scratch.broker_died then
                                        cxt.enc.scratch.broker_here = false
                                    end
                                    cxt:Dialog("DIALOG_WON_FIGHT")
                                    cxt:Dialog("DIALOG_GET_SAMPLE")
                                    cxt.quest:Complete("collect_sample")
                                    StateGraphUtil.AddLeaveLocation(cxt)  
                                end)
                    end)

            cxt:Opt("OPT_ATTACK")
                :Dialog("DIALOG_ATTACK")
                :Battle{
                    allies = allies,
                    flags = BATTLE_FLAGS.ISOLATED | BATTLE_FLAGS.NO_FLEE,
                }
                    :OnWin():Fn(function() 
                        cxt.enc.scratch.broker_died = cxt.enc.scratch.broker_here and cxt.quest:GetCastMember("bilebroker"):IsDead()                        
                        if cxt.enc.scratch.broker_died then
                            cxt.enc.scratch.broker_here = false
                        end
                        cxt:Dialog("DIALOG_WON_FIGHT")
                        cxt:Dialog("DIALOG_GET_SAMPLE")
                        cxt.quest:Complete("collect_sample")
                        StateGraphUtil.AddLeaveLocation(cxt)  
                    end)

            cxt:Opt("OPT_OFFER_DONATION")
                :Dialog("DIALOG_OFFER_DONATION")
                :DeliverMoney(200)
                :Fn(function() 
                    cxt:Dialog("DIALOG_GET_SAMPLE")
                    cxt.quest:Complete("collect_sample")
                    StateGraphUtil.AddLeaveLocation(cxt)  
                end)
        end)


QDEF:AddConvo("deliver_sample")
    :ConfrontState("CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("rendezvous_point") end)
        :Loc{
            DIALOG_INTRO = [[
                * The rendezvous isn't anywhere remarkable, but you smell something like overturned earth, deep and musky.
                {broker_here?
                    bilebroker:
                        !left
                        Hail, {bogger}. We've brought the sample, as requested. 
                        Now you'll tell me of the Roaloch, yes?
                    bogger:
                        !right 
                        Show me the sample, first. Then I'll answer your questions.
                }
                {not broker_here?
                    player:
                        !left
                        Uh, hello? Anyone here?
                    bogger:
                        !right 
                        You are no bilebroker. But you have the stink of change upon you.
                    player:
                        Uh, thanks. 
                        {bilebroker} couldn't make it. But it's okay--I've got the stuff right here.
                    }
                {smith_did_sample?
                    player:
                        I hope this is right. I mighta gotten more of the goop on my shirt than I did in the vial.
                    bogger:
                        You... touched the beast directly? With your flesh?
                    player:
                        Dunno what you're accusing me of with that tone, buddy, but it was strictly platonic.
                    bogger:
                        Let me see you. Step closer.
                }
                {not smith_did_sample?
                    * The vial is sealed, but {bogger} takes the sample with long, leaden tongs.
                }
            ]],
            DIALOG_GET_MUTATION = [[
                    player:
                        Uh. Sure? I guess?
                * You step forward into {bogger}'s watery gaze. {bogger.HeShe} studies you, nostrils flaring.
                    bogger:
                        You <i>are</i> changed already. It works quickly, the belly of the beast.
                    player:
                        Whaddya mean? Like, metaphorically?
                        I reckon I've come into my own a bit these last few days, sure.
                    bogger:
                        No. Can't you feel it?        
                * You're about to argue, but something turns in your gut, and you realize {bogger} isn't wrong. 
                * The skin of your hand ripples, like something's churning just below the surface.
                    player:
                        What the Hesh?!
                    bogger:
                        No. Not of Hesh. 
                        Not for much longer, anyway.
            ]],

            DIALOG_TELL_OF_DANGER = [[
                player:
                    !left
                    What's the deal, here, buddy? That behemoth has been on the beach for days.
                bogger:
                    !right
                    Plenty of time to work its mischief indeed. The Beaching was merely the beginning.
                    Whatever parasite created the behemoth, it has moved on to a new host.
                    One Vixmalli Banquod, if I'm not mistaken. Now <i>that</i> is a sample I'd like to collect.
                player:
                    What the--?! Vix? Whaddya mean, Vix's got a parasite?
                bogger:
                    Ah. A friend of yours, this Banquod? 
                    I'm afraid you'll have to kill him. That is, assuming you want to save the world. 
                player:
                    <i>What</i>?
                bogger:
                    I spoke plainly. Process this information as you will, but do not plague me with your rhetorical questions.
                    Hah. Plague. That's a ... little joke, you see.

            ]],

            OPT_HEAD_BACK_TO_BAR = "Go save the world",
            DIALOG_HEAD_BACK_TO_BAR = [[
                player:
                    Look, I'm not gonna pretend to have any clue what you're talking about.
                    But if this is a problem that can be solved with fists, I'm pretty sure I can fix it.
                bogger:
                    An ancient problem warrants an ancient solution. Your fists should do just fine.
                player:
                    I'm going to need backup for this. I need to go have a drink. 
                    I mean a think?
                    Actually, probably both.
                    Bye.
                    !exit
            ]],

            OPT_BEACHING = "Ask about Vix",
            DIALOG_BEACHING = [[
                player:
                    Look, I just--I'm trying to wrap my head around--
                    What does my brother have to do with the <i>end of the world</i>?
                    He's a jackass, but not a... maniacal one. He can't destroy a whole planet.
                bogger:
                    Destroy? Certainly not. But remake, yes.
                    Vixmalli--your brother, I see--is host to a force as old as the Vagrants. Even older, perhaps.
                    You're familiar with the Roaloch?
                player:
                    Sure. Yeah. Gross goop that mutates anything it touches. 
                bogger:
                    The bilebrokers manage its tides. But once every thousand or so years... the neap tide drowns even the most adaptable swimmers.
                    With your brother acting as its champion, the Roaloch will consume everything on this planet, and remake it as it sees fit.
                    Crayote will fly. Kra'deshi will drown. Or perhaps the other way around? 
                    It's unlikely our consciousness will survive the transformation to even remember a time when things were different.
            ]],

            OPT_Roaloch = "Ask about the Roaloch",
            DIALOG_Roaloch = [[
                player:
                    Let me be sure I got this. The Roaloch--the floating turd mass that bilebrokers mine for medicine and stuff--it's... aware? Evil?
                bogger:
                    Don't be foolish. It's no more evil than the wind, or the sea. 
                    It is a natural part of this planet. It comes, it remakes, then it rests.
                    The Vagrants didn't know about that middle part, of course, which is why none of them are here to tell us their stories.
                player:
                    No way. Kra'deshi are as old as the stars. We got Vagrant ships with our faces on 'em!
                bogger:
                    Yes. And perhaps those are your ancestors--or perhaps it's just a passing resemblance. 
                    Would you like to test your luck and see what the Roaloch makes of you now?
                {smith_did_sample?
                bogger:
                    Look at your hand, if you require more evidence.}
                {broker_here?
                bilebroker:
                    !right 
                    This... is remarkable. 
                    We've never underestimated the danger of the Roaloch, of course. 
                    And my brethren and I have often discussed the possibility of--well...
                    Of what could go wrong. A worst case scenario.
                    I--I need to sit down.
                    !exit
                bogger:
                    !right
                }
            ]],

            OPT_BOG = "Ask about Hesh",
            DIALOG_BOG = [[
                player:
                    W-what about Hesh? Ain't it got anything to say about this?
                bogger:
                    Hesh? Your god from beneath the sea?
                    If it does exist, then it's as much a creation of the Roaloch as you are. And it stands to lose just as much.
                    Where I come from, Hesh is a name spoken only by outsiders. And soon it won't be a name spoken at all.
            ]],

            OPT_VIX = "Ask about {bogger}",
            DIALOG_VIX = [[
                player:
                    And what about you? How do you know all this, huh?
                bogger:
                    Deep in the bog, there's an old memory. A consciousness as ancient as the squelching mud beneath our feet.
                    It... is often confused. And its grasp on the present is brittle.
                    But it burbles in our ears as we sleep, and my brethren and I have been able to glean certain truths from its ramblings.
                    Many of us worshipped that consciousness, until recently. 
                    Perhaps I'm just in the mood to test boundaries. But if you <i>can</i> stop your brother, I'd love to see you succeed.
            ]],
            OPT_MUTATION = "Ask about your new condition",
            DIALOG_MUTATION = [[
                player:
                    Am I going to be alright?
                bogger:
                    In the short term:
                    No. Definitely not.
                    In the long term...
                    !shrug
                    You have an unusually strong healing factor, even for a Kra'deshi. That might save you.
                    !thought
                    Or it may just prolong your agony.
            ]],
        }
        :Fn(function(cxt) 
            cxt.enc.scratch.broker_here = cxt.quest:GetCastMember("bilebroker"):IsInPlayerParty()
            cxt:Dialog("DIALOG_INTRO")
            
            if cxt.quest.param.smith_did_sample then
                cxt:Dialog("DIALOG_GET_MUTATION")
                cxt:ForceTakeCards{"shifting_mutation"}
            end

            cxt:Dialog("DIALOG_TELL_OF_DANGER")

            cxt:RunLoopingFn(function() 

                if cxt.quest.param.smith_did_sample then
                    cxt:QST("MUTATION")
                end
                cxt:QST("BEACHING")
                cxt:QST("Roaloch")
                cxt:QST("BOG")
                cxt:QST("VIX")

                cxt:Opt("OPT_HEAD_BACK_TO_BAR")
                    :Dialog("DIALOG_HEAD_BACK_TO_BAR")
                    :CompleteQuest()
                    :Travel()
            end)

        end)