
local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    title = "Tei's request",
    on_init = function(quest) 
    end,
    icon = engine.asset.Texture("icons/quests/smith_story_tei.tex"),

    collect_agent_locations = function(quest, t)
        table.insert (t, { agent = quest:GetCastMember("tei"), location =  quest:GetCastMember("meeting_spot"), role = CHARACTER_ROLES.VISITOR} )
    end,

}


:AddObjective{
    id = "start",
    title = "Go to the meeting spot",
    desc = "Check in with {tei}",
    state = QSTATUS.ACTIVE,
    mark = { "tei" },
}

:AddObjective{
    id = "do_job",
    hide_in_overlay = true,
    events = 
    {
        quests_changed = function(quest, event_quest)
            if quest.param.job == event_quest and not event_quest:IsActive() then
                quest:Complete()
            end
        end
    },
}


:AddLocationCast{
    cast_id = "meeting_spot",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("TEI_SHRINE"))
    end,
}

:AddCastByAlias{
    cast_id = "tei",
    alias = "TEI",
}

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo("start", "tei")
    :ConfrontState("CONF")
        :Loc{
            DIALOG_INTRO = [[
                * You've had a long day, so you almost don't remember {tei}'s waiting for you.
                * You figure it's best to follow up, though, since you're still a far cry from getting a leg up on Vix.
                player:
                    !left
                agent:
                    !right
                    I hear you've been busy. Looking forward to a nice reunion?
                player:
                    Not exactly. But I've taken enough gut punches, my insides may as well be hardened leather.
                agent:
                    Hm. Not exactly the bravado you had on display yesterday, is it?
                    All the same, I have another job for you. 
                    And if your heart isn't quite in it, maybe consider that you aren't the only person who Vixmalli has wronged.

            ]],
            
            OPT_QUESTION = "Ask about Vixmalli",
            DIALOG_ANSWER = [[
                player:
                    All this is really to take down Vix? 
                    I get that he's an egomaniac, but he's also just one of a pack. 
                agent:
                    I'm aware that there are tendrils of corruption within the Cult, yes. 
                    But your brother is at its center. And I intend to deracinate him.
                player:
                    Not sure what that means but he probably deserves it. 
                agent:
                    And what about you? Is ostracizing you now the <i>only</i> thing that he's ever done to earn your ire?
                player:
                    Naw. Not really. But Mullifee's the girl, right?
                    Normally she'd become head of the family. 
                    But he always had a way of makin' us feel bad for doing anything but what he wanted.
                agent:
                    Now imagine him doing that but with all of Heshiandom.
            ]],


            OPT_START_WORK = "Ask about the job",
            DIALOG_START_WORK = [[
                player:
                    So lay it on me, ex-salted one. What do you need me to do?
                agent:
                    ...
            ]],
            OPT_ACCEPT_JOB = "Accept the job",
        }

        :Fn(function(cxt) 
            cxt:Dialog("DIALOG_INTRO")

            cxt:RunLoop(function()
                
                cxt:Question("OPT_QUESTION", "DIALOG_ANSWER")

                cxt:Opt("OPT_START_WORK")
                    :SetQuestMark()                
                    :Dialog("DIALOG_START_WORK")
                    :CompleteQuest("start")
                    :Fn(function() 
                        cxt.quest.param.job = TheGame:GetGameState():GetMainQuest():DefFn("SpawnPoolJob", "TEI_JOB", cxt:GetAgent())
                        if cxt.quest.param.job then
                            cxt.quest:Activate("do_job")
                            cxt:PlayQuestConvo(cxt.quest.param.job, QUEST_CONVO_HOOK.INTRO)

                            cxt:Opt("OPT_ACCEPT_JOB")
                                :Fn(function() 
                                    cxt:PlayQuestConvo(cxt.quest.param.job, QUEST_CONVO_HOOK.ACCEPTED)
                                    StateGraphUtil.AddLeaveLocation(cxt)
                                end)
                        end

                        
                    end)
            end)
        end)


