
local QDEF = QuestDef.Define
{
    qtype = QTYPE.STORY,
    title = "Cult Machinations",
    on_init = function( quest )
    end,
    icon = engine.asset.Texture( "icons/quests/smith_story_tei.tex" ),
    
    on_destroy = function( quest )
        if quest:GetCastMember( "tei" ) and quest:GetCastMember( "tei" ):IsInPlayerParty( ) then
            quest:GetCastMember( "tei" ):Dismiss( )
        end
    end,

    on_init = function( quest )
        quest.param.boss_id = quest.param.boss_id or TheGame:GetGameProfile():GetNoStreakRandom("SMITH_DAY_3_BOSS_PICK", {"SHREDMAW", "DROAD"}, 2)
    end,
    
    collect_agent_locations = function( quest, t )
        table.insert ( t, { agent = quest:GetCastMember( "tei" ), location = quest:GetCastMember( "meeting_spot" ), role = CHARACTER_ROLES.VISITOR })
    end,
}

:AddObjective{
    id = "start",
    title = "Talk to {tei}",
    desc = "Check in with {tei}",
    state = QSTATUS.ACTIVE,
    mark = { "tei" },
    
    on_complete = function(quest)
        quest:Activate("ritual")
    end,
}

:AddObjective{
    id = "ritual",
    title = "Attend the ritual",
    desc = "New priests are being anointed by {bishop} at a ritual near the Beaching. Sneak in with {tei}, and kill {bishop}.",
    mark = { "ritual_location" },
    on_complete = function(quest)
        quest:Activate("sleep")
    end,
}


:AddObjective{
    id = "sleep",
    title = "Head back to the bar to sleep",
    desc = "",
    mark = {"moreef"},
    on_activate = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = true
        if quest:GetCastMember("tei") and quest:GetCastMember("tei"):IsInPlayerParty() then
            quest:GetCastMember("tei"):Dismiss()
        end
    end,
    on_complete = function(quest)
        TheGame:GetGameState():GetMainQuest().param.restrict_room = false
    end,
}

:AddLocationCast{
    cast_id = "meeting_spot",
    cast_fn = function( quest, t )
        table.insert( t, TheGame:GetGameState( ):GetLocation( "TEI_SHRINE" ))
    end,
}

:AddQuestLocation{
    cast_id = "ritual_location",
    name = "Ritual Location",
    plax = "EXT_SMITH_BEACHCREATURE",
    map_tags = { "beach" },
}

:AddQuestLocation{
    cast_id = "ritual_location_2",
    name = "Ritual Location",
    plax = "EXT_SMITH_BISHOP_RITUAL",
    map_tags = { "beach" },
}

:AddCastByAlias{
    cast_id = "vixmalli",
    alias = "VIXMALLI",
}

:AddCastByAlias{
    cast_id = "moreef",
    alias = "SWEET_MOREEF",
}

:AddCastByAlias{
    cast_id = "tei",
    alias = "TEI",
}

:AddCastByAlias{
    cast_id = "bishop",
    alias = "BISHOP_OF_FOAM",
    no_validation = true,
}


:AddOpinionEvents{
    killed_the_bishop =  
    {
        delta = OPINION_DELTAS.MAJOR_GOOD,
        txt = "Killed the corrupt bishop",
    },
}

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QDEF:AddConvo( "start", "tei" )
:ConfrontState( "CONF" )
:Loc{
    DIALOG_INTRO = [[
                    tei:
                        Good to see you again, Smith.
                    player:
                        Well, thanks, Tei. That's nice to hear.
                        Most people think I'm too gruff to receive a little sincerity every now and then, ya know?
                    tei:
                        Ah. Well. I <i>sincerely</i> hope you're up for a little assassination.
                    player:
                        ...
                        What the Hesh, Tei.
                    tei:
                        Your brother's right hand, {bishop}, has been using the Beaching as license to anoint as many new priests as possible.
                        My superiors take issue not with {bishop.hisher} fervor but with {bishop.hisher} selection methods: selling the priesthood to the highest bidder.
                        I'd like you to approach {bishop} as an aspirant and then eliminate {bishop.himher}.
                    player:
                        That's a big ask! Killing a bishop? 
                    tei:
                        And what, exactly, do you intend to do with Vixmalli when you get to him?
                    player:
                        Well... 
                        Do I get to wear a funny hat?
                    agent:
                        Of course. You need to look the part.
                ]], 
    
    OPT_Q1 = "Ask about the priesthood",
    DIALOG_Q1 = [[
                    player:
                        How can {bishop} just slap the priesthood on a bunch of goons at once? 
                        Aren't there initiations and doodads that gotta be done?
                    tei:
                        Normally, yes. But these are exceptional times. Pearl-on-the-Foam has been declared holy territory. 
                        Technically, anything the upper echelons do is inherently sanctioned by Hesh.
                    player:
                        So if you stop them, what does that make you?
                    tei:
                        A Luminari. I go where I'm pointed and do what needs to be done.
                        And the consequences are mine alone. 
                ]], 
    OPT_Q2 = "Ask about the bribes",
    DIALOG_Q2 = [[
                    player:
                        No offense, but aren't bribes the <i>standard</i> way priests get chosen?
                    tei:
                        I beg your pardon?
                    player:
                        Vixmalli didn't become a Heshian because he was drawn to the cult.
                        My folks paid big, big tithes and in return he got a place at that fancy school.
                    tei:
                        Well, yes. The church honors the devout with the chance for clergical study.
                    player:
                        Right. And then... those students become priests.
                    tei:
                        I see your point. But they've also applied themselves to studies for years. 
                        The priests {bishop} seeks to anoint are alien to the clergy.
                        They seek only to profit from this chaotic upheaval.
                ]], 
    
    OPT_TAKE_DISGUISE = "Take the disguise",
    DIALOG_TAKE_DISGUISE = [[
                    player:
                        Alright. Gimme the funny hat, then. 
                        I'll kill your bishop for you.
                    tei:
                        !give
                        Remember--if you get caught...
                    player:
                        !take 
                        Yeah, yeah. You were none the wiser.
                        !exit
                ]], 
    DIALOG_TAKE_DISGUISE_2 = [[
                    * The clothes fit surprisingly well. And cozily.
                    player:
                        !left 
                        How do I look?
                    tei:
                        Quite striking, actually. Perhaps your family sent the wrong son to the cult.
                    player:
                        No way. Theroux woulda been terrible at it. 
                ]], 
    
    OPT_HEAD_OUT = "Head out",
    DIALOG_HEAD_OUT = [[
                    player:
                        Let's go, then. I'll do my best not to embarrass you.
                    tei:
                        Appreciated.
                    !exit  
                ]], 
    
}

:Fn( function( cxt )
    cxt:Dialog( "DIALOG_INTRO" )
    
    local has_disguise = false
    cxt:RunLoop( function( )
        cxt:QST( "Q1" )
        cxt:QST( "Q2" )
        
        
        if not has_disguise then
            cxt:Opt( "OPT_TAKE_DISGUISE" )
                :SetQuestMark( )
                :Dialog( "DIALOG_TAKE_DISGUISE" )
                :Fn( function( )
                    has_disguise = true
                    cxt.player.build = Content.GetOutfit( "smith_hesh_disguise" ).build
                    cxt.encounter:PresentAgent( cxt.player, SIDE.LEFT )
                    TheGame:GetGameProfile():AcquireUnlock("smith_hesh_disguise")
                end )
                :Dialog( "DIALOG_TAKE_DISGUISE_2" )
        else
            cxt:Opt( "OPT_HEAD_OUT" )
                :Dialog( "DIALOG_HEAD_OUT" )
                :SetQuestMark( )
                :CompleteQuest( "start" )
                :RecruitMember( PARTY_MEMBER_TYPE.ESCORT )
                :Travel( )
        end
        
    end )
end )

QDEF:AddConvo( "ritual" )
    :ConfrontState( "CONF", function(cxt) return cxt.location == cxt.quest:GetCastMember("ritual_location") end )
    :Loc{
        DIALOG_INTRO = [[
            * Getting close to the Great Beaching is difficult, but with a Luminari at your side you're able to push through the glut of pilgrims.
            * You see what must be {bishop} standing knee-deep in fishy excreta, daubing slime onto the foreheads of those gathered near.
            * Before you can get too close, however, you and {tei} are both stopped by armed guards.
            player:
                !left
            agent:
                !right
                Hold it. Only guests of honor are welcome here.
        ]], 

        OPT_CONVINCE_GUARDS = "Convince {agent} that you're actually with the Cult",
        
        DIALOG_CONVINCE_GUARDS = [[
            player:
                Yield, you low-born slug. I'm gonna be a priest and I got the Luminari to prove it.
        ]], 
        DIALOG_CONVINCE_GUARDS_SUCCESS = [[
            tei:
                !left 
                This man is a member of the clergy of Sludge-by-the-Sea. 
                The Bishop there sent me to see him honored with oaths at the Great Beaching.
            agent:
                Hrm, well. As long as he's of noble birth.
                {bishop} doesn't want any pilgrims getting too close.
            player:
                !left 
                Oh, the noblest. I'm even related to Vixmalli Banquod.
            player:
                !exit
            agent:
                !exit
        ]], 
        DIALOG_CONVINCE_GUARDS_FAILURE = [[
            player:
                Why, I oughta have you lashed for the way you're treating me!
            agent:
                Alright, that's enough! I gave you a chance to walk away, wise guy.
            !fight
        ]], 
        
        OPT_FORCE_WAY_THROUGH = "Force your way through the guards",
        DIALOG_FORCE_WAY_THROUGH = [[
            player:
                I'm not much bothered by what I'm <i>allowed</i> to do, chum.
                !fight 
        ]],
        DIALOG_WON_FIGHT = [[
            {tei_alive?
                * With the guards broken at your feet, there's no one to stop Tei from cussing you out.
                tei:
                    !right 
                    !suspicious
                    $misc_hushedUrgent
                    By the tides, Smith! Do you understand the point of this whole charade?
                player:
                    Hey, Luminari. Relax. 
                    It doesn't even look like {bishop} heard a thing.
                * Sure enough, {bishop}'s face is smeared with loam, {bishop.hisher} eyes closed in rapture as the aspiring nobles kneel around {bishop.himher}.
            }
            {not tei_alive?
                * With the guards broken at your feet, the battle is won.
                player:
                    Aw, Hesh. That ain't good.
                * Tei has fallen, and so has any pretense you had for being there. 
                * Still, the idea of getting one-up on Vixmalli by taking out his second in command is compelling enough. You'll have to take out {bishop} alone. 
            }
            player:
                !exit
            agent:
                !exit
        ]]
    }

    :Fn( function( cxt )
        cxt.enc.scratch.opfor = CreateCombatParty({ "ZEALOT", "LUMINARI", "LUMICYTE" }, 1, cxt.location )
        cxt:TalkTo(cxt.enc.scratch.opfor[1])
        cxt:Dialog( "DIALOG_INTRO" )
        
        cxt:BasicNegotiation( "CONVINCE_GUARDS" )
            :OnSuccess( )
                :GoTo( "STATE_RITUAL" )
            
            :OnFailure( )
                :Fn( function( )
                    cxt:Opt( "OPT_DEFEND" )
                    :Battle{
                            flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE,
                            enemies = cxt.enc.scratch.opfor,
                            no_oppo_limit = true,
                            allies = { "tei" }
                        }
                            :OnWin( )
                                :Fn( function( )
                                    cxt.quest.param.tei_alive = cxt.quest:GetCastMember( "tei" ):IsAlive( )
                                    cxt.quest.param.fought_guards = true
                                    cxt:Dialog("DIALOG_WON_FIGHT")
                                end )
                                :GoTo( "STATE_RITUAL" )
                end )

        cxt:Opt("OPT_FORCE_WAY_THROUGH")
            :Dialog("DIALOG_FORCE_WAY_THROUGH")
            :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    enemies = cxt.enc.scratch.opfor,
                    no_oppo_limit = true,
                    allies = { "tei" }
                }
                    :OnWin( )
                        :Fn( function( )
                            cxt.quest.param.tei_alive = cxt.quest:GetCastMember( "tei" ):IsAlive( )
                            cxt.quest.param.fought_guards = true
                            cxt:Dialog("DIALOG_WON_FIGHT")
                        end )
                        :GoTo( "STATE_RITUAL" )
    end )

:State( "STATE_RITUAL" )
    :Loc{
        DIALOG_MOVE = [[
            * The Heshians are gathered in a patch of sandy coral, exposed by the neap tide. A great beast is chained up at the water's edge, its leash taut.
        ]],

        DIALOG_INTRO = [[
            * The bishop stands just outside the beast's reach, performing the consecration rite for a line of aspirant priests.
            * You can't quite see what they're doing, but it's probably not fun.

            {tei_alive?
                player:
                    !left
                tei:
                    !right
                    {bishop} <i>would</i> pick the Trial of the Spawn. It's archaic, but intimidating. 
                    These aspirants may be able to buy the priesthood, but they can't buy {bishop}'s power.
                player:
                    What the Hesh am I supposed to do? I don't know anything about any of this.
                tei:
                    You're not actually becoming a priest. Get in line, bow graciously, and when it's your turn, shove {bishop} into the beast's mouth.
                    The poor thing has been purposefully starved, so it won't reject anything its fed.   
                    !exit
                player:
                    !exit
            }
            {not tei_alive?
                * If {tei} were here, she'd be able to tell you what's expected. But she isn't.
                * You join the line, continuing your lifelong habit of improvising along the path of least resistance.
            }

            * As you wait your turn, you watch the others each offer a sacrifice to the "spawn".
            * Ceremonial daggers, family tapestries, and Vagrant Age relics are all choked down by the beast.
            player:
                !left
            bishop:
                !right
                Ah, supplicant. You are called by the tides, yes? 
                And what sacrifice did you bring, to pledge yourself to the Abyss?
        ]], 

        OPT_Q1 = "Ask about the trial",
        DIALOG_Q1 = [[
            player:
                Er, forgive me, sire. But I've travelled far for this and, uh, I'm not sure I'm familiar with this practice.
            bishop:
                Hah! No, most parishes wouldn't be. This is the Trial by Spawn. 
                In olden days, kra'deshi supplicants would make corporeal offerings to a beast of the sea.
                The practice was outlawed as other races found it too debilitating. But I've reinstated it.
                You may offer material goods; if the beast accepts your token, then Hesh finds you worthy. 
                If the beast accepts your token, then you've passed. 
        ]],
        OPT_Q2 = "Ask about the offerings",
        DIALOG_Q2 = [[
            player:
                Is there a suggested value of the goods?
            bishop:
                Traditionally, a hand. Or one of the less vital organs.
                But today? Ah, why don't you show me what you're offering? And I'll tell you whether you need to...
                Up the ante.
        ]],

        OPT_STICK_YOUR_HEAD_IN = "Stick your hand in the beast's mouth",
        DIALOG_STICK_YOUR_HEAD_IN = [[
            * You've never been drawn to the cult much. But you picture the look on Vix's face if he realizes his trusted second had you anointed as a priest.
            player:
                !exit 
            * It's too tempting to pass up. You roll up your sleeve and thrust your hand into the beast's mouth.
            * You're close enough to see the creature's nostrils flaring. A bulbous bead of saliva drips from its fang.
            * The beast clamps down just as you change your mind. You're fast, but not fast enough to keep from losing a finger.
            bishop:
                !right 
                !surprise
                You did it! Unbelievable! I--I've never seen anyone attempt it!
            player:
                !left
            * Your hand stings, but you can't neglect the feeling of pride.
            bishop:
                Ah, well, however, it is nevertheless the <i>new</i> custom to, ah, pay a tithe...
            player:
                !dubious
                Hey, I gave you the finger. If you wanted shills, you should've been more specific.
        ]],

        OPT_WARN_THE_BISHOP = "Warn {bishop} about {tei}",
        DIALOG_WARN_THE_BISHOP = [[
            player:
                !suspicious
                $misc_hushedUrgent
                I ain't got much time, {bishop}, but you should know you're in danger.
                That Luminari over there hired me to kill you. But I'm no hitman.
            bishop:
                What?! Who are you? Is this some sort of joke.
            player:
                It ain't no joke! I'm Smith Banquod, and--
            bishop:
                <i>You're</i> Smith? Vixmalli warned me you might try something!
            * {tei} had been keeping a safe distance, but {bishop}'s raised voice draws her in.
            tei: 
                !right
                !fight
                The heresy dies here!
            * {tei} charges the two of you, weapon drawn!
        ]],

        OPT_PUSH_IN_BISHOP = "Push the bishop into the beast's mouth",
        DIALOG_PUSH_IN_BISHOP = [[
            * {bishop} is so busy being full of {bishop.himher}self that {bishop.heshe} never sees it coming.
            * You pantomime outrageously as you pretend to trip, hurling {bishop.himher} directly into the beast's ravenous maw.
            bishop:
                !flinch
                !exit
                #event:/sfx/general/conversation/foley/some_armor/flinch_scream
            * It's over with one gulp, and the other priests shriek in alarm.
            {tei_alive?
                tei:
                    !right
                    Supplicant! Are you alright?
                player:
                    Oh, uh, careful there, Luminari. The ground is unsteady.
                tei:
                    I can see that. Now, let me escort you away from--
                * Tei staggers as the creature lets out a bellowing roar.
            }
            player:
                Uh oh. 
                Think we might've given it a taste for Heshians?
            * Sure enough, the beast tears its chain from its mooring in the sand, and attacks!
        ]],

        OPT_DEFEND = "Defend yourself!",
        DIALOG_DEFEND = [[
            player:
                Ready your butthole, Bishop. 
                !fight
        ]],
        DIALOG_ON_WIN_VS_TEI = [[
            {not tei_alive?
                * The battle is barely over before the noise of rending metal rips through the sea air.
                * Frenzied by the sight of spilled blood, the beast tears free of its chains and lunges towards you.
            }
            {tei_alive?
                player:
                    !placate
                    Calm down, Tei! 
                    No reason you gotta die just 'cause you were following orders.
                tei:
                    !angry_accuse
                    I go as the tides command! 
                    Your whims only weigh me down.
                * Tei turns and, with a mighty heave, releases the chained beast. She throws herself out of the way as it lunges towards you.
            }
        ]],



    }
    :Fn( function( cxt )
        cxt:FadeOut()
        cxt.quest.param.tei_alive = cxt.quest:GetCastMember( "tei" ):IsAlive( )


        cxt.quest.param.boss_id = cxt.quest.param.boss_id or TheGame:GetGameProfile():GetNoStreakRandom("SMITH_DAY_3_BOSS_PICK", {"SHREDMAW", "DROAD"}, 2)
        cxt:ReassignCastMember("beast", TheGame:GetGameState():AddSkinnedAgent(cxt.quest.param.boss_id))
        cxt.encounter:DoLocationTransition(cxt.quest:GetCastMember("ritual_location_2"))
        cxt.quest:GetCastMember("ritual_location_2"):SetMapPos(cxt.quest:GetCastMember("ritual_location"):GetMapPos())        
        cxt:Dialog("DIALOG_MOVE")
        cxt:FadeIn()
        cxt:Dialog("DIALOG_INTRO")

        local has_done_ritual = false
        cxt:RunLoop(function() 
            
            if not has_done_ritual then
                cxt:QST("Q1")
                cxt:QST("Q2")
            end
            
            cxt:Opt("OPT_PUSH_IN_BISHOP")
                :Dialog("DIALOG_PUSH_IN_BISHOP")
                :ReceiveOpinion("killed_the_bishop", nil, "tei")
                :Fn(function()
                        cxt.quest:GetCastMember("bishop"):Kill()
                    end)
                :GoTo("STATE_BOSS_FIGHT")

            local tei = cxt.quest:GetCastMember("tei")
            if not tei:IsRetired() then
                cxt:Opt("OPT_WARN_THE_BISHOP")
                    :Dialog("DIALOG_WARN_THE_BISHOP")
                    :ReceiveOpinion(OPINION.BETRAYED, nil, tei)
                    :ReceiveOpinion(OPINION.SAVED_LIFE, nil, cxt.quest:GetCastMember("bishop"))
                    :Fn(function() 
                        cxt.quest.param.warned_bishop = true 
                        cxt:Opt("OPT_DEFEND")
                            :Dialog("DIALOG_DEFEND")
                            :Battle{
                                flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE,
                                allies = {"bishop"},
                                enemies = {"tei"},
                            }
                                :OnWin()
                                    :Fn(function() 
                                        cxt.quest.param.tei_alive = cxt.quest:GetCastMember( "tei" ):IsAlive( )
                                        cxt.quest.param.bishop_alive = cxt.quest:GetCastMember( "bishop" ):IsAlive( )
                                        cxt:Dialog("DIALOG_ON_WIN_VS_TEI")
                                        
                                        if cxt.quest.param.tei_alive then
                                            cxt.quest:GetCastMember( "tei" ):MoveToLimbo()
                                        end

                                        cxt:GoTo("STATE_BOSS_FIGHT")
                                    end)

                    end)
            end
            

            if not has_done_ritual then
                cxt:Opt("OPT_STICK_YOUR_HEAD_IN")
                    :Dialog("DIALOG_STICK_YOUR_HEAD_IN")
                    :Fn(function() 
                        has_done_ritual = true 
                        cxt.player.graft_owner:AddGraft( GraftInstance("claimed_by_hesh") )
                    end)
            end

        end)
    end )

    :State("STATE_BOSS_FIGHT")
        :Loc{
            OPT_DEFEND = "Defend yourself against the beast!",
            DIALOG_DEFEND = [[
                player:
                    Ho boy. 
                    I can't say I'm trained for this but...
                    I'm not trained for anything, really.
                    !fight
            ]],
            DIALOG_ON_WIN_VS_BEAST_NO_LIVING_ALLY = [[
                * The beast is dead, and the Heshians have scurried, leaving you alone with nothing but bloated beach corpses.
                player:
                    Ugh. This is definitely the worst party I've ever crashed.
                    Better blow this place before the Cult sends reinforcements.
                !exit 
            ]],

            DIALOG_ON_WIN_VS_BEAST_TEI_ALLY = [[
                * The beast is dead and the Heshians have scurried, leaving you alone with Tei.
                tei:
                    !right
                    Well, that was an unfortunate complication.
                player:
                    !left
                    What now?
                tei:
                    Well, <i>you</i> get out of here, before there are any questions you find difficult to answer.
                    I'll find you later.
                player:
                    Yeah, I guess you will.
                    See ya.
                    !exit
            ]],

            DIALOG_ON_WIN_VS_BEAST_BISHOP_ALLY = [[
                * The beast is dead and the other Heshians have scurried, leaving you alone with {bishop}.
                player:
                    !left
                    Alright there, bishop?
                bishop:
                    !right
                    Well, I suppose so. Now I won't have to wait for the beast to void itself in order to claim the offerings.
                player:
                    Yeah, I guess that's lucky.
                    Wait, wha--
                bishop:
                    And you're Smith, is that right? I'd been led to believe you were seeking revenge on Vixmalli. 
                    Why betray your Luminari ally to spare me?
                player:
                    Well, I dunno. I just didn't feel like doing a hitjob I guess.
                    Look, I'm no scumbag. I'm just a guy who likes to live by my own rules. You can relate to that, right?
                bishop:
                    ...
                player:
                    Tomorrow, I'm gonna talk to Vix. If you could, you know, put in a good word for me...
                    That's all I want in return.
                bishop:
                    I... suppose that's simple enough. 
                    Very well.
                    Good day, young Banquod. I have some artifacts to retrieve.
                * You leave {bishop} to squelch around in the beasts's viscera. 
            ]],


        }
        :Fn(function(cxt) 
            
            local ally
            if cxt.quest.param.warned_bishop then
                if cxt.quest:GetCastMember("bishop"):IsAlive() then
                    ally = cxt.quest:GetCastMember("bishop")
                end
            else
                if cxt.quest:GetCastMember("tei"):IsAlive() then
                    ally = cxt.quest:GetCastMember("tei")
                end
            end
            
            local allies = {ally}

            cxt:Opt("OPT_DEFEND")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                        flags = BATTLE_FLAGS.SELF_DEFENCE | BATTLE_FLAGS.NO_FLEE | BATTLE_FLAGS.BOSS_FIGHT,
                        allies = allies,
                        enemies = {"beast"},
                    }
                        :OnWin()
                            :Fn(function() 
                                if not ally or ally:IsDead() then
                                    cxt:Dialog("DIALOG_ON_WIN_VS_BEAST_NO_LIVING_ALLY")
                                else
                                    if cxt.quest.param.warned_bishop then
                                        cxt:Dialog("DIALOG_ON_WIN_VS_BEAST_BISHOP_ALLY")
                                    else
                                        cxt:Dialog("DIALOG_ON_WIN_VS_BEAST_TEI_ALLY")
                                    end
                                end
                            end)
                            :CompleteQuest("ritual")
                            :Travel()
        end)

if IsEarlyAccess() then

    QDEF:AddConvo("sleep", "moreef")
        :Loc{
            --  Possible states:
            --    you sided with tei:
            --      tei is alive, and likes you for helping. Partner is dead or captured
            --      tei is dead, partner is dead or captured
            --    
            --    you sided with partner:
            --      tei is dead, partner likes you
            --      tei is alive, and partner didn't figure out you were working with her, so they like you
            --      tei is alive, and partner doesn't like you because you set them up.
            

            OPT_GET_A_ROOM = "Head to your room",
            
            DIALOG_GET_ROOM = [[
                * Back at the Hideaway, you and Moreef catch up. 
                * {moreef} opens a door behind the bar for you.
                moreef:
                    !exit
            ]],

            DIALOG_WAKE = [[
                * You wake up, feeling groggy in the morning light.
            ]],
        }

        :Hub(function(cxt)
            cxt:Opt("OPT_GET_A_ROOM")
                :SetQuestMark()
                :Dialog("DIALOG_GET_ROOM")
                :PreIcon(global_images.sleep)
                :Fn(function()
                    cxt.quest:Complete()
                    local grog = cxt.location
                    cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("MOREEF_BAR.inn_room") )
                    grog:SetPlax()
                    ConvoUtil.DoSleep(cxt, "DIALOG_WAKE")
                    cxt:Opt("OPT_LEAVE")
                        :MakeUnder()
                        :Fn(function() 
                            cxt:AwardUnlock("SMITH_BRAWL_PLAYABLE", "SMITH", LOC"UI.CHARACTER_UNLOCKED.SMITH_BRAWL_TITLE", LOC"UI.CHARACTER_UNLOCKED.SMITH_BRAWL_BODY")
                            DoSmithMessage()
                            --cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("MOREEF_BAR") )
                            cxt:End()
                        end)
                end)
            end)
else

    QDEF:AddConvo("sleep", "moreef")
        :Loc{

            
            --  Possible states:
            --    you sided with tei:
            --      tei is alive, and likes you for helping. Partner is dead or captured
            --      tei is dead, partner is dead or captured
            --    
            --    you sided with partner:
            --      tei is dead, partner likes you
            --      tei is alive, and partner didn't figure out you were working with her, so they like you
            --      tei is alive, and partner doesn't like you because you set them up.
            

            OPT_GET_A_ROOM = "Head to your room",
            
            DIALOG_GET_ROOM = [[
                * Back at the Hideaway, you and Moreef catch up. 
                * {moreef} opens a door behind the bar for you.
                moreef:
                    !exit
            ]],

            DIALOG_WAKE = [[
                * You wake to air thicker than normal. 
                vixmalli:
                !right
                * A shape congeals out of the shadows into something you might recognize as your brother, Vixmalli, if there wasn't something... off.
                player:
                    !tired 
                    Well, look what the tide brought in. 
                    Have to admit, I didn't expect you to come find me. But then I guess I have given you a lot of things to regret these past couple of days, huh?
                vixmalli:
                    ...
                    You're running out of time. 
                player:
                    Er... what? To do what?
                    Beat your head in before it vanishes up your ass?
                vixmalli:
                    To realize whatever potential you've been trying to unfurl all these years.
                    Soon the behemoth will unfurl, and then it will be out of your hands.
                    The world will be remade in the new god's image, and the tides of Hesh will dry into the cracked earth.
            ]],

            OPT_Q1 = "Ask about Vixmalli",
            DIALOG_Q1 = [[
                player:
                    !placate 
                    I may be a godless rube, but that sounded an awful lot like heresy.
                    You doin' alright, Vix? Did I rattle you so bad you've lost touch with reality?
                vixmalli:
                    Ah, yes. That old liferaft. 
                    I thought you always knew that. Living moment to moment, enjoying only what was manifest before you.
                    No, brother. I'm as well as I will ever be again. 
                    Once I'm remade--once the new god curls around me--there's no telling what my lot will be.
                player:
                    What the Hesh are you talking about?! You're a damn priest! 
                vixmalli: 
                    We're all damned, Smith. My priesthood has nothing to do with it. 
            ]],
            OPT_Q2 = "Ask about being remade",
            DIALOG_Q2 = [[
                player:
                    What's this "remade" talk, huh? What's that supposed to mean?
                vixmalli:
                    You'll see. Or perhaps you won't.
                    Our cells regenerate faster than most, but even then we don't notice the turnover. 
                    And you--you've been battered and bruised enough to become a new Smith twenty times over. 
                    Have you felt it? The slow becoming of someone new?
                player:
                    I mean, I guess I'm feeling that now--
                    Thirty seconds ago I was a guy waking up all refreshed. Now I'm a guy who just wants to go back to bed.
            ]],
            OPT_Q3 = "Ask what Vix wants",
            DIALOG_Q3 = [[
                player:
                    What are you doing here, Vix? You have to know I was gunning for ya.
                    You gonna let me back into the family, or just spit nonsense and hope I'll forget what you did?
                vixmalli:
                    You wish for a seat at the table? Perhaps even to <i>helm</i> the Banquod name?
                    Fine. As you please. I no longer care.
                    Stay away from the Beach, and you may even last long enough to enjoy whatever solace that brings you.
                player:
                    Last long enough--hey, are you being serious? 
                * Vixmalli stares into the middle distance, as if listening to a song only he can hear. Suffice it to say, it's creepy.    
            ]],

            OPT_SET_OUT = "Ask Vixmalli to leave",
            DIALOG_SET_OUT = [[
                player:
                    Look, if you're just gonna be a creepy weirdo, you can leave.
                * Without a word, Vixmalli slips out the door. 
                vixmalli:
                    !exit
                player:
                    H-hey, wait, I still gotta--
                * You make to pursue him, but you're still tangled in the bed sheets, and lose your footing.
                * By the time you exit your room, he's gone. 


            ]],   
        }

        :Hub(function(cxt)
            cxt:Opt("OPT_GET_A_ROOM")
                :SetQuestMark()
                :Dialog("DIALOG_GET_ROOM")
                :PreIcon(global_images.sleep)
                :Fn(function()
                    local grog = cxt.location
                    cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("MOREEF_BAR.inn_room") )
                    grog:SetPlax()
                    
                    ConvoUtil.DoSleep(cxt, nil, function() 
                        cxt:Dialog("DIALOG_WAKE")
                        cxt:TalkTo("vixmalli")
                        cxt:RunLoopingFn(function() 
                            
                            cxt:QST("Q1")
                            cxt:QST("Q2")
                            cxt:QST("Q3")
                            
                            cxt:Opt("OPT_SET_OUT")
                                :SetQuestMark()
                                :Dialog("DIALOG_SET_OUT")
                                :Fn(function() 
                                    if cxt:GetAgent() then
                                        cxt:GetAgent():MoveToLimbo() 
                                    end
                                    cxt.quest:Complete()
                                    cxt.encounter:DoLocationTransition( TheGame:GetGameState():GetLocation("MOREEF_BAR") )
                                    cxt:End()
                                end)
                                
                        end)
                    end)
                end)
            end)
end