-- The key is the epoch generated from time (eg. GetEpochDay)
-- Fields are optional, and override the generated daily parameters where they exist.

local DAILY_OVERRIDES =
{
    [18459] = {
        -- act_id = "SMITH",
        -- id = "DAILY_ESCAPE_CAVE",
        -- --seed = 999,
        -- mutators = { },
    },

    [18522] = {
        id = "DAILY_BOSS_NEGOTIATION",
    },
    [18523] = {
        id = "DAILY_BOSS_NEGOTIATION",
    },
    [18524] = {
        id = "DAILY_BOSS_NEGOTIATION",
    },
    [18516] = {
        id = "DAILY_BOSS_NEGOTIATION",
    },
    [18517] = {
        id = "DAILY_BOSS_NEGOTIATION",
    },
    [18518] = {
        id = "DAILY_BOSS_NEGOTIATION",
    },
    -- -- Conversion to new daily seeding.
    [18527] = {
        seed = { 1600732800 },
    },
    -- [18528] = {
    --     seed = { 1600819200 },
    -- },
}

--these feat/mutator combinations are known bad always. specific dailies can add more bad combos if they want to
local common_bad_combos = 
{

    {"parasitology", "not_a_scratch"},
    {"battle_bundle", "negotiation_bundle"},
    {"contaminated_equipment", "negotiation_bundle"},
    {"contaminated_equipment", "battle_bundle"},
    {"contaminated_equipment", "parasitology"},
    {"mutator", "cheapskate"},
    {"keepsake_mutator", "score_card_mutator"},
    {"mental_overload", "roll_with_it"},

}


local function GenerateDaily( time )
    local day, seconds = GetEpochDay(time)
    local override = DAILY_OVERRIDES[ day ] or table.empty

    -- Initialize RNG
    local hash = engine.inst:HashString( tostring(seconds) )
    local w, z = hash >> 16, hash % 4294967295 
    if override.seed then
        w, z = table.unpack( override.seed )
    end
    local rng = engine.Random( w, z )

    -- Pick Quest
    local quest_def
    if override.id then
        quest_def = Content.GetQuestDef( override.id )
    else
        local t = {}
        for k, questdef in pairs( Content.GetAllQuests() ) do
            if questdef.qtype == QTYPE.DAILY and (not questdef.dev_only or BRANCH == "dev") then
                table.insert(t, questdef)
            end
        end
        table.sort(t, function(a,b) return a.id > b.id end)
        local idx = 1 + day % #t
        quest_def = t[ idx ]
    end

    local act_id = override.act_id or quest_def.ChooseCharacter and quest_def:ChooseCharacter(rng) or "SAL"

    local mutators, feats
    if quest_def.GenerateMutatorsAndFeats then
        mutators, feats = quest_def:GenerateMutatorsAndFeats(rng, act_id)
    end
    mutators = override.mutators or mutators
    feats = override.feats or feats

    local params = quest_def.GenerateParams and quest_def:GenerateParams(rng)
    
    local progression = quest_def.GenerateProgressionData and quest_def:GenerateProgressionData(params)

    local daily_table = {def = quest_def, id = quest_def.id, character = act_id, mutators = mutators, feats = feats, params = params, progression = progression, time = time, day = day, worldseed = { rng:GetSeeds() } }
    
    return daily_table
end


local function GenerateMutatorsAndFeats(rng, character_id, arg)

    local mutators = {}
    local feats = {}

    local char = Agent(character_id)
    local function ValidForCharacter(id)
        local def = Content.GetGraft(id)
        if def == nil or not AgentUtil.CanUseGraft(char, def) then
            return false
        end
        return true
    end

    local function is_using(val)
        return table.arrayfind(mutators, val) or table.arrayfind(feats, val)
    end

    


    local function is_bad_addition(v, bad_combos)
        if not bad_combos then
            return false
        end
        
        for _,bad_combo in pairs(bad_combos) do
            
            local is_bad = true
            for _, bad in ipairs(bad_combo) do
                local is_using_this_one = bad == v or is_using(bad)
                if not is_using_this_one then
                    is_bad = false
                    break
                end
            end
            if is_bad then
                return true
            end
        end
        return false

    end

    local function TryAddMutator(mutator)
        if not table.arrayfind(mutators, mutator) and ValidForCharacter(mutator) and not is_bad_addition(mutator, arg.bad_combos) and not is_bad_addition(mutator, common_bad_combos) then
            table.insert(mutators, mutator)
            return true
        end
        return false
    end


    local function TryPickMutator(list)
        if list and #list > 0 then
            local candidates = {}
            
            for k,v in ipairs(list) do
                if not table.arrayfind(mutators, v) and ValidForCharacter(v) and not is_bad_addition(v, arg.bad_combos) and not is_bad_addition(v, common_bad_combos) then
                    table.insert(candidates, v)
                end
            end
            if #candidates > 0 then
                local mutator = table.seeded_pick(candidates, rng)
                table.insert(mutators, mutator)
                return true
            end
        end
    end

    if arg.feats and #arg.feats > 0 then
        local candidates = {}
        for _, feat_list in ipairs(arg.feats) do
            local bad = false
            for _, feat in ipairs(feat_list) do
                if not ValidForCharacter(feat) then
                    bad = true
                    break
                end
            end
            if not bad then
                table.insert(candidates, feat_list)
            end
        end

        if #candidates > 0 then
            local picked_feats = table.seeded_pick(candidates, rng)
            for k,v in ipairs(picked_feats) do
                table.insert(feats, v)
            end
        end
    end

    if arg.always_take then
        for k,v in ipairs(arg.always_take) do
            table.insert(mutators, v)
        end
    end

    TryPickMutator(arg.battle_draft)
    TryPickMutator(arg.negotiation_draft)
    
    table.reverse(mutators)

    if arg.mutators then
        local possible_mutators = shallowcopy(arg.mutators)
        table.seeded_shuffle(possible_mutators, rng)

        local min = arg.min_mutators or 0
        local max = math.max(arg.max_mutators or 0, min)
        local num_mutators_to_add = rng:Random(min, max)

        for _, mutator in ipairs(possible_mutators) do
            if TryAddMutator(mutator) then
                num_mutators_to_add = num_mutators_to_add - 1
                if num_mutators_to_add <= 0 then
                    break
                end

            end

        end
    end
    
    return mutators, feats
end



return {
    GenerateDaily = GenerateDaily,
    GenerateMutatorsAndFeats = GenerateMutatorsAndFeats,
}