local DataUtil = class( "DataUtil" )
DataUtil.can_reload = true

local function clean_str(str)
    return "\""..(str or "").."\""
end

local COLS =
{
    ID = { name="Agent ID", fn = function(agent) return agent, agent:GetID() end },
    NAME = { name="Agent", fn = function(agent) return agent, agent:GetName() end },
    HOME = { name="Home ID", fn = function(agent) 
        return agent:GetBrain():GetHome(), agent:GetBrain():GetHome() and agent:GetBrain():GetHome():GetID() or 0
    end},
    HOME_NAME = { name="Home Name", fn = function(agent) return agent:GetBrain():GetHome() and agent:GetBrain():GetHome():GetName() end},

    WHERE = {name = "Where ID", fn = function(agent) 
        if agent:GetLocation() then
            return agent:GetLocation(), agent:GetLocation():GetID()
        else
            return nil, 0
        end
    end},

    WHERE_NAME = {name = "Where Name", fn = function(agent) 
        if agent:GetLocation() then
            return agent:GetLocation(), agent:GetLocation():GetName()
        end
    end},

    ACTIVITY = { name = "Activity", fn = function(agent)  
        if agent:GetBrain():IsOnDuty() then
            return "DUTY"
        elseif agent:GetBrain():IsPatronizing() then
            return "R_N_R"
        else
            return "???"
        end
    end},
    WORK_ID = { name="Work ID", fn = function(agent) 
        if agent:GetBrain():GetWorkPosition() and agent:GetBrain():GetWorkPosition():GetLocation() then
            return agent:GetBrain():GetWorkPosition():GetLocation(), agent:GetBrain():GetWorkPosition():GetLocation():GetID() 
        else
            return nil, 0
        end
    end},
    WORK_TITLE = { name="Work Title", fn = function(agent) return agent:GetBrain():GetWorkPosition() and agent:GetBrain():GetWorkPosition():GetTitle() end},
    DEF =  { name="CharacterDef", fn = function(agent) return agent.def, agent.def.id end},
    RENOWN =  { name="Renown", fn = function(agent) return agent:GetRenown() end},
    FACTION =  { name="Faction", fn = function(agent) return agent:GetFaction(), agent:GetFaction() and agent:GetFaction():GetName() end},
    CONNECTIONS = { name="Connections", fn = function(agent) 
        local connections = agent:GetSocialConnections()
        local num = 0
        if connections then
            for other_agent in connections:Relationships() do
                num = num + 1
            end
        end
        return num
    end},
    FRIENDS = { name="Friends", fn = function(agent) 
        local connections = agent:GetSocialConnections()
        local num_friends = 0
        if connections then
            for other_agent in connections:Relationships() do
                if agent:GetRelationship(other_agent) >= RELATIONSHIP.NEUTRAL then
                    num_friends = num_friends + 1
                end
            end
        end
        return num_friends
    end},
    ENEMIES = { name="Enemies", fn = function(agent) 
        local connections = agent:GetSocialConnections()
        local num = 0
        if connections then
            for other_agent in connections:Relationships() do
                if agent:GetRelationship(other_agent) <= RELATIONSHIP.NEUTRAL then
                    num = num + 1
                end
            end
        end
        return num 
    end},
}

local function GetParentName(location)
    local root = location:GetRootLocation()
    if root then
        return root:GetName()
    end
end

local LOCATION_COLS =
{
    ID = { name="Location ID", fn = function(location) return location, location:GetID() end },
    NAME = { name="Location Name", fn = function(location) return location, location:GetName() end },
    PARENT_NAME = { name="Parent Name", fn = function(location) return location, GetParentName(location) end },
    IS_PARENT = { name="IsParent", fn = function(location) return location, tostring(GetParentName(location) == location:GetName()) end },
}




function DataUtil.Agents(faction_id)
    local cols = {
        COLS.ID,
        COLS.NAME,
        COLS.FACTION,
        COLS.WHERE,
        COLS.WHERE_NAME,        
        --COLS.ACTIVITY,
        COLS.HOME,
        COLS.HOME_NAME,

        COLS.WORK_ID,
        COLS.WORK_TITLE,
        COLS.RENOWN,
        COLS.DEF,
        --COLS.SPAWNER,
        --COLS.CONNECTIONS,
        --COLS.FRIENDS,
        --COLS.ENEMIES,
    }

    local filter = function(agent)
        return agent:IsSentient() and (not faction_id or agent:GetFactionID() == faction_id)
    end

    local tab = DebugTabular( cols, TheGame:GetGameState().agents, filter )
    TheGame:GetDebug():CreatePanel( tab )

    return tab
end

function DataUtil.Locations()
    local cols = {
        LOCATION_COLS.ID,
        LOCATION_COLS.NAME,
        LOCATION_COLS.PARENT_NAME,
        LOCATION_COLS.IS_PARENT,
    }

    local tab = DebugTabular( cols, TheGame:GetGameState().locations )
    TheGame:GetDebug():CreatePanel( tab )

    return tab
end

function DataUtil.AgentDefs()

    local get_avg_health = function(def)
        local max_health = def.fight_data.MAX_HEALTH and def.fight_data.MAX_HEALTH or 0
        return type(max_health) == "table" and math.round((max_health[1] + max_health[2])/2) or def.fight_data.MAX_HEALTH
    end

    local cols = {
        { name="ID", fn = function(def) return def, def.id end },
        { name="BaseDef", fn = function(def) return def, def.base_def end },
        { name="Title", fn = function(def) return def, def.title end },
        { name="Name", fn = function(def) return def, def.name end },
        { name="Faction", fn = function(def) return def, def.faction_id end },
        { name="CombatStrength", fn = function(def) return def, def.combat_strength or 0 end },
        { name="Renown", fn = function(def) return def, def.renown or 1 end },
        { name="MinHealth", fn = function(def) 
            local max_health = def.fight_data.MAX_HEALTH and def.fight_data.MAX_HEALTH or 0
            local num = type(max_health) == "table" and max_health[1] or def.fight_data.MAX_HEALTH
            return def, num end },
        { name="MaxHealth", fn = function(def) 
            local max_health = def.fight_data.MAX_HEALTH and def.fight_data.MAX_HEALTH or 0
            local num = type(max_health) == "table" and max_health[2] or def.fight_data.MAX_HEALTH
            return def, num end },
        { name="AvgHealth", fn = function(def) 
            return def, get_avg_health(def) end },
        { name="SupportNegotiation", fn = function(def) return def, def.support_negotiation or "" end },        
        { name="LovedGraft", fn = function(def) return def, def.loved_graft or "" end },        
        { name="HatedGraft", fn = function(def) return def, def.hated_graft or "" end },        
        { name="DesignerNote", fn = function(def) return def, clean_str(def.designer_note) end },
    }

    local function FilterCard( def )
        return not (def.faction_id == "PLAYER" or def.faction_id == "NEUTRAL" or def.base_def == "BASE") and def.title
    end

    local defs = copyvalues( Content.GetAllCharacterDefs() )
    local tab = DebugTabular( cols, defs, FilterCard )
    TheGame:GetDebug():CreatePanel( tab )

    return tab
end

function DataUtil.Grafts()

    local cols = {
        { name="ID", fn = function(def) return def, def.id end },
        { name="Name", fn = function(def) return def, def.name end },
        { name="Desc", fn = function(def) 
            local desc = def.desc_fn and def.desc_fn(def, def.desc) or def.desc
            return def, clean_str(desc) end },
        { name="Type", fn = function(def) return def, GRAFT_TYPE_NAMES[def.type] end },
        { name="Rarity", fn = function(def) return def, def.rarity end },
        { name="IsGood", fn = function(def) return def, def.is_good == true and "TRUE" or "FALSE" end },
       
    }

    local cards = copyvalues( Content.GetAllGrafts() )
    local tab = DebugTabular( cols, cards, nil )
    TheGame:GetDebug():CreatePanel( tab )

    return tab

end

function DataUtil.BattleCards()
    local battle_defs = require "battle/battle_defs"
    local CARD_FLAGS = battle_defs.CARD_FLAGS


    local cols = {
        { name="ID", fn = function(def) return def, def.id end },
        { name="Name", fn = function(def) return def, def.name end },
        { name="Cost", fn = function(def) return def, def.cost end },
        { name="Type", fn = function(def)
            if CheckBits( def.flags, CARD_FLAGS.ITEM ) then
                return def, "ITEM"
            elseif CheckBits( def.flags, CARD_FLAGS.SKILL ) then
                return def, "SKILL"
            elseif CheckBits( def.flags, CARD_FLAGS.RANGED ) or CheckBits( def.flags, CARD_FLAGS.MELEE ) then
                return def, "ATTACK"
            elseif CheckBits( def.flags, CARD_FLAGS.STATUS ) then
                return def, "STATUS"
            end
        end },
        { name="Rarity", fn = function(def) return def, def.rarity end },
        { name="MinDmg", fn = function(def)
            if def.min_damage then
                return def, def.min_damage
            else
                return def
            end
         end },
        { name="MaxDmg", fn = function(def)
            if def.max_damage then
                return def, def.max_damage
            else
                return def
            end
         end },
        { name="AvgDmg", fn = function(def)
            if def.min_damage and def.max_damage then
                return def, (def.min_damage + def.max_damage)/2
            else
                return def
            end
         end },
        { name="IsUpgrade", fn = function(def) 
            return def, def.id:match( "(.*)_plus.*$" ) and "TRUE" or "FALSE"
        end },
        { name="Desc", fn = function(def) 
            local card = Content.GetBattleCard( def.id )
            local battle_defs = require "battle/battle_defs"
            local desc = CardEngine.GenerateCardDesc( card, nil, nil, battle_defs  )
            return def, clean_str(desc)
        end },
        { name = "Series", fn = function(def)
            return def, def.series
        end }
    }

    local function FilterCard( def )
        return not CheckAnyBits( def.flags, CARD_FLAGS.SPECIAL | CARD_FLAGS.NPC )
    end

    local cards = copyvalues( Content.GetAllBattleCards() )
    local tab = DebugTabular( cols, cards, FilterCard )
    TheGame:GetDebug():CreatePanel( tab )

    return tab
end

function DataUtil.NegotiationCards()
    local negotiation_defs = require "negotiation/negotiation_defs"
    local CARD_FLAGS = negotiation_defs.CARD_FLAGS

    local cols = {
        { name="ID", fn = function(def) return def, def.id end },
        { name="Name", fn = function(def) return def, def.name end },
        { name="Cost", fn = function(def) return def, def.cost end },
        { name="Type", fn = function(def)
            if CheckBits( def.flags, CARD_FLAGS.ITEM ) then
                return def, "ITEM"
            elseif CheckBits( def.flags, CARD_FLAGS.DIPLOMACY ) then
                return def, "DIPLOMACY"
            elseif CheckBits( def.flags, CARD_FLAGS.HOSTILE ) then
                return def, "HOSTILE"
            elseif CheckBits( def.flags, CARD_FLAGS.MANIPULATE ) then
                return def, "MANIPULATE"
            elseif CheckBits( def.flags, CARD_FLAGS.BYSTANDER ) then
                return def, "BYSTANDER"
            elseif CheckBits( def.flags, CARD_FLAGS.STATUS ) then
                return def, "STATUS"
            end
        end },
        { name="Rarity", fn = function(def) return def, def.rarity end },
        { name="MinDmg", fn = function(def)
            if def.min_persuasion then
                return def, def.min_persuasion
            else
                return def
            end
         end },
        { name="MaxDmg", fn = function(def)
            if def.max_persuasion then
                return def, def.max_persuasion
            else
                return def
            end
         end },
        { name="AvgDmg", fn = function(def)
            if def.min_persuasion and def.max_persuasion then
                return def, (def.min_persuasion + def.max_persuasion)/2
            else
                return def
            end
         end },
        { name="Desc", fn = function(def) 
            local card = Content.GetNegotiationCard( def.id )
            local battle_defs = require "negotiation/negotiation_defs"
            local desc = CardEngine.GenerateCardDesc( card, nil, nil, negotiation_defs )
            return def, clean_str(desc)
        end },
        { name="IsUpgrade", fn = function(def) 
            return def, def.id:match( "(.*)_plus.*$" ) and "TRUE" or "FALSE"
        end },
        { name = "Series", fn = function(def)
            return def, def.series
        end }
    }

    local function FilterCard( def )
        return not CheckAnyBits( def.flags, CARD_FLAGS.OPPONENT )
    end

    local cards = copyvalues( Content.GetAllNegotiationCards() )
    local tab = DebugTabular( cols, cards, FilterCard )
    TheGame:GetDebug():CreatePanel( tab )

    return tab
end



function DataUtil.AllCards()

    local cols = {
        { name="ID", fn = function(def) return def, def.id end },
        { name="Name", fn = function(def) return def, def.name end },
        { name="Icon", fn = function(def) return def, def.icon end },
        { name="Type", fn = function(def) return def, def.card_type end },
        { name="Rarity", fn = function(def) return def, def.rarity end },
        { name="Desc", fn = function(def) return def, def.desc end },
        { name = "Series", fn = function(def) return def, def.series end }
    }

    local t = {}

    do
        local negotiation_defs = require "negotiation/negotiation_defs"
        local CARD_FLAGS = negotiation_defs.CARD_FLAGS
        for id, def in pairs( Content.GetAllNegotiationCards() ) do
            if not CheckAnyBits( def.flags, CARD_FLAGS.OPPONENT | CARD_FLAGS.BYSTANDER ) then
                local id = def.id
                assert( not CheckBits( def.flags, CARD_FLAGS.BYSTANDER ))
                local card = Negotiation.Card( id )
                local card_type = GetCardType( card )
                local desc = card:GetDesc()
                desc = desc and SanitizeString( desc )

                table.insert( t,
                {
                    id = id,
                    name = def.name,
                    icon = def.path_str,
                    rarity = def.rarity,
                    desc = desc,
                    card_type = card_type,
                    series = def.series,
                })
            end
        end
    end

    do
        local battle_defs = require "battle/battle_defs"
        local CARD_FLAGS = battle_defs.CARD_FLAGS
        for i, def in ipairs( Content.GetAllBattleCards() ) do
            if not CheckAnyBits( def.flags, CARD_FLAGS.SPECIAL | CARD_FLAGS.NPC ) then
                local card = Battle.Card( def.id )
                local card_type = GetCardType( card )
                local desc = card:GetDesc()
                desc = desc and SanitizeString( desc )

                table.insert( t,
                {
                    id = def.id,
                    name = def.name,
                    icon = def.path_str,
                    rarity = def.rarity,
                    desc = desc,
                    card_type = card_type,
                    series = def.series,
                })
            end
        end
    end

    local tab = DebugTabular( cols, t )
    TheGame:GetDebug():CreatePanel( tab )

    return tab
end



function DataUtil.AllGrafts()

    local cols = {
        { name="ID", fn = function(def) return def, def.id end },
        { name="Name", fn = function(def) return def, def.name end },
        { name="Icon", fn = function(def) return def, def.icon end },
        { name="Type", fn = function(def) return def, def.graft_type end },
        { name="Rarity", fn = function(def) return def, def.rarity end },
        { name="Desc", fn = function(def) return def, def.desc end },
        { name = "Series", fn = function(def) return def, def.series end }
    }

    local t = {}

    do
        for i, def in ipairs( Content.GetAllGrafts() ) do
            local id = def.id
            local graft = GraftInstance( id )
            local desc = graft:GetDesc()
            desc = desc and SanitizeString( desc )

            local graft_type = graft:GetType()
            if graft_type == GRAFT_TYPE.SOCIAL then
                graft_type = def.is_good and "BOON" or "BANE"
            end

            table.insert( t,
            {
                id = id,
                name = def.name,
                icon = def.img_path,
                rarity = def.rarity,
                desc = desc,
                graft_type = graft_type,
                series = def.series,
            })
        end
    end

    local tab = DebugTabular( cols, t )
    TheGame:GetDebug():CreatePanel( tab )

    return tab
end