
local function ShowSpawnBattleCard( self, ui )
    local DebugUtil = require "debug/debug_util"
    local player = TheGame:GetGameState():GetPlayerAgent()
    
    if not self.set_focus then
        ui:SetKeyboardFocusHere()
        self.set_focus = true
    end

    local search_txt = ui:InputText( "Search", self.battle_search_txt or "" )
    if self.battle_search_txt == nil then
        search_txt = DebugUtil.GetLocalDebugData( "battle_card_search" ) or ""
    end
    if search_txt and self.battle_search_txt ~= search_txt then
        local results = {}
        for i, v in ipairs( Content.GetAllBattleCards() ) do
            local k = v.id
            local ok, i, j = pcall( string.find, k, search_txt )
            if ok and i then
                table.insert( results, { id = k, graft = v, p = #search_txt / #k + (i == 1 and 1 or 0)} )
            else
                ok, i, j = pcall( string.find, v.name, search_txt )
                if ok and i then
                    table.insert( results, { id = k, graft = v, p = #search_txt / #v.name + (i == 1 and 1 or 0)} )
                end
            end
        end
        table.sort( results, function( a, b ) return a.p > b.p end )
        self.results = results
        self.battle_search_txt = search_txt

        DebugUtil.SetLocalDebugData( "battle_card_search", search_txt )
        DebugUtil.SaveLocalDebugData()
    end

    if self.spawn_battle_txt then
        ui:TextColored( 0xCCCC00FF, self.spawn_battle_txt )
    else
        ui:TextColored( 0x777777FF, "Enter battle card search text" )
    end

    if self.results then
        ui:Separator()
        ui:TextColored( 0x00CCCCFF, string.format( "%d results for '%s'", #self.results, self.search_txt ))    
        ui:BeginChild( "Results", 350, 150 )
        for i, result in ipairs( self.results or table.empty ) do
            if ui:Selectable( result.id ) then
                player.battler:LearnCard( result.id)
                self.spawn_battle_txt = string.format( "Spawned %s!", result.id )
                print( "DEBUG: ", self.spawn_battle_txt )
            end
        end
        ui:EndChild()
    end
end


local function ShowSpawnNegotiationCard( self, ui )
    local DebugUtil = require "debug/debug_util"
    local player = TheGame:GetGameState():GetPlayerAgent()
    
    if not self.set_focus then
        ui:SetKeyboardFocusHere()
        self.set_focus = true
    end

    local search_txt = ui:InputText( "Search", self.negotiation_search_txt or "" )
    if self.negotiation_search_txt == nil then
        search_txt = DebugUtil.GetLocalDebugData( "negotiation_card_search" ) or ""
    end
    if search_txt and self.negotiation_search_txt ~= search_txt then
        local results = {}
        for i, v in ipairs( Content.GetAllNegotiationCards() ) do
            local k = v.id
            local ok, i, j = pcall( string.find, k, search_txt, 1, true )
            if ok and i then
                table.insert( results, { id = k, graft = v, p = #search_txt / #k + (i == 1 and 1 or 0)} )
            else
                ok, i, j = pcall( string.find, v.name, search_txt )
                if ok and i then
                    table.insert( results, { id = k, graft = v, p = #search_txt / #v.name + (i == 1 and 1 or 0)} )
                end
            end
        end
        table.sort( results, function( a, b ) return a.p > b.p end )
        self.results = results
        self.negotiation_search_txt = search_txt

        DebugUtil.SetLocalDebugData( "negotiation_card_search", search_txt )
        DebugUtil.SaveLocalDebugData()
    end

    if self.spawn_negotiation_txt then
        ui:TextColored( 0xCCCC00FF, self.spawn_negotiation_txt )
    else
        ui:TextColored( 0x777777FF, "Enter negotiation card search text" )
    end

    if self.results then
        ui:Separator()
        ui:TextColored( 0x00CCCCFF, string.format( "%d results for '%s'", #self.results, self.search_txt ))    
        ui:BeginChild( "Results", 350, 150 )
        for i, result in ipairs( self.results or table.empty ) do
            if ui:Selectable( result.id ) then
                player.negotiator:LearnCard( result.id)
                self.spawn_negotiation_txt = string.format( "Spawned %s!", result.id )
                print( "DEBUG: ", self.spawn_negotiation_txt )
            end
        end
        ui:EndChild()
    end
end


local function ShowSpawnGraft( self, ui )
    local DebugUtil = require "debug/debug_util"
    local player = TheGame:GetGameState():GetPlayerAgent()
    
    if not self.set_focus then
        ui:SetKeyboardFocusHere()
        self.set_focus = true
    end

    local search_txt = ui:InputText( "Search", self.search_txt or "" )
    if self.search_txt == nil then
        search_txt = DebugUtil.GetLocalDebugData( "graft_search" ) or ""
    end
    if search_txt and self.search_txt ~= search_txt then
        local results = {}
        for i, v in ipairs( Content.GetAllGrafts() ) do
            local k = v.id
            local ok, i, j = pcall( string.find, k, search_txt )
            if ok and i then
                table.insert( results, { id = k, graft = v, p = #search_txt / #k + (i == 1 and 1 or 0)} )
            end
        end
        table.sort( results, function( a, b ) return a.p > b.p end )
        self.results = results
        self.search_txt = search_txt

        DebugUtil.SetLocalDebugData( "graft_search", search_txt )
        DebugUtil.SaveLocalDebugData()
    end

    if self.spawn_txt then
        ui:TextColored( 0xCCCC00FF, self.spawn_txt )
    else
        ui:TextColored( 0x777777FF, "Enter graft search text" )
    end

    if self.results then
        ui:Separator()
        ui:TextColored( 0x00CCCCFF, string.format( "%d results for '%s'", #self.results, self.search_txt ))    
        ui:BeginChild( "Results", 250, 150 )
        for i, result in ipairs( self.results or table.empty ) do
            if ui:Selectable( result.id ) then
                player.graft_owner:AddGraft( GraftInstance( result.id, nil, player ))
                self.spawn_txt = string.format( "Spawned %s!", result.id )
                print( "DEBUG: ", self.spawn_txt )
            end
        end
        ui:EndChild()
    end
end

local function ShowSpawnPet( self, ui )
    local DebugUtil = require "debug/debug_util"
    local player = TheGame:GetGameState():GetPlayerAgent()
    local pets = {"VROC", "CRAYOTE", "AUTODOG", "MERCHANT_SNAIL", "ERCHIN", "SHROOGLET", "LUMICYTE_UPGRADED", "LUMICYTE" }

    for _, pet in ipairs( pets ) do
        if ui:Selectable( pet ) then
            local pet_agent =  TheGame:GetGameState():AddAgent(Agent(pet))
            pet_agent:Recruit( PARTY_MEMBER_TYPE.CREW)
        end
    end
end

local function ShowCoinsMenu( self, ui )
    local DebugUtil = require "debug/debug_util"
    local player = TheGame:GetGameState():GetPlayerAgent()
    
    if not self.set_focus then
        ui:SetKeyboardFocusHere()
        self.set_focus = true
    end

    local search_txt = ui:InputText( "Search", self.search_txt or "" )
    if self.search_txt == nil then
        search_txt = DebugUtil.GetLocalDebugData( "graft_search" ) or ""
    end
    if search_txt and self.search_txt ~= search_txt then
        local results = {}
        for i, v in ipairs( Content.GetAllGrafts() ) do
            local k = v.id
            local ok, i, j = pcall( string.find, k, search_txt )
            if ok and i and v.type == GRAFT_TYPE.COIN then
                table.insert( results, { id = k, graft = v, p = #search_txt / #k + (i == 1 and 1 or 0)} )
            end
        end
        table.sort( results, function( a, b ) return a.p > b.p end )
        self.results = results
        self.search_txt = search_txt

        DebugUtil.SetLocalDebugData( "graft_search", search_txt )
        DebugUtil.SaveLocalDebugData()
    end

    if self.spawn_txt then
        ui:TextColored( 0xCCCC00FF, self.spawn_txt )
    else
        ui:TextColored( 0x777777FF, "Enter coin search text" )
    end

    if self.results then
        ui:Separator()
        ui:TextColored( 0x00CCCCFF, string.format( "%d results for '%s'", #self.results, self.search_txt ))    
        ui:BeginChild( "Results", 250, 150 )
        for i, result in ipairs( self.results or table.empty ) do
            if ui:Selectable( result.id ) then
                local existing_coin = player.graft_owner:FindGraft(function(graft) return graft:GetDef().type == GRAFT_TYPE.COIN end)
                if existing_coin then
                    player.graft_owner:ReplaceGraft( existing_coin, GraftInstance( result.id ))
                else
                    player.graft_owner:AddGraft( GraftInstance( result.id, nil, player ))
                end
                self.spawn_txt = string.format( "Spawned %s!", result.id )
            end
        end
        ui:EndChild()
    end
end

-----------------------------------------------------------------------
-- Some shared bindings between the editor and teh game.

local SHARED_BINDINGS =
{
    {
        Binding = {key ="TILDE", CTRL = true },
        Text = "Toggle Console",
        Do = function()
            TheGame:GetDebug():ToggleDebugConsole()
        end
    },
    {
        Binding = {key="BACKSPACE", CTRL = true },
        Text = "Toggle Debug Render",
        Do = function()
            TheGame:GetDebug():ToggleDebugFlags( DBG_FLAGS.RENDER | DBG_FLAGS.SIM )
        end
    },
    {
        Binding = {key = "TILDE", SHIFT = true },
        Text = "Create Inspector Panel",
        Do = function()
            local dbg_env = TheGame:GetDebugEnv()
            if dbg_env.console == nil then
                TheGame:GetDebug():CreatePanel()
            end
        end
    },
    {
        Binding = {key = "TILDE"},
        Text = "Toggle Inspector Panel",
        Do = function()
            local dbg_env = TheGame:GetDebugEnv()
            if dbg_env.console == nil then
                TheGame:GetDebug():TogglePanel()
            end
        end
    },
    {
        Text = "Toggle Hud Visibility",
        Binding = {key ="H", CTRL = true },
        Do = function()
            TheGame:FE():ToggleHUD( HUD_FLAGS.HIDDEN )
        end
    },
    {
        Text = "Toggle Hud Panel",
        Binding = {key ="H", CTRL = true, SHIFT = true },
        Do = function()
            local panel = TheGame:GetDebug():FindPanelByClass( DebugHUD )
            if panel then
                panel:Close()
            else
                TheGame:GetDebug():CreatePanel( DebugHUD() )
            end
        end
    },
    {
        Binding = {key="S",  CTRL = true, SHIFT = true },
        Text = "Inspect Character Skins",
        Do = function()
            DBG( DebugCharacterSkins() )
        end
    },
    {
        Binding = {key="BACKSLASH", CTRL = true },
        Text = "Profiler Save",
        Do = function()
            engine.inst:ProfilerSave()
        end,
    },
    {
        ID = "EXECUTE_DEBUG",
        Binding = {key="D", CTRL = true},
        Text = function()
            return string.format( "Execute '%s'", TheGame:GetLocalSettings().DEBUG_FILE )
        end,
        Do = function()
            TheGame:GetDebug():ExecuteDebugFile( TheGame:GetLocalSettings().DEBUG_FILE )
        end
    },
    {
        Binding = {key="R", CTRL = true},
        Text = "Reload Game",
        Do = function()
            engine.inst:Reset()
        end
    },
    {
        Binding = {key="G", CTRL = true},
        Text = "Reload Shaders",
        Do = function()
            print("Reloading shaders")
            engine.inst:ReloadShaders()
        end
    },
    {
        Binding = {key="T", CTRL = true, SHIFT = true },
        Text = "Toggle Trace Allocs",
        Do = function()
            local dbg_env = TheGame:GetDebugEnv()
            dbg_env.trace_alloc = not dbg_env.trace_alloc
            print (dbg_env.trace_alloc and "ENABLE TRACE ALLOC" or "DISABLE TRACE ALLOC")
            engine.inst:SetTraceAlloc(dbg_env.trace_alloc)
        end
    },
    {
        Text = "Engine Warning",
        Do = function()
            LOGWARN( "debug warning!" )
        end,
    },
    {
        Binding = {key="B", CTRL = true },
        Text = "BGFX Stats",
        Do = function()
            engine.inst:ToggleBGFXStats()
        end,
    },
    {
        Binding ={key="T", CTRL = true },
        Text = "Quick Switcher",
        Do = function()
            local panel = TheGame:GetDebug():FindPanelByClass( DebugQuickSwitcher )
            if panel then
                TheGame:GetDebug():ClosePanel( panel )
            else
                DBG( DebugQuickSwitcher() )
            end
        end,
    },
}

-----------------------------------------------------------------------
-- Non-game based debug bindings.

local TOOL_BINDINGS =
{
    name = "Tool",
    {
        Text = "Editors...",
        CustomMenu = function( self, ui )
            ShowDebugToolsSubmenu( TheGame:FE(), ui )
        end,
    },
    {}
}
table.arrayadd( TOOL_BINDINGS, SHARED_BINDINGS )

-----------------------------------------------------------------------
-- In-game based debug bindings.

local wall_time_debug_scale_idx = 5
local wall_time_debug_scales  = {0, 1/16, 1/8, 1/4, 1/2, 1, 1.5, 2, 2.5, 3, 6, 9, 12}

local GAME_BINDINGS =
{
    name = "Game",
    {
        Text = "Inspectors...",
        Menu =
        {
            {
                Text = "Easing",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugEasing() )
                end
            },
            {
                Binding = {key="A",  CTRL = true },
                Text = "Inspect Audio",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugAudio() )
                end
            },
            {
                Binding = {key="F",  CTRL = true, SHIFT = true },
                Text = "Inspect FightTester",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugFightTest() )
                end
            },
            {
                Binding = {key="T",  CTRL = true, SHIFT = true },
                Text = "Inspect Tuning",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugTuning() )
                end
            },
            {
                Binding = {key="BACKSPACE", CTRL = true, SHIFT = true },
                Text = "Inspect UI",
                Do = function()
                    TheGame:GetDebug():ToggleDebugFlags( DBG_FLAGS.UI )
                end
            },

            {
                Binding = {key="Q",  CTRL = true },
                Text = "Inspect Quests",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugQuests() )
                end
            },
            {
                Binding = {key="F",  CTRL = true },
                Text = "Inspect Factions",
                Enabled = function()
                    return TheGame:GetGameState()
                end,
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugFactionMgr( TheGame:GetGameState():GetFactions() ))
                end
            },
            {
                Binding = { key = "L", CTRL = true },
                Text = "Inspect Lister",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugLister() )
                end
            },
            {
                Binding = { key = "NUM_1", CTRL = true },
                Text = "Inspect Negotiation Cards",
                Do = function()
                    local negotiation_defs = require "negotiation/negotiation_defs"
                    TheGame:GetDebug():CreatePanel( DebugCardCollection( negotiation_defs ) )
                end
            },
            {
                Binding = { key = "NUM_2", CTRL = true },
                Text = "Inspect Battle Cards",
                Do = function()
                    local battle_defs = require "battle/battle_defs"
                    TheGame:GetDebug():CreatePanel( DebugCardCollection( battle_defs ) )
                end
            },
            {
                Binding = { key = "NUM_3", CTRL = true },
                Text = "Inspect Icons",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugIcons())
                end
            },
            {
                Binding = { key = "NUM_4", CTRL = true },
                Text = "Inspect Argument Sounds",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugNegotiationModifiers():ShowSounds() )
                end
            },
            {
                Binding = {key = "NUM_5", CTRL = true },
                Text = "Inspect Mods",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugMods() )
                end
            },
            {
                Text = "Inspect Unused Card Art",
                Do = function()
                    TheGame:GetDebug():CreatePanel( DebugUnusedCardArt() )
                end
            },
        }
    },
    {}, -----------------
    {
        Bindings = {{key="KP_MINUS", CTRL = true},{key="MINUS", CTRL = true}},
        Text = "-Wall Time Scale",
        Do = function()
            wall_time_debug_scale_idx = math.max( (wall_time_debug_scale_idx - 1),  1)
            TheGame:SetWallTimeScale(wall_time_debug_scales[wall_time_debug_scale_idx])
            print ("WALL TIME SCALE", wall_time_debug_scales[wall_time_debug_scale_idx])
            return true
        end
    },
    {
        Bindings = {{key="KP_PLUS", CTRL = true},{key="EQUALS", CTRL = true}},
        Text = "+Wall Time Scale",
        Do = function()
            wall_time_debug_scale_idx = math.min( (wall_time_debug_scale_idx + 1),  #wall_time_debug_scales)
            TheGame:SetWallTimeScale(wall_time_debug_scales[wall_time_debug_scale_idx])
            print ("WALL TIME SCALE", wall_time_debug_scales[wall_time_debug_scale_idx])
            return true
        end
    },
    {
        Bindings = {{ key = "PAUSE" }},
        Text = "Toggle Debug",
        Do = function()
            local dbg_env = TheGame:GetDebugEnv()
            dbg_env.dbg_toggle()
        end
    },
    {
        Binding = {key="C",  CTRL = true, SHIFT = true },
        Text = "Give All Cards",
        Do = function()
            local dbg_env = TheGame:GetDebugEnv()
            dbg_env.c_givecards()
        end
    },

    {
        Binding = {key="I", CTRL = true },
        Text = "Widget Tracer",
        Do = function()
            require "ui/widgettracer"
            local dbg = TheGame:GetDebug()
            for i, panel in dbg:DebugPanels() do
                if is_instance( panel:GetNode(), DebugWidgetTracer ) then
                    panel:GetNode():RunTrace()
                    return
                end
            end
            dbg:CreatePanel( DebugWidgetTracer())
        end
    },

    {
        Binding = {key="ENTER", CTRL = true, SHIFT = true },
        Text = "UI Freeze",
        Do = function()
            TheGame:FE():ToggleDebugFreeze()
        end
    },

    {
        Binding = {key="F4" },
        Text = "Save Inspector",
        Do = function( )
            TheGame:GetDebug():CreatePanel( DebugSaveGames())
        end
    },
    {
        Binding = {key="F3" },
        Text = "Deck Inspector",
        Do = function( )
            TheGame:GetDebug():CreatePanel( DebugDecks())
        end
    },

    {
        Binding = {key="F5" },
        Text = "Quicksave",
        Enabled = function( entity )
            return true
        end,
        Do = DoQuickSave
    },
    {
        Binding = {key="F9" },
        Text = "Quickload",
        Enabled = function( entity )
            return true
        end,
        Do = DoQuickLoad
    },
    {
        Binding = {key="F9", CTRL = true },
        Text = "Quickload Auto",
        Enabled = function( entity )
            return true
        end,
        Do = function() TheGame:LoadGame("autosave") end
    },
	{
		Binding = {key = "F10"},
		Text = "Generate PO Template",
		Enabled = function(entity)
			return true
		end,
		Do = function ()
            TheGame:GetDebug():CreatePanel( DebugLocalization())
		end
	},
    {
        Binding = {key = "P", CTRL=true},
        Text = "Switch between touch and mouse input modes",
        Enabled = function(entity)
            return true
        end,
        Do = function ()
            TheGame:FE():SetPendingControlMode( TheGame:FE():GetControlMode() == CONTROL_MODE.TOUCH and CONTROL_MODE.MOUSE_KEYBOARD or CONTROL_MODE.TOUCH )
        end
    },
    {
        Binding = {key="C", ALT = true },
        Text = "Reload Conversations",
        Enabled = function( entity )
            return true
        end,
        Do = function()
            print( "...Reloading conversation, quests, quips.")
            Content.ReloadConversation()

            local screen = TheGame:FE():FindScreen( Screen.ConversationScreen )
            if screen then
                screen:ReloadConversation()
            end
        end
    },
    {
        Binding = {key = "M", CTRL = true },
        Text = "Give Money",
        Do = function()
            local dbg_env = TheGame:GetDebugEnv()
            dbg_env.c_givemoney( 1000 )
        end
    },
    {
        Binding = {key="A",  SHIFT = true, CTRL = true },
        Text = "Toggle Battle Auto-Resolve",
        Do = function()
            TheGame:GetLocalSettings().ROBOTICS = not TheGame:GetLocalSettings().ROBOTICS
            print( "Toggling ROBOTICS:", TheGame:GetLocalSettings().ROBOTICS )
        end
    },

}
table.arrayadd( GAME_BINDINGS, SHARED_BINDINGS )

-------------------------------------------------------------------------
-- TABLE BINDINGS

local TABLE_BINDINGS =
{
    name = "Table",
    {
        Text = function( t )
            return string.format( "set t = %s", rawstring(t))
        end,
        Checked = function( t )
            return t and TheGame:GetDebugEnv().t == t
        end,
        Do = function( t )
            TheGame:GetDebugEnv().t = t
        end
    },
    {
        Text = function( t )
            return string.format( "clear t (%s)", rawstring(t))
        end,
        CustomMenu = function( self, ui, t )
            if ui:MenuItem( string.format( "Clear %d fields", table.count( t ))) then
                table.clear( t )
            end
        end
    },
    {
        Text = function( t )
            return string.format( "View t (%s)", rawstring(t))
        end,
        Do = function( t )
            TheGame:GetDebug():CreatePanel( DebugTable( t ))
        end
    },
}

-------------------------------------------------------------------------
-- LOCATION BINDINGS

local LOCATION_BINDINGS =
{
    name = "Location",
    {
        Text = function(location)
            return string.format( "set loc = %s", rawstring(location))
        end,
        Checked = function( location )
            return TheGame:GetDebugEnv().loc == location
        end,
        Do = function( location )
            TheGame:GetDebugEnv().loc = location
        end
    },
    {
        Text = "Spawn Agent",
        CustomMenu = function( self, ui, t )
            local function IsNPC( def )
                return def.base_def ~= "PLAYER_BASE"
            end
            local function IsSkinEnabled( agent, skin_table )
                return skin_table == nil or TheGame:GetGameState():IsSkinAvailable( skin_table.uuid )
            end
            local id, skin_table = DebugUtil.ShowAgentSearch( self, ui, IsNPC, IsSkinEnabled )
            if id then
                local agent = Agent( id, skin_table )
                TheGame:GetGameState():AddAgent( agent )
                agent:MoveToLocation( t )
                print( "DEBUG: Spawned:", id, agent )
            end
        end,
    },
    {
        Text = "Teleport To",
        Do = function( location )
            TheGame:GetGameState():GetCaravan():MoveToLocation( location )
        end,
    },
    table.empty,
    {
        Text = "Clear Agents",
        Do = function( location )
            local agents = {}
            for i, agent in location:Agents() do
                if not agent:IsInPlayerParty() then
                    table.insert( agents, agent )
                end
            end
            while #agents > 0 do
                table.remove( agents ):MoveToLimbo()
            end
        end
    }
}


-------------------------------------------------------------------------
-- FIGHTER BINDINGS

local FIGHTER_BINDINGS =
{
    name = "Fighter",
    {
        Text = function(fighter)
            return string.format( "set f = %s", rawstring(fighter))
        end,
        Checked = function( fighter )
            return TheGame:GetDebugEnv().f == fighter
        end,
        Do = function( fighter )
            TheGame:GetDebugEnv().f = fighter
        end
    },
    {
        Text = function( fighter )
            local screen = TheGame:FE():FindScreen( Screen.FightScreen )
            local ent = screen:GetAnimFighterEntity( fighter )
            return string.format( "set entity = %s", ent )
        end,
        Visible = function( fighter )
            local screen = TheGame:FE():FindScreen( Screen.FightScreen )
            return screen ~= nil
        end,
        Checked = function( fighter )
            local screen = TheGame:FE():FindScreen( Screen.FightScreen )
            local ent = screen:GetAnimFighterEntity( fighter )
            return TheGame:GetDebugEnv().entity == ent
        end,
        Do = function( fighter )
            local screen = TheGame:FE():FindScreen( Screen.FightScreen )
            local ent = screen:GetAnimFighterEntity( fighter )
            TheGame:GetDebugEnv().entity = ent
        end
    },
    {
        Text = "DBG( UnitFrame )",
        Do = function ( fighter )
            local screen = TheGame:FE():FindScreen( Screen.FightScreen )
            DBG( screen.status_widgets[ fighter ] )
        end,
    },
    {
        Text = "DBG( HealthBar )",
        Do = function ( fighter )
            local screen = TheGame:FE():FindScreen( Screen.FightScreen )
            DBG( screen.status_widgets[ fighter ].health_bar )
        end,
    },
    {
        Text = "Health",
        CustomMenu = function( self, ui, fighter )
            local value = fighter:GetStat( COMBAT_STAT.HEALTH )
            local min, max = fighter:GetStatBounds( COMBAT_STAT.HEALTH )
            local new_value = ui:SliderInt(COMBAT_STAT.HEALTH, value, min, max)
            if new_value and new_value ~= value then
                fighter:DeltaStat( COMBAT_STAT.HEALTH, new_value - value )
            end
        end,        
    },
    {
        Text = "Morale",
        CustomMenu = function( self, ui, fighter )
            local value = fighter:GetStat( COMBAT_STAT.MORALE ) or 0
            local new_value = ui:SliderFloat(COMBAT_STAT.MORALE, value, 0, 1.0)
            if new_value and new_value ~= value then
                fighter:EnableMorale( true )
                fighter:DeltaStat( COMBAT_STAT.MORALE, new_value - value )
            end
        end,        
    },
    {
        Text = "Defend",
        CustomMenu = function( self, ui, fighter )
            local value = fighter:GetConditionStacks( "DEFEND" )
            local new_value = ui:SliderInt( "DEFEND", value, 0, 99)
            if new_value and new_value ~= value then
                if new_value > value then
                    fighter:AddCondition( "DEFEND", new_value - value )
                else
                    fighter:RemoveCondition( "DEFEND", value - new_value )
                end
            end
        end,        
    },
    {
        Text = "Evasion",
        CustomMenu = function( self, ui, fighter )
            local value = fighter:GetConditionStacks( "EVASION" )
            local new_value = ui:SliderInt( "EVASION", value, 0, 99)
            if new_value and new_value ~= value then
                if new_value > value then
                    fighter:AddCondition( "EVASION", new_value - value )
                else
                    fighter:RemoveCondition( "EVASION", value - new_value )
                end
            end
        end,        
    },
    {
        Text = "Combo",
        Visible = function( fighter )
            return fighter.agent:GetContentID() == "SAL"
        end,
        CustomMenu = function( self, ui, fighter )
            local value = fighter:GetConditionStacks( "COMBO" )
            local new_value = ui:SliderInt( "COMBO", value, 0, 99)
            if new_value and new_value ~= value then
                if new_value > value then
                    fighter:AddCondition( "COMBO", new_value - value )
                else
                    fighter:RemoveCondition( "COMBO", value - new_value )
                end
            end
        end,        
    },
    {
        Text = "Charges",
        Visible = function( fighter )
            return fighter:GetCondition( "lumin_tracker" ) ~= nil
        end,
        CustomMenu = function( self, ui, fighter )
            local tracker = fighter:GetCondition("lumin_tracker")
            local value, max_charges = tracker:GetCharges()
            local new_value = ui:SliderInt( "CHARGES", value, 0, max_charges)
            if new_value and new_value ~= value then
                if new_value > value then
                    tracker:AddLuminCharges( new_value - value )
                else
                    tracker:RemoveCharges( value - new_value )
                end
            end
        end,        
    },
    {
        Text = "Flourish",
        Visible = function( fighter )
            return fighter:GetCondition( "flourish_tracker" ) ~= nil
        end,
        CustomMenu = function( self, ui, fighter )
            local tracker = fighter:GetCondition("flourish_tracker")
            local flourish_graft = fighter.agent.graft_owner and fighter.agent.graft_owner:GetGraft( "flourish_tracker" )

            if flourish_graft then
                local new_value = ui:SliderFloat("FLOURISH", flourish_graft.userdata.battle_flourish or 0, 0, flourish_graft.userdata.battle_max or 200)
                if new_value and new_value ~= flourish_graft.userdata.battle_flourish then
                    flourish_graft:GetDef().SetBattleFlourish(flourish_graft, fighter.battle, new_value)
                end
            end
        end,        
    },
    {},
    {
        Text = "Kill",
        Enabled = function( fighter )
            return fighter:IsAlive()
        end,
        Do = function( fighter )
            fighter:Kill()
        end,
    },
    {
        Text = "Surrender",
        Enabled = function( fighter )
            return fighter:IsActive()
        end,
        Do = function( fighter )
            fighter:Surrender()
        end,
    },
    {
        Text = "Flee",
        Enabled = function( fighter )
            return fighter:IsAlive()
        end,
        Do = function( fighter )
            fighter:Flee()
        end,
    }
}

-------------------------------------------------------------------------
-- AGENT BINDINGS

local AGENT_BINDINGS =
{
    name = "Agent",
    {
        Text = function(agent)
            return string.format( "set agent = %s", rawstring(agent))
        end,
        Checked = function( agent )
            return TheGame:GetDebugEnv().agent == agent
        end,
        Do = function( agent )
            TheGame:GetDebugEnv().agent = agent
        end
    },

    {
        Text = "Battle Cards...",
        Visible = function ( agent )
            return agent.battler ~= nil
        end,
        CustomMenu = function( self, ui, agent )
            local DebugUtil = require "debug/debug_util"
            DebugUtil.RenderBattleCards( self, ui, agent )
        end
    },

    {
        Text = "Negotiation Cards...",
        Visible = function ( agent )
            return agent.negotiator ~= nil
        end,
        CustomMenu = function( self, ui, agent )
            self.filter_txt = ui:InputText( "Filter", self.filter_txt or "" ) or self.filter_txt

            if ui:Button( "Reset" ) then
                table.clear( agent.negotiator:GetCards() )
            end

            ui:Separator()

            local negotiation_defs = require "negotiation/negotiation_defs"
            ui:BeginChild( "CARDS", 400, 400 )
            for i, carddef in ipairs( Content.GetAllNegotiationCards() ) do
                if not CheckBits( carddef.flags, negotiation_defs.CARD_FLAGS.OPPONENT ) then
                    local id = carddef.id
                    if self.filter_txt == nil or id:find( self.filter_txt ) then
                        local count = agent.negotiator:GetCardCount( id )
                        local new_count = ui:SliderInt( id, count, 0, 9 )
                        if new_count ~= count then
                            agent.negotiator:SetCardCount( id, new_count )
                        end
                    end
                end
            end
            ui:EndChild()
        end
    },

   
    {
        Text = "Set Aspects...",
        CustomMenu = function( self, ui, agent )
            self.filter_txt = ui:InputText( "Filter", self.filter_txt or "" ) or self.filter_txt

            if ui:RadioButton( "All", self.filter_type or 0, 0 ) then
                self.filter_type = 0
            end
            ui:SameLine( nil, 10 )
            if ui:RadioButton( "Traits", self.filter_type or 0, 1 ) then
                self.filter_type = 1
            end
            ui:SameLine( nil, 10 )
            if ui:RadioButton( "Status", self.filter_type or 0, 2 ) then
                self.filter_type = 2
            end
            ui:SameLine( nil, 10 )
            if ui:RadioButton( "Skills", self.filter_type or 0, 3 ) then
                self.filter_type = 3
            end
            ui:Separator()

            ui:BeginChild( "ASPECTS", 260, 300 )
            for i, aspect_name, aspect in Content.SortedAspects() do
                local ok = self.filter_txt == nil or aspect.id:find( self.filter_txt ) ~= nil
                if self.filter_type == 1 then
                    ok = ok and aspect:IsTrait()
                elseif self.filter_type == 2 then
                    ok = ok and aspect:IsStatusEffect()
                elseif self.filter_type == 3 then
                    ok = ok and aspect:IsSkill()
                end

                if ok then
                    local has_aspect = agent:HasAspect( aspect_name )
                    if ui:Checkbox( aspect_name, has_aspect ) then
                        if has_aspect then
                            agent:RemoveAspect( aspect_name )
                            has_aspect = false
                        else
                            agent:GainAspect( aspect_name )
                        end
                    end
                end
            end
            ui:EndChild()
        end,
    },

    {
        Text = "Set Relation",
        Visible = function( agent )
            return not agent.DEBUG_FIGHT_TEST and not agent:IsPlayer()
        end,
        CustomMenu = function ( self, ui ,agent )
            for v, relation in ipairs( RELATIONSHIP_ARRAY ) do
                if ui:MenuItem( relation, nil, agent:GetRelationship() == relation ) then
                    agent.social_connections:SetRelationship( v, TheGame:GetGameState():GetPlayerAgent() )
                end
            end
        end
    },

    {
        Text = function(agent)
            local current, max = agent:GetHealth()
            return loc.format("Set Health ({1} / {2})", current, max)
        end,
        Visible = function( agent )
            return agent:GetHealth() ~= nil
        end,
        CustomMenu = function( self, ui, agent )
            local current, max = agent:GetHealth()
            local hp = ui:SliderInt( "Health", current, 1, max )
            if hp ~= current then
                agent.health:Set( hp )
            end
        end,
    },

    {
        Text = "Edit Skin...",
        Visible = function( agent )
            return agent:GetSkinID() ~= nil
        end,
        Do = function( agent )
            local panel = DebugCharacterSkins()
            panel:SetFilterText( agent:GetName() )
            DBG( panel )
        end,
    },
    {},
    {
        Text = function( agent )
            if agent:GetLocation() then
                return tostring( agent:GetLocation() )
            elseif agent:InLimbo() then
                return "Limbo"
            else
                return "???"
            end
        end,
        Enabled = function( agent )
            return false
        end
    },
    {
        Text = "Teleport To",
        Visible = function( agent )
            return not agent.DEBUG_FIGHT_TEST
        end,
        Enabled = function( agent )
            return not agent:InLimbo()
        end,
        Do = function( agent )
            local location = agent:GetLocation()
            if location then
                TheGame:GetGameState():GetCaravan():MoveToLocation( location )
            end
        end,
    },
    {
        Text = "Speak To",
        Visible = function( agent )
            return not agent.DEBUG_FIGHT_TEST
        end,
        Enabled = function( agent )
            return not agent:InLimbo()
        end,
        Do = function( agent )
            local location = agent:GetLocation()
            if location then
                TheGame:GetGameState():GetCaravan():MoveToLocation( location )

                local screen = TheGame:FE():GetTopScreen()
                if is_instance( screen, Screen.LocationScreen ) then
                    screen:EnterConvo( agent )
                end
            end
        end,
    },
    {},
    {
        Text = "Send...",
        CustomMenu = function( self, ui, agent )
            if ui:MenuItem( "Retire" ) then
                agent:Retire()
            end
            if ui:MenuItem( "Kill" ) then
                agent:Kill()
            end
            if ui:MenuItem( "Send To Limbo" ) then
                agent:MoveToLimbo()
            end
            if agent:GetBrain() and ui:MenuItem( "Send Home" ) then
                agent:GetBrain():MoveToHome()
            end
            if TheGame:GetGameState() and ui:MenuItem( "Send to Player") then
                agent:MoveToAgent( TheGame:GetGameState():GetPlayerAgent() )
            end
        end,
    }
}

-------------------------------------------------------------------------
-- ENTITY BINDINGS

local ENTITY_BINDINGS =
{
    name = "Entity",
    {
        Text = function(entity)
            return string.format( "set entity = %s", rawstring(entity))
        end,
        Checked = function( entity )
            return TheGame:GetDebugEnv().entity == entity
        end,
        Do = function( entity )
            TheGame:GetDebugEnv().entity = entity
        end
    },
    {},
    {
        Text = "Despawn",
        Enabled = function( entity )
            return not entity.removed
        end,
        Do = function( entity )
            entity:Despawn()
        end,
    },
}

local WIDGET_BINDINGS =
{
    name = "Widget",
    {
        Text = function( widget )
            return string.format( "set w = %s", rawstring(widget))
        end,
        Checked = function( widget )
            return TheGame:GetDebugEnv().w == widget
        end,
        Do = function( widget )
            TheGame:GetDebugEnv().w = widget
        end
    },
    {
        Text = "Hide",
        Visible = function( widget )
            return widget:IsShown()
        end,
        Do = function( widget )
            widget:Hide()
        end,
    },
    {
        Text = "Show",
        Visible = function( widget )
            return not widget:IsShown()
        end,
        Do = function( widget )
            widget:Show()
        end,
    },
    {
        Text = "Blink",
        Do = function( widget )
            widget:Blink( 0.2, 5 )
        end,
    },
    {
        Text = "Show Bounding Box",
        Visible = function( widget )
            return TheGame:FE().debugwidget == nil or not ( TheGame:FE().debugwidget == widget )
        end,
        Do = function( widget )
            TheGame:FE().debugwidget = widget
            TheGame:FE():SetDebugRender( true )
        end,
    },
    {
        Text = "Hide Bounding Box",
        Visible = function( widget )
            return TheGame:FE().debugwidget == widget
        end,
        Do = function( widget )
            TheGame:FE().debugwidget = nil
            TheGame:FE():SetDebugRender( false )
        end,
    },
    {
        Text = "Enable",
        Visible = function( widget )
            return not widget.enabled
        end,
        Do = function( widget )
            widget:Enable()
        end
    },
    {
        Text = "Disable",
        Visible = function( widget )
            return widget.enabled
        end,
        Do = function( widget )
            widget:Disable()
        end
    },
    {
        Text = nil, -- Separator
        Visible = function( widget )
            return is_instance( widget, Widget.Screen )
        end,
    },
    {
        Text = "Pop Screen",
        Visible = function( widget )
            return is_instance( widget, Widget.Screen )
        end,
        Enabled = function( widget )
            return widget:IsOnStack()
        end,
        Do = function( widget )
            widget:GetFE():PopScreen( widget )
        end,
    }
}

local QUEST_BINDINGS =
{
    name = "Quest",
    {
    },
}

local DEBUG_CONTEXT_MENU =
{
    name = "Context",
    {
        Text = "Party...",
        CustomMenu = function( self, ui )
            for i, member in TheGame:GetGameState():GetCaravan():Members() do
                if ui:MenuItem( string.format( "Debug %s", member:GetName() )) then
                    DBG(member)
                end
            end
        end,
    },

    {
        Text = "Travel To...",
        Enabled = function( self )
            local travel_screen = TheGame:FE():FindScreen( Screen.TravelScreen )
            return not travel_screen or not travel_screen:IsTravelling()
        end,
        CustomMenu = function( self, ui )
            for guid, loc in TheGame:GetGameState():AllLocations() do
                local styled = loc:FindAgentIf( Agent.IsPlayer )
                if styled then
                    ui:PushStyleColor( ui.Style_Text, 0x00CCCCFF )
                end
                if ui:MenuItem( tostring(loc) ) then
                    TheGame:GetGameState():GetCaravan():MoveToLocation( loc )
                end
                if styled then
                    ui:PopStyleColor()
                end
            end
        end,
    },
    {
        Text = "Health",
        CustomMenu = function( self, ui )
            local hp, max_hp =  TheGame:GetGameState():GetPlayerAgent():GetHealth()
            
            local new_hp = ui:SliderInt("Health", hp, 1, max_hp )
            if new_hp ~= hp then
                TheGame:GetGameState():GetPlayerAgent().health:Delta( new_hp - hp )
            end

        end,
    },
    {
        Text = "Resolve",
        CustomMenu = function( self, ui )
            local resolve, max_resolve =  TheGame:GetGameState():GetCaravan():GetResolve()
            
            local new_resolve = ui:SliderInt("Resolve", resolve, 0, max_resolve)
            if new_resolve ~= resolve then
                TheGame:GetGameState():GetCaravan():SetResolve( new_resolve )
            end
        end,
    },

    { --[[SEPARATOR--]] },

    {
        Text = "Gain all Unlocks",
        Do = function()
            local dbg_env = TheGame:GetDebugEnv()
            dbg_env.c_unlockall()
        end,
    },
    {
        Text = "Load Deck",
        Do = function() 
            TheGame:GetDebug():CreatePanel( DebugDecks())
        end,
    },
    {
        Text = "Add a Pet",
        CustomMenu = ShowSpawnPet,
    },
    {
        Text = "Spawn Graft",
        CustomMenu = ShowSpawnGraft,
    },
    {
        Text = "Spawn Coin...",
        CustomMenu = ShowCoinsMenu,
    },
    {
        Text = "Spawn Battle Card",
        CustomMenu = ShowSpawnBattleCard,
    },
    {
        Text = "Spawn Negotiation Card",
        CustomMenu = ShowSpawnNegotiationCard,
    },
    {
        Text = "Draft Battle Card...",
        Do = function()
            local draft_popup = Screen.DraftChoicePopup()
            local cards = RewardUtil.GetBattleCards( TheGame:GetGameState():GetCurrentBaseDifficulty(), 3)
            draft_popup:DraftCards( nil, Battle.Card, cards )
            TheGame:FE():InsertScreen( draft_popup )
        end,
    },
    {
    Text = "Draft Negotiation Card...",
        Do = function()
            local draft_popup = Screen.DraftChoicePopup()
            local cards = RewardUtil.GetNegotiationCards( TheGame:GetGameState():GetCurrentBaseDifficulty(), 3)
            draft_popup:DraftCards( nil, Negotiation.Card, cards )
            TheGame:FE():InsertScreen( draft_popup )
        end,
    },
    {
        Text = "Draft Graft...",
        Do = function()
            local grafts
            if math.random() < 0.25 then
                grafts = RewardUtil.GetBossGrafts( 3 )
            else
                local difficulty = math.random( 1, #GRAFT_DROP_RARITY )
                grafts = RewardUtil.GetGrafts(difficulty, TheGame:GetGameState():GetGraftDraftDetails().count)
            end
            local popup = Screen.PickGraftScreen(grafts)
            TheGame:FE():InsertScreen( popup )
        end
    },

}


-------------------------------------------------------------------------
-- DEBUG TOGGLES

local DEBUG_TOGGLES =
{
     name = "Toggles",
}

for i, k in sorted_pairs(DBG_FLAGS) do
    local flag = DBG_FLAGS[k]
    if k ~= DBG_FLAGS.NONE then
        local menu_item =
        {
            Text = tostring(k),
            Checked = function()
                return TheGame:GetDebug():IsDebugFlagged( flag )
            end,
            Do = function()
                TheGame:GetDebug():ToggleDebugFlags( flag )
            end,
        }
        table.insert( DEBUG_TOGGLES, menu_item )
    end
end

return
{
    GAME_BINDINGS = GAME_BINDINGS,
    TOOL_BINDINGS = TOOL_BINDINGS,
    DEBUG_CONTEXT_MENU = DEBUG_CONTEXT_MENU,
    DEBUG_TOGGLES = DEBUG_TOGGLES,
    TABLE_BINDINGS = TABLE_BINDINGS,
    LOCATION_BINDINGS = LOCATION_BINDINGS,
    FIGHTER_BINDINGS = FIGHTER_BINDINGS,
    AGENT_BINDINGS = AGENT_BINDINGS,
    ENTITY_BINDINGS = ENTITY_BINDINGS,
    QUEST_BINDINGS = QUEST_BINDINGS,

    WIDGET_BINDINGS = WIDGET_BINDINGS,
}
