 DebugUtil = {}


function DBG( v )
    if TheGame:GetDebug() then
        local debug_node = v
        if not is_instance( v, DebugNode ) then
            
            debug_node = DebugUtil.CreateDebugNode( v )
        end
        return TheGame:GetDebug():CreatePanel( debug_node )
    end
end

function DBG_SET( key, v )
    local env = TheGame:GetDebugEnv()
    if env and key ~= nil then
        env[ key ] = v
    end
end

function DBQ( k, v )
    local env = TheGame:GetDebugEnv()
    return env and env[ k ] or v
end


function DebugUtil.FindRegisteredClass( v, base_class )
    local best_class
    for i, class in ipairs( get_subclasses( base_class or DebugNode )) do
        local found_class = DebugUtil.FindRegisteredClass( v, class )
        if found_class and (best_class == nil or is_class( found_class.REGISTERED_CLASS, best_class.REGISTERED_CLASS )) then
            best_class = found_class
        end

        if class.REGISTERED_CLASS and is_instance( v, class.REGISTERED_CLASS ) then
            if best_class == nil or is_class( class.REGISTERED_CLASS, best_class.REGISTERED_CLASS ) then
                best_class = class
            end
        end
    end

    return best_class
end

function DebugUtil.LightingPicker(ui, colour_name, comboName, editButtonWidth )
    colour_name = colour_name or "DEFAULT"
    local snapshot_list = {}
    for k,v in pairs(require "content/colour_settings") do
        table.insert(snapshot_list, k)
    end
    table.sort (snapshot_list)
    
    local snapshot_idx = ui:Combo( comboName, table.arrayfind(snapshot_list, colour_name) or 1, snapshot_list )
    local new_colour
    if snapshot_list[snapshot_idx] ~= colour_name then
        new_colour = snapshot_list[snapshot_idx]
    end

    ui:SameLine(nil, 5.0)
    if ui:Button("Edit Colours##"..comboName, editButtonWidth or 80.0) then
        TheGame:GetDebug():CreatePanel( DebugColourSettings() )
    end

    return new_colour
end

function DebugUtil.FightCamDebug(ui, camera)
    if ui:CollapsingHeader("CAMERA") then
        if ui:Button( "Reset to Defaults" ) then
            camera:ResetToDefaults()
        end
        
        local fov = camera:GetFOV()
        local new_fov = ui:SliderFloat( "FOV: ", fov, 1, 90)
        if new_fov ~= fov then
            camera:SetFOV(new_fov*DEGREES)
        end

        local angle = camera:GetAngle()/DEGREES
        local new_angle = ui:SliderFloat( "Angle: ", angle, 1, 89)
        if new_angle ~= angle then
            camera:SetAngle(new_angle*DEGREES)
        end

        local direction = camera:GetDirection()/DEGREES
        local new_direction = ui:SliderFloat( "direction: ", direction, -90, 90)
        if new_direction ~= direction then
            camera:SetDirection(new_direction*DEGREES)
        end

        local dist = camera:GetDistance()
        local new_dist = ui:SliderFloat( "Distance: ", dist, 0, 400)

        if new_dist ~= dist then
            camera:SetDistance(new_dist)
        end

        local x, y, z = camera:GetCurrentBasePos()
        local nx, ny, nz = ui:DragFloat3("Base Pos",x,y,z, -10, 10, .1)
        if nx then
            camera:SetBasePos(nx, ny, nz)
        end

        local ortho_scale = math.floor(camera:GetOrthoScale())
        local new_ortho_scale = ui:SliderInt( "Ortho Scale", ortho_scale, 1, 256 )
        if new_ortho_scale ~= ortho_scale then
            camera:SetOrthoScale( new_ortho_scale )
        end
    end
end

function DebugUtil.ShowLocationSearch(t, ui)
    if not t.set_focus then
        ui:SetKeyboardFocusHere()
        t.set_focus = true
    end

    local search_txt = ui:InputText( "", t.location_search_txt or "" ) or t.location_search_txt

    if search_txt then
        local results = {}
        for k, v in pairs( Content.GetLocationContents() ) do
            local ok, i, j = pcall( string.find, string.upper(k), string.upper(search_txt) )
            if ok and i then
                table.insert( results, { id = k, p = #search_txt / #k + (i == 1 and 1 or 0)} )
            end
        end
        table.sort( results, function( a, b ) return a.p > b.p end )
        t.results = results
        t.location_search_txt = search_txt
    end

    if t.spawn_txt then
        ui:TextColored( 0xCCCC00FF, t.spawn_txt )
    else
        ui:TextColored( 0x777777FF, "Enter Location ID" )
    end


    local selection = nil

    if t.results then
        ui:Separator()
        ui:TextColored( 0x00CCCCFF, string.format( "%d results for '%s'", #t.results, t.location_search_txt ))    
        ui:BeginChild( "Results", 250, 150 )
        for i, result in ipairs( t.results or table.empty ) do
            if ui:Selectable( tostring(result.id) ) then
                selection = result.id
            end
        end
        ui:EndChild()
    end

    return selection
end

function DebugUtil.ShowAgentSearch(t, ui, pred, skin_pred)
    if not t.set_focus then
        ui:SetKeyboardFocusHere()
        t.set_focus = true
    end

    local search_txt = ui:InputText( "", t.agent_search_txt or "" ) or t.agent_search_txt

    local results = {}
    for k, v in pairs( Content.GetAllCharacterDefs() ) do
        if not v.is_template and (pred == nil or pred(v)) then
            if search_txt then
                local ok, i, j = pcall( string.find, string.upper(k), string.upper(search_txt) )
                if ok and i ~= nil then
                    table.insert( results, { id = k, p = #search_txt / #k + (i == 1 and 1 or 0)} )
                end
            else
                table.insert( results, { id = k, p = 0 })
            end
        end
    end
    table.sort( results, function( a, b ) return a.p > b.p end )
    t.results = results
    t.agent_search_txt = search_txt

    if t.spawn_txt then
        ui:TextColored( 0xCCCC00FF, t.spawn_txt )
    else
        ui:TextColored( 0x777777FF, "Enter agent id" )
    end

    local selection = nil
    local selected_skin_table = nil

    if t.results then
        ui:Separator()
        ui:TextColored( 0x00CCCCFF, string.format( "%d results for '%s'", #t.results, t.agent_search_txt ))    
        ui:BeginChild( "Results", 300, 200 )
        for i, result in ipairs( t.results or table.empty ) do
            local content_id = result.id
            local skins = Content.GetAllCharacterSkins()[ content_id ]
            if skins == nil or #skins == 1 then
                if ui:Selectable( result.id ) then
                    selection = result.id
                    selected_skin_table = skins and skins[1]
                end
            
            elseif ui:TreeNode( content_id ) then
                if ui:Selectable( "[Generate Random]" ) then
                    selection = result.id
                    selected_skin_table = nil
                end
                for i, skin_table in ipairs( skins or table.empty ) do
                    local str
                    if skin_table.alias then
                        str = string.format( "%s (%s)", skin_table.name, skin_table.alias )
                    else
                        str = tostring(skin_table.name)
                    end
                    if skin_pred == nil or skin_pred( content_id, skin_table ) then
                        if ui:Selectable( str ) then
                            selection = result.id
                            selected_skin_table = skin_table
                        end
                    else
                        ui:TextColored( 0x777777FF, str )
                    end
                end
                ui:TreePop()
            end
        end
        ui:EndChild()
    end

    return selection, selected_skin_table
end

function DebugUtil.CreateDebugNode( v, offset )
    if v == nil then
        return DebugNil()
    elseif type(v) == "function" then
        return DebugCustom( v )
    elseif type(v) == "table" then
        -- Try to link this table to a specialized DebugNode.
        if v._class then
            local class = DebugUtil.FindRegisteredClass( v )
            if class then
                return class( v )
            end
        end

        return DebugTable( v, nil, offset )
    else
        return DebugValue( v )
    end
end

local LOCAL_DEBUG_DATA_FILE = 'scripts/local_debug_data.lua'

function DebugUtil.GetLocalDebugData(key)
    if not DebugUtil.LOCAL_DEBUG_DATA then
        DebugUtil.LOCAL_DEBUG_DATA = (#engine.inst:ListFiles(LOCAL_DEBUG_DATA_FILE) > 0 and dofile(LOCAL_DEBUG_DATA_FILE)) or {}
    end
    if not key then
        return DebugUtil.LOCAL_DEBUG_DATA
    else
        return DebugUtil.LOCAL_DEBUG_DATA[key]
    end
end

function DebugUtil.SetLocalDebugData(key, data)
    local local_debug_data = DebugUtil.GetLocalDebugData()
    local_debug_data[key] = data
end

function DebugUtil.DeleteLocalDebugData(key)
    local local_debug_data = DebugUtil.GetLocalDebugData()
    local_debug_data[key] = nil
end

function DebugUtil.SaveLocalDebugData()
    local local_debug_data = DebugUtil.GetLocalDebugData()
    local save_str = serpent.block(local_debug_data, {comment = false})
    save_str = "return "..save_str
    engine.data:SaveFile(LOCAL_DEBUG_DATA_FILE, save_str)
end

----------------------------------------------------------------
-- Debug render helpers.

function DebugUtil.RenderCoroUI( ui, panel, key, coro )
    ui:PushID( rawstring(key) )

    if type(key) == "table" then
        panel:AppendTable( ui, key )
    else
        ui:Text( tostring(key) )
    end
    ui:Indent( 40 )

    if not coro:IsDone() then
        ui:SameLine( nil, 10 )
        if ui:Button( "Coroutine" ) then
            TheGame:GetDebug():CreatePanel( DebugTraceback( coro.c ))
        end

        if coro.breakpoint then
            ui:SameLine( nil, 10 )
            ui:PushStyleColor( ui.Style_Button, 0x770000FF )
            if ui:Button( "Break" ) then
                coro.breakpoint = nil
            end
            ui:PopStyleColor()
            if ui:IsItemHovered() then
                ui:BeginTooltip()
                ui:TextColored( 0x770000FF, tostring( coro.breakpoint ))
                ui:Separator()
                ui:Text( debug.traceback( coro.c ))
                ui:EndTooltip()
            end
        end
    end

    ui:Unindent( 40 )
    ui:PopID()
end

function DebugUtil.TogglePauseTime( self )
    self.debug_paused = not self.debug_paused
    if self.debug_paused then
        self.speed = TheGame:GetWallTimeScale()
        TheGame:SetWallTimeScale( 0 )
    else
        TheGame:SetWallTimeScale( self.speed )
        self.speed = nil
    end
end

function DebugUtil.RenderTimeScale( self, ui, panel )
    if ui:Checkbox( "Paused", self.debug_paused ) then
        DebugUtil.TogglePauseTime( self )
    end

    ui:SameLine( nil, 10 )

    local speed = self.speed or TheGame:GetWallTimeScale()
    ui:PushItemWidth( 150 )
    local new_speed = ui:SliderFloat("SPEED", speed, 0.01, 2)
    ui:PopItemWidth()

    ui:SameLine( nil, 10 )
    if ui:Button( "Reset" ) then
        new_speed = 1.0
    end

    if new_speed and new_speed ~= speed then
        if self.debug_paused then
            self.debug_paused = false
            self.speed = nil
        end
        TheGame:SetWallTimeScale( new_speed )

    elseif self.debug_paused and ui:Button("Step") then
        TheGame:Update( 1 * FRAMES )
    end
end

local function RenderAssignPlaxSlots( settings, plax, ui, panel )
    local changed = false
    ui:Columns( 3 )

    ui:Text( "Slot ID / Name" )
    ui:NextColumn()

    ui:Text( "Role" )
    ui:NextColumn()

    ui:Text( "Occupant" )
    ui:NextColumn()

    for slot_id, slot in pairs( plax.slot_roots ) do
        ui:PushID( tostring(slot_id) )

        local txt = string.format( "%d [%s]", slot_id, slot.data.name or "NO-NAME" )
        panel:AppendTable( ui, slot, txt )
        ui:NextColumn()

        if slot.data.character_role then
            ui:Text( tostring(slot.data.character_role))
        end
        ui:NextColumn()

        local txt
        if slot.agent then
            txt = tostring( slot.agent )
        else
            txt = "EMPTY"
        end

        if ui:Button( txt ) then
            ui:OpenPopup( "ADD_AGENT" )
            settings.popup_plax_slot = slot_id
        end
        ui:NextColumn()

        ui:PopID()
    end
    ui:Columns( 1 )

    if settings.popup_plax_slot then
        local slot_id = settings.popup_plax_slot
        local slot = plax.slot_roots[ slot_id ]
        ui:PushID( tostring(slot_id) )
        if ui:BeginPopup( "ADD_AGENT" ) then
            local DebugUtil = require "debug/debug_util"
            
            if slot.agent and ui:Button( "Clear" ) then
                settings.plax_slots[ slot_id ] = nil
                changed = true

            else
                if settings.agent_search == nil then
                    settings.agent_search = {}
                end
                local id, skin_table = DebugUtil.ShowAgentSearch( settings.agent_search, ui)
                if id then
                    if slot.agent then
                        plax:ClearSlot( slot_id )
                    end

                    if settings.plax_slots == nil then
                        settings.plax_slots = {}
                    end

                    settings.plax_slots[ slot_id ] = { id, skin_table and skin_table.uuid }
                    changed = true
                end
            end
            ui:EndPopup()
        end
        ui:PopID()
    end

    return changed
end


function DebugUtil.RenderLocationSettings( settings, plax, ui, panel )
    local changed = false
    if ui:Button( "Change Location..." ) then
        ui:OpenPopup( "PICK_LOCATION" )
    end
    if settings.location_id ~= nil then
        ui:SameLine( 0, 10 )
        if ui:Button( string.format( "Clear %s", tostring(settings.location_id) )) then
            settings.location_id = nil
            settings.plax_slots = nil
            changed = true
        end
    end

    local phase_idx = table.arrayfind( PHASE_MASK_STRINGS, settings.day_phase )
    local new_phase_idx = ui:Combo( "Day Phase", phase_idx or 1, PHASE_MASK_STRINGS )
    if new_phase_idx and new_phase_idx ~= phase_idx then
        settings.day_phase = PHASE_MASK_STRINGS[ new_phase_idx ]
        plax:SetDayPhase( 1 << (new_phase_idx - 1) )
        changed = true
    end

    if ui:TreeNode( "Plax Slots" ) then
        changed = RenderAssignPlaxSlots( settings, plax, ui, panel )
        ui:TreePop()
    end


    if ui:BeginPopup("PICK_LOCATION", ui.AlwaysAutoResize) then
        if settings.location_search == nil then
            settings.location_search = {}
        end

        local selection, skin_table = DebugUtil.ShowLocationSearch( settings.location_search, ui )
        if selection then
            settings.location_id = selection
            settings.plax_slots = nil

            ui:CloseCurrentPopup()
            changed = true
        end
        ui:EndPopup()
    end

    return changed
end

function DebugUtil.RenderDebugWorkPositions( ui, panel, work )
    if panel:CollapsingHeader( ui, "Work Positions" ) then
        
        ui:Columns(3)
        
        for k,v in pairs(work) do
            ui:Text( k )
            if ui:IsItemHovered() then
                ui:SetTooltip( v.id )
            end

            ui:NextColumn()
            if v.agent then
                panel:AppendTable(ui, v.agent)
            else
                ui:Text("--")
            end

            ui:NextColumn()

            if v.agent then
                if ui:SmallButton("Fire##"..k) then
                    v:Fire(v.agent)
                end
            else
                if ui:SmallButton("Hire##"..k) then
                    v:TryHire(true)
                end
            end

            ui:NextColumn()

            
        end

        ui:Columns(1)
    end
end

function DebugUtil.RenderBattleCards( self, ui, agent )
    local t = ObtainWorkTable()
    for i, card in ipairs( agent.battler:GetCards() ) do
        if t[ card.id ] == nil then
            t[ card.id ] = true
            local count = agent.battler:GetCardCount( card.id )
            local new_count = ui:SliderInt( card.id, count, 0, 9 )
            if new_count ~= count then
                agent.battler:SetCardCount( card.id, new_count )
            end
        end
    end
    ReleaseWorkTable( t )
    ui:Separator()

    ui:Spacing()
    self.filter_txt = ui:InputText( "Filter", self.filter_txt or "" ) or self.filter_txt
    if ui:Checkbox( "Show All", self.show_all ) then
        self.show_all = not self.show_all
    end

    ui:SameLine( nil, 20 )
    if ui:Button( "Reset" ) then
        table.clear( agent.battler:GetCards() )
    end

    ui:Separator()

    ui:BeginChild( "CARDS", 400, 400 )
    for i, carddef in ipairs( Content.GetAllBattleCards() ) do
        if self.show_all or AgentUtil.CanUseCard( agent, carddef ) then
            local id = carddef.id
            if self.filter_txt == nil or id:find( self.filter_txt ) then
                local count = agent.battler:GetCardCount( id )
                local new_count = ui:SliderInt( id, count, 0, 9 )
                if new_count ~= count then
                    agent.battler:SetCardCount( id, new_count )
                end
            end
        end
    end
    ui:EndChild()
end

function DebugUtil.RenderBitField( ui, flags, bit_field, bit_field_strings )
    if ui:Button( StringizeBitField( flags, bit_field_strings, ", " )) then
        ui:OpenPopup( "FLAGS_POPUP" )
    end
    if flags ~= bit_field.NONE then
        ui:SameLine( nil, 10 )
        if ui:Button( "Clear Flags" ) then
            flags = bit_field.NONE
        end
    end
    if ui:BeginPopup( "FLAGS_POPUP" ) then
        for i, v in ipairs( bit_field_strings ) do
            local bit = bit_field[ v ]
            if ui:MenuItem( v, nil, CheckBits( flags, bit )) then
                flags = ToggleBits( flags, bit )
            end
        end
        ui:EndPopup()
    end
    return flags
end

return DebugUtil
