local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

--------------------------------------------------------------------
-- A debug source for Agents.

local DebugAgent = class( "DebugAgent", DebugNode )
DebugAgent.can_reload = true
DebugAgent.REGISTERED_CLASS = Agent
DebugAgent.MENU_BINDINGS = { debug_menus.AGENT_BINDINGS }

function DebugAgent:init( agent )
    self.agent = agent
    self.menu_params = { agent }
end

function DebugAgent:GetName()
    return tostring(self.agent)
end

function DebugAgent:GetSubject()
    return self.agent
end

function DebugAgent:GetUID()
    return rawstring(self.agent)
end

function DebugAgent:RenderMemory( ui, panel, memory )
    ui:Columns( 3 )
    ui:Text( "Memory Token" )
    ui:NextColumn()

    ui:Text( "Data" )
    ui:NextColumn()

    ui:NextColumn()
    ui:Separator()

    for k, v in pairs(memory) do
        ui:PushID(k)
        panel:AppendKeyValue( ui, k, v )

        if ui:SmallButton( "Clear" ) then
            memory[k] = nil
        end
        ui:NextColumn()
        ui:PopID()
    end
    ui:Columns( 1 )
    ui:Separator()
end

function DebugAgent:RenderPanel( ui, panel )
    local agent = self.agent

    ui:Text( tostring(agent) )

    ui:SameLine( nil, 10 )
    ui:TextColored( 0x00FFFFFF, agent:GetContentID() )

    if agent:GetFactionID() then
        ui:SameLine( nil, 20 )
        ui:TextColored( 0xFF44FFFF, agent:GetFactionID() )
    end

    -- Tags
    if #agent.tags > 0 then
        ui:Text( table.concat( agent.tags, " " ))
    end
    if agent.def.tags then
        if #agent.tags > 0 then
            ui:SameLine( nil, 10 )
        end
        ui:TextColored( 0xCC00CCFF, table.concat( agent.def.tags, " " ))
    end

    ui:Spacing()

    
    if agent:GetParty() then
        ui:Text("Party:")
        ui:SameLine()
        panel:AppendTable( ui, agent:GetParty(), loc.format( "{1} members", #agent:GetParty().members) )
    end

    local brain = agent:GetBrain()
    if brain then
        ui:Text( "Home:" )
        ui:SameLine( nil, 10 )
        panel:AppendTable( ui, brain.home_location )

        if brain:GetWorkPosition() then
            ui:Text( "Work:" )
            ui:SameLine( nil, 10 )
            panel:AppendTable( ui, brain:GetWorkPosition() )
            if brain:GetWorkplace() then
                ui:SameLine( nil, 10 )
                panel:AppendTable( ui, brain:GetWorkplace() )
            end
        end
    end

    if agent:GetLocation() then
        ui:Text( "Location: ")
        ui:SameLine( nil, 10 )
        panel:AppendTable( ui, agent:GetLocation() )
    end

    if TheGame:GetGameState() then
        local relation = ui:Combo( "relationship", agent:GetRelationship(), RELATIONSHIP_ARRAY )
        if relation ~= agent:GetRelationship() then
            agent.social_connections:SetRelationship( relation, TheGame:GetGameState():GetPlayerAgent() )
        end
    end
    
    if agent.work_positions then
        ui:Separator()
        DebugUtil.RenderDebugWorkPositions( ui, panel, agent.work_positions )
    end

    ui:Separator()

    if agent.memory then
        self:RenderMemory( ui, panel, agent.memory )
    end
    if agent.memory_negotiation then
        self:RenderMemory( ui, panel, agent.memory_negotiation )
    end

    ui:NewLine()
    panel:AppendKeyValues( ui, agent )
end