local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

--------------------------------------------------------------------
-- A debug source for Agents.

local DebugAudio = class( "DebugAudio", DebugNode )

function DebugAudio:init()
    self.audio = AUDIO -- For the 'View' menu (access to the audio table)
end

function DebugAudio:RenderPanel( ui, panel )

    local now = GetWallTime()


    if panel:CollapsingHeader(ui, "Live Instances (entity / total) ") then
        local snds = {}
        local tots = {}
        for inst,name in pairs(AUDIO.current_sounds) do
            snds[name] = snds[name] or {}
            if inst:GetUserData() > 0 then
                tots[name] = (tots[name] or 0) + 1
            end
            table.insert(snds[name], inst)
        end
        
        for i, name, v in sorted_pairs(snds) do
            if ui:TreeNode(string.format("%s (%d/%d)", name, (tots[name] or 0), #snds[name])) then
                for i,inst in ipairs(v) do
                    local guid = inst:GetUserData()
                    ui:Text( "guid:" )
                    ui:SameLine( nil, 5 )

                    if type( guid ) == "number" and guid > 0 then
                        ui:TextColored( 0xCC0000FF, "removed entity" )
                    else
                        ui:Text( serpent.line( guid ))
                    end
                end
                ui:TreePop()
            end

        end
    end


    local ambience = TheGame.ambience
    local music = TheGame.music
    if panel:CollapsingHeader(ui, "Music") then
        if music.music_event_name then
            ui:Text(music.music_event_name)
        end

        if ui:Checkbox("Mute##music", music.mute) then
            music:ToggleMute()
        end

        if music.nav_loop_name then
            ui:Text( "Nav Loop:" )
            ui:SameLine( nil, 10 )
            ui:TextColored( 0xDDDDFFFF, tostring(music.nav_loop_name))
        end
    end

    if panel:CollapsingHeader(ui, "Ambience") then
        if ui:TreeNode( ambience.ambience_event_id or "NONE" ) then
            if ambience.param_vals then
                panel:AppendTable( ui, ambience.param_vals, "params" )
            end
            ui:TreePop()
        end

        if ui:Checkbox("Mute##locamb", ambience.loc_mute) then
            ambience:ToggleLocMute()
        end
    end

    if panel:CollapsingHeader(ui, "Recent Sounds") then
        local time = GetWallTime()

        local idx = AUDIO.recent_start_idx
        for k = 1, #AUDIO.recent_sounds do
            idx = idx - 1 
            if idx == 0 then
                idx =  #AUDIO.recent_sounds
            end

            local snd = AUDIO.recent_sounds[idx]
            if snd then
                local time_ago = now - snd.time
                if ui:TreeNode( string.format( "%s (%.2f ago)###%s",snd.name, time_ago, rawstring(snd))) then
                    if ui:Button( "Play" ) then
                        if snd.params then
                            local params = {}
                            for k, v in pairs( snd.params ) do
                                table.insert( params, k )
                                table.insert( params, v )
                            end
                            AUDIO:PlayParamEvent( snd.name, table.unpack( params ))
                        else
                            AUDIO:PlayEvent( snd.name )
                        end
                    end

                    ui:Text( "guid:" )
                    ui:SameLine( nil, 5 )

                    if type(snd.guid) == "number" and snd.guid > 0 then
                        ui:TextColored( 0xCC0000FF, "removed entity" )
                    else
                        ui:Text( serpent.line(snd.guid))
                    end

                    if snd.params then
                        panel:AppendTable( ui, snd.params, "params" )
                    end
                    if snd.callstack then
                        ui:Text(snd.callstack)
                    end

                    ui:TreePop()
                end
            end            
        end
    end
end