local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"



function BP(vals)
    local co = coroutine.running()
    assert(co, "CAN ONLY CALL BP FROM WITHIN A COROUTINE")
    if co then
        TheGame:GetDebug():CreatePanel( DebugBreakpoint( co, vals ))
        coroutine.yield()
    end
end


--------------------------------------------------------------------
-- A debug source for Agents.
function harvest_locals(coro, depth)
  local variables = {}
  local idx = 1
  while true do
    local ln, lv = debug.getlocal(coro, depth, idx)
    if ln ~= nil then
      variables[ln] = lv
    else
      break
    end
    idx = 1 + idx
  end
  return variables
end


local DebugBreakpoint = class( "DebugBreakpoint", DebugNode )
DebugBreakpoint.PANEL_WIDTH = 1000

function DebugBreakpoint:init(coroutine, vals)
    self.coro = coroutine
    self.vals = vals
end

function DebugBreakpoint:RenderPanel( ui, panel )
    if coroutine.status(self.coro) == "suspended" then
        if ui:Button("CONTINUE") then
            coroutine.resume(self.coro)
            panel:Close()
        end
    end
    ui:Separator()
    
    ui:Text( "stack traceback:" )

    if ui:Button( "Coroutine", 0, 14 ) then
        TheGame:GetDebug():CreatePanel( DebugTraceback( self.coro ))
    end

    if self.vals then
        if panel:CollapsingHeader( ui, "VALUES" ) then
            panel:AppendKeyValues( ui, self.vals )
        end
    end

    if panel:CollapsingHeader( ui, "LOCALS" ) then
        panel:AppendKeyValues( ui, harvest_locals(self.coro, 2) )
    end

    if panel:CollapsingHeader( ui, "CALLSTACK" ) then
        local i = 1
        while true do
            local info = debug.getinfo( self.coro, i )
            if info then
                local fnname = info.name or string.format( "<%s:%d>", info.short_src, info.linedefined )
                local txt = string.format( "%s:%d in function '%s'", info.short_src, info.currentline, fnname )
                if ui:Selectable( txt ) then
                    self.selected_frame = i
                end

                if self.selected_frame == i then
                    ui:Indent( 20 )
                    panel:AppendKeyValues( ui, harvest_locals(self.coro, i) )
                    ui:Unindent( 20 )
                end
                i = i + 1
            else
                break
            end
        end
    end
end
