local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

--------------------------------------------------------------------

local DebugCharacterBuilds = class( "DebugCharacterBuilds", DebugNode )


function DebugCharacterBuilds:RefreshCounts()
    local build_counts = {}
    for i, v in pairs( Content.GetAllCharacterBuilds()) do
        build_counts[i] = {}
    end

    for guid, v in TheGame:GetGameState():Agents() do
        local build_name = v:GetBuild().id
        table.insert( build_counts[ build_name ], v )
    end

    self.build_counts = build_counts
end

function DebugCharacterBuilds:RenderPanel( ui, panel )
    if self.build_counts == nil or ui:Button( "Recount Builds" ) then
        self:RefreshCounts()
    end

    self.filter_txt = ui:InputText( "Filter", self.filter_txt or "" ) or self.filter_txt

    ui:Spacing()

    ui:Columns( 2 )
    ui:Text( "Build Name" )
    ui:NextColumn()

    ui:Text( "Current Usage" )
    ui:NextColumn()
    ui:Separator()

    for i, build_name, agents in sorted_pairs( self.build_counts ) do
        if self.filter_txt == nil or build_name:match( self.filter_txt ) then
            ui:Text( build_name )
            ui:NextColumn()

            if #agents == 0 then
                ui:PushStyleColor( ui.Style_Button, 0x770000FF )
            else
                ui:PushStyleColor( ui.Style_Button, 0x007700FF )
            end

            if ui:Button( string.format( "%d##%s", #agents, build_name ), 50 ) then
                ui:OpenPopup( "SHOW_AGENTS" )
                self.popup_agents = agents
            end
            ui:PopStyleColor()
            ui:NextColumn()
        end
    end
    ui:Columns( 1 )

    if ui:BeginPopup( "SHOW_AGENTS" ) then
        ui:Text( string.format( "%d agents", #self.popup_agents ))
        ui:Separator()
        for i, agent in ipairs( self.popup_agents ) do
            panel:AppendTable( ui, agent )
        end
        ui:EndPopup()
    end
end

