local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

--------------------------------------------------------------------


local DebugDecks = class( "DebugDecks", DebugNode )
DebugDecks.can_reload = true
DebugDecks.PANEL_WIDTH = 800

function DebugDecks:init()
    self:RefreshDecks()
    self.output = TheGame:GetGameState() and "Double-click a deck to load it!" or "No current game"
end

function DebugDecks:RefreshDecks()
    self.sorted_decks = {}
    for k, filename in pairs(engine.inst:ListFiles( 'scripts/saved_decks/*.lua')) do
        local root_name = filename:match( "^(.+)[.]lua$" )
        local filename = "saved_decks/" .. root_name
        package[ filename ] = nil
        
        local deck = require(filename)
        deck.name = root_name
        table.insert(self.sorted_decks, deck)
    end

    table.sort(self.sorted_decks, function(a,b) 
        if a.character == b.character then
            return a.name < b.name
        else
            return a.character < b.character
        end
    end)
end

function DebugDecks:RenderDecks( ui, panel, player_id )
    local player_id = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() and TheGame:GetGameState():GetPlayerAgent():GetContentID()
    ui:Columns( 5 )
    for i, t in ipairs( self.sorted_decks ) do
        if player_id == nil or t.character == player_id then
            if ui:Selectable( t.name.."###_IDENTIFIER"..i, self.selected_save == t, ui.Selectable_SpanAllColumns ) then
                if self.selected_save == t then
                    if TheGame:GetGameState() then
                        if TheGame:GetGameState():SetDecks(t) then
                            self.output = "Loaded " .. t.name
                        else
                            self.output = "Error Loading " .. t.name
                        end


                    end
                else
                    self.selected_save = t
                    self.output = ""
                end
            end

            ui:NextColumn()
            ui:Text( t.character )
            ui:NextColumn()

            ui:Text( loc.format("{1} Battle", #t.battle_cards) )
            if ui:IsItemHovered() then
                ui:BeginTooltip()
                ui:Text( tostr(t.battle_cards ))
                ui:EndTooltip()
            end
            ui:NextColumn()

            ui:Text( loc.format("{1} Negotiation", #t.negotiation_cards) )
            if ui:IsItemHovered() then
                ui:BeginTooltip()
                ui:Text( tostr(t.negotiation_cards ))
                ui:EndTooltip()
            end
            ui:NextColumn()

            ui:Text( loc.format("{1} Grafts", #t.grafts) )
            if ui:IsItemHovered() then
                ui:BeginTooltip()
                ui:Text( tostr(t.grafts ))
                ui:EndTooltip()
            end
            ui:NextColumn()
        end
    end
    ui:Columns( 1 )
end

function DebugDecks:RenderPanel( ui, panel )


    self:RenderDecks( ui, panel )

    if TheGame:GetGameState() then
        if ui:Button("Save Current",nil,nil, self.file_name == nil or self.file_name == "") then
            local save_str = TheGame:GetGameState():GetDeckStateString()
            if save_str and self.file_name:trim() ~= "" then
                local fname = 'scripts/saved_decks/' .. self.file_name:trim() .. '.lua'
                engine.data:SaveFile( fname, save_str, function(ok)
                    if ok then
                        self.output = "Saved " .. fname
                    else
                        self.output = "Error saving " .. fname
                    end

                 end)

                self:RefreshDecks()
            end
        end
        ui:SameLine( nil, 10 )
        self.file_name = ui:InputText( "name", self.file_name or "" ) or self.file_name

    end

    if self.output then
        ui:Text(self.output)
    end

end