local DebugUtil = require "debug/debug_util"
local debug_menus = require( "debug/debug_menus" )

--------------------------------------------------------------------
-- A debug source for Entities.

local DebugEntity = class( "DebugEntity", DebugNode )
DebugEntity.REGISTERED_CLASS = Entity
DebugEntity.MENU_BINDINGS = { debug_menus.ENTITY_BINDINGS }

-------------------------------------------------------------------
--

function DebugEntity:init( entity )
	assert( is_instance( entity, Entity ))
	self.entity = entity
	self.menu_params = { entity }
end

function DebugEntity:GetName()
	return tostring(self.entity)
end

function DebugEntity:GetUID()
	return rawstring(self.entity)
end

function DebugEntity:RenderPanel( ui, panel )
	if self.entity.removed then
		ui:TextColored( 0xFF0000FF, "DESPAWNED" )
	end

	ui:Text( "Tags:" )
	for k,v in pairs(self.entity.tags or table.empty) do
		ui:SameLine(nil, 10) ui:Text( k )
    end
    local prefab = self.entity.prefab
    for k,v in pairs((prefab.def and prefab.def.tags) or table.empty) do
    	ui:SameLine(nil, 10) ui:TextColored( 0xFFCCFFFF, k )
    end
   
	if ui:CollapsingHeader( "Spawn Data" ) then
		panel:AppendKeyValues( ui, self.entity.spawn_data )
	end

	if panel:CollapsingHeader( ui, "Components" ) then
	    for k, v in ipairs(self.entity:GetSortedComponents()) do
	    	panel:AppendTable( ui, v, v._classname )
	    end
	end
end

--------------------------------------------------------------------
-- A debug source for Entity components.

local DebugComponent = class( "DebugComponent", DebugNode )
DebugComponent.REGISTERED_CLASS = Component

function DebugComponent:init( cmp )
	self.entity = cmp.entity
	self.cmp = cmp
	self.menu_params = { cmp }
end

function DebugComponent:GetName()
	return string.format( "%s [%d]", self.cmp._classname, self.entity and self.entity:GetGUID() or -1 )
end

function DebugComponent:GetUID()
	return rawstring(self.cmp)
end

function DebugComponent:RenderPanel( ui, panel )
	if self.tickupdating or self.wallupdating or self.lateupdating then
		if self.tickupdating then
			ui:SameLine( nil, 10 )
			ui:TextColored( 0xFF00FFFF, "TickUpdate")
		end
		if self.lateupdating then
			ui:SameLine( nil, 10 )
			ui:TextColored( 0xFF00FFFF, "LateUpdate")
		end
		if self.wallupdating then
			ui:SameLine( nil, 10 )
			ui:TextColored( 0xFF00FFFF, "WallUpdate")
		end
	end

	if self.cmp.RenderDebugUI then
		self.cmp:RenderDebugUI( ui, panel )
	else
		panel:AppendKeyValues( ui, self.cmp )
	end
end
