--------------------------------------------------------------------
-- Show and modify in-game factions.

local DebugFactionMgr = class( "DebugFactionMgr", DebugNode )
DebugFactionMgr.REGISTERED_CLASS = FactionMgr

function DebugFactionMgr:init( faction_mgr )
	self.faction_mgr = faction_mgr
	self.faction_ids = {}
	for faction_id, faction in faction_mgr:Factions() do
		table.insert( self.faction_ids, faction_id )
	end
	table.sort( self.faction_ids )
	self.current_faction_idx = table.arrayfind( self.faction_ids, PLAYER_FACTION )
    self.menu_params = { faction_mgr }
end

function DebugFactionMgr:RenderPanel( ui, panel )
	local mgr = self.faction_mgr

	local idx = ui:Combo( "Faction", self.current_faction_idx or 1, self.faction_ids )
	if idx and idx ~= self.current_faction_idx then
		self.current_faction_idx = idx
	end
	local current_faction_id = self.faction_ids[ idx ]

	ui:Spacing()
	ui:Separator()
	ui:Spacing()

    for i, faction_id, faction in sorted_pairs( mgr.factions ) do
		if faction_id ~= current_faction_id and current_faction_id then
	    	ui:PushID( faction_id )
	    	local icon = faction:GetIcon()
			ui:Image( icon or global_images.black, 36, 36 )
    		
    		ui:SameLine( nil, 10 )
    		ui:BeginGroup()

    		panel:AppendTable( ui, faction, faction_id )
    		if faction:IsLawful() then
	    		ui:SameLine( nil, 5 )
	    		ui:TextColored( 0x77FFFFFF, "Lawful" )
	    	end
	    	if faction:IsUnlawful() then
	    		ui:SameLine( nil, 5 )
	    		ui:TextColored( 0xFF7777FF, "Unlawful" )
	    	end

    		local rep = faction:GetFactionRelationship( current_faction_id )
            local new_rep = ui:Combo( "", rep, RELATIONSHIP_ARRAY )
            if new_rep and new_rep ~= rep then
                faction:SetFactionRelationship(current_faction_id, new_rep)
            end
            ui:EndGroup()
	    	ui:PopID()

			ui:Separator()
	    end
	end
	ui:Columns(1)
end
