local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"
require "battle/fighter"

--------------------------------------------------------------------
-- A debug source for fighters.

local DebugFighter = class( "DebugFighter", DebugNode )
DebugFighter.REGISTERED_CLASS = Fighter
DebugFighter.MENU_BINDINGS = { debug_menus.FIGHTER_BINDINGS }

function DebugFighter:init( fighter )
    self.fighter = fighter
    self.menu_params = { fighter }
end

function DebugFighter:GetName()
    return tostring(self.fighter)
end

function DebugFighter:GetUID()
    return rawstring(self.fighter.agent)
end

function DebugFighter:RenderPanel( ui, panel )
    panel:AppendTable( ui, self.fighter.agent )

    ui:Text( self.fighter:GetStatus() )
    ui:Spacing()

    local flourish_graft = self.fighter.agent.graft_owner and self.fighter.agent.graft_owner:GetGraft( "flourish_tracker" )

    if flourish_graft then
        local new_value = ui:SliderFloat("FLOURISH", flourish_graft.userdata.battle_flourish or 0, 0, flourish_graft.userdata.battle_max or 200)
        if new_value and new_value ~= flourish_graft.userdata.battle_flourish then
            flourish_graft:GetDef().SetBattleFlourish(flourish_graft, self.fighter.battle, new_value)
        end
    end

    for i, stat, raw_value in sorted_pairs(self.fighter.stat_value) do
        local value = self.fighter:GetStat( stat )
        local min, max = self.fighter:GetStatBounds(stat)
        local new_value
        if stat == "MORALE" then
            new_value = ui:SliderFloat(stat, value, min, max)
        else
            new_value = ui:SliderInt(stat, value, min, max)
        end        
        if new_value and new_value ~= value then
            self.fighter:DeltaStat( stat, new_value - value )
        end
    end

    if ui:CollapsingHeader( "Conditions" ) then
        local battle_defs = require "battle/battle_defs"
        ui:Columns( 2 )
        for i, id, def in sorted_pairs( battle_defs.CONDITIONS ) do
            local condition = self.fighter:GetCondition( id )
            local current_stacks = condition and condition:GetStacks() or 0
            local stacks = ui:SliderInt( id, current_stacks, 0, def.max_stacks )
            ui:NextColumn()

            panel:AppendTable( ui, condition )
            ui:NextColumn()

            if stacks and stacks ~= current_stacks then
                if condition then
                    if stacks == 0 then
                        self.fighter:RemoveCondition( id )
                    else
                        self.fighter:AddCondition( id, stacks - condition:GetStacks() )
                    end
                else
                    self.fighter:AddCondition( id, stacks )
                end
            end
        end
        ui:Columns( 1 )
    end

    if self.fighter.behaviour and panel:CollapsingHeader( ui, "Behaviour" ) then
        panel:AppendKeyValues( ui, self.fighter.behaviour )
    end
    ui:Separator()

    panel:AppendKeyValues( ui, self.fighter )
end
