local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"
require "debug/inspectors/debug_collection"

--------------------------------------------------------------------

local COLS =
{
    ID = { "ID", "id",
        function( self, def )
            return def.id
        end
    },

    TYPEID = { "Type", "typeid",
        function( self, def )
            return def.typeid
        end
    },

    NAME = { "Name", "name",
        function( self, def )
            return def.name
        end
    },

    ART = { "Art",
        function( self, ui, panel, def )
            if def.hidden then
                ui:Text( "Hidden" )
                ui:NewLine()
                ui:NewLine()
           else
                local blended = true
                ui:Image( def.icon, 48, 48, blended )
                if ui:IsItemHovered() then
                    ui:SetTooltip( tostring(def.icon_path) )
                end
            end
        end,
        function( self, def )
            return tostring(def.icon_path)
        end,
    },
}
--------------------------------------------------------------------

local DebugIcons = class( "DebugIcons", DebugCollection )
DebugIcons.COLS = COLS
DebugIcons.DEFAULT_COLS =  { "TYPEID", "ART", "NAME" }
DebugIcons.SETTINGS_ID = "DEBUG_LIST_ICONS"

DebugIcons.MENU_BINDINGS =
{
    {
        name = "Conditions",
        {
            Text = function( t )
                return "Missing Only"
            end,
            Checked = function( self )
                return self.settings.missing_only == true
            end,
            Do = function( self )
                self.settings.missing_only = not self.settings.missing_only
                self:RefreshItems()
                self:SaveSettings()
            end
        },
    }
}

function DebugIcons:RefreshItems()
    local negotiation_defs = require "negotiation/negotiation_defs"    
    local battle_defs = require "battle/battle_defs"

    local items = {}
    for id, def in pairs( Content.GetAllNegotiationModifiers()) do
        local item =
        {
            id = id, name = def.name,
            typeid = "Modifier",
            hidden = def.modifier_type == MODIFIER_TYPE.HIDDEN,
            icon = def.icon,
            icon_path = def.icon_path
        }
        table.insert( items, item )
    end
    for id, def in pairs( Content.GetAllBattleConditions()) do
        local item =
        {
            id = id, name = def.name,
            typeid = "Condition",
            hidden = def.hidden,
            icon = def.icon,
            icon_path = def.icon_path
        }
        table.insert( items, item )
    end
    for i, def in ipairs( Content.GetAllGrafts()) do
        local item =
        {
            id = def.id, name = def.name,
            typeid = string.format( "%s Graft", def.type ),
            hidden = def.hidden,
            icon = def.img,
            icon_path = def.img_path
        }
        table.insert( items, item )
    end
    for i, def in ipairs( Content.GetAllBattleCards()) do
        local item =
        {
            id = def.id, name = def.name,
            typeid = "Battle Card",
            hidden = CheckAnyBits( def.flags, battle_defs.CARD_FLAGS.NPC | battle_defs.CARD_FLAGS.SPECIAL ),
            icon = def.icon,
            icon_path = def.path_str,
        }
        table.insert( items, item )
    end
    for i, def in ipairs( Content.GetAllNegotiationCards()) do
        local item =
        {
            id = def.id, name = def.name,
            typeid = "Negotiation Card",
            hidden = CheckAnyBits( def.flags, negotiation_defs.CARD_FLAGS.OPPONENT | negotiation_defs.CARD_FLAGS.BYSTANDER | negotiation_defs.CARD_FLAGS.SPECIAL ),
            icon = def.icon,
            icon_path = def.path_str,
        }
        table.insert( items, item )
    end
    for id, def in pairs( Content.GetAllQuests() ) do
        -- if def.qtype ~= QTYPE.FOLLOWUP and def.qtype ~= QTYPE.SCENARIO and def.qtype ~= QTYPE.STORY and def.qtype ~= QTYPE.EXPERIMENT then
        local title, icon
        for i, obj in def:Objectives() do
            if obj.mark then
                title = obj.title
                icon = obj.icon or def.icon
                break
            end
        end
        if title ~= nil then
            icon = icon or global_images.default
            local item =
            {
                id = def.id, name = string.format( "%s (%s)", tostring(title), def.qtype ),
                typeid = "Quest",
                hidden = false,
                icon = icon,
                icon_path = def.icon_path or (icon and icon:GetFileName())
            }
            table.insert( items, item )
        end
    end
    for id, def in pairs( Content.GetAllAchievements() ) do
        local item = 
        {
            id = def.id, name = def.name,
            typeid = "Achievement",
            icon = def.icon,
            icon_path = def.path_str,
        }
        table.insert( items, item )
        local item = 
        {
            id = def.id, name = def.name,
            typeid = "Achievement (Locked)",
            icon = def.locked_icon,
            icon_path = def.locked_path_str,
        }
        table.insert( items, item )
    end

    return items
end

function DebugIcons:IsShown( def )
    if self.settings.no_hidden and def.hidden then
        return false
    end
    if self.settings.missing_only and (def.hidden or def.icon ~= global_images.default) then
        return false
    end
    return true
end
