local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

--------------------------------------------------------------------
-- A debug source for Agents.

local DebugLocalization = class( "DebugLocalization", DebugNode )

DebugLocalization.PANEL_AUTOSIZE = true

function DebugLocalization:init()
	self.export_option = 1
	self.lookup_msgstr = false
end

function DebugLocalization:RenderPanel( ui, panel )
	if ui:RadioButton( "Core Game", self.export_option, 1 ) then
		self.export_option = 1
	end

	for i, id in ipairs( Content.GetInstalledMods() ) do
		if ui:RadioButton( tostring(id), self.export_option, i + 1 ) then
			self.export_option = i + 1
		end
	end

	ui:Separator()

	if ui:Checkbox( string.format( "Populate msgstr with %s strings", TheGame:GetLanguage() ), self.lookup_msgstr ) then
		self.lookup_msgstr = not self.lookup_msgstr
	end

	if ui:Button( "Export Localization" ) then
		local filter_count = 0

		local loc_translator = require "util/loc_translator"
		if self.export_option == 1 then
			
			local function Filter( t, ... )
				if t.GetModID and t:GetModID() then
					print( "Filtered out", t.id, " because it is mod content." )
					return false
				end

				if is_instance( t, Convo ) then
					if t.quest_id then
						local qdef = Content.GetQuestDef(t.quest_id)
						
						if qdef and qdef.unfinished then
							print( "Filtered out unfinshed quest convo: ", t.id )
							filter_count = filter_count + 1
							return false
						end

					end

				end
				
				if is_instance( t, QuestDef ) then

					if t.unfinished then
						print( "Filtered out unfinshed quest: ", t.id )
						filter_count = filter_count + 1
						return false
					end

				end

				return true
			end


			loc_translator.generateUnifiedPot( self.lookup_msgstr, Filter )
			self.msg = string.format( "Exported unified .pot file! (%d content objects filtered)", filter_count )
		else
			local mod_id = Content.GetInstalledMods()[ self.export_option - 1 ]
			loc_translator.generateModPot( mod_id )
			self.msg = string.format( "Exported .pot for mod %s! (%d content objects filtered)", mod_id, filter_count )
		end
		self.msg_time = os.clock()
	end

	if self.msg then
		ui:Text( self.msg )
		if self.msg_time + 5 < os.clock() then
			self.msg = nil
		end
	end

	-- local enabled = engine.inst:IsDebuggingLocalization()
	-- if ui:Checkbox( "Toggle Loc Debug", enabled ) then
	-- 	engine.inst:SetDebuggingLocalization( not enabled )
	-- end
end
