local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

--------------------------------------------------------------------
-- A debug source for Agents.

local DebugLocation = class( "DebugLocation", DebugNode )
DebugLocation.REGISTERED_CLASS = Location
DebugLocation.MENU_BINDINGS = { debug_menus.LOCATION_BINDINGS }

function DebugLocation:init( location )
    self.location = location
    self.menu_params = { location }
end

function DebugLocation:GetSubject()
    return self.location
end

function DebugLocation:GetUID()
    return rawstring(self.location)
end


function DebugLocation:RenderPanel( ui, panel )

    if self.screen then
        panel:AppendTable( ui, self.screen )
    end
       
    if self.location == nil then
        return
    end

    ui:TextColored( 0x00FFFFFF, tostring(self.location) )
    ui:SameLine( nil, 10 )
    panel:AppendTable( ui, self.location:GetContent(), self.location:GetContentID() )

    ui:Text( loc.format( "Plax: {1}", tostring(self.location:GetPlax())))
    
    local faction_id = self.location:GetFactionID()
    if faction_id then
        ui:Text( "Faction:")
        ui:SameLine( nil, 10 )
        panel:AppendTable( ui, TheGame:GetGameState():GetFaction( faction_id ), faction_id )
    end

    if self.location.room_occupants then
        ui:Text( "Occupants: ")
        for i, agent in ipairs( self.location.room_occupants ) do
            ui:SameLine( nil, 10 )
            panel:AppendTable( ui, agent )
        end
    end
    ui:Separator()

    ui:Text( "Tags: ")
    for tag, _ in self.location:Tags() do
        ui:SameLine( nil, 10 )
        ui:TextColored( 0xCC00CCFF, tag )
    end
    for i, tag in ipairs( self.location:GetContent().tags or table.empty ) do
        ui:SameLine( nil, 10 )
        ui:TextColored( 0x770077FF, tag )
    end

    ui:Text( "Flags: " .. StringizeBitField( self.location:GetFlags(), LOCFLAGS_STRINGS ) )

    if self.location.work then
        DebugUtil.RenderDebugWorkPositions( ui, panel, self.location.work )       
    end

    if #self.location.agents > 0 then
        if panel:CollapsingHeader( ui, "Agents" ) then
            ui:Columns(3)
            for k,agent in ipairs(self.location.agents) do
                panel:AppendTable( ui, agent )
                ui:NextColumn()
                ui:Text( agent:GetContentID() )
                ui:NextColumn()
                ui:Text( agent:GetRoleAtLocation() or "")
                ui:NextColumn()
            end
            ui:Columns(1)
        end
    end
    

    if self.location.memory ~= nil and panel:CollapsingHeader( ui, "MEMORY" ) then
        panel:AppendKeyValues( ui, self.location.memory )
    end
end

-----------------------------------------------------------------
-- Just a cheap hook to create a DebugLocation from the screen.

local DebugLocationScreen = class( "DebugLocationScreen", DebugLocation )
DebugLocationScreen.REGISTERED_CLASS = Screen.LocationScreen

function DebugLocationScreen:init( screen )
    DebugLocation.init( self, screen.location )
    self.screen = screen
end

function DebugLocationScreen:RenderPanel( ui, panel )
    DebugLocationScreen._base.RenderPanel( self, ui, panel )
end

function DebugLocationScreen:RenderContextMenu( ui, panel )
    if ui:BeginMenu( "Debug Flags" ) then
        panel:AddDebugMenu( ui, debug_menus.DEBUG_TOGGLES )
        ui:EndMenu()
    end

    panel:AddDebugMenu( ui, debug_menus.DEBUG_CONTEXT_MENU )
end

