local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"
require "debug/inspectors/debug_collection"

--------------------------------------------------------------------

local COLS =
{
    ID = { "ID", "id",
        function( self, modifier )
            return modifier.id
        end
    },

    NAME = { "Name", "name",
        function( self, modifier )
            return modifier.name
        end
    },

    ART = { "Art",
        function( self, ui, panel, modifier )
            if modifier.modifier_type == MODIFIER_TYPE.HIDDEN then
                ui:Text( "Hidden" )
                ui:NewLine()
                ui:NewLine()
           else
                ui:Image( modifier.icon, 48, 48 )
                if ui:IsItemHovered() then
                    ui:SetTooltip( tostring(modifier.icon_path) )
                end
            end
        end,
        function( self, modifier )
            return tostring(modifier.icon_path)
        end,
    },

    SOUND = { "Sound", "sound",
        function( self, modifier )
            return tostring(modifier.sound)
        end,        
    },
}
--------------------------------------------------------------------

local DebugNegotiationModifiers = class( "DebugNegotiationModifiers", DebugCollection )
DebugNegotiationModifiers.COLS = COLS
DebugNegotiationModifiers.DEFAULT_COLS =  { "ART", "NAME" }
DebugNegotiationModifiers.SETTINGS_ID = "DEBUG_LIST_NEGOTIATION_MODIIFERS"

DebugNegotiationModifiers.MENU_BINDINGS =
{
    {
        name = "Modifiers",
        {
        },
    }
}

function DebugNegotiationModifiers:ShowSounds()
    self.settings.columns = { "ART", "NAME", "SOUND" }
    self.settings.no_hidden = true
    return self
end

function DebugNegotiationModifiers:RefreshItems()
    local negotiation_defs = require "negotiation/negotiation_defs"    

    local modifiers = {}
    for id, carddef in pairs( Content.GetAllNegotiationModifiers()) do
        table.insert( modifiers, carddef )
    end
    return modifiers
end

function DebugNegotiationModifiers:IsShown( def )
    if self.settings.no_hidden and def.modifier_type == MODIFIER_TYPE.HIDDEN then
        return false
    end
    if def.modifier_type == MODIFIER_TYPE.CORE then
        return false
    end
    return true
end
