local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

require "ui/screens/PlaxEditor"
require "ui/screens/NegotiationScreen"
require "ui/screens/FightScreen"
require "ui/screens/AttackTester"
require "ui/screens/ConvoBuildDebug"
require "ui/screens/TestScreen"
require "ui/screens/cardpreviewscreen"
require "ui/screens/convotestscreen"
require "ui/screens/particleeditorscreen"
require "ui/screens/bosspreviewscreen"
require "ui/screens/battlepreviewtestscreen"
local daily = require "daily_util"

--------------------------------------------------------------------
-- A quick-switch panel for tools.

local DebugQuickSwitcher = class( "DebugQuickSwitcher", DebugNode )

function DebugQuickSwitcher:BeginRenderPanel( ui, panel )
    return ui:Begin( string.format( "Quick Switcher###%d", panel.uid ), ui.MenuBar| ui.AlwaysAutoResize )
end

function DebugQuickSwitcher:RenderPanel( ui, panel )
    local title_color = 0xaaaaaaff
    local indent = 10
    ui:TextColored( title_color,  "New run" )
    ui:Indent( indent )
    for k, data in ipairs(GetAllPlayerBackgrounds()) do
        local act_id = data.acts and data.acts[1] and data.acts[1].id
        if act_id and ui:Selectable( string.format( "Quick Play %s", act_id ) ) then
            local settings = TheGame:GetLocalSettings()
            settings.SKIP_SLIDE_SHOWS = true
            settings.FAST_STARTUP = true
    
            TheGame.scheduler:DoTaskInTime( "QUICKSTART", 0.01,
                function()
                    local options = GameState.CreateOptions( act_id )
                    TheGame:StartNewGame( options )
                end )
        end
    end

    if ui:Selectable( "Daily..." ) then
        ui:OpenPopup( "DAILIES" )
    end
    if ui:BeginPopup( "DAILIES" ) then
        self.daily_difficulty = ui:SliderInt( "Difficulty", self.daily_difficulty or 1, 1, 3 )
        ui:Separator()
        local now = os.time()
        local when = {
            [-2] = "Yesterday - 1",
            [-1] = "Yesterday",
            [0] = "Today",
            [1] = "Tomorrow",
        }
        for i = -2, 4 do
            local time = now + i * 60*60*24
            local daily_data = daily.GenerateDaily( time )
            local txt = string.format( "%s : %s (%s)", daily_data.character, daily_data.id, when[i] or string.format( "%+d", i ) )
            if ui:MenuItem( txt ) then
                local prestige = DAILY_DIFFICULTY_OPTIONS[ self.daily_difficulty ].prestige
                Screen.DailyChallengeScreen.StartDaily( daily_data, prestige )
            end
        end
        ui:EndPopup()
    end

    self:AddScreenOption( ui, panel, "MainMenu", Screen.MainMenu )
    self:AddScreenOption( ui, panel, "Card Compendium", Screen.Compendium )
    ui:Unindent( indent )
    -- ui:Separator()
ui:Spacing()
ui:Spacing()
ui:Spacing()
    ui:TextColored( title_color,  "Gameplay" )
    ui:Indent( indent )
    ui:PushStyleColor( ui.Style_Text, 0xFF9C5Aff )
    self:AddScreenOption( ui, panel, "Fight screen", Screen.FightScreen )
    ui:PopStyleColor()
    ui:PushStyleColor( ui.Style_Text, 0x9E91FFff )
    self:AddScreenOption( ui, panel, "Negotiation Screen", Screen.NegotiationScreen )
    ui:PopStyleColor()
    self:AddScreenOption( ui, panel, "Test UI Screen", Screen.TestScreen )
    ui:Unindent( indent )
    -- ui:Separator()
ui:Spacing()
ui:Spacing()
ui:Spacing()
    ui:TextColored( title_color, "Anim, FX and Plax" )
    ui:Indent( indent )
    self:AddScreenOption( ui, panel, "Plax Editor", Screen.PlaxEditor )
    self:AddScreenOption( ui, panel, "Attack Tester", Screen.AttackTester )
    self:AddScreenOption( ui, panel, "Build Tester", Screen.ConvoBuildDebug )
    self:AddScreenOption( ui, panel, "Convo Tester", Screen.ConvoTestScreen )
    self:AddScreenOption( ui, panel, "Particle Editor", Screen.ParticleEditorScreen )
    self:AddScreenOption( ui, panel, "Boss Preview", Screen.BossPreviewScreen )
    self:AddScreenOption( ui, panel, "Battle Preview", Screen.BattlePreviewScreen )
    if ui:Selectable( "Colour Settings" ) then TheGame:GetDebug():CreatePanel( DebugColourSettings() ) end
    ui:Unindent( indent )
    -- ui:Separator()
ui:Spacing()
ui:Spacing()
ui:Spacing()
    ui:TextColored( title_color, "Content tools" )
    ui:Indent( indent )
    self:AddScreenOption( ui, panel, "View Cards", Screen.CardPreviewScreen )
    if ui:Selectable( "Export Portraits to folder" ) then TheGame:FE():GetTopScreen():StartCoroutine(DebugQuickSwitcher.ExportPortraitsCoro, TheGame:FE():GetTopScreen()) end
    ui:Unindent( indent )
    -- ui:Separator()
ui:Spacing()
ui:Spacing()
ui:Spacing()
    ui:TextColored( title_color, "Autoplay" )
    ui:Indent( indent )
    if ui:Selectable( "Autoplay Game" ) then TheGame:GetDebugEnv().dbg_test( "test_game" ) end
    if ui:Selectable( "Autoplay Fight" ) then TheGame:GetDebugEnv().dbg_test( "test_battle" ) end
    if ui:Selectable( "Autoplay Negotiation" ) then TheGame:GetDebugEnv().dbg_test( "test_negotiation" ) end
    ui:Unindent( indent )
end

function DebugQuickSwitcher:AddScreenOption( ui, panel, text, class, ... )
    local checked = TheGame:FE():FindScreen( class ) ~= nil
    if ui:Selectable( tostring(text), checked ) and not checked then
        local params = { ... }
        TheGame:Reset( function()
            local screen = class( table.unpack( params ))
            TheGame:FE():InsertScreen( screen )

            if TheGame:GetLocalSettings().AUTOGEN_SETTINGS then
                local classname = class._classname:match( "Screen.(.+)$" )
                if classname then
                    self:SaveToLocalSettings( classname )
                end
            end 
        end )
    end
end

function DebugQuickSwitcher:SaveToLocalSettings( screen_override )
    local lines = {}
    for line in io.lines( "scripts/local_settings.lua" ) do
        table.insert( lines, line )
    end
    lines = table.concat( lines, "\n" )

    local override = string.format( "SCREEN_OVERRIDE = \"%s\"", screen_override )
    lines = lines:gsub( "SCREEN_OVERRIDE%s*=%s*\"([^\"]*)\"", override )

    engine.data:SaveFile( "scripts/local_settings.lua", lines, function(...)
        print( "Saved out new local_settings", ... )
    end )
end

function DebugQuickSwitcher:ExportPortraitsCoro()
    local portrait_size = 300
    local anim_scale = 3.0

    TheGame:FE():PushScreen( Screen.InfoPopup( "GENERATING PORTRAIT IMAGES", "Will start saving images to images/portraits/\nIt will take a while.\n\nA new dialog will show up when it's done." ) )

    -- Get all the characters
    local count = 0
    local all_skins = Content.GetAllCharacterSkins()
    for content_id, skins in pairs( all_skins ) do
        for i, skin in ipairs(skins) do
            -- if count == 18 then break end
            local agent = Agent( content_id, skin )
            if agent:GetSkinID() ~= nil then

                local scene = engine.rendering.Scene()
                local capture = Widget.CaptureImage(scene)
                    :SetSize( portrait_size, portrait_size )
                local anim_container = Widget()
                local anim = anim_container:AddChild(Widget.AnimatedCharacter())
                    :SetPortraitMode(true)
                    :Refresh( agent )
                    :SetScale(anim_scale, anim_scale)
                    :SetPos( -30, -100 )

                local root = engine.rendering.SceneGraphNode()
                root:SetScene(scene)
                root:AddChild(anim_container.node)

                coroutine.yield()
                coroutine.yield()
                coroutine.yield()
                TheGame:FE():GetTopScreen():Delay( 0.1 )

                while not engine.inst:IsAsyncLoadingComplete() do
                    coroutine.yield()
                end


                capture:RenderToScene( anim_container, -portrait_size/2, -portrait_size/2, portrait_size/2, portrait_size/2 )
                engine.inst:SaveTextureToPNG( capture:GetTexture(), "CONTENTSRC:images/portraits/" .. agent:GetSkinID() .. ".png" );

                -- Make sure to call collect garbage a lot so that capture images get their resources freed:
                collectgarbage()

                count = count + 1
            end
        end
    end

    collectgarbage()

    TheGame:FE():PushScreen( Screen.InfoPopup( "PORTRAIT IMAGES GENERATED", count .. " images saved to Griftlands/contentsrc/images/portraits/" ) )
end