--------------------------------------------------------------------
-- Conversation debug master control panel.

local DebugQuips = class( "DebugQuips", DebugNode )

DebugQuips.REGISTERED_CLASS = QuipDatabase

function DebugQuips:init( db )
	self.match_string = ""
	self.db = db or Content.internal.QUIPS -- For View menu option
	self.menu_params = { self.db }
end

function DebugQuips:DoSearch( tags )
	self.search_tags = tags
	self.search_result, self.search_results = self.db:LookupQuip( tags )
end

function DebugQuips:RenderDBEntry( ui, panel, entry, stats )
	local line_count = 1
	if type(entry.dialog) == "table" then
		line_count = #entry.dialog
	end
	if ui:TreeNode( string.format( "%s (%d lines)", serpent.line( entry.tags, { comment = false } ), line_count )) then
		if type( entry.dialog ) == "string" then
			ui:TextColored( 0x7777CCFF, entry.dialog, stats[entry.dialog] or 0 )
		elseif type( entry.dialog ) == "table" then
			for i, line in ipairs( entry.dialog ) do
				ui:TextColored( 0x7777CCFF, string.format( "%d) %s (%d times)", i, line, stats[line] or 0 ))
			end
		else
			ui:TextColored( 0xFF0000FF, "???" )
		end
		ui:TreePop()
	end
end

function DebugQuips:RenderPanel( ui, panel )
	local match_string = ui:InputText( "Tags", self.match_string )
	if match_string then
		self.match_string = match_string
	end
	ui:SameLine( nil, 10 )
	if ui:Button( "Search" ) then
		self:DoSearch( string.split( self.match_string, " " ))
	end

	if self.search_tags then
		ui:Spacing()
		ui:Separator()
		ui:TextColored( 0x00CCCCFF, string.format( "Search: %s (%d candidates)", table.concat( self.search_tags, ", " ), self.search_results and #self.search_results or 0 ))
		if self.search_result then
			ui:Spacing()
			ui:TextColored( 0xCCFFCCFF, self.search_result )
			ui:Spacing()
		end
	end

	if ui:CollapsingHeader( "Recent Lookups", ui.TreeFlags_DefaultOpen ) then
		ui:PushID( "recent" )
		for i, lookup in self.db:RecentLookups() do
			local matches, tags = lookup[1], lookup[2]
			ui:PushStyleColor( ui.Style_Text, 0x00CCCCFF )
			ui:TextWrapped( string.format( "%d) %s", i, table.concat( tags, ", " )))
			ui:PopStyleColor()
			for i, match in ipairs( matches ) do
				if match[1] < matches[1][1] then
					ui:TextColored( 0x770077FF, string.format( "Score: %f.1", match[1] ))
				else
					ui:TextColored( 0xFF00FFFF, string.format( "Score: %f.1", match[1] ))
				end
				ui:SameLine( nil, 10 )
				ui:PushID( rawstring(lookup) )
				self:RenderDBEntry( ui, panel, match[2], self.db.stats )
				ui:PopID()
			end
		end
		ui:PopID()
	end


	if ui:CollapsingHeader( "DB" ) then
		for i, entry in self.db:AllEntries() do
			self:RenderDBEntry( ui, panel, entry, self.db.stats)
		end
	end
end

