local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

--------------------------------------------------------------------
-- Lists save games.

local DebugSaveGames = class( "DebugSaveGames", DebugNode )
DebugSaveGames.can_reload = true
DebugSaveGames.PANEL_WIDTH = 650

function DebugSaveGames:init()
    self.save_games = table.empty
    self:RefreshSaves()
end

function DebugSaveGames:RefreshSaves()
    self.seelcted_save = nil
    TheGame:ListSaveGames( function( save_games )
        self.save_games = copyvalues( save_games )
        table.sort( self.save_games, function( t1, t2 )
                return (t1.data.time or 0) > (t2.data.time or 0)
            end )
    end )
end

function DebugSaveGames:BeginRenderPanel( ui, panel )
    return ui:Begin( "Save Games", ui.MenuBar| ui.AlwaysAutoResize )
end

function DebugSaveGames:RenderPanel( ui, panel )

    if self.refresh then
        self:RefreshSaves()
        self.refresh = false
    end

    if TheGame:GetGameState() then
        local filename = ui:InputText( "Filename", self.save_filename or "" )
        if filename then
            if #filename > 0 then
                self.save_filename = filename
            else
                self.save_filename = nil
            end
        end
        ui:SameLine( nil, 40 )
        ui:Spacing()

        ui:SameLine( nil, 2 )
        if ui:Button( "Save Game", nil, nil, self.save_filename == nil ) then
            TheGame:SaveGame( self.save_filename,
                function()
                    self.refresh = true
                end )
        end

        ui:Separator()
    end

    local filter_str = ui:InputText( "Filter", self.filter_str or "" )
    if filter_str then
        if #filter_str > 0 then
            self.filter_str = filter_str
        else
            self.filter_str = nil
        end
    end
    ui:Spacing()

    ui:Columns( 3 )
    for i, t in ipairs( self.save_games ) do
        if self.filter_str == nil or t.filename:find( self.filter_str ) then
            if ui:Selectable( t.filename, self.selected_save == t, ui.Selectable_SpanAllColumns ) then
                if self.selected_save == t then
                    TheGame:LoadGame( self.selected_save.filename, "autosave" )
                else
                    self.selected_save = t
                end
            end

            ui:NextColumn()

            ui:Text( tostring(t.data and t.data.desc) )
            ui:NextColumn()

            ui:Text( tostring( os.date( "%x %X", t.data.time )))
            ui:NextColumn()
        end
    end
    ui:Columns( 1 )

    ui:Separator()

    if self.selected_save ~= nil then
        ui:TextColored( 0xAAAAAAFF, "Double-click selected save to load" )
        ui:Spacing()
    end

    if ui:Button( "Refresh" ) then
        self.refresh = true
    end

    if self.selected_save ~= nil then
        ui:SameLine( nil, 10 )
        if ui:Button( "DBG" ) then
            DBG( self.selected_save )
        end

        ui:SameLine( nil, 10 )
        ui:PushStyleColor( ui.Style_Button, 0x770000FF )
        if ui:Button( "Delete" ) then
            TheGame:DeleteSave(self.selected_save.name,
                function()
                    self.refresh = true
                end )
        end
        ui:PopStyleColor()
    end
end