local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

--------------------------------------------------------------------
-- A debug source for Agents.

local DebugTextures = class( "DebugTextures", DebugNode )
DebugTextures.PANEL_WIDTH = 800

function DebugTextures:init()
    self.info = engine.inst:GetTextureInfo()
    self.audio = AUDIO -- For the 'View' menu (access to the audio table)
    self.count, self.total_count, self.size = 0, 0, 0
    self.PANEL_WIDTH = 800
end

function DebugTextures:RenderPanel( ui, panel )

    local filter_txt = ui:InputText( "Filter", self.filter_txt or "", ui.TextFlags_EnterReturnsTrue )
    if filter_txt and filter_txt ~= self.filter_txt then
        self.filter_txt = filter_txt
    end
    ui:Text( string.format( "%d of %d textures shown (totalling %s)", self.count, self.total_count, FormatMemorySize( self.size )))

    if ui:Button( "Refresh" ) then
        self.info = engine.inst:GetTextureInfo()
    end
    ui:Separator()

    self.count, self.total_count, self.size = 0, 0, 0
    ui:Columns(7)
    local xoffset = 350
    for i = 1, 7 do
        ui:SetColumnOffset(i, xoffset + i * 70 )
    end
    
    if ui:Selectable("filename") then
        table.sort(self.info, function(a,b) if a.name == b.name then return a.size < b.size else return a.name < b.name end end)
        self.reverse = not self.reverse
    end
    
    ui:NextColumn()

    if ui:Selectable("width") then
        table.sort(self.info, function(a,b) if a.width == b.width then return a.name < b.name else return a.width < b.width end end)
        self.reverse = not self.reverse
    end
    ui:NextColumn()
    
    if ui:Selectable("height") then
        table.sort(self.info, function(a,b) if a.height == b.height then return a.name < b.name else return a.height < b.height end end)
        self.reverse = not self.reverse
    end
    ui:NextColumn()
    
    if ui:Selectable("format") then
        table.sort(self.info, function(a,b) if a.format == b.format then return a.size < b.size else return a.format < b.format end end)
        self.reverse = not self.reverse
    end
    ui:NextColumn()

    if ui:Selectable("mips") then
        table.sort(self.info, function(a,b) if a.mips == b.mips then return a.name < b.name else return a.mips < b.mips end end)
        self.reverse = not self.reverse
    end
    ui:NextColumn()

    if ui:Selectable("size") then
        table.sort(self.info, function(a,b) if a.size == b.size then return a.name < b.name else return a.size < b.size end end)
        self.reverse = not self.reverse
    end
    ui:NextColumn()

    if ui:Selectable("refs") then
        table.sort(self.info, function(a,b) if a.refs == b.refs then return a.name < b.name else return a.refs < b.refs end end)
        self.reverse = not self.reverse
    end
    ui:NextColumn()

    ui:Separator()

    for k = 1, #self.info do
        local v = self.info[self.reverse and #self.info - k + 1 or k]        
        if self.filter_txt == nil or v.name:find( self.filter_txt, 1, true ) then
            if v.texture == nil then
                ui:TextColored( 0xFF0000FF , v.name)
            else
                ui:Text(v.name)
            end
            if ui:IsItemHovered( ) and v.texture then
                ui:BeginTooltip()
                local w = v.width
                local h = v.height
                local m = math.max(v.width, v.height)
                if m > 512 then
                    local f = 512/m
                    w = w * f
                    h = h * f
                end
                ui:RawImage(v.texture, w or 0, h or 0)
                ui:EndTooltip()
            end 
            --ui:Text(v.name or "")
            ui:NextColumn()

            ui:Text(v.width or 0)
            ui:NextColumn()

            ui:Text(v.height or 0)
            ui:NextColumn()

            ui:Text(PIXEL_FORMAT_REV[v.format] or "???")
            ui:NextColumn()

            ui:Text(v.mips or 0)
            ui:NextColumn()

            ui:Text( FormatMemorySize( v.size ))
            ui:NextColumn()

            ui:Text(v.refs or 0)
            ui:NextColumn()

            self.count = self.count + 1
            self.size = self.size + v.size
        end
        self.total_count = self.total_count + 1
    end
    ui:Columns(1)

end