--------------------------------------------------------------------
-- Inspect a traceback and the stack frames therein.

local DebugTraceback = class( "DebugTraceback", DebugNode )
DebugTraceback.can_reload = true

function DebugTraceback:init( thread )
    self.thread = thread
    self.locals = {}
end

function DebugTraceback:RenderPanel( ui, panel )
    ui:Text( "Status:" )
    ui:SameLine( nil, 5 )
    ui:TextColored( 0xFF55FFFF, coroutine.status( self.thread ))
    ui:Text( "stack traceback:" )

    local i = 1
    while true do
        local info = debug.getinfo( self.thread, i )
        if info then
            local fnname = info.name or string.format( "<%s:%d>", info.short_src, info.linedefined )
            local txt = string.format( "%s:%d in function '%s'", info.short_src, info.currentline, fnname )
            if ui:Selectable( txt ) then
                self.selected_frame = i
            end
            if self.selected_frame == i then
                ui:Indent( 20 )
                self:RenderLocals( ui, panel, i, info )
                ui:Unindent( 20 )
            end
            i = i + 1
        else
            break
        end
    end
end

function DebugTraceback:RenderLocals( ui, panel, frame_idx, info )
    table.clear( self.locals )
    local i = 1
    while true do
        local k, v = debug.getlocal( self.thread, frame_idx, i )
        if k == nil then
            break
        else
            self.locals[ k ] = v
            i = i + 1
        end
    end
    panel:AppendKeyValues( ui, self.locals )
end

 --------------------------------------------------------------------
-- A debug source for a Coro object

local DebugCoro = class( "DebugCoro", DebugTraceback )
DebugCoro.REGISTERED_CLASS = Coro

function DebugCoro:init( coro )
    DebugTraceback.init( self, coro and coro.c )
    self.coro = coro
end

function DebugCoro:RenderPanel( ui, panel )
    panel:AppendKeyValues( ui, self.coro )
    ui:Separator()

    self.thread = self.coro.c
    DebugTraceback.RenderPanel( self, ui, panel )
end
