local DebugUtil = require "debug/debug_util"
local debug_menus = require "debug/debug_menus"

local TUNING_SETS = {
	"Tuning Constants",
	"UI Colours",
}

local TUNING_FILE = "scripts/tuning_autogen.lua"

local TUNING_BINDINGS =
{
	name = "Tuning",
	{
		Text = "Save Values",
        Do = function()
			local fl = io.open( TUNING_FILE, "w+" )
			fl:write( string.format( "-- This file was autogenerated on %s by debug_tuning.lua\n", os.date() ) )
			fl:write( "TUNING_CONSTANTS =\n" )
			fl:write( serpent.block( TUNING_CONSTANTS, {comment = false, nocode = true} ))
			fl:write( "\n\n" )

			fl:write( "UICOLOURS =\n{\n" )
			for i, k, clr in sorted_pairs( UICOLOURS ) do
				fl:write( string.format( "\t%s = 0x%08x,\n", k, clr ))
			end
			fl:write( "}\n" )
			fl:close()
		end,
	},
	{
		Text = "Reload Values",
		Do = function()
			package.loaded[ "tuning_autogen" ] = nil
			require "tuning_autogen"
		end,
	}
}

--------------------------------------------------------------------
-- A debug source for Agents.

local DebugTuning = class( "DebugTuning", DebugNode )
DebugTuning.MENU_BINDINGS = { TUNING_BINDINGS }

function DebugTuning:RenderPanel( ui, panel )
	self.tuning_idx = ui:Combo( "Tuning", self.tuning_idx or 1, TUNING_SETS ) or self.tuning_idx
	ui:Separator()

	if self.tuning_idx == 1 then
		self:RenderTuningConstants( ui, panel, TUNING_CONSTANTS )

	elseif self.tuning_idx == 2 then
		self:RenderTuningColours( ui, panel, UICOLOURS)
	end
end

function DebugTuning:RenderTuningConstants( ui, panel, t )
	if t == nil then
		return
	end
	for i, id, v in sorted_pairs( t ) do
		local value = v.current_value
		if v.min_value and v.max_value and value then
			local new_value = ui:SliderFloat( id, value, v.min_value, v.max_value )
			if new_value ~= value and new_value ~= nil then
				v.current_value = new_value
			end
		else
			ui:Text( string.format( "%s: %s", id, tostring(value) )) -- Unsupported type.
		end
	end
end


function DebugTuning:RenderTuningColours( ui, panel, t )
	for i, k, clr in sorted_pairs( t ) do
		local new_clr = ui:ColorEdit3( k, clr )
		if new_clr then
			t[k] = new_clr
		end
	end
end
