local battle_defs = require "battle/battle_defs"
require "ui/screens/fightscreen"
require "cards/card_collection"

------------------------------------------------------------------

TheGame:FE():ClearScreens()
TheGame:GetLocalSettings().ROBOTICS = true
TheGame:SetWallTimeScale( 3.0 )
AUDIO:FinalizeVoiceLoad()

local function GiveRandomGrafts( player, graft_type, n )
	for i = 1, n do
		local defs = Content.GetAllGrafts()
		local t = {}
		for i, def in ipairs( defs ) do
			if def.type == graft_type and AgentUtil.CanUseGraft( player, def ) then
				table.insert( t, def.id )
			end
		end
		local graft_id = table.arraypick( t )
		player.graft_owner:AddGraft( GraftInstance( graft_id, nil, player ))
	end
end

local function GiveCardsFromDeck( player, deck_name )
    local filename = "saved_decks/" .. deck_name
    package[ filename ] = nil
    
    local deck = require(filename)
    for i, id in ipairs( deck.battle_cards ) do
    	player.battler:LearnCard( id )
    end

    print("Cards:", table.concat(deck.battle_cards, " "))
end

local function GiveRandomCards( player, n )
	local t = BattleCardCollection.AllUsableCards( nil, player ):GenerateRandom( n, player )
	local str = {}

	for i, card in ipairs(t) do
		card = player.battler:LearnCard( card.id )
		table.insert( str, card.id )
	end

	print( "Cards:", table.concat( str, " " ))
end

local function GiveRandomAllies( scenario, n )
	local NPCS = {}
	for k, v in pairs( Content.GetAllCharacterDefs() ) do
		if not v.boss and not v.is_template and v.fight_data and v.faction_id ~= PLAYER_FACTION and not v.hide_in_compendium then
			table.insert( NPCS, k )
		end
	end

	local content_id = table.arraypick( NPCS )

	for i = 1, n do
		local agent = Agent( content_id )
		scenario:AddAgentToTeam( agent, TEAM.BLUE )
	end
end

local function GetBosses()
	local bosses = {}
	for k, v in pairs( Content.GetAllCharacterDefs() ) do
		if v.boss and not v.is_template and v.fight_data then
			table.insert( bosses, k )
		end
	end
	return bosses
end

local function GiveRandomEnemies( scenario, difficulty )
	local combat_parties = require "content/combat_parties"
	local t = copykeys( combat_parties )
	local generator = table.arraypick( t )
	local enemies = CreateCombatParty( generator, difficulty )

	for i, agent in ipairs( enemies ) do
		scenario:AddAgentToTeam( agent, TEAM.RED )
	end
end

local function GiveEnemies( scenario, agent_ids )
	for i, id in ipairs( agent_ids ) do
		scenario:AddAgentToTeam( Agent( id ), TEAM.RED )
	end
end

------------------------------------------------------------------

local function CreateBattle()

	TheGame.gamestate = GameState( GameState.CreateOptions( nil, math.random( #DEFAULT_ADVANCEMENT )) )

	local scenario = Scenario( Location( "GROG_N_DOG" ))
	local difficulty = math.random( 1, 5 )
	local PLAYERS = { SAL = 0, ROOK = 1, SMITH = 0 }
	local id = weightedpick( PLAYERS )
	local player = Agent( id )
	TheGame.gamestate:AddPlayerAgent( player )

	print( "Constructing auto-battle..." )
	print( "===========================" )

	local preset_deck -- = "BURN_BOTBISS"
	if preset_deck then
		GiveCardsFromDeck( player, preset_deck )
	else
		GiveRandomGrafts( player, GRAFT_TYPE.COMBAT, math.random(5)-1 )
		GiveRandomGrafts( player, GRAFT_TYPE.SOCIAL, math.random(8)-1 )
		GiveRandomCards( player, math.random(12, 24) )
		GiveRandomAllies( scenario, math.max( 0, difficulty - 3 ))
	end
	
	local enemy_team -- = { "AUTOMECH_BOSS" }
	if enemy_team then
		for i, id in ipairs( enemy_team ) do
			local agent = TheGame:GetGameState():AddSkinnedAgent(id)
			scenario:AddAgentToTeam( agent, TEAM.RED )
		end

	else
		local bosses = GetBosses()
		local vs_boss = #bosses > 0 and math.random() < .2
		if vs_boss then
			local boss_id = table.arraypick(bosses)
			local boss = TheGame:GetGameState():AddSkinnedAgent(boss_id)
			scenario:AddAgentToTeam( boss, TEAM.RED )
		else
			GiveRandomEnemies( scenario, difficulty )
		end
	end

	scenario:AddAgentToTeam( player, TEAM.BLUE )
	scenario:SetPlayerControlled( player )

	print( scenario:GetStringDump() )

	return BattleEngine( scenario )
end

local function RestartTester()
	DoFullGC()

	local screen = TheGame:FE():FindScreen( Screen.FightScreen )
	if screen then
	    TheGame:FE():PopScreen( screen )
	end
    TheGame:FE():InsertScreen( Screen.FightScreen( CreateBattle(), RestartTester ) )
end


RestartTester()
