require "ui/screens/negotiationscreen"
require "cards/card_collection"

------------------------------------------------------------------

TheGame:FE():ClearScreens()
TheGame:GetLocalSettings().ROBOTICS = true
TheGame:SetWallTimeScale( 12.0 )
AUDIO:FinalizeVoiceLoad()

local function GiveRandomGrafts( preset, graft_type, n )
	for i = 1, n do
		local defs = Content.GetAllGrafts()
		local t = {}
		for i, def in ipairs( defs ) do
			if def.type == graft_type and AgentUtil.CanUseGraft( preset.player, def )  then
				table.insert( t, def.id )
			end
		end
		local graft_id = table.arraypick( t )
		preset.starting_grafts[ graft_id ] = 1
	end
end

local function GiveRandomCards( preset, n )
	local t = NegotiationCardCollection.AllUsableCards( nil, preset.player ):GenerateRandom( n )
	local str = {}

	for i, card in ipairs(t) do
		preset.starting_cards[ card.id ] = (preset.starting_cards[ card.id ] or 0) + 1
		table.insert( str, card.id )
	end

	print( "Cards:", table.concat( str, " "))
end

local function GiveCardsFromDeck( preset, deck_name )
    local filename = "saved_decks/" .. deck_name
    package[ filename ] = nil
    
    local deck = require(filename)
    for i, id in ipairs( deck.negotiation_cards ) do
		preset.starting_cards[ id ] = (preset.starting_cards[ id ] or 0) + 1
    end

    print("Cards:", table.concat(deck.negotiation_cards, " "))

    for i, id in ipairs( deck.grafts ) do
    	local def = Content.GetGraft( id )
    	-- Not mettle, cause we don't preserve userdata.
    	if def and def.type ~= GRAFT_TYPE.METTLE_UPGRADE then
	    	preset.starting_grafts[ id ] = 1
	    end
    end

    print("Grafts:", table.concat(deck.grafts, " "))
end

------------------------------------------------------------------

local function CreateNegotiation()

	local NPCS = {}
	for k, v in pairs( Content.GetAllCharacterDefs() ) do
		if not v.is_template and v.negotiation_data and v.faction_id ~= PLAYER_FACTION then
			table.insert( NPCS, k )
		end
	end

	local content_id = table.arraypick( NPCS )

	local PLAYERS = { SMITH = 2, SAL = 1, ROOK = 1 }
	local player_id = weightedpick( PLAYERS )

	print( "Constructing auto-negotiation..." )
	print( "===========================" )

	local preset = {
	    difficulty = math.random( 1, 5 ),
	    opponent_id = { content_id },
	    starting_grafts = {},
	    location_settings = {},
	    -- helpers = { { "BANDIT_GOON" } },
	    -- hinders = {},
	    money = math.random( 0, 50 ),
	    player = Agent( player_id ),
	    starting_cards = {}
	}

	TheGame:GetGameState():GetOptions().advancement_level = math.random( 0, #DEFAULT_ADVANCEMENT )

	-- Define a deck name to use a preset.
	local deck_name = nil -- "HANG_TEST"
	if deck_name then
		GiveCardsFromDeck( preset, deck_name )
	else
		GiveRandomGrafts( preset, GRAFT_TYPE.NEGOTIATION, math.random(5)-1 )
		GiveRandomGrafts( preset, GRAFT_TYPE.SOCIAL, math.random(8)-1 )
		GiveRandomCards( preset, math.random(12, 24) )
	end


	return preset
end

TheGame:FE():InsertScreen( Screen.NegotiationScreen( CreateNegotiation ))

