local Deck = class("Deck")

function Deck:init(id, owner)
    assert(id, "no id for deck")
    self.id = id
    self.owner = owner
    self.cards = {}
end

function Deck:GetContentID()
    return self.id
end

function Deck:SetModID( mod_id )
    self.mod_id = mod_id
end

function Deck:GetModID()
    return self.mod_id
end

function Deck:GetOwner() 
    return self.owner
end

function Deck:AddCards(cards)
    for card_id, count in pairs(cards) do
        for j = 1, count do
            table.insert( self.cards, card_id )
        end
    end
    return self
end

function Deck:AddCardArray( card_ids )
    for i, card_id in ipairs( card_ids ) do
        table.insert( self.cards, card_id )
    end
    return self
end

function Deck:Cards()
    return ipairs( self.cards )
end

function Deck:GenerateCards(t)
    t = t or {}
    for i, card_id in ipairs(self.cards) do
        table.insert(t, self.card_class(card_id))
    end
    return t
end

function Deck:Preview()
    TheGame:FE():InsertScreen(Screen.DeckScreen( self:GenerateCards(), nil, self.widget_class ))
end

function Deck:CountCards()
    return #self.cards
end

function Deck:GetCardCounts()
    local t = {}
    for i, card_id in ipairs( self.cards ) do
        t[ card_id ] = (t[ card_id ] or 0) + 1
    end
    return t
end

--------------------------------------------------------------------------------------------------------------------

local NegotiationDeck = class("NegotiationDeck", Deck)
function NegotiationDeck:init(...)
    self.card_class = Negotiation.Card
    self.widget_class = Widget.NegotiationCard
    NegotiationDeck._base.init(self, ...)
end

--------------------------------------------------------------------------------------------------------------------

local BattleDeck = class("BattleDeck", Deck)
function BattleDeck:init(...)
    self.card_class = Battle.Card
    self.widget_class = Widget.BattleCard
    BattleDeck._base.init(self, ...)
end
