HUB_STATE = MakeEnum{ "HUB", "ATTRACT", "CONFRONT", "LOCATION"}

function IsHubState(hub_state)
    if not IsEnum(hub_state, HUB_STATE) then
        return false
    end
    if hub_state == HUB_STATE.ATTRACT or hub_state == HUB_STATE.CONFRONT then
        return false
    end
    return true
end

-------------------------------------------------------------------------------
-- A hub drives an Encounter.

local BaseHub = class("BaseHub" )
BaseHub.can_reload = true

function BaseHub:init( encounter )
    assert( is_instance( encounter, Encounter ))
    self.cast = {}

    self.enc = encounter
    self.encounter = encounter
    self.caravan = TheGame:GetGameState():GetCaravan()
    self.player = TheGame:GetGameState():GetPlayerAgent()
    self.location = encounter:GetLocation()


    if encounter:GetAgent() then
        self:AssignCastMember( "agent", encounter:GetAgent() )
    elseif encounter:GetPrimaryCast() then
        self:AssignCastMember( "primary", encounter:GetPrimaryCast() )
    end
    self:AssignCastMember( "player", TheGame:GetGameState():GetPlayerAgent() )
    self:AssignCastMember( "location", self.location )
    if encounter:GetLeftAgent() then
        self:AssignCastMember( "left", encounter:GetLeftAgent() )
    end
    if encounter:GetRightAgent() then
        self:AssignCastMember( "right", encounter:GetRightAgent() )
    end
end

function BaseHub:RunHub()
    error( "Missing implementation" )
end

function BaseHub:OnStopConvo()
end

function BaseHub:TryValidateHub( ... )
    local ok, result, reason = xpcall( self.ValidateHub, generic_error, self, ... )
    if ok then
        return result, reason
    else
        LOGWARN( "%s\n%s", tostring(self), tostring(result) )
    end
end


function BaseHub:GetLocString(id, ...)

    if id == nil then
        return ""
    end
    
    local loc_str = GetLocalizedConvoCommonString(id)
     
    if loc_str == nil then
        loc_str = loc.format( id, ... )
    else
        loc_str = loc.format( loc_str, ... )
    end
    loc_str = self.enc:LocFormat( loc_str )

    return loc_str
end

function BaseHub:ReassignCastMember( name, cast_member )
    if self.cast[ name ] then
        self:UnassignCastMember( name )
    end
    if cast_member then
        self:AssignCastMember( name, cast_member )
    end
end

function BaseHub:AssignCastMember( name, cast_member )
    name = name:lower()
    assert_warning( cast_member ~= nil, "Casting nil cast member" )
    assert_warning( self.cast[ name ] == nil, "Redundant cast assignment" )
    self.cast[ name ] = cast_member
end

function BaseHub:UnassignCastMember( name )
    name = name:lower()
    assert( self.cast[ name ] ~= nil ) -- wha? no cast
    self.cast[ name ] = nil
end

function BaseHub:GetCastAgent( cast_id )
    local cast_member = self:GetCastMember( cast_id )
    if is_instance( cast_member, Agent ) then
        return cast_member
    end
end

function BaseHub:GetCastMember( cast_id )
    cast_id = cast_id:lower()
    return self.cast[ cast_id ]
end

function BaseHub:LookupQuip( tags, seed )
    return nil
end

function BaseHub:SubstituteFormatter( text )
    local cast_member = self:GetCastMember( text )
    if cast_member then
        return cast_member
    end
end
