local ConvoUtil = class( "ConvoUtil" )
ConvoUtil.can_reload = true

function ConvoUtil.Leave(cxt)
    cxt.enc:WaitOnLine()

    local exit = cxt.location and cxt.location:GetDefaultExit()
    if exit and exit:GetTo() then
        cxt:End()
        cxt.encounter:DoLocationTransition( exit:GetTo() )
    else
        cxt:Travel()
    end
end

function ConvoUtil.DoHealthDelta(cxt, delta, animate)
    cxt.enc:GetScreen():ForceWaitOnLine()
    delta = math.floor(delta)
    if delta > 0 then
        local health_gained = TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):Delta(delta)
        if health_gained > 0 then
            if animate ~= false then 
                AUDIO:PlayEvent("event:/ui/char/gen/gain_health")
                cxt.enc:GetScreen():FireHealthOverlay( 0x5dee5faf )
                cxt:Dialog("DIALOG_SHOW_HEALING", health_gained)
            end
        end
    else
        local health_lost = TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):Delta(delta)
        if animate ~= false then 
            AUDIO:PlayEvent("event:/sfx/general/conversation/damage_taken_gen")
            cxt.enc:GetScreen():FireHealthOverlay( 0xee5d5faf ) -- punch to red
            cxt:Dialog("DIALOG_SHOW_HEALTH_LOSS", math.abs( delta ) )
        end
    end
end

function ConvoUtil.DoResolveDelta(cxt, delta, animate)
    cxt.enc:GetScreen():ForceWaitOnLine()
    delta = math.floor(delta)
    if delta > 0 then
        local resolve_gained = TheGame:GetGameState():GetCaravan():DeltaResolve( delta )
        if resolve_gained > 0 then
            if animate ~= false then 
                AUDIO:PlayEvent("event:/ui/char/gen/gain_resolve")
                cxt.enc:GetScreen():FireHealthOverlay( 0x09E1DCaf )
                cxt:Dialog("DIALOG_SHOW_RESOLVE_GAIN", resolve_gained)
            end
        end
    else
        local resolve_lost = TheGame:GetGameState():GetCaravan():DeltaResolve( delta )
        if resolve_lost ~= 0 then
            if animate ~= false then 
                AUDIO:PlayEvent(SoundEvents.battle_impact_dull_fur)
                cxt.enc:GetScreen():FireHealthOverlay( 0xE109DCaf ) -- punch to red
                cxt:Dialog("DIALOG_SHOW_RESOLVE_LOSS", math.abs(resolve_lost))
            end
        end
    end
end


--------------------------------------
-- Utility queries for Hub and Attract

function ConvoUtil.DoFireEmployee(cxt, employee)
    cxt:Opt("OPT_FIRE_TARGET", employee )
        :SetPortrait(employee)
        :Dialog("DIALOG_WHY_FIRE")
        :Negotiation{
            acting_against = {employee},
            subject = employee,
            on_fail = function(cxt) 
                cxt:Dialog("DIALOG_NO_FIRE")
            end,
            on_success = function() 
                
                if employee:GetLocation() == cxt.location then
                    cxt:Dialog("DIALOG_DO_FIRING_HERE")
                    employee:GetBrain():MoveToHome()
                else
                    cxt:Dialog("DIALOG_DO_FIRING_REMOTE")
                end
                employee:OpinionEvent(OPINION.GOT_FIRED)
                employee:GetBrain():GetWorkPosition():Fire()
            end,
        }    
end


function ConvoUtil.SpawnNewAgent( cxt, def_id, quest_cast_id )
    local new_spawn = TheGame:GetGameState():AddSkinnedAgent( def_id )
    new_spawn:MoveToLocation(cxt.location)
    cxt.enc:SetPrimaryCast(new_spawn)
    if quest_cast_id then
        cxt.quest:AssignCastMember(quest_cast_id, new_spawn)
    end
    return new_spawn
end


--should we just use quips? I hate quips that are multi-line.
function ConvoUtil.RelationshipDialog( cxt, agent, neutral, friendly, unfriendly)
    if agent:GetRelationship() < RELATIONSHIP.NEUTRAL and unfriendly then
        cxt:Dialog(unfriendly)
    elseif agent:GetRelationship() > RELATIONSHIP.NEUTRAL and friendly then
        cxt:Dialog(friendly)
    elseif neutral then
        cxt:Dialog(neutral)
    end
end


function ConvoUtil.CollectSurvivors(agents, move_to_limbo)
    local survivors, killed = {}, {}
    for k,v in ipairs(agents) do
        if v:IsDead() then
            if v:IsSentient() then
                table.insert(killed, v)
            end
        else
            if v:IsSentient() then
                table.insert(survivors, v)
            end
            
            if move_to_limbo then
                v:MoveToLimbo()
            end
        end
    end
    return survivors, killed
end

function ConvoOption:DoDrink( cost, resolve_gain, agent )
    local drink_effects = {
        cards = { "drunk_player", "drunk" },
        resolve_gain = resolve_gain
    }

    TheGame:GetEvents():BroadcastEvent( "do_drink", drink_effects ) 
 
    self:PreIcon( global_images.drink )
    self:PostText("OPT_FORCE_GAIN_CARDS", drink_effects.cards )
    for i, card_id in ipairs( drink_effects.cards ) do
        self:PostCard( card_id, true )
    end
    if drink_effects.resolve_gain > 0 then
        self:DeltaResolve( drink_effects.resolve_gain )
    end
    if drink_effects.health_gain and drink_effects.health_gain > 0 then
        self:DeltaHealth( drink_effects.health_gain, true )
    end
    if cost and cost > 0 then
        self:DeliverMoney( cost, {is_shop = true}, self.hub.location:GetProprietor() )
    end

    self:Fn( function( cxt )
        if #drink_effects.cards > 0 then
            cxt:ForceTakeCards( drink_effects.cards )
        end
        if agent then
            AgentUtil.MakeDrunk( agent )
        end
        TheGame:GetEvents():BroadcastEvent( "had_drink", drink_effects ) 
    end )

    return self
end

function ConvoOption:DoEat( cost, health_gain )
    local eat_effects = {
        cards = { "bloated" },
        health_gain = health_gain
    }

    TheGame:GetEvents():BroadcastEvent( "do_eat", eat_effects ) 
 
    self:PostText("OPT_FORCE_GAIN_CARDS", eat_effects.cards )
    for i, card_id in ipairs( eat_effects.cards ) do
        self:PostCard( card_id, true )
    end
    if eat_effects.resolve_gain and eat_effects.resolve_gain > 0 then
        self:DeltaResolve( eat_effects.resolve_gain )
    end
    if eat_effects.health_gain and eat_effects.health_gain > 0 then
        self:DeltaHealth( eat_effects.health_gain, true )
    end
    if cost and cost > 0 then
        self:DeliverMoney( cost, {is_shop = true}, self.hub.location:GetProprietor() )
    end
    local health_percent = TheGame:GetGameState():GetPlayerAgent():GetAspect("health"):GetPercent()
    self:ReqCondition(health_percent < 1, "REQ_AT_FULL_HEALTH")

    local num_bloat = TheGame:GetGameState():GetPlayerAgent().battler:GetCardCount("bloated")
    self:ReqCondition(num_bloat < 3, "REQ_TOO_FULL", "bloated")

    self:Fn( function( cxt )
        if #eat_effects.cards > 0 then
            cxt:ForceTakeCards( eat_effects.cards )
        end
        TheGame:GetEvents():BroadcastEvent( "did_eat", eat_effects ) 
    end )

    return self
end

function ConvoUtil.DoSleep(cxt, post_dialog, post_fn)

    local json_t = {
        PLAYER_DATA = TheGame:GetGameState():GetPlayerState(),
    }
    SendMetricsData("DAY_END", json_t )

    cxt.enc:GetScreen():ForceWaitOnLine()
    cxt.enc:GetScreen():ClearHistory()
    cxt.enc:GetScreen():SetBlur(false)

    cxt:RunFn(function() 

        if not cxt.player.graft_owner:GetGraft( "loyalty" ) then
            local leaving = {}
            for _, member in TheGame:GetGameState():GetCaravan():Members() do
                if member:IsHiredMember() then
                    if (member.aspects.party_member.battles_fought or 0) > 0 then
                        table.insert(leaving, member)
                    end
                end
            end
            for k,v in ipairs(leaving) do
                cxt:TalkTo(v)
                cxt:Dialog("DIALOG_HIRED_LEAVE_AT_NIGHT")
                v:Dismiss()
                v:MoveToLimbo()
            end
        end

        cxt.enc:SetPrimaryCast()
        -- Show the default dream
        cxt:Dialog("DIALOG_REST_START")
        

        --this is not great, but it's really late in the project, so...
        local extra_tags = {}
        if cxt.player:GetContentID() == "SMITH" then
            local theroux = TheGame:GetGameState():GetAgentOrMemento( "THEROUX" )
            local mullifee = TheGame:GetGameState():GetAgentOrMemento( "MULLIFEE" )
            if theroux then
                if theroux:IsAlive() then
                    table.insert(extra_tags, "theroux_alive")
                elseif theroux:IsDead() then
                    table.insert(extra_tags, "theroux_dead")
                end
            end
            if mullifee then
                if mullifee:IsAlive() then
                    table.insert(extra_tags, "mullifee_alive")
                elseif mullifee:IsDead() then
                    table.insert(extra_tags, "mullifee_dead")
                end
            end
        end


        cxt:Quip(nil, "go_to_sleep", cxt.player:GetContentID(), loc.format("day{1}", TheGame:GetGameState():GetActProgress() or 1), table.unpack(extra_tags))

        cxt:Opt("OPT_SLEEP")
            :PreIcon( global_images.sleep )
            :Fn(function()

                local player = TheGame:GetGameState():GetPlayerAgent()
                local hp, maxhp = player:GetHealth()
                local resolve, maxresolve = TheGame:GetGameState():GetCaravan():GetResolve()
                local initial_state =
                {
                    health = hp,
                    max_health = maxhp,
                    resolve = resolve,
                    max_resolve = maxresolve
                }

                TheGame:GetGameState():DoPlayerRest()
                cxt.enc:GetScreen():ForceWaitOnLine()
                cxt.enc:GetScreen():ClearHistory()

                -- Handle Health/Resolve restoring
                local HEALTH_SLEEP_HEAL_RATE = GetAdvancementModifier( ADVANCEMENT_OPTION.SLEEP_HEALTH_RESTORE ) or 1
                local RESOLVE_SLEEP_HEAL_RATE = GetAdvancementModifier( ADVANCEMENT_OPTION.SLEEP_RESOLVE_RESTORE ) or 1

                local sleep_data =
                {
                    health_gain = math.round(maxhp*HEALTH_SLEEP_HEAL_RATE),
                    resolve_gain = math.round(maxresolve*RESOLVE_SLEEP_HEAL_RATE),
                    cards_removed = {},
                }
                TheGame:BroadcastEvent( "do_sleep", player, sleep_data )

                ConvoUtil.DoHealthDelta(cxt, sleep_data.health_gain, false)
                ConvoUtil.DoResolveDelta(cxt, sleep_data.resolve_gain, false)
                for i, card in ipairs( sleep_data.cards_removed ) do
                    -- could do some UI here.
                    if is_instance( card, Battle.Card ) then
                        player.battler:RemoveCard( card )
                    else
                        player.negotiator:RemoveCard( card )
                    end
                end
                
                -- Instantiate the screen here so it can checkpoint data before sleep
                -- bonuses are applied.                
                local function OnDone( ... )
                    cxt.enc:ResumeEncounter( ... )
                end
                TheGame:FE():PushScreen( Screen.DayOverScreen( initial_state, sleep_data, OnDone ))
                cxt.enc:YieldEncounter()
                
                UIHelpers.PassTime(DAY_PHASE.DAY)

                for i,member in player:GetParty():Members() do
                    if not member:IsPlayer() then
                        member:GetAspect("health"):Restore(1)
                    end
                end

                if post_dialog then
                    cxt:Dialog( post_dialog )
                end

                ConvoUtil.DoMorningMail(cxt)

                if post_fn then
                    post_fn()
                end

        end)
    end)
end

function ConvoUtil.DoMorningMail(cxt)
    local player_agent = TheGame:GetGameState():GetPlayerAgent()

    cxt:RunLoop(function(cxt)
        TheGame:GetEvents():BroadcastEvent( "morning_mail", cxt ) 
        if cxt.enc:CountOptions() > 0 then
            cxt:Opt("OPT_DONE")
                :MakeUnder()
                :Pop()
        else
            cxt:Pop()
        end
    end)
end




function ConvoUtil.GiveGraft(cxt, graft)
    local installed = cxt.enc:TryInstallGraft(graft)
    if installed then
        cxt:Dialog("DIALOG_PLAYER_INSTALL_GRAFT", graft)
    end
    return installed
end


function ConvoUtil.GiveGraftChoice(cxt, grafts)
    local opt 
    if grafts and #grafts > 0 then
        opt = cxt:Opt("OPT_GET_GRAFT")
            :PreIcon( global_images.receiving )
            :Fn(function() 
                
                local popup = Screen.PickGraftScreen(grafts, false, function(...) cxt.enc:ResumeEncounter(...) end)
                TheGame:FE():InsertScreen( popup )

                local chosen_graft = cxt.enc:YieldEncounter()
                if chosen_graft then
                    cxt:Dialog("DIALOG_PLAYER_INSTALL_GRAFT", chosen_graft)
                    return chosen_graft
                end
            end)
    end
    return opt
end

function ConvoUtil.GiveReward(cxt, rank, extra_reward, extra_reward_data)
    cxt.enc:WaitOnLine()

    local caravan = TheGame:GetGameState():GetCaravan()

    if extra_reward == EXTRA_QUEST_REWARD.GRAFT_CHOICE then 
        local grafts = RewardUtil.GetGrafts( rank, TheGame:GetGameState():GetGraftDraftDetails().count )
        ConvoUtil.GiveGraftChoice(cxt, grafts)
    
    elseif extra_reward == EXTRA_QUEST_REWARD.CARD_REMOVE then
        cxt:Dialog("DIALOG_GAIN_REMOVAL_BONUS")
        cxt:RunLoop(function()

            cxt:Opt("OPT_REMOVE_NEGOTIATION_CARD")
                :PreIcon( global_images.removenegotiation )
                :Fn( function( cxt )
                    cxt:Wait()
                    AgentUtil.RemoveNegotiationCard( cxt.player, function( card )
                        cxt.enc:ResumeEncounter( card )
                    end)
                    
                    local card = cxt.enc:YieldEncounter()
                    if card then
                        cxt:Pop()
                    end
                end)
            cxt:Opt("OPT_REMOVE_BATTLE_CARD")
                :PreIcon( global_images.removecombat )
                :Fn( function( cxt )
                    cxt:Wait()
                    AgentUtil.RemoveBattleCard( cxt.player, function( card )
                        cxt.enc:ResumeEncounter( card )
                    end)
                    
                    local card = cxt.enc:YieldEncounter()
                    if card then
                        cxt:Pop()
                    end
                end)

            cxt:Opt("OPT_SKIP_BONUS")
                :MakeUnder()
                :Pop()
        end)

    elseif extra_reward == EXTRA_QUEST_REWARD.CARD_UPGRADE then
        cxt:Dialog("DIALOG_CARD_UPGRADE_BONUS")
        cxt:RunLoop(function()

            cxt:Opt("OPT_UPGRADE_NEGOTIATION_CARD")
                :PreIcon( global_images.upgradenegotiation )
                :Fn( function( cxt )
                    cxt:Wait()
                    AgentUtil.UpgradeNegotiationCard(function( card )
                        cxt.enc:ResumeEncounter( card )
                    end)
                    
                    local card = cxt.enc:YieldEncounter()
                    if card then
                        cxt:Pop()
                    end
                end)
            cxt:Opt("OPT_UPGRADE_BATTLE_CARD")
                :PreIcon( global_images.upgradecombat )
                :Fn( function( cxt )
                    cxt:Wait()
                    AgentUtil.UpgradeBattleCard(function( card )
                        cxt.enc:ResumeEncounter( card )
                    end)
                    
                    local card = cxt.enc:YieldEncounter()
                    if card then
                        cxt:Pop()
                    end
                end)

            cxt:Opt("OPT_SKIP_BONUS")
                :MakeUnder()
                :Pop()
        end)            
    
    elseif extra_reward == EXTRA_QUEST_REWARD.FREE_ITEM then
        cxt:Dialog("DIALOG_GAIN_ITEM_BONUS", extra_reward_data)
        
        cxt:RunLoop(function()
            local opt
            
            cxt:Opt(Content.GetNegotiationCard( extra_reward_data ) and "OPT_FORCE_GAIN_NEGOTIATION_CARD" or "OPT_FORCE_GAIN_BATTLE_CARD", extra_reward_data)
                    :PreIcon( Content.GetNegotiationCard( extra_reward_data ) and global_images.buynegotiation or global_images.buycombat )
                :GainCards{extra_reward_data}
                :Pop()
            
            cxt:Opt("OPT_SKIP_BONUS")
                :MakeUnder()
                :Pop()
        end)

    elseif extra_reward == EXTRA_QUEST_REWARD.EXTRA_CASH then

        cxt.encounter:GainMoney( extra_reward_data )
        cxt:Dialog("DIALOG_GAIN_MONEY_BONUS", extra_reward_data)

    elseif extra_reward == EXTRA_QUEST_REWARD.RESTORATION then
        cxt:Dialog("DIALOG_RESTORATION_BONUS")
        
        cxt:RunLoop(function()
            local opt
            
            local current_health, max_health = TheGame:GetGameState():GetPlayerAgent():GetHealth()
            cxt:Opt("OPT_RESTORE_HEALTH_BONUS")
                :PreIcon(global_images.heal)
                :DeltaHealth(2*extra_reward_data)
                :Pop()
            
            local current_resolve, max_resolve = TheGame:GetGameState():GetCaravan():GetResolve()
            cxt:Opt("OPT_RESTORE_RESOLVE_BONUS")
                :PreIcon(global_images.restoreresolve)
                :ReqCondition(current_resolve < max_resolve, "REQ_AT_FULL_RESOLVE")
                :DeltaResolve(extra_reward_data)
                :Pop()

            cxt:Opt("OPT_SKIP_BONUS")
                :MakeUnder()
                :Pop()
        end)
    elseif extra_reward == EXTRA_QUEST_REWARD.METTLE_POINTS then
        cxt:Dialog("DIALOG_METTLE_BONUS", extra_reward_data)

        cxt:Opt("OPT_ACCEPT_METTLE")
            :Fn(function(cxt)
                local character_id = cxt.player:GetContentID()
                TheGame:GetGameProfile():AddMettlePoints( character_id, extra_reward_data, "QUEST_BONUS" )
            end)
    end
end

function ConvoUtil.GiveQuestRewards(cxt, quest, dialog_id)
    quest = quest or cxt.quest
    if quest.rewarded then
        return
    end

    cxt:Run(function() 
        quest.rewarded = true
        cxt.enc:WaitOnLine()

        local caravan = TheGame:GetGameState():GetCaravan()
        
        local reward = quest:GetParameter("rewards")

        if reward and reward > 0 then
            caravan:AddMoney( reward )
            cxt:Dialog(dialog_id or "DIALOG_RECEIVED_MONEY_QUEST", reward)
        end

        
        local mettle_reward = quest:GetMettleReward()
        if mettle_reward and mettle_reward > 0 then
            cxt:Dialog("DIALOG_RECEIVED_METTLE_QUEST", mettle_reward)
            TheGame:GetGameProfile():AddMettlePoints( TheGame:GetGameState():GetPlayerAgent():GetContentID(), mettle_reward, "REWARD_METTLE_QUEST") 
        end

        local extra_reward, extra_reward_data = quest:GetExtraReward()
        ConvoUtil.GiveReward( cxt, quest:GetRank(), extra_reward, extra_reward_data )


    end)
end

function ConvoUtil.GiveBossRewards(cxt, graft_type)
    cxt:Run(function() 
        cxt.enc:WaitOnLine()

        local grafts = RewardUtil.GetBossGrafts(TheGame:GetGameState():GetGraftDraftDetails().count, TheGame:GetGameState():GetPlayerAgent(), graft_type)
        if #grafts > 0 then
            cxt:Opt("OPT_GET_GRAFT")
                :PreIcon( global_images.receiving )
                :Fn(function() 
                    local popup = Screen.PickGraftScreen(grafts, false, function(...) cxt.enc:ResumeEncounter(...) end, false, BOSS_GRAFT_SKIP_REWARD)
                    TheGame:FE():InsertScreen( popup )
                    local chosen_graft = cxt.enc:YieldEncounter()
                    if chosen_graft then
                        cxt:Dialog("DIALOG_PLAYER_INSTALL_GRAFT", chosen_graft)
                        return chosen_graft
                    end
                end)
        end
    end)
end


function ConvoUtil.GiveGraftDraft(cxt, rarity)

    local used_graft_ids = {}

    local owner = TheGame:GetGameState():GetPlayerAgent()
    local collection = GraftCollection.Rewardable(owner):Rarity(rarity)
    local grafts = collection:Generate(TheGame:GetGameState():GetGraftDraftDetails().count)

    cxt:Run(function() 
        cxt.enc:WaitOnLine()
        if #grafts > 0 then
            cxt:Opt("OPT_GET_GRAFT")
                :PreIcon( global_images.receiving )
                :Fn(function() 
                    local popup = Screen.PickGraftScreen(grafts, false, function(...) cxt.enc:ResumeEncounter(...) end)
                    TheGame:FE():InsertScreen( popup )
                    local chosen_graft = cxt.enc:YieldEncounter()
                    if chosen_graft then
                        cxt:Dialog("DIALOG_PLAYER_INSTALL_GRAFT", chosen_graft)
                        return chosen_graft
                    end
                end)
        end
    end)

end

function ConvoUtil.CanInterruptTravel(cxt)
    for k, quest in pairs( TheGame:GetGameState().available_quests ) do
        if quest:PreventsRandomEvents() then
            return false
        end
    end

    if cxt.caravan:HasEscort() then
        return false
    end
    
    if cxt.location:HasTag("in_transit") then
        return true
    end
end

function ConvoUtil.OptBuyGraftSlot( cxt, graft_type, dialog_id, slot_health, base_cost )
    local max_grafts = cxt.player.graft_owner:GetMaxGrafts( graft_type )
    
    local SLOT_COST = (base_cost or 100) + math.max( 0, (max_grafts - 3) * 50 )
    local MAX_GRAFTS = 6 -- Per character?

    if slot_health == nil then
        slot_health = 15
    end

    local opt = cxt:Opt( graft_type == GRAFT_TYPE.NEGOTIATION and "OPT_ADD_NEGOTIATION_SLOT" or "OPT_ADD_COMBAT_SLOT" )
        :Dialog( dialog_id )
        :Fn(function() cxt.player.graft_owner:IncreaseMaxGrafts( graft_type, 1 ) end)
        :DeliverMoney(SLOT_COST, {is_shop = true})
        :ReqCondition(max_grafts < MAX_GRAFTS, "REQ_TOO_MANY_SLOTS")
        :DeltaHealth(-slot_health)

    return opt
end
