require "encounter/base_hub"

--------------------------------------------------------------------------------
-- Poses a default HUB by collecting relevant quests/conversations and injecting
-- them as options.

local DefaultHub = class( "DefaultHub", BaseHub )

function DefaultHub:init( encounter, attract_disabled, start_hub_state )
    BaseHub.init( self, encounter )
    self.hub_contexts = {}
    self.attract_contexts = {}
    self.attract_disabled = attract_disabled
    self.hub_state = start_hub_state or HUB_STATE.HUB
end

function DefaultHub:RunHub( enc, state )
    self:Attract( enc, state )
    self:Hub( enc )
end

function DefaultHub:Attract( enc, state )
    assert( state == nil )
    if self.attract_disabled then
        return
    end

    table.clear( self.attract_contexts )
    enc:CollectHubs( self.attract_contexts, HUB_STATE.CONFRONT )
    if enc:RunHubs( self.attract_contexts, HUB_STATE.CONFRONT ) then
        return
    end

    enc:PresentDefault()

    table.clear( self.attract_contexts )
    enc:CollectHubs( self.attract_contexts, HUB_STATE.ATTRACT )

    if not enc:RunHubs( self.attract_contexts, HUB_STATE.ATTRACT ) then
        local agent = enc:GetAgent()
        if agent then
            enc:SayQuip( agent, "greeting" )
        end
    end
end

function DefaultHub:Hub( enc )
    while not enc:IsEnded() do
        
        if enc:GetPrimaryCast() and enc:GetPrimaryCast():IsDead() then
            break
        end

        enc:ClearOptions()

        local leave_opt = enc:AddHubOptions( CONVERSATION_FLAGS.ROOT | CONVERSATION_FLAGS.BUSINESS, self.hub_state )

        --this is a special case for when hubs do execution stuff, and then end the encounter. if this isn't here, we end up having to click an extra end button.
        if enc:IsEnded() then
            break
        end

        local opt = enc:ChooseOption()
        if opt == nil then
            break
        else
            opt:RunOption()
            if opt == leave_opt then
                break
            end
        end
    end
end

function DefaultHub:RenderDebugPanel( ui, panel )
    if ui:TreeNode( string.format( "attract_contexts [%d]", #self.attract_contexts ) ) then
        for i, context in ipairs( self.attract_contexts ) do
            panel:AppendTable( ui, context )
        end
        ui:TreePop()
    end

    if ui:TreeNode( string.format( "hub_contexts [%d]", #self.hub_contexts ) ) then
        for i, context in ipairs( self.hub_contexts ) do
            panel:AppendTable( ui, context )
        end
        ui:TreePop()
    end
end

function DefaultHub:__tostring()
    return loc.format( "DefaultHub {1}", self.hub_state)
end
