local StateGraphUtil = class( "StateGraphUtil" )
StateGraphUtil.can_reload = true

function StateGraphUtil.AddBackButton(cxt)
    return cxt:Opt("OPT_BACK_BUTTON"):Pop():MakeUnder()
end

function StateGraphUtil.AddLeaveLocation(cxt, str_id)
    return cxt:Opt(str_id or "OPT_LEAVE_LOCATION"):Fn(function(cxt) ConvoUtil.Leave(cxt) end):MakeUnder()
end


function StateGraphUtil.AddEndOption(cxt, str_id)
    return cxt:Opt(str_id or "OPT_DONE"):SetSFX( SoundEvents.leave_conversation ):Fn(function(cxt) cxt:End() end):MakeUnder()
end

function StateGraphUtil.AddRemoveBattleCardOption( cxt, dialog_id, fn )
    local remove_data =
    {
        price = cxt.player.battler:GetRemovePrice(),
        agent = cxt:GetAgent(),
        card_class = Battle.Card,
    }
    TheGame:BroadcastEvent( "pre_remove_card", remove_data )

    return cxt:Opt( "OPT_REMOVE_BATTLE_CARD" )
        :Dialog( dialog_id )
        :PreIcon( global_images.removecombat )
        :ReqTimer( "BATTLE_CARD_REMOVED", 1 )
        :RequireMoney( remove_data.price, { is_shop = true } )
        :Fn( function( cxt )
                cxt:Wait()
                AgentUtil.RemoveBattleCard( cxt.player, function( card_id )
                    if card_id then
                        cxt.caravan:PayMoney( CalculatePayment(cxt:GetAgent(), remove_data.price, 1, {is_shop = true}), cxt:GetAgent() )
                        cxt.player.battler:IncPaidRemovals()
                        cxt:GetAgent():Remember("BATTLE_CARD_REMOVED")
                        if fn then
                            fn()
                        end
                        
                    end
                    cxt.enc:ResumeEncounter()
                end)
            cxt.enc:YieldEncounter()
        end)
end

function StateGraphUtil.AddRemoveNegotiationCardOption( cxt, dialog_id, fn )
    local remove_data =
    {
        price = cxt.player.negotiator:GetRemovePrice(),
        agent = cxt:GetAgent(),
        card_class = Negotiation.Card,
    }
    TheGame:BroadcastEvent( "pre_remove_card", remove_data )

    return cxt:Opt( "OPT_REMOVE_NEGOTIATION_CARD" )
        :Dialog( dialog_id )
        :PreIcon( global_images.removenegotiation )
        :ReqTimer( "NEGOTIATION_CARD_REMOVED", 1 )
        :RequireMoney( remove_data.price, { is_shop = true } )
        :Fn( function( cxt )
            cxt:Wait()
            AgentUtil.RemoveNegotiationCard( cxt.player, function( card )
                if card then
                    cxt.caravan:PayMoney( CalculatePayment(cxt:GetAgent(), remove_data.price, 1, {is_shop=true}), cxt:GetAgent() )
                    cxt.player.negotiator:IncPaidRemovals()
                    cxt:GetAgent():Remember("NEGOTIATION_CARD_REMOVED")
                    if fn then
                        fn()
                    end

                end
                cxt.enc:ResumeEncounter()
            end)
            cxt.enc:YieldEncounter()
        end )
end

function StateGraphUtil.PresentQuestOffer( cxt, quest, objective_id, accept_fn, decline_fn, must_accept )

    if not quest:IsValid() then
        -- ccc: DRAGONS.
        -- The quest must be activated before QUEST_CONVO_HOOK.INTRO, because it may reference agents
        -- that need to be in the GameState for lookup.  Now, it so happens that side jobs are already
        -- activated before accepting (so whatever), and handler jobs can't be declined (so spawning is fine)
        quest:Activate()
        TheGame:GetGameState():AddActiveQuest( quest )
    end

     cxt:PlayQuestConvo( quest, QUEST_CONVO_HOOK.INTRO )
   
    local accepted = cxt.encounter:ShowJobOffer(quest, objective_id, must_accept)
    
    if cxt.enc:GetScreen() then
        cxt.enc:GetScreen():ClearHistory()
    end
    cxt.enc:Wait(.5)

    if accepted then
        if accept_fn then
            accept_fn(cxt)
        else
            cxt:Quip( cxt:GetAgent(), "job_accept")

            if cxt:GetAgent() == quest:GetProvider() then
                cxt:GetAgent():OpinionEvent(OPINION.TOOK_JOB)
            end

            cxt:Pop()
        end

    else
        if decline_fn then
            decline_fn(cxt)
        else
            cxt:Quip( cxt:GetAgent(), "job_decline")
            cxt:Pop()
        end
    end

    cxt:Wait()
end

function StateGraphUtil.DoRunAway( cxt, battle, no_fail )

    cxt:End()

    local params = { convo_id = "RUN_AWAY_CHAT", battle = battle, no_fail = no_fail }
    params.secret = CheckBits( battle:GetScenario():GetFlags(), BATTLE_FLAGS.ISOLATED ) or not battle:GetScenario():GetPrimary(TEAM.RED):IsSentient()
    params.state_id = params.secret and "STATE_RAN_SECRET" or "STATE_RAN"
    TheGame:BroadcastEvent( "on_run_away", params )

    if params.convo_id then
        UIHelpers.DoSpecificConvo( nil, params.convo_id, params.state_id, nil, params, cxt.quest )
    end
end

function StateGraphUtil.DoRunAwayNoFail( cxt, battle )
    StateGraphUtil.DoRunAway( cxt, battle, true )
end

function StateGraphUtil.DoRunAwayEffects( cxt, battle, no_fail )
    if not no_fail and cxt.quest and cxt.quest:IsActive() then
        cxt.quest:Fail()
    end

    -- If the player fled, allies don't like them.
    for i, fighter in battle:GetPlayerTeam():Fighters() do
        local agent = fighter.agent
        if not agent:IsPlayer() then
            fighter.agent:OpinionEvent( OPINION.ABANDONED )
        end
    end

    -- .. Your followers leave you...
    cxt.caravan:RemoveFollowers()

    -- Runaway to a locatiom
    if TheGame:GetGameState():GetGameType() == GAME_TYPE.CAMPAIGN then
        local runaway_loc = TheGame:GetGameState():AddLocation( Location( TheGame:GetGameState():GetWorldRegion():GetOutdoorLocationID() ) )
        runaway_loc:SetFlags( LOCFLAGS.TRANSIENT )
        cxt.encounter:DoLocationTransition( runaway_loc )
    end
    
    -- You might be a coward, if someone saw.
    local no_one_knows = CheckBits( battle:GetScenario():GetFlags(), BATTLE_FLAGS.ISOLATED ) or not battle:GetScenario():GetPrimary(TEAM.RED):IsSentient()
    if not no_one_knows then
        cxt:GainCards{"coward"}
    end
end

-----------------------------------------------------------------------
-- Common chum bonus implementations

function StateGraphUtil.OfferSpareChange( amount )
    return function( cxt )
        cxt:Opt( "OFFER_SPARE_CHANGE", amount ):PreIcon( global_images.shills )
            :Quip( cxt:GetAgent(), "chum_bonus", "money")
            :ReceiveMoney( amount ):Pop()
    end
end

function StateGraphUtil.OfferUpgradeNegotiation()
    return function( cxt )
        cxt:Opt( "OFFER_NEGOTIATION_UPGRADE" ):PreIcon( global_images.upgradenegotiation )
        :Quip( cxt:GetAgent(), "chum_bonus", "negotiation" )
        :Fn( function( cxt )
            cxt:Wait()

            AgentUtil.UpgradeNegotiationCard( function( card )
                cxt.enc:ResumeEncounter( card )
            end )
            
            local card = cxt.enc:YieldEncounter()
            if card then
                cxt:Pop()
            end
        end )
    end
end

function StateGraphUtil.OfferUpgradeBattle()
    return function( cxt )
        cxt:Opt( "OFFER_BATTLE_UPGRADE" ):PreIcon( global_images.upgradecombat )
        :Quip( cxt:GetAgent(), "chum_bonus", "battle" )
        :Fn( function( cxt )
            cxt:Wait()

            AgentUtil.UpgradeBattleCard( function( card )
                cxt.enc:ResumeEncounter( card )
            end )
            
            local card = cxt.enc:YieldEncounter()
            if card then
                cxt:Pop()
            end
        end )
    end
end


function StateGraphUtil.OfferCompressionGear()
    return function( cxt )
       cxt:Opt( "OFFER_COMPRESSION_GEAR" )
       :PreIcon( global_images.health )
       :PostText( "OFFER_UPGRADE_HEALTH", 8 )
       :PostText( "OFFER_UPGRADE_RESOLVE", 2 )
        :Quip( cxt:GetAgent(), "chum_bonus", "compression_gear" )
       :Fn( function( cxt )
            cxt.caravan:UpgradeHealth( 8, "COMPRESSION_GEAR" )
            cxt.caravan:UpgradeResolve( 2 )
        end ):Pop()
    end
end

function StateGraphUtil.OfferCard( card_id )
    return function( cxt )
        cxt:Opt( "OFFER_CARD", card_id )
            :PreIcon( global_images.giving )
            :Quip( cxt:GetAgent(), "chum_bonus" )
            :GainCard(card_id)
            :Pop()
    end
end

-- function StateGraphUtil.OfferShrokianPepperBox()
--     return function( cxt )
--     cxt:Opt( "OFFER_PEPPER_BOX", "threaten", "feint" )
--         :PreIcon( global_images.giving )
--         :Quip( cxt:GetAgent(), "chum_bonus", "pepper_box" )
--         :Fn( function( cxt )
--             AgentUtil.LearnCard( "threaten" )
--             AgentUtil.LearnCard( "feint" )
--         end ):Pop()
--     end
-- end

function StateGraphUtil.OfferTriageKit()
    return function( cxt )
    cxt:Opt( "OFFER_TRIAGE_KIT", "healing_vapors" )
        :PreIcon( global_images.giving )
        :Quip( cxt:GetAgent(), "chum_bonus", "triage_kit" )
        :GainCards{"healing_vapors", "healing_vapors"}
        :Pop()
    end
end


function StateGraphUtil.OfferScoreCards( next_state )
    return function( cxt )
        local big_offer = {"big_score"}
        local little_offer = {"little_score"}
        
        cxt:Opt( "OFFER_CARDS", big_offer )
            :PreIcon( global_images.giving )
            :GainCards(big_offer)
            :GoTo(next_state)
        
        cxt:Opt( "OFFER_CARDS", little_offer )
            :PreIcon( global_images.giving )
            :GainCards(little_offer)
            :GoTo(next_state)
    end
end

function StateGraphUtil.OfferBattleDraft( count )
    return function( cxt )
        cxt:Opt( "OFFER_BATTLE_DRAFT", count )
            :PreIcon( global_images.buycombat )
            :Quip( cxt:GetAgent(), "chum_bonus", "battle" )
            :Fn( function( cxt )
                cxt:Wait()

                for i = 1, count or 1 do
                    local draft_popup = Screen.DraftChoicePopup()
                    local function OnDone()
                        cxt.encounter:ResumeEncounter()
                    end
                    local cards = RewardUtil.GetBattleCards( 1, 3, cxt.player )
                    draft_popup:DraftCards( cxt.player, Battle.Card, cards, OnDone )
                    TheGame:FE():InsertScreen( draft_popup )

                    cxt.enc:YieldEncounter()
                end

            end ):Pop()
    end
end

function StateGraphUtil.OfferNegotiationDraft( count )
    return function( cxt )
        cxt:Opt( "OFFER_NEGOTIATION_DRAFT", count )
            :PreIcon( global_images.buynegotiation )
            :Quip( cxt:GetAgent(), "chum_bonus", "negotiation" )
            :Fn( function( cxt )
                cxt:Wait()

                for i = 1, count or 1 do
                    local draft_popup = Screen.DraftChoicePopup()
                    local function OnDone()
                        cxt.encounter:ResumeEncounter()
                    end

                    local cards = RewardUtil.GetNegotiationCards( 1, 3, cxt.player )
                    draft_popup:DraftCards( cxt.player, Negotiation.Card, cards, OnDone )
                    TheGame:FE():InsertScreen( draft_popup )

                    cxt.enc:YieldEncounter()
                end

            end ):Pop()
    end
end

function StateGraphUtil.OfferBoomBox( count )

    return function( cxt )
        local function IsGrenade( def )
            return CheckBits(def.item_tags or 0, ITEM_TAGS.GRENADE)
        end

        cxt.enc.scratch.boombox_items = cxt.enc.scratch.boombox_items or {}
        if #cxt.enc.scratch.boombox_items == 0 then
            for i = 1, 2 do
                local def = BattleCardCollection.AllLocalItems( IsGrenade ):Pick(1)[1]
                if def then
                    table.insert( cxt.enc.scratch.boombox_items, def.id )
                end
            end
        end

        cxt:Opt( "OFFER_BOOM_BOX", cxt.enc.scratch.boombox_items )
            :PreIcon( global_images.giving )
            :Quip( cxt:GetAgent(), "chum_bonus", "boom_box" )
            :GainCards(cxt.enc.scratch.boombox_items)
            :Pop()
    end
end


function StateGraphUtil.OfferGraftSlot( graft_type, count )
    return function( cxt )
        local txt
        if graft_type == GRAFT_TYPE.COMBAT then
            txt = "OFFER_BATTLE_SLOT"
        elseif graft_type == GRAFT_TYPE.NEGOTIATION then
            txt = "OFFER_NEGOTIATION_SLOT"
        end

        cxt:Opt( txt )
            :PreIcon( global_images.graft )
            :Quip( cxt:GetAgent(), "chum_bonus", "graft" )
            :Fn( function( cxt )
                cxt.player.graft_owner:IncreaseMaxGrafts( graft_type, count )
            end ):Pop()
    end
end

function StateGraphUtil.ValidateChumBonuses( chum_bonuses )
    for i, bonus in ipairs( chum_bonuses ) do
        assert( type(bonus.fn) == "function", "invalid chum_bonus:"..tostr(bonus) )
    end
    return chum_bonuses
end

function StateGraphUtil.OfferChumBonuses(cxt, chum_bonuses)
    local bonuses
    if cxt.chum_bonuses then
        -- Already rolled.
        bonuses = cxt.chum_bonuses
    else
        bonuses = {}

        for i, bonus in ipairs( chum_bonuses or table.empty ) do
            table.insert( bonuses, bonus )
        end

        while #bonuses > 3 do
            table.remove( bonuses, cxt:SeededRandom( #bonuses ))
        end
        cxt.chum_bonuses = bonuses
    end

    assert( #bonuses > 0 )
    for i, bonus in ipairs( bonuses ) do
        bonus.fn( cxt )
    end
end


function StateGraphUtil.OfferSideJob(cxt, start_obj, dialog_offer, accept_fn, decline_fn)

    if cxt.quest and not cxt.quest:IsActive() then
        local agent = cxt:GetAgent()
        local opt = ConvoOption( loc.format(LOC"CONVO_OPTION.BASIC_OPPORTUNITY", cxt.quest:GetTitle() or "???", cxt.quest:GetRank())) 
        opt:SetQuestMark(cxt.quest, agent)
        cxt.enc:AddOption( opt )

        opt:Dialog(dialog_offer or "DIALOG_GENERIC_OFFER")
            :ReqCondition(QuestUtil.CanTakeSideJob(), "REQ_SIDE_JOB_LIMIT")
            -- :ReqRelationship(RELATIONSHIP.DISLIKED)
            :Fn(function(cxt) StateGraphUtil.PresentQuestOffer(cxt, cxt.quest, start_obj, accept_fn, decline_fn) end)

        return opt
    end
end


function StateGraphUtil.RestoreDaily( cxt )
    local encounter_state = TheGame:GetGameState():GetEncounterState()
    if encounter_state then
        assert_warning( encounter_state.convo_id == cxt:GetContentID(), "Restoring daily to wrong convo?? %s", encounter_state.convo_id )
        --
        print( "RESTORING DAILY TO:", cxt:GetStateID(), encounter_state.convo_id, encounter_state.state_id, tostr(encounter_state.scratch))

        if encounter_state.state_id == cxt:GetStateID() then
            -- This probably means we saved at the entry Confront, in which case we don't want to restore anything.
        else
            cxt.encounter.scratch = shallowcopy( encounter_state.scratch )
            cxt:GoTo( encounter_state.state_id )
            return true
        end
    end
end