function GetWallTime()
	return TheGame:GetWallTime()
end

function GetEpochDay(time)
    if time == nil then
        time = os.time()
    end
    time = time - (time % (60*60*24))
    local day = math.floor( time / (60*60* 24))
    return day, time
end

function Now()
    return TheGame and TheGame:GetGameState() and TheGame:GetGameState():GetDateTime() or 0
end

function DoQuickSave()
    local st = os.clock()
    print ("DO QUICK SAVE...")
	TheGame:SaveGame("quicksave")
    print( string.format( "\tQuicksave took %.2f ms", (os.clock() - st) * 1000 ))
end

function DoQuickLoad()
	print ("DO QUICK LOAD...")
	TheGame:LoadGame("quicksave", "autosave")
end

function DoAutoSave(fn)
    if TheGame:GetGameState().save_file_name then
        local st = os.clock()
        TheGame:SaveGame( TheGame:GetGameState().save_file_name, fn )
        --print( string.format( "\tAutosave took %.2f ms", (os.clock() - st) * 1000 ))
        return true
    end
end

function LOGWARN( ... )

    local str
    if select( "#", ... ) > 0 then
        str = debug.traceback( string.format( ... ), 2 )
    else
        str = debug.traceback( "failed assert", 2 )
    end

    print(str)
    if TheGame and TheGame:GetDebug() then
        engine.inst:ShowWarning( str )
    end
end

function assert_warning(assertion, ...)
    if not assertion then
        LOGWARN(...)
    end
end

function DrawDebugCircle( render, area, mapx, h, mapz, radius, r, g, b )
    local h0 = h
    local h1 = h0
    local subdivisions = math.max( 12, math.ceil( radius ))
    local dtheta = 2 * math.pi / subdivisions
    local theta = 0
    for i = 0, subdivisions do
        local x, y = radius * math.cos( theta ) + mapx, radius * math.sin ( theta ) + mapz
        theta = theta + dtheta
        local x1, y1 = radius * math.cos( theta ) + mapx, radius * math.sin ( theta ) + mapz
        render:DrawDebugLine( x, h0, y, x1, h1, y1, r or 1, g or 1, b or 1, 1 )
    end
end

function CalculatePayment( agent, base_value, denom, params )
    local reasons = {}
    
    if not (params and params.no_scale) then
        local m = {}
        local total_modifier = 0
        if agent then

            TheGame:GetEvents():BroadcastEvent( "calc_payment", agent, m, reasons, params)

            if m.generic_modifier then
                total_modifier = total_modifier + m.generic_modifier
            end

            local relationship = agent:GetRelationship()        
            m.rel_modifier = RELATIONSHIP_COST_MODIFIERS[ relationship ]
            
            if m.rel_modifier then
                TheGame:GetEvents():BroadcastEvent( "calc_relationship_discount",  m, reasons) 
                if m.rel_modifier < 0 then
                    table.insert(reasons, loc.format( LOC "UI.RELATIONSHIP_DISCOUNT", math.abs(m.rel_modifier), agent ))
                elseif m.rel_modifier > 0 then
                    table.insert(reasons, loc.format( LOC "UI.RELATIONSHIP_SURCHARGE", m.rel_modifier, agent ))
                end
                
                total_modifier = total_modifier + (base_value * m.rel_modifier)
            end
        end

        local adv_modifier = GetAdvancementModifier( ADVANCEMENT_OPTION.HIGHER_COSTS )
        if adv_modifier then
            total_modifier = total_modifier + (base_value * adv_modifier)
            local reason = LOC(GetAdvancementModifier( ADVANCEMENT_OPTION.HIGHER_COSTS_STR ))
            table.insert(reasons, loc.format( LOC "UI.PRESTIGE_SURCHARGE", reason))
        end

        base_value = math.max( 1, base_value + total_modifier)
    end

    if denom == nil then
        denom = math.max( 1, round_zero( base_value / 50, 5 )) -- a round number that scales with base_value
    end

    local new_price = math.max( 1, round_zero( base_value, denom ))

    return new_price, reasons
end

function CalculateReward( agent, base_value, denom )
    local reasons = {}
    if agent then
        local relationship = agent:GetRelationship()
        local m = RELATIONSHIP_COST_MODIFIERS[ relationship ]
        if m then
            if m > 0 then
                table.insert(reasons, loc.format( LOC "UI.RELATIONSHIP_PENALTY", math.abs(m) ))
            elseif m < 0 then
                table.insert(reasons, loc.format( LOC "UI.RELATIONSHIP_BONUS", m ))
            end
             base_value = base_value + base_value * m
        end

       
    end

    if denom == nil then
        denom = math.max( 1, round_zero( base_value / 50, 5 )) -- a round number that scales with base_value
    end

    return math.max( 1, round_zero( base_value, denom )), reasons
end

function GetAdvancementModifier( opt )
    if TheGame:GetGameState() then
        return TheGame:GetGameState():GetAdvancementModifier( opt )
    else
        return DEFAULT_ADVANCEMENT_OPTION[ opt ]
    end
end

function GetAdvancementTooltip( act, level )
    if (TheGame:GetGameState() or act) and level > 0 then
        local act = GetPlayerActData( act or TheGame:GetGameState().current_act )
        if act and act.data and act.background.data.advancement and act.background.data.advancement[level] then

            local tt = {
                loc.format( LOC(act.background.data.advancement[level].name) ),
            }
            for k, lvl in ipairs( act.background.data.advancement ) do
                local adv_data = act.background.data.advancement[k]
                if not adv_data.excluded or k == level then
                    table.insert( tt, LOC(adv_data.desc) )
                    if k == level then
                        return table.concat( tt, "\n" )
                    end
                end
            end
        end
    end
    return ""
end

function GetDailyDifficultyTooltip( act, level, difficulty )
    if TheGame:GetGameState() then
        local act = GetPlayerActData( TheGame:GetGameState().current_act )
        if act and act.data and act.background.data.advancement then
            local tt = {
                loc.format( "<#DAILY><b>{1}</></>" ,DAILY_DIFFICULTY_OPTIONS[difficulty].title()),
                LOC "ADVANCEMENT.ADVANCEMENT_LEVEL_0_TT",
            }
            for k, lvl in ipairs( act.background.data.advancement ) do
                local adv_data = act.background.data.advancement[k]
                if not adv_data.excluded then
                    table.insert( tt, LOC(adv_data.desc) )
                    if k == level then
                        return table.concat( tt, "\n" )
                    end
                end
            end
        end
    end
    return ""
end

function LookupTuningValue( key, default )
    local v
    local t = TUNING_CONSTANTS[ key ]
    if t then
        v = t.current_value
    end
    if v == nil then
        v = default
    end
    return v
end

function GetIsFastMode()
    if TheGame:GetLocalSettings().ROBOTICS then
        return true
    end
    return engine.inst:GetSetting("gameplay", "fast_mode", false) and true or false
end

function GetIsAutoSkip()
    if TheGame:GetLocalSettings().ROBOTICS then
        return true
    end
    return engine.inst:GetSetting("gameplay", "auto_skip", false) and true or false
end

function GetStreamerMode()
    -- 0 if disabled, 1 if the bottom left should be clear, 2 if the bottom right should be clear
    return engine.inst:GetSetting( "gameplay", "streamer_mode", 0 )
end

function GetSkipMovies()
    return engine.inst:GetSetting("gameplay", "skip_movies", false) and true or false
end

local Util = class( "Util" )

function Util.GetGraftPrice(id)
    local def = Content.GetGraft(id)
    local price = def and GRAFT_COSTS[def.rarity] or 999    
    return price
end

function Util.GetCardPrice(id)
    local def = Content.GetBattleCard(id) or Content.GetNegotiationCard(id)
    if not def then return 999 end

    local is_item = false

    if def.is_negotiation then
        local negotiation_defs = require "negotiation/negotiation_defs"
        is_item = CheckBits( def.flags, negotiation_defs.CARD_FLAGS.ITEM ) 
    else
        local battle_defs = require "battle/battle_defs"
        is_item = CheckBits( def.flags, battle_defs.CARD_FLAGS.ITEM ) 
    end

    local price = def.shop_price or is_item and ITEM_CARD_RARITY_VALUE[def.rarity] or CARD_RARITY_VALUE[def.rarity] or 999
    assert_warning(price ~= 999, def.name)
    return price
end


function Util.IsPlayerWorkingWithFaction(faction_id)
    local mem = TheGame:GetGameState():GetPlayerAgent():HasMemory("WORKING_WITH_FACTION")
    return mem and mem == faction_id
end

function Util.SetPlayerWorkingWithFaction(faction_id)
    TheGame:GetGameState():GetPlayerAgent():Remember("WORKING_WITH_FACTION", faction_id)
end
