
class("ImageProcessor")


function ImageProcessor:init( name )

    local renderer = engine.rendering.SceneRenderNode(name or "Image Processing Node")
    
    local model = engine.model.PlaneModel()
    model:SetSize( 2, 2 )
    model:SetXY()

    local node = engine.rendering.SceneGraphNode(model)
    local scene = engine.rendering.Scene()
    node:SetScene(scene)
    local cam = engine.rendering.CameraInfo()
    
    cam:SetOrtho()
    cam:SetUp( 1, 0, 0 )
    renderer:SetFlatScene(scene, node, cam)
    
    self.renderer = renderer
    self.model = model
    self.node = node
    self.cam = cam
    self.constants = {}
end

function ImageProcessor:SetConstant(name, ...)
    if self.constants[name] then
        self.constants[name]:SetValue(...)
    else
        self.constants[name] = engine.rendering.ShaderConstantValue( name, ... )
        self.model:SetShaderConstant(self.constants[name])
    end
end

function ImageProcessor:Process(effect, input_image, w, h, format)
    format = format or PIXEL_FORMAT.A8R8G8B8
    if input_image and ( not w or not h) then
        w, h = input_image:GetSize()
    end

    self.cam:SetOrthoProjection(-w/2, w/2, -h/2, h/2) 
    self.cam:SetPosition(0,0,0)
    
    if self.w ~= w or self.h ~= h then
        self.renderer:SetRenderFormat(math.round(w), math.round(h), format, -1, false, false)
        self.w, self.h = w, h
    end
    
    if self.input_texture ~= input_image then
        self.input_texture = input_image
        self.model:SetTexture(0, self.input_texture)
    end

    if self.effect ~= effect then
        self.model:SetEffect(effect)
        self.effect = effect
    end

    self.renderer:ClearTexture()
    self.renderer:RenderToTexture(TheGame:GetLevelOfDetail())
    self.last_result = self.renderer:GetOutputTexture()
    return self.last_result
end


function ImageProcessor:GetOutput()
    return self.last_result
end
