----------------------------------------------------------------------------
-- UserSettings file that keeps track of control rebindings and other input

local InputSettings = class( "InputSettings", UserSettings )

function InputSettings:init()
    UserSettings.init( self, "input_settings.lua", engine.settings )
    self:VerifySetting( "key_binds", {} )
end

function InputSettings:AssignInputBindings( bindings )
    if bindings.key_binds then
        local t = self:GetSetting( "key_binds" )
        table.clear( t )
        for control_name, binding in pairs( bindings.key_binds ) do
            t[ control_name ] = binding
        end
        self.dirty = true
    end
end

function InputSettings:GetKeyBinding( control_name )
    local rebinds = self:GetSetting( "key_binds" )
    if rebinds[ control_name ] then
        local binding = shallowcopy( rebinds[ control_name ] )
        binding.control = rawget( Controls.Digital, control_name )
        if binding.control then
            return binding
        end
    end
end

function InputSettings:ApplyInputBindings( input )
    local rebinds = self:GetSetting( "key_binds" )
    for control_name, t in pairs( rebinds ) do
        local binding = self:GetKeyBinding( control_name )
        if binding then
            input:RebindKey( binding )
        else
            print( "ApplyToInput: Unknown key control binding:", control_name )
            rebinds[ control_name ] = nil
        end
    end
end
