----------------------------------------------------------

local BasicLocalizedDef = class("BasicLocalizedDef")
function BasicLocalizedDef:init(id, data)
    assert(id)
    self.id = id
    for k, v in pairs(data) do
        self[k] = v
    end
end

function BasicLocalizedDef:SetModID( mod_id )
    self.mod_id = mod_id
end

function BasicLocalizedDef:GetModID()
    return self.mod_id
end

function BasicLocalizedDef:GetLocNameKey()
    return self:GetLocPrefix() .. ".NAME"
end

function BasicLocalizedDef:GetLocDescKey()
    return self:GetLocPrefix() .. ".DESC"
end

function BasicLocalizedDef:GetLocTitleKey()
    return self:GetLocPrefix() .. ".TITLE"
end

function BasicLocalizedDef:GetLocFlavourKey()
    return self:GetLocPrefix() .. ".FLAVOUR"
end

function BasicLocalizedDef:GetLocalizedFlavour()
    local result = nil
    if self.flavour then
        result = LOC(self:GetLocFlavourKey())
    end
    return result
end


function BasicLocalizedDef:GetLocalizedName()
    local result = nil
    if self.name then
        result = LOC(self:GetLocNameKey())
    end
    return result
end

function BasicLocalizedDef:GetLocalizedDesc()
    local result = nil
    if self.desc then
        result = LOC(self:GetLocDescKey())
    end
    return result
end

function BasicLocalizedDef:GetLocalizedTitle()
    local result = nil
    if self.title then
        result = LOC(self:GetLocTitleKey())
    end
    return result
end

function BasicLocalizedDef:GetLocalizedString( key )
    return LOC(self:GetLocPrefix() .. "." .. key:upper())
end

function BasicLocalizedDef:HarvestString(t, key, str)
    if str == nil then
        str = self[ key ]
    end
    if type(str) == "string" then
        key = self:GetLocPrefix() .. "." .. key:upper()
        assert_warning( t[ key ] == nil, "Duplicate loc key: %s", key )
        t[ key ] = str
    else
        LOGWARN( "Missing key '%s'", key )
    end
end

function BasicLocalizedDef:HarvestStrings(t)
    assert(self.id)

    if self.name then
        local key = self:GetLocNameKey()
        t[key] = self.name
        --print(key, t[key])
    end

    if self.desc then
        local key = self:GetLocDescKey()
        t[key] = self.desc
        --print(key, t[key])
    end

    if self.title then
        local key = self:GetLocTitleKey()
        t[key] = self.title
        --print(key, t[key])
    end

    if self.flavour then
        local key = self:GetLocFlavourKey()
        t[key] = self.flavour
    end

    if self.loc_strings then
        for key, str in pairs( self.loc_strings ) do
            self:HarvestString( t, key, str )
        end
    end
end

----------------------------------------------------------

local SlideShowDef = class("SlideShowDef", BasicLocalizedDef)

function SlideShowDef:init(id, data)
    BasicLocalizedDef.init(self, id, data)
end

function SlideShowDef:GetLocPrefix()
    return "SLIDESHOW." .. string.upper(self.id)
end

function SlideShowDef:GetLocTxtKey( idx )
    return string.format( "%s.%d.TXT", self:GetLocPrefix(), idx )
end

function BasicLocalizedDef:GetLocalizedTxt( idx )
    local result = nil
    if self.script[ idx ] and self.script[ idx ].txt then
        result = LOC(self:GetLocTxtKey( idx ))
    end
    return result
end

function SlideShowDef:HarvestStrings( t )
    for i, data in ipairs(self.script) do
        if data.txt then
            local key = self:GetLocTxtKey( i )
            t[key] = data.txt
        end
    end
end

----------------------------------------------------------

local FactionDef = class("FactionDef", BasicLocalizedDef)

function FactionDef:init(id, data)
    BasicLocalizedDef.init(self, id, data)
end

function FactionDef:GetLocPrefix()
    return "FACTION." .. string.upper(self.id)
end

----------------------------------------------------------

local NegotiationCardDef = class("NegotiationCardDef", BasicLocalizedDef)

function NegotiationCardDef:init(id, data)
    BasicLocalizedDef.init(self, id, data)
end

function NegotiationCardDef:CalculateComposureText( amount )
    return tostring(amount)
end

function NegotiationCardDef:GetLocPrefix()
    return "CARD." .. string.upper(self.id)
end

function NegotiationCardDef:HarvestStrings(t)
    BasicLocalizedDef.HarvestStrings( self, t )

    if self.alt_desc then
        self:HarvestString( t, "alt_desc" )
    end

    if self.quips then
        self.quips:HarvestStrings( t )
    end
end

----------------------------------------------------------

local BattleCardDef = class("BattleCardDef", BasicLocalizedDef)

function BattleCardDef:init(id, data)
    BasicLocalizedDef.init(self, id, data)
end

function BattleCardDef:CalculateDefendText( amount )
    return tostring(amount)
end

function BattleCardDef:GetLocPrefix()
    return "ATTACK." .. string.upper(self.id)
end

function BattleCardDef:HarvestStrings(t)
    BasicLocalizedDef.HarvestStrings( self, t )

    if self.alt_desc then
        self:HarvestString( t, "alt_desc" )
    end
end

---------------------------------------------------------

local GraftDef = class("GraftDef", BasicLocalizedDef)

function GraftDef:init(id, data)
    BasicLocalizedDef.init(self, id, data)
end

function GraftDef:GetLocPrefix()
    return "GRAFT." .. string.upper(self.id)
end

---------------------------------------------------------

local CombatConditionDef = class("CombatConditionDef", BasicLocalizedDef)

function CombatConditionDef:init(id, data)
    BasicLocalizedDef.init(self, id, data)
end

function CombatConditionDef:GetLocPrefix()
    return "CONDITION." .. string.upper(self.id)
end

function CombatConditionDef:HarvestStrings(t)
    -- The basic combat condition strings are localized as FeatureDef.
    -- BasicLocalizedDef.HarvestStrings( self, t )

    if self.alt_desc then
        self:HarvestString( t, "alt_desc" )
    end
end

---------------------------------------------------------

local ModifierDef = class("ModifierDef", BasicLocalizedDef)

function ModifierDef:init(id, data)
    BasicLocalizedDef.init(self, id, data)
end

function ModifierDef:GetLocPrefix()
    return "MODIFIER." .. string.upper(self.id)
end

function ModifierDef:GetLocalizedName()
    return self._base.GetLocalizedName( self ) or (self.base_def and self.base_def:GetLocalizedName())
end

function ModifierDef:HarvestStrings(t)
    BasicLocalizedDef.HarvestStrings( self, t )

    if self.alt_desc then
        self:HarvestString( t, "alt_desc" )
    end
end

---------------------------------------------------------

local FeatureDef = class("FeatureDef", BasicLocalizedDef)

function FeatureDef:init(id, data)
    BasicLocalizedDef.init(self, id, data)

    assert(self.desc or self.desc_fn or self.posted_feature, loc.format("FeatureDef '{1}' must have desc or desc_fn set", id))
end

function FeatureDef:GetLocPrefix()
    return self.PREFIX .. string.upper(self.id)
end

function FeatureDef:GetLocFeatureDescKey()
    return self:GetLocPrefix() .. ".FEATURE_DESC"
end

function FeatureDef:GetFeatureDesc()
    if self.feature_desc then
        local key = self:GetLocFeatureDescKey()
        return LOC(key)
    end
end

function FeatureDef:GetLocFeatureTooltipKey()
    return self:GetLocPrefix() .. ".FEATURE_TOOLTIP"
end

function FeatureDef:GetFeatureTooltip()
    local key = self:GetLocFeatureTooltipKey()
    return LOC(key)
end

function FeatureDef:HarvestStrings(t)
    BasicLocalizedDef.HarvestStrings(self, t)
    if self.feature_desc then
        t[self:GetLocFeatureDescKey()] = self.feature_desc
    end
    if self.feature_tooltip then
        t[self:GetLocFeatureTooltipKey()] = self.feature_tooltip
    end
end

---------------------------------------------------------

local BattleFeatureDef = class( "BattleFeatureDef", FeatureDef )
BattleFeatureDef.PREFIX = "BATTLE_FEATURE."

local NegotiationFeatureDef = class( "NegotiationFeatureDef", FeatureDef )
NegotiationFeatureDef.PREFIX = "NEGOTIATION_FEATURE."

----------------------------------------------------------
-- THESE DO NOT INHREIT FROM BasicLocalizedDef
----------------------------------------------------------

local CharacterSkin = class("CharacterSkin" )
function CharacterSkin:init(id, data)
    self.id = id
    for k, v in pairs(data) do
        self[k] = v
    end

    if self.loved_graft then
        local graft = Content.GetGraft( self.loved_graft )
        assert( graft, self.loved_graft )
        assert( graft.is_good == nil or graft.is_good == true, graft.id )
        graft.is_good = true
    end
    if self.hated_graft then
        local graft = Content.GetGraft( self.hated_graft )
        assert( graft, self.hated_graft )
        assert( graft.is_good == nil or graft.is_good == false, graft.id )
        graft.is_good = false
    end

    local loc_name = self.name and Content.LookupString( self:GetLocNameKey() )
    if loc_name then
        local loc_translator = require "util/loc_translator"
        self.name, self.name_declensions = loc_translator.parseNameDeclensions( loc_name )
    end

    assert( self.uuid )
end

function CharacterSkin:SetModID( mod_id )
    self.mod_id = mod_id
end

function CharacterSkinGetModID()
    return self.mod_id
end

function CharacterSkin:GetContentID()
    return self.uuid
end

function CharacterSkin:GetLocPrefix()
    return string.format("CHARACTER_SKIN.%s.%s", string.upper(self.id), string.upper(self.uuid))
end

function CharacterSkin:GetLocNameKey()
    return self:GetLocPrefix() .. ".NAME"
end

function CharacterSkin:GetLocBioKey()
    return self:GetLocPrefix() .. ".BIO"
end

function CharacterSkin:GetLocLovedBioKey()
    return self:GetLocPrefix() .. ".LOVED_BIO"
end

function CharacterSkin:GetLocHatedBioKey()
    return self:GetLocPrefix() .. ".HATED_BIO"
end

function CharacterSkin:GetLocKilledBioKey()
    return self:GetLocPrefix() .. ".KILLED_BIO"
end

function CharacterSkin:GetLocDrankBioKey()
    return self:GetLocPrefix() .. ".DRANK_BIO"
end

function CharacterSkin:GetLocNickNameKey()
    return self:GetLocPrefix() .. ".NICKNAME"
end

function CharacterSkin:GetLocSmallTalkKey()
    return self:GetLocPrefix() .. ".SMALLTALK"
end

function CharacterSkin:GetLocalizedName( agent )
    return self.name -- Pre-Localized
    -- return self.name and loc.format( LOC(self:GetLocNameKey()), agent )
end

function CharacterSkin:GetLocalizedNameWithDeclension( agent, case )
    local result = nil
    if self.name_declensions and self.name_declensions[ case ] then
        result = loc.format( self.name_declensions[ case ], agent )
    else
        result = self:GetLocalizedName( agent )
    end

    return result
end

function CharacterSkin:GetLocalizedNameDeclensions()
    return self.name_declensions or table.empty
end

function CharacterSkin:GetLocalizedBio( agent )
    return self.bio and loc.format( LOC(self:GetLocBioKey()), agent )
end

function CharacterSkin:GetLocalizedLovedBio( agent )
    return self.loved_bio and loc.format( LOC(self:GetLocLovedBioKey()), agent )
end

function CharacterSkin:GetLocalizedHatedBio( agent )
    return self.hated_bio and loc.format( LOC(self:GetLocHatedBioKey()), agent )
end

function CharacterSkin:GetLocalizedNickName( agent )
    return self.nickname and loc.format( LOC(self:GetLocNickNameKey()), agent )
end

function CharacterSkin:GetLocalizedSmallTalk( agent )
    return self.smalltalk and loc.format( LOC(self:GetLocSmallTalkKey()), agent )
end

function CharacterSkin:HarvestStrings(t)
    local fields = {
        name = "GetLocNameKey",
        bio = "GetLocBioKey",
        loved_bio = "GetLocLovedBioKey",
        hated_bio = "GetLocHatedBioKey",
        killed_bio = "GetLocKilledBioKey",
        drank_bio = "GetLocDrankBioKey",
        nickname = "GetLocNickNameKey",
        smalltalk = "GetLocSmallTalkKey",
    }

    for k, fn in pairs(fields) do
        if self[k] then
            local key = self[fn](self)
            t[key] = self[k]
        end
    end
end

            
----------------------------------------------------------

local CharacterSkinsList = class( "CharacterSkinsList" )

function CharacterSkinsList:HarvestStrings( t )
    for _, skin in ipairs( self ) do
        skin:HarvestStrings( t )
    end
end
            
----------------------------------------------------------

local OpinionEventDef = class( "OpinionEventDef" )

function OpinionEventDef:GetLocPrefix()
    return "OPINION." .. string.upper( self.id )
end

function OpinionEventDef:GetLocTxtKey()
    return self:GetLocPrefix() .. ".TXT"
end

function OpinionEventDef:GetLocalizedTxt()
    return LOC( self:GetLocTxtKey() )
end

function OpinionEventDef:HarvestStrings( t )
    if type(self.txt) == "string" then
        t[ self:GetLocTxtKey() ] = self.txt
    end
end


----------------------------------------------------------

local UnlockableDef = class( "UnlockableDef" )

function UnlockableDef:init( data )
    for k, v in pairs(data) do
        self[k] = v
    end
    assert( self.id )
end

function UnlockableDef:GetLocPrefix()
    return "UNLOCKABLE." .. string.upper( self.id )
end

function UnlockableDef:GetLocNameKey()
    return self:GetLocPrefix() .. ".NAME"
end

function UnlockableDef:GetLocNegotiationNameKey()
    return self:GetLocPrefix() .. ".NEGOTIATION_NAME"
end

function UnlockableDef:GetLocBattleNameKey()
    return self:GetLocPrefix() .. ".BATTLE_NAME"
end

function UnlockableDef:GetLocalizedName()
    return LOC( self:GetLocNameKey() )
end

function UnlockableDef:GetLocalizedNegotiationName()
    return LOC( self:GetLocNegotiationNameKey() )
end

function UnlockableDef:GetLocalizedBattleName()
    return LOC( self:GetLocBattleNameKey() )
end

function UnlockableDef:HarvestStrings( t )
    t[ self:GetLocNameKey() ] = self.name
    t[ self:GetLocNegotiationNameKey() ] = self.negotiation_name
    t[ self:GetLocBattleNameKey() ] = self.battle_name
end

----------------------------------------------------------

local UnlockTrackDef = class( "UnlockTrackDef" )

function UnlockTrackDef:init( track )

    track = self:FixPts( track )

    for i, track_item in ipairs( track ) do
        assert( is_instance( track_item, UnlockableDef ))

        for k, item in ipairs(track_item.items) do
            if item.battle_card then
                assert( Content.GetBattleCard( item.battle_card ), item.battle_card )
                item.unlock_id = Content.AddUnlock(item.battle_card)
            elseif item.deck_id then
                 assert( Content.GetDeck( item.deck_id ), item.deck_id )
                 item.unlock_id = Content.AddUnlock(item.deck_id)
            elseif item.negotiation_card then
                assert( Content.GetNegotiationCard( item.negotiation_card ), item.negotiation_card )
                item.unlock_id = Content.AddUnlock(item.negotiation_card)
            elseif item.graft_id then
                assert( Content.GetGraft( item.graft_id ), item.graft_id )
                item.unlock_id = Content.AddUnlock(item.graft_id)
            elseif item.outfit_id then
                assert( Content.GetOutfit( item.outfit_id ), item.outfit_id )
                item.unlock_id = Content.AddUnlock(item.outfit_id)
            elseif item.player_background then
                assert( GetPlayerBackground( item.player_background ), item.player_background )
                item.unlock_id = Content.AddUnlock(item.player_background)
            end
        end
        if track_item.icon then
            track_item.icon = engine.asset.Texture( track_item.icon, true )
        end
        if track_item.icon_locked then
            track_item.icon_locked = engine.asset.Texture( track_item.icon_locked, true )
        end

        self[ i ] = track_item
    end
end

function UnlockTrackDef:HarvestStrings( t )
    for i, track_item in ipairs( self ) do
        track_item:HarvestStrings( t )
    end
end

--calculate rolling totals
function UnlockTrackDef:FixPts( track )
    local pts = 0
    for k,v in ipairs(track) do
        v.start_pts = pts
        pts = pts + v.pts
        v.total_pts = pts
    end
    return track
end


----------------------------------------------------------

local GriftDef = class( "GriftDef", BasicLocalizedDef )

function GriftDef:init( data )
    for k, v in pairs(data) do
        self[k] = v
    end
    assert( type(self.id) == "string" )
    assert( self.name, self.id )
    assert( type(self.progress) == "function", self.id )
    assert( self.perk_points, self.id )
end

function GriftDef:CanComplete()
    local progress, max_progress = self:GetProgress()
    if progress < max_progress then
        return false
    end

    return true
end

function GriftDef:GetProgress()
    local progress, max_progress = self.progress( self, TheGame:GetGameStats() )
    progress = math.min( progress, max_progress )
    return progress, max_progress
end

function GriftDef:IsComplete()
    return TheGame:GetGameProfile():IsGriftComplete( self.id )
end

function GriftDef:Complete()
    print( "Grift Complete:", self.id )
    local profile = TheGame:GetGameProfile()
    profile:AddPerkPoints( self.perk_points )
    profile:CompleteGrift( self.id )

    local tex -- = engine.asset.Texture("UI/char_2_story.tex" )
    local title = loc.format( LOC"NOTIFICATION.GRIFT_COMPLETE", loc.format( self:GetLocalizedName(), self.count ))
    local desc = loc.format( LOC"NOTIFICATION.GRIFT_COMPLETE_DETAILS", self.perk_points )

    TheGame:FE():ShowAchievementToast( tex, title, desc )
end

function GriftDef:GetLocPrefix()
    return "GRIFT." .. string.upper( self.id )
end


----------------------------------------------------------

local AchievementDef = class( "AchievementDef", BasicLocalizedDef )

function AchievementDef:init( data )
    for k, v in pairs(data) do
        self[k] = v
    end
    assert( type(self.id) == "string" )
    assert( self.name, self.id )

    if type(self.icon) ~= "userdata" then
        -- Generate conventional icon paths for non-upgraded cards. 
        local path = self.icon or string.format( "icons/achievements/%s.tex", self.id:lower() )
        self.path_str = path
        self.icon = engine.asset.Texture( path, true )
        if self.icon:IsDefault() then
            self.icon = global_images.default
        end
    end

    if type(self.locked_icon) ~= "userdata" then
        -- Generate conventional icon paths for non-upgraded cards. 
        local path = self.locked_icon or string.format( "icons/achievements/%s_locked.tex", self.id:lower() )
        self.locked_path_str = path
        self.locked_icon = engine.asset.Texture( path, true )
        if self.locked_icon:IsDefault() then
            self.locked_icon = global_images.default
        end
    end
end

function AchievementDef:GetLocalizedDesc()
    local result = self._base.GetLocalizedDesc( self )
    if self.desc_fn then
        result = self.desc_fn( self, result )
    end
    return result
end

function AchievementDef:GetLocPrefix()
    return "ACHIEVEMENT." .. string.upper( self.id )
end

