require "locationutil"

local LocationDef = class("LocationDef")

function LocationDef:init( id, data )
	self.id = id 
	for k, v in pairs(data) do
		self[k] = v
	end
end

function LocationDef:GetLocPrefix()
	local prefix = "LOCATION." .. string.upper(self.id)
	return prefix
end

function LocationDef:GetLocNameKey()
	local key = self:GetLocPrefix() .. ".NAME"
	return key
end

function LocationDef:GetLocDescKey()
	local key = self:GetLocPrefix() .. ".DESC"
	return key
end

function LocationDef:GetLocalizedName()
	return self.name and LOC(self:GetLocNameKey())
end

function LocationDef:GetLocalizedDesc()
	return self.desc and LOC(self:GetLocDescKey())
end


function LocationDef:HarvestStrings(t)
	assert(self.id)

	if self.name then
		local name_key = self:GetLocNameKey()
		if type(self.name) ~= 'function' then
			t[name_key] = self.name
		else
			print("ERROR:", name_key, "is a function")
		end
	end

	if self.desc then
		local desc_key = self:GetLocDescKey()
		t[desc_key] = self.desc
	end


	if self.sublocations then
		local prefix = self:GetLocPrefix()
		for sub_loc_name, data in pairs(self.sublocations) do

			local sl_name = string.upper(sub_loc_name)
			if data.name then
				local sl_name_key = loc.format("{1}.{2}.NAME", prefix, sl_name)
				t[sl_name_key] = data.name
			end

			if data.desc then
				local sl_desc_key = loc.format("{1}.{2}.NAME", prefix, sl_name)
				t[sl_desc_key] = data.desc
			end
		end
	end
end


