local LocationUtil = class( "LocationUtil" )

function LocationUtil.Kerfuffle(location)
    for i, exit in location:Exits() do
        if exit.can_hear_through then
            local other_loc = exit:GetTo()
            if other_loc then
                local guard = other_loc:GetAgentByRole(CHARACTER_ROLES.GUARD)
                if guard then
                    
                    guard:OpinionEvent(OPINION.HEARD_KERFUFFLE, {acted_upon = false})

                end
            end
        end
    end
end

function LocationUtil.SendPatronsAway( location, predFn )
    local t = ObtainWorkTable()

    for k,agent in pairs(location.agents) do
        if agent:GetBrain():IsPatronizing() and (predFn == nil or predFn(agent)) then
            table.insert(t, agent)
        end
    end

    for k, agent in pairs(t) do
        agent:MoveToLimbo()
    end
    ReleaseWorkTable(t)
end


function LocationUtil.CanLeaveLocation(location)
    
    if location:GetContent().no_exit then
        return false
    end


    for i, quest in TheGame:GetGameState():ActiveQuests() do
        if quest:IsActive() and quest:GetQuestDef().restrict_exit then
            if quest:GetQuestDef().restrict_exit(quest, location) then
                return false
            end
        end
    end

    --make sure everyone who you might need to go see is somewhere
    TheGame:GetGameState():PutImportantAgentsInWorld()
    
    if location:GetRootLocation() == location then
        --is there anywhere else to go?
        for _, other_location in TheGame:GetGameState():AllLocations() do
            if other_location ~= location and other_location:IsCurrentlyImportant() then
                return true
            end
        end

        return false
    else
        return true
    end
end

function LocationUtil.PopulateLocation(location, patron_filter, to_seat_override)

    local to_limbo = {}
    for k,agent in pairs(location.agents) do
        if AgentUtil.IsInHiding(agent) then
            table.insert(to_limbo, agent)
        end
    end
    
    for k, agent in ipairs(to_limbo) do
        agent:MoveToLimbo()
    end

    local t = ObtainWorkTable()
    TheGame:GetGameState():GetQuestAgentsForLocation(location, t)
    for agent, role in pairs(t) do
        if agent:IsInPlayerParty() then
            agent:Dismiss()
        else
            agent:ClearParty()
        end
        agent:MoveToLocation(location)
        agent:SetLocationRole(role)
    end
    ReleaseWorkTable(t)

    --people working here
    if location.work then
        for k,wp in pairs(location.work) do
            wp:DoHiring()
            if wp:GetAgent() and wp:ShouldBeWorking() and not AgentUtil.IsInHiding(wp:GetAgent()) then
                if not AgentUtil.IsAgentBarredFromLocation(wp:GetAgent(), location) then
                    wp:GetAgent():GetBrain():SendToWork()
                end
            end
        end
    end

    local to_seat = to_seat_override or ( location:GetCurrentPatronCapacity() - location:GetNumCurrentPatrons() )
    if to_seat > 0 then

        local candidates = ObtainWorkTable()

        --find all of the agents who are free to move around
        for _, agent in TheGame:GetGameState():Agents() do
            if not AgentUtil.CanAct(agent) then
                -- disabled
            elseif agent:GetBrain():GetWorkPosition() and agent:GetBrain():GetWorkPosition():ShouldBeWorking() then
                --we're busy
            elseif patron_filter and not patron_filter(agent) then
                -- we're filtered
            elseif not agent:InLimbo() then
                -- already in the world.

            elseif agent:HasInactiveQuestMembership() then
                -- don't tempt fate
                
            elseif agent:HasTag("no_patron") then
                -- not someone that we can touch

            elseif AgentUtil.HasDynamicPlotArmour(agent) then
                -- probably shouldn't be here

            elseif AgentUtil.CanAgentPatronizeLocation( location, agent ) then
                table.insert(candidates, agent)
            end
        end

        for k = 1, to_seat do
            local num_friends = 0
            local num_enemies = 0
            for _,v in pairs( location.agents ) do
                if v:GetRelationship() > RELATIONSHIP.NEUTRAL then
                    num_friends = num_friends + 1
                elseif v:GetRelationship() < RELATIONSHIP.NEUTRAL then
                    num_enemies = num_enemies + 1
                end
            end

            -- try not to stack the locations too badly.
            local enough_friends = num_friends >= 1
            local enough_enemies = num_enemies >= 1

            local function candidate_sorter_fn(a,b)
                if a:IsCurrentlyImportant() == b:IsCurrentlyImportant() then
                    
                    local a_friend = a:GetRelationship() > RELATIONSHIP.NEUTRAL
                    local b_friend = b:GetRelationship() > RELATIONSHIP.NEUTRAL

                    local a_enemy = a:GetRelationship() < RELATIONSHIP.NEUTRAL
                    local b_enemy = b:GetRelationship() < RELATIONSHIP.NEUTRAL
                            

                    local a_score = 0
                    local b_score = 0
                    
                    if enough_enemies then
                        if a_enemy then
                            a_score = a_score - 5
                        end
                        if b_enemy then
                            b_score = b_score - 5
                        end
                    end

                    if enough_friends then
                        if a_friend then
                            a_score = a_score - 5
                        end
                        if b_friend then
                            b_score = b_score - 5
                        end
                    end

                    if a_score == b_score then
                        return a.guid < b.guid    
                    else
                        return b_score < a_score
                    end

                    
                else
                    return a:IsCurrentlyImportant()
                end
            end

            table.sort(candidates, candidate_sorter_fn)
            if candidates[1] then
                -- DBG({location = location, candidates = shallowcopy(candidates) } )
                candidates[1]:GetBrain():SendToPatronize(location)    
                print ("FOUND PATRON", candidates[1], candidates[1]:HasQuestMembership())
                table.remove(candidates, 1)
            end
        end

        --after all that, if we still need more patrons, generate them
        local to_generate = location:GetCurrentPatronCapacity() - location:GetNumCurrentPatrons()
        
        if to_generate > 0 then
            if location:GetContent().patron_data.patron_generator then
                for k = 1, to_generate do
                    location:GetContent().patron_data.patron_generator(location)
                end
            elseif location:GetContent().patron_data.patron_defs then
                for k = 1, to_generate do
                    TheGame:GetGameState():AddSkinnedAgent( location:GetContent().patron_data.patron_defs ):GetBrain():SendToPatronize(location)
                end
            end

        end

        location:SetCurrentPatronCapacity( 0 )

        ReleaseWorkTable(candidates)
    end
end





