
engine.inst:ProfilerPush("MAIN.LUA")
----------------------------------------
-- lua API patching.

--5.3 compatibility stuff
loadstring = load

function GetActiveDebugger()
    return rawget( _G, "TheDebugger")
end

--monkey-patch in utf8-aware version of the string library.
local utf8_ex = require "lua-utf8"
for k,v in pairs(string) do
    if utf8_ex[k] then
        string[k] = utf8_ex[k]
    end
end

local oldrequire = require
require = function(path) 
    local a, b, c = oldrequire( path )
    return a, b, c
end


package.path = "scripts/?.lua;?.lua;scripts/lib/?.lua"
----------------------------------------
if PLATFORM == "WIN32" then
    loadfile("scripts/lib/socket.lua")()
    loadfile("scripts/lib/mime.lua")()
end

require "printing"
TheDebugger = nil
function StartDebugger(redirect_output)
    local debugger = require 'vscode-debuggee'
    if GetActiveDebugger() == nil then
        rawset(_G, "TheDebugger", debugger)
        local startResult, breakerType, base_path = TheDebugger.Start{redirectPrint = redirect_output}
        print('Debugger Started ->', startResult, breakerType, base_path)
    end
end

----------------------------------------
if engine.inst:HasCommandLineOption("attach_debugger") then
    StartDebugger(true)
end

require "util/strict"
strictify( _G )
require "util/class"
require "util/util"
json = require "util/json"

easing = require "util/easing"
loc = require "util/locstring"

serpent = require "util/serpent"
require "util/iter"
require "helpers"
require "user_settings"
require "data_util"
SoundEvents = require "sound_event_names"

require "constants"
require "commonassets"
require "animcache"
Content = require "content/content"

--a hack to diambiguate our release builds for now...
function IsEarlyAccess()
    return false
end

---------------------------------


local debug_settings =
{
    DEBUG = false,
    NO_SOUND = false,
    NO_MUSIC = false,
    NO_MOVIES = false,
}

do
    engine.inst:ProfilerPush("Settings")

    local local_settings_path = 'scripts/local_settings.lua'
    local ok, result = pcall( dofile, local_settings_path )
    if ok and result then
        for k,v in pairs(result) do
            debug_settings[k] = v
        end
    else
        print( "No local settings:", result )
    end

    if engine.inst:HasCommandLineOption("debug") or BRANCH == "dev" then
        debug_settings.DEBUG = true
    end

    if engine.inst:HasCommandLineOption("unlockall") then
        debug_settings.ALL_UNLOCKS = true
    end

    print ("LOCAL DEBUG SETTINGS:", serpent.block(debug_settings))
    engine.inst:ProfilerPop()
end


require "audiosystem"
AUDIO = AudioSystem(not debug_settings.NO_SOUND)
engine.inst:ProfilerPush( "Loading Sound Banks" )
AUDIO:DoInitialLoad()


engine.inst:ProfilerPop()


require "input/inputconstants"
require "game"

-- prime the generator
math.randomseed(os.time())

-- local oldgc = collectgarbage
-- collectgarbage = function ( ... )
--  engine.inst:ProfilerPush("collectgarbage")
--  local ret = oldgc(...)
--  engine.inst:ProfilerPop()
--  --print(debugstack()) -- who you gunna ... get called by?
--  return ret
-- end

local function screen_resize(w,h)
    TheGame:OnScreenResize(w,h)
end

local function render(dt)

    engine.inst:ProfilerPush("LuaRender", 0xFF9090B0)
    TheGame:Render(dt)
    engine.inst:ProfilerPop()
end



local function update(dt)
    -- buffered_error is a magic string set by soundsystem.cpp.
    if rawget( _G, "buffered_error" ) then
        assert(false, "Buffered Error:\n************\n" .. buffered_error .. "\n***********")
    end

    AUDIO:Update(dt)
    TheGame:Update(dt * TheGame:GetWallTimeScale())

    if GetActiveDebugger() then
        GetActiveDebugger().poll()
    end

end


local function key_down_callback(keyid, modifiers)
    TheGame:GetInput():OnKeyDown(keyid, modifiers);
end

local function key_repeat_callback(keyid, modifiers)
    TheGame:GetInput():OnKeyRepeat(keyid, modifiers);
end

local function key_up_callback(keyid, modifiers)
    TheGame:GetInput():OnKeyUp(keyid, modifiers);
end

local function text_input_callback(text)
    TheGame:GetInput():OnTextInput(text);
end

local function text_edit_callback(text)
    TheGame:GetInput():OnTextEdit(text);
end


-- Mouse:
local function mouse_move_callback(x, y)
    TheGame:GetInput():OnMouseMove(x, y);
end

local function mouse_wheel_callback(wheeldelta)
    TheGame:GetInput():OnMouseWheel(wheeldelta);
end

local function mouse_button_down_callback(x, y, button)
    TheGame:GetInput():OnMouseButtonDown(x, y, button);
end

local function mouse_button_up_callback(x, y, button)
    TheGame:GetInput():OnMouseButtonUp(x, y, button);
end

local function touch_began_callback(x, y)
    TheGame:GetInput():OnTouchBegan(x, y);
end

local function touch_move_callback(x, y)
    TheGame:GetInput():OnTouchMove(x, y);
end

local function touch_ended_callback(x, y)
    TheGame:GetInput():OnTouchEnded(x, y);
end



-- Gamepad:
local function gamepad_connected_callback(gamepad_id, gamepad_name)
    TheGame:GetInput():OnGamepadConnected(gamepad_id, gamepad_name);
end

local function gamepad_disconnected_callback(gamepad_id)
    TheGame:GetInput():OnGamepadDisconnected(gamepad_id);
end

local function gamepad_button_down_callback(gamepad_id, button)
    TheGame:GetInput():OnGamepadButtonDown(gamepad_id, button, false);
end

local function gamepad_button_repeat_callback(gamepad_id, button)
    TheGame:GetInput():OnGamepadButtonRepeat(gamepad_id, button);
end

local function gamepad_button_up_callback(gamepad_id, button)
    TheGame:GetInput():OnGamepadButtonUp(gamepad_id, button);
end

local function gamepad_analog_input_callback(gamepad_id, ls_x, ls_y, rs_x, rs_y, lt, rt)
    TheGame:GetInput():OnGamepadAnalogInput(gamepad_id, ls_x, ls_y, rs_x, rs_y, lt, rt);
end

local function system_event_callback( event, ... )
    if event == engine.inst.UserSignInWrongAccount then
        TheGame:PromptSignInWrongAccount()

    elseif event == engine.inst.UserSignIn then
        TheGame:OnSignIn()

    elseif event == engine.inst.UserSignOut or event == engine.inst.UserSignInCancelled then
        TheGame:PromptSignIn()
    end
end

local function error_callback( str )
    if BRANCH ~= "dev" then
        local s = { "If you are experiencing issues, disabling some of your installed mods may help:" }
        for i, mod in ipairs( Content.GetInstalledMods() ) do
            table.insert( s, "\t"..tostring(mod))
        end
        if #s > 1 then
            return table.concat( s, "\n" )
        end
    end
end

local function wall_time_query()
    if TheGame then
        return TheGame:GetWallTime()
    end
    return 0
end

local function filedrop(txt)
    if TheGame then
        TheGame:OnDropFile(txt)
    end
end

local function screenshot(texture, texture_string)
    if TheGame then
        TheGame:ScreenShotReady(texture, texture_string)
    end
end

engine.inst:SetScreenSizeChangeFn(screen_resize)
engine.inst:SetUpdateFn(update)
engine.inst:SetFileDropFn(filedrop)
engine.inst:SetScreenShotReadyFn(screenshot)
engine.inst:SetRenderFn(render)

--  Keyboard
engine.inst:SetKeyDownFn(key_down_callback);
engine.inst:SetKeyRepeatFn(key_repeat_callback);
engine.inst:SetKeyUpFn(key_up_callback);
engine.inst:SetTextInputFn(text_input_callback);
engine.inst:SetTextEditFn(text_edit_callback);

-- Mouse:
engine.inst:SetMouseMoveFn(mouse_move_callback);
engine.inst:SetMouseWheelFn(mouse_wheel_callback);
engine.inst:SetMouseButtonDownFn(mouse_button_down_callback);
engine.inst:SetMouseButtonUpFn(mouse_button_up_callback);

-- Touch:
engine.inst:SetTouchBeganFn(touch_began_callback);
engine.inst:SetTouchMoveFn(touch_move_callback);
engine.inst:SetTouchEndedFn(touch_ended_callback);

-- Gamepad:
engine.inst:SetGamepadConnectedFn(gamepad_connected_callback);
engine.inst:SetGamepadDisconnectedFn(gamepad_disconnected_callback);
engine.inst:SetGamepadButtonDownFn(gamepad_button_down_callback);
engine.inst:SetGamepadButtonRepeatFn(gamepad_button_repeat_callback);
engine.inst:SetGamepadButtonUpFn(gamepad_button_up_callback);
engine.inst:SetGamepadAnalogInputFn(gamepad_analog_input_callback);

engine.inst:SetSystemEventFn(system_event_callback)
engine.inst:SetErrorFn(error_callback)
engine.inst:SetWallTimeFn(wall_time_query)

engine.inst:ProfilerPop()
 
-- printing the User name won't work here, because it is likely not logged in yet. 


-------------------------------------------------------------
-- And.... go!


local game = Game( debug_settings )

--Make sure the menu sounds have finished loading.
--We will not force the voice sound load yet. That can wait.
AUDIO:FinalizeMainLoad()


game:Reset()
--StartDebugger()


local test_suite = engine.inst:GetCommandLineOption("run_test_suite")
if test_suite and test_suite ~= "" then
    local dbg_env = TheGame:GetDebugEnv()
    engine.inst:SetAutoBugReportMode(true)

    TheGame.scheduler:DoTaskInTime("KILL SWITCH", TEST_RUN_TIME,
        function() 
            print ("SHUT DOWN TIME!!")
            TheGame:Shutdown()
        end)

    dbg_env.dbg_test( test_suite )
end


Content.InvokeModAPI( "OnGameStart" )
