--------------------------------------------------------------------------------
-- Instanced faction data.

local Faction = class("Faction")

function Faction:init( mgr, id, content_id )
    self.mgr = mgr
    self.id = id
    self.content = Content.GetFaction( content_id )

    assert( self.id )
    assert( self.content, content_id )
end

function Faction:GetColour()
    return self.content.colour or 0xffffffff
end

function Faction:GetID()
    return self.id
end

function Faction:SetModID( mod_id )
    self.mod_id = mod_id
end

function Faction:GetModID()
    return self.mod_id
end

function Faction:HasTag( tag )
    return self.content.tags and self.content.tags[ tag ] == true
end

function Faction:IsLawful()
    return self.content.unlawful ~= true
end

function Faction:IsUnlawful()
    return self.content.unlawful == true
end

function Faction:Defends(protector, victim)
    if type(self.content.mutual_defence) == "function" then
        return self.content.mutual_defence(protector, victim)
    else
        return self.content.mutual_defence
    end
end

function Faction:IsPlayer()
    return self.content.player == true
end

function Faction:GetName()
	if self.content.name then
		local key = self.content:GetLocNameKey()
		local name = LOC(key)
		return name
	else
		return nil
	end
end

function Faction:GetIcon()
    return self.content.icon
end

function Faction:GetColour()
    return self.content.colour
end

function Faction:GetColourBg()
    return self.content.colour_bg
end

function Faction:GetMapIcon()
    return self.content.map_icon
end

function Faction:GetContent()
    return self.content
end

function Faction:GetFactionRelationship( faction_id )
    return self.mgr:GetFactionRelationship( self.id, faction_id )
end

function Faction:SetFactionRelationship( faction_id, rep )
    assert( faction_id )
    
    self.mgr:SetFactionRelationship( self.id, faction_id, rep )
end


function Faction:Members()
    return ipairs ( TheGame:GetGameState():GetAgentBucket( self.id ) )
end

function Faction:Save()
    return
    {
        content_id = self.content.id,
        default_opinion_event_id = self.default_opinion_event_id,
    }
end


function Faction:SetDefaultOpinionEventID(default_opinion_event_id)
    self.default_opinion_event_id = default_opinion_event_id
end

function Faction:PostLoad( id, mgr )
    self.mgr = mgr
    self.id = id
    self.content = Content.GetFaction( self.content_id )
    self.content_id = nil
    assert( self.content, self.id )
end

function Faction:GetCharacterColours()
    return self.content.character_colours
end

function Faction:__tostring()
    return string.format("<%s>", self:GetName() )
end

