require "component"

local Music = class("Music", Component)

function Music:init()
    self.character_music = {}
end

function Music:ToggleMute()
    if self.mute then
        self.mute = false
        if self.music_event then
            self.music_event:SetVolume(1)
        end
    else
        if self.music_event then
            self.music_event:SetVolume(0)
        end
        self.mute = true
    end
end

function Music:SetParameter(name, val)
    if self.music_event then
        self.music_event:SetParameterValue(name, val)                
    end
end

function Music:StopCharacterMusic()
    for k,v in pairs(self.character_music) do
        v:Stop()
        v:Release()
    end
    table.clear(self.character_music)
    self.playing_character_music = nil
    
end


function Music:PlayCharacterMusic(music)
    if TheGame:GetLocalSettings().NO_MUSIC then
        return
    end
    
    if self.playing_character_music == music then
        return
    end
    
    self:StopMusic()

    if not self.character_music[music] then
        local event = AUDIO:CreateEventInstance(music)
        if event then
            event:Start()
            local is_day = TheGame:GetGameState() == nil or TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY 
            event:SetParameterValue("time_of_day", is_day and 0 or 1)
            if self.mute then
                event:SetVolume(0)
            end
        end
        self.character_music[music] = event
    end

    for k,v in pairs(self.character_music) do
        if k == music then
            v:SetParameterValue("duckstorymusic", 0)
        else
            v:SetParameterValue("duckstorymusic", 1)
        end
    end

    self.playing_character_music = music
end


function Music:PlayMusic(music)
    if TheGame:GetLocalSettings().NO_MUSIC then
        return
    end

    self.playing_character_music = nil
    for k,v in pairs(self.character_music) do
        v:SetParameterValue("duckstorymusic", 1)
    end

    if self.music_event_name then
        if self.music_event_name == music then
            return
        end

        if self.music_event then
            self.music_event:Stop()
            self.music_event:Release()
            self.music_event = nil
        end
    end

    self.music_event_name = music
    
    if music then
       
        self.music_event = AUDIO:CreateEventInstance(music)
        if self.music_event then
            self.music_event:Start()
            local is_day = TheGame:GetGameState() == nil or TheGame:GetGameState():GetDayPhase() == DAY_PHASE.DAY 
            self:SetParameter("time_of_day", is_day and 0 or 1)
            if self.mute then
                self.music_event:SetVolume(0)
            end
        end

        return self.music_event
    end
end

function Music:StopMusic( event )
    if self.music_event and (event == nil or event == self.music_event) then
        self.music_event:Stop()
        self.music_event:Release()
        self.music_event = nil
        self.music_event_name = nil
    end
end

function Music:PlayNavMusic()
    local nav_music = TheGame:LookupPlayerMusic( "deck_music" )
    if nav_music and nav_music ~= self.music_event_name and self.nav_loop == nil then
        self.nav_loop_name = nav_music
        if self.nav_loop_name then
            self.nav_loop = AUDIO:CreateEventInstance( self.nav_loop_name )
            if self.nav_loop then
                self.nav_loop:Start()
            end
        end
    end
end

function Music:StopNavMusic()
    if self.nav_loop then
        self.nav_loop:Stop()
        self.nav_loop:Release()
        self.nav_loop = nil
        self.nav_loop_name = nil
    end
end



