local NameList = class( "NameList" )

function NameList:init( id, filename )
    self.id = id
    self.filename = filename
    self.usage = {}

    self.name_defs = {}

    -- Localized names and declension table, parsed on demand.
    self.loc_names, self.loc_declensions = {}, {}

    self:ReadFile( filename )
end

function NameList:ReadFile( filename )
    assert(filename)
    for line in io.lines( filename ) do
        line = string.trim( line )
        local is_comment = line:find( "^[#]" )
        if not is_comment and #line > 0 then
            table.insert( self.name_defs, line )
        end
    end
end

function NameList:CreatePendingList( )
    --this is a bit ridiculous, but we want to be able to add more names between runs of the program.    
    self.pending_list = self.pending_list or {}
    table.clear(self.pending_list)

    local lowest_usage = nil
    for id, usage in pairs(self.usage) do
        lowest_usage = lowest_usage and math.min(usage, lowest_usage) or usage
    end

    for id, name in pairs(self.name_defs) do
        if not lowest_usage or self.usage[id] == lowest_usage then
            table.insert(self.pending_list, id)
        end
    end
    
    table.shuffle(self.pending_list)

end

function NameList:GenerateName()
    if #self.name_defs == 0 then
        return nil
    end

    if not self.pending_list or #self.pending_list == 0 then
        self:CreatePendingList()
    end
    
    assert(#self.pending_list > 0, "OUT OF NAMES")
    
    local handle = table.remove(self.pending_list)
    self.usage[handle] = self.usage[handle] and (self.usage[handle] + 1) or 1

    local key = self:GetLocKey( handle )
    return self:GetName( handle ), handle
end

function NameList:GetName( handle )
    if self.loc_names[ handle ] == nil then
        local key = self:GetLocKey( handle )
        local fullname = TheGame:Str( key )
        local loc_translator = require "util/loc_translator"
        self.loc_names[ handle ], self.loc_declensions[ handle ] = loc_translator.parseNameDeclensions( fullname )
    end

    return self.loc_names[ handle ], self.loc_declensions[ handle ]
end


function NameList:GetLocKey( handle )
    local idx = 1 + ( (handle-1) % #self.name_defs) 
    local name = self.name_defs[ idx ]
    if name then
        return string.format( "NAMEPOOL.%s.%s", self.id:upper(), name:upper() )
    end
end

function NameList:HarvestStrings( t )
    for i, name in ipairs( self.name_defs ) do
        local loc_key = self:GetLocKey( i )
        t[ loc_key ] = name
    end
end

