require "eventsystem"

local RESULT = MakeEnum
{
    "WIN",
    "LOSE",
    "CANCELLED"
}

local EVENT = MakeEnum
{   
    "BEGIN_NEGOTIATION",
    "END_NEGOTIATION",

    "SHUFFLE_DISCARDS",

    "INTENTS_CHANGED",
    "INTENT_REMOVED",
    "INTENT_CHANGED",
    "INTENT_ADDED",

    "TARGETS_CHANGED",

    "BEGIN_PLAYER_TURN",
    "HAND_DRAWN",
    "END_PLAYER_TURN",
    "BEGIN_TURN",

    "BEGIN_ENEMY_TURN",
    "END_ENEMY_TURN",
    "END_TURN",

    "START_RESOLVE",
    "PRE_RESOLVE",
    "POST_RESOLVE",
    "END_RESOLVE",

    "CALC_PERSUASION",          -- Calculating the min/max persuasion range
    "CALC_MONEY_COST",
    "CALC_RESTORE_RESOLVE",
    "CALC_ACTION_COST",
    "CALC_ACTIONS_PER_TURN",
    "CALC_COMPOSURE_DECAY",
    "CALC_TURN_DRAW",  -- Event for calculating the number of cards drawn on turn
    "CALC_BOOLEAN",     -- Query for a boolean value (eg: is this "mod behaviour" enabled?)
    "CALC_CARD_DRAW", -- How many cards can be drawn.
    "CALC_TARGET_MOD", -- Event for calculating a card's target_mod.

    "ATTACK_RESOLVE",
    
    "DELTA_ACTION_COUNT",
    "DELTA_RESOLVE",
    "RESTORE_RESOLVE",
    "SPLASH_RESOLVE",
    "SHIELD_STATUS_CHANGE",
    "DELTA_MONEY",
    "DELTA_COMPOSURE",
    "CALC_DELTA_COMPOSURE",

    "MODIFIER_ADDED",
    "MODIFIER_REMOVED",
    "MODIFIER_CHANGED",
    "MODIFIER_TRIGGERED",
    "UPDATE_MODIFIER_ICON",
    "UPDATE_COIN",
    "GRAFT_TRIGGERED",
    "REFRESH_GRAFTS",

    "DRAW_CARD",
    "CARD_DEALT",
    "CARD_STOLEN",
    "CARD_MOVED",
    "CARD_CHANGED",
    "CARD_DISCARDED",
    "CARD_EXPENDED",
    "CARD_TRIGGERED_PRE",
    "CARD_TRIGGERED_POST",
    "CARD_IMPROVISED",
    "CARD_UPGRADED",
    "CARDS_INCEPTED",
    "CARDS_SWAPPED",

    "DECK_TOO_LARGE",

    "QUIP",
    "DIALOG",
    "CUSTOM",
    "FLOURISH_CHANGED",
}

local STATUS = MakeEnum{
    "INACTIVE", -- Engine hasn't been started yet.
    "ACTIVE", -- Currently executing.
    "PLAY_CARD", -- Player is choosing a card from hand to play.
    "TARGET_CARD", -- Player is choosing a target for a card
    "CHOOSE_CARDS", -- Player is choosing card(s), such as what to expend/etc.
    "SYNC", -- Waiting for UI to sync.
    "UI", -- Waitinf for some kind of custom ui.
}

NEGOTIATION_FLAGS, NEGOTIATION_FLAGS_STRINGS = MakeBitField
{
    "HOSTILE", -- this is a hostile negotiation!  Immediately applies Hostile.
    "NO_BACKUP",
    "INTIMIDATION",
    "PROVOCATION",
    "NO_BYSTANDERS", -- No bystanders will join
    "NO_AUTOFAIL", -- this negotiation is disabled if you have 0 resolve.
    "ALLY_GRAB", --special negotiation with special win conditions
    "ALLY_SCARE", --special negotiation with special win conditions
    "NO_CORE_RESOLVE",
    "NO_IMPATIENCE",
    "WORDSMITH",
    "NO_LOOT",
}


local CARD_FLAGS, CARD_FLAG_BITSTRINGS = MakeBitField
{
    -- Cannot be played.
    "UNPLAYABLE",
    -- Automatically played at end of turn.
    "AUTOPLAY",
    -- "one-shot": Moved into the trash deck once played.
    "EXPEND",
    -- "multi-shot": Stays in hand when resolved.
    "MULTISHOT",
    -- Destroys this card, permanently (eg. from your meta deck)
    "CONSUME",
    -- Hostility card: will trigger the Hostile modifier.
    "HOSTILE",
    -- Bribe card: has a monetary cost associated with it.
    "BRIBE",
    -- This card's resolve cost is variable/calculated dynamically.
    -- Shows "X" in place of the resolve cost.
    "VARIABLE_COST",
    -- Costs 0 until played.
    "FREEBIE",
    -- Extend card draw by 1 when drawn.
    "REPLENISH",
    -- This card is an upgraded version
    "UPGRADED",
    -- This is an opponent's card.
    "OPPONENT",
    -- This card was incepted
    "INCEPTED",
    -- This card is not removed from your hand at the end of your turn
    "STICKY",
    -- This card doesn't upgrade but triggers an event when ready to upgrade
    "HATCH",
    -- Costs health to remove from deck
    "PARASITE",
    "FLOURISH",
    "SLEEP_IT_OFF",
    "BURNOUT",

    -- The following flags are card type tags for applying various card rules.
    "DIPLOMACY", "MANIPULATE", "BYSTANDER", "ITEM", "STATUS", "SPECIAL",

    "AMBUSH",
    "ONE_OF_A_KIND",
    "DUD",
}

local CARD_FLAG_STRINGS =
{
    [CARD_FLAGS.UNPLAYABLE] = "Unplayable",
    -- [CARD_FLAGS.AUTOPLAY] = "Autoplay",
    [CARD_FLAGS.EXPEND] = "Expend",
    [CARD_FLAGS.MULTISHOT] = "Multishot",
    [CARD_FLAGS.CONSUME] = "Destroy",
    [CARD_FLAGS.AMBUSH] = "Ambush",
    [CARD_FLAGS.PARASITE] = "Parasite",
    [CARD_FLAGS.FLOURISH] = "Flourish",
    [CARD_FLAGS.STICKY] = "Sticky",
    [CARD_FLAGS.SLEEP_IT_OFF] = "Sleep It Off",
    [CARD_FLAGS.BURNOUT] = "Burnout",
    [CARD_FLAGS.DUD] = "Dud",
}

local CARD_FLAG_DETAILS =
{
    [CARD_FLAGS.UNPLAYABLE] = "This card cannot be played.",
    [CARD_FLAGS.EXPEND] = "Remove this card for the rest of this negotiation.",
    [CARD_FLAGS.MULTISHOT] = "This card can be played multiple times per turn.",
    [CARD_FLAGS.CONSUME] = "Remove this card permanently.",
    -- [CARD_FLAGS.AUTOPLAY] = "This card cannot be played, and resolves automatically at the end of your turn.",
    [CARD_FLAGS.AMBUSH] = "Automatically placed in your hand at the start of negotiation.",
    [CARD_FLAGS.ONE_OF_A_KIND] = "Your deck can only contain 1 copy of this card.",
    [CARD_FLAGS.PARASITE] = "Removing this card from your deck lowers your maximum health by 10.",
    [CARD_FLAGS.FLOURISH] = "This is a special attack that will {EXPEND} and {BURNOUT}.",
    [CARD_FLAGS.STICKY] = "This card is not discarded at the end of your turn.",
    [CARD_FLAGS.SLEEP_IT_OFF] = "This card is removed upon sleep.",
    [CARD_FLAGS.BURNOUT] = "If this card is in your hand at the end of your turn, {EXPEND} it.",
    [CARD_FLAGS.DUD] = "Created when trying to improvise a card type you do not have.",
}

-- Set of flags constituting 'card type'.
CARD_TYPE_FLAGS = CARD_FLAGS.HOSTILE | CARD_FLAGS.DIPLOMACY | CARD_FLAGS.MANIPULATE | CARD_FLAGS.STATUS | CARD_FLAGS.ITEM | CARD_FLAGS.BYSTANDER | CARD_FLAGS.PARASITE

MODIFIER_TYPE = MakeEnum{ "CORE", "ARGUMENT", "INCEPTION", "BOUNTY", "HIDDEN", "PERMANENT" }
MODIFIER_SCALING = MakeEnum{ "LOW", "MED", "HIGH" }

TARGET_FLAG, TARGET_FLAG_STRINGS = MakeBitField{ "CORE", "ARGUMENT", "INCEPTION", "BOUNTY", "INTENT", "PERMANENT" }
TARGET_ANY_RESOLVE = TARGET_FLAG.CORE | TARGET_FLAG.ARGUMENT | TARGET_FLAG.BOUNTY

--------------------------------------------------------------------------------
-- Modifiers follow.

local MODIFIERS = 
{
    HEART =
    {
        name = "Core Argument",
        modifier_type = MODIFIER_TYPE.CORE,
    },

    GRIFTER =
    {
        name = "Sal's Savvy",
        desc = "Gain 1 {COMPOSURE} at the start of your turn.",
        modifier_type = MODIFIER_TYPE.CORE,

        event_handlers =
        {
            [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:DeltaComposure( 1, self )
                end
            end,
        }

    },


    ANIMOSITY = 
    {
        name = "Animosity",
        desc = "If you end the negotiation with Animosity, the opponent's opinion of you will decrease. Remove 1 stack at the start of your turn.",
        modifier_type = MODIFIER_TYPE.PERMANENT,

        event_handlers =
        {
            [ EVENT.BEGIN_PLAYER_TURN ] = function ( self, minigame )
                self.negotiator:RemoveModifier(self, 1)
            end,
        }
    },

    IMPATIENCE = 
    {
        name = "Impatience",
        desc = "{1}'s Intents and Arguments deploy with +{2} damage.\n{1}'s Arguments deploy with +{3} resolve.",
        desc_fn = function( self, fmt_str, minigame )
            return loc.format(fmt_str, loc.cap( self:GetOwnerName() ), self.stacks * self.persuasion_scaling, self.stacks * self.resolve_scaling )
        end,
        modifier_type = MODIFIER_TYPE.PERMANENT,

        resolve_scaling = 1,

        persuasion_scaling = 1,

        current_level = 0,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
        },

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source.owner == self.owner and self.current_level > 0 then
                    if source.modifier_type ~= MODIFIER_TYPE.INCEPTION then
                        persuasion:AddPersuasion( self.current_level * self.persuasion_scaling, self.current_level * self.persuasion_scaling, self )
                    end
                end
            end,

            -- Make sure we don't change damage values in the middle of their turn
            [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                if negotiator ~= self.negotiator then
                    self.current_level = self.stacks
                    minigame:BroadcastEvent( EVENT.CUSTOM, function( panel )
                            panel.opponent_modifiers:UpdatePersuasionLabels()
                        end )
                end
            end,


            [ EVENT.MODIFIER_ADDED ] = function( self, modifier, source )
                if modifier.negotiator == self.negotiator and modifier.modifier_type == MODIFIER_TYPE.ARGUMENT then
                    self.engine:PushPostHandler( function()
                        if self.current_level > 0 then
                            modifier:ModifyResolve(self.resolve_scaling*self.current_level, self)
                        end
                    end )
                end
            end


        }
    },

    FATIGUED =
    {
        name = "Fatigued",
        desc = "Cards no longer gain XP for this negotiation.",

        modifier_type = MODIFIER_TYPE.PERMANENT,

        sound = "event:/sfx/battle/cards/neg/create_argument/fatigue",

        OnInit = function( self, source )
            if self.negotiator then
                if self.negotiator.agent:GetContentID() == "SMITH" then
                    self.icon = engine.asset.Texture( "negotiation/modifiers/fatigued_smith.tex", true )
                elseif self.negotiator.agent:GetContentID() == "ROOK" then
                    self.icon = engine.asset.Texture( "negotiation/modifiers/fatigued_rook.tex", true )
                end
            end
        end,
    },

    INFLUENCE =
    {
        name = "Influence",
		desc = "All Diplomacy cards deal maximum damage.",
        desc_fn = function( self, fmt_str, minigame )
            return loc.format(fmt_str, self.stacks or 1 )
        end,
        sound = "event:/sfx/battle/cards/neg/create_argument/influence",
        feature_desc = "Gain {INFLUENCE {1}}.",

        max_resolve = 2,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_SETTOR,
        },

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if is_instance( source, Negotiation.Card ) and CheckBits( source.flags, CARD_FLAGS.DIPLOMACY ) and source.min_persuasion then
                    persuasion:AddPersuasion(persuasion.max_persuasion - persuasion.min_persuasion, 0, self)
                end
            end
        }

    },

    COOL_HEAD = 
    {
        name = "Cool Head",
        desc = "Apply {COMPOSURE 3} to your {INFLUENCE} at the beginning of your turn.",
        sound = "event:/sfx/battle/cards/neg/create_argument/cool_head",

        max_resolve = 5,

        event_handlers =
        {
            [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                local modifier = self.negotiator:FindModifier("INFLUENCE")
                if negotiator == self.negotiator and modifier then
                    modifier:DeltaComposure( self.stacks * 3, self )
                end
            end,
        }
    },

    IMPENDING_DOOM = 
    {
        name = "Impending Doom",
        desc = "When destroyed, gives {1} {IMPATIENCE 1}. {1} will start the next battle with <#HILITE>{2}</> additional {battle.EXISTENTIAL_CRISIS}.\n\n<#PENALTY>If you lose the negotiation, all bonuses are lost!</>",
        modifier_type = MODIFIER_TYPE.BOUNTY,

        loc_strings =
        {
            SHIELD_DESC = "{SHIELDED}. Destroy or remove <b>Impending Doom</b> to release the shield!",
        },

        desc_fn = function(self, fmt_str)
            return loc.format(fmt_str, self.negotiator:GetName(), self.stacks ) 
        end,
        
        OnInit = function( self, source )
            self:SetResolve( 3 + self.engine:GetDifficulty() + self.stacks*2 )
            AUDIO:PlayEvent("event:/sfx/battle/cards/neg/create_argument/bonus")
            local core = self.negotiator:FindCoreArgument()
            if core then
                core:SetShieldStatus( true, self.def:GetLocalizedString("SHIELD_DESC") )
            end
        end,

        OnBounty = function( self, card )
            if card then
                if self.result_table then
                    table.insert(self.result_table, self.stacks)
                end
            end
            self.negotiator:AddModifier("IMPATIENCE")
        end,

        OnUnapply = function( self, minigame )
            local core = self.negotiator:FindCoreArgument()
            if core then
                core:SetShieldStatus( false)
            end
        end,
    },

    VULNERABILITY = 
    {
        name = "Vulnerability",
        desc = "All resolve loss is increased by {1}. Remove 1 stack at the start of your turn.",
        desc_fn = function( self, fmt_str, minigame )
            return loc.format(fmt_str, self.dmg_buff )
        end,
        sound = "event:/sfx/battle/cards/neg/create_argument/vulnerability",

        dmg_buff = 1,     
        modifier_type = MODIFIER_TYPE.INCEPTION,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
        },

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion, minigame, target )
                if target and target.owner == self.owner then
                    persuasion:AddPersuasion( self.dmg_buff, self.dmg_buff, self )
                end
            end,

            [ EVENT.BEGIN_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self.negotiator:RemoveModifier( self, 1 )
                end
            end,
        }
    },

    DOMINANCE =
    {
        name = "Dominance",
		desc = "Hostility cards deal an additional <#HILITE>{1%+d}</> damage. Remove 1 stack at the start of your turn.",
        desc_fn = function( self, fmt_str, minigame )
            return loc.format( fmt_str, self.stacks or 1 )
        end,
        feature_desc = "Gain {DOMINANCE {1}}.",
        sound = "event:/sfx/battle/cards/neg/create_argument/dominance",

        max_resolve = 2,

        OnStartNegotiation = function( self, caravan, agent )           
            local count = 0
            if agent:HasAspect("intimidated") then
                count = count + agent:GetAspect("intimidated"):GetStacks()
            end

            return count

        end,

        -- This avoids dominance being removed on the start of the same turn that it was created.
        skipped_instant = false,

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
        },

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source.owner == self.owner and is_instance( source, Negotiation.Card ) then
                    if CheckBits( source.flags, CARD_FLAGS.HOSTILE ) then
                        persuasion:AddPersuasion( self.stacks, self.stacks, self )
                    end
                end
            end,

            [ EVENT.BEGIN_PLAYER_TURN ] = function ( self, minigame )
                if self.skipped_instant then
                    self.negotiator:RemoveModifier(self, 1)
                else
                    self.skipped_instant = true
                end
            end,

            [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                if not self.skipped_instant then
                    self.skipped_instant = true
                end
            end
        }
    },
    
    BOUNTY_REWARD = 
    {
        name = "Wanted!",
        modifier_type = MODIFIER_TYPE.BOUNTY,
        desc = "When destroyed, {1}'s core argument takes {2} resolve damage and gains {IMPATIENCE 1}.",
        alt_desc = "When destroyed, {1}'s core argument takes 6 to 9 resolve damage and gains {IMPATIENCE 1}.",
        desc_fn = function(self, fmt_str)
            if self:IsApplied() then
                return loc.format(fmt_str, self:GetOwnerName(), self.stacks * self.bounty_resolve )
            else
                --ew
                return loc.format( (self.def or self):GetLocalizedString("alt_desc"), self:GetOwnerName(), self.stacks * self.bounty_resolve )
            end

        end,

        max_resolve = 2,
        bounty_resolve = 8,
        
        OnBounty = function( self, card )
            self.negotiator:AddModifier("IMPATIENCE")
            self.negotiator:AttackResolve( self.bounty_resolve * self.stacks, self )
        end,
    },

    FLUSTERED =
    {
        name = "Flustered",
        desc = "Reduces damage by {1#percent}. Remove 1 stack at the end of {2}'s turn.",
        desc_fn = function( self, fmt_str, minigame )
            return loc.format( fmt_str, self.percent or .5, self:GetOwnerName() )
        end,
        feature_desc = "{INCEPT} {FLUSTERED {1}}.",

        modifier_type = MODIFIER_TYPE.INCEPTION,

        percent = .33,

        sound = "event:/sfx/battle/cards/neg/create_argument/flustered",

        event_priorities =
        {
            [ EVENT.CALC_PERSUASION ] = EVENT_PRIORITY_ADDITIVE,
        },

        event_handlers =
        {
            [ EVENT.CALC_PERSUASION ] = function( self, source, persuasion )
                if source.owner == self.owner then
                    if source.modifier_type ~= MODIFIER_TYPE.INCEPTION then
                        persuasion:AddPersuasion( - math.round(persuasion.min_persuasion * self.percent), -math.round(persuasion.max_persuasion * self.percent), self )
                    end
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    negotiator:RemoveModifier( self, 1 )
                end
            end,
        },
    },

    FREE_HAND =
    {
        name = "Free Hand",
        desc = "Cards in your hand cost 0 this turn.",
        hidden = true,

        event_handlers = 
        {
            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if self.owner == card.owner and self.engine:GetHandDeck():HasCard( card ) then
                    acc:ModifyValue( 0, self )
                end
            end,

            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    negotiator:RemoveModifier( self, 1 )
                end
            end,
        },
    },
    
    CARD_DRAW =
    {
        name = "Card Draw",
        desc = "Draw <#HILITE>{1}</> extra {1*card|cards} at the start of your turn.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, (self.draw_count or 1)*self.stacks)
        end,

        modifier_type = MODIFIER_TYPE.PERMANENT,

        event_handlers =
        {
            [ EVENT.CALC_TURN_DRAW ] = function( self, acc )
                acc:AddValue( (self.draw_count or 1) * self.stacks, self )
            end
        }
    },

    SMARTS =
    {
        name = "Smarts",
        desc = "Gain an extra action at the start of your turn, then reduce <b>Smarts</b> by 1.",

        max_resolve = 6,
        sound = "event:/sfx/battle/cards/neg/create_argument/smarts",

        event_handlers =
        {
            [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                minigame:ModifyActionCount( 1  )
                self.negotiator:RemoveModifier( self, 1 )
            end,
        }
    },

    FREE_DRAW =
    {
        name = "Free Draw",
        desc =  "Next turn, draw {1} additional {1*card|cards}.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.stacks )
        end,
        feature_desc = "Draw {1} extra {1*card|cards} next turn.",
        hidden = true,

        event_handlers = 
        {
            [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                minigame:DrawCards( self.stacks )
                self.negotiator:RemoveModifier( self )
            end,
        },
    },

    FREE_ACTION =
    {
        name = "Free Action",
        desc = "Gain an extra action next turn.",
        feature_desc = "Gain {1} bonus {1*action|actions} next turn.",

        hidden = true,

        event_handlers =
        {
            [ EVENT.BEGIN_PLAYER_TURN ] = function( self, minigame )
                minigame:ModifyActionCount( self.stacks )
                self.negotiator:RemoveModifier( self )
                AUDIO:PlayEvent( "event:/sfx/battle/cards/neg/gain_action_next_turn" )
            end,
        }
    },

    HOSTILITY =
    {
        name = "Hostility",
		desc = "Can be used in various ways to bully your opponent into submission.",
        feature_desc = "Gain {1} {HOSTILITY}.",
        sound = "event:/sfx/battle/cards/neg/create_argument/hostility",

        max_resolve = 2,
    },

    HEATED = 
    {
        name = "Heated",
        desc = "At the end of {1}'s turn, add 1 stack of <b>Heated</>.",
        alt_desc = "Targets an opponent argument each turn for 1 damage per stack.",
        desc_fn = function( self, fmt_str, minigame )
            local str = loc.format( fmt_str, self:GetOwnerName(), self.stacks)

            if not minigame then
                str = loc.format( "{1}\n\n{2}", (self.def or self):GetLocalizedString("alt_desc"), str )
            end

            return str
        end,
        sound = "event:/sfx/battle/cards/neg/create_argument/heated",

        max_resolve = 3,
        target_enemy = TARGET_ANY_RESOLVE,

        IsValidTarget = function( target )
            return target:GetResolve() ~= nil
        end,

        OnSetStacks = function( self, old_stacks )
            self.min_persuasion, self.max_persuasion = self.stacks, self.stacks
        end,

        OnApply = function( self )
            self:PrepareTurn()
        end,

        event_handlers =
        {
            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ApplyPersuasion()
                    self.negotiator:AddModifier( self, 1 )
                end
            end,
        }
    },

    DISTRACTED =
    {
        name = "Distracted",
		desc = "Whenever you play a card, your core argument takes {1} resolve damage. \nReduce <b>Distracted</b> by 1 whenever you play a card and at the end of your turn.",
        desc_fn = function( self, fmt_str )
            return loc.format(fmt_str, self.damage)
        end,
        sound = "event:/sfx/battle/cards/neg/create_argument/distracted",

        modifier_type = MODIFIER_TYPE.INCEPTION,
        damage = 1,
        
        event_handlers =
        {
            [ EVENT.POST_RESOLVE ] = function(self, minigame, card)
                if self.owner == card.owner then
                    self.negotiator:AttackResolve( self.damage, self )
                    self.negotiator:RemoveModifier( self, 1 )
                end
            end,

            [ EVENT.END_PLAYER_TURN ] = function( self, minigame )
                self.negotiator:RemoveModifier( self, 1 )
            end
        }
    },

    DOUBT =
    {
        name = "Doubt",

        desc = "Targets a friendly argument each turn for 1 damage per stack of <b>Doubt</b>.",
        no_damage_tt = true,
        feature_desc = "{INCEPT} {DOUBT {1}}.",
        sound = "event:/sfx/battle/cards/neg/create_argument/doubt",

        min_persuasion = 1,
        max_persuasion = 1,
        modifier_type = MODIFIER_TYPE.INCEPTION,
        target_self = TARGET_ANY_RESOLVE,

        OnApply = function( self )
            if self.anti_negotiator and self.anti_negotiator:IsPlayer() then
                self:CheckAchievement( 0 )
            end
        end,

        CheckAchievement = function( self, old_stacks )
            local def = Content.FindAchievement( "soluble_fish" )
            if def then
                local total = self.negotiator:GetModifierStacks( "DOUBT" ) -- Across all stacks.
                local prev_total = total - (self.stacks - old_stacks)
                if prev_total < def.count and total >= def.count then
                    TheGame:UnlockAchievement( def.id )
                end
            end
        end,

        OnSetStacks = function( self, old_stacks )
            self.min_persuasion = math.max( self.min_persuasion, self.stacks )
            self.max_persuasion = self.min_persuasion

            if self.anti_negotiator and self.anti_negotiator:IsPlayer() then
                self:CheckAchievement( old_stacks )
            end
        end,

        event_handlers =
        {
            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    self:ApplyPersuasion()
                end
            end,
        },
    },

    WARY =
    {
        name = "Wary",
        desc = "Manipulate cards have 1 additional cost.",

        max_resolve = 1,
        sound = "event:/sfx/battle/cards/neg/create_argument/wary",

        OnInit = function( self )
            if self.engine then
                self:SetResolve( self.engine:GetDifficulty() )
            end
        end,

        event_handlers =
        {
            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    -- self.negotiator:RemoveModifier( self, 1 )
                end
            end,

            [ EVENT.CALC_ACTION_COST ] = function( self, acc, card, target )
                if card:IsFlagged( CARD_FLAGS.MANIPULATE ) then
                    acc:AddValue( 1, self )
                end
            end,
        },
    },

    BAIT =
    {
        name = "Bait",
        desc = "Must be targeted before anything else. Gains 2 resolve for each stack of <b>Bait</b>.",
        max_resolve = 2,
        sound = "event:/sfx/battle/cards/neg/create_argument/entrapment",
        force_target = true,

        OnSetStacks = function( self, old_stacks )
            local delta = self.stacks - math.max( 1, old_stacks )
            self:ModifyResolve( delta * 2 )
        end,

        CanPlayCard = function( self, source, engine, target )
            -- Only verify forced targets if you are not targetting yourself
            if source:IsAttack() and target:GetNegotiator() == self.negotiator then
                if source.modifier_type == MODIFIER_TYPE.INCEPTION or source:GetNegotiator() ~= self.negotiator then
                    if not target.force_target then
                        return false, loc.format( LOC "CONVO_COMMON.MUST_TARGET", self:GetName() )
                    end
                end
            end

            return true
        end,
    },

    METTLE_BOUNTY =
    {
        name = "Mettlesome",
        desc = "If destroyed, gain <#HILITE>{1}</> {METTLE}. At the end of {2}'s turn, remove 1 <b>Mettlesome</>.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.stacks, self:GetOwnerName() )
        end,

        OnInit = function( self, source )
            if self.engine then
                local resolve = 3 + self.engine:GetDifficulty()*2
                self:SetResolve( resolve )
            end
        end,

        modifier_type = MODIFIER_TYPE.BOUNTY,
        
        OnBounty = function( self )
            self.engine:GetPlayerNegotiator():AddModifier("METTLE_BANK", self.stacks, self)
            --TheGame:GetGameProfile():AddMettlePoints(self.engine:GetPlayer():GetContentID(), self.stacks, "METTLE_BOUNTY")
        end,

        event_handlers =
        {
            [ EVENT.END_TURN ] = function( self, minigame, negotiator )
                if negotiator == self.negotiator then
                    negotiator:RemoveModifier( self, 1 )
                end
            end,
        },
    },

    METTLE_BANK =
    {
        name = "Bonus Mettle",
        desc = "You will gain {1} Mettle if you win this negotiation.",
        desc_fn = function( self, fmt_str )
            return loc.format( fmt_str, self.stacks )
        end,
        modifier_type = MODIFIER_TYPE.PERMANENT,
    },    
}

---------------------------------------------------------
-- Validate modifiers.

for i, id, def in sorted_pairs( MODIFIERS ) do
    Content.AddNegotiationModifier( id, def )
end
--strictify( MODIFIERS )

-------------------------------------------------------------
-- Generate features, a superset of CARD_FLAGS and CONDITIONS.


for i, name in ipairs( CARD_FLAG_BITSTRINGS ) do
    local bit = 1 << (i-1)
    local details = CARD_FLAG_DETAILS[ bit ]
    if details then
        local feature = 
        {
            name = CARD_FLAG_STRINGS[ bit ],
            desc = details,
        }

		local def = NegotiationFeatureDef(name, feature)
        Content.AddNegotiationCardFeature(name, def)
        Content.AddNegotiationCardFeature(bit, def)
    end
end

local FEATURES =
{
	INCEPT = {
		name = "Incept",
		desc = "This effect is added to the opponent's arguments.",
	},

    PREPARED = 
    {
        name = "Prepared",
        desc = "A card is <b>Prepared</> if it is the left-most card in your hand.",
    },

    PREPARE = {
        name = "Prepare",
        desc = "Choose a card in your hand and move it to the prepared (left-most) position.",
    },

    REPLENISH =
    {
        name = "Replenish",
        desc = "When this card is drawn, draw another card.",
    },
	
    COMPOSURE =
    {
        name = "Composure",
        desc = "Reduce resolve loss for one turn.",
        feature_desc = "Gain {1} {COMPOSURE}.",
        feature_desc_fn = function( self, fmt_str, num, engine, card )
            if card.target_self ~= nil and card.target_self ~= TARGET_FLAG.CORE then
                return loc.format( LOC "UI.CARDS.APPLY_COMPOSURE", card:CalculateComposureText( num ))
            else
                return loc.format( fmt_str, card:CalculateComposureText( num ))
            end
        end,


        apply = function( self, engine, resolve_card, targets, stacks )
            if #targets == 0 then
                resolve_card:GetNegotiator():DeltaComposure( stacks, resolve_card )
            else
                for i, target in ipairs( targets ) do
                    if not target or target.DeltaComposure == nil then
                        LOGWARN( "Bad target for composure: %s", tostring(target))
                    else
                        target:DeltaComposure( stacks, resolve_card )
                    end
                end
            end
        end,
    },

    EVOKE =
    {
        name = "Evoke",
        desc = "Once per turn, play this card for free from wherever it is when the specified requirements are met.",
    },

    DISMISS = 
    {
        name = "Dismiss",
        desc = "Prevent an opponent's intent from occuring.",
        feature_desc = "{DISMISS} one opponent Intent.",
        apply = function( self, engine, resolve_card, targets, stacks )
            for i, target in ipairs( targets ) do
                if is_instance( target, Negotiation.Card ) then
                    target:GetNegotiator():DismissIntent( target )
                end
            end
        end,
    },

    HATCH_X =
    {
        name = "<p img='icons/ic_hatch.tex' scale=1.0> Hatch",
        desc = "When this card's effect is triggered the specified number of times, it is destroyed and a special event occurs. The hatch counter is not changed while fatigued.",
    },

    IMPROVISE =
    {
        name = "Improvise",
        desc = "Choose 1 of 3 cards to put into your hand.",
    },

    IMPROVISE_PLUS =
    {
        name = "Improvise+",
        desc = "Choose 1 of 5 cards to put into your hand.",
    },

    SNAILS =
    {
        name = "<p img='icons/ic_coin_snails.tex' scale=1.0>",
        posted_feature = "HEADS_AND_SNAILS",
    },

    HEADS =
    {
        name = "<p img='icons/ic_coin_heads.tex' scale=1.0>",
        posted_feature = "HEADS_AND_SNAILS",
    },

    HEADS_AND_SNAILS =
    {
        name = "Heads <p img='icons/ic_coin_heads.tex' scale=1.0> and Snails <p img='icons/ic_coin_snails.tex' scale=1.0>",
        desc = "Heads and Snails are the two sides of your lucky coin.",
    },

    SHIELDED =
    {
        name = "Shielded",
        desc = "This argument cannot be attacked.",
    },

    STACKING =
    {
        name = "Stacking",
        desc = "Whenever a copy of this card is added to your deck, the two cards are merged into one card that gains a bonus."
    },

    METTLE =
    {
        name = "Mettle",
        desc = "Accumulate Mettle to purchase bonuses that permanently upgrade your character."
    }
}

for id, data in pairs(FEATURES) do
	local def = NegotiationFeatureDef(id, data)
	Content.AddNegotiationCardFeature(id, def)
end

return
{
    IS_NEGOTIATION = true,
    
    RESULT = RESULT,

    CARD_FLAGS = CARD_FLAGS,
    CARD_FLAG_BITSTRINGS = CARD_FLAG_BITSTRINGS,
    CARD_FLAG_STRINGS = CARD_FLAG_STRINGS,
    CARD_FLAG_DETAILS = CARD_FLAG_DETAILS,
    EVENT = EVENT,
    STATUS = STATUS,
    MODIFIER_TYPE = MODIFIER_TYPE,

    GetFeature = Content.GetNegotiationCardFeature,
}
