-- generates UI info a from raw (save/loadable) notification data.
local function CreateNotification( data )
    local def = NOTIFY[ data.id ]
    local notification = shallowcopy( def )
    notification.data = data
    if def.FormatNotification then
        def:FormatNotification( notification, table.unpack( data ))
    end
    return notification
end

------------------------------------------------------------------
-- Save/loadable queue of logged in-game events.

local Notifications = class("Notifications")

function Notifications:init()
    self.items = {}
    self.banners_enabled = true
end

function Notifications:Save()
    return
    {
        items = self.items,
        -- suppressed_events is not save/loadable state
    }
end

function Notifications:Load()
    self.banners_enabled = true
end

function Notifications:IsEnabled( notification_def )
    if notification_def then
        return self.suppressed_events and table.arrayfind( self.suppressed_events, notification_def ) ~= nil
    else
        return self.suppress_all_events ~= true
    end
end

function Notifications:EnableNotifications( on, ... )
    local n = select( "#", ... )
    if n == 0 then
        self.suppress_all_events = not on
    else
        if self.suppressed_events == nil then
            self.suppressed_events = {}
        end
        for i = 1, n do
            local notification_def = select( i, ... )
            if on then
                table.arrayremove( self.suppressed_events, notification_def )
            else
                table.insert( self.suppressed_events, notification_def )
            end
        end
    end
end


function Notifications:BannersEnabled()
    return self.banners_enabled
end

function Notifications:SetBannersEnabled(val)
    self.banners_enabled = val
end

function Notifications:LogNotification( notification_def, ... )
    
    if self.suppress_all_events or (self.suppressed_events and table.arrayfind( self.suppressed_events, notification_def )) then
        return
    end

    local now = Now()
    local gs = TheGame:GetGameState()

    local data = { ... }
    data.id = notification_def.id
    data.time = now
    data.playtime = gs:GetPlayTime()

    local def = NOTIFY[ data.id ]
    if def.OnCreate then
        def:OnCreate( data, gs )
    end

    local notification = CreateNotification( data )

    TheGame:GetEvents():BroadcastEvent( "notification_logged", notification )

    table.insert( self.items, data )
end

function Notifications:Clear()
    table.clear( self.items )
end

function Notifications:GetNotification(idx)
    local item = self.items[ idx ]
    if item then
        return CreateNotification( item )
    end
end


function Notifications:GetCount()
    return #self.items
end
