local hands = 
{
    hands_human_dredger_female = "anim/hand_human_male_glove_dredger.zip",
    hands_human_dredger_male = "anim/hand_human_male_glove_dredger.zip",

    hands_human_male = "anim/hand_human_male.zip",
    hands_human_male_glove = "anim/hand_human_male_glove.zip",
    hands_human_male_glove_sal = "anim/hand_human_male_glove_sal.zip",
    hands_human_male_armour = "anim/hand_human_male_armour.zip",
    hands_human_female = "anim/hand_human_female.zip",
    hands_human_female_glove = "anim/hand_human_female_glove.zip",
    hands_automech = "anim/hand_automech.zip",
    hands_kra = "anim/hand_kra.zip",
    hands_kra_female = "anim/hand_kra_female.zip",
    hands_kra_female_glove = "anim/hand_kra_female_glove.zip",
    hand_kra_male_glove = "anim/hand_kra_male_glove.zip",
    hand_jarackle_male_glove = "anim/hand_jarackle_male_glove.zip",
    hand_jarackle_female_glove = "anim/hand_jarackle_female_glove.zip",
    hand_jarackle_male = "anim/hand_jarackle_male.zip",
    hand_jarackle_female = "anim/hand_jarackle_female.zip",
    hand_jarackle_male_armour = "anim/hand_jarackle_male_glove_armour.zip",
}

local function PrecacheAnims()
    for k,v in pairs(hands) do
        CacheAnim(v)
    end
end

GLOVES = MakeEnum{"NONE", "SIMPLE", "ARMOURED", "DREDGER"}

local function GetHands(species, gender, glovetype)
    if species == SPECIES.BEAST then return end

    local hand_table = hands    

    if species == SPECIES.JARACKLE then
        if glovetype == GLOVES.SIMPLE then
            return GetAnim(gender == GENDER.MALE and hand_table.hand_jarackle_male_glove or hand_table.hand_jarackle_female_glove)
        elseif glovetype == GLOVES.ARMOURED then
            return GetAnim(hand_table.hand_jarackle_male_armour)
        else
            return GetAnim(gender == GENDER.MALE and hand_table.hand_jarackle_male or hand_table.hand_jarackle_female)
        end
    elseif species == SPECIES.KRADESHI or species == SPECIES.SHROKE then
        if gender == GENDER.MALE then
            if glovetype == GLOVES.SIMPLE then
                return GetAnim(hand_table.hand_kra_male_glove)
            else
                return GetAnim(hand_table.hands_kra)
            end
        else
            if glovetype == GLOVES.SIMPLE then
                return GetAnim(hand_table.hands_kra_female_glove)
                
            else
                return GetAnim(hand_table.hands_kra_female)
            end
        end
    else
        if gender == GENDER.MALE then
            if glovetype == GLOVES.SIMPLE then
                return GetAnim(hand_table.hands_human_male_glove)
            elseif glovetype == GLOVES.DREDGER then
                return GetAnim(hand_table.hands_human_dredger_male)
            elseif glovetype == GLOVES.ARMOURED then
                return GetAnim(hand_table.hands_human_male_armour)
            else
                return GetAnim(hand_table.hands_human_male)
            end
        else
            if glovetype == GLOVES.SIMPLE then
                return GetAnim(hand_table.hands_human_female_glove)
            elseif glovetype == GLOVES.DREDGER then
                return GetAnim(hand_table.hands_human_dredger_female)
            elseif glovetype == GLOVES.ARMOURED then
                return GetAnim(hand_table.hands_human_male_armour)
            else
                return GetAnim(hand_table.hands_human_female)
            end
        end
    end

    return GetAnim(hand_table.hands_human_male)
end


local function SetColour(anim_model, channel, colour)
    local r,g,b,a = HexColour(colour)
    anim_model:SetLayerTint(channel, r,g,b,a)
end




local function ApplyColours( model, clr )
    for k, v in pairs(clr) do
        SetColour(model, k, v )    
    end
end

local function ApplyAgentColours( model, agent )
    if agent:GetFaction() and agent:GetFaction():GetCharacterColours() then
        ApplyColours( model, agent:GetFaction():GetCharacterColours() )
    end

    if agent.skin_colour then
        SetColour( model, "skin", agent.skin_colour | 0x000000FF )
    end
    if agent.hair_colour then
        SetColour( model, "hair", agent.hair_colour )
    end
end

local function GetCharacterScale( char )
    local scale = 1.0
    local scale_chart = SPECIES_PROPS[ char.species ].scale_chart
    -- Species and gender scaling
    scale = scale * (scale_chart and scale_chart[char.gender] or 1)
    -- Per-character scaling
    scale = scale * (char.scale_adjust or 1.0)
    return scale
end


local function SetCharacterData(anim_model, char, combat, scale )
    anim_model:Reset()
    local build = char:GetBuild()

    
    local build_file = build.build
    local build_anim = GetAnim(build_file)
    anim_model:SetBuild(build_anim)
    anim_model:AddOverrideFile(build_anim:GetFileName()) 

    if not build.no_hand_swap then
        if build.hand_build_override then
            GetAnim(build.hand_build_override)
            anim_model:AddOverrideFile(build.hand_build_override) 
        else
            local hands = GetHands(char.species, char.gender, build.gloves)
            if hands then
                anim_model:AddOverrideFile(hands:GetFileName()) 
            end
        end
    else
        --print ("NO HAAAAAAANDS!")
    end

    -- pass the glow colour through to the anim  model:
    if build.glow then
        if build.glow.colour then
            anim_model:SetGlowColour(build.glow.colour);
            anim_model:SetGlowThresholdAndAmount(build.glow.threshold or 0.02, build.glow.bloom or 1.0);
        end

    end

    if combat then
        for k,v in pairs(build.combatanims) do
            anim_model:AddAnims(GetAnim(v))
            anim_model:AddOverrideFile(v) 
        end
        if char.combat_anims then
            for k,v in pairs(char.combat_anims) do
                anim_model:AddAnims(GetAnim(v))
                anim_model:AddOverrideFile(v) 
            end
        end

        -- We sometimes set UI animatedcharacters to fight mode, but they don't actually need the depth state, like the ones in combat
        if anim_model.SetDepthState then anim_model:SetDepthState(DEPTH_TEST.LESSEQUAL, true) end
    else

        if build.dialoganims then
            for k,v in pairs(build.dialoganims) do
                anim_model:AddAnims(GetAnim(v))
                anim_model:AddOverrideFile(v) 
            end
        end

        for k,v in pairs(build.anims) do
            anim_model:AddAnims(GetAnim(v))
            anim_model:AddOverrideFile(v) 
        end
    end

    if char.anims then
        for k,v in pairs(char.anims) do
            anim_model:AddAnims(GetAnim(v))
            anim_model:AddOverrideFile(v) 
        end
    end

    if not build.force_hat and (char.nohat or SPECIES_PROPS[ char.species ].nohat) then
        anim_model:HideSymbol("hat_swap")
    end

    if not char.has_hood and not SPECIES_PROPS[ char.species ].has_hood then
        anim_model:HideSymbol("hood")
    end

    if build.hide_symbols then
        for k,v in ipairs(build.hide_symbols) do
            anim_model:HideSymbol(v)

        end
    end

    if build.colour then
        ApplyColours( anim_model, build.colour )
    end

    ApplyAgentColours( anim_model, char )
    
    scale = scale * GetCharacterScale( char )
    anim_model:SetScale( scale )
    --anim_model:ClearOverrideSlot("HEAD_CMP") 
    
    local head = char.head and Content.GetCharacterHead(char.head)

    if head and head.nohat and not build.force_hat then
        anim_model:HideSymbol("hat_swap")
    end


    if build_anim and head and head.hat_hides and (build_anim:HasSymbol("hat") or build_anim:HasAnim("hat_swap") ) then
        for k,v in ipairs(head.hat_hides) do
            anim_model:HideSymbol(v)
        end
    end


    if head and head.hides then
        for k,v in ipairs(head.hides) do
            anim_model:HideSymbol(v)
        end
    end

    

    local head_anim_to_use
    if head then
        head_anim_to_use = head.anim
    end 

    if head_anim_to_use then

        local head_anim = GetAnim(head_anim_to_use)
        anim_model:AddOverrideFile(head_anim:GetFileName()) 
        if anim_model.AddSymbolAnim then

            for k = 0, head_anim:GetNumAnims() do
                local anim_name, root_sym = head_anim:GetAnimName(k)
                if root_sym == "eye_left_swap_blink" then
                    anim_model:AddSymbolAnim("eye_left_swap",anim_name, head_anim, k )
                elseif root_sym == "eye_right_swap_blink" then
                    anim_model:AddSymbolAnim("eye_right_swap",anim_name, head_anim, k )
                elseif root_sym == "mouth_swap_talk" then
                    anim_model:AddSymbolAnim("mouth_swap",anim_name, head_anim, k )
                end
            end
        end
    end

end


local function LoadAgentAnims(char, combat, cache )
    
    local build = char:GetBuild()
    local build_file = build.build
    local anims = {}

    table.insert(anims, build_file)

    if not build.no_hand_swap then
        if build.hand_build_override then
            table.insert(anims, build.hand_build_override)
        end
    end

    if combat then
        for k,v in pairs(build.combatanims) do
            table.insert(anims, v)
        end
        if char.combat_anims then
            for k,v in pairs(char.combat_anims) do
                table.insert(anims, v)
            end
        end
    else
        for k,v in pairs(build.anims) do
            table.insert(anims, v)
        end
    end

    if char.anims then
        for k,v in pairs(char.anims) do
            table.insert(anims, v)
        end
    end

    local head = char.head and Content.GetCharacterHead(char.head)
    local head_anim_to_use = head and head.anim

    if head_anim_to_use then
        table.insert(anims, head_anim_to_use)
    end

    for k,v in pairs(anims) do
        if cache then
            CacheAnim(v)
        else
            GetAnim(v)
        end
    end
end

return 
{
    PrecacheAnims = PrecacheAnims,
    LoadAgentAnims = LoadAgentAnims,
    SetCharacterData = SetCharacterData,
    
    SetColour = SetColour,
    GetCharacterScale = GetCharacterScale,
}