Party = class("Party")

function Party:init(leader)
    self.members = {}
end

function Party:AddMember(member, member_type)
    assert( not table.arrayfind(self.members, member))
    assert(member:GetAspect('party_member') == nil)

    table.insert(self.members, member)
    member:AddAspect('party_member', self, member_type or PARTY_MEMBER_TYPE.CREW)

    self:NamePets()

    if self:GetLeader() ~= member then
        if not member:IsWithAgent( self:GetLeader() ) then
            member:MoveToAgent( self:GetLeader() )
        end
    end
end

function Party:GetLeader()
    for k,agent in pairs(self.members) do
        if agent.party_member and agent.party_member:IsLeader() then
            return agent
        end
    end
end

function Party.NotLeader( agent )
    return agent:IsCrewMember() and not agent:IsLeader()
end

function Party:GetRandomMember( fn )
    local t = ObtainWorkTable()
    table.arrayadd( t, self.members )

    local agent
    while #t > 0 do
        agent = table.remove( t, math.random( #t ))
        if fn == nil or fn( agent ) then
            break
        else
            agent = nil
        end
    end
    ReleaseWorkTable( t )
    return agent
end

function Party:NamePets()
    local leader = self:GetLeader()
    if leader and leader:IsSentient() then
        for i, member in ipairs( self.members ) do
            if not member:IsSentient() and member:GetName() == nil then
                member:GenerateRandomName()
            end
        end
    end
end

function Party:Disband()
    local leader = self:GetLeader()    
    for k,agent in pairs(self.members) do
        agent:RemoveAspect( "party_member" )
    end
    table.clear(self.members)
end

function Party:RemoveMember(member)
    if table.arrayremove(self.members, member) then
        if member.party_member:IsLeader() then
            -- reassign leadership to next crew member, if possible.
            for i, agent in ipairs( self.members ) do
                if agent.party_member:GetMemberType() == PARTY_MEMBER_TYPE.CREW then
                    agent.party_member:SetMemberType( PARTY_MEMBER_TYPE.LEADER )
                    self:NamePets()
                    break
                end
            end
        end
        member:RemoveAspect("party_member")
    end
end

function Party:HasMember( agent )
    return table.arrayfind( self.members, agent ) ~= nil
end

function Party:GetMemberType( member_type )
    for k,agent in pairs(self.members) do
        if agent.party_member:GetMemberType() == member_type then
            return agent
        end
    end
end

function Party:Members()
    return ipairs( self.members )
end

function Party:GetMembers()
    return self.members
end

function Party:MoveToLeader()
    local leader = self:GetLeader()

    for i, member in ipairs( self.members ) do
        if member ~= leader then
            member:MoveToAgent( leader )
        end
    end
end

function Party:MoveToLimbo()
    for i, member in ipairs( self.members ) do
        member:MoveToLimbo()
    end
end

function Party:MoveToLocation( location )
    for i, member in ipairs( self.members ) do
        member:MoveToLocation( location )
    end
end

function Party:Save()
    return self
end

