local DataDef = class("DataDef")

function DataDef:init()
end

function DataDef:HarvestStrings(t)
	assert(self.id)

	if self.data.name then
		local key = self:GetLocNameKey()
		t[key] = self.data.name
	end

	if self.data.desc then
		local key = self:GetLocDescKey()
		t[key] = self.data.desc
	end

    if self.data.title then
        local key = self:GetLocTitleKey()
        t[key] = self.data.title
    end
end

function DataDef:SetModID( mod_id )
    self.mod_id = mod_id
end

function DataDef:GetModID()
    return self.mod_id
end

function DataDef:GetSubStringID(base, id, key)
    return loc.format( "{1}.{2}.{3}.{4}", self:GetLocPrefix(), base, id, key)
end

function DataDef:GetLocalizedName()
	local result = nil
	if self.data.name then
		result = LOC(self:GetLocNameKey())
	end
	return result
end

function DataDef:GetLocalizedDesc()
    local result = nil
    if self.data.desc then
        result = LOC(self:GetLocDescKey())
    end
    return result
end


function DataDef:GetLocalizedTitle()
	local result = nil
	if self.data.title then
		result = LOC(self:GetLocTitleKey())
	end
	return result
end

function DataDef:GetLocNameKey()
	return self:GetLocPrefix() .. ".NAME"
end

function DataDef:GetLocDescKey()
    return self:GetLocPrefix() .. ".DESC"
end



function DataDef:GetLocTitleKey()
	return self:GetLocPrefix() .. ".TITLE"
end


--these are non-serializing data structures for looking up data about the act/etc
local PlayerBackground = class("PlayerBackground", DataDef)
function PlayerBackground:init(data)
    self.id = data.id
    self.data = data
    self.acts = {}

    assert( self.id ~= nil, "Missing player background identifier" )
    assert( data.player_agent, "missing player_agent:"..tostring(self.id))

    assert( self.data.name, "Missing background name" )
    assert( self.data.title, "Missing background title" )
    assert( self.data.desc, "Missing background desc" )

    assert( self.data.advancement, "Missing prestige table 'advancement'")
end 

function PlayerBackground:AddAct(act_data)
    local act = PlayerAct(self, act_data)
    table.insert(self.acts, act)
    return self
end

function PlayerBackground:AddClonedAct( act, new_id )
    assert( is_instance( act, PlayerAct ))
    assert( new_id )

    -- Clone the internal act data, and assign it a new ID.
    local new_act_data = deepcopy( act.data )
    new_act_data.id = new_id

    self:AddAct( new_act_data )
    local new_act = self.acts[ #self.acts ]

    -- Add to content.
    AddPlayerActData( new_act )
    return new_act
end

function PlayerBackground:GetLocPrefix()
	return "PLAYER_BACKGROUND." .. string.upper(self.id)
end

function PlayerBackground:CreateAgent()
    local skin_table = self.data.player_agent_skin and Content.GetCharacterSkin( self.data.player_agent_skin )
    return Agent( self.data.player_agent, skin_table )
end

----------------------------

local PlayerAct = class("PlayerAct", DataDef)

function PlayerAct:init(background, data)
    self.background = background
    self.id = data.id
    self.data = data

    assert_warning( self.data.game_type, tostring(self.id))
end

function PlayerAct:GetLocPrefix()
	return "PLAYER_ACT." .. string.upper(self.id)
end

function PlayerAct:SpawnQuests( options )
    TheGame:GetGameState():GetNotifications():EnableNotifications( false )

    local main_quest = options.main_quest or self.data.main_quest
    if main_quest then
        local quest, err = QuestUtil.SpawnQuest(main_quest, options.main_quest_params)
        if quest then
            TheGame:GetGameState():SetMainQuest(quest)
        else
            LOGWARN( "FAILED TO SPAWN: %s (%s)", quest, err )
        end

    end

    local quests = options.quests or self.data.quests
    if quests then
        for k,v in ipairs(quests) do
            local ok, err = QuestUtil.SpawnQuest(v)
            if not ok then
                LOGWARN( "FAILED TO SPAWN: %s (%s)", v, err )
            end
        end
    end

    TheGame:GetGameState():GetNotifications():EnableNotifications( true )
end


function PlayerAct:ApplyNewPlayerConfig( game_state, config_options)
    local player = game_state:GetPlayerAgent()
    local negotiation_deck = Content.GetDeck(config_options and config_options.starting_negotiation_deck) or Content.GetNegotiationDecks( player:GetContentID() )[1]
    local battle_deck = Content.GetDeck(config_options and config_options.starting_battle_deck) or Content.GetBattleDecks( player:GetContentID() )[1]

    if negotiation_deck then
        player.negotiator:AddDeck( negotiation_deck )
    end

    if battle_deck then
        player.battler:AddDeck( battle_deck )
    end

    local outfit = config_options and config_options.outfit and Content.GetOutfit(config_options.outfit)
    player.build = TheGame:GetLocalSettings().PLAYER_BUILD or outfit and outfit.build or player.build
end

function PlayerAct:InitializeAct( game_state, config_options)
    
    local player = game_state:GetPlayerAgent()
    if player == nil then
        player = self.background:CreateAgent()
        game_state:AddPlayerAgent( player )
    end

    if self.data.starting_fn then
        self.data.starting_fn(player)
    end        

    self:ApplyNewPlayerConfig( game_state, config_options )
end


