local PropertyBag = class("PropertyBag")

function PropertyBag:PropertyStack()
	return ipairs( self )
end

function PropertyBag:PushProperties( t )
	assert( table.arrayfind( self, t ) == nil )
	table.insert( self, t )
end

function PropertyBag:RemoveProperties( t )
	local idx = table.find( self, t )
	table.remove( self, idx )
	return idx
end

function PropertyBag:PopProperties()
	return table.remove( self )
end

function PropertyBag:LookupPropertiesByClass( class )
	for i = #self, 1, -1 do
		if is_instance( self[i], class ) then
			return self[i]
		end
	end
end

function PropertyBag:LookupProperty( key )
	for i = #self, 1, -1 do
		local v = self[i][key]
		if v ~= nil then
			return v
		end
	end
end

function PropertyBag:ResolveIDs()
	-- Resolve content IDs
	for i = 1, #self do
		local id = self[i]
		local def = Content.GetCharacterDef( self[i] ) or Content.GetCharacterSkin( self[i] )
		self[i] = def
		if def == nil then
			if i == 1 then -- Missing character skins are not a fatal error.
				error( id )
			end
		end
	end
end

function PropertyBag:__index( key )
	return PropertyBag[ key ] or self:LookupProperty( key )
end

function PropertyBag:Save()
	-- Save out content IDs.
	local data = {}
	for i = 1, #self do
		data[i] = self[i]:GetContentID()
		assert( data[i] )
	end
	return data
end
