require "testcreator"
require "selector"

--------------------------------------------------------------------------
-- A quest definition.

local QuestDef = class("QuestDef" )

function QuestDef.Define( v )
    local t = debug.getinfo( 2 )
    local qdef = QuestDef()
    qdef.path = t.short_src -- Ergh.
    qdef.parameters = {}
    qdef.cast = {}
    qdef.objectives = {}
    qdef.followups = {}
    qdef.neg_card_defs = {}
    qdef.location_defs = {}
    qdef.opinion_defs = {}
    qdef.score_defs = {}
    qdef:ParseDefinition( v )
    qdef.id = v.id or qdef.path:match( "/?([%w_]+)[.]lua$" ):upper()
    Content.AddQuestDef( qdef.id, qdef )
    return qdef
end

function QuestDef:SetModID( mod_id )
    self.mod_id = mod_id
end

function QuestDef:GetModID()
    return self.mod_id
end

function QuestDef:HarvestStrings(t)

    local quest_id = "QUEST." .. self.id

	if self.loc then
		for k, v in pairs(self.loc) do
			local loc_key = self:GetLocKey(k)
			t[loc_key] = v
		end
	end

    --we're going to have to separate out the code from the data here, sadly. These checks can go away then.
    if type(self.title) == "function" then
        print ("WARNING: FUNCTION QUEST TITLE:", self.id)
    else
        -- we're going to have to query by these strings later. I'm not in love with this formatting being done here. Perhaps we let the objective generate it itself?
        t[quest_id .. ".TITLE"] = self.title
    end

    if type(self.desc) == "function" then
        print ("WARNING: FUNCTION QUEST DESC:", self.id)
    else
        t[quest_id .. ".DESC"] = self.desc
    end


    for _,obj in pairs(self.objectives) do
        local obj_id = quest_id .. ".OBJECTIVE."..string.upper(obj.id)
        
        if type(obj.title) == "function" then
            print ("WARNING: FUNCTION OBJECTIVE TITLE:", self.id, obj.id)
        else
            t[obj_id .. ".TITLE"] = obj.title
        end

        if type(obj.desc) == "function" then
            print ("WARNING: FUNCTION OBJECTIVE DESC:", self.id, obj.id)
        else
            t[obj_id .. ".DESC"] = obj.desc
        end

    end

    if self.registered_convos then
        for _,convo in pairs(self.registered_convos) do
            convo:HarvestStrings(t, quest_id)
        end
    end
end

function QuestDef:AddLocationDefs(args)
    for id, locdef in pairs(args) do
        local loc_id = string.format( "%s_%s", self.id, id)
        locdef.id = loc_id
        Content.AddLocationContent(locdef)
        self.location_defs[id] = Content.GetLocationContent(loc_id)
    end
    return self
end

function QuestDef:AddQuestLocation(args)
    args.id = args.id or args.cast_id
    self:AddLocationDefs{[args.id] = args}
    self:AddLocationCast{
        cast_id = args.cast_id, 
        cast_fn = function(quest, t) table.insert(t,  quest:SpawnTempLocation(args.id)) end,
        on_assign = args.on_assign,
    }

    return self
end


function QuestDef:AddDefCast(id, def, filter)

    self:AddCast{
        cast_id = id,
        condition = function(agent, quest) 
            local good_def = type(def) == "string" and def == agent:GetContentID() or table.arrayfind(def, agent:GetContentID())
            return good_def and (not filter or filter(agent, quest))
        end,
    }
    
    self:AddCastFallback{
        cast_fn = function(quest, t)
            local def_to_use = type(def) == "string" and def or table.arraypick(def)
            table.insert( t, quest:CreateSkinnedAgent( def_to_use ) )
        end,
    }

    return self
end

function QuestDef:AddDefCastSpawn(id, def)
    self:AddCast{
        cast_id = id,
        cast_fn = function(quest, t)
            local def_to_use = type(def) == "string" and def or table.arraypick(def)
            table.insert( t, quest:CreateSkinnedAgent( def_to_use ) )
        end,
    }
    return self
end


function QuestDef:AddNegotiationCardDefs(args)
    for id, carddef in pairs(args) do
        Content.AddNegotiationCard( id, carddef)
        self.neg_card_defs[id] = Content.GetNegotiationCard(id)
    end
    return self
end


function QuestDef:AddOpinionEvents(args)
    self.opinion_defs = self.opinion_defs or {}
    for id, opdef in pairs(args) do
        local loc_id = loc.format( "{1}_{2}", self.id, id)
        self.opinion_defs[id] = opdef
        AddOpinionEvent(loc_id, opdef)
    end
    return self
end


function QuestDef:GetScoreDef(id)
    return self.score_defs[id]
end

function QuestDef:AddScoreDefs(args)
    self.score_defs = self.score_defs or {}
    for id, scoredef in pairs(args) do
        local loc_id = loc.format( "{1}_{2}", self.id, id)
        scoredef.id = loc_id
        self.score_defs[id] = scoredef
        Content.AddScoreComponent(ScoreComponentDef(scoredef))
    end
    return self
end


function QuestDef:GetLocationDef(id)
    return self.location_defs[id]
end


function QuestDef:GetOpinionEvent(id)
    return self.opinion_defs[id]
end

function QuestDef:GetNegotiationCardDef(id)
    return self.neg_card_defs[id]
end

function QuestDef:ParseDefinition( def )
    for k, v in pairs( def ) do
        assert( self[k] == nil )
        self[k] = v
    end

    assert( IsEnum( self.qtype, QTYPE ))

    if self.icon == nil then
        local filename = self.path:match( "/([^/]+)[.]lua$" )
        self.icon_path = string.format( "icons/quests/%s.tex", tostring(filename))

        if engine.inst:FileExists( self.icon_path ) then
            self.icon = engine.asset.Texture( self.icon_path, true )
        end
    end
    assert( type(self.icon) ~= "string" )

    local qrank = def.rank
    if type(qrank) == "number" then
        self.qrank_min, self.qrank_max = qrank, qrank
    elseif type(qrank) == "table" then
        self.qrank_min, self.qrank_max = table.unpack( qrank )
    end

    self.priority = def.priority or 0
    self.qrank_min = self.qrank_min or 1
    self.qrank_max = self.qrank_max or QUEST_MAX_RANK

    if self.qrank_max and self.qrank_min then
        assert( self.qrank_max >= self.qrank_min )
        assert( self.qrank_max <= QUEST_MAX_RANK )
    end

    if self.events == nil then
        self.events = table.empty
    end
    if self.tags == nil then
        self.tags = table.empty
    end

    if self.qflags == nil then
        self.qflags = QFLAGS.NONE
    end

    if def.codex then
        self.codex = Codex()
        self.codex:AddTable( def.codex )
    end
    
    if self.can_abandon == nil then
        self.can_abandon = ( self.qtype ~= QTYPE.STORY and self.qtype ~= QTYPE.EVENT)
    end
    if ( self.qtype == QTYPE.SIDE or self.qtype == QTYPE.CONTRACT) then
        if not self.desc then
            print ("MISSING QUEST DESCRIPTION:", self.id)
        end
    end
    self.spawn_event_mask = self.spawn_event_mask or 0

    self.registered_convos = {}
end

function QuestDef:GetRegisteredConvo(id)
    return self.registered_convos[id]
end

function QuestDef:GetSpawnEventMask()
    return self.spawn_event_mask
end

function QuestDef:AddSubQuest( child, cast )
    assert("MISSING quest_id", child.quest_id)
    
    
    local id = "sub_quest_" .. child.quest_id
    child.hide_in_overlay = true
    local old_activate = child.on_activate
    
    child.on_activate = function(quest)
        if old_activate then
            old_activate(quest)
        end
        
        --local cast = {}
        cast = cast or {}
        if child.addCast then
            child.addCast(quest, cast)
        end

        local params = {parent_quest = quest}
        if child.addParams then
            child.addParams(quest, params)
        end

        local overrides = {cast = cast, parameters = params}

        local child_quest, err = QuestUtil.SpawnQuest(child.quest_id, overrides)
        if child_quest == nil then
            DBG(err)
        else
            quest.param[id] = child_quest
        end
    end


    child.events= child.events or {}
    local old_changed = child.events.quests_changed
    child.events.quests_changed = function(quest, event_quest)
        --print ("!!!!!", self.id, subquest, event_quest, subquest == event_quest)
        if old_changed then
            old_changed(quest, event_quest)
        end

        if quest.param[id] == event_quest then
            if not event_quest:IsActive() then
                quest:Complete(child.id)
            end
        end
    end

    return self:AddObjective(child)
end

function QuestDef:AddObjective( child )
    assert_warning( child.id, "Objective missing ID '%s'", self.path )
    assert_warning( child.title == nil or type(child.title) == "string" or type(child.title) == "function",  "Objective invalid title '%s'", self.path )

    if child.cast then
        for cast_id, v in pairs( child.cast ) do
            assert( self:GetCast( cast_id ))
            if v.mark then
                if child.mark == nil then
                    child.mark = {}
                end
                table.insert( child.mark, cast_id )
            end
        end
    end

--[[    if (child.title and not child.desc) then
        print ("MISSING OBJECTIVE DESCRIPTION", self.id, child.id)
    end

    if (child.desc and not child.title) then
        print ("MISSING OBJECTIVE TITLE", self.id, child.id)
    end
--]]

    table.insert( self.objectives, child )
    return self
end

function QuestDef:AddQuips(lines)
    if not self.quips then
        self.quips = QuipDatabase()
    end
    
    self.quips:ParseTable( lines )
    return self
end

function QuestDef:AddCast( cast )

    assert( self:GetCast( cast.cast_id ) == nil, cast.cast_id )
    assert( cast.condition == nil or type(cast.condition) == "function", cast.condition )
    assert( cast.on_assign == nil or type(cast.on_assign) == "function" )
    assert( cast.on_unassign == nil or type(cast.on_unassign) == "function" )
    assert( cast.when == nil or IsEnum( cast.when, QWHEN ), "Invalid when field" )

    if cast.what == nil then
        cast.what = QWHAT.AGENT
    end
    assert( IsEnum( cast.what, QWHAT ))
    assert( cast.limit == nil or IsEnum( cast.limit, QLIMIT ), "Invalid cast limit: " .. tostring(cast.limit) )
    
    table.insert( self.cast, cast )

    return self
end



function QuestDef:AddCastFallback( cast )
    
    if type(cast) == "string" then
        local cast_id = cast
        cast = 
        {
            cast_fn = function(quest, t)
                table.insert( t, quest:CreateSkinnedAgent( cast_id ) )
            end,            
        }

    end

    local cast_def = self.cast[ #self.cast ]
    cast_def.fallback = cast
    cast.is_fallback = true
    cast.what = cast_def.what
    cast.cast_id = cast_def.cast_id

    return self
end

function QuestDef:AddParameter( cast )
    cast.what = QWHAT.PARAMETER
    return self:AddCast( cast )
end


function QuestDef:AddCastByAlias( cast )
    
    local filter_fn = cast.filter_fn

    assert( cast.alias or error( string.format( "No alias specified for cast '%s'", cast.cast_id )))

    cast.cast_fn = function( quest, t )
        local alias = type (cast.alias) == "table" and table.arraypick(cast.alias) or cast.alias
        local agent = AgentUtil.GetOrSpawnAgentbyAlias(alias)
        if not agent:IsRetired() and (filter_fn == nil or filter_fn(agent)) then
            table.insert( t, agent )
        end
    end

    return self:AddCast( cast )
end

function QuestDef:AddLocationCast( cast )
    cast.what = QWHAT.LOCATION
    return self:AddCast( cast )
end


function QuestDef:AddExistingLocation( cast_id, id )
    return self:AddLocationCast{
        cast_id = cast_id,
        no_validation = true,
        cast_fn = function(quest, t)
            table.insert(t,  TheGame:GetGameState():GetLocation(id))
        end,
    }

end

function QuestDef:AddConvo( objective_id, cast_id, registration_id )
    
    local q = QuestConvo( self.id, objective_id, cast_id)
    if registration_id then
        self.registered_convos[registration_id] = q
    end
    --quest can override this, but it's almost always true
    q:Priority(CONVO_PRIORITY_HIGHEST)


    return q
end

function QuestDef:AddIntro(intro_text, accept_text, activate_id)

    if intro_text then
        
        self:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
        :Loc{
            DIALOG_INTRO = intro_text
        }
        :State("START")
            :Fn(function(cxt)
                cxt:Dialog("DIALOG_INTRO")
            end)
    end
    
    if accept_text then
        
        self:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
        :Loc{
            DIALOG_INTRO = accept_text
        }
        :State("START")
            :Fn(function(cxt)
                cxt:Dialog("DIALOG_INTRO")
                if activate_id then
                    cxt.quest:Activate(activate_id)
                end

            end)
    end
end


function QuestDef:AddAttract(objective_id, cast_id, txt, fn)
    
    local q = QuestConvo(self.id, objective_id, cast_id)
        :Attract(function(cxt, agent)
            return "STATE_ATTRACT"
        end)
        :State("STATE_ATTRACT")
            :Loc{
                DIAL_TXT = txt
            }
            :Fn(function(cxt)
                if txt then
                    cxt:Dialog("DIAL_TXT")
                end
                if fn then
                    fn(cxt)
                end
            end)
    
    return q
end


function QuestDef:AddPostConvo( cast_id )
    local q = QuestPostConvo( self.id, cast_id)
    return q
end

function QuestDef:GetFlags()
    return self.qflags
end

function QuestDef:GetID()
    return self.id
end

function QuestDef:GetTitle(quest)
	local result = nil

	if self.title then
		local key = "QUEST." .. self.id .. ".TITLE"
		result = LOC(key)
		if self.title_fn then
			result = self.title_fn(quest, result)
		end
	end
	return result
end

function QuestDef:GetDesc(quest)
	local result = nil
	if self.desc then
        local key = "QUEST." .. self.id .. ".DESC"
		result = LOC(key)

		if self.desc_fn then
			result = self.desc_fn(quest, result)
		end
	end
	return result
end

function QuestDef:GetPriority()
    return self.priority
end

function QuestDef:RegisterConvoStateGraph( sg )
    if self.convo_sg == nil then
        self.convo_sg = { sg }
    else
        table.insert( self.convo_sg, sg )
    end
end
function QuestDef:GetConvoStateGraph()
    return self.convo_sg or table.empty
end

function QuestDef:CanAbandon()
    return self.can_abandon
end

function QuestDef:HasPostConvos()
    if self.convo_sg then
        for i, convodef in ipairs( self.convo_sg ) do
            if convodef.post_convo then
                return true
            end
        end
    end
end

function QuestDef:HasTag( tag )
    return table.arrayfind( self.tags, tag ) ~= nil
end

function QuestDef:IsDisabled()
    return self.disabled
end

function QuestDef:CastDefs()
    return ipairs( self.cast )
end

function QuestDef:GetCastDefs()
    return self.cast
end

function QuestDef:GetCast( cast_id )
    for i = 1, #self.cast do
        if self.cast[i].cast_id == cast_id then
            return self.cast[i]
        end
    end
end


function QuestDef:GetParameter( name )
    return self.parameters[ name ]
end

function QuestDef:GetType( )
    return self.qtype
end

function QuestDef:GetRankRange()
    return self.qrank_min, self.qrank_max 
end

function QuestDef:CheckObjective( objective_id )
    for i,obj in ipairs(self.objectives) do
        if obj.id == objective_id then
            return obj
        end
    end
end

function QuestDef:GetObjective( objective_id )
    for i,obj in ipairs(self.objectives) do
        if obj.id == objective_id then
            return obj
        end
    end
    LOGWARN( "Missing objective: '%s'", objective_id )
end

function QuestDef:GetObjectiveTitle(objective_id)
	local key = "QUEST." .. self.id .. ".OBJECTIVE.".. string.upper(objective_id) .. ".TITLE"
	local str = LOC(key)
	return str
end

function QuestDef:GetObjectiveDesc(objective_id)
	local key = "QUEST." .. self.id .. ".OBJECTIVE.".. string.upper(objective_id) .. ".DESC"
	local str = LOC(key)
	return str
end

function QuestDef:FindObjectiveTag( tag )
    for i,obj in ipairs(self.objectives) do
        if obj.tags and table.arrayfind( obj.tags, tag ) ~= nil then
            return obj
        end
    end
end

function QuestDef:Objectives()
    return ipairs( self.objectives )
end


function QuestDef:IsOnCooldown()
    if self.cooldown and self.cooldown > 0 then
        local count = QuestUtil.CountQuestActive( self.id, self.cooldown )
        if count > 0 then
            return true
        end
    end
end

function QuestDef:IsValidForRank(rank)
    local min_rank, max_rank = self:GetRankRange()
    if min_rank and max_rank and ( rank < min_rank or rank > max_rank) then
        return false
    end

    return true
end


-- Constructs a QuestState (quest instance) from this definition.
-- returns <ok, QuestState>.
-- NOTE: if spawning failed, the quest object is still returned for debug purposes, but the first parameter is false.
function QuestDef:CreateQuest( spawn_params, force_spawn )
    local rank = spawn_params and spawn_params.qrank 
    rank = rank or TheGame:GetGameState():GetCurrentBaseDifficulty() 

    if not force_spawn and self:GetType() ~= QTYPE.STORY then
        if not self:IsValidForRank(rank) then
            -- quest:Trace( "Invalid for rank %s", rank )
            return false
        end
        
        if self:IsOnCooldown() then
            -- quest:Trace( "On cooldown." )
            return false
        end
    end

    local quest = QuestState( self.id, rank, spawn_params )

    for k, v in pairs( self.parameters ) do
        quest:SetParameter( k, v )
    end

    if self.precondition then
        local ok, reason = self.precondition( quest )
        if not ok then
            quest:Trace( "Failed precondition: %s", reason )
            return false, quest
        end
    end

    local when = spawn_params and spawn_params.when or QWHEN.DEFAULT
    if spawn_params and spawn_params.cast_by_tag then
        for tag, agent in pairs( spawn_params.cast_by_tag ) do
            local found = false
            for i, cast_def in ipairs( self.cast ) do
                if cast_def[ tag ] and (cast_def.when or when) == when then
                    found = true
                    break
                end
            end
            if not found then
                quest:Trace( "Missing cast for specified cast_by_tag[ %s ]", tag )
                return false, quest
            end
        end
    end

    local provider
    for i, cast_def in ipairs( self.cast ) do
        if cast_def.when ~= nil and (cast_def.when ~= when) then
            quest:Trace( "[%s] - when mismatch: %s ~= %s (not casting)", cast_def.cast_id, cast_def.when, tostring(spawn_params and spawn_params.when))
        else
            local registrant = self:RegisterCastMember( quest, cast_def, spawn_params )
            if registrant ~= nil then

                quest:AssignCastMember( cast_def.cast_id, registrant )

                if cast_def.provider then
                    provider = registrant
                end

            elseif not cast_def.optional then
                quest:Trace( "[%s] Non-optional cast registration failed", cast_def.cast_id )
                
                return false, quest
            end
        end
    end

    if self.postcondition then
        local ok, reason = self.postcondition( quest )
        if not ok then
            quest:Trace( "Failed postcondition: %s", reason )
            return false, quest
        end
    end

    quest:VerifyRewards()

    return true, quest -- Success!
end

local function GetCastOverride( cast_def, spawn_params )
    local cast_override
    if spawn_params then
        if spawn_params.cast then
            cast_override = spawn_params.cast[ cast_def.cast_id ]
        end
        if spawn_params.cast_by_tag then
            for tag, agent in pairs( spawn_params.cast_by_tag ) do
                if cast_def[ tag ] then
                    cast_override = agent
                end
            end
        end
    end
    return cast_override
end


function QuestDef:RegisterCastMember( quest, cast_def, spawn_params )
    local cast_id = cast_def.cast_id
    local registrant = nil
    
    while cast_def do
        if quest:GetCastMember( cast_id ) then
            quest:Trace( "[%s] - already cast", cast_id )
            return
        end

        quest:Trace( "[%s] - casting %s", cast_id, cast_def.what )

        if cast_def.debug_cast then
            -- NOTE: ignores all validation!
            registrant = cast_def.debug_cast
            quest:Trace( "[%s] - cast %s via debug_cast", cast_id, registrant )

        elseif cast_def.what == QWHAT.LOCATION then
            registrant = self:RegisterCastLocation( quest, cast_def, spawn_params )

        elseif cast_def.what == QWHAT.AGENT then
            registrant = self:RegisterCastAgent( quest, cast_def, spawn_params )

        elseif cast_def.what == QWHAT.PARAMETER then
            registrant = self:RegisterParameter( quest, cast_def, spawn_params )

        else
            error()
        end

        if registrant ~= nil then
            quest:Trace( "[%s] - cast %s!", cast_id, registrant )
            break
        else
            cast_def = cast_def.fallback
            if cast_def then
                quest:Trace( "[%s] - no registrant; using fallback", cast_id )
            else
                quest:Trace( "[%s] - no registrant", cast_id )
            end
        end
    end

    return registrant
end

function QuestDef:RegisterCastLocation( quest, cast_def, spawn_params )
    local selector = Selector()
    selector:BeginTrace( quest.trace )
    
    local cast_override = GetCastOverride( cast_def, spawn_params )
    if cast_override then
        -- As an optimization, restrict the search to the cast override requirement.
        selector:FromSingle( cast_override )

    elseif cast_def.cast_fn then
        local t = ObtainWorkTable()
        cast_def.cast_fn( quest, t )
        if #t == 0 then
            quest:Trace( "[%s] - No candidates returned by cast_fn", cast_def.cast_id )
        end
        selector:FromTable( t )
        ReleaseWorkTable( t )
    else
        selector:FromLocationsInRegion()
    end
   
    if not cast_def.no_validation then
        selector:Where( QuestDef.IsValidLocationPick, cast_def, quest, spawn_params )
    elseif cast_def.condition then
        selector:Where( cast_def.condition, quest )
    end
    
    return selector:Pick()
end

function QuestDef:RegisterParameter( quest, cast_def, spawn_params )
    local selector = Selector()
    selector:BeginTrace( quest.trace )

    local cast_override = GetCastOverride( cast_def, spawn_params )
    if cast_override then
        selector:FromSingle( cast_override )

    elseif cast_def.cast_fn then
        local t = ObtainWorkTable()
        cast_def.cast_fn( quest, t )
        if #t == 0 then
            quest:Trace( "[%s] - No candidates returned by cast_fn", cast_def.cast_id )
        end
        selector:FromTable( t )
        ReleaseWorkTable( t )       
    end

    return selector:Pick()
end

function QuestDef:RegisterCastAgent( quest, cast_def, spawn_params )
    local selector = Selector()
    selector:BeginTrace( quest.trace )


    local cast_override = GetCastOverride( cast_def, spawn_params )
    if cast_override then
        -- As an optimization, restrict the search to the cast override requirement.
        selector:FromSingle( cast_override )

    elseif cast_def.bucket_id then
        selector:FromAgentBucket( cast_def.bucket_id )

    elseif cast_def.cast_fn then  
        local t = ObtainWorkTable()
        cast_def.cast_fn( quest, t )
        if #t == 0 then
            quest:Trace( "[%s] - No candidates returned by cast_fn", cast_def.cast_id )
        end
        selector:FromTable( t )
        ReleaseWorkTable( t )

    elseif cast_def.from_current_location then
        selector:FromAgentsInLocation(TheGame:GetGameState():GetPlayerAgent() and TheGame:GetGameState():GetPlayerAgent():GetLocation())
    else
        selector:FromAllAgents() 
    end
   
    if not cast_def.no_validation then
        selector:Where( QuestDef.IsValidAgentPick, cast_def, quest, spawn_params )
    elseif cast_def.condition then
        selector:Where( cast_def.condition, quest )
    end

    if cast_def.score_fn then
        return selector:ScoreAndPick( cast_def.score_fn, quest )
    else
        return selector:Pick()
    end
end

local function IsNotStoryQuest( quest )
    return quest:GetType() ~= QTYPE.STORY
end

function QuestDef.IsValidAgentPick( agent, cast_def, quest, spawn_params )
    local cast_id = cast_def.cast_id

    if spawn_params then
        if spawn_params.condition then
            local ok, reason = spawn_params.condition( agent, cast_def, quest )
            if not ok then
                return false, reason or "Override condition failed"
            end
        end
        local cast_override = GetCastOverride( cast_def, spawn_params )
        if cast_override and agent ~= cast_override then
            return false, "Not override cast agent"
        end
    end

    if AgentUtil.IsInHiding(agent) then
        return false, "in hiding"
    end


    if agent:IsRetired() then
        return false, "retired"

    elseif quest:IsCastMember( agent ) ~= nil then
        return false, "already cast "

    elseif agent:IsInPlayerParty() and not cast_def.party_member_valid then
        return false, "party members not cast"

    elseif AgentUtil.HasPlotArmour(agent) and not cast_def.alias then
        return false, "This agent has plot armour and is not cast by default"
 
    elseif not agent:IsSentient() and not cast_def.non_sentient_valid then
        return false, "non-sentients not cast"
    end

    if not agent:IsInPlayerParty() and not cast_def.is_fallback and agent:GetSkinID() == nil and not cast_def.cast_fn then
        return false, "non-skinned agents can only be used as fallback casts"
    end


    if quest:GetType() ~= QTYPE.STORY and cast_def.limit ~= QLIMIT.NOLIMIT then
        local count = QuestUtil.CountQuests( agent, IsNotStoryQuest )
        if count > 0 then
            return false, "Already assigned to quest"
        end
    end


    if quest:GetType() == QTYPE.SIDE then

        if cast_def.provider then

            if agent.handler then
                return false, "Handlers don't offer side quests"
            elseif quest:GetType() == QTYPE.SIDE and agent:GetRelationship() ~= RELATIONSHIP.NEUTRAL then
                return false, "Requires NEUTRAL relationship"
            end

            if agent:GetLocation() == TheGame:GetGameState():GetPlayerAgent():GetLocation() then
                return false, "No side job providers in our current location"
            end

            if agent:GetLocation() then
                for _, other_agent in agent:GetLocation():Agents() do
                    if AgentUtil.HasSideQuest(other_agent) then
                        return false, "already in a location with another side quest offer"
                    end
                end
            end

            if agent:GetBrain():GetWorkplace() then
                for k, wp in agent:GetBrain():GetWorkplace():WorkPositions() do
                    if wp:GetAgent() and AgentUtil.HasSideQuest(wp:GetAgent()) then
                        return false, "A coworker already has a side quest"
                    end
                end

                for _, other_agent in agent:GetBrain():GetWorkplace():Agents() do
                    if AgentUtil.HasSideQuest(other_agent) then
                        return false, "someone at work has a side quest already"
                    end
                end

            end
        end
    end

    if cast_def.condition then
        local ok, reason = cast_def.condition( agent, quest )
        if not ok then
            return false, reason
        end
    end

    return true
end

function QuestDef.IsValidLocationPick( location, cast_def, quest, spawn_params )
    if location:IsClosed() then
        return false, "Closed"
    end
    
    if cast_def.condition then
        local ok, reason = cast_def.condition( location, quest )
        if not ok then
            return false, reason
        end
    end

    return true
end

function QuestDef:FilterForAct( act_id )
    local valid = true
    if type(self.act_filter) == "function" and not self.act_filter( self, act_id ) then
        valid = false

    elseif type(self.act_filter) == "string" and self.act_filter ~= act_id then
        valid = false
    end

    return valid
end

function QuestDef:AddFollowup(t)
    t = t or {}
    t.qtype = QTYPE.FOLLOWUP
    t.source_quest = self
    
    local qdef = QuestDef()
    qdef.path = debug.getinfo( 2 ).short_src -- Ergh.
    qdef.id = self.id .. "_" .. tostring(#self.followups)
    qdef.parameters = {}
    qdef.cast = {}
    qdef.objectives = {}
    qdef:ParseDefinition( t )

    for k, v in ipairs(self.cast) do
        qdef:AddCast{cast_id = v.cast_id, what = v.what, no_validation = true}            
    end

    Content.AddQuestDef( qdef.id, qdef )
    table.insert(self.followups, qdef.id)
    qdef.parent_quest = self.id
    return qdef
end

function QuestDef:Loc(dict)
    --local line = debug.getinfo(1).currentline
    for k,v in pairs(dict) do
        self:AddLocString(k, v)
    end
    return self
end

function QuestDef:AddLocString(string_id, str)
    assert( string.len( str ) ~= nil or error( string.format( "Mal-formed utf8 string: %s. Is your file the wrong encoding?", string_id )))
	self.loc = self.loc or {}
	self.loc[string_id] = str
end

function QuestDef:GetLocKey(str_id)
    local quest_id = "QUEST." .. string.upper(self.id)
	local loc_key = quest_id .. "." .. string.upper(str_id)
	return loc_key
end

function QuestDef:GetLocalizedStr(str_id)
	local loc_key = self:GetLocKey(str_id)
	return LOC(str_id)
end


function QuestDef:AddTest(test)
    self.tests = self.tests or {}
    if type(test) == "table" and is_instance(test, TestCreator) then
        table.insert(self.tests, {tests = test:GenerateTest()})
    else
        table.insert(self.tests, test)
    end
    return self

end

function QuestDef:IsolatedTest(params, rank, deck, advancement)

    TheGame:GetLocalSettings().SKIP_SLIDE_SHOWS = true

    TheGame:FE():ClearScreens()
    local act_id = type(self.act_filter) == "string" and self.act_filter or "SAL"
    local options = GameState.CreateOptions(act_id, advancement)

    local act_data = GetPlayerActData( act_id )        
    options.no_act_quests = self.id == act_data.data.main_quest
    options.experiment = true
    
    TheGame:StartNewGame( options )

    --cancel existing quests?
    local to_cancel = {}
    
    while ( next(TheGame:GetGameState().available_quests )) do
        local to_kill = shallowcopy(TheGame:GetGameState().available_quests)
        for _, quest in pairs(to_kill) do
            print ("CANCEL QUEST:", quest)
            quest:Cancel()
        end
    end

    if self.qtype == QTYPE.EVENT and CheckBits( self:GetSpawnEventMask(), QEVENT_TRIGGER.TRAVEL) then
        local runaway_loc = TheGame:GetGameState():AddLocation( Location( TheGame:GetGameState():GetWorldRegion():GetOutdoorLocationID() ) )
        runaway_loc:SetFlags( LOCFLAGS.TRANSIENT )
        runaway_loc:AddTag("in_transit")
        TheGame:GetGameState():GetCaravan():MoveToLocation(runaway_loc)
    end


    local overrides = {qrank = rank, parameters = params}
    local quest, err = QuestUtil.SpawnQuest( self.id, overrides, true )
    if not quest and  err then
        DBG(err)
        return
    end

    
    if deck then
        TheGame:GetGameState():SetDecks(deck)
    end

    if advancement then
        TheGame:GetGameState():SetAdvancementLevel(advancement)
    end

    if self.qtype ~= QTYPE.SIDE and not quest:IsActive() then
        quest:Activate()
    end

    --[[if  then
        local provider = quest:GetProvider()
        if provider then
            
        end

    end--]]

    quest:OnDebugStart()


end

function QuestDef:AddDebugOption(name, options, test_fn)
    self.debug_options = self.debug_options or {}
    table.insert(self.debug_options,  {name=name, opt = options, test = test_fn})
    return self
end

function QuestDef:RunTests(on_done)

    if self.tests and #self.tests > 0 then
            
        local max_test_index = #self.tests
        local test_index = 1
        local step = 1
        local rep = 1
        local function Run()
            
            local test = self.tests[test_index]

            TheGame:FE():ClearScreens()
            TheGame:GetLocalSettings().ROBOTICS = true
            TheGame:GetLocalSettings().AUTO_RESOLVE_NEGOTIATION = true
            TheGame:GetLocalSettings().SKIP_SLIDE_SHOWS = true
            TheGame:SetWallTimeScale( 12.0 )
            
            local options = GameState.CreateOptions(self.act_filter or "SAL")
            TheGame:StartNewGame( options )

            --cancel existing quests?
            local to_cancel = {}
            
            while ( next(TheGame:GetGameState().available_quests )) do
                local to_kill = shallowcopy(TheGame:GetGameState().available_quests)
                for _, quest in pairs(to_kill) do
                    print ("CANCEL QUEST:", quest)
                    quest:Cancel()
                end
            end

            local test_battery = test.tests or table.empty
            local current_test = test_battery and test_battery[step] or table.empty
            local max_steps = test_battery and #test_battery or 1

            local overrides = current_test.overrides and shallowcopy(current_test.overrides) or table.empty
            local name = current_test.name or "TEST"
            local max_reps = current_test.reps or 1
            
            if test.game_setup then
                test.game_setup()
            end
            
            if current_test.game_setup then
                current_test.game_setup()
            end
            
            if current_test.game_setup_fns then
                for k,v in ipairs(current_test.game_setup_fns) do
                    v()
                end
            end

            local test_title = loc.format( "*******{4}/{5}: {1} rep {2}/{3}*******", name, rep, max_reps, step, max_steps)
            TheGame:GetRoboticsSettings().status_string = test_title

            print (test_title)
            
            if self.qtype == QTYPE.EVENT and CheckBits( self:GetSpawnEventMask(), QEVENT_TRIGGER.TRAVEL) then
                local runaway_loc = TheGame:GetGameState():AddLocation( Location( TheGame:GetGameState():GetWorldRegion():GetOutdoorLocationID() ) )
                runaway_loc:SetFlags( LOCFLAGS.TRANSIENT )
                runaway_loc:AddTag("in_transit")
                TheGame:GetGameState():GetCaravan():MoveToLocation(runaway_loc)
            end

            if test.cast_overrides then
                overrides.cast = {}
                for k,v in pairs( test.cast_overrides) do
                    overrides.cast[k] = v()
                end
            end

            local quest = QuestUtil.SpawnQuest( self.id, overrides, true )
            if quest then
                quest:OnDebugStart()
            end
            local weights = current_test.choice_weights

            TheGame:GetRoboticsSettings().choice_weights = weights
            TheGame:GetRoboticsSettings().battle_policy = current_test.battle_policy
            TheGame:GetRoboticsSettings().negotiation_policy = current_test.battle_policy

            TheGame:SetTestUpdateFn(function() 
                if TheGame:GetGameState() then
                    if (quest:IsComplete() or quest:IsFailed()) and TheGame:GetGameState():GetCaravan():GetCurrentEncounter() == nil then
                        TheGame:SetTestUpdateFn(nil)
                        --done?
                        
                        if rep < max_reps then
                            rep = rep + 1    
                            TheGame:Reset(function() Run() end)
                        else
                            if step >= max_steps then
                                if test_index == max_test_index then
                                    on_done()
                                else
                                    test_index = test_index + 1
                                    step = 1
                                    rep = 1
                                    TheGame:Reset(function() Run() end)
                                end
                            else
                                step = step + 1
                                rep = 1
                                TheGame:Reset(function() Run() end)
                            end

                        end
                    end                   
                end
            end)
            assert(quest, "COULD NOT SPAWN QUEST" .. self.id)

            if self.tests[test_index].quest_setup then
                self.tests[test_index].quest_setup(quest)
            end

            if current_test.quest_setup then
                current_test.quest_setup(quest)
            end

            if current_test.quest_setup_fns then
                for k,v in ipairs(current_test.quest_setup_fns) do
                    v(quest)
                end
            end


            if self.test_fn then
                self.test_fn()        
            end
        end
        
        Run()
    end
end


function QuestDef:TryAddKnownCast(cast_id, def_id)
    
    self:AddCast{
        cast_id = cast_id,
        condition = function(agent, quest) 
            return agent:GetContentID() == def_id
        end,

        score_fn = function(agent, quest)
            if agent:GetRelationship() ~= RELATIONSHIP.NEUTRAL then
                return 10
            end
            return 1
        end,
    }:AddCastFallback{
        cast_fn = function(quest, t)
            table.insert( t, quest:CreateSkinnedAgent( def_id ) )
        end,
    }
    
    return self
end



function QuestDef:SideJobOffer(start_objective, offer_text, accepted_text, fn)
    self:AddConvo(nil, "giver")
        :Priority(CONVO_PRIORITY_HIGHEST)
        :Loc{
            DIALOG_OFFER = offer_text,
            DIALOG_ACCEPTED = accepted_text,
        }
        :Hub(function(cxt) 
            StateGraphUtil.OfferSideJob(cxt, start_objective, "DIALOG_OFFER", 
                function() 
                    if accepted_text then
                        cxt:Dialog("DIALOG_ACCEPTED") 
                    end
                    if fn then
                        fn(cxt)
                    end
                end)
        end)
end



function QuestDef:AddDormancyState(name, next_state, wait_for_next_time, min_time, max_time, active)


    self:AddObjective{
        id = name,
        hide_in_overlay = true,
        state = active and QSTATUS.ACTIVE,

        events = 
        {
            action_clock_advance = function(quest, location)
                if quest.param.dormant_start_time ~= Now() then
                    quest.param.dormant_timer = (quest.param.dormant_timer or 0) - 1
                    if quest.param.dormant_timer <= 0 then
                        quest:Complete(name)
                        quest:Activate(next_state)
                    end

                end
            end,
        },

        on_activate = function(quest)
            quest.param.dormant_timer = min_time + math.random(max_time - min_time)
            if wait_for_next_time then
                quest.param.dormant_start_time = Now()
            end
        end,
    }
    
    return self
end
