
--------------------------------------------------------------------------------------
-- Runtime expression of an Region, stored in GameState.

local Region = class("Region")

function Region:init(content_id )
    self.content_id = content_id
    self.content = Content.GetWorldRegion( self.content_id )
end

function Region:GetContent()
    return self.content
end

function Region:GenerateHome(agent)
end

function Region:GetContentID()
    return self.content_id
end

function Region:GetLocName()
    return self.content.loc_name or ""
end

function Region:GetLocDesc()
    return self.content.loc_desc or ""
end

function Region:PopulateGameState( game_state )

    if self.content.locations then
        for _,loc_id in pairs(self.content.locations) do
            local location = Location(loc_id, self)
            game_state:AddLocation( location )
        end
    end
end

function Region:DepopulateGameState( game_state )
    -- All agents in this area must go to limbo.
    -- NOTE: this may cause some locations to self-destruct, hence why it goes first.
    for guid, agent in game_state:Agents() do
        if agent:GetWorldRegion() == self.content_id then
            agent:MoveToLimbo()
        end
    end

    -- Destroy all locations.
    for i, v in game_state:AllLocations() do
        local region = v:GetWorldRegion()
        print (v, region, self.content_id)
        if region == self then
            game_state:RemoveLocation( v )
        end
    end

end

function Region:Save()
    return
    {
        id = self.id,
        content_id = self.content_id,
        simdata = self.simdata,
        region_id = self.region_id,
    }
end

function Region:Load()
    self.content = Content.GetWorldRegion( self.content_id )
end

function Region:GetOutdoorLocationID(tags)
    local location_id = self.content.default_outdoor_location or "OUTDOORS"
    
    local location_candidates = {}
    if self.content.outdoor_locations then
        for _, id in pairs( self.content.outdoor_locations ) do
            local score = 0
            local data = Content.GetLocationContent(id)
            if tags then
                for k,tag in ipairs(tags) do
                    
                    if data.map_tags and table.arrayfind( data.map_tags,  tag ) then
                        score = score + 1
                    end

                end
            else
                score = 1
            end
            if score > 0 then
                location_candidates[id] = score
            end
        end
    end

    if next(location_candidates) then
        location_id = weightedpick(location_candidates)
    end

    return location_id
end

function Region:__tostring()
    if type(self.id) == "number" then
        return string.format( "%d:%s", self.id, self.content_id )
    else
        return string.format( "Region:%s",self.content_id)
    end
end

