---------------------------------------------------------------
-- UserSettings file that keeps track of all runs.

local RunHistory = class( "RunHistory", UserSettings )

function RunHistory:init()
    UserSettings.init( self, "run_history.lua", engine.settings )
    self:SetDefaults()
end

function RunHistory:SetDefaults()
    self:VerifySetting( "history", {} )
end

function RunHistory:GetHistory()
    return self:GetSetting( "history" )
end

function RunHistory:AddRun( game_state )
	local data = game_state:GetSummaryData()
	if data then
		assert_warning( IsEnum( data.result, GAMEOVER ), "Storing run state in progress?" )

		local history = self:GetHistory()
		table.insert( history, data )
        self.dirty = true

		print( string.format( "Added run to history: %d runs stored.", #history ))
	    self:SaveSettings()
	end
end

function RunHistory:Reset()
    self:ClearSettings()
    self:SetDefaults()
    self:SaveSettings()
end
