SCORE_POLICY = MakeEnum{"KEEP_HIGHEST", "KEEP_LATEST", "ACCUMULATE"}

local ScoreComponentDef = class( "ScoreComponentDef" )

function ScoreComponentDef:init( data )
    for k, v in pairs(data) do
        self[k] = v
    end
    assert( self.id )
end

function ScoreComponentDef:GetLocPrefix()
    self.loc_prefix = self.loc_prefix or "DAILY_RUN_SCORE." .. string.upper( self.id )
    return self.loc_prefix
end

function ScoreComponentDef:GetLocNameKey()
    self.loc_name_key = self.loc_name_key or self:GetLocPrefix() .. ".NAME"
    return self.loc_name_key
end

function ScoreComponentDef:GetName()
    local function ScoreFormatter( word, operator, rest )
        if word == "player" then
            local player = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent()
            return loc.format( string.format( "{1%s%s}", operator, rest ), player:LocTable() )
        else
            return string.format( "{%s%s%s}", word, operator, rest )
        end
    end
    return loc.custom_format( LOC( self:GetLocNameKey() ), ScoreFormatter )
end

function ScoreComponentDef:HarvestStrings( t )
    t[ self:GetLocNameKey() ] = self.name
end



local t = 
{
    
    PROGRESS = 
    {
        name = "Progress",
        policy = SCORE_POLICY.KEEP_HIGHEST,
        value = 100,
    },
    ONE_TURN_BATTLE_WIN = 
    {
        name = "One-Turn Battle Victory",
        value = 300,
    },
    QUICK_BATTLE_WIN = 
    {
        name = "Fast Battle Victory",
        value = 100,
    },

    BATTLES_WON = 
    {
        name = "Battles Won",
        value = 50,
    },
    NEGOTIATIONS_WON = 
    {
        name = "Negotiations Won",
        value = 50,
    },
    CARDS_UPGRADED = 
    {
        name = "Cards Upgraded",
        value = 10,
    },
    
    JOBS_COMPLETED = 
    {
        name = "Jobs Completed",
        value = 100,
    },

    SKIPPED_CARD_DRAFT = 
    {
        name = "Skipped Card Draft",
        value = 10,
    },
    
    SKIPPED_GRAFT_DRAFT = 
    {
        name = "Skipped Graft Draft",
        value = 50,
    },

    --[[LEFTOVER_MONEY = 
    {
        name = "Leftover Money",
        value = .25,
    },--]]

    GAINED_MONEY = 
    {
        name = "Gained Money",
        value = .5,
    },
    LOVED = 
    {
        name = "Loved",
        value = 40,     
    },
    LIKED = 
    {
        name = "Liked",
        value = 10
    },
    
    WON_DAILY = 
    {
        name = "Won Daily Run",
        value = 500
    },
} 


for k,v in pairs(t) do
    v.id = k
    Content.AddScoreComponent( ScoreComponentDef(v) )
end
