
-- Helper for pruning lists of things, and selecting one at random.

local Selector = class("Selector")
Selector.can_reload = true

function Selector:init()
    self.candidates = table.empty
end

function Selector:FromSingle( object )
    self.candidates = { object }
end

function Selector:FromTable( t )
    self.candidates = shallowcopy( t )
end

function Selector:FromAgentsInLocation( location )
    
    self.candidates = {}
    if location then
        for k,agent in location:Agents() do
            table.insert(self.candidates, agent)
        end
    end
end


function Selector:FromAgentBucket( bucket_id )
    self.candidates = TheGame:GetGameState():GetAgentBucket( bucket_id )
end

function Selector:FromAllAgents()
    local tbl = {}
    for _, agent in TheGame:GetGameState():Agents() do
        table.insert(tbl, agent)
    end
    self.candidates = tbl
end


function Selector:FromLocationsInRegion(  )
    local current_region = TheGame:GetGameState():GetWorldRegion()
    self.candidates = {}
    for _, location in TheGame:GetGameState():AllLocations() do
        if location:GetWorldRegion() == current_region then
            table.insert(self.candidates, location)
        end
    end
end

function Selector:FromAllEntities( sim )
    self.candidates = {}
    for guid, ent in pairs( sim.entities ) do
        table.insert( self.candidates, ent )
    end
end

function Selector:BeginTrace( trace )
    self.trace = trace
end

function Selector:Trace( str )
    if self.trace then
        table.insert( self.trace, str )
    else
        print( str )
    end
end

function Selector:Where( fn, ... )
    local candidates = {}
    for i = 1, #self.candidates do
        local candidate = self.candidates[i]
        local ok, result = fn( candidate, ... )
        if ok then
            table.insert( candidates, candidate )
        end

        if ok then
            self:Trace( string.format( "%s: OK", candidate ))
        else
            self:Trace( string.format( "%s: %s", candidate, result or "condition failed" ))
        end
    end
    self.candidates = candidates
    self:Trace( string.format( "%d candidates remaining", #candidates ))

    return self
end

function Selector:Pick()
    if self.candidates == table.empty then
        LOGWARN( "No candidates set specified" )
    elseif #self.candidates > 0 then
        self.pick = self.candidates[ math.random( #self.candidates ) ]
    end

    return self.pick
end

function Selector:ScoreAndPick( score_fn, quest )
    if self.candidates == table.empty then
        LOGWARN( "No candidates set specified" )
    elseif #self.candidates > 0 then
        local scored_picks = {}
        for i, candidate in ipairs(self.candidates) do
            table.insert(scored_picks, {candidate = candidate, score = score_fn(candidate, quest)})
        end

        table.sort(scored_picks, function(a, b) return a.score > b.score end)

        self.pick = scored_picks[1].candidate
    end

    return self.pick
end