--defines the machinery for managing a list of updating components
class ("SimUpdater")
function SimUpdater:init(sim, name, upfnname)
    self.updatefnname = upfnname
    self.name = name
    self.profilename = string.format("%sUpdate", name)
    self.sim = sim
    self.updating_components = {}
    self.updating_component_iter_list = {}
    self.stop_updaters = {}
    self.new_updaters = {}
    self.updating_component_order = {}
    self.num_updaters = 0

    self.updatefunctions = {}
end

function SimUpdater:AddUpdateFunction(fn)
    table.insert(self.updatefunctions, fn)
end

function SimUpdater:RemoveUpdateFunction(fn)
    for k, v in pairs(self.updatefunctions) do
        if v == fn then
            table.remove(self.updatefunctions, fn)
            return
        end
    end
end


function SimUpdater:StartUpdating(cmp)
    local t = self.updating_components[cmp._classname]
    self.stop_updaters[cmp] = nil
    if t and t[cmp] then return end
    self.new_updaters[cmp] = true
end

function SimUpdater:StopUpdating(cmp)
    local t = self.updating_components[cmp._classname]
    if t and t[cmp] then
        self.stop_updaters[cmp] = true
    end
    self.new_updaters[cmp] = nil
end

function SimUpdater:Update(dt)

    engine.inst:ProfilerPush(self.profilename)


    for k,v in ipairs(self.updatefunctions) do
        v(self.sim, dt)
    end

    for k,v in pairs(self.new_updaters) do
        local cmp_type = k._classname
        
        if not self.updating_components[cmp_type] then
            self.updating_components[cmp_type] = {}
            self.updating_component_iter_list[cmp_type] = {}

            table.insert(self.updating_component_order, {name = k._classname, sortkey=k.UpdateOrderSortKey or 0, class = k._class, list = self.updating_components[cmp_type], iterlist = self.updating_component_iter_list[cmp_type]} )
            table.sort(self.updating_component_order, function(a, b) return a.sortkey < b.sortkey  end)
        end

        assert( not self.updating_components[cmp_type][k] )
        table.insert(self.updating_component_iter_list[cmp_type], k)
        self.updating_components[cmp_type][k] = true
        self.num_updaters = self.num_updaters + 1
    end
    table.clear(self.new_updaters)

    for cmp,_ in pairs(self.stop_updaters) do
        local t = self.updating_components[cmp._classname]
        if t and t[cmp] then
            t[cmp] = nil
            self.num_updaters = self.num_updaters - 1

            for k = 1, #self.updating_component_iter_list[cmp._classname] do
                if self.updating_component_iter_list[cmp._classname][k] == cmp then
                    table.remove(self.updating_component_iter_list[cmp._classname], k)
                    break
                end
            end
        end
    end
    table.clear(self.stop_updaters) 

    -- Invoke the actual updates once lists have been refreshed.
    for k,v in ipairs(self.updating_component_order) do
        if #v.iterlist > 0 then
            engine.inst:ProfilerPush(v.name)
            local onupdate = v.class[self.updatefnname]
            for k = 1,  #v.iterlist do
                local cmp = v.iterlist[k]
                if cmp.entity and not cmp.entity.removed then
                    onupdate(cmp, dt)
                else
                    self:StopUpdating(cmp)
                end
            end
            engine.inst:ProfilerPop()
        end
    end

    engine.inst:ProfilerPop()
end
