
local TestCreator = class("TestCreator")

function TestCreator:init()
    self.dims = {}
end

function TestCreator:AddRelationshipSwitch(cast_id)
    local dim = {values = {}}

    local loved_fn = function(quest) quest:GetCastMember(cast_id).social_connections:SetRelationship(RELATIONSHIP.LOVED, TheGame:GetGameState():GetPlayerAgent())  end
     table.insert(dim.values, {
        name = loc.format( "{1} LOVED", cast_id ),
        fn = loved_fn})
    
    local neutral_fn = function(quest) quest:GetCastMember(cast_id).social_connections:SetRelationship(RELATIONSHIP.NEUTRAL, TheGame:GetGameState():GetPlayerAgent())  end
    table.insert(dim.values, {
        name = loc.format( "{1} NEUTRAL", cast_id ),
        fn = neutral_fn})
    
    local hated_fn = function(quest) quest:GetCastMember(cast_id).social_connections:SetRelationship(RELATIONSHIP.HATED, TheGame:GetGameState():GetPlayerAgent())  end
    table.insert(dim.values, {
        name = loc.format( "{1} HATED", cast_id ),
        fn = hated_fn})

    table.insert(self.dims, dim) 

    local opts = { 
            {text = "NEUTRAL", fn = neutral_fn},
            {text = "LOVED", fn = loved_fn}, 
            {text = "HATED", fn = hated_fn}}
    
    return self
end

function TestCreator:OverrideCast(name, fn)
    self.cast_overrides[name] = fn
    return self
end

function TestCreator:AddGameFn(name, fn)
    local dim = {
        --name = loc.format( "Do {1}", name ), 
        values = {
            {
                name = name, 
                game_fn = fn,
            },
            {
                --name = "NO", 
                game_fn = function() end,
            },
        }
    }
    table.insert(self.dims, dim)

    return self
end

function TestCreator:AddQuestFn(name, fn)
    local dim = {
        --name = loc.format( "Do {1}", name ), 
        values = {
            {
                name = name, 
                quest_fn = fn,
            },
            {
                --name = "NO", 
                quest_fn = function(quest)  end,
            },
        }
    }
    table.insert(self.dims, dim)

    local opts = {{text = "NO", fn = function() end}, {text = "YES", fn = fn}}
    return self
end

function TestCreator:AddBoolParams(params)
    for k,param in ipairs(params) do
        local dim = {
            --name = loc.format( "{1}=", param ), 
            values = {
                {
                    name = param, 
                    param_name = param,
                    param_value = true,
                },
                {
                    --name = "F", 
                    param_name = param,
                    param_value = false,
                },
            }
        }
        table.insert(self.dims, dim)

        local opts = {{text = "False", val = false}, {text = "True", val = true}}
    end

    return self
end

function TestCreator:AddParam(name, opts)
    local dim = { values = {} }
    for k,v in ipairs(opts) do
        table.insert( dim.values, {name = tostring(k), param_name = name, param_value = v})
    end
    table.insert(self.dims, dim)

    local h_opts = {}
    for k,v in ipairs(opts) do
        local text = tostr(v)
        table.insert( h_opts, {text = text, val = v} )
    end

    return self
end

function TestCreator:AddChoices(choices)
    for k,choice in ipairs(choices) do
        local dim = {
            --name = loc.format( "{1}?", choice ), 
            values = {
                {
                    name = "?"..choice, 
                    choice_name = choice,
                    choice_weight = 10,
                },
                {
                    --name = "N", 
                    choice_name = choice,
                    choice_weight = -10,
                },
            }
        }
        table.insert(self.dims, dim)
    end

    return self
end

--a 'one-hot' choice selector
function TestCreator:AddChoiceSet(choices)
    local dim = { values = {} }

    for k,selected_choice in ipairs(choices) do
        local choice_set = {}
        choice_set[selected_choice] = 10
        table.insert( dim.values, {choice_set = choice_set} )
    end
    table.insert(self.dims, dim)
    return self
end

function TestCreator:AddGameSetup(fn)
    self.game_setup_fns = self.game_setup_fns or {}
    table.insert(self.game_setup_fns, fn)
    return self
end

function TestCreator:AddQuestSetup(fn)
    self.quest_setup_fns = self.quest_setup_fns or {}
    table.insert(self.quest_setup_fns, fn)
    return self
end

function TestCreator:AddNegotiationDimension()
    local dim = { values = { {negotiation_policy = "WIN"}, {negotiation_policy = "LOSE"} } }
    table.insert(self.dims, dim)
    return self
end

function TestCreator:AddBattleDimension()
    local dim = { values = { {battle_policy = "KILL"}, {battle_policy = "SPARE"} } }
    table.insert(self.dims, dim)
    return self
end

function TestCreator:SetChoices(choices)
    self.set_choices = choices
    return self
end

function TestCreator:SetParams(params)
    self.set_params = self.set_params or {}
    for k,v in pairs(params) do
        self.set_params[k] = v
    end
    return self
end

function TestCreator:SetReps(reps)
    self.reps = reps
    return self
end


function TestCreator:GenerateTest()
    local dims = {}
    for k,v in ipairs(self.dims) do
        table.insert(dims, #v.values)
    end
    local input_vectors = CalcInputVectors(dims)
    local tests = {}

    for k, input_vector in ipairs(input_vectors) do
        local name = {loc.format( "TEST {1}: ", k) }
        local test = {
            reps = self.reps or 1,
            overrides = {parameters = {}},
            cast_overrides = self.cast_overrides,
            choice_weights = {},
            battle_policy = "RANDOM",
            negotiation_policy = "RANDOM",
            game_setup_fns = self.game_setup_fns and shallowcopy(self.game_setup_fns) or {},
            quest_setup_fns = self.quest_setup_fns and shallowcopy(self.quest_setup_fns) or {},
        }
        
        if self.set_choices then
            for k,v in ipairs( self.set_choices ) do
                test.choice_weights[v] = 10
            end
        end

        if self.set_params then
            for k,v in pairs(self.set_params) do
                test.overrides.parameters[k] = v
            end
        end


        for dim_idx, value_idx in ipairs(input_vector) do
            local dim = self.dims[dim_idx]
            local value = dim.values[value_idx]
            table.insert(name, loc.format("{1}{2}", dim.name or"", value.name or ""))

            if value.param_name then
                test.overrides.parameters[value.param_name] = value.param_value
            end

            if value.battle_policy then
                test.battle_policy = value.battle_policy
            end

            if value.negotiation_policy then
                test.negotiation_policy = value.negotiation_policy
            end


            if value.choice_name then
                test.choice_weights[value.choice_name] = value.choice_weight
            end

            if value.choice_set then
                for k,v in pairs(value.choice_set) do
                    test.choice_weights[k] = v
                end
            end

            if value.game_fn then
                table.insert( test.game_setup_fns, value.game_fn)
            end
            
            if value.quest_fn then
                table.insert( test.quest_setup_fns, value.quest_fn)
            end
        end

        test.name = table.concat(name, "|")
        table.insert(tests, test)
    end

    return tests
end


