local Tracker = class ("Tracker")

function Tracker:init( gamestate )

    --dynamic data
    self:SetupTracker( gamestate )
end

function Tracker:PostLoad()
    self:UpdateAllQuests()
end


function Tracker:OnGlobalEvent( eventname, ... )
    if eventname == "quests_changed" then
        local quest = ...
        self:UpdateQuest( quest )
    elseif eventname == "quest_destroyed" then
        local quest = ...
        self:ClearQuest( quest )
    end
end

function Tracker:SetupTracker( gamestate )
    self.gamestate = gamestate
    self.by_quest = {}
    self.all_tracked_objects = {}

    TheGame:GetEvents():ListenForEvents( self, "quests_changed", "quest_destroyed" )
end

function Tracker:DestroyTracker()
    TheGame:GetEvents():RemoveListener( self )
end


function Tracker:ClearQuest(quest)
    self.key_table_dirty = true
    self.by_quest[quest] = nil
end

function Tracker:UpdateQuest(quest)
    self.key_table_dirty = true

    local quest_table = self.by_quest[quest]
    if quest_table == nil then
        quest_table = {}
        self.by_quest[quest] = quest_table
    else
        table.clear(quest_table)
    end

    -- Collect a map of [cast_member] -> list of marking objective_ids
    quest:CollectMarkedCast( quest_table )
end

function Tracker:UpdateAllQuests()
    for i, quest in self.gamestate:ActiveQuests() do
        self:UpdateQuest(quest)
    end
end

function Tracker:RebuildTrackedObjects()
    if self.key_table_dirty then
        local tracked = self.all_tracked_objects
        table.clear( tracked )

       
        for quest, t in pairs(self.by_quest) do
            if not quest:IsQuestHidden() then
                for cast_member, objective_ids in pairs( t ) do
                    if tracked[ cast_member ] == nil then
                        tracked[ cast_member ] = {}
                    end
                    local userdata = { quest = quest, objective_ids = objective_ids }
                    table.insert( tracked[ cast_member ], userdata )
                end
            end
        end

        self.key_table_dirty = false
    end
end
--------------------------------------------

function Tracker:IsTracked( obj )
    self:RebuildTrackedObjects()
    return self.all_tracked_objects[ obj ] ~= nil
end

local function RecurseTrackedObjectsInternal( self, obj, fn )
    local sources = self.all_tracked_objects[ obj ]
    if fn( obj, sources ) then
        if is_instance( obj, Agent ) then

        elseif is_instance( obj, Location ) then
            for i, agent in obj:Agents() do
                RecurseTrackedObjectsInternal( self, agent, fn )
            end
        end
    end
end

function Tracker:RecurseTrackedObjects( obj, fn )
    self:RebuildTrackedObjects()
    RecurseTrackedObjectsInternal( self, obj, fn )
end
