
local ReticuleMixin = {}

ReticuleMixin.CONTROL_MAP =
{ 
    {
        controls = { Controls.Digital.MENU_CANCEL, Controls.Digital.MENU_UP, Controls.Digital.MENU_RIGHT, Controls.Digital.MENU_DOWN, Controls.Digital.MENU_LEFT },
        test = function(self)
            return self.show_reticule == true
        end,
        fn = function(self)
            self:SetReticuleMode( false )
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.NEGOTIATION_PANEL.CONTROLS_LEFT_INSPECT_RELEASE", Controls.Digital.MENU_CANCEL ) )
        end,
    },
    {
        hint = function(self, left, right)
            if self.show_reticule then
                table.insert(left, LOC"UI.NEGOTIATION_PANEL.CONTROLS_LEFT_MOVE_INSPECT" )
            else
                table.insert(left, LOC"UI.NEGOTIATION_PANEL.CONTROLS_LEFT_INSPECT" )
            end
        end,
    },
}

function ReticuleMixin:Mixin( class )
    assert( is_class( class, Widget.Screen ))
    self:MixinValue( class, "WantsReticuleMode" )
    self:MixinValue( class, "SetReticuleMode" )
    self:MixinValue( class, "TestReticule" )
    self:MixinValue( class, "OnUpdateReticule" )
    self:MixinControlMap( class )

    local OnUpdate = class.OnUpdate
    if OnUpdate == nil then
        class.OnUpdate = self.OnUpdateReticule
    else
        class.OnUpdate = function( self, dt )
            OnUpdate( self, dt )
            self:OnUpdateReticule( dt )
        end
    end
end

function ReticuleMixin:MixinValue( class, name )
    local value = self[ name ]
    assert( value ~= nil )
    assert( rawget( class, name ) == nil, name )
    class[ name ] = value
end

function ReticuleMixin:MixinControlMap( class )
    if class.CONTROL_MAP then
        table.arrayadd( class.CONTROL_MAP, self.CONTROL_MAP )
    else
        class.CONTROL_MAP = self.CONTROL_MAP
    end
end

function ReticuleMixin:OnUpdateReticule( dt )
    if self.fe:IsGamepadMode() then
        local lx,ly,rx,ry = TheGame:GetInput():GetActiveGamepadStickVals()
        if math.length2d( rx, ry ) > 0.5 then
            self:SetReticuleMode( true )
        elseif math.length2d( lx, ly ) > 0.5 then
            self:SetReticuleMode( false )
        end
    else
        self:SetReticuleMode( false )
    end
end

function ReticuleMixin:WantsReticuleMode()
    return self.show_reticule
end

function ReticuleMixin:SetReticuleMode( show_reticule )
    if show_reticule ~= self.show_reticule then
        print( self, "SetReticuleMode", show_reticule )
        self.show_reticule = show_reticule

        if not self.show_reticule then
            self:SetDefaultFocus()
        end

        self.fe:RefreshControlOptions()
   end
end


function ReticuleMixin:TestReticule(wx, wy)
    local main_overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    if main_overlay then
        local mouse_blocked, hover_widget = main_overlay:CheckMouseHover(wx, wy)
        if hover_widget then
            return hover_widget
        end
    end

    local mouse_blocked, hover_widget = self:CheckMouseHover(wx, wy)
    if hover_widget then
        return hover_widget
    end
end


return ReticuleMixin
