local DebugUtil = require "debug/debug_util"
require "ui/screens/conversationscreen"
require "ui/widgets/negotiationpreviewpanel"
require "ui/screens/mainoverlay"
require "encounter/base_hub"


-------------------------------------------------------------
-- SCREEN IS NOT USED IN GAME, ONLY FOR DEBUGGING USE.

local BattlePreviewScreen = class( "Screen.BattlePreviewScreen", Screen.ConversationScreen )

function BattlePreviewScreen:init()
    BattlePreviewScreen._base.init( self )

    self.agent_search = {}

    self.plax = Widget.Plax()
    self:AddChild( self.plax, 1 ):SetAnchors( "center", "center" )

    self:LoadPreset()
end

function BattlePreviewScreen:OnOpen()
    self:ShowPreview()
end

function BattlePreviewScreen:CreateScenario()
    TheGame.gamestate = GameState()

    if self.preset.player_id and Content.GetCharacterDef( self.preset.player_id[1] ) then
        TheGame:GetGameState():AddPlayerAgent( Agent( self.preset.player_id[1] ))
    else
        TheGame:GetGameState():AddPlayerAgent( Agent( "SAL" ))
    end
    if self.preset.outfit_id then
        TheGame:GetGameState():GetPlayerAgent().build = self.preset.outfit_id
    end

    local enemy
    if self.preset.enemy_id and Content.GetCharacterDef( self.preset.enemy_id[1] ) then
        local id, skin_uuid = table.unpack( self.preset.enemy_id )
        local skin_table = Content.GetCharacterSkin( skin_uuid )
        enemy = TheGame:GetGameState():AddAgent( Agent( id, skin_table ))
    else
        enemy = TheGame:GetGameState():AddAgent( Agent( "SPARK_BARON_GOON" ))
    end

    -- Show Battle Preview
    local params = {
        -- advantage = TEAM.BLUE,
        flags = self.preset.flags,
        noncombatants = { },
        allies = { },
        enemies = { enemy },
    }

    local scenario = Scenario( TheGame:GetGameState():GetPlayerAgent():GetLocation(), params )
    scenario:AddAgentToTeam( TheGame:GetGameState():GetPlayerAgent(), TEAM.BLUE, SUPPORT_REASON.PARTY)
    scenario:SetPlayerControlled( TheGame:GetGameState():GetPlayerAgent() )
    local non_combat = {}
    if params.noncombatants then
        for k,v in pairs(params.noncombatants) do
            non_combat[v] = true
        end
    end
    if params.allies then
        for i, cast in ipairs( params.allies ) do
            if cast and scenario:GetTeamForAgent( cast ) == nil  then
                scenario:AddAgentToTeam( cast, scenario:GetPlayerTeam(), SUPPORT_REASON.ALLY, LOC"SUPPORT_DESC.ALLY")
            end
        end
    end
    if params.enemies then
        for i, cast in ipairs( params.enemies ) do
            local agent = cast
            if agent and not non_combat[agent] then
                scenario:AddAgentToTeam( agent, scenario:GetEnemyTeam() )
                if not agent:IsInPlayerParty() and agent:IsLeader() then
                    for k,party_agent in agent:Party() do
                        if party_agent ~= agent and party_agent:IsCrewMember() and party_agent:IsWithAgent( agent ) and not non_combat[party_agent] and  scenario:GetTeamForAgent( party_agent ) == nil then
                            scenario:AddAgentToTeam(party_agent, TEAM.RED, SUPPORT_REASON.PARTY, LOC"SUPPORT_DESC.SAME_PARTY")
                        end
                    end
                end
            end
        end
    end

    return scenario
end

function BattlePreviewScreen:ShowPreview()
    if self.panel then
        self.panel:Remove()
    end

    self.panel = self:AddChild( Widget.NegotiationPreviewPanel( QCARD_TYPE.BATTLE ) )
        :SetAnchors("center","center" )
        :Hide()

    self.panel:Refresh( self:CreateScenario() )
    self.panel:FadeIn()
end

function BattlePreviewScreen:LoadPreset()
    local data = DebugUtil.GetLocalDebugData("BATTLE_PREVIEW_SCREEN")
    local load_ok
    local preset
    if data then
        local fn, err = loadstring(data)
        if err then
            LOGWARN( "Failed to load preset: %s", err )
        else
            load_ok, preset = pcall( fn )
        end
    end

    preset = preset or {}
    
    if type(preset) == "string" then
        LOGWARN( "Failed to load preset: %s", preset )
        preset = nil
    end

    if type(preset.enemy_id) ~= "table" then
        preset.enemy_id = { "SPARK_BARON_GOON" }
    end
    if type(preset.player_id) ~= "table" then
        preset.player_id = { "SAL" }
    end

    self.preset = preset or {}
end

function BattlePreviewScreen:SavePreset()
    local s = engine.LuaSerializer()
    local data = s:Serialize( self.preset )
    DebugUtil.SetLocalDebugData("BATTLE_PREVIEW_SCREEN", data )
    DebugUtil.SaveLocalDebugData()
end

function BattlePreviewScreen:RenderDebugUI( ui, panel )
    local player_id = self.preset.player_id and self.preset.player_id[1] or "???"
    if ui:Button( player_id .. "##PLAYER" ) then
        ui:OpenPopup( "POPUP_PLAYER" )
    end

    ui:SameLine( nil, 20 )
    local txt = self.preset.enemy_id and self.preset.enemy_id[1] or "???"
    if ui:Button( txt .. "##BATTLE" ) then
        ui:OpenPopup( "POPUP_ENEMY" )
    end

    local outfits = {}
    for k,v in ipairs( Content.GetOutfitsForCharacter(player_id) ) do
        table.insert(outfits, v.build)
    end

    local idx = table.find( outfits, self.preset.outfit_id ) or 1
    local outfit_idx = ui:Combo( "Outfit", idx, outfits )
    if outfit_idx ~= idx then
        self.preset.outfit_id = outfits[ outfit_idx ]
        self:SavePreset()
        self:ShowPreview()
    end

    if ui:BeginPopup( "POPUP_PLAYER" ) then
        local function IsPlayer( def )
            return def.base_def == "PLAYER_BASE"
        end
        local id, skin_table = DebugUtil.ShowAgentSearch(self.agent_search, ui, IsPlayer )
        if id then
            self.preset.player_id = { id, skin_table and skin_table.uuid }
            self:SavePreset()
            self:ShowPreview()
        end
        ui:EndPopup()
    end

    if ui:BeginPopup( "POPUP_ENEMY" ) then
        local id, skin_table = DebugUtil.ShowAgentSearch(self.agent_search, ui )
        if id then
            self.preset.enemy_id = { id, skin_table and skin_table.uuid }
            self:SavePreset()
            self:ShowPreview()
        end
        ui:EndPopup()
    end

    ui:Separator()

    local new_flags = DebugUtil.RenderBitField( ui, self.preset.flags or 0, NEGOTIATION_FLAGS, NEGOTIATION_FLAGS_STRINGS )
    if new_flags ~= self.preset.flags then
        self.preset.flags =  new_flags
        self:ShowPreview()
        self:SavePreset()
    end
end
