local BoonsBanesScreen = class( "Screen.BoonsBanesScreen", Widget.Screen )

local PANEL_W = RES_X * 0.95
local PANEL_H = RES_Y * 0.70
local PANEL_PADDING = 8
local CONTAINER_W = RES_X * 0.75
local CONTAINER_H = PANEL_H - PANEL_PADDING*2
local GRAFT_SLOT_SIZE = 110
local DETAILS_W = RES_X*0.6
local GRAFT_SLOT_W = (DETAILS_W-SPACING.M1)/2

BoonsBanesScreen.CONTROL_MAP = {
    {
        hint = function(self, left, right)
            table.insert(left, LOC"UI.CONTROLS.NAV")
        end
    },
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self)
            AUDIO:PlayEvent( "event:/ui/grafts_menu/close" )
            self:FadeOut()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end
    }
}


function BoonsBanesScreen:init( owner )
    BoonsBanesScreen._base.init( self )

    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.RELATIONSHIP_LOVED )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Setup character anim display
    self.owner = owner or TheGame:GetGameState():GetPlayerAgent()
    self.character = self:AddChild( Widget.AnimatedCharacter() )
        :SetScale( 1, 1 )
        :Refresh( self.owner )
    self.character:SetHoverOutlineColour( UICOLOURS.RELATIONSHIP_LOVED )
    self.character_animation_done = true
    self.character.anim:SetAnimDoneFn( function() self.character_animation_done = true end )
    self.character:SetClickFn( function()
        if self.character_animation_done then
            self.character_animation_done = false
            self.character:Emote( "greeting", true )
        end
    end )
    self.character:SetAutoLoopEnabled( false )
    self.character:Emote( "greeting", true )
    self.character:SetPos( -RES_X*0.35, -350 )

    -- Our scroll container
    self.scroll = self:AddChild( Widget.ScrollPanel() )
        :SetVirtualMargin( 0 )
        :SetVirtualTopMargin( SPACING.M1*4 )
        :SetVirtualBottomMargin( SPACING.M1*4 )
        :SetScrollBarOuterMargin( 0 )
        :ShowBar( SCROLLBAR.NEVER )
        :SetSize( DETAILS_W+SPACING.M1*4, self.height )
    self.content = self.scroll:AddScrollChild( Widget() )

    -- Show text info
    self.text_content = self.content:AddChild(Widget())
    self.title = self.text_content:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_TITLE ) )
        :SetText( loc.upper( loc.format( LOC"UI.BOONS_BANES_SCREEN.TITLE", self.owner:GetName() ) ) )
        :SetGlyphColour( UICOLOURS.RELATIONSHIP_LOVED )
        :SetAutoSize(DETAILS_W)
        :LeftAlign()
        :SetWordWrap(true)
        :Bloom(0.15)
    self.subtitle = self.text_content:AddChild( Widget.Label("title", FONT_SIZE.BODY_TEXT ) )
        :SetText( loc.format( LOC"UI.BOONS_BANES_SCREEN.DESC", self.owner:GetName() ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetTintColour( UICOLOURS.RELATIONSHIP_LOVED )
        :SetAutoSize(DETAILS_W)
        :LeftAlign()
        :SetWordWrap(true)
        :SetTintAlpha( 0.8 )
        :Bloom(0.1)

    -- Containers for all the graft widgets
    self.boon_widgets = self.content:AddChild( Widget() )
    self.bane_widgets = self.content:AddChild( Widget() )

    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.close_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", 
        function()
            AUDIO:PlayEvent( "event:/ui/grafts_menu/close" )
            self:FadeOut()
        end ) )
    self.close_button:SetIcon( global_images.close )
    self.close_button:LayoutBounds( "left", "above", 60, 60 )
    
    self:Refresh()
end

function BoonsBanesScreen:Refresh()

    -- Remove old ones
    self.boon_widgets:DestroyAllChildren()
    self.bane_widgets:DestroyAllChildren()

    if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() then
        local player_agent = TheGame:GetGameState():GetPlayerAgent()
        local graft_owner = player_agent:GetAspect("graft_owner")
        local grafts = graft_owner:GetGrafts( GRAFT_TYPE.SOCIAL )

        -- Add boons
        if #grafts > 0 then
            for k, graft in ipairs( grafts ) do
                -- Create widget
                local widget = Widget.SocialGraftDetails( GRAFT_SLOT_W, player_agent, true )
                    -- :HideFrame()
                    :Refresh( graft )
                -- Add to respective container
                if graft:GetDef().is_boon then
                    self.boon_widgets:AddChild( widget )
                    -- if self.click_fn then widget:SetOnClickFn( self.click_fn ) end
                    -- if self.tooltiplayoutfn then widget:SetToolTipLayoutFn( self.tooltiplayoutfn ) end
                else
                    self.bane_widgets:AddChild( widget )
                end
            end
            self:Layout()
        end
    end

    return self
end

function BoonsBanesScreen:Layout()

	self.text_content:SetPos( 0, 0 )
    self.subtitle:LayoutBounds( "left", "below", self.title )
        :Offset( 0, SPACING.TITLE_DESC )
    self.boon_widgets:StackChildren( SPACING.M1/2 )
    	:LayoutBounds( "left", "below", self.text_content ):Offset( 0, -30 )
    self.bane_widgets:StackChildren( SPACING.M1/2 )
    	:LayoutBounds( "after", "top", self.boon_widgets ):Offset( SPACING.M1, 0 )

    -- Get MainOverlay's top bar height
    local overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    local overlay_topbar_height = overlay and overlay:GetTopBarHeight() or 0

	-- Calculate scroll panel size
    local left_offset = RES_X*0.15 -- How much the grid should cross to the left of the screen center
    local left_padding = 100 -- How much space should there be between the left edge of the scroll area and the first graft (allows space for glow/scaling without clipping)
    local left_screen_padding = 15
    -- self.scroll:SetLeftPadding( left_padding )
    self.content:LayoutBounds( "left", "top", 0, 0 )
    self.scroll:SetSize( self.screen_w/2 + left_offset + left_padding, self.screen_h - overlay_topbar_height )
        :SetPos( self.screen_w/4 - left_offset/2 - left_padding/2 - left_screen_padding, -overlay_topbar_height/2 )
        :RefreshView()

	return self
end

function BoonsBanesScreen:OnControlModeChange( cm, device_id )
    BoonsBanesScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.bottom_left:Show()
        self.scroll:ShowBar( SCROLLBAR.IF_NEEDED )
    end
    if cm == CONTROL_MODE.TOUCH then
        self.bottom_left:Show()
        self.scroll:ShowBar( SCROLLBAR.NEVER )
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.bottom_left:Hide()
        self.scroll:ShowBar( SCROLLBAR.NEVER )

        local first = self:GetDefaultFocus()
        if first then first:SetFocus() end
    end
end

function BoonsBanesScreen:OnScreenModeChange( sm )
    BoonsBanesScreen._base.OnScreenModeChange( self, sm )

    local content_w = DETAILS_W
    if sm == SCREEN_MODE.MONITOR then
    elseif sm == SCREEN_MODE.TV then
        content_w = content_w * 0.9
    elseif sm == SCREEN_MODE.SMALL then
        content_w = content_w * 0.8
    end

    self.title:SetAutoSize(content_w)
    self.subtitle:SetAutoSize(content_w)
    for k, v in ipairs( self.boon_widgets.children ) do
    	v:SetWidth( (content_w-SPACING.M1)/2 )
    end
    for k, v in ipairs( self.bane_widgets.children ) do
    	v:SetWidth( (content_w-SPACING.M1)/2 )
    end

    self.content:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:Layout()
end

function BoonsBanesScreen:GetDefaultFocus()
    for k, v in ipairs( self.boon_widgets.children ) do
    	if v:IsShown() then return v end
    end
    for k, v in ipairs( self.bane_widgets.children ) do
    	if v:IsShown() then return v end
    end
end

------------------------------------------------------------------------------------------
-- Shows the selected social graft's details

local SocialGraftDetails = class( "Widget.SocialGraftDetails", Widget.GraftDetails )

SocialGraftDetails.CONTROL_MAP = {
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function(self)
            return self and self.onclickfn
        end,
        fn = function(self)
            self:Click()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.BOONS_BANES_SCREEN.CONTROLS_OPEN_GRAFT", Controls.Digital.MENU_ACCEPT))
        end
    }
}

function SocialGraftDetails:init( content_w, owner )
    SocialGraftDetails._base.init( self, content_w, owner, true )
end

function SocialGraftDetails:Refresh(graft)
    self.graft = graft
    table.clear( self.card_keywords )

    if self.graft then

    	-- Get more data
    	self.graft_def = self.graft:GetDef()
    	self.graft_agents = self.graft.userdata and self.graft.userdata.agents or {}


        -- Update icon
        self.graft_icon:Refresh(self.graft)

        -- Update text
        self.name:SetText( loc.upper( self.graft:GetName() ) )
        if self.graft_def.is_boon then
            self.name:SetGlyphColour( UICOLOURS.RELATIONSHIP_LOVED )
            self.background:SetTintColour( UICOLOURS.RELATIONSHIP_LOVED )
            if self.new_graft_background then self.new_graft_background:SetTintColour( UICOLOURS.RELATIONSHIP_LOVED ):SetTintAlpha( 0.4 ) end
        	self.subtitle:SetText( loc.format( LOC"UI.BOONS_BANES_SCREEN.SOCIAL_BOON", self.graft_agents ) )
        		:SetGlyphColour( UICOLOURS.RELATIONSHIP_LIKED )
        		:SetHoverLinkGlyphColour( UICOLOURS.RELATIONSHIP_LIKED )
        	-- Set click to go to relationship screen
    		self:SetOnClickFn( function()
    			TheGame:GetGameState().relationship_screen_last_tab = RELATIONSHIP.LOVED
    			TheGame:FE():InsertScreen( Screen.RelationshipScreen( TheGame:GetGameState() ) )
    				:AnimateToAgent( self.graft_agents[1] )
    		end )
        else
            self.name:SetGlyphColour( UICOLOURS.RELATIONSHIP_HATED )
            self.background:SetTintColour( UICOLOURS.RELATIONSHIP_HATED )
            if self.new_graft_background then self.new_graft_background:SetTintColour( UICOLOURS.RELATIONSHIP_HATED ):SetTintAlpha( 0.4 ) end
        	self.subtitle:SetText( loc.format( LOC"UI.BOONS_BANES_SCREEN.SOCIAL_BANE", self.graft_agents ) )
        		:SetGlyphColour( UICOLOURS.RELATIONSHIP_DISLIKED )
        		:SetHoverLinkGlyphColour( UICOLOURS.RELATIONSHIP_DISLIKED )
        	-- Set click to go to relationship screen
    		self:SetOnClickFn( function()
    			TheGame:GetGameState().relationship_screen_last_tab = RELATIONSHIP.HATED
    			TheGame:FE():InsertScreen( Screen.RelationshipScreen( TheGame:GetGameState() ) )
    				:AnimateToAgent( self.graft_agents[1] )
    		end )
        end

        if self.desc then self.desc:SetText( self.graft:GetDesc(self) ) end

        -- Go through all the keywords for this graft
        self:SetToolTipClass( Widget.TooltipKeywords )
        self:SetToolTip( self.card_keywords )
    end

    self:Layout()
    return self
end