
local CardPreviewScreen = class("Screen.CardPreviewScreen", Widget.Screen)

function CardPreviewScreen:init( card_id )
    Widget.Screen.init(self)
    
    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )
    
    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    
    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- A hitbox for alignment
    self.area_w = RES_X*0.8
    self.area_h = RES_Y*0.8
    
    -- The animation box
    self.animation_box = self:AddChild( Widget() )
        :SetPos( 0, 50)
    self.animation_box_frame = self.animation_box:AddChild( Widget.Panel( engine.asset.Texture("ui/empty_slot_frame.tex") ) )
        :SetNineSliceCoords( 30, 30, 44, 44)
        :SetSize( self.area_w, self.area_h )
        :SetScale( 1, -1 )
        :Bloom( 0.2 )
        :UnblockMouse()
    self.animation_box_label = self.animation_box:AddChild( Widget.Label( "body", 24, "SAFE AREA FOR ANIMATION RECORDING" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize( 200 )
        :Bloom( 0.15 )
        :SetWordWrap( true )
        :UnblockMouse()
    self.hitbox = self.animation_box:AddChild( Widget.SolidBox( self.area_w, self.area_h, 0xFF00FFFF ) )
        :Hide()
    self.anim_container = self.animation_box:AddChild( Widget() )

    self.card_source = self:AddChild( Widget.SolidBox( 10, 10, 0x00FF00aa ) ):Hide()
    self.card_centre = self:AddChild( Widget.SolidBox( 10, 10, 0x00FF00aa ) ):Hide()
    self.card_dest = self:AddChild( Widget.SolidBox( 10, 10, 0x00FF00aa ) ):Hide()

    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.bottom_center = self:AddChild( Widget() ):SetAnchors( "center", "bottom" )
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )

    -- Back button
    self.close_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", function() self:DoClose() end ) )
    self.close_button:SetIcon( global_images.close )
    self.close_button:LayoutBounds( "left", "above", 60, 60 )
    self:RegisterHotkeyButton( Controls.Digital.MENU_CANCEL, self.close_button )

    -- Deck button
    self.deck_button = self.bottom_left:AddChild( Widget.IconButton( "Preview decks", function() self:OpenDeckScreen() end ) )
    self.deck_button:SetIcon( engine.asset.Texture( "UI/ic_buy_combat.tex" ) )
    self.deck_button:LayoutBounds( "after", "center", self.close_button ):Offset( 20, 0 )

    -- Export button
    self.export_button = self.bottom_right:AddChild( Widget.IconButton( "Export card image", function() self:ExportImage() end ) )
    self.export_button:SetIcon( engine.asset.Texture( "UI/ic_items_enabled.tex" ) )
    self.export_button:LayoutBounds( "right", "above", -60, 60 )
    -- Card selection
    self.negotiation_label = self.bottom_right:AddChild( Widget.Label( "body", 22, "NEGOTIATION CARDS" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.negotiation_selector = self.bottom_right:AddChild( Widget.DropDown() )
        :SetFn( function(...) self:SetCard(...) end )
        :SetDirectionUp()
        :SetToolTip( "All the Negotiation cards in the game, including upgrades" )
        :SetSize( 270, 40 )
    self.fight_label = self.bottom_right:AddChild( Widget.Label( "body", 22, "BATTLE CARDS" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.fight_selector = self.bottom_right:AddChild( Widget.DropDown() )
        :SetFn( function(...) self:SetCard(...) end )
        :SetDirectionUp()
        :SetToolTip( "All the Battle cards in the game, including upgrades" )
        :SetSize( 270, 40 )

    -- Play button
    self.play_button = self.bottom_center:AddChild( Widget.ImageButton( engine.asset.Texture( "UI/ic_nav_playaudio.tex" ), function() self:PlayAnimation() end ) )
        :SetSize( 80, 80 )
        :Bloom( 0.05 )
        :SetToolTip( "Show card-reveal animation. For trailer recording" )
        :Disable()
    -- Stop button
    self.stop_button = self.bottom_center:AddChild( Widget.ImageButton( engine.asset.Texture( "UI/ic_nav_stopaudio.tex" ), function() self:ResetAnimation() end ) )
        :SetSize( 70, 70 )
        :Bloom( 0.05 )
        :SetToolTip( "Resets the card to its original position" )
        :Disable()

    -- Animation options
    self.show_flash_anims = self.bottom_center:AddChild( Widget.CheckBox() )
        :SetText("Show anims")
        :SetFn( function(val) self.opt_show_anims = val end )
        :SetToolTip( "If the card-reveal animation should include particles/anims" )
        :SetValue( true )
    self.show_glow = self.bottom_center:AddChild( Widget.CheckBox() )
        :SetText("Show glow")
        :SetFn( function(val) 
            self.opt_show_glow = val
            if self.card_widget then
                if val then
                    self.card_widget:SetOutline( UICOLOURS.FOCUS )
                else
                    self.card_widget:SetOutline( nil )
                end
            end
        end )
        :SetToolTip( "If the card should show the focus-coloured glow around it" )
        :SetValue( true )
    self.show_chromakey = self.bottom_center:AddChild( Widget.CheckBox() )
        :SetText("Show magic pink")
        :SetFn( function(val) self.hitbox:SetShown( val ) end )
        :SetToolTip( "Shows a magic-pink background behind the card, for chroma-key screenshots" )
        :SetValue( false )
    self.show_waypoints = self.bottom_center:AddChild( Widget.CheckBox() )
        :SetText("Show waypoints")
        :SetFn( function(val) self.card_source:SetShown( val ) self.card_centre:SetShown( val ) self.card_dest:SetShown( val ) end )
        :SetToolTip( "Shows the waypoints of the card during the card-reveal animation. These are for reference only. Not editable here" )
        :SetValue( false )

    self:RefreshCardsSelection()
    self:Layout()
    self.fight_selector:SetIndex( 1 )
end

function CardPreviewScreen:RefreshCardsSelection()
    local battle_defs = require "battle/battle_defs"
    local battle_card_options = {}
    local battle_cards = BattleCardCollection():Filter( function( card_def )
        if self.filter_txt == nil or card_def.id:find(self.filter_txt) then
            return not CheckAnyBits( card_def.flags, battle_defs.CARD_FLAGS.SPECIAL | battle_defs.CARD_FLAGS.NPC )
        end
    end )
    for k, card_def in ipairs( battle_cards.items ) do
        table.insert( battle_card_options, { txt = "" .. LOC( card_def:GetLocNameKey() ) .. " <b>[" .. card_def.id .. "]</>", card_def = card_def })
    end
    table.sort( battle_card_options, function( a, b ) return a.txt < b.txt end )
    self.fight_selector:SetOptions( battle_card_options )
    self.battle_card_options = battle_card_options

    local negotiation_defs = require "negotiation/negotiation_defs"
    local negotiation_card_options = {}
    local negotiation_cards = NegotiationCardCollection()
    for k, card_def in ipairs( negotiation_cards.items ) do
        if self.filter_txt == nil or card_def.id:find(self.filter_txt) then
            if not CheckAnyBits( card_def.flags, negotiation_defs.CARD_FLAGS.SPECIAL | negotiation_defs.CARD_FLAGS.OPPONENT ) then
                table.insert( negotiation_card_options, { txt = LOC( card_def:GetLocNameKey() ) .. " [" .. card_def.id .. "]", card_def = card_def })
            end
        end
    end
    table.sort( negotiation_card_options, function( a, b ) return a.txt < b.txt end )
    self.negotiation_selector:SetOptions( negotiation_card_options )
    self.negotiation_card_options = negotiation_card_options
end

function CardPreviewScreen:SetCard( index, option )

    -- Remove existing card
    if self.card_widget then
        self.card_widget:Remove()
        self.card_widget = nil
    end

    if option.card_def._classname == "BattleCardDef" then
        -- Battle Card!
        self.card = Battle.Card( option.card_def.id )
        self.card_widget = self.anim_container:AddChild( Widget.BattleCard() )
            :RefreshCard( self.card )
    else
        -- Negotiation Card!
        self.card = Negotiation.Card( option.card_def.id )
        self.card_widget = self.anim_container:AddChild( Widget.NegotiationCard() )
            :RefreshCard( self.card )
    end

    if self.card_widget then
        self.play_button:Enable()
        if self.opt_show_glow then self.card_widget:SetOutline( UICOLOURS.FOCUS ) else self.card_widget:SetOutline( nil ) end
        self.card_widget:LayoutBounds( "center", "center", self.card_centre )
    end

end

function CardPreviewScreen:OnScreenResize(w, h)
    self._base.OnScreenResize(self, w, h)
    self:Layout()
    self:RefreshStreamerMode()
end

function CardPreviewScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
end

function CardPreviewScreen:Layout()
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Size up the grid
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Card waypoints
    self.card_source:LayoutBounds( "before", "bottom", self.animation_box_frame ):Offset( -100, -50 )
    self.card_centre:LayoutBounds( "center", "center", self.animation_box_frame )
    self.card_dest:LayoutBounds( "after", "bottom", self.animation_box_frame ):Offset( 100, -50 )

    -- Center controls
    self.play_button:LayoutBounds( "right", "above", -6, 40 )
    self.stop_button:LayoutBounds( "left", "above", 6, 45 )

    -- Card selection
    self.negotiation_selector:LayoutBounds( "before", "bottom", self.export_button ):Offset( -20, 5 )
    self.negotiation_label:LayoutBounds( "left", "above", self.negotiation_selector ):Offset( 0, 3 )
    self.fight_selector:LayoutBounds( "before", "center", self.negotiation_selector ):Offset( -20, 0 )
    self.fight_label:LayoutBounds( "left", "above", self.fight_selector ):Offset( 0, 3 )

    -- Right controls
    self.show_chromakey:LayoutBounds( "before", "center", self.play_button ):Offset( -30, 15 )
    self.show_flash_anims:LayoutBounds( "before", "center", self.show_chromakey ):Offset( -30, 0 )
    self.show_glow:LayoutBounds( "left", "below", self.show_flash_anims ):Offset( 0, -6 )
    self.show_waypoints:LayoutBounds( "left", "below", self.show_chromakey ):Offset( 0, -6 )

end

function CardPreviewScreen:DoClose()
    self:GetFE():PopScreen( self )
end

function CardPreviewScreen:PlayAnimation()
    if self.card_widget then

        local duration = 0.4
        local pause_duration = 1.0
        
        local card_source_x, card_source_y = self.card_source:GetPos()
        local card_centre_x, card_centre_y = self.card_centre:GetPos()
        local card_dest_x, card_dest_y = self.card_dest:GetPos()

        local XCURVE = 0.3
        local cx
        local cy

        self.animations_list = {}

        self.card_animation = Updater.Series{
                Updater.Do( function() self:HideAnimUI() end ),
                Updater.Wait( duration ),
                Updater.Parallel{
                    Updater.Do( function()
                        self.card_widget:SetPos( card_source_x, card_source_y )
                        cx = (card_source_x * XCURVE) + (card_centre_x * (1.0 - XCURVE))
                        cy = math.max( card_centre_y, card_source_y, math.min( card_centre_y, card_source_y ) + 500 )
                    end ),
                    Updater.Ease( function(v) self.card_widget:SetScale(v) end, 0.4, 1.5, duration, easing.inOutQuad),
                    Updater.Ease( function(v) self.card_widget:SetTintAlpha(v) end, 0, 1, duration, easing.inOutQuad),
                    Updater.Ease( function(v) self.card_widget:SetRotation(v*DEGREES) end, (card_centre_x < card_source_x) and 180 or -180, 0, duration, easing.inOutQuad),
                    Updater.Do( function() self.card_widget:CurveTo( card_centre_x, card_centre_y, cx, cy, duration, easing.inOutQuad ) end ),
                },

                Updater.Series{
                    Updater.Ease( function(v) self.card_widget:SetScale(v) end, 1.5, 1, 0.1, easing.inQuad),
                }
        }

        if self.opt_show_anims then
            self.card_animation:Add( Updater.Series{
                Updater.Do( function()
                    -- Circular impact
                    local animname = "anim/UI_negotiation_round_fx.zip"
                    local anim = self.anim_container:AddChild( Widget.Anim() )
                        :SetHiddenBoundingBox(true)
                        :SetTintColour( UICOLOURS.ADVANCEMENT )
                        :SetScale( math.random() * 1.4 + 3.8 )
                        :SetSpeed( 1.4 )
                        :SetBloom( 0.2 )
                        :SetRotation( math.random( 0, 230 )*DEGREES )
                    anim:AddAnims( GetAnim( animname ) )
                    anim:AddOverrideFile( animname )
                    anim:Play( "hit_3", false )
                    anim:SetPos( card_centre_x, card_centre_y )
                    anim:SendToBack()
                    table.insert( self.animations_list, anim ) -- For later recycling
                end ),
                Updater.Wait( 0.1 ),
                Updater.Do( function()
                    -- Particles
                    local animname = "anim/UI_negotiation_round_fx.zip"
                    local anim = self.anim_container:AddChild( Widget.Anim() )
                        :SetHiddenBoundingBox(true)
                        :SetTintColour( UICOLOURS.ADVANCEMENT )
                        :SetRotation( 24*DEGREES )
                        :SetRotation( math.random( 60, 140 )*DEGREES )
                        :SetScale( math.random() * 0.6 + 2.2 )
                        :SetSpeed( 1.0 )
                        :SetBloom( 0.2 )
                    anim:AddAnims( GetAnim( animname ) )
                    anim:AddOverrideFile( animname )
                    anim:Play( "hit_4", false )
                    anim:SetPos( card_centre_x, card_centre_y )
                    anim:SendToBack()
                    table.insert( self.animations_list, anim ) -- For later recycling
                end ),

                Updater.Wait( 0.1 ),
                Updater.Do( function()
                    -- Particles
                    local animname = "anim/UI_negotiation_round_fx.zip"
                    local anim = self.anim_container:AddChild( Widget.Anim() )
                        :SetHiddenBoundingBox(true)
                        :SetTintColour( UICOLOURS.QUEST )
                        :SetRotation( math.random( 0, 45 )*DEGREES )
                        :SetScale( 2.0 )
                        :SetScale( math.random() * 0.6 + 1.8 )
                        :SetSpeed( 0.8 )
                        :SetBloom( 0.2 )
                    anim:AddAnims( GetAnim( animname ) )
                    anim:AddOverrideFile( animname )
                    anim:Play( "hit_4", false )
                    anim:SetPos( card_centre_x, card_centre_y )
                    anim:SendToBack()
                    table.insert( self.animations_list, anim ) -- For later recycling
                end )
            } )
        end

        self.card_animation:Add( Updater.Series{
            Updater.Wait( pause_duration ),
            Updater.Parallel{
                Updater.Do( function()
                    self.card_widget:SetPos( card_centre_x, card_centre_y )
                    cx = (card_centre_x * XCURVE) + (card_dest_x * (1.0 - XCURVE))
                    cy = math.max( card_dest_y, card_centre_y, math.min( card_dest_y, card_centre_y ) + 500 )
                end ),
                Updater.Ease( function(v) self.card_widget:SetScale(v) end, 1, 0.4, duration, easing.outQuad),
                Updater.Ease( function(v) self.card_widget:SetTintAlpha(v) end, 1, 0, duration, easing.outQuad),
                Updater.Ease( function(v) self.card_widget:SetRotation(v*DEGREES) end, 0, (card_dest_x < card_centre_x) and 180 or -180, duration, easing.outQuad),
                Updater.Do( function() self.card_widget:CurveTo( card_dest_x, card_dest_y, cx, cy, duration, easing.inQuad ) end ),
            },
            Updater.Wait( duration ),
            Updater.Do( function() self:ShowAnimUI() end ),
            Updater.Do( function()
                -- Recycle anims
                for k, v in ipairs(self.animations_list) do
                    v:Remove()
                    v = nil
                end
            end ),
        } )

        self:RunUpdater( self.card_animation )

        self.stop_button:Enable()

    end
end

function CardPreviewScreen:ResetAnimation()
    if self.card_widget then
        -- Stop animation
        if self.card_animation then self.card_animation:Stop() end
        -- Reset card position
        self.card_widget:SetRotation( 0 )
            :SetTintAlpha( 1 )
            :SetScale( 1 )
            :LayoutBounds( "center", "center", self.card_centre )
        -- Show UI again
        self:ShowAnimUI()
    end
end

function CardPreviewScreen:ShowAnimUI()
    self.play_button:Enable()
    self.animation_box_frame:AlphaTo( 1, 0.1 )
    self.animation_box_label:AlphaTo( 1, 0.1 )
end

function CardPreviewScreen:HideAnimUI()
    self.play_button:Disable()
    self.animation_box_frame:AlphaTo( 0, 0.1 )
    self.animation_box_label:AlphaTo( 0, 0.1 )
end

function CardPreviewScreen:ExportImage()
    self:StartCoroutine(CardPreviewScreen.ExportImageCoro, self)
end

function CardPreviewScreen:OpenDeckScreen()
    require "ui/screens/exportdeckscreen"
    self:GetFE():PushScreen(Screen.ExportDeckScreen())
end

function CardPreviewScreen:ExportImageCoro()

    if not self.card_widget then
        return
    end

    local scale = 3
    local CARD_W = 370 * scale
    local CARD_H = 542 * scale
    local PADDING = 20 * scale

    local render_w = CARD_W+PADDING
    local render_h = CARD_H+PADDING

    local scene = engine.rendering.Scene()
    local capture = Widget.CaptureImage(scene)
        :SetSize( render_w, render_h )
    local anim_container = Widget()
    self.card_widget:Reparent( anim_container )
    self.card_widget:SetPos( 0, 0 )
        :SetOutline( nil )
    local sx, sy = self.card_widget:GetScale()
    self.card_widget:SetScale( 2 * scale, 2 * scale )
    self.card_widget.shadow:Hide()
    local root = engine.rendering.SceneGraphNode()
    root:SetScene(scene)
    root:AddChild(anim_container.node)

    self:Delay( 0.1 )

    while not engine.inst:IsAsyncLoadingComplete() do
        coroutine.yield()
    end

    capture:RenderToScene( anim_container, -render_w, -render_h, render_w, render_h )
    engine.inst:SaveTextureToPNG( capture:GetTexture(), "../image_exports/" .. self.card_widget.card.id .. ".png" );

    self.card_widget:SetScale( sx, sy )
    self.card_widget.shadow:Show()
    self.card_widget:Reparent( self.anim_container )
    self.card_widget:LayoutBounds( "center", "center", self.card_centre )
        :SetOutline( self.opt_show_glow and UICOLOURS.FOCUS or nil )

    -- Make sure to call collect garbage a lot so that capture images get their resources freed:
    collectgarbage()

    TheGame:FE():PushScreen( Screen.InfoPopup( "CARD IMAGE GENERATED", "Card image saved to Griftlands/image_exports/".. self.card_widget.card.id .. ".png" ) )

end


function CardPreviewScreen:RenderDebugUI( ui, panel )
    local filter_txt = ui:InputText( "Filter", self.filter_txt or "" ) or self.filter_txt
    if filter_txt and self.filter_txt ~= filter_txt then
        self.filter_txt = filter_txt
        self:RefreshCardsSelection()
    end
    ui:Separator()

    ui:Columns( 3 )
    for k, v in ipairs( self.battle_card_options or table.empty ) do
         if ui:Selectable( v.card_def.id, nil, ui.Selectable_SpanAllColumns ) then
            self:SetCard( k, v )
         end
        ui:NextColumn()

        ui:Text( v.card_def:GetLocalizedName() or "???" )
        ui:NextColumn()

        ui:TextColored( 0xCC0000FF, "battle" )
        ui:NextColumn()
    end
    for k, v in ipairs( self.negotiation_card_options or table.empty ) do
        if ui:Selectable( v.card_def.id, nil, ui.Selectable_SpanAllColumns ) then
            self:SetCard( k, v )
        end
        ui:NextColumn()

        ui:Text( v.card_def:GetLocalizedName() or "???" )
        ui:NextColumn()

        ui:TextColored( 0xCC00CCFF, "negotiation" )
        ui:NextColumn()
    end
    ui:Columns(1)
end
