
local CharacterAtlasScreen = class( "Screen.CharacterAtlasScreen", Widget.Screen )

function CharacterAtlasScreen:init()
    self._base.init( self )

    self.bg = self:AddChild( Widget.SolidBox( RES_X, RES_Y, 0, 0, 0, 1 )):SetAnchors( "fill", "fill" )

    self.character_scale = 0.2

    self.characters = {}
    self.scroll = Widget.ScrollPanel():SetAnchors( "center", "center" )
    self:AddChild( self.scroll ):SetSize( RES_X, RES_Y )
end

function CharacterAtlasScreen:RefreshSkins( skins )
    if skins then
        self.shown_skins = skins
    else
        self.shown_skins = {}
        local all_skins = Content.GetAllCharacterSkins()
        for content_id, skins in pairs( all_skins ) do
            table.arrayadd( self.shown_skins, skins )
        end
    end

    table.clear( self.characters )
    self.scroll:Clear()

    local row = self:AddRow()

    for i, skin in pairs( self.shown_skins ) do
        local content_id = Content.GetCharacterForSkin( skin )
        local agent = Agent( content_id, skin )
        local char = Widget.AnimatedCharacter( true ):SetScale( self.character_scale )

        char:SetToolTipClass( Widget.TooltipAgent )
        char:SetToolTip( agent )
        
        row:AddChild( char )
        char:Refresh( agent )
        char:LayoutBounds( "after", "bottom" )
        assert( char:GetSize() > 0 )
        table.insert( self.characters, char )

        if row:GetSize() > RES_X - 150 then
            row = self:AddRow( row )
        end
    end

    self:AddRow( row )
end

function CharacterAtlasScreen:SetBackgroundColour( colour )
    self.bg:SetColour( colour )
    self.bg_colour = colour
end

function CharacterAtlasScreen:GetBackgroundColour()
    return self.bg_colour or UICOLOURS.BLACK
end

function CharacterAtlasScreen:ChangeScale( delta )
    self.character_scale = clamp( self.character_scale + delta, 0.05, 0.5 )
    self:RefreshSkins( self.shown_skins )
end

function CharacterAtlasScreen:HandleControlUp( control )
    if control:Has( Controls.Digital.MENU_CANCEL ) then
        self:GetFE():PopScreen( self )
        return true

    elseif control:Has( Controls.Digital.MENU_ACCEPT ) then
        for i, char in ipairs( self.characters ) do
            char:Play( "unarmed_taunt" )
        end
        return true
    end

    return false
end


function CharacterAtlasScreen:AddRow( row )
    if row then
        row:LayoutBounds( "left", "below")
    end
    
    row = self.scroll:AppendWidget(Widget())
    self.scroll:Submit()
    return row
end
