require "ui/widgets/screen"
require "ui/widgets/cardwidget"
require "ui/widgets/widepanel"

local CARD_SCALE =
{
    [SCREEN_MODE.MONITOR] = 0.6,
    [SCREEN_MODE.TV] = 0.9,
    [SCREEN_MODE.SMALL] = 0.9,
}

local CARD_SCALE_HOVER =
{
    [SCREEN_MODE.MONITOR] = 0.65,
    [SCREEN_MODE.TV] = 0.95,
    [SCREEN_MODE.SMALL] = 0.95,
}

local PANEL_H = 
{
    [SCREEN_MODE.MONITOR] = 610,
    [SCREEN_MODE.TV] = 770,
    [SCREEN_MODE.SMALL] = 770,
}

local PANEL_W = RES_X * 0.95
local CARDS_W = PANEL_W

-------------------------------------------------------------------------------
local ChooseCardsPopup = class( "Screen.ChooseCardsPopup", Widget.Screen )

ChooseCardsPopup.CONTROL_MAP = 
{

    {
        --this one's just a hint, because the generic card widget handles the pick
        control = Controls.Digital.MENU_ACCEPT,
        test = function(self)
            return not self.hide_popup
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.ACCEPT", Controls.Digital.MENU_ACCEPT))
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_Y,
        fn = function(self) 
            self:TogglePopup()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format ( self.hide_popup and LOC"UI.CONTROLS.VIEW_CHOICES" or LOC"UI.CONTROLS.VIEW_HAND", Controls.Digital.GAMEPAD_Y))
        end,
    }
}

function ChooseCardsPopup:init( cards, min_count, max_count, pred, card_class, on_inspect_hand, on_inspect_options, on_close )
    ChooseCardsPopup._base.init( self )
    
    self.is_popup = true

    if cards == nil then
        cards = {}
        min_count, max_count = 3, 3
        for i = 1, math.random( 1, 6 ) do
            local all_cards = Content.GetAllNegotiationCards()
            local carddef = table.arraypick( all_cards )
            for j = 1, math.random( 1, 5 ) do
                table.insert( cards, Negotiation.Card( carddef.id ) )
            end
        end
        card_class = Widget.NegotiationCard
    end

    self.on_inspect_hand = on_inspect_hand
    self.on_inspect_options = on_inspect_options
    self.on_close = on_close
    self.cards = cards
    self.min_count = min_count
    self.max_count = max_count
    assert( is_class( card_class, Widget.CardWidget ))
    
    self:SetAnchors( "center", "center" )

    -- Background darkening
    self.container = self:AddChild( Widget.SolidBox( nil, nil, 0x000000A0 ):SetAnchors( "fill", "fill" ):SetSize( RES_X, RES_Y ))
    
    -- Panel
    local sm = TheGame:FE():GetScreenMode()
    self.bg_panel = self:AddChild( Widget.WidePanel( PANEL_H[ sm ], card_class == Widget.NegotiationCard and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT ) )
        :SetIcon( card_class == Widget.NegotiationCard and engine.asset.Texture("ui/card_type_negotiation.tex") or engine.asset.Texture("ui/card_type_fight.tex") )
        :SetPos( 0, 40 )
        :PrepareAnimation()

    self.divider_top = self:AddChild( Widget.Image( engine.asset.Texture("large/dialog_divider_top.tex" ) ) )
        :SetTintColour( card_class == Widget.NegotiationCard and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT )
        :SetSize( RES_X*1, 4 )
        :Bloom( 0.1 )
    self.divider_bottom = self:AddChild( Widget.Image( engine.asset.Texture("large/dialog_divider_bottom.tex" ) ) )
        :SetTintColour( card_class == Widget.NegotiationCard and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT )
        :SetSize( RES_X*1, 4 )
        :Bloom( 0.1 )

    self.scroll_root = Widget.ScrollPanel()
        :SetVirtualMargin( 30 )
    self:AddChild( self.scroll_root )
    self.scroll_root:ShowBar( SCROLLBAR.IF_NEEDED )


    self.card_widgets = {}
    for i, card in ipairs( cards ) do
        -- Create card and set its scale
        local widget = card_class( card )
        self.card_widget_width, self.card_widget_height = widget:GetSize()

        if pred and not pred( card ) then
            widget:Disable()
            widget:SetSaturation( 0 ) 
        else
            widget:Enable()
        end

        -- Handle events
        widget.OnGainFocus = function() widget._base.OnGainFocus(widget) self.scroll_root:EnsureVisible(widget) self:UpdateCardImageState( widget ) end
        widget.OnLoseFocus = function() widget._base.OnLoseFocus(widget) self:UpdateCardImageState( widget ) end
        widget.OnGainHover = function() widget._base.OnGainHover(widget) self:UpdateCardImageState( widget ) end
        widget.OnLoseHover = function() widget._base.OnLoseHover(widget) self:UpdateCardImageState( widget ) end

        -- Handle clicks
        widget:SetOnClick( function() self:OnCardSelect( widget ) end )

        widget:SetOutline( widget.is_negotiation and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT, true )

        -- Add card to the scrolling list
        self.scroll_root:AddScrollChild( widget )

        -- Add the widget to the list of widgets
        table.insert( self.card_widgets, widget )
    end
    table.clear( self.cards )

    -- Add button
    -- Continue button
    self.hide_btn = self:AddChild( Widget.IconButton() )
        :SetText( LOC"CHOOSE_CARDS_POPUP.MINIMIZE_TITLE", LOC"CHOOSE_CARDS_POPUP.MINIMIZE_DESC" )
        :SetIcon( engine.asset.Texture("UI/ic_preview_open.tex" ) )
        :SetupFocusGlow()
        :SetFn( function() self:TogglePopup() end )

    if min_count == 0 then
        self.cancel_btn = self:AddChild( Widget.IconButton() )
            :SetText( LOC"CHOOSE_CARDS_POPUP.CANCEL_TITLE" )
            :SetIcon( engine.asset.Texture("UI/ic_nav_close.tex" ) )
            :SetupFocusGlow()
            :LayoutBounds( "center", nil, self.bg_panel )
            :LayoutBounds( nil, "bottom", self.bg_panel )
            :Offset( 0, 20 )
            :SetFn( function() self:Close() end )
    end
end

function ChooseCardsPopup:SetTitle( title, source )
    local subtitle
    if is_instance( source, CardEngine.Card ) then
        subtitle = loc.format( LOC "CARD_ENGINE.SOURCE_CARD", source.id )
        self.bg_panel.subtitle:SetHoverLinkFn( UIHelpers.ItemTooltips )
    elseif is_instance( source, CombatCondition ) or is_instance( source, Negotiation.Modifier ) then
        if source.graft then
            subtitle = loc.format( LOC "CARD_ENGINE.SOURCE_GRAFT", source.graft.id )
        else
            subtitle = loc.format( LOC "CARD_ENGINE.SOURCE_OTHER", source:GetName() )
        end
    elseif is_instance( source, GraftInstance ) then
        subtitle = loc.format( LOC "CARD_ENGINE.SOURCE_GRAFT", source.id )
    end

    self.bg_panel:SetTitles( title, subtitle )
end

function ChooseCardsPopup:TogglePopup()
    if self.hide_popup then
        AUDIO:PlayEvent( "event:/sfx/battle/cards/gen/cardselect_popup/hide_hand" )
        self:RestorePopup()
    else
        AUDIO:PlayEvent( "event:/sfx/battle/cards/gen/cardselect_popup/show_hand" )
        self:HidePopup()
    end
end

function ChooseCardsPopup:HidePopup()
    if self.on_inspect_hand then self.on_inspect_hand( self ) end

    self.hide_popup = true

    self.last_focus = self.fe:GetFocusWidget()

    self.is_overlay = true
    self.fe:RefreshTopScreen()
    self.fe:GetTopScreen():SetDefaultFocus()

    self.scroll_root:Hide() --AlphaTo( 0, 0.2 )
    self.divider_top:Hide()
    self.divider_bottom:Hide()
    self.bg_panel:Hide()
    
    if self.cancel_btn then
        self.cancel_btn:Hide()
    end

    self.container:Hide()
    local screenw, screenh = self:GetFE():GetScreenDims()
    self.hide_btn:SetText( LOC"CHOOSE_CARDS_POPUP.RESTORE_TITLE", LOC"CHOOSE_CARDS_POPUP.RESTORE_DESC" )
        :ShowFocusGlow()
        :SetIcon( engine.asset.Texture("UI/ic_preview_close.tex" ) )
end

function ChooseCardsPopup:RestorePopup()
    local screen = self:GetFE():FindScreen( Screen.DeckScreen )
    if screen then
        self:GetFE():PopScreen( screen )
    end

    if self.on_inspect_options then self.on_inspect_options( self ) end

    self.hide_popup = false

    self.is_overlay = nil
    self:GetFE():RefreshTopScreen()

    self:SetDefaultFocus()
    self.last_focus = nil

    self:MoveTo( 0, 0, 0.3, easing.outQuad )
    self.divider_top:Show()
    self.divider_bottom:Show()
    self.bg_panel:Show()
    if self.cancel_btn then
        self.cancel_btn:Show()
    end

    self.container:Show()
    self.scroll_root:Show():AlphaTo( 1.0, 0.2 )
    self.hide_btn:SetText( LOC"CHOOSE_CARDS_POPUP.MINIMIZE_TITLE", LOC"CHOOSE_CARDS_POPUP.MINIMIZE_DESC" )
        :HideFocusGlow( true )
        :SetIcon( engine.asset.Texture("UI/ic_preview_open.tex" ) )
end

function ChooseCardsPopup:OnUpdate(dt)
    ChooseCardsPopup._base.OnUpdate(self, dt)

    self.select_timer = self.select_timer - dt

    if TheGame:GetLocalSettings().ROBOTICS and self.select_timer < -1.0 then
        local cards = {}
        for i, w in ipairs( self.card_widgets ) do
            table.insert( cards, w.card )
        end
        table.arrayadd( self.cards, table.multipick( cards, self.min_count ))
        self:Close()
    end
end

function ChooseCardsPopup:OnOpen()
    ChooseCardsPopup._base.OnOpen( self )
    self.bg_panel:FadeIn( function() 
    end )
    self.select_timer = 0.1

end

function ChooseCardsPopup:OnControlModeChange( cm, device_id )
    ChooseCardsPopup._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.hide_btn:Show()
    elseif cm == CONTROL_MODE.TOUCH then
        self.hide_btn:Show()
    elseif cm == CONTROL_MODE.GAMEPAD then
        self.hide_btn:Hide()
    end
end

function ChooseCardsPopup:OnScreenModeChange( sm )
    local card_scale = CARD_SCALE[ sm ]
    for i, w in ipairs( self.card_widgets ) do
        self:UpdateCardImageState( w )
    end

    self.bg_panel:SetHeight( PANEL_H[ sm ], true )

    self.hide_btn:LayoutBounds( "left", nil, self.bg_panel )
        :LayoutBounds( nil, "bottom", self.bg_panel )
        :Offset( 50, 40 )
    
    self:LayoutCards( PANEL_H[ sm ] )

    ChooseCardsPopup._base.OnScreenModeChange( self, sm )
end


function ChooseCardsPopup:GetDefaultFocus()
    if not self.hide_popup then
        return self.last_focus or (self.card_widgets and self.card_widgets[1])
    end
end

function ChooseCardsPopup:Close()
    AUDIO:PlayEvent( "event:/ui/select_cards/close" )
    -- self.bg_panel:FadeOut( function()
        for k, card in ipairs(self.cards) do
            local widget = self:GetWidgetForCard(card)
            if widget then
                -- Restore default events
                widget.OnGainFocus = function() widget._base.OnGainFocus(widget) end
                widget.OnLoseFocus = function() widget._base.OnLoseFocus(widget) end
                widget.OnGainHover = function() widget._base.OnGainHover(widget) end
                widget.OnLoseHover = function() widget._base.OnLoseHover(widget) end
                widget:SetOnClick(nil)
            end
        end
        if self.on_close then
            self.on_close( self, self.cards )
        end
        self:GetFE():PopScreen( self )
    -- end )
end

function ChooseCardsPopup:GetWidgetForCard(card)
    for k,v in pairs(self.card_widgets) do
        if v.card == card then
            return v
        end
    end
end

function ChooseCardsPopup:UpdateCardImageState( widget )
    local sm = self:GetFE():GetScreenMode()
    if widget.enabled and table.contains( self.cards, widget.card ) then
        widget:SetOutline( UICOLOURS.WHITE, true )
        widget:SetScale( CARD_SCALE_HOVER[ sm ] )

    elseif widget.enabled and ( widget.hover or widget.focus ) then
        widget:SetOutline( UICOLOURS.FOCUS, true )
        widget:SetScale( CARD_SCALE_HOVER[ sm ] )
    else
        widget:SetOutline( widget.is_negotiation and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT, true )
        widget:SetScale( CARD_SCALE[ sm ] )
    end
end

function ChooseCardsPopup:OnCardSelect( widget )
    if self.select_timer > 0 then return end

    local selected = table.contains( self.cards, widget.card )
    if selected then
        table.arrayremove( self.cards, widget.card )
        self:UpdateCardImageState( widget )
    else
        while #self.cards >= self.max_count do
            local card = table.remove( self.cards, 1 )
            local widget = self:GetWidgetForCard( card )
            if widget then self:UpdateCardImageState( widget ) end
        end
        table.insert( self.cards, widget.card )
        self:UpdateCardImageState( widget )
    end

    -- Auto-close on final pick. (Option?)
    if #self.cards >= self.max_count then
        self:Close()
    end
end

function ChooseCardsPopup:LayoutCards( panel_h )

    local SPACING = 20
    local card_y_spacing = 30
    local scale = CARD_SCALE[ self:GetFE():GetScreenMode() ]

    local pos = 1
    local first_visible_card
    for i, widget in ipairs( self.card_widgets ) do
        if widget:IsShown() then
            if not first_visible_card then first_visible_card = widget end

            local slotw, sloth = self.card_widget_width * scale, self.card_widget_height * scale
            local xslots = math.min( #self.card_widgets, math.floor( CARDS_W / ( slotw + SPACING ) ) )
            local xmargin = ( CARDS_W - ( xslots * ( slotw + SPACING ) ) ) / 2
            local x = ( pos - 1 ) % xslots
            local y = math.floor( ( pos - 1 ) / xslots )

            widget:SetPos( xmargin + x * ( slotw + SPACING ) + slotw / 2, -y * ( sloth + card_y_spacing ) - sloth / 2 )

            pos = pos + 1
        end
    end
    self.scroll_root:SetSize( CARDS_W - SPACING, panel_h - 180 )
    self.scroll_root:RefreshView()
    self.scroll_root
        :LayoutBounds( "center", "bottom", self.bg_panel )
        :Offset( 0, 50 )
    self.divider_top:LayoutBounds( "center", "top", self.scroll_root )
        :Offset( 0, 2 )
    self.divider_bottom:LayoutBounds( "center", "bottom", self.scroll_root )
        :Offset( 0, 20 )
end

function ChooseCardsPopup:GetDebugEnv( debug_env )
    debug_env.cards = self.cards
end
