require "ui/widgets/installed_grafts"

local CoinScreen = class( "Screen.CoinScreen", Widget.Screen )

local GRAFT_SLOT_SIZE = 150
local GRAFT_SLOT_CONTENT_W = 540
local DETAILS_W = 540

CoinScreen.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function(self)
            return self.character_animation_done and TheGame:FE():IsGamepadMode()
        end,
        fn = function(self)
            self.character_animation_done = false
            self.character:Emote( "coin_toss", true )
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.COIN_SCREEN.CONTROLS_FLIP_COIN", Controls.Digital.MENU_ACCEPT))
        end,
    },
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self)
            self:OnClickClose()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end,
    },
}

function CoinScreen:init( owner, target_graft_type )
    CoinScreen._base.init( self )

    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.MONEY )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )


    -- Setup character anim display
    self.owner = owner or TheGame:GetGameState():GetPlayerAgent()
    self.character = self:AddChild( Widget.AnimatedCharacter() )
        :Refresh( self.owner )
    -- self.character:SetXRayOutline( 0x0a4434FF, UICOLOURS.MONEY, 0.0, 0.2 )
    self.character:SetHoverOutlineColour( UICOLOURS.MONEY )
    self.character_animation_done = true
    self.character.anim:SetAnimDoneFn( function() self.character_animation_done = true end )
    self.character:SetClickFn( function()
        if self.character_animation_done then
            self.character_animation_done = false
            self.character:Emote( "coin_toss", true )
        end
    end )
    self.character:SetAutoLoopEnabled( false )
    self.character:SetPos( RES_X*0.25, -350 )
        :SetScale( -1, 1 )
    self.character:Emote( "coin_toss", true )

    -- Coin display!
    self.content = self:AddChild(Widget())
    self.title = self.content:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_TITLE ) )
        :SetText( loc.upper( loc.format( LOC"UI.COIN_SCREEN.TITLE", self.owner:GetName() ) ) )
        :SetGlyphColour( UICOLOURS.MONEY )
        :SetAutoSize(DETAILS_W)
        :LeftAlign()
        :SetWordWrap(true)
        :Bloom(0.15)
    self.subtitle = self.content:AddChild( Widget.Label("title", FONT_SIZE.BODY_TEXT ) )
        :SetText( loc.format( LOC"UI.COIN_SCREEN.SUBTITLE", self.owner:GetName() ) )
        :SetGlyphColour( UICOLOURS.MONEY )
        :SetAutoSize(DETAILS_W)
        :LeftAlign()
        :SetWordWrap(true)
        :SetTintAlpha( 0.8 )
        :Bloom(0.1)
    self.coin_widget = self.content:AddChild( Widget.CoinScreenSlot( self.graft_type, GRAFT_SLOT_SIZE ) )

    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.close_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", function() self:OnClickClose() end ) )
    self.close_button:SetIcon( global_images.close )
    self.close_button:LayoutBounds( "left", "above", 60, 60 )
    
    self:AddRobotOption(self.close_button)
    self:Refresh()
end

function CoinScreen:Refresh()

    self.owner = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent()
    self.graft_type = GRAFT_TYPE.COIN
    local grafts = self.owner:GetAspect("graft_owner"):GetGrafts( self.graft_type )

    -- Add new
    self.coin_widget:SetGraft( grafts[1] )
        :Refresh()

    -- Layout
    self.coin_widget:LayoutBounds( "left", "below", self.title )
        :Offset( 0, -SPACING.M1 )
    self.subtitle:LayoutBounds( "left", "below", self.coin_widget )
        :Offset( 0, -SPACING.M1 )
    self.content:LayoutBounds( "right", "center", -RES_X*0, 0 )

    return self
end

function CoinScreen:OnGainTop()
    CoinScreen._base.OnGainTop( self )

    local anim_duration = 0.7
    local horizontal_movement = 90

    -- Prepare for animation by getting current values
    local content_x, content_y = self.content:GetPos()

    -- Hide objects until they're animated in
    self.content:SetTintAlpha(0)

    -- Setup and start animation
    self.fadein_anim = self:RunUpdater(
        Updater.Parallel{
            -- Fade and move in the coin container
            Updater.Ease( function(v) self.content:SetTintAlpha(v) end, 0, 1, anim_duration, easing.outQuad),
            Updater.Ease2D( function(x,y) self.content:SetPos(x,y) end, content_x - horizontal_movement, content_y, content_x, content_y, anim_duration, easing.outQuad),
        }
    )

    self:RefreshStreamerMode()
end

function CoinScreen:OnControlModeChange( cm, device_id )
    CoinScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.bottom_left:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.bottom_left:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.bottom_left:Hide()
    end
end

function CoinScreen:OnScreenModeChange( sm )
    CoinScreen._base.OnScreenModeChange( self, sm )

    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.content:SetLayoutScale( LAYOUT_SCALE[sm] )

    self.content:LayoutBounds( "right", "center", -RES_X*0, 0 )
end

function CoinScreen:GetDefaultFocus()
    return self.content or self.close_button
end

function CoinScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
end

function CoinScreen:OnOpen()
    CoinScreen._base.OnOpen( self )
    self.screen_loop = AUDIO:CreateEventInstance("event:/ui/grafts_menu/open")
    self.screen_loop:Start()
end

function CoinScreen:OnClickClose()
    AUDIO:PlayEvent( "event:/ui/grafts_menu/close" )
    self:FadeOut()
end

function CoinScreen:OnClose()
    CoinScreen._base.OnClose( self )

    self.screen_loop:Stop()
    self.screen_loop:Release()
    self.screen_loop = nil
end

------------------------------------------------------------------------------------------
-- Shows graft slot

local CoinScreenSlot = class( "Widget.CoinScreenSlot", Widget.GraftSlot )

function CoinScreenSlot:init( type, size )
    self._base.init( self, type, size )

    self.slot_size = size or 64
    self.graft_type = type

    self.features = {}

    if not self.bg then
        self.bg = self:AddChild( Widget.Image() )
    end
    self.bg:SetTexture( engine.asset.Texture ("UI/grafts_slot.tex" ) )
    self.bg:SetSize(self.slot_size,self.slot_size)
    self.bg:Bloom(0.2)
    self.bg:SetTintColour( UICOLOURS.MONEY )

    self.img = self:AddChild( Widget.Image():SetSize( self.slot_size*0.9, self.slot_size*0.9 ) )

    self.text_content = self:AddChild( Widget() )
    self.name = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_NAME ) )
        :SetWordWrap( true )
        :SetAutoSize( GRAFT_SLOT_CONTENT_W )
        :LeftAlign()
        :OverrideLineHeight( FONT_SIZE.ITEM_NAME*LINE_HEIGHT_TITLE_MULT )
        :SetGlyphColour( UICOLOURS.MONEY )
        :Bloom(0.1)
    self.subtitle = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE ) )
        :SetAutoSize( GRAFT_SLOT_CONTENT_W )
        :LeftAlign()
        :Bloom(0.05)
    self.desc = self.text_content:AddChild( Widget.Label( "body", FONT_SIZE.BODY_TEXT ) )
        :SetWordWrap( true )
        :SetAutoSize( GRAFT_SLOT_CONTENT_W )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TITLE )
        :SetTintAlpha( 0.7 )
        :Bloom(0.1)

    self:SetFocusable()
end

function CoinScreenSlot:SetGraft( graft )
    if self.graft ~= graft then
        self.graft = graft
        self:Refresh()
    end

    self:SetToolTipClass( Widget.TooltipGraft )
    self:SetToolTip(graft)
    self:ShowToolTipOnFocus()
    return self
end

function CoinScreenSlot:Refresh()
    if self.graft then
        -- Set values
        self.img:SetTexture(self.graft:GetImage())
        self.name:SetText( loc.upper( self.graft:GetName() ) or LOC"UI.GRAFT_COLLECTION.SLOT_EMPTY" )
        self.desc:SetText( self.graft:GetDesc( self ) )

        local rarity_color = CARD_RARITY_COLOURS[ self.graft:GetRarity() or CARD_RARITY.UNIQUE ]
        local rarity_icon_string = GetCardRarityIconString( self.graft:GetRarity() )
        local subtitle_text = TheGame:Str( string.format( "UI.GRAFT_COLLECTION.%s.%s", self.graft:GetRarity(), self.graft:GetType() ))

        self.subtitle:SetText( rarity_icon_string .. subtitle_text )
        self.subtitle:SetTintAlpha(1)

        -- Layout
        self.subtitle:LayoutBounds( "left", "below", self.name ):Offset( 0, 2 )
        self.desc:LayoutBounds( "left", "below", self.subtitle ):Offset( 0, -SPACING.M1 )
    else
        self.img:SetTexture( nil )
        self.name:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_EMPTY" )
        self:SetToolTip(LOC"UI.GRAFTSLOT.EMPTY_COIN")
        self.desc:SetText( "" )
        local subtitle_text = loc.format( LOC"UI.GRAFT_COLLECTION.SLOT_EMPTY_SUBTITLE" )
        self.subtitle:SetText( subtitle_text )
        self.subtitle:SetTintAlpha(0.2)

        -- Layout
        self.name:LayoutBounds( "after", "center", self.img ):Offset( 15, 10 )
        self.subtitle:LayoutBounds( "left", "below", self.name ):Offset( 0, 2 )
        self.desc:LayoutBounds( "left", "top", self.name ):Offset( 0, 0 )
    end
    self.text_content:LayoutBounds( "after", "center", self.img):Offset( 40, 1 )
end

function CoinScreenSlot:SetFn( fn )
    self:SetOnClickFn( function() fn( self.graft, self ) end )
end

function CoinScreenSlot:UpdateImage()
    if self.enabled and ( self.hover or self.focus ) then
        self.bg:ScaleTo( nil, 1.02, 0.2, easing.outQuad )
            :Bloom(0.15)
        if self.connector then self.connector:Bloom(0.5) end
        if self.name then self.name:Bloom(0.2) end
        self.img:ScaleTo( nil, 1.1, 0.3, easing.outQuad )
    else
        self.bg:ScaleTo( nil, 1, 0.2, easing.outQuad )
            :Bloom(0.1)
        if self.connector then self.connector:Bloom(0.3) end
        if self.name then self.name:Bloom(0.1) end
        self.img:ScaleTo( nil, 1, 0.2, easing.outQuad )
    end
end

function CoinScreenSlot:OnGainFocus()
    CoinScreenSlot._base.OnGainFocus(self)
    self:UpdateImage()
end

function CoinScreenSlot:OnLoseFocus()
    CoinScreenSlot._base.OnLoseFocus(self)
    self:UpdateImage()
end

function CoinScreenSlot:OnGainHover()
    CoinScreenSlot._base.OnGainHover(self)
    self:UpdateImage()
end

function CoinScreenSlot:OnLoseHover()
    CoinScreenSlot._base.OnLoseHover(self)
    self:UpdateImage()
end

function CoinScreenSlot:PostFeature( keyword, name, desc )
    self.features[ keyword ] = { name = name, desc = desc }
end

function CoinScreenSlot:PostCard( feature_id, card )
    self.features[ feature_id ] = { card = card }
end