require "ui/widgets/screen"
require "ui/widgets/editlabel"
require "ui/widgets/agentportrait"
require "ui/widgets/cardgrid"
require "ui/widgets/visibilitypanel"
require "ui/widgets/tabnavigation"
require "ui/widgets/compendiumwidget"
require "ui/widgets/cardcompendium"
require "ui/widgets/graftscompendium"
require "ui/widgets/peoplecompendium"
require "ui/widgets/bosscompendium"
require "ui/widgets/griftscompendium"
require "ui/widgets/mettlecompendium"


local assets = {
    deckfilter_bg = engine.asset.Texture("UI/deckfilter_bg.tex"),
    deckfilter_bg_disabled = engine.asset.Texture("UI/deckfilter_bg_disabled.tex"),
    gradient = engine.asset.Texture("UI/deckfilter_gradient.tex"),
}

--------------------------------------------------------------------------------
-- Displays a searchable list of all the cards in the game

local Compendium = class("Screen.Compendium", Widget.Screen)

local CONTENT_PANEL_H = 780
local CONTENT_W = 1800
local CONTENT_H = CONTENT_PANEL_H
local CONTENT_LEFT_MARGIN = 15 -- Extra size added on the left of scroll boxes, so that the contents can scale up on hover without clipping
local NAV_H = 66
local SIDEBAR_W = 380
local SIDEBAR_LEFT_MARGIN = 10 -- Extra size added on the left of scroll boxes, so that the contents can scale up on hover without clipping

Compendium.CONTROL_MAP =
{
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function()
            TheGame:FE():FadeTransition( function() TheGame:FE():PopScreen() end )
            return true
        end,
        test = function(self)
            return self:GetCurrentPanel():CanNav()
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end
    },
    {
        control = Controls.Digital.MENU_PREV,
        test = function(self)
            return self:GetCurrentPanel():CanNav()
        end,
        fn = function(self)
            self.navigation:SetCurrent( math.max(1, self.navigation:GetCurrentIdx() - 1 ))
            return true
        end,
    },
    {
        control = Controls.Digital.MENU_NEXT,
        test = function(self)
            return self:GetCurrentPanel():CanNav()
        end,
        fn = function(self)
            self.navigation:SetCurrent( math.min(#self.navigation:GetTabs(), self.navigation:GetCurrentIdx() + 1  ))
            return true
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_LT,
        fn = function(self)
            self:GetCurrentPanel():ToggleSideBar()
            return true
        end,
        test = function(self)
            return self:GetCurrentPanel():CanNav() and self:GetCurrentPanel():HasSidebar()
        end,
        hint = function(self, left, right)
            table.insert(left, loc.format(self:GetCurrentPanel().sidebar_focus and LOC"UI.CONTROLS.SELECT_CONTENT" or LOC"UI.CONTROLS.SELECT_SIDEBAR" , Controls.Digital.GAMEPAD_LT))
        end
    },

}


function Compendium:init( target_tab_index )
    Compendium._base.init(self)

    engine.inst:ProfilerPush("Compendium:init")

	self.is_popup = true
    self:SetupUnderlay( true )
	self:SetAnchors("center", "center")

	-- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    CONTENT_W = self.screen_w - 60
    
    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Tab navigation
    self.navigation = self:AddChild( Widget.TabNavigation( 200, NAV_H, function( tab ) self:OnNavChange( tab ) end ) )
    engine.inst:ProfilerPush("Compendium:init cards")
    self.cards_tab = self.navigation:AddTab( Widget.ProgressIconTab(engine.asset.Texture( "UI/ic_cardcompendium.tex" ),LOC"UI.CARDCOMPENDIUM.NAV_CARDS",LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_SUBTITLE","0%",0,0,UICOLOURS.SUBTITLE) )
    engine.inst:ProfilerPop()
    engine.inst:ProfilerPush("Compendium:init people")
    self.people_tab = self.navigation:AddTab( Widget.ProgressIconTab(engine.asset.Texture( "UI/ic_peoplecompendium.tex" ),LOC"UI.CARDCOMPENDIUM.NAV_PEOPLE",LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_SUBTITLE","0%",0,0,UICOLOURS.ACCENT) )
    engine.inst:ProfilerPop()
    engine.inst:ProfilerPush("Compendium:init boss")
    self.boss_tab = self.navigation:AddTab( Widget.ProgressIconTab(engine.asset.Texture( "UI/ic_bosscompendium.tex" ),LOC"UI.CARDCOMPENDIUM.NAV_BOSS",LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_SUBTITLE","0%",0,0,UICOLOURS.DEAD) )
    engine.inst:ProfilerPop()
    engine.inst:ProfilerPush("Compendium:init grafts")
    self.grafts_tab = self.navigation:AddTab( Widget.ProgressIconTab(engine.asset.Texture( "UI/ic_graftscompendium.tex" ),LOC"UI.CARDCOMPENDIUM.NAV_GRAFTS",LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_SUBTITLE","0%",0,0,UICOLOURS.OBJECTIVE) )
    engine.inst:ProfilerPop()
    engine.inst:ProfilerPush("Compendium:init grifts")
    self.grifts_tab = self.navigation:AddTab( Widget.ProgressIconTab(engine.asset.Texture( "UI/ic_griftscompendium.tex" ),LOC"UI.CARDCOMPENDIUM.NAV_GRIFTS",LOC"UI.CARDCOMPENDIUM.GRIFT_FILTER_SUBTITLE","0%",0,0,UICOLOURS.GRIFTS) )
    engine.inst:ProfilerPop()
    engine.inst:ProfilerPush("Compendium:init mettle")
    self.mettle_tab = self.navigation:AddTab( Widget.ProgressIconTab(engine.asset.Texture( "UI/ic_mettlecompendium.tex" ),LOC"UI.CARDCOMPENDIUM.NAV_METTLE",LOC"UI.CARDCOMPENDIUM.METTLE_FILTER_SUBTITLE","0%",0,0,UICOLOURS.METTLE) )
    engine.inst:ProfilerPop()

    -- Set Navigation width
    self.navigation:SetTotalWidth( CONTENT_W )

    -- Navigation tab labels
    self.navigation_label_left = self:AddChild( Widget.Label( "title", 30, LOC"UI.OPTIONS_SCREEN.CONTROLS_LABEL_TAB_LEFT" ) )
        :Bloom( 0.1 )
    self.navigation_label_right = self:AddChild( Widget.Label( "title", 30, LOC"UI.OPTIONS_SCREEN.CONTROLS_LABEL_TAB_RIGHT" ) )
        :Bloom( 0.1 )

    -- Bottom subtext.
    self.bottom_subtext = self:AddChild( Widget.Label( "title", 24 )):Bloom( 0.1 ):SetAnchors( "center", "bottom" ):Offset( 0, 130 )

	-- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.back_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", function() TheGame:FE():FadeTransition( function() TheGame:FE():PopScreen() end ) end ) )
    self.back_button:SetIcon( global_images.close )
    self.back_button:LayoutBounds( "left", "above", 60, 60 )

    self:RegisterHotkeyButton( Controls.Digital.MENU_CANCEL, self.back_button )

    -- Content panel
    self.content_panel = self:AddChild( Widget.WidePanel( CONTENT_PANEL_H ) )

    -- Content widgets
    self.cards_widget = self:AddChild( Widget.CardCompendium( CONTENT_W, CONTENT_H, SIDEBAR_W ) )
    self.people_widget = self:AddChild( Widget.PeopleCompendiumWidget( CONTENT_W, CONTENT_H, SIDEBAR_W ) )
    self.boss_widget = self:AddChild( Widget.BossCompendiumWidget( CONTENT_W, CONTENT_H, SIDEBAR_W ) )
    self.grafts_widget = self:AddChild( Widget.GraftsCompendiumWidget( CONTENT_W, CONTENT_H, SIDEBAR_W ) )
    self.grifts_widget = self:AddChild( Widget.GriftsCompendiumWidget( CONTENT_W, CONTENT_H, SIDEBAR_W ) )
    self.mettle_widget = self:AddChild( Widget.MettleCompendiumWidget( CONTENT_W, CONTENT_H, SIDEBAR_W ) )

    self:RefreshContent()

    -- Position the navigation tabs
    self.navigation:LayoutBounds( nil, "above", self.content_panel.panel )
        :LayoutBounds( "left", nil, -CONTENT_W/2, 0 )
        :Offset( 0, -2 )

    self:Layout()

    self.navigation:SetCurrent( target_tab_index or 1 )

    -- Play screen sound
    self.screen_loop = AUDIO:CreateEventInstance( "event:/ui/card_codex/open" )
    self.screen_loop:Start()
    engine.inst:ProfilerPop()
end


function Compendium:OnClose()
    Compendium._base.OnClose( self )

    -- Stop the screen sound
    self.screen_loop:Stop()
    self.screen_loop:Release()
    self.screen_loop = nil

    AUDIO:PlayEvent( "event:/ui/card_codex/close" )
end

function Compendium:OnScreenResize(w, h)
    Compendium._base.OnScreenResize(self, w, h)
    self:Layout()
    self:RefreshStreamerMode()
end

function Compendium:OnRemoved()
    collectgarbage()
end

function Compendium:OnGainTop()
    Compendium._base.OnGainTop(self)
    self._last_focus_widget = nil
end


function Compendium:OnControlModeChange( cm, device_id )
    Compendium._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.bottom_left:Show()
        self.navigation_label_left:Hide()
        self.navigation_label_right:Hide()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.bottom_left:Show()
        self.navigation_label_left:Hide()
        self.navigation_label_right:Hide()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.bottom_left:Hide()
        self.navigation_label_left:Show()
        self.navigation_label_right:Show()
    end

    self:Layout()
end

function Compendium:GetMusicEvent()
    return "event:/music/town"
end

function Compendium:GetDefaultFocus()
    if self._last_focus_widget and self:GetCurrentPanel():IsAncestorOf(self._last_focus_widget) then
        return self._last_focus_widget
    end
    return self:GetCurrentPanel():GetDefaultFocus( )
end

function Compendium:GetCurrentPanel()
    if self.current_tab == self.cards_tab then return self.cards_widget end
    if self.current_tab == self.people_tab then return self.people_widget end
    if self.current_tab == self.boss_tab then return self.boss_widget end
    if self.current_tab == self.grafts_tab then return self.grafts_widget end
    if self.current_tab == self.grifts_tab then return self.grifts_widget end
    if self.current_tab == self.mettle_tab then return self.mettle_widget end
end


function Compendium:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
end

function Compendium:OnNavChange( tab )

    self:SetBottomText( nil )

    self.current_tab = tab

    if self.current_tab == self.cards_tab then
        AUDIO:PlayEvent( "event:/ui/card_codex/select_card_tab" )
        self.cards_widget:Show()
        self.people_widget:Hide()
        self.boss_widget:Hide()
        self.grafts_widget:Hide()
        self.mettle_widget:Hide()
        self.grifts_widget:Hide()

    elseif self.current_tab == self.people_tab then
        AUDIO:PlayEvent( "event:/ui/card_codex/select_people_tab" )
        self.cards_widget:Hide()
        self.people_widget:Show()
        self.boss_widget:Hide()
        self.grafts_widget:Hide()
        self.mettle_widget:Hide()
        self.grifts_widget:Hide()
    elseif self.current_tab == self.boss_tab then
        AUDIO:PlayEvent( "event:/ui/card_codex/select_people_tab" )
        self.cards_widget:Hide()
        self.people_widget:Hide()
        self.boss_widget:Show()
        self.grafts_widget:Hide()
        self.mettle_widget:Hide()
        self.grifts_widget:Hide()
    elseif self.current_tab == self.grafts_tab then
        AUDIO:PlayEvent( "event:/ui/card_codex/select_graft_tab" )
        self.cards_widget:Hide()
        self.people_widget:Hide()
        self.boss_widget:Hide()
        self.grafts_widget:Show()
        self.mettle_widget:Hide()
        self.grifts_widget:Hide()
    elseif self.current_tab == self.grifts_tab then
        AUDIO:PlayEvent( "event:/ui/card_codex/select_grifts_tab" )
        self.cards_widget:Hide()
        self.people_widget:Hide()
        self.boss_widget:Hide()
        self.grafts_widget:Hide()
        self.mettle_widget:Hide()
        self.grifts_widget:Show()
    elseif self.current_tab == self.mettle_tab then
        AUDIO:PlayEvent( "event:/ui/card_codex/select_mettle_tab" )
        self.cards_widget:Hide()
        self.people_widget:Hide()
        self.boss_widget:Hide()
        self.grafts_widget:Hide()
        self.grifts_widget:Hide()
        self.mettle_widget:Show()
    end

    self:GetCurrentPanel():RefreshContent()
        
    -- The top icon obscures tabs.
    -- self.content_panel:SetIcon( self.current_tab:GetIcon(), self.current_tab:GetColour() ):SetColour( self.current_tab:GetColour() )
    self.content_panel:SetColour( self.current_tab:GetColour() )

    self.navigation:SetBGColour( self.current_tab:GetColour() )
    self:SetDefaultFocus()
end

function Compendium:Layout()
	-- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Size up the grid
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    self.navigation_label_left:LayoutBounds( "before", "center", self.navigation ):Offset( -15, 0 )
    self.navigation_label_right:LayoutBounds( "after", "center", self.navigation ):Offset( 15, 0 )
end

function Compendium.ShowAllUnlocks()
    return Compendium.show_all_unlocks
end

function Compendium:SetBottomText( txt )
    self.bottom_subtext:SetText( txt )
end

function Compendium:ToggleShowingAllUnlocks()
    Compendium.show_all_unlocks = not Compendium.show_all_unlocks
    self:RefreshContent( true )
end

function Compendium:RefreshContent( force_refresh )
    -- print("REFRESH CONTENT. ALL UNLOCKS ", Compendium.show_all_unlocks and "TRUE" or "FALSE" )

    local panel = self:GetCurrentPanel()
    if panel then
        engine.inst:ProfilerPush("Compendium:RefreshContent")
        panel:RefreshContent( force_refresh )
        engine.inst:ProfilerPop()
    end

    -- Refresh the nav's progress bars
    local cards_unlocked, cards_unlockable = self.cards_widget:GetCount()
    self.cards_tab:SetProgress( cards_unlocked, cards_unlockable, loc.format( LOC"UI.CARDCOMPENDIUM.NAV_PROGRESS", cards_unlocked/cards_unlockable ) )
        :SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.NAV_CARDS_TT", cards_unlocked, cards_unlockable, cards_unlocked/cards_unlockable ) )

    local people_unlocked, people_unlockable = self.people_widget:GetCount()
    self.people_tab:SetProgress( people_unlocked, people_unlockable, loc.format( LOC"UI.CARDCOMPENDIUM.NAV_PROGRESS", people_unlocked/people_unlockable ) )
        :SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.NAV_PEOPLE_TT", people_unlocked, people_unlockable, people_unlocked/people_unlockable ) )

    local boss_unlocked, boss_unlockable = self.boss_widget:GetCount()
    self.boss_tab:SetProgress( boss_unlocked, boss_unlockable, loc.format( LOC"UI.CARDCOMPENDIUM.NAV_PROGRESS", boss_unlocked/boss_unlockable ) )
        :SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.NAV_BOSS_TT", boss_unlocked, boss_unlockable, boss_unlocked/boss_unlockable ) )

    local grafts_unlocked, grafts_unlockable = self.grafts_widget:GetCount()
    self.grafts_tab:SetProgress( grafts_unlocked, grafts_unlockable, loc.format( LOC"UI.CARDCOMPENDIUM.NAV_PROGRESS", grafts_unlocked/grafts_unlockable ) )
        :SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.NAV_GRAFTS_TT", grafts_unlocked, grafts_unlockable, grafts_unlocked/grafts_unlockable ) )

    local grifts_completed, grifts_total = self.grifts_widget:GetCount()
    self.grifts_tab:SetProgress( grifts_completed, grifts_total, loc.format( LOC"UI.CARDCOMPENDIUM.NAV_PROGRESS", grifts_completed/grifts_total ) )
        :SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.NAV_GRIFTS_TT", grifts_completed, grifts_total, grifts_completed/grifts_total ) )

    local mettle_bought, mettle_total = self.mettle_widget:GetCount()
    self.mettle_tab:SetProgress( mettle_bought, mettle_total, loc.format( LOC"UI.CARDCOMPENDIUM.NAV_PROGRESS", mettle_bought/mettle_total ) )
        :SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.NAV_METTLE_TT", mettle_bought, mettle_total, mettle_bought/mettle_total ) )
end

------------------------------------------------------------------------------------------
-- Shows the filters sidebar

local CompendiumSidebar = class( "Widget.CompendiumSidebar", Widget )

function CompendiumSidebar:init( width, height, on_filters_changed_fn )
    CompendiumSidebar._base.init( self )

    self.widget_w = width
    self.filters_w = self.widget_w - 40
    self.widget_h = height
    self.on_filters_changed_fn = on_filters_changed_fn

    self.hitbox = self:AddChild( Widget.SolidBox( self.widget_w+SIDEBAR_LEFT_MARGIN, self.widget_h, 0xFF000030 ) )
    	:Hide()

    -- Add scrollable area for the widgets
    self.scroll_root = self:AddChild( Widget.ScrollPanel() )
    	:SetVirtualMargin( 15 )
    	:SetSize( self.widget_w+SIDEBAR_LEFT_MARGIN, self.widget_h )

    -- Add filters widget
    self.filters_widget = self.scroll_root:AddScrollChild( Widget.SidebarFilters( function( filter, is_toggled ) self:OnFilterChanged( filter, is_toggled ) end ) )
    	:SetWidth( self.filters_w )
        :SetPos( SIDEBAR_LEFT_MARGIN, nil )
end

function CompendiumSidebar:SetScrollEnabled( enabled )
    self.scroll_root:SetEnabled( enabled )
    return self
end

function CompendiumSidebar:Enable()
	CompendiumSidebar._base.Enable( self )
	self.ignore_input = false
	self.filters_widget:Enable()
    return self
end

function CompendiumSidebar:Disable()
	CompendiumSidebar._base.Disable( self )
	self.ignore_input = true
	self.filters_widget:Disable()
    return self
end

function CompendiumSidebar:AddFilterHeader( header )
    self.filters_widget:AddFilterHeader( header )
end 

function CompendiumSidebar:AddFilterWidget( filter )
    return self.filters_widget:AddFilter( filter )
end

function CompendiumSidebar:AddFilterRow( filters, count )
    return self.filters_widget:AddFilterRow( filters, count )
end

function CompendiumSidebar:Layout()
    self.filters_widget:Layout()
    self.scroll_root:RefreshView()
end

function CompendiumSidebar:OnFilterChanged( filter, is_toggled )
	if self.on_filters_changed_fn then self.on_filters_changed_fn( self.filters_widget:GetFilters() ) end
end

function CompendiumSidebar:GetFilters()
	return self.filters_widget:GetFilters()
end

function CompendiumSidebar:RefreshFilters()
	self.on_filters_changed_fn( self.filters_widget:GetFilters() )
end

function CompendiumSidebar:AddSortingWidget( options, fn, selected_idx )
	self.filters_widget:AddSortingRow( options, fn, selected_idx )
    self.filters_widget:Layout()
    self.scroll_root:RefreshView()
end

function CompendiumSidebar:GetDefaultFocus()
    return self.filters_widget:GetDefaultFocus()
end

------------------------------------------------------------------------------------------
-- Shows the current filters

local SidebarFilters = class( "Widget.SidebarFilters", Widget )

local MARGIN = 10

function SidebarFilters:init( on_filters_changed_fn )
    SidebarFilters._base.init( self )

    self.on_filters_changed_fn = on_filters_changed_fn
    self.widget_w = 200

    self.filters = {}
end

function SidebarFilters:SetWidth( w )
	self.widget_w = w or 200
	return self
end

function SidebarFilters:Layout()
	local previous
	for k, w in ipairs( self.children ) do
		if w:IsShown() then
			w:LayoutBounds( "left", "below", previous )
				:Offset( 0, -MARGIN )
			previous = w
		end
	end
end

function SidebarFilters:AddFilterHeader( title )
	local header = self:AddChild( Widget.FilterHeader( title ) )
		:LayoutBounds( "left", "below" )
		:Offset( 0, -MARGIN )
	table.insert( self.filters, header )
	return header
end

function SidebarFilters:AddFilter( filter )
	table.insert( self.filters, filter )
	self.default_focus = self.default_focus or filter
    return self:AddChild( filter )
		:SetWidth( self.widget_w )
		:SetOnToggleFn( function( filter, is_toggled ) self:FilterToggled( filter, is_toggled ) end )
		:LayoutBounds( "left", "below" )
		:Offset( 0, -MARGIN )
end

function SidebarFilters:AddFilterRow( filters, count_override )
	local row = self:AddChild( Widget() )
	local padding = 10
	local count = count_override or #filters
	local filter_w = (self.widget_w - (count-1)*padding)/count

	for k, filter in ipairs( filters ) do
		table.insert( self.filters, filter )
		row:AddChild( filter )
			:SetWidth( filter_w )
			:SetOnToggleFn( function( filter, is_toggled ) self:FilterToggled( filter, is_toggled ) end )
			:LayoutBounds( "after", "top" )
			:Offset( padding, 0 )
	end
	return row:LayoutBounds( "left", "below" )
		:Offset( 0, -MARGIN )
end

function SidebarFilters:GetFilters()
	return self.filters
end

-- List
function SidebarFilters:AddSortingRow( sorting_options, fn, selected_idx )
    self.sorting = self:AddChild( Widget.FilterRadioGroup() )
        :SetWidth( self.widget_w )
    for k, option in ipairs(sorting_options) do
        local row = self.sorting:AddFilter( Widget.SortingOptionFilterRow( option.txt ) )
            :SetOnToggleFn( function( filter, is_toggled )
                self.sorting:FilterToggled( filter, is_toggled )
                if is_toggled and fn then fn( k, option ) end
            end )

            if selected_idx == k and fn then
                fn( k, option )
            end
    end
    self.sorting:SetSelected( selected_idx or 1 )
        :LayoutBounds( "left", "below" )
        :Offset( 0, -MARGIN )
end

function SidebarFilters:Enable()
	SidebarFilters._base.Enable( self )
	self.ignore_input = false
	for k, filter in ipairs( self.filters ) do
		filter:Enable()
	end
    return self
end

function SidebarFilters:Disable()
	SidebarFilters._base.Disable( self )
	self.ignore_input = true
	for k, filter in ipairs( self.filters ) do
		filter:Disable()
	end
    return self
end

function SidebarFilters:FilterToggled( filter, is_toggled )
	if self.on_filters_changed_fn then self.on_filters_changed_fn( filter, is_toggled ) end
end

function SidebarFilters:GetDefaultFocus()
    if self.default_focus then
        if self.default_focus.GetDefaultFocus then
            return self.default_focus:GetDefaultFocus()
        else
            return self.default_focus
        end
    end
end

------------------------------------------------------------------------------------------
-- Shows a single dropdown

local GenericDropdownBlock = class( "Widget.GenericDropdownBlock", Widget.DropDown )

function GenericDropdownBlock:init( ... )
    GenericDropdownBlock._base.init( self, ... )
end

function GenericDropdownBlock:UpdateImage()
    if ( self.hover or self.focus ) then
        self:ScaleTo( nil, 1.03, 0.2, easing.outQuad )
    else
        local timing = 0.15
        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
    end
end

function GenericDropdownBlock:OnGainFocus()
    GenericDropdownBlock._base.OnGainFocus(self)
    self:UpdateImage()
end

function GenericDropdownBlock:OnLoseFocus()
    GenericDropdownBlock._base.OnLoseFocus(self)
    self:UpdateImage()
end

function GenericDropdownBlock:OnGainHover()
    GenericDropdownBlock._base.OnGainHover(self)
    self:UpdateImage()
end

function GenericDropdownBlock:OnLoseHover()
    GenericDropdownBlock._base.OnLoseHover(self)
    self:UpdateImage()
end



------------------------------------------------------------------------------------------
-- Shows a single filter

local GenericFilterBlock = class( "Widget.GenericFilterBlock", Widget.Clickable )

function GenericFilterBlock:init()
    GenericFilterBlock._base.init( self )
    self.widget_w = self.widget_w or 200
    self.widget_h = self.widget_h or 45

    self.hitbox = self:AddChild( Widget.SolidBox(self.widget_w,self.widget_h,0x00FF0000))

    self.scalable_contents = self:AddChild( Widget() ) -- Contains everything that will scale. Hidden bounding box
        :SetHiddenBoundingBox( true )
    
    -- Background panel
    self.bg_border = 30
    self.bg_border_scale = 0.5
    self.bg = self.scalable_contents:AddChild( Widget.Panel( assets.deckfilter_bg ) )
    	:SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :Bloom( 0.05 )
    self:SetHighlightWidget( self.bg )
    self.contents = self.scalable_contents:AddChild( Widget() )
    self.gradient = self.scalable_contents:AddChild( Widget.Panel( assets.gradient ) )
    	:SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :UnblockMouse()

    self:ShowToolTipOnFocus( true )
    self:SetOnClickFn( function() self:OnClick() end )
    self.toggled = true

end

function GenericFilterBlock:SetWidth( w )
    self.widget_w = w or 200
    self.hitbox:SetSize(self.widget_w,self.widget_h)
    local nineslice_w, nineslice_h = self.widget_w + self.bg_border*2*self.bg_border_scale, self.widget_h + self.bg_border*2*self.bg_border_scale
    self.bg:SetSize( nineslice_w, nineslice_h )
    self.gradient:SetSize( math.min( self.widget_w/2, 80 ), nineslice_h )

    self:Layout()
    return self
end

function GenericFilterBlock:Layout()
	self.gradient:LayoutBounds( "left", nil, self.bg )
	return self
end

function GenericFilterBlock:Filter( card )
	-- Return false if the card doesn't fit
	return true
end

function GenericFilterBlock:SetOnToggleFn( on_toggle_fn )
    self.on_toggle_fn = on_toggle_fn
	return self
end

function GenericFilterBlock:OnClick()
    if self.locked then
        AUDIO:PlayEvent( "event:/ui/card_codex/filter_off" )
    else
    	self:SetToggled( not self.toggled )
        AUDIO:PlayEvent( self.toggled and "event:/ui/card_codex/filter_on" or "event:/ui/card_codex/filter_off" )
    	if self.on_toggle_fn then self.on_toggle_fn( self, self.toggled ) end
    end
end

function GenericFilterBlock:IsToggled()
	return self.toggled
end

function GenericFilterBlock:SetToggled( toggled )
	self.toggled = toggled
	self.bg:Bloom( self.toggled and 0.05 or 0 )
	self.bg:SetTexture( self.toggled and assets.deckfilter_bg or assets.deckfilter_bg_disabled )
	self.contents:SetSaturation( self.toggled and 1 or 0.2 )
    return self
end

function GenericFilterBlock:IsLocked()
    return self.locked
end

function GenericFilterBlock:SetLocked( locked )
    self.locked = locked
    return self
end

function GenericFilterBlock:Enable()
	-- self._base.Enable( self )
	self.ignore_input = false

	self.bg:Bloom( self.toggled and 0.05 or 0 )
	self.bg:SetTexture( self.toggled and assets.deckfilter_bg or assets.deckfilter_bg_disabled )
	self.contents:SetSaturation( self.toggled and 1 or 0.2 )

    return self
end

function GenericFilterBlock:Disable()
	-- self._base.Disable( self )
	self.ignore_input = true

	self.bg:Bloom( 0 )
	self.bg:SetTexture( assets.deckfilter_bg_disabled )
	self.contents:SetSaturation( 0.2 )
	
    return self
end

function GenericFilterBlock:UpdateImage()
    if ( self.hover or self.focus ) and not self.locked then
        self.scalable_contents:ScaleTo( nil, 1.03, 0.2, easing.outQuad )
    else
        local timing = 0.15
        self.scalable_contents:ScaleTo( nil, 1, 0.2, easing.outQuad )
    end
end

function GenericFilterBlock:OnGainFocus()
    GenericFilterBlock._base.OnGainFocus(self)
    self:UpdateImage()
end

function GenericFilterBlock:OnLoseFocus()
    GenericFilterBlock._base.OnLoseFocus(self)
    self:UpdateImage()
end

function GenericFilterBlock:OnGainHover()
    GenericFilterBlock._base.OnGainHover(self)
    self:UpdateImage()
end

function GenericFilterBlock:OnLoseHover()
    GenericFilterBlock._base.OnLoseHover(self)
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- A filter header. Doesn't actually filter anything, but can be included in the same containers

local FilterHeader = class( "Widget.FilterHeader", Widget.Label )

function FilterHeader:init( title )
    FilterHeader._base.init( self, "title", 24, title )

    self:SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.05 )
end

function FilterHeader:SetWidth( w )
    return self
end

function FilterHeader:Enable()
    self:SetSaturation( 1 )
    return self
end

function FilterHeader:Disable()
    self:SetSaturation( 0.2 )
    return self
end

function FilterHeader:Filter( card )
    return true
end

------------------------------------------------------------------------------------------
-- A group of filters, of which only one can be checked

local FilterRadioGroup = class( "Widget.FilterRadioGroup", Widget )

function FilterRadioGroup:init()
    FilterRadioGroup._base.init( self )

    self.widget_w = 200
    self.filters = {}
end

-- Displays the filters in a row, dividing the width among them
function FilterRadioGroup:SetRowMode( column_count_override )
    self.row_mode = true
    self.column_count_override = column_count_override
    self:Layout()
    return self
end

function FilterRadioGroup:AddFilter( filter )
	table.insert( self.filters, filter )
	self:AddChild( filter )
		:SetOnToggleFn( function( filter, is_toggled ) self:FilterToggled( filter, is_toggled ) end )
    self:Layout()
    return filter
end

function FilterRadioGroup:SetWidth( w )
    self.widget_w = w or 200

    for k, filter in ipairs(self.children) do
    	filter:SetWidth( self.widget_w )
    end

    self:Layout()
    return self
end

function FilterRadioGroup:Layout()
    local padding = 10
    if self.row_mode then
        local col_count = self.column_count_override or #self.filters
        local filter_w = (self.widget_w - (col_count-1)*padding)/col_count
        for k, w in ipairs( self.children ) do
            w:SetWidth( filter_w )
        end
        self:LayoutChildrenInGrid( col_count, padding )
    else
        for k, w in ipairs( self.children ) do
            w:SetWidth( self.widget_w )
        end
        self:StackChildren( padding )
    end
	return self
end

function FilterRadioGroup:Filter( ... )
	-- Go through the filters and check if any of them fits
	for k, filter in ipairs( self.children ) do
		if filter:Filter( ... ) then return true end
	end
	return false
end

function FilterRadioGroup:SetOnToggleFn( on_toggle_fn )
    self.on_toggle_fn = on_toggle_fn
	return self
end

function FilterRadioGroup:SetSelected( pos )
	-- Untoggle all the filters
	for k, w in ipairs( self.children ) do
		if k == pos then
			self.selected_filter = w
			if not w.toggled then w:SetToggled( true ) end
		else
			w:SetToggled( false )
		end
	end
    return self
end

function FilterRadioGroup:GetSelected()
    return self.selected_filter
end

function FilterRadioGroup:FilterToggled( filter, is_toggled )
	-- Untoggle all the filters
	for k, w in ipairs( self.children ) do
		if w == filter then
			self.selected_filter = w
            if not w.toggled then w:SetToggled( true ) end
		else
			w:SetToggled( false )
		end
	end
	if self.on_toggle_fn then self.on_toggle_fn( filter, is_toggled ) end
end

function FilterRadioGroup:Enable()
	FilterRadioGroup._base.Enable( self )
	self.ignore_input = false
	for k, filter in ipairs( self.children ) do
		filter:Enable()
	end
    return self
end

function FilterRadioGroup:Disable()
	FilterRadioGroup._base.Disable( self )
	self.ignore_input = true
	for k, filter in ipairs( self.children ) do
		filter:Disable()
	end
    return self
end

function FilterRadioGroup:GetDefaultFocus()
    for k, w in ipairs( self.filters ) do
        if w:IsShown() then return w end
    end
end


------------------------------------------------------------------------------------------
-- A group of filters, of which only up to one can be checked at a time.
-- None is checked by default, which filters nothing out

local FilterOptionalRadioGroup = class( "Widget.FilterOptionalRadioGroup", Widget.FilterRadioGroup )

function FilterOptionalRadioGroup:init( filters )
    FilterOptionalRadioGroup._base.init( self, filters )
end

function FilterOptionalRadioGroup:Filter( card )
    if not self.selected_filter or not self.selected_filter:IsVisible() then
        return true
    else
        return self.selected_filter:Filter( card ) == true
    end
end

function FilterOptionalRadioGroup:FilterToggled( filter, is_toggled )
    -- Untoggle all the filters
    for k, w in ipairs( self.children ) do
        if w == filter then
            self.selected_filter = is_toggled and w
            w:SetToggled( is_toggled )
        else
            w:SetToggled( false )
        end
    end
    if self.on_toggle_fn then self.on_toggle_fn( filter, is_toggled ) end
end

------------------------------------------------------------------------------------------
-- A group of filters, any of which can be toggled

local DeckFilterCheckGroup = class( "Widget.DeckFilterCheckGroup", Widget.FilterRadioGroup )

function DeckFilterCheckGroup:init( filters )
    DeckFilterCheckGroup._base.init( self, filters )
end

function DeckFilterCheckGroup:Filter( card )
	-- Go through the filters and check if all the checked ones fit
	for k, filter in ipairs( self.children ) do
		if filter:Filter( card ) then return true end
	end
	return false
end

function DeckFilterCheckGroup:SetOnToggleFn( on_toggle_fn )
    self.on_toggle_fn = on_toggle_fn
	return self
end

function DeckFilterCheckGroup:SetSelected( pos )
	for k, w in ipairs( self.children ) do
		if k == pos then
            if not w.toggled then w:SetToggled( true ) end
		end
	end
    return self
end

function DeckFilterCheckGroup:FilterToggled( filter, is_toggled )
	if self.on_toggle_fn then self.on_toggle_fn( filter, is_toggled ) end
end

------------------------------------------------------------------------------------------
-- Shows a series/character filter row for cards

local CardSeriesFilterRow = class( "Widget.CardSeriesFilterRow", Widget.GenericFilterBlock )

local vertical_portrait_overshoot = 10

function CardSeriesFilterRow:init( character, series_table )
    self.widget_h = 60
    CardSeriesFilterRow._base.init( self )
    self.series_table = series_table

    local skins = Content.GetAllCharacterSkins( character ) or table.empty
    local skin = table.arraypick( skins )
    self.agent = Agent( character, skin )

    local title
    if table.contains( self.series_table, CARD_SERIES.NPC ) then
        self.title_name = LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_NPC"
        title = LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_NPC"
        self.tt = LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_NPC_TT" 

    elseif table.contains( self.series_table, CARD_SERIES.GENERAL ) then
        self.title_name = LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_GENERAL"
        title = LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_GENERAL"
        self.tt = LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_GENERAL_TT"

    else
        self.title_name = self.agent:GetName()
        title = loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_TITLE", loc.upper( self.title_name ))
    end

    -- Character anim
    self.anim_container = self.contents:AddChild( Widget() )
    	:SetScissor( 0, 0, 100, self.widget_h+vertical_portrait_overshoot )
    self.anim = self.anim_container:AddChild( Widget.AnimatedCharacter() )
    	:SetHiddenBoundingBox( true )
    	:SetScale( 0.18 )
    	:SetPos( 20, -68 )
    	:Refresh( self.agent )
        :SetSpeed( 0 )
        :SetOutline( 8, UICOLOURS.FOCUS )
        :SetAutoLoopEnabled( false )

    -- Text
    self.title = self.contents:AddChild( Widget.Label("title", 30 ))
    	:SetGlyphColour( UICOLOURS.SUBTITLE )
    	:SetText( title )
    	:LeftAlign()
    	:Bloom(0.1)
    self.subtitle = self.contents:AddChild( Widget.Label("body", 22 ))
    	:SetGlyphColour( UICOLOURS.SUBTITLE )
    	:SetTintAlpha( 0.8 )
    	:SetText( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_SUBTITLE" )
    	:LeftAlign()
    	:Bloom(0.05)
    	:Hide()
    self.value = self.contents:AddChild( Widget.Label("body", 22 ))
    	:SetGlyphColour( UICOLOURS.SUBTITLE )
    	:SetTintAlpha( 0.8 )
    	:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_PROGRESS", 0, 0 ) )
    	:RightAlign()
    	:Bloom(0.05)
    	:Hide()
    self.progress_bar = self.contents:AddChild( Widget.ProgressBar( self.widget_w, 3 ) )
    	:OrientH()
    	:SetOutlineColour( 0x00000000 )
    	:SetBGColour( 0x00000050 )
    	:SetBarColour( UICOLOURS.SUBTITLE )
    	:SetBarBloom( 0.05 )
    	:Hide()

    self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_TT", self.title:GetText(), self.tt ) )
end

function CardSeriesFilterRow:Filter( card )
    return self.toggled and table.contains( self.series_table, card.series )
end

function CardSeriesFilterRow:SetProgress( counts )
	self.counts = counts

    if self.counts and self.counts.UNLOCKABLE > 0 and not self.locked then
    	self.value:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_PROGRESS", self.counts.UNLOCKED, self.counts.UNLOCKABLE ) )
    	self.progress_bar:SetProgress( self.counts.UNLOCKED/self.counts.UNLOCKABLE )
    	self.subtitle:Show()
    	self.value:Show()
    	self.progress_bar:Show()

        local tt = self.tt or loc.format( self.tt_format or LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_PLAYER_TT", self.title_name, self.counts.UNLOCKED, self.counts.UNLOCKABLE )
        self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_TT", self.title:GetText(), tt ) )
    else
        self.subtitle:Hide()
        self.value:Hide()
        self.progress_bar:Hide()

        local tt = self.tt or loc.format( self.tt_format or LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_PLAYER_TT", self.title_name, 0, 0 )
        self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_TT", self.title:GetText(), tt ) )
    end
    self:Layout()

    -- Refresh locked status, since that changes the tooltips
    self:SetLocked( self.locked )

    return self
end

function CardSeriesFilterRow:SetWidth( w )
	CardSeriesFilterRow._base.SetWidth( self, w )

    self.progress_bar:SetSize( self.widget_w - 80, 3 )

    self:Layout()
    return self
end

function CardSeriesFilterRow:Layout()
	CardSeriesFilterRow._base.Layout( self )

	self.anim_container:LayoutBounds( "left", "bottom", self.bg )
		:Offset( 3, 0 )
	self.title:LayoutBounds( "left", "top", self.bg )
		:Offset( 70, self.value:IsShown() and -1 or -14 )
	self.subtitle:LayoutBounds( "left", "below", self.title )
		:Offset( 0, 5 )
	self.value:LayoutBounds( nil, "bottom", self.subtitle )
		:LayoutBounds( "right", nil, self.bg )
		:Offset( -10, 0 )
	self.progress_bar:LayoutBounds( "right", "below", self.value )
		:Offset( 0, -1 )

	return self
end

function CardSeriesFilterRow:SetToggled( toggled )
    CardSeriesFilterRow._base.SetToggled( self, toggled )

    self.anim:SetOutline( 8, self.toggled and UICOLOURS.FOCUS or UICOLOURS.DARK_BODYTEXT )
    if self.toggled then AUDIO:PlayEvent( "event:/ui/card_codex/select_character" ) end

    return self
end

function CardSeriesFilterRow:GetAgent()
    return self.agent
end

function CardSeriesFilterRow:Enable()
	CardSeriesFilterRow._base.Enable( self )
	self.ignore_input = false

	self.bg:Bloom( self.toggled and 0.05 or 0 )
	self.bg:SetTexture( self.toggled and assets.deckfilter_bg or assets.deckfilter_bg_disabled )
	self.contents:SetSaturation( self.toggled and 1 or 0.2 )
	self.anim:SetOutline( 8, self.toggled and UICOLOURS.FOCUS or UICOLOURS.DARK_BODYTEXT )

    return self
end

function CardSeriesFilterRow:Disable()
	CardSeriesFilterRow._base.Disable( self )
	self.ignore_input = true

	self.bg:Bloom( 0 )
	self.bg:SetTexture( assets.deckfilter_bg_disabled )
	self.contents:SetSaturation( 0.2 )
	self.anim:SetOutline( 8, UICOLOURS.DARK_BODYTEXT )
	
    return self
end

function CardSeriesFilterRow:SetLocked( locked )
    CardSeriesFilterRow._base.SetLocked( self, locked )

    self.anim:SetTintColour( self.locked and 0x021A18FF or 0xffffffff )
    if self.locked then
        self.title:SetText( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_LOCKED_TITLE" )
        self:SetToolTip( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_LOCKED_TT" )
    end
    self:Layout()

    return self
end

------------------------------------------------------------------------------------------
-- Shows a series/character filter row for grafts

local GraftSeriesFilterRow = class( "Widget.GraftSeriesFilterRow", Widget.CardSeriesFilterRow )

function GraftSeriesFilterRow:init( character, series_table, tt )
    GraftSeriesFilterRow._base.init( self, character, series_table, tt )

    if table.contains( self.series_table, CARD_SERIES.NPC ) then
        self.title_name = LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_SERIES_NPC"
    elseif table.contains( self.series_table, CARD_SERIES.GENERAL ) then
        self.title_name = LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_SERIES_GENERAL"
    end

    -- Text
    self.title:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_SERIES_TITLE", loc.upper( self.title_name ) ) )

    -- Progress
    self.seen_installed_counter = self.contents:AddChild( Widget.SeenInstalledCounterWidget( self.widget_w+20 ) )
        :SetColor( UICOLOURS.SUBTITLE )
        :SetValueSize(22)
        :SetTextSize(22)
        :Hide()

    local tt = self.tt or loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_SERIES_PLAYER_TT", self.title_name, 0, 0, 0 )
    self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_SERIES_TT", self.title:GetText(), tt ) )
end

function GraftSeriesFilterRow:Filter( graft_def, graft_widget )
    return self.toggled and table.contains( self.series_table, graft_def.series )
end

function GraftSeriesFilterRow:SetProgress( counts )
    self.counts = counts
    if self.counts and self.counts.SEEN~=nil and self.counts.INSTALLED~=nil and self.counts.TOTAL and not self.locked then
        self.seen_installed_counter:SetProgress( self.counts.SEEN, self.counts.INSTALLED, self.counts.TOTAL )
            :Show()

        local tt = self.tt or loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_SERIES_PLAYER_TT", self.title_name, self.counts.SEEN, self.counts.INSTALLED, self.counts.TOTAL )
        self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_SERIES_TT", self.title:GetText(), tt ) )
    else
        self.seen_installed_counter:Hide()
    end
    self:Layout()

    -- Refresh locked status, since that changes the tooltips
    self:SetLocked( self.locked )

    return self
end

function GraftSeriesFilterRow:Layout()
    GraftSeriesFilterRow._base.Layout( self )

    self.title:LayoutBounds( "left", "top", self.bg )
        :Offset( 70, self.seen_installed_counter:IsShown() and -1 or -14 )
    self.seen_installed_counter:LayoutBounds( "left", "below", self.title )
        :Offset( 0, 5 )

    return self
end
------------------------------------------------------------------------------------------
-- Shows a series/character filter row for mettle

local MettleSeriesFilterRow = class( "Widget.MettleSeriesFilterRow", Widget.CardSeriesFilterRow )

function MettleSeriesFilterRow:init( character, series_table, tt )
    MettleSeriesFilterRow._base.init( self, character, series_table, tt )

    -- Text
    self.title:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.METTLE_FILTER_SERIES_TITLE", loc.upper( self.title_name ) ) )
    
    self.tt_format = LOC"UI.CARDCOMPENDIUM.METTLE_FILTER_SERIES_PLAYER_TT"
end

function MettleSeriesFilterRow:Filter( graft_def, graft_widget )
    return self.toggled and table.contains( self.series_table, graft_widget.character )
end

------------------------------------------------------------------------------------------
-- Shows a battle/negotiation filter row for cards

local CardMinigameFilterRow = class( "Widget.CardMinigameFilterRow", Widget.GenericFilterBlock )

function CardMinigameFilterRow:init( card_class, audio_event )
    self.widget_h = 60
    CardMinigameFilterRow._base.init( self )
    self.card_class = card_class
    self.audio_event = audio_event

    -- Text
    self.icon = self.contents:AddChild( Widget.Image( card_class == Battle.Card and engine.asset.Texture("UI/cardview_class_battle.tex") or engine.asset.Texture("UI/cardview_class_negotiation.tex") ) )
    	:SetSize( 50, 50 )
    	:SetHiddenBoundingBox( true )
    	:Bloom( 0.05 )
    self.title = self.contents:AddChild( Widget.Label("title", 30 ))
    	-- :SetGlyphColour( UICOLOURS.SUBTITLE )
    	:SetGlyphColour( card_class == Battle.Card and UICOLOURS.FIGHT or UICOLOURS.NEGOTIATION )
    	:SetText( card_class == Battle.Card and LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_TITLE_BATTLE" or LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_TITLE_NEGOTIATION" )
    	:LeftAlign()
    	:Bloom(0.1)
    self.subtitle = self.contents:AddChild( Widget.Label("body", 22 ))
    	-- :SetGlyphColour( UICOLOURS.SUBTITLE )
    	:SetGlyphColour( card_class == Battle.Card and UICOLOURS.FIGHT or UICOLOURS.NEGOTIATION )
    	:SetTintAlpha( 0.8 )
    	:SetText( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_SUBTITLE" )
    	:LeftAlign()
    	:Bloom(0.05)
    self.value = self.contents:AddChild( Widget.Label("body", 22 ))
    	-- :SetGlyphColour( UICOLOURS.SUBTITLE )
    	:SetGlyphColour( card_class == Battle.Card and UICOLOURS.FIGHT or UICOLOURS.NEGOTIATION )
    	:SetTintAlpha( 0.8 )
    	:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_PROGRESS", 0, 0 ) )
    	:RightAlign()
    	:Bloom(0.05)
    self.progress_bar = self.contents:AddChild( Widget.ProgressBar( self.widget_w, 3 ) )
    	:OrientH()
    	:SetOutlineColour( 0x00000000 )
    	:SetBGColour( 0x00000050 )
    	-- :SetBarColour( UICOLOURS.SUBTITLE )
    	:SetBarColour( card_class == Battle.Card and UICOLOURS.FIGHT or UICOLOURS.NEGOTIATION )
    	:SetBarBloom( 0.05 )
end

function CardMinigameFilterRow:Filter( card )
	return self.toggled and is_instance( card, self.card_class )
end

function CardMinigameFilterRow:SetProgress( counts )
	self.counts = counts

	self.value:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_PROGRESS", self.counts.UNLOCKED, self.counts.UNLOCKABLE ) )
	self.progress_bar:SetProgress( self.counts.UNLOCKED/self.counts.UNLOCKABLE )

    self:Layout()
    return self
end

function CardMinigameFilterRow:SetWidth( w )
	CardMinigameFilterRow._base.SetWidth( self, w )

    self.progress_bar:SetSize( self.widget_w - 80, 3 )

    self:Layout()
    return self
end

function CardMinigameFilterRow:Layout()
	CardMinigameFilterRow._base.Layout( self )

	self.icon:LayoutBounds( "left", "center", self.bg )
	 	:Offset( 12, 0 )
	self.title:LayoutBounds( "left", "top", self.bg )
		:Offset( 70, -1 )
	self.subtitle:LayoutBounds( "left", "below", self.title )
		:Offset( 0, 5 )
	self.value:LayoutBounds( nil, "bottom", self.subtitle )
		:LayoutBounds( "right", nil, self.bg )
		:Offset( -10, 0 )
	self.progress_bar:LayoutBounds( "right", "below", self.value )
		:Offset( 0, -1 )

	return self
end

function CardMinigameFilterRow:SetToggled( toggled )
    self.toggled = toggled
    self.bg:Bloom( self.toggled and 0.05 or 0 )
    self.bg:SetTexture( self.toggled and assets.deckfilter_bg or assets.deckfilter_bg_disabled )
    self.contents:SetSaturation( self.toggled and 1 or 0.2 )
    if self.toggled and self.audio_event then AUDIO:PlayEvent( self.audio_event ) end
end


------------------------------------------------------------------------------------------
-- Shows a battle/negotiation filter row for grafts

local GraftMinigameFilterRow = class( "Widget.GraftMinigameFilterRow", Widget.CardMinigameFilterRow )

function GraftMinigameFilterRow:init( graft_type, audio_event )
    self.graft_type = graft_type or GRAFT_TYPE.COMBAT
    self.card_class = (self.graft_type == GRAFT_TYPE.COMBAT) and Battle.Card or Negotiation.Card
    GraftMinigameFilterRow._base.init( self, self.card_class, audio_event )

    -- Text
    self.title:SetText( self.card_class == Battle.Card and LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_MINIGAME_TITLE_BATTLE" or LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_MINIGAME_TITLE_NEGOTIATION" )
    self.title:SetSize( self.widget_w, self.widget_h ):LeftAlign():TopAlign()
    self.title:TryFitToRegion(24, 30)

    -- Progress
    self.seen_installed_counter = self.contents:AddChild( Widget.SeenInstalledCounterWidget( self.widget_w+20 ) )
        :SetColor( (self.graft_type == GRAFT_TYPE.COMBAT) and UICOLOURS.FIGHT or UICOLOURS.NEGOTIATION )
        :SetValueSize(22)
        :SetTextSize(22)
        :Hide()
end

function GraftMinigameFilterRow:Filter( graft_def, graft_widget )
    return self.toggled and graft_def.type == self.graft_type
end

function GraftMinigameFilterRow:SetProgress( counts )
    self.counts = counts
    if self.counts.SEEN~=nil and self.counts.INSTALLED~=nil and self.counts.TOTAL then
        self.seen_installed_counter:SetProgress( self.counts.SEEN, self.counts.INSTALLED, self.counts.TOTAL )
            :Show()

        if self.graft_type == GRAFT_TYPE.COMBAT then
            self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_MINIGAME_BATTLE_TT", self.counts.SEEN, self.counts.INSTALLED, self.counts.TOTAL ) )
        else
            self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_MINIGAME_NEGOTIATION_TT", self.counts.SEEN, self.counts.INSTALLED, self.counts.TOTAL ) )
        end

        self.subtitle:Hide()
        self.value:Hide()
        self.progress_bar:Hide()

        self:Layout()
    end

    return self
end

function GraftMinigameFilterRow:Layout()
    GraftMinigameFilterRow._base.Layout( self )

    self.title:LayoutBounds( "left", "top", self.bg )
        :Offset( 70, self.seen_installed_counter:IsShown() and -1 or -14 )
    self.seen_installed_counter:LayoutBounds( nil, "bottom", self.bg )
        :LayoutBounds( "left", nil, self.title )
        :Offset( 0, 5 )

    return self
end

------------------------------------------------------------------------------------------
-- Shows a filter row for agents the player hasn't met

local AgentUnknownFilterRow = class( "Widget.AgentUnknownFilterRow", Widget.GenericFilterBlock )

function AgentUnknownFilterRow:init()
    self.widget_h = 35
    AgentUnknownFilterRow._base.init( self )

    -- Text
    self.title = self.contents:AddChild( Widget.Label("body", 22 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetText( LOC"UI.CARDCOMPENDIUM.AGENT_FILTER_UNKNOWN" )
        :LeftAlign()
        :Bloom(0.1)
end

function AgentUnknownFilterRow:Filter( agent, agent_portrait )
    return self.toggled and agent.is_locked
end

function AgentUnknownFilterRow:Layout()
	AgentUnknownFilterRow._base.Layout( self )

	self.title:LayoutBounds( "left", "center", self.bg )
	 	:Offset( 15, 0 )

	return self
end

------------------------------------------------------------------------------------------
-- Shows a filter row for bosses the player hasn't met

local BossUnknownFilterRow = class( "Widget.BossUnknownFilterRow", Widget.AgentUnknownFilterRow )

function BossUnknownFilterRow:init()
    BossUnknownFilterRow._base.init( self )

    self.title:SetText( LOC"UI.CARDCOMPENDIUM.BOSS_FILTER_UNKNOWN" )
end

------------------------------------------------------------------------------------------
-- Shows a boss filter row

local AgentBossFilterRow = class( "Widget.AgentBossFilterRow", Widget.GenericFilterBlock )

function AgentBossFilterRow:init()
    self.widget_h = 35
    AgentBossFilterRow._base.init( self )

    -- Text
    self.title = self.contents:AddChild( Widget.Label("body", 22 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetText( LOC"UI.CARDCOMPENDIUM.AGENT_FILTER_BOSS" )
        :LeftAlign()
        :Bloom(0.1)
end

function AgentBossFilterRow:Filter( agent, agent_portrait )
    return self.toggled and agent.boss
end

function AgentBossFilterRow:Layout()
    AgentBossFilterRow._base.Layout( self )

    self.title:LayoutBounds( "left", "center", self.bg )
        :Offset( 15, 0 )

    return self
end

------------------------------------------------------------------------------------------
-- Shows a regular-person (non-boss) filter row

local AgentNonBossFilterRow = class( "Widget.AgentNonBossFilterRow", Widget.GenericFilterBlock )

function AgentNonBossFilterRow:init()
    self.widget_h = 35
    AgentNonBossFilterRow._base.init( self )

    -- Text
    self.title = self.contents:AddChild( Widget.Label("body", 22 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetText( LOC"UI.CARDCOMPENDIUM.AGENT_FILTER_NONBOSS" )
        :LeftAlign()
        :Bloom(0.1)
end

function AgentNonBossFilterRow:Filter( agent, agent_portrait )
    return self.toggled and not agent.boss
end

function AgentNonBossFilterRow:Layout()
    AgentNonBossFilterRow._base.Layout( self )

    self.title:LayoutBounds( "left", "center", self.bg )
        :Offset( 15, 0 )

    return self
end

------------------------------------------------------------------------------------------
-- Shows a sorting option filter row

local SortingOptionFilterRow = class( "Widget.SortingOptionFilterRow", Widget.GenericFilterBlock )

function SortingOptionFilterRow:init( text )
    self.widget_h = 35
    SortingOptionFilterRow._base.init( self )

    -- Text
    self.title = self.contents:AddChild( Widget.Label("body", 22 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetText( text )
        :LeftAlign()
        :Bloom(0.1)
end

function SortingOptionFilterRow:SetFn( fn )

    return self
end

function SortingOptionFilterRow:Filter( agent, agent_portrait )
    return true
end

function SortingOptionFilterRow:Layout()
    SortingOptionFilterRow._base.Layout( self )

    self.title:LayoutBounds( "left", "center", self.bg )
        :Offset( 15, 0 )

    return self
end

------------------------------------------------------------------------------------------
-- Shows a faction filter row

local AgentFactionFilterRow = class( "Widget.AgentFactionFilterRow", Widget.GenericFilterBlock )

function AgentFactionFilterRow:init( faction_id, tt, icon )
    self.widget_h = 45
    AgentFactionFilterRow._base.init( self )
    self.faction_id = faction_id
    self.faction_data = faction_id and Content.GetFaction( faction_id ) or Content.GetFaction( "FEUD_CITIZEN" )

    -- Text
    self.icon = self.contents:AddChild( Widget.Image( ) )
        :SetSize( self.widget_h-12, self.widget_h-12 )
        :SetTexture( self.faction_data and self.faction_data.icon )
        :SetHiddenBoundingBox( true )
        :Bloom( 0.05 )
    if icon then self.icon:SetTexture( icon ) end

    self:SetToolTip( tt or loc.format( LOC"UI.CARDCOMPENDIUM.FILTER_FACTION_TT", LOC(self.faction_data:GetLocNameKey()), LOC(self.faction_data:GetLocDescKey()) ) )
end

function AgentFactionFilterRow:Filter( agent, agent_portrait )
    return self.toggled and agent:GetFactionID() == self.faction_id
end

function AgentFactionFilterRow:Layout()
    AgentFactionFilterRow._base.Layout( self )

    self.icon:LayoutBounds( "center", "center", self.bg )
        :Offset( 2, 0 )

    return self
end

------------------------------------------------------------------------------------------
-- Shows a rarity filter row for cards

local CardRarityFilterRow = class( "Widget.CardRarityFilterRow", Widget.GenericFilterBlock )

function CardRarityFilterRow:init( rarity, tt )
    self.widget_h = 45
    CardRarityFilterRow._base.init( self )
    self.rarity = rarity or CARD_RARITY.BASIC

    -- Text
    self.icon = self.contents:AddChild( Widget.Image( ) )
        :SetTexture( global_images.card_rarity_icons[self.rarity] )
        :SetSize( self.widget_h-12, self.widget_h-12 )
        :SetHiddenBoundingBox( true )
        :Bloom( 0.05 )

    self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_RARITY_TT", GetCardRarityString(self.rarity), tt ) )
end

function CardRarityFilterRow:Filter( card )
    return self.toggled and card.rarity == self.rarity
end

function CardRarityFilterRow:Layout()
    CardRarityFilterRow._base.Layout( self )

    self.icon:LayoutBounds( "center", "center", self.bg )
        :Offset( 2, 0 )

    return self
end

------------------------------------------------------------------------------------------
-- Shows a rarity filter row for grafts

local GraftRarityFilterRow = class( "Widget.GraftRarityFilterRow", Widget.GenericFilterBlock )

function GraftRarityFilterRow:init( rarity, tt )
    GraftRarityFilterRow._base.init( self )
    self.rarity = rarity or CARD_RARITY.BASIC

    -- Text
    self.icon = self.contents:AddChild( Widget.Image( ) )
        :SetTexture( global_images.card_rarity_icons[self.rarity] )
        :SetSize( self.widget_h-12, self.widget_h-12 )
        :SetHiddenBoundingBox( true )
        :Bloom( 0.05 )

    self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_RARITY_TT", GetCardRarityString(self.rarity), tt ) )
end

function GraftRarityFilterRow:Filter( graft_def, graft_widget )
    return self.toggled and graft_def.rarity == self.rarity
end

function GraftRarityFilterRow:Layout()
    GraftRarityFilterRow._base.Layout( self )

    self.icon:LayoutBounds( "center", "center", self.bg )
        :Offset( 2, 0 )

    return self
end

------------------------------------------------------------------------------------------
-- Shows a type filter row

local CardTypeFilterRow = class( "Widget.CardTypeFilterRow", Widget.GenericFilterBlock )

function CardTypeFilterRow:init( type, tt )
    self.widget_h = 45
    CardTypeFilterRow._base.init( self )
    self.type = type or CARD_TYPE.ITEM
    self.types = {}
	table.insert( self.types, type )

    -- Text
    self.icon = self.contents:AddChild( Widget.Image( ) )
    	:SetTexture( GetCardTypeIcon(self.type) )
    	:SetTintColour( GetCardTypeColour(self.type) )
    	:SetSize( self.widget_h-12, self.widget_h-12 )
    	:SetHiddenBoundingBox( true )
    	:Bloom( 0.06 )

    self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.FILTER_TYPE_TT", GetCardTypeString(self.type), tt ) )
end

function CardTypeFilterRow:AddType( type )
	table.insert( self.types, type )
	return self
end

function CardTypeFilterRow:Filter( card )
	return self:IsVisible() and self.toggled and table.arrayfind( self.types, GetCardType( card ) ) ~= nil
end

function CardTypeFilterRow:Layout()
	CardTypeFilterRow._base.Layout( self )

	self.icon:LayoutBounds( "center", "center", self.bg )
	 	:Offset( 2, 0 )

	return self
end

------------------------------------------------------------------------------------------
-- Shows a search filter row for cards. This one is extended for the other types of content

local CardSearchFilterBlock = class( "Widget.CardSearchFilterBlock", Widget.GenericFilterBlock )

function CardSearchFilterBlock:init()
    self.widget_h = 35
    CardSearchFilterBlock._base.init( self )

    -- Text
    self.icon = self.contents:AddChild( Widget.Image() )
    	:SetTexture( engine.asset.Texture("ui/ic_searchfilter_icon.tex") )
    	:SetTintColour( GetCardTypeColour(self.type) )
    	:SetSize( self.widget_h-20, self.widget_h-20 )
    	:SetTintColour( UICOLOURS.SUBTITLE )
    	:SetHiddenBoundingBox( true )
    	:Bloom( 0.06 )
    self.editlabel = self.contents:AddChild( Widget.EditLabel( 24, self.widget_w - 34, 1 ) )
    	:SetString( LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH_DEFAULT_STRING" )
    	:SetFont( "body" )
    	:SetBGColour( 0x00000000 )
    	:SetTextColour( UICOLOURS.SUBTITLE )
    	:SetFn( function() self:OnTextChange() end )
    	:SetOnEnterFn( function() self:OnTextEnterPressed() end )
    	:SetGainFocusFn( function() self:OnTextGainFocus() end )
    	:SetLoseFocusFn( function() self:OnTextLoseFocus() end )
    	:Bloom( 0.05 )
    self.editlabel.outline:Hide()
    self:OnTextLoseFocus()
end

function CardSearchFilterBlock:Filter( card )
	-- Do a search based on the card's name and description
	local search_string = self.editlabel:GetString()
	if self.toggled and search_string ~= LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH_DEFAULT_STRING" and search_string ~= "" then
		
        -- Return false if it's still locked
        if card.card_locked then return false end

        -- Get this card's description
        local defs
        if card._class == Battle.Card then
            defs = require "battle/battle_defs"
        elseif card._class == Negotiation.Card then
            defs = require "negotiation/negotiation_defs"
        end
        local card_text = card:GetName() .. ( CardEngine.GenerateCardDesc( card, nil, nil, defs ) or "" )

        -- Look for the search string on the card's title and description
		if string.find ( string.lower(card_text), string.lower(search_string), 1, true ) ~= nil then
			-- This card matches!
			return true
		else
			return false
		end

	end
	return true
end

function CardSearchFilterBlock:SetWidth( w )
	CardSearchFilterBlock._base.SetWidth( self, w )

    self.editlabel:SetSize( self.widget_w - 34, self.widget_h-10 )

    self:Layout()
    return self
end

function CardSearchFilterBlock:SetAlignRight( align_right )
	self.align_right = align_right
	self:Layout()
	return self
end

function CardSearchFilterBlock:Layout()
	CardSearchFilterBlock._base.Layout( self )

	self.bg:SetScale( 1, 1 )
	self.icon:LayoutBounds( self.align_right and "right" or "left", "center", self.bg )
	 	:Offset( self.align_right and -12 or 12, 0 )
	self.editlabel:LayoutBounds( self.align_right and "left" or "right", "center", self.bg )
	 	:Offset( self.align_right and 8 or -8, 0 )
	self.gradient:LayoutBounds( self.align_right and "right" or "left", nil, self.bg )
	self.gradient:SetScale( self.align_right and -1 or 1, 1 )
	self.bg:SetScale( self.align_right and -1 or 1, 1 )

	return self
end

function CardSearchFilterBlock:OnClick()
	if self.editlabel:GetString() == LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH_DEFAULT_STRING" then self.editlabel:SetValue("",true) end
	self.editlabel:SetFocus()
end

function CardSearchFilterBlock:OnTextChange()

end

function CardSearchFilterBlock:OnTextEnterPressed()
	self:TriggerUpdate()
end

function CardSearchFilterBlock:OnTextGainFocus()
	self.bg:Bloom( self.toggled and 0.05 or 0 )
	self.bg:SetTexture( assets.deckfilter_bg )
	self.contents:SetSaturation( 1 )
end

function CardSearchFilterBlock:OnTextLoseFocus()
	self:TriggerUpdate( true )
end

function CardSearchFilterBlock:TriggerUpdate( skip_audio )
    if not skip_audio then AUDIO:PlayEvent( "event:/ui/card_codex/search_execute_success" ) end
	local search_string = self.editlabel:GetString()
	if search_string == LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH_DEFAULT_STRING" or search_string == "" then
		self.editlabel:SetValue(LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH_DEFAULT_STRING",true)
		self.toggled = false
	else
		self.toggled = true
	end
	self.bg:Bloom( self.toggled and 0.05 or 0 )
	self.bg:SetTexture( self.toggled and assets.deckfilter_bg or assets.deckfilter_bg_disabled )
	self.contents:SetSaturation( self.toggled and 1 or 0.2 )

	if self.on_toggle_fn then self.on_toggle_fn( self, self.toggled ) end
end

------------------------------------------------------------------------------------------
-- Shows a search filter row for people

local PeopleSearchFilterBlock = class( "Widget.PeopleSearchFilterBlock", Widget.CardSearchFilterBlock )

function PeopleSearchFilterBlock:init()
    PeopleSearchFilterBlock._base.init( self )
end

function PeopleSearchFilterBlock:Filter( agent_def, agent_widget )
    -- Do a search based on the agent's name and description
    local search_string = self.editlabel:GetString()
    if self.toggled and search_string ~= LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH_DEFAULT_STRING" and search_string ~= "" then
        
        -- Return false if it's still locked
        if agent_widget.is_locked then return false end

        if string.find ( string.lower(agent_widget.search_string), string.lower(search_string), 1, true ) ~= nil then
            -- This agent matches!
            return true
        else
            return false
        end

    end
    return true
end

------------------------------------------------------------------------------------------
-- Shows a search filter row for grafts

local GraftSearchFilterBlock = class( "Widget.GraftSearchFilterBlock", Widget.CardSearchFilterBlock )

function GraftSearchFilterBlock:init()
    GraftSearchFilterBlock._base.init( self )
end

function GraftSearchFilterBlock:Filter( graft_def, graft_widget )
    -- Do a search based on the graft's name and description
    local search_string = self.editlabel:GetString()
    if self.toggled and search_string ~= LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH_DEFAULT_STRING" and search_string ~= "" then
        
        -- Return false if it's still locked
        if graft_widget.is_locked then return false end

        if string.find ( string.lower(graft_widget.search_string), string.lower(search_string), 1, true ) ~= nil then
            -- This graft matches!
            return true
        else
            return false
        end

    end
    return true
end


