require "ui/widgets/screen"
require "ui/widgets/inlinenotification"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/multichoice"
require "ui/widgets/presentation"
require "ui/widgets/panel"
require "ui/widgets/panelbutton"
require "ui/widgets/characterstage"
require "ui/widgets/agentstatusportrait"
require "ui/widgets/textbox"
require "ui/widgets/scrolllog"
require "ui/widgets/negotiationpreviewpanel"
require "ui/widgets/negotiationpanel"
require "ui/widgets/overlayeffect"
local paperdoll = require "paperdoll_util"
require "ui/widgets/tooltip_convo_option"
require "ui/widgets/tooltipcodex"
require "ui/widgets/moneyindicator"
require "ui/uihelpers"


local MARGIN = 25
local BOTTOM_MARGIN = 65
local BUBBLE_WIDTH = 500
local CONTROL_BAR_OFFSET = 50
local SPEECH_ADJUST = 310
local SPEECH_ALIGN_LEFT, SPEECH_ALIGN_RIGHT = -SPEECH_ADJUST, SPEECH_ADJUST

local NEGOTIATION_TOP_MARGIN = 400 - 200
local TOP_MARGIN = 400

local GUTTER = 8
local CHOICE_H = 400
local VOICE_DELAY = .2

local EMOTIONS =
{
    "alerted",
    "alluring",
    "amazed",
    "anxious",
    "bored",
    "cheerful",
    "cocky",
    "condescending",
    "depressed",
    "direct",
    "disapproving",
    "dubious",
    "embarrassed",
    "fearful",
    "hopeless",
    "hostile",
    "hushed",
    "impressed",
    "interested",
    "irritated",
    "neutral",
    "nostalgic",
    "panicked",
    "playful",
    "relieved",
    "resigned",
    "seething",
    "skeptical",
    "thankful",
    "thoughtful",

	-- NEW AND SHIPPING CONTENT BELOW
	
	"angryDefault",
	"angryAside",
	"angryCruel",
	"angryGreeting",
	"angryHostile",
	"angryPatienceLost",
	"angryReluctant",
	"angrySeething",
	"angryTakenAback",
	"angryThanks",
	
	"happyDefault",
	"happyAmazed",
	"happyCocky",
	"happyGreeting",
	"happyJoke",
	"happyResigned",
	"happyTakenAback",
	"happyThanks",
	
	"miscAlluring",
	"miscDepressed",
	"miscEmbarrassed",
	"miscHopeless",
	"miscHushedUrgent",
	"miscMischievous",
	"miscMocking",
	"miscNostalgic",
	"miscPersuasive",
	"miscRelieved",
	
	"neutralDefault",
	"neutralAside",
	"neutralDirect",
	"neutralDubious",
	"neutralGreeting",
	"neutralJoke",
	"neutralResigned",
	"neutralTakenAback",
	"neutralThanks",
	"neutralThoughtful",
	"neutralWhatever",
	
	"scaredDefault",
	"scaredAnguished",
	"scaredFearful",
	"scaredGreeting",
	"scaredPowerless",
	"scaredStammering",
	"scaredTakenAback",
	"scaredThanks",
}

---------------------------------------------------------------------------------

local function TextBubbleTooltips( widget, tag, text, hub )
    if text ~= nil then
        AUDIO:PlayEvent(SoundEvents.hover_link)
    end

    if hub then
        -- Custom codex and default codex.
        local codex_entry
        local codex = hub.GetCodex and hub:GetCodex()

        if codex then
            codex_entry = codex:LookupEntry( tag )
        end
        codex_entry = codex_entry or Content.LookupCodexEntry( tag )

        if codex_entry then
            widget:SetToolTip( codex_entry )
            widget:SetToolTipClass( Widget.TooltipCodex )
            return
        end
    end

    UIHelpers.ItemTooltips( widget, tag, text )
end

local function TextBubbleClicked( widget, tag, text, hub )
    UIHelpers.ItemTooltipsClicked( widget, tag, text )
end

---------------------------------------------------------------------------------

local ConversationScreen = class("Screen.ConversationScreen", Widget.Screen)
ConversationScreen.sinks_input = true

ConversationScreen.CONTROL_MAP = {
    {
        test = function(self)
            return not self.show_reticule
        end,
        hint = function(self, left, right)
            table.insert(left, LOC"UI.CONTROLS.NAV")
        end
    },
    {
        control = Controls.Digital.NEXT_LINE,
        fn = function(self)
            if self.hurry_cards then
                self.hurry_cards()
                return true
            end
            if self.waiting_for_skip then
                AUDIO:PlayEvent("event:/ui/main/cnv/dia_advance_subtle")
                self:NextLine()
                return true
            end
        end,

        test = function(self) return (self.hurry_cards or self.waiting_for_skip) and not self.negotiation_panel and not self.show_reticule end,
        hint = function(self, left, right) 
            table.insert( right, loc.format( LOC"UI.CONTROLS.NEXT_LINE", Controls.Digital.NEXT_LINE ))
        end,
    },
    {
        control = Controls.Digital.SKIP,
        fn = function(self)
            if self.waiting_for_skip then
                AUDIO:PlayEvent(SoundEvents.button_click)
                self:SetAutoSkip( true )
                self:NextLine()
                return true
            end
        end,

        test = function(self) return self.waiting_for_skip and not self.negotiation_panel and not self.show_reticule end,
        hint = function(self, left, right) 
            table.insert( right, loc.format( LOC"UI.CONTROLS.CONVO_SKIP", Controls.Digital.SKIP ))
        end,
    },
    {
        -- This scrolls the conversation back to the bottom on left-click if the person scrolled up
        control = Controls.Digital.NEXT_LINE,
        fn = function(self)
            AUDIO:PlayEvent(SoundEvents.button_click)
            self.scroll_panel:SetScrollPercent(1,false)
            return true
        end,

        test = function(self) return not self.waiting_for_skip and not self.negotiation_panel end,
    },
    {
        -- This scrolls the conversation back to the bottom on right-click if the person scrolled up
        control = Controls.Digital.SKIP,
        fn = function(self)
            AUDIO:PlayEvent(SoundEvents.button_click)
            self.scroll_panel:SetScrollPercent(1,false)
            return true
        end,

        test = function(self) return not self.waiting_for_skip and not self.negotiation_panel end,
    },

    {
        control = Controls.Digital.PAUSE, 
        fn = function(self)
            self:GetFE():InsertScreen(Screen.PauseMenu())
            return true
        end,
    },

    {
        controls = { Controls.Digital.MENU_CANCEL, Controls.Digital.MENU_UP, Controls.Digital.MENU_RIGHT, Controls.Digital.MENU_DOWN, Controls.Digital.MENU_LEFT },
        test = function(self)
            return self.show_reticule == true
        end,
        fn = function(self)
            self:SetReticuleMode( false )
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.NEGOTIATION_PANEL.CONTROLS_LEFT_INSPECT_RELEASE", Controls.Digital.MENU_CANCEL ) )
        end,
    },
    {
        hint = function(self, left, right)
            if self.show_reticule then
                table.insert(left, LOC"UI.NEGOTIATION_PANEL.CONTROLS_LEFT_MOVE_INSPECT" )
            else
                table.insert(left, LOC"UI.NEGOTIATION_PANEL.CONTROLS_LEFT_INSPECT" )
            end
        end,
    },
    {
        control = Controls.Digital.MENU_CANCEL, 
        test = function(self)
            if self.multi:HasFocus() then
                return (self.negotiation_preview_panel and self.negotiation_preview_panel:IsFadedIn()) or
                    (self.battle_preview_panel and self.battle_preview_panel:IsFadedIn())
            end
        end,
        hint = function(self, left, right)
            if self.negotiation_preview_panel and self.negotiation_preview_panel:IsFadedIn() then
                table.insert(right, loc.format(LOC"UI.CONTROLS.CANCEL", Controls.Digital.MENU_CANCEL))
            elseif self.battle_preview_panel and self.battle_preview_panel:IsFadedIn() then
                table.insert(right, loc.format(LOC"UI.CONTROLS.CANCEL", Controls.Digital.MENU_CANCEL))
            -- else
            --     table.insert(right, loc.format(LOC"UI.CONTROLS.UNSELECT", Controls.Digital.MENU_CANCEL))
            end
        end,
        fn = function(self)
            if self.negotiation_preview_panel and self.negotiation_preview_panel:IsFadedIn() then
                self:NegotiationPreviewPanelLoseHover()
            elseif self.battle_preview_panel and self.battle_preview_panel:IsFadedIn() then
                self:BattlePreviewPanelLoseHover()
            -- else
            --     TheGame:FE():ClearFocusWidget()
            end
            return true
        end,
    },
    {
        -- Goes back if no option is selected, and there is a Leave option
        control = Controls.Digital.MENU_CANCEL, 
        test = function(self)
            if not self.show_reticule then
                return self.multi:GetCloseOption() ~= nil and not self.waiting_for_skip and not self.negotiation_panel
            end
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.LEAVE", Controls.Digital.MENU_CANCEL))
        end,
        fn = function(self)
            local opt = self.multi:GetCloseOption()
            if opt then
                opt:Click()
                return true
            end
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_RS_DOWN,
        test = function(self) return TheGame:FE():IsGamepadMode() end,
        hint = function(self, left, right) 
            table.insert( left, loc.format( LOC"UI.CONTROLS.DIALOG" ))
        end,
    },

    {
        control = Controls.Digital.GAMEPAD_RT, 
        test = function(self)
            if not self.show_reticule then
                if self:IsNegotiationPreviewPanelShowing() or self:IsBattlePreviewPanelShowing() then
                    return false
                end
                return self.right_character_portrait:GetCharacter() and self.right_character_portrait:IsShown()
            end
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.INSPECT", Controls.Digital.GAMEPAD_RT, self.right_character_portrait:GetCharacter() ))
        end,
        fn = function(self)
            self.right_character_portrait:OnClick()
            return true
        end,
    },

}

function ConversationScreen:init( plax )
    ConversationScreen._base.init(self)

    self.is_transparent = true
    
    self.choice_timer = 0
    self.encounter_stack = {}
    self.plax = plax

    -- Add our character stage
    self.stage = self:AddChild( Widget.CharacterStage() )
        :InheritTransform(true)
        :SetAnchors("center", "bottom")
        :Offset(0, RES_Y/2)

    -- And a container for our UI
    self.interface = self:AddChild( Widget() )
        :InheritTransform(true)
        :SetAnchors("center", "bottom")

    -- A darkening background
    self.fullscreen_fade = self.interface:AddChild( Widget.SolidBox( RES_X + 4, RES_Y + 4, 0xFFFFFFFF ) )
        :SetAnchors("fill", "fill")
        :Hide()
    self.scroll_panel = self.interface:AddChild( Widget.ScrollPanel() )
        :SetAnchors("center", "center")
        :SetVirtualTopMargin( TOP_MARGIN )
        :SetVirtualBottomMargin( TheGame:FE():IsGamepadMode() and BOTTOM_MARGIN + CONTROL_BAR_OFFSET or BOTTOM_MARGIN )
        :SetScrollBarOuterMargin( 0 )
        :SetScrollSpeed( 7.0, 1200 )
        :ShowBar( SCROLLBAR.NEVER )
    -- The speech bubbles container
    self.bubbles = self.scroll_panel:AddScrollChild( Widget.ScrollLog() )
    self.bubbles:SetShown( TheGame:FE():IsHUDVisible( HUD_FLAGS.SHOW_CHAT_BUBBLES ))

    -- The convo options
    self.multi = self.scroll_panel:AddScrollChild( Widget.MultiChoice() )
        :SetOnSelectOptionFn( function(idx) self:OnSelectOption(idx) end )
        :ClearOptions()

    -- Click-to-continue widget
    self.next = self.interface:AddChild( Widget.ClickToContinue() )
        :Hide()
        :SetPos(0, CHOICE_H/2 + 300)
    -- Add the portraits of the agents
    self.portraits = self.interface:AddChild( Widget() )
    -- Left side portrait
    self.bottom_left = self.portraits:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.left_character_portrait = self.bottom_left:AddChild( Widget.AgentStatusPortrait(true):ShowOpinion(true) )
        :SetFocusable( true )
        :ShowToolTipOnFocus()
        :LayoutBounds("left", "bottom", 0, 0)
        :Offset(MARGIN, BOTTOM_MARGIN)
        :Hide()
    -- Right side portrait
    self.bottom_right = self.portraits:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.right_character_portrait = self.bottom_right:AddChild( Widget.AgentStatusPortrait(false):ShowOpinion(true) )
        :SetFocusable( true )
        :ShowToolTipOnFocus()
        :LayoutBounds("right", "bottom", 0, 0)
        :Offset(-MARGIN, BOTTOM_MARGIN)
        :Hide()

    -- Add an overlay effect
    self.overlayeffect = self:AddChild( Widget.OverlayEffect() )
        :SetAnchors("fill", "fill")
        :SetLayerSpeeds(-0.025,-0.08,-0.4)
        :SetLayerScale(2.0)
        :SetLayerAlphas(0.3,0.1,0.2)
        :SetColour(0xFFFF00FF)
        :SetShutterHeight(0.2)
        :SetBloom(0.3)
        :SetDuration(2.5)

    self:SetAutoSkip( false )
    self:Layout()
end

function ConversationScreen:GetControlMap()
    if self.negotiation_panel then
        return self.negotiation_panel.CONTROL_MAP, self.negotiation_panel
    else
        return self.CONTROL_MAP, self
    end
end

function ConversationScreen:GetMusicEvent()
    local music_event = self.encounter and self.encounter:GetMusicEvent() or self.plax:GetMusicEvent()
    if self.negotiation_panel then
        return music_event
    else
        return music_event, self.character_music
    end
end

function ConversationScreen:ClearHistory()
    self.bubbles:Clear()
    self.waiting_for_skip = false
    return self
end

function ConversationScreen:TestReticule(wx, wy)
    local main_overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    if main_overlay then
        local mouse_blocked, hover_widget = main_overlay:CheckMouseHover(wx, wy)
        if hover_widget then
            return hover_widget
        end
    end

    local mouse_blocked, hover_widget = self:CheckMouseHover(wx, wy)
    if hover_widget and hover_widget ~= self.scroll_panel then
        return hover_widget
    end
end

function ConversationScreen:WantsReticuleMode()
    if self.show_reticule then
        return self.show_reticule
    end
    return self.negotiation_panel and self.negotiation_panel:WantsReticuleMode()
end

function ConversationScreen:SetReticuleMode( show_reticule )
    if show_reticule ~= self.show_reticule then
        print( self, "SetReticuleMode", show_reticule )
        self.show_reticule = show_reticule

        if not self.show_reticule then
            self:SetDefaultFocus()
        end

        self.fe:RefreshControlOptions()
   end
end

function ConversationScreen:ToggleHUD( visible, hud_flags )
    self.bubbles:SetShown( visible or CheckBits( hud_flags, HUD_FLAGS.SHOW_CHAT_BUBBLES ))

    if self.negotiation_panel then
        self.negotiation_panel:ToggleHUD( visible, hud_flags )
        self.multi:SetShown( false )
    else
        self.multi:SetShown( visible or CheckBits( hud_flags, HUD_FLAGS.SHOW_CHAT_OPTIONS ))
    end
    return self
end

function ConversationScreen:StartConvo( encounter, ondone )
    
    
    for k,v in pairs(encounter.left_agents) do
        paperdoll.LoadAgentAnims(v)
    end
    for k,v in pairs(encounter.right_agents) do
        paperdoll.LoadAgentAnims(v)
    end

    if #self.encounter_stack > 0 then
        for i, enc in ipairs( self.encounter_stack ) do
            if enc.coro then
                print( i, coroutine.status( enc.coro ), enc.location, debug.traceback( enc.coro ))
            else
                print( i, "no coro", enc.location )
            end
        end
    end
    table.insert( self.encounter_stack, { encounter, ondone } )

    self.encounter = encounter

    self.stage:SetWindy( not encounter:GetLocation():IsIndoors() )
    self.encounter:StartConvo( self )

    if self.plax then
        self.plax:ShowLabels( false )
        --self.plax:DoBlurEffect(0)
    end
    return self
end

function ConversationScreen:FinishConvo()

    TheGame:GetMusic():StopCharacterMusic()
    self.character_music = nil

    local t = table.remove( self.encounter_stack )
    local encounter, ondone = table.unpack( t )

    self.encounter:StopConvo()

    if #self.encounter_stack > 0 then
        self.encounter = self.encounter_stack[ #self.encounter_stack ][1]
    else
        self.encounter = nil
    end

    if self.encounter == nil then
        assert( not self.closing )
        self.closing = true

        self.interface:Hide()
        self.stage:RemoveAllCharacters()
        self.stage:Center()
    else
        if self.encounter:GetLeftAgent() then
            self:AddCharacter( self.encounter:GetLeftAgent(), true )
        else
            self:RemoveCharacter( nil, true )
        end
        if self.encounter:GetRightAgent() then
            self:AddCharacter( self.encounter:GetRightAgent(), false )
        else
            self:RemoveCharacter( nil, false )
        end
    end

    DoAutoSave()

    if ondone then
        ondone( self )
    end
    return self
end

function ConversationScreen:ReloadConversation()
    self.encounter:StopConvo()
    self.stage:Clear()
    self.multi:ClearOptions()

    self.next:Hide()
    self.show_skip_in_time = nil
    self.waiting_for_skip = false

    self.bubbles:Clear()
    self.stage:Center()
    
    self:Refresh()

    if self.negotiation_panel then
        self.negotiation_panel:Remove()
        self.negotiation_panel = nil
    end

    self.encounter:StartConvo( self )
    return self
end

function ConversationScreen:SetBlur(val)
    if val then
        self.plax:DoBlurEffect()
    else
        self.plax:StopBlurEffect()
    end
    return self
end


function ConversationScreen:OnClose()
    ConversationScreen._base.OnClose( self )
    
    if self.plax then
        self.plax:ShowLabels( true )
        self.plax:StopBlurEffect()
    end

    if self.encounter then
        self.encounter:StopConvo()
    end
    return self
end

function ConversationScreen:AddText(txt, style)
    self:WaitOnLine()

    txt = Content.SubstituteCodexFormatting( txt )

    assert( txt and #txt > 0 )

    local widget = Widget.TextBox()
    widget:SetMaxWidth(BUBBLE_WIDTH):SetPadding( 0 )
    widget:GetLabel():SetHoverLinkFn( TextBubbleTooltips, self.encounter.active_hub )
    widget:GetLabel():SetClickLinkFn( TextBubbleClicked, self.encounter_active_hub )
    if style then
        widget:SetSpeechBubble( nil, style )
    end
    widget:SetText(txt)

    self:SnapSpool()

    widget:SetLayoutScale( self:GetTextBubbleScale()  )
    self.bubbles:AddWidget( widget )
    self:LayoutBubbles()
    self.scroll_panel:SetScrollPercent( 1, true ) -- Scroll to end

    widget:Spool()

    if self.negotiation_panel then
        local tx, ty = widget:GetPos()
        widget:SetPos( tx, ty )
        widget:AlphaTo( 1.0, 0.2 )
    end


    self.waiting_for_skip = true
    return self
end


function ConversationScreen:ShowGainCards(cards, fn)
    self.hurry_cards = UIHelpers.ShowGainCards(cards, nil, nil, function() self.hurry_cards = nil if fn then fn() end end )
    return self
end        

function ConversationScreen:ShowLostCards(cards, fn)
    local overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    self.hurry_cards = UIHelpers.ShowLostCards(cards, overlay, function() self.hurry_cards = nil if fn then fn() end end )
    return self
end        

function ConversationScreen:AddBubbleWidget( widget, wait_for_skip )
    self:WaitOnLine()

    self:SnapSpool()

    widget:SetLayoutScale( LAYOUT_SCALE[TheGame:FE():GetScreenMode()] )
    self.bubbles:AddWidget( widget )
    self:LayoutBubbles()
    self.scroll_panel:SetScrollPercent( 1, true ) -- Scroll to end

    if wait_for_skip then
        self.waiting_for_skip = true
    end
    return self
end

function ConversationScreen:RemoveBubbleWidget( widget )
    self.bubbles:RemoveWidget( widget )
    return self
end

function ConversationScreen:AddCustomBubble( txt, custom_widget )
    assert(type(txt) == "string")
    self:WaitOnLine()
    
    local widget = Widget.TextBox():SetColour(UICOLOURS.COMIC_PANEL_BG):SetMaxWidth(BUBBLE_WIDTH)
    widget:SetText( txt, custom_widget )
    
    self:SnapSpool()

    widget:SetLayoutScale( LAYOUT_SCALE[TheGame:FE():GetScreenMode()] )
    self.bubbles:AddWidget( widget )
    self:LayoutBubbles()
    self.scroll_panel:SetScrollPercent( 1, true ) -- Scroll to end

    self.waiting_for_skip = true
    return self
end

function ConversationScreen:Fade( duration, fade_in, r, g, b )
    if not self.fullscreen_fade:IsShown() then
        self.fullscreen_fade:SetTintColour( r or 0, g or 0, b or 0, 0 ):Show()
    end

    local alpha
    if fade_in == true then
        alpha = 0
    elseif type(fade_in) == "number" then
        alpha = fade_in
    else
        alpha = 1
    end

    if duration > 0 then
        local onFade
        if alpha <= 0 then
            onFade = function() self.fullscreen_fade:Hide() end
        end
        self.fullscreen_fade:ColourTo( nil, nil, nil, nil, r or 0, g or 0, b or 0, alpha, duration, easing.linear, onFade )
        if self.encounter and coroutine.running() == self.encounter.coro then
            self.encounter:WaitOnWidget( self.fullscreen_fade )
        end
    else
        self.fullscreen_fade:SetTintColour( r or 0, g or 0, b or 0, alpha )
    end
    return self
end

function ConversationScreen:FireHealthOverlay( colour )
    self.overlayeffect:Fire( colour )
    return self
end

function ConversationScreen:FadeTo( duration, clr, async, start_clr )
    if not self.fullscreen_fade:IsShown() then
        if start_clr then
            self.fullscreen_fade:SetTintColour(start_clr)
        else
            self.fullscreen_fade:SetTintColour( 1, 1, 1, 0 )
        end
        self.fullscreen_fade:Show()
    end

    if clr == nil then
        self.fullscreen_fade:Hide()
    else
        local r, g, b, a = HexColour( clr )
        if duration > 0 then
            self.fullscreen_fade:ColourTo( nil, nil, nil, nil, r, g, b, a, duration, easing.linear )
            if not async then
                self.encounter:WaitOnWidget( self.fullscreen_fade )
            end
        else
            self.fullscreen_fade:SetTintColour( r, g, b, a )
        end
    end
    return self
end

function ConversationScreen:ClearOptions()   
    self:ClearRobotOptions()
    self.multi:ClearOptions()
    return self
end

function ConversationScreen:SnapSpool()
    for i, w in self.bubbles:BubbleWidgets() do
        if w.SnapSpool then
            w:SnapSpool()
        end
    end
    return self
end

function ConversationScreen:RunNegotiation( minigame )
    local function IsDoneNegotiation()
        if self.negotiation_panel == nil or self.negotiation_panel:IsDone() then
            return true
        else
            return false
        end
    end

    self.multi:Hide()
    self.next:Hide()
    self.show_skip_in_time = nil
    self:ClearHistory()
    self.portraits:Hide()

    local overlay = TheGame:FE():FindScreen( Screen.MainOverlay )

    if overlay then
        overlay:SetModeOverride( OVERLAY_MODE.NEGOTIATION )
    end

    self:WaitOnLine()

    assert( self.negotiation_panel == nil )
    self.negotiation_panel = Widget.NegotiationPanel( minigame, self.encounter )
    self:AddChild( self.negotiation_panel )
    TheGame:FE():RefreshMusic()

    TheGame:SetAmbienceParam("playing_cards", 1)

    self.encounter:YieldEncounter( IsDoneNegotiation )

    TheGame:SetAmbienceParam("playing_cards", 0)

    if overlay then
        overlay:SetModeOverride( nil )
        overlay:SetPlaxMode()
    end

    self:SetAutoSkip( false )
    self.waiting_for_skip = false

    self.portraits:Show()
    self.negotiation_panel:Remove()
    self.negotiation_panel = nil
    self.overlay_mode_override = nil
    self.multi:SetShown( self:GetFE():IsHUDVisible( HUD_FLAGS.SHOW_CHAT_OPTIONS ))

    return minigame:GetResult()
end

function ConversationScreen:HideBattlePreviewPanel()
    if self.battle_preview_panel then
        self.battle_preview_panel:FadeOut()

        self.bubbles:AlphaTo( 1, 0.1, easing.outQuad )

        AUDIO:PlayEvent( "event:/ui/main/cnv/inspect_menu_close" )
        TheGame:FE():RefreshControlOptions()
    end
    return self
end

function ConversationScreen:HideNegotiationPreviewPanel()
    if self.negotiation_preview_panel then
        self.negotiation_preview_panel:FadeOut()        
        self.bubbles:AlphaTo( 1, 0.1, easing.outQuad )

        AUDIO:PlayEvent( "event:/ui/main/cnv/inspect_menu_close" )
        TheGame:FE():RefreshControlOptions()

    end
    return self
end

function ConversationScreen:IsNegotiationPreviewPanelShowing()
    return self.negotiation_preview_panel and self.negotiation_preview_panel:IsFadedIn()
end

function ConversationScreen:IsBattlePreviewPanelShowing()
    return self.battle_preview_panel and self.battle_preview_panel:IsFadedIn()
end


function ConversationScreen:PresentBattlePreviewPanel( scenario, fn, focused_mode )
    if scenario == nil then
        self:HideBattlePreviewPanel()
        return
    end

    self:HideNegotiationPreviewPanel()

    self:SetAutoSkip( false )
    self:SetReticuleMode( false )
    
    if self.battle_preview_panel == nil then
        self.battle_preview_panel = Widget.NegotiationPreviewPanel( QCARD_TYPE.BATTLE ):SetAnchors("center","center" )
        self:AddChild( self.battle_preview_panel )
        self.battle_preview_panel:Hide()
    end
    self.battle_preview_panel:Refresh( scenario )
    self.battle_preview_panel:SetPos( 0, self:CalculatePreviewPanelVerticalPosition() )
    self.battle_preview_panel:FadeIn()

    self.bubbles:AlphaTo( 0, 0.1, easing.outQuad )

    AUDIO:PlayEvent( "event:/ui/main/cnv/inspect_menu_open" )
    
    self.battle_preview_panel:SetFn( fn )

    local main_overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    main_overlay:FadeOutBars()

    TheGame:FE():RefreshControlOptions()

    return self.battle_preview_panel
end

function ConversationScreen:BattlePreviewPanelLoseHover()
    if self.battle_preview_panel then
        self:HideBattlePreviewPanel()

        local main_overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
        main_overlay:FadeInBars()
    end
    return self
end

function ConversationScreen:PresentNegotiationPreviewPanel( scenario, fn, focused_mode )
    if scenario == nil then
        self:HideNegotiationPreviewPanel()
        return
    end

    self:HideBattlePreviewPanel()

    self:SetAutoSkip( false )
    self:SetReticuleMode( false )

    if self.negotiation_preview_panel == nil then
        self.negotiation_preview_panel = Widget.NegotiationPreviewPanel( QCARD_TYPE.NEGOTIATION ):SetAnchors("center","center" )
        self:AddChild( self.negotiation_preview_panel )
        self.negotiation_preview_panel:Hide()
    end
    self.negotiation_preview_panel:Refresh( scenario )
    self.negotiation_preview_panel:SetPos( 0, self:CalculatePreviewPanelVerticalPosition() )
    self.negotiation_preview_panel:FadeIn()

    self.bubbles:AlphaTo( 0, 0.1, easing.outQuad )

    AUDIO:PlayEvent( "event:/ui/main/cnv/inspect_menu_open" )
    
    self.negotiation_preview_panel:SetFn( fn )

    local main_overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    main_overlay:FadeOutBars()

    TheGame:FE():RefreshControlOptions()

    return self.negotiation_preview_panel
end

-- Calculates how much the Preview Panel should move up based on how many buttons are in the multichoice
function ConversationScreen:CalculatePreviewPanelVerticalPosition()

    local min_multi_height = 36 -- A single button height
    local max_multi_height = 358 -- Height of a multichoice with 8 buttons

    local min_vertical_offset = 60 -- Centered vertically on the screen
    local max_vertical_offset = 220 -- High up in the clouds. Not really

    local vertical_offset = remap( self.multi:GetHeight(), min_multi_height, max_multi_height, min_vertical_offset, max_vertical_offset )

    return vertical_offset
end

function ConversationScreen:NegotiationPreviewPanelLoseHover()
    if self.negotiation_preview_panel then
        self:HideNegotiationPreviewPanel()

        local main_overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
        main_overlay:FadeInBars()
    end
    return self
end

function ConversationScreen:HidePreviewPanels()
    self:HideNegotiationPreviewPanel()
    self:HideBattlePreviewPanel()

    local main_overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    main_overlay:FadeInBars()

    return self
end

function ConversationScreen:SetAutoSkip( autoskip )
    if autoskip == true then
        self.autoskip = true
    else
        self.autoskip = false
    end
    return self
end

function ConversationScreen:IsAutoSkip()
    return (self.autoskip or GetIsAutoSkip()) and self.negotiation_panel == nil
end


function ConversationScreen:PresentOptions( options, options_type, on_select )
    self.encounter:ShowRelationshipChanges()

    AUDIO:PlayEvent(SoundEvents.dialogue_open)

    if self:IsAutoSkip() then
        -- Play the last event, etc.
        if self.last_speech then
            self:DoCharacterSpeech( self.last_speech.who, self.last_speech.txt )
            self.last_speech = nil
        end
        if self.last_emote then
            self.stage:Emote( self.last_emote.who, self.last_emote.emote )
            self.last_emote = nil
        end
    end

    self:SetAutoSkip( false )

    self:ClearOptions()
    
    self.waiting_for_skip = false


    for i, opt in ipairs( options ) do
        local opt_widget = opt:CreateWidget( self.encounter, self, self.multi, i )
        self:AddRobotOption(opt_widget)
    end

    self.multi:LayoutOptions()
    self.multi:SetShown( self:GetFE():IsHUDVisible( HUD_FLAGS.SHOW_CHAT_OPTIONS ))
    self.scroll_panel:SetScrollPercent( 1, true ) -- Scroll to end
    -- Focus on the first one
    if TheGame:FE():IsGamepadMode() then
        local widget = self.multi:GetOption(1)
        widget:DoTaskInTime( "focus_on_first_convo_option", 0.4, 
            function()
                -- If nothing is focused yet
                if not self.multi.focus and widget:IsEnabled() then
                    widget:SetFocus()
                end
            end
        )
    end


    if not TheGame:GetLocalSettings().ROBOTICS then
        --self.multi:AnimateIn()
    end

    self.on_select_option = on_select
    self.choice_timer = 0

    return self
end


function ConversationScreen:OnGainTop()
    ConversationScreen._base.OnGainTop(self)

    self.interface:Show()

    local location_screen = TheGame:FE():FindScreen( Screen.LocationScreen )
    if location_screen and location_screen.needs_refresh then location_screen:Refresh() end
end

function ConversationScreen:OnControlModeChange( cm, device_id )
    ConversationScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.next:SetText( LOC"UI.CONVERSATION_SCREEN.CLICK_TO_CONTINUE", loc.format(LOC"UI.CONVERSATION_SCREEN.PRESS_TO_SKIP", Controls.Digital.SKIP, Controls.Digital.MENU_ALT ):upper() )
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.next:SetText( loc.format( LOC"UI.CONVERSATION_SCREEN.CLICK_TO_CONTINUE_GAMEPAD", Controls.Digital.NEXT_LINE ), loc.format(LOC"UI.CONVERSATION_SCREEN.PRESS_TO_SKIP_GAMEPAD", Controls.Digital.SKIP ):upper() )
    end
    if cm == CONTROL_MODE.TOUCH then
        self.next:SetText( LOC"UI.CONVERSATION_SCREEN.CLICK_TO_CONTINUE_TOUCH" )
    end

    self.scroll_panel:SetVirtualBottomMargin( cm == CONTROL_MODE.GAMEPAD and BOTTOM_MARGIN + CONTROL_BAR_OFFSET or BOTTOM_MARGIN )

    self:LayoutBubbles()

    if cm == CONTROL_MODE.MOUSE_KEYBOARD or cm == CONTROL_MODE.TOUCH then
        self:SetReticuleMode( false )
    end
end

local TEXT_LAYOUT_SCALE =
{
    [ SCREEN_MODE.MONITOR ] = 1.0,
    [ SCREEN_MODE.TV ] = 1.5,
    [ SCREEN_MODE.SMALL ] = 1.5
}

local OPTION_LAYOUT_SCALE =
{
    [SCREEN_MODE.MONITOR] = 1,
    [SCREEN_MODE.TV] = 1.4,
    [SCREEN_MODE.SMALL] = 1.4,
}

function ConversationScreen:OnScreenModeChange( sm )
    ConversationScreen._base.OnScreenModeChange( self, sm )

    local text_bubble_scale = self:GetTextBubbleScale()
    for i, w in self.bubbles:BubbleWidgets() do
        w:SetLayoutScale( text_bubble_scale )
    end

    self.next:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.multi:SetLayoutScale( OPTION_LAYOUT_SCALE[sm] )
    self:LayoutBubbles()

    -- Scale portraits
    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:Layout()
end

function ConversationScreen:GetTextBubbleScale()
    local sm = TheGame:FE():GetScreenMode()
    return TEXT_LAYOUT_SCALE[ sm ] or 1.0
end

function ConversationScreen:OnScreenResize(w,h)
    ConversationScreen._base.OnScreenResize(self,w,h)

    self:Layout()
    self:LayoutBubbles()
end

function ConversationScreen:Layout()
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Resize the scroll panel to fill the screen
    self.scroll_panel:SetSize( self.screen_w, self.screen_h )

    return self
end

function ConversationScreen:LayoutBubbles()

    -- Position the convo options at the bottom
    local offset = TheGame:FE():IsGamepadMode() and BOTTOM_MARGIN + CONTROL_BAR_OFFSET or BOTTOM_MARGIN
    self.multi:SetPos( self.screen_w/2, offset )

    -- Y position for the bubbles to appear at
    local target_y = self.screen_h*0.6

    self.bubbles:SetPos( self.screen_w/2, nil )
        :LayoutBounds( nil, "above", 0, target_y )

    if self.negotiation_panel then
        self.scroll_panel:SetVirtualTopMargin( NEGOTIATION_TOP_MARGIN )
    else
        self.scroll_panel:SetVirtualTopMargin( TOP_MARGIN )
    end

    self.scroll_panel:RefreshView()
    return self
end

function ConversationScreen:GetDefaultFocus()
    if self.negotiation_panel then
        return self.negotiation_panel:GetDefaultFocus()
    end
    local best_opt
    for k, opt in ipairs( self.multi:GetOptions() ) do
        if opt:HasFocus() then
            return opt
        elseif opt:IsShown() and opt.enabled and best_opt == nil then 
            best_opt = opt
        end
    end
    return best_opt
end

function ConversationScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 380, 0 )
    elseif GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -380, 0 )
    end
    return self
end

function ConversationScreen:OnLoseTop()
    ConversationScreen._base.OnLoseTop(self)
    self.interface:Hide()
end

function ConversationScreen:IsCharacterLeft(character)
    return self.stage:IsCharacterLeft(character)
end

function ConversationScreen:IsCharacterRight(character)
    return self.stage:IsCharacterRight(character)
end

function ConversationScreen:ZoomIn()
    self.stage:SetZoom( 1.3, 50 )
    return self
end

function ConversationScreen:ZoomOut()
    self.stage:SetZoom()
    return self
end

function ConversationScreen:DoBloomBurst( magnitude, duration )
    if TheGame:ScreenFlashEnabled() then
        self.bloom_burst_mag = magnitude
        self.bloom_burst_duration = duration
        self.bloom_burst_t = 0
    end
    
    return self
end

function ConversationScreen:OnUpdate(dt)
    ConversationScreen._base.OnUpdate( self, dt )

    self.choice_timer = self.choice_timer + dt
    
    if self.encounter then
        self.encounter:OnUpdate( dt )

        if not self.encounter:IsRunning() then
            self:FinishConvo()
        end

        -- Who knows what callbacks FinishConvo() may trigger.
        if not self:IsOnStack() then
            return
        end
    end

    if not self.stage:IsSettled() and self.choice_timer > 5 then
        self.stage:Center()
    end

    if self.bloom_burst_t then
        self.bloom_burst_t = self.bloom_burst_t + dt
        local t = math.sin( self.bloom_burst_t * (math.pi / self.bloom_burst_duration ))
        -- lerp from 1.0 -> bloom_burst_mag
        local bloom = (1.0 - t) * 1.0 + t * self.bloom_burst_mag
        self.plax:SetBloom( bloom )

        if self.bloom_burst_t >= self.bloom_burst_duration then
            self.bloom_burst_duration = nil
            self.bloom_burst_mag = nil
            self.bloom_burst_t = nil
        end
    end

    if self.shaker then
        if self.shaker:IsDone() then
            self.shakex, self.shakey = nil, nil
            self.shaker = nil
        else
            self.shakex, self.shakey = self.shaker:UpdateShake( dt )
        end
        self.plax:SetPlaxShift( self.shakex or 0, self.shakey or 0 )
        self.stage.plax:SetPlaxShift( (self.shakex or 0) * NEGOTIATION_CHARACTER_SHAKE, (self.shakey or 0) * NEGOTIATION_CHARACTER_SHAKE )
    end

    if self.show_skip_in_time then
        self.show_skip_in_time = self.show_skip_in_time - dt
        if self.show_skip_in_time <= 0 then
            self.show_skip_in_time = nil
            self.next:SetTintColour(1,1,1,0)
            self.next:Show()
        end
    end

    if self.next:IsShown() then
        -- Manual control over easing to target alpha, because it can change if the user scrolls.
        local p = self.bubbles:GetScrollPercent()
        local target_fade = p < 1.0 and 0 or 1.0
        local alpha = self.next:GetAlpha()
        local delta = (target_fade - alpha) * dt * 10
        self.next:SetTintAlpha( alpha + delta )
    end

    if TheGame:FE():IsGamepadMode() and not self.negotiation_panel then
        if (self.negotiation_preview_panel and self.negotiation_preview_panel:IsFadedIn()) or
            (self.battle_preview_panel and self.battle_preview_panel:IsFadedIn()) then
            -- No reticule
        else
            local lx,ly,rx,ry = TheGame:GetInput():GetActiveGamepadStickVals()
            if math.length2d( rx, ry ) > 0.5 then
                self:SetReticuleMode( true )
            elseif math.length2d( lx, ly ) > 0.5 then
                self:SetReticuleMode( false )
            end
        end
    end

    if TheGame:GetLocalSettings().ROBOTICS and not self.negotiation_panel then
        if self.encounter and self.waiting_for_skip then
            self.wait_for_skip_time = self.wait_for_skip_time or 0
            self.wait_for_skip_time = self.wait_for_skip_time + dt
            if self.wait_for_skip_time > 0.5 then
                self:NextLine()
                self.wait_for_skip_time = nil
            end
        end
    end

end

function ConversationScreen:StopTalkingAnimation()
    self.stage:StopAllTalking()
    return self
end

function ConversationScreen:AddCharacter( agent, facing_right )
    assert( agent )
    self.plax:DoBlurEffect()

    if self.plax then
        local slot = self.plax:GetAgentSlot( agent )
        if slot then
            self.plax:ClearSlot( slot.slot_id )
        end
    end

    local prev_agent = self.stage:GetCharacter( facing_right )
    if prev_agent then
        -- Remove tails from any bubbles prev_agent may have spoke.
        for i, bubble in self.bubbles:BubbleWidgets() do
            if is_instance( bubble, Widget.TextBox ) and bubble:GetSpeaker() == prev_agent then
                bubble:SetSpeechBubble( prev_agent, bubble.STYLE.SPEECH )
            end
        end
    end

    local animated_character = self.stage:AddCharacter( agent, facing_right )

    self.stage:SetPoseDef(agent, "conversation")

    if animated_character and agent:HasAspect("drunk") then
        animated_character:SetMood( MOOD.NEUTRAL )
        animated_character:SetMood( MOOD.DRUNK )
    end
    
    -- Add tails to the bubbles this agent may have spoke.
    for i, bubble in self.bubbles:BubbleWidgets() do
        if is_instance( bubble, Widget.TextBox ) and bubble:GetSpeaker() == agent then
            bubble:SetSpeechBubble( agent, facing_right and bubble.STYLE.SPEECH_LEFT or bubble.STYLE.SPEECH_RIGHT )
        end
    end

    if facing_right then
        self:SetLeftPortrait(agent)
    else
        self:SetRightPortrait(agent)
    end

    if agent.theme_music and not self.negotiation_panel then
        TheGame:GetMusic():PlayCharacterMusic(agent.theme_music)
        self.character_music = agent.theme_music
    end

    return self
end


function ConversationScreen:RemoveCharacter( agent, is_left )
    self:WaitOnLine()

    agent = agent or self.stage:GetCharacter( is_left )

    if agent then
        -- Remove tails from the bubbles this agent may have spoke.
        for i, bubble in self.bubbles:BubbleWidgets() do
            if is_instance( bubble, Widget.TextBox ) and bubble:GetSpeaker() == agent then
                bubble:SetSpeechBubble( agent, bubble.STYLE.SPEECH )
            end
        end

        self.stage:RemoveCharacter( agent )
    end

    if is_left then
        self.left_character_portrait:Hide()
    else
        self.right_character_portrait:Hide()
    end

    --[[if agent.theme_music and self.encounter:GetMusicEvent() == agent.theme_music and (agent:IsRetired() or agent:InLimbo()) then
        self.encounter:SetMusicEvent(nil)
    end--]]

    return self
end

function ConversationScreen:GetCharacterStage()
    return self.stage
end

function ConversationScreen:GetCharacterWidget( agent )
    local slot = self.stage.characters[ agent ]
    if slot then
        return slot.body
    end
end

function ConversationScreen:WaitOnLine( force_wait )
    if self.negotiation_panel then
        if self.waiting_for_skip then
            self.negotiation_panel:Delay( 0.5 )
        end

    elseif self.waiting_for_skip and (force_wait or not self:IsAutoSkip()) then
        if force_wait then
            self.show_skip_in_time = 0
        else
            if TheGame:GetGameProfile():HasSeenMessage('click_to_continue') then
                self.show_skip_in_time = 20
            else
                TheGame:GetGameProfile():SetHasSeenMessage('click_to_continue')
                self.show_skip_in_time = 3
            end
        end
        self.encounter:YieldEncounter()
    end
end

function ConversationScreen:ForceWaitOnLine()
    self:SetAutoSkip( false )
    self:WaitOnLine( true )
end

function ConversationScreen:NextLine()
    if self.waiting_for_skip and not self.scroll_panel:IsDragging() then
        self.waiting_for_skip = false
        AUDIO:StopSpeech()
        self:StopTalkingAnimation()
        self.next:Hide()
        self.show_skip_in_time = nil

        if self.encounter:IsRunning() then
            self.encounter:ResumeEncounter()
        else
            self:WaitOnLine()
        end
    end
    return self
end

function ConversationScreen:OnSelectOption( idx )
    self:StopTalkingAnimation()
    AUDIO:StopSpeech()
    self:ClearOptions()
    self:HidePreviewPanels()
    self.waiting_for_skip = nil

    if self.on_select_option then
        self.on_select_option( idx )
    end
end

function ConversationScreen:Refresh()
    self:StopTalkingAnimation()
    AUDIO:StopSpeech()

    self:SetLeftPortrait(self.encounter:GetLeftAgent())
    self:SetRightPortrait(self.encounter:GetRightAgent())

    return self
end

function ConversationScreen:DoCharacterSpeech( character, txt )
    local is_question = string.find(txt, "?") ~= nil
    self.stage:DoCharacterSpeech(character, is_question, VOICE_DELAY)
    return self
end

function ConversationScreen:AddSpeech(character, txt)
    txt = Content.SubstituteCodexFormatting( txt )
    if TheGame:FE():IsFading() then
        self.encounter:YieldEncounter( function() return not TheGame:FE():IsFading() end)
    end

    self:WaitOnLine()

    self:SnapSpool()
    
    if self.last_speaker ~= character then
        self.last_speaker = character
    end

    local left = self:IsCharacterLeft(character)
    local right = self:IsCharacterRight( character )

    if not left and not right then
        txt = loc.format( "({1}): ", character:GetName() ) .. txt -- Offstage.
    end

    local widget =  Widget.TextBox( txt, "speech" )
    widget:SetText(txt)
    widget:SetMaxWidth(BUBBLE_WIDTH - 50):SetPadding( 0 )
    widget:GetLabel():SetHoverLinkFn( TextBubbleTooltips, self.encounter.active_hub )
    widget:GetLabel():SetClickLinkFn( TextBubbleClicked, self.encounter_active_hub )
    widget:GetLabel():SetGlyphColour(UICOLOURS.CONVO_SPEECH_TEXT)
    widget:SetSpeechBubble( character, left and widget.STYLE.SPEECH_LEFT or widget.STYLE.SPEECH_RIGHT )

    if self.negotiation_panel then
        widget:SetTintAlpha( 0 )
    else
        widget:Spool()
    end

    widget:SetLayoutScale( self:GetTextBubbleScale() )
    self.bubbles:AddWidget( widget )
    self:LayoutBubbles()
    self.scroll_panel:SetScrollPercent( 1, true ) -- Scroll to end

    if self.negotiation_panel then
        local tx, ty = widget:GetPos()
        widget:SetPos( left and -1000 or 1000, ty )
        widget:MoveTo( tx, ty, 0.2, easing.outQuad )
        widget:AlphaTo( 1.0, 0.2 )
    else
        local x1, y1, x2, y2 = widget:GetBoundingBox()
        if left then
            local dx = SPEECH_ALIGN_LEFT - x1
            widget:Offset( dx, 0 )
        else
            local dx = SPEECH_ALIGN_RIGHT - x2
            widget:Offset( dx, 0 )
        end
    end

    if txt ~= "..." then
        if self:IsAutoSkip() then
            self.last_speech = { who = character, txt = txt }
            if self.last_emote then
                self.last_emote.speech_count = self.last_emote.speech_count + 1
                if self.last_emote.speech_count > 2 then
                    self.last_emote = nil
                end
            end

        else
            self:DoCharacterSpeech( character, txt )
        end
    end
    
    self.waiting_for_skip = true
    return self
end

function ConversationScreen:ApplyColourCorrectionValues(v)
    self.stage.plax:ApplyColourCorrectionValues(v)
    return self
end


function ConversationScreen:GetCurrentColourName()
    return self.stage.plax:GetCurrentColourName()
end  

function ConversationScreen:SetVoice(who, voice)
    self.stage:SetVoice( who, voice )
    return self
end

function ConversationScreen:Emote(who, emote)
    if self:IsAutoSkip() then
        self.last_emote = { who = who, emote = emote, speech_count = 0 }
    else
        self.stage:Emote( who, emote )
    end
    return self
end

function ConversationScreen:ScreenShake( shake_params )
    if TheGame:CamShakeEnabled() then
        local mag, speed, time = table.unpack( shake_params )
        self.shaker = CamShake( mag, speed, time )
    end

    return self
end

function ConversationScreen:IsShaking()
    return self.shaker ~= nil
end

function ConversationScreen:HandleControlUp(control, device)
    
    if self.negotiation_panel then
        if self.negotiation_panel:HandleControlUp( control, device ) then
            return true
        end
    end

    if not self.show_reticule then
        if self.multi:HandleControlUp( control, device ) then
            return true
        end
    end
    
    return  ConversationScreen._base.HandleControlUp( self, control, device )
end

function ConversationScreen:HandleControlDown(control, device)
    if self.negotiation_panel then
        if self.negotiation_panel:HandleControlDown( control, device ) then
            return true
        end
    end

    if not self.show_reticule then
        if self.multi:HandleControlDown( control, device ) then
            return true
        end
    end

    return ConversationScreen._base.HandleControlDown( self, control, device )
end

function ConversationScreen:GetDebugEnv( debug_env )
    debug_env.encounter = self.encounter
    debug_env.location = self.encounter and self.encounter:GetLocation()
    debug_env.agent = self.encounter and self.encounter:GetAgent()

    if self.negotiation_panel then
        self.negotiation_panel:GetDebugEnv( debug_env )
    end
end

function ConversationScreen:RenderContextMenu( ui, panel )
    if self.negotiation_panel then
        self.negotiation_panel:RenderContextMenu( ui, panel )
    end
    return self
end

function ConversationScreen:RenderDebugUI( ui, panel )   
    local enc = self.encounter
    if self.interface.shown then
        if ui:Button( "Hide Interface" ) then
            self.interface:Hide()
        end
    else
        if ui:Button( "Show Interface" ) then
            self.interface:Show()
        end
    end
    if TheGame:FE().screen_tinted and TheGame:FE().screen_tinted.removed or TheGame:FE().screen_tinted == nil then
        if ui:Button( "Tint Background" ) then
            local screen = TheGame:FE():FindScreen( Screen.LocationScreen )
            if screen then
                local overlay = Widget.SolidBox( 6000, 6000, 0xFF00FFFF )
                screen:AddChild( overlay )
                overlay:SendToFront()
                TheGame:FE().screen_tinted = overlay
            end
        end
    else
        if ui:Button( "Remove Background Tint" ) then
            if TheGame:FE().screen_tinted then TheGame:FE().screen_tinted:Remove() end
            TheGame:FE().screen_tinted = nil
        end
    end

    ui:SameLine( nil, 10 )

    if enc then
        panel:AppendTable( ui, enc, "Encounter" )
        if enc.coro then
            ui:SameLine( nil, 10 )
            if ui:Button( "Coroutine" ) then
                TheGame:GetDebug():CreatePanel( DebugTraceback( enc.coro ))
            end
        end
        ui:SameLine( nil, 10 )
        panel:AppendTable( ui, Content.internal.QUIPS, "Quips" )
        if enc:GetLocation() then
            ui:SameLine( nil, 10 )
            panel:AppendTable( ui, enc:GetLocation() )
        end

        if self.negotiation_panel then
            self.negotiation_panel:RenderDebugUI( ui, panel )
        end


        if #enc.current_options > 0 and panel:CollapsingHeader( ui, "Options" ) then
            for i, option in ipairs( enc.current_options ) do
                local txt = SetFormattedText( nil, option.text )
                if ui:TreeNode( txt ) then
                    panel:AppendTable( ui, option )
                    panel:AppendKeyValues( ui, option )
                    ui:TreePop()
                end
            end
        end
    end

    ui:Separator()
    ui:Spacing()
    ui:Separator()

    return true
end

function ConversationScreen:SetLeftPortrait(agent)
    if agent and not agent:IsPlayer() then
        self.left_character_portrait:SetCharacter(agent):Show()
    else
        self.left_character_portrait:Hide()
    end
    return self
end

function ConversationScreen:SetRightPortrait(agent)
    if agent and not agent:IsPlayer() then
        self.right_character_portrait:SetCharacter(agent):Show()
    else
        self.right_character_portrait:Hide()
    end

    self.fe:RefreshControlOptions()

    return self
end

function ConversationScreen:ActivateFlourish( flourish_bar )
    if self.negotiation_panel then
        self.negotiation_panel:ActivateFlourish( flourish_bar )
    end
end

------------------------------------------------------------------------------------------
-- Shows an info panel when the player is inactive

local ClickToContinue = class( "Widget.ClickToContinue", Widget )

function ClickToContinue:init()
    ClickToContinue._base.init( self )

    self.bg = self:AddChild( Widget.Image( engine.asset.Texture( "large/conversation_banner.tex") ) )
        :SetHiddenBoundingBox( true )
        :Bloom( 0.05 )
        :SetSize( 650, 280 )
    self.title = self:AddChild( Widget.Label() )
        :SetText(loc.format(LOC"UI.CONVERSATION_SCREEN.CLICK_TO_CONTINUE"))
        :SetFont("title")
        :SetFontSize( FONT_SIZE.SCREEN_SUBTITLE )
        :SetGlyphColour(UICOLOURS.SUBTITLE)
        :SetPos( 0, 14 )
        :Bloom( 0.05 )
    self.subtitle = self:AddChild( Widget.Label() )
        :SetText(loc.format(LOC"UI.CONVERSATION_SCREEN.PRESS_TO_SKIP", Controls.Digital.SKIP, Controls.Digital.MENU_ALT ):upper())
        :SetFont("body")
        :SetFontSize( FONT_SIZE.BODY_TEXT )
        :SetGlyphColour(UICOLOURS.SUBTITLE)
        :LayoutBounds( "center", "below", self.title ):Offset( 0, SPACING.TITLE_DESC )
        :Bloom( 0.05 )
    self.arrow = self:AddChild( Widget.Image( engine.asset.Texture( "ui/ic_conversation_skip.tex") ) )
        :SetHiddenBoundingBox( true )
        :Bloom( 0.05 )
        :SetSize( 20, 20 )
        :SetPos( 0, -60 )
    local anim_x, anim_y = self.arrow:GetPos()
    local anim_x_end, anim_y_end = anim_x, anim_y - 5
    local pacing = 0.2
    self.next_arrow_updater = Updater.Loop{
        Updater.Ease2D( function(x,y) self.arrow:SetPos(x, y) end, anim_x, anim_y, anim_x_end, anim_y_end, pacing, easing.outQuad),
        Updater.Ease2D( function(x,y) self.arrow:SetPos(x, y) end, anim_x_end, anim_y_end, anim_x, anim_y, pacing, easing.outQuad),
        Updater.Ease2D( function(x,y) self.arrow:SetPos(x, y) end, anim_x, anim_y, anim_x_end, anim_y_end, pacing, easing.outQuad),
        Updater.Ease2D( function(x,y) self.arrow:SetPos(x, y) end, anim_x_end, anim_y_end, anim_x, anim_y, pacing, easing.outQuad),
        Updater.Wait( pacing*4 ),
        Updater.Ease2D( function(x,y) self.arrow:SetPos(x, y) end, anim_x, anim_y, anim_x_end, anim_y_end, pacing, easing.outQuad),
        Updater.Ease2D( function(x,y) self.arrow:SetPos(x, y) end, anim_x_end, anim_y_end, anim_x, anim_y, pacing, easing.outQuad),
        Updater.Ease2D( function(x,y) self.arrow:SetPos(x, y) end, anim_x, anim_y, anim_x_end, anim_y_end, pacing, easing.outQuad),
        Updater.Ease2D( function(x,y) self.arrow:SetPos(x, y) end, anim_x_end, anim_y_end, anim_x, anim_y, pacing, easing.outQuad),
        Updater.Wait( pacing*10 )
    }
end

function ClickToContinue:OnVizChange( viz )
    if viz then
        self.bg:Pulse( 0.4, 1.0, 0.003 )
        self.arrow:RunUpdater(self.next_arrow_updater)
        self.next_arrow_updater:Reset()
    else
        self.bg:StopPulse()
        self.arrow:StopUpdater(self.next_arrow_updater)
    end
    return self
end

function ClickToContinue:SetText( title, subtitle )
    self.title:SetText( title or "" )
        :SetPos( 0, subtitle and 14 or 1 )
    self.subtitle:SetText( subtitle or "" )
    return self
end
