local DebugUtil = require "debug/debug_util"
require "ui/widgets/screen"
require "ui/widgets/labelspinner"
require "ui/widgets/dropdown"
require "ui/widgets/checkbox"
require "ui/widgets/agentstatusportrait"
require "ui/screens/ConversationScreen"
require "encounter/encounter"
require "ui/screens/debugtoolsmenu"
local debug_menus = require "debug/debug_menus"

--------------------------------------------------------------------------------

local ConvoBuildDebug = class("Screen.ConvoBuildDebug", Widget.Screen)
ConvoBuildDebug.IS_EDITOR = true

local moods =
{
    {txt = "NEUTRAL", mood = MOOD.NEUTRAL},
    {txt = "HAPPY", mood = MOOD.HAPPY},
    {txt = "ANGRY", mood = MOOD.ANGRY},
    {txt = "SCARED", mood = MOOD.SCARED},
    {txt = "INJURED", mood = MOOD.INJURED},
    {txt = "DRUNK", mood = MOOD.DRUNK},
}

local NO_HEAD = ""

local SHAPES = 
{
    "A", "B", "C", "D", "E", "F", "G", "H", "X",
}

local BUILD_TESTER =
{
	base_def = "NPC_BASE",
	no_outfit_swap = true,
    skinnable = false,
    unique = true,
}

local pdef = require "content/character_pose_defs"
local poses = {}
for k,v in pairs(pdef) do table.insert(poses, k) end
table.sort(poses)

local all_emotes = require "content/convo_emotes"
local emotes = {}
for k,v in pairs(all_emotes) do table.insert(emotes, k) end
table.sort(emotes)

function ConvoBuildDebug:init() 
    ConvoBuildDebug._base.init(self)

    self.plax = Widget.Plax()
    self.solid_bg = Widget.SolidBox( RES_X, RES_Y, 0xFF0000FF ):Hide()
    self.puppet = Widget()
    self.left_portrait = Widget.AgentStatusPortrait(true)

    self:AddChildren{
        self.plax:SetAnchors("center", "center"),
        self.solid_bg:SetAnchors("fill", "fill"),
        Widget.CharacterStage():SetName("STAGE"):SetAnchors("center", "center"),

        self.puppet:SetName("PUPPET"):SetAnchors("center","center"):AddChildren{
            Widget.LabelSpinner():SetName("SELECTOR"):SetFn(function(idx) self.puppet_char_right = idx == 2 self:SelectCharacter() end):SetOptions{"LEFT", "RIGHT"},
            
            Widget():AddChildren
            {
                Widget.DropDown():SetName("MOODSEL"):SetOptions(moods):SetSize(150, 30),
                Widget.PanelButton():SetText("SET MOOD"):SetFn(function() self:SetMood() end):SetSize(150, 30),
            }:ArrangeChildrenRow(155),

            Widget():AddChildren
            {
                Widget.DropDown():SetName("POSESEL"):SetOptions(poses):SetSize(150, 30),
                Widget.PanelButton():SetText("SET POSE"):SetFn(function() self:SetPose() end):SetSize(150, 30),
            }:ArrangeChildrenRow(155),

            Widget():AddChildren
            {
                Widget.DropDown():SetName("EMOTESEL"):SetOptions(emotes):SetSize(150, 30),
                Widget.PanelButton():SetText("EMOTE!"):SetFn(function() self:DoEmote() end):SetSize(150, 30),
                Widget.PanelButton():SetText("TALK!"):SetFn(function() self:DoTalk() end):SetSize(150, 30),
            }:ArrangeChildrenRow(155),

            Widget():AddChildren{
                self.left_portrait,
            }:ArrangeChildrenRow(200)
        }:ArrangeChildrenColumn(54),

    }
    self.left_portrait:Offset(0, -100)


    -- Invert the stacking order of the dropdowns, so that their popup shows above the others
    for k, row in ipairs( self.puppet.children ) do
        row:SendToBack()
    end


    self.location_search = {}

    self:LoadSettings()

    self.solid_bg:SetColour( self.settings.bg_colour )
    self.solid_bg:SetShown( self.settings.bg_shown )

    if Content.GetCharacterDef( "BUILD_TESTER" ) == nil then
        local def = CharacterDef( "BUILD_TESTER", BUILD_TESTER )
        Content.AddCharacterDef( def )
        def:InheritBaseDef()
    end
    self.right_char = Agent("BUILD_TESTER")

    self.stage = self:FindChild("STAGE")
    
    self.character_skins_panel = DebugCharacterSkins()
    self:PopulateHeads()
    self:PopulateBuilds()
    self:PopulateAgents()

    self.voice_actors = {}
    self.samples_per_actor = {}
    self.sample_display_text_per_actor = {}
    for actor_id,emotes in pairs( AUDIO.speech_event_lookup ) do
        table.insert(self.voice_actors, actor_id)
        self.samples_per_actor[actor_id] = {}
        self.sample_display_text_per_actor[actor_id] = {}
        for k,v in pairs(emotes) do
            for k,v in pairs(v) do
                for k,v in pairs(v) do
                    table.insert(self.samples_per_actor[actor_id], v)
                end
            end
        end
        table.sort(self.samples_per_actor[actor_id], function(a,b) return a.event_name < b.event_name end)
        for k,v in ipairs(self.samples_per_actor[actor_id]) do
            table.insert(self.sample_display_text_per_actor[actor_id], v.event_name)
        end
    end
    
    table.sort(self.voice_actors)

    self:SetLocationID( self.settings.location_settings )

    self:SetCharacter( table.unpack( self.settings.left_char ))

    self:ApplyOverrides()

    self.build_queue = {}
end


function ConvoBuildDebug:OnAdded()
    AUDIO:FinalizeVoiceLoad()
    TheGame:GetDebug():RemoveBindingGroup( debug_menus.GAME_BINDINGS )
    TheGame:GetDebug():AddBindingGroup( debug_menus.TOOL_BINDINGS )
    TheGame:GetEvents():ListenForEvents( self, "character_skins_changed" )

    DBG( self )
end

function ConvoBuildDebug:OnRemoved()
    TheGame:GetEvents():RemoveListener( self )
    TheGame:GetDebug():RemoveBindingGroup( debug_menus.TOOL_BINDINGS )
    TheGame:GetDebug():AddBindingGroup( debug_menus.GAME_BINDINGS )
end

function ConvoBuildDebug:LoadSettings()
    self.settings = DebugUtil.GetLocalDebugData("CONVO_DEBUG") or {}

    if self.settings.left_char == nil then
        self.settings.left_char = { "SAL" }
    end
    if self.settings.location_settings == nil then
        self.settings.location_settings = {}
    end

    if self.settings.bg_colour == nil then
        self.settings.bg_colour = 0xFFFFFFFF
    end
end

function ConvoBuildDebug:SaveSettings()
    DebugUtil.SetLocalDebugData( "CONVO_DEBUG", self.settings )
    DebugUtil.SaveLocalDebugData()
end

function ConvoBuildDebug:OnGlobalEvent( event_name )
    if event_name == "character_skins_changed" then
        self:PopulateAgents()
    end
end

function ConvoBuildDebug:SetLocationID( location_settings )
    local location = Location( location_settings.location_id or "OUTDOORS" )

    self.plax:Clear()
    self.plax:SetPlaxData( location:GetPlax() )

    for slot_id, t in pairs( location_settings.plax_slots or table.empty ) do
        local id, skin_uuid = table.unpack( t )
        local skin_table = Content.GetCharacterSkin( skin_uuid )
        local agent = Agent( id, skin_table )
        self.plax:FillSlot( agent, slot_id )
    end

    if location_settings.day_phase then
        local idx = table.arrayfind( PHASE_MASK_STRINGS, location_settings.day_phase )
        if idx then
            self.plax:SetDayPhase( 1 << (idx - 1) )
        end
    end

    self.stage:Clear()

    if self.right_char then
        self.stage:AddCharacter(self.right_char, false)
        self.stage:SetPoseDef(self.right_char, "conversation")
        self.stage:SetMood(self.right_char, MOOD.NEUTRAL)
    end

    if self.left_char then
        self.stage:AddCharacter(self.left_char, true)    
        self.stage:SetPoseDef(self.left_char, "conversation")
        self.stage:SetMood(self.left_char, MOOD.NEUTRAL)
    end
end

function ConvoBuildDebug:SelectCharacter()
    if self.puppet_char_right then
        self.stage:Right()
    else
        self.stage:Left()
    end

end

function ConvoBuildDebug:DeltaHead(delta)
    local idx = self.head_idx
    local build = Content.GetCharacterBuild( self.builds[ self.build_idx ] )
    if build.gender and build.species ~= SPECIES.BEAST then
        idx = idx + delta
        
        while idx >= 1 and idx <= #self.heads do
            --print (idx, self.heads[idx].species, self.heads[idx].gender)
            local head = Content.GetCharacterHead(self.heads[idx])
            if head and head.species ~= SPECIES.BEAST and head.gender == build.gender then
                break
            end
            idx = idx + delta
        end

    else    
        idx = idx + delta
    end
    local head = self.heads[idx] and Content.GetCharacterHead(self.heads[idx])
    
    if head and head.species ~= SPECIES.BEAST and head.gender == build.gender then
        self:SetHeadOverrideIdx( idx )
    end
end   

function ConvoBuildDebug:DeltaBuild(delta)
    local idx = self.build_idx
    local head = Content.GetCharacterHead( self.heads[ self.head_idx ] )
    if head.gender and head.species ~= SPECIES.BEAST then
        idx = idx + delta
        
        while idx >= 1 and idx <= #self.builds do
            local build = Content.GetCharacterBuild(self.builds[idx])
            if build and build.species ~= SPECIES.BEAST and head.gender == build.gender then
                break
            end
            idx = idx + delta
        end

    else    
        idx = idx + delta
    end
    local build = self.builds[idx] and Content.GetCharacterBuild(self.builds[idx])
    
    if build and build.species ~= SPECIES.BEAST and build.gender == head.gender then
        self:SetBuildOverrideIdx( idx )
    end
end

function ConvoBuildDebug:SetPose()
    local char = self.puppet_char_right and self.right_char or self.left_char
    local pose = poses[self:FindChild("POSESEL"):GetIndex()]
    self.stage:SetPoseDef(char, pose)

end

function ConvoBuildDebug:SetMood()
    local char = self.puppet_char_right and self.right_char or self.left_char
    local mood = moods[self:FindChild("MOODSEL"):GetIndex()].mood
    self.stage:SetMood(char, mood)
end

function ConvoBuildDebug:DoEmote()
    local char = self.puppet_char_right and self.right_char or self.left_char

    self.stage:SetPoseDef( char, "conversation" )    
    self.stage:Emote(char, emotes[self:FindChild("EMOTESEL"):GetIndex()])
end

function ConvoBuildDebug:DoTalk()
    local char = self.puppet_char_right and self.right_char or self.left_char
    
    self.stage:DoCharacterSpeech(char, math.random() > .5)
end

function ConvoBuildDebug:StopTalk()
    local char = self.puppet_char_right and self.right_char or self.left_char
    self.stage:StopTalk(char)
end


function ConvoBuildDebug:RandomColour()
    local chargen = require "charactergen"

    self.right_char.hair_colour = nil
    self.right_char.skin_colour = nil
    self.right_char.eye_colour = nil

    chargen.ApplyColours(self.right_char)

    self.stage.characters[self.right_char].body:Refresh(self.right_char)

end

function ConvoBuildDebug:ApplyOverrides()
    
    self.right_char.build = self.builds[ self.build_idx ]
    self.right_char.head = self.heads[ self.head_idx ]
    if self.right_char.head == NO_HEAD then
        self.right_char.head = nil
    end
    if self.right_char.head then
        self.right_char.species = Content.GetCharacterHead(self.right_char.head).species or SPECIES.HUMAN
        self.right_char.gender = Content.GetCharacterHead(self.right_char.head).gender or GENDER.FEMALE
    else
        self.right_char.species = SPECIES.BEAST
    end

    self:RandomColour()
end

function ConvoBuildDebug:SetHeadOverrideIdx(idx)
    self.head_idx = idx

    if self.settings.head_idx ~= idx then
        self.settings.head_idx = idx
        self:SaveSettings()
    end

    self:ApplyOverrides()
end

function ConvoBuildDebug:SetBuildOverrideIdx(idx)
    self.build_idx = idx

    if self.settings.build_idx ~= idx then
        self.settings.build_idx = idx
        self:SaveSettings()
    end

    self:ApplyOverrides()
end

local function MakeHeadLabel( head_id, head )
    local animfile = head.anim:match("^anim/(.+)$")
    return string.format("%s (%s)", animfile, head_id)
end

local function SortByLabel( a, b )
    return a.txt < b.txt
end

function ConvoBuildDebug:PopulateHeads( filter_txt )
    local old_head = self.heads and self.heads[ self.head_idx ]
    local t = ObtainWorkTable()
    for k,v in pairs(Content.GetCharacterHeads()) do
        local txt = MakeHeadLabel( k, v )
        if filter_txt == nil or txt:match( filter_txt ) then
            table.insert( t, { txt = txt, build_id = k })
            if k == old_head then
                old_head = nil -- Including current head; can ignore now.
            end
        end
    end

    table.sort( t, SortByLabel )

    self.heads = { NO_HEAD }
    self.heads_by_label = { "<NO HEAD>" }
    for i, v in ipairs(t) do
        table.insert( self.heads, v.build_id )
        table.insert( self.heads_by_label, v.txt )
    end
    ReleaseWorkTable( t )

    self.head_idx = self.settings.head_idx or 1
    if old_head and old_head ~= NO_HEAD then
        local txt = MakeHeadLabel( old_head, Content.GetCharacterHead( old_head ))
        table.insert( self.heads, old_head )
        table.insert( self.heads_by_label, txt )
    end
end

local function MakeBuildLabel( build_id, build )
    local animfile = build.build:match("^anim/(.+)$")
    return string.format("%s (%s)", animfile, build_id )
end

function ConvoBuildDebug:PopulateBuilds( filter_txt )
    local old_build = self.builds and self.builds[ self.build_idx ]
    local t = ObtainWorkTable()
    for k,v in pairs(Content.GetAllCharacterBuilds()) do
        local txt = MakeBuildLabel( k, v )
        if filter_txt == nil or txt:match( filter_txt ) then
            table.insert( t, { txt = txt, build_id = k })
            if k == old_build then
                old_build = nil -- Including current build; can ignore now.
            end
        end
    end

    table.sort( t, SortByLabel )

    self.builds = {}
    self.builds_by_label = {}
    for i, v in ipairs(t) do
        table.insert( self.builds, v.build_id )
        table.insert( self.builds_by_label, v.txt )
    end
    ReleaseWorkTable( t )

    self.build_idx = self.settings.build_idx or 1
    if old_build then
        local txt = MakeBuildLabel( old_build, Content.GetCharacterBuild( old_build ))
        table.insert( self.builds, 1, old_build )
        table.insert( self.builds_by_label, 1, txt )
    end
end

function ConvoBuildDebug:SetCharacter(content_id, skin_index)
    if Content.GetCharacterDef( content_id ) == nil then
        content_id = "SAL"
    end

    local skins = Content.GetAllCharacterSkins()[ content_id ]
    local skin_table
    if skins and #skins > 0 then
        skin_index = skin_index or 1
        skin_table = skins[ skin_index ]
        self.current_skin_index = skin_index
    end

    local agent = Agent( content_id, skin_table)
    self.left_char = agent
    self.stage:AddCharacter(agent, true)
    self.stage:SetPoseDef(agent, "conversation")
    self.stage:SetMood(agent, MOOD.NEUTRAL)

    self.settings.left_char = { content_id, skin_index } 

    self.left_portrait:SetCharacter(agent)

    self:SaveSettings()
end

function ConvoBuildDebug:PopulateAgents()
    if self.agent_defs == nil then
        self.agent_defs = {}
    else
        table.clear( self.agent_defs )
    end

    self.beasts = {}
    for content_id, v in pairs(Content.GetAllCharacterDefs()) do
        if not v.is_template and (v.species == SPECIES.BEAST or v.species == SPECIES.MECH) then
            table.insert(self.beasts, content_id)
        end
    end
    table.sort(self.beasts)


    if self.aliased_agents == nil then
        self.aliased_agents = {}
    else
        table.clear( self.aliased_agents )
    end

    for content_id, v in pairs(Content.GetAllCharacterDefs()) do
        if v.alias then
            table.insert( self.aliased_agents, content_id )
        elseif v.skinnable and not v.is_template then
            table.insert( self.agent_defs, content_id )
        end
    end
    table.sort( self.aliased_agents )
    table.sort( self.agent_defs )
end

function ConvoBuildDebug:ToggleHUD( visible, hud_flags )
    self.puppet:SetShown( visible )
end

function ConvoBuildDebug:HandleRawKeyDown( key )
    if key == "E" then
        self:DoEmote()
        return true
    elseif key == "T" and not TheGame:GetInput():IsModifierAny() then
        self:DoTalk()
        return true
    end
end


function ConvoBuildDebug:HandleControlDown(control)
    if control:Has( Controls.Digital.MENU_ACCEPT ) then
        local w, h = engine.inst:GetScreenSize()
        local x, y = TheGame:GetInput():GetMousePos()
        if x > w*.66 or x < w*.33 then
            self:FindChild("SELECTOR"):SetIndex(x > w/2 and 2 or 1)
        end
        return true

    elseif control:Has( Controls.Digital.MENU_UP ) then
        self.stage:Center()
        return true

    elseif control:Has( Controls.Digital.MENU_LEFT ) then
        self.stage:Left(self.left_char)
        return true

    elseif control:Has( Controls.Digital.MENU_RIGHT ) then
        self.stage:Right(self.right_char)
        return true

    elseif control:Has( Controls.Digital.EDITOR_UNDO ) then
        if self.character_skins_panel:CanUndo() then
            self.character_skins_panel:Undo()
        end
        return true

    elseif control:Has( Controls.Digital.EDITOR_REDO ) then
        if self.character_skins_panel:CanRedo() then
            self.character_skins_panel:Redo()
        end
        return true
    end

    return false
end



function ConvoBuildDebug:GetDebugEnv( debug_env )
    debug_env.right = self.right_char
    debug_env.left = self.left_char
end

function ConvoBuildDebug:DoRender()
    if #self.build_queue > 0 then
        local task = table.remove(self.build_queue)
        AUDIO:RegenVoiceData( task.actor, task.event_name )
    end
    self.stage.plax:RenderToScene()
end


function ConvoBuildDebug:RenderDebugUI( ui, panel )
    local toggle_bg = ui:Checkbox( "Solid Background", self.solid_bg:IsShown() )
    if toggle_bg then
        self.solid_bg:SetShown( not self.solid_bg:IsShown() )
        self.settings.bg_shown = self.solid_bg:IsShown()
        self:SaveSettings()
    end

    ui:SameLine( nil, 10 )
    local new_clr = ui:ColorEdit3( "Current", self.settings.bg_colour or 0 )
    if new_clr and new_clr ~= self.settings.bg_colour then
        self.settings.bg_colour = new_clr
        self.solid_bg:SetColour( new_clr )
        self:SaveSettings()
    end

    if DebugUtil.RenderLocationSettings( self.settings.location_settings, self.plax, ui, panel ) then
        self:SetLocationID( self.settings.location_settings )
        self:SaveSettings()
    end

    ui:Separator()

    if ui:CollapsingHeader( "Speech" ) then
        self:RenderSpeechPanel(ui)
    end

    self:RenderBottomRightPanel( ui )
    self:RenderBottomLeftPanel( ui )

    return false
end



function ConvoBuildDebug:RenderSpeechPanel(ui)
    if ui:Button("Play") then
        local actor_id = self.voice_actors[self.voice_actor_idx]
        if self.samples_per_actor[actor_id] then
            local info = self.samples_per_actor[actor_id][self.voice_sample_idx or 1]
            AUDIO:PlayEvent(info.event_name, "sound/vo.bank")     
            local phonemes = AUDIO:GetPhonemes(info.id)
            if phonemes then
                self.stage:GetLeftAnim():QueueLipsynch(phonemes)
                self.stage:GetRightAnim():QueueLipsynch(phonemes)
            end
        end
    end

    if ui:Button("Build Meta") then
        AUDIO:SaveSpeechMeta()
    end

    if ui:Button("Rebuild") and #self.build_queue == 0 then
        local actor_id = self.voice_actors[self.voice_actor_idx]
        if self.sample_display_text_per_actor[actor_id] then
            local info = self.samples_per_actor[actor_id][self.voice_sample_idx or 1]
            if info and info.event_name then
                table.insert(self.build_queue, {actor = actor_id, event_name = info.event_name})
                --AUDIO:RegenVoiceData(info.event_name)
            end
        end
        --os.execute("")
    end

    local actor_id = self.voice_actors[self.voice_actor_idx]        

    local num_missing = 0
    if self.samples_per_actor[actor_id] then
        for k,info in ipairs(self.samples_per_actor[actor_id]) do
            local phonemes = AUDIO:GetPhonemes(info.id)
            if not phonemes then
                num_missing = num_missing + 1
            end
        end
    end

    if num_missing > 0 then
        if ui:Button(loc.format( "Build {1} Missing", num_missing)) and #self.build_queue == 0 then
            
            if self.sample_display_text_per_actor[actor_id] then
                for k,info in ipairs(self.samples_per_actor[actor_id]) do
                    if info and info.event_name and not AUDIO:GetPhonemes(info.id) then
                        table.insert(self.build_queue, {actor = actor_id, event_name = info.event_name})
                    end
                end
            end
            --os.execute("")
        end
    end

    if ui:Button( "<###downactor", 20 ) then
        if self.voice_actor_idx == 1 then
            self.voice_actor_idx = #self.voice_actors
        else
            self.voice_actor_idx = self.voice_actor_idx - 1
        end
    end

    ui:SameLine( nil, 5 )
    if ui:Button( ">###upactor", 20 ) then
        if self.voice_actor_idx == #self.voice_actors then
            self.voice_actor_idx = 1
        else
            self.voice_actor_idx = self.voice_actor_idx + 1
        end
    end
    ui:SameLine( nil, 5 )

    local actor_idx = ui:Combo( "Voice Actor", self.voice_actor_idx or 1, self.voice_actors )
    if actor_idx and actor_idx ~= self.voice_actor_idx then
        self.voice_actor_idx = actor_idx
        self.voice_sample_idx = 1
    end
    local actor_id = self.voice_actors[self.voice_actor_idx]

    if actor_id then

        if ui:Button( "<###downsample", 20 ) then
            if self.voice_sample_idx == 1 then
                self.voice_sample_idx = #self.sample_display_text_per_actor[actor_id]
            else
                self.voice_sample_idx = (self.voice_sample_idx or 1) - 1
            end
        end

        ui:SameLine( nil, 5 )
        if ui:Button( ">###upsample", 20 ) then
            if self.voice_sample_idx == #self.sample_display_text_per_actor[actor_id] then
                self.voice_sample_idx = 1
            else
                self.voice_sample_idx = (self.voice_sample_idx or 1) + 1

            end
        end
        ui:SameLine( nil, 5 )


        local sample_idx = ui:Combo( "Voice Sample", self.voice_sample_idx or 1, self.sample_display_text_per_actor[actor_id] )
        if sample_idx and sample_idx ~= self.voice_sample_idx then
            self.voice_sample_idx = sample_idx
        end
        
    end
end


function ConvoBuildDebug:RenderBottomRightPanel( ui )
    local w, h = ui:DisplaySize()
    ui:SetNextWindowSize( w * 0.45, h )
    ui:SetNextWindowPos( w - w * 0.45, h - 120 )

    local show = ui:Begin( "RIGHT TESTER", ui.NoTitleBar | ui.NoMove )
    if show then
        if ui:Button( "Randomize Colour" ) then
            self:RandomColour()
        end
        ui:SameLine( nil, 10 )
        if ui:Button( string.format( "Filter: %s", self.filter_txt or "none" )) then
            ui:OpenPopup( "SET_FILTER" )
        end
        if ui:BeginPopup( "SET_FILTER" ) then
            local populate = false
            local filter_txt = ui:InputText( "Filter", self.filter_txt, ui.TextFlags_EnterReturnsTrue )
            if filter_txt and filter_txt ~= self.filter_txt then
                populate = true
            end
            if ui:Button( "Clear" ) then
                filter_txt = nil
                populate = true
            end
            if populate then
                self.filter_txt = filter_txt
                self:PopulateBuilds( self.filter_txt )
                self:PopulateHeads( self.filter_txt )
                ui:CloseCurrentPopup()
            end

            ui:EndPopup()
        end

        ui:Text( string.format( "%d builds, %d heads", #self.builds, #self.heads ))
        ui:Spacing()

        ui:PushStyleColor( ui.Style_Button, 0x337733FF )
        if ui:Button( "<##prev_head" ) then
            self:DeltaHead( -1 )
        end
        ui:SameLine( nil, 5 )
        if ui:Button( ">##next_head" ) then
            self:DeltaHead( 1 )
        end
        ui:PopStyleColor()

        ui:SameLine( nil, 10 )
        local head_idx = ui:Combo( "Heads", self.head_idx, self.heads_by_label )
        if head_idx and head_idx ~= self.head_idx then
            self:SetHeadOverrideIdx( head_idx )
        end

        ui:SameLine( nil, 10 )
        if ui:SmallButton( "Clear" ) then
            self:SetHeadOverrideIdx( 1 ) -- No head idx
        end

        ui:PushStyleColor( ui.Style_Button, 0x337733FF )
        if ui:Button( "<##prev_build" ) then
            self:DeltaBuild( -1 )
        end
        ui:SameLine( nil, 5 )
        if ui:Button( ">##next_build" ) then
            self:DeltaBuild( 1 )
        end
        ui:PopStyleColor()

        ui:SameLine( nil, 10 )
        local build_idx = ui:Combo( "Builds", self.build_idx, self.builds_by_label )
        if build_idx and build_idx ~= self.build_idx then
            self:SetBuildOverrideIdx( build_idx )
        end

        ui:End()
    end
end

local function IsValidSkinName( name )
    if not name or #name == 0 then
        return false
    end

    for id, skins in pairs( Content.GetAllCharacterSkins() ) do
        for i, skin in ipairs(skins) do
            if skin.name == name then
                return false
            end
        end
    end
    return true
end

function ConvoBuildDebug:RenderBottomLeftPanel( ui )
    local w, h = ui:DisplaySize()
    ui:SetNextWindowSize( w * 0.5, h )
    ui:SetNextWindowPos( 0, h - (self.left_panel_height or 150) )

    local show = ui:Begin( "LEFT AGENT", ui.NoTitleBar | ui.NoMove )
    if show then
        if ui:Button( "Choose..." ) then
            ui:OpenPopup( "AGENT_LIST" )
        end
        if ui:BeginPopup( "AGENT_LIST" ) then
            if ui:BeginMenu( "Special Agents" ) then
                for i, content_id in ipairs( self.aliased_agents ) do
                    if ui:MenuItem( content_id ) then
                        self:SetCharacter( content_id )
                    end
                end
                ui:EndMenu()
            end
            if ui:BeginMenu( "Beasts" ) then
                for i, content_id in ipairs( self.beasts ) do
                    if ui:MenuItem( content_id ) then
                        self:SetCharacter( content_id )
                    end
                end
                ui:EndMenu()
            end

             for i, content_id in ipairs( self.agent_defs ) do
                if ui:BeginMenu( content_id ) then
                    local skins = Content.GetAllCharacterSkins()[ content_id ]
                    if skins == nil or #skins == 0 then
                        if ui:MenuItem( "Create New Skin" ) then
                            local skin_index = Content.CreateCharacterSkin( content_id )
                            Content.SaveAllCharacterSkins()
                            self:SetCharacter( content_id, skin_index )
                        end
                    else
                        for i, skin_table in ipairs( skins ) do
                            local str
                            if skin_table.alias then
                                str = string.format( "%s (%s)", skin_table.name, skin_table.alias )
                            else
                                str = tostring(skin_table.name)
                            end
                            if ui:MenuItem( str ) then
                                self:SetCharacter( content_id, i )
                            end
                        end
                    end
                    ui:EndMenu()
                end
            end
            ui:EndPopup()
        end

        local skins
        if self.left_char then
            ui:SameLine( nil, 10 )
            ui:PushStyleColor( ui.Style_Button, 0x007700FF )
            if ui:Button( "New", 60 ) then
                ui:OpenPopup("NAME_SKIN")
            end
            ui:PopStyleColor()

            if ui:BeginPopupModal( "NAME_SKIN", ui.NoTitleBar | ui.NoResize | ui.NoMove | ui.NoScrollbar ) then
                ui:Dummy( 300, 10 )
                ui:Text( "  Enter a name for this new " ..tostring(self.left_char:GetContentID()).. ":" )
                self.new_skin_name = ui:InputText( "Name", self.new_skin_name or "" ) or self.new_skin_name
                local disabled = not IsValidSkinName( self.new_skin_name )
                ui:Spacing()
                if ui:Button( "OK", nil, nil, disabled ) then
                    local skin_index, skin = Content.CreateCharacterSkin( self.left_char:GetContentID() )
                    skin.name = self.new_skin_name
                    Content.SaveAllCharacterSkins()
                    self:SetCharacter( self.left_char:GetContentID(), skin_index )
                    ui:CloseCurrentPopup()
                end

                ui:SameLine( nil, 20 )
                if ui:Button( "Cancel" ) then
                    ui:CloseCurrentPopup()
                end
                ui:Dummy( 300, 10 )

                ui:EndPopup()
            end

            ui:SameLine( nil, 40 )
            if ui:SmallButton( "?" ) then
                DBG( self.left_char )
            end

            skins = Content.GetAllCharacterSkins()[ self.left_char:GetContentID() ]
        end
        if skins then
            local skin_index = self.current_skin_index
            local current_skin = skins[ skin_index ]
            if current_skin then
                ui:SameLine( nil, 5 )
                ui:Text( string.format( "%s skin %d/%d", self.left_char:GetContentID(), skin_index, #skins ))

                ui:SameLine( nil, 10 )
                if ui:Button( "<", 20 ) then
                    if skin_index == 1 then
                        skin_index = #skins
                    else
                        skin_index = skin_index - 1
                    end
                    self:SetCharacter( self.left_char:GetContentID(), skin_index )
                end

                ui:SameLine( nil, 5 )
                if ui:Button( ">", 20 ) then
                    if skin_index == #skins then
                        skin_index = 1
                    else
                        skin_index = skin_index + 1
                    end
                    self:SetCharacter( self.left_char:GetContentID(), skin_index )
                end

                ui:SameLine( nil, 20 )
                if ui:Checkbox( "Show all heads", self.character_skins_panel.show_all_heads == true ) then
                    self.character_skins_panel.show_all_heads = not self.character_skins_panel.show_all_heads
                end

                ui:Spacing()
                self.character_skins_panel:RenderSingleColumn( ui, current_skin )
                if self.character_skins_panel:IsExpanded( current_skin ) then
                    self.left_panel_height = 240
                else
                    self.left_panel_height = 120
                end

                ui:Columns(1)
            end
        end

        ui:End()
    end
end


