require "ui/screens/debugtoolsmenu"
require "ui/screens/conversationscreen"



local VOICE_LIST = 
{    
    "angryDefault",
    "angryAside",
    "angryCruel",
    "angryGreeting",
    "angryHostile",
    "angryPatienceLost",
    "angryReluctant",
    "angrySeething",
    "angryTakenAback",
    "angryThanks",
    
    "happyDefault",
    "happyAmazed",
    "happyCocky",
    "happyGreeting",
    "happyJoke",
    "happyResigned",
    "happyTakenAback",
    "happyThanks",
    
    "miscAlluring",
    "miscDepressed",
    "miscEmbarrassed",
    "miscHopeless",
    "miscHushedUrgent",
    "miscMischievous",
    "miscMocking",
    "miscNostalgic",
    "miscPersuasive",
    "miscRelieved",
    
    "neutralDefault",
    "neutralAside",
    "neutralDirect",
    "neutralDubious",
    "neutralGreeting",
    "neutralJoke",
    "neutralResigned",
    "neutralTakenAback",
    "neutralThanks",
    "neutralThoughtful",
    "neutralWhatever",
    
    "scaredDefault",
    "scaredAnguished",
    "scaredFearful",
    "scaredGreeting",
    "scaredPowerless",
    "scaredStammering",
    "scaredTakenAback",
    "scaredThanks",
}
table.sort(VOICE_LIST)
local emotes = require "content/convo_emotes"
local EMOTE_LIST = copykeys(emotes)
table.sort(EMOTE_LIST)

local CHARACTER_DEFS_ARRAY = {}
for content_id, def in pairs( Content.GetAllCharacterDefs() ) do
    if not def.is_template then
        table.insert( CHARACTER_DEFS_ARRAY, content_id )
    end
end
table.sort( CHARACTER_DEFS_ARRAY )


local function to_array(t, fn)
    local out = {}
    for k,v in pairs(t) do
        table.insert(out, v)
    end
    if fn then
        table.sort(out, fn)
    end
    return out
end

local sortfn = function(a,b) return (a.txt or "") < (b.txt or "") end


local function convert_tree(t)
    if t.entries then
        table.sort(t.entries, function(a, b) return a.id < b.id end)
        for k,v in pairs(t.entries) do
            v.parent = t
        end
    end

    if t.children then
        t.children = to_array(t.children, sortfn)
        for k,v in ipairs(t.children) do
            t.children[k] = convert_tree(v, sortfn) 
            t.children[k].parent = t
        end
    end
    return t
end


local TestHub = class( "TestHub", BaseHub )
TestHub.can_reload = true

function TestHub:init( encounter, convotest )
    TestHub._base.init( self, encounter )
    self.convotest = convotest
end

function TestHub:RunHub( enc, state )
    self.playing = false
    while true do
        if self.next_dialog_to_play then
            self.playing = true
            self.dialog_to_play = self.next_dialog_to_play
            self.next_dialog_to_play = nil
            self.convotest:SetFastMode(false)
            self.encounter:PresentAgent(self.encounter:GetCastAgent(self.convotest.start_right_cast), SIDE.RIGHT)
            self.encounter:PresentAgent(self.encounter:GetCastAgent(self.convotest.start_left_cast), SIDE.LEFT)

            self.enc:Dialog(self.enc:LocFormat(self.dialog_to_play))
        end
        self.playing = false
        enc:YieldEncounter()
    end

end


function TestHub:PlayDialog(dialog)
    self.enc:StopConvo()
    self.enc:StartConvo(self.convotest)
    self.convotest:NextLine()
    self.convotest:ClearHistory()
    self.next_dialog_to_play = dialog
    self.enc:ResumeEncounter()
end
    



local ConvoTestScreen = class("Screen.ConvoTestScreen", Screen.ConversationScreen)


function ConvoTestScreen:init()
    ConvoTestScreen._base.init(self)
    TheGame.gamestate = GameState()
    TheGame:GetGameState():AddPlayerAgent( Agent( "SAL" ))

    --self:SetAnchors("center", "center")
    self.speaker_idx = {}

    self.plax = Widget.Plax()
    self:AddChild( self.plax, 1 ):SetAnchors( "center", "center" )

    self.plax:SetPlaxData( "INT_GrogNDog_1" )
    
    self:FindConvos()
end

function ConvoTestScreen:LoadLocalData()
    self.local_data = deepcopy(DebugUtil.GetLocalDebugData("CONVOTESTER")) or { }

    self.local_data.convo_settings = self.local_data.convo_settings or {}
    
    local entry = self.local_data.current_selection and self.lookups[self.local_data.current_selection]
    if entry then
        self:SetCurrentEntry(entry)
    end
    if self.current_entry then
        local function set_open(node)
            node.open = true
            if node.parent then
                set_open(node.parent)
            end
        end
        set_open(self.current_entry)
    end
end

function ConvoTestScreen:ApplySelectionDefaults()

    self.start_right_cast = nil
    self.start_left_cast = nil
    table.clear( self.speaker_idx )
    
    local id = self.current_entry and self.current_entry.address

    if id then
        local my_data = self.local_data.convo_settings[id]    
        if my_data then
            if my_data.cast then
                for cast, def in pairs(my_data.cast) do
                    self.speaker_idx[cast] = table.arrayfind(CHARACTER_DEFS_ARRAY, def) or 1
                    self:AssignCastMember(cast, def)
                end
            end
            self.start_right_cast = my_data.right
            self.start_left_cast = my_data.left
        end
    end

    if self.start_right_cast == nil then
        self.start_right_cast = "agent"
    end

    if self.start_left_cast == nil then
        self.start_left_cast = "player"
    end

    self.hub.encounter:PresentAgent(self.hub.encounter:GetCastAgent(self.start_right_cast), SIDE.RIGHT)
    self.hub.encounter:PresentAgent(self.hub.encounter:GetCastAgent(self.start_left_cast), SIDE.LEFT)


end




function ConvoTestScreen:SaveLocalData()
    self.local_data.current_selection = self.current_entry and self.current_entry.address
    
    if self.local_data.current_selection then
        self.local_data.convo_settings[self.local_data.current_selection] = self.local_data.convo_settings[self.local_data.current_selection] or {}
        local my_data = self.local_data.convo_settings[self.local_data.current_selection]
        my_data.cast = my_data.cast or {}
        for cast, idx in pairs(self.speaker_idx) do
            my_data.cast[cast] = CHARACTER_DEFS_ARRAY[idx]
        end

        my_data.left = self.start_left_cast
        my_data.right = self.start_right_cast
    end

    DebugUtil.SetLocalDebugData("CONVOTESTER", self.local_data)
    DebugUtil.SaveLocalDebugData()
end


function ConvoTestScreen:FindConvos()
    local convos = {}
    local function GetConvoStrings(quest_id, convo_id, state_id)
        for k, convo in ipairs(convos) do
            if convo.quest_id == quest_id and convo.convo_id == convo_id and convo.state_id == state_id then
                return convo
            end
        end

        local newconvo = {quest_id = quest_id, convo_id = convo_id, state_id = state_id, loc = {}}
        table.insert(convos, newconvo)

        return newconvo
    end


    for convo_id, convodef in Content.IterateConvoStateGraphs( ) do
        if convodef.loc then
            local entry = GetConvoStrings(convodef.quest_id, convo_id, nil)
            for k, v in pairs(convodef.loc) do
                entry.loc[k] = v
            end
        end

        if convodef.states then
            for k, state in pairs(convodef.states) do
                if state.loc then

                    local entry = GetConvoStrings(convodef.quest_id, convo_id, k)
                    for k,v in pairs(state.loc) do
                        entry.loc[k] = v
                    end
                end
            end
        end
    end


    local story_quests = {txt = "Story", children = {}}
    local side_quests = {txt = "Side", children = {}}
    local contracts = {txt = "Contracts", children = {}}
    local events = {txt = "Events", children = {}}
    local gen_convos = {txt = "Convos", children = {}}
    local scenarios = {txt = "Scenarios", children = {}}
    local oppos = {txt = "Opportunities", children = {}}
    
    
    local string_lookups = {}

    local address = {}
    for k,v in ipairs(convos) do

        local quest_def = v.quest_id and Content.GetQuestDef( v.quest_id )
        
        local qtype = quest_def and quest_def.qtype
        
        local parent_table = gen_convos
        if qtype == QTYPE.STORY then
            parent_table = story_quests
        elseif qtype == QTYPE.CONTRACT then
            parent_table = contracts
        elseif qtype == QTYPE.EVENT then
            parent_table = events
        elseif qtype == QTYPE.SIDE then
            parent_table = side_quests
        elseif qtype == QTYPE.SCENARIO then
            parent_table = scenarios
        elseif qtype == QTYPE.OPPORTUNITY then
            parent_table = oppos
        end

        local quest_id = v.quest_id 

        local convo_entry
        if quest_id then
            table.insert(address, quest_id)
            parent_table.children[quest_id] = parent_table.children[quest_id] or {txt = quest_id, quest_id = quest_id, children = {} }
            local quest_entry = parent_table.children[quest_id] 

            quest_entry.children[v.convo_id] = quest_entry.children[v.convo_id] or { txt = v.convo_id, children = {}, entries = {} }
            convo_entry = quest_entry.children[v.convo_id]
        else
            parent_table.children[v.convo_id] = parent_table.children[v.convo_id] or { txt = v.convo_id, children = {}, entries = {} }
            convo_entry = parent_table.children[v.convo_id]
        end
        
        table.insert(address, v.convo_id)

        local convo_id = v.convo_id

        if v.state_id then
            table.insert(address, v.state_id)
            
            convo_entry.children[v.state_id] = convo_entry.children[v.state_id] or {txt = v.state_id, entries = {}}
            local state_entry = convo_entry.children[v.state_id]
            if v.loc then
                local base = table.concat(address, ".") .. "."
                for k,v in pairs(v.loc) do
                    local entry = {id=k, str=v, address = base..k, convo_id = convo_id}
                    table.insert( state_entry.entries, entry) 
                    string_lookups[entry.address] = entry   

                end
            end

            table.remove(address)
        else
            if v.loc then
                local base = table.concat(address, ".") .. "."
                for k,v in pairs(v.loc) do
                    local entry = {id=k, str=v, address = base..k, convo_id = convo_id}
                    table.insert( convo_entry.entries, entry)    
                    string_lookups[entry.address] = entry
                end
            end
        end

        if quest_id then
            table.remove(address)
        end

        table.remove(address)
    end
    local all = {txt = "All", open = true, 
        children = {
            convert_tree(story_quests), 
            convert_tree(contracts), 
            convert_tree(side_quests), 
            convert_tree(events), 
            convert_tree(oppos),
            convert_tree(scenarios),
            convert_tree(gen_convos), 
        }}    
    self.quests, self.lookups =  all, string_lookups
end


function ConvoTestScreen:OnOpen()
    ConvoTestScreen._base.OnOpen( self )


    local encounter = Encounter(nil, Location("GROG_N_DOG"))
    self.hub = TestHub( encounter, self)
    encounter:SetInitialHub( self.hub )
    self:StartConvo(encounter)
    self.plax:PostProcess( true )
    self:LoadLocalData()
    self:ApplySelectionDefaults()

end

function ConvoTestScreen:DoRender()
    self.plax:RenderToScene()
    local ui = TheGame:GetDebug():GetDebugUI()
    self:RenderPanel( ui )
end


function ConvoTestScreen:SetCurrentEntry(v)
    self.current_entry = v

    local lines = v.str:split("\n")
    local speakers = {}
    
    for i, line in ipairs(lines) do
        line = line:trim()
        local speaker = string.match(line, "^([_%w]*):$")
        if speaker then
            speakers[speaker] = true
            
        end

    end
    self.sorted_speakers = {"agent"}
    for k,v in pairs(speakers) do
        if k ~= "right" and k ~= "left" and k ~= "agent" and k ~= "player" then
            table.insert(self.sorted_speakers, k)
        end
    end

    table.sort(self.sorted_speakers)
    table.clear(self.speaker_idx)

    self:ApplySelectionDefaults()
end

function ConvoTestScreen:AssignCastMember(cast_id, def)
    local current_agent = self.hub:GetCastAgent(cast_id)

    if not current_agent or current_agent:GetContentID() ~= def then
        self.hub:ReassignCastMember(cast_id, Agent(def))
    end                                          
end

function ConvoTestScreen:RenderPanel( ui )
    
    if self.hub.playing then
        do return end
    end

    local w, h = ui:DisplaySize()

    
    ui:SetNextWindowSize( w * .33, h*.67, ui.ImGuiSetCond_Once )
    ui:SetNextWindowPos( 0, 0, ui.ImGuiSetCond_Once )


    if ui:Begin( "Convo Browser") then

        local function show_tree(node)
            
            if ui:TreeNode( node.txt or "", node.open and ui.TreeFlags_DefaultOpen ) then
                if node.children then
                    for k,node in ipairs(node.children) do
                        show_tree(node)
                    end
                end

                if node.entries then
                    ui:Indent( 20 )
                    ui:PushStyleColor( ui.Style_Text, 0x00CCCCFF )
                    for k,v in pairs(node.entries) do
                        if ui:Selectable(v.id, self.current_entry == v) then
                            self:SetCurrentEntry(v)
                            self:ApplySelectionDefaults()
                            self:SaveLocalData()
                            
                        end
                    end
                    ui:PopStyleColor()
                    ui:Unindent( 20 )

                end

                ui:TreePop()
            end
        end
        
        ui:BeginChild("Tree", 300, 0)
        show_tree(self.quests)
        ui:EndChild()
        
        ui:SameLine()
        local w,h = ui:GetWindowSize()
        
        ui:BeginChild("Text", 0, 0)
        if self.current_entry then
            
   
            if ui:Button("Play") then
                if self.current_entry then
                    self.hub:PlayDialog(self.current_entry.str)
                end
            end

            ui:SameLine(nil, 10)

            if ui:Button("Reload") then
                Content.ReloadConversation()
                self:FindConvos()
                if self.current_entry then
                    self.current_entry = self.lookups[self.current_entry.address]
                end
            end

            ui:SameLine(nil, 10)
            
            if ui:Button("Open File") then
                local convo_def = Content.GetConvoStateGraph(self.current_entry.convo_id)
                if convo_def and convo_def.def_path then
                    local path = BASE_PATH .. "..\\data\\" .. convo_def.def_path
                    os.execute( "subl " .. path .. ":" .. (convo_def.def_line or 1) )
                end
            end
            
            ui:Separator()
            ui:TextWrapped(self.current_entry.str or "")
            ui:Separator()
            
            ui:PushItemWidth( 200 )

            if self.sorted_speakers then
                for k,cast_id in ipairs(self.sorted_speakers) do
                    local new_idx = ui:Combo( cast_id, self.speaker_idx[cast_id] or 1, CHARACTER_DEFS_ARRAY ) 
                    if new_idx ~= self.speaker_idx[cast_id] then
                        self.speaker_idx[cast_id] = new_idx
                        self:SaveLocalData()
                        local def = CHARACTER_DEFS_ARRAY[new_idx] 
                        self:AssignCastMember(cast_id, def)

                    end
                end

                ui:Separator()
                local candidates = {"NOBODY", "player", "agent"}
                for k,v in pairs(self.sorted_speakers) do
                    table.insert(candidates, v)
                end
                local left_idx, right_idx = 1, 1
                if self.start_left_cast then
                    left_idx = table.find( candidates, self.start_left_cast) or 1
                end

                if self.start_right_cast then
                    right_idx = table.find( candidates, self.start_right_cast) or 1
                end

                local new_right_idx = ui:Combo( "RIGHT START", right_idx, candidates ) 
                if new_right_idx ~= right_idx then
                    self.start_right_cast = candidates[new_right_idx]
                    self:SaveLocalData()

                    if self.start_right_cast then
                        local current_agent = self.hub.encounter:GetCastAgent(self.start_right_cast)
                        self.hub.encounter:PresentAgent(current_agent, SIDE.RIGHT)
                    else
                        self.hub.encounter:PresentAgent(nil, SIDE.RIGHT)
                    end
                end

                local new_left_idx = ui:Combo( "LEFT START", left_idx, candidates ) 
                if new_left_idx ~= left_idx then
                    self.start_left_cast = candidates[new_left_idx]
                    self:SaveLocalData()
                    if self.start_left_cast then
                        local current_agent = self.hub.encounter:GetCastAgent(self.start_left_cast)
                        self.hub.encounter:PresentAgent(current_agent, SIDE.LEFT)
                    else
                        self.hub.encounter:PresentAgent(nil, SIDE.LEFT)
                    end
                end

                ui:Separator()
                ui:PopItemWidth()

                ui:PushItemWidth( 100 )
                self.voice_test_idx = ui:Combo( "Voice", self.voice_test_idx or 1, VOICE_LIST )             
                ui:PopItemWidth()
                ui:SameLine(nil, 10)
                if ui:Button("Left###VOICELEFT") then
                    local txt = loc.format( "left:\n${1}\nI am saying some dialog with voice {1}.", VOICE_LIST[self.voice_test_idx])
                    self.hub:PlayDialog(txt)
                end
                ui:SameLine(nil, 10)
                if ui:Button("Right###VOICERIGHT") then
                    local txt = loc.format( "right:\n${1}\nI am saying some dialog with voice {1}.", VOICE_LIST[self.voice_test_idx])
                    self.hub:PlayDialog(txt)
                end

                ui:PushItemWidth( 100 )
                self.emote_test_idx = ui:Combo( "Emote", self.emote_test_idx or 1, EMOTE_LIST )             
                ui:PopItemWidth()
                ui:SameLine(nil, 10)
                if ui:Button("Left###EMOTELEFT") then
                    local txt = loc.format( "left:\n!{1}", EMOTE_LIST[self.emote_test_idx])
                    self.hub:PlayDialog(txt)
                end
                ui:SameLine(nil, 10)
                if ui:Button("Right###EMOTERIGHT") then
                    local txt = loc.format( "right:\n!{1}", EMOTE_LIST[self.emote_test_idx])
                    self.hub:PlayDialog(txt)
                end


            end

             

        end
        ui:EndChild()
    end
    ui:End()
end
