
local CreditsScreen = class("Screen.CreditsScreen", Widget.Screen)

local ANIM_OFFSET = RES_X/4    

local CREDITS_KLEI_NAMES = { 
    "Aaron Bouthillier",
    "Abraham Stolk",
    "Andrea Alcaraz",
    "Angela An",
    "Brook Miles",
    "Bryce Doig",
    "Chris Costa",
    "Corey Rollins",
    "Daniel Yu",
    "Dave Downes",
    "Elaine Chen",
    "Eugene ( Sloth ) Trakhtenberg",
    "Emmett Hall",
    "Geoff Pitsch",
    "Hugo Moraleida",
    "Jamie Cheng",
    "Jan Hutchings",
    "Janis Osenenko",
    "Jeddy Zhang",
    "Jeff Agala",
    "Jesse Turner",
    "Joe Wreggelsworth",
    "Jordan Krahn",
    "Ju-lian Kwan",
    "Kelly Graham",
    "Kengo Ito",
    "Kevin Forbes",
    "Kris Fortune",
    "Lou Papa",
    "Margaret Lu",
    "Matthew Marteinsson",
    "Meghan Shaw",
    "Mike Bell",
    "Nana Fujino",
    "Nick Waanders",
    "Nicole Dempsey",
    "Quentin Proust",
    "Ragnar Guidote",
    "Rasmus Dahl Tindborg",
    "Ricardo Pinto",
    "Shannon Campbell",
    "Vangelis Christou",
    "Vincent Kerdraon",
    "Wade Lindley",
    "Will Turnbull",
    "Yorman Arias",
    "Andrea Alacaraz",
    "Ian Bartolo",
    "Vince De Vera",
}

local CREDITS_POWERUP_NAMES = {
    "Kevin Regamey",
    "Cole Verderber",
    "Jeff Tangsoc",
    "Joey Godard",
    "Craig Barnes"
}

local CREDITS_VOICE_NAMES = {
    "Veena Sood",
    "Mavis Ada",
    "Becky Bright",
    "Dustin Gillis",
    "Emma Barratt",
    "Uri Burstyn",
    "Branden Kupfer",
    "Conan Freeman",
    "Jericho Reid",
    "Ashleigh Somerville",
    "Brandon Thornhill",
    "Michael Dobson",
    "Mae Matchett",
    "Adam Trask",
    "Katharine Morgan Taylor",
    "Andrew Bukowski",
    "Theodora Holmes",
    "Hampton Green",
    "Rae Hall",
    "MAK",
    "Asia Mattu",
    "Max Johnson",
}

local CREDITS_MUSIC_NAMES = {
    "Jim Guthrie",
}

local CREDITS_SKYMAP_NAMES = {
    "Neal Laurenza", 
    "Jeremy Quinn", 
    "Matthew Dube",
}

local CREDITS_HUWIZ_NAMES = {
    "Alexandre Do Rego",
    "Camille Boudreault",
    "Carolljo Maher",
    "Charles Hamel",
    "Christopher Sauvé",
    "Éliane Dumesnil",
    "Frédéric Renard",
    "Gabriel Levac",
    "Guillaume Lanéel",
    "Jan Brotto",
    "Jimmy-Lee Daneault",
    "Julie Vieira",
    "Julien Lessard",
    "Kariane Lacharité",
    "Louis St-Denis",
    "Martin Gagnon",
    "Mathew Potvin",
    "Maxime Landry Lamy",
    "Nathan Legault",
    "Nicolas Lefebvre",
    "Olivier Boivin",
    "Phillipe Bouthillier",
    "Roxane Paradis",
    "Samuel Audet",
    "Sandra Malenfant",
    "Simon Chabot",
    "Stéphane Gaulin",
}

CreditsScreen.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self) 
            self:DoClose()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL) )
        end,
    },
}

function CreditsScreen:init( card_id )
    Widget.Screen.init(self)
    
    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )
    
    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    
    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Our fancy character anim!
    self.anim_root = self:AddChild(Widget()):SetHiddenBoundingBox( true )

    -- A container for the elements to be displayed
    self.container = self:AddChild(Widget())

    -- A hitbox for alignment
    self.hitbox = self:AddChild( Widget.SolidBox( 1000, 800, 0xFF000030 ) )
        :SetPos( 100, 0 )
        :Hide()

    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.close_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", function() self:DoClose() end ) )
    self.close_button:SetIcon( global_images.close )
    self.close_button:LayoutBounds( "left", "above", 60, 60 )
    self:RegisterHotkeyButton( Controls.Digital.MENU_CANCEL, self.close_button )

    self:Layout()

end

function CreditsScreen:ShowCreditsAnimationCoro()

    self:Delay(0.4)
    self:ShowMultiSlideCredits( nil, nil, CREDITS_KLEI_NAMES, 3 )
    self:ShowSingleSlideCredits( LOC"UI.CREDITS_SCREEN.POWERUP_TITLE", LOC"UI.CREDITS_SCREEN.POWERUP_SUBTITLE", CREDITS_POWERUP_NAMES, 3, LOC"UI.CREDITS_SCREEN.ADDITIONAL_SOUND" )
    self:ShowSingleSlideCredits( LOC"UI.CREDITS_SCREEN.MUSIC_TITLE", nil, CREDITS_MUSIC_NAMES, 3, LOC"UI.CREDITS_SCREEN.ADDITIONAL_MUSIC" )
    self:ShowMultiSlideCredits( LOC"UI.CREDITS_SCREEN.VOICE_TITLE", nil, CREDITS_VOICE_NAMES, 3 )

    if CheckAnyBits( SPECIAL_BUILD_TYPE, 128 | 256 | 1024 ) then
        self:ShowSingleSlideCredits( LOC"UI.CREDITS_SCREEN.SKYMAP_TITLE", nil, CREDITS_SKYMAP_NAMES, 3)
        self:ShowMultiSlideCredits( LOC"UI.CREDITS_SCREEN.HUWIZ_TITLE", nil, CREDITS_HUWIZ_NAMES, 3, true )
    end

    self:ShowFmodLogo( 3 )
    self:ShowLogo( engine.asset.Texture("UI/credits_klei.tex"), 0.9, nil ) -- nil duration means it doesn't fade out
    self.credits_ended = true
    
end

function CreditsScreen:OnScreenResize(w, h)
    CreditsScreen._base.OnScreenResize(self, w, h)
    self:Layout()
    self:RefreshStreamerMode()
end

function CreditsScreen:OnControlModeChange( cm, device_id )
    CreditsScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.close_button:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.close_button:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.close_button:Hide()
    end
end

function CreditsScreen:OnScreenModeChange( sm )
    CreditsScreen._base.OnScreenModeChange( self, sm )

    self.container:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )
end

function CreditsScreen:GetDefaultFocus()
    return self.close_button
end


function CreditsScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
end

function CreditsScreen:Layout()
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Size up the grid
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

end

function CreditsScreen:OnOpen()
    self._base.OnOpen( self )
    
    --AUDIO:PlayEvent( "event:/ui/main/fld/feedback_menu_open" )

    -- Kick-off the animation!
    self:StartCoroutine( CreditsScreen.ShowCreditsAnimationCoro, self )
end

function CreditsScreen:ShowMultiSlideCredits( title_txt, subtitle_txt, names, slide_duration, no_shuffle )
    
    slide_duration = slide_duration or 3

    local COLUMNS = 2
    local ROWS_PER_COLUMN = 4

    -- Our names display widget
    local names_widget = self.container:AddChild( Widget() )
    names_widget.columns = {}
    for k = 1, COLUMNS do
        -- Add each column container
        local column = names_widget:AddChild( Widget() )
        table.insert( names_widget.columns, column )
        -- To each column, add the right number of rows
        column.rows = {}
        for j = 1, ROWS_PER_COLUMN do
            local row_label = column:AddChild( Widget.Label("title", 40 ))
                :SetGlyphColour( UICOLOURS.SUBTITLE )
                :Bloom(0.1)
                :SetTintAlpha(0)
            table.insert( column.rows, row_label )
        end
    end

    -- Add a shadow under the names
    local shadow = self.container:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
        :SetSize( 1000, 100 )
        :SetHiddenBoundingBox( true )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0 )

    -- Titles
    local title = self.container:AddChild( Widget.Label("title", 30 ))
        :SetText( title_txt or "" )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0 )
        :Bloom(0.05)
    local subtitle = self.container:AddChild( Widget.Label("title", 40 ))
        :SetText( subtitle_txt or "" )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0 )
        :Bloom(0.1)

    -- Mix up all the names
    if not no_shuffle then
        table.shuffle(names)
    end
    local name_idx = 1
    local function GetNextName()
        local name = names[name_idx]
        name_idx = name_idx + 1
        return name
    end

    -- Display all the names in groups
    local first_time = true
    local names_shown = 0
    while names_shown < #names do

        -- Get the next group
        local group = {}
        for k = 1, COLUMNS*ROWS_PER_COLUMN do
            local name = GetNextName()
            if name then
                names_shown = names_shown + 1
                table.insert(group, name)
            else
                break
            end
        end

        -- Display the next group
        for k = 1, COLUMNS do
            for j = 1, ROWS_PER_COLUMN do
                local pos = (k-1)*ROWS_PER_COLUMN + j
                if group[ pos ] then
                    names_widget.columns[k].rows[j]:SetText( group[pos] )
                else
                    names_widget.columns[k].rows[j]:SetText( "" )
                end
            end
            -- Layout all the rows
            names_widget.columns[k]:ArrangeChildrenColumn( 80 )
        end
        -- Layout all the columns
        names_widget:StackChildrenRow( 140 )

        if first_time or not title_txt then
            -- Align the names
            names_widget:LayoutBounds( "center", "center", self.hitbox )

            -- Position the titles
            subtitle:LayoutBounds( "center", "above", names_widget )
                :Offset( 0, 20 )
            title:LayoutBounds( "center", "above", subtitle )
                :Offset( 0, -6 )
        else
            -- Align the names
            if subtitle then
                names_widget:LayoutBounds( "center", "below", subtitle )
                    :Offset( 0, -20 )
            elseif title then
                names_widget:LayoutBounds( "center", "below", title )
                    :Offset( 0, -26 )
            end

        end

        -- Update the anim
        self:ShowBigCharacter()

        if first_time then
            -- Fade in the title
            if title_txt then
                local fade_in_delay = 0.4
                local t = 0
                while t < fade_in_delay do
                    local dt = coroutine.yield()
                    t = t + dt
                    local alpha = (t/fade_in_delay)
                    title:SetTintAlpha( alpha*0.7 )
                end
                self:Delay(0.2)
            end


            -- Fade in the subtitle
            if subtitle_txt then
                local fade_in_delay = 0.2
                local t = 0
                while t < fade_in_delay do
                    local dt = coroutine.yield()
                    t = t + dt
                    local alpha = (t/fade_in_delay)
                    subtitle:SetTintAlpha( alpha )
                end
                self:Delay(0.3)
            end
        end
        first_time = false


        -- Position the shadow
        shadow:LayoutBounds( "center", nil, names_widget )
            :LayoutBounds( nil, "center", self.grid )
            :Offset( 0, -20 )

        -- Fade in the names
        local fade_in_delay = 0.4
        local t = 0
        while t < fade_in_delay do
            local dt = coroutine.yield()
            t = t + dt
            local alpha = (t/fade_in_delay)
            for k = 1, COLUMNS do
                for j = 1, ROWS_PER_COLUMN do
                    names_widget.columns[k].rows[j]:SetTintAlpha( alpha )
                end
            end
            shadow:SetTintAlpha( alpha/2 )
        end
    
        -- Wait for a beat
        self:Delay( slide_duration )

        -- Remove the current character
        self:HideBigCharacter()

        -- Fade out the names
        local fade_out_delay = 0.4
        local t = 0
        while t < fade_out_delay do
            local dt = coroutine.yield()
            t = t + dt
            local alpha = 1 - (t/fade_out_delay)
            for k = 1, COLUMNS do
                for j = 1, ROWS_PER_COLUMN do
                    names_widget.columns[k].rows[j]:SetTintAlpha( alpha )
                end
            end
            shadow:SetTintAlpha( alpha/2 )
        end

        -- Wait for a beat
        self:Delay( 0.4 )

    end

    self.container:DestroyAllChildren()
end

function CreditsScreen:ShowSingleSlideCredits( title_txt, subtitle_txt, names, duration, extra )
    local COLUMNS = 2

    local extra_widget = extra and self.container:AddChild( Widget.Label("title", 30, extra ) )
                                                        :SetGlyphColour( UICOLOURS.SUBTITLE )
                                                        :SetTintAlpha( 0 )
                                                        :Bloom(0.05)

    -- Our powerup names display widget
    local names_widget = self.container:AddChild( Widget() )
    local first_name
    for k = 1, #names do
        local row_label = names_widget:AddChild( Widget.Label("title", 40 ))
            :SetGlyphColour( UICOLOURS.SUBTITLE )
            :SetRegionSize( 330, 60 )
            :SetText( names[k] )
            :Bloom(0.1)
            :SetTintAlpha(0)
        if first_name == nil then first_name = row_label end
        row_label:LayoutInGrid( COLUMNS, k, 20, first_name )
    end

    -- Align the names
    self.container:LayoutBounds( "center", "center", self.hitbox )

    -- Add a shadow under the names
    local shadow = self.container:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
        :SetSize( 900, 100 )
        :SetHiddenBoundingBox( true )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0 )

    -- Position the shadow
    shadow:LayoutBounds( "center", nil, names_widget )
        :LayoutBounds( nil, "center", self.grid )
        :Offset( 0, -20 )

    -- Titles
    local title = self.container:AddChild( Widget.Label("title", 30 ))
        :SetText( title_txt or "" )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0 )
        :Bloom(0.05)
    local subtitle = self.container:AddChild( Widget.Label("title", 40 ))
        :SetText( subtitle_txt or "" )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0 )
        :Bloom(0.1)

    -- Position the titles
    subtitle:LayoutBounds( "center", "above", names_widget )
        :Offset( 0, 20 )
    title:LayoutBounds( "center", "above", subtitle )
        :Offset( 0, -6 )

    if extra_widget then
        extra_widget:LayoutBounds("center", "below", names_widget):Offset(0, -30)
    end

    -- Update the anim
    self:ShowBigCharacter()

    -- Fade in the title
    if title_txt then
        local fade_in_delay = 0.4
        local t = 0
        while t < fade_in_delay do
            local dt = coroutine.yield()
            t = t + dt
            local alpha = (t/fade_in_delay)
            title:SetTintAlpha( alpha*0.7 )
        end
    self:Delay(0.2)
    end

    -- Fade in the subtitle
    if subtitle_txt then
        local fade_in_delay = 0.2
        local t = 0
        while t < fade_in_delay do
            local dt = coroutine.yield()
            t = t + dt
            local alpha = (t/fade_in_delay)
            subtitle:SetTintAlpha( alpha )
        end
    self:Delay(0.3)
    end


    -- Fade in the names
    local fade_in_delay = 0.4
    local t = 0
    while t < fade_in_delay do
        local dt = coroutine.yield()
        t = t + dt
        local alpha = (t/fade_in_delay)
        for k, label in ipairs( names_widget.children ) do
            label:SetTintAlpha( alpha )
        end
        shadow:SetTintAlpha( alpha/2 )

        if extra_widget then 
            extra_widget:SetTintAlpha( alpha/2 )
        end
    end

    self:Delay(duration)

    -- Remove the current character
    self:HideBigCharacter()

    -- Fade out everything
    local fade_out_delay = 0.4
    local t = 0
    while t < fade_out_delay do
        local dt = coroutine.yield()
        t = t + dt
        local alpha = 1 - (t/fade_out_delay)
        for k, label in ipairs( names_widget.children ) do
            label:SetTintAlpha( alpha )
        end
        title:SetTintAlpha( alpha*0.7 )
        subtitle:SetTintAlpha( alpha )
        shadow:SetTintAlpha( alpha/2 )
    end

    self.container:DestroyAllChildren()
end

function CreditsScreen:ShowFmodLogo( duration )
    duration = duration or 3

    -- Create our Fmod logo
    local fmod_widget = self.container:AddChild( Widget() )
    fmod_widget.logo = fmod_widget:AddChild( Widget.Image( engine.asset.Texture("UI/credits_fmod.tex") ) )
        :SetSize( 200*0.6, 60*0.6 )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
        :SetTintAlpha( 0 )
    fmod_widget.label = fmod_widget:AddChild( Widget.Label("title", 28 ) )
        :SetText( LOC"UI.CREDITS_SCREEN.FMOD" )
        :LeftAlign()
        :OverrideLineHeight( 23 )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom(0.1)
        :LayoutBounds( "after", "center", fmod_widget.logo )
        :Offset( 15, 0 )
        :SetTintAlpha( 0 )
    fmod_widget.shadow = fmod_widget:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
        :SetSize( 900, 100 )
        :SetHiddenBoundingBox( true )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0 )

    -- Display the Fmod logo
    fmod_widget:LayoutBounds( "center", "center", self.hitbox )

    -- Position its shadow
    fmod_widget.shadow:SetPos( 0, 0 )
        :LayoutBounds( nil, "center", self.grid )
        :Offset( 150, -20 )

    -- Update the anim
    self:ShowBigCharacter()

    -- Fade in the logo
    local fade_in_delay = 0.4
    local t = 0
    while t < fade_in_delay do
        local dt = coroutine.yield()
        t = t + dt
        local alpha = (t/fade_in_delay)
        fmod_widget.logo:SetTintAlpha( alpha )
        fmod_widget.label:SetTintAlpha( alpha )
        fmod_widget.shadow:SetTintAlpha( alpha/2 )
    end

    -- Wait for a beat
    self:Delay( duration )

    -- Remove the current character
    self:HideBigCharacter()

    -- Fade out the logo
    local fade_out_delay = 0.4
    local t = 0
    while t < fade_out_delay do
        local dt = coroutine.yield()
        t = t + dt
        local alpha = 1 - (t/fade_out_delay)
        fmod_widget.logo:SetTintAlpha( alpha )
        fmod_widget.label:SetTintAlpha( alpha )
        fmod_widget.shadow:SetTintAlpha( alpha/2 )
    end

    self.container:DestroyAllChildren()
end

function CreditsScreen:ShowLogo( logo_texture, scale, duration )

    -- Create our Klei logo
    local logo = self.container:AddChild( Widget.Image( logo_texture ) )
        :SetScale( scale )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
        :SetTintAlpha( 0 )
    local shadow = self.container:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
        :SetSize( 900, 100 )
        :SetHiddenBoundingBox( true )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0 )

    -- Display the Klei logo
    self.container:LayoutBounds( "center", "center", self.hitbox )

    -- Position its shadow
    shadow
        :LayoutBounds( "center", nil, logo )
        :LayoutBounds( nil, "center", self.grid )
        :Offset( 0, -20 )

    -- Update the anim
    self:ShowBigCharacter()

    -- Fade in the logo
    local fade_in_delay = 0.4
    local t = 0
    while t < fade_in_delay do
        local dt = coroutine.yield()
        t = t + dt
        local alpha = (t/fade_in_delay)
        logo:SetTintAlpha( alpha )
        shadow:SetTintAlpha( alpha/2 )
    end

    if duration then

        -- Wait for a beat
        self:Delay( duration )

        -- Remove the current character
        self:HideBigCharacter()

        -- Fade out the logo
        local fade_out_delay = 0.4
        local t = 0
        while t < fade_out_delay do
            local dt = coroutine.yield()
            t = t + dt
            local alpha = 1 - (t/fade_out_delay)
            logo:SetTintAlpha( alpha )
            shadow:SetTintAlpha( alpha/2 )
        end

        self.container:DestroyAllChildren()
    end
end

function CreditsScreen:HideBigCharacter()
    -- There's already an agent on screen, send that one away
    if self.biganim then
        local old = self.biganim
        local x, y = old:GetPos()
        old:MoveTo(x-ANIM_OFFSET, y, .2, easing.inQuad, function() old:Remove() end)
        self.biganim = nil
    end
end

function CreditsScreen:ShowBigCharacter()
    -- Show new emoting agent
    local agent = self:GetRandomAgent()

    -- Set default last emotewink
    self.last_emote = "wink"
    
    local outfit = self.outfit_id and Content.GetOutfit(self.outfit_id)
    agent.build = outfit and outfit.build or agent.build
    
    self.biganim = self.anim_root:AddChild( Widget.AnimatedCharacter() )
    self.biganim:Refresh(agent)
    
    local x,y = -RES_X/4-150, -260
    self.biganim:SetScale(1)
    self.biganim:SetPos(x-ANIM_OFFSET, y)
    self.biganim:MoveTo(x,y, .6, easing.outQuad)
    local emotes = {
        ["wink"] = 5,
        ["greeting"] = 6,
        ["chuckle"] = 4,
        ["handwave"] = 3,
        ["laugh"] = 5,
        ["clap"] = 7,
        ["permit"] = 1,
    }
    self.biganim:SetMood( MOOD.HAPPY )
    local next_emote = weightedpick(emotes)
    while self.last_emote == next_emote do
        next_emote = weightedpick(emotes)
    end
    self.biganim:Emote( next_emote )
    self.last_emote = next_emote
end

function CreditsScreen:GetRandomAgent()
    local t = {}
    for content_id, skins in pairs( Content.GetAllCharacterSkins() ) do
        if Content.GetCharacterDef( content_id ) ~= nil then
            for i, skin in ipairs(skins) do
                table.insert( t, { content_id, skin } )
            end
        end
    end

    local content_id, skin = table.unpack( table.arraypick( t ))
    while self.last_agent_id and self.last_agent_id == content_id do
        content_id, skin = table.unpack( table.arraypick( t ))
    end
    local agent = Agent( content_id, skin )
    if self.last_agent_id == nil then self.last_agent_id = content_id end
    return agent
end

function CreditsScreen:DoClose()
    self:GetFE():PopScreen( self )
end
