
require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/solidbox"
require "ui/widgets/panelbutton"
require "ui/widgets/iconbutton"
require "ui/widgets/panel"
require "ui/widgets/slider"
require "ui/widgets/checkbox"

-------------------------------------------------------------------------
--

local PANEL_H = 760
local CARD_SCALE = 0.75
local CARD_SCALE_HOVER = 0.8

local BONUS_TRACK = {0,0,0,10,15,20,30,30,30,50}
local function CalcBonus(bounty_level) 
    local idx = bounty_level+1
    return BONUS_TRACK[math.min(#BONUS_TRACK, idx)]
end

local DraftChoicePopup= class("Screen.DraftChoicePopup", Widget.Screen)


DraftChoicePopup.CONTROL_MAP = 
{

    {
        control = Controls.Digital.GAMEPAD_X,
        
        test = function(self)
            return UIHelpers.GetFocusCard() ~= nil and not self.show_reticule
        end,
        
        fn = function(self)
            self:OnCardClick( UIHelpers.GetFocusCard() )
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.DRAFT_CARD", Controls.Digital.GAMEPAD_X, UIHelpers.GetFocusCard().card.id))
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_B,
        
        test = function(self)
             return not self.no_skip and not self.selected_card and not self.show_reticule
        end,
        
        fn = function(self)
            self.money_btn:Click()
            return true
        end,
        
        hint = function(self, left, right)
            local is_daily = TheGame:GetGameState():GetGameType() == GAME_TYPE.DAILY 
            if is_daily then
                table.insert(right, loc.format ( LOC"UI.CONTROLS.TAKE_POINTS", Controls.Digital.GAMEPAD_B, self.skip_shills_reward or 0 ))
            else
                table.insert(right, loc.format ( LOC"UI.CONTROLS.TAKE_CARD_MONEY", Controls.Digital.GAMEPAD_B, self.skip_shills_reward or 0 ))
            end
        end,
    },

    {
        control = Controls.Digital.PAUSE, 
        fn = function(self)
            self:GetFE():InsertScreen(Screen.PauseMenu())
            return true
        end,
    },

}

function DraftChoicePopup:init()
    Widget.Screen.init(self)

    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors("center", "center")

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Backing panel
    self.panel = self:AddChild( Widget.WidePanel( PANEL_H ) )
        :SetIcon( engine.asset.Texture( "UI/ic_shop_icon.tex" ) )

    -- Card container
    self.card_container = self:AddChild( Widget() )
    self.bounty_container = self:AddChild( Widget() )
    self.bounty_widgets = {}
    self.shadow_container = self:AddChild( Widget() )
        :SetAnchors( "center", "bottom" )

    self.upgrade_info_label = self:AddChild( Widget.Label( "body", 22 ) )
        :SetGlyphColour( UICOLOURS.OBJECTIVE )
        :SetAutoSize(800)
        :SetWordWrap(true)
        :SetText( LOC"UI.DRAFT_SCREEN.UPGRADES_PREVIEW_INFO" )
        :SetTintAlpha( 0.9 )
        :Bloom( 0.08 )

	-- Used for reporting metrics
	self.metrics_event_type = nil
	self.metrics_cards_presented = nil
	self.metrics_money_presented = nil

    self.select_then_continue_mode = false -- If true, you'll have to click a choice, then click the continue button

    -- Back button
    self.money_btn = self:AddChild( Widget.IconButton( "", function() 
        self.panel:FadeOut( function() self:GetFE():PopScreen( self ) end ) end ) )
    self.money_btn:SetIcon( global_images.close )
        :SetSize( 280, 50 )

    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.button_continue = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.DRAFT_SCREEN.CONFIRM_BTN", function() self:OnClickContinue(self.selected_card) end ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Hide()
        :SetToolTip( LOC"UI.OPTIONS_SCREEN.CONFIRM_SELECT_TOOLTIP" )

    self:SetSounds( nil, SoundEvents.card_select_screen_close )
    self:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
end

function DraftChoicePopup:OnOpen()
    DraftChoicePopup._base.OnOpen( self )

    self.screen_loop = AUDIO:CreateEventInstance(SoundEvents.card_select_screen_open)
    self.screen_loop:Start()
end

function DraftChoicePopup:OnGainTop()
    DraftChoicePopup._base.OnGainTop( self )

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )

    -- If a card is selected, unselect it
    self.selected_card = nil
    -- Unselect options
    for k, widget in ipairs( self.card_container.children ) do
        widget.selected = false
        self:UpdateCardImageState( widget )
    end
end

function DraftChoicePopup:OnControlModeChange( cm, device_id )
    DraftChoicePopup._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.select_then_continue_mode = false or engine.inst:GetSetting("gameplay", "confirm_select", true)
        for k, w in ipairs( self.card_container.children ) do
            w:SetOnClick( function() self:OnCardClick( w ) end )
        end
        if not self.no_skip then self.money_btn:Show() end
        self.bottom_right:SetShown( self.select_then_continue_mode )
        self.upgrade_info_label:Show()
        -- Unselect options
        for k, widget in ipairs( self.card_container.children ) do
            widget.selected = false
        end
        self.selected_card = nil
    end
    if cm == CONTROL_MODE.TOUCH then
        self.select_then_continue_mode = true
        for k, w in ipairs( self.card_container.children ) do
            w:SetOnClick( function() self:OnCardClick( w ) end )
        end
        if not self.no_skip then self.money_btn:Show() end
        self.bottom_right:SetShown( self.select_then_continue_mode )
        self.upgrade_info_label:Hide()
        -- Unselect options
        for k, widget in ipairs( self.card_container.children ) do
            widget.selected = false
        end
        self.selected_card = nil
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.select_then_continue_mode = false
        for k, w in ipairs( self.card_container.children ) do
            -- ccc: disabling normal click on Gamepad, it's too easy to accidentally spam through.
            w:SetOnClick(nil ) -- function() self:OnCardClick( w ) end )
        end
        self.money_btn:Hide()
        self.upgrade_info_label:Hide()
        self.bottom_right:Hide()
        -- Unselect options
        for k, widget in ipairs( self.card_container.children ) do
            widget.selected = false
        end
        self.selected_card = nil
    end

    self:DoLayout()
end

function DraftChoicePopup:OnScreenModeChange( sm )
    DraftChoicePopup._base.OnScreenModeChange( self, sm )

    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.upgrade_info_label:SetLayoutScale( LAYOUT_SCALE[sm] )
    
    local new_panel_h, new_cardcontainer_scale
    if sm == SCREEN_MODE.MONITOR then
        new_panel_h = PANEL_H
        new_cardcontainer_scale = 1

    elseif sm == SCREEN_MODE.SMALL
        or sm == SCREEN_MODE.TV then
        new_panel_h = PANEL_H + 110
        new_cardcontainer_scale = 1.2
    end
    self.card_container:SetLayoutScale( new_cardcontainer_scale )
    if self.money_btn then self.money_btn:SetLayoutScale( LAYOUT_SCALE[sm] ) end
    -- Resize container panel
    self.panel:SetHeight( new_panel_h )

    self:DoLayout()
end

function DraftChoicePopup:DoLayout()
    -- Layout things!
    self.card_container:StackChildrenRow( 20 )
    self.upgrade_info_label:LayoutBounds( "center", "bottom", self.panel )
        :Offset( 0, 30 )
    local container_y = (TheGame:FE():IsGamepadMode() and -40 or -20)
    container_y = container_y + ( self.upgrade_info_label:IsShown() and 20 or 0 )
    self.card_container:LayoutBounds( "center", "center", self.panel )
        :Offset( 0, container_y )
    if self.upgrade_info_label:IsShown() then
        self.money_btn:LayoutBounds( "center", "above", self.upgrade_info_label ):Offset( 0, 10 )
    else
        self.money_btn:LayoutBounds( "center", "bottom", self.panel ):Offset( 0, 30 )
    end
    for k,v in ipairs(self.card_container.children) do
        local card_x, card_y = v:GetPos()
        v.shadow:SetPos( card_x, 0 )
    end
    local grid_w, grid_h = self.grid:GetSize()
    self.shadow_container:LayoutBounds( "center", "above", 0, grid_h*0.2 )
    
    for k,v in ipairs(self.card_container.children) do
        
        if self.bounty_widgets[v] then
            self.bounty_widgets[v]:LayoutBounds("center", "above", v):Offset(0, 10)
        end

    end

end

function DraftChoicePopup:RefreshStreamerMode()
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

function DraftChoicePopup:GetDefaultFocus()
    if self.last_focus and self.last_focus:GetScreen() == self then
        return self.last_focus
    end

    return self.card_container and self.card_container.children and self.card_container.children[1]
end


function DraftChoicePopup:OnClose()
    DraftChoicePopup._base.OnClose( self )

    self.screen_loop:Stop()
    self.screen_loop:Release()
    self.screen_loop = nil
end

function DraftChoicePopup:DraftCards( agent, card_class, cards, on_end_fn, title_text, subtitle_text, no_skip )
    self.agent = agent or TheGame:GetGameState():GetPlayerAgent()
    self.on_end_fn = on_end_fn
    self.card_selection_done = false

    local draft_options =
    {
        cards = cards,
        no_skip = no_skip,
    }
    TheGame:BroadcastEvent( "card_draft", draft_options )

    if card_class == Negotiation.Card then
        cards = cards or NegotiationCardCollection.AllRewardableCards():GenerateRandom( 3, self.agent )
    else
        cards = cards or BattleCardCollection.AllRewardableCards():GenerateRandom( 3, self.agent )
    end
    self.cards = cards

    self.has_cards_with_upgrades = false

    if card_class == Negotiation.Card then
        self:ConfigureMetrics("DRAFT_NEGOTIATION_CARD", cards)
        self.panel:SetColour( UICOLOURS.NEGOTIATION )
            :SetIcon( engine.asset.Texture( "UI/card_type_negotiation.tex" ) )
        self.panel:SetTitles( 
            LOC"UI.DRAFT_SCREEN.NEGOTIATION_ADDCARD_TITLE", 
            loc.format( LOC"UI.DRAFT_SCREEN.PANEL_NEGOTIATION_SUBTITLE", #self.cards ) )
    elseif card_class == Battle.Card then
        self:ConfigureMetrics("DRAFT_BATTLE_CARD", cards)
        self.panel:SetColour( UICOLOURS.FIGHT )
            :SetIcon( engine.asset.Texture( "UI/card_type_fight.tex" ) )
        self.panel:SetTitles( 
            LOC"UI.DRAFT_SCREEN.BATTLE_ADDCARD_TITLE", 
            loc.format( LOC"UI.DRAFT_SCREEN.PANEL_COMBAT_SUBTITLE", #self.cards ) )
    else
        self:ConfigureMetrics("DRAFT_SPECIFIC_CARD", cards)
        self.panel:SetColour( UICOLOURS.SUBTITLE )
            :SetIcon( engine.asset.Texture( "UI/ic_shop_icon.tex" ) )
        self.panel:SetTitles( 
            LOC"UI.DRAFT_SCREEN.GENERIC_ADDCARD_TITLE", 
            loc.format( LOC"UI.DRAFT_SCREEN.PANEL_GENERIC_SUBTITLE", #self.cards ) )
    end

    if title_text or subtitle_text then
        self.panel:SetTitles( title_text, subtitle_text )
    end

    self.no_skip = draft_options.no_skip

    


    if self.no_skip then
        self.money_btn:Hide()
    else
        self:SetMoneyButton()
        self:AddRobotOption(self.money_btn)
    end

    -- Create cards
    for i, card in ipairs( self.cards ) do
        local widget
        StatUtil.PresentedCard(card.id)

        if card.is_negotiation then
            widget = Widget.NegotiationCard( card )
        else
            widget = Widget.BattleCard( card )
        end
        
        self:AddRobotOption(widget)
        
        widget:SetScale( CARD_SCALE )
        self.card_container:AddChild( widget )
        widget:SetTintAlpha( TheGame:GetLocalSettings().ROBOTICS and 1 or 0 )
        widget:ShowToolTipOnFocus()

        -- Add sub card info labels
        widget.show_class_label = true
        widget.sub_card_info = widget:AddChild( Widget.SubCardInfo( widget ) )
        widget.sub_card_info:SetHiddenBoundingBox( true )
        widget.sub_card_info:LayoutBounds( "center", "below", widget.bg ):Offset( 0, -8 )

        -- Check if there are upgrades
        if widget.card:HasUpgrades() then self.has_cards_with_upgrades = true end

        -- Handle events
        widget.OnGainFocus = function() widget._base.OnGainFocus(widget) self:UpdateCardImageState( widget, card.id ) self.last_focus = widget end
        widget.OnLoseFocus = function() widget._base.OnLoseFocus(widget) self:UpdateCardImageState( widget, card.id ) end
        widget.OnGainHover = function() widget._base.OnGainHover(widget) self:UpdateCardImageState( widget, card.id ) end
        widget.OnLoseHover = function() widget._base.OnLoseHover(widget) self:UpdateCardImageState( widget, card.id ) end

        -- Handle clicks
        widget:SetOnClick( function() self:OnCardClick( widget ) end )

        widget:SetOutline( card.is_negotiation and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT, true )

        -- Add a shadow
        widget.shadow = self.shadow_container:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
            :SetSize( 360, 70 )
            :SetTintColour( card.is_negotiation and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT )
            :SetTintAlpha( 0.8 )

        if TheGame:GetGameState() and TheGame:GetGameState():ShouldDoDraftBounties() then
            local bounty_level = TheGame:GetGameProfile():GetDraftBounty(card.id)
            local bonus = CalcBonus(bounty_level)

            if bonus > 0 then
                local bounty_widget = Widget.Label("title", 44, loc.format(LOC"UI.DRAFT_SCREEN.DRAFT_BOUNTY_LABEL", bonus))
                self.bounty_container:AddChild(bounty_widget)
                self.bounty_widgets[widget] = bounty_widget
                bounty_widget:SetToolTip(loc.format( LOC"UI.DRAFT_SCREEN.DRAFT_BOUNTY_TT", bounty_level, bonus))
                bounty_widget:SetTintAlpha(0)
            end
        end
    end

    -- Layout things!
    self:DoLayout()
    
    if not TheGame:GetLocalSettings().ROBOTICS then
        -- Prepare card animation
        local card_anim_duration_total = 0.3
        local card_number = math.min( #self.card_container.children, 14 )
        local card_anim_duration = 0.1
        local delay_per_card = card_anim_duration_total / card_number
        local card_adder = Updater.Parallel()
        for i, widget in ipairs( self.card_container.children ) do
            card_adder:Add( 
                Updater.Series{
                    Updater.Wait(delay_per_card*i),
                    Updater.Parallel{
                        Updater.Ease( function(v) widget:SetTintAlpha(v) end, 0, 1, card_anim_duration, easing.inQuad),
                        Updater.Ease( function(v) widget:SetScale(v,v) end, CARD_SCALE * 1.2, CARD_SCALE, card_anim_duration, easing.inQuad ),
                        Updater.Ease( function(v) if self.bounty_widgets[widget] then self.bounty_widgets[widget]:SetTintAlpha(v) end end, 0, 1, card_anim_duration, easing.inQuad)
                    }
                }
            )
        end
        -- Animate the cards
        self.fadein_anim = self:RunUpdater(
            card_adder
        )
    end
end

function DraftChoicePopup:HandleControlUp(control, device)
    local hotkey_idx = UIHelpers.FindHotkeyIdx( control )
    if hotkey_idx and self.card_container.children[ hotkey_idx ] then
        for k, widget in ipairs( self.card_container.children ) do
            if k == hotkey_idx then
                self:OnCardClick( widget )
                return true
            end
        end
    end
end

function DraftChoicePopup:PreviewUpgrade( upgrade_card, agent )
    if not self.coro then
        self.coro = self:StartCoroutine( self.PreviewUpgradeCoro, self, upgrade_card, agent )
    end
end

function DraftChoicePopup:PreviewUpgradeCoro( upgrade_card, agent )
    
    local preview_only = true
    local popup = Screen.UpgradeCardPopup( upgrade_card, agent, preview_only )
    TheGame:FE():PushScreen( popup )

    while TheGame:FE():FindScreen( popup ) do
        coroutine.yield()
    end

    self.coro = nil
end

function DraftChoicePopup:ConfigureMetrics(event_type, cards, money_amount)
	--print(debug.traceback())
	self.metrics_event_type = event_type
	self.metrics_cards_presented = cards 
	self.metrics_money_presented = money_amount
end

function DraftChoicePopup:ReportMetrics(selected_card_id)
    if self.metrics_event_type == nil then
        print( "DraftChoicePopup: metrics not configured for", selected_card_id )
        return
    end

	local selected_idx = -1
	local card_choices = {}

	if self.metrics_cards_presented then
		for num, card in ipairs(self.metrics_cards_presented) do
			table.insert(card_choices, {ID = card.id, XP = card.userdata.xp})

			if selected_card_id == card.id then
				selected_idx = num 
			end
		end
	end

	local json_t = {
		SELECTED_ID = selected_card_id,
		SELECTED_INDEX = selected_idx,
		CARD_CHOICES = card_choices,
		PLAYER_DATA = TheGame:GetGameState():GetPlayerState(),
	}
	if self.metrics_money_presented then
		json_t["MONEY"] = self.metrics_money_presented
	end

	SendMetricsData(self.metrics_event_type, json_t )
end

function DraftChoicePopup:SetMoneyButton()
    self.skip_shills_reward = TheGame:GetGameState():GetCardSkipReward()

    local title = LOC"UI.DRAFT_SCREEN.MONEY_BTN_TITLE"

    local subtitle, tt
    local is_daily = TheGame:GetGameState():GetGameType() == GAME_TYPE.DAILY 
    if is_daily then
        subtitle = loc.format( LOC"UI.DRAFT_SCREEN.MONEY_BTN_SUBTITLE_POINTS", Content.GetScoreComponent("SKIPPED_CARD_DRAFT").value )
        tt = loc.format( LOC"UI.DRAFT_SCREEN.MONEY_BTN_TT_POINTS", Content.GetScoreComponent("SKIPPED_CARD_DRAFT").value )
    else
        subtitle = loc.format( LOC"UI.DRAFT_SCREEN.MONEY_BTN_SUBTITLE", self.skip_shills_reward )
        tt = loc.format( LOC"UI.DRAFT_SCREEN.MONEY_BTN_TT", self.skip_shills_reward )
    end


    self.money_btn:SetFn( function( )
        if is_daily then
            if TheGame:GetGameState() then
                TheGame:GetGameState():AddScore("SKIPPED_CARD_DRAFT", 1)
            end
        else
            AUDIO:PlayEvent( "event:/ui/select_cards/aquired_money" )
            self.agent:DeltaMoney( self.skip_shills_reward )
        end
        TheGame:GetGameState():IncreaseCardSkipReward()

		self:ReportMetrics("MONEY")

        if self:IsOnStack() then self:GetFE():PopScreen( self ) end

        -- Ensure this happens after popping self, so we get more consistent screen stack unrolling.
        -- (Specifically, the LootScreen should close after we do)
        if self.on_end_fn then self.on_end_fn() end
    end )
    self.money_btn:Show()
    self.money_btn:SetIcon( global_images.shills )
    self.money_btn:SetText( title, subtitle )
    self.money_btn:SetToolTip( tt )
end

function DraftChoicePopup:UpdateCardImageState( widget, card_id )

    if widget.enabled and widget.selected then
        widget:SetOutline( UICOLOURS.WHITE, true )
        widget:ScaleTo( nil, CARD_SCALE_HOVER, 0.2, easing.outQuad )
        if widget.shadow then
            local r, g, b, a = HexColour( UICOLOURS.WHITE )
            widget.shadow:ColourFromCurrentTo( r, g, b, a, 0.2 )
            widget.shadow:ScaleTo( nil, 1.4, 0.2 )
        end
    elseif widget.enabled and ( widget.hover or widget.focus ) then
        widget:SetOutline( UICOLOURS.FOCUS, true )
        widget:ScaleTo( nil, CARD_SCALE_HOVER, 0.2, easing.outQuad )
        if widget.shadow then
            local r, g, b, a = HexColour( UICOLOURS.FOCUS )
            widget.shadow:ColourFromCurrentTo( r, g, b, a, 0.2 )
            widget.shadow:ScaleTo( nil, 1.4, 0.2 )
        end
    else
        widget:SetOutline( widget.card.is_negotiation and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT, true )
        widget:ScaleTo( nil, CARD_SCALE, 0.2, easing.outQuad )
        if widget.shadow then
            local r, g, b, a = HexColour( widget.card.is_negotiation and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT )
            a = 0.8
            widget.shadow:ColourFromCurrentTo( r, g, b, a, 0.2 )
            widget.shadow:ScaleTo( nil, 1, 0.2 )
        end
    end
end

function DraftChoicePopup:OnCardClick( widget )
    AUDIO:PlayEvent(SoundEvents.button_click)

    if not TheGame:GetLocalSettings().ROBOTICS and self.select_then_continue_mode then
        if self.selected_card == widget then
            -- This card was already selected, and was confirmed again. Just draft it
            self:OnClickContinue()
        else
            -- We're in select-then-continue mode, and the selected graft isn't the clicked one
            -- Select this one
            self.selected_card = widget
            -- Unselect options
            for k, widget in ipairs( self.card_container.children ) do
                widget.selected = ( widget == self.selected_card )
                self:UpdateCardImageState( widget )
            end
            -- And show the confirm button
            self.button_continue:Show()
        end
    elseif widget and self.selected_card == nil then
        -- Not select-then-continue mode, so let's just draft this one straight up!
        self.selected_card = widget
        self:OnClickContinue()
    end
end

-- Buys the self.selected_card card
function DraftChoicePopup:OnClickContinue( selected_card_widget )
    selected_card_widget = selected_card_widget or self.selected_card

    if self.fadein_anim and not self.fadein_anim:IsDone() then
        self.fadein_anim:Stop()
    end

    if selected_card_widget then
        -- Prevent clicks on other cards
        for k,w in ipairs(self.card_container.children) do
            w:SetOnClick( nil )
            w:SetOnRightClickFn( nil )
            w:SetToolTip( nil )
            w:SetToolTipClass( nil )
            w.hover_check = false
        end
        self.button_continue:SetFn( nil )

        if self.money_btn then 
            self.money_btn.hover_check = false 
            self.money_btn:SetFn( nil )
            self.money_btn:AlphaTo( 0, 0.2 )
        end

        -- Prepare cards fading out
        local card_anim_duration = 0.2
        local card_remover = Updater.Parallel()
        for i, widget in ipairs( self.card_container.children ) do
            local pos_x, pos_y = widget:GetPos()
            widget.hover_check = false
            widget.OnGainHover = nil
            widget.OnLoseHover = nil
            if widget ~= selected_card_widget then
                card_remover:Add( 
                    Updater.Series{
                        Updater.Do(function() if self.bounty_widgets[widget] then self.bounty_widgets[widget]:Hide() end end),
                        Updater.Parallel{
                            Updater.Ease( function(v) if widget.shadow then widget.shadow:SetTintAlpha(v) end end, 0.8, 0, card_anim_duration, easing.inQuad),
                            Updater.Ease( function(v) widget:SetTintAlpha(v) end, 1, 0, card_anim_duration, easing.inQuad),
                            Updater.Ease( function(v) widget:SetScale(v,v) end, CARD_SCALE, CARD_SCALE*1.1, card_anim_duration, easing.inQuad),
                            Updater.Ease2D( function(x,y) widget:SetPos(x,y) end, pos_x, pos_y, pos_x, pos_y-40, card_anim_duration, easing.inQuad)
                        }
                    }
                )
            else
                selected_card_widget:SetOutline( UICOLOURS.FOCUS )
            end
        end

        local pos_x, pos_y = selected_card_widget:GetPos()
        selected_card_widget:LayoutBounds( "center", "center", self.panel )
        local pos_x_end, pos_y_end = selected_card_widget:GetPos()
        selected_card_widget:SetPos( pos_x, pos_y )
        
        local card = selected_card_widget.card
        --AUDIO:PlayEvent( CARD_RARITY_SOUNDS[ card.rarity ] )
        --AUDIO:PlayEvent( CARD_TYPE_SOUNDS[ GetCardType( card )] )

        local done = false
        self:RunUpdater(
            Updater.Series{
                -- Remove and animate our fancy selected card
                Updater.Parallel{
                    card_remover,
                    Updater.Do( function() 
                        local x,y = selected_card_widget:GetWorldPos()
                        selected_card_widget:Hide()
                        if self.bounty_widgets[selected_card_widget] then 
                            self.bounty_widgets[selected_card_widget]:Hide() 
                        end

                        UIHelpers.ShowGainCards({card}, x, y, function() done = true end, 1.1)
                    end )

                },
                Updater.Until(function() return done end),
                Updater.Do( function() 
                    StatUtil.SelectedCard(card.id)
                    if card.is_negotiation then
                        self.agent.negotiator:AddCard( card, true )
                    else
                        self.agent.battler:AddCard( card, true )
                    end
                    self:ReportMetrics( card.id )

                    if TheGame:GetGameState():ShouldDoDraftBounties() then
                        for k,v in ipairs(self.cards) do
                            if v.id ~= card.id then
                                TheGame:GetGameProfile():IncreaseDraftBounty(v.id)
                            end
                        end
                        
                        local bounty_level = TheGame:GetGameProfile():GetDraftBounty(card.base_id or card.id)
                        local bonus = CalcBonus(bounty_level)
                        if bonus > 0 then
                            AUDIO:PlayEvent( "event:/ui/select_cards/aquired_money" )
                            self.agent:DeltaMoney( bonus )
                        end
                        TheGame:GetGameProfile():ClearDraftBounty(card.base_id or card.id)
                    end

                    if self:IsOnStack() then
                        self:GetFE():PopScreen( self )
                    end

                    -- Ensure this happens after popping self, so we get more consistent screen stack unrolling.
                    -- (Specifically, the LootScreen should close after we do)
                    if self.on_end_fn then self.on_end_fn() end
                end )
            }
        )
    end
end


local ReticuleMixin = require "ui/screens/ReticuleMixin"
ReticuleMixin:Mixin( DraftChoicePopup )
