
require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/solidbox"
require "ui/widgets/panelbutton"
require "ui/widgets/iconbutton"
require "ui/widgets/panel"
require "ui/widgets/slider"
require "ui/widgets/checkbox"

-------------------------------------------------------------------------
--

local PANEL_H = 680
local CARD_SCALE = 0.70
local CARD_SCALE_HOVER = 0.75

local DraftMulliganPopup= class("Screen.DraftMulliganPopup", Widget.Screen)

DraftMulliganPopup.CONTROL_MAP = 
{

    {
        control = Controls.Digital.GAMEPAD_A,
        
        fn = function(self)
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.TOGGLE_CARD", Controls.Digital.GAMEPAD_A ))
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_X,
        
        fn = function(self)
            self:OnClickContinue()
            return true
        end,
        
        hint = function(self, left, right)
            local repicks = table.count( self.shown_cards ) - #self.chosen_cards
            if repicks == 0 then
                table.insert(right, loc.format ( LOC"UI.CONTROLS.DRAFT_CARDS", Controls.Digital.GAMEPAD_X ))
            else
                table.insert(right, loc.format( LOC"UI.CONTROLS.DRAFT_MULLIGAN", Controls.Digital.GAMEPAD_X, repicks ))
            end
        end,
    },
    {
        control = Controls.Digital.VIEW_UPGRADES,
        
        test = function(self)
            return UIHelpers.GetFocusCard() and UIHelpers.GetFocusCard().card:HasUpgrades()
        end,        
        fn = function(self)
            self:PreviewUpgrade( UIHelpers.GetFocusCard().card, self.agent )
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.VIEW_UPGRADES", Controls.Digital.VIEW_UPGRADES))
        end,
    },

    {
        control = Controls.Digital.PAUSE, 
        fn = function(self)
            self:GetFE():InsertScreen(Screen.PauseMenu())
            return true
        end,
    },

}

function DraftMulliganPopup:init( agent, cards, mulligans, on_end_fn, title )
    self.agent = agent or TheGame:GetGameState():GetPlayerAgent()
    self.on_end_fn = on_end_fn
    self.title = title

    Widget.Screen.init(self)

    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors("center", "center")

    -- list of all possible cards.  Draft picks are taken from the tail. [CardEngine.Card, ... ]
    self.cards = cards

    -- Table of currently shown cards:  slot -> Widget.Card
    self.shown_cards = {}
    -- Array of selected cards (ones to mulligan) [CardEngine.Card, ... ]
    self.chosen_cards = {}

    self.mulligans = mulligans or 1

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Backing panel
    self.panel = self:AddChild( Widget.WidePanel( PANEL_H ) )
        :SetIcon( engine.asset.Texture( "UI/ic_shop_icon.tex" ) )

    local card_class = cards[1]._class

    if card_class == Negotiation.Card then
        self.panel:SetColour( UICOLOURS.NEGOTIATION )
            :SetIcon( engine.asset.Texture( "UI/card_type_negotiation.tex" ) )

    elseif card_class == Battle.Card then
        self.panel:SetColour( UICOLOURS.FIGHT )
            :SetIcon( engine.asset.Texture( "UI/card_type_fight.tex" ) )
    else
        self.panel:SetColour( UICOLOURS.SUBTITLE )
            :SetIcon( engine.asset.Texture( "UI/ic_shop_icon.tex" ) )
    end

    -- Card container
    self.card_container = self:AddChild( Widget() )

    self.upgrade_info_label = self:AddChild( Widget.Label( "body", 22 ) )
        :SetGlyphColour( UICOLOURS.OBJECTIVE )
        :SetAutoSize(800)
        :SetWordWrap(true)
        :SetText( LOC"UI.DRAFT_SCREEN.UPGRADES_PREVIEW_INFO" )
        :SetTintAlpha( 0.9 )
        :Bloom( 0.08 )


    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.button_continue = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.DRAFT_SCREEN.CONFIRM_PACK_BTN", function() self:OnClickContinue() end, 250 ))
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )

    self:SetSounds( nil, SoundEvents.card_select_screen_close )
    self:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
end

function DraftMulliganPopup:OnOpen()
    DraftMulliganPopup._base.OnOpen( self )

    self.screen_loop = AUDIO:CreateEventInstance(SoundEvents.card_select_screen_open)
    self.screen_loop:Start()
end

function DraftMulliganPopup:OnGainTop()
    DraftMulliganPopup._base.OnGainTop( self )

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
end

function DraftMulliganPopup:OnControlModeChange( cm, device_id )
    DraftMulliganPopup._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        for k, w in ipairs( self.card_container.children ) do
            w:SetOnClick( function() self:OnCardClick( w ) end )
        end

        self.bottom_right:SetShown( true )
        self.upgrade_info_label:Show()
        -- Unselect options
        for k, widget in ipairs( self.card_container.children ) do
            widget.selected = false
        end
        self.selected_card = nil
    end
    if cm == CONTROL_MODE.TOUCH then
        self.select_then_continue_mode = true
        for k, w in ipairs( self.card_container.children ) do
            w:SetOnClick( function() self:OnCardClick( w ) end )
        end
        self.bottom_right:SetShown( true )
        self.upgrade_info_label:Hide()
        -- Unselect options
        for k, widget in ipairs( self.card_container.children ) do
            widget.selected = false
        end
        self.selected_card = nil
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.select_then_continue_mode = false
        for k, w in ipairs( self.card_container.children ) do
            w:SetOnClick( function() self:OnCardClick( w ) end )
        end
        self.upgrade_info_label:Hide()
        self.bottom_right:Hide()
        -- Unselect options
        for k, widget in ipairs( self.card_container.children ) do
            widget.selected = false
        end
        self.selected_card = nil
    end

    self:DoLayout()
end

function DraftMulliganPopup:OnScreenModeChange( sm )
    DraftMulliganPopup._base.OnScreenModeChange( self, sm )

    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.upgrade_info_label:SetLayoutScale( LAYOUT_SCALE[sm] )
    
    local new_panel_h, new_cardcontainer_scale
    if sm == SCREEN_MODE.MONITOR then
        new_panel_h = PANEL_H
        new_cardcontainer_scale = 1
    elseif sm == SCREEN_MODE.SMALL
        or sm == SCREEN_MODE.TV then
        new_panel_h = PANEL_H + 130
        new_cardcontainer_scale = 1.2
    end
    self.card_container:SetLayoutScale( new_cardcontainer_scale )
    -- Resize container panel
    self.panel:SetHeight( new_panel_h )

    self:DoLayout()
end

function DraftMulliganPopup:DoLayout()
    self.card_container:StackChildrenRow( 20 )

    self.upgrade_info_label:LayoutBounds( "center", "bottom", self.panel )
        :Offset( 0, 30 )
    local container_y = 0 -- TheGame:FE():IsGamepadMode() and -30 or 0
    container_y = container_y + ( self.upgrade_info_label:IsShown() and -20 or -40 )
    self.card_container:LayoutBounds( "center", "center", self.panel )
        :Offset( 0, container_y )
end

function DraftMulliganPopup:RefreshStreamerMode()
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

function DraftMulliganPopup:GetDefaultFocus()
    return self.card_container and self.card_container.children and self.card_container.children[1]
end


function DraftMulliganPopup:OnClose()
    DraftMulliganPopup._base.OnClose( self )

    self.screen_loop:Stop()
    self.screen_loop:Release()
    self.screen_loop = nil
end

function DraftMulliganPopup:RefreshTitle()
    if self.mulligans <= 0 then
        self.panel:SetTitles( self.title, LOC"UI.DRAFT_SCREEN.NO_MULLIGANS_SUBTITLE" )

    else
        local subtitle = loc.format( LOC"UI.DRAFT_SCREEN.MULLIGAN_SUBTITLE", self.count, self.mulligans )
        self.panel:SetTitles( self.title, subtitle )  
    end
end

function DraftMulliganPopup:DraftCards( count )
    self.count = count

    self:RefreshTitle()

    for i = 1, count do
        local card = table.remove( self.cards )
        if card == nil then
            break
        end

        table.insert( self.chosen_cards, card )

        local widget
        if card.is_negotiation then
            widget = Widget.NegotiationCard( card )
        else
            widget = Widget.BattleCard( card )
        end
        self.shown_cards[i] = widget
        
        self:AddRobotOption(widget)
        
        widget:SetScale( CARD_SCALE )
        self.card_container:AddChild( widget )
        widget:SetTintAlpha( TheGame:GetLocalSettings().ROBOTICS and 1 or 0 )
        widget:ShowToolTipOnFocus()

        -- Add sub card info labels
        widget.hide_accept_hint = true
        widget.show_class_label = true
        widget.sub_card_info = widget:AddChild( Widget.SubCardInfo( widget ) )
        widget.sub_card_info:SetHiddenBoundingBox( true )
        widget.sub_card_info:LayoutBounds( "center", "below", widget.bg ):Offset( 0, -8 )

        -- Handle events
        widget.OnGainFocus = function() widget._base.OnGainFocus(widget) self:UpdateCardImageState( widget ) end
        widget.OnLoseFocus = function() widget._base.OnLoseFocus(widget) self:UpdateCardImageState( widget ) end
        widget.OnGainHover = function() widget._base.OnGainHover(widget) self:UpdateCardImageState( widget ) end
        widget.OnLoseHover = function() widget._base.OnLoseHover(widget) self:UpdateCardImageState( widget ) end

        -- Handle clicks
        widget:SetOnClick( function() self:OnCardClick( widget ) end )

        widget:SetOutline( card.is_negotiation and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT, true )
    end

    -- Layout things!
    self:DoLayout()
    
    if not TheGame:GetLocalSettings().ROBOTICS then
        -- Prepare card animation
        local card_anim_duration_total = 0.3
        local card_number = math.min( #self.card_container.children, 14 )
        local card_anim_duration = 0.1
        local delay_per_card = card_anim_duration_total / card_number
        local card_adder = Updater.Parallel()
        for i, widget in ipairs( self.card_container.children ) do
            card_adder:Add( 
                Updater.Series{
                    Updater.Wait(delay_per_card*i),
                    Updater.Parallel{
                        Updater.Ease( function(v) widget:SetTintAlpha(v) end, 0, 1, card_anim_duration, easing.inQuad),
                        Updater.Ease( function(v) widget:SetScale(v,v) end, CARD_SCALE * 1.2, CARD_SCALE, card_anim_duration, easing.inQuad )
                    }
                }
            )
        end
        -- Animate the cards
        self.fadein_anim = self:RunUpdater(
            card_adder
        )
    end
end


function DraftMulliganPopup:PreviewUpgrade( upgrade_card, agent )
    if not self.coro then
        self.coro = self:StartCoroutine( self.PreviewUpgradeCoro, self, upgrade_card, agent )
    end
end

function DraftMulliganPopup:PreviewUpgradeCoro( upgrade_card, agent )
    
    local preview_only = true
    local popup = Screen.UpgradeCardPopup( upgrade_card, agent, preview_only )
    TheGame:FE():PushScreen( popup )

    while TheGame:FE():FindScreen( popup ) do
        coroutine.yield()
    end

    self.coro = nil
end

function DraftMulliganPopup:UpdateCardImageState( widget )

    if widget.enabled and not table.contains( self.chosen_cards, widget.card ) then
        widget:SetOutline( 0xCC0000FF, true )
        widget:SetTintColour( 0.6, 0.6, 0.6, 1 )
    else
        widget:SetOutline( widget.card.is_negotiation and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT, true )
        widget:SetTintColour( 1, 1, 1, 1 )
    end

    if widget.enabled and ( widget.hover or widget.focus ) then
        widget:ScaleTo( nil, CARD_SCALE_HOVER, 0.2, easing.outQuad )
    else
        widget:ScaleTo( nil, CARD_SCALE, 0.2, easing.outQuad )
    end
end

function DraftMulliganPopup:ToggleCard( widget )
    if table.contains( self.chosen_cards, widget.card ) then
        table.arrayremove( self.chosen_cards, widget.card )
    else
        table.insert( self.chosen_cards, widget.card )
    end

    local repicks = table.count( self.shown_cards ) - #self.chosen_cards
    if repicks == 0 then
        self.button_continue:SetText( LOC"UI.DRAFT_SCREEN.CONFIRM_PACK_BTN" )
    else
        self.button_continue:SetText( loc.format( LOC"UI.DRAFT_SCREEN.CONFIRM_MULLIGAN_BTN", repicks ))
    end

    self:UpdateCardImageState( widget )
    self:GetFE():RefreshControlOptions()
end

function DraftMulliganPopup:OnCardClick( widget )
    if self.mulligans > 0 then
        AUDIO:PlayEvent(SoundEvents.button_click)

        if not TheGame:GetLocalSettings().ROBOTICS then
            self:ToggleCard( widget )

        elseif widget then
            self:OnClickContinue()
        end
    end
end

-- Buys the self.selected_card card
function DraftMulliganPopup:OnClickContinue()

    local repicks = table.count( self.shown_cards ) - #self.chosen_cards
    if self.mulligans == 0 or repicks == 0 then

        for i, card in ipairs( self.chosen_cards ) do
            if card.is_negotiation then
                self.agent.negotiator:AddCard( card )
            else
                self.agent.battler:AddCard( card )
            end
        end

        if self.on_end_fn then
            self.on_end_fn()
        end
        if self:IsOnStack() then
            self:GetFE():PopScreen( self )
        end

    else
        self.mulligans = self.mulligans - 1

        -- Remove unchosen cards.
        for i, widget in pairs( self.shown_cards ) do
            if not table.contains( self.chosen_cards, widget.card ) then
                local card = table.remove( self.cards )
                if card == nil then
                    widget:Remove()
                    self.shown_cards[ i ] = nil
                else
                    widget:RefreshCard( card )
                    self:ToggleCard( widget )
                    widget:ScaleTo( CARD_SCALE * 1.5, CARD_SCALE, 0.2, easing.outQuad )
                end
            end
        end

        self:RefreshTitle()
    end
end