
require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/solidbox"
require "ui/widgets/panelbutton"
require "ui/widgets/iconbutton"
require "ui/widgets/panel"
require "ui/widgets/slider"
require "ui/widgets/checkbox"
require "ui/widgets/cardpack"

-------------------------------------------------------------------------
--

local PANEL_H = 680
local CARD_SCALE = 0.70
local CARD_SCALE_HOVER = 0.75

local DraftPackPopup= class("Screen.DraftPackPopup", Widget.Screen)


DraftPackPopup.CONTROL_MAP = 
{

    {
        control = Controls.Digital.GAMEPAD_X,
        
        test = function(self)
            local focus = self:GetFE():GetFocusWidget()
            return self.card_container:IndexOf( focus ) ~= nil
        end,
        
        fn = function(self)
            self:OnClickPack( self:GetFE():GetFocusWidget() )
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.DRAFT_PACK", Controls.Digital.GAMEPAD_X ))
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_B,
        
        test = function(self)
             return not self.no_skip 
        end,
        
        fn = function(self)
            self.money_btn:Click()
            return true
        end,
        
        hint = function(self, left, right)
            local is_daily = TheGame:GetGameState():GetGameType() == GAME_TYPE.DAILY 
            if is_daily then
                table.insert(right, loc.format ( LOC"UI.CONTROLS.TAKE_POINTS", Controls.Digital.GAMEPAD_B, self.skip_shills_reward or 0 ))
            else
                table.insert(right, loc.format ( LOC"UI.CONTROLS.TAKE_CARD_MONEY", Controls.Digital.GAMEPAD_B, self.skip_shills_reward or 0 ))
            end
        end,
    },

    {
        control = Controls.Digital.PAUSE, 
        fn = function(self)
            self:GetFE():InsertScreen(Screen.PauseMenu())
            return true
        end,
    },

}

function DraftPackPopup:init()
    Widget.Screen.init(self)

    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors("center", "center")

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Backing panel
    self.panel = self:AddChild( Widget.WidePanel( PANEL_H ) )
        :SetIcon( engine.asset.Texture( "UI/ic_shop_icon.tex" ) )

    -- Card pack container
    self.card_container = self:AddChild( Widget() )

    self.select_then_continue_mode = false -- If true, you'll have to click a choice, then click the continue button

    -- Back button
    self.money_btn = self:AddChild( Widget.IconButton( "", function() 
        self.panel:FadeOut( function() self:GetFE():PopScreen( self ) end ) end, 230 ) )
    self.money_btn:SetIcon( global_images.close )
        :SetScale( 1.4, 1.4 )

    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.button_continue = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.DRAFT_SCREEN.CONFIRM_PACK_BTN", function() self:OnClickContinue() end ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Hide()
        :SetToolTip( LOC"UI.OPTIONS_SCREEN.CONFIRM_SELECT_TOOLTIP" )

    self:SetSounds( nil, SoundEvents.card_select_screen_close )
end

function DraftPackPopup:OnOpen()
    DraftPackPopup._base.OnOpen( self )

    self.screen_loop = AUDIO:CreateEventInstance(SoundEvents.card_select_screen_open)
    self.screen_loop:Start()

    self:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
end

function DraftPackPopup:OnControlModeChange( cm, device_id )
    DraftPackPopup._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.select_then_continue_mode = false or engine.inst:GetSetting("gameplay", "confirm_select", true)
        if not self.no_skip then self.money_btn:Show() end
        self.bottom_right:SetShown( self.select_then_continue_mode )
        self:SelectPack( nil )

    elseif cm == CONTROL_MODE.TOUCH then
        self.select_then_continue_mode = true
        if not self.no_skip then self.money_btn:Show() end
        self.bottom_right:SetShown( self.select_then_continue_mode )
        self:SelectPack( nil )

    elseif cm == CONTROL_MODE.GAMEPAD then
        self.select_then_continue_mode = false
        self.money_btn:Hide()
        self.bottom_right:Hide()
        self:SelectPack( nil )
    end

    self:DoLayout()
end


function DraftPackPopup:OnScreenModeChange( sm )
    DraftPackPopup._base.OnScreenModeChange( self, sm )

    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )
    
    local new_panel_h, new_cardcontainer_scale
    if sm == SCREEN_MODE.MONITOR then
        new_panel_h = PANEL_H
        new_cardcontainer_scale = 1
    elseif sm == SCREEN_MODE.SMALL
        or sm == SCREEN_MODE.TV then
        new_panel_h = PANEL_H + 130
        new_cardcontainer_scale = 1
    end
    self.card_container:SetLayoutScale( new_cardcontainer_scale )
    if self.money_btn then self.money_btn:SetLayoutScale( LAYOUT_SCALE[sm] ) end
    -- Resize container panel
    self.panel:SetHeight( new_panel_h )

    self:DoLayout()
end

function DraftPackPopup:SelectPack( pack )
    if self.selected_pack then
        self.selected_pack:SetOutline( nil )
    end

    self.selected_pack = pack
 
    if pack then
        pack:SetOutline( UICOLOURS.WHITE )
    end

    -- Selecting a pack implies a confirm button
    self.button_continue:SetShown( pack ~= nil )
end

function DraftPackPopup:DoLayout()
    -- Layout things!

    self.card_container:StackChildrenRow( 100 )

    self.card_container:LayoutBounds( "center", "center", self.panel )
        :Offset( 0, -80 )

    self.money_btn:LayoutBounds( "center", "bottom", self.panel ):Offset( 0, 30 )
end

function DraftPackPopup:RefreshStreamerMode()
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

function DraftPackPopup:GetDefaultFocus()
    return self.card_container and self.card_container.children and self.card_container.children[1]
end


function DraftPackPopup:OnClose()
    DraftPackPopup._base.OnClose( self )

    self.screen_loop:Stop()
    self.screen_loop:Release()
    self.screen_loop = nil
end

function DraftPackPopup:DraftPacks( agent, card_class, diff, on_end_fn, subtitle_text, rng, no_skip )
    self.agent = agent or TheGame:GetGameState():GetPlayerAgent()
    self.on_end_fn = on_end_fn
    self.card_selection_done = false

    local draft_options =
    {
        no_skip = no_skip,
    }
    TheGame:BroadcastEvent( "card_draft", draft_options )

    if rng then
        push_random( function( n, m )
            return rng:Random( n, m )
        end )
    end

    local packs
    if card_class == Negotiation.Card then
        packs = RewardUtil.GetNegotiationPacks( diff, 3, 3, self.agent )

        self.panel:SetColour( UICOLOURS.NEGOTIATION ):SetIcon( engine.asset.Texture( "UI/card_type_negotiation.tex" ) )
    else
        packs = RewardUtil.GetBattlePacks( diff, 3, 3, self.agent )

        self.panel:SetColour( UICOLOURS.FIGHT ):SetIcon( engine.asset.Texture( "UI/card_type_fight.tex" ) )
    end

    if rng then
        pop_random()
    end

    if card_class == Battle.Card then
        self.panel:SetTitles( LOC"UI.DRAFT_SCREEN.BATTLE_PACK_TITLE", subtitle_text )
    else
        self.panel:SetTitles( LOC"UI.DRAFT_SCREEN.NEGOTIATION_PACK_TITLE", subtitle_text )
    end

    self.no_skip = draft_options.no_skip

    if self.no_skip then
        self.money_btn:Hide()
    else
        self:SetMoneyButton()
        self:AddRobotOption(self.money_btn)
    end


    -- Create cards
    for i, pack in ipairs( packs ) do
        local widget = Widget.CardPack( pack )
        
        self:AddRobotOption(widget)
        
        self.card_container:AddChild( widget )

        -- Handle clicks
        widget:SetOnClick( function() self:OnClickPack( widget ) end )
    end

    -- Layout things!
    self:DoLayout()
end


function DraftPackPopup:SetMoneyButton()
    self.skip_shills_reward =  TheGame:GetGameState():GetCardSkipReward()

    local title = LOC"UI.DRAFT_SCREEN.MONEY_BTN_TITLE"
    local subtitle, tt
    local is_daily = TheGame:GetGameState():GetGameType() == GAME_TYPE.DAILY 
    
    if is_daily then
        subtitle = loc.format( LOC"UI.DRAFT_SCREEN.MONEY_BTN_SUBTITLE_POINTS", Content.GetScoreComponent("SKIPPED_CARD_DRAFT").value )
        tt = loc.format( LOC"UI.DRAFT_SCREEN.MONEY_BTN_TT_POINTS", Content.GetScoreComponent("SKIPPED_CARD_DRAFT").value )
    else
        subtitle = loc.format( LOC"UI.DRAFT_SCREEN.MONEY_BTN_SUBTITLE", self.skip_shills_reward )
        tt = loc.format( LOC"UI.DRAFT_SCREEN.MONEY_BTN_TT", self.skip_shills_reward )
    end

    self.money_btn:SetFn( function( )
        AUDIO:PlayEvent( "event:/ui/select_cards/aquired_money" )

        if is_daily then
            if TheGame:GetGameState() then
                TheGame:GetGameState():AddScore("SKIPPED_CARD_DRAFT", 1)
            end
        else
            self.agent:DeltaMoney( self.skip_shills_reward )
        end

        TheGame:GetGameState():IncreaseCardSkipReward()

        if self.on_end_fn then self.on_end_fn() end

        if self:IsOnStack() then self:GetFE():PopScreen( self ) end
    end )
    self.money_btn:Show()
    self.money_btn:SetIcon( global_images.shills )
    self.money_btn:SetText( title, subtitle )
    self.money_btn:SetToolTip( tt )
end

function DraftPackPopup:OnClickPack( widget )
    AUDIO:PlayEvent(SoundEvents.button_click)

    if not TheGame:GetLocalSettings().ROBOTICS and self.select_then_continue_mode then
        -- We're in select-then-continue mode, and the selected graft isn't the clicked one
        -- Select this one
        self:SelectPack( widget )

    elseif widget then
        -- Not select-then-continue mode, so let's just draft this one straight up!
        self:OnClickContinue( widget )
    end
end

-- Buys the self.selected_card card
function DraftPackPopup:OnClickContinue( selected_card_widget )
    selected_card_widget = selected_card_widget or self.selected_pack

    if is_instance( selected_card_widget, Widget.CardPack ) then
        for k,w in ipairs(self.card_container.children) do
            w:SetOnClick( nil )
            w.hover_check = false
        end

        if self.money_btn then 
            self.money_btn:Disable()
            self.money_btn:AlphaTo( 0, 0.2 )
        end

        for i, card in ipairs(selected_card_widget.pack) do
            if card.is_negotiation then
                self.agent.negotiator:AddCard( card )
            else
                self.agent.battler:AddCard( card )
            end
        end

        if self.on_end_fn then
            self.on_end_fn()
        end

        if self:IsOnStack() then
            self:GetFE():PopScreen( self )
        end
    end
end