
-- A screen where we select a few cards and it lays out a deck that it can export to a png, to use for promo materials
local ExportDeckScreen = class("Screen.ExportDeckScreen", Widget.Screen)

function ExportDeckScreen:init( card_id )
    Widget.Screen.init(self)
    
    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )
    
    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    
    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- A hitbox for alignment
    self.area_w = RES_X*0.8
    self.area_h = RES_Y*0.8
    
    -- The animation box
    self.animation_box = self:AddChild( Widget() )
        :SetPos( 0, 50)
    self.animation_box_frame = self.animation_box:AddChild( Widget.Panel( engine.asset.Texture("ui/empty_slot_frame.tex") ) )
        :SetNineSliceCoords( 30, 30, 44, 44)
        :SetSize( self.area_w, self.area_h )
        :SetScale( 1, -1 )
        :Bloom( 0.2 )
        :UnblockMouse()
    self.hitbox = self.animation_box:AddChild( Widget.SolidBox( self.area_w, self.area_h, 0xFF00FFFF ) )
        :Hide()
    self.anim_container = self.animation_box:AddChild( Widget() )

    self.top_left = self:AddChild( Widget() ):SetAnchors( "left", "top" )
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.bottom_center = self:AddChild( Widget() ):SetAnchors( "center", "bottom" )
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )

    -- Cards list
    self.cards_list = self.top_left:AddChild( Widget() )

    -- Back button
    self.close_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", function() self:DoClose() end ) )
    self.close_button:SetIcon( global_images.close )
    self.close_button:LayoutBounds( "left", "above", 60, 60 )
    self:RegisterHotkeyButton( Controls.Digital.MENU_CANCEL, self.close_button )

    -- Export button
    self.export_button = self.bottom_right:AddChild( Widget.IconButton( "Export image", function() self:ExportImage() end ) )
    self.export_button:SetIcon( engine.asset.Texture( "UI/ic_items_enabled.tex" ) )
    self.export_button:LayoutBounds( "right", "above", -60, 60 )

    -- Animation options
    self.show_glow = self.bottom_right:AddChild( Widget.CheckBox() )
        :SetText("Show glow")
        :SetFn( function(val) 
            self.opt_show_glow = val
            for k, v in ipairs( self.anim_container.children ) do
                if val then
                    v:SetOutline( UICOLOURS.FOCUS )
                else
                    v:SetOutline( nil )
                end
            end
        end )
        :SetToolTip( "If the card should show the focus-coloured glow around it" )
        :SetValue( false )
    self.show_chromakey = self.bottom_right:AddChild( Widget.CheckBox() )
        :SetText("Show magic pink")
        :SetFn( function(val) self.hitbox:SetShown( val ) end )
        :SetToolTip( "Shows a magic-pink background behind the card, for chroma-key screenshots" )
        :SetValue( false )
    self.easing_slider_label = self.bottom_right:AddChild( Widget.Label( "body", 22, "Position" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.easing_blend_ratio = 1
    self.easing_slider = self.bottom_right:AddChild( Widget.Slider(100) )
        :SetRange( 0, 1 )
        :SetStepSize( 0.05 )
        :SetFn( function(v)
            self.easing_blend_ratio = v
            self:LayoutDeck()
        end )
        :HideButtons()
        :SetValue(self.easing_blend_ratio,true)
    self.width_slider_label = self.bottom_right:AddChild( Widget.Label( "body", 22, "Width" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.max_width_mult = 1
    self.width_slider = self.bottom_right:AddChild( Widget.Slider(100) )
        :SetRange( 0, 3 )
        :SetStepSize( 0.05 )
        :SetFn( function(v)
            self.max_width_mult = v
            self:LayoutDeck()
        end )
        :HideButtons()
        :SetValue(self.max_width_mult,true)

    -- Card selection
    self.negotiation_label = self.bottom_center:AddChild( Widget.Label( "body", 22, "ADD NEGOTIATION CARDS" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.negotiation_selector = self.bottom_center:AddChild( Widget.DropDown() )
        :SetFn( function(...) self:SetCard(...) end )
        :SetDirectionUp()
        :SetToolTip( "All the Negotiation cards in the game, including upgrades" )
        :SetSize( 270, 40 )
    self.fight_label = self.bottom_center:AddChild( Widget.Label( "body", 22, "ADD BATTLE CARDS" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.fight_selector = self.bottom_center:AddChild( Widget.DropDown() )
        :SetFn( function(...) self:SetCard(...) end )
        :SetDirectionUp()
        :SetToolTip( "All the Battle cards in the game, including upgrades" )
        :SetSize( 270, 40 )

    self.fan_radius = 2000

    self:RefreshCardsSelection()
    self:Layout()
end

function ExportDeckScreen:RefreshCardsSelection()
    local battle_defs = require "battle/battle_defs"
    local battle_card_options = {}
    local battle_cards = BattleCardCollection():Filter( function( card_def ) return not CheckAnyBits( card_def.flags, battle_defs.CARD_FLAGS.SPECIAL | battle_defs.CARD_FLAGS.NPC) end )
    for k, card_def in ipairs( battle_cards.items ) do
        table.insert( battle_card_options, { txt = "" .. LOC( card_def:GetLocNameKey() ) .. " <b>[" .. card_def.id .. "]</>", card_def = card_def })
    end
    table.sort( battle_card_options, function( a, b ) return a.txt < b.txt end )
    self.fight_selector:SetOptions( battle_card_options )

    local negotiation_defs = require "negotiation/negotiation_defs"
    local negotiation_card_options = {}
    local negotiation_cards = NegotiationCardCollection():Filter( function( card_def ) return not CheckAnyBits( card_def.flags, negotiation_defs.CARD_FLAGS.SPECIAL | negotiation_defs.CARD_FLAGS.OPPONENT ) end )
    for k, card_def in ipairs( negotiation_cards.items ) do
        table.insert( negotiation_card_options, { txt = LOC( card_def:GetLocNameKey() ) .. " [" .. card_def.id .. "]", card_def = card_def })
    end
    table.sort( negotiation_card_options, function( a, b ) return a.txt < b.txt end )
    self.negotiation_selector:SetOptions( negotiation_card_options )

end

function ExportDeckScreen:SetCard( index, option )

    -- Create this card
    local card, card_widget
    if option.card_def._classname == "BattleCardDef" then
        -- Battle Card!
        card = Battle.Card( option.card_def.id )
        card_widget = self.anim_container:AddChild( Widget.BattleCard() )
            :RefreshCard( card )
    else
        -- Negotiation Card!
        card = Negotiation.Card( option.card_def.id )
        card_widget = self.anim_container:AddChild( Widget.NegotiationCard() )
            :RefreshCard( card )
    end

    if card_widget then
        -- Add button to remove this card from the deck
        local remove_btn = self.cards_list:AddChild( Widget.PanelButton( "Remove " .. (card_widget.card:GetName() or card_widget.card.id), nil, 280, 20, 30 ) )
            :SetScale(0.7)
        remove_btn:SetFn( function()
            -- When clicked, remove this button from the list
            remove_btn:Remove()
            -- Relayout the list
            self:LayoutButtonList()
            -- Remove the card from the deck
            card_widget:Remove()
            -- Relayout the deck
            self:LayoutDeck()
        end )
        if self.opt_show_glow then card_widget:SetOutline( UICOLOURS.FOCUS ) else card_widget:SetOutline( nil ) end
        self:LayoutButtonList()
        self:LayoutDeck()
    end

end

function ExportDeckScreen:OnScreenResize(w, h)
    self._base.OnScreenResize(self, w, h)
    self:Layout()
end

function ExportDeckScreen:Layout()
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Size up the grid
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Card selection
    self.negotiation_selector:LayoutBounds( "left", "above", 0, 60 ):Offset( 10, 5 )
    self.negotiation_label:LayoutBounds( "left", "above", self.negotiation_selector ):Offset( 0, 3 )
    self.fight_selector:LayoutBounds( "right", "above", 0, 60 ):Offset( -10, 5 )
    self.fight_label:LayoutBounds( "left", "above", self.fight_selector ):Offset( 0, 3 )

    -- Right controls
    self.show_glow:LayoutBounds( "before", "top", self.export_button ):Offset( -60, 15 )
    self.show_chromakey:LayoutBounds( "left", "below", self.show_glow ):Offset( 0, -6 )
    self.easing_slider:LayoutBounds( "before", "center", self.show_chromakey ):Offset( -20, 0 )
    self.easing_slider_label:LayoutBounds( "left", "above", self.easing_slider ):Offset( 0, 5 )
    self.width_slider:LayoutBounds( "before", "center", self.easing_slider ):Offset( -20, 0 )
    self.width_slider_label:LayoutBounds( "left", "above", self.width_slider ):Offset( 0, 5 )
end

function ExportDeckScreen:LayoutButtonList()
    self.cards_list:StackChildren(4)
        :LayoutBounds( "left", "top", 30, -60 )
end

function ExportDeckScreen:LayoutDeck()
    self.anim_container:ArrangeChildrenRow( 40 )
    for k, w in ipairs(self.anim_container.children) do
        local x, y, rot = self:FindCardPositionInArrangement( k, w )
        w:SetPos( x, y )
        w:SetRotation( rot )
    end
end

function ExportDeckScreen:GetArrangementWidth()
    local card_width = 140 -- Rough card width for purposes of hand layout, dictates spacing.
    local max_hand_width = 900
    local n = table.count( self.anim_container.children )
    return math.min( max_hand_width, n * card_width )*self.max_width_mult
end

function ExportDeckScreen:FindCardPositionInArrangement( idx, card_widget )
    local n = table.count( self.anim_container.children )
    local x, y, scale, rot

    if n > 1 then
        -- layout cards in 'slots' of identical width
        local max_width = self:GetArrangementWidth()
        local slot_width = max_width / n
        local x1 = easing.inCubic( idx, -slot_width/2, max_width, n )
        local x2 = easing.linear( idx, -slot_width/2, max_width, n )
        local blend_ratio = 1-self.easing_blend_ratio
        x = (blend_ratio*x1) + (1-blend_ratio)*x2
        x = x - max_width/2
    else
        card_widget:SetTintAlpha(1)
        x = 0
    end
    y = 0

    if self.fan_radius then
        x, y, rot = self:TransformToFan( x, y, self.fan_radius )
    else
        rot = 0
    end

    return x, y, rot
end

function ExportDeckScreen:TransformToFan( x, y, r )
    local n = table.count( self.anim_container.children )
    local max_width = self:GetArrangementWidth()
    local half_width = max_width*0.5

    local radius = math.sqrt( r^2 + half_width^2 )
    local theta_min = math.atan( r / half_width )
    local t = 1.0 - math.abs(x) / half_width
    local theta = Lerp( theta_min, math.pi/2, t )

    local tan_theta = math.tan( theta )
    local y1 = (radius + y) * tan_theta / math.sqrt( tan_theta^2 + 1 )
    local x1 = math.sqrt( (radius + y)^2 - y1^2 )
    if x < 0 then
        return -x1, y1 - r, math.pi/2 - theta
    elseif x == 0 then
        return 0, radius - r + y, 0
    else
        return x1, y1 - r, theta - math.pi/2
    end
end

function ExportDeckScreen:DoClose()
    self:GetFE():PopScreen( self )
end

function ExportDeckScreen:ShowAnimUI()
    self.animation_box_frame:AlphaTo( 1, 0.1 )
end

function ExportDeckScreen:HideAnimUI()
    self.animation_box_frame:AlphaTo( 0, 0.1 )
end

function ExportDeckScreen:ExportImage()
    self:StartCoroutine(ExportDeckScreen.ExportImageCoro, self)
end

function ExportDeckScreen:ExportImageCoro()

    local RENDER_SCALE = 3
    local render_w, render_h = self.animation_box_frame:GetSize()
    render_w = render_w*RENDER_SCALE/TheGame:FE():GetBaseWidgetScale()
    render_h = render_h*RENDER_SCALE/TheGame:FE():GetBaseWidgetScale()

    local scene = engine.rendering.Scene()
    local capture = Widget.CaptureImage(scene)
        :SetSize( render_w, render_h )
    local anim_container = self:AddChild( Widget() )

    -- Reparent cards
    while #self.anim_container.children > 0 do
        local w = self.anim_container.children[1]
        w:Reparent( anim_container )
        w:SetScale( 1, 1 )
        -- w:SetOutline( nil )
        w.shadow:Hide()
    end

    anim_container:SetScale( RENDER_SCALE, RENDER_SCALE )
    local root = engine.rendering.SceneGraphNode()
    root:SetScene(scene)
    root:AddChild(anim_container.node)

    coroutine.yield()
    coroutine.yield()
    coroutine.yield()
    self:Delay( 0.1 )

    while not engine.inst:IsAsyncLoadingComplete() do
        coroutine.yield()
    end

    capture:RenderToScene( anim_container, -render_w, -render_h, render_w, render_h )
    local time = os.time()
    engine.inst:SaveTextureToPNG( capture:GetTexture(), "../image_exports/deck_" .. time .. ".png" );

    -- Reparent cards
    anim_container:SetAnchors( "center", "center" )
        :SetScale( 1, 1 )
    while #anim_container.children > 0 do
        local w = anim_container.children[1]
        w:Reparent( self.anim_container )
        -- w:SetOutline( self.opt_show_glow and UICOLOURS.FOCUS or nil )
        w.shadow:Show()
    end

    anim_container:Remove()

    -- Make sure to call collect garbage a lot so that capture images get their resources freed:
    collectgarbage()


    TheGame:FE():PushScreen( Screen.InfoPopup( "CARD IMAGE GENERATED", "Deck image saved to Griftlands/image_exports/deck_".. time .. ".png" ) )

end