require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/editlabel"


local W, H = 1400, 740
local FeedbackScreen = class("Screen.FeedbackScreen", Widget.Screen)
local TEXT_W = 640


local emotion_data = 
{
    {img = engine.asset.Texture("UI/very_bad.tex"), txt = "FEEDBACK.VERY_BAD", val = -2},
    {img = engine.asset.Texture("UI/bad.tex"), txt = "FEEDBACK.BAD", val = -1},
    {img = engine.asset.Texture("UI/neutral.tex"), txt = "FEEDBACK.NEUTRAL", val = 0},
    {img = engine.asset.Texture("UI/good.tex"), txt = "FEEDBACK.GOOD", val = 1},
    {img = engine.asset.Texture("UI/very_good.tex"), txt = "FEEDBACK.VERY_GOOD", val = 2},
}


--- TIME DOES NOT PASS WHILE THIS SCREEN IS UP! DO NOT ANIMATE ANYTHING!!!
function FeedbackScreen:init(gamestate, screen_shot_texture)
    FeedbackScreen._base.init(self)
    self.is_popup = true
    self:SetupUnderlay( false )
    self:SetAnchors( "center", "center" )

    self.send_save_file = true
    self.send_screenshot = true
    self.send_log = true

    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Setup the grid
    -- self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    -- self.grid:SetTintColour( UICOLOURS.ACCENT )
    -- self.grid:SetTintAlpha( 0.8 )
    -- local target_w = self.screen_w*1.1
    -- local g_w, g_h = self.grid:GetSize();
    -- local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    -- self.grid:SetSize( target_w, target_h )
    -- self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- A container for all the background elements
    self.panel = self:AddChild( Widget() )
    self.panel_glow_top = self.panel:AddChild( Widget.Image( engine.asset.Texture( "images/glow.tex" ), 1900, 500 ) )
    self.panel_glow_top:SetHiddenBoundingBox( true )
    self.panel_glow_bottom = self.panel:AddChild( Widget.Image( engine.asset.Texture( "images/glow.tex" ), 1500, 500 ) )
    self.panel_glow_bottom:SetHiddenBoundingBox( true )
    self.panel_bg = self.panel:AddChild( Widget.Image( engine.asset.Texture( "large/dialog_bg.tex" ) ) )
    self.panel_grid = self.panel:AddChild( Widget.Image( engine.asset.Texture( "large/dialog_grid.tex" ) ) )
    self.panel_border_top = self.panel:AddChild( Widget.SolidBox( RES_X, 6, 0xffffffff ) )
    self.panel_border_bottom = self.panel:AddChild( Widget.SolidBox( RES_X, 3, 0xffffffff ) )

    -- Set background colours
    self.panel_bg:SetTintAlpha( 0.98 )
    self.panel_grid:Pulse( 0.4, 1.0, 0.003 )
    self.panel_glow_top:SetTintColour( UICOLOURS.SUBTITLE )
    self.panel_glow_top:SetTintAlpha( 0.4 )
    self.panel_glow_bottom:SetTintColour( UICOLOURS.SUBTITLE )
    self.panel_glow_bottom:SetTintAlpha( 0.3 )
    self.panel_border_top:SetTintColour( UICOLOURS.SUBTITLE )
    self.panel_border_bottom:SetTintColour( UICOLOURS.SUBTITLE )

    -- Hitbox for content alignment
    self.hitbox = self:AddChild( Widget.SolidBox( W, H, 0xff000030 ))
        :Hide()

    -- Calculate screenshot size
    local sw, sh = screen_shot_texture:GetSize()
    local img_scale = 700 / sw
    if sh*img_scale > 400 then 
        img_scale = 400 / sh
    end

    -- Titles
    self.title = self:AddChild( Widget.Label("title", 40 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE ):Bloom(0.1)
    self.about = self:AddChild( Widget.Label("title", 24 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.6 )
        :Bloom(0.1)
        :SetWordWrap(true)
        :SetAutoSize(W-30)
    -- Update titles
    self.title:SetText( LOC"UI.FEEDBACK_SCREEN.TITLE" )
    self.about:SetText( LOC"UI.FEEDBACK_SCREEN.ABOUT" )

    -- Form container widget
    self.content = self:AddChild(Widget())

    -- Form contents
    self.name_prompt = self.content:AddChild( Widget.Label("title", 24, string.upper( LOC"UI.FEEDBACK_SCREEN.NAME_PROMPT" )))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
    local name = TheGame:GetGameProfile():GetFeedbackName() or ""
    self.name_inputbox = self.content:AddChild( Widget.EditLabel(28, TEXT_W, 1):SetString(name):SetFn(function() self:RefreshSendButton() end) )

    self.short_prompt = self.content:AddChild( Widget.Label("title", 24, string.upper( LOC"UI.FEEDBACK_SCREEN.SHORT_PROMPT" )))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
    self.short_inputbox = self.content:AddChild( Widget.EditLabel(28, TEXT_W, 1) )

    self.prompt = self.content:AddChild( Widget.Label("title", 24, string.upper( LOC"UI.FEEDBACK_SCREEN.PROMPT")))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
    self.inputbox = self.content:AddChild( Widget.EditLabel(28, TEXT_W, 10):SetString() )

    self:SetTabLoop( { self.name_inputbox, self.short_inputbox, self.inputbox } )

    self.img_prompt = self.content:AddChild( Widget.Label("title", 24, string.upper( LOC"UI.FEEDBACK_SCREEN.IMG_PROMPT" )))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
    self.img = self.content:AddChild(Widget.Image(screen_shot_texture):SetSize(sw*img_scale, sh*img_scale)):Offset(0, -20)
    
    local EMO_SZ = 150
    self.emotion_buttons = {}

    self.emotion_highlight = self:AddChild( Widget() )
        :Hide()
    self.emotion_highlight.wheel1 = self.emotion_highlight:AddChild( Widget.Image( engine.asset.Texture( "UI/arguments_wheel.tex" ) ) )
        :SetTintColour(UICOLOURS.SUBTITLE)
        :SetSize(EMO_SZ+20, EMO_SZ+20)
        :SetTintAlpha(0.7)
        :Bloom( 0.2 )
        :RotateIndefinitely( 0.02 )
    self.emotion_highlight.wheel2 = self.emotion_highlight:AddChild( Widget.Image( engine.asset.Texture( "UI/arguments_wheel.tex" ) ) )
        :SetTintColour(UICOLOURS.SUBTITLE)
        :SetSize(EMO_SZ+20, EMO_SZ+20)
        :SetTintAlpha(0.9)
        :Bloom( 0.3 )
        :RotateIndefinitely( -0.035 )
    self.emotion_button_root = self.content:AddChild( Widget())

    local categories = {
        "GAMEPLAY",
        "STORY",
        "UI",
        "GRAPHICS",
        "AUDIO",
        "OTHER",
    }

    self.attachments_root = self.content:AddChild( Widget() )
    self.attachments_root:AddChild( Widget.CheckBox():SetText(LOC"UI.FEEDBACK_SCREEN.SEND_LOG"):SetFn(function(val) self.send_log = val end) ):SetValue(true)
    self.attachments_root:AddChild( Widget.CheckBox():SetText(LOC"UI.FEEDBACK_SCREEN.SEND_SCREENSHOT"):SetFn(function(val) self.send_screenshot = val end) ):SetValue(true)
    if gamestate then
        self.attachments_root:AddChild( Widget.CheckBox():SetText(LOC"UI.FEEDBACK_SCREEN.SEND_SAVE"):SetFn(function(val) self.send_save_file = val end) ):SetValue(true)
    end

    self.categories_root = self.content:AddChild( Widget() )
    self.category_checks = {}
    for k,v in ipairs( categories ) do
        local cat = v
        self.category_checks[v] = self.categories_root:AddChild( Widget.CheckBox():SetText(LOC("UI.FEEDBACK_SCREEN.CATEGORIES." .. v)):SetFn(function(val) self:SetCategory(cat) end) ):LayoutBounds("left", "below"):Offset(0, -10)
    end
    for k,v in ipairs(emotion_data) do
        local w = self.emotion_button_root:AddChild(Widget.ImageButton(v.img):SetSize(EMO_SZ, EMO_SZ):SetOnClickFn(function() self:SetEmotion(k) end):SetToolTip(LOC(v.txt)))
        table.insert(self.emotion_buttons, w)
        w:LayoutBounds("after", "center"):Offset(30, 0):SetTintColour(v.col or 0xffffffff)
    end


    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.close_btn = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.FEEDBACK_SCREEN.CANCEL", 
        function()
            AUDIO:PlayEvent( "event:/ui/feedback_menu/close" )
            engine.inst:CancelFeedback();
            self:GetFE():PopScreen( self )
        end ) )
    self.close_btn:SetIcon( global_images.close )
    self.close_btn:LayoutBounds( "left", "above", 60, 60 )
    self:RegisterHotkeyButton( Controls.Digital.MENU_CANCEL, self.close_btn )

    -- Submit button
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.send_btn = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.FEEDBACK_SCREEN.SUBMIT", function() 
        self:SendFeedback()
    end ) )
    self.send_btn:SetIcon( global_images.accept )
    self.send_btn:LayoutBounds( "right", "above", -60, 60 )

    self:Layout()

    self:SetCategory("OTHER")
    self:SetEmotion(3)

    AUDIO:DumpDebugInfo()
    self:RefreshSendButton()

    -- Play screen sound
end

function FeedbackScreen:OnGainTop()
    FeedbackScreen._base.OnGainTop( self )
    self.short_inputbox:SetFocus()
end

function FeedbackScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    elseif GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

function FeedbackScreen:OnClose()
    FeedbackScreen._base.OnClose(self)

    -- Stop the screen sound
    self.screen_loop:Stop()
    self.screen_loop:Release()
    self.screen_loop = nil
    
    AUDIO:PlayEvent( "event:/ui/feedback_menu/close" )

    --resume time. Maybe clear out the screenshot?
    TheGame:SetWallTimeScale(1)
end

function FeedbackScreen:SendFeedback()
    TheGame:SetWallTimeScale(1)
    TheGame:GetGameProfile():SetFeedbackName(self.name_inputbox:GetString())
    local gamestatus = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerStateString() or "---"

    local deckstate = TheGame:GetGameState() and TheGame:GetGameState():GetDeckStateString() or "---"
    engine.inst:CreateFeedback(self.short_inputbox:GetString() or "", self.inputbox:GetString() or "", self.name_inputbox:GetString() or "Anonymous", self.category, gamestatus, deckstate, self.emotion_idx and emotion_data[self.emotion_idx] and emotion_data[self.emotion_idx].val, self.send_save_file, self.send_screenshot, self.send_log)
    self:GetFE():PopScreen( self )
    TheGame:FE():PushScreen(Screen.InfoPopup( LOC"UI.FEEDBACK_SCREEN.SUBMITTED_TITLE", LOC"UI.FEEDBACK_SCREEN.SUBMITTED_BODY"))
end

function FeedbackScreen:Layout()
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Size up the background
    self.panel_bg:SetSize( self.screen_w, H )
    local grid_w, grid_h = self.panel_grid:GetSize()
    self.panel_grid:SetSize( grid_w * H/grid_h,  H )
    self.panel_border_top:SetSize( self.screen_w, 3 )
    self.panel_border_bottom:SetSize( self.screen_w, 2 )

    -- Layout the background
    self.panel_border_top:LayoutBounds( nil, "above", self.panel_bg ):Offset( 0, -1 )
    self.panel_glow_top:LayoutBounds( nil, "top", self.panel_bg ):Offset( 0, 150 )
    self.panel_glow_bottom:LayoutBounds( nil, "bottom", self.panel_bg ):Offset( 0, -130 )
    self.panel_border_bottom:LayoutBounds( nil, "below", self.panel_bg ):Offset( 0, 1 )

    -- Layout the glows
    self.panel_glow_top:LayoutBounds( nil, "top", self.panel_bg ):Offset( 0, 150 )
    self.panel_glow_bottom:LayoutBounds( nil, "bottom", self.panel_bg ):Offset( 0, -130 )
    self.panel_grid:LayoutBounds( "right", "center", self.panel_bg )

    -- Layout titles
    self.about:LayoutBounds( "center", "above", self.panel_bg ):Offset( 0, 15 )
    self.title:LayoutBounds( "center", "above", self.about ):Offset( 0, -4 )

    -- Layout form content
    self.name_prompt:LayoutBounds( "left", "top", self.hitbox ):Offset( 0, -40 )
    self.name_inputbox:LayoutBounds("left", "below", self.name_prompt):Offset(0, -5)

    self.short_prompt:LayoutBounds("left", "below", self.name_inputbox):Offset(0, -15)
    self.short_inputbox:LayoutBounds("left", "below", self.short_prompt):Offset(0, -5)

    self.prompt:LayoutBounds("left", "below", self.short_inputbox):Offset(0, -20)
    self.inputbox:LayoutBounds("left", "below", self.prompt):Offset(0, -5)
    self.img:LayoutBounds("after", "top", self.name_inputbox):Offset(60, 0)
    self.img_prompt:LayoutBounds("left", "above", self.img):Offset(0, 5)
    self.emotion_button_root:LayoutBounds("left", "below", self.inputbox):Offset(0, -40)
    self.attachments_root:StackChildrenRow( 30 )
    self.attachments_root:LayoutBounds("left", "below", self.img):Offset( 0, -10 )
    self.categories_root:LayoutBounds("after", "top", self.img):Offset(20, 0)

end

function FeedbackScreen:OnOpen()
    self._base.OnOpen( self )
    
    self.screen_loop = AUDIO:CreateEventInstance("event:/ui/main/fld/feedback_menu_open")
    self.screen_loop:Start()

end


function FeedbackScreen:RefreshSendButton()
    if self.name_inputbox:GetString() == "" then
        self.send_btn:Disable():SetToolTip("Please enter a name first!")
    else
        self.send_btn:Enable():SetToolTip()
    end
end

function FeedbackScreen:SetCategory(cat)
    for k,v in pairs(self.category_checks) do
        v:SetValue( k == cat, true )
    end
    self.category = cat
end

function FeedbackScreen:SetEmotion(idx)
    if idx and self.emotion_buttons[idx] then
        self.emotion_highlight:LayoutBounds( "center", "center", self.emotion_buttons[idx] )
        self.emotion_highlight:Show()
        self.emotion_idx = idx
    end
end


function FeedbackScreen:HandleControlUp(control, device)
    if control:Has( Controls.Digital.MENU_CANCEL ) then
        self:GetFE():PopScreen( self )
        return true

    elseif control:Has( Controls.Digital.MENU_SUBMIT ) then
        self:SendFeedback()
        return true
    end

    return false
end
