require "ui/widgets/installed_grafts"

local FlourishScreen = class( "Screen.FlourishScreen", Widget.Screen )

local cell_scale = 0.85
local BUTTON_W = cell_scale*260
local BUTTON_H = cell_scale*168
local CONTAINER_H = 600

FlourishScreen.CONTROL_MAP = Widget.MergeControlMap( Widget.Screen.CONTROL_MAP,
{
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self) 
            self:DoClose()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL) )
        end,
    },
})

function FlourishScreen:init( owner, on_close )
    FlourishScreen._base.init( self )
    self.on_close = on_close

    self.owner = owner or TheGame:GetGameState():GetPlayerAgent()

    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.FLOURISH )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Backing
    self.widepanel = self:AddChild( Widget.WidePanel( 800, UICOLOURS.FLOURISH ) )

    -- Header
    self.header = self:AddChild( Widget() ):SetAnchors( "left", "top" )
    self.title = self.header:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_TITLE ) )
        :SetText( loc.format( LOC"UI.FLOURISH_SCREEN.FLOURISH_SCREEN_TITLE", self.owner:GetName() ) )
        :SetGlyphColour( UICOLOURS.FLOURISH )
        :SetAutoSize( RES_X*0.6 )
        :SetWordWrap( true )
        :Bloom( 0.1 )
        :LayoutBounds( "left", "top", 0, -60 )
    self.subtitle = self.header:AddChild( Widget.Label( "title", FONT_SIZE.BODY_TEXT ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetText( loc.format( LOC"UI.FLOURISH_SCREEN.FLOURISH_SCREEN_SUBTITLE", 0 ) )
        :SetTintColour( UICOLOURS.FLOURISH_LIGHT )
        :SetHoverLinkGlyphColour( UICOLOURS.FLOURISH_LIGHT )
        :SetHoverLinkFn( function(...) self:HandleBossCompendiumKeywordHover(...) end )
        :SetClickLinkFn( function(...) self:HandleBossCompendiumKeywordClick(...) end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "center", "below", w ):Offset( 0, -SPACING.M1 ) end )
        :LeftAlign()
        :SetAutoSize( RES_X*0.6 )
        :SetWordWrap( true )
        :Bloom( 0.15 )
        :LayoutBounds( "left", "below", self.title ):Offset( 0, 0 )
    
    -- List scroll
    self.flourish_scroll = self:AddChild( Widget.ScrollPanel() )
        :SetVirtualTopMargin( 10 )
        :SetVirtualBottomMargin( 10 )
        :SetScrollBarOuterMargin( 0 )
        :SetSize( 600, CONTAINER_H )
    -- Add Battle flourishes
    self.battle_title = self.flourish_scroll:AddScrollChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE ) )
        :SetText( LOC"UI.FLOURISH_SCREEN.BATTLE_FLOURISHES" )
        :SetGlyphColour( UICOLOURS.FIGHT )
        :Bloom( 0.1 )
    self.battle_container = self.flourish_scroll:AddScrollChild( Widget() )
    -- Add Negotiation flourishes
    self.negotiation_title = self.flourish_scroll:AddScrollChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE ) )
        :SetText( LOC"UI.FLOURISH_SCREEN.NEGOTIATION_FLOURISHES" )
        :SetGlyphColour( UICOLOURS.NEGOTIATION )
        :Bloom( 0.1 )
    self.negotiation_container = self.flourish_scroll:AddScrollChild( Widget() )

    -- Setup character anim display
    self.character = self:AddChild( Widget.AnimatedCharacter() )
        :Refresh( self.owner )
    self.character:SetHoverOutlineColour( UICOLOURS.FLOURISH )
    self.character_animation_done = true
    self.character.anim:SetAnimDoneFn( function() self.character_animation_done = true end )
    self.character:SetClickFn( function()
        if self.character_animation_done then
            self.character_animation_done = false
            self.character:Emote( "handwring", true )
        end
    end )
    self.character:SetPos( -RES_X*0.35, -250 )
    self.character:Emote( "handwring", true )
    
    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.close_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", function() self:DoClose() end ) )
        :SetIcon( global_images.close )
        :LayoutBounds( "left", "above", 60, 60 )
    
    self:AddRobotOption(self.close_button)
    self:Refresh()
end

function FlourishScreen:DoClose()
    AUDIO:PlayEvent( "event:/ui/grafts_menu/close" )
    self:FadeOut()
end

function FlourishScreen:HandleBossCompendiumKeywordHover( widget, tag, text, ... )
    if tag and tag=="boss_compendium" then
        widget:SetToolTip( LOC"UI.FLOURISH_SCREEN.FLOURISH_SCREEN_BOSSCOMPENDIUM_TOOLTIP" )
    else
        widget:SetToolTip( nil )
    end
    return self
end

function FlourishScreen:HandleBossCompendiumKeywordClick( widget, tag, text )
    if tag and tag=="boss_compendium" then
        TheGame:FE():InsertScreen( Screen.Compendium( 3 ) )
    end
    return self
end

function FlourishScreen:Refresh()

    -- Get current character
    self.owner = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent()

    -- Remove old widgets
    self.battle_container:DestroyAllChildren()
    self.negotiation_container:DestroyAllChildren()

    -- Get flourishes for this character
    self.flourishes_battle = Content.GetAllBattleFlourishes( self.owner.id )
    self.flourishes_negotiation = Content.GetAllNegotiationFlourishes( self.owner.id )
    self.flourishes_battle_unlocked = TheGame:GetGameProfile():GetUnlockedFlourishes( self.owner.id, "BATTLE" )
    self.flourishes_negotiation_unlocked = TheGame:GetGameProfile():GetUnlockedFlourishes( self.owner.id, "NEGOTIATION" )

    -- Add base flourishes
    local battle_base_defs = {}
    for k, card_def in ipairs( self.flourishes_battle ) do
        if not card_def.base_id then
            local widget = self.battle_container:AddChild( Widget.FlourishColumnWidget( "BATTLE", card_def, function(...) self:OnFlourishUnlocked(...) end ) )
            battle_base_defs[card_def.id] = widget
        end
    end
    local negotiation_base_defs = {}
    for k, card_def in ipairs( self.flourishes_negotiation ) do
        if not card_def.base_id then
            local widget = self.negotiation_container:AddChild( Widget.FlourishColumnWidget( "NEGOTIATION", card_def, function(...) self:OnFlourishUnlocked(...) end ) )
            negotiation_base_defs[card_def.id] = widget
        end
    end
    -- Send the base ones to the start of the list, so they show first
    for k, row in ipairs( self.battle_container.children ) do
        if row:GetDef().basic_flourish then
            -- Send widget to left
            row:SendToBack()
            -- Also make sure they're unlocked
            table.insert_unique( self.flourishes_battle_unlocked, row:GetDef().id )
        end
    end
    for k, row in ipairs( self.negotiation_container.children ) do
        if row:GetDef().basic_flourish then
            -- Send widget to left
            row:SendToBack()
            -- Also make sure they're unlocked
            table.insert_unique( self.flourishes_negotiation_unlocked, row:GetDef().id )
        end
    end
    -- Add upgrades
    for k, card_def in ipairs( self.flourishes_battle ) do
        if card_def.base_id then
            battle_base_defs[card_def.base_id]:AddUpgrade( card_def )
        end
    end
    for k, card_def in ipairs( self.flourishes_negotiation ) do
        if card_def.base_id then
            negotiation_base_defs[card_def.base_id]:AddUpgrade( card_def )
        end
    end

    -- Get available prestige points
    self.prestige_points = TheGame:GetGameProfile():GetPrestigePoints()
    self.subtitle:SetText( loc.format( LOC"UI.FLOURISH_SCREEN.FLOURISH_SCREEN_SUBTITLE", self.prestige_points ) )
        :LayoutBounds( "left", "below", self.title ):Offset( 0, 0 )

    -- Update rows
    for k, row in ipairs( self.battle_container.children ) do
        row:SetCurrentPoints( self.prestige_points )
            :SetUnlockedUpgrades( self.flourishes_battle_unlocked )
    end
    for k, row in ipairs( self.negotiation_container.children ) do
        row:SetCurrentPoints( self.prestige_points )
            :SetUnlockedUpgrades( self.flourishes_negotiation_unlocked )
    end

    self:Layout()

    return self
end

function FlourishScreen:OnFlourishUnlocked( card_id, flourish_type, name, cost, widget )
    -- Confirm acquisition
    TheGame:FE():PushScreen( 
        Screen.YesNoPopup( LOC"UI.FLOURISH_SCREEN.CONFIRM_TITLE", loc.format(LOC"UI.FLOURISH_SCREEN.CONFIRM_BODY", name, cost) )
            :SetFn(
                function(res) 
                    if res == Screen.YesNoPopup.YES then

                        -- Acquire upgrade
                        TheGame:GetGameProfile():UnlockFlourish( self.owner.id, flourish_type, card_id )
                        AUDIO:PlayEvent("event:/ui/mettle_shop/buy_flourishcard")
                        -- Remove mettle
                        TheGame:GetGameProfile():SpendPrestigePoints( cost )

                        -- Get available mettle
                        self.prestige_points = TheGame:GetGameProfile():GetPrestigePoints()
                        self.subtitle:SetText( loc.format( LOC"UI.FLOURISH_SCREEN.FLOURISH_SCREEN_SUBTITLE", self.prestige_points ) )
                            :LayoutBounds( "left", "below", self.title ):Offset( 0, 0 )

                        -- Get updated flourishes
                        self.flourishes_battle_unlocked = TheGame:GetGameProfile():GetUnlockedFlourishes( self.owner.id, "BATTLE" )
                        self.flourishes_negotiation_unlocked = TheGame:GetGameProfile():GetUnlockedFlourishes( self.owner.id, "NEGOTIATION" )

                        -- Refresh all rows
                        for k, row in ipairs( self.battle_container.children ) do
                            row:SetCurrentPoints( self.prestige_points )
                                :SetUnlockedUpgrades( self.flourishes_battle_unlocked )
                        end
                        for k, row in ipairs( self.negotiation_container.children ) do
                            row:SetCurrentPoints( self.prestige_points )
                                :SetUnlockedUpgrades( self.flourishes_negotiation_unlocked )
                        end

                    end
                end
            )
        )
end

function FlourishScreen:OnOpen()
    FlourishScreen._base.OnOpen( self )

    local anim_duration = 0.7
    local horizontal_movement = -90

    -- Prepare for animation by getting current values
    local content_x, content_y = self.content:GetPos()

    -- Hide objects until they're animated in
    self.content:SetTintAlpha(0)

    -- Setup and start animation
    self.fadein_anim = self:RunUpdater(
        Updater.Parallel{
            -- Fade and move in the coin container
            Updater.Ease( function(v) self.content:SetTintAlpha(v) end, 0, 1, anim_duration, easing.outQuad),
            Updater.Ease2D( function(x,y) self.content:SetPos(x,y) end, content_x - horizontal_movement, content_y, content_x, content_y, anim_duration, easing.outQuad),
        }
    )

end

function FlourishScreen:OnScreenResize(w, h)
    FlourishScreen._base.OnScreenResize(self, w, h)
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
end

function FlourishScreen:GetDefaultFocus()
    return self.battle_container and self.battle_container.children and #self.battle_container.children>0 and self.battle_container.children[1]:GetDefaultFocus()
end

function FlourishScreen:OnControlModeChange( cm, device_id )
    FlourishScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.flourish_scroll:ShowBar( SCROLLBAR.IF_NEEDED )
        self.bottom_left:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.flourish_scroll:ShowBar( SCROLLBAR.NEVER )
        self.bottom_left:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.flourish_scroll:ShowBar( SCROLLBAR.NEVER )
        self.bottom_left:Hide()
    end

end

function FlourishScreen:OnScreenModeChange( sm )
    FlourishScreen._base.OnScreenModeChange( self, sm )

    

    -- if sm == SCREEN_MODE.MONITOR then
    -- elseif sm == SCREEN_MODE.SMALL
    --     or sm == SCREEN_MODE.TV then
    -- end

    -- self.character:SetPos( -RES_X*0.27*LAYOUT_SCALE[sm], -350 )
    -- for k, v in pairs(self.flourishes_container.children) do
    --     v:SetLayoutScale( LAYOUT_SCALE[sm] )
    -- end

    self:Layout()
end

function FlourishScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end

    self:Layout()
end

function FlourishScreen:Layout()
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    
    local left_offset = self.screen_w*0.18 -- How much the scroll panel should cross to the left of the screen center
    
    -- Scale contents
    local sm = TheGame:FE():GetScreenMode()
    self.header:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )

    -- Layout flourishes
    self.battle_title:LayoutBounds( "left", "top", 0, 0 )
        :Offset( self.screen_w/2-left_offset, 0 )
    self.battle_container:StackChildrenRow( 30 )
        :LayoutBounds( "left", "below", self.battle_title )
        :Offset( 0, -10 )
    self.negotiation_title:LayoutBounds( "left", "below", self.battle_container )
        :Offset( 0, -30 )
    self.negotiation_container:StackChildrenRow( 30 )
        :LayoutBounds( "left", "below", self.negotiation_title )
        :Offset( 0, -10 )

    -- Content width
    local content_w = self.battle_container:GetSize()
    content_w = content_w/LAYOUT_SCALE[sm]

    -- Layout header
    self.title:SetAutoSize( content_w )
        :LayoutBounds( "left", "top", 0, -60 )
    self.subtitle:SetAutoSize( content_w )
        :LayoutBounds( "left", "below", self.title ):Offset( 0, 0 )

    -- Header height
    local w, h = self.header:GetSize()
    local header_height = ( 60 + h + SPACING.M1 )*LAYOUT_SCALE[sm]
    -- Footer height
    local w, h = self.bottom_left:GetSize()
    local footer_height = ( 60 + h + SPACING.M1 )*LAYOUT_SCALE[sm]
    -- Content height
    self.content_h = self.screen_h - header_height - footer_height


    -- Update containers
    self.widepanel:SetHeight( self.content_h )
        :LayoutBounds( nil, "below", self.header )
        :Offset( 0, -SPACING.M1 )

    self.flourish_scroll:SetSize( self.screen_w-10, self.content_h-1 )
        :LayoutBounds( "left", "center", self.widepanel )
        :Offset( 0, 4 )
    
    self.header:LayoutBounds( "left", nil, self.widepanel )
        :Offset( self.screen_w/2-left_offset, 0 )

    self.flourish_scroll:RefreshView()

    return self
end

function FlourishScreen:HandleControlDown(control, device)
    if control:Has( Controls.Digital.SCROLL_DOWN ) then
        self.flourish_scroll:ScrollDown()
        return true
    elseif control:Has( Controls.Digital.SCROLL_UP ) then
        self.flourish_scroll:ScrollUp()
        return true
    end
end

function FlourishScreen:HandleControlUp( control, device )
    if control:Has( Controls.Digital.MENU_CANCEL ) then
        AUDIO:PlayEvent( "event:/ui/grafts_menu/close" )
        self:FadeOut()
    end

    return FlourishScreen._base.HandleControlUp( self, control, device )
end

function FlourishScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
end

function FlourishScreen:OnOpen()
    FlourishScreen._base.OnOpen( self )
    self.screen_loop = AUDIO:CreateEventInstance("event:/ui/grafts_menu/open")
    self.screen_loop:Start()
end

function FlourishScreen:OnClose()
    FlourishScreen._base.OnClose( self )

    self.screen_loop:Stop()
    self.screen_loop:Release()
    self.screen_loop = nil

    if self.on_close then
        self.on_close()
    end

end

-------------------------------------------------------------------------
-- A single Flourish Upgrade ID column, with one or more flourishes

local FlourishColumnWidget = class( "Widget.FlourishColumnWidget", Widget )

function FlourishColumnWidget:init( gametype, card_def, on_upgrade_fn )
    FlourishColumnWidget._base.init( self )
    self.features = {}

    self.gametype = gametype
    self.base_card_def = card_def
    self.on_upgrade_fn = on_upgrade_fn

    self.buttons = self:AddChild( Widget() )
    self.cards = {}
    self.arrows = {}
    self:AddUpgrade( card_def )
end

function FlourishColumnWidget:GetDef()
    return self.base_card_def
end

function FlourishColumnWidget:AddUpgrade( card_def )

    -- Add upgrade arrow between buttons
    if #self.cards > 0 then
        local arrow = self.buttons:AddChild( Widget.Image( engine.asset.Texture("UI/ic_nav_upgrade.tex") ) )
            :SetSize( 30, 30 )
            :LayoutBounds( "center", "below" )
            :Offset( 0, -6 )
            :SetScale( 1, -1 )
            :SetTintColour( self.gametype == "BATTLE" and UICOLOURS.FIGHT or UICOLOURS.NEGOTIATION )
            :Bloom( 0.1 )
        table.insert( self.arrows, arrow )
    end

    -- Create card
    local card_widget = self.buttons:AddChild( Widget.FlourishOptionButton( card_def ) )
        :LayoutBounds( "center", "below" )
        :Offset( 0, -6 )
    card_widget:SetFn( function() self:OnOptionClicked( card_widget ) end )
    table.insert( self.cards, card_widget )

    return self
end

function FlourishColumnWidget:OnOptionClicked( option_widget )
    self.on_upgrade_fn( option_widget:GetID(), option_widget:GetFlourishType(), option_widget:GetName(), option_widget:GetCost(), option_widget )
end

function FlourishColumnWidget:SetCurrentPoints( prestigepoints )
    for k, button in ipairs( self.cards ) do
        button:SetExpensive( button.prestigepoints_cost > prestigepoints )
    end
    return self
end

function FlourishColumnWidget:SetUnlockedUpgrades( unlocked )
    local acquired
    for k, button in ipairs( self.cards ) do
        button:SetAvailable( k==1 or k>1 and acquired )
        acquired = table.arrayfind( unlocked, button:GetID() )
        button:SetAcquired( acquired )
    end
    return self
end

function FlourishColumnWidget:GetDefaultFocus()
    return self.buttons and self.buttons.children and #self.buttons.children>0 and self.buttons.children[1]
end

-------------------------------------------------------------------------
-- A single Mettle option button

local FlourishOptionButton = class( "Widget.FlourishOptionButton", Widget.Clickable )

FlourishOptionButton.CONTROL_MAP =
{
    {
        control = Controls.Digital.GAMEPAD_RT,
        fn = function(self)
            self:ShowCardInfoPopup()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CARD_INFO", Controls.Digital.GAMEPAD_RT))
        end
    },
}

function FlourishOptionButton:init( flourish_card, onclick_fn )
    FlourishOptionButton._base.init( self )

    self.flourish_card = flourish_card
    self.prestigepoints_cost = self.flourish_card.pp_cost or 0
    self.acquired = false
    self.selected = false
    self.expensive = false
    self.available = false
    self.onclick_fn = onclick_fn
    self:SetOnClickFn( onclick_fn )

    -- Background panel
    local bg_border = 33
    local bg_border_scale = 0.5
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/rewardwidget_bg.tex") ) )
        :SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
        :SetNineSliceBorderScale( bg_border_scale )
        :SetSize( BUTTON_W, BUTTON_H )
        :Bloom( 0.2 )

    -- Card image
    self.mask = self:AddChild( Widget() ):SetStencilContext( true ):SetHiddenBoundingBox( true )
    self.mask_image = Widget.Panel( engine.asset.Texture("UI/rewardwidget_mask.tex") )
    self.mask_image:SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
    self.mask_image:SetNineSliceBorderScale( bg_border_scale )
    self.mask_image:SetSize( BUTTON_W, BUTTON_H )
    self.card_image = Widget.Image( flourish_card.icon )
    self.coloroverlay = Widget.Panel( engine.asset.Texture("UI/rewardwidget_mask.tex") )
        :SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
        :SetNineSliceBorderScale( bg_border_scale )
        :SetSize( BUTTON_W, BUTTON_H )
        :SetTintAlpha( 1 )
    self.mask:AddChildren
    {
        self.mask_image:SetMask(true),
        self.card_image:SetStencilTest(true),
        self.coloroverlay,
    }
    self.mask_image:LayoutBounds( "center", "bottom", self.bg )

    -- Overlay panel
    self.overlay = self:AddChild( Widget.Panel( engine.asset.Texture("UI/rewardwidget_upgradeoverlay.tex") ) )
        :SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
        :SetNineSliceBorderScale( bg_border_scale )
        :SetSize( BUTTON_W, BUTTON_H )
        :Bloom( 0.2 )

    -- Add cost badge
    self.cost_badge = self:AddChild( Widget.Panel( engine.asset.Texture("UI/mettle_cost.tex") ) )
        :SetHiddenBoundingBox( true )
    self.cost_label = self:AddChild(Widget.Label( "body", FONT_SIZE.BODY_TEXT ) )
        :SetText( loc.format( LOC"UI.FLOURISH_SCREEN.FLOURISH_COST", self.prestigepoints_cost ) )
        :OverrideLineHeight( FONT_SIZE.BODY_TEXT*LINE_HEIGHT_TEXT_MULT )
        :SetHiddenBoundingBox( true )


    local card = nil
    if is_instance( self.flourish_card, BattleCardDef ) then
        self.gametype = "BATTLE"
        self.color = UICOLOURS.FIGHT
        self.color_dark = 0x3D2A1Eff
        card = Battle.Card( self.flourish_card.id )
    elseif is_instance( self.flourish_card, NegotiationCardDef ) then
        self.gametype = "NEGOTIATION"
        self.color = UICOLOURS.NEGOTIATION
        self.color_dark = 0x19143Eff
        card = Negotiation.Card( self.flourish_card.id )
    end
    if card then
        self:SetToolTipClass( Widget.TooltipAbridgedCard )
        self:SetToolTip( card )
    end

    self:ShowToolTipOnFocus()
    self:SetOnRightClickFn( function() self:ShowCardInfoPopup() end )
    self:Refresh()
end

function FlourishOptionButton:GetID()
    return self.flourish_card.id
end

function FlourishOptionButton:GetCost()
    return self.prestigepoints_cost
end

-- "BATTLE" or "NEGOTIATION"
function FlourishOptionButton:GetFlourishType()
    return self.gametype
end

function FlourishOptionButton:GetName()
    return self.flourish_card:GetLocalizedName()
end

function FlourishOptionButton:ShowCardInfoPopup()
    local card
    if is_instance( self.flourish_card, BattleCardDef ) then
        card = Battle.Card( self.flourish_card.id )
    elseif is_instance( self.flourish_card, NegotiationCardDef ) then
        card = Negotiation.Card( self.flourish_card.id )
    end
    if card then TheGame:FE():PushScreen( Screen.CardDetailsPopup( card ) ) end
end

function FlourishOptionButton:SetFn( onclick_fn )
    self.onclick_fn = onclick_fn
    return self
end

-- If it has been acquired. Can be the selected one simultaneously
function FlourishOptionButton:SetAcquired( acquired )
    self.acquired = acquired
    self:Refresh()
    return self
end

-- If its cost is higher than this character's amount of mettle
function FlourishOptionButton:SetExpensive( expensive )
    self.expensive = expensive
    self:Refresh()
    return self
end

-- If it can be purchased. Can be expensive simultaneously
function FlourishOptionButton:SetAvailable( available )
    self.available = available
    self:Refresh()
    return self
end

function FlourishOptionButton:Refresh()
    if self.acquired then
        self.bg:SetTintColour( self.color ):SetSaturation( 1 )
        self.coloroverlay:SetTintColour( 0x00000000 )
        self.overlay:SetTintColour( self.color ):Bloom(0.15)
        self.card_image:SetSaturation( 1 )
        self.cost_label:SetGlyphColour( 0xC6AB3Eff )
        self.cost_badge:SetTintColour( UICOLOURS.SUBTITLE )
        self:SetOnClickFn(nil)
    elseif self.available and self.expensive then
        self.bg:SetTintColour( self.color ):SetSaturation( 1 )
        self.coloroverlay:SetTintColour( self.color_dark ):SetTintAlpha( 0.8 )
        self.overlay:SetTintColour( self.color_dark ):Bloom(0)
        self.card_image:SetSaturation( 0.2 )
        self.cost_label:SetGlyphColour( 0x666666ff )
        self.cost_badge:SetTintColour( UICOLOURS.PENALTY )
        self:SetOnClickFn(nil)
    elseif self.available and not self.expensive then
        self.bg:SetTintColour( self.color ):SetSaturation( 1 )
        self.coloroverlay:SetTintColour( self.color_dark ):SetTintAlpha( 0.8 )
        self.overlay:SetTintColour( self.color ):Bloom(0.15)
        self.card_image:SetSaturation( 0.2 )
        self.cost_label:SetGlyphColour( 0x40D988ff )
        self.cost_badge:SetTintColour( UICOLOURS.BONUS )
        self:SetOnClickFn( self.onclick_fn )
    else
        self.bg:SetTintColour( self.color ):SetSaturation( 0.2 )
        self.coloroverlay:SetTintColour( 0x222222ff ):SetTintAlpha( 0.85 )
        self.overlay:SetTintColour( 0x444444ff ):Bloom(0)
        self.card_image:SetSaturation( 0.2 )
        self.cost_label:SetGlyphColour( 0x666666ff )
        self.cost_badge:SetTintColour( 0x666666ff )
        self:SetOnClickFn(nil)
    end

    -- Update cost badge
    if self.acquired then
        self.cost_badge:Hide()
        self.cost_label:Hide()
    else
        self.cost_badge:Show()
        self.cost_label:Show()
            :SetText( loc.format(LOC"UI.FLOURISH_SCREEN.FLOURISH_COST",self.prestigepoints_cost) )
        local w, h = self.cost_label:GetSize()
        self.cost_badge:SetSize( w + 10, h + 0 )
    end
    self.cost_badge:LayoutBounds( "right", "bottom", self.bg )
        :Offset( 4, -4 )
    self.cost_label:LayoutBounds( "center", "center", self.cost_badge )
        :Offset( 0, 0 )

    self:UpdateImage()
    return self
end

function FlourishOptionButton:UpdateImage()

    if self.hover or self.focus then
        self:ScaleTo( nil, self.selected and 1.15 or 1.05, 0.2, easing.outQuad )
    else
        self:ScaleTo( nil, self.selected and 1.05 or 1, 0.2, easing.outQuad )
    end
    self.cost_label:SetGlyphColour( UICOLOURS.WHITE )
        :SetTintColour( UICOLOURS.DARK_BODYTEXT )

    return self
end

function FlourishOptionButton:OnGainFocus()
    FlourishOptionButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function FlourishOptionButton:OnLoseFocus()
    FlourishOptionButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function FlourishOptionButton:OnGainHover()
    FlourishOptionButton._base.OnGainHover(self)
    self:UpdateImage()
end

function FlourishOptionButton:OnLoseHover()
    FlourishOptionButton._base.OnLoseHover(self)
    self:UpdateImage()
end
