require "ui/widgets/screen"

--------------------------------------------------------------------------------
-- Screen for testing font readability

local FontTestScreen = class("Screen.FontTestScreen", Widget.Screen)

function FontTestScreen:init()
    FontTestScreen._base.init(self)

	self.is_popup = true
    self:SetupUnderlay( true )
	self:SetAnchors("center", "center")

	-- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    
    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*2.0
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    local bloom = 0.1
    local outline = false
    local test_strings = self:AddChild( Widget() )
        :SetAnchors( "center", "center" )

    local english_bold = 0.36
    local english_body = 0.5
    local string_english_1 = test_strings:AddChild( Widget.Label( "body", 46, "Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat." ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :EnableOutline( outline )
        :SetSDFThreshold(english_bold)
        :Bloom(bloom)
    local string_english_2 = test_strings:AddChild( Widget.Label( "body", 32, "Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat." ) )
        :SetGlyphColour( UICOLOURS.TITLE )
        :EnableOutline( outline )
        :LayoutBounds( "left", "below" ):Offset( 0, 4 )
        :SetSDFThreshold(english_bold)
        :Bloom(bloom)
    local string_english_3 = test_strings:AddChild( Widget.Label( "body", 24, "Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur.\nExcepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum." ) )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :EnableOutline( outline )
        :LeftAlign()
        :LayoutBounds( "left", "below" ):Offset( 0, -6 )
        :SetSDFThreshold(english_body)
        :Bloom(bloom)

    local chinese_bold = 0.48
    local chinese_body = 0.5
    local string_chinese_1 = test_strings:AddChild( Widget.Label( "title", 46, "供覇著万取原作第場寄要質。授績欲金断至展型湯号響提石新塚。断引画会沖真本媚状問集秀転形予主" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :EnableOutline( outline )
        :LayoutBounds( "left", "below" ):Offset( 0, -30 )
        :SetSDFThreshold(chinese_bold)
        :Bloom(bloom)
    local string_chinese_2 = test_strings:AddChild( Widget.Label( "title", 32, "供覇著万取原作第場寄要質。授績欲金断至展型湯号響提石新塚。断引画会沖真本媚状問集秀転形予主" ) )
        :SetGlyphColour( UICOLOURS.TITLE )
        :EnableOutline( outline )
        :LayoutBounds( "left", "below" ):Offset( 0, 4 )
        :SetSDFThreshold(chinese_bold)
        :Bloom(bloom)
    local string_chinese_3 = test_strings:AddChild( Widget.Label( "title", 24, "供覇著万取原作第場寄要質。授績欲金断至展型湯号響提石新塚。断引画会沖真本媚状問集秀転形予主\n都生覧代掲講武必禁辺弁人提立内訃構庫深。許打事最納応影歓奪常実像職女巨化。病田政職患武応来県求元一全約答娘。社談思始盗改制好多設今川助助前院国済油芸。" ) )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :EnableOutline( outline )
        :LeftAlign()
        :LayoutBounds( "left", "below" ):Offset( 0, -6 )
        :SetSDFThreshold(chinese_body)
        :Bloom(bloom)

    test_strings:LayoutBounds( "center", "center", 0, 0 )

end
