require "ui/widgets/screen"
require "ui/widgets/unlocktrack"
require "ui/widgets/pointsfloater"
require "ui/screens/unlockinspectorscreen"
require "ui/screens/deckscreen"


local STATS_SCORE =
{
    PROGRESS = {
        COLOUR = UICOLOURS.QUEST,
        TITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_PROGRESS_TITLE" end,
        SUBTITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_PROGRESS_SUBTITLE" end,
        ICON = "UI/gameover_stats_days.tex",
        EXP = 100,
        BUTTON_TEXT = function() return LOC"UI.GAME_OVER_SCREEN.STATS_PROGRESS_BUTTON" end,
    },
    BATTLE = {
        COLOUR = UICOLOURS.FIGHT,
        TITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_BATTLE_TITLE" end,
        SUBTITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_BATTLE_SUBTITLE" end,
        ICON = "UI/gameover_stats_battle.tex",
        EXP = 50,
        BUTTON_TEXT = function() return LOC"UI.GAME_OVER_SCREEN.STATS_BATTLE_BUTTON" end,
    },
    NEGOTIATION = {
        COLOUR = UICOLOURS.NEGOTIATION,
        TITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_NEGOTIATION_TITLE" end,
        SUBTITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_NEGOTIATION_SUBTITLE" end,
        ICON = "UI/gameover_stats_negotiation.tex",
        EXP = 50,
        BUTTON_TEXT = function() return LOC"UI.GAME_OVER_SCREEN.STATS_NEGOTIATION_BUTTON" end,
    },
    GRAFT = {
        COLOUR = UICOLOURS.GRAFT,
        TITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_GRAFT_TITLE" end,
        SUBTITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_GRAFT_SUBTITLE" end,
        ICON = "UI/gameover_stats_grafts.tex",
        EXP = 75,
        BUTTON_TEXT = function() return LOC"UI.GAME_OVER_SCREEN.STATS_GRAFT_BUTTON" end,
    },
    RELATIONSHIPS = {
        COLOUR = UICOLOURS.RELATIONSHIP_LOVED,
        TITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_RELATIONSHIPS_TITLE" end,
        SUBTITLE = function() return LOC"UI.GAME_OVER_SCREEN.STATS_RELATIONSHIPS_SUBTITLE" end,
        ICON = "UI/opinion_icon_loved.tex",
        EXP = 100,
        BUTTON_TEXT = function() return LOC"UI.GAME_OVER_SCREEN.STATS_RELATIONSHIPS_BUTTON" end,
    },

}

local APPROACH_SCORE =
{
    WIN = {
        TITLE = function() return LOC"UI.APPROACH_SCORE.WON_TITLE" end,
        SUBTITLE = function() return LOC"UI.APPROACH_SCORE.WON_SUBTITLE" end,
        ICON = "UI/ic_score_win.tex",
        EXP = 1000
    },
    QUEST = {
        TITLE = function() return LOC"UI.APPROACH_SCORE.QUEST_TITLE" end,
        SUBTITLE = function( number ) return loc.format( LOC"UI.APPROACH_SCORE.QUEST_SUBTITLE", number ) end,
        ICON = "UI/ic_score_quest.tex",
        EXP = 75
    },
    UPGRADED = {
        TITLE = function() return LOC"UI.APPROACH_SCORE.UPGRADED_TITLE" end,
        SUBTITLE = function( number )
            return loc.format( LOC"UI.APPROACH_SCORE.UPGRADED_SUBTITLE", number )
        end,
        ICON = "UI/ic_score_upgrade.tex",
        EXP = 5
    },
    MONEY = {
        TITLE = function() return LOC"UI.APPROACH_SCORE.MONEY_TITLE" end,
        SUBTITLE = function( number )
            return loc.format( LOC"UI.APPROACH_SCORE.MONEY_SUBTITLE", number )
        end,
        ICON = "UI/ic_score_money.tex",
        EXP = .5
    },
}


local GameOverScreen = class( "Screen.GameOverScreen", Widget.Screen )

GameOverScreen.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function(self)
            return self.animation_started and not self.animation_ended
        end,
        fn = function(self) 
            -- Skip animation
            if self.animation_started then
                self.fast = true
                self.stats_progress:SkipAnimation()
                self.stats_battle:SkipAnimation()
                self.stats_negotiation:SkipAnimation()
                self.stats_grafts:SkipAnimation()
                self.stats_relationships:SkipAnimation()
                self.approach:SkipAnimation()
            end
            self.progress.fast = true
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.GAME_OVER_SCREEN.CONTROLS_RIGHT_SKIP", Controls.Digital.MENU_ACCEPT))
        end
    },
    {
        control = Controls.Digital.SKIP,
        test = function(self)
            return self.animation_started and not self.animation_ended
        end,
        fn = function(self) 
            -- Skip animation
            if self.animation_started then
                self.fast = true
                self.stats_progress:SkipAnimation()
                self.stats_battle:SkipAnimation()
                self.stats_negotiation:SkipAnimation()
                self.stats_grafts:SkipAnimation()
                self.stats_relationships:SkipAnimation()
                self.approach:SkipAnimation()
            end
            self.progress.fast = true
            return true
        end
    },
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function(self)
            return self.animation_ended and TheGame:FE():IsGamepadMode()
        end,
        fn = function(self)
            -- Next screen
            if self.button_continue_lost:IsShown() then self.button_continue_lost:Click() return true end
            if self.button_accept_win:IsShown() then self.button_accept_win:Click() return true end
            if self.button_confirm_abandon:IsShown() then self.button_confirm_abandon:Click() return true end
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.GAME_OVER_SCREEN.CONTROLS_RIGHT_CONTINUE", Controls.Digital.MENU_ACCEPT))
        end
    },
    {
        control = Controls.Digital.MENU_CANCEL,
        test = function(self)
            return self.animation_ended and TheGame:FE():IsGamepadMode()
        end,
        fn = function(self)
            -- Next screen
            if self.button_continue_lost:IsShown() then self.button_continue_lost:Click() return true end
            if self.button_accept_win:IsShown() then self.button_accept_win:Click() return true end
            if self.button_confirm_abandon:IsShown() then self.button_confirm_abandon:Click() return true end
            return true
        end,
    },
}

local PANEL_H = 426
local CONTENT_W = RES_X/4 * 2.2
local PADDING = 20

function GameOverScreen:init(gamestate)
    Widget.Screen.init(self)

    local result = gamestate:GetGameOver().result
    local win = result == GAMEOVER.ALPHA_VICTORY or result == GAMEOVER.VICTORY

    self.is_abandon_mode = result == GAMEOVER.ABANDONED
    self.killed_by = gamestate:GetGameOver().killed_by

    if self.killed_by then TheGame:GetGameProfile():SetKilledByAgent( self.killed_by:GetUniqueID() ) end
    -- self.killed_by = killed_by or Agent( "FSSH" )
    -- self.killed_by = killed_by or Agent( "LUMINTHIAN" )
    -- self.killed_by = killed_by or Agent( "SHROOG" )
    -- self.killed_by = killed_by or Agent( "SHROOGLET" )
    -- self.killed_by = killed_by or Agent( "CRAYOTE" )
    -- self.killed_by = killed_by or Agent( "CRAYOTE_SPITTER" )
    -- self.killed_by = killed_by or Agent( "COMBAT_DRONE" )
    -- self.killed_by = killed_by or Agent( "KASHIO" )
    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )
    self:SetSounds( SoundEvents.tut_popup, SoundEvents.tut_popout )

    -- Meta progression UI.
    ----------------------------
    self.score = {
        won = win
    }

    if gamestate then
        -- Get play data
        self.character_id = gamestate:GetPlayerBackgroundID()
        
        self.score.battles_won = gamestate:GetScoreCount("BATTLES_WON")
        self.score.negotiations_won = gamestate:GetScoreCount("NEGOTIATIONS_WON")
        self.score.cards_upgraded = gamestate:GetScoreCount("CARDS_UPGRADED")
        self.score.jobs_completed = gamestate:GetScoreCount("JOBS_COMPLETED")
        self.score.money = gamestate:GetScoreCount("GAINED_MONEY")
        self.score.liked = gamestate:GetScoreCount("LIKED")
        self.score.loved = gamestate:GetScoreCount("LOVED")
        self.score.progress = gamestate:GetScoreCount("PROGRESS")

        local act = GetPlayerActData( gamestate:GetOptions().start_act )
        
        if act.data.score_modifiers then
            
            if act.data.score_modifiers.money then
                self.score.money = self.score.money + act.data.score_modifiers.money 
            end

            if act.data.score_modifiers.expected_relationships then
                for k, alias in ipairs( act.data.score_modifiers.expected_relationships ) do
                    local agent = gamestate:GetAgent(alias)
                    if agent then
                        if agent:GetRelationship() == RELATIONSHIP.LIKED then
                            self.score.liked = self.score.liked - 1
                        end
                        
                        if agent:GetRelationship() == RELATIONSHIP.LOVED then
                            self.score.loved = self.score.loved - 1
                        end

                     end
                end
            end
        end

        if not gamestate:GetOptions().experiment then
            -- Contribute to card statistics
            TheGame:GetGameStats():AddCardStats(gamestate:GetCardStats())

            -- Remove this save file! Rogue!
            if gamestate.save_file_name and gamestate.save_file_name ~= "autosave" then --and gamestate:GetGameOver().result ~= GAMEOVER.ALPHA_VICTORY then
                self.save_file_name = gamestate.save_file_name
                TheGame:DeleteSave(gamestate.save_file_name, function() end)
            end

            if gamestate:GetRollbackSave() then
                TheGame:DeleteSave( gamestate:GetRollbackSave(), function() end)
            end

            -- Earn advancement!
            if win and not gamestate:IsStoryMode() and not gamestate:IsCustomMode() then
                local act_id = gamestate:GetCurrentActID()
                self.current_prestige = TheGame:GetGameProfile():GetAdvancementLevel( act_id )
                TheGame:GetGameProfile():AcquireAdvancement( act_id, gamestate:GetOptions().advancement_level or 0 )
                self.new_prestige = TheGame:GetGameProfile():GetAdvancementLevel( act_id )
                self.max_prestige = TheGame:GetGameProfile():GetMaxAdvancement( act_id )
            end

            -- Send metrics
            local json_t = { PLAYER_DATA = gamestate:GetPlayerState() }
            if win then
                SendMetricsData("GAME_OVER_WON", json_t)
            else
                SendMetricsData("GAME_OVER_LOST", json_t)
            end
        end
    else
        gamestate = GameState()
        TheGame.gamestate = gamestate
        gamestate:AddPlayerAgent( Agent( "ROOK" ))
        -- Fake play data for screen testing
        self.character_id = "SAL"
        self.score.won = true
        self.score.battles_won = 8
        self.score.negotiations_won = 13
        self.score.cards_upgraded = 12
        self.score.jobs_completed = 7
        self.score.money = 620
        self.score.liked = 13
        self.score.loved = 6
        self.score.progress = 0

        self.current_prestige = 2
        self.new_prestige = 3
        self.max_prestige = TheGame:GetGameProfile():GetMaxAdvancement( gamestate:GetCurrentActID() )
    end
    self.current_act = gamestate.current_act and gamestate.current_act or "SAL"
    self.total_exp = 0

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )
    
    -- An animated character that killed the player
    self.enemy = self:AddChild( Widget() ):Hide()
    self.enemy_glow = self.enemy:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ), 800, 600 ) )
        :SetHiddenBoundingBox( true )
        :SetTintColour( UICOLOURS.HEALTH )
        :SetTintAlpha( 0.7 )
    self.enemy_anim_root = self.enemy:AddChild(Widget())
        :Hide()
    self.enemy_anim = self.enemy_anim_root:AddChild( Widget.AnimatedCharacter() )
    self.enemy_text_container = self.enemy:AddChild( Widget() )
    local enemy_text_width = 280
    self.enemy_title = self.enemy_text_container:AddChild( Widget.Label("title", FONT_SIZE.ITEM_SUBTITLE ))
        :SetGlyphColour( UICOLOURS.HEALTH )
        :RightAlign()
        :SetWordWrap( true )
        :SetAutoSize( enemy_text_width )
        :Bloom( 0.09 )
    self.enemy_subtitle = self.enemy_text_container:AddChild( Widget.Label("body", FONT_SIZE.ITEM_DESC ))
        :SetGlyphColour( UICOLOURS.HEALTH )
        :SetTintAlpha( 0.9 )
        :RightAlign()
        :SetWordWrap( true )
        :SetAutoSize( enemy_text_width )
        :Bloom( 0.05 )

    -- A container for all the background elements
    self.panel = self:AddChild( Widget.WidePanel( PANEL_H, self.score.won and UICOLOURS.OBJECTIVE or UICOLOURS.PENALTY ) )

    -- Our fancy character anim!
    self.anim_root = self:AddChild(Widget()):SetHiddenBoundingBox( true )
    self:RefreshBigCharacter( gamestate:GetPlayerAgent() )

    -- Prestige label
    self.prestige_widget = self:AddChild( Widget() ):Hide()
    self.prestige_backing = self.prestige_widget:AddChild( Widget.Image( engine.asset.Texture("UI/gameover_progress_bg.tex") ) )
        :SetSize( 400, 42 )
        :SetTintColour( UICOLOURS.ADVANCEMENT )
    self.prestige_label = self.prestige_widget:AddChild( Widget.Label( "body", FONT_SIZE.ITEM_SUBTITLE, loc.format( LOC"UI.GAME_OVER_SCREEN.PRESTIGE", gamestate:GetAdvancementLevel() ) ) )
        :SetGlyphColour( UICOLOURS.ADVANCEMENT )
        :Bloom( 0.1 )
        :LayoutBounds( "center", "center", self.prestige_backing )
        :Offset( FONT_SIZE.ITEM_NAME/2, 0 )
    self.prestige_icon = self.prestige_widget:AddChild( Widget.PrestigeIndicator( FONT_SIZE.ITEM_NAME ) )
    self.prestige_backing:SetScale( 1, -1 )
    if ( gamestate and gamestate:GetAdvancementLevel() and gamestate:GetAdvancementLevel() > 0 ) or self.current_prestige then
        self.prestige_level = gamestate:GetAdvancementLevel() or self.current_prestige
        self.prestige_widget:Show()
        self.prestige_icon:Refresh( gamestate )
        self.prestige_icon:LayoutBounds( "before", "center", self.prestige_label )
            :Offset( -10, 0 )
    end
    self.prestige_widget:LayoutBounds( "center", "above", self.panel )
        :Offset( 0, 0 )

    -- Our titles above the background panel
    self.header = self:AddChild( Widget() )
    self.subtitle = self.header:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_TABS ))
        :SetText( GetPlayerActData(self.current_act):GetLocalizedTitle() )
        :Bloom( 0.1 )
    self.title = self.header:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_TITLE) )
        :SetText( self.score.won and LOC"UI.GAME_OVER_SCREEN.TITLE_SUCCESS" or LOC"UI.GAME_OVER_SCREEN.TITLE_FAIL" )
        :SetGlyphColour( self.score.won and UICOLOURS.OBJECTIVE or UICOLOURS.PENALTY )
        :Bloom( 0.05 )
    self.subtitle:LayoutBounds( "center", "below", self.title )
        :Offset( 0, SPACING.TITLE_SUBTITLE )

    self.playtime = self.header:AddChild( Widget.Label("title", 24, FormatTime( gamestate:GetPlayTime() ) ))
        :LayoutBounds( "center", "below", self.subtitle )

    self.header:LayoutBounds( "center", "above", (self.prestige_icon and self.prestige_widget:IsShown()) and self.prestige_widget or self.panel )
        :Offset( 0, SPACING.M1 )

    -- Hitbox for content alignment
    self.hitbox = self:AddChild( Widget.SolidBox( CONTENT_W, PANEL_H, 0xFF000000 ) )
        :LayoutBounds( "left", "center", -350, 0 )

    -- Container for the info blocks
    self.panel_contents = self:AddChild( Widget() )

    -- Stats section
    local spacing = 20
    local stats_widgets_w = ( CONTENT_W - 20*4 ) / 5
    local stats_widgets_h = 240
    local max_day = gamestate and gamestate:GetMainQuest() and gamestate:GetMainQuest():GetQuestDef().max_day or 5
    self.stats = self.panel_contents:AddChild( Widget() )
    -- Add stats blocks
    self.stats_progress = self.stats:AddChild( 
        Widget.StatsBlockDisplay( stats_widgets_w, stats_widgets_h, 
            STATS_SCORE.PROGRESS,
            self.score.progress,
            max_day
            ))
    self.stats_battle = self.stats:AddChild( 
        Widget.StatsBlockDisplay( stats_widgets_w, stats_widgets_h, 
            STATS_SCORE.BATTLE,
            self.score.battles_won,
            nil,
            function()
                -- Show the player's battle cards
                local cards = gamestate:GetPlayerAgent():GetAspect("battler"):GetCards()
                local screen = Screen.DeckScreen( cards, nil, Widget.BattleCard, nil, true )
                screen:SetTitles( LOC"UI.GAME_OVER_SCREEN.STATS_BATTLE_SCREEN_TITLE", LOC"UI.SHOW_DECK_SCREEN.BATTLE_DECK_UPPERCASE" )
                TheGame:FE():InsertScreen( screen )
            end ) )
    self.stats_negotiation = self.stats:AddChild( 
        Widget.StatsBlockDisplay( stats_widgets_w, stats_widgets_h, 
            STATS_SCORE.NEGOTIATION, 
            self.score.negotiations_won,
            nil,
            function()
                -- Show the player's negotiation cards
                local cards = gamestate:GetPlayerAgent():GetAspect("negotiator"):GetCards()
                local screen = Screen.DeckScreen( cards, nil, Widget.NegotiationCard, nil, true )
                screen:SetTitles( LOC"UI.GAME_OVER_SCREEN.STATS_NEGOTIATION_SCREEN_TITLE", LOC"UI.SHOW_DECK_SCREEN.NEGOTIATION_DECK_UPPERCASE" )
                TheGame:FE():InsertScreen( screen )
            end ) )
    -- Get graft counts
    local graft_count, max_grafts = gamestate:GetPlayerAgent():GetAspect("graft_owner"):GetTotalGraftsCount()
    self.stats_grafts = self.stats:AddChild( 
        Widget.StatsBlockDisplay( stats_widgets_w, stats_widgets_h, 
            STATS_SCORE.GRAFT,
            graft_count,
            max_grafts,
            function()
                -- Show the player's grafts
                local agent = gamestate:GetPlayerAgent()
                local screen = Screen.GraftCollectionScreen( agent, true )
                TheGame:FE():InsertScreen( screen )
            end ) )
    self.stats_relationships = self.stats:AddChild( 
        Widget.StatsBlockDisplay( stats_widgets_w, stats_widgets_h, 
            STATS_SCORE.RELATIONSHIPS,
            self.score.loved,
            nil,
            function()
                -- Show the player's relationships
                local screen = Screen.RelationshipScreen( gamestate )
                TheGame:FE():InsertScreen( screen )
            end ) )
    self.stats:StackChildrenRow( spacing )
        :LayoutBounds( "left", "top", self.hitbox ):Offset( 0, -PADDING*1 )

    -- Approach section
    self.approach = self.panel_contents:AddChild( Widget.ApproachContainer( CONTENT_W, 125, 4 ) )
        :LayoutBounds( "left", "below", self.stats ):Offset( 0, -PADDING )
    -- Add approach blocks
    if self.score.won then self.approach:AddBlock( APPROACH_SCORE.WIN, 1 ) end
    if self.score.jobs_completed > 0 then self.approach:AddBlock( APPROACH_SCORE.QUEST, self.score.jobs_completed ) end
    if self.score.cards_upgraded > 0 then self.approach:AddBlock( APPROACH_SCORE.UPGRADED, self.score.cards_upgraded ) end
    if self.score.money > 0 then self.approach:AddBlock( APPROACH_SCORE.MONEY, self.score.money ) end
    -- Lay them out
    self.approach:Layout()

    -- Progress bar + Rewards!
    self.starting_points = TheGame:GetGameProfile():GetPoints(self.character_id)
    
    self.next_unlock = TheGame:GetGameProfile():GetNextUnlock(self.character_id)
    self.progress = self:AddChild( Widget.ProgressUnlocks( CONTENT_W, self.character_id, gamestate ) )
    self.progress.fast = false
    self.progress:LayoutBounds( nil, "below", self.panel )
        :LayoutBounds( "left", nil, self.approach )
        :Offset( 0, -10 )
    self.progress:SetPts( self.starting_points, self.next_unlock )
    self.progress:SetOnNewUnlock( function(...) self:OnNewUnlock(...) end )
    if self.prestige_level then 
        self.prestige_bonus = 10*self.prestige_level/100 -- 10% more per prestige level
        self.progress:SetAdvancementBonus( self.prestige_bonus )
    end
    self.approach:SetFocusDir( "down", self.progress:GetDefaultFocus(), true )
    self.progress:SetFocusDir( "up", self.approach:GetDefaultFocus() )

    -- Continue button
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    
    self.button_continue_lost = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.GAME_OVER_SCREEN.BUTTON_CONTINUE", 
        function()
            self:GetFE():FadeTransition(
                    function()
                        TheGame:Reset( function() 
                            TheGame:FE():InsertScreen( Screen.MainMenu() )
                        end)
                    end, .4)
        end ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Hide()

    self.button_accept_win = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.GAME_OVER_SCREEN.BUTTON_CONTINUE", 
        function()
            --print( "savegame", self.save_file_name or "none" )
            
            
            self:GetFE():FadeTransition(function()
                self:GetFE():PopScreen(self) 
                TheGame:Reset( function() 
                    TheGame:FE():InsertScreen( Screen.MainMenu() )
                end)
            end, .4)
        end ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Hide()

    --shown if you are abandoning an old save
    local function OnAbandon()
        self:GetFE():FadeTransition(function()
            TheGame:Reset( function() 
                TheGame:FE():InsertScreen( Screen.MainMenu() )
            end)
        end, 0.4 )
    end

    self.button_confirm_abandon = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.GAME_OVER_SCREEN.BUTTON_CLOSE", OnAbandon ))
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Hide()
end

function GameOverScreen:GetDefaultFocus()
    if self.stats_progress:IsShown() then
        return self.stats_progress
    end
end

function GameOverScreen:OnControlModeChange( cm, device_id )
    GameOverScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.bottom_right:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.bottom_right:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.bottom_right:Hide()
    end
end

function GameOverScreen:OnScreenModeChange( sm )
    GameOverScreen._base.OnScreenModeChange( self, sm )

    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )
    
    self.panel:SetHeight( PANEL_H*LAYOUT_SCALE[sm] )

    self.enemy_text_container:SetLayoutScale( LAYOUT_SCALE[sm] )
    self:LayoutEnemyCharacter()

    self.prestige_widget:SetLayoutScale( LAYOUT_SCALE[sm] )
        :LayoutBounds( "center", "above", self.panel )
        :Offset( 0, 0 )

    self.header:SetLayoutScale( LAYOUT_SCALE[sm] )
        :LayoutBounds( "center", "above", (self.prestige_icon and self.prestige_widget:IsShown()) and self.prestige_widget or self.panel )
        :Offset( 0, SPACING.M1 )

    self.panel_contents:SetLayoutScale( LAYOUT_SCALE[sm] )

    if sm == SCREEN_MODE.MONITOR then
        self.anim_root:SetPos( 0, 0 )
        self.panel_contents:SetPos( 0, 0 )
        self.progress:LayoutBounds( nil, "below", self.panel.panel_bg )
            :LayoutBounds( "left", nil, self.approach )
            :Offset( 0, 17 )
        self.progress.progress_container:LayoutBounds( "left", nil, self.progress.unlock_track )
    elseif sm == SCREEN_MODE.SMALL
        or sm == SCREEN_MODE.TV then
        self.anim_root:SetPos( -160, 0 )
        self.panel_contents:SetPos( -50, 0 )
        self.progress:LayoutBounds( "center", "below", self.panel.panel_bg )
            :Offset( 0, 17 )
        self.progress.progress_container:LayoutBounds( "center", nil, self.progress.unlock_track )
    end

end

function GameOverScreen:RefreshStreamerMode()
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

function GameOverScreen:OnOpen()
    GameOverScreen._base.OnOpen( self )

    -- Kick-off the animation!
    self:StartCoroutine( GameOverScreen.ShowScoreCoro, self )

    -- Show the enemy, if any
    self:StartCoroutine( self.RefreshEnemyCharacter, self )


    if TheGame:GetLocalSettings().ROBOTICS then
        engine.inst:Quit()
        return
    end
end

function GameOverScreen:OnEXPChange( delta )

    -- Add prestige level bonus (5% per level)
    if self.prestige_level then delta = delta + self.prestige_bonus*delta end

    -- Add it up!
    self.total_exp = self.total_exp + math.round( delta )

    -- Update the progress bar
    local earned = self.total_exp
    
    if earned >= 1 then
        self.progress:UpdateDisplayTotal( earned )
    end
end

function GameOverScreen:OnNewUnlock()
    self.biganim:Emote("clap")
end

function GameOverScreen:ShowScoreCoro()
    self.stats_progress:PrepareAnimation()
    self.stats_battle:PrepareAnimation()
    self.stats_negotiation:PrepareAnimation()
    self.stats_grafts:PrepareAnimation()
    self.stats_relationships:PrepareAnimation()
    self.approach:PrepareAnimation()

    self.animation_started = true
    self.stats_progress:AnimateIn( function( exp_delta ) self:OnEXPChange( exp_delta ) end )
    self.stats_battle:AnimateIn( function( exp_delta ) self:OnEXPChange( exp_delta ) end )
    self.stats_negotiation:AnimateIn( function( exp_delta ) self:OnEXPChange( exp_delta ) end )
    self.stats_grafts:AnimateIn( function( exp_delta ) self:OnEXPChange( exp_delta ) end )
    self.stats_relationships:AnimateIn( function( exp_delta ) self:OnEXPChange( exp_delta ) end )
    self.approach:AnimateIn( function( exp_delta ) self:OnEXPChange( exp_delta ) end )

    -- Update the meta progress
    local delta_pts = self.total_exp
    --print(self.total_exp, self.starting_points, delta_pts)

    self.unlocks_awarded, self.next_award = TheGame:GetGameProfile():DeltaPoints( self.character_id, delta_pts )

    self.progress:ShowGain( self.character_id, self.starting_points, delta_pts, self.unlocks_awarded, self.next_award,
            function()
                AUDIO:PlayEvent("event:/ui/game_over_screen/end")
                if self.is_abandon_mode then
                    TheGame:SubmitLog("ABANDONED GAME")
                    self.button_confirm_abandon:Show()
                else
                    local killed_txt = self.killed_by and loc.format("Killed by: {1.Who}", self.killed_by:LocTable()) or ""
                    TheGame:SubmitLog(self.score.won and "WON THE GAME" or "LOST THE GAME", killed_txt)
                    
                    if self.score.won then
                        -- If we gained prestige, show it
                        if self.new_prestige and self.new_prestige > self.current_prestige and self.new_prestige <= self.max_prestige then
                            self.prestige_panel = TheGame:FE():PushScreen( Screen.PrestigeIncreasePanel() )
                                :AnimatePrestige( self.current_prestige, self.new_prestige, GetPlayerActData(self.current_act), function() self.button_accept_win:Show() end )
                        else
                            self.button_accept_win:Show()
                        end

                    else
                        self.button_continue_lost:Show()
                    end
                end
                self.animation_ended = true
            end
        )
end

function GameOverScreen:FadeInWidget( widget, timing)
    timing = timing or 1

    while widget:GetAlpha() < 1 do
        local dt = coroutine.yield()
        widget:SetTintAlpha( widget:GetAlpha() + timing*dt )
        if self.fast then
            break
        end
    end
    self.is_done_animating = true
    widget:SetTintAlpha( 1 )

    return self
end

function GameOverScreen:RefreshBigCharacter( agent )
    local OFFSET = RES_X/4
    
    if agent then
        self.biganim = self.anim_root:AddChild( Widget.AnimatedCharacter() )
        self.biganim:Refresh( agent )

        local x = -RES_X/4-150
        local y = -270
        if self.score.won then
            self.biganim:SetMood( MOOD.HAPPY )
            x = -RES_X/4-150
        else
            self.biganim:SetMood( MOOD.INJURED )
            x = -RES_X/4-250
        end
        self.biganim:SetScale( 1 )
        self.biganim:SetPos( x-OFFSET, y )
        self.biganim:MoveTo( x, y, .6, easing.outQuad)
    end
end

function GameOverScreen:RefreshEnemyCharacter()
    if self.score.won then return end
    if self.killed_by then

        -- Update the info text
        local loctable = self.killed_by:LocTable()
        loctable.name = loctable.name or loctable.fullname or loctable.Who
        self.enemy_title:SetText( string.upper( loc.format( LOC"UI.GAME_OVER_SCREEN.ENEMY_TITLE", loctable ) ) )
        self.enemy_subtitle:SetText( loc.format( LOC"UI.GAME_OVER_SCREEN.ENEMY_SUBTITLE", loctable ) )
        self.enemy:Show()

        -- Update the anim
        local scale = 0.27 --* ( fight_data.battle_scale or 1 )
        self.enemy_anim:SetScale( -scale, scale )
            :Refresh( self.killed_by )
            :SetOutline( 5, UICOLOURS.HEALTH )

        self:Delay( 0.1 )
        self:LayoutEnemyCharacter()

        -- Animate in
        -- local pos_x, pos_y = self.enemy_anim_root:GetPos()
        -- self.enemy_anim_root:SetPos( pos_x, pos_y-40 )
        -- self.enemy_anim_root:MoveTo( pos_x, pos_y, .4, easing.outQuad )
    end
end

function GameOverScreen:LayoutEnemyCharacter()
    if self.score.won then return end
    if self.killed_by then

        -- Align the enemy info text
        self.enemy_subtitle:LayoutBounds( "right", "below", self.enemy_title )
            :Offset( 0, SPACING.TITLE_DESC )
        self.enemy_text_container
            :LayoutBounds( "right", nil, self.hitbox )
            :LayoutBounds( nil, "above", self.panel )
            :Offset( -60, 20 )
        self.enemy_glow:LayoutBounds( "center", "center", self.enemy_text_container )
            :Offset( 80, -200 )

        -- And the enemy relative to the text
        local text_x, text_y = self.enemy_text_container:GetPos()
        local text_w, text_h = self.enemy_text_container:GetScaledSize()
        local text_right_x, text_top_y = text_x+text_w/2, text_y+text_h/2

        -- Update the anim
        self.enemy_anim_root:SetPos( 0, 0 )
        local fight_data = self.killed_by.fight_data
        local head_offset_x, head_offset_y = 60, 0
        local anim_w, anim_h = self.enemy_anim_root:GetSize()
        if anim_w < 200 then head_offset_x = head_offset_x + 20 end -- Things the width of a regular humanoid (but not as wide as a Luminthian) should move right a bit
        if anim_w > 600 then head_offset_x = head_offset_x + 70 end -- Things the size of the Shroog should move right a bit
        if anim_h < 300 then head_offset_y = head_offset_y - 30 end -- Things the height of a Crayote should move down a bit

        -- Get the anim's own size (it has its own weird scale)
        local local_size_w, local_size_h = self.enemy_anim.anim:GetSize()
        -- Get the head's position within the anim's own size
        local local_head_x, local_head_y, local_head_z = self.enemy_anim.anim.model:SymbolToLocal( "head_swap", 0, 0, 0 )
        if fight_data.track_symbol then
            local_head_x, local_head_y, local_head_z = self.enemy_anim.anim.model:SymbolToLocal(fight_data.track_symbol, 0, 0, 0)
            head_offset_y = head_offset_y + 30 -- Drones should move up a bit
        end
        -- Calculate the proportional position of the head within the anim
        local relative_head_x, relative_head_y = local_head_x/local_size_w, local_head_y/local_size_h

        -- Get the widget's current size within the screen
        local scaled_size_w, scaled_size_h = self.enemy_anim:GetScaledSize()
        -- Calculate the head's position relative to that size
        local scaled_head_x, scaled_head_y = relative_head_x*scaled_size_w, relative_head_y*scaled_size_h

        -- Position the widget so the head is aligned with the top right corner of the text box
        self.enemy_anim_root:Show()
            :SetPos( text_right_x+scaled_head_x+head_offset_x, text_top_y-scaled_head_y+head_offset_y )

        -- local anim_x, anim_y, anim_z = self.enemy_anim:TransformToWorld(0,0,0)
        -- local dx, dy, dz = anim_x - head_x, anim_y - head_y, anim_z - head_z
        -- print ( "local size", local_size_w, local_size_h, "local head pos", local_head_x, local_head_y, "rel", relative_head_x, relative_head_y )
        -- print ( "- scaled size", scaled_size_w, scaled_size_h, "scaled head pos", scaled_head_x, scaled_head_y )
        -- print (dx, dy, "WWWWW")
        --if fight_data.track_symbol then
            --head_x, head_y, head_z = self.enemy_anim.anim.model:SymbolToWorld(fight_data.track_symbol, 0, 0, 0)
            -- head_offset_y = head_offset_y - 120
        --end
        -- local dx, dy, dz  = self.enemy_anim:TransformFromWorld( head_x, head_y, head_z )
        -- print (dx, dy,dz)
        -- self.enemy_anim_root:Show()
            --:SetPos( text_right_x - head_x + head_offset_x, text_top_y - head_y + head_offset_y )
            -- :SetPos(text_right_x, text_top_y)
        -- print("Offset", text_right_x, text_top_y)
        -- self.enemy_anim_root:LayoutBounds( "after", "top", self.enemy_text_container )
        --     :Offset( 0, 0 )
    end
end

------------------------------------------------------------------------------------------
-- Shows a large stats block

local StatsBlockDisplay = class( "Widget.StatsBlockDisplay", Widget )

local PADDING = 20
local ICON_SIZE = 40
local BUTTON_HEIGHT = 30


function StatsBlockDisplay:init( width, height, stats_score_data, amount, fraction, button_fn )
    StatsBlockDisplay._base.init(self)

    local colour = stats_score_data.COLOUR
    self.amount = amount
    self.exp = stats_score_data.EXP or 0

    -- Add our content block
    self.bg_border = 45
    self.bg_border_scale = 0.5
    self.content_bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/gameover_block.tex") ) )
    self.content_bg:SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :SetTintColour( colour )

    -- Size up our content block
    local nineslice_w, nineslice_h = width + self.bg_border*2*self.bg_border_scale, height + self.bg_border*2*self.bg_border_scale
    self.content_bg:SetSize( nineslice_w, nineslice_h )

    -- Add some glow!
    self.glow = self:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ), width+50, height+50 ) )
        :SetHiddenBoundingBox( true )
        :SetTintColour( colour )
        :SetTintAlpha( 0.2 )

    -- Our content block
    self.content = self:AddChild( Widget() )
    self.icon = self.content:AddChild( Widget.Image( engine.asset.Texture(stats_score_data.ICON), ICON_SIZE, ICON_SIZE ) )
        :Bloom( 0.08 )
    self.title = self.content:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE, stats_score_data.TITLE() ) )
        :SetGlyphColour( colour )
        :SetWordWrap( true )
        :SetAutoSize( width - PADDING*2 )
        :Bloom( 0.1 )
        :LayoutBounds( "center", "below", self.icon ):Offset( 0, -5 )
    self.subtitle = self.content:AddChild( Widget.Label( "body", FONT_SIZE.SMALL_TEXT, stats_score_data.SUBTITLE() ) )
        :SetGlyphColour( colour )
        :SetWordWrap( true )
        :SetAutoSize( width - PADDING*2 )
        :SetTintAlpha( 0.7 )
        :Bloom( 0.08 )
        :LayoutBounds( "center", "below", self.title ):Offset( 0, 5 )
    self.left_dash = self.content:AddChild( Widget.Image( engine.asset.Texture("images/white.tex" ), 30, 1.2 ) )
        :SetTintColour( colour )
        :SetTintAlpha( 0.5 )
        :Bloom( 0.1 )
        :LayoutBounds( "before", "center", self.subtitle ):Offset( -10, 0 )
    self.right_dash = self.content:AddChild( Widget.Image( engine.asset.Texture("images/white.tex" ), 30, 1.2 ) )
        :SetTintColour( colour )
        :SetTintAlpha( 0.5 )
        :Bloom( 0.1 )
        :LayoutBounds( "after", "center", self.subtitle ):Offset( 10, 0 )
    self.value_container = self.content:AddChild( Widget() )
    self.value = self.value_container:AddChild( Widget.Label( "body", 80, amount ) )
        :SetGlyphColour( colour )
        :Bloom( 0.12 )
    self.value_fraction = self.value_container:AddChild( Widget.Label( "title", 40, fraction and "/" .. fraction or "" ) )
        :SetGlyphColour( colour )
        :SetTintAlpha( 0.5 )
        :Bloom( 0.08 )
        :LayoutBounds( "after", "bottom", self.value ):Offset( 0, 7 )
    self.value_container:LayoutBounds( "center", "below", self.subtitle ):Offset( 0, 10 )
    self.content:LayoutBounds( "center", "center", self.content_bg ):Offset( 0, BUTTON_HEIGHT/3 )

    -- EXP floater!
    self.exp_floater = self:AddChild( Widget.ExpFloater( self.exp ) )
        :LayoutBounds( "center", "below", self.content ):Offset( 0, 10 ):Hide()

    -- Add a button if needed
    if stats_score_data.BUTTON_TEXT and button_fn then
        self:SetFocusable(false)
        self.button = self:AddChild( Widget.Clickable() )
        self.button:SetFocusable( true )
        self.button:SetOnClickFn( button_fn )
        self.button.OnGainFocus = function() self:UpdateButtonImage() end
        self.button.OnLoseFocus = function() self:UpdateButtonImage() end
        self.button.OnGainHover = function() self:UpdateButtonImage() end
        self.button.OnLoseHover = function() self:UpdateButtonImage() end

        -- Button bg
        self.button_bg_border = 20
        self.button_bg_border_scale = 0.5
        self.button_bg = self.button:AddChild( Widget.Panel( engine.asset.Texture("UI/gameover_button.tex") ) )
        self.button_bg:SetNineSliceCoords( self.button_bg_border, self.button_bg_border, self.button_bg_border, self.button_bg_border )
            :SetNineSliceBorderScale( self.button_bg_border_scale )
            :SetTintColour( colour )
            :Bloom( 0.03 )
        local nineslice_w, nineslice_h = width + self.button_bg_border*2*self.button_bg_border_scale, BUTTON_HEIGHT + self.button_bg_border*2*self.button_bg_border_scale
        self.button_bg:SetSize( nineslice_w, nineslice_h )

        -- Button label
        self.button_text = self.button:AddChild( Widget.Label( "body", FONT_SIZE.SMALL_TEXT, stats_score_data.BUTTON_TEXT() ) )
            :SetGlyphColour( UICOLOURS.DARK_BODYTEXT )
            :LayoutBounds( "left", "center", self.button_bg ):Offset( 10, 1 )

        -- Button icon
        self.button_icon = self.button:AddChild( Widget.Image( engine.asset.Texture("UI/cardxp_viewupgrade.tex"), 12, 12 ) )
            :LayoutBounds( "right", "center", self.button_bg ):Offset( -8, 0 )

        -- Align the button
        self.button:LayoutBounds( "left", "bottom", self.content_bg ):Offset( 0, 0 )
    else
        self:SetFocusable(true)
    end
end

function StatsBlockDisplay:UpdateButtonImage()
    if self.button.enabled and ( self.button.hover or self.button.focus ) then
        self.button_bg:Bloom( 0.1 )
    else
        self.button_bg:Bloom( 0.03 )
    end
    return self
end

function StatsBlockDisplay:PrepareAnimation()
    self.is_done_animating = false
    self:SetTintAlpha( 0 )
    self.value:SetText( 0 )
    self.value_container:LayoutBounds( "center", "below", self.subtitle ):Offset( 0, 10 )
    self.exp_floater:SetEXP( 0 )
        :LayoutBounds( "center", "below", self.content ):Offset( 0, 10 )
        :SetTintAlpha( 0 )

    return self
end

function StatsBlockDisplay:AnimateIn( on_done_fn )
    local fade_in_rate = 3
    local count_up_rate = 8

    -- Fade in
    AUDIO:PlayParamEvent( "event:/ui/game_over_screen/stat_appear", "position", self:GetPanPos() )
    while self:GetAlpha() < 1 and not self.fast do
        local dt = coroutine.yield()
        self:SetTintAlpha( self:GetAlpha() + fade_in_rate*dt )
    end

    -- Count up the score
    local current_value = 0
    local target_value = self.amount
    local exp_mult = self.amount>0 and (self.exp/self.amount) or 0
    while current_value < target_value and not self.fast do
        local dt = coroutine.yield()
        current_value = current_value + count_up_rate*dt
        -- Update the value and align it
        AUDIO:PlayParamEvent( "event:/ui/game_over_screen/xp_tick_up", "position", self.value_container:GetPanPos() )
        self.value:SetText( tostring( math.round( current_value ) ) )
        self.value_container:LayoutBounds( "center", "below", self.subtitle ):Offset( 0, 10 )
        -- If there's EXP, update that too
        if self.exp and exp_mult > 0 then
            self.exp_floater:SetEXP( math.round( current_value )*exp_mult )
                :LayoutBounds( "center", "below", self.content ):Offset( 0, 10 )
        end
    end
    self.value:SetText( tostring( self.amount ) )
    self.value_container:LayoutBounds( "center", "below", self.subtitle ):Offset( 0, 10 )
    if self.exp and exp_mult > 0 then
        self.exp_floater:SetEXP( self.amount*self.exp )
            :LayoutBounds( "center", "below", self.content ):Offset( 0, 10 )
            :AnimateIn()
        if on_done_fn then on_done_fn( self.amount*self.exp ) end
    end


    self.is_done_animating = true
    self:SetTintAlpha( 1 )

    return self
end

function StatsBlockDisplay:SkipAnimation()
    self.fast = true
end

function StatsBlockDisplay:IsDoneAnimating()
    return self.is_done_animating
end

function StatsBlockDisplay:SetButtonText( text )
    self.button_text:SetText( text )
        :LayoutBounds( "left", "center", self.button_bg ):Offset( 15, 1 )
    return self
end

function StatsBlockDisplay:UpdateImage()
    if self.enabled and self.focus then
        self.icon:ScaleTo( nil, 1.1, 0.3, easing.inOutQuad )
        self.title:Bloom( 0.15 )
        self:ScaleTo( nil, 1.05, 0.2, easing.outQuad )
    else
        self.icon:ScaleTo( nil, 1, 0.3, easing.inOutQuad )
        self.title:Bloom( 0.1 )
        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
    end
end

function StatsBlockDisplay:OnGainFocus()
    StatsBlockDisplay._base.OnGainFocus(self)
    self:UpdateImage()
end

function StatsBlockDisplay:OnLoseFocus()
    StatsBlockDisplay._base.OnLoseFocus(self)
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- Shows a smaller stats display widget

local ApproachContainer = class("Widget.ApproachContainer", Widget)

local SIDE_SPACING = 40
local BLOCK_SPACING = 20
local ICON_SIZE = 30

function ApproachContainer:init( width, height, columns )
    ApproachContainer._base.init(self)

    -- Add our content block
    self.bg_border = 45
    self.bg_border_scale = 0.5
    self.content_bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/gameover_wideblock.tex") ) )
    self.content_bg:SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )

    -- Size up our content block
    local nineslice_w, nineslice_h = width + self.bg_border*2*self.bg_border_scale, height + self.bg_border*2*self.bg_border_scale
    self.content_bg:SetSize( nineslice_w, nineslice_h )

    self.column_count = columns
    self.block_width = ( width - (columns-1)*BLOCK_SPACING - SIDE_SPACING*2 )/columns
    self.block_height = height - BLOCK_SPACING*2

    -- Add a container for our blocks
    self.block_container = self:AddChild( Widget() )

    -- And left/right arrows
    local arrow_size = 24
    self.arrow_left = self:AddChild( Widget.ImageButton( engine.asset.Texture("UI/gameover_arrow_left.tex"), function() self:SetPage( self.current_page - 1 ) end ) )
        :SetHiddenBoundingBox( true )
        :SetFocusable( false )
        :SetSize( arrow_size, arrow_size )
        :LayoutBounds( "left", "center", self.content_bg )
        :Offset( 10, 0 )
        :Bloom( 0.1 )
        :Hide()
    self.arrow_right = self:AddChild( Widget.ImageButton( engine.asset.Texture("UI/gameover_arrow.tex"), function() self:SetPage( self.current_page + 1 ) end ) )
        :SetHiddenBoundingBox( true )
        :SetFocusable( false )
        :SetSize( arrow_size, arrow_size )
        :LayoutBounds( "right", "center", self.content_bg )
        :Offset( -10, 0 )
        :Bloom( 0.1 )
        :Hide()

    -- Container for the pagination dots
    self.pagination = self:AddChild( Widget() )
        :Hide()

    -- Divider above the pagination dots
    self.divider = self:AddChild( Widget.Image( engine.asset.Texture("UI/gameover_divider.tex"), width*0.7, 2 ) )
        :LayoutBounds( "center", "bottom", self.content_bg )
        :Offset( 0, 0 )
        :Bloom( 0.1 )
        :Hide()
end

function ApproachContainer:OnAdded()
    TheGame:GetEvents():ListenForEvent( "focus_changed", self, self.OnFocusChanged )
end

function ApproachContainer:OnRemoved()
    TheGame:GetEvents():RemoveListener( self )
end

function ApproachContainer:OnGainFocus()
    local focused_idx = self:GetFocusedIndex()
end

function ApproachContainer:GetDefaultFocus()
    for k, box in ipairs( self.block_container.children ) do
        if box:IsShown() then return box end
    end
end

function ApproachContainer:GetFocusedIndex()
    for k, box in ipairs( self.block_container.children ) do
        if box:IsShown() and box.focus then return k end
    end
    return 1
end

function ApproachContainer:SetFocusedIndex( idx )
    if idx < 1 then idx = #self.block_container.children end
    if idx > #self.block_container.children then idx = 1 end
    for k, box in ipairs( self.block_container.children ) do
        if k == idx then box:SetFocus() end
    end
end

function ApproachContainer:OnFocusChanged( event_name, prev_focus, new_focus )
    if new_focus and self:IsAncestorOf( new_focus ) then
        if new_focus and new_focus.index and new_focus.page and new_focus.page ~= self.current_page then
            self:SetPage( new_focus.page )
        end
    end
end

function ApproachContainer:OnFocusMove( dir )
    if dir == "left" or dir == "right" then
        local focus = self:GetFE():GetFocusWidget()
            if focus and self:IsAncestorOf( focus ) then
            local next_focus = focus:GetFocusDir( dir )
            if next_focus then
                next_focus:SetFocus()
                return true
            end
        end
    else
        self._base.OnFocusMove( self, dir )
    end
end

function ApproachContainer:AddBlock( SCORE_DATA, value )
    local block = self.block_container:AddChild( Widget() )
    block.data = SCORE_DATA
    block.value = value
    block.hitbox = block:AddChild( Widget.SolidBox( self.block_width, self.block_height, 0x00FF0000 ) ):Hide()

    block.icon = block:AddChild( Widget.Image( engine.asset.Texture( SCORE_DATA.ICON ), ICON_SIZE, ICON_SIZE ) )
        :Bloom( 0.05 )
        :LayoutBounds( "left", "top", block.hitbox )

    local title_text = SCORE_DATA.TITLE and SCORE_DATA.TITLE( value ) or ""
    block.name = title_text .. #self.block_container.children
    block.title = block:AddChild( Widget.Label( "title", FONT_SIZE.BUTTON, title_text ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetWordWrap( true )
        :SetAutoSize( self.block_width - BLOCK_SPACING*2.5 )
        :OverrideLineHeight( 18 )
        :LeftAlign()
        :Bloom( 0.08 )
        :LayoutBounds( "after", "top", block.icon ):Offset( BLOCK_SPACING*0.5, 3 )

    local subtitle_text = SCORE_DATA.SUBTITLE and SCORE_DATA.SUBTITLE( value ) or ""
    block.subtitle = block:AddChild( Widget.Label( "body", FONT_SIZE.SMALL_TEXT, subtitle_text ) )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :SetWordWrap( true )
        :SetAutoSize( self.block_width - BLOCK_SPACING*3 )
        :OverrideLineHeight( 14 )
        :LeftAlign()
        :Bloom( 0.1 )
        :LayoutBounds( "left", "below", block.title ):Offset( 0, 5 )

    if SCORE_DATA.EXP and SCORE_DATA.EXP > 0 then
        local score = value*SCORE_DATA.EXP
        block.exp = score
        block.exp_floater = block:AddChild( Widget.ExpFloater( score ) )
            :LayoutBounds( "left", "below", block.subtitle ):Offset( 0, -5 )
    end

    -- Handle events
    block:SetFocusable( true )
    block.OnGainFocus = function() block:ScaleTo( nil, 1.1, 0.2, easing.outQuad ) end
    block.OnLoseFocus = function() block:ScaleTo( nil, 1, 0.2, easing.outQuad ) end
end

function ApproachContainer:ReNav()
    local previous_v = nil
    for k, v in ipairs( self.block_container.children ) do
        if previous_v then v:SetFocusDir( "left", previous_v, true ) end
        previous_v = v
    end
    if self.block_container.children[1] then
        self.block_container.children[1]:SetFocusDir( "left", previous_v, true )
    end
end


function ApproachContainer:Layout()

    -- Align the blocks
    for k, block in ipairs( self.block_container.children ) do
        -- Assign this item a spot. If there are three items per page,
        -- the first will get 0, the second will get 1, the third will get 2, 
        -- the fourth will get 0 again, and so on
        local spot = ( k - 1 ) % self.column_count
        -- Calculate the x-position based on what spot this block is at, including spacings
        local pos_x = self.block_width * spot + BLOCK_SPACING * spot
        -- Set the position
        block:SetPos( pos_x, 0 )
        -- Inform the block of what page it's on
        block.page = math.ceil( k / self.column_count )
        block.index = k
    end

    -- Clear any old pagination markers
    self.pagination:DestroyAllChildren()

    local page_count = math.ceil ( #self.block_container.children / self.column_count )
    if page_count > 1 then
        -- Add brand new ones
        for k = 1, page_count do
            local dot = self.pagination:AddChild( Widget.Clickable( ) )
            dot:SetFocusable( false )
            dot.hitbox = dot:AddChild( Widget.SolidBox( 67, 30, 0xFF000000 ))
            dot.image = dot:AddChild( Widget.Image( engine.asset.Texture("UI/motd_page.tex"), 67, 8 ) ):Bloom( 0.1 )
            dot:SetOnClickFn( function() self:SetPage( k ) end )
            dot:SetHoverFn( function( hover ) if hover then dot:ScaleTo( nil, 1.05, 0.1 ) dot:Bloom(0.2) else dot:ScaleTo( nil, 1, 0.1 ) dot:Bloom(0.1) end end )
        end
        -- Align them
        self.pagination:StackChildrenRow( 15 )
        self.pagination:LayoutBounds( "center", "below", self.content_bg ):Offset( 0, 12 )

        self.pagination:Show()
        self.arrow_left:Show()
        self.arrow_right:Show()
    else
        self.pagination:Hide()
        self.arrow_left:Hide()
        self.arrow_right:Hide()
    end

    self:ReNav()

    -- Set the current page
    self:SetPage( 1 )
end

function ApproachContainer:SetPage( page )
    local page_count = math.ceil ( #self.block_container.children / self.column_count )
    page = page or 1
    if page > page_count then page = 1 end
    if page < 1 then page = page_count end

    -- Update the pagination dots
    for k, dot in ipairs( self.pagination.children ) do
        if k == page then
            -- Current page
            dot:SetTintAlpha( 1 )
        else
            -- Not the current page..
            dot:SetTintAlpha( 0.3 )
        end
    end

    -- Update page widgets visibility
    for k, block in ipairs( self.block_container.children ) do
        if k > ( page*self.column_count - self.column_count ) and k <= ( page*self.column_count ) then
            block:Show()
        else
            block:Hide()
        end
    end

    -- Align vertically
    self.block_container:LayoutBounds( "left", "center", self.content_bg ):Offset( SIDE_SPACING, 0 )

    self.current_page = page
end

function ApproachContainer:NextPage()
    local pages = math.ceil( #self.block_container.children/self.column_count )
    self.current_page = self.current_page or 1
    self.current_page = self.current_page + 1
    if self.current_page > pages then self.current_page = 1 end
    self:SetPage( self.current_page )
end

function ApproachContainer:PrevPage()
    local pages = math.ceil( #self.block_container.children/self.column_count )
    self.current_page = self.current_page or pages
    self.current_page = self.current_page - 1
    if self.current_page < 1 then self.current_page = pages end
    self:SetPage( self.current_page )
end

function ApproachContainer:PrepareAnimation()
    self.is_done_animating = false
    self:SetTintAlpha( 0 )
    for k, block in ipairs( self.block_container.children ) do
        block:SetTintAlpha( 0 )
        if block.exp_floater then block.exp_floater:SetTintAlpha( 0 ) end
    end

    return self
end

function ApproachContainer:AnimateIn( on_done_fn )
    local fade_in_rate = 8
    local block_fade_in_rate = 3

    -- Fade in
    AUDIO:PlayEvent("event:/ui/game_over_screen/approach_list_appears")
    while self:GetAlpha() < 1 do
        local dt = coroutine.yield()
        self:SetTintAlpha( self:GetAlpha() + fade_in_rate*dt )
        if self.fast then
            break
        end
    end
    -- Not the blocks yet
    for k, block in ipairs( self.block_container.children ) do
        block:SetTintAlpha( 0 )
    end

    -- Fade in each approach block
    local page = 1
    local alpha = 0
    for k, block in ipairs( self.block_container.children ) do

        -- Fade in this block
        AUDIO:PlayParamEvent( "event:/ui/game_over_screen/approach_stat_appears", "position", block:GetPanPos() )
        alpha = 0
        while alpha < 1 and not self.fast do
            local dt = coroutine.yield()
            alpha = alpha + block_fade_in_rate*dt
            block:SetTintAlpha( alpha )
        end
        block:SetTintAlpha( 1 )
        if block.exp_floater then block.exp_floater:AnimateIn() end
        if on_done_fn then on_done_fn( block.exp ) end

        -- Check if we should change page
        if k%self.column_count == 0 then 
            self:SetPage( page+1, true )
        end

    end

    self.is_done_animating = true
    self:SetTintAlpha( 1 )

    return self
end

function ApproachContainer:SkipAnimation()
    self.fast = true
end

function ApproachContainer:IsDoneAnimating()
    return self.is_done_animating
end


------------------------------------------------------------------------------------------
-- Shows a progress bar and the upcoming rewards, if any

local ProgressUnlocks = class("Widget.ProgressUnlocks", Widget)

local PROGRESS_W = 800 * 0.85
local PROGRESS_H = 40 * 0.85

function ProgressUnlocks:init( content_w, character_id, gamestate )
    ProgressUnlocks._base.init(self)

    self.content_w = content_w

    -- Unlocks track!
    self.unlock_track = self:AddChild( Widget.UnlockTrack( self.content_w, 90 ) )
        :SetGamestate( gamestate )
        :SetCharacter( character_id )
        :SetFocusable(true)

    -- EXP progress bar
    self.progress_container = self:AddChild( Widget() ):SetStencilContext( true )
    self.progress_mask = Widget.Image( engine.asset.Texture("UI/gameover_progressbar_mask.tex") )
        :SetSize( PROGRESS_W, PROGRESS_H )
    self.progress = Widget.ProgressBar( PROGRESS_W, PROGRESS_H )
        :OrientH()
        :SetBGColour( 0x021A18FF )
        :SetBarColour( UICOLOURS.EXP )
        :SetBarBloom(0.05)
    self.progress_glow = Widget.Image( engine.asset.Texture("UI/gameover_progressbar_glow.tex") )
        :SetSize( PROGRESS_W/6, PROGRESS_H )
        :Bloom( 0.13 )
    self.progress_overlay = Widget.Image( engine.asset.Texture("UI/gameover_progressbar_overlay.tex") )
        :SetSize( PROGRESS_W, PROGRESS_H )
        :Bloom( 0.0 )
    self.progress_container:AddChildren
    {
        self.progress_mask:SetMask(true),
        self.progress:SetStencilTest(true),
        self.progress_glow:SetStencilTest(true),
        self.progress_overlay
    }:LayoutBounds( "left", "below", self.unlock_track )
        :Offset( 0, 15 + 3 )
        :SendToBack()

    -- Exp ratio under the bar
    self.exp_label = self.progress_container:AddChild( Widget.ExpFloater( 0, FONT_SIZE.SMALL_TEXT ) )
    -- Background texture
    self.total_backing = self.progress_container:AddChild( Widget.Image( engine.asset.Texture("UI/gameover_progress_bg.tex") ) )
        :SetSize( 500, 65 )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :LayoutBounds( "center", "below", self.progress )
        :Offset( 0, 2 )
    -- Total exp earned this round
    self.exp_total = self.progress_container:AddChild( Widget.ExpFloater( 0, 60 ) )
        :SetEXP( 0 )
        :LayoutBounds( "center", "below", self.progress )
        :Offset( 0, 0 )
        :Bloom( 0.14 )

    -- Showing if the user gained a bonus based on their advancement level
    self.advancement_bonus_backing = self.progress_container:AddChild( Widget.Image( engine.asset.Texture("UI/gameover_progress_bg.tex") ) )
        :SetSize( 300, 46 )
        :SetTintColour( UICOLOURS.OBJECTIVE )
        :LayoutBounds( "center", "below", self.total_backing )
        :Offset( 0, 0 )
        :Hide()
        :SetTintAlpha( 0 )
    self.advancement_bonus_label = self.progress_container:AddChild( Widget.Label( "body", FONT_SIZE.SCREEN_TABS, loc.format( LOC"UI.GAME_OVER_SCREEN.ADVANCEMENT_BONUS", 17 ) ) )
        :SetGlyphColour( UICOLOURS.OBJECTIVE )
        :Bloom( 0.08 )
        :LayoutBounds( "center", "below", self.total_backing )
        :Offset( 0, -5 )
        :Hide()
        :SetTintAlpha( 0 )

    -- Rewards!
    self.reward_backing = self:AddChild( Widget.Image( engine.asset.Texture("UI/gameover_progress_reward_bg.tex") ) )
        :SetSize( PROGRESS_W, PROGRESS_H )
        :LayoutBounds( "right", "top", self.progress_mask )
        :Offset( self.content_w - PROGRESS_W, 0 )
        :SetTintAlpha( 0.2 )
        :SendToBack()
        :Hide()
    self.reward_arrow = self:AddChild( Widget.Image( engine.asset.Texture("UI/gameover_rewards_arrow.tex") ) )
        :SetSize( 9, 9 )
        :LayoutBounds( "right", "center", self.reward_backing )
        :Offset( -184, 0 )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.08 )
        :Hide()
    self.rewards_label = self:AddChild( Widget.Label( "body", 22, LOC"UI.GAME_OVER_SCREEN.NEXT_REWARD" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.08 )
        :LayoutBounds( "before", "center", self.reward_arrow )
        :Offset( -6, 0.5 )
        :Hide()


    self.next_unlock_root = self:AddChild(Widget()):Hide()
    self.given_unlocks_root = self:AddChild( Widget() )
        :LayoutBounds( "center", "below", self.progress_overlay )
        :Offset( 0, -154 )
end

function ProgressUnlocks:OnAdded()
    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
end

function ProgressUnlocks:OnControlModeChange( cm, device_id )
    ProgressUnlocks._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
    end
    if cm == CONTROL_MODE.TOUCH then
    end
    if cm == CONTROL_MODE.GAMEPAD then
    end
end

function ProgressUnlocks:OnScreenModeChange( sm )
    ProgressUnlocks._base.OnScreenModeChange( self, sm )

    self.exp_label:SetLayoutScale( LAYOUT_SCALE[sm] )
        :LayoutBounds( "left", "below", self.progress )
        :Offset( 22, -2 )
end

function ProgressUnlocks:GetDefaultFocus()
    return self.unlock_track:GetDefaultFocus()
end

function ProgressUnlocks:SetAdvancementBonus( bonus )
    self.advancement_bonus_backing:Show()
        :AlphaTo( 1, 0.6 )
    self.advancement_bonus_label:SetText( loc.format( LOC"UI.GAME_OVER_SCREEN.ADVANCEMENT_BONUS", bonus ) )
        :LayoutBounds( "center", "below", self.total_backing )
        :Offset( 0, -5 )
        :Show()
        :AlphaTo( 1, 0.4 )
end

function ProgressUnlocks:UpdateDisplayTotal( exp )
    self.exp_total:SetEXP( exp )
        :LayoutBounds( "center", "below", self.progress )
        :Offset( 0, 2 )
end

function ProgressUnlocks:GetUnlockWidget( unlock_id, left_text )
    return Widget.RewardWidget( unlock_id, left_text )
end

function ProgressUnlocks:SetPts(pts, next_unlock )
    self.pts = pts
    
    if next_unlock then
        local p = clamp( (pts - next_unlock.start_pts) / (next_unlock.total_pts - next_unlock.start_pts), 0, 1)
        self.progress:SetProgress( p )
        self.progress_glow:LayoutBounds( "right", "center", self.progress.bar ):Offset( 15, 0 )
        self.exp_label:SetProgress( math.round( pts ), math.round( next_unlock.total_pts ) )
        self:RefreshUnlockWidget(next_unlock)
    else
        self.progress:SetProgress(1)
        self.progress_glow:LayoutBounds( "right", "center", self.progress.bar ):Offset( 15, 0 )
        self.exp_label:SetProgress( math.round( self.pts ), math.round( self.pts ) )
    end
    self.unlock = next_unlock

    self.exp_label:LayoutBounds( "left", "below", self.progress )
        :Offset( 22, -2 )
end

function ProgressUnlocks:ShowGain(character_id, start_pts, gain_pts, unlocks_awarded, next_award, on_done)
    if gain_pts > 0 then
        self:StartCoroutine(ProgressUnlocks.ShowGainCoro, self, character_id, start_pts, gain_pts, unlocks_awarded, next_award, on_done)
    else
        if on_done then
            on_done()
        end
    end
end

function ProgressUnlocks:AnimateBar( start_progress, dest_progress, ease )
    if self.unlock then
        local FULL_BAR_TIME = 1.8
        local MIN_ANIM_TIME = .5
        ease = ease or easing.linear
        local time_to_take = math.max( MIN_ANIM_TIME, (dest_progress - start_progress) * FULL_BAR_TIME)
        self.progress:SetBarBloom(.08)
        self.exp_label:Bloom(.15)
        self.exp_label:SetScale(1.1)
        
        local base_time_between_ticks = 1/20		--in seconds
        local num_ticks = time_to_take / base_time_between_ticks
        local p_per_tick = num_ticks > 0 and ( (dest_progress - start_progress) / num_ticks ) or 999999

        local tick_p_accum = p_per_tick
        local last_p = start_progress
        local t = 0
        while t < time_to_take do
            
            local p = ease(t, start_progress, dest_progress-start_progress, time_to_take)

            local delta_p = p - last_p
            tick_p_accum = tick_p_accum - delta_p
            last_p = p
            self.progress:SetProgress(p)
            self.progress_glow:LayoutBounds( "right", "center", self.progress.bar ):Offset( 15, 0 )
            
            if tick_p_accum <= 0 then
                AUDIO:PlayParamEvent( "event:/ui/game_over_screen/xp_tick_up", "position", self.progress_glow:GetPanPos() )
                tick_p_accum = p_per_tick
            end

            local pts = math.floor( self.unlock.pts *p )
            self.exp_label:SetProgress( pts, self.unlock.pts )
                :LayoutBounds( "left", "below", self.progress )
                :Offset( 22, -2 )
            t = t + coroutine.yield()
        end
        self.progress:SetProgress( dest_progress )

        local pts = math.floor( self.unlock.pts *dest_progress )
        self.exp_label:SetProgress( pts, self.unlock.pts )
            :LayoutBounds( "left", "below", self.progress )
            :Offset( 22, -2 )

        self.progress:SetBarBloom(0.05)
        self.exp_label:Bloom(0.1)
        self.exp_label:SetScale(1)
    end
end

function ProgressUnlocks:RefreshUnlockWidget(unlock)
    
    self.unlock = unlock
    
    local left_text = true
    local w = unlock and self:GetUnlockWidget( unlock, left_text )
    self.next_unlock_root:DestroyAllChildren()
    -- if w then
    --     self.next_unlock_root:Show():AddChild(w)
    --     w:LayoutBounds("right", "top", self.reward_backing ):Offset( -15, -10 )
    -- end
end

function ProgressUnlocks:PresentUnlock(unlock)
    --local t = .3
    local done = false
    if self.on_new_unlock_fn then self.on_new_unlock_fn() end
    self.unlock_track:UnlockItem( unlock, function()
        done = true
    end )

    while not done do
        coroutine.yield()
    end

    done = false
    TheGame:FE():InsertScreen( Screen.UnlockInspectorScreen( unlock, true, function() done = true end ) )
    while not done do
        coroutine.yield()
    end
end

function ProgressUnlocks:SetOnNewUnlock( fn )
    self.on_new_unlock_fn = fn
end

function ProgressUnlocks:ShowGainCoro(character_id, start_pts, gain_pts, unlocks_awarded, next_award, on_done)


    local show = {}
    local pts = start_pts
    for k,v in ipairs(unlocks_awarded) do
        table.insert( show, {unlock = v, start_pts = pts, end_pts = v.total_pts, do_unlock = true } )
        pts = v.total_pts
    end
    
    if next_award then
        table.insert( show, {unlock = next_award, start_pts = pts, end_pts = start_pts+gain_pts } )
    end

    for k, v in ipairs(show) do
        self:SetPts(v.start_pts, v.unlock)

        local start_progress = self.progress:GetProgress()
        local dest_progress = (v.end_pts - v.unlock.start_pts) / (v.unlock.total_pts - v.unlock.start_pts)

        self:AnimateBar(start_progress, dest_progress, easing.outQuad)
        if v.do_unlock then
            AUDIO:PlayEvent("event:/ui/game_over_screen/reward_unlocked")
            self:PresentUnlock(v.unlock)
        end
    end


    if on_done then
        on_done()
    end
end

------------------------------------------------------------------------------------------
-- Displays a reward unlock as a widget

local RewardWidget = class("Widget.RewardWidget", Widget.Clickable)

local REWARD_W = 160
local REWARD_H = 90
local ANIM_SCALE = 0.3
local CARD_SCALE = 0.26
local MASK_TOP_OVERFLOW = 30

function RewardWidget:init( unlock, left_text )
    RewardWidget._base.init(self)

    -- Background panel
    local bg_border = 33
    local bg_border_scale = 0.5
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/rewardwidget_bg.tex") ) )
        :SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
        :SetNineSliceBorderScale( bg_border_scale )
        :SetSize( REWARD_W, REWARD_H )
        :Bloom( 0.2 )

    -- Character anim
    self.mask = self:AddChild( Widget() ):SetStencilContext( true ):SetHiddenBoundingBox( true )
    self.mask_image = Widget.Panel( engine.asset.Texture("UI/rewardwidget_mask.tex") )
    self.mask_image:SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
    self.mask_image:SetNineSliceBorderScale( bg_border_scale )
    self.mask_image:SetSize( REWARD_W, REWARD_H+MASK_TOP_OVERFLOW )
    self.anim = Widget.AnimatedCharacter()
        :SetScale( ANIM_SCALE )
        :SetOutline( 5, UICOLOURS.SUBTITLE )
    self.content = Widget()
        :SetHiddenBoundingBox( true )
    self.gradient = Widget.Panel( engine.asset.Texture("UI/rewardwidget_gradient.tex") )
    self.gradient:SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
    self.gradient:SetNineSliceBorderScale( bg_border_scale )
    self.gradient:SetSize( REWARD_W, REWARD_H )
    self.gradient:SetTintAlpha( 1 )
    self.mask:AddChildren
    {
        self.mask_image:SetMask(true),
        self.anim:SetStencilTest(true),
        self.content,--:SetStencilTest(true),
        self.gradient,
    }
    self.mask_image:LayoutBounds( "center", "bottom", self.bg )
    self.anim:SetPos( -30, -160 )

    -- Overlay panel
    self.overlay = self:AddChild( Widget.Panel( engine.asset.Texture("UI/rewardwidget_overlay.tex") ) )
        :SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
        :SetNineSliceBorderScale( bg_border_scale )
        :SetSize( REWARD_W, REWARD_H )
        :Bloom( 0.2 )

    -- Text info
    self.text_content = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.title = self.text_content:AddChild( Widget.Label("title", 22 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :RightAlign()
        :SetWordWrap( true )
        :OverrideLineHeight( 18 )
        :SetAutoSize( REWARD_W )
        :Bloom( 0.09 )
    self.subtitle = self.text_content:AddChild( Widget.Label("body", 18 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.8 )
        :RightAlign()
        :SetWordWrap( true )
        :OverrideLineHeight( 14 )
        :SetAutoSize( REWARD_W )
        :Bloom( 0.05 )

    self:Refresh( unlock, left_text )

    self:SetOnClickFn(function() if self.unlock then TheGame:FE():InsertScreen( Screen.UnlockInspectorScreen( self.unlock ) ) end end)
end

function RewardWidget:Refresh( unlock, left_text )
    self.unlock = unlock
    self.left_text = left_text

    self.title:SetText( self.unlock:GetLocalizedName() )

    self.content:AddChild( Widget.Image( unlock.icon or global_images.circle, 64, 64 ))
    self.title:SetText( unlock:GetLocalizedName() )
    self.subtitle:SetText( LOC"UI.GAME_OVER_SCREEN.REWARD_UNLOCK" )
    self.bg:SetTintColour( UICOLOURS.SUBTITLE )
    self.gradient:SetTintColour( UICOLOURS.SUBTITLE )
    self.overlay:SetTintColour( UICOLOURS.SUBTITLE )


    self.gradient:SetTintAlpha( 0.5 )
    if self.left_text then
        self.title
            :RightAlign()
            :SetFontSize( 32 )
            :OverrideLineHeight( 28 )
        self.subtitle
            :RightAlign()
            :SetFontSize( 22 )
            :OverrideLineHeight( 18 )
            :LayoutBounds( "right", "below", self.title )
            :Offset( 0, 2 )
        self.text_content
            :LayoutBounds( "before", "center", self.bg )
            :Offset( -20, -10 )
    else
        self.title
            :CenterH()
            :SetFontSize( 22 )
            :OverrideLineHeight( 18 )
            :LayoutBounds( "center", "below", self.bg )
            :Offset( 0, 0 )
        self.subtitle
            :CenterH()
            :SetFontSize( 18 )
            :OverrideLineHeight( 14 )
            :LayoutBounds( "center", "below", self.title )
            :Offset( 0, 2 )
    end
end

------------------------------------------------------------------------------------------
-- Shows the prestige increasing

local PrestigeIncreasePanel = class("Screen.PrestigeIncreasePanel", Widget.Screen)

function PrestigeIncreasePanel:init()
    PrestigeIncreasePanel._base.init(self)

    self:SetAnchors( "center", "center" )
    self.is_popup = true

    self.backing = self:AddChild( Widget.SolidBox( RES_X, RES_Y, 0x301730FF ) )
        :SetAnchors( "fill", "fill" )
        :SetTintAlpha( 0 )
        :BlockMouse()
    self.panel = self:AddChild( Widget.WidePanel( 580, UICOLOURS.ADVANCEMENT ) )
        :SetIcon( engine.asset.Texture( "UI/ic_advancement_icon.tex" ) )
        :PrepareAnimation()

    self.anim_container = self:AddChild( Widget() )
        :SetPos( 0, -60 )
    self.current_prestige_icon = self.anim_container:AddChild( Widget.Image() )
        :SetSize( 300, 300 )
        :Bloom( 0.25 )
        :SetScale( 0.8 )

    -- Continue button
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    if GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
    self.button_continue = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.GAME_OVER_SCREEN.PRESTIGE_BUTTON", function() self:FadeOut() end ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Hide()
end

function PrestigeIncreasePanel:AnimatePrestige( current_prestige, new_prestige, act_data, fn )
    self.current_prestige = current_prestige
    self.new_prestige = new_prestige
    self.prestige_data = act_data.background.data.advancement

    self.panel:SetTitles( 
        LOC"UI.GAME_OVER_SCREEN.PRESTIGE_TITLE",
        act_data:GetLocalizedName()
    )

    AUDIO:PlayParamEvent("event:/ui/game_over_screen/prestige_up", self.new_prestige)

    local tex = self.current_prestige > 0 and self.prestige_data[self.current_prestige].icon or engine.asset.Texture( "UI/advancement_level_0.tex" )
    self.current_prestige_icon:SetTexture( tex )
    self.backing:AlphaTo( 0.95, 0.3, easing.inOutQuad )
    self.panel:FadeIn( function() self:FadedIn() end )
    self.on_done_fn = fn

    return self
end

function PrestigeIncreasePanel:FadedIn()
    self:RunUpdater(
        Updater.Series{
            Updater.Do( function()

                -- Lens flares
                local animname = "anim/lens_flare_fx.zip"
                local anim = self.anim_container:AddChild( Widget.Anim() )
                    :SetHiddenBoundingBox(true)
                    :SetTintColour( UICOLOURS.QUEST )
                    :SetScale( 1, 2 )
                    :SetPos( 0, 0 )
                    :SetSpeed( 1.0 )
                    :SetBloom( 0.2 )
                    :SetTintAlpha( 0.1 )
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:Play( "fx", true )

            end ),
            Updater.Do( function()
                -- Lens flares
                local animname = "anim/UI_negotiation_round_fx.zip"
                local anim = self.anim_container:AddChild( Widget.Anim() )
                    :SetHiddenBoundingBox(true)
                    :SetTintColour( UICOLOURS.SUBTITLE )
                    :SetScale( 6, 3 )
                    :SetPos( -200, 0 )
                    :SetSpeed( 0.26 )
                    :SetBloom( 0.2 )
                    :SetTintAlpha( 0.05 )
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:Play( "lens_flare", true )
            end ),
            Updater.Do( function()

                -- Gather energy
                local animname = "anim/UI_negotiation_round_fx.zip"
                local anim = self.anim_container:AddChild( Widget.Anim() )
                    :SetHiddenBoundingBox(true)
                    :SetTintColour( UICOLOURS.ADVANCEMENT )
                    :SetTintAlpha( 0 )
                    :AlphaTo( 1, 0.4, easing.outQuad )
                    :SetScale( 2.7 )
                    :SetSpeed( 1.0 )
                    :SetBloom( 0.2 )
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:Play( "special_3pre", false )
                    :Push( "special_3loop" )
                    :Push( "special_3pst" )

            end ),
            Updater.Ease( function(v) self.current_prestige_icon:SetScale(v) end, 0.8, 1, 1.8, easing.outQuad ),
            Updater.Do( function()

                -- Lens flares
                local animname = "anim/UI_negotiation_round_fx.zip"
                local anim = self.anim_container:AddChild( Widget.Anim() )
                    :SetHiddenBoundingBox(true)
                    :SetTintColour( UICOLOURS.TITLE )
                    :SetScale( 8, 2 )
                    :SetPos( 100, 10 )
                    :SetSpeed( 0.67 )
                    :SetBloom( 0.2 )
                    :SetTintAlpha( 0.06 )
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:Play( "lens_flare", true )

            end ),
            Updater.Do( function() 

                -- Circular impact
                local animname = "anim/UI_negotiation_round_fx.zip"
                local anim = self.anim_container:AddChild( Widget.Anim() )
                    :SetHiddenBoundingBox(true)
                    :SetTintColour( UICOLOURS.ADVANCEMENT )
                    :SetScale( 2.4 )
                    :SetSpeed( 1.4 )
                    :SetBloom( 0.2 )
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:Play( "hit_3", false )

            end ),
            Updater.Wait( 0.2 ),
            Updater.Do( function()

                -- Lens flares
                local animname = "anim/UI_negotiation_round_fx.zip"
                local anim = self.anim_container:AddChild( Widget.Anim() )
                    :SetHiddenBoundingBox(true)
                    :SetTintColour( UICOLOURS.SUBTITLE )
                    :SetScale( 7, 4 )
                    :SetPos( 150, -20 )
                    :SetSpeed( 0.41 )
                    :SetBloom( 0.2 )
                    :SetTintAlpha( 0.08 )
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:Play( "lens_flare", true )

            end ),
            Updater.Do( function()

                -- Particles
                local animname = "anim/UI_negotiation_round_fx.zip"
                local anim = self.anim_container:AddChild( Widget.Anim() )
                    :SetHiddenBoundingBox(true)
                    :SetTintColour( UICOLOURS.ADVANCEMENT )
                    :SetRotation( 24*DEGREES )
                    :SetScale( 2.5 )
                    :SetSpeed( 1.0 )
                    :SetBloom( 0.2 )
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:Play( "hit_4", false )

            end ),
            Updater.Wait( 0.1 ),
            Updater.Do( function()

                -- Particles
                local animname = "anim/UI_negotiation_round_fx.zip"
                local anim = self.anim_container:AddChild( Widget.Anim() )
                    :SetHiddenBoundingBox(true)
                    :SetTintColour( UICOLOURS.QUEST )
                    :SetScale( 2.0 )
                    :SetSpeed( 0.8 )
                    :SetBloom( 0.2 )
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:Play( "hit_4", false )

            end ),
            Updater.Do( function()
                self.current_prestige_icon:SetTexture( self.prestige_data[self.new_prestige].icon )
            end ),
            Updater.Ease( function(v) self.current_prestige_icon:SetScale(v) end, 1.3, 1, 0.5, easing.outQuad ),
            Updater.Wait( 0.5 ),
            Updater.Do( function()
                self.panel:SetTitles( 
                    LOC"UI.GAME_OVER_SCREEN.PRESTIGE_TITLE",
                    loc.format( LOC"UI.GAME_OVER_SCREEN.PRESTIGE_SUBTITLE", self.new_prestige )
                )
            end ),
            Updater.Wait( 0.8 ),
            Updater.Do( function()
                self.button_continue:Show()
                self.button_continue:SetFocus()
            end ),
        }
    )
end

function PrestigeIncreasePanel:FadeOut()
    self.anim_container:AlphaTo( 0, 0.3, easing.inOutQuad )
    self.backing:AlphaTo( 0, 0.3, easing.inOutQuad )
    self.panel:FadeOut( function() self:FadedOut() end )
end

function PrestigeIncreasePanel:FadedOut()
    self:GetFE():PopScreen( self )
    if self.on_done_fn then self.on_done_fn() end
end

function PrestigeIncreasePanel:GetDefaultFocus()
    if self.button_continue:IsShown() then return self.button_continue end
end