
require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/solidbox"
require "ui/widgets/panelbutton"
require "ui/widgets/iconbutton"
require "ui/widgets/panel"
require "ui/widgets/slider"
require "ui/widgets/checkbox"
require "ui/widgets/cardpack"

-------------------------------------------------------------------------
--

local PANEL_H = 850

local DESELECTED_TINT = .8

local GridDraftPopup= class("Screen.GridDraftPopup", Widget.Screen)


GridDraftPopup.CONTROL_MAP = 
{

    {
        controls = { Controls.Digital.GAMEPAD_X },
        
        fn = function(self)
            self:OnClickContinue()
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.ACCEPT_CARDS", Controls.Digital.GAMEPAD_X ))
        end,
    },

    {
        control = Controls.Digital.PAUSE, 
        fn = function(self)
            self:GetFE():InsertScreen(Screen.PauseMenu())
            return true
        end,
    },

}

function GridDraftPopup:init()
    Widget.Screen.init(self)


    self.rows = 3
    self.card_scale = .5
    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors("center", "center")

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Backing panel
    self.panel = self:AddChild( Widget.WidePanel( PANEL_H ) )
        --:SetIcon( engine.asset.Texture( "UI/ic_shop_icon.tex" ) )


    -- Cards panel
    self.card_root = self.panel:AddChild( Widget() )
    self.card_widgets = {}
    
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.button_continue = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.DRAFT_SCREEN.GRID_PICK_BUTTON", function() self:OnClickContinue() end ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Disable()
        --:Hide()
        --:SetToolTip( LOC"UI.OPTIONS_SCREEN.CONFIRM_SELECT_TOOLTIP" )



    self:SetSounds( nil, SoundEvents.card_select_screen_close )
    self:SetMusicEvent( TheGame:LookupPlayerMusic( "deck_music" ))
end

function GridDraftPopup:OnOpen()
    GridDraftPopup._base.OnOpen( self )
    self.screen_loop = AUDIO:CreateEventInstance(SoundEvents.card_select_screen_open)
    self.screen_loop:Start()

    local overlay = self:GetFE():FindScreen( Screen.MainOverlay )
    if overlay then
        overlay:HideOverlay()
    end
end

function GridDraftPopup:OnClose()
    GridDraftPopup._base.OnClose( self )

    local overlay = self:GetFE():FindScreen( Screen.MainOverlay )
    if overlay then
        overlay:ShowOverlay()
    end

    self.screen_loop:Stop()
    self.screen_loop:Release()
    self.screen_loop = nil
end


function GridDraftPopup:OnControlModeChange( cm, device_id )
    GridDraftPopup._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.bottom_right:SetShown( true )
    elseif cm == CONTROL_MODE.TOUCH then
        self.bottom_right:SetShown( true )
    elseif cm == CONTROL_MODE.GAMEPAD then
        self.bottom_right:Hide()
        
        if self.card_widgets and self.card_widgets[1] then
            self.card_widgets[1]:SetFocus()
        end
    end
    self:DoLayout()
end


function GridDraftPopup:OnScreenModeChange( sm )
    GridDraftPopup._base.OnScreenModeChange( self, sm )
    self:DoLayout()
end


function GridDraftPopup:DoLayout()

    self.panel:SetPos(0, -50)
    local sm = TheGame:FE():GetScreenMode()        
    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:UpdateTitle()

    local offset, panel_h
    if sm == SCREEN_MODE.MONITOR then
        self.rows = 2
        self.card_scale = 0.6
        offset = 0
        panel_h = PANEL_H - 250
        self.button_continue:LayoutBounds( "right", "above", -60, 60 )

    elseif sm == SCREEN_MODE.SMALL or sm == SCREEN_MODE.TV then
        self.rows = 2
        self.card_scale = 0.7
        offset = 0
        panel_h = PANEL_H
        self.button_continue:LayoutBounds( "right", "above", -60, 20 )
    end

    -- self.panel:SetHeight( panel_h )

    local columns = math.ceil(#self.card_widgets / self.rows)
    for k,v in pairs(self.card_widgets) do
        v:SetScale(self.card_scale)
    end

    self.card_root:LayoutChildrenInGrid( columns, 10 )
    
    for k,widget in ipairs(self.card_widgets) do
        local col = (k-1) % columns
        local row = math.floor(k / 10)
        local left_card = col > 0 and self.card_widgets[k-1]
        local right_card = col < columns-1 and self.card_widgets[k+1]
        local up_card = self.card_widgets[k - columns]
        local down_card = self.card_widgets[k + columns]
        
        widget:SetFocusDir("up", up_card)
        widget:SetFocusDir("down", down_card)
        widget:SetFocusDir("left", left_card)
        widget:SetFocusDir("right", right_card)
    end



    self.card_root:LayoutBounds("center", "center", 0, offset)
end


function GridDraftPopup:OnClickCard( card_widget )
    
    local selected = table.arrayfind( self.selected_card_widgets, card_widget) 
    
    if selected then
        card_widget:SetOutline()
        card_widget:SetTintColour(DESELECTED_TINT,DESELECTED_TINT,DESELECTED_TINT,1)
        table.arrayremove(self.selected_card_widgets, card_widget)
    else
        card_widget:SetTintColour(1.5,1.5,1.5,1)
        card_widget:SetOutline( UICOLOURS.FOCUS, true )
        table.insert(self.selected_card_widgets, card_widget)
    end

    self:UpdateTitle()
    
end


function GridDraftPopup:DoDraft( agent, card_class, diff, num_drafts, min_pick, max_pick, choices, on_end_fn, rng )
    self.drafted_cards = {}
    self.num_drafts = num_drafts
    self.diff = diff
    self.min_pick = min_pick
    self.max_pick = max_pick
    self.choices = choices

    self.agent = agent or TheGame:GetGameState():GetPlayerAgent()
    self.on_end_fn = on_end_fn
    self.rng = rng
    self.card_class = card_class
    self.current_draft = 1
    self:StartDraftPhase()
end


function GridDraftPopup:StartDraftPhase()
    
    self.selected_card_widgets = {}

    if self.rng then
        push_random( function( n, m )
            return self.rng:Random( n, m )
        end )
    end
    local cards
    if self.card_class == Negotiation.Card then
        cards = RewardUtil.GetNegotiationCards( self.diff, self.choices, self.agent )
        self.panel:SetColour( UICOLOURS.NEGOTIATION ):SetIcon( engine.asset.Texture( "UI/card_type_negotiation.tex" ) )
    else
        cards = RewardUtil.GetBattleCards( self.diff, self.choices, self.agent )
        self.panel:SetColour( UICOLOURS.FIGHT ):SetIcon( engine.asset.Texture( "UI/card_type_fight.tex" ) )
    end

    if self.rng then
        pop_random()
    end

    self.card_root:DestroyAllChildren()
    self.card_widgets = {}

    for k,card in ipairs(cards) do
        local card_widget
        if is_instance( card, Battle.Card ) then
            -- Battle Card!
            card_widget = self.card_root:AddChild( Widget.BattleCard() )
                :RefreshCard( card )
        else
            -- Negotiation Card!
            card_widget = self.card_root:AddChild( Widget.NegotiationCard() )
                :RefreshCard( card )
        end
        card_widget:SetScale(self.card_scale)
        card_widget:SetTintColour(DESELECTED_TINT,DESELECTED_TINT,DESELECTED_TINT,1)
        --local card_widget = card_class(card.id)
        --DBG(card_widget)
        --card_widget:RefreshCard( card )    
        table.insert(self.card_widgets, card_widget)
        --self.card_root:AddChild(card_widget)

        card_widget:SetOnClick( function() self:OnClickCard( card_widget ) end )
        
        card_widget.OnGainFocus = function() card_widget._base.OnGainFocus(card_widget) self:UpdateCardImageState( card_widget ) end
        card_widget.OnLoseFocus = function() card_widget._base.OnLoseFocus(card_widget) self:UpdateCardImageState( card_widget ) end

        card_widget.OnGainHover = function() card_widget._base.OnGainHover(card_widget) self:UpdateCardImageState( card_widget ) end
        card_widget.OnLoseHover = function() card_widget._base.OnLoseHover(card_widget) self:UpdateCardImageState( card_widget ) end

    end

    self:DoLayout()
    self:UpdateTitle()

    local cm = TheGame:FE():GetControlMode()
    if cm == CONTROL_MODE.GAMEPAD then
        if self.card_widgets and self.card_widgets[1] then
            self.card_widgets[1]:SetFocus()
        end
    end


end

function GridDraftPopup:UpdateCardImageState( widget )

    if widget:HasFocus() or widget.hover then
        widget:ScaleTo( nil, self.card_scale*1.1, 0.2, easing.outQuad )
    else
        widget:ScaleTo( nil, self.card_scale, 0.2, easing.outQuad )
    end

    local cm = TheGame:FE():GetControlMode()
    
    if cm == CONTROL_MODE.GAMEPAD then
        if widget:HasFocus() then
            widget:SetOutline( UICOLOURS.FOCUS, true ) 
        else
            widget:SetOutline( ) 
        end
    else
        if table.arrayfind(self.selected_card_widgets, widget) then
            widget:SetOutline( UICOLOURS.FOCUS, true ) 
        else
            widget:SetOutline( ) 
        end

    end
end


function GridDraftPopup:GetDefaultFocus()
    return self.card_widgets[1]
end

function GridDraftPopup:UpdateTitle()
    if not self.selected_card_widgets then
        return
    end

    local title = loc.format( self.card_class == Negotiation.Card and LOC"UI.DRAFT_SCREEN.NEGOTIATION_GRID_TITLE" or LOC"UI.DRAFT_SCREEN.BATTLE_GRID_TITLE", self.current_draft, self.num_drafts)
    local subtitle = ""
    local num_selected = #self.selected_card_widgets
    if num_selected < self.min_pick then
        subtitle = loc.format(LOC"UI.DRAFT_SCREEN.GRID_PICK_SUBTITLE_TOO_LITTLE", num_selected, self.min_pick, self.max_pick)
    elseif num_selected > self.max_pick then
        subtitle = loc.format(LOC"UI.DRAFT_SCREEN.GRID_PICK_SUBTITLE_TOO_MANY", num_selected, self.min_pick, self.max_pick)
    else
        subtitle = loc.format(LOC"UI.DRAFT_SCREEN.GRID_PICK_SUBTITLE", num_selected, self.min_pick, self.max_pick)
    end

    local title, subtitle, titles = self.panel:UpdateTitles( title, subtitle )
    titles:SetPos( 0, -150 ):SetAnchors( "center", "top" )
    title:LayoutBounds( "before", "bottom", -150, 0 )
    subtitle:LayoutBounds( "after", "bottom", 150, 0 )

    self.button_continue:SetEnabled(num_selected >= self.min_pick and num_selected <= self.max_pick)
end

-- Buys the self.selected_card card
function GridDraftPopup:OnClickContinue( )
    
    local num_selected = #self.selected_card_widgets
    if num_selected < self.min_pick or num_selected > self.max_pick then
        return
    end

    for k,w in ipairs(self.selected_card_widgets) do
        
        local card = w.card
        if card.is_negotiation then
            self.agent.negotiator:AddCard( card )
        else
            self.agent.battler:AddCard( card )
        end
    end


    if self.num_drafts > self.current_draft then
        self.current_draft = self.current_draft + 1
        self:StartDraftPhase()        
    else
        if self.on_end_fn then
            self.on_end_fn()
        end

        if self:IsOnStack() then
            self:GetFE():PopScreen( self )
        end

    end
end