require "ui/widgets/screen"


local SIDEBAR_W = 540

----------------------------------------------------------------------

local HistoryScreen = class("Screen.HistoryScreen", Widget.Screen)

HistoryScreen.CONTROL_MAP = 
{
    -- {
    --     control = Controls.Digital.MENU_LEFT,
    --     fn = function(self)
    --         self:SwitchGamepadFocus( true )
    --         return true
    --     end,
    --     test = function(self)
    --         return not self.sidebar_focus
    --     end,
    -- },
    {
        control = Controls.Digital.MENU_RIGHT,
        fn = function(self)
            -- self:SwitchGamepadFocus( false )
            return true
        end,
        test = function(self)
            return self.sidebar_focus
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.HISTORY_SCREEN.CONTROLS_DETAILS", Controls.Digital.MENU_RIGHT))
        end
    },
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self)
            if TheGame:FE():GetControlMode() == CONTROL_MODE.GAMEPAD then
                -- If we're in gamepad mode and focused on the details, pressing back should take us to the sidebar focus
                if not self.sidebar_focus then
                    self:SwitchGamepadFocus( true )
                else
                    -- We're in sidebar focus, so leave the screen
                    if not self.animating_out then self:OnClickClose() end
                end
            else
                -- Not in gamepad mode, so just leave the screen
                if not self.animating_out then self:OnClickClose() end
            end
            return true
        end,
        hint = function(self, left, right)
            if not self.sidebar_focus then
                -- Focusing on details. Pressing back should take us to sidebar focus
                table.insert(right, loc.format(LOC"UI.HISTORY_SCREEN.CONTROLS_LIST", Controls.Digital.MENU_CANCEL))
            else
                table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
            end
        end
    },
}


function HistoryScreen:init(on_close)
	Widget.Screen.init( self )

    self:SetupUnderlay( true )
	self:SetAnchors("center", "center")

	-- Header
    self.header = self:AddChild( Widget() ):SetAnchors( "left", "top" )
    self.title = self.header:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_TITLE, LOC"UI.HISTORY_SCREEN.TITLE" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize( RES_X/2 )
        :SetWordWrap( true )
        :Bloom( 0.1 )
        :LayoutBounds( "left", "top", 60, -60 )
    self.subtitle = self.header:AddChild( Widget.Label( "title", FONT_SIZE.BODY_TEXT, LOC"UI.HISTORY_SCREEN.SUBTITLE" ) )
        :LeftAlign()
        :SetAutoSize( RES_X/2 )
        :SetWordWrap( true )
        :Bloom( 0.05 )
        :LayoutBounds( "left", "below", self.title ):Offset( 0, 0 )
    self.balance = self.header:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_TITLE, "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :RightAlign()
        :SetSize( SIDEBAR_W-22, 60 )
        :Bloom( 0.1 )

    -- Backing
    self.widepanel = self:AddChild( Widget.WidePanel( 800, UICOLOURS.SUBTITLE ) )

    -- For gamepad mode
    self.sidebar_gradient = self:AddChild( Widget.Image( engine.asset.Texture("UI/compendium_graft_gradient.tex") ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
    self.details_gradient = self:AddChild( Widget.Image( engine.asset.Texture("UI/compendium_graft_gradient.tex") ) )
        :SetTintColour( UICOLOURS.SUBTITLE )

    -- Sidebar
    self.sidebar_scroll = self:AddChild( Widget.ScrollPanel() )
        :SetVirtualTopMargin( 10 )
        :SetVirtualBottomMargin( 10 )
        :SetScrollBarOuterMargin( 0 )
        :LeftScrollBar( true )
    self.sidebar_list = self.sidebar_scroll:AddScrollChild( Widget() )
    -- Allow focus to jump from the details panel back to the sidebar on the left
    self.sidebar_list.OnFocusMove = function( elem, dir )
        if dir == "right" then
            self:SwitchGamepadFocus( false )
            return true
        end
    end

    -- Details area scroll
    self.details_scroll = self:AddChild( Widget.ScrollPanel() )
        :SetVirtualTopMargin( 10 )
        :SetVirtualBottomMargin( 50 )
        :SetScrollBarOuterMargin( 0 )
    self.details_panel = self.details_scroll:AddScrollChild( Widget.HistoryDetailsPanel() ):Hide()
    -- Allow focus to jump from the details panel back to the sidebar on the left
    self.details_panel.OnFocusMove = function( elem, dir )
        if dir == "left" then
            self.last_details_focus = TheGame:FE():GetFocusWidget()
            self:SwitchGamepadFocus( true )
            return true
        end
    end

    -- Close button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.close_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", function() self:OnClickClose() end ))
        :SetIcon( global_images.close )
        :LayoutBounds( "left", "above", 60, 60 )

    self:Refresh()
end

function HistoryScreen:Refresh()

	-- Clear old widgets
	self.sidebar_list:DestroyAllChildren()
	-- Add new ones
	local history = TheGame:GetRunHistory()
    local valid_runs = 0
    
    for k = #history, 1, -1 do
    	local gamestate = history[k]
    	-- if #self.sidebar_list.children == 8 then break end
    	if gamestate and gamestate.options and gamestate.options.start_act and GetPlayerActData( gamestate.options.start_act ) then
            local act = GetPlayerActData( gamestate.options.start_act )
            local agent_id = act.background.data.player_agent
            local def = Content.GetCharacterDef( agent_id )
            if def then
    	    	valid_runs = valid_runs + 1
                self.sidebar_list:AddChild( Widget.HistoryRunButton( SIDEBAR_W-45 ) )
    	            :SetOnClickFn( function() self:SelectRun( gamestate ) end )
    	            :SetFocusDelta( 10 )
    	            -- :ShowArrow()
    	    		:Refresh( gamestate )
    	            :LayoutBounds( "left", "below" )
    	            :Offset( 0, -10 )
            end
	    else
	    	-- Invalid act. Maybe corresponds to a run made with a mod that doesn't exist anymore
        end
    end

    -- Set run count
    self.balance:SetText( loc.format( LOC"UI.HISTORY_SCREEN.TOTAL_RUNS", valid_runs ))

	self:Layout()
	return self
end

function HistoryScreen:OnAdded()
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
end

function HistoryScreen:OnGainTop()
    -- Restore focus to the right widget
    if TheGame:FE():GetControlMode() == CONTROL_MODE.GAMEPAD and self.focus_widget then
        self.focus_widget:SetFocus()
    end
end

function HistoryScreen:OnLoseTop()
    -- Save which widget had focus
    self.focus_widget = TheGame:FE():GetFocusWidget()
end

function HistoryScreen:OnScreenResize(w, h)
    HistoryScreen._base.OnScreenResize(self, w, h)
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
end

function HistoryScreen:OnControlModeChange( cm, device_id )
    HistoryScreen._base.OnControlModeChange( self, cm, device_id )
 
    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.close_button:Show()
        self:StopGamepadFocus()
    end

    if cm == CONTROL_MODE.TOUCH then
        self.close_button:Show()
        self:StopGamepadFocus()
    end

    if cm == CONTROL_MODE.GAMEPAD then
        self.close_button:Hide()
        self:SwitchGamepadFocus( true )
    end

    self:Layout()
end

function HistoryScreen:GetDefaultFocus()
    if self.sidebar_focus then
        if self.selected_button then
            -- If there was a selected button, focus on it
            return self.selected_button
        else
            -- If not, then focus on the first one
            local button = self.sidebar_list.children and #self.sidebar_list.children > 0 and self.sidebar_list.children[1]
            if button then return button end
        end
    else
        -- Set focus to the top element on the details panel
        if self.last_details_focus then
            return self.last_details_focus
        else
            local focus = self.details_panel:GetDefaultFocus()
            if focus then return focus end
        end
    end
end

-- Focusing on the sidebar or on the details panel
function HistoryScreen:SwitchGamepadFocus( is_sidebar_focus )

    if self.sidebar_gradient:IsShown() and not is_sidebar_focus then
        -- Fade it out
        self.sidebar_gradient:AlphaTo( 0, 0.1, easing.outQuad )
    end
    if self.details_gradient:IsShown() and is_sidebar_focus then
        -- Fade it out
        self.details_gradient:AlphaTo( 0, 0.1, easing.outQuad )
    end
    if is_sidebar_focus then
        -- Fade it in
        self.sidebar_gradient:SetTintAlpha( 0 )
            :Show()
            :AlphaTo( 0.15, 0.2, easing.outQuad )

        -- Make sure only the correct scroll panel scrolls
        self.sidebar_scroll:Enable()
        self.details_scroll:Disable()
    end
    if not is_sidebar_focus then
        -- Fade it in
        self.details_gradient:SetTintAlpha( 0 )
            :Show()
            :AlphaTo( 0.15, 0.2, easing.outQuad )

        -- Make sure only the correct scroll panel scrolls
        self.sidebar_scroll:Disable()
        self.details_scroll:Enable()
    end

    self.sidebar_focus = is_sidebar_focus

    local focus = self:GetDefaultFocus()
    if focus then focus:SetFocus() end

    TheGame:FE():RefreshControlOptions()
end

function HistoryScreen:StopGamepadFocus()
    self.sidebar_gradient:Hide()
    self.details_gradient:Hide()
    self.sidebar_scroll:Enable()
    self.details_scroll:Enable()
end

function HistoryScreen:OnScreenModeChange( sm )
    HistoryScreen._base.OnScreenModeChange( self, sm )


    -- Scale contents
    self.header:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.sidebar_list:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:Layout()
    self:SelectIndex(1)
end

function HistoryScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end

    self:Layout()
end

function HistoryScreen:Layout()

    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    local sm = TheGame:FE():GetScreenMode()

	-- Layout sidebar contents
    self.balance:LayoutBounds( "left", "center", self.title )
    -- self.subtitle:LayoutBounds( "left", "below", self.title ):Offset( 0, 0 )
    self.sidebar_list:LayoutBounds( "left", "top", 0, 0 )
    self.details_panel:LayoutBounds( "left", "top", 0, 0 )
    self.details_scroll:RefreshView()

    -- Header height
    local w, h = self.header:GetSize()
    local header_height = ( 60 + h + SPACING.M1 )*LAYOUT_SCALE[sm]
    -- Footer height
    local button_shown = self.close_button:IsShown()
    self.close_button:Show() -- We have to show the button, so the footer size calculates properly
    local w, h = self.bottom_left:GetSize()
    self.close_button:SetShown( button_shown )
    local footer_height = ( 60 + h + SPACING.M1 )*LAYOUT_SCALE[sm]
    -- Content height
    local content_h = self.screen_h - header_height - footer_height

    -- Update containers
    self.widepanel:SetHeight( content_h )
        :LayoutBounds( nil, "below", self.header )
        :Offset( 0, -SPACING.M1 )
    self.sidebar_scroll:SetSize( SIDEBAR_W*LAYOUT_SCALE[sm], content_h )
        :LayoutBounds( "left", "below", self.header )
        :Offset( 0, -SPACING.M1 - 3 )
        :RefreshView()
    local sidebar_gap = 15
    local details_w = self.screen_w - SIDEBAR_W*LAYOUT_SCALE[sm] - 60*2 - sidebar_gap
    self.details_scroll:SetSize( details_w, content_h )
        :LayoutBounds( "after", "top", self.sidebar_scroll )
        :Offset( sidebar_gap, 0 )
        :RefreshView()
    self.details_panel:SetWidth( details_w-30 )

    self.sidebar_gradient:SetSize( self.screen_w*0.4, content_h+4 )
        :LayoutBounds( "left", "center", self.widepanel )
        :Offset( -8, -4 )
    self.details_gradient:SetSize( self.screen_w*0.7, content_h+4 )
        :SetScale( 1, 1 )
        :LayoutBounds( "right", "center", self.widepanel )
        :Offset( 8, -4 )
        :SetScale( -1, 1 )

	return self
end

function HistoryScreen:SelectIndex( index )
    if not self.sidebar_list:IsEmpty() and #self.sidebar_list.children >= index then
        self.sidebar_list.children[index]:Click()
    end
    return self
end

function HistoryScreen:OnClickClose()
    self:FadeOut()
end

-- A run was selected
-- Display this run's info on the screen.
function HistoryScreen:SelectRun( gamestate )
	-- Show details
    self.details_panel:Refresh(gamestate):Show()
	-- Select correct sidebar button, unselecting others
    for k, btn in ipairs( self.sidebar_list.children ) do
        btn:SetSelected( btn.gamestate == gamestate )
        if btn:IsSelected() then self.selected_button = btn end
    end
    -- Position everything properly
    self:Layout()
    -- Scroll to top
    self.details_scroll:SetScrollOffset( 0 )
    -- Reset last focused details widget
    self.last_details_focus = nil

    return self
end

-------------------------------------------------------------------------
-- A button to show a specific run's details

local HistoryRunButton = class("Widget.HistoryRunButton", Widget.ShortButton)

function HistoryRunButton:init( width )
    HistoryRunButton._base.init( self, width )

    self.sidebar_gradient = self.contents:AddChild( Widget.Image( engine.asset.Texture("UI/compendium_graft_gradient.tex") ) )
        :SendBehind( self.text_content )

    self.button_text:SetFont( "title" )
        :SetFontSize( FONT_SIZE.ITEM_SUBTITLE )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.WHITE )
    self.progress = self.text_content:AddChild( Widget.Label( "body", FONT_SIZE.SMALL_TEXT ) )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.WHITE )
        :Bloom( 0.1 )
    self.date = self.text_content:AddChild( Widget.Label( "tooltip", FONT_SIZE.SMALL_TEXT ) )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom( 0.1 )

    self.act_image_w = 138
    self.act_image_h = 115
    self.act_image = self.contents:AddChild( Widget.Image(engine.asset.Texture("UI/char_0_locked.tex")) )
        :SetHiddenBoundingBox( true )
        :SetSize( self.act_image_w, self.act_image_h )

    self.prestige_icon = self.contents:AddChild( Widget.Image() )
        :SetSize( 24, 24 )

    self:SetSize( width, 100 )
end

function HistoryRunButton:Refresh( gamestate )
    self.gamestate = gamestate
    local story = gamestate.options.story_mode
	local act = GetPlayerActData( gamestate.options.start_act )
    local act_data = act.data
    local player_background = act.background
    local prestige_level = gamestate.options.advancement_level

    if story then
    	-- Story difficulty!
        if act_data then
        	if act_data.story_image then self.act_image:SetTexture( act_data.story_image ) end
        	if act_data.act_image then self.act_image:SetTexture( act_data.act_image ) end
        	if act_data.story_colour_background then
        		self.sidebar_gradient:SetTintColour( act_data.story_colour_background )
        			:SetTintAlpha( 0.4 ) end
        	if act_data.story_colour_text then
	            self.date:SetGlyphColour( act_data.story_colour_text )
	            self.button_arrow:SetTintColour( act_data.story_colour_text ) end
        end
        self.button_text:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_STORY_TITLE", act:GetLocalizedName() ) )
        -- Set story icon
        self.prestige_icon:SetTexture( engine.asset.Texture("UI/ic_story_mode.tex" ) )
    else
    	-- Regular campaign
        if act_data then
        	if act_data.act_image then self.act_image:SetTexture( act_data.act_image ) end
        	if act_data.colour_frame then self.button_bg:SetTintColour( act_data.colour_frame ) end
        	if act_data.colour_background then
                self.sidebar_gradient:SetTintColour( act_data.colour_background )
                    :SetTintAlpha( 0.4 ) end
        	if act_data.colour_text then
	            self.date:SetGlyphColour( act_data.colour_text )
        		self.button_arrow:SetTintColour( act_data.colour_text ) end
        end
        self.button_text:SetText( act:GetLocalizedName() )
        -- Set prestige icon
        if player_background then
        	local prestige_data = player_background.data and player_background.data.advancement and player_background.data.advancement[prestige_level]
        	self.prestige_icon:SetTexture( prestige_data and prestige_data.icon )
        end
    end

    if act_data.game_type == GAME_TYPE.DAILY then
    	-- Daily run!
    	if gamestate.result == GAMEOVER.VICTORY or gamestate.result == GAMEOVER.ALPHA_VICTORY then
    		self.progress:SetText( loc.format( LOC"UI.HISTORY_SCREEN.PROGRESS_DAILY_SUCCESS", gamestate.score or 0 ) )
    			:SetTintColour( UICOLOURS.BONUS )
    	else
    		self.progress:SetText( loc.format( LOC"UI.HISTORY_SCREEN.PROGRESS_DAILY_DEATH", gamestate.score or 0 ) )
    			:SetTintColour( UICOLOURS.PENALTY )
    	end

    	-- Set difficulty icon
    	local difficulty_level = 1
	    for difficulty, difficulty_data in ipairs(DAILY_DIFFICULTY_OPTIONS) do
	        if difficulty_data.prestige == prestige_level then difficulty_level = difficulty end
	    end
	    self.prestige_icon:SetTexture( DAILY_DIFFICULTY_OPTIONS[difficulty_level].icon() )
    else
    	-- Regular runs
    	if gamestate.result == GAMEOVER.VICTORY or gamestate.result == GAMEOVER.ALPHA_VICTORY then
    		self.progress:SetText( LOC"UI.HISTORY_SCREEN.PROGRESS_SUCCESS" )
    			:SetTintColour( UICOLOURS.BONUS )
    	else
    		self.progress:SetText( loc.format( LOC"UI.HISTORY_SCREEN.PROGRESS_DEATH_DAY",  GameState.ConvertDatetimeToDay( gamestate.datetime ) ) )
    			:SetTintColour( UICOLOURS.PENALTY )
    	end
    end

    -- Savefile start date
    self.date:SetText( os.date( "%x %X", gamestate.creation_time, gamestate.creation_time ) )

    self:Layout()
    return self
end

function HistoryRunButton:Layout()
    if self.progress then
        local text_w = self.width - self.act_image_w - SPACING.M1*2
        self.button_text:SetAutoSize( text_w )
        self.progress:SetAutoSize( text_w )
        self.date:SetAutoSize( text_w )
        self.sidebar_gradient:SetSize( self.width*0.75, self.height-2 )

        self.sidebar_gradient:LayoutBounds( "left", "center", self.button_bg )
            :Offset( 3, 0 )
        self.act_image:LayoutBounds( "left", "bottom", self.button_bg )
            :Offset( 3, 1 )
        self.progress:LayoutBounds( "left", "below", self.button_text )
            :Offset( 0, 2 )
        self.date:LayoutBounds( "left", "below", self.progress )
        	:Offset( 0, 0 )
        self.text_content:LayoutBounds( "after", "center", self.act_image )
            :Offset( 0, -7 )
        self.prestige_icon:LayoutBounds( "right", "bottom", self.button_bg ):Offset( -12, 10 )
        self.button_arrow:LayoutBounds( "right", "center", self.button_bg ):Offset( -12, 0 )
    end
    return self
end

function HistoryRunButton:IsSelected()
    return self.selected
end

function HistoryRunButton:SetSelected( selected )
    self.selected = selected
    self:UpdateImage()
    return self
end

function HistoryRunButton:UpdateImage()
    if self.hover or self.focus or self.selected then
        if self.enabled and ( self.hover or self.focus ) then
            self.button_bg:Bloom( 0.4 )
            self.contents:ScaleTo( nil, 1.02, 0.1 )
        end
        self.contents:MoveTo( self.focus_x_delta, 0, 0.1 )
    else
        self.contents:ScaleTo( nil, 1, 0.2 )
        self.contents:MoveTo( 0, 0, 0.2 )
    end
    if not ( self.hover or self.focus ) then
        self.button_bg:Bloom( self.default_bloom )
    end
    return self
end

-------------------------------------------------------------------------
-- 

local HistoryDetailsPanel = class("Widget.HistoryDetailsPanel", Widget)

local divider_h = 2

function HistoryDetailsPanel:init()
    HistoryDetailsPanel._base.init( self )

    self.width = 300
    self.card_scale = 0.55
    self.card_scale_hover = 0.56
    self.graft_size = 110
    self.perk_size = 120
    self.mutator_size = 80

    -- Header section
    self.header = self:AddChild( Widget.Clickable() )
    self.run_type = self.header:AddChild( Widget.Label( "title", FONT_SIZE.BODY_TEXT, LOC"UI.HISTORY_SCREEN.SUBTITLE" ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetTintAlpha( 0.7 )
        :LeftAlign()
        :SetAutoSize( RES_X/2 )
        :SetWordWrap( true )
        :Bloom( 0.05 )
    self.run_date = self.header:AddChild( Widget.Label( "tooltip", FONT_SIZE.BODY_TEXT ) )
        :RightAlign()
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetTintAlpha( 0.7 )
        :Bloom( 0.1 )
        :SetToolTip( LOC"UI.HISTORY_SCREEN.RUN_DATE_TT" )
    self.playtime = self.header:AddChild( Widget.Label( "tooltip", FONT_SIZE.BODY_TEXT ) )
        :RightAlign()
        :SetGlyphColour( UICOLOURS.WHITE )
        :Bloom( 0.1 )
    self.run_title = self.header:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_TITLE, LOC"UI.HISTORY_SCREEN.TITLE" ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :LeftAlign()
        :SetAutoSize( RES_X/2 )
        :SetWordWrap( true )
        :Bloom( 0.1 )
        :LayoutBounds( "left", "below", self.title ):Offset( -1, 10 )
    self.header_divider = self.header:AddChild( Widget.Image( engine.asset.Texture("images/white.tex" ), 30, divider_h ) )
    	:SetTintColour( UICOLOURS.WHITE )
        :SetTintAlpha( 0.7 )
        :Bloom( 0.05 )
    self.header:SetHighlightWidget( self.run_type )

    -- Info section
    self.info_blocks = self.header:AddChild( Widget() )
    self.info_result = self.info_blocks:AddChild( Widget.HistoryDetailsBlock() )
    self.info_health = self.info_blocks:AddChild( Widget.HistoryDetailsBlock() )
    self.info_resolve = self.info_blocks:AddChild( Widget.HistoryDetailsBlock() )
    self.info_prestige = self.info_blocks:AddChild( Widget.HistoryDetailsBlock() )
    self.info_score = self.info_blocks:AddChild( Widget.HistoryDetailsBlock() )
    self.info_battles = self.info_blocks:AddChild( Widget.HistoryDetailsBlock() )
    self.info_negotiations = self.info_blocks:AddChild( Widget.HistoryDetailsBlock() )
    self.info_money = self.info_blocks:AddChild( Widget.HistoryDetailsBlock() )

    -- Feats section
    self.feats_section = self:AddChild( Widget() )
    self.feats_header = self.feats_section:AddChild( Widget.HistoryHeader() )
    	:SetTitle( LOC"UI.HISTORY_SCREEN.FEATS_TITLE" )
    self.feats_blocks = self.feats_section:AddChild( Widget() )

    -- Relationships section
    self.relationships_section = self:AddChild( Widget() )
    self.relationships_header = self.relationships_section:AddChild( Widget.HistoryHeader() )
    	:SetTitle( LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_TITLE" )
    self.relationships_blocks = self.relationships_section:AddChild( Widget() )
    self.relationships_loved = self.relationships_blocks:AddChild( Widget.HistoryDetailsBlock() )
        :SetTitleColour( RELATIONSHIP_COLOURS[RELATIONSHIP.LOVED] )
    self.relationships_liked = self.relationships_blocks:AddChild( Widget.HistoryDetailsBlock() )
        :SetTitleColour( RELATIONSHIP_COLOURS[RELATIONSHIP.LIKED] )
    self.relationships_neutral = self.relationships_blocks:AddChild( Widget.HistoryDetailsBlock() )
        :SetTitleColour( RELATIONSHIP_COLOURS[RELATIONSHIP.NEUTRAL] )
    self.relationships_disliked = self.relationships_blocks:AddChild( Widget.HistoryDetailsBlock() )
        :SetTitleColour( RELATIONSHIP_COLOURS[RELATIONSHIP.DISLIKED] )
    self.relationships_hated = self.relationships_blocks:AddChild( Widget.HistoryDetailsBlock() )
        :SetTitleColour( RELATIONSHIP_COLOURS[RELATIONSHIP.HATED] )
    self.relationships_dead = self.relationships_blocks:AddChild( Widget.HistoryDetailsBlock() )
        :SetTitleColour( UICOLOURS.DEAD )

    -- Battle deck section
    self.battledeck_section = self:AddChild( Widget() )
    self.battledeck_header = self.battledeck_section:AddChild( Widget.HistoryHeader() )
        :SetTitle( LOC"UI.HISTORY_SCREEN.BATTLEDECK_TITLE" )
    self.battledeck_container = self.battledeck_section:AddChild( Widget() )

    -- Negotiation deck section
    self.negotiationdeck_section = self:AddChild( Widget() )
    self.negotiationdeck_header = self.negotiationdeck_section:AddChild( Widget.HistoryHeader() )
        :SetTitle( LOC"UI.HISTORY_SCREEN.NEGOTIATIONDECK_TITLE" )
    self.negotiationdeck_container = self.negotiationdeck_section:AddChild( Widget() )

    -- Battle grafts section
    self.battlegrafts_section = self:AddChild( Widget() )
    self.battlegrafts_header = self.battlegrafts_section:AddChild( Widget.HistoryHeader() )
        :SetTitle( LOC"UI.HISTORY_SCREEN.BATTLEGRAFTS_TITLE" )
    self.battlegrafts_container = self.battlegrafts_section:AddChild( Widget() )

    -- Negotiation grafts section
    self.negotiationgrafts_section = self:AddChild( Widget() )
    self.negotiationgrafts_header = self.negotiationgrafts_section:AddChild( Widget.HistoryHeader() )
        :SetTitle( LOC"UI.HISTORY_SCREEN.NEGOTIATIONGRAFTS_TITLE" )
    self.negotiationgrafts_container = self.negotiationgrafts_section:AddChild( Widget() )

    -- Rook coin section
    self.rookcoin_section = self:AddChild( Widget() )
    self.rookcoin_header = self.rookcoin_section:AddChild( Widget.HistoryHeader() )
        :SetTitle( LOC"UI.HISTORY_SCREEN.ROOKCOIN_TITLE" )
    self.rookcoin_container = self.rookcoin_section:AddChild( Widget() )

    -- Perks section
    self.perks_section = self:AddChild( Widget() )
    self.perks_header = self.perks_section:AddChild( Widget.HistoryHeader() )
        :SetTitle( LOC"UI.HISTORY_SCREEN.PERKS_TITLE" )
    self.perks_container = self.perks_section:AddChild( Widget() )

    -- Mutators section
    self.mutators_section = self:AddChild( Widget() )
    self.mutators_header = self.mutators_section:AddChild( Widget.HistoryHeader() )
        :SetTitle( LOC"UI.HISTORY_SCREEN.MUTATORS_TITLE" )
    self.mutators_container = self.mutators_section:AddChild( Widget() )

end

function HistoryDetailsPanel:SetWidth( width )
	self.width = width or 300
	return self
end

function HistoryDetailsPanel:OnScreenModeChange( sm )
    HistoryDetailsPanel._base.OnScreenModeChange( self, sm )

    self.run_type:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:Layout()
end


function HistoryDetailsPanel:Refresh( gamestate )
	local story = gamestate.options.story_mode
	local act = GetPlayerActData( gamestate.options.start_act )
    local act_data = act.data
    local player_background = act.background
    local prestige_level = gamestate.options.advancement_level
    local agent = player_background and Agent( player_background.data.player_agent )
    	
    -- Header and info blocks
	local result_title, result_text, result_colour
	local prestige_title, prestige_text

	if gamestate.creation_time then self.run_date:SetText( os.date( "%x %X", gamestate.creation_time ) ) end
    if gamestate.play_time then self.playtime:SetText( FormatTime( gamestate.play_time )) end

    if act_data.game_type == GAME_TYPE.DAILY then
    	-- Daily run!
    	local date = gamestate.creation_time and (gamestate.creation_time - (gamestate.creation_time % (60*60*24)))
    	local quest = gamestate.quest_stats and copykeys(gamestate.quest_stats)[1]
    	quest = quest and Content.GetQuestDef( quest )
    	self.run_type:SetText( loc.format(LOC"UI.HISTORY_SCREEN.DETAILS_TYPE_DAILY",agent:GetLocalizedName(),date and os.date("%x",date) ) )
    	self.run_title:SetText( quest and quest:GetTitle() )

    	if prestige_level then
	    	local difficulty_level = 1
		    for difficulty, difficulty_data in ipairs(DAILY_DIFFICULTY_OPTIONS) do
		        if difficulty_data.prestige == prestige_level then difficulty_level = difficulty end
		    end
		    local image = DAILY_DIFFICULTY_OPTIONS[difficulty_level].icon():GetPath()
	    	prestige_title = loc.format(LOC"UI.HISTORY_SCREEN.INFO_DIFFICULTY_TITLE", image)
	    	prestige_text = loc.format(LOC"UI.HISTORY_SCREEN.INFO_DIFFICULTY_TEXT", DAILY_DIFFICULTY_OPTIONS[difficulty_level].title())
	    end
    else
    	if act_data.game_type == GAME_TYPE.BRAWL then
	    	-- Brawl run
	    	self.run_type:SetText( loc.format(LOC"UI.HISTORY_SCREEN.DETAILS_TYPE_BRAWL",agent:GetLocalizedName()) )
	    	self.run_title:SetText( act:GetLocalizedName() )
	    else
	    	-- Regular runs
	    	if story then
	    		self.run_type:SetText( loc.format(LOC"UI.HISTORY_SCREEN.DETAILS_TYPE_CAMPAIGN",agent:GetLocalizedName()) )
		    	self.run_title:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_STORY_TITLE", act:GetLocalizedName() ) )
			else
	    		self.run_type:SetText( loc.format(LOC"UI.HISTORY_SCREEN.DETAILS_TYPE_CAMPAIGN",agent:GetLocalizedName()) )
		        self.run_title:SetText( act:GetLocalizedName() )
		    end
	    end

        if story then
            prestige_title = LOC"UI.HISTORY_SCREEN.INFO_STORY_TITLE"
            prestige_text = LOC"UI.HISTORY_SCREEN.INFO_STORY_TEXT"
        elseif prestige_level then
	    	local prestige_data = player_background.data and player_background.data.advancement and player_background.data.advancement[prestige_level]
        	local image = prestige_data and prestige_data.icon and prestige_data.icon:GetPath()
	    	prestige_title = loc.format(LOC"UI.HISTORY_SCREEN.INFO_PRESTIGE_TITLE", image)
	    	prestige_text = loc.format(LOC"UI.HISTORY_SCREEN.INFO_PRESTIGE_TEXT", prestige_level)
	    end
    end

    if gamestate.result == GAMEOVER.VICTORY or gamestate.result == GAMEOVER.ALPHA_VICTORY then
    	result_title = LOC"UI.HISTORY_SCREEN.INFO_RESULT_SUCCESS_TITLE"
    	result_text = LOC"UI.HISTORY_SCREEN.INFO_RESULT_SUCCESS_TEXT"
    	result_colour = UICOLOURS.BONUS
    elseif gamestate.killed_by then
        local skin_table = Content.GetCharacterSkin( gamestate.killed_by.skin_id )
		result_title = loc.format( LOC"UI.HISTORY_SCREEN.INFO_RESULT_KILLEDBY_TITLE", Agent(gamestate.killed_by.content_id, skin_table):LocTable() )
    	result_text = LOC"UI.HISTORY_SCREEN.INFO_RESULT_KILLEDBY_TEXT"
    	result_colour = UICOLOURS.PENALTY
    else
		result_title = LOC"UI.HISTORY_SCREEN.INFO_RESULT_KILLED_TITLE"
    	result_text = LOC"UI.HISTORY_SCREEN.INFO_RESULT_KILLED_TEXT"
    	result_colour = UICOLOURS.PENALTY
    end

    self.info_result:Refresh( result_title, result_text )
    	:SetTintColour(result_colour)
	self.info_health:Refresh( LOC"UI.HISTORY_SCREEN.INFO_HEALTH_TITLE", loc.format( LOC"UI.HISTORY_SCREEN.INFO_HEALTH_TEXT", gamestate.health, gamestate.max_health ) )
	self.info_resolve:Refresh( LOC"UI.HISTORY_SCREEN.INFO_RESOLVE_TITLE", loc.format( LOC"UI.HISTORY_SCREEN.INFO_RESOLVE_TEXT", gamestate.resolve, gamestate.max_resolve ) )
	self.info_prestige:Refresh( prestige_title, prestige_text )
	self.info_score:Refresh( LOC"UI.HISTORY_SCREEN.INFO_SCORE_TITLE", loc.format( LOC"UI.HISTORY_SCREEN.INFO_SCORE_TEXT", gamestate.score or 0 ) )
		:SetShown( gamestate.score )
	self.info_battles:Refresh( LOC"UI.HISTORY_SCREEN.INFO_BATTLES_TITLE", loc.format( LOC"UI.HISTORY_SCREEN.INFO_BATTLES_TEXT", gamestate.battles_won or 0 ) )
		:SetShown( gamestate.battles_won )
	self.info_negotiations:Refresh( LOC"UI.HISTORY_SCREEN.INFO_NEGOTIATIONS_TITLE", loc.format( LOC"UI.HISTORY_SCREEN.INFO_NEGOTIATIONS_TEXT", gamestate.negotiations_won or 0 ) )
		:SetShown( gamestate.negotiations_won )
    self.info_money:Refresh( LOC"UI.HISTORY_SCREEN.INFO_MONEY_TITLE", loc.format( LOC"UI.HISTORY_SCREEN.INFO_MONEY_TEXT", gamestate.money or 0 ) )
        :SetShown( act_data.game_type ~= GAME_TYPE.DAILY )

	-- Feats section
	self.feats_blocks:DestroyAllChildren() -- Clear old feats
	self.feats_missing = false
	if act_data.game_type == GAME_TYPE.DAILY then
		for k, graft in ipairs( gamestate.grafts ) do
			if Content.GetGraft( graft.id ) then
	            local graft_instance = GraftInstance( graft.id )
	            if graft and graft_instance:GetType() == GRAFT_TYPE.FEAT then
			        self.feats_blocks:AddChild( Widget.HistoryFeatBlock() )
			        	:Refresh( graft, graft_instance )
			    end
	        end
	    end
	    if self.feats_blocks:IsEmpty() then
	    	-- This is an old run without this data
	    	self.feats_missing = true
	    	self.feats_blocks:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_DESC ) )
	    		:SetText( LOC"UI.HISTORY_SCREEN.FEATS_MISSING_DATA" )
		        :SetGlyphColour( UICOLOURS.WHITE )
		        :SetTintAlpha( 0.7 )
		        :LeftAlign()
		        :SetWordWrap( true )
		        :Bloom( 0.05 )
	    end
    end
	self.feats_section:SetShown( not self.feats_blocks:IsEmpty() )

	-- Relationships section
	if gamestate.social_connections then
		local counts = {
			[RELATIONSHIP.HATED] = 0,
	        [RELATIONSHIP.DISLIKED] = 0,
	        [RELATIONSHIP.NEUTRAL] = 0,
	        [RELATIONSHIP.LIKED] = 0,
	        [RELATIONSHIP.LOVED] = 0,
		}
		for k, connection in ipairs( gamestate.social_connections ) do
			counts[connection.relationship] = counts[connection.relationship] + 1
	    end
		self.relationships_loved:Refresh( LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_LOVED", loc.format(LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_COUNT",counts[RELATIONSHIP.LOVED] ) )
		self.relationships_liked:Refresh( LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_LIKED", loc.format(LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_COUNT",counts[RELATIONSHIP.LIKED] ) )
		self.relationships_neutral:Refresh( LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_NEUTRAL", loc.format(LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_COUNT",counts[RELATIONSHIP.NEUTRAL] ) )
		self.relationships_disliked:Refresh( LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_DISLIKED", loc.format(LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_COUNT",counts[RELATIONSHIP.DISLIKED] ) )
		self.relationships_hated:Refresh( LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_HATED", loc.format(LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_COUNT",counts[RELATIONSHIP.HATED] ) )
		if gamestate.dead_agents then
			self.relationships_dead:Show():Refresh( LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_DEAD", loc.format(LOC"UI.HISTORY_SCREEN.RELATIONSHIPS_COUNT",#gamestate.dead_agents ) )
		else
			self.relationships_dead:Hide()
		end
    end

    -- Battle deck section
    if gamestate.battle_deck and #gamestate.battle_deck > 0 then
        self.battledeck_header:SetTitle( loc.format( LOC"UI.HISTORY_SCREEN.BATTLEDECK_TITLE", #gamestate.battle_deck ) )
        self.battledeck_container:DestroyAllChildren() -- Clear old cards
        -- Assemble card data from history record
        local cards = {}
        for k, card_data in ipairs( gamestate.battle_deck ) do
            if Content.GetBattleCard( card_data.id ) then
                local card = Battle.Card( card_data.id )
                local widget = self.battledeck_container:AddChild( Widget.BattleCard() )
                    :RefreshCard( card )
                    :SetScale( self.card_scale )
                    :SetToolTipClass( nil )
                    :SetToolTip( nil )
                    :ShowInfoOnClick()
                    :SetOutline( UICOLOURS.FIGHT, false )
                -- Handle events
                widget.OnGainFocus = function() widget._base.OnGainFocus(widget) self.focused_card_widget = widget self:UpdateCardImageState( widget ) end
                widget.OnLoseFocus = function() widget._base.OnLoseFocus(widget) self.focused_card_widget = nil self:UpdateCardImageState( widget ) end
                widget.OnGainHover = function() widget._base.OnGainHover(widget) self:UpdateCardImageState( widget ) end
                widget.OnLoseHover = function() widget._base.OnLoseHover(widget) self:UpdateCardImageState( widget ) end
            end
        end
    end

    -- Negotiation deck section
    if gamestate.negotiation_deck and #gamestate.negotiation_deck > 0 then
        self.negotiationdeck_header:SetTitle( loc.format( LOC"UI.HISTORY_SCREEN.NEGOTIATIONDECK_TITLE", #gamestate.negotiation_deck ) )
        self.negotiationdeck_container:DestroyAllChildren() -- Clear old cards
        -- Assemble card data from history record
        for k, card_data in ipairs( gamestate.negotiation_deck ) do
            if Content.GetNegotiationCard( card_data.id ) then
                local card = Negotiation.Card( card_data.id )
                local widget = self.negotiationdeck_container:AddChild( Widget.NegotiationCard() )
                    :RefreshCard( card )
                    :SetScale( self.card_scale )
                    :SetToolTipClass( nil )
                    :SetToolTip( nil )
                    :ShowInfoOnClick()
                    :SetOutline( UICOLOURS.NEGOTIATION, false )
                -- Handle events
                widget.OnGainFocus = function() widget._base.OnGainFocus(widget) self.focused_card_widget = widget self:UpdateCardImageState( widget ) end
                widget.OnLoseFocus = function() widget._base.OnLoseFocus(widget) self.focused_card_widget = nil self:UpdateCardImageState( widget ) end
                widget.OnGainHover = function() widget._base.OnGainHover(widget) self:UpdateCardImageState( widget ) end
                widget.OnLoseHover = function() widget._base.OnLoseHover(widget) self:UpdateCardImageState( widget ) end
            end
        end
    end

    -- Battle grafts section
    self.battlegrafts_container:DestroyAllChildren() -- Clear old grafts
    for k, graft in ipairs( gamestate.grafts ) do
        if Content.GetGraft( graft.id ) then
            local graft = GraftInstance( graft.id )
            if graft and graft:GetType() == GRAFT_TYPE.COMBAT then
                local widget = self.battlegrafts_container:AddChild( Widget.GraftSlot( GRAFT_TYPE.COMBAT, self.graft_size ) )
                    :Refresh( graft )
                    :ShowToolTipOnFocus( true )
                widget:SetHighlightWidget( widget.bg )
            end
        end
    end
    self.battlegrafts_header:SetTitle( loc.format( LOC"UI.HISTORY_SCREEN.BATTLEGRAFTS_TITLE", not self.battlegrafts_container:IsEmpty() and #self.battlegrafts_container.children or 0 ) )

    -- Negotiation grafts section
    self.negotiationgrafts_container:DestroyAllChildren() -- Clear old grafts
    for k, graft in ipairs( gamestate.grafts ) do
        if Content.GetGraft( graft.id ) then
            local graft = GraftInstance( graft.id )
            if graft and graft:GetType() == GRAFT_TYPE.NEGOTIATION then
                local widget = self.negotiationgrafts_container:AddChild( Widget.GraftSlot( GRAFT_TYPE.NEGOTIATION, self.graft_size ) )
                    :Refresh( graft )
                    :ShowToolTipOnFocus( true )
                widget:SetHighlightWidget( widget.bg )
            end
        end
    end
    self.negotiationgrafts_header:SetTitle( loc.format( LOC"UI.HISTORY_SCREEN.NEGOTIATIONGRAFTS_TITLE", not self.negotiationgrafts_container:IsEmpty() and #self.negotiationgrafts_container.children or 0 ) )

    -- Rook's coin section
    self.rookcoin_container:DestroyAllChildren() -- Clear old grafts
    for k, graft in ipairs( gamestate.grafts ) do
        if Content.GetGraft( graft.id ) then
            local graft = GraftInstance( graft.id )
            if graft and graft:GetType() == GRAFT_TYPE.COIN then
                local widget = self.rookcoin_container:AddChild( Widget.GraftSlot( GRAFT_TYPE.COIN, self.graft_size ) )
                    :Refresh( graft )
                    :ShowToolTipOnFocus( true )
                widget:SetHighlightWidget( widget.bg )
            end
        end
    end
    self.rookcoin_header:SetTitle( LOC"UI.HISTORY_SCREEN.ROOKCOIN_TITLE" )

    -- Perks section
    self.perks_container:DestroyAllChildren() -- Clear old grafts
    for k, graft in ipairs( gamestate.grafts ) do
        if Content.GetGraft( graft.id ) then
            local graft = GraftInstance( graft.id )
            if graft and graft:GetType() == GRAFT_TYPE.PERK then
                local widget = self.perks_container:AddChild( Widget.GraftSlot( GRAFT_TYPE.PERK, self.perk_size ) )
                    :Refresh( graft )
                    :ShowToolTipOnFocus( true )
                widget:SetHighlightWidget( widget.bg )
            end
        end
    end
    self.perks_header:SetTitle( loc.format( LOC"UI.HISTORY_SCREEN.PERKS_TITLE", not self.perks_container:IsEmpty() and #self.perks_container.children or 0 ) )

    -- Mutators section
    self.mutators_container:DestroyAllChildren() -- Clear old grafts
    if gamestate.options and gamestate.options.mutators then
        for k, graft_id in ipairs( gamestate.options.mutators ) do
            if Content.GetGraft( graft_id ) then
                local graft = GraftInstance( graft_id )
                if graft and graft:GetType() == GRAFT_TYPE.MUTATOR and not graft:IsHidden() then
                    local widget = self.mutators_container:AddChild( Widget.GraftSlot( GRAFT_TYPE.MUTATOR, self.mutator_size ) )
                        :Refresh( graft )
                        :ShowToolTipOnFocus( true )
                    widget:SetHighlightWidget( widget.img )
                end
            end
        end
    end
    self.mutators_header:SetTitle( loc.format( LOC"UI.HISTORY_SCREEN.MUTATORS_TITLE", not self.mutators_container:IsEmpty() and #self.mutators_container.children or 0 ) )

    self:Layout()

	return self
end

function HistoryDetailsPanel:UpdateCardImageState( widget )
    if widget.hover or widget.focus then
        if is_instance( widget, Widget.BattleCard ) then
        elseif is_instance( widget, Widget.NegotiationCard ) then
        end

        widget:SetOutline( UICOLOURS.FOCUS, true )
        widget:ScaleTo( nil, self.card_scale_hover, 0.2, easing.outQuad )
    else
        if is_instance( widget, Widget.BattleCard ) then
            widget:SetOutline( UICOLOURS.FIGHT, true )
        elseif is_instance( widget, Widget.NegotiationCard ) then
            widget:SetOutline( UICOLOURS.NEGOTIATION, true )
        end

        widget:ScaleTo( nil, self.card_scale, 0.2, easing.outQuad )
    end
end

function HistoryDetailsPanel:Layout()

	self.run_type:LayoutBounds( "left", "below", 0, 0 )
    self.run_title:LayoutBounds( "left", "below", self.run_type ):Offset( -1, 10 )
    self.header_divider:SetSize( self.width, divider_h )
    	:LayoutBounds( "left", "below", self.run_title ):Offset( 0, -6 )
    self.run_date:LayoutBounds( "right", nil, self.header_divider )
    	:LayoutBounds( nil, "center", self.run_type )
    self.playtime:LayoutBounds( "right", "below", self.run_date )
    local content_margin = 20
    local content_width = self.width - content_margin*3
    local columns = 4
    local spacing = 20
    local column_width = (content_width-(spacing*(columns-1)))/columns
    for k, w in ipairs( self.info_blocks.children ) do
    	w:SetWidth( column_width )
    end
   	self.info_blocks:LayoutChildrenInGrid( columns, spacing )
   		:LayoutBounds( "left", "below", self.header_divider )
   		:Offset( content_margin, -spacing )

	-- Feats section
	self.feats_header:SetWidth( self.width )
	if self.feats_missing then
		self.feats_blocks.children[1]:SetAutoSize( content_width )
			:LayoutBounds( "left", "below", self.feats_header )
	   		:Offset( content_margin, -spacing )
	else
		columns = 3
	    column_width = (content_width-(spacing*(columns-1)))/columns
	    for k, w in ipairs( self.feats_blocks.children ) do
	    	w:SetWidth( column_width )
	    end
	   	self.feats_blocks:LayoutChildrenInGrid( columns, spacing )
	   		:LayoutBounds( "left", "below", self.feats_header )
	   		:Offset( content_margin, -spacing )
   	end

   	-- Relationships section
   	self.relationships_header:SetWidth( self.width )
	columns = self.relationships_dead:IsShown() and 6 or 5
	column_width = (content_width-(spacing*(columns-1)))/columns
	for k, w in ipairs( self.relationships_blocks.children ) do
    	w:SetWidth( column_width )
    end
    self.relationships_blocks:LayoutChildrenInGrid( columns, spacing )
   		:LayoutBounds( "left", "below", self.relationships_header )
   		:Offset( content_margin, -spacing )

    -- Battle deck section
    if not self.battledeck_container:IsEmpty() then
        self.battledeck_header:SetWidth( self.width )
        local card_w = self.battledeck_container.children[1]:GetScaledSize()
        local card_spacing = spacing/2
        local cols = content_width/(card_w+card_spacing)
        self.battledeck_container:LayoutChildrenInGrid( math.floor(cols), card_spacing )
            :LayoutBounds( "left", "below", self.battledeck_header )
            :Offset( content_margin, -spacing )
        self.battledeck_section:Show()
    else
        self.battledeck_section:Hide()
    end

    -- Negotiation deck section
    if not self.negotiationdeck_container:IsEmpty() then
        self.negotiationdeck_header:SetWidth( self.width )
        local card_w = self.negotiationdeck_container.children[1]:GetScaledSize()
        local card_spacing = spacing/2
        local cols = content_width/(card_w+card_spacing)
        self.negotiationdeck_container:LayoutChildrenInGrid( math.floor(cols), card_spacing )
            :LayoutBounds( "left", "below", self.negotiationdeck_header )
            :Offset( content_margin, -spacing )
        self.negotiationdeck_section:Show()
    else
        self.negotiationdeck_section:Hide()
    end

    -- Battle grafts section
    if not self.battlegrafts_container:IsEmpty() then
        self.battlegrafts_header:SetWidth( self.width )
        local graft_w = self.battlegrafts_container.children[1]:GetSize()
        local graft_spacing = spacing/2
        local cols = content_width/(graft_w+graft_spacing)
        self.battlegrafts_container:LayoutChildrenInGrid( math.floor(cols), graft_spacing )
            :LayoutBounds( "left", "below", self.battlegrafts_header )
            :Offset( content_margin, -spacing )
        self.battlegrafts_section:Show()
    else
        self.battlegrafts_section:Hide()
    end

    -- Negotiation grafts section
    if not self.negotiationgrafts_container:IsEmpty() then
        self.negotiationgrafts_header:SetWidth( self.width )
        local graft_w = self.negotiationgrafts_container.children[1]:GetSize()
        local graft_spacing = spacing/2
        local cols = content_width/(graft_w+graft_spacing)
        self.negotiationgrafts_container:LayoutChildrenInGrid( math.floor(cols), graft_spacing )
            :LayoutBounds( "left", "below", self.negotiationgrafts_header )
            :Offset( content_margin, -spacing )
        self.negotiationgrafts_section:Show()
    else
        self.negotiationgrafts_section:Hide()
    end

    -- Rook's coin section
    if not self.rookcoin_container:IsEmpty() then
        self.rookcoin_header:SetWidth( self.width )
        local graft_w = self.rookcoin_container.children[1]:GetSize()
        local graft_spacing = spacing/2
        local cols = content_width/(graft_w+graft_spacing)
        self.rookcoin_container:LayoutChildrenInGrid( math.floor(cols), graft_spacing )
            :LayoutBounds( "left", "below", self.rookcoin_header )
            :Offset( content_margin, -spacing )
        self.rookcoin_section:Show()
    else
        self.rookcoin_section:Hide()
    end

    -- Perks section
    if not self.perks_container:IsEmpty() then
        self.perks_header:SetWidth( self.width )
        local graft_w = self.perks_container.children[1]:GetSize()
        local graft_spacing = spacing
        local cols = content_width/(graft_w+graft_spacing)
        self.perks_container:LayoutChildrenInGrid( math.floor(cols), graft_spacing )
            :LayoutBounds( "left", "below", self.perks_header )
            :Offset( content_margin*2, -spacing )
        self.perks_section:Show()
    else
        self.perks_section:Hide()
    end

    -- Mutators section
    if not self.mutators_container:IsEmpty() then
        self.mutators_header:SetWidth( self.width )
        local graft_w = self.mutators_container.children[1]:GetSize()
        local graft_spacing = spacing
        local cols = content_width/(graft_w+graft_spacing)
        self.mutators_container:LayoutChildrenInGrid( math.floor(cols), graft_spacing )
            :LayoutBounds( "left", "below", self.mutators_header )
            :Offset( content_margin*1.5, -spacing )
        self.mutators_section:Show()
    else
        self.mutators_section:Hide()
    end

   	-- Layout sections
   	self:StackChildren( spacing*2 )

	return self
end

function HistoryDetailsPanel:GetDefaultFocus()
    return self.header
end

-------------------------------------------------------------------------
-- 

local HistoryHeader = class("Widget.HistoryHeader", Widget)

function HistoryHeader:init()
    HistoryHeader._base.init( self )

    self.width = 200
    self.divider_h = 2

	self.divider = self:AddChild( Widget.Image( engine.asset.Texture("images/white.tex" ), self.width, self.divider_h ) )
    	:SetTintColour( UICOLOURS.WHITE )
        :SetTintAlpha( 0.7 )
        :Bloom( 0.05 )
    self.title = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetTintAlpha( 0.7 )
        :LeftAlign()
        :Bloom( 0.05 )
end

function HistoryHeader:SetWidth( width )
	self.width = width or 300
	self.divider:SetSize( self.width, self.divider_h )
	self:Layout()
	return self
end

function HistoryHeader:SetTitle( title )
	self.title:SetText( title )
	self:Layout()
	return self
end

function HistoryHeader:Layout()
	self.title:LayoutBounds( "left", "above", self.divider )
		:Offset( 0, 4 )
	return self
end

-------------------------------------------------------------------------
-- 

local HistoryDetailsBlock = class("Widget.HistoryDetailsBlock", Widget)

function HistoryDetailsBlock:init()
    HistoryDetailsBlock._base.init( self )

    self.width = 200

	self.hitbox = self:AddChild( Widget.Image( engine.asset.Texture("images/white.tex" ), self.width, 30 ) )
    	:SetTintColour( 0xff00ff00 )
    self.title = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetTintAlpha( 0.7 )
        :LeftAlign()
        :SetAutoSize( self.width )
        :SetWordWrap( true )
        :Bloom( 0.05 )
    self.value = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_NAME ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :LeftAlign()
        :SetAutoSize( self.width )
        :SetWordWrap( true )
        :Bloom( 0.1 )
end

function HistoryDetailsBlock:SetTitleColour( colour )
    self.title:SetTintColour( colour )
    return self
end

function HistoryDetailsBlock:SetWidth( width )
	self.width = width or 300
	self.hitbox:SetSize( self.width, 30 )
	self.title:SetAutoSize( self.width )
	self.value:SetAutoSize( self.width )
	self:Layout()
	return self
end

function HistoryDetailsBlock:Refresh( title, value )
	self.title:SetText( title )
	self.value:SetText( value )
	self:Layout()
	return self
end

function HistoryDetailsBlock:Layout()
	self.title:LayoutBounds( "left", nil, self.hitbox )
	self.value:LayoutBounds( "left", "below", self.title )
		:Offset( 12, 8 )
	return self
end

-------------------------------------------------------------------------
-- 

local HistoryFeatBlock = class("Widget.HistoryFeatBlock", Widget)

function HistoryFeatBlock:init()
    HistoryFeatBlock._base.init( self )

    self.width = 200
    self.icon_size = 50
    self.icon_spacing = 4
    self.content_width = self.width - self.icon_spacing - self.icon_size

	self.hitbox = self:AddChild( Widget.Image( engine.asset.Texture("images/white.tex" ), self.width, 30 ) )
    	:SetTintColour( 0xff00ff00 )
    self.icon = self:AddChild( Widget.Image( engine.asset.Texture("images/white.tex" ), self.icon_size, self.icon_size ) )
        :Bloom( 0.08 )
    self.title = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_NAME ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :LeftAlign()
        :SetAutoSize( self.content_width )
        :SetWordWrap( true )
        :Bloom( 0.05 )
    self.result = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :LeftAlign()
        :SetAutoSize( self.content_width )
        :SetWordWrap( true )
        :Bloom( 0.1 )
    self.desc = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_DESC ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetTintAlpha( 0.7 )
        :LeftAlign()
        :SetAutoSize( self.content_width )
        :SetWordWrap( true )
        :Bloom( 0.05 )
end

function HistoryFeatBlock:SetWidth( width )
	self.width = width or 300
	self.hitbox:SetSize( self.width, 30 )
    self.content_width = self.width - self.icon_spacing - self.icon_size
	self.title:SetAutoSize( self.content_width )
	self.result:SetAutoSize( self.content_width )
	self.desc:SetAutoSize( self.content_width )
	self:Layout()
	return self
end

function HistoryFeatBlock:Refresh( feat_data, graft_inst )
	local feat_def = graft_inst:GetDef()
	self.victory = feat_data.userdata.status == FEAT_STATUS.COMPLETE
	self.icon:SetTexture( engine.asset.Texture( feat_def.feat_icon ) )
	self.title:SetText( graft_inst:GetName() )
	self.result:SetText( self.victory and LOC"UI.HISTORY_SCREEN.FEAT_COMPLETE" or LOC"UI.HISTORY_SCREEN.FEAT_MISSED" )
	self.desc:SetText( graft_inst:GetDesc() )

	self:Layout()
	return self
end

function HistoryFeatBlock:Layout()
	self.icon:LayoutBounds( "left", "top", self.hitbox )
	self.title:LayoutBounds( "after", "top", self.icon )
		:Offset( 4, 0 )
	self.result:LayoutBounds( "left", "below", self.title )
		:Offset( 0, 4 )
	self.desc:LayoutBounds( "left", "below", self.result )
		:Offset( 0, -4 )
	return self
end