require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/solidbox"
require "ui/widgets/panelbutton"
require "ui/widgets/iconbutton"
require "ui/widgets/panel"
require "ui/widgets/slider"
require "ui/widgets/checkbox"

-------------------------------------------------------------------------
--

local InfoPopup = class("Screen.InfoPopup", Widget.Screen)

local DEFAULT_TEXT_W = RES_X*.36

InfoPopup.CONTROL_MAP = {
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function()
            return TheGame:FE():GetControlMode() == CONTROL_MODE.GAMEPAD
        end,
        fn = function(self)
            self:GetFE():PopScreen(self)
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format( LOC"UI.DIALOGS.CUSTOM", Controls.Digital.MENU_ACCEPT, self.button_accept_text ))
        end
    },
    {
        control = Controls.Digital.MENU_CANCEL,

        fn = function(self)
            self:GetFE():PopScreen(self)
            return true
        end,
    },
}

function InfoPopup:init( title, txt, buttontxt, fn )
    Widget.Screen.init(self)

    self.is_popup = true
    self.on_close = fn

    -- Get screen dimensions
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    
    self.scrim = self:AddChild( Widget.SolidBox( RES_X, RES_Y, 0x021A18F0 ) )--:BlockMouse()

    -- A container for all the background elements
    self.panel = self:AddChild( Widget.WidePanel(400) )
        :SetTitles( title, txt )

    self.buttons = self:AddChild( Widget() )
    self.button_accept_text = buttontxt or LOC"UI.DIALOGS.OK"
    self.button_ok = self.buttons:AddChild( Widget.IconButton() )
        :SetName("BUTTON")
        :SetText(self.button_accept_text)
        :SetFn(function() self:GetFE():PopScreen(self) end)
        :SetIcon( global_images.accept )

    -- Make everything transparent
    self.scrim:SetTintAlpha(0)
    self.buttons:SetTintAlpha(0)

    self:SetAnchors( "center", "center" )
    self:SetSounds( SoundEvents.tut_popup, SoundEvents.tut_popout )

    self:Layout()
end

function InfoPopup:GetTextWidget()
    return self.panel:GetSubtitleWidget()
end

function InfoPopup:SetOnAccept( fn )
    self.button_ok:SetFn( fn )
end

function InfoPopup:SetOnClose( fn )
    self.on_close = fn
end

function InfoPopup:OnClose()
    if self.on_close then
        self.on_close( self )
    end
end

function InfoPopup:Layout()
    -- Get screen dimensions
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    self.scrim:SetSize( self.screen_w, self.screen_h )

    -- Size up the background
    local _, h = self.panel:GetTitlesWidget():GetScaledSize()
    h = h + SPACING.M05*2.5
    self.panel:SetHeight( h )

    -- Layout the buttons
    self.buttons:StackChildrenRow( SPACING.M1*0.5 )
        :LayoutBounds( "center", "below", self.panel )
        :Offset( 0, -SPACING.M1 )
end


function InfoPopup:AddButton(txt, img, fn)
    local button = self.buttons:AddChild( Widget.IconButton() )
        :SetText(txt)
        :SetFn(fn)
        :SetIcon( img )
    self:Layout()
    return self
end

function InfoPopup:OnScreenResize(w, h)
    InfoPopup._base.OnScreenResize(self, w, h)
    self:Layout()
end

function InfoPopup:OnGainTop()
    InfoPopup._base.OnGainTop( self )
    self:AnimateIn()
end


function InfoPopup:OnControlModeChange( cm, device_id )
    InfoPopup._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.buttons:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.buttons:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.buttons:Hide()
    end
end

function InfoPopup:OnScreenModeChange( sm )
    InfoPopup._base.OnScreenModeChange( self, sm )

    -- Size up the background
    local _, h = self.panel:GetTitlesWidget():GetScaledSize()
    h = h + SPACING.M05*2.5
    self.panel:SetHeight( h )

    self.buttons:SetLayoutScale( LAYOUT_SCALE[sm] )
        :LayoutBounds( "center", "below", self.panel )
        :Offset( 0, -SPACING.M1 )

    self:Layout()
end

function InfoPopup:AnimateIn()
    self.panel:FadeIn( function()
        self.scrim:AlphaTo( 1, 0.3, easing.outQuad, function()
            if TheGame:FE():GetControlMode() == CONTROL_MODE.GAMEPAD then
                self.buttons:SetTintAlpha(1):Hide()
            else
                self.buttons:AlphaTo( 1, 0.1, easing.outQuad )
            end
        end )
    end )
end


-----------------------------------------------------------------------------------------------
--

local BetaMessagePopup = class( "Screen.BetaMessagePopup", Screen.InfoPopup )

function BetaMessagePopup:init( title, body, button, fn )
    BetaMessagePopup._base.init( self, nil, nil, button, fn )
    self.panel:SetTitles( title, nil, body )
    self:GetTextWidget():CenterH():Spool()
    
    self.smith_image_container = self:AddChild( Widget() )
    local img_scale = 0.41
    local img_w = 1000
    local img_h = 700
    self.smith_image_back = self.smith_image_container:AddChild( Widget.Image( engine.asset.Texture("battle/decks/smith_beta_back.tex") ) )
    self.smith_image_back:SetSize( img_w*img_scale, img_h*img_scale )
    self.smith_image_back:Bloom(0.2)
    self.smith_image = self.smith_image_container:AddChild( Widget.Image( engine.asset.Texture("battle/decks/smith_beta.tex") ) )
    self.smith_image:SetSize( img_w*img_scale, img_h*img_scale )
    self.smith_image:Bloom(0.03)
end

function BetaMessagePopup:OnScreenModeChange( sm )
    BetaMessagePopup._base.OnScreenModeChange( self, sm )

    self.smith_image_container:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:Layout()
end

function BetaMessagePopup:Layout()
    BetaMessagePopup._base.Layout( self )

    if self.smith_image_container then
        self.smith_image_container:LayoutBounds( "center", "bottom", self.panel:GetBottomEdgeWidget() )
            :LayoutBounds( "before", nil, self.panel:GetTitlesWidget() )
            :Offset( 40, 2 )
    end
end

function BetaMessagePopup:AnimateIn()
    BetaMessagePopup._base.AnimateIn( self )

    local anim_delay = 0.3
    local anim_duration = 0.3
    local vertical_change = -36
    local smith_x, smith_y = self.smith_image_container:GetPos()

    self.smith_image_container:SetTintAlpha(0)

    self.smith_image_container:RunUpdater(
        Updater.Series{
            Updater.Wait( anim_delay ),
            Updater.Parallel{
                Updater.Ease( function(v) self.smith_image_container:SetTintAlpha(v) end, 0, 1, anim_duration, easing.linear ),
                Updater.Ease( function(v) self.smith_image_container:SetPos(smith_x,v) end, smith_y+vertical_change, smith_y, anim_duration, easing.linear ),
            }
        }
    )
end

-----------------------------------------------------------------------------------------------
--

local TutorialPopup = class( "Screen.TutorialPopup", Screen.InfoPopup )

function TutorialPopup:init( seen_key, title, txt )
    TutorialPopup._base.init( self, title, txt )
    self.seen_key = seen_key

    self.checkbox = Widget.CheckBox():SetText( "Don't show again" )
    self:AddChild( self.checkbox ):LayoutBounds( "left", "below", self:FindChild( "TEXTBG" ))
end

function TutorialPopup:OnClose()
    if self.checkbox:GetValue() then
       TheGame:GetGameState():SetHasSeen( self.seen_key )
   end
    TutorialPopup._base.OnClose( self )
end

-----------------------------------------------------------------------------------------------
--

local YesNoPopup = class( "Screen.YesNoPopup", Screen.InfoPopup )

YesNoPopup.CANCEL = 0
YesNoPopup.YES = 1
YesNoPopup.NO = 2

YesNoPopup.CONTROL_MAP = {
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function()
            return TheGame:FE():GetControlMode() == CONTROL_MODE.GAMEPAD
        end,
        fn = function(self)
            self:DoRet(YesNoPopup.YES)
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.CUSTOM", Controls.Digital.MENU_ACCEPT, self.button_yes_text or LOC"UI.CONTROLS.ACCEPT"))
        end,
    },
    {
        control = Controls.Digital.MENU_CANCEL,

        fn = function(self)
            self:DoRet(YesNoPopup.CANCEL)
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.CUSTOM",Controls.Digital.MENU_CANCEL, self.button_cancel_text or LOC"UI.CONTROLS.CANCEL"))
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_X,

        test = function(self)
            return self.nobtn
        end,

        fn = function(self)
            self:DoRet(YesNoPopup.NO)
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.CUSTOM", Controls.Digital.GAMEPAD_X, self.button_negative_text))
        end,
    },    
}

function YesNoPopup:init( title, txt, no_button_txt, timeout, cancel_txt )
    self._base.init( self, title, txt, LOC "UI.DIALOGS.YES" )

    self.timeout = timeout

    -- Update OK button's function
    self.button_yes_text = LOC"UI.DIALOGS.ACCEPT"
    self.button_ok:SetFn(function() self:DoRet(YesNoPopup.YES) end)

    -- Add no-button
    if no_button_txt then
        self.button_negative_text = type(no_button_txt) == "string" and no_button_txt or LOC"UI.DIALOGS.NO"
        self.nobtn = self.buttons:AddChild( Widget.IconButton() )
            :SetName("NO_BUTTON")
            :SetText( self.button_negative_text )
            :SetFn(function() self:DoRet(YesNoPopup.NO) end)
            :SetIcon( global_images.cancel )
            :IncreaseSizeToLabel( SPACING.M1 )
            :LayoutBounds("after","center")
            :Offset(15,0)
    end

    -- Add cancel-button
    self.button_cancel_text = cancel_txt or LOC"UI.DIALOGS.CANCEL"
    self.cancelbutton = self.buttons:AddChild( Widget.IconButton() )
        :SetName("CANCEL_BUTTON")
        :SetText(cancel_txt or LOC"UI.DIALOGS.CANCEL")
        :SetFn(function() self:DoRet(YesNoPopup.CANCEL) end)
        :SetIcon( global_images.cancel )
        :LayoutBounds("after","center")
        :Offset(15,0)

    self:Layout()
end

function YesNoPopup:SetNoText(txt, tt)
    self.button_negative_text = txt
    if self.nobtn then
        self.nobtn:SetText(txt):IncreaseSizeToLabel( SPACING.M1 ):SetToolTip(tt)
        self:Layout()
    end
    return self
end

function YesNoPopup:SetYesText(txt, tt)
    self.button_yes_text = txt
    if self.button_ok then
        self.button_ok:SetText(txt):SetToolTip(tt)
        self:Layout()
    end
    return self
end

function YesNoPopup:SetCancelText(txt, tt)
    self.button_cancel_text = txt
    if self.cancelbutton then
        self.cancelbutton:SetText(txt):SetToolTip(tt)
        self:Layout()
    end
    return self
end


function YesNoPopup:HarvestRobotOptions()
    self:AddRobotOption(self.button_ok)
end

function YesNoPopup:DoRet(ret)
    self:GetFE():PopScreen(self)
    if self.close_fn then call_delegate( self.close_fn, ret ) end
    return self
end

function YesNoPopup:OnUpdate(dt)
    YesNoPopup._base.OnUpdate( self, dt )

    if self.timeout then
        self.timeout = self.timeout - dt
        if self.timeout <= 0 then
            self:DoRet(YesNoPopup.CANCEL)
        else
            self.button_cancel_text = loc.format(LOC"UI.DIALOGS.CANCEL_TIMEOUT", math.ceil(self.timeout))
            self.cancelbutton:SetText( self.button_cancel_text )
        end
    end
    return self
end

function YesNoPopup:SetFn( fn, ... )
    self.close_fn = { fn, ... }
    return self
end


-----------------------------------------------------------------------------------------------
--

local GameModeUnlockedMessagePopup = class( "Screen.GameModeUnlockedMessagePopup", Screen.InfoPopup )

function GameModeUnlockedMessagePopup:init( unlock_id, agent_id, title, body, fn )

    TheGame:GetGameProfile():AcquireUnlock(unlock_id)
    
    local button = LOC"UI.DIALOGS.CONTINUE"

    GameModeUnlockedMessagePopup._base.init( self, title, body, button, fn )

    self:GetTextWidget():CenterH():Spool()
    
    self.agent_anim = self:AddChild( Widget.AnimatedCharacter() )
        :SetOutline( 4, UICOLOURS.SUBTITLE )
        :Refresh( Agent( agent_id ) )
        :Emote( "greeting", true )
        :Hide()

    

end

function GameModeUnlockedMessagePopup:OnScreenModeChange( sm )
    GameModeUnlockedMessagePopup._base.OnScreenModeChange( self, sm )
    self:Layout()
end

function GameModeUnlockedMessagePopup:Layout()
    GameModeUnlockedMessagePopup._base.Layout( self )

    if self.agent_anim then
        self.agent_anim:LayoutBounds( "before", nil, self.panel:GetTitlesWidget() )
            :SetPos( nil, -310 )
    end
end

function GameModeUnlockedMessagePopup:AnimateIn()
    GameModeUnlockedMessagePopup._base.AnimateIn( self )

    local anim_delay = 0.3
    local anim_duration = 0.4
    local horizontal_change = -100
    local anim_x, anim_y = self.agent_anim:GetPos()

    self.agent_anim:RunUpdater(
        Updater.Series{
            Updater.Wait( anim_delay ),
            Updater.Do( function() self.agent_anim:Show() end ),
            Updater.Ease( function(v) self.agent_anim:SetPos(v,anim_y) end, anim_x+horizontal_change, anim_x, anim_duration, easing.linear ),
        }
    )
end

------------------------------------------------------------------------------------------


local RebindInputPopup = class( "Screen.RebindInputPopup", Screen.InfoPopup )

function RebindInputPopup:init( title, binding, fn )
    local body = LOC "UI.DIALOGS.REBIND_DESC"
    RebindInputPopup._base.init( self, title, body, LOC"UI.DIALOGS.CANCEL", fn )

    self.input = TheGame:GetInput()
    self.rebinding = binding

    self:AddButton( "Revert to default", nil, function()
        self:Rebind()
    end )
end

function RebindInputPopup:Rebind( binding )
    self.rebinding = binding
    self:GetFE():PopScreen( self )       
end

function RebindInputPopup:GetRebinding()
    return self.rebinding
end

function RebindInputPopup:HandleRawKeyDown( key )
    if not self.input:IsModifierKey( key ) then
        local binding = { key = key }
        if self.input:IsModifierControl() then
            binding.CTRL = true
        end
        if self.input:IsModifierShift() then
            binding.SHIFT = true
        end
        self:Rebind( binding )
    end
    return true
end
