require "ui/widgets/screen"

require "ui/widgets/rewardswidget"

local PANEL_PADDING = 40
local CONTENT_PADDING = 20
local DESC_PADDING = 25

local CONTENT_W = 1100

-----------------------------------------------------------------------------------------------
--

local AcceptJobPopup = class("Screen.AcceptJobPopup", Widget.Screen )

AcceptJobPopup.CONTROL_MAP =
{
    {
        control = Controls.Digital.MENU_ACCEPT,
        fn = function(self) 
            self:OnSelect(true)
        end,
        
        test = function(self) return TheGame:FE():IsGamepadMode() and self.accept end,
        hint = function(self, left, right) 
            table.insert(right, loc.format(LOC"UI.CONTROLS.ACCEPT_JOB", Controls.Digital.MENU_ACCEPT))
        end,
    },
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self) 
            self:OnSelect(false)
        end,
        test = function(self) return self.decline end,
        hint = function(self, left, right) 
            table.insert(right, loc.format(LOC"UI.CONTROLS.DECLINE_JOB", Controls.Digital.MENU_CANCEL))
        end,
    }
}


function AcceptJobPopup:init( quest, must_accept )
    AcceptJobPopup._base.init( self )

    self.quest = quest
    self.is_popup = true

    self:SetAnchors( "center", "center" )

    self.sound_loop = AUDIO:CreateEventInstance("event:/ui/main/fld/contract_briefing_popup")
    self.sound_loop:Start()

    -- A dark overlay blocking interaction with the background
    self.scrim = self:AddChild( Widget() ):SetTintAlpha(0.0)
    self.scrim_bg = self.scrim:AddChild( Widget.SolidBox( RES_X, RES_Y, 0x021A18F8 ) )
        :BlockMouse()
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    self.scrim_bg:SetSize( self.screen_w, self.screen_h )

    -- A container for all the background elements
    self.panel = self:AddChild( Widget.WidePanel( 680 ) )
        :SetIcon( engine.asset.Texture( "UI/icon_large_quest.tex" ) )
        :SetColour( UICOLOURS.QUEST )

    self.content = self:AddChild( Widget() )
    self.content_bg = self.content:AddChild( Widget.SolidBox( CONTENT_W, 200 ) ):SetTintAlpha( 0.0 )
    self.title_label = self.content:AddChild( Widget.Label( "body", FONT_SIZE.ITEM_LABEL, LOC"UI.JOB_POPUP.TITLE_LABEL" ) ):SetGlyphColour( UICOLOURS.SUBTITLE )
    self.title = self.content:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_SUBTITLE, self.quest:GetTitle() ) ):SetGlyphColour( UICOLOURS.TITLE )

    self.difficulty_widget = self.content:AddChild( Widget.QuestDifficultyWidget() )
        :Refresh( self.quest:GetRank() )
    self.focus_widget = self.content:AddChild( Widget.QuestFocusWidget() )
        :Refresh( self.quest:GetRatingString() )

    self.desc_bg = self.content:AddChild( Widget.Panel( engine.asset.Texture("UI/empty_inner_frame_border_selected.tex") ) )
        :SetBloom( 0.2 )
    local desc_text = self.quest:GetDesc()
    self.desc = self.content:AddChild( Widget.Label( "body", FONT_SIZE.BODY_TEXT, desc_text ) ):SetWordWrap( true ):SetAutoSize( CONTENT_W-DESC_PADDING*2 ):LeftAlign()
    self.desc:SetHoverLinkFn( UIHelpers.ItemTooltips )

    -- Quest response buttons
    self.buttons = self:AddChild( Widget() )
    self.accept = self.buttons:AddChild(Widget.IconButton(LOC"UI.JOB_POPUP.ACCEPT", function() self:OnSelect(true) end) )
        :SetIcon( global_images.accept )
        :SetColour( UICOLOURS.QUEST )
        :SetCta( true )

    if not must_accept then
        self.decline = self.buttons:AddChild( Widget.IconButton(LOC"UI.JOB_POPUP.DECLINE", function() self:OnSelect(false) end) )
            :SetIcon( global_images.close )
            :LayoutBounds( "before", "center", self.accept ):Offset( -130, 0 )
    end

    self.rewards = self.content:AddChild( Widget.RewardsWidget( CONTENT_W, 2 ) )
    self.rewards:RefreshRewards( self.quest )

    -- Set everything to quest colors
    self.title:SetGlyphColour( UICOLOURS.QUEST_LIGHT )
    self.title_label:SetGlyphColour( UICOLOURS.QUEST )

    self.title_label:LayoutBounds( "left", "top", self.content_bg ):Offset( 0, 0 )
    self.title:LayoutBounds( "left", "below", self.title_label):Offset( 0, 10 )
    self.focus_widget:LayoutBounds( nil, "top", self.title_label )
    self.focus_widget:LayoutBounds( "right", nil, self.content_bg )
    self.difficulty_widget:LayoutBounds( "before", "top", self.focus_widget ):Offset( -20, 0 )

    self.desc_bg:SizeToWidgets( 0, self.desc )
    local bg_w, bg_h = self.desc_bg:GetSize()
    self.desc_bg:SetSize( CONTENT_W, bg_h + CONTENT_PADDING ):LayoutBounds( "left", "below", self.title ):Offset( 0, -CONTENT_PADDING )
    self.desc:LayoutBounds( "left", "top", self.desc_bg ):Offset( DESC_PADDING, -DESC_PADDING )
    self.rewards:LayoutBounds( "left", "below", self.desc_bg ):Offset( 0, -CONTENT_PADDING )

    self.optionwidgets = {}

    self:Layout()

    self:SetSounds( SoundEvents.tut_popup, SoundEvents.tut_popout )
end

function AcceptJobPopup:OnOpen()
    AcceptJobPopup._base.OnOpen( self )
    if not self.loopsound then
        self.loopsound = AUDIO:CreateEventInstance("event:/ui/main/fld/contract_briefing_popup")
        self.loopsound:Start()
    end
    
end

function AcceptJobPopup:OnClose()
    AcceptJobPopup._base.OnClose( self )
    if self.loopsound then
        self.loopsound:Stop()
        self.loopsound:Release()
        self.loopsound = nil
    end
end

function AcceptJobPopup:GetMusicEvent()
    return "event:/music/new_quest"
end

function AcceptJobPopup:OnGainTop()
    AcceptJobPopup._base.OnGainTop( self )

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )


    -- Prepare for animation by getting current values
    local bg_anim_duration = 0.3
    local vertical_size_change = 40
    local title_label_x, title_label_y = self.title_label:GetPos()
    local title_x, title_y = self.title:GetPos()
    local desc_bg_x, desc_bg_y = self.desc_bg:GetPos()
    local desc_x, desc_y = self.desc:GetPos()

    -- Hide objects until they're animated in
    self.panel:SetTintAlpha(0)
    self.title_label:SetTintAlpha(0)
    self.title:SetTintAlpha(0)
    self.desc_bg:SetTintAlpha(0)
    self.desc:SetTintAlpha(0)
    self.rewards:PrepareAnimation()

    local updater = Updater.Series{
        Updater.Parallel{
            -- Fade and move in the background frame
            Updater.Ease( function(v) self.panel:SetTintAlpha(v) end, 0, 1, bg_anim_duration, easing.linear),
        },
    }

    updater:AddList{
        Updater.Parallel{
            Updater.Do( function() AUDIO:PlayEvent("event:/ui/game_over_screen/approach_list_appears") end ),
            -- Fade and move in the title label
            Updater.Ease( function(v) self.title_label:SetTintAlpha(v) end, 0, 1, bg_anim_duration, easing.linear),
            Updater.Ease2D( function(x,y) self.title_label:SetPos(x,y) end, title_label_x - 40, title_label_y, title_label_x, title_label_y, bg_anim_duration, easing.linear),
            
            -- Fade and move in the title
            Updater.Ease( function(v) self.title:SetTintAlpha(v) end, 0, 1, bg_anim_duration/2, easing.linear),
            Updater.Ease2D( function(x,y) self.title:SetPos(x,y) end, title_x - 60, title_y, title_x, title_y, bg_anim_duration/2, easing.linear),
            
        },
        Updater.Parallel{
            -- Fade and move in the description box
            Updater.Do( function() AUDIO:PlayEvent( "event:/ui/game_over_screen/stat_appear" ) end ),
            Updater.Ease( function(v) self.desc_bg:SetTintAlpha(v) end, 0, 1, bg_anim_duration/2, easing.linear),
            Updater.Ease2D( function(x,y) self.desc_bg:SetPos(x,y) end, desc_bg_x + 60, desc_bg_y, desc_bg_x, desc_bg_y, bg_anim_duration/2, easing.linear),
            
            -- Fade and move in the description
            Updater.Ease( function(v) self.desc:SetTintAlpha(v) end, 0, 1, bg_anim_duration, easing.linear),
            Updater.Ease2D( function(x,y) self.desc:SetPos(x,y) end, desc_x + 60, desc_y, desc_x, desc_y, bg_anim_duration/2, easing.linear),
        },
        -- Start animating the rewards panel
        Updater.Wait( bg_anim_duration/2 ),
        Updater.Do( function() self.rewards:AnimateIn() end )
    }

    -- start the animation
    self.fadein_anim = self:RunUpdater(updater)
end

function AcceptJobPopup:OnControlModeChange( cm, device_id )
    AcceptJobPopup._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.buttons:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.buttons:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.buttons:Hide()
    end
end

function AcceptJobPopup:OnScreenModeChange( sm )
    AcceptJobPopup._base.OnScreenModeChange( self, sm )

    self.content:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.buttons:SetLayoutScale( LAYOUT_SCALE[sm] )
    self:Layout()
end

function AcceptJobPopup:Layout()
    local w, h = self.content:GetScaledSize()
    h = h + PANEL_PADDING*2
    self.panel:SetHeight( h )

    self.content:LayoutBounds( nil, "bottom", self.panel ):Offset( 0, PANEL_PADDING*1.25 )
    self.buttons:LayoutBounds( "center", "below", self.panel ):Offset( 0, -10 )
end

function AcceptJobPopup:GetDefaultFocus()
    return self
end

function AcceptJobPopup:OnSelect( accepted )
    self.sound_loop:Stop()
    self.sound_loop:Release()
    self.sound_loop = nil

    if accepted then
        AUDIO:PlayEvent("event:/ui/main/fld/contract_accept")
    else
        AUDIO:PlayEvent("event:/ui/main/fld/contract_decline")
    end

    if self.close_fn then
        self.close_fn(accepted)
    end

    self:GetFE():PopScreen(self)
end

function AcceptJobPopup:AddOption( text )
    local OPTION_WIDTH = 200
    local idx = #self.optionwidgets + 1
    
    widget:SetIndex( self, idx )

    table.insert( self.optionwidgets, widget )
    self:AddChild( widget )

    local n = #self.optionwidgets
    for i, widget in ipairs( self.optionwidgets ) do
        widget:LayoutBounds( "center", "below", self.panel ):Offset( (i - (n+1)/2) * (OPTION_WIDTH + 12), 4 )
    end

    return widget
end

function AcceptJobPopup:SetFn( fn )
    self.close_fn = fn
    return self
end

function AcceptJobPopup:HarvestRobotOptions()
    self:ClearRobotOptions()
    self:AddRobotOption(self.accept)
    if self.decline then
        self:AddRobotOption(self.decline)
    end
 end


function AcceptJobPopup:OnUpdate(dt)
    self:HarvestRobotOptions()
    AcceptJobPopup._base.OnUpdate(self, dt)
end
