require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/menu"
require "ui/screens/popupmenu"
require "ui/screens/optionscreen"
require "ui/screens/negotiationscreen"
require "ui/screens/compendium"
require "ui/screens/moviescreen"
require "ui/screens/newgamescreen"
require "ui/screens/modmanagerscreen"
require "ui/screens/dailychallengescreen"
require "ui/screens/historyscreen"
require "ui/screens/achievementsscreen"
require "ui/widgets/imagebutton"
require "ui/widgets/panelbutton"
require "util/networkutil"
require "util/networktwitch"

local function TutorialsMenu( screen )
    
    local t = {}

    if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('negotiation_tutorial') then
        table.insert(t, {txt = LOC"UI.MAINMENU.NEGOTIATION", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "negotiation_tutorial"):SetAutoAdvance(false) ) end, icon = engine.asset.Texture("large/tutorial_negotiation.tex"), colour = UICOLOURS.NEGOTIATION, buttonclass = Widget.TutorialsButton } )
    end

    if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('battle_tutorial') then
        table.insert(t, {txt = LOC"UI.MAINMENU.BATTLE", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "battle_tutorial"):SetAutoAdvance(false) ) end, icon = engine.asset.Texture("large/tutorial_battle.tex"), colour = UICOLOURS.FIGHT, buttonclass = Widget.TutorialsButton } )
    end

    if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('rook_negotiation_tutorial') then
        table.insert(t, {txt = LOC"UI.MAINMENU.ROOK_NEGOTIATION", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "rook_negotiation_tutorial"):SetAutoAdvance(false) ) end, icon = engine.asset.Texture("large/tutorial_negotiation_rook.tex"), colour = UICOLOURS.NEGOTIATION, buttonclass = Widget.TutorialsButton } )
    end

    if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('rook_battle_tutorial') then
        table.insert(t, {txt = LOC"UI.MAINMENU.ROOK_BATTLE", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "rook_battle_tutorial"):SetAutoAdvance(false) ) end, icon = engine.asset.Texture("large/tutorial_battle_rook.tex"), colour = UICOLOURS.FIGHT, buttonclass = Widget.TutorialsButton } )
    end

    -- if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('smith_negotiation_tutorial') then
    --     table.insert(t, {txt = LOC"UI.MAINMENU.SMITH_NEGOTIATION", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "smith_negotiation_tutorial"):SetAutoAdvance(false) ) end, icon = engine.asset.Texture("large/tutorial_negotiation_smith.tex"), colour = UICOLOURS.NEGOTIATION, buttonclass = Widget.TutorialsButton } )
    -- end

    if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('smith_battle_tutorial') then
        table.insert(t, {txt = LOC"UI.MAINMENU.SMITH_BATTLE", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "smith_battle_tutorial"):SetAutoAdvance(false) ) end, icon = engine.asset.Texture("large/tutorial_battle_smith.tex"), colour = UICOLOURS.FIGHT, buttonclass = Widget.TutorialsButton } )
    end

    table.insert(t,  {txt = LOC"UI.MAINMENU.BACK", fn = function() AUDIO:PlayEvent("event:/ui/main/gen/back_general") screen:PopMenu() end, icon = engine.asset.Texture("UI/ic_mainmenu_back.tex"), buttonclass = Widget.AdvancedMenuButton })
    
    return t
end

local function ExperimentsMenu( screen )
    
    local t = {}

    for k, questdef in pairs( Content.GetAllQuests() ) do
        if questdef.qtype == QTYPE.EXPERIMENT and (not questdef.dev_only or BRANCH == "dev") and not questdef.hidden then
            table.insert(t, 
                {
                    txt = questdef.title, 
                    fn = function() 
                        questdef:IsolatedTest()  
                    end, 
                    icon = engine.asset.Texture("UI/ic_mainmenu_experimental.tex"),
                    buttonclass = Widget.ExperimentsButton
                })
        end
        if questdef.qtype == QTYPE.DAILY and questdef.daily_experiment then
            table.insert(t, 
            {
                txt = questdef.title, 
                fn = function() 
                    local daily = require "daily_util"
                    local daily_data = daily.GenerateDaily( 1599177073 )
                    Screen.DailyChallengeScreen.StartDaily( daily_data, 0 )
                end, 
                icon = engine.asset.Texture("UI/ic_mainmenu_experimental.tex"),
                buttonclass = Widget.ExperimentsButton
            })
        end
    end
    table.insert(t, 
    {
        txt = "Smith Day 3 (Full Day)", 
        fn = function() 
            UIHelpers.EditString( 
                    "Select Prestige", 
                    "Please enter a prestige level to test. (number between 0 and 7)",
                    "", 
                    function( val )
                        if val then
                            local adv = tonumber(val)
                            if adv and adv >= 0 and adv <= 7 then
                                local DECKS = require "content/quests/experiments/smith_day_2_decks"
                                local deck_idx = math.random(#DECKS)
                                local deck = DECKS[deck_idx]
                                local qdef = Content.GetQuestDef( "SMITH_STORY" ) 
                                qdef:IsolatedTest({start_on_day = 3}, 3, deck, adv)
                            else
                                TheGame:FE():PushScreen( Screen.InfoPopup( "Invalid Prestige", "Please enter a number between 0 and 7" ) )
                            end
                        end
                    end
                )
        end, 
        icon = engine.asset.Texture("UI/ic_mainmenu_experimental.tex"),
        buttonclass = Widget.ExperimentsButton
    })

    table.sort(t, function(a,b) return a.txt < b.txt end )
    table.insert(t,  {txt = LOC"UI.MAINMENU.BACK", fn = function() AUDIO:PlayEvent("event:/ui/main/gen/back_general") screen:PopMenu() end, icon = global_images.close, buttonclass = Widget.AdvancedMenuButton })
    return t
end


local function QuickPlay( screen, id )
    local settings = TheGame:GetLocalSettings()
    settings.SKIP_SLIDE_SHOWS = true
    settings.FAST_STARTUP = true

    local options = GameState.CreateOptions(id)
    TheGame:StartNewGame( options )
end

local function MainDemo( screen )

    local continue 
    if screen.current_save then
        continue = {txt = LOC"UI.MAINMENU.CONTINUE", fn = function() screen:OnPressContinue(screen.current_save) end, icon = engine.asset.Texture("UI/ic_menu_play.tex"), save_file = screen.current_save, cta = true, buttonclass = Widget.ContinueMenuButton }
    end

    local play = {txt = LOC"UI.MAINMENU.PLAY_DEMO", fn = function() screen:OnPressPlay() end, icon = engine.asset.Texture("UI/ic_menu_play.tex"), cta = true, buttonclass = Widget.PlayMenuButton }
    local options = {txt = LOC"UI.MAINMENU.OPTIONS", fn = function() screen:Options() end, icon = engine.asset.Texture("UI/ic_mainmenu_options.tex"), buttonclass = Widget.AdvancedMenuButton }
    local quit = {txt = LOC"UI.MAINMENU.QUIT", fn = function() screen:Quit() end, icon = engine.asset.Texture("UI/ic_mainmenu_quit.tex"), buttonclass = Widget.AdvancedMenuButton }

    if continue then
        return { continue, play, options, quit }
    else
        return { play, options, quit }
    end
end


local function Main( screen )
    -- Start menu list
    local opts = { }

    if not TheGame:IsDemo() then
        -- Add daily challenge
    	table.insert( opts, {
            txt = LOC"UI.MAINMENU.DAILY",
            fn = function() screen:OnPressDaily() end,
            cta = true,
            buttonclass = Widget.DailyChallengeMenuButton,
            disabled = not TheGame:GetGameProfile():HasUnlock("DAILY_UNLOCKED"),
            tt = not TheGame:GetGameProfile():HasUnlock("DAILY_UNLOCKED") and LOC"UI.MAINMENU.DAILY_LOCKED_TT"
        } ) 
    end

    -- Add play button
    table.insert( opts, {
        txt = LOC"UI.MAINMENU.PLAY",
        fn = function() screen:OnPressPlay() end,
        icon = engine.asset.Texture("UI/ic_menu_play.tex"),
        cta = true,
        buttonclass = Widget.PlayMenuButton
    } )

    -- Add continue button
    local continue 
    if screen.current_save then
        table.insert( opts, {
            txt = LOC"UI.MAINMENU.CONTINUE",
            fn = function() screen:OnPressContinue(screen.current_save) end,
            icon = engine.asset.Texture("UI/ic_menu_play.tex"),
            save_file = screen.current_save,
            cta = true,
            buttonclass = Widget.ContinueMenuButton
        } )
    end
    
    -- Compendium
    if SPECIAL_BUILD_TYPE ~= (1024+64) then
        table.insert( opts, {
            txt = LOC"UI.MAINMENU.CARDCOMPENDIUM",
            fn = function() screen:ShowCardCompendium() end,
            icon = engine.asset.Texture("UI/ic_mainmenu_compendium.tex"),
            buttonclass = Widget.AdvancedMenuButton
        } )
    end

    -- Achievements
    table.insert( opts, {
        txt = LOC"UI.MAINMENU.ACHIEVEMENTS",
        fn = function() screen:ShowAchievements() end,
        icon = engine.asset.Texture("UI/ic_achievements.tex"),
        buttonclass = Widget.AdvancedMenuButton
    } )
    
    return opts
end

----------------------------------------------------------------------------------------
-- Main menu.  You know the drill.

local MainMenu = class("Screen.MainMenu", Widget.Screen)

MainMenu.CONTROL_MAP = Widget.MergeControlMap(Widget.Screen.CONTROL_MAP,
{
    {
        control = Controls.Digital.MENU_CANCEL, 
        
        fn = function(self)
            self:PopMenu()
            return true
        end,
        
        test = function(self) return #self.menu_stack > 1 end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.BACK", Controls.Digital.MENU_CANCEL))
        end
    },
})

SCREEN_PADDING_X = 60
SCREEN_PADDING_Y = 40
FORUMS_URL = "https://forums.kleientertainment.com/forums/forum/148-griftlands-general-discussion/"
PATCH_NOTES_URL = "https://forums.kleientertainment.com/game-updates/griftlands/"
KLEI_GAMES_URL = "https://klei.com/"
KLEI_COMMUNITY_URL = "https://forums.kleientertainment.com/forums/forum/148-griftlands-general-discussion/"

local showed_language_warning = {}

function MainMenu:init()
    engine.inst:ProfilerPush("MainMenu:init")
    Widget.Screen.init(self)

    self:SetAnchors( "center", "center" )
    
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    self.gamepad_bottom_bar_offset = 0
   
    local last_played_character_id = TheGame:GetGameProfile():GetLastCharacterPlayed()
    local def = last_played_character_id and Content.GetCharacterDef( last_played_character_id )
    local title_screen_id = def and def.title_screen_plax or "title_screen"

    engine.inst:ProfilerPush("MainMenu:init Plax")
    self.plax = Widget.Plax():SetName("MAINMENU_PLAX"):SetAnchors( "center", "center" )
    self.plax:SetPlaxData( title_screen_id )
    engine.inst:ProfilerPop()

    engine.inst:ProfilerPush("MainMenu:init Logo")
    
    local language_info = Content.GetLocalizations()[ TheGame:GetLanguage() ]
    local logo_texture
    if TheGame:IsDemo() then
        logo_texture = language_info and language_info.demo_logo_image or "large/logo_demo.tex"
    elseif language_info and language_info.logo_image then
        logo_texture = language_info and language_info.logo_image
    end
    -- BRANCH = "testing"
    if BRANCH == "testing" then
        logo_texture = "large/logo_experimental.tex"
    end
    if logo_texture then
        self.logo = Widget.Image( engine.asset.Texture( logo_texture ) )
            :SetTintColour( 0xFFBED8FF )
            :Bloom( 0.2 )
    end

    if BRANCH == "testing" then
        -- Experimental-branch logo
        self.experimental_branch_btn = self:AddChild( Widget.PanelButton() )
            :SetText( LOC"UI.MAINMENU.EXPERIMENTAL_BRANCH_BTN" )
            :SetImage( engine.asset.Texture("UI/empty_inner_frame_border_selected_clear.tex") )
            :SetHoverImage( engine.asset.Texture("UI/empty_inner_frame_border_selected_clear.tex") )
            :SetBGColour( 0xFFBED8FF )
            :SetTextColour( 0xFFBED8FF )
            :SizeToLabel( 25, 8 )
            :Bloom( 0.2 )
            :Hide()
    elseif not self.logo then
        -- Regular logo
        local anim = engine.asset.Anim( "anim/logo_text_anim.zip" )
        local scale = 1
        self.logo = Widget.Anim():AddAnims(anim):SetBuild(anim):Play("fx", true):Bloom(0.32):SetScale(scale,scale)
        self.logo:SetGlitch(true)
        self.logo:SetGlitchSpeedAndScale(0.07, 6.0)
    end

    engine.inst:ProfilerPop()
   
    self.right_side = Widget():SetAnchors("right", "center")
    self.menu = self.right_side:AddChild( Widget.Menu( Widget.IconButton ) )
        :AllowMultiSizeButtons()
        :SetFontSize( 24 )
        :SetLayout( "right", nil, 0, -6 )
        :PrepareAnimation()
    self.menu.OnGainFocus = function() if TheGame:FE():IsGamepadMode() then self.menu:ScaleTo( nil, 1.05, 0.2, easing.outQuad ) end end
    self.menu.OnLoseFocus = function() self.menu:ScaleTo( nil, 1, 0.2, easing.outQuad ) end

    self.bottom_right = Widget():SetAnchors("right", "bottom"):SetName( "BOTTOM_RIGHT" )
    self.bottom_center = Widget():SetAnchors("center", "bottom"):SetName( "BOTTOM_CENTER" )
    self.bottom_left = Widget():SetAnchors("left", "bottom"):SetName( "BOTTOM_LEFT" )

    -- Add message-of-the-day widget
    
    if IsEarlyAccess() and not self:ShouldSkipExternalLinks() then
        engine.inst:ProfilerPush("MainMenu:init MOTD widget")
        self.motd_root_bg = self.bottom_left:AddChild( Widget.Image( engine.asset.Texture("large/mainmenu_motd_bg.tex") ) )
            :SetSize( 1200*0.6, 325*0.6 )
            :SetTintAlpha( 0 )
            :Hide()
            :Bloom( 0.4 )
        self.motd_root = self.bottom_left:AddChild( Widget.MOTDContainer() )
            :SetFn( function( motd_data ) self:MessageClicked( motd_data ) end )
        self.motd_root.OnGainFocus = function() self.motd_root:ScaleTo( nil, 1.05, 0.2, easing.outQuad ) end
        self.motd_root.OnLoseFocus = function() self.motd_root:ScaleTo( nil, 1, 0.2, easing.outQuad ) end
        engine.inst:ProfilerPop()
    end

    -- Add build version label
    engine.inst:ProfilerPush("MainMenu:init Version label")
    self.build = self.bottom_left:AddChild(Widget()):Hide()
    self.build_bg = self.build:AddChild( Widget.Panel( engine.asset.Texture("UI/mainmenu_build_bg.tex") ) )
        :SetNineSliceCoords( 75, 0, 150, 57 )
        :SetNineSliceBorderScale( 0.5 )
        :Bloom( 0.1 )
    self.build_txt = self.build:AddChild( Widget.Label( "default", 24, BUILD_ID_STRING ):SetGlyphColour( UICOLOURS.DARK_BODYTEXT ) )
    engine.inst:ProfilerPop()

    -- Add link buttons to the bottom of the screen
    engine.inst:ProfilerPush("MainMenu:init External link buttons")
    self.link_buttons = self.bottom_left:AddChild( Widget() )
        :SetFocusGroup( FOCUS_GROUP.LAST_FOCUS )
        :SetFocusLock(true)
    self.link_buttons.OnGainFocus = function()
        self.link_buttons.parent:ScaleTo( nil, 1.05, 0.2, easing.outQuad )
        for k, v in ipairs( self.link_buttons.children ) do
            if v:IsShown() then
                v:SetFocus( true )
                break
            end
        end
    end
    self.link_buttons.OnLoseFocus = function()
        self.link_buttons.parent:ScaleTo( nil, 1, 0.2, easing.outQuad )
    end

    -- External links to the Klei website and forums
    if not self:ShouldSkipExternalLinks() then
        if IsEarlyAccess() then
            self.updates_link = self.link_buttons:AddChild( Widget.MainMenuButton( engine.asset.Texture("UI/ic_mainmenu_updates.tex"), LOC"UI.MAINMENU.GAME_UPDATES", function() self:ShowMOTDListPopup() end ) )
        end
        self.community_link = self.link_buttons:AddChild( Widget.MainMenuLink( engine.asset.Texture("UI/ic_mainmenu_community.tex"), LOC"UI.MAINMENU.KLEI_COMMUNITY", KLEI_COMMUNITY_URL ) )
        self.klei_link = self.link_buttons:AddChild( Widget.MainMenuLink( engine.asset.Texture("UI/ic_mainmenu_klei.tex"), LOC"UI.MAINMENU.KLEI_GAMES", KLEI_GAMES_URL ) )
        self.links_divider = self.link_buttons:AddChild( Widget.SolidBox( 2, 110, UICOLOURS.SUBTITLE ) ):Bloom( 0.1 ):LayoutBounds( "after", "center" )
    end
    -- Game buttons
    self.history_link = self.link_buttons:AddChild( Widget.MainMenuButton( engine.asset.Texture("UI/ic_mainmenu_history.tex"), LOC"UI.MAINMENU.HISTORY", function() self:OnPressHistory() end ) )
    if TheGame:GetGameProfile():HasSeenMessage('negotiation_tutorial') or TheGame:GetGameProfile():HasSeenMessage('battle_tutorial') then
        self.tutorials_link = self.link_buttons:AddChild( Widget.MainMenuButton( engine.asset.Texture("UI/ic_mainmenu_tutorials.tex"), LOC"UI.MAINMENU.TUTORIALS", function()
            if self:GetCurrentMenu() ~= TutorialsMenu then
                if not self:IsMenuStart() then self:PopMenu() end
                self:PushMenu( TutorialsMenu )
            end
        end ) )
    end
    self.mods_link = self.link_buttons:AddChild( Widget.MainMenuButton( engine.asset.Texture("UI/ic_mods.tex"), LOC"UI.MAINMENU.MODS", function() self:ShowMods() end ) )
    if false and (BRANCH == "testing" or BRANCH == "dev") then
        local EXPERIMENTAL_PASSWORD = "krashio"
        self.experiments_link = self.link_buttons:AddChild( Widget.MainMenuButton( engine.asset.Texture("UI/ic_mainmenu_experimental.tex"), LOC"UI.MAINMENU.EXPERIMENTAL", function()
            if TheGame:GetGameProfile():HasSeenMessage('EXPERIMENTAL_PASSWORD_CHECK') then
                if self:GetCurrentMenu() ~= ExperimentsMenu then
                    if not self:IsMenuStart() then self:PopMenu() end
                    self:PushMenu( ExperimentsMenu )
                end
            else
                UIHelpers.EditString( 
                    "Password required", 
                    "There are bugs and spoilers lurking behind this menu, so a password is required. The password is: KRASHIO",
                    "", 
                    function( val )
                        if val then
                            if loc.tolower(val) == EXPERIMENTAL_PASSWORD then
                                TheGame:GetGameProfile():SetHasSeenMessage('EXPERIMENTAL_PASSWORD_CHECK')
                                TheGame:FE():PushScreen( Screen.InfoPopup( "Let the Experiments Begin!", "That's the right password! You can now run experiments!" ) )
                            else
                                TheGame:FE():PushScreen( Screen.InfoPopup( "That's not the right password", "Hint: The password is KRASHIO" ) )
                            end
                        end
                    end
                )
            end
        end ) )
    end
    self.options_link = self.link_buttons:AddChild( Widget.MainMenuButton( engine.asset.Texture("UI/ic_mainmenu_options.tex"), LOC"UI.MAINMENU.OPTIONS", function() self:Options() end ) )
    if SPECIAL_BUILD_TYPE ~= (1024+64) then
        self.quit_link = self.link_buttons:AddChild( Widget.MainMenuButton( engine.asset.Texture("UI/ic_mainmenu_quit.tex"), LOC"UI.MAINMENU.QUIT", function() self:Quit() end ) )
    end

    engine.inst:ProfilerPop()
    
    engine.inst:ProfilerPush("MainMenu:init Add children")
    self:AddChildren{
        self.plax,
        self.logo,

        self.bottom_left,
        self.bottom_center,
        self.bottom_right,
        self.right_side,
    }
    engine.inst:ProfilerPop()

    self:UpdateBuildWidget( BUILD_ID_STRING )

    local gamertag = engine.inst:GetGamerTag()
    if gamertag and #gamertag > 0 then
        local tagbg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/menubutton_outline.tex"), 300, 38 )):SetTintColour( 0x00CCCCCC )
            :SetAnchors( "right", "top")
        self.gamertag_label = self:AddChild(Widget.Label("title", 32, gamertag )):SetAnchors("right", "top"):LayoutBounds( "before", "below", -48, -48)
        tagbg:SizeToWidgets( { 100, -24 }, self.gamertag_label )
    end
    
    self.menu_stack = {}

    if self.motd_root and not self:ShouldSkipExternalLinks() then
        NetworkUtil.GetMOTD(function(motd) if not self.removed then self:RefreshMOTD(motd) end end)
    end
    
    engine.inst:ProfilerPop()

    -- If we have Twitch mode active, try to connect
    if engine.inst:GetSetting("gameplay", "twitch_mode", 0) == 1 then
        -- Twitch mode is active. Make initial request
        Twitch.MainMenuInitialConnection()
    end

end

function MainMenu:UpdateBuildWidget( build_id_string )
    local min_w = 200
    local padding_x = 40
    local bg_scale = 0.5
    if build_id_string then self.build_txt:SetText( build_id_string ) end
    local label_w, label_h = self.build_txt:GetSize()
    self.build_bg:SetSize( math.max( label_w+padding_x, min_w )/bg_scale, 4 )
    self.build_bg:SetScale( -bg_scale, 0.3 )
    self.build_txt:LayoutBounds( "center", "center", self.build_bg ):Offset( -5, 0 )
    
    self.build:LayoutBounds( "left", "bottom", 0,0 ):Offset( SCREEN_PADDING_X/2 + 30, -3 )
end

function MainMenu:MessageClicked( motd_data )
    if motd_data.text and motd_data.text ~= '' then
        -- There's long text, open the popup
        self:GetFE():FadeTransition(function() 
            self:GetFE():PushScreen(Screen.MOTDPopup( motd_data ))
                :SetFn( function( motd_data ) self:MessageClicked( motd_data ) end )
        end)
    elseif motd_data.link then
        -- No long text, open the link directly
        engine.inst:OpenUrlInBrowser( motd_data.link )
    end
end

function MainMenu:OnPressDaily()
    AUDIO:PlayEvent("event:/ui/main/gen/press_play")
    TheGame:FE():FadeTransition(function()
        if self.daily_save then
            TheGame:FE():InsertScreen( Screen.YesNoPopup(LOC"UI.MAINMENU.DAILY_CONTINUE_TITLE", LOC"UI.MAINMENU.DAILY_CONTINUE_BODY", true ))
                :SetYesText(LOC"UI.MAINMENU.DAILY_CONTINUE")
                :SetNoText(LOC"UI.MAINMENU.DAILY_START_OVER")
                :SetFn(
                    function(v) 
                        if v == Screen.YesNoPopup.YES then 
                            TheGame:LoadGame( self.daily_save )
                        elseif v == Screen.YesNoPopup.NO then
                            TheGame:DeleteSave(self.daily_save, function() 
                                self.daily_save = nil
                                TheGame:FE():InsertScreen( Screen.DailyChallengeScreen())
                            end)
                        end
                    end )
        else
            TheGame:FE():InsertScreen( Screen.DailyChallengeScreen() )
        end
    end)
end

function MainMenu:OnPressPlay()
    AUDIO:PlayEvent("event:/ui/main/gen/press_play")
    TheGame:FE():FadeTransition(function()
        self.current_save_dirty = true
        TheGame:FE():InsertScreen( Screen.NewGameScreen() )
    end)
end

function MainMenu:OnPressHistory()
    TheGame:FE():FadeTransition(function()
        TheGame:FE():InsertScreen( Screen.HistoryScreen() )
    end)
end

function MainMenu:OnPressContinue()
    if self.current_save then
        AUDIO:PlayEvent("event:/ui/main/gen/press_play")
        TheGame:FE():FadeTransition(function()
            TheGame:LoadGame( self.current_save )
        end)
    end
end

function MainMenu:ShowCardCompendium()
    AUDIO:FinalizeVoiceLoad() -- We may need to play voices in the compendium. Make sure they are loaded.
    TheGame:FE():FadeTransition(function()
        local st = engine.inst:QueryPreciseTime()
        local screen = TheGame:FE():InsertScreen( Screen.Compendium() )
        print( "Compendium() took:", (engine.inst:QueryPreciseTime() - st) * 1000 )
    end) 
end

function MainMenu:ShowAchievements()
    TheGame:FE():FadeTransition(function()
        TheGame:FE():InsertScreen( Screen.AchievementsScreen() )
    end) 
end

function MainMenu:ShowMods()
    TheGame:FE():FadeTransition(function()
        TheGame:FE():InsertScreen( Screen.ModManagerScreen() )
    end) 
end

function MainMenu:HandleControlUp( control, device )
    if control:Has( Controls.Digital.MENU_CANCEL ) then
        self:Quit()
        return true

    elseif control:Has( Controls.Digital.OPEN_HOME ) then
        engine.inst:OpenUserHome()
        return true
    end

end

function MainMenu:RefreshMOTD(motd)
    self.motd_data = motd
    self.motd_list = {}

    local boxes = self.motd_data.motd.boxes
    local language_info = Content.GetLocalizations()[ TheGame:GetLanguage() ]
    if self.motd_data.motd.locale and language_info and self.motd_data.motd.locale[ language_info.motd_key ] ~= nil then
        boxes = self.motd_data.motd.locale[ language_info.motd_key ]
    end

    -- Go through all the available boxes and prepare the data for display
    for k, message in pairs( boxes or table.empty ) do
        -- Get the texture for this message's image
        message.texture = self.motd_data.images[ message.image ]
        if message.title and message.title ~= '' then
            -- Put this message on the list
            table.insert( self.motd_list, message )
        end
    end

    -- Update the widget
    self.motd_root:Refresh( self.motd_list )
    -- Position it
    self.motd_root:LayoutBounds( "left", "bottom", 0, 0 ):Offset( SCREEN_PADDING_X, SCREEN_PADDING_Y+16 )
    self.motd_root_bg:Show():AlphaTo( 0.3, 0.5 )

    -- Get the next update info, and display that too
    --[[local next_update = self.motd_data and self.motd_data.motd and self.motd_data.motd.NEXT_UPDATE_COUNTER
    if next_update then
        self.motd_root:ShowNextUpdateTime( self.motd_data.motd.NEXT_UPDATE_COUNTER )
        self:LayoutMotd()
    end--]]

    -- Update the Experimental branch info button
    if self.experimental_branch_btn and self.motd_data.motd.EXPERIMENTAL_BRANCH_INFO_URL then
        self.experimental_branch_btn:SetFn( function() engine.inst:OpenUrlInBrowser( self.motd_data.motd.EXPERIMENTAL_BRANCH_INFO_URL ) end ):Show()
    end

end

function MainMenu:DoRender()
    self.plax:RenderToScene()
end

function MainMenu:LayoutMotd()

    if TheGame:FE():GetScreenMode() == SCREEN_MODE.MONITOR then
        self.motd_root:Show()
        self.motd_root_bg:LayoutBounds( "left", "bottom", 0, 0 )
        self.motd_root:LayoutBounds( "left", "bottom", 0, 0 ):Offset( SCREEN_PADDING_X, SCREEN_PADDING_Y + self.gamepad_bottom_bar_offset - 10 )
        self.updates_link:Hide()
    elseif TheGame:FE():GetScreenMode() == SCREEN_MODE.SMALL or TheGame:FE():GetScreenMode() == SCREEN_MODE.TV then
        self.motd_root:Hide()
        self.updates_link:Show()
    end
end

function MainMenu:Layout()
    local cm = self:GetFE():GetControlMode()
    if cm == CONTROL_MODE.GAMEPAD then
        self.gamepad_bottom_bar_offset = 40
    else
        self.gamepad_bottom_bar_offset = 0
    end
    
    local sm = self:GetFE():GetScreenMode()
    if sm == SCREEN_MODE.MONITOR then
        SCREEN_PADDING_X = 60
    elseif sm == SCREEN_MODE.TV then
        SCREEN_PADDING_X = 60
    elseif sm == SCREEN_MODE.SMALL then
        SCREEN_PADDING_X = 80
    end
    
    if self.experimental_branch_btn then self.experimental_branch_btn:SetLayoutScale( LAYOUT_SCALE[sm] ) end
    
    self.link_buttons:SetLayoutScale( (LAYOUT_SCALE[sm]+1)/2 )
    self.menu:SetLayoutScale( (LAYOUT_SCALE[sm]+1)/2 )

    if TheGame:FE():GetScreenMode() == SCREEN_MODE.MONITOR then
        if self.motd_root then
            self:SetFocusRing{ self.motd_root, self.menu, self.link_buttons }
            self:LayoutMotd()
        else
            self:SetFocusRing{ self.menu, self.link_buttons }
        end
        self.link_buttons:Reparent( self.bottom_right )
            :StackChildrenRow( 20 )
            :LayoutBounds( "right", "bottom", 0,0 ):Offset( -SCREEN_PADDING_X, SCREEN_PADDING_Y + self.gamepad_bottom_bar_offset )
    elseif TheGame:FE():GetScreenMode() == SCREEN_MODE.SMALL or TheGame:FE():GetScreenMode() == SCREEN_MODE.TV then
        self:SetFocusRing{ self.menu,  self.link_buttons }
        if self.motd_root then
            self:LayoutMotd()
        end
        self.link_buttons:Reparent( self.bottom_left )
            :StackChildrenRow( 20 )
            :LayoutBounds( "left", "bottom", 0,0 ):Offset( SCREEN_PADDING_X, SCREEN_PADDING_Y + self.gamepad_bottom_bar_offset )
    end
   
    if #self.menu_stack > 0 then
        self:RefreshMenu()
    end
    
    self.logo:LayoutBounds( "right", "top", 0, 0 )
        :Offset( self.screen_w/2 - SCREEN_PADDING_X, self.screen_h/2 - SCREEN_PADDING_Y )
        :SetPos( nil, 290 )
    if BRANCH == "testing" then
        self.logo:SetPos( nil, 350 )
    end
    if self.experimental_branch_btn then
        self.experimental_branch_btn:LayoutBounds( "left", "center", self.logo ):Offset( 11, -105 )
    end
    TheGame:FE():RefreshControlOptions()
end

function MainMenu:RefreshMenu()
    local menu = self.menu_stack[ #self.menu_stack ]
    self.menu:SetOptions( menu(self) )
    self.menu:LayoutBounds( "right", nil, -SCREEN_PADDING_X, 0 )
    if TheGame:FE():GetScreenMode() == SCREEN_MODE.MONITOR then
        self.menu:LayoutBounds( nil, "above", self.link_buttons ):Offset( 0, 60 )
    else
        self.menu:LayoutBounds( nil, "bottom", 0, -RES_Y/2 + SCREEN_PADDING_Y + self.gamepad_bottom_bar_offset + 30 )
    end
    self.menu:AnimOptions()
end

function MainMenu:PushMenu( menu )
    table.insert( self.menu_stack, menu )
    self:RefreshMenu()
end

function MainMenu:GetCurrentMenu()
    return self.menu_stack and self.menu_stack[ #self.menu_stack ]
end

function MainMenu:IsMenuStart()
    return self.menu_stack and #self.menu_stack == 1
end

function MainMenu:PopMenu()
    table.remove( self.menu_stack )
    self:RefreshMenu()
end

local FIRST_OPEN = true
local has_shown_beta_message = false

local function ApplyRetroactiveUnlocks()
    local profile = TheGame:GetGameProfile()

    -- Unlock any point unlocks that should be available.
    local unlocks = profile:CheckUnlocks()
    if unlocks and #unlocks > 0 then
        for k,v in ipairs(unlocks) do
            print ("RETRO UNLOCK:", v.unlock_id)
        end
    end
    profile:VerifyPrestigePoints()
    profile:VerifyPerkPoints()
end

function MainMenu:OnOpen()
    MainMenu._base.OnOpen(self)

    --     if CONFIG ~= "PRODUCTION" and SPECIAL_BUILD_TYPE < 1024 then
    --         local memutil = require "util/memutil"
    --         self:DoTaskInTime("checklua", 1, memutil.CheckEntityRefs )
    --     end
    if TheGame:GetGameProfile():GetFatalError() then
        local function OnConfirm()
            TheGame:Shutdown()
        end
        UIHelpers.InfoPopup( LOC"UI.MAINMENU.PROFILE_ERR_TITLE", LOC"UI.MAINMENU.PROFILE_ERR_BODY", nil, OnConfirm )
        return
    end

    local language = TheGame:GetLanguage()
    local locinfo = Content.GetLocalizations()[language]
    if locinfo and locinfo.incomplete and not showed_language_warning[ language ] then
        UIHelpers.ShowBetaMessage(LOC"UI.MAINMENU.INCOMPLETE_LANGUAGE_TITLE", LOC"UI.MAINMENU.INCOMPLETE_LANGUAGE_BODY", LOC"UI.MAINMENU.INCOMPLETE_LANGUAGE_BUTTON", function() self:doStart() end )
        showed_language_warning[ language ] = true
    else
        self:doStart()
    end
    
    self:EnumerateSaveGames()

    if not TheGame:IsDemo() then
        ApplyRetroactiveUnlocks()
    end
    
    if not FIRST_OPEN then
        TheGame:FE():FadeIn( 0.25 )
    end

    FIRST_OPEN = false
end


function MainMenu:doStart()

    if TheGame:IsDemo() then
        if not has_shown_beta_message then
            has_shown_beta_message = true
            local popup = Screen.BetaMessagePopup( LOC"UI.MAINMENU.DEMO_TITLE", LOC"UI.MAINMENU.DEMO_BODY", LOC"UI.MAINMENU.DEMO_BUTTON", function() self:PushMenu( MainDemo ) end )
            TheGame:FE():PushScreen(popup)
        else
            self:PushMenu( MainDemo )
        end

    elseif IsEarlyAccess() and not TheGame:GetDebug() and not has_shown_beta_message then
        has_shown_beta_message = true
        UIHelpers.ShowBetaMessage(LOC"UI.MAINMENU.EA_TITLE", LOC"UI.MAINMENU.BETA_BODY", LOC"UI.MAINMENU.BETA_BUTTON", function() self:PushMenu(  Main ) end )
    else
        self:PushMenu( Main )
    end
end

function MainMenu:EnumerateSaveGames()
    self.save_count = 0
    self.daily_save = nil
    self.current_save = nil

    print( "MainMenu:EnumerateSaveGames()")

    -- Iterate existing save games to see if we need to auto-unlock any characters.
    local function EnumerateSaves( t )
        for name, header in pairs( t ) do
            if header.data and header.data.options then
                local act_data = GetPlayerActData( header.data.options.start_act )
                local bg = act_data and GetPlayerBackground( act_data.background.id )

                print ("\t", header.name, header.data.options.start_act, type(header.data.time) == "number" and os.date("%x %X", header.data.time ) or "-no time-" )

                if act_data and act_data.data and act_data.data.game_type == GAME_TYPE.DAILY then
                    self.daily_save = header.filename

                elseif string.match( name, "^slot_" ) and not string.match( name, "_rollback$" ) then

                    self.save_count = self.save_count + 1

                    if TheGame:GetGameProfile():GetCurrentSaveGame() == header.filename and TheGame:CanLoadSave( header.data ) then
                        self.current_save = header.filename
                    end
                end

                if not TheGame:IsDemo() and FIRST_OPEN then
                    if bg and bg.data.unlock_id and not TheGame:GetGameProfile():HasUnlock( bg.data.unlock_id ) then
                        if act_data and act_data.data.game_type ~= GAME_TYPE.DAILY then
                            print( "Auto-unlocking", bg.data.unlock_id, " due to existence of save game:", name )
                            TheGame:GetGameProfile():AcquireUnlock( bg.data.unlock_id )
                        end
                    end
                end
            end
        end

        print( "-----------" )
    end
        
    TheGame:ListSaveGames( EnumerateSaves )
end

function MainMenu:OnGainTop()
    MainMenu._base.OnGainTop( self )
    if self.current_save_dirty then
        self.current_save_dirty = false
        self:EnumerateSaveGames()
        self:RefreshMenu()
    end
    self.plax:PlaySounds()
end

function MainMenu:GetMusicEvent()
    return self.plax:GetMusicEvent() or "event:/music/town"
end

function MainMenu:OnControlModeChange( cm, device_id )
    MainMenu._base.OnControlModeChange( self, cm, device_id )
    self:Layout()
end

function MainMenu:OnScreenModeChange( sm )
    MainMenu._base.OnScreenModeChange( self, sm )
    self:Layout()
end

function MainMenu:GetDefaultFocus()
    return self.menu
end

function MainMenu:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    elseif GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end


function MainMenu:ShowMOTDListPopup()
    if self.motd_list then
        local update_string = self.motd_root:GetNextUpdateString()
        self:GetFE():FadeTransition(function() 
            self:GetFE():PushScreen(Screen.MOTDListPopup(self.motd_list,update_string))
                :SetFn( function( motd_data ) self:MessageClicked( motd_data ) end )
        end)
    end
end

function MainMenu:SetCurrentSaveDirty()
    self.current_save_dirty = true
end

function MainMenu:Options()
    self:GetFE():FadeTransition(function() 
        self:GetFE():PushScreen(Screen.OptionScreen())
    end)
end


function MainMenu:Quit()
    TheGame:FE():PushScreen( Screen.YesNoPopup( LOC"UI.DIALOGS.QUIT_TITLE", LOC"UI.DIALOGS.QUIT_BODY" ) )
        :SetFn(
            function(v)
                if v == Screen.YesNoPopup.YES then 
                    self:GetFE():FadeTransition( function() TheGame:Shutdown() end )
                end 
            end )
end

function MainMenu:OnScreenResize(w,h)
    MainMenu._base.OnScreenResize(self, w, h)

    if self.versionstr then
        local x1, y1, x2, y2 = self.versionstr:GetBoundingBox()
        self.versionstr:SetPos((x2-x1)*0.5*self:GetFE():GetBaseWidgetScale(), (y2-y1)*0.5*self:GetFE():GetBaseWidgetScale())
    end

    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    self:Layout()
end

function MainMenu:OnUpdate( dt )
    MainMenu._base.OnUpdate(self, dt)

    self.t = (self.t or 0) + dt
    self.plax:SetPlaxShift(.67*math.cos(self.t/3), 0)
end


function MainMenu:ApplyColourCorrectionValues(v)
    self.plax:ApplyColourCorrectionValues(v)
end

function MainMenu:GetCurrentColourName()
    return self.plax:GetCurrentColourName()
end

function MainMenu:ShouldSkipExternalLinks()
    return SPECIAL_BUILD_TYPE == (1024+64) or TheGame:IsDemo()
end

------------------------------------------------------------------------------------------
-- Shows an image button with text that links to an external website

local MainMenuLink = class( "Widget.MainMenuLink", Widget.Clickable )
MainMenuLink.can_focus = true

local IMG_SIZE = 45
local TEXT_W = 80

function MainMenuLink:init( icon, text, url )
    MainMenuLink._base.init( self )

    self.image = self:AddChild( Widget.Image( icon, IMG_SIZE, IMG_SIZE ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
    self.text = self:AddChild( Widget.Label( "title", 20, text and text:upper() or "" ) )
    self.text:SetGlyphColour( UICOLOURS.SUBTITLE )
    self.text:SetWordWrap( true ):SetAutoSize( TEXT_W )
    self.text:OverrideLineHeight( 18 )
    self.text:LayoutBounds( "center", "below", self.image ):Offset( 0, -10 )

    self:Bloom( 0.1 )

    self:SetOnClickFn( function() engine.inst:OpenUrlInBrowser( url ) end )
    self:SetScales( 1.0, 1.1, 1.1, 0.1 )
    
end

function MainMenuLink:SetText( text )
    self.text:SetText( text )
    self.text:LayoutBounds( "center", "below", self.image ):Offset( 0, -10 )
end

function MainMenuLink:Click()
    self:HotkeyDown()
    self:HotkeyUp()
end

------------------------------------------------------------------------------------------
-- Shows an image button with text

local MainMenuButton = class( "Widget.MainMenuButton", Widget.MainMenuLink )

function MainMenuButton:init( icon, text, fn )
    MainMenuButton._base.init( self, icon, text, "" )

    self:SetOnClickFn( fn )
end

------------------------------------------------------------------------------------------
-- Shows a widget containing messages of the day

local MOTDContainer = class( "Widget.MOTDContainer", Widget )

local ITEMS_PER_PAGE = 3
local ITEM_SPACING = 15

function MOTDContainer:init( box_data )
    MOTDContainer._base.init( self )

    self.box_data = box_data
    self.boxes = self:AddChild( Widget() )
    self.pagination = self:AddChild( Widget() )
    self.next_update_label = self:AddChild( Widget.Label( "title", FONT_SIZE.SMALL_TEXT, "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetWordWrap( true )
        :OverrideLineHeight( FONT_SIZE.BUTTON*LINE_HEIGHT_TEXT_MULT )
        :SetHiddenBoundingBox( true )
        :Bloom( 0.1 )
        :Hide()
    self:SetFocusGroup( FOCUS_GROUP.LAST_FOCUS )
    self:SetFocusLock(true )
end

function MOTDContainer:Refresh( box_data )
    self.box_data = box_data

    -- Clear any old messages
    self.boxes:DestroyAllChildren()

    -- Add brand new ones
    local box_w, box_h = 0, 0
    for k, message in pairs( self.box_data ) do
        local box = self.boxes:AddChild( Widget.MOTDWidget( message ) )
        box_w, box_h = box:GetSize()
        box:SetOnClickFn( function() if self.fn then self.fn( message ) end end )
    end

    -- Align them
    for k, box in ipairs( self.boxes.children ) do
        -- Assign this item a spot. If there are three items per page,
        -- the first will get 0, the second will get 1, the third will get 2, 
        -- the fourth will get 0 again, and so on
        local spot = ( k - 1 ) % ITEMS_PER_PAGE
        -- Calculate the x-position based on what spot this box is at, including spacings
        local pos_x = box_w * spot + ITEM_SPACING * spot
        -- Set the position
        box:SetPos( pos_x, 0 )
        -- Hide it
        box:SetTintAlpha( 0 )
    end

    -- Animate in the boxes
    local delay_per_box = 0.2
    local timing = 0.15
    for k = 1, math.max( ITEMS_PER_PAGE, #self.boxes.children ) do
        if k <= #self.boxes.children then
        local box = self.boxes.children[k]
            box:RunUpdater(
                Updater.Series{
                    Updater.Wait( delay_per_box*k ),
                    Updater.Parallel{
                        Updater.Ease( function(v) box:SetTintAlpha(v) end, 0, 1, timing, easing.linear),
                        Updater.Ease( function(v) box:SetScale(v,v) end, 1.1, 1, timing, easing.linear)
                    }
                }
            )
        end
    end

    -- Clear any old pagination markers
    self.pagination:DestroyAllChildren()

    -- Add brand new ones
    local page_count = math.ceil ( #self.boxes.children / ITEMS_PER_PAGE )
    for k = 1, page_count do
        local dot = self.pagination:AddChild( Widget.Clickable( ) )
        dot.hitbox = dot:AddChild( Widget.SolidBox( 67, 30, 0xFF000000 ))
        dot.image = dot:AddChild( Widget.Image( engine.asset.Texture("UI/motd_page.tex"), 67, 8 ) ):Bloom( 0.1 )
        dot:SetOnClickFn( function() self:SetPage( k ) end )
        dot:SetHoverFn( function( hover ) if hover then dot:ScaleTo( nil, 1.05, 0.1 ) dot:Bloom(0.2) else dot:ScaleTo( nil, 1, 0.1 ) dot:Bloom(0.1) end end )
        dot:SetFocusable( false )
    end
    -- Align them
    self.pagination:StackChildrenRow( 15 )

    -- Position everything
    self.pagination:SetPos( 0, 0 )
    self.boxes:LayoutBounds( "left", "above", self.pagination ):Offset( 0, 0 )

    self:SetPage( 1 )

end

function MOTDContainer:ShowNextUpdateTime( motd_data )
    if motd_data and motd_data.last_time and motd_data.next_time then

        local seconds_per_hour = 3600
        local seconds_per_day = seconds_per_hour * 24
        local seconds_per_week = seconds_per_day * 7
        local days_per_week = 7

        local last_update_date = tonumber(motd_data.last_time)
        local next_update_date = tonumber(motd_data.next_time)+seconds_per_day or 0
        local current_date = os.time()
        local difference_last_update = current_date - last_update_date
        local difference_next_update = next_update_date - current_date

        -- Write down the time remaining in a readable way
        self.next_update_label:Show()
        if difference_last_update <= seconds_per_day then
            self.next_update_label:SetText( LOC"UI.MAINMENU.UPDATE_TEXT_RECENT" )
        elseif difference_next_update/seconds_per_day <= -3 then
            -- If the update should've gone out 3 days ago
            self:Hide()
        elseif difference_next_update/seconds_per_day < 1 then
            -- If the update is going out in a day
            self.next_update_label:SetText( LOC"UI.MAINMENU.UPDATE_TEXT" )
        elseif difference_next_update/seconds_per_day < 7  then
            -- If the update is going out in a few days
            self.next_update_label:SetText( loc.format( LOC"UI.MAINMENU.UPDATE_TEXT_DAYS", math.floor( difference_next_update/seconds_per_day ) ) )
        else
            -- If the update is going out in a few weeks
            local weeks = difference_next_update/seconds_per_week
            local days = ( weeks - math.floor(weeks) )*days_per_week
            if math.floor( days ) > 0 then
                self.next_update_label:SetText( loc.format( LOC"UI.MAINMENU.UPDATE_TEXT_WEEKS_DAYS", math.floor( weeks ), math.floor( days ) ) )
            else
                self.next_update_label:SetText( loc.format( LOC"UI.MAINMENU.UPDATE_TEXT_WEEKS", math.floor( weeks ) ) )
            end
        end

        -- Layout
        self.next_update_label:LayoutBounds( "left", "above", self.boxes )
            :Offset( 0, 8 )
    end
end

function MOTDContainer:GetNextUpdateString()
    return self.next_update_label:GetText()
end

function MOTDContainer:IsInitialized()
    return self.page ~= nil
end

function MOTDContainer:SetPage( page )
    self.page = page or 1

    -- Update the pagination dots
    for k, dot in ipairs( self.pagination.children ) do
        if k == self.page then
            -- Current page
            dot:SetTintAlpha( 1 )
        else
            -- Not the current page..
            dot:SetTintAlpha( 0.3 )
        end
    end

    -- Update page widgets visibility
    for k, box in ipairs( self.boxes.children ) do
        if k > ( self.page*ITEMS_PER_PAGE - ITEMS_PER_PAGE ) and k <= ( self.page*ITEMS_PER_PAGE ) then
            box:Show()
        else
            box:Hide()
        end
    end
end

function MOTDContainer:NextPage()
    local pages = math.ceil( #self.boxes.children/ITEMS_PER_PAGE )
    self.page = self.page or 1
    self.page = self.page + 1
    if self.page > pages then self.page = 1 end
    self:SetPage( self.page )
end

function MOTDContainer:PrevPage()
    local pages = math.ceil( #self.boxes.children/ITEMS_PER_PAGE )
    self.page = self.page or pages
    self.page = self.page - 1
    if self.page < 1 then self.page = pages end
    self:SetPage( self.page )
end

function MOTDContainer:SetFocus(from_child)
    if not from_child and self.boxes.children[1] then
        self.boxes.children[1]:SetFocus()
    else
        MOTDContainer._base.SetFocus(self, from_child)
    end
end
function MOTDContainer:GetDefaultFocus( last_item_in_page )
    for k, box in ipairs( self.boxes.children ) do
        if last_item_in_page then
            if k == #self.boxes.children or k == self.page*ITEMS_PER_PAGE then return box end
        else
            if box:IsShown() then return box end
        end
    end
end

function MOTDContainer:SetFn( fn )
    -- To be called when a contained message is clicked
    self.fn = fn
    return self
end


function MOTDContainer:SetFocusedIndex( idx )
    if idx < 1 then idx = #self.boxes.children end
    if idx > #self.boxes.children then idx = 1 end
    for k, box in ipairs( self.boxes.children ) do
        if k == idx then box:SetFocus() end
    end
end

function MOTDContainer:GetFocusedIndex()
    for k, box in ipairs( self.boxes.children ) do
        if box:IsShown() and box.focus then return k end
    end
    return 1
end


function MOTDContainer:HandleControlDown( control, device )

    if control:Has( Controls.Digital.MENU_LEFT ) and self.page then
        local focused_idx = self:GetFocusedIndex()
        if focused_idx == self.page*ITEMS_PER_PAGE - ITEMS_PER_PAGE + 1 then
            self:PrevPage()
            self:GetDefaultFocus(true):SetFocus()
        else
            self:SetFocusedIndex( focused_idx-1 )
        end
        return true
    end

    if control:Has( Controls.Digital.MENU_RIGHT ) and self.page then
        local focused_idx = self:GetFocusedIndex()
        if focused_idx == #self.boxes.children or focused_idx == self.page*ITEMS_PER_PAGE then
            self:NextPage()
            self:GetDefaultFocus():SetFocus()
        else
            self:SetFocusedIndex( focused_idx+1 )
        end
        return true
    end

end

------------------------------------------------------------------------------------------
-- Shows a widget with a single message-of-the-day

local MOTDWidget = class( "Widget.MOTDWidget", Widget.Clickable )
MOTDWidget.can_focus = true

function MOTDWidget:init( box_data )
    MOTDWidget._base.init( self )

    self.content_w = 300*.9
    self.content_h = 170*.9

    local content_padding_w = 12
    local content_padding_h = 12
    local bg_border = 25
    local bg_border_scale = 0.5

    -- Background panel
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/motd_bg.tex") ) )
    self.bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.bg:SetNineSliceBorderScale( bg_border_scale )
    self.bg:Bloom( 0.1 )

    self.mask = self:AddChild( Widget() ):SetStencilContext( true )
    self.mask_image = Widget.Panel( engine.asset.Texture("UI/motd_image_mask.tex") )
    self.mask_image:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.mask_image:SetNineSliceBorderScale( bg_border_scale )
    self.image = Widget.Image():SetHiddenBoundingBox( true )
    self.mask:AddChildren
    {
        self.mask_image:SetMask(true),
        self.image:SetStencilTest(true)
    }

    -- This gradient separates the text from the image, so that it's readable
    local gradient_margin = 3
    self.gradient_overlay = self:AddChild( Widget.Panel( engine.asset.Texture("UI/motd_gradient.tex") ) )
    self.gradient_overlay:SetNineSliceCoords( 1, 0, 40, 50 )
    self.gradient_overlay:SetHiddenBoundingBox( true )
    self.gradient_overlay:SetTintColour( 0x021A18ff )
    self.gradient_overlay:SetTintAlpha( 0.9 )

    -- This frame is drawn over everything else, to hide the edge of the gradient
    self.image_overlay = self:AddChild( Widget.Panel( engine.asset.Texture("UI/motd_image_overlay.tex") ) )
    self.image_overlay:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.image_overlay:SetNineSliceBorderScale( bg_border_scale )
    self.image_overlay:Bloom( 0.1 )

    -- Text
    self.content = self:AddChild( Widget() )
    self.title = self.content:AddChild( Widget.Label("title", 22 )):LeftAlign()
    self.title:SetWordWrap( true )
    self.title:SetAutoSize( self.content_w - content_padding_w*2 )
    self.title:SetGlyphColour( UICOLOURS.SUBTITLE )
    self.title:SetText( LOC"UI.MAINMENU.INFO_TITLE" )
    self.text = self.content:AddChild( Widget.Label("body", 20 )):LeftAlign()
    self.text:OverrideLineHeight( 18 )
    self.text:SetWordWrap( true )
    self.text:SetAutoSize( self.content_w - content_padding_w*2 )
    self.text:SetGlyphColour( UICOLOURS.TITLE )

    -- Set contents
    local nineslice_w, nineslice_h = self.content_w + bg_border*2*bg_border_scale, self.content_h + bg_border*2*bg_border_scale
    self.bg:SetSize( nineslice_w, nineslice_h )
    self.mask_image:SetSize( nineslice_w, nineslice_h )
    self.image_overlay:SetSize( nineslice_w, nineslice_h )

    -- Set correct image, with the right size too
    self.image:SetTexture( box_data and box_data.texture or engine.asset.Texture("battle/dismantle.tex") )
    local img_w, img_h = self.image.texture:GetSize()
    -- Scale image so that it covers the entire width of the widget
    self.image:SetSize( self.content_w, img_h * self.content_w / img_w )
    -- If the image now doesn't cover the height of the widget, it's a wide image, so scale it to cover the height
    local scaled_w, scaled_h = self.image:GetSize()
    if scaled_h < self.content_h then
        self.image:SetSize( img_w * self.content_h / img_h, self.content_h )
    end

    -- Set text content
    self.title:SetText( box_data and box_data.title )
    self.text:SetText( box_data and box_data.subtitle )

    -- Layout everything
    self.text:LayoutBounds( "left", "bottom", self.bg ):Offset( content_padding_w, content_padding_h )
    self.title:LayoutBounds( "left", "above", self.text ):Offset( 0, 0 )
    local content_w, content_h = self.content:GetSize()
    self.gradient_overlay:SetSize( self.content_w - gradient_margin*2, content_h + 50 )
    self.gradient_overlay:LayoutBounds( "center", "bottom", self.bg ):Offset( 0, gradient_margin )

    --self:SetFocusable( false )

    
end

function MOTDWidget:OnGainFocus()
    MOTDWidget._base.OnGainFocus(self)
    self:UpdateImage()
end

function MOTDWidget:OnLoseFocus()
    MOTDWidget._base.OnLoseFocus(self)
    self:UpdateImage()
end

function MOTDWidget:OnGainHover()
    MOTDWidget._base.OnGainHover(self)
    self:UpdateImage()
end

function MOTDWidget:OnLoseHover()
    MOTDWidget._base.OnLoseHover(self)
    self:UpdateImage()
end

function MOTDWidget:UpdateImage()
    if self.enabled and ( self.hover or self.focus ) then
        self:ScaleTo( nil, 1.08, 0.2, easing.outQuad )
        self.image_overlay:Bloom( 0.4 )
    else
        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
        self.image_overlay:Bloom( 0.1 )
    end
end

function MOTDWidget:AddShine()
    -- Animated reflection shine on the slot
    if not self.shine_coro then
        self.shine = self.mask:AddChild( Widget.Image( engine.asset.Texture( "UI/argument_bg_shine.tex") ) )
            :SetStencilTest( true )
            :SetHiddenBoundingBox( true )
            :SetSize( self.content_w, self.content_w*2 )
            :SetTintColour( UICOLOURS.SUBTITLE )
            :SetTintAlpha( 0.2 )
            :Bloom( 0.4 )
            :SetPos( 0, 400 )

        self.shine_coro = self:StartCoroutine( self.ShowShine, self )
    end

    -- The hit animation
    self.shine_anim = Widget.Anim()
    local animname = "anim/UI_front_page_highlight_fx.zip"
    self.shine_anim:AddAnims( GetAnim( animname ) )
    self.shine_anim:AddOverrideFile( animname )

    self:AddChild( self.shine_anim )
        :SetHiddenBoundingBox(true)
        :SetPos( 0, 0 )
        :SetScale( 0.515 )
        :SetBloom( 0.2 )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.7 )
end

function MOTDWidget:OnVizChange(viz)
    if viz then    
        self:AddShine()
    else
        self.shine_coro:Stop()
        self.shine_coro = nil
        if self.shine_anim and not self.shine_anim.removed then
            self.shine_anim:Remove()
            self.shine_anim = nil
        end
    end
    return self
end


function MOTDWidget:ShowShine()
    local anim_pacing = 2.6

    while true do
        self:Delay( 1 + math.random() * 5.0 )

        self.shine_anim:Play( "1", false )

        self.shine:Show()

        self.shine:SetPos( 0, 400 )
            :MoveTo( 0, -400, anim_pacing, easing.outQuad )

        self:Delay( anim_pacing )
        self.shine:Hide()
    end
end

------------------------------------------------------------------------------------------
-- Shows a popup with a single message-of-the-day, and maybe action buttons

local MOTDPopup = class( "Screen.MOTDPopup", Widget.Screen )

MOTDPopup.CONTROL_MAP = Widget.MergeControlMap(Widget.Screen.CONTROL_MAP,
{
    {
        control = Controls.Digital.MENU_CANCEL, 
        
        fn = function(self)
            self:Close()
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.BACK", Controls.Digital.MENU_CANCEL))
        end
    },

})

local bg_border = 25
local bg_border_scale = 0.5
local text_inset = 15

function MOTDPopup:init( box_data )
    MOTDPopup._base.init( self )
    self.is_popup = true
    self:SetAnchors( "center", "center" )

    self.content_w = 800
    self.scroll_w = self.content_w + 40
    self.content_h = 700

    -- A dark overlay blocking interaction with the background
    self.scrim = self:AddChild( Widget.Clickable() ):SendToBack()
    self.scrim_bg = self.scrim:AddChild( Widget.SolidBox( RES_X, RES_Y, 0x021A18F8 ) ):BlockMouse()
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    self.scrim_bg:SetSize( self.screen_w, self.screen_h )
    self.scrim:SetOnClickFn( function() self:Close() end )

    -- A container for all the background elements
    self.panel = self:AddChild( Widget.WidePanel(self.content_h) )
    self.panel:AddSubButton( LOC"UI.MAINMENU.CLOSE", global_images.close, function() self:Close() end )

    -- Contains everything inside the panel
    self.scroll = self:AddChild( Widget.ScrollPanel() )
        :SetSize( self.screen_w, self.content_h )
        :SetVirtualMargin( 15 )
        :ShowBar( SCROLLBAR.NEVER )
    self.content = self.scroll:AddScrollChild( Widget() )
    self.mask = self.content:AddChild( Widget() ):SetStencilContext( true )
    self.mask_image = Widget.Panel( engine.asset.Texture("UI/motd_image_mask.tex") )
    self.mask_image:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.mask_image:SetNineSliceBorderScale( bg_border_scale )
    self.image = Widget.Image( ):SetHiddenBoundingBox( true )
    self.mask:AddChildren
    {
        self.mask_image:SetMask(true),
        self.image:SetStencilTest(true)
    }
    self.image_overlay = self.content:AddChild( Widget.Panel( engine.asset.Texture("UI/motd_image_overlay.tex") ) )
    self.image_overlay:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.image_overlay:SetNineSliceBorderScale( bg_border_scale )
    self.image_overlay:Bloom( 0.1 )
    self.title = self.content:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_SUBTITLE ) )
    self.title:SetGlyphColour( UICOLOURS.SUBTITLE ):LeftAlign()
    self.title:SetWordWrap( true ):SetAutoSize( self.content_w - text_inset*2 ):Bloom( 0.1 )
    self.text = self.content:AddChild( Widget.Label("body", FONT_SIZE.SCREEN_TABS ) )
    self.text:SetGlyphColour( UICOLOURS.BODYTEXT ):LeftAlign()
    self.text:SetWordWrap( true ):SetAutoSize( self.content_w - text_inset*2 ):Bloom( 0.05 )

    self:Refresh( box_data )
end

function MOTDPopup:Refresh( box_data )
    self.box_data = box_data

    -- Set correct image, with the right size too
    if box_data.hide_img then
        local img_w, img_h = self.content_w, 1
        self.image:SetSize( img_w, img_h ):Hide()
        self.mask_image:SetSize( img_w, img_h ):Hide()
        self.image_overlay:SetSize( img_w, img_h ):Hide()
    else
        self.image:SetTexture( box_data and box_data.texture or engine.asset.Texture("battle/dismantle.tex") )
        local img_w, img_h = self.image.texture:GetSize()
        -- Scale image so that it covers the entire width of the widget
        img_h = img_h * self.content_w / img_w
        img_w = self.content_w
        self.image:SetSize( self.content_w, img_h ):Show()
        local nineslice_w, nineslice_h = img_w + bg_border*2*bg_border_scale, img_h + bg_border*2*bg_border_scale
        self.mask_image:SetSize( nineslice_w, nineslice_h ):Show()
        self.image_overlay:SetSize( nineslice_w, nineslice_h ):Show()
    end

    -- Set text content
    self.title:SetText( box_data and box_data.title or LOC"UI.MAINMENU.INFO_TITLE" )
        :LayoutBounds( "left", "below", self.image_overlay ):Offset( 0, -SPACING.M1 )
    self.text:SetText( box_data and box_data.text or LOC"UI.MAINMENU.INFO_TEXT" )
        :LayoutBounds( "left", "below", self.title ):Offset( 0, SPACING.TITLE_DESC )

    -- Layout everything
    local scroll_w, scroll_h = self.scroll:GetSize()
    local content_w, content_h = self.content:GetSize()
    self.content:LayoutBounds( "left", "top" ):Offset( (scroll_w-content_w)/2, 0 )

    -- Add new buttons
    if box_data.link and box_data.link ~= '' then
        self.panel:AddSubButton( box_data.link_text and box_data.link_text or LOC"UI.MAINMENU.MORE_INFO", global_images.accept, function() engine.inst:OpenUrlInBrowser( box_data.link ) end )
    end
    if box_data.link2 and box_data.link2 ~= '' then
        self.panel:AddSubButton( box_data.link2_text and box_data.link2_text or LOC"UI.MAINMENU.MORE_INFO", global_images.accept, function() engine.inst:OpenUrlInBrowser( box_data.link2 ) end )
    end
end

function MOTDPopup:SetFn( fn )
    -- To be called when a contained message is clicked
    self.fn = fn
    return self
end

function MOTDPopup:OnScreenModeChange( sm )
    MOTDPopup._base.OnScreenModeChange( self, sm )

    self.title:SetFontSize( FONT_SIZE.SCREEN_SUBTITLE*LAYOUT_SCALE[sm] )
    self.text:SetFontSize( FONT_SIZE.SCREEN_TABS*LAYOUT_SCALE[sm] )

    self:Refresh( self.box_data )
end

function MOTDPopup:Close()
    self:AlphaTo( 0, 0.1, easing.linear, function() self.fe:PopScreen(self) end )
end

------------------------------------------------------------------------------------------
-- Shows a popup with a list of messages-of-the-day

local MOTDListPopup = class( "Screen.MOTDListPopup", Widget.Screen )

MOTDListPopup.CONTROL_MAP = Widget.MergeControlMap(Widget.Screen.CONTROL_MAP,
{
    {
        control = Controls.Digital.MENU_CANCEL, 
        
        fn = function(self)
            self:Close()
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.BACK", Controls.Digital.MENU_CANCEL))
        end
    },

})

local bg_border = 25
local bg_border_scale = 0.5
local text_inset = 15

function MOTDListPopup:init( motd_list, update_string )
    MOTDListPopup._base.init( self )
    self.is_popup = true
    self:SetAnchors( "center", "center" )

    self.content_w = 800
    self.scroll_w = self.content_w + 40
    self.content_h = 700

    -- A dark overlay blocking interaction with the background
    self.scrim = self:AddChild( Widget.Clickable() ):SendToBack()
    self.scrim_bg = self.scrim:AddChild( Widget.SolidBox( RES_X, RES_Y, 0x021A18F8 ) ):BlockMouse()
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    self.scrim_bg:SetSize( self.screen_w, self.screen_h )
    self.scrim:SetOnClickFn( function() self:Close() end )

    -- A container for all the background elements
    self.panel = self:AddChild( Widget.WidePanel(self.content_h) )
    self.panel:AddSubButton( LOC"UI.MAINMENU.CLOSE", global_images.close, function() self:Close() end )

    -- Contains everything inside the panel
    self.scroll = self:AddChild( Widget.ScrollPanel() )
        :SetSize( self.screen_w, self.content_h )
        :SetVirtualMargin( 15 )
        :ShowBar( SCROLLBAR.NEVER )
    self.content = self.scroll:AddScrollChild( Widget() )
    self.title = self.content:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_SUBTITLE, LOC"UI.MAINMENU.MOTD_LIST_POPUP_TITLE" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE ):LeftAlign():Bloom( 0.1 )
    self.subtitle = self.content:AddChild( Widget.Label("body", FONT_SIZE.SCREEN_TABS, LOC"UI.MAINMENU.MOTD_LIST_POPUP_SUBTITLE" ) )
        :SetGlyphColour( UICOLOURS.BODYTEXT ):LeftAlign():Bloom( 0.05 )
    self.boxes = self.content:AddChild( Widget() )

    self:Refresh( motd_list, update_string )
end

function MOTDListPopup:Refresh( box_data, update_string )
    self.box_data = box_data

    -- Clear any old messages
    self.boxes:DestroyAllChildren()

    -- Add brand new ones
    local box_w, box_h = 0, 0
    for k, message in pairs( self.box_data ) do
        local box = self.boxes:AddChild( Widget.MOTDWidget( message ) )
        box:SetLayoutScale( LAYOUT_SCALE[TheGame:FE():GetScreenMode()] )
        box:SetTintAlpha( 0 )
        box_w, box_h = box:GetSize()
        box:SetOnClickFn( function() if self.fn then self.fn( message ) end end )
    end

    -- Align everything
    self.title:LayoutBounds( "left", "top" )
    if update_string then self.subtitle:SetText( update_string ) end
    self.subtitle:LayoutBounds( "left", "below", self.title ):Offset( 0, SPACING.TITLE_SUBTITLE )
    self.boxes:LayoutChildrenInGrid( 3, 20 )
    self.boxes:LayoutBounds( "left", "below", self.subtitle ):Offset( 0, -SPACING.M1 )
    local scroll_w, scroll_h = self.scroll:GetSize()
    local content_w, content_h = self.content:GetSize()
    self.content:LayoutBounds( "left", "top" ):Offset( (scroll_w-content_w)/2, 0 )

    -- Animate in the boxes
    local delay_per_box = 0.05
    local timing = 0.1
    for k = 1, math.max( ITEMS_PER_PAGE, #self.boxes.children ) do
        if k <= #self.boxes.children then
        local box = self.boxes.children[k]
            box:RunUpdater(
                Updater.Series{
                    Updater.Wait( delay_per_box*k ),
                    Updater.Parallel{
                        Updater.Ease( function(v) box:SetTintAlpha(v) end, 0, 1, timing, easing.linear),
                        Updater.Ease( function(v) box:SetScale(v,v) end, 1.1, 1, timing, easing.linear)
                    }
                }
            )
        end
    end
end

function MOTDListPopup:OnGainTop()
    MOTDListPopup._base.OnGainTop( self )

    if TheGame:FE():GetControlMode() == CONTROL_MODE.GAMEPAD then
        local focus = self:GetDefaultFocus()
        if focus then focus:SetFocus() end
    end
end

function MOTDListPopup:SetFn( fn )
    -- To be called when a contained message is clicked
    self.fn = fn
    return self
end

function MOTDListPopup:GetDefaultFocus()
    return self.boxes.children and #self.boxes.children>0 and self.boxes.children[1]
end

function MOTDListPopup:OnControlModeChange( cm, device_id )
    MOTDListPopup._base.OnControlModeChange( self, cm, device_id )
end

function MOTDListPopup:OnScreenModeChange( sm )
    MOTDListPopup._base.OnScreenModeChange( self, sm )

    self.title:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.subtitle:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:Refresh( self.box_data )
end

function MOTDListPopup:Close()
    self:AlphaTo( 0, 0.1, easing.linear, function() self.fe:PopScreen(self) end )
end


------------------------------------------------------------------------------------------
-- Shows a menu button with just an icon before the single-line text (options, quit, etc)

local AdvancedMenuButton = class( "Widget.AdvancedMenuButton", Widget.Clickable )

function AdvancedMenuButton:init( txt, fn, opt )
    AdvancedMenuButton._base.init( self )

    -- Set sizes
    self.width = 260
    self.height = 40
    self.padding = 8
    opt = opt or {}

    -- A hitbox for alignment
    self.hitbox = self:AddChild( Widget.SolidBox( self.width, self.height, 0xffff0000 ) )

    -- Background
    self.bg_border = 33
    self.bg_border_scale = 0.5
    self.bg_gradient_w = 200
    self.bg_gradient = self:AddChild( Widget.Panel( engine.asset.Texture("UI/menubutton_gradient.tex") ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.5 )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_gradient_w-self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
    self.bg_outline = self:AddChild( Widget.Panel( engine.asset.Texture("UI/menubutton_outline.tex") ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :Bloom( 0.1 )
    -- Icon
    self.image = self:AddChild( Widget.Image( engine.asset.Texture("UI/btn_normal_hover.tex") ) )
        :SetHiddenBoundingBox( true )
        :SetBloom( 0.03 )
    -- Animation
    if opt.cta then self:SetCta( true ) end

    -- Text
    self.text_container = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.label = self.text_container:AddChild(Widget.Label( "button", FONT_SIZE.BUTTON ) )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetWordWrap( true )
        :OverrideLineHeight( FONT_SIZE.BUTTON*LINE_HEIGHT_TEXT_MULT*0.9 )

    self:DisableAnimateScales()
    self:LeftAlign()
    self:SetText( txt )
    
    self:Layout()
    self:SetOnClickFn( fn )
end

function AdvancedMenuButton:SetIcon( icon, colour )
    self.image:SetTexture( icon )
    if colour then self.image:SetTintColour( colour ) end
    self:Layout()
    return self
end

function AdvancedMenuButton:SetCta( is_cta )
    if is_cta and not self.mask and TheGame:GetLevelOfDetail() > 1 then
        self.mask = self:AddChild( Widget() )
            :SetStencilContext( true )
            :SetHiddenBoundingBox( true )
            :SendToBack()
        self.mask_image = self.mask:AddChild( Widget.Panel( engine.asset.Texture("UI/menubutton_mask.tex") ) )
            :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
            :SetNineSliceBorderScale( self.bg_border_scale )
            :SetMask( true )
        -- self.mask_anim = self.mask:AddChild( Widget.Image( engine.asset.Texture("battle/accelerant.tex") ) )
        local animname = "anim/UI_button_decor_fx.zip"
        self.mask_anim = self.mask:AddChild( Widget.Anim() )
            :AddAnims( GetAnim( animname ) )
            :AddOverrideFile( animname )
            :SetTintColour( UICOLOURS.TITLE )
            :SetScale( 0.9 )
            :SetBloom( 0.5 )
            :SetTintAlpha( 0.5 )
            :Play( "loop", true )
            :SetPhase( math.random() )
            :SetPos( 65, -10 )
            :SetRotation( 40*DEGREES )
            :SetStencilTest( true )
    end
    return self
end

function AdvancedMenuButton:SetSize( w, h )
    -- Set sizes
    self.width = w or self.width or 260
    self.height = h or self.height or 40
    self:Layout()
    return self
end

function AdvancedMenuButton:SetText( text )
    self.label:SetText( text )
    self:Layout()
    return self
end

function AdvancedMenuButton:SetColour( colour )
    self.bg_gradient:SetTintColour( colour )
        :SetTintAlpha( 0.5 )
    self.bg_outline:SetTintColour( colour )
    if self.mask_anim then
        self.mask_anim:SetTintColour( colour )
            :SetTintAlpha( 0.5 )
    end
    return self
end

function AdvancedMenuButton:SetTextSize( text_size )
    return self
end

function AdvancedMenuButton:LeftAlign()
    return self
end

function AdvancedMenuButton:RightAlign()
    return self
end

function AdvancedMenuButton:Layout()
    self.hitbox:SetSize( self.width, self.height )
    local nineslice_w, nineslice_h = self.width - self.bg_border*2*self.bg_border_scale, self.height - self.bg_border*2*self.bg_border_scale
    self.bg_outline:SetInnerSize( nineslice_w, nineslice_h )
    self.bg_gradient:SetInnerSize( nineslice_w, nineslice_h )
    if self.mask_image then self.mask_image:SetInnerSize( nineslice_w, nineslice_h ) end
    local image_size = self.height-self.padding*2
    self.image:SetSize( image_size, image_size )
    self.label:SetAutoSize( self.width - self.padding*2 - image_size )

    self.image:LayoutBounds( "left", "center", self.hitbox )
        :Offset( self.padding, 0 )
    self.text_container:LayoutBounds( "after", "center", self.image )
        :Offset( self.padding, 1 )
    return self
end

function AdvancedMenuButton:UpdateImage()
    if self.hover or self.focus then
        self:ScaleTo( nil, 1.03, 0.2, easing.outQuad )
        self.bg_outline:Bloom( 0.2 )
        self.bg_gradient:AlphaTo( 0.65, 0.1, easing.outQuad )
        if self.hover and self.focus then
            self.bg_outline:Bloom( 0.23 )
        end
    else
        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
        self.bg_outline:Bloom( 0.1 )
        self.bg_gradient:AlphaTo( 0.5, 0.2, easing.outQuad )
    end
end

function AdvancedMenuButton:OnGainFocus()
    AdvancedMenuButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function AdvancedMenuButton:OnLoseFocus()
    AdvancedMenuButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function AdvancedMenuButton:OnGainHover()
    AdvancedMenuButton._base.OnGainHover(self)
    self:UpdateImage()
end

function AdvancedMenuButton:OnLoseHover()
    AdvancedMenuButton._base.OnLoseHover(self)
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- Shows a menu button for Play with two lines of text

local PlayMenuButton = class( "Widget.PlayMenuButton", Widget.AdvancedMenuButton )

function PlayMenuButton:init( txt, fn, opt )
    PlayMenuButton._base.init( self, txt, fn, opt )

    -- Set sizes
    self.width = 360
    self.height = 54

    -- Hide the line
    self.image:Hide()

    -- Update colour
    self.bg_gradient:SetTintColour( UICOLOURS.OBJECTIVE )
        :SetTintAlpha( 0.5 )
    self.bg_outline:SetTintColour( UICOLOURS.OBJECTIVE )

    -- Add second line of text
    self.subtitle = self.text_container:AddChild(Widget.Label( "body", FONT_SIZE.SMALL_TEXT * 0.7 ) )
        :SetGlyphColour( 0x6BFFD2ff )
        :SetWordWrap( true )
        :OverrideLineHeight( FONT_SIZE.BUTTON*LINE_HEIGHT_TEXT_MULT )
end

function PlayMenuButton:OnAdded()
    local screen = self:GetScreen()
    if (screen.save_count or 0) > 0 then
        self.subtitle:SetText( loc.format( LOC"UI.MAINMENU.PLAY_SAVEFILES", screen.save_count )):Show()
    else
        self.subtitle:Hide()
    end
    self:Layout()
end

function PlayMenuButton:SetSubtitle( txt )
    self.subtitle:SetText( txt )
end

function PlayMenuButton:Layout()
    if not self.subtitle then return self end
    self.hitbox:SetSize( self.width, self.height )
    local nineslice_w, nineslice_h = self.width - self.bg_border*2*self.bg_border_scale, self.height - self.bg_border*2*self.bg_border_scale
    self.bg_outline:SetInnerSize( nineslice_w, nineslice_h )
    self.bg_gradient:SetInnerSize( nineslice_w, nineslice_h )
    if self.mask_image then self.mask_image:SetInnerSize( nineslice_w, nineslice_h ) end
    self.label:SetAutoSize( self.width - self.padding*2 )
    self.subtitle:SetAutoSize( self.width - self.padding*2 )

    self.subtitle:LayoutBounds( "left", "below", self.label )
        :Offset( 0, 3 )
    self.text_container:LayoutBounds( "left", "center", self.hitbox )
        :Offset( self.padding*2, 1 )
    return self
end

------------------------------------------------------------------------------------------
-- Shows a menu button for Continue with two lines of text

local ContinueMenuButton = class( "Widget.ContinueMenuButton", Widget.PlayMenuButton )

function ContinueMenuButton:init( txt, fn, opt )
    ContinueMenuButton._base.init( self, txt, fn, opt )

    -- Update colour
    self.bg_gradient:SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.5 )
    self.bg_outline:SetTintColour( UICOLOURS.SUBTITLE )

    -- Check if the save file is present
    if opt.save_file then
        TheGame:LoadSaveHeader( opt.save_file, function( header )
            if header then
                local act_data = GetPlayerActData( header.options.start_act )
                local act_name = act_data:GetLocalizedName()
                local def = Content.GetCharacterDef( act_data.background.data.player_agent )
                local agent_name = def:GetLocalizedName()
                self.subtitle:SetText( loc.format( LOC"UI.MAINMENU.CONTINUE_SAVEFILE", agent_name, act_name, header.desc, header.progress ) )
            end
        end )
    end

    self:Layout()
end

function ContinueMenuButton:OnAdded()
    self:Layout()
end

------------------------------------------------------------------------------------------
-- Shows a menu button for the Daily challenge

local DailyChallengeMenuButton = class( "Widget.DailyChallengeMenuButton", Widget.PlayMenuButton )

function DailyChallengeMenuButton:init( txt, fn, opt )
    DailyChallengeMenuButton._base.init( self, txt, fn, opt )

    self.width = 360
    self.height = 90

    -- Update colour
    self.bg_gradient:SetTintColour( UICOLOURS.DAILY )
        :SetTintAlpha( 0.5 )
    self.bg_outline:SetTintColour( UICOLOURS.DAILY )
    self.subtitle:SetGlyphColour( UICOLOURS.DAILY_LIGHT )

    self.image:Show()
        :SetHiddenBoundingBox( true )
    self.label:SetText( txt )

    -- Add the time left info
    self.expiry_info = self:AddChild(Widget.Label( "title", FONT_SIZE.SMALL_TEXT * 0.65 ) )
        :SetGlyphColour( UICOLOURS.DAILY_LIGHT )
        :SetWordWrap( true )
        :OverrideLineHeight( FONT_SIZE.BUTTON*LINE_HEIGHT_TEXT_MULT )
        :SetText("8 hours and 32 minutes left")
        :SetHiddenBoundingBox( true )

    -- Get Daily Challenge info
    self:Refresh()

    -- Update the time left every 15 seconds
    self:DoPeriodicTask("update_daily_challenge_button", 15, 15, function()
        self:Refresh()
    end )
end

function DailyChallengeMenuButton:Refresh()
    local daily = require "daily_util"
    local daily_data = daily.GenerateDaily()
    if daily_data then
        self.subtitle:SetText( loc.format( LOC"UI.MAINMENU.DAILY_SUBTITLE", daily_data.def:GetTitle() ) )
        self.expiry_info:SetText( Screen.DailyChallengeScreen.GetRemainingTimeString() )
        local daily_data_agent_images = {
            SAL = engine.asset.Texture("large/daily_sal.tex"),
            ROOK = engine.asset.Texture("large/daily_rook.tex"),
            SMITH = engine.asset.Texture("large/daily_smith.tex")
        }
        self.image:SetTexture( daily_data_agent_images[daily_data.character or "SMITH"] )
    end
    self:Layout()

    return self
end

function DailyChallengeMenuButton:Enable()
    self._base.Enable(self)
    self:SetSaturation(1)
end
function DailyChallengeMenuButton:Disable()
    self._base.Disable(self)
    self:SetSaturation(0)
end


function DailyChallengeMenuButton:OnAdded()
end

function DailyChallengeMenuButton:SetCta( is_cta )
    DailyChallengeMenuButton._base.SetCta( self, is_cta )
    if self.mask_anim then
        self.mask_anim:SetTintColour( UICOLOURS.DAILY )
            :SetTintAlpha( 0.8 )
    end
    return self
end

function DailyChallengeMenuButton:Layout()
    DailyChallengeMenuButton._base.Layout( self )
    if not self.expiry_info then return self end

    -- Size up and position the image
    self.image:ResetSize()
    local target_h = self.height + 25
    local i_w, i_h = self.image:GetSize();
    local target_w = target_h/i_h*i_w; -- Scale the image to match the height, mantaining aspect ratio
    self.image:SetSize( target_w, target_h )
        :LayoutBounds( "right", "bottom", self.hitbox )
        :Offset( -1.5, 1.5 )

    self.expiry_info:LayoutBounds( "left", "above", self.hitbox )
        :Offset( 0, 2 )
    self.text_container:LayoutBounds( nil, "bottom", self.hitbox )
        :Offset( nil, 8 )
    return self
end

------------------------------------------------------------------------------------------
-- Shows a tutorial button

local TutorialsButton = class( "Widget.TutorialsButton", Widget.AdvancedMenuButton )

function TutorialsButton:init( txt, fn, opt )
    TutorialsButton._base.init( self, txt, fn, opt )

    -- Set sizes
    self.width = 330
    self.height = 54

    -- Update colour
    self.bg_gradient:SetTintColour( opt.colour or UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.5 )
    self.bg_outline:SetTintColour( opt.colour or UICOLOURS.SUBTITLE )

    self:Layout()
end

function TutorialsButton:Layout()
    TutorialsButton._base.Layout( self )
    
    -- Size up and position the image
    self.image:ResetSize()
    local target_h = self.height + 10
    local i_w, i_h = self.image:GetSize();
    local target_w = target_h/i_h*i_w; -- Scale the image to match the height, mantaining aspect ratio
    self.image:SetSize( target_w, target_h )
        :LayoutBounds( "right", "bottom", self.hitbox )
        :Offset( -1.5, 1.5 )

    self.text_container:LayoutBounds( "left", "center", self.hitbox )
        :Offset( self.padding*2, 1 )

    return self
end

------------------------------------------------------------------------------------------
-- Shows an experiment button

local ExperimentsButton = class( "Widget.ExperimentsButton", Widget.AdvancedMenuButton )

function ExperimentsButton:init( txt, fn, opt )
    ExperimentsButton._base.init( self, txt, fn, opt )

    -- Set size
    self.width = 330

    self:Layout()
end