require "ui/screens/travelscreen"
require "ui/screens/pausemenu"
require "ui/screens/deckscreen"
require "ui/screens/locationscreen"
require "ui/screens/conversationscreen"
require "ui/screens/cardshopscreen"
require "ui/screens/dailyoverscreen"
require "ui/screens/graftcollectionscreen"
require "ui/screens/boonsbanesscreen"
require "ui/screens/coinscreen"
require "ui/screens/blasterscreen"
require "ui/screens/choosecardspopup"
require "ui/widgets/tooltip_graft"
require "ui/widgets/cardarrangement"
require "ui/widgets/pointsfloater"
require "ui/widgets/feats"
require "ui/widgets/dailyprogress"

require "ui/widgets/to_do_list"
require "ui/widgets/upgradeindicator"
require "ui/widgets/crewportraits"
require "ui/widgets/progressindicator"
require "ui/widgets/healthindicator"
require "ui/widgets/resolveindicator"
require "ui/widgets/mettleindicator"
require "ui/widgets/prestigeindicator"
require "ui/widgets/fightlocationstatus"
require "ui/widgets/clickable"
require "ui/screens/jobpopup"
require "ui/screens/relationshipsscreen"
require "ui/screens/mutatorsscreen"
require "ui/screens/perkscreen"
require "ui/screens/mettlescreen"
require "ui/screens/flourishscreen"

local negotiation_defs = require "negotiation/negotiation_defs"
local battle_defs = require "battle/battle_defs"

-------------------------------------------------------------------------------------
-- The main overlay superimposed over most UI, allowing access to common things like 
-- the player's caravan, system menu, resource readout, etc.

local MARGIN = DEFAULT_SCREEN_MARGIN
OVERLAY_MODE = MakeEnum{ "MAP", "PLAX", "INFO", "POPUP", "TRAVEL", "BATTLE", "NEGOTIATION", "CONVO", "RELATIONSHIP", "DECK" }
local TRACK_ITEM_PADDING = 6
local ITEM_LABEL_MAXW = 256
local ITEM_ICON_SZ = 24
local BTN_SPACING = 10

local NAV_SCREENS = { Screen.FeedbackScreen, Screen.NegotiationDeck, Screen.BattleDeck, Screen.GraftCollectionScreen, Screen.CoinScreen, Screen.BlasterScreen, Screen.MutatorsScreen, Screen.RelationshipScreen, Screen.BoonsBanesScreen, Screen.PauseMenu, Screen.PerkScreen }

local OVERLAY_MODES =
{
    [ Screen.LocationScreen ] = OVERLAY_MODE.PLAX,
    [ Screen.DeckScreen ] = OVERLAY_MODE.DECK,
    [ Screen.NegotiationDeck ] = OVERLAY_MODE.DECK,
    [ Screen.BattleDeck ] = OVERLAY_MODE.DECK,
    [ Screen.DrawDeckScreen ] = OVERLAY_MODE.DECK,
    [ Screen.DiscardDeckScreen ] = OVERLAY_MODE.DECK,
    [ Screen.ConversationScreen ] = OVERLAY_MODE.CONVO,
    [ Screen.FightScreen ] = OVERLAY_MODE.BATTLE,
    [ Screen.CardShopScreen ] = OVERLAY_MODE.PLAX,
    [ Screen.PickGraftScreen ] = OVERLAY_MODE.TRAVEL,
    [ Screen.LootScreen ] = OVERLAY_MODE.CONVO,
    [ Screen.AcceptJobPopup ] = OVERLAY_MODE.POPUP,
    [ Screen.ChooseCardsPopup ] = OVERLAY_MODE.BATTLE,
    [ Screen.PauseMenu ] = OVERLAY_MODE.POPUP,
    [ Screen.TravelScreen ] = OVERLAY_MODE.TRAVEL,
    [ Screen.RelationshipScreen ] = OVERLAY_MODE.RELATIONSHIP,
}

local MainOverlay = class("Screen.MainOverlay", Widget.Screen)

MainOverlay.CONTROL_MAP = 
{

    {
        control = Controls.Digital.OPEN_OVERLAY_MENU, 
        fn = function(self)
            if self:TopBarHasFocus() then
                local top_screen = self:GetFE():GetTopScreen()
                if top_screen then
                    local default_widget = top_screen:GetDefaultFocus()
                    if default_widget then
                        default_widget:SetFocus()
                    else
                        self:GetFE():ClearFocusWidget()
                    end
                    return true
                end
            else
                local default_widget = self.top_bar:GetDefaultFocus()
                if default_widget then default_widget:SetFocus() end
            end
        end,
    },

    {
        control = Controls.Digital.MENU_CANCEL, 
        fn = function(self)
            local top_screen = self:GetFE():GetTopScreen()
            if top_screen then
                local default_widget = top_screen:GetDefaultFocus()
                if default_widget then
                    default_widget:SetFocus()
                else
                    self:GetFE():ClearFocusWidget()
                end
                return true
            end
        end,
        test = function(self)
            return self:TopBarHasFocus()
        end,

        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end,
    },

    {
        control = Controls.Digital.TOGGLE_CONTRACTS, 
        fn = function(self)
            self:ToggleQuests()
        end,
    },

    {
        hint = function(self, left, right)
            if self:TopBarHasFocus() then
                table.insert(left, loc.format(LOC"UI.CONTROLS.NAV"))
            end
        end,
    }
}


function MainOverlay:init( )
    MainOverlay._base.init(self)

    self.is_overlay = true
    self.overlay_mode = OVERLAY_MODE.MAP
    self.showing_quests = false

    -- All the UI elements
    self.interface = self:AddChild( Widget() )

    -- A backing panel shown when the MainOverlay is in focus
    -- self.scrim = self.interface:AddChild( Widget.SolidBox( RES_X, RES_Y, 0xFF000040 ) )
    --     :SetAnchors( "fill", "fill" )
    --     :SetHiddenBoundingBox( true )

    -- Top left corner
    self.top_left = self.interface:AddChild( Widget() )
        :SetAnchors("left", "top")
    self.top_left_shadow = self.top_left:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
        :SetHiddenBoundingBox( true )
        :SetSize( 1000, 500 )
        :SetTintColour( 0x4D254Bff )
    self.top_left_content = self.top_left:AddChild( Widget() )
    self.battle_friend_reserves = self.top_left_content:AddChild( Widget.BattleReserves( true ) )

    -- Top right corner
    self.top_right = self.interface:AddChild( Widget() )
        :SetAnchors("right", "top")
    self.top_right_shadow = self.top_right:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
        :SetHiddenBoundingBox( true )
        :SetSize( 1000, 500 )
        :SetTintColour( 0x4D254Bff )
    self.top_right_content = self.top_right:AddChild( Widget() )
    self.to_do_list = self.top_right_content:AddChild( Widget.ToDoList() )
        :SetOnChangeFn( function() self:OnToDoListChanged() end )
        :ShowToolTipOnFocus()
        :SetFocusable( true )
    self.battle_enemy_reserves = self.top_right_content:AddChild( Widget.BattleReserves( false ) )

    -- Top bar
    self.top_bar = self.interface:AddChild( Widget.MainOverlayTopBar(self) )
        :SetAnchors( "center", "top" )
        :RefreshSize()
        :SetFocusLock(true)

    -- Bottom center spot: convo options
    self.bottom_center = self.interface:AddChild( Widget() )
        :SetAnchors("center", "bottom")
    self.bottom_center_inner = self.bottom_center:AddChild( Widget() )

    -- Center of the screen
    self.center = self.interface:AddChild( Widget() )
        :SetAnchors("center", "center")

    -- Bottom left: portraits
    self.bottom_left = self.interface:AddChild( Widget() )
        :SetAnchors("left", "bottom")
    self.bottom_left_inner = self.bottom_left:AddChild( Widget() )

    -- Notification widget
    self.bottom_left_notification = self.interface:AddChild( Widget.InlineNotification() )

    -- Notification widget
    self.top_right_quests_toggle = self.interface:AddChild( Widget.MainOverlayQuestButton( function() return self:ToggleQuests() end ) )
        :SetAnchors("right", "top")
        :Bloom( 0.05 )

    self.bars_shown = true
    self.top_bar_shown = true
    self.top_left_shown = true
    self.top_right_shown = true
    self.quests_toggled = false

    self:LayoutTopLeft()
    self:LayoutTopRight()
end

function MainOverlay:OnAdded()
    TheGame:GetEvents():ListenForAny( self )
    self.top_bar:RefreshListeners()
    self:SetMapMode()

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
    self:RefreshOverlayMode()
end

function MainOverlay:OnRemoved()
    TheGame:GetEvents():RemoveListener( self )
    self.top_bar:RemoveListeners()
end

function MainOverlay:OnUpdate(dt)
    MainOverlay._base.OnUpdate(self, dt)
    if self.quest_show_timer then
        self.quest_show_timer = self.quest_show_timer - dt
        if self.quest_show_timer <= 0 then
            self.quest_show_timer = nil
            self:RefreshToDoListVisibility()
        end
    end
end

function MainOverlay:SetModeOverride( overlay_mode )
    self.overlay_mode_override = overlay_mode
    self:RefreshOverlayMode()
end

function MainOverlay:ToggleHUD( visible, hud_flags )
    self:RefreshOverlayMode()
    self.top_bar:SetShown(visible)
    self.bottom_left_notification:SetShown( visible )
    self.top_right_quests_toggle:SetShown( visible )
end

function MainOverlay:OnGainTop( top_screen )
    MainOverlay._base.OnGainTop( self )

    self.top_bar:RefreshNavButtons()
    self:RefreshOverlayMode()
    self.bottom_left_notification:RefreshStreamerMode()

    if top_screen and not table.contains( NAV_SCREENS, top_screen._class ) then
        TheGame:GetMusic():StopNavMusic()
    end
end

function MainOverlay:OnControlModeChange( cm, device_id )
    MainOverlay._base.OnControlModeChange( self, cm, device_id )
    
    if cm == CONTROL_MODE.GAMEPAD then
        self.top_right_quests_toggle:Hide()
    else
        self.top_right_quests_toggle:Show()
    end
end

function MainOverlay:OnScreenModeChange( sm )
    MainOverlay._base.OnScreenModeChange( self, sm )

    self.top_left:SetLayoutScale( (LAYOUT_SCALE[sm]+1)/2 )
    self.top_right:SetLayoutScale( (LAYOUT_SCALE[sm]+1)/2 )
    self.top_right_quests_toggle:SetLayoutScale( (LAYOUT_SCALE[sm]+1)/2 )
    self.bottom_left_notification:SetLayoutScale( LAYOUT_SCALE[sm] )
    self:LayoutTopLeft()
    self:LayoutTopRight()
end

function MainOverlay:GetOverlayMode()
    return self.overlay_mode
end

function MainOverlay:RefreshOverlayMode()
    if self.overlay_mode_override then
        self:SetOverlayMode( self.overlay_mode_override )
    else
        local top_screen = self:GetFE():GetTopScreen()
        if top_screen then
            local overlay_mode = OVERLAY_MODES[top_screen._class]
            self:SetOverlayMode( overlay_mode )
        end
    end
end

function MainOverlay:SetOverlayMode( overlay_mode )
    if not overlay_mode or self.overlay_mode == overlay_mode then return end

    -- Save overlay mode
    self.overlay_mode = overlay_mode

    -- Untoggle quests if toggled
    --if self.quests_toggled then self:ToggleQuests() end

    self.top_bar:SetOverlayMode( self.overlay_mode )
    if self.overlay_mode == OVERLAY_MODE.PLAX then
        self:SetPlaxMode()
    elseif self.overlay_mode == OVERLAY_MODE.MAP then
        self:SetMapMode()
    elseif self.overlay_mode == OVERLAY_MODE.CONVO then
        self:SetConvoMode()
    elseif self.overlay_mode == OVERLAY_MODE.POPUP then
        self:SetPopupMode()
    elseif self.overlay_mode == OVERLAY_MODE.TRAVEL then
        self:SetTravelMode()
    elseif self.overlay_mode == OVERLAY_MODE.BATTLE then
        self:SetBattleMode()
    elseif self.overlay_mode == OVERLAY_MODE.NEGOTIATION then
        self:SetNegotiationMode()
    else
        self:SetInfoMode()
    end
end

function MainOverlay:LayoutTopLeft()
    self.top_left_shadow:LayoutBounds( "center", "center", 0, 0 )
        :Offset( 100, 0 )
    self.top_left_content:StackChildrenRow( 30 )
        :LayoutBounds( "left", "below", self.top_bar )
        :Offset( 35, -20 )

    -- Hide the shadow if there are no visible widgets on the corner
    local widget_count = 0
    for k, v in ipairs( self.top_left_content.children ) do
        if v:IsShown() then widget_count = widget_count + 1 end
    end
    if widget_count == 0 and self.top_left_shadow:IsShown() then
        self.top_left_shadow:AlphaTo( 0, 0.1, easing.outQuad, function() self.top_left_shadow:Hide() end )
    elseif widget_count ~= 0 and not self.top_left_shadow:IsShown() then
        self.top_left_shadow:Show():AlphaTo( 1, 0.1, easing.outQuad )
    end

    -- Scale up the shadow to cover the contents
    local w, h = self.top_left_content:GetSize()
    self.top_left_shadow:SetSize( 1000 + w, 500 + h )
end

function MainOverlay:LayoutTopRight()
    self.top_right_shadow:LayoutBounds( "center", "center", 0, 0 )
        :Offset( -100, 0 )
    self.top_right_content:StackChildrenRow( 30 )
        :LayoutBounds( "right", "below", self.top_bar )
        :Offset( -35, -20 )
    for k, w in ipairs( self.top_right_content.children ) do
        -- For animation purposes
        local x, y = w:GetPos()
        w.default_x = x
        w.default_y = y
    end

    -- Hide the shadow if there are no visible widgets on the corner
    local widget_count = 0
    for k, v in ipairs( self.top_right_content.children ) do
        if v:IsShown() then widget_count = widget_count + 1 end
    end
    if widget_count == 0 and self.top_right_shadow:IsShown() then
        self.top_right_shadow:AlphaTo( 0, 0.1, easing.outQuad, function() self.top_right_shadow:Hide() end )
    elseif widget_count ~= 0 and not self.top_right_shadow:IsShown() then
        self.top_right_shadow:Show():AlphaTo( 1, 0.1, easing.outQuad )
    end

    -- Scale up the shadow to cover the contents
    local w, h = self.top_right_content:GetSize()
    self.top_right_shadow:SetSize( 1000 + w, 500 + h )

    -- Position the quests button
    self.top_right_quests_toggle:LayoutBounds( "right", "below", self.top_bar ):Offset(0, -20)
end

-- Clicking the quests icon 
function MainOverlay:ToggleQuests()
    if self.showing_quests then
        self.quests_toggled = false
    else
        self.quests_toggled = true
    end
    self.quest_show_timer = nil

    self:RefreshToDoListVisibility()
    return self.quests_toggled
end

function MainOverlay:OnToDoListChanged()
    -- The quest list changed, make it visible for a beat
    if not self.showing_quests then
        self.quest_show_timer = 3 -- Three seconds visible whenever a quest changes
        self:RefreshToDoListVisibility()
    else
        -- If quests changed, the layout may have changed.
        self:LayoutTopRight()
    end
end

function MainOverlay:RefreshToDoListVisibility()

    local want_to_show = self.quests_toggled or (self.quest_show_timer ~= nil)

    

    if self.showing_quests ~= want_to_show then
        if not want_to_show  then
            
            self.showing_quests = false
            -- Hide it
            if self.to_do_list:IsShown() then
                local start_x, start_y = self.to_do_list.default_x or self.to_do_list.x, self.to_do_list.default_y or self.to_do_list.y
                local end_x, end_y = start_x + 30, start_y
                self.to_do_list:StopAll()
                self.to_do_list:AlphaTo( 0, 0.1, easing.outQuad, function() self.to_do_list:Hide() end )
                    :MoveTo( end_x, end_y, 0.1, easing.outQuad )
            end
            self.top_right_quests_toggle:SetState( false )
            -- If the battle reserves are meant to be shown, show them again
            self:RefreshEnemyReservesVisibility()
        else
            self:FadeInTopRight()

            self.showing_quests = true
            local end_x, end_y = self.to_do_list.default_x or self.to_do_list.x, self.to_do_list.default_y or self.to_do_list.y
            local start_x, start_y = end_x + 30, end_y
            self.to_do_list:StopAll()
            self.to_do_list:Show()
                :AlphaTo( 1, 0.1, easing.outQuad )
                :SetPos( end_x, end_y )
                --:SetPos( start_x, start_y )
                --:MoveTo( end_x, end_y, 0.2, easing.outQuad )
            self.top_right_quests_toggle:SetState( true )
            -- Hide the enemy reserves
            self.battle_enemy_reserves:Hide()
            self:LayoutTopRight()
        end
    end
    return self
end

function MainOverlay:RefreshEnemyReservesVisibility()
    if self.battle_enemy_reserves:HasReserves() and self.overlay_mode == OVERLAY_MODE.BATTLE and self:GetFE():IsHUDVisible() then
        self.battle_enemy_reserves:Show()
        self:LayoutTopRight()
        -- self:FadeInTopRight()
    else
        self.battle_enemy_reserves:Hide()
        -- self:FadeOutTopRight()
    end
    return self
end

function MainOverlay:OnGlobalEvent( eventname, ... )
    if eventname == "update_overlay" then
        self:RefreshDisplay()
    elseif eventname == "graft_slot_added" then
        self:RefreshDisplay()
    elseif eventname == "quests_changed" or eventname == "phase_change" then
        self.top_bar:RefreshProgress()
    elseif eventname == "caravan_member_event" then
        local character_event = select(1, ...)
        if character_event == "health_change" then
            local agent, new_health, old_health, new_max, old_max = select(2, ...)
            if agent:IsPlayer() then
                self.top_bar:AnimateHealth( new_health, old_health, new_max, old_max )
            end
        elseif character_event == "loc_changed" then
            --don't do anything
        else
            self:RefreshDisplay()
        end
    elseif eventname == "deck_upgrade_status_change" then
        self.top_bar:RefreshCardUpgrades()
    elseif eventname == "resolve_change" then
        local new_resolve, old_resolve, new_max, old_max = ...
        self.top_bar:AnimateResolve( new_resolve, old_resolve, new_max, old_max )
        self:RefreshDisplay()
    elseif eventname == "player_money_change" then
        self.top_bar.money_indicator:Refresh()
        self:RefreshDisplay()
    elseif eventname == "notification_logged" then
        if TheGame:GetGameState():GetNotifications():BannersEnabled() then
            local notification = ...
            if notification.banner_txt then
                self:QueueNotification( notification )
            end
        end
    elseif eventname == "agent_relationship_changed" then
        self:RefreshDisplay()

    elseif eventname == "card_added" or eventname == "card_removed" then
        local card = ...
        if is_instance( card, Battle.Card ) then
            self.top_bar.btn_battle:RefreshCount( TheGame:GetGameState():GetPlayerAgent().battler:GetCardCount() )
        else
            self.top_bar.btn_negotiation:RefreshCount( TheGame:GetGameState():GetPlayerAgent().negotiator:GetCardCount() )
        end
    end
end

function MainOverlay:TopBarHasFocus()
    return self.top_bar.main_bar.focus
end

function MainOverlay:GetTopBarHeight()
    return self.top_bar:GetHeight()
end

function MainOverlay:HighlightWidget( widget )
    local bloom = 0.8
    while bloom > 0.1 do
        bloom = bloom - FRAMES
        widget:Bloom(math.max(0.1, bloom))
        coroutine.yield()
    end
end

function MainOverlay:ShowGainCards( cards, x, y, end_fn, scale )

    scale = scale or 1
    local dismiss = false
    self:StartCoroutine(function() 
    
        local card_widgets = {}
        local root = self.center:AddChild(Widget())

        for k,card in ipairs(cards) do

            local card_widget
            if card.is_negotiation then
                card_widget = root:AddChild( Widget.NegotiationCard( card ))
            else
                card_widget = root:AddChild( Widget.BattleCard( card ) )
            end
            table.insert(card_widgets, card_widget)
            card_widget:LayoutBounds("after", "center"):Offset(20, 0)
        end
            
        if x and y then
            x,y = root:TransformFromWorld(x,y)
            root:LayoutBounds("center", "center", x, y)
        else
            root:LayoutBounds("center", "center", 0, 0)
        end

        for k, card_widget in ipairs(card_widgets) do
            card_widget:Hide()
        end

        for k, card_widget in ipairs(card_widgets) do
            local card = card_widget.card
            AUDIO:PlayEvent( CARD_RARITY_SOUNDS[ card.rarity ] )
            AUDIO:PlayEvent( CARD_TYPE_SOUNDS[ GetCardType( card )] )
            card_widget:Show():ScaleTo(scale*.6, scale*.7, 1, easing.outCirc)

            if k < #card_widgets and not dismiss then
                self:Delay(.4)
            end
        end

        local wait_time = 1
        local t = 0 
        while t < wait_time and not dismiss do
            t = t  + coroutine.yield()
        end

        for k, card_widget in ipairs(card_widgets) do

            local target_widget
            if card_widget.card.is_negotiation then
                target_widget = self.top_bar:GetNegotiationButton()
            else
                target_widget = self.top_bar:GetBattleButton()
            end
            
            self:StartCoroutine(function() 
                AUDIO:PlayEvent( card_widget.card.is_negotiation and "event:/sfx/battle/cards/gen/action_puts_card_negotiationdeck" or "event:/sfx/battle/cards/gen/action_puts_card_combatdeck" )

                card_widget:AnimateToWidget( target_widget ) -- internally calls widget:Remove()
                self:HighlightWidget( target_widget )
            end)
            
            
            if k < #card_widgets and not dismiss then
                self:Delay(.2)
            end
        end

        if end_fn then
            end_fn()
        end

    end)

    
    return function() dismiss = true end
end

function MainOverlay:QueueNotification( notification )
    self.bottom_left_notification:Queue( notification )
end

function MainOverlay:IsShowingNotification()
    return self.bottom_left_notification and self.bottom_left_notification:IsShowingNotification() 
end

function MainOverlay:QueueTip( tip )
    local top_screen = self:GetFE():GetTopScreen()
    if is_instance( top_screen, Screen.ConversationScreen ) then
        top_screen:SnapSpool()
        if top_screen.fastmode then
            return
        end
    end

    if self.current_tip then
        table.insert( self.tip_queue, tip )
    else
        self:ShowTip( tip )
    end

    if is_instance( top_screen, Screen.ConversationScreen ) then
        top_screen.encounter:YieldEncounter( function() return self.current_tip ~= tip end )
    end
end

function MainOverlay:ShowTip( tip )
end

function MainOverlay:IsShowingTip()
    return self.current_tip ~= nil
end

function MainOverlay:OnCompleteTip()
    self.current_tip = nil
    local t = table.remove( self.tip_queue, 1 )
    if t then
        self:ShowTip( t )
    end
end

function MainOverlay:HideOverlay()
    -- Can't hide the screen because FE:UpdateScreenViz doesn't care about anybody :p
    -- (It will overwrite our shown status anytime the FE changes)
    self.interface:Hide()
end

function MainOverlay:ShowOverlay()
    self.interface:Show()
end


function MainOverlay:SetPlaxMode()
    local show_hud = self:GetFE():IsHUDVisible()
    self.overlay_mode = OVERLAY_MODE.PLAX
    self.battle_friend_reserves:Hide()
    self.battle_enemy_reserves:Hide()
    self.top_right:SetShown( show_hud and self.top_right_shown )
    self.top_left:SetShown( show_hud and self.top_left_shown )
    self.top_bar:SetShown( show_hud )
    self:RefreshDisplay()
    return self
end

function MainOverlay:SetConvoMode()
    local show_hud = self:GetFE():IsHUDVisible()
    self.overlay_mode = OVERLAY_MODE.CONVO
    self.battle_friend_reserves:Hide()
    self.battle_enemy_reserves:Hide()
    self.top_right:SetShown( show_hud and self.top_right_shown )
    self.top_left:SetShown( show_hud and self.top_left_shown )
    self.top_bar:SetShown( show_hud )
    self:RefreshDisplay()
    return self
end

function MainOverlay:SetMapMode()
    local show_hud = self:GetFE():IsHUDVisible()
    self.overlay_mode = OVERLAY_MODE.MAP
    self.battle_friend_reserves:Hide()
    self.battle_enemy_reserves:Hide()
    self.top_right:SetShown( show_hud and self.top_right_shown )
    self.top_left:SetShown( show_hud and self.top_left_shown )
    self.top_bar:SetShown( show_hud )
    self:RefreshDisplay()
    return self
end

function MainOverlay:SetNegotiationMode()
    local show_hud = self:GetFE():IsHUDVisible()
    self.overlay_mode = OVERLAY_MODE.NEGOTIATION
    self.battle_friend_reserves:Hide()
    self.battle_enemy_reserves:Hide()
    self.top_right:SetShown( show_hud and self.top_right_shown )
    self.top_left:SetShown( show_hud and self.top_left_shown )
    self.top_bar:SetShown( show_hud )
    self:RefreshDisplay()
    return self
end

function MainOverlay:SetBattleMode()
    local show_hud = self:GetFE():IsHUDVisible()
    self.overlay_mode = OVERLAY_MODE.BATTLE
    self.battle_friend_reserves:SetShown( self.battle_friend_reserves:HasReserves() and show_hud )
    self.battle_enemy_reserves:SetShown( self.battle_enemy_reserves:HasReserves() and show_hud )
    self.top_right:SetShown( show_hud and self.top_right_shown )
    self.top_left:SetShown( show_hud and self.top_left_shown )
    self.top_bar:SetShown( show_hud )
    self:RefreshDisplay()
    return self
end

function MainOverlay:SetInfoMode()
    local show_hud = self:GetFE():IsHUDVisible()
    self.overlay_mode = OVERLAY_MODE.INFO
    self.battle_friend_reserves:Hide()
    self.battle_enemy_reserves:Hide()
    self.top_right:SetShown( show_hud and self.top_right_shown )
    self.top_left:SetShown( show_hud and self.top_left_shown )
    self.top_bar:SetShown( show_hud )
    self:RefreshDisplay()
    return self
end

function MainOverlay:SetPopupMode()
    local show_hud = self:GetFE():IsHUDVisible()
    self.overlay_mode = OVERLAY_MODE.POPUP
    self.battle_friend_reserves:Hide()
    self.battle_enemy_reserves:Hide()
    self.top_left:Hide()
    self.top_right:Hide()
    self.top_bar:SetShown( show_hud )
    self:RefreshDisplay()
    return self
end

function MainOverlay:SetTravelMode()
    local show_hud = self:GetFE():IsHUDVisible()
    self.overlay_mode = OVERLAY_MODE.TRAVEL
    self.battle_friend_reserves:Hide()
    self.battle_enemy_reserves:Hide()
    self.top_right:SetShown( show_hud and self.top_right_shown )
    self.top_left:SetShown( show_hud and self.top_left_shown )
    self.top_bar:SetShown( show_hud )
    self:RefreshDisplay()
    return self
end

function MainOverlay:OnScreenResize(w, h)
    MainOverlay._base.OnScreenResize(self, w, h)
    self:RefreshDisplay()
end

function MainOverlay:RefreshDisplay()
    self.top_bar:Refresh()
    self:RefreshToDoListVisibility()
    self:LayoutTopLeft()
    self:LayoutTopRight()
    self.bottom_center_inner:LayoutBounds("center", "bottom", 0, 0):Offset(0, MARGIN)
    self.bottom_left_inner:LayoutBounds("left", "bottom", 0, 0):Offset(MARGIN, MARGIN)
    return self
end

function MainOverlay:GetNegotiationGraftsWidget()
    return self.top_bar:GetNegotiationGraftsWidget()
end

function MainOverlay:GetBattleGraftsWidget()
    return self.top_bar:GetBattleGraftsWidget()
end

function MainOverlay:GetBoonGraftsWidget()
    return self.top_bar:GetBoonGraftsWidget()
end

function MainOverlay:GetBaneGraftsWidget()
    return self.top_bar:GetBaneGraftsWidget()
end

function MainOverlay:GetPerksGraftsWidget()
    return self.top_bar:GetPerksGraftsWidget()
end

function MainOverlay:SetFriendReserves( battle, team )
    self.battle_friend_reserves:Refresh( battle, team )
    return self
end

function MainOverlay:SetEnemyReserves( battle, team )
    self.battle_enemy_reserves:Refresh( battle, team )
    self:RefreshToDoListVisibility()
    self:RefreshEnemyReservesVisibility()
    return self
end

function MainOverlay:RefreshScenario( battle )
    self.top_bar:RefreshScenario( battle )
    return self
end

function MainOverlay:GetDefaultFocus()
    return self:TopBarHasFocus() and self.top_bar:GetDefaultFocus()
end

function MainOverlay:IsTopLeftShown()
    return self.top_left and self.top_left_shown
end

function MainOverlay:FadeInTopLeft( duration )
    duration = duration or 0.1
    self.top_left_shown = true
    self.top_left:Show():AlphaTo( 1, duration, easing.inQuad )
    self.top_bar:FadeInTopLeft( duration )
    return self
end

function MainOverlay:FadeOutTopLeft( duration )
    duration = duration or 0.1
    self.top_left_shown = false
    self.top_left:AlphaTo( 0, duration, easing.inQuad, function() self.top_left:Hide() end )
    self.top_bar:FadeOutTopLeft( duration )
    return self
end

function MainOverlay:IsTopRightShown()
    return self.top_right and self.top_right_shown
end

function MainOverlay:FadeInTopRight( duration )
    duration = duration or 0.1
    self.top_right_shown = true
    self.top_right:Show():AlphaTo( 1, duration, easing.inQuad )
    self.top_bar:FadeInTopRight( duration )
    return self
end

function MainOverlay:FadeOutTopRight( duration, keep_pause_btn )
    duration = duration or 0.1
    self.top_right_shown = false
    self.top_right:AlphaTo( 0, duration, easing.inQuad, function() self.top_right:Hide() end )
    self.top_bar:FadeOutTopRight( duration, keep_pause_btn )
    return self
end

function MainOverlay:FadeInBars( duration )
    if not self.bars_shown then
        duration = duration or 0.1
        self.bars_shown = true
        self.interface:Show():AlphaTo( 1, duration, easing.inQuad )
    end
    return self
end

function MainOverlay:FadeOutBars( duration )
    if self.bars_shown then
        duration = duration or 0.1
        self.bars_shown = false
        self.interface:AlphaTo( 0, duration, easing.inQuad, function() self.top_right:Hide() end )
    end
    return self
end

------------------------------------------------------------------------------------------
-- 

local BattleReserves = class( "Widget.BattleReserves", Widget )

function BattleReserves:init( left )
    BattleReserves._base.init( self )

    self.left = left or false
    self.title = self:AddChild( Widget.Label( "body", 20, LOC"UI.FIGHT_SCREEN.RESERVE_FIGHTERS" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
    self.portraits = self:AddChild( Widget() )
end

function BattleReserves:Refresh( battle_engine, team )

    local reserve_fighters = battle_engine:GetTeam(team).reserve_fighters
    if #reserve_fighters > 0 then
        local max_fighters = math.min( #reserve_fighters, 4 )
        if max_fighters > #self.portraits.children then
            for k = 1, max_fighters - #self.portraits.children  do
                local w = Widget()
                w.portrait = w:AddChild( Widget.CharacterPortrait() ):SetScale(.67)
                if team == team.RED then
                    w.portrait:SetFacing(false)
                end
                self.portraits:AddChild(w)
            end

        elseif #self.portraits.children > max_fighters then
            for k = 1, #self.portraits.children - max_fighters do
                self.portraits.children[#self.portraits.children]:Remove()
            end
        end

        if #reserve_fighters > max_fighters then
            local txt = loc.format( "{1%+d}", #reserve_fighters - max_fighters )
            if self.overflow == nil then
                local w = Widget.Label( "title", 36, txt )
                self:AddChild( w )
                self.overflow = w
            else
                self.overflow:SetText( txt )
            end
        else
            if self.overflow then
                self.overflow:Remove()
                self.overflow = nil
            end
        end
        
        for k,v in ipairs(reserve_fighters) do
            local offset = team == TEAM.BLUE and 48 or -48
            local w = self.portraits.children[k]
            if w then
                w.portrait:SetCharacter(v.agent)
                w:SetToolTip( loc.format( LOC"UI.FIGHT_SCREEN.RESERVES_ARRIVE_TOOLTIP", v:GetName() ))
                w:SetPos((k-1)*offset, -40)
            end
        end
        self:Show()
    else
        -- Remove any existing widgets
        self.portraits:DestroyAllChildren()

        self:Hide()
    end

    self.portraits:LayoutBounds( self.left and "left" or "right", "below", self.title )
        :Offset( 0, 3 )

    if self.overflow then
        self.overflow:LayoutBounds( "after", "top", self.portraits ):Offset( -8, 8 )
    end
end

function BattleReserves:HasReserves()
    return self.portraits.children and #self.portraits.children > 0
end

------------------------------------------------------------------------------------------
-- 

local MainOverlayNavButton = class( "Widget.MainOverlayNavButton", Widget.Clickable )

function MainOverlayNavButton:init( img, fn, txt )
    MainOverlayNavButton._base.init( self )

    self.backing = self:AddChild( Widget.SolidBox( 20, 20, 0xFFFF0000 ) )
    self.img = self:AddChild( Widget.Image( img, 20, 20 ) )
        :SetHiddenBoundingBox( true )

    self:SetFocusable( true )
    self.selected = false
end

function MainOverlayNavButton:SetSize( w, h )
    self.backing:SetSize( w, h )
    self.img:SetSize( w, h )
    return self
end

function MainOverlayNavButton:UpdateImage()
    if self.enabled and ( self.selected and ( self.hover or self.focus ) ) then
        self.img:ScaleTo( nil, 1.15, 0.2, easing.outQuad )
        self.img:Bloom(0.25)
    elseif self.enabled and ( self.selected or self.hover or self.focus ) then
        self.img:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
        self.img:Bloom(0.2)
    else
        self.img:ScaleTo( nil, 1, 0.2, easing.outQuad )
        self.img:Bloom(0.1)
    end
    self:SetClickSoundEvent( (self.selected and self.audio_close) and self.audio_close or self.audio_open )
end

function MainOverlayNavButton:SetToolTipSuffix( suffix )
    self.tooltip_suffix = suffix
    return self
end

function MainOverlayNavButton:OnGainFocus()
    MainOverlayNavButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function MainOverlayNavButton:OnLoseFocus()
    MainOverlayNavButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function MainOverlayNavButton:OnGainHover()
    MainOverlayNavButton._base.OnGainHover(self)
    self:UpdateImage()
end

function MainOverlayNavButton:OnLoseHover()
    MainOverlayNavButton._base.OnLoseHover(self)
    self:UpdateImage()
end

function MainOverlayNavButton:SetSelected( selected )
    self.selected = selected or false
    self:UpdateImage()
    return self
end

function MainOverlayNavButton:AddTargetScreenClass( target_screen_class )
    self.target_screen_classes = self.target_screen_classes or {}
    table.insert( self.target_screen_classes, target_screen_class )
    return self
end

function MainOverlayNavButton:IsInScreen( current_screen_class )
    return table.contains( self.target_screen_classes or {}, current_screen_class )
end


------------------------------------------------------------------------------------------

local MainOverlayDeckButton = class( "Widget.MainOverlayDeckButton", MainOverlayNavButton )

function MainOverlayDeckButton:init( img, fn, txt, clr  )
    MainOverlayDeckButton._base.init( self, img, fn, txt )

    self.box = self:AddChild( Widget.SolidBox( 30, 20, HexColour( clr, 0.4 )):Bloom( 0.1 ))
    self.box:SendBehind( self.img )
    self.box:LayoutBounds( "before", "center", self.img ):Offset( -4, 0 )

    self.count = self:AddChild( Widget.Label( "title", 22 ):SetGlyphColour( UICOLOURS.BLACK ):Bloom(0.1))
    self.count:LayoutBounds( "left", "center", self.box ):Offset( 2, 0 )
end

function MainOverlayDeckButton:RefreshCount( count )
    self.count:SetText( tostring(count) )
    self.count:LayoutBounds( "left", "center", self.box ):Offset( 2, 0 )
end

function MainOverlayDeckButton:UpdateImage()
    MainOverlayDeckButton._base.UpdateImage( self )
    if self.box then
        if self.enabled and ( self.selected and ( self.hover or self.focus ) ) then
            self.box:Bloom(0.25)
        elseif self.enabled and ( self.selected or self.hover or self.focus ) then
            self.box:Bloom(0.2)
        else
            self.box:Bloom(0.1)
        end
    end
end


------------------------------------------------------------------------------------------
-- 

local MainOverlayTopBar = class( "Widget.MainOverlayTopBar", Widget )

local BAR_HEIGHT = 50
local BTN_SIZE = 40

function MainOverlayTopBar:init(main_overlay)
    MainOverlayTopBar._base.init( self )
    self.main_overlay = main_overlay

    local mq = TheGame:GetGameState() and TheGame:GetGameState():GetMainQuest()
    local is_brawl = mq and mq:GetQuestDef().get_progression

    -- Add our two main bars
    self.main_bar = self:AddChild( Widget() ) -- The main bar on the top
    self.secondary_bar = self:AddChild( Widget() ) -- The bar under it

    -- Bar heights
    self.main_bar_h = BAR_HEIGHT
    self.secondary_bar_h = BAR_HEIGHT*0.8

    -- Setup their backgrounds
    self.main_bar_bg = self.main_bar:AddChild( Widget.Image( engine.asset.Texture("UI/controls_top_bar.tex") ) )
    self.main_bar_bg.hover_check = true
    self.main_bar_bg_focus = self.main_bar:AddChild( Widget.Image( engine.asset.Texture("UI/controls_top_bar_focus.tex") ) )
        :SetTintAlpha( 0 )
    self.secondary_bar_bg = self.secondary_bar:AddChild( Widget.Image( engine.asset.Texture("UI/controls_secondary_bar.tex") ) )
        :SetTintAlpha( 0.5 )
    self.secondary_bar_bg.hover_check = true
    self.secondary_bar_bg_focus = self.secondary_bar:AddChild( Widget.Image( engine.asset.Texture("UI/controls_top_bar_focus.tex") ) )
        :SetTintAlpha( 0 )


    -------------------------------------------------------------------------------------------
    -- MAIN BAR
    -------------------------------------------------------------------------------------------

    -- Main bar left
    self.main_bar_left_widgets = self.main_bar:AddChild( Widget() )
    self.prestige_indicator = Widget.PrestigeIndicator( 50 )
        :SetFocusable( true )
        :ShowToolTipOnFocus( false )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.left_corner_button = self.main_bar_left_widgets:AddChild( Widget.MainOverlayCornerButton( self.prestige_indicator, self.h, true ))
    
    if not is_brawl then
        self.progress_container = self.main_bar_left_widgets:AddChild( Widget() )
        self.progress_indicator = self.progress_container:AddChild( Widget.ProgressIndicator( 31, 30, true) )
            :Bloom(0.1)
            :SetFocusable( true )
            :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
        self.progress_container:LayoutBounds( "after", "center" ):Offset( 20, 0 )
    end    

    local icon_size = 24
    local text_size = 30
    self.money_indicator = self.main_bar_left_widgets:AddChild( Widget.MoneyIndicator( icon_size, text_size, true) )
        :Bloom(0.1)
        :SetFocusable( true )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
        :LayoutBounds( "after", "center" ):Offset( 20, 0 )
    self.mettle_indicator = self.main_bar_left_widgets:AddChild( Widget.MettleIndicator( icon_size, text_size, true) )
        :Bloom(0.1)
        :SetFocusable( true )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
        :LayoutBounds( "after", "center" ):Offset( 20, 0 )
    self.resolve_indicator = self.main_bar_left_widgets:AddChild( Widget.ResolveIndicator( icon_size, text_size ) )
        :Bloom(0.1)
        :SetFocusable( true )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
        :LayoutBounds( "after", "center" ):Offset( 20, 0 )
    self.health_indicator = self.main_bar_left_widgets:AddChild( Widget.HealthIndicator( icon_size, text_size ) )
        :Bloom(0.1)
        :SetFocusable( true )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
        :LayoutBounds( "after", "center" ):Offset( 20, 0 )
    self.fight_location_status = self.main_bar_left_widgets:AddChild( Widget.FightLocationStatus( text_size ) )
        :Bloom(0.1)
        :LayoutBounds( "after", "center" ):Offset( 20, 0 )

    -- Main bar centre
    self.main_bar_center_widgets = self.main_bar:AddChild( Widget() )

    if TheGame:GetGameState() and TheGame:GetGameState():GetGameType() == GAME_TYPE.DAILY then
        self.daily_progress = self.main_bar_center_widgets:AddChild( Widget.DailyProgress() )
    end


    -- Main bar right
    self.main_bar_right_widgets = self.main_bar:AddChild( Widget() )

    self.crew_container = self.main_bar_right_widgets:AddChild( Widget() )
    self.crew = self.crew_container:AddChild( Widget.CrewPortraits( 54 ) )
        :ShowToolTipOnFocus( false )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )

    local relationships_size = 26
    self.relationships_indicator = self.main_bar_right_widgets:AddChild( Widget.MainOverlayRelationshipsIndicator( 54, relationships_size ) )
        :SetFn( function()
            local screen = self:NavigateTo( Screen.RelationshipScreen )
            screen:SetNavLeft( loc.format( LOC "UI.CONTROLS.NAV_BATTLE_DECK", Controls.Digital.GAMEPAD_LT ),
                function( screen )
                    screen:DoClose()
                    self.btn_battle:Click()
                end )
            screen:SetNavRight( loc.format( LOC "UI.CONTROLS.NAV_NEGOTIATION_DECK", Controls.Digital.GAMEPAD_RT ),
                function( screen )
                    screen:DoClose()
                    self.btn_negotiation:Click()
                end )
        end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )

    self.nav_buttons = self.main_bar_right_widgets:AddChild( Widget() )

    self.btn_negotiation = self.nav_buttons:AddChild( Widget.MainOverlayDeckButton( engine.asset.Texture("UI/ic_nav_negotiation.tex" ), nil, nil, UICOLOURS.NEGOTIATION ) )
        :SetSize( BTN_SIZE, BTN_SIZE )
        :SetOnClickFn( function()
            -- Check if we're in one of this button's screens
            local top_screen = TheGame:FE():GetTopScreen()
            if self.btn_negotiation:IsInScreen( top_screen._class ) then
                -- Close the screen
                top_screen:FadeOut()
            else
                -- If not, navigate to the deck screen
                local screen = self:NavigateTo( Screen.NegotiationDeck, self.overlay_mode == OVERLAY_MODE.BATTLE or self.overlay_mode == OVERLAY_MODE.NEGOTIATION )
               screen:SetNavLeft( loc.format( LOC "UI.CONTROLS.NAV_RELATIONSHIPS", Controls.Digital.GAMEPAD_LT ),
                    function( screen )
                        screen:Close()
                        self.relationships_indicator:Click()
                    end )
                screen:SetNavRight( loc.format( LOC "UI.CONTROLS.NAV_BATTLE_DECK", Controls.Digital.GAMEPAD_RT ),
                    function( screen )
                        screen:Close()
                        self.btn_battle:Click()
                    end )
            end
        end )
        :AddTargetScreenClass( Screen.NegotiationDeck )
        :AddTargetScreenClass( Screen.NegotiationDeckFiltersSidebar )
        :SetFocusable( true )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.negotiation_upgrade_indicator = self.btn_negotiation:AddChild( Widget.UpgradeIndicator() )
        :SetHiddenBoundingBox( true )
    self.btn_battle = self.nav_buttons:AddChild( Widget.MainOverlayDeckButton( engine.asset.Texture("UI/ic_nav_combat.tex" ), nil, nil, UICOLOURS.FIGHT ) )
        :SetSize( BTN_SIZE, BTN_SIZE )
        :SetOnClickFn( function()
            -- Check if we're in one of this button's screens
            local top_screen = TheGame:FE():GetTopScreen()
            if self.btn_battle:IsInScreen( top_screen._class ) then
                -- Close the screen
                top_screen:FadeOut()
            else
                -- If not, navigate to the deck screen
                local screen = self:NavigateTo( Screen.BattleDeck, self.overlay_mode == OVERLAY_MODE.BATTLE or self.overlay_mode == OVERLAY_MODE.NEGOTIATION )
               screen:SetNavLeft( loc.format( LOC "UI.CONTROLS.NAV_NEGOTIATION_DECK", Controls.Digital.GAMEPAD_LT ),
                    function( screen )
                        screen:Close()
                        self.btn_negotiation:Click()
                    end )
                screen:SetNavRight( loc.format( LOC "UI.CONTROLS.NAV_RELATIONSHIPS", Controls.Digital.GAMEPAD_RT ),
                    function( screen )
                        screen:Close()
                        self.relationships_indicator:Click()
                    end )
            end
        end )
        :AddTargetScreenClass( Screen.BattleDeck )
        :AddTargetScreenClass( Screen.BattleDeckFiltersSidebar )
        :SetFocusable( true )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.battle_upgrade_indicator = self.btn_battle:AddChild( Widget.UpgradeIndicator() )
        :SetHiddenBoundingBox( true )
    -- self.btn_graft = self.nav_buttons:AddChild( Widget.MainOverlayNavButton( global_images.graft ) )
    --     :SetSize( BTN_SIZE, BTN_SIZE )
    --     :SetOnClickFn( function() self:NavigateTo( Screen.GraftCollectionScreen, TheGame:GetGameState():GetPlayerAgent(), self.overlay_mode == OVERLAY_MODE.BATTLE or self.overlay_mode == OVERLAY_MODE.NEGOTIATION ) end )
    --     :AddTargetScreenClass( Screen.GraftCollectionScreen )
    --     :SetFocusable( true )
    --     :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    -- self.graft_upgrade_indicator = self.btn_graft:AddChild( Widget.UpgradeIndicator() )
    --     :SetGraftMode()
    --     :SetHiddenBoundingBox( true )
    -- self.btn_coin = self.nav_buttons:AddChild( Widget.MainOverlayNavButton( global_images.coin ) )
    --     :SetSize( BTN_SIZE, BTN_SIZE )
    --     :SetOnClickFn( function() self:NavigateTo( Screen.CoinScreen ) end )
    --     :AddTargetScreenClass( Screen.CoinScreen )
    --     :SetFocusable( true )
    --     :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    -- self.btn_blaster = self.nav_buttons:AddChild( Widget.MainOverlayNavButton( global_images.blaster ) )
    --     :SetSize( BTN_SIZE, BTN_SIZE )
    --     :SetOnClickFn( function() self:NavigateTo( Screen.BlasterScreen ) end )
    --     :AddTargetScreenClass( Screen.BlasterScreen )
    --     :SetFocusable( true )
    --     :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    -- self.btn_mutators = self.nav_buttons:AddChild( Widget.MainOverlayNavButton( global_images.mutators ) )
    --     :SetSize( BTN_SIZE, BTN_SIZE )
    --     :SetOnClickFn( function() self:NavigateTo( Screen.MutatorsScreen ) end )
    --     :AddTargetScreenClass( Screen.MutatorsScreen )
    --     :SetFocusable( true )
    --     :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    -- self.btn_relationships = self.nav_buttons:AddChild( Widget.MainOverlayNavButton( global_images.agents ) )
    --     :SetSize( BTN_SIZE, BTN_SIZE )
    --     :SetOnClickFn( function() self:NavigateTo( Screen.RelationshipScreen ) end )
    --     :AddTargetScreenClass( Screen.RelationshipScreen )
    --     :SetFocusable( true )
    --     :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    -- self.btn_quests = self.nav_buttons:AddChild( Widget.MainOverlayNavButton( global_images.quest ) )
    --     :SetSize( BTN_SIZE, BTN_SIZE )
    --     :SetOnClickFn( function() self:ToggleQuests() end )
    --     :SetFocusable( true )
    --     :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )

    self.btn_feedback = Widget.MainOverlayNavButton( engine.asset.Texture("UI/ic_nav_question.tex" ) )
        :SetSize( BTN_SIZE, BTN_SIZE )
        :SetOnClickFn( function() TheGame:StartFeedback() end )
        :SetFocusable( true )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
        :SetShown( SPECIAL_BUILD_TYPE ~= (1024+64) )

    self.btn_pause = Widget.MainOverlayNavButton( global_images.pause )
        :SetSize( BTN_SIZE, BTN_SIZE )
        :SetOnClickFn( function() self:NavigateTo( Screen.PauseMenu ) end )
        :AddTargetScreenClass( Screen.PauseMenu )
        :SetFocusable( true )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.controller_toggle_label = Widget.Label( "title", 30, loc.format( LOC"UI.CONTROLS.OPEN_MENU", Controls.Digital.OPEN_OVERLAY_MENU ) )
        :Bloom( 0.1 )
    self.right_corner_button = self.main_bar_right_widgets:AddChild( Widget.MainOverlayCornerButton( self.btn_pause, self.h, false ))
        :SetControllerLabel( self.controller_toggle_label )
    self.right_corner_button:AddContent( self.btn_feedback )

    -------------------------------------------------------------------------------------------
    -- SECONDARY BAR
    -------------------------------------------------------------------------------------------

    self.secondary_bar_left_widgets = self.secondary_bar:AddChild( Widget() )
    self.secondary_bar_left_lower_widgets = self.secondary_bar:AddChild( Widget() )
    self.secondary_bar_center_widgets = self.secondary_bar:AddChild( Widget() )
    self.secondary_bar_right_widgets = self.secondary_bar:AddChild( Widget() )

    local height = 44
    local icon_size = 34
    local mutators_size = 28
    local feats_size = 40
    self.grafts_coin = self.secondary_bar_left_widgets:AddChild( Widget.MainOverlayGraftsContainer( height, icon_size, GRAFT_TYPE.COIN ) )
        :SetFn( function() self:NavigateTo( Screen.CoinScreen ) end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.grafts_blaster = self.secondary_bar_left_widgets:AddChild( Widget.MainOverlayGraftsContainer( height, icon_size, GRAFT_TYPE.BLASTER ) )
        :SetFn( function() self:NavigateTo( Screen.BlasterScreen ) end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.grafts_negotiation = self.secondary_bar_left_widgets:AddChild( Widget.MainOverlayGraftsContainer( height, icon_size, GRAFT_TYPE.NEGOTIATION ) )
        :SetFn( function() self:NavigateTo( Screen.GraftCollectionScreen ) end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.grafts_battle = self.secondary_bar_left_widgets:AddChild( Widget.MainOverlayGraftsContainer( height, icon_size, GRAFT_TYPE.COMBAT ) )
        :SetFn( function() self:NavigateTo( Screen.GraftCollectionScreen ) end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.grafts_mutators = self.secondary_bar_right_widgets:AddChild( Widget.MainOverlayGraftsContainer( height, mutators_size, GRAFT_TYPE.MUTATOR ) )
        :SetFn( function() self:NavigateTo( Screen.MutatorsScreen ) end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.grafts_perks = self.secondary_bar_right_widgets:AddChild( Widget.MainOverlayPerksContainer( height, mutators_size ) )
        :SetFn( function() self:NavigateTo( Screen.PerkScreen ) end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "right", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.grafts_boons = self.secondary_bar_left_lower_widgets:AddChild( Widget.MainOverlaySocialGraftsContainer( height, icon_size, true ) )
        :SetFn( function() self:NavigateTo( Screen.BoonsBanesScreen ) end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
    self.grafts_banes = self.secondary_bar_left_lower_widgets:AddChild( Widget.MainOverlaySocialGraftsContainer( height, icon_size, false ) )
        :SetFn( function() self:NavigateTo( Screen.BoonsBanesScreen ) end )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
        -- For daily challenges
    self.feats = self.secondary_bar_center_widgets:AddChild( Widget.DailyFeatsStatus( feats_size ) )
    self.high_score = self.secondary_bar_center_widgets:AddChild( Widget.HighScoreWidget( 0, FONT_SIZE.SCREEN_TITLE*0.9, function() self:Layout() end))
        :OverrideHeight( 33, 3 ) -- make the hitbox visible on pointsfloater.lua HighScoreWidget:OverrideHeight() to be able to position this
        :SetToolTipClass( Widget.TooltipHighScoreWidget )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "center", "below", w ):Offset( 0, -SPACING.M1 ) end )

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )

end

function MainOverlayTopBar:NavigateTo( screen, ... )
    local top_screen = TheGame:FE():GetTopScreen()
    if top_screen._class == screen._class then
        -- We're on this screen already. Close it
        top_screen:FadeOut()

    elseif table.contains( NAV_SCREENS, screen._class ) then
        -- We're going to a different nav screen
        if top_screen and table.contains( NAV_SCREENS, top_screen._class ) then
            -- Close the old screen
            top_screen:FadeOut()
        end

        TheGame:GetMusic():PlayNavMusic()
        top_screen = screen( ... )
        TheGame:FE():InsertScreen( top_screen )
    end

    return top_screen
end

function MainOverlayTopBar:OnControlModeChange( cm, device_id )
    MainOverlayTopBar._base.OnControlModeChange( self, cm, device_id )

    local negotiation_cards, battle_cards = 0, 0
    if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() then
        negotiation_cards = #TheGame:GetGameState():GetPlayerAgent():GetAspect("negotiator"):GetCards()
        battle_cards = #TheGame:GetGameState():GetPlayerAgent():GetAspect("battler"):GetCards()
    end

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        -- Show button, but hide controller-only labels
        self.right_corner_button:Show()
        self.right_corner_button:HideControllerLabel()

        -- Allow looping of focus
        self.prestige_indicator:SetFocusDir( "left", self.btn_pause, true )

        -- Update button tooltips with hotkey info
        self.btn_feedback:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", LOC"UI.MAIN_OVERLAY.FEEDBACK", Controls.Digital.FEEDBACK ) )
        self.btn_negotiation:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", loc.format(LOC"UI.MAIN_OVERLAY.VIEW_NEGOTIATION_DECK",negotiation_cards) .. (self.btn_negotiation.tooltip_suffix or ""), Controls.Digital.TOGGLE_NEGOTIATION_DECK ) )
        self.btn_battle:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", loc.format(LOC"UI.MAIN_OVERLAY.VIEW_BATTLE_DECK",battle_cards) .. (self.btn_battle.tooltip_suffix or ""), Controls.Digital.TOGGLE_BATTLE_DECK ) )
        self.grafts_battle:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", LOC"UI.MAIN_OVERLAY.VIEW_GRAFTS" .. (self.grafts_battle.tooltip_suffix or ""), Controls.Digital.TOGGLE_GRAFTS ) )
        self.grafts_negotiation:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", LOC"UI.MAIN_OVERLAY.VIEW_GRAFTS" .. (self.grafts_negotiation.tooltip_suffix or ""), Controls.Digital.TOGGLE_GRAFTS ) )
        self.grafts_coin:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", LOC"UI.MAIN_OVERLAY.VIEW_COIN", Controls.Digital.TOGGLE_COIN ) )
        self.grafts_blaster:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", LOC"UI.MAIN_OVERLAY.VIEW_BLASTER", Controls.Digital.TOGGLE_BLASTER ) )
        self.grafts_mutators:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", LOC"UI.MAIN_OVERLAY.VIEW_MUTATORS", Controls.Digital.TOGGLE_MUTATORS ) )
        self.relationships_indicator:RefreshToolTip()
        self.btn_pause:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", LOC"UI.MAIN_OVERLAY.PAUSE", Controls.Digital.PAUSE ) )

    end
    if cm == CONTROL_MODE.TOUCH then
        -- Show button, but hide controller-only labels
        self.right_corner_button:Show()
        self.right_corner_button:HideControllerLabel()
        
        -- Allow looping of focus
        self.prestige_indicator:SetFocusDir( "left", self.btn_pause, true )

        -- Update button tooltips without hotkey info
        self.btn_feedback:SetToolTip( LOC"UI.MAIN_OVERLAY.FEEDBACK" )
        self.btn_negotiation:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.VIEW_NEGOTIATION_DECK", negotiation_cards) .. (self.btn_negotiation.tooltip_suffix or "") )
        self.btn_battle:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.VIEW_BATTLE_DECK", battle_cards ) .. (self.btn_battle.tooltip_suffix or "") )
        self.grafts_battle:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_GRAFTS" .. (self.grafts_battle.tooltip_suffix or "") )
        self.grafts_negotiation:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_GRAFTS" .. (self.grafts_negotiation.tooltip_suffix or "") )
        self.grafts_coin:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_COIN" )
        self.grafts_blaster:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_BLASTER" )
        self.grafts_mutators:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_MUTATORS" )
        self.relationships_indicator:RefreshToolTip()

        self.btn_pause:SetToolTip( LOC"UI.MAIN_OVERLAY.PAUSE" )
    end
    if cm == CONTROL_MODE.GAMEPAD then
        -- Show controller-only
        self.controller_toggle_label:SetText(loc.format( LOC"UI.CONTROLS.OPEN_MENU", Controls.Digital.OPEN_OVERLAY_MENU ))

        if SPECIAL_BUILD_TYPE == (1024+64) then
            -- Show button, but no hint
            self.right_corner_button:Show()
            self.right_corner_button:HideControllerLabel()
        elseif CheckBits( SPECIAL_BUILD_TYPE, 128 ) then
            -- Hide button and hint
            self.right_corner_button:Hide()
            self.right_corner_button:HideControllerLabel()
        elseif CheckBits( SPECIAL_BUILD_TYPE, 256 ) then
            -- Hide button and hint
            self.right_corner_button:Hide()
            self.right_corner_button:HideControllerLabel()
        else
            -- Show button and hint
            self.right_corner_button:Show()
            self.right_corner_button:ShowControllerLabel()
        end

        -- Allow looping of focus even if the pause button isn't visible
        if self.right_corner_button:IsShown() then
            self.prestige_indicator:SetFocusDir( "left", self.btn_pause, true )
        else
            self.prestige_indicator:SetFocusDir( "left", self.relationships_indicator, true )
        end

        -- Update button tooltips without hotkey info
        self.btn_feedback:SetToolTip( LOC"UI.MAIN_OVERLAY.FEEDBACK" )
        self.btn_negotiation:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.VIEW_NEGOTIATION_DECK", negotiation_cards) .. (self.btn_negotiation.tooltip_suffix or "") )
        self.btn_battle:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.VIEW_BATTLE_DECK", battle_cards ) .. (self.btn_battle.tooltip_suffix or "") )
        self.grafts_battle:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_GRAFTS" .. (self.grafts_battle.tooltip_suffix or "") )
        self.grafts_negotiation:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_GRAFTS" .. (self.grafts_negotiation.tooltip_suffix or "") )
        self.grafts_coin:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_COIN" )
        self.grafts_blaster:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_BLASTER" )
        self.grafts_mutators:SetToolTip( LOC"UI.MAIN_OVERLAY.VIEW_MUTATORS" )
        self.relationships_indicator:RefreshToolTip()
        self.btn_pause:SetToolTip( LOC"UI.MAIN_OVERLAY.PAUSE" )
    end


    for k, w in ipairs( self.main_bar_left_widgets.children ) do
        w:ShowToolTipOnFocus( cm == CONTROL_MODE.GAMEPAD )
    end

    for k, w in ipairs( self.main_bar_center_widgets.children ) do
        w:ShowToolTipOnFocus( cm == CONTROL_MODE.GAMEPAD )
    end

    for k, w in ipairs( self.main_bar_right_widgets.children ) do
        w:ShowToolTipOnFocus( cm == CONTROL_MODE.GAMEPAD )
    end

    for k, w in ipairs( self.nav_buttons.children ) do
        w:ShowToolTipOnFocus( cm == CONTROL_MODE.GAMEPAD )
    end


    self:Layout()
end

function MainOverlayTopBar:OnScreenModeChange( sm )
    MainOverlayTopBar._base.OnScreenModeChange( self, sm )

    self.main_bar_h = BAR_HEIGHT * self:GetLayoutScale()
    self.secondary_bar_h = self.main_bar_h*0.8
    local bar_scale = self:GetLayoutScale()
    self.main_bar_left_widgets:SetLayoutScale( bar_scale )
    self.main_bar_center_widgets:SetLayoutScale( bar_scale )
    self.main_bar_right_widgets:SetLayoutScale( bar_scale )
    self.secondary_bar_left_widgets:SetLayoutScale( bar_scale )
    self.secondary_bar_center_widgets:SetLayoutScale( bar_scale )
    self.secondary_bar_right_widgets:SetLayoutScale( bar_scale )
    self.secondary_bar_left_lower_widgets:SetLayoutScale( bar_scale )

    -- Set the max number of icons per screen scale
    if sm == SCREEN_MODE.MONITOR then
        self.grafts_battle:SetIconLimit( 6, function() self:Layout() end )
        self.grafts_negotiation:SetIconLimit( 6, function() self:Layout() end )
        self.grafts_perks:SetIconLimit( 6, function() self:Layout() end )
        self.grafts_mutators:SetIconLimit( 10, function() self:Layout() end )
        self.grafts_boons:SetIconLimit( 5, function() self:Layout() end )
        self.grafts_banes:SetIconLimit( 5, function() self:Layout() end )
    elseif sm == SCREEN_MODE.SMALL
        or sm == SCREEN_MODE.TV then
        self.grafts_battle:SetIconLimit( 6, function() self:Layout() end )
        self.grafts_negotiation:SetIconLimit( 6, function() self:Layout() end )
        self.grafts_perks:SetIconLimit( 4, function() self:Layout() end )
        self.grafts_mutators:SetIconLimit( 8, function() self:Layout() end )
        self.grafts_boons:SetIconLimit( 2, function() self:Layout() end )
        self.grafts_banes:SetIconLimit( 2, function() self:Layout() end )
    end
    
    self:RefreshSize()
    self:Layout()
end

function MainOverlayTopBar:GetHeight()
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() ) -- Refresh the height, in case another screen's requested it before this one
    return self.main_bar_h + self.secondary_bar_h
end

function MainOverlayTopBar:FadeInTopLeft( duration )
    duration = duration or 0.1
    self.top_left_shown = true
    self.main_bar_left_widgets:Show():AlphaTo( 1, duration, easing.inQuad )
    self.secondary_bar_left_widgets:Show():AlphaTo( 1, duration, easing.inQuad )
    self.secondary_bar_left_lower_widgets:Show():AlphaTo( 1, duration, easing.inQuad )
    return self
end

function MainOverlayTopBar:FadeOutTopLeft( duration )
    duration = duration or 0.1
    self.top_left_shown = false
    self.main_bar_left_widgets:AlphaTo( 0, duration, easing.inQuad, function() self.main_bar_left_widgets:Hide() end )
    self.secondary_bar_left_widgets:AlphaTo( 0, duration, easing.inQuad, function() self.secondary_bar_left_widgets:Hide() end )
    self.secondary_bar_left_lower_widgets:AlphaTo( 0, duration, easing.inQuad, function() self.secondary_bar_left_lower_widgets:Hide() end )
    return self
end

function MainOverlayTopBar:FadeInTopRight( duration )
    duration = duration or 0.1
    self.top_right_shown = true
    for k, v in ipairs( self.main_bar_right_widgets.children ) do
        v:AlphaTo( 1, duration, easing.inQuad )
    end
    self.secondary_bar_right_widgets:Show():AlphaTo( 1, duration, easing.inQuad )
    return self
end

function MainOverlayTopBar:FadeOutTopRight( duration, keep_pause_btn )
    duration = duration or 0.1
    self.top_right_shown = false
    -- Hide everything except the pause button if it's supposed to stay visible
    for k, v in ipairs( self.main_bar_right_widgets.children ) do
        if ( keep_pause_btn and v ~= self.right_corner_button ) or not keep_pause_btn then
            v:AlphaTo( 0, duration, easing.inQuad, function() v:Hide() end )
        end
    end
    self.secondary_bar_right_widgets:AlphaTo( 0, duration, easing.inQuad, function() self.secondary_bar_right_widgets:Hide() end )
    return self
end

function MainOverlayTopBar:GetLayoutScale()
    local sm = TheGame:FE():GetScreenMode()
    if sm == SCREEN_MODE.MONITOR then
        return 1
    elseif sm == SCREEN_MODE.TV then
        return 1.2
    elseif sm == SCREEN_MODE.SMALL then
        return 1.3
    end
    return 1
end

function MainOverlayTopBar:RefreshListeners()

    -- Set hotkeys
    local parent_screen = TheGame:FE():FindScreen( Screen.MainOverlay )
    if parent_screen then
        if SPECIAL_BUILD_TYPE ~= (1024+64) then parent_screen:RegisterHotkeyButton( Controls.Digital.FEEDBACK, self.btn_feedback ) end
        parent_screen:RegisterHotkeyButton( Controls.Digital.TOGGLE_NEGOTIATION_DECK, self.btn_negotiation )
        parent_screen:RegisterHotkeyButton( Controls.Digital.TOGGLE_BATTLE_DECK, self.btn_battle )
        parent_screen:RegisterHotkeyButton( Controls.Digital.TOGGLE_GRAFTS, self.grafts_battle )
        parent_screen:RegisterHotkeyButton( Controls.Digital.TOGGLE_COIN, self.grafts_coin )
        parent_screen:RegisterHotkeyButton( Controls.Digital.TOGGLE_BLASTER, self.grafts_blaster )
        parent_screen:RegisterHotkeyButton( Controls.Digital.TOGGLE_MUTATORS, self.grafts_mutators )
        parent_screen:RegisterHotkeyButton( Controls.Digital.TOGGLE_RELATIONSHIPS, self.relationships_indicator )
        --parent_screen:RegisterHotkeyButton( Controls.Digital.PAUSE, self.btn_pause )

        -- Check when the crew changes
        TheGame:GetEvents():ListenForEvent( "caravan_member_remove", self )
        TheGame:GetEvents():ListenForEvent( "caravan_member_add", self )
        TheGame:GetEvents():ListenForEvent( "mettle_gained", self )
        TheGame:GetEvents():ListenForEvent( "mettle_spent", self )
        TheGame:GetEvents():ListenForEvent( "score_change", self )
        TheGame:GetEvents():ListenForEvent( "FEAT_TRIGGERED", self )
    end
end

function MainOverlayTopBar:RemoveListeners()

    -- Cancel hotkeys
    local parent_screen = TheGame:FE():FindScreen( Screen.MainOverlay )
    if parent_screen then
        if SPECIAL_BUILD_TYPE ~= (1024+64) then parent_screen:UnregisterHotkeyButton( Controls.Digital.FEEDBACK, self.btn_feedback ) end
        parent_screen:UnregisterHotkeyButton( Controls.Digital.TOGGLE_NEGOTIATION_DECK, self.btn_negotiation )
        parent_screen:UnregisterHotkeyButton( Controls.Digital.TOGGLE_BATTLE_DECK, self.btn_battle )
        parent_screen:UnregisterHotkeyButton( Controls.Digital.TOGGLE_GRAFTS, self.grafts_battle )
        parent_screen:UnregisterHotkeyButton( Controls.Digital.TOGGLE_COIN, self.grafts_coin )
        parent_screen:UnregisterHotkeyButton( Controls.Digital.TOGGLE_BLASTER, self.grafts_blaster )
        parent_screen:UnregisterHotkeyButton( Controls.Digital.TOGGLE_MUTATORS, self.grafts_mutators )
        parent_screen:UnregisterHotkeyButton( Controls.Digital.TOGGLE_RELATIONSHIPS, self.relationships_indicator )
        -- parent_screen:UnregisterHotkeyButton( Controls.Digital.PAUSE, self.btn_pause )

        TheGame:GetEvents():RemoveListener( self )
    end
end

function MainOverlayTopBar:OnGlobalEvent( eventname, ... )
    if eventname =="caravan_member_add" or eventname == "caravan_member_remove" then
        self:Layout()
    end
    if eventname == "mettle_gained" or eventname == "mettle_spent" then
        self.mettle_indicator:Refresh()
            :SetShown( self.mettle_indicator:ShouldBeShown() )
        self:Layout()
    end
    if eventname == "score_change" then
        local old_score, new_score, score_event = table.unpack{...}
        self.high_score:ShowGain( old_score, new_score, score_event )
    end
    if eventname == "FEAT_TRIGGERED" then
        self.feats:Refresh()
        self:Layout()
    end
end

function MainOverlayTopBar:ToggleQuests()
    local overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    if overlay then
        self.quests_toggled = overlay:ToggleQuests()
        self.btn_quests:SetSelected(self.quests_toggled)
    end
    return self
end

function MainOverlayTopBar:OnScreenResize(w,h)
    self._base.OnScreenResize(self,w,h)

    self:RefreshSize()
end

function MainOverlayTopBar:RefreshSize()
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )

    self.main_bar_bg_focus:SetSize( self.w, self.main_bar_h )
    self.main_bar_bg:SetSize( self.w, self.main_bar_h )
    self.secondary_bar_bg:SetSize( self.w, self.secondary_bar_h )
    self.secondary_bar_bg_focus:SetSize( self.w, self.secondary_bar_h )

    self:SetPos( 0, -self.main_bar_h/2 )
    self:Layout()
    return self
end

function MainOverlayTopBar:UpdateImage()
    if self.hover or self.focus then
        self.main_bar_bg_focus:AlphaTo( 1, 0.3, easing.outQuad )
        self.secondary_bar_bg_focus:AlphaTo( 0.8, 0.2, easing.outQuad )
        if self.focus then
            self.main_bar_bg_focus:SizeTo( self.w, self.w, self.main_bar_h, 75, 0.3, easing.outQuad )
            self.main_bar_bg:SizeTo( self.w, self.w, self.main_bar_h, 75, 0.3, easing.outQuad )
            -- self:RunUpdater( Updater.Ease( function(v) self.left_corner_button:SetSize( nil, v ) end, self.main_bar_h, 70, 0.3, easing.outQuad ) )
            -- self:RunUpdater( Updater.Ease( function(v) self.right_corner_button:SetSize( nil, v ) end, self.main_bar_h, 70, 0.3, easing.outQuad ) )
            self:MoveTo( nil, -self.main_bar_h/2 - 5, 0.35, easing.outQuad )
            self.scaled = true
        end
    else
        self.main_bar_bg_focus:AlphaTo( 0, 0.3, easing.outQuad )
        self.secondary_bar_bg_focus:AlphaTo( 0, 0.2, easing.outQuad )
        if self.scaled then
            self.main_bar_bg_focus:SizeTo( self.w, self.w, 75, self.main_bar_h, 0.3, easing.outQuad )
            self.main_bar_bg:SizeTo( self.w, self.w, 75, self.main_bar_h, 0.3, easing.outQuad )
            -- self:RunUpdater( Updater.Ease( function(v) self.left_corner_button:SetSize( nil, v ) end, 70, self.main_bar_h, 0.3, easing.outQuad ) )
            -- self:RunUpdater( Updater.Ease( function(v) self.right_corner_button:SetSize( nil, v ) end, 70, self.main_bar_h, 0.3, easing.outQuad ) )
            self:MoveTo( nil, -self.main_bar_h/2, 0.35, easing.outQuad )
            self.scaled = false
        end
    end
    return self
end

function MainOverlayTopBar:OnGainFocus()
    MainOverlayTopBar._base.OnGainHover(self)
    self:UpdateImage()
    
    if TheGame:FE():IsGamepadMode() then
        self.main_overlay.quests_toggled = true
        self.main_overlay:RefreshToDoListVisibility()
    end
end

function MainOverlayTopBar:OnLoseFocus()
    MainOverlayTopBar._base.OnGainHover(self)
    self:UpdateImage()

    if TheGame:FE():IsGamepadMode() then
        self.main_overlay.quests_toggled = false
        self.main_overlay:RefreshToDoListVisibility()
    end
end

function MainOverlayTopBar:OnGainHover()
    MainOverlayTopBar._base.OnGainHover(self)
    self:UpdateImage()
end

function MainOverlayTopBar:OnLoseHover()
    MainOverlayTopBar._base.OnLoseHover(self)
    self:UpdateImage()
end

function MainOverlayTopBar:Refresh()
    if self.progress_indicator then
        self.progress_indicator:Refresh()
    end
    
    if TheGame:GetGameState() and TheGame:GetGameState():GetGameType() == GAME_TYPE.DAILY then
        self.feats:Refresh()
        self.daily_progress:Refresh()
        local score, tooltip = CalculateDailyScore( TheGame:GetGameState() )
        if not self.high_score:IsBusy() then
            self.high_score:SetValue( score )
        end
        self.high_score:SetToolTip( tooltip )
        self.high_score:Show()
    else
        self.high_score:Hide()
    end
    self.prestige_indicator:Refresh()
    self.left_corner_button:SetShown( self.prestige_indicator:IsShown() )
    self.money_indicator:Refresh()
    self.mettle_indicator:Refresh()
        :SetShown( self.mettle_indicator:ShouldBeShown() )
    self.health_indicator:Refresh()
    self.resolve_indicator:Refresh()
    self.relationships_indicator:RefreshToolTip()
    self.crew:Refresh()
    self:RefreshCardUpgrades()

    self.grafts_coin:Refresh()
    self.grafts_blaster:Refresh()
    self.grafts_battle:Refresh()
    self.relationships_indicator:Refresh()
    self.grafts_negotiation:Refresh()
    self.grafts_perks:Refresh()
    self.grafts_mutators:Refresh()
    self.grafts_boons:Refresh()
    self.grafts_banes:Refresh()

    if not TheGame:GetGameState() or not TheGame:GetGameState():GetPlayerAgent() then
        -- For tools, like the plax editor
        self.btn_negotiation:SetEnabled( false )
        self.btn_battle:SetEnabled( false )
        self.grafts_battle:SetEnabled( false )
        self.grafts_negotiation:SetEnabled( false )
        self.grafts_coin:SetEnabled( false )
        self.grafts_blaster:SetEnabled( false )
        self.grafts_mutators:SetEnabled( false )
        self.relationships_indicator:SetEnabled( false )
        -- self.btn_quests:SetEnabled( false )
        self.btn_pause:SetEnabled( false )

    else
        self.btn_battle:RefreshCount( TheGame:GetGameState():GetPlayerAgent().battler:GetCardCount() )
        self.btn_negotiation:RefreshCount( TheGame:GetGameState():GetPlayerAgent().negotiator:GetCardCount() )
    end

    self:Layout()
    return self
end

function MainOverlayTopBar:Layout()

    -------------------------------------------------------------------------------------------
    -- MAIN BAR
    -------------------------------------------------------------------------------------------
    self.main_bar_left_widgets:StackChildrenRow( SPACING.M1 )
        :LayoutBounds( "left", "center", self.main_bar_bg )
        :Offset( self.left_corner_button:IsShown() and 0 or SPACING.M1, 0 )
    self.main_bar_center_widgets:StackChildrenRow( SPACING.M1 )
        :LayoutBounds( "center", "center", self.main_bar_bg )

    -- RIGHT
    self.nav_buttons:StackChildrenRow( SPACING.M1/2 )
    self.main_bar_right_widgets:StackChildrenRow( SPACING.M1*0.5 )
        :LayoutBounds( "right", "bottom", self.main_bar_bg )
        :Offset( self.right_corner_button:IsShown() and 0 or -SPACING.M1, 0 )

    -------------------------------------------------------------------------------------------
    -- SECONDARY BAR
    -------------------------------------------------------------------------------------------
    self.secondary_bar_left_widgets:StackChildrenRow( SPACING.M1/2 )
        :LayoutBounds( "left", "bottom", self.secondary_bar_bg )
        :Offset( SPACING.M1, 0 )
    self.secondary_bar_center_widgets:StackChildrenRow( SPACING.M1/2 )
        :LayoutBounds( "center", "bottom", self.secondary_bar_bg )

    self.secondary_bar_right_widgets:StackChildrenRow( SPACING.M1/2 )
        :LayoutBounds( "right", "bottom", self.secondary_bar_bg )
        :Offset( -SPACING.M1, 0 )
    -- self.grafts_boons:LayoutBounds( "left", "below", self.secondary_bar_bg )
    --     :Offset( SPACING.M1, -4 )
    self.grafts_banes:LayoutBounds( "left", "below", self.grafts_boons )
        :Offset( 0, -4 )
    self.secondary_bar_left_lower_widgets:LayoutBounds( "left", "below", self.secondary_bar_bg )
        :Offset( SPACING.M1, -4 )
    self.secondary_bar:LayoutBounds( "left", "below", self.main_bar )

    return self
end

function MainOverlayTopBar:RefreshScenario( battle )
    self.fight_location_status:Refresh( battle )
    self:Layout()
    return self
end

function MainOverlayTopBar:RefreshNavButtons()
    local top_screen = TheGame:FE():GetTopScreen()
    for k, btn in ipairs( self.nav_buttons.children ) do
        btn:SetSelected( btn:IsInScreen( top_screen._class ) )
    end
    -- self.btn_quests:SetSelected(self.quests_toggled)
    return self
end

function MainOverlayTopBar:RefreshCardUpgrades()
    if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() then
        local in_time = .5
        local battle_ready = TheGame:GetGameState():GetPlayerAgent().battler:GetNumCardsReadyForUpgrade()
        if battle_ready == 0 or self.overlay_mode == OVERLAY_MODE.BATTLE or self.overlay_mode == OVERLAY_MODE.NEGOTIATION  then
            self.battle_upgrade_indicator:Hide()
            self.btn_battle:SetToolTipSuffix(nil)
        else
            local new = not self.battle_upgrade_indicator:IsShown() or self.battle_upgrade_indicator.num ~= battle_ready
            self.battle_upgrade_indicator:Refresh(battle_ready):Show()
                :SetPos( 14, 14 )
            if new then
                self.battle_upgrade_indicator:SetPos( 14, -5 )
                    :MoveTo( 14, 14, in_time, easing.outQuad )
            end
            self.btn_battle:SetToolTipSuffix( loc.format( LOC"UI.MAIN_OVERLAY.CARD_BTN_TT_SUFFIX", battle_ready ) )
        end

        local negotiation_ready = TheGame:GetGameState():GetPlayerAgent().negotiator:GetNumCardsReadyForUpgrade()
        if negotiation_ready == 0 or self.overlay_mode == OVERLAY_MODE.BATTLE or self.overlay_mode == OVERLAY_MODE.NEGOTIATION then
            self.negotiation_upgrade_indicator:Hide()
            self.btn_negotiation:SetToolTipSuffix(nil)
        else
            local new = not self.negotiation_upgrade_indicator:IsShown() or self.negotiation_upgrade_indicator.num ~= negotiation_ready
            self.negotiation_upgrade_indicator:Refresh(negotiation_ready):Show()
                :SetPos( 14, 14 )
            if new then
                self.negotiation_upgrade_indicator:SetPos( 14, -5 )
                    :MoveTo( 14, 14, in_time, easing.outQuad )
            end
            self.btn_negotiation:SetToolTipSuffix( loc.format( LOC"UI.MAIN_OVERLAY.CARD_BTN_TT_SUFFIX", negotiation_ready ) )
        end

        local battle_grafts_ready = TheGame:GetGameState():GetPlayerAgent().graft_owner:GetBattleGraftsReadyForUpgrade()
        if #battle_grafts_ready == 0 or self.overlay_mode == OVERLAY_MODE.BATTLE or self.overlay_mode == OVERLAY_MODE.NEGOTIATION then
            self.grafts_battle:SetToolTipSuffix(nil)
        else
            self.grafts_battle:SetToolTipSuffix( loc.format( LOC"UI.MAIN_OVERLAY.GRAFT_BTN_TT_SUFFIX", #battle_grafts_ready ) )
        end

        local negotiation_grafts_ready = TheGame:GetGameState():GetPlayerAgent().graft_owner:GetNegotiationGraftsReadyForUpgrade()
        if #negotiation_grafts_ready == 0 or self.overlay_mode == OVERLAY_MODE.BATTLE or self.overlay_mode == OVERLAY_MODE.NEGOTIATION then
            self.grafts_negotiation:SetToolTipSuffix(nil)
        else
            self.grafts_negotiation:SetToolTipSuffix( loc.format( LOC"UI.MAIN_OVERLAY.GRAFT_BTN_TT_SUFFIX", #negotiation_grafts_ready ) )
        end
    else
        -- For tools like the plax editor
        self.battle_upgrade_indicator:Hide()
        self.negotiation_upgrade_indicator:Hide()
        -- self.graft_upgrade_indicator:Hide()
    end
end


function MainOverlayTopBar:OnAdded()
    TheGame:GetEvents():ListenForEvent( "focus_changed", self, self.OnFocusChanged )
end

function MainOverlayTopBar:OnRemoved()
    TheGame:GetEvents():RemoveListener( self )
end


function MainOverlayTopBar:GetDefaultFocus()
    return self.btn_battle
end

function MainOverlayTopBar:AnimateHealth( new_health, old_health, new_max, old_max )
    self.health_indicator:AnimateHealth( new_health, old_health, new_max, old_max )
    return self
end

function MainOverlayTopBar:AnimateResolve( new_resolve, old_resolve, new_max, old_max )
    self.resolve_indicator:AnimateResolve( new_resolve, old_resolve, new_max, old_max )
    return self
end

function MainOverlayTopBar:RefreshProgress()
    if self.progress_indicator then
        self.progress_indicator:Refresh()
    end

    self:Layout()
    return self
end

function MainOverlayTopBar:GetBattleButton()
    return self.btn_battle
end

function MainOverlayTopBar:GetNegotiationButton()
    return self.btn_negotiation
end

function MainOverlayTopBar:GetNegotiationGraftsWidget()
    return self.grafts_negotiation
end

function MainOverlayTopBar:GetBattleGraftsWidget()
    return self.grafts_battle
end

function MainOverlayTopBar:GetBoonGraftsWidget()
    return self.grafts_boons
end

function MainOverlayTopBar:GetBaneGraftsWidget()
    return self.grafts_banes
end

function MainOverlayTopBar:GetPerksGraftsWidget()
    return self.grafts_perks
end

function MainOverlayTopBar:SetOverlayMode( overlay_mode )

    -- Save overlay mode
    self.overlay_mode = overlay_mode

    -- Untoggle button
    -- self.btn_quests:SetSelected(false)

    -- Set defaults
    self.crew_container:Show()
    if self.progress_container then
        self.progress_container:Show()
    end
    self.money_indicator:Show()
    self.mettle_indicator:SetShown( self.mettle_indicator:ShouldBeShown() )
    self.resolve_indicator:Show()
    self.health_indicator:Show()
    self.secondary_bar_left_lower_widgets:Show()
    -- self.installed_grafts:Hide()
    -- self.btn_graft:Show()

    -- if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() then
    --     self.btn_coin:SetShown( TheGame:GetGameState():GetPlayerAgent().graft_owner:GetMaxGrafts(GRAFT_TYPE.COIN) >= 1 )
    --     self.btn_blaster:SetShown( TheGame:GetGameState():GetPlayerAgent().graft_owner:GetMaxGrafts(GRAFT_TYPE.BLASTER) >= 1 )
    --     self.btn_mutators:SetShown( TheGame:GetGameState():GetPlayerAgent().graft_owner:CountGraftsByType(GRAFT_TYPE.MUTATOR) >= 1 )
    -- end
    -- self.btn_relationships:Show()
    self.btn_negotiation:Show()
    self.btn_battle:Show()
    self.fight_location_status:Hide()

    if overlay_mode == OVERLAY_MODE.PLAX then
    elseif overlay_mode == OVERLAY_MODE.RELATIONSHIP then
        self.secondary_bar_left_lower_widgets:Hide()
    elseif overlay_mode == OVERLAY_MODE.DECK then
        self.secondary_bar_left_lower_widgets:Hide()
    elseif overlay_mode == OVERLAY_MODE.MAP then
    elseif overlay_mode == OVERLAY_MODE.CONVO then
    elseif overlay_mode == OVERLAY_MODE.POPUP then
    elseif overlay_mode == OVERLAY_MODE.TRAVEL then
    elseif overlay_mode == OVERLAY_MODE.BATTLE then
        self.crew_container:Hide()
        if self.progress_container then
            self.progress_container:Hide()
        end
        self.money_indicator:Show()
        self.mettle_indicator:Hide()
        self.resolve_indicator:Hide()
        self.health_indicator:Hide()

        -- self.installed_grafts:Show()
        self.fight_location_status:Show()
    elseif overlay_mode == OVERLAY_MODE.NEGOTIATION then
        self.crew_container:Hide()
        if self.progress_container then
            self.progress_container:Hide()
        end
        self.money_indicator:Show()
        self.mettle_indicator:Hide()
        self.resolve_indicator:Hide()
        self.health_indicator:Hide()
        -- self.installed_grafts:Show()
    else
        -- Info mode
    end

    self:RefreshCardUpgrades()

    self:Layout()
end

function MainOverlayTopBar:SetFocus(from_child)
    if not from_child then
        local focus = self:GetDefaultFocus()
        if focus then focus:SetFocus() end
    else
        MainOverlayTopBar._base.SetFocus(self, from_child)
    end
end

------------------------------------------------------------------------------------------
-- 

local MainOverlayQuestButton = class( "Widget.MainOverlayQuestButton", Widget.Clickable )

function MainOverlayQuestButton:init( fn )
    MainOverlayQuestButton._base.init( self )

    local scale = 0.46
    self.open_button = self:AddChild( Widget.Image( engine.asset.Texture( "UI/topright_quests_button.tex" ) ) )
        :SetSize( 68*scale, 310*scale )
    self.open_button_icon = self:AddChild( Widget.Image( engine.asset.Texture( "UI/ic_quests_open.tex" ) ) )
        :SetSize( 22, 22 )
        :LayoutBounds( "center", "top", self.open_button )
        :Offset( 0, -9 )

    self.toggle_quests_fn = fn
    self:SetOnClickFn( function() self:ButtonClicked() end )

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
end

function MainOverlayQuestButton:ButtonClicked()
    if self.toggle_quests_fn then
        local open = self.toggle_quests_fn()
        self:SetState( open )
    end
end

function MainOverlayQuestButton:SetState( open )
    if open then
        -- The quests are open now
        self.open_button_icon:SetTexture( engine.asset.Texture( "UI/ic_notifications_close.tex" ) )
            :SetScale( -1, 1 )
        self:SetToolTip( nil )
    else
        -- The quests are closed
        self.open_button_icon:SetTexture( engine.asset.Texture( "UI/ic_quests_open.tex" ) )
            :SetScale( 1, 1 )
        self:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", LOC"UI.MAIN_OVERLAY.QUESTS", Controls.Digital.TOGGLE_CONTRACTS ) )
    end
    return self
end

function MainOverlayQuestButton:OnControlModeChange( cm, device_id )
    MainOverlayQuestButton._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", LOC"UI.MAIN_OVERLAY.QUESTS", Controls.Digital.TOGGLE_CONTRACTS ) )
    end
    if cm == CONTROL_MODE.TOUCH then
        self:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.QUESTS" ) )
    end
    if cm == CONTROL_MODE.GAMEPAD then
		
    end

end

------------------------------------------------------------------------------------------
-- 

local MainOverlayCornerButton = class( "Widget.MainOverlayCornerButton", Widget.Clickable )

function MainOverlayCornerButton:init( widget, height, left )
    MainOverlayCornerButton._base.init( self )

    self.left = left
    self.padding = 20
    self.w = 100
    self.h = height or 50
    local texture_w = 70
    local texture_h = 50
    self.panel = self:AddChild( Widget.Panel( engine.asset.Texture("UI/controls_top_bar_button.tex" ) ) )
        :SetSize( self.w, self.h )
        :SetScale( left and 1 or -1, 1 )
    self.content = self:AddChild( Widget() )
    self.main_widget = self.content:AddChild( widget )

    self:SetFocusable( false ) -- Makes it so that only child widgets can be focused, though this widget will still respond to focus gained and lost
    self:SetSize( self.w, self.h )
end

function MainOverlayCornerButton:SetControllerLabel( widget )
    self.controller_label = self.content:AddChild( widget )
    return self
end

function MainOverlayCornerButton:HideControllerLabel()
    self.controller_label:Hide()
        :LayoutBounds( "center", "center", self.main_widget )
    self:SetSize()
    return self
end

function MainOverlayCornerButton:ShowControllerLabel()
    self.controller_label:Show()
        :LayoutBounds( "after", "center", self.main_widget )
        :Offset( self.padding/2, 0 )
    self:SetSize()
    return self
end

function MainOverlayCornerButton:AddContent( child )
    self.content:AddChild( child )
    child:LayoutBounds( "before", "center", self.main_widget ):Offset( -self.padding/2 )
end

function MainOverlayCornerButton:SetSize( w, h )
    local content_w, content_h = self.content:GetSize()
    self.w = content_w + self.padding*2
    self.h = h or self.h or 50
    self.panel:SetSize( self.w, self.h )
    self.content:LayoutBounds( "center", "center", self.panel )
        :Offset( self.left and -4 or 4, 0 )
    return self
end

function MainOverlayCornerButton:UpdateImage()
    if self.focus then
        self:RunUpdater( Updater.Ease( function(v) self.panel:Bloom( v ) self.current_bloom = v end, self.current_bloom or 0, 0.1, 0.2, easing.outQuad ) )
    elseif self.current_bloom and self.current_bloom > 0 then
        self:RunUpdater( Updater.Ease( function(v) self.panel:Bloom( v ) self.current_bloom = v end, self.current_bloom or 0.1, 0, 0.2, easing.outQuad ) )
    end
    return self
end

function MainOverlayCornerButton:OnGainFocus()
    MainOverlayCornerButton._base.OnGainFocus(self)
    self.main_widget:OnGainFocus()
    self:UpdateImage()
end

function MainOverlayCornerButton:OnLoseFocus()
    MainOverlayCornerButton._base.OnLoseFocus(self)
    self.main_widget:OnLoseFocus()
    self:UpdateImage()
end

function MainOverlayCornerButton:OnGainHover()
    MainOverlayCornerButton._base.OnGainHover(self)
    self.main_widget:OnGainHover()
    self:UpdateImage()
end

function MainOverlayCornerButton:OnLoseHover()
    MainOverlayCornerButton._base.OnLoseHover(self)
    self.main_widget:OnLoseHover()
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- Generic icons container for the top bars

local MainOverlayIconContainer = class( "Widget.MainOverlayIconContainer", Widget.Clickable )

function MainOverlayIconContainer:init( height )
    MainOverlayIconContainer._base.init( self )

    self.padding = 6
    self.outer_padding = 6
    self.h = height or 50
    self.icon_size = self.h - self.padding*2
    self.bg_border = 16
    self.bg_border_scale = 0.5
    self.panel = self:AddChild( Widget.Panel( engine.asset.Texture("UI/mainoverlay_icons_bg.tex" ) ) )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
    self.content = self:AddChild( Widget() ) -- Contains the label icon, and the icons container
    self.icons = self.content:AddChild( Widget() ) -- Icons container
    self.overflow_icon = self.content:AddChild( Widget() )
    self.overflow_bg = self.overflow_icon:AddChild( Widget.Panel( engine.asset.Texture("UI/mainoverlay_icons_overflow.tex" ) ) )
        :SetSize( self.h*0.55, self.h*0.55 )
        :Bloom( 0.1 )
    self.overflow_label = self.overflow_icon:AddChild( Widget.Label( "body", FONT_SIZE.SMALL_TEXT, "+4" ) )
        :SetGlyphColour( UICOLOURS.DARK_BODYTEXT )
    self:SetFocusable( false ) -- Makes it so that only child widgets can be focused, though this widget will still respond to focus gained and lost
end

function MainOverlayIconContainer:SetFn( fn )
    self.click_fn = fn
    self:SetOnClickFn( self.click_fn )
    if self.icons and #self.icons.children > 0 then
        for k, widget in ipairs( self.icons.children ) do
            if widget.SetOnClickFn then widget:SetOnClickFn( self.click_fn ) end
        end
    end
    return self
end

function MainOverlayIconContainer:SetIconLimit( count, on_change_fn )
    self.icon_limit = count
    self.on_overflow_icons_shown_fn = on_change_fn
    self:Layout()
    return self
end

function MainOverlayIconContainer:Layout()
    self.icons:StackChildrenRow( self.padding )
    if self.icon_limit and not self.showing_overflow_icons then -- If we have a set limit on the number of icons to show, only show up to it
        local actual_limit = self.icon_limit+1 -- Otherwise, we'd hide the last icon with a [+1] instead of the icon itself
        for k, v in ipairs( self.icons.children ) do
            v:SetShown( k <= self.icon_limit or #self.icons.children <= actual_limit )
        end
        local extra_icons = #self.icons.children - self.icon_limit
        if #self.icons.children > actual_limit then
            self.overflow_label:SetText( loc.format( LOC"UI.MAIN_OVERLAY.ICONS_OVERFLOW", extra_icons ) )
            local w, h = self.overflow_label:GetSize()
            self.overflow_bg:SetSize( w+6, self.h*0.55 )
            self.overflow_label:LayoutBounds( "center", "center", self.overflow_icon ):Offset( 0, 0.5 )
            self.overflow_icon:Show()
        else
            self.overflow_icon:Hide()
        end
    else
        for k, v in ipairs( self.icons.children ) do
            v:Show()
        end
        self.overflow_icon:Hide()
    end
    self.content:StackChildrenRow( self.padding )
    local w, h = self.content:GetSize()
    w = w + self.outer_padding*3
    h = self.h
    local nineslice_w, nineslice_h = w + self.bg_border*2*self.bg_border_scale, h + self.bg_border*2*self.bg_border_scale
    self.panel:SetSize( nineslice_w, nineslice_h )
    self.content:LayoutBounds( "center", "center", self.panel )
        :Offset( 0, 2 )
end

function MainOverlayIconContainer:SetToolTipSuffix( suffix )
    self.tooltip_suffix = suffix
    return self
end

function MainOverlayIconContainer:UpdateImage()
    if self.focus or self.hover then
        self:RunUpdater( Updater.Ease( function(v) self.panel:Bloom( v ) self.current_bloom = v end, self.current_bloom or 0, 0.1, 0.2, easing.outQuad ) )
        if self.overflow_icon:IsShown() then
            -- If we have hidden icons, show those and relayout
            self.showing_overflow_icons = true
            self:Layout()
            if self.on_overflow_icons_shown_fn then self.on_overflow_icons_shown_fn(self) end
        end
    elseif self.current_bloom and self.current_bloom > 0 then
        self:RunUpdater( Updater.Ease( function(v) self.panel:Bloom( v ) self.current_bloom = v end, self.current_bloom or 0.1, 0, 0.2, easing.outQuad ) )
        if self.showing_overflow_icons then
            -- If we have hidden icons, hide them again and relayout
            self.showing_overflow_icons = false
            self:Layout()
            if self.on_overflow_icons_shown_fn then self.on_overflow_icons_shown_fn(self) end
        end
    end
    return self
end

function MainOverlayIconContainer:OnGainFocus()
    MainOverlayIconContainer._base.OnGainFocus(self)
    self:UpdateImage()
end

function MainOverlayIconContainer:OnLoseFocus()
    MainOverlayIconContainer._base.OnLoseFocus(self)
    self:UpdateImage()
end

function MainOverlayIconContainer:OnGainHover()
    MainOverlayIconContainer._base.OnGainHover(self)
    self:UpdateImage()
end

function MainOverlayIconContainer:OnLoseHover()
    MainOverlayIconContainer._base.OnLoseHover(self)
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- Displays graft slots

local MainOverlayGraftsContainer = class( "Widget.MainOverlayGraftsContainer", Widget.MainOverlayIconContainer )

function MainOverlayGraftsContainer:init( height, icon_size, graft_type )
    MainOverlayGraftsContainer._base.init( self, height )

    self.graft_type = graft_type
    self.icon_size = icon_size

    if self.graft_type == GRAFT_TYPE.COMBAT then
        self.panel:SetTintColour( UICOLOURS.FIGHT )
        self.overflow_icon:SetTintColour( UICOLOURS.FIGHT )
    elseif self.graft_type == GRAFT_TYPE.NEGOTIATION then
        self.panel:SetTintColour( UICOLOURS.NEGOTIATION )
        self.overflow_icon:SetTintColour( UICOLOURS.NEGOTIATION )
    elseif self.graft_type == GRAFT_TYPE.MUTATOR then
        self.panel:SetTintColour( UICOLOURS.MUTATOR )
        self.overflow_icon:SetTintColour( UICOLOURS.MUTATOR )
    elseif self.graft_type == GRAFT_TYPE.BLASTER then
        self.panel:SetTintColour( UICOLOURS.OBJECTIVE )
        self.overflow_icon:SetTintColour( UICOLOURS.OBJECTIVE )
    elseif self.graft_type == GRAFT_TYPE.COIN then
        self.panel:SetTintColour( UICOLOURS.MONEY )
        self.overflow_icon:SetTintColour( UICOLOURS.MONEY )
    elseif self.graft_type == GRAFT_TYPE.PERK then
        self.panel:SetTintColour( UICOLOURS.PERK )
        self.overflow_icon:SetTintColour( UICOLOURS.PERK )
    end

    self:Refresh()
end

function MainOverlayGraftsContainer:Refresh()
    -- Remove all the icons
    self.icons:DestroyAllChildren()

    -- Add new ones
    if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() and self.graft_type then
        local player_agent = TheGame:GetGameState():GetPlayerAgent()
        local graft_owner = player_agent:GetAspect("graft_owner")
        local max_grafts = graft_owner:GetMaxGrafts( self.graft_type )
        local grafts = graft_owner:GetGrafts( self.graft_type )
        for i,graft in ipairs(grafts) do
            if graft:IsHidden() then
                table.arrayremove(grafts, graft)
            end
        end
        if math.max( max_grafts, #grafts ) > 0 then
            for k = 1, math.max( max_grafts, #grafts ) do
                local widget = self.icons:AddChild( Widget.GraftSlot( self.graft_type, self.icon_size ) )
                    :HideFrame()
                    :Refresh( grafts[k] )
                    :RefreshXP()
                    :ShowToolTipOnFocus( true )
                if self.click_fn then widget:SetOnClickFn( self.click_fn ) end
                if self.tooltiplayoutfn then widget:SetToolTipLayoutFn( self.tooltiplayoutfn ) end
            end
            self:Show()
            self:Layout()
        else
            self:Hide()
        end
    else
        self:Hide()
    end
    return self
end

function MainOverlayGraftsContainer:FindWidget( graft )
    for k, v in ipairs( self.icons.children ) do
        if v.graft == graft then return v end
    end
    return 
end

------------------------------------------------------------------------------------------
-- Displays social graft slots

local MainOverlaySocialGraftsContainer = class( "Widget.MainOverlaySocialGraftsContainer", Widget.MainOverlayGraftsContainer )

function MainOverlaySocialGraftsContainer:init( height, icon_size, graft_good )
    MainOverlaySocialGraftsContainer._base.init( self, height, icon_size, GRAFT_TYPE.SOCIAL )

    self.graft_good = graft_good
    self.type_icon = self.content:AddChild( Widget.Image() )
        :SetSize( self.icon_size*0.7, self.icon_size*0.7 )
        :Bloom( 0.1 )
        :SendToBack()

    if self.graft_good then
        self.type_icon:SetTexture( engine.asset.Texture( "UI/opinion_icon_loved.tex") )
            :SetTintColour( UICOLOURS.BONUS )
        self.panel:SetTintColour( UICOLOURS.BONUS )
        self.overflow_icon:SetTintColour( UICOLOURS.BONUS )
    else
        self.type_icon:SetTexture( engine.asset.Texture( "UI/opinion_icon_hated.tex") )
            :SetTintColour( UICOLOURS.PENALTY )
        self.panel:SetTintColour( UICOLOURS.PENALTY )
        self.overflow_icon:SetTintColour( UICOLOURS.PENALTY )
    end

    self:Refresh()
end

function MainOverlaySocialGraftsContainer:Refresh()
    if not self.type_icon then return self end

    -- Remove all the icons
    self.icons:DestroyAllChildren()

    -- Add new ones
    if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() then
        local player_agent = TheGame:GetGameState():GetPlayerAgent()
        local graft_owner = player_agent:GetAspect("graft_owner")
        local grafts = graft_owner:GetGrafts( self.graft_type )
        if #grafts > 0 then
            for k, graft in ipairs( grafts ) do
                if graft:GetDef().is_boon and self.graft_good
                or not graft:GetDef().is_boon and not self.graft_good then
                    local widget = self.icons:AddChild( Widget.GraftSlot( self.graft_type, self.icon_size ) )
                        :HideFrame()
                        :Refresh( graft )
                        :ShowToolTipOnFocus( true )
                    if self.click_fn then widget:SetOnClickFn( self.click_fn ) end
                    if self.tooltiplayoutfn then widget:SetToolTipLayoutFn( self.tooltiplayoutfn ) end
                end
            end
            self:SetShown( self.icons.children and #self.icons.children > 0 )
            self:Layout()
        else
            self:Hide()
        end
    else
        self:Hide()
    end
    return self
end

------------------------------------------------------------------------------------------
-- Displays social graft slots

local MainOverlayPerksContainer = class( "Widget.MainOverlayPerksContainer", Widget.MainOverlayGraftsContainer )

function MainOverlayPerksContainer:init( height, icon_size, graft_good )
    MainOverlayPerksContainer._base.init( self, height, icon_size, GRAFT_TYPE.PERK )
    self.icon_size = self.h - self.padding
    self:Refresh()
end

function MainOverlayPerksContainer:Refresh()
    -- Remove all the icons
    self.icons:DestroyAllChildren()

    -- Add new ones
    if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() and self.graft_type then
        local player_agent = TheGame:GetGameState():GetPlayerAgent()
        local graft_owner = player_agent:GetAspect("graft_owner")
        local grafts = graft_owner:GetGrafts( self.graft_type )
        if #grafts > 0 then
            for k, graft in ipairs( grafts ) do
                local widget = self.icons:AddChild( Widget.GraftSlot( self.graft_type, self.icon_size ) )
                    :Refresh( grafts[k] )
                    :ShowToolTipOnFocus( true )
                if self.click_fn then widget:SetOnClickFn( self.click_fn ) end
                if self.tooltiplayoutfn then widget:SetToolTipLayoutFn( self.tooltiplayoutfn ) end
            end
            self:Show()
            self:Layout()
        else
            self:Hide()
        end
    else
        self:Hide()
    end
    return self
end

------------------------------------------------------------------------------------------
-- Displays relationship counts

local MainOverlayRelationshipsIndicator = class( "Widget.MainOverlayRelationshipsIndicator", Widget.MainOverlayIconContainer )

function MainOverlayRelationshipsIndicator:init( height, icon_size )
    MainOverlayRelationshipsIndicator._base.init( self, height )
        
    self.panel:SetTintColour( UICOLOURS.ACCENT )
    self.overflow_icon:SetTintColour( UICOLOURS.ACCENT )
    self:SetFocusable( true )

    self.padding = 5
    self.outer_padding = 7

    self.loved_container = self.icons:AddChild( Widget() )
    self.loved = self.loved_container:AddChild( Widget.Label( "body", icon_size*0.9 ) )
        :CenterH()
        :OverrideLineHeight( icon_size*0.6 )
        :SetGlyphColour(RELATIONSHIP_COLOURS[RELATIONSHIP.LOVED])
        :Bloom(0.1)
    self.liked_container = self.icons:AddChild( Widget() )
    self.liked = self.liked_container:AddChild( Widget.Label( "body", icon_size*0.9 ) )
        :CenterH()
        :OverrideLineHeight( icon_size*0.6 )
        :SetGlyphColour(RELATIONSHIP_COLOURS[RELATIONSHIP.LIKED])
        :Bloom(0.1)
    self.disliked_container = self.icons:AddChild( Widget() )
    self.disliked = self.disliked_container:AddChild( Widget.Label( "body", icon_size*0.9 ) )
        :CenterH()
        :OverrideLineHeight( icon_size*0.6 )
        :SetGlyphColour(RELATIONSHIP_COLOURS[RELATIONSHIP.DISLIKED])
        :Bloom(0.1)
    self.hated_container = self.icons:AddChild( Widget() )
    self.hated = self.hated_container:AddChild( Widget.Label( "body", icon_size*0.9 ) )
        :CenterH()
        :OverrideLineHeight( icon_size*0.6 )
        :SetGlyphColour(RELATIONSHIP_COLOURS[RELATIONSHIP.HATED])
        :Bloom(0.1)
    self.dead_container = self.icons:AddChild( Widget() )
    self.dead = self.dead_container:AddChild( Widget.Label( "body", icon_size*0.9 ) )
        :CenterH()
        :OverrideLineHeight( icon_size*0.6 )
        :SetGlyphColour(UICOLOURS.DEAD)
        :Bloom(0.1)

    self:Refresh()
end

function MainOverlayRelationshipsIndicator:SetFn( fn )
    self:SetOnClickFn( function() fn() end )
    return self
end

function MainOverlayRelationshipsIndicator:Refresh()
    if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() then
        local player_agent = TheGame:GetGameState():GetPlayerAgent()
        local social_connections = player_agent.social_connections
        local agents_by_relationship = {
            [RELATIONSHIP.LOVED] = {},
            [RELATIONSHIP.LIKED] = {},
            [RELATIONSHIP.DISLIKED] = {},
            [RELATIONSHIP.HATED] = {},
            ["DEAD"] = {}
        }

        for k, agent in pairs( TheGame:GetGameState().removed_agents or {} ) do
            if agent:IsSentient() and agent:IsDead() then 
                table.insert_unique( agents_by_relationship["DEAD"], agent ) 
            elseif not agent:IsDead() and agent:GetRelationship() ~= RELATIONSHIP.NEUTRAL then
                table.insert_unique( agents_by_relationship[agent:GetRelationship()], agent)
            end
        end

        if player_agent and player_agent.social_connections then
            for other_agent, agent_rel in pairs(player_agent.social_connections.relationships) do
                if agent_rel ~= RELATIONSHIP.NEUTRAL and not other_agent:IsDead() then
                    local has_effect = agent_rel == RELATIONSHIP.LOVED or agent_rel == RELATIONSHIP.HATED
                    if has_effect or not other_agent.removed then
                        table.insert_unique( agents_by_relationship[agent_rel], other_agent)
                    end
                end
            end
        end

        -- Save counts
        self.loved_count = #agents_by_relationship[RELATIONSHIP.LOVED]
        self.liked_count = #agents_by_relationship[RELATIONSHIP.LIKED]
        self.disliked_count = #agents_by_relationship[RELATIONSHIP.DISLIKED]
        self.hated_count = #agents_by_relationship[RELATIONSHIP.HATED]
        self.dead_count = #agents_by_relationship["DEAD"]

        self.loved:SetText( loc.format( LOC"UI.MAIN_OVERLAY.RELATIONSHIP_INDICATOR_LOVED", self.loved_count ) )
        self.liked:SetText( loc.format( LOC"UI.MAIN_OVERLAY.RELATIONSHIP_INDICATOR_LIKED", self.liked_count ) )
        self.disliked:SetText( loc.format( LOC"UI.MAIN_OVERLAY.RELATIONSHIP_INDICATOR_DISLIKED", self.disliked_count ) )
        self.hated:SetText( loc.format( LOC"UI.MAIN_OVERLAY.RELATIONSHIP_INDICATOR_HATED", self.hated_count ) )
        self.dead:SetText( loc.format( LOC"UI.MAIN_OVERLAY.RELATIONSHIP_INDICATOR_DEAD", self.dead_count ) )
            :SetShown( self.dead_count > 0 )
    
        if self.tooltiplayoutfn then 
            self.loved:SetToolTipLayoutFn( self.tooltiplayoutfn )
            self.liked:SetToolTipLayoutFn( self.tooltiplayoutfn )
            self.disliked:SetToolTipLayoutFn( self.tooltiplayoutfn )
            self.hated:SetToolTipLayoutFn( self.tooltiplayoutfn )
            self.dead:SetToolTipLayoutFn( self.tooltiplayoutfn )
        end

        self:Layout()
    else
        self:Hide()
    end
    return self
end

function MainOverlayRelationshipsIndicator:RefreshToolTip()
    local loved, liked, disliked, hated, dead = self:GetRelationshipCounts()
    local tt = loc.format( LOC"UI.MAIN_OVERLAY.VIEW_RELATIONSHIPS", loved, liked, disliked, hated, dead )
    if TheGame:FE():GetControlMode() == CONTROL_MODE.MOUSE_KEYBOARD then
        self:SetToolTip( loc.format( LOC"UI.MAIN_OVERLAY.HOTKEY_BINDING", tt, Controls.Digital.TOGGLE_RELATIONSHIPS ) )
    else
        self:SetToolTip( tt )
    end
end

function MainOverlayRelationshipsIndicator:GetRelationshipCounts()
    return self.loved_count or 0, self.liked_count or 0, self.disliked_count or 0, self.hated_count or 0, self.dead_count or 0
end

function MainOverlayRelationshipsIndicator:Layout()
    MainOverlayRelationshipsIndicator._base.Layout( self )
    self.content:LayoutBounds( "center", "center", self.panel )
        :Offset( 0, 0 )
end
