require "ui/widgets/screen"
require "ui/widgets/movieplayer"


local FADE_START = 4.0 -- Fade out the 'Press to skip' message after this many seconds.
local FADE_DURATION = 2.5 -- Duration of fade.
local SHOW_ESC_DURATION = 2.0 -- Duration of message to press ESC to skip

-------------------------------------------------------------------------------
-- Plays a full screen movie.

local MovieScreen = class("Screen.MovieScreen", Widget.Screen)

MovieScreen.CONTROL_MAP = 
{
}

function MovieScreen:init( filename, w, h, on_done_fn, include_audio )
    Widget.Screen.init(self)

    self.movie = self:AddChild( Widget.MoviePlayer( filename, w, h, include_audio ) )
        :SetAnchors( "center", "center" )
    self.movie:PlayMovie()

    local skip_key = TheGame:GetInput():GetBoundKeyString( Controls.Digital.SKIP )

    -- Click-to-skip widget
    self.next = self:AddChild( Widget() )
        :SetAnchors( "center", "bottom" )
    self.next.bg = self.next:AddChild( Widget.Image( engine.asset.Texture( "large/conversation_banner.tex") ) )
        :SetHiddenBoundingBox( true )
        :Bloom( 0.05 )
        :SetSize( 650, 250 )
    self.next.text = self.next:AddChild( Widget.Label() )
        :SetText(loc.format(LOC"UI.MOVIE_SCREEN.SKIP", Controls.Digital.SKIP))
        :SetFont("title")
        :SetFontSize( FONT_SIZE.SCREEN_SUBTITLE )
        :SetGlyphColour(UICOLOURS.SUBTITLE)
        :SetPos( 0, 1 )
        :Bloom( 0.05 )
    self.next:Hide()
    self.next:SetPos( 0, 200 )

    self.scrim_container = self:AddChild( Widget.Clickable() )
        :SetOnClickFn( function() self:ShowEscMessage() end )
        :SetAnchors( "fill", "fill" )
    self.scrim = self.scrim_container:AddChild( Widget.SolidBox( RES_X, RES_Y, 0x00ff0000 ) )

    self.elapsed = 0

    self:StartUpdating()
    self.on_done_fn = on_done_fn

    self:HideEscMessage()       -- Don't show the message at the start
end

function MovieScreen:OnControlModeChange( cm, device_id )
    MovieScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.TOUCH then
        self.next.text:SetText( LOC"UI.MOVIE_SCREEN.SKIP_TOUCH" )
        self.scrim_container:SetOnDoubleClickFn( function() self:Finish() end )
    else
        self.next.text:SetText( loc.format(LOC"UI.MOVIE_SCREEN.SKIP", Controls.Digital.SKIP) )
        self.scrim_container:SetOnDoubleClickFn( nil )
    end
end

function MovieScreen:OnScreenModeChange( sm )
    MovieScreen._base.OnScreenModeChange( self, sm )

    self.next:SetLayoutScale( LAYOUT_SCALE[sm] )
end

function MovieScreen:OnClose()
    
    self.movie:StopMovie()
    
    if self.on_done_fn then
        self.on_done_fn()
    end

end

function MovieScreen:OnUpdate(dt)
    if self.movie:IsDone() then
        self:Finish()
    end

    if self:IsShowingESCMessage() then
        self.showEscTime = self.showEscTime + dt
        if self.showEscTime > SHOW_ESC_DURATION then
            self.next:AlphaTo( 0, 0.4, easing.linear, function() self.next:Hide() end )
        end
    end
end

function MovieScreen:Finish()
    if not self.finishing then
        self.finishing = true
        self:GetFE():FadeTransition( function() self:GetFE():PopScreen( self ) end, 0.25 )
    end
end

function MovieScreen:IsShowingESCMessage()
    return self.showEscTime < SHOW_ESC_DURATION
end

function MovieScreen:ShowEscMessage()
    self.next:Show():AlphaTo( 1, 0.1 )
    self.showEscTime = 0.0
end

function MovieScreen:HideEscMessage()
    self.next:AlphaTo( 0, 0.4, easing.linear, function() self.next:Hide() end )
    self.showEscTime = SHOW_ESC_DURATION + 5.0
end

function MovieScreen:HandleControlDown(control)

    if self:IsShowingESCMessage() and (control:Has( Controls.Digital.MENU_CANCEL ) or control:Has( Controls.Digital.SKIP )) then
        self:Finish()
    end
    self:ShowEscMessage()

    return true -- sink all input
end
