require "act_util"
require "ui/widgets/screen"
require "ui/widgets/radialprogress"
require "ui/widgets/tooltip_savefile"

require "ui/widgets/sidebarblock"
require "ui/widgets/sidebarunlocks"
require "ui/widgets/sidebarmutators"
require "ui/widgets/sidebarperks"
require "ui/widgets/sidebardecks"
require "ui/widgets/sidebaroutfits"
require "ui/widgets/sidebarprestige"
require "ui/widgets/sidebarcustom"

--------------------------------------------------------------------------------
-- Dialog for acessing (loading/saving, depending on save_mode) save games

local NewGameScreen = class("Screen.NewGameScreen", Widget.Screen)

local SIDEBAR = MakeEnum{ "CHARACTERS", "SINGLE", "SAVEFILES", "LOADOUT", "NEGOTIATION", "BATTLE", "PRESTIGE", "UNLOCKS", "OUTFITS", "MUTATORS", "PERKS", "CUSTOM" }
local SCREEN_INSET = 60
local SIDEBAR_W = 500
local SIDEBAR_CHARACTERS_W = 350

local SIDEBAR_LEFT_PADDING = 60 -- The sidebar has this padding on the left, so that glows aren't visibly clipped off. And then the whole sidebar is moved left this same amount, so the contents display in the same spot


local function IsCharacterUpcoming(character_data)
    if character_data.data and character_data.data and character_data.data.in_progress and BRANCH == "dev" then
        return false
    end
    return character_data and character_data.data and character_data.data.upcoming ~= nil
end



local function IsActUpcoming(act_data)
    if act_data.data and act_data.data and act_data.data.in_progress and BRANCH == "dev" then
        return false
    end

    return act_data.data and act_data.data and act_data.data.upcoming
end



NewGameScreen.CONTROL_MAP = 
{
    {
        test = function(self) 
            return self.current_sidebar == self.sidebar_characters
        end,
        hint = function(self, left, right)
            table.insert(left, LOC"UI.CONTROLS.NAV")
        end
    },

    {
        control = Controls.Digital.GAMEPAD_X,
        test = function(self) 
            return self:CanStart() 
        end,
        fn = function(self)
            self:OnClickStart()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.START_GAME", Controls.Digital.GAMEPAD_X))
        end,
    },

    -- PERKS SIDEBAR
    {
        control = Controls.Digital.GAMEPAD_Y,
        test = function(self) 
            return self.current_sidebar == self.sidebar_perks
        end,
        fn = function(self)
            self.sidebar_perks:ClearAll()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.NEW_GAME_SCREEN.CONTROLS_PERKS_CLEAR_ALL", Controls.Digital.GAMEPAD_Y ))
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_LT,
        test = function(self) 
            return self.current_sidebar == self.sidebar_perks
        end,
        hint = function(self, left, right)
            table.insert(left, loc.format( LOC"UI.NEW_GAME_SCREEN.CONTROLS_PERKS_GRIFTS", Controls.Digital.GAMEPAD_LT))
        end,
        fn = function(self)
            self.sidebar_perks:ShowGrifts()    
            return true
        end,
    },

    -- BACK
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self)
            if self.sidebar_back_button:IsShown() then
                self:OnBackButton()
            else
                TheGame:FE():PopScreen()
            end
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.BACK", Controls.Digital.MENU_CANCEL))
        end,
    },
}


function NewGameScreen:init()
    NewGameScreen._base.init(self)

	self.is_popup = true
    self:SetupUnderlay( true )
	self:SetAnchors("center", "center")

	-- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    
    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*2.0
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Backing gradient for the sidebar
    self.sidebar_gradient = self:AddChild( Widget.Image( engine.asset.Texture("UI/newgame_sidebar_gradient.tex") ) )
    	:SetAnchors( "left", "fill" )
    	:SetSize( SIDEBAR_W*2, self.screen_h )

    -- Selectable character anims
    self.anims = self:AddChild( Widget.NewGameAnims( function( index ) self:OnCharacterSelected( index ) end ) )
        :SetAnchors( "center", "bottom" )

    -- Character info sidebar
    self.side_characters_root = self:AddChild(Widget():SetName("SIDEBAR_CHARACTERS_ROOT")):SetAnchors( "left", "top" )
    self.sidebar_characters = self.side_characters_root:AddChild( Widget.SidebarCharacters( SIDEBAR_CHARACTERS_W, function( index ) self:OnCharacterSelected( index ) end ) )

    -- Other sidebars
    self.side_root = self:AddChild(Widget():SetName("SIDEBAR_ROOT")):SetAnchors( "left", "top" )

    self.sidebar_savefiles = self.side_root:AddChild( Widget.SidebarSavefiles( SIDEBAR_W, function(act_data, story) self.act_data = act_data self.story_mode = story self:ShowSidebar( SIDEBAR.LOADOUT ) end ) )
    self.sidebar_loadout = self.side_root:AddChild( Widget.SidebarLoadout( SIDEBAR_W, function( sidebar ) self:ShowSidebar( sidebar ) end ) )
    self.sidebar_negotiation = self.side_root:AddChild( Widget.SidebarNegotiation( SIDEBAR_W ) )
    self.sidebar_battle = self.side_root:AddChild( Widget.SidebarBattle( SIDEBAR_W ) )
    self.sidebar_prestige = self.side_root:AddChild( Widget.SidebarPrestige( SIDEBAR_W ) )
    self.sidebar_unlocks = self.side_root:AddChild( Widget.SidebarUnlocks( SIDEBAR_W ) )
    self.sidebar_outfits = self.side_root:AddChild( Widget.SidebarOutfits( SIDEBAR_W ) )
    self.sidebar_mutators = self.side_root:AddChild( Widget.SidebarMutators( SIDEBAR_W ) )
    self.sidebar_perks = self.side_root:AddChild( Widget.SidebarPerks( SIDEBAR_W ) )
    self.sidebar_custom = self.side_root:AddChild( Widget.SidebarCustom( SIDEBAR_W, function( sidebar ) self:ShowSidebar( sidebar ) end ) )

	-- Left corner
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    -- Back button
    self.sidebar_back_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_BACK_BUTTON", function() self:OnBackButton() end ) )
    self.sidebar_back_button:SetIcon( global_images.close )
    self.sidebar_back_button:LayoutBounds( "left", "above", SCREEN_INSET, SCREEN_INSET )
    self.sidebar_back_button.shadow = self.sidebar_back_button:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
        :SetTintColour( 0x021A18ff )
        :SetTintAlpha( 1 )
        :SetHiddenBoundingBox( true )
        :SetSize( 600, 250 )
        :SendToBack()

    -- Menu button
    self.menu_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.NEW_GAME_SCREEN.CLOSE_BUTTON", function() TheGame:FE():PopScreen() end ) )
    self.menu_button:SetIcon( global_images.close )
    self.menu_button:LayoutBounds( "left", "above", SCREEN_INSET, SCREEN_INSET )

    -- Right corner
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    -- Start buttons
    self.start_button = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.NEW_GAME_SCREEN.START_BUTTON" ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -SCREEN_INSET, SCREEN_INSET )
        :SetFn( function() self:OnClickStart( self.start_button.story_mode ) end )
        :SetCta( true )
        :SetColour( UICOLOURS.QUEST )

    self:RefreshCharacterData()

    self:Layout()
end

function NewGameScreen:OnClose()
    NewGameScreen._base.OnClose( self )

    -- Flush dirty status in case anything was edited.
    TheGame:GetGameProfile():SaveSettings()
end

function NewGameScreen:RefreshCharacterData()

	-- List of act/character data
    self.character_data = shallowcopy( GetAllPlayerBackgrounds() )

    -- Show more character anims, for testing
    -- for k, v in ipairs( GetAllPlayerBackgrounds() ) do
    --     table.insert( self.character_data, v )
    -- end

    for k,character_bg in ipairs(self.character_data) do
        if IsCharacterUpcoming(character_bg) then
            self.anims:AddLocked( k, character_bg.data.player_agent, nil, character_bg )
        elseif TheGame:GetGameProfile():IsCharacterLocked(character_bg.id) then
            self.anims:AddLocked( k, character_bg.data.player_agent, nil, character_bg )
        else
            self.anims:AddAnim( k, character_bg.data.player_agent, nil, character_bg )
        end
    end

    -- Save the anim positions
    self.anims:SetDefaultPositions()

    self.anims:AnimateIn()
end

function NewGameScreen:OnGainTop()
    NewGameScreen._base.OnGainTop( self )

	if not self.selected_character then
		self:OnCharacterSelected( 1 )
	    self:ShowSidebar( SIDEBAR.CHARACTERS )
	end
end

function NewGameScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    elseif GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

-- function NewGameScreen:HandlePreControlDown( control, device )
--     if control:Has( Controls.Digital.SCROLL_DOWN ) then
--         self.anims:ScrollDown()
--     elseif control:Has( Controls.Digital.SCROLL_UP ) then
--         self.anims:ScrollUp()
--     end
-- end

function NewGameScreen:HandleGamepadSticks()
    if TheGame:FE():IsGamepadMode() then
        local _,_,rx,ry = TheGame:GetInput():GetActiveGamepadStickVals()
        local SPD = 50
        if math.abs(rx) > 0.1 then
            self.anims:Scroll(-SPD*rx)
        end
    end
    return self
end

function NewGameScreen:OnControlModeChange( cm, device_id )
    NewGameScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.bottom_left:Show()
        self.bottom_right:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.bottom_left:Show()
        self.bottom_right:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.bottom_left:Hide()
        self.bottom_right:Hide()
    end
end

function NewGameScreen:OnScreenModeChange( sm )

    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    self.sidebar_savefiles:SetHeight(self.screen_h)
    self.sidebar_loadout:SetHeight(self.screen_h)
    self.sidebar_negotiation:SetHeight(self.screen_h)
    self.sidebar_battle:SetHeight(self.screen_h)
    self.sidebar_prestige:SetHeight(self.screen_h)
    self.sidebar_unlocks:SetHeight(self.screen_h)
    self.sidebar_outfits:SetHeight(self.screen_h)
    self.sidebar_mutators:SetHeight(self.screen_h)
    self.sidebar_perks:SetHeight(self.screen_h)
    self.sidebar_custom:SetHeight(self.screen_h)

    NewGameScreen._base.OnScreenModeChange( self, sm )

    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:Layout()
end

function NewGameScreen:OnScreenResize(w, h)
    NewGameScreen._base.OnScreenResize(self, w, h)
    self:Layout()
end

function NewGameScreen:OnUpdate( dt )
    -- Hide the "Leave screen" button if we're not at the top level of the sidebar hierarchy
    self.menu_button:SetShown( self.current_sidebar == self.sidebar_characters )

    -- Handle gamepad scrolling
    self:HandleGamepadSticks()

    return NewGameScreen._base.OnUpdate( self, dt )
end

function NewGameScreen:GetDefaultFocus()
    if self.current_sidebar == self.sidebar_characters then
        return self.anims:GetDefaultFocus()
    elseif self._last_focus_widget and self._last_focus_widget:IsVisible() and not self._last_focus_widget.removed then
        return self._last_focus_widget
    else
        return self.current_sidebar and self.current_sidebar:GetDefaultFocus()
    end
end

function NewGameScreen:Layout()
	-- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Size up the grid
    local target_w = self.screen_w*2.0
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Size up the gradient
    self.sidebar_gradient:SetSize( SIDEBAR_W*2, self.screen_h )
    self.sidebar_gradient:SetPos( SIDEBAR_W, 0 )

    -- Set up the sidebar
    self.sidebar_characters:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
    self.sidebar_savefiles:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
    self.sidebar_loadout:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
    self.sidebar_negotiation:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
	self.sidebar_battle:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
	self.sidebar_prestige:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
	self.sidebar_unlocks:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
	self.sidebar_outfits:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
    self.sidebar_mutators:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
    self.sidebar_perks:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
    self.sidebar_custom:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
end

function NewGameScreen:OnBackButton()
    self._last_focus_widget = nil
    if self.current_sidebar == self.sidebar_unlocks
    or self.current_sidebar == self.sidebar_mutators then
        -- If we're on one of the custom-game sidebars, pressing back brings us to the Custom sidebar again
        self:ShowSidebar( SIDEBAR.CUSTOM )
    elseif self.current_sidebar == self.sidebar_savefiles then
        -- If we're on the savefiles list, go to character select
        self.sidebar_characters:SetSaveFileCount( self.sidebar_savefiles:GetSaveFileCount( self.character_data[self.selected_character] ) )
        self:ShowSidebar( SIDEBAR.CHARACTERS )

    elseif self.current_sidebar == self.sidebar_loadout then
        self.sidebar_loadout.last_btn = nil
        if self.sidebar_savefiles:HasSaveFiles( self.character_data[self.selected_character] )
        or self.sidebar_savefiles:HasMultipleGameModes( self.character_data[self.selected_character] ) then
            -- We show the savefiles sidebar whether there are savefiles to show, or more than one game mode to choose
            self:ShowSidebar( SIDEBAR.SAVEFILES )
        else
            self:ShowSidebar( SIDEBAR.CHARACTERS )
        end
    else
        -- If we're on the other sidebars, go to loadout
        self:ShowSidebar( SIDEBAR.LOADOUT )
    end
end

function NewGameScreen:OnCustomGameButton()
    self:ShowSidebar( SIDEBAR.CUSTOM )
end

function NewGameScreen:ShowSingleCharacter( index )
    self:ShowSidebar( SIDEBAR.SAVEFILES )
end

function NewGameScreen:OnCharacterSelected( index )
    if is_instance( self.current_sidebar, Widget.SidebarSavefiles ) then
        return
    end

    if self.selected_character == index
        and not TheGame:GetGameProfile():IsCharacterLocked(self.character_data[self.selected_character].id)
        and not IsCharacterUpcoming(self.character_data[self.selected_character]) then
            -- Second click on the same character. Show loadout
            self.anims:SelectIndex( self.selected_character )
            self:ShowSingleCharacter( self.selected_character )
    else
        -- First click on this character. Select it
        self.selected_character = index
        self.anims:SelectIndex( self.selected_character )
        self.sidebar_characters:SetCharacterIndex( self.selected_character )
        self.sidebar_characters:SetSaveFileCount( self.sidebar_savefiles:GetSaveFileCount( self.character_data[self.selected_character] ) )
        self:ShowSidebar( SIDEBAR.CHARACTERS )
    end
	self:Layout()
end

function NewGameScreen:SelectPreviousCharacter()
	local index = self.selected_character or 1
	index = index - 1
	if index < 1 then
		index = #self.character_data
	end
	self:OnCharacterSelected( index )
end

function NewGameScreen:SelectNextCharacter()
	local index = self.selected_character or 1
	index = index + 1
	if index > #self.character_data then
		index = 1
	end
	self:OnCharacterSelected( index )
end

function NewGameScreen:CanStart()
    return self.can_start
end

function NewGameScreen:_DoStart()
    local character_data = self.character_data[self.selected_character]
    local character_id = character_data.id
    local act_id = self.act_data.id

    local character_config = {}
    character_config.outfit = TheGame:GetGameProfile():GetSelectedOutfit( character_id )
    local toggled_off_unlocks = TheGame:GetGameProfile():GetToggledOffUnlocks( character_id )

    local options
    if self.story_mode then
        options = GameState.CreateOptions( act_id, 0, character_config, toggled_off_unlocks, true )
    else
        local advancement_level = TheGame:GetGameProfile():GetSelectedAdvancement( act_id )
        TheGame:GetGameProfile():SetHighestPlayedAdvancement( act_id, advancement_level )
        options = GameState.CreateOptions( act_id, advancement_level, character_config, toggled_off_unlocks, false )
    end

    options.mutators = TheGame:GetGameProfile():HasUnlock("MUTATORS_UNLOCKED") and TheGame:GetGameProfile():GetSelectedMutators( character_id )
    options.perks = TheGame:GetGameProfile():GetSelectedPerks( character_id )

    AUDIO:PlayEvent( "event:/ui/main/gen/start_game" )

    TheGame:GetGameProfile():AcquireUnlock("DAILY_UNLOCKED")

    local save_name = "slot_" .. act_id .. "_" .. tostring( os.time() )
    TheGame:FE():FadeTransition( function()
        TheGame:StartNewGame( options, save_name ) 
    end, 0.5 )
end

local function DoFreeAdvancementOffer(act_id, fn)
    local screen = TheGame:FE():PushScreen( Screen.YesNoPopup( LOC"UI.SKIP_P0_TITLE", LOC"UI.SKIP_P0_BODY" ))
        :SetYesText(LOC"UI.SKIP_P0_OPT1", LOC"UI.SKIP_P0_TT1")
        :SetCancelText(LOC"UI.SKIP_P0_OPT2", LOC"UI.SKIP_P0_TT2")
        :SetFn( function(v)
                if v ~= Screen.YesNoPopup.YES then 
                    TheGame:GetGameProfile():AcquireAdvancement(act_id, 0)
                    TheGame:GetGameProfile():UpdateSelectedAdvancement( act_id, 1)
                end
                if fn then
                    fn()
                end
            end)
    screen.cancelbutton:SetIcon( global_images.accept )        
end

function NewGameScreen:OnClickStart()
    if self:CanStart() then
        local should_ask_about_p1 = not self.story_mode and TheGame:GetGameProfile():GetAdvancementLevel(self.act_data.id) == 0 
        if should_ask_about_p1 then
            DoFreeAdvancementOffer(self.act_data.id, function() self:_DoStart() end)
        else
            self:_DoStart()
        end
    end
end

function NewGameScreen:ShowSidebar( sidebar )
    if not self.selected_character then
        return
    end

	self.sidebar_characters:Refresh(self.character_data[self.selected_character], self.act_data, self.story_mode)

    if sidebar == SIDEBAR.LOADOUT
    and self.act_data ~= nil
    and not TheGame:GetGameProfile():IsActLocked(self.act_data.id) then
        self.start_button:Show()
        self.start_button.story_mode = self.story_mode
        self.can_start = true
    else
        self.start_button:Hide()
        self.can_start = false
    end

	-- Show the back button and background gradient for all the sidebars other than the main one
	if sidebar == SIDEBAR.CHARACTERS then
		self.sidebar_gradient:AlphaTo( 0, 0.3, easing.outQuad, function() self.sidebar_back_button:Hide() end  )
        self.start_button:Hide()
        self.menu_button:Show()
        self.sidebar_back_button:Hide()
        -- Show all the character anims
        self.anims:ShowAll()
	else
		self.sidebar_gradient:Show():AlphaTo( 1, 0.3, easing.outQuad )
        self.menu_button:Hide()
        self.sidebar_back_button:Show()
        -- Show only the current character
        self.anims:SetSidebarMode( sidebar ~= SIDEBAR.LOADOUT )
            :ShowSingle( self.selected_character )
	end

	local new_sidebar
    if sidebar == SIDEBAR.CHARACTERS then
        new_sidebar = self.sidebar_characters
    elseif sidebar == SIDEBAR.SAVEFILES then
        new_sidebar = self.sidebar_savefiles
    elseif sidebar == SIDEBAR.LOADOUT then
        new_sidebar = self.sidebar_loadout
    elseif sidebar == SIDEBAR.NEGOTIATION then
        new_sidebar = self.sidebar_negotiation
    elseif sidebar == SIDEBAR.BATTLE then
        new_sidebar = self.sidebar_battle
    elseif sidebar == SIDEBAR.PRESTIGE then
        new_sidebar = self.sidebar_prestige
    elseif sidebar == SIDEBAR.UNLOCKS then
        new_sidebar = self.sidebar_unlocks
    elseif sidebar == SIDEBAR.OUTFITS then
        new_sidebar = self.sidebar_outfits
    elseif sidebar == SIDEBAR.MUTATORS then
        new_sidebar = self.sidebar_mutators
    elseif sidebar == SIDEBAR.PERKS then
        new_sidebar = self.sidebar_perks
    elseif sidebar == SIDEBAR.CUSTOM then
        new_sidebar = self.sidebar_custom
    end
    if new_sidebar ~= self.current_sidebar then
        if self.current_sidebar then self.current_sidebar:AnimateOut() end
        self.current_sidebar = new_sidebar
        new_sidebar:Refresh( self.character_data[self.selected_character], self.act_data, self.anims:GetSelected(), self.story_mode )
        new_sidebar:PrepareAnimateIn()
        new_sidebar:LayoutBounds( "left", "top", SCREEN_INSET-SIDEBAR_LEFT_PADDING, 0 )
        new_sidebar:AnimateIn()
        -- Hide the anims if this sidebar requires it
        self.anims:SetShown( new_sidebar:IsAnimVisible() )
    end

	-- Auto-select first option if using controller
	if TheGame:FE():IsGamepadMode() then
		local focus_widget = self:GetDefaultFocus()
		if focus_widget then focus_widget:SetFocus() end
	end
end

function NewGameScreen:GetCurrentSidebarWidget()
	return self.current_sidebar or self.sidebar_main
end


------------------------------------------------------------------------------------------
-- A sidebar showing the bio for the selected character

local SidebarCharacters = class("Widget.SidebarCharacters", Widget.SidebarBlock)


function SidebarCharacters:init( width, select_character_fn )
    SidebarCharacters._base.init(self, width )

    self.select_character_fn = select_character_fn
    self.savefile_counter = self.scrolling_contents:AddChild( Widget.SaveFileCounter( self.sidebar_width, 44, self.select_character_fn ) )
end

function SidebarCharacters:CheckMouseHover(x,y,trace)
    return false
end

function SidebarCharacters:SetCharacterIndex( character_index )
    self.savefile_counter:SetCharacterIndex( character_index )
    return self
end

function SidebarCharacters:SetSaveFileCount( count )
    self.savefile_counter:SetSaveFileCount( count )
    return self
end

function SidebarCharacters:Refresh( character_data, act_data, story_mode )
    SidebarCharacters._base.Refresh( self, character_data )

    if IsCharacterUpcoming(self.character_data) then
        -- This is an upcoming character. Show as locked
        self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_UPCOMING_TITLE" ):SetGlyphColour( UICOLOURS.SUBTITLE )
        self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_UPCOMING_SUBTITLE" ):SetGlyphColour( UICOLOURS.TITLE )
        self.text:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_UPCOMING_TEXT" )
        self.savefile_counter:Hide()
    elseif TheGame:GetGameProfile():IsCharacterLocked(self.character_data.id) then
        -- This is a character the player hasn't unlocked yet. Show as locked
        self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_LOCKED_TITLE" ):SetGlyphColour( UICOLOURS.SUBTITLE )
        self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_LOCKED_SUBTITLE" ):SetGlyphColour( UICOLOURS.TITLE )
        self.text:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_LOCKED_TEXT" )
        self.savefile_counter:Hide()
    else
        -- This is a regular character, with existing save files
        self.title:SetText( loc.toupper( character_data:GetLocalizedName() ) ):SetGlyphColour( UICOLOURS.SUBTITLE )
        self.subtitle:SetText( character_data:GetLocalizedTitle() ):SetGlyphColour( UICOLOURS.TITLE )
        self.text:SetText( character_data:GetLocalizedDesc() ):SetGlyphColour( UICOLOURS.TITLE )
        self.savefile_counter:Refresh( character_data )
    end
    -- Layout
    self:Layout()
end

function SidebarCharacters:Layout()
    SidebarCharacters._base.Layout( self )
end

function SidebarCharacters:OnScreenModeChange( sm )
    SidebarCharacters._base.OnScreenModeChange( self, sm )
end

------------------------------------------------------------------------------------------
-- A sidebar showing the savefiles available for the selected character

local SidebarSavefiles = class("Widget.SidebarSavefiles", Widget.SidebarBlock)

function SidebarSavefiles:init(  width, on_select_fn )
    SidebarSavefiles._base.init(self,width)

    self.on_select_fn = on_select_fn

    -- Load save files
    TheGame:ListSaveGames( function( save_data ) self:PopulateSaveFiles( save_data ) end )
end

function SidebarSavefiles:OnAdded()
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
end

function SidebarSavefiles:OnControlModeChange( cm, device_id )
    SidebarSavefiles._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.TOUCH then
        self.info_text = LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_SAVEFILES_INFO_TOUCH"
    else
        self.info_text = ""
    end
end

function SidebarSavefiles:PopulateSaveFiles( save_data )
    self.save_slots = self.save_slots or {}

    -- Add save files to list
    local save_files_list = {}
    for name, data in pairs( save_data ) do
        if data.data.time then
            data.name = name
            table.insert( save_files_list, data )
        else
            print( "\tSave has no time:", tostr(data))
        end
    end
    -- Sort list by savefile date
    table.stable_sort(save_files_list, function(a, b) return a.data.time > b.data.time end)

  	-- Go through all the save files, and organize them by act_id
    for i, header in ipairs( save_files_list ) do
        
        local act_id = header.data.options and header.data.options.start_act
        local act_data = GetPlayerActData( act_id )
        if act_data == nil then
            print( "\tSave file missing act_data:", header.name, tostring(act_id))
        else
            local character_id = act_data.background.id            
            local skip = act_data.data.game_type == GAME_TYPE.DAILY 

            if character_id and not skip then
                self.save_slots[character_id] = self.save_slots[character_id] or {}
                if string.match( header.name, "^slot_" ) and not string.match( header.name, "_rollback$" ) then
                    table.insert( self.save_slots[character_id], header )
                elseif TheGame:GetLocalSettings().DEBUG then
                    header.is_dev = true
                    table.insert( self.save_slots[character_id], header )
                end
            end
        end
    end

end

function SidebarSavefiles:OnDeleteSlot( slot )
    TheGame:FE():PushScreen( Screen.YesNoPopup( TheGame:Str"UI.DIALOGS.ABANDON_TITLE", TheGame:Str"UI.DIALOGS.ABANDON_BODY" ) )
        :SetFn( function(v)
            if v == Screen.YesNoPopup.YES then 
                TheGame:DeleteSave( slot.save_data.name, 
                    function( deleted, ... )
                        local pos = table.find( self.save_slots[self.character_data.id], slot.save_data )
                        table.remove( self.save_slots[self.character_data.id], pos )
                        slot.enabled = false
                        AUDIO:PlayEvent( "event:/ui/save_slot_menu/save_file_deleted" )
                        local idx = self.scrolling_contents:IndexOf( slot )
                        slot:Reparent( self )
                        local pos_x, pos_y = slot:GetPos()
                        slot:MoveTo( pos_x-40, pos_y, 0.15, easing.outQuad )
                        slot:AlphaTo( 0, 0.15, easing.outQuad,
                            function()
                                slot:Remove()
                                -- Re-layout remaining buttons
                                for k, w in ipairs( self.scrolling_contents.children ) do
                                    local p_x, p_y = w:GetPos()
                                    w.old_x = p_x
                                    w.old_y = p_y
                                end
                                self.scrolling_contents:StackChildren( SPACING.M1 )
                                self.scroll_panel:RefreshView()
                                for k, w in ipairs( self.scrolling_contents.children ) do
                                    local p_x, p_y = w:GetPos()
                                    w.new_x = p_x
                                    w.new_y = p_y
                                    w:SetPos( w.old_x, w.old_y )
                                    w:MoveTo( w.new_x, w.new_y, 0.08, easing.outQuad )
                                end
                                if idx and TheGame:FE():GetControlMode() == CONTROL_MODE.GAMEPAD then
                                    local focus = self.scrolling_contents.children[ idx ] or self.scrolling_contents.children[ idx - 1 ]
                                    if focus then
                                        focus:SetFocus()
                                    end
                                end
                            end )
                    end 
                )
            end
            end
        )
end

function SidebarSavefiles:Refresh( character_data )
    SidebarSavefiles._base.Refresh( self, character_data )

    -- Set text
    self.title:SetText( loc.toupper( character_data:GetLocalizedName()) ):SetGlyphColour( UICOLOURS.SUBTITLE )
    self.subtitle:SetText( character_data:GetLocalizedTitle() ):SetGlyphColour( UICOLOURS.TITLE )
    self.text:Hide()
    self.info:SetText( self.info_text or "" )

    -- Remove old save files
    self.scrolling_contents:DestroyAllChildren()
    -- Add new run options
    self.scrolling_contents:AddChild( Widget.Label( "title", FONT_SIZE.BODY_TEXT, LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_NEW_RUN" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize( self.sidebar_width )
        :SetWordWrap( true )
        :Bloom( 0.05 )
    for k, act in ipairs( self.character_data.acts ) do
        
        if act.data.game_type ~= GAME_TYPE.DAILY then
            local widget = Widget.NewActButton( self.sidebar_width+200, act:GetLocalizedName() )
                :SetOnClickFn( function() self.on_select_fn(act, false) end )
                :SetFocusDelta( 10 )
                :ShowArrow()
                :Refresh( self.character_data, act )
            
            self.scrolling_contents:AddChild(widget)
            if act.data.story_mode then
                local widget = Widget.NewActButton( self.sidebar_width+200, act:GetLocalizedName() )
                    :SetOnClickFn( function() self.on_select_fn(act, true) end )
                    :SetFocusDelta( 10 )
                    :ShowArrow()
                    :Refresh( self.character_data, act, true )
                
                self.scrolling_contents:AddChild(widget)
            end
        end
    end

    -- List this act's save files
    if self.save_slots[character_data.id] then
        local savefiles_title
        for k, save_data in ipairs( self.save_slots[character_data.id] ) do
            -- Add a title if there isn't one yet
            if not savefiles_title then
                savefiles_title = self.scrolling_contents:AddChild( Widget.Label( "title", FONT_SIZE.BODY_TEXT, LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_CONTINUE_RUN" ) )
                    :SetGlyphColour( UICOLOURS.SUBTITLE )
                    :SetAutoSize( self.sidebar_width )
                    :SetWordWrap( true )
                    :Bloom( 0.05 )
            end
            -- Add this savefile
            local slot = self.scrolling_contents:AddChild( Widget.SaveSlotButton( character_data.id, self.sidebar_width - 24, 70, save_data ) )
                :LayoutBounds( "left", "below" )
                :Offset( 0, -10 )
            slot:SetDeleteFn( function() self:OnDeleteSlot( slot ) end )
        end
    end

    -- Layout
    self:Layout()
end

function SidebarSavefiles:Layout()

    -- Layout the savefiles
    self.scrolling_contents:StackChildren( SPACING.M1 )

    SidebarSavefiles._base.Layout( self )
end

function SidebarSavefiles:OnScreenModeChange( sm )
    SidebarSavefiles._base.OnScreenModeChange( self, sm )

end

function SidebarSavefiles:HasSaveFiles( character_data )
    return self.save_slots[character_data.id] and #self.save_slots[character_data.id] > 0
end

function SidebarSavefiles:GetSaveFileCount( character_data )
    return self.save_slots[character_data.id] and #self.save_slots[character_data.id] or 0
end

function SidebarSavefiles:HasMultipleGameModes( character_data )
    local count = 0
    for k, act in ipairs( character_data.acts ) do
        if not TheGame:GetGameProfile():IsActLocked( act.id )
        and not IsActUpcoming( act ) 
        and not act.data.game_type == GAME_TYPE.DAILY then
            count = count + 1
        end
    end
    return count > 1
end

function SidebarSavefiles:GetDefaultFocus()
    return self.scrolling_contents and self.scrolling_contents.children and self.scrolling_contents.children[2]
end

------------------------------------------------------------------------------------------
-- A sidebar showing the loadout options for the selected character

local SidebarLoadout = class("Widget.SidebarLoadout", Widget.SidebarBlock)

local BUTTON_DISTANCE_X = 550
local BUTTON_DISTANCE_X_MIDDLE = 50
local BUTTON_DISTANCE_Y = 180

function SidebarLoadout:init( width, sidebar_selected_fn )
    SidebarLoadout._base.init(self,width)

    self.button_distance_x = BUTTON_DISTANCE_X
    self.button_distance_x_middle = BUTTON_DISTANCE_X_MIDDLE
    self.button_distance_y = BUTTON_DISTANCE_Y
    self.button_width = 400

    self.center_buttons = self:AddChild( Widget() )
        :SetAnchors( "center", "center" )
        :SetFocusLock( true )

    self.health_indicator = self:AddChild( Widget.HealthIndicator( 42, 42 ) )
    self.resolve_indicator = self:AddChild( Widget.ResolveIndicator( 42, 42 ) )

    self.btn_negotiation = self.center_buttons:AddChild( Widget.SidebarButton( self.button_width, SIDEBAR.NEGOTIATION, function() self.last_btn = self.btn_negotiation AUDIO:PlayEvent( "event:/ui/new_game_menu/negotiation_deck_preview" ) sidebar_selected_fn( SIDEBAR.NEGOTIATION ) end ) )
        :SetFacingLeft( true )
        :SetTintAlpha( 0 )
        :Hide()
    self.btn_battle = self.center_buttons:AddChild( Widget.SidebarButton( self.button_width, SIDEBAR.BATTLE, function() self.last_btn = self.btn_battle AUDIO:PlayEvent( "event:/ui/new_game_menu/battle_deck_preview" ) sidebar_selected_fn( SIDEBAR.BATTLE ) end ) )
        :SetFacingLeft( false )
        :SetTintAlpha( 0 )
        :Hide()
    self.btn_prestige = self.center_buttons:AddChild( Widget.SidebarButton( self.button_width, SIDEBAR.PRESTIGE, function() self.last_btn = self.btn_prestige sidebar_selected_fn( SIDEBAR.PRESTIGE ) end ) )
        :SetFacingLeft( true )
        :SetTintAlpha( 0 )
        :Hide()
    self.btn_outfits = self.center_buttons:AddChild( Widget.SidebarButton( self.button_width, SIDEBAR.OUTFITS, function() self.last_btn = self.btn_outfits sidebar_selected_fn( SIDEBAR.OUTFITS ) end ) )
        :SetFacingLeft( false )
        :SetTintAlpha( 0 )
        :Hide()
    self.btn_perks = self.center_buttons:AddChild( Widget.SidebarButton( self.button_width, SIDEBAR.PERKS, function() self.last_btn = self.btn_perks AUDIO:PlayEvent( "event:/ui/new_game_menu/open_mutators_list" ) sidebar_selected_fn( SIDEBAR.PERKS ) end ) )
        :SetFacingLeft( true )
        :SetTintAlpha( 0 )
        :Hide()
    self.btn_custom = self.center_buttons:AddChild( Widget.SidebarButton( self.button_width, SIDEBAR.CUSTOM, function() self.last_btn = self.btn_custom AUDIO:PlayEvent( "event:/ui/new_game_menu/open_mutators_list" ) sidebar_selected_fn( SIDEBAR.CUSTOM ) end ) )
        :SetFacingLeft( false )
        :SetTintAlpha( 0 )
        :Hide()

    -- Setup focus direction overrides
    self.btn_negotiation:SetFocusDir( "right", self.btn_battle, true )
    self.btn_perks:SetFocusDir( "right", self.btn_outfits, true )
    self.btn_prestige:SetFocusDir( "right", self.btn_custom, true )
    
    self.btn_negotiation:SetFocusDir( "down", self.btn_perks, true )
    self.btn_perks:SetFocusDir( "down", self.btn_prestige, true )
    self.btn_battle:SetFocusDir( "down", self.btn_outfits, true )
    self.btn_outfits:SetFocusDir( "down", self.btn_custom, true )
end

function SidebarLoadout:Refresh( character_data, act_data, anim, story )
    SidebarLoadout._base.Refresh( self, character_data )
    -- Set text
    self.title:SetText( character_data and loc.toupper( character_data:GetLocalizedName()) ):SetGlyphColour( UICOLOURS.SUBTITLE )
    local txt = act_data and ( story and loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_STORY_TITLE", act_data:GetLocalizedName() ) or act_data:GetLocalizedName() ) or ""
    self.subtitle:SetText(txt):SetGlyphColour( UICOLOURS.TITLE )
    
    local def = Content.GetCharacterDef( character_data.data.player_agent )
    if type(def.fight_data.MAX_HEALTH) == "table" then
        self.health_indicator:SetHealth( def.fight_data.MAX_HEALTH[1] )
    else
        self.health_indicator:SetHealth( def.fight_data.MAX_HEALTH or 1 )
    end
    self.resolve_indicator:SetResolve( act_data.data.max_resolve or def.max_resolve or 1 )

    -- Update button data
    self.btn_negotiation:Refresh( act_data, story )
    self.btn_battle:Refresh( act_data, story )
    self.btn_prestige:Refresh( act_data, story )
    self.btn_outfits:Refresh( act_data, story )
    self.btn_perks:Refresh( act_data, story )
    self.btn_custom:Refresh( act_data, story )

    self:Layout()
end

function SidebarLoadout:Layout()
    SidebarLoadout._base.Layout( self )

    self.health_indicator:LayoutBounds( "left", "below", self.subtitle ):Offset( 0, -16 )
    self.resolve_indicator:LayoutBounds( "after", "top", self.health_indicator ):Offset( 32, 0 )

    self.btn_negotiation:SetPos( -self.button_distance_x, self.button_distance_y )
    self.btn_battle:SetPos( self.button_distance_x, self.button_distance_y )
    self.btn_perks:SetPos( -self.button_distance_x-self.button_distance_x_middle, 0 )
    self.btn_outfits:SetPos( self.button_distance_x+self.button_distance_x_middle, 0 )
    self.btn_prestige:SetPos( -self.button_distance_x, -self.button_distance_y )
    self.btn_custom:SetPos( self.button_distance_x, -self.button_distance_y )

    return self
end

function SidebarLoadout:OnScreenModeChange( sm )
    SidebarLoadout._base.OnScreenModeChange( self, sm )

    local button_scale = 1
    if sm == SCREEN_MODE.MONITOR then
        button_scale = 1.4
    elseif sm == SCREEN_MODE.SMALL then
        button_scale = 1.5
    elseif sm == SCREEN_MODE.TV then
        button_scale = 1.5
    end

    self.btn_negotiation:SetLayoutScale( button_scale )
    self.btn_battle:SetLayoutScale( button_scale )
    self.btn_prestige:SetLayoutScale( button_scale )
    self.btn_outfits:SetLayoutScale( button_scale )
    self.btn_perks:SetLayoutScale( button_scale )
    self.btn_custom:SetLayoutScale( button_scale )
    -- self.btn_unlocks:SetLayoutScale( button_scale )
    -- self.btn_mutators:SetLayoutScale( button_scale )

    self.button_distance_y = BUTTON_DISTANCE_Y + 20*LAYOUT_SCALE[sm]
    self:Layout()
end

function SidebarLoadout:AnimateIn()
    SidebarLoadout._base.AnimateIn( self )

    local timing = 0.2
    local spacing = 60
    self.btn_negotiation:Show():SetPos( -self.button_distance_x+spacing, self.button_distance_y-spacing )
        :MoveTo( -self.button_distance_x, self.button_distance_y, timing, easing.inOutQuad )
        :AlphaTo( 1, timing, easing.inOutQuad )
    self.btn_battle:Show():SetPos( self.button_distance_x-spacing, self.button_distance_y-spacing )
        :MoveTo( self.button_distance_x, self.button_distance_y, timing, easing.inOutQuad )
        :AlphaTo( 1, timing, easing.inOutQuad )
    self.btn_perks:Show():SetPos( -self.button_distance_x-self.button_distance_x_middle+spacing, 0 )
        :MoveTo( -self.button_distance_x-self.button_distance_x_middle, 0, timing, easing.inOutQuad )
        :AlphaTo( 1, timing, easing.inOutQuad )
    self.btn_outfits:Show():SetPos( self.button_distance_x+self.button_distance_x_middle-spacing, 0 )
        :MoveTo( self.button_distance_x+self.button_distance_x_middle, 0, timing, easing.inOutQuad )
        :AlphaTo( 1, timing, easing.inOutQuad )
    self.btn_prestige:Show():SetPos( -self.button_distance_x+spacing, -self.button_distance_y+spacing )
        :MoveTo( -self.button_distance_x, -self.button_distance_y, timing, easing.inOutQuad )
        :AlphaTo( 1, timing, easing.inOutQuad )
    self.btn_custom:Show():SetPos( self.button_distance_x-spacing, -self.button_distance_y+spacing )
        :MoveTo( self.button_distance_x, -self.button_distance_y, timing, easing.inOutQuad )
        :AlphaTo( 1, timing, easing.inOutQuad )

    return self
end

function SidebarLoadout:AnimateOut()
    SidebarLoadout._base.AnimateOut( self )

    local timing = 0.1
    local spacing = -60
    self.btn_negotiation:SetPos( -self.button_distance_x, self.button_distance_y )
        :MoveTo( -self.button_distance_x+spacing, self.button_distance_y-spacing, timing, easing.inOutQuad )
        :AlphaTo( 0, timing, easing.inOutQuad, function() self.btn_negotiation:Hide() end )
    self.btn_battle:SetPos( self.button_distance_x, self.button_distance_y )
        :MoveTo( self.button_distance_x-spacing, self.button_distance_y-spacing, timing, easing.inOutQuad )
        :AlphaTo( 0, timing, easing.inOutQuad, function() self.btn_battle:Hide() end )
    self.btn_perks:SetPos( -self.button_distance_x-self.button_distance_x_middle, 0 )
        :MoveTo( -self.button_distance_x-self.button_distance_x_middle+spacing, 0, timing, easing.inOutQuad )
        :AlphaTo( 0, timing, easing.inOutQuad, function() self.btn_perks:Hide() end )
    self.btn_outfits:SetPos( self.button_distance_x+self.button_distance_x_middle, 0 )
        :MoveTo( self.button_distance_x+self.button_distance_x_middle-spacing, 0, timing, easing.inOutQuad )
        :AlphaTo( 0, timing, easing.inOutQuad, function() self.btn_outfits:Hide() end )
    self.btn_prestige:SetPos( -self.button_distance_x, -self.button_distance_y )
        :MoveTo( -self.button_distance_x+spacing, -self.button_distance_y+spacing, timing, easing.inOutQuad )
        :AlphaTo( 0, timing, easing.inOutQuad, function() self.btn_prestige:Hide() end )
    self.btn_custom:SetPos( self.button_distance_x, -self.button_distance_y )
        :MoveTo( self.button_distance_x-spacing, -self.button_distance_y+spacing, timing, easing.inOutQuad )
        :AlphaTo( 0, timing, easing.inOutQuad, function() self.btn_custom:Hide() end )

    return self
end

function SidebarLoadout:GetDefaultFocus()
    return self.last_btn or self.btn_negotiation
end

-------------------------------------------------------------------------
-- A single save file

local SaveSlotButton = class( "Widget.SaveSlotButton", Widget.Clickable )

local ICON_W = 56
local TEXT_LEFT_MARGIN = 64
local RIGHT_HITBOX_OVERFLOW = 40 -- So that the contents are still clickable when zoomed in

SaveSlotButton.CONTROL_MAP =
{
    {
        control = Controls.Digital.GAMEPAD_RT,
        fn = function(self)
            self.delete_fn()
        end,
        hint = function(self, left, right)
            table.insert(left, loc.format(LOC"UI.NEW_GAME_SCREEN.CONTROLS_RIGHT_SAVEFILE_DELETE"))
        end,
    },    
}

function SaveSlotButton:init( character_id, w, h, save_data, dev )

    local act_id = save_data and save_data.data and save_data.data.options and save_data.data.options.start_act
    local act_data = GetPlayerActData(act_id)
    local act_name = act_data and act_data:GetLocalizedName()

    SaveSlotButton._base.init( self )
    self.features = {}

    self.save_data = save_data
    
    self.hitbox = self:AddChild( Widget.SolidBox( w+RIGHT_HITBOX_OVERFLOW, h, 0x00FFFF00 ) )
    self.contents = self:AddChild( Widget() )
    self.glow = self.contents:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
        :SetTintColour( UICOLOURS.MUTATOR )
        :SetTintAlpha( 0.2 )
        :SetHiddenBoundingBox( true )
        :SetSize( ICON_W*2, ICON_W*2 )
    self.bg_color = self.save_data.is_dev and UICOLOURS.FIGHT or UICOLOURS.OBJECTIVE
    self.bg = self.contents:AddChild( Widget.Panel( engine.asset.Texture("UI/mutator_bg_active.tex" ) ) )
        :SetTintColour( self.bg_color )
        :SetBloom(0.1)
    self.icon = self.contents:AddChild( Widget.Image( engine.asset.Texture("images/white.tex" ) ) )
        :SetSize( ICON_W, ICON_W )
        :Bloom( 0.15 )

    if self.save_data.data.options.story_mode then
        self.icon:SetTexture( engine.asset.Texture("UI/ic_story_mode.tex" ) )
    else
        local advancement_level = self.save_data.data and self.save_data.data.options and self.save_data.data.options.advancement_level or 0
        if advancement_level > 0 then
            self.icon:SetTexture( engine.asset.Texture( loc.format( "UI/advancement_level_{1}.tex", advancement_level ) ) )
        else
            self.icon:Hide()
        end
    end
    self.delete_button = self.contents:AddChild( Widget.ImageButton( engine.asset.Texture("UI/saveslot_delete.tex" ) ) )
        :SetSize( 40, 40 )
        :SetTintAlpha( 0 )
        :Bloom( 0.1 )
        :SetToolTip( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_SAVEFILES_DELETE" )
        :Hide()
    self.delete_button.OnGainHover = function() self.delete_button._base.OnGainHover( self.delete_button ) self:OnDeleteGainHover() end
    self.delete_button.OnLoseHover = function() self.delete_button._base.OnLoseHover( self.delete_button ) self:OnDeleteLoseHover() end
    self.text_content = self.contents:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.title = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_TABS ) )
    	:SetText( self.save_data.is_dev and loc.format( LOC"UI.NEW_GAME_SCREEN.SAVEFILE_TITLE_DEV", act_name, save_data.name ) or loc.format( LOC"UI.NEW_GAME_SCREEN.SAVEFILE_TITLE", act_name ))
        :LeftAlign()
        :SetAutoSize( w-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :SetGlyphColour( self.save_data.is_dev and UICOLOURS.FIGHT_LIGHT or UICOLOURS.OBJECTIVE )
        :Bloom( 0.1 )
    self.progress = self.text_content:AddChild( Widget.Label( "tooltip", FONT_SIZE.ITEM_LABEL ) )
    	:SetText( loc.format( "{1} - {2#percent}", self.save_data.data and self.save_data.data.desc or "", self.save_data.data and self.save_data.data.progress or 0 ) )
        :LeftAlign()
        :SetAutoSize( w-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :SetGlyphColour( UICOLOURS.OBJECTIVE_LIGHT )
        :Bloom( 0.1 )
    self.labels_container = self.text_content:AddChild( Widget() )
    if self.save_data.data.options.mutators and #self.save_data.data.options.mutators > 0 then
        self.mutators_widget = self.labels_container:AddChild( Widget() )
        self.mutators_bg = self.mutators_widget:AddChild( Widget.Panel( engine.asset.Texture("UI/saveslot_legacy.tex") ) )
            :SetTintColour( UICOLOURS.TOOLTIP_TEXT )
            :Bloom( 0.15 )
        self.mutators_label = self.mutators_widget:AddChild( Widget.Label( "title", 21, loc.format( LOC"UI.NEW_GAME_SCREEN.SAVEFILE_MUTATORS", #self.save_data.data.options.mutators ) ) )
            :SetGlyphColour( UICOLOURS.SUBTITLE )
            :Bloom( 0.1 )
        local text_w, text_h = self.mutators_label:GetSize()
        self.mutators_bg:SetSize( text_w + 24, text_h + 2 )
        self.mutators_label:LayoutBounds( "center", "center", self.mutators_bg ):Offset( 0, 1 )
    end
    local time = self.save_data.data and self.save_data.data.time
    self.date = self.text_content:AddChild( Widget.Label( "tooltip", FONT_SIZE.SMALL_TEXT ) )
    	:SetText( os.date( "%x %X", time, time ) )
        :LeftAlign()
        :SetAutoSize( w-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom( 0.1 )

    local _, _, save_num
    if self.save_data.data.build_id then
        _, _, save_num = string.find(self.save_data.data.build_id,  ".* r(%d+) .*" )
    end
    save_num = save_num and tonumber(save_num) or 0

    self:SetToolTip( self.save_data )
    self:SetToolTipClass( Widget.TooltipSaveFile )
    self:ShowToolTipOnFocus( true )

    self:SetSize( w, h )

    if not TheGame:CanLoadSave( self.save_data.data ) then
        self.cannot_load = true
        self:UpdateImage()

    else
        self:SetOnClickFn( function()
            TheGame:FE():FadeTransition( function()
                TheGame:LoadGame( self.save_data.name, self.save_data.is_dev and "autosave" or nil )
            end, 0.5 )
        end )
    end
end

function SaveSlotButton:SetDeleteFn( fn )
    self.delete_fn = fn
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    return self
end

function SaveSlotButton:SetSize( w, h )
    self.title:SetAutoSize( w-TEXT_LEFT_MARGIN-12 )
    self.progress:SetAutoSize( w-TEXT_LEFT_MARGIN-12 )
    self.date:SetAutoSize( w-TEXT_LEFT_MARGIN-12 )
    self.progress:LayoutBounds( "left", "below", self.title ):Offset( 0, 4 )
    self.labels_container:StackChildrenRow(6):LayoutBounds( "left", "below", self.progress ):Offset( 0, -2 )
    self.date:LayoutBounds( "left", "below", self.labels_container ):Offset( 0, 1 )
    local text_w, text_h = self.text_content:GetSize()
    text_h = text_h + 20
    self.hitbox:SetSize( w, text_h )
    self.bg:SetSize( w, text_h )
    self.icon:LayoutBounds( "left", "center", self.hitbox ):Offset( 6, 0 )
    self.glow:LayoutBounds( "center", "center", self.icon ):Offset( 0, 0 )
    self.text_content:LayoutBounds( "left", "center", self.hitbox ):Offset( TEXT_LEFT_MARGIN, 1 )
    self.delete_button:LayoutBounds( "right", "top", self.bg ):Offset( -2, -2 )
    return self
end

function SaveSlotButton:OnAdded()
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
end

function SaveSlotButton:OnControlModeChange( cm, device_id )
    SaveSlotButton._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.TOUCH then
        -- Long press to delete
        self.delete_button:Hide():SetEnabled( false ):SetOnClickFn( nil )
        self:SetOnRightClickFn( nil )
        self:SetOnLongClickFn( self.delete_fn )
    end
    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        -- Click the X button to delete
        self.delete_button:Show():SetEnabled( true ):SetOnClickFn( self.delete_fn )
        self:SetOnRightClickFn( self.delete_fn )
        self:SetOnLongClickFn( nil )
    end
    if cm == CONTROL_MODE.GAMEPAD then
        -- Press controller button to delete
        self.delete_button:Hide():SetEnabled( false ):SetOnClickFn( nil )
        self:SetOnRightClickFn( nil )
        self:SetOnLongClickFn( nil )
    end
end

function SaveSlotButton:UpdateImage()
    if self.hover or self.focus then
        self.glow:ScaleTo( nil, 1.2, 0.1 )
        self.contents:ScaleTo( nil, 1.05, 0.1 )
        self.contents:MoveTo( 20, 0, 0.1 )
        self.bg:Bloom( 0.2 )
        if TheGame:FE():GetControlMode() == CONTROL_MODE.MOUSE_KEYBOARD then
            self.delete_button:SetEnabled( true ):AlphaTo( 1, 0.1 )
        end
    else
        self.glow:ScaleTo( nil, 1, 0.1 )
        self.contents:ScaleTo( nil, 1, 0.2 )
        self.contents:MoveTo( 0, 0, 0.2 )
        self.bg:Bloom( 0.1 )
        self:SetSaturation( 1 )
        self.delete_button:SetEnabled( false ):AlphaTo( 0, 0.1 )
    end


    if self.cannot_load then
        self.bg:Bloom( 0 )
        self:SetSaturation( 0 )
    else
        self:SetSaturation( 1 )
    end
end

function SaveSlotButton:OnGainFocus()
    SaveSlotButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function SaveSlotButton:OnLoseFocus()
    SaveSlotButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function SaveSlotButton:OnGainHover()
    SaveSlotButton._base.OnGainHover(self)
    self:UpdateImage()
end

function SaveSlotButton:OnLoseHover()
    SaveSlotButton._base.OnLoseHover(self)
    self:UpdateImage()
end

function SaveSlotButton:OnDeleteGainHover()
    local r, g, b, a = HexColour( UICOLOURS.PENALTY )
    self.bg:ColourFromCurrentTo( r, g, b, a, 0.2 )
end

function SaveSlotButton:OnDeleteLoseHover()
    local r, g, b, a = HexColour( self.bg_color )
    self.bg:ColourFromCurrentTo( r, g, b, a, 0.2 )
end

-------------------------------------------------------------------------
-- A savefile counter button that displays how many runs are in progress

local SaveFileCounter = class( "Widget.SaveFileCounter", Widget.Clickable )

local ICON_W = 24
local TEXT_LEFT_MARGIN = 46
local RIGHT_HITBOX_OVERFLOW = 40 -- So that the contents are still clickable when zoomed in

function SaveFileCounter:init( w, h, fn )
    SaveFileCounter._base.init( self )
    
    self.hitbox = self:AddChild( Widget.SolidBox( w+RIGHT_HITBOX_OVERFLOW, h, 0x00FFFF00 ) )
    self.contents = self:AddChild( Widget() )
    self.glow = self.contents:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
        :SetTintColour( UICOLOURS.MUTATOR )
        :SetTintAlpha( 0.2 )
        :SetHiddenBoundingBox( true )
        :SetSize( ICON_W*3, ICON_W*3 )
    self.bg_color = UICOLOURS.MUTATOR
    self.bg = self.contents:AddChild( Widget.Panel( engine.asset.Texture("UI/mutator_bg_active.tex" ) ) )
        :SetTintColour( self.bg_color )
        :SetBloom(0.1)
    self.icon = self.contents:AddChild( Widget.Image( engine.asset.Texture("UI/ic_checkmark.tex" ) ) )
        :SetSize( ICON_W, ICON_W )
        :SetTintColour( self.bg_color )
        :Bloom( 0.1 )
    self.text_content = self.contents:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.progress = self.text_content:AddChild( Widget.Label( "tooltip", FONT_SIZE.ITEM_LABEL ) )
        :SetText( "3 Sal runs in progress" )
        :LeftAlign()
        :SetAutoSize( w-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :SetGlyphColour( UICOLOURS.MUTATOR )
        :Bloom( 0.1 )

    self:SetSize( w, h )

    self:SetOnClickFn( function() if fn then fn(self.character_index) end end )
end

function SaveFileCounter:SetSize( w, h )
    self.progress:SetAutoSize( w-TEXT_LEFT_MARGIN-12 )
    local text_w, text_h = self.text_content:GetSize()
    text_h = text_h + 20
    self.hitbox:SetSize( w, text_h )
    self.bg:SetSize( w, text_h )
    self:Layout()
    return self
end

function SaveFileCounter:Layout()
    self.icon:LayoutBounds( "left", "center", self.hitbox ):Offset( 14, 0 )
    self.glow:LayoutBounds( "center", "center", self.icon ):Offset( 0, 0 )
    self.text_content:LayoutBounds( "left", "center", self.hitbox ):Offset( TEXT_LEFT_MARGIN, 1 )
    return self
end

function SaveFileCounter:SetCharacterIndex( character_index )
    self.character_index = character_index
    return self
end

function SaveFileCounter:SetSaveFileCount( savefile_count )
    self.savefile_count = savefile_count or 0
    return self
end

function SaveFileCounter:Refresh( character_data )
    self.character_data = character_data
    self.progress:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_CHARACTERS_RUNS_IN_PROGRESS", character_data:GetLocalizedName(), self.savefile_count or 0 ) )
    self:Layout()
    self:SetShown( self.savefile_count > 0 )
    return self
end

function SaveFileCounter:OnAdded()
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
end

function SaveFileCounter:UpdateImage()
    if self.hover or self.focus then
        self.glow:ScaleTo( nil, 1.2, 0.1 )
        self.contents:ScaleTo( nil, 1.05, 0.1 )
        self.contents:MoveTo( 20, 0, 0.1 )
        self.bg:Bloom( 0.2 )
    else
        self.glow:ScaleTo( nil, 1, 0.1 )
        self.contents:ScaleTo( nil, 1, 0.2 )
        self.contents:MoveTo( 0, 0, 0.2 )
        self.bg:Bloom( 0.1 )
    end
end

function SaveFileCounter:OnGainFocus()
    SaveFileCounter._base.OnGainFocus(self)
    self:UpdateImage()
end

function SaveFileCounter:OnLoseFocus()
    SaveFileCounter._base.OnLoseFocus(self)
    self:UpdateImage()
end

function SaveFileCounter:OnGainHover()
    SaveFileCounter._base.OnGainHover(self)
    self:UpdateImage()
end

function SaveFileCounter:OnLoseHover()
    SaveFileCounter._base.OnLoseHover(self)
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- A fancy button for the loadout sidebar, linking to one of the other sidebars

local SidebarButton = class("Widget.SidebarButton", Widget.Clickable)

local HEIGHT = 90
local ICON_W = 120
local TEXT_LEFT_MARGIN = 100

function SidebarButton:init( width, type, on_click_fn )
    SidebarButton._base.init(self)

    self.button_width = width
    self.type = type

    self.hitbox = self:AddChild( Widget.SolidBox( self.button_width, HEIGHT, 0x00ffff00 ) )
    self.contents = self:AddChild( Widget() )
    self.glow = self.contents:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
   		:SetHiddenBoundingBox( true )
        :SetSize( ICON_W*2, ICON_W*2 )
	self.icon_hover = self.contents:AddChild( Widget.Image( engine.asset.Texture( "UI/newgame_sidebar_button_hover.tex") ) )
		:SetHiddenBoundingBox( true )
        :SetSize( ICON_W, ICON_W )
        :Bloom( 0.1 )
    local aura_size = ICON_W*0.61
    self.aura_layer_1 = self.contents:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer1_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5)
    self.aura_layer_2 = self.contents:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer2_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5)
    self.aura_layer_3 = self.contents:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer3_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5)
	self.icon_bg = self.contents:AddChild( Widget.Image( engine.asset.Texture( "UI/newgame_sidebar_button_bg.tex") ) )
		:SetHiddenBoundingBox( true )
        :SetSize( ICON_W, ICON_W )
        :Bloom( 0.1 )
    self.icon_progress = self.contents:AddChild( Widget.RadialProgress( ICON_W, ICON_W, engine.asset.Texture( "UI/newgame_sidebar_button_progress.tex") ) )
        :SetHiddenBoundingBox( true )
        :Bloom( 0.15 )
        :SetProgress( 1.0 )
	self.icon = self.contents:AddChild( Widget.Image( engine.asset.Texture( "UI/gameover_stats_negotiation.tex") ) )
        :SetSize( ICON_W/2.6, ICON_W/2.6 )
        :Bloom( 0.15 )
    self.text_content = self.contents:AddChild( Widget() )
    self.title = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE ) )
        :SetAutoSize( self.button_width-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :Bloom( 0.1 )
    self.subtitle = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL ) )
        :SetAutoSize( self.button_width-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :Bloom( 0.1 )
    self.new_items_label = self.contents:AddChild( Widget.NewItemsLabel() )
		:SetHiddenBoundingBox( true )
    self.label_progress = self.contents:AddChild( Widget.Label( "body", FONT_SIZE.SMALL_TEXT ) )
		:SetHiddenBoundingBox( true )
        :Bloom( 0.15 )
    if self.type == SIDEBAR.PERKS then
        -- The perks button shows the active perk slots
        self.perks_container = self.contents:AddChild( Widget() )
    end

    self.facing_left = false

    self.on_click_fn = on_click_fn
    self:SetOnClickFn( function() self:OnClick() end )

end

function SidebarButton:OnClick()
    if not self.locked then self.on_click_fn() end
end

function SidebarButton:SetFacingLeft( facing_left )
    self.facing_left = facing_left
    if self.facing_left then
        -- Icon on the right, text on its left edge
        self.title:RightAlign()
        self.subtitle:RightAlign()
    else
        -- Icon on the left, text on its right edge
        self.title:LeftAlign()
        self.subtitle:LeftAlign()
    end
    self.new_items_label:SetFacingLeft( facing_left )
    return self
end

function SidebarButton:Layout()
    self.contents:SetScale( 1, 1 )
        :SetPos( 0, 0 )
    self.icon_bg:SetRotation( 0*DEGREES )
    self.icon_bg:LayoutBounds( self.facing_left and "right" or "left", "center", self.hitbox ):Offset( self.facing_left and 15 or -15, 0 )
    local pos_x, pos_y = self.icon_bg:GetPos()
	self.icon_hover:SetPos( pos_x, pos_y )
	self.glow:SetPos( pos_x, pos_y )
    self.aura_layer_1:SetPos( pos_x, pos_y )
    self.aura_layer_2:SetPos( pos_x, pos_y )
    self.aura_layer_3:SetPos( pos_x, pos_y )
	self.icon_progress:SetPos( pos_x, pos_y )
	self.icon:SetPos( pos_x, pos_y )
	self.label_progress:LayoutBounds( "center", "bottom", self.icon_bg ):Offset( 0, 3 )
	-- Layout text
	self.subtitle:LayoutBounds( self.facing_left and "right" or "left", "below", self.title ):Offset( 0, 4 )
	self.text_content:LayoutBounds( self.facing_left and "right" or "left", "center", self.hitbox ):Offset( self.facing_left and -TEXT_LEFT_MARGIN or TEXT_LEFT_MARGIN, self.subtitle:IsShown() and 4 or 1 )
    if self.new_items_label:IsShown() then self.new_items_label:LayoutBounds( self.facing_left and "right" or "left", "below", self.text_content ):Offset( self.facing_left and 2 or -2, 0 ) end
	if self.perks_container then
        self.perks_container:LayoutBounds( self.facing_left and "right" or "left", "below", self.text_content ):Offset( 0, -4 )
        self.new_items_label:LayoutBounds( self.facing_left and "right" or "left", "below", self.perks_container ):Offset( self.facing_left and 2 or -2, 0 )
    end
    
end

function SidebarButton:Refresh( act_data, story )
    --print (story, debugstack())
	self.type_colour = UICOLOURS.SUBTITLE
    if self.type == nil then
        self.locked = true
        self.type_colour = UICOLOURS.TITLE
        self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_UPCOMING_FEATURE" )
        self.subtitle:Hide()
        self.icon:SetTexture( engine.asset.Texture( "UI/ic_locked.tex") )
        self.new_items_label:Hide()
	elseif self.type == SIDEBAR.NEGOTIATION then
        self.type_colour = UICOLOURS.NEGOTIATION
        self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_NEGOTIATION_TITLE" )
        self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_NEGOTIATION_SUBTITLE" )
            :SetGlyphColour( UICOLOURS.WHITE )
            :SetTintColour( UICOLOURS.TITLE )
        self.icon:SetTexture( engine.asset.Texture( "UI/newgame_sidebar_negotiation.tex") )
        self.new_items_label:Hide()
    elseif self.type == SIDEBAR.BATTLE then
		self.type_colour = UICOLOURS.FIGHT
		self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_BATTLE_TITLE" )
		self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_BATTLE_SUBTITLE" )
            :SetGlyphColour( UICOLOURS.WHITE )
            :SetTintColour( UICOLOURS.TITLE )
        self.icon:SetTexture( engine.asset.Texture( "UI/newgame_sidebar_battle.tex") )
		self.new_items_label:Hide()
	elseif self.type == SIDEBAR.PRESTIGE then
		self.type_colour = UICOLOURS.ADVANCEMENT
		-- Get act id
		-- Get last selected prestige level
		local selected_prestige = TheGame:GetGameProfile():GetSelectedAdvancement( act_data.id )
		self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_PRESTIGE_TITLE" )
		self.subtitle:SetText( loc.format( LOC"ADVANCEMENT.ADVANCEMENT_LEVEL", selected_prestige ) )
		if selected_prestige == 0 then
            self.icon:SetTexture( engine.asset.Texture( "UI/advancement_level_0.tex" ) )
		else
			self.subtitle:SetText( LOC(act_data.background.data.advancement[selected_prestige].name) )
            self.icon:SetTexture( act_data.background.data.advancement[selected_prestige].icon )
		end
		self.icon:SetSize( ICON_W/2.6, ICON_W/2.6 )
        -- Get prestige progress
        local unlocked, total = ActUtil.GetPrestigeProgress( act_data.id )
		self.label_progress:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_PRESTIGE_PROGRESS", unlocked, total ) )
        self.icon_progress:SetProgress( unlocked/total )
        self.new_items_label:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_PRESTIGE_NEW" )
		local higher_available = selected_prestige < unlocked
		self.new_items_label:SetShown( higher_available )
        self.locked = unlocked < 1 or story -- This is locked if only Level 0 prestige is available
        
        if self.locked then 
            self.subtitle:SetText( story and LOC"UI.NEW_GAME_SCREEN.STORY_BUTTON" or LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_PRESTIGE_LOCKED" ) 
        end

    elseif self.type == SIDEBAR.OUTFITS then
		self.type_colour = UICOLOURS.ACCENT
		local character_id = act_data.background.data.id
        -- Get total unlocks
        local unlocked, total = ActUtil.GetOutfitsProgress( character_id )
        self.locked = unlocked <= 1 -- This is locked if only one outfit is available
        unlocked = unlocked 
        total = total
		-- Get selected outfit
		local unseen_outfits = false
		local selected_outfit_id = TheGame:GetGameProfile():GetSelectedOutfit( character_id )
		local selected_outfit_data
		for k, outfit_data in ipairs( Content.GetOutfitsForCharacter( character_id ) ) do
    		selected_outfit_id = selected_outfit_id or outfit_data.id -- If there wasn't a previously selected outfit, make the first one it
    		if outfit_data.id == selected_outfit_id then
    			selected_outfit_data = outfit_data
    		end
    		if TheGame:GetGameProfile():HasUnlock( outfit_data.id ) and TheGame:GetGameProfile():IsOutfitSeen( character_id, outfit_data.id ) == false then unseen_outfits = true end
    	end
		self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_OUTFITS_TITLE" )
        self.subtitle:SetText( selected_outfit_data and selected_outfit_data:GetName() )
        self.icon:SetTexture( engine.asset.Texture( "UI/newgame_sidebar_outfits.tex") )
        self.label_progress:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_OUTFITS_PROGRESS", unlocked, total ) )
        self.icon_progress:SetProgress( unlocked/total )
        self.new_items_label:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_OUTFITS_NEW" )
        if self.locked then self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_OUTFITS_LOCKED" ) end
		self.new_items_label:SetShown( unseen_outfits )

    elseif self.type == SIDEBAR.PERKS then
        self.type_colour = UICOLOURS.PERK
        self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_PERKS_TITLE" )
        self.icon:SetTexture( engine.asset.Texture( "UI/newgame_sidebar_perks.tex") )
        self.new_items_label:Hide()

        -- Get current selected perks
        local selected = TheGame:GetGameProfile():GetSelectedPerks( act_data.background.data.id ) or {}
        local points = TheGame:GetGameProfile():GetPerkPoints()
        local unlocks = TheGame:GetGameProfile():GetPerkUnlocks()
        local has_unlock = next(unlocks) ~= nil

        if not has_unlock and points <= 0 then
            self.locked = true
            self:SetToolTip( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_PERKS_SUBTITLE_LOCKED_TT")
            self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_PERKS_SUBTITLE_LOCKED" )
        else
            -- Get max perk slots
            local num_perk_slots = TheGame:GetGameProfile():GetNumPerkSlots()
            local num_unlocked = TheGame:GetGameProfile():GetNumPerksUnlocked()
            local num_perks = 0
            for id, def in pairs(Content.GetAllGrafts()) do
                if def.type == GRAFT_TYPE.PERK then
                    num_perks = num_perks + 1
                end
            end

            -- Update content
            self.locked = false
            self.subtitle:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_PERKS_UNSELECTED", #selected, num_perk_slots) )
            self.new_items_label:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_PERKS_POINTS", points) )
            self.new_items_label:SetShown( points > 0 and num_perks > num_unlocked )

            -- Refresh perk icons
            self.perks_container:DestroyAllChildren()
            for k = 1, num_perk_slots do
                -- Get perk, if any
                local graft = selected[k] and GraftInstance( selected[k] ) or nil
                -- Create icon
                local widget = self.perks_container:AddChild( Widget.GraftSlot( GRAFT_TYPE.PERK, 40 ) )
                    :Refresh( graft )
            end 
            self.perks_container:StackChildrenRow( 10 )
        end

    elseif self.type == SIDEBAR.CUSTOM then
        self.type_colour = UICOLOURS.SUBTITLE
        self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_CUSTOM_TITLE" )
        self.icon:SetTexture( engine.asset.Texture( "UI/ic_custom_game.tex") )
        local active = Widget.SidebarCustom:IsCustomGameActive( act_data.background.data )
        if active then
            self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_CUSTOM_ENABLED" )
            self.new_items_label:Show()
                :SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_CUSTOM_ACTIVE_BUTTON" )
                :SetColour( UICOLOURS.PENALTY )
        else
            self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_CUSTOM_DISABLED" )
            self.new_items_label:Hide()
        end
	end

	self.glow:SetTintColour( self.type_colour )
    	:SetTintAlpha( 0.2 )
	self.icon_hover:SetTintColour( self.type_colour )
	self.icon_bg:SetTintColour( self.type_colour )
	self.title:SetGlyphColour( self.type_colour )
	self.label_progress:SetGlyphColour( self.type_colour )
    self.aura_layer_1:SetTintColour( self.type_colour )
    self.aura_layer_2:SetTintColour( self.type_colour )
    self.aura_layer_3:SetTintColour( self.type_colour )
    if self.locked then
        self.icon:SetTexture( engine.asset.Texture( "UI/newgame_sidebar_locked.tex") )
            :SetTintColour( 0x131E25ff )
        self.icon_progress:SetTintColour( 0x131E25ff )
        self.label_progress:Hide()
        self.new_items_label:Hide()
        self.aura_layer_1:Hide()
        self.aura_layer_2:Hide()
        self.aura_layer_3:Hide()
    else
        self.icon:SetTintColour( UICOLOURS.WHITE )
        self.icon_progress:SetTintColour( self.type_colour )
        self.label_progress:Show()
        self.aura_layer_1:Show()
        self.aura_layer_2:Show()
        self.aura_layer_3:Show()
    end

    self:Layout()
end

function SidebarButton:UpdateImage()
    if not self:IsVisible() then
        return    
    end

    if ( not self.locked and self.hover ) or self.focus then
        if self.locked then
            AUDIO:PlayEvent( "event:/ui/card_codex/mouseover_locked_card" )
        end
        self.icon_hover:ScaleTo( nil, 1.2, 0.1 )
        self.icon_bg:RotateTo( -45*DEGREES, 0.2 )
        self.contents:ScaleTo( nil, 1.05, 0.2 )
        self.contents:MoveTo( self.facing_left and -20 or 20, 0, 0.2 )
        self.aura_layer_1:RotateIndefinitely( -0.015 )
        self.aura_layer_2:RotateIndefinitely( 0.05 )
        self.aura_layer_3:RotateIndefinitely( -0.04 )
        self.aura_layer_1:ScaleTo( nil, 1.19, 0.4 )
        self.aura_layer_2:ScaleTo( nil, 1.25, 0.3 )
        self.aura_layer_3:ScaleTo( nil, 1.35, 0.2 )
    else
        self.icon:SetTintColour( self.locked and 0x131E25ff or UICOLOURS.WHITE )
        self.icon_hover:ScaleTo( nil, 1, 0.1 )
        self.icon_bg:RotateTo( 0*DEGREES, 0.2 )
        self.contents:ScaleTo( nil, 1, 0.3 )
        self.contents:MoveTo( 0, 0, 0.3 )
        self.aura_layer_1:RotateIndefinitely( -0.01 )
        self.aura_layer_2:RotateIndefinitely( 0.03 )
        self.aura_layer_3:RotateIndefinitely( -0.025 )
        self.aura_layer_1:ScaleTo( nil, 1, 0.2 )
        self.aura_layer_2:ScaleTo( nil, 1, 0.3 )
        self.aura_layer_3:ScaleTo( nil, 1, 0.4 )
    end
end

function SidebarButton:OnGainFocus()
    SidebarButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function SidebarButton:OnLoseFocus()
    SidebarButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function SidebarButton:OnGainHover()
    SidebarButton._base.OnGainHover(self)
    self:UpdateImage()
end

function SidebarButton:OnLoseHover()
    SidebarButton._base.OnLoseHover(self)
    self:UpdateImage()
end

function SidebarButton:OnVizChange(viz)
    if viz then  
        self:UpdateImage()
        --self.aura_layer_1:RotateIndefinitely( -0.01 ):ScaleTo( nil, 1, 0.2 )
        --self.aura_layer_2:RotateIndefinitely( 0.03 ):ScaleTo( nil, 1, 0.3 )
        --self.aura_layer_3:RotateIndefinitely( -0.025 ):ScaleTo( nil, 1, 0.4 )
    else
        self.aura_layer_1:StopSpin()
        self.aura_layer_2:StopSpin()
        self.aura_layer_3:StopSpin()
    end
    return self
end

------------------------------------------------------------------------------------------
-- The sidebar showing the Negotiation cards

local NewItemsLabel = class("Widget.NewItemsLabel", Widget)

function NewItemsLabel:init( text )
    NewItemsLabel._base.init(self)

    self.new_items_icon = self:AddChild( Widget.Image( engine.asset.Texture("UI/ic_new_items.tex"), 22, 22 ) )
        :SetTintColour( UICOLOURS.QUEST )
        :Bloom(0.06)
    self.new_items_label = self:AddChild( Widget.Label( "body", FONT_SIZE.SMALL_TEXT, text or "" ) )
    	:LeftAlign()
    	:SetGlyphColour( UICOLOURS.QUEST )
        :Bloom(0.09)
        :LayoutBounds( "after", "center", self.new_items_icon ):Offset( 4, 1 )
end

function NewItemsLabel:SetFacingLeft( facing_left )
    self.facing_left = facing_left
    return self
end

function NewItemsLabel:SetText( text )
	self.new_items_label:SetText( text or "" )
        :LayoutBounds( self.facing_left and "before" or "after", "top", self.new_items_icon ):Offset( self.facing_left and -4 or 4, 2 )
    return self
end

function NewItemsLabel:SetColour( colour )
    self.new_items_icon:SetTintColour( colour )
    self.new_items_label:SetGlyphColour( colour )
    return self
end

-------------------------------------------------------------------------
-- A small wide button for customizations

local ShortButton = class("Widget.ShortButton", Widget.Clickable)

local WIDTH = 200
local BUTTON_HEIGHT = 42

function ShortButton:init( width, text, icon, color )
    ShortButton._base.init(self)

    self.hitbox = self:AddChild( Widget.SolidBox( width, BUTTON_HEIGHT, 0x00FFFF00 ) )
    self.contents = self:AddChild( Widget() )
    
    -- Button bg
    self.default_bloom = 0.2
    self.button_bg_border = 20
    self.button_bg_border_scale = 0.5
    self.button_bg = self.contents:AddChild( Widget.Panel( engine.asset.Texture("UI/characterselect_deck_button.tex") ) )
    self.button_bg:SetNineSliceCoords( self.button_bg_border, self.button_bg_border, self.button_bg_border, self.button_bg_border )
        :SetTintColour( color and color or UICOLOURS.SUBTITLE )
        :SetNineSliceBorderScale( self.button_bg_border_scale )
        :Bloom( self.default_bloom )

    -- Button icon
    self.button_icon = self.contents:AddChild( Widget.Image( icon or engine.asset.Texture("images/white.tex"), BUTTON_HEIGHT-14, BUTTON_HEIGHT-14 ) )
        :Bloom( 0.15 )
        :SetShown( icon ~= nil )

    -- Button label
    self.text_content = self.contents:AddChild( Widget() )
    self.button_text = self.text_content:AddChild( Widget.Label( "body", FONT_SIZE.SMALL_TEXT, text or "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )

    -- Arrow icon
    self.button_arrow = self.contents:AddChild( Widget.Image( engine.asset.Texture("UI/characterselect_deck_arrow.tex"), 12, 12 ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
        :Hide()

    -- How much the button animates to the right when focused/hovered
    self.focus_x_delta = 0

    self:SetSize( width, BUTTON_HEIGHT )
end

function ShortButton:SetText( text )
    self.button_text:SetText( text )
    self:SetSize( self.width )
end

function ShortButton:SetFocusDelta( focus_x_delta )
    self.focus_x_delta = focus_x_delta or 0
    return self
end

function ShortButton:ShowArrow()
    self.button_arrow:Show()
    return self
end

function ShortButton:UpdateImage()
    if self.hover or self.focus then
        if self.enabled then
            self.button_bg:Bloom( 0.4 )
            self.contents:ScaleTo( nil, 1.02, 0.1 )
        end
        self.contents:MoveTo( self.focus_x_delta, 0, 0.1 )
    else
        self.button_bg:Bloom( self.default_bloom )
        self.contents:ScaleTo( nil, 1, 0.2 )
        self.contents:MoveTo( 0, 0, 0.2 )
    end
    return self
end

function ShortButton:OnGainFocus()
    ShortButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function ShortButton:OnLoseFocus()
    ShortButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function ShortButton:OnGainHover()
    ShortButton._base.OnGainHover(self)
    self:UpdateImage()
end

function ShortButton:OnLoseHover()
    ShortButton._base.OnLoseHover(self)
    self:UpdateImage()
end

function ShortButton:SetSize( width, height )
    self.width = width or self.width or WIDTH
    self.height = height or self.height or BUTTON_HEIGHT
    self.hitbox:SetSize( self.width, self.height )
    local nineslice_w, nineslice_h = self.width + self.button_bg_border*2*self.button_bg_border_scale, self.height + self.button_bg_border*2*self.button_bg_border_scale
    self.button_bg:SetSize( nineslice_w, nineslice_h )
    self:Layout()
    return self
end

function ShortButton:Layout()
    self.button_icon:LayoutBounds( "left", "center", self.button_bg ):Offset( 8, 0 )
    if self.button_icon:IsShown() then
        self.text_content:LayoutBounds( "after", "center", self.button_icon ):Offset( 6, 1 )
    else
        self.text_content:LayoutBounds( "left", "center", self.button_bg ):Offset( 14, 1 )
    end
    self.button_arrow:LayoutBounds( "right", "center", self.button_bg ):Offset( -12, 0 )
    return self
end

-------------------------------------------------------------------------
-- A button to start a new run in a specific game mode

local NewActButton = class("Widget.NewActButton", Widget.ShortButton)

function NewActButton:init( width )
    NewActButton._base.init( self, width )

    self.sidebar_gradient = self.contents:AddChild( Widget.Image( engine.asset.Texture("UI/compendium_graft_gradient.tex") ) )
        :SendBehind( self.text_content )

    self.button_text:SetFont( "title" )
        :SetFontSize( FONT_SIZE.ITEM_SUBTITLE )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.WHITE )
    self.button_desc = self.text_content:AddChild( Widget.Label( "body", FONT_SIZE.SMALL_TEXT, "" ) )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )

    self.act_image_w = 138
    self.act_image_h = 115
    self.act_image = self.contents:AddChild( Widget.Image(engine.asset.Texture("UI/char_0_locked.tex")) )
        :SetHiddenBoundingBox( true )
        :SetSize( self.act_image_w, self.act_image_h )

    self:SetSize( width, 100 )
end

function NewActButton:Refresh( character_data, act, story )
    if TheGame:GetGameProfile():IsActLocked( act.id ) then
        -- Act locked to the player so far
        self.button_text:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_LOCKED" )
        self.button_desc:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_LOCKED_DESC" )
        self.default_bloom = 0
        self.button_bg:SetTintColour( 0x364B5Aff ):Bloom( self.default_bloom )
        self.sidebar_gradient:SetTintColour( 0x364B5Aff ):SetTintAlpha( 0.4 )
        self.button_desc:SetGlyphColour( 0xA5A5A5ff )
        self.button_arrow:Hide()
        self:Disable()
    elseif IsActUpcoming( act ) then
        -- Act in development
        self.button_text:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_DEVELOPMENT" )
        self.button_desc:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_DEVELOPMENT_DESC" )
        self.default_bloom = 0
        self.button_bg:SetTintColour( 0x364B5Aff ):Bloom( self.default_bloom )
        self.sidebar_gradient:SetTintColour( 0x364B5Aff ):SetTintAlpha( 0.4 )
        self.button_desc:SetGlyphColour( 0xA5A5A5ff )
        self.button_arrow:Hide()
        self:Disable()
    else
        -- Act available
        self.story = story
        if story then
            if act.data and act.data.story_image then self.act_image:SetTexture( act.data.story_image ) end
        else
            if act.data and act.data.act_image then self.act_image:SetTexture( act.data.act_image ) end
        end
        if story then
            if act.data and act.data.story_colour_frame then self.button_bg:SetTintColour( act.data.story_colour_frame ) end
        else
            if act.data and act.data.colour_frame then self.button_bg:SetTintColour( act.data.colour_frame ) end
        end
        if story then
            if act.data and act.data.story_colour_background then
                self.sidebar_gradient:SetTintColour( act.data.story_colour_background )
                    :SetTintAlpha( 0.4 ) end
        else
            if act.data and act.data.colour_background then
                self.sidebar_gradient:SetTintColour( act.data.colour_background )
                    :SetTintAlpha( 0.4 ) end
        end
        if story and act.data.story_colour_text then
            self.button_desc:SetGlyphColour( act.data.story_colour_text )
            self.button_arrow:SetTintColour( act.data.story_colour_text )
        elseif act.data and act.data.colour_text then
            self.button_desc:SetGlyphColour( act.data.colour_text )
            self.button_arrow:SetTintColour( act.data.colour_text ) end
        if act.data and act.data.name then 
            if story then
                self.button_text:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_STORY_TITLE", act:GetLocalizedName() ) ) 
            else
                self.button_text:SetText( act:GetLocalizedName() ) 
            end
        end
        if act.data and act.data.desc then 
            if story then
                self.button_desc:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_STORY_DESC" )
            else
                self.button_desc:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_ACT_DESC", act:GetLocalizedDesc() ) )
            end
        end
    end
    self:Layout()
    return self
end

function NewActButton:Layout()
    if self.button_desc then
        local text_w = self.width - self.act_image_w - SPACING.M1*2
        self.button_text:SetAutoSize( text_w )
        self.button_desc:SetAutoSize( text_w )
        self.sidebar_gradient:SetSize( self.width/2, self.height-2 )

        self.sidebar_gradient:LayoutBounds( "left", "center", self.button_bg )
            :Offset( 3, 0 )
        self.act_image:LayoutBounds( "left", "bottom", self.button_bg )
            :Offset( 3, 1 )
        self.button_desc:LayoutBounds( "left", "below", self.button_text )
            :Offset( 0, 2 )
        self.text_content:LayoutBounds( "after", "center", self.act_image )
            :Offset( 0, -7 )
        self.button_arrow:LayoutBounds( "right", "center", self.button_bg ):Offset( -12, 0 )
    end
    return self
end

------------------------------------------------------------------------------------------
-- Displays a few selectable character anims

local NewGameAnims = class("Widget.NewGameAnims", Widget)

local standard_spacing = 30 -- space between anims when at regular scale
local scaled_spacing = 30 -- space between anims when scaled up
local ANIM_DURATION = 0.2
local SELECTED_X_CENTER = 0
local SELECTED_X_SIDEBAR_MODE = 0
local SELECTED_Y = 0

function NewGameAnims:init( on_click_fn )
    NewGameAnims._base.init(self)
    self.on_click_fn = on_click_fn
    self.anims = {}
    self.move_single_character = false -- If the selected character should move to the right when a sidebar shows up

    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    self.scroll_offset = self:GetPos()
    self.scroll_target = nil

    self:StartUpdating()
end

function NewGameAnims:AddAnim( index, agent_id, outfit_id, character_data )
	local anim = self:AddChild( Widget.NewGameAnimatedCharacter( function(index) self:OnClicked(index) end, index, agent_id, outfit_id, character_data ) )
	   :SetHoverFn( function(has_hover) self:OnAnimHover(index,has_hover) end )
    self.anims[index] = anim
	return anim
end

function NewGameAnims:AddLocked( index, agent_id, outfit_id, character_data )
    local anim = self:AddChild( Widget.NewGameLockedCharacter( function(index) self:OnClicked(index) end, index, agent_id, outfit_id, character_data ) )
    self.anims[index] = anim
    return anim
end

function NewGameAnims:SetDefaultPositions()
    -- Layout the anims and position self on the screen, aligned to the left, after the sidebar
    self:StackChildrenRow( standard_spacing )
    
    -- Save every anim's position for later reference
    for k, v in ipairs( self.anims ) do
        v.default_x, v.default_y = v:GetPos()
    end

    -- Calculate maximum scroll
    self:LayoutBounds( "left", "above", -self.screen_w/2 + SIDEBAR_W*LAYOUT_SCALE[TheGame:FE():GetScreenMode()], 62 )
    self.max_scroll_offset = self:GetPos()
    self.max_scroll_offset = self.max_scroll_offset + 100

    -- And minimum scroll
    self:LayoutBounds( "right", nil, self.screen_w/2 )
    self.min_scroll_offset = self:GetPos()
    self.min_scroll_offset = math.min( self.max_scroll_offset, self.min_scroll_offset - 100 ) -- Add some extra padding

    -- And starting scroll position
    self.scroll_offset = self.max_scroll_offset
    self.scroll_target = nil
    self:SetPos( self.scroll_offset )

    self:RefreshCenter()

    return self
end

function NewGameAnims:OnUpdate( dt )

    -- If we have some scrolling to do, ease it
    if self.scroll_target and self.scroll_target ~= self.scroll_offset then
        self:Scroll( (self.scroll_target - self.scroll_offset)*dt*10 )
    end

    return NewGameAnims._base.OnUpdate( self, dt )
end

function NewGameAnims:OnScreenModeChange( sm )
    NewGameAnims._base.OnScreenModeChange( self, sm )

    self.move_single_character = false
    if sm == SCREEN_MODE.MONITOR then
        self.move_single_character = false
    elseif sm == SCREEN_MODE.SMALL then
        self.move_single_character = true
    elseif sm == SCREEN_MODE.TV then
        self.move_single_character = true
    end

    self:RefreshCenter()

    -- If we're in sidebar mode, force move the character to the right spot
    if self.sidebar_shown then
        self.sidebar_shown = false
        self:SetSidebarMode( true )
    end
end

function NewGameAnims:OnScreenResize(w,h)
    NewGameAnims._base.OnScreenResize( self,w,h )
    self:RefreshCenter()
end

function NewGameAnims:OnClicked( index )
	self:SelectIndex( index )
	self.on_click_fn( index )
end

function NewGameAnims:OnAnimHover( index, has_hover )
    -- If we're in desktop mode and this character isn't selected, select it
    if TheGame:FE():GetControlMode() == CONTROL_MODE.MOUSE_KEYBOARD and self.anims[index] and not self.anims[index].selected and not self.showing_single then
        self:SelectIndex( index )
        self.on_click_fn( index )
    end
end

function NewGameAnims:SelectIndex( index )
    for k, v in ipairs( self.anims ) do
        if k == index then
            -- The one we want!
            v:SetSelected( true )
            local delta = Lerp(self.max_scroll_offset, self.min_scroll_offset, k/#self.anims) - self.scroll_offset
            self:EaseScroll( delta )
        else
            -- One of the others
            v:SetSelected( false )
            -- Clear focus, so we don't have a hovered and a focused anim simultaneously
            if v:HasFocus() then v:ClearFocus() end
        end
    end
end

function NewGameAnims:EnsureVisible( anim )
    -- Make sure the anim is within the bounds of the screen
    local anim_x, _ = anim:GetPos()
    local anim_w, _ = anim:GetSize()
    local left_x = -self.screen_w/2
    local right_x = self.screen_w/2
    local padding = 60 -- From the edge of the screen

    -- Check if the right side of the anim is past the right edge of the screen
    if ( self.scroll_offset + anim_x + anim_w/2 + padding ) > right_x then
        -- Calculate difference, and scroll that delta
        self:EaseScroll( right_x - ( self.scroll_offset + anim_x + anim_w/2 + padding ) )
    end
    -- Check if the left side of the anim is past the left edge of the screen
    if ( self.scroll_offset + anim_x - anim_w/2 - padding ) < left_x then
        -- Calculate difference, and scroll that delta
        self:EaseScroll( left_x - ( self.scroll_offset + anim_x - anim_w/2 - padding ) )
    end
end

function NewGameAnims:GetSelected()
	for k, v in ipairs( self.anims ) do
		if v.selected then return v end
	end
end

-- Hides the other anims and centers the selected index
function NewGameAnims:ShowSingle( index )

    self:RefreshCenter()

    if self.showing_single ~= index then
        self.showing_single = index

        -- Scale up all the anims
        for k, v in ipairs( self.anims ) do
            v:SetScale( k == index and 1.5 or 1.2 )
            v:SetFocusable( false )
        end
        -- Lay them out
        self:StackChildrenRow( scaled_spacing )
        -- Save the amount the selected character will move to its new position
        local x_offset = ( self.sidebar_shown and SELECTED_X_SIDEBAR_MODE or SELECTED_X_CENTER ) - self.anims[index].x
        -- Save their new positions
        for k, v in ipairs( self.anims ) do
            v.new_x, v.new_y = v:GetPos()
            v.new_x = v.new_x + x_offset -- Add that offset to all anims, so they all move in the direction of the selected one
        end
        -- Put them back
        for k, v in ipairs( self.anims ) do
            v:SetScale( 1 )
            v:SetPos( v.default_x, v.default_y )
        end

        -- Animate them into their new position and scale
        for k, v in ipairs( self.anims ) do
            if k == index then
                -- Selected character. Animate it forward
                v:Show()
                v:AlphaTo( 1, ANIM_DURATION, easing.inOutQuad )
                -- v:MoveTo( SELECTED_X_SIDEBAR_MODE, SELECTED_Y, 0.2, easing.inOutQuad )
                v:MoveTo( v.new_x, SELECTED_Y, ANIM_DURATION, easing.inOutQuad )
                v:ScaleTo( 1, 1.5, ANIM_DURATION, easing.inOutQuad )
            else
                -- Other characters. Animate them out
                v:AlphaTo( 0, ANIM_DURATION, easing.inOutQuad, function() v:Hide() end )
                v:MoveTo( v.new_x, v.new_y, ANIM_DURATION, easing.inOutQuad )
                v:ScaleTo( 1, 1.2, ANIM_DURATION, easing.inOutQuad )
            end
            v:Disable()
        end
    end

    self:RefreshCenter()

    return self
end

function NewGameAnims:SetSidebarMode( sidebar_shown )

    self:RefreshCenter()
    
    if sidebar_shown ~= self.sidebar_shown then
        self.sidebar_shown = sidebar_shown
        -- If we already have a character in focus, move it to the right position
        if self.showing_single ~= nil then
            -- Position anim
            self.anims[self.showing_single]:MoveTo( ( self.sidebar_shown and SELECTED_X_SIDEBAR_MODE or SELECTED_X_CENTER ), nil, ANIM_DURATION, easing.inOutQuad )
        end
    end

    return self
end

-- Goes from focusing on a single anim to showing them all again
function NewGameAnims:ShowAll()
    if self.showing_single ~= nil then
        self.showing_single = nil

        for k, v in ipairs( self.anims ) do
            v:Show()
            v:AlphaTo( 1, ANIM_DURATION, easing.inOutQuad, function() v:Enable() end )
            v:ScaleTo( nil, 1, ANIM_DURATION, easing.inOutQuad )
            v:MoveTo( v.default_x, v.default_y, ANIM_DURATION, easing.inOutQuad )
            v:SetFocusable( true )
        end
    end

    self:RefreshCenter()

    return self
end

function NewGameAnims:AnimateIn()
	-- Prepare anim fade-in sequence
    local timing = 0.5
    local spacing = 300
    local button_adder = Updater.Parallel()
    button_adder:Add( Updater.Do( function() AUDIO:PlayEvent( "event:/ui/relationships_menu/character_whoosh_in" ) end ) )
    for i, widget in ipairs( self.anims ) do
    	local pos_x, pos_y = widget:GetPos()
        widget.enabled = false
        button_adder:Add( Updater.Ease( function(v) widget:SetPos(v, nil) end, pos_x+spacing*i, pos_x, timing, easing.outQuad ) )
    end
    -- Enable anims after they've animated in
    button_adder:Add( Updater.Do( function()
    	for i, widget in ipairs( self.anims ) do
    		widget.enabled = true
    	end
	end ) )
    self:RunUpdater( button_adder )
end

function NewGameAnims:GetDefaultFocus()
    for k, v in ipairs( self.anims ) do
        if v.selected then return v end
    end
end

function NewGameAnims:ScrollUp()
    self:Scroll( 100 )
    return self
end

function NewGameAnims:ScrollDown()
    self:Scroll( -100 )
    return self
end

function NewGameAnims:EaseScroll( delta )
    self.scroll_target = self.scroll_offset + delta
    return self
end

function NewGameAnims:Scroll( delta )
    -- If showing a single character, don't allow scrolling
    if self.showing_single then return self end

    -- Apply delta
    self.scroll_offset = self.scroll_offset + delta
    -- Enforce constraints, so characters don't go offscreen
    self.scroll_offset = math.min( self.scroll_offset, self.max_scroll_offset )
    self.scroll_offset = math.max( self.scroll_offset, self.min_scroll_offset )

    -- Position them
    self:SetPos( self.scroll_offset )

    -- If there was a target, and it's close enough to the current value, then reset it
    if self.scroll_target and math.abs( self.scroll_target - self.scroll_offset ) < 1 then
        self.scroll_target = nil
    end

    self:RefreshCenter()

    return self
end

function NewGameAnims:RefreshCenter()

    -- Get the screen center
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    local screen_center_x, screen_center_y = self:TransformFromWorld( screen_w/2, screen_h/2 )

    -- Get the screen size
    screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    SELECTED_Y = screen_center_y - screen_h*0.15 -- Calculate a vertical position so that the character isn't too low on 4:3 screens
    SELECTED_X_CENTER = screen_center_x -- The selected character goes to the horizontal center of the screen
    SELECTED_X_SIDEBAR_MODE = SELECTED_X_CENTER + (self.move_single_character and screen_w*0.2 or 0)

    return self
end

------------------------------------------------------------------------------------------
-- Displays a selectable character anim

local NewGameAnimatedCharacter = class("Widget.NewGameAnimatedCharacter", Widget.Clickable)

local NORMAL_SCALE = 0.52
local FOCUS_SCALE = 0.56
local SELECTED_FOCUS_SCALE = FOCUS_SCALE*1.02

function NewGameAnimatedCharacter:init( on_click_fn, index, agent_id, outfit_id, character_data )
    NewGameAnimatedCharacter._base.init(self)

    -- Set values
	self.on_click_fn = on_click_fn
	self.index = index
	self.agent_id = agent_id
	self.focus_color = UICOLOURS.SUBTITLE
    self:SetOnClickFn( function() self.on_click_fn( index ) end )
    self:DisableAnimateScales()

	-- Add hitbox
	self.hitbox = self:AddChild( Widget.SolidBox( 300, 900, 0x00FF0000 ) )

	-- Add selected glow
    self.glow = self:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ), 900, 2000 ) )
   		:SetHiddenBoundingBox( true )
    	:SetTintColour( 0xD86392ff )
    	:SetTintAlpha( 0 )

	-- Add shadow
    self.shadow = self:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ), 500, 200 ) )
   		:SetHiddenBoundingBox( true )
    	:SetTintColour( 0xD86392ff )
    	:SetTintAlpha( 0.3 )

    -- The floor animation
    self.floor_anim = self:AddChild( Widget.Anim() )
    local animname = "anim/hits_360.zip"
    self.floor_anim:AddAnims( GetAnim( animname ) )
    self.floor_anim:AddOverrideFile( animname )
    self.floor_anim:SetSpeed( 1.5 )
    self.floor_anim
        :SetHiddenBoundingBox(true)
        :SetBloom( 0.2 )
        :SetTintColour( UICOLOURS.ACCENT )
        :SetTintAlpha( 1 )
        :SetScale( 1, 0.33 )

	self.agent = character_data:CreateAgent()
    
    -- Look for this agent's selected outfit
    self.outfit_id = TheGame:GetGameProfile():GetSelectedOutfit( self.agent_id )
    for k, outfit_data in ipairs( Content.GetOutfitsForCharacter( self.agent_id ) ) do
        self.outfit = self.outfit or outfit_data.id -- If there wasn't a previously selected outfit, make the first one it
    end
	self.outfit = self.outfit_id and Content.GetOutfit( self.outfit_id )
	self.agent.build = self.outfit and self.outfit.build or self.agent.build
	-- Add anim
    self.anim = self:AddChild( Widget.AnimatedCharacter() )
   		:SetHiddenBoundingBox( true )
    self.anim:Refresh( self.agent )
    self.anim:SetScale( NORMAL_SCALE )
    local phase = math.random()
    self.anim.anim:SetPhase( phase )
	self.anim:SetHoverOutlineColour( self.focus_color )

    -- Add transition effect
    self.transition_glow = self:AddChild( Widget.Image( engine.asset.Texture("UI/newgame_outfit_transition.tex" ), 400, 400 ) )
        :SetHiddenBoundingBox( true )
        :SetTintColour( 0xD86392ff )
        :SetBloom( 0.15 )
        :SetTintAlpha( 0 )

    -- Add info label under anim
    local locked = TheGame:GetGameProfile():IsCharacterLocked(character_data.id)
    self.anim_progress = self:AddChild( Widget.NewGameAnimProgress( character_data, locked ) )

    if not locked then
        self:SetFocusDir( "down", self.anim_progress )
        self.anim_progress:SetFocusDir( "up", self )
        self.anim_progress:SetFocusDir( "down", self.anim_progress ) -- Prevents recursion to parent via FindFocusDir
    end

	-- Layout
	self.shadow:LayoutBounds( "center", "bottom", self.anim ):Offset( 0, -60 )
	self.shadow.default_x = self.shadow.x
	self.shadow.default_y = self.shadow.y
    self.anim_progress:LayoutBounds( "center", "bottom", self.anim ):Offset( 0, -110 )
end


function NewGameAnimatedCharacter:SetSelected( selected )
	self.selected = selected
	if self.anim then
		self.anim.selected = self.selected
		self.anim:UpdateButtonState()
	end
    self.glow:AlphaTo( self.selected and 0.2 or 0, 0.4 )
    self:UpdateImage()
	return self
end

function NewGameAnimatedCharacter:ChangeOutfit( outfit_id )

    if self.outfit_id ~= outfit_id then
    	-- Remove the old anim
    	local old
    	if self.anim then
            old = self.anim
            self.anim = nil
        end
    	
    	-- Add a new anim with the new outfit
    	self.agent = Agent( self.agent_id )
    	self.outfit_id = outfit_id
        self.outfit = self.outfit_id and Content.GetOutfit( self.outfit_id )
        self.agent.build = self.outfit and self.outfit.build or self.agent.build
        self.anim = self:AddChild( Widget.AnimatedCharacter() )
   			:SetHiddenBoundingBox( true )
        if old then self.anim:SendBehind( old ) end
        self.anim:Refresh( self.agent )
        self.anim:SetScale( old and old:GetScale() or FOCUS_SCALE )
        self.anim.anim:SetPhase( old and old.anim.phase or math.random() )
    	self.anim:SetHoverOutlineColour( self.focus_color )
        self.anim:SetClickFn( function() self.on_click_fn( self.index ) end )
    	self.anim.selected = true
    	self.anim:UpdateButtonState()
    	self.anim:Hide()

    	-- Transition
    	local duration = 0.5
        local padding = 100
    	local x, y = old:GetPos()
        local w, h = old:GetSize()
        self.w = self.w or w+padding
        self.h = self.h or h+padding
        local scaledh = self.h * (old and old:GetScale() or FOCUS_SCALE)
        local feet_y_offset = -50
        self.floor_anim:SetPos( 0, y-scaledh/2-feet_y_offset )
        self.anim:RunUpdater(
        	Updater.Series {
                Updater.Do( function()
                    old:SetSelectedColor( UICOLOURS.ACCENT )
                    old:SetSelected( true, 0.2 )
                    self.anim:SetSelectedColor( UICOLOURS.ACCENT )
                    self.anim:SetSelected( true, 0.2 )
                    self.anim:Show()
                    self.transition_glow:SetScale( 1 )
					AUDIO:PlayEvent( "event:/ui/main/fld/switch_outfit" )
                    AUDIO:PlayEvent( "event:/ui/main/gen/FE_change_outfit_scan" )
                end ),
            	Updater.Parallel{
                    Updater.Ease( function(v) old:SetScissor( x-self.w/2, (y-self.h/2)-self.h*v, self.w, self.h ) end, 0, 1, duration, easing.linear ),
            		Updater.Ease( function(v) self.anim:SetScissor( x-self.w/2, (y-self.h/2)+self.h*v, self.w, self.h ) end, 1, 0, duration, easing.linear ),
                    Updater.Ease( function(v) self.transition_glow:SetPos( 0, v ) end, y+scaledh/2, y-scaledh/2-feet_y_offset, duration, easing.linear ),
                    Updater.Series {
                        Updater.Ease( function(v) self.glow:SetTintAlpha( v ) end, 0.2, 0.4, duration/2, easing.linear ),
                        Updater.Ease( function(v) self.glow:SetTintAlpha( v ) end, 0.4, 0.2, duration/2, easing.linear ),
                    },
                    Updater.Series {
                        Updater.Ease( function(v) self.transition_glow:SetTintAlpha( v ) end, 0, 1, duration/4*2, easing.linear ),
                        Updater.Wait( duration/4*2 ),
                        Updater.Parallel{
                            Updater.Ease( function(v) self.transition_glow:SetTintAlpha( v ) end, 1, 0, duration/4, easing.linear ),
                            -- Updater.Ease( function(v) self.transition_glow:SetScale( v ) end, 1, 0.5, duration/4, easing.linear ),
                        },
                    },
                    Updater.Series {
                        Updater.Wait( duration*0.98 ),
                        Updater.Do( function() self.floor_anim:Play( "10", false ) end ),
                    },
            	},
                Updater.Do( function()
                    old:Remove()
                    self.anim:SetSelected( false, 0.2 )
                    self.anim:SetScissor( nil )
                    -- self.anim:Play( "outfit_change", false )
                end )
        	}
        )
    end

	return self
end

function NewGameAnimatedCharacter:UpdateImage()
    if ( (self.enabled and self.hover) or self.focus or self.selected ) then
    	AUDIO:PlayEvent(SoundEvents.mouseover)
        if self.anim then
            self.anim:ScaleTo( nil, (self.selected and ((self.enabled and self.hover) or self.focus)) and SELECTED_FOCUS_SCALE or FOCUS_SCALE, 0.1 )
            self.anim:SetOutline( 5, UICOLOURS.SUBTITLE )
        end
        if self.image then self.image:ScaleTo( nil, (self.selected and ((self.enabled and self.hover) or self.focus)) and SELECTED_FOCUS_SCALE or FOCUS_SCALE, 0.1 ) end
        self.shadow:ScaleTo( nil, 1.2, 0.1 )
        self.shadow:MoveTo( self.shadow.default_x, self.shadow.default_y-20, 0.1 )
    else
        if self.anim then
            self.anim:ScaleTo( nil, NORMAL_SCALE, 0.1 )
                :HideOutline()
        end
        if self.image then self.image:ScaleTo( nil, NORMAL_SCALE, 0.1 ) end
        self.shadow:ScaleTo( nil, 1, 0.1 )
        self.shadow:MoveTo( self.shadow.default_x, self.shadow.default_y, 0.1 )
    end
end

function NewGameAnimatedCharacter:OnGainFocus()
    NewGameAnimatedCharacter._base.OnGainFocus(self)

    if not self.selected then self.on_click_fn( self.index ) end

    self:UpdateImage()
end

function NewGameAnimatedCharacter:OnLoseFocus()
    NewGameAnimatedCharacter._base.OnLoseFocus(self)
    self:UpdateImage()
end

function NewGameAnimatedCharacter:OnGainHover()
    NewGameAnimatedCharacter._base.OnGainHover(self)
    self:UpdateImage()
end

function NewGameAnimatedCharacter:OnLoseHover()
    NewGameAnimatedCharacter._base.OnLoseHover(self)
    self:UpdateImage()
end


------------------------------------------------------------------------------------------
-- Displays a locked available character image

local NewGameLockedCharacter = class("Widget.NewGameLockedCharacter", Widget.NewGameAnimatedCharacter)

function NewGameLockedCharacter:init( on_click_fn, index, agent_id, outfit_id, character_data )
    NewGameLockedCharacter._base.init( self, on_click_fn, index, agent_id, outfit_id, character_data )

    self.anim.anim:SetTintColour( 0x000000FF )
    if not self.anim.tint_add_constant then
       self.anim.tint_add_constant = engine.rendering.ShaderConstantValue("TINT_ADD", 0,0,0,0)
       self.anim.anim.model:SetShaderConstant(self.anim.tint_add_constant)
    end
    self.anim.tint_add_constant:SetValue( HexColour( 0x121d24FF ) )

end

------------------------------------------------------------------------------------------
-- Displays a character's progress

local NewGameAnimProgress = class("Widget.NewGameAnimProgress", Widget)

NewGameAnimProgress.show_tooltip_on_focus = true
NewGameAnimProgress.can_focus = true

local ICON_W = 50
local ICON_SPACING = 6
local CONTENT_W = 190

function NewGameAnimProgress:init( character_data, locked )
    NewGameAnimProgress._base.init(self)

    self.icon = self:AddChild( Widget.Image( nil, ICON_W, ICON_W ) )
        :Bloom( 0.05 )
    self.label = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL, "" ) )
        :LeftAlign()
        :SetAutoSize( CONTENT_W )
        :SetWordWrap( true )
        :Bloom( 0.1 )
    self.value = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL, "" ) )
        :LeftAlign()
        :SetAutoSize( CONTENT_W )
        :SetWordWrap( true )
        :Bloom( 0.1 )
    self.progress_bar = self:AddChild( Widget.ProgressBar( CONTENT_W, 3 ) )
        :OrientH()
        :SetOutlineColour( 0x00000000 )
        :SetBGColour( 0x00000050 )
        :SetBarColour( UICOLOURS.ADVANCEMENT )
        :SetBarBloom( 0.1 )

    self:Refresh( character_data, locked )
end

function NewGameAnimProgress:OnGainFocus()
    self.label:SetGlyphColour( UICOLOURS.WHITE )
end

function NewGameAnimProgress:OnLoseFocus()
    self.label:SetGlyphColour( UICOLOURS.ADVANCEMENT )
end

function NewGameAnimProgress:Refresh( character_data, locked )

    if character_data.upcoming_character or locked then
        self.icon:SetTexture( engine.asset.Texture( "UI/newgame_sidebar_locked.tex") )
            :SetTintColour( 0x4A6A82FF ):Show()
        self.label:SetText( LOC"UI.NEW_GAME_SCREEN.ANIM_PROGRESS_LOCKED" )
            :SetGlyphColour( 0x4A6A82FF )
        self.value:Hide()
        self.progress_bar:Hide()
        -- Layout!
        self.label:LayoutBounds( "after", "center", self.icon ):Offset( ICON_SPACING/3, 0 )
    else
        self.icon:Hide()

        local unlocks_unlocked, unlocks_total = ActUtil.GetUnlocksProgress( character_data.id )
        local outfits_unlocked, outfits_total = ActUtil.GetOutfitsProgress( character_data.id )
        -- Decrement because every character has 1 default outfit which we don't count towards progress.
        outfits_unlocked = math.max( 0, outfits_unlocked - 1 )
        outfits_total = math.max( 0, outfits_total - 1 )

        local unlocked, total = unlocks_unlocked + outfits_unlocked, unlocks_total + outfits_total

        local agent_name = Content.GetCharacterDef( character_data.data.player_agent ):GetLocalizedName()
        local tt = loc.format( LOC"UI.NEW_GAME_SCREEN.ANIM_PROGRESS_TOOLTIP", agent_name, unlocks_unlocked, unlocks_total, outfits_unlocked, outfits_total )

        for i, act in ipairs( character_data.acts ) do  
            if act.data.game_type == GAME_TYPE.CAMPAIGN or act.data.game_type == GAME_TYPE.BRAWL then
                local u, t = ActUtil.GetPrestigeProgress( act.id )
                if t > 0 then
                    unlocked = unlocked + u
                    total = total + t
                    tt = tt .. "\n" .. loc.format( LOC"UI.NEW_GAME_SCREEN.ANIM_PROGRESS_PRESTIGE", u, t, act:GetLocalizedName() )
                end
            end
        end
        self:SetToolTip( tt )

        self.label:SetText( LOC"UI.NEW_GAME_SCREEN.ANIM_PROGRESS_PROGRESS" )
            :SetGlyphColour( UICOLOURS.ADVANCEMENT )
        self.value:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.ANIM_PROGRESS_VALUE", unlocked/total ) )
            :SetGlyphColour( UICOLOURS.ADVANCEMENT )
        

        self.progress_bar:SetProgress( unlocked/total )
        -- Layout!
        self.progress_bar:LayoutBounds( "after", "center", self.icon ):Offset( ICON_SPACING, -10 )
        self.label:LayoutBounds( "left", "above", self.progress_bar ):Offset( -1, 4 )
        self.value:LayoutBounds( "right", "above", self.progress_bar ):Offset( 0, 4 )
 
    end
end

function NewGameAnimProgress:OnScreenModeChange( sm )
    NewGameAnimProgress._base.OnScreenModeChange( self, sm )

    self:SetLayoutScale( LAYOUT_SCALE[sm] )
end