
require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/menu"
require "ui/widgets/solidbox"
require "ui/widgets/panelbutton"
require "ui/screens/optionscreen"
require "ui/screens/compendium"

local PauseMenu = class("Screen.PauseMenu", Widget.Screen)

local PANEL_H = 500
local PANEL_PADDING = 45
local BUTTON_SPACING = 10
local CONTENT_W = 400
local CONTENT_H = 400

local function IsSignedIn()
    if SPECIAL_BUILD_TYPE == 128 then
        return TheGame:IsSignedIn()
    else
        return true
    end
end

function PauseMenu:init()
    PauseMenu._base.init(self)
    self.is_popup = true
    self:SetupUnderlay( true )
    self.screen_mix_event = "event:/mix_events/PauseMenu"
    
    local sep = 32 --between title and body
    local button_size = 60 --height

    self.main_options = {}
    table.insert(self.main_options, {txt=TheGame:Str"UI.PAUSEMENU.RESUME", name = "RESUME", fn = function() self:OnResume() end, cta = true})
    table.insert(self.main_options, {txt=TheGame:Str"UI.PAUSEMENU.OPTIONS", fn = function() self:OnOptions() end})
    
    if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('negotiation_tutorial') or TheGame:GetGameProfile():HasSeenMessage('battle_tutorial') then
        table.insert(self.main_options, {txt=TheGame:Str"UI.PAUSEMENU.TUTORIALS", fn = function() self:OnTutorials() end})
    end
    
    if not TheGame:IsDemo() then
        table.insert(self.main_options, {txt=TheGame:Str"UI.PAUSEMENU.CARDCOMPENDIUM", fn = function() self:ShowCompendium() end, snd = "event:/ui/save_slot_menu/select_abandon_run"})

        if TheGame:GetGameState() and TheGame:GetGameState().save_file_name then
            table.insert(self.main_options, {txt=TheGame:Str"UI.PAUSEMENU.ABANDONRUN", fn = function() self:OnAbandon() end, snd = "event:/ui/save_slot_menu/select_abandon_run"})
        end
    end

    local in_convo = TheGame:FE():FindScreen( Screen.ConversationScreen )
    local signed_in = IsSignedIn()
    table.insert(self.main_options, {txt= (in_convo or not signed_in) and LOC"UI.PAUSEMENU.RETURNTOMAINMENU" or LOC"UI.PAUSEMENU.RETURNTOMAINMENU_SAVE", fn = function() self:OnExit() end})

    if SPECIAL_BUILD_TYPE ~= (1024+64) then
        table.insert(self.main_options, {txt= (in_convo or not signed_in) and LOC"UI.PAUSEMENU.RETURNTODESKTOP" or LOC"UI.PAUSEMENU.RETURNTODESKTOP_SAVE", fn = function() self:OnExit(true) end})
    end

    self:SetAnchors( "center", "center" )
    CONTENT_H = ( button_size + BUTTON_SPACING )*#self.main_options
    self.panel = self:AddChild( Widget.WidePanel( PANEL_H ) )
        :SetIcon( engine.asset.Texture("UI/ic_pause.tex" ) )

    self.menu = self:AddChild( Widget.Menu( Widget.PanelButton, 350, button_size ) )
        :SetLayout( nil, nil, nil, -BUTTON_SPACING )
        :SetFontSize( 32 )
        :SetLastDefault(true)
    self.menu:SetOptions( self.main_options ):SetAnchors( "center", "center" )
    self.menu:SetHoverCheck( false ) -- Disable the buttons until they're shown
    self.menu:LayoutBounds( "center", "bottom", self.panel ):Offset( 0, 60 )

    -- Setup character anim display
    self.owner = TheGame:GetGameState():GetPlayerAgent()
    self.character = self:AddChild( Widget.AnimatedCharacter() )
        :SetScale( 1, 1 )
        :Refresh( self.owner )
    self.character:SetHoverOutlineColour( UICOLOURS.SUBTITLE )
    self.character_animation_done = true
    self.character.anim:SetAnimDoneFn( function() self.character_animation_done = true end )
    self.character:SetClickFn( function()
        if self.character_animation_done then
            self.character_animation_done = false
            self.character:Emote( "crossed", true )
        end
    end )
    self.character:SetAutoLoopEnabled( false )
    self.character:Emote( "crossed", true )
    self.character:SetPos( -RES_X*0.35, -350 )

    self.open_sound = "event:/ui/main/gen/menu_open"
    self.close_sound = "event:/ui/main/gen/menu_close"

    -- Hide objects until they're animated in
    self.panel:PrepareAnimation()
    self.menu:PrepareAnimation()
end

function PauseMenu:OnOpen()
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
    self.panel:FadeIn( function() 
        -- Start sequencing in the cards
        self.menu:AnimOptions()
        -- Activate the menu buttons
        self.menu:SetHoverCheck( true )
    end )
end

function PauseMenu:OnControlModeChange( cm, device_id )
    PauseMenu._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
    end
    if cm == CONTROL_MODE.TOUCH then
    end
    if cm == CONTROL_MODE.GAMEPAD then
    end
end

function PauseMenu:OnScreenModeChange( sm )
    PauseMenu._base.OnScreenModeChange( self, sm )

    if sm == SCREEN_MODE.MONITOR then
        self.character:SetPos(-RES_X*0.33, -310)
    elseif sm == SCREEN_MODE.SMALL
        or sm == SCREEN_MODE.TV then
        self.character:SetPos(-RES_X*0.34, -310)
    end
    self.menu:SetLayoutScale( LAYOUT_SCALE[sm] )
    self:LayoutMenu()
end

function PauseMenu:LayoutMenu()
    local sm = TheGame:FE():GetScreenMode()
    local w, h = self.menu:GetScaledSize()
    self.panel:SetHeight( h + SPACING.M1*2*LAYOUT_SCALE[sm] )
    self.menu:LayoutBounds( "center", "center", self.panel ):Offset( 0, 4 )
end

function PauseMenu:GetDefaultFocus()
    return self.menu:GetButton( 1 )
end

function PauseMenu:OnExit(to_desktop)
        
    local in_convo = self:GetFE():FindScreen( Screen.ConversationScreen )
    local can_not_save = not ( TheGame:GetGameState() and TheGame:GetGameState().save_file_name) or in_convo

    if to_desktop then
        local body = can_not_save and TheGame:Str"UI.DIALOGS.INGAME_DESKTOP_BODY_IN_CONVO" or TheGame:Str"UI.DIALOGS.INGAME_DESKTOP_BODY"
        TheGame:FE():PushScreen( Screen.YesNoPopup( LOC"UI.DIALOGS.QUIT_TITLE", body ) )
            :SetFn(
                function(v)
                    if v == Screen.YesNoPopup.YES then 
                        if can_not_save then
                            self:GetFE():FadeTransition( function() TheGame:Shutdown() end )
                        else
                            self:GetFE():FadeTransition( function() 
                                if not DoAutoSave(function() TheGame:Shutdown() end) then
                                    TheGame:Shutdown()
                                end
                            end )
                        end
                    end 
                end )
    else
        local title = TheGame:Str"UI.DIALOGS.EXIT_TITLE"
        local body
        if not IsSignedIn() then
            body = LOC"UI.DIALOGS.EXIT_BODY_NO_SAVE"
        else
            body = can_not_save and TheGame:Str"UI.DIALOGS.EXIT_BODY_IN_CONVO" or TheGame:Str"UI.DIALOGS.EXIT_BODY"
        end
        TheGame:FE():PushScreen( Screen.YesNoPopup( title, body ) )
            :SetFn(
                function(v)
                    if v == Screen.YesNoPopup.YES then 
                        AUDIO:PlayEvent( SoundEvents.quit_game )
                        if can_not_save then
                            TheGame:Reset()
                        else
                            if not DoAutoSave(function() TheGame:Reset() end) then
                                TheGame:Reset()
                            end

                        end
                        
                    end 
                end )
    end


end

function PauseMenu:OnTutorials()
    local options = {}

    if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('negotiation_tutorial') then
        table.insert(options, {txt=TheGame:Str"UI.PAUSEMENU.NEGOTIATION", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "negotiation_tutorial")) end})
    end

    if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('battle_tutorial') then
        table.insert(options, {txt=TheGame:Str"UI.PAUSEMENU.BATTLE", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "battle_tutorial")) end})
    end

    if TheGame:GetGameProfile():HasSeenMessage('rook_negotiation_tutorial') then
        table.insert(options, {txt=TheGame:Str"UI.PAUSEMENU.ROOK_NEGOTIATION", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "rook_negotiation_tutorial")) end})
    end

    if TheGame:GetGameProfile():HasSeenMessage('rook_battle_tutorial') then
        table.insert(options, {txt=TheGame:Str"UI.PAUSEMENU.ROOK_BATTLE", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "rook_battle_tutorial")) end})
    end

    if TheGame:GetDebug() or TheGame:GetGameProfile():HasSeenMessage('smith_battle_tutorial') then
        table.insert(options, {txt=TheGame:Str"UI.PAUSEMENU.SMITH_BATTLE", fn = function() TheGame:FE():PushScreen( Screen.SlideshowScreen( "smith_battle_tutorial"):SetAutoAdvance(false) ) end } )
    end

    
    table.insert(options, {txt=TheGame:Str"UI.PAUSEMENU.BACK", name = "BACK", fn = function() self:OnBack() end})
    self.menu:SetOptions( options )
    self.menu:AnimOptions()
    self.showing_submenu = true
end

function PauseMenu:OnBack()
    self.showing_submenu = false
    self.menu:SetOptions( self.main_options )
    self:LayoutMenu()
    self.menu:AnimOptions()
end

function PauseMenu:OnResume()
    self:GetFE():PopScreen( self )
end

function PauseMenu:OnOptions()
    self:GetFE():PushScreen(Screen.OptionScreen())
end

function PauseMenu:ShowCompendium()
    TheGame:FE():FadeTransition(function()
        TheGame:FE():PushScreen( Screen.Compendium() )
    end) 
end

function PauseMenu:OnAbandon()
    TheGame:FE():PushScreen( Screen.YesNoPopup( TheGame:Str"UI.PAUSEMENU.ABANDONRUN_TITLE", TheGame:Str"UI.PAUSEMENU.ABANDONRUN_BODY" ) )
        :SetFn(
            function(v)
                if v == Screen.YesNoPopup.YES then
                    TheGame:AbandonGame()
                end 
            end )
end


function PauseMenu:HandleControlUp( control, device )
    if control:Has( Controls.Digital.MENU_CANCEL ) then
        if self.showing_submenu then
            self:OnBack()
        else
            self:OnResume() 
        end
        return true

    end

end

