local PerkViewPopup = class( "Screen.PerkViewPopup", Screen.InfoPopup )

PerkViewPopup.CONTROL_MAP = {
    {
        control = Controls.Digital.GAMEPAD_X,

        test = function(self)
            return self.can_unlock
        end,

        fn = function(self)
            TheGame:GetGameProfile():BuyPerk(self.id)
            self:GetFE():PopScreen(self)
            if self.fn then
                self.fn(true)
            end
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.UNLOCK", Controls.Digital.GAMEPAD_X ))
        end,
    },
    {
        control = Controls.Digital.MENU_CANCEL,

        fn = function(self)
            
            self:GetFE():PopScreen(self)
            if self.fn then
                self.fn(false)
            end
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.CLOSE",Controls.Digital.MENU_CANCEL))
        end,
    },
}

function PerkViewPopup:init( id, fn )
    local perk = Content.GetGraft(id)
    local cost = perk.unlock_cost
    local points = TheGame:GetGameProfile() and TheGame:GetGameProfile():GetPerkPoints() or 0
    local unlocked = TheGame:GetGameProfile() and TheGame:GetGameProfile():HasPerkUnlocked(id)
    self.can_unlock = false
    
    self.id = id
    self.fn = fn

    local txt = ""
    
    if unlocked then
    elseif points >= cost then
        self.can_unlock = true
        txt = loc.format(LOC"UI.PERK_SCREEN.UNLOCK_PERK", cost)
    else
        txt = loc.format(LOC"UI.PERK_SCREEN.NOT_ENOUGH_POINTS", cost)
    end

    local title = perk:GetLocalizedName() or ""

    --Probably want to show the perk details here. Might want to make a totally unique screen?

    self._base.init( self, title, txt, LOC"UI.PERK_SCREEN.UNLOCK_BUTTON" )

    if not unlocked then
        self.button_ok:SetIcon( global_images.perk )
            :SetFn(function()
                TheGame:GetGameProfile():BuyPerk(id)
                self:GetFE():PopScreen(self)
                if self.fn then
                    self.fn(true)
                end
            end)
    end
    
    -- Add cancel-button
    self.cancel = self.buttons:AddChild( Widget.IconButton() )
        :SetIcon( global_images.cancel )
        :SetText(LOC"UI.PERK_SCREEN.CLOSE")
        :SetFn(function() 
            self:GetFE():PopScreen(self)
            if self.fn then
                self.fn(false)
            end
        end)

    if not self.can_unlock then
        self.button_ok:Disable()
    end
        
    self:Layout()
end


function PerkViewPopup:HarvestRobotOptions()
    self:AddRobotOption(self.cancel)
end

