require "ui/widgets/optionpicker"
require "ui/widgets/screen"
require "ui/widgets/tooltip_graft"
require "ui/screens/replacegraftscreen"

-----------------------------------------------------------------------
--

local PickCoinScreen = class("Screen.PickCoinScreen", Widget.Screen)

local PANEL_H = 640
local SHADOW_OFFSET = 480

local SLOT_W = 390
local SLOT_H = 490

PickCoinScreen.CONTROL_MAP = {
    {
        control = Controls.Digital.MENU_CANCEL,
        test = function(self)
            if self.close_button and not self.show_reticule then
                return true
            end
        end,
        fn = function(self)
            self.close_button:Click()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end
    },
    {
        control = Controls.Digital.MENU_CANCEL,
        test = function(self)
            if self.money_btn and not self.show_reticule then
                return true
            end
        end,
        fn = function(self)
            self.money_btn:Click()
            return true
        end,
        hint = function(self, left, right)
            if self.skip_shills_reward and self.skip_shills_reward > 0 then
                table.insert(right, loc.format(LOC"UI.CONTROLS.TAKE_CARD_MONEY", Controls.Digital.MENU_CANCEL, self.skip_shills_reward))
            else
                table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
            end
        end
    }
}

function PickCoinScreen:init( grafts, no_skip, on_done, no_money_for_skip, no_replacement )
    PickCoinScreen._base.init( self )
    self.screen_mix_event = "event:/mix_events/Popup"
    self.no_replacement = no_replacement
    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors("center", "center")

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Backing panel
    self.panel = self:AddChild( Widget.WidePanel( 700 ) )
        :SetColour( UICOLOURS.MONEY )
        :SetIcon( engine.asset.Texture( "UI/ic_draft_luckycoin.tex") )
        :SetTitles( LOC"UI.PICK_COIN_SCREEN.TITLE", LOC"UI.PICK_COIN_SCREEN.SUBTITLE" )
        :SetTopTitlesMargin( 15 )

    self.chosen_graft = nil
    self.grafts = grafts
    self.on_done = on_done

    self.options = self:AddChild( Widget() )
    self.shadows = self:AddChild( Widget() )
    self.warnings = self:AddChild( Widget() )

    -- Show current coin
    self.owner = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent()
    self.current_coin_option = self.options:AddChild( Widget.PickCoinOption( self.owner:GetAspect("graft_owner"):GetGrafts( GRAFT_TYPE.COIN )[1], self ) )
        :SetCurrentCoin()
    -- Add a shadow
    self.current_coin_option.shadow = self.shadows:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
        :SetHiddenBoundingBox( true )
        :SetSize( 500, 100 )
        :SetTintColour( UICOLOURS.MONEY )
        :SetPos( 0, -SHADOW_OFFSET )
        :SetTintAlpha( 0.8 )

    -- Add graft options
    for i, graft in ipairs(grafts) do
        local widget = self.options:AddChild( Widget.PickCoinOption( graft, self ) )
        widget:SetOnClickFn( function() self:OnSelectGraft( graft ) end )

        -- Add a shadow
        widget.shadow = self.shadows:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
            :SetHiddenBoundingBox( true )
            :SetSize( 500, 100 )
            :SetTintColour( UICOLOURS.MONEY )
            :SetPos( 0, -SHADOW_OFFSET )
            :SetTintAlpha( 0.8 )

        TheGame:GetGameProfile():SetSeenGraft( graft.id )
    end
    self.options:StackChildrenRow( 30 )

    if not no_skip then
        self.no_money_for_skip = no_money_for_skip
        -- self:SetMoneyButton()
    end

    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.close_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", 
        function()
            AUDIO:PlayEvent( "event:/ui/grafts_menu/close" )
            self:GetFE():PopScreen( self )
        end ) )
    self.close_button:SetIcon( global_images.close )
    self.close_button:LayoutBounds( "left", "above", 60, 60 )
    -- Continue button
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.button_continue = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.DAY_OVER_SCREEN.BUTTON_CONTINUE", function() self:OnClickContinue() end ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Hide()
        :SetToolTip( LOC"UI.OPTIONS_SCREEN.CONFIRM_SELECT_TOOLTIP" )

    self:Layout()

    self:SetSounds( "event:/ui/choose_grafts_screen/open", "event:/ui/choose_grafts_screen/close" )
end

function PickCoinScreen:OnClickContinue( graft )
    graft = graft or self.selected_graft
    if self.no_replacement then
        self.chosen_graft = graft
        self:GetFE():PopScreen( self )
    else
        if graft then
            if TheGame:GetGameState():GetPlayerAgent().graft_owner:CanInstallGraft(graft.id) then
                TheGame:GetGameState():GetPlayerAgent().graft_owner:AddGraft(graft)
                self:GetFE():PopScreen( self )
            else
                if graft:GetDef().type == GRAFT_TYPE.COIN then
                    local graft_owner = TheGame:GetGameState():GetPlayerAgent().graft_owner
                    graft_owner:ReplaceGraft( graft_owner:FindGraft(function(graft) return graft:GetDef().type == GRAFT_TYPE.COIN end), graft )
                    self.chosen_graft = graft
                    self:GetFE():PopScreen( self )
                else
                    local screen = Screen.ReplaceGraftScreen( graft, 
                        function(replaced) 
                            if replaced then
                                self.chosen_graft = graft
                                self:GetFE():PopScreen( self )
                            end
                        end )
                    TheGame:FE():InsertScreen( screen )
                end
            end
        end
    end
end

function PickCoinScreen:OnSelectGraft( graft )
    AUDIO:PlayEvent( "event:/ui/choose_grafts_screen/select_graft" )
    if self.select_then_continue_mode then
        -- We're in select-then-continue mode, and the selected graft isn't the clicked one
        -- Select this one
        self.selected_graft = graft
        -- Unselect options
        for k, widget in ipairs( self.options.children ) do
            widget:SetSelected( widget.graft == self.selected_graft )
        end
        -- And show the confirm button
        self.button_continue:Show()
    elseif graft then
        self.selected_graft = graft
        self:OnClickContinue()
    end
end

function PickCoinScreen:OnOpen()
    PickCoinScreen._base.OnOpen( self )
    self.options:SetTintAlpha( 0 )
    self.shadows:SetTintAlpha( 0 )

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )

    -- self.panel:FadeIn( function()
        self.options:AlphaTo( 1, 0.3 )
        self.shadows:AlphaTo( 1, 0.3 )
    -- end )
end

function PickCoinScreen:OnControlModeChange( cm, device_id )
    PickCoinScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.select_then_continue_mode = false or engine.inst:GetSetting("gameplay", "confirm_select", true)
        if self.money_btn then self.money_btn:Show() end
        self.bottom_left:Show()
        self.bottom_right:Show()
        -- Unselect options
        for k, widget in ipairs( self.options.children ) do
            widget:SetSelected( false )
        end
        self.selected_graft = nil
        self.button_continue:Hide()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.select_then_continue_mode = true
        if self.money_btn then self.money_btn:Show() end
        self.bottom_left:Show()
        self.bottom_right:Show()
        -- Unselect options
        for k, widget in ipairs( self.options.children ) do
            widget:SetSelected( false )
        end
        self.selected_graft = nil
        self.button_continue:Hide()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.select_then_continue_mode = false
        if self.money_btn then self.money_btn:Hide() end
        self.bottom_left:Hide()
        self.bottom_right:Hide()
        -- Unselect options
        for k, widget in ipairs( self.options.children ) do
            widget:SetSelected( false )
        end
        self.selected_graft = nil
        self.button_continue:Hide()
    end
end

function PickCoinScreen:OnScreenModeChange( sm )
    PickCoinScreen._base.OnScreenModeChange( self, sm )

    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )
    
    local new_option_h, new_panel_h
    if sm == SCREEN_MODE.MONITOR then
        new_option_h = SLOT_H
        new_panel_h = PANEL_H
    elseif sm == SCREEN_MODE.SMALL
        or sm == SCREEN_MODE.TV then
        new_option_h = SLOT_H + 60
        new_panel_h = PANEL_H + 110
    end

    -- Resize options
    for k, widget in ipairs( self.options.children ) do
        widget:SetHeight( new_option_h )
    end
    -- Resize container panel
    self.panel:SetHeight( new_panel_h )
    self.options:LayoutBounds( "center", "bottom", self.panel )
        :Offset( 0, 40 )

    -- Align the shadows
    for k, widget in ipairs( self.options.children ) do
        widget.shadow:LayoutBounds( "center", nil, widget )
    end

    if self.money_btn then self.money_btn:SetLayoutScale( LAYOUT_SCALE[sm] ) end

    self:Layout()
end

function PickCoinScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
    if GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

function PickCoinScreen:Layout()
    if self.money_btn then self.money_btn:LayoutBounds( "center", "bottom", self.panel ):Offset( 0, 43 ) end
end

function PickCoinScreen:OnClose()
    PickCoinScreen._base.OnClose( self )
    

    if self.chosen_graft then
        self:SendMetrics( self.chosen_graft.id )
    end

    if self.on_done then
        self.on_done( self.chosen_graft )
    end
end

function PickCoinScreen:SendMetrics( selected_id )
    local graft_choices = {}
    for i, graft in ipairs(self.grafts) do
        table.insert(graft_choices, {ID = graft.id})
    end

    SendMetricsData("DRAFTED_GRAFT", {
        SELECTED_ID = selected_id,
        GRAFT_CHOICES = graft_choices,
        PLAYER_DATA = TheGame:GetGameState():GetPlayerState(),
    })
end

function PickCoinScreen:SetMoneyButton()
    if self.no_money_for_skip then
        self.skip_shills_reward = 0
        
        self.money_btn = self:AddChild( Widget.IconButton( LOC"UI.PICK_COIN_SCREEN.SKIP_GRAFT", nil, 230 ) )
        self.money_btn:SetFn( function( )
            self:GetFE():PopScreen( self )
        end )

    else
        self.skip_shills_reward = GRAFT_SKIP_REWARD
        local title = LOC"UI.DRAFT_SCREEN.MONEY_BTN_TITLE"
        local subtitle = loc.format( LOC"UI.DRAFT_SCREEN.MONEY_BTN_SUBTITLE", self.skip_shills_reward )

        self.money_btn = self:AddChild( Widget.IconButton( LOC"UI.PICK_COIN_SCREEN.SKIP_GRAFT", nil, 230 ) )
        self.money_btn:SetFn( function( )
            AUDIO:PlayEvent( "event:/ui/select_cards/aquired_money" )
            TheGame:GetGameState():GetCaravan():AddMoney( GRAFT_SKIP_REWARD )

            self:SendMetrics( "MONEY" )

            self:GetFE():PopScreen( self )
        end )

        self.money_btn:SetIcon( global_images.shills )
            :SetSize( 300, 50 )
            :SetTextSize( 24 )
            -- :SetScale( 1.4, 1.4 )
        self.money_btn:SetText( title, subtitle )
    end
    self.money_btn:LayoutBounds( "center", "bottom", self.panel )
        :Offset( 0, 43 )
end

function PickCoinScreen:HarvestRobotOptions()
    self:AddRobotOption(self.money_btn)
    for k, widget in ipairs(self.options.children) do
        self:AddRobotOption(widget)
    end
end

function PickCoinScreen:GetDefaultFocus()
    return self.options and self.options.children and #self.options.children > 1 and self.options.children[2]
end

-----------------------------------------------------------------------
-- A single clickable graft

local PickCoinOption = class("Widget.PickCoinOption", Widget.Clickable)

local PADDING = 20
local GRAFT_SIZE = 180
local bg_border = 70
local bg_border_scale = 0.5

function PickCoinOption:init(graft)
    PickCoinOption._base.init( self )
    self.close_sound = "event:/ui/choose_grafts_screen/close"
    
    self.width = SLOT_W
    self.height = SLOT_H
    self.graft = graft
    self.features = {}
    self:SetToolTipClass( Widget.TooltipKeywords ):SetToolTip( self.features )

    -- Background panel
    local nineslice_w, nineslice_h = SLOT_W + bg_border*2*bg_border_scale, SLOT_H + bg_border*2*bg_border_scale
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/graft_bg.tex") ) )
        :SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
        :SetNineSliceBorderScale( bg_border_scale )
        :SetSize( nineslice_w, nineslice_h )
        :SetTintColour( UICOLOURS.MONEY )

    -- Gradient
    self.gradient = self:AddChild( Widget.Panel( engine.asset.Texture("UI/graft_gradient.tex") ) )
        :SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
        :SetNineSliceBorderScale( bg_border_scale )
        :SetSize( nineslice_w, nineslice_h )
        :SetTintColour( UICOLOURS.MONEY )
        :SetTintAlpha( 1 )

    -- Graft icon
    self.graft_icon = self:AddChild( Widget.GraftSlot( GRAFT_TYPE.COIN, GRAFT_SIZE ) )
        :DisableToolTip(true)
        :Refresh( self.graft )
        :SetFrameImageRatio( 1.25 )

    -- Graft details
    self.title = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_NAME ) )
        :SetWordWrap( true )
        :SetAutoSize( self.width - PADDING*2 )
        :LeftAlign()
        :Bloom( 0.1 )
        :SetText( graft:GetName() )
        :SetGlyphColour( UICOLOURS.MONEY )

    local rarity_color = CARD_RARITY_COLOURS[ self.graft:GetRarity() or CARD_RARITY.UNIQUE ]
    local rarity_icon_string = GetCardRarityIconString( self.graft:GetRarity() )
    local subtitle_text = TheGame:Str( string.format( "UI.GRAFT_COLLECTION.%s.%s", self.graft:GetRarity(), self.graft:GetType() ))

    self.subtitle = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE ) )
        :SetWordWrap( true )
        :SetAutoSize( self.width - PADDING*2 )
        :SetGlyphColour( rarity_color )
        :LeftAlign()
        :SetText( rarity_icon_string .. subtitle_text )

    self.desc = self:AddChild( Widget.Label( "body", FONT_SIZE.ITEM_DESC ) )
        :SetWordWrap( true )
        :SetAutoSize( SLOT_W - PADDING*2 )
        :LeftAlign()
        :OverrideLineHeight( 23 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
        :SetText( graft:GetDesc( self ) )

    -- Overlay panel
    self.overlay = self:AddChild( Widget.Panel( engine.asset.Texture("UI/graft_overlay.tex") ) )
        :SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
        :SetNineSliceBorderScale( bg_border_scale )
        :SetSize( nineslice_w, nineslice_h )
        :SetTintColour( UICOLOURS.MONEY )
        :Bloom( 0.2 )
end

function PickCoinOption:SetCurrentCoin()
    self.current_coin = true

    self.gradient:Hide()
    self.overlay:Hide()

    self.current_coin_bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/dropdown_inner.tex") ) )
        :SetTintColour( UICOLOURS.MONEY )
        :Bloom( 0.03 )
    self.current_coin_label = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE, LOC"UI.PICK_COIN_SCREEN.CURRENT_COIN" ) )
        :SetGlyphColour( UICOLOURS.DARK_BODYTEXT )
    local label_w, label_h = self.current_coin_label:GetSize()
    label_w = self.width
    label_h = label_h + 4
    self.current_coin_bg:SetSize( label_w, label_h )

    self:SetFocusable( false )
    return self
end

function PickCoinOption:SetSelected( selected )
    self.selected = selected
    self:UpdateImage()
    return self
end

function PickCoinOption:UpdateImage()
    local timing = 0.2
    if self.enabled and ( self.hover or self.focus or self.selected ) and not self.current_coin then
        self:ScaleTo( nil, self.selected and 1.08 or 1.05, 0.2, easing.outQuad )
        if self.shadow then self.shadow:AlphaTo( 1, timing ) self.shadow:ScaleTo( nil, 1.05, timing ) end
        self.title:Bloom(0.2)
        self.overlay:Bloom(0.4)
        self.graft_icon:ScaleTo( nil, 1.05, 0.1, easing.outQuad )
    else
        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
        if self.shadow then self.shadow:AlphaTo( 0.6, timing ) self.shadow:ScaleTo( nil, 1, timing ) end
        self.title:Bloom(0.1)
        self.overlay:Bloom(0.2)
        self.graft_icon:ScaleTo( nil, 1, 0.1, easing.outQuad )
    end
end

function PickCoinOption:OnGainFocus()
    PickCoinOption._base.OnGainFocus(self)
    self:UpdateImage()
end

function PickCoinOption:OnLoseFocus()
    PickCoinOption._base.OnLoseFocus(self)
    self:UpdateImage()
end

function PickCoinOption:OnGainHover()
    PickCoinOption._base.OnGainHover(self)
    self:UpdateImage()
end

function PickCoinOption:OnLoseHover()
    PickCoinOption._base.OnLoseHover(self)
    self:UpdateImage()
end

function PickCoinOption:PostFeature( keyword, name, desc )
    self.features[ keyword ] = { name = name, desc = desc }
end

function PickCoinOption:PostCard( feature_id, card )
    self.features[ feature_id ] = { card = card }
end

function PickCoinOption:OnAdded()
    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
end

function PickCoinOption:OnControlModeChange( cm, device_id )
    PickCoinOption._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
    end
    if cm == CONTROL_MODE.TOUCH then
    end
    if cm == CONTROL_MODE.GAMEPAD then
    end
end

function PickCoinOption:OnScreenModeChange( sm )
    PickCoinOption._base.OnScreenModeChange( self, sm )
    
    self.title:SetFontSize( FONT_SIZE.ITEM_NAME*LAYOUT_SCALE[sm] )
    self.subtitle:SetFontSize( FONT_SIZE.ITEM_SUBTITLE*LAYOUT_SCALE[sm] )
    self.desc:SetFontSize( FONT_SIZE.ITEM_DESC*LAYOUT_SCALE[sm] )

    self:Layout()
end

function PickCoinOption:SetHeight( height )
    self.height = height or SLOT_H
    local nineslice_w, nineslice_h = self.width + bg_border*2*bg_border_scale, self.height + bg_border*2*bg_border_scale
    self.bg:SetSize( nineslice_w, nineslice_h )
    self.gradient:SetSize( nineslice_w, nineslice_h )
    self.overlay:SetSize( nineslice_w, nineslice_h )
    self:Layout()
    return self
end

function PickCoinOption:Layout()
    if self.current_coin_label then
        self.current_coin_bg:LayoutBounds( "center", "top", self.bg )
            :Offset( 0, 0 )
        self.current_coin_label:LayoutBounds( "center", "center", self.current_coin_bg )
            :Offset( 0, 1 )
        self.title:LayoutBounds( "left", "below", self.current_coin_bg )
            :Offset( PADDING, 0 )
    else
        self.title:LayoutBounds( "left", "top", self.bg )
            :Offset( PADDING, -PADDING/2 )
    end
    self.subtitle:LayoutBounds( "left", "below", self.title )
        :Offset( 0, 4 )
    self.graft_icon:LayoutBounds( "center", nil, self.bg )
        :LayoutBounds( nil, "below", self.subtitle )
        :Offset( 0, -PADDING )
    self.desc:LayoutBounds( "left", nil, self.subtitle )
        :LayoutBounds( nil, "below", self.graft_icon )
        :Offset( 0, -PADDING )
    return self
end


local ReticuleMixin = require "ui/screens/ReticuleMixin"
ReticuleMixin:Mixin( PickCoinScreen )
