require "ui/widgets/optionpicker"
require "ui/widgets/screen"
require "ui/widgets/tooltip_graft"
require "ui/widgets/particlesystem"
require "ui/screens/replacegraftscreen"
require "ui/widgets/installed_grafts"

-----------------------------------------------------------------------
--

local PANEL_W = RES_X * 0.5
local PANEL_H = RES_Y * 0.5

local PickLegacyScreen = class("Screen.PickLegacyScreen", Widget.Screen)

PickLegacyScreen.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_CANCEL,
        
        fn = function(self)
            self:OnClickBack()             
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.CANCEL", Controls.Digital.MENU_CANCEL))
        end,
    },
}

function PickLegacyScreen:init( save_name, gamestate )
    
    PickLegacyScreen._base.init( self )

    self:SetupUnderlay()
    self:SetAnchors("center", "center")

    assert_warning( save_name, "missing save name" )
    save_name = save_name or "slot_1"
    self.gamestate = gamestate
    self.save_name = save_name

    local screen_w, screen_h = TheGame:FE():GetScreenDims()

    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = screen_w*1.1
    local g_w, g_h = self.grid:GetSize()
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -screen_h/2 + target_h/2 )
    self.shadow_container = self:AddChild( Widget() )

    -- Contents
    self.content = self:AddChild(Widget())
    self.title = self.content:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_TITLE, LOC"UI.PICKLEGACY_SCREEN.TITLE") )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom(0.1)
    self.subtitle = self.content:AddChild( Widget.Label("title", FONT_SIZE.BODY_TEXT, LOC"UI.PICKLEGACY_SCREEN.PERMA_DESC") )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize(800)
        :SetWordWrap(true)
        :SetTintAlpha( 0.6 )
        :Bloom(0.1)

    self.options = self.content:AddChild( Widget() )
    
    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.back_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.PICKLEGACY_SCREEN.BUTTON_BACK", function() self:OnClickBack() end ) )
    self.back_button:SetIcon( global_images.close )
    self.back_button:LayoutBounds( "left", "above", 60, 60 )

    -- Continue button
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.button_continue = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.PICKLEGACY_SCREEN.BUTTON_CONTINUE", function() self:OnClickContinue() end ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Hide()
        :SetToolTip( LOC"UI.OPTIONS_SCREEN.CONFIRM_SELECT_TOOLTIP" )

    self:RefreshContent()
end

function PickLegacyScreen:OnClickBack()
    if self.gamestate:CanRollback() then
        TheGame:FE():InsertScreen( Screen.YesNoPopup(LOC"UI.PICKLEGACY_SCREEN.BACK_DIALOG_TITLE", LOC"UI.PICKLEGACY_SCREEN.BACK_DIALOG_TEXT", nil, nil, LOC"UI.PICKLEGACY_SCREEN.BACK_DIALOG_NO" ) )
            :SetFn(
                function(v) 
                    if v == Screen.YesNoPopup.YES then 
                        self:GetFE():PopScreen(self) 
                        local screen = Screen.GameOverScreen( self.gamestate )
                        self:GetFE():PushScreen( screen )
                    end
                end )

    else
        self:GetFE():PopScreen(self) 
        local screen = Screen.GameOverScreen( self.gamestate )
        self:GetFE():PushScreen( screen )
    end
end

function PickLegacyScreen:OnControlModeChange( cm, device_id )
    PickLegacyScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.select_then_continue_mode = false or engine.inst:GetSetting("gameplay", "confirm_select", true)
        self.bottom_left:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.select_then_continue_mode = true
        self.bottom_left:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.select_then_continue_mode = false
        self.bottom_left:Hide()
    end
end

function PickLegacyScreen:OnScreenModeChange( sm )
    PickLegacyScreen._base.OnScreenModeChange( self, sm )

    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.content:SetLayoutScale( LAYOUT_SCALE[sm] )
end

function PickLegacyScreen:GetDefaultFocus()
    return self.options and self.options.children and #self.options.children>0 and self.options.children[1]
end


function PickLegacyScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

function PickLegacyScreen:RefreshContent()
    
    self.options:DestroyAllChildren()

    local txt, desc = LOC"UI.PICKLEGACY_SCREEN.RETRY_FROM_MORNING", LOC"UI.PICKLEGACY_SCREEN.RETRY_DESC_DAY2"
    
    if self.gamestate:CanRollback() then
        if self.gamestate:IsStoryMode() then
            txt, desc = LOC"UI.PICKLEGACY_SCREEN.RETRY_FROM_MORNING", LOC"UI.PICKLEGACY_SCREEN.RETRY_DESC_NO_LIMIT"
        else
            txt, desc = LOC"UI.PICKLEGACY_SCREEN.RETRY_FROM_MORNING", LOC"UI.PICKLEGACY_SCREEN.RETRY_DESC"
        end
    else
        txt, desc = LOC"UI.PICKLEGACY_SCREEN.RETRY_FROM_MORNING", LOC"UI.PICKLEGACY_SCREEN.RETRY_DESC_ALREADY_DONE"
    end

    local w = self.options:AddChild(Widget.RollbackOption(txt, desc)):LayoutBounds("after", "center"):Offset(40, 0)

    w.shadow = self.shadow_container:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
        :SetSize( 500, 100 )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.5 )
        :LayoutBounds( "center", nil, w)

    if self.gamestate:CanRollback() then
        w.select_fn = function() 
            self:GetFE():FadeTransition( function() 
                TheGame:LoadGame(self.gamestate:GetRollbackSave(), self.save_name, 
                        function(gamestate)
                            gamestate:UseRollback()
                            DoAutoSave()
                        end)    
            end, .5)
        end
        w:SetOnClickFn( function() self:OnRewardSelected(w) end )
    else
        w:SetDisabled()
        w:SetSaturation(0)
        w:SetTintColour(.4,.4,.4,1)
        w.shadow:SetTintColour(0,0,0,0.5)
    end

    -- Layout everything
    self:DoLayout()
end

function PickLegacyScreen:OnRewardSelected(reward_widget)
    if self.select_then_continue_mode then
        -- We're in select-then-continue mode, and the selected graft isn't the clicked one
        -- Select this one
        self.selected_reward = reward_widget
        -- Unselect options
        for k, widget in ipairs( self.options.children ) do
            widget:SetSelected( widget == self.selected_reward )
        end
        -- And show the confirm button
        self.button_continue:Show()
    elseif reward_widget then
        self.selected_reward = reward_widget
        self:OnClickContinue()
    end
end

function PickLegacyScreen:OnClickContinue( reward_widget )
    reward_widget = reward_widget or self.selected_reward
    if reward_widget then
        reward_widget.select_fn()
    end
end

function PickLegacyScreen:DoLayout()
    self.options:LayoutBounds( "center", "center", 0, 0 )
    self.shadow_container:LayoutBounds( "center", "center", 0, 0 )
    local grid_w, grid_h = self.grid:GetSize()
    self.shadow_container:LayoutBounds( nil, "bottom", self.grid ):Offset( 0, grid_h*0.2 )
    self.subtitle:LayoutBounds( "center", "above", self.options )
        :Offset( 0, 30 )
    self.title:LayoutBounds( "center", "above", self.subtitle )
end

-----------------------------------------------------------------------
-- A selectable option

local SLOT_W = 250
local SLOT_H = 500
local PADDING = 30
local ICON_SIZE = 170
local GRAFT_SIZE = 200
local CARD_SCALE = 0.48
local bg_border = 33
local bg_border_scale = 0.5

local PickLegacyOption = class("Widget.PickLegacyOption", Widget.Clickable)
function PickLegacyOption:init(legacy_data)
    PickLegacyOption._base.init( self )

    -- Background panel
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/saveslot_bg.tex") ) )
    self.bg:SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
    self.bg:SetNineSliceBorderScale( bg_border_scale )
    self.bg:SetSize( SLOT_W, SLOT_H )

    self.grid_texture = self:AddChild( Widget.Image( engine.asset.Texture("large/legacy_option_bg.tex")))
        :SetSize( SLOT_W, SLOT_H )
        :SetTintAlpha( 0.2 )
        :SetBloom( 0.5 )
        :Pulse( 0.2, 0.5, 0.002 + math.random()*0.002 )

    -- Content
    self.contents_back = self:AddChild( Widget() )

    -- Gradient
    self.gradient = self:AddChild( Widget.Panel( engine.asset.Texture("UI/saveslot_gradient.tex") ) )
    self.gradient:SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
    self.gradient:SetNineSliceBorderScale( bg_border_scale )
    self.gradient:SetSize( SLOT_W, SLOT_H )
    self.gradient:SetTintAlpha( 1 )

    -- Overlay panel
    self.overlay = self:AddChild( Widget.Panel( engine.asset.Texture("UI/saveslot_overlay.tex") ) )
    self.overlay:SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
    self.overlay:SetNineSliceBorderScale( bg_border_scale )
    self.overlay:SetSize( SLOT_W, SLOT_H )
    self.overlay:Bloom( 0.2 )

    -- Content
    self.contents_front = self:AddChild( Widget() )

    -- Contents panel
    self.contents = self:AddChild( Widget() )
    self.title = self.contents:AddChild( Widget.Label( "title", 32 ) )
        :SetHoverLinkFn( UIHelpers.ItemTooltips )
        :SetAutoSize( SLOT_W-PADDING*1.5 )
        :SetWordWrap( true )
        :Bloom(0.1)
    self.desc = self.contents:AddChild( Widget.Label( "body", 26 ) )
        :SetHoverLinkFn( UIHelpers.ItemTooltips )
        :SetAutoSize( SLOT_W-PADDING*1.5 )
        :SetWordWrap( true )
        :OverrideLineHeight( 23 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    
    if legacy_data then
        self:Refresh( legacy_data )
    end
end

function PickLegacyOption:Refresh( legacy_data )
    local leg_def = Content.GetLegacy(legacy_data.ID)
    local title = leg_def and leg_def:GetName(legacy_data) or "???"
    local desc = leg_def and leg_def:GetDesc(legacy_data) or "???"
    desc = Content.SubstituteCodexFormatting( desc )

    -- Set texts
    self.title:SetText( title )
        :LayoutBounds( "center", "top", self.bg )
        :Offset( 0, -PADDING )
    self.desc:SetText( desc )
        :LayoutBounds( "center", "bottom", self.bg )
        :Offset( 0, PADDING )

    -- Set option-specific imagery
    if legacy_data.ID == "MONEY" then
        self.contents_back:AddChild( Widget.Image( engine.asset.Texture("UI/money_transparent.tex") ) )
            :SetSize( ICON_SIZE, ICON_SIZE )
            :Bloom( 0.2 )
    elseif legacy_data.ID == "PITY_MONEY" then
        self.contents_back:AddChild( Widget.Image( engine.asset.Texture("UI/money_transparent.tex") ) )
            :SetSize( ICON_SIZE, ICON_SIZE )
            :Bloom( 0.2 )
    elseif legacy_data.ID == "ROLLBACK" then
        self.contents_back:AddChild( Widget.Image( engine.asset.Texture("UI/money_transparent.tex") ) )
            :SetSize( ICON_SIZE, ICON_SIZE )
            :Bloom( 0.2 )
    elseif legacy_data.ID == "MAX_HEALTH" then
        self.contents_back:AddChild( Widget.Image( engine.asset.Texture("UI/health_transparent.tex") ) )
            :SetSize( ICON_SIZE, ICON_SIZE )
            :Bloom( 0.2 )
    elseif legacy_data.ID == "MAX_RESOLVE" then
        self.contents_back:AddChild( Widget.Image( engine.asset.Texture("UI/resolve_transparent.tex") ) )
            :SetSize( ICON_SIZE, ICON_SIZE )
            :Bloom( 0.2 )
    elseif legacy_data.ID == "COMBAT_GRAFT" then
        self.contents_front:AddChild( Widget.GraftSlot( GRAFT_TYPE.COMBAT, GRAFT_SIZE ) )
            :Refresh( GraftInstance( legacy_data.graft_id ) )
    elseif legacy_data.ID == "NEGOTIATION_GRAFT" then
        self.contents_front:AddChild( Widget.GraftSlot( GRAFT_TYPE.NEGOTIATION, GRAFT_SIZE ) )
            :Refresh( GraftInstance( legacy_data.graft_id ) )
    elseif legacy_data.ID == "LOVED" then
        local skin = Content.GetCharacterSkin( legacy_data.skin_id )
        if skin then
            local content_id = Content.GetCharacterForSkin( skin )
            local agent = Agent( content_id, skin )
            local mask = self.contents_back:AddChild( Widget() )
                :SetStencilContext( true )
                :SetHiddenBoundingBox( true )
            local mask_image = Widget.Panel( engine.asset.Texture("UI/saveslot_mask.tex") )
                :SetNineSliceCoords( bg_border, bg_border, bg_border*2, bg_border*2 )
                :SetNineSliceBorderScale( bg_border_scale )
                :SetSize( SLOT_W, SLOT_H )
            local anim = Widget.AnimatedCharacter()
            mask:AddChildren
            {
                mask_image:SetMask(true),
                anim:SetStencilTest(true),
            }
            anim:SetScale( 0.45 )
                :SetPos( -40, -170 )
                :Refresh(agent)
                :SetAutoLoopEnabled(false)
                :SetOutline( 7, UICOLOURS.RELATIONSHIP_LOVED )
                :Emote("happy")
                :Bloom( 0.08 )
        end
    elseif legacy_data.ID == "BATTLE_CARD" then
        local card = Battle.Card( legacy_data.card_id )
        self.contents_front:AddChild( Widget.BattleCard() )
            :RefreshCard( card )
            :SetScale( CARD_SCALE )
            :SetOutline( UICOLOURS.FIGHT )
    elseif legacy_data.ID == "NEGOTIATION_CARD" then
        local card = Negotiation.Card( legacy_data.card_id )
        self.contents_front:AddChild( Widget.NegotiationCard() )
            :RefreshCard( card )
            :SetScale( CARD_SCALE )
            :SetOutline( UICOLOURS.NEGOTIATION )
    elseif legacy_data.ID == "FACTION_LIKE" then
        local faction = Content.GetFaction(legacy_data.faction_id)
        self.contents_back:AddChild( Widget.Image( faction.icon ) )
            :SetSize( ICON_SIZE, ICON_SIZE )
            :Bloom( 0.2 )
    end
end

function PickLegacyOption:SetDisabled()
    self.disabled = true -- not clickable, but still focusable
end

function PickLegacyOption:HandleControlDown(control)
    if control:Has( Controls.Digital.MENU_ACCEPT ) or control:Has( Controls.Digital.MENU_ALT ) then
        if self.disabled then
            return
        end
    end
    return PickLegacyOption._base.HandleControlDown(self, control)
end

function PickLegacyOption:SetSelected(selected)
    self.selected = selected or false
    self:UpdateImage()
    return self
end

function PickLegacyOption:UpdateImage()
    local timing = 0.15
    if self.hover or self.focus or self.selected then
        if self.shadow then self.shadow:AlphaTo( 1, timing ) self.shadow:ScaleTo( nil, 1.05, timing ) end
        self:ScaleTo( nil, 1.05, 0.2, easing.outQuad )
        self.overlay:Bloom(0.3)
    else
        if self.shadow then self.shadow:AlphaTo( 0.6, timing ) self.shadow:ScaleTo( nil, 1, timing ) end
        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
        self.overlay:Bloom(0.2)
    end
end

function PickLegacyOption:OnGainFocus()
    PickLegacyOption._base.OnGainFocus(self)
    self:UpdateImage()
end

function PickLegacyOption:OnLoseFocus()
    PickLegacyOption._base.OnLoseFocus(self)
    self:UpdateImage()
end

function PickLegacyOption:OnGainHover()
    PickLegacyOption._base.OnGainHover(self)
    self:UpdateImage()
end

function PickLegacyOption:OnLoseHover()
    PickLegacyOption._base.OnLoseHover(self)
    self:UpdateImage()
end

-----------------------------------------------------------------------
-- A rollback selectable option

local RollbackOption = class("Widget.RollbackOption", Widget.PickLegacyOption)

function RollbackOption:init(txt, desc)
    RollbackOption._base.init( self )

    -- Set texts
    self.title:SetText( txt )
        :LayoutBounds( "center", "top", self.bg )
        :Offset( 0, -PADDING )
    self.desc:SetText( desc )
        :LayoutBounds( "center", "bottom", self.bg )
        :Offset( 0, PADDING )

    self.contents_back:AddChild( Widget.Image( engine.asset.Texture("UI/rollback_transparent.tex") ) )
        :SetSize( ICON_SIZE, ICON_SIZE )
        :Bloom( 0.2 )
end
