require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/menu"
require "ui/widgets/solidbox"
require "ui/widgets/panelbutton"

local PopUpMenu = class("Screen.PopUpMenu", Widget.Screen)


function PopUpMenu:init( title, body, options, w, h, button_width )
	PopUpMenu._base.init(self)
	self.is_popup = true
	
	local border = 32 --above title
	local sep = 32 --between title and body
	local button_size = 40 --height
	local titlesize = 48
	button_width = button_width or 150

	self.w = w or 512
	self.h = h or 480
	
	local textblockheight = self.h - button_size - titlesize - border*2 - sep*2
	local textblockwidth = self.w - border*2

	local border_padding = -8

	self.bg = Widget.Panel(engine.asset.Texture("UI/panel_window_opaque.9.tex")):SetName("BG"):SetInnerSize(self.w, self.h)
	self.inner_panel = Widget.Panel(engine.asset.Texture("UI/panel_inner.9.tex")):SetInnerSize(self.w+border_padding, self.h+border_padding)
	self.title = Widget.Label( "title", titlesize, title or ""):SetTintColour(UICOLOURS.TITLE):SetName("TITLE")
	self.body = Widget.Label( "body", 30, body):SetAutoSize(textblockwidth):SetWordWrap(true):SetTintColour(UICOLOURS.BODYTEXT):TopAlign():SetName("BODY")
	self.menu = Widget.Menu( Widget.PanelButton, button_width, button_size):SetLayout( "after", "center", 12, 0 ):SetOptions( options )

	self:AddChild(
		Widget():SetAnchors("center", "center"):AddChildren{
			self.bg,
			self.inner_panel,
			self.title,
			self.body,
			self.menu
		}:SetName("CONTENTS")
	)
	self.menu:LayoutBounds("center","bottom",self.bg):Offset(0,30):SetName("MENU")

	self.title:SetPos( 0, self.h/2 - sep )
	self.body:LayoutBounds("center", "below", self.title):Offset(0, -sep)

	local close_btn = self:FindChild( "CLOSE" )
	if close_btn then
		self:RegisterHotkeyButton( Controls.Digital.MENU_CANCEL, close_btn )
		self:RegisterHotkeyButton( Controls.Digital.MENU_ACCEPT, close_btn )
	end
end
