require "ui/widgets/emptypanel"

local RelationshipScreen = class("Screen.RelationshipScreen", Widget.Screen)

local SCREEN_LEFT_PADDING = 0
local TAB_HEIGHT = 70

local PANEL_W = 0.5 -- actual width = self.screen_w/2 - SCREEN_LEFT_PADDING - RES_X/2 + RES_X*PANEL_W
local CONTENT_X_OFFSET = 200

local TAB_W = 190
local CONTENT_W = RES_X*PANEL_W-CONTENT_X_OFFSET-SCREEN_LEFT_PADDING
local DETAILS_W = CONTENT_W - 140
local ANIM_COUNT = 5
local CHARACTER_ANIM_DURATION = 0.2

local DEAD_AGENTS_TAB = "DEAD"

local ANIM_POS = {
    [0] = {
        tint = 0x1C0F2100,
        scale = 1.3,
        x = -0.20,
        y = 0.185,
    },
    [1] = {
        tint = 0xFFFFFFFF,
        scale = 1,
        x = 0.00,
        y = 0.25,
    },
    [2] = {
        tint = 0xD5D1D8FF,
        scale = 0.67,
        x = 0.40,
        y = 0.35,
    },
    [3] = {
        tint = 0xA9A2ADFF,
        scale = 0.6,
        x = 0.60,
        y = 0.35,
    },
    [4] = {
        tint = 0x7A717FFF,
        scale = 0.53,
        x = 0.80,
        y = 0.35,
    },
    [5] = {
        tint = 0x493F4DFF,
        scale = 0.5,
        x = 1.00,
        y = 0.35,
    },
    [6] = {
        tint = 0x493F4D00,
        scale = 0.4,
        x = 1.20,
        y = 0.35,
    }
}

RelationshipScreen.CONTROL_MAP = {    
    {
        hint = function(self, left, right)
            table.insert(left, LOC"UI.CONTROLS.NAV")
        end
    },

    {
        control = Controls.Digital.GAMEPAD_LT,
        fn = function(self)
            self.nav_left.fn( self )
        end,
        test = function(self)
            return self.nav_left ~= nil
        end,        
        hint = function(self, left, right)
            table.insert(left, self.nav_left.hint_text)
        end
    },

    {
        control = Controls.Digital.GAMEPAD_RT,
        fn = function(self)
            self.nav_right.fn( self )
        end,
        test = function(self)
            return self.nav_right ~= nil
        end,
        
        hint = function(self, left, right)
            table.insert(left, self.nav_right.hint_text)
        end
    },

    {
        control = Controls.Digital.SCROLL_DOWN,
        fn = function(self)
            AUDIO:PlayEvent( "event:/ui/relationships_menu/arrow" )
            self:NextCharacter() 
            return true
        end,
    },
    {
        control = Controls.Digital.SCROLL_UP,
        fn = function(self)
            AUDIO:PlayEvent( "event:/ui/relationships_menu/arrow" )
            self:PreviousCharacter()
            return true
        end,
    },
    {
        control = Controls.Digital.MENU_CANCEL,
        hint = function(self, left, right)
            table.insert(right, loc.format( LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end,
        fn = function(self)
            self.close_button:Click()    
            return true
        end,
    }
}

function RelationshipScreen:init( gamestate )
    Widget.Screen.init(self)
    
    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )
    
    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    self.screen_x_split = RES_X * 0.5 -- Right edge of the details panel, and also center of the left-most anim
    self.content_right_padding = 200 -- Padding between the content and the left-most anim, so they don't touch
    self.screen_left_padding = 70 -- Padding between the edge of the screen and the content
    self.panel_h = 500 -- Height of the content panel
    self.content_w = self.screen_x_split-self.screen_left_padding-self.content_right_padding -- Total width of the content/tabs/nav dots
    self.anims_total_w = RES_X - self.screen_x_split -- Total width taken by the anims on the screen

    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- A container for all the background elements
    self.panel = self:AddChild( Widget() )
    self.panel_glow_top = self.panel:AddChild( Widget.Image( engine.asset.Texture( "images/glow.tex" ), 1900, 500 ) )
    self.panel_glow_top:SetHiddenBoundingBox( true )
    self.panel_glow_bottom = self.panel:AddChild( Widget.Image( engine.asset.Texture( "images/glow.tex" ), 1500, 500 ) )
    self.panel_glow_bottom:SetHiddenBoundingBox( true )
    self.panel_bg = self.panel:AddChild( Widget.Image( engine.asset.Texture( "large/dialog_bg.tex" ) ) )
    self.panel_grid = self.panel:AddChild( Widget.Image( engine.asset.Texture( "large/dialog_grid.tex" ) ) )
    self.panel_border_size = 20
    self.panel_border_scale = 0.5
    self.panel_border = self.panel:AddChild( Widget.Panel( engine.asset.Texture("UI/relationshipsscreen_frame.tex") ) )
    self.panel_border:SetNineSliceCoords( self.panel_border_size, self.panel_border_size, self.panel_border_size, self.panel_border_size )
        :SetNineSliceBorderScale( self.panel_border_scale )
        :Bloom( 0.1 )

    -- Set background colours
    self.panel_bg:SetTintAlpha( 0.98 )
    self.panel_grid:Pulse( 0.4, 1.0, 0.003 )
    self.panel_glow_top:SetTintColour( UICOLOURS.SUBTITLE )
    self.panel_glow_top:SetTintAlpha( 0.4 )
    self.panel_glow_bottom:SetTintColour( UICOLOURS.SUBTITLE )
    self.panel_glow_bottom:SetTintAlpha( 0.3 )
    self.panel_border:SetTintColour( UICOLOURS.SUBTITLE )

    -- Get data to display beautifully
    self.gamestate = gamestate or TheGame:GetGameState()
    self.social_connections = self.gamestate:GetPlayerAgent() and self.gamestate:GetPlayerAgent().social_connections
    self.dead_agents = {}
    
    self.agents_by_relationship = {
        [RELATIONSHIP.LOVED] = {},
        [RELATIONSHIP.LIKED] = {},
        [RELATIONSHIP.DISLIKED] = {},
        [RELATIONSHIP.HATED] = {},
    }

    for k, agent in pairs( TheGame:GetGameState().removed_agents or {} ) do
        if agent:IsSentient() and agent:IsDead() then 
            table.insert_unique( self.dead_agents, agent ) 
        elseif not agent:IsDead() and agent:GetRelationship() ~= RELATIONSHIP.NEUTRAL then
            table.insert_unique( self.agents_by_relationship[agent:GetRelationship()], agent)
        end
    end


    local player_agent = self.gamestate and self.gamestate:GetPlayerAgent()
    
    if player_agent and player_agent.social_connections then
        for other_agent, agent_rel in pairs(player_agent.social_connections.relationships) do
            if agent_rel ~= RELATIONSHIP.NEUTRAL and not other_agent:IsDead() then
                local has_effect = agent_rel == RELATIONSHIP.LOVED or agent_rel == RELATIONSHIP.HATED
                if has_effect or not other_agent.removed then
                    table.insert_unique( self.agents_by_relationship[agent_rel], other_agent)
                end
            end
        end
    end

    -- Top navigation tabs
    self.navigation = self:AddChild( Widget.TabNavigation( TAB_W, TAB_HEIGHT, function( tab ) self:NavigationChanged( tab ) end ) )
        :SetFocusable( false )
    self.navigation:AddTab( Widget.RelationshipCountTab( RELATIONSHIP.LOVED, LOC"UI.RELATIONSHIP_SCREEN.TAB_LOVED", #self.agents_by_relationship[RELATIONSHIP.LOVED] or 0 ) )
    self.navigation:AddTab( Widget.RelationshipCountTab( RELATIONSHIP.LIKED, LOC"UI.RELATIONSHIP_SCREEN.TAB_LIKED", #self.agents_by_relationship[RELATIONSHIP.LIKED] or 0 ) )
    self.navigation:AddTab( Widget.RelationshipCountTab( RELATIONSHIP.DISLIKED, LOC"UI.RELATIONSHIP_SCREEN.TAB_DISLIKED", #self.agents_by_relationship[RELATIONSHIP.DISLIKED] or 0 ) )
    self.navigation:AddTab( Widget.RelationshipCountTab( RELATIONSHIP.HATED, LOC"UI.RELATIONSHIP_SCREEN.TAB_HATED", #self.agents_by_relationship[RELATIONSHIP.HATED] or 0 ) )
    local dead_tab = self.navigation:AddTab( Widget.IconCountTab( engine.asset.Texture("UI/ic_deathloot.tex"), loc.format( LOC"UI.RELATIONSHIP_SCREEN.TAB_DEAD", string.upper( self.gamestate:GetPlayerAgent():GetName() ) ), #self.dead_agents, UICOLOURS.DEAD ) )
        :SetBGColour( UICOLOURS.SUBTITLE )
    dead_tab.dead_tab = DEAD_AGENTS_TAB

    -- Move the panel over the tabs
    self.panel:SendToFront()

    -- Hitbox for content alignment
    self.hitbox = self:AddChild( Widget.SolidBox( CONTENT_W, self.panel_h, 0xff000030 ) )
        :Hide()

    -- Container for the pagination dots
    self.pagination = self:AddChild( Widget() )

    -- Details panel
    self.details = self:AddChild( Widget.RelationshipsScreenDetails( DETAILS_W, self.panel_h, self.gamestate ) )

    -- Navigation tab labels
    self.navigation_label_left = self:AddChild( Widget.Label( "title", 30, LOC"UI.RELATIONSHIP_SCREEN.CONTROLS_LABEL_TAB_LEFT" ) )
        :Bloom( 0.1 )
    self.navigation_label_right = self:AddChild( Widget.Label( "title", 30, LOC"UI.RELATIONSHIP_SCREEN.CONTROLS_LABEL_TAB_RIGHT" ) )
        :Bloom( 0.1 )

    -- Large anim container
    self.character_anims_container = self:AddChild( Widget() )
    self.character_anims = {}
    table.insert( self.character_anims, self.character_anims_container:AddChild( Widget.AnimatedCharacter() ) )
    table.insert( self.character_anims, self.character_anims_container:AddChild( Widget.AnimatedCharacter() ):SendToBack() )
    table.insert( self.character_anims, self.character_anims_container:AddChild( Widget.AnimatedCharacter() ):SendToBack() )
    table.insert( self.character_anims, self.character_anims_container:AddChild( Widget.AnimatedCharacter() ):SendToBack() )
    table.insert( self.character_anims, self.character_anims_container:AddChild( Widget.AnimatedCharacter() ):SendToBack() )
    table.insert( self.character_anims, self.character_anims_container:AddChild( Widget.AnimatedCharacter() ):SendToBack() )
    table.insert( self.character_anims, self.character_anims_container:AddChild( Widget.AnimatedCharacter() ):SendToBack() )
    self.current_character_index = 0 -- The :NextCharacter() function will increase this to 1

    -- Left/right arrows
    local arrow_size = 34
    self.arrow_left = self:AddChild( Widget.ImageButton( engine.asset.Texture("UI/gameover_arrow_left.tex"), 
        function()
            AUDIO:PlayEvent( "event:/ui/relationships_menu/arrow" )
            self:PreviousCharacter()
        end ) )
        :SetSize( arrow_size, arrow_size )
        :SetStateScales( 1, 1.3 )
        :SetFocusable( false )
    self.arrow_right = self:AddChild( Widget.ImageButton( engine.asset.Texture("UI/gameover_arrow.tex"), 
        function()
            AUDIO:PlayEvent( "event:/ui/relationships_menu/arrow" )
            self:NextCharacter() 
        end ) )
        :SetSize( arrow_size, arrow_size )
        :SetStateScales( 1, 1.3 )
        :SetFocusable( false )

    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.close_button = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.CLOSE", 
        function()
            AUDIO:PlayEvent( "event:/ui/relationships_menu/close" )
            self:DoClose()
        end ) )
    self.close_button:SetIcon( global_images.close )
    self.close_button:LayoutBounds( "left", "above", 60, 60 )
    self:RegisterHotkeyButton( Controls.Digital.MENU_CANCEL, self.close_button )

    -- Go to the last viewed tab, or the liked one
    local target_tab = self.gamestate.relationship_screen_last_tab or RELATIONSHIP.LIKED
    for k, tab in ipairs( self.navigation:GetTabs() ) do
        if tab.relationship == target_tab then
            self.navigation:SetCurrent( k )
        elseif tab.dead_tab == target_tab then
            self.navigation:SetCurrent( k )
        end
    end

    self:Layout()

    -- Play screen sound
    self.screen_loop = AUDIO:CreateEventInstance( "event:/ui/main/fld/relationships_menu_open" )
    self.screen_loop:Start()
end

function RelationshipScreen:OnClose()
    self._base.OnClose( self )

    -- Stop the screen sound
    self.screen_loop:Stop()
    self.screen_loop:Release()
    self.screen_loop = nil
end

function RelationshipScreen:NavigationChanged( tab )
    self.selected_tab = tab.relationship or tab.dead_tab

    -- Save the current tab, so that the screen goes back to it when opening it again next time
    self.gamestate.relationship_screen_last_tab = self.selected_tab

    -- List of agents to display
    self.agents = {}
    if tab.relationship then
        self.agents = self.agents_by_relationship[self.selected_tab]
    elseif tab.dead_tab then
        self.agents = self.dead_agents
    end
    table.sort(self.agents, function(a,b) return a:GetContentID() < b:GetContentID() end)

    -- Clear any old pagination markers
    self.pagination:DestroyAllChildren()
    -- Add brand new ones
    local dot_spacing = 5
    local dot_w = math.min( 67, (CONTENT_W-dot_spacing*(#self.agents-1))/#self.agents )
    for k = 1, #self.agents do
        local dot = self.pagination:AddChild( Widget.Clickable( ) )
        dot.hitbox = dot:AddChild( Widget.SolidBox( dot_w, 30, 0xFF000000 ))
        dot.image = dot:AddChild( Widget.Image( engine.asset.Texture("UI/motd_page.tex"), dot_w, 8 ) ):Bloom( 0.1 )
        dot:SetOnClickFn( 
            function()
                self:AnimateToCharacter(k)
                AUDIO:PlayEvent( "event:/ui/relationships_menu/arrow" )
        end )
        -- Handle events
        local function UpdateDotImage()
            if dot.enabled and ( dot.hover or dot.focus ) then
                dot:ScaleTo( nil, 1.05, 0.2, easing.outQuad )
                dot:Bloom(0.25)
            else
                dot:ScaleTo( nil, 1, 0.2, easing.outQuad )
                dot:Bloom(0.1)
            end
        end
        dot.OnGainFocus = function() UpdateDotImage() end
        dot.OnLoseFocus = function() UpdateDotImage() end
        dot.OnGainHover = function() UpdateDotImage() end
        dot.OnLoseHover = function() UpdateDotImage() end
        dot:SetToolTip( self.agents[k]:GetName() )
        dot:SetFocusable( false )
    end
    -- Align them
    self.pagination:StackChildrenRow( dot_spacing )
    self.pagination:LayoutBounds( "center", "below", self.hitbox ):Offset( 0, 10 )

    -- Hide all the animated characters
    for k, anim in ipairs( self.character_anims ) do
        anim:Hide()
    end

    -- Hide the arrows
    self.arrow_left:Hide()
    self.arrow_right:Hide()

    -- Update the animated characters
    self:LayoutAnims()

    if #self.agents > 1 then
        -- Show the arrows
        self.arrow_left:Show()
        self.arrow_right:Show()
    end
    
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )

    -- Set the current character
    self.current_character_index = 0 -- The :NextCharacter() function will increase this to 1
    self:NextCharacter()
end

function RelationshipScreen:OnScreenResize(w, h)
    self._base.OnScreenResize(self, w, h)

    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    self:Layout()
    self:RefreshStreamerMode()
end

function RelationshipScreen:OnControlModeChange( cm, device_id )
    RelationshipScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.close_button:Show()
        self.navigation_label_left:Hide()
        self.navigation_label_right:Hide()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.close_button:Show()
        self.navigation_label_left:Hide()
        self.navigation_label_right:Hide()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.close_button:Hide()
        self.navigation_label_left:Show()
        self.navigation_label_right:Show()
    end
end

function RelationshipScreen:OnScreenModeChange( sm )
    RelationshipScreen._base.OnScreenModeChange( self, sm )

    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )

    -- Calculate sizes for various needed dimensions
    self.screen_x_split = 0 -- Right edge of the details panel, and also center of the left-most anim
    self.content_right_padding = 330 -- Padding between the content and the left-most anim, so they don't touch
    self.screen_left_padding = 70 -- Padding between the edge of the screen and the content
    self.panel_h = 500 -- Height of the content panel

    if sm == SCREEN_MODE.MONITOR then
        self.panel_h = 610
        self.screen_x_split = self.screen_w * 0.6
    elseif sm == SCREEN_MODE.SMALL
        or sm == SCREEN_MODE.TV then
        self.panel_h = 610
        self.screen_x_split = self.screen_w * 0.75
    end
    self.content_w = self.screen_x_split-self.screen_left_padding-self.content_right_padding -- Total width of the content/tabs/nav dots
    self.anims_total_w = self.screen_w - self.screen_x_split -- Total width taken by the anims on the screen

    self:Layout()
end

function RelationshipScreen:GetDefaultFocus()
end

function RelationshipScreen:SetNavLeft( hint_text, fn )
    self.nav_left = { hint_text = hint_text, fn = fn }
end

function RelationshipScreen:SetNavRight( hint_text, fn )
    self.nav_right = { hint_text = hint_text, fn = fn }
end

function RelationshipScreen:HandleControlUp( control, device )
    if control:Has( Controls.Digital.MENU_CANCEL ) then
        AUDIO:PlayEvent( "event:/ui/relationships_menu/close" )
        self:DoClose()
        return true
    end

    return RelationshipScreen._base.HandleControlUp( self, control, device )
end

function RelationshipScreen:HandleControlDown( control, device )

    if control:Has( Controls.Digital.GAMEPAD_LB ) then
        self.navigation:SetCurrent( self.navigation:GetCurrentIdx() - 1 )
        return true
    end
    if control:Has( Controls.Digital.GAMEPAD_RB ) then
        self.navigation:SetCurrent( self.navigation:GetCurrentIdx() + 1 )
        return true
    end

    if control:Has( Controls.Digital.MENU_LEFT ) then
        self:PreviousCharacter()
        return true
    end
    if control:Has( Controls.Digital.MENU_RIGHT ) then
        self:NextCharacter()
        return true
    end

    return RelationshipScreen._base.HandleControlDown( self, control, device )
end

function RelationshipScreen:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
end

function RelationshipScreen:Layout()
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Size up the background
    local panel_bg_w = self.screen_w/2 - RES_X/2 + self.screen_x_split
    self.panel_bg:SetSize( panel_bg_w, self.panel_h )
    local grid_w, grid_h = self.panel_grid:GetSize()
    self.panel_grid:SetSize( grid_w * self.panel_h/grid_h,  self.panel_h )
    self.panel:SetPos( -self.screen_w/2 + panel_bg_w/2, 0 )
    local nineslice_w, nineslice_h = panel_bg_w + self.panel_border_size*2*self.panel_border_scale, self.panel_h + self.panel_border_size*2*self.panel_border_scale
    self.panel_border:SetSize( nineslice_w+5, nineslice_h+5 )

    -- Update the floor grid
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize()
    local target_h = target_w/g_w*g_h -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Hitbox for content alignment
    self.hitbox:SetSize( self.content_w, self.panel_h )
        :LayoutBounds( "left", nil, -self.screen_w/2 + self.screen_left_padding, 0 )

    -- Layout the background
    self.panel_glow_top:LayoutBounds( nil, "top", self.panel_bg ):Offset( 0, 150 )
    self.panel_glow_bottom:LayoutBounds( nil, "bottom", self.panel_bg ):Offset( 0, -130 )
    self.panel_grid:LayoutBounds( "right", "center", self.panel_bg )

    -- Top navigation tabs
    self.navigation:SetTotalWidth( self.content_w )
    self.navigation:LayoutBounds( "left", nil, self.hitbox )
    self.navigation:LayoutBounds( nil, "above", self.panel_bg ):Offset( 0, 0 )
    self.navigation_label_left:LayoutBounds( "before", "center", self.navigation ):Offset( -20, 0 )
    self.navigation_label_right:LayoutBounds( "after", "center", self.navigation ):Offset( 20, 0 )

    -- Bottom pagination dots
    self.pagination:LayoutBounds( "center", "below", self.hitbox ):Offset( 0, 10 )

    -- Details panel
    self.details:SetSize( self.content_w, self.panel_h, self.selected_tab )
    self.details:LayoutBounds( "left", "center", self.hitbox )

    -- Re-layout the animated characters
    self:LayoutAnims()

    -- Left/right arrows
    self.arrow_left:LayoutBounds( "before", "center", self.hitbox ):Offset(-SPACING.M1,0)
    self.arrow_right:LayoutBounds( "after", "center", self.hitbox ):Offset(SPACING.M1,0)
end

function RelationshipScreen:LayoutAnims()
    self.anims_length = math.min( ANIM_COUNT, #self.agents )
    for k = 1, self.anims_length do
        local agent = self.agents[k]
        if k < ANIM_COUNT + 1 then
            self.character_anims[k]
                :Refresh( agent )
                :SetPos( -self.screen_w/2 + self.screen_x_split + ANIM_POS[k].x*self.anims_total_w, -RES_Y/2 + ANIM_POS[k].y*RES_Y )
                :SetTintColour( ANIM_POS[k].tint )
                :SetScale( ANIM_POS[k].scale )
                :SetFacing( false )
                :Show()
            -- Make them breathe at different rates
            self.character_anims[k].anim:SetPhase( math.random() )
        end
    end
    return self
end

-- Called by outside widgets wanting to display a specific agent
function RelationshipScreen:AnimateToAgent( agent )
    -- Find agent
    local index = nil
    for k, v in ipairs( self.agents ) do
        if v == agent then
            index = k
            break
        end
    end
    if index ~= nil then self:StartCoroutine( self.GoToCharacter, self, index, true ) end
    return self
end

function RelationshipScreen:AnimateToCharacter( index, forward_only )
    self:StartCoroutine( self.GoToCharacter, self, index, forward_only )
end

-- Focuses on this agent's index
function RelationshipScreen:GoToCharacter( index, forward_only )

    -- Move the list
    local distance = index - self.current_character_index
    local anim_duration = CHARACTER_ANIM_DURATION / ( math.abs( distance ) or 1 )
    while self.current_character_index ~= index do
        if index > self.current_character_index or forward_only then
            self:NextCharacter( anim_duration )
        else
            self:PreviousCharacter( anim_duration )
        end

        -- Stop click listening
        for k, anim in ipairs( self.character_anims ) do anim.isbutton = false end

        self:Delay( anim_duration )
    end

    self:Delay( 0.1 )

    -- Restart click listening
    for k, anim in ipairs( self.character_anims ) do anim.isbutton = true end
end

-- Move all characters back one index
function RelationshipScreen:NextCharacter( animation_duration )
    animation_duration = animation_duration or CHARACTER_ANIM_DURATION

    -- Update the current index
    self.current_character_index = self.current_character_index + 1
    if self.current_character_index > #self.agents then self.current_character_index = 1 end

    -- Move all positions except the zeroth one back one spot (pos 1 will end up at pos 0)
    for k=1, self.anims_length, 1 do
        self:SetCharacterByIndex( k, 1 )
    end

    -- Update the details panel with the newly selected agent
    self.details:Refresh( self.agents[self.current_character_index] or nil, self.selected_tab )

    -- Update the pagination dots
    for k, dot in ipairs( self.pagination.children ) do
        if k == self.current_character_index then
            -- Current page
            dot:SetTintAlpha( 1 )
        else
            -- Not the current page..
            dot:SetTintAlpha( 0.3 )
        end
    end

    AUDIO:PlayEvent( "event:/ui/relationships_menu/character_whoosh_in" )
end

-- Move all characters forward one index
function RelationshipScreen:PreviousCharacter( animation_duration )
    animation_duration = animation_duration or CHARACTER_ANIM_DURATION

    -- Move all positions except the last one forward one spot (pos 6 will end up at pos 7)
    self.current_character_index = self.current_character_index - 1
    if self.current_character_index == 0 then self.current_character_index = #self.agents end

    for k=1, self.anims_length, 1 do
        self:SetCharacterByIndex( k, -1 )
    end

    -- Update the details panel with the newly selected agent
    self.details:Refresh( self.agents[self.current_character_index] or nil, self.selected_tab )

    -- Update the pagination dots
    for k, dot in ipairs( self.pagination.children ) do
        if k == self.current_character_index then
            -- Current page
            dot:SetTintAlpha( 1 )
        else
            -- Not the current page..
            dot:SetTintAlpha( 0.3 )
        end
    end

    AUDIO:PlayEvent( "event:/ui/relationships_menu/character_whoosh_in" )
end

function RelationshipScreen:SetCharacterMood( anim, relationship, show_emote )

    -- Set their mood
    if relationship == RELATIONSHIP.LOVED then anim:SetMood( MOOD.HAPPY ) end
    if relationship == RELATIONSHIP.LIKED then anim:SetMood( MOOD.NEUTRAL ) end
    if relationship == RELATIONSHIP.DISLIKED then anim:SetMood( MOOD.NEUTRAL ) end
    if relationship == RELATIONSHIP.HATED then anim:SetMood( MOOD.ANGRY ) end

    -- If they're the focused character, show an emote matching their relationship
    local mood_emotes = {
        [RELATIONSHIP.LOVED] = {
            ["happy"] = 4,
            ["greeting"] = 4,
            ["chuckle"] = 2,
            ["laugh"] = 1,
            ["clap"] = 1
        },
        [RELATIONSHIP.LIKED] = {
            ["wink"] = 5,
            ["permit"] = 1,
            ["agree"] = 4,
            ["interest"] = 5,
            ["handwave"] = 2,
            ["shrug"] = 1
        },
        [RELATIONSHIP.DISLIKED] = {
            ["disappoint"] = 3,
            ["hips"] = 5,
            ["angry_shrug"] = 2,
            ["crossed"] = 5,
        },
        [RELATIONSHIP.HATED] = {
            ["throatcut"] = 5,
            ["point"] = 3,
            ["fight"] = 3,
            ["cruel"] = 4,
            ["threaten"] = 3,
            ["angry_accuse"] = 2,
            ["angry"] = 4,
        },
        [DEAD_AGENTS_TAB] = {
            ["neutral"] = 1,
        },
    }
    -- Animate!
    if show_emote then
        anim:Emote( weightedpick( mood_emotes[relationship] ) )
        anim.anim:SetPhase( 0 )
    end
end

function RelationshipScreen:SetCharacterByIndex( k, delta )
    local agent_idx = (self.current_character_index - 2 + k) % #self.agents + 1

    local next_agent = self.agents[agent_idx]
    self.character_anims[k]:Refresh( next_agent )
    local show_emote = false
    -- Set click function if it's not the first character
    if k > 1 then 
        local forward_only = true
        self.character_anims[k]:SetClickFn( function() self:AnimateToCharacter(agent_idx,forward_only) end )
        self.character_anims[k]:SetHiddenBoundingBox( false )
    else
        -- This is the character in focus!
        self.character_anims[k]:SetClickFn( nil )
        self.character_anims[k].hover_check = false
        self.character_anims[k]:SetHiddenBoundingBox( true )
        show_emote = true
    end
    -- Get this agent's relationship to the player
    local rel = next_agent:GetRelationship( self.gamestate:GetPlayerAgent() )
    -- Update their mood to match
    if self.selected_tab == DEAD_AGENTS_TAB then
        self:SetCharacterMood( self.character_anims[k], DEAD_AGENTS_TAB, show_emote )
    else
        self:SetCharacterMood( self.character_anims[k], rel, show_emote )
    end

    -- Move anims to the left
    local r, g, b, a = HexColour( self.selected_tab == DEAD_AGENTS_TAB and 0x3C3635FF or ANIM_POS[k].tint )
    self.character_anims[k]
        :SetPos( -self.screen_w/2 + self.screen_x_split + ANIM_POS[k + delta].x*self.anims_total_w, -RES_Y/2 + ANIM_POS[k + delta].y*RES_Y )
        :MoveTo( -self.screen_w/2 + self.screen_x_split + ANIM_POS[k].x*self.anims_total_w, -RES_Y/2 + ANIM_POS[k].y*RES_Y, 0.1 )
        :SetTintColour( self.selected_tab == DEAD_AGENTS_TAB and 0x3C3635FF or ANIM_POS[k + delta].tint )
        :ColourTo( nil, nil, nil, nil, r, g, b, a, 0.1 )
        :SetScale( ANIM_POS[k + delta].scale )
        :ScaleTo( nil, ANIM_POS[k].scale, 0.1 )
        :SetFacing( false )

    local r1, g1, b1, a1 = HexColour( ANIM_POS[k].tint )
    local r2, g2, b2, a2 = HexColour( ANIM_POS[k + delta].tint )
end

function RelationshipScreen:DoClose()
    self:GetFE():PopScreen( self )
end


------------------------------------------------------------------------------------------
-- Shows a character details panel

local RelationshipsScreenDetails = class( "Widget.RelationshipsScreenDetails", Widget )

local PADDING = 20
local BOON_SIZE = 120

function RelationshipsScreenDetails:init( width, height, gamestate )
    self._base.init( self )

    self.width = width
    self.height = height
    self.gamestate = gamestate

    self.hitbox = self:AddChild( Widget.EmptyPanel( self.width, self.height ) )

    self.name = self:AddChild( Widget.Label("title", FONT_SIZE.ITEM_NAME ) )
        :SetWordWrap(true)
        :SetAutoSize( width )
        :LeftAlign()
        :SetTintColour( UICOLOURS.TITLE )
        :Bloom( 0.1 )
    self.faction_icon = self:AddChild( Widget.Image() )
        :SetSize( 20, 20 )
        :Bloom( 0.05 )
    self.rank = self:AddChild( Widget.Label("title", FONT_SIZE.ITEM_SUBTITLE ) )
        :SetWordWrap(true)
        :SetAutoSize( width )
        :LeftAlign()
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.desc = self:AddChild( Widget.Label("body", FONT_SIZE.ITEM_DESC ) )
        :SetWordWrap(true)
        :SetAutoSize( width )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
        :SetHoverLinkFn( UIHelpers.ItemTooltips )
        
    self.boon = self:AddChild( Widget.RelationshipsScreenBoon( self.width, BOON_SIZE, true ) )
    self.bane = self:AddChild( Widget.RelationshipsScreenBoon( self.width, BOON_SIZE, false ) )
end

function RelationshipsScreenDetails:SetSize( width, height, selected_tab )
    self.width = width
    self.height = height
    self.selected_tab = selected_tab
    self.hitbox:SetSize( self.width, self.height )
    self.name:SetAutoSize( self.width )
    self.rank:SetAutoSize( self.width )
    self.desc:SetAutoSize( self.width )

    local social_graft_w = self.width/LAYOUT_SCALE[TheGame:FE():GetScreenMode()]
    local columns = 2
    local row_w = (social_graft_w-PADDING)/columns
    self.boon:SetSize( row_w, BOON_SIZE )
    self.boon:SetLayoutScale( LAYOUT_SCALE[TheGame:FE():GetScreenMode()] )
    self.bane:SetSize( row_w, BOON_SIZE )
    self.bane:SetLayoutScale( LAYOUT_SCALE[TheGame:FE():GetScreenMode()] )

    self:Refresh( self.agent or nil, self.selected_tab )
    return self
end

function RelationshipsScreenDetails:OnScreenModeChange( sm )
    RelationshipsScreenDetails._base.OnScreenModeChange( self, sm )

    local content_w = self.width * LAYOUT_SCALE[sm]
    self.name:SetFontSize( FONT_SIZE.ITEM_NAME * LAYOUT_SCALE[sm] )
    self.name:SetAutoSize( content_w )
    self.faction_icon:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.rank:SetFontSize( FONT_SIZE.ITEM_SUBTITLE * LAYOUT_SCALE[sm] )
    self.rank:SetAutoSize( content_w )
    self.desc:SetFontSize( FONT_SIZE.ITEM_DESC * LAYOUT_SCALE[sm] )
    self.desc:SetAutoSize( content_w )

    local social_graft_w = self.width/LAYOUT_SCALE[TheGame:FE():GetScreenMode()]
    local columns = 2
    local row_w = (social_graft_w-PADDING)/columns
    self.boon:SetSize( row_w, BOON_SIZE )
    self.boon:SetLayoutScale( LAYOUT_SCALE[TheGame:FE():GetScreenMode()] )
    self.bane:SetSize( row_w, BOON_SIZE )
    self.bane:SetLayoutScale( LAYOUT_SCALE[TheGame:FE():GetScreenMode()] )

    self:Refresh( self.agent, self.selected_tab )
end

function RelationshipsScreenDetails:Refresh( agent, selected_tab )
    self.selected_tab = selected_tab
    if agent then
        self.agent = agent

        -- Set the agent's name
        self.name:SetText( agent:GetName() or agent:GetShortTitle() )

        local faction = agent:GetFaction( self.gamestate )
        local player = self.gamestate:GetPlayerAgent()

        -- Set the agent's title
        self.rank:SetText( agent:GetIdentity() )

        -- Set the agent's bio and reason for their opinion of you
        local why_event = agent.social_connections and agent.social_connections:GetRelationshipReason( player )
        if self.selected_tab ~= DEAD_AGENTS_TAB and why_event then
            self.desc:SetText( loc.format( LOC"UI.RELATIONSHIP_SCREEN.REASON", agent:GetBio() or "", FormatOpinionEvent( why_event, agent ), why_event.time ) )
        else
            self.desc:SetText( agent:GetBio() )
        end

        -- Set the agent's faction
        self.faction_icon:SetTexture( faction and faction:GetIcon() )
            :SetToolTip( )

        -- Get this agent's relationship to the player
        local rel = agent:GetRelationship( self.gamestate:GetPlayerAgent() )

        -- Set this agent's boon, if any
        local boon_id = agent:GetSocialGraft( RELATIONSHIP.LOVED )
        local boon_graft = boon_id and GraftInstance( boon_id )
        self.boon:Refresh( boon_graft, self.selected_tab ~= DEAD_AGENTS_TAB and rel == RELATIONSHIP.LOVED, agent )

        -- Set this agent's bane, if any
        local bane_id = agent:GetSocialGraft( RELATIONSHIP.HATED )
        local bane_graft = bane_id and GraftInstance( bane_id )
        self.bane:Refresh( bane_graft, self.selected_tab ~= DEAD_AGENTS_TAB and rel == RELATIONSHIP.HATED, agent )

        -- Align things!
        self.name:LayoutBounds( "left", "top", self.hitbox ):Offset( 0, -PADDING+9 )
        self.faction_icon:LayoutBounds( "left", "below", self.name ):Offset( 0, 0 )
        self.rank:LayoutBounds( "after", "center", self.faction_icon ):Offset( 5, 1 )
        self.desc:LayoutBounds( "left", nil, self.faction_icon )
        self.desc:LayoutBounds( nil, "below", self.rank ):Offset( 0, -PADDING )
        if self.bane:IsShown() and self.boon:IsShown() then
            self.bane:LayoutBounds( "left", "bottom", self.hitbox ):Offset( 0, PADDING )
            self.boon:LayoutBounds( "right", "bottom", self.hitbox ):Offset( 0, PADDING )
        elseif self.bane:IsShown() == false then
            self.bane:LayoutBounds( "left", "bottom", self.hitbox ):Offset( 0, PADDING )
            self.boon:LayoutBounds( "right", "bottom", self.hitbox ):Offset( 0, PADDING )
        end

    else
        self.name:SetText( LOC"UI.RELATIONSHIP_SCREEN.EMPTY_TITLE" )
        self.rank:SetText( LOC"UI.RELATIONSHIP_SCREEN.EMPTY_SUBTITLE" )
        if self.selected_tab == RELATIONSHIP.HATED then self.desc:SetText( LOC"UI.RELATIONSHIP_SCREEN.EMPTY_DESC_HATED" )
        elseif self.selected_tab == RELATIONSHIP.DISLIKED then self.desc:SetText( LOC"UI.RELATIONSHIP_SCREEN.EMPTY_DESC_DISLIKED" )
        elseif self.selected_tab == RELATIONSHIP.LIKED then self.desc:SetText( LOC"UI.RELATIONSHIP_SCREEN.EMPTY_DESC_LIKED" )
        elseif self.selected_tab == RELATIONSHIP.LOVED then self.desc:SetText( LOC"UI.RELATIONSHIP_SCREEN.EMPTY_DESC_LOVED" )
        elseif self.selected_tab == DEAD_AGENTS_TAB then self.desc:SetText( LOC"UI.RELATIONSHIP_SCREEN.EMPTY_DESC_DEAD" ) end
        self.faction_icon:SetTexture( nil )
        self.boon:Hide()
        self.bane:Hide()
        self.name:LayoutBounds( "left", "top", self.hitbox ):Offset( 0, -PADDING+9 )
        self.faction_icon:LayoutBounds( "left", "below", self.name ):Offset( 0, 0 )
        self.rank:LayoutBounds( "left", "center", self.faction_icon ):Offset( 0, 1 )
        self.desc:LayoutBounds( "left", nil, self.faction_icon )
        self.desc:LayoutBounds( nil, "below", self.rank ):Offset( 0, -PADDING )
    end
    return self
end


------------------------------------------------------------------------------------------
-- Shows a character details panel

local RelationshipsScreenBoon = class( "Widget.RelationshipsScreenBoon", Widget )

function RelationshipsScreenBoon:init( width, height )
    RelationshipsScreenBoon._base.init( self )

    self.features = {}

    -- Add our content block
    self.bg_border = 45
    self.bg_border_scale = 0.5
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/gameover_block.tex") ) )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )

    -- Add our title label
    self.label = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL ) )
        :Bloom( 0.05 )

    -- Add the Active label and icon
    self.active_icon = self:AddChild( Widget.Image( engine.asset.Texture( "UI/ic_checkmark.tex") ) )
        :SetTintColour( UICOLOURS.OBJECTIVE )
        :SetSize( 18, 18 )
        :Bloom( 0.05 )
    self.active_label = self:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL, LOC( "UI.RELATIONSHIP_SCREEN.ACTIVE" ) ) )
        :SetGlyphColour( UICOLOURS.OBJECTIVE )
        :Bloom( 0.05 )

    -- Contents!
    self.icon_frame = self:AddChild( Widget.Panel( engine.asset.Texture("UI/empty_inner_frame_border_selected_clear.tex") ) )
        :Bloom( 0.05 )
    self.icon = self:AddChild( Widget.Image() )
        :Bloom( 0.05 )
    self.text_content = self:AddChild( Widget() )
    self.title = self.text_content:AddChild( Widget.Label("title", FONT_SIZE.ITEM_SUBTITLE ))
        :SetGlyphColour( UICOLOURS.TITLE )
        :LeftAlign()
        :SetWordWrap( true )
        :OverrideLineHeight( 22 )
        :Bloom( 0.0 )
    self.subtitle = self.text_content:AddChild( Widget.Label("body", FONT_SIZE.ITEM_DESC ))
        :SetGlyphColour( UICOLOURS.BODYTEXT )
        :LeftAlign()
        :SetWordWrap( true )
        :OverrideLineHeight( 18 )
        :Bloom( 0.0 )

    self:SetIconSize( height-SPACING.M1*2 )
    self:SetSize( width, height )
end

function RelationshipsScreenBoon:SetSize( w, h )
    self.width = w or 200
    self.height = h or 90
    self:Layout()
    return self
end

function RelationshipsScreenBoon:SetIconSize( icon_size )
    self.icon_size = icon_size or self.icon_size or 90
    return self
end

function RelationshipsScreenBoon:Layout()

    -- Background
    local nineslice_w, nineslice_h = self.width + self.bg_border*2*self.bg_border_scale, self.height + self.bg_border*2*self.bg_border_scale
    self.bg:SetSize( nineslice_w, nineslice_h )

    -- Layout our title
    self.label:LayoutBounds( "left", "above", self.bg )
        :Offset( 0, 4 )

    -- Layout the Active label and icon
    self.active_icon:LayoutBounds( "right", "above", self.bg )
        :Offset( 0, 8 )
    self.active_label:LayoutBounds( "before", "center", self.active_icon )
        :Offset( -6, 1 )

    -- Layout our contents
    self.icon_frame:SetSize( self.icon_size, self.icon_size )
        :LayoutBounds( "left", "top", self.bg ):Offset( SPACING.M1+2, -SPACING.M1 )
    self.icon:SetSize( self.icon_size, self.icon_size )
        :LayoutBounds( "center", "center", self.icon_frame )
    self.title:SetAutoSize( self.width - self.icon_size - SPACING.M1*3 )
    self.subtitle:SetAutoSize( self.width - self.icon_size - SPACING.M1*3 )
end

function RelationshipsScreenBoon:PostFeature( keyword, name, desc )
    self.features[ keyword ] = { name = name, desc = desc }
end

function RelationshipsScreenBoon:PostCard( feature_id, card )
    self.features[ feature_id ] = { card = card }
end

function RelationshipsScreenBoon:Refresh( boon, active, agent )
    if boon then
        local is_good = boon:GetDef().is_good
        self.boon = boon
        self.agent = agent
        self.active_boon = active or false
        self.icon:SetTexture( boon:GetImage() )

        if is_good then
            self.bg:SetTintColour( UICOLOURS.BONUS )
            self.label:SetText( LOC "UI.RELATIONSHIP_SCREEN.BOON_TITLE" )
            self.label:SetGlyphColour( UICOLOURS.BONUS )
            self.icon_frame:SetTintColour( UICOLOURS.BONUS )
        else
            self.bg:SetTintColour( UICOLOURS.PENALTY )
            self.label:SetText( LOC"UI.RELATIONSHIP_SCREEN.BANE_TITLE" )
            self.label:SetGlyphColour( UICOLOURS.PENALTY )
            self.icon_frame:SetTintColour( UICOLOURS.PENALTY )
        end

        table.clear( self.features )

        if active then
            self.title:SetText( boon:GetName() )
                :SetSaturation(1)
                :SetTintAlpha(1)
            self.subtitle:SetText( boon:GetDesc( self ) )
                :SetSaturation(1)
                :SetTintAlpha(1)
            self.icon:SetSaturation(1)
                :SetTintColour(1,1,1,1)
            self:SetToolTipClass( Widget.TooltipKeywords ):SetToolTip( self.features )
        elseif self.agent and (( is_good and TheGame:GetGameProfile():HasUnlockedBoon( self.agent:GetUniqueID() ) )
        or ( not is_good and TheGame:GetGameProfile():HasUnlockedBane( self.agent:GetUniqueID() ) )) then
            -- Unlocked, but not active
            self.title:SetText( boon:GetName() )
                :SetSaturation(0)
                :SetTintAlpha(.6)
            self.subtitle:SetText( boon:GetDesc( self ) )
                :SetSaturation(0)
                :SetTintAlpha(.5)
            self.icon:SetSaturation(1)
                :SetTintColour(1,1,1,1)
            self:SetToolTipClass( Widget.TooltipKeywords ):SetToolTip( self.features )
        else
            -- Locked
            self.title:SetText( is_good and LOC"UI.RELATIONSHIP_SCREEN.BOON_UNKNOWN_NAME" or LOC"UI.RELATIONSHIP_SCREEN.BANE_UNKNOWN_NAME" )
                :SetSaturation(0)
                :SetTintAlpha(.6)
            self.subtitle:SetText( is_good and LOC"UI.RELATIONSHIP_SCREEN.BOON_UNKNOWN_DESC" or LOC"UI.RELATIONSHIP_SCREEN.BANE_UNKNOWN_DESC" )
                :SetSaturation(0)
                :SetTintAlpha(.5)
            self.icon:SetSaturation(0)
                :SetTintColour(0,0,0,0.4)
            self:SetToolTipClass( nil ):SetToolTip( nil )
        end
        self.active_icon:SetShown( active )
        self.active_label:SetShown( active )
        -- Align text
        self.subtitle
            :LayoutBounds( "left", "below", self.title )
            :Offset( 0, 4 )
        self.text_content
            :LayoutBounds( "after", "center", self.icon )
            :Offset( SPACING.M1, 0 )

        self:Layout()
        
        self:Show()
    else
        self:Hide()
    end
    return self
end
