require "ui/screens/graftcollectionscreen"

local ReplaceGraftScreen = class( "Screen.ReplaceGraftScreen", Screen.GraftCollectionScreen )

local DETAILS_W = RES_X*0.6

function ReplaceGraftScreen:init( graft, on_done )
    ReplaceGraftScreen._base.init( self )

    self.graft_to_install = graft
    self.on_done = on_done

    -- Graft image
    self.new_graft_icon = self.content:AddChild( Widget.GraftDetails( DETAILS_W ) )
        :SetNewGraft()

    self.close_button:SetText( LOC"UI.OVERLAYS.CANCEL" )
        :SetFn( function() self:DoClose(false) end )

    self:RefreshReplacePanel( self.graft_to_install )
end

function ReplaceGraftScreen:RefreshReplacePanel( graft )

    local player = TheGame:GetGameState():GetPlayerAgent()
    local type_name_lowercase = LOC( GRAFT_TYPE_NAMES[ graft:GetType() ] )
    local type_name = LOC( GRAFT_TYPE_NAMES[ graft:GetType() ] )
    self.title:SetText( loc.format( LOC"UI.REPLACE_GRAFT_SCREEN.TITLE", type_name ) )
    self.subtitle:SetText( loc.format( LOC"UI.REPLACE_GRAFT_SCREEN.SUBTITLE", player:GetName(), type_name ) )
        :LayoutBounds( "left", "below", self.title )
        :Offset( 0, 0 )
    self.new_graft_icon:Refresh( graft )
        :SetOwner( player )
        :LayoutBounds( "left", "below", self.title )
        :Offset( -10, -SPACING.M1 )

    if graft:GetType() == GRAFT_TYPE.NEGOTIATION then
        self.graft_widgets[GRAFT_TYPE.COMBAT]:Hide()
        self.title:SetGlyphColour( UICOLOURS.NEGOTIATION )
        self.subtitle:SetGlyphColour( UICOLOURS.NEGOTIATION_LIGHT )
        self.graft_widgets[GRAFT_TYPE.NEGOTIATION]:SetFn( function(...) self:OnSelectGraft(...) end )
            :SetReplacingGraft()
    elseif graft:GetType() == GRAFT_TYPE.COMBAT then
        self.graft_widgets[GRAFT_TYPE.NEGOTIATION]:Hide()
        self.title:SetGlyphColour( UICOLOURS.FIGHT )
        self.subtitle:SetGlyphColour( UICOLOURS.FIGHT_LIGHT )
        self.graft_widgets[GRAFT_TYPE.COMBAT]:SetFn( function(...) self:OnSelectGraft(...) end )
            :SetReplacingGraft()
    end
    self:Layout()
end

function ReplaceGraftScreen:OnSelectGraft( graft, graft_slot_widget )
    
    if graft:IsRemovable() then
        self.close_button:Disable():Hide()
        if graft_slot_widget then
            graft_slot_widget:ShowGained( function()
                TheGame:GetGameState():GetPlayerAgent().graft_owner:ReplaceGraft( graft, self.graft_to_install )
                self:DoClose(true)
            end )
        else
            TheGame:GetGameState():GetPlayerAgent().graft_owner:ReplaceGraft( graft, self.graft_to_install )
        end        
    else

    end
end

function ReplaceGraftScreen:Layout()
    ReplaceGraftScreen._base.Layout( self )

    if self.new_graft_icon then
        self.new_graft_icon
            :LayoutBounds( "left", "below", self.title )
            :Offset( 0, -SPACING.M1 )
        self.subtitle:LayoutBounds( "left", "below", self.new_graft_icon )
            :Offset( 0, -SPACING.M1 )
        self.graft_widgets[GRAFT_TYPE.NEGOTIATION]:LayoutBounds( "left", "below", self.subtitle ):Offset( 0, -SPACING.M1*2 )
        self.graft_widgets[GRAFT_TYPE.COMBAT]:LayoutBounds( "left", "below", self.subtitle ):Offset( 0, -SPACING.M1*2 )
        self.scroll:RefreshView()
    end

    return self
end

function ReplaceGraftScreen:DoClose(installed)
    if self:IsOnStack() then 
        TheGame:FE():PopScreen(self)
        
        if self.on_done then
            self.on_done(installed)
        end
    end
end

function ReplaceGraftScreen:OnFocusMove( dir )
end


function ReplaceGraftScreen:OnControlModeChange( cm, device_id )
    ReplaceGraftScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.close_button:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.close_button:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.close_button:Hide()
    end
end

function ReplaceGraftScreen:OnScreenModeChange( sm )
    ReplaceGraftScreen._base.OnScreenModeChange( self, sm )

    if self.new_graft_icon then
        local content_w = self.title:GetAutoSize()
        self.new_graft_icon:SetWidth( content_w )
        self.graft_widgets[GRAFT_TYPE.COMBAT]:SetWidth( content_w - 40 )
        self.graft_widgets[GRAFT_TYPE.NEGOTIATION]:SetWidth( content_w - 40 )
    end

    self:Layout()
end

function ReplaceGraftScreen:HandleControlDown(control, device)

    if control:Has( Controls.Digital.MENU_CANCEL, Controls.Digital.GAMEPAD_B ) then
        self.close_button:Click()
        return true
    end

    return ReplaceGraftScreen._base.HandleControlDown( self, control, device )
end


function ReplaceGraftScreen:OnUpdate(dt)
    self:ClearRobotOptions()
    self:AddRobotOption(self.close_button)

    ReplaceGraftScreen._base.OnUpdate(self, dt)
end
