require "ui/widgets/screen"

----------------------------------------------------------------------------------------
-- A splash screen notifying the player about the game-saving anim

local SavingScreen = class("Screen.SavingScreen", Widget.Screen)

function SavingScreen:init( on_done_fn )
    SavingScreen._base.init(self)
    
    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )
    
    -- The function we'll call after displaying the save-game anim
    self.on_done_fn = on_done_fn
end

function SavingScreen:OnOpen()
    SavingScreen._base.OnOpen( self )

    -- Ask the frontend to display the saving animation
	TheGame:FE():ShowSaving( 4, self.on_done_fn, true )
end


------------------------------------------------------------------------------------------
-- Shows an anim displaying the save-game animation

local SaveGameAnim = class( "Widget.SaveGameAnim", Widget )

function SaveGameAnim:init()
    SaveGameAnim._base.init( self )

    self.size = 70

    -- The hitbox
    self.hitbox = self:AddChild( Widget.SolidBox( self.size, self.size, 0xffff0000 ) )

    -- The anim
    local anim_size = 460
    local animname = "anim/UI_saving_save_fx.zip"
    self.anim = self:AddChild( Widget.Anim() )
        :SetScale( self.size/anim_size )
        :SetBloom( 0.1 )
    self.anim:AddAnims( GetAnim( animname ) )
    self.anim:AddOverrideFile( animname )

    self.text_content = self:AddChild( Widget() )
    self.title = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_SUBTITLE ) )
        :SetText( LOC"UI.SAVING_SCREEN.SAVING_TITLE" )
        :SetGlyphColour( 0xFFFFCCff )
        :LeftAlign()
        :Bloom( 0.15 )
    self.desc = self.text_content:AddChild( Widget.Label( "body", FONT_SIZE.BODY_TEXT ) )
        :SetWordWrap( true )
        :SetAutoSize( RES_X*0.4 )
        :SetText( LOC"UI.SAVING_SCREEN.SAVING_DESC" )
        :SetGlyphColour( 0xFFFFCCff )
        :LeftAlign()
        :Bloom( 0.05 )
    
    self:LayoutText()
end

function SaveGameAnim:AnimateIn( show_info )
	self:Show()

	-- Show anim
	self.anim:Play( "save_pre", false )
    self.anim:Push( "save_loop", true )

    -- Re-layout text
    self:LayoutText( show_info )

    -- Animate the title
    self.title:SetTintAlpha( 0 )
    local title_x, title_y = self.title:GetPos()
    self.title:SetPos( title_x-20, title_y )
    	:AlphaTo( 1, 0.1, easing.outQuad )
    	:MoveTo( title_x, title_y, 0.2, easing.outQuad )

    -- Animate the description
    self.desc:SetTintAlpha( 0 )
    local desc_x, desc_y = self.desc:GetPos()
    if show_info then
	    self.desc:SetPos( desc_x-40, desc_y )
	    	:AlphaTo( 1, 0.1, easing.outQuad )
	    	:MoveTo( desc_x, desc_y, 0.2, easing.outQuad )
    end

   	return self
end

function SaveGameAnim:AnimateOut()
	-- Show anim
    self.anim:Push( "save_pst", false )

    -- Animate the title
    local title_x, title_y = self.title:GetPos()
    self.title:AlphaTo( 0, 0.2, easing.outQuad, function() self:Hide() end )
    	:MoveTo( title_x-20, title_y, 0.2, easing.outQuad )

    -- Animate the desc
    local desc_x, desc_y = self.desc:GetPos()
    self.desc:AlphaTo( 0, 0.2, easing.outQuad )
    	:MoveTo( desc_x-40, desc_y, 0.2, easing.outQuad )

   	return self
end

function SaveGameAnim:OnScreenModeChange( sm )
    SaveGameAnim._base.OnScreenModeChange( self, sm )

    self:SetLayoutScale( LAYOUT_SCALE[sm] )

   	return self
end

function SaveGameAnim:LayoutText( show_info )
	self.title:SetText( show_info and LOC"UI.SAVING_SCREEN.SAVING_TITLE_INFO" or LOC"UI.SAVING_SCREEN.SAVING_TITLE" )
    self.desc:LayoutBounds( "left", "below", self.title )
        :Offset( 0, 6 )
        :SetShown( show_info )
    self.text_content:LayoutBounds( "after", "center", self.hitbox )
        :Offset( SPACING.M1/2, 4 )
   	return self
end