------------------------------------------------------------------------------------------
-- Shows a sliding sidebar screen. Fill it with whatever content you'd like!

local SlidingSidebarScreen = class( "Screen.SlidingSidebarScreen", Widget.Screen )

SlidingSidebarScreen.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self)
            self:Close()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end
    },
}

SlidingSidebarScreen.SIDEBAR_WIDTH = 470

function SlidingSidebarScreen:init()
    SlidingSidebarScreen._base.init( self )

    self.is_popup = true
    self:SetAnchors("center", "center")

    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Sizes
    self.sidebar_w = SlidingSidebarScreen.SIDEBAR_WIDTH
    
    self.overlay_container = self:AddChild( Widget.Clickable() )
        :SetOnClickFn( function() self:Close() end )
    self.overlay = self.overlay_container:AddChild( Widget.SolidBox( self.screen_w, self.screen_h, 0x072F34ff ) )
        :SetTintAlpha( 0 )
        :BlockMouse()

    self.sidebar_container = self:AddChild( Widget() )
    self.sidebar_hitbox = self.sidebar_container:AddChild( Widget.Image( engine.asset.Texture( "splash/underlay.tex" ) ) )
        :BlockMouse()
    self.scroll = self.sidebar_container:AddChild( Widget.ScrollPanel() )
        :SetVirtualTopMargin( SPACING.M1*2 )
        :SetVirtualBottomMargin( SPACING.M1*2 )
        :SetScrollBarOuterMargin( SPACING.M1 )
        :LeftScrollBar( true )
        :ShowBar( SCROLLBAR.ALWAYS )
    self.contents = self.scroll:AddScrollChild( Widget() )

    self.sidebar_edge = self.sidebar_container:AddChild( Widget.SolidBox( 3, self.screen_h, UICOLOURS.SUBTITLE ) )
    self.close_button = self.sidebar_container:AddChild( Widget.ImageButton( engine.asset.Texture("UI/btn_filter_close.tex"), function() self:Close() end ) )
        :SetSize( FONT_SIZE.SCREEN_TITLE*0.8, FONT_SIZE.SCREEN_TITLE*0.8 )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.15 )
end

function SlidingSidebarScreen:Close()
    self:AnimateOut()
end

function SlidingSidebarScreen:OnGainTop()
    SlidingSidebarScreen._base.OnGainTop(self)

    if not self.animated_in then
        self.animated_in = true
        self:AnimateIn()
    end
end

function SlidingSidebarScreen:OnControlModeChange( cm, device_id )
    SlidingSidebarScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.close_button:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.close_button:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.close_button:Hide()

        local focus = self:GetDefaultFocus()
        if focus then focus:SetFocus() end
    end
end

function SlidingSidebarScreen:OnScreenModeChange( sm )
    SlidingSidebarScreen._base.OnScreenModeChange( self, sm )

    self.sidebar_w = SlidingSidebarScreen.SIDEBAR_WIDTH * LAYOUT_SCALE[sm]
    self.close_button:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.contents:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:Layout()
end

function SlidingSidebarScreen:AnimateIn()
    local sidebar_x, sidebar_y = self.sidebar_container:GetPos()
    self.sidebar_container:SetPos( sidebar_x-100, sidebar_y )
        :MoveTo( sidebar_x, sidebar_y, 0.1, easing.outQuad )
        :AlphaTo( 1, 0.1, easing.outQuad )
    self.overlay:AlphaTo( 0.5, 0.2, easing.outQuad )
    return self
end

function SlidingSidebarScreen:AnimateOut()
    local sidebar_x, sidebar_y = self.sidebar_container:GetPos()
    self.sidebar_container:MoveTo( sidebar_x-100, sidebar_y, 0.2, easing.outQuad )
        :AlphaTo( 0, 0.2, easing.outQuad )
    self.overlay:AlphaTo( 0, 0.4, easing.outQuad, function() TheGame:FE():PopScreen() end )
    return self
end

function SlidingSidebarScreen:Layout()
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Get MainOverlay's top bar height
    local overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    local overlay_topbar_height = overlay and overlay:GetTopBarHeight() or 0

    self.overlay:SetSize( self.screen_w, self.screen_h )
    self.sidebar_hitbox:SetSize( self.sidebar_w, self.screen_h-overlay_topbar_height )
    self.scroll:SetSize( self.sidebar_w, self.screen_h-overlay_topbar_height )
        :RefreshView()
    self.sidebar_edge:SetSize( 3, self.screen_h-overlay_topbar_height )
        :LayoutBounds( "right", "center", self.sidebar_hitbox )
    self.close_button:LayoutBounds( "after", "top", self.sidebar_edge )
        :Offset( -2, 0 )
    self.sidebar_container:LayoutBounds( "left", "bottom", self.overlay )

    return self
end

function SlidingSidebarScreen:GetDefaultFocus()
    if not self.contents:IsEmpty() then
        return self.contents.children[1]
    end
end