require "ui/widgets/screen"
require "ui/screens/relationshipsscreen"

----------------------------------------------------------------------------------------------------------------------------
-- Displays a newly received social boon or bane
local SocialGraftScreen = class( "Screen.SocialGraftScreen", Widget.Screen )

local CONTENT_W = RES_X*0.4
local GRAFT_ICON_SIZE = 90


SocialGraftScreen.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self) 
            self.button_continue:Click()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end
    },
}

function SocialGraftScreen:init( agents, fn )
    Widget.Screen.init( self )
    self.agents = agents

    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )


    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.ACCENT )
    self.grid:SetTintAlpha( 0.8 )
    
    -- Backing panel
    self.panel_bg = self:AddChild( Widget.WidePanel( 400 ) )

    -- Our fancy character anim!
    self.anim_root = self:AddChild(Widget()):SetHiddenBoundingBox( true )

    -- Content
    self.contents = self:AddChild(Widget())
    self.title = self.contents:AddChild( Widget.Label( "title", 40 ) )
        :SetAutoSize( CONTENT_W )
        :SetWordWrap( true )
        :LeftAlign()
		:Bloom( 0.1 )
    self.subtitle = self.contents:AddChild( Widget.Label( "body", 24 ) )
        :SetAutoSize( CONTENT_W )
        :SetWordWrap( true )
        :OverrideLineHeight( 19 )
        :LeftAlign()
        :Bloom( 0.05 )
    self.reason = self.contents:AddChild( Widget.Label( "body", 24 ) )
		:SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :SetAutoSize( CONTENT_W )
        :SetWordWrap( true )
        :OverrideLineHeight( 19 )
        :LeftAlign()
        :Bloom( 0.1 )

    -- Graft details
    self.graft_root = self.contents:AddChild( Widget.RelationshipsScreenBoon( CONTENT_W, 90 ))

    -- Continue button
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.button_continue = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.DAY_OVER_SCREEN.BUTTON_CONTINUE", 
        function()
            if (self.idx or 0) < #self.agents then
                self:ShowNextAgent()
            else
                self:GetFE():FadeTransition( function()
                    self:GetFE():PopScreen(self) 
                    if fn then 
                        fn()
                    end
                end, .2 )
            end
        end ) )
        :SetIcon( global_images.accept )
        :LayoutBounds( "right", "above", -60, 60 )
        :Hide()
end


function SocialGraftScreen:OnUpdate(dt)
    self:ClearRobotOptions()
    self:AddRobotOption(self.button_continue)
    SocialGraftScreen._base.OnUpdate(self, dt)
end

function SocialGraftScreen:ShowNextAgent()
    self.idx = (self.idx or 0) + 1
    local agent = self.agents[self.idx]

    -- Set the agent's reason for their opinion of you
    local gamestate = TheGame:GetGameState()
    local player = gamestate:GetPlayerAgent()
    
    -- Setup data
    self.agent = agent and agent or Agent("JAKES_SMUGGLER")
    self.relationship = agent:GetRelationship()
    local social_graft_id = self.agent:GetSocialGraft( self.relationship )
    self.social_graft = social_graft_id and GraftInstance( social_graft_id )
    
    local why_event = self.agent.social_connections and self.agent.social_connections:GetRelationshipReason( player )
    if why_event then
        self.reason:SetText( loc.format( LOC"UI.SOCIAL_GRAFT_SCREEN.REASON", FormatOpinionEvent( why_event, self.agent ), why_event.time ) )
    end

    self:RefreshBigCharacter( self.agent )

    if self.social_graft then
        self.graft_root:Refresh( self.social_graft, true ):Show()
    else
        self.graft_root:Hide()
    end

    -- Update context content
    if self.relationship == RELATIONSHIP.HATED then
        AUDIO:PlayEvent("event:/ui/notification/screen_hated")
        local event_name = self.agent:GetCombatVOEvent("growl_fast_closed")
        if event_name then
            AUDIO:PlayEvent( event_name, "sound/vo.bank" )
        end

        self.panel_bg:SetIcon( global_images.graft_icons[ self.relationship ], UICOLOURS.RELATIONSHIP_HATED )
            :SetColour( UICOLOURS.RELATIONSHIP_HATED )
        self.title:SetGlyphColour( UICOLOURS.RELATIONSHIP_HATED )
            :SetText( string.upper( loc.format( LOC"UI.SOCIAL_GRAFT_SCREEN.TITLE_HATED", self.agent:LocTable() ) ) )
        self.subtitle:SetGlyphColour( 0xE47676FF )
            :SetText( loc.format( LOC"UI.SOCIAL_GRAFT_SCREEN.SUBTITLE_HATED", self.agent:LocTable() ) )

    elseif self.relationship == RELATIONSHIP.LOVED then
        AUDIO:PlayEvent("event:/ui/notification/screen_loved")
        self.panel_bg:SetIcon( global_images.graft_icons[ self.relationship ], UICOLOURS.RELATIONSHIP_LOVED )
            :SetColour( UICOLOURS.RELATIONSHIP_LOVED )
        self.title:SetGlyphColour( UICOLOURS.RELATIONSHIP_LOVED )
            :SetText( string.upper( loc.format( LOC"UI.SOCIAL_GRAFT_SCREEN.TITLE_LOVED", self.agent:LocTable() ) ) )
        self.subtitle:SetGlyphColour( 0xA1D285FF )
            :SetText( loc.format( LOC"UI.SOCIAL_GRAFT_SCREEN.SUBTITLE_LOVED", self.agent:LocTable() ) )

    else
        local clr = RELATIONSHIP_COLOURS[ self.relationship ]
        self.panel_bg:SetIcon( global_images.graft_icons[ self.relationship ], clr )
            :SetColour( clr )
        self.title:SetGlyphColour( clr )
            :SetText( string.upper( GetRelationshipModifierString( self.relationship, self.agent )))
        self.subtitle:SetText( nil )
    end

    self:Layout()
end

function SocialGraftScreen:Layout()

	-- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Floor grid
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )

    -- Contents
    self.title:LayoutBounds( "center", "center", 0, 0 )
    self.subtitle:LayoutBounds( "left", "below", self.title ):Offset( 0, 4 )
    self.reason:LayoutBounds( "left", "below", self.subtitle ):Offset( 0, -10 )
    self.graft_root:LayoutBounds( "left", "below", self.reason ):Offset( 0, -20 )

    -- Get content size
    local w, h = self.contents:GetScaledSize()
    self.panel_bg:SetSize( w, h+60 )
    self.contents:LayoutBounds( "center", "below", self.panel_bg:GetTopEdgeWidget() )
        :Offset( 0, -25 )
end

function SocialGraftScreen:OnScreenResize(w, h)
    self._base.OnScreenResize(self, w, h)
    self:Layout()
    self:RefreshStreamerMode()
end

function SocialGraftScreen:RefreshStreamerMode()
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

function SocialGraftScreen:OnOpen()
    SocialGraftScreen._base.OnOpen( self )

    -- Refresh control mode display
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )

    self:ShowNextAgent()
    self:AnimateIn()
end

function SocialGraftScreen:OnControlModeChange( cm, device_id )
    SocialGraftScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.button_continue:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.button_continue:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.button_continue:Hide()
    end
end

function SocialGraftScreen:OnScreenModeChange( sm )
    SocialGraftScreen._base.OnScreenModeChange( self, sm )

    self.contents:SetLayoutScale( LAYOUT_SCALE[sm] )
    self:Layout()

    return self
end

local OFFSET = RES_X/4

function SocialGraftScreen:RefreshBigCharacter( agent )
    
    if agent then
        
        self.biganim = self.biganim or self.anim_root:AddChild( Widget.AnimatedCharacter() )
        self.biganim:Refresh( agent )

        local x = 0
        local y = -340
        if TheGame:FE():GetScreenMode() == SCREEN_MODE.SMALL then
            x = -RES_X/2.4
        elseif TheGame:FE():GetScreenMode() == SCREEN_MODE.TV then
            x = -RES_X/2.5
        elseif TheGame:FE():GetScreenMode() == SCREEN_MODE.MONITOR then
            x = -RES_X/3
        end

        if self.relationship == RELATIONSHIP.HATED then
        	self.biganim:SetMood( MOOD.ANGRY )
        	local k, emote = table.tablepick( { "angry_accuse", "spit" } )
            self.current_emote = emote
        elseif self.relationship == RELATIONSHIP.LOVED then
        	self.biganim:SetMood( MOOD.HAPPY )
        	local k, emote = table.tablepick( { "bashful" } )--, "greeting" } )
            self.current_emote = emote
        end
        self.biganim:SetScale( 1 )
        self.biganim:SetPos( x-OFFSET, y )
        self.biganim:MoveTo( x, y, .6, easing.outQuad)
    end
end

function SocialGraftScreen:AnimateIn()
	self.panel_bg:PrepareAnimation()
	self.title:SetTintAlpha(0)
	self.subtitle:SetTintAlpha(0)
	self.reason:SetTintAlpha(0)
    self.graft_root:SetTintAlpha(0)
	self.panel_bg:FadeIn( function() self:FadedIn() end )
end

function SocialGraftScreen:FadedIn()
	local pacing = 0.1
	local title_x, title_y = self.title:GetPos()
	local subtitle_x, subtitle_y = self.subtitle:GetPos()
	local reason_x, reason_y = self.reason:GetPos()
	local graft_title_x, graft_title_y = self.graft_root:GetPos()

	self:RunUpdater(
		Updater.Series{
            Updater.Parallel{
                Updater.Ease( function(v) self.title:SetTintAlpha(v) end, 0, 1, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.title:SetPos(x,y) end, title_x+40, title_y, title_x, title_y, pacing, easing.linear),
            },
            Updater.Parallel{
                Updater.Ease( function(v) self.subtitle:SetTintAlpha(v) end, 0, 1, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.subtitle:SetPos(x,y) end, subtitle_x+30, subtitle_y, subtitle_x, subtitle_y, pacing, easing.linear),
            },
            Updater.Wait( pacing ),
            Updater.Parallel{
                Updater.Ease( function(v) self.reason:SetTintAlpha(v) end, 0, 1, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.reason:SetPos(x,y) end, reason_x+20, reason_y, reason_x, reason_y, pacing, easing.linear),
            },
            Updater.Wait( pacing*3 ),
            Updater.Parallel{
                Updater.Ease( function(v) self.graft_root:SetTintAlpha(v) end, 0, 1, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.graft_root:SetPos(x,y) end, graft_title_x+30, graft_title_y, graft_title_x, graft_title_y, pacing, easing.linear),
            },
            Updater.Do( function() self.biganim:Emote( self.current_emote ) end ),
            Updater.Wait( pacing*4 ),
            Updater.Do( function() self.button_continue:SetShown( TheGame:FE():GetControlMode() == CONTROL_MODE.MOUSE_KEYBOARD or TheGame:FE():GetControlMode() == CONTROL_MODE.TOUCH ) end ),
        }
	)
end