require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/panelbutton"
require "ui/widgets/dropdown"
require "ui/widgets/movieplayer"
local debug_menus = require "debug/debug_menus"


local TestScreen = class( "Screen.TestScreen", Widget.Screen )
TestScreen.IS_EDITOR = true
TestScreen.can_reload = true

local SZ = 92
local FONT = "default"
local RED = 0xFF0000FF
local GREEN = 0x00FF00FF
local BLUE = 0x0000FFFF
local show_test_window = false
local LONG_TXT = "This is a line of text. It's really long and will wrap. Long long long long."

-----------------------------------------------------------------------------

local FocusGroup = class( "Widget.FocusGroup", Widget )

function FocusGroup:OnGainFocus()
    Widget.OnGainFocus( self )
    local w, h = self:GetSize()
    local cx, cy = self:GetCentroid()
    self.bg = self:AddChild( Widget.SolidBox( w + 32, h + 32, 0x555555AA ), 1 )
    self.bg:SetPos( cx, cy )
end

function FocusGroup:OnLoseFocus()
    Widget.OnLoseFocus( self )
    self.bg:Remove()
    self.bg = nil
end


-----------------------------------------------------------------------------

local FocusBox = class( "Widget.FocusBox", Widget.SolidBox )
FocusBox.can_focus = true

function FocusBox:OnGainFocus()
    Widget.OnGainFocus( self )
    self.prev_color = { self.image.tint_r, self.image.tint_g, self.image.tint_b, self.image.tint_a }
    self:SetColour( 0xFFFFFFFF )
end

function FocusBox:OnLoseFocus()
    Widget.OnLoseFocus( self )
    self:SetColour( table.unpack( self.prev_color ))
end

-----------------------------------------------------------------------------


function TestScreen:init( )
    TestScreen._base.init(self)

    local w, h = TheGame:FE():GetScreenDims()

    self.group_root = self:AddChild( Widget() )

    self.box = self:AddChild( Widget.SolidBox( RES_X, 256, 1, 0, 0, 0.2 )):SetPos(0,130)
    self.box:SetAnchors( "center", "bottom" )

    self.box2 = self.box:AddChild( Widget.SolidBox( 64, 64, 0, 1, 0, 1 ))

    -- for i = 1, 3 do
    --     local box = self.group3:AddChild( Widget.FocusBox( 128, 128, 0x77CCCCFF ))
    --     box:SetName( string.format( "GRP3.BOX%d", i ))
    --     box:SetPos( i * 200, 0 )
    -- end

end

function TestScreen:OnAdded()
    -- self.group1.children[1]:SetFocus()
end

function TestScreen:OnRemoved()
end

-- function TestScreen:OnScreenResize( w, h )
--     print (" RESIZE" )
--     --self.box:SetPos( w/2, h/2 )
-- end

function TestScreen:GetDebugEnv( env )
    env.focus = self:GetFE():GetFocusWidget()
    env.scroll = self.scroll
end

function TestScreen:DoRender()
	local ImGui = TheGame:GetDebug():GetDebugUI()
    -- 3. Show the ImGui test window. Most of the sample code is in ImGui::ShowTestWindow()
    if (show_test_window) then
        ImGui:SetNextWindowPos( 650, 20, ImGui.ImGuiSetCond_FirstUseEver )
        show_test_window = ImGui:ShowTestWindow( show_test_window );
	end
end

function TestScreen:RenderDebugUI( ui, panel )
    if self.scroll then
        if ui:Checkbox( "Left Bar", self.scroll.left_scroll ) then
            self.scroll:LeftScrollBar( not self.scroll.left_scroll )
        end

        local t = MakeEnumArray( SCROLLBAR )
        local idx = table.find( t, self.scroll.show_bar ) or 1
        local new_idx = ui:Combo( "Scroll Vis", idx, t )
        if new_idx and new_idx ~= idx then
            self.scroll:ShowBar( t[ new_idx ])
        end

        local new_margin = ui:SliderFloat( "Inner Margin", self.scroll.scrollbar_margin, 0, 50 )
        if new_margin and new_margin ~= self.scroll.scrollbar_margin then
            self.scroll:SetScrollBarMargin( new_margin )
        end

        local new_margin = ui:SliderFloat( "Outer Margin", self.scroll.scrollbar_outer_margin, 0, 50 )
        if new_margin and new_margin ~= self.scroll.scrollbar_outer_margin then
            self.scroll:SetScrollBarOuterMargin( new_margin )
        end

        local new_margin = ui:SliderFloat( "Virtual Top Margin", self.scroll.virtual_top_margin or 0, 0, 200 )
        if new_margin and new_margin ~= self.scroll.virtual_top_margin then
            self.scroll:SetVirtualTopMargin( new_margin )
        end

        local new_margin = ui:SliderFloat( "Virtual Bottom Margin", self.scroll.virtual_bottom_margin or 0, 0, 200 )
        if new_margin and new_margin ~= self.scroll.virtual_bottom_margin then
            self.scroll:SetVirtualBottomMargin( new_margin )
        end

        local new_lines = ui:SliderInt( "Line Count", self.line_count or 0, 0, 20 )
        if new_lines ~= self.line_count then
            self.line_count = new_lines
            self.scroll:Clear()
            for i = 1, new_lines do
                local str = string.format( "%d) A long line that will probably cut off", i )
                local label = self.scroll:AddScrollChild( Widget.Label( "title", 46, tostring(i) .. " This is a line of text will it cut and how long does it go" ))
                label:LayoutBounds( nil, "below" )
            end
            self.scroll:RefreshView()
        end
    end
end

function TestScreen:HandleControlDown(control, device)
    if control:Has( Controls.Digital.MENU_CANCEL ) then
    	print( "RELOAD" )
		TheGame:FE():PopScreen()

        package.loaded[ "ui/screens/testscreen" ] = nil
        local TestScreen = require( "ui/screens/testscreen" )

        TheGame:GetDebug():ClearPanels()
		TheGame:FE():PushScreen( TestScreen() )
    end
end

return TestScreen




