require "ui/widgets/screen"
require "ui/widgets/trackedagentportrait"
require "ui/widgets/tooltipquest"

function MakeHermite(points_out, num_points, x1, y1, x2, y2, tx1, ty1, tx2, ty2)
    local t = 0
    local step = 1/num_points

    for i = 0, num_points do 
        t = i * step
 
        local h00 = (2.0*t*t*t - 3.0*t*t + 1)
        local h10 = (t*t*t - 2.0*t*t + t) 
        local h01 = (-2.0*t*t*t + 3.0*t*t)
        local h11 = (t*t*t - t*t)

        local x =  x1 * h00
                + tx1 * h10
                + x2 * h01
                + tx2 * h11

        local y =  y1 * h00
                + ty1 * h10
                + y2 * h01
                + ty2 * h11

        table.insert(points_out, {x=x, y=y})
    end
end

local function MakeSmoothPath(control_points)

    local A = .5
    local POINTS_PER = 4
    local path = {}

    for k = 1, #control_points-1 do
        
        local x1 = control_points[k].x
        local y1 = control_points[k].y

        local x2 = control_points[k+1].x
        local y2 = control_points[k+1].y

        local tx1, ty1, tx2, ty2
        if k == 1 then
            tx1 = A * ( control_points[k+1].x - control_points[k].x)
            ty1 = A * ( control_points[k+1].y - control_points[k].y)
        else
            tx1 = A * ( control_points[k+1].x - control_points[k-1].x)
            ty1 = A * ( control_points[k+1].y - control_points[k-1].y)
        end

        if k+1 == #control_points then
            tx2 = A * ( control_points[k+1].x - control_points[k].x)
            ty2 = A * ( control_points[k+1].y - control_points[k].y)
        else
            tx2 = A * ( control_points[k+2].x - control_points[k].x)
            ty2 = A * ( control_points[k+2].y - control_points[k].y)
        end
    
        MakeHermite(path, POINTS_PER, x1, y1, x2, y2, tx1, ty1, tx2, ty2)

    end

    local total_length = 0
    for k = 1, #path -1 do
        local length = distance(path[k].x, path[k].y, path[k+1].x, path[k+1].y)

        path[k].total_length = total_length
        path[k].length = length

        total_length = total_length + length
    end

    return path, total_length
end



local function TravelAlongPath(path, distance_to_travel, start_dist)

    local target_dist = start_dist + distance_to_travel
    --yeah, we should probably binary search this, but...
    for k,v in ipairs(path) do
        if k < #path then
            local start_dist = path[k].total_length
            local seg_length = path[k].length
            if target_dist <= start_dist + seg_length then
                local p = seg_length > 0 and ( (target_dist - start_dist) / seg_length ) or 1
                local x1, y1, x2, y2 = path[k].x , path[k].y, path[k+1].x, path[k+1].y
                return x1 + p * (x2-x1), y1 + p * (y2-y1)
            end
        end
    end

    return path[#path].x, path[#path].y
end

------------------------------------------------------------------------------

local CONFRONT_ICON_OFFSET = 45
local PLAYER_HEIGHT = 74


local WIDTH = 200
local HEIGHT = 40
local PADDING = 10
local ICON_SIZE = 50
local QUEST_ICONS_SIZE = 60
local TEXT_SIZE = 20

------------------------------------------------------------------------------

local PlayerMarker = class( "Widget.PlayerMarker", Widget)

function PlayerMarker:init()
    PlayerMarker._base.init(self)
    self.hover_check = true

    self.player_portrait = self:AddChild(Widget.TrackedAgentPortrait( TheGame:GetGameState():GetPlayerAgent() ):SetHeight(PLAYER_HEIGHT) )
    self.player_portrait:ShowReflection()
    self.player_portrait:LayoutBounds("center", "bottom", 0, 0)
    self.player_portrait:SetToolTipClass(nil)
    self.player_portrait:SetToolTip(LOC"UI.TRAVEL_SCREEN.PLAYER_PORTRAIT_TOOLTIP")
end

function PlayerMarker:Appear()
    self:SetShown( self:GetFE():IsHUDVisible() )
    self:PlayPositionalEvent("event:/ui/overworld/map_location_player")
    self:ScaleTo(.2, 1, .2, easing.inQuad)
end

function PlayerMarker:Refresh()
    self.player_portrait:SetToolTipClass(nil)
    self.player_portrait:SetToolTip(LOC"UI.TRAVEL_SCREEN.PLAYER_PORTRAIT_TOOLTIP")
    return self
end

function PlayerMarker:ShowName( visible )
    return self
end

------------------------------------------------------------------------------

local TravelWidget = class( "Widget.TravelWidget", Widget.Clickable)

function TravelWidget:init( location, quest, objectives, fn )
    TravelWidget._base.init(self)


    self.location = location
    self.quest = quest
    self.objectives = objectives

    self.anim_back = self:AddChild(Widget()):SetHiddenBoundingBox( true )

    self.location_overlay = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.location_shadow = self.location_overlay:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex") ) )
        :SetHiddenBoundingBox( true )
        :SetTintColour( 0x021A1860 )
    self.location_name = self.location_overlay:AddChild( Widget.Label("title", 34 ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetHoverLinkGlyphColour( UICOLOURS.SUBTITLE )
        :SetText( self.location:GetName() )
        :Bloom( 0.1 )
    self.location_type = self.location_overlay:AddChild( Widget.Label("title", 22) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetHoverLinkGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.7 )
        :SetText( LOC"UI.TRAVEL_SCREEN.LOCATION_SUBTITLE" )
        :Bloom( 0.1 )
        :LayoutBounds( "center", "above", self.location_name )
        :Offset( 0, -7 )
    local w, h = self.location_name:GetSize()
    self.location_shadow:SetSize( w+300, h+300 )
    self.location_overlay:SetPos( 0, 40 )
        :SetTintAlpha( 0 )
        :Hide()

    self.scalable_contents = self:AddChild( Widget() )
    self.arrow = self.scalable_contents:AddChild( Widget.Image( engine.asset.Texture("UI/location_widget_pointer.tex") ) )
        :SetSize( QUEST_ICONS_SIZE*.7, QUEST_ICONS_SIZE*.7 )
        :SetPos(0, -QUEST_ICONS_SIZE*.34)
        :SetTintColour(0x09e1dcff)
    self.icon = self.scalable_contents:AddChild( Widget.Image() )
        :SetSize( QUEST_ICONS_SIZE, QUEST_ICONS_SIZE )


    self:SetOnClickFn( function()
        fn( self, self.location )
    end )

    -- self:SetScales(1, 1.05, 1)
    -- self:SetHighlightWidget(self.icon)
    self:Refresh()

    if quest:GetRank() > TheGame:GetGameState():GetDefaultQuestRank() then
        self:SetDangerous()
    end
    if quest:GetType() == QTYPE.OPPORTUNITY then
        self:SetTemporaryEvent()
    end
    if quest:GetQuestDef().home then
        self:SetHomeLocation()
    end
    
    self.scalable_contents:Bloom( 0.02 )
end


function TravelWidget:__tostring()
    return loc.format("TRAVELWIDGET({1})", self.location and self.location:GetName() or "")
end

function TravelWidget:Refresh()
    self:Enable()
    local texture = self.quest:GetMarkData(self.objectives) or self.location:GetIcon() or global_images.default_quest_icon
    self.icon:SetTexture(texture)
    self:SetToolTipClass( Widget.TooltipQuest )
    self:SetToolTip( { quest = self.quest, objectives = self.objectives, location = self.location } )
    return self
end

function TravelWidget:GetLocation()
    return self.location
end

function TravelWidget:GetQuest()
    return self.quest
end

function TravelWidget:GetObjectives()
    return self.objectives
end

function TravelWidget:SetDangerous()
    self.arrow:SetTintColour(0xFF4646ff)
    self.danger_overlay = self.icon:AddChild( Widget.Image( engine.asset.Texture('UI/danger_overlay.tex') ) )
        :SetSize( QUEST_ICONS_SIZE, QUEST_ICONS_SIZE )
        :LayoutBounds( "center", "center", self.icon )
end

function TravelWidget:SetTemporaryEvent()
    self.arrow:SetTintColour( UICOLOURS.OPPORTUNITY )
    self.location_name:SetGlyphColour( UICOLOURS.OPPORTUNITY )
    self.location_name:SetHoverLinkGlyphColour( UICOLOURS.OPPORTUNITY )
    self.location_type:SetGlyphColour( UICOLOURS.OPPORTUNITY )
    self.location_type:SetHoverLinkGlyphColour( UICOLOURS.OPPORTUNITY )
end

function TravelWidget:SetHomeLocation()
    self.arrow:SetTintColour(UICOLOURS.QUEST)
        :SetSize( QUEST_ICONS_SIZE*.88, QUEST_ICONS_SIZE*.88 )
        :SetPos( 0, -QUEST_ICONS_SIZE*.25 )
    self.location_name:SetGlyphColour(UICOLOURS.QUEST)
    self.location_name:SetHoverLinkGlyphColour(UICOLOURS.QUEST)
    self.location_type:SetGlyphColour(UICOLOURS.QUEST)
    self.location_type:SetHoverLinkGlyphColour(UICOLOURS.QUEST)
end

function TravelWidget:AppearCoro()
    self:Show():Disable()
    if self.anim then
        self.anim:Hide()
    end

    self.icon:Hide()
    self.arrow:Hide()
    local oppo = self.quest and self.quest:GetType() == QTYPE.OPPORTUNITY
    local anim = engine.asset.Anim( oppo and "anim/map_new_opportunity_fx.zip" or "anim/map_new_location_fx.zip" )
    self:PlayPositionalEvent(oppo and "event:/ui/overworld/map_location_new_opportunity" or "event:/ui/overworld/map_location_new")

    self.anim = self.anim or self.anim_back:AddChild( Widget.Anim() )

    self.anim:AddAnims(anim):SetBuild(anim):SetScale(.33, .33):Play("pre"):Push("loop", true):SetBloom(.4)
    self:Delay(.3)
    
    self.icon:ScaleTo(.2, 1, .2, easing.inQuad, function() self.arrow:Show() end)
    
    self.icon:Show()
    self.appearing_coro = nil
end

function TravelWidget:Appear()
    self.appearing_coro = self:StartCoroutine(TravelWidget.AppearCoro, self)
end

function TravelWidget:IsAppearing()
    return self.appearing_coro and not self.appearing_coro:IsDone()
end

function TravelWidget:ExplodeCoro()

    
    self:PlayPositionalEvent("event:/ui/overworld/map_location_expired")

    self.arrow:Hide()
    if self.anim then
        self.anim:Play("pst")
    end

    self.icon:ScaleTo(1, 2, .4, easing.inQuad)
    self.icon:AlphaTo(0, .3)
    self:Delay(2)
    self:Hide()
end

function TravelWidget:Explode()
    self.exploded = true
    self:StartCoroutine(TravelWidget.ExplodeCoro, self)
end

function TravelWidget:DisappearCoro(delay)
    --self:PlayPositionalEvent("event:/ui/overworld/map_location_expired")
    if delay then
        self:Delay(delay)
    end
    self.arrow:Hide()
    if self.anim then
        self.anim:Play("pst")
    end

    self.icon:ScaleTo(1, .2, .3, easing.outQuad)
    self.icon:AlphaTo(0, .2)
    self:Delay(.5)
    self:Hide()
end

function TravelWidget:Disappear(delay)
    if not self.exploded then
        self:StartCoroutine(TravelWidget.DisappearCoro, self, delay)
    end
end

function TravelWidget:SetOnHoverFn( fn )
    self.on_hover_fn = fn
    return self
end

function TravelWidget:UpdateImage()

    if self.enabled and ( self.hover or self.focus or self.selected or self.show_name ) then
        self.scalable_contents:ScaleTo( nil, 1.1, 0.1, easing.inQuad )
        self.scalable_contents:MoveTo( 0, 4, 0.1, easing.inQuad )
        self.scalable_contents:Bloom( 0.05 )

        UIHelpers.KeepOnScreen( self.location_overlay, 50 )
        self.location_overlay:Show()
            :AlphaTo( 1, 0.2, easing.inQuad )
            :MoveTo( nil, 70, 0.2, easing.inQuad )

        if self.on_hover_fn then self.on_hover_fn( true, self ) end
    else
        self.scalable_contents:ScaleTo( nil, 1, 0.1, easing.inQuad )
        self.scalable_contents:MoveTo( 0, 0, 0.1, easing.inQuad )
        self.scalable_contents:Bloom( 0.02 )

        self.location_overlay
            :AlphaTo( 0, 0.1, easing.inQuad, function() self.location_overlay:Hide() end )
            :MoveTo( nil, 40, 0.1, easing.inQuad )

        if self.on_hover_fn then self.on_hover_fn( false, self ) end
    end
end

function TravelWidget:OnGainFocus()
    TravelWidget._base.OnGainFocus(self)
    self:UpdateImage()
end

function TravelWidget:OnLoseFocus()
    TravelWidget._base.OnLoseFocus(self)
    self:UpdateImage()
end

function TravelWidget:OnGainHover()
    TravelWidget._base.OnGainHover(self)
    self:UpdateImage()
end

function TravelWidget:OnLoseHover()
    TravelWidget._base.OnLoseHover(self)
    self:UpdateImage()
end

function TravelWidget:SetSelected( selected )
    self.selected = selected or false
    self:UpdateImage()
    return self
end

function TravelWidget:ShowName( visible )
    self.show_name = visible or false
    self:UpdateImage()
    return self
end

------------------------------------------------------------------------------------------
-- Shows an interruption icon

local EventTravelWidget = class( "Widget.EventTravelWidget", Widget )

function EventTravelWidget:init( )
    EventTravelWidget._base.init( self )

    self.anim_back = self:AddChild( Widget() ):SetHiddenBoundingBox( true )
    self.container = self:AddChild( Widget() )

    self.arrow = self.container:AddChild( Widget.Image( engine.asset.Texture("UI/location_widget_pointer.tex") ) )
        :SetSize( ICON_SIZE * 0.7, ICON_SIZE * 0.7 )
        :SetBloom(0.05)
    self.icon = self.container:AddChild( Widget.Image( engine.asset.Texture("UI/location_surprise.tex") ) )
        :SetSize( ICON_SIZE, ICON_SIZE )
        :SetBloom(0.05)
    self.arrow:LayoutBounds( "center", "bottom", self.icon ):Offset( 0, -6 )
    self.container:Hide()
end

function EventTravelWidget:AppearCoro()
    self:Show()
    
    self.container:Hide()
    local anim = engine.asset.Anim( "anim/map_new_location_fx.zip" )
    -- self:PlayPositionalEvent("event:/ui/overworld/map_location_new")
    self.anim = self.anim or self.anim_back:AddChild( Widget.Anim() ) 
    self.anim:AddAnims(anim):SetBuild(anim):SetScale(.33, .33):Play("pre"):Push("loop", true):SetBloom(.4)
    self:PlayPositionalEvent("event:/ui/overworld/interrupt_popup")
    self:Delay(.3)
    self.container:ScaleTo(.2, 1, .2, easing.inQuad):Show()
end

function EventTravelWidget:Appear()
    self:StartCoroutine(EventTravelWidget.AppearCoro, self)
end

function EventTravelWidget:ShowName( visible )
    return self
end

---------------------------------------------------------

local SOUND_PARAM_VALUE =
{
    [DAY_PHASE.DAY] = 1,
    [DAY_PHASE.NIGHT] = 3,
}

local TravelScreen = class( "Screen.TravelScreen", Widget.Screen)

TravelScreen.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function(self) return self.traveling and not self.traveling_fast end,
        fn = function(self) 
            self.traveling_fast = true
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.FAST_TRAVEL", Controls.Digital.MENU_ACCEPT))
        end
    },
    {
        control = Controls.Digital.SKIP,
        test = function(self) return self.traveling and not self.traveling_fast end,
        fn = function(self) 
            self.traveling_fast = true
            return true
        end,
    },
    {
        control = Controls.Digital.MENU_ALT,
        test = function(self) return self.traveling and not self.traveling_fast end,
        fn = function(self) 
            self.traveling_fast = true
            return true
        end,
    },
    {
        control = Controls.Digital.PAUSE, 
        fn = function(self)
            self:GetFE():InsertScreen(Screen.PauseMenu())
            return true
        end,
    },
    {
        hint = function(self, left, right)
            table.insert(left, LOC"UI.CONTROLS.CURSOR_MOVE")
        end
    },
    {
        test = function(self) 
            return not TheGame:GetLanguageInfo().hide_cursor_hint
        end,       
        hint = function(self, left, right)
            table.insert(left, LOC"UI.CONTROLS.CURSOR_SPEEDUP")
        end
    },
    {
        hint = function(self, left, right)
            table.insert(right, loc.format( LOC"UI.CONTROLS.OPEN_OVERLAY_MENU", Controls.Digital.OPEN_OVERLAY_MENU ))
        end
    }
}

local PLAYER_WIDGET_Y_OFFSET = 35

function TravelScreen:init(must_leave)
    TravelScreen._base.init(self)
    self:SetAnchors("center", "center")

    --make sure this is in the cache
    GetAnim("anim/map_new_location_fx.zip")

    self.zoom = 1
    self.look_x, self.look_y = 0, 0
    self.hover_check = true

    self.world_root = self:AddChild(Widget())
    self.plax = self.world_root:AddChild( Widget.Plax():SetName("TRAVELSCREEN_PLAX"):PostProcess( true ) )
    self.plax:SetVigOn(false)

    self.must_leave = must_leave
    self.location_popup_mode = false
    
    self.banner = self:AddChild(Widget.Label("title", 80):SetGlyphColour(0xff0000ff))
    AUDIO:PlayEvent(SoundEvents.map_zoom_out_large)

    self.player_widget = self:AddChild(Widget.PlayerMarker())
    self.travel_widget_root = self:AddChild(Widget())
    

    self.travel_widgets = {}

    self.taken_slots = {}   
    self.location_roots = {}

    -- Get the screen size
    self.native_screen_w, self.native_screen_h = TheGame:FE():GetScreenDims()
    self.main_overlay = TheGame:FE():FindScreen( Screen.MainOverlay )

    local regiondata = TheGame:GetGameState():GetWorldRegion():GetContent()
    if regiondata.plax then
        self.plax:SetPlaxData( regiondata.plax )
        local path = self.plax:GetPathElement()
        self.nav_graph = path and Graph(path.nodes, path.edges, path.x, path.y)
    end

    TheGame:GetEvents():ListenForEvents( self, "phase_change", "quests_changed", "quest_destroyed", "caravan_member_event")
    --self:Refresh()
    self.location_to_quests = {}
    self.traveling_fast = GetIsFastMode()
end

function TravelScreen:DoRender()
    self.plax:RenderToScene()
end


function TravelScreen:GetReticulePosition()
    return self.player_widget:TransformToWorld(0,0)
end

function TravelScreen:FindMapLocation(location)

    local exit = location.exits and location.exits[1]
    local parent_location = exit and exit:GetTo()
    
    local placed = false
    for k,elem in ipairs(self.plax.location_slots) do
        if elem.location_name == location.content_id or (parent_location and elem.location_name and elem.location_name == parent_location.content_id) then
            
            self.taken_slots[elem] = location
            location:SetMapPos(elem.x, elem.y)
            placed = true
            break
        end
    end
    
    local eps = 1
    local x, y = location:GetMapPos()
    local map_tags = location:GetContent().map_tags    

    if not placed and x and y then
        local slot_taken_over = false
        for k,elem in ipairs(self.plax.location_slots) do
            local same_spot = math.abs(x - elem.x) < eps and math.abs(y - elem.y) < eps
            if not self.taken_slots[elem] and same_spot then
                if not map_tags or all_tags(elem.location_tag_list, table.unpack(map_tags)) then
                    self.taken_slots[elem] = location
                    break
                end
            end

            if same_spot then
                slot_taken_over = true
            end
        end

        if slot_taken_over then
            x, y = nil, nil
        end


        -- If this location has a map position that cannot be placed, where did it come from? (plax element deleted?)
        -- assert_warning( table.find( self.taken_slots, location ), tostring(location))
   end


    if not (x and y) then

        local candidates = {}
        for k,elem in ipairs(self.plax.location_slots) do
            if not self.taken_slots[elem] and (not elem.location_name or elem.location_name == "") then
                if not map_tags or all_tags(elem.location_tag_list, table.unpack(map_tags)) then
                    table.insert(candidates, elem)
                end
            end
        end
        
        if #candidates == 0 then
            print ("NO MAP CANDIDATES FOR " .. tostring(location))
            local maxw, maxh = self:GetFE():GetScreenDims()
            x, y = 0.1 * maxw + math.random( math.floor( maxw * 0.8 ) ), 0.1 * maxh + math.random( math.floor( maxh * 0.8 ))

        else
            table.shuffle(candidates)
            local best_score, best_candidate
            for _, candidate in ipairs(candidates) do
                local score = 0

                for _, travel_widget in pairs(self.travel_widgets) do
                    local x, y = travel_widget:GetWorldPos()
                    local dsq = (x - candidate.x)*(x - candidate.x) + (y - candidate.y)*(y - candidate.y)
                    local max_dsq = 400*400
                    if dsq < max_dsq then
                        score = score - (max_dsq - dsq)
                    end
                end
                if best_score == nil or best_score <= score then
                    best_score = score
                    best_candidate = candidate
                end
            end

            if best_candidate then
                
                x,y = best_candidate.x, best_candidate.y
                self.taken_slots[best_candidate] = location
                
                location:SetMapPos(x, y)
            end
        end
    end

    -- Create a root widget for this location.
    if not self.location_roots[location] then
        self.location_roots[location] = self.travel_widget_root:AddChild( Widget() )
        self.location_roots[location]:SetPos(x or 0, y or 0)
        self.location_roots[location]:DisableToolTip( self.location_popup_mode )
        self.location_roots[location]:SetShown( self:GetFE():IsHUDVisible() )
    end

    return self.location_roots[ location ]
end


function TravelScreen:AddTravelWidget(location, quest, objectives)
    self.location_to_quests[location] = self.location_to_quests[location] or {}
    table.insert( self.location_to_quests[location], quest )

    local root = self:FindMapLocation(location)

    local w = Widget.TravelWidget(location, quest, objectives,
        function( locwidget ) self:OnPickDestination( locwidget ) end)
        :SetOnHoverFn( function( has_hover ) self:OnTravelWidgetHover( has_hover, root ) end )

    local locwidget = self.location_roots[location]:AddChild( w )
    locwidget:ShowToolTipOnFocus(true)
    locwidget:DisableToolTip( self.location_popup_mode )
    table.insert(self.travel_widgets, locwidget)
    return locwidget
end

function TravelScreen:OnOpen()
    TravelScreen._base.OnOpen( self )
    
    collectgarbage()
    self:Refresh()
end

function TravelScreen:OnLoseTop()
    TravelScreen._base.OnLoseTop( self )

    if self.main_overlay then
        self.main_overlay:FadeInTopLeft( 0.1 ) 
        -- self.main_overlay:FadeInTopRight( 0.1 )
    end
end

function TravelScreen:ToggleHUD( visible, hud_flags )
    for location, w in pairs( self.location_roots ) do
        w:SetShown( visible )
    end
    self.player_widget:SetShown( visible )
end

function TravelScreen:OnControlModeChange( cm, device_id )
    TravelScreen._base.OnControlModeChange( self, cm, device_id )

    self:UpdatePopupMode()
end

function TravelScreen:OnScreenModeChange( sm )
    TravelScreen._base.OnScreenModeChange( self, sm )

    -- Scale the markers
    for k,v in pairs(self.location_roots) do
        v:SetLayoutScale( LAYOUT_SCALE[sm] )
    end
    self.player_widget:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:UpdatePopupMode()

end

function TravelScreen:UpdatePopupMode()
    if TheGame:FE():GetControlMode() == CONTROL_MODE.TOUCH
        or TheGame:FE():GetControlMode() == CONTROL_MODE.GAMEPAD then
        self.location_popup_mode = true -- If true, opening a location triggers a popup instead of tooltips
    else
        self.location_popup_mode = false
    end

    -- Update tooltips
    for k,v in pairs(self.travel_widgets) do
        v:DisableToolTip( self.location_popup_mode )
    end 

    return self
end

function TravelScreen:OnScreenResize(w, h)
    TravelScreen._base.OnScreenResize(self, w, h)

    self.native_screen_w, self.native_screen_h = TheGame:FE():GetScreenDims()
end

function TravelScreen:OnTravelWidgetHover( has_hover, travel_widget )
    if self.main_overlay then
        local w_x, w_y = travel_widget:GetWorldPos()
        if has_hover then
            -- Is this widget on the top left quadrant?
            if w_y > self.native_screen_h*0.6 and w_x < self.native_screen_w*0.5 then
                -- self.main_overlay:FadeOutTopLeft( 0.3 ) 
            end
            -- Is this widget on the top right quadrant?
            if w_y > self.native_screen_h*0.6 and w_x > self.native_screen_w*0.5 then
                -- self.main_overlay:FadeOutTopRight( 0.3 )
            end
        else
            -- self.main_overlay:FadeInTopLeft( 0.1 ) 
            -- self.main_overlay:FadeInTopRight( 0.1 )
        end
    end
end

local function HasQuestMarked( marks, location, quest )
    for k,v in pairs(marks) do
        if v.location == location and v.quest == quest then
            return true
        end
    end
    return false
end

function TravelScreen:Refresh()

    self.plax:SetDayPhase( TheGame:GetGameState():GetDayPhase())
    TheGame:GetGameState():PutImportantAgentsInWorld()
    TheGame:GetGameState():GetTracker():UpdateAllQuests()

    for k,v in pairs(self.travel_widgets) do
        v:Remove()
    end
    table.clear(self.travel_widgets)

    local marks = {}
    for k,quest in TheGame:GetGameState():ActiveQuests() do
        local all_marks = {}
        quest:CollectMarkedCast( all_marks, false )
        for obj, objectives in pairs(all_marks) do
            if is_instance(obj, Location) then
                if not HasQuestMarked(marks, obj, quest) then
                    table.insert(marks, {location = obj, quest = quest, objectives = objectives})
                end
            elseif is_instance(obj, Agent) then
                if obj:GetLocation() then
                    if not HasQuestMarked(marks, obj:GetLocation(), quest) then
                        table.insert(marks, {location = obj:GetLocation(), agent = obj, quest = quest, objectives = objectives})
                    end
                end
            end
        end
    end

    for k,v in pairs(marks) do
        v.x, v.y = v.location:GetMapPos()
    end

    for k,v in pairs(self.location_roots) do
        v:Remove()
    end
    table.clear(self.location_roots)

    self:StartCoroutine(TravelScreen.ShowNewMarksCoro, self, marks)
end


function TravelScreen:ShowNewMarksCoro(marks)
    self.player_widget:Hide()

    -- Compile a map of [Location] -> { array of marks }
    local marks_by_location = {}
    for k,v in pairs(marks) do
        marks_by_location[v.location] = marks_by_location[v.location] or {}
        table.insert(marks_by_location[v.location], v)
    end

    --we want to place previously fixed locations first, because we need to route the new ones around them
    local placement_order = {}
    for location,mark_list in pairs(marks_by_location) do
        for k,mark in ipairs(mark_list) do
            table.insert(placement_order, mark)
        end
    end

    table.sort(placement_order, function(a,b) 
        local a_has_pos = a.location:GetMapPos() ~= nil 
        local b_has_pos = b.location:GetMapPos() ~= nil 
        if a_has_pos ~= b_has_pos then
            return a_has_pos
        end
        return a.location:GetID() < b.location:GetID()
    end)

    -- Add all travel markers at each location.
    for _, mark in ipairs(placement_order) do
        local w = self:AddTravelWidget(mark.location, mark.quest, mark.objectives)
        w:LayoutBounds("after", "center"):Offset(4, 0)
        --print( "TravelScreen: Creating marker for", mark.location, mark.quest )
    end

    for location,mark_list in pairs(marks_by_location) do
        -- Centre root on top of the location coordinates.
       self.location_roots[location]:LayoutBounds("center", "bottom", location:GetMapPos())
    end

    table.sort(self.travel_widgets, function(a,b) 
        local a_opp = a.quest:GetType() == QTYPE.OPPORTUNITY 
        local b_opp = b.quest:GetType() == QTYPE.OPPORTUNITY 
        if a_opp == b_opp then
            local ax, ay = a:GetWorldPos()
            local bx, by = b:GetWorldPos()
            if a_opp then
                return by < by
            else
                return ay < by
            end
        else
            return a_opp 
        end

    end)

    -- Initially hide all travel widgets so they can animate in.
    for k,v in pairs(self.travel_widgets) do
        v:Hide()
    end
    
    self.travel_widget_root:ArrangeChildrenByY()

    for k,v in ipairs(self.travel_widgets) do
        v:Appear()
        self:Delay(.2)
    end

    while true do
        local animating = false
        for k,v in ipairs(self.travel_widgets) do
            if v:IsAppearing() then
                animating = true
                break
            end
        end
        if not animating then
            break
        else
            coroutine.yield()
        end
    end

    self:Delay( 0.3 )

    for k,v in ipairs(self.travel_widgets) do
        v:Enable()
    end
    
    self:RefreshRobotOptions()

    -- Find a location for the player's current location.
    local player_location = TheGame:GetGameState():GetPlayerAgent():GetLocation():GetRootLocation()
    local root = self:FindMapLocation( player_location )
    self.player_widget:SetShown( self:GetFE():IsHUDVisible() ):LayoutBounds( "before", "center", root ):Appear()

    self.fe:SetReticulePos( self:GetReticulePosition() )
end

function TravelScreen:OnGainTop()
    self.plax:PlaySounds()

    -- Necessary to check mouse positions continuously
    self:StartUpdating()

    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
    self:UpdatePopupMode()
end

function TravelScreen:OnLoseTop()
    TravelScreen._base.OnLoseTop( self )

    self:StopUpdating()
end

function TravelScreen:GetMusicEvent()
    return self.plax:GetMusicEvent()
end

function TravelScreen:OnUpdate(dt)
    TravelScreen._base.OnUpdate(self, dt)

    -- Check if the mouse is close to one of the top quadrants of the screen, and hide the main overlays there
    if self.main_overlay and TheGame:FE():GetControlMode() == CONTROL_MODE.MOUSE_KEYBOARD then
        local sx, sy = TheGame:GetInput():GetMousePos()
        -- Is the mouse on the top left quadrant?
        if sy > self.native_screen_h*0.6 and sx < self.native_screen_w*0.5 then
            if not self.top_left_overlay_hidden then
                self.top_left_overlay_hidden = true
                -- self.main_overlay:FadeOutTopLeft( 0.3 )
            end
        elseif self.top_left_overlay_hidden then
            self.top_left_overlay_hidden = false
            -- self.main_overlay:FadeInTopLeft( 0.1 ) 
        end
        -- Is the mouse on the top right quadrant?
        if sy > self.native_screen_h*0.6 and sx > self.native_screen_w*0.5 then
            if not self.top_right_overlay_hidden then
                self.top_right_overlay_hidden = true
                -- self.main_overlay:FadeOutTopRight( 0.3 )
            end
        elseif self.top_right_overlay_hidden then
                self.top_right_overlay_hidden = false
                -- self.main_overlay:FadeInTopRight( 0.1 )
        end
    end
end

function TravelScreen:RefreshRobotOptions()
    if not TheGame:GetLocalSettings().ROBOTICS then
        return
    end

    self:ClearRobotOptions()

    local oppos = false
    for k,locwidget in ipairs(self.travel_widgets) do
        if locwidget.quest and locwidget.quest:GetType() == QTYPE.OPPORTUNITY then    
            oppos = true
        end
    end

    for k,locwidget in ipairs(self.travel_widgets) do
        if not oppos or (locwidget.quest and locwidget.quest:GetType() == QTYPE.OPPORTUNITY) then
            self:AddRobotOption(locwidget)
        end
    end
end

function TravelScreen:GetDefaultFocus()
    for k, opt in ipairs( self.travel_widgets ) do
        if opt:IsShown() then
            return opt
        end
    end
    return self.player_widget
end


function TravelScreen:Close()
    self.fe:FadeTransition(function() 
        local screen = self.fe:FindScreen( Screen.LocationScreen )
        if screen then
            screen:UpdateDayPhase()
            screen:Refresh(nil, true)
        end
        self.fe:PopScreen(self)
    end, .3)
end

function TravelScreen:_DoEnterLocation(dest_location)
    AUDIO:PlayEvent(SoundEvents.map_zoom_in_large)

    TheGame:GetGameState():GetCaravan():MoveToLocation( dest_location )

    if dest_location:GetContent().entry_scenario then            
        QuestUtil.SpawnQuest(dest_location:GetContent().entry_scenario, {cast = {location=dest_location}})
    end

    self:Close()
end

function TravelScreen:HandleControlDown(control, device)

    -- If we're in touch mode, tapping the map will toggle the quests overlay on and off
    -- When we toggle the quests off, we show the location names instead
    if TheGame:FE():GetControlMode() == CONTROL_MODE.TOUCH then
        if not self.hiding_todo_list then
            self.hiding_todo_list = true
            -- Hide overlays
            if self.main_overlay then
                -- self.main_overlay:FadeOutTopLeft( 0.3 ) 
                -- self.main_overlay:FadeOutTopRight( 0.3 )
            end
            -- Show location names
            for k,v in pairs(self.travel_widgets) do
                v:ShowName( true )
            end 
        else
            self.hiding_todo_list = false
            -- Show overlays
            if self.main_overlay then
                -- self.main_overlay:FadeInTopLeft( 0.1 ) 
                -- self.main_overlay:FadeInTopRight( 0.1 )
            end
            -- Hide location names
            for k,v in pairs(self.travel_widgets) do
                v:ShowName( false )
            end 
        end
    end

    return TravelScreen._base.HandleControlDown( self, control, device )
end

function TravelScreen:OnPickDestination( locwidget )
    if self.location_popup_mode then
        -- Show the location popup
        local location_popup = Screen.LocationInfoPopup( locwidget:GetLocation() )
            :SetTravelFn( function() self.travel_coro = self:StartCoroutine(TravelScreen.TravelCoro, self, locwidget) end )
        TheGame:FE():InsertScreen( location_popup )
    else
        self.travel_coro = self:StartCoroutine(TravelScreen.TravelCoro, self, locwidget)
    end
end

function TravelScreen:OnGlobalEvent( event_name, ... )
    
    if event_name == "phase_change" then
        self:Refresh()
        -- self.plax:SetDayPhase( TheGame:GetGameState():GetDayPhase())

    elseif event_name == "caravan_member_event" then
        event_name = select( 1, ... )
        if event_name == "agent_location_changed" then
            if not self:IsTravelling() then
                self:Close()
            end
        end
    end
end

function TravelScreen:_GetLocationTravelCoordinates(location)

    local x, y = location:GetMapPos()
    if x and y then
        return x, y
    end

    if location.parent_location then
        x, y = location.parent_location:GetMapPos()
    end
    if x and y then
        return x, y
    end
    return 0, 0
end


function TravelScreen:IsTravelling()
    return self.traveling
end


function TravelScreen:TravelCoro(locwidget) 

    local dest_location = locwidget.location
    local source_location = TheGame:GetGameState():GetPlayerAgent():GetLocation()
    print (loc.format( "TRAVELING: {1} -> {2}", source_location, dest_location))
    
    TheGame:GetGameState():GetCaravan().last_travel_destination = dest_location
    
    TheGame:BroadcastEvent( "player_traveled" )

    if source_location == dest_location then
        
        local screen = self.fe:FindScreen( Screen.LocationScreen )
        if screen then
            screen.plax:PlayEnterSound()
        end

        self:Close()
        return
    end
    self.traveling = true
    self.travel_widget_root:Disable()


    local oppo_to_keep = nil
    local keep_oppos = {}
    if locwidget.quest and locwidget.quest:GetType() == QTYPE.OPPORTUNITY then
        table.insert(keep_oppos, locwidget.quest)
    end

    if self.location_to_quests and self.location_to_quests[locwidget.location] then
        for k,q in ipairs(self.location_to_quests[locwidget.location]) do
            if q ~= locwidget.quest and q:GetType() == QTYPE.OPPORTUNITY then
                table.insert(keep_oppos, q)
            end
        end
    end
    
    local cleared_quests = TheGame:GetGameState():ClearOpportunities(keep_oppos)
    if cleared_quests and #cleared_quests > 0 then
        local markers_to_clear = {}
        for _,quest in ipairs(cleared_quests) do
            for _,tw in pairs(self.travel_widgets) do
                if tw.quest == quest then
                    table.insert(markers_to_clear, tw)
                end
            end
        end

        for k,v in ipairs(markers_to_clear) do
            v:Explode()
            self:Delay(.3)
        end
        
    end


    local act = TheGame:GetGameState():GetCurrentAct()


    AUDIO:PlayEvent(act.data.travel_sound or "event:/ui/overworld/fast_travel")

    self.player_widget:Show()   
    
    local player_location = TheGame:GetGameState():GetPlayerAgent():GetLocation()
    
    if player_location and self.travel_widgets[player_location] then
        self.travel_widgets[player_location]:HidePlayer()
    end

    local x1, y1 = self.player_widget:GetPos()
    local x2, y2  = self:_GetLocationTravelCoordinates(dest_location)
    
    local delay = 0
    
    for location,widget in pairs(self.travel_widgets) do
        if widget ~= locwidget then
            widget:Disappear(delay)
            delay = delay + .1
        end
    end


    local distsq = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)

    if distsq >= .01 then
        local path
        
        if self.nav_graph then
            path = self.nav_graph:FindPath(x1,y1,x2,y2)
            if path then
                table.insert(path, 1, {x=x1, y=y1})
                table.insert(path, {x=x2, y=y2})
            end
        end

        if not path then
            path = { {x=x1, y=y1}, {x=x2, y=y2} }
        end

        local total_distance = 0
        for k in ipairs(path) do
            if path[k+1] then
                local x1, y1 = path[k].x, path[k].y
                local x2, y2 = path[k+1].x, path[k+1].y
                local distsq = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)
                local dist = 0 
                if distsq > 0 then
                    dist = math.sqrt(distsq)
                    total_distance = total_distance + dist 
                end
            end
        end
        
        --allow you to be interrupted twice, because some quests rely upon that. We throttle interrupts enough that it shouldn't be neccessary to throttle them again here.
        local can_spawn_new_event = not ( source_location and source_location:HasTag("in_transit") )
        
        --figure out distances so we can smoothly interpolate along the path
        local confront_dist = total_distance > 200 and math.randomGauss(.2, .8)*total_distance
        local confront_tags = nil


        if confront_dist then
            for k in ipairs(path) do
                if path[k+1] then
                    --if there are tags at the given confront dist, remember them
                    if  (total_distance > confront_dist) and confront_tags == nil then
                        
                        if path[k].edges then
                            for edge_idx, edge in pairs(path[k].edges) do
                                if edge.node1 == path[k+1] or edge.node2 == path[k+1] then
                                    if edge.userdata and edge.userdata.tag_list and edge.userdata.tag_list ~= "" then
                                        confront_tags = edge.userdata.tag_list:split(",")
                                    end
                                end
                            end
                        end

                        if not confront_tags then
                            confront_tags = {"road"}
                        end

                    end
                end
            end
        end

        local smooth_path, total_distance = MakeSmoothPath(path)

        local confront_hub, confront_hub_state 
        local confront_x, confront_y
        if confront_dist then
            confront_x, confront_y = TravelAlongPath(smooth_path, confront_dist, 0)
            confront_hub, confront_hub_state = self:TryConfronts(can_spawn_new_event, confront_tags, confront_x, confront_y)
        end

        -- if confront_hub then
        --     --play a sound! play an anim!
        --     if not self.confront_icon then 
        --         self.confront_icon = self.travel_widget_root:AddChild( Widget.EventTravelWidget() ) 
        --     end

        --     self.confront_icon:SetPos(confront_x, confront_y + CONFRONT_ICON_OFFSET)
        --     self.confront_icon:Appear()
        -- end

        local max_speed = 800 -- in normalized res pixels / second, obviously
        local min_speed = 10

        local speed = max_speed
        local dist_traveled = 0

        while dist_traveled < total_distance do
            local p = dist_traveled / total_distance--(confront_hub and confront_dist or total_distance)

            if p < .5 then
                speed = easing.outExpo(2*p, min_speed, max_speed-min_speed, 1)
            else
                speed = easing.inExpo(2*(p-.5), max_speed, -max_speed, 1)
            end

            if self.traveling_fast then
                speed = speed *4
            end

            local dt = coroutine.yield()
            local dd = dt*speed
            local x, y = TravelAlongPath(smooth_path, dd, dist_traveled)

            dist_traveled = dist_traveled + dd
            self.player_widget:SetPos( x, y + (math.min(dist_traveled / 100.0, 1.0) * PLAYER_WIDGET_Y_OFFSET) )

            if confront_hub and dist_traveled >= confront_dist * .55 and not self.confront_icon then
                --play a sound! play an anim!
                self.confront_icon = self.travel_widget_root:AddChild( Widget.EventTravelWidget() ) 

                self.confront_icon:SetPos(confront_x, confront_y + CONFRONT_ICON_OFFSET)
                self.confront_icon:Appear()
            end
                
            if confront_hub and dist_traveled >= confront_dist then
                self:DoTravelConfrontCoro(confront_hub, confront_hub_state)
                confront_hub = nil
                return
            end

        end

    end
    self.traveling_fast = GetIsFastMode()
    self.traveling = false
    self:_DoEnterLocation(dest_location)    

    DoAutoSave()
end


function TravelScreen:TryConfronts(can_spawn_new_event, confront_tags, x, y)
    -- print ("TRYCONFRONT", can_spawn_new_event)

    self:MoveToNodeLocation(confront_tags, x, y)
    
    --we run general confronts first in case we have an existing quest that's interested in the fact that we are traveling
    local confront_hub, confront_hub_state = TheGame:GetGameState():GetCaravan():EvaluateConfronts()

    if not confront_hub then
        local event = can_spawn_new_event and TheGame:GetGameState():TrySpawnNewEvent(QEVENT_TRIGGER.TRAVEL)
        if event then
            confront_hub, confront_hub_state = TheGame:GetGameState():GetCaravan():EvaluateConfronts()
            if confront_hub then
                DoAutoSave()
            end

        end
    end

    return confront_hub, confront_hub_state
end



--this moves us to a temporary location as we travel, so that travel events and confronts can evaluate
function TravelScreen:MoveToNodeLocation(tags, x, y)
    local location_id = TheGame:GetGameState():GetWorldRegion():GetOutdoorLocationID(tags)
    local location = Location( location_id )
    location:SetMapPos(x,y)
    location:AddTag("in_transit")
    location:SetFlags( LOCFLAGS.TRANSIENT )
    TheGame:GetGameState():AddLocation( location )
    TheGame:GetGameState():GetCaravan():MoveToLocation(location)
end


function TravelScreen:DoTravelConfrontCoro(hub, hub_state)
    local negative = hub.quest and hub.quest:GetQuestDef() and hub.quest:GetQuestDef().is_negative
    local stinger = TheGame:LookupPlayerMusic( negative and "ambush_bad" or "ambush_neutral" )
    AUDIO:PlayEvent( stinger )
    self:Delay(.5)
    self:Close()
end


function TravelScreen:WantsReticuleMode()

    return not self:IsTravelling() and self.player_widget and self.player_widget.shown
end


function TravelScreen:TestReticule(x, y)
    if self.main_overlay then
        local mouse_blocked, hover_widget = self.main_overlay:CheckMouseHover(x, y)
        if hover_widget then
            return hover_widget
        end
    end

    local closest, closest_dist

    for k,v in pairs( self.travel_widgets ) do
        local wx, wy = v:TransformToWorld(0,0)
        local distsq = (wx-x)*(wx-x) + (wy-y)*(wy-y)
        if closest_dist == nil or closest_dist >= distsq then
            closest, closest_dist = v, distsq
        end
    end

    if closest_dist <= 100*100 then
        return closest
    end

end

function TravelScreen:OnRemoved()
    self.plax = nil
end


------------------------------------------------------------------------------------------
-- Displays a map location popup

local LocationInfoPopup = class( "Screen.LocationInfoPopup", Widget.Screen )

LocationInfoPopup.CONTROL_MAP =
{
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self)
            self:Close()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.BACK", Controls.Digital.MENU_CANCEL) )
        end,
    },
    {
        control = Controls.Digital.MENU_ACCEPT,
        fn = function(self) 
            if TheGame:FE():IsGamepadMode() then
                self:Travel()
                return true
            end
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.TRAVEL", Controls.Digital.MENU_ACCEPT) )
        end,
    },
    {
        test = function(self)
            return TheGame:FE():IsGamepadMode() and self.scroll:HasScrolling()
        end,
        hint = function(self, left, right) 
            table.insert( left, loc.format( LOC"UI.CONTROLS.SCROLL" ))
        end,
    }
}

local MAX_PANEL_HEIGHT = 780

function LocationInfoPopup:init( location )
    LocationInfoPopup._base.init( self )

    self.is_popup = true
    -- Set custom underlay
    self.clickable_bg = self:AddChild( Widget.Clickable() )
        :SetOnClickFn( function() self:Close() end )
    self.underlay = self.clickable_bg:AddChild( Widget.Image( engine.asset.Texture( "splash/underlay.tex" )), 1 )
        :SetAnchors( "fill", "fill" )
        :SetTintAlpha( 0 )
        :AlphaTo( 0.95, 0.3, easing.outQuad )
    self:SetAnchors( "center", "center" )
    
    self.height = 540
    self.contents_w = 1050

    -- Background
    self.bg = self:AddChild( Widget.WidePanel( self.height ) )
        :BlockMouse()
    
    -- Contents
    self.scroll = self:AddChild( Widget.ScrollPanel() )
        :SetVirtualMargin( 0 )
        :SetVirtualTopMargin( SPACING.M1 )
        :SetVirtualBottomMargin( SPACING.M1 )
        :SetScrollBarOuterMargin( 0 )
        :ShowBar( SCROLLBAR.NEVER )
        :SetSize( self.contents_w+SPACING.M1*4, self.height )
    self.contents = self.scroll:AddScrollChild( Widget() )
    self.hitbox = self.contents:AddChild( Widget.SolidBox( self.contents_w, 1, 0xff00ff00 ) )

    -- Texts
    self.titles = self.contents:AddChild( Widget() )
    self.home_icon = self.titles:AddChild( Widget.HomeQuestIcon() )
        :Hide()
    self.title = self.titles:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_TITLE, "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetHoverLinkGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize( self.contents_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom( 0.1 )
    self.subtitle = self.titles:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL, "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetHoverLinkGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize( self.contents_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetTintAlpha( 0.8 )
        :Bloom( 0.05 )
    self.desc = self.titles:AddChild( Widget.Label( "tooltip", FONT_SIZE.BODY_TEXT, "" ) )
        :SetGlyphColour( UICOLOURS.QUEST )
        :SetHoverLinkGlyphColour( UICOLOURS.QUEST )
        :SetAutoSize( self.contents_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetTintAlpha( 0.8 )
        :Bloom( 0.1 )
        :Hide()
    self.player_icon = self.titles:AddChild( Widget() )
        :Hide()
    self.player_portrait = self.player_icon:AddChild( Widget.TrackedAgentPortrait( TheGame:GetGameState():GetPlayerAgent() ) )
        :SetHeight( 30 )
    self.player_label = self.player_icon:AddChild( Widget.Label( "body", FONT_SIZE.ITEM_LABEL, string.upper( LOC"UI.TRAVEL_SCREEN.PLAYER_PORTRAIT_TOOLTIP" ) ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :LayoutBounds( "after", "center", self.player_portrait )
        :Offset( 6, -4 )
        :Bloom( 0.1 )

    -- Quest tiles
    self.tiles = self.contents:AddChild( Widget() )
        :Hide()

    -- Buttons
    self.close_button = self.bg:AddSubButton( LOC"UI.DIALOGS.BACK", global_images.close, function() self:Close() end )
    self.travel_button = self.bg:AddSubButton( LOC"UI.TRAVEL_SCREEN.SELECT_LOCATION", global_images.accept, function() self:Travel() end )

    self:Refresh( location )
end

function LocationInfoPopup:Refresh( location )

    self.location = location

    self.contents:SetTintAlpha(0)

    -- Update location's name
    self.subtitle:SetText( LOC"UI.TRAVEL_SCREEN.LOCATION_SUBTITLE" )
        :LayoutBounds( "left", nil, self.hitbox )
    self.title:SetText( self.location:GetName() )
        :LayoutBounds( "left", "below", self.subtitle )
        :Offset( 0, 8 )

    -- Go through the quests and find the ones for this location
    
    local marks = {}
    for k, quest in TheGame:GetGameState():ActiveQuests() do
        local all_marks = {}
        quest:CollectMarkedCast( all_marks, false )
        for obj, objectives in pairs(all_marks) do
            if is_instance(obj, Location) and obj == self.location then
                if not HasQuestMarked(marks, obj, quest) then
                    table.insert(marks, {location = obj, quest = quest, objectives = objectives})
                end
            elseif is_instance(obj, Agent) then
                if obj:GetLocation() and obj:GetLocation() == self.location then
                    if not HasQuestMarked(marks, obj:GetLocation(), quest) then
                        table.insert(marks, {location = obj:GetLocation(), agent = obj, quest = quest, objectives = objectives})
                    end
                end
            end
        end
    end

    -- Go through the quests and display them accordingly
    for k, mark in ipairs( marks ) do
        -- Check if it's a home quest
        if mark.quest:GetQuestDef().home then
            self.home_icon:Show()
                :Refresh( mark )
                :LayoutBounds( "left", "top", self.subtitle )
                :Offset( 0, -4 )
            self.subtitle:SetText( LOC"UI.TRAVEL_SCREEN.LOCATION_SUBTITLE_HOME" )
                :SetGlyphColour( UICOLOURS.QUEST )
                :SetTintAlpha( 0.8 )
                :LayoutBounds( "after", nil, self.home_icon )
                :Offset( SPACING.M1 )
            self.title:SetGlyphColour( UICOLOURS.QUEST )
                :LayoutBounds( "after", nil, self.home_icon )
                :Offset( SPACING.M1 )
            self.desc:Show()
                :SetText( self.home_icon:GetDesc() )
                :LayoutBounds( "left", "below", self.title )
                :Offset( 0, -2 )
            self.bg:SetColour( UICOLOURS.QUEST )
        else
            self.tiles:Show()
            self.tiles:AddChild( Widget.QuestTile( self.contents_w ) )
                :Refresh( mark, self.location )
        end
    end

    -- Check if the player is at this location
    if TheGame:GetGameState():GetPlayerAgent():GetLocation() == self.location then
        self.player_icon:Show()
            :LayoutBounds( "after", "center", self.subtitle )
            :Offset( 8, 2 )
    end

    -- Show quests
    self.tiles:StackChildren( SPACING.M1/2 )
        :LayoutBounds( "left", "below", self.titles )
        :Offset( 0, -SPACING.M1 )

    return self
end

function LocationInfoPopup:SetTravelFn( fn )
    self.travel_fn = fn
    return self
end

function LocationInfoPopup:Close()
    self:AlphaTo( 0, 0.1, easing.outQuad, function() self:GetFE():PopScreen(self) end )
    return self
end

function LocationInfoPopup:Travel()
    self:GetFE():PopScreen(self)
    self.travel_fn()
    return self
end

function LocationInfoPopup:OnGainTop()
    LocationInfoPopup._base.OnGainTop( self )

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )

    if TheGame:GetLocalSettings().ROBOTICS then
        self:SetRobotTimer( 1.0 )
        self:AddRobotOption(self.travel_button)
    end

    self.bg:FadeIn( function()
        self.contents:AlphaTo( 1, 0.1, easing.outQuad )
    end )

    -- To keep track of gamepad scroll
    self:StartUpdating()
end

function LocationInfoPopup:OnLoseTop()
    LocationInfoPopup._base.OnLoseTop( self )

    -- Stop keeping track of gamepad scroll
    self:StopUpdating()
end

function LocationInfoPopup:OnScreenModeChange( sm )
    LocationInfoPopup._base.OnScreenModeChange( self, sm )

    self.contents:SetLayoutScale( LAYOUT_SCALE[sm] )

    -- Resize the panel if there's little content
    local c_w, c_h = self.contents:GetScaledSize()
    c_h = c_h + SPACING.M1*2
    c_h = math.min( c_h, MAX_PANEL_HEIGHT )
    self.scroll:SetSize( (self.contents_w+SPACING.M1*4)*LAYOUT_SCALE[sm], c_h )
    self.bg:SetHeight( c_h )
    self.contents:LayoutBounds( "left", "top", 0, 0 )
    self.scroll:RefreshView()

end

------------------------------------------------------------------------------------------
-- Displays a home-quest icon on the popup

local HomeQuestIcon = class( "Widget.HomeQuestIcon", Widget )

function HomeQuestIcon:init()
    HomeQuestIcon._base.init(self)

    self.home_icon = self:AddChild( Widget() )
    self.quest_arrow = self.home_icon:AddChild( Widget.Image( engine.asset.Texture("UI/location_widget_pointer.tex") ) )
        :SetSize( QUEST_ICONS_SIZE*.88, QUEST_ICONS_SIZE*.88 )
        :SetPos( 0, -QUEST_ICONS_SIZE*.25 )
        :SetTintColour( UICOLOURS.QUEST )
    self.quest_icon = self.home_icon:AddChild( Widget.Image() )
        :SetSize( QUEST_ICONS_SIZE, QUEST_ICONS_SIZE )
end

function HomeQuestIcon:Refresh( mark )
    self.mark = mark
    self.quest = mark.quest
    self.objectives = mark.objectives

    self.quest_icon:SetTexture( mark.quest:GetIcon() )

    local tt = {}
    local show_obj = false
    for i, obj in ipairs(self.quest.objectives) do
        if self.quest:IsActive(obj.id) and table.contains( self.objectives, obj.id ) then
            local title, desc = self.quest:GetObjectiveTitle(obj.id), self.quest:GetObjectiveDesc(obj.id)
            if title and desc then
                show_obj = true
                table.insert(tt, desc)
                self.quest_icon:SetTexture( mark.quest:GetIcon( obj ) )
            end
        end
    end

    if not show_obj then
        local desc = self.quest:GetDesc()
        if desc then
            table.insert(tt, desc)
        end
    end
    self.desc = table.concat(tt, "\n")

    return self
end

function HomeQuestIcon:GetDesc()
    return self.desc or ""
end


------------------------------------------------------------------------------------------
-- Displays a quest on the popup

local QuestTile = class( "Widget.QuestTile", Widget )

function QuestTile:init( width )
    QuestTile._base.init(self)

    self.width = width or 400
    self.contents_w = self.width - QUEST_ICONS_SIZE - SPACING.M1*3

    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/left_border_frame_faint.tex" ) ) )
        :SetSize( self.width, 60 )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetBloom( 0.2 )
    self.contents = self:AddChild( Widget() )

    self.icon = self.contents:AddChild( Widget.Image() )
        :SetSize( QUEST_ICONS_SIZE, QUEST_ICONS_SIZE )
    self.label = self.contents:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_LABEL ) )
        :SetGlyphColour( UICOLOURS.QUEST )
        :SetHoverLinkGlyphColour( UICOLOURS.QUEST )
        :SetAutoSize( self.contents_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom( 0.05 )
    self.title = self.contents:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_NAME ) )
        :SetWordWrap( true )
        :SetAutoSize( self.contents_w )
        :LeftAlign()
        :OverrideLineHeight( FONT_SIZE.ITEM_NAME*LINE_HEIGHT_TITLE_MULT )
        :SetGlyphColour( UICOLOURS.QUEST_LIGHT )
        :SetHoverLinkGlyphColour( UICOLOURS.QUEST_LIGHT )
        :Bloom( 0.05 )
    self.desc = self.contents:AddChild( Widget.Label( "body", FONT_SIZE.BODY_TEXT ) )
        :SetWordWrap( true )
        :SetAutoSize( self.contents_w )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :SetHoverLinkGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom( 0.1 )
    self.difficulty_widget = self.contents:AddChild( Widget.QuestDifficultyWidget() )
    self.focus_widget = self.contents:AddChild( Widget.QuestFocusWidget() )
    self.rewards = self.contents:AddChild( Widget.RewardsWidget( self.contents_w, 2 ) )

end

function QuestTile:Refresh( mark )
    
    self.mark = mark
    self.quest = mark.quest

    self.icon:SetTexture( self.quest:GetIcon() )
    self.title:SetText( self.quest:GetTitle() )

    if self.quest:IsInactive() then
        -- Available quest
        if self.quest:GetRank() >  TheGame:GetGameState():GetDefaultQuestRank() then
            self.label:SetText( LOC"UI.TRAVEL_SCREEN.QUEST_CHALLENGING" )
                :SetGlyphColour( UICOLOURS.WHITE )
                :SetTintColour( UICOLOURS.PENALTY )
                :Pulse( 0.6, 1.0, 0.003 )
            self.bg:SetTintColour( UICOLOURS.PENALTY )
        else
            self.label:SetText( LOC"UI.JOB_POPUP.TT_TITLE_LABEL" )
        end
        self.desc:SetText( self.quest:GetDesc() )
        self.difficulty_widget:Refresh( self.quest:GetRank() )
        self.focus_widget:Refresh( self.quest:GetRatingString() )
        self.rewards:RefreshRewards( self.quest )
    else

        -- Active quest
        self.label:SetText( LOC"UI.TO_DO_LIST_LABEL" )
        local tt = {}
        local show_obj = false
        for i, obj in ipairs(self.quest.objectives) do
            if self.quest:IsActive(obj.id) then
                local title = self.quest:GetObjectiveTitle( obj.id )
                local desc = self.quest:GetObjectiveDesc( obj.id )
                if title and desc then
                    show_obj = true
                    table.insert(tt, desc)
                end
                local obj_def = self.quest:GetQuestDef():GetObjective( obj.id )
                if obj_def.icon then
                    self.icon:SetTexture( obj_def.icon )
                end
            end
        end

        if not show_obj then
            local desc = self.quest:GetDesc()
            if desc then
                table.insert(tt, desc)
            end
        end
        self.desc:SetText( table.concat(tt, "\n") )
        self.difficulty_widget:Hide()
        self.focus_widget:Hide()
        self.rewards:Hide()
    end

    if self.quest:GetType() == QTYPE.OPPORTUNITY then
        self.label:SetText( LOC"UI.TRAVEL_SCREEN.QUEST_OPPORTUNITY" )
            :SetGlyphColour( UICOLOURS.OPPORTUNITY )
        self.bg:SetTintColour( UICOLOURS.OPPORTUNITY )
    end

    self.label:LayoutBounds( "after", "top", self.icon )
        :Offset( SPACING.M1, 0 )
    self.title:LayoutBounds( "left", "below", self.label )
        :Offset( 0, 8 )
    self.desc:LayoutBounds( "left", "below", self.title )
        :Offset( 0, 0 )
    self.difficulty_widget:LayoutBounds( "left", "below", self.desc )
        :Offset( 0, -SPACING.M1 )
    self.focus_widget:LayoutBounds( "after", "top", self.difficulty_widget )
        :Offset( 30, 0 )
    self.rewards:LayoutBounds( "left", "below", self.difficulty_widget )
        :Offset( 0, -SPACING.M1 )

    local w, h = self.contents:GetSize()
    h = h + SPACING.M1*1.5

    self.bg:SetSize( self.width, h )
    self.contents:LayoutBounds( "left", "center", self.bg )
        :Offset( SPACING.M1 )

    return self
end
