require "ui/widgets/screen"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/menu"
require "ui/widgets/solidbox"
require "ui/widgets/panelbutton"

local ORIGINAL_CARD_SCALE = 0.8
local CARD_SCALE = 0.7
local PANEL_H = RES_Y * 0.55

local UpgradeCardPopup = class("Screen.UpgradeCardPopup", Widget.Screen)

UpgradeCardPopup.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function(self)
            return not self.preview_only and UIHelpers.GetFocusCard()
        end,
        fn = function(self)
            if UIHelpers.GetFocusCard() then
                UIHelpers.GetFocusCard():Click()
                return true
            end
        end,

        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.UPGRADE_CARD", Controls.Digital.MENU_ACCEPT))
        end,
    },

    {
        control = Controls.Digital.MENU_CANCEL,
        
        fn = function(self)
            self:OnClickCancel()             
            return true
        end,
        
        hint = function(self, left, right)
            table.insert(right, loc.format ( LOC"UI.CONTROLS.CANCEL", Controls.Digital.MENU_CANCEL))
        end,
    },
}


function UpgradeCardPopup:init( card, agent, preview_only, fn )
	UpgradeCardPopup._base.init(self)
	self.preview_only = preview_only
    self.is_popup = true
    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )
	
	self.card = card
	self.agent = agent
    self.fn = fn

    -- Start assembling our screen
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- Setup the grid
    self.grid = self:AddChild( Widget.Image( engine.asset.Texture( "UI/grid.tex" ) ) )
    self.grid:SetTintColour( UICOLOURS.SUBTITLE )
    self.grid:SetTintAlpha( 0.8 )
    local target_w = self.screen_w*1.1
    local g_w, g_h = self.grid:GetSize();
    local target_h = target_w/g_w*g_h; -- Scale the grid to match the width, mantaining aspect ratio
    self.grid:SetSize( target_w, target_h )
    self.grid:SetPos( nil, -self.screen_h/2 + target_h/2 )


    -- Central circle
    local circle_size = 550
    self.wheel_container = self:AddChild( Widget() )
    self.wheel_rotating = self.wheel_container:AddChild( Widget.Image( engine.asset.Texture( "UI/arguments_wheel.tex" ) ) )
        :SetSize( circle_size, circle_size )
        :SetTintColour(UICOLOURS.SUBTITLE)
        :SetTintAlpha(0.5)
        :Bloom( 0.2 )
        :RotateIndefinitely( 0.02 )
    self.wheel_pointer = self.wheel_container:AddChild( Widget.Image( engine.asset.Texture( "UI/arguments_wheel.tex" ) ) )
        :SetSize( circle_size, circle_size )
        :SetTintColour(UICOLOURS.SUBTITLE)
        :SetTintAlpha(0.7)
        :Bloom( 0.3 )
    -- Central circle's shadow
    self.wheel_shadow = self:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
        :SetSize( 800, 100 )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :LayoutBounds( "center", "below", self.wheel_rotating )
        :Offset( 0, -150 )

    -- Arrows
    local arrow_size = 24
    self.arrow_right = self.wheel_container:AddChild( Widget() )
    self.arrow_right.arrow = self.arrow_right:AddChild( Widget.Image( engine.asset.Texture( "UI/upgrade_arrow.tex" ) ) )
        :SetSize( arrow_size, arrow_size )
        :SetTintAlpha(0.7)
        :Bloom( 0.2 )
    self.arrow_right.arrow_edge = self.arrow_right:AddChild( Widget.Image( engine.asset.Texture( "UI/upgrade_arrow_edge.tex" ) ) )
        :SetSize( arrow_size, arrow_size )
        :SetTintAlpha(0.7)
        :Bloom( 0.2 )
    self.arrow_right:LayoutBounds( "after", "center", self.wheel_rotating )
        :Offset( 1, 0 )
    self.arrow_left = self.wheel_container:AddChild( Widget() )
    self.arrow_left.arrow = self.arrow_left:AddChild( Widget.Image( engine.asset.Texture( "UI/upgrade_arrow.tex" ) ) )
        :SetSize( arrow_size, arrow_size )
        :SetTintAlpha(0.7)
        :Bloom( 0.2 )
        :SetScale( -1, 1 )
    self.arrow_left.arrow_edge = self.arrow_left:AddChild( Widget.Image( engine.asset.Texture( "UI/upgrade_arrow_edge.tex" ) ) )
        :SetSize( arrow_size, arrow_size )
        :SetTintAlpha(0.7)
        :Bloom( 0.2 )
        :SetScale( -1, 1 )
    self.arrow_left:LayoutBounds( "before", "center", self.wheel_rotating )
        :Offset( -1, 0 )
        :Hide()


    -- Titles
    self.header_contents = self:AddChild( Widget() )
    self.title = self.header_contents:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_TITLE ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.15 )
    self.subtitle = self.header_contents:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_SUBTITLE ))
        :SetGlyphColour( UICOLOURS.TITLE )
        :Bloom( 0.05 )
    -- Update titles
    self.title:SetText( LOC"UI.UPGRADE_CARD_POPUP.TITLE" )
    self.subtitle:SetText( preview_only and LOC"UI.UPGRADE_CARD_POPUP.PREVIEW_UPGRADE" or LOC"UI.UPGRADE_CARD_POPUP.CONFIRM_UPGRADE" )
    -- Layout titles
    self.title:LayoutBounds( "center", "above", self.subtitle ):Offset( 0, -4 )
    self.header_contents:LayoutBounds( "center", "above", self.wheel_pointer ):Offset( 0, 20 )

    -- Show starting card
    local original_card
    if is_instance( card, Battle.Card ) then
        original_card = Widget.BattleCard( card ):SetScale( ORIGINAL_CARD_SCALE )
    else
        original_card = Widget.NegotiationCard( card ):SetScale( ORIGINAL_CARD_SCALE )
    end

    self.original_card = self:AddChild( original_card )
        :LayoutBounds( "center", "center", self.wheel_rotating )
        :Offset( 0, 0 )

    -- Add option cards container
    self.option_card_widgets = {}
    self.option_cards = self:AddChild(Widget())

    -- Add floater cards container
    self.floater_cards = self:AddChild(Widget())

    -- Add card shadows container
    self.card_shadows = self:AddChild(Widget())

	self.metrics_base_card = card

    self:AddUpgradeOptions( card, preview_only )

    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.cancel_btn = self.bottom_left:AddChild( Widget.IconButton( preview_only and LOC"UI.DIALOGS.BACK" or LOC"UI.DIALOGS.CANCEL", function() self:OnClickCancel() end ) )
    self.cancel_btn:SetIcon( global_images.close )
    self.cancel_btn:LayoutBounds( "left", "above", 60, 60 )
    
    -- Confirm button
    self.bottom_right = self:AddChild( Widget() ):SetAnchors( "right", "bottom" )
    self.confirm_btn = self.bottom_right:AddChild( Widget.IconButton( LOC"UI.DIALOGS.CONTINUE", function() self:OnClickSelect() end ) )
    self.confirm_btn:SetIcon( global_images.accept )
    self.confirm_btn:SetShown( false )
    self.confirm_btn:LayoutBounds( "right", "above", -60, 60 )

    -- Set starting direction
	self:SetDirectionAngle( nil )

    self.anim_pacing = GetIsFastMode() and 0.2 or 0.5
end

function UpgradeCardPopup:AddUpgradeOptions( card, preview_only )
    local upgrade_ids = shallowcopy( card.userdata.upgrade_ids )

    if upgrade_ids then
        local graft_modifiers = {}
        TheGame:GetEvents():BroadcastEvent( "modify_upgrades", card, upgrade_ids, graft_modifiers )

        for i, graft in ipairs( graft_modifiers ) do
            -- Check if there's a graft limiting your number of options
            self.reason_widget = self:AddChild( Widget() )
            local bg_border = 45
            local bg_border_scale = 0.5
            self.reason_widget.bg = self.reason_widget:AddChild( Widget.Panel( engine.asset.Texture("UI/gameover_block.tex") ) )
                :SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
                :SetNineSliceBorderScale( bg_border_scale )
                :SetTintColour( UICOLOURS.PENALTY )
                :Bloom( 0.1 )
            self.reason_widget.desc_w = 230
            self.reason_widget.desc = self.reason_widget:AddChild( Widget.Label("body", 24 ))
                :SetGlyphColour( UICOLOURS.BODYTEXT )
                :LeftAlign()
                :SetWordWrap( true )
                :SetAutoSize( self.reason_widget.desc_w )
                :OverrideLineHeight( 21 )
                :Bloom( 0.05 )
                :SetText( loc.format( LOC"UI.UPGRADE_CARD_POPUP.GRAFT_LIMIT", graft:GetName() ) )
            local text_w, text_h = self.reason_widget.desc:GetSize()
            text_w = text_w + 50
            text_h = text_h + 40
            local nineslice_w, nineslice_h = text_w + bg_border*2*bg_border_scale, text_h + bg_border*2*bg_border_scale
            self.reason_widget.bg:SetSize( nineslice_w, nineslice_h )
            self.reason_widget:LayoutBounds( "before", "center", self.wheel_rotating )
                :Offset( -40, 0 )
                :SetToolTipClass( Widget.TooltipGraft )
                :SetToolTip( graft )

            break -- UI only supports one graft modifier.
        end


        -- We sort so that we have consistent placement of upgrade cards.
        table.sort( upgrade_ids )
        -- Because we place the first one to the right, reverse the order, so that the left-to-right order
        -- matches that shown in the CardDetailsPopup upgrades block.
        table.reverse( upgrade_ids )

        for i, upgrade_id in ipairs( upgrade_ids ) do
            self:AddUpgradeOption( upgrade_id, preview_only, card.wild )
        end
    end
end

function UpgradeCardPopup:AddUpgradeOption(id, preview_only, is_wild)
    local option_card, option_card_widget, floater_card_widget

    local style
    if table.contains( self.card.upgrade_ids or table.empty, id ) then
        style = Widget.CardWidget.STYLE.UPGRADE
    else
        -- base_def *might* be nil if certain mutators are active that change how upgrades occur.
        style = Widget.CardWidget.STYLE.NORMAL
    end
    if is_wild and preview_only then
        style = Widget.CardWidget.STYLE.WILD
    end

    -- If this is not a preview, but this card is wild, show a temporary floater to be animated out
    if is_wild and not preview_only and not GetIsFastMode() then
        if is_instance( self.card, Battle.Card ) then
            option_card = Battle.Card( id, self.agent ) 
            floater_card_widget = self.floater_cards:AddChild(Widget.BattleCard():SetScale( CARD_SCALE ))
            floater_card_widget:RefreshCard( option_card, nil, Widget.CardWidget.STYLE.WILD )
        else
            option_card =  Negotiation.Card( id, self.agent )
            floater_card_widget = self.floater_cards:AddChild( Widget.NegotiationCard():SetScale( CARD_SCALE ))
            floater_card_widget:RefreshCard( option_card, nil, Widget.CardWidget.STYLE.WILD )
        end
    end
    
    if is_instance( self.card, Battle.Card ) then
        option_card = Battle.Card( id, self.agent ) 
        option_card.userdata.charges = self.card:GetCharges() -- Upgrading does not change # of charges.
        option_card_widget = self.option_cards:AddChild(Widget.BattleCard():SetScale( CARD_SCALE ))
        option_card_widget:RefreshCard( option_card, nil, style )
    else
        option_card =  Negotiation.Card( id, self.agent )
        option_card.userdata.charges = self.card:GetCharges() -- Upgrading does not change # of charges.
        option_card_widget = self.option_cards:AddChild( Widget.NegotiationCard():SetScale( CARD_SCALE ))
        option_card_widget:RefreshCard( option_card, nil, style )
    end

    option_card_widget:SetFocusable( true )
    option_card_widget:ShowToolTipOnFocus( true )

    -- Set the card's hover function
    option_card_widget.OnGainFocus = function() option_card_widget._base.OnGainFocus(option_card_widget) self:UpdateCardImageState( option_card_widget ) end
    option_card_widget.OnLoseFocus = function() option_card_widget._base.OnLoseFocus(option_card_widget) self:UpdateCardImageState( option_card_widget ) end
    option_card_widget.OnGainHover = function() option_card_widget._base.OnGainHover(option_card_widget) self:UpdateCardImageState( option_card_widget ) end
    option_card_widget.OnLoseHover = function() option_card_widget._base.OnLoseHover(option_card_widget) self:UpdateCardImageState( option_card_widget ) end

    -- Handle clicks
    if not preview_only then
        option_card_widget:SetOnClick(function() self:SelectUpgrade( option_card ) end)
    end

    -- Preset the glow color
    option_card_widget:SetOutline( nil, false )

    -- Add shadow
    option_card_widget.shadow = self.card_shadows:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) )
        :SetSize( 400, 90 )
        :SetHiddenBoundingBox( true )
        :SetTintColour( is_instance( self.card, Battle.Card ) and UICOLOURS.FIGHT or UICOLOURS.NEGOTIATION )
        :SetTintAlpha( 0.6 )

    self.card_shadows:LayoutBounds( nil, "center", self.wheel_shadow )

    if #self.option_card_widgets == 0 then
        -- First card, goes to the right
        option_card_widget.left = false
        option_card_widget:LayoutBounds( "after", "center", self.wheel_rotating ):Offset( 60, 0 )
        -- Center shadow under the card
        option_card_widget.shadow:LayoutBounds( "center", nil, option_card_widget ):Offset( 0, 0 )

    else
        assert_warning( #self.option_card_widgets == 1, tostring(id) ) -- Stop adding more than 1 upgrade. ILLEGAL
        -- Second card, goes to the left
        option_card_widget.left = true
        option_card_widget:LayoutBounds( "before", "center", self.wheel_rotating ):Offset( -60, 0 )
        -- Center shadow under the card
        option_card_widget.shadow:LayoutBounds( "center", nil, option_card_widget ):Offset( 0, 0 )
        self.arrow_left:Show()
    end

    -- Align the floater
    if floater_card_widget then floater_card_widget:SetPos( option_card_widget:GetPos() ) end

    table.insert(self.option_card_widgets, option_card_widget)
    self:AddRobotOption(option_card_widget)
end

function UpgradeCardPopup:UpdateCardImageState( widget )

    if widget.enabled and ( widget.hover or widget.focus or ( TheGame:FE():GetControlMode() == CONTROL_MODE.TOUCH and widget.selected ) ) then

        if is_instance( widget, Widget.BattleCard ) then
            AUDIO:PlayParamEvent(SoundEvents.battle_deck_card_mouseover, "position", widget:GetPanPos())
        elseif is_instance( widget, Widget.NegotiationCard ) then
            AUDIO:PlayParamEvent(SoundEvents.negotiation_deck_card_mouseover, "position", widget:GetPanPos())
        end

        local r, g, b, a = HexColour( UICOLOURS.FOCUS )
        widget.shadow:ColourFromCurrentTo( r, g, b, a, 0.2 )
        widget.shadow:ScaleTo( nil, 1.4, 0.2 )

        widget:SetOutline( UICOLOURS.FOCUS, true )
        widget:ScaleTo( nil, CARD_SCALE * 1.1, 0.2, easing.outQuad )

        self:GetAncestor( Screen.UpgradeCardPopup ):SetDirectionAngle( -math.pi/2 * ( widget.left and -1 or 1 ) )
    else

        if is_instance( widget, Widget.BattleCard ) then
            AUDIO:PlayParamEvent(SoundEvents.battle_deck_card_mouseoff, "position", widget:GetPanPos())
        elseif is_instance( widget, Widget.NegotiationCard ) then
            AUDIO:PlayParamEvent(SoundEvents.negotiation_deck_card_mouseoff, "position", widget:GetPanPos())
        end

        local r, g, b, a = HexColour( is_instance( self.card, Battle.Card ) and UICOLOURS.FIGHT or UICOLOURS.NEGOTIATION )
        widget.shadow:ColourFromCurrentTo( r, g, b, 0.6, 0.2 )
        widget.shadow:ScaleTo( nil, 1, 0.3 )

        widget:SetOutline( nil, true )
        widget:ScaleTo( nil, CARD_SCALE, 0.2, easing.outQuad )

        self:GetAncestor( Screen.UpgradeCardPopup ):SetDirectionAngle( nil )
    end
end

function UpgradeCardPopup:GetDefaultFocus()
    for k, c in ipairs( self.option_card_widgets ) do
        return c
    end
end

function UpgradeCardPopup:SetDirectionAngle( angle )
    if angle then
        self.wheel_pointer:RotateTo( angle, 0.3, easing.inOutQuad )
        if angle < 0 then
            self.arrow_right:ScaleTo( nil, 1.3, 0.2 )
                :AlphaTo( 1, 0.1 )
            self.arrow_right.arrow_edge:AlphaTo( 0.6, 0.1 )
            self.arrow_left:ScaleTo( nil, 1, 0.1 )
                :AlphaTo( 0.4, 0.05 )
            self.arrow_left.arrow_edge:AlphaTo( 0, 0.05 )
            self.wheel_shadow:ScaleTo( nil, 1.2, 0.3 )
        else
            self.arrow_left:ScaleTo( nil, 1.3, 0.2 )
                :AlphaTo( 1, 0.1 )
            self.arrow_left.arrow_edge:AlphaTo( 0.6, 0.1 )
            self.arrow_right:ScaleTo( nil, 1, 0.1 )
                :AlphaTo( 0.4, 0.05 )
            self.arrow_right.arrow_edge:AlphaTo( 0, 0.05 )
            self.wheel_shadow:ScaleTo( nil, 1.2, 0.3 )
        end
        -- Start audio
        self.hover_loop = AUDIO:CreateEventInstance( "event:/ui/card_upgrade/card_mouseover" )
        self.hover_loop:Start()
    else
        -- Resetting position
        self.wheel_pointer:RotateTo( 0, 0.5, easing.inOutQuad )
        self.arrow_left:ScaleTo( nil, 1, 0.1 )
            :AlphaTo( 0.4, 0.05 )
        self.arrow_left.arrow_edge:AlphaTo( 0, 0.05 )
        self.arrow_right:ScaleTo( nil, 1, 0.1 )
            :AlphaTo( 0.4, 0.05 )
        self.arrow_right.arrow_edge:AlphaTo( 0, 0.05 )
        self.wheel_shadow:ScaleTo( nil, 1, 0.5 )
        -- Stop audio
        if self.hover_loop then
            self.hover_loop:Stop()
            self.hover_loop:Release()
            self.hover_loop = nil
        end
    end
end

function UpgradeCardPopup:SelectUpgrade(card)
    self.selected_card = card

    if TheGame:FE():GetControlMode() == CONTROL_MODE.TOUCH then
        for k, widget in ipairs( self.option_card_widgets ) do
            if self.selected_card.id == widget.card.id then
                widget.selected = true
            else
                widget.selected = false
            end
            self:UpdateCardImageState( widget )
        end
        self.confirm_btn:Show()
    else
    	self:OnClickSelect()
    end
end

function UpgradeCardPopup:OnAdded()

	-- Prepare for animation by getting current values
    local anim_pacing = 0.04
    local title_x, title_y = self.title:GetPos()
    local cancel_btn_x, cancel_btn_y = self.cancel_btn:GetPos()

	-- Hide objects until they're animated in
    -- self.title:SetTintAlpha(0)

    AUDIO:PlayEvent(SoundEvents.upgrade_card_popup)

    -- Our main animation sequence
    local animation_series = Updater.Parallel()

    -- Add each of the sides
    for i, option in ipairs( self.option_cards.children ) do

        -- Hide the original card
        option:SetTintAlpha(0)

        -- Shows the animation on each side of the screen, one for left, one for right
        local side_animation = Updater.Series()

        -- Show the floater, if any
        if #self.floater_cards.children > 0 and self.floater_cards.children[i] then
            local floater = self.floater_cards.children[i]
                :SetTintAlpha(0)

            local anim_rotation = 0.1
            local floater_animation = 
            Updater.Series {
                Updater.Do( function() AUDIO:PlayEvent("event:/ui/card_upgrade/cardupgrade_wiggle") end ),
                Updater.Parallel{
                    Updater.Parallel{
                        Updater.Ease( function(v) floater:SetScale(v,v) end, CARD_SCALE*0.7, CARD_SCALE*1.2, anim_pacing*8, easing.outQuad),
                        Updater.Ease( function(v) floater:SetTintAlpha(v) end, 0, 1, anim_pacing*3.0, easing.inQuad),
                    },
                    Updater.Series {
                        Updater.Ease( function(v) floater:SetRotation(v) end, 0, -anim_rotation, anim_pacing*1.5, easing.inQuad ),
                        Updater.Ease( function(v) floater:SetRotation(v) end, -anim_rotation, anim_rotation, anim_pacing*3.0, easing.inQuad ),
                        Updater.Ease( function(v) floater:SetRotation(v) end, anim_rotation, -anim_rotation, anim_pacing*3.0, easing.inQuad ),
                        Updater.Ease( function(v) floater:SetRotation(v) end, -anim_rotation, 0, anim_pacing*1.5, easing.inQuad ),
                    },
                },
            }
            side_animation:Add( floater_animation )
        end

        -- -- Shows the card coming in, after the floater, if any
        local option_animation = 
            Updater.Parallel {
                Updater.Do( function() 
                    -- Play audio for the upgrade option
                    AUDIO:PlayParamEvent( "event:/ui/card_upgrade/upgrade_flyin", "position", option:GetPanPos() )
                end ),
                Updater.Ease( function(v) option:SetTintAlpha(v) end, 0, 1, anim_pacing*3, easing.inQuad),
            }

        -- Make the floater fade out 
        if #self.floater_cards.children > 0 and self.floater_cards.children[i] then
            local floater = self.floater_cards.children[i]
            option_animation:Add(
                Updater.Series {
                    Updater.Ease( function(v) floater:SetScale(v,v) end, CARD_SCALE*1.2, CARD_SCALE*1.0, anim_pacing*3, easing.inQuad),
                    Updater.Ease( function(v) floater:SetTintAlpha(v) end, 1, 0, 0.1, easing.inQuad),
                    Updater.Do( function() floater:Hide() end )
                }
            )
        end
        side_animation:Add( Updater.Wait( 0.05 ) )
        side_animation:Add( option_animation )

        -- Add this side to the main animation
        animation_series:Add( side_animation )
    end

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )

    -- Start animation
    self.fadein_anim = self:RunUpdater( animation_series )
end


function UpgradeCardPopup:OnControlModeChange( cm, device_id )
    UpgradeCardPopup._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.cancel_btn:Show()
        self.confirm_btn:Hide()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.cancel_btn:Show()
        self.confirm_btn:SetShown( self.selected_card ~= nil )
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.cancel_btn:Hide()
        self.confirm_btn:Hide()
    end
end

function UpgradeCardPopup:OnScreenModeChange( sm )
    UpgradeCardPopup._base.OnScreenModeChange( self, sm )

    self.bottom_left:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.bottom_right:SetLayoutScale( LAYOUT_SCALE[sm] )

    self.wheel_container:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.original_card:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.floater_cards:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.option_cards:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.card_shadows:SetLayoutScale( LAYOUT_SCALE[sm] )

    -- Scale header
    self.header_contents:SetLayoutScale( LAYOUT_SCALE[sm] )
        :LayoutBounds( "center", "above", self.wheel_pointer ):Offset( 0, 20 )
end

function UpgradeCardPopup:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
    self.bottom_right:SetPos( 0, 0 )
    if GetStreamerMode() == 2 then
        -- Nudge the right side widgets
        self.bottom_right:SetPos( -360, 0 )
    end
end

function UpgradeCardPopup:OnClickCancel()
    self:GetFE():PopScreen(self)
    if self.fn then
        self.fn()
    end
end

function UpgradeCardPopup:OnClickSelect()
    local selected_idx = -1
    local card_choices = {}

    for k = 1, #self.option_card_widgets do
        local option_card = self.option_card_widgets[k].card
        table.insert(card_choices, {ID = option_card.id})
        if self.selected_card.id == option_card.id then
            selected_idx = k
        end
    end

    local json_t = {
        SELECTED_ID=self.selected_card.id,
        SELECTED_INDEX=selected_idx,
        CARD_CHOICES = card_choices,
        PLAYER_DATA=TheGame:GetGameState():GetPlayerState(),
    }
    SendMetricsData("UPGRADED_CARD", json_t )

    -- Do the upgrade.        
    if is_instance( self.selected_card, Negotiation.Card ) then
        self.upgraded_card = self.selected_card.owner.negotiator:UpgradeCard( self.card, self.selected_card.id )
    elseif is_instance( self.selected_card, Battle.Card ) then
        self.upgraded_card = self.selected_card.owner.battler:UpgradeCard( self.card, self.selected_card.id )
    end
        
    AUDIO:PlayEvent(SoundEvents.upgrade_card_confirm)
    
    self:GetFE():PopScreen(self)
    if self.fn then
        self.fn(self.selected_card)
    end
end

