require "ui/widgets/tooltipcodex"
require "ui/widgets/tooltipagent"
require "ui/widgets/locationtooltip"
require "ui/screens/draftchoicepopup"
require "ui/screens/draftpackpopup"
require "ui/screens/edit_string_popup"

----------------------------------------

local UIHelpers = class( "UIHelpers" )

function UIHelpers.InfoPopup( ... )
    local screen = Screen.InfoPopup( ... )
    TheGame:FE():InsertScreen( screen )
    return screen
end

function UIHelpers.TutorialPopup( seen_key, ...)
    if not TheGame:GetGameState():HasSeen( seen_key ) then
        local screen = Screen.TutorialPopup( seen_key, ... )
        TheGame:FE():PushScreen( screen )
        return screen
    end
end

local HOTKEY_TO_IDX =
{
    [Controls.Digital.OPTION_1]=1,
    [Controls.Digital.OPTION_2]=2,
    [Controls.Digital.OPTION_3]=3,
    [Controls.Digital.OPTION_4]=4,
    [Controls.Digital.OPTION_5]=5,
    [Controls.Digital.OPTION_6]=6,
    [Controls.Digital.OPTION_7]=7,
    [Controls.Digital.OPTION_8]=8,
    [Controls.Digital.OPTION_9]=9,
    [Controls.Digital.OPTION_0]=10,
}

function UIHelpers.FindHotkeyIdx( control )
    for i, control in control:Controls() do
        if HOTKEY_TO_IDX[ control ] then
            return HOTKEY_TO_IDX[ control ]
        end
    end
end

function UIHelpers.KeepOnScreen(widget, margin)
    local screenw, screenh = TheGame:FE():GetScreenDims()
    local xmin, ymin, xmax, ymax = widget:GetWorldBoundingBox()
    local tw, th = xmax - xmin, ymax - ymin
    
    if xmax > screenw then
        widget:Offset( screenw - xmax - (margin or 0))
    elseif xmin < 0 then
        widget:Offset( -xmin + (margin or 0))
    end

    if ymax > screenh then
        widget:Offset( nil, screenh - ymax - (margin or 0))
    elseif ymin < 0 then
        widget:Offset( 0, -ymin + (margin or 0))
    end
end

function UIHelpers.DoEncounter( encounter, on_finish, plax )
    --TheGame:FE():InkTransition(.3)

    local screen = TheGame:FE():FindScreen( Screen.LocationScreen )
    if screen == nil then
        screen = Screen.LocationScreen( encounter:GetLocation() )
        TheGame:FE():InsertScreen( screen )
    end
    --TheGame:FE():FadeIn(0.5)

    screen:EnterEncounter( encounter, on_finish )
end


function UIHelpers.DoSpecificConvo(agent, convo_id, state_id, on_done, params, quest )

    local location = agent and agent:GetLocation() or TheGame:GetGameState():GetPlayerAgent():GetLocation()
    local encounter = Encounter( agent, location )
    local convo_def = Content.GetConvoStateGraph(convo_id)
    if convo_def then
        state_id = state_id or "START"
        local hub = ConvoStateGraph(encounter, convo_def, state_id, quest)
        if not hub:TryValidateHub( agent, state_id ) then
            LOGWARN( "Failed validation for convo %s.%s", convo_id, state_id )
        else
            encounter:SetInitialHub( hub, state_id, params )
            UIHelpers.DoEncounter( encounter, on_done )
        end
    else
        assert("MISSING CONVO", convo_id)
    end
end

function UIHelpers.ItemTooltipsClicked( widget, tag, text )
    if tag then
        local agent_guid = tonumber(tag:match("agent_(%d+)" ))
        local agent = TheGame:GetGameState():GetAgent( agent_guid )
        if agent and not agent:IsPlayer() and agent:IsSentient() then
            local details = Screen.PeopleCompendiumDetailsScreen()
            TheGame:FE():PushScreen( details )
            details:ShowAgent( agent )
        end
    end
end

function UIHelpers.ItemTooltips( widget, tag, text, ... )
    
    if tag then
        local agent_guid = tonumber(tag:match("agent_(%d+)" ))
        if agent_guid then
            widget:SetToolTip( agent_guid )
            widget:SetToolTipClass( Widget.TooltipAgent )
            return
        end

        local location_guid = tonumber(tag:match("location_(%d+)" ))
        if location_guid then
            local location = TheGame:GetGameState():GetLocation( location_guid )
            widget:SetToolTip( location )
            widget:SetToolTipClass( Widget.LocationToolTip )
            return
        end

        local card_idx = tag:match( "card_([_%d]+)" )
        if card_idx then
            local card = select( card_idx, ... )
            if is_instance( card, Battle.Card ) then
                widget:SetToolTipClass( Widget.TooltipBattleCard )
                widget:SetToolTip( { CARD = card } )

            elseif is_instance( card, Negotiation.Card ) then
                widget:SetToolTipClass( Widget.TooltipNegotiationCard )
                widget:SetToolTip( { CARD = card } )
            end 
            return
        end

        local attack_id = tag:match( "battlecard_([_%w]+)")
        if attack_id then
            local carddef = Content.GetBattleCard( attack_id )
            widget:SetToolTipClass( Widget.TooltipBattleCard )
            widget:SetToolTip( { card_def = carddef } )
            return
        end

        local graft_id = tag:match( "graft_([_%w]+)")
        if graft_id then

            widget:SetToolTipClass( Widget.TooltipGraft )
            --this is gross, because we lose the connection to userdata and owner.
            widget:SetToolTip( GraftInstance(graft_id) )
            return
        end

        local card_id = tag:match( "negotiationcard_([_%w]+)")
        if card_id then
            local carddef = Content.GetNegotiationCard( card_id )
            widget:SetToolTipClass( Widget.TooltipNegotiationCard )
            widget:SetToolTip( { card_def = carddef } )
            return
        end

        local user_idx = tonumber(tag:match( "userdata_([_%w]+)"))
        if user_idx then
            local userdata = widget:GetUserData( user_idx )
            if is_instance( userdata, Negotiation.Card ) then
                widget:SetToolTip( { CARD = userdata } )
                widget:SetToolTipClass( Widget.TooltipNegotiationCard )
            elseif is_instance( userdata, Battle.Card ) then
                widget:SetToolTip( { CARD = userdata } )
                widget:SetToolTipClass( Widget.TooltipBattleCard )
            elseif is_instance( userdata, GraftInstance ) then
                widget:SetToolTip( userdata )
                widget:SetToolTipClass( Widget.TooltipGraft )
            end
            return
        end

        local codex_entry = Content.LookupCodexEntry( tag )
        if codex_entry then
            widget:SetToolTip( codex_entry )
            widget:SetToolTipClass( Widget.TooltipCodex )
            return
        end
    end

    widget:SetToolTip( nil )
    widget:SetToolTipClass( nil )
end

function UIHelpers.PulseWidget( widget, period )
    if period then
        local function PulseIn()
            local function PulseOut()
                widget:ColourTo( 3, 3, 3, 1, 1, 1, 1, 1, period, easing.linear, PulseIn )
            end
            widget:ColourTo( 1, 1, 1, 1, 3, 3, 3, 1, period, easing.linear, PulseOut )
        end

        PulseIn()
    else
        widget:StopAll()
        widget:SetTintColour( 1, 1, 1, 1 )
    end
end

function UIHelpers.PulseBloom( widget, period, magnitude )
    if period then
        widget.bloom_coro = widget:StartCoroutine( function()
            local elapsed = 0
            while true do
                local dt = coroutine.yield()
                elapsed = dt + elapsed

                local bloom = math.sin( math.pi * 2 * elapsed / period )
                widget:SetBloom( bloom * (magnitude or 1.0) )
            end

        end )

    elseif widget.bloom_coro then
        widget:SetBloom( 0 )
        widget:StopCoroutine( widget.bloom_coro )
    end
end


function UIHelpers.PulseAlpha( widget, period, magnitude )
    if period then
        if widget.alpha_coro then
            widget:StopCoroutine( widget.alpha_coro )
        end

        widget.alpha_coro = widget:StartCoroutine( function()
            local elapsed = 0
            while true do
                local dt = coroutine.yield()
                elapsed = dt + elapsed

                local alpha = (1.0 - magnitude) + magnitude * (0.5 + 0.5 * math.sin( math.pi * 2 * elapsed / period ))
                widget:SetTintAlpha( alpha )
            end

        end )

    elseif widget.alpha_coro then
        widget:SetTintAlpha( 1 )
        widget:StopCoroutine( widget.alpha_coro )
        widget.alpha_coro = nil
    end
end

function UIHelpers.PingImage( widget, parent, fn )
    local ping = Widget.Image( widget:GetTexture(), widget:GetSize() )
    ping:SetHiddenBoundingBox( true ) -- Don't want this animated thing to affect layout!
    ping:SetBloom( 0.1 )

    if parent then
        parent:AddChild( ping )
        ping:SetPos( parent:TransformFromWidget( widget, 0, 0 ))
    else
        widget:AddChild( ping )
    end
 
    if fn == nil then
        fn = function( ping )
            ping:ScaleTo( 1, 2, 0.5, easing.outQuad )
            ping:AlphaTo( 0, 0.5, easing.outQuad )
            ping:DelayIfAnimating()
            ping:Remove()
        end
    end

    ping:StartCoroutine( fn, ping )
    return ping
end

function UIHelpers.BounceWidget( widget, start_scale, end_scale, duration )
    widget:StartCoroutine( function()
        widget:ScaleTo( start_scale, end_scale, duration/2, easing.outExpo )
        widget:DelayIfAnimating()
        widget:ScaleTo( end_scale, start_scale, duration/2, easing.outExpo )
    end)
end

function UIHelpers.EditString( title, subtitle, initial_value, on_done_fn )
    local screen = Screen.EditStringPopup( title, subtitle, initial_value, on_done_fn )
    TheGame:FE():PushScreen(screen)
end

function UIHelpers.PlayAnimOnce( widget, animname, anim )
    local widget = widget:AddChild( Widget.Anim() )
    widget:AddAnims(GetAnim(animname))
    widget:AddOverrideFile(animname)
    widget:SetAutoRemove()
    widget:Play( anim )
    return widget
end

function UIHelpers.PlayParticleBurst( parent, id )
    local widget = parent:AddChild( Widget.ParticleSystem() )
    widget:LoadParams( id )
    widget:StartCoroutine( function() 
        repeat
            coroutine.yield()
        until widget:IsDone()
        widget:Remove()
    end )
    return widget
end

function UIHelpers.PassTime( dest_phase )
    TheGame:GetGameState():AdvanceTime( dest_phase )
end

function UIHelpers.ShowGainCards(cards, x, y, on_end, scale )
    local overlay = TheGame:FE():FindScreen( Screen.MainOverlay )
    if overlay then
        return overlay:ShowGainCards( cards, x, y, on_end, scale )
    end
end

function UIHelpers.ShowLostCards(cards, screen, on_end)

    local scale = 1
    local dismiss = false

    local coro = screen:StartCoroutine(function() 
        local card_widgets = {}
        local root = screen.center:AddChild(Widget())
        local total_width = 0

        for k,card in ipairs(cards) do

            local card_widget
            if is_instance( card, Negotiation.Card ) then
                card_widget = root:AddChild( Widget.NegotiationCard( card ))
            else
                card_widget = root:AddChild( Widget.BattleCard( card ) )
            end
            table.insert(card_widgets, card_widget)
            local w, h = card_widget:GetSize()
            total_width = total_width + w
        end

        local spacing = ((RES_X * 0.6) - total_width ) / (#cards - 1)
        for i, card_widget in ipairs( card_widgets ) do
            card_widget:LayoutBounds("after", "center"):Offset( spacing, 0)
        end
            
        root:LayoutBounds("center", "center", 0, 0)

        for k, card_widget in ipairs(card_widgets) do
            card_widget:Hide()
        end

        for k, card_widget in ipairs(card_widgets) do
            local card = card_widget.card
            AUDIO:PlayEvent( CARD_RARITY_SOUNDS[ card.rarity ] )
            AUDIO:PlayEvent( CARD_TYPE_SOUNDS[ GetCardType( card )] )
            card_widget:Show():ScaleTo(scale*.6, scale*.7, 1, easing.outCirc)

            if k < #card_widgets and not dismiss then
                screen:Delay(.4)
            end
        end

        local wait_time = 1
        local t = 0 
        while t < wait_time and not dismiss do
            t = t  + coroutine.yield()
        end


        for k, card_widget in ipairs(card_widgets) do
            card_widget:StartCoroutine( card_widget.ShowErosion, card_widget, "expend_flourish" )

            if k < #card_widgets and not dismiss then
                screen:Delay(.2)
            end
        end

        if on_end then
            on_end()
        end
    end)

    return function() dismiss = true end
end

function UIHelpers.ShowBetaMessage(title, body, button, fn)
    local popup = Screen.BetaMessagePopup(title, body, button, fn)
    TheGame:FE():PushScreen(popup)
end



function UIHelpers.ShowDemoMessage()
    local function OnDone()
        local popup = UIHelpers.InfoPopup( LOC"UI.DEMO.TITLE", LOC"UI.DEMO.BODY", LOC"UI.DEMO.RESET" )
        popup:AddButton(LOC"UI.DEMO.BUTTON", global_images.accept, function() engine.inst:OpenDemoPage() end)
        TheGame:FE():InsertScreen( Screen.MainMenu() )
    end

    if TheGame:GetGameState() and TheGame:GetGameState().save_file_name then
        TheGame:DeleteSave( TheGame:GetGameState().save_file_name, function( ... ) print( "Deleted demo save", TheGame:GetGameState().save_file_name, ... ) end )
    end

    TheGame:FE():FadeTransition( function()
        --local MOVIE_ASPECT = 1920/1080
        --local w, h = 2560, 2560 / MOVIE_ASPECT
        local w, h = 1920, 1080

        -- By default, the main menu will FadeTransition in so we we have override that flow.
        TheGame:Reset( function()
            local screen = Screen.MovieScreen( 'movies/gameplay_trailer.ogv', w, h, OnDone, true )
            TheGame:FE():InsertScreen( screen )
        end )
    end )
end

-- Shows a popup saying a character has been unlocked and is playable now
function UIHelpers.ShowUnlockMessage( unlock, agent_id, title, text, fn )
    local popup = Screen.GameModeUnlockedMessagePopup( unlock, agent_id, title, text, fn )
    TheGame:FE():PushScreen(popup)
end

function UIHelpers.GetHint()

    -- Weighted hints
    local random_hints = {
        ["UI.HINTS.HATE"] = 1,
        ["UI.HINTS.LOVE"] = 1,
        ["UI.HINTS.NEARBY"] = 1,
        ["UI.HINTS.OVERFLOW"] = 1,
        ["UI.HINTS.DRINK"] = 1,
        ["UI.HINTS.SLEEP"] = 1,
        ["UI.HINTS.DECKS"] = 1,
        ["UI.HINTS.UPGRADE"] = 1,
        ["UI.HINTS.VARIANTS"] = 1,
        ["UI.HINTS.FASTMODE"] = 0
    }

    -- Check if the player's resolve is low
    local resolve, maxresolve = 0, 0
    if TheGame:GetGameState() and TheGame:GetGameState():GetCaravan() then
        resolve, maxresolve = TheGame:GetGameState():GetCaravan():GetResolve()
    end
    if resolve / maxresolve < 0.3 then
        random_hints["UI.HINTS.DRINK"] = 6
    end

    -- Get player relationships
    local loved = 0
    local liked = 0
    local disliked = 0
    local hated = 0
    if TheGame:GetGameState() then
        local player = TheGame:GetGameState():GetPlayerAgent()
        if player and player.social_connections then
            -- Go through all the player's relationships
            for other_agent, agent_rel in pairs(player.social_connections.relationships) do
                if agent_rel == RELATIONSHIP.LOVED and not other_agent:IsDead() then
                    loved = loved +1
                end
                if agent_rel == RELATIONSHIP.LIKED and not other_agent:IsDead() then
                    liked = liked +1
                end
                if agent_rel == RELATIONSHIP.DISLIKED and not other_agent:IsDead() then
                    disliked = disliked +1
                end
                if agent_rel == RELATIONSHIP.HATED and not other_agent:IsDead() then
                    hated = hated +1
                end
            end
        end
        if loved > 2 then
            random_hints["UI.HINTS.LOVE"] = 4
        end
        if liked+loved > 4 then
            random_hints["UI.HINTS.NEARBY"] = 4
        end
        if hated > 2 then
            random_hints["UI.HINTS.HATE"] = 5
        end

        -- Check the player's deck size difference
        if math.abs( #player.battler:GetCards() - #player.negotiator:GetCards() ) > 5 then
            random_hints["UI.HINTS.DECKS"] = 9
        end

        -- Check the player's deck sizes
        if #player.battler:GetCards() > 20 or #player.negotiator:GetCards() > 20 then
            random_hints["UI.HINTS.OVERFLOW"] = 5
        end
        if #player.battler:GetCards() > 30 or #player.negotiator:GetCards() > 30 then
            random_hints["UI.HINTS.OVERFLOW"] = 8
        end
    end

    -- Check the player's number of meta unlocks
    local num_unlocked = TheGame:GetGameProfile():GetNumUnlocked()
    if num_unlocked > 2 then
        random_hints["UI.HINTS.FASTMODE"] = 2
    end

    return LOC( weightedpick( random_hints ) )
end

function UIHelpers.GetFocusCard()
    local focus_widget = TheGame:FE():GetFocusWidget()
    if focus_widget and ( focus_widget._class == Widget.BattleCard._class or focus_widget._class == Widget.NegotiationCard._class ) then
        return focus_widget
    end
end


function UIHelpers.ShowTutorials(tutorials_to_test, title, body, on_done_fn)
    local need_to_see = {}
    for k,tut in ipairs(tutorials_to_test) do
        if not TheGame:GetGameProfile():HasSeenMessage(tut) then
            table.insert(need_to_see, tut)
        end
    end

    if #need_to_see > 0 then
        local screen = TheGame:FE():GetTopScreen()
        screen:Delay( 1.2 )
        for i,tut in ipairs(need_to_see) do
            TheGame:GetGameProfile():SetHasSeenMessage(tut)
        end

        TheGame:FE():InsertScreen( Screen.YesNoPopup(title, body, nil, nil, LOC"UI.NEGOTIATION_PANEL.TUTORIAL_NO" ))
            :SetFn(function(v)
                if v == Screen.YesNoPopup.YES then 
                    local coro = screen:StartCoroutine(function() 
                        
                        for k,tut in ipairs(need_to_see) do
                            local advance = false
                            TheGame:FE():PushScreen( Screen.SlideshowScreen( tut, function() advance = true end ):SetAutoAdvance(false) ) 
                            while not advance do                            
                                coroutine.yield()
                            end
                        end

                        if on_done_fn then
                            on_done_fn(true)
                        end

                    end )
                else
                    if on_done_fn then
                        on_done_fn(false)
                    end
                end
            end)
    else
        if on_done_fn then
            on_done_fn(false)
        end
    end
end

function UIHelpers.SaveTextureAsPNG(texture, filename)
    local w, h = texture:GetSize()
    local scene = engine.rendering.Scene()
    local capture = Widget.CaptureImage(scene):SetSize( w, h )
    local container = Widget()
    local anim = container:AddChild(Widget.Image(texture):SetSize(w,h))
    local root = engine.rendering.SceneGraphNode()
    root:SetScene(scene)
    root:AddChild(container.node)
    capture:RenderToScene( container, -w/2, -h/2, w/2, h/2 )
    engine.inst:SaveTextureToPNG( capture:GetTexture(), "SETTINGS:"..filename .. ".png" );
end

