require ("util/coro")
require ("ui/updater")
require "util/bezier"
local easing = require "util/easing"

--we tend to create and destroy a lot of widgets, so we recycle their engine-side parts to reduce gc churn
local GetShaderConstant, RecycleShaderConstant = CreateRecyclers(200, function(name, ...) return engine.rendering.ShaderConstantValue(name, ...) end, function(node, name, ...) node:SetName(name) node:SetValue(...) end)
local GetSceneGraphNode, RecycleSceneGraphNode = CreateRecyclers(200, function(model) return engine.rendering.SceneGraphNode(model) end, function(node, model) if model then node:SetModel(model) end end, function(v) v:Clear() end)

class("Widget")
Widget._start_table_size = 32
Widget.can_reload = true
local min, max = math.min, math.max

function Widget:init(model, hanchor, vanchor)
    self.children = table.empty
    self.enabled = true
    self.shown = true
    self.focus_targets = nil
    self.dirtytransform = false

    self.node = GetSceneGraphNode(model)
    assert(self.node)
    self.model = model
    self.hover = false

    self.x = 0
    self.y = 0
    self.z = 0
    self.sx = 1
    self.sy = 1
    self.r = 0
    self.layout_scale = 1

    self.hanchor = nil
    self.vhanchor = nil
    self.anchorx = 0
    self.anchory = 0
    self.visible = false

    if vanchor and hanchor then
        self:SetAnchors(hanchor, vanchor)
    end

    self.bboxcache = {valid = true, x1 = 0, y1 = 0, x2 = 0, y2 = 0 }
end

function Widget:SetScissor( x, y, w, h )
    if x then
        self.scissor_rect = { x, y, x + w, y + h }
        self.node:SetScissor( x, y, w, h )
    else
        self.scissor_rect = nil
        self.node:ClearScissor()
    end
    return self
end

function Widget:IgnoreInput(ignore)
    self.ignore_input = ignore
    return self
end

function Widget:SetStencilWrite(val)
    self.node:SetStencilWrite(val)
    return self
end

function Widget:SetStencilTest(val)
    self.node:SetStencilTest(val)
    return self
end

function Widget:SetStencilContext(val)
    self.node:SetStencilContext(val)
    return self
end

function Widget:DrawDebugText(str, x,y,sz,col)
    sz =sz or 10
    x = x or 0
    y = y or 0
    x, y = self:TransformToWorld(x,y)
    self.fe:DrawDebugText(str, x, y, sz, col)
end

function Widget:DrawDebugBounds(col, xoff, yoff)
    if self.model == nil then
        return
    end
    
    local x1, y1, x3, y3 = self:GetBoundingBox()
    if x1 then
        x1 = x1 + (xoff or 0)
        x3 = x3 + (xoff or 0)

        y1 = y1 + (yoff or 0)
        y3 = y3 + (yoff or 0)

        local x2, y2 = x3, y1
        local x4, y4 = x1, y3
    --self:DrawDebugTriangle(x1,y1, x2,y2, x3,y3, col)
        self:DrawDebugTriangle(x2,y2, x3,y3, x1,y1, col)
        self:DrawDebugTriangle(x1,y1, x4,y4, x3,y3, col)
    end
end

function Widget:DrawDebugTriangle(x1,y1,x2,y2,x3,y3,col)
    x1, y1 = self:TransformToWorld(x1,y1)
    x2, y2 = self:TransformToWorld(x2,y2)
    x3, y3 = self:TransformToWorld(x3,y3)
    self.fe:DrawDebugTriangle(x1,y1,0, x2,y2,0, x3,y3,0, col)
end

function Widget:DrawDebugLine(x1,y1,x2,y2,col)
    x1, y1 = self:TransformToWorld(x1,y1)
    x2, y2 = self:TransformToWorld(x2,y2)
    self.fe:DrawDebugLine(x1,y1,x2,y2, col)
end

function Widget:DoDebugRender(dbg, level, force)
    if not dbg:IsDebugFlagged( DBG_FLAGS.UI ) then
        return
    end
    
    level = level or 0
    if not self.shown or self.nodebug then
        return "CONTINUE"
    end
    local do_render = false

    if not force and self.children then
        for k = #self.children, 1 , -1 do
        --for k = 1, #self.children do
            local v = self.children[k]
            local res = v:DoDebugRender(dbg, level + 1)
            if res == "RENDERUP" then
                do_render = true
                break
            end
            if res == "STOP" then
                return "STOP"
            end
        end
    end


    local x1, y1, x2, y2 = self:GetBoundingBox()
    local bounds_hit
    if not do_render then
        if x1 and y1 and x2 and y2 then
            local lx, ly = self:TransformFromWorld(TheGame:GetInput():GetMousePos())
            local bounds_hit = force or (lx >= x1 and lx <= x2 and ly >= y1 and ly <= y2)
            if bounds_hit then
                do_render = true
                bounds_hit = true
                self:DrawDebugBounds(0xFF000060)
                if self.fe.debugfreeze then
                    self:DrawDebugBounds(0x00FFFFA0, lx, ly)
                end
                self.fe:SetDebugWidget(self)
            end
        end
    end

    if do_render then
        local axis_len = 20
        local wx, wy = self:TransformToWorld(0,0)
        local rx, ry = self:TransformToWorld(1,0)
        local ux, uy = self:TransformToWorld(0,1)

        local drx, dry = rx - wx, ry - wy
        local rlen = math.sqrt(drx*drx + dry*dry)
        drx, dry = axis_len*drx/rlen, axis_len*dry/rlen

        local dux, duy = ux - wx, uy - wy
        local ulen = math.sqrt(dux*dux + duy*duy)
        dux, duy = axis_len*dux/ulen, axis_len*duy/ulen

        self.fe:DrawDebugLine(wx,wy, wx+ drx,wy + dry, 0xFF0000FF)
        self.fe:DrawDebugLine(wx,wy, wx+dux,wy+duy, 0x00FF00FF)
        
        if self.hanchor then
            self.fe:DrawDebugLine(wx+ drx,wy + dry, wx+dux,wy+duy, 0x0000FFFF)
            self.fe:DrawDebugText(string.format("%s\n%s", self.hanchor, self.vanchor), wx, wy, 8, 0x3BC382FF)
        end

        if self.regx or self.regy then
    
            local base_scale = 1
            if self.fe then
                base_scale = self:GetFE():GetBaseWidgetScale()
            end

            local trans_scale = 1
            local parent_scale_x, parent_scale_y
            if self.base_scaling ~= false and (self.hanchor or self.vanchor or self.base_scaling) then
                trans_scale = base_scale
            else
                trans_scale = 1
            end

            self.fe:DrawDebugLine(wx,wy,wx-trans_scale*(self.regx or 0), wy-trans_scale*(self.regy or 0), 0xff00ffff)
        end

        if x1 and y1 and x2 and y2 then
            local bbcol = 0xDBD223BD
            self:DrawDebugLine(x1,y1,x1,y2, bbcol)
            self:DrawDebugLine(x2,y1,x2,y2, bbcol)
            self:DrawDebugLine(x1,y1,x2,y1, bbcol)
            self:DrawDebugLine(x1,y2,x2,y2, bbcol)
        end

        if self.parent and not self.hanchor then
            local px, py = self.parent:TransformToWorld(0,0)
            local toparent = 0x44BA65B6
            self.fe:DrawDebugLine(wx,wy,px,py, toparent)
        end

        if bounds_hit then
            self:DrawDebugText( self:GetFullName(),(x1+x2)/2, (y1+y2)/2,20, 0xA33136FF )
        end
        

        if self.OnDebugRender then
            self:OnDebugRender()
        end


        if self.hanchor then
            return "STOP"
        end


        return "RENDERUP"
    end


    return "CONTINUE"
end    

local fullnamet = {}
function Widget:GetFullName()
    table.clear(fullnamet)
    
    local function internal(w)
        if w.parent then
            internal(w.parent)
        end
        if w.name then
            table.insert(fullnamet, w.name)
        end
    end
    internal(self)

    return table.concat(fullnamet, ".")
end

function Widget:InvalidateBBox()
    self.bboxcache.valid = false
    if not self.hidden_bb then
        if self.parent then
            self.parent:InvalidateBBox()
        end
    end
end

function Widget:SetName(name)
    self.name = name
    --self.node:SetName(name)
    return self
end

function Widget:InheritTransform(val)
    self.node:InheritTransform(val)
    self:MarkTransformDirty()
    return self
end

function Widget:SetAnchors(h,v)
    
    if h and v then
        assert( h == "left" or h == "right" or h == "fill" or h == "center" )
        assert( v == "top" or v == "bottom" or v == "fill" or v == "center" )
        self:InheritTransform(false)
    else
        self:InheritTransform(true)
    end
    
    self.hanchor = h 
    self.vanchor = v
    self:UpdateAnchors()

    return self
end

function Widget:SetToolTipLayoutFn(fn)
    self.tooltiplayoutfn = fn
    return self
end

function Widget:SetToolTipFn(fn)
    assert( type(fn) == "function" )
    self.hover_check = true
    self.tooltipfn = fn
    return self
end

function Widget:GetToolTipClass()
    return self.tooltip_class
end

function Widget:SetToolTipClass( tt_class )
    self.tooltip_class = tt_class
    return self
end

function Widget:DisableToolTip( disabled )
    self.tooltip_disabled = disabled
    return self
end

function Widget:SetToolTip(tt)
    if not self.hover_check then
        self.hover_check = (tt ~= nil)
    end

    if type(tt) == "string" and #tt == 0 then
        tt = nil
    end

    if tt ~= self.tooltip then
        self.tooltip = tt
        self:SetToolTipDirty( true )
    end

    return self
end

function Widget:GetToolTip()
    if not self.tooltip_disabled then
        if self.tooltipfn then
            return self.tooltipfn(self)
        end

        return self.tooltip
    end
end

function Widget:ShowToolTipOnFocus( show )
    self.show_tooltip_on_focus = show ~= false
    return self
end

function Widget:SetToolTipDirty( dirty )
    self.tooltip_dirty = dirty
end

function Widget:GetToolTipDirty()
    return self.tooltip_dirty
end

function Widget:GetSize()
    local bx1,by1,bx2,by2 = self:GetBoundingBox()
    if bx1 and bx2 and by1 and by2 then
        return math.abs(bx2 - bx1), math.abs(by2 - by1)
    else
        return 0, 0
    end
end

function Widget:GetCentroid()
    local bx1,by1,bx2,by2 = self:GetBoundingBox()
    if bx1 then
        return (bx1+bx2)/2, (by2+by1)/2
    end
    return 0,0
end

function Widget:GetScaledSize()
    local cx1, cy1, cx2, cy2 = self:GetBoundingBox()
    
    if cx1 and cy1 and cx2 and cy2 then
        
        local px1, py1 = self:TransformToParent(cx1, cy1)
        local px2, py2 = self:TransformToParent(cx2, cy2)
        local px3, py3 = self:TransformToParent(cx1, cy2)
        local px4, py4 = self:TransformToParent(cx2, cy1)
        
        local min_x = min( min(px1, px2), min(px3, px4) )
        local max_x = max( max(px1, px2), max(px3, px4) )

        local min_y = min( min(py1, py2), min(py3, py4) )
        local max_y = max( max(py1, py2), max(py3, py4) )

        return max_x - min_x, max_y - min_y
    else
        return 0, 0
    end
end

function Widget:SetEnabled( enabled )
    if enabled then
        self:Enable()
    else
        self:Disable()
    end
    return self
end

function Widget:IsEnabled()
    return self.enabled
end

function Widget:Enable()
    self.enabled = true
    return self
end

function Widget:Disable()
    self.enabled = false
    self:ClearFocus()
    return self
end

function Widget:CheckHit(x,y)
    local lx, ly = self:TransformFromWorld(x,y)
    local x1,y1,x2,y2 = self:GetBoundingBox()

    if x1 and y1 and x2 and y2 then
        if lx >= x1 and lx <= x2 and ly >= y1 and ly <= y2 then
            return true
        end
    end

    return false
end

-- Returns the scissor rect in world coordinates. If this widget doesn't use a scissor, the call is passed on to the parent
function Widget:GetScissorRect()
    if self.scissor_rect then
        local x1, y1, x2, y2 = table.unpack( self.scissor_rect )

        local wx1, wy1 = self:TransformToWorld(x1, y1)
        local wx2, wy2 = self:TransformToWorld(x2, y2)

        return wx1, wy1, wx2, wy2

    elseif self.parent then
        return self.parent:GetScissorRect()
    end

    return 0, 0, RES_X, RES_Y
end

function Widget:CheckMouseHover(x,y,trace)
    
    if trace then
        trace:PushWidget( self )
    end

    if self.ignore_input or not self.shown or self.dragging then
        if trace then
            trace:PopWidget( "false : early bail" )
        end
        return false
    end
    

    if self.hover_check or self.blocks_mouse then
        local hit = self:CheckHit(x, y)
        if not hit then                
            if trace then
                trace:PopWidget( string.format( "false : interesction %.1f, %.1f", x, y ))
            end
            return false
        end
    end

    if self.scissor_rect then
        local lx, ly = self:TransformFromWorld(x,y)
        local x1, y1, x2, y2 = table.unpack( self.scissor_rect )
        if lx < x1 or lx > x2 or ly < y1 or ly > y2 then
            if trace then
                trace:PopWidget( "false : scissor" )
            end
            return false -- Never hit success outside the scissor region.
        end
    end
    
    local is_blocked = false

    if self.children then
        for i = #self.children, 1, -1 do
            local child = self.children[i]
            if child and child.shown and ((child.children and #child.children > 0) or child.hover_check or child.blocks_mouse) then
                local blocked, hover = child:CheckMouseHover(x,y,trace)
                if blocked and hover then
                    if trace then
                        trace:PopWidget( "true : child blocked and hover" )
                    end
                    return blocked, hover
                elseif blocked then
                    is_blocked = true
                    break
                end
            end
        end
    end

    if self.hover_check or self.blocks_mouse then
        if self.hover_check then                
            if trace then
                trace:PopWidget( "true : hover check" )
            end
            return true, self
        end

        if self.blocks_mouse then
            if trace then
                trace:PopWidget( "true : blocks_mouse" )
            end
            return true
        end
    end

    if trace then
        trace:PopWidget( string.format( "%s: return", tostring(is_blocked)))
    end
    return is_blocked
end

function Widget:TransformToParent(x,y)
    self:UpdateTransform()
    return self.node:TransformPoint(x,y)
end

function Widget:TransformFromParent(x,y)
    self:UpdateTransform()
    return self.node:TransformPointInv(x,y)
end

function Widget:TransformFromWorld(x,y)
    self:UpdateTransform(true)
    return self.node:WorldToLocal(x or 0, y or 0, 0)
end

function Widget:TransformToWorld(x,y)
    self:UpdateTransform(true)
    if self.node then
        return self.node:LocalToWorld(x or 0, y or 0, 0)
    else
        return 0,0,0
    end
end

--takes in x and y in w space and puts it into local space
function Widget:TransformFromWidget(w, x, y)
    w:UpdateTransform(true)
    self:UpdateTransform(true)
    return self:TransformFromWorld(w:TransformToWorld(x,y))
end


function Widget:InFocusLoop()
    return self.focus_targets and next(self.focus_targets) ~= nil
end

function Widget:OnTextInput(text)
    if not self.ignore_input and self.shown then
        if not self.enabled then
            return self.blocks_mouse
         end

        if self.children then
            for k = 1, #self.children do 
                local v = self.children[k]
                if v.focus then
                    if v:OnTextInput(text) then
                        return true
                    end
                end
            end
        end

        if self.HandleTextInput and self:HandleTextInput(text) then
            return true
        end
    end
end

function Widget:OnTextEdit(text)
    if not self.ignore_input and self.shown then
        if not self.enabled then
            return self.blocks_mouse
         end

        if self.children then
            for k = 1, #self.children do 
                local v = self.children[k]
                if v.focus then
                    if v:OnTextEdit(text) then
                        return true
                    end
                end
            end
        end

        if self.HandleTextEdit and self:HandleTextEdit(text) then
            return true
        end
    end
end

function Widget:OnRawKeyDown(key)
    if not self.ignore_input and self.shown then
        if not self.enabled then
            return self.blocks_mouse
         end

        if self.children then
            for k = 1, #self.children do 
                local v = self.children[k]
                if v.focus then
                    if v:OnRawKeyDown(key) then
                        return true
                    end
                end
            end
        end

        if self.HandleRawKeyDown and self:HandleRawKeyDown(key) then
            return true
        end
    end
end

function Widget:OnRawKeyUp(key, charval)
    if not self.ignore_input and self.shown then
        if not self.enabled then
            return self.blocks_mouse
         end

        if self.children then
            for k = 1, #self.children do 
                local v = self.children[k]
                if v.focus then
                    if v:OnRawKeyUp(key, charval) then
                        return true
                    end
                end
            end
        end

        if self.HandleRawKeyUp and self:HandleRawKeyUp(key, charval) then
            return true
        end
    end
end

function Widget:OnControlDown(control, device, trace, device_id)
    if trace then
        trace:PushWidget( self )
    end

    if not self.ignore_input and self.shown then
        

        if self.enabled then

            if self.HandlePreControlDown and self:HandlePreControlDown(control, device) then
                if trace then
                    trace:PopWidget( "true: handled by self" )
                end
                return true
            end

            for k = 1, #self.children do 
                local v = self.children[k]
                if v and (v.focus or v.hover) then
                    if v:OnControlDown(control, device) then
                        if trace then
                            trace:PopWidget( "true: handled by child" )
                        end
                        return true
                    end
                end
            end

            if self.HandleControlDown and self:HandleControlDown(control, device) then
                if trace then
                    trace:PopWidget( "true: handled by self" )
                end
                return true
            end
        end

        if not self.fe:ReticuleActive() then
            if (control:Has( Controls.Digital.MENU_LEFT ) and self:OnFocusMove("left")) or
                (control:Has( Controls.Digital.MENU_RIGHT ) and self:OnFocusMove("right")) or
                (control:Has( Controls.Digital.MENU_DOWN ) and self:OnFocusMove("down")) or
                (control:Has( Controls.Digital.MENU_UP ) and self:OnFocusMove("up")) then
                --
                if trace then
                    trace:PopWidget( "true: handled by focus movement" )
                end
                return true
            end
        end

        if not self.enabled then
            if trace then
                trace:PopWidget( "false: not enabled" )
            end
            return self.blocks_mouse and ( device == "mouse" or device == "touch" )
         end


        if self.blocks_mouse and ( device == "mouse" or device == "touch" ) then
            if not control:Has( Controls.Digital.SCROLL_DOWN, Controls.Digital.SCROLL_UP ) then
                if trace then
                    trace:PopWidget( "true: blocks_mouse" )
                end
                return true
            end
        end
    end

    if trace then
        trace:PopWidget( "false" )
    end
end

function Widget:GetControlMap()
    return self.CONTROL_MAP, self
end

function Widget:OnControlUp(control, device)

    if not self.ignore_input and self.shown then
        if not self.enabled then
            return self.blocks_mouse
        end

        for k = 1, #self.children do 
            local v = self.children[k]
            if v and (v.focus or v.hover) then
                if v:OnControlUp(control, device) then
                    return true
                end
            end
        end

        local control_map, owner = self:GetControlMap()

        if control_map then
            for _,mapping in ipairs(control_map) do
                if (mapping.control and control:Has( mapping.control )) or (mapping.controls and control:Has( table.unpack( mapping.controls ))) then
                    if not mapping.test or mapping.test(owner) then
                        if mapping.fn and mapping.fn(owner) then
                            return true
                        end
                    end
                end
            end
        end

        if self.HandleControlUp and self:HandleControlUp(control, device) then
            return true
        end

        if self.blocks_mouse and ( device == "mouse" or device == "touch" ) then
            if not control:Has( Controls.Digital.SCROLL_DOWN, Controls.Digital.SCROLL_UP ) then
                return true
            end
        end
    end
end

function Widget:CollectControlHints(left, right, handled)
    handled = handled or {}
    if self.children then
        for k,v in ipairs(self.children) do
            if v.focus then
                v:CollectControlHints(left, right, handled)
            end
        end
    end

    local control_map, owner = self:GetControlMap()
    if control_map then
        for k,mapping in ipairs(control_map) do
            if mapping.hint and (not mapping.control or not handled[mapping.control]) and (not mapping.test or mapping.test(owner)) then
                if mapping.control then
                    handled[mapping.control] = owner
                end
                mapping.hint(owner, left, right)
            end
        end
    end
end

function Widget:UnblockMouse()
    self.blocks_mouse = false
    return self
end

function Widget:BlockMouse()
    self.blocks_mouse = true
    return self
end

function Widget:GetParent()
    return self.parent
end

function Widget:GetDescendant( class )
    assert( is_class( class ))
    if is_instance( self, class ) then
        return self
    elseif self.children then
        for k,v in pairs(self.children) do
            return v:GetDescendant( class )
        end
    end
    
    return nil
end

function Widget:GetAncestor( class )
    assert( is_class( class ))
    if is_instance( self, class ) then
        return self
    elseif self.parent then
        return self.parent:GetAncestor( class )
    end
    
    return nil
end

function Widget:IsAncestorOf( w )
    local parent = w.parent
    while parent do
        if parent == self then
            return true
        end
        parent = parent.parent
    end

    return false
end

function Widget:GetScreen()
    if is_instance( self, Widget.Screen ) then
        return self
    elseif self.parent then
        return self.parent:GetScreen()
    end

    return nil
end
function Widget:GetPos()
    return self.x, self.y
end

function Widget:GetWorldPos()
    return self.node:GetWorldPosition()
end

function Widget:SetScale(x,y, keep_anim)
    if not keep_anim then
        self.scale = nil
    end

    if self.sx ~= x or self.sy ~= y or x then
        self.sx, self.sy = x, y or x
        self:InvalidateBBox()
        self:MarkTransformDirty()
    end
    return self
end

--this is a scale meant to accomdate different screen layouts (TV, mobile, monitor, etc) without changing the underlying code
function Widget:SetLayoutScale(sc)
    if self.layout_scale ~= sc then
        self.layout_scale = sc
        self:InvalidateBBox()
        self:MarkTransformDirty()
    end
    return self
end

function Widget:GetScale()
    return self.sx, self.sy
end

function Widget:SetBaseScaling( val )
    -- true: base scaling applied
    -- false: base scaling NOT applied
    -- nil: base scaling applied if hanchor/vanchor set (default)

    self.base_scaling = val
    if self.children then
        for k,v in pairs(self.children) do
            v:SetBaseScaling( val )
        end
    end
    self:MarkTransformDirty()
    self:InvalidateBBox()
    return self
end

function Widget:UpdateTransform(world_space)
    if not self.dirtytransform and not self.ancestordirty then
        return
    end

    if world_space and self.ancestordirty then
        if self.parent then
            self.parent:UpdateTransform( true )
        end
        self.ancestordirty = false
    end

    self.dirtytransform = false

    local base_scale = 1
    if self.fe then
        base_scale = self:GetFE():GetBaseWidgetScale()
    end

    local parent_scale_x, parent_scale_y
    if self.base_scaling ~= false and (self.hanchor or self.vanchor or self.base_scaling) then
        parent_scale_x, parent_scale_y = base_scale, base_scale
    else
        parent_scale_x, parent_scale_y = 1, 1
    end

    if self.hanchor == "fill" or self.vhanchor == "fill" then
        
        local w, h = RES_X, RES_Y
        
        if self.fe then
            w, h = self.fe:GetScreenDims()
        end
        
        if self.hanchor == "fill" then
            parent_scale_x = w/RES_X
        end
        
        if self.vanchor == "fill" then
            parent_scale_y = h/RES_Y
        end
    end

    local trans_scale = 1
    if self.base_scaling ~= false and (self.hanchor or self.vanchor or self.base_scaling) then
        trans_scale = base_scale
    else
        trans_scale = 1
    end

    if self.hreg and self.vreg then
        self:SetRegistration(self.hreg, self.vreg)
    end

    self.resolved_transform = self.resolved_transform or {}
    local x, y, z, sx, sy, sz, rx, ry, rz = self.x*trans_scale+self.anchorx + trans_scale*(self.regx or 0), self.y*trans_scale+self.anchory + trans_scale*(self.regy or 0), self.z, self.layout_scale*parent_scale_x*self.sx, self.layout_scale*parent_scale_y*self.sy, 1.0, 0, 0, self.r
    
    if self.resolved_transform.x ~= x or self.resolved_transform.y ~= y or self.resolved_transform.z ~= z or self.resolved_transform.sx ~= sx or self.resolved_transform.sy ~= sy or self.resolved_transform.sz ~= sz or self.resolved_transform.rx ~=  rx or self.resolved_transform.ry ~=  ry or self.resolved_transform.rz ~=  rz then
        self.node:SetTransform(x, y, z, sx, sy, sz, rx, ry, rz)
        self.resolved_transform.x = x
        self.resolved_transform.y = y
        self.resolved_transform.z = z
        self.resolved_transform.sx = sx
        self.resolved_transform.sy = sy
        self.resolved_transform.sz = sz
        self.resolved_transform.rx = rx
        self.resolved_transform.ry = ry
        self.resolved_transform.rz = rz
    end
end

function Widget:Offset( dx, dy )
    self:SetPos( self.x + (dx or 0), self.y + (dy or 0) )
    return self
end
    
function Widget:SetPos(x,y,z)
    if self.x ~= x or self.y ~= y or self.z ~= (z or 0)then
        self.x, self.y, self.z = x or self.x, y or self.y, z or self.z
        self:InvalidateBBox()
        self:MarkTransformDirty()
    end
    return self
end

function Widget:GetRotation()
    return self.r or 0
end

function Widget:SetRotation(r)
    if self.r ~= r then
        self.r = r
        self:InvalidateBBox()
        self:MarkTransformDirty()
    end

    return self
end

function Widget:SetFocus()
    self:GetFE():SetFocusWidget( self )
    return self
end

function Widget:ClearFocus()
    if self.focus then
        self:GetFE():ClearFocusWidget()
    end
    return self
end

function Widget:HasFocus()
    return self.focus == true
end

-- This should only be called from FrontEnd:SetFocusWidget.
-- focus_widget : the terminal Widget receiving focus.
function Widget:GainFocus( focus_widget )

    if self.parent then
        self.parent:GainFocus( focus_widget or self )
    end

    if self.focus_group == FOCUS_GROUP.LAST_FOCUS then
        self.default_focus = focus_widget
    end

    if not self.focus then
        self.focus = true
        if self.focus_sound then
            AUDIO:PlayEvent( self.focus_sound )
        end

        self:OnGainFocus()
    end
    return self
end

-- This should only be called from FrontEnd:ClearFocusWidget.
-- focus_widget : the terminal Widget receiving focus, if switching focus.
function Widget:LoseFocus( focus_widget )
    if self.focus and (focus_widget == nil or not self:IsAncestorOf( focus_widget )) then
        self.focus = false

        self:OnLoseFocus()

        if self.parent then
            self.parent:LoseFocus( focus_widget )
        end
    end
end

function Widget:SetHoverSound( hover_sound )
    self.hover_sound = hover_sound
    return self
end

function Widget:SetHover(from_child)
    
    if not self.hover then
        if self.parent then
            self.parent:SetHover(self)
        end
        self.hover = true
        if self.hover_sound then
            AUDIO:PlayEvent( self.hover_sound )
        end
        self:OnGainHover()
    end

    if self.children then
        for k = 1, #self.children do 
            local v = self.children[k]
            if v.hover and v ~= from_child then
                v:ClearHover()
            end
        end
    end
end

function Widget:ClearHover()
    if self.hover then
        self.hover = false
        self:OnLoseHover()
        if self.children then
            for k = 1, #self.children do 
                local v = self.children[k]
                if v.hover then 
                    v:ClearHover()
                end
            end
        end
    end
end

function Widget:SetFocusSound( focus_sound )
    self.focus_sound = focus_sound
    return self
end

function Widget:GetDeepestHover()
    if self.hover then

        if self.children then
            for k = 1, #self.children do 
                local v = self.children[k]
                if v.hover then
                    return v:GetDeepestHover()
                end
            end
        end
    else
        return nil
    end

    return self
end

local dir_op =
{
    left = "right",
    right = "left",
    up = "down",
    down = "up",
    ["next"] = "prev",
    prev = "next",
}

function Widget:ClearFocusDirs()
    self.focus_targets = nil
end

function Widget:SetFocusDir(direction, target, bidi, force_visible)
    self.focus_targets = self.focus_targets or {}
    self.focus_targets[direction] = target
    if bidi and target then
        target:SetFocusDir(dir_op[direction], self)
    end
end

function Widget:GetFocusDir( direction )
    if self.focus_lock then
        return self
    elseif self.focus_targets then
        local w = self.focus_targets[direction]
        if w and w.removed then
            self.focus_targets[direction] = nil
        else
            return w
        end
    end
end

local function IsFocusDir( dir, from, to )
    local from_x1, from_y1, from_x2, from_y2 = from:GetWorldBoundingBox()
    local to_x1, to_y1, to_x2, to_y2 = to:GetWorldBoundingBox()
    local is_dir

    if (dir == "right" and to_x1 > from_x1 and to_x2 > from_x2) then
        return CalculateRectDistance( from_x1, from_y1, from_x2, from_y2,
                                      to_x1, to_y1, to_x2, to_y2 )

    elseif dir == "left" and to_x1 < from_x1 and to_x2 < from_x2 then
        return CalculateRectDistance( from_x1, from_y1, from_x2, from_y2,
                                      to_x1, to_y1, to_x2, to_y2 )

    elseif dir == "up" and to_y1 > from_y1 and to_y2 > from_y2 then
        return CalculateRectDistance( from_x1, from_y1, from_x2, from_y2,
                                      to_x1, to_y1, to_x2, to_y2 )

    elseif dir == "down" and to_y1 < from_y1 and to_y2 < from_y2 then
        return CalculateRectDistance( from_x1, from_y1, from_x2, from_y2,
                                      to_x1, to_y1, to_x2, to_y2 )
    end
end

function Widget:SetFocusable( focusable )
    self.can_focus = focusable ~= false
    return self
end

function Widget:SetFocusGroup( policy, default_widget )
    assert( policy == nil or IsEnum( policy, FOCUS_GROUP ))
    self.focus_group = policy

    if policy == FOCUS_GROUP.ALWAYS_DEFAULT then
        assert( self:IsAncestorOf( default_widget ))
        self.default_focus = default_widget
    end
    return self
end

-- Focus lock prevents focus from leaving this Widget or its children via OnFocusMove
-- by forwarding focus to itself.
function Widget:SetFocusLock( focus_lock )
    self.focus_lock = focus_lock
    return self
end

function Widget:HasFocusLock()
    return self.focus_lock
end

function Widget:FindFocusWidget( dir, from, ignore_child )
    assert( dir and from )
    --not self.enabled or
    if not self.shown or self.ignore_input or self.dragging then
        return
    end

    if self.can_focus then        
        local score = IsFocusDir( dir, from, self )
        if score or from:GetFocusDir( dir ) == self then
            -- print( from, dir, self, score )
            return self, score
        end

    else
        local best_widget, best_score
        local default_focus
        if self.default_focus and not self.default_focus.removed and self.default_focus.shown and not self:IsAncestorOf( from )then
            default_focus = self.default_focus
        end

        for i, child in ipairs( self.children ) do
            if child ~= ignore_child then
                local widget, score = child:FindFocusWidget( dir, from )
                if widget and (best_score == nil or score < best_score) then
                    best_widget, best_score = widget, score
                end
            end
        end

        if best_widget then
            return default_focus or best_widget, best_score
        end
    end

    -- Found nothing, crawl up the parent.
    if ignore_child ~= nil and self.parent and not self.is_screen then
        local to = self:GetFocusDir( dir ) or self.parent
        if to ~= self then
            assert( not self:IsAncestorOf( to ))
            return to:FindFocusWidget( dir, from, self )
        end
    end
end

function Widget:OnFocusMove( dir )
    if self == self:GetFE():GetFocusWidget() then
        local to = self:GetFocusDir( dir ) or self.parent
        -- print( "OnFocusMove", self, dir, to )
        if to == self then
            self:OnFocusNudge( dir )
            return true
        else
            local focus, score = to:FindFocusWidget( dir, self, self )
            if focus then
                -- print( "\tNew Focus:", focus, score )
                focus:SetFocus()
                return true
            else
                self:OnFocusNudge( dir )
                return false
            end
        end
    end
end

function Widget:OnGainFocus()
end

function Widget:OnLoseFocus()
end

function Widget:OnGainHover()
end

function Widget:OnLoseHover()
end

-- When trying to navigate in a direction and there are no widgets to move focus to, animate this widget to show input happened
function Widget:OnFocusNudge( direction )
    local distance = 15
    local duration = 0.1
    if not self.focus_nudging then
        self.focus_nudging = true
        self.focus_nudge_x, self.focus_nudge_y = self:GetPos()
        if direction == "prev" or direction == "left" then
            self.focus_nudge_x_target = self.focus_nudge_x-distance
            self.focus_nudge_y_target = self.focus_nudge_y
        elseif direction == "next" or direction == "right" then
            self.focus_nudge_x_target = self.focus_nudge_x+distance
            self.focus_nudge_y_target = self.focus_nudge_y
        elseif direction == "down" then
            self.focus_nudge_x_target = self.focus_nudge_x
            self.focus_nudge_y_target = self.focus_nudge_y-distance
        elseif direction == "up" then
            self.focus_nudge_x_target = self.focus_nudge_x
            self.focus_nudge_y_target = self.focus_nudge_y+distance
        end
        self:MoveTo( self.focus_nudge_x_target, self.focus_nudge_y_target, duration, easing.outQuad, function() self:MoveTo( self.focus_nudge_x, self.focus_nudge_y, duration, easing.inOutQuad, function() self.focus_nudging = false end ) end )
    end
    return self
end

function Widget:RunUpdater(updater, _guard)
    assert(_guard == nil, "Should you be using a series node, Ricardo?")
    self.running_updaters = self.running_updaters or {}
    table.insert(self.running_updaters, updater)
    self:TestUpdateStart()
    return updater
end

function Widget:StopUpdater(updater)
    if table.arrayremove(self.running_updaters, updater) then
        updater:Stop()
    end
end

function Widget:StartCoroutine(fn, ...)
    local coro = Coro(fn, ...)
    self.coros = self.coros or {}
    table.insert( self.coros, coro )
    self:TestUpdateStart()
    return coro
end

function Widget:StopCoroutine( coro )
    coro:Stop()
    table.arrayremove( self.coros, coro )
end

function Widget:StopListeningToObject(object)
    if self.listening_to and self.listening_to[object] then
        object:RemoveListener(self)
        self.listening_to[object] = nil
    end
end

function Widget:ListenToObject(object, fn)
    if object.ListenForAny then
        self.listening_to = self.listening_to or {}
        
        if not self.listening_to[object] then
            self.listening_to[object] = true
            object:ListenForAny(self, fn)
        end
    end
end

function Widget:Remove()
    assert( not self.removed, self._classname )

    self:ClearFocus()
    self:ClearHover()
    
    while #self.children > 0 do
        self.children[ #self.children ]:Remove()
    end

    -- Detach from parent first: access to children etc. must still be valid for
    -- OnRemoved handlers.
    self.parent:RemoveChild(self)
    assert( self.parent == nil )


    -- Destroy widget properties.
    self.removed = true

    if self.model then
        self.model:Clear()
    end
    self.model = nil
    
    self.node:Clear()
    RecycleSceneGraphNode(self.node)

    if self.satval then
        RecycleShaderConstant(self.satval)
        self.satval = nil
    end

    if self.tint_mult_constant then
        RecycleShaderConstant(self.tint_mult_constant)
        self.tint_mult_constant = nil
    end

    self.node = nil
    if self.fe then
        self.fe.dirtytransforms[self] = nil
    end

    if self.fe and self.is_updating then
        self.is_updating = false
        self.fe:StopUpdatingWidget(self)    
    end
    
end

function Widget:IsShown()
    return self.shown == true
end

function Widget:IsVisible()
    return self.visible == true
end

function Widget:SetShown( is_shown )
    if not is_shown and self.shown then
        self:Hide()
    elseif is_shown and not self.shown then
        self:Show()
    end
    return self
end

function Widget:SetRenderTerminator(v)
    self.node:SetRenderTerminator(v)
    return self
end

function Widget:Show()
    
    if not self.shown then
        self.shown = true
        if self.node then
            self.node:Show()
        end
    
        if self.OnShown then
            self:OnShown()
        end

        self:InvalidateBBox()

        self:UpdateViz()

    end
    return self
end

function Widget:Hide()
    if not self.shown then
        return self
    end

    self.shown = false
    if self.node then
        self.node:Hide()
    end
    self:ClearFocus()
    self:ClearHover()

    if self.OnHidden then
        self:OnHidden()
    end

    self:UpdateViz()

    self:InvalidateBBox()
    return self

end

function Widget:SetForceViz()
    self.force_viz = true
    self:UpdateViz()
end

function Widget:UpdateViz()

    local viz
    if not self.shown then 
        viz = false
    elseif not self.parent then 
        viz = self.force_viz == true
    else
        viz = self.parent:IsVisible()
    end
    
    if viz ~= self.visible then
        self.visible = viz
        
        if self.OnVizChange then
            self:OnVizChange(self.visible)
        end

        if self.children then
            for k,child in ipairs(self.children) do
                child:UpdateViz()
            end
        end
    end
end


function Widget:__tostring()
    if self.name then
        return string.format("%s (%s)", self._classname, self.name)
    else
        return self._classname
    end
end

function Widget:StopAll()
    if self.move then
        self.move = nil
    end
    if self.scale then
        self.scale = nil
    end
    if self.colour then
        self.colour = nil
    end
    if self.blink then
        self.blink = nil
    end
    return self
end

function Widget:MoveTo(x, y, t, ease, fn)
    self.move =
    {
        sx = self.x,
        sy = self.y,
        dx = x or self.x,
        dy = y or self.y,
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:StopMoveTo()
    if self.move then
        self.move = nil
    end
end

function Widget:MoveXTo(x, t, ease, fn)
    self.move =
    {
        sx = self.x,
        sy = self.y,
        dx = x,
        dy = self.y,
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:CurveTo(x, y, control_x, control_y, t, ease, fn)
    self.move =
    {
        sx = self.x,
        sy = self.y,
        cx = control_x,
        cy = control_y,
        dx = x,
        dy = y,
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:SizeTo(start_w, end_w, start_h, end_h, t, ease, fn)
    local w,h = self:GetSize()
    if start_w == nil then
        start_w = w
    end
    if end_w == nil then
        end_w = w
    end
    if start_h == nil then
        start_h = h
    end
    if end_h == nil then
        end_h = h
    end
    self.size =
    {
        start_w = start_w,
        end_w = end_w,
        start_h = start_h,
        end_h = end_h,
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:ScissorTo( start_scissor, end_scissor, t, ease, fn)
    self.scissor =
    {
        start_scissor = start_scissor,
        end_scissor = end_scissor,
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:ScaleTo(start_scale, end_scale, t, ease, fn)
    assert( t and end_scale )

    self.scale =
    {
        start_scale = start_scale or self:GetScale(),
        end_scale = end_scale,
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:ColourTo(r1,g1,b1,a1, r2,g2,b2,a2, t, ease, fn)
    self.colour =
    {
        start_colour = {r1 or self.tint_r or 1, g1 or self.tint_g or 1, b1 or self.tint_b or 1, a1 or self.tint_a or 1 },
        end_colour = {r2,g2,b2,a2},
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:ColourFromCurrentTo(r2,g2,b2,a2, t, ease, fn)
    self.colour =
    {
        start_colour = {self.tint_r or 1,self.tint_g or 1, self.tint_b or 1, self.tint_a or 1},
        end_colour = {r2,g2,b2,a2},
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:AlphaTo(a, t, ease, fn)
    if a > 0 then
        self:Show()
    end

    self.colour =
    {
        start_colour = {self.tint_r or 1,self.tint_g or 1, self.tint_b or 1, self.tint_a or 1},
        end_colour = {self.tint_r or 1,self.tint_g or 1, self.tint_b or 1, a or 1},
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:Pulse( from, to, speed )
    self.pulse = 
    {
        alpha = from,
        from = from,
        to = to,
        increasing = from < to,
        speed = speed
    }
    self:TestUpdateStart()
    return self
end

function Widget:PulseColour( r1,g1,b1,a1, r2,g2,b2,a2, speed, ease )

    self.pulse_colour = 
    {
        start_colour = {r1 or self.tint_r,g1 or self.tint_g, b1 or self.tint_b, a1 or self.tint_a },
        end_colour = {r2,g2,b2,a2},
        increasing = true,
        t = 0,
        ease = ease or easing.linear,
        speed = speed or 0.1
    }
    self:TestUpdateStart()
    return self
end

function Widget:StopPulse()
    self.pulse = nil
    self.pulse_colour = nil
end

function Widget:RotateTo(r, t, ease, fn)
    self.rotation = 
    {
        start_rotation = self.r,
        end_rotation = r,
        total_t = t,
        t = 0,
        ease = ease or easing.linear,
        fn = fn,
    }
    self:TestUpdateStart()
    return self
end

function Widget:StopSpin( )
    if self.spin then
        self.spin = nil
    end
    return self
end

function Widget:RotateIndefinitely( speed )
    self.spin = 
    {
        rotation = self.r,
        speed = speed
    }
    self:TestUpdateStart()
    return self
end

function Widget:Blink( period_t, max_count, blink_fn, fn )
    self.blink =
    {
        on = true,
        t = 0,
        period_t = period_t,
        count = 0,
        max_count = max_count,
        blink_fn = blink_fn or self.SetShown,
        fn = fn
    }
    self:TestUpdateStart()
    return self
end

function Widget:ShowFloater( txt, x, y, clr, dy, duration, sz )
    local label = self:AddChild( Widget.Label( "title", sz or 72 ) )

    label:SetText( txt ):SetGlyphColour( table.unpack(clr or Colours.White) )
    label:Show()
    label:SetHiddenBoundingBox(true)
    label:SetPos( x, y )
    label:MoveTo( x, y+(dy or 200), duration or 1, easing.outQuad,
        function()
            label:Remove()
        end )

   return label
end

function Widget:SetUpdateSpeed( speed )
    self.update_speed = speed
end

function Widget:GetUpdateSpeed()
    local update_speed = 1.0
    local w = self
    while w do
        update_speed = update_speed * (w.update_speed or 1.0)
        w = w.parent
    end
    return update_speed
end

function Widget:OnUpdate(dt)

    dt = dt * self:GetUpdateSpeed()

    if self.colour then
        
        self.colour.t = self.colour.t + dt
        
        if self.colour.t > self.colour.total_t then
            self:SetTintColour(table.unpack(self.colour.end_colour))
            local fn = self.colour.fn
            self.colour = nil
            if fn then
                fn()
            end
        else
            local r = self.colour.ease(self.colour.t, self.colour.start_colour[1], self.colour.end_colour[1] - self.colour.start_colour[1], self.colour.total_t)
            local g = self.colour.ease(self.colour.t, self.colour.start_colour[2], self.colour.end_colour[2] - self.colour.start_colour[2], self.colour.total_t)
            local b = self.colour.ease(self.colour.t, self.colour.start_colour[3], self.colour.end_colour[3] - self.colour.start_colour[3], self.colour.total_t)
            local a = self.colour.ease(self.colour.t, self.colour.start_colour[4], self.colour.end_colour[4] - self.colour.start_colour[4], self.colour.total_t)
            
            self:SetTintColour(r,g,b,a)
        end

    end

    if self.uv_speed_set then
        self:SetUVOffset((self.u_offset or 0) + self.u_speed*dt, (self.v_offset or 0) + self.v_speed*dt)
    end

    if self.move then
        self.move.t = self.move.t + dt

        if self.move.t > self.move.total_t then
            self:SetPos(self.move.dx,self.move.dy)
            local fn = self.move.fn
            self.move = nil
            if fn then
                fn()
            end
        else
            local x,y
            if self.move.cx and self.move.cy then
                -- curve to target
                local factor = self.move.ease(self.move.t, 0.0, 1.0, self.move.total_t)

                -- bezier path: (not a constant speed bezier, so easying may be a bit weird!
                x, y = BezierEvaluate(factor, self.move.sx, self.move.sy, self.move.cx, self.move.cy, self.move.dx, self.move.dy);
            else
                x = self.move.ease(self.move.t, self.move.sx, self.move.dx - self.move.sx, self.move.total_t)
                y = self.move.ease(self.move.t, self.move.sy, self.move.dy - self.move.sy, self.move.total_t)
            end
            self:SetPos(x,y)
        end
    end

    if self.scale then
        self.scale.t = self.scale.t + dt

        if self.scale.t > self.scale.total_t then
            self:SetScale(self.scale.end_scale,self.scale.end_scale, true)
            local fn = self.scale.fn
            self.scale = nil
            if fn then
                fn()
            end
        else
            local s = self.scale.ease(self.scale.t, self.scale.start_scale, self.scale.end_scale - self.scale.start_scale, self.scale.total_t)
            self:SetScale(s,s,s)
        end
    end

    if self.rotation then
        self.rotation.t = self.rotation.t + dt

        if self.rotation.t > self.rotation.total_t then
            self:SetRotation(self.rotation.end_rotation, self.rotation.start_rotation, true)
            local fn = self.rotation.fn
            self.rotation = nil
            if fn then
                fn()
            end
        else
            local rot = self.rotation.ease(self.rotation.t, self.rotation.start_rotation, self.rotation.end_rotation - self.rotation.start_rotation, self.rotation.total_t)
            self:SetRotation(rot)
        end
    end

    if self.spin then
        self.spin.rotation = self.spin.rotation + self.spin.speed
        self:SetRotation( self.spin.rotation )
    end 

    if self.size then
        self.size.t = self.size.t + dt

        if self.size.t > self.size.total_t then
            self:SetSize(self.size.end_w,self.size.end_h)
            local fn = self.size.fn
            self.size = nil
            if fn then
                fn()
            end
        else
            local w = self.size.ease(self.size.t, self.size.start_w, self.size.end_w - self.size.start_w, self.size.total_t)
            local h = self.size.ease(self.size.t, self.size.start_h, self.size.end_h - self.size.start_h, self.size.total_t)
            self:SetSize(w,h)
        end
    end

    if self.scissor then
        self.scissor.t = self.scissor.t + dt

        if self.scissor.t > self.scissor.total_t then
            self:SetScissor( self.scissor.end_scissor.x, self.scissor.end_scissor.y, self.scissor.end_scissor.w, self.scissor.end_scissor.h )
            local fn = self.scissor.fn
            self.scissor = nil
            if fn then
                fn()
            end
        else
            local x = self.scissor.ease(self.scissor.t, self.scissor.start_scissor.x, self.scissor.end_scissor.x - self.scissor.start_scissor.x, self.scissor.total_t)
            local y = self.scissor.ease(self.scissor.t, self.scissor.start_scissor.y, self.scissor.end_scissor.y - self.scissor.start_scissor.y, self.scissor.total_t)
            local w = self.scissor.ease(self.scissor.t, self.scissor.start_scissor.w, self.scissor.end_scissor.w - self.scissor.start_scissor.w, self.scissor.total_t)
            local h = self.scissor.ease(self.scissor.t, self.scissor.start_scissor.h, self.scissor.end_scissor.h - self.scissor.start_scissor.h, self.scissor.total_t)
            self:SetScissor( x, y, w, h )
        end
    end
    
    if self.pulse then
        if self.pulse.increasing == true and self.pulse.alpha >= self.pulse.to then
            self.pulse.increasing = false
        end
        if self.pulse.increasing == false and self.pulse.alpha <= self.pulse.from then
            self.pulse.increasing = true
        end
        if self.pulse.increasing then
            self.pulse.alpha = self.pulse.alpha + self.pulse.speed*3
        else
            self.pulse.alpha = self.pulse.alpha - self.pulse.speed
        end
        --print( "alpha", self.pulse.increasing and "increasing" or "decreasing", " at", self.pulse.alpha )
        self:SetTintAlpha( self.pulse.alpha )
    end
    
    if self.pulse_colour then
        if self.pulse_colour.increasing == true and self.pulse_colour.t >= self.pulse_colour.speed then
            self.pulse_colour.increasing = false
        end
        if self.pulse_colour.increasing == false and self.pulse_colour.t <= 0 then
            self.pulse_colour.increasing = true
        end
        if self.pulse_colour.increasing then
            self.pulse_colour.t = self.pulse_colour.t + 3*dt
        else
            self.pulse_colour.t = self.pulse_colour.t - dt
        end
        local r = self.pulse_colour.ease(self.pulse_colour.t, self.pulse_colour.start_colour[1], self.pulse_colour.end_colour[1] - self.pulse_colour.start_colour[1], self.pulse_colour.speed)
        local g = self.pulse_colour.ease(self.pulse_colour.t, self.pulse_colour.start_colour[2], self.pulse_colour.end_colour[2] - self.pulse_colour.start_colour[2], self.pulse_colour.speed)
        local b = self.pulse_colour.ease(self.pulse_colour.t, self.pulse_colour.start_colour[3], self.pulse_colour.end_colour[3] - self.pulse_colour.start_colour[3], self.pulse_colour.speed)
        local a = self.pulse_colour.ease(self.pulse_colour.t, self.pulse_colour.start_colour[4], self.pulse_colour.end_colour[4] - self.pulse_colour.start_colour[4], self.pulse_colour.speed)
        self:SetTintColour(r,g,b,a)
    end

    if self.blink then
        self.blink.t = self.blink.t + dt
        if self.blink.t > self.blink.period_t then
            self.blink.on = not self.blink.on
            self.blink.blink_fn( self, self.blink.on )
            self.blink.t = self.blink.t - self.blink.period_t
            if self.blink.on then
                self.blink.count = self.blink.count + 1
                if self.blink.count >= self.blink.max_count then
                    local fn = self.blink.fn
                    self.blink.blink_fn( self, true )
                    self.blink = nil
                    if fn then
                        fn()
                   end
                end
            end
        end
    end

    if self.coros and #self.coros > 0 then
        local i = 1
        while i <= #self.coros do
            local coro = self.coros[i]
            if not coro:Update(dt) then
                table.remove( self.coros, i )
            else
                i = i + 1
            end
        end
    end

    if self.running_updaters and #self.running_updaters > 0 then
        local i = 1
        while i <= #self.running_updaters do
            local updater = self.running_updaters[i]
            updater:Update(dt)
            if updater:IsDone() then
                -- Updating could have affected the updaters list
                if self.running_updaters[i] == updater then
                    table.remove( self.running_updaters, i )
                end
            else
                i = i + 1
            end
        end
    end

    --there's no reason to be updating. stop it.
    if not self:ShouldBeUpdating() then
        self.is_updating = false
        if self.fe then
            self.fe:StopUpdatingWidget(self)
        end
    end
end


function Widget:ShouldBeUpdating()
    return self.colour or
        self.move or
        self.scale or
        self.rotation or
        self.spin or
        self.size or
        self.scissor or
        self.pulse or
        self.pulse_colour or
        self.uv_speed_set or
        self.blink or
        (self.update_requested and self.OnUpdate ~= Widget.OnUpdate) or
        (self.coros and #self.coros > 0) or
        (self.running_updaters and #self.running_updaters > 0)
end

function Widget:StartUpdating()
    self.update_requested = true
    self:TestUpdateStart()
end

function Widget:StopUpdating()
    self.update_requested = false

    if not self:ShouldBeUpdating() then
        self.is_updating = false
        if self.fe then
            self.fe:StopUpdatingWidget(self)
        end
    end
end

function Widget:SetUVSpeed(u_speed, v_speed)
    u_speed = u_speed or 0
    v_speed = v_speed or 0
    if not self.uv_speed_set and (u_speed ~= 0 or v_speed ~= 0) then
        self.model:SetTextureSampler(0, "linear","linear","linear","wrap", "wrap", true)
    end

    self.u_speed, self.v_speed = u_speed or 0, v_speed or 0
    self.uv_speed_set = u_speed ~= 0 or v_speed ~= 0
    self:TestUpdateStart()
end

function Widget:SetUVOffset(u, v)
    u = u or 0
    v = v or 0
    self.u_offset, self.v_offset = u, v
    if not self.uv_offset_constant then
        self.uv_offset_constant = engine.rendering.ShaderConstantValue("UVOFFSET", u, v )
        self.model:SetShaderConstant(self.uv_offset_constant)
    else
        self.uv_offset_constant:SetValue(u, v)
    end
end

function Widget:ResetUVAnim()
    self:SetUVOffset(0,0)
    self:SetUVSpeed(0,0)
    return self
end

function Widget:RemoveChild(child)
    for k = 1, #self.children do 
        local v = self.children[k]
        if v == child then
            if child.listening_to then
                for k,v in pairs(child.listening_to) do
                    k:RemoveListener( child )
                end
                child.listening_to = nil
            end
            
            child:ClearFocus()
            child:ClearHover()

            child:StopListeningForAllGlobalEvents()
            child:StopAll()
             
            if child.is_updating and self.fe then
                child.is_updating = false
                self.fe:StopUpdatingWidget(child)
            end

            if child.OnRemoved then
                child:OnRemoved()
            end

            child:CancelTasks()

            if child.coros then
                for k,v in pairs(child.coros) do
                    v:Stop()
                end
            end

            if child.running_updaters then
                table.clear( child.running_updaters )
            end

            child.parent = nil
            table.remove(self.children, k)
            child.node:Orphan()

            self:InvalidateBBox()
            child:UpdateViz()
            return
        end
    end
    error() -- Stop removing children that don't exist.
end

-- Removes from current parent and attaches to a new parent while preserving world bounds.
function Widget:Reparent( new_parent )
    
    assert( new_parent )
    assert( self.parent )

    self:ClearFocus()

    local x, y = new_parent:TransformFromWidget( self, 0, 0 )
    local old_sx, old_sy = self:GetResolvedScale()

    local idx = table.arrayfind( self.parent.children, self )
    assert( idx )
    table.remove( self.parent.children, idx )
    self.node:Orphan()
    self.parent = nil

    new_parent:AddChild( self )

    local new_sx, new_sy = self:GetResolvedScale()
    local adj_sx, adj_sy = old_sx / new_sx, old_sy / new_sy
    self:SetPos( x, y ):SetScale( self.sx * adj_sx, self.sy * adj_sy )
    return self
end

function Widget:SendToBack()
    assert( table.arrayremove( self.parent.children, self ))
    table.insert( self.parent.children, 1, self )
    self.node:MoveToBack()
    return self
end

function Widget:SendToFront()
    assert( table.arrayremove( self.parent.children, self ))
    table.insert( self.parent.children, self )
    self.node:MoveToFront()
    return self
end

function Widget:SendBehind( widget )
    assert( widget.node )
    assert( table.arrayremove( self.parent.children, self ))
    local idx = table.arrayfind( self.parent.children, widget )
    table.insert( self.parent.children, idx, self )
    self.node:MoveBehind( widget.node )
    return self
end

function Widget:ArrangeChildrenByX( reverse )
    local children = shallowcopy(self.children)
    table.sort(children, function(a,b) if reverse then return a.x > b.x else return a.x < b.x end end)
    for k = 2, #children do
        children[k].node:MoveBehind(children[k-1].node)
    end
end

function Widget:ArrangeChildrenByY()
    local children = shallowcopy(self.children)
    table.sort(children, function(a,b) return a.y < b.y end)
    for k = 2, #children do
        children[k].node:MoveBehind(children[k-1].node)
    end
end

function Widget:SetFE(fe)
    
    if self.fe then
        assert( self.fe == fe )
        return
    end

    self.fe = fe

    if self.hanchor or self.vanchor then
        self:UpdateAnchors() -- Dirtys transform internally.
    else
        self:MarkTransformDirty()
    end

    if self.children then
        for k,v in pairs(self.children) do
            v:SetFE(fe)
        end 
    end
    
    if self.is_updating then
        self:GetFE():StartUpdatingWidget(self)
    end


    return self
end

function Widget:GetFE()
    assert(self.fe)
    return self.fe 
end

function Widget:SetIndex(idx)
    self.node:SetIndex(idx)
    return self
end

function Widget:MarkAncestorTransformDirty()
    if not self.ancestordirty then
        self.ancestordirty = true
        if self.children then
            for k,v in pairs(self.children) do
                if not v.ancestordirty then
                    v:MarkAncestorTransformDirty()
                end
            end
        end
    end
end

function Widget:MarkTransformDirty()

    if not self.dirtytransform then
        self.dirtytransform = true
        if self.fe and not self.removed then
            self.fe:AddDirtyTransformWidget(self)
        end
        if self.children then
            for k,v in pairs(self.children) do
                v:MarkAncestorTransformDirty()
            end
        end
    end
end

function Widget:CollectDirtyDescendants()
    
    if self.fe and not self.removed then
        self.fe:AddDirtyTransformWidget(self)
    end

    if self.children then
        for k,v in pairs(self.children) do
            v:CollectDirtyDescendants()
        end
    end
end

function Widget:CountChildren()
    local count = 0
    if self.children then
        count = count + #self.children
        for i, child in ipairs( self.children ) do
            count = count + child:CountChildren()
        end
    end
    return count
end

function Widget:SwapChildOrder(c1, c2)
    if self.children then
        local idx1 = table.arrayfind(self.children, c1)
        local idx2 = table.arrayfind(self.children, c2)
        if idx1 and idx2 then
            self.children[idx1], self.children[idx2] = self.children[idx2], self.children[idx1]
            self.node:SwapChildOrder(idx1-1, idx2-1)
            c1:MarkTransformDirty()
            c2:MarkTransformDirty()
        end
    end
end

function Widget:IsEmpty()
    return not self.children or #self.children == 0
end

function Widget:HasVisibleChildren()
    if self.children then
        for i, child in ipairs( self.children ) do
            if child:IsShown() then return true end
        end
        return false
    end
    return false
end

function Widget:Children()
    return ipairs( self.children or table.empty )
end

function Widget:AddChild(child, idx)
    assert( child ~= self )
    assert( child.parent == nil, "widget already has parent: "..tostring(child.parent) )
    assert( not child.removed )

    self:InvalidateBBox()

    if self.fe then
        child:SetFE(self.fe)
    end

    child:CollectDirtyDescendants()

    child.parent = self
    child.hover = false
    child.focus = false

    if self.children == table.empty then
        self.children = {}
    end

    if idx then
        idx = math.min(idx, #self.children+1)
        table.insert(self.children, idx, child)
        self.node:AddChild(child.node, idx-1 ) -- 0-based
    else
        table.insert(self.children, child)
        self.node:AddChild(child.node) -- 0-based
    end

    if child.OnAdded then
        child:OnAdded()
    end
    
    if self.resolved_tint_r then
        child:ApplyTintFromParent(self.resolved_tint_r, self.resolved_tint_g, self.resolved_tint_b, self.resolved_tint_a)
    end

    if self.resolved_sat then
        child:ApplySaturationFromParent(self.resolved_sat)
    end

    if self.ancestordirty or self.dirtytransform then
        child:MarkAncestorTransformDirty()
    end

    child:UpdateViz()    
    return child
end

function Widget:DestroyAllChildren()
    while #self.children > 0 do
        self.children[ #self.children ]:Remove()
    end
    table.clear( self.children )

    self:InvalidateBBox()
    self:MarkTransformDirty()

    return self
end

function Widget:IsUpdating()
    return self.is_updating
end

function Widget:TestUpdateStart()
    
    assert( not self.removed )
    if self:ShouldBeUpdating() and not self.is_updating then
        self.is_updating = true
        if self.fe then
            self.fe:StartUpdatingWidget(self)
        end
    end
    return self
end

function Widget:SetBlendMode(mode)
    if self.model then
        self.model:SetBlendMode(mode)
    end
    return self
end

function Widget:SetEffect( effect )
    if self.model then
        self.model:SetEffect( effect )
    end
    return self
end

function Widget:SetHiddenBoundingBox( hidden )
    if hidden then
        self.hover_check = false
        self.blocks_mouse = false
    end
    self.hidden_bb = hidden -- If true this widget's BB will not contribute to the parent's BB.
    return self
end

function Widget:GetBoundingBoxArea()
    local x1,y1,x2,y2 = self:GetBoundingBox()

    return (x2-x1) * (y2-y1)
end

function Widget:GetBoundingBox()
    if self.bboxcache.valid then
        return self.bboxcache.x1, self.bboxcache.y1, self.bboxcache.x2, self.bboxcache.y2 
    else
        local x1,x2,y1,y2
        if self.children then
            for k = 1, #self.children do 
                local v = self.children[k]
                if v.shown and not v.hidden_bb then
                    local cx1, cy1, cx2, cy2 = v:GetBoundingBox()
                    if cx1 and cy1 and cx2 and cy2 then

                        local px1, py1 = v:TransformToParent(cx1, cy1)
                        local px2, py2 = v:TransformToParent(cx2, cy2)
                        local px3, py3 = v:TransformToParent(cx1, cy2)
                        local px4, py4 = v:TransformToParent(cx2, cy1)
                        
                        local min_x = min( min(px1, px2), min(px3, px4) )
                        local max_x = max( max(px1, px2), max(px3, px4) )

                        local min_y = min( min(py1, py2), min(py3, py4) )
                        local max_y = max( max(py1, py2), max(py3, py4) )

                        x1 = x1 and min(x1, min_x) or min_x
                        x2 = x2 and max(x2, max_x) or max_x

                        y1 = y1 and min(y1, min_y) or min_y
                        y2 = y2 and max(y2, max_y) or max_y
                    end
                end
            end
        end
        if not x1 or not y1 then
            x1, y1, x2, y2 = 0, 0, 0, 0

        elseif self.scissor_rect then
            -- Intersect with scissor rect.
            x1, y1, x2, y2 = IntersectRect( x1, y1, x2, y2, table.unpack( self.scissor_rect ))
        end

        self.bboxcache.valid = true
        self.bboxcache.x1, self.bboxcache.y1, self.bboxcache.x2, self.bboxcache.y2 = x1,y1,x2,y2    
        return x1,y1,x2,y2
    end
end

function Widget:CalculateBoundingBox( ... )
    local xmin, ymin, xmax, ymax = math.huge, math.huge, -math.huge, -math.huge
    local n = select( "#", ... )
    for i = 1, n do
        local widget = select( i, ... )
        local x0, y0, x1, y1 = widget:GetBoundingBox()
        if y1 then
            x0, y0 = self:TransformFromWorld(widget:TransformToWorld( x0, y0 ))
            x1, y1 = self:TransformFromWorld(widget:TransformToWorld( x1, y1 ))
            xmin = math.min( x0, xmin )
            xmax = math.max( x1, xmax )
            ymin = math.min( y0, ymin )
            ymax = math.max( y1, ymax )
        end
    end
    return xmin, ymin, xmax, ymax
end

function Widget:GetWorldBoundingBox()
    local x1, y1, x2, y2 = self:GetBoundingBox()
    x1, y1 = self:TransformToWorld( x1, y1 )
    x2, y2 = self:TransformToWorld( x2, y2 )
    local xmin = math.min( x1, x2 )
    local xmax = math.max( x1, x2 )
    local ymin = math.min( y1, y2 )
    local ymax = math.max( y1, y2 )

    return xmin, ymin, xmax, ymax
end

function Widget:GetRect()
    local x1, y1, x2, y2 = self:GetBoundingBox()
    local w, h = x1 and x2 - x1, y1 and y2 - y1
    return x1, y1, w, h
end

function Widget:SetRect(x, y, w, h)
    if self.SetSize then
        -- not all widgets have their own concrete size... not sure what's correct here
        self:SetSize( w, h )
    end
    self:SetPos( x + w/2, y - h/2 )
    return self
end

function Widget:UpdateAnchors()    
    if not self.fe then
        return
    end

    local sw, sh = self:GetFE():GetScreenDims()
    local w,h = sw, sh
    local x,y = 0,0
    --too wide!
    if sw/sh > 21/9 then
        w = sh*(21/9)
        x = (sw-w)/2
    --too tall!
    elseif sw/sh < 4/3 then
        h = w / (4/3)
        y = (sh-h)/2
    end
    
    if self.hanchor == "left" then
        self.anchorx = w*self:GetFE().horizontal_safe_zone*.5
    elseif self.hanchor == "right" then
        self.anchorx = w - w*self:GetFE().horizontal_safe_zone*.5
    elseif self.hanchor == "center" then
        self.anchorx = w/2
    elseif self.hanchor == "fill" then
        self.anchorx = w/2
    else
        self.anchorx = 0
    end
    self.anchorx = self.anchorx + x

    if self.vanchor == "top" then
        self.anchory = h - h*self:GetFE().vertical_safe_zone*.5
    elseif self.vanchor == "bottom" then
        self.anchory = 0--h*self:GetFE().vertical_safe_zone*.5
    elseif self.vanchor == "center" then
        self.anchory = h/2
    elseif self.vanchor == "fill" then
        self.anchory = h/2
    else
        self.anchory = 0
    end
    self.anchory = self.anchory + y
    
    self:MarkTransformDirty()
end

function Widget:GetResolvedScale()
    local x0, y0 = self:TransformToWorld( 0, 0 )
    local x1, y1 = self:TransformToWorld( 0, 1 )
    local x2, y2 = self:TransformToWorld( 1, 1 )
    return distance( x1, y1, x2, y2 ), distance( x1, y1, x0, y0 )
end

-- HSV

function BlendColorAndHSV( parentsetting, selfsetting )
    local result = {}
    result.colour = {}
    result.hsv = {}

    -- Colour and H: Do a weighted multiply
    local h1 = parentsetting.hsv[1]
    local h2 = selfsetting.hsv[1]
    local sumH = h1 + h2

    if sumH > 0 then
        -- Note: None if this is correct. It's a hacky approximation.
        local colorAFactor = h1 / sumH
        local colorBFactor = h2 / sumH
        result.colour[1] = (parentsetting.colour[1] * colorAFactor) + (selfsetting.colour[1] * colorBFactor)
        result.colour[2] = (parentsetting.colour[2] * colorAFactor) + (selfsetting.colour[2] * colorBFactor)
        result.colour[3] = (parentsetting.colour[3] * colorAFactor) + (selfsetting.colour[3] * colorBFactor)
        result.hsv[1] = math.max(h1, h2)
    else
        result.colour[1] = parentsetting.colour[1]
        result.colour[2] = parentsetting.colour[2]
        result.colour[3] = parentsetting.colour[3]
        result.hsv[1] = 0.0
    end

    -- Saturation: Multiply
    local s1 = parentsetting.hsv[2]
    local s2 = selfsetting.hsv[2]
    result.hsv[2] = s1 * s2

    -- Value: Multiply
    local v1 = parentsetting.hsv[3]
    local v2 = selfsetting.hsv[3]
    result.hsv[3] = v1 * v2


--        print("Blending colours: ");
--        print("Parent: ");
--        print(serpent.block(parentsetting));
--        print("self: ");
--        print(serpent.block(selfsetting));

--        print("result: ");
--        print(serpent.block(result));

    return result
end

local IdentityHSV = {   highlights = { colour = { 1, 1, 1 }, hsv = { 0, 1, 1 } },
                        midtones = { colour = { 1, 1, 1 }, hsv = { 0, 1, 1 } },
                        shadows = { colour = { 1, 1, 1 }, hsv = { 0, 1, 1 } } }

function BlendHSV(parenthsv, selfhsv, hsv_ignore_parent)
    if selfhsv then
        if parenthsv and not hsv_ignore_parent then
            local result = {}

            result.highlights = BlendColorAndHSV( parenthsv.highlights, selfhsv.highlights )
            result.midtones = BlendColorAndHSV( parenthsv.midtones, selfhsv.midtones )
            result.shadows = BlendColorAndHSV( parenthsv.shadows, selfhsv.shadows )

            return result
        else
            return selfhsv
        end
    else
        if hsv_ignore_parent then
            -- ignore parent is set to true, but the current HSV is empty. Return the default hsv:
            return IdentityHSV
        else
            return parenthsv
        end
    end
end


function Widget:HasNonIdentityHSVSettings()
    if not self.resolved_hsv then
        return false
    end
    return not IsIdentityHSV(self.resolved_hsv)
end

function Widget:ApplyHSV(hsvtable)
    if self.model then
        if not self.colour_correct_constants then
            self.colour_correct_constants = {}
            self.colour_correct_constants.shadows = engine.rendering.ShaderConstantValue("SHADOWS_ADJUST", 0,1,1)
            self.colour_correct_constants.midtones = engine.rendering.ShaderConstantValue("MIDTONES_ADJUST", 0,1,1)
            self.colour_correct_constants.highlights = engine.rendering.ShaderConstantValue("HIGHLIGHTS_ADJUST", 0,1,1)
            self.colour_correct_constants.shadows_col = engine.rendering.ShaderConstantValue("SHADOWS_ADJUST_COL", 1,1,1)
            self.colour_correct_constants.midtones_col = engine.rendering.ShaderConstantValue("MIDTONES_ADJUST_COL", 1,1,1)
            self.colour_correct_constants.highlights_col = engine.rendering.ShaderConstantValue("HIGHLIGHTS_ADJUST_COL", 1,1,1)

            self.model:SetShaderConstant(self.colour_correct_constants.shadows)
            self.model:SetShaderConstant(self.colour_correct_constants.midtones)
            self.model:SetShaderConstant(self.colour_correct_constants.highlights)
            self.model:SetShaderConstant(self.colour_correct_constants.shadows_col)
            self.model:SetShaderConstant(self.colour_correct_constants.midtones_col)
            self.model:SetShaderConstant(self.colour_correct_constants.highlights_col)
        end

        if hsvtable then
            self.colour_correct_constants.shadows:SetValue(table.unpack(hsvtable.shadows.hsv))
            self.colour_correct_constants.midtones:SetValue(table.unpack(hsvtable.midtones.hsv))
            self.colour_correct_constants.highlights:SetValue(table.unpack(hsvtable.highlights.hsv))
            self.colour_correct_constants.shadows_col:SetValue(table.unpack(hsvtable.shadows.colour))
            self.colour_correct_constants.midtones_col:SetValue(table.unpack(hsvtable.midtones.colour))
            self.colour_correct_constants.highlights_col:SetValue(table.unpack(hsvtable.highlights.colour))
        else
            self.colour_correct_constants.shadows:SetValue(0,1,1)
            self.colour_correct_constants.midtones:SetValue(0,1,1)
            self.colour_correct_constants.highlights:SetValue(0,1,1)
            self.colour_correct_constants.shadows_col:SetValue(1,1,1)
            self.colour_correct_constants.midtones_col:SetValue(1,1,1)
            self.colour_correct_constants.highlights_col:SetValue(1,1,1)
        end
    end
end

function Widget:GetResolvedHSV()

    -- It could be that the parent also has a dirty HSV table, because the update order of widgets may not be in order
    if self.parent and (not self.hsvtable or not self.hsvtable.ignoreParent) then
        return BlendHSV(self.parent:GetResolvedHSV(), self.hsvtable, self.hsv_ignore_parent)
    end

    return self.hsvtable
end

function Widget:ApplyHSVFromParent(parent_resolved_hsv)
    self.resolved_hsv = BlendHSV(parent_resolved_hsv, self.hsvtable, self.hsv_ignore_parent)

    self:ApplyHSV(self.resolved_hsv)

    if self.children then
        for k,v in pairs(self.children) do
            v:ApplyHSVFromParent(self.resolved_hsv)
        end
    end
end

function Widget:SetHSV(hsvtable, ignoreParent)
    assert( hsvtable == nil or type(hsvtable) == "table" )

    self.hsvtable = hsvtable
    self.hsv_ignore_parent = ignoreParent
    
    -- Update the HSV in this widget and all children:
    self.resolved_hsv = self:GetResolvedHSV()

    self:ApplyHSV(self.resolved_hsv)
    
    if self.children then
        for k,v in pairs(self.children) do
            v:ApplyHSVFromParent(self.resolved_hsv)
        end
    end

    return self
end

-- Tint

function Widget:GetResolvedTint()
    
    if self.parent and not self.ignoreparenttint then
        local r,g,b,a = self.parent:GetResolvedTint()
        return (self.tint_r or 1)*r, (self.tint_g or 1)*g, (self.tint_b or 1)*b, (self.tint_a or 1)*a
    end

    return self.tint_r or 1, self.tint_g or 1, self.tint_b or 1, self.tint_a or 1
end

function Widget:ApplyTintFromParent(parent_r,parent_g,parent_b,parent_a)
    if self.ignoreparenttint then
        self.resolved_tint_r = self.tint_r or 1
        self.resolved_tint_g = self.tint_g or 1
        self.resolved_tint_b = self.tint_b or 1
        self.resolved_tint_a = self.tint_a or 1
    else
        self.resolved_tint_r = (parent_r or 1) * (self.tint_r or 1)
        self.resolved_tint_g = (parent_g or 1) * (self.tint_g or 1)
        self.resolved_tint_b = (parent_b or 1) * (self.tint_b or 1)
        self.resolved_tint_a = (parent_a or 1) * (self.tint_a or 1)
    end

    if self.model then
        self:ApplyTint(self.resolved_tint_r, self.resolved_tint_g, self.resolved_tint_b, self.resolved_tint_a)
    end

    if self.children then
        for k,v in pairs(self.children) do
            v:ApplyTintFromParent(self.resolved_tint_r, self.resolved_tint_g, self.resolved_tint_b, self.resolved_tint_a)
        end
    end
end

function Widget:ApplyTint(r,g,b,a)
    if not self.tint_mult_constant then
        self.tint_mult_constant = GetShaderConstant("TINT_MULT", 1,1,1,1)
        --self.tint_mult_constant = engine.rendering.ShaderConstantValue("TINT_MULT", 1,1,1,1)
        self.model:SetShaderConstant(self.tint_mult_constant)
    end
    self.tint_mult_constant:SetValue(r or 1, g or 1, b or 1, a or 1)
    return self
end

function Widget:UpdateTintInternal()
    self.resolved_tint_r, self.resolved_tint_g, self.resolved_tint_b, self.resolved_tint_a = self:GetResolvedTint()

    if self.model then
        self:ApplyTint(self.resolved_tint_r, self.resolved_tint_g, self.resolved_tint_b, self.resolved_tint_a)
    end

    if self.children then
        for k,v in pairs(self.children) do
            v:ApplyTintFromParent(self.resolved_tint_r, self.resolved_tint_g, self.resolved_tint_b, self.resolved_tint_a)
        end
    end
end

function Widget:IgnoreParentTint(val)
    self.ignoreparenttint = val
    self:UpdateTintInternal()
    return self
end

function Widget:SetTintColour(r,g,b,a)
    
    if r and g and b then
        a = a or 1
    elseif r then
        r,g,b,a = HexColour(r)
    end
    self.tint_r = r
    self.tint_g = g
    self.tint_b = b
    self.tint_a = a

    self:UpdateTintInternal()
    return self
end

function Widget:SetTintAlpha(a)
    local old_a = self.tint_a or 1
    assert(a)
    self.tint_a = a
    self:UpdateTintInternal()
    
    return self
end

function Widget:GetAlpha()
    return self.tint_a
end

-- Saturation

function Widget:GetResolvedSaturation()
    if self.parent then
        local sat = self.parent:GetResolvedSaturation()
        return (self.saturation or 1) * sat;
    end
    return self.saturation or 1
end

function Widget:ApplySaturation(s)
    if self.model then
        if self.model.SetSaturation then
            self.model:SetSaturation(s)
        else
            if not self.satval and s ~= 1 then
                self.satval = engine.rendering.ShaderConstantValue("SATURATION", s)
                self.model:SetShaderConstant(self.satval)
            end
            if self.satval then
                self.satval:SetValue(s)
            end
        end
    end
    return self
end

function Widget:ApplySaturationFromParent(s)
    s = s * (self.saturation or 1)

    if self.model then
        self:ApplySaturation(s)
        --self.model:SetSaturation(s)
    end

    if self.children then
        for k,v in pairs(self.children) do
            v:ApplySaturationFromParent(s)
        end
    end
end

function Widget:Bloom(bloom_amt)
    if self.children then
        for k,v in pairs(self.children) do
            v:Bloom(bloom_amt)
        end
    end

    if self.SetBloom then
        self:SetBloom(bloom_amt)
    end

    return self
end

function Widget:SetSaturation(s)
    
    self.saturation = s
    self.resolved_sat = self:GetResolvedSaturation()

    if self.model then
        self:ApplySaturation(self.resolved_sat)
        --self.model:SetSaturation(s)
    end

    if self.children then
        for k,v in pairs(self.children) do
            v:ApplySaturationFromParent(self.resolved_sat)
        end
    end

    return self
end

function Widget:StopListeningForAllGlobalEvents()
    TheGame:GetEvents():RemoveListener(self)
end

function Widget:ListenForGlobalEvent(eventname, handler)
    TheGame:GetEvents():ListenForEvent(eventname, self)
    self.eventhandlers = self.eventhandlers or {}
    self.eventhandlers[eventname] = handler
end

function Widget:StopListeningForGlobalEvent(eventname)
    TheGame:GetEvents():StopListening(eventname, self)
end

function Widget:OnGlobalEvent(event, ...)
    if self.eventhandlers and self.eventhandlers[event] then
        self.eventhandlers[event](self, ...)
    end
end

function Widget:CancelTasks()
    if self.tasks then
        for k,v in pairs(self.tasks) do
            k:Cancel()
        end
    end
    return self
end

function Widget:CleanTasks()
    if self.tasks then
        for k,v in pairs(self.tasks) do
            if not k:Running() then
                self.tasks[k] = nil
            end
        end
    end
end

function Widget:DoTaskInTime(name, delay, fn, ... )
    assert( not self.removed )
    self:CleanTasks()
    local task = TheGame.scheduler:DoTaskInTime(name, delay, fn, ...)
    self.tasks = self.tasks or {}
    self.tasks[task] = true
    return task
end

function Widget:DoPeriodicTask(name, inital_delay, period, fn, ... )
    assert( not self.removed )
    self:CleanTasks()
    local task = TheGame.scheduler:DoPeriodicTask(name, inital_delay, period, fn, ...)
    self.tasks = self.tasks or {}
    self.tasks[task] = true
    return task
end

function Widget:CheckNamePathAsc(path, match_depth)
    match_depth = match_depth or 0
    if self.name == path[#path - match_depth] then
        if match_depth + 1 == #path then return self end
        return self.parent and self.parent:CheckNamePathAsc(path, match_depth + 1) 
    end
end

function Widget:CheckNamePathDesc(path)
    
    if self:CheckNamePathAsc(path) then
        return self
    end

    if self.children then
        for k,v in pairs(self.children) do
            local match = v:CheckNamePathDesc(path)
            if match then return match end
        end
    end
end

function Widget:FindCommonAncestor( other )
    local w1 = self
    while w1 do
        local w2 = other
        while w2 do
            if w2 == w1 then
                return w2
            end
            w2 = w2.parent
        end
        w1 = w1.parent
    end
    return nil
end

function Widget:FindChild(name)

    local path = string.split(name, "%.")
    if #path == 0 then return end
    return self:CheckNamePathDesc(path) 
end

function Widget:IndexOf(child)
    return table.arrayfind( self.children, child )
end

function Widget:AddChildren(children)
    for k, child in ipairs(children) do
        self:AddChild(child)
    end
    self:InvalidateBBox()
    
    return self
end

--take our root as the top left and line up everything below us
function Widget:StackChildren(margin, center_w)
    margin = margin or 0
    local y = 0
    if self.children then
        for k,v in ipairs(self.children) do
            if v:IsShown() then
                local w, h = v:GetScaledSize()

                local cx, cy = v:GetCentroid()
                v:SetPos(center_w and 0 or (w/2 - cx), y-h/2-cy)
                y = y - h - margin
            end
        end
    end
    return self
end

-- Lays out the children in a row next to each other, based on their widths and spacing
function Widget:StackChildrenRow( spacing )
    local num = self.children and #self.children or 0
    local current_x = 0
    spacing = spacing or 0
    if num > 0 then
        for k, v in ipairs(self.children) do
            -- Get current widget dimensions
            local w, h = v:GetScaledSize()
            -- Check if it should be laid out
            if v:IsShown() and w > 0 then
                current_x = current_x + w/2
                v:LayoutBounds( "center", nil, current_x, 0 )
                current_x = current_x + w/2 + spacing
            end
        end
    end
    return self
end

function Widget:GetNumChildren()
    return self.children and #self.children or 0
end

-- Lays out the children at a fixed interval, independently of their widths
function Widget:ArrangeChildrenRow(offset)
    local num = self.children and #self.children or 0
    if num > 0 then
        local w = (num-1)*offset
        for k,v in ipairs(self.children) do
            local x = -w/2+(k-1)*offset
            v:SetPos(x, nil)
        end
    end

    return self
end

function Widget:ArrangeChildrenColumn(offset)
    local num = self.children and #self.children or 0
    if num > 0 then
        local h = (num-1)*offset
        for k,v in ipairs(self.children) do
            local y = h/2-(k-1)*offset
            v:SetPos(0, y)
        end
    end
    return self
end

function Widget:Delay(t)
    t = t or 0
    while (t > 0) do 
        local dt = coroutine.yield()
        t = t - dt
    end
end

function Widget:IsAnimating()
    if self.running_updaters and #self.running_updaters > 0 then
        return true
    end
    return self.colour ~= nil or self.move ~= nil or self.scale ~= nil or self.blink ~= nil
end

function Widget:DelayIfAnimating()
    while self:IsAnimating() do
        coroutine.yield()
    end
end

function Widget:OnScreenResize(w,h)
    if self.children then
        for k,v in pairs(self.children) do
            v:OnScreenResize(w,h)
        end
    end

    if self.hanchor or self.vhanchor or self.base_scaling == true then
        self:UpdateAnchors()
        self:MarkTransformDirty()
    end
end

function Widget:OnControlModeChange(cm, device_id)
    if self.children then
        for k,v in pairs(self.children) do
            v:OnControlModeChange(cm, device_id)
        end
    end
end

function Widget:OnScreenModeChange(sm)
    if self.children then
        for k,v in pairs(self.children) do
            v:OnScreenModeChange(sm)
        end
    end
end

-- Lays out this widget relative to the bounding box of 'prev'
function Widget:LayoutBounds( hreg, vreg, prev, _y )
    local dx, dy = self:CalcLayoutBoundsOffset( hreg, vreg, prev, _y )
    self:Offset( dx, dy )
    return self
end

-- Lays out this widget relative to its existing siblings in a grid. 
-- When it wraps to a new row, it aligns itself to the left edge of leftmost_element (which could be the first element on the grid for every call)
function Widget:LayoutInGrid( columns, index, spacing, leftmost_element )
    if self ~= leftmost_element then
        local newrow = (index % columns == 1) or columns == 1
        local horizontal_spacing = spacing
        local vertical_spacing = spacing
        if (type(spacing) == "table") then
            horizontal_spacing = spacing["h"]
            vertical_spacing = spacing["v"]
        end

        self:LayoutBounds( newrow and "left" or "after", newrow and "below" or "top" ):Offset( newrow and 0 or horizontal_spacing, newrow and -vertical_spacing or 0 )
        if newrow then self:LayoutBounds( "left", nil, leftmost_element ) end
    end

    return self
end

function Widget:LayoutChildrenInGrid( columns, spacing )
    local first_child
    local pos = 1
    for k, v in ipairs( self.children ) do
        if v:IsShown() then
            first_child = first_child or v
            v:LayoutInGrid( columns, pos, spacing, first_child )
            pos = pos + 1
        end
    end
    return self
end

function Widget:CalcLayoutBoundsOffset( hreg, vreg, prev, _y )
    local px1, py1, px2, py2
    local ancestor
    if prev and type(prev) == "number" then
        ancestor = self.parent
        px1, py1 = prev, _y or 0
        px2, py2 = px1, py1

    else
        if prev == nil then
            -- Default to the previous sibling.
            local i = self.parent:IndexOf( self )
            for j = i-1, 1, -1 do
                -- Go backwards through the siblings until you find a visible one
                local sibling = self.parent.children[ j ]
                if sibling and sibling:IsShown() then
                    prev = sibling
                    break
                end
            end
            -- prev = self.parent.children[ i - 1 ]
        end
        if prev then
            ancestor = self:FindCommonAncestor( prev )
            if ancestor == nil then
                -- Diagnostics for catching crash fix.
                print( "No common ancestor:", prev, self )
                print( prev.removed, self.removed)
            end

            px1, py1, px2, py2 = prev:GetBoundingBox()
            px1, py1 = ancestor:TransformFromWidget( prev, px1, py1 )
            px2, py2 = ancestor:TransformFromWidget( prev, px2, py2 )
            px1, px2 = math.min( px1, px2 ), math.max( px1, px2 )
            py1, py2 = math.min( py1, py2 ), math.max( py1, py2 )
        else
            -- LOGWARN( "widget laying out with respect to nil: '%s'", tostring(self) )
            ancestor = self.parent
            px1, py1 = 0, 0
            px2, py2 = px1, py1
        end
    end

    local x1, y1, x2, y2 = self:GetBoundingBox()
    x1, y1 = ancestor:TransformFromWidget( self, x1, y1 )
    x2, y2 = ancestor:TransformFromWidget( self, x2, y2 )

    -- Align this bounding box next to previous bounding box according to hreg, vreg.
    local dx, dy = 0, 0
    if hreg == "before" then
        dx = px1 - x2 -- Align previous left-bounds to this right-bounds (ie. before prev)
    elseif hreg == "left" then
        dx = px1 - x1 -- Align previous left-bounds to this left-bounds
    elseif hreg == "left_center" then
        dx = (px1 + px2)/2 - x1 -- Align left-bounds to previous centre point
    elseif hreg == "center" then
        dx = (px1 + px2)/2 - (x1 + x2)/2 -- Align centre points
    elseif hreg == "after" then
        dx = px2 - x1 -- Align previous right-bounds to this left-bounds (ie. after prev)
    elseif hreg == "right_center" then
        dx = (px1 + px2)/2 - x2 -- Align right-bounds to previous centre point
    elseif hreg == "right" then
        dx = px2 - x2 -- Align previous right-bounds to this right-bounds
    elseif hreg ~= nil then
        LOGWARN( "Bad layout hreg specifier:"..tostring(hreg))
    end

    if vreg == "below" then
        dy = py1 - y2 -- Align previous bottom-bounds to this top-bounds (ie. below prev)
    elseif vreg == "bottom" then
        dy = py1 - y1 -- Align previous bottom-bounds to this bottom_bounds
    elseif vreg == "center" then
        dy = (py1 + py2)/2 - (y1 + y2)/2 -- Align centre points
    elseif vreg == "above" then
        dy = py2 - y1 -- Align previous top-bounds to this bottom-bounds (ie. above prev)
    elseif vreg == "top" then
        dy = py2 - y2 -- Align previous top-bounds to this top-bounds
    elseif vreg ~= nil then
        LOGWARN( "Bad layout vreg specifier:"..tostring(vreg))
    end

    -- This offset is ancestor-space: convert to a local delta.
    dx, dy = self:TransformFromWidget( ancestor, dx, dy )
    local dx0, dy0 = self:TransformFromWidget( ancestor, 0, 0 )
    dx, dy = (dx - dx0) * self.sx * self.layout_scale, (dy - dy0) * self.sy * self.layout_scale

    return dx, dy
end

function Widget:SetRegistration(hreg, vreg)
    self.hreg = hreg
    self.vreg = vreg

    local w, h = self:GetSize()
    local x, y = self:GetCentroid()

    local oldrx, oldry = self.regx, self.regy
    if hreg == "center" then
        self.regx = -x
    elseif hreg == "left" then
        self.regx = -x + w/2
    elseif hreg == "right" then
        self.regx = -x - w/2
    else
        self.regx = 0
    end


    if vreg == "center" then
        self.regy = -y
    elseif vreg == "bottom" then
        self.regy = -y + h/2
    elseif vreg == "top" then
        self.regy = -y - h/2
    else
        self.regy = 0
    end

    if oldrx ~= self.regx or oldry ~= self.regy then
        self:MarkTransformDirty()
        self:InvalidateBBox()
    end
    return self
end

function Widget:GetParentExtents()
    local x1, y1, x2, y2 = self:GetBoundingBox()
    x1, y1 = self:TransformToParent(x1, y1)
    x2, y2 = self:TransformToParent(x2, y2)
    local top = math.max(y1,y2)
    local bottom = math.min(y1, y2)

    local left = math.min(x1,x2)
    local right = math.max(x1, x2)

    return left, right, top, bottom
end

function Widget:PlayPositionalEvent( event_name )
    AUDIO:PlayParamEvent( event_name, "position", self:GetPanPos() )
end

function Widget:GetPanPos()
    if self.removed then
        return self.last_pan_pos or 0
    end

    -- For sound use! Returns where on the screen the widget sits left to right, from -1 to 1. 0 being the center.
    local x, y, z = self:GetWorldPos()
    local width = TheGame:FE():GetScreenDims()
    local screen_pos = math.abs(x)/width
    self.last_pan_pos = easing.linear( screen_pos, -1, 2, 1 )
    return self.last_pan_pos
end

function Widget.MergeControlMap(base, override)
    local t = {}    
    
    for k,v in ipairs(base) do
        local overridden = false
        if v.control then
            for kk, vv in ipairs(override) do
                if vv.control == v.control then
                    overridden = true
                    break
                end
            end
        end
        if not overridden then
            table.insert(t, v)
        end
    end

    for k,v in ipairs(override) do
        table.insert(t, v)
    end
    return t
end

