
------------------------------------------------------------------------------------------
-- Shows a single agent portrait

local AgentPortrait = class( "Widget.AgentPortrait", Widget )

function AgentPortrait:init( size, agent )
    AgentPortrait._base.init( self )

    self.hitbox = self:AddChild( Widget.SolidBox() )
    	:SetColour( 0xFF00FF00 )
	self.portrait_image = self:AddChild( Widget.Image() )
		:SetHiddenBoundingBox( true ) 
	self.portrait_anim_container = self:AddChild( Widget() )
		:SetHiddenBoundingBox( true )
	self.portrait_anim = self.portrait_anim_container:AddChild( Widget.AnimatedCharacter() )
		:SetPortraitMode()
        :Play( "portrait_neutral" )

    self:SetSize( size )
    self:SetAgent( agent )
end

function AgentPortrait:SetSize( size )
	self.portrait_size = size or 90
	self.hitbox:SetSize( self.portrait_size, self.portrait_size )
	self.portrait_image:SetSize( self.portrait_size, self.portrait_size )
	self.portrait_anim_container:SetScissor( 0, 0, self.portrait_size, self.portrait_size )
    	:SetPos( -self.portrait_size/2, -self.portrait_size/2)
	local default_portrait_h = 100
    local s = self.portrait_size/default_portrait_h
    self.portrait_anim:SetScale( s )
    	:SetPos( self.portrait_size*0.4, self.portrait_size*0.17 )
	return self
end

function AgentPortrait:SetAgent( agent )
	self.agent = agent
	if self.agent and self.agent:GetSkinID() and engine.inst:TextureExists( "portraits/" .. self.agent:GetSkinID() .. ".tex" ) then
		self.portrait_image:Show():SetTexture( engine.asset.Texture( "portraits/" .. self.agent:GetSkinID() .. ".tex" ) )
		self.portrait_anim:Hide()
	elseif self.agent then
		self.portrait_anim:Show():Refresh( self.agent )
        self.portrait_anim:Show():SetMood(MOOD.NEUTRAL)
		self.portrait_image:Hide()
	end
	-- Check if this agent should have an exported portrait
	if self.agent and self.agent:GetSkinID() and not engine.inst:TextureExists( "portraits/" .. self.agent:GetSkinID() .. ".tex" ) then
	    if TheGame:FE():FindScreen( "Screen.ConvoBuildDebug" ) == nil then
			LOGWARN( "Portrait texture missing for agent %s. Skin ID: %s. Generate portraits in the Tools menu to re-export all.", self.agent:GetName(), self.agent:GetSkinID() )
		end
	end
	return self
end