require "ui/widget"
require "ui/widgets/animatedcharacter"

-- This is just an extension of the AnimatedCharacter object, but this one makes only the character head count towards the bounding box, 
-- so that it can be aligned to things in UI
local AnimatedCharacterUI = class( "Widget.AnimatedCharacterUI", Widget.AnimatedCharacter )

function AnimatedCharacterUI:init()
    AnimatedCharacterUI._base.init( self )

    -- Add our hitbox
    self.hitbox = self:AddChild( Widget.SolidBox( 300, 300, 0xff00ff00 ) )
end

function AnimatedCharacterUI:Refresh( ... )
	AnimatedCharacterUI._base.Refresh( self, ... )
	return self:RefreshHeadPosition()
end

function AnimatedCharacterUI:Play( ... )
	AnimatedCharacterUI._base.Play( self, ... )
	return self:RefreshHeadPosition()
end

function AnimatedCharacterUI:Emote( ... )
	AnimatedCharacterUI._base.Emote( self, ... )
	return self:RefreshHeadPosition()
end

function AnimatedCharacterUI:RefreshHeadPosition()
	-- Make the anim take up space
    self.anim:SetHiddenBoundingBox( false )
    -- Get the anim's own size (it has its own weird scale)
    local local_size_w, local_size_h = self.anim:GetSize()
    -- Get the head's position within the anim's own size
    local local_head_x, local_head_y, local_head_z = self.anim.model:SymbolToLocal( "head_swap", 0, 0, 0 )
    if self.char and self.char.fight_data and self.char.fight_data.track_symbol then
        local_head_x, local_head_y, local_head_z = self.anim.model:SymbolToLocal(self.char.fight_data.track_symbol, 0, 0, 0)
    end
    -- Calculate the proportional position of the head within the anim
    local relative_head_x, relative_head_y = local_head_x/local_size_w, local_head_y/local_size_h

    -- Get the widget's current size within the screen
    local scaled_size_w, scaled_size_h = self.anim:GetScaledSize()
    -- Calculate the head's position relative to that size
    local scaled_head_x, scaled_head_y = relative_head_x*scaled_size_w, relative_head_y*scaled_size_h

	-- Position the hitbox over the character's head
    self.hitbox:SetPos( scaled_head_x, scaled_head_y )
	-- Make the anim not take up space again
    self.anim:SetHiddenBoundingBox( true )

    return self
end