require "ui/widgets/peoplecompendium"

local assets = {
    bg = engine.asset.Texture("UI/compendium_boss_bg.tex")
}
------------------------------------------------------------------------------------------
-- Shows the people panel

local BossCompendiumWidget = class( "Widget.BossCompendiumWidget", Widget.PeopleCompendiumWidget )

local columns = 3
local portrait_w
local portrait_h
local spacing = 20

function BossCompendiumWidget:AddSidebarFilters()
    -- Add a text search bar
    if SPECIAL_BUILD_TYPE < 1024 then
        self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH" )
        self.sidebar:AddFilterWidget( Widget.PeopleSearchFilterBlock() )
    end

    -- Add a faction filter
    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.FILTER_FACTION" )
    self.faction_filter_group = self.sidebar:AddFilterWidget( Widget.FilterOptionalRadioGroup() )
        :SetRowMode( 4 )
    -- self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "GRIFTER" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "FEUD_CITIZEN" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "ADMIRALTY" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "CULT_OF_HESH" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "SPARK_BARONS" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "BILEBROKERS" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "JAKES" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "BANDITS" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "RISE" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "RENTORIAN" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "BOGGERS" ) )
    self.faction_filter_group:SetSelected( 0 )

    -- Add filters for a few other aspects
    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.AGENT_FILTER_OTHER" )
    self.other_filter_row = self.sidebar:AddFilterWidget( Widget.FilterOptionalRadioGroup() )
    self.other_filter_row:AddFilter( Widget.BossUnknownFilterRow() )
    self.other_filter_row:SetSelected( 0 )

    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.BOSS_SORT" )
    self:AddSortingWidget()
end

function BossCompendiumWidget:UpdateCount()
    -- Reset counts
    self.agents = {}
    self.agent_counts = {}
    self.agent_counts["UNLOCKABLE"] = 0
    self.agent_counts["UNLOCKED"] = 0

    -- Get all the characters
    local all_defs = Content.GetAllCharacterDefs()
    for content_id, def in pairs( all_defs ) do
        local skins = Content.GetAllCharacterSkins( content_id )
        if skins then
            for i, skin in ipairs( skins ) do
                if def.faction_id ~= PLAYER_FACTION and not def.hide_in_compendium then
                    local agent = Agent( content_id, skin )
                    if agent.boss then table.insert( self.agents, agent ) end
                end
            end
        elseif (def.unique or def.boss) and not def.hide_in_compendium then
            local agent = Agent( content_id )
            if agent.boss then table.insert( self.agents, agent ) end
        end
    end

    -- Go through all the agents
    for k, agent in ipairs( self.agents ) do
        agent.unlocks_list = {}

        -- Check if this agent has a social boon unlockable
        if agent:GetSocialGraft( RELATIONSHIP.LOVED ) then
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1

            -- Has this boon been unlocked?
            if TheGame:GetGameProfile():HasUnlockedBoon( agent:GetUniqueID() ) then
                agent.unlocked = (agent.unlocked or 0) + 1
                self.agent_counts["UNLOCKED"] = self.agent_counts["UNLOCKED"] + 1
                table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_BOON" ) )
            else
                table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_INCOMPLETE", LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_BOON" ) )
            end
        end

        -- Check if this agent has a social bane unlockable
        if agent:GetSocialGraft( RELATIONSHIP.HATED ) then
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1

            -- Has this bane been unlocked?
            if TheGame:GetGameProfile():HasUnlockedBane( agent:GetUniqueID() ) then
                agent.unlocked = (agent.unlocked or 0) + 1
                self.agent_counts["UNLOCKED"] = self.agent_counts["UNLOCKED"] + 1
                table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_BANE" ) )
            else
                table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_INCOMPLETE", LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_BANE" ) )
            end
        end

        -- Check if this agent has been met
        if TheGame:GetGameProfile():HasMetCharacter( agent:GetUniqueID() ) or Screen.Compendium.ShowAllUnlocks() then
            agent.is_locked = false
            agent.unlocked = (agent.unlocked or 0) + 1
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1
            self.agent_counts["UNLOCKED"] = self.agent_counts["UNLOCKED"] + 1
            table.insert( agent.unlocks_list, 1, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_MET", agent:GetName() ) ) )
        else
            agent.is_locked = true
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1
        end

        -- Defeated?
        agent.unlockable = (agent.unlockable or 0) + 1
        self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1

        local stat = string.format( "defeated_%s", agent:GetContentID() )
        if TheGame:GetGameStats():GetStat( stat ) > 0 or Screen.Compendium.ShowAllUnlocks() then
            agent.unlocked = (agent.unlocked or 0) + 1
            self.agent_counts["UNLOCKED"] = self.agent_counts["UNLOCKED"] + 1
            table.insert( agent.unlocks_list, 1, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_DEFEATED", agent:GetName() ) ) )
        else
            table.insert( agent.unlocks_list, 1, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_INCOMPLETE", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_DEFEATED", agent:GetName() ) ) )
        end

        -- Check if there are any lore blocks to show for this agent
        local lore_unlocked, lore_unlockables = 0, 0
        local lore_unlocks = agent:GetLoreUnlocks()
        -- Go through them and count them up
        for action, unlock_text in pairs( lore_unlocks ) do
            lore_unlockables = lore_unlockables + 1
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1
            -- Check if this has been achieved
            if TheGame:GetGameProfile():HasCustomAgentUnlock( agent:GetUniqueID(), action ) then
                lore_unlocked = lore_unlocked + 1
                agent.unlocked = (agent.unlocked or 0) + 1
                self.agent_counts["UNLOCKED"] = self.agent_counts["UNLOCKED"] + 1
            end
        end
        if lore_unlocked > 0 then
            -- Add these to the list
            table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_LORE", lore_unlocked, lore_unlockables, agent:GetName() ) ) )
        elseif lore_unlockables > 0 then
            table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_INCOMPLETE", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_LORE", lore_unlocked, lore_unlockables, agent:GetName() ) ) )
        end
    end
end

------------------------------------------------------------------------------------------
-- Shows a single agent portrait

local BossCompendiumPortrait = class( "Widget.BossCompendiumPortrait", Widget.PeopleCompendiumPortrait )

local default_portrait_h = 100
local vertical_portrait_overshoot = 10

function BossCompendiumPortrait:init( w, h, agent )
    BossCompendiumPortrait._base.init( self, w, h, agent )

    self.bg:SetTexture( assets.bg )

    -- self:Layout()
end

function BossCompendiumPortrait:SetLocked( locked )
    BossCompendiumPortrait._base.SetLocked( self, locked )

    if self.is_locked then
        self:SetToolTip( LOC"UI.CARDCOMPENDIUM.BOSS_LOCKED_TT" )
    end

    return self
end
