require "ui/widget"

local assets =
{
    bottle_scale = 0.30,
    bottle_normal = engine.asset.Texture( "UI/ic_bottles.tex"),
    bottle_empty = engine.asset.Texture( "UI/ic_bottles_empty.tex"),
    bottle_outline = engine.asset.Texture( "UI/ic_bottles_outline.tex"),
}

-------------------------------------------------------------------------------
-- Widget wrapping a deck of cards.

local BottlesIndicator = class( "Widget.BottlesIndicator", Widget.Clickable )

function BottlesIndicator:init()
    BottlesIndicator._base.init( self )

    self.pacing = 0.1
    self.left = false -- For the right side of the screen

    self.bottles = self:AddChild( Widget() )
        :SetScale( assets.bottle_scale )

    self.bottle2back = self.bottles:AddChild( Widget.Image( assets.bottle_outline ) ):SetBloom( 0.2 ):SetScale( 0.87 )
    self.bottle1back = self.bottles:AddChild( Widget.Image( assets.bottle_outline ) ):SetBloom( 0.2 )
    self.bottle2 = self.bottles:AddChild( Widget.Image( assets.bottle_normal ) ):SetScale( 0.87 )
    self.bottle1 = self.bottles:AddChild( Widget.Image( assets.bottle_normal ) )

    self.counter = self.bottles:AddChild( Widget.Label( "body", 76, "0" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetBloom( 0.1 )
    
    -- Setup values
    self.orientation_mult = self.left and -1 or 1
    self.counter_x = self.left and -20 or -23
    self.counter_y_start, self.counter_y_end = -10, -2

    self.bottle1_x_start, self.bottle1_x_end = 0, 0
    self.bottle1_y_start, self.bottle1_y_end = 0, 0
    self.bottle1_rotation_start, self.bottle1_rotation_end = -20 * DEGREES, -34 * DEGREES

    self.bottle2_x_start, self.bottle2_x_end = -55, -55
    self.bottle2_y_start, self.bottle2_y_end = 55, 75
    self.bottle2_rotation_start, self.bottle2_rotation_end = 9 * DEGREES, 17 * DEGREES

    self:Refresh( 0 )
    self:UpdateImage()

    self:SetFocusable( false )
end

function BottlesIndicator:UpdateImage()
    self.pacing = 0.1
    self.counter:SetScale( self.orientation_mult, 1 )

    if self.hover or self.focus then
        self:RunUpdater(
            Updater.Parallel{
                Updater.Ease( function(y) self.counter:SetPos(self.counter_x,y) end, self.counter_y_start, self.counter_y_end, self.pacing, easing.outQuad),
                
                Updater.Ease2D( function(x,y) self.bottle1:SetPos(x,y) end, self.bottle1_x_start, self.bottle1_y_start, self.bottle1_x_end, self.bottle1_y_end, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle1back:SetPos(x,y) end, self.bottle1_x_start, self.bottle1_y_start, self.bottle1_x_end, self.bottle1_y_end, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.bottle2:SetPos(x,y) end, self.bottle2_x_start, self.bottle2_y_start, self.bottle2_x_end, self.bottle2_y_end, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle2back:SetPos(x,y) end, self.bottle2_x_start, self.bottle2_y_start, self.bottle2_x_end, self.bottle2_y_end, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottle1:SetRotation(v) end, self.bottle1_rotation_start, self.bottle1_rotation_end, self.pacing, easing.linear),
                Updater.Ease( function(v) self.bottle1back:SetRotation(v) end, self.bottle1_rotation_start, self.bottle1_rotation_end, self.pacing, easing.linear),

                Updater.Ease( function(v) self.bottle2:SetRotation(v) end, self.bottle2_rotation_start, self.bottle2_rotation_end, self.pacing, easing.linear),
                Updater.Ease( function(v) self.bottle2back:SetRotation(v) end, self.bottle2_rotation_start, self.bottle2_rotation_end, self.pacing, easing.linear),

                Updater.Ease( function(v) self.bottles:SetScale(v*self.orientation_mult,v) end, assets.bottle_scale * 1.0, assets.bottle_scale * 1.25, self.pacing, easing.outQuad),
            }
        )
        if self.reflection then self.reflection:ScaleTo( 1.0, 1.1, self.pacing, easing.outQuad ) end
    else
        self:RunUpdater(
            Updater.Parallel{
                Updater.Ease( function(y) self.counter:SetPos(self.counter_x,y) end, self.counter_y_end, self.counter_y_start, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.bottle1:SetPos(x,y) end, self.bottle1_x_end, self.bottle1_y_end, self.bottle1_x_start, self.bottle1_y_start, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle1back:SetPos(x,y) end, self.bottle1_x_end, self.bottle1_y_end, self.bottle1_x_start, self.bottle1_y_start, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.bottle2:SetPos(x,y) end, self.bottle2_x_end, self.bottle2_y_end, self.bottle2_x_start, self.bottle2_y_start, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle2back:SetPos(x,y) end, self.bottle2_x_end, self.bottle2_y_end, self.bottle2_x_start, self.bottle2_y_start, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottle1:SetRotation(v) end, self.bottle1_rotation_end, self.bottle1_rotation_start, self.pacing, easing.outQuad),
                Updater.Ease( function(v) self.bottle1back:SetRotation(v) end, self.bottle1_rotation_end, self.bottle1_rotation_start, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottle2:SetRotation(v) end, self.bottle2_rotation_end, self.bottle2_rotation_start, self.pacing, easing.outQuad),
                Updater.Ease( function(v) self.bottle2back:SetRotation(v) end, self.bottle2_rotation_end, self.bottle2_rotation_start, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottles:SetScale(v*self.orientation_mult,v) end, assets.bottle_scale * 1.25, assets.bottle_scale * 1.0, self.pacing, easing.outQuad),
            }
        )
        if self.reflection then self.reflection:ScaleTo( 1.1, 1.0, self.pacing, easing.outQuad ) end
    end
end

function BottlesIndicator:GrabAttention()
    self.pacing = 0.15
    self:RunUpdater(
        Updater.Series{
            Updater.Parallel{
                Updater.Ease( function(y) self.counter:SetPos(self.counter_x,y) end, self.counter_y_start, self.counter_y_end, self.pacing, easing.outQuad),
                
                Updater.Ease2D( function(x,y) self.bottle1:SetPos(x,y) end, self.bottle1_x_start, self.bottle1_y_start, self.bottle1_x_end, self.bottle1_y_end, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle1back:SetPos(x,y) end, self.bottle1_x_start, self.bottle1_y_start, self.bottle1_x_end, self.bottle1_y_end, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.bottle2:SetPos(x,y) end, self.bottle2_x_start, self.bottle2_y_start, self.bottle2_x_end, self.bottle2_y_end, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle2back:SetPos(x,y) end, self.bottle2_x_start, self.bottle2_y_start, self.bottle2_x_end, self.bottle2_y_end, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottle1:SetRotation(v) end, self.bottle1_rotation_start, self.bottle1_rotation_end, self.pacing, easing.linear),
                Updater.Ease( function(v) self.bottle1back:SetRotation(v) end, self.bottle1_rotation_start, self.bottle1_rotation_end, self.pacing, easing.linear),

                Updater.Ease( function(v) self.bottle2:SetRotation(v) end, self.bottle2_rotation_start, self.bottle2_rotation_end, self.pacing, easing.linear),
                Updater.Ease( function(v) self.bottle2back:SetRotation(v) end, self.bottle2_rotation_start, self.bottle2_rotation_end, self.pacing, easing.linear),

                Updater.Ease( function(v) self.bottles:SetScale(v*self.orientation_mult,v) end, assets.bottle_scale * 1.0, assets.bottle_scale * 1.25, self.pacing, easing.outQuad),
            },
            Updater.Parallel{
                Updater.Ease( function(y) self.counter:SetPos(self.counter_x,y) end, self.counter_y_end, self.counter_y_start, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.bottle1:SetPos(x,y) end, self.bottle1_x_end, self.bottle1_y_end, self.bottle1_x_start, self.bottle1_y_start, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle1back:SetPos(x,y) end, self.bottle1_x_end, self.bottle1_y_end, self.bottle1_x_start, self.bottle1_y_start, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.bottle2:SetPos(x,y) end, self.bottle2_x_end, self.bottle2_y_end, self.bottle2_x_start, self.bottle2_y_start, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle2back:SetPos(x,y) end, self.bottle2_x_end, self.bottle2_y_end, self.bottle2_x_start, self.bottle2_y_start, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottle1:SetRotation(v) end, self.bottle1_rotation_end, self.bottle1_rotation_start, self.pacing, easing.outQuad),
                Updater.Ease( function(v) self.bottle1back:SetRotation(v) end, self.bottle1_rotation_end, self.bottle1_rotation_start, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottle2:SetRotation(v) end, self.bottle2_rotation_end, self.bottle2_rotation_start, self.pacing, easing.outQuad),
                Updater.Ease( function(v) self.bottle2back:SetRotation(v) end, self.bottle2_rotation_end, self.bottle2_rotation_start, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottles:SetScale(v*self.orientation_mult,v) end, assets.bottle_scale * 1.25, assets.bottle_scale * 1.0, self.pacing, easing.outQuad),
            },
            Updater.Wait( self.pacing*0.5 ),
            Updater.Parallel{
                Updater.Ease( function(y) self.counter:SetPos(self.counter_x,y) end, self.counter_y_start, self.counter_y_end, self.pacing, easing.outQuad),
                
                Updater.Ease2D( function(x,y) self.bottle1:SetPos(x,y) end, self.bottle1_x_start, self.bottle1_y_start, self.bottle1_x_end, self.bottle1_y_end, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle1back:SetPos(x,y) end, self.bottle1_x_start, self.bottle1_y_start, self.bottle1_x_end, self.bottle1_y_end, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.bottle2:SetPos(x,y) end, self.bottle2_x_start, self.bottle2_y_start, self.bottle2_x_end, self.bottle2_y_end, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle2back:SetPos(x,y) end, self.bottle2_x_start, self.bottle2_y_start, self.bottle2_x_end, self.bottle2_y_end, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottle1:SetRotation(v) end, self.bottle1_rotation_start, self.bottle1_rotation_end, self.pacing, easing.linear),
                Updater.Ease( function(v) self.bottle1back:SetRotation(v) end, self.bottle1_rotation_start, self.bottle1_rotation_end, self.pacing, easing.linear),

                Updater.Ease( function(v) self.bottle2:SetRotation(v) end, self.bottle2_rotation_start, self.bottle2_rotation_end, self.pacing, easing.linear),
                Updater.Ease( function(v) self.bottle2back:SetRotation(v) end, self.bottle2_rotation_start, self.bottle2_rotation_end, self.pacing, easing.linear),

                Updater.Ease( function(v) self.bottles:SetScale(v*self.orientation_mult,v) end, assets.bottle_scale * 1.0, assets.bottle_scale * 1.25, self.pacing, easing.outQuad),
            },
            Updater.Parallel{
                Updater.Ease( function(y) self.counter:SetPos(self.counter_x,y) end, self.counter_y_end, self.counter_y_start, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.bottle1:SetPos(x,y) end, self.bottle1_x_end, self.bottle1_y_end, self.bottle1_x_start, self.bottle1_y_start, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle1back:SetPos(x,y) end, self.bottle1_x_end, self.bottle1_y_end, self.bottle1_x_start, self.bottle1_y_start, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.bottle2:SetPos(x,y) end, self.bottle2_x_end, self.bottle2_y_end, self.bottle2_x_start, self.bottle2_y_start, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.bottle2back:SetPos(x,y) end, self.bottle2_x_end, self.bottle2_y_end, self.bottle2_x_start, self.bottle2_y_start, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottle1:SetRotation(v) end, self.bottle1_rotation_end, self.bottle1_rotation_start, self.pacing, easing.outQuad),
                Updater.Ease( function(v) self.bottle1back:SetRotation(v) end, self.bottle1_rotation_end, self.bottle1_rotation_start, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottle2:SetRotation(v) end, self.bottle2_rotation_end, self.bottle2_rotation_start, self.pacing, easing.outQuad),
                Updater.Ease( function(v) self.bottle2back:SetRotation(v) end, self.bottle2_rotation_end, self.bottle2_rotation_start, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.bottles:SetScale(v*self.orientation_mult,v) end, assets.bottle_scale * 1.25, assets.bottle_scale * 1.0, self.pacing, easing.outQuad),
            },
        }
    )
    return self
end

function BottlesIndicator:OnGainFocus()
    BottlesIndicator._base.OnGainFocus(self)
    self:UpdateImage()
end

function BottlesIndicator:OnLoseFocus()
    BottlesIndicator._base.OnLoseFocus(self)
    self:UpdateImage()
end

function BottlesIndicator:OnGainHover()
    BottlesIndicator._base.OnGainHover(self)
    self:UpdateImage()
end

function BottlesIndicator:OnLoseHover()
    BottlesIndicator._base.OnLoseHover(self)
    self:UpdateImage()
end

-- To be displayed on the left side of the screen
function BottlesIndicator:OrientLeft()
    self.left = true
    self.orientation_mult = self.left and -1 or 1
    self:UpdateImage()
    return self
end

-- To be displayed on the right side of the screen
function BottlesIndicator:OrientRight()
    self.left = false
    self.orientation_mult = self.left and -1 or 1
    self:UpdateImage()
    return self
end

function BottlesIndicator:Refresh( count )
    self.count = count or 0
    if self.count <= 0 then
        self.counter:SetGlyphColour( 0x3E8D8Bff )
        self.bottle1:SetTexture( assets.bottle_empty )
        self.bottle2:SetTexture( assets.bottle_empty )
    else
        self.counter:SetGlyphColour( UICOLOURS.SUBTITLE )
        self.bottle1:SetTexture( assets.bottle_normal )
        self.bottle2:SetTexture( assets.bottle_normal )
    end
    self.counter:SetText( tostring( self.count ))
    return self
end

function BottlesIndicator:HasBottles()
    return self.count and self.count > 0
end