require "ui/widgets/dailyprogress"

--------------------------------------------------------------------------------

local BrawlProgress = class( "Widget.BrawlProgress", Widget.DailyProgress )

function BrawlProgress:init()
    BrawlProgress._base.init( self )
    
    self.day_spacing = 10

    self.label_bg_border_scale = 0.5
    self.label_container = self:AddChild( Widget() )
        :SendToBack()
    self.label_bg = self.label_container:AddChild( Widget.Panel( engine.asset.Texture("UI/brawl_label.tex") ) )
        :SetNineSliceCoords( 80, 56, 80, 58 )
        :SetNineSliceBorderScale( self.label_bg_border_scale )
        :Bloom( 0.03 )
    self.label = self.label_container:AddChild( Widget.Label( "title", FONT_SIZE.SMALL_TEXT, LOC"UI.PROGRESS.TITLE" ) )
        :SetGlyphColour( UICOLOURS.DARK_BODYTEXT )
        :SetPos( 0, 0 )
        :Bloom(0.05)
    local w, h = self.label:GetSize()
    w = w + 144
    h = h + 0
    self.label_bg:SetSize( w, h )
        -- :SetScale( 1, -1 )
    self.label:LayoutBounds( "left", "top", self.label_bg )
        :Offset( 20, -2.5 )

    self.bg_border = 60
    self.bg_border_scale = 0.5
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/brawl_container.tex") ) )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :Bloom( 0.05 )
        :SendToBack()

end

function BrawlProgress:OnAdded()
    local progress = TheGame:GetGameState():GetMainQuest():DefFn("get_progression")
    for k, dayplan in ipairs(progress) do
        local icon = self.icons_container:AddChild( Widget.BrawlProgressIcon(self.icon_size) )
            :SetToolTip( loc.format(LOC"UI.PROGRESS.DAY", k) )
            :LayoutBounds( "after", "center" )
            :Offset( self.day_spacing, 0 )
            :SetTypeDay( k )
        local day_icons = {} -- A container of the current day's icons
        for j, event in ipairs(dayplan) do
            local icon = self.icons_container:AddChild( Widget.BrawlProgressIcon(self.icon_size) )
                :LayoutBounds( "after", "center" )
                :Offset( j==1 and self.icon_spacing+2 or self.icon_spacing, 0 )

            if event.icon or event.tt then
                icon:SetTypeCustom(event.icon or global_images.circle, event.tt or "")
            else
                icon:SetTypeBalanced()
                local quest = event.quest or event.chosen_quest
                local qdef = quest and Content.GetQuestDef(quest)
                if qdef then
                    if qdef.focus == QUEST_FOCUS.COMBAT then
                        icon:SetTypeCombat()
                    elseif qdef.focus == QUEST_FOCUS.NEGOTIATION then
                        icon:SetTypeNegotiation()
                    elseif qdef.focus == QUEST_FOCUS.BALANCED then
                        icon:SetTypeBalanced()
                    end
                elseif event.boss then
                    icon:SetTypeBoss( event.boss )
                end
            end
            table.insert(self.events, icon)
            table.insert(day_icons, icon)
        end
        -- Add a line connecting today's icons
        local x1 = day_icons[1]:GetPos()
        local x2 = day_icons[#day_icons]:GetPos()
        local line = self:AddChild( Widget.SolidBox( x2-x1, 2, UICOLOURS.SUBTITLE ) )
            :SetName( "DAY" .. k .. "CONNECTOR_LINE" )
            :LayoutBounds( "left", "center", day_icons[1] )
            :Offset( self.icon_size/2 )
            :SendBehind( self.icons_container )
    end
    self.setup = true
    self:Refresh(progress)
end

function BrawlProgress:Refresh(progress)
    if not self.setup then self:OnAdded() end
    
    progress = progress or TheGame:GetGameState():GetMainQuest():DefFn("get_progression")
    local k = 1
    for _, day in ipairs(progress) do
        for _, event in ipairs(day) do
            self.events[k]:SetCurrent( event.right_now )
            if event.success then
                self.events[k]:SetResultSuccess()
            elseif event.failed then
                self.events[k]:SetResultFail()
            end
            k = k + 1
        end
    end

    local w, h = self.icons_container:GetSize()
    w = w + 40
    h = h + 30
    local nineslice_w, nineslice_h = w + self.bg_border*2*self.bg_border_scale, h + self.bg_border*2*self.bg_border_scale
    self.bg:SetSize( nineslice_w, nineslice_h )
        :LayoutBounds( "center", "center", self.icons_container )

    self.label_container:LayoutBounds( "left", "top", self.bg )
        :Offset( 15, 27 )
end

------------------------------------------------------------------------------------------
-- Shows an individual icon in the brawl progress track

local BrawlProgressIcon = class( "Widget.BrawlProgressIcon", Widget )

function BrawlProgressIcon:init( size )
    BrawlProgressIcon._base.init( self )

    self.size = size or 40
    self.hitbox = self:AddChild( Widget.SolidBox( self.size, self.size, 0xff00ff00 ) )
    self.icon = self:AddChild( Widget.Image() )
        :SetSize( self.size, self.size )
        :Bloom( 0.02 )
    self.result = self:AddChild( Widget.Image() )
        :SetSize( self.size, self.size )
        :Bloom( 0.05 )

    self:SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "center", "above", w ):Offset( 0, 6 ) end )
end

function BrawlProgressIcon:SetTypeDay( day_number )
    if not self.label_day then self.label_day = self:AddChild( Widget.Label( "title", self.size*0.6 ) ):SetGlyphColour( 0xFFFBEAff ):SetPos( nil, 1.5 ):Bloom(0.05) end
    self.label_day:SetText( day_number )
    self.icon:SetTexture( engine.asset.Texture("UI/brawl_day.tex") )
    return self
end
function BrawlProgressIcon:SetTypeCombat()
    self.icon:SetTexture( engine.asset.Texture("UI/brawl_battle.tex") )
    self.tt_text = LOC"UI.PROGRESS.QUEST"
    self:SetToolTip( self.tt_text )
    return self
end
function BrawlProgressIcon:SetTypeCustom(icon, tt)
    self.icon:SetTexture( icon )
    self.tt_text = tt
    self:SetToolTip( tt )
    return self
end
function BrawlProgressIcon:SetTypeNegotiation()
    self.icon:SetTexture( engine.asset.Texture("UI/brawl_negotiation.tex") )
    self.tt_text = LOC"UI.PROGRESS.QUEST"
    self:SetToolTip( self.tt_text )
    return self
end
function BrawlProgressIcon:SetTypeBalanced()
    self.icon:SetTexture( engine.asset.Texture("UI/brawl_balanced.tex") )
    self.tt_text = LOC"UI.PROGRESS.QUEST"
    self:SetToolTip( self.tt_text )
    return self
end
function BrawlProgressIcon:SetTypeBoss( content_id )
    if content_id then
        local skins = Content.GetAllCharacterSkins( content_id )
        local boss = Agent( content_id, skins and skins[1] )
        local def = Content.GetCharacterDef( content_id )
        if def.icon then
            self.icon:SetTexture( def.icon )
        else
            self.icon:SetTexture( engine.asset.Texture("UI/brawl_boss.tex") )
        end
        self:SetToolTip( boss )
        self:SetToolTipClass( Widget.TooltipBoss )
    else
        self.icon:SetTexture( engine.asset.Texture("UI/brawl_boss.tex") )
        self.tt_text = LOC"UI.PROGRESS.BOSS_FIGHT"
        self:SetToolTip( self.tt_text )
        self:SetToolTipClass( nil )
    end
    return self
end

function BrawlProgressIcon:SetResultSuccess()
    self.result:SetTexture( engine.asset.Texture("UI/brawl_success.tex") )
    return self
end
function BrawlProgressIcon:SetResultFail()
    self.result:SetTexture( engine.asset.Texture("UI/brawl_fail.tex") )
    return self
end

function BrawlProgressIcon:SetCurrent( is_current_day )
    if self.tt_text then
        if is_current_day and not self.current then
            self.current = self:AddChild( Widget() )
            local aura_size = self.size*1.1
            self.aura_layer_1 = self.current:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer1_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5):RotateIndefinitely( -0.015 )
            self.aura_layer_2 = self.current:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer2_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5):RotateIndefinitely( 0.05 )
            self.aura_layer_3 = self.current:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer3_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5):RotateIndefinitely( -0.04 )

        elseif not is_current_day and self.current then
            self.current:Remove()
            self.current = nil
        end
    end
    return self
end

---------------------------------------------------------------------------------------------

local TooltipBoss = class("Widget.TooltipBoss", Widget.TooltipAgent )

function TooltipBoss:init()
    TooltipBoss._base.init( self )

    self.boss = Widget.Label( "title", 24, LOC "UI.PROGRESS.BOSS_FIGHT" ):LeftAlign()
        :SetGlyphColour( UICOLOURS.HEALTH ):Bloom(0.15)
    self:AddChild( self.boss )
end

function TooltipBoss:DoLayout( agent_or_guid )
    local ret = TooltipBoss._base.DoLayout( self, agent_or_guid )
    if ret == false then
        return false
    end

    self.boss:LayoutBounds( "left", "below", self.desc )
    self.bg:SizeToWidgets( 10, self.portrait, self.name, self.body, self.social_details )
end

