require "ui/widget"

local assets =
{
    card_scale = 0.5,
    card_normal = engine.asset.Texture( "UI/ic_deck_card.tex"),
    card_empty = engine.asset.Texture( "UI/ic_deck_card_empty.tex"),
    card_outline = engine.asset.Texture( "UI/ic_deck_outline.tex"),
}

-------------------------------------------------------------------------------
-- Widget wrapping a deck of cards.

local DeckWidget = class( "Widget.DeckWidget", Widget.Clickable )

function DeckWidget:init( deck, title, card_widget_class, view_deck_class )
    DeckWidget._base.init( self )

    self.card_widget_class = card_widget_class or Widget.NegotiationCard
    self.view_deck_class = view_deck_class or Screen.DeckScreen

    self.deck = deck
    self.tt = {}

    self.pacing = 0.1

    self.cards = Widget()

    self.card1 = Widget.Image( assets.card_normal )
    self.card1back = Widget.Image( assets.card_outline ):SetBloom( 0.2 )
    self.card2 = Widget.Image( assets.card_normal )
    self.card2back = Widget.Image( assets.card_outline ):SetBloom( 0.2 )
    self.card3 = Widget.Image( assets.card_normal )
    self.card3back = Widget.Image( assets.card_outline ):SetBloom( 0.2 )

    self.counter = self.card1:AddChild( Widget.Label( "body", 50, "0" ) ):SetGlyphColour( UICOLOURS.SUBTITLE ):SetBloom( 0.1 )

    self.cards:AddChildren{
        self.card1back,
        self.card2back,
        self.card3back,
        self.card3,
        self.card2,
        self.card1,
    }

    self.card2:Offset( 6, 0 )
    self.card2back:Offset( 6, 0 )
    self.card3:Offset( 12, 0 )
    self.card3back:Offset( 12, 0 )

    self:AddChild( self.cards ):SetScale( assets.card_scale )

    self.label = self:AddChild( Widget.Label( "title", 20, title and title:upper() or "" ) ):SetGlyphColour( UICOLOURS.SUBTITLE ):SetBloom( 0.1 )
    self.label:LayoutBounds( "center", "below", self.cards ):Offset( 0, -10 )

    self:OrientRight()
    self:RefreshDeck()

    -- Prepare for the Grab Attention animation
    self.cards_x, self.cards_y = self.cards:GetPos()
    self.cards_x_end1, self.cards_y_end1 = self.cards_x, self.cards_y + 10
    self.cards_x_end2, self.cards_y_end2 = self.cards_x, self.cards_y + 6

    self.title = ""
    self.subtitle = ""

    self:SetFocusable( false )

    self:SetOnClickFn(function() self:ViewDeck() end)
end

function DeckWidget:UpdateImage()
    local card2rotation = -4 * math.pi / 180
    local card3rotation = -8 * math.pi / 180
    if self.hover or self.focus then
        self:RunUpdater(
            Updater.Parallel{
                Updater.Ease2D( function(x,y) self.card2:SetPos(x,y) end, 6, 0, 12, 0, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.card2back:SetPos(x,y) end, 6, 0, 12, 0, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.card3:SetPos(x,y) end, 12, 0, 24, 0, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.card3back:SetPos(x,y) end, 12, 0, 24, 0, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.card2:SetRotation(v) end, 0, card2rotation, self.pacing, easing.linear),
                Updater.Ease( function(v) self.card2back:SetRotation(v) end, 0, card2rotation, self.pacing, easing.linear),
                Updater.Ease( function(v) self.card3:SetRotation(v) end, 0, card3rotation, self.pacing, easing.linear),
                Updater.Ease( function(v) self.card3back:SetRotation(v) end, 0, card3rotation, self.pacing, easing.linear),
            }
        )
        if self.reflection then self.reflection:ScaleTo( 1.0, 1.1, self.pacing, easing.outQuad ) end
        self.cards:ScaleTo( assets.card_scale * 1.0, assets.card_scale * 1.25, self.pacing, easing.outQuad )
    else
        self:RunUpdater(
            Updater.Parallel{
                Updater.Ease2D( function(x,y) self.card2:SetPos(x,y) end, 12, 0, 6, 0, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.card2back:SetPos(x,y) end, 12, 0, 6, 0, self.pacing, easing.outQuad),

                Updater.Ease2D( function(x,y) self.card3:SetPos(x,y) end, 24, 0, 12, 0, self.pacing, easing.outQuad),
                Updater.Ease2D( function(x,y) self.card3back:SetPos(x,y) end, 24, 0, 12, 0, self.pacing, easing.outQuad),

                Updater.Ease( function(v) self.card2:SetRotation(v) end, card2rotation, 0, self.pacing, easing.outQuad),
                Updater.Ease( function(v) self.card2back:SetRotation(v) end, card2rotation, 0, self.pacing, easing.outQuad),
                Updater.Ease( function(v) self.card3:SetRotation(v) end, card3rotation, 0, self.pacing, easing.outQuad),
                Updater.Ease( function(v) self.card3back:SetRotation(v) end, card3rotation, 0, self.pacing, easing.outQuad),
            }
        )
        if self.reflection then self.reflection:ScaleTo( 1.1, 1.0, self.pacing, easing.outQuad ) end
        self.cards:ScaleTo( assets.card_scale * 1.25, assets.card_scale * 1.0, self.pacing, easing.outQuad )
    end
end

function DeckWidget:OnGainFocus()
    DeckWidget._base.OnGainFocus(self)
    self:UpdateImage()
end

function DeckWidget:OnLoseFocus()
    DeckWidget._base.OnLoseFocus(self)
    self:UpdateImage()
end

function DeckWidget:OnGainHover()
    DeckWidget._base.OnGainHover(self)
    self:UpdateImage()
end

function DeckWidget:OnLoseHover()
    DeckWidget._base.OnLoseHover(self)
    self:UpdateImage()
end

function DeckWidget:GrabAttention()
    self:RunUpdater(
        Updater.Series{
            Updater.Ease2D( function(x,y) self.cards:SetPos(x, y) end, self.cards_x, self.cards_y, self.cards_x_end1, self.cards_y_end1, self.pacing*2, easing.outQuad),
            Updater.Ease2D( function(x,y) self.cards:SetPos(x, y) end, self.cards_x_end1, self.cards_y_end1, self.cards_x, self.cards_y, self.pacing*2, easing.outQuad),
            Updater.Ease2D( function(x,y) self.cards:SetPos(x, y) end, self.cards_x, self.cards_y, self.cards_x_end2, self.cards_y_end2, self.pacing*2, easing.outQuad),
            Updater.Ease2D( function(x,y) self.cards:SetPos(x, y) end, self.cards_x_end2, self.cards_y_end2, self.cards_x, self.cards_y, self.pacing*2, easing.outQuad),
            Updater.Wait( self.pacing*8 ),
            Updater.Ease2D( function(x,y) self.cards:SetPos(x, y) end, self.cards_x, self.cards_y, self.cards_x_end1, self.cards_y_end1, self.pacing*2, easing.outQuad),
            Updater.Ease2D( function(x,y) self.cards:SetPos(x, y) end, self.cards_x_end1, self.cards_y_end1, self.cards_x, self.cards_y, self.pacing*2, easing.outQuad),
            Updater.Ease2D( function(x,y) self.cards:SetPos(x, y) end, self.cards_x, self.cards_y, self.cards_x_end2, self.cards_y_end2, self.pacing*2, easing.outQuad),
            Updater.Ease2D( function(x,y) self.cards:SetPos(x, y) end, self.cards_x_end2, self.cards_y_end2, self.cards_x, self.cards_y, self.pacing*2, easing.outQuad),
        }
    )
    return self
end

function DeckWidget:SetTitle( title )
    self.label:SetText( title and title:upper() or "" )
    --self.label:LayoutBounds( "center", "below", self.cards ):Offset( 0, -10 )
    return self
end

function DeckWidget:SetDialogTitle( title, subtitle )
    self.title = title
    self.subtitle = subtitle
    return self
end

function DeckWidget:ViewDeck()
    local cards
    if is_instance( self.deck, CardEngine.Deck ) then
        cards = self.deck:GetCards()
    elseif is_instance( self.deck, Deck ) then
        cards = self.deck:GenerateCards()
    end

    local screen = self.view_deck_class( cards, nil, self.card_widget_class, nil, true )
    screen:Refresh( cards )
    screen:SetTitles( self.title, nil, self.subtitle )
    if self.other_deck_data then
        screen:SetOtherDeckButton( self.other_deck_data.CARDS, self.other_deck_data.TITLE, nil, self.other_deck_data.SUBTITLE )
    end
    TheGame:FE():InsertScreen( screen )

    return self
end

function DeckWidget:SetOtherDeckButton( deck, title, subtitle, desc )
    -- For when we want our deck screen to have a link to a different deck
    local cards
    if is_instance( deck, CardEngine.Deck ) then
        cards = deck:GetCards()
    elseif is_instance( deck, Deck ) then
        cards = deck:GenerateCards()
    end
    self.other_deck_data = {
        CARDS = cards,
        TITLE = title,
        SUBTITLE = subtitle,
        TEXT = desc
    }
end

function DeckWidget:OrientLeft()
    return self
end

function DeckWidget:OrientRight()
    return self
end

function DeckWidget:ShowReflection( height, yoffset )
    local w, h = self.cards:GetSize()
    self.reflection = self:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) ):SetSize(w-20, height or 40):SetBloom(0.1):SetHiddenBoundingBox(true):SetTintColour(UICOLOURS.SUBTITLE)
    self.reflection:LayoutBounds("center","below",self.cards):Offset(0, yoffset or -20)
    return self
end

function DeckWidget:PrepareAnimation()
    self.cards:SetTintAlpha(0)
    self.label:SetTintAlpha(0)
    if self.reflection then self.reflection:SetTintAlpha(0) end
    return self
end

function DeckWidget:AnimateIn()
    -- Prepare for animation by getting current values
    local cards_x, cards_y = self.cards:GetPos()
    local label_x, label_y = self.label:GetPos()

    -- Start animation
    self:RunUpdater(
        Updater.Series{
            Updater.Parallel{
                Updater.Ease2D( function(x,y) self.cards:SetPos(x,y) end, cards_x, cards_y - 30, cards_x, cards_y, self.pacing, easing.linear),
                Updater.Ease( function(v) self.cards:SetTintAlpha(v) end, 0, 1, self.pacing*2, easing.linear),
            },
            Updater.Parallel{
                Updater.Ease2D( function(x,y) self.label:SetPos(x,y) end, label_x, label_y - 10, label_x, label_y, self.pacing, easing.linear),
                Updater.Ease( function(v) self.label:SetTintAlpha(v) end, 0, 1, self.pacing*2, easing.linear),
            }
        }
    )
    if self.reflection then self.reflection:AlphaTo( 0.6, self.pacing*5, easing.outQuad ) end
    return self
end

function DeckWidget:RefreshDeck( deck )
    if deck then
        self.deck = deck
    end 

    local card_count = 0
    if self.deck then
        card_count = card_count + self.deck:CountCards()
    end

    if card_count <= 0 then
        self.counter:SetGlyphColour( 0x3E8D8Bff )
        self.card1:SetTexture( assets.card_empty )
        self.card2:SetTexture( assets.card_empty )
        self.card3:SetTexture( assets.card_empty )
    else
        self.counter:SetGlyphColour( UICOLOURS.SUBTITLE )
        self.card1:SetTexture( assets.card_normal )
        self.card2:SetTexture( assets.card_normal )
        self.card3:SetTexture( assets.card_normal )
    end

    self.counter:SetText( tostring( card_count ))
    
    return self.deck
end

function DeckWidget:HasCards()
    if self.deck then
        return self.deck:CountCards() > 0
    end
    return false
end

