require "ui/widget"
require "ui/widgets/particlesystem"
require "util/util"
require "ui/widgets/segmentedbar"
local filepath = require "util/filepath"

--old defaults
--38, 30
local assets = {
    glow = engine.asset.Texture( "UI/cardview_glow.tex"),
    common_outline = engine.asset.Texture( "UI/cardview_common_outline.tex" ),
    common_back = engine.asset.Texture( "UI/cardview_common_back.tex"),
    common_overlay = engine.asset.Texture( "UI/cardview_common_overlay.tex"),
    common_upgrade = engine.asset.Texture( "UI/cardview_common_upgrade.tex"),
    lock = engine.asset.Texture("UI/card_lock.tex"),
    motd_bg = engine.asset.Texture("UI/motd_info_bg.tex"),
    shine = engine.asset.Texture( "UI/cardview_shine.tex"),
    negotiation_wild =  engine.asset.Texture( "negotiation/negotiation_wild.tex" ),
    wild = engine.asset.Texture( "battle/cards_wild.tex" ),
    smoke = engine.asset.Texture( "particles/smoke.tex"),
    charges_full = engine.asset.Texture("UI/ic_charges_full.tex"),
    charges_empty = engine.asset.Texture("UI/ic_charges_empty.tex"),
    upgradebar = engine.asset.Texture("UI/cardupgrade_bar.tex")
}

local SMALLEST_TITLE_TEXT_SZ, LARGEST_TITLE_TEXT_SZ = 20, 50
local SMALLEST_DESC_TEXT_SZ, LARGEST_DESC_TEXT_SZ = 20, 38

local CARD_DESC_W = 245
local CARD_DESC_H = 160

local CLR_OUTLINE = UICOLOURS.FOCUS
local CLR_DARK_LABEL = UICOLOURS.DARK_BODYTEXT
local CLR_BONUS = 0xCCFF00FF
local CLR_PENALTY = 0xCC5555FF
local CLR_DISABLED = 0x706f69ff

-- Bloom level for card outlines.
local OUTLINE_BLOOM = 0.2
local PARTICLE_BLOOM = 2
local PARTICLE_BLOOM_PREVIEW = 0.9

-- These must be adjusted based on the bg texture of the card for hit-testing.
local CARD_BOUNDS = { -141, -255, 141, 260  }

-- 370*542
local CARD_BOUNDING_BOX = { -185, -271, 185, 271  }

local CARD_ART_BOUNDS = { 13, 88, 260, 168 }


local card_rarity_assets = 
{
    [ CARD_RARITY.BASIC ] = {
        bg = engine.asset.Texture( "UI/cardview_common_back.tex" ),
        overlay = engine.asset.Texture( "UI/cardview_common_overlay.tex" ),
        outline = engine.asset.Texture( "UI/cardview_common_outline.tex" ),
        upgrade = engine.asset.Texture( "UI/cardview_common_upgrade.tex" ),
        nameplate = engine.asset.Texture( "UI/cardview_common_nameplate.tex" ),
        shinecolor = 0xae7858ff,
        shinebloom = 0.3,
        upgradeshinemask = engine.asset.Texture( "UI/cardview_common_upgrade_shinemask.tex" ),
    },
    [ CARD_RARITY.COMMON ] = {
        bg = engine.asset.Texture( "UI/cardview_common_back.tex" ),
        overlay = engine.asset.Texture( "UI/cardview_common_overlay.tex" ),
        outline = engine.asset.Texture( "UI/cardview_common_outline.tex" ),
        upgrade = engine.asset.Texture( "UI/cardview_common_upgrade.tex" ),
        nameplate = engine.asset.Texture( "UI/cardview_common_nameplate.tex" ),
        shinecolor = 0xae7858ff,
        shinebloom = 0.3,
        upgradeshinemask = engine.asset.Texture( "UI/cardview_common_upgrade_shinemask.tex" ),
    },
    [ CARD_RARITY.UNCOMMON ] = {
        bg = engine.asset.Texture( "UI/cardview_uncommon_back.tex" ),
        overlay = engine.asset.Texture( "UI/cardview_uncommon_overlay.tex" ),
        outline = engine.asset.Texture( "UI/cardview_uncommon_outline.tex" ),
        upgrade = engine.asset.Texture( "UI/cardview_uncommon_upgrade.tex" ),
        nameplate = engine.asset.Texture( "UI/cardview_uncommon_nameplate.tex" ),
        shinecolor = 0x9fd4ffff,
        shinebloom = 0.3,
        shinemask = engine.asset.Texture( "UI/cardview_uncommon_shinemask.tex" ),
        upgradeshinemask = engine.asset.Texture( "UI/cardview_uncommon_upgrade_shinemask.tex" ),
    },
    [ CARD_RARITY.RARE ] = {
        bg = engine.asset.Texture( "UI/cardview_rare_back.tex" ),
        overlay = engine.asset.Texture( "UI/cardview_rare_overlay.tex" ),
        outline = engine.asset.Texture( "UI/cardview_rare_outline.tex" ),
        upgrade = engine.asset.Texture( "UI/cardview_rare_upgrade.tex" ),
        nameplate = engine.asset.Texture( "UI/cardview_rare_nameplate.tex" ),
        shinecolor = 0xf7e068ff,
        shinebloom = 0.4,
        shinemask = engine.asset.Texture( "UI/cardview_rare_shinemask.tex" ),
        upgradeshinemask = engine.asset.Texture( "UI/cardview_rare_upgrade_shinemask.tex" ),
    },
    [ CARD_RARITY.UNIQUE ] = {
        bg = engine.asset.Texture( "UI/cardview_rare_back.tex" ),
        overlay = engine.asset.Texture( "UI/cardview_rare_overlay.tex" ),
        outline = engine.asset.Texture( "UI/cardview_rare_outline.tex" ),
        upgrade = engine.asset.Texture( "UI/cardview_rare_upgrade.tex" ),
        nameplate = engine.asset.Texture( "UI/cardview_rare_nameplate.tex" ),
        shinecolor = 0xf7e068ff,
        shinebloom = 0.4,
        shinemask = engine.asset.Texture( "UI/cardview_rare_shinemask.tex" ),
        upgradeshinemask = engine.asset.Texture( "UI/cardview_rare_upgrade_shinemask.tex" ),
    },
}

------------------------------------------------------------------------------------------
-- Common code for card widgets.

local CardWidget = class( "Widget.CardWidget", Widget )
CardWidget.CLR_OUTLINE = CLR_OUTLINE
CardWidget.STYLE = MakeEnum{ "NORMAL", "UPGRADE", "WILD" }
CardWidget.can_focus = true

CardWidget.CONTROL_MAP =
{
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function(self)
            return self.on_click_handler and not self.hide_accept_hint
        end,
        -- fn = function(self)
        --     self.on_click_handler()
        --     self.down = false
        --     return true
        -- end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.SELECT", Controls.Digital.MENU_ACCEPT))
        end
    },
    {
        control = Controls.Digital.GAMEPAD_RT,
        -- test = function(self)
        --     return self.parent and not self.parent:GetAncestor( Widget.CardArrangement )
        -- end,
        fn = function(self)
            AUDIO:PlayEvent( "event:/ui/battle_deck/click_card_in_menu" )
            self:ShowCardInfoPopup()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CARD_INFO", Controls.Digital.GAMEPAD_RT))
        end
    },
}

CardWidget.Width = CARD_BOUNDING_BOX[3]*2
CardWidget.Height = CARD_BOUNDING_BOX[4]*2


function CardWidget:init( assets_in, defs )
    Widget.init( self )

    self.hover_check = true
    self.assets = assets_in
    self.defs = defs
    self.card_features = {}
    self.card_info_popup_mode = CARD_DETAILS_MODE.LOCKED

    self.scroll_list_mode = false -- False if in hand, true if in a scrolling list. Used for mouse dragging behaviour
    self.particles_enabled = false -- If particles should be shown on this card. True for Fight/Negotiation in-game, but not for UI
    
    -- Card back shadow, usually shown in purple for Negotiation or orange for Battle
    self.shadow = self:AddChild( Widget.Image( assets.glow ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetHiddenBoundingBox( true )
        :SetTintAlpha(0.3)

    -- Card outline glow, shown ingame, if this card is playable
    self.outline_bg = Widget.Image( assets.common_outline):SetHiddenBoundingBox( true ):SetBloom( OUTLINE_BLOOM ):Hide()
    self:AddChild( self.outline_bg )

    self.contents = self:AddChild( Widget() )

    -- Card illustration image
    self.art = self.contents:AddChild( Widget.Image() )
        :SetSize(260, 168)
        :Offset( 13, 88 )


    -- This is the tintable part
    self.bg = self.contents:AddChild( Widget.Image( assets.common_back) )
    -- This is the non-tintable part over it
    self.bg_overlay = self.contents:AddChild( Widget.Image( assets.common_overlay ) )
    -- This is the upgrade part
    self.upgrade_overlay = self.contents:AddChild( Widget.Image( assets.common_upgrade ) )
        :Hide()

    -- Animated reflection shine on the card is no longer added here. It is now lazily added when needed.

    -- Icon showing what kind of card this is ( Hostility, Diplomacy, Attack, etc )
    self.type_icon = self.contents:AddChild( Widget.Image() )
        :SetBloom( 0.05 )
        :SetSize( 46, 46 )
        :LayoutBounds("center", "center", self.bg)
        :Offset(-131, -35)

    -- Cost value on the top left
    self.cost_label = self.contents:AddChild( Widget.Label( "title", 90 ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetSize( 110, 110 )
        :EnableOutline(true)
        :CenterH()
        :CenterV()
        :LayoutBounds( "left", "top", self.bg )
        :Offset( 16, 6 )

    -- Damage value on the top of the card
    self.effect_label = self.contents:AddChild( Widget.Label( "title", 60 ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetSize( 180, 60 )
        :CenterH()
        :CenterV()
        :LayoutBounds( "center", "top", self.bg )
        :Offset( 22, -26 )

    -- Card name
    self.title = self.contents:AddChild( Widget.Label( "title", 38 ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetSize(240,40)
        :LeftAlign()

    -- Card experience bar
    self.xp_bar = self.contents:AddChild( Widget.CardWidgetUpgradeBar( 240 ) )
        :Hide()

    self.feature_root = self.contents:AddChild(Widget())

    -- Right clicks to any cards will show the info popup by default
    self:ShowInfoOnRightClick()


    local x, y, w, h = table.unpack(CARD_ART_BOUNDS)
    self.art:SetSize(w, h)
    self.art:SetPos(x, y)
end

function CardWidget:SetLocked( locked )
    if locked then
        self.card_locked = true

        if not self.lock_icon then
            self.lock_icon = self:AddChild( Widget.Image( assets.lock ) )
                :SetSize( 200, 200 )
                :SetTintColour( UICOLOURS.SUBTITLE )
                :SetTintAlpha( 0.4 )
                :Bloom( 0.2 )
        end

        self.bg:SetTintColour( 0x021A18FF )
        self.bg_overlay:SetTintColour( 0x021A18FF )
        self.upgrade_overlay:SetTintColour( 0x021A18FF )
        self.art:SetTexture( assets.motd_bg )
        self.art:SetTintColour( 0x021A18FF )
        self.cost_label:Hide()
        self.type_icon:Hide()
        self.title:Hide()
        self.xp_bar:Hide()
        self.effect_label:Hide()
        self.feature_root:Hide()
        if self.charges_root then self.charges_root:Hide() end

        self:SetToolTipClass( nil )
        self:SetToolTip( LOC"UI.CARDCOMPENDIUM.CARD_LOCKED_TT" )

    end
    return self
end

-- If another screen wants to customize the hint shown for selecting a card
-- Set this to false
function CardWidget:SetAcceptHintShown( shown )
    self.hide_accept_hint = not shown
    return self
end

function CardWidget:GetTitle()
    return self.title:GetText() or ""
end

function CardWidget:GetBoundingBox()
    return table.unpack(CARD_BOUNDING_BOX)
end


function CardWidget:SetChooseFn( fn )
    self.choose_fn = fn
end

function CardWidget:OnAdded()
    TheGame:GetEvents():ListenForAny( self )
end

function CardWidget:OnRemoved()
    TheGame:GetEvents():RemoveListener( self )
end

function CardWidget:OnGlobalEvent( eventname, ... )
    if eventname == "colourblind_mode_changed" then
        if self.card then
            local card_back_tint, type_icon, text_colour, tt = self:RefreshCardType( self.card )
            self.bg:SetTintColour( card_back_tint )
            self.type_icon:SetTexture( type_icon )
                :SetTintColour( text_colour )
        end
    end
end


function CardWidget:AddShine( clr, bloom, mask_tex )
    assert( mask_tex )

    -- Animated reflection shine on the card
    if not self.mask then
        local idx = self.contents:IndexOf(self.type_icon)
        self.mask = self.contents:AddChild( Widget(), idx )
            :SetStencilContext( true )
            :SetHiddenBoundingBox( true )
        self.mask_image = Widget.Image()
            :SetHiddenBoundingBox( true )
        self.shine = Widget.Image( assets.shine )
            :SetHiddenBoundingBox( true )
            :SetSize( 900, 900 )
            :SetTintColour( clr or UICOLOURS.WHITE )
            :Bloom( bloom or 0.0 )
            :SetRotation(0.30)
        self.mask:AddChildren
        {
            self.mask_image
                :SetMask( true ),
            self.shine
                :SetStencilTest( true )
                :SetPos( 0, 550 )
        }

        assert( self.shine_coro == nil )
        self.shine_coro = self:StartCoroutine( self.ShowShine, self )

    end
    
    self.mask_image:SetTexture( mask_tex )
end


function CardWidget:IsShining()
    return self.shine ~= nil
end

function CardWidget:RemoveShine()
    -- Animated reflection shine on the card
    if self.mask then
        if self.shine_coro then
            self.shine_coro:Stop()
            self.shine_coro = nil
        end

        self.mask:Remove()

        self.mask_image = nil
        self.shine = nil
        self.mask = nil
    end
end


function CardWidget:UpdateDescription(card_engine, force)
    local desc
    
    table.clear( self.card_features )
    if self.style == self.STYLE.WILD then
        desc = LOC"UI.CARD_WILD_DESC"
    else
        desc = CardEngine.GenerateCardDesc( self.card, self, card_engine, self.defs, self.style == self.STYLE.UPGRADE )
    end

    if desc ~= self.last_desc or force then
        self.last_desc = desc
        self.feature_root:DestroyAllChildren()
        self:PostText( desc )
    end
end

function CardWidget:RefreshCard( card, card_engine, style )
    --self.times_refreshed = (self.times_refreshed or 0) + 1    
    --print ("REFRESH CARD", card, self.times_refreshed, debugstack())

    self.card = card
    self.style = style

    self:Enable()

    if style == self.STYLE.WILD then
        local tex
        if is_instance( card, Battle.Card ) then
            tex = assets.negotiation_wild
        else
            tex = assets.wild
        end
        self.art:SetTexture( tex )

    else
        self.art:SetTexture( card.icon )
    end

    local new_title
    if style == self.STYLE.WILD then
        new_title = LOC"UI.CARD_WILD_TITLE"
    else
        new_title = self.card:GetName()
	end
    
    if self.title:GetText() ~= new_title then
        self.title:SetText( new_title )
        self.title:TryFitToRegion(SMALLEST_TITLE_TEXT_SZ, LARGEST_TITLE_TEXT_SZ)
    end

    self.upgrade_overlay:SetShown( card:IsUpgraded() )

    -- Update card background according to its rarity
    local rarity_assets = card_rarity_assets[ card.rarity or CARD_RARITY.COMMON ]
    self.bg:SetTexture( rarity_assets.bg or assets.common_back )
    self.bg_overlay:SetTexture( rarity_assets.overlay or assets.common_overlay )
    self.upgrade_overlay:SetTexture( rarity_assets.upgrade or assets.common_upgrade )
    self.outline_bg:SetTexture( rarity_assets.outline or assets.common_outline )

    if self.shine_enabled and rarity_assets.shinecolor then
        if card:IsUpgraded() and rarity_assets.upgradeshinemask then
            if not self:IsShining() then
                self:AddShine( rarity_assets.shinecolor, rarity_assets.shinebloom, rarity_assets.upgradeshinemask )
            end
        elseif not card:IsUpgraded() and rarity_assets.shinemask then
            if not self:IsShining() then
                self:AddShine( rarity_assets.shinecolor, rarity_assets.shinebloom, rarity_assets.shinemask )
            end
        end
    else
        if self:IsShining() then
            self:RemoveShine()
        end
    end

    self:RefreshDuplicate( rarity_assets )
    self:RefreshElectric()
    self:RefreshSlime()

    if card.rarity == CARD_RARITY.RARE then
        self.bg_overlay:Bloom(0.00)
    end

    if style == self.STYLE.WILD then
        self.cost_label:SetText( "-" )
    else
        local cost_txt, details = self:RefreshCostLabel( card, card_engine )
        self.cost_label:SetText( cost_txt ):SetToolTip( details )
    end

    if style == self.STYLE.WILD then
        self.effect_label:SetText( "" )
    else
        local txt, details = self:RefreshEffectLabel( card, card_engine )
        self.effect_label:SetText( txt ):SetToolTip( details )
    end

    self:UpdateDescription(card_engine)

    if card_engine then
        

        local ok, reason
        if self.choose_fn then
            ok, reason = self.choose_fn( card )
        else
            ok, reason = card_engine:CanPlayCard( card )

            if card.PreReq and card.engine and card:PreReq( card_engine, card.target ) then
                --it passed a special requirement, so make it glow more
                self:SetOutline( UICOLOURS.UPGRADE, true )
                self.outline_bg:SetScale(1.025)
            else
                self.outline_bg:SetScale(1.0)
            end
        end
        self:ReqCondition( ok, reason )
        
        if ok then
            self:SetOutline( CLR_OUTLINE )
        end
    else
        self:SetOutline( nil )
        self.outline_bg:SetScale(1)
    end

    self:RefreshCharges( card, card_engine )
    self:RefreshXP( card, card_engine )

    if style == self.STYLE.WILD then
        self:SetToolTip( nil )
    else
        self:SetToolTip( { CARD = card, FEATURES = self.card_features } )
        self:SetToolTipClass( Widget.FullCardPanel )
    end

    return self
end

function CardWidget:GetShownCard()
    return self.card
end

function CardWidget:ShowParticles( show )

    if show then
        if self.particles_top == nil then
            local particle_size = { 20, 30 }
            local ttl_mult = 0.6
            self.particles_top = self:AddChild( Widget.ParticleSystem() ):SetPos(0,150)
            self.particles_left = self:AddChild( Widget.ParticleSystem() ):SetPos(-140,0)
            self.particles_right = self:AddChild( Widget.ParticleSystem() ):SetPos(140,0)

            self.particles_top:SendBehind( self.outline_bg )
            self.particles_left:SendBehind( self.outline_bg )
            self.particles_right:SendBehind( self.outline_bg )

            self.particles_top:LoadParams("card_edge")
            self.particles_left:LoadParams("card_edge")
            self.particles_right:LoadParams("card_edge")

            self.particles_top:GetEmitter(1):SetEmitterBounds(0,70,300,2)
            self.particles_left:SetRotation(90*DEGREES):GetEmitter(1):SetEmitterBounds(0,-5, 475, 2)
            self.particles_right:SetRotation(-90*DEGREES):GetEmitter(1):SetEmitterBounds(0,-5, 475, 2)
        end

        self.particles_top:Restart()
        self.particles_left:Restart()
        self.particles_right:Restart()

    else
        if self.particles_top then
            self.particles_top:Stop()
            self.particles_left:Stop()
            self.particles_right:Stop()
        end
    end
    return self
end

function CardWidget:RemoveParticles()
    if self.particles_top then
        self.particles_top:Remove()
        self.particles_top = nil
        self.particles_left:Remove()
        self.particles_left = nil
        self.particles_right:Remove()
        self.particles_right = nil
    end
    return self
end

function CardWidget:SetHighlight( hilite )
    if hilite then
        self:SetOutline( UICOLOURS.SUBTITLE )
    else
        self:SetOutline( self.CLR_OUTLINE )
    end
    return true
end

function CardWidget:AnimateChosen()
    self.anim = nil

    self:Enable() -- Shouldn't appear disabled once resolved.
    self:SetTintAlpha( 1.0 )

    local w, h = self:GetFE():GetScreenDims()
    local tx, ty = self.parent:TransformFromWorld( w/2, h/2 )

    self:MoveTo( tx, ty, 0.2, easing.outCubic )
    self:ScaleTo( self.sx, 0.8, 0.2, easing.outCubic )
    self:RotateTo( 0, 0.2 )
    return self
end

function CardWidget:AnimateUpgraded()

    self:Delay( 0.1 )
    self:RefreshCard( self.card, self.card.minigame )
    
    local particles = self:AddChild( Widget.ParticleSystem() )
    particles:LoadParams( "card_expend_burst" )
    particles:StartCoroutine( function()
        while not particles:IsDone() do
            coroutine.yield()
        end
        particles:Remove()
    end )
end

function CardWidget:AnimateDiscard()

    local particles = self.parent:AddChild( Widget.ParticleSystem() )
    particles:SetPos( self:TransformToParent( 0, 0 ))
    particles:SetScale( 0.5, 0.5 )
    particles:LoadParams( "card_discard_purple_haze" )
    particles:StartCoroutine( function()
        while not particles:IsDone() do
            coroutine.yield()
        end
        particles:Remove()
    end )

    self:Delay( 0.1 )

end

function CardWidget:SetParticlesEnabled( enabled )
    self.particles_enabled = enabled
    return self
end

function CardWidget:SetShineEnabled( enabled )
    self.shine_enabled = enabled
    return self
end

function CardWidget:SetOutline( tint, animate )
    if self.outline_tint == tint then
        return self
    end

    self.outline_tint = tint

    if tint and self:IsEnabled() then
        self.outline_bg:Show()
        local r,g,b,a = HexColour(tint)
        
        if animate then
            self.outline_bg:ColourFromCurrentTo( r, g, b, a, 0.2 )
        else
            self.outline_bg:SetTintColour( r, g, b, a )
        end

        if self.particles_enabled then
            self:ShowParticles( true )
            if animate then
                self.particles_top:ColourFromCurrentTo( r, g, b, a, 0.2 )
                self.particles_left:ColourFromCurrentTo( r, g, b, a, 0.2 )
                self.particles_right:ColourFromCurrentTo( r, g, b, a, 0.2 )
            else
                self.particles_top:SetTintColour( r, g, b, a)
                self.particles_left:SetTintColour( r, g, b, a)
                self.particles_right:SetTintColour( r, g, b, a)
            end
        end

    else
        if animate then
            self.outline_bg:AlphaTo( 0, 0.4 )
        else
            self.outline_bg:Hide()
        end
        if self.particles_enabled then self:ShowParticles( false ) end
    end
    return self
end

function CardWidget:GetBonusColour( card )
    return CLR_BONUS
end

function CardWidget:GetPenaltyColour( card )
    return CLR_PENALTY
end

function CardWidget:GetDisabledColour( card )
    return CLR_DISABLED
end

function CardWidget:OnGainHover()
    -- We don't want cards within a scroll panel to gain down when the mouse is getting over them, in case it's dragging the list
    local scrollpanel = self:GetAncestor( Widget.ScrollPanel )
    if not scrollpanel then self.down = true end
    AUDIO:PlayParamEvent(SoundEvents.card_mouseover, "position", self:GetPanPos())
end

function CardWidget:OnLoseHover()
    self.down = false
    self.alt_down = false
    self:StopUpdating()
    AUDIO:PlayParamEvent(SoundEvents.card_mouseoff, "position", self:GetPanPos())
end

function CardWidget:OnGainFocus()
    AUDIO:PlayParamEvent(SoundEvents.card_mouseover, "position", self:GetPanPos())
end

function CardWidget:OnLoseFocus()
    self.down = false
    self:StopUpdating()
    AUDIO:PlayParamEvent(SoundEvents.card_mouseoff, "position", self:GetPanPos())
end

function CardWidget:FitDescText(smallest_text_size, largest_text_size)

    local size = math.round( (smallest_text_size + largest_text_size)/2)
    while smallest_text_size < largest_text_size do
        for k,v in pairs(self.feature_root.children) do
            v:SetFontSize(size)
            v:LayoutBounds("left", "below"):Offset( 0, -4 )
        end

        local textw, texth = self.feature_root:GetSize()

        local new_size
        if textw > CARD_DESC_W or texth > CARD_DESC_H then
            largest_text_size = size
            new_size = math.floor( (smallest_text_size + size)/2)

        else
            smallest_text_size = size
            new_size = math.floor( (largest_text_size + size)/2)
        end
        if new_size == size then
            break
        else
            size = new_size
        end
    end
    self.feature_root:LayoutBounds( "left", "below", self.art ):Offset( -5, -80 )

end

function CardWidget:GetDescText()
    if not self.desc_text then
        self.desc_text = ""
        for k,v in pairs(self.feature_root.children) do
            self.desc_text = self.desc_text .. ( v:GetText() or "" ) .. " "
        end
    end
    return self.desc_text
end

function CardWidget:PostText( txt )
    
    local card_width = self.bg:GetSize()
    local w = Widget.Label( "body", 30, txt )
        :SetWordWrap( true )
        :LeftAlign()
        :SetAutoSize( CARD_DESC_W )
        :SetGlyphColour( UICOLOURS.DARK_BODYTEXT )
        :SetHoverLinkGlyphColour( UICOLOURS.DARK_TEXT_LINK )
    self.feature_root:AddChild( w )
    
    self:FitDescText(SMALLEST_DESC_TEXT_SZ, LARGEST_DESC_TEXT_SZ)
    return w
end

function CardWidget:CheckHit(x,y)
    local lx, ly = self:TransformFromWorld(x,y)
    return lx >= CARD_BOUNDS[1] and lx <= CARD_BOUNDS[3] and ly >= CARD_BOUNDS[2] and ly <= CARD_BOUNDS[4]
    -- return self.bg:CheckHit(x, y) or self.cost_icon:CheckHit(x, y)
end

function CardWidget:RefreshDuplicate( rarity_assets )
    if self.card and self.card:IsTemporary() then
        if not self.duplicate_widget and not self.eroding then
            self.duplicate_widget = self.contents:AddChild( Widget() )
                :SetStencilContext( true )
                :SetHiddenBoundingBox( true )
                :SendToFront()
                :SendBehind( self.type_icon )
            self.duplicate_mask_image = Widget.Image()
                :SetTexture( rarity_assets.nameplate )
                :SetHiddenBoundingBox( true )
            self.duplicate_smoke1 = Widget.Image( assets.smoke )
                :SetHiddenBoundingBox( true )
                :SetSize( 900, 900 )
                :SetTintColour( UICOLOURS.TITLE )
                :SetRotation( 40*DEGREES )
                :SetTintAlpha( 0.3 )
                :SetBloom( 0.3 )
            self.duplicate_smoke2 = Widget.Image( assets.smoke )
                :SetHiddenBoundingBox( true )
                :SetSize( 900, 900 )
                :SetTintColour( UICOLOURS.DARK_BODYTEXT )
                :SetRotation( -60*DEGREES )
                :SetTintAlpha( 0.7 )
                :SetBloom( 0.1 )
            self.duplicate_widget:AddChildren
            {
                self.duplicate_mask_image
                    :SetMask( true ),
                self.duplicate_smoke1
                    :SetStencilTest( true )
                    :SetPos( 0, 150 ),
                self.duplicate_smoke2
                    :SetStencilTest( true )
                    :SetPos( 0, 130 )
            }

            self.duplicate_widget:RunUpdater(
                Updater.Loop{
                    Updater.Parallel{
                        Updater.Series{
                            Updater.Ease( function(v) self.duplicate_smoke1:SetPos(v,-v*0.5) end, 150, -150, 3, easing.inOutQuad):SetStartValuePercent(3 * math.random()),
                            Updater.Ease( function(v) self.duplicate_smoke1:SetPos(v,-v*0.5) end, -150, 150, 3, easing.inOutQuad),
                        },
                        Updater.Series{
                            Updater.Ease( function(v) self.duplicate_smoke1:SetScale(v,v) end, 1, 0.8, 5, easing.inOutQuad):SetStartValuePercent(5 * math.random()),
                            Updater.Ease( function(v) self.duplicate_smoke1:SetScale(v,v) end, 0.8, 1, 5, easing.inOutQuad),
                        },
                        Updater.Series{
                            Updater.Ease( function(v) self.duplicate_smoke2:SetPos(v,v*0.5) end, 130, -140, 7, easing.inOutQuad):SetStartValuePercent(7 * math.random()),
                            Updater.Ease( function(v) self.duplicate_smoke2:SetPos(v,v*0.5) end, -140, 130, 7, easing.inOutQuad),
                        },
                        Updater.Series{
                            Updater.Ease( function(v) self.duplicate_smoke2:SetScale(v,v) end, 1, 0.7, 8, easing.inOutQuad):SetStartValuePercent(8 * math.random()),
                            Updater.Ease( function(v) self.duplicate_smoke2:SetScale(v,v) end, 0.7, 1, 8, easing.inOutQuad),
                        }
                    }
                }
            )
            self.duplicate_smoke1:RotateIndefinitely( 0.005 + 0.003*math.random() )
            self.duplicate_smoke2:RotateIndefinitely( 0.003 + 0.003*math.random() * -1 )
        end
    else
        if self.duplicate_widget then
            self.duplicate_widget:Remove()
            self.duplicate_widget = nil
        end
    end
end

function CardWidget:RefreshElectric( )
    if is_instance( self.card, Battle.Card ) then
        if self.card and self.card:IsFlagged(self.card.CARD_FLAGS.GALVANIZED) then
            if not self.electic_widget then
                local anim = Widget.Anim()
                local animname = "anim/card_electric_fx.zip"
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:SetScale( 0.7 )
                anim:SetBloom( 0.25 )
                anim:Play("1", true)
                
                self.electic_widget = self:AddChild( anim )
            end
        else
            if self.electic_widget then
                self.electic_widget:Remove()
                self.electic_widget = nil
            end
        end
    end
end

function CardWidget:RefreshSlime( )
    if is_instance( self.card, Battle.Card ) then
        if self.card and self.card:IsFlagged(self.card.CARD_FLAGS.SLIMY) then
            if not self.slimy_widget then
                local anim = Widget.Anim()
                local animname = "anim/card_sticky_fx.zip"
                anim:AddAnims( GetAnim( animname ) )
                anim:AddOverrideFile( animname )
                anim:SetScale( 0.7 )
                anim:SetBloom( 0.25 )
                anim:Play("1", true, math.random(0,10) * 0.15)
                
                self.slimy_widget = self:AddChild( anim )
            end
        else
            if self.slimy_widget then
                self.slimy_widget:Remove()
                self.slimy_widget = nil
            end
        end
    end
end

function CardWidget:RefreshXP( card, minigame )
    local current_xp, max_xp = self.card:GetXP()
    if current_xp and max_xp and not self.card:IsHatched() then
        self.xp_bar:Show()
        self.xp_bar:Refresh( math.floor(current_xp), max_xp )
        -- Align title and xp count
        self.title:LayoutBounds( "after", "center", self.type_icon ):Offset( 6, 8 )
        self.xp_bar:LayoutBounds( "left", "below", self.title ):Offset( 0, 0 )
    else
        -- Align title
        self.title:LayoutBounds("after", "center", self.type_icon):Offset( 6, 0 )
        self.xp_bar:Hide()
    end
end

function CardWidget:AnimateXP( starting, delta, max, t )
    local delay_per_pip = t / delta
    local pip_adder = Updater.Series()
    for i = 1, delta do
        pip_adder:Add( Updater.Do(
                            function()
                                self.xp_bar:Refresh( starting+i, max )
                                AUDIO:PlayEvent( SoundEvents.card_xp_increase )
                            end
                    ) )
        pip_adder:Add( Updater.Wait(delay_per_pip) )
    end
    -- Animate the cards
    self.xp_anim = self:RunUpdater( pip_adder )
end

function CardWidget:RefreshCharges( card, minigame )
    local charges, max_charges = card:GetCharges()
    if max_charges == nil or charges == nil then
        if self.charges_root then
            self.charges_root:Remove()
            self.charges_root = nil
        end
    else
        if not self.charges_root then
            self.charges_root = self.contents:AddChild( Widget.SegmentedBar() )
        end

        local pip_size = 28
        local pip_spacing = 0
        local pip_container_length = max_charges * pip_size + (max_charges-1) * pip_spacing

        self.charges_root:Show()
        self.charges_root:SetSize( pip_container_length, pip_size )
        self.charges_root:SetOnTexture( assets.charges_full, 0, 0, 0, 0 )
        self.charges_root:SetOffTexture( assets.charges_empty, 0, 0, 0, 0 )
        self.charges_root:SetSpaceBetweenPips( pip_spacing )
        self.charges_root:SetProgress( max_charges, charges )

        local x, y = self.art:GetPos()
        self.charges_root:SetRotation( -90*DEGREES )
        self.charges_root:SetPos( x - 140 - pip_size/2, y )
    end
end

function CardWidget:AnimateToWidget( widget, fn )
    local x, y = self.parent:TransformFromWidget( widget, 0, 0 )
    self:AnimateToPos( x, y, fn)
    return self
end

function CardWidget:AnimateToPos( x, y, fn )
    local duration = 0.4
    self:RegisterCoro()
    self:DisableToolTip( true )
    self:SetTintAlpha( 1 )
    self:ScaleTo( self.sx, 0.1, duration, easing.inQuad )
    self:AlphaTo( 0, duration + 0.1, easing.inQuad )

    local XCURVE = 0.3
    local cx = (self.x * XCURVE) + (x * (1.0 - XCURVE))
    local cy = math.max( y, self.y, math.min( y, self.y ) + 500 )

    if x < self.x then
        self:RotateTo( 3.14, duration, easing.inQuad )
    else
        self:RotateTo( -3.14, duration, easing.inQuad )
    end

    local done = false
    self:CurveTo( x, y, cx, cy, duration, easing.inQuad, function() done = true end )

    while not done do
        coroutine.yield()
    end
    if fn then
        fn()
    end

    local arrangement = self:GetAncestor( Widget.CardArrangement )
    if arrangement then
        arrangement:RemoveCardWidget( self )
        self:DisableToolTip( false )
    else
        self:Remove()
    end
end

function CardWidget:ClearAnim()
    self.anim = nil
end

function CardWidget:AnimateToPlacement( dt, x, y, alpha, scale, rot )

    
    local changed = false
    x = x or self.x
    y = y or self.y
    alpha = alpha or 1.0
    scale = scale or 1.0
    rot = rot or 0

    if self.anim == nil then
        changed = x ~= self.x or y ~= self.y or alpha ~= self.tint_a or scale ~= self.sx or rot ~= self.r
    else
        changed = x ~= self.anim.target_x or y ~= self.anim.target_y or
                    alpha ~= self.anim.target_alpha or scale ~= self.anim.target_scale or
                    rot ~= self.anim.target_rot
    end

    if changed then
        self.anim =
        {
            start_x = self.x,
            start_y = self.y,
            start_alpha = self.tint_a or 1.0,
            start_scale = self.sx,
            start_rot = self.r,

            target_x = x,
            target_y = y,
            target_alpha = alpha,
            target_scale = scale,
            target_rot = rot,

            ease = easing.outCubic,
            duration = 0.4,
            t = 0.0,
        }
    end

    local anim = self.anim
    if anim then
        local t
        anim.t = anim.t + dt
        if anim.t >= anim.duration then
            t = 1.0
            self.anim = nil
        else
            t = anim.t / anim.duration
        end

        local x = anim.ease(t, anim.start_x, anim.target_x - anim.start_x, 1.0)
        local y = anim.ease(t, anim.start_y, anim.target_y - anim.start_y, 1.0)
        local sx = anim.ease(t, anim.start_scale, anim.target_scale - anim.start_scale, 1.0)
        local alpha = anim.ease(t, anim.start_alpha, anim.target_alpha - anim.start_alpha, 1.0)
        local rot = anim.ease(t, anim.start_rot, anim.target_rot - anim.start_rot, 1.0)

        self:SetTintAlpha( alpha )
        self:SetScale( sx, sx )
        self:SetPos( x, y )
        self:SetRotation( rot )
    end
end

function CardWidget:AnimateToDeck( deck_widget )
    self.anim = nil
    self:StartCoroutine( self.AnimateToWidget, self, deck_widget )
end

function CardWidget:PostFeature( feature_id, name, desc )
    -- Check if the feature already exists
    for k, v in ipairs( self.card_features ) do
        if v.id == feature_id then return end
    end
    table.insert( self.card_features, { id = feature_id, name = name, desc = desc } )
end

function CardWidget:PostCard( feature_id, card )
    -- Check if the feature already exists
    for k, v in ipairs( self.card_features ) do
        if v.id == feature_id then return end
    end
    table.insert( self.card_features, { id = feature_id, card = card } )
end

function CardWidget:GetCardFeatures()
    return self.card_features or {}
end

function CardWidget:ReqCondition( ok, condition, ... )
    if not ok then
        self:SetOutline( nil )
    end
end

function CardWidget:IsAnimating()
    return self.anim ~= nil or CardWidget._base.IsAnimating( self )
end

function CardWidget:SetOnClick(fn)
    self.on_click_handler = fn
    return self
end

function CardWidget:SetOnRightClickFn(onclickfn)
    self.on_alt_click_handler = onclickfn
    return self
end

function CardWidget:ShowToolTipOnFocus( show )
    if show == nil then
        show = TheGame and TheGame:ShowCardToolTipsOnFocus()
    end
    Widget.ShowToolTipOnFocus( self, show )
    return self
end


function CardWidget:SetInfoPopupFn(fn)
    self.infopopupfn = fn
end

function CardWidget:ShowInfoOnClick()
    self.on_click_handler = function() self:ShowCardInfoPopup() end
    return self
end

function CardWidget:ShowInfoOnRightClick()
    self.on_alt_click_handler = function() self:ShowCardInfoPopup() end
    return self
end

function CardWidget:ShowCardInfoPopup( mode )
    if self.infopopupfn then
        self.infopopupfn()
    end

    if self.card_locked then return self end
    local card
    if self.card and self.card.deck ~= nil then
        if is_instance( self.card, Battle.Card ) then
            card = Battle.Card( self.card.id )
        else
            card = Negotiation.Card( self.card.id )
        end
    else
        card = self.card
    end
    if card then
        TheGame:FE():PushScreen( Screen.CardDetailsPopup( card, mode or self.card_info_popup_mode ) )
    end
    return self
end

-- Sets the mode, so that, independently of how the popup is triggered, it carries that info
function CardWidget:SetCardInfoPopupMode( mode )
    self.card_info_popup_mode = mode
    return self
end

function CardWidget:SetScrollListMode( list_mode )
    -- Should be false if in hand, and true if in a scrolling list. Used for mouse dragging behaviour
    self.scroll_list_mode = list_mode
    return self
end

function CardWidget:HandleControlDown(control, device)
    if control:Has( Controls.Digital.MENU_ACCEPT ) then
        if TheGame:GetDebug() and TheGame:GetInput():IsModifierControl() then
            DBG( self )
            return true
        elseif control:Has( Controls.Digital.MENU_ACCEPT ) then
            self.down = true
            self:StartUpdating()
            if (device == "mouse" or device == "touch" ) then
                self.down_mouse_last_x, self.down_mouse_last_y = TheGame:GetInput():GetMousePos() -- Save click's initial mouse position
            end
        end
    end
    if control:Has( Controls.Digital.MENU_ALT ) then
        self.alt_down = true
        self:StartUpdating()
        -- Don't return true here, otherwise it won't be possible to cancel card dragging in game with right-click
    end
end

function CardWidget:Click()
    if self.on_click_handler then
        self.on_click_handler()
        self.down = false
        return true
    end

    return false
end

function CardWidget:HandleControlUp( control, device )
    if control:Has( Controls.Digital.MENU_ACCEPT ) then
        if ((device == "mouse" or device == "touch" ) and self.down) or ( device ~= "mouse" and device ~= "touch" ) then
            return self:Click()
        end
    end
    if control:Has( Controls.Digital.MENU_ALT ) and self.alt_down then
        if self.on_alt_click_handler then
            self.alt_down = false
            self:StopUpdating()
            if self.on_alt_click_handler( self ) == false then
                return false
            end
            return true
        end
    end
end

function CardWidget:OnUpdate( dt )
    CardWidget._base.OnUpdate( self, dt )

    if self.scroll_list_mode and self.down then
        -- If we're on a scroll list, remove self.down when dragging more than a threshold
        local width, height = TheGame:FE():GetScreenDims()
        local current_x, current_y = TheGame:GetInput():GetMousePos() -- Get current mouse position
        local dist_y = current_y - (self.down_mouse_last_y or current_y) -- Check how much it has moved since last time
        dist_y = dist_y * RES_Y / height -- Convert the distance dragged to screen coordinates
        if math.abs( dist_y ) > RES_Y*0.05 then -- Let's see if we dragged enough to prevent a click
            self.down = false
            self:StopUpdating()
        end
    end
end

function CardWidget:AnimateToTrash()
    assert_warning( not self.eroding )
    self.eroding = true
    self.anim = nil

    if self.card:IsFlagged( self.card.CARD_FLAGS.CONSUME ) then
        AUDIO:PlayEvent("event:/sfx/battle/cards/neg/card_destroyed")
        self:StartCoroutine( self.ShowErosion, self, "destroy" )
    else
        AUDIO:PlayEvent(SoundEvents.card_expend)
        self:StartCoroutine( self.ShowErosion, self, "expend" )
    end
end

function CardWidget:ShowErosion( erosion_anim_name )
    self.eroding = true
    self:RegisterCoro()
    self:RemoveShine()
    self:RemoveParticles()
    self:SetStencilContext( true )

    if self.duplicate_widget then
        self.duplicate_widget:Remove()
        self.duplicate_widget = nil
    end

    self.outline_bg:Hide()

    local anim = Widget.Anim()
    local animname = "anim/card_expend_mask_fx.zip"
    anim:AddAnims( GetAnim( animname ) )
    anim:AddOverrideFile( animname )
    anim:SetHiddenBoundingBox(true)
    anim:SetScale( 0.65 )
    anim:SetMaskOnly()
    anim:SetSpeed( 0.6 )
    anim:Play( erosion_anim_name )

    local duration = anim:GetAnimDuration( erosion_anim_name )
    local idx = self:IndexOf( self.contents )
    self:AddChild( anim, idx )

    self.contents:SetStencilTest( true )

    if erosion_anim_name == "expend" then
        -- Scale and fade out to help along the explodiness.
        self:ScaleTo( self.sx, self.sx * 1.3, duration, easing.linear )
        self:AlphaTo( 0, duration, easing.inQuad )
        self.shadow:SetTintColour( 0xFFFFFF00 ):SetBloom( 0.15 )
        self.shadow:AlphaTo( 0.8, duration / 2, nil, function() self.shadow:AlphaTo( 0, duration / 2 ) end )
    else
        self.shadow:SetTintColour( 0x000000FF ):SetBloom( 0.15 )
        self.shadow:AlphaTo( 0, duration, easing.inQuad )
    end

    -- Gah, why doesn't Widget.Anim give us an OnDone handler?
    while not anim.removed and not anim:IsDone() do
        coroutine.yield()
    end
    if not self.removed then
        local arrangement = self:GetAncestor( Widget.CardArrangement )
        if arrangement then
            arrangement:RemoveCardWidget( self )
        else
            self:Remove()
        end
    end
end

function CardWidget:ShowShine()
    local anim_pacing = 1.6

    while true do
        self:Delay( 1 + math.random() * 5.0 )

        self.mask:Show()

        self.shine:SetPos( 0, 550 )
            :MoveTo( 0, -550, anim_pacing, easing.outQuad )

        self:Delay( anim_pacing )
        self.mask:Hide()
    end
end

function CardWidget:PlaySelectSound()
    AUDIO:PlayEvent(SoundEvents.battle_card_select)
end

function CardWidget:PlayDeselectSound()
    AUDIO:PlayEvent(SoundEvents.card_dselect)
end

function CardWidget:RegisterCoro()
    local c, is_main = coroutine.running()
    if c and not is_main then
        if self.registered_coros == nil then
            self.registered_coros = {}
        end
        table.insert( self.registered_coros, c )
    end
end

function CardWidget:WaitOnCoros()
    if self.registered_coros and #self.registered_coros > 0 then
        local running = coroutine.running()
        local done = false

        while not done and not self.removed do
            done = true
            for i = #self.registered_coros, 1, -1 do
                local c = self.registered_coros[i]
                if c ~= running then
                    if coroutine.status( c ) == "dead" then
                        table.remove( self.registered_coros, i )
                    else
                        done = false
                    end
                end
            end
            if not done then
                coroutine.yield()
            end
        end
    end
end

function CardWidget:RenderDebugUI( ui, panel )
    local card = self.card

    if self.registered_coros then
        for i, coro in ipairs( self.registered_coros) do
            local status = coroutine.status( coro )
            if ui:Button( string.format( "%d) coroutine (%s)", i, status )) then
                DBG( DebugTraceback( coro ))
            end
        end
    end

    if card then
        local current_xp, max_xp = card:GetXP()
        if current_xp ~= nil and max_xp ~= nil then
            local xp = ui:SliderInt( "XP", current_xp, 0, max_xp)
            if xp ~= current_xp then
                card.userdata.xp = xp
                self:RefreshXP( card )
            end
        end
        
        local charges, max_charges = card:GetCharges()
        if max_charges then
            local new_charges = ui:SliderInt( "CHARGES", charges, 0, max_charges )
            if new_charges ~= charges then
                card.userdata.charges = new_charges
                self:RefreshCharges( card )
            end
        end
    end

    ui:Separator()

    return true
end

function CardWidget:__tostring()
    if self.card then
        return string.format("%s (%s)", self._classname, self.card)
    else
        return self._classname
    end
end

------------------------------------------------------------------------------------------
-- Shows upgrade status

local CardWidgetUpgradeBar = class( "Widget.CardWidgetUpgradeBar", Widget.SegmentedBar )

function CardWidgetUpgradeBar:init( width )
    CardWidgetUpgradeBar._base.init( self, width, 8 )

    self.width = width
    self.current_xp = 0
    self.max_xp = 0

    self:SetOnTexture( assets.upgradebar, 6, 6, 12, 12 )
    self:SetOffTexture( assets.upgradebar, 6, 6, 12, 12 )
    self:SetSpaceBetweenPips( 4 )

    self:Bloom( 0.3 )
end

function CardWidgetUpgradeBar:Refresh( current_xp, max_xp )

    self:SetProgress( max_xp, current_xp )

    if current_xp >= max_xp then
        -- Use bright colors if this card is ready to upgrade
        local r, g, b, a = HexColour( UICOLOURS.QUEST )
        self:SetOnTint( r, g, b, a )
        self:SetOffTint( r, g, b, a )
    else
        local r, g, b, a = HexColour( UICOLOURS.OBJECTIVE )
        self:SetOnTint( r, g, b, a )
        self:SetOffTint( r, g, b, 0.1 )
    end

    self.current_xp = current_xp

end
