require "ui/widget"

local assets =
{
    empty = engine.asset.Texture("UI/chamber_empty.tex"),
    charged = engine.asset.Texture("UI/chamber_round_lumin.tex" ),
    primed = engine.asset.Texture("UI/chamber_round_lumin.tex" ),
}

---------------------------------------------------------------------------------
-- Manages the UI for Rook's cell charges.

local ChargeCell = class( "Widget.ChargeCell", Widget )

function ChargeCell:init()
    Widget.init( self )

    self.w = 140
    self.h = self.w * 0.666666

    -- self.panel = self:AddChild( Widget.Image( engine.asset.Texture("UI/actionscounter_bg.tex"), self.w, self.h ) ):Bloom( 0.05 )

    -- self.count = self:AddChild( Widget.Label( "body", 50 ) ):SetTintColour( UICOLOURS.SUBTITLE ):Bloom( 0.1 )
    -- self.count:LayoutBounds( "center", "center", self.panel ):Offset( 0, 2 )

    self.bg_anim = self:AddChild( Widget.Anim() )
    local animname = "anim/UI_ROOK_charge_fx.zip"
    self.bg_anim:AddAnims( GetAnim( animname ) )
    self.bg_anim:AddOverrideFile( animname )
    self.bg_anim:SetSpeed( 1.0 )
        :SetHiddenBoundingBox( true )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetScale( 0.15 )
    self.bg_anim:Play( "container_fully_empty", true )

    self.pip_root = self:AddChild( Widget() )
    self.pips = {}

    self.surge_counter = self:AddChild( Widget.SurgeCounter() )

    self.charges = 0
    self.max_charges = 0
    self.surge = 0


    self:SetToolTip( { charges = self.charges, max_charges = self.max_charges, surge = self.surge, malfunction = self.damaged and self.charges or 0 } )
end

function ChargeCell:SetSilent( silent )
    self.silent = silent
    return self
end

function ChargeCell:UpdateCellsCoro( charges, max_cells )
    if max_cells then
        while #self.pips > max_cells do
            table.remove( self.pips ):Remove()
        end
        while #self.pips < max_cells do
            local pip = self.pip_root:AddChild( Widget.SingleCharge() )
				:Bloom( 0.1 )
            if #self.pips > 0 then
                pip:LayoutBounds( "after", "bottom" ):Offset( -8, 0 )
            end
            table.insert( self.pips, pip )
        end
    end

    if charges then
        if self.charges ~= charges then
            if charges > self.charges then
                -- Gained charges
                if self.charges == 0 then
                    -- Gained first charges
                    self.bg_anim:Play( "container_become_unempty", false )
                        :Push( "container_gain_charge1", false )
                        :Push( "container_fully_empty", true )
                else
                    -- Gained some charges
                    if charges > 1 and charges < 5 then self.bg_anim:Play( "container_gain_charge" .. charges, false ) end
                    self.bg_anim:Push( "container_fully_empty", true )
                end
            else
                -- Lost charges
                if charges == 0 then
                    -- Lost all charges
                    self.bg_anim:Play( "container_become_empty", false )
                        :Push( "container_fully_empty", true )
                else
                    -- Lost some charges
                    self.bg_anim:Play( "container_lose_charge", false )
                        :Push( "container_fully_empty", true )
                end
            end
            
        end

        self.charges = charges
        for i, pip in ipairs( self.pips ) do
            if i <= charges then
                local was_empty = not pip:IsCharged()
                if charges == #self.pips then
                    -- Became full
                    pip:SetCharged( true )
						:Bloom( 0.06 )
                else
                    pip:SetCharged( true )
						:Bloom( 0.05 )
                end
                if was_empty then
                    -- UIHelpers.PingImage( pip )
                    if not self.silent then
                        AUDIO:PlayParamEvent("event:/sfx/battle/cards/neg/rook_chargeup", "position", pip:GetPanPos(), "charge_increase", charges)
                    end
                end
            else
                pip:SetCharged( false )
					:Bloom( 0 )
            end
        end
    end

    self:Layout()

    self:SetToolTipClass( Widget.TooltipCharges )

    self:Delay( .2 )

    self.coro = nil

    -- check self.queue
    if #self.queue > 0 then
        local charges, max_cells = table.unpack( table.remove( self.queue, 1 ))
        self:UpdateCells( charges, max_cells )
    end
end

function ChargeCell:UpdateCells( charges, max_cells )
    self.queue = self.queue or {}

    self.charges = charges
    self.max_charges = max_cells

    self:SetToolTip( { charges = self.charges, max_charges = self.max_charges, surge = self.surge, malfunction = self.damaged and self.charges or 0 } )

    if self.coro then
        table.insert( self.queue, { charges, max_cells } )
    else
        self.coro = self:StartCoroutine( self.UpdateCellsCoro, self, charges, max_cells )
    end

    return self
end

function ChargeCell:SetDamaged(damaged)
    self.damaged = damaged
    for k,v in ipairs(self.pips) do
        v:SetDamaged(damaged)
    end

    self:SetToolTip( { charges = self.charges, max_charges = self.max_charges, surge = self.surge, malfunction = self.damaged and self.charges or 0 } )
end

function ChargeCell:AnimatePrimed()
    if self.primed_coro then
        self.primed_coro:Stop()
    end
    self.primed_coro = self:StartCoroutine( function()

        self:Layout()

        self.bg_anim:Play( "container_become_full", false )
            :Push( "container_fully_empty", true )

        for bloom = 0, 1.0, 0.05 do
            for i, pip in ipairs( self.pips ) do
                pip:Bloom( bloom )
            end
            self:Delay( 0.025 )
        end
        for bloom = 1.0, 0.1, -0.05 do
            for i, pip in ipairs( self.pips ) do
                pip:Bloom( bloom )
            end
            self:Delay( 0.025 )
        end

        self.primed_coro = nil
    end )
end

function ChargeCell:UpdateSurge( surge_count )
    if self.surge_counter:GetSurge() ~= surge_count then
        self.bg_anim:Play( "container_become_unempty", false )
            :Push( "container_fully_empty", true )

        self.surge = surge_count

        self:SetToolTip( { charges = self.charges, max_charges = self.max_charges, surge = self.surge, malfunction = self.damaged and self.charges or 0 } )
    end
    self.surge_counter:UpdateSurge( surge_count )
    self:Layout()
    return self
end

function ChargeCell:Layout() 
    self.bg_anim:SetPos( 62, 0 )

    if self.surge_counter:HasSurge() then
        self.surge_counter:Show():LayoutBounds( "after", "center", self.pip_root )
    else
        self.surge_counter:Hide():LayoutBounds( "center", "center", self.pip_root )
    end
    -- self:LayoutBounds( "center", "above", self:GetAncestor( Widget.CardsPanel ).actions_widget )
        -- :Offset( self.surge_counter.w/2, 0 )
    return self
end

---------------------------------------------------------------------------------
-- A surge counter

local SingleCharge = class( "Widget.SingleCharge", Widget )

function SingleCharge:init()
    SingleCharge._base.init( self )

    self.w = 50
    self.h = 50

    self.hitbox = self:AddChild( Widget.SolidBox( self.w, self.h, 0xFF00FF00 ) )

    self.icon_anim = self:AddChild( Widget.Anim() )
    local animname = "anim/UI_ROOK_charge_fx.zip"
    self.icon_anim:AddAnims( GetAnim( animname ) )
    self.icon_anim:AddOverrideFile( animname )
    self.icon_anim:SetSpeed( 1.0 )
    self.icon_anim
        :SetHiddenBoundingBox(true)
        :SetScale( 0.15 )

    self.is_charged = false
    self.icon_anim:Play( "cell_empty", true )
end

function SingleCharge:SetCharged( is_charged )
    if self.is_charged ~= is_charged then
        if self.is_charged then
            -- We lost charge
            self.icon_anim:Play( "cell_become_empty", false )
                :Push( "cell_empty", true )
        else
            -- We gained charge
            self.icon_anim:Play( self.is_damaged and "cell_become_damaged" or "cell_become_charged", false )
                :Push( self.is_damaged and "cell_damaged" or "cell_charged", true )
        end
    end
    self.is_charged = is_charged
    return self
end

function SingleCharge:SetDamaged(is_damaged)
    if self.is_damaged ~= is_damaged then
        
        if self.is_charged then
            self.icon_anim:Play( is_damaged and "cell_become_damaged" or "cell_become_charged", false )
                :Push( is_damaged and "cell_damaged" or "cell_charged", true )
        end

        self.is_damaged = is_damaged
    end
end


function SingleCharge:IsCharged()
    return self.is_charged
end

function SingleCharge:IsDamaged()
    return self.is_damaged
end

---------------------------------------------------------------------------------
-- A surge counter

local SurgeCounter = class( "Widget.SurgeCounter", Widget )

function SurgeCounter:init()
    SurgeCounter._base.init( self )

    self.w = 60
    self.h = 60

    self.hitbox = self:AddChild( Widget.SolidBox( self.w, self.h, 0xFF00FF00 ) )

    self.cloud_fx = self:AddChild( Widget.Anim() )
    local animname = "anim/UI_ROOK_charge_fx.zip"
    self.cloud_fx:AddAnims( GetAnim( animname ) )
    self.cloud_fx:AddOverrideFile( animname )
    self.cloud_fx:SetSpeed( 1.0 )
        :SetHiddenBoundingBox( true )
        :SetScale( 0.15 )
        :Bloom( 0.04 )
        :Hide()

    self.bg_anim = self:AddChild( Widget.Anim() )
    local animname = "anim/UI_ROOK_charge_fx.zip"
    self.bg_anim:AddAnims( GetAnim( animname ) )
    self.bg_anim:AddOverrideFile( animname )
    self.bg_anim:SetSpeed( 1.0 )
        :SetHiddenBoundingBox( true )
        :SetScale( 0.15 )
        :Bloom( 0.02 )

    self.label = self:AddChild( Widget.Label( "body", 32, "" ) )
        :SetTintColour( UICOLOURS.DARK_BODYTEXT )
        :Bloom( 0.1 )

    self.arcs_anim = self:AddChild( Widget.Anim() )
    local animname = "anim/UI_ROOK_charge_fx.zip"
    self.arcs_anim:AddAnims( GetAnim( animname ) )
    self.arcs_anim:AddOverrideFile( animname )
    self.arcs_anim:SetSpeed( 0.5 )
        :SetHiddenBoundingBox( true )
        :SetScale( 0.15 )
        :Bloom( 0.4 )
        :Hide()
    self.arcs_anim:Play( "surge_loop_arcs", true )

    self.surge_less_slash = self:AddChild( Widget.Anim() )
    local animname = "anim/UI_ROOK_charge_fx.zip"
    self.surge_less_slash:AddAnims( GetAnim( animname ) )
    self.surge_less_slash:AddOverrideFile( animname )
    self.surge_less_slash:SetSpeed( 1.0 )
        :SetHiddenBoundingBox( true )
        :SetScale( 0.15 )
        :Bloom( 0.4 )
        :Hide()

    self.surge_count = 0
end

function SurgeCounter:SetMuted( muted )
    self.muted = muted
    return self
end

function SurgeCounter:UpdateSurge( count )
    if count ~= self.surge_count then
        if count > self.surge_count then
            -- Gained surge
            if not self.parent.silent and not self.muted then
                AUDIO:PlayParamEvent("event:/sfx/battle/status/system/Status_Buff_Overcharge_fill", "position", self:GetPanPos())
            end
            if self.surge_count == 0 then
                -- Gained first surge
                self.label:SetText( loc.format( "{1}", count ) )
                    :Show()
                self.bg_anim:Play( "surge_in", false )
                    :Push( "surge_loop", true )
                self.arcs_anim:Show()
            else
                -- Gained some surge
                self.label:SetText( loc.format( "{1}", count ) )
                    :Show()
            end
            self.cloud_fx:Show()
                :Play( "surge_more", false )
            self:ScaleTo( 1.4, 1, 0.1, easing.inQuad )
        else
            -- Lost surge
            if not self.parent.silent and not self.muted then
                AUDIO:PlayParamEvent("event:/sfx/battle/status/system/Status_Buff_Overcharge_use", "position", self:GetPanPos())
            end
            if count == 0 then
                -- Lost all surge
                self.label:SetText( loc.format( "{1}", count ) )
                    :Hide()
                self.bg_anim:Play( "surge_out", false )
                self.arcs_anim:Play( "surge_loop_arcs", true )
                self.arcs_anim:Hide()
            else
                -- Lost some surge
                self.label:SetText( loc.format( "{1}", count ) )
                    :Show()
                self:ScaleTo( 0.7, 1, 0.1, easing.outQuad )
            end
            self.cloud_fx:Show()
                :Play( "surge_less", false )
            self.surge_less_slash:Show()
                :Play( "surge_less_slash", false )
        end
        self.surge_count = count
        self.label:LayoutBounds( "center", "center", self.hitbox )
            :Offset( 0, -3 )
    end
    return self
end

function SurgeCounter:HasSurge()
    return self.surge_count>0
end

function SurgeCounter:GetSurge()
    return self.surge_count
end

---------------------------------------------------------------------------------
-- A tooltip for the charges widget

local panel_w = 412
local panel_h = 128
local padding = 8

local TooltipCharges = class("Widget.TooltipCharges", Widget)

function TooltipCharges:init()
    TooltipCharges._base.init(self)
    self.bg = self:AddChild(Widget.Panel(global_images.tooltipbg))
    self.bg:SetNineSliceBorderScale( 0.5 )

    self.contents = self:AddChild(Widget())

    self.header_widget = self.contents:AddChild( Widget() )
    self.title = self.header_widget:AddChild( Widget.Label( "title", 26 ) )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom(0.1)
    self.text = self.header_widget:AddChild( Widget.Label( "tooltip", 22 ) )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    self.charges_widget = self.contents:AddChild( Widget() )
    self.charges_label = self.charges_widget:AddChild( Widget.Label( "body", 26 ))
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.OBJECTIVE )
        :Bloom(0.15)
    self.charges_text = self.charges_widget:AddChild( Widget.Label( "body", 22 ))
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    self.surge_widget = self.contents:AddChild( Widget() ):Hide()
    self.surge_label = self.surge_widget:AddChild( Widget.Label( "body", 26 ))
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.SURGE )
        :Bloom(0.15)
    self.surge_text = self.surge_widget:AddChild( Widget.Label( "body", 22 ))
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

    self.malfunction_widget = self.contents:AddChild( Widget() ):Hide()
    self.malfunction_label = self.malfunction_widget:AddChild( Widget.Label( "body", 26 ))
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.PENALTY )
        :Bloom(0.15)
    self.malfunction_text = self.malfunction_widget:AddChild( Widget.Label( "body", 22 ))
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

    self:Hide()
end

function TooltipCharges:DoLayout( data )
    self.charges = data.charges or 0
    self.max = data.max_charges or 0
    self.surge = data.surge or 0
    self.malfunction = data.malfunction or 0

    self.title:SetText( LOC"ROOK.CELLS_TT_TITLE" )
    self.text:SetText( LOC"ROOK.CELLS_TT_TEXT" )
    self.text:LayoutBounds( "left", "below", self.title )
        :Offset( 0, 0 )

    self.charges_label:SetText( loc.format( LOC"ROOK.CELLS_TT_CHARGES", self.charges, self.max ) )
    if self.charges >= self.max then
        self.charges_text:SetText( LOC"ROOK.CELLS_TT_CHARGES_PRIMED" )
    else
        self.charges_text:SetText( LOC"ROOK.CELLS_TT_CHARGES_TEXT" )
    end
    self.charges_text:LayoutBounds( "left", "below", self.charges_label ):Offset( 0, 0 )



    if self.malfunction > 0 then
        self.malfunction_label:SetText( loc.format( LOC"ROOK.CELLS_TT_MALFUNCTION", self.malfunction ) )
        local DMG_PER = 2
        self.malfunction_text:SetText( loc.format( LOC"ROOK.CELLS_TT_MALFUNCTION_TEXT", DMG_PER, self.malfunction*DMG_PER ))
        self.malfunction_text:LayoutBounds( "left", "below", self.malfunction_label ):Offset( 0, 0 )
        self.malfunction_widget:Show()
    else
        self.malfunction_widget:Hide()
    end


    -- Check if we have surge
    if self.surge > 0 then
        self.surge_label:SetText( loc.format( LOC"ROOK.CELLS_TT_SURGE", self.surge ) )
        self.surge_text:SetText( LOC"ROOK.CELLS_TT_SURGE_TEXT" )
        self.surge_text:LayoutBounds( "left", "below", self.surge_label ):Offset( 0, 0 )
        self.surge_widget:Show()
    else
        self.surge_widget:Hide()
    end

    self.contents:SetRegistration("center", "center")
    self.contents:StackChildren( 11 )
    self.bg:SizeToWidgets( padding, self.contents )
    
end

---------------------------------------------------------------------------------
-- Displays a widget on the Fighter Info Popup explaining Rook's charges

local ChargeCellInfoWidget = class( "Widget.ChargeCellInfoWidget", Widget )

function ChargeCellInfoWidget:init( fighter, condition, width, silent )
    ChargeCellInfoWidget._base.init( self )

    self.fighter = fighter
    self.condition = condition
    self.width = width or 300
    local content_w = self.width - SPACING.M1*2

    self.background = self:AddChild( Widget.Panel( engine.asset.Texture("UI/left_border_frame_faint.tex" ) ) )
        :SetTintColour( UICOLOURS.OBJECTIVE )
        :SetBloom( 0.15 )

    self.contents = self:AddChild(Widget())
    self.header_widget = self.contents:AddChild( Widget() )
    self.title = self.header_widget:AddChild( Widget.Label( "title", 26 ) )
        :SetGlyphColour( UICOLOURS.OBJECTIVE )
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom(0.1)
    self.text = self.header_widget:AddChild( Widget.Label( "tooltip", 22 ) )
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    self.charge_cells = self.contents:AddChild( Widget.ChargeCell() )
        :DisableToolTip( true )
        :SetSilent( silent == true )
        :UpdateCells( self.condition:GetCharges() )
    self.charges_widget = self.contents:AddChild( Widget() )
    self.charges_label = self.charges_widget:AddChild( Widget.Label( "body", 26 ))
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.OBJECTIVE )
        :Bloom(0.15)
    self.charges_text = self.charges_widget:AddChild( Widget.Label( "body", 22 ))
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    self.surge_widget = self.contents:AddChild( Widget() ):Hide()
    self.surge_label = self.surge_widget:AddChild( Widget.Label( "body", 26 ))
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.SURGE )
        :Bloom(0.15)
    self.surge_text = self.surge_widget:AddChild( Widget.Label( "body", 22 ))
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

    self.malfunction_widget = self.contents:AddChild( Widget() ):Hide()
    self.malfunction_label = self.malfunction_widget:AddChild( Widget.Label( "body", 26 ))
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.PENALTY )
        :Bloom(0.15)
    self.malfunction_text = self.malfunction_widget:AddChild( Widget.Label( "body", 22 ))
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

end

function ChargeCellInfoWidget:Layout()

    self.charges = self.condition.charges or 0
    self.max = self.condition.max_charges or 0
    self.surge = self.fighter:GetConditionStacks("SURGE") or 0
    self.malfunction = self.condition.malfunction or 0

    self.title:SetText( LOC"ROOK.CELLS_TT_TITLE" )
    self.text:SetText( LOC"ROOK.CELLS_TT_TEXT" )
    self.text:LayoutBounds( "left", "below", self.title )
        :Offset( 0, 0 )

    self.charge_cells:UpdateCells( self.charges, self.max )
        :UpdateSurge( self.surge )

    self.charges_label:SetText( loc.format( LOC"ROOK.CELLS_TT_CHARGES", self.charges, self.max ) )
    if self.charges >= self.max then
        self.charges_text:SetText( LOC"ROOK.CELLS_TT_CHARGES_PRIMED" )
    else
        self.charges_text:SetText( LOC"ROOK.CELLS_TT_CHARGES_TEXT" )
    end
    self.charges_text:LayoutBounds( "left", "below", self.charges_label ):Offset( 0, 0 )

    if self.malfunction > 0 then
        self.malfunction_label:SetText( loc.format( LOC"ROOK.CELLS_TT_MALFUNCTION", self.malfunction ) )
        local DMG_PER = 2
        self.malfunction_text:SetText( loc.format( LOC"ROOK.CELLS_TT_MALFUNCTION_TEXT", DMG_PER, self.malfunction*DMG_PER ))
        self.malfunction_text:LayoutBounds( "left", "below", self.malfunction_label ):Offset( 0, 0 )
        self.malfunction_widget:Show()
    else
        self.malfunction_widget:Hide()
    end

    -- Check if we have surge
    if self.surge > 0 then
        self.surge_label:SetText( loc.format( LOC"ROOK.CELLS_TT_SURGE", self.surge ) )
        self.surge_text:SetText( LOC"ROOK.CELLS_TT_SURGE_TEXT" )
        self.surge_text:LayoutBounds( "left", "below", self.surge_label ):Offset( 0, 0 )
        self.surge_widget:Show()
    else
        self.surge_widget:Hide()
    end

    self.contents:SetRegistration("center", "center")
    self.contents:StackChildren( 11 )

    local w, h = self.contents:GetSize()
    self.background:SetSize( self.width, h + SPACING.M1 )
    self.contents:LayoutBounds( "center", "center", self.background )
    return self
end