require "ui/widget"
require "ui/widgets/label"
require "ui/widgets/image"

local CheckBox = class("Widget.CheckBox", Widget)

local assets = {
	on =  engine.asset.Texture("UI/checkbox_on.tex"),
	off =  engine.asset.Texture("UI/checkbox_off.tex"),
}


function CheckBox:init( text_size )
	CheckBox._base.init(self)
	self.hover_check = true
	self.state = false
	self.text_size = text_size or 20
	self.blocks_mouse = true
	self.w, self.h = 24, 24
	self.img = Widget.Image(assets.off)
		:SetSize( 24, 24 )
		:Bloom( 0.1 )
	self:AddChildren{
		self.img
	}

end

function CheckBox:SetSize(w, h)
	self.w, self.h = w, h
	self.img:SetSize(w,h)
	return self
end
function CheckBox:SetFn(fn)
	self.fn = fn
	return self
end

function CheckBox:SetUserData(data)
	self.data = data
	return self
end

function CheckBox:GetValue()
	return self.state
end

function CheckBox:SetValue(state, silent)
	self.state = state
	self:UpdateImage()
		
	if not silent and self.fn then
		self.fn(state)
	end
	
	return self
end

function CheckBox:SetText(text)
	if not self.text then
		self.text = self:AddChild( Widget.Label("body", self.text_size, text) )
        	:SetWordWrap(true)
        	:SetAutoSize(200)
        	:Bloom( 0.05 )
        	:LeftAlign()
	end
	self.text:SetText( text )
		:LayoutBounds( "after", "center", self.img )
		:Offset( 10, 0 )
	return self
end


function CheckBox:UpdateImage()
	self.img:SetTexture(self.state and assets.on or assets.off)
	if self.enabled then
		if self.hover then
			self:SetTintColour(1.5,1.5,1.5,1)
		elseif self.focus then
			self:SetTintColour(1.5,1.5,1.5,1)
		else
			self:SetTintColour(1,1,1,1)
		end
	else
		self:SetTintColour(.6, .6, .6, 1)
	end
end

function CheckBox:HandleControlDown(control)
	if control:Has( Controls.Digital.MENU_ACCEPT ) then
		
		if self.state and self.radio_group then return true end

		self:SetValue(not self.state)
		return true
	end
end

function CheckBox:OnGainHover()
    CheckBox._base.OnGainHover(self)
	self:UpdateImage()
end

function CheckBox:OnLoseHover()
    CheckBox._base.OnLoseHover(self)
	self:UpdateImage()
end

function CheckBox:OnGainFocus()
    CheckBox._base.OnGainFocus(self)
	self:UpdateImage()
end


function CheckBox:OnLoseFocus()
    CheckBox._base.OnLoseFocus(self)
	self:UpdateImage()
end

function CheckBox:Enable()
    CheckBox._base.Enable(self)
    self:UpdateImage()
    return self
end

function CheckBox:Disable()
    CheckBox._base.Disable(self)
    self:UpdateImage()
    return self
end
