local ConditionImage = class("Widget.ConditionImage", Widget)

local assets =
{
    bg = engine.asset.Texture("battle/conditions/condition_bg.tex"),
    bg_def = engine.asset.Texture("battle/conditions/condition_bg_def.tex"),
}

function ConditionImage:init( size )
    ConditionImage._base.init( self )
    size = size or 64
    self.img_size = size
    self.bg = self:AddChild( Widget.Image( assets.bg_def ) ):SetSize( self.img_size, self.img_size )
    self.img = self:AddChild( Widget.Image() ):SetSize( self.img_size-6, self.img_size-6 ):Bloom( 0.03 )
        :SetHiddenBoundingBox( true )
    self.stacks_label = self:AddChild( Widget.Label( "title", 22 ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :EnableOutline( true )
        :SetAutoSize( self.img_size )
    self.range_label = self:AddChild( Widget.Label( "title", 20 ) )
        :SetGlyphColour( UICOLOURS.WHITE )
        :EnableOutline( true )
        :SetAutoSize( self.img_size )
    self:Bloom(0.05)
end

function ConditionImage:GetCondition()
    return self.condition_id
end

function ConditionImage:DeltaStacks( stacks )
    self.stacks = self.stacks + stacks

    if self.stacks == 1 then
        self.stacks_label:Hide()
    else
        self.stacks_label:SetText( tostring(self.stacks) ):Show()
        self.stacks_label:LayoutBounds( "right", "bottom", self.bg )
            :Offset( -self.img_size*0.15, self.img_size*0.05 )
    end

    return self.stacks > 0
end

function ConditionImage:SetCondition( condition_id, stacks, condition )
    assert( condition_id )

    self.condition_id = condition_id
    self.condition = nil
    self.stacks = 0
    self.range = nil

    local battle_defs = require "battle/battle_defs"
    local def = battle_defs.CONDITIONS[ condition_id ]
    if def.ctype == CTYPE.INNATE then
        self.bg:SetTexture( assets.bg_def )
            :SetTintColour( UICOLOURS.WHITE )
        self.img:SetSize( self.img_size+2, self.img_size+2 )
    else
        self.bg:SetTexture( assets.bg )
        if def.ctype == CTYPE.BUFF then
            self.bg:SetTintColour( UICOLOURS.BONUS )
        elseif def.ctype == CTYPE.DEBUFF then
            self.bg:SetTintColour( UICOLOURS.PENALTY )
        else
            self.bg:SetTintColour( UICOLOURS.SUBTITLE )
        end
    end

    self.img:SetTexture( condition and condition.icon or def.icon )

    self:DeltaStacks( stacks or 1 )

    self:RefreshHighlight()

    return self
end

function ConditionImage:RefreshConditionImage(condition)
    self.condition = condition

    -- Update icon
    self.img:SetTexture(engine.asset.Texture( condition.icon_path, true ))

    -- Check if there are stacks of this condition
    self.stacks = condition:GetStacks()
    if self.stacks > 1 then
        self.stacks_label:SetText( tostring(self.stacks) )
            :Show()
            :LayoutBounds( "right", "bottom", self.bg )
            :Offset( -self.img_size*0.15, self.img_size*0.05 )
    else
        self.stacks_label:Hide()
    end

    -- Check if this condition has an effect range
    self.range = condition.GetRange and condition:GetRange()
    if self.range then
        self.range_label:SetText(self.range)
            :Show()
            :LayoutBounds( "center", "bottom", self.bg )
            :Offset( 0, 3 )
    else
        self.range_label:Hide()
    end

    self:RefreshHighlight()

    return self.stacks > 0 
end

function ConditionImage:HasImage()
    return self.img:GetTexture() ~= nil
end

function ConditionImage:RefreshHighlight()
    if self.condition and self.condition.highlighted then
        if not self.highlight then
            -- Is highlighted, but doesn't have an anim yet. Add it
            local animname = "anim/UI_combat_square_highlight_fx.zip"
            local original_size = 300 -- Anim size
            local scale = self.img_size / original_size -- Scale to make the anim match the current icon size
            scale = scale * 1.26 -- Magic number to make the anim align to the icon perfectly
            local anim = Widget.Anim()
                :SetHiddenBoundingBox(true)
                :SetScale( scale )
                :SetBloom( 0.2 )
            anim:AddAnims( GetAnim( animname ) )
            anim:AddOverrideFile( animname )
            self.highlight = self:AddChild( anim )
                :Play( "loop", true )
                :SetTintColour( self.bg.tint_r, self.bg.tint_g, self.bg.tint_b, self.bg.tint_a )
        end
    elseif self.highlight then
        -- Not highlighted anymore, but the anim is still here. Delete it
        self.highlight:Remove()
        self.highlight = nil
    end
    return self
end