
--------------------------------------------------------------------------------

local DailyProgress = class( "Widget.DailyProgress", Widget )

function DailyProgress:init()
    DailyProgress._base.init( self )

    self.icon_size = 34
    self.icon_spacing = 3

    self.icons_container = self:AddChild( Widget() )
    self.events = {}
    self.lines = {}
    
    TheGame:GetEvents():ListenForEvents( self, "quests_changed")
end


function DailyProgress:OnGlobalEvent( eventname, ... )
    if eventname == "quests_changed" then
        self:Refresh()
    end
end

function DailyProgress:OnAdded()
    self:Refresh()
end

function DailyProgress:Refresh(progress, no_tt_layout)
    progress = progress or (TheGame:GetGameState() and TheGame:GetGameState():GetMainQuest() and TheGame:GetGameState():GetMainQuest():DefFn("get_progression"))
    
    local plan = progress and progress[1]
    if plan then
        local num_icons = #plan
        if num_icons < #self.events then
            for k = 1, #self.events - num_icons do
                table.remove(self.events):Remove()
                if #self.lines > 0 then
                    table.remove(self.lines):Remove()
                end
            end
        elseif num_icons > #self.events then
            for k = 1, num_icons - #self.events do
                local icon = self.icons_container:AddChild( Widget.DailyProgressIcon(self.icon_size, no_tt_layout) )
                    :LayoutBounds( "after", "center" )
                    :Offset( k==1 and self.icon_spacing+2 or self.icon_spacing, 0 )
                table.insert(self.events, icon)
                
                if #self.events > 1 then
                    local x1 = self.events[#self.events]:GetPos()
                    local x2 = self.events[#self.events-1]:GetPos()
                    local line = self:AddChild( Widget.SolidBox( x2-x1, 2, UICOLOURS.SUBTITLE ) )
                        :SetName( "DAY" .. k .. "CONNECTOR_LINE" )
                        :LayoutBounds( "left", "center", self.events[#self.events] )
                        :Offset( self.icon_size/2 )
                        :SendBehind( self.icons_container )
                    table.insert(self.lines, line)
                end

            end
        end

        for j, event in ipairs(plan) do
            local icon = self.events[j]

            if event.icon then
                icon:SetIcon( event.icon )
            end
            if event.boss then
                icon:SetTypeBoss( event.boss )
            end
            if event.tt then
                icon:SetToolTipText( event.tt )
            end

            icon:SetCurrent( event.right_now )
            if event.success then
                icon:SetResultSuccess()
            elseif event.failed then
                icon:SetResultFail()
            end
        end
    end
    

    --[[-- Add a line connecting today's icons
    if #day_icons > 0 then
        local x1 = day_icons[1]:GetPos()
        local x2 = day_icons[#day_icons]:GetPos()
        local line = self:AddChild( Widget.SolidBox( x2-x1, 2, UICOLOURS.SUBTITLE ) )
            :SetName( "DAY" .. k .. "CONNECTOR_LINE" )
            :LayoutBounds( "left", "center", day_icons[1] )
            :Offset( self.icon_size/2 )
            :SendBehind( self.icons_container )
    end--]]

    self:Show()
end

------------------------------------------------------------------------------------------
-- Shows an individual icon in the brawl progress track

local DailyProgressIcon = class( "Widget.DailyProgressIcon", Widget )

function DailyProgressIcon:init( size, no_tt_layout )
    DailyProgressIcon._base.init( self )

    self.size = size or 40
    self.hitbox = self:AddChild( Widget.SolidBox( self.size, self.size, 0xff00ff00 ) )
    self.icon = self:AddChild( Widget.Image() )
        :SetSize( self.size, self.size )
        :Bloom( 0.02 )
    self.result = self:AddChild( Widget.Image() )
        :SetSize( self.size, self.size )
        :Bloom( 0.05 )

    if not no_tt_layout then
        self:SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "center", "below", w ):Offset( 0, -8 ) end )
    end
end

function DailyProgressIcon:SetTypeBoss( boss_id )

    local skins = Content.GetAllCharacterSkins( boss_id )
    local skin = skins and skins[1]
    local def = Content.GetCharacterDef( boss_id )
    assert( def, boss_id ) 

    if def.icon then
        self.icon:SetTexture( def.icon )
    else
        self.icon:SetTexture( engine.asset.Texture("UI/daily_boss.tex") )
    end
    self.tt_text = loc.format( LOC"UI.PROGRESS.DAILY_BOSS", skin and skin:GetLocalizedName() or def:GetLocalizedName() )
    self:SetToolTip( self.tt_text )
    return self
end

function DailyProgressIcon:SetIcon( icon )
    self.icon:SetTexture( icon )
    return self
end

function DailyProgressIcon:SetToolTipText( text )
    self.tt_text = text
    self:SetToolTip( self.tt_text )
    return self
end

function DailyProgressIcon:SetResultSuccess()
    self.result:SetTexture( engine.asset.Texture("UI/daily_success.tex") )
    return self
end
function DailyProgressIcon:SetResultFail()
    self.result:SetTexture( engine.asset.Texture("UI/daily_fail.tex") )
    return self
end

function DailyProgressIcon:SetCurrent( is_current_day )
    if is_current_day and not self.current then
        self.current = self:AddChild( Widget() )
        local aura_size = self.size*1.5
        self.aura_layer_1 = self.current:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer1_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5):RotateIndefinitely( -0.015 )
        self.aura_layer_2 = self.current:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer2_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5):RotateIndefinitely( 0.05 )
        self.aura_layer_3 = self.current:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer3_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5):RotateIndefinitely( -0.04 )
        self:SetToolTip( loc.format( LOC"UI.PROGRESS.CURRENT_DAY", self.tt_text ) )
    elseif not is_current_day and self.current then
        self.current:Remove()
        self.current = nil
        self:SetToolTip( self.tt_text )
    end
    return self
end