require "ui/widget"

local WIDTH = 40
local TARGET_RADIUS = 0

local assets =
{
    arrow = engine.asset.Texture("rgba/negotiationarrow.tex")
}
local DamageArrow = class("Widget.DamageArrow", Widget)


function DamageArrow:init()
    DamageArrow._base.init(self)  

    self.model = engine.model.FightArrowModel()
    self.model:SetTextures(assets.arrow, engine.asset.Texture("rgba/character_hover.tex"))
    self.model:SetStretchPoints(123.0/700.0, 550.0/700.0)
    self.model:SetLayerSpeeds(0.02, 0.05, 0.1)
    self.model:SetLayerScale(0.3)
    self.model:SetLayerAlphas(0.4,0.5,0.2)
    self.model:SetFade( 0.8 )

    self.node = engine.rendering.SceneGraphNode(self.model) 

    self:SetColour( 0xDD000033 )
    self:SetBloom(0.3)    
end

function DamageArrow:OnAdded()
    self:StartUpdating()
end

function DamageArrow:SetBloom(b)
    self.model:SetBloom(b)
    return self
end

function DamageArrow:SetColour(colour)
    local r,g,b,a = HexColour(colour)
    self.model:SetColour(r, g, b, a)
    return self
end

function DamageArrow:Attach( source, target, target_offset )
    assert( is_instance( source, Widget ))
    self.source = source

    assert( is_instance( target, Widget ))
    self.target = target
    self.target_offset = target_offset
end

function DamageArrow:OnUpdate( dt )
    if self.source and self.source.removed then
        self.source = nil
    end
    if self.target and self.target.removed then
        self.target = nil
    end

    if self.source and (self.target or self.target_mouse) then
        local x1, y1 = self.parent:TransformFromWidget( self.source, 0, 0 )
        local x2, y2 = self.parent:TransformFromWidget( self.target, 0, 0 )

        if self.target_offset then
            local dx, dy = normalizeVec2( x2 - x1, y2 - y1 )
            x2 = x2 + dx * self.target_offset, y2 + dy * self.target_offset
        end

        self:SetPos( x1, y1 )

        local base_scale = self:GetFE():GetBaseWidgetScale()
        local len = math.max( 0, distance( x1, y1, x2, y2 )) - (self.target.TARGET_RADIUS or 0) * base_scale
        local width = WIDTH * base_scale
        self.model:SetWidthAndLength( width, len, width*1.5 )

        local angle = math.atan2( x2 - x1, y2 - y1 )
        self:SetRotation( 2 * math.pi - angle )
    end
end