require "ui/widget"

local DebugRenderOverlay = class("Widget.DebugRenderOverlay", Widget)

function DebugRenderOverlay:init()
	DebugRenderOverlay._base.init(self)
	self.nodebug = true
    self.fontsize = 20


    self.bgtop = self:AddChild(Widget.SolidBox( RES_X, self.fontsize, 0,0,0, .75))
    self.bgtop:SetBaseScaling( false )
	self.texttop = self.bgtop:AddChild(Widget.Label( "debugfont", self.fontsize, ""))
	self.texttop:SetRegionSize(RES_X, self.fontsize)
	self.texttop:LeftAlign()

	
    self.bgbottom = self:AddChild(Widget.SolidBox( RES_X, self.fontsize, 0,0,0, .75))
    self.bgbottom:SetBaseScaling( false )
    self.textbottom = self.bgbottom:AddChild(Widget.Label( "debugfont", self.fontsize, ""))
    self.textbottom:SetRegionSize(RES_X, self.fontsize)
    self.textbottom:LeftAlign()

    self.versionstr = self.bgbottom:AddChild(Widget.Label( "debugfont", self.fontsize, BUILD_ID_STRING))
    self.versionstr:SetRegionSize(RES_X, self.fontsize)
    self.versionstr:RightAlign()

    self.str = {}
    self.bottom_str = {}

    self.right_debug_text = self:AddChild(Widget.Label( "debugfont", self.fontsize, "Behaviour"))
    self.right_debug_text:LeftAlign()
    self.right_debug_text:TopAlign()
    self.right_debug_text:SetWordWrap(true)

    self.left_debug_text = self:AddChild(Widget.Label( "debugfont", self.fontsize, "Entity"))
    self.left_debug_text:LeftAlign()
    self.left_debug_text:TopAlign()
    self.left_debug_text:SetWordWrap(true)

    self:StartUpdating()
end

function DebugRenderOverlay:OnAdded()
    self:OnScreenResize(self:GetFE():GetScreenDims())
end

function DebugRenderOverlay:OnUpdate(dt)

    DebugRenderOverlay._base.OnUpdate(self, dt)

    self.delay = self.delay or 0
    self.delay = self.delay - dt
    if self.delay > 0 then
        return
    end
    self.delay = .5

	table.clear(self.str)
    table.clear(self.bottom_str)

	local mem = collectgarbage("count") * 1024
	local memunit = "B"
	if mem > 1024*1024 then
		mem = mem / (1024*1024)
		memunit = "M"
	elseif mem > 1024 then
		mem = mem / 1024
		memunit = "K"
	end

	table.insert(self.str, string.format("mem: %2.1f%s",mem,memunit))
    table.insert(self.str, string.format("textures: %2.1fMB",engine.inst:GetTextureMemUsed() / (1024*1024)))

    

    table.insert(self.str, string.format("Frame: %2.2f, Update: %2.2f (lua %2.2f), Render: %2.2f", TheGame.smoothtimes.frame*1000, TheGame.smoothtimes.update*1000, TheGame.smoothtimes.update_lua*1000, TheGame.smoothtimes.render*1000))
    
	self.texttop:SetText( table.concat(self.str, "  ") )
    self.textbottom:SetText(table.concat(self.bottom_str, "  "))
	table.clear(self.str)

    self.left_debug_text:SetText("")
    self.right_debug_text:SetText("")


end

function DebugRenderOverlay:OnScreenResize(w,h)
    --print ("RESIZE")
    DebugRenderOverlay._base.OnScreenResize(self, w, h)
    self.bgtop:SetPos(w/2, h-self.fontsize*.5)
    self.bgtop:SetSize(w, self.fontsize)
    self.bgbottom:SetPos(w/2, self.fontsize*.5)
    self.bgbottom:SetSize(w, self.fontsize)

    self.texttop:SetRegionSize(w, self.fontsize)
    self.textbottom:SetRegionSize(w, self.fontsize)
    self.versionstr:SetRegionSize(w, self.fontsize)

    self.right_debug_text:SetPos(w/2 + w/4, h/2)
    self.right_debug_text:SetRegionSize(w/2-self.fontsize, 4*h/5)
    self.left_debug_text:SetPos(w/2 -w/4, h/2)
    self.left_debug_text:SetRegionSize(w/2-self.fontsize, 4*h/5)
end
