require "ui/widget"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/solidbox"
require "ui/widgets/menubutton"



local DragButton = class("Widget.DragButton", Widget)


function DragButton:init(w, h, c, img)
    DragButton._base.init(self)
    self.hover_check = true
    self.w, self.h = w or 0, h or 0
    if img and type(img) == "table" then
        self.bg = Widget.Panel()
        self.bg:SetNineSlice(img)
    else
        self.bg = Widget.SolidBox(w, h, c)
    end
    self:AddChild(self.bg)

    self.min = 0
    self.max = 0
end

function DragButton:SetDragFn(fn)
    self.dragfn = fn
    return self
end

function DragButton:Enable()
    DragButton._base.Enable(self)
    self:UpdateImage()
    return self
end

function DragButton:Disable()
    DragButton._base.Disable(self)
    self:UpdateImage()
    return self
end

function DragButton:SetColour(c)
    self.bg:SetColour(c)
    return self
end
function DragButton:SetHorizontal()
    self.horizontal = true
    return self
end

function DragButton:SetExtents(min, max)
    self.min, self.max = min, max
    return self
end

function DragButton:SetSize( w, h )
    self.w, self.h = w, h
    self.bg:SetSize(w, h)
    return self
end

function DragButton:UpdateImage()

    self:Bloom( 0.1 )
    if not self.enabled then
        self:SetTintAlpha( .7 )
        self:Bloom( 0 )
    elseif self.down then
        self:SetTintAlpha( .5 )
    elseif self.hover and self.enabled then
        self:SetTintAlpha( 1.4 )
        self:Bloom( 0.15 )
    else
        self:SetTintAlpha( 1 )
    end
    self:SetSaturation( self.enabled and 1 or 0.1 )
end


function DragButton:StartDragging()
    self:SetFocus(true)
    self.down = true
    self.dragging = true
    local x, y = self:TransformFromWorld(TheGame:GetInput():GetMousePos())
    self.drag_offset = y
    self:StartUpdating()
    self:UpdateImage()
end

function DragButton:StopDragging()
    self.down = false
    self.dragging = false
    self:UpdateImage()
    self:StopUpdating()
end

function DragButton:HandleControlDown(control)
    if control:Has( Controls.Digital.MENU_ACCEPT ) and self.hover and not self.dragging then
        self:StartDragging()
        return true
    end
end

function DragButton:HandleControlUp(control, dev)
    if control:Has( Controls.Digital.MENU_ACCEPT ) and self.dragging then
        self:StopDragging()
    end
end

function DragButton:OnGainHover()
    DragButton._base.OnGainHover(self)
    self:UpdateImage()
end

function DragButton:OnLoseHover()
    DragButton._base.OnLoseHover(self)
    if not self.dragging then
        self.down = false
        self:UpdateImage()
    end
end

function DragButton:OnLoseFocus()
    DragButton._base.OnLoseFocus(self)
    if not self.dragging then
        self.down = false
        self:UpdateImage()
    end
end

function DragButton:OnUpdate(dt)
    self._base.OnUpdate(self, dt)

    if self.dragging then
        local mx, my = self:TransformFromWorld(TheGame:GetInput():GetMousePos())
        local x,y = self:GetPos()
        
        if self.horizontal then
            local new_x = clamp(x + mx - self.drag_offset, self.min, self.max)
            
            if self.dragfn then
                local p = (new_x - self.min)/(self.max - self.min)  
                p = self.dragfn( p ) or p
                new_x = self.min + (self.max - self.min)*p
            end
            self:SetPos(new_x, y)
        else
            local new_y = clamp(y + my - self.drag_offset, self.min, self.max)
            
            
            if self.dragfn then
                local p = 1 - (new_y - self.min)/(self.max - self.min)  
                --self.dragfn( p )
                p = self.dragfn( p ) or p
                new_y = self.min + (self.max - self.min)*(1-p)
            end
            self:SetPos(x, new_y)
        end
    else
        self:StopUpdating()
    end

end
