require "ui/widgets/arrowbutton"

require "ui/widgets/solidbox"
require "ui/widgets/menubutton"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/scrollpanel"

local DropDown = class("Widget.DropDown", Widget)

local assets = 
{
    arrow = engine.asset.Texture(   "UI/dropdown_arrow.tex" ),
    inner = engine.asset.Texture(   "UI/dropdown_inner.tex" ),
    normal = engine.asset.Texture(   "UI/btn_normal.tex" ),
    hover = engine.asset.Texture(   "UI/btn_normal_hover.tex" ),
}

local DEFAULT_TEXTSIZE = 20
local DEFAULT_MARGIN = 5

function DropDown:init(w, h )

    w = w or 200
    h = h or 30
    DropDown._base.init(self)

    self.font_size = DEFAULT_TEXTSIZE
    self.bg_border_scale = 0.5
    self.bg_border = 36
    self.bg = Widget.Panel( assets.normal )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetBloom( 0.08 )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :SetSize( w, h )
    self.txt = Widget.Label("body", self.font_size, "...")
        :SetGlyphColour( UICOLOURS.BUTTON_TEXT )
        :LeftAlign()
    self.arrow = Widget.Image( assets.arrow )
        :SetTintColour( UICOLOURS.BUTTON_TEXT )
        :SetSize( self.font_size*0.7, self.font_size*0.7 )
    self.direction_down = true

    self:AddChildren{
        self.bg,
        self.txt,
        self.arrow,
    }

    self.scroll_root = Widget()

    self.hover_check = true
    self.blocks_mouse = true
    self:UpdateState()
    self:SetSize(w, h)
end

function DropDown:SetTexture( normal, hover )
    if normal then self.bg_normal = normal end
    if hover then self.bg_hover = hover end
    return self
end

function DropDown:SetTextColour( colour )
    self.txt:SetGlyphColour( colour )
    self.arrow:SetTintColour( colour )
    return self
end

function DropDown:SetDropdownBGColour( colour )
    self.inner_colour = colour
    return self
end

function DropDown:GetMenu()
    return self.menu
end

function DropDown:SetLabel( font )
    if font then
        self.txt:SetFont( font )
    end
    return self
end

function DropDown:SetSize( w, h )
    self.w = w or self.w or 200
    self.h = h or self.h or 40

    local nineslice_w, nineslice_h = self.w + self.bg_border*2*self.bg_border_scale, self.h + self.bg_border*2*self.bg_border_scale
    self.bg:SetSize( nineslice_w, nineslice_h )

    self.arrow:LayoutBounds( "right", "center", self.bg )
        :Offset( -10, 1 )
    self.txt:SetRegionSize( self.w - DEFAULT_MARGIN*2, self.h )
        :SetPos(10, 1)

    self.inner_h = self.options and math.min(200, 20*(#self.options+2)) or 0
    
    if self.inner then
        self.inner:SetPos(0, -self.inner_h/2-self.h/2)
        self.scroll:SetSize(self.w, self.inner_h)
        self.innerbg:SetSize(self.w, self.inner_h)
    end

    self.scroll_root:SetPos(0, 0)
    return self
end

function DropDown:SetInnerLayout( xlayout, ylayout )
    self.inner_xlayout, self.inner_ylayout = xlayout, ylayout
    return self
end

function DropDown:SetDirectionUp()
    self.direction_down = false
    self:RefreshArrow()
    return self
end

function DropDown:SetDirectionDown()
    self.direction_down = true
    self:RefreshArrow()
    return self
end

function DropDown:UpdateState()
    self:SetSaturation(1)

    if not self.enabled then
        self:SetSaturation(0.1)
    elseif self.down then
        self:SetTintColour(1,1,1,.7)
    elseif self.hover or self.focus then
        self.bg:SetTexture( self.bg_hover or assets.hover )
        self:SetTintColour(1.2,1.2,1.2,1)
    else
        self.bg:SetTexture( self.bg_normal or assets.normal )
        self:SetTintColour(1,1,1,.7)
    end
    self:RefreshArrow()
end


function DropDown:SetOptions(opt, idx)
    self.options = opt
    self:SetIndex(idx or 1, true)
    self:SetSize(self.w, self.h)
    self.populated = false
    return self
end

function DropDown:SetEnabled(val)
    DropDown._base.SetEnabled(self, val)
    self:UpdateState()
    return self
end

function DropDown:Clear()
    self.menu:Clear()
end

function DropDown:SetText(txt)
    self.txt:SetText(txt)
    return self
end


function DropDown:SetIndex(idx, nofn)
    if self.options[idx] then
        if idx ~= self.index then
            self.index = idx
        
            self:SetText(type(self.options[idx]) == "string" and self.options[idx] or self.options[idx].txt)
            
            if not nofn then
                if self.fn then
                    self.fn(idx, self.options[idx])
                end
            end
        end

    else
        self:SetText("...")
    end
end

function DropDown:SetIndexByData(data, key_name, nofn)
    
    for k,v in pairs(self.options) do
        if v == data or (type(v) == "table" and v[key_name or "data"] == data) then
            
            self:SetIndex(k, nofn)
            return true
        end
    end
    
end

function DropDown:GetValue()
    return self.options[self.index]
end

function DropDown:SetFn(fn)
    self.fn = fn
    return self
end

function DropDown:OnClickArrow()
    if not self.open then
        self:Open()
    else
        self:Close()
    end
end

function DropDown:GetIndex()
    return self.index
end

function DropDown:OnGainHover()
    self:UpdateState()
end

function DropDown:OnLoseHover()
    self:UpdateState()
end

function DropDown:OnGainFocus()
    self:UpdateState()
end 

function DropDown:OnLoseFocus()
    if self.open then
        if self.scroll then
            if not self.scroll.hover then
                self:Close()
            end
        end
    end
    self:UpdateState()
end

function DropDown:Populate()
    self.scroll_root:DestroyAllChildren()
    local inner_w = self.scroll:GetInnerWidth()    

    if self.options then

        local h = self.font_size + 2
        local children = {}
        for k,v in ipairs(self.options) do
            local str = type(v) == "string" and v or v.txt
            local fn = function() self:SetIndex(k) self:Close() end
            local w = Widget.MenuButton( str, fn, nil, nil, self.font_size )
            w:SetBGColour( 0x00000000 )
            w:SetTextColour( UICOLOURS.TITLE )
            w:SetRect(0, -h*(k-1), inner_w, h)
            w.label:LeftAlign():SetWordWrap(false)
            table.insert(children, w)
        end
        self.scroll_root:AddChildren(children)
        self.scroll:RefreshView()
    end
    self.populated = true
end

function DropDown:SetOpenFn ( open_fn )
    self.open_fn = open_fn
    return self
end

function DropDown:SetCloseFn ( close_fn )
    self.close_fn = close_fn
    return self
end

function DropDown:Open()
    if self.open then
        return
    end

    if self.open_fn then
        self.open_fn()
        self.open = true
    
    elseif #self.options > 0 then
        self.open = true
        
        if not self.inner then
            self.inner = self:AddChild( Widget() )
            self.innerbg = self.inner:AddChild( Widget.Panel( assets.inner ) )
                :SetTintColour( self.inner_colour or UICOLOURS.PANEL_BG)
                :SetSize( self.w, self.inner_h )

            self.scroll = self.inner:AddChild( Widget.ScrollPanel() )
            self.scroll:AddScrollChild(self.scroll_root)
            self.scroll.OnLoseFocus = function() if not self.hover then self:Close() end end

            self.inner:SetPos( 0, -self.inner_h/2-self.h/2 )
            self.scroll:SetSize( self.w, self.inner_h )
        end

        self.inner:LayoutBounds( "left", self.direction_down and "below" or "above", self.bg )
        self.inner:Show()
        self:SendToFront()

        if not self.populated then
            self:Populate()
        elseif self.scroll_root.children then
            for k,v in pairs(self.scroll_root.children) do
                v:UpdateImage()
            end
        end

        -- Make focus stay within the dropdown options
        self.scroll_root:SetFocusLock( true )
        self.scroll_root.children[1]:SetFocus()

    end
    self:RefreshArrow()
end

function DropDown:Close()
    if self.open then
        -- Stop focus from staying within the dropdown options
        self.scroll_root:SetFocusLock( false )
        local focus = self:GetFE():GetFocusWidget()
        if focus and self:IsAncestorOf( focus ) then
            -- If an option was focused, give focus back to the dropdown
            self:SetFocus()
        end

        if self.close_fn then
            self.close_fn()
            self.open = false
        else
            self.open = false
            self.inner:Hide()
        end
    end
    self:RefreshArrow()
end

function DropDown:HandleControlUp(control, dev)
    if control:Has( Controls.Digital.MENU_CANCEL ) then
        if self.open then
            self:Close()
            return true
        end
        
    elseif control:Has( Controls.Digital.MENU_ACCEPT ) then
        self:OnClickArrow()
        return true
    end
end

function DropDown:RefreshArrow()
    local point_up = ( self.direction_down and self.open ) or ( not self.direction_down and not self.open )
    self.arrow:SetScale( 1, point_up and 1 or -1 )
        :SetPos( nil, point_up and 1 or -1 )
end
