require "ui/widget"
require "ui/widgets/solidbox"
require "ui/widgets/label"
local EditLabel = class("Widget.EditLabel", Widget)
EditLabel.can_focus = true

function EditLabel:init( font_size, w, lines)
    EditLabel._base.init(self)
    self.lines = lines or 1
    self.hover_check = true
    self.open = true
    self.current_str = ""
    self.cursor = self.cursor or 1
    self.offset = self.offset or 0
    self.font_size = font_size or 30
    self.outline_w = 2
    self.w = w or 200
    self.h = self.font_size*self.lines

    self.outline = self:AddChild(Widget.SolidBox( self.w+self.outline_w*2, self.h + self.outline_w*2, 0, 0, 0, 1))
    self.bg = self:AddChild(Widget.SolidBox( self.w, self.h))
    self.text = self:AddChild(Widget.Label( "body", self.font_size):SetWordWrap(true):TopAlign():LeftAlign():SetSize(self.w-8, self.font_size*self.lines):SetGlyphColour(0x000000ff):SetPos(4, 0) )
    self.text.model:SetBreakLongWords(true)
    self:UpdateDisplayString()

    self:UpdateOutline()
end

function EditLabel:SetSize( w, h, outline )
    self.outline_w = outline or self.outline_w or 2
    self.w = w or self.w or 200
    self.h = h or self.font_size*self.lines
    self.outline:SetSize( self.w+self.outline_w*2, self.h+self.outline_w*2 )
    self.bg:SetSize( self.w, self.h )
    self.text:SetSize( self.w-8, self.h )
end

function EditLabel:GoToEndOfLine()
    self.cursor = utf8.len(self.current_str or "") + 1
    self:UpdateDisplayString()
end

function EditLabel:UpdateOutline()
    self.outline:Bloom(0.05)
    if self.hover then
        self.outline:SetColour(UICOLOURS.FOCUS)
        self.outline:Bloom(0.2)
    elseif self.focus then
        self.outline:SetColour(UICOLOURS.FOCUS)
    else
        self.outline:SetColour(UICOLOURS.SUBTITLE)
    end   
end

function EditLabel:SetFont(font)
    self.text:SetFont(font)
    return self
end

function EditLabel:SetTextColour(r,g,b,a)
    self.text:SetGlyphColour(r,g,b,a)
    return self
end

function EditLabel:SetBGColour(r,g,b,a)
    self.bg:SetColour(r,g,b,a)
    return self
end

function EditLabel:GetString()
    return self.current_str
end

function EditLabel:InsertCharacters(str)
    local left = utf8.sub(self.current_str, 1, self.cursor-1) or ""
    local right = utf8.sub(self.current_str, self.cursor) or ""
    if left and right then
        local new_string = left.. str .. right
        self:SetString(new_string)
        if self.current_str == new_string then
            self.cursor = self.cursor + utf8.len(str)
            self:UpdateDisplayString()
        end
    end
end

function EditLabel:SetValue(str, silent)
    self:SetString(str, silent)
    return self
end

function EditLabel:SetString(str, silent)
    local old_cursor = self.cursor
    local old_string = self.current_str or ""
    str = str or ""
    local len = utf8.len( str )
    if str ~= self.current_str and len ~= nil then
        self.current_str = str
        self.cursor = math.min(self.cursor, len + 1)
        self:UpdateDisplayString()

        if self.text.model:GetLines() > self.lines then
            self.current_str = old_string
            self.cursor = old_cursor
            self:UpdateDisplayString()
        else
            if self.onchange and not silent then
                self.onchange( str )
            end
        end
    end

    return self

end

function EditLabel:SetFn(fn)
    self.onchange = fn
    return self
end

function EditLabel:UpdateDisplayString()
    local str = self.current_str
    
    self.text:SetRawText(str)
    if self.focus then
       self.text:SetCursor(self.cursor)
    else
       self.text:SetCursor()
    end
end

function EditLabel:HandleTextInput(text)
    
    self:InsertCharacters(text)
    self:UpdateDisplayString()
    return true
end

function EditLabel:HandleTextEdit(text)
    
    --self:InsertCharacters(text)
    --self:SetString(text, true)
    --self:UpdateDisplayString()
    return true
end


function EditLabel:HandleControlDown(control, device)
    if control:Has( Controls.Digital.MENU_ACCEPT ) and device == "mouse" then
        self:SetFocus()
        local cursor = self.text:GetCursorAtPoint(TheGame:GetInput():GetMousePos())
        if cursor == nil or cursor > 0 then
            self.cursor = cursor or 1
            self:UpdateDisplayString()
        else
            self:GoToEndOfLine()
        end
    end
end

function EditLabel:SetOnEnterFn(fn)
    self.onenterpressed = fn
    return self
end

function EditLabel:HandleRawKeyDown(key)
    if key == "ENTER" and not TheGame:GetInput():IsModifierControl() then
        if math.max( 1, self.text.model:GetLines() ) < self.lines then
            --print( utf8.len(self.current_str), self.cursor )

            local left = utf8.sub(self.current_str, 1, self.cursor-1)

            local right = utf8.sub(self.current_str, self.cursor)
            local right_len = utf8.len(right)
            local left_len = utf8.len(left)

            if right_len > 0 and left_len > 0 then
                self:SetString(left .. "\n" .. right)
                self.cursor = self.cursor + 1
            elseif left_len > 0 then
                self:SetString(left .. "\n")
                self.cursor = utf8.len(self.current_str) + 1
            elseif right_len > 0  then
                self:SetString("\n" .. right)
                self.cursor = 0
            else
                self:SetString("\n")
                self.cursor = 1
            end
        end

        if self.onenterpressed then self.onenterpressed() end
        
    elseif key == "BACKSPACE" then
        if self.cursor > 1 then
            local new_str = utf8.sub(self.current_str, 1, self.cursor-2) .. utf8.sub(self.current_str, self.cursor)
            if self.cursor < (utf8.len(self.current_str) + 1) then
                self.cursor = self.cursor - 1
            end
            self:SetString(new_str)
        end

    elseif key == "DELETE" or key == "KP_PERIOD" then
        if self.cursor <= utf8.len(self.current_str) then
            self:SetString(
				utf8.sub(self.current_str, 1, self.cursor-1)
				..
				utf8.sub(self.current_str, self.cursor+1)
			)
        end
    elseif key == "DOWN" or key == "KP_2" then
        local new_cursor = self.text.model:GetVerticalCursorMove(self.cursor, false)
        self.cursor = new_cursor and new_cursor > 0 and new_cursor or utf8.len(self.current_str) + 1
    elseif key == "UP" or key == "KP_8" then
        local new_cursor = self.text.model:GetVerticalCursorMove(self.cursor, true)
        self.cursor = new_cursor and new_cursor > 0 and new_cursor or 1
    
    elseif key == "LEFT" or key == "KP_4" then
        self.cursor = math.max(1, self.cursor - 1)
    elseif key == "RIGHT" or key == "KP_6" then
        self.cursor = math.min(utf8.len(self.current_str)+1, self.cursor + 1)
    elseif key == "HOME" or key == "KP_7" then
        self.cursor = 1
    elseif key == "END" or key == "KP_1" then
        local len = utf8.len(self.current_str)
        self.cursor = len + 1
    else
        return false
    end
    self:UpdateDisplayString()
    return true
end

function EditLabel:SetGainFocusFn(fn)
    self.ongainfocus = fn
    return self
end

function EditLabel:OnGainFocus()
    engine.inst:StartTextInput()
    self:UpdateDisplayString()
    self:UpdateOutline()
    self:GoToEndOfLine()
    if self.ongainfocus then self.ongainfocus() end
end

function EditLabel:SetLoseFocusFn(fn)
    self.onlosefocus = fn
    return self
end

function EditLabel:OnLoseFocus()
    --print ("LOSE FOC")
    engine.inst:StopTextInput()
    self:UpdateDisplayString()
    self:UpdateOutline()
    if self.onlosefocus then self.onlosefocus() end
end


function EditLabel:OnGainHover()
    --print ("GAIN HOV")
    self:UpdateOutline()
end


function EditLabel:OnLoseHover()
    --print ("LOSE HOV")
    self:UpdateOutline()
end
