--------------------------------------------------------------------------------
-- Displays an horizontal list of icons showing the various details about a fight location

local FightLocationStatus = class( "Widget.FightLocationStatus", Widget )

function FightLocationStatus:init( icon_size )
    FightLocationStatus._base.init( self )

    -- Add a background
    self.icon_size = icon_size
    self.bg_border = 50
    self.bg_border_scale = 0.5
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture ("UI/compendium_tab_selected.tex") ) )
    	:SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
    	:SetHiddenBoundingBox( true )
        :SetTintColour( UICOLOURS.PENALTY )
        :Bloom( 0.1 )

    -- Create the various icons
    self.icons = self:AddChild( Widget() )
    self.no_run = self.icons:AddChild( Widget.FightLocationStatusIcon( icon_size, engine.asset.Texture("ui/ic_handicaps_noescape.tex"), UICOLOURS.PENALTY, LOC"CONVO_OPTION.NO_RUN_TITLE", LOC"CONVO_OPTION.NO_RUN_DESC" ) )
    self.isolated = self.icons:AddChild( Widget.FightLocationStatusIcon( icon_size, engine.asset.Texture("ui/ic_handicaps_isolated.tex"), UICOLOURS.SUBTITLE, LOC"CONVO_OPTION.ISOLATED_TITLE", LOC"CONVO_OPTION.ISOLATED_DESC" ) )
    self.no_surrender = self.icons:AddChild( Widget.FightLocationStatusIcon( icon_size, engine.asset.Texture("ui/ic_handicaps_no_surrender.tex"), UICOLOURS.PENALTY, LOC"CONVO_OPTION.NOSURRENDER_TITLE", LOC"CONVO_OPTION.NOSURRENDER_DESC" ) )
    self.survival = self.icons:AddChild( Widget.FightLocationTurnCounter( self.icon_size, engine.asset.Texture("ui/ic_handicaps_noescape.tex")))

    self:Refresh()
end

function FightLocationStatus:Refresh( battle )
	if battle then
        local scenario = battle:GetScenario()
		self.no_run:SetShown( not scenario:CanFlee() )
		self.isolated:SetShown(CheckBits( scenario.flags, BATTLE_FLAGS.ISOLATED ))
        self.no_surrender:SetShown(CheckBits( scenario.flags, BATTLE_FLAGS.NO_SURRENDER ))
        if scenario:GetSurvivalTurns() ~= nil then
            local turns_left = scenario:GetSurvivalTurns() - math.max( 0, battle:GetTurns() - 1)
            self.survival:RefreshTurns( turns_left )
            self.survival:Show()
        else
            self.survival:Hide()
        end

		-- Layout icons
		self.icons:StackChildrenRow( 10 )
		local icons_w, icons_h = self.icons:GetSize()
		icons_w = icons_w + SPACING.M1*0.5
		icons_h = icons_h + SPACING.M1*1.25
		local nineslice_w, nineslice_h = icons_w + self.bg_border*2*self.bg_border_scale, icons_h + self.bg_border*2*self.bg_border_scale
    	self.bg:SetSize( nineslice_w, nineslice_h )
			:LayoutBounds( "center", "center", self.icons )
			:SetScale( 1, -1 )

        self.bg:SetShown( self.no_run:IsShown() or self.isolated:IsShown() or self.no_surrender:IsShown() or self.survival:IsShown() )
	else
		self:Hide()
	end
	return self
end

--------------------------------------------------------------------------------
-- Displays a fight location icon

local FightLocationStatusIcon = class( "Widget.FightLocationStatusIcon", Widget )

function FightLocationStatusIcon:init( icon_size, icon, colour, title, desc )
    FightLocationStatusIcon._base.init( self )

    self.icon_size = icon_size or 48
    self.icon = self:AddChild( Widget.Image( icon, self.icon_size, self.icon_size ) )
    	:SetTintColour( colour and colour or UICOLOURS.TITLE )
    self:SetToolTip( loc.format( LOC"CONVO_OPTION.LOCATION_STATUS_TOOLTIP", title, desc ) )
		:SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
		:SetFocusable( true )
		:ShowToolTipOnFocus( true )
end

function FightLocationStatusIcon:OnGainFocus()
    self:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
end

function FightLocationStatusIcon:OnLoseFocus()
    self:ScaleTo( nil, 1, 0.2, easing.outQuad )
end


--------------------------------------------------------------------------------
-- Displays a turn timer for Survival

local FightLocationTurnCounter = class( "Widget.FightLocationTurnCounter", Widget )

function FightLocationTurnCounter:init( icon_size, icon )
    FightLocationStatusIcon._base.init( self )

    self.icon_size = icon_size or 48
    self.icon = self:AddChild( Widget.Image( icon, self.icon_size, self.icon_size ) )
        :SetTintColour( UICOLOURS.TITLE )
    self.label = self:AddChild( Widget.Label( "title", 32 ))
end

function FightLocationTurnCounter:RefreshTurns( turns )
    self.label:SetText( loc.format( LOC "UI.FIGHT_SCREEN.TURNS_LEFT", turns ))
    self.label:LayoutBounds( "after", "bottom", self.icon ):Offset( 5, 0 )

    local desc = loc.format( LOC"CONVO_OPTION.SURVIVAL_DESC", turns )
    self:SetToolTip( loc.format( LOC"CONVO_OPTION.LOCATION_STATUS_TOOLTIP", LOC"CONVO_OPTION.SURVIVAL_TITLE", desc ) )
        :SetToolTipLayoutFn( function( w, tooltip_widget ) tooltip_widget:LayoutBounds( "left", "below", w ):Offset( 0, -SPACING.M1 ) end )
        :SetFocusable( true )
        :ShowToolTipOnFocus( true )
end


function FightLocationTurnCounter:OnGainFocus()
    self:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
end

function FightLocationTurnCounter:OnLoseFocus()
    self:ScaleTo( nil, 1, 0.2, easing.outQuad )
end