require "ui/widget"

local FloatLabel = class( "Widget.FloatLabel", Widget )

function FloatLabel:init( font, sz, txt )
	Widget.init( self )
	self.label = Widget.Label( font, sz, txt )
    self.label:EnableOutline( LookupTuningValue( "FIGHT_FLOATER_OUTLINE" ))

	self:AddChild( self.label )

	self.dt = 0
end

function FloatLabel:ApplyValue( key, value )
    if key == "dx" then
        self.dx = value
    elseif key == "dy" then
        self.dy = value
    elseif key == "size" then
        self.label:SetFontSize( value )
    elseif key == "scale" then
        local base_scale = self:GetFE():GetBaseWidgetScale()
        value = value * base_scale
        self:SetScale( value, value, value )

    elseif key == "bloom" then
        self.label:SetBloom( value )
        if self.icons then
            for i, icon in ipairs(self.icons) do
                icon:SetBloom( value )
            end
        end
    elseif key == "alpha" then
    	self:SetTintAlpha( value )
        if self.icons then
            for i, icon in ipairs(self.icons) do
            	icon:SetTintAlpha( value )
            end
        end
    end
end

function FloatLabel:OnUpdate(dt)
	Widget.OnUpdate( self, dt )
    if self.keyframes == nil then
        return
    end

    self.dt = self.dt + dt
    local dt = self.dt

    self.dx, self.dy = nil, nil, nil

    local done = true
    for k, v in pairs( self.keyframes ) do
        for i = 1, #v, 3 do
            local t0, v0, ease_fn, t1, v1 = table.unpack( v, i, i + 5 )
            if dt >= t0 then
                if t1 == nil then
                    self:ApplyValue( k, v0 )
                    break
                elseif dt <= t1 then
                    local t = clamp( (dt - t0) / (t1 - t0), 0, 1.0 )
                    local v = ease_fn( t, v0, v1 - v0, 1.0 )
                    self:ApplyValue( k, v )
                    done = false
                    break
                end
            else
                done = false
                break
            end
        end
    end

    if self.dx or self.dy then
        self:SetPos( self.x0 + (self.dx or 0), self.y0 + (self.dy or 0) )
    end

    if done then
        self:Remove()
    end
end

function FloatLabel:SetKeyFrames( keyframes )
    self.keyframes = keyframes
end

function FloatLabel:AddKeyFrames( prop, keyframes )
    assert( #keyframes % 3 == 2 )
    if self.keyframes == nil then
        self.keyframes = {}
    end
    self.keyframes[ prop ] = keyframes
end

function FloatLabel:SetBloom( bloom )
    self.label:SetBloom( bloom )
    return self
end

function FloatLabel:SetGlyphColour( clr )
    self.label:SetGlyphColour( clr )
    return self
end

local function DefaultFloat( w )
    w:SetRotation( 0 )
    local sx = w.sx
    w:ScaleTo( sx, sx * 2.0, 0.5, easing.outCubic )
    w:MoveTo( w.x, w.y + 220 * w:GetFE():GetBaseWidgetScale(), 0.3, easing.outCubic )
    w:Delay( 0.2 )
    --w:ScaleTo( sx * 3, sx * 2.5, 0.15, easing.outQuad )    w:Delay( 0.3 )
    w:AlphaTo( 0, 1.2 ):DelayIfAnimating()
    w:Remove()
end

function FloatLabel:StartFloater( fn )
	if self.keyframes == nil then
        self:StartCoroutine( fn or DefaultFloat, self )
    else
        self.dt = 0
        self.x0, self.y0 = self.x, self.y

        self:OnUpdate( 0 ) -- Apply values at t=0
        self:StartUpdating()
    end
end
