local CONTENT_LEFT_MARGIN = 15 -- Extra size added on the left of scroll boxes, so that the contents can scale up on hover without clipping
require "ui/widgets/compendiumwidget"
------------------------------------------------------------------------------------------
-- Shows the grafts panel

local assets = {
    bg = engine.asset.Texture("UI/compendium_graft_bg.tex"),
    glow = engine.asset.Texture("images/glow.tex" ),
    lock = engine.asset.Texture("UI/card_lock.tex"),
    graft_draft = engine.asset.Texture("UI/ic_draft_graft.tex"),
    checkmark = engine.asset.Texture("UI/ic_checkmark.tex" ),
    locked = engine.asset.Texture("UI/ic_locked.tex" ),
}


local GraftsCompendiumWidget = class( "Widget.GraftsCompendiumWidget", Widget.CompendiumWidget )

GraftsCompendiumWidget.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_ACCEPT,
        fn = function(self)
            local focus = self:GetFE():GetFocusWidget()
            if focus and focus._class == Widget.GraftsCompendiumGraft._class then
                focus:Click()
                return true
            end
        end,
        test = function(self)
            local focus = self:GetFE():GetFocusWidget()
            if self:CanNav() and focus and focus._class == Widget.GraftsCompendiumGraft._class and not focus.is_locked then
                return true
            end
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.VIEW_CARD", Controls.Digital.MENU_ACCEPT))
        end
    },

}

local columns = 3
local graft_w
local graft_h
local spacing = 20

function GraftsCompendiumWidget:init( width, height, sidebar_width )
    GraftsCompendiumWidget._base.init( self, width, height, sidebar_width )

    self.graft_defs = {}
    local grafts = Content.GetAllGrafts()
    for i, graft_def in ipairs( grafts ) do
        local graft_id = graft_def.id
        if (graft_def.type == GRAFT_TYPE.COMBAT or graft_def.type == GRAFT_TYPE.NEGOTIATION) and not graft_def.hide_in_compendium then 
            self.graft_defs[ graft_id ] = graft_def
        end
    end

end

function GraftsCompendiumWidget:AddSidebarFilters()
    -- Add character filters
    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.FILTER_CHARACTERS" )
    self.series_filter_group = self.sidebar:AddFilterWidget( Widget.FilterRadioGroup() )
    self.agent_filters = {}
    for k, data in ipairs(GetAllPlayerBackgrounds()) do
        if data.id then
            local row = self.series_filter_group:AddFilter( Widget.GraftSeriesFilterRow( data.data.player_agent, { data.id } ) )
            self.agent_filters[data.id] = row
            -- Lock filters for locked characters
            local is_character_locked = TheGame:GetGameProfile():IsCharacterLocked( data.data.player_agent )
            if Screen.Compendium.ShowAllUnlocks() then is_character_locked = false end
            self.agent_filters[data.id]:SetLocked( is_character_locked )
        end
    end

    local general_character = table.arraypick( { "BANDIT_GOON", "JAKES_SMUGGLER", "ADMIRALTY_CLERK" } )
    self.series_general = self.series_filter_group:AddFilter( Widget.GraftSeriesFilterRow( general_character, { CARD_SERIES.GENERAL }, LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_SERIES_GENERAL_TT" ) )
    self.series_filter_group:SetSelected( 1 )

    -- Add a text search bar
    if SPECIAL_BUILD_TYPE < 1024 then
        self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH" )
        self.sidebar:AddFilterWidget( Widget.GraftSearchFilterBlock() )
    end

    -- Add a negotiation/battle filter
    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_MINIGAME" )
    self.minigame_filter_group = self.sidebar:AddFilterWidget( Widget.FilterRadioGroup() )
    self.series_battle = self.minigame_filter_group:AddFilter( Widget.GraftMinigameFilterRow( GRAFT_TYPE.COMBAT, "event:/ui/card_codex/select_battle_cards" ) )
    self.series_negotiation = self.minigame_filter_group:AddFilter( Widget.GraftMinigameFilterRow( GRAFT_TYPE.NEGOTIATION, "event:/ui/card_codex/select_negotiation_cards" ) )
    self.minigame_filter_group:SetSelected( 1 )

    -- Add a rarity filter
    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_RARITY" )
    self.rarity_filter_group = self.sidebar:AddFilterWidget( Widget.FilterOptionalRadioGroup() )
        :SetRowMode()
    self.rarity_filter_group:AddFilter( Widget.GraftRarityFilterRow( CARD_RARITY.COMMON, LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_RARITY_COMMON_TT" ) )
    self.rarity_filter_group:AddFilter( Widget.GraftRarityFilterRow( CARD_RARITY.UNCOMMON, LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_RARITY_UNCOMMON_TT" ) )
    self.rarity_filter_group:AddFilter( Widget.GraftRarityFilterRow( CARD_RARITY.RARE, LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_RARITY_RARE_TT" ) )
    self.rarity_filter_group:AddFilter( Widget.GraftRarityFilterRow( CARD_RARITY.BOSS, LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_RARITY_BOSS_TT" ) )
    self.rarity_filter_group:AddFilter( Widget.GraftRarityFilterRow( CARD_RARITY.COSMIC, LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_RARITY_COSMIC_TT" ) )
    self.rarity_filter_group:SetSelected( 0 )

    -- Add a sorting filter
    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.GRAFT_FILTER_SORT" )
    self:AddSortingWidget()
end

function GraftsCompendiumWidget:AddSortingWidget()
    local sorting_options = {
        { txt = LOC"UI.CARDCOMPENDIUM.SORT_OPTION_GRAFT_NAME", fn = function( graft_widget_1, graft_widget_2 ) return graft_widget_1.name < graft_widget_2.name end },
        { txt = LOC"UI.CARDCOMPENDIUM.SORT_OPTION_GRAFT_RARITY", fn = function( graft_widget_1, graft_widget_2 )
            local graft1, graft2 = graft_widget_1.graft_def, graft_widget_2.graft_def
            local rarity1 = 0
            if graft1.rarity == CARD_RARITY.BASIC then
                rarity1 = 1
            elseif graft1.rarity == CARD_RARITY.COMMON then
                rarity1 = 2
            elseif graft1.rarity == CARD_RARITY.UNCOMMON then
                rarity1 = 3
            elseif graft1.rarity == CARD_RARITY.RARE then
                rarity1 = 4
            elseif graft1.rarity == CARD_RARITY.BOSS then
                rarity1 = 5
            end

            local rarity2 = 0
            if graft2.rarity == CARD_RARITY.BASIC then
                rarity2 = 1
            elseif graft2.rarity == CARD_RARITY.COMMON then
                rarity2 = 2
            elseif graft2.rarity == CARD_RARITY.UNCOMMON then
                rarity2 = 3
            elseif graft2.rarity == CARD_RARITY.RARE then
                rarity2 = 4
            elseif graft2.rarity == CARD_RARITY.BOSS then
                rarity2 = 5
            end
            
            if rarity1 == rarity2 then 
                return graft_widget_1.name < graft_widget_2.name
            end

            return rarity1 < rarity2
        end },
        { txt = LOC"UI.CARDCOMPENDIUM.SORT_OPTION_GRAFT_SEEN", fn = function( graft_widget_1, graft_widget_2 ) return (graft_widget_1.graft_def.was_seen and 1 or 0) > (graft_widget_2.graft_def.was_seen and 1 or 0) end },
        { txt = LOC"UI.CARDCOMPENDIUM.SORT_OPTION_GRAFT_INSTALLED", fn = function( graft_widget_1, graft_widget_2 ) return (graft_widget_1.graft_def.was_installed and 1 or 0) > (graft_widget_2.graft_def.was_installed and 1 or 0) end },
        
    }
    local selected_idx = 2 -- Rarity
    self.sidebar:AddSortingWidget( sorting_options, function( idx, option ) self:SortOrderSelected( option ) end, selected_idx )
end

function GraftsCompendiumWidget:CanNav()
    return true
end

function GraftsCompendiumWidget:SortOrderSelected( option )
    self.sorting_fn = option.fn
    if self.graft_widgets then
        self:LayoutContent()
    end
end

function GraftsCompendiumWidget:RefreshContent()
    GraftsCompendiumWidget._base.RefreshContent()

    if self.graft_widgets then
        return
    end

    self.graft_widgets = {}

    -- Refresh locked status
    for series, row in pairs(self.agent_filters) do
        local is_character_locked = TheGame:GetGameProfile():IsCharacterLocked( series )
        if Screen.Compendium.ShowAllUnlocks() then is_character_locked = false end
        row:SetLocked( is_character_locked )
    end

    -- Calculate the size of each widget
    self.scroll_inner_w = self.content_w - 40
    graft_w = (self.scroll_inner_w-(spacing*(columns-1)))/columns
    graft_h = 90

    local function OnLazyLoad( loader )
        local graft_def = loader.graft_def
        local widget = loader:AddChild( Widget.GraftsCompendiumGraft( graft_w, graft_h, loader.graft ) )
        if graft_def.unlockable then
            widget:SetProgress( Screen.Compendium.ShowAllUnlocks() and graft_def.unlockable or graft_def.unlocked or 0, graft_def.unlockable )
        end
        if graft_def.is_locked ~= nil and not Screen.Compendium.ShowAllUnlocks() then
            widget:SetLocked( graft_def.is_locked )
        end
        if not graft_def.base_id and not graft_def.is_locked then
            widget:SetOnClickFn( function() self:OnGraftSelect( graft_def, widget ) end )
        end
    end

    -- Add widgets
    for id, graft_def in pairs( self.graft_defs ) do
        -- Create widget
        if not graft_def.base_id then
            local loader = Widget.LazyLoadWidget( graft_w, graft_h, OnLazyLoad )
            loader.graft = GraftInstance( id )
            loader.graft_def = graft_def
            loader.name = loader.graft:GetName()
            -- Setup the search string data, for text based searches
            loader.search_string = loader.graft:GetName() .. " " .. loader.graft:GetDesc()

            table.insert( self.graft_widgets, loader )
        end
    end
    self.scroll_root:AddScrollChildren( self.graft_widgets )

    -- Layout all the grafts
    self:LayoutContent()

    -- Update filtering
    self.sidebar:RefreshFilters()

    return self
end

function GraftsCompendiumWidget:UpdateCount()
    -- Reset counts
    self.graft_counts = {}
    self.graft_counts["UNLOCKED"] = 0
    self.graft_counts["UNLOCKABLE"] = 0
    
    -- Go through all the grafts and check which are locked
    for id, graft_def in pairs( self.graft_defs ) do
        graft_def.unlocks_list = {}
        
        -- If this series isn't being counted yet, do it
        if not self.graft_counts[graft_def.series] then 
            self.graft_counts[graft_def.series] = {
                SEEN = 0,
                INSTALLED = 0,
                TOTAL = 0,
            }
        end

        -- If this card's minigame isn't being counted yet, do it
        if not self.graft_counts[graft_def.type] then
            self.graft_counts[graft_def.type] = {
                SEEN = 0,
                INSTALLED = 0,
                TOTAL = 0,
            }
        end

        -- Add this to the totals if it's not just an upgrade
        if not graft_def.base_id then
            graft_def.unlockable = 2
            graft_def.unlocked = 0
            graft_def.is_locked = true
            graft_def.was_seen = false
            graft_def.was_installed = false
            self.graft_counts["UNLOCKABLE"] = self.graft_counts["UNLOCKABLE"] + 2
            self.graft_counts[graft_def.series].TOTAL = self.graft_counts[graft_def.series].TOTAL + 1
            self.graft_counts[graft_def.type].TOTAL = self.graft_counts[graft_def.type].TOTAL + 1

            -- Check if this graft has been seen
            if TheGame:GetGameProfile():HasSeenGraft( graft_def.id ) or Screen.Compendium.ShowAllUnlocks() then
                graft_def.is_locked = false
                graft_def.was_seen = true
                graft_def.unlocked = (graft_def.unlocked or 0) + 1
                self.graft_counts["UNLOCKED"] = self.graft_counts["UNLOCKED"] + 1
                self.graft_counts[graft_def.series].SEEN = self.graft_counts[graft_def.series].SEEN + 1
                self.graft_counts[graft_def.type].SEEN = self.graft_counts[graft_def.type].SEEN + 1
                table.insert( graft_def.unlocks_list, 1, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_MET", graft_def.name ) ) )
                -- Check if this graft has been installed
                if TheGame:GetGameProfile():HasInstalledGraft( graft_def.id ) or Screen.Compendium.ShowAllUnlocks() then
                    graft_def.was_installed = true
                    graft_def.unlocked = (graft_def.unlocked or 0) + 1
                    self.graft_counts["UNLOCKED"] = self.graft_counts["UNLOCKED"] + 1
                    self.graft_counts[graft_def.series].INSTALLED = self.graft_counts[graft_def.series].INSTALLED + 1
                    self.graft_counts[graft_def.type].INSTALLED = self.graft_counts[graft_def.type].INSTALLED + 1
                    table.insert( graft_def.unlocks_list, 1, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_MET", graft_def.name ) ) )
                end
            end
        end

    end

    for series, row in pairs(self.agent_filters) do
        row:SetProgress( self.graft_counts[series] )
    end
    self.series_general:SetProgress( self.graft_counts["GENERAL"] )
    self.series_battle:SetProgress( self.graft_counts[GRAFT_TYPE.COMBAT] )
    self.series_negotiation:SetProgress( self.graft_counts[GRAFT_TYPE.NEGOTIATION] )
end

-- Returns the number of unlocked grafts and the number of unlockable grafts, to be displayed on the nav bar
function GraftsCompendiumWidget:GetCount()
    if self.graft_counts == nil then
        self:UpdateCount()
    end
    return Screen.Compendium.ShowAllUnlocks() and self.graft_counts["UNLOCKABLE"] or self.graft_counts["UNLOCKED"] or 0, self.graft_counts["UNLOCKABLE"] or 0
end


function GraftsCompendiumWidget:OnVizChange(viz)
    if viz then
        self:StartUpdating()
    else
        self:StopUpdating()
    end
end

function GraftsCompendiumWidget:OnUpdate(dt)
    GraftsCompendiumWidget._base.OnUpdate(self, dt)
    for k,v in ipairs(self.graft_widgets) do
        if v:CheckForLoad() then
            -- skip this
        end
    end
end

-- Lays its content out
function GraftsCompendiumWidget:LayoutContent()

    table.stable_sort( self.graft_widgets, self.sorting_fn )

    local pos = 0
    local xstart = graft_w/2 + 20
    local ystart = graft_h/2
    local idx = 1
    for k, widget in ipairs( self.graft_widgets ) do
        if widget:IsShown() then
            local column = (idx-1) % columns
            local row = math.floor( (idx-1) / columns )
            local x =  xstart + column*(graft_w+spacing)
            local y = ystart - row*(graft_h+spacing) 

            widget:SetPos( x, y )
            idx = idx + 1
        end
    end

    self:ReNav()

    -- Layout things!
    self.scroll_root:SetVirtualMargin( 20 )
    self.scroll_root:RefreshView()
    self.scroll_root:LayoutBounds( "after", "top", self.sidebar )
        :Offset( 8-CONTENT_LEFT_MARGIN, -1 )

    return self
end

function GraftsCompendiumWidget:GetDefaultFocus()
    for k, v in ipairs( self.graft_widgets ) do
        if v:IsShown() then 
            return v:GetDefaultFocus() 
        end
    end
end

function GraftsCompendiumWidget:HasSidebar()
    return true
end

function GraftsCompendiumWidget:OnFiltersChanged( filters )
    self:FilterGraftWidgets( self.graft_widgets, filters )
    self:LayoutContent()
end

function GraftsCompendiumWidget:FilterGraftWidgets( graft_widgets, filters )
    -- Go through each of the grafts
    for i, graft in pairs( graft_widgets ) do

        -- Make this graft visible by default
        graft:Show()

        -- Go through each filter
        for j, filter in ipairs( filters ) do

            -- If this filter rejects the graft, hide the graft widget
            if filter:Filter( graft.graft_def, graft ) == false then graft:Hide() end
        end
    end
end

function GraftsCompendiumWidget:OnGraftSelect( graft, graft_widget )
    if not graft_widget.is_locked then
        local details = Screen.GraftsCompendiumDetailsPopup()
        TheGame:FE():PushScreen( details )
        details:ShowGraft( graft )
    end
end

------------------------------------------------------------------------------------------
-- Shows a widget with two icons, seen and installed, which just toggle on or off if the player
-- has seen a specific unlock and instaled it or not

local SeenInstalledWidget = class( "Widget.SeenInstalledWidget", Widget )

function SeenInstalledWidget:init( width, height, icon_size, text_size )
    SeenInstalledWidget._base.init( self )

    self.icon_padding = 4

    -- Add our content
    self.seen_icon = self:AddChild( Widget.Image( assets.checkmark ) )
        :Bloom( 0.05 )
    self.seen_label = self:AddChild( Widget.Label( "title", 24 ) )
        :SetText( loc.upper( LOC"UI.CARDCOMPENDIUM.GRAFT_SEEN" ) )
        :SetGlyphColour( UICOLOURS.BONUS )
        :Bloom( 0.05 )
    self.installed_icon = self:AddChild( Widget.Image( assets.checkmark ) )
        :Bloom( 0.05 )
    self.installed_label = self:AddChild( Widget.Label( "title", 24 ) )
        :SetText( loc.upper( LOC"UI.CARDCOMPENDIUM.GRAFT_INSTALLED" ) )
        :SetGlyphColour( UICOLOURS.BONUS )
        :Bloom( 0.05 )

    self:SetIconSize( icon_size, false )
    self:SetTextSize( text_size, false )
    self:SetSize( width, height, false )
end

function SeenInstalledWidget:SetIconSize( icon_size, dont_layout )
    self.icon_size = icon_size or self.icon_size or 20
    self.seen_icon:SetSize( self.icon_size, self.icon_size )
    self.installed_icon:SetSize( self.icon_size, self.icon_size )
    if dont_layout == false then self:Layout() end
    return self
end

function SeenInstalledWidget:SetTextSize( text_size, dont_layout )
    self.text_size = text_size or self.text_size or 22
    self.seen_label:SetFontSize( self.text_size )
    self.installed_label:SetFontSize( self.text_size )
    if dont_layout == false then self:Layout() end
    return self
end

function SeenInstalledWidget:SetSize( width, height, dont_layout )
    self.width = width or self.width or 200
    self.height = height or self.height or 26
    if dont_layout == false then self:Layout() end
    return self
end

function SeenInstalledWidget:Layout()
    self.seen_label:LayoutBounds( "after", "center", self.seen_icon )
        :Offset( self.icon_padding, 0 )
    self.installed_label:LayoutBounds( "before", "center", self.seen_icon )
        :Offset( self.width, 0 )
    self.installed_icon:LayoutBounds( "before", "center", self.installed_label )
        :Offset( -self.icon_padding, 0 )
    return self
end

function SeenInstalledWidget:SetProgress( seen, installed )
    if seen then
        self.seen_icon:SetTintColour( UICOLOURS.OBJECTIVE )
            :SetTexture( assets.checkmark )
            :SetSaturation( 1 )
            :SetTintAlpha( 1 )
            :Bloom( 0.05 )
        self.seen_label:SetGlyphColour( UICOLOURS.OBJECTIVE )
            :SetSaturation( 1 )
            :SetTintAlpha( 1 )
            :Bloom( 0.05 )
    else
        self.seen_icon:SetTintColour( UICOLOURS.OBJECTIVE )
            :SetTexture( assets.locked )
            :SetSaturation( 0.1 )
            :SetTintAlpha( 0.7 )
            :Bloom( 0 )
        self.seen_label:SetGlyphColour( UICOLOURS.OBJECTIVE )
            :SetSaturation( 0.1 )
            :SetTintAlpha( 0.7 )
            :Bloom( 0 )
    end
    if installed then
        self.installed_icon:SetTintColour( UICOLOURS.OBJECTIVE )
            :SetTexture( assets.checkmark )
            :SetSaturation( 1 )
            :SetTintAlpha( 1 )
            :Bloom( 0.05 )
        self.installed_label:SetGlyphColour( UICOLOURS.OBJECTIVE )
            :SetSaturation( 1 )
            :SetTintAlpha( 1 )
            :Bloom( 0.05 )
    else
        self.installed_icon:SetTintColour( UICOLOURS.OBJECTIVE )
            :SetTexture( assets.locked )
            :SetSaturation( 0.1 )
            :SetTintAlpha( 0.7 )
            :Bloom( 0 )
        self.installed_label:SetGlyphColour( UICOLOURS.OBJECTIVE )
            :SetSaturation( 0.1 )
            :SetTintAlpha( 0.7 )
            :Bloom( 0 )
    end
    return self
end

------------------------------------------------------------------------------------------
-- Shows a widget with two icons, seen and installed, which just toggle on or off if the player
-- has seen a specific unlock and instaled it or not

local SeenInstalledCounterWidget = class( "Widget.SeenInstalledCounterWidget", Widget )

function SeenInstalledCounterWidget:init( width, height, value_size, text_size )
    SeenInstalledCounterWidget._base.init( self )

    self.value_padding = 4

    -- Add our content
    self.seen_value = self:AddChild( Widget.Label( "body", 28 ) )
        :SetGlyphColour( UICOLOURS.BONUS )
        :Bloom( 0.05 )
    self.seen_progress_bar = self:AddChild( Widget.ProgressBar( (width-10)/2, 3 ) )
        :OrientH()
        :SetOutlineColour( 0x00000000 )
        :SetBGColour( 0x00000050 )
        :SetBarColour( UICOLOURS.SUBTITLE )
        :SetBarBloom( 0.05 )
        :Hide()
    self.installed_value = self:AddChild( Widget.Label( "body", 28 ) )
        :SetGlyphColour( UICOLOURS.BONUS )
        :Bloom( 0.05 )
    self.installed_progress_bar = self:AddChild( Widget.ProgressBar( (width-10)/2, 3 ) )
        :OrientH()
        :SetOutlineColour( 0x00000000 )
        :SetBGColour( 0x00000050 )
        :SetBarColour( UICOLOURS.SUBTITLE )
        :SetBarBloom( 0.05 )
        :Hide()

    self:SetColor( UICOLOURS.OBJECTIVE )
    self:SetValueSize( value_size )
    self:SetTextSize( text_size )
    self:SetSize( width, height )
    self:SetProgress( 0, 0, 0 )
end

function SeenInstalledCounterWidget:SetColor( color )
    self.color = color or self.color or UICOLOURS.OBJECTIVE
    self.seen_value:SetGlyphColour( self.color )
        :SetTintAlpha( 0.8 )
    self.seen_progress_bar:SetBarColour( self.color )
    self.installed_value:SetGlyphColour( self.color )
        :SetTintAlpha( 0.8 )
    self.installed_progress_bar:SetBarColour( self.color )
    return self
end

function SeenInstalledCounterWidget:SetValueSize( value_size )
    self.value_size = value_size or self.value_size or 28
    self.seen_value:SetFontSize( self.value_size )
    self.installed_value:SetFontSize( self.value_size )
    return self
end

function SeenInstalledCounterWidget:SetTextSize( text_size )
    self.text_size = text_size or self.text_size or 22
    return self
end

function SeenInstalledCounterWidget:SetSize( width, height )
    self.width = width or self.width or 200
    self.height = height or self.height or 26
    self.seen_progress_bar:SetSize( (self.width-10)/2, 3 )
    self.installed_progress_bar:SetSize( (self.width-10)/2, 3 )
    return self
end

function SeenInstalledCounterWidget:Layout()
    self.seen_value:LayoutBounds( "left", "above", self.seen_progress_bar )
        :Offset( 0, 0 )
    self.installed_progress_bar:LayoutBounds( "after", "bottom", self.seen_progress_bar )
        :Offset( 10, 0 )
    self.installed_value:LayoutBounds( "left", "above", self.installed_progress_bar )
        :Offset( 0, 0 )
    return self
end

function SeenInstalledCounterWidget:SetProgress( seen, installed, total )
    self.seen_value:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_SEEN_COUNTER", seen/total ) )
    self.seen_progress_bar:SetProgress( seen/total ):Show()
    self.installed_value:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.GRAFT_INSTALL_COUNTER", installed/total ) )
    self.installed_progress_bar:SetProgress( installed/total ):Show()
    self:Layout()
    return self
end

------------------------------------------------------------------------------------------
-- Shows a single graft

local GraftsCompendiumGraft = class( "Widget.GraftsCompendiumGraft", Widget.Clickable )

local ICON_SIZE = 100

function GraftsCompendiumGraft:init( width, height, graft )
    GraftsCompendiumGraft._base.init( self )

    self.widget_w = width or 240
    self.widget_h = height or 90
    self.graft_def = graft:GetDef()
    self.graft = graft

    -- Background panel
    self.bg_border = 30
    self.bg_border_scale = 0.5
    local nineslice_w, nineslice_h = self.widget_w + self.bg_border*2*self.bg_border_scale, self.widget_h + self.bg_border*2*self.bg_border_scale
    self.bg = self:AddChild( Widget.Panel( assets.bg ) )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :SetSize( nineslice_w, nineslice_h )
        :Bloom( 0.05 )

    -- Overlay glow
    self.glow = self:AddChild( Widget.Image( assets.glow ) )
        :SetSize( self.widget_h*1.2, self.widget_h*1.2 )
        :SetHiddenBoundingBox( true )

    -- Graft icon
    self.graft_icon = self:AddChild( Widget.GraftSlot( self.graft:GetType(), ICON_SIZE ) )
        :DisableToolTip( true )
        :Refresh( self.graft )

    -- Contents widget
    self.contents = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )

    -- Text
    self.title = self.contents:AddChild( Widget.Label("title", 30 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetText( self.graft:GetName() )
        :Bloom(0.1)
    self.type = self.contents:AddChild( Widget.Label("title", 22))
        :Bloom(0.1)
    self.seen_installed_widget = self:AddChild( Widget.SeenInstalledWidget( self.widget_w - ICON_SIZE - 28 ) )

    self.rarity = self.graft:GetRarity()

    -- Rarity stuffs
    local rarity_color = CARD_RARITY_COLOURS[ self.graft:GetRarity() or CARD_RARITY.UNIQUE ]
    self.bg:SetTintColour( rarity_color )
    self.rarity_icon = self:AddChild(Widget.Image( global_images.card_rarity_icons[self.graft:GetRarity()] ))
        :SetSize( 20, 20 )
        :Bloom( 0.1 )

    -- Set type color
    if self.graft:GetType() == GRAFT_TYPE.COMBAT then
        self.type:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_COMBAT" )
        self.type:SetGlyphColour( UICOLOURS.FIGHT )
        self.glow:SetTintColour( UICOLOURS.FIGHT )
    elseif self.graft:GetType() == GRAFT_TYPE.NEGOTIATION then
        self.type:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_NEGOTIATION" )
        self.type:SetGlyphColour( UICOLOURS.NEGOTIATION )
        self.glow:SetTintColour( UICOLOURS.NEGOTIATION )
    end
    self.glow:SetTintAlpha( 0.6 )

    self:SetToolTipClass( Widget.TooltipGraft )
    self:SetToolTip( self.graft )
    self:ShowToolTipOnFocus( true )

    self:Layout()
end

function GraftsCompendiumGraft:PostFeature( keyword, name, desc )
    self.features = self.features or {}
    self.features[ keyword ] = { name = name, desc = desc }
end

function GraftsCompendiumGraft:PostCard( feature_id, card )
    self.features = self.features or {}
    self.features[ feature_id ] = { card = card }
end

function GraftsCompendiumGraft:SetLocked( locked )
    self.is_locked = locked

    if self.is_locked then
        if not self.lock_icon then
            self.lock_icon = self:AddChild( Widget.Image( assets.lock ) )
                :SetSize( 30, 30 )
                :SetTintColour( UICOLOURS.SUBTITLE )
                :Bloom( 0.05 )
        end
        self.bg:SetTintColour( UICOLOURS.SUBTITLE )
            :SetSaturation( 0.2 )
            :SetTintAlpha( 0.7 )
        self.graft_icon:SetTintColour( 0x021A18FF )
        self.title:SetText( LOC"UI.CARDCOMPENDIUM.GRAFT_LOCKED_NAME" )
        self.type:Hide()
        self.rarity_icon:Hide()
        self.seen_installed_widget:Hide()
        self.glow:SetTintColour( 0x021A18FF )
            :SetTintAlpha( 0.4 )
        self:SetToolTipClass( nil )
        self:SetToolTip( LOC"UI.CARDCOMPENDIUM.GRAFT_LOCKED_TT" )
    else
        self:SetToolTipClass( Widget.TooltipGraft )
        self:SetToolTip( self.graft )
    end

    self:Layout()

    return self
end

function GraftsCompendiumGraft:SetProgress( unlocked, unlockable )

    self.seen_installed_widget:SetProgress( unlocked>0, unlocked>1 )
        :Show()

    self:Layout()

    return self
end

function GraftsCompendiumGraft:Layout()

    if self.is_locked then
        if self.lock_icon then
            self.lock_icon:LayoutBounds( "right", "center", self.bg )
                :Offset( -20, 0 )
        end
        self.title:LayoutBounds( "after", "center", self.graft_frame )
            :Offset( 10, 0 )
    else
        self.graft_icon:LayoutBounds( "left", "center", self.bg )
            :Offset( -1, 0 )
        self.glow:LayoutBounds( "center", "center", self.graft_icon )

        self.title:LayoutBounds( "after", "center", self.graft_icon )
            :Offset( -1, 25 )
        self.type:LayoutBounds( "left", "below", self.title )
            :Offset( 0, 4 )
        self.seen_installed_widget:LayoutBounds( "left", "below", self.type )
            :Offset( 0, -4 )
        self.rarity_icon:LayoutBounds( "right", "top", self.bg )
            :Offset( -9, -10 )
    end

    return self
end

function GraftsCompendiumGraft:UpdateImage()
    if ( self.hover or self.focus ) then
        if self.is_locked then
            AUDIO:PlayEvent( "event:/ui/card_codex/mouseover_locked_card" )
        end
        self.glow:ScaleTo( nil, 1.1, 0.25 )
        self.title:Bloom( 0.15 )
        self.bg:Bloom( 0.1 )

        self:ScaleTo( nil, 1.03, 0.2, easing.outQuad )
    else
        self.glow:ScaleTo( nil, 1, 0.25 )
        self.title:Bloom( 0.1 )
        self.bg:Bloom( 0.05 )

        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
    end
end

function GraftsCompendiumGraft:OnGainFocus()
    GraftsCompendiumGraft._base.OnGainFocus(self)
    self:UpdateImage()
end

function GraftsCompendiumGraft:OnLoseFocus()
    GraftsCompendiumGraft._base.OnLoseFocus(self)
    self:UpdateImage()
end

function GraftsCompendiumGraft:OnGainHover()
    GraftsCompendiumGraft._base.OnGainHover(self)
    self:UpdateImage()
end

function GraftsCompendiumGraft:OnLoseHover()
    GraftsCompendiumGraft._base.OnLoseHover(self)
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- Popup that shows a single graft's details

local GraftsCompendiumDetailsPopup = class( "Screen.GraftsCompendiumDetailsPopup", Widget.Screen )

local PANEL_H = 500
local PANEL_W = 1200
local CONTENT_W = 860
local PADDING = 20
local TITLES_H = 85
local CONTENT_H = PANEL_H - TITLES_H

GraftsCompendiumDetailsPopup.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self)
            self:FadeOut()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end
    },

}

function GraftsCompendiumDetailsPopup:init()
    GraftsCompendiumDetailsPopup._base.init(self)

    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )
    self.is_popup = true

    -- Background
    self.panel = self:AddChild( Widget.WidePanel( PANEL_H, UICOLOURS.GRAFT ) )
        :SetIcon( assets.graft_draft, nil, 1 )
        :PrepareAnimation()

    -- Hitbox
    self.hitbox = self:AddChild( Widget.SolidBox( PANEL_W, PANEL_H, 0xFFFF0030 ) )
        :Hide()

    -- Add contents!
    self.grafts_container = self:AddChild( Widget() )

    -- Show starting graft
    self.original_graft_container = self.grafts_container:AddChild( Widget() )
    self.original_graft_title = self.original_graft_container:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_SUBTITLE ))
        :SetText( LOC"UI.CARDCOMPENDIUM.ORIGINAL_GRAFT_TITLE" )
        :SetGlyphColour( UICOLOURS.GRAFT )
        :Bloom(0.1)
    self.original_graft_subtitle = self.original_graft_container:AddChild( Widget.Label("title", FONT_SIZE.BODY_TEXT ) )
        :SetGlyphColour( UICOLOURS.GRAFT )
        :SetAutoSize( 540 )
        :LeftAlign()
        :SetWordWrap( true )
        :SetTintAlpha( 0.8 )
        :Bloom( 0.1 )
    self.original_graft_widget = self.original_graft_container:AddChild( Widget.GraftDetails( 540, nil, true ) )
    self.seen_installed_widget = self.grafts_container:AddChild( Widget.SeenInstalledWidget() )

    -- Show upgraded graft
    self.upgraded_graft_container = self.grafts_container:AddChild( Widget() )
    self.upgraded_graft_title = self.upgraded_graft_container:AddChild( Widget.Label("title", FONT_SIZE.SCREEN_SUBTITLE ))
        :SetText( LOC"UI.CARDCOMPENDIUM.UPGRADED_GRAFT_TITLE" )
        :SetGlyphColour( UICOLOURS.MONEY )
        :Bloom(0.1)
    self.upgraded_graft_widget = self.upgraded_graft_container:AddChild( Widget.GraftDetails( 540, nil, true ) )

    -- Layout
    self.original_graft_subtitle:LayoutBounds( "left", "below", self.original_graft_title ):Offset( 0, SPACING.TITLE_DESC )
    self.original_graft_widget:LayoutBounds( "left", "below", self.original_graft_subtitle ):Offset( 0, -10 )
    self.upgraded_graft_widget:LayoutBounds( "left", "below", self.upgraded_graft_title ):Offset( 5, -10 )
    self.upgraded_graft_container:LayoutBounds( "after", "top", self.original_graft_container ):Offset( SPACING.M05, 0 )
    self.grafts_container:LayoutBounds( "center", "top", self.hitbox )
    self.seen_installed_widget:LayoutBounds("center", "bottom", self.grafts_container):Offset(0, -40)
    
    -- Back button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.button_back = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.BACK", function() self:FadeOut() end ) )
        :SetIcon( global_images.cancel )
        :LayoutBounds( "left", "above", 60, 60 )
end

function GraftsCompendiumDetailsPopup:RefreshStreamerMode()
    self.bottom_left:SetPos( 0, 0 )
    if GetStreamerMode() == 1 then
        -- Nudge the left side widgets
        self.bottom_left:SetPos( 360, 0 )
    end
end

function GraftsCompendiumDetailsPopup:ShowGraft( graft )
    self:Refresh( graft )
    self.panel:FadeIn()

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:RefreshStreamerMode()
end

function GraftsCompendiumDetailsPopup:FadeOut()
    AUDIO:PlayEvent( "event:/ui/card_codex/card_inspect_close" )
    self.panel:FadeOut( function()
        self:GetFE():PopScreen( self )
    end )
end

function GraftsCompendiumDetailsPopup:Refresh( graft_def )
    -- Update the details
    self.original_graft = GraftInstance( graft_def.id )
    self.original_graft_subtitle:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.UPGRADED_GRAFT_SUBTITLE", self.original_graft:GetXPDesc() ) )
    self.original_graft_widget:Refresh( self.original_graft )
        :SetOnClickFn( nil )

    local upgrade_id = self.original_graft.userdata.upgrade_ids and self.original_graft.userdata.upgrade_ids[1]
    if upgrade_id then
        self.upgraded_graft = GraftInstance( upgrade_id )
        self.upgraded_graft_widget:Refresh( self.upgraded_graft )
    end

    -- Layout things
    self.original_graft_subtitle:LayoutBounds( "left", "below", self.original_graft_title ):Offset( 0, SPACING.TITLE_DESC )
    self.original_graft_widget:LayoutBounds( "left", "below", self.original_graft_subtitle ):Offset( 0, -SPACING.M1 )
    
    if upgrade_id then
        self.upgraded_graft_widget:Show():LayoutBounds( nil, "top", self.original_graft_widget )
        self.upgraded_graft_title:Show():LayoutBounds( "left", "above", self.upgraded_graft_widget ):Offset( 0, SPACING.M1 )
        self.upgraded_graft_container:Show():LayoutBounds( "after", nil, self.original_graft_container ):Offset( SPACING.M05, 0 )
    else
        self.upgraded_graft_container:Hide()
        self.upgraded_graft_title:Hide()
        self.upgraded_graft_widget:Hide()
    end

    self.seen_installed_widget:SetProgress(TheGame:GetGameProfile():HasSeenGraft( graft_def.id), TheGame:GetGameProfile():HasInstalledGraft( graft_def.id ) )
    self.seen_installed_widget:LayoutBounds("center", "bottom", self.grafts_container):Offset(0, -40)

    self.grafts_container:LayoutBounds( "center", "center", self.hitbox )
    local w, h = self.grafts_container:GetSize()
    self.panel:SetHeight( h + SPACING.M1*3 )
end

function GraftsCompendiumDetailsPopup:OnControlModeChange( cm, device_id )
    GraftsCompendiumDetailsPopup._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.bottom_left:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.bottom_left:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.bottom_left:Hide()
    end
end

function GraftsCompendiumDetailsPopup:OnScreenModeChange( sm )
    self:SetLayoutScale( LAYOUT_SCALE[ sm ] )
end
