require "ui/widgets/progressbar"
local battle_defs = require "battle/battle_defs"

local line_h = 2
local border_size = 2
local bar_h = 12
local bar_l = 90
local outline_color = UICOLOURS.BLACK--0x270D0Dff
local current_health_color = UICOLOURS.HEALTH
local max_health_color = UICOLOURS.HEALTH

local BAR_LENGTH_PER_POINT = 3
local BAR_LENGTH_BASE = 25

local MIN_HEALTH = 5
local MAX_HEALTH = 200
local MIN_HEALTHBAR_WIDTH = 50
local MAX_HEALTHBAR_WIDTH = 230
local MAX_DEF_BAR_LENGTH = 50

local HealthBar = class( "Widget.HealthBar", Widget )

function HealthBar:init( fighter )
    HealthBar._base.init( self )
    
    ----------------------------------------------------------------------
    -- Get values
    self.fighter = fighter
    
    local min_health, max_health = fighter:GetStatBounds( COMBAT_STAT.HEALTH )

    self.defend_current = 0

    self.health_current = 0
    self.health_target = 0
    self.health_damage = 0
    self.defend_damage = 0

    self.health_damage_preview = 0
    self.defend_damage_preview = 0

    self.health_max = max_health or 1

    ----------------------------------------------------------------------
    -- Show HEALTH info
    self.health_root = Widget()
    self.under_bar_info = Widget()

    self.bg_border = 10
    self.bg_border_scale = 0.5
    self.bg_border_padding = 1.5
    self.bar_bg = Widget.Panel( engine.asset.Texture("UI/resolvebar_bg.tex") )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border+1, self.bg_border+1 )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :Bloom(0.15)

    self.bar_container = Widget() -- Contains bar widgets and hit anim
    self.bar_widgets = Widget() -- Contains bar widgets only

    self.current_health_number = Widget.Label("title", 22, "000")
        :SetGlyphColour(current_health_color)
        :EnableOutline(0.14)
        :SetOutlineColour(outline_color)
        :Bloom(0.1)
    self.health_number = Widget.Label("title", 22, "000")
        :SetGlyphColour(max_health_color)
        :EnableOutline(0.14)
        :SetOutlineColour(outline_color)
        :Bloom(0.1)
    self.health_icon = Widget.Image( engine.asset.Texture( "UI/fightscreen_health.tex" ), 16, 16 )
        :SetTintColour(max_health_color)
        :Bloom(0.1)
        :Hide()


    -- Bar hit animation
    local animname = "anim/UI_healthbar_hits_fx.zip"
    local anim = Widget.Anim()
        :SetHiddenBoundingBox(true)
        :SetScale( 0.23 )
    anim:AddAnims( GetAnim( animname ) )
    anim:AddOverrideFile( animname )
    self.hit_anim_bloom = 0.3
    self.hit_anim = anim
    self.hit_anim:SetBloom( self.hit_anim_bloom )


    -- Bar width scaling linearly
    --local bar_width = BAR_LENGTH_BASE + BAR_LENGTH_PER_POINT*self.health_max
    -- Bar width scaling between preset values
    local bar_width = remap( self.health_max, MIN_HEALTH, MAX_HEALTH, MIN_HEALTHBAR_WIDTH, MAX_HEALTHBAR_WIDTH )

    self.bar_line = Widget.SolidBox( bar_width, line_h, UICOLOURS.WHITE )
        :SetTintColour( UICOLOURS.WHITE )
        :SetTintAlpha( 0.2 )
        :Bloom( 0.2 )
        -- :Hide()`

    self.damage_bar = Widget.ProgressBar(bar_width, bar_h):OrientH()
        :SetBarColour(UICOLOURS.HEALTH_DAMAGE):SetOutline(0):SetBGColour(0x462424ff)

    self.health_bar = Widget.ProgressBar(bar_width, bar_h):OrientH():SetOutline(0)
        :SetBarColour(current_health_color)
        :SetBGColour(0x021A18ee):SetOutlineColour(0)

    self.damage_preview_bar = Widget.SolidBox(bar_width-4, bar_h-4, UICOLOURS.HEALTH_DAMAGE)
        :SetHiddenBoundingBox( true )
        :Bloom( 0 )
        :Hide()

    self.defend_damage_preview_bar = Widget.SolidBox(bar_width-4, bar_h-4, UICOLOURS.DEFEND_DAMAGE )
        :SetHiddenBoundingBox( true )
        :Bloom( 0 )
        :Hide()

    ----------------------------------------------------------------------
    -- Show DEFEND info
    self.defend_icon_container = Widget()
    self.defend_icon_hitbox = Widget.SolidBox( 30, 30, 0xff40ff00 )
    self.defend_icon_anim_scale = 0.04
    local animname = "anim/UI_healthbar_hits_fx.zip"
    local anim = Widget.Anim()
        :SetHiddenBoundingBox( true )
        :SetScale( self.defend_icon_anim_scale )
        :SetBloom( 0.1 )
    anim:AddAnims( GetAnim( animname ) )
    anim:AddOverrideFile( animname )
    self.defend_icon = anim
    self.defend_icon_label = Widget.Label( "title", 22 )
        :SetGlyphColour( UICOLOURS.BLACK )

    self.defend_bar = Widget.ProgressBar( BAR_LENGTH_PER_POINT, bar_h-4):OrientH()
        :SetBGColour( UICOLOURS.BLACK ):SetOutline(0)
    self.defend_bar:SetBarColour(UICOLOURS.DEFEND) 

    self.defend_icon:SetToolTip(LOC"UI.HEALTH_BAR.DEFEND_ICON_TOOLTIP")
    self.defend_bar:SetToolTip(LOC"UI.HEALTH_BAR.DEFEND_BAR_TOOLTIP")

    self.morale_bar = Widget.SolidBox(1, self.health_bar.h , UICOLOURS.MORALE, .9)

    ----------------------------------------------------------------------

    self:AddChildren{
        self.health_root:AddChildren{
            self.bar_container:AddChildren{
                self.hit_anim,
                self.bar_widgets:AddChildren{
                    self.damage_bar,
                    self.health_bar,
                    self.defend_bar:Hide(),
                    self.morale_bar,
                    self.damage_preview_bar,
                    self.defend_damage_preview_bar,
                },
            },
            self.under_bar_info:AddChildren{
                self.current_health_number,
                self.health_number,
                self.health_icon,
                self.defend_icon_container:AddChildren{
                    self.defend_icon_hitbox,
                    self.defend_icon:Play( "armor_idle", true ),
                    self.defend_icon_label:SetPos( 0, 2 ),
                }:Hide(),
            },
            self.bar_line,
            self.bar_bg,
        },
    }

    self:Layout()
end

function HealthBar:SetNoTextMode()
    -- If you want to show just the bar with no labels/icons
    self.no_text_mode = true

    self.under_bar_info:Hide()

    return self
end

function HealthBar:SetFixedSize( width, height )
    -- If you want the bar to always have the same width
    self.fixed_width = width
    self.fixed_height = height or bar_h

    self:Layout()

    return self
end

function HealthBar:GetMoraleBar()
    return self.morale_bar
end

function HealthBar:GetDefendBar()
    return self.defend_bar
end

function HealthBar:Layout()

    local health_width, used_health_width, morale_width, defend_width

    if self.fixed_width then
        -- Our bar has to fit within a specified width
        local used_health = self.health_max-self.health_current
        local total = self.health_max + self.defend_current - used_health
        BAR_LENGTH_PER_POINT = self.fixed_width / (self.health_max + self.defend_current)
        -- Calculate the size of each bar
        health_width = self.health_max * BAR_LENGTH_PER_POINT
        used_health_width = used_health * BAR_LENGTH_PER_POINT
        morale_width = (self.morale or 0) * health_width
        defend_width = math.min( self.defend_current * BAR_LENGTH_PER_POINT, MAX_DEF_BAR_LENGTH)
    else
        -- Our max_health fits within a preset scale, with the defend making the bar grow further
        local used_health = self.health_max-self.health_current
        -- Calculate how wide the health bar should be
        local health_bar_width = remap( self.health_max, MIN_HEALTH, MAX_HEALTH, MIN_HEALTHBAR_WIDTH, MAX_HEALTHBAR_WIDTH )
        BAR_LENGTH_PER_POINT = health_bar_width / self.health_max
        -- Calculate the size of each bar
        health_width = self.health_max * BAR_LENGTH_PER_POINT
        used_health_width = used_health * BAR_LENGTH_PER_POINT
        morale_width = (self.morale or 0) * health_width
        defend_width = math.min( self.defend_current * BAR_LENGTH_PER_POINT, MAX_DEF_BAR_LENGTH)
    end

    local bar_w = health_width+defend_width-used_health_width
    self.damage_bar:SetSize( bar_w, self.fixed_height or bar_h ) -- Full width
    self.health_bar:SetSize( health_width, self.fixed_height or bar_h ) -- Health width
        :SetProgress( self.health_current / self.health_max )
        :LayoutBounds( "left", "center", self.damage_bar )
    self.morale_bar:SetSize( morale_width, self.fixed_height or bar_h ) -- Ratio of the health's width, displayed over the health bar
        :LayoutBounds( "left", "center", self.health_bar )
        :SetShown( self.morale and self.morale > 0 ) -- and self.morale*self.health_max < self.health_current )
    self.defend_bar:SetSize( defend_width, self.fixed_height or bar_h ) -- Added to the current health, overlaps the used health, if any
        :LayoutBounds( "after", "center", self.health_bar.bar )
        :SetShown( self.defend_current > 0 )

    self.defend_damage_preview_bar:SetSize( self.defend_damage*BAR_LENGTH_PER_POINT, self.fixed_height or bar_h )
        :LayoutBounds( "right", "center", self.defend_bar )
    self.damage_preview_bar:SetSize( self.health_damage*BAR_LENGTH_PER_POINT, self.fixed_height or bar_h )
        :LayoutBounds( "before", "center", self.defend_bar )

    -- Highlight on the top of the bars
    self.bar_line:SetSize( bar_w, (self.fixed_height or bar_h)*0.2 )
        :LayoutBounds( "left", "top", self.health_bar )

    -- Align the bar's colored frame
    local bar_w, bar_h = self.bar_container:GetSize()
    local nineslice_w, nineslice_h = bar_w + self.bg_border_padding*2 + self.bg_border*2*self.bg_border_scale, (self.fixed_height or bar_h) + self.bg_border_padding*2 + self.bg_border*2*self.bg_border_scale
    self.bar_bg:SetSize( nineslice_w, nineslice_h )
        :LayoutBounds( "center", "center", self.bar_container )

    -- Position numbers and icons
    self.health_number:LayoutBounds( "before", "center", self.bar_container ):Offset( -6, 1 )
    self.current_health_number:LayoutBounds( "before", "center", self.health_number ):Offset( 0, 0 )
    self.health_icon:LayoutBounds( "after", "center", self.health_number ):Offset( 2, -1 )
    self.defend_icon_container:LayoutBounds( "after", "center", self.bar_container ):Offset( 4, 0 )

    if self.on_layout_fn then self.on_layout_fn() end
end

function HealthBar:OnLayout( fn )
    self.on_layout_fn = fn
end

function HealthBar:SetDefend( defend )
    self.defend_current = defend
    self:Layout()
    return self
end

function HealthBar:AnimDefend( defend, attacked )
    self.defend_current = self.defend_current + defend

    if defend > 0 then
        local out_time = 0.30
        local in_time = out_time/3

        if not self.defend_bar:IsShown() then
            self.defend_bar:Show()
            self.defend_icon:Play( "armor_in", false )
                :Push( "armor_idle", true )
            self.defend_icon_container:Show()
        else
            self.defend_icon:Play( "armor_gain", false )
                :Push( "armor_idle", true )
        end

        self.defend_icon_label:SetText( tostring( self.defend_current ) )
        self.defend_bar:SetSize( math.min(MAX_DEF_BAR_LENGTH, BAR_LENGTH_PER_POINT*self.defend_current), bar_h )
        local r,g,b,a = HexColour( UICOLOURS.MONEY )
        self.bar_bg:ColourFromCurrentTo( r, g, b, a, 0.2 )
        self:Layout()

    elseif defend < 0 then
        -- taking defend damage
        local out_time = 0.15

        self.defend_icon_label:SetText( tostring( self.defend_current ) )
        self.defend_bar:SetProgress( 1 )
        self.defend_bar:SetSize( math.min(MAX_DEF_BAR_LENGTH, BAR_LENGTH_PER_POINT*self.defend_current), bar_h )
        local r,g,b,a = HexColour( UICOLOURS.MONEY )
        self.bar_bg:ColourFromCurrentTo( r, g, b, a, 0.2 )
        self:Layout()

        if attacked then
            self:CreateFloater( defend )

            self.defend_icon:Play( "armor_hit", false )
                :Push( "armor_idle", true )

                local anim_duration = 0.1
                local bar_bloom = 0.2
                self:RunUpdater(
                    Updater.Series{
                        Updater.Do( function() self.defend_bar:SetTintAlpha(0.4) self.defend_bar:Bloom(0) end ),
                        Updater.Wait( anim_duration ),
                        Updater.Do( function() self.defend_bar:SetTintAlpha(1) self.defend_bar:Bloom(bar_bloom) end ),
                        Updater.Wait( anim_duration ),

                        Updater.Do( function() self.defend_bar:SetTintAlpha(0.4) self.defend_bar:Bloom(0) end ),
                        Updater.Wait( anim_duration ),
                        Updater.Do( function() self.defend_bar:SetTintAlpha(1) self.defend_bar:Bloom(bar_bloom) end ),
                        Updater.Wait( anim_duration ),

                        Updater.Do( function() self.defend_bar:SetTintAlpha(0.4) self.defend_bar:Bloom(0) end ),
                        Updater.Wait( anim_duration ),
                        Updater.Do( function() self.defend_bar:SetTintAlpha(1) self.defend_bar:Bloom(bar_bloom) end ),
                        Updater.Wait( anim_duration ),

                        Updater.Do( function() self.defend_bar:SetTintAlpha(0.4) self.defend_bar:Bloom(0) end ),
                        Updater.Wait( anim_duration ),
                        Updater.Do( function() self.defend_bar:SetTintAlpha(1) self.defend_bar:Bloom(bar_bloom) end ),
                        Updater.Wait( anim_duration ),

                        Updater.Do( function() self.defend_bar:SetTintAlpha(0.4) self.defend_bar:Bloom(0) end ),
                        Updater.Wait( anim_duration ),
                        Updater.Ease( function(v) self.defend_bar:SetTintAlpha(v) end, 0, 1, anim_duration*2, easing.outQuad ),
                    }
                )
        end

        if self.defend_current <= 0 then
            self.defend_bar:Hide()
            self.defend_icon:Play( attacked and "armor_break" or "armor_out", false )
                :AlphaTo( 1, out_time*3, nil, function() 
                    if self.defend_current <= 0 then
                        self.defend_icon_container:Hide()
                    else
                        self.defend_icon:Play( "armor_in", false )
                            :Push( "armor_idle", true )
                    end
                    self:Layout()
                end )

            local r,g,b,a = HexColour( UICOLOURS.SUBTITLE )
            self.bar_bg:ColourFromCurrentTo( r, g, b, a, 0.2 )
        end
    end

    self:Layout()
end

function HealthBar:AnimHealth( health, new_cap, delta_cap )

    if delta_cap and new_cap ~= self.health_max then
        self.health_max = new_cap
        local bar_width = remap( self.health_max, MIN_HEALTH, MAX_HEALTH, MIN_HEALTHBAR_WIDTH, MAX_HEALTHBAR_WIDTH )
        self.damage_bar:SetWidth(bar_width)
        self.health_bar:SetWidth(bar_width - 4)
    end

    self.health_target = clamp( health, 0, self.health_max )

    self.current_health_number:SetText( loc.format( "{1%d}", self.health_target ) )--:SetGlyphColour( current_health_color )
    self.health_number:SetText( loc.format( "/{1%d}", self.health_max ) )--:SetGlyphColour( max_health_color )

    if health < self.health_current then
        -- If being hit, show animation
        self.hit_anim:Play( "healthbar_hit_alt", false )
        local anim_duration = 0.1
        local bar_bloom = 0.2
        self:RunUpdater(
            Updater.Series{
                Updater.Do( function() self.bar_widgets:SetTintAlpha(0.4) self.bar_widgets:Bloom(0) end ),
                Updater.Wait( anim_duration ),
                Updater.Do( function() self.bar_widgets:SetTintAlpha(1) self.bar_widgets:Bloom(bar_bloom) end ),
                Updater.Wait( anim_duration ),

                Updater.Do( function() self.bar_widgets:SetTintAlpha(0.4) self.bar_widgets:Bloom(0) end ),
                Updater.Wait( anim_duration ),
                Updater.Do( function() self.bar_widgets:SetTintAlpha(1) self.bar_widgets:Bloom(bar_bloom) end ),
                Updater.Wait( anim_duration ),

                Updater.Do( function() self.bar_widgets:SetTintAlpha(0.4) self.bar_widgets:Bloom(0) end ),
                Updater.Wait( anim_duration ),
                Updater.Do( function() self.bar_widgets:SetTintAlpha(1) self.bar_widgets:Bloom(bar_bloom) end ),
                Updater.Wait( anim_duration ),

                Updater.Do( function() self.bar_widgets:SetTintAlpha(0.4) self.bar_widgets:Bloom(0) end ),
                Updater.Wait( anim_duration ),
                Updater.Do( function() self.bar_widgets:SetTintAlpha(1) self.bar_widgets:Bloom(bar_bloom) end ),
                Updater.Wait( anim_duration ),

                Updater.Do( function() self.bar_widgets:SetTintAlpha(0.4) self.bar_widgets:Bloom(0) end ),
                Updater.Wait( anim_duration ),
                Updater.Ease( function(v) self.bar_widgets:SetTintAlpha(v) end, 0, 1, anim_duration*2, easing.outQuad ),
            }
        )
    end

    self.health_bar:SetProgress( self.health_target / self.health_max )
    self.damage_bar:SetProgress( self.health_current / self.health_max )

    if not self.animating then
        self.animating = true
        self:StartUpdating()
    end

    self:Layout()
end

function HealthBar:OnUpdate( dt )
    HealthBar._base.OnUpdate( self, dt )

    local delta_health = self.health_target - self.health_current
    if delta_health ~= 0 then
        local dp
        if math.abs( delta_health ) < self.health_max / 60 then
            dp = delta_health
        else
            dp = delta_health / 30
        end

        self.health_current = self.health_current + dp
        self.damage_bar:SetProgress( self.health_current / self.health_max )
        self:RefreshHealthBar()

    else
        self.damage_bar:SetProgress( self.health_current / self.health_max )
    end

    self:Layout()

    if delta_health == 0 and self.animating then
        self.animating = false
        self:StopUpdating()
    end

end

function HealthBar:SetVals(health, morale)
    self.health_target = clamp( health, 0, self.health_max )

    self.health_current = self.health_target

    self.current_health_number:SetText( loc.format( "{1%d}", self.health_current ) )--:SetGlyphColour( current_health_color )
    self.health_number:SetText( loc.format( "/{1%d}", self.health_max ) )--:SetGlyphColour( max_health_color )

    self.damage_bar:SetProgress( self.health_current / self.health_max )
    self.health_bar:SetProgress( self.health_current / self.health_max )

    self.morale = morale or 0

    self:RefreshHealthBar()

    return self
end

function HealthBar:CalculateDamageRanges()
    local health_damage, defend_damage = 0, 0
    if self.damage_previews then
        for i, preview in ipairs( self.damage_previews ) do
            if preview.owner:IsPlayer() then
                local damage = preview.min_damage
                if damage then
                    if preview.card and preview.card:IsFlagged( battle_defs.CARD_FLAGS.PIERCING ) then
                        health_damage = health_damage + damage

                    else
                        defend_damage = defend_damage + damage
                    end
                end
            end
        end
    end

    if defend_damage > self.defend_current then
        health_damage = health_damage + (defend_damage - self.defend_current)
        defend_damage = self.defend_current
    end

    health_damage = math.min( self.health_current, health_damage )

    return health_damage, defend_damage
end

function HealthBar:RefreshHealthBar()
    local health_damage, defend_damage = self:CalculateDamageRanges()

    if health_damage > 0 then
        self.health_damage = health_damage
        local w1, y1 = self.damage_bar.bar:GetSize()
        self.damage_preview_bar:Show()
        local r1, g1, b1, a1 = HexColour( 0xFF555577 ) -- bright red
        local r2, g2, b2, a2 = HexColour( UICOLOURS.HEALTH_DAMAGE ) -- dark red
        self.damage_preview_bar:PulseColour( r1, g1, b1, a1, r2, g2, b2, a2, 0.9 )
    else
        self.health_damage = 0
        self.damage_preview_bar:StopPulse()
        self.damage_preview_bar:Hide()
    end

    if defend_damage > 0 then
        self.defend_damage = defend_damage
        self.defend_damage_preview_bar:Show()
        local r1, g1, b1, a1 = HexColour( UICOLOURS.DEFEND ) -- bright yellow
        local r2, g2, b2, a2 = HexColour( UICOLOURS.DEFEND_DAMAGE ) -- dark yellow
        self.defend_damage_preview_bar:PulseColour( r1, g1, b1, a1, r2, g2, b2, a2, 0.9 )
    else
        self.defend_damage = 0
        self.defend_damage_preview_bar:StopPulse()
        self.defend_damage_preview_bar:Hide()
    end

    self:Layout()

end

function HealthBar:SetDamagePreview( damage_previews )
    self.damage_previews = damage_previews
    self:RefreshHealthBar()
end

function HealthBar:AnimMorale( morale )
    self.morale = morale

    if self.morale > 0 then
        AUDIO:PlayEvent( "event:/sfx/battle/gen/enemy_morale_up" )
    end

    self:Layout()
end

function HealthBar:HideMorale()
    self.morale_bar:Hide()
end

function HealthBar:ShowMorale()
    self.morale_bar:Show()
end

function HealthBar:CreateFloater( number )
    local floater_icon = Widget.Image(engine.asset.Texture("images/pentagon_broken.tex"), 32, 32):SetTintColour(UICOLOURS.DEFEND)
    local floater_number = Widget.Label("title", 22, number):EnableOutline(true):SetGlyphColour( UICOLOURS.WHITE )
    local floater_root = Widget():SetHiddenBoundingBox(true)

    floater_root:AddChildren{
        floater_icon,
        floater_number:SetRegistration("center", "center"),
    }

    local x, y = self.defend_icon_container:GetPos()
    self:AddChild(floater_root):SetPos( x, y )

    -- floater_root:ScaleTo( 1.0, 1.4, 0.5, easing.outQuad )
    floater_root:MoveTo(x, y + 80 * self.fe:GetBaseWidgetScale(), 0.5, easing.outExpo)
    floater_root:AlphaTo(0, 1.0, easing.inBack, function() floater_root:Remove() end)
end
