--------------------------------------------------------------------------------

local HealthIndicator = class( "Widget.HealthIndicator", Widget )

function HealthIndicator:init( icon_size, text_size )
    HealthIndicator._base.init( self )
    self.hover_check = true
    icon_size = icon_size or 32
    text_size = text_size or 32
    self.text_spacing = 0
    self.icon = Widget.Image( global_images.health, icon_size, icon_size )
    self:AddChild( self.icon )
    self.label = Widget.Label( "body", text_size )
    self.label:EnableOutline( SMALL_TEXT_OUTLINE )
    self.label:SetGlyphColour( UICOLOURS.HEALTH )
    self:AddChild( self.label ):LayoutBounds( "after", "center" ):Offset( 6, 0 )

    self.vertical = false
end

function HealthIndicator:SetSpacing( text_spacing )
    self.text_spacing = text_spacing
    self:Layout()

    return self
end

function  HealthIndicator:Layout()
    if self.vertical then
        self.label:LayoutBounds( "center", "below", self.icon ):Offset( 0, -self.text_spacing )
    else
        self.label:LayoutBounds( "after", "center", self.icon ):Offset( 6 + self.text_spacing, 0 )
    end

    return self
end

function HealthIndicator.GetDesc()
    return LOC"UI.HEALTH_TOOLTIP"
end

function HealthIndicator:SetHealth( health, max_health )
    if max_health then
        self.label:SetText( loc.format( "{1}/{2}", math.floor(health), math.floor(max_health) ))
    else
        self.label:SetText( tostring(math.floor(health) ))
    end

    self:Layout()

    local tt = {}
    local prefix = LOC"UI.HEALTH_INDICATOR.TOOLTIP"
    table.insert(tt, prefix..HealthIndicator.GetDesc() )
    self:SetToolTip( table.concat(tt, "\n\n" )) 

end

function HealthIndicator:Refresh( agent )
    if agent == nil and TheGame:GetGameState() then
        agent = TheGame:GetGameState():GetPlayerAgent()
    end

    if agent then
        local health, max_health = agent:GetHealth()
        self:SetHealth( health, max_health )
    end
    
    return self
end

function HealthIndicator:AnimateHealth( new_health, old_health, new_max, old_max )
    local TOTAL_TIME = 0.3

    self.anim = self:RunUpdater(
        Updater.Ease( 
            function(t)
                local health = Lerp( old_health, new_health, t )
                local max_health = Lerp( old_max, new_max, t )
                self.label:SetText( loc.format( "{1%.0f}/{2%.0f}", health, max_health ))
                self:Layout()
            end, 
        0, 1.0, TOTAL_TIME, easing.outQuad)
    )
end

function HealthIndicator:SetVertical()
    self.vertical = true
    return self
end

function HealthIndicator:OnGainFocus()
    self:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
end

function HealthIndicator:OnLoseFocus()
    self:ScaleTo( nil, 1, 0.2, easing.outQuad )
end