require "ui/widget"
require "ui/widgets/panelbutton"

local assets = {
    mask = engine.asset.Texture("UI/menubutton_mask.tex"),
    gradient = engine.asset.Texture("UI/menubutton_gradient.tex"),
    outline = engine.asset.Texture("UI/menubutton_outline.tex"),
    hover = engine.asset.Texture("UI/btn_normal_hover.tex"),
    glow = engine.asset.Texture("UI/negotiation_buttonglow.tex"),
}


local IconButton = class( "Widget.IconButton", Widget.Clickable )

function IconButton:init( text, fn, w, h, text_size )
	IconButton._base.init( self )

	-- Set sizes
    self.width = w or 200
    self.height = h or 40
    self.padding = 8
    self.pacing = 0.3
    self.left = true

    -- Background
    self.bg_border = 33
    self.bg_border_scale = 0.5
    self.bg_gradient_w = 200
    self.bg_panel = self:AddChild( Widget.Panel( assets.mask ) )
        :SetTintColour( UICOLOURS.DARK_BODYTEXT )
        :SetTintAlpha( 0.7 )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
    self.bg_gradient = self:AddChild( Widget.Panel( assets.gradient ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.5 )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_gradient_w-self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
    self.bg_outline = self:AddChild( Widget.Panel( assets.outline ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :Bloom( 0.1 )

    -- Icon
    self.pre_image = self:AddChild( Widget.Image( assets.hover ) )
        :SetHiddenBoundingBox( true )
        :SetBloom( 0.03 )
    self.post_images = self:AddChild( Widget() )

	-- Text
    self.text_container = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.label = self.text_container:AddChild(Widget.Label( "button", FONT_SIZE.BUTTON ) )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetWordWrap( true )
        :OverrideLineHeight( FONT_SIZE.BUTTON*LINE_HEIGHT_TEXT_MULT*0.9 )
    -- Add second line of text
    self.subtitle = self.text_container:AddChild(Widget.Label( "body", FONT_SIZE.BUTTON * 0.7 ) )
        :SetGlyphColour( 0x6BFFD2ff )
        :SetWordWrap( true )
        :OverrideLineHeight( FONT_SIZE.BUTTON*LINE_HEIGHT_TEXT_MULT )

	self:DisableAnimateScales()
	self:SetTextSize( text_size )
    self:SetText( text )
    
    self:Layout()
    self:SetOnClickFn( fn )
end

function IconButton:SetFn( fn )
	self:SetOnClickFn( fn )
	return self
end

function IconButton:SetBgTexture( image )
    self.bg_panel:SetTexture( image )
    return self
end

function IconButton:SetOutlineTexture( image )
    self.bg_outline:SetTexture( image )
    return self
end

function IconButton:HideGradient()
    self.bg_gradient:Hide()
    return self
end

function IconButton:SetIcon( icon, colour )
    self.pre_image:SetTexture( icon )
    if colour then self.pre_image:SetTintColour( colour ) end
    self:Layout()
    return self
end

function IconButton:HideIcon()
    self.pre_image:Hide()
    self:Layout()
    return self
end

function IconButton:CenterText()
    self.center_text = true
    self:Layout()
    return self
end

function IconButton:SetCta( is_cta )
    if is_cta and not self.mask and TheGame:GetLevelOfDetail() > 1 then
        self.mask = self:AddChild( Widget() )
            :SetStencilContext( true )
            :SetHiddenBoundingBox( true )
            :SendToBack()
        self.mask_image = self.mask:AddChild( Widget.Panel( assets.mask ) )
            :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
            :SetNineSliceBorderScale( self.bg_border_scale )
            :SetMask( true )
        -- self.mask_anim = self.mask:AddChild( Widget.Image( engine.asset.Texture("battle/accelerant.tex") ) )
        local animname = "anim/UI_button_decor_fx.zip"
        self.mask_anim = self.mask:AddChild( Widget.Anim() )
            :AddAnims( GetAnim( animname ) )
            :AddOverrideFile( animname )
            :SetTintColour( UICOLOURS.TITLE )
            :SetScale( 0.9 )
            :SetBloom( 0.5 )
            :SetTintAlpha( 0.5 )
            :Play( "loop", true )
            :SetPhase( math.random() )
            :SetSpeed( 2*math.random() + 0.3 )
            :SetPos( 65, -10 )
            :SetRotation( 40*DEGREES )
            :SetStencilTest( true )
    end
    self:Layout()
    return self
end

function IconButton:SetSize( w, h )
	-- Set sizes
    self.width = w or self.width or 260
    self.height = h or self.height or 40
    self:Layout()
	return self
end

function IconButton:SetIconSize( w, h )
	-- Set sizes
    self.icon_width = w
    self.icon_height = h or w
    self:Layout()
	return self
end

function IconButton:SetTextColour( colour )
	self.text_colour = colour
	self:UpdateImage()
    return self
end

function IconButton:SetColour( colour )
	self.bg_colour = colour
	self:UpdateImage()
    return self
end

function IconButton:SetText( text, subtitle )
    self.label:SetText( text )
	-- Check if there's a subtitle
	if subtitle and subtitle ~= "" then
		self.subtitle:SetText( subtitle ):Show()
	else
		self.subtitle:SetText(""):Hide()
	end
	self:Layout()
	return self
end

function IconButton:SetTextSize(text_size)
	self.text_size = text_size or FONT_SIZE.BUTTON
    self.label:SetFontSize(self.text_size)
    if self.subtitle then self.subtitle:SetFontSize(self.text_size * 0.7) end
    self:Layout()
    return self
end

function IconButton:LeftAlign()
	self.left = true
    self:Layout()
    return self
end

function IconButton:RightAlign()
	self.left = false
    self:Layout()
    return self
end

function IconButton:Layout()
    local nineslice_w, nineslice_h = self.width - self.bg_border*2*self.bg_border_scale, self.height - self.bg_border*2*self.bg_border_scale
    self.bg_panel:SetInnerSize( nineslice_w, nineslice_h )
    self.bg_outline:SetInnerSize( nineslice_w, nineslice_h )
    self.bg_gradient:SetInnerSize( nineslice_w, nineslice_h )
    if self.mask_image then self.mask_image:SetInnerSize( nineslice_w, nineslice_h ) end
    local image_size = self.pre_image:IsShown() and self.height-self.padding*2.5 or 0
    self.pre_image:SetSize( self.icon_width or image_size, self.icon_height or image_size )
    self.label:SetAutoSize( self.width - self.padding*2 - image_size )
	self.subtitle:SetAutoSize( self.width - self.padding*2 - image_size )

    self.subtitle:LayoutBounds( "left", "below", self.label )
        :Offset( 0, 4 )
    self.pre_image:LayoutBounds( "left", "center", -self.width/2, 0 )
        :Offset( self.padding, 0 )

    self.post_images:StackChildrenRow( 4 )
    self.post_images:LayoutBounds( "right", "center", self.width/2, 0 )
        :Offset( -self.padding, 0 )
    
    self.text_container:LayoutBounds( "after", "center", self.pre_image )
        :Offset( self.padding, 1 )
    if self.center_text then
        self.text_container:LayoutBounds( "center", nil, 0, 0 ) end
	return self
end

function IconButton:GetText()
	return self.label.text
end

function IconButton:SizeToLabel( hpad )
    -- Disable text folding
    self.label:SetWordWrap( false )
    self.label:ForceSetText( self.label:GetText() or "" )
    self.subtitle:SetWordWrap( false )
    self.subtitle:ForceSetText( self.subtitle:GetText() or "" )
    -- Get text widths
    local label_w = self.label:GetSize()
    local subtitle_w = self.subtitle:GetSize()
    -- Get icon widths
    local pre_size = self.pre_image:IsShown() and self.height-self.padding*2.5 or 0
    local post_size = self.post_images:GetSize()
    -- Size the button appropriately
    local labels_w = math.max(label_w,subtitle_w) + hpad
    local total_w = labels_w + self.padding*2 + pre_size + post_size
    self:SetSize( total_w, nil )
    return self
end

-- If the label is longer than the button, increase the button's length. But don't decrease
function IconButton:IncreaseSizeToLabel( hpad )
    -- Get initial width
    local initial_w = self:GetSize()
    -- Scale to match text
    self:SizeToLabel( hpad )
    local final_w = self:GetSize()
    -- Check if we want to keep it
    if initial_w > final_w then
        self:SetSize( initial_w, nil )
    end
    return self
end

function IconButton:SetupFocusGlow()
	-- self.outerglow = self:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_button_outerglow.tex") ) )
	-- self.innerglow1 = self:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_button_innerglow.tex") ) )
 --    self.innerglow2 = self:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_button_innerglow.tex") ) )
    
 --    self.outerglow:SendToBack()

	-- self.outerglow:SetHiddenBoundingBox(true)
	-- self.innerglow1:SetHiddenBoundingBox(true)
	-- self.innerglow2:SetHiddenBoundingBox(true)
	-- self.outerglow.hover_check = false
	-- self.innerglow1.hover_check = false
	-- self.innerglow2.hover_check = false

 --    local w, h = self.w, self.h
 --    w = w + (h+10)/2 -- width of the button + half the icon width, since it's halfway outside the button bounds

 --    self.outerglow:SetSize( w+30, h+38):LayoutBounds( "right", "center", self.bg ):Offset(16,0):SetTintColour(UICOLOURS.FOCUS)
 --    self.innerglow1:SetSize( h+24, h+24):LayoutBounds( "left", "center", self.outerglow ):Offset(7,0):SetTintColour(UICOLOURS.FOCUS)
 --    self.innerglow2:SetSize( h+30, h+30):LayoutBounds( "left", "center", self.outerglow ):Offset(4,0):SetTintColour(UICOLOURS.FOCUS)

 --    self.innerglow1:RotateIndefinitely( -0.04 )
 --    self.innerglow2:RotateIndefinitely( 0.07 )

 --    self.outerglow:SetTintAlpha(0)
 --    self.innerglow1:SetTintAlpha(0)
 --    self.innerglow2:SetTintAlpha(0)

    return self
end


function IconButton:ShowReflection( height, yoffset )
    self.reflection = self:AddChild( Widget.Image( assets.glow ) )
    	:SetSize(self.w+60, height or 40)
    	:SetBloom(0.1)
    	:SetHiddenBoundingBox(true)
	    :SetTintColour(UICOLOURS.SUBTITLE)
	    :SetTintAlpha(0.5)
    	:LayoutBounds("center","below", 0, -self.height/2)
	    :Offset(0, yoffset or -20)
    return self
end

function IconButton:IsGlowing()
    return self.glowing
end

function IconButton:ShowFocusGlow()
	if self.outerglow then

    	self.glowing = true
	    self.outerglow:SetBloom(0.2):Show()
	    self.innerglow1:SetBloom(0.1):Show()
	    self.innerglow2:SetBloom(0.1):Show()
	    self.glow:SetTintColour( UICOLOURS.FOCUS ):SetTintAlpha(0)
	    self.glow_animator = self:RunUpdater(
	        Updater.Series{
	            Updater.Parallel{
	                Updater.Ease( function(v) self.outerglow:SetTintAlpha(v) end, 0.0, 1.0, self.pacing/2, easing.linear),
	                Updater.Ease( function(v) self.innerglow1:SetTintAlpha(v) end, 0.0, 0.6, self.pacing/2, easing.linear),
	                Updater.Ease( function(v) self.innerglow2:SetTintAlpha(v) end, 0.0, 0.3, self.pacing/2, easing.linear),
	                Updater.Ease( function(v) if self.reflection then self.reflection:SetTintColour( (UICOLOURS.FOCUS & 0xffffff00) | clamp( math.floor(v * 0xff), 0, 0xff ) ) end end, 0.8, 1.0, self.pacing/2, easing.linear),
	                Updater.Ease( function(v) self.icon:SetScale(v) end, 1.0, 1.05, self.pacing/2, easing.linear),
	            },
	            Updater.Parallel{
	                Updater.Ease( function(v) self.outerglow:SetTintAlpha(v) end, 1.0, 0.9, self.pacing/2, easing.linear),
	                Updater.Ease( function(v) if self.reflection then self.reflection:SetTintColour( (UICOLOURS.FOCUS & 0xffffff00) | clamp( math.floor(v * 0xff), 0, 0xff ) ) end end, 1.0, 0.8, self.pacing/2, easing.linear),
	                Updater.Ease( function(v) self.icon:SetScale(v) end, 1.05, 1.02, self.pacing/2, easing.linear),
	            },
	            Updater.Loop{
	                Updater.Wait(0.1),
	                Updater.Parallel{
	                    Updater.Ease( function(v) self.outerglow:SetTintAlpha(v) end, 0.9, 0.6, self.pacing*2, easing.linear),
	                    Updater.Ease( function(v) if self.reflection then self.reflection:SetTintColour( (UICOLOURS.FOCUS & 0xffffff00) | clamp( math.floor(v * 0xff), 0, 0xff ) ) end end, 0.8, 0.7, self.pacing*2, easing.linear),
	                    Updater.Ease( function(v) self.icon:SetScale(v) end, 1.02, 1.0, self.pacing*2, easing.linear),
	                },
	                Updater.Parallel{
	                    Updater.Ease( function(v) self.outerglow:SetTintAlpha(v) end, 0.6, 0.9, self.pacing*2, easing.linear),
	                    Updater.Ease( function(v) if self.reflection then self.reflection:SetTintColour( (UICOLOURS.FOCUS & 0xffffff00) | clamp( math.floor(v * 0xff), 0, 0xff ) ) end end, 0.7, 0.8, self.pacing*2, easing.linear),
	                    Updater.Ease( function(v) self.icon:SetScale(v) end, 1.0, 1.02, self.pacing*2, easing.linear),
	                }
	            }
	        }
	    )

	end

	return self
end

function IconButton:HideFocusGlow( animate )
	if self.outerglow then

	    self.outerglow:SetBloom(0)
	    self.innerglow1:SetBloom(0)
	    self.innerglow2:SetBloom(0)
	    self.glow:SetTintColour( UICOLOURS.SUBTITLE ):SetTintAlpha(0)

	    if self.glow_animator then
	        self.glow_animator:Stop()
	        self.glow_animator:Reset()
	        self.glow_animator = nil
	    end

        if animate then
            self.outerglow:AlphaTo( 0.0, self.pacing/2, easing.outQuad, function() self.outerglow:Hide() end )
            self.innerglow1:AlphaTo( 0.0, self.pacing/2, easing.outQuad, function() self.innerglow1:Hide() end )
            self.innerglow2:AlphaTo( 0.0, self.pacing/2, easing.outQuad, function() self.innerglow2:Hide() end )
            if self.reflection then self.reflection:ColourFromCurrentTo(0x09,0xE1,0xDC,0xcc, self.pacing/2, easing.outQuad) end
            local scale_x, scale_y = self.icon:GetScale()
            self.icon:ScaleTo( scale_x, 1.0, self.pacing/2, easing.outQuad )
        else
            self.outerglow:SetTintAlpha(0):Hide()
            self.innerglow1:SetTintAlpha(0):Hide()
            self.innerglow2:SetTintAlpha(0):Hide()
            if self.reflection then self.reflection:SetTintColour(UICOLOURS.SUBTITLE):SetTintAlpha(0.8) end
            self.icon:SetScale(1.0)
        end

    end

    return self
end

function IconButton:UpdateImage()
	self.label:SetGlyphColour( self.text_colour or UICOLOURS.WHITE )
	self.subtitle:SetGlyphColour( self.text_colour or UICOLOURS.WHITE )
	self.bg_outline:SetTintColour( self.bg_colour or UICOLOURS.SUBTITLE )
	if self.enabled and self.bg_gradient:IsShown() then
		self.bg_gradient:SetTintColour( self.bg_colour or UICOLOURS.SUBTITLE )
			:SetTintAlpha( 0.5 )
	elseif self.bg_gradient:IsShown() then
		self.bg_gradient:SetTintColour( UICOLOURS.BLACK )
			:SetTintAlpha( 0.5 )
	end
	self:SetSaturation( self.enabled and 1 or 0.2 )
	self:SetTintColour( self.enabled and UICOLOURS.WHITE or 0xbbbbbbff )
	if self.hover or self.focus then
        self:ScaleTo( nil, 1.05, 0.2, easing.outQuad )
        self.bg_outline:Bloom( 0.2 )
        if self.bg_gradient:IsShown() then self.bg_gradient:AlphaTo( 0.65, 0.1, easing.outQuad ) end
        if self.hover and self.focus then
            self.bg_outline:Bloom( 0.23 )
        end
    else
        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
        self.bg_outline:Bloom( 0.1 )
        if self.bg_gradient:IsShown() then self.bg_gradient:AlphaTo( 0.5, 0.2, easing.outQuad ) end
    end
	return self
end

function IconButton:OnGainFocus()
    IconButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function IconButton:OnLoseFocus()
    IconButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function IconButton:OnGainHover()
    IconButton._base.OnGainHover(self)
    self:UpdateImage()
end

function IconButton:OnLoseHover()
    IconButton._base.OnLoseHover(self)
    self:UpdateImage()
end

function IconButton:SetEnabled( enabled )
    if enabled then
        self:Enable()
    else
        self:Disable()
    end
    return self
end

function IconButton:Enable()
    self.enabled = true
    self:UpdateImage()
    return self
end

function IconButton:Disable()
    self.enabled = false
    self:ClearFocus()
    self:UpdateImage()
    return self
end

function IconButton:PrepareAnimation()
    self:SetTintAlpha(0)
    return self
end

function IconButton:AnimateIn()
    self:Show():AlphaTo( 1, self.pacing, easing.outQuad )
    return self
end

function IconButton:AnimateOut()
    self:AlphaTo( 0, self.pacing, easing.outQuad, function() 
		self.animating_out = nil
		self:Hide()
	end )
	self.animating_out = self.colour
    return self
end

function IconButton:IsAnimatingOut()
    return self.animating_out and self.animating_out == self.colour
end

function IconButton:HandleControlDown( control, device )
	if self:IsAnimatingOut() then
		return false
	end
	IconButton._base.HandleControlDown( self, control, device )
end