require "ui/widget"


local Image = class("Widget.Image", Widget)

function Image:init( texture_asset, w, h)
    Image._base.init(self, engine.model.PlaneModel(global_shaders.UI, texture_asset, false, w, h))
    self.model:SetDepthState(DEPTH_TEST.LESSEQUAL, false)  
    self.w, self.h = w, h

    if texture_asset then
        assert( type(texture_asset) ~= "string" )
        self.texture = texture_asset
        if not (w and h) then
            self.w, self.h = self.texture:GetSize()
        end
    end
    
    --self.blocks_mouse = true
end

function Image:OnRemoved()
    self.model:ClearTexture(0)
    self.texture = nil
end

function Image:__tostring()
    local texname = self.texture and self.texture:GetFileName() or "null"
    if self.name then
        return string.format("Image[%s: %s]", self.name, texname )
    else
        return string.format("Image[%s]", texname )
    end
end


function Image:SetBloom(b)
    self.model:SetBloom(b)
    return self
end

function Image:IsDefaultTexture()
    return self.texture ~= nil and self.texture:IsDefault()
end

function Image:GetTexture()
    return self.texture
end

function Image:SetTexture(texture_asset, sampler)
    
    sampler = sampler or 0
    
    if sampler == 0 and self.texture == texture_asset then
        return self
    end
    
    if texture_asset then
        self.model:SetTexture(sampler, texture_asset)
        self.texture = texture_asset
        if sampler == 0 then
            if self.w and self.h then
                self:SetSize(self.w, self.h)
            elseif self.texture then
                self.w, self.h = self.texture:GetSize()
            end
        end
        self.model:SetSize(self.w, self.h)
    else
        self.model:ClearTexture(sampler)
        self.texture = nil      
    end
    
    --self.model:Rebuild()
    return self
end

function Image:SetMask()
    self.special_effect = true
    self.model:SetColourWrite(false)
    self.model:SetEffect(global_shaders.UI_MASK)
    self:SetStencilWrite(STENCIL_MODES.SET)
    return self
end

function Image:SetMaskClear()
    self.special_effect = true
    self.model:SetColourWrite(false)
    self.model:SetEffect(global_shaders.UI_MASK)
    self:SetStencilWrite(STENCIL_MODES.CLEAR)
    return self
end

function Image:SetAlphaMask( mask_tex )
    self.special_effect = true
    self.model:SetEffect(global_shaders.UI_ALPHA_MASK)
    self.model:SetTexture( 1, mask_tex )
    return self
end

function Image:ApplyHSV(hsvtable)
    Image._base.ApplyHSV(self, hsvtable)

    if not self.special_effect then
        if self:HasNonIdentityHSVSettings() then
            self.model:SetEffect(global_shaders.UI_HSVADJUST)
        else
            self.model:SetEffect(global_shaders.UI)
        end
    end

end

function Image:GetBoundingBox()
    local w, h = self.w, self.h
    if not w or not h then
        return 0,0,0,0
    end
    return -w/2, -h/2, w/2, h/2
end

function Image:ResetSize()
    if self.texture then
        self:SetSize( self.texture:GetSize() )
    end
    return self
end

function Image:SetSize(dw,dh)
    if self.w ~= dw or self.h ~= dh then
        self.w = dw or self.w or 1
        self.h = dh or self.h or 1
        if self.texture then
            self.model:SetSize(self.w, self.h)
        end
        self:InvalidateBBox()
    end

    return self
end

function Image:GetSize()
    return self.w, self.h
end

function Image:SetFlip(flip)
    self.flip = flip
    self:SetScale(flip and -1 or 1, 1)
end

