require "ui/widget"
require "ui/widgets/image"

local ImageButton = class("Widget.ImageButton", Widget.Clickable)
ImageButton.can_focus = true

local text_size = 24

function ImageButton:init( img, fn, txt )
    ImageButton._base.init(self)
    self.img = self:AddChild( Widget.Image() )
    self:SetOnClickFn( fn )

    self.normal_scale = 1
    self.hover_scale = 1.05

    self:SetImage(img)
    self:SetText(txt)
    self:Bloom(0)
end

function ImageButton:Bloom( bloom )
    self.default_bloom = bloom or 0
    self.img:SetBloom( bloom )
    return self
end

function ImageButton:SetText(text)
    if text and self.label then
        self.label:SetText(text)
    elseif text then
        self.label = Widget.Label( "button", text_size, text):EnableOutline(false):SetWordWrap(true)
        self:AddChild(self.label)
    end

    return self
end

function ImageButton:SetImage(img)
    self.img:SetTexture(img)
    return self
end

function ImageButton:SetSize(w,h)
    if self.label then
        self.label:SetRegionSize(w-8,h)
    end
    self.img:SetSize(w,h)
    return self
end

function ImageButton:SetStateScales( normal_scale, hover_scale )
    self.normal_scale = normal_scale or 1
    self.hover_scale = hover_scale or 1.05
    self:UpdateImage()
    return self
end

function ImageButton:Enable()
    ImageButton._base.Enable(self)
    self:UpdateImage()
    return self
end

function ImageButton:Disable()
    ImageButton._base.Disable(self)
    self:UpdateImage()
    return self
end

function ImageButton:UpdateImage()
    if self.enabled and ( self.hover or self.focus ) then
        self:ScaleTo( nil, self.hover_scale, 0.2, easing.outQuad )
        self.img:Bloom( self.default_bloom * 1.5 )
    else
        self:ScaleTo( nil, self.normal_scale, 0.2, easing.outQuad )
        self.img:Bloom( self.default_bloom )
    end
    self:SetSaturation( self.enabled and 1 or 0.1 )
end

function ImageButton:OnGainFocus()
    ImageButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function ImageButton:OnLoseFocus()
    ImageButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function ImageButton:OnGainHover()
    ImageButton._base.OnGainHover(self)
    self:UpdateImage()
end

function ImageButton:OnLoseHover()
    ImageButton._base.OnLoseHover(self)
    self:UpdateImage()
end