require "ui/screens/dayoverscreen"

local InlineNotification = class("Widget.InlineNotification", Widget)

local POS_Y = 200
local POS_Y_STREAMER_MODE = 400



function InlineNotification:init( )
    InlineNotification._base.init(self)

    self:SetAnchors( "left", "bottom" )

    -- Button to open recent notifications
    local scale = 0.46
    self.open_button = self:AddChild( Widget.ImageButton( engine.asset.Texture( "UI/lownotification_button.tex" ) ) )
        :SetSize( 68*scale, 310*scale )
        :SetPos( 68*scale*0.5-1, 0 )
        :SetOnClickFn( function() self:ButtonClicked() end )
        :SetToolTip( LOC"NOTIFICATION.BUTTON_OPEN_TT" )
        :Hide()
    self.open_button_icon = self.open_button:AddChild( Widget.Image( engine.asset.Texture( "UI/ic_notifications_open.tex" ) ) )
        :SetSize( 22, 22 )
        :LayoutBounds( "center", "top", self.open_button.img )
        :Offset( 0, -9 )

    -- Notification widget. It shows a single notification
    self.notification_widget = self:AddChild( Widget() )
    self.bg = self.notification_widget:AddChild( Widget.Image( engine.asset.Texture( "large/lownotification_banner.tex") ) )
        :SetHiddenBoundingBox( true )
        :Bloom( 0.05 )
        :SetSize( 1100*0.6, 640*0.6 )
        :LayoutBounds( "left", "center", self.open_button )
        :Offset( 0, -10 )
    self.contents = self.notification_widget:AddChild( Widget() )
    self.text_group = self.contents:AddChild( Widget() )
        --:SetHiddenBoundingBox( true )

    self.hitbox = self.notification_widget:AddChild( Widget.SolidBox( 600, 110, 0x00FF0000 ) )
        :LayoutBounds( "left", "center", self.bg )
    self.hitbox.hover_check = true
    self.title = self.text_group:AddChild( Widget.Label( "title", 30 ) )
        :SetGlyphColour(0xcbffefff)
        :SetWordWrap(true)
        :LeftAlign()
        :SetAutoSize(400)
        :Bloom(0.05)
        :SetHoverLinkFn( UIHelpers.ItemTooltips )
    self.text = self.text_group:AddChild( Widget.Label( "body", 28 ) )
        :SetWordWrap(true)
        :LeftAlign()
        :SetAutoSize(400)
        :Bloom(0.03)
        :OverrideLineHeight( 21 )
        :LayoutBounds( "left", "below", self.title )
        :Offset( 0, 4 )
        :SetHoverLinkFn( UIHelpers.ItemTooltips )
    self.portrait = self.notification_widget:AddChild( Widget.CharacterPortrait() )
        :SetHiddenBoundingBox( true )
        :SetHeight( 110 )
        :SetPos( 90, -5 )
        :Hide()
    self.img = self.notification_widget:AddChild( Widget.Image() )
        :SetHiddenBoundingBox( true )
        :SetSize( 64, 64 )
        :SetPos( 100, 0 )
        :Hide()
    self.negotiation_card = self.notification_widget:AddChild( Widget.NegotiationCard() )
        :SetScale( 0.35 )
        :SetPos( 90, 0 )
        :Hide()
    self.battle_card = self.notification_widget:AddChild( Widget.BattleCard() )
        :SetScale( 0.35 )
        :SetPos( 90, 0 )
        :Hide()
    self.boon_bane = self.notification_widget:AddChild( Widget.RelationshipGraftIcon( 60 ) )
        :SetOpaque()
        :SetPos( 120, -40 )
        :Hide()

    -- List with the last 5 notifications
    self.past_notifications = {}
    -- Container for the pagination dots
    self.pagination = self.notification_widget:AddChild( Widget() )
        :Hide()
    local arrow_size = 38
    self.arrow_left = self.notification_widget:AddChild( Widget.ImageButton( engine.asset.Texture("UI/ic_notification_arrow_left.tex"), function() AUDIO:PlayEvent( "event:/ui/notification_tab/tab" ) self:NextPage() end ) )
        :SetSize( arrow_size, arrow_size )
        :LayoutBounds( "left", "center", self.hitbox )
        :Offset( 150, 0 )
        :Bloom( 0.03 )
        :Hide()
    self.arrow_right = self.notification_widget:AddChild( Widget.ImageButton( engine.asset.Texture("UI/ic_notification_arrow.tex"), function() AUDIO:PlayEvent( "event:/ui/notification_tab/tab" ) self:PreviousPage() end ) )
        :SetSize( arrow_size, arrow_size )
        :LayoutBounds( "right", "center", self.hitbox )
        :Bloom( 0.03 )
        :Hide()

    self.queue = {}
    self.notification_widget:Hide()

    self.open_button:SendToFront()
    self:RefreshStreamerMode()
    self.notification_x, self.notification_y = self.notification_widget:GetPos()

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
end

function InlineNotification:OnControlModeChange( cm, device_id )
    InlineNotification._base.OnControlModeChange( self, cm, device_id )
    
    if cm == CONTROL_MODE.GAMEPAD or cm == CONTROL_MODE.TOUCH then
        self:HideButton()
    else
        self:ShowButton()
    end
end

function InlineNotification:RefreshStreamerMode()
    self:SetPos( 0, ( GetStreamerMode() == 1 ) and POS_Y_STREAMER_MODE or POS_Y )
end

function InlineNotification:RefreshNotification( notification )
    local title, txt, img, boon_bane_id, data = notification.banner_txt, notification.details, notification.img, notification.boon_bane_id, notification.data
    self.current_notification = notification
    self.title:SetText( string.upper( title ) )
    self.text:SetText( txt )
    self.boon_bane:Hide()
    self.contents:Show()

    if is_instance(img, Agent) then
        self.portrait:SetCharacter( img ):Show()
        self.img:Hide()
        self.negotiation_card:Hide()
        self.battle_card:Hide()

        if boon_bane_id then
            self.boon_bane:Refresh( GraftInstance( boon_bane_id ) )
                :Show()
        end

    elseif is_instance(img, Battle.Card) then
        self.battle_card:Show()
        self.battle_card:RefreshCard( img )
        self.negotiation_card:Hide()
        self.portrait:Hide()
        self.img:Hide()

    elseif is_instance(img, Negotiation.Card) then
        self.negotiation_card:Show()
        self.negotiation_card:RefreshCard( img )
        self.battle_card:Hide()
        self.portrait:Hide()
        self.img:Hide()

    elseif (type(img) == "table" and img.icon) then 
        self.img:SetPos( 80, 0 )
        self.img:SetTexture( img.icon ):Show()
        self.portrait:Hide()
        self.negotiation_card:Hide()
        self.battle_card:Hide()

    else
        self.img:SetPos( 80, 0 )
        self.img:SetTexture( img ):Show()
        self.portrait:Hide()
        self.negotiation_card:Hide()
        self.battle_card:Hide()
    end

    self.text:LayoutBounds( "left", "below", self.title )
        :Offset( 0, 4 )
    self.text_group:LayoutBounds( "left", "center", self.hitbox )
        :Offset( 190, 0 )
    self.battle_card:LayoutBounds( nil, "center", self.hitbox )
    self.negotiation_card:LayoutBounds( nil, "center", self.hitbox )
    self.portrait:LayoutBounds( nil, "center", self.hitbox )
    self.img:LayoutBounds( nil, "center", self.hitbox )
end


function InlineNotification:SkippableDelay(t)
    self.can_skip = true
    t = t or 0
    while (t > 0) and not self.skip do 
        local dt = coroutine.yield()
        t = t - dt
    end

    self.can_skip = false
    self.skip = false
end

function InlineNotification:NotifyCoro()
    while true do
        if not self.past_notifications_open then
            if #self.queue > 0 then
                self:HideButton()
                self.arrow_left:Hide()
                self.arrow_right:Hide()
                    
                --animate in the backround
                self.notification_widget:Show()
                self.notification_widget:SetPos( self.notification_x - 40, self.notification_y )
                self.notification_widget:SetTintColour( 1, 1, 1, 0 )
                self.notification_widget:AlphaTo( 1, .3 )
                self.notification_widget:MoveTo( self.notification_x, self.notification_y, .3, easing.outQuad )
                
                while #self.queue > 0 do 

                    local notification = table.remove(self.queue, 1)
                    self:RefreshNotification( notification )

                    
                    if notification.sfx then self:PlayPositionalEvent(notification.sfx) end

                    if notification.fx then
                        local overlay = self:GetScreen()
                        if is_instance( overlay, Screen.MainOverlay ) then
                            local anim = UIHelpers.PlayAnimOnce( self.img, notification.fx.animfile, notification.fx.anim )
                            if anim and notification.fx.scale then
                                anim:SetScale( notification.fx.scale )
                            end
                            if anim and notification.fx.bloom then
                                anim:SetBloom( notification.fx.bloom )
                            end
                        end
                    end

                    local contents_x, contents_y = self.contents:GetPos()
                    self.bg:AlphaTo( 1, .3 )
                    self.contents:SetPos( contents_x-60, contents_y )
                    self.contents:MoveTo( contents_x, contents_y, .3, easing.outQuad )
                    self.contents:SetTintColour( 1, 1, 1, 0 )
                    self.contents:AlphaTo( 1, .2 )
                    self:SkippableDelay( 3 )


                    self.contents:MoveTo( contents_x+10, contents_y, .3, easing.outQuad )
                    self.contents:AlphaTo( 0, .2 )
                    
                    self:Delay( .3 )

                    self.bg:AlphaTo( 0, .3 )
                    self:Delay( .3 )
                end
                
                --animate out
                AUDIO:PlayEvent( SoundEvents.notification_slide_out )
                self.notification_widget:MoveTo( self.notification_x - 30, self.notification_y, .3, easing.inQuad )
                local speed = 1
                self.notification_widget:AlphaTo(0, .3*speed)
                self:Delay(.4*speed)
                self.done = true
                self.notification_widget:Hide()
                self:RefreshPastNotifications()
                self:ShowButton()
            else
                self.notification_widget:Hide()
                self:ShowButton()
                coroutine.yield()
            end
        end
        coroutine.yield()
        self:Delay( .5 )
    end
    self.coro = nil
end

function InlineNotification:Queue( notification )
    table.insert( self.queue, notification )
    -- Add this to the list of past notifications
    table.insert( self.past_notifications, notification )
    if #self.past_notifications > 5 then table.remove( self.past_notifications, 1 ) end
end

function InlineNotification:Present()
    if #self.queue > 0 then
    end
end

function InlineNotification:Next()
    if self.can_skip then
        self.skip = true
    end
end

function InlineNotification:Dismiss()
    --self.done = true
    --self.done = false
end

function InlineNotification:ShowButton()
    local cm = TheGame:FE():GetControlMode()
    if not (cm == CONTROL_MODE.GAMEPAD or cm == CONTROL_MODE.TOUCH) and #self.past_notifications > 0 then
        self.open_button:Show()
    end
end

function InlineNotification:HideButton()
    self.open_button:Hide()
end


function InlineNotification:OnLoseFocus()
    -- Let's close the notifications panel
    if self.past_notifications_open then
        self:ClosePanel()
    end
end

function InlineNotification:ButtonClicked()
    if self.past_notifications_open then
        -- Let's close the notifications panel
        self:ClosePanel()
    else
        -- Let's open the past notifications
        self:OpenPanel()
    end
end

function InlineNotification:ClosePanel()
    AUDIO:PlayEvent( "event:/ui/notification_tab/close" )
    self.open_button_icon:SetTexture( engine.asset.Texture( "UI/ic_notifications_open.tex" ) )
    self.notification_widget:AlphaTo( 0, .2 )
    self.contents:AlphaTo( 0, .2 )
    self.bg:AlphaTo( 0, .3, easing.linear, function() 
        self.notification_widget:Hide() 
        self.past_notifications_open = false
    end )
    self.open_button:SetToolTip( LOC"NOTIFICATION.BUTTON_OPEN_TT" )
end

function InlineNotification:OpenPanel()
    AUDIO:PlayEvent( "event:/ui/notification_tab/open" )
    self.open_button_icon:SetTexture( engine.asset.Texture( "UI/ic_notifications_close.tex" ) )
    self.past_notifications_open = true
    self:ShowPastNotification( #self.past_notifications )
    self.open_button:SetToolTip( LOC"NOTIFICATION.BUTTON_CLOSE_TT" )
end

function InlineNotification:RefreshPastNotifications()
    -- Clear any old pagination markers
    self.pagination:DestroyAllChildren()
    self.pagination:Hide()
    self.arrow_left:Hide()
    self.arrow_right:Hide()

    -- Add brand new ones
    local dot_w = 40
    self.current_notification_index = 1
    for k = #self.past_notifications, 1, -1 do
        local dot = self.pagination:AddChild( Widget.Clickable( ) )
        dot.hitbox = dot:AddChild( Widget.SolidBox( dot_w, 30, 0xFF000000 ))
        dot.image = dot:AddChild( Widget.Image( engine.asset.Texture("UI/motd_page.tex"), dot_w, 8 ) ):Bloom( 0.1 )
        dot:SetOnClickFn( function() if self.current_notification_index ~= k then AUDIO:PlayEvent( "event:/ui/notification_tab/tab" ) self:ShowPastNotification(k) else AUDIO:PlayEvent() end end )
        dot:SetTintAlpha( 0.3 )
        dot:MakeSilent()
        dot.notification_index = k
        dot:SetHoverFn( function( hover ) 
            if hover then 
                dot:ScaleTo( nil, 1.05, 0.1 )
                    :AlphaTo( 1, 0.1 )
                    :Bloom(0.2) 
            else 
                dot:ScaleTo( nil, 1, 0.1 )
                    :AlphaTo( 0.3, 0.1 )
                    :Bloom(0.1) 
            end
            if dot.selected then
                dot:AlphaTo( 1, 0.1 )
            end
        end )
    end

    if #self.past_notifications > 1 then
        self.pagination:Show()
        self.arrow_left:Show()
        self.arrow_right:Show()
    end

    -- Align them
    self.pagination:StackChildrenRow( 15 )
    self.pagination:LayoutBounds( "left", "bottom", self.open_button )
        :Offset( 170, -10 )
end

function InlineNotification:PreviousPage()
    local previous_index = self.current_notification_index - 1
    if previous_index < 1 then previous_index = #self.past_notifications end
    self:ShowPastNotification( previous_index )
end

function InlineNotification:NextPage()
    local next_index = self.current_notification_index + 1
    if next_index > #self.past_notifications then next_index = 1 end
    self:ShowPastNotification( next_index )
end

function InlineNotification:ShowPastNotification( index )
    self.current_notification_index = index
    self.notification_widget:Show()
    self.notification_widget:SetPos( self.notification_x, self.notification_y )
    self:RefreshNotification( self.past_notifications[ index ] )
    self.notification_widget:AlphaTo( 1, .2 )
    self.contents:AlphaTo( 1, .2 )
    self.bg:AlphaTo( 1, .3 )
    -- Update the pagination dots
    for k, dot in ipairs( self.pagination.children ) do
        if dot.notification_index == index then
            -- Current notification
            dot:SetTintAlpha( 1 )
            dot.selected = true
        else
            -- Not the current notification..
            dot:SetTintAlpha( 0.3 )
            dot.selected = false
        end
    end
end

function InlineNotification:IsShowingNotification()
    return self.coro and not self.done
end

function InlineNotification:OnAdded()
    self.coro = self:StartCoroutine( InlineNotification.NotifyCoro, self )
end


function InlineNotification:HandleControlDown(control)   
    if control:Has( Controls.Digital.MENU_ACCEPT ) or control:Has( Controls.Digital.MENU_ALT )  then
        self:Next()
        return true
    end
end


    
