require "ui/widget"

local graft_activate_sounds = 
{
    [GRAFT_TYPE.COMBAT] ="event:/sfx/battle/status/system/Status_Graft_Activated",
    [GRAFT_TYPE.NEGOTIATION] ="event:/sfx/battle/cards/neg/Negotiation_Graft_Activated",
    [GRAFT_TYPE.SOCIAL]  = "event:/sfx/battle/status/system/Status_SocialBoon_Activated",
}

local assets = {
    graft_slot = engine.asset.Texture ("UI/grafts_slot_common.tex"),

    graft_frame =
    {
        [CARD_RARITY.BASIC] = engine.asset.Texture("UI/grafts_slot_basic.tex"),
        [CARD_RARITY.COMMON] = engine.asset.Texture("UI/grafts_slot_common.tex"),
        [CARD_RARITY.UNCOMMON] = engine.asset.Texture("UI/grafts_slot_uncommon.tex"),
        [CARD_RARITY.RARE] = engine.asset.Texture("UI/grafts_slot_rare.tex"),
        [CARD_RARITY.UNIQUE] = engine.asset.Texture("UI/grafts_slot_unique.tex"),
        [CARD_RARITY.BOSS] = engine.asset.Texture("UI/grafts_slot_boss.tex"),
    },
    graft_frame_progress = engine.asset.Texture("UI/grafts_slot_progress.tex"),
    graft_frame_icon = engine.asset.Texture("UI/grafts_slot_icon.tex"),
    perk_earned = engine.asset.Texture( "UI/perk_bg_earned.tex"),
    perk_normal = engine.asset.Texture( "UI/perk_bg_normal.tex"),
    upgrade = engine.asset.Texture("UI/ic_upgrade_indicator.tex" ),
}

local GraftSlot = class( "Widget.GraftSlot", Widget.Clickable )

function GraftSlot:init( type, size )
    GraftSlot._base.init( self )
    self.graft_size = size or 64
    self.frame_ratio = 1
    self.graft_type = type
    self.hitbox = self:AddChild( Widget.SolidBox( self.graft_size, self.graft_size, 0xff00ff00 ) )
    self.bg = self:AddChild(Widget.Image(assets.graft_slot))
        :SetHiddenBoundingBox( true )
        :SetSize(self.graft_size*self.frame_ratio,self.graft_size*self.frame_ratio)
        :Bloom(0.2)
    self.img = self:AddChild( Widget.Image():SetSize( self.graft_size, self.graft_size ) )
        :SetHiddenBoundingBox( true )
    if self.graft_type == GRAFT_TYPE.COMBAT then
        self:SetToolTip(LOC"UI.GRAFTSLOT.EMPTY_COMBAT")
        self.bg:SetTintColour( UICOLOURS.FIGHT )
    elseif self.graft_type == GRAFT_TYPE.NEGOTIATION then
        self:SetToolTip(LOC"UI.GRAFTSLOT.EMPTY_NEGOTIATION")
        self.bg:SetTintColour( UICOLOURS.NEGOTIATION )
    end
    self.number = self:AddChild( Widget.Label("title", FONT_SIZE.ITEM_LABEL) )
        :SetHiddenBoundingBox( true )
        :SetGlyphColour(0xffffffff)
        :EnableOutline(true)
        :Hide()
end

function GraftSlot:Refresh( graft )
    if graft then
        self.graft = graft
        self:SetToolTipClass( Widget.TooltipGraft )
            :SetToolTip( self.graft )
        self.bg:SetTexture( assets.graft_frame[self.graft:GetRarity()] )
            :SetTintColour( UICOLOURS.WHITE )
        self.img:SetTexture(self.graft:GetImage())

        if graft:GetType() == GRAFT_TYPE.PERK then
            self.hitbox:SetSize(self.graft_size, self.graft_size*(280/420))
            self.bg:SetSize(self.graft_size, self.graft_size*(280/420))
                :SetTexture( assets.perk_earned )
                :SendToFront()
            self.img:SetSize(self.graft_size, self.graft_size*(280/420))
        else
            self.hitbox:SetSize(self.graft_size, self.graft_size)
            self.bg:SetSize(self.graft_size, self.graft_size)
                :SendToBack()
            self.img:SetSize(self.graft_size, self.graft_size)
        end
        if graft:GetType() == GRAFT_TYPE.MUTATOR or self.hiding_frame then
            self.bg:Hide()
        else
            self.bg:Show()
        end

        -- max_number is optional and defaults to max.huge.
        -- if it is 1, then we interpret the value of number as a binary on/off.
        local number, max_number = self.graft:GetDisplayNumber()
        if number and number <= 0 and max_number == 1 then 
            self:SetTintColour( 0.4, 0.4, 0.4, 1 )
        else
            self:SetTintColour( 1, 1, 1, 1 )
            if number ~= nil and (max_number or math.huge) ~= 1 then
                self.number:SetText( tostring(number) ):Show():LayoutBounds("right", "bottom", self.img):Offset( -2, 0 )
            else
                self.number:Hide()
            end
        end
    else
        self.bg:SetTexture( assets.graft_slot )
        if self.graft_type == GRAFT_TYPE.COMBAT then
            self:SetToolTip(LOC"UI.GRAFTSLOT.EMPTY_COMBAT")
            self.bg:SetTintColour( UICOLOURS.FIGHT )
        elseif self.graft_type == GRAFT_TYPE.NEGOTIATION then
            self:SetToolTip(LOC"UI.GRAFTSLOT.EMPTY_NEGOTIATION")
            self.bg:SetTintColour( UICOLOURS.NEGOTIATION )
        elseif self.graft_type == GRAFT_TYPE.PERK then
            self.hitbox:SetSize(self.graft_size, self.graft_size*(280/420))
            self.bg:SetSize(self.graft_size, self.graft_size*(280/420))
                :SetTexture( assets.perk_normal )
                :SendToFront()
            self.img:SetSize(self.graft_size, self.graft_size*(280/420))
        end
        self.img:SetTexture( nil )
    end
    self:RefreshProgressFrame()
    return self
end

function GraftSlot:RefreshXP()
    if self.graft and self.graft:UpgradeReady() and not self.upgrade_indicator then
        self.upgrade_indicator = self:AddChild( Widget.Image( global_images.circle ) )
            :SetTintColour( UICOLOURS.QUEST ):SetSize( 20, 20 ):Bloom( 0.08 )
            :SetHiddenBoundingBox( true )
            :LayoutBounds( "right", "bottom", self.bg )
            :Offset( 3, -5 )
        self.upgrade_indicator.icon = self.upgrade_indicator:AddChild( Widget.Image( assets.upgrade ) )
            :SetSize( 14, 14 )
    elseif self.upgrade_indicator then
        self.upgrade_indicator:Remove()
        self.upgrade_indicator = nil
    end
    return self
end

function GraftSlot:HideFrame()
    self.hiding_frame = true
    if self.graft then self.bg:Hide() end
    return self
end

function GraftSlot:SetFrameImageRatio( ratio )
    -- Ratio is 1 by default. 0.5 will make the frame 50% smaller. 1.5 will make the frame 50% larger
    self.frame_ratio = ratio or self.frame_ratio or 1
    self.bg:SetSize(self.graft_size*self.frame_ratio,self.graft_size*self.frame_ratio)
    return self
end

-- Displays a label over the icon frame with the upgrade progress for this graft
function GraftSlot:ShowProgressFrame()
    self.progress_frame = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.progress_frame.img = self.progress_frame:AddChild( Widget.Image(assets.graft_frame_progress) )
        :SetSize(self.graft_size,self.graft_size)
        :Bloom(0.05)
    self.progress_frame.icon = self.progress_frame:AddChild( Widget.Image(assets.graft_frame_icon))
        :SetSize( FONT_SIZE.SMALL_TEXT*0.6, FONT_SIZE.SMALL_TEXT*0.6 )
    self.progress_frame.label = self.progress_frame:AddChild( Widget.Label( "title", FONT_SIZE.SMALL_TEXT ) )
        :SetGlyphColour( 0x021A18ff )
        :RightAlign()
    self:RefreshProgressFrame()
    return self
end

function GraftSlot:RefreshProgressFrame()
    if self.graft then
        if self.progress_frame then
            if self.graft:HasUpgrades() then
                self.progress_frame:Show()
                self.progress_frame.img:SetTintColour( CARD_RARITY_COLOURS[ self.graft:GetRarity() or CARD_RARITY.UNIQUE ] )
                if self.graft:UpgradeReady() then
                    self.progress_frame.label:SetText( LOC"UI.GRAFTSLOT.UPGRADE_READY" )
                    self.progress_frame:RunUpdater(
                        Updater.Loop{
                            Updater.Ease( function(y) self.progress_frame:SetPos(nil, y) end, 0, -3, 0.7, easing.inOutQuad),
                            Updater.Ease( function(y) self.progress_frame:SetPos(nil, y) end, -3, 0, 0.2, easing.inOutQuad),
                        }
                    )
                else
                    local graft_xp, max_xp = self.graft:GetXP()
                    self.progress_frame.label:SetText( loc.format( LOC"UI.GRAFTSLOT.UPGRADE_PROGRESS", graft_xp, max_xp ) )
                end
                self.progress_frame.icon:LayoutBounds( "right", "bottom", self.progress_frame.img ):Offset( -self.graft_size*0.18, self.graft_size*0.18 )
                self.progress_frame.label:LayoutBounds( "before", "center", self.progress_frame.icon ):Offset( -self.graft_size*0.05, self.graft_size*0.01 )
            else
                self.progress_frame:Hide()
            end
        end
    else
        if self.progress_frame then
            self.progress_frame:Hide()
        end
    end
    return self
end

function GraftSlot:Triggered()
    local sound = graft_activate_sounds[ self.graft:GetType() ]
    if sound then
        self:PlayPositionalEvent( sound )
        self:ScaleTo( 1.3, 1, .7, easing.inQuad )
    end
    self:Refresh( self.graft )
end

function GraftSlot:UpdateImage()
    if self.enabled and ( self.selected and ( self.hover or self.focus ) ) then
        self.img:ScaleTo( nil, 1.15, 0.2, easing.outQuad )
        self.bg:ScaleTo( nil, 1.15, 0.2, easing.outQuad )
        self.bg:Bloom(0.23)
    elseif self.enabled and ( self.selected or self.hover or self.focus ) then
        self.img:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
        self.bg:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
        self.bg:Bloom(0.22)
    else
        self.img:ScaleTo( nil, 1, 0.2, easing.outQuad )
        self.bg:ScaleTo( nil, 1, 0.2, easing.outQuad )
        self.bg:Bloom(0.2)
    end
end

function GraftSlot:OnGainFocus()
    GraftSlot._base.OnGainFocus(self)
    self:UpdateImage()
end

function GraftSlot:OnLoseFocus()
    GraftSlot._base.OnLoseFocus(self)
    self:UpdateImage()
end

function GraftSlot:OnGainHover()
    GraftSlot._base.OnGainHover(self)
    self:UpdateImage()
end

function GraftSlot:OnLoseHover()
    GraftSlot._base.OnLoseHover(self)
    self:UpdateImage()
end