
--------------------------------------------------------------
-- End turn button

local LargeRoundButton = class( "Widget.LargeRoundButton", Widget.Clickable )

function LargeRoundButton:init( size )
    LargeRoundButton._base.init(self)

    self.button_size = size or 200
    self.icon_size = self.button_size*0.13
    self.pacing = 0.3
    self.hitbox = self:AddChild( Widget.SolidBox( self.button_size*0.7, self.button_size*0.7, 0x00ffff00 ) )
    self.contents = self:AddChild( Widget() )
    self.glow = self.contents:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
        :SetHiddenBoundingBox( true )
        :SetSize( self.button_size*2, self.button_size*2 )
    self.icon_hover = self.contents:AddChild( Widget.Image( engine.asset.Texture( "UI/cardspanel_endturnbtn_hover.tex") ) )
        :SetHiddenBoundingBox( true )
        :SetSize( self.button_size, self.button_size )
        :Bloom( 0.1 )

    -- ROTATING AURA
    local aura_size = self.button_size*0.61
    self.aura_layer_1 = self.contents:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer1_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5)
    self.aura_layer_2 = self.contents:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer2_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5)
    self.aura_layer_3 = self.contents:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer3_0.tex"), aura_size, aura_size )):SetHiddenBoundingBox(true):SetBloom(0.5)
    self.icon_bg = self.contents:AddChild( Widget.Image( engine.asset.Texture( "UI/cardspanel_endturnbtn_bg.tex") ) )
        :SetHiddenBoundingBox( true )
        :SetSize( self.button_size, self.button_size )
        :Bloom( 0.1 )

    -- TEXT
    -- local width_guide = self:AddChild( Widget.SolidBox( self.button_size*0.38, 3, 0x00ffff60 ) )
    self.text_content = self.contents:AddChild( Widget() )
    self.icon = self.text_content:AddChild( Widget.Image( engine.asset.Texture( "UI/gameover_stats_negotiation.tex") ) )
        :SetTintColour( UICOLOURS.TITLE )
        :SetSize( self.icon_size, self.icon_size )
        :Bloom( 0.15 )
    self.title = self.text_content:AddChild( Widget.Label( "title", self.button_size*0.1 ) )
        :OverrideLineHeight( self.button_size*0.1*0.8 )
        :SetGlyphColour( UICOLOURS.TITLE )
        :SetAutoSize( self.button_size*0.38 )
        :SetWordWrap( true )
        :Bloom( 0.1 )
    self.subtitle = self.text_content:AddChild( Widget.Label( "title", self.button_size*0.08 ) )
        :OverrideLineHeight( self.button_size*0.08*0.8 )
        :SetGlyphColour( UICOLOURS.TITLE )
        :SetAutoSize( self.button_size*0.38 )
        :SetWordWrap( true )
        :SetTintAlpha( 0.6 )
        :Bloom( 0.15 )

    self.icon_overlay = self.contents:AddChild( Widget.Image( engine.asset.Texture( "UI/cardspanel_endturnbtn_overlay.tex") ) )
        :SetHiddenBoundingBox( true )
        :SetSize( self.button_size, self.button_size )
        :Bloom( 0.1 )

    self:Refresh()
    self:Layout()
end

function LargeRoundButton:OnAdded()
    self.aura_layer_1:RotateIndefinitely( -0.01 )
    self.aura_layer_2:RotateIndefinitely( 0.03 )
    self.aura_layer_3:RotateIndefinitely( -0.025 )
end

function LargeRoundButton:SetupFocusGlow()
    self.outerglow = self:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
        :SetHiddenBoundingBox( true )
        :SetSize( self.button_size*0.7, self.button_size*0.7 )
        :SendToBack()
        :SetTintColour(UICOLOURS.FOCUS)
        :SetTintAlpha(0)
    self.innerglow1 = self.contents:AddChild( Widget.Image( engine.asset.Texture( "UI/negotiation_button_innerglow.tex") ) )
        :SetHiddenBoundingBox( true )
        :SetSize( self.button_size*0.8, self.button_size*0.8 )
        :Bloom( 0.05 )
        :SetTintColour(UICOLOURS.FOCUS)
        :RotateIndefinitely( -0.04 )
        :SetTintAlpha(0)
    self.innerglow2 = self.contents:AddChild( Widget.Image( engine.asset.Texture( "UI/negotiation_button_innerglow.tex") ) )
        :SetHiddenBoundingBox( true )
        :SetSize( self.button_size*0.8, self.button_size*0.8 )
        :Bloom( 0.05 )
        :SetTintColour(UICOLOURS.FOCUS)
        :RotateIndefinitely( 0.07 )
        :SetTintAlpha(0)
    return self
end

function LargeRoundButton:IsGlowing()
    return self.glowing
end

function LargeRoundButton:ShowFocusGlow()
    if self.outerglow then

        self.glowing = true
        self.outerglow:SetBloom(0.2):Show()
        self.innerglow1:SetBloom(0.1):Show()
        self.innerglow2:SetBloom(0.1):Show()
        self.glow:SetTintColour( UICOLOURS.FOCUS ):SetTintAlpha(0)
        self.glow_animator = self:RunUpdater(
            Updater.Series{
                Updater.Parallel{
                    Updater.Ease( function(v) self.outerglow:SetTintAlpha(v) end, 0.0, 1.0, self.pacing/2, easing.linear),
                    Updater.Ease( function(v) self.innerglow1:SetTintAlpha(v) end, 0.0, 0.6, self.pacing/2, easing.linear),
                    Updater.Ease( function(v) self.innerglow2:SetTintAlpha(v) end, 0.0, 0.3, self.pacing/2, easing.linear),
                    Updater.Ease( function(v) if self.reflection then self.reflection:SetTintColour( (UICOLOURS.FOCUS & 0xffffff00) | clamp( math.floor(v * 0xff), 0, 0xff ) ) end end, 0.8, 1.0, self.pacing/2, easing.linear),
                    Updater.Ease( function(v) self.icon:SetScale(v) end, 1.0, 1.05, self.pacing/2, easing.linear),
                },
                Updater.Parallel{
                    Updater.Ease( function(v) self.outerglow:SetTintAlpha(v) end, 1.0, 0.9, self.pacing/2, easing.linear),
                    Updater.Ease( function(v) if self.reflection then self.reflection:SetTintColour( (UICOLOURS.FOCUS & 0xffffff00) | clamp( math.floor(v * 0xff), 0, 0xff ) ) end end, 1.0, 0.8, self.pacing/2, easing.linear),
                    Updater.Ease( function(v) self.icon:SetScale(v) end, 1.05, 1.02, self.pacing/2, easing.linear),
                },
                Updater.Loop{
                    Updater.Wait(0.1),
                    Updater.Parallel{
                        Updater.Ease( function(v) self.outerglow:SetTintAlpha(v) end, 0.9, 0.6, self.pacing*2, easing.linear),
                        Updater.Ease( function(v) if self.reflection then self.reflection:SetTintColour( (UICOLOURS.FOCUS & 0xffffff00) | clamp( math.floor(v * 0xff), 0, 0xff ) ) end end, 0.8, 0.7, self.pacing*2, easing.linear),
                        Updater.Ease( function(v) self.icon:SetScale(v) end, 1.02, 1.0, self.pacing*2, easing.linear),
                    },
                    Updater.Parallel{
                        Updater.Ease( function(v) self.outerglow:SetTintAlpha(v) end, 0.6, 0.9, self.pacing*2, easing.linear),
                        Updater.Ease( function(v) if self.reflection then self.reflection:SetTintColour( (UICOLOURS.FOCUS & 0xffffff00) | clamp( math.floor(v * 0xff), 0, 0xff ) ) end end, 0.7, 0.8, self.pacing*2, easing.linear),
                        Updater.Ease( function(v) self.icon:SetScale(v) end, 1.0, 1.02, self.pacing*2, easing.linear),
                    }
                }
            }
        )

    end

    return self
end

function LargeRoundButton:HideFocusGlow( animate )
    if self.outerglow then

        self.glowing = false
        self.outerglow:SetBloom(0)
        self.innerglow1:SetBloom(0)
        self.innerglow2:SetBloom(0)
        self.glow:SetTintColour( UICOLOURS.SUBTITLE ):SetTintAlpha(0)

        if self.glow_animator then
            self.glow_animator:Stop()
            self.glow_animator:Reset()
            self.glow_animator = nil
        end

        if animate then
            self.outerglow:AlphaTo( 0.0, self.pacing/2, easing.outQuad, function() self.outerglow:Hide() end )
            self.innerglow1:AlphaTo( 0.0, self.pacing/2, easing.outQuad, function() self.innerglow1:Hide() end )
            self.innerglow2:AlphaTo( 0.0, self.pacing/2, easing.outQuad, function() self.innerglow2:Hide() end )
            if self.reflection then self.reflection:ColourFromCurrentTo(0x09,0xE1,0xDC,0xcc, self.pacing/2, easing.outQuad) end
            local scale_x, scale_y = self.icon:GetScale()
            self.icon:ScaleTo( scale_x, 1.0, self.pacing/2, easing.outQuad )
        else
            self.outerglow:SetTintAlpha(0):Hide()
            self.innerglow1:SetTintAlpha(0):Hide()
            self.innerglow2:SetTintAlpha(0):Hide()
            if self.reflection then self.reflection:SetTintColour(UICOLOURS.SUBTITLE):SetTintAlpha(0.8) end
            self.icon:SetScale(1.0)
        end
    end

    return self
end


function LargeRoundButton:Layout()
    self.contents:SetScale( 1, 1 )
        :SetPos( 0, 0 )
    self.icon_bg:SetRotation( 0*DEGREES )
    self.icon_bg:LayoutBounds( "center", "center", self.hitbox )
    local pos_x, pos_y = self.icon_bg:GetPos()
    self.icon_overlay:SetPos( pos_x, pos_y )
    self.icon_hover:SetPos( pos_x, pos_y )
    self.glow:SetPos( pos_x, pos_y )
    self.aura_layer_1:SetPos( pos_x, pos_y )
    self.aura_layer_2:SetPos( pos_x, pos_y )
    self.aura_layer_3:SetPos( pos_x, pos_y )
    self.icon:SetPos( pos_x, pos_y )
    return self
end

function LargeRoundButton:Refresh()
    self.glow:SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.2 )
    self.icon_hover:SetTintColour( UICOLOURS.SUBTITLE )
    self.icon_bg:SetTintColour( UICOLOURS.SUBTITLE )
    self.icon_overlay:SetTintColour( UICOLOURS.SUBTITLE )
    self.aura_layer_1:SetTintColour( UICOLOURS.SUBTITLE )
    self.aura_layer_2:SetTintColour( UICOLOURS.SUBTITLE )
    self.aura_layer_3:SetTintColour( UICOLOURS.SUBTITLE )
    self.title:LayoutBounds( "center", "below", self.icon )
        :Offset( 0, -2 )
    self.subtitle:LayoutBounds( "center", "below", self.title )
        :Offset( 0, 4 )
    self.text_content:LayoutBounds( "center", "center", self.hitbox )
    self:UpdateImage()
    return self
end

function LargeRoundButton:SetIcon( icon )
    self.icon:SetTexture( icon )
    return self
end

function LargeRoundButton:SetText( title, subtitle )
    self:SetName( title )
    self.title:SetText( title )
    self.subtitle:SetText( subtitle or "" )
    self:Refresh()
    return self
end

function LargeRoundButton:Enable( ... )
    LargeRoundButton._base.Enable( self, ... )
    self:UpdateImage()
    return self
end

function LargeRoundButton:Disable( ... )
    LargeRoundButton._base.Disable( self, ... )
    self.hover = false
    self.focus = false
    self:UpdateImage()
    return self
end

function LargeRoundButton:UpdateImage()
    if self.enabled then
        self.icon:SetSaturation( 1 )
        self.title:SetSaturation( 1 )
        self.subtitle:SetSaturation( 1 )
        self.icon_bg:SetTintColour( UICOLOURS.SUBTITLE )
        self.aura_layer_1:SetTintColour( UICOLOURS.SUBTITLE )
        self.aura_layer_2:SetTintColour( UICOLOURS.SUBTITLE )
        self.aura_layer_3:SetTintColour( UICOLOURS.SUBTITLE )
        if self.hover or self.focus then
            self.icon_hover:ScaleTo( nil, 1.2, 0.1 )
            self.contents:ScaleTo( nil, 1.05, 0.2 )
            self.text_content:ScaleTo( nil, 1.05, 0.2 )
            self.aura_layer_1:RotateIndefinitely( -0.015 )
            self.aura_layer_2:RotateIndefinitely( 0.05 )
            self.aura_layer_3:RotateIndefinitely( -0.04 )
            self.aura_layer_1:ScaleTo( nil, 1.19, 0.4 )
            self.aura_layer_2:ScaleTo( nil, 1.25, 0.3 )
            self.aura_layer_3:ScaleTo( nil, 1.35, 0.2 )
        else
            self.icon_hover:ScaleTo( nil, 1, 0.1 )
            self.contents:ScaleTo( nil, 1, 0.3 )
            self.text_content:ScaleTo( nil, 1, 0.3 )
            self.aura_layer_1:RotateIndefinitely( -0.01 )
            self.aura_layer_2:RotateIndefinitely( 0.03 )
            self.aura_layer_3:RotateIndefinitely( -0.025 )
            self.aura_layer_1:ScaleTo( nil, 1, 0.2 )
            self.aura_layer_2:ScaleTo( nil, 1, 0.3 )
            self.aura_layer_3:ScaleTo( nil, 1, 0.4 )
        end
    else
        self.icon:SetSaturation( 0.1 )
        self.title:SetSaturation( 0.1 )
        self.subtitle:SetSaturation( 0.1 )
        self.icon_bg:SetTintColour( 0x061F22ff )
        self.aura_layer_1:SetTintAlpha( 0 )
        self.aura_layer_2:SetTintAlpha( 0 )
        self.aura_layer_3:SetTintAlpha( 0 )
        -- If it was zoomed-in, go back to normal
        self.icon_hover:ScaleTo( nil, 1, 0.1 )
        self.contents:ScaleTo( nil, 1, 0.3 )
        self.text_content:ScaleTo( nil, 1, 0.3 )
        self.aura_layer_1:RotateIndefinitely( -0.01 )
        self.aura_layer_2:RotateIndefinitely( 0.03 )
        self.aura_layer_3:RotateIndefinitely( -0.025 )
        self.aura_layer_1:ScaleTo( nil, 1, 0.2 )
        self.aura_layer_2:ScaleTo( nil, 1, 0.3 )
        self.aura_layer_3:ScaleTo( nil, 1, 0.4 )
    end
    return self
end

function LargeRoundButton:OnGainFocus()
    LargeRoundButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function LargeRoundButton:OnLoseFocus()
    LargeRoundButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function LargeRoundButton:OnGainHover()
    LargeRoundButton._base.OnGainHover(self)
    self:UpdateImage()
end

function LargeRoundButton:OnLoseHover()
    LargeRoundButton._base.OnLoseHover(self)
    self:UpdateImage()
end

function LargeRoundButton:PrepareAnimation()
    self:UpdateImage()
    self:SetTintAlpha(0)
    return self
end

