local LazyLoadWidget = class( "Widget.LazyLoadWidget", Widget )

-- This widget displays an empty panel at a set width and height until it becomes visible, 
-- upon which it calls load_fn to display itself properly. You can also call SetVisible
-- to make it show
function LazyLoadWidget:init( w, h, load_fn )
    LazyLoadWidget._base.init( self )
    self.load_fn = load_fn
    self.w, self.h = w, h
end

function LazyLoadWidget:GetDefaultFocus()
  if self.children and self.children[1] then
      return self.children[1]
  end
  return self
end

function LazyLoadWidget:GetBoundingBox()
  return -self.w/2, -self.h/2, self.w/2, self.h/2 
end

function LazyLoadWidget:CheckForLoad()
    self.bb_x1, self.bb_y1, self.bb_x2, self.bb_y2 = self:GetWorldBoundingBox()
    local screen_w, screen_h = TheGame:FE():GetScreenDims()

    if ( ( self.bb_y1 > 0 and self.bb_y1 < screen_h )
       or ( self.bb_y2 > 0 and self.bb_y2 < screen_h ) )
       and ( ( self.bb_x1 > 0 and self.bb_x1 < screen_w )
       or ( self.bb_x2 > 0 and self.bb_x2 < screen_w ) ) then
          self:LazyLoad()
          return true
    end
end

function LazyLoadWidget:LazyLoad()
	if self.loaded then return end
	-- print( "LOADING WIDGET", self.load_fn )
	self.loaded = true
	self.load_fn( self )

  if self:HasFocus() then
    self:GetDefaultFocus():SetFocus()
  end

end
