require "ui/widget"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/clickable"
require "ui/widgets/solidbox"


local MenuButton = class( "Widget.MenuButton", Widget.Clickable )
MenuButton.can_focus = true

function MenuButton:init( text, fn, w, h, text_size)
	MenuButton._base.init( self )

	w = w or 200
	h = h or 32
	text_size= text_size or 18
	self.w = w
	self.h = h
	self.border = 1

	self.outline = self:AddChild(Widget.SolidBox( w+self.border*2, h+self.border*2):SetColour(UICOLOURS.BUTTON_TEXT):Hide())
	self.bg = self:AddChild(Widget.SolidBox( w, h):SetColour(UICOLOURS.BUTTON_BG))
	self.hover_check = true
	self.blocks_mouse = true

	self.label = self:AddChild(Widget.Label( "button", text_size))
	self.label:SetGlyphColour(UICOLOURS.BUTTON_TEXT)
	self.label:EnableOutline(false)
	self.label:SetWordWrap(true)
	self.textcolour = 0xFFFFFFFF

	if text then	
		self:SetText(text, text_size)
	end

	self:SetOnClickFn( fn )
	self:Enable()
	self:UpdateImage()
end

function MenuButton:ShowOutline(viz)
	self.outline:SetShown(viz)
	return self
end


function MenuButton:SetFont(font)
	self.label:SetFont(font)
	return self
end

function MenuButton:SetBGColour(r,g,b,a)
	self.bg:SetColour(r,g,b,a)
	return self
end

function MenuButton:ShrinkToText(height)
	self.shrink = true
	
	self.label:SetAutoSize()
	local w, h = self.label:GetRenderSize()
	self.w, self.h = w+8, height
	
	self.bg:SetSize(self.w, self.h)
	self.outline:Hide()
	
	return self
end

function MenuButton:SetSize( w, h )
	self.w, self.h = w, h
	self.outline:SetSize(w + self.border*2, h+ self.border*2)
	self.bg:SetSize(w, h)
	self.label:SetRegionSize(w-8, h)

	return self
end


function MenuButton:SetTextHoverColour(col)
	self.texthovercolour = col
	self:UpdateImage()
	return self
end

function MenuButton:SetTextColour(col)
	self.textcolour = col
	self:UpdateImage()
	return self
end

function MenuButton:SetTextSize(text_size)
	self.label:SetFontSize(text_size)
	return self
end

function MenuButton:SetText(text)
	self.label:SetText(text)
	if self.shrink then
		self:ShrinkToText()
	end
	return self
end

function MenuButton:UpdateImage()
    if self.removed then return end
	self:SetSaturation(1)
	self.label:SetPos(0,0)
	self.label:SetGlyphColour(self.textcolour)
	if not self.enabled then
		self:SetSaturation(0)
		self:SetTintColour(.5,.5,.5,1)
		self.label:SetGlyphColour(UICOLOURS.DISABLED)
	elseif self.down then
		self:SetTintColour(.7,.7,.7,1)
		self.label:SetPos(0,-2)
	elseif self.hover or self.focus then
		self:SetTintColour(1.4,1.4,1.4,1)
	else
		self:SetTintColour(1,1,1,1)
	end

	if self.enabled then
		if self.texthovercolour and (self.hover or self.focus) then
			self.label:SetGlyphColour(self.texthovercolour)
		elseif self.textcolour then
			self.label:SetGlyphColour(self.textcolour)
		end
	end	
end

function MenuButton:__tostring()
	return string.format("MenuButton(%s)", self.label and self.label.text)
end

function MenuButton:OnGainFocus()
    MenuButton._base.OnGainFocus(self)
	self:UpdateImage()
end	

function MenuButton:OnLoseFocus()
    MenuButton._base.OnLoseFocus(self)
	self.down = false
	self:UpdateImage()
end	

function MenuButton:OnGainHover()
    MenuButton._base.OnGainHover(self)
	AUDIO:PlayEvent(SoundEvents.mouseover)
	self:UpdateImage()
end

function MenuButton:OnLoseHover()
    MenuButton._base.OnLoseHover(self)
	self.down = false
	self:UpdateImage()
end
