--------------------------------------------------------------------------------
-- Pretty basic.  Mettle icon to the left of a label with the amount.

local MettleIndicator = class( "Widget.MettleIndicator", Widget )

function MettleIndicator:init( icon_size, text_size, outline )
    MettleIndicator._base.init( self )
    self.hover_check = true
    self.icon_size = icon_size or 32
    self.text_size = text_size or 32
    self.outline = outline or false
    self.text_spacing = 0
    self.icon = Widget.Image( engine.asset.Texture("UI/ic_mettle.tex"), self.icon_size, self.icon_size )
    self:AddChild( self.icon )

    self.price_container = self:AddChild( Widget() )
    self.price_label = self.price_container:AddChild( Widget.Label( "body", self.text_size ) )
        :SetGlyphColour( UICOLOURS.METTLE )

    if outline then 
        self.price_label:EnableOutline( SMALL_TEXT_OUTLINE )
    end

    self.vertical = false
end

function MettleIndicator:SetMettle( mettle )
    return self:FormatMettle( loc.format( "{1#thousands}", mettle or 0 ))
end

function MettleIndicator:SetSpacing( text_spacing )
    self.text_spacing = text_spacing
    self:Layout()

    return self
end

function  MettleIndicator:Layout()
    if self.vertical then
        self.price_container:LayoutBounds( "center", "below", self.icon ):Offset( 0, -self.text_spacing )
    else
        self.price_container:LayoutBounds( "after", "center", self.icon ):Offset( 6 + self.text_spacing, self.text_size*0.05 )
    end

    return self
end

function MettleIndicator:FormatMettle( str )
    self.price_label:SetText( str )
    self:Layout()
    return self
end

function MettleIndicator:Refresh()
    self.player = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent()
    if self.player then
        self.current_mettle = TheGame:GetGameProfile():GetMettlePoints( self.player.id )
        self:SetMettle( self.current_mettle )
        self:SetToolTip( loc.format( LOC"UI.METTLE_INDICATOR.TOOLTIP", self.player:LocTable() ) )
        self:Show()
    else
        self:SetToolTip()
        self:Hide()
    end

    return self
end

function MettleIndicator:SetVertical()
    self.vertical = true
    return self
end

function MettleIndicator:OnGainFocus()
    self:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
end

function MettleIndicator:OnLoseFocus()
    self:ScaleTo( nil, 1, 0.2, easing.outQuad )
end

function MettleIndicator:HasMettle()
    return self.current_mettle and self.current_mettle > 0
end

-- Checks if this indicator should be shown yet to this player
function MettleIndicator:ShouldBeShown()
    return self.player and TheGame:GetGameProfile():HasMettleUnlocked( self.player.id )
end