--------------------------------------------------------------------------------
-- Pretty basic.  Money icon to the left of a label with monetary amount.

local MoneyIndicator = class( "Widget.MoneyIndicator", Widget )

function MoneyIndicator:init( icon_size, text_size, outline )
    MoneyIndicator._base.init( self )
    self.hover_check = true
    self.icon_size = icon_size or 32
    self.text_size = text_size or 32
    self.outline = outline or false
    self.text_spacing = 0
    self.icon = Widget.Image(global_images.money, self.icon_size, self.icon_size )
    self:AddChild( self.icon )

    self.price_container = self:AddChild( Widget() )
    self.price_label = self.price_container:AddChild( Widget.Label( "body", self.text_size ) )
        :SetGlyphColour( UICOLOURS.MONEY )
    self.old_price_label = self.price_container:AddChild( Widget.Label( "body", self.text_size ) )
        :SetGlyphColour( UICOLOURS.MONEY_DESATURATED )
    self.old_price_strikethrough = self.price_container:AddChild( Widget.Panel( engine.asset.Texture("UI/moneyindicator_strikethrough.tex") ) )
        :SetTintColour( UICOLOURS.MONEY_DESATURATED )

    if outline then 
        self.price_label:EnableOutline( SMALL_TEXT_OUTLINE )
        self.old_price_label:EnableOutline( SMALL_TEXT_OUTLINE )
    end

    self:SetToolTip(LOC"UI.MONEY_INDICATOR.TOOLTIP")

    self.vertical = false
end

function MoneyIndicator:SetMoney( money )
    return self:FormatMoney( loc.format( "<#MONEY>{1#thousands}</>", money or 0 ))
end

function MoneyIndicator:SetSpacing( text_spacing )
    self.text_spacing = text_spacing
    self:Layout()

    return self
end

function  MoneyIndicator:Layout()
    if self.old_price_label:IsShown() then
        self.old_price_label:LayoutBounds( "before", "center", self.price_label ):Offset( -self.text_spacing*1.4, 0 )
        local old_p_w, old_p_h = self.old_price_label:GetSize()
        old_p_w = old_p_w + 11
        self.old_price_strikethrough:SetSize( old_p_w, 3 )
            :SetRotation( 7*DEGREES )
            :LayoutBounds( "center", "center", self.old_price_label )
            :Offset( 0, -7 )
    else
        self.old_price_label:LayoutBounds( "center", "center", self.price_label )
        self.old_price_strikethrough:LayoutBounds( "center", "center", self.price_label )
            :SetRotation( 0 )
    end

    if self.vertical then
        self.price_container:LayoutBounds( "center", "below", self.icon ):Offset( 0, -self.text_spacing )
    else
        self.price_container:LayoutBounds( "after", "center", self.icon ):Offset( 6 + self.text_spacing, self.text_size*0.05 )
    end

    return self
end

function MoneyIndicator:FormatMoney( str, old_price_str )
    self.price_label:SetText( str )
    if old_price_str then
        self.old_price_strikethrough:Show()
        self.old_price_label:Show():SetText( old_price_str )
    else
        self.old_price_strikethrough:Hide()
        self.old_price_label:Hide()
    end
    
    self:Layout()

    return self
end

function MoneyIndicator:Refresh()
    local player = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent()
    if player then
        local money = player:GetMoney()
        self:SetMoney( money )
        self:Show()
    else
        self:Hide()
    end

    return self
end

function MoneyIndicator:SetVertical()
    self.vertical = true
    return self
end

function MoneyIndicator:OnGainFocus()
    self:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
end

function MoneyIndicator:OnLoseFocus()
    self:ScaleTo( nil, 1, 0.2, easing.outQuad )
end