require "ui/widget"

local BUFFERED_FRAMES = 4 -- How many frames are bufferd before movie playback begins.

local MoviePlayer = class("Widget.MoviePlayer", Widget)

function MoviePlayer:init( filename, w, h, include_audio )
    local bus
    if include_audio == true and AUDIO:IsEnabled() then
        bus = AUDIO:GetBus( "bus:/Master/sfx/movies" )
    end

	MoviePlayer._base.init(self, engine.model.MovieModel( w, h, bus, BUFFERED_FRAMES ))
	if filename then
		if not self.model:LoadMovie( filename ) then
			LOGWARN( "Could not find movie: %s", filename )
		end
		self.filename = filename
	end
end

function MoviePlayer:OnRemoved()
	self:StopMovie()
end

function MoviePlayer:SetSize( w, h )
	self.model:SetSize( w, h )
end

function MoviePlayer:PlayMovie( filename, loop )
	if filename and filename ~= self.filename then
		if not self.model:LoadMovie( filename ) then
			LOGWARN( "Could not find movie: %s", filename )
		else
			print( "MoviePlayer:PlayMovie:", filename, loop )
		end
		self.filename = filename
	end

	self.model:PlayMovie(loop)
end

function MoviePlayer:GetDuration()
	return self.model:GetDuration()
end

function MoviePlayer:StopMovie()
	self.model:StopMovie()
end

function MoviePlayer:PauseMovie()
	self.model:PauseMovie()
end

function MoviePlayer:IsDone()
	return self.model:IsDone()
end

function MoviePlayer:IsPlaying()
	return self.model:IsPlaying()
end

function MoviePlayer:IsPaused()
	return self.model:IsPaused()
end


function MoviePlayer:SetBlendTexture(texture)
	self.model:SetBlendTexture(texture)
    self.model:SetBlendMode(BLEND_MODE.BLENDED)
end

function MoviePlayer:SetBlendFactors(factor, threshold)
	self.model:SetBlendFactorAndThreshold(factor, threshold)
end

