require "ui/widget"

---------------------------------------------------------------------------------
-- Manages the UI for Smith's Moxie

local MoxieWidget = class( "Widget.MoxieWidget", Widget )

function MoxieWidget:init( fighter )
    MoxieWidget._base.init( self )

    self.fighter = fighter
    self.h = 50
    self.w = self.h*1.5

    self.hitbox = self:AddChild( Widget.SolidBox( self.w, self.h, 0xFF00FF00 ) )

    self.contents = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.bg_border = 25
    self.bg_border_scale = 0.5
    self.bg_width = self.h*0.9
    self.bg_height = self.h*0.5
    local nineslice_w, nineslice_h = self.bg_width + self.bg_border*2*self.bg_border_scale, self.bg_height + self.bg_border*2*self.bg_border_scale
    self.bg = self.contents:AddChild( Widget.Panel( engine.asset.Texture("UI/condition_moxie_bg.tex") ) ) -- Panel behind the value
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :SetSize( nineslice_w, nineslice_h )
        :SetTintAlpha( 0 )
        :Bloom( 0.1 )
    self.icon = self.contents:AddChild( Widget.Image( engine.asset.Texture("UI/condition_moxie.tex" ) ) ) -- Animated moxie icon
        :SetSize( self.h, self.h )
        :Bloom( 0.15 )
    self.glow = self.contents:AddChild( Widget.Image( engine.asset.Texture("UI/condition_moxie_glow.tex" ) ) ) -- Glow that rotates over the icon
        :SetSize( self.h, self.h )
        :SetTintAlpha( 0 )
        :RotateIndefinitely( 0.2 )
        :Bloom( 0.2 )
    self.label = self.contents:AddChild( Widget.Label( "body", 24, "" ) )
        :SetGlyphColour( 0xB6D940ff )
        :SetTintAlpha( 0 )
        :Bloom( 0.1 )

    -- Layout
    self.bg:LayoutBounds( "center", "center", self.icon )
    self.contents:LayoutBounds( "center", nil, self.hitbox )
        :Offset( 0, 0 )
end

function MoxieWidget:SetFighter( fighter )
    self.fighter = fighter
    return self
end

function MoxieWidget:Refresh()
    if not self.fighter then return end

    if not self.condition then
        -- First time updating the widget. Get all the values we need
        self.condition = self.fighter:GetCondition( "smith_regeneration" )
        self.max_moxie = self.condition and self.condition.max_heal
        -- Show the value
        self:ShowLabel( self.max_moxie )
    elseif self.condition.max_heal > self.max_moxie then
        -- Value increased
        self:ShowIncreaseValue( self.max_moxie, self.condition.max_heal )
        self.max_moxie = self.condition.max_heal
    elseif self.condition.max_heal < self.max_moxie then
        -- Value decreased
        self:ShowDecreaseValue( self.max_moxie, self.condition.max_heal )
        self.max_moxie = self.condition.max_heal
    end

    self:SetToolTipClass( Widget.TooltipMoxie )
    self:SetToolTip( { condition = self.condition, fighter = self.fighter } )

    return self
end

function MoxieWidget:ShowLabel( value )
    -- Get starting positions
    local contents_x1, contents_y1 = self.contents:GetPos()
    local bg_x1, bg_y1 = self.bg:GetPos()
    -- Do final layout
    self.bg:LayoutBounds( "after", "center", self.icon )
        :Offset( -self.w*0.2, 0 )
    self.label:LayoutBounds( "center", "center", self.bg )
        :Offset( self.w*0.03, 1 )
    self.contents:LayoutBounds( "center", nil, self.hitbox )
        :Offset( -4, 0 )
    -- Get final positions
    local bg_x2, bg_y2 = self.bg:GetPos()
    local contents_x2, contents_y2 = self.contents:GetPos()
    -- Reset to starting positions
    self.bg:SetPos( bg_x1, bg_y1 )
    self.contents:SetPos( contents_x1, contents_y1 )
    -- Animate in
    self:RunUpdater(
        Updater.Parallel{
            Updater.Series{
                Updater.Ease( function(v) self.icon:SetScale(v) self.glow:SetScale(v) end, 1, 1.2, 0.3, easing.linear ),
                Updater.Ease( function(v) self.icon:SetScale(v) self.glow:SetScale(v) end, 1.2, 1, 0.3, easing.linear ),
            },
            Updater.Series{
                Updater.Do( function() self.glow:Bloom( 0.4 ) end ),
                Updater.Ease( function(v) self.glow:SetTintAlpha(v) end, 0, 1, 0.25, easing.linear ),
                Updater.Parallel{
                    Updater.Ease( function(v) self.bg:SetPos(v,bg_y2) end, bg_x1, bg_x2, 0.1, easing.outQuad ),
                    Updater.Ease( function(v) self.bg:SetTintAlpha(v) end, 0, 1, 0.1, easing.linear ),
                    Updater.Ease( function(v) self.contents:SetPos(v,contents_y2) end, contents_x1, contents_x2, 0.1, easing.outQuad ),
                },
                Updater.Parallel{
                    Updater.Do( function() self.label:SetText( value ) end ),
                    Updater.Ease( function(v) self.label:SetTintAlpha(v) end, 0, 1, 0.1, easing.linear ),
                    Updater.Ease( function(v) self.label:SetScale(v) end, 1.3, 1, 0.1, easing.linear ),
                },
                Updater.Ease( function(v) self.glow:SetTintAlpha(v) end, 1, 0, 0.25, easing.linear ),
            }
        }
    )
end

function MoxieWidget:ShowPickValue( result )

end

function MoxieWidget:ShowIncreaseValue( from, to )
    -- Show animation
    self:RunUpdater(
        Updater.Parallel{
            Updater.Series{
                Updater.Ease( function(v) self.icon:SetScale(v) self.glow:SetScale(v) end, 1, 1.2, 0.3, easing.linear ),
                Updater.Ease( function(v) self.icon:SetScale(v) self.glow:SetScale(v) end, 1.2, 1, 0.3, easing.linear ),
            },
            Updater.Series{
                Updater.Do( function() self.glow:Bloom( 0.4 ) end ),
                Updater.Ease( function(v) self.glow:SetTintAlpha(v) end, 0, 1, 0.25, easing.linear ),
                Updater.Parallel{
                    Updater.Ease( function(v) self.label:SetText( math.floor(v) ) end, from, to, 0.3, easing.outQuad ),
                    Updater.Ease( function(v) self.label:SetScale(v) end, 1, 1.3, 0.3, easing.linear ),
                },
                Updater.Parallel{
                    Updater.Ease( function(v) self.label:SetScale(v) end, 1.3, 1, 0.1, easing.linear ),
                    Updater.Ease( function(v) self.glow:SetTintAlpha(v) end, 1, 0, 0.25, easing.linear ),
                }
            }
        }
    )
end

function MoxieWidget:ShowDecreaseValue( from, to )
    -- Show animation
    self:RunUpdater(
        Updater.Parallel{
            Updater.Series{
                Updater.Ease( function(v) self.icon:SetScale(v) self.glow:SetScale(v) end, 1, 1.2, 0.3, easing.linear ),
                Updater.Ease( function(v) self.icon:SetScale(v) self.glow:SetScale(v) end, 1.2, 1, 0.3, easing.linear ),
            },
            Updater.Series{
                Updater.Do( function() self.glow:Bloom( 0.4 ) end ),
                Updater.Ease( function(v) self.glow:SetTintAlpha(v) end, 0, 1, 0.25, easing.linear ),
                Updater.Parallel{
                    Updater.Ease( function(v) self.label:SetText( math.floor(v) ) end, from, to, 0.3, easing.outQuad ),
                    Updater.Ease( function(v) self.label:SetScale(v) end, 1, 1.3, 0.3, easing.linear ),
                },
                Updater.Parallel{
                    Updater.Ease( function(v) self.label:SetScale(v) end, 1.3, 1, 0.1, easing.linear ),
                    Updater.Ease( function(v) self.glow:SetTintAlpha(v) end, 1, 0, 0.25, easing.linear ),
                }
            }
        }
    )
end

---------------------------------------------------------------------------------
-- A tooltip for the moxie widget

local panel_w = 412

local TooltipMoxie = class("Widget.TooltipMoxie", Widget)

function TooltipMoxie:init()
    TooltipMoxie._base.init(self)
    self.bg = self:AddChild(Widget.Panel(global_images.tooltipbg))
    self.bg:SetNineSliceBorderScale( 0.5 )

    self.contents = self:AddChild(Widget())

    self.title = self.contents:AddChild( Widget.Label( "title", 26 ) )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom(0.1)
    self.subheader = self.contents:AddChild( Widget() )
    self.moxie_widget = self.subheader:AddChild( Widget.MoxieWidget() )
        :DisableToolTip( true )
    local w, h = self.moxie_widget:GetSize()
    self.desc_1 = self.subheader:AddChild( Widget.Label( "tooltip", 22 ) )
        :SetAutoSize( panel_w - w - SPACING.M1/2 )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    self.moxie_label = self.contents:AddChild( Widget.Label( "body", 26 ) )
        :SetGlyphColour( UICOLOURS.FOCUS )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom(0.15)
    self.desc_2 = self.contents:AddChild( Widget.Label( "tooltip", 22 ) )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

    self:Hide()
end

function TooltipMoxie:DoLayout( data )
    self.fighter = data.fighter
    self.condition = data.condition

    local moxie_widget_x_offset = 4

    self.title:SetText( LOC"SMITH.MOXIE_TITLE" )
    self.moxie_widget:SetFighter( self.fighter )
        :Refresh()
        :LayoutBounds( "left", "below", self.title )
        :Offset( moxie_widget_x_offset, -SPACING.M1/2 )
        :Bloom( 0.05 )
    self.desc_1:SetText( loc.format( LOC"SMITH.MOXIE_DESC_1", self.condition.passive_gain ) )
        :LayoutBounds( "after", "top", self.moxie_widget )
        :Offset( SPACING.M1/2-moxie_widget_x_offset, 6 )
    self.moxie_label:SetText( loc.format( LOC"SMITH.MOXIE_LABEL", self.condition.max_heal ) )
        :LayoutBounds( "left", "below", self.subheader )
        :Offset( -moxie_widget_x_offset, -SPACING.M1/2 )
    self.desc_2:SetText( loc.format( LOC"SMITH.MOXIE_DESC_2", self.condition.base_max ) )
        :LayoutBounds( "left", "below", self.moxie_label )
        :Offset( 0, 0 )

    self.contents:SetRegistration("center", "center")
    self.bg:SizeToWidgets( SPACING.M1/2, self.contents )
end


---------------------------------------------------------------------------------
-- Displays a widget on the Fighter Info Popup explaining Smith's Moxie

local MoxieInfoWidget = class( "Widget.MoxieInfoWidget", Widget )

function MoxieInfoWidget:init( fighter, condition, width )
    MoxieInfoWidget._base.init( self )

    self.fighter = fighter
    self.condition = condition
    self.width = width or 300
    local content_w = self.width - SPACING.M1*2

    self.background = self:AddChild( Widget.Panel( engine.asset.Texture("UI/left_border_frame_faint.tex" ) ) )
        :SetTintColour( UICOLOURS.FOCUS )
        :SetBloom( 0.15 )
        :SetSize( 300, 300 )

    self.contents = self:AddChild(Widget())

    self.title = self.contents:AddChild( Widget.Label( "title", 26 ) )
        :SetGlyphColour( UICOLOURS.FOCUS )
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom(0.1)
    self.subheader = self.contents:AddChild( Widget() )
    self.moxie_widget = self.subheader:AddChild( Widget.MoxieWidget( self.fighter ) )
        :DisableToolTip( true )
        :Refresh()
    local w, h = self.moxie_widget:GetSize()
    self.desc_1 = self.subheader:AddChild( Widget.Label( "tooltip", 22 ) )
        :SetAutoSize( content_w - w - SPACING.M1/2 )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    self.moxie_label = self.contents:AddChild( Widget.Label( "body", 26 ) )
        :SetGlyphColour( UICOLOURS.FOCUS )
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom(0.15)
    self.desc_2 = self.contents:AddChild( Widget.Label( "tooltip", 22 ) )
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

end

function MoxieInfoWidget:Layout()

    local moxie_widget_x_offset = 4

    self.title:SetText( LOC"SMITH.MOXIE_TITLE" )
    self.moxie_widget:LayoutBounds( "left", "below", self.title )
        :Offset( moxie_widget_x_offset, -SPACING.M1/2 )
        :Bloom( 0.05 )
    self.desc_1:SetText( loc.format( LOC"SMITH.MOXIE_DESC_1", self.condition.passive_gain ) )
        :LayoutBounds( "after", "top", self.moxie_widget )
        :Offset( SPACING.M1/2-moxie_widget_x_offset, 6 )
    self.moxie_label:SetText( loc.format( LOC"SMITH.MOXIE_LABEL", self.condition.max_heal ) )
        :LayoutBounds( "left", "below", self.subheader )
        :Offset( -moxie_widget_x_offset, -SPACING.M1/2 )
    self.desc_2:SetText( loc.format( LOC"SMITH.MOXIE_DESC_2", self.condition.base_max ) )
        :LayoutBounds( "left", "below", self.moxie_label )
        :Offset( 0, 0 )

    local w, h = self.contents:GetSize()
    self.background:SetSize( self.width, h + SPACING.M1 )
    self.contents:LayoutBounds( "center", "center", self.background )
    return self
end

---------------------------------------------------------------------------------
-- Displays a custom health bar for Smith, which displays his moxie healing

local SmithHealthBar = class( "Widget.SmithHealthBar", Widget.HealthBar )

function SmithHealthBar:init( fighter )
    SmithHealthBar._base.init( self, fighter )

    -- self.bar_bg:SetTintColour( UICOLOURS.FOCUS )

end