local CIRCLE_SIZE = 16
local EDGE_WIDTH = 6




local NavPath = class("Widget.NavPath", Widget)

function NavPath:init(graph)
    NavPath._base.init(self)
    self.graph = graph or Graph()

    self.edge_root = self:AddChild(Widget())
    
    self.closest_point = self:AddChild(Widget.Image(global_images.circle):SetSize(CIRCLE_SIZE*.5, CIRCLE_SIZE*.5):SetTintColour(0xff00ffff))

    self.node_root = self:AddChild(Widget())
    self.widgets_elements = {}
    self.elements_widgets = {}
    self:Refresh()
    self:StartUpdating()
end



function NavPath:GetBoundingBox()
    if self.bboxcache.valid then
        return self.bboxcache.x1, self.bboxcache.y1, self.bboxcache.x2, self.bboxcache.y2 
    else
        local x1 = 10000000
        local x2 = -10000000
        local y1 = 10000000
        local y2 = -10000000

        for k,node in pairs(self.graph.nodes) do
            x1 = math.min(x1, node.x)
            x2 = math.max(x2, node.x)
            y1 = math.min(y1, node.y)
            y2 = math.max(y2, node.y)
        end

        self.bboxcache.valid = true
        self.bboxcache.x1, self.bboxcache.y1, self.bboxcache.x2, self.bboxcache.y2 = x1,y1,x2,y2    
        return x1,y1,x2,y2
    end
end




function NavPath:SetSubSelection(x,y)

    local selected
    if self.node_root.children then
        for k,v in pairs(self.node_root.children) do
            if v:CheckHit(x,y) then
                selected = v
                break
            end
        end
    end

    if self.edge_root.children and not selected then
        for k,v in pairs(self.edge_root.children) do
            if v:CheckHit(x,y) then
                selected = v
                break
            end
        end
    end

    if selected then
        selected:SendToFront()
    end

    self:SelectSubWidget(selected)
end

function NavPath:SelectSubWidget(selected)
    if self.selected ~= selected then
        if self.selected then
            self.selected:SetTintColour(0xffffffff)
        end
    end
    self.selected = selected

    if self.selected then
        self.selected:SetTintColour(0xffff00ff)
    end

end

function NavPath:GetSelectedEdge()
    local el = self.selected and self.widgets_elements[self.selected]
    if el and self.graph:IsEdge(el) then
        return el
    end
end

function NavPath:GetSelectedNode()
    local el = self.selected and self.widgets_elements[self.selected]
    if el and self.graph:IsNode(el) then
        return el
    end
end


function NavPath:GetSubWidgetPos()
    local node = self:GetSelectedNode()
    if node then
        return node.x, node.y
    end
end

function NavPath:ExtendNode(node)
    local new_node = self.graph:ExtendNode(node)
    self:Refresh()
    self:SelectSubWidget(self.elements_widgets[new_node])
end

function NavPath:RemoveEdge(edge)
    self.graph:RemoveEdge(edge)
    self:Refresh()
end

function NavPath:RemoveNode(node)
    self.graph:RemoveNode(node)
    self:Refresh()
end

function NavPath:Weld(node, other_node)
    self.graph:WeldNodes(node, other_node)
    self:Refresh()
end

function NavPath:CanWeldNode(node)
    return self.graph:CanWeldNode(node)
end

function NavPath:BisectEdge(edge)
    local new_node = self.graph:BisectEdge(edge)
    self:Refresh()
    self:SelectSubWidget(self.elements_widgets[new_node])
end


function NavPath:SetSubWidgetPos(x,y)
    local node = self:GetSelectedNode()
    if node then
        local oldx, oldy = self.selected:TransformToParent(0,0)
        local newx, newy = self:TransformFromParent(x or oldx, y or oldy)
        node.x, node.y = newx, newy
        self:Refresh()
        return true
    end
end

function NavPath:CheckHit(x,y)
    if self.node_root.children then
        for k,v in pairs(self.node_root.children) do
            if v:CheckHit(x,y) then
                return true
            end
        end
    end

    if self.edge_root.children then
        for k,v in pairs(self.edge_root.children) do
            if v:CheckHit(x,y) then
                return true
            end
        end
    end

    return false
end

local function UpdateNode(widget, node)
    widget:SetPos(node.x, node.y)
end

local function UpdateEdge(widget, edge)
    local x1, y1 = edge.node1.x, edge.node1.y
    local x2, y2 = edge.node2.x, edge.node2.y

    local dx = x2 - x1
    local dy = y2 - y1
    local distsq = dx*dx+dy*dy

    if distsq > 0 then
        widget:Show()
        local dist = math.sqrt(distsq)
        local r = math.atan2(dy, dx)
        
        widget:SetPos( (x1+x2)/2, (y1+y2)/2 ):SetSize(dist, EDGE_WIDTH):SetRotation(r)
    
        if edge.userdata and edge.userdata.tag_list and edge.userdata.tag_list ~= "" then
            widget:SetToolTip(edge.userdata.tag_list)
            if not widget.tag_widget then
                widget.tag_widget = widget:AddChild(Widget.Label("body", 10))
            end
            widget.tag_widget:Show()
            widget.tag_widget:SetText(edge.userdata.tag_list)
            
            widget.tag_widget:SetPos(0, 10)
        else
            if widget.tag_widget then
                widget.tag_widget:Hide()
            end
        end

    else
        widget:Hide()
    end
end


local navpath_last_refreshed = 0

function NavPath:Refresh()
    navpath_last_refreshed = navpath_last_refreshed + 1

    for k,node in pairs(self.graph.nodes) do
        local widget = self.elements_widgets[node]
        if not widget then
            widget = self.node_root:AddChild(Widget.Image(global_images.circle):SetSize(CIRCLE_SIZE, CIRCLE_SIZE))    
            self.elements_widgets[node] = widget
            self.widgets_elements[widget] = node
        end

        widget.navpath_last_refreshed = navpath_last_refreshed
        UpdateNode(widget, node)
    end

    for k, edge in pairs(self.graph.edges) do
        local widget = self.elements_widgets[edge]
        if not widget then
            widget =  self.edge_root:AddChild(Widget.SolidBox(1,1,0xffffff))
            self.widgets_elements[widget] = edge
            self.elements_widgets[edge] = widget
        end

        widget.navpath_last_refreshed = navpath_last_refreshed
        UpdateEdge(widget, edge)
    end

    for widget, element in pairs(self.widgets_elements) do
        if widget.navpath_last_refreshed < navpath_last_refreshed then
            -- old widget, remove it:
            widget:Remove()
            self.widgets_elements[widget] = nil -- setting to nil is ok, as long as no new keys are created
            self.elements_widgets[element] = nil
        end
    end
end


function NavPath:OnUpdate(dt)
    
    if Widget.NavPath.FIND_PATH_POINT then
        self.pathx, self.pathy = self:TransformFromWorld(TheGame:GetInput():GetMousePos())
        Widget.NavPath.FIND_PATH_POINT = false
        print ("FINDING POINT", self.pathx, self.pathy)
    end

    if self.pathx then
        local mx, my = TheGame:GetInput():GetMousePos()
        local x, y = self:TransformFromWorld(mx,my)
        local cx, cy, closest_edge, distsq = self.graph:FindClosestPoint(x,y)
        local path = self.graph:FindPath(self.pathx, self.pathy, x, y)
        if path then
            local lastx, lasty
            --local lastx, lasty = self:TransformToWorld( self.pathx, self.pathy )
            for k, v in ipairs(path) do
                local x,y = self:TransformToWorld( v.x, v.y )
                if lastx then
                    TheGame:FE():DrawDebugLine(lastx, lasty,x,y, 0xff0000ff)
                end
                lastx, lasty = x, y
            end
            --TheGame:FE():DrawDebugLine(lastx, lasty,mx,my, 0xff0000ff)
        end
    end
end


