require "ui/widgets/vsteamdisplay"
----------------------------------------------------------------------------------------------------------------------------
-- Displays Negotiation or Fight previews
local NegotiationPreviewPanel = class("Widget.NegotiationPreviewPanel", Widget)

local CONTENT_PADDING = 20
local DETAILS_TILE_W = 380
local BG_TINT = 0x191221ee
local BG_TINT_DURATION = 0.3

-- card_type: QCARD_TYPE.NEGOTIATION or QCARD_TYPE.BATTLE
function NegotiationPreviewPanel:init( card_type )
    NegotiationPreviewPanel._base.init(self)
    self.hover_check = false
    self.card_type = card_type or QCARD_TYPE.NEGOTIATION--QCARD_TYPE.BATTLE

    if TheGame:GetGameState() and TheGame:GetGameState():GetGameType() == GAME_TYPE.DAILY then
        self.music_event = TheGame:LookupPlayerMusic( "pre_battle_music" )
    end

    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- A container for all the background elements
    self.panel = self:AddChild( Widget.WidePanel( 300 ) )
        :SetTopTitlesMargin( 20 )
        :SetColour( self.card_type == QCARD_TYPE.NEGOTIATION and UICOLOURS.NEGOTIATION or UICOLOURS.FIGHT )
        :SetIcon( self.card_type == QCARD_TYPE.NEGOTIATION and engine.asset.Texture("ui/card_type_negotiation.tex") or engine.asset.Texture("ui/card_type_fight.tex") )

    -- Contains everything inside the panel
    self.contents = self:AddChild(Widget())
    self.content_divider_top = self.contents:AddChild( Widget.Image( engine.asset.Texture("large/dialog_divider_top.tex" ) ) )
        :SetSize( self.screen_w*0.9, 4 )
        :Bloom( 0.15 )
        :SetTintColour( self.card_type == QCARD_TYPE.NEGOTIATION and UICOLOURS.NEGOTIATION_LIGHT or UICOLOURS.FIGHT_LIGHT )
    self.content_divider_bottom = self.contents:AddChild( Widget.Image( engine.asset.Texture("large/dialog_divider_bottom.tex" ) ) )
        :SetSize( self.screen_w*0.9, 4 )
        :Bloom( 0.15 )
        :SetTintColour( self.card_type == QCARD_TYPE.NEGOTIATION and UICOLOURS.NEGOTIATION_LIGHT or UICOLOURS.FIGHT_LIGHT )

    -- Displays both sides of the negotiation
    self.teams_root = self.contents:AddChild(Widget())

    -- Agent listings
    self.right_team = self.teams_root:AddChild( Widget.VSTeamDisplay( false, self.card_type ) )
    self.left_team = self.teams_root:AddChild( Widget.VSTeamDisplay( true, self.card_type ) )
    self.content_divider_bottom:SendToFront()
    self.vs = self.contents:AddChild( Widget.VSTeamBossTitle( 80, nil, 0x021A18cc, 3, -2 ) )
        :SetText( LOC"UI.NEGOTIATION_PREVIEW.VS" )
        :SetGlyphColour( self.card_type == QCARD_TYPE.NEGOTIATION and UICOLOURS.NEGOTIATION_LIGHT or UICOLOURS.FIGHT_LIGHT )
    
    -- All the possible handicaps
    self.handicaps = self.contents:AddChild( Widget() )
    self.team_advantage = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_advantage.tex"), UICOLOURS.BONUS, LOC"CONVO_OPTION.BATTLE_ADVANTAGE", LOC"CONVO_OPTION.BATTLE_ADVANTAGE_DESC" ) )
    self.team_advantage:SetToolTip(LOC"CONVO_OPTION.BATTLE_ADVANTAGE_TT")
    self.team_disadvantage = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_disadvantage.tex"), UICOLOURS.PENALTY, LOC"CONVO_OPTION.BATTLE_DISADVANTAGE", LOC"CONVO_OPTION.BATTLE_DISADVANTAGE_DESC" ) )
    self.team_disadvantage:SetToolTip(LOC"CONVO_OPTION.BATTLE_DISADVANTAGE_TT")
    self.no_backup = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_nobackup.tex"), UICOLOURS.PENALTY, LOC"CONVO_OPTION.BATTLE_NOBACKUP", LOC"CONVO_OPTION.BATTLE_NOBACKUP_DESC" ) )
    self.no_resolve = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_noresolve.tex"), UICOLOURS.PENALTY, LOC"CONVO_OPTION.AUTOLOSE_TITLE", LOC"CONVO_OPTION.AUTOLOSE_DESC" ) )
    self.isolated = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_isolated.tex"), UICOLOURS.SUBTITLE, LOC"CONVO_OPTION.ISOLATED_TITLE", LOC"CONVO_OPTION.ISOLATED_DESC" ) )
    self.no_surrender = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_no_surrender.tex"), UICOLOURS.SUBTITLE, LOC"CONVO_OPTION.NOSURRENDER_TITLE", LOC"CONVO_OPTION.NOSURRENDER_DESC" ) )    
    self.survival = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_no_surrender.tex"), UICOLOURS.SUBTITLE, LOC"CONVO_OPTION.SURVIVAL_TITLE" ) )    
    self.no_run = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_noescape.tex"), UICOLOURS.PENALTY, LOC"CONVO_OPTION.NO_RUN_TITLE", LOC"CONVO_OPTION.NO_RUN_DESC" ) )
    self.intimidation = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_intimidation.tex"), UICOLOURS.SUBTITLE, LOC"CONVO_OPTION.INTIMIDATION_TITLE", LOC"CONVO_OPTION.INTIMIDATION_DESC" ) )
    self.provocation = self.handicaps:AddChild( Widget.PreviewDetailsTile( DETAILS_TILE_W, engine.asset.Texture("ui/ic_handicaps_intimidation.tex"), UICOLOURS.SUBTITLE, LOC"CONVO_OPTION.PROVOCATION_TITLE", LOC"CONVO_OPTION.PROVOCATION_DESC" ) )

    self.faded_in = false
    
    self:SetHiddenBoundingBox( true )
end

function NegotiationPreviewPanel:OnAdded()
    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )


end

function NegotiationPreviewPanel:OnControlModeChange( cm, device_id )
    NegotiationPreviewPanel._base.OnControlModeChange( self, cm, device_id )
end

function NegotiationPreviewPanel:OnScreenModeChange( sm )
    NegotiationPreviewPanel._base.OnScreenModeChange( self, sm )

    -- Scale the handicaps
    for i, w in ipairs( self.handicaps.children ) do
        w:SetLayoutScale( LAYOUT_SCALE[sm] )
    end
    
    if sm == SCREEN_MODE.MONITOR then
        self.panel:SetTopTitlesMargin( 20 )
    elseif sm == SCREEN_MODE.SMALL
        or sm == SCREEN_MODE.TV then
        self.panel:SetTopTitlesMargin( 10 )
    end

    self.handicaps:StackChildrenRow( 15 )
    self.handicaps:LayoutBounds( nil, "below", self.content_divider_bottom ):Offset( 0, -CONTENT_PADDING )
    self.handicaps:LayoutBounds( "center", nil, self.panel ):Offset( 0, 0 )
end

function NegotiationPreviewPanel:SetFn( fn )
    self.fn = fn
    return self
end

function NegotiationPreviewPanel:Refresh( scenario )

    self.scenario = scenario
    local team1, team2 = {}, {}
    local team1reasons, team2reasons = {}, {}
    local team2difficulty = 0
    local team2boss = false
    self.handicap_count = 0

    -- Audio to be played when the enemies animate in
    self.slide_audio = "event:/ui/prebattle_overlay/prebattle_overlay_whooshin_regular_generic"

    if self.card_type == QCARD_TYPE.NEGOTIATION then

        -- Calculate the resolve for each side
        local delta_resolve = 0
        if #scenario.resolve_modifiers > 0 then
            for i, mod in ipairs( scenario.resolve_modifiers ) do
                if mod.value then
                    delta_resolve = delta_resolve + mod.value
                end
            end
        end

        team2difficulty = scenario.difficulty
        local adjusted_base_resolve = scenario.enemy_resolve_required-- or Negotiation.MiniGame.GetPersuasionRequired( scenario.difficulty )
        delta_resolve  = math.max(delta_resolve, -adjusted_base_resolve + 10)
        local calc_resolve = adjusted_base_resolve
        adjusted_base_resolve = adjusted_base_resolve - delta_resolve

        -- Gather the team members on each side
        team1 = shallowcopy( scenario.helpers )
        team2 = shallowcopy( scenario.hinders )

        table.insert(team1, 1, TheGame:GetGameState():GetPlayerAgent())
        table.insert(team2, 1, scenario.agent)

        -- Update the team listings and info
        self.left_team:Refresh(team1)
        self.left_team:SetResolve( scenario.resolve, scenario.max_resolve, UICOLOURS.NEGOTIATION_LIGHT, UICOLOURS.NEGOTIATION )
        self.right_team:Refresh(team2)

        -- Get the audio file to play when the enemies show up
        self.slide_audio = self.right_team:GetTeamAudio() or self.slide_audio

        -- Generate an info tooltip for the enemy team's resolve
        local resolve_tt = {}
        table.insert( resolve_tt, loc.format( LOC"UI.TOOLTIP_CONVO_OPTION.BASE_RESOLVE", adjusted_base_resolve))
        if #scenario.resolve_modifiers > 0 then
            for i, mod in ipairs( scenario.resolve_modifiers ) do
                if mod.value then
                    local text = mod.text
                    if mod.value < 0 then
                        table.insert( resolve_tt, loc.format( LOC"UI.TOOLTIP_CONVO_OPTION.MODIFIER_BONUS", text, mod.value ))
                    elseif mod.value > 0 then
                        table.insert( resolve_tt, loc.format( LOC"UI.TOOLTIP_CONVO_OPTION.MODIFIER_PENALTY", text, mod.value ))
                    end
                end
            end
        end
        local enemy_resolve_color = UICOLOURS.NEGOTIATION
        if delta_resolve > 0 then
            enemy_resolve_color = UICOLOURS.PENALTY
        elseif delta_resolve < 0 then
            enemy_resolve_color = UICOLOURS.BONUS
        end
        self.right_team:SetResolve( calc_resolve, calc_resolve, UICOLOURS.NEGOTIATION_LIGHT, enemy_resolve_color, table.concat(resolve_tt, "\n") )
        
        -- Hide the battle handicaps
        self.team_advantage:SetShown( false )
        self.team_disadvantage:SetShown( false )
        self.no_backup:Hide()
        self.isolated:Hide()
        self.no_surrender:Hide()
        self.no_run:Hide()
        self.survival:Hide()
        self.intimidation:SetShown(CheckBits( scenario.flags, NEGOTIATION_FLAGS.INTIMIDATION ))
        self.provocation:SetShown(CheckBits( scenario.flags, NEGOTIATION_FLAGS.PROVOCATION ))
        self.no_resolve:SetShown( scenario.resolve <= 0 )

    elseif self.card_type == QCARD_TYPE.BATTLE then

        -- Gather the team members on each side
        team1 = table.map(scenario:GetTeam(TEAM.BLUE), function(agent) if not agent:IsPlayer() then return agent end end)
        team2 = table.map(scenario:GetTeam(TEAM.RED), function(agent) if scenario:GetPrimary(TEAM.RED) ~= agent then return agent end end)
        table.insert(team1, 1, TheGame:GetGameState():GetPlayerAgent())
        table.insert(team2, 1, scenario:GetPrimary(TEAM.RED))

        
        for k,agent in pairs(scenario:GetTeam(TEAM.BLUE)) do
            local _, txt = scenario:GetFightReason(agent)
            team1reasons[agent] = txt
        end

        local team_strength_total = 0
        for k,agent in pairs(scenario:GetTeam(TEAM.RED)) do
            if agent.boss then team2boss = true end

            local _, txt = scenario:GetFightReason(agent)
            team_strength_total = team_strength_total + (agent.combat_strength or 1)
            
            team2reasons[agent] = txt
        end

        team2difficulty = remap( team_strength_total, 1, 20, 1, 5 )

        -- Update the team listings and info
        self.right_team:Refresh(team2)
        self.left_team:Refresh(team1, self.right_team:IsBossBattle())

        -- Get the audio file to play when the enemies show up
        self.slide_audio = self.right_team:GetTeamAudio() or self.slide_audio

        -- Check what fight handicaps are present
        self.team_advantage:SetShown( scenario:GetAdvantageTeam() == TEAM.BLUE )
        self.team_disadvantage:SetShown( scenario:GetAdvantageTeam() == TEAM.RED )
        self.no_backup:SetShown( not scenario:AllowsBackup() )
        self.isolated:SetShown(CheckBits( scenario.flags, BATTLE_FLAGS.ISOLATED ))
        self.no_surrender:SetShown(CheckBits( scenario.flags, BATTLE_FLAGS.NO_SURRENDER ))
        
        self.no_run:SetShown( not scenario:CanFlee() )
        local survival_turns = scenario:GetSurvivalTurns()
        self.survival:SetShown( survival_turns ~= nil )
        if survival_turns then
            self.survival:SetText( nil, loc.format( LOC"CONVO_OPTION.SURVIVAL_DESC", scenario:GetSurvivalTurns() ))
        end

        self.intimidation:Hide()
        self.provocation:Hide()
        self.no_resolve:Hide()

        self.right_team:ShowCombatStrengths()
        self.left_team:ShowCombatStrengths()

    end

    -- Layout the handicaps
    self.handicaps:StackChildrenRow( 15 )

    -- Scale up the dividers if there are lots of teams
    local large_teams = #team1 >= 3 or #team2 >= 3 or team2boss
    self.content_divider_top:SetSize( self.screen_w * ( large_teams and 1 or 0.7 ), 5 )
    local w_mult = 0.6
    if large_teams then w_mult = 0.9 end
    if team2boss then w_mult = 1.4 end
    self.content_divider_bottom:SetSize( self.screen_w * w_mult, 5 )
    
    -- Update texts
    if self.card_type == QCARD_TYPE.NEGOTIATION then
        self.panel:SetTitles( LOC"UI.NEGOTIATION_PREVIEW.TITLE" )
        self.vs:SetText( LOC"UI.NEGOTIATION_PREVIEW.VS" )
    elseif self.card_type == QCARD_TYPE.BATTLE then
        self.panel:SetTitles( LOC"UI.BATTLE_PREVIEW_PANEL.TITLE", scenario:IsSelfDefence() and LOC"CONVO_OPTION.BATTLE_SELF_DEFENSE" or LOC"CONVO_OPTION.BATTLE_AGGRESSOR" )
        self.vs:SetText( LOC"UI.BATTLE_PREVIEW_PANEL.VS" )
    end
    
    -- Layout contents
    local BOSS_X_OFFSET = 0
    self.content_divider_top:LayoutBounds( nil, "below", self.panel:GetTitlesWidget() ):Offset( 0, -CONTENT_PADDING )
    
    self.left_team:LayoutBounds( nil, "below", self.content_divider_top )
        :LayoutBounds( "before", nil, self.vs )
        :Offset( team2boss and BOSS_X_OFFSET-30 or -30, -CONTENT_PADDING )
    self.right_team:LayoutBounds( nil, "below", self.content_divider_top )
        :LayoutBounds( "after", nil, self.vs )
        :Offset( team2boss and BOSS_X_OFFSET+30 or 30, -CONTENT_PADDING )
    self.vs:LayoutBounds( nil, "center", self.teams_root ):Offset( team2boss and BOSS_X_OFFSET or 0 )
    self.content_divider_bottom:LayoutBounds( nil, "below", self.teams_root ):Offset( 0, 2 -CONTENT_PADDING )
    self.handicaps:LayoutBounds( nil, "below", self.content_divider_bottom ):Offset( 0, -CONTENT_PADDING )
    self.handicaps:LayoutBounds( "center", nil, self.panel_bg ):Offset( 0, 0 )
    
    -- Size up the background
    local titles_w, titles_h = self.panel:GetTitlesWidget():GetSize()
    local title_fudge_factor = titles_h
    local w, h = self.contents:GetSize()
    h = h + CONTENT_PADDING*3 + title_fudge_factor
    self.panel:SetSize( self.screen_w, h )
    self.contents:LayoutBounds( nil, "bottom", self.panel:GetBottomEdgeWidget() ):Offset( 0, CONTENT_PADDING )
end

function NegotiationPreviewPanel:FadeIn()
    self.panel:CancelFade()
    if self.fadein_anim then 
        self.fadein_anim:Stop() self:SetFadedOutFinalState()
        self.screen_loop = AUDIO:CreateEventInstance("event:/ui/store/menu_open")
    end
    if self.faded_in then return end



    if self.music_event then
        self.music_loop = TheGame:GetMusic():PlayMusic(self.music_event)
    end

    local r, g, b, a = HexColour(BG_TINT)
    self:GetScreen():Fade( BG_TINT_DURATION, a, r, g, b )

    self:Show()
    self.faded_in = true

    -- Prepare for animation by getting current values
    local pacing = 0.1
    self.left_team:PrepareAnimation()
    self.right_team:PrepareAnimation()

    -- Prepare handicap display sequence
    for i, widget in ipairs( self.handicaps.children ) do
        widget:SetTintAlpha(0)
    end
    local handicap_anim_duration = pacing/2
    local delay_per_handicap = pacing/2
    local handicap_adder = Updater.Parallel()
    for i, widget in ipairs( self.handicaps.children ) do
        if widget:IsShown() then
            handicap_adder:Add( 
                Updater.Series{
                    Updater.Wait( delay_per_handicap*i ),
                    Updater.Parallel{
                        Updater.Ease( function(v) widget:SetTintAlpha(v) end, 0, 1, handicap_anim_duration, easing.outQuad),
                        Updater.Ease( function(v) widget:SetScale(v,v) end, 1.2, 1, handicap_anim_duration, easing.outQuad)
                    }
                }
            )
        end
    end

    -- Setup and start animation
    self.panel:FadeIn( 
        function()
            self.fadein_anim = self:RunUpdater(
                Updater.Series{
                    Updater.Parallel{
                        Updater.Ease( function(v) self.vs:SetTintAlpha(v) end, 0, 1, pacing, easing.linear),
                        Updater.Ease( function(v) self.content_divider_top:SetTintAlpha(v) end, 0, 1, pacing, easing.linear),
                        Updater.Ease( function(v) self.content_divider_bottom:SetTintAlpha(v) end, 0, 1, pacing, easing.linear),
                        Updater.Do( function() self.left_team:AnimateIn() end ),
                        Updater.Do( function() self.right_team:AnimateIn() end ),
                        Updater.Do( function() 
                            self.slide_audio_event = AUDIO:CreateEventInstance( self.slide_audio )
                            self.slide_audio_event:Start()
                        end ),
                        handicap_adder,
                    },
                }
            )
        end
    )

end

function NegotiationPreviewPanel:IsFadedIn()
    return self.faded_in
end

function NegotiationPreviewPanel:FadeOut()
    self.panel:CancelFade()
    self.faded_in = false

    self:GetScreen():Fade( BG_TINT_DURATION, 0 )

    local pacing = 0.05
    self.vs:AlphaTo( 0, pacing )
    self.content_divider_top:AlphaTo( 0, pacing )
    self.content_divider_bottom:AlphaTo( 0, pacing )
    self.left_team:AnimateOut()
    self.right_team:AnimateOut()
    for i, widget in ipairs( self.handicaps.children ) do
        widget:AlphaTo( 0, pacing )
    end
    self.panel:FadeOut( function() self:SetFadedOutFinalState() end )
    if self.slide_audio_event then
        self.slide_audio_event:Stop()
        self.slide_audio_event:Release()
        self.slide_audio_event = nil
    end


end

function NegotiationPreviewPanel:SetFadedOutFinalState()

    self.faded_in = false
    self:Hide()
    self.vs:SetTintAlpha( 0 )
    self.content_divider_top:SetTintAlpha( 0 )
    self.content_divider_bottom:SetTintAlpha( 0 )
    self.left_team:PrepareAnimation()
    self.right_team:PrepareAnimation()
    for i, widget in ipairs( self.handicaps.children ) do
        widget:SetTintAlpha( 0 )
    end

    if self.music_loop then
        TheGame:GetMusic():StopMusic(self.music_loop)
        self.music_loop = nil
    end

end

-------------------------------------------------------------

local PreviewDetailsTile = class("Widget.PreviewDetailsTile", Widget)

local TILE_PADDING = 10
local TILE_ICON_SIZE = 58
local LEFT_EDGE_FUDGE_FACTOR = 3 -- The texture has a line on the left edge

function PreviewDetailsTile:init( w, icon, colour, title, desc )
    PreviewDetailsTile._base.init( self )

    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/empty_corner_frame_border.tex" ) ) ):SetSize( w, TILE_ICON_SIZE + TILE_PADDING*2 ):SetBloom(0.1)
    self.icon = self:AddChild( Widget.Image( icon and icon or engine.asset.Texture("UI/favour.tex") ) ):SetSize( TILE_ICON_SIZE, TILE_ICON_SIZE )
    self.content = self:AddChild( Widget() ):SetHiddenBoundingBox( true ) -- Holds the title and desc, so they can be centered vertically to the icon
    self.title = self.content:AddChild( Widget.Label("title", 24, title and title or "" ) ):SetGlyphColour(UICOLOURS.RELATIONSHIP_NEUTRAL):SetAutoSize( w - TILE_PADDING*3 - TILE_ICON_SIZE - LEFT_EDGE_FUDGE_FACTOR ):LeftAlign()
    self.desc = self.content:AddChild( Widget.Label("body", 22, desc and desc or "" ) ):SetGlyphColour(UICOLOURS.TEXT):SetWordWrap( true ):SetAutoSize( w - TILE_PADDING*3 - TILE_ICON_SIZE - LEFT_EDGE_FUDGE_FACTOR ):LeftAlign()
    self.desc:SetHoverLinkFn( UIHelpers.ItemTooltips )
    self.desc:OverrideLineHeight(15)

    self.bg:SetTintColour( colour )

    self:Layout( )
end

function PreviewDetailsTile:SetText( title, desc )
    if title then self.title:SetText( title ) end
    if desc then self.desc:SetText( desc ) end
    self:Layout( )
end

function PreviewDetailsTile:Layout( )
    self.icon:LayoutBounds( "left", "top", self.bg ):Offset( TILE_PADDING + LEFT_EDGE_FUDGE_FACTOR, -TILE_PADDING )
    self.desc:LayoutBounds( "left", "below", self.title ):Offset( 0, 5 )

    self.content:LayoutBounds( "after", "center", self.icon ):Offset( TILE_PADDING, 1 )
end

function PreviewDetailsTile:SetItemImage( item_image )
    self.icon:Remove()
    self.icon = self:AddChild( item_image ):SetSize( TILE_ICON_SIZE, TILE_ICON_SIZE )
    self.icon:LayoutBounds( "left", "top", self.bg ):Offset( TILE_PADDING, -TILE_PADDING )
end
