require "ui/widget"

local CLR_RESOLVE_BAR = UICOLOURS.NEGOTIATION
local CLR_RESOLVE_BG = 0x120D32FF

local BAR_HEIGHT = 12
local BAR_WIDTH = 100

------------------------------------------------------------------------------
-- Main Resolve bar.

local NegotiationResolveBar = class("Widget.NegotiationResolveBar", Widget )

function NegotiationResolveBar:init( is_player )
    NegotiationResolveBar._base.init( self )

    self.is_player = is_player

    self.composure = 0

    self.current_resolve = 0
    self.current_resolve_max = 0

    self.target_resolve = 0
    self.target_resolve_max = 0

    self.bar = Widget.ProgressBar( 100, BAR_HEIGHT ):OrientH():SetOutline(4)
    self.bar:SetBarColour( CLR_RESOLVE_BAR ):SetBGColour( CLR_RESOLVE_BG ):SetOutlineColour( 0x00000000 )
    self.bar:SetSize( BAR_WIDTH+2, BAR_HEIGHT+2 ):SetProgress( 0 ):Bloom(0.1)
    self:AddChild( self.bar )

    local bg_border = 12
    local bg_border_scale = 0.5
    self.bar_bg = Widget.Panel( engine.asset.Texture("UI/argument_bar_bg.tex") )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
        :SetNineSliceBorderScale( bg_border_scale ):Bloom(0.15)
    local nineslice_w, nineslice_h = BAR_WIDTH + bg_border*2*bg_border_scale, BAR_HEIGHT + bg_border*2*bg_border_scale
    self.bar_bg:SetSize( nineslice_w, nineslice_h )
    self:AddChild( self.bar_bg )

    self.label = Widget.Label( "title", 28 ):SetGlyphColour( UICOLOURS.NEGOTIATION ):EnableOutline(true)
    self:AddChild( self.label ):Bloom(0.1)

    self:RefreshResolveText( 0, 0 )
end

function NegotiationResolveBar:OnAdded()
    self:StartUpdating()
end

function NegotiationResolveBar:RefreshResolveText( resolve, max_resolve, composure )
    self.composure = composure or self.composure
    self.resolve = resolve or self.resolve

    if (self.composure or 0) > 0 then
        self.label:SetText(loc.format("{1} <#COMPOSURE>+{2}</>", self.resolve, self.composure ))
    else
        self.label:SetText(loc.format("{1}", self.resolve))
    end

    self:Layout()
end

function NegotiationResolveBar:Layout()
    self.label:LayoutBounds( "center", "below", self.bar_bg ):Offset( 0, 3 )
end

function NegotiationResolveBar:RefreshResolve( resolve, max_resolve )
    self.target_resolve = resolve
    self.target_resolve_max = math.max( 1, max_resolve )

    local delta = resolve - self.current_resolve
    if delta ~= 0 then
        self.delta_delay_t = remap( delta, 1, 20, 0.2, 1 )
        if self.is_player then
            AUDIO:PlayParamEvent( "event:/sfx/battle/cards/neg/resolve_change_player", "intensity", delta)
        else
            AUDIO:PlayParamEvent( "event:/sfx/battle/cards/neg/resolve_change_enemy", "intensity", delta)
        end
    end
end

function NegotiationResolveBar:AnimateResolve( resolve, max_resolve )
    self:RefreshResolve( resolve, max_resolve )
end

function NegotiationResolveBar:OnUpdate( dt )
    NegotiationResolveBar._base.OnUpdate( self, dt )

    if self.current_resolve ~= self.target_resolve or self.current_resolve_max ~= self.target_resolve_max then
        local max_dt = self.delta_delay_t or 0.2
        self.dt = math.min( max_dt, (self.dt or 0) + dt )
        local t = easing.outQuad( self.dt, 0, 1.0, max_dt )
        if t >= 1.0 then
            self.current_resolve = self.target_resolve
            self.current_resolve_max = self.target_resolve_max
            self.dt = nil
            self.bar:SetProgress( self.current_resolve / self.current_resolve_max )
            self.bar:SetBarBloom( 0.1 )
            self:RefreshResolveText( self.current_resolve, self.current_resolve_max )
            if self.onResolveChangeFn then self.onResolveChangeFn( self.current_resolve, self.current_resolve_max ) end
        else
            local resolve = math.floor( easing.linear( t, self.current_resolve, self.target_resolve - self.current_resolve, 1.0 ))
            self.bar:SetProgress( resolve / self.current_resolve_max ):SetBarBloom( 0.15 )
            self:RefreshResolveText( resolve, self.current_resolve_max )
            if self.onResolveChangeFn then self.onResolveChangeFn( resolve, self.current_resolve_max ) end
        end
    end
end

function NegotiationResolveBar:SetOnResolveChangeFn( fn )
    self.onResolveChangeFn = fn
end

function NegotiationResolveBar:SetOnComposureChangeFn( fn )
    self.onComposureChangeFn = fn
end

function NegotiationResolveBar:RefreshComposure( value )
    self:RefreshResolveText( nil, nil, value )
end

function NegotiationResolveBar:AnimateComposure( value, old_value, start_of_turn )
    self:RefreshResolveText( nil, nil, value )
    if self.onComposureChangeFn then self.onComposureChangeFn( value ) end

    self.label:ScaleTo( 1.0, 2, 0.2, easing.outQuad )
    self.label:DelayIfAnimating()

    self.label:DelayIfAnimating()
    self.label:ScaleTo( 2, 1.0, 0.5, easing.outQuad )
    self.label:DelayIfAnimating()
end
