require "ui/widgets/image"

------------------------------------------------
-- Reflects an agent's opinion of the player.

local OpinionIndicator = class("Widget.OpinionIndicator", Widget)

function OpinionIndicator:init( size, label_size )
    OpinionIndicator._base.init( self )

    size = size or 40

    self.icon = Widget.Image( global_images.relationship_level[RELATIONSHIP.NEUTRAL], size, size )
    self:AddChild( self.icon )
    self.showBg = true

    if label_size then
        self.opinion_txt = Widget.Label( "body", size )
        self:AddChild( self.opinion_txt )
    end
end

function OpinionIndicator:SetRightSide( is_right )
    self.is_right = is_right
    return self
end

function OpinionIndicator:ShowBg( show )
    self.showBg = show
    self:UpdateInternal()
    return self
end

function OpinionIndicator:SetIconSize( size )
    self.icon:SetSize( size, size )
    return self
end

function OpinionIndicator:OnRemoved()
    if self.current_agent and not self.current_agent:IsPlayer() then
        self.current_agent:RemoveListener( self )
    end
    if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() then
        TheGame:GetGameState():GetPlayerAgent():RemoveListener( self )
    end
end

function OpinionIndicator:OnAdded()
    if TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent() then
        TheGame:GetGameState():GetPlayerAgent():ListenForAny( self, self.OnAgentEvent )
    end
end

function OpinionIndicator:OnAgentEvent(event, agent)
    if event == "relationship_changed" then
        self:UpdateInternal()
    end
end

function OpinionIndicator:UpdateInternal()

    local agent = self.current_agent

    if agent and not agent:IsPlayer() and agent.social_connections and TheGame:GetGameState() then
        local rel = agent:GetRelationship()
        self:Show()
        

        local why_event = agent.social_connections and agent.social_connections:GetRelationshipReason(TheGame:GetGameState():GetPlayerAgent())
        local str
        if why_event then
            str = loc.format( LOC"UI.OPTION_INDICATOR.REASON", FormatOpinionEvent( why_event, agent ), why_event.time )
        end

        if agent:IsDead() then
            self:SetToolTip( str )
        elseif str then
            self:SetToolTip( loc.format("{1}\n\n{2}", GetRelationshipPlayerToolTipString(rel), str))
        else
            self:SetToolTip( GetRelationshipPlayerToolTipString(rel) )
        end
        
        if self.showBg then
            self.icon:SetTexture( global_images.relationship_level[rel] )
        else
            self.icon:SetTexture( global_images.relationship_icons[rel] )
        end

        if self.opinion_txt then
            self.opinion_txt:SetText(GetRelationshipString(rel))
            -- is the indicator on the 'right' side of the label?
            if self.is_right then
                self.opinion_txt:LayoutBounds( "before", "center", self.icon):Offset(-5, 0)
            else
                self.opinion_txt:LayoutBounds( "after", "center", self.icon):Offset(5, 0)
            end
        end
    else
        self:Hide()
    end
end

function OpinionIndicator:UpdateOpinion( agent )
    if self.current_agent then
        self.current_agent:RemoveListener(self)
    end

    self.current_agent = agent

    if agent and not agent:IsPlayer() then
        self.current_agent:ListenForAny(self, self.OnAgentEvent )
    end
    
    self:UpdateInternal()

    return self
end


