-- lays out widgets, allows you to click on one of them, executes a custom function and closes the screen.
local OptionPicker = class("Widget.OptionPicker", Widget)

function OptionPicker:init( on_close, no_skip)
    OptionPicker._base.init( self )
    self.options = {}

    self.on_close = on_close

    self.bg = Widget.SolidBox( nil, nil, 0xFFFFFF00 )
    self.content = Widget()

    self.title = self.content:AddChild(Widget.Label( "title", 50, "" ))
    self.options_base = self.content:AddChild(Widget())
    
    if not no_skip then
        self.cancel_button = self.content:AddChild( Widget.IconButton( LOC"UI.OPTION_PICKER.SKIP", on_close ) )
        self.cancel_button:SetIcon( engine.asset.Texture("UI/ic_end_turn.tex") )
    end

    self:AddChildren{ self.bg, self.content }
end

function OptionPicker:SetTitle( text )
    self.title:SetText(text)
    self:DoLayout()
end

function OptionPicker:AddOption( option, hover_fn )
    local click_base = Widget.Clickable()
    if hover_fn then click_base:SetHoverFn( hover_fn ) end
    click_base:SetOnClickFn( function() self:PickOption( option ) end )
    click_base:AddChild( option )
    click_base.highlight_widget = option

    self.options_base:AddChild(click_base)
    click_base:LayoutBounds("after", "center"):Offset(30, 0)
    table.insert(self.options, click_base)
    self:DoLayout()
    return click_base
end

function OptionPicker:SetHoverFn( fn )
    self.hover_fn = fn
end

function OptionPicker:DoLayout()
    self.options_base:LayoutBounds("center", "center", self.content)
    self.title:LayoutBounds("center", "above", self.options_base):Offset(0, 10)
    
    if self.cancel_button then
        self.cancel_button:LayoutBounds("center", "below", self.options_base):Offset(0, -10)
    end

    self.bg:SizeToWidgets( 20, self.content )
end

function OptionPicker:PickOption( option )
    if option.OnSelected then
        option:OnSelected( function() self.on_close() end)
        
    else
        self.on_close()
    end
end
