require "ui/widget"
require "ui/widgets/label"
require "ui/widgets/image"
require "ui/widgets/solidbox"

local assets =
{
    bg = engine.asset.Texture ("UI/btn_normal.tex"),
    bg_hover = engine.asset.Texture ("UI/btn_normal_hover.tex"),
    glow = engine.asset.Texture ("UI/btn_glow.tex"),
}

local PanelButton = class("Widget.PanelButton", Widget)
PanelButton.can_focus = true

local SMALLEST_LABEL_TEXT = 15
local LARGEST_LABEL_TEXT = 20

PanelButton.CONTROL_MAP = 
{
    {
        control = Controls.Digital.MENU_ACCEPT, 
        fn = function(self)
            if self.down then
                self.down = false
                self:UpdateImage()

                if self.hover or self.focus then
                    self:Click()
                    return true
                end
            end
        end,
        
        test = function(self)
            return self.enabled
        end,

        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.SELECT", Controls.Digital.MENU_ACCEPT))
        end
    },

}



function PanelButton:init( text, fn, w, h, text_size, img, img_hover )
    w = w or 200
    h = h or 40
    text_size= text_size or 20
    PanelButton._base.init(self)
    self.w = w
    self.h = h
    self.border = 1
    self.enabled = true

    self.normalTexture = img or assets.bg
    self.hoverTexture = img_hover or assets.bg_hover

    self.bg = self:AddChild(Widget.Panel(self.normalTexture))
    self.bg:SetSize(w, h):SetBloom(0.05)
    self.hover_check = true
    self.blocks_mouse = true
    self.scale_on_focus = true

    self.label = self:AddChild(Widget.Label( "button", text_size))
    self.text_size = text_size
    self.label:SetRegionSize( self.bg:GetInnerSize() - 8, h )
    self.label:SetHoverLinkGlyphColour(UICOLOURS.DARK_TEXT_LINK)

    self.glow = self:AddChild(Widget.Image( assets.glow ):SetBloom(0.05))
    self.glow:SetHiddenBoundingBox(true)
    self.glow:UnblockMouse()
    self.animating = false
    local glow_w, glow_h = self.bg:GetSize()
    self.glow:SetSize( glow_w, glow_h + 10 )
    self.glow:SetTintColour( UICOLOURS.TITLE )
    self.glow:SetTintAlpha(0)

    self:SetTextColour( UICOLOURS.BUTTON_TEXT )
    self:SetBGColour( UICOLOURS.SUBTITLE )

    if text then    
        self:SetText(text)
    end

    self.fn = fn
    self:Enable()
    self:EnableGlow()
    self:UpdateImage()
end

function PanelButton:SetBloom( bloom_amount )
    self.bg:SetBloom( bloom_amount )
    return self
end

function PanelButton:SetRed()
    self.bg:SetTintColour(0xf35e5cFF)
    return self
end


function PanelButton:SetImage(img)
    self.normalTexture = img or assets.bg
    self:UpdateImage()
    return self
end

function PanelButton:SetHoverImage(img)
    self.hoverTexture = img or assets.bg_hover
    self:UpdateImage()
    return self
end

function PanelButton:SetFn(fn)
    self.fn = fn
    return self
end

function PanelButton:ShowBG(viz)
    self.bg:SetShown(viz)
    return self
end

function PanelButton:SetFont(font)
    self.label:SetFont(font)
    return self
end

function PanelButton:SetBGColour(r,g,b,a)
    if r and g and b then
        a = a or 1
    elseif r then
        r,g,b,a = HexColour(r)
    else
        r,g,b,a = HexColour(UICOLOURS.SUBTITLE)
    end
    self.bg_tint_r = r
    self.bg_tint_g = g
    self.bg_tint_b = b
    self.bg_tint_a = a
    self:UpdateImage()
    return self
end

function PanelButton:SetMultiLine(outer_width)
    self.multiline = outer_width
    self.bg:SetSize(outer_width, self.h)
    local inner_w = self.bg:GetInnerSize()
    self.label:LeftAlign()
    self.label:SetWordWrap(true)

    self.label:SetRegionSize(inner_w, RES_Y)
    local textw, texth = self.label:GetRenderSize()
    self.label:SetRegionSize(inner_w, texth)
    self.bg:SetInnerSize( inner_w, texth+8)
    self.w, self.h = self.bg:GetSize()

    return self
end

function PanelButton:LeftAlign()
    self.label:LeftAlign()
    
    return self
end
function PanelButton:RightAlign()
    self.label:RightAlign()
    
    return self
end

function PanelButton:SetSize( w, h )
    
    self.w, self.h = w, h
    self.bg:SetSize(self.w, self.h)

    --local text_w, text_h = self.bg:GetInnerSize()
    --print ("SETTING REGION SIZE", text_w - 8, text_h, debugstack())
    self.label:SetRegionSize( self.w - 8, self.h)

    local glow_w, glow_h = self.bg:GetSize()
    self.glow:SetSize( glow_w, glow_h )

    return self
end

function PanelButton:SizeToLabel( hpad, vpad )
    local w, h = self.label:GetRenderSize()
    self:SetSize( w + (hpad or 0) * 2, h + (vpad or 0) * 2 )
    return self
end

function PanelButton:SetTextHoverColour(col)
    self.texthovercolour = col
    self:UpdateImage()
    return self
end

function PanelButton:SetTextColour(col)
    self.textcolour = col
    self:UpdateImage()
    return self
end

function PanelButton:SetTextSize(text_size)
    --self.label:SetFontSize(text_size)
    self.text_size = text_size
    local sz = self.label:TryFitToRegion(SMALLEST_LABEL_TEXT, self.text_size)    
    self.label:SetFontSize(sz)
    return self
end

function PanelButton:SetText(text)
    self.label:SetText(text)
    if self.multiline then
        self:SetMultiLine(self.multiline)
    end
    local sz = self.label:TryFitToRegion(SMALLEST_LABEL_TEXT, self.text_size)    
    self.label:SetFontSize(sz)
    return self
end

function PanelButton:EnableShadow( enable )
    self.label:EnableShadow( enable )
    return self
end

function PanelButton:EnableGlow()
    self.glow_enabled = true
    return self
end


function PanelButton:DisableGlow()
    self.glow_enabled = false
    return self
end

function PanelButton:Enable()
    PanelButton._base.Enable(self)
    self.enabled = true
    self:UpdateImage()
    return self
end


function PanelButton:Disable()
    PanelButton._base.Disable(self)
    self.enabled = false
    self:UpdateImage()
    return self
end

function PanelButton:SetEnabled(enabled)
    if enabled then
        self:Enable()
    else
        self:Disable()
    end
    return self
end

function PanelButton:SetCta( cta )
    self.cta = cta or false
    return self
end

function PanelButton:ScaleOnFocus( scale )
    self.scale_on_focus = scale
    self:SetScale( self.cta and 1.05 or 1 )
    return self
end

function PanelButton:UpdateImage()
    self.bg:SetSaturation(1)
    self.label:SetGlyphColour( self.textcolour )
    if not self.enabled then
        self.bg:SetSaturation(0.1)
        self.bg:SetTintColour(self.bg_tint_r,self.bg_tint_g,self.bg_tint_b,self.bg_tint_a*.7)
        self.label:SetSaturation(1)
        local r, g, b, a = HexColour( self.textcolour )
        self.label:SetGlyphColour( r, g, b, 0.5 )
        self.label:SetHoverLinkGlyphColour( UICOLOURS.DARK_TEXT_LINK )
    elseif self.down then
        self.bg:SetTintColour(self.bg_tint_r,self.bg_tint_g,self.bg_tint_b,self.bg_tint_a*.7)
    elseif self.hover or self.focus then
        self.bg:SetTexture(self.hoverTexture)
        self.bg:SetTintColour(self.bg_tint_r*1.2,self.bg_tint_g*1.2,self.bg_tint_b*1.2,self.bg_tint_a)
    else
        self.bg:SetTexture(self.normalTexture)
        self.bg:SetTintColour(self.bg_tint_r,self.bg_tint_g,self.bg_tint_b,(self.bg_tint_a or 1)*.7)
    end

    if self.scale_on_focus and ( self.hover or self.focus ) then
        self:ScaleTo( nil, self.cta and 1.08 or 1.05, 0.2, easing.outQuad )
    elseif self.scale_on_focus then
        self:ScaleTo( nil, self.cta and 1.05 or 1, 0.2, easing.outQuad )
    end

    if self.enabled then
        if self.texthovercolour and (self.hover) then
            self.label:SetGlyphColour(self.texthovercolour)
        elseif self.textcolour then
            self.label:SetGlyphColour(self.textcolour)
        end
    end
end


function PanelButton:__tostring()
    return string.format("PanelButton(%s)", self.label and self.label.text)
end

function PanelButton:HandleControlDown(control)
    if control:Has( Controls.Digital.MENU_ACCEPT ) and ( self.hover or self.focus ) then
        if self.enabled then
            self.down = true
            self:UpdateImage()
            return true
        end
    end
end

function PanelButton:SetClickSoundEvent(ev)
    self.click_sound_event = ev
    return self
end

function PanelButton:OnGainFocus()
    if not self.silent and self.enabled and self.glow_enabled then
        AUDIO:PlayEvent(SoundEvents.mouseover)
        self:ShowGlow()
    end
    self:UpdateImage()
    if self.hoverfocusfn then
        self.hoverfocusfn(self.hover, self.focus)
    end
end

function PanelButton:OnLoseFocus()
    self.down = false
    self:UpdateImage()
    if self.hoverfocusfn then
        self.hoverfocusfn(self.hover, self.focus)
    end
end

function PanelButton:MakeSilent()
    self.silent = true
    return self
end

function PanelButton:OnGainHover()
    PanelButton._base.OnGainHover(self)

    if not self.silent and self.enabled and self.glow_enabled then
        AUDIO:PlayEvent(SoundEvents.mouseover)
        self:ShowGlow()
    end
    self:UpdateImage()
    if self.hoverfocusfn then
        self.hoverfocusfn(self.hover, self.focus)
    end

end

function PanelButton:OnLoseHover()
    PanelButton._base.OnLoseHover(self)
    
    self.down = false
    self:UpdateImage()
    if self.hoverfocusfn then
        self.hoverfocusfn(self.hover, self.focus)
    end

end

function PanelButton:SetHoverFocusFn(fn)
    self.hoverfocusfn = fn
    return self
end

function PanelButton:HotkeyDown()
    if not self.down and self:IsEnabled() then
        self.down = true
        self:UpdateImage()
    end
    return true
end

function PanelButton:HotkeyUp()
    if self.down then
        self.down = false
        self:UpdateImage()
        if self.fn then self.fn("hotkey") end        
        return true
    end
end

function PanelButton:Click()
    if self.enabled then
        if not self.silent then
            AUDIO:PlayEvent(self.click_sound_event or SoundEvents.button_click)
        end
        if self.fn then self.fn() end
    end
    return self
end

function PanelButton:HideBG()
    self.bg:Hide()
    return self
end

function PanelButton:ShowGlow( force_animation )
    if self.animating==false or force_animation then
        self.animating = true
        self:DoTaskInTime( "fade out infolabel glow", 0.3, function() self.glow:AlphaTo( 0, 0.2 ) end )
        self.glow:AlphaTo( 0.4, 0.2 )
        self.glow:SetPos( -self.bg:GetSize()*0.4, self.glow.y )
        self.glow:MoveXTo( self.bg:GetSize()*0.4, 0.5, nil, function() self.animating = false end)
    end
    return self
end