local CONTENT_LEFT_MARGIN = 15 -- Extra size added on the left of scroll boxes, so that the contents can scale up on hover without clipping
require "ui/widgets/compendiumwidget"

local assets = {
    bg = engine.asset.Texture("UI/compendium_portrait_bg.tex"),
    gradient = engine.asset.Texture("UI/deckfilter_gradient.tex"),
    lock = engine.asset.Texture("UI/card_lock.tex"),
    divider = engine.asset.Texture("large/dialog_divider_bottom.tex" ),
    lore_edge_block = engine.asset.Texture("UI/lore_edge_block.tex"),
    lore_edge = engine.asset.Texture("UI/lore_edge.tex"),
    checkmark = engine.asset.Texture("UI/ic_checkmark.tex" ),
    locked = engine.asset.Texture("UI/ic_locked.tex" ),
    lore_panel = engine.asset.Texture("UI/lore_panel.tex"),
    voice = engine.asset.Texture("UI/ic_agentvoice.tex" ),
    voice_button = engine.asset.Texture("UI/voice_button.tex"),
    voice_button_arrow = engine.asset.Texture("UI/voice_btn_arrow.tex"),
    death_loot = engine.asset.Texture("UI/ic_deathloot.tex" ),
    glow = engine.asset.Texture("images/glow.tex" ),
    inner_frame = engine.asset.Texture("UI/empty_inner_frame_border_selected_clear.tex"),
}


------------------------------------------------------------------------------------------
-- Shows the people panel

local PeopleCompendiumWidget = class( "Widget.PeopleCompendiumWidget", Widget.CompendiumWidget )

local columns = 3
local portrait_w
local portrait_h
local spacing = 20

function PeopleCompendiumWidget:init( width, height, sidebar_width )
    PeopleCompendiumWidget._base.init( self, width, height, sidebar_width )

    -- Default sorting function
    self.sorting_fn = function( w1, w2 ) return w1.agent:GetName() < w2.agent:GetName() end
end

function PeopleCompendiumWidget:AddSidebarFilters()
    -- Add a text search bar
    if SPECIAL_BUILD_TYPE < 1024 then
        self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.FILTER_SEARCH" )
        self.sidebar:AddFilterWidget( Widget.PeopleSearchFilterBlock() )
    end

    -- Add a faction filter
    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.FILTER_FACTION" )
    self.faction_filter_group = self.sidebar:AddFilterWidget( Widget.FilterOptionalRadioGroup() )
        :SetRowMode( 4 )
    -- self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "PLAYER", LOC"UI.CARDCOMPENDIUM.FILTER_FACTION_PLAYER", engine.asset.Texture( "UI/faction_player.tex" ) ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "GRIFTER" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "FEUD_CITIZEN" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "ADMIRALTY" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "CULT_OF_HESH" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "SPARK_BARONS" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "BILEBROKERS" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "JAKES" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "BANDITS" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "RISE" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "RENTORIAN" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "DELTREAN" ) )
    self.faction_filter_group:AddFilter( Widget.AgentFactionFilterRow( "BOGGERS" ) )
    self.faction_filter_group:SetSelected( 0 )

    -- Add filters for a few other aspects
    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.AGENT_FILTER_OTHER" )
    self.other_filter_row = self.sidebar:AddFilterWidget( Widget.FilterOptionalRadioGroup() )
    -- self.other_filter_row:AddFilter( Widget.AgentBossFilterRow() )
    -- self.other_filter_row:AddFilter( Widget.AgentNonBossFilterRow() )
    self.other_filter_row:AddFilter( Widget.AgentUnknownFilterRow() )
    self.other_filter_row:SetSelected( 0 )

    self.sidebar:AddFilterHeader( LOC"UI.CARDCOMPENDIUM.PEOPLE_SORT" )
    self:AddSortingWidget()
end

function PeopleCompendiumWidget:GetDefaultFocus()

    for k, v in ipairs( self.agent_widgets ) do
        if v:IsShown() then 
            return v:GetDefaultFocus() 
        end
    end
end

function PeopleCompendiumWidget:OnVizChange(viz)
    if viz then
        self:StartUpdating()
    else
        self:StopUpdating()
    end
end

function PeopleCompendiumWidget:OnUpdate(dt)
    PeopleCompendiumWidget._base.OnUpdate(self, dt)
    for k,v in ipairs(self.agent_widgets) do
        if v:CheckForLoad() then
            -- skip this
        end
    end
end

function PeopleCompendiumWidget:HasSidebar()
    return true
end

function PeopleCompendiumWidget:AddSortingWidget()
    local sorting_options = {
        { txt = LOC"UI.CARDCOMPENDIUM.SORT_OPTION_AGENT_NAME", fn = function( w1, w2 ) return w1.agent_name < w2.agent_name end },
        { txt = LOC"UI.CARDCOMPENDIUM.SORT_OPTION_AGENT_TITLE", fn = function( w1, w2 ) return w1.agent_title < w2.agent_title end },
        { txt = LOC"UI.CARDCOMPENDIUM.SORT_OPTION_AGENT_FACTION", fn = function( w1, w2 ) return w1.faction_name < w2.faction_name end },
        { txt = LOC"UI.CARDCOMPENDIUM.SORT_OPTION_AGENT_UNLOCKABLE", fn = function( w1, w2 )
            local agent1, agent2 = w1.agent, w2.agent
            local progress1 = 0
            if agent1.is_locked then
                progress1 = 100
            else
                local unlocked = Screen.Compendium.ShowAllUnlocks() and agent1.unlockable or agent1.unlocked or 0
                progress1 = agent1.unlockable - unlocked
            end
            local progress2 = 0
            if agent2.is_locked then
                progress2 = 100
            else
                local unlocked = Screen.Compendium.ShowAllUnlocks() and agent2.unlockable or agent2.unlocked or 0
                progress2 = agent2.unlockable - unlocked
            end
            return progress1 > progress2
        end },
    }
    self.sidebar:AddSortingWidget( sorting_options, function( idx, option ) self:SortOrderSelected( option ) end )
end

function PeopleCompendiumWidget:SortOrderSelected( option )
    if option.fn then self.sorting_fn = option.fn end
    self:FilterPeopleWidgets( self.agent_widgets, self.sidebar:GetFilters() )
    self:LayoutContent()
end

function PeopleCompendiumWidget:SortPortraits()
    return table.stable_sort( self.agent_widgets, self.sorting_fn )
end

function PeopleCompendiumWidget:RefreshContent()
    PeopleCompendiumWidget._base.RefreshContent()

    if self.agent_widgets ~= nil then
        return -- Already constructed.
    end

    self.agent_widgets = {}

    -- Calculate the size of each widget
    self.scroll_inner_w = self.content_w - 40
    portrait_w = (self.scroll_inner_w-(spacing*(columns-1)))/columns
    portrait_h = 90

    local function OnLazyLoad( loader )
        local agent = loader.agent
        local widget = loader:AddChild( Widget.PeopleCompendiumPortrait( portrait_w, portrait_h, agent ) )
        if agent.unlockable then
            widget:SetProgress( Screen.Compendium.ShowAllUnlocks() and agent.unlockable or agent.unlocked or 0, agent.unlockable )
        end
        if agent.is_locked ~= nil and not Screen.Compendium.ShowAllUnlocks() then
            widget:SetLocked( agent.is_locked )
        end

        widget:SetOnClickFn( function() self:OnAgentSelect( agent, widget ) end )

        loader.portrait = widget

    end

    -- Add new portraits
    for k, agent in ipairs( self.agents ) do
        local loader = Widget.LazyLoadWidget( portrait_w, portrait_h, OnLazyLoad )
        loader.agent = agent
        local faction_id = agent:GetFactionID()
        local faction = Content.GetFaction( faction_id )
        loader.faction_name = faction and faction.name or ""
        loader.agent_name = agent:GetName() or agent:GetShortTitle() or ""
        loader.agent_title = agent:GetIdentity() or ""
        loader.search_string = loader.agent_title .. " " .. loader.agent_name

        table.insert( self.agent_widgets, loader )
    end
    self.scroll_root:AddScrollChildren( self.agent_widgets )

    -- Layout all the characters
    self:LayoutContent()

    -- Update filtering
    self.sidebar:RefreshFilters()

    return self
end

function PeopleCompendiumWidget:UpdateCount()
    self.agents = {}
    self.agent_counts = {}
    self.agent_counts["UNLOCKABLE"] = 0
    self.agent_counts["UNLOCKED"] = 0

    -- Get all the characters
    local all_defs = Content.GetAllCharacterDefs()
    for content_id, def in pairs( all_defs ) do
        if def.boss then
            -- skip, these go in the boss tab.
        elseif def.faction_id == PLAYER_FACTION then
            -- skip player defs.
        elseif def.hide_in_compendium then
            -- explicitly skipped.
        else
            local skins = Content.GetAllCharacterSkins( content_id )
            if skins then
                for i, skin in ipairs( skins ) do
                    local agent = Agent( content_id, skin )
                    table.insert( self.agents, agent )
                end
            
            elseif def.unique then
                local agent = Agent( content_id )
                table.insert( self.agents, agent )
            end
        end
    end

    -- Go through all the agents
    for k, agent in ipairs( self.agents ) do
        agent.unlocks_list = {}

        -- Check if this agent has a social boon unlockable
        if agent:GetSocialGraft( RELATIONSHIP.LOVED ) then
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1

            -- Has this boon been unlocked?
            if TheGame:GetGameProfile():HasUnlockedBoon( agent:GetUniqueID() ) then
                agent.unlocked = (agent.unlocked or 0) + 1
                self.agent_counts["UNLOCKED"] = self.agent_counts["UNLOCKED"] + 1
                table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_BOON" ) )
            else
                table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_INCOMPLETE", LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_BOON" ) )
            end
        end

        -- Check if this agent has a social bane unlockable
        if agent:GetSocialGraft( RELATIONSHIP.HATED ) then
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1

            -- Has this bane been unlocked?
            if TheGame:GetGameProfile():HasUnlockedBane( agent:GetUniqueID() ) then
                agent.unlocked = (agent.unlocked or 0) + 1
                self.agent_counts["UNLOCKED"] = self.agent_counts["UNLOCKED"] + 1
                table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_BANE" ) )
            else
                table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_INCOMPLETE", LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_BANE" ) )
            end
        end

        -- Check if this agent has been met
        if TheGame:GetGameProfile():HasMetCharacter( agent:GetUniqueID() ) or Screen.Compendium.ShowAllUnlocks() then
            agent.is_locked = false
            agent.unlocked = (agent.unlocked or 0) + 1
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1
            self.agent_counts["UNLOCKED"] = self.agent_counts["UNLOCKED"] + 1
            table.insert( agent.unlocks_list, 1, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_MET", agent:GetName() ) ) )
        else
            agent.is_locked = true
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1
        end

        -- Check if there are any lore blocks to show for this agent
        local lore_unlocked, lore_unlockables = 0, 0
        local lore_unlocks = agent:GetLoreUnlocks()
        -- Go through them and count them up
        for action, unlock_text in pairs( lore_unlocks ) do
            lore_unlockables = lore_unlockables + 1
            agent.unlockable = (agent.unlockable or 0) + 1
            self.agent_counts["UNLOCKABLE"] = self.agent_counts["UNLOCKABLE"] + 1
            -- Check if this has been achieved
            if TheGame:GetGameProfile():HasCustomAgentUnlock( agent:GetUniqueID(), action ) then
                lore_unlocked = lore_unlocked + 1
                agent.unlocked = (agent.unlocked or 0) + 1
                self.agent_counts["UNLOCKED"] = self.agent_counts["UNLOCKED"] + 1
            end
        end
        if lore_unlocked > 0 then
            -- Add these to the list
            table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_SUCCESS", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_LORE", lore_unlocked, lore_unlockables, agent:GetName() ) ) )
        elseif lore_unlockables > 0 then
            table.insert( agent.unlocks_list, loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_INCOMPLETE", loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT_LORE", lore_unlocked, lore_unlockables, agent:GetName() ) ) )
        end
    end
end

-- Returns the number of unlocked characters and the number of unlockable characters, to be displayed on the nav bar
function PeopleCompendiumWidget:GetCount()
    if self.agent_counts == nil then
        self:UpdateCount()
    end

    return Screen.Compendium.ShowAllUnlocks() and self.agent_counts["UNLOCKABLE"] or self.agent_counts["UNLOCKED"], self.agent_counts["UNLOCKABLE"]
end

-- Lays its content out
function PeopleCompendiumWidget:LayoutContent()
    PeopleCompendiumWidget._base.LayoutContent()

    self:SortPortraits()

    local xstart = portrait_w/2 + 20
    local ystart = portrait_h/2
    local idx = 1
    for i, widget in ipairs( self.agent_widgets ) do
        if widget:IsShown() then
            local column = (idx-1) % columns
            local row = math.floor( (idx-1) / columns )
            local x, y = xstart + column*(portrait_w+spacing),  ystart - row*(portrait_h+spacing) 
            widget:SetPos( x, y )
            idx = idx + 1
        end
    end

    self:ReNav()

    self.scroll_root:SetVirtualMargin( 20 )
    self.scroll_root:RefreshView()
    self.scroll_root:LayoutBounds( "after", "top", self.sidebar )
        :Offset( 8-CONTENT_LEFT_MARGIN, -1 )
    
    return self
end

function PeopleCompendiumWidget:OnFiltersChanged( filters )
    self:FilterPeopleWidgets( self.agent_widgets, filters )
    self:LayoutContent()
end

function PeopleCompendiumWidget:FilterPeopleWidgets( agent_widgets, filters )
    -- Go through each of the portraits
    for i, portrait in pairs( agent_widgets ) do

        -- Make this portrait visible by default
        portrait:Show()

        -- Go through each filter
        for j, filter in ipairs( filters ) do

            -- If this filter rejects the portrait, hide the portrait widget
            if filter:Filter( portrait.agent, portrait ) == false then portrait:Hide() end
        end
    end
end

function PeopleCompendiumWidget:OnAgentSelect( agent, agent_portrait )
    if not agent_portrait.is_locked then
        local details = Screen.PeopleCompendiumDetailsScreen()
        TheGame:FE():PushScreen( details )
        details:ShowAgent( agent )
    end
end

------------------------------------------------------------------------------------------
-- Shows a single agent portrait

local PeopleCompendiumPortrait = class( "Widget.PeopleCompendiumPortrait", Widget.Clickable )

local default_portrait_h = 100
local vertical_portrait_overshoot = 10

function PeopleCompendiumPortrait:init( w, h, agent )
    PeopleCompendiumPortrait._base.init( self )
    self.widget_w = w or 240
    self.widget_h = h or 90
    self.agent = agent

    -- Background panel
    self.bg_border = 30
    self.bg_border_scale = 0.5
    local nineslice_w, nineslice_h = self.widget_w + self.bg_border*2*self.bg_border_scale, self.widget_h + self.bg_border*2*self.bg_border_scale
    self.bg = self:AddChild( Widget.Panel( assets.bg ) )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :SetSize( nineslice_w, nineslice_h )
        :Bloom( 0.05 )
    self:SetHighlightWidget( self.bg )
    self.contents = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )

    -- Character portrait
    self.agent_portrait = self:AddChild( Widget.AgentPortrait( portrait_h, agent ) )

    self.gradient = self:AddChild( Widget.Panel( assets.gradient ) )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :SetSize( self.widget_h*0.5, nineslice_h )
        :SetHiddenBoundingBox( true )
        :UnblockMouse()
        :LayoutBounds( "left", nil, self.bg )

    -- Text
    self.agent_name = agent:GetName() or agent:GetShortTitle()
    self.title = self.contents:AddChild( Widget.Label("title", 30 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetText( self.agent_name )
        :Bloom(0.1)

    local faction_id = agent:GetFactionID()
    local faction = Content.GetFaction( faction_id )

    self.agent_title = agent:GetIdentity() or ""

    self.rank = self.contents:AddChild( Widget.Label("title", 22 ) )
        :SetText( self.agent_title )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.faction_icon = self.contents:AddChild( Widget.Image() )
        :SetSize( 18, 18 )
        :SetTexture( faction and faction.icon )
        :SetShown( faction and faction.icon ~= nil )
    self.subtitle = self.contents:AddChild( Widget.Label("body", 22 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.8 )
        :SetText( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_SUBTITLE" )
        :Bloom(0.05)
        :Hide()
    self.value = self.contents:AddChild( Widget.Label("body", 22 ))
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.8 )
        :SetText( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_PROGRESS", 0, 0 ) )
        :Bloom(0.05)
        :Hide()
    self.progress_bar = self.contents:AddChild( Widget.ProgressBar( self.widget_w, 3 ) )
        :OrientH()
        :SetSize( self.widget_w - self.widget_h - 10, 3 )
        :SetOutlineColour( 0x00000000 )
        :SetBGColour( 0x00000050 )
        :SetBarColour( UICOLOURS.SUBTITLE )
        :SetBarBloom( 0.05 )
        :Hide()

    self:ShowToolTipOnFocus( true )

    self:Layout()
end

function PeopleCompendiumPortrait:GetAgent()
    return self.agent
end

function PeopleCompendiumPortrait:GetName()
    return self.agent_name or ""
end

function PeopleCompendiumPortrait:GetTitle()
    return self.agent_title or ""
end

function PeopleCompendiumPortrait:SetLocked( locked )
    self.is_locked = locked

    if self.is_locked then
        if not self.lock_icon then
            self.lock_icon = self:AddChild( Widget.Image( assets.lock ) )
                :SetSize( 30, 30 )
                :SetTintColour( UICOLOURS.SUBTITLE )
                :Bloom( 0.05 )
        end
        self.agent_portrait:SetTintColour( 0x021A18FF )
            -- :SetTintAlpha( 0.8 )
        self.title:SetText( LOC"UI.CARDCOMPENDIUM.AGENT_LOCKED_NAME" )
        self.faction_icon:Hide()
        self.rank:Hide()
        self.subtitle:Hide()
        self.value:Hide()
        self.progress_bar:Hide()
        self.bg:SetSaturation( 0.2 )
        self:SetToolTip( LOC"UI.CARDCOMPENDIUM.AGENT_LOCKED_TT" )
    else
        self:SetToolTip( loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_UNLOCKS_TT", self.agent.unlocked or 0, self.agent.unlockable or 0, self:GetName(), table.concat( self.agent.unlocks_list, "\n" ) ) )
    end

    self:Layout()

    return self
end

function PeopleCompendiumPortrait:SetProgress( unlocked, unlockable )

    self.progress_unlocked = unlocked or 0
    self.progress_unlockable = unlockable or 0
    self.value:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.CARD_FILTER_SERIES_PROGRESS", unlocked, unlockable ) )
    self.progress_bar:SetProgress( unlocked/unlockable )
    self.subtitle:Show()
    self.value:Show()
    self.progress_bar:Show()

    self:Layout()

    return self
end

function PeopleCompendiumPortrait:GetProgress()
    return self.progress_unlocked, self.progress_unlockable
end

function PeopleCompendiumPortrait:Layout()

    if self.is_locked then
        if self.lock_icon then
            self.lock_icon:LayoutBounds( "right", "center", self.bg )
                :Offset( -20, 0 )
        end
        self.title:LayoutBounds( "left", "center", self.bg )
            :Offset( self.widget_h, 0 )
    else
        self.agent_portrait:LayoutBounds( "left", "bottom", self.bg )
            :Offset( 3, 0 )
        self.title:LayoutBounds( "left", "center", self.bg )
            :Offset( self.widget_h, self.value:IsShown() and 25 or 10 )
        self.faction_icon:LayoutBounds( "left", "below", self.title )
            :Offset( 0, 0 )
        self.rank:LayoutBounds( self.faction_icon:IsShown() and "after" or "left", "center", self.faction_icon )
            :Offset( self.faction_icon:IsShown() and 4 or 0, 1 )
        self.subtitle:LayoutBounds( "left", "below", self.faction_icon )
            :Offset( 0, 0 )
        self.value:LayoutBounds( nil, "bottom", self.subtitle )
            :LayoutBounds( "right", nil, self.bg )
            :Offset( -10, 0 )
        self.progress_bar:LayoutBounds( "right", "below", self.value )
            :Offset( 0, -1 )
    end

    return self
end

function PeopleCompendiumPortrait:UpdateImage()
    if ( self.hover or self.focus ) then
        if self.is_locked then
            AUDIO:PlayEvent( "event:/ui/card_codex/mouseover_locked_card" )
        end
        self.title:Bloom( 0.15 )
        self:ScaleTo( nil, 1.03, 0.2, easing.outQuad )
    else
        self.title:Bloom( 0.1 )
        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
    end
end

function PeopleCompendiumPortrait:OnGainFocus()
    PeopleCompendiumPortrait._base.OnGainFocus(self)
    self:UpdateImage()
end

function PeopleCompendiumPortrait:OnLoseFocus()
    PeopleCompendiumPortrait._base.OnLoseFocus(self)
    self:UpdateImage()
end

function PeopleCompendiumPortrait:OnGainHover()
    PeopleCompendiumPortrait._base.OnGainHover(self)
    self:UpdateImage()
end

function PeopleCompendiumPortrait:OnLoseHover()
    PeopleCompendiumPortrait._base.OnLoseHover(self)
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- Popup that shows a single person's details

local PeopleCompendiumDetailsScreen = class( "Screen.PeopleCompendiumDetailsScreen", Widget.Screen )

local PANEL_H = 600
local PANEL_W = 1200
local CONTENT_W = 1060
local PADDING = 20
local TITLES_H = 85
local CONTENT_H = PANEL_H - TITLES_H

PeopleCompendiumDetailsScreen.CONTROL_MAP = 
{
    {   
        control = Controls.Digital.GAMEPAD_LT,
        fn = function(self) self.social_details_panel.voice_block:OnClick() return true end,
        test = function(self) 
            return self.social_details_panel.voice_block:IsShown()
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.HEAR_VOICE", Controls.Digital.GAMEPAD_LT))
        end
    },
    {
        control = Controls.Digital.GAMEPAD_RT,
        test = function(self)
            return self.social_details_panel.death_loot:IsShown()
        end,
        fn = function(self)
            AUDIO:PlayEvent( "event:/ui/battle_deck/click_card_in_menu" )
            self.social_details_panel.death_loot:ShowCardInfoPopup()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CARD_INFO", Controls.Digital.GAMEPAD_RT))
        end
    },
    {
        control = Controls.Digital.MENU_CANCEL,
        fn = function(self)
            self:FadeOut()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.CLOSE", Controls.Digital.MENU_CANCEL))
        end
    },

    {   
        control = Controls.Digital.MENU_PREV,
        fn = function(self) self.social_tab:Click() return true end,
    },

    --[[{   
        control = Controls.Digital.MENU_NEXT,
        fn = function(self) if self.lore_tab:IsEnabled() then self.lore_tab:Click() end return true end,
    },--]]

}

function PeopleCompendiumDetailsScreen:init()
    PeopleCompendiumDetailsScreen._base.init( self )

    self:SetupUnderlay( true )
    self:SetAnchors( "center", "center" )
    self.is_popup = true

    -- Background
    self.panel = self:AddChild( Widget.WidePanel( PANEL_H ) )
        :PrepareAnimation()

    -- Hitbox
    self.hitbox = self:AddChild( Widget.SolidBox( PANEL_W, PANEL_H, 0xFFFF0010 ) )
        :Hide()

    -- Title area
    self.name = self:AddChild( Widget.Label("title", 48 ))
        :SetWordWrap( true )
        :SetAutoSize( CONTENT_W )
        :LeftAlign()
        :Bloom(0.1)
    self.faction_icon = self:AddChild( Widget.Image() )
        :SetSize( 18, 18 )
    self.rank = self:AddChild( Widget.Label("title", 36 ) )
        :SetWordWrap( true )
        :SetAutoSize( CONTENT_W )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.divider_top = self:AddChild( Widget.Image( assets.divider ) )
        :Bloom( 0.1 )

    -- Block with the character social details (bio and social grafts)
    self.social_details_panel = self:AddChild( Widget.AgentDetailsSocial( CONTENT_W, CONTENT_H-2 ) )
    self.social_details_panel:Hide()

    -- Block with the character lore details
    self.lore_details_panel = self:AddChild( Widget.AgentDetailsLore( CONTENT_W, CONTENT_H-2 ) )

    local tab_w = 250
    local tab_h = 66
    self.navigation = self:AddChild( Widget.TabNavigation( tab_w, tab_h, function(tab) self:OnNavChange(tab) end ) )
    self.social_tab = self.navigation:AddTab( Widget.ProgressTab( LOC"UI.CARDCOMPENDIUM.AGENT_SOCIAL_TAB", LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_SUBTITLE", "0%", 0, 0, UICOLOURS.SUBTITLE ) )
    --self.lore_tab = self.navigation:AddTab( Widget.ProgressTab( LOC"UI.CARDCOMPENDIUM.AGENT_LORE_TAB", LOC"UI.CARDCOMPENDIUM.CARD_FILTER_MINIGAME_SUBTITLE", "0%", 0, 0, UICOLOURS.SUBTITLE ) )
    self.navigation:SetCurrent( 1 )
    self.navigation:Hide() -- ccc: we dont use this damn thing, but I also don't quite want to delete it.
    -- Navigation tab labels
    self.navigation_label_left = self:AddChild( Widget.Label( "title", 30, LOC"UI.CARDCOMPENDIUM.CONTROLS_LABEL_TAB_LEFT" ) )
        :Bloom( 0.1 )
    self.navigation_label_right = self:AddChild( Widget.Label( "title", 30, LOC"UI.CARDCOMPENDIUM.CONTROLS_LABEL_TAB_RIGHT" ) )
        :Bloom( 0.1 )
    
    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )
    
    -- Character anim
    self.anim = self:AddChild( Widget.AnimatedCharacter() )
        :SetScale( -1, 1 )
    
    -- Continue button
    self.bottom_left = self:AddChild( Widget() ):SetAnchors( "left", "bottom" )
    self.button_back = self.bottom_left:AddChild( Widget.IconButton( LOC"UI.OVERLAYS.BACK", function() self:FadeOut() end ) )
        :SetIcon( global_images.cancel )
        :LayoutBounds( "left", "above", 60, 60 )
end

function PeopleCompendiumDetailsScreen:OnNavChange( tab )
    if tab == self.social_tab then
        self.social_details_panel:Show()
        self.lore_details_panel:Hide()
    end
    --[[elseif tab == self.lore_tab then
        self.social_details_panel:Hide()
        self.lore_details_panel:Show()
    end--]]
end

function PeopleCompendiumDetailsScreen:ShowAgent( agent )
    self:Refresh( agent )
    self.panel:FadeIn()

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
end

function PeopleCompendiumDetailsScreen:FadeOut()
    AUDIO:PlayEvent( "event:/ui/card_codex/card_inspect_close" )
    self.panel:FadeOut( function()
        self:GetFE():PopScreen( self )
    end )
end

function PeopleCompendiumDetailsScreen:Refresh( agent )
    -- Reset the navigation
    self.navigation:SetCurrent( 1 )

    -- Update the details
    self.name:SetText( agent:GetName() or agent:GetShortTitle() )
    local faction_id = agent:GetFactionID()
    local faction = Content.GetFaction( faction_id )
    self.faction_name = faction and faction.name

    self.agent_title = agent:GetIdentity()
    self.rank:SetText( self.agent_title )
    self.faction_icon:SetTexture( faction and faction.icon )
        :SetShown( faction and faction.icon ~= nil )
    self.divider_top:SetSize( CONTENT_W*1.5, 4 )

    -- Update the anim
    self.anim:Refresh( agent )

    -- Update the social details panel
    self.social_details_panel:Refresh( agent, self.anim ):Show()
    self.social_tab:SetProgress( self.social_details_panel:GetProgress() )

    -- Update the lore details panel
    self.lore_details_panel:Refresh( agent )
    local unlocked, unlockable = self.lore_details_panel:GetProgress()
    
--[[
    if unlockable == 0 then
        self.lore_tab:SetSubtitle( LOC"UI.CARDCOMPENDIUM.AGENT_LORE_TAB_NO_UNLOCKS" )
        self.lore_tab:SetValue( "" )
    else
        self.lore_tab:SetSubtitle( LOC"UI.CARDCOMPENDIUM.AGENT_LORE_TAB_SUBTITLE" )
        self.lore_tab:SetProgress( unlocked, unlockable )
    end
    self.lore_tab:SetEnabled( unlockable ~= 0 )
--]]
    -- Update the panel icon/colours
    if faction and faction.colour then
        self.panel:SetColour( faction.colour )
        self.divider_top:SetTintColour( faction.colour )
        self.rank:SetGlyphColour( faction.colour )
        self.anim:SetOutline( 4, faction.colour )
        self.navigation:SetBGColour( faction.colour )
    else
        self.panel:SetColour( UICOLOURS.SUBTITLE )
        self.divider_top:SetTintColour( UICOLOURS.SUBTITLE )
        self.rank:SetGlyphColour( UICOLOURS.SUBTITLE )
        self.anim:SetOutline( 4, UICOLOURS.SUBTITLE )
        self.navigation:SetBGColour( UICOLOURS.SUBTITLE )
    end
    if faction and faction.icon then
        self.panel:SetIcon( faction.icon, nil, 0.8 )
    else
        self.panel:SetIcon( nil )
    end

    -- Layout things
    self.hitbox:SetPos( 0, 0 )
    local anim_dx, anim_dy = 0, 0
    if type(agent.compendium_offset) == "table" then
        anim_dx, anim_dy = table.unpack( agent.compendium_offset )
    end

    self.anim:SetPos( 0, 0 )
        :LayoutBounds( "right", nil, self.hitbox )
        :Offset( -120 + anim_dx, -RES_Y/2+200 + anim_dy )
    self.name:LayoutBounds( "left", "top", self.hitbox )
        :Offset( 0, -10 )
    self.faction_icon:LayoutBounds( "left", "below", self.name )
        :Offset( 0, 1 )
    self.rank:LayoutBounds( self.faction_icon:IsShown() and "after" or "left", "center", self.faction_icon )
        :Offset( self.faction_icon:IsShown() and 4 or 0, 1 )
    self.divider_top:SetScale( 1, 1 )
        :LayoutBounds( "center", nil, self.rank )
        :LayoutBounds( nil, "top", self.hitbox )
        :Offset( CONTENT_W*0.40, -TITLES_H )
        :SetScale( -1, 1 )
    self.social_details_panel:LayoutBounds( "left", "bottom", self.hitbox )
        :Offset( 0, 0 )
    self.lore_details_panel:LayoutBounds( "left", "bottom", self.hitbox )
        :Offset( 0, 0 )
    self.navigation:LayoutBounds( "right", nil, self.social_details_panel )
        :LayoutBounds( nil, "above", self.divider_top )
        :Offset( -250, -1.8 )
end

function PeopleCompendiumDetailsScreen:OnControlModeChange( cm, device_id )
    PeopleCompendiumDetailsScreen._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.bottom_left:Show()
        self.navigation_label_left:Hide()
        self.navigation_label_right:Hide()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.bottom_left:Show()
        self.navigation_label_left:Hide()
        self.navigation_label_right:Hide()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.bottom_left:Hide()
        if #self.navigation:GetTabs() > 1 then
            self.navigation_label_left:Show()
            self.navigation_label_right:Show()
        else
            self.navigation_label_left:Hide()
            self.navigation_label_right:Hide()
        end
    end
    self.navigation_label_left:LayoutBounds( "before", "center", self.navigation ):Offset( -15, 0 )
    self.navigation_label_right:LayoutBounds( "after", "center", self.navigation ):Offset( 15, 0 )
end

------------------------------------------------------------------------------------------
-- Shows a character details panel

local AgentDetailsLore = class( "Widget.AgentDetailsLore", Widget )

function AgentDetailsLore:init( width, height )
    AgentDetailsLore._base.init( self )
    self.width = width
    self.height = height

    -- Add scrollable area for the lore blocks
    self.scroll_root = self:AddChild( Widget.ScrollPanel() )
        :SetSize( self.width, self.height )
        :ShowBar( SCROLLBAR.IF_NEEDED )
    self.block_w = width - 34 -- 34 is the scroll bar width
    -- Add container for the blocks
    self.block_container = self.scroll_root:AddScrollChild( Widget() )

    self.unlocked = 0
    self.unlockable = 0
end

function AgentDetailsLore:Refresh( agent )

    -- Remove old blocks, if any
    self.block_container:DestroyAllChildren()

    -- Reset the count
    self.unlocked = 0
    self.unlockable = 0

    -- Check if there are any lore blocks to show for this agent
    local lore_unlocks = agent:GetLoreUnlocks()
    -- Go through them and add them to the list
    for action, unlock_text in pairs( lore_unlocks ) do
        self.unlockable = self.unlockable + 1
        -- Check if this has been done
        local is_unlocked = TheGame:GetGameProfile():HasCustomAgentUnlock( agent:GetUniqueID(), action ) or Screen.Compendium.ShowAllUnlocks()
        if is_unlocked then self.unlocked = self.unlocked + 1 end
        self.block_container:AddChild( Widget.AgentLoreBlock( self.block_w, action, unlock_text, is_unlocked ) )
            :LayoutBounds( "center", "below" )
            :Offset( 0, -10 )
    end

    -- Layout the scroll area
    self.block_container:SetPos( self.block_w/2, 0 )
    self.scroll_root:SetVirtualMargin( 15 )
    self.scroll_root:RefreshView()

    return self
end

function AgentDetailsLore:GetProgress()
    return self.unlocked, self.unlockable
end

------------------------------------------------------------------------------------------
-- Shows a lore block of text

local AgentLoreBlock = class( "Widget.AgentLoreBlock", Widget )

function AgentLoreBlock:init( width, action, text, unlocked )
    AgentLoreBlock._base.init( self )

    local left_block_w = 50
    local text_overlap_left = 10
    local text_padding = 15
    local max_text_w = width - left_block_w - text_padding*2 + text_overlap_left

    text = ( unlocked and text ) or LOC"UI.CARDCOMPENDIUM.AGENT_LORE_LOCKED_TEXT"
    unlocked = unlocked or false

    self.icon_block = self:AddChild( Widget.Image( assets.lore_edge_block ) )
        :SetSize( left_block_w, left_block_w )
        :SetTintColour( unlocked and UICOLOURS.SUBTITLE or 0x8B8B8BFF )
        :Bloom( 0.05 )
    local bg_border = 50
    local bg_border_scale = 0.5
    self.icon_block_edge = self:AddChild( Widget.Panel( assets.lore_edge ) )
        :SetTintColour( unlocked and UICOLOURS.SUBTITLE or 0x8B8B8BFF )
        :SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
        :SetNineSliceBorderScale( bg_border_scale )
        :Bloom( 0.05 )
    self.icon = self:AddChild( Widget.Image( assets.lore_edge_block ) )
        :SetSize( 18, 18 )
        :SetTexture( unlocked and assets.checkmark or assets.locked )
        :SetTintColour( 0x021A18FF )
    self.text_bg = self:AddChild( Widget.Panel( assets.lore_panel ) )
        :SetTintColour( unlocked and UICOLOURS.SUBTITLE or 0x8B8B8BFF )
        :SetTintAlpha( unlocked and 0.1 or 0.1 )
        :SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
        :SetNineSliceBorderScale( bg_border_scale )
        :Bloom( 0.05 )

    self.text_label = self:AddChild( Widget.Label( "body", 24 ) )
        :SetWordWrap( true )
        :SetAutoSize( max_text_w )
        :SetText( text )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
        :SetHoverLinkFn( UIHelpers.ItemTooltips )

    -- Get text size
    local t_w, t_h = self.text_label:GetSize()
    local block_w = max_text_w + text_padding*2
    local block_h = t_h + text_padding*2
    local nineslice_w, nineslice_h = left_block_w, block_h + bg_border*2*bg_border_scale
    self.icon_block_edge:SetSize( nineslice_w, nineslice_h )
    nineslice_w, nineslice_h = block_w + bg_border*2*bg_border_scale, block_h + bg_border*2*bg_border_scale
    self.text_bg:SetSize( nineslice_w, nineslice_h )

    -- Layout things
    self.icon_block_edge:LayoutBounds( "left", "top", self.icon_block )
    self.icon:LayoutBounds( "center", "center", self.icon_block )
        :Offset( -8, 7 )
    self.text_bg:LayoutBounds( "left", "top", self.icon_block_edge )
        :Offset( left_block_w-text_overlap_left, 0 )
    self.text_label:LayoutBounds( "left", "top", self.text_bg )
        :Offset( text_padding, -text_padding )
end

------------------------------------------------------------------------------------------
-- Shows a character details panel

local AgentDetailsSocial = class( "Widget.AgentDetailsSocial", Widget )

function AgentDetailsSocial:init( width, height )
    AgentDetailsSocial._base.init( self )

    self.width = width or 300
    self.height = height or 300
    self.padding = 30
    self.column_w = (self.width-self.padding)/2

    self.hitbox = self:AddChild( Widget.SolidBox( width, height, 0xFF00FF00 ) )

    -- Character bio
    self.desc = self:AddChild( Widget.Label("body", 32 ) )
        :SetWordWrap( true )
        :SetAutoSize( width )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
        :SetHoverLinkFn( UIHelpers.ItemTooltips )

    self.unlocked = 0
    self.unlockable = 0

    -- Agent voice
    self.voice_block = self:AddChild( Widget.AgentVoiceBlock( self.column_w, 142 ))

    -- Boss prestige advancement
    self.advancement_block = self:AddChild( Widget.BossAdvancementBlock( self.column_w, 142 ))

    -- Death loot info
    self.death_loot = self:AddChild( Widget.AgentDeathLoot( self.column_w, 150 ))

    -- Social grafts
    self.social_boon = self:AddChild( Widget.AgentSocialGraft( self.column_w, 100) )
    self.social_bane = self:AddChild( Widget.AgentSocialGraft( self.column_w, 100) )
end

function AgentDetailsSocial:Refresh( agent, animated_character )

    -- Align the character bio
    self.desc:SetText( agent:GetBio() )
        :LayoutBounds( "left", "top", self.hitbox )
        :Offset( 0, -self.padding )

    self.unlocked = 0
    self.unlockable = 0

    -- Set this agent's death loot
    self.death_loot:Refresh( agent )
        :LayoutBounds( "left", "bottom", self.hitbox )
        :Offset( 0, 0 )

    -- Set this agent's bane, if any
    local bane_id = agent:GetSocialGraft( RELATIONSHIP.HATED )
    local bane_graft = bane_id and GraftInstance( bane_id )
    if bane_graft then
        local is_unlocked = TheGame:GetGameProfile():HasUnlockedBane( agent:GetUniqueID() ) or Screen.Compendium.ShowAllUnlocks()
        if is_unlocked then self.unlocked = self.unlocked + 1 end
        bane_graft.is_locked = not is_unlocked
        self.unlockable = self.unlockable + 1
    end
    self.social_bane:Refresh( bane_graft )
        :LayoutBounds( "left", "bottom", self.hitbox )
        :Offset( self.column_w+self.padding, self.padding )

    -- Set this agent's boon, if any
    local boon_id = agent:GetSocialGraft( RELATIONSHIP.LOVED )
    local boon_graft = boon_id and GraftInstance( boon_id )
    if boon_graft then
        local is_unlocked = TheGame:GetGameProfile():HasUnlockedBoon( agent:GetUniqueID() ) or Screen.Compendium.ShowAllUnlocks()
        if is_unlocked then self.unlocked = self.unlocked + 1 end
        boon_graft.is_locked = not is_unlocked
        self.unlockable = self.unlockable + 1
    end
    self.social_boon:Refresh( boon_graft )
        :LayoutBounds( "left", "above", self.social_bane )
        :Offset( 0, self.padding + (agent.boss and 70 or 30) ) -- Make room for the prestige info

    
    -- Set this agent's voice tester block
    self.voice_block:Refresh( agent, animated_character )
        :LayoutBounds( "left", nil, self.death_loot )
        :LayoutBounds( nil, "top", self.social_boon )

    if agent.voice_actor then
        self.voice_block:Show()
    else
        self.voice_block:Hide()
    end

    -- Set this agent's boss advancement
    self.advancement_block:Refresh( agent )
        :LayoutBounds( "left", nil, self.voice_block )
    -- Let's try to position this between the voice and death loot blocks
    local voice_x, voice_y = self.voice_block:GetPos()
    local voice_w, voice_h = self.voice_block:GetSize()
    local death_x, death_y = self.death_loot:GetPos()
    local death_w, death_h = self.death_loot:GetSize()
    local voice_bottom_y = voice_y - voice_h/2
    local death_top_y = death_y + death_h/2
    local vertical_center = (voice_bottom_y + death_top_y)/2
    self.advancement_block:LayoutBounds( nil, "center", 0, vertical_center -10 )

    return self
end

function AgentDetailsSocial:GetProgress()
    return self.unlocked, self.unlockable
end

------------------------------------------------------------------------------------------
-- Shows a boss's current prestige advancement

local BossAdvancementBlock = class( "Widget.BossAdvancementBlock", Widget )

function BossAdvancementBlock:init( width, height )
    BossAdvancementBlock._base.init( self )

    -- Text
    self.label_icon = self:AddChild( Widget.Image( engine.asset.Texture( "UI/ic_prestigepoints.tex" ) ) )
        :SetSize( 19, 19 )
        -- :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.05 )
    self.label = self:AddChild( Widget.Label( "title", 32 ) )
        :SetText( LOC"UI.CARDCOMPENDIUM.BOSS_ADVANCEMENT_TITLE_EMPTY" )
        :SetGlyphColour( UICOLOURS.ADVANCEMENT )
        :Bloom( 0.05 )
    self.desc = self:AddChild( Widget.Label( "body", 28 ) )
        :SetText( LOC"UI.CARDCOMPENDIUM.BOSS_ADVANCEMENT_DESC_EMPTY" )
        :SetGlyphColour( UICOLOURS.BODYTEXT )
        :LeftAlign()
        :SetWordWrap( true )
        :OverrideLineHeight( 18 )
        :Bloom( 0.05 )
    
    self:SetSize( width, height )
end

function BossAdvancementBlock:SetSize( width, height )
    self.width = width or 200
    self.height = height or 200

    -- Layout our contents
    self.desc:SetAutoSize( self.width - 24 )

    return self
end

function BossAdvancementBlock:Refresh( agent )
    if agent and agent.boss and agent:GetContentID() then
        self.agent = agent

        -- Get prestige level
        local prestige = TheGame:GetGameProfile():GetBossAdvancement( self.agent.alt_boss_id or self.agent:GetContentID() )

        -- Update texts
        if prestige then
            self.label:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.BOSS_ADVANCEMENT_TITLE", prestige ) )
            
            if prestige == #DEFAULT_ADVANCEMENT then
                self.desc:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.BOSS_ADVANCEMENT_DESC_MAX", self.agent:LocTable(), prestige ) )
            else
                self.desc:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.BOSS_ADVANCEMENT_DESC", self.agent:LocTable(), prestige ) )
            end
        else
            self.label:SetText( LOC"UI.CARDCOMPENDIUM.BOSS_ADVANCEMENT_TITLE_EMPTY" )
            self.desc:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.BOSS_ADVANCEMENT_DESC_EMPTY", self.agent:LocTable() ) )
        end
    
        -- Align text
        self.label_icon:SetPos( 0, 0 )
        self.label:LayoutBounds( "after", "center", self.label_icon )
            :Offset( 5, 1 )
        self.desc:LayoutBounds( "left", "below", self.label )
            :Offset( 0, 4 )

        self:Show()
    else
        self:Hide()
    end

    return self
end

function BossAdvancementBlock:OnClick()
    if self.animated_character then
        self.animated_character:Talk(false)
    end
end


------------------------------------------------------------------------------------------
-- Allows the player to hear an agent's voice

local AgentVoiceBlock = class( "Widget.AgentVoiceBlock", Widget )

function AgentVoiceBlock:init( width, height )
    AgentVoiceBlock._base.init( self )

    -- Text
    self.label_icon = self:AddChild( Widget.Image( assets.voice ) )
        :SetSize( 19, 19 )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.05 )
    self.label = self:AddChild( Widget.Label( "title", 32 ) )
        :SetText( LOC"UI.CARDCOMPENDIUM.AGENT_VOICE_TITLE" )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.05 )
    self.voice_btn = self:AddChild( Widget.PlayVoiceButton( width ) )
        :SetOnClickFn( function() self:OnClick() end )
    
    self:SetSize( width, height )
end

function AgentVoiceBlock:SetSize( width, height )
    self.width = width or 200
    self.height = height or 200

    -- Layout our contents
    self.voice_btn:SetSize( self.width - 24 )

    return self
end

function AgentVoiceBlock:Refresh( agent, animated_character )

    if agent and animated_character then
        self.agent = agent
        self.animated_character = animated_character

        -- Update texts
        self.voice_btn:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.AGENT_VOICE_BTN", agent:GetName() ) )
    
        -- Align text
        self.label_icon:SetPos( 0, 0 )
        self.label:LayoutBounds( "after", "center", self.label_icon )
            :Offset( 5, 1 )
        self.voice_btn
            :LayoutBounds( "left", "below", self.label )
            :Offset( 0, -11 )

        self:Show()
    else
        self:Hide()
    end

    return self
end

function AgentVoiceBlock:OnClick()
    if self.animated_character then
        self.animated_character:Talk(false)
    end
end

------------------------------------------------------------------------------------------
-- Shows a voice button

local PlayVoiceButton = class("Widget.PlayVoiceButton", Widget.Clickable)

function PlayVoiceButton:init( width, height )
    PlayVoiceButton._base.init(self)

    -- Button bg
    self.button_bg_border = 20
    self.button_bg_border_scale = 0.5
    self.button_bg = self:AddChild( Widget.Panel( assets.voice_button ) )
    self.button_bg:SetNineSliceCoords( self.button_bg_border, self.button_bg_border, self.button_bg_border, self.button_bg_border )
        :SetNineSliceBorderScale( self.button_bg_border_scale )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.2 )

    -- Button label
    self.button_text = self:AddChild( Widget.Label( "body", 28, "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )

    -- Arrow icon
    self.button_arrow = self:AddChild( Widget.Image( assets.voice_button_arrow, 12, 12 ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )

    self:SetSize( width, height )
end

function PlayVoiceButton:SetSize( w, h )
    self.width = w or self.width or 200
    self.height = h or self.height or 32

    local nineslice_w, nineslice_h = self.width + self.button_bg_border*2*self.button_bg_border_scale, self.height + self.button_bg_border*2*self.button_bg_border_scale
    self.button_bg:SetSize( nineslice_w, nineslice_h )

    self.button_text:LayoutBounds( "left", "center", self.button_bg ):Offset( 15, 1 )
    self.button_arrow:LayoutBounds( "right", "center", self.button_bg ):Offset( -10, 0 )

    return self
end

function PlayVoiceButton:SetText( text )
    self.button_text:SetText( text )
        :LayoutBounds( "left", "center", self.button_bg ):Offset( 15, 1 )

    return self
end

function PlayVoiceButton:UpdateImage()
    if ( self.hover or self.focus ) then
        self.button_text:SetGlyphColour( UICOLOURS.TITLE )
        self.button_arrow:SetTintColour( UICOLOURS.TITLE )
        self.button_bg:SetTintColour( UICOLOURS.TITLE )
            :Bloom( 0.25 )
        if self.focus then self:ScaleTo( nil, 1.03, 0.2, easing.outQuad ) end
    else
        self.button_text:SetGlyphColour( UICOLOURS.SUBTITLE )
        self.button_arrow:SetTintColour( UICOLOURS.SUBTITLE )
        self.button_bg:SetTintColour( UICOLOURS.SUBTITLE )
            :Bloom( 0.2 )
        self:ScaleTo( nil, 1, 0.2, easing.outQuad )
    end
end

function PlayVoiceButton:OnGainFocus()
    PlayVoiceButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function PlayVoiceButton:OnLoseFocus()
    PlayVoiceButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function PlayVoiceButton:OnGainHover()
    PlayVoiceButton._base.OnGainHover(self)
    self:UpdateImage()
end

function PlayVoiceButton:OnLoseHover()
    PlayVoiceButton._base.OnLoseHover(self)
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- Shows a character's death loot

local AgentDeathLoot = class( "Widget.AgentDeathLoot", Widget.Clickable )

local CARD_SCALE = 0.55

function AgentDeathLoot:init( width, height )
    AgentDeathLoot._base.init( self )
    
    self.hitbox = self:AddChild( Widget.SolidBox( width, height, 0x00FF0000 ) )
    self.hitbox.hover_check = true
    self.card_pos_x, self.card_pos_y = 0, 0

    -- Text
    self.label_icon = self:AddChild( Widget.Image( assets.death_loot ) )
        :SetSize( 19, 19 )
        :SetTintColour( 0xFFCA78ff )
        :Bloom( 0.05 )
    self.label = self:AddChild( Widget.Label( "title", 32 ) )
        :SetText( LOC"UI.CARDCOMPENDIUM.DEATH_LOOT_TITLE" )
        :SetGlyphColour( 0xFFCA78ff )
        :Bloom( 0.05 )
    self.desc = self:AddChild( Widget.Label("body", 28 ))
        :SetText( LOC"UI.CARDCOMPENDIUM.DEATH_LOOT_DESC" )
        :SetGlyphColour( UICOLOURS.BODYTEXT )
        :LeftAlign()
        :SetWordWrap( true )
        :OverrideLineHeight( 18 )
        :Bloom( 0.05 )

    -- Card
    self.card_container = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.battle_card_widget = self.card_container:AddChild( Widget.BattleCard() )
        :SetScale( CARD_SCALE )
        :SetFocusable( true )
        :ShowToolTipOnFocus( true )
        :ShowInfoOnClick()
        :ShowInfoOnRightClick()
    self.negotiation_card_widget = self.card_container:AddChild( Widget.NegotiationCard() )
        :SetScale( CARD_SCALE )
        :SetFocusable( true )
        :ShowToolTipOnFocus( true )
        :ShowInfoOnClick()
        :ShowInfoOnRightClick()
    

    self:SetFocusable( false )
    self:SetSize( width, height )
end

function AgentDeathLoot:SetSize( width, height )
    self.width = width or 200
    self.height = height or 200

    -- Layout our contents
    self.hitbox:SetSize( self.width, self.height )
    self.desc:SetAutoSize( self.width*0.55 )
    self.card_container:SetScissor( -self.width/2, -self.height/2, self.width+100, self.height+450)

    -- Refresh
    self:Refresh( self.agent )

    return self
end

function AgentDeathLoot:ShowCardInfoPopup()
    if self.negotiation_card_widget.card then
        self.negotiation_card_widget:ShowCardInfoPopup()
    elseif self.battle_card_widget.card then
        self.battle_card_widget:ShowCardInfoPopup()
    end
end

function AgentDeathLoot:Refresh( agent )

    if agent and agent:GetDeathItem() then
        self.agent = agent

        -- Update texts
        self.desc:SetText( loc.format( LOC"UI.CARDCOMPENDIUM.DEATH_LOOT_DESC", agent:GetName() ) )
    
        -- Align text
        self.label_icon:LayoutBounds( "left", "top", self.hitbox )
            :Offset( 0, 0 )
        self.label:LayoutBounds( "after", "center", self.label_icon )
            :Offset( 5, 1 )
        self.desc
            :LayoutBounds( "left", "below", self.label )
            :Offset( 0, 4 )

        -- Update card
        self.loot_id = agent:GetDeathItem()

        local carddef = Content.GetBattleCard( self.loot_id ) or Content.GetNegotiationCard( self.loot_id )
        local card = carddef.is_negotiation and Negotiation.Card( self.loot_id ) or Battle.Card( self.loot_id )
        if is_instance( card, Negotiation.Card ) then
            self:Show()
            self.negotiation_card_widget:RefreshCard( card )
                -- :SetOutline( 0xFFCA78ff )
                :Show()
                :SetRotation( 0 )
                :LayoutBounds( "right", "top", self.hitbox )
                :Offset( -20, 40 )
                :SetRotation( -8*DEGREES )
                -- :SetToolTipClass( Widget.TooltipNegotiationCard )
                -- :SetToolTip( { CARD = card } )
            self.card_pos_x, self.card_pos_y = self.negotiation_card_widget:GetPos()
            self.battle_card_widget:Hide()
        elseif is_instance( card, Battle.Card ) then
            self:Show()
            self.battle_card_widget:RefreshCard( card )
                -- :SetOutline( 0xFFCA78ff )
                :Show()
                :SetRotation( 0 )
                :LayoutBounds( "right", "top", self.hitbox )
                :Offset( -20, 40 )
                :SetRotation( -8*DEGREES )
                -- :SetToolTipClass( Widget.TooltipBattleCard )
                -- :SetToolTip( { CARD = card } )
            self.card_pos_x, self.card_pos_y = self.battle_card_widget:GetPos()
            self.negotiation_card_widget:Hide()
        else
            self:Hide()
        end
    else
        self:Hide()
    end

    return self
end

function AgentDeathLoot:GetKeywords()
    local keywords = {}
    if self.negotiation_card_widget:IsShown() then
        table.arrayadd( keywords, self.negotiation_card_widget:GetCardFeatures() )
    elseif self.battle_card_widget:IsShown() then
        table.arrayadd( keywords, self.battle_card_widget:GetCardFeatures() )
    end
    return keywords
end

function AgentDeathLoot:UpdateImage()
    if ( self.hover or self.focus ) then
        self.label_icon:Bloom( 0.1 )
        self.label:Bloom( 0.1 )
        if self.battle_card_widget:IsShown() then
            self.battle_card_widget:ScaleTo( nil, CARD_SCALE*1.1, 0.3, easing.outCubic )
            self.battle_card_widget:RotateTo( 0, 0.3, easing.outCubic )
            self.battle_card_widget:MoveTo( self.card_pos_x, self.card_pos_y+120, 0.2, easing.outCubic )
            self.battle_card_widget:SetOutline( 0xFFCA78ff, true )
        end
        if self.negotiation_card_widget:IsShown() then
            self.negotiation_card_widget:ScaleTo( nil, CARD_SCALE*1.1, 0.3, easing.outCubic )
            self.negotiation_card_widget:RotateTo( 0, 0.3, easing.outCubic )
            self.negotiation_card_widget:MoveTo( self.card_pos_x, self.card_pos_y+120, 0.2, easing.outCubic )
            self.negotiation_card_widget:SetOutline( 0xFFCA78ff, true )
        end
    else
        self.label_icon:Bloom( 0.05 )
        self.label:Bloom( 0.05 )
        if self.battle_card_widget:IsShown() then
            self.battle_card_widget:ScaleTo( nil, CARD_SCALE, 0.1 )
            self.battle_card_widget:RotateTo( -8*DEGREES, 0.1 )
            self.battle_card_widget:MoveTo( self.card_pos_x, self.card_pos_y, 0.1 )
            self.battle_card_widget:SetOutline( nil, true )
        end
        if self.negotiation_card_widget:IsShown() then
            self.negotiation_card_widget:ScaleTo( nil, CARD_SCALE, 0.1 )
            self.negotiation_card_widget:RotateTo( -8*DEGREES, 0.1 )
            self.negotiation_card_widget:MoveTo( self.card_pos_x, self.card_pos_y, 0.1 )
            self.negotiation_card_widget:SetOutline( nil, true )
        end
    end
end

function AgentDeathLoot:OnGainFocus()
    AgentDeathLoot._base.OnGainFocus(self)
    self:UpdateImage()
end

function AgentDeathLoot:OnLoseFocus()
    AgentDeathLoot._base.OnLoseFocus(self)
    self:UpdateImage()
end

function AgentDeathLoot:OnGainHover()
    AgentDeathLoot._base.OnGainHover(self)
    self:UpdateImage()
end

function AgentDeathLoot:OnLoseHover()
    AgentDeathLoot._base.OnLoseHover(self)
    self:UpdateImage()
end

------------------------------------------------------------------------------------------
-- Shows a character's social graft

local AgentSocialGraft = class( "Widget.AgentSocialGraft", Widget.Clickable )

local PADDING = 15
local ICON_SIZE = 0

function AgentSocialGraft:init( width, icon_size )
    AgentSocialGraft._base.init( self )
    self.hover_check = true

    width = width or 200
    icon_size = icon_size or 90

    -- Add our title label
    self.label_icon = self:AddChild( Widget.Image( assets.checkmark ) )
        :SetSize( 19, 19 )
        :Bloom( 0.05 )
    self.label = self:AddChild( Widget.Label( "title", 32 ) )
        :SetText( LOC"UI.RELATIONSHIP_SCREEN.BOON_TITLE" )
        :SetGlyphColour( UICOLOURS.BONUS )
        :Bloom( 0.05 )

    -- Contents!
    self.icon_frame = self:AddChild( Widget.Panel( assets.inner_frame ) )
        :Bloom( 0.1 )
    self.icon_glow = self:AddChild( Widget.Image( assets.glow ) )
        :SetHiddenBoundingBox( true )
    self.icon = self:AddChild( Widget.Image() )
        :Bloom( 0.05 )
    self.text_content = self:AddChild( Widget() )
    self.title = self.text_content:AddChild( Widget.Label("title", 32 ))
        :SetGlyphColour( UICOLOURS.TITLE )
        :LeftAlign()
        :SetWordWrap( true )
        :OverrideLineHeight( 22 )
        :Bloom( 0.05 )
    self.subtitle = self.text_content:AddChild( Widget.Label("body", 28 ))
        :SetGlyphColour( UICOLOURS.BODYTEXT )
        :LeftAlign()
        :SetWordWrap( true )
        :OverrideLineHeight( 18 )
        :Bloom( 0.05 )

    self:SetSize( width, icon_size )
end

function AgentSocialGraft:SetSize( width, icon_size )
    self.width = width
    self.icon_size = icon_size

    -- Layout our contents
    self.icon_frame:SetSize( self.icon_size, self.icon_size )
        :LayoutBounds( "left", "top", self.bg ):Offset( PADDING+2, -PADDING )
    self.icon_glow:SetSize( self.icon_size*1.5, self.icon_size*1.5 )
        :LayoutBounds( "center", "center", self.icon_frame )
    self.icon:SetSize( self.icon_size, self.icon_size )
        :LayoutBounds( "center", "center", self.icon_frame )
    self.title:SetAutoSize( self.width - self.icon_size - PADDING*2 )
    self.subtitle:SetAutoSize( self.width - self.icon_size - PADDING*2 )

    -- Refresh
    self:Refresh( self.boon )
end

function AgentSocialGraft:Refresh( graft )
    if graft then
        local graft_def = graft:GetDef() or graft
        -- Set appropriate colors!
        if graft:GetType() == GRAFT_TYPE.NEGOTIATION then
            self.icon_glow:SetTintColour( UICOLOURS.NEGOTIATION )
                :SetTintAlpha( 0.4 )
            self.icon_frame:SetTintColour( UICOLOURS.NEGOTIATION )
        elseif graft:GetType() == GRAFT_TYPE.COMBAT then
            self.icon_glow:SetTintColour( UICOLOURS.FIGHT )
                :SetTintAlpha( 0.4 )
            self.icon_frame:SetTintColour( UICOLOURS.FIGHT )
        elseif graft:GetType() == GRAFT_TYPE.SOCIAL and graft_def.is_good then
            self.label:SetText( LOC"UI.CARDCOMPENDIUM.BOON_TITLE" )
                :SetGlyphColour( UICOLOURS.BONUS )
            self.label_icon:SetTintColour( UICOLOURS.BONUS )
            self.title:SetGlyphColour( UICOLOURS.BONUS )
            self.icon_glow:SetTintColour( UICOLOURS.BONUS )
                :SetTintAlpha( 0.4 )
            self.icon_frame:SetTintColour( UICOLOURS.BONUS )
        elseif graft:GetType() == GRAFT_TYPE.SOCIAL and not graft_def.is_good then
            self.label:SetText( LOC"UI.CARDCOMPENDIUM.BANE_TITLE" )
                :SetGlyphColour( UICOLOURS.PENALTY )
            self.label_icon:SetTintColour( UICOLOURS.PENALTY )
            self.title:SetGlyphColour( UICOLOURS.PENALTY )
            self.icon_glow:SetTintColour( UICOLOURS.PENALTY )
                :SetTintAlpha( 0.4 )
            self.icon_frame:SetTintColour( UICOLOURS.PENALTY )
        end
        self.graft = graft
        self.icon:SetTexture( graft:GetImage() )
        if not graft.is_locked or Screen.Compendium.ShowAllUnlocks() then
            self.label_icon:SetTexture( assets.checkmark )
            self.title:SetText( graft:GetName() )
                :SetSaturation(1)
                :SetTintAlpha(1)
            self.subtitle:SetText( graft:GetDesc() )
                :SetSaturation(1)
                :SetTintAlpha(1)
            self.icon:SetSaturation(1)
                :SetTintColour(1,1,1,1)
            self.icon_frame:SetSaturation(1)
            self.icon_glow:SetSaturation(1)
        else
            self.label_icon:SetTexture( assets.locked )
            self.title:SetText( graft_def.is_good and LOC"UI.CARDCOMPENDIUM.BOON_UNKNOWN_NAME" or LOC"UI.CARDCOMPENDIUM.BANE_UNKNOWN_NAME" )
                :SetSaturation(0)
                :SetTintAlpha(.6)
            self.subtitle:SetText( graft_def.is_good and LOC"UI.CARDCOMPENDIUM.BOON_UNKNOWN_DESC" or LOC"UI.CARDCOMPENDIUM.BANE_UNKNOWN_DESC" )
                :SetSaturation(0)
                :SetTintAlpha(.5)
            self.icon:SetSaturation(0)
                :SetTintColour(0,0,0,0.4)
            self.icon_frame:SetSaturation(0)
            self.icon_glow:SetSaturation(0)
        end
        -- Align text
        self.label_icon:LayoutBounds( "left", "above", self.icon )
            :Offset( 0, 12 )
        self.label:LayoutBounds( "after", "center", self.label_icon )
            :Offset( 5, 1 )
        self.subtitle
            :LayoutBounds( "left", "below", self.title )
            :Offset( 0, 4 )
        self.text_content
            :LayoutBounds( "after", "top", self.icon )
            :Offset( PADDING, 7 )
        self:Show()
    else
        self:Hide()
    end

    return self
end

