require "ui/widget"
require "ui/widgets/opinion_indicator"
require "ui/widgets/roleicon"
require "ui/widgets/clickable"
require "ui/widgets/tooltip_aspect"

local assets =
{
    WHEEL = engine.asset.Texture("UI/location_wheel.tex"),
    WHEEL_GAP = engine.asset.Texture("UI/location_wheel_gap.tex"),
    QUEST = engine.asset.Texture("icons/role_icon_quest.tex"),
    SUPPORT = engine.asset.Texture("icons/support_icon.tex"),
}

local SCALE_MIN = 0.4
local SCALE_MAX = 1
local WHEEL_SIZE = 300
local ROLE_SIZE = 90
local ICON_SIZE = 50
local ICON_SCALE_MIN = 1
local ICON_SCALE_MAX = 1
local ICON_STARTING_ANGLE = 90
local ICON_STARTING_ANGLE_GAP = 36
local ICON_STARTING_ANGLE_GAP_FOCUS = 3
local ICON_ANGLE_GAP = 11
local ICON_ANGLE_GAP_FOCUS = 8
local RADIUS = WHEEL_SIZE / 2 - 4

local PlaxNameplate = class( "Widget.PlaxNameplate", Widget.Clickable )

PlaxNameplate.CONTROL_MAP =
{
    {
        control = Controls.Digital.MENU_ACCEPT, 
        fn = function(self)
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.TALK_TO", Controls.Digital.MENU_ACCEPT, self.agent))
        end
    },
}

local CHARACTER_ROLE_DETAILS = 
{
    PLAYER = {
        icon = engine.asset.Texture("icons/role_icon_bystander.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.PLAYER_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.PLAYER_TOOLTIP" end
    },
    [CHARACTER_ROLES.PROPRIETOR] = {
        icon = engine.asset.Texture("icons/role_icon_proprietor.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.PROPRIETOR_LABEL" end,
        tooltip = function( agent ) return loc.format( LOC"UI.PLAX_NAMEPLATE.PROPRIETOR_TOOLTIP", agent ) end
    },
    [CHARACTER_ROLES.GUARD] = {
        icon = engine.asset.Texture("icons/role_icon_guard.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.GUARDS_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.GUARDS_TOOLTIP" end
    },
    [CHARACTER_ROLES.PATRON] = {
        icon = engine.asset.Texture("icons/role_icon_patron.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.PATRON_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.PATRON_TOOLTIP" end
    },
    [CHARACTER_ROLES.VENDOR] = {
        icon = engine.asset.Texture("icons/role_icon_shopkeeper.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.VENDOR_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.VENDOR_TOOLTIP" end
    },
    [CHARACTER_ROLES.VISITOR] = {
        icon = engine.asset.Texture("icons/role_icon_visitor.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.VISITOR_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.VISITOR_TOOLTIP" end
    },
    [CHARACTER_ROLES.PRISONER] = {
        icon = engine.asset.Texture("icons/role_icon_visitor.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.PRISONER_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.PRISONER_TOOLTIP" end
    },
    [CHARACTER_ROLES.LABOUR] = {
        icon = engine.asset.Texture("icons/role_icon_labour.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.LABOUR_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.LABOUR_TOOLTIP" end
    },
    [CHARACTER_ROLES.WAITING_PET] = {
        icon = engine.asset.Texture("icons/role_icon_pet.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.WAITING_PET_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.WAITING_PET_TOOLTIP" end
    },
    [CHARACTER_ROLES.JOB_BOARD] = {
        icon = engine.asset.Texture("icons/role_icon_jobboard.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.JOB_BOARD_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.JOB_BOARD_TOOLTIP" end
    },
    [CHARACTER_ROLES.RESTING] = {
        icon = engine.asset.Texture("icons/role_icon_bystander.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.RESTING_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.RESTING_TOOLTIP" end
    },
    [CHARACTER_ROLES.HEALER] = {
        icon = engine.asset.Texture("icons/role_icon_healer.tex"),
        label = function() return LOC"UI.PLAX_NAMEPLATE.HEALER_LABEL" end,
        tooltip = function() return LOC"UI.PLAX_NAMEPLATE.HEALER_TOOLTIP" end
    },
}

function PlaxNameplate:init()
    PlaxNameplate._base.init( self )

    self.shadow = self:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex") ) )
        :SetHiddenBoundingBox( true )
        :SetTintColour( 0x021A1860 )
        :SetTintAlpha( 0 )

    self.scaled_content = self:AddChild( Widget() )
    self.wheel = self.scaled_content:AddChild( Widget.Image( assets.WHEEL, WHEEL_SIZE, WHEEL_SIZE ) ):Bloom( 0.1 )
    self.wheel:SetPos( 0, 0 )

    self.icons = self.scaled_content:AddChild( Widget() )
    self.icons_left = {} -- Quest and opinion icons
    self.icons_right = {} -- Conversation options

    self.role_icon = self.scaled_content:AddChild( Widget.RoleIcon( assets.QUEST, ROLE_SIZE ) ):ShowAura( true )

    self.name_label = self:AddChild( Widget.Label( "title", 28, "" ) )
        :SetHiddenBoundingBox( true )
        :EnableOutline( SMALL_TEXT_OUTLINE )
        :Bloom(0.1)
    self.role_label = self:AddChild( Widget.Label("title", 22, "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetHoverLinkGlyphColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.7 )
        :EnableOutline( SMALL_TEXT_OUTLINE )
        :Bloom( 0.1 )

    self.opinion_icon = self.icons:AddChild( Widget.OpinionIndicator( ICON_SIZE ) ):ShowBg( true ):Bloom( 0.05 )
    table.insert( self.icons_left, self.opinion_icon )
    self.support_icon = self.icons:AddChild( Widget.SupportIcon( ICON_SIZE ) ):Hide()
    table.insert( self.icons_left, self.support_icon )

    self.expanded = false -- True if the name and icons are expanded and visible
end

function PlaxNameplate:OnAdded()
    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
end

function PlaxNameplate:OnControlModeChange( cm, device_id )
    PlaxNameplate._base.OnControlModeChange( self, cm, device_id )
end

function PlaxNameplate:OnScreenModeChange( sm )
    PlaxNameplate._base.OnScreenModeChange( self, sm )

    -- Scale the icons
    for k, v in ipairs( self.icons_left ) do
        v:SetLayoutScale( LAYOUT_SCALE[sm] )
    end
    for k, v in ipairs( self.icons_right ) do
        v:SetLayoutScale( LAYOUT_SCALE[sm] )
    end
    self.role_icon:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.role_label:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.name_label:SetLayoutScale( LAYOUT_SCALE[sm] )
    self:Layout()
end

function PlaxNameplate:SetComparison(txt, col)
    self.name_label:Show()
    self.name_label:Bloom(0.0)
    self.comparison_text:SetText(txt)
    self:Layout()
end

function PlaxNameplate:RefreshNameplate( agent, scale )
    self.agent = agent

    local quest_marks = self.agent:CollectQuestMarks(nil, true)
    for k,v in pairs(self.icons_right) do
        v:Remove()
    end
    table.clear(self.icons_right)

    for k, v in ipairs(quest_marks) do
        local icon = v.quest:GetMarkData( v.objectives )
        local w =  self.icons:AddChild( Widget.Image( icon, ICON_SIZE, ICON_SIZE ) )
            :SetToolTipClass( Widget.TooltipQuest )
            :SetToolTip( { quest = v.quest, objectives = v.objectives } )
            :LayoutBounds("after", "center")
        table.insert(self.icons_right, w)
    end

    -- Update the contents to the character_widget's scaling
    self.new_scale = remap( scale or 1, 0.1, 0.5, SCALE_MIN, SCALE_MAX )
    self.icon_scale = remap( scale or 1, 0.1, 0.5, ICON_SCALE_MIN, ICON_SCALE_MAX )
    self.scaled_content:SetScale( self.new_scale, self.new_scale )
    
    -- Show the agent's opinion of you, if strong
    local player = TheGame:GetGameState():GetPlayerAgent()
    self.opinion_icon:UpdateOpinion(agent)
    if agent:GetRelationship() == RELATIONSHIP.NEUTRAL and (agent.social_connections == nil or not agent.social_connections:HasOpinionEvents( player )) then
        self.opinion_icon:Hide()
    else
        self.opinion_icon:Show()
    end
    self.opinion_icon:Bloom( 0 )

    -- Update the agent's name
    self.name_label:SetText( agent:GetName() )

    -- Set the default color for the name
    self.name_label:SetGlyphColour( UICOLOURS.SUBTITLE )

    -- Check if the agent is buttered up or not
    self.support_icon:RefreshSupport( self.agent )

    -- Check if this agent performs a role at this location: owner, shopkeep, or guard
    self.role_icon:Hide()
    self.role_label:SetText( "" )
    local details
    if agent:IsPlayer() then
        details = CHARACTER_ROLE_DETAILS["PLAYER"]
    elseif CHARACTER_ROLE_DETAILS[agent:GetRoleAtLocation()] then
        details = CHARACTER_ROLE_DETAILS[agent:GetRoleAtLocation()]
    else
        details = CHARACTER_ROLE_DETAILS[CHARACTER_ROLES.VISITOR]
    end
    if details then
        self.role_icon:SetTexture( details.icon )
        self.role_icon:SetToolTip( loc.format( details.tooltip( agent ), agent ) )
        self.role_icon:Show()
        self.role_label:SetText( loc.format( details.label(), agent ) )
        self.wheel:SetTexture( assets.WHEEL_GAP )
    end

    -- If this agent has a quest, highlight them
    if quest_marks and #quest_marks > 0 then
        self.role_icon:SetTintColour( UICOLOURS.QUEST )
        self.role_label:SetGlyphColour( UICOLOURS.QUEST )
        self.name_label:SetGlyphColour( UICOLOURS.QUEST )
        self.role_icon:SetTintColour( UICOLOURS.QUEST )
        self.wheel:SetTintColour( UICOLOURS.QUEST )
    else
        self.role_icon:SetTintColour( UICOLOURS.SUBTITLE )
        self.role_label:SetGlyphColour( UICOLOURS.SUBTITLE )
        self.name_label:SetGlyphColour( UICOLOURS.SUBTITLE )
        self.role_icon:SetTintColour( UICOLOURS.SUBTITLE )
        self.wheel:SetTintColour( UICOLOURS.SUBTITLE )
    end

    -- Refresh shadow size
    local w, h = self.name_label:GetSize()
    self.shadow:SetSize( w+300, h+300 )

    self:RefreshWheel()

    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
    self:Layout()
end

function PlaxNameplate:RefreshWheel()
    local has_icons = false
    for k, v in ipairs( self.icons_left ) do
        if v:IsShown() then has_icons = true end
    end
    for k, v in ipairs( self.icons_right ) do
        if v:IsShown() then has_icons = true end
    end

    self.wheel:SetShown( has_icons or self.expanded )
    return self
end

function PlaxNameplate:Layout( t )
    t = t or 0 -- The delta between 0 (unexpanded) and 1 (fully expanded)

    -- Layout role icon and name
    self.role_icon:LayoutBounds( "center", "center", self.wheel ):Offset( 0, RADIUS )
        :SetScale( 1 + 0.1*t )
    if t==0 then
        self.name_label:LayoutBounds( "center", "above", self.role_icon ):Offset( 0, 10 )
        self.name_label.default_x, self.name_label.default_y = self.name_label:GetPos()
        self.role_label:LayoutBounds( "center", "above", self.name_label )
            :Offset( 0, -7 )
        self.role_label.default_x, self.role_label.default_y = self.role_label:GetPos()
    end
    local y = easing.linear(t, self.name_label.default_y-40, 40, 1)
    self.name_label:SetPos( self.name_label.default_x, y )
        :SetTintAlpha( t )
    local y = easing.linear(t, self.role_label.default_y-40, 40, 1)
    self.role_label:SetPos( self.role_label.default_x, y )
        :SetTintAlpha( t*0.7 )
    self.shadow:LayoutBounds( "center", "center", self.name_label )
        :Offset( 0, -40 )
        :SetTintAlpha( t*0.25 )
    
    -- Layout the icons on each side of the wheel
    local i = 0
    for k, v in ipairs( self.icons_left ) do
        if v:IsShown() then
            local s = self.icon_scale or 1
            local x = math.cos( ( ICON_STARTING_ANGLE + ICON_STARTING_ANGLE_GAP + ICON_STARTING_ANGLE_GAP_FOCUS*s*t + ICON_ANGLE_GAP*s*i + ICON_ANGLE_GAP_FOCUS*s*t*i ) * DEGREES ) * RADIUS
            local y = math.sin( ( ICON_STARTING_ANGLE + ICON_STARTING_ANGLE_GAP + ICON_STARTING_ANGLE_GAP_FOCUS*s*t + ICON_ANGLE_GAP*s*i + ICON_ANGLE_GAP_FOCUS*s*t*i ) * DEGREES ) * RADIUS
            v:SetScale( self.icon_scale, self.icon_scale )
            v:SetPos( x, y )
            v:SendToBack()
            i = i + 1
        end
    end

    i = 0
    for k, v in ipairs( self.icons_right ) do
        if v:IsShown() then
            local s = self.icon_scale or 1
            local x = math.cos( ( ICON_STARTING_ANGLE - ICON_STARTING_ANGLE_GAP - ICON_STARTING_ANGLE_GAP_FOCUS*s*t - ICON_ANGLE_GAP*s*i - ICON_ANGLE_GAP_FOCUS*s*t*i ) * DEGREES ) * RADIUS
            local y = math.sin( ( ICON_STARTING_ANGLE - ICON_STARTING_ANGLE_GAP - ICON_STARTING_ANGLE_GAP_FOCUS*s*t - ICON_ANGLE_GAP*s*i - ICON_ANGLE_GAP_FOCUS*s*t*i ) * DEGREES ) * RADIUS
            v:SetScale( self.icon_scale, self.icon_scale )
            v:SetPos( x, y )
            v:SendToBack()
            i = i + 1
        end
    end

end

-- When an anim is focused/hovered, it notifies this function
function PlaxNameplate:SetCharacterFocused( focused )
    self.character_focused = focused
    self:UpdateImage()
    return self
end

function PlaxNameplate:ShowName( visible )
    self.show_name = visible or false
    self:UpdateImage()
    return self
end

-- When the focus/hover state of the plate changes, it notifies the locationscreen
function PlaxNameplate:SetOnHoverFn( fn )
    self.hover_fn = fn
    return self
end

function PlaxNameplate:UpdateImage()
    -- print( self.agent.id, self.hover and "+hover" or "-hover", self.focus and "+focus" or "-focus", self.character_focused and "+character_focused" or "-character_focused" )
    if not self.expanded and ( self.hover or self.focus or self.character_focused or self.show_name ) then
        if self.hover_fn then self.hover_fn( self.agent ) end
        self.focus_anim = self:RunUpdater( Updater.Ease( function(v) self:Layout( v ) end, 0, 1, 0.15, easing.outQuad ) )
        self.role_icon:GainHover()
        self.expanded = true
    end
    if self.expanded and not ( self.hover or self.focus or self.character_focused or self.show_name ) then
        if self.hover_fn then self.hover_fn( nil ) end
        if self.focus_anim then self:StopUpdater( self.focus_anim ) end
        self.focus_anim = self:RunUpdater( Updater.Ease( function(v) self:Layout(v) end, 1, 0, 0.15, easing.outQuad ) )
        self.role_icon:LoseHover()
        self.expanded = false
    end
    self:RefreshWheel()
    return self
end

function PlaxNameplate:OnGainFocus()
    PlaxNameplate._base.OnGainFocus(self)
    self:UpdateImage()
end

function PlaxNameplate:OnLoseFocus()
    PlaxNameplate._base.OnLoseFocus(self)
    self.character_focused = false
    self:UpdateImage()
end

function PlaxNameplate:OnGainHover()
    PlaxNameplate._base.OnGainHover(self)
    self:UpdateImage()
end

function PlaxNameplate:OnLoseHover()
    PlaxNameplate._base.OnLoseHover(self)
    self.character_focused = false
    self:UpdateImage()
end


-------------------------------------------------------------------------------
-- Widget wrapping a Buttered-up Icon

local SupportIcon = class( "Widget.SupportIcon", Widget )

function SupportIcon:init( size )
    SupportIcon._base.init( self )
    self.hover_check = true
    self.tex = assets.SUPPORT
    size = size or 30
    self.icon = self:AddChild( Widget.Image( self.tex, size, size )):SetBloom(0.06)
end

function SupportIcon:RefreshSupport( agent )
    local tt = {}

    local intimidated = agent:GetAspect( "intimidated" )
    local paidoff = agent:GetAspect( "paidoff" )
    local bribed = agent:GetAspect( "bribed" )

    if intimidated then
        self.tex = intimidated:GetTexture()
        table.insert( tt, intimidated:FormatDesc() )
    end

    if bribed then
        self.tex = bribed:GetTexture()
        table.insert( tt, bribed:FormatDesc() )
    end

    if paidoff then
        if bribed == nil then
            self.tex = paidoff:GetTexture()
            table.insert( tt, paidoff:FormatDesc() )
        elseif bribed:GetStacks() == 1 and paidoff:GetStacks() > 1 then
            self.tex = paidoff:GetTexture()
            table.insert( tt, paidoff:FormatDesc() )
        end
    end

    if #tt == 0 then
        self:Hide()
    else
        self:SetToolTip( table.concat( tt, "\n" ))
        self.icon:SetTexture(self.tex)
        self:Show()
    end
end
