

------------------------------------------------------------------------------------------
-- Shows a Points floater for Daily Challenge screens

local PtsFloater = class("Widget.PtsFloater", Widget)

function PtsFloater:init( delta, font_size )
    PtsFloater._base.init(self)

    self.font_size = font_size or FONT_SIZE.SMALL_TEXT
    self.delta = delta or 0

    self.value = self:AddChild( Widget.Label( "body", self.font_size, loc.format( LOC"UI.DAILY_OVER_SCREEN.PTS_FLOATER_VALUE", math.round(self.delta) ) ) )
        :SetGlyphColour( UICOLOURS.OBJECTIVE )
        :Bloom( 0.08 )
    self.label = self:AddChild( Widget.Label( "title", self.font_size*0.6, LOC"UI.DAILY_OVER_SCREEN.PTS_FLOATER" ) )
        :SetGlyphColour( UICOLOURS.OBJECTIVE )
        :Bloom( 0.08 )
        :LayoutBounds( "after", "top", self.value ):Offset( self.font_size*0.1, -self.font_size*0.12 )
end

function PtsFloater:SetPts( delta )
    
    self.delta = delta or 0
    if math.floor(self.delta) == 0 then
        self:Hide()
    else
        self.value:SetText( loc.format( LOC"UI.DAILY_OVER_SCREEN.PTS_FLOATER_VALUE", math.round(self.delta) ) )
        self.label:LayoutBounds( "after", "top", self.value ):Offset( self.font_size*0.1, -self.font_size*0.12 )
        self:Show()
    end
    return self
end

function PtsFloater:SetProgress( value, total )
    self.value:SetText( loc.format( LOC"UI.DAILY_OVER_SCREEN.POINTS", math.min(value, total), total ) )
    self.label:LayoutBounds( "after", "top", self.value ):Offset( self.font_size*0.1, -self.font_size*0.12 )
    return self
end

function PtsFloater:AnimateIn()
    if self:IsShown() == false then return end
    local duration = 0.1

    self:SetTintAlpha( 0 )
    local target_x, target_y = self:GetPos()
    
    self:RunUpdater(
        Updater.Parallel{
            Updater.Ease( function(v) self:SetTintAlpha(v) end, 0, 1, duration, easing.linear),
            Updater.Ease( function(v) self:SetScale(v,v) end, 1.2, 1, duration, easing.linear),
            Updater.Ease2D( function(x,y) self:SetPos(x,y) end, target_x, target_y + 30, target_x, target_y, duration, easing.linear),
        }
    )
    return self
end


------------------------------------------------------------------------------------------
-- A widget showing a score during daily challenges. Large text, white and italic

local HighScoreWidget = class("Widget.HighScoreWidget", Widget.PtsFloater)

function HighScoreWidget:init( value, font_size, layout_fn )
    HighScoreWidget._base.init(self, value, font_size )
    self.layout_fn = layout_fn
    self.value:SetGlyphColour( UICOLOURS.WHITE )
        :Bloom( 0.1 )
        :SetOutlineColour( 0x000000FF )
        :EnableOutline( 0.25 )

    self.label:SetText( LOC"UI.DAILY_OVER_SCREEN.HIGH_SCORE_FLOATER" )
        :SetGlyphColour( UICOLOURS.WHITE )
        :Bloom( 0.1 )
        :SetOutlineColour( 0x000000FF )
        :EnableOutline( 0.1 )

    self.current_value = 0
    if value then self:SetValue( value ) end
    
    self.value_accum = 0
    self.queue = {}
end

-- Used if we want this widget to be layed out in a specific manner, independently of its height
-- Desired height is the height it'll report
-- Vertical alignment is how the number will be aligned in relation to the desired height ( "top", "centre", "bottom" )
function HighScoreWidget:OverrideHeight( desired_height, vertical_offset )
    self.desired_height = desired_height or 30
    self.vertical_offset = vertical_offset
    self.hitbox = self:AddChild( Widget.SolidBox( 40, self.desired_height, 0xff00ff00 ) )
    self.contents = self:AddChild( Widget() )
    self.value:Reparent( self.contents )
    self.label:Reparent( self.contents )
    self:SetValue( self.current_value )
    local w, h = self.contents:GetSize()
    self.hitbox:SetSize( w, self.desired_height )
    self.contents:SetHiddenBoundingBox( true )
        :LayoutBounds( "center", "center", self.hitbox )
        :Offset( 0, self.vertical_offset )
    return self
end

function HighScoreWidget:SetValue( value )
    local old_value_digits = string.len(tostring(math.floor(self.current_value)))
    local new_value_digits = string.len(tostring(math.floor(value)))

    self.current_value = value
    -- Update text
    self.value:SetText( loc.format( LOC"UI.DAILY_CHALLENGE_SCREEN.HIGH_SCORE_VALUE", math.round(value) ) )
    self.value:LayoutBounds( "before", "center", 0, 0)
    self.label:LayoutBounds( "after", "top", self.value ):Offset( self.font_size*0.1, -self.font_size*0.14 )
    if self.hitbox then
        self.contents:SetHiddenBoundingBox( false )
        local w, h = self.contents:GetSize()
        self.hitbox:SetSize( w, self.desired_height )
        self.contents:SetHiddenBoundingBox( true )
            :LayoutBounds( "center", "center", self.hitbox )
            :Offset( 0, self.vertical_offset )
    end

    if old_value_digits ~= new_value_digits then
        if self.layout_fn then
            self.layout_fn()
        end

    end
    return self
end

function HighScoreWidget:IsBusy( )
    return self.target_value ~= nil or #self.queue > 0 
end

function HighScoreWidget:AnimateToValue( start_value, value )
    self:SetValue(start_value)
    if not self.target_value then
        
        self:StartUpdating()
        self:ScaleTo(1, 1.1, .2)
        self.value_accum = 0
        --"event:/ui/game_over_screen/score_tick_up"
    end

    self.target_value = value
    
    local delta = self.target_value - self.current_value
    self.anim_time = 1.5 * ( .1 + .9*clamp(delta / 500, 0, 1) )
    self.anim_start_value = self.current_value
    self.t = 0

    -- Update text
    self.value:SetText( loc.format( LOC"UI.DAILY_CHALLENGE_SCREEN.HIGH_SCORE_VALUE", math.round(value) ) )
    self.label:LayoutBounds( "after", "top", self.value ):Offset( self.font_size*0.1, -self.font_size*0.14 )


    return self
end

function HighScoreWidget:ShowGain( old_score, new_score, score_event )

    table.insert(self.queue, {old_score = old_score, new_score = new_score, delta = new_score - old_score, score_event = score_event})
    self:ShowIndicators()
    --
end

function HighScoreWidget:OnUpdate(dt)
    
    HighScoreWidget._base.OnUpdate(self, dt)
    

    if self.target_value then
        self.t = self.t + dt
        
        local new_value = clamp( math.round(easing.outQuad(self.t, self.anim_start_value, self.target_value - self.anim_start_value, self.anim_time)), self.anim_start_value, self.target_value)
        local delta = new_value - self.current_value
        
        
        self.value_accum = self.value_accum - delta 

        if self.value_accum < 0 then
            AUDIO:PlayEvent( "event:/ui/game_over_screen/score_tick_up")
            self.value_accum = 10
        end

        self:SetValue(new_value)
        if self.t >= self.anim_time then
            self.target_value = nil
            self:ScaleTo(1.1, 1, .1)
            AUDIO:PlayEvent( "event:/ui/game_over_screen/score_tick_up_done" )
        end
    end

    if not self.target_value then
        self:StopUpdating()
    end
end

function HighScoreWidget:ShowIndicators()
    if not self.current_indicator and #self.queue > 0  then
        local item = table.remove(self.queue)

        if item.score_event then
            local adjusted, raw, multiplier = item.score_event:GetValue()
            local name = item.score_event:GetName()
            local txt = loc.format("{1}: {2#points}", name, {raw, multiplier} )

            self.current_indicator = Widget.Label( "title", 30, txt)
            self:AddChild(self.current_indicator)
            self.current_indicator:LayoutBounds("after", "center", self.value):Offset(100, 0)
            self.current_indicator:SetHiddenBoundingBox(true)
        end
        
        self:RunUpdater(
            Updater.Series{
                Updater.Do(function() self:AnimateToValue(item.old_score, item.new_score) end),
                Updater.Ease( function(v) if self.current_indicator then self.current_indicator:SetTintAlpha(v) end end, 0, 1, 1, easing.linear),
                Updater.Wait( 1.5 ),
                Updater.Ease( function(v)  if self.current_indicator then self.current_indicator:SetTintAlpha(v) end end, 1, 0, .5, easing.linear),
                
                Updater.Do(function() 
                    if self.current_indicator then
                        self.current_indicator:Remove()
                        self.current_indicator = nil 
                    end
                    self:ShowIndicators() 
                end)
            }
        )
    end
end

require "ui/widgets/tooltip"
require "ui/screens/relationshipsscreen"

----------------------------------------------------------------
-- High Score Widget tooltip

local TooltipHighScoreWidget = class("Widget.TooltipHighScoreWidget", Widget.Tooltip )

function TooltipHighScoreWidget:init()
    TooltipHighScoreWidget._base.init(self)
    
    self.block_w = 260
    self.text_w = self.block_w - 60 - 10 -- 60 is the width of the pts_floater with a 3 digit score (+110pts, for instance)
    self.list_container = self:AddChild( Widget() )
end

function TooltipHighScoreWidget:DoLayout( high_score_widget )
    self.list_container:DestroyAllChildren()

    local gamestate = TheGame:GetGameState()


    local approach = gamestate.scorekeeper:GetScoreComponents()
    
    local displayed_total = 0
    for _, approach_item in ipairs(approach) do
        

        if approach_item.total > 0 then
            local block = self.list_container:AddChild( Widget() )
            -- local block = self.list_container:AddChild( Widget.Label( "title", FONT_SIZE.BUTTON, cmp.GetText(cmp, num, mult) ) )
            displayed_total = displayed_total + approach_item.raw
            block.title = block:AddChild( Widget.Label( "title", FONT_SIZE.BUTTON, approach_item.txt))
                :SetGlyphColour( UICOLOURS.SUBTITLE )
                :SetWordWrap( true )
                :SetAutoSize( self.text_w )
                :OverrideLineHeight( 18 )
                :LeftAlign()
                :Bloom( 0.08 )

            block.exp_floater = block:AddChild( Widget.PtsFloater( approach_item.raw ) )
                :LayoutBounds( "before", "bottom", block.title )
                :Offset( self.block_w, 0 )
        end
    end

    local score_mult = gamestate.scorekeeper:GetScoreMultiplier()
    if score_mult > 1 then
        local block = self.list_container:AddChild( Widget() )
        
        local total_score = gamestate.scorekeeper:GetScore()
        local bonus = total_score - displayed_total

        local txt = loc.format( LOC"UI.DAILY_CHALLENGE_SCREEN.HIGH_SCORE_TOOLTIP_DIFFICULTY", score_mult - 1 )
        block.title = block:AddChild( Widget.Label( "title", FONT_SIZE.BUTTON, txt))
            :SetGlyphColour( UICOLOURS.SUBTITLE )
            :SetWordWrap( true )
            :SetAutoSize( self.text_w )
            :OverrideLineHeight( 18 )
            :LeftAlign()
            :Bloom( 0.08 )

            block.exp_floater = block:AddChild( Widget.PtsFloater( bonus) )
                :LayoutBounds( "before", "bottom", block.title )
                :Offset( self.block_w, 0 )

    end

    self.list_container:StackChildren( 6 )
    self.bg:SizeToWidgets( SPACING.M1, self.list_container )
end

------------------------------------------------------------------------------------------
-- Shows an EXP floater for the Game Over Screen

local ExpFloater = class("Widget.ExpFloater", Widget)

function ExpFloater:init( delta, font_size )
    ExpFloater._base.init(self)

    self.font_size = font_size or FONT_SIZE.SMALL_TEXT
    self.delta = delta or 0

    self.value = self:AddChild( Widget.Label( "body", self.font_size, loc.format( LOC"UI.GAME_OVER_SCREEN.EXP_FLOATER_VALUE", math.round(self.delta) ) ) )
        :SetGlyphColour( UICOLOURS.EXP )
        :Bloom( 0.08 )
    self.label = self:AddChild( Widget.Label( "title", self.font_size*0.6, LOC"UI.GAME_OVER_SCREEN.EXP_FLOATER" ) )
        :SetGlyphColour( UICOLOURS.EXP )
        :Bloom( 0.08 )
        :LayoutBounds( "after", "top", self.value ):Offset( self.font_size*0.1, -self.font_size*0.12 )
end

function ExpFloater:SetEXP( delta )
    
    self.delta = delta or 0
    if math.floor(self.delta) == 0 then
        self:Hide()
    else
        self.value:SetText( loc.format( LOC"UI.GAME_OVER_SCREEN.EXP_FLOATER_VALUE", math.round(self.delta) ) )
        self.label:LayoutBounds( "after", "top", self.value ):Offset( self.font_size*0.1, -self.font_size*0.12 )
        self:Show()
    end
    return self
end

function ExpFloater:SetProgress( value, total )
    self.value:SetText( loc.format( LOC"UI.GAME_OVER_SCREEN.POINTS", math.min(value, total), total ) )
    self.label:LayoutBounds( "after", "top", self.value ):Offset( self.font_size*0.1, -self.font_size*0.12 )
    return self
end

function ExpFloater:AnimateIn()
    if self:IsShown() == false then return end
    local duration = 0.1

    self:SetTintAlpha( 0 )
    local target_x, target_y = self:GetPos()
    
    self:RunUpdater(
        Updater.Parallel{
            Updater.Ease( function(v) self:SetTintAlpha(v) end, 0, 1, duration, easing.linear),
            Updater.Ease( function(v) self:SetScale(v,v) end, 1.2, 1, duration, easing.linear),
            Updater.Ease2D( function(x,y) self:SetPos(x,y) end, target_x, target_y + 30, target_x, target_y, duration, easing.linear),
        }
    )
    return self
end
