require "ui/widget"
require "ui/widgets/anim"
require "ui/widgets/image"
require "ui/widgets/particlesystem"



local fade_tex =  engine.asset.Texture("splash/inkBleed.tex")

--local fx = engine.asset.Effect('FSQDISSOLVE', 'shaders/composite.vs', 'shaders/composite.ps')


local Presentation = class("Widget.Presentation", Widget)

function Presentation:init(w,h)
	Presentation._base.init(self, engine.rendering.FullScreenQuad())
	self.w, self.h = w,h

	self.model:SetBlendMode(BLEND_MODE.BLENDED)
	self.model:SetTexture(1, fade_tex)
	self.model:SetEffect(global_shaders.UI_DISSOLVE)
	self.dissolve = engine.rendering.ShaderConstantValue("dissolve", 0)
	self.model:SetShaderConstant(self.dissolve)
	self:SetSize(w,h)
end


function Presentation:GetBoundingBox()
	return -self.w/2, -self.h/2, self.w/2, self.h/2
end

function Presentation:SetSize(w,h)
	self.w, self.h = w, h
	self:SetScale(w/2, h/2)
	self:InvalidateBBox()
	return self
end

function Presentation:GetSize()
	return self.w, self.h
end

function Presentation:Dissolve(img, time, fn)
	self.model:SetTexture(0, img)
	self.fade_time = 0
	self.total_fade_time = time or .67
	self.dissolve:SetValue(1)
	self:Show()
	self.fn = fn
	self:StartUpdating()
	return self
end


function Presentation:OnUpdate(dt)
    Presentation._base.OnUpdate( self, dt )
	self.fade_time = self.fade_time + dt
	local t = math.min(self.fade_time / self.total_fade_time, 1)
	if t >= 1 then
		self.fade_time = nil
		self:Hide()
		self.model:SetTexture(0, fade_tex)
		self:StopUpdating()
		if self.fn then
			self.fn()
		end
	else
		self.dissolve:SetValue(1-t)
	end
end