--------------------------------------------------------------------------------

local PrestigeIndicator = class( "Widget.PrestigeIndicator", Widget )

function PrestigeIndicator:init( icon_size )
    PrestigeIndicator._base.init( self )
    self.hover_check = true
    icon_size = icon_size or 50
    
    self.prestige_glow = self:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ), icon_size*1.2, icon_size*1.2 ) )
        :SetTintColour( UICOLOURS.ADVANCEMENT )
        :SetHiddenBoundingBox( true )
        :Pulse( 0.5, 1.0, 0.003 )
    self.prestige_icon_shadow = self:AddChild( Widget.Image() )
        :SetTintColour( 0x000000aa )
        :SetSize( icon_size+1, icon_size+1 )
        :SetHiddenBoundingBox( true )
    self.prestige_icon = self:AddChild( Widget.Image() )
        :SetSize( icon_size, icon_size )
        :SetBloom( 0.15 )

    self:ShowToolTipOnFocus()
end

function PrestigeIndicator:Refresh( gamestate )
    
    local gamestate = gamestate or TheGame:GetGameState()
    if gamestate and gamestate:GetGameType() == GAME_TYPE.DAILY then
        self:RefreshDaily( gamestate )
    elseif gamestate and gamestate.options and gamestate.options and gamestate.options.story_mode then
        self.prestige_icon_shadow:SetTexture( engine.asset.Texture("UI/ic_story_mode.tex" ) )
        self.prestige_icon:SetTexture( engine.asset.Texture("UI/ic_story_mode.tex" ) )
        self:SetToolTip( LOC"UI.NEW_GAME_SCREEN.STORY_TT" )
        self:Show()
    elseif gamestate then
        self:RefreshPrestige( gamestate )
    else
        self:SetShown( false )
    end

    return self
end

-- Used to display a prestige level. Used for standard and brawl runs
function PrestigeIndicator:RefreshPrestige( gamestate )
    if gamestate and gamestate:GetCurrentAct() then
        local current_act = gamestate:GetCurrentAct()
        local prestige = gamestate:GetAdvancementLevel()
        local prestige_data = current_act and current_act.background.data and current_act.background.data.advancement and current_act.background.data.advancement[prestige]
        self.prestige_icon_shadow:SetTexture( prestige_data and prestige_data.icon )
        self.prestige_icon:SetTexture( prestige_data and prestige_data.icon )

        self:SetToolTip( GetAdvancementTooltip( gamestate.current_act, prestige ) )
        self:SetShown( prestige > 0 )
    else
        self:SetShown( false )
    end
    return self
end

-- Used to display a difficulty level. Used for daily runs
function PrestigeIndicator:RefreshDaily( gamestate )
    if gamestate and gamestate:GetCurrentAct() then
        local current_act = gamestate:GetCurrentAct()
        local prestige = gamestate:GetAdvancementLevel()
        local difficulty_level = 1
        for difficulty, difficulty_data in ipairs(DAILY_DIFFICULTY_OPTIONS) do
            if difficulty_data.prestige == prestige then difficulty_level = difficulty end
        end
        self.prestige_icon_shadow:SetTexture( DAILY_DIFFICULTY_OPTIONS[difficulty_level].icon() )
        self.prestige_icon:SetTexture( DAILY_DIFFICULTY_OPTIONS[difficulty_level].icon() )

        self:SetToolTip( GetDailyDifficultyTooltip( gamestate.current_act, prestige, difficulty_level ) )
        self:SetShown( difficulty_level > 1 )
    end
    return self
end

function PrestigeIndicator:OnGainFocus()
    self.prestige_glow:ScaleTo( nil, 1.4, 0.2, easing.outQuad )
    self.prestige_icon_shadow:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
    self.prestige_icon:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
    self.prestige_icon:MoveTo( 0, 1, 0.2, easing.outQuad )
    self.prestige_icon:Bloom(0.2)
end

function PrestigeIndicator:OnLoseFocus()
    self.prestige_glow:ScaleTo( nil, 1, 0.2, easing.outQuad )
    self.prestige_icon_shadow:ScaleTo( nil, 1, 0.2, easing.outQuad )
    self.prestige_icon:ScaleTo( nil, 1, 0.2, easing.outQuad )
    self.prestige_icon:MoveTo( 0, 0, 0.2, easing.outQuad )
    self.prestige_icon:Bloom(0.15)
end