--------------------------------------------------------------------------------

local ProgressIndicator = class( "Widget.ProgressIndicator", Widget )

function ProgressIndicator:init( icon_size, text_size )
    ProgressIndicator._base.init( self )
    self.hover_check = true
    icon_size = icon_size or 32
    text_size = text_size or 32
    self.text_spacing = 0
    self.icon = Widget.Image( global_images.day, icon_size, icon_size )
    self:AddChild( self.icon )

    self.label = Widget.Label( "body", text_size )
    self.label:EnableOutline( SMALL_TEXT_OUTLINE )
    self.label:SetGlyphColour( UICOLOURS.QUEST_LIGHT )
    self:AddChild( self.label ):LayoutBounds( "after", "center" ):Offset( 6, 0 )

    self.vertical = false
end

function ProgressIndicator:SetSpacing( text_spacing )
    self.text_spacing = text_spacing
    self:Layout()

    return self
end

function ProgressIndicator:Layout()
    if self.vertical then
        self.label:LayoutBounds( "center", "below", self.icon ):Offset( 0, -self.text_spacing )
    else
        self.label:LayoutBounds( "after", "center", self.icon ):Offset( 6 + self.text_spacing, 0 )
    end

    return self
end

function ProgressIndicator.GetDesc()
    return ""
end

function ProgressIndicator:Refresh()
    if TheGame:GetGameState() and TheGame:GetGameState():GetGameType() ~= GAME_TYPE.DAILY then
        local percent, title, subtitle = TheGame:GetGameState():GetNarrativeProgress()
        if percent then
            self.old_percent = percent
            self.label:SetText( loc.format( "{1}", title ) )
            self:SetToolTip( loc.format( LOC"UI.PROGRESS_INDICATOR", subtitle, percent) )
            self:Layout()
        end
        self:Show()
    else
        self:Hide()
    end

    return self
end

function ProgressIndicator:SetVertical()
    self.vertical = true
    return self
end

function ProgressIndicator:OnGainFocus()
    self:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
end

function ProgressIndicator:OnLoseFocus()
    self:ScaleTo( nil, 1, 0.2, easing.outQuad )
end