local PANELH = 700 
local ICON_SZ = 32

local TILEW, TILEH, TILE_PADDING = 500, 60, 15

local QuestTile = class("Widget.QuestTile", Widget.Clickable)
function QuestTile:init( owner, quest )
    QuestTile._base.init(self)
    
    self.quest= quest
    self.owner = owner    
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/empty_inner_frame_border_selected.tex" ) ) ):SetSize( TILEW, TILEH ):SetBloom(0.1)

    self.normal_texture =  engine.asset.Texture("UI/empty_inner_frame_border_nonselected.tex" ) 
    self.hover_texture =  engine.asset.Texture("UI/empty_inner_frame_border_selected.tex" ) 

    if quest:IsComplete() then
        self.normal_texture = engine.asset.Texture("UI/empty_inner_frame_border_nonselected_clear.tex" )
        self.hover_texture = engine.asset.Texture("UI/empty_inner_frame_border_selected_clear.tex" )
        self:SetTintColour( UICOLOURS.BONUS )
    end
    if quest:IsFailed() then
        self.normal_texture = engine.asset.Texture("UI/empty_inner_frame_border_nonselected_clear.tex" )
        self.hover_texture = engine.asset.Texture("UI/empty_inner_frame_border_selected_clear.tex" )
        self:SetTintColour( UICOLOURS.PENALTY )
    end

    self:SetHighlightWidget(self.bg)
    self.icon = self:AddChild( Widget.Image(quest:GetIcon() ):SetSize(ICON_SZ, ICON_SZ))
    self.title = self:AddChild( Widget.Label( "body", 25, quest:GetTitle() ) ):SetAutoSize( TILEW - TILE_PADDING*2.5 - ICON_SZ ):SetWordWrap( true ):LeftAlign()
    self.icon:LayoutBounds("left", "top", self.bg):Offset( TILE_PADDING, -TILE_PADDING )
    self.title:LayoutBounds("after", "center", self.icon):Offset( TILE_PADDING/2, 0 )
    self:SetOnClickFn(function() owner:SelectQuest(self.quest) end)

    -- regular quest or storyline quest
    self.title:SetGlyphColour( self.quest:CanAbandon() and UICOLOURS.ACCENT or UICOLOURS.STORYLINE )

    self:SetSelected(false)
end

function QuestTile:SetSelected( val )
    if val ~= self.selected then
        self.selected = val
        self.bg:SetTexture( self.selected and self.hover_texture or self.normal_texture )
        self:UpdateHighlight()
    end
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------

local QuestSelector = class("Widget.QuestSelector", Widget)
function QuestSelector:init( fn, w, h )
    QuestSelector._base.init(self)

    self.width = w or TILEW + 20
    TILEW = self.width - 36 -- scrollbar width + padding
    self.heigth = h or PANELH

    self.quest_tiles = {}
    self.quests = {}

    self.scroll_bg = self:AddChild( Widget.SolidBox( 1, 1, UICOLOURS.SCROLL_BACKGROUND ):SetTintAlpha(0.2) ):SetSize( self.width, self.heigth )
    self.scroll = self:AddChild ( Widget.ScrollPanel():SetSize( self.width, self.heigth ) ):LayoutBounds("center", "center", self.scroll_bg):SetVirtualMargin( 10 )

    self.jobtiles = self.scroll:AddScrollChild(Widget())
    self.fn = fn
end

function QuestSelector:Clear(quest)
    table.clear( self.quests )
    table.clear( self.quest_tiles )
    self.jobtiles:DestroyAllChildren()
    self.scroll:RefreshView()
end

function QuestSelector:AddQuest(quest)
    local tile = QuestTile(self, quest)
    self.quest_tiles[quest] = tile
    self.jobtiles:AddChild(tile)
    tile:LayoutBounds("left", "below"):Offset(0,-4)
    self.jobtiles:LayoutBounds("left", "top", 0, 0)

    if not self.selected_quest then
        self:SelectQuest(quest)
    end

    self.scroll:RefreshView()
end

function QuestSelector:GetQuest()
    return self.selected_quest or self.quests[1]
end

function QuestSelector:SelectQuest( selected_quest )
    selected_quest = selected_quest or self.quests[1]
    for quest, tile in pairs( self.quest_tiles ) do
        tile:SetSelected(quest == selected_quest)
    end
    self.selected_quest = selected_quest
    if self.fn then
        self.fn(selected_quest)
    end
end

function QuestSelector:AddHeader( text )
    self.jobtiles:AddChild( Widget.Label( "title", 24, " " .. text ) ):SetGlyphColour( UICOLOURS.SUBTITLE ):SetAutoSize( TILEW ):LeftAlign():LayoutBounds( "left", "below" ):Offset(0,-10)
end

function QuestSelector:AddButton( text, fn )
    local btn = self.jobtiles:AddChild( Widget.PanelButton( text ) ):SetFn( fn ):SetSize( TILEW-4, 40 ):LayoutBounds( "left", "below" ):Offset(2,0)
    local bg_border_scale = 0.5
    local bg_border = 36
    btn.bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    btn.bg:SetNineSliceBorderScale( bg_border_scale )
    local nineslice_w, nineslice_h = TILEW-4 + bg_border*2*bg_border_scale, 40 + bg_border*2*bg_border_scale
    btn.bg:SetSize( nineslice_w, nineslice_h )
end

function QuestSelector:PrepareAnimation()
    -- Hide objects until they're animated in
    for i, widget in ipairs( self.jobtiles.children ) do
        widget:SetTintAlpha( 0 )
    end
end

function QuestSelector:AnimateIn()

    local bg_anim_duration = 0.3

    -- Hide objects until they're animated in
    self:PrepareAnimation()

    -- Prepare list display sequence
    local row_anim_duration_total = 0.3
    local row_number = math.min( #self.jobtiles.children, 8 )
    local row_anim_duration = 0.1
    local delay_per_row = row_anim_duration_total / row_number
    local row_adder = Updater.Parallel()
    for i, widget in ipairs( self.jobtiles.children ) do
        local widget_x, widget_y = widget:GetPos()
        row_adder:Add( 
            Updater.Series{
                Updater.Wait( delay_per_row * i ),
                Updater.Parallel{
                    Updater.Ease( function(v) widget:SetTintAlpha(v) end, 0, 1, row_anim_duration, easing.linear),
                    Updater.Ease2D( function(x,y) widget:SetPos(x,y) end, widget_x + 60, widget_y, widget_x, widget_y, row_anim_duration/2, easing.linear),
                }
            }
        )
    end

    self:RunUpdater( row_adder )

end