--------------------------------------------------------------------------------

local ResolveIndicator = class( "Widget.ResolveIndicator", Widget )

function ResolveIndicator:init( icon_size, text_size )
    ResolveIndicator._base.init( self )
    self.hover_check = true
    icon_size = icon_size or 32
    text_size = text_size or 32
    self.text_spacing = 0
    self.icon = Widget.Image( global_images.resolve, icon_size, icon_size )
    self:AddChild( self.icon )

    self.label = Widget.Label( "body", text_size )
    self.label:EnableOutline( SMALL_TEXT_OUTLINE )
    self.label:SetGlyphColour( UICOLOURS.RENOWN )
    self:AddChild( self.label ):LayoutBounds( "after", "center" ):Offset( 6, 0 )

    self.vertical = false
end

function ResolveIndicator:SetSpacing( text_spacing )
    self.text_spacing = text_spacing
    self:Layout()

    return self
end

function  ResolveIndicator:Layout()
    if self.vertical then
        self.label:LayoutBounds( "center", "below", self.icon ):Offset( 0, -self.text_spacing )
    else
        self.label:LayoutBounds( "after", "center", self.icon ):Offset( 6 + self.text_spacing, 0 )
    end

    return self
end

function ResolveIndicator.GetDesc()
    return LOC"UI.RESOLVE_TOOLTIP"
end

function ResolveIndicator:SetResolve( resolve, max_resolve )
    if resolve and max_resolve then
        self.label:SetText( loc.format( "{1}/{2}", math.floor(resolve), math.floor(max_resolve) ))
    else
        self.label:SetText( tostring( math.floor(resolve) ))
    end
    self:Layout()
    local tt = {}
    local prefix = LOC("UI.RESOLVE_INDICATOR.TOOLTIP_PREFIX")
    table.insert(tt, prefix .. ResolveIndicator.GetDesc() )
    self:SetToolTip( table.concat(tt, "\n\n" ))
end


function ResolveIndicator:Refresh()
    if TheGame:GetGameState() then
        local resolve = TheGame:GetGameState():GetCaravan().resolve
        local max_resolve = TheGame:GetGameState():GetCaravan().max_resolve
        self:SetResolve( resolve, max_resolve )
    end

    return self
end

function ResolveIndicator:AnimateResolve( new_resolve, old_resolve, new_max, old_max )
    local TOTAL_TIME = 0.3

    self.anim = self:RunUpdater(
        Updater.Ease( 
            function(t)
                local value = Lerp( old_resolve, new_resolve, t )
                local max_value = Lerp( old_max, new_max, t )
                self.label:SetText( loc.format( "{1}/{2}", math.floor(value), math.floor(max_value) ))
                self:Layout()
            end, 
        0, 1.0, TOTAL_TIME, easing.outQuad)
    )
end

function ResolveIndicator:SetVertical()
    self.vertical = true
    return self
end

function ResolveIndicator:OnGainFocus()
    self:ScaleTo( nil, 1.1, 0.2, easing.outQuad )
end

function ResolveIndicator:OnLoseFocus()
    self:ScaleTo( nil, 1, 0.2, easing.outQuad )
end