require "ui/widgets/screen"
require "ui/widgets/clickable"

-------------------------------------------------------------
-- Displays quest rewards.

local RewardsWidget = class ("Widget.RewardsWidget", Widget )

local TILE_SPACING = 10
local TILE_PADDING = 10
local TILE_ICON_SIZE = 40

function RewardsWidget:init( w, columns )
    RewardsWidget._base.init( self )
    self.width = w or 600
    self.columns = columns or 2
    self.tile_w = ( self.width - ( TILE_SPACING * ( self.columns - 1 ) ) ) / self.columns
end

function RewardsWidget:RefreshRewards( quest ) 
    
    local rewards = quest:GetParameter( "rewards" )
    local mettle_rewards = quest:GetMettleReward()
    local extra_reward, extra_reward_data = quest:GetExtraReward()
    local negotiated_bonus = quest:GetParameter( "reward_negotiated_bonus" ) or 0

    local relationship_modifier = quest:GetParameter( "reward_modifier_relationship" ) or 1
    
    self:DestroyAllChildren()

    self.font_size = self.font_size or 28

    self.title = self:AddChild( Widget.Label( "title", 24) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize( self.width )
        :LeftAlign()
        :SetText( LOC"UI.REWARDS_WIDGET.REWARDS" )

    -- Display the rewards bonus/penalty, if any
    if ( relationship_modifier > 1 or relationship_modifier < 1 ) then
        self.rewards_modifier = self:AddChild( Widget.RewardsModifierTile( self.width, relationship_modifier ) ):LayoutBounds( "left", "below" ):Offset( 0, -TILE_SPACING )
    end

    -- Reward tile container, for the animation
    self.rewards_container = self:AddChild( Widget() )

    local reward_count = 0

    if quest:HasUpfrontReward() then
        local icon = engine.asset.Texture("UI/ic_reward_card.tex" )
        local title = LOC"UI.REWARDS_WIDGET.UPFRONT_TITLE"
        local desc = LOC"UI.REWARDS_WIDGET.UPFRONT_DESC"
        reward_count = reward_count + 1
        local upfront_reward = self.rewards_container:AddChild( Widget.UpfrontRewardsTile( self.tile_w, icon, title, desc ) )
            :LayoutInGrid( self.columns, reward_count, TILE_SPACING, self.title )
        upfront_reward:SetToolTip( LOC"UI.REWARDS_WIDGET.UPFRONT_TOOLTIP" )
    end

    if negotiated_bonus > 0 then
        rewards = rewards + negotiated_bonus
    end

    if extra_reward == EXTRA_QUEST_REWARD.EXTRA_CASH then
        rewards = (rewards or 0) + extra_reward_data
    end

    if extra_reward == EXTRA_QUEST_REWARD.METTLE_POINTS then
        mettle_rewards = (mettle_rewards or 0) + extra_reward_data
    end


    --EXTRA_QUEST_REWARD = MakeEnum{"GRAFT_CHOICE", "CARD_REMOVE", "FREE_ITEM", "EXTRA_CASH", "RESTORATION"}
    -- Display extra rewards
    local icon, title, desc, tt
    if extra_reward == EXTRA_QUEST_REWARD.GRAFT_CHOICE then
        icon = engine.asset.Texture("UI/ic_grafts.tex" )
        title = LOC"UI.REWARDS_WIDGET.GRAFT_TITLE"
        desc = LOC"UI.REWARDS_WIDGET.GRAFT_DESC"
        tt = LOC"UI.REWARDS_WIDGET.GRAFT_TT"
    elseif extra_reward == EXTRA_QUEST_REWARD.CARD_REMOVE then
        icon = engine.asset.Texture("UI/ic_levelup_removecombat.tex" )
        title = LOC"UI.REWARDS_WIDGET.REMOVECARD_TITLE"
        desc = LOC"UI.REWARDS_WIDGET.REMOVECARD_DESC"
        tt = LOC"UI.REWARDS_WIDGET.REMOVECARD_TT"
    elseif extra_reward == EXTRA_QUEST_REWARD.FREE_ITEM then
        icon = engine.asset.Texture("UI/ic_nav_give.tex" )
        title = loc.format( LOC"UI.REWARDS_WIDGET.FREEITEM_TITLE", extra_reward_data )
        desc = loc.format( LOC"UI.REWARDS_WIDGET.FREEITEM_DESC", extra_reward_data )
        tt = LOC"UI.REWARDS_WIDGET.FREEITEM_TT"
    elseif extra_reward == EXTRA_QUEST_REWARD.EXTRA_CASH then
        --[[icon = engine.asset.Texture("UI/ic_nav_shills.tex" )
        title = LOC"UI.REWARDS_WIDGET.EXTRACASH_TITLE"
        desc = loc.format( LOC"UI.REWARDS_WIDGET.EXTRACASH_DESC", extra_reward_data )
        tt = LOC"UI.REWARDS_WIDGET.EXTRACASH_TT"--]]
        extra_reward = nil
    elseif extra_reward == EXTRA_QUEST_REWARD.RESTORATION then
        icon = engine.asset.Texture("UI/ic_nav_heal.tex" )
        title = LOC"UI.REWARDS_WIDGET.RESTORATION_TITLE"
        desc = loc.format( LOC"UI.REWARDS_WIDGET.RESTORATION_DESC", 2 * extra_reward_data, extra_reward_data )
        tt = loc.format( LOC"UI.REWARDS_WIDGET.RESTORATION_TT", 2 * extra_reward_data, extra_reward_data )
    elseif extra_reward == EXTRA_QUEST_REWARD.CARD_UPGRADE then
        icon = global_images.upgradecombat
        title = LOC"UI.REWARDS_WIDGET.UPGRADE_TITLE"
        desc = LOC"UI.REWARDS_WIDGET.UPGRADE_DESC"
        tt = LOC"UI.REWARDS_WIDGET.UPGRADE_TT"
    elseif extra_reward == EXTRA_QUEST_REWARD.METTLE_POINTS then
        --[[icon = engine.asset.Texture("UI/ic_mettle.tex" )
        title = LOC"UI.REWARDS_WIDGET.METTLE_POINTS_TITLE"
        desc = loc.format( LOC"UI.REWARDS_WIDGET.METTLE_POINTS_DESC", extra_reward_data, TheGame:GetGameState():GetPlayerAgent():GetName() )
        tt = LOC"UI.REWARDS_WIDGET.METTLE_POINTS_TT"--]]
        extra_reward = nil
    else
        extra_reward = nil
    end
    
    if extra_reward then
        reward_count = reward_count + 1
        local w = self.rewards_container:AddChild( Widget.RewardsTile( self.tile_w, icon, title, desc ) )
            :LayoutInGrid( self.columns, reward_count, TILE_SPACING, self.title )
            :SetToolTip( tt )
    end

    if rewards and rewards > 0 then
        local icon = global_images.money
        local title = loc.format("{1}", rewards)
        local desc = loc.cap( loc.format( "{1#currency}", rewards ))
        reward_count = reward_count + 1
        local money_tile = self.rewards_container:AddChild( Widget.RewardsTile( self.tile_w, icon, title, desc ) ):LayoutInGrid( self.columns, reward_count, TILE_SPACING, self.title )
        money_tile.value:SetGlyphColour( UICOLOURS.MONEY )
        money_tile:SetToolTip(LOC"UI.REWARDS_WIDGET.MONEY_TOOLTIP")
    end

    if mettle_rewards and mettle_rewards > 0 then
        local icon = global_images.mettle
        local title = loc.format("{1#mettle}", mettle_rewards)
        local desc = loc.format( LOC"UI.REWARDS_WIDGET.METTLE_POINTS_DESC", mettle_rewards, TheGame:GetGameState():GetPlayerAgent():GetName() )
        reward_count = reward_count + 1
        local mettle_tile = self.rewards_container:AddChild( Widget.RewardsTile( self.tile_w, icon, title, desc ) ):LayoutInGrid( self.columns, reward_count, TILE_SPACING, self.title )
        mettle_tile.value:SetGlyphColour( UICOLOURS.METTLE )
        mettle_tile:SetToolTip(LOC"UI.REWARDS_WIDGET.METTLE_TOOLTIP")
    end    
    

    self.rewards_container:LayoutBounds( "left", "below" ):Offset( 0, -TILE_SPACING )
    if reward_count == 0 then
        self:Hide()
    else
        self:Show()
    end


    return true
end

function RewardsWidget:PrepareAnimation()
    -- Hide objects until they're animated in

    if self.title then self.title:SetTintAlpha( 0 ) end
    if self.rewards_modifier then
        self.rewards_modifier:SetTintAlpha( 0 )
    end

    if self.rewards_container and not self.rewards_container:IsEmpty() then
        for i, widget in ipairs( self.rewards_container.children ) do
            widget:SetTintAlpha( 0 )
        end
    end
end

function RewardsWidget:AnimateIn()

    if self:CountChildren() <= 0 then
        return
    end

    local bg_anim_duration = 0.2

    -- Prepare for animation by getting current values
    local title_x, title_y = self.title:GetPos()

    -- Hide everything
    self:PrepareAnimation()

    -- Setup and start animation
    self.fadein_anim = Updater.Series{
        Updater.Parallel{
            -- Fade and move in the title
            Updater.Ease( function(v) self.title:SetTintAlpha(v) end, 0, 1, bg_anim_duration/2, easing.linear),
            Updater.Ease2D( function(x,y) self.title:SetPos(x,y) end, title_x + 60, title_y, title_x, title_y, bg_anim_duration/2, easing.linear),
        },
    }

    -- Prepare reward display sequence
    if self.rewards_modifier then
        -- Fade and move in the bonus panel
        self.fadein_anim:Add( Updater.Ease( function(v) self.rewards_modifier:SetTintAlpha(v) end, 0, 1, bg_anim_duration/2, easing.linear) )
    end

    if not self.rewards_container:IsEmpty() then
        local reward_anim_duration_total = 0.1
        local reward_number = math.min( #self.rewards_container.children, 14 )
        local reward_anim_duration = 0.1
        local delay_per_reward = reward_anim_duration_total / reward_number
        local reward_adder = Updater.Parallel()
        for i, widget in ipairs( self.rewards_container.children ) do
            reward_adder:Add( 
                Updater.Series{
                    Updater.Wait( delay_per_reward * i ),
                    Updater.Parallel{
                        Updater.Do( function() AUDIO:PlayEvent( "event:/ui/main/gen/tiny_popup" ) end ),
                        Updater.Ease( function(v) widget:SetTintAlpha(v) end, 0, 1, reward_anim_duration, easing.linear),
                        Updater.Ease( function(v) widget:SetScale(v,v) end, 1.2, 1, reward_anim_duration, easing.linear)
                    }
                }
            )
        end
        self.fadein_anim:Add( reward_adder )
    end

    self:RunUpdater( self.fadein_anim )

end

-------------------------------------------------------------

local RewardsTile = class("Widget.RewardsTile", Widget)

function RewardsTile:init( w, icon, value, desc )
    RewardsTile._base.init( self )

    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/empty_inner_frame_border.tex" ) ) ):SetSize( w, TILE_ICON_SIZE + TILE_PADDING*2 ):SetBloom(0.1)
    self.icon = self:AddChild( Widget.Image( icon and icon or engine.asset.Texture("UI/favour.tex") ) ):SetSize( TILE_ICON_SIZE, TILE_ICON_SIZE )
    self.text_content = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.value = self.text_content:AddChild( Widget.Label("title", 38, value and value or "" ) ):SetGlyphColour(UICOLOURS.SUBTITLE):SetAutoSize( w - TILE_PADDING*3 - TILE_ICON_SIZE ):LeftAlign():SetHoverLinkFn( UIHelpers.ItemTooltips )
    self.desc = self.text_content:AddChild( Widget.Label("body", 22, desc and desc or "" ) ):SetGlyphColour(UICOLOURS.BODYTEXT):SetWordWrap( true ):SetAutoSize( w - TILE_PADDING*3 - TILE_ICON_SIZE ):LeftAlign():SetHoverLinkFn( UIHelpers.ItemTooltips )

    self.icon:LayoutBounds( "left", "top", self.bg ):Offset( TILE_PADDING, -TILE_PADDING )
    local long_value
    if value then
        long_value = tonumber( value ) == nil -- If it's text, it's long_value. If it's a number, it's not long_value
        if long_value then
            self.value:SetFontSize( 24 )
            self.desc:LayoutBounds( "left", "below", self.value ):Offset( 0, 3 )
        else
            self.desc:LayoutBounds( "left", "below", self.value ):Offset( 0, 6 )
        end
    else
        self.desc:LayoutBounds( "center", "center", self.value ):Hide()
    end
    self.text_content:LayoutBounds( "after", "center", self.icon ):Offset( TILE_PADDING, long_value and 0 or 4 )
end

function RewardsTile:SetItemImage( item_image )
    self.icon:Remove()
    self.icon = self:AddChild( item_image ):SetSize( TILE_ICON_SIZE, TILE_ICON_SIZE )
    self.icon:LayoutBounds( "left", "top", self.bg ):Offset( TILE_PADDING, -TILE_PADDING )
end

function RewardsTile:OnGainHover()
    AUDIO:PlayParamEvent( SoundEvents.card_mouseover, "position", self:GetPanPos() )
end

function RewardsTile:OnLoseHover()
    AUDIO:PlayParamEvent( SoundEvents.card_mouseoff, "position", self:GetPanPos() )
end

-------------------------------------------------------------

local RewardsModifierTile = class("Widget.RewardsModifierTile", Widget)

function RewardsModifierTile:init( w, relationship_modifier )
    RewardsModifierTile._base.init( self )

    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/panel_skill_tier.tex" ) ) ):SetSize( w, 67 ):SetBloom(0.1)
    self.text_content = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.value = self.text_content:AddChild( Widget.Label("title", 24, "" ) ):SetGlyphColour(UICOLOURS.SUBTITLE):SetAutoSize( w - TILE_PADDING*4 ):LeftAlign()
    self.desc = self.text_content:AddChild( Widget.Label("body", 22, "" ) ):SetGlyphColour(UICOLOURS.BODYTEXT):SetWordWrap( true ):SetAutoSize( w - TILE_PADDING*4 ):LeftAlign()

    if relationship_modifier > 1 then
        self.bg:SetTintColour( UICOLOURS.BONUS )
        self.value:SetText( loc.format(LOC"UI.REWARDS_WIDGET.BONUS_PERCENT", relationship_modifier - 1) ):SetGlyphColour( UICOLOURS.BONUS )
        self.desc:SetText( LOC"UI.REWARDS_WIDGET.BONUS_PERCENT_DESC" )
    elseif relationship_modifier < 1 then
        self.bg:SetTintColour( UICOLOURS.PENALTY )
        self.value:SetText( loc.format( LOC"UI.REWARDS_WIDGET.PENALTY_PERCENT", 1 - relationship_modifier) ):SetGlyphColour( UICOLOURS.PENALTY )
        self.desc:SetText( LOC"UI.REWARDS_WIDGET.PENALTY_PERCENT_DESC" )
    end

    self.desc:LayoutBounds( "left", "below", self.value ):Offset( 0, 0 )
    self.text_content:LayoutBounds( "left", "center", self.bg ):Offset( TILE_PADDING*2, 0 )

end

function RewardsModifierTile:OnGainHover()
    AUDIO:PlayParamEvent( SoundEvents.card_mouseover, "position", self:GetPanPos() )
end

function RewardsModifierTile:OnLoseHover()
    AUDIO:PlayParamEvent( SoundEvents.card_mouseoff, "position", self:GetPanPos() )
end

-------------------------------------------------------------

local UpfrontRewardsTile = class("Widget.UpfrontRewardsTile", Widget)

function UpfrontRewardsTile:init( w, icon, value, desc )
    RewardsTile._base.init( self )

    local h = TILE_ICON_SIZE + TILE_PADDING*2
    self.bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/empty_inner_frame_border_selected_clear.tex" ) ) )
        :SetSize( w, h )
        :SetBloom(0.1)
        :SetTintColour( UICOLOURS.OBJECTIVE )
        :SetTintAlpha(0.6)

    self.particles = self:AddChild( Widget.ParticleSystem() )
    self.particles:LoadParams( "rewardswidget_upfront" )
    self.particles:SetPos( 0, -h/2 )

    self.icon = self:AddChild( Widget.Image( icon and icon or engine.asset.Texture("UI/favour.tex") ) ):SetSize( TILE_ICON_SIZE, TILE_ICON_SIZE ):Bloom(0.1)
    self.text_content = self:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.value = self.text_content:AddChild( Widget.Label("title", 24, value and value or "" ) )
        :SetGlyphColour(UICOLOURS.OBJECTIVE):SetWordWrap( true ):SetAutoSize( w - TILE_ICON_SIZE - TILE_PADDING*2 ):LeftAlign():Bloom(0.05)
    self.desc = self.text_content:AddChild( Widget.Label("body", 22, desc and desc or "" ) )
        :SetGlyphColour(UICOLOURS.BODYTEXT):SetWordWrap( true ):SetAutoSize( w - TILE_ICON_SIZE - TILE_PADDING*2 ):LeftAlign()

    self.icon:Bloom( 0.05 )

    self.icon:LayoutBounds( "left", "top", self.bg ):Offset( TILE_PADDING, -TILE_PADDING )
    self.desc:LayoutBounds( "left", "below", self.value ):Offset( 0, 3 )
    self.text_content:LayoutBounds( "after", "center", self.icon ):Offset( TILE_PADDING, 2 )
end

function UpfrontRewardsTile:SetItemImage( item_image )
    self.icon:Remove()
    self.icon = self:AddChild( item_image ):SetSize( TILE_ICON_SIZE, TILE_ICON_SIZE )
    self.icon:LayoutBounds( "left", "top", self.bg ):Offset( TILE_PADDING, -TILE_PADDING )
end

function UpfrontRewardsTile:OnGainHover()
    AUDIO:PlayParamEvent( SoundEvents.card_mouseover, "position", self:GetPanPos() )
end

function UpfrontRewardsTile:OnLoseHover()
    AUDIO:PlayParamEvent( SoundEvents.card_mouseoff, "position", self:GetPanPos() )
end
