require "ui/widget"

-------------------------------------------------------------------------------
-- Widget wrapping a Role Icon

local RoleIcon = class( "Widget.RoleIcon", Widget )

function RoleIcon:init( texture, size )
    RoleIcon._base.init( self )
    self.hover_check = true
    
    self.bg = self:AddChild( Widget.Image( engine.asset.Texture("icons/role_icon_bg.tex"), size, size )):SetBloom(0.1)--:SetHiddenBoundingBox(true)
    self.icon = self:AddChild( Widget.Image( texture, size, size )):SetBloom(0.04)--:SetHiddenBoundingBox(true)

    self:SetSize( size or 30 )
end

--[[function RoleIcon:GetBoundingBox()
    return -self.icon_size/2, -self.icon_size/2, self.icon_size/2, self.icon_size/2
end--]]

function RoleIcon:SetSize( size )
    self.icon_size = size or 30

    self.bg:SetSize( self.icon_size, self.icon_size )
    self.icon:SetSize( self.icon_size*0.8, self.icon_size*0.8 )
    if self.aura_layer_1 then self.aura_layer_1:SetSize( self.icon_size, self.icon_size ) end
    if self.aura_layer_2 then self.aura_layer_2:SetSize( self.icon_size, self.icon_size ) end
    if self.aura_layer_3 then self.aura_layer_3:SetSize( self.icon_size, self.icon_size ) end
    
    return self
end

function RoleIcon:OnVizChange(viz)
    if viz then
        self.aura_layer_1:RotateIndefinitely( -0.01 )
        self.aura_layer_2:RotateIndefinitely( 0.03 )
        self.aura_layer_3:RotateIndefinitely( -0.025 )
    else
        self.aura_layer_1:StopSpin()
        self.aura_layer_2:StopSpin()
        self.aura_layer_3:StopSpin()
    end

end


function RoleIcon:ShowAura( show )
    if show then
        if not self.aura_layer_1 then self.aura_layer_1 = self:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer1_0.tex"), self.icon_size, self.icon_size )):SetHiddenBoundingBox(true):SetBloom(0.3) end
        if not self.aura_layer_2 then self.aura_layer_2 = self:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer2_0.tex"), self.icon_size, self.icon_size )):SetHiddenBoundingBox(true):SetBloom(0.3) end
        if not self.aura_layer_3 then self.aura_layer_3 = self:AddChild( Widget.Image( engine.asset.Texture("icons/role_aura_layer3_0.tex"), self.icon_size, self.icon_size )):SetHiddenBoundingBox(true):SetBloom(0.3) end
        self.aura_layer_1:RotateIndefinitely( -0.01 )
        self.aura_layer_2:RotateIndefinitely( 0.03 )
        self.aura_layer_3:RotateIndefinitely( -0.025 )
    else
        if self.aura_layer_1 then self.aura_layer_1:Remove() self.aura_layer_1 = nil end
        if self.aura_layer_2 then self.aura_layer_2:Remove() self.aura_layer_2 = nil end
        if self.aura_layer_3 then self.aura_layer_3:Remove() self.aura_layer_3 = nil end
    end
    return self
end

function RoleIcon:SetTexture( texture )
    self.icon:SetTexture( texture )
    return self
end

function RoleIcon:Animate( animate )
    if animate then
        if self.animation then
            self:StopUpdater( self.animation )
            self.animation = nil
        end
    else
        if self.animation == nil then
            local anim_pacing = 0.3
            local anim_delay = 1.2
            local anim_rotation = 0.5
            self.animation = self:RunUpdater(
                Updater.Loop{
                    Updater.Wait( anim_delay ),
                    Updater.Ease( function(v) self.icon:SetRotation(v) end, 0, -anim_rotation, anim_pacing, easing.outQuad),
                    Updater.Ease( function(v) self.icon:SetRotation(v) end, -anim_rotation, anim_rotation, anim_pacing, easing.outQuad),
                    Updater.Ease( function(v) self.icon:SetRotation(v) end, anim_rotation, 0, anim_pacing, easing.outQuad),
                }
            )
        end
    end
    
end

function RoleIcon:OnGainHover()
    RoleIcon._base:OnGainHover( self )

    self:GainHover()

    return self
end

function RoleIcon:GainHover()
    -- Speed up the animation
    if self.aura_layer_1 then self.aura_layer_1:RotateIndefinitely( -0.02 ) end
    if self.aura_layer_2 then self.aura_layer_2:RotateIndefinitely( 0.06 ) end
    if self.aura_layer_3 then self.aura_layer_3:RotateIndefinitely( -0.05 ) end

    -- Scale up the glows
    local scale, scale = self.icon:GetScale()
    self.icon:ScaleTo( scale, 1.05, 0.3 )
    if self.aura_layer_1 then scale, scale = self.aura_layer_1:GetScale() end
    if self.aura_layer_1 then self.aura_layer_1:ScaleTo( scale, 1.1, 0.3 ) end
    if self.aura_layer_2 then self.aura_layer_2:ScaleTo( scale, 1.2, 0.3 ) end
    if self.aura_layer_3 then self.aura_layer_3:ScaleTo( scale, 1.3, 0.3 ) end

    return self
end

function RoleIcon:OnLoseHover()
    RoleIcon._base:OnLoseHover( self )

    self:LoseHover()

    return self
end

function RoleIcon:LoseHover()

    -- Slow down the animation
    if self.aura_layer_1 then self.aura_layer_1:RotateIndefinitely( -0.01 ) end
    if self.aura_layer_2 then self.aura_layer_2:RotateIndefinitely( 0.03 ) end
    if self.aura_layer_3 then self.aura_layer_3:RotateIndefinitely( -0.025 ) end

    -- Scale down the glows
    local scale, scale = self.icon:GetScale()
    self.icon:ScaleTo( scale, 1, 0.3 )
    if self.aura_layer_1 then scale, scale = self.aura_layer_1:GetScale() end
    if self.aura_layer_1 then self.aura_layer_1:ScaleTo( scale, 1, 0.3 ) end
    if self.aura_layer_2 then self.aura_layer_2:ScaleTo( scale, 1, 0.3 ) end
    if self.aura_layer_3 then self.aura_layer_3:ScaleTo( scale, 1, 0.3 ) end

    return self
end