
local function DoAnimLerp(v, dt)
    if v.target then
        local da = v.target - v.val
        local delta = (da*v.k*4*dt)
        
        if math.abs(da) < .001 then
            v.val = v.target
            v.target = nil
        else
            v.val = v.val + delta
        end
    end
end
--------------------------------------------------

local ScrollLog = class("Widget.ScrollLog", Widget)

ScrollLog.show_child_tooltip_on_focus = true

function ScrollLog:init( )
    ScrollLog._base.init(self)
    self.bubble_widgets = {}
    -- self.w = 500
    self.next_bubble_top = 0
    self.bubbles = Widget()
    self:AddChild(self.bubbles)
    self.speech_pos = {val=0, k = 2}
    self.was_click_down = false
    self:StartUpdating()
end

function ScrollLog:SetWidth(w)
    self.w = w
    return self
end
function ScrollLog:HideHistory()
    for k,v in pairs(self.bubble_widgets) do
        v:Hide()
    end
end

function ScrollLog:ShowHistory()
    for k,v in pairs(self.bubble_widgets) do
        v:Show()
    end
end

function ScrollLog:BubbleWidgets()
    return ipairs( self.bubble_widgets )
end

function ScrollLog:AddWidget(bubble_widget)
    self.bubbles:AddChild(bubble_widget)
    local w, h = bubble_widget:GetScaledSize()
    local y = self.next_bubble_top - h/2
    
    bubble_widget:SetPos(0, y)
    table.insert(self.bubble_widgets, bubble_widget)

    self.next_bubble_top = self.next_bubble_top - h - 40
    self.max_y = -y
end

function ScrollLog:RecalculateWidgets()

    -- The lowest bubble maintains its position; adjust everything upwards.
    local y = -self.max_y

    for i = #self.bubble_widgets, 1, -1 do
        local bubble_widget = self.bubble_widgets[i]
        local w, h = bubble_widget:GetScaledSize()
        if i == #self.bubble_widgets then
            self.max_y = self.max_y + h/2
            self.next_bubble_top = y - h/2 - 40
        else
            y = y + h/2
        end

        bubble_widget:SetPos( nil, y )

        y = y + h/2 + 40
    end
end

function ScrollLog:RemoveWidget( bubble_widget )
    table.arrayremove( self.bubble_widgets, bubble_widget )
    bubble_widget:Remove()
end


function ScrollLog:GetScrollPercent()
    -- if (self.max_y or 0) > 0 then
    --     return self.speech_pos.val / self.max_y
    -- end

    return 0
end

function ScrollLog:Clear()
    while #self.bubble_widgets > 0 do
        table.remove( self.bubble_widgets ):Remove()
    end

    self.max_y = 0
    self.speech_pos.val = 0
    self.speech_pos.target = nil
    self.next_bubble_top = 0

    self.bubbles:SetPos(0,0)

end 
