require "ui/widget"


local SegmentedBar = class("Widget.SegmentedBar", Widget)

function SegmentedBar:init( dw, dh )

    SegmentedBar._base.init(self, engine.model.SegmentedBarModel())

    self:SetSize(dw, dh)
    
    self.blocks_mouse = true
end

function SegmentedBar:OnRemoved()
    self.ontexture = nil
    self.offtexture = nil
end


function SegmentedBar:__tostring()
    return string.format("SegmentedBar Widget (%2.2fx%2.2f)", self:GetSize())
end

function SegmentedBar:SetBloom(b)
    self.model:SetBloom(b)
    return self
end

function SegmentedBar:SetOnTint(r,g,b,a)
    self.model:SetOnTint(r,g,b,a)
    return self
end

function SegmentedBar:SetOffTint(r,g,b,a)
    self.model:SetOffTint(r,g,b,a)
    return self
end

function SegmentedBar:GetBoundingBox()
    return self.model:GetBoundingBox()
end

function SegmentedBar:SetSize(dw,dh)
    self.model:SetSize( dw or 200, dh or 10 )
    self:MarkTransformDirty()
    self:InvalidateBBox()
    return self
end

function SegmentedBar:GetSize()
    return self.model:GetSize()
end

function SegmentedBar:SetOnTexture(texture, minx, miny, maxx, maxy)
    self.ontexture = texture
    self.model:SetOnTexture(texture)

    if minx and miny and maxx and maxy then
        local tx, ty = texture:GetSize()
        -- print("SetNineSliceCoords()", minx, miny, maxx, maxy, tx, ty)
        local uvminx = minx/tx
        local uvminy = (ty-maxy)/ty -- Yes miny/maxy are flipped on purpose
        local uvmaxx = maxx/tx
        local uvmaxy = (ty-miny)/ty -- Yes miny/maxy are flipped on purpose
        self.model:SetOnInnerUVs(uvminx, uvminy, uvmaxx, uvmaxy)
    end

    return self
end

function SegmentedBar:SetOffTexture(texture, minx, miny, maxx, maxy)
    self.offtexture = texture
    self.model:SetOffTexture(texture)

    if minx and miny and maxx and maxy then
        local tx, ty = texture:GetSize()
        -- print("SetNineSliceCoords()", minx, miny, maxx, maxy, tx, ty)
        local uvminx = minx/tx
        local uvminy = (ty-maxy)/ty -- Yes miny/maxy are flipped on purpose
        local uvmaxx = maxx/tx
        local uvmaxy = (ty-miny)/ty -- Yes miny/maxy are flipped on purpose
        self.model:SetOffInnerUVs(uvminx, uvminy, uvmaxx, uvmaxy)
    end

    return self
end

function SegmentedBar:SetProgress(totalnrpips, nrpipsenabled)
    self.model:SetNrPips(totalnrpips, nrpipsenabled);
    return self
end

function SegmentedBar:SetSpaceBetweenPips(space)
    self.model:SetSpaceBetweenPips(space);
    self:InvalidateBBox()
    return self
end

function SegmentedBar:SetMask()
    self.model:SetColourWrite(false)
    self.model:SetEffect(global_shaders.UI_MASK)
    self:SetStencilWrite(STENCIL_MODES.SET)
    return self
end

function SegmentedBar:Expand( dw, dh )
    local w, h = self:GetSize()
    self:SetSize( w + (dw or 0), h + (dh or 0) )
    return self
end

function SegmentedBar:GetSize()
    return self.model:GetSize()
end

function SegmentedBar:__tostring()
    return "segmented bar"
end

    