local SIDEBAR_H = 600

local SCROLLPANEL_RIGHT_PADDING = 600 -- Extra space on the right of the sidebars to prevent stuff from getting clipped
local SIDEBAR_LEFT_PADDING = 60 -- The sidebar has this padding on the left, so that glows aren't visibly clipped off. And then the whole sidebar is moved left this same amount, so the contents display in the same spot

------------------------------------------------------------------------------------------
-- A default sidebar block of content

local SidebarBlock = class("Widget.SidebarBlock", Widget)

function SidebarBlock:init( width )
    SidebarBlock._base.init(self)
    self.sidebar_width = width
    self.current_layout_scale = 1

    -- Scroll panel which contains everything in the sidebar
    self.scroll_panel = self:AddChild( Widget.ScrollPanel() )
        :SetVirtualTopMargin( 60 )
        :SetVirtualBottomMargin( 250 )
        :ShowBar( SCROLLBAR.IF_NEEDED )
        :SetScrollBarOuterMargin( 0 )
        :LeftScrollBar()
        :ShowBar( SCROLLBAR.NEVER )
        :SetSize( self.sidebar_width+SCROLLPANEL_RIGHT_PADDING, SIDEBAR_H )
    -- Header, which contains text elements on the top of the sidebar
    self.header_content = self.scroll_panel:AddScrollChild( Widget() )
    -- Main title (character name or current page)
    self.title = self.header_content:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_TITLE, "TITLE" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :LeftAlign()
        :SetAutoSize( self.sidebar_width )
        :SetWordWrap( true )
		:Bloom( 0.1 )
    -- Smaller title (act name)
    self.subtitle = self.header_content:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_SUBTITLE, "SUBTITLE" ) )
        :SetGlyphColour( UICOLOURS.TITLE )
        :LeftAlign()
        :SetAutoSize( self.sidebar_width )
        :SetWordWrap( true )
        :Bloom( 0.05 )
    -- Text block describing things!
    self.text = self.header_content:AddChild( Widget.Label( "body", FONT_SIZE.BODY_TEXT ) )
		:SetAutoSize( self.sidebar_width )
		:SetWordWrap( true )
        :LeftAlign()
		:SetGlyphColour( 0xCBFFEFff )
        :Bloom( 0.1 )
    -- Single line of text with controls or selection info
    self.info = self.header_content:AddChild( Widget.Label( "title", FONT_SIZE.BODY_TEXT, "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize( self.sidebar_width )
        :SetWordWrap( true )
        :Bloom( 0.05 )
    -- Container for the rest of the elements on this sidebar, to be shown under the header contents
    self.scrolling_contents = self.scroll_panel:AddScrollChild( Widget() )

    self.shown = false
	self:SetTintAlpha( 0 )
end

function SidebarBlock:Refresh( character_data )
    self.character_data = character_data
end

function SidebarBlock:SetHeight(h)
    self.sidebar_h = h
    return self
end

function SidebarBlock:Layout()
    if self.subtitle:IsShown() then self.subtitle:LayoutBounds( "left", "below", self.title ):Offset( 0, SPACING.TITLE_SUBTITLE ) end
    self.text:LayoutBounds( "left", "below", self.subtitle:IsShown() and self.subtitle or self.title ):Offset( 0, self.subtitle:IsShown() and -SPACING.M1 or 0 )
    if self.info:IsShown() then self.info:LayoutBounds( "left", "below", self.text:IsShown() and self.text or self.subtitle ):Offset( 0, self.text:IsShown() and -SPACING.M1/2 or -SPACING.M1 ) end

    self.header_content:LayoutBounds( "left", "top", SIDEBAR_LEFT_PADDING, 0 )
    self.scrolling_contents:LayoutBounds( "left", "below", self.header_content ):Offset( 0, -SPACING.M1 )
    self.scroll_panel:SetSize( self.sidebar_width+SCROLLPANEL_RIGHT_PADDING, self.sidebar_h )

    self.scroll_panel:RefreshView()
end

function SidebarBlock:OnScreenModeChange( sm )
    SidebarBlock._base.OnScreenModeChange( self, sm )

    self.current_layout_scale = (LAYOUT_SCALE[sm]+1)/2
    self.header_content:SetLayoutScale( self.current_layout_scale )
    self.scrolling_contents:SetLayoutScale( self.current_layout_scale )
    self:Layout()
end

-- Call this to make this element shown, but transparent, right before laying it out on the screen
-- If it's hidden, it might not lay out properly
function SidebarBlock:PrepareAnimateIn()
    self:Show()
    self.scroll_panel:Show()
    self:SetTintAlpha(0)
end

-- This fades in the sidebar, and potentially any elements within it
function SidebarBlock:AnimateIn()
    self.scroll_panel:Show()
	self:AlphaTo( 1, 0.1, easing.outQuad )
end

-- This fades out the sidebar
function SidebarBlock:AnimateOut()
	self:AlphaTo( 0, 0.1, easing.outQuad, function()
        self.scroll_panel:Hide()
        self:Hide()
    end )
end

function SidebarBlock:GetDefaultFocus()
	return nil
end

-- If the selected character anim should show when this sidebar's on screen. Default is true
function SidebarBlock:IsAnimVisible()
    return true
end
