------------------------------------------------------------------------------------------
-- The sidebar showing the various selectable perks

local SidebarCustom = class("Widget.SidebarCustom", Widget.SidebarBlock)

function SidebarCustom:init( width, sidebar_selected_fn )
    SidebarCustom._base.init(self,width)

    self.btn_cardsets = self.scrolling_contents:AddChild( Widget.CustomOptionToggle( self.sidebar_width, "", nil ) )
        :SetOnClickFn( function() sidebar_selected_fn( "UNLOCKS" ) end )
        :SetType( "UNLOCKS" )
    self.btn_mutators = self.scrolling_contents:AddChild( Widget.CustomOptionToggle( self.sidebar_width, "", nil ) )
        :SetOnClickFn( function() sidebar_selected_fn( "MUTATORS" ) end )
        :SetType( "MUTATORS" )
end

function SidebarCustom:Refresh( character_data, act_data, anim, story )
    self.character_data = character_data
	if not character_data then return end

	SidebarCustom._base.Refresh( self, character_data )

    -- Update header
    self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_CUSTOM_TITLE" ):SetGlyphColour( UICOLOURS.SUBTITLE )
    self.subtitle:Hide()
    self.text:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_CUSTOM_SUBTITLE" )

    -- Update buttons
    self.btn_cardsets:Refresh( character_data, story )
    self.btn_mutators:Refresh( character_data, story )

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )

	-- Layout
    self:Layout()
end

function SidebarCustom:Layout()
    SidebarCustom._base.Layout( self )

    self.scrolling_contents:StackChildren( 10 )
        :LayoutBounds( "left", "below", self.header_content )
        :Offset( 0, -SPACING.M1 )
end

function SidebarCustom:AnimateIn()
	SidebarCustom._base.AnimateIn( self )
end

function SidebarCustom:GetDefaultFocus()
	return self.btn_cardsets
end

-- Static function used by the SidebarButton to find out if custom game options are active
function SidebarCustom:IsCustomGameActive( character_data )

    -- Check card sets state
    local toggled_off_unlocks = TheGame:GetGameProfile():GetToggledOffUnlocks( character_data.id )
    local unlocks_active = #toggled_off_unlocks > 0

    -- Check mutators state
    local mutators = TheGame:GetGameProfile():GetSelectedMutators( character_data.id )
    local mutators_active = mutators and #mutators > 0

    -- Custom is active if either option is active
    return unlocks_active or mutators_active
end

-------------------------------------------------------------------------
-- A single custom toggle

local CustomOptionToggle = class( "Widget.CustomOptionToggle", Widget.Clickable )

local ICON_W = 54
local TEXT_LEFT_MARGIN = 78

function CustomOptionToggle:init( w )
    CustomOptionToggle._base.init( self )

    self.width = w or 200
    self.hitbox = self:AddChild( Widget.SolidBox( w, 50, 0x00FFFF00 ) )
    self.contents = self:AddChild( Widget() )
    self.bg = self.contents:AddChild( Widget.Panel( engine.asset.Texture("UI/mutator_bg_active.tex" ) ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.2 )
        :SetBloom(0.1)
    self.glow = self.contents:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
        :SetTintColour( UICOLOURS.SUBTITLE )
        :SetTintAlpha( 0.2 )
        :SetHiddenBoundingBox( true )
        :SetSize( ICON_W*2, ICON_W*2 )
    self.icon = self.contents:AddChild( Widget.Image() )
        :SetSize( ICON_W, ICON_W )
        :Bloom( 0.15 )
    self.text_content = self.contents:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.title = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE ) )
        :LeftAlign()
        :SetAutoSize( w-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom( 0.1 )
    self.subtitle = self.text_content:AddChild( Widget.Label( "tooltip", FONT_SIZE.ITEM_DESC ) )
        :LeftAlign()
        :SetAutoSize( w-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom( 0.1 )
    self.active_label = self.text_content:AddChild( Widget.NewItemsLabel() )
        :SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_CUSTOM_ACTIVE" )
end

function CustomOptionToggle:Layout()
    self.title:SetAutoSize( self.width-TEXT_LEFT_MARGIN-12 )
    self.subtitle:SetAutoSize( self.width-TEXT_LEFT_MARGIN-12 )
    self.subtitle:LayoutBounds( "left", "below", self.title ):Offset( 0, SPACING.TITLE_DESC )
    self.active_label:LayoutBounds( "left", "below", self.subtitle ):Offset( 0, 0 )
    local text_w, text_h = self.text_content:GetSize()
    text_h = text_h + 20
    self.hitbox:SetSize( self.width, text_h )
    self.bg:SetSize( self.width, text_h )
    self.icon:LayoutBounds( "left", "top", self.hitbox ):Offset( 14, -9 )
    self.glow:LayoutBounds( "center", "center", self.icon ):Offset( 0, 0 )
    self.text_content:LayoutBounds( "left", "center", self.hitbox ):Offset( TEXT_LEFT_MARGIN, 1 )
    return self
end

function CustomOptionToggle:SetType( type )
    self.type = type
    return self
end

function CustomOptionToggle:Refresh( character_data, story )
    if self.type == "UNLOCKS" then
        -- Get act id
        local character_id = character_data.id
        -- Get unlocks state
        local toggled_off_unlocks = TheGame:GetGameProfile():GetToggledOffUnlocks( character_id, self.gamestate )
        self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_UNLOCKS_TITLE" )
        if #toggled_off_unlocks == 0 then
            self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_UNLOCKS_SUBTITLE_ENABLED" )
        else
            self.subtitle:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_UNLOCKS_SUBTITLE", #toggled_off_unlocks ) )
        end
        -- Get total unlocks
        local unlocked, total = ActUtil.GetUnlocksProgress( character_id )
        self.icon:SetTexture( engine.asset.Texture( "UI/newgame_sidebar_sets.tex") )
        self.glow:SetTintColour( UICOLOURS.OBJECTIVE ):SetTintAlpha( 0.2 )
        self.title:SetGlyphColour( UICOLOURS.OBJECTIVE )
        self.locked = unlocked < 1 and not TheGame:GetLocalSettings().ALL_UNLOCKS -- This is locked if no sets are unlocked
        if self.locked then
            self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_UNLOCKS_LOCKED" )
            self:Disable()
        else
            self:Enable()
        end
        self.active_label:SetShown( #toggled_off_unlocks > 0 )
        self:SetChecked( #toggled_off_unlocks > 0 )
    elseif self.type == "MUTATORS" then
        -- Get act id
        local character_id = character_data.id
        local mutators = TheGame:GetGameProfile():GetSelectedMutators( character_id )
        if mutators then
            -- Vet the mutators
            for i = #mutators, 1, -1 do
                if Content.GetGraft( mutators[i] ) == nil then
                    table.remove( mutators, i )
                    TheGame:GetGameProfile():SetSelectedMutators( character_id, mutators )
                end
            end
        end
        self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_MUTATORS_TITLE" )
        self.locked = not TheGame:GetGameProfile():HasUnlock("MUTATORS_UNLOCKED")
        if self.locked then
            self:SetToolTip( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_MUTATORS_SUBTITLE_LOCKED_TT")
            self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_MUTATORS_SUBTITLE_LOCKED" )
            self:Disable()
        elseif mutators == nil or #mutators == 0 then
            self.subtitle:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_MUTATORS_SUBTITLE_NONE" )
            self:Enable()
        else
            self.subtitle:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MAIN_MUTATORS_SUBTITLE", #mutators ) )
            self:Enable()
        end
        self.icon:SetTexture( engine.asset.Texture( "UI/newgame_sidebar_mutators.tex") )
        self.glow:SetTintColour( UICOLOURS.MUTATORS ):SetTintAlpha( 0.2 )
        self.title:SetGlyphColour( UICOLOURS.MUTATOR )
        self.active_label:SetShown( mutators and #mutators > 0 )
        self:SetChecked( mutators and #mutators > 0 )
    end
    self:Layout()
    return self
end

function CustomOptionToggle:IsChecked()
    return self.checked
end

function CustomOptionToggle:SetChecked( checked )
    self.checked = checked
    if self.checked then
        self.bg:SetTexture( engine.asset.Texture("UI/mutator_bg_active.tex" ) )
            :SetTintColour( UICOLOURS.SUBTITLE )
        self.title:SetGlyphColour( UICOLOURS.SUBTITLE )
        self.icon:SetTintColour( UICOLOURS.WHITE )
        self.glow:SetTintColour( UICOLOURS.SUBTITLE )
            :SetTintAlpha( 0.3 )
    else
        self.bg:SetTexture( engine.asset.Texture("UI/mutator_bg_normal.tex" ) )
            :SetTintColour( 0x4A6A82FF )
        self.glow:SetTintColour( 0x4A6A82FF )
            :SetTintAlpha( 0.2 )
    end
end

function CustomOptionToggle:UpdateImage()
    if self.enabled and ( self.hover or self.focus ) then
        if self.is_locked then
            AUDIO:PlayEvent( "event:/ui/card_codex/mouseover_locked_card" )
        end
        self.glow:ScaleTo( nil, 1.2, 0.1 )
        self.contents:ScaleTo( nil, 1.05, 0.2 )
        self.contents:MoveTo( 20, 0, 0.2 )
    else
        self.glow:ScaleTo( nil, 1, 0.1 )
        self.contents:ScaleTo( nil, 1, 0.3 )
        self.contents:MoveTo( 0, 0, 0.3 )
    end
end

function CustomOptionToggle:OnGainFocus()
    CustomOptionToggle._base.OnGainFocus(self)
    self:UpdateImage()
end

function CustomOptionToggle:OnLoseFocus()
    CustomOptionToggle._base.OnLoseFocus(self)
    self:UpdateImage()
end

function CustomOptionToggle:OnGainHover()
    CustomOptionToggle._base.OnGainHover(self)
    self:UpdateImage()
end

function CustomOptionToggle:OnLoseHover()
    CustomOptionToggle._base.OnLoseHover(self)
    self:UpdateImage()
end

