
------------------------------------------------------------------------------------------
-- The sidebar showing the Negotiation cards

local SidebarNegotiation = class("Widget.SidebarNegotiation", Widget.SidebarBlock)

local CARD_SCALE = 0.55
local CARD_SCALE_HOVER = 0.56

function SidebarNegotiation:init( width )
    SidebarNegotiation._base.init(self,width)

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
end

function SidebarNegotiation:GetDeck( character_data )
	if character_data and character_data.data then
		self.deck = Content.GetNegotiationDecks( character_data.data.player_agent )[1]
	end
end

function SidebarNegotiation:Refresh( character_data )
	if not character_data then return end

	self:GetDeck( character_data )

	SidebarNegotiation._base.Refresh( self, character_data )

	self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_NEGOTIATION_TITLE" )
        :SetGlyphColour( UICOLOURS.NEGOTIATION )
    self.subtitle:Hide()
	self.text:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_NEGOTIATION_TEXT" )
	self.info:SetGlyphColour( UICOLOURS.SUBTITLE )

	self:UpdateCardsContainer( character_data )

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )

	-- Layout
    self:Layout()
end

function SidebarNegotiation:Layout()
    SidebarNegotiation._base.Layout( self )
end

function SidebarNegotiation:OnControlModeChange( cm, device_id )
    SidebarNegotiation._base.OnControlModeChange( self, cm, device_id )

    if self.info then
        if cm == CONTROL_MODE.MOUSE_KEYBOARD then
            self.info:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_NEGOTIATION_UPGRADE" )
        end
        if cm == CONTROL_MODE.GAMEPAD then
            self.info:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_NEGOTIATION_UPGRADE_CONTROLLER" )
        end
        if cm == CONTROL_MODE.TOUCH then
            self.info:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_NEGOTIATION_UPGRADE_TOUCH" )
        end
    end

    self:Layout()
end

SidebarNegotiation.CONTROL_MAP = 
{
    {
        control = Controls.Digital.VIEW_UPGRADES,
        test = function(self)
            return self.focused_card_widget
        end,
        fn = function(self)
            self:PreviewUpgrade( self.focused_card_widget )
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.VIEW_UPGRADES", Controls.Digital.VIEW_UPGRADES))
        end,
    },
}

function SidebarNegotiation:PreviewUpgrade( widget )
	self.last_inspect_card = widget
    local preview_only = true
	local agent = nil
    local popup = Screen.UpgradeCardPopup( widget.card, agent, preview_only )
    TheGame:FE():PushScreen( popup )
end

function SidebarNegotiation:UpdateCardsContainer( character_data )
	if not self.deck then return end

	-- Check how many card widgets we need
	local deck_count = self.deck:CountCards()
    local existing_widgets = #self.scrolling_contents.children

    -- Hide all old cards
    for k, v in ipairs( self.scrolling_contents.children ) do
    	v:Hide()
    end

    -- Refresh all the cards
    local index = 1
    for i, card_id in self.deck:Cards() do
        local card = Negotiation.Card( card_id )
        local widget
        if index > existing_widgets then
        	-- Create a new widget if needed
        	widget = self.scrolling_contents:AddChild( Widget.NegotiationCard() )
        else
        	-- Get an existing widget
        	widget = self.scrolling_contents.children[index]
        end
        widget:Show()
        widget:SetTintAlpha( 0 )
        widget:RefreshCard( card )
        widget:SetScale( CARD_SCALE )
        widget:SetInfoPopupFn(function() self.last_inspect_card = widget end)
        widget:ShowInfoOnClick()
        widget:SetOutline( UICOLOURS.NEGOTIATION, false )
        -- Handle events
        widget.OnGainFocus = function() widget._base.OnGainFocus(widget) self.focused_card_widget = widget self:UpdateCardImageState( widget ) end
        widget.OnLoseFocus = function() widget._base.OnLoseFocus(widget) self.focused_card_widget = nil self:UpdateCardImageState( widget ) end
        widget.OnGainHover = function() widget._base.OnGainHover(widget) self:UpdateCardImageState( widget ) end
        widget.OnLoseHover = function() widget._base.OnLoseHover(widget) self:UpdateCardImageState( widget ) end
        	index = index + 1
    end

    self.scrolling_contents:LayoutChildrenInGrid( 3, 20 )
end

function SidebarNegotiation:UpdateCardImageState( widget )
    if widget.hover or widget.focus then
        if is_instance( widget, Widget.BattleCard ) then
        elseif is_instance( widget, Widget.NegotiationCard ) then
        end

        widget:SetOutline( UICOLOURS.FOCUS, true )
        widget:ScaleTo( nil, CARD_SCALE_HOVER, 0.2, easing.outQuad )
    else
        if is_instance( widget, Widget.BattleCard ) then
        	widget:SetOutline( UICOLOURS.FIGHT, true )
        elseif is_instance( widget, Widget.NegotiationCard ) then
        	widget:SetOutline( UICOLOURS.NEGOTIATION, true )
        end

        widget:ScaleTo( nil, CARD_SCALE, 0.2, easing.outQuad )
    end
end

function SidebarNegotiation:AnimateIn()
	SidebarNegotiation._base.AnimateIn( self )
    self.last_inspect_card = nil
    -- Prepare card fade-in sequence
    local card_anim_duration_total = 0.2
    local card_number = math.min( #self.scrolling_contents.children, 10 )
    local card_anim_duration = 0.1
    local delay_per_card = card_anim_duration_total / card_number
    local card_adder = Updater.Parallel()
    local spacing = 10
    for i, widget in ipairs( self.scrolling_contents.children ) do
    	local pos_x, pos_y = widget:GetPos()
    	widget:SetTintAlpha( 0 )
        card_adder:Add( 
            Updater.Series{
                Updater.Wait(delay_per_card*i),
                Updater.Parallel{
                    Updater.Ease( function(v) widget:SetTintAlpha(v) end, 0, 1, card_anim_duration, easing.linear),
                    Updater.Ease( function(v) widget:SetScale(v,v) end, CARD_SCALE * 0.8, CARD_SCALE, card_anim_duration, easing.linear),
                    Updater.Ease2D( function(x,y) widget:SetPos(x,y) end, pos_x, pos_y-spacing*i, pos_x, pos_y, card_anim_duration, easing.linear),
                }
            }
        )
    end

    self:RunUpdater( card_adder )
end

function SidebarNegotiation:AnimateOut()
	SidebarNegotiation._base.AnimateOut( self )
end

function SidebarNegotiation:GetDefaultFocus()

    if self.last_inspect_card and not self.last_inspect_card.removed then
        return self.last_inspect_card
    end

	return self.scrolling_contents and self.scrolling_contents.children and #self.scrolling_contents.children > 0 and self.scrolling_contents.children[1]
end

------------------------------------------------------------------------------------------
-- The sidebar showing the Battle cards

local SidebarBattle = class("Widget.SidebarBattle", Widget.SidebarNegotiation)

function SidebarBattle:init( width, back_fn )
    SidebarBattle._base.init(self,width,back_fn)
end

function SidebarBattle:GetDeck( character_data )
	if character_data and character_data.data then
		self.deck = Content.GetBattleDecks( character_data.data.player_agent )[1]
	end
end

function SidebarBattle:OnControlModeChange( cm, device_id )
    SidebarBattle._base.OnControlModeChange( self, cm, device_id )

    if self.info then
        if cm == CONTROL_MODE.MOUSE_KEYBOARD then
            self.info:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_BATTLE_UPGRADE" )
        end
        if cm == CONTROL_MODE.GAMEPAD then
            self.info:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_BATTLE_UPGRADE_CONTROLLER" )
        end
        if cm == CONTROL_MODE.TOUCH then
            self.info:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_BATTLE_UPGRADE_TOUCH" )
        end
    end

    self:Layout()
end

function SidebarBattle:Refresh( character_data )
	if not character_data then return end

	SidebarBattle._base.Refresh( self, character_data )

	self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_BATTLE_TITLE" )
        :SetGlyphColour( UICOLOURS.FIGHT )
    self.subtitle:Hide()
	self.text:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_BATTLE_TEXT" )
	self.info:SetGlyphColour( UICOLOURS.SUBTITLE )

	self:Layout()
end

function SidebarBattle:UpdateCardsContainer( character_data )
	if not self.deck then return end

	-- Check how many card widgets we need
	local deck_count = self.deck:CountCards()
    local existing_widgets = #self.scrolling_contents.children

    -- Hide all old cards
    for k, v in ipairs( self.scrolling_contents.children ) do
    	v:Hide()
    end

    -- Refresh all the cards
    local index = 1
	for i, card_id in self.deck:Cards() do
        local card = Battle.Card( card_id )
        local widget
        if index > existing_widgets then
        	-- Create a new widget if needed
        	widget = self.scrolling_contents:AddChild( Widget.BattleCard() )
        else
        	-- Get an existing widget
        	widget = self.scrolling_contents.children[index]
        end
        widget:Show()
        widget:SetTintAlpha( 0 )
        widget:RefreshCard( card )
        widget:SetScale( CARD_SCALE )
        widget:SetInfoPopupFn(function() self.last_inspect_card = widget end)
        widget:ShowInfoOnClick()
        widget:SetOutline( UICOLOURS.FIGHT, false )
        -- Handle events
        widget.OnGainFocus = function() widget._base.OnGainFocus(widget) self.focused_card_widget = widget self:UpdateCardImageState( widget ) end
        widget.OnLoseFocus = function() widget._base.OnLoseFocus(widget) self.focused_card_widget = nil self:UpdateCardImageState( widget ) end
        widget.OnGainHover = function() widget._base.OnGainHover(widget) self:UpdateCardImageState( widget ) end
        widget.OnLoseHover = function() widget._base.OnLoseHover(widget) self:UpdateCardImageState( widget ) end
    	index = index + 1
    end

    self.scrolling_contents:LayoutChildrenInGrid( 3, 10 )
end
