------------------------------------------------------------------------------------------
-- The sidebar showing the various selectable Mutators

local SidebarMutators = class("Widget.SidebarMutators", Widget.SidebarBlock)

function SidebarMutators:init( width )
    SidebarMutators._base.init(self,width)

    local columns = 2

    self.btn_random = self.header_content:AddChild( Widget.ShortButton( (self.sidebar_width-10)/2, LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_RANDOM", nil ) )
        :SetOnClickFn( function() self.mutators_container:Randomize(3) end )
        :SetFocusable( false )
    self.btn_unselect = self.header_content:AddChild( Widget.ShortButton( (self.sidebar_width-10)/2, LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_UNSELECT", nil ) )
        :SetOnClickFn( function() self.mutators_container:UnselectAll() end )
        :SetFocusable( false )
        :Hide()

    self.mutators_container = self.scrolling_contents:AddChild( Widget.MutatorSelector( self.sidebar_width, 650, nil, columns, function( mutators ) self:OnMutatorsSelected( mutators ) end ) )

    self.toggled_on_mutators = 0
end

function SidebarMutators:OnMutatorsSelected( mutators )
    self.toggled_on_mutators = #mutators
    self.info:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_INFO", self.toggled_on_mutators ) ):SetGlyphColour( UICOLOURS.SUBTITLE )
    self.btn_unselect:SetShown( TheGame:FE():GetControlMode() ~= CONTROL_MODE.GAMEPAD and self.toggled_on_mutators > 0 )
    -- Save selected mutators
    TheGame:GetGameProfile():SetSelectedMutators( self.character_data.id, mutators )
end

function SidebarMutators:Refresh( character_data )
    self.character_data = character_data
	if not character_data then return end

	SidebarMutators._base.Refresh( self, character_data )

	self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_TITLE" ):SetGlyphColour( UICOLOURS.MUTATOR )
    self.subtitle:Hide()
	self.text:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_TEXT" )

    -- Get selected mutators, if any
    local mutators = TheGame:GetGameProfile():GetSelectedMutators( character_data.id )
    self.toggled_on_mutators = mutators and #mutators or 0
    -- Refresh mutators container
    self.mutators_container:Refresh( mutators )

    -- Update count
    self.info:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_INFO", self.toggled_on_mutators ) ):SetGlyphColour( UICOLOURS.SUBTITLE )
    self.btn_unselect:SetShown( TheGame:FE():GetControlMode() ~= CONTROL_MODE.GAMEPAD and self.toggled_on_mutators > 0 )
    
    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )

	-- Layout
    self:Layout()
end

function SidebarMutators:Layout()
    SidebarMutators._base.Layout( self )

    self.btn_random:LayoutBounds( "left", "below", self.info ):Offset( 0, -SPACING.M1 )
    self.btn_unselect:LayoutBounds( "after", "center", self.btn_random ):Offset( 10, 0 )

    self.scrolling_contents:LayoutBounds( "left", "below", self.header_content ):Offset( 0, -SPACING.M1 )
end

function SidebarMutators:OnControlModeChange( cm, device_id )
    SidebarMutators._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.btn_random:Show():SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_RANDOM" )
        self.btn_unselect:Show():SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_UNSELECT" )
    end
    if cm == CONTROL_MODE.TOUCH then
        self.btn_random:Show():SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_RANDOM" )
        self.btn_unselect:Show():SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_MUTATORS_UNSELECT" )
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.btn_random:Hide()
        self.btn_unselect:Hide()
    end
    
    self:Layout()
end


SidebarMutators.CONTROL_MAP = 
{
    {
        control = Controls.Digital.GAMEPAD_LT,
        fn = function(self)
            self.mutators_container:UnselectAll()
            return true
        end,
        hint = function(self, left, right)
            table.insert(left, loc.format(LOC"UI.CONTROLS.DISABLE_ALL", Controls.Digital.GAMEPAD_LT))
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_RT,
        fn = function(self)
            self.mutators_container:Randomize(3)
            return true
        end,
        hint = function(self, left, right)
            table.insert(left, loc.format(LOC"UI.CONTROLS.ENABLE_RANDOM", Controls.Digital.GAMEPAD_RT))
        end,
    },
}


function SidebarMutators:AnimateIn()
	SidebarMutators._base.AnimateIn( self )
    self.mutators_container:AnimateIn()
end


function SidebarMutators:GetDefaultFocus()
	return self.mutators_container:GetDefaultFocus()
end

-------------------------------------------------------------------------
-- A mutator toggle container

local MutatorSelector = class( "Widget.MutatorSelector", Widget )

local WIDTH = 300
local COLUMN_SPACING = 15

function MutatorSelector:init( width, height, mutators, columns, on_select_fn )
    MutatorSelector._base.init( self )

    self.mutators = mutators or {}
    self.on_select_fn = on_select_fn
    self.checkboxes = {} -- Keeps a list of [ID] = Widget.MutatorOption

    self:SetSize( width or WIDTH )
    self:Refresh( mutators )
end

function MutatorSelector:SetSize( w )
    self.w = w or WIDTH
end

function MutatorSelector:Randomize( count )
    AUDIO:PlayEvent( "event:/ui/new_game_menu/mutator_randomize" )

    -- Clear existing.
    for id, option in pairs( self.checkboxes ) do
        option:SetValue( false )
    end

    local mutator_defs = {}
    for i, def in ipairs( Content.GetAllGrafts() ) do
        if def.type == GRAFT_TYPE.MUTATOR and not def.restricted then
            table.insert( mutator_defs, def )
        end
    end

    -- This is not completely correct, as depending on the selection it is possible
    -- to pick LESS than 3 due to mutually exclusive options.
    while #mutator_defs > 0 and count > 0 do
        local def = table.remove( mutator_defs, math.random( #mutator_defs ))
        self.checkboxes[ def.id ]:SetValue( true )
        count = count - 1

        if def.exclusion_ids then
            for i, id in ipairs( def.exclusion_ids ) do
                local idx = table.findif( mutator_defs, function( x ) return x.id == id end )
                if idx then
                    table.remove( mutator_defs, idx )
                end
            end
        end
    end

    self:NotifyChanged()
end

function MutatorSelector:UnselectAll()
    AUDIO:PlayEvent( "event:/ui/new_game_menu/mutator_unselect_all" )

    for id, option in pairs( self.checkboxes ) do
        option:SetValue( false )
    end
    self:NotifyChanged()
end

function MutatorSelector:Refresh( mutators )

    -- Get all mutators and fill the list if empty
    if self:IsEmpty() then
        for i, def in ipairs( Content.GetAllGrafts() ) do
            if def.type == GRAFT_TYPE.MUTATOR and not def.restricted then
                self:AddOption( def.id, def, false )
            end
        end
    end

    -- Check all the selected ones
    self.mutators = mutators or {}
    for id, option in pairs( self.checkboxes ) do
        local checked = table.arrayfind( self.mutators, id )
        option:SetValue( checked )
    end
end

function MutatorSelector:AddOption( id, def, checked )

    local height = 74
    local w = Widget.MutatorOption( self.w or 300, height, def, checked )

    self:AddChild( w )
        :LayoutBounds( "left", "below" )
        :Offset( 0, -10 )
    self.checkboxes[ id ] = w

    w:SetOnChangeFn( function( option, state ) self:OnOptionChanged( id, state ) end )
end

function MutatorSelector:OnOptionChanged( id, state )
    if state then
        -- Turn off mutually exclusive options.
        local def = Content.GetGraft( id )
        if def.exclusion_ids then
            for i, id in ipairs( def.exclusion_ids ) do
                if self.checkboxes[ id ] then
                    self.checkboxes[ id ]:SetValue( false )
                end
            end
        end
    end
    self:NotifyChanged()
end

function MutatorSelector:NotifyChanged()
    -- Generate a list of the active mutators
    self.mutators = {}
    for id, w in pairs( self.checkboxes ) do
        if w:IsChecked() then
            table.insert( self.mutators, id )
        end
    end
    table.sort( self.mutators )

    if self.on_select_fn then self.on_select_fn( self.mutators ) end
end

function MutatorSelector:AnimateIn()
    -- Prepare button fade-in sequence
    local timing = 0.2
    local spacing = 30
    local button_adder = Updater.Parallel()
    for i, widget in ipairs( self.children ) do
        local pos_x, pos_y = widget:GetPos()
        widget.enabled = false
        button_adder:Add( Updater.Ease( function(v) widget:SetPos(nil,v) end, pos_y-spacing*i, pos_y, timing, easing.outQuad ) )
    end
    -- Enable buttons after they've animated in
    button_adder:Add( Updater.Do( function()
        for i, widget in ipairs( self.children ) do
            widget.enabled = true
        end
    end ) )
    self:RunUpdater( button_adder )
end


function MutatorSelector:GetDefaultFocus()
    return self.children and #self.children > 0 and self.children[1]
end


-------------------------------------------------------------------------
-- A single mutator toggle

local MutatorOption = class( "Widget.MutatorOption", Widget.Clickable )

local ICON_W = 44
local TEXT_LEFT_MARGIN = 68

function MutatorOption:init( w, h, def, checked )
    MutatorOption._base.init( self )
    self.features = {}

    self.def = def
    self.graft = GraftInstance( def.id )
    
    self.hitbox = self:AddChild( Widget.SolidBox( w, h, 0x00FFFF00 ) )
    self.contents = self:AddChild( Widget() )
    self.bg = self.contents:AddChild( Widget.Panel( engine.asset.Texture("UI/mutator_bg_active.tex" ) ) )
        :SetTintColour( UICOLOURS.MUTATOR )
        :SetTintAlpha( 0.2 )
        :SetBloom(0.1)
    self.glow = self.contents:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
        :SetTintColour( UICOLOURS.MUTATOR )
        :SetTintAlpha( 0.2 )
        :SetHiddenBoundingBox( true )
        :SetSize( ICON_W*2, ICON_W*2 )
    self.icon = self.contents:AddChild( Widget.Image( def.img ) )
        :SetSize( ICON_W, ICON_W )
        :Bloom( 0.15 )
    self.text_content = self.contents:AddChild( Widget() )
        :SetHiddenBoundingBox( true )
    self.title = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE, def:GetLocalizedName() ) )
        :LeftAlign()
        :SetAutoSize( w-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :SetGlyphColour( UICOLOURS.MUTATOR )
        :Bloom( 0.1 )
    self.text = self.text_content:AddChild( Widget.Label( "tooltip", FONT_SIZE.ITEM_DESC, self.graft:GetDesc( self ) ) )
        :LeftAlign()
        :SetAutoSize( w-TEXT_LEFT_MARGIN )
        :OverrideLineHeight( FONT_SIZE.ITEM_DESC*LINE_HEIGHT_TITLE_MULT )
        :SetWordWrap( true )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom( 0.1 )

    self:SetSize( w, h )

    self:SetValue( checked )
    self:SetToolTipClass( Widget.TooltipGraft ):SetToolTip( self.graft )
    self:SetOnClickFn( function()
        self:SetValue( not self.checked )

        if self.checked then
            AUDIO:PlayEvent( "event:/ui/new_game_menu/mutator_on" )
        else
            AUDIO:PlayEvent( "event:/ui/new_game_menu/mutator_off" )
        end

        if self.on_change_fn then self.on_change_fn( self, self.def.id, self.checked ) end
    end )

end

function MutatorOption:SetSize( w, h )
    self.title:SetAutoSize( w-TEXT_LEFT_MARGIN-12 )
    self.text:SetAutoSize( w-TEXT_LEFT_MARGIN-12 )
    self.text:LayoutBounds( "left", "below", self.title ):Offset( 0, SPACING.TITLE_DESC )
    local text_w, text_h = self.text_content:GetSize()
    text_h = text_h + 20
    self.hitbox:SetSize( w, text_h )
    self.bg:SetSize( w, text_h )
    self.icon:LayoutBounds( "left", "center", self.hitbox ):Offset( 14, 0 )
    self.glow:LayoutBounds( "center", "center", self.icon ):Offset( 0, 0 )
    self.text_content:LayoutBounds( "left", "center", self.hitbox ):Offset( TEXT_LEFT_MARGIN, 1 )
    return self
end

function MutatorOption:IsChecked()
    return self.checked
end

function MutatorOption:SetValue( checked, notify_changed )
    self.checked = checked
    if self.checked then
        self.bg:SetTexture( engine.asset.Texture("UI/mutator_bg_active.tex" ) )
            :SetTintColour( UICOLOURS.MUTATOR )
        self.title:SetGlyphColour( UICOLOURS.MUTATOR )
        self.icon:SetTintColour( UICOLOURS.WHITE )
        self.glow:SetTintColour( UICOLOURS.MUTATOR )
            :SetTintAlpha( 0.3 )
    else
        self.bg:SetTexture( engine.asset.Texture("UI/mutator_bg_normal.tex" ) )
            :SetTintColour( 0x4A6A82FF )
        self.glow:SetTintColour( 0x4A6A82FF )
            :SetTintAlpha( 0.2 )
    end
    if notify_changed and self.on_change_fn then
        self.on_change_fn( self, self.def.id, self.checked )
    end
end

function MutatorOption:SetOnChangeFn( fn )
    self.on_change_fn = fn
end

function MutatorOption:PostFeature( keyword, name, desc )
    self.features[ keyword ] = { name = name, desc = desc }
end

function MutatorOption:PostCard( feature_id, card )
    self.features[ feature_id ] = { card = card }
end

function MutatorOption:UpdateImage()
    if self.enabled and ( self.hover or self.focus ) then
        if self.is_locked then
            AUDIO:PlayEvent( "event:/ui/card_codex/mouseover_locked_card" )
        end
        self.glow:ScaleTo( nil, 1.2, 0.1 )
        self.contents:ScaleTo( nil, 1.05, 0.2 )
        self.contents:MoveTo( 20, 0, 0.2 )
    else
        self.glow:ScaleTo( nil, 1, 0.1 )
        self.contents:ScaleTo( nil, 1, 0.3 )
        self.contents:MoveTo( 0, 0, 0.3 )
    end
end

function MutatorOption:OnGainFocus()
    MutatorOption._base.OnGainFocus(self)
    self:UpdateImage()
end

function MutatorOption:OnLoseFocus()
    MutatorOption._base.OnLoseFocus(self)
    self:UpdateImage()
end

function MutatorOption:OnGainHover()
    MutatorOption._base.OnGainHover(self)
    self:UpdateImage()
end

function MutatorOption:OnLoseHover()
    MutatorOption._base.OnLoseHover(self)
    self:UpdateImage()
end

