
------------------------------------------------------------------------------------------
-- The sidebar showing the card unlocks

local SidebarUnlocks = class("Widget.SidebarUnlocks", Widget.SidebarBlock)

function SidebarUnlocks:init( width )
    SidebarUnlocks._base.init(self,width)

    self.btn_selectall = self.header_content:AddChild( Widget.ShortButton( (self.sidebar_width-10)/2, LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_SELECTALL", nil ) )
        :SetOnClickFn( function() self:EnableAll() end )
        :SetFocusable( false )
    self.btn_unselectall = self.header_content:AddChild( Widget.ShortButton( (self.sidebar_width-10)/2, LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_UNSELECTALL", nil ) )
        :SetOnClickFn( function() self:DisableAll() end )
        :SetFocusable( false )

    self.toggled_off_sets = 0

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
end

function SidebarUnlocks:Refresh( character_data )
    if not character_data then return end

	SidebarUnlocks._base.Refresh( self, character_data )

	self.title:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_TITLE" ):SetGlyphColour( UICOLOURS.ADVANCEMENT )
    self.subtitle:Hide()
	self.text:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_TEXT", loc.toupper( character_data:GetLocalizedName() ) ))

    -- Remove old unlocks buttons
	self.scrolling_contents:DestroyAllChildren()

	-- Add new ones
	local character_id = character_data.data.player_agent
	local track_data = Content.GetUnlockTrack( character_id )
	local num_unlocked = TheGame:GetGameProfile():GetNumUnlocked( character_id )
	if #track_data > 0 then
		for k, unlock_data in ipairs( track_data ) do
			local unlocked = k <= num_unlocked or TheGame:GetLocalSettings().ALL_UNLOCKS
			local toggled_off = TheGame:GetGameProfile():IsUnlockToggledOff( character_id, unlock_data.id )
    		self.scrolling_contents:AddChild( Widget.UnlocksButton( self.sidebar_width, character_id, unlock_data, unlocked, not toggled_off, function() self:UpdateToggledCount() end ) )
		end
	end
	self:UpdateToggledCount()
    self.animations_done = true

	-- Layout
    self:Layout()
end

function SidebarUnlocks:Layout()
    self.scrolling_contents:StackChildren( 10, 0 )

    SidebarUnlocks._base.Layout( self )

    self.btn_selectall:LayoutBounds( "left", "below", self.info ):Offset( 0, -SPACING.M1 )
    self.btn_unselectall:LayoutBounds( "after", "center", self.btn_selectall ):Offset( 10, 0 )
    
    self.scrolling_contents:LayoutBounds( "left", "below", self.header_content ):Offset( 0, -SPACING.M1 )
end

function SidebarUnlocks:OnControlModeChange( cm, device_id )
    SidebarUnlocks._base.OnControlModeChange( self, cm, device_id )

    if self.info then
	    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
	    	self.info:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_INFO", self.toggled_off_sets ) )
            self.btn_selectall:Show():SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_SELECTALL" )
            self.btn_unselectall:Show():SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_UNSELECTALL" )
	    end
	    if cm == CONTROL_MODE.GAMEPAD then
            self.info:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_INFO_CONTROLLER", self.toggled_off_sets ) )
            self.btn_selectall:Hide()
            self.btn_unselectall:Hide()
	    end
        if cm == CONTROL_MODE.TOUCH then
            self.info:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_INFO_TOUCH", self.toggled_off_sets ) )
            self.btn_selectall:Show():SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_SELECTALL" )
            self.btn_unselectall:Show():SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_UNSELECTALL" )
        end
    end
end

function SidebarUnlocks:OnScreenModeChange( sm )
    SidebarUnlocks._base.OnScreenModeChange( self, sm )

    self.header_content:SetLayoutScale( LAYOUT_SCALE[sm] )
    self.scrolling_contents:SetLayoutScale( LAYOUT_SCALE[sm] )
    self:Layout()
end

function SidebarUnlocks:UpdateToggledCount()
	self.toggled_off_sets = 0
	for k, v in ipairs( self.scrolling_contents.children ) do
		if v:IsToggledOff() then self.toggled_off_sets = self.toggled_off_sets + 1 end
	end
	if TheGame:FE():GetControlMode() == CONTROL_MODE.MOUSE_KEYBOARD then
		self.info:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_INFO", self.toggled_off_sets ) )
	elseif TheGame:FE():IsGamepadMode() then
    	self.info:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_INFO_CONTROLLER", self.toggled_off_sets ) )
    elseif TheGame:FE():GetControlMode() == CONTROL_MODE.TOUCH then
        self.info:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_INFO_TOUCH", self.toggled_off_sets ) )
    end
    self.info:LayoutBounds( "left", "below", self.text ):Offset( 0, -10 )
end


SidebarUnlocks.CONTROL_MAP = 
{
    {
        control = Controls.Digital.GAMEPAD_RT,
        fn = function(self)
            self:EnableAll()
            return true
        end,
        hint = function(self, left, right)
            table.insert(left, loc.format(LOC"UI.CONTROLS.ENABLE_ALL", Controls.Digital.GAMEPAD_RT))
        end,
    },
    {
        control = Controls.Digital.GAMEPAD_LT,
        fn = function(self)
            self:DisableAll()
            return true
        end,
        hint = function(self, left, right)
            table.insert(left, loc.format(LOC"UI.CONTROLS.DISABLE_ALL", Controls.Digital.GAMEPAD_LT))
        end,
    },
}


function SidebarUnlocks:EnableAll()
    if self.animations_done then
        -- Disable animations being triggered again until it's done
        self.animations_done = false
        -- Setup animation
        local total_time = 0.3
        local button_adder = Updater.Parallel()
        for i, widget in ipairs( self.scrolling_contents.children ) do
            local delay = easing.outQuad( i, 0, 1, #self.scrolling_contents.children ) * total_time
            button_adder:Add( 
                Updater.Series{
                    Updater.Wait( delay ),
                    Updater.Do( function() widget:OnToggleChanged( true ) end )
                })
        end
        -- Enable this after the animation is done
        button_adder:Add( 
            Updater.Series{
                Updater.Wait( total_time*2 ),
                Updater.Do( function() self.animations_done = true end )
            })
        self:RunUpdater( button_adder )
    end
    return self
end

function SidebarUnlocks:DisableAll()
    if self.animations_done then
        -- Disable animations being triggered again until it's done
        self.animations_done = false
        -- Setup animation
        local total_time = 0.3
        local button_adder = Updater.Parallel()
        for i, widget in ipairs( self.scrolling_contents.children ) do
            local delay = easing.outQuad( i, 0, 1, #self.scrolling_contents.children ) * total_time
            button_adder:Add( 
                Updater.Series{
                    Updater.Wait( delay ),
                    Updater.Do( function() widget:OnToggleChanged( false ) end )
                })
        end
        -- Enable this after the animation is done
        button_adder:Add( 
            Updater.Series{
                Updater.Wait( total_time*2 ),
                Updater.Do( function() self.animations_done = true end )
            })
        self:RunUpdater( button_adder )
    end
    return self
end

function SidebarUnlocks:AnimateIn()
	SidebarUnlocks._base.AnimateIn( self )

    -- Prepare button fade-in sequence
    local timing = 0.2
    local spacing = 30
    local button_adder = Updater.Parallel()
    for i, widget in ipairs( self.scrolling_contents.children ) do
    	local pos_x, pos_y = widget:GetPos()
        widget:Show()
        button_adder:Add( Updater.Ease( function(v) widget:SetPos(nil,v) end, pos_y-spacing*i, pos_y, timing, easing.outQuad ) )
    end
    button_adder:Add( Updater.Do( function()
    	for i, widget in ipairs( self.scrolling_contents.children ) do
    		widget.enabled = true
    	end
	end ) )
    self:RunUpdater( button_adder )
end


function SidebarUnlocks:GetDefaultFocus()
	return self.scrolling_contents.children and #self.scrolling_contents.children > 0 and self.scrolling_contents.children[1]
end

------------------------------------------------------------------------------------------
-- A button for each available unlock

local UnlocksButton = class("Widget.UnlocksButton", Widget.Clickable)

local HEIGHT = 50
local ICON_W = 60
local TEXT_LEFT_MARGIN = 80

function UnlocksButton:init( width, character_id, unlock_data, unlocked, toggled, on_toggle_fn )
    UnlocksButton._base.init(self)

    self.button_width = width
    self.hitbox = self:AddChild( Widget.SolidBox( self.button_width, HEIGHT, 0x00FFFF00 ) )
    self.contents = self:AddChild( Widget() )
    self.glow = self.contents:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ) ) )
    	:SetTintColour( UICOLOURS.OBJECTIVE )
    	:SetTintAlpha( 0.2 )
   		:SetHiddenBoundingBox( true )
        :SetSize( ICON_W*2, ICON_W*2 )
	self.icon = self.contents:AddChild( Widget.Image( engine.asset.Texture("images/white.tex" ) ) )
        :SetSize( ICON_W, ICON_W )
        :Bloom( 0.15 )
    self.text_content = self.contents:AddChild( Widget() )
		:SetHiddenBoundingBox( true )
    self.title = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_SUBTITLE, "" ) )
        :LeftAlign()
        :SetAutoSize( self.button_width-TEXT_LEFT_MARGIN )
        :SetWordWrap( true )
        :Bloom( 0.1 )
    self.status_icon = self.text_content:AddChild( Widget.Image( engine.asset.Texture("UI/ic_item_locked.tex" ) ) )
    	:SetSize( 20, 20 )
    	:Bloom( 0.05 )
    self.status_label = self.text_content:AddChild( Widget.Label( "title", FONT_SIZE.ITEM_DESC, "" ) )
    	:LeftAlign()
        :SetAutoSize( self.button_width )
        :SetWordWrap( true )
        :Bloom( 0.05 )

    self:ShowToolTipOnFocus( false )
    self.on_toggle_fn = on_toggle_fn

    self:Refresh( character_id, unlock_data, unlocked, toggled )
    self:Layout()
    self:UpdateImage()

    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
end

function UnlocksButton:Refresh( character_id, unlock_data, unlocked, toggled )
    self.character_id = character_id or self.character_id
    self.unlock_data = unlock_data or self.unlock_data
	self.unlocked = unlocked or self.unlocked
	self.toggled = toggled or self.toggled

	if self.unlocked then
		self.title:SetText( self.unlock_data:GetLocalizedName() )
    	self.icon:SetTexture( self.unlock_data.icon )
		if self.toggled then
			self:SetToolTip( loc.format( LOC"UI.UNLOCK_TRACK.UNLOCKED_ACTIVE_TOOLTIP", self.unlock_data:GetLocalizedName() ) )
	    	self.title:SetGlyphColour( UICOLOURS.OBJECTIVE )
	    	self.icon:SetTintColour( UICOLOURS.WHITE )
			self.glow:SetTintColour( UICOLOURS.OBJECTIVE )
    			:SetTintAlpha( 0.2 )
    		self.status_label:SetGlyphColour( UICOLOURS.QUEST )
    			:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_STATUS_ENABLED" )
    		self.status_icon:SetTexture( engine.asset.Texture("UI/ic_item_enabled.tex" ) )
		else
			self:SetToolTip( loc.format( LOC"UI.UNLOCK_TRACK.UNLOCKED_DISABLED_TOOLTIP", self.unlock_data:GetLocalizedName() ) )
	    	self.title:SetGlyphColour( 0x4A6A82ff )
	    	self.icon:SetTintColour( 0x4A6A82ff )
			self.glow:SetTintColour( 0x4A6A82FF )
    			:SetTintAlpha( 0.2 )
    		self.status_label:SetGlyphColour( UICOLOURS.PENALTY )
    			:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_STATUS_DISABLED"  )
    		self.status_icon:SetTexture( engine.asset.Texture("UI/ic_item_disabled.tex" ) )
		end
	else
		self:SetToolTip( loc.format( LOC"UI.UNLOCK_TRACK.LOCKED_TOOLTIP", self.unlock_data:GetLocalizedName() ) )
		self.title:Hide()
    	self.glow:SetTintColour( 0x4A6A82FF )
    	self.glow:SetTintAlpha( 0.2 )
		self.status_label:SetGlyphColour( UICOLOURS.OBJECTIVE )
    		:SetText( LOC"UI.NEW_GAME_SCREEN.SIDEBAR_UNLOCKS_STATUS_LOCKED" )
		self.status_icon:SetTexture( engine.asset.Texture("UI/ic_item_locked.tex" ) )
    	self.icon:SetTexture( self.unlock_data.icon_locked )
	end
end

function UnlocksButton:Layout()
	-- Layout things
	self.icon:LayoutBounds( "left", "center", self.hitbox ):Offset( 5, 0 )
	self.glow:LayoutBounds( "center", "center", self.icon ):Offset( 0, 0 )
	self.status_icon:LayoutBounds( "left", "below", self.title ):Offset( 0, 0 )
	self.status_label:LayoutBounds( "after", "center", self.status_icon ):Offset( 4, 1 )
	self.text_content:LayoutBounds( "left", "center", self.hitbox ):Offset( TEXT_LEFT_MARGIN, 0 )
end

function UnlocksButton:ViewSet()
    if self.unlocked then
        TheGame:FE():InsertScreen( Screen.UnlockInspectorScreen( self.unlock_data, false, function() end ) )
    end
    return self
end

function UnlocksButton:OnControlModeChange( cm, device_id )
    UnlocksButton._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self:SetOnClickFn( function() self:OnToggleChanged( not self.toggled ) end )
        self:SetOnRightClickFn( function() self:ViewSet() end )
        self:SetOnLongClickFn( nil )
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self:SetOnClickFn( nil )
        self:SetOnRightClickFn( nil )
        self:SetOnLongClickFn( nil )
    end
    if cm == CONTROL_MODE.TOUCH then
        self:SetOnClickFn( function() self:OnToggleChanged( not self.toggled ) end )
        self:SetOnRightClickFn( nil )
        self:SetOnLongClickFn( function() self:ViewSet() end )
    end
end

UnlocksButton.CONTROL_MAP = 
{
    {
        control = Controls.Digital.GAMEPAD_X,
        fn = function(self)
            self:ViewSet()
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.VIEW", Controls.Digital.GAMEPAD_X))
        end,
    },
    {
        control = Controls.Digital.MENU_ACCEPT,
        test = function(self)
            return TheGame:FE():IsGamepadMode()
        end,
        fn = function(self)
            self:OnToggleChanged( not self.toggled )
            return true
        end,
        hint = function(self, left, right)
            table.insert(right, loc.format(LOC"UI.CONTROLS.TOGGLE", Controls.Digital.MENU_ACCEPT))
        end,
    },
}

function UnlocksButton:OnToggleChanged( toggled )
    if self.unlocked then
    	if toggled ~= self.toggled then
            if toggled then
        		AUDIO:PlayEvent( "event:/ui/main/gen/cardset_makeactive" )
        	else
        		AUDIO:PlayEvent( "event:/ui/main/gen/cardset_deactivate" )
        	end
        end
        self.toggled = toggled
        self:Refresh( nil, nil, nil, self.toggled )
        self:Layout()
    	TheGame:GetGameProfile():SetUnlockToggledOff( self.character_id, self.unlock_data.id, not self.toggled )
    	if self.on_toggle_fn then self.on_toggle_fn() end
    end
	return self
end

function UnlocksButton:IsToggledOff()
	return self.unlocked and not self.toggled
end

function UnlocksButton:UpdateImage()
    if ( self.unlocked and self.hover ) or self.focus then
        if self.is_locked then
            AUDIO:PlayEvent( "event:/ui/card_codex/mouseover_locked_card" )
        end
        self.glow:ScaleTo( nil, 1.2, 0.1 )
        self.contents:ScaleTo( nil, 1.05, 0.2 )
        self.contents:MoveTo( 20, 0, 0.2 )
    else
        self.glow:ScaleTo( nil, 1, 0.1 )
        self.contents:ScaleTo( nil, 1, 0.3 )
        self.contents:MoveTo( 0, 0, 0.3 )
    end
end

function UnlocksButton:OnGainFocus()
    UnlocksButton._base.OnGainFocus(self)
    self:UpdateImage()
end

function UnlocksButton:OnLoseFocus()
    UnlocksButton._base.OnLoseFocus(self)
    self:UpdateImage()
end

function UnlocksButton:OnGainHover()
    UnlocksButton._base.OnGainHover(self)
    self:UpdateImage()
end

function UnlocksButton:OnLoseHover()
    UnlocksButton._base.OnLoseHover(self)
    self:UpdateImage()
end
