

------------------------------------------------------------------------------------------
-- Shows a fancy label, with customizable space between each character

local SpacedLabel = class( "Widget.SpacedLabel", Widget )

function SpacedLabel:init( font, sz, txt, spacing )
    SpacedLabel._base.init( self )

    self.font = font or "body"
    self.text_size = sz or 20
    self.text = txt or ""
    self.spacing = spacing or 0

    -- To align things against
    self.hitbox = self:AddChild( Widget.SolidBox( self.text_size, self.text_size, 0x00000000 ) )

    -- Contains the text widgets
    self.container = self:AddChild( Widget() )

    self:SetText( self.text )
    self:SetGlyphColour( UICOLOURS.TITLE )
end

function SpacedLabel:SetText( text )
    self.text = SanitizeString( text )

    -- Remove all old characters
    self.container:DestroyAllChildren()

    -- Add all new ones!
    if #self.text > 1 then
	    for i = 1, #self.text do
	    	local c = self.text:sub(i,i)
	    	local label = self.container:AddChild( Widget.Label( self.font, self.text_size, c ) )
		end
	end
end

function SpacedLabel:SetSpacing( spacing )
    if self.spacing ~= spacing then

	    self.spacing = spacing or 0
		
		if self.container.children and #self.container.children > 1 then
			for i, label in ipairs( self.container.children ) do
				if i > 1 then
					label:LayoutBounds( "after", nil, self.container.children[ i - 1 ] ):Offset( self.spacing, 0 )
				end
			end
		end
		self.container:LayoutBounds( "center", "center", self.hitbox )

	end
end

function SpacedLabel:SetFontSize( sz )
	if self.text_size ~= sz then

	    self.text_size = sz or 20
		
		if self.container.children and #self.container.children > 1 then
			for i, label in ipairs( self.container.children ) do
				if i > 1 then
					label:SetFontSize( self.text_size )
				end
			end
		end

	end
end

function SpacedLabel:SetGlyphColour( r, g, b, a )
	if r and not g and not b then
		r,g,b,a = HexColour(r)
	end
	self.colour_r = r
	self.colour_g = g
	self.colour_b = b
	self.colour_a = a

	if self.container.children and #self.container.children > 1 then
		for i, label in ipairs( self.container.children ) do
			if i > 1 then
				label:SetGlyphColour( self.colour_r, self.colour_g, self.colour_b, self.colour_a )
			end
		end
	end

end

function SpacedLabel:Animate( from_spacing, to_spacing, from_size, to_size, from_alpha, to_alpha, t, ease )
	from_spacing = from_spacing or self.spacing
	to_spacing = to_spacing or 20
	from_size = from_size or self.text_size
	to_size = to_size or self.text_size
	from_alpha = from_alpha or 0
	to_alpha = to_alpha or 1
	t = t or 0.2
	local ease = ease or easing.inOutQuad

	self:RunUpdater(
        Updater.Parallel{
        	-- Update the size
        	Updater.Ease( function(v)
        			self:SetFontSize( v )
        		end, from_size, to_size, t, ease ),

        	-- Update the spacing
        	Updater.Ease( function(v)
        			self:SetSpacing( v )
        		end, from_spacing, to_spacing, t, ease ),

        	-- Update the alpha
        	Updater.Ease( function(v)
        			self.container:SetTintAlpha( v )
        		end, from_alpha, to_alpha, t, ease ),
    	}
	)
end