local assets = 
{
    tail_player = engine.asset.Texture( "UI/convo_speech_tail_player.tex" ),
    tail = engine.asset.Texture( "UI/convo_speech_tail.tex" ),
    speech_player = engine.asset.Texture ("UI/convo_speech_player.tex"),
    speech = engine.asset.Texture ("UI/convo_speech.tex"),
    text = engine.asset.Texture ("UI/convo_narration_box.tex"),
    tutorial_text = engine.asset.Texture ("UI/convo_tutorial_box.tex"),
    tldr_text = engine.asset.Texture ("UI/convo_tldr_box.tex"),
}


local ICON_PADDING = 15
local STYLE = MakeEnum{ "PLAIN", "TUTORIAL", "TLDR", "SPEECH", "SPEECH_LEFT", "SPEECH_RIGHT" }
local bg_border = 45
local bg_border_scale = 0.5
local tail_size = 30

local TextBox = class("Widget.TextBox", Widget)
TextBox.STYLE = STYLE

function TextBox:init(txt, font)
    TextBox._base.init(self)

    self.bg = Widget.Panel()
        :SetTexture( assets.text )
        :SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
        :SetNineSliceBorderScale( bg_border_scale )
        :SetName("BG")
    self.content = Widget():SetRegistration("center","center")
    self.text = Widget.Label( font or "speech", 30 ):SetName("TXT"):SetWordWrap(true):LeftAlign()
    self.text.no_reticule_snap = true
    self.text:SetGlyphColour(UICOLOURS.CONVO_NARRATION_TEXT)
    self.text:SetHoverLinkGlyphColour( UICOLOURS.DARK_TEXT_LINK )
    self.tail = Widget.Image( assets.tail ):Hide():SetSize( tail_size, tail_size )

    self:AddChildren
    {
        self.bg,
        self.tail,
        self.content:AddChildren{
            self.text,
        }
    }

    self.padding = 0
    self.max_w = 500
    self.min_height = 10
    self.style = STYLE.PLAIN

    if txt then
        self:SetText( txt )
    end
end

function TextBox:SetMinimumHeight(h)
    self.min_height = h
    self:DoLayout()
    return self
end

function TextBox:SetFont(font)
    self.text:SetFont(font)
    return self
end

function TextBox:SetFontSize(sz)
    self.text:SetFontSize(sz)
    self:DoLayout()
    return self
end

function TextBox:SetLineHeight(sz)
    self.text:OverrideLineHeight(sz)
    self:DoLayout()
    return self
end

function TextBox:GetLabel()
    return self.text
end

function TextBox:Spool( t )
    self.text:Spool( t )
end

function TextBox:SnapSpool( )
    self.text:SnapSpool()
end

function TextBox:SetColour(col)
    self.bg:SetTintColour(col)
    return self
end

function TextBox:SetSpeechBubble( speaker, style )
    self.speaker = speaker
    self.style = style or STYLE.SPEECH
    self.text:SetGlyphColour(UICOLOURS.CONVO_SPEECH_TEXT)
    self:DoLayout()
    return self
end

function TextBox:GetSpeaker()
    return self.speaker
end

function TextBox:SetText(txt, icon)
    if self.txt ~= txt then
        self.txt = txt

        if self.icon then
            self.icon:Remove()
            self.icon = nil
        end
        if icon then
            self.icon = self.content:AddChild( icon )
        end

        self.text:SetText(txt)
        self:DoLayout()
    end

    return self
end

function TextBox:SetPadding(w)
    self.padding = w
    self:DoLayout()
    return self
end

function TextBox:SetMaxWidth(w)
    self.max_w = w
    self:DoLayout()
    return self
end

function TextBox:DoLayout()
    if self.txt then
        if self.style == STYLE.SPEECH or self.style == STYLE.SPEECH_LEFT or self.style == STYLE.SPEECH_RIGHT then
            if self.speaker and self.speaker:IsPlayer() then
                self.bg:SetTexture(assets.speech_player)
                self.tail:SetTexture( assets.tail_player )
            else
                self.bg:SetTexture(assets.speech)
                self.tail:SetTexture( assets.tail )
            end

        elseif self.style == STYLE.TUTORIAL then
            self.bg:SetTexture(assets.tutorial_text)
        elseif self.style == STYLE.TLDR then
            self.bg:SetTexture(assets.tldr_text)
        else
            self.bg:SetTexture(assets.text)
        end

        local icon_width, icon_height, icon_xmax = 0, 0, 0
        if self.icon then
            icon_width, icon_height = self.icon:GetSize()
            icon_xmax = self.icon.x + icon_width
        end
        local bw = self.max_w - ICON_PADDING - icon_xmax
        self.text:SetRegionSize(bw, RES_Y)

        local textw, texth = self.text:GetRenderSize()
        self.text:SetRegionSize(textw, texth)
        if self.icon then
            self.icon:LayoutBounds("after","center",self.text):Offset( ICON_PADDING, 0 )
        end

        self.bg:SetInnerSize( textw + ICON_PADDING + icon_xmax * 2 + self.padding * 2, math.max( self.min_height, icon_height + self.padding * 1.5, texth + self.padding * 1.5 ))
        self.w, self.h = self.bg:GetSize()

        if self.style == STYLE.SPEECH_LEFT then
            self.tail:SetScale( 1, 1 )
            self.tail:LayoutBounds( "before", "bottom", self.bg ):Offset( 2, 0 ):Show()
        elseif self.style == STYLE.SPEECH_RIGHT then
            self.tail:SetScale( 1, 1 ) -- need to reset scale before LayoutBounds
            self.tail:LayoutBounds( "after", "bottom", self.bg ):Offset( -2, 0 ):SetScale( -1, 1 ):Show()
        else
            self.tail:Hide()
        end
    end
end

function TextBox:GetSize()
    return self.w, self.h
end




