require "ui/widget"

local GAIN_ACTION_CLR = 0xA9FFFCff

local TimeIndicator = class( "Widget.TimeIndicator", Widget )

function TimeIndicator:init()
    TimeIndicator._base.init( self )
    self.hover_check = true
    
    self.w = 140
    self.h = self.w * 0.666666

    self.outerglow = self:AddChild( Widget.Image( engine.asset.Texture("UI/actionscounter_outerglow.tex"), self.w, self.h ) ):SetBloom( 0.3 ):SetTintAlpha(0)
    self.panel = self:AddChild( Widget.Image( engine.asset.Texture("UI/actionscounter_bg.tex"), self.w, self.h ) ):SetBloom( 0.05 )
    self.innerglow = self:AddChild( Widget.Image( engine.asset.Texture("UI/actionscounter_innerglow.tex"), self.w, self.h ) ):SetBloom( 0.1 ):SetTintAlpha(0)

    self.count = self:AddChild( Widget.Label( "body", 50 ) ):SetBloom( 0.1 )
    self.count:LayoutBounds( "center", "center", self.panel ):Offset( 0, 2 )

    self.label = self:AddChild( Widget.Label( "title", 20, LOC"UI.TIME_INDICATOR.ACTIONS" ) ):SetGlyphColour( UICOLOURS.SUBTITLE ):SetBloom( 0.1 )
    self.label:LayoutBounds( "center", "below", self.panel ):Offset( 0, 10 )

    self:SetToolTip(LOC"UI.TIME_INDICATOR.TOOLTIP")

    self.anim_pacing = 0.2
end

function TimeIndicator:ShowLabel( show )
    self.label:SetShown( show )
    return self
end

function TimeIndicator:SetTime( value, max_value )
    self.value = value
    self.max_value = max_value
    return self:FormatTime( loc.format( "{1}/{2}", value, max_value ))
end

function TimeIndicator:RestoreTime( value, max_value )
    self.count:ColourFromCurrentTo(9/255,225/255,220/255,1, self.anim_pacing/2, easing.linear)
    self.panel:ColourFromCurrentTo(1,1,1,1, self.anim_pacing/2, easing.linear)
    self:AnimateNumberChange( value, max_value, true)
    return self
end

function TimeIndicator:FormatTime( str )
    self.count:SetText( str )
    return self
end

function TimeIndicator:SetReflectionShown( show )
    if self.reflection then self.reflection:SetShown( show ) end
    return self
end

function TimeIndicator:ShowReflection( height, yoffset )
    local w, h = self.panel:GetSize()
    self.reflection = self:AddChild( Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ) ):SetSize(w + 20 , height or 40):SetBloom(0.1):SetHiddenBoundingBox(true):SetTintColour(UICOLOURS.SUBTITLE)
    self.reflection:LayoutBounds("center","below",self.panel):Offset(0, yoffset or -20)
    return self
end

function TimeIndicator:AnimateNumberChange( value, max_value, no_sound )
    if self.value ~= value then
        local old_value = self.value
        self.value = value
        self.max_value = max_value

        self.count:SetTintColour( GAIN_ACTION_CLR )
        self.count:ScaleTo( 1.0, 1.6, 0.1, easing.outCubic)
        self.innerglow:AlphaTo( 0.2, 0.1, easing.linear)
        self.outerglow:AlphaTo( 0.5, 0.1, easing.linear)
        self.count:Delay( 0.25 )

        if not old_value or value > old_value then
            if not no_sound then
                AUDIO:PlayEvent(  "event:/sfx/battle/cards/gen/actions_increase")
            end
        end
        self:SetTime( value, max_value )

        self.count:SetTintColour( UICOLOURS.SUBTITLE )
        self.count:ScaleTo( 1.6, 1.0, 0.3, easing.outQuad)
        self.innerglow:AlphaTo( 0, 0.3, easing.linear)
        self.outerglow:AlphaTo( 0, 0.3, easing.linear)
    end
end

function TimeIndicator:AnimateOutOfTime()
    if self.animating then return end
    local count_r,count_g,count_b,count_a = self.count.tint_r or 1, self.count.tint_g or 1, self.count.tint_b or 1, self.count.tint_a or 1
    self:RunUpdater(
        Updater.Series{
            Updater.Do( function() self.animating = true end ),
            Updater.Do( function() self.count:ColourFromCurrentTo(240/255,80/255,90/255,0.8, self.anim_pacing/2, easing.linear) end ),
            Updater.Wait( self.anim_pacing*4 ),
            Updater.Do( function() self.count:ColourFromCurrentTo(count_r,count_g,count_b,count_a, self.anim_pacing, easing.linear) end ),
            Updater.Wait( self.anim_pacing ),
            Updater.Do( function() self.animating = false end )
        }
    )
end

function TimeIndicator:PrepareAnimation()
    self.panel:SetTintAlpha(0)
    self.count:SetTintAlpha(0)
    self.label:SetTintAlpha(0)
    if self.reflection then self.reflection:SetTintAlpha(0) end
end

function TimeIndicator:AnimateIn()
    -- Prepare for animation by getting current values
    local panel_x, panel_y = self.panel:GetPos()
    local count_x, count_y = self.count:GetPos()
    local count_scale_x, count_scale_y = self.count:GetScale()
    local label_x, label_y = self.label:GetPos()

    -- Start animation
    self:RunUpdater(
        Updater.Parallel{
            Updater.Parallel{
                Updater.Ease2D( function(x,y) self.panel:SetPos(x,y) end, panel_x, panel_y - 30, panel_x, panel_y, self.anim_pacing, easing.linear),
                Updater.Ease( function(v) self.panel:SetTintAlpha(v) end, 0, 1, self.anim_pacing*2, easing.linear),
                Updater.Series{
                    Updater.Wait(self.anim_pacing),
                    Updater.Parallel{
                        Updater.Ease2D( function(x,y) self.count:SetPos(x,y) end, count_x, count_y - 10, count_x, count_y, self.anim_pacing/2, easing.linear),
                        Updater.Ease2D( function(x,y) self.count:SetScale(x,y) end, count_scale_x*1.4, count_scale_y*1.4, count_scale_x, count_scale_y, self.anim_pacing/2, easing.linear),
                        Updater.Ease( function(v) self.count:SetTintAlpha(v) end, 0, 1, self.anim_pacing/2, easing.linear),
                    },
                }
            },
            Updater.Series{
                Updater.Wait(self.anim_pacing),
                Updater.Parallel{
                    Updater.Ease2D( function(x,y) self.label:SetPos(x,y) end, label_x, label_y - 10, label_x, label_y, self.anim_pacing, easing.linear),
                    Updater.Ease( function(v) self.label:SetTintAlpha(v) end, 0, 1, self.anim_pacing*2, easing.linear),
                }
            }
        }
    )
    if self.reflection then self.reflection:AlphaTo( 0.6, self.anim_pacing, easing.outQuad ) end
end
