require "ui/widget"
require "ui/widgets/anim"
require "ui/widgets/animatedcharacter"

local TinyPortrait = class("Widget.TinyPortrait", Widget)

function TinyPortrait:init( agent, height )
    TinyPortrait._base.init(self)
    self.agent = agent

    self.highlight_widget = Widget()

    self.portrait_h = 128
    self.portrait_bg = Widget.Image( engine.asset.Texture( "UI/tinyportrait_bg.tex" ) ):SetSize( self.portrait_h, self.portrait_h ):SetBloom( 0.05 )
    self.portrait_mask = Widget.Image( engine.asset.Texture( "UI/tinyportrait_mask.tex" ) ):SetSize( self.portrait_h, self.portrait_h )
    self.portrait_top = Widget.Image( engine.asset.Texture( "UI/tinyportrait_top.tex" ) ):SetSize( self.portrait_h, self.portrait_h )
    
    self.anim_mask = Widget()
    self.anim_scale = Widget()

    self.anim = Widget.AnimatedCharacter():SetPos(0, -self.portrait_h/2):SetPortraitMode()
    self.anim.hover_check = true
    self.anim:SetHiddenBoundingBox(true)
    self.anim:DisableBlink()
    self.anim:Refresh( agent ):Show()
    self.anim:Play( "portrait_neutral" )

    self:AddChildren
    {
        self.highlight_widget:AddChildren
        {
            self.portrait_bg,
            self.anim_mask:SetStencilContext( true ):AddChildren
            {
                self.portrait_mask:SetMask(true),
                self.anim_scale:AddChildren 
                {
                    self.anim:SetStencilTest(true)
                }
            },
            self.portrait_top
        }
    }

    self:SetHeight( 30 )

end

function TinyPortrait:SetHeight( h )
    local s = h/self.portrait_h 
    self.highlight_widget:SetScale(s, s)
    self.arth = s*self.portrait_h
    self.anim:SetPos( 0, -self.arth/2 + 5 )
    self:SetFacing(self.facing or true)
    return self
end

function TinyPortrait:SetFacing(facing)
    local sx = (facing and 1 or -1)
    self.facing = facing
    self.anim_mask:SetScale( sx, 1 )
    return self
end