require "ui/widget"
require "quest"

local assets = 
{
    highlight_arrow = engine.asset.Texture( "UI/arrow_button_normal.tex" )
}

local TRACKED_STATE = MakeEnum{"ALL", "NONE", "SOME"}
local INTER_ENTRY_GUTTER = 15
local INTER_COLLAPSED_ENTRY_GUTTER = -6

local IMG_SZ = 32
local TITLE_FONT_SZ = 30
local BODY_FONT_SZ = 23

local TEXT_W = 300
local OUTER_MARGIN = 12
local BODY_TEXT_INSET = 15
local BAR_WIDTH = 0
local BAR_SPACING = 6
local MAIN_OVERLAY_QUEST_ICON_BUTTON_WIDTH = 35
local ENTRY_W = BAR_WIDTH + BAR_SPACING + IMG_SZ + TEXT_W + MAIN_OVERLAY_QUEST_ICON_BUTTON_WIDTH + OUTER_MARGIN*2
local TITLE_FUDGE_FACTOR = 2
local FUDGE_OBJECTIVE_NUDGE = 2

local ToDoListEntry = class("Widget.ToDoListEntry", Widget)

function ToDoListEntry:init(to_do_list, quest)
    ToDoListEntry._base.init(self)

    self.to_do_list = to_do_list
    self.quest = quest

    self.normal_texture = engine.asset.Texture("UI/todolist_back.tex" )
    self.hover_texture = engine.asset.Texture("UI/todolist_back_hover.tex" )

    self.hover_check = true
    self.contents = self:AddChild( Widget() )
    local bar_color = quest:GetType() == QTYPE.STORY and UICOLOURS.QUEST or 0x00000000
    self.bar = self.contents:AddChild( Widget.SolidBox( BAR_WIDTH, IMG_SZ, bar_color ) )
    self.img = self.contents:AddChild( Widget.Image( quest:GetIcon() ):SetSize( IMG_SZ, IMG_SZ ) )
    self.header_label = self.contents:AddChild( Widget.Label( "body", 20, LOC"UI.TO_DO_LIST_LABEL" ) )
        :SetGlyphColour( UICOLOURS.QUEST )
    self.header_text = self.contents:AddChild( Widget.Label( "title", TITLE_FONT_SZ ) )
        :SetWordWrap( true )
        :SetAutoSize( TEXT_W )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.QUEST_LIGHT )
        :SetHoverLinkGlyphColour( UICOLOURS.QUEST_LIGHT )
    self.objective_root = self.contents:AddChild( Widget() )
    self.objectives = {}
    
    self:SetToolTipFn(function() 
        return self.quest:GetDesc()
    end)

    self:Refresh()
end

function ToDoListEntry:_DoFlash()

    local t = 0
    local TIME_IN = .3
    local TIME_OUT = .1
    local ADD_AMT = 3
    local MAX_BLOOM = .1

-- t = elapsed time
-- b = begin
-- c = change == ending - beginning
-- d = duration (total time)

    while t < TIME_IN do
        local dt = coroutine.yield()
        t = t + dt
        local a = easing.linear(t, 1, ADD_AMT, TIME_IN)
        self:SetTintColour(a,a,a,1)
    end
    
    self:Delay(.1)
    AUDIO:PlayEvent( SoundEvents.todolist_flash )
    local t = 0
    while t < TIME_OUT do
        local dt = coroutine.yield()
        t = t + dt
        local a = easing.linear(t, 1+ADD_AMT, -ADD_AMT, TIME_OUT)
        self:SetTintColour(a,a,a,1)
    end

    self:SetTintColour(1,1,1,1)
    self.coro = nil
end

function ToDoListEntry:Flash()
    if not self.coro then
        self.coro = self:StartCoroutine(ToDoListEntry._DoFlash, self)
    end
end

function ToDoListEntry:Refresh( )
    
    local changed = false
    
    self.header_text:SetText(self.quest:GetTitle())
    
    local next_objective_index = 1
    
    for i,obj in ipairs(self.quest.objectives) do
        if self.quest:IsActive(obj.id) and not self.quest:GetQuestDef():GetObjective(obj.id).hide_in_overlay then
            
            local title_str = self.quest:GetObjectiveTitle(obj.id)
            if title_str then

                --local obj_str = 
                if not self.objectives[next_objective_index] then
                    changed = true
                    local t= {}
                    self.objectives[next_objective_index] = t
                    t.w = self.objective_root:AddChild(Widget())
                    t.title = t.w:AddChild( Widget.Label("body", BODY_FONT_SZ, title_str ) )
                    t.title:SetAutoSize(TEXT_W-16):SetWordWrap(true):LeftAlign():SetHoverLinkFn( UIHelpers.ItemTooltips ):SetClickLinkFn( UIHelpers.ItemTooltipsClicked )
                    t.title:SetGlyphColour( UICOLOURS.TITLE )
                    t.title:SetHoverLinkGlyphColour( UICOLOURS.TEXT_LINK )
                    t.bullet = t.w:AddChild( Widget.Image( global_images.circle ) ):SetSize( 4,4 )
                    t.bullet:SetTintColour( UICOLOURS.TITLE )
                    t.w.title = t.title
                    t.title:Spool(300)

                    t.w:SetToolTipFn(function() 
                        return self.quest:GetObjectiveDesc(obj.id)
                    end)

                else
                    if self.objectives[next_objective_index].title:GetText() ~= title_str then
                        changed = true
                        self.objectives[next_objective_index].title:SetText(title_str):Spool(300)
                    end
                end
                next_objective_index = next_objective_index + 1
            end
        end
    end
    
    for k,obj_table in ipairs(self.objectives) do
        if k < next_objective_index then
            obj_table.w:Show()
            if next_objective_index > 2 then
                obj_table.bullet:Show()
                obj_table.title:LayoutBounds("after", "top", obj_table.bullet):Offset(4, BODY_FONT_SZ/2 - 2)
            else
                obj_table.bullet:Hide()
            end
            if k > 1 then
                obj_table.w:LayoutBounds("left", "below"):Offset(0, 2)
            end
        else
            obj_table.w:Hide()
        end
    end


    self.bar:LayoutBounds( "before", "top", self.img ):Offset( -BAR_SPACING, 0 )
    self.header_label:LayoutBounds( "after", "top", self.img ):Offset( OUTER_MARGIN, TITLE_FUDGE_FACTOR )
    self.header_text:LayoutBounds( "left", "below", self.header_label ):Offset( 0, 3 )
    self.objective_root:LayoutBounds( "left", "below", self.header_text ):Offset( 0, FUDGE_OBJECTIVE_NUDGE )

    self.needs_layout = false
    local is_terminal = self.quest:IsReadyForTurnIn()
    local is_high_priority = self.quest:IsHighPriority()
    if changed then
        self.changed = true
    end
end


------------------------------------------------------------------------------------------------------------------------------------------------------------------

local ToDoList = class("Widget.ToDoList", Widget)
function ToDoList:init()
    ToDoList._base.init(self)

    self.entries = {}    
    self.tracked_quests = {}
    self:StartUpdating()
    self:ListenForGlobalEvent( "quests_changed", ToDoList.OnQuestChanged )
    self:ListenForGlobalEvent( "phase_change", ToDoList.Refresh )

    self.item_root = self:AddChild( Widget() )
    self.extra_entries = self:AddChild( Widget() ):Hide()
    self.extra_entries:SetHiddenBoundingBox( true )

    self.needs_layout = true
    self.hover_check = true
end

function ToDoList:SetOnChangeFn( fn )
    self.onChangeFn = fn
    return self
end

function ToDoList:OnQuestChanged(quest)
    if (self.tracked_quests[quest] == nil) == quest:IsQuestVisibleInOverlay() then
        if self.tracked_quests[quest] then
            self:RemoveEntry(self.tracked_quests[quest])
            if self.onChangeFn then self.onChangeFn(true) end
        else
            self:AddEntry(ToDoListEntry(self, quest))
            if self.onChangeFn then self.onChangeFn(true) end
        end
    elseif self.tracked_quests[quest] then
        self.tracked_quests[quest]:Refresh()
        self.needs_layout = true
        if self.onChangeFn then self.onChangeFn(true) end
    end
end

function ToDoList:AddEntry(entry)
    table.insert(self.entries, entry)
    self.tracked_quests[entry.quest] = entry
    self.item_root:AddChild(entry)
    self.needs_layout = true
    return entry
end

function ToDoList:RemoveEntry(entry)
    self.tracked_quests[entry.quest] = nil
    table.arrayremove(self.entries, entry)
    entry:Remove()
    self.needs_layout = true
end


local QUEST_TYPE_SORT = {
    [QTYPE.STORY] = 4,
    [QTYPE.CONTRACT] = 2,
    [QTYPE.SIDE] = 3,
}

local sort_fn = function(a,b) 
    if (QUEST_TYPE_SORT[a.quest:GetType()] or 0) == (QUEST_TYPE_SORT[b.quest:GetType()] or 0) then
        return a.quest:GetID() < b.quest:GetID()
    else 
        return (QUEST_TYPE_SORT[a.quest:GetType()] or 0) < (QUEST_TYPE_SORT[b.quest:GetType()] or 0)
    end
end


function ToDoList:DoLayout()

    if self.needs_layout then
        table.sort(self.entries, sort_fn)

        local prev
        for k, entry in ipairs(self.entries) do
            if prev then
                entry:LayoutBounds( "left", "below", prev ):Offset( 0, -INTER_ENTRY_GUTTER )
                entry.pos_x, entry.pos_y = entry:GetPos()
            else
                entry:SetPos( 0, 0 )
                entry.pos_x, entry.pos_y = 0, 0
            end
            prev = entry
        end

        self.extra_entries:DestroyAllChildren()

        if #self.entries > 1 then
            -- Go through every entry past the first, and add a new extra-entry indicator to show
            -- there are more quests when they're collapsed
            local width
            local height
            local alpha
            for k, entry in ipairs(self.entries) do
                if k == 1 then
                    width, height = entry:GetSize()
                    alpha = 1
                else
                    width = width - 5
                    height = height + 5
                    alpha = alpha - 0.25

                    local new_extra = self.extra_entries:AddChild( Widget.Panel( engine.asset.Texture("UI/todolist_extra.tex" ), 1, 1, .2, .2) )
                    new_extra:SetSize( width, height )

                    local tint_r, tint_g, tint_b = entry.tint_r, entry.tint_g, entry.tint_b 
                    new_extra:SetTintColour( tint_r, tint_g, tint_b, alpha )

                    new_extra:LayoutBounds( "right", "top" )
                end
            end
            self.extra_entries:LayoutBounds( "right", "top", self.entries[1] )
        end

        if self.expanded then
            -- self:ExpandList( false )
        else
            -- self:CollapseList( false )
        end

    end
    self.needs_layout = false
end

function ToDoList:OnUpdate(dt)
    ToDoList._base.OnUpdate( self, dt )

    -- I feel dirty
    local no_flash = false
    if not self.first_update then
        self:Refresh()
        no_flash = true
        self.first_update = true
    end

    self:DoLayout()

    for quest,entry in pairs(self.tracked_quests) do
        if entry.changed then
            if not no_flash then
                local w = Widget.Image(assets.highlight_arrow):SetSize(20, 40):SetHiddenBoundingBox(true)
                entry:AddChild(w)
                w:SetPos(-300, 0)
                w:SetTintColour(1,1,1,0)
                w:AlphaTo(1, .3, easing.inCubic)
                w:MoveTo( -40, 0, .5, easing.inCubic, function( ) entry:Flash() w:AlphaTo(0, 2, easing.inCubic, function() w:Remove() end) end) 
            end
            entry.changed = nil
        end
    end

end


function ToDoList:Refresh()

    if TheGame:GetGameState() then
        local items = ObtainWorkTable()

        --our current quests
        for i, quest in TheGame:GetGameState():ActiveQuests() do
            if quest:IsQuestVisibleInOverlay() then
                items[quest] = true
            end
        end

        for quest, entry in pairs(self.tracked_quests) do
            if not items[quest] then
                self:RemoveEntry(entry)
                if self.onChangeFn then self.onChangeFn(true) end
            end
        end

        for k, entry in pairs(self.entries) do
            entry:Refresh()
        end

        for quest in pairs(items) do
            if not self.tracked_quests[quest] then
                self:AddEntry(Widget.ToDoListEntry(self, quest))
                if self.onChangeFn then self.onChangeFn(true) end
            end
        end

        self.needs_layout = true

        ReleaseWorkTable(items)
    end

end

function ToDoList:UpdateImage()
    if self.enabled and ( self.hover or self.focus ) then
        -- self:Bloom(0.1)

        -- Expand quests list
        -- if not self.expanded then self:ExpandList( true ) end
    else
        -- self:Bloom(0)

        -- Collapse quests list
        -- if self.expanded then self:CollapseList( true ) end
    end
end

function ToDoList:OnGainFocus()
    self:UpdateImage()
end

function ToDoList:OnLoseFocus()
    self:UpdateImage()
end

function ToDoList:OnGainHover()
    ToDoList._base.OnGainHover(self)
    self:UpdateImage()
end

function ToDoList:OnLoseHover()
    ToDoList._base.OnLoseHover(self)
    self:UpdateImage()
end

function ToDoList:ExpandList( animate )

    AUDIO:PlayEvent("event:/ui/main/gen/mouseover_objective_dropdown")
    local prev
    for k, entry in ipairs(self.entries) do
        if prev then
            if animate then
                entry:Show()
                local pos_x, pos_y = entry.pos_x, entry.pos_y
                entry:SetPos( pos_x, pos_y + 20 )
                entry:MoveTo( pos_x, pos_y, 0.1 )
                entry:AlphaTo( 1, 0.1, nil, function() entry.hover_check = true end )
            else
                entry:Show()
                entry:SetTintAlpha( 1 )
                entry.hover_check = true
            end
        else
            entry.hover_check = true
            entry:SetTintAlpha( 1 )
            entry:Show()
        end
        prev = entry
    end

    self.extra_entries:Hide()
    self.expanded = true
end

function ToDoList:CollapseList( animate )

    local prev
    for k, entry in ipairs(self.entries) do
        if prev then
            self.extra_entries:Show()
            entry.hover_check = false
            if animate then
                local pos_x, pos_y = entry.pos_x, entry.pos_y
                entry:SetPos( pos_x, pos_y )
                entry:MoveTo( pos_x, pos_y + 20, 0.1 )
                entry:AlphaTo( 0, 0.1, nil, function() entry:Hide() end )
            else
                entry:Hide()
                entry:SetTintAlpha( 0 )
            end
        else
            entry.hover_check = true
            entry:SetTintAlpha( 1 )
            entry:Show()
        end
        prev = entry
    end

    self.expanded = false
end

function ToDoList:IsEmpty()
    for k, entry in ipairs(self.entries) do
        if entry:IsShown() then
            return false
        end
    end
    return true
end
