require "ui/widget"

-------------------------------------------------------------------------
-- Displays a toast popup for achievement/unlocks notifications

local ToastWidget = class("Widget.ToastWidget", Widget )

function ToastWidget:init( icon, title, description )
    ToastWidget._base.init(self)

    self.max_w = 460
    self.max_icon_w = 90

    self.bg_border = 70
    self.bg_border_scale = 0.5
    self.background = self:AddChild( Widget.Panel( engine.asset.Texture("UI/toast_bg.tex" ) ) )
        :SetNineSliceCoords( self.bg_border, self.bg_border, self.bg_border, self.bg_border )
        :SetNineSliceBorderScale( self.bg_border_scale )
        :Bloom( 0.1 )
    self.contents = self:AddChild( Widget() )
    self.icon = self.contents:AddChild( Widget.Image( icon ) )
    self.text_container = self.contents:AddChild( Widget() )
    self.title = self.text_container:AddChild(Widget.Label( "button", FONT_SIZE.ITEM_SUBTITLE ) )
        :SetText( title )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.WHITE )
        :SetWordWrap( true )
        :OverrideLineHeight( FONT_SIZE.BUTTON*LINE_HEIGHT_TEXT_MULT*0.9 )
    self.description = self.text_container:AddChild(Widget.Label( "body", FONT_SIZE.ITEM_DESC ) )
        :SetText( description )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :SetWordWrap( true )
        :OverrideLineHeight( FONT_SIZE.BUTTON*LINE_HEIGHT_TEXT_MULT )

    self:Layout()
end

function ToastWidget:SetBackgroundColor( color )
    self.background:SetTintColour( color )
    return self
end

function ToastWidget:Layout()

    local icon_w, icon_h = 0, 0
    if self.icon:IsDefaultTexture() then -- No icon
        -- Scale the icon to match the desired width, mantaining aspect ratio
        local target_w = self.max_icon_w
        local g_w, g_h = self.icon:GetSize();
        local target_h = target_w/g_w*g_h;
        icon_w, icon_h = target_w, target_h
        self.icon:SetSize( icon_w, icon_h )
            :Show()
    else
        self.icon:Hide()
    end

    -- Set text size
    local text_w = self.max_w - SPACING.M1*3 - icon_w
    self.title:SetAutoSize( text_w )
    self.description:SetAutoSize( text_w )

    -- Layout text
    self.description:LayoutBounds( "left", "below", self.title )
        :Offset( 0, 4 )
    self.text_container:LayoutBounds( "after", "top", self.icon )
        :Offset( SPACING.M1, 0 )

    -- Scale background
    local w, h = self.contents:GetSize()
    w = w + SPACING.M1*4
    h = h + SPACING.M1*2
    local nineslice_w, nineslice_h = w + self.bg_border*2*self.bg_border_scale, h + self.bg_border*2*self.bg_border_scale
    self.background:SetSize( nineslice_w, nineslice_h )
    self.contents:LayoutBounds( "left", "top", self.background )
        :Offset( SPACING.M1*2, -SPACING.M1 )

    return self
end

function ToastWidget:Animate()


    return self
end