require "ui/widgets/tooltip"
require "ui/widgets/tooltip_card"

----------------------------------------------------------------
-- Argument tooltips.

local TooltipArgument = class("Widget.TooltipArgument", Widget.Tooltip )

local padding = 20
local bg_border = 30
local bg_border_scale = 0.5
local content_w = DEFAULT_TT_WIDTH - padding*2
local PORTRAIT_HEIGHT = 72

function TooltipArgument:init()
    TooltipArgument._base.init( self )

    self.content = self:AddChild( Widget() )

    self.header_bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.05 )
    self.header_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.header_bg:SetNineSliceBorderScale( bg_border_scale )
    self.header_bg:SetHiddenBoundingBox( true )
    self.header_bg:SetTintColour( UICOLOURS.NEGOTIATION )
    self.header_bg:SetTintAlpha( 0.2 )

    -- Header content
    self.header = self.content:AddChild( Widget() )

    -- A label for the type label
    self.corner_label_bg = self.header:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.1 )
    self.corner_label_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.corner_label_bg:SetNineSliceBorderScale( bg_border_scale )
    self.corner_label_bg:SetTintColour( UICOLOURS.NEGOTIATION )
    self.corner_label_bg:SetTintAlpha( 0.2 )

    -- The type label
    self.header_left = self.header:AddChild( Widget.Label( "body", 20 ) )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.NEGOTIATION )
        :Bloom( 0.05 )

    -- Resolve label
    self.header_right = self.header:AddChild( Widget.Label( "title", 20 )):LeftAlign():Bloom(0.1)
    self.header_right:SetAutoSize( content_w )

    -- Title
    self.argument_name = self.content:AddChild( Widget.Label("title", 25 )):LeftAlign():Bloom(0.1)
    self.argument_name:SetAutoSize( content_w )

    -- Stacks
    self.stacks_row = self.content:AddChild( Widget() )
    self.stacks_label = self.stacks_row:AddChild( Widget.Label("body", 20 )):SetGlyphColour(UICOLOURS.OBJECTIVE):SetRegionSize(content_w,20):LeftAlign():Bloom(0.05)
    self.stacks_value = self.stacks_row:AddChild( Widget.Label("body", 20 )):SetGlyphColour(UICOLOURS.OBJECTIVE):SetRegionSize(content_w,20):RightAlign():Bloom(0.09)

    -- Description
    self.desc = self.content:AddChild( Widget.Label( "tooltip", 24, "" ) )
        :SetAutoSize( content_w )
        :OverrideLineHeight( 20 )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.05)

    self.portrait_root = self.content:AddChild( Widget() ):Hide()
    self.portrait = self.portrait_root:AddChild( Widget.CharacterPortrait():SetHeight( PORTRAIT_HEIGHT ) )
    self.portrait_desc = self.portrait_root:AddChild( Widget.Label( "tooltip", 24 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :SetWordWrap( true )
        :OverrideLineHeight( 20 )
        :LeftAlign()
        :Bloom(0.05))

    self.keywords = self:AddChild( Widget.TooltipKeywords())
end

function TooltipArgument:PostFeature( keyword, name, desc )
    self.keywords:AddFeaturePanel( name, desc )
end

function TooltipArgument:PostCard( feature_id, card, engine )
    self.keywords:AddCardPanel( card, card.is_negotiation and engine )
end

function TooltipArgument:PostGraft( feature_id, graft )
    self.keywords:AddGraftPanel( graft )
end

function TooltipArgument:PostPortrait( agent, text )
    self.portrait:SetCharacter( agent )

    self.portrait_desc:SetAutoSize( content_w - self.portrait:GetSize() - 6 )
    self.portrait_desc:SetText( text ):LayoutBounds( "after", "top", self.portrait ):Offset( 6, 0 )
    self.portrait_root:Show()
end

function TooltipArgument:DoLayout( modifier )
    self.keywords:DoLayout( nil ) -- Hide previously instantiated panels.
    self.portrait_root:Hide()

    -- Get the corner label size, and scale up its background to match
    self.header_left:SetGlyphColour( UICOLOURS.NEGOTIATION )
    self.header_bg:SetTintColour( UICOLOURS.NEGOTIATION ):SetTintAlpha( 0.2 )
    self.corner_label_bg:SetTintColour( UICOLOURS.NEGOTIATION ):SetTintAlpha( 0.2 )
    local modifier_type = modifier.modifier_type or MODIFIER_TYPE.ARGUMENT
    if modifier_type == MODIFIER_TYPE.CORE then
        self.header_left:SetText( LOC"UI.NEGOTIATION_PANEL.CORE" )
    elseif modifier_type == MODIFIER_TYPE.BOUNTY then
        self.header_left:SetText( LOC"UI.NEGOTIATION_PANEL.BOUNTY" )
    elseif modifier_type == MODIFIER_TYPE.INCEPTION then
        self.header_left:SetText( LOC"UI.NEGOTIATION_PANEL.INCEPTION" )
    elseif modifier_type == MODIFIER_TYPE.PERMANENT then
        self.header_left:SetText( LOC"UI.NEGOTIATION_PANEL.PERMANENT" )
            :SetGlyphColour( UICOLOURS.SUBTITLE )
        self.header_bg:SetTintColour( UICOLOURS.SUBTITLE ):SetTintAlpha( 0.2 )
        self.corner_label_bg:SetTintColour( UICOLOURS.SUBTITLE ):SetTintAlpha( 0.2 )
    else
        self.header_left:SetText( LOC"UI.NEGOTIATION_PANEL.ARGUMENT" )
    end
    
    local label_w, label_h = self.header_left:GetSize()
    local nineslice_w, nineslice_h = label_w + bg_border*2*bg_border_scale, 68
    self.corner_label_bg:SetSize( nineslice_w+padding*2.5, nineslice_h )
    self.corner_label_bg:SetPos( 0, 0 )
    self.header_left:LayoutBounds( "left", "bottom", self.corner_label_bg ):Offset( padding, 8 )

    -- Show the header background
    local h_w, h_h = self.header:GetSize()
    local nineslice_w, nineslice_h = DEFAULT_TT_WIDTH + bg_border*2*bg_border_scale, 68
    self.header_bg:SetSize( nineslice_w, nineslice_h )
    self.header_bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )

    -- Set the resolve
    local resolve, max_resolve = modifier:GetResolve()
    if resolve and max_resolve then
        self.header_right:SetText( loc.format( LOC"UI.NEGOTIATION_PANEL.ARGUMENT_RESOLVE", resolve, max_resolve ) )
    else
        self.header_right:SetText( "<b>"..LOC"UI.NEGOTIATION_PANEL.NO_RESOLVE".."</>" )
    end
    self.header_right:LayoutBounds( "right", "bottom", self.header_bg ):Offset( -padding, 9 )

    -- Set title
    self.argument_name:SetText( modifier:GetName() )
    self.argument_name:LayoutBounds( "left", "below", self.corner_label_bg ):Offset( padding, -6 )

    -- Set the strength
    if not modifier.hide_stacks then
        local stacks = modifier:GetStacks()
        self.stacks_label:SetText( LOC"UI.NEGOTIATION_PANEL.STRENGTH" )
        self.stacks_value:SetText( loc.format( LOC"UI.NEGOTIATION_PANEL.ARGUMENT_STRENGTH", tostring(stacks) ) )
        self.stacks_row:Show()
            :LayoutBounds( "left", "below", self.argument_name )
            :Offset( 0, -2 )
    else
        self.stacks_row:Hide()
            :LayoutBounds( "left", "bottom", self.argument_name )
    end

    -- Set the description
    local desc = modifier:GetDesc( nil, self )
    self.desc:SetText( desc )
    self.desc:LayoutBounds( "left", "below", self.stacks_row ):Offset( 0, -6 )

    if self.portrait_root:IsShown() then
        self.portrait_root:LayoutBounds( "left", "below", self.desc ):Offset( 0, -8 )
    end

    -- Size up the background
    local c_w, c_h = self.content:GetSize()
    self.bg:SetSize( DEFAULT_TT_WIDTH, c_h + padding )
    self.bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )

    self.keywords:TransitionPanels()
    self.keywords:LayoutBounds( "left", "below", self.bg )
end
