require "ui/widgets/tooltip"
local battle_defs = require "battle/battle_defs"


-----------------------------------------------------------------------------
-- Card tooltip, showing its name, type, rarity, upgrade progress, and keywords

local FullCardPanel = class("Widget.FullCardPanel", Widget.Tooltip )

local padding = 20
local bg_border = 30
local bg_border_scale = 0.5
local HEADER_H = 68
local content_w = DEFAULT_TT_WIDTH - padding*2
local FLAVOUR_W = 300

function FullCardPanel:init()
    FullCardPanel._base.init( self )

    self.content = self:AddChild( Widget() )

    -- A header bg
    self.header_bg = self.content:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.05 )
    self.header_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.header_bg:SetNineSliceBorderScale( bg_border_scale )
    self.header_bg:SetHiddenBoundingBox( true )

    -- Header content
    self.header = self.content:AddChild( Widget() )

    -- A label for the type label
    self.corner_label_bg = self.header:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.1 )
    self.corner_label_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.corner_label_bg:SetNineSliceBorderScale( bg_border_scale )

    -- The type label
    self.corner_label = self.header:AddChild( Widget.Label("body", 20 ) ):LeftAlign():Bloom( 0.05 )

    -- Titles
    self.subtitle = self.header:AddChild( Widget.Label("body", 20 )):RightAlign():Bloom(0.05)
    self.subtitle:SetGlyphColour( UICOLOURS.SUBTITLE )

    -- For all the description content
    self.stats = self.content:AddChild( Widget() )

    -- Charges
    self.charges_row = self.stats:AddChild( Widget() )
    self.charges_label = self.charges_row:AddChild( Widget.Label("body", 20 )):LeftAlign():Bloom(0.05)
    self.charges_value = self.charges_row:AddChild( Widget.Label("body", 20 )):RightAlign():Bloom(0.09)

    -- Upgrade
    self.upgrade_row = self.stats:AddChild( Widget() )
    self.upgrade_label = self.upgrade_row:AddChild( Widget.Label("body", 20 )):LeftAlign():Bloom(0.05)
    self.upgrade_value = self.upgrade_row:AddChild( Widget.Label("body", 20 )):RightAlign():Bloom(0.09)

    self.remove_timer_row = self.stats:AddChild( Widget() )
    self.remove_timer_label = self.remove_timer_row:AddChild( Widget.Label("body", 20 )):LeftAlign():Bloom(0.05)
    self.remove_timer_value = self.remove_timer_row:AddChild( Widget.Label("body", 20 )):RightAlign():Bloom(0.09)

    -- Keywords
    self.keyword_entries = self.content:AddChild( Widget() )
    self.flavour_text = self.content:AddChild( Widget.Label("body", 20 ):SetAutoSize( FLAVOUR_W ):SetWordWrap( true ):LeftAlign() )

    -- Card widget previews
    self.card_entries = self:AddChild( Widget() )
end

function FullCardPanel:DoLayout( data )
    

    local card = data.CARD or self.card 
    local features = data.FEATURES or self.features

    local card_class = card._class

    -- Set appropriate colors!
    if card_class == Negotiation.Card then
        self.corner_label:SetText( LOC"UI.CARD_WIDGET.CARD_CLASS_NEGOTIATION" )
        self.corner_label:SetGlyphColour( UICOLOURS.NEGOTIATION )
        self.corner_label_bg:SetTintColour( UICOLOURS.NEGOTIATION )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.NEGOTIATION )
        self.header_bg:SetTintAlpha( 0.2 )
    elseif card_class == Battle.Card then
        self.corner_label:SetText( LOC"UI.CARD_WIDGET.CARD_CLASS_BATTLE" )
        self.corner_label:SetGlyphColour( UICOLOURS.FIGHT )
        self.corner_label_bg:SetTintColour( UICOLOURS.FIGHT )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.FIGHT )
        self.header_bg:SetTintAlpha( 0.2 )
    end

    -- Get the corner label size, and scale up its background to match
    local label_w, label_h = self.corner_label:GetSize()
    local nineslice_w, nineslice_h = label_w + bg_border*2*bg_border_scale, HEADER_H
    self.corner_label_bg:SetSize( nineslice_w+padding*2.5, nineslice_h )
    self.corner_label_bg:SetPos( 0, 0 )
    self.corner_label:LayoutBounds( "left", "bottom", self.corner_label_bg ):Offset( padding, 8 )

    -- Show the header background
    local h_w, h_h = self.header:GetSize()
    local nineslice_w, nineslice_h = DEFAULT_TT_WIDTH + bg_border*2*bg_border_scale, HEADER_H
    self.header_bg:SetSize( nineslice_w, nineslice_h )
    self.header_bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )

    -- Show duplicate info, if relevant
    self:RefreshDuplicate( card, nineslice_w, nineslice_h )

    -- Set rarity and type
    local rarity_color = MakeColourString( HexColour( CARD_RARITY_COLOURS[ card.rarity or CARD_RARITY.UNIQUE ] ) )
    local rarity_name = GetCardRarityString( card.rarity )
    local card_type = GetCardType( card )
    local type_color = MakeColourString( HexColour( GetCardTypeColour( card_type ) ) )
    local type_name = GetCardTypeString( card_type )
    local subtitle_text = loc.format( LOC"UI.CARD_WIDGET.DETAILS_SUBTITLE", rarity_color, rarity_name, type_color, type_name )
    self.subtitle:SetSize( DEFAULT_TT_WIDTH - label_w - bg_border*2*bg_border_scale - 2*padding, label_h )
    self.subtitle:SetText( subtitle_text )
    self.subtitle:TryFitToRegion( 16, 20 )
    self.subtitle:LayoutBounds( "right", "bottom", self.header_bg ):Offset( -padding, 8 )

    self.stats:Hide()

    -- Available uses
    if card:GetCharges() then
        self.charges_row:Show()
        self.stats:Show()
        local charges, max_charges = card:GetCharges()
        self.charges_label:SetText( LOC"UI.CARD_WIDGET.CHARGES_LABEL" )
        self.charges_value:SetText( loc.format( LOC"UI.CARD_WIDGET.CHARGES_VALUE", charges, max_charges ) )
    else
        self.charges_row:Hide()
    end

    -- if card.remove_on_rest then
    --     self.remove_timer_row:Show()
    --     self.stats:Show()
    --     self.remove_timer_label:SetText( LOC"UI.CARD_WIDGET.REMOVE_ON_REST_LABEL" )
    --     self.remove_timer_value:Hide()
    -- else
    --     self.remove_timer_row:Hide()
    -- end


    -- Upgrade progress
    local card_xp, max_xp = card:GetXP()
    if card_xp and max_xp then
        self.upgrade_row:Show()
        self.stats:Show()
        -- Use bright colors if this card is ready to upgrade
        if card:UpgradeReady() then
            self.upgrade_label:SetText( LOC"UI.CARD_WIDGET.UPGRADE_LABEL_READY" ):SetGlyphColour( UICOLOURS.QUEST )
            self.upgrade_value:SetText( loc.format( LOC"UI.CARD_WIDGET.UPGRADE_VALUE", card_xp, max_xp ) ):SetGlyphColour( UICOLOURS.QUEST )
        else
            self.upgrade_label:SetText( LOC"UI.CARD_WIDGET.UPGRADE_LABEL" ):SetGlyphColour( UICOLOURS.OBJECTIVE )
            self.upgrade_value:SetText( loc.format( LOC"UI.CARD_WIDGET.UPGRADE_VALUE", card_xp, max_xp ) ):SetGlyphColour( UICOLOURS.OBJECTIVE )
        end
    else
        self.upgrade_row:Hide()
    end

    -- Layout the stats
    self.stats:StackChildren( 6 )

    -- Go through all the keywords for this card
    self.card_entries:DestroyAllChildren()
    self.card_entries:Hide()
    self.keyword_entries:DestroyAllChildren()
    self.keyword_entries:Hide()
    self.flavour_text:Hide()
    if features then
        for k, v in ipairs( features ) do
            if v.card then
                local row
                if is_instance( v.card, Battle.Card ) then
                    row = Widget.BattleCard( v.card )
                elseif is_instance( v.card, BattleCardDef ) then
                    row = Widget.BattleCard( Battle.Card( v.card.id ))
                elseif is_instance( v.card, Negotiation.Card ) then
                    row = Widget.NegotiationCard( v.card )
                elseif is_instance( v.card, NegotiationCardDef ) then
                    row = Widget.NegotiationCard( Negotiation.Card( v.card.id ))
                end
                if row then
                    -- Check if this card has features as well, and pile them on here
                    if row.card_features then
                        for j, l in ipairs( row.card_features ) do
                            local found = false
                            for m, n in ipairs( features ) do
                                if n.id == l.id then found = true end
                            end
                            if not found then table.insert( features, l ) end
                        end
                    end
                    -- Style the card
                    row:SetScale( 0.5 )
                        :SetOutline( UICOLOURS.SUBTITLE )
                    self.card_entries:AddChild( row )
                end
            else
                local entry = loc.format( LOC"UI.CARD_WIDGET.KEYWORD_ENTRY", v.name, v.desc )
                local row = Widget.Label("tooltip",20,entry):SetAutoSize( content_w ):SetWordWrap( true ):LeftAlign():SetGlyphColour( UICOLOURS.TOOLTIP_TEXT ):Bloom(0.05)
                self.keyword_entries:AddChild( row )
            end
        end
    end

    if card.engine then
        if is_instance( card, Negotiation.Card ) then
            local min_persuasion, max_persuasion, details = card.engine:PreviewPersuasion( card )
            if details then
                local entry = loc.format( LOC"UI.NEGOTIATION_PANEL.EFFECT_DETAILS_TOOLTIP", details )
                local row = Widget.Label("tooltip",20,entry):SetAutoSize( content_w ):SetWordWrap( true ):LeftAlign():SetGlyphColour( 0x09E1DCd0 ):Bloom(0.05)        
                self.keyword_entries:AddChild( row )
            end
        end
    end

    if not self.keyword_entries:IsEmpty() then
        self.keyword_entries:Show()
        self.keyword_entries:StackChildren( 5 )
    end

    local sm = TheGame:FE():GetScreenMode()
    if not self.card_entries:IsEmpty() and sm == SCREEN_MODE.MONITOR then
        self.card_entries:Show()
        self.card_entries:StackChildrenRow( -10 )
    end

    -- Layout everything!
    -- self.stats:LayoutBounds( "left", nil, self.subtitle ):Offset( 0, 0 )
    self.stats:LayoutBounds( "left", "below", self.header ):Offset( padding, -8 )
    self.keyword_entries:LayoutBounds( "left", "below", self.stats ):Offset( 0, self.stats:IsShown() and -10 or -5 )
    

    local flavour = card:GetFlavourText()
    if flavour then
        self.flavour_text:SetText(loc.format("<i><#FLAVOUR_TEXT>{1}</></>", flavour))
        self.flavour_text:LayoutBounds("left", "below", self.keyword_entries):Offset(0, -10):Show()
    end

    -- Size up the background
    local c_w, c_h = self.content:GetSize()
    self.bg:SetSize( DEFAULT_TT_WIDTH, c_h + padding )
    self.bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )
    
    self.card_entries:LayoutBounds( "after", "top", self.bg ):Offset( 0, 0 )
    self.remove_timer_value:LayoutBounds( "right", nil, self.bg ):Offset( -padding, 0 )
    self.charges_value:LayoutBounds( "right", nil, self.bg ):Offset( -padding, 0 )
    self.upgrade_value:LayoutBounds( "right", nil, self.bg ):Offset( -padding, 0 )

end

function FullCardPanel:RefreshDuplicate( card, w, h )
    if card and card:IsTemporary() then
        if not self.duplicate_widget then
            self.duplicate_widget = self.header:AddChild( Widget() )
                :SetStencilContext( true )
            self.duplicate_mask_image = Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") )
                :SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
                :SetNineSliceBorderScale( bg_border_scale )
                :SetSize( w, h )
            self.duplicate_smoke1 = Widget.Image( engine.asset.Texture( "particles/smoke.tex") )
                :SetHiddenBoundingBox( true )
                :SetSize( 900, 900 )
                :SetTintColour( UICOLOURS.TITLE )
                :SetRotation( 40*DEGREES )
                :SetTintAlpha( 0.3 )
                :SetBloom( 0.3 )
            self.duplicate_smoke2 = Widget.Image( engine.asset.Texture( "particles/smoke.tex") )
                :SetHiddenBoundingBox( true )
                :SetSize( 900, 900 )
                :SetTintColour( UICOLOURS.DARK_BODYTEXT )
                :SetRotation( -60*DEGREES )
                :SetTintAlpha( 0.7 )
                :SetBloom( 0.1 )
            self.duplicate_label = Widget.Label( "body", 20, is_instance( card, Negotiation.Card ) and LOC "UI.CARD_WIDGET.TEMPORARY_NEGOTIATION" or LOC "UI.CARD_WIDGET.TEMPORARY_BATTLE" )
                :SetGlyphColour( UICOLOURS.TITLE )
                :LeftAlign()
                :Bloom( 0.1 )
            self.duplicate_widget:AddChildren
            {
                self.duplicate_mask_image
                    :SetMask( true ),
                self.duplicate_smoke1
                    :SetStencilTest( true )
                    :SetPos( 0, 150 ),
                self.duplicate_smoke2
                    :SetStencilTest( true )
                    :SetPos( 0, 130 ),
                self.duplicate_label
            }
            self.duplicate_widget:SendToBack()
                :LayoutBounds( "left", "below", self.corner_label_bg )
                :Offset( 0, 7 )
            self.duplicate_label:LayoutBounds( "left", "bottom", self.duplicate_mask_image )
                :Offset( padding, 10 )

            local start_value = 2 * math.random()
            self.duplicate_widget:RunUpdater(
                Updater.Loop{
                    Updater.Parallel{
                        Updater.Series{
                            Updater.Ease( function(v) self.duplicate_smoke1:SetPos(v,-v*0.5) end, 150, -150, 3, easing.inOutQuad):SetStartValuePercent(start_value),
                            Updater.Ease( function(v) self.duplicate_smoke1:SetPos(v,-v*0.5) end, -150, 150, 3, easing.inOutQuad),
                        },
                        Updater.Series{
                            Updater.Ease( function(v) self.duplicate_smoke1:SetScale(v,v) end, 1, 1.3, 5, easing.inOutQuad):SetStartValuePercent(start_value),
                            Updater.Ease( function(v) self.duplicate_smoke1:SetScale(v,v) end, 1.3, 1, 5, easing.inOutQuad),
                        },
                        Updater.Series{
                            Updater.Ease( function(v) self.duplicate_smoke2:SetPos(v,v*0.5) end, 130, -140, 4, easing.inOutQuad):SetStartValuePercent(start_value),
                            Updater.Ease( function(v) self.duplicate_smoke2:SetPos(v,v*0.5) end, -140, 130, 4, easing.inOutQuad),
                        },
                        Updater.Series{
                            Updater.Ease( function(v) self.duplicate_smoke2:SetScale(v,v) end, 1, 1.2, 6, easing.inOutQuad):SetStartValuePercent(start_value),
                            Updater.Ease( function(v) self.duplicate_smoke2:SetScale(v,v) end, 1.2, 1, 6, easing.inOutQuad),
                        }
                    }
                }
            )
            self.duplicate_smoke1:RotateIndefinitely( 0.005 )
            self.duplicate_smoke2:RotateIndefinitely( -0.003 )
        end
    else
        if self.duplicate_widget then
            self.duplicate_widget:Remove()
            self.duplicate_widget = nil
        end
    end
end

function FullCardPanel:HideCardEntries()
    self.card_entries:Hide()
    return self
end

-------------------------------------------------------------------------------------
-- A single keyword panel, used by various tooltips to show descriptive keyword text.

local KeywordPanel = class("Widget.KeywordPanel", Widget)

local REGION_WIDTH = 312

function KeywordPanel:init()
    KeywordPanel._base.init(self)
    self.bg = self:AddChild(Widget.Panel(global_images.tooltipbg))
    self.bg:SetNineSliceBorderScale( 0.5 )
    self.text = self:AddChild(Widget.Label("tooltip", 24, ""))
        :SetAutoSize( REGION_WIDTH )
        :OverrideLineHeight( 20 )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.05)
end

function KeywordPanel:DoLayout( txt )
    assert( type(txt) == "string" or error(tostr(txt,2)))
    self.text:SetText(txt)
    self.bg:SizeToWidgets( 6, self.text )
end

-------------------------------------------------------------------------------------
-- A card panel, showing a Card widget in the style of a keyword.

local CardPanel = class("Widget.CardPanel", Widget)
CardPanel.MARGIN = { 30, 0 }

local REGION_WIDTH = 312

function CardPanel:init( card_class )
    CardPanel._base.init(self)

    self.card_class = card_class
    self.bg = self:AddChild(Widget.Panel( global_images.tooltipbg ))
    self.bg:SetNineSliceBorderScale( 0.5 )
    self.card = self:AddChild( card_class():SetScale( 0.6 ) )
end

function CardPanel:DoLayout( card, engine )
    self.card:RefreshCard( card, engine )

    self.bg:SizeToWidgets( self.MARGIN, self.card )
end
-------------------------------------------------------------------------------------
-- A graft panel, showing a Graft widget in the style of a keyword.

local GraftPanel = class("Widget.GraftPanel", Widget)
GraftPanel.MARGIN = { 30, 0 }

local REGION_WIDTH = 312

function GraftPanel:init( )
    GraftPanel._base.init(self)

    self.bg = self:AddChild(Widget.Panel( global_images.tooltipbg ))
    self.bg:SetNineSliceBorderScale( 0.5 )
    self.graft = self:AddChild( Widget.TooltipGraft() )
    self.graft:Show()
end

function GraftPanel:DoLayout( graft )
    self.graft:DoLayout( graft, nil )

    self.bg:SizeToWidgets( self.MARGIN, self.graft )
end

-----------------------------------------------------------------------------
-- Tooltip that shows keyword panels.

local TooltipKeywords = class("Widget.TooltipKeywords", Widget)

function TooltipKeywords:init()
    TooltipKeywords._base.init(self)
    self.keyword_panels = {}
    self.keyword_count = 0
end

function TooltipKeywords:DoLayout( features )
    self.keyword_count = 0
    if self.coro then
        for i, panel in ipairs( self.keyword_panels ) do
            panel:StopAll()
        end
        self.coro:Stop()
        self.coro = nil
    end

    self.existing_keywords = {}
    for name, v in pairs( features or table.empty ) do
        if v.name then
            self:AddFeaturePanel( v.name, v.desc )
        elseif v.card then
            self:AddCardPanel( v.card )
        end
    end

    for i = self.keyword_count + 1, #self.keyword_panels do
        self.keyword_panels[i]:Hide()
    end

    self:TransitionPanels()
end

local function TransitionPanelsCoro( self )
    for i = 1, self.keyword_count do
        -- Must be 'shown' even with alpha 0, for bounding box calculations.
        self.keyword_panels[i]:SetTintAlpha( 0 ):Show()
    end
 
    self:Delay( 0.2 )
    
    for i = 1, self.keyword_count do
        self.keyword_panels[i]:AlphaTo(1.0, 0.3,easing.outCubic)
        self:Delay( 0.15 )
    end
end

function TooltipKeywords:TransitionPanels()
    if self.coro then
        self.coro:Stop()
        self.coro = nil
    end

    if self.keyword_count > 0 then
        self.coro = self:StartCoroutine( TransitionPanelsCoro, self )
    end
end

function TooltipKeywords:AddFeaturePanel( name, desc )
    if self.existing_keywords[name] == nil then
        self.existing_keywords[name] = true -- Prevent keywords from being listed more than once

        self.keyword_count = self.keyword_count + 1
        
        local panel = self.keyword_panels[ self.keyword_count ]
        if panel == nil then
            panel = Widget.KeywordPanel()
            self:AddChild( panel )
            self.keyword_panels[ self.keyword_count ] = panel

        elseif not is_instance( panel, Widget.KeywordPanel ) then
            -- FML
            panel = Widget.KeywordPanel()
            self:AddChild( panel )
            self.keyword_panels[ self.keyword_count ]:Remove()
            self.keyword_panels[ self.keyword_count ] = panel
        end

        local tt
        if name then
            tt = loc.format( LOC"UI.CARD_WIDGET.KEYWORD_ENTRY", name, desc )
        else
            tt = desc
        end

        panel:DoLayout( tt )

        if self.keyword_count == 1 then
            panel:LayoutBounds( "after", "top" )
        else
            panel:LayoutBounds( "left", "below", self.keyword_panels[ self.keyword_count - 1 ]):Offset( 0, -4 )
        end

        return panel
    end
end

function TooltipKeywords:AddCardPanel( carddef, engine )
    self.keyword_count = self.keyword_count + 1
    
    -- FEH.
    local card_class, card
    if carddef.is_negotiation then
        card_class = Widget.NegotiationCard
        if is_instance( carddef, Negotiation.Card ) then
            card = carddef
        else
            card = Negotiation.Card( carddef.id )
        end
    else
        card_class = Widget.BattleCard
        if is_instance( carddef, Battle.Card ) then
            card = carddef
        else
            card = Battle.Card( carddef.id )
        end
    end

    local panel = self.keyword_panels[ self.keyword_count ]
    if panel == nil then
        panel = Widget.CardPanel( card_class )
        self:AddChild( panel )
        self.keyword_panels[ self.keyword_count ] = panel

    elseif not is_instance( panel, Widget.CardPanel ) or panel.card_class ~= card_class then
        -- FML
        panel = Widget.CardPanel( card_class )
        self:AddChild( panel )
        self.keyword_panels[ self.keyword_count ]:Remove()
        self.keyword_panels[ self.keyword_count ] = panel
    end

    panel:DoLayout( card, engine )
    
    if self.keyword_count == 1 then
        panel:LayoutBounds( "after", "top" )
    elseif self.keyword_count > 1 then
        panel:LayoutBounds( "after", "top", self.keyword_panels[ self.keyword_count - 1 ]):Offset( 4, 0 )
    end
    return panel
end

function TooltipKeywords:AddGraftPanel( graft )
    self.keyword_count = self.keyword_count + 1
    
    local panel = self.keyword_panels[ self.keyword_count ]
    
    if panel == nil then
        panel = Widget.GraftPanel( graft )
        self:AddChild( panel )
        self.keyword_panels[ self.keyword_count ] = panel
    elseif not is_instance( panel, Widget.GraftPanel ) then
        -- FML
        panel = Widget.GraftPanel( graft )
        self:AddChild( panel )
        self.keyword_panels[ self.keyword_count ]:Remove()
        self.keyword_panels[ self.keyword_count ] = panel
    end

    panel:DoLayout( graft )
    
    if self.keyword_count == 1 then
        panel:LayoutBounds( "after", "top" )
    elseif self.keyword_count > 1 then
        panel:LayoutBounds( "left", "below", self.keyword_panels[ self.keyword_count - 1 ]):Offset( 4, 0 )
    end
    return panel
end

-----------------------------------------------------------------------------
-- Standard text tooltip, with supporting keyword panels.

local BattleTooltip = class("Widget.BattleTooltip", Widget.Tooltip )

function BattleTooltip:init()
    BattleTooltip._base.init( self )
    self.keywords = self:AddChild( Widget.TooltipKeywords())
end

function BattleTooltip:PostFeature( keyword, name, desc )
    self.keywords:AddFeaturePanel( name, desc )
end

function BattleTooltip:DoLayout( tt )
    self.keywords:DoLayout( nil ) -- Hide previously instantiated panels.

    local battle_defs = require "battle/battle_defs"
    tt = CardEngine.ExtractKeywords( tt, self, nil, battle_defs )

    Widget.Tooltip.DoLayout( self, tt )

    self.keywords:TransitionPanels()
    self.keywords:LayoutBounds( "left", "below", self.bg )
end

-----------------------------------------------------------------------------
-- Standard text tooltip, with supporting keyword panels.

local NegotiationTooltip = class("Widget.NegotiationTooltip", Widget.Tooltip )

function NegotiationTooltip:init()
    NegotiationTooltip._base.init( self )
    self.keywords = self:AddChild( Widget.TooltipKeywords())
end

function NegotiationTooltip:PostFeature( keyword, name, desc )
    self.keywords:AddFeaturePanel( name, desc )
end

function NegotiationTooltip:PostCard( keyword, card )
    self.keywords:AddCardPanel( card )
end

function NegotiationTooltip:DoLayout( tt )
    self.keywords:DoLayout( nil ) -- Hide previously instantiated panels.

    local defs = require "negotiation/negotiation_defs"
    tt = CardEngine.ExtractKeywords( tt, self, nil, defs )

    Widget.Tooltip.DoLayout( self, tt )

    self.keywords:TransitionPanels()
    self.keywords:LayoutBounds( "after", "top", self.bg )
end

-----------------------------------------------------------------------------
-- Shows the same FullCardPanel tooltip as above, but with a full card widget beside it

local TooltipCard = class("Widget.TooltipCard", Widget.FullCardPanel)

function TooltipCard:init()
    TooltipCard._base.init(self)

    -- Set the scale of the tooltip's card based on the current device's screen
    local scale = 0.5
    local sm = TheGame:FE():GetScreenMode()
    if sm == SCREEN_MODE.MONITOR then
        scale = 0.5
    elseif sm == SCREEN_MODE.SMALL then
        scale = 0.6
    elseif sm == SCREEN_MODE.TV then
        scale = 0.6
    end
    
    self.battle_card = self:AddChild( Widget.BattleCard():SetScale( scale ) )
    self.negotiation_card = self:AddChild( Widget.NegotiationCard():SetScale( scale ) )
end

function TooltipCard:DoLayout( data )
    if is_instance( data, CardEngine.Card ) then
        self.card = data
        self.last_def = nil

    elseif is_instance( data, BattleCardDef ) then
        if self.last_def ~= data then
            self.card = Battle.Card( data.id )
            self.last_def = data
        end

    elseif is_instance( data, NegotiationCardDef ) then
        if self.last_def ~= data then
            self.card = Negotiation.Card( data.id )
            self.last_def = data
        end

    else
        self.card = nil
    end

    local card_widget
    if is_instance( self.card, Battle.Card ) then
        card_widget = self.battle_card
        self.negotiation_card:Hide()
    elseif is_instance( self.card, Negotiation.Card ) then
        card_widget = self.negotiation_card
        self.battle_card:Hide()
    end

    if card_widget then

        card_widget:Show():RefreshCard( self.card, self.card.engine )
        self.features = data.FEATURES or card_widget.card_features
        
        self._base.DoLayout( self, { CARD = self.card } )
        card_widget:LayoutBounds( "before", "top", self.content )
    else
        print( "TooltipCard missing data!", tostr(data), debug.traceback() )
        return false
    end
end


-----------------------------------------------------------------------------
-- A card widget with no keyword panels.

local TooltipAbridgedCard = class("Widget.TooltipAbridgedCard", Widget )

function TooltipAbridgedCard:init()
    TooltipAbridgedCard._base.init(self)

    -- Set the scale of the tooltip's card based on the current device's screen
    local scale = 0.5
    local sm = TheGame:FE():GetScreenMode()
    if sm == SCREEN_MODE.MONITOR then
        scale = 0.5
    elseif sm == SCREEN_MODE.SMALL then
        scale = 0.65
    elseif sm == SCREEN_MODE.TV then
        scale = 0.65
    end
    
    self.battle_card = self:AddChild( Widget.BattleCard():SetScale( scale ) )
    self.negotiation_card = self:AddChild( Widget.NegotiationCard():SetScale( scale ) )
end

function TooltipAbridgedCard:DoLayout( data )
    if is_instance( data, CardEngine.Card ) then
        self.card = data
        self.last_def = nil

    elseif is_instance( data, BattleCardDef ) then
        if self.last_def ~= data then
            self.card = Battle.Card( data.id )
            self.last_def = data
        end

    elseif is_instance( data, NegotiationCardDef ) then
        if self.last_def ~= data then
            self.card = Negotiation.Card( data.id )
            self.last_def = data
        end

    else
        self.card = nil
    end

    local card_widget
    if is_instance( self.card, Battle.Card ) then
        card_widget = self.battle_card
        self.negotiation_card:Hide()
    elseif is_instance( self.card, Negotiation.Card ) then
        card_widget = self.negotiation_card
        self.battle_card:Hide()
    end

    if card_widget then
        card_widget:Show():RefreshCard( self.card, self.card.engine )
    else
        print( "TooltipAbridgedCard missing data!", tostr(data), debug.traceback() )
        return false
    end
end


-----------------------------------------------------------------------------
-- Shows the same FullCardPanel tooltip as above, but with a full card widget beside it

local TooltipBattleCard = class("Widget.TooltipBattleCard", Widget.FullCardPanel)

function TooltipBattleCard:init()
    TooltipBattleCard._base.init(self)

    -- Set the scale of the tooltip's card based on the current device's screen
    local scale = 0.5
    local sm = TheGame:FE():GetScreenMode()
    if sm == SCREEN_MODE.MONITOR then
        scale = 0.5
    elseif sm == SCREEN_MODE.SMALL then
        scale = 0.6
    elseif sm == SCREEN_MODE.TV then
        scale = 0.6
    end

    self.card_widget = self:AddChild( Widget.BattleCard():SetScale( scale ) )
end

function TooltipBattleCard:DoLayout( data )

    if data.CARD then
        self.card = data.CARD
        self.last_def = nil

    elseif data.card_def then
        if self.last_def ~= data.card_def then
            self.card = Battle.Card( data.card_def.id )
            self.last_def = data.card_def
        end
    else
        self.card = nil
    end
    
    if self.card then
        self.card_widget:RefreshCard( self.card, self.card.minigame )
        self.features = data.FEATURES or self.card_widget.card_features

        self._base.DoLayout( self, data )
        self.card_widget:LayoutBounds( "before", "top", self.content )
    else
        print( "TooltipBattleCard missing data!", tostr(data), debug.traceback() )
        return false
    end
end

-----------------------------------------------------------------------------
-- Shows the same FullCardPanel tooltip as above, but with a full card widget beside it

local TooltipNegotiationCard = class("Widget.TooltipNegotiationCard", Widget.FullCardPanel)

function TooltipNegotiationCard:init()
    TooltipNegotiationCard._base.init(self)

    -- Set the scale of the tooltip's card based on the current device's screen
    local scale = 0.5
    local sm = TheGame:FE():GetScreenMode()
    if sm == SCREEN_MODE.MONITOR then
        scale = 0.5
    elseif sm == SCREEN_MODE.SMALL then
        scale = 0.6
    elseif sm == SCREEN_MODE.TV then
        scale = 0.6
    end
    
    self.card_widget = self:AddChild( Widget.NegotiationCard():SetScale( scale ) )
end

function TooltipNegotiationCard:DoLayout( data )

    if data.CARD then
        self.card = data.CARD
        self.last_def = nil

    elseif data.card_def then
        if self.last_def ~= data.card_def then
            self.card = Negotiation.Card( data.card_def.id )
            self.last_def = data.card_def
        end
    else
        self.card = nil
    end

    if self.card then
        self.card_widget:RefreshCard( self.card, self.card.minigame )
        self.features = data.FEATURES or self.card_widget.card_features
        
        self._base.DoLayout( self, data )
        self.card_widget:LayoutBounds( "before", "top", self.content )
    else
        print( "TooltipNegotiationCard missing data!", tostr(data), debug.traceback() )
        return false
    end
end
