require "ui/widgets/tooltip"
require "ui/widgets/tooltip_card"
local battle_defs = require "battle/battle_defs"

----------------------------------------------------------------
-- Argument tooltips.

local TooltipCondition = class("Widget.TooltipCondition", Widget.Tooltip )

local padding = 20
local bg_border = 30
local bg_border_scale = 0.5
local content_w = DEFAULT_TT_WIDTH - padding*2
local MARGIN = { -24, -10 }

function TooltipCondition:init()
    TooltipCondition._base.init( self )

    self.content = self:AddChild( Widget() )

    self.header_bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.05 )
    self.header_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.header_bg:SetNineSliceBorderScale( bg_border_scale )
    self.header_bg:SetHiddenBoundingBox( true )
    self.header_bg:SetTintColour( UICOLOURS.FIGHT )
    self.header_bg:SetTintAlpha( 0.2 )

    -- Header content
    self.header = self.content:AddChild( Widget() )

    -- A label for the type label
    self.corner_label_bg = self.header:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.1 )
    self.corner_label_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.corner_label_bg:SetNineSliceBorderScale( bg_border_scale )
    self.corner_label_bg:SetTintColour( UICOLOURS.FIGHT )
    self.corner_label_bg:SetTintAlpha( 0.2 )

    -- The type label
    self.header_left = self.header:AddChild( Widget.Label( "body", 20 ) )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.FIGHT )
        :Bloom( 0.05 )

    -- Title
    self.condition_name = self.header:AddChild( Widget.Label("title", 25 )):LeftAlign():Bloom(0.1)
    self.condition_name:SetAutoSize( content_w )

    -- Stacks
    self.stacks_row = self.content:AddChild( Widget() )
    self.stacks_label = self.stacks_row:AddChild( Widget.Label("body", 20 )):SetGlyphColour(UICOLOURS.OBJECTIVE):SetRegionSize(content_w,20):LeftAlign():Bloom(0.05)
    self.stacks_value = self.stacks_row:AddChild( Widget.Label("body", 20 )):SetGlyphColour(UICOLOURS.OBJECTIVE):SetRegionSize(content_w,20):RightAlign():Bloom(0.09)

    -- Description
    self.desc = self.content:AddChild( Widget.Label( "tooltip", 24, "" ) )
        :SetAutoSize( content_w )
        :OverrideLineHeight( 20 )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.05)

    -- Keywords
    self.keywords = self:AddChild( Widget.TooltipKeywords())
end

function TooltipCondition:PostFeature( keyword, name, desc )
    self.keywords:AddFeaturePanel( name, desc )
end

function TooltipCondition:PostCard( keyword, name, desc )
    self.keywords:AddFeaturePanel( name, desc )
end

function TooltipCondition:PostCard( feature_id, card )
    self.keywords:AddCardPanel( card )
end

function TooltipCondition:DoLayout( condition )
    self.keywords:DoLayout( nil ) -- Hide previously instantiated panels.

    self.header_left:SetText( LOC"UI.FIGHT_SCREEN.CONDITION" )

    local label_w, label_h = self.header_left:GetSize()
    local nineslice_w, nineslice_h = label_w + bg_border*2*bg_border_scale, 68
    self.corner_label_bg:SetSize( nineslice_w+padding*2.5, nineslice_h )
    self.corner_label_bg:SetPos( 0, 0 )
    self.header_left:LayoutBounds( "left", "bottom", self.corner_label_bg ):Offset( padding, 8 )

    -- Show the header background
    local h_w, h_h = self.header:GetSize()
    local nineslice_w, nineslice_h = DEFAULT_TT_WIDTH + bg_border*2*bg_border_scale, 68
    self.header_bg:SetSize( nineslice_w, nineslice_h )
    self.header_bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )

    -- Set title
    self.condition_name:SetText( condition:GetName() )
    self.condition_name:LayoutBounds( "left", "below", self.corner_label_bg ):Offset( padding, -6 )

    -- Set the strength
    if not condition.hide_stacks then
        local stacks = condition:GetStacks()
        self.stacks_label:SetText( LOC"UI.NEGOTIATION_PANEL.STRENGTH" )
        self.stacks_value:SetText( loc.format( LOC"UI.NEGOTIATION_PANEL.ARGUMENT_STRENGTH", tostring(stacks) ) )
        self.stacks_row:Show()
            :LayoutBounds( "left", "below", self.condition_name )
            :Offset( 0, -2 )
    else
        self.stacks_row:Hide()
            :LayoutBounds( "left", "bottom", self.condition_name )
    end

    -- Set the description
    local desc_text = condition:GetDesc( condition.battle, self )
    self.desc:SetText( desc_text )
    self.desc:LayoutBounds( "left", "below", self.stacks_row ):Offset( 0, -6 )

    -- Size up the background
    local c_w, c_h = self.content:GetSize()
    self.bg:SetSize( DEFAULT_TT_WIDTH, c_h + padding )
    self.bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )

    self.keywords:TransitionPanels()
    self.keywords:LayoutBounds( "after", "top", self.bg )
end
