require "ui/widget"
require "ui/widgets/solidbox"
require "ui/widgets/label"
require "ui/widgets/solidbox"

local TooltipConvoOption = class("Widget.TooltipConvoOption", Widget )
local REGION_WIDTH = 312
local REGION_HEIGHT = 1283

TooltipConvoOption.LAYOUT_SCALE =
{
    [SCREEN_MODE.MONITOR] = 1,
    [SCREEN_MODE.TV] = 1.5,
    [SCREEN_MODE.SMALL] = 1.5,
}

function TooltipConvoOption:init()
	TooltipConvoOption._base.init(self)

	self.bg = Widget.Panel(global_images.tooltipbg)
    self.bg:SetNineSliceBorderScale( 0.5 )
    self:AddChild( self.bg )

	self.text = Widget.Label("tooltip", 24, ""):SetWordWrap( true ):LeftAlign():SetGlyphColour(UICOLOURS.BODYTEXT):SetAutoSize( 800 )
	self:AddChild( self.text )

    self.quest_tt = Widget.TooltipQuest()
    self:AddChild( self.quest_tt )

    self.card_widgets = {}
end

function TooltipConvoOption:LayoutChildTooltip( widget )
    if self.last_child == nil then
        if self.bg:IsShown() then
            widget:Show():LayoutBounds( "after", "bottom", self.bg ):Offset( 0, -4 )
        else
            widget:Show():SetPos( 0, 0 )
        end
    else
        widget:Show():LayoutBounds( "after", "bottom", self.last_child )
    end

    self.last_child = widget
end

function TooltipConvoOption:DoLayout( opt )
    local tt = ObtainWorkTable()

    if type(opt.tt) == "function" then
        table.insert( tt, opt.tt() )

    elseif type(opt.tt) == "string" then
        table.insert( tt, opt.tt )
    end

    if opt.post_text and #opt.post_text > 0 then
        table.arrayadd( tt, opt.post_text )
	end

    if opt.relationship_changes then
        for i, t in ipairs( opt.relationship_changes ) do
            local agent, new_relationship = t.agent, t.relationship
            local relationship = agent:GetRelationship()
            if relationship ~= new_relationship then
                -- Only show relative gain if they already HATE/LOVE you, so it's less confusing.
                if relationship == RELATIONSHIP.HATED or relationship == RELATIONSHIP.LOVED then
                    if new_relationship > relationship + 1 then
                        table.insert( tt, opt:GetLocString( "IMPROVE_OPINION_LARGE", agent ))
                    elseif new_relationship > relationship then
                        table.insert( tt, opt:GetLocString( "IMPROVE_OPINION", agent ))
                    elseif new_relationship < relationship then
                        table.insert( tt, opt:GetLocString( "DIMINISH_OPINION", agent ))
                    end
                end

                table.insert( tt, GetRelationshipPreviewString( new_relationship, agent ))
            end
        end
    end

    if opt.disabled_reasons and #opt.disabled_reasons > 0 then
        table.insert( tt, "<#REQUIRES>" .. table.concat( opt.disabled_reasons, "\n" ) .. "</>" )
    end

    self.last_child = nil

    local layout_widget
    if #tt > 0 then
    	self.text:SetText( table.concat( tt, "\n" ))
    	local x1, y1, x2, y2 = self.text.model:GetRenderBounds()

    	self.bg:SetInnerSize( (x2 - x1) + 10, (y2 - y1) + 10 )
        self.bg:SetPos( (x2 + x1) / 2, (y2 + y1) / 2 )
        self.text:Show()
        self.bg:Show()
        layout_widget = self.bg
    else
        self.text:Hide()
        self.bg:Hide()
    end
    ReleaseWorkTable( tt )

    for i = 1, math.max( #(opt.post_cards or table.empty), #self.card_widgets ) do
        local card_id = opt.post_cards and opt.post_cards[i]
        if card_id then
            local def = Content.GetBattleCard( card_id ) or Content.GetNegotiationCard( card_id )
            if self.card_widgets[i] == nil then
                if opt.no_keywords then
                    self.card_widgets[i] = self:AddChild( Widget.TooltipAbridgedCard())
                else
                    self.card_widgets[i] = self:AddChild( Widget.TooltipCard())
                end
            end
            self.card_widgets[i]:DoLayout( def )
            self:LayoutChildTooltip( self.card_widgets[i] )

        elseif self.card_widgets[i] then
            self.card_widgets[i]:Hide()
        end
    end

    if opt.quest then
        self.quest_tt:DoLayout( { quest = opt.quest, show_as_inactive = opt.show_quest_as_inactive } )
        self:LayoutChildTooltip( self.quest_tt )
    else
        self.quest_tt:Hide()
    end

end

----------------------------------------------------------

