require "ui/widgets/tooltip"
require "ui/screens/relationshipsscreen"

----------------------------------------------------------------
-- Graft tooltips.

local TooltipGraft = class("Widget.TooltipGraft", Widget.Tooltip )

local padding = 20
local bg_border = 30
local bg_border_scale = 0.5
local IMG_SIZE = 128
local HEADER_H = 68
local content_w = DEFAULT_TT_WIDTH-IMG_SIZE-padding*3
local keywords_w = DEFAULT_TT_WIDTH-padding*2
local FLAVOUR_W = 300

function TooltipGraft:init()
    TooltipGraft._base.init(self)

    self.content = self:AddChild(Widget())

    -- A header bg
    self.header_bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.05 )
    self.header_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.header_bg:SetNineSliceBorderScale( bg_border_scale )
    self.header_bg:SetHiddenBoundingBox( true )

    -- Header content
    self.header = self.content:AddChild( Widget() )

    -- A label for the type label
    self.corner_label_bg = self.header:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.1 )
    self.corner_label_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.corner_label_bg:SetNineSliceBorderScale( bg_border_scale )

    -- The type label
    self.corner_label = self.header:AddChild( Widget.Label("body", 20 ) ):LeftAlign():Bloom( 0.05 )

    -- Rarity label
    self.rarity_icon = self:AddChild( Widget.Image() )
        :SetSize( 20, 20 )
        :Bloom( 0.1 )
    self.rarity_label = self.header:AddChild( Widget.Label("body", 20 ) )
        :LeftAlign()
        :SetAutoSize( content_w )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :Bloom(0.1)

    -- Image
    self.graft_icon = self.content:AddChild( Widget.GraftSlot( nil, IMG_SIZE ) )
        :SetFrameImageRatio( 1.35 )

    -- Text column container
    self.text_content = self.content:AddChild( Widget() )
    self.name = self.text_content:AddChild(Widget.Label("title", 26))
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour(UICOLOURS.BODYTEXT)
        :Bloom(0.1)
    self.desc = self.text_content:AddChild(Widget.Label("body", 22))
        :SetAutoSize( content_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    
    -- For all the description content
    self.stats = self.content:AddChild( Widget() ):Hide()

    -- Upgrade
    self.upgrade_row = self.stats:AddChild( Widget() )
    self.upgrade_label = self.upgrade_row:AddChild( Widget.Label("body", 20 )):LeftAlign():Bloom(0.05)
    self.upgrade_value = self.upgrade_row:AddChild( Widget.Label("body", 20 )):RightAlign():Bloom(0.09)

    -- Keywords
    self.keyword_entries = self.content:AddChild( Widget() )
    self.flavour_text = self.content:AddChild( Widget.Label("body", 20 ):SetAutoSize( FLAVOUR_W ):SetWordWrap( true ):LeftAlign() )

    self.card_keywords = {}
end

function TooltipGraft:PostFeature( feature_id, name, desc )
    self.card_keywords[ feature_id ] = { name = name, desc = desc }
end

function TooltipGraft:PostCard( feature_id, card )
    self.card_keywords[ feature_id ] = { card = card }
end

function TooltipGraft:DoLayout( graft )

    local graft_def = graft:GetDef()
    local desc_suffix = ""

    -- Set appropriate colors!
    if graft:GetType() == GRAFT_TYPE.NEGOTIATION then
        self.corner_label:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_NEGOTIATION" )
        self.corner_label:SetGlyphColour( UICOLOURS.NEGOTIATION )
        self.corner_label_bg:SetTintColour( UICOLOURS.NEGOTIATION )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.NEGOTIATION )
        self.header_bg:SetTintAlpha( 0.2 )
        self.rarity_icon:Show()
        self.rarity_label:Show()
    elseif graft:GetType() == GRAFT_TYPE.COMBAT then
        self.corner_label:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_COMBAT" )
        self.corner_label:SetGlyphColour( UICOLOURS.FIGHT )
        self.corner_label_bg:SetTintColour( UICOLOURS.FIGHT )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.FIGHT )
        self.header_bg:SetTintAlpha( 0.2 )
        self.rarity_icon:Show()
        self.rarity_label:Show()
    elseif graft:GetType() == GRAFT_TYPE.SOCIAL and graft_def.is_good then
        self.corner_label:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_SOCIAL_BOON" )
        self.corner_label:SetGlyphColour( UICOLOURS.BONUS )
        self.corner_label_bg:SetTintColour( UICOLOURS.BONUS )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.BONUS )
        self.header_bg:SetTintAlpha( 0.2 )
        self.rarity_icon:Hide()
        self.rarity_label:Hide()
    elseif graft:GetType() == GRAFT_TYPE.SOCIAL and not graft_def.is_good then
        self.corner_label:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_SOCIAL_BANE" )
        self.corner_label:SetGlyphColour( UICOLOURS.PENALTY )
        self.corner_label_bg:SetTintColour( UICOLOURS.PENALTY )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.PENALTY )
        self.header_bg:SetTintAlpha( 0.2 )
        self.rarity_icon:Hide()
        self.rarity_label:Hide()
    elseif graft:GetType() == GRAFT_TYPE.MUTATOR then
        self.corner_label:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_MUTATOR" )
        self.corner_label:SetGlyphColour( UICOLOURS.MUTATOR )
        self.corner_label_bg:SetTintColour( UICOLOURS.MUTATOR )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.MUTATOR )
        self.header_bg:SetTintAlpha( 0.2 )
        self.rarity_icon:Hide()
        self.rarity_label:Hide()
    elseif graft:GetType() == GRAFT_TYPE.COIN then
        self.corner_label:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_COIN" )
        self.corner_label:SetGlyphColour( UICOLOURS.MONEY )
        self.corner_label_bg:SetTintColour( UICOLOURS.MONEY )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.MONEY )
        self.header_bg:SetTintAlpha( 0.2 )
        self.rarity_icon:Hide()
        self.rarity_label:Hide()
    elseif graft:GetType() == GRAFT_TYPE.BLASTER then
        self.corner_label:SetText( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_BLASTER" )
        self.corner_label:SetGlyphColour( UICOLOURS.OBJECTIVE )
        self.corner_label_bg:SetTintColour( UICOLOURS.OBJECTIVE )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.OBJECTIVE )
        self.header_bg:SetTintAlpha( 0.2 )
        self.rarity_icon:Hide()
        self.rarity_label:Hide()
    elseif graft:GetType() == GRAFT_TYPE.PERK then
        self.corner_label:SetText( loc.format( LOC"UI.GRAFT_COLLECTION.SLOT_TYPE_PERK", graft:GetDef().tier ) )
        self.corner_label:SetGlyphColour( UICOLOURS.PERK )
        self.corner_label_bg:SetTintColour( UICOLOURS.PERK )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.PERK )
        self.header_bg:SetTintAlpha( 0.2 )
        self.rarity_icon:Hide()
        self.rarity_label:Hide()
        local is_played = TheGame:GetGameProfile():IsPerkPlayed( graft:GetDef().id )
        desc_suffix = is_played and LOC"UI.PERK_SCREEN.PERK_PLAYED_TT_SUFFIX" or ""
    end

    -- Get the corner label size, and scale up its background to match
    local label_w, label_h = self.corner_label:GetSize()
    local nineslice_w, nineslice_h = label_w + bg_border*2*bg_border_scale, HEADER_H
    self.corner_label_bg:SetSize( nineslice_w+padding*2.5, nineslice_h )
    self.corner_label_bg:SetPos( 0, 0 )
    self.corner_label:LayoutBounds( "left", "bottom", self.corner_label_bg ):Offset( padding, 8 )

    -- Show the header background
    local h_w, h_h = self.header:GetSize()
    local nineslice_w, nineslice_h = DEFAULT_TT_WIDTH + bg_border*2*bg_border_scale, HEADER_H
    self.header_bg:SetSize( nineslice_w, nineslice_h )
    self.header_bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )

    -- Set rarity and type
    self.rarity_icon:SetTexture( global_images.card_rarity_icons[graft:GetRarity()] )
        :LayoutBounds( "right", "bottom", self.header_bg ):Offset( -padding+4, 8 )
    local rarity_color = MakeColourString( HexColour( CARD_RARITY_COLOURS[ graft:GetRarity() or CARD_RARITY.UNIQUE ] ) )
    local rarity_name = GetCardRarityString( graft:GetRarity() )
    local subtitle_text = loc.format( LOC"UI.GRAFT_SCREEN.DETAILS_SUBTITLE", rarity_color, rarity_name )
    self.rarity_label:SetText( subtitle_text )
        :LayoutBounds( "before", "center", self.rarity_icon ):Offset( -1, 0 )

    -- Update the image
    self.graft_icon:Refresh( graft )
        :LayoutBounds( "left", "below", self.header_bg )
        :Offset( padding, -padding*0.7 )

    -- Update the texts
    self.name:SetText( graft:GetName() )
    self.desc:SetText( graft:GetDesc( self ) .. desc_suffix )
    self.name:LayoutBounds("after", "top", self.graft_icon)
        :Offset( padding*0.8, 7 )
    self.desc:LayoutBounds( "left", "below", self.name )
        :Offset( 0, 2 )

    local bottom_widget = self.graft_icon
    -- Upgrade progress
    if graft:HasUpgrades() then
        local graft_xp, max_xp = graft:GetXP()
        self.upgrade_row:Show()
        self.stats:Show()
        bottom_widget = self.stats
        -- Use bright colors if this graft is ready to upgrade
        if graft:UpgradeReady() then
            self.upgrade_label:SetText( LOC"UI.CARD_WIDGET.UPGRADE_LABEL_READY" ):SetGlyphColour( UICOLOURS.QUEST )
            self.upgrade_value:SetText( loc.format( LOC"UI.CARD_WIDGET.UPGRADE_VALUE", graft_xp, max_xp ) ):SetGlyphColour( UICOLOURS.QUEST )
        else
            self.upgrade_label:SetText( LOC"UI.CARD_WIDGET.UPGRADE_LABEL" ):SetGlyphColour( UICOLOURS.OBJECTIVE )
            self.upgrade_value:SetText( loc.format( LOC"UI.CARD_WIDGET.UPGRADE_VALUE", graft_xp, max_xp ) ):SetGlyphColour( UICOLOURS.OBJECTIVE )
        end
    else
        self.upgrade_row:Hide()
    end

    -- Layout the stats
    self.stats:StackChildren( 6 )

    -- Go through all the keywords for this card
    self.keyword_entries:DestroyAllChildren()
    self.keyword_entries:Hide()
    self.flavour_text:Hide()
    
    
    if self.card_keywords then
        for k,v in pairs( self.card_keywords ) do
            self.keyword_entries:Show()
            if v.card then
                local row
                if is_instance( v.card, Battle.Card ) then
                    row = Widget.BattleCard( v.card )
                elseif is_instance( v.card, BattleCardDef ) then
                    row = Widget.BattleCard( Battle.Card( v.card.id ))
                elseif is_instance( v.card, Negotiation.Card ) then
                    row = Widget.NegotiationCard( v.card )
                elseif is_instance( v.card, NegotiationCardDef ) then
                    row = Widget.NegotiationCard( Negotiation.Card( v.card.id ))
                end
                if row then
                    row:SetScale( 0.5 )
                    self.keyword_entries:AddChild( row )
                end

            else
                local entry = loc.format( LOC"UI.CARD_WIDGET.KEYWORD_ENTRY", v.name, v.desc )
                local row = Widget.Label("tooltip",20,entry):SetAutoSize( keywords_w ):SetWordWrap( true ):LeftAlign():SetGlyphColour( UICOLOURS.TOOLTIP_TEXT ):Bloom(0.05)
                self.keyword_entries:AddChild( row )
            end
        end
        self.keyword_entries:StackChildren( 5 )
        self.keyword_entries:LayoutBounds( "left", "below", self.graft_icon )
            :Offset( 0, -padding*0.5 )

        bottom_widget = self.keyword_entries
    end

    
    if self.stats:IsShown() then
        self.stats:LayoutBounds( "left", "below", self.graft_icon ):Offset( 0, -padding*0.5 )
        self.keyword_entries:LayoutBounds( "left", "below", self.stats ):Offset( 0, -10 )
    end

    local flavour = graft:GetFlavourText()
    if flavour and flavour ~= "" then
        local txt = loc.format("<i><#FLAVOUR_TEXT>{1}</></>", flavour)
        self.flavour_text:SetText(txt)
        self.flavour_text:LayoutBounds("left", "below", bottom_widget):Offset(0, -10):Show()
    else
        self.flavour_text:Hide()
    end

    -- Size up the background
    local c_w, c_h = self.content:GetSize()
    self.bg:SetSize( DEFAULT_TT_WIDTH, c_h + padding*1 )
    self.bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )

    self.upgrade_value:LayoutBounds( "right", nil, self.bg ):Offset( -padding, 0 )

    table.clear( self.card_keywords )
end

----------------------------------------------------------------
-- Tooltip for the social grafts on the day-end-screen

local TooltipAgentSocialGrafts = class("Widget.TooltipAgentSocialGrafts", Widget.Tooltip )

local padding = 20
local bg_border = 30
local bg_border_scale = 0.5
local HEADER_H = 68
local IMG_SIZE = 128
local BOON_SIZE = 90
local content_w = DEFAULT_TT_WIDTH-padding*2

function TooltipAgentSocialGrafts:init()
    TooltipAgentSocialGrafts._base.init(self)

    self.content = self:AddChild(Widget())

    -- A header bg
    self.header_bg = self.content:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.05 )
    self.header_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.header_bg:SetNineSliceBorderScale( bg_border_scale )

    -- The type label
    self.agent_name = self.content:AddChild( Widget.Label("title", 26 ) )
        :LeftAlign()
        :Bloom( 0.1 )

    self.boon = self.content:AddChild( Widget.SocialGraftWidget( content_w, BOON_SIZE ) )
    self.bane = self.content:AddChild( Widget.SocialGraftWidget( content_w, BOON_SIZE ) )
end

function TooltipAgentSocialGrafts:DoLayout( data )

    self.agent = data.AGENT
    self.relationship = data.RELATIONSHIP

    -- Show the header background
    local nineslice_w, nineslice_h = DEFAULT_TT_WIDTH + bg_border*2*bg_border_scale, HEADER_H
    self.header_bg:SetSize( nineslice_w, nineslice_h )
    self.header_bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )
    self.header_bg:SetTintColour( RELATIONSHIP_COLOURS[self.relationship] )
    self.header_bg:SetTintAlpha( 0.2 )
    self.agent_name:SetText( string.upper( self.agent:GetName() ) )
        :SetGlyphColour( RELATIONSHIP_COLOURS[self.relationship] )
        :LayoutBounds( "left", "center", self.header_bg )
        :Offset( padding, 0 )

    -- Get social boon/bane
    local boon_id = self.agent:GetSocialGraft( RELATIONSHIP.LOVED )
    local boon_graft = boon_id and GraftInstance( boon_id )
    self.boon:Refresh( boon_graft, self.relationship == RELATIONSHIP.LOVED, self.agent )
        :LayoutBounds( "left", "below", self.header_bg )
        :Offset( padding, -padding/2 )
    local bane_id = self.agent:GetSocialGraft( RELATIONSHIP.HATED )
    local bane_graft = bane_id and GraftInstance( bane_id )
    self.bane:Refresh( bane_graft, self.relationship == RELATIONSHIP.HATED, self.agent )
        :LayoutBounds( "left", "below", self.boon )
        :Offset( 0, -padding/2 )
    
    -- Size up the background
    local c_w, c_h = self.content:GetSize()
    self.bg:SetSize( DEFAULT_TT_WIDTH, c_h + padding*1.5 )
    self.bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )
end

----------------------------------------------------------------
-- Widget that displays a social boon/bane

local SocialGraftWidget = class( "Widget.SocialGraftWidget", Widget.RelationshipsScreenBoon )

local PADDING = 15

function SocialGraftWidget:init( width, height )
    SocialGraftWidget._base.init( self, width, height )

    self.bg:Remove()
end

function SocialGraftWidget:Layout()
    local ICON_SIZE = self.height - PADDING*2

    -- Layout our contents
    self.icon_frame:SetSize( ICON_SIZE, ICON_SIZE )
    self.icon:SetSize( ICON_SIZE, ICON_SIZE )
        :LayoutBounds( "center", "center", self.icon_frame )
    self.title:SetAutoSize( self.width - ICON_SIZE - PADDING )
    self.subtitle:SetAutoSize( self.width - ICON_SIZE - PADDING )

    -- Layout our title
    self.label:LayoutBounds( "left", "above", self.icon_frame )
        :Offset( 0, 4 )

    -- Layout the Active label and icon
    self.active_icon:LayoutBounds( "before", "center", self.label )
        :Offset( self.width, -1 )
    self.active_label:LayoutBounds( "before", "center", self.active_icon )
        :Offset( -6, 1 )
end