require "ui/widgets/tooltip"
require "ui/widgets/tooltip_card"

----------------------------------------------------------------
-- Argument tooltips.

local TooltipIntent = class("Widget.TooltipIntent", Widget.Tooltip )

local padding = 20
local bg_border = 30
local bg_border_scale = 0.5
local content_w = DEFAULT_TT_WIDTH - padding*2

function TooltipIntent:init()
    TooltipIntent._base.init( self )

    self.content = self:AddChild( Widget() )

    self.header_bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.05 )
    self.header_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.header_bg:SetNineSliceBorderScale( bg_border_scale )
    self.header_bg:SetHiddenBoundingBox( true )

    -- Header content
    self.header = self.content:AddChild( Widget() )

    -- A label for the type label
    self.corner_label_bg = self.header:AddChild( Widget.Panel( engine.asset.Texture("UI/tooltip_header.tex") ) ):Bloom( 0.1 )
    self.corner_label_bg:SetNineSliceCoords( bg_border, bg_border, bg_border, bg_border )
    self.corner_label_bg:SetNineSliceBorderScale( bg_border_scale )

    -- The type label
    self.header_left = self.header:AddChild( Widget.Label( "body", 20 ) )
        :LeftAlign()
        :Bloom( 0.05 )

    -- Countdown label
    self.header_right = self.header:AddChild( Widget.Label("title", 20 )):LeftAlign():Bloom(0.1)
    self.header_right:SetAutoSize( content_w )

    -- Description
    self.desc = self.content:AddChild( Widget.Label( "tooltip", 24, "" ) )
        :SetAutoSize( content_w )
        :OverrideLineHeight( 20 )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.05)

    -- Keywords
    self.keyword_entries = self.content:AddChild( Widget() )
end

function TooltipIntent:PostFeature( keyword, name, desc )
    local entry = loc.format( LOC"UI.CARD_WIDGET.KEYWORD_ENTRY", name, desc )
    local row = Widget.Label("tooltip",20,entry):SetAutoSize( content_w ):SetWordWrap( true ):LeftAlign():SetGlyphColour( UICOLOURS.TOOLTIP_TEXT ):Bloom(0.05)
    self.keyword_entries:AddChild( row )
    row:LayoutBounds( "left", "below" )
end

function TooltipIntent:PostCard( feature_id, card )
	--self.keywords:AddCardPanel( card )
end

function TooltipIntent:DoLayout( card )
    local negotiation_defs = require "negotiation/negotiation_defs"

    self.keyword_entries:DestroyAllChildren()

    -- Set appropriate colors!
    if card._class == Negotiation.Card then
        self.header_left:SetGlyphColour( UICOLOURS.NEGOTIATION )
        self.corner_label_bg:SetTintColour( UICOLOURS.NEGOTIATION )
        self.corner_label_bg:SetTintAlpha( 0.2 )
        self.header_bg:SetTintColour( UICOLOURS.NEGOTIATION )
        self.header_bg:SetTintAlpha( 0.2 )
    end

    -- Get the corner label size, and scale up its background to match
    if card:IsFlagged( negotiation_defs.CARD_FLAGS.INCEPTED ) then
        -- Incepted intent!
        self.header_left:SetText( string.upper( LOC"UI.NEGOTIATION_PANEL.INTENT_INCEPTED" ) )
        self.header_left:SetGlyphColour( UICOLOURS.PENALTY )
        self.corner_label_bg:SetTintColour( UICOLOURS.PENALTY )
        self.corner_label_bg:SetTintAlpha( 0.2 )
    else
        self.header_left:SetText( string.upper( LOC"UI.NEGOTIATION_PANEL.INTENT" ) )
    end

    local label_w, label_h = self.header_left:GetSize()
    local nineslice_w, nineslice_h = label_w + bg_border*2*bg_border_scale, 68
    self.corner_label_bg:SetSize( nineslice_w+padding*2.5, nineslice_h )
    self.corner_label_bg:SetPos( 0, 0 )
    self.header_left:LayoutBounds( "left", "bottom", self.corner_label_bg ):Offset( padding, 8 )

    -- Show the header background
    local h_w, h_h = self.header:GetSize()
    local nineslice_w, nineslice_h = DEFAULT_TT_WIDTH + bg_border*2*bg_border_scale, h_h + bg_border*2*bg_border_scale
    self.header_bg:SetSize( nineslice_w, nineslice_h )
    self.header_bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )

    -- Set the countdown
    local delay = card:GetTurnDelay() or 1
    self.header_right:SetText( loc.format( LOC"UI.NEGOTIATION_PANEL.INTENT_TURNS", delay ))
    self.header_right:LayoutBounds( "right", "bottom", self.header_bg ):Offset( -padding, 9 )

    -- Set the description
    local persuasion, _, details = card.engine:PreviewPersuasion( card )
    local desc_text
    if (card.max_persuasion or 0) > 0 then
        desc_text = loc.format( LOC"UI.NEGOTIATION_PANEL.INTENT_DAMAGE_TT", persuasion )
        if details then
            local text = loc.format( LOC"UI.NEGOTIATION_PANEL.EFFECT_DETAILS_TOOLTIP", details )

            local row = Widget.Label("tooltip",20, text ):SetAutoSize( content_w ):SetWordWrap( true ):LeftAlign():SetGlyphColour( UICOLOURS.TOOLTIP_TEXT ):Bloom(0.05)
            self.keyword_entries:AddChild( row )
            row:LayoutBounds( "left", "below" )
        end

    elseif card.features and card.features.COMPOSURE then
        desc_text = LOC"UI.NEGOTIATION_PANEL.INTENT_COMPOSURE_TT"
    elseif card:IsFlagged( negotiation_defs.CARD_FLAGS.INCEPTED ) then
        desc_text = loc.format( "<b>{1}</b>\n{2}", card:GetName(), card:GetDesc() )
    else
        desc_text = loc.format( LOC"UI.NEGOTIATION_PANEL.INTENT_ARGUMENT_TT", card.negotiator:GetName() )
    end
    desc_text = CardEngine.ExtractKeywords( desc_text, self, card.engine, negotiation_defs )

    self.desc:SetText( desc_text )
    self.desc:LayoutBounds( "left", "below", self.header_bg ):Offset( 16, -6 )

    -- Go through all the keywords for this card
    -- Add details
    self.keyword_entries:LayoutBounds( "left", "below", self.desc ):Offset( 0, -15 )

    -- Size up the background
    local c_w, c_h = self.content:GetSize()
    self.bg:SetSize( DEFAULT_TT_WIDTH, c_h + 4 )
    self.bg:LayoutBounds( "left", "top", self.corner_label_bg ):Offset( 0, 0 )
end
