require "ui/widget"
require "ui/widgets/panel"
require "ui/widgets/label"

local panel_w = 412
local panel_h = 128
local padding = 8
local columns = 2
local icon_size = 26
local text_w = (panel_w/2)-icon_size

local TooltipSaveFile = class("Widget.TooltipSaveFile", Widget)

function TooltipSaveFile:init()
	TooltipSaveFile._base.init(self)
	self.bg = self:AddChild(Widget.Panel(global_images.tooltipbg))
    self.bg:SetNineSliceBorderScale( 0.5 )

    self.contents = self:AddChild(Widget())

    self.header_widget = self.contents:AddChild( Widget() )
    self.title = self.header_widget:AddChild( Widget.Label("title", 26, "") )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour(UICOLOURS.OBJECTIVE)
        :Bloom(0.1)
    self.text = self.header_widget:AddChild( Widget.Label("tooltip", 22) )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

    self.mutators_widget = self.contents:AddChild( Widget() ):Hide()
    self.mutators_label = self.mutators_widget:AddChild( Widget.Label("body", 26 ))
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom(0.15)
    self.mutators_text = self.mutators_widget:AddChild( Widget.Label("body", 22 ))
        :SetText( LOC"UI.NEW_GAME_SCREEN.SAVEFILE_MUTATORS_TT" )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    self.mutators_container = self.mutators_widget:AddChild( Widget() )

    self.mods_label = self.contents:AddChild( Widget.Label( "body", 26 ))
        :SetAutoSize( panel_w )
        :LeftAlign()
        :SetWordWrap( true )
        :Bloom( 0.15 )

    self.invalid_save_label = self.contents:AddChild( Widget.Label( "body", 26 ))
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :SetGlyphColour( UICOLOURS.REQUIRES )
        :LeftAlign()

	self:Hide()
end

function TooltipSaveFile:DoLayout( save_data )
    self.save_data = save_data

    local options = save_data and save_data.data.options
    if options then
        local act_id = options.start_act
        local act_data = GetPlayerActData(act_id)
        local act_name = act_data and act_data:GetLocalizedName()
        self.title:SetText( self.save_data.is_dev and loc.format( LOC"UI.NEW_GAME_SCREEN.SAVEFILE_TITLE_DEV", act_name, self.save_data.name ) or loc.format( LOC"UI.NEW_GAME_SCREEN.SAVEFILE_TITLE", act_name )):Show()
        self.text:SetText( self.save_data.data and self.save_data.data.desc or "" )
        self.text:LayoutBounds( "left", "below", self.title )
            :Offset( 0, 0 )

        -- Check if we have mutators
        if options.mutators and #options.mutators > 0 then
            self.mutators_label:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SAVEFILE_MUTATORS", #options.mutators ) )
            self.mutators_text:LayoutBounds( "left", "below", self.mutators_label ):Offset( 0, 0 )
            self.mutators_container:DestroyAllChildren()
            for k, v in ipairs( options.mutators ) do
                local graft = Content.GetGraft( v )
                if graft then
                    local row = self.mutators_container:AddChild( Widget.Panel( engine.asset.Texture("UI/empty_inner_frame_border.tex" ) ) )
                        :SetSize( panel_w/2, 32 )
                    local icon = row:AddChild( Widget.Image( graft.img ) )
                        :SetSize( icon_size, icon_size )
                        :Bloom( 0.15 )
                        :LayoutBounds( "left", "center", row )
                        :Offset( 3, 0 )
                    local label = row:AddChild( Widget.Label( "body", 22 ) )
                        :SetAutoSize( text_w )
                        :SetWordWrap( true )
                        :SetText( graft:GetLocalizedName() )
                        :Bloom(0.1)
                        :LayoutBounds( "after", "center", icon )
                        :Offset( 6, 1.5 )
                end
            end
            self.mutators_container:LayoutChildrenInGrid( columns, 3 )
            self.mutators_container:LayoutBounds( "left", "below", self.mutators_text ):Offset( 0, -4 )
            self.mutators_widget:Show()
        else
            self.mutators_widget:Hide()
        end

        if options.required_mods and #options.required_mods > 0 then
            local mod_names = {}
            for i, v in ipairs( options.required_mods ) do
                local mod_name = tostring(v.title or v.id)
                if v.version then
                    mod_name = mod_name .. " v"..tostring(v.version)
                end
                table.insert( mod_names, mod_name )
            end
            self.mods_label:SetText( loc.format( LOC"UI.NEW_GAME_SCREEN.SAVE_MODS_REQUIRED", mod_names )):Show()
            self.mods_label:LayoutBounds( "left", "below", self.mutators_container )
        else
            self.mods_label:Hide()
        end

        local ok, reason = TheGame:CanLoadSave( save_data.data )
        if not ok then
            self.invalid_save_label:SetText( reason or LOC "UI.NEW_GAME_SCREEN.CORRUPT_SAVE" ):Show()
            self.mods_label:LayoutBounds( "left", "below", self.mods_label:IsShown() and self.mods_label or self.mutators_container )
        else
            self.invalid_save_label:Hide()
        end
    end

    self.contents:SetRegistration("center", "center")
    self.contents:StackChildren( 11 )
    self.bg:SizeToWidgets( padding, self.contents )
    
end
