require "ui/widget"

local portrait_size = 80

---------------------------------------------------------------------------------------------

local TooltipAgent = class("Widget.TooltipAgent", Widget )

function TooltipAgent:init()
	TooltipAgent._base.init( self )

    self.portrait = Widget.CharacterPortrait():SetHeight( portrait_size )
    self:AddChild( self.portrait )

    self.name = Widget.Label( "title", 32 ):SetSize( 180, 36 ):LeftAlign()
        :SetGlyphColour( UICOLOURS.TITLE ):Bloom(0.15)
        :SetAutoSize( DEFAULT_TT_WIDTH-portrait_size-SPACING.M1*2 )
        :SetWordWrap( true )
    self:AddChild( self.name )

    self.body = Widget.Label( "tooltip", 24 ):SetWordWrap( true ):LeftAlign()
        :SetAutoSize( DEFAULT_TT_WIDTH-portrait_size-SPACING.M1*2 )
        :SetWordWrap( true )
    self:AddChild( self.body )

    self.social_details = Widget.Label( "tooltip", 24 ):LeftAlign()
    self:AddChild( self.social_details )

    self.bg = Widget.Panel(global_images.tooltipbg)
    self.bg:SetNineSliceBorderScale( 0.5 )
    self:AddChild( self.bg, 1 )
end

function TooltipAgent:DoLayout( agent_or_guid )
    
    local agent
    if type(agent_or_guid) == "number" then
        agent = TheGame:GetGameState():GetAgentOrMemento( agent_or_guid )
    else
        agent = agent_or_guid
    end

    if is_instance(agent, Agent) then
    	self.portrait:SetCharacter( agent )

		self.name:SetText( agent:GetName() or agent:GetShortTitle() )
    	self.name:LayoutBounds( "after", "top", self.portrait ):Offset( SPACING.M1, 0 )

        self.body:SetText( agent:GetIdentity() )
        self.body:LayoutBounds( "left", "below" )

        self.social_details:Hide()
        local social = agent:GetSocialConnections()
        if social then
            local player = TheGame:GetGameState() and TheGame:GetGameState():GetPlayerAgent()
            if player then
                local history = social:GetRelationshipHistory( player )
                if #history > 0 then
                    local body = loc.format( "<#HILITE>{1}</>", history )
                    self.social_details:SetText( body ):Show():LayoutBounds( "left", "below", self.body ):Offset( 0, -4 )
                end
            end
        end

        if self.social_details:IsShown() then
            self.bg:SizeToWidgets( 10, self.portrait, self.name, self.body, self.social_details )
        else
            self.bg:SizeToWidgets( 10, self.portrait, self.name, self.body )
        end

    else
        return false -- Couldn't find agent, suppress tooltip.
    end
end

---------------------------------------------------------------------------------------------


local TooltipAgentRelationship = class("Widget.TooltipAgentRelationship", Widget )

function TooltipAgentRelationship:init()
    TooltipAgentRelationship._base.init( self )

    self.bg = self:AddChild( Widget.Panel(global_images.tooltipbg) )
        :SetNineSliceBorderScale( 0.5 )

    self.portrait = self:AddChild( Widget.CharacterPortrait() )
        :SetHeight( 80 )

    self.name = self:AddChild( Widget.Label("title", 28, "") )
        --:SetAutoSize( text_w )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour(UICOLOURS.BODYTEXT)
        :Bloom(0.1)
    self.body = self:AddChild( Widget.Label("tooltip", 24) )
        --:SetAutoSize( text_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

    self.reason = self:AddChild( Widget.Label("body", 22 ) )
        --:SetAutoSize( text_w )
        :SetWordWrap(true)
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

end

function TooltipAgentRelationship:DoLayout( agent_or_guid )
    local agent
    if type(agent_or_guid) == "number" then
        agent = TheGame:GetGameState():GetAgentOrMemento( agent_or_guid )
    else
        agent = agent_or_guid
    end

    if agent and is_instance(agent, Agent) then
        self.portrait:SetCharacter( agent )

        self.name:SetText( agent:GetName() or agent:GetShortTitle() )
            :LayoutBounds( "after", "top", self.portrait ):Offset( 17, 0 )

        self.body:SetText( agent:GetIdentity() )
            :LayoutBounds( "left", "below", self.name )
            :Offset( 0, 2 )


        -- Set the agent's bio and reason for their opinion of you
        local why_event = agent.social_connections and agent.social_connections:GetRelationshipReason(TheGame:GetGameState():GetPlayerAgent())
        if why_event then
            self.reason:SetText( loc.format( LOC"UI.AGENT_RELATIONSHIP_REASON_TT", FormatOpinionEvent( why_event, agent ) ) )
                :LayoutBounds( "left", "below", self.body )
                :Offset( 0, -6 )
        else
            self.reason:SetText( "" )
        end

        self.bg:SizeToWidgets( 10, self.portrait, self.name, self.body, self.reason )
    end
end
