require "ui/widget"

local ICON_SIZE = 28

---------------------------------------------------------------------------------------------
-- Tooltip for crew member portraits.

local TooltipCrew = class("Widget.TooltipCrew", Widget )

function TooltipCrew:init()
	TooltipCrew._base.init( self )

    self.name = Widget.Label( "title", 32 ):LeftAlign()
        :SetGlyphColour( UICOLOURS.TITLE ):Bloom(0.15)
    self:AddChild( self.name )

    self.body = Widget.Label( "tooltip", 24 ):LeftAlign()
    self:AddChild( self.body )

    self.opinion = Widget.OpinionIndicator( ICON_SIZE )
    self:AddChild( self.opinion )

    self.health = Widget.HealthIndicator( ICON_SIZE, 26 )
    self:AddChild( self.health )

    self.bg = Widget.Panel(global_images.tooltipbg)
    self.bg:SetNineSliceBorderScale( 0.5 )
    self:AddChild( self.bg, 1 )
end

function TooltipCrew:DoLayout( agent_or_guid )
    
    local agent
    if type(agent_or_guid) == "number" then
        agent = TheGame:GetGameState():GetAgentOrMemento( agent_or_guid )
    else
        agent = agent_or_guid
    end

    if is_instance(agent, Agent) then

        local name = loc.format( "{1}", agent:GetName() )
		self.name:SetText( name )
        
        self.body:SetText( agent:GetIdentity() )
        self.body:LayoutBounds( "left", "below", self.name )

        self.opinion:UpdateOpinion( agent )
            :LayoutBounds( "left", "below", self.body ):Offset( 0, -8 )

        self.health:Refresh( agent )
        if self.opinion:IsShown() then
            self.health:LayoutBounds( "after", "bottom", self.opinion ):Offset( 12, 0 )
        else
            self.health:LayoutBounds( "left", "below", self.body ):Offset( 0, -8 )
        end

        self.bg:SizeToWidgets( 10, self.name, self.body, self.health, self.opinion )
    end
end

