require "ui/widget"

local TooltipFlourish = class("Widget.TooltipFlourish", Widget )

local panel_w = 412

function TooltipFlourish:init()
	TooltipFlourish._base.init( self )

    self.bg = self:AddChild(Widget.Panel(global_images.tooltipbg))
    self.bg:SetNineSliceBorderScale( 0.5 )

    self.contents = self:AddChild(Widget())

    self.title = self.contents:AddChild( Widget.Label( "title", 26 ) )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom(0.1)
    self.subheader = self.contents:AddChild( Widget() )
    self.flourish_widget = self.subheader:AddChild( Widget.FlourishProgressWidget() )
        :SetToolTipClass( nil )
        :SetToolTip( nil )
        :SetScale( 0.6 )
    local w, h = self.flourish_widget:GetScaledSize()
    self.desc_1 = self.subheader:AddChild( Widget.Label( "tooltip", 22 ) )
        :SetAutoSize( panel_w - w - SPACING.M1/2 )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
    self.flourish_label = self.contents:AddChild( Widget.Label( "body", 26 ) )
        :SetGlyphColour( UICOLOURS.FLOURISH )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :Bloom(0.15)
    self.desc_2 = self.contents:AddChild( Widget.Label( "tooltip", 22 ) )
        :SetAutoSize( panel_w )
        :SetWordWrap( true )
        :LeftAlign()
        :OverrideLineHeight( 18 )
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)
end

function TooltipFlourish:DoLayout( flourish_tracker )

    -- Set data
    local current, max = flourish_tracker:GetValues()
    self.flourish_label:SetText( loc.format(LOC"UI.FIGHT_SCREEN.FLOURISH_LABEL", math.floor(current), max) )
    self.flourish_widget:AnimValue( current )
    local agent = flourish_tracker:GetAgent()
    local graft = flourish_tracker:GetGraft()
    -- Check whether we're in a fight or negotiation
    if flourish_tracker:IsBattle() then
        -- Get point multipliers from the graft
        local card_played = graft.userdata.battle_gains["CARD_PLAY"]
        local damage_dealt = graft.userdata.battle_gains["DAMAGE_DEALT"]
        local damage_received = graft.userdata.battle_gains["DEFENDED"]
        local enemy_killed = graft.userdata.battle_gains["KILL"]
        -- Update text
        self.title:SetText( LOC"UI.FIGHT_SCREEN.FLOURISH_TITLE_FIGHT" )
        self.desc_1:SetText( loc.format( LOC"UI.FIGHT_SCREEN.FLOURISH_DESC_1_FIGHT", agent:GetName() ) )
        self.desc_2:SetText( loc.format( LOC"UI.FIGHT_SCREEN.FLOURISH_DESC_2_FIGHT", card_played, damage_dealt, damage_received, enemy_killed ) )
    end
    if flourish_tracker:IsNegotiation() then
        -- Get point multipliers from the graft
        local card_played = graft.userdata.negotiation_gains["CARD_PLAY"]
        local damage_dealt = graft.userdata.negotiation_gains["DAMAGE_DEALT"]
        local damage_mitigated = graft.userdata.negotiation_gains["MITIGATED"]
        local argument_destroyed = graft.userdata.negotiation_gains["DESTROY"]
        -- Update text
        self.title:SetText( LOC"UI.FIGHT_SCREEN.FLOURISH_TITLE_NEGOTIATION" )
        self.desc_1:SetText( loc.format( LOC"UI.FIGHT_SCREEN.FLOURISH_DESC_1_NEGOTIATION", agent:GetName() ) )
        self.desc_2:SetText( loc.format( LOC"UI.FIGHT_SCREEN.FLOURISH_DESC_2_NEGOTIATION", card_played, damage_dealt, damage_mitigated, argument_destroyed ) )
    end

    -- Layout
    self.flourish_widget:LayoutBounds( "left", "below", self.title ):Offset( 0, -SPACING.M1/2 )
    self.desc_1:LayoutBounds( "after", "top", self.flourish_widget ):Offset( SPACING.M1/2, 0 )
    self.flourish_label:LayoutBounds( "left", "below", self.subheader ):Offset( 0, -SPACING.M1/2 )
    self.desc_2:LayoutBounds( "left", "below", self.flourish_label )
    self.bg:SizeToWidgets( SPACING.M1/2, self.contents )
end

