require "ui/widget"

local assets = {
    danger_skull = engine.asset.Texture( "UI/danger_skull.tex" )
}

local icon_sz = 32

---------------------------------------------------------------------------------------------

local TooltipQuest = class("Widget.TooltipQuest", Widget )

function TooltipQuest:init()
	TooltipQuest._base.init( self )

    self.others = self:AddChild(Widget())
    self.battle_card_widget = self.others:AddChild( Widget.TooltipBattleCard() ):Hide()
    self.negotiation_card_widget = self.others:AddChild( Widget.TooltipNegotiationCard() ):Hide()

    self.bg = self:AddChild( Widget.Panel(global_images.tooltipbg) )
    self.bg:SetNineSliceBorderScale( 0.5 )

    self.content = self:AddChild( Widget() )

    -- Title and subtitle
    self.title_label = self.content:AddChild( Widget.Label( "body", 22, LOC"UI.JOB_POPUP.TT_TITLE_LABEL" ) )
        :SetAutoSize( DEFAULT_TT_WIDTH  )
        :SetWordWrap( true )
        :LeftAlign()
        :SetHoverLinkGlyphColour( UICOLOURS.QUEST )
        :SetGlyphColour( UICOLOURS.QUEST )
        :Bloom( 0.05 )
    self.title = self.content:AddChild( Widget.Label( "title", 30 ) )
        :SetAutoSize( DEFAULT_TT_WIDTH  )
        :SetWordWrap( true )
        :LeftAlign()
        :SetHoverLinkGlyphColour( UICOLOURS.QUEST_LIGHT )
        :SetGlyphColour( UICOLOURS.QUEST_LIGHT )
        :Bloom( 0.05 )

    self.difficulty_widget = self.content:AddChild( Widget.QuestDifficultyWidget() )
    self.focus_widget = self.content:AddChild( Widget.QuestFocusWidget() )

    self.desc = self.content:AddChild( Widget.Label("tooltip", 22) )
        :SetAutoSize( DEFAULT_TT_WIDTH )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :SetHoverLinkGlyphColour( UICOLOURS.TOOLTIP_TEXT )
        :Bloom(0.1)

    self.rewards = self.content:AddChild( Widget.RewardsWidget( DEFAULT_TT_WIDTH, 1 ) )

    self.danger_indicator = self:AddChild(Widget.Image(assets.danger_skull)):SetSize(28,28):Hide()
        :SetTintColour( UICOLOURS.PENALTY )
        :Bloom( 0.05 )

    self:Hide()
end

function TooltipQuest:DoLayout( data )
    self.quest = data.quest

    self.title:SetText( self.quest:GetTitle() )
    if self.quest:IsInactive() or data.show_as_inactive then
        -- Available quest
        self.title_label:Show()
        self.title:LayoutBounds( "left", "below", self.title_label):Offset( 0, 4 )

        self.difficulty_widget:Refresh( self.quest:GetRank() )
            :LayoutBounds( "left", "below", self.title ):Offset( 0, -5 ):Show()

        self.focus_widget:Refresh( self.quest:GetRatingString() )
            :LayoutBounds( "after", "top", self.difficulty_widget ):Offset( 30, 0 ):Show()

        self.desc:SetText( self.quest:GetDesc() )
        self.desc:LayoutBounds( "left", "below", self.difficulty_widget ):Offset( 0, -10 )

        self.rewards:RefreshRewards( self.quest )
        self.rewards:LayoutBounds( "left", "below", self.desc ):Offset( 0, -10 ):Show()


    else
        -- Active quest
        self.title_label:Hide()
        self.difficulty_widget:Hide()
        self.focus_widget:Hide()
        self.rewards:Hide()

        local tt = {}

        if self.quest:GetType() == QTYPE.OPPORTUNITY then
            table.insert (tt, LOC"UI.JOB_POPUP.TT_OPPORTUNITY")
        end

        local show_obj = false
        for i, obj in ipairs(self.quest.objectives) do
            if self.quest:IsActive(obj.id) and (data.objectives == nil or table.contains(data.objectives, obj.id)) then
                local title, desc = self.quest:GetObjectiveTitle(obj.id), self.quest:GetObjectiveDesc(obj.id)
                if title and desc then
                    show_obj = true
                    table.insert(tt, desc)
                end
            end
        end

        if not show_obj then
            local desc = self.quest:GetDesc()
            if desc then
                table.insert(tt, desc)
            end
        end

        self.desc:SetText( table.concat(tt, "\n") )
        self.desc:LayoutBounds( "left", "below", self.title ):Offset( 0, -4 )

    end

    self.bg:SizeToWidgets( 2, self.content )

    if data.quest:GetRank() >  TheGame:GetGameState():GetDefaultQuestRank() then
        self.title_label:SetText( LOC"UI.JOB_POPUP.TT_TITLE_LABEL_DANGER" )
            :LayoutBounds( "left", nil, self.title)
        self.danger_indicator:LayoutBounds("right", "top", self.bg):Offset(-10, -13):Show()
        self.danger_indicator:Pulse( 0.6, 1.0, 0.003 )
    else
        self.title_label:SetText( LOC"UI.JOB_POPUP.TT_TITLE_LABEL" )
            :LayoutBounds( "left", nil, self.title)
        self.danger_indicator:Hide()
        self.danger_indicator:StopPulse()
        self.danger_indicator:SetTintAlpha( 1 )
    end

    self.battle_card_widget:Hide()
    self.negotiation_card_widget:Hide()

    if self.quest:IsInactive() or self.quest:GetType() == QTYPE.OPPORTUNITY then
        local t = {}
        data.quest:CollectTooltipItems(t)
        table.sort(t)
        

        for k, card_id in ipairs(t) do
            local battle_card = card_id and Content.GetBattleCard( card_id )
            local negotiation_card = card_id and Content.GetNegotiationCard( card_id )
            if battle_card then
                self.battle_card_widget:DoLayout({card_def =battle_card})
                self.battle_card_widget:Show():LayoutBounds("after", "top"):Offset(5,0)
            end

            if negotiation_card then
                self.negotiation_card_widget:DoLayout({card_def=negotiation_card})
                self.negotiation_card_widget:Show():LayoutBounds("after", "top"):Offset(5,0)
            end
        
        end



        self.others:LayoutBounds("after", "top", self.bg):Offset(5,0)
    end

end

------------------------------------------------------------------------------------------
-- Displays a quest difficulty widget

local QuestDifficultyWidget = class( "Widget.QuestDifficultyWidget", Widget )

function QuestDifficultyWidget:init()
    QuestDifficultyWidget._base.init(self)

    self.difficulty_label = self:AddChild( Widget.Label( "body", 22, LOC"UI.JOB_POPUP.RANK_LABEL" ) )
        :SetAutoSize( self.width )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.QUEST )
    self.difficulty_stars = self:AddChild( Widget() )
    for k=1, 5 do
        self.difficulty_stars:AddChild( Widget.Image( engine.asset.Texture( "UI/star_full.tex" ) ) )
            :SetSize( 30, 30 ):LayoutBounds( "after", "center" ):Offset( 1, 0 )
    end
    self.difficulty_stars:LayoutBounds( "left", "below", self.difficulty_label )

end

function QuestDifficultyWidget:Refresh( difficulty )
    self.difficulty = difficulty or 3

    for k, star in ipairs( self.difficulty_stars.children ) do
        star:SetTexture( engine.asset.Texture( k<=self.difficulty and "UI/star_full.tex" or "UI/star_empty.tex" ) )
    end

    return self
end

------------------------------------------------------------------------------------------
-- Displays a quest focus (combat/negotiation/both) widget

local QuestFocusWidget = class( "Widget.QuestFocusWidget", Widget )

function QuestFocusWidget:init()
    QuestFocusWidget._base.init(self)

    self.focus_label = self:AddChild( Widget.Label( "body", 22, LOC"UI.JOB_POPUP.FOCUS_LABEL" ) )
        :SetAutoSize( self.width  )
        :SetWordWrap( true )
        :LeftAlign()
        :SetGlyphColour( UICOLOURS.QUEST )
    self.focus_bg = self:AddChild( Widget.Panel( engine.asset.Texture("UI/dropdown_inner.tex" ) ) )
        :SetBloom(0.1)
    self.focus_value = self:AddChild( Widget.Label( "title", 22 ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetPos( 0, 1 )

end

function QuestFocusWidget:Refresh( focus_value_string, focus_color, focus_tooltip_string )
    self.focus_value_string = focus_value_string or ""
    self.focus_color = focus_color or UICOLOURS.TITLE
    self.focus_tooltip_string = focus_tooltip_string or ""

    self.focus_value:SetText( self.focus_value_string )
        :SetGlyphColour( UICOLOURS.BLACK )
    local text_w, text_h = self.focus_value:GetSize()
    self.focus_bg:SetSize( text_w + 24, text_h + 10 )
        :SetTintColour( self.focus_color )
        :SetTintAlpha( 0.7 )
    self.focus_label:LayoutBounds( "left", "above", self.focus_bg )
        :Offset( 0, 0 )

    return self
end