
local TrackedAgentPortrait = class("Widget.TrackedAgentPortrait", Widget )

local portrait_w, portrait_h = 60, 80
local portrait_padding = 10
local portrait_extra_space = 20 -- extra padding added to the scissor on the side they're facing

function TrackedAgentPortrait:init( agent )
    TrackedAgentPortrait._base.init(self)
    self.agent = agent
    self.size_scale = 1

    self.highlight_widget = Widget()

    self.portrait_bg = Widget.Image( engine.asset.Texture( "UI/location_portrait_bg.tex" ) ):SetSize( 120, 120 ):SetBloom( 0.05 )
    self.portrait_mask = Widget.Image( engine.asset.Texture( "UI/location_portrait_mask.tex" ) ):SetSize( 120, 120 )
    self.portrait_top = Widget.Image( engine.asset.Texture( "UI/location_portrait_top.tex" ) ):SetSize( 120, 120 )
    
    self.anim_mask = Widget()
    self.anim_scale = Widget()

    self.anim = Widget.AnimatedCharacter():SetPos(0, -portrait_h/2):SetPortraitMode()
    self.anim.hover_check = true
    self.anim:SetHiddenBoundingBox(true)
    self.anim:DisableBlink()
    self.anim:Refresh( agent ):Show()
    self.anim:Play( "portrait_neutral" )

    self:AddChildren
    {
        self.highlight_widget:AddChildren
        {
            self.portrait_bg,
            self.anim_mask:SetStencilContext( true ):AddChildren
            {
                self.portrait_mask:SetMask(true),
                self.anim_scale:AddChildren 
                {
                    self.anim:SetStencilTest(true)
                }
            },
            self.portrait_top
        }
    }

    self:SetHeight(60)
end

function TrackedAgentPortrait:ShowReflection( height, yoffset )
    height = height or 50
    self.shadow_offset = yoffset
    self.shadow1 = Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ):SetBloom(0.1)
    self.shadow1:SetHiddenBoundingBox(true)
    self.shadow1:SetTintColour( UICOLOURS.SUBTITLE )
    self.shadow1:SetSize( 180, height )
    self:AddChild( self.shadow1 )
    self.shadow2 = Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ):SetBloom(0)
    self.shadow2:SetHiddenBoundingBox(true)
    self.shadow2:SetTintColour( UICOLOURS.SUBTITLE )
    self.shadow2:SetSize( 90, height*0.3 )
    self:AddChild( self.shadow2 )
    self.shadow3 = Widget.Image( engine.asset.Texture("UI/negotiation_iconglow.tex") ):SetBloom(0)
    self.shadow3:SetHiddenBoundingBox(true)
    self.shadow3:SetTintColour( UICOLOURS.SUBTITLE )
    self.shadow3:SetSize( 50, height*0.2 )
    self:AddChild( self.shadow3 )
    self:SetHeight( self.portrait_h )
    return self
end

function TrackedAgentPortrait:GetBoundingBox()
    local w = self.portrait_h*(portrait_w/portrait_h)
    return -w/2, -self.portrait_h/2, w/2, (self.portrait_h/2)*.67
end

function TrackedAgentPortrait:SetHeight( h )
    self.portrait_h = h
    self.size_scale = h/portrait_h
    self.highlight_widget:SetScale( self.facing and self.size_scale or -self.size_scale, self.size_scale )
    -- self.anim_scale:SetScale( self.size_scale, self.size_scale )
    self.artw = self.size_scale*portrait_w
    self.arth = self.size_scale*portrait_h
    self.anim:SetPos( 0, -self.arth/2 + 5*self.size_scale )
    if self.shadow1 then self.shadow1:SetPos( 0, self.shadow_offset and -self.shadow_offset or -self.arth ) end
    if self.shadow2 then self.shadow2:SetPos( 0, self.shadow_offset and -self.shadow_offset or -self.arth ) end
    if self.shadow3 then self.shadow3:SetPos( 0, self.shadow_offset and -self.shadow_offset or -self.arth ) end
    self:SetFacing(self.facing or true)
    return self
end

function TrackedAgentPortrait:SetFacing(facing)
    self.facing = facing
    self.highlight_widget:SetScale( self.facing and self.size_scale or -self.size_scale, self.size_scale )
    return self
end