require "ui/widget"


local VisibilityPanel = class("Widget.VisibilityPanel", Widget)


function VisibilityPanel:init( )
    VisibilityPanel._base.init(self)

    self.w = 0
    self.h = 0
   
end


function VisibilityPanel:SetSize(w, h)
    if self.w ~= w or self.h ~= h then
        self.w = w
        self.h = h
        self:InvalidateBBox()
    end
end

function VisibilityPanel:OnVizChange(viz)
    if viz then
        self:StartUpdating()
    else
        self:StopUpdating()
    end
end

function VisibilityPanel:GetBoundingBox()
    local half_width =  (self.w or 0) / 2
    local half_height =  (self.h or 0) / 2
    return -half_width, -half_height, half_width, half_height;
end

function VisibilityPanel:SetVisibility(visible, force)
    if (self.currently_visible ~= visible) or force then
        self.currently_visible = visible

        if self.children then
            for k,v in pairs(self.children) do
                v:SetShown(self.currently_visible)
            end
        end

        return true -- changed!
    end

    return false    -- no change
end


function VisibilityPanel:UpdateVisibility(force)
    local x1, y1, x2, y2 = self:GetBoundingBox()
    x1, y1 = self:TransformToWorld( x1, y1 )
    x2, y2 = self:TransformToWorld( x2, y2 )
    local xmin = math.min( x1, x2 )
    local xmax = math.max( x1, x2 )
    local ymin = math.min( y1, y2 )
    local ymax = math.max( y1, y2 )

    -- Get the scissor rect of the parent, and clip against that:
    local sx1, sy1, sx2, sy2 = self:GetScissorRect()
    local sxmin = math.min( sx1, sx2 )
    local sxmax = math.max( sx1, sx2 )
    local symin = math.min( sy1, sy2 )
    local symax = math.max( sy1, sy2 )

    local invisible = (xmax < sxmin) or (xmin > sxmax) or (ymax < symin) or (ymin > symax)

    self:SetVisibility(not invisible, force)

end

function VisibilityPanel:OnUpdate(dt)
    VisibilityPanel._base:OnUpdate(dt)

    self:UpdateVisibility()
end

