require "ui/widgets/iconbutton"

local TITLE_LAYOUT_SCALE =
{
    [SCREEN_MODE.MONITOR] = 1,
    [SCREEN_MODE.TV] = 1,
    [SCREEN_MODE.SMALL] = 1,
}

----------------------------------------------------------------------------------------------------------------------------
-- Displays a panel that takes up the entire width of the screen, which has custom icons, color and height

local WidePanel = class("Widget.WidePanel", Widget)

function WidePanel:init( height, colour )
    WidePanel._base.init(self)

    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

    -- A container for all the background elements
    self.panel = self:AddChild( Widget() )--:BlockMouse()
    self.panel_glow_top = self.panel:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ), 1200, 300 ) )
        :SetHiddenBoundingBox( true )
    self.panel_glow_bottom = self.panel:AddChild( Widget.Image( engine.asset.Texture("images/glow.tex" ), 800, 300 ) )
        :SetHiddenBoundingBox( true )
    self.panel_bg = self.panel:AddChild( Widget.Image( engine.asset.Texture("large/dialog_bg.tex" ) ) )
    self.panel_behind = self.panel:AddChild( Widget.Image( engine.asset.Texture("large/dialog_behind.tex" ), 400, 200 ) ):Bloom( 0.1 )
        :SetHiddenBoundingBox( true )
    self.panel_grid = self.panel:AddChild( Widget.Image( engine.asset.Texture("large/dialog_grid.tex" ) ) )
        :SetHiddenBoundingBox( true )
    self.panel_top = self.panel:AddChild( Widget.Image( engine.asset.Texture("large/dialog_top.tex" ), 300, 150 ) ):Bloom( 0.1 )
        :SetHiddenBoundingBox( true )
    self.panel_icon = self.panel:AddChild( Widget.Image( engine.asset.Texture("ui/card_type_negotiation.tex"), 70, 70 ) ):SetBloom(0.1)
        :SetHiddenBoundingBox( true )
    self.panel_border_top = self.panel:AddChild( Widget.SolidBox( RES_X, 6, 0xffffffff ) )
    self.panel_bottom = self.panel:AddChild( Widget.Image( engine.asset.Texture("large/dialog_bottom.tex" ), 300, 150 ) ):Bloom( 0.1 )
        :SetHiddenBoundingBox( true )
    self.panel_border_bottom = self.panel:AddChild( Widget.SolidBox( RES_X, 3, 0xffffffff ) )
        :SetHiddenBoundingBox( true )

    -- Texts
    self.titles = self:AddChild( Widget() )
	self.title = self.titles:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_TITLE, "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize(800)
        :SetWordWrap(true)
		:Bloom( 0.1 )
    self.subtitle = self.titles:AddChild( Widget.Label( "title", FONT_SIZE.SCREEN_SUBTITLE, "" ) )
        :SetGlyphColour( UICOLOURS.SUBTITLE )
        :SetAutoSize(800)
        :SetWordWrap(true)
        :SetTintAlpha( 0.8 )
        :Bloom( 0.05 )
    self.text = self.titles:AddChild( Widget.Label( "body", FONT_SIZE.BODY_TEXT ) )
        :SetAutoSize(800)
        :SetWordWrap(true)
        :SetGlyphColour( 0xCBFFEFff )
        :Bloom( 0.1 )
    self.title_y_margin = 30

    -- Sub-buttons
    self.subbuttons = self:AddChild( Widget() )

    -- Set initial values
    self.panel_height = height or 560
    self.panel_colour = colour or UICOLOURS.SUBTITLE
    self.icon_scale = 1

    -- Set this entire thing up
    self:Layout()
end

function WidePanel:OnAdded()
    -- Refresh control mode display
    self:OnControlModeChange( TheGame:FE():GetControlMode(), TheGame:FE():GetControlDeviceID() )
    self:OnScreenModeChange( TheGame:FE():GetScreenMode() )
end

function WidePanel:OnControlModeChange( cm, device_id )
    WidePanel._base.OnControlModeChange( self, cm, device_id )

    if cm == CONTROL_MODE.MOUSE_KEYBOARD then
        self.subbuttons:Show()
    end
    if cm == CONTROL_MODE.TOUCH then
        self.subbuttons:Show()
    end
    if cm == CONTROL_MODE.GAMEPAD then
        self.subbuttons:Hide()
    end
end

function WidePanel:OnScreenModeChange( sm )
    WidePanel._base.OnScreenModeChange( self, sm )

    self.titles:SetLayoutScale( TITLE_LAYOUT_SCALE[sm] )
    self.subbuttons:SetLayoutScale( LAYOUT_SCALE[sm] )

    self:Layout()
end

function WidePanel:HidePanelFrame()
    self.panel:Hide()
    return self
end

function WidePanel:SetIcon( texture, colour, scale )
	self.icon_texture = texture
    self.icon_colour = colour
    self.icon_scale = scale or 1
    self:Layout()
    return self
end

function WidePanel:SetTitles( title, subtitle, desc, y_margin )
    if title then self.title:SetText( title ):Show() else self.title:Hide() end
    if subtitle then self.subtitle:SetText( subtitle ):Show() else self.subtitle:Hide() end
    if desc then self.text:SetText( desc ):Show() else self.text:Hide() end
    if y_margin then self.title_y_margin = y_margin end
    self:Layout()
    return self
end

function WidePanel:UpdateTitles( title, subtitle )
    if title then self.title:SetText( title ):Show() else self.title:Hide() end
    if subtitle then self.subtitle:SetText( subtitle ):Show() else self.subtitle:Hide() end

    -- Layout texts
    self.subtitle:LayoutBounds( "center", self.subtitle:IsShown() and "below" or "bottom", self.title ):Offset( 0, SPACING.TITLE_SUBTITLE )
    self.text:LayoutBounds( "center", self.text:IsShown() and "below" or "bottom", self.subtitle )
    self.titles:LayoutBounds( "center", "top", self.panel_bg ):Offset( 0, -self.title_y_margin )

    return self.title, self.subtitle, self.titles
end

function WidePanel:SetTopTitlesMargin( y_margin )
    self.title_y_margin = y_margin or self.title_y_margin or 40
    return self
end

function WidePanel:SetColour( colour )
	self.panel_colour = colour or UICOLOURS.SUBTITLE
    self:Layout()
    return self
end

function WidePanel:SetSize( w, h )
    self:SetHeight( h )
    self:Layout()
    return self
end

function WidePanel:SetHeight( height )
    if height ~= self.panel_height then
    	self.panel_height = height or 560
        self:Layout()
    end
    return self
end

function WidePanel:GetPanelWidget()
    return self.panel
end

function WidePanel:GetTitlesWidget()
    return self.titles
end

function WidePanel:GetTitleWidget()
    return self.title
end

function WidePanel:SetTitle( title )
    if title then self.title:SetText( title ):Show() else self.title:Hide() end
    self:Layout()
    return self
end

function WidePanel:GetSubtitleWidget()
    return self.subtitle
end

function WidePanel:SetSubtitle( subtitle )
    if subtitle then self.subtitle:SetText( subtitle ):Show() else self.subtitle:Hide() end
    self:Layout()
    return self
end

function WidePanel:SetDesc( desc )
    if desc then self.text:SetText( desc ):Show() else self.text:Hide() end
    self:Layout()
    return self
end

function WidePanel:GetTopEdgeWidget()
    return self.panel_border_top
end

function WidePanel:GetBottomEdgeWidget()
    return self.panel_border_bottom
end

function WidePanel:Layout()

	-- Set colours
	self.panel_bg:SetTintAlpha( 0.98 )
    self.panel_grid:Pulse( 0.4, 1.0, 0.003 )
    self.panel_glow_top:SetTintColour( self.panel_colour )
    self.panel_glow_top:SetTintAlpha( 0.4 )
    self.panel_glow_bottom:SetTintColour( self.panel_colour )
    self.panel_glow_bottom:SetTintAlpha( 0.3 )
    self.panel_border_top:SetTintColour( self.panel_colour )
    self.panel_border_bottom:SetTintColour( self.panel_colour )
    self.title:SetGlyphColour( self.panel_colour )
    self.subtitle:SetGlyphColour( self.panel_colour )
        :SetTintAlpha( 0.8 )

	if self.icon_texture then
    	self.panel_icon:SetTexture( self.icon_texture ):Show():SetScale( self.icon_scale, self.icon_scale )
        if self.icon_colour then self.panel_icon:SetTintColour( self.icon_colour ) end
        if self.icon_scale then self.panel_icon:SetScale( self.icon_scale, self.icon_scale ) end
	    self.panel_top:SetTintColour( self.panel_colour ):Show()
	    self.panel_behind:SetTintColour( self.panel_colour ):SetTintAlpha( 0.03 ):Show()
	    self.panel_bottom:SetTintColour( self.panel_colour ):Show()
	else
    	self.panel_icon:Hide()
	    self.panel_top:Hide()
	    self.panel_behind:Hide()
	    self.panel_bottom:Hide()
	end

    -- Get the screen size
    local screen_w, screen_h = TheGame:FE():GetScreenScissorDims()
    self.screen_w = math.ceil( screen_w / TheGame:FE():GetBaseWidgetScale() )
    self.screen_h = math.ceil( screen_h / TheGame:FE():GetBaseWidgetScale() )

	-- Size up everything
    self.panel_bg:SetSize( self.screen_w, self.panel_height )
    local grid_w, grid_h = self.panel_grid:GetSize()
    self.panel_grid:SetSize( grid_w * self.panel_height/grid_h, self.panel_height )
    self.panel_border_top:SetSize( self.screen_w, 3 )
    self.panel_border_bottom:SetSize( self.screen_w, 2 )
    self.panel_glow_top:SetSize( self.screen_w*1.3, 300 )
    self.panel_glow_bottom:SetSize( self.screen_w*0.9, 300 )

    -- Layout background everything
    self.panel_border_top:LayoutBounds( nil, "above", self.panel_bg ):Offset( 0, -1 )
    self.panel_top:LayoutBounds( nil, "above", self.panel_bg ):Offset( 0, -1 )
    self.panel_icon:LayoutBounds( "center", "center", self.panel_top ):Offset( 0, -35 )
    self.panel_behind:LayoutBounds( nil, "top", self.panel_bg ):Offset( 0, 1 )
    self.panel_glow_top:LayoutBounds( nil, "top", self.panel_bg ):Offset( 0, 100 )
    self.panel_glow_bottom:LayoutBounds( nil, "bottom", self.panel_bg ):Offset( 0, -100 )
    self.panel_border_bottom:LayoutBounds( nil, "below", self.panel_bg ):Offset( 0, 1 )
    self.panel_bottom:LayoutBounds( nil, "below", self.panel_bg ):Offset( 0, 1 )

    -- Layout texts
    self.subtitle:LayoutBounds( "center", self.subtitle:IsShown() and "below" or "bottom", self.title ):Offset( 0, SPACING.TITLE_SUBTITLE )
    self.text:LayoutBounds( "center", self.text:IsShown() and "below" or "bottom", self.subtitle )
    self.titles:LayoutBounds( "center", "top", self.panel_bg ):Offset( 0, -self.title_y_margin )

    -- Layout buttons
    self:LayoutButtons()

    return self
end

function WidePanel:LayoutButtons()
	self.subbuttons:StackChildrenRow( 15 )
	self.subbuttons:LayoutBounds( "center", "below", self.panel_bg ):Offset( 0, -20 )

	return self
end

function WidePanel:AddSubButton( text, icon, fn )
	local button = self.subbuttons:AddChild( Widget.IconButton( text or "" ) )
		:SetIcon( icon or global_images.accept )
		:SetFn( fn )

    self:LayoutButtons()

    return button
end

-- Hides everything
function WidePanel:PrepareAnimation()
    -- Hide objects until they're animated in
    self.panel:SetTintAlpha(0)

    return self
end

-- Fades in and calls the function at the end
function WidePanel:FadeIn( fn )
    self:Show()

	-- Prepare for animation by getting current values
    local pacing = 0.1
    local vertical_size_change = 40
    local panel_bg_w, panel_bg_h = self.panel_bg:GetSize()
    local panel_grid_w, panel_grid_h = self.panel_grid:GetSize()
    local panel_top_x, panel_top_y = self.panel_top:GetPos()
    local panel_icon_x, panel_icon_y = self.panel_icon:GetPos()
    local panel_border_top_x, panel_border_top_y = self.panel_border_top:GetPos()
    local panel_border_bottom_x, panel_border_bottom_y = self.panel_border_bottom:GetPos()
    local panel_bottom_x, panel_bottom_y = self.panel_bottom:GetPos()
    local title_x, title_y = self.title:GetPos()
    local subtitle_x, subtitle_y = self.subtitle:GetPos()

	-- Setup and start animation
    self.fadein_anim = self:RunUpdater(
        Updater.Series{
            Updater.Parallel{
                -- Fade and move in the background frame
                Updater.Ease( function(v) self.panel:SetTintAlpha(v) end, 0, 1, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.panel_bg:SetSize(x,y) end, panel_bg_w, panel_bg_h + vertical_size_change*2, panel_bg_w, panel_bg_h, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.panel_grid:SetSize(x,y) end, panel_grid_w + vertical_size_change*2, panel_grid_h + vertical_size_change*2, panel_grid_w, panel_grid_h, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.panel_icon:SetPos(x,y) end, panel_icon_x, panel_icon_y + vertical_size_change*1.5, panel_icon_x, panel_icon_y, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.panel_top:SetPos(x,y) end, panel_top_x, panel_top_y + vertical_size_change, panel_top_x, panel_top_y, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.panel_border_top:SetPos(x,y) end, panel_border_top_x, panel_border_top_y + vertical_size_change, panel_border_top_x, panel_border_top_y, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.panel_border_bottom:SetPos(x,y) end, panel_border_bottom_x, panel_border_bottom_y - vertical_size_change, panel_border_bottom_x, panel_border_bottom_y, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.panel_bottom:SetPos(x,y) end, panel_bottom_x, panel_bottom_y - vertical_size_change, panel_bottom_x, panel_bottom_y, pacing, easing.linear),
                Updater.Ease( function(v) self.title:SetTintAlpha(v) end, 0, 1, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.title:SetPos(x,y) end, title_x, title_y + vertical_size_change/2, title_x, title_y, pacing, easing.linear),
                Updater.Ease( function(v) self.subtitle:SetTintAlpha(v) end, 0, 0.8, pacing, easing.linear),
                Updater.Ease2D( function(x,y) self.subtitle:SetPos(x,y) end, subtitle_x, subtitle_y + vertical_size_change/3, subtitle_x, subtitle_y, pacing, easing.linear),
            },
            Updater.Do( function() if fn then fn() end end )
        }
    )
    return self
end

-- Fades out and calls the function at the end
function WidePanel:CancelFade( )
    self.title:StopAll()
    self.subtitle:StopAll()
    self.panel:StopAll()
    return self
end

function WidePanel:FadeOut( fn )
    local pacing = 0.1
    self.title:AlphaTo( 0, pacing )
    self.subtitle:AlphaTo( 0, pacing )
    self.panel:AlphaTo( 0, pacing, easing.linear, function() self:Hide() if fn then fn() end end )

    return self
end
