require "ui/widget"

----------------------------------------------------
-- A debug class used to trace hover/control handling through the Widget hierarchy.

local DebugWidgetTracer = class( "DebugWidgetTracer", DebugNode )

function DebugWidgetTracer:init( tracer )
    self.tracer = tracer
    self.menu_params = { tracer }

    self.all_controls = { "HOVER" }
    for i, k, v in sorted_pairs( Controls.Digital ) do
        table.insert( self.all_controls, k )
    end
end

function DebugWidgetTracer:PushWidget( widget )
    assert( is_instance( widget, Widget ))
    table.insert( self.trace, widget )
end

function DebugWidgetTracer:PopWidget( desc )
    table.insert( self.trace, desc )
end

function DebugWidgetTracer:RunTrace()
    self.trace = {}
    if (self.control_idx or 1) == 1 then
        local sx, sy = TheGame:GetInput():GetMousePos()
        TheGame:FE():CheckMouseHover( sx, sy, self )
    else
        local controls = ControlSet()
        controls:AddControl( self.all_controls[ self.control_idx or 1 ] )
        TheGame:FE():OnControlDown( controls, nil, self )
    end
end

function DebugWidgetTracer:RenderPanel( ui, panel )
    ui:Text( string.format( "Focus: %s", tostring( TheGame:FE():GetFocusWidget() )))
    ui:Text( string.format( "Hover: %s", tostring( TheGame:FE():GetHoverWidget() )))

    local idx = ui:Combo( "Control", self.control_idx or 1, self.all_controls )
    if idx and idx ~= self.control_idx then
        self.control_idx = idx
    end
    ui:Separator()

    if self.trace == nil then
        ui:TextColored( 0x777777FF, "No trace." )
        ui:Text( "Use CTRL+I to trace a Control over the current mouse position." )
    else
        ui:Text( string.format( "%d widgets traced", #self.trace / 2 ))
        local depth = 0
        for i, v in ipairs( self.trace ) do
            if is_instance( v, Widget ) then
                if depth == 0 and ui:TreeNode( string.format( "%s##%d", tostring(v), i )) then
                    panel:AppendTable( ui, v )
                else
                    depth = depth + 1
                end
            else
                if depth <= 0 then
                    ui:Text( v )
                    ui:TreePop()
                else
                    depth = depth - 1
                end
            end
        end
    end
end