local ControlSys = class("ControlSys")
--P = 15
--I = .5
--D = 5
function ControlSys:init(P, I, D, min, max)
    self.cP = P or 1
    self.cI = I or 1
    self.cD = D or 1
    self.outmin = min or -1
    self.outmax = max or 1
    self.control = 0
end

function ControlSys:SetConstants(P, I, D, min, max)
    self.cP = P or 1
    self.cI = I or 1
    self.cD = D or 1
    self.outmin = min or -1
    self.outmax = max or 1
end

function ControlSys:SetOutputLimits(min, max)
    self.outmin = min
    self.outmax = max
end

function ControlSys:SetTarget(target)

    --self:ClearTarget()    
    self.target = target
    
end

function ControlSys:ClearTarget()
    self.target = nil
    self.lasterr = nil
    self.errintegral = 0
end

function ControlSys:HasTarget()
    return self.target ~= nil
end

function ControlSys:Update(current_output, dt)
    local err = self.target - current_output 


    local sat = (err > 0 and self.control >= self.outmax) or (err < 0 and self.control <= self.outmin)

    if not sat then
        self.errintegral = self.errintegral + err*dt*self.cI
    end

    local pterm = self.cP*err
    local iterm = self.errintegral
    local dterm = self.cD * ( self.lasterr and (err - self.lasterr)/dt or 0)


    local control = pterm + dterm + iterm
    self.lasterr = err
    self.control = math.max(math.min(control, self.outmax), self.outmin)


    --print (string.format("target: %2.2f, val: %2.2f, control: %2.2f, err: %2.2f, P: %2.2f, I: %2.2f, D: %2.2f, %s", self.target, current_output, self.control, err, pterm, iterm, dterm, sat and "SAT" or ""))
end

function ControlSys:GetControlValue()
    return self.control or 0
end