class("CoTask")

function CoTask:init(deadline, taskfn, params)
	local info = debug.getinfo(taskfn, "Sl")
    self.deffile = info.source
    self.defline = info.linedefined

	self.deadline = deadline
	self.params = params
	self.co = coroutine.create(taskfn)
end

function CoTask:Running()
	return self.co ~= nil and coroutine.status(self.co) ~= "dead"
end

function CoTask:Trigger(time)
	if self.co then
		local success, sleep
		
		if self.params then
			success, sleep = coroutine.resume(self.co, table.unpack(self.params))
			self.params = nil
		else
			success, sleep = coroutine.resume(self.co)
		end
		assert(success, string.format("Coroutine error: %s\n%s", sleep, debug.traceback(self.co)))

		if coroutine.status(self.co) == "suspended" then
			self.deadline = time + (sleep or 0)
			return true
		end
	end
end

function CoTask:__lt(task)
	return self.deadline < task.deadline
end

function CoTask:Cancel()
	self.co = nil
	self.deadline = 0
	self.params = nil
end

function CoTask:__tostring()
	return string.format("COROUTINE:%s:%d %2.2fs", self.deffile, self.defline, self.deadline)
end
