class("Heap")

local VALIDATE = false

function Heap:init(fn)
	self.fn = fn or function(a,b) return a > b end
	self.nodes = {}
	self.node_to_idx = {}
end

function Heap:clear()
	table.clear(self.nodes)
	table.clear(self.node_to_idx)
end

function Heap:push(val)
	--self:Validate()
	local idx = #self.nodes + 1
	self.nodes[idx] = val
	self.node_to_idx[val] = idx
	self:FilterUp(idx)

	if VALIDATE and not self:isValid() then
		print ("INVALID ON PUSH", idx, #self.nodes)
		self:Validate()
	end
	--self:Validate()
end

function Heap:Validate()
	if not self:isValid() then
		print("NODES", #self.nodes)
		for i,v in ipairs(self.nodes) do
			print(i, v.deadline)
		end
		assert(false, "Heap is invalid")
	end
end

function Heap:size()
	return #self.nodes
end

function Heap:isempty()
	return #self.nodes == 0
end

function Heap:__tostring()
	return string.format( "heap(%d)", #self.nodes)
end

function Heap:remove(node)
	local idx = self.node_to_idx[node]
	if idx then
		return self:removebyidx(idx)
	end
	assert(false, "NOT IN THIS LIST!")
end


function Heap:FilterUp(idx)
	while true do
		local parent_idx = math.floor(idx / 2)
		if parent_idx > 0 and parent_idx ~= idx and not self.fn(self.nodes[parent_idx], self.nodes[idx]) then
			self.nodes[idx], self.nodes[parent_idx] = self.nodes[parent_idx], self.nodes[idx]

			self.node_to_idx[self.nodes[parent_idx]] = parent_idx
			self.node_to_idx[self.nodes[idx]] = idx

			idx = parent_idx

		else
			break
		end
	end

end

function Heap:FilterDown(idx)
	while idx < #self.nodes do

		local val = self.nodes[idx]
		local left = idx*2 <= #self.nodes and self.nodes[idx*2]
		local right = idx*2+1 <= #self.nodes and self.nodes[idx*2+1]

		if left and not self.fn(val, left) and not (right and self.fn(right, left)) then
			self.nodes[idx], self.nodes[idx*2] = self.nodes[idx*2], self.nodes[idx]

			self.node_to_idx[self.nodes[idx*2]] = idx*2
			self.node_to_idx[self.nodes[idx]] = idx
			idx = idx*2

		elseif right and not self.fn(val, right) then
			self.nodes[idx], self.nodes[idx*2+1] = self.nodes[idx*2+1], self.nodes[idx]
			self.node_to_idx[self.nodes[idx*2+1]] = idx*2+1
			self.node_to_idx[self.nodes[idx]] = idx
			idx = idx*2+1
		else
			break
		end
	end
end

function Heap:removebyidx(remidx)
	if remidx == 1 then
		return self:pop()
	end

	if remidx > #self.nodes then
		assert(false, "TRIED TO REMOVE THAT WHICH ISN'T THERE")
		return nil
	end

	local ret = self.nodes[remidx]
	if remidx == #self.nodes then
		
		self.node_to_idx[ret] = nil
		self.nodes[#self.nodes] = nil
		return ret
	end

	
	local lastnode = self.nodes[#self.nodes]


	self.nodes[#self.nodes] = nil
	self.node_to_idx[ret] = nil

	self.nodes[remidx] = lastnode
	self.node_to_idx[lastnode] = remidx

	local parent_idx = math.floor(remidx/2)
	

	if self.fn(self.nodes[parent_idx], lastnode) then
		self:FilterDown(remidx)
	else
		self:FilterUp(remidx)
	end


	if VALIDATE and not self:isValid() then
		print ("TRIED TO REMOVE", remidx)
		self:Validate()
	end
	return ret
	

end


function Heap:peek()
	return self.nodes[1]
end

function Heap:pop()
	local pop_node, last_node = self.nodes[1], self.nodes[#self.nodes]

	if pop_node == nil then
		assert(false, "TRIED TO REMOVE THAT WHICH ISN'T THERE")

	elseif pop_node == last_node then
		self.nodes[1], self.node_to_idx[pop_node] = nil, nil

	else
		self.nodes[1] = last_node
		self.nodes[#self.nodes] = nil

		self.node_to_idx[pop_node] = nil
		self.node_to_idx[last_node] = 1

		self:FilterDown(1)
	end

	assert(table.count(self.nodes) == table.count(self.node_to_idx))
	if VALIDATE and not self:isValid() then
		print ("INVALID ON POP!")
		self:Validate()
	end

	return pop_node
end

function Heap:isValid()
	if #self.nodes <= 1 then return true end
	local i = 1
	local lfIndex,rtIndex
	for i = 1,(math.floor(#self.nodes/2)) do
		lfIndex = 2*i
		rtIndex = lfIndex+1
		if self.nodes[lfIndex] then
			if self.fn(self.nodes[lfIndex], self.nodes[i]) then
				return false
			end
		end
		if self.nodes[rtIndex] then
			if self.fn(self.nodes[rtIndex], self.nodes[i]) then
				return false
			end
		end
	end
	return true
end
