----------------------------------------------------------------
-- Localization macros

return
{
    -- List things:
    -- "foo"
    -- "foo and bar"
    -- "foo, bar, and foobar"
    listing = function( t )
        if #t == 0 then
            return ""
        end

        local concat = t[1]
        for i = 2, #t - 1 do
            concat = concat .. ", " ..t[i]
        end
        if #t >= 3 then
            concat = concat .. ", " .. LOC"LOCMACROS.LIST_AND" .. " " .. t[#t]
        elseif #t >= 2 then
            concat = concat .. " " .. LOC"LOCMACROS.LIST_AND" .. " " .. t[#t]
        end
        return concat
    end,

    upper = function( s )
        return loc.toupper( s )
    end,

    lower = function( s )
        return loc.tolower( s )
    end,

    comma_listing = function( t )
        local concat = t[1]
        for i = 2, #t do
            concat = concat .. ", " ..t[i]
        end
        return concat
    end,

    number_postfix = function( n )
        if n == 0 then
            return "0th"
        elseif n == 1 then
            return "1st"
        elseif n == 2 then
            return "2nd"
        elseif n == 3 then
            return "3rd"
        else
            return string.format( "%sth", n )
        end
    end,

    relative_time = function( n )
        local phase = TheGame:GetGameState() and TheGame:GetGameState():GetDayPhase() or DAY_PHASE.NIGHT
        if n == 0 then
            return LOC"LOCMACROS.RELTIME.NOW"
        elseif n == 1 then
            return phase == DAY_PHASE.NIGHT and LOC"LOCMACROS.RELTIME.IN_MORNING" or LOC"LOCMACROS.RELTIME.TONIGHT"
        elseif n == 2 then
            return phase == DAY_PHASE.NIGHT and LOC"LOCMACROS.RELTIME.TOMORROW_NIGHT" or LOC"LOCMACROS.RELTIME.TOMORROW_MORNING"
        else
            return loc.format( LOC"LOCMACROS.RELTIME.IN_DAYS", math.ceil( phase/2 ) )
        end
    end,
    

    concat_line = function( s )
        if s ~= nil then
            return "\n"..tostring(s)
        else
            return ""
        end
    end,

    dayphase = function (phase)
        if phase == DAY_PHASE.DAY then
            return LOC"LOCMACROS.DAY_PHASE.DAY"
        elseif phase == DAY_PHASE.NIGHT then
            return LOC"LOCMACROS.DAY_PHASE.NIGHT"
        else
            return "?????"
        end
    end,

    agent_list = function( agents )
        local t = {}
        for i, agent in pairs( agents ) do
            table.insert( t, loc.format( "{1#agent}", agent ))
        end
        if #t == 0 then
            return "None"
        else
            return loc.format( "{1#listing}", t )
        end
    end,

    card_list = function( cards )
        if type(cards) == "table" then
            local txt = {}
            local ids = {}
            for i, card in pairs( cards ) do
                local id = type(card) == "table" and card.id or card
                if type(id) == "string" then
                    if ids[ id ] == nil then
                        ids[ id ] = 1
                        table.insert( txt, id )
                    else
                        ids[ id ] = ids[ id ] + 1
                    end
                end
            end

            table.sort( txt )

            -- Now stringify our array of card ids.
            for i, id in ipairs( txt ) do
                if ids[ id ] > 1 then
                    txt[ i ] = loc.format( LOC"LOCMACROS.MULTI_CARD", id, ids[ id ] )
                else
                    txt[ i ] = loc.format( "{1#card}", id )
                end
            end

            if #txt == 0 then
                return LOC"LOCMACROS.MULTI_CARD_NONE"
            else
                return loc.format( "{1#listing}", txt )
            end
        else
            return tostring(cards)
        end
    end,
    
    location_list = function( locations )
        local t = {}
        for i, location in pairs( locations ) do
            table.insert( t, loc.format( "{1#location}", location ))
        end
        return loc.format( "{1#listing}", t )
    end,

    faction_list = function( factions )
        local t = {}
        for i, faction in pairs( factions ) do
            table.insert( t, loc.format( "{1#faction}", faction ))
        end
        return loc.format( "{1#listing}", t )
    end,

    delivery_list = function( items )
        local t = ObtainWorkTable()
        for item, amount in pairs(items) do
            table.insert(t, loc.format( "{1#item_name} (x{2})", item, amount))
        end
        local str = loc.format("{1#listing}", t)
        ReleaseWorkTable(t)
        return str
    end,

    a_an = function( str )
        local c = loc.tolower( str:sub(1,1) )
        if c == "a" or c == "e" or c == "i" or c == "o" or c == "u" then
            return "an"
        else
            return "a"
        end
    end, 

    colour = function( rgba )
        return "#"..MakeColourString( HexColour( rgba ))
    end,

    binding = function ( control )
        if TheGame and TheGame:FE() then
            if TheGame:FE():IsGamepadMode() then
                return string.format( "<c img='%s'>", TheGame:GetInput():GetBoundButtonString( control ))
            else
                return string.format( "<#WHITE>[%s]</>", TheGame:GetInput():GetBoundKeyString( control ))
            end

        end
        return "???"
    end,

    points = function(point_data)
        
        local multiplier = TheGame:GetGameState() and TheGame:GetGameState().scorekeeper:GetScoreMultiplier() or 1
        local raw
        
        if type(point_data) == "number" then
            raw = point_data 
        elseif type(point_data) == "table" then
            raw, multiplier = point_data.raw or point_data[1] or 0, point_data.multiplier or point_data[2] or multiplier
        end

        local total = math.round(raw*multiplier)
        local bonus = total - raw
        if bonus > 0 then
            return loc.format( LOC "LOCMACROS.POINTS_BONUS", raw, bonus, total, multiplier)
        else
            return loc.format( LOC "LOCMACROS.POINTS", raw)
        end
    end,
    
    points_total = function(point_data)
        
        local multiplier = TheGame:GetGameState() and TheGame:GetGameState().scorekeeper:GetScoreMultiplier() or 1
        local raw
        if type(point_data) == "number" then
            raw = point_data 
        elseif type(point_data) == "table" then
            raw, multiplier = point_data.raw or point_data[1] or 0, point_data.multiplier or point_data[2] or multiplier
        end
        local total = math.round(raw*multiplier)

        return loc.format( LOC "LOCMACROS.POINTS", total)
    end,
    
    points_calc = function(point_data)
        
        local multiplier = TheGame:GetGameState() and TheGame:GetGameState().scorekeeper:GetScoreMultiplier() or 1
        local raw
        
        if type(point_data) == "number" then
            raw = point_data 
        elseif type(point_data) == "table" then
            raw, multiplier = point_data.raw or point_data[1] or 0, point_data.multiplier or point_data[2] or multiplier
        end

        local total = math.round(raw*multiplier)
        local bonus = total - raw
        if bonus > 0 then
            return loc.format( LOC "LOCMACROS.POINTS_CALC_BONUS", raw, bonus, total, multiplier)
        else
            return loc.format( LOC "LOCMACROS.POINTS", raw)
        end
    end,

    -- Percentage, 0 places of precision.
    percent = function( num )
        local percent
        if num == 0 then
            percent = 0
        elseif num < 0 then
            -- Only want >=1.0 to show up as 100%, and anything non-zero to be at least 1%.
            percent = math.min( -1, math.floor( (num or 0) * 100 )) 
        else
            -- Only want >=1.0 to show up as 100%, and anything non-zero to be at least 1%.
            percent = math.max( 1, math.floor( (num or 0) * 100 )) 
        end

        return loc.format( LOC "LOCMACROS.PERCENT", percent )
    end,

    -- Signed percent, 0 places of precision.
    spercent = function( num )
        local percent = (num or 0) * 100
        return loc.format( LOC "LOCMACROS.SPERCENT", percent )
    end,

    relationship = function( relationship )
        return GetRelationshipString(relationship)
    end,

    money = function( amount )        
        return loc.format( LOC "LOCMACROS.MONEY_FORMAT", amount )
    end,

    mettle = function( amount )        
        return loc.format( LOC "LOCMACROS.METTLE_FORMAT", amount )
    end,
    
    currency = function( amount )        
        return loc.format( LOC "LOCMACROS.CURRENCY_FORMAT", amount )
    end,

    location = function( arg )
        local location
        if is_instance( arg, Agent ) then
            location = arg:GetLocation()
        elseif is_instance( arg, Location ) then
            location = arg
        else
            location = TheGame:GetGameState():GetLocation( arg )
        end

        if location then
            if location:GetDesc() then
                return string.format( "<!location_%d>%s</>", location:GetGUID() or 0, location:GetName() )
            else
                return location:GetName()
            end
        end
    end,

    quest = function( quest )
        if quest then
            return string.format( "<#QUEST>%s</>", quest:GetTitle() or "???" )
        end
        return "???"
    end,

    agent = function( agent )
        if is_instance( agent, Agent ) then
            return string.format( "<!agent_%d>%s</>", agent and agent:GetID() or 0000, agent and agent:LocTable().who )
        end

        return tostring(agent)
    end,

    -- Name declension macros. These only are used for languages that support name declensions.
    name_nominative = function( agent )
        if type(agent) == "string" then
            return agent
        elseif agent and agent.GetID then
            return string.format( "<!agent_%d>%s</>", agent and agent:GetID() or 0000, agent and agent:LocTable().nominative or agent:LocTable().who )
        end
    end,

    name_accusative = function( agent )
        if type(agent) == "string" then
            return agent
        elseif agent and agent.GetID then
            return string.format( "<!agent_%d>%s</>", agent and agent:GetID() or 0000, agent and agent:LocTable().accusative or agent:LocTable().who )
        end
    end,

    name_genetive = function( agent )
        if type(agent) == "string" then
            return agent
        elseif agent and agent.GetID then
            return string.format( "<!agent_%d>%s</>", agent and agent:GetID() or 0000, agent and agent:LocTable().genetive or agent:LocTable().who )
        end
    end,

    name_dative = function( agent )
        if type(agent) == "string" then
            return agent
        elseif agent and agent.GetID then
            return string.format( "<!agent_%d>%s</>", agent and agent:GetID() or 0000, agent and agent:LocTable().dative or agent:LocTable().who )
        end
    end,

    name_vocative = function( agent )
        if type(agent) == "string" then
            return agent
        elseif agent and agent.GetID then
            return string.format( "<!agent_%d>%s</>", agent and agent:GetID() or 0000, agent and agent:LocTable().vocative or agent:LocTable().who )
        end
    end,

    name_locative = function( agent )
        if type(agent) == "string" then
            return agent
        elseif agent and agent.GetID then
            return string.format( "<!agent_%d>%s</>", agent and agent:GetID() or 0000, agent and agent:LocTable().locative or agent:LocTable().who )
        end
    end,

    name_instrumental = function( agent )
        if type(agent) == "string" then
            return agent
        elseif agent and agent.GetID then
            return string.format( "<!agent_%d>%s</>", agent and agent:GetID() or 0000, agent and agent:LocTable().instrumental or agent:LocTable().who )
        end
    end,


    faction = function( faction_id )
        if is_instance( faction_id, Faction ) then
            faction_id = faction_id:GetID()
        elseif is_instance( faction_id, Agent ) then
            --it's an agent?
            faction_id = faction_id:GetFactionID()
        else
            --it's an item?
            faction_id = faction_id.faction_id or faction_id
        end

        local faction = TheGame:GetGameState() and TheGame:GetGameState():GetFactions():FindFaction( faction_id ) or nil
        if not faction then return tostring(faction_id) end

        return faction:GetName() or "???"
    end,

    card = function( card )
        if type(card) == "string" then

            local carddef = Content.GetNegotiationCard( card )
            if carddef then
                return string.format( "<!negotiationcard_%s>%s</>", card or 0, carddef:GetLocalizedName() )
            end
            
            carddef = Content.GetBattleCard( card )
            if carddef then
                return string.format( "<!battlecard_%s>%s</>", card or 0, carddef:GetLocalizedName() )
            end

            return string.format( "INVALID CARD:%s", card)

        elseif is_instance(card, Battle.Card) then
            return string.format( "<!battlecard_%s>%s</>", card.id or 0, card:GetName() )
        elseif is_instance(card, Negotiation.Card) then
            return string.format( "<!negotiationcard_%s>%s</>", card.id or 0, card:GetName() )
        end
        return "???"

    end,

    graft = function( graft )
        if type(graft) == "string" then
           local graft_item = GraftInstance(graft)
           return string.format( "<!graft_%s>%s</>", graft_item.id, graft_item:GetName() )
        end
        return string.format( "<!graft_%s>%s</>", graft.id, graft:GetName() )
    end,

    graft_list = function( grafts )
        local t = {}
        for i, graft in pairs( grafts ) do
            if type(graft) == "table" then
                table.insert( t, loc.format( "{1#graft}", graft.id ))
            else
                table.insert( t, loc.format( "{1#graft}", graft ))
            end
        end
        if #t == 0 then
            return "None"
        else
            return loc.format( "{1#listing}", t )
        end
    end,

    thousands = function( num )
        local s = string.format( "%d", math.floor( num ) )
        local pos = string.len( s ) % 3
        if pos == 0 then pos = 3 end
        return string.sub( s, 1, pos ) .. string.gsub( string.sub( s, pos+1 ), "(...)", LOC"LOCMACROS.THOUSANDS_SEPARATOR" .. "%1" )
    end
}
