local RewardUtil = class("RewardUtil")

local function GetCards(diff, collection, count, owner)

    local rng = TheGame:GetGameState() and TheGame:GetGameState():GetRNG( "CARDS" )
    if rng then
        push_random( function( n, m )
            return rng:Random( n, m )
        end )
    end


    diff = diff or 1
    local prob = CARD_DROP_RARITY[math.max(1, math.min( diff, #CARD_DROP_RARITY))]
    local xp_range = CARD_DROP_START_XP[math.max(1, math.min( diff, #CARD_DROP_START_XP))]

    local used_card_defs = {}
    local cards = {}

    local generate_card = function()
        local rarity = weighted_arraypick(prob)
        
        local new_cards = collection(function(cd) return rarity == cd.rarity and table.arrayfind(used_card_defs, cd) == nil end):GenerateRandom( 1, owner )
        if new_cards[1] then
            if (new_cards[1].max_xp or 0) > 0 then
                new_cards[1]:AddRandomXP( xp_range[1], xp_range[2] )
            end
            table.insert(cards, new_cards[1])
            table.insert(used_card_defs, new_cards[1].def)
        end
    end

    for k = 1, count-1 do
        generate_card()
    end

    --jcheng: always get at least an uncommon card
    local has_uncommon = table.findif( cards, function(cd) return cd.rarity == CARD_RARITY.UNCOMMON or cd.rarity == CARD_RARITY.RARE end )
    if has_uncommon then
        generate_card()
    else
        local uncommons = collection( function(cd) return cd.rarity == CARD_RARITY.UNCOMMON end ):GenerateRandom( 1, owner )
        for i, card in ipairs(uncommons) do
            table.insert(cards, card)
        end
    end

    table.shuffle(cards)

    if rng then
        pop_random()
    end

    return cards
end

function RewardUtil.GetBossCards(owner, collection)

    local rng = TheGame:GetGameState() and TheGame:GetGameState():GetRNG( "CARDS" )
    if rng then
        push_random( function( n, m )
            return rng:Random( n, m )
        end )
    end

    local day = TheGame:GetGameState():GetActProgress()
    day = clamp( day or 1, 1, #BOSS_CARD_DROP_RARITY )
    local rarity_table = BOSS_CARD_DROP_RARITY[ day ]

    local cards = {}
    for rarity, num in pairs(rarity_table) do
        local new_cards = collection.AllRewardableCards( function(def) return def.rarity == rarity end, owner ):GenerateRandom(num, owner)
        table.arrayadd( cards, new_cards )
    end

    if rng then
        pop_random()
    end

    return cards
end

function RewardUtil.GetMixedCards(diff, count, owner)

    local cards = {}
    local negotiation_cards = RewardUtil.GetNegotiationCards( diff, count, owner )
    local battle_cards = RewardUtil.GetBattleCards( diff, count, owner )
    for i, v in ipairs(battle_cards) do
        table.insert(negotiation_cards, v)
    end
    table.shuffle(negotiation_cards)

    --jcheng: put at least one uncommon in there
    local uncommon = table.findif(negotiation_cards, function(cd) return cd.rarity == CARD_RARITY.UNCOMMON or cd.rarity == CARD_RARITY.RARE end )
    if uncommon then
        table.insert(cards, table.remove(negotiation_cards, uncommon))
    else
        LOGWARN("no available non-commons for owner %s", tostring(owner))
    end

    --add two other random ones
    for i = 1, count-1 do
        table.insert(cards, negotiation_cards[i])
    end

    return cards
end

function RewardUtil.GetBattleCards(diff, count, owner)
    return GetCards(diff, function(fn) return BattleCardCollection.AllRewardableCards( fn, owner ) end, count, owner)
end

function RewardUtil.GetNegotiationCards(diff, count, owner)
    return GetCards(diff, function(fn) return NegotiationCardCollection.AllRewardableCards( fn, owner ) end, count, owner)
end

function RewardUtil.GetCards(diff, generator, count, owner)
    return GetCards(diff, generator, count, owner)
end

function RewardUtil.GetBattlePacks(diff, count, pack_size, owner)
    local packs = {}
    for i = 1, count do
       packs[i] = GetCards(diff, function(fn) return BattleCardCollection.AllRewardableCards( fn, owner ) end, pack_size or 3, owner)
    end
    return packs
end

function RewardUtil.GetNegotiationPacks(diff, count, pack_size, owner)
    local packs = {}
    for i = 1, count do
       packs[i] = GetCards(diff, function(fn) return NegotiationCardCollection.AllRewardableCards( fn, owner ) end, pack_size or 3, owner)
    end
    return packs
end


function RewardUtil.GetGrafts(difficulty, num, owner, pred)

    local used_graft_ids = {}
    local grafts = {}
    local prob = GRAFT_DROP_RARITY[math.max(1, math.min( difficulty, #GRAFT_DROP_RARITY))]
    owner = owner or TheGame:GetGameState():GetPlayerAgent()
    
    for k = 1, num do
        local rarity = weighted_arraypick(prob)
        local collection = GraftCollection.Rewardable(owner, function(graft_def) return table.arrayfind(used_graft_ids, graft_def.id) == nil end):Rarity(rarity):Filter( pred )
        local graft = collection:Generate(1)[1]
        
        if graft then
            table.insert(grafts, graft)
            TheGame:GetGameProfile():SetSeenGraft( graft.id )
            table.insert(used_graft_ids, graft.id)
        end
    end
    table.shuffle(grafts)
    return grafts
end

-- graft_pool is map of rarity -> array of graft ids
function RewardUtil.GetPooledGrafts(difficulty, num, graft_pool)
    local grafts = {}
    local prob = GRAFT_DROP_RARITY[math.max(1, math.min( difficulty, #GRAFT_DROP_RARITY))]
    local owner = TheGame:GetGameState():GetPlayerAgent()
    
    for k = 1, num do
        local rarity = weighted_arraypick(prob)
        local graft_id = table.remove( graft_pool[ rarity ] )
        if graft_id then
            local graft = GraftInstance( graft_id )
            table.insert(grafts, graft)
        end
    end
    return grafts
end

function RewardUtil.GetBossGrafts( num, owner, graft_type )

    local used_graft_ids = {}
    local grafts = {}
    owner = owner or TheGame:GetGameState():GetPlayerAgent()
    local rng = TheGame:GetGameState():GetRNG( "GRAFTS" )
    
    for k = 1, num do
        local rarity = CARD_RARITY.BOSS
        local collection = GraftCollection.BossGrafts(owner, function(graft_def) return table.arrayfind(used_graft_ids, graft_def.id) == nil and (graft_type == nil or graft_def.type == graft_type) end):Rarity(rarity)
        local graft = collection:Generate(1, rng)[1]
        
        if graft then
            table.insert(grafts, graft)
            table.insert(used_graft_ids, graft.id)
        end
    end

    return grafts
end