require "util/heap"
require "util/task"
require "util/cotask"
--require "util"

--------------------------------------------------------------------------------
class("Scheduler")

function Scheduler:init()
    self.upcoming_tasks = Heap(function(a,b) return a.deadline < b.deadline end)
    self.time = 0
end

function Scheduler:DoCoroutine(delay, taskfn, ...)
    assert( delay > 0, delay )
    local task = CoTask(delay + self.time, taskfn, ... and {...} or nil)
    self.upcoming_tasks:push(task)
    return task
end

function Scheduler:DoTaskInTime(name, delay, taskfn, ...)
    assert( delay > 0, delay )
    local task = Task(delay + self.time, taskfn, ... and {...} or nil)
    task.name = name
    self.upcoming_tasks:push(task)
    return task
end

function Scheduler:DoPeriodicTask(name, initial_delay, period, taskfn, ...)
    assert( initial_delay == nil or initial_delay > 0, initial_delay )
    local task = Task((initial_delay or period) + self.time, taskfn, ... and {...} or nil)
    task.name = name
    task:MakePeriodic(period)
    self.upcoming_tasks:push(task)
    return task
end

function Scheduler:__tostring()
    return string.format("t = %.2f; %d pending tasks", self.time, self.upcoming_tasks:size())
end

function Scheduler:SetTimeFn(fn)
    self.settimefn = fn
end

function Scheduler:GetTime()
    return self.time
end

function Scheduler:InternalSetTime(t)
    assert( t >= self.time )
    if t > self.time then
        self.time = t
        if self.settimefn then
            self.settimefn(t)
        end
    end
end

function Scheduler:Update(dt)
    local cap_time = self.time + dt
    local tasks = self.upcoming_tasks
    local EPSILON = 1e-6

    while not tasks:isempty() and tasks:peek().deadline - cap_time < EPSILON do
        self:InternalSetTime( math.min( cap_time, tasks:peek().deadline ))
        
        local task = tasks:pop()
        --[[if task.name then
            engine.inst:ProfilerPush(task.name)
        end--]]
        if task:Trigger(self.time) then
            assert(task.deadline >= self.time, "periodic task has negative period")
            tasks:push(task)
        else
            task:Cancel()
        end
        --[[if task.name then
            engine.inst:ProfilerPop()
        end--]]
    end
    self:InternalSetTime(cap_time)
end

