local weighted_list = class( "WeightedList" )

function weighted_list:init( t )
	self._totalWeight = 0

	if t then
        if #t > 0 and type(t[1]) == "table" then
            -- { { choice1, weight1 }, { choice2, weight2 }, ... }
		    for i, choice in ipairs(t) do
			    self:AddChoice( choice[1], choice[2] )
		    end
        else
            -- { [choice1] = weight1, [choice2] = weight2, ... }
            for choice, weight in pairs(t) do
                self:AddChoice( choice, weight )
            end
        end
	end
end

function weighted_list:Clear()
	self._totalWeight = 0
	while #self > 0 do
		table.remove( self )
	end
end

function weighted_list:GetTotalWeight()
	return self._totalWeight
end

function weighted_list:GetCount()
	return #self / 2
end

function weighted_list:AddChoice( item, weight )
	assert( weight > 0 )
	table.insert( self, weight )
	table.insert( self, item )
	self._totalWeight = self._totalWeight + weight
end

function weighted_list:GetChoice( weight )
    if weight == nil then
        weight = math.random() * self._totalWeight
    end

	for i = 1, #self, 2 do
		if weight <= self[i] then
			return self[i + 1], i
		end
		weight = weight - self[i]
	end

	return nil
end

function weighted_list:RemoveIndex( index )
	local weight = self[ index ]
	self._totalWeight = self._totalWeight - weight
	table.remove( self, index )
	table.remove( self, index )
end

function weighted_list:RemoveChoice( weight )
	local item, index = self:GetChoice( weight )
	if item then
		self:RemoveIndex( index )
	end
	return item, index
end

function weighted_list:RemoveHighest( )
	local maxWeight, maxIndex = 0, 0
	for i = 1, #self, 2 do
		if self[ i ] > maxWeight then
			maxWeight, maxIndex = self[ i ], i
		end
	end
	if maxIndex > 0 then
		self._totalWeight = self._totalWeight - maxWeight
		table.remove( self, maxIndex )
		return table.remove( self, maxIndex ), maxWeight
	end
end

function weighted_list:Clone()
	local wt = weighted_list()
	for i = 1, #self, 2 do
		wt:AddChoice( self[i+1], self[i])
	end
	return wt
end

function weighted_list:Print()
    local str = {}
    for i = 1, #self, 2 do
        table.insert( str, string.format( "%d] - %3d wt, %s\n", i, self[i], tostring(self[i+1]) ))
    end
    return table.concat( str )
end

return weighted_list
