local QDEF = QuestDef.Define{
    title = "Bulk Discount",
    qtype = QTYPE.CONTRACT,
    faction = "RISE",
    rank = {1, 3},
    desc = [[The Barons are overcharging workers at {company_store}. Put an end to it.]],
    icon = engine.asset.Texture("icons/quests/rook_story_rise_bulk_discount.tex"),
    reward_size = QREWARDSIZE.SMALLER,
    focus = QUEST_FOCUS.NEGOTIATION,
    on_start = function(quest)
        quest:Activate("deal_with_store")
    end,
}

:AddCast{
    cast_id = "giver",
    provider = true,
    no_validation = true,
}

:AddLocationCast{
    cast_id = "company_store",
    cast_fn = function(quest, t)
        table.insert(t, TheGame:GetGameState():GetLocation("GB_COMPANY_STORE"))
    end,
}

:AddCast{
    cast_id = "shopkeep",
    cast_fn = function(quest, t)
        table.insert( t, quest:GetCastMember("company_store"):GetProprietor() )
    end,
}


:AddObjective{
    id = "deal_with_store",
    title = [[Go to {company_store} and convince the shopkeeper to lower their prices.]],
    desc = "The workers of the Bog are dependant upon {company_store} for most of their goods. This leaves them vulnerable to exploitation by whatever Baron happens to be minding the till.",
    --state = QSTATUS.ACTIVE,
    mark = {"shopkeep"},
    negotiation_targets = {"shopkeep"},
    combat_targets = {"shopkeep"},

    on_complete = function(quest) 
        quest:Activate("return_for_reward")
    end
}

:AddObjective{
    id = "return_for_reward",
    title = "Get reward from {giver}",
    desc = [[Go tell {giver} that the recent pricing discrepancies have been rectified.]],
    ready_for_completion = true,
    mark = { "giver" },
    terminal = true,
}


QDEF:AddConvo("deal_with_store", "shopkeep")
    :Loc{
        OPT_ASK_ABOUT_BOOT_PRICES = "Ask about prices",
        DIALOG_ASK_ABOUT_BOOT_PRICES = [[
            player:
                What would it cost to buy some new boots?
            agent:
                20 shills.
            player:
                !thought
                That's not so bad.
            agent:
                That's per boot, of course.
            player:
                Oh. I see.
            agent:
                !shrug
                Don't blame me. That's just supply and demand in action!
        ]],
        DIALOG_ASK_ABOUT_BOOTS_AGAIN = [[
            player:
                We were talking about prices.
            agent:
                Yeah, so?
        ]],
        OPT_CONVINCE_TO_LOWER = "Arrange a 'bulk discount' for the workers",
        DIALOG_CONVINCE_TO_LOWER = [[
            player:
                If you lower your prices, you'll make up the difference in volume.
        ]],
        DIALOG_CONVINCE_TO_LOWER_SUCCESS = [[
            agent:
                !thought
                Maybe you're right. 
                I'll give it a shot, but if it doesn't work I'm going to have to raise the prices again.
                !exit
        ]],
        DIALOG_CONVINCE_TO_LOWER_FAIL = [[
            agent:
                That's not true, since most of the goods I'm selling are subject to inelastic demand.
                !shrug
                I'm just pricing things rationally.
        ]],


        OPT_THREATEN = "Demand lower prices for the workers",
        DIALOG_THREATEN = [[
            player:
                Let me spell it out for you. Lower your prices, or I'll lower your life expectancy.
        ]],
        DIALOG_THREATEN_SUCCESS = [[
            agent:
                You win! I'll do it - just don't hurt me!
                !exit
        ]],
        DIALOG_THREATEN_FAIL = [[
            agent:
                I'm calling your bluff, grifter. 
                !point
                I'm a Spark Baron. Touch me, and the invisible hand will touch <b>you</>.
        ]],


        OPT_BEAT_UP = "Beat up {agent}",
        DIALOG_BEAT_UP = [[
            player:
                !fight
                I'll show <i>you</> some market forces.
        ]],

        DIALOG_BEAT_UP_DEAD = [[
            * {agent} is dead. You leave the shop before the looting starts.
        ]],
        DIALOG_BEAT_UP_ALIVE = [[
            agent:
                !injured
                I give up! I'll do what you asked.
                You... you hooligan!
        ]],


        
    }
    :Hub(function(cxt) 
        
        cxt:Opt("OPT_ASK_ABOUT_BOOT_PRICES")
            :Dialog(cxt.quest.param.asked_about_boots and "DIALOG_ASK_ABOUT_BOOTS_AGAIN" or "DIALOG_ASK_ABOUT_BOOT_PRICES")
            :Fn(function(cxt) 
                cxt.quest.param.asked_about_boots = true 
                
                cxt:Opt("OPT_CONVINCE_TO_LOWER")
                    :Dialog("DIALOG_CONVINCE_TO_LOWER")
                    :Negotiation{
                        on_success = function() 
                            cxt:Dialog("DIALOG_CONVINCE_TO_LOWER_SUCCESS")
                            cxt.quest:Complete("deal_with_store")
                            StateGraphUtil.AddEndOption(cxt)
                        end,

                        on_fail = function() 
                            cxt:Dialog("DIALOG_CONVINCE_TO_LOWER_FAIL")
                        end,
                    }
                
                cxt:Opt("OPT_THREATEN")
                    :Dialog("DIALOG_THREATEN")
                    :Negotiation{
                        flags = NEGOTIATION_FLAGS.INTIMIDATION,
                        on_success = function() 
                            cxt:Dialog("DIALOG_THREATEN_SUCCESS")
                            cxt.quest:Complete("deal_with_store")
                            StateGraphUtil.AddEndOption(cxt)
                        end,

                        on_fail = function() 
                            cxt:Dialog("DIALOG_THREATEN_FAIL")
                        end,
                    }

                cxt:Opt("OPT_BEAT_UP")
                    :Dialog("DIALOG_BEAT_UP")
                    :Battle{
                        on_win = function() 
                            cxt.quest:Complete("deal_with_store")
                            cxt.quest.param.fought = true
                            if cxt:GetAgent():IsDead() then
                                cxt:Dialog("DIALOG_BEAT_UP_DEAD")
                                StateGraphUtil.AddEndOption(cxt)
                            else
                                cxt:Dialog("DIALOG_BEAT_UP_ALIVE")
                                StateGraphUtil.AddLeaveLocation(cxt)
                            end
                        end,
                    }
                StateGraphUtil.AddBackButton(cxt)
            end)

    end)    

QDEF:AddConvo("return_for_reward", "giver")
    :Loc{
        OPT_REPORT_DEATH = "Report {shopkeep}'s death",
        DIALOG_REPORT_DEATH = [[
            player:
                I confronted {shopkeep}, but things went sideways.
            agent:
                How sideways?
            player:
                {shopkeep} is dead. I don't know what that means for the shop prices.
            agent:
                !facepalm
                You imbecile. It means the Barons are going to crack down on us unless I can cover it up.
                At least try to follow my instructions next time, OK?
        ]],
        OPT_REPORT_PRICES = "Report lowered prices",
        
        DIALOG_REPORT_PRICES_FOUGHT = [[
            player:
                {shopkeep} agreed that it was in {shopkeep.hisher} economic interest to lower {shopkeep.hisher} prices.
            agent:
                You were able to spell it out plainly, huh?
            player:
                Something like that...
            agent:
                Watch yourself. The Barons are nothing to trifle with.
        ]],
        DIALOG_REPORT_PRICES = [[
            player:
                {shopkeep} agreed that it was in {shopkeep.hisher} economic interest to lower {shopkeep.hisher} prices.
            agent:
                Nice work. Violence is a useful tool, but not if we use it every time.
        ]],
    }
    :Hub(function(cxt) 
        
        if cxt.quest:GetCastMember("shopkeep"):IsDead() then
            cxt:Opt("OPT_REPORT_DEATH")
                :SetQuestMark()
                :Dialog("DIALOG_REPORT_DEATH")
                :FailQuest()
        else
            cxt:Opt("OPT_REPORT_PRICES")
                :SetQuestMark()
                :Dialog(cxt.quest.param.fought and "DIALOG_REPORT_PRICES_FOUGHT" or "DIALOG_REPORT_PRICES")
                :CompleteQuest()
        end
    end)


QDEF:AddConvo("return_for_reward")
    :ConfrontState("CONFRONT", function(cxt) return not cxt.quest.param.has_had_post_confront and cxt.location:HasTag("in_transit") end)    
        :Loc{
            DIALOG_INTRO = [[
                * You are stopped by a contingent of Spark Barons.
                player:
                    !left
                agent:
                    !right
                    There's the grifter!
                player:
                    What seems to be the trouble?
                agent:
                    !angry
                    The trouble is you sticking your nose in our pocketbooks.
                    The prices at the company store are high because I get a cut.
                    Now that they're not as high, I won't get as much of a cut.
                    Not getting a high cut makes me sad. And that makes me want to do violent things to you.
                    Unless, of course, you can make up the difference.
                *** {agent} is angry with you because {agent.heshe} was making money off of the store's high prices.
            ]],
            OPT_HAND_OVER_MONEY = "Hand over money",
            DIALOG_HAND_OVER_MONEY = [[
                player:
                    I didn't know that the livelihood of such fine people was dependant upon those sales.
                    I hope this will help make things right again.
                    !give
                agent:
                    !take
                    !neutral
                    It's a start. But you're in for a world of hurt if you keep upsetting the natural order of things here.
                    !point
                    The Bog works. Don't break it, or we'll break <i>you</>.
                    !exit
            ]],
            OPT_EXPLAIN_YOURSELF = "Explain the concept of demand elasticity",
            DIALOG_EXPLAIN_YOURSELF = [[
                player:
                    The prices are lower, but that will just mean that the workers can buy a greater volume of goods.
                    You're still going to get your money - that's simple economics.
            ]],
            DIALOG_WIN_NEGOTIATION = [[
                agent:
                    !thought
                    It's a crazy plan, but it might work.
                    !point
                    You'll hear from me if it doesn't!
                    !exit
            ]],
            DIALOG_LOSE_NEGOTIATION = [[
                agent:
                    "Blah blah blah". 
                    I can feel myself getting poorer just listening to you!
                    Cough up the money, or you'll be coughing up blood!
            ]],
            OPT_DEFEND_YOURSELF = "Defend yourself",
            DIALOG_DEFEND = [[
                player:
                    !fight
                    You'll find I don't bully as easily as the local workforce.
            ]],
            DIALOG_WON_FIGHT = [[
                * Shots have been fired. You leave before that attracts further attention.
                player:
                    !exit
            ]]
        }
        :SetLooping()
        :Fn(function(cxt) 
            
            local money = cxt.caravan:GetMoney()
            
            --make it payable, but not devastating.
            local fine_amount = 50
            if money >= 150 then
                fine_amount = 100
            elseif money >= 200 then
                fine_amount = 150
            elseif money >= 300 then
                fine_amount = 200
            end

            
            if cxt:FirstLoop() then
                cxt.quest.param.has_had_post_confront = true
                local baron = AgentUtil.GetFreeAgent("SPARK_BARON_TASKMASTER", function(agent) return agent:GetRelationship() == RELATIONSHIP.NEUTRAL end)
                local backup = CreateCombatBackup(baron, "SPARK_BARON_BACKUP", cxt.quest:GetRank() + 1)
                cxt.enc:SetPrimaryCast(baron)
                cxt:Dialog("DIALOG_INTRO")
            end

            cxt:Opt("OPT_HAND_OVER_MONEY")
                :Dialog("DIALOG_HAND_OVER_MONEY")
                :DeliverMoney(fine_amount)
                :Travel()

            cxt:Opt("OPT_EXPLAIN_YOURSELF")
                :Dialog("DIALOG_EXPLAIN_YOURSELF")
                :Negotiation{
                    on_success = function()
                        cxt:Dialog("DIALOG_WIN_NEGOTIATION")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_fail = function() 
                        cxt:Dialog("DIALOG_LOSE_NEGOTIATION")
                    end,
                }

            cxt:Opt("OPT_DEFEND_YOURSELF")
                :Dialog("DIALOG_DEFEND")
                :Battle{
                    flags = BATTLE_FLAGS.SELF_DEFENCE,
                    on_win = function() 
                        cxt:Dialog("DIALOG_WON_FIGHT")
                        StateGraphUtil.AddLeaveLocation(cxt)
                    end,
                    on_runaway = StateGraphUtil.DoRunAwayNoFail,
                }

        end)


QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.INTRO )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                There's a lot of industry here in the Bog, but we don't actually make anything useful.
                Take boots, for example. Every boot on every foot here was made far away, and was brought in by the Barons, and sold at a Baron store.
                The workers need boots, so they need to pay whatever price the Barons are asking at {company_store}.
                Those prices have been going up, lately. Let's go talk to the shopkeep and see if we can push them back down again.
            *** {giver} needs you to convince {shopkeep} to stop fleecing the local workers.
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)



QDEF:AddConvo( nil, nil, QUEST_CONVO_HOOK.ACCEPTED )
    :Loc{
        DIALOG_INTRO = [[
            agent:
                Don't cause too much of a ruckus, or the Barons will just find another way to make us pay.
                This one's all about soft power, right?
        ]]
    }
    :State("START")
        :Fn(function(cxt)
            cxt:Dialog("DIALOG_INTRO")
        end)
